dojo.provide("com.ibm.bpm.wpd.assetlistview.view.AssetListAccordionContainer");

/**
 * Subclass of the AccordianContainer to customize it for our purpose
 */
dojo.declare("com.ibm.bpm.wpd.assetlistview.view.AssetListAccordionContainer",
		[dijit.layout.AccordionContainer],
		{
	
		templateString : "<div style=\"\" dojoAttachPoint=\"containerNode\">" +
			"</div>",

		widgetsInTemplate : true,

		buttonWidget: "com.ibm.bpm.wpd.assetlistview.view.AssetListAccordionButton",
		
		expandedSections:false,
		
		manualResizing: false,
	
		/**
		 * Build the view, subclass that of the parent but add scroll bars. The parent disables the scroll
		 * bars causing any data that falls outside the viewable area to be unreachable
		 */
		buildRendering: function(){
			this.inherited(arguments);
			//this.domNode.style.overflow = "auto";
		},
		
		paieSize: 0,
		
		defaultSectionSize:300,
	
		/**
		 * Overwrite the superclass implementation to customize for ourselves. Compute the value based on the number of assets within the section.
		 * We could potentially cap this to a max size in the future.
		 * The reason for overwritng is the susperclass behaves differently depending on the browser we are viewing from. In the case of IE
		 * it appears to always come back with a height of around 200 px. In FF it's appears to be based on the size of the smallest section
		 * in the accordion. So if we have one small section and numerous large sections it'll always allocate based ont he smallest (which is
		 * kind of useless)
		 */
		_getTargetHeight: function(/* Node */ node){
			// Slight difference in IE as we have to use images to create the rounded corners so the height per
			// section is a little more
			
			// In the case of the defualt section it'll be expanded before the children are added.
			if (node.childNodes.length == 0)
				return this.defaultSectionSize;
			
			if (this.expandedSections) {
				
				var assetItems = 0;
				var assetItemsRevisions = 0;
				
				if (node.childNodes.length == 1) {
					if (dojo.isIE) {
						assetItems = node.childNodes[0].childNodes.length * 35;
						for (var i=0;i<node.childNodes[0].childNodes.length; i++) {
							assetItemsRevisions = assetItemsRevisions + (node.childNodes[0].childNodes[i].childNodes.length * 30);
						}
						return assetItems+assetItemsRevisions;
					}

					assetItems = node.childNodes[0].childNodes.length * 35;
					for (var i=0;i<node.childNodes[0].childNodes.length; i++) {
						assetItemsRevisions = assetItemsRevisions + (node.childNodes[0].childNodes[i].childNodes.length * 30);
					}
					return assetItems+assetItemsRevisions;
				}
				
				assetItems = node.childNodes.length * 35;
				for (var i=0;i<node.childNodes.length; i++) {
					assetItemsRevisions = assetItemsRevisions + (node.childNodes[i].childNodes.length * 30);
				}
				return assetItems+assetItemsRevisions;
			}
			else {
				// Will do this for now, what I'd like to do is separates section to there own widget and set a size
				if (node.childNodes.length == 1) {
					
					if (node.childNodes[0].id.indexOf("ProcessAppsSection") != -1) {
						return this.processAppsSize(node.childNodes[0]);
					}
					
					// IE is padded due to using graphics for round edges
					if (dojo.isIE == 6)
						return node.childNodes[0].childNodes.length * 58.70;
					
					if (dojo.isIE) {
						//return node.childNodes[0].childNodes.length * 47;
						return node.childNodes[0].childNodes.length * 34;
					}
					
					// Safari doesn't seem to have growth pattern
					if (dojo.isSafari)
						return (node.childNodes[0].childNodes.length * 34) + 10;
					
					//other browsers
					return node.childNodes[0].childNodes.length * 39.06;
				}
				
				return node.childNodes.length * 35;
			}
		},
		
		processAppsSize: function(processApps) {
			if (dojo.isIE && this.paieSize > 0) {
				return this.paieSize;
			}
			
			var tempN = processApps;
			// 3 that don't bubble
			var defaultSize = 90;
			if (dojo.isIE == 6)
				defaultSize = 100;
			
			// Check to see if it have env vars
			if (processApps.children[0].style.display && processApps.children[0].style.display=="none") {
				defaultSize = defaultSize - 20;
			}
				

//			if (dojo.isIE == 6) {
//				// add the 4 non graphic sections and the 3 graphic sections
//				defaultSize = 112 + 140;
//			}

			// figure out the env vars;
			var pa = processApps.children[1].children.length;
			// figure out the toolkits
			var tk = processApps.children[3].children.length;
			
			var totalBubbled = pa + tk + 3;
			
			if (dojo.isIE) {
				if (dojo.isIE > 6)
					defaultSize = defaultSize + (totalBubbled * 33.5);
				else
					defaultSize = defaultSize + (totalBubbled * 48);
			}
			else {
				defaultSize = defaultSize + (totalBubbled * 32);
			}

			// In the case of IE cache it as for some reason it'll call this multiple times, but a child each time?
			if (dojo.isIE && (this.paieSize == 0)) {
				
				if (dojo.isIE == 7)
					defaultSize = defaultSize + 10;
				
				this.paieSize = defaultSize;
			}

			return defaultSize;
			
		},
		
		resizeSelectedPane: function(size) {
			
			// We are triggering a resize of the container. In IE the frameworks received multiple resize events which cause issues
			// Set a flag to prevent this events form being acted upon. 
			this.manualResizing = true;
			
			var openPane = this.selectedChildWidget;
			if(openPane){
				var mySize = this._contentBox;
				
				this._containerContentBox = {
						h: this._getTargetHeight(openPane.domNode) + size,
						w: mySize.w
				};
				openPane.resize(this._containerContentBox);
			}	
		},
		
		/**
		 * Transition is used to slide the accordion from one selected container to another. Override the superclass
		 * as the height isn't desirable in the case of a larger number of accordion sections
		 * @param newWidget: dijit._Widget - The accordion section that is about to be expanded
		 * @param oldWidget:dijit._Widget - The accordion section about to be collapsed
		 * @param animate:boolean - whether to provide an animiation when switching section, meaning a look of sliding from one
		 * section to the other
		 */
		_transition: function(newWidget, oldWidget, animate){
			
			this.manualResizing = false;
			
			if(dojo.isIE < 8){
				// workaround animation bugs by not animating; not worth supporting animation for IE6 & 7
				animate = false;
			}

			if(this._animation){
				// there's an in-progress animation.  speedily end it so we can do the newly requested one
				this._animation.stop(true);
				delete this._animation;
			}

			var self = this;

			if(newWidget){
				newWidget._wrapperWidget.set("selected", true);
				newWidget._wrapperWidget.button.setSelectedState(true);

				var d = this._showChild(newWidget);	// prepare widget to be slid in

				// Size the new widget, in case this is the first time it's being shown,
				// or I have been resized since the last time it was shown.
				// Note that page must be visible for resizing to work.
//				if(this.doLayout && newWidget.resize){
//					newWidget.resize(this._containerContentBox);
//				}
			}

			if(oldWidget){
				oldWidget._wrapperWidget.set("selected", false);
				oldWidget._wrapperWidget.button.setSelectedState(false);
				if(!animate){
					this._hideChild(oldWidget);
				}
			}

			if(animate){
				var newContents = newWidget._wrapperWidget.containerNode,
					oldContents = oldWidget._wrapperWidget.containerNode;

				// During the animation we will be showing two dijitAccordionChildWrapper nodes at once,
				// which on claro takes up 4px extra space (compared to stable AccordionContainer).
				// Have to compensate for that by immediately shrinking the pane being closed.
				var wrapperContainerNode = newWidget._wrapperWidget.containerNode,
					wrapperContainerNodeMargin = dojo._getMarginExtents(wrapperContainerNode),
					wrapperContainerNodePadBorder = dojo._getPadBorderExtents(wrapperContainerNode),
					animationHeightOverhead = wrapperContainerNodeMargin.h + wrapperContainerNodePadBorder.h;

				oldContents.style.height = (self._verticalSpace - animationHeightOverhead) + "px";

				var newWidgetHeight = this._getTargetHeight(newWidget.domNode);
				var oldWidgetHeight = this._getTargetHeight(oldWidget.domNode);
				this._animation = new dojo.Animation({
					node: newContents,
					duration: this.duration,
					curve: [1, this._verticalSpace - animationHeightOverhead - 1],
					onAnimate: function(value){
						value = Math.floor(value);	// avoid fractional values
						//newContents.style.height = value + "px";
						newContents.style.height = newWidgetHeight + "px";
						//oldContents.style.height = (self._verticalSpace - animationHeightOverhead - value) + "px";
						oldContents.style.height = oldWidgetHeight + "px";
					},
					onEnd: function(){
						delete self._animation;
						newContents.style.height = "auto";
						oldWidget._wrapperWidget.containerNode.style.display = "none";
						oldContents.style.height = "auto";
						self._hideChild(oldWidget);
					}
				});
				this._animation.onStop = this._animation.onEnd;
				this._animation.play();
			}

			return d;	// If child has an href, promise that fires when the widget has finished loading
		},
		
		/**
		 * Override the superclass to trigger it's title to be selected
		 * @param child
		 */
		_setupChild: function(child){
			this.inherited(arguments);
			if (child.selected)
				child._wrapperWidget.button.setSelectedState(true);
		},
		

		resize: function(changeSize, resultSize){
			// For now we're going to eat this if it's ie 6. For some reason in IE6 if you have more than a few sections in the accordion
			// this resize triggers an endless loop of resizing. Will need to pick apart this resize to see just what it's doing
			if (dojo.isIE && this.manualResizing)
				return;
			this.inherited(arguments);
		},
		
		/**
		 * Overwrite the supserclass to provide the correct size on the default section
		 */
		layout: function(){
			this.inherited(arguments);

			// Overwrite the initial size of the default pane to prevent cutoff
			var openPane = this.selectedChildWidget;
			if(openPane){
				var mySize = this._contentBox;

				this._containerContentBox = {
						h: this._getTargetHeight(openPane.domNode) + 10,
						w: mySize.w
				};
				openPane.resize(this._containerContentBox);
			}			
		},
		
		prepareAccordianForPrint: function() {
			var children = this.getChildren();
			for (var i=0;i<children.length;i++) {
				var newWidget = children[i];
				var d = this._showChild(newWidget);	// prepare widget to be slid in
			}
			
		}

});

/**
 * Subclass of AccordianButton to custom for our application
 */
dojo.declare("com.ibm.bpm.wpd.assetlistview.view.AssetListAccordionButton",
		[dijit.layout._AccordionButton],
		{

		templateString: //"<div dojoAttachEvent='onclick:_onTitleClick' class='bpmnAccordionTitle'>" +
			"<div dojoAttachPoint='titleNode,focusNode' dojoAttachEvent='onkeypress:_onTitleKeyPress, onclick:_onTitleClick' class='bpmnAccordionTitle' role=\"tab\" aria-expanded=\"false\">" +
			"<span class='dijitInline dijitAccordionArrow' role=\"presentation\"></span>" +
			"<img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon\" dojoAttachPoint='iconNode' style=\"vertical-align: middle;\" role=\"presentation\"/>" +
			"<span role=\"presentation\" dojoAttachPoint='titleTextNode' class='dijitAccordionText'></span>" +
			//"</div>" +
			"</div>",

		baseClass: "bpmnAccordionTitle",

		setSelectedState: function(/*Boolean*/ isSelected){
			this.selected = isSelected;
			// overwite the superclass behaviour for now, selected colour doesn't change
			dojo[(isSelected ? "addClass" : "removeClass")](this.titleNode,"bpmnAccordionSelectedTitle");
			dijit.setWaiState(this.focusNode, "expanded", isSelected);
			dijit.setWaiState(this.focusNode, "selected", isSelected);
			this.focusNode.setAttribute("tabIndex", isSelected ? "0" : "-1");
			var t = this.titleTextNode;
			
			if (this.titleTextNode.childNodes.length > 1) {
				if (isSelected) {
					this.titleTextNode.childNodes[1].style.visibility="visible";
					this.titleTextNode.childNodes[2].style.visibility="visible";
				}
				else  {
					this.titleTextNode.childNodes[1].style.visibility="hidden";
					this.titleTextNode.childNodes[2].style.visibility="hidden";
				}
			}
		}
});