dojo.provide("com.ibm.bpm.wpd.copy.control.commands.OpenCopyDocumentCommand");
dojo.require("com.ibm.bpm.wpd.core.control.commands.RestCommand");
dojo.declare("com.ibm.bpm.wpd.copy.control.commands.OpenCopyDocumentCommand", [com.ibm.bpm.wpd.core.control.commands.RestCommand], {
	/**
	 * Open a document/po in the compare and copy context
	 */
	
	docTypeProvider : null, // a instance of BaseDocTypeProvider


	//Requires the following data to be passed as parameters.
	//
	//	- type                - the document type to open
	//  - branchId
	//	- snapshotId
	//  - documentName        - name of the document
	//  - documentId          - uuid of document
	//  - fromOrTo            - is this going into the from or to pane?
	constructor : function (pEvent) {
		dojo.requireLocalization("com.ibm.bpm.wpd.copy.view", "CopyResources");
		this.copymessages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.copy.view", "CopyResources");
	},
	
	
	/**
	 * Request data from the server, then create a ViewStateModel using the response data
	 */
	execute : function () {
		debugger;
		// make the rest call to get the data to display
		if (dojo.global.BPM.WPD_useWLEServer){
			
			if ( (this.event.fromOrTo == "from") ||
				 ((this.event.existsInTarget)&&(this.event.fromOrTo == "to")) ) {
				
				this.sendRequest();
				/*
				 * Do some work wile waiting for respond,
				 * Check how many I have cached, if reach 20, then remove the old ones
				 * Actually, the vsm is not that heavy
				 */
			
			} else {
				
				// create the new doc
				var viewerAttachPoint = document.createElement("div");

				viewerAttachPoint.style.height = "600px";
				viewerAttachPoint.style.width = "500px";
				
				viewerAttachPoint.style.overflowX = "hidden"; 
				viewerAttachPoint.style.overflowY = "hidden"; 
			

			
				// create sub div for the document to get added to
				//    - seems that a fixed div directly in the table cell cannot be made to scroll
				var inDiv = document.createElement("div");
				viewerAttachPoint.appendChild(inDiv);
				var txt = document.createTextNode(this.copymessages.COPY_SIDE_BY_SIDE_MISSING_TARGET);
				inDiv.appendChild(txt);
				inDiv.style.margin = "25% 25%";
				
				
				// populate the right pane with message that asset doesn't exist in target
				this.event.view.copyToSectionBody.appendChild(viewerAttachPoint);
			}
		}
		else {	
			// should never get here
		}	

	},
		
	getRequestParams:function(){		
		//This assumes that the docType is always carried in the event for drill in at process instance view
		var docType = this._getDocType();
		this.docTypeProvider = com.ibm.bpm.wpd.core.model.providers.ModelTypeProviderRegistry.getInstance().getDocTypeProvider(docType);
		
		var parms = this.docTypeProvider.getRequestParams(this.event);
		return parms;
	},
	

	
	onSuccess : function (response){
		
		//console.debug( "onSuccess open: " + new Date().getTime());
		
		var documentObj;		
		if(!!this.docTypeProvider.processResponseData){
			documentObj = this.docTypeProvider.processResponseData(response.data, this.event);		
		}
		else{
			documentObj = response.data;
		}
		
		// create the view state model
		var documentName = this._getDocumentName(documentObj);
		var documentId = this._getDocumentId();
		var documentType = this._getDocType();		
		var vsmObj = {documentName: documentName, documentType:documentType, documentId:documentId, document:documentObj};
		debugger;
		vsmObj.branchOrSnapshotId = this._getBranchOrSnapshotId();
		var vsm = this.createViewStateModel(vsmObj);
		
		// create the new doc
		var viewerAttachPoint = document.createElement("div");
		viewerAttachPoint.style.height = "600px";
		viewerAttachPoint.style.width = "500px";
		
		// check to see if this is a graphical diagram
		if ((this.docTypeProvider.getDocType() == "BPDDocType")||(this.docTypeProvider.getDocType() == "Service")) {
			viewerAttachPoint.style.overflowX = "hidden"; 
			viewerAttachPoint.style.overflowY = "hidden"; 
		} else {
			viewerAttachPoint.style.overflowX = "scroll";  
			viewerAttachPoint.style.overflowY = "scroll"; 
		}

	
		// create sub div for the document to get added to
		//    - seems that a fixed div directly in the table cell cannot be made to scroll
		var inDiv = document.createElement("div");
		viewerAttachPoint.appendChild(inDiv);
		
		
		// populate the right pane in the view depending on whether this is a from or to document.
		if (this.event.fromOrTo == "from") {
			this.event.view.copyFromSectionBody.appendChild(viewerAttachPoint);
		}else {
			this.event.view.copyToSectionBody.appendChild(viewerAttachPoint);
		}

		var viewer = this.docTypeProvider.createViewer(vsm, inDiv);	

		viewer.onShow();
		
	},
	
	/**
	 * Create and returns ViewStateModel
	 * @param data
	 * @returns a ViewStateModel for the data
	 */
	createViewStateModel : function (data){
		
		var vsm;
		
		//find docTypeProvider
		//var docTypeProvider = com.ibm.bpm.wpd.core.model.providers.ModelTypeProviderRegistry.getInstance().getDocTypeProvider(data.documentType);
		if (!!this.docTypeProvider){
			vsm = this.docTypeProvider.createViewStateModel(data);
		} else{
			vsm = new com.ibm.bcompass.core.model.viewstate.ViewStateModel();
		}
		
		//ViewStateModel has a name, type and documentId, branchOrSanpshotId and subProcParentId
		//Keep the information needed by ViewStateModel in vsm, because the json object format are controlled by each DocumentViewer
		vsm.name = data.documentName;
		vsm.documentId = data.documentId;
		vsm.type = data.documentType;
		vsm.branchOrSnapshotId = data.branchOrSnapshotId;
		vsm.openSideBySide = true;

				
		//documentModel. No need to ask the docType provider to do this for the webviewer
		var dm = data.document; //this.createDocumentModel(documentObj);
		vsm.setDocumentModel(dm);	
			
		return vsm;
	},
	
	_getDocumentId:function(){
		if(!!this.event.documentId){
			return this.event.documentId;
		}		
		var docType = this._getDocType();
		this.docTypeProvider = com.ibm.bpm.wpd.core.model.providers.ModelTypeProviderRegistry.getInstance().getDocTypeProvider(docType);
	   
		var docId = this.docTypeProvider.getDocumentId(this.event);
		if(!docId){
			throw new Error("No Unique document Id is provided");
		}
		return docId;
	},
	
	_getDocumentName:function(documentObj){
		if(!!this.event.documentName){
			return this.event.documentName;
		}
		else if( !!documentObj && !!documentObj.name){
			return documentObj.name;
		}
		//for process instance view
		var docType = this._getDocType();
		//var docTypeProvider = com.ibm.bpm.wpd.core.model.providers.ModelTypeProviderRegistry.getInstance().getDocTypeProvider(docType);
		return this.docTypeProvider.getDisplayName(this.event);		
	},
	
	_getBranchOrSnapshotId:function(){
		var branchOrSnapshotId = this.event.branchOrSnapshotId;
		if(!branchOrSnapshotId){
			// see if this is to populate the compare/copy 'from' pane or 'to' pane
			if (this.event.fromOrTo == "from") {
				branchOrSnapshotId = !!this.event.branchId ? this.event.branchId : this.event.snapshotId;		
			} else {
				branchOrSnapshotId = this.event.branchId;
			}
		}
		return branchOrSnapshotId;
	},
	
	_getDocType: function(){
		var documentType;
		if (!!this.event.type){
			documentType = com.ibm.bpm.wpd.core.model.providers.ModelTypeProviderRegistry.getInstance().getDocumentTypeByPO(this.event.documentId, this.event.type);
		}
		return documentType;
	}
});
