dojo.provide("com.ibm.bpm.wpd.copy.control.commands.QueryCopyCommand");

dojo.require("com.ibm.bpm.wpd.core.model.viewstate.ViewStateModel");
dojo.require("com.ibm.bpm.wpd.core.control.commands.RestCommand");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");

/**
 * Command to load the Copy Asset data and launch the Copy view
 */
dojo.declare("com.ibm.bpm.wpd.copy.control.commands.QueryCopyCommand", [com.ibm.bpm.wpd.core.control.commands.RestCommand], {
	
	/**
	 * Exceute the REST call to obtain the copy data
	 */
	execute : function () {
				
		// always obtain the latest assets list from server
		//this.callbackFunction = this.event.params.callbackFunction;
		if (!!dojo.global.BPM.WPD_useWLEServer){		
			//this.sendRequest();
			this.sendRequestForData();
		}
		else{
			this.sendRequestForDatajson();
		}
	},
	
	/**
	 * Create the REST call and send the call to the server
	 */
	sendRequestForData : function(){
		
		var restCall = "/v1/assets/compare?destBranchId=" + this.event.params.track2Id + "&srcSnapshotId=" + com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().snapshotId;
		
		com.ibm.bpm.wpd.core.control.WpdRestHandler.getInstance().performRequest("get", 
				restCall, dojo.hitch(this, this.onSuccess), dojo.hitch(this, this.onError),null, null, null, null, null,null,true);
	},
	
	
	sendRequestForDatajson : function(){

        var xhrArgs = {
                url: com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_WEBVIEWER) + "/prototype/copyData.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, this.onSuccess),
                error: dojo.hitch(this, this.onError)
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
		
		//var response = {};
		//this.onSuccess(response);
	},
	
	
	getRequestParams:function(){
		// For now will just return the asset data as I'm testing the view sizing.
		var branchId = this.event.params.branchId;
		var snapshotId = this.event.params.snapshotId;
		
		var params = {};
		params.action = "get";
		params.url = "/v1/assets" + "?";
		if (!!branchId){
			params.url = params.url + "branchId=" + branchId;
		}
		else{
			params.url = params.url + "snapshotId=" + snapshotId;
		}
		
		return params;
	},
		
	/**
	 * Successful REST result. Extract the data from the response and build the copy model. Create a breadcrumb for user navigation and launch the
	 * copy view
	 * @param response
	 */
	onSuccess : function (response){
		// assetModel
		var copyAssetModel = this.createCopyAssetsModel(response.data);
		copyAssetModel.copyToTrack = this.event.params.track2Name;
		copyAssetModel.copyToTrackId = this.event.params.track2Id;
		
		dojo.requireLocalization("com.ibm.bpm.wpd.copy.view", "CopyResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.copy.view", "CopyResources");
		
		// Until we get the rest call, just use the main asset model
		var branchDisplayName =com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().brachName;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(branchDisplayName);
		if (dir=="rtl") {
			branchDisplayName = "<span dir='rtl'>" + branchDisplayName + "</span>";
		}
		var snapshotDisplayName = com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().snapshotName;
		dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(snapshotDisplayName);
		if (dir=="rtl") {
			snapshotDisplayName = "<span dir='rtl'>" + snapshotDisplayName + "</span>";
		}
		var track2DisplayName = this.event.params.track2Name;
		dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(track2DisplayName);
		if (dir=="rtl") {
			track2DisplayName = "<span dir='rtl'>" + track2DisplayName + "</span>";
		}
		var title = com.ibm.bpm.wpd.util.Helper.format(this.messages.CL_BREADCRUMB,
				branchDisplayName,
				snapshotDisplayName,
				dojo.date.locale.format(new Date(com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().snapshotDate)),
				track2DisplayName);


		com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance()._copyModel = copyAssetModel;
		var name = !!com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().snapshotName  
			? com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().snapshotName : com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().branchName;
		var vsm = new com.ibm.bpm.wpd.core.model.viewstate.ViewStateModel();
		vsm.type = "copyListModel";

		vsm.name = title;
		vsm.documentId = "copyListModel";
		vsm.branchOrSnapshotId = this.event.params.snapshot2;
		
		// Used to trick the event manager to force reload
		if (this.event.params.snapshot2 == "tip");
			vsm.branchOrSnapshotId = "" + new Date().getTime();
		var ssm = com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance();
		ssm.addOpenViewStateModel(vsm);
		//find the one and set it
		ssm.setActiveViewStateModel(vsm);
	},
	
	createCopyAssetsModel : function (data){
		var am = {};
		
		var type = "copyListModel";	

		//find provider
		var provider = com.ibm.bpm.wpd.core.model.providers.ModelTypeProviderRegistry.getInstance().getDocTypeProvider(type);
		if (!!provider){
			am = provider.createDataModel(data);
		}
		else{
			console.error("failed to look up CopyModelProvider for type=", type);
		}
		
		return am;
	}
});