dojo.provide("com.ibm.bpm.wpd.copy.model.CopyModelCoordinator");

dojo.require("com.ibm.bpm.wpd.core.model.MObject");



/**
 * Will use this to sync up the assets and their states (checked or unchecked). As well coordinate a check in
 * one accordion with another.
 * Note: this is a first crack. I'm leary about using events do to the amount of events and listeners I'd have to
 * create. As well the framework does not destroy the widget when it closes it (tries to cache it for speed).
 */
dojo.declare("com.ibm.bpm.wpd.copy.model.CopyModelCoordinator", [ com.ibm.bpm.wpd.core.model.MObject ], {
	
	assets: [],
	registeredInterests: {},

	/**
	 * Insert asset into the coordinator. Ignore if the asset has already been added
	 */
	insertAsset: function(assetValue) {
		if (!this._exists(assetValue))
			this.assets[this.assets.length] = assetValue;
	},
	
	/**
	 * The asset has been either checked or unchecked for copy. Update the asset as to it new state. As well update any checkboxes that
	 * have associated themselves to the asset
	 * @param id
	 * @param newValue
	 * @param fromWidget
	 */
	checkAsset: function(id, newValue, fromWidget) {
		for (var i=0; i<this.assets.length;i++) {
			if (this.assets[i].assetId == id)
				this.assets[i].assetChecked = newValue;
		}
		
		// Now check all of them that are registered
		var registeredCheckboxes = this.registeredInterests[id];
		for (var i=0;i<registeredCheckboxes.length;i++) {
			registeredCheckboxes[i].setChecked(newValue);
		}
	},
	
	/**
	 * Return an array of the uids of the checked assets
	 * @returns {String}
	 */
	getCheckedAssets: function() {
		var assetids = "";
		for (var i=0; i<this.assets.length;i++) {
			if (this.assets[i].assetChecked == true)
				assetids = assetids + this.assets[i].assetId + " ";
		}
		
		return assetids;
	},
	
	/**
	 * Return an array of the uids of the checked assets
	 * @returns {String}
	 */
	getCheckedAssetsAsArray: function() {
		var assetids = [];
		for (var i=0; i<this.assets.length;i++) {
			if (this.assets[i].assetChecked == true) {
				var tempAsset = {};
				tempAsset.id = this.assets[i].assetId;
				tempAsset.name = this.assets[i].assetName;
				tempAsset.imageLocation = this.assets[i].imageLocation;
				assetids[assetids.length] = tempAsset;
			}
		}
		
		return assetids;
	},
	
	_getAssetValue: function(id) {
		for (var i=0; i<this.assets.length;i++) {
			if (this.assets[i].assetId == id)
				return this.assets[i].assetChecked;
		}
		
		return false;
	},
	
	_exists: function(newValue) {
		for (var i=0; i<this.assets.length;i++) {
			if (this.assets[i].assetId == newValue.assetId)
				return true;
		}
		
		return false;
	},
	
	/**
	 * Register interest in an asset. When that asset is updated I will as well be updated
	 * @param id
	 * @param button
	 */
	registerInterestIn: function(id, button) {
		if (!this.registeredInterests[id])
			this.registeredInterests[id] = [];
		else {
			button.setChecked(this._getAssetValue(id));
		}
		
		this.registeredInterests[id][this.registeredInterests[id].length] = button;
	},
	
	reset: function() {
		this.assets=[];
		this.registeredInterests={};
	}

});


dojo.declare("com.ibm.bpm.wpd.copy.model.AssetValue", [ com.ibm.bpm.wpd.core.model.MObject ], {
	
	assetId:'',
	assetName:'',
	assetChecked:false,
	newAsset:false,
	changedAsset:false,
	conflictAsset:false,
	imageLocation:'',
	
	constructor : function(id, checked, na, ca, cona,name, imageLocation) {
		this.assetId = id;
		this.assetChecked = checked;
		this.newAsset = na;
		this.changedAsset = ca;
		this.conflictAsset = cona;
		this.assetName = name;
		this.imageLocation = imageLocation;
	}

});