dojo.provide("com.ibm.bpm.wpd.copy.model.RecursiveDependency");

dojo.require("com.ibm.bpm.wpd.core.model.MObject");


/**
 * Collect all the direct and indirect dependencies for a given poId. Cached currently searched paths for improved performance.
 * NOTE: This is not the way to do this. There are implications when trying to resolve these on the client. The browser
 * will detect that a recursion is going on and will trigger an exception along the lines on an unresponsive script. The number
 * of recursions allows is dependant on the browser and what is currently loaded into the browser. As large project with a semi-deep
 * tree will trigger the error.
 * For now a try..catch will be placed around the code to prevent deep dependencies from crashing.
 * This needs to be reinvestigated bewfore the release
 * @deprecaited No longer in use
 */
dojo.declare("com.ibm.bpm.wpd.copy.model.RecursiveDependency", [com.ibm.bpm.wpd.core.model.MObject], {
	
	flattenedStructure: {},
	alreadyVisited: {},
	
	constructor: function(listToParse) {
		for (asset in listToParse) {
			var assets = listToParse[asset];
			for (var i=0;i<assets.length;i++) {
				this.flattenedStructure[assets[i].poId] = assets[i].dependencies;
			}
		}
	},
	
	findStructureFor: function(poId) {
		// First off check if we already visited it. Might already have it calculated.
		if (this.alreadyVisited[poId])
			return this.alreadyVisited[poId];
		
		// Get the dependencies from the flattened file
		var dep = this.flattenedStructure[poId];
		if (dep.length == 0) {
			var emp = [];
			this.alreadyVisited[poId] = emp;
			return emp;
		}
		
		var internalVisisted = {};
		internalVisisted[poId] = "placeholder";
		
		var allDeps = this.findDependenciesFor(poId,internalVisisted);
		this.alreadyVisited[poId] = allDeps;
		return allDeps;
	},
	
	findDependenciesFor: function(poId, internalVisited) {
		var depsForLevel = [];
		try {
			var dep = this.flattenedStructure[poId];
			
			if (!dep)
				return depsForLevel;
			
			for (var i=0; i<dep.length;i++) {
				var depPoId = dep[i].poId;
				if (internalVisited[depPoId])
					continue;
				
				
				depsForLevel[depsForLevel.length] = dep[i];
				
				internalVisited[depPoId] = "placeholder";
				
				var subDeps = this.findDependenciesFor(depPoId, internalVisited);
				depsForLevel = depsForLevel.concat(subDeps);
			}
		}
		catch(ex) {
			console.debug("Hit a recursion limit error. JSON design issue");
		}
		
		return depsForLevel;
	},
	
	destroy: function() {
		flattenedStructure = null;
		alreadyVisited = null;
	}
	
	
});