dojo.provide("com.ibm.bpm.wpd.core.control.WpdRestHandler");

/**
 */
dojo.declare("com.ibm.bpm.wpd.core.control.WpdRestHandler", null, {
	
	iContext: null,
	serviceURLRoot: null, //will hardcode some value here
	
	_outstandingRequest: null,
	
	/**
	 * Value is set from bootstrap.js
	 */
	constructor : function (iContext, serviceURLRoot) {
		this.iContext = iContext;
		this.serviceURLRoot = serviceURLRoot;
		this._outstandingRequest = {};
	},
	
	calculateRemoteRequestURL : function(pathURL, acceptType, webAppName) {
		var url = com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_REPOSITORY) + "/repository/RemoteProcessDocService";  // servlet url
		var registrationId = com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().remoteId;
		url += "?registrationId=" + encodeURIComponent(registrationId);
		if(!!acceptType)
			url += "&acceptType=" + encodeURIComponent(acceptType);
		if(!!webAppName)
			url += "&webAppName=" + encodeURIComponent(webAppName);
		url += "&pathURL=" + encodeURIComponent(pathURL);
		return url;
	},
	
	
	performRequest: function(verb, url, loadFn, errorFn, headers, isSync, message, deferred, handleAsType, xMethodOverride, preventCache, timeoutValue) {
		var requestUrl = this.serviceURLRoot + url;
		if (!!com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().remoteId) {
			requestUrl = this.calculateRemoteRequestURL(url, null, com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_BPMREST);
		}		
		this._performRequest(verb, requestUrl, loadFn, errorFn, headers, isSync, message, deferred, handleAsType, xMethodOverride, null, null, preventCache, timeoutValue);
	},

	performProxiedRequest: function(verb, serviceURLRoot, url, loadFn, errorFn, headers, isSync, message, deferred, handleAsType, xMethodOverride, user, password, timeoutValue) {
		var requestUrl = serviceURLRoot + url;
		this._performRequest(verb, requestUrl, loadFn, errorFn, headers, isSync, message, deferred, handleAsType, xMethodOverride, user, password, timeoutValue);
	},
	
	_performRequest: function(verb, requestUrl, loadFn, errorFn, headers, isSync, message, deferred, handleAsType, xMethodOverride, user, password, preventCache, timeoutValue) {		
		if(!!this.isOutstandingUrl(requestUrl)){
			return;
		}
		this._outstandingRequest[requestUrl] = true;
		
		var request = {};
		request.sync = !!isSync;
		
		request.load = function(response){
			com.ibm.bpm.wpd.core.control.WpdRestHandler.getInstance().removeUrl(requestUrl);
			loadFn(response);
		};
		//request.url = this.iContext.io.rewriteURI(requestUrl);
		request.url = requestUrl;
		
		if(timeoutValue == undefined || timeoutValue == null){
			request.timeout = 1800000; //change this to 30 mins
		}
		else{
		   request.timeout = timeoutValue;  // 0 means infinite
		}
		
		if (!handleAsType) {
			request.handleAs = "json";
		} else {
			request.handleAs = handleAsType;
		}
		
		if (preventCache)
			request.preventCache = preventCache;

		//request.preventCache = true;
		if (errorFn) {
			request.error = function(response){
				com.ibm.bpm.wpd.core.control.WpdRestHandler.getInstance().removeUrl(requestUrl);
				errorFn(response);
			};    	
		}
		if (headers) {
			request.headers = dojo.clone(headers);
		} else {
			request.headers = new Object();
		}
		
		if (!!xMethodOverride) {
			request.headers["X-Method-Override"] = xMethodOverride;
		}
		
		if (!!user){
			request.user = user;
		}
		
		if (!!password){
			request.password = password;
		}

		if (verb == "put" || verb == "rawPut"){
			request.mimetype = "text/json;charset=utf-8";
			request.putData = message;
		} else if (verb == "post" || verb == "rawPost") {
			request.mimetype = "text/json;charset=utf-8";
			request.postData = message;
		}
		
		request.headers["Accept"] = "application/json";

		if (verb == "get") {
			deferred = dojo.xhrGet(request);
		} else if (verb == "rawPost") {
			request.postData = message;
			dojo.rawXhrPost(request);
		} else if (verb == "post") {
			dojo.xhrPost(request);
		} else if (verb == "rawPut") {
			dojo.rawXhrPut(request);
		} else if (verb == "put") {
			dojo.xhrPut(request);
		} else if (verb == "delete") {
			dojo.xhrDelete(request);
		}
		
	},
	
	isOutstandingUrl:function(url){
		return !!this._outstandingRequest[url];
	},
	
	removeUrl:function(url){
		delete this._outstandingRequest[url];
	}
	
	
});




com.ibm.bpm.wpd.core.control.WpdRestHandler.getInstance = function() {
	// summary: 
	//	retrieve the singleton WpdRestHandler instance 
	//  It is initialized in bootstrap.js during start up 
	return dojo.global.BPM.WPD_RestHandler;
};
	