dojo.provide("com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel");

dojo.require("com.ibm.bpm.wpd.core.model.MObject");
dojo.require("com.ibm.bpm.wpd.core.control.notification.Notification");

/**
 * Model contains view states for currently openned views
 * 			AssetListView
 * 			Instances of BaseViewer for different document type
 * Also store a reference to the currently visible ViewState
 */
dojo.declare("com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel", [com.ibm.bpm.wpd.core.model.MObject], { 
	
	//constants : todo use statics
	ACTIVE_VIEWSTATEMODEL: "_activeViewStateModel",
	OPEN_VIEWSTATEMODELS: "_openViewStateModels",
	STATE_PROCESS_CENTER: "ProcessCenter",
	STATE_PORTAL: "Portal",
	STATE_INSPECTOR : "Inspector",
			
	_currentUser: null,				// type: User
	_openViewStateModels: null, 	// type: [ViewStateModel] 
	_activeViewStateModel: null,	// type: ViewStateModel
	_processAppModel: null,         // type: ProcessAppModel
	_filteredProcessAppModel: null,   // type: ProcessAppModel unpdated model based on applied filter
	_cachedViewStateModels : null,
	projectName:null,
	projectId:null,	
	
	snapshotName:null,
	snapshotId: null,
	
	branchId: null,
	branchName: null,
	
	breadCrumbName: null,
	breadCrumbImage: null,
	remoteId: null,
	showAssetListView : true,
	
	btdPreference: null,
	
	//instanceId: null,
	
	state: null,  //state in which the component is used. type:String processCenter/Portal/Inspector
	
	constructor : function(state) {
		this._openViewStateModels = [];
		this._cachedViewStateModels = [];
		this.state = state;
	},
	
	getActiveViewStateModel : function(){
		return this.getAttribute("_activeViewStateModel");
	},

	setActiveViewStateModel : function(value){
		this.setAttribute("_activeViewStateModel", value);
	},
	
	addCachedViewStateModel: function(vsm){
		this._cachedViewStateModels.push(vsm);
	},
	
	getOpenViewStateModels : function(){
		return this.getAttribute("_openViewStateModels");
	},
	
	addOpenViewStateModel : function(value){
		this.getOpenViewStateModels().push(value);
		
		
		//Dispatch the notification here just for trying new ways.
		//later will change _openViewStateModels to use MCollection, the notification will be generated by MCollection. Then I will change 
		//all the array notation[] to Mcollection.get for  accessing element
		
		
		//The notification is not needed anymore for a webViewer
		var index = this.getOpenViewStateModels().length - 1;
		var n = new com.ibm.bpm.wpd.core.control.notification.Notification(2, this, "_openViewStateModels", null, value, index); 
		dojo.publish(this.declaredClass + "::" + "_openViewStateModels", [n]);
	},
	
	findViewStateModelForModelType: function (type){
		var i;
		var esm;
		for (i=0; i<this._openViewStateModels.length; i++ ){
			esm = this._openViewStateModels[i];
			if (esm.type == type){
				return esm;
			}
		}
		return null;
	},
	
	findOpenViewStateModelForDocId: function(documentId, branchOrSnapshotId){
		return this._findViewStateModelForDocId(this._openViewStateModels, documentId, branchOrSnapshotId);
	},
	
	findCachedViewStateModelForDocId: function(documentId, branchOrSnapshotId){
		return this._findViewStateModelForDocId(this._cachedViewStateModels, documentId, branchOrSnapshotId);
	},
	
	removeCachedViewStateModel : function(vsm){
		var current, i;
		for (i=0; i<this._cachedViewStateModels.length; i++ ){
			current = this._cachedViewStateModels[i];
			if (current == vsm){
				this._cachedViewStateModels.splice(i, 1);
				return;
			}
		}
	},
	
	_findViewStateModelForDocId : function (list, documentId, branchOrSnapshotId){
		var i;
		var esm;
		for (i=0; i<list.length; i++ ){
			esm = list[i];
			if (!!esm.documentId && esm.documentId == documentId
					&& (!esm.branchOrSnapshotId || esm.branchOrSnapshotId == branchOrSnapshotId)){
				return esm;
			}
		}
		return null;
	},
	
	
	
	removeViewStateModelForDocId : function (documentId, branchOrSnapshotId){
		var i;
		var esm;
		for (i=0; i<this._openViewStateModels.length; i++ ){
			esm = this._openViewStateModels[i];
			if (!!esm.documentId && esm.documentId == documentId && esm.branchOrSnapshotId == branchOrSnapshotId){
			
				break;
			}
		}
		if (i >=0 && i<this._openViewStateModels.length){
			var value = this._openViewStateModels[i];
			//this._openViewStateModels.splice(i, 1);
			this._removeViewStateModel(value, i);
			//The notification is not needed anymore
			//var n = new com.ibm.bpm.wpd.core.control.notification.Notification(3, this, "_openViewStateModels", value, null, i); 
			//com.ibm.bpm.wpd.core.control.notification.NotificationManager.getInstance().registerNotification(n);
		}
	},
	
	_removeViewStateModel:function(vsm, index){
		this._openViewStateModels.splice(index, 1);
		var n = new com.ibm.bpm.wpd.core.control.notification.Notification(3, this, "_openViewStateModels", vsm, null, index);
		dojo.publish(this.declaredClass + "::" + "_openViewStateModels", [n]);
	},
	
	removeTrailingViewStateModel:function(activeVsm){
		var openViewStates = this.getOpenViewStateModels();
		var i;
		var vsm;
		for (i=openViewStates.length - 1; i>=0; i-- ){
			
			vsm = openViewStates[i];
			if(!activeVsm 
					|| vsm.documentId != activeVsm.documentId ||  vsm.branchOrSnapshotId != activeVsm.branchOrSnapshotId){
				//openViewStates.splice(i, 1);
				//var n = new com.ibm.bpm.wpd.core.control.notification.Notification(3, this, "_openViewStateModels", vsm, null, i); 
				//com.ibm.bpm.wpd.core.control.notification.NotificationManager.getInstance().registerNotification(n);
				this._removeViewStateModel(vsm, i);
			}
			else{
				break;
			}
		}
	},
	
	setProcessAppModel : function (assetModel, context) {
		this.setAttribute("_processAppModel", assetModel, context);
	},
	
	getProcessAppModel : function(){
		return this._processAppModel;
	},
	
	setFilteredProcessAppModel : function (assetModel, context) {
		this.setAttribute("_filteredProcessAppModel", assetModel, context);
	},
	
	getFilteredProcessAppModel : function(){
		return this._filteredProcessAppModel;
	},
	
	setBTDPreference : function (btdValue) {
		this.btdPreference = btdValue;
	},
	
	getBTDPreference : function(){
		return this.btdPreference;
	},
	
	setProjectInfor: function (app){
		if (!!app){
			this.projectId = app.projectId;
			this.projectName  = app.projectName;
			this.snapshotId = app.snapshotId;		
			this.snapshotName = app.snapshotName;
			this.branchId = app.branchId;
			this.branchName = app.branchName;
			if (!!app.breadCrumbName)
				this.breadCrumbName = app.breadCrumbName;
			if (!!app.breadCrumbImage)
				this.breadCrumbImage = app.breadCrumbImage;
			if (!!app.remoteId)
				this.remoteId = app.remoteId;
			if (app.showAssetListView == null)
				this.showAssetListView = true;
			else
				this.showAssetListView = app.showAssetListView;
		}
	}
	
});


com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance = function() {
	// summary: 
	//	retrieve the singleton ShellStateModel instance 
	//  It is initialized in bootstrap.js during start up 
	return dojo.global.BPM.WPD_ShellStateModel;
};