dojo.provide("com.ibm.bpm.wpd.core.view.BaseViewer");

dojo.require("dijit._WidgetBase");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");



/**
 * BaseViewer
 */
dojo.declare("com.ibm.bpm.wpd.core.view.BaseViewer", [ dijit._WidgetBase,
		dijit._Templated ], {

	widgetsInTemplate : true,
	
	//The ViewStateModel for this viewer
	viewStateModel:null,
		
	constructor : function(args) {
		// dojo.safeMixin(this, args); //Use this when there are lots of them
		 this.viewStateModel = args.viewStateModel;
	},

	postCreate:function(){
		
		
	},
	
	/**
	 * Hide the shareLink
	 */
	_hideLink: function() {
		var shareTable = this._findDomNodeByTypeAndClassName("td", "shareLinkClass");
		if (shareTable) {
			shareTable.style.visibility = 'hidden';
		}
	},
	
	/**
	 * Show the shareLink
	 */
	_showLink: function() {
		var shareTable = this._findDomNodeByTypeAndClassName("td", "shareLinkClass");
		if (shareTable) {
			shareTable.style.visibility = 'visible';
		}
	},
	
	/**
	 * Search for the section and class defined
	 * @param tagName
	 * @param className
	 * @returns
	 */
	_findDomNodeByTypeAndClassName: function(tagName, className) {
		
		var rootNode = this.domNode;
		
		var aNode = dojo.query(tagName + "." + className, rootNode);
		if (!!aNode && aNode.length > 0) {
			return aNode[0];
		} else {
			return null;
		}
	},
	
	/**
	 * The viewer become visible
	 * Override this function if you have anything to do onshow
	 */
	onShow : function() {

		this._shareLinkEnablement();
	},
	
	/**
	 * Determine whether the share link should be diaplayed. For now we don't want to display the link is the view in being shown via the
	 * copy asset list (as the original idea was to show the asset in the asset list keyed on snapshot, what would copy mean). This will
	 * be called via the onShow on the viewer. This is called when the view is displayed. We are doing it here, as opposed to the port create
	 * because the view could be cached if it was already open. In this case the postCreate is not called (whereas onShow is always called). 
	 */
	_shareLinkEnablement: function() {
		
		// Look for the breadcrumb information in the global
		var breadcrumb = com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().breadcrumb;
		
		// Not sure where we're coming from. Could be portal, leave the original behaviour
		if (!breadcrumb)
			return;
		
			
		var breadcrumbModels = breadcrumb._divToModel;
		var hideLabel = false;
		
		// Loop through the breadcrumb entries. If copyListModel type is in any of the breadcrumb entries we will hide the shareLink
		for (var breadcrumbKey in breadcrumbModels) {
			if ("copyListModel" == breadcrumbModels[breadcrumbKey].type) {
				hideLabel = true;
				break;
			}
		}
		
		if(hideLabel) {
			this._hideLink();
		}
		else
			this._showLink();
	},
	
	/**
	 * The viewer become invisible
	 * Override this function if you have anything to do here
	 */
	onHide: function(){
		
	},

	/**
	 * Set the viewStateModel for this viewer
	 * @param viewStateModel
	 */
	setViewStateModel : function(viewStateModel) {
		this.viewStateModel = viewStateModel;
	}
});


dojo.global.copyLinkTooltipDialog =null;

com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref = function(copyLinkHref, rootId) {
	
	var copyLinkTooltipDialogName =    "copyLinkTooltipDialog";
	var copyLinkConnectorName =        "copyLinkConnector";	
	var copyLinkTooltipFocusNodeName = "copyLinkTooltipFocusNode";
	var copyLinkTooltipContainerName = "copyLinkTooltipContainer";
	var containerNodeCopyLink = null;
	var domeNodeTooltipDialog = null;
	var connectorNodeCopyLink = null;	
	var href="";
	
	dojo.requireLocalization("com.ibm.bpm.wpd.base", "DocumentationViewerResources");
	this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.base", "DocumentationViewerResources");
	
	/*
	 *  for proxy, IHS, etc. web server, URL will have no port
	 */
	
	if(copyLinkHref.port.length == 0){ 
		port = copyLinkHref.port;
	}else{
		port = ":" + copyLinkHref.port; 
	}
	
	
	// In the case of a subprocess we want to link to the root process as opposed to the subprocess. Add a optional variable
    // called rootId, if it's passed in use it otherwise default to the documentId global.
	// On a side note if might be safe to always have the view pass in it's doc id and snaphot/branch id. I'm not sure if the
	// global is always updated properly (maybe not in the case of drilldown)
	var docId = (com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().getActiveViewStateModel()).documentId;
	if (rootId)
		docId = rootId;
	
	href= com.ibm.bpm.wpd.base.getEndPointAndContextRootForModuleKey(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_ASSET, com.ibm.bpm.wpd.base.wpdConstant.BPM_KEY_EXTERNAL_CLIENT) + "/" + docId
    + "?";

    //var branchId=(com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance()).branchId;
	// The branchId is never actually set, we work with an internal snapshotId, however the true branchId is stored in something called tipInfo
	// First check is tipInfo is set, if it is the user is browsing the current
	// Second check is the snapshotId is equal to the vsm brnachOrSnaphotId. If it's the same we know the viewer has opened something
	// in the current process app. If they are different. The the view has opened up a refernce element in another toolkit.
	// Third is they are the same then use the tipInfo in the branchId's place to get the proper link
	var branchId = null;
	if (com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().tipInfo 
		&& (com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().snapshotId 
		    == com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().getActiveViewStateModel().branchOrSnapshotId)) {
		branchId = com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().tipInfo;
	}

	// Use the active vsm. The branchOrSnapshot should always be the snapshot
    branchId ? href = href + "branchId=" +branchId:
    	href = href + "snapshotId=" + com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance().getActiveViewStateModel().branchOrSnapshotId;
       
    copyLinkHref.protocol= "javascript:";   /// disable opening copy link onclick even 
   

    /*
     *  Create TooltipDialog for copy link
     */
    var content= "<p ><strong>" + href +"</strong></p><p style=\"margin-bottom : 0em;\">" + this.messages.COPY_LINK_TOOLTIP_DIALOG + "</p>";
    
    if( !dojo.global.copyLinkTooltipDialog){
	    var _copyLinkToolTip = dojo.declare("_copyLinkToolTip",[dijit.TooltipDialog], {
			templateString: '<div role="presentation" tabIndex="-1">' +
							'	<div class="dijitTooltipContainer"  role ="presentation">' +		
							'		<div class ="dijitTooltipContents dijitTooltipFocusNode '+ 
							      copyLinkTooltipFocusNodeName + "\""+
							'  data-dojo-attach-point="containerNode" role="dialog"></div>' +
							'	</div>' +
							'	<div class="dijitTooltipConnector" role="presentation" data-dojo-attach-point="connectorNode" ></div>' +
							'</div>' 
		});
	    dojo.global.copyLinkTooltipDialog = new _copyLinkToolTip({
	        id: copyLinkTooltipDialogName
	        });
	    
	    containerNodeCopyLink = dojo.global.copyLinkTooltipDialog.containerNode.parentNode;
	    if(containerNodeCopyLink != null) {
	        dojo.attr(containerNodeCopyLink, "style", {background: "#FFC"});
	        dojo.attr(containerNodeCopyLink, "style", {fontsize:  "8pt"});
	        dojo.attr(containerNodeCopyLink, "style", {border:  "2px #C0C09F solid"});        
	        dojo.addClass(containerNodeCopyLink,copyLinkTooltipContainerName);
	    }
	    
	    domeNodeTooltipDialog = dojo.global.copyLinkTooltipDialog.domNode;
	    if( domeNodeTooltipDialog != null){        
	        dojo.addClass(domeNodeTooltipDialog,copyLinkTooltipDialogName);
	        dojo.attr( domeNodeTooltipDialog, "style", {width:  "500px"});
	        
	       
	    }
	    
	
	    connectorNodeCopyLink= dojo.global.copyLinkTooltipDialog.connectorNode;
	    if( connectorNodeCopyLink != null){      
	        dojo.attr(connectorNodeCopyLink, "style", {background: "url(" + com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_WEBVIEWER) + "/widgets/com/ibm/bpm/wpd/images/tooltiparrow.png) no-repeat top left"});
	        dojo.attr(connectorNodeCopyLink, "style", {width:  "26px"});
	        dojo.attr(connectorNodeCopyLink, "style", {height:  "15px"});	 
	    	dojo.addClass(connectorNodeCopyLink,copyLinkConnectorName);
	    }
    }
   
     /*
      *  Display TooltipDialog for copy link
     */
    
    dojo.connect(copyLinkHref, "onclick", function(){
    	
    	copyLinkHrefPosition = dojo.position(copyLinkHref,true); 
    	dojo.global.copyLinkTooltipDialog.containerNode.innerHTML = content; /// set TooltipDialog content
		dijit.popup.open({
             popup: dojo.global.copyLinkTooltipDialog,
             around: copyLinkHref
         });		
		
	  });  
    
    dojo.connect(copyLinkTooltipDialog, "onOpen", function(event){     ///  set connector location
    	 if(copyLinkHrefPosition.x==event.x){
    	   dojo.attr(connectorNodeCopyLink, "style", {left:  "15px"});
    	 }else{
    	   dojo.attr(connectorNodeCopyLink, "style", {right:  "15px"}); 
    	 }
	  }); 
    
    dojo.connect(copyLinkHref, "onmouseout", function(event){
    	
    	if(event.relatedTarget!= null ){
    	
    	if(((event.relatedTarget.className).indexOf(copyLinkTooltipDialogName)==-1) && 
    			((event.relatedTarget.className).indexOf(copyLinkConnectorName)==-1) &&
    			((event.relatedTarget.className).indexOf(copyLinkTooltipFocusNodeName)==-1)&&
    			((event.relatedTarget.className).indexOf(copyLinkTooltipContainerName)==-1)&&
    			((event.relatedTarget.innerText).indexOf(href)==-1))
    		
    			dijit.popup.close(copyLinkTooltipDialog); 
    	}
    	
	  });
    
 
    dojo.connect(copyLinkTooltipDialog, "onMouseLeave", function(){  	
    	dijit.popup.close(copyLinkTooltipDialog);
    	
	  }); 

};