
dojo.provide("com.ibm.bpm.wpd.core.view.ViewManager");

dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("com.ibm.bpm.wpd.core.view.ViewerContentPane");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.StackContainer");
dojo.require( "dijit.layout.BorderContainer");
/**
 * ViewManager is used to control the open/close/show/hide/setModel for all viewers 
 */
dojo.declare("com.ibm.bpm.wpd.core.view.ViewManager",[ dijit._Widget, dijit._Templated, dijit.layout.StackContainer], {
	widgetsInTemplate : true,
	_subscriptionHandles : null,

	constructor : function(params) {		
		this._subscriptionHandles = [];
	},
	
	postCreate : function() {		
		this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel" + "::" + "_openViewStateModels", this, this.modelChanged));
		this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel" + "::" + "_activeViewStateModel", this, this.modelChanged));
	},
	
	/**
	 * Receives notifications, either via dojo subscription or direct listening at the instance level
	 */
	modelChanged : function(notification) {
	
		if (notification.attributeName == "_openViewStateModels" && notification.changeType == 3){
			//do we want to remove the content pane for it? it is cached now for fast response time
			//TODO Remove some cached content pane (with editor) based on  based on some performance measurement
		}
		else if (notification.attributeName == "_activeViewStateModel"){		
			if(!!notification.newValue){
				try{ 
					this._openViewer(notification.newValue);
				}
				catch(err){
					//1. log error to server, log into debug console,  and show error dialog if it is severe					
					var className = this.declaredClass;
					var methodName = "_openViewer";
					var message = " Catch error when opening viewer of type (" + notification.newValue.type + ")" +
					    ' \n Error Type: ' + err.name +
						' \n Error Message: ' + err.message;
						
					var loadingImg = dojo.byId("wpdLoadingImag");
					if (!!loadingImg) {
						loadingImg.style.display = "none";
					}
										
					com.ibm.bpm.wpd.base.errorhandling.WpdLogger.logError(className, methodName, message, err.level);
				}
			}
		}
	},

    /**
     * Finds/Creates and shows a viewer for the viewStateModel
     * @param viewStateModel It contains the information to be displayed by the viewer
     */
    _openViewer : function(viewStateModel /* IViewStateModel */) {

		// 1. Hide the current viewer
		var cpToHide = this.selectedChildWidget;					
		if (!!cpToHide && !!cpToHide.viewer) {	
			if(!!dojo.hasClass(cpToHide.domNode, "bpmPrintShow")){
				 dojo.removeClass(cpToHide.domNode, "bpmPrintShow");
			}

			cpToHide.viewer.onHide();
		}
			
		// 2. find whether an viewer for this ViewStateModel already exists. This depends on the implementation
		var viewer, viewerCp;
		var viewerCps = this.getChildren();
		var i;
		for (i=0; i<viewerCps.length; i++){
			viewerCp = viewerCps[i];
			//note that the branchOrSnapshotId maybe null when view the top level process instance
			if (!!viewerCp.viewStateModel && viewerCp.viewStateModel.type == viewStateModel.type 
					&& viewerCp.viewStateModel.documentId == viewStateModel.documentId 
					&& viewerCp.viewStateModel.branchOrSnapshotId == viewStateModel.branchOrSnapshotId){
				viewer = viewerCp.viewer;
				console.debug("found for vsm: " + viewStateModel.type + " " + viewStateModel.name);
				
				//we found the cached pane, so the response time here is good. let us use this time for some cleaning up work
				this._removeCachedPanes();
				break;
			}
		}
		
		var newlyCreated = false;
		var docType = viewStateModel.type;		
		var provider = com.ibm.bpm.wpd.core.model.providers.ModelTypeProviderRegistry.getInstance().getDocTypeProvider(docType);
		provider.setViewStateModel(viewStateModel);
		/*3. if a viewer for this ViewStateModel does not exist,  
		 */
		if (!viewer) {			
			
			//if reuse, then find the viewer for this doctype and reuse it if one exist
			if (provider.reuseViewer()){
				for (i=0; i<viewerCps.length; i++){
					viewerCp = viewerCps[i];
					if (!!viewerCp.viewStateModel && viewerCp.viewStateModel.type == viewStateModel.type){
						viewer = viewerCp.viewer;
						break;
					}
				}
			}
			
			if(!!viewer ){			
				viewer.setViewStateModel(viewStateModel);
				viewerCp.viewStateModel = viewStateModel;
			}
			//if not reuse or no existing one, then create one
			else{							
				viewerCp = new com.ibm.bpm.wpd.core.view.ViewerContentPane({ parseOnLoad: false, style: "height:98%;width:900px;"});
				newlyCreated = true;
				var viewerAttachPoint = document.createElement("div");
				viewerAttachPoint.style.height = "100%";
				viewerAttachPoint.style.width = "100%";
				if ( dojo.version.minor < 5 )
					viewerCp.attr("content", viewerAttachPoint);
				else
					viewerCp.set("content", viewerAttachPoint);
				this.addChild(viewerCp);
				this.selectChild(viewerCp);
				this.resize();	
				viewer = provider.createViewer(viewStateModel, viewerAttachPoint);	
			
				if (viewStateModel.type == 'assetListModel'){
					var self = this;				
					window.setTimeout(function() {
						dojo.byId("wpdLoadingImag").style.display = "none";
					},0);
				}
					
				viewerCp.viewStateModel = viewStateModel;
				viewerCp.viewer = viewer;								
			}
		}
		
		// 4 show it in the pane // select the pane for the viewer just opened.		
		if(!newlyCreated){
			this.selectChild(viewerCp);
			this.resize();	
		}

		// 5. ask viewer to show itself
		viewer.onShow();
		if(!dojo.hasClass(viewerCp.domNode, "bpmPrintShow")){
			dojo.addClass(viewerCp.domNode, "bpmPrintShow");
		}
	
	},

	_removeCachedPanes : function(){
		try{
			//debugger;
			var viewers =  dijit.registry.byClass("com.ibm.bpm.wpd.document.bpd.view.BPMNViewer");		
			var panes =  dijit.registry.byClass("com.ibm.bpm.wpd.core.view.ViewerContentPane");			
			console.debug("before: " + viewers.length + " panes: " + panes.length);
			
			 	var viewerCps = this.getChildren();			
				var ssm = com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel.getInstance();	
				var nodesNumber = this._countNotesNumber(ssm);
				
				//TODO: need to do some performance measurement to get more reasonable numbers
				if (viewerCps.length > 20 ||  nodesNumber > 1000){	
					console.debug("measurement: " + viewerCps.length  + " " + nodesNumber);
					var vsm, i;					
					/*remove one cached pane, using oldest out algorithm. 
					 *of course, not asset list view, nor remove the ones on breadcrumb path
					 */
					for (i=0; i < viewerCps.length; i++){
						vsm = viewerCps[i].viewStateModel;
						if(vsm.type != 'assetListModel'){				
							//first try to find it in the list of openViewStateModels, ie, in the breadcrum
							if(! ssm.findOpenViewStateModelForDocId(vsm.documentId, vsm.branchOrSnapshotId)){							
								viewerCp = viewerCps[i];	
								console.debug("remove pane for vsm: " + vsm.type + " " + vsm.name);
								this._removeViewerAndCp(viewerCp);
								ssm.removeCachedViewStateModel(vsm);
							    break;  //remove 1
							}				
						}			
					}
			    }		
				viewers =  dijit.registry.byClass("com.ibm.bpm.wpd.document.bpd.view.BPMNViewer");		
				panes =  dijit.registry.byClass("com.ibm.bpm.wpd.core.view.ViewerContentPane");
				console.debug("after: " + viewers.length + " panes: " + panes.length);	
		}
		catch(err){
			var className = this.declaredClass;
			var methodName = "_removeCachedPanes";
			var message = " Catch error when remove cached viewers" +
			    ' \n Error Type: ' + err.name +
				' \n Error Message: ' + err.message;
			com.ibm.bpm.wpd.base.errorhandling.WpdLogger.logError(className, methodName, message, err.level);
		}	
	},
	
	_countNotesNumber : function(ssm){
		var i;
		var count=0;
		var dm;
		var vm;
		for (i=0; i<ssm._cachedViewStateModels.length; i++){
			if( !!ssm._cachedViewStateModels[i].getDocumentModel && !! ssm._cachedViewStateModels[i].getDocumentModel() &&  !!ssm._cachedViewStateModels[i].getDocumentModel()["items"]){
				count += ssm._cachedViewStateModels[i].getDocumentModel()["items"].length;
			}
		}
		return count;
	},
	
	_removeViewerAndCp : function (cp){
		var viewer = cp.viewer;
		//debugger;
		try{
			this.removeChild(cp);
			cp.destroyRecursive();			
		}
		catch(err){
		    throw {
		    		name : err.name,
		    		message: "Error when destroy viewer and its content pane: " + viewer.declaredClass
		    };
		}
	},
	
	destroyFlashPlayer : function (){
		var viewerCps = this.getChildren();
		for (i=0; i<viewerCps.length; i++){
			viewerCp = viewerCps[i];	
			viewer = viewerCp.viewer;
			if (!! viewer && viewer["destroyFlashPlayer"]){
				viewer.destroyFlashPlayer();
			}
		}
	}, 
	
	
	/**
	 * Destroy the widget
	 */
	destroy : function() {
		for ( var i = 0; i < this._subscriptionHandles.length; i++) {
			if(!!this._subscriptionHandles[i]){
				dojo.unsubscribe(this._subscriptionHandles[i]);
			}
		}
		this.inherited(arguments);
		com.ibm.bpm.wpd.base.cleanUpWebViewerGlobal();
	}
});
