dojo.provide("com.ibm.bpm.wpd.document.ParticipantGrp.view.ParticipantGrpViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");



/**
 * Participant Group viewer
 * 
 * 		Participant group has a couple of different types of data
 * 			- lists of users and groups
 *          - expression
 *      This implementation has a couple of different types of table to reflect the different types of data to show. We
 *      hide the one that is not needed and show the the one that is.
 */
dojo.declare("com.ibm.bpm.wpd.document.ParticipantGrp.view.ParticipantGrpViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.ParticipantGrp.view",
	"templates/ParticipantGrpViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	pgModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 
	
	},

	
	postCreate : function() {
		
		
		if ( this.debug ){
			//Get all the data from the server (Direct file read in this case).
			this.getpgData();
		}else {
			//Get the server side data
			pgModel = this.viewStateModel.getDocumentModel();;
		}
		
		

		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (typeof(pgModel.richDesc) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(pgModel.richDesc));
			this.pgDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(pgModel.desc);
			if (dir=="rtl")
				this.pgDescription.setAttribute("style","direction:rtl; text-align:left;");

		} else { 
			this.pgDescription.innerHTML = "&nbsp;";
		}

		this.pgName.innerHTML = pgModel.name;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(pgModel.name);
		if (dir=="rtl")
			this.pgName.setAttribute("style","direction:rtl; text-align:left;");
		
		if (pgModel.type == "Expression") {
			this.pgStandardTableHeader.style.display = "none";
			this.pgDialogBody.style.display="none";
			this.fillInExpressionData();
		} else {
			this.pgExpressionTableHeader.style.display = "none";
			this.pgExDialogBody.style.display="none";
			this.fillInStandardTable();
		}
		
		/*
		 *  Create Copy link href Document 
		 */
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.paCopyLinkHref);
		
	},
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.ParticipantGrp.view", "ParticipantGrpViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.ParticipantGrp.view", "ParticipantGrpViewerResources");


	},
	
	
	
	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},

	

	//*****
	//Insert a new row into the table
	//		In this case it is a list of users and groups
	//
	//Returns: reference to the new row
	//*****
	fillInStandardTable: function()
	{
		
		var newRow = null;
		var m = pgModel;
		
		if (m.members != null) {
			
			var memLen = m.members.length;
			for (var i=0; i<memLen; i++) {
				newRow = this.insertRow(this.pgTableBody);
				this.addTextCellToRow( newRow, pgModel.members[i].name, this.bidiLeft, null);
				var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(pgModel.members[i].name);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
				this.addTextCellToRow( newRow, pgModel.members[i].type, this.bidiLeft, null);
				dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(pgModel.members[i].type);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
				if (typeof(pgModel.members[i].desc) !== "undefined") {
					this.addTextCellToRow( newRow, pgModel.members[i].desc, this.bidiLeft, null);
					var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(pgModel.members[i].desc);
					if (dir=="rtl") {
						if (newRow.cells[newRow.cells.length-1])
							newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
					}
				} else {
					this.addTextCellToRow( newRow, "", this.bidiLeft, null);
				}
			}
			
		} else if (m.type == "System") {
			newRow = this.insertRow(this.pgTableBody);
			this.addTextCellToRow( newRow, m.name, this.bidiLeft, null);
			this.addTextCellToRow( newRow, m.type, this.bidiLeft, null);
			this.addTextCellToRow( newRow, m.desc, this.bidiLeft, null);

		}
	
	},
	//*****
	//Insert a new row into the table
	//   	In this case this is an expression type of participant group
	//
	//Returns: reference to the new row
	//*****
	fillInExpressionData: function()
	{
		
		var newRow = null;
		
		newRow = this.insertRow(this.pgExTableBody);


		if (pgModel.orgExpression.ruleSetOperator == "Or") {
			this.addTextCellToRow( newRow, this.messages.PG_EXPRESSION_ORS, this.bidiLeft, null);
		} else {
			this.addTextCellToRow( newRow, this.messages.PG_EXPRESSION_ANDS, this.bidiLeft, null);
		}
        
		if (pgModel.orgExpression.rules != null) {
			var numRules = pgModel.orgExpression.rules.length;
			var str = null;
			var parms = [];
			var m = pgModel;
				
			// for all rules in the expression - there can be any number
			for (var i=0; i<numRules; i++) {
				newRow = this.insertRow(this.pgExTableBody);
				
				//*****
				//There are three types of rules, participant, expression, user attribute
				//*****
				if (pgModel.orgExpression.rules[i].ruleType == "Participant") {
					
					switch (pgModel.orgExpression.rules[i].matchType) {
					case "belong":
						parms[0] = this.messages.PG_EXP_PARTCIPANT_BELONG; 
						break;
					case "notBelong":
						parms[0] = this.messages.PG_EXP_PARTCIPANT_NOT_BELONG;
						break;
					}
					parms[1] = pgModel.orgExpression.rules[i].participantName;
					str = dojo.string.substitute(this.messages.PG_EXP_PARTICIPANT_STRING, parms);
					this.addBulletTextCellToRow( newRow, str, this.bidiLeft, null, pgModel.orgExpression.rules[i]);
				} 
				else if (pgModel.orgExpression.rules[i].ruleType == "Expression") {	
					
					switch (pgModel.orgExpression.rules[i].matchType) {
					case "match":
						parms[0] = this.messages.PG_EXP_EXPRESSION_MATCH; 
						break;
					case "notMatch":
						parms[0] = this.messages.PG_EXP_EXPRESSION_NOT_MATCH;
						break;
					}
					
					parms[1] = pgModel.orgExpression.rules[i].value;
					str = dojo.string.substitute(this.messages.PG_EXP_EXPRESSION_STRING, parms);
					this.addBulletTextCellToRow( newRow, str, this.bidiLeft, null, null);
				} 
				else if (pgModel.orgExpression.rules[i].ruleType == "UserAttribute") {
					
//					PG_EXP_USERATTRIBUTE_EQUAL: "equal to",
//					PG_EXP_USERATTRIBUTE_NOT_EQUAL: "not equal to",
//					PG_EXP_USERATTRIBUTE_LESS: "less than",
//					PG_EXP_USERATTRIBUTE_LESS_OR_EQUAL: "less than or equal to",
//					PG_EXP_USERATTRIBUTE_GREATER: "greater than",
//					PG_EXP_USERATTRIBUTE_GREATER_OR_EQUAL: "greater than or equal to"
					parms[0] = com.ibm.bpm.wpd.util.Helper.text2htmlText_1(pgModel.orgExpression.rules[i].userAttributeName);
					
					switch (pgModel.orgExpression.rules[i].comparator) {
					case "Equal":
						parms[1] = this.messages.PG_EXP_USERATTRIBUTE_EQUAL; 
						break;
					case "NotEqual":
						parms[1] = this.messages.PG_EXP_USERATTRIBUTE_NOT_EQUAL;
						break;
					case "LessThan":
						parms[1] = this.messages.PG_EXP_USERATTRIBUTE_LESS;
						break;
					case "LessThanEqual":
						parms[1] = this.messages.PG_EXP_USERATTRIBUTE_LESS_OR_EQUAL;
						break;
					case "GreaterThan":
						parms[1] = this.messages.PG_EXP_USERATTRIBUTE_GREATER;
						break;
					case "GreaterThanEqual":
						parms[1] = this.messages.PG_EXP_USERATTRIBUTE_GREATER_OR_EQUAL;
						break;
					}
					
					parms[2] = pgModel.orgExpression.rules[i].userAttributeValue;
					str = dojo.string.substitute(this.messages.PG_EXP_USERATTRIBUTE_STRING, parms);
					this.addBulletTextCellToRow( newRow, str, this.bidiLeft, null, null);
				} else {
					alert("Invalid rule type in Team expression.");
				}
				
			}
		}
	},
	
	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function(pgTable)
	{
		var tbodyElem = pgTable;
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);

		return trElem;
	},
	//*****
	// Add a Bullet text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addBulletTextCellToRow: function(trElem, value, align, classname, modelProperty)
	{
//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), Started : " + value);
		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		
		// if this is a participant we want to enable drillin on the particpant
		if ((modelProperty != null) && (modelProperty.ParticipantId != null)) {

			//add the participant onto the end and give it an id so we can connect to it later
			tdElem.innerHTML = "<ul style='margin-top:0px;margin-bottom:0px;'><li>"
										+ value
										+ "<u id ="
										+modelProperty.ParticipantId
										+ "><b>"
										+ com.ibm.bpm.wpd.util.Helper.text2htmlText_1(modelProperty.participantName)
										+ "</b></u>";
										+"</li></ul>";  
			
			// style the participant as a link
			var l = dojo.byId(modelProperty.ParticipantId);
			l.style.cursor="pointer";
			dojo.addClass(l, "linkFieldFontDrillin");

			tdElem.modelProp = modelProperty;
			
			// connect to the participant for drillin
			dojo.connect( l, "onclick", function(){

				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.ParticipantId;
			    	params.documentName = tdElem.modelProp.participantName;
			    	
			    	if (tdElem.modelProp.snapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.snapshotId;
			    	}
			    	
			    	if (tdElem.modelProp.sranchId != null ) {
			    		params.branchId = tdElem.modelProp.branchId;
			    	}
			    		
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		} else {
			tdElem.innerHTML = "<ul style='margin-top:0px;margin-bottom:0px;'><li>"+value+"</li></ul>"; 
		}
		    

 


//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), ended");
	},

	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		    

		tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  


	},

	getpgData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	pgModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
