dojo.provide("com.ibm.bpm.wpd.document.ais.view.aisViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");

/**
 * BaseViewer
 */
dojo.declare("com.ibm.bpm.wpd.document.ais.view.aisViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.ais.view",
	"templates/aisViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	aisModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 
	
	},

	
	postCreate : function() {
		
		
		if ( this.debug ){
			//Get all the data from the server (Direct file read in this case).
			this.getaisData();
		}else {
			//Get the server side data
			aisModel = this.viewStateModel.getDocumentModel();;
		}
		
		var m = aisModel;
		this.bidiLeft = "left";
		this.bidiRight = "right";
		this.bidiCenter = "center";

		if (typeof(m.richDesc) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(m.richDesc));
			this.aisDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(m.desc);
			if (dir=="rtl")
				this.aisDescription.setAttribute("style","direction:rtl; text-align:left;");
		} else { 
			this.aisDescription.innerHTML = "&nbsp;";
		}

		this.aisName.innerHTML = m.name;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(m.name);
		if (dir=="rtl")
			this.aisName.setAttribute("style","direction:rtl; text-align:left;");
		this.aisNameHeader.innerHTML = this.messages.AIS_DETAILS;
		

		
		this.fillInSummaryTable();
		this.fillInVariablesTable();
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.aisCopyLinkHref);

	},
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.ais.view", "aisViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.ais.view", "aisViewerResources");


	},

	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},
	

	//*****
	//Fill in the summary table
	//
	//Returns: reference to the new row
	//*****
	fillInSummaryTable: function()
	{
		
		var newRow = null;
		var m = aisModel;
		
		newRow = this.insertRow(this.aisSummaryTableBody);
		
		var value = null;
		if (m.item.moduleName == null) {
			value = "";
		} else {
			value = m.item.moduleName;
		}
		var wholeString = "<b>" + this.messages.AIS_MODULE_NAME + "</b>" + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +value;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(value);
		if (dir=="rtl") {
			wholeString = "<b>" + this.messages.AIS_MODULE_NAME + "</b>" + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
						"<span dir='rtl'>" + value + "</span>";
		}
		this.addTextCellToRow(newRow, wholeString, this.bidiLeft, null, null);
		
		newRow = this.insertRow(this.aisSummaryTableBody);
		if (m.item.exportName == null) {
			value = "";
		} else {
			value = m.item.exportName;
		}
		wholeString = "<b>" + this.messages.AIS_EXPORT_NAME + "</b>" + "&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +value;
		dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(value);
		if (dir=="rtl") {
			wholeString = "<b>" + this.messages.AIS_EXPORT_NAME + "</b>" + "&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
						"<span dir='rtl'>" + value + "</span>";		
		}
		this.addTextCellToRow(newRow, wholeString, this.bidiLeft, null, null);
		
		newRow = this.insertRow(this.aisSummaryTableBody);
		if (m.item.operationName == null) {
			value = "";
		} else {
			value = m.item.operationName;
		}
		wholeString = "<b>" + this.messages.AIS_OPERATION_NAME + "</b>" + "&nbsp;&nbsp;" +value;
		dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(value);
		if (dir=="rtl") {
			wholeString = "<b>" + this.messages.AIS_OPERATION_NAME + "</b>" + "&nbsp;&nbsp;" +
						"<span dir='rtl'>" + value + "</span>";		
		}
		this.addTextCellToRow(newRow, wholeString, this.bidiLeft, null, null);
		
		newRow = this.insertRow(this.aisSummaryTableBody);
		
		
		// Logic from the client side - we replicate the logic here
		//
		//                    response = !getIsAsyncInvocation() || process.getAllOutputParameters().isEmpty() ?
		//                            UIMessages.getMessage("SCAConnectorTest_canBeUsedInService_yes", LocaleUtils.getCurrentUserLocale()):
		//                            UIMessages.getMessage("SCAConnectorTest_canBeUsedInService_no", LocaleUtils.getCurrentUserLocale());                                        

		
		if (m.item.isAsyncInvocation) {
			
			var numVars = m.variables.length;
			var areOutputs = false;
			for (var i=0; i<numVars; i++) {
				if (m.variables[i].type == "output"){
					areOutputs = true;
					break;
				}
			}
			
			if (!areOutputs) {
				value = this.messages.AIS_SYNCH_YES;
			} else {
				value = this.messages.AIS_SYNCH_NO;
			}

		} else {
			value = this.messages.AIS_SYNCH_YES;
		}
		this.addTextCellToRow(newRow, "<b>" + this.messages.AIS_ASYNCH_INVOKE+ "</b>" + "&nbsp;&nbsp;&nbsp;" +value, this.bidiLeft, null, null);	
		
	},
	
	//*****
	//Fill in the Variables table
	//
	//Returns: reference to the new row
	//*****
	fillInVariablesTable: function()
	{
		
		var newRow = null;
		var m = aisModel;
		

		if (m.variables != null) {
			var numVars = m.variables.length;
			for (var i=0; i<numVars; i++) {
				if (m.variables[i].type != "private") {
					
					newRow = this.insertRow(this.aisVariablesTableBody);
					
					// add name
					this.addTextCellToRow( newRow, m.variables[i].name, this.bidiLeft, null, null );
					var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(m.variables[i].name);
					if (dir=="rtl") {
						if (newRow.cells[newRow.cells.length-1])
							newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
					}
					
					//add type
					this.addTextCellToRow( newRow, m.variables[i].poName, this.bidiLeft, null, m.variables[i] );
					dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(m.variables[i].poName);
					if (dir=="rtl") {
						if (newRow.cells[newRow.cells.length-1])
							newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
					}
					
					// add check boxes (input)
					if (m.variables[i].type == "input") {
						this.addTextCellToRow( newRow, "<input type='checkbox' checked='checked' disabled='disabled' />", this.bidiCenter , null, null );
					} else {
						this.addTextCellToRow( newRow, "<input type='checkbox' id='checkin' disabled='disabled' />", this.bidiCenter, null, null );
					}
					
					// add check boxes (output)
					if (m.variables[i].type == "output") {
						this.addTextCellToRow( newRow, "<input type='checkbox' checked='checked' disabled='disabled' />", this.bidiCenter, null, null );
					} else {
						this.addTextCellToRow( newRow, "<input type='checkbox' id='checkin' disabled='disabled' />",this.bidiCenter, null, null );
					}
					
					// add check boxes (error)
					if (m.variables[i].type == "fault") {
						this.addTextCellToRow( newRow, "<input type='checkbox' checked='checked' disabled='disabled' />", this.bidiCenter, null, null );
					} else {
						this.addTextCellToRow( newRow, "<input type='checkbox' id='checkin' disabled='disabled' />",this.bidiCenter, null, null );
					}
					
					// add check boxes (List ?)
					if (m.variables[i].isList) {
						this.addTextCellToRow( newRow, "<input type='checkbox' checked='checked' disabled='disabled' />", this.bidiCenter, null, null );
					} else {
						this.addTextCellToRow( newRow, "<input type='checkbox' id='checkin' disabled='disabled' />",this.bidiCenter, null, null );
					}
				}

			}
		}

	},
	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function(tbodyElem)
	{
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);
		return trElem;
	},
	
	
	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname, modelProperty)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		
		tdElem.style.wordWrap="break-word";
		tdElem.align=align;

		tdElem.vAlign="top";

//		tdElem.style.whiteSpace="nowrap";
//
//		tdElem.align=align;

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		    
//		tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  

		if ((modelProperty != null) && (modelProperty.poId != null)) {
			tdElem.innerHTML = "<u><b>"+value+"</b></u>";  
			tdElem.style.cursor="pointer";
			dojo.addClass(tdElem, "linkFieldFont");	    

			tdElem.modelProp = modelProperty;
			dojo.connect( tdElem, "onclick", function(){

				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.poId;
			    	params.documentName = tdElem.modelProp.poName;
			    	
			    	if (tdElem.modelProp.snapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.snapshotId;
			    	}
			    	
			    	if (tdElem.modelProp.branchId != null ) {
			    		params.branchId = tdElem.modelProp.branchId;
			    	}
			    		
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		} else {
			tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  
		}


	},

	getboData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	boModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
