dojo.provide("com.ibm.bpm.wpd.document.bo.view.BOViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");

dojo.require("com.ibm.bpm.wpd.util.BidiUtils");

/**
 * BaseViewer
 */
dojo.declare("com.ibm.bpm.wpd.document.bo.view.BOViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.bo.view",
	"templates/BOViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	boModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 
	
	},

	
	postCreate : function() {
		
		
		if ( this.debug ){
			//Get all the data from the server (Direct file read in this case).
			this.getboData();
		}else {
			//Get the server side data
			boModel = this.viewStateModel.getDocumentModel();;
		}
		
		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (typeof(boModel.richDescription) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(boModel.richDescription));
			this.boDescription.innerHTML = str;
			var dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(boModel.description);
			if (dir=="rtl")
				this.boDescription.setAttribute("style","direction:rtl; text-align:left;");
		} else { 
//			this.boDescription.innerHTML = this.messages.BO_NO_DESC;
			this.boDescription.innerHTML = "&nbsp;";
		}

		this.boName.innerHTML = boModel.name;
		var dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(boModel.name);
		if (dir=="rtl")
			this.boName.setAttribute("style","direction:rtl; text-align:left;");
		
		var m = boModel;
		if (boModel.isComplex){
			this.fillInTable();
		} else {
			this.boTableHeader.style.display = "none";
			newRow = this.insertRow();
			this.addTextCellToRow( newRow, "<b>" + this.messages.BO_SIMPLE_TYPE_LABEL + "</b>" + boModel.type, this.bidiLeft, null, null );
//			this.addTextCellToRow( newRow, boModel.type, this.bidiLeft, null, null);
		}
		
	
		
		/*
		 *  Create Share link href Document
		 */
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.boCopyLinkHref);
		
	},	
	
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.bo.view", "BOViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.bo.view", "BOViewerResources");


	},

	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},
	

	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	fillInTable: function()
	{
		
		var newRow = null;
		var m = boModel;
		
		for (var i=0; i<boModel.properties.length; i++) {
			newRow = this.insertRow();
			var modelName = boModel.properties[i].name;
			if (boModel.properties[i].isArray) {
				this.addTextCellToRow( newRow, boModel.properties[i].name + this.messages.BO_LIST, this.bidiLeft, null, null );
				modelName = boModel.properties[i].name + this.messages.BO_LIST;
			} else {
				this.addTextCellToRow( newRow, boModel.properties[i].name, this.bidiLeft, null, null );
			}
			var dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(modelName);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}

			this.addTextCellToRow( newRow, boModel.properties[i].typeClass, this.bidiLeft, null, boModel.properties[i] );
			dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(boModel.properties[i].typeClass);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
			
			if (typeof(boModel.properties[i].richDescription) !== "undefined") {
				var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(boModel.properties[i].richDescription));
				dojo.addClass(newRow, "richTextFormat");
				this.addTextCellToRow( newRow, str, this.bidiLeft, null, null );
				dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(boModel.properties[i].description);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
			} else {
				this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );
			}

	
		}
	},
	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function()
	{
		var tbodyElem = this.boTableBody;
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);

		return trElem;
	},

	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//		modelProperty
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname, modelProperty)
	{
//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), Started : " + value);
		var tdElem = trElem.insertCell(trElem.cells.length);
//		tdElem.style.whiteSpace="nowrap";
//
//		tdElem.align=align;
		
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		    

		if ((modelProperty != null) && (modelProperty.typeClassRef != null)) {
			tdElem.innerHTML = "<u><b>"+value+"</b></u>";  
			tdElem.style.cursor="pointer";
			dojo.addClass(tdElem, "linkFieldFont");	    

			tdElem.modelProp = modelProperty;
			dojo.connect( tdElem, "onclick", function(){
//				alert("click!");
				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.typeClassRef;
			    	params.documentName = tdElem.modelProp.typeClass;
			    	
			    	if (tdElem.modelProp.typeClassSnapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.typeClassSnapshotId;
			    	}
			    	
			    	if (tdElem.modelProp.typeClassBranchId != null ) {
			    		params.branchId = tdElem.modelProp.typeClassBranchId;
			    	}
			    		
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		} else {
			tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  
		}

//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), ended");
	},

	getboData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	boModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
