
// dojo.provide allows pages to use all of the types declared in this resource.
dojo.provide("com.ibm.bpm.wpd.document.bpd.view.BPMNViewer");

//dojo.require the necessary dijit hierarchy
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require( "dijit.layout.ContentPane");
dojo.require("dojox.html.ellipsis");
dojo.require( "dijit.Tooltip");
dojo.require( "dojox.layout.ExpandoPane" );

dojo.require("com.ibm.bpm.wpd.document.bpd.view.BPDViewer");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");


dojo.declare("com.ibm.bpm.wpd.document.bpd.view.BPMNViewer", [ com.ibm.bpm.wpd.document.bpd.view.BPDViewer ], {
	
	// Path to the template
	templateString : dojo.cache("com.ibm.bpm.wpd.document.bpd.view","templates/BPMNViewer.html"),
	
	widgetsInTemplate : true,

	data: null, 	

	serverSideImageRendering: ( dojo.isIE < 10 || navigator.userAgent.toLowerCase().indexOf( 'ipad') >= 0 ), 

	constructor: function(){
	},


	postMixInProperties : function() {
		dojo.requireLocalization("com.ibm.bpm.wpd.document.bpd.view", "BPDViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.bpd.view", "BPDViewerResources");

	
	},

	postCreate: function(){

		if ( this.viewStateModel.openSideBySide && this.propertiesPane ){
			this.propertiesPane.startExpanded = false;
		}
		//I18N Work

		try{
			//Update description field
			this.spanDescription.innerHTML = this.messages.BPD_DESC_HEADER;


			//Variables area
			this.variableBoxTitle.innerHTML = this.messages.BPD_VAR_TITLE;
			this.varTitleName.innerHTML = this.messages.BPD_NAME;
			this.varTitleType.innerHTML = this.messages.BPD_TYPE;
			this.varTitleIn.innerHTML = this.messages.BPD_VAR_IN;
			this.varTitleOut.innerHTML = this.messages.BPD_VAR_OUT;
			this.varTitlePrivate.innerHTML = this.messages.BPD_LIST;

			//Work description
			this.workScheduleTitle.innerHTML = this.messages.BPD_WORK_TITLE;

			this.wsTimeSchedule.innerHTML = this.messages.TIME_SCHEDULE;
			this.wsTimeZone.innerHTML = this.messages.TIME_ZONE;
			this.wsHolidaySchedule.innerHTML = this.messages.HOLIDAY_SCHEDULE;


			//KPI table
			this.kpiTitle.innerHTML = this.messages.BPD_PROCESS_KPIS;
			this.loadingMessaage = this.messages.BPD_LOADING;
			
		}catch( err ){
			console.debug( err.message );
		}
		
		

		this.displayAll();

		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.bpmCopyLinkHref, this.viewStateModel.rootId);
		

	},


	/**
		Display all data about a given process, graphical and property.
	 **/
	displayAll: function(){

		//Load the data
		this.getData();

		//Pull out the process data
		var properties = this.data.properties;

		//Display the process properties pane first. That way if it's empty we can dispose of it before sizing
		//The main view area.
		if ( this.data.items.length > 0 ){
			//Display the properties data first
			var varPaneVisible = this.displayProcessProperties(properties);

			//Display the proecess property
			this.displayDescription(com.ibm.bpm.wpd.util.Helper.richTextFilter(this.data.richDesc));
		}

		//Tell the widget layout containers to size and display themselves.
		
	  				 
		this.mainBC.startup();

		if ( this.serverSideImageRendering ){
			this.displayImage();
			this.hideLoadingDiv();
			this.connectImageZoomControl();
			this.connectPrintButton();
				
		}else{
			this.displayEditor();
			this.hideLoadingDiv();
			this.connectZoomControl();
		}

		
		//Bug: WLE-6606 Moving the splitter in the main border container does not always force a refresh for svgWeb.
		//Capture the event and force a repaint.

		// if ( varPaneVisible ){
		// 	var bc = this.mainBC;
		// 	var pane = this.propertiesPane;
		// 	var splitter = bc.getSplitter('right');

		// 	if ( ibm_ilog.diagram.isSvgWeb ){
		// 		dojo.connect( splitter, "_stopDrag", function(){
		// 			svgweb._onWindowResize();
		// 		});
		// 	}
		// }


		


	},

	/**
		There may be a loading graphic on the middle of the screen. 
		Find and hide it, if it's there.
	 **/
	 hideLoadingDiv: function(){
	 	var div = this.loadingDiv;
	 	if ( div )
	 		div.style.visibility = 'hidden'; 
	 },

	/**
	 Display the process description at the top of the diagram.
	 **/
	displayDescription: function( description ){
		
		//debugger;

		if ( !description )
			return;

		var div = this.processDescription;

//		var nameText = document.createTextNode( description );
//		div.appendChild( nameText );
		var spanEle = document.createElement("span");
		div.appendChild(spanEle);
		spanEle.innerHTML = description;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(this.data.desc);
		if (dir=="rtl") {
			spanEle.setAttribute("dir","rtl");
			spanEle.setAttribute("style","direction:rtl; text-align:left;");
		}

		var container = this.topControlsContainer.domNode;

//		new dijit.Tooltip({ connectId: [div, container],  
////			label: "<span dir = '" + dir + "'>" + description + "</span>", // Bidi Support
//			label: description, 
//			position: ['below'] });



	},


	/**
	 * Display the process properties in the right hand pane.
	    @return: true if the pane was displayed, false if no data was available and the component was removed.
	 */
	displayProcessProperties: function( properties ){
		
		//Sanity check...
		if ( !properties ){
			this.mainBC.removeChild( this.propertiesPane );
			return false;
		}
		
		//Display each table of data in its appropriate place

		var successVaribles = this.displayVariables(this._cleanUneededVariables(properties.variables));
		var successWork = this.displayWorkSchedule( properties.workSchedule );
		var successKPIs = this.displayKPIs( properties.KPIs );
		
		//Remove the properties pane if there's nothing to show
		if ( (!successKPIs && !successWork && !successVaribles)){
			this.mainBC.removeChild( this.propertiesPane );
			return false;
		}else{
			
			//debugger;
			 
			//Tweak the expando pane so it has the title bar on the bottom (looks better)
			var expandoPane = this.propertiesPane;
			var expandoTitle = expandoPane.titleWrapper;
			var expandoContents = expandoPane.cwrapper;
			expandoPane.domNode.insertBefore( expandoContents, expandoTitle );
			
			//Set the properties width wide enough that the scroll bar goes away, iff the screen is wide enough.
			if (  this.mainBC.domNode.offsetWidth > 1024 ){
				this.propertiesPane.domNode.style.width = "425px";
			}
			
		}

		return true;
	},
	
	_cleanUneededVariables: function(variables) {
		// return a list minus the unwanted variables
		var cleanedVars = [];
		for (var i =0;i<variables.length;i++) {
			
			/* Check variable type, if an error end event is added it adds a variable to type fault (why???). This gives us an unknown
			 * variable with no labels showing up
			 */
			if ("fault" == variables[i].type)
				continue;
			
			cleanedVars[cleanedVars.length] = variables[i];
		}
		
		return cleanedVars;
	},
	
	/**
	 * Display the variables in the property pane.
	 * 
	 * @param variables: An array of JSON data
	 * @return false if no data was displayed, true if anything was put up.
	 */
	displayVariables: function( variables ){
		
		//Hide the variables area if there's no content
		if ( !variables || variables.length == 0 ){
			this.variableBox.removeChild( this.variableTable );
			this.variableBox.removeChild( this.variableBoxTitle );

			return false;
		}
			
		
		for ( var i = 0; i < variables.length; i++ ){
			var variable = variables[i];
			var row = this.displayVariable( variable, (i%2 == 0) );

			//Alternating highlighting for even and odd rows
			if ( i % 2 == 0 ){
				//Even row
				row.setAttribute("style","background-color:#F0F0F0");
			}
		}

		return true;
		
	},
	
	
	//I see the main logic is in super class: bpdviewer, but here, no call to supers destroy
//	destroy: function(){
//		this.mainBC.destroyRecursive();
//	},
//	
	/**
	 * Add a table row to display information about a single variable
	 */
	displayVariable: function( variable){
		
		var table = this.variableTableBody;
		var row = document.createElement( 'tr' );
		
		
		var nameCell = document.createElement( 'td' );
		var nameText = document.createTextNode( variable.name );
		nameCell.appendChild( nameText );
		row.appendChild( nameCell );
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(variable.name);
		if (dir=="rtl")
			nameCell.setAttribute("style","direction:rtl; text-align:left");
		
//		debugger;


		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(variable.poName);
		if ( !variable.poId ){
			var typeCell = document.createElement( 'td' );
			var typeText = document.createTextNode( variable.poName );
			typeCell.appendChild( typeText );
			row.appendChild( typeCell );
			if (dir=="rtl")
				typeCell.setAttribute("style","direction:rtl; text-align:left");

		}else{

			var typeCell = document.createElement( 'td' );
			var anchor = document.createElement('a'); 
			var typeText = document.createTextNode( variable.poName );
			anchor.appendChild( typeText );
			anchor.setAttribute('href', '#');
			typeCell.appendChild( anchor );
			row.appendChild( typeCell );
			if (dir=="rtl")
				typeCell.setAttribute("style","direction:rtl;");

			//Variable drill-down code
			dojo.connect( anchor, "onclick", this, function(evt){
				debugger;

				//Type name
				var variableType = evt.target.innerHTML;

				//Find Walk the variable array to find information associated with this type.
				var vars = this.data.properties.variables;
				var variable_info = undefined;
				for ( var i = 0; i < vars.length; i++ ){
					variable_info = vars[i];
					if ( variable_info.poName && variable_info.poName == variableType )
						break;
				}

				var branchId = variable_info.branchId ? variable_info.branchId : null;
				var snapshotId = variable_info.snapshotId ? variable_info.snapshotId : null;
				var poId = variable_info.poId ? variable_info.poId : undefined;
				
				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
					
					//Process ID
			    	params.documentId = poId;
			    	
			    	//Type name
			    	params.documentName = variableType;
			    	
			    	if ( snapshotId ){
			    		params.snapshotId = snapshotId;
			    	}

					// If branch is set, but snapshot isn't then pass branch along
			    	if ( branchId && !snapshotId){
			    		params.branchId=branchId;
			    	}
			    		
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();
			});


		}

		
		var inCell = document.createElement( 'td' );

			if ( variable.type == 'input'){
				var img = document.createElement( 'img');
				img.setAttribute('src', com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_WEBVIEWER) + '/widgets/com/ibm/bpm/wpd/images/BPD/graycheck.png');
				img.setAttribute( 'alt', 'true');			
				inCell.appendChild( img );
				inCell.setAttribute("style","text-align:center");
			}

		row.appendChild( inCell );
		var outCell = document.createElement( 'td' );


		if ( variable.type == 'output'){
			var img = document.createElement( 'img');
			img.setAttribute('src', com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_WEBVIEWER) + '/widgets/com/ibm/bpm/wpd/images/BPD/graycheck.png');
			img.setAttribute( 'alt', 'true');	
			outCell.appendChild( img );
			outCell.setAttribute("style","text-align:center");
		}
	
		row.appendChild( outCell );
		
		var privCell = document.createElement( 'td' );

		// change third column to be a list indicator - JS
		if ( variable.isList){
			var img = document.createElement( 'img');
			img.setAttribute('src', com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_WEBVIEWER) + '/widgets/com/ibm/bpm/wpd/images/BPD/graycheck.png');
			img.setAttribute( 'alt', 'true');			
			privCell.appendChild( img );
			privCell.setAttribute("style","text-align:center");
		}
		
		 
		//td.setAttribute( 'style' : 'text-align:center');
		row.appendChild( privCell );
		
		table.appendChild( row );	
		 return row;
		
		
	},

	/* Display work schedule data.
	 *
	 * @return: false if no data was displayed, true if anything was put up.
	 */

	displayWorkSchedule: function(workSchedule){
	
		if ( !workSchedule  || (dojo.isArray(workSchedule )  && workSchedule.length == 0)){
			this.variableBox.removeChild( this.workScheduleTitle );
			this.variableBox.removeChild( this.workTable );
			return false;
			
		}
			


		if ( dojo.isArray(workSchedule ) ){
			for ( var i = 0 ;i < workSchedule.length; i++ ){
				var item = workSchedule[i];
				this.displayWorkScheduleItem(item);
			}
			
		}
		else{
			this.displayWorkScheduleItem( workSchedule );
		}

		return true;

	},

	displayWorkScheduleItem: function(item){
	
			var scheduleTable = this.workTableBody;

			var scheduleRow = document.createElement( 'tr' );
			scheduleTable.appendChild( scheduleRow );

			var scheduleCell = document.createElement( 'td' );
			
			var str_timeScheduleName = item.timeScheduleName;
			if (item.timeScheduleType==1) {
				str_timeScheduleName = item.timeScheduleExpression;
			} else {
				if ('(use default)'==item.timeScheduleName) {
					try{
						str_timeScheduleName = this.messages.USE_DEFAULT;		
					}catch( err ){			
					}					
				}
			}
			
			var scheduleText = document.createTextNode(str_timeScheduleName);
			scheduleCell.appendChild( scheduleText );
			scheduleRow.appendChild( scheduleCell );

			var timeZoneCell = document.createElement('td');
			
			var str_timezoneName = item.timezone;
			if (item.timezoneType==1) {
				str_timezoneName = item.timezoneExpression;
			} else {
				if ('(use default)'==item.timezone) {
					try{
						str_timezoneName = this.messages.USE_DEFAULT;		
					}catch( err ){			
					}					
				}
			}
			
						
			var timeZoneText = document.createTextNode( str_timezoneName );
			timeZoneCell.appendChild( timeZoneText );
			scheduleRow.appendChild( timeZoneCell );

			var holidayCell = document.createElement( 'td' );
			
			var str_holidayScheduleName = item.holidayScheduleName;
			if (item.holidayScheduleType==1) {
				str_holidayScheduleName = item.holidayScheduleExpression;
			} else {
				if ('(use default)'==item.holidayScheduleName) {
					try{
						str_holidayScheduleName = this.messages.USE_DEFAULT;		
					}catch( err ){			
					}					
				}
			}
						
			var holidayText = document.createTextNode( str_holidayScheduleName );
			holidayCell.appendChild( holidayText );
			scheduleRow.appendChild( holidayCell );
		
	},

	displayKPIs: function( KPIs ){
		if ( !KPIs ||KPIs.length == 0 ){
			this.variableBox.removeChild( this.kpiTitle );
			this.variableBox.removeChild( this.kpiTable );
			return false;
		}
			

		for ( var i = 0; i < KPIs.length; i++ ){
			var kpi = KPIs[i]; 
			this.displayKPI( kpi );		
		}
		

		return true;
		
	},

	displayKPI: function( KPI ){

		
		var kpiTable = this.kpiTableBody;

		var kpiRow = document.createElement( 'tr' );
		kpiTable.appendChild( kpiRow );

		var kpiNameCell = document.createElement( 'td' );
		var kpiNameText = document.createTextNode( KPI.name );
		kpiNameCell.appendChild( kpiNameText );
		kpiRow.appendChild( kpiNameCell );
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(KPI.name);
		if (dir=="rtl"){
			kpiNameCell.innerHTML = com.ibm.bpm.wpd.util.BidiUtils.getRTLTextWithBracket(KPI.name);
			kpiNameCell.setAttribute("style","direction:rtl; text-align:left");
		}

		/*
		var kpiTypeCell = document.createElement( 'td' );
		var kpiTypeText = document.createTextNode( KPI.SettingsType );
		kpiTypeCell.appendChild( kpiTypeText );
		kpiRow.appendChild( kpiTypeCell );
		*/
	}




});