dojo.provide("com.ibm.bpm.wpd.document.ea.view.eaViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");

/**
 * eaViewer
 */
dojo.declare("com.ibm.bpm.wpd.document.ea.view.eaViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.ea.view",
	"templates/eaViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	eaModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 

		

	
	},

	
	postCreate : function() {
		
		
		if ( this.debug ){
			//Get all the data from the server (Direct file read in this case).
			this.geteaData();
		}else {
			//Get the server side data
			eaModel = this.viewStateModel.getDocumentModel();;
		}
		
		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (typeof(eaModel.richDescription) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(this.replaceNewLines(eaModel.richDescription)));
			this.eaDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(eaModel.desc);
			if (dir=="rtl")
				this.eaDescription.setAttribute("style","direction:rtl; text-align:left;");

		} else { 
			this.eaDescription.innerHTML = "&nbsp;";
		}

		this.eaName.innerHTML = eaModel.name;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(eaModel.name);
		if (dir=="rtl")
			this.eaName.setAttribute("style","direction:rtl; text-align:left;");
		
		var m = eaModel;
	
		this.fillInInputsTable();
		this.fillInOutputsTable();
		this.fillInCustomProperties();

		/*
		 *  Create Share link href Document
		 */
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.eaCopyLinkHref);

	},
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.ea.view", "eaViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.ea.view", "eaViewerResources");


	},
	
	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},

	//*****
	//Insert inputs into the input table
	//
	//Returns: reference to the new row
	//*****
	fillInCustomProperties: function()
	{
		
		var newRow = null;
		var m = eaModel;
		
		var cps = eaModel.customProperties;
		for (var cp in cps)
		{
			newRow = this.insertRow(this.eaCustomPTableBody);
			this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(cp), this.bidiLeft, null, null );
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(cp);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
			
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlText(this.replaceNewLines(this.replaceNewLines(cps[cp])));
			this.addTextCellToRow( newRow, str, this.bidiLeft, null, null);
			dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(cps[cp]);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
		}
		
		newRow = this.insertRow(this.eaCustomPTableBody);
		this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );

	},
	

	//*****
	//Insert inputs into the input table
	//
	//Returns: reference to the new row
	//*****
	fillInInputsTable: function()
	{
		
		var newRow = null;
		var m = eaModel;
		
		var inputs = eaModel.inputs;
		for (var inputName in inputs)
		{
			
			newRow = this.insertRow(this.eaInputTableBody);
			var inputStr = inputName;
			if (inputs[inputName].isList) {
				this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(inputName) + this.messages.EA_LIST, this.bidiLeft, null, null );
				inputStr = inputName + this.messages.EA_LIST;
			} else {
				this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(inputName), this.bidiLeft, null, null );
			}
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(inputStr);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}

			this.addTextCellToRow( newRow, inputs[inputName].type, this.bidiLeft, null, inputs[inputName]);
			dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(inputs[inputName].type);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
			if ((typeof(inputs[inputName].richDesc) !== "undefined") && (inputs[inputName].richDesc != null)) {
				var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(inputs[inputName].richDesc));
				dojo.addClass(newRow, "richTextFormat");
				this.addTextCellToRow( newRow, str, this.bidiLeft, null, null );
				dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(inputs[inputName].desc);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
			} else {
				this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );
			}

		}
		newRow = this.insertRow(this.eaInputTableBody);
		this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );

	},
	
	//*****
	//Insert outputs into the output table
	//
	//Returns: reference to the new row
	//*****
	fillInOutputsTable: function()
	{
		
		var newRow = null;
		var m = eaModel;
		
		var outputs = eaModel.outputs;
		for (var outputName in outputs)
		{
			
			newRow = this.insertRow(this.eaOutputTableBody);
			var outputStr = outputName;
			if (outputs[outputName].isList) {
				this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(outputName) + this.messages.EA_LIST, this.bidiLeft, null, null );
				outputStr = outputName + this.messages.EA_LIST;
			} else {
				this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(outputName), this.bidiLeft, null, null );
			}
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(outputStr);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}

			this.addTextCellToRow( newRow, outputs[outputName].type, this.bidiLeft, null, outputs[outputName]);
			dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(outputs[outputName].type);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
			if ((typeof(outputs[outputName].richDesc) !== "undefined") && (outputs[outputName].richDesc != null)) {
				var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(outputs[outputName].richDesc));
				dojo.addClass(newRow, "richTextFormat");
				this.addTextCellToRow( newRow, str, this.bidiLeft, null, null );
				dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(outputs[outputName].desc);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
			} else {
				this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );
			}

		}	
		newRow = this.insertRow(this.eaOutputTableBody);
		this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );

	},
	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function(tbodyElem)
	{
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);

		return trElem;
	},

	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname, modelProperty)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		    
		if ((modelProperty != null) && (modelProperty.classId != null)) {
			tdElem.innerHTML = "<u><b>"+value+"</b></u>";  
			tdElem.style.cursor="pointer";
			dojo.addClass(tdElem, "linkFieldFont");	    

			tdElem.modelProp = modelProperty;
			dojo.connect( tdElem, "onclick", function(){

				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.classId;
			    	params.documentName = tdElem.modelProp.type;
			    	
			    	if (tdElem.modelProp.snapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.snapshotId;
			    	}
			    	
			    	if (tdElem.modelProp.branchId != null ) {
			    		params.branchId = tdElem.modelProp.branchId;
			    	}
			    	
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		} else {
			tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  
		}


	},

	geteaData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	boModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
