dojo.provide("com.ibm.bpm.wpd.document.epv.view.EPVViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");


dojo.declare("com.ibm.bpm.wpd.document.epv.view.EPVViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.epv.view","templates/EPVViewer.html"),

	epvModel: {},

	constructor : function() { 	
	},
	
	postCreate : function() {

		epvModel = this.viewStateModel.getDocumentModel();

		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (epvModel.richDesc) {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(epvModel.richDesc));
			this.epvDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvModel.desc);
			if (dir=="rtl")
				this.epvDescription.setAttribute("style","direction:rtl; text-align:left;");

		} else { 
			this.epvDescription.innerHTML = "&nbsp;";
		}
		
		if (epvModel.participantId && (epvModel.participantRefSnapshotId || epvModel.participantClassBranchId) && !!epvModel.participantName) {
			this.epvExposedTo.innerHTML = "<u>"+epvModel.participantName+"</u>";
			this.epvExposedTo.style.cursor="pointer";
			dojo.addClass(this.epvExposedTo, "linkFieldFont");	    
	
			dojo.connect( this.epvExposedTo, "onclick", function(){
				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = epvModel.participantId;
			    	params.documentName = epvModel.participantName;
			    	
			    	if (epvModel.participantRefSnapshotId) {
			    		params.snapshotId = epvModel.participantRefSnapshotId;
			    	}
			    	
			    	if (epvModel.participantClassBranchId) {
			    		params.branchId = epvModel.participantClassBranchId;
			    	}
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		}
		else {
			if (!!epvModel.participantName) {
				this.epvExposedTo.innerHTML = epvModel.participantName;
			}
		}
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvModel.participantName);
		if (dir=="rtl")
			this.epvExposedTo.setAttribute("style","direction:rtl; text-align:left;");

		this.epvName.innerHTML = epvModel.name;
		dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvModel.name);
		if (dir=="rtl")
			this.epvName.setAttribute("style","direction:rtl; text-align:left;");
		
		this.fillInTable();
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.evpCopyLinkHref);
	},
	
	postMixInProperties: function()
	{
        dojo.requireLocalization("com.ibm.bpm.wpd.document.epv.view", "EPVViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.epv.view", "EPVViewerResources");
	},
	
	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},

	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	fillInTable: function()
	{
		
		var newRow = null;
		
		for (var i=0; i<epvModel.vars.length; i++) {
			newRow = this.insertRow();
			var epvVar = epvModel.vars[i];
			this.addTextCellToRow( newRow, epvVar.name, "15%", this.bidiLeft, null, null );
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvVar.name);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
			this.addTextCellToRow( newRow, epvVar.varname,"15%", this.bidiLeft, null, null );
			dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvVar.varname);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
			if (epvVar.extDesc) {
				var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(epvVar.extDesc));
				this.addTextCellToRow( newRow, str, "35%",this.bidiLeft, null, null );
				dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvVar.plainExtDesc);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
			} else {
				this.addTextCellToRow( newRow, "", "35%", this.bidiLeft, null, null );
			}
			if (epvVar.defaultValue) {
				this.addTextCellToRow( newRow, epvVar.defaultValue, "15%", this.bidiLeft, null, null );
				dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvVar.defaultValue);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
			} else {
				this.addTextCellToRow( newRow, "", "15%", this.bidiLeft, null, null );
			}
			
			if (epvVar.isChgActiveProc)
				this.addTextCellToRow( newRow, this.messages.EPV_TRUE, "10%", this.bidiLeft, null, null );
			else
				this.addTextCellToRow( newRow, this.messages.EPV_FALSE, "10%", this.bidiLeft, null, null );

			if (epvVar.typeClassRef)
				this.addTextCellToRow( newRow, epvVar.typeClass, "10%", this.bidiLeft, null, epvVar );
			else
				this.addTextCellToRow( newRow, epvVar.typeClass, "10%", this.bidiLeft, null, null);
			dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(epvVar.typeClass);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}

		}
	},
	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function()
	{
		var tbodyElem = this.epvTableBody;
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);

		return trElem;
	},

	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, columnWidth, align, classname, modelProperty)
	{
		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";

		tdElem.width = columnWidth;
		tdElem.style.whiteSpace="normal";
		tdElem.style.verticalAlign="top";
		if (modelProperty) {
			tdElem.innerHTML = "<u>"+value+"</u>";  
			tdElem.style.cursor="pointer";
			dojo.addClass(tdElem, "linkFieldFont");	    

			tdElem.modelProp = modelProperty;
			dojo.connect( tdElem, "onclick", function(){
				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.typeClassRef;
			    	params.documentName = tdElem.modelProp.typeClass;
			    	if (tdElem.modelProp.typeClassSnapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.typeClassSnapshotId;
			    	}

			    	if (tdElem.modelProp.typeClassBranchId != null ) {
			    		params.branchId = tdElem.modelProp.typeClassBranchId;
			    	}
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		} else {
			tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  
		}
	}
}
		
);	
	
	
