dojo.provide("com.ibm.bpm.wpd.document.sla.view.slaViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");

/**
 * Participant Group viewer
 * 
 * 		Participant group has a couple of different types of data
 * 			- lists of users and groups
 *          - expression
 *      This implementation has a couple of different types of table to reflect the different types of data to show. We
 *      hide the one that is not needed and show the the one that is.
 */
dojo.declare("com.ibm.bpm.wpd.document.sla.view.slaViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.sla.view",
	"templates/slaViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	slaModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 
	
	},

	
	postCreate : function() {
		
		

		//Get the server side data
		slaModel = this.viewStateModel.getDocumentModel();

		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (typeof(slaModel.richDesc) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(slaModel.richDesc));
			this.slaDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(slaModel.desc);
			if (dir=="rtl")
				this.slaDescription.setAttribute("style","direction:rtl; text-align:left;");
		} else { 
			this.slaDescription.innerHTML = "&nbsp;";
		}

		this.slaName.innerHTML = slaModel.name;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(slaModel.name);
		if (dir=="rtl")
			this.slaName.setAttribute("style","direction:rtl; text-align:left;");
		
		this.fillInDetailsData();
		this.fillInExposingData();
		this.fillInTriggerData();
		this.fillInConditionData();
		this.fillInConsequenceData();

		/*
		 *  Create Copy link href Document
		 */
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.slaCopyLinkHref);
		
	},
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.sla.view", "slaViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.sla.view", "slaViewerResources");
		
		dojo.requireLocalization("com.ibm.bpm.wpd.document.sla.view", "slaEclViewerResources");
		this.eclMessages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.sla.view", "slaEclViewerResources");


	},
	
	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},

	//*****
	//Insert a new row into the table
	//   	In this case these are the sla details
	//
	//Returns: reference to the new row
	//*****
	fillInDetailsData: function()
	{
		
		var newRow = null;
		
		newRow = this.insertRow(this.slaDetailsTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_TARGET_TYPE + this.eclMessages.SLA_ACTIVITY_TYPE, this.bidiLeft, null);

	},
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla condition details
	//
	//Returns: reference to the new row
	//*****
	fillInConditionData: function()
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		// if this is a BOOLEAN type KPI
		
		if (tmodel.conditions[0].kpiUnit == this.messages.SLA_CONDITION_KPIUNIT_KEY_BOOLEAN) {
			// few fields to display
			this.fillInConditionKPI(tmodel);
			this.fillInConditionActivities(tmodel);
			this.fillInConditionBoolean(tmodel);
			
		} else {
			// larger number of fields
			this.fillInConditionScope(tmodel);
			this.fillInConditionKPI(tmodel);
			this.fillInConditionActivities(tmodel);
			this.fillInStartTime(tmodel);
			this.fillInConditionRelation(tmodel);
			this.fillInConditionCompareWith(tmodel);
			if (tmodel.conditions[0].compareTarget.type != this.messages.SLA_CONDITION_COMPARE_WITH_KEY_VALUE) {
				this.fillInConditionKindOfThreshold(tmodel);
			}
		}
		
		

	
	},
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla start time in the condition
	//
	//Returns: reference to the new row
	//*****
	fillInConditionBoolean: function(tmodel)
	{
		
		var newRow = null;
		var tmodel = slaModel;
		

			newRow = this.insertRow(this.slaConditionTableBody);
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_TRUE_OR_FALSE, this.bidiLeft, null);
			if (tmodel.conditions[0].trueOrfalse) {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_TRUE, this.bidiLeft, null);
			} else {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_FALSE, this.bidiLeft, null);
			}

	},
	
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla start time in the condition
	//
	//Returns: reference to the new row
	//*****
	fillInStartTime: function(tmodel)
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		if (tmodel.conditions[0].startTime != null) {
			newRow = this.insertRow(this.slaConditionTableBody);
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_START_TIME_LABEL, this.bidiLeft, null);
			
			if (tmodel.conditions[0].startTime.rollingTime) {
				// this is N hours, days, ...
	
				var str = null;
				var parms = [];
				switch (tmodel.conditions[0].startTime.unitType) {
					case this.messages.SLA_TIME_PERIOD_HOURS:
						if (tmodel.conditions[0].startTime.timeCount == 1) {
							this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_START_TIME_LAST_HOUR, this.bidiLeft, null);
						} else {
							parms[0] = tmodel.conditions[0].startTime.timeCount;
							str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_START_TIME_LAST_HOURS, parms);
							this.addTextCellToRow( newRow, str, this.bidiLeft, null);
						}
						break;
					case this.messages.SLA_TIME_PERIOD_DAYS:
						if (tmodel.conditions[0].startTime.timeCount == 1) {
							this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_START_TIME_LAST_DAY, this.bidiLeft, null);
						} else {
							parms[0] = tmodel.conditions[0].startTime.timeCount;
							str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_START_TIME_LAST_DAYS, parms);
							this.addTextCellToRow( newRow, str, this.bidiLeft, null);
						}
						break;	
					case this.messages.SLA_TIME_PERIOD_WEEKS:
						if (tmodel.conditions[0].startTime.timeCount == 1) {
							this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_START_TIME_LAST_WEEK, this.bidiLeft, null);
						} else {
							parms[0] = tmodel.conditions[0].startTime.timeCount;
							str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_START_TIME_LAST_WEEKS, parms);
							this.addTextCellToRow( newRow, str, this.bidiLeft, null);
						}						

						break;	
					case this.messages.SLA_TIME_PERIOD_MONTHS:			
						if (tmodel.conditions[0].startTime.timeCount == 1) {
							this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_START_TIME_LAST_MONTH, this.bidiLeft, null);
						} else {
							parms[0] = tmodel.conditions[0].startTime.timeCount;
							str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_START_TIME_LAST_MONTHS, parms);
							this.addTextCellToRow( newRow, str, this.bidiLeft, null);
						}												
						break;
				}
				
			} else {
				// this is a single hour, day, ...	
							
				switch (tmodel.conditions[0].startTime.unitType) {
					case this.messages.SLA_TIME_PERIOD_HOURS:
						this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_HOUR, this.bidiLeft, null);
						break;
					case this.messages.SLA_TIME_PERIOD_DAYS:
						this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_TODAY, this.bidiLeft, null);
						break;	
					case this.messages.SLA_TIME_PERIOD_WEEKS:
						this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_WEEK, this.bidiLeft, null);
						break;	
					case this.messages.SLA_TIME_PERIOD_MONTHS:
						this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_MONTH, this.bidiLeft, null);
						break;
					}
				
			}
//		} else {
//			newRow = this.insertRow(this.slaConditionTableBody);
//			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_START_TIME_LABEL, this.bidiLeft, null);
//			this.addTextCellToRow( newRow, this.eclMessages.SLA_NONE, this.bidiLeft, null);
		}
		
	},
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla condition compare with
	//
	//Returns: reference to the new row
	//*****
	fillInConditionCompareWith: function(tmodel)
	{
		
		var newRow = null;
		var tmodel = slaModel;
		var str = null;
		var parms = [];

		
		newRow = this.insertRow(this.slaConditionTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_COMPARE_WITH_LABEL, this.bidiLeft, null);
		
		switch (tmodel.conditions[0].compareTarget.type) {
		case this.messages.SLA_CONDITION_COMPARE_WITH_KEY_THRESH:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_COMPARE_WITH_THRESH, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_COMPARE_WITH_KEY_PERCENT_ABOVE:
			parms[0] = tmodel.conditions[0].compareTarget.percent;
			str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_WITH_PERCENT_ABOVE, parms);
			this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_COMPARE_WITH_KEY_PERCENT_BELOW:
			parms[0] = tmodel.conditions[0].compareTarget.percent;
			str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_WITH_PERCENT_BELOW, parms);
			this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_COMPARE_WITH_KEY_VALUE_ABOVE:
			if (tmodel.conditions[0].kpiUnit == this.messages.SLA_CONDITION_KPIUNIT_KEY_TIME) {
				parms[0] = this.createTimeStringFromSecs(tmodel.conditions[0].compareTarget.value);
				str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_WITH_VALUE_MORE_THAN, parms);
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			} else {
				parms[0] = tmodel.conditions[0].compareTarget.value;
				str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_WITH_VALUE_ABOVE, parms);
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			}
			break;
		case this.messages.SLA_CONDITION_COMPARE_WITH_KEY_VALUE_BELOW:
			if (tmodel.conditions[0].kpiUnit == this.messages.SLA_CONDITION_KPIUNIT_KEY_TIME) {
				parms[0] = this.createTimeStringFromSecs(tmodel.conditions[0].compareTarget.value);
				str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_WITH_VALUE_LESS_THAN, parms);
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			} else {
				parms[0] =  tmodel.conditions[0].compareTarget.value;
				str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_WITH_VALUE_BELOW, parms);
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			}
			break;
		case this.messages.SLA_CONDITION_COMPARE_WITH_KEY_VALUE:
			
			if (tmodel.conditions[0].kpiUnit == this.messages.SLA_CONDITION_KPIUNIT_KEY_TIME) {
				this.addTextCellToRow( newRow, this.createTimeStringFromSecs(tmodel.conditions[0].compareTarget.value), this.bidiLeft, null);
			} else {
				this.addTextCellToRow( newRow, tmodel.conditions[0].compareTarget.value, this.bidiLeft, null);

			}
		
			break;			
		default:
			throw new Error("Not a valid threshold");
		
		}
		
	},
	
	//*****
	// create time string for time values in time target
	//   	
	//
	//Returns: the time value string
	//*****
	createTimeStringFromSecs: function(numSecs)
	{
		
		var secsInDay = 60*60*24;
		var secsInHour = 60*60;
		var secsInMin = 60;
		var str = null;
		var parms = [];


		
		// find the number of days
		

		var numDays = Math.floor(numSecs/secsInDay);
		
		// find the number of hours
		
		var leftOverFromDays = numSecs - (numDays*secsInDay);
		var numHours = Math.floor(leftOverFromDays/secsInHour);
		
		// find the number of minutes
		var leftOverFromHours = leftOverFromDays - (numHours*secsInHour);
		var numMins = leftOverFromHours/secsInMin;
		var timeStr = "";
		
		if (numDays != 0) {
			//there are days
			if (numHours != 0) {
				if (numMins != 0) {
					// there are days, hours and minutes
					parms[0] =  numDays;
					parms[1] =  numHours;
					parms[2] =  numMins;
					if (numDays == 1) {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAY_HOURS_MINUTES, parms);
					} else {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAYS_HOURS_MINUTES, parms);
					}
				} else {
					//there are days and hours
					parms[0] =  numDays;
					parms[1] =  numHours;
					if (numDays == 1) {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAY_HOURS, parms);
					} else {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAYS_HOURS, parms);
					}		
				}	
			} else {
				if (numMins != 0) {
					// there are days, and minutes
					parms[0] =  numDays;
					parms[1] =  numMins;
					if (numDays == 1) {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAY_MINUTES, parms);
					} else {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAYS_MINUTES, parms);
					}
				} else {
					//there are days 
					parms[0] =  numDays;
					if (numDays == 1) {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAY, parms);
					} else {
						str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_DAYS, parms);
					}
				}
			}
		}else {
			//there are no days
			if (numHours != 0) {
				if (numMins != 0) {
					// there are hours and minutes
					parms[0] =  numHours;
					parms[1] =  numMins;
					str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_HOURS_MINUTES, parms);
				} else {
					//there are hours
					parms[0] =  numHours;
					str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_HOURS, parms);
				}
			} else {
				parms[0] =  numMins;
				str = dojo.string.substitute(this.eclMessages.SLA_CONDITION_COMPARE_MINUTES, parms);
			}
		}
		
		return str;
	},
	
	
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla condition threshold kind
	//
	//Returns: reference to the new row
	//*****
	fillInConditionKindOfThreshold: function(tmodel)
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		newRow = this.insertRow(this.slaConditionTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_KIND_OF_THRESHOLD_LABEL, this.bidiLeft, null);
		

		switch (tmodel.conditions[0].compareTarget.thresholdType) {
		case this.messages.SLA_CONDITION_THRESHOLD_TYPE_KEY_MIN:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_THRESHOLD_TYPE_MIN, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_THRESHOLD_TYPE_KEY_MAX:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_THRESHOLD_TYPE_MAX, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_THRESHOLD_TYPE_KEY_EXP:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_THRESHOLD_TYPE_EXP, this.bidiLeft, null);
			break;
		default:
			throw new Error("Not a valid threshold");
		
		}
		
	},
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla condition relation type
	//
	//Returns: reference to the new row
	//*****
	fillInConditionRelation: function(tmodel)
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		newRow = this.insertRow(this.slaConditionTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_RELATION_LABEL, this.bidiLeft, null);
		
		
		switch (tmodel.conditions[0].relation) {
		case this.messages.SLA_CONDITION_RELATION_KEY_EQUALTO:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_RELATION_EQUALTO, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_RELATION_KEY_LESSTHAN:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_RELATION_LESSTHAN, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_RELATION_KEY_GREATERTHAN:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_RELATION_GREATERTHAN, this.bidiLeft, null);
			break;
		default:
			throw new Error("Not a valid condition");
		
		}
		
	},
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla condition KPIs
	//
	//Returns: reference to the new row
	//*****
	fillInConditionActivities: function(tmodel)
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		newRow = this.insertRow(this.slaConditionTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_ACTIVITIES_LABEL, this.bidiLeft, null);
		
		// add the activities
		var numActs = tmodel.conditions[0].activities.length;
		for (var i=0; i<numActs; i++) {
			this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(tmodel.conditions[0].activities[i].activityName), this.bidiLeft, null);
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tmodel.conditions[0].activities[i].activityName);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}

			newRow = this.insertRow(this.slaConditionTableBody);
			this.addTextCellToRow( newRow, "", this.bidiLeft, null);
		}
		
		
	},
	
	//*****
	//Insert a new row into the table
	//   	In this case these are the sla condition KPIs
	//
	//Returns: reference to the new row
	//*****
	fillInConditionKPI: function(tmodel)
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		newRow = this.insertRow(this.slaConditionTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_KPI_LABEL, this.bidiLeft, null);
		this.addTextCellToRow( newRow, tmodel.conditions[0].kpi.kpiName, this.bidiLeft, null);
		var kpiName = tmodel.conditions[0].kpi.kpiName;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(kpiName);
		if (dir=="rtl") {
			if (newRow.cells[newRow.cells.length-1]) {
				newRow.cells[newRow.cells.length-1].innerHTML = com.ibm.bpm.wpd.util.BidiUtils.getRTLTextWithBracket(newRow.cells[newRow.cells.length-1].innerHTML );
				newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
		}
		
	},
	
	
	//*****
	//Insert a new row into the table
	//   	In this case this is the condition scope data
	//
	//Returns: reference to the new row
	//*****
	fillInConditionScope: function(tmodel)
	{
		
		var newRow = null;

		
		newRow = this.insertRow(this.slaConditionTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_SCOPE_LABEL, this.bidiLeft, null);
		
		switch (tmodel.conditions[0].scope) {
		case this.messages.SLA_CONDITION_SCOPE_KEY_SINGLEVALUE:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_SCOPE_SINGLEVALUE, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_SCOPE_KEY_SUMOFVALUES:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_SCOPE_SUMOFVALUE, this.bidiLeft, null);
			break;
		case this.messages.SLA_CONDITION_SCOPE_KEY_AVERAGE:
			this.addTextCellToRow( newRow, this.eclMessages.SLA_CONDITION_SCOPE_AVERAGE, this.bidiLeft, null);
			break;
		default:
			throw new Error("Not a valid condition");
		
		}
	
	},
	
	
	//*****
	//Insert a new row into the table
	//   	In this case this is who the sla is exposed to
	//
	//Returns: reference to the new row
	//*****
	fillInExposingData: function()
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		newRow = this.insertRow(this.slaExposingTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_EXPOSE_TO, this.bidiLeft, null);
		if (tmodel.exposedTo != null) {
			this.addParticipantGrpCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(tmodel.exposedTo.participantGroupName), this.bidiLeft, null, tmodel.exposedTo);
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tmodel.exposedTo.participantGroupName);
			if (dir=="rtl") {
				if (newRow.cells[newRow.cells.length-1])
					newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
			}
		} else {
			this.addTextCellToRow( newRow, this.eclMessages.SLA_NONE, this.bidiLeft, null);		
		}
	
	},
	
	//*****
	//Insert a new row into the table
	//   	In this case this is the consequence of the SLA
	//
	//Returns: reference to the new row
	//*****
	fillInConsequenceData: function()
	{
		
		var newRow = null;
		var tmodel = slaModel;
		
		//fill in consequence email
		var emailStr = null;
		
		newRow = this.insertRow(this.slaConsequenceTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_SEND_TO, this.bidiLeft, null);
		if (tmodel.consequence.sendEmail) {
			emailStr = tmodel.consequence.emailAddress;
		} else {
			emailStr = this.eclMessages.SLA_NONE;
		}
		this.addTextCellToRow( newRow, emailStr, this.bidiLeft, null);
	
		// fill in consequence process fired
		var processStr = null;
		
		newRow = this.insertRow(this.slaConsequenceTableBody);
		this.addTextCellToRow( newRow, this.eclMessages.SLA_INITIATE_PROCESS, this.bidiLeft, null);
		if (tmodel.consequence.fireBPD) {
			if (tmodel.consequence.bpdRefInfo) {
			   processStr = tmodel.consequence.bpdRefInfo.bpdName;
			   this.addProcessRefCellToRow( newRow, processStr, this.bidiLeft, null, tmodel.consequence.bpdRefInfo);
			}
		} else {
			processStr = this.eclMessages.SLA_NONE;
			this.addTextCellToRow( newRow, processStr, this.bidiLeft, null);
		}
		
	},
	
	//*****
	//Insert a new row into the table
	//   	In this case this is the trigger for the sla
	//
	//Returns: reference to the new row
	//*****
	fillInTriggerData: function()
	{
		
		var newRow = null;
		var tmodel = slaModel;
		var str = null;
		var parms = [];
		
		switch(tmodel.trigger.occurrenceType){
		
			case this.messages.SLA_WHENEVER_VIOLATED:
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_LABEL, this.bidiLeft, null);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_WHENEVER_VIOLATED_MSG, this.bidiLeft, null);
				break;
			case this.messages.SLA_VIOLATED_A_OF_LAST_B_TIMES:
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_LABEL, this.bidiLeft, null);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_VIOLATED_A_OF_LAST_B_TIMES_MSG, this.bidiLeft, null);
				
				//add count A row
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_COUNTA, this.bidiLeft, null);
				
				parms[0] = tmodel.trigger.countA ;
				str = dojo.string.substitute(this.eclMessages.SLA_TIMES, parms);
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
				
				//add count B row
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_COUNTB, this.bidiLeft, null);
				parms[0] = tmodel.trigger.countB ;
				str = dojo.string.substitute(this.eclMessages.SLA_TIMES, parms);
				
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);

				break;				
			case this.messages.SLA_VIOLATED_N_TIMES_OVER_PERIOD:
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_LABEL, this.bidiLeft, null);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_VIOLATED_N_TIMES_OVER_PERIOD_MSG, this.bidiLeft, null);
				
				//add count N row
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_COUNTN, this.bidiLeft, null);
				parms[0] =  tmodel.trigger.countN ;
				str = dojo.string.substitute(this.eclMessages.SLA_TIMES, parms);
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
				
				//add over period row
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_OVER_PERIOD_LABEL, this.bidiLeft, null);
				
				this.fillInTriggerPeriod(tmodel, newRow, false);
				
				break;					
			case this.messages.SLA_VIOLATED_PERCENT_OF_TIME_OVER_PERIOD:
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_LABEL, this.bidiLeft, null);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_VIOLATED_PERCENT_OF_TIME_OVER_PERIOD_MSG, this.bidiLeft, null);
				
				//add percentage
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_PERCENTAGE_LABEL, this.bidiLeft, null);
				
				parms[0] =  tmodel.trigger.percent ;
				str = dojo.string.substitute(this.eclMessages.SLA_TRIGGER_PERCENTAGE_OF_TIME, parms);
				
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
				
				//add over period row
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_OVER_PERIOD_LABEL, this.bidiLeft, null);
				
				this.fillInTriggerPeriod(tmodel, newRow, false);
				break;				
			case this.messages.SLA_VIOLATED_PERCENT_MORE_OF_TIME_OVER_PERIOD:
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_LABEL, this.bidiLeft, null);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_VIOLATED_PERCENT_MORE_OF_TIME_OVER_PERIOD_MSG, this.bidiLeft, null);
				
				//add percentage
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_PERCENTAGE_LABEL, this.bidiLeft, null);
				
				parms[0] =  tmodel.trigger.percent ;
				str = dojo.string.substitute(this.eclMessages.SLA_TRIGGER_PERCENTAGE_MORE_OF_TIME, parms);
				
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
				
				//add over period row
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_OVER_PERIOD_LABEL, this.bidiLeft, null);
					
				this.fillInTriggerPeriod(tmodel, newRow, true);
				
				break;				
			case this.messages.SLA_VIOLATED_PERCENT_LESS_OF_TIME_OVER_PERIOD:
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_LABEL, this.bidiLeft, null);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_VIOLATED_PERCENT_LESS_OF_TIME_OVER_PERIOD_MSG, this.bidiLeft, null);
				
				//add percentage
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_PERCENTAGE_LABEL, this.bidiLeft, null);
				
				parms[0] =  tmodel.trigger.percent ;
				str = dojo.string.substitute(this.eclMessages.SLA_TRIGGER_PERCENTAGE_LESS_OF_TIME, parms);
				this.addTextCellToRow( newRow, str, this.bidiLeft, null);
				
				//add less period row
				newRow = this.insertRow(this.slaTriggerTableBody);
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TRIGGER_OVER_PERIOD_LABEL, this.bidiLeft, null);
					
				this.fillInTriggerPeriod(tmodel, newRow, true);
				break;				
					
			default:
				throw new Error("Not a valid occurenceType");
		}
	},
	
	//*****
	//Fill in the trigger period 
	//
	//Returns: reference to the new row
	//*****
	fillInTriggerPeriod: function(tmodel, newRow, percentLessOrEq)
	{
		switch (tmodel.trigger.timeUnit.unitType) {
		case this.messages.SLA_TIME_PERIOD_HOURS:
			this.handleHoursTimePeriod(tmodel, newRow, percentLessOrEq);
			break;
		case this.messages.SLA_TIME_PERIOD_DAYS:
			this.handleDaysTimePeriod(tmodel, newRow, percentLessOrEq);
			break;
		case this.messages.SLA_TIME_PERIOD_WEEKS:
			this.handleWeeksTimePeriod(tmodel, newRow, percentLessOrEq);
			break;
		case this.messages.SLA_TIME_PERIOD_MONTHS:
			this.handleMonthsTimePeriod(tmodel, newRow, percentLessOrEq);
			break;
		case this.messages.SLA_TIME_PERIOD_YEARS:
			this.handleYearsTimePeriod(tmodel, newRow, percentLessOrEq);
			break;
		default:
			throw new Error("Invalid time period.");	
		}			
	
	},


	//*****
	//If the time unit of the time period is hours, output proper thing here by adding 
	// proper text to row.
	//
	//Returns: reference to the new row
	//*****
	handleHoursTimePeriod: function(tmodel, newRow, percentLessOrEq)
	{
		if (tmodel.trigger.timeUnit.rollingTime) {
			// this the the last n hours
			var str = null;
			var parms = [];
			parms[0] = tmodel.trigger.timeUnit.timeCount;
			if (percentLessOrEq) {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_HOURS_PLUS, parms);
			} else {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_HOURS, parms);
			}
			this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			
		} else {
			//this is just the current hour 	SLA_TIME_PERIOD_THIS_HOUR
			if (percentLessOrEq) {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_HOUR_PLUS, this.bidiLeft, null);
			} else {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_HOUR, this.bidiLeft, null);
			}
		
		}
	},
	
	//*****
	//If the time unit of the time period is days, output proper thing here by adding 
	// proper text to row.
	//
	//Returns: reference to the new row
	//*****
	handleDaysTimePeriod: function(tmodel, newRow, percentLessOrEq)
	{
		if (tmodel.trigger.timeUnit.rollingTime) {
			// this the the last n days
			var str = null;
			var parms = [];
			parms[0] = tmodel.trigger.timeUnit.timeCount;
			if (percentLessOrEq) {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_DAYS_PLUS, parms);
			} else {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_DAYS, parms);
			}
			this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			
		} else {
			//this is just the current days
			if (percentLessOrEq) {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_DAY_PLUS, this.bidiLeft, null);
			} else {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_DAY, this.bidiLeft, null);
			}
			
		}
	},
	
	//*****
	//If the time unit of the time period is weeks, output proper thing here by adding 
	// proper text to row.
	//
	//Returns: reference to the new row
	//*****
	handleWeeksTimePeriod: function(tmodel, newRow, percentLessOrEq)
	{
		if (tmodel.trigger.timeUnit.rollingTime) {
			// this the the last n weeks
			var str = null;
			var parms = [];
			parms[0] = tmodel.trigger.timeUnit.timeCount;
			if (percentLessOrEq) {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_WEEKS_PLUS, parms);
			} else {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_WEEKS, parms);
			}
			this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			
		} else {
			//this is just the current week
			if (percentLessOrEq) {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_WEEK_PLUS, this.bidiLeft, null);
			} else {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_WEEK, this.bidiLeft, null);
			}
			
		}
	},
	
	//*****
	//If the time unit of the time period is weeks, output proper thing here by adding 
	// proper text to row.
	//
	//Returns: reference to the new row
	//*****
	handleMonthsTimePeriod: function(tmodel, newRow, percentLessOrEq)
	{
		if (tmodel.trigger.timeUnit.rollingTime) {
			// this the the last n months
			var str = null;
			var parms = [];
			parms[0] = tmodel.trigger.timeUnit.timeCount;
			if (percentLessOrEq) {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_MONTHS_PLUS, parms);
			} else {
				str = dojo.string.substitute(this.eclMessages.SLA_TIME_PERIOD_N_MONTHS, parms);
			}
			this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			
		} else {
			//this is just the current month 	
			if (percentLessOrEq) {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_MONTH_PLUS, this.bidiLeft, null);

			} else {
				this.addTextCellToRow( newRow, this.eclMessages.SLA_TIME_PERIOD_THIS_MONTH, this.bidiLeft, null);

			}
		}
	},
	
	
	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function(tbodyElem)
	{
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);
		return trElem;
	},
	

	//*****
	// Add a Bullet text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addBulletTextCellToRow: function(trElem, value, align, classname)
	{
//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), Started : " + value);
		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.whiteSpace="nowrap";

		tdElem.align=align;

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		
		
		    

		tdElem.innerHTML = "<ul style='margin-top:0px;margin-bottom:0px;'><li>"+value+"</li></ul>";  


//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), ended");
	},

	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.whiteSpace="nowrap";

		tdElem.align=align;

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		
		tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  


	},
	//*****
	// Add a process reference link cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addProcessRefCellToRow: function(trElem, value, align, classname, modelProperty)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.whiteSpace="nowrap";

		tdElem.align=align;

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		

		tdElem.innerHTML = "<u><b>"+value+"</b></u>";  
		tdElem.style.cursor="pointer";
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(value);
		if (dir=="rtl")
			tdElem.style.direction="rtl";

		dojo.addClass(tdElem, "linkFieldFont");	    

		tdElem.modelProp = modelProperty;
		dojo.connect( tdElem, "onclick", function(){
//				alert("click!");
			var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
			var params = {};
		    	params.documentId = tdElem.modelProp.bpdId;
		    	params.documentName = tdElem.modelProp.bpdName;
		    	
		    	if (tdElem.modelProp.snapshotId != null) {
		    		params.snapshotId = tdElem.modelProp.snapshotId;
		    	}
		    	
		    	if (tdElem.modelProp.branchId != null ) {
		    		params.branchId = tdElem.modelProp.branchId;
		    	}
		    		
		    	openDocEvent.params = params;
		    	openDocEvent.dispatch();				
		});

	},
	
	//*****
	// Add a particpant group link cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addParticipantGrpCellToRow: function(trElem, value, align, classname, modelProperty)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.whiteSpace="nowrap";

		tdElem.align=align;

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		
		if (modelProperty.snapshotId || modelProperty.branchId) {
			tdElem.innerHTML = "<u><b>"+value+"</b></u>";  
			tdElem.style.cursor="pointer";
			dojo.addClass(tdElem, "linkFieldFont");	    
	
			tdElem.modelProp = modelProperty;
			dojo.connect( tdElem, "onclick", function(){
	//				alert("click!");
				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.participantGroupId;
			    	params.documentName = tdElem.modelProp.participantGroupName;
			    	
			    	if (tdElem.modelProp.snapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.snapshotId;
			    	}
			    	
			    	if (tdElem.modelProp.branchId != null ) {
			    		params.branchId = tdElem.modelProp.branchId;
			    	}
			    		
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		
	  } else {
	  	tdElem.innerHTML = value;  
	  }

	},



	getpgData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	pgModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
