dojo.provide("com.ibm.bpm.wpd.document.tg.view.tgViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");


/**
 * tgViewer
 */
dojo.declare("com.ibm.bpm.wpd.document.tg.view.tgViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.tg.view",
	"templates/tgViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	tgModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 

	
	},

	
	postCreate : function() {
		
		
		if ( this.debug ){
			//Get all the data from the server (Direct file read in this case).
			this.geteaData();
		}else {
			//Get the server side data
			tgModel = this.viewStateModel.getDocumentModel();;
		}
		
		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (typeof(tgModel.richDesc) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(tgModel.richDesc));
			this.tgDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tgModel.desc);
			if (dir=="rtl")
				this.tgDescription.setAttribute("style","direction:rtl; text-align:left;");
		} else { 
			this.tgDescription.innerHTML = "&nbsp;";
		}

		this.tgName.innerHTML = tgModel.name;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tgModel.name);
		if (dir=="rtl")
			this.tgName.setAttribute("style","direction:rtl; text-align:left;");
		
		var m = tgModel;
		
		this.fillInFieldsTable();
		
		/*
		 *  Create Copy link href Document
		 */
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.tgCopyLinkHref);
		

	},
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.tg.view", "tgViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.tg.view", "tgViewerResources");


	},

	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},
	

	//*****
	//Insert fields into the tracking group fields table
	//
	//Returns: reference to the new row
	//*****
	fillInFieldsTable: function()
	{
	
		var newRow = null;
		
		if (tgModel.variables != null) {
		
			var numFields = tgModel.variables.length;
			
			for (var i=0; i<numFields; i++) {
				newRow = this.insertRow(this.tgFieldsTableBody);
				this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(tgModel.variables[i].name), this.bidiLeft, null, null );
				var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tgModel.variables[i].name);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
				this.addTextCellToRow( newRow, tgModel.variables[i].type, this.bidiLeft, null, tgModel.variables[i]);
				dir = com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tgModel.variables[i].type);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
				if ((typeof(tgModel.variables[i].richDesc) !== "undefined")&&(tgModel.variables[i].richDesc != null)) {
					var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(tgModel.variables[i].richDesc));
					dojo.addClass(newRow, "richTextFormat");
					this.addTextCellToRow( newRow, str, this.bidiLeft, null, null );
					var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tgModel.variables[i].desc);
					if (dir=="rtl") {
						if (newRow.cells[newRow.cells.length-1])
							newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
					}
				} else {
					this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );
				}
			}
		}

	},
	

	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function(tbodyElem)
	{
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);

		return trElem;
	},

	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname, modelProperty)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";


		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";

		tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  


//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), ended");
	},

	geteaData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	boModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
