dojo.provide("com.ibm.bpm.wpd.document.uca.view.ucaViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");


/**
 * UCA viewer
 * 
 * 		
 */
dojo.declare("com.ibm.bpm.wpd.document.uca.view.ucaViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.uca.view",
	"templates/ucaViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	ucaModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 
	
	},

	
	postCreate : function() {
		
		
		if ( this.debug ){
			//Get all the data from the server (Direct file read in this case).
			this.getpgData();
		}else {
			//Get the server side data
			ucaModel = this.viewStateModel.getDocumentModel();;
		}
		
		
		console.log("in postCreate: " + ucaModel.name);
		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (typeof(ucaModel.richDesc) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(ucaModel.richDesc));
			this.ucaDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(ucaModel.desc);
			if (dir=="rtl")
				this.ucaDescription.setAttribute("style","direction:rtl; text-align:left;");

		} else { 
			this.ucaDescription.innerHTML = "&nbsp;";
		}

		this.ucaName.innerHTML = com.ibm.bpm.wpd.util.Helper.text2htmlText(ucaModel.name);
		this.ucaNameHeader.innerHTML = com.ibm.bpm.wpd.util.Helper.text2htmlText(ucaModel.name);
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(ucaModel.name);
		if (dir=="rtl") {
			this.ucaName.setAttribute("style","direction:rtl; text-align:left;");
			this.ucaNameHeader.setAttribute("style","direction:rtl; text-align:left;");
		}
		this.fillInUcaData();
        
		/*
		 *  Create Share link href Document
		 */
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.ucaCopyLinkHref);
		
	},
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.uca.view", "ucaViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.uca.view", "ucaViewerResources");


	},
	
	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},

	
	//*****
	//Insert a new row into the table
	//   	In this case this is an expression type of participant group
	//
	//Returns: reference to the new row
	//*****
	fillInUcaData: function()
	{
		
		var newRow = null;
		var m = ucaModel;
		
		
		// Service row
		newRow = this.insertRow(this.ucaTableBody);
		this.addTextCellToRow( newRow, this.messages.UCA_ATTACHED_SERVICE, this.bidiLeft, null);
		this.addLinkCellToRow( newRow, m.serviceName, this.bidiLeft, null, m); 
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(m.serviceName);
		if (dir=="rtl") {
			if (newRow.cells[newRow.cells.length-1])
				newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
		}
		
		// Associated queue row
		newRow = this.insertRow(this.ucaTableBody);
		this.addTextCellToRow( newRow, this.messages.UCA_ASSOCIATED_QUEUE, this.bidiLeft, null);
		this.addTextCellToRow( newRow, m.queue, this.bidiLeft, null);
		
		
		if (m.schedType == "OnEvent") {
			// Event Message row
			newRow = this.insertRow(this.ucaTableBody);
			this.addTextCellToRow( newRow, this.messages.UCA_EVENT_MESSAGE, this.bidiLeft, null);
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlText(this.replaceNewLines(m.eventMessage));
			this.addTextCellToRow( newRow, str, this.bidiLeft, null);
			
			// Enabled row
			newRow = this.insertRow(this.ucaTableBody);
			this.addTextCellToRow( newRow, this.messages.UCA_ENABLED, this.bidiLeft, null);
			this.addTextCellToRow( newRow, m.enabled, this.bidiLeft, null);
			
			
		} else {
			
			// Enabled row
			newRow = this.insertRow(this.ucaTableBody);
			this.addTextCellToRow( newRow, this.messages.UCA_ENABLED, this.bidiLeft, null);
			this.addTextCellToRow( newRow, m.enabled, this.bidiLeft, null);
			
			
			// Schedule header
			newRow = this.insertRow(this.ucaTableBody);
			this.addTextCellToRow( newRow, "", this.bidiLeft, null);
			newRow = this.insertRow(this.ucaTableBody);
			this.addTextCellToRow( newRow, "", this.bidiLeft, null);

			newRow = this.insertRow(this.ucaTableBody);
			this.addTextCellToRow( newRow, this.messages.UCA_SCHEDULE_HEADER, this.bidiLeft, null);

			
			
			// Create Month Table
			
			var i = 1;
			var numMons = m.timeSchedule.month.length;
			for (i=0;i<numMons;i++)
			{
				newRow = this.insertRow(this.ucaMonthTableBody);
				this.addTextCellToRow( newRow, this.getMonthString(m.timeSchedule.month[i]), this.bidiLeft, null);
			}

			
			// Create Freq Table
			
			if (m.timeSchedule.freqType != null) {
				newRow = this.insertRow(this.ucaFreqTableBody);
				var freq = null;
				
				switch (m.timeSchedule.freqType) {
				case "First":
					freq = this.messages.UCA_FREQ_FIRST;
					break;
				case "Last":
					freq = this.messages.UCA_FREQ_LAST;
					break;
				case "Every":
					freq = this.messages.UCA_FREQ_EVERY;
					break;					
				}
				this.addTextCellToRow( newRow, freq, this.bidiLeft, null);
			}
			
			


			
			
			// Create Days Table
			var i = 1;
			var numDays = m.timeSchedule.weekDay.length;
			for (i=0;i<numDays;i++)
			{
				newRow = this.insertRow(this.ucaDaysTableBody);
				this.addTextCellToRow( newRow, this.getDayString(m.timeSchedule.weekDay[i]), this.bidiLeft, null);
			}
			
			
		
			// Create Hours Table
			var i=0;
			var numHours = m.timeSchedule.hours.length;
			for (i=0;i<numHours;i++)
			{
				newRow = this.insertRow(this.ucaHoursTableBody);
				this.addTextCellToRow( newRow, m.timeSchedule.hours[i] + ":00", this.bidiLeft, null);
			}


			
			// Create the Minutes table
			
			var i=0;
			var parms = [];
			var str = null;
			var numMins = m.timeSchedule.minutes.length;
			for (i=0;i<numMins;i++)
			{
				newRow = this.insertRow(this.ucaMinsTableBody);
				if (m.timeSchedule.minutes[i] == "0") {
					this.addTextCellToRow( newRow, this.messages.UCA_MINS_ONTHEHOUR, this.bidiLeft, null);
				} else {
					parms[0] = m.timeSchedule.minutes[i];
					str = dojo.string.substitute(this.messages.UCA_MINS_PASTHOUR, parms);
					this.addTextCellToRow( newRow, str, this.bidiLeft, null);
				}
				
			}
			
			
			
			
//			this.addTextCellToRow( newRow, m.eventMessage, this.bidiLeft, null);
			
		}
			

		


	},
	
	
	//*****
	//Convert from a day  to a day string
	//
	//Returns: month day of week
	//*****
	getDayString: function(dayParm)
	{


		var dayString = null;
		switch(dayParm) {
		case "MONDAY":
			dayString = this.messages.UCA_DAYS_MON;
			break;
		case "TUESDAY":
			dayString = this.messages.UCA_DAYS_TUES;
			break;		
		case "WEDNESDAY":
			dayString = this.messages.UCA_DAYS_WED;
			break;
		case "THURSDAY":
			dayString = this.messages.UCA_DAYS_THURS;
			break;
		case "FRIDAY":
			dayString = this.messages.UCA_DAYS_FRI;
			break;	
		case "SATURDAY":
			dayString = this.messages.UCA_DAYS_SAT;
			break;
		case "SUNDAY":
			dayString = this.messages.UCA_DAYS_SUN;
			break;			
			
		default:
			throw new Error("not a valid day");
	
		}
		return dayString;
	},
	
	//*****
	//Convert from a number to a month
	//
	//Returns: month string
	//*****
	getMonthString: function(monthNum)
	{

		
		var monString = null;
		switch(monthNum) {
		case 1:
			monString = this.messages.UCA_JAN;
			break;
		case 2:
			monString = this.messages.UCA_FEB;
			break;		
		case 3:
			monString = this.messages.UCA_MAR;
			break;
		case 4:
			monString = this.messages.UCA_APR;
			break;
		case 5:
			monString = this.messages.UCA_MAY;
			break;	
		case 6:
			monString = this.messages.UCA_JUNE;
			break;
		case 7:
			monString = this.messages.UCA_JULY;
			break;			
		case 8:
			monString = this.messages.UCA_AUG;
			break;		
		case 9:
			monString = this.messages.UCA_SEPT;
			break;
		case 10:
			monString = this.messages.UCA_OCT;
			break;			
		case 11:
			monString = this.messages.UCA_NOV;
			break;		
		case 12:
			monString = this.messages.UCA_DEC;
			break;		
			
		default:
			throw new Error("Invalid month!");
	
		}
		return monString;
	},
	

	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function(tbodyElem)
	{
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);
		return trElem;
	},

	//*****
	// Add a Link cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addLinkCellToRow: function(trElem, value, align, classname, modelProperty)
	{

		var tdElem = trElem.insertCell(trElem.cells.length);
		
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		    

//        "serviceId":"1.30397b90-a06d-4970-8f0c-ba8058c1271b",
//        "serviceName":" Installation Service",
//        "serviceSnapshotId":null,
//        "serviceBranchId":"2063.1a52abd6-b068-4f9e-91a9-ded9793eb34e",		
		if ((modelProperty != null) && (modelProperty.serviceId != null)) {
			tdElem.innerHTML = "<u><b>"+com.ibm.bpm.wpd.util.Helper.text2htmlText(value)+"</b></u>";  
			tdElem.style.cursor="pointer";
			dojo.addClass(tdElem, "linkFieldFont");	    

			tdElem.modelProp = modelProperty;
			dojo.connect( tdElem, "onclick", function(){

				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.serviceId;
			    	params.documentName = tdElem.modelProp.serviceName;
			    	
			    	if (tdElem.modelProp.serviceSnapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.serviceSnapshotId;
			    	}
			    	
			    	if (tdElem.modelProp.serviceBranchId != null ) {
			    		params.branchId = tdElem.modelProp.serviceBranchId;
			    	}
			    		
			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		} else {
			tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+com.ibm.bpm.wpd.util.Helper.text2htmlText(value)+"</font>";  
		}


	},
	
	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname)
	{
//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), Started : " + value);
		var tdElem = trElem.insertCell(trElem.cells.length);
		tdElem.style.whiteSpace="nowrap";

		tdElem.align=align;

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		    

		tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  
//		tdElem.innerHTML = "<font\">"+value+"</font>";  


//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), ended");
	},
	

	getpgData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	ucaModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
