dojo.provide("com.ibm.bpm.wpd.document.webservice.view.webserviceViewer");

dojo.require("com.ibm.bpm.wpd.core.view.BaseViewer");
dojo.require("com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.util.Helper");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils");

/**
 * webservice viewer
 */
dojo.declare("com.ibm.bpm.wpd.document.webservice.view.webserviceViewer", [ dijit._Widget,
		dijit._Templated, com.ibm.bpm.wpd.core.view.BaseViewer], {
	templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.document.webservice.view",
	"templates/webserviceViewer.html"),
	
	//Development flag. Used to force a load of data from the local file system without trying to talk to TeamWorks
	debug: false,

	//Model to display - either file if debug, or model already read via rest api
	wsModel: {},

	// Constructor function. Called when instance of this class is created
	constructor : function() { 

	},

	
	postCreate : function() {
		
		
		if ( this.debug ){
			//Get all the data from the server (Direct file read in this case).
			this.getboData();
		}else {
			//Get the server side data
			wsModel = this.viewStateModel.getDocumentModel();;
		}
		
		
		console.log("in postCreate: " + wsModel.name);
		this.bidiLeft = "left";
		this.bidiRight = "right";

		if (typeof(wsModel.richDesc) !== "undefined") {
			var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(wsModel.richDesc));
			this.webserviceDescription.innerHTML = str;
			var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(wsModel.desc);
			if (dir=="rtl")
				this.webserviceDescription.setAttribute("style","direction:rtl; text-align:left;");
		} else { 
			this.webserviceDescription.innerHTML = "&nbsp;";
		}

		this.webserviceName.innerHTML = wsModel.name;
		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(wsModel.name);
		if (dir=="rtl") {
			this.webserviceName.setAttribute("style","direction:rtl; text-align:left;");
		}
		
		this.fillInTable();
		
		this.webserviceWSDLLink.href = wsModel.wsdlUrl;
		
		var parms = [];
		parms[0] = wsModel["protected"];
		var str = dojo.string.substitute(this.messages.WEBSERVICE_PROTECTED_LABEL, parms);
		this.webserviceProtectedInd.innerHTML = str;
		
		com.ibm.bpm.wpd.core.view.BaseViewer.createCopyLinkHref(this.webserviceCopyLinkHref);
	},
	
	postMixInProperties: function()
	{

        dojo.requireLocalization("com.ibm.bpm.wpd.document.webservice.view", "webserviceViewerResources");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.document.webservice.view", "webserviceViewerResources");
		


	},

	replaceNewLines: function(str)
	{
		var newStr =  str.replace(/\r\n/g, "<br>");
		return newStr.replace(/\n/g, "<br>");
	},
	

	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	fillInTable: function()
	{
		
		var newRow = null;
		
		if (wsModel.operations != null) {
			var numOps = wsModel.operations.length;
			
			for (var i=0; i<numOps; i++) {
				newRow = this.insertRow();
				this.addTextCellToRow( newRow, wsModel.operations[i].name, this.bidiLeft, null, null );
				var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(wsModel.operations[i].name);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
				this.addTextCellToRow( newRow, com.ibm.bpm.wpd.util.Helper.text2htmlText(wsModel.operations[i].serviceName), this.bidiLeft, null, wsModel.operations[i] );
				var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(wsModel.operations[i].serviceName);
				if (dir=="rtl") {
					if (newRow.cells[newRow.cells.length-1])
						newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
				}
				if (typeof(wsModel.operations[i].richDesc) !== "undefined") {
					var str = com.ibm.bpm.wpd.util.Helper.text2htmlTextWeakEncoding(this.replaceNewLines(wsModel.operations[i].richDesc));
					dojo.addClass(newRow, "richTextFormat");
					this.addTextCellToRow( newRow, str, this.bidiLeft, null, null );
					var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(wsModel.operations[i].desc);
					if (dir=="rtl") {
						if (newRow.cells[newRow.cells.length-1])
							newRow.cells[newRow.cells.length-1].setAttribute("style","direction:rtl; text-align:left;");
					}
				} else {
					this.addTextCellToRow( newRow, "", this.bidiLeft, null, null );
				}
			}
		}
	},
	
	//*****
	//Insert a new row into the table
	//
	//Returns: reference to the new row
	//*****
	insertRow: function()
	{
		var tbodyElem = this.webserviceTableBody;
		var trElem = tbodyElem.insertRow(tbodyElem.rows.length);

		return trElem;
	},

	//*****
	// Add a text cell to a row
	// Parameters
	//		trElem - the row to add to
	//		value  - the string to add to the cell
	//		align
	//		classname
	//	
	//
	//*****
	addTextCellToRow: function(trElem, value, align, classname, modelOp)
	{
//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), Started : " + value);
		var tdElem = trElem.insertCell(trElem.cells.length);
//		tdElem.style.whiteSpace="nowrap";
//		tdElem.style.whiteSpace="normal";
		tdElem.style.wordWrap="break-word";

		tdElem.align=align;
		tdElem.vAlign="top";

		if (classname)
		    tdElem.className = classname;
		else
		    tdElem.className = "celborder1";
		    

		if ((modelOp != null) && (modelOp.serviceId != null)) {
			tdElem.innerHTML = "<u>"+value+"</u>";  
			tdElem.style.cursor="pointer";
			dojo.addClass(tdElem, "linkFieldFont");	    

			tdElem.modelProp = modelOp;
			dojo.connect( tdElem, "onclick", function(){

				var openDocEvent = new com.ibm.bpm.wpd.core.control.events.OpenDocumentEvent();
				var params = {};
			    	params.documentId = tdElem.modelProp.serviceId;
			    	params.documentName = tdElem.modelProp.serviceName;
			    	
			    	if (tdElem.modelProp.serviceSnapshotId != null) {
			    		params.snapshotId = tdElem.modelProp.serviceSnapshotId;
			    	}
			    	
			    	if (tdElem.modelProp.serviceBranchId != null ) {
			    		params.branchId = tdElem.modelProp.serviceBranchId;
			    	}

			    	openDocEvent.params = params;
			    	openDocEvent.dispatch();				
			});
		} else {
			
			tdElem.innerHTML = "<font class=\"defaultFieldFont\">"+value+"</font>";  

		}

//		console.debug("JobStatus Widget, addTextCellToRow: funtion(), ended");
	},

	getboData : function(){

		var result = {};
        var xhrArgs = {
                url: "bodata.json",
                sync: true,
                handleAs: "json",
                load: dojo.hitch(this, function(response) {
                	wsModel = response.data;
                }),
                error: dojo.hitch(this, function(response) {
                	alert("failure " + response);
                })
            };

        //Call the synchronous xhrGet
        dojo.xhrGet(xhrArgs);
     
	}
}
//Uncomment above comma and add comma-separated functions here. Do not leave a 
// trailing comma after last element.			
);	
	
	
