dojo.provide('com.ibm.bpm.wpd.visualization.BPMNNavigator');

dojo.require("ibm_ilog.diagram.interactors.Navigator");

(function(){

/**
 * Subclass the iLog implementation to address issues. Currently the navigator isn't extendible to
 * include different styles. subclassing to overwrite the setNavigationStyle to include our implementation
 */
dojo.declare("com.ibm.bpm.wpd.visualization.BPMNNavigator",ibm_ilog.diagram.interactors.Navigator,{
	
	
	setNavigationStyle: function(style){
		// Summary:
		//		sets the navigation style.
		// style: number
		//		one of the navigation styles
		this._strategyNumber = style;
		switch(style){
			case ibm_ilog.diagram.interactors.Navigator.TreeStyle: this._strategy = new ibm_ilog.diagram.interactors.TreeNavigator(this._diagram);break;
			case ibm_ilog.diagram.interactors.Navigator.GraphStyle: this._strategy = new ibm_ilog.diagram.interactors.GraphNavigator(this._diagram);break;
			case com.ibm.bpm.wpd.visualization.BPMNNavigator.GraphStyle: this._strategy = new com.ibm.bpm.wpd.visualization.BPMNGraphNavigator(this._diagram);break;
			default: this._strategy = new ibm_ilog.diagram.interactors.NullNavigator(this._diagram); this._strategyNumber = ibm_ilog.diagram.interactors.Navigator.NullStyle;break;
		}
	}
});

/**
 * Overwrite the Graph implementation to include nodes that don't have any connections (i.e. swimlane headers). The provided navigation implements are
 * pretty rigid. Graph based requires all nodes to be connected. Tree based doens't require nodes to be connected, but the order would appear random
 * to an end user
 */
dojo.declare("com.ibm.bpm.wpd.visualization.BPMNGraphNavigator",[ibm_ilog.diagram.interactors.GraphNavigator],{
	
	/**
	 * In the even we have no connection we could be at the swimlane. Move to the first node
	 */
	_followConnectionForward: function(ge){
		if(ge._isIBMDiagramNode && !this._stack.peek()){
			if(ge && ge._isIBMDiagramSubgraph){
				if(ge.isCollapsed() && ge.canCollapse()){
					ge.setCollapsed(false);	
				}else{
					var children = ge.getGraph().children;
					if(children.length>0){
						this.setFocusedElement(children[0]);
						return;
					}
				}
			}
		}
		else
			this.inherited(arguments);
		
	},

	/**
	 * If no connection exists move to the next child
	 */
	next: function(ge){
		var ge = this.getFocusedElement();
		if(ge._isIBMDiagramNode){
			var links = this._getElementLinks(ge);
			if(links.length==0){
				return this._nextNode(ge,1);
			}
		}
		this.inherited(arguments);
	},
	
	/**
	 * If no connection exists move to the parent
	 */
	previous: function(ge){
		var ge = this.getFocusedElement();
		if(ge._isIBMDiagramNode){
			var links = this._getElementLinks(ge);
			if(links.length==0){
				return this._nextNode(ge,-1);
			}
		}
		this.inherited(arguments);
	},

	_nextNode: function(c,increment){
		if(c){
			if(c._isIBMDiagramNode){
				var p = c.getParent();
				var next = this._getFollowingNode(p.children,c,increment);
				if(next){
					this.setFocusedElement(next);
				}else{
					this._nextNode(p._owningSubgraph,increment);
				}
			}
		}else{
			if(increment>0){
				this.setFocusedElement(this._getFirstNode());
			}else{
				this.setFocusedElement(this._getLastNode());
			}
		}
	},

	_getFollowingNode: function(array,element,increment){
		var index = this._getChildIndex(array,element); 
		for(var i = index+increment;i>=0 && i<array.length;i=i+increment){
			if(array[i]._isIBMDiagramNode){
				return array[i];
			}
		}
		return null;
	}
	
	});

com.ibm.bpm.wpd.visualization.BPMNNavigator.GraphStyle = 2;

})();