dojo.provide("com.ibm.bpm.wpd.visualization.BPMNViewerLite");

dojo.require("com.ibm.bpm.wpd.document.bpd.view.BPMNViewer");
dojo.require("dijit.form.HorizontalSlider");
dojo.require("ibm_ilog.diagram.adorners.AdornerHandle");
dojo.require("ibm_ilog.diagram.adorners.AnnotationAdorner");
dojo.require("ibm_ilog.diagram.Node");
dojo.require("com.ibm.bpm.wpd.visualization.model.SessionManagement");
dojo.require("com.ibm.bpm.wpd.util.BidiUtils" );  // Bidi Support
dojo.require("com.ibm.bpm.wpd.visualization.BPMNNavigator");

dojo.require("com.ibm.bpm.wpd.visualization.model.BPMNViewerVisualizationModel");

dojo.declare("com.ibm.bpm.wpd.visualization.BPMNViewerLite",[ com.ibm.bpm.wpd.document.bpd.view.BPMNViewer ], {
	// Path to the template
	templateString : dojo.cache(
			"com.ibm.bpm.wpd.visualization",
			"templates/BPMNViewerLite.html"),

	widgetsInTemplate : true,
	serverSideImageRendering : false,
	navigationStyle: "graph",
	
	keyStroke: null,
	
	// Adorner name
	PARAMETER_ADORNER:"ParameterAdorner",
	PARAMETER_HANDLE: "ParameterHandle",
	
	visualizationModel:null,

	postCreate : function() {
		keyStroke = null;
		
		this.visualizationModel = new com.ibm.bpm.wpd.visualization.model.BPMNViewerVisualizationModel();

		try {
			this.loadingMessaage = this.messages.BPD_LOADING;

		} catch (err) {
			console.debug(err.message);
		}

		this.displayAll();

		// Change the navigation style from tree to graph
		//this.ilogDiagrammer._keyInteractors.get(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId).setNavigationStyle(ibm_ilog.diagram.interactors.Navigator.GraphStyle);
		// Register our implementation and use the modified graph navigation
		this.ilogDiagrammer._keyInteractors.remove(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId);
		this.ilogDiagrammer._keyInteractors.add(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId, new com.ibm.bpm.wpd.visualization.BPMNNavigator().initialize(this.ilogDiagrammer));
		this.ilogDiagrammer._keyInteractors.get(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId).setNavigationStyle(com.ibm.bpm.wpd.visualization.BPMNNavigator.GraphStyle);
		
		// For some reason in Chrome the scrollbars seem to be interferred with by the diagram itself. Resizing sometimes works.
		// I'm not sure if this is an iLog issue or a Chrome isssue. Set the right to the width of the scrollbar. Don't
		// use the zIndex as it'll prevent diagram mouse clicks
//		if (dojo.isChrome) {
//			//this.ilogDiagrammer.scrollbarsNode.style.zIndex = 999;
//			//this.ilogDiagrammer.canvasNode.style.right="13px";
//			this.ilogDiagrammer.canvasNode.style.right= this.ilogDiagrammer.getViewport().getScrollbarWidth() + "px";
//		}

	},

	displayAll : function() {

		// Load the data
		this.getData();

		this.mainBC.startup();

		this.displayEditor();
		this.hideLoadingDiv();
		this.connectZoomControl();

//		this.ilogDiagrammer.getGraph().setAnimatedLayout(true);
//		this.ilogDiagrammer._setViewportAnimatedAttr(true);
//		this.mainBC.resize();
//		this.ilogDiagrammer.resize();
	},

	// SECTION: Parameter adornment
	
	/**
	 * Adorn the diagram with a given parameter. this is be be used internally within this class
	 * @param - parameter object
	 */
	_adornDigramWithParameter : function(aParameter) {
		// 1) If the parameter already exists, don't re adorn
		if (this.visualizationModel.adornmentExists(aParameter.name)) {
			this._updateLabelOnBreadcrumb(aParameter.name);
			return;
		}

		// 2) Add it to our model
		this.visualizationModel.addParameter(aParameter);
		
		com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().addParameter(aParameter.name);
		
		var nodesToAdorn = aParameter.nodes;

		// 3) Adorn the the nodes
		for ( var j = 0; j < nodesToAdorn.length; j++) {
			var activityId = nodesToAdorn[j].id;
			var activityPosition = nodesToAdorn[j].position;
			this._adorn(activityId, activityPosition, aParameter.name, aParameter.colour, aParameter.type);
		}
		
		var ex = com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().getParametersWithValues()[aParameter.name];
		if (ex && ex == com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().PARAMETER_EXPANDED)
			this._adornWithLabel(aParameter.name);

	},
	
	/**
	 * Adorn the diagram with a given parameter
	 * @param - This could be either a collection of parameters or a single parameter.
	 * NOTE: the collection of parameters is used when the view is refreshed (i.e. a drill down)
	 */
	adornDigramWithParameter: function(parameter) {

		if (dojo.isArray(parameter)) {
			
			this._cleanParameterAdornmentCollection(parameter);
			
			for (var i=0;i<parameter.length; i++) {
				this._adornDigramWithParameter(parameter[i]);
				
				var ex = com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().getParametersWithValues()[parameter[i].name];
				if (ex && ex == com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().PARAMETER_EXPANDED)
					this.visualizationModel.adornedElementsByParameterMap[parameter[i].name].adornedNodeLabelsExpanded = true;
			}
		}
		else {
			this._adornDigramWithParameter(parameter);
		}
	},
	
	/**
	 * Add the adorner to the id that is specified in the
	 * diagram. We obtain the iLog diagram and the node within
	 * the diagram. We create the adorner and then set the
	 * diagram node to it NOTE: If we ask the diagram for a node
	 * by id and it doesn't exist it'll throw an exception
	 * (would actually prefer just a null return)
	 * 
	 * @param id - the id of the node
	 * @param parameterName - the parameter name
	 * @param colour - colour of the adorner
	 * @param activityPosition - the side of the activity
	 */
	_adorn: function(id, activityPosition, parameterName, colour, parameterType) {

		// Sanity check, can't adorn a node without an id
		if (!id)
			return;

		// Find the node to adorn.
		var node = this._getNodeById(this.ilogDiagrammer, id);

		var elementHeightPosition = this.visualizationModel.getAdornedElementPosition(id, activityPosition);

		// Only create an adorner if it'll be visble, otherwise jsut add it to the collection
		if (!elementHeightPosition || elementHeightPosition < 3) {
			var gfx = this.ilogDiagrammer.getGraphElementFromDataItem(node);

			var dictionaryData = this.getAdornerInfoForIcon(colour, activityPosition,elementHeightPosition,parameterType,id);

			var a = this.ilogDiagrammer.createAdorner(ParameterAdorner,dictionaryData, this.PARAMETER_ADORNER);
			
			a.parameterName = parameterName;
			a.activityId = id;
			a.setAdorned(gfx);

			this.visualizationModel.addParameterIconAdornment(id, a, activityPosition, parameterName);
		}
		else {
			
			//add to the collection
			this.visualizationModel.addParameterIconAdornment(id, null, activityPosition, parameterName);
			
			// add the spinners
			if (this.visualizationModel.needsDownSpinner(id, activityPosition)) {
				this.createSpinnersForNode(id, activityPosition, "down");
			}
		}

	},

	/**
	 * An icon has been click, expand the label
	 * @param variableName
	 */
	_adornWithLabel: function(variableName) {

		// Find the variable name data
		var variableData = this.visualizationModel.getAdornmentInfoViaParameterName(variableName);

		if (!variableData)
			return;

		// Are we adorning or removing? If the label has already been expanded, close it
		if (variableData.adornedNodeLabelsExpanded) {
			this.visualizationModel.closeNameAdormentsForParameterName(variableName);
			com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().updateParameterStatus(variableName,
					com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().PARAMETER_COLLPASED);
			return;
		}

		com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().updateParameterStatus(variableName,
				com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().PARAMETER_EXPANDED);
		for ( var i = 0; i < variableData.nodes.length; i++) {
			var id = variableData.nodes[i].id;
			var elementHeightPosition = variableData.nodes[i].position;
			var activityPosition = variableData.nodes[i].side;

			var node = this._getNodeById(this.ilogDiagrammer, id);
			var gfx = this.ilogDiagrammer.getGraphElementFromDataItem(node);

			// if -t's -1 we know it's not showing
			if (elementHeightPosition > -1) {
				
				var expandTo = "right";
				if (activityPosition == "left") {
					if (node.x[0] > 93)
						expandTo = "left";
				}
				var dictionaryData = this.getAdornerInfoForName(variableData.colour,activityPosition,elementHeightPosition,variableName,expandTo,id);
				
				var a = this.ilogDiagrammer.createAdorner(ParameterAdorner,dictionaryData, this.PARAMETER_ADORNER);
				a.setAdorned(gfx);
				a.moveToBack();

				// Bidi Support
//				var paraName = this.truncateText(variableName, 13);
//				var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(paraName);
//				if (dir=="rtl") {
//					this.modifyAdornTextDir(a, paraName, activityPosition);
//				}
				this.visualizationModel.addParameterLabelAdornment(id, variableName, activityPosition, a, elementHeightPosition);
			}
		}
	},

	/**
	 * Remove a parameter from the diagram
	 * @param aParameter
	 */
	removeParameterAdornment : function(aParameter) {
		//1) Remove the parameter form the model
		var containersToRedraw = this.visualizationModel.removeParameterAdornment(aParameter.name);

		com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().removeParameter(aParameter.name);
		
		// Possibly trigger a diagram redraw on containers that lost a parameter adornment
		for (var i=0; i<containersToRedraw.length;i++) {
			this._spinParameters(containersToRedraw[i].id, containersToRedraw[i].side, "neutral");
		}

	},
	
	/**
	 * Clear the parameters from the model
	 * @param parameters - collection of parameter names
	 */
	_cleanParameterAdornmentCollection: function(parameters) {
		
		// First get the adorned values
		var pCollection = this.visualizationModel.getAdornmentParameters();
		
		// Remove any names that are coming in via the parameter array
		for (var j=0;j<parameters.length;j++) {
			if (pCollection[parameters[j].name])
				pCollection[parameters[j].name] = null;
		}
		
		// Whatever is left has been unadorned by a another process
		for (var name in pCollection) {
			if (pCollection[name] != null) {
				this.removeParameterAdornment(pCollection[name].data);
			}
		}
		
	},
	
	/**
	 * Add the label adorner on adorments that have been expanded
	 * @param parameterName
	 */
	_updateLabelOnBreadcrumb: function(parameterName) {
		var ex = com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().getParametersWithValues()[parameterName];
		
		if (!ex)
			return;
		
		var localElement = this.visualizationModel.adornedElementsByParameterMap[parameterName];
		if (!localElement)
			return;
		
		if (localElement.adornedNodeLabelsExpanded && ex == com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().PARAMETER_EXPANDED)
			return;
		
		if (!localElement.adornedNodeLabelsExpanded && ex == com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().PARAMETER_COLLPASED)
			return;
		
		// the session value is different than our local value, the expansion/collapse took place on a different process. Sync up
		this._adornWithLabel(parameterName);
		
	},
	
	// SECTION: Parameter adornment helper functions
	
	/**
	 * Get the adorner dictionary for the given position on the node 
	 * @param colour - hex colour of the adorner
	 * @param position - the side of the node to be adornered (left or right)
	 * @param heightPosition - the slot position on the side. 1, 2 or 3.
	 * @param parameterType - the type of the parameter, either input,output or private
	 * @param nodeId - the id of the node
	 * @returns {dojox.collections.Dictionary}
	 */
	getAdornerInfoForIcon : function(colour, position, heightPosition, parameterType, nodeId) {

		var aDictionary = new dojox.collections.Dictionary();

		// Right now we can support 3 heights
		var aHeightPosition = "3";

		if (heightPosition == 1)
			aHeightPosition = "25";

		if (heightPosition == 2)
			aHeightPosition = "47";
		
		if (this.isNodeMessage(nodeId))
			aHeightPosition = aHeightPosition - 22;

		aHeightPosition = aHeightPosition * this.zoomLevel;

		var tRight = 5 * this.zoomLevel;
		var convertedPosition = "{{elemBaseRight}} - " + tRight;
		//var imageToUse = "Everest_outputVis_icon.png";
		var imageToUse = null;

		if (position == this.visualizationModel.CONTAINER_LEFT_SIDE) {
			var tLeft = 13 * this.zoomLevel;
			convertedPosition = "{{elemBaseLeft}} - " + tLeft;
			//imageToUse = "Everest_inputVis_icon.png";
		}
		
		if(parameterType) {
			if (parameterType == "input")
				imageToUse = "Everest_inputVis_icon.png";
			else if (parameterType == "output")
				imageToUse = "Everest_outputVis_icon.png";
			else
				imageToUse = null;
		}

		aDictionary.add(this.PARAMETER_ADORNER, "[{"
			+ " dojoAttachPoint:'button'," + " template:'" + this.PARAMETER_HANDLE + "'," + " transform: { dx: " + convertedPosition
			+ " , dy: {{elemBaseTop}} + " + aHeightPosition + " }" + "}]");

		var tWidth = 20 * this.zoomLevel;
		var tHeight = 20 * this.zoomLevel;

		var tCX = 10 * this.zoomLevel;
		var tCY = 10 * this.zoomLevel;
		var tR = 10 * this.zoomLevel;
		var darkColour = this.gradientColor(colour.substring(1));
		
		if (imageToUse) {
			aDictionary.add(this.PARAMETER_HANDLE, "["
				+ "{ shape: { type: 'circle', cx:" + tCX + ", cy:" + tCY + ", r:" + tR + " }," +
				//"fill: '" + colour + "'}," +
				"fill: {type:'linear',x:10,y:0,x2:0,y2:20, colors:[{offset: 0.5, color:'" + colour  + "'},{offset: 1, color:'" + darkColour  + "'}]}}," +
						"{ shape: { type: 'image', width: " + tWidth + ", height: " + tHeight
				+ ", src: '" + com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_WEBVIEWER) + "/widgets/com/ibm/bpm/wpd/images/visualization/" + imageToUse + "' } }]");
		}
		else {
			aDictionary.add(this.PARAMETER_HANDLE, "["
					+ "{ shape: { type: 'circle', cx:" + tCX + ", cy:" + tCY + ", r:" + tR + " }," +
					//"fill: '" + colour + "'}]");
					 "fill: {type:'linear',x:10,y:0,x2:0,y2:20, colors:[{offset: 0.5, color:'" + colour  + "'},{offset: 1, color:'" + darkColour  + "'}]}" +
					 		"}]");
		}


		return aDictionary;

	},
	
	/**
	 * Return whether a node is of type event. In this case the node is about a 1/3 of the size of an activity so the adorner positions will change
	 * @param id
	 * @returns {Boolean}
	 */
	isNodeMessage: function(id) {
		var ti = this.data.items;
		for (var i=0;i<ti.length;i++) {
			var tId = dojo.isArray(ti[i].id) ? ti[i].id[0] : ti[i].id;
			
			if (id == tId) {
				var tCat = dojo.isArray(ti[i].category) ? ti[i].category[0] : ti[i].category;
				if (tCat == "IntermediateEvent")
					return true;
				else
					return false;
			}
		}
		return false;
	},
	
	/**
	 * Returns the gradient colour for a given colour. The gradient colour will be the colour darkened by 85%
	 * @param maxColor
	 * @param maxDepth
	 * @param depth
	 * @returns
	 */
	gradientColor: function(maxColor, maxDepth, depth) {
		
		var depth = 0.15;
		var maxDepth = 1;
		function digit2hex(digit) {return digit.toString(16);}
		function hex2digit(hex) {return parseInt(hex,16);}
		var black = "000000"; // black
		var minColor = black;
		if(depth == 0){
			return minColor;
		}
		if(depth == maxDepth){ 
			return maxColor;
		}
		 
		var color = "#";
		 
		for(var i=0; i <6; i+=2){
			var minVal = new Number(hex2digit(minColor.substr(i,2)));
			var minStr = minVal.toString();
			var maxVal = new Number(hex2digit(maxColor.substr(i,2)));	
			var maxStr = maxVal.toString();			
			var nVal = minVal + (maxVal-minVal) * (depth/maxDepth);
			var val = digit2hex(Math.floor(nVal));
			while(val.length < 2){
				val = "0"+val;
			}
			color += val;
		}
		return color;
	},
	
	/**
	 * Get the adorner dictionary for the given label
	 * @param colour - the hex colour fo the label (which will be semi transparent
	 * @param position - the side of the node to adorn (left or right)
	 * @param heightPosition - the slot position for the label
	 * @param parameterName - the name of the label
	 * @param expandTo - direction to expand the label if the node is against the edge of the viewer (left or right)
	 * @param nodeId - the id of the node
	 * @returns {dojox.collections.Dictionary}
	 */
	getAdornerInfoForName : function(colour, position, heightPosition, parameterName, expandTo, nodeId) {
		
		parameterName = this.truncateText(parameterName, 13);
		
		var fontSize = 10*this.zoomLevel;

		var aDictionary = new dojox.collections.Dictionary();

		// Right now we can support 3 heights
		var aHeightPosition = "4";

		if (heightPosition == 1)
			aHeightPosition = "26";

		if (heightPosition == 2)
			aHeightPosition = "48";
		
		if(this.isNodeMessage(nodeId))
			aHeightPosition = aHeightPosition - 22;

		aHeightPosition = aHeightPosition * this.zoomLevel;

		var convertedPosition = "{{elemBaseRight}} + " + (5 * this.zoomLevel);
		
		var inout = "outputs";
		var textPosition = "start";
		var textAnchorPoint = 10 * this.zoomLevel;

		if (position == this.visualizationModel.CONTAINER_LEFT_SIDE) {
			convertedPosition = "{{elemBaseLeft}} - " + (93 * this.zoomLevel);
			
			inout = "inputs";
			textPosition = "end";
			textAnchorPoint = 80 * this.zoomLevel;
			
			if (expandTo != "left") {
				convertedPosition = "{{elemBaseLeft}}";
				textAnchorPoint = 60 * this.zoomLevel;
			}
		}

		aDictionary.add(this.PARAMETER_ADORNER, "[{"
			+ " dojoAttachPoint:'button'," + " template:'"
			+ this.PARAMETER_HANDLE + "',"
			+ " transform: { dx: " + convertedPosition
			+ " , dy: {{elemBaseTop}} + " + aHeightPosition
			+ " }" + "}]");

		var tWidth = 100 * this.zoomLevel;
		var tHeight = 18 * this.zoomLevel;
		aDictionary.add(this.PARAMETER_HANDLE, "["
			+ "{ shape: { type: 'rect', width: " + tWidth + ", height: " + tHeight + ", r:5}, fill: " + this._getRGBForColourString(colour,false) + "}," +
			"{ shape: { type: 'text', width: " + tWidth + ", height: " + tHeight + ", x:" + textAnchorPoint + ",y: " + (13 * this.zoomLevel) + ", text: '"
			+ parameterName + "', align: '" + textPosition + "'}, font:{type:'font',weight: 'normal', size: '" + fontSize + "pt', family:'arial'}," +
			" fill: {r: 0, g: 0, b: 0, a: 1} }]");

		return aDictionary;
	},

	/**
	 * Conver the hex colour to an rgb colour and return the prebuilt string for the dojo node
	 * @param colour - hex colour
	 * @param isOpaque - whether the node is fully opaque or semi transparent
	 * @returns {String}
	 */
	_getRGBForColourString : function(colour, isOpaque) {
		// This becomes obsolete when input/output view changes
		// over
		
		var opaqueness = "0.5";
		
		if (isOpaque)
			opaqueness = "1";

		//TODO remove
//		if (colour == 'red')
//			return "{r: 214, g: 53, b: 51, a: " + opaqueness + "}";
//
//		if (colour == 'blue')
//			return "{r: 0, g: 27, b: 255, a: " + opaqueness + "}";
//
//		if (colour == 'green')
//			return "{r: 44, g: 177, b: 61, a: " + opaqueness + "}";
//
//		if (colour == 'purple')
//			return "{r: 128, g: 91, b: 213, a: " + opaqueness + "}";
		
		// Hex based colour, convert to RGB
		var dColour = dojo.colorFromHex(colour);
		var rgbColour = dColour.toRgb();
		return "{r: " + rgbColour[0] + ", g: " + rgbColour[1] + ", b:" + rgbColour[2] + ", a: " + opaqueness + "}";
		//return "'" + colour + "'";

	},
	
	// SECTION: Parameter adornment spinner
	/**
	 * Create the spinner adornment for the node if necessary
	 * @param id - the id of the node
	 * @param side - the side of the node (left or right)
	 * @param direction - the direction of the spinner (up or down)
	 */
	createSpinnersForNode: function(id, side, direction) {

		var node = this._getNodeById(this.ilogDiagrammer, id);

		var gfx = this.ilogDiagrammer.getGraphElementFromDataItem(node);
		
		var spinnerDirection = null;
		var dictionaryData = null
		if (direction == "up") {
			var spinnerDirection = this.visualizationModel.CONTAINER_SPINNER_UP;
			var dictionaryData = this.getAdornerInfoForSpinner(side,spinnerDirection);
		}
		else {
			var spinnerDirection = this.visualizationModel.CONTAINER_SPINNER_DOWN;
			var dictionaryData = this.getAdornerInfoForSpinner(side,spinnerDirection);
		}
		
		var a = this.ilogDiagrammer.createAdorner(SpinnerAdorner,dictionaryData, "SpinnerAdorner");
		a.spinnerDirection = spinnerDirection;
		a.elementId=id;
		a.side = side;
		a.setAdorned(gfx);
		this.visualizationModel.addSpinner(id, side, spinnerDirection, a);
	},
	
	/**
	 * Return the adorner dictionary for the spiner
	 * @param side - the side of the node for the spinner (left or right)
	 * @param spinnerDirection - the direction of the spinner (up or down)
	 * @returns {dojox.collections.Dictionary}
	 */
	getAdornerInfoForSpinner : function(side, spinnerDirection) {

		var aDictionary = new dojox.collections.Dictionary();

		var adornerHandleName = "SpinnerHandle";
		var adornerName = "SpinnerAdorner";

		
		var tWidth = 12 * this.zoomLevel;
		var tHeight = 8 * this.zoomLevel;
		var imageToUse = 'up.jpg';
		
		var xPosition = "{{elemBaseLeft}} - " + (9 * this.zoomLevel);
		var yPosition = "{{elemBaseTop}} - 10";
		if(spinnerDirection == this.visualizationModel.CONTAINER_SPINNER_DOWN) {
			imageToUse = 'down.jpg';
			yPosition = "{{elemBaseBottom}} + 2";
		}
		if (side == this.visualizationModel.CONTAINER_RIGHT_SIDE) {
			//xPosition = "{{elemBaseRight}} + " + (1 * this.zoomLevel);
			xPosition = "{{elemBaseRight}} - 1";
		}

		aDictionary.add(adornerName, "[{"
			+ " dojoAttachPoint:'button'," + " template:'"
			+ adornerHandleName + "',"
			+ " transform: { dx: " + xPosition
			+ " , dy: " + yPosition
			+ " }" + "}]");

		
		aDictionary.add(adornerHandleName,
				"["
				+ "{ shape: { type: 'image', width: " + tWidth + ", height: " + tHeight
				+ ", src:'" +  com.ibm.bpm.wpd.base.getEndPointAndContextRootForModule(com.ibm.bpm.wpd.base.wpdConstant.BPM_WARNAME_WEBVIEWER) + "/widgets/com/ibm/bpm/wpd/images/visualization/" + imageToUse + "' } }]");

		return aDictionary;

	},
	
	/**
	 * Spin the parameters for the given side
	 * @param id - the id of the node
	 * @param side - the side where the parameters reside (left or right)
	 * @param spinnerDirection - the direction to spin the parameters (up or down)
	 */
	_spinParameters: function(id, side, spinnerDirection) {
		var parameters = this.visualizationModel.getNextParameters(id, side,spinnerDirection);
		if (parameters == null)
			return;
		
		this.visualizationModel.clearParameterContainer(id,side);
		
		var diagram = this.ilogDiagrammer;
		var node = this._getNodeById(diagram, id);
		if (!node)
			return;
		var gfx = diagram.getGraphElementFromDataItem(node);
		
		var expandTo = "right";
		if (side == "left") {
			if (node.x[0] > 93)
				expandTo = "left";
		}

		var newVisible=[];
		
		for (var slotPosition=0;slotPosition<parameters.length;slotPosition++) {
			
			if (!parameters[slotPosition])
				continue;
			
			var type = this.visualizationModel.adornedElementsByParameterMap[parameters[slotPosition]].data.type;
			var dictionaryData = this.getAdornerInfoForIcon(this.visualizationModel.adornedElementsByParameterMap[parameters[slotPosition]].data.colour, side,slotPosition,type,id);
			var a = diagram.createAdorner(ParameterAdorner,dictionaryData, this.PARAMETER_ADORNER);
			a.parameterName = parameters[slotPosition];
			a.activityId = id;
			a.setAdorned(gfx);
			
			var slotValue = {};
			slotValue.iconAdornment = a;
			slotValue.nameAdornment=null;
			slotValue.parameterName = parameters[slotPosition];

			if (com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().getParameterStatus(parameters[slotPosition]) == com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().PARAMETER_EXPANDED) {
				
				if (!this.visualizationModel.adornedElementsByParameterMap[parameters[slotPosition]].adornedNodeLabelsExpanded)
					this.visualizationModel.adornedElementsByParameterMap[parameters[slotPosition]].adornedNodeLabelsExpanded = true;
				// Expand the label
				dictionaryData = this.getAdornerInfoForName(this.visualizationModel.adornedElementsByParameterMap[parameters[slotPosition]].data.colour,
						side,slotPosition,parameters[slotPosition],expandTo,id);
				
				a = diagram.createAdorner(ParameterAdorner,dictionaryData, this.PARAMETER_ADORNER);
				a.setAdorned(gfx);
				a.moveToBack();
				// Bidi Support
//				var paraName = this.truncateText(parameters[slotPosition], 13);
//				var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(paraName);
//				if (dir=="rtl") {
//					this.modifyAdornTextDir(a, paraName, expandTo);
//				}
				slotValue.nameAdornment=a;
			}
			
			newVisible[slotPosition] = slotValue;
		}
		
		this.visualizationModel.updateVisibleNodes(id,side,newVisible,spinnerDirection);
		this.checkSpinnerAdornment(id, side);
		
	},
	
	/**
	 * When spinning check to see which spinner (up and/or down are required)
	 * @param id the node id
	 * @param side the side being spun
	 */
	checkSpinnerAdornment: function(id, side) {
		var element = this.visualizationModel.adornedElementsByIdMap[id];
		if (side == "left") {
			var sp = element.containerLeft.spinnerPosition;
			var il = element.containerLeft.inputs.length;
			
			if(sp == 0 && element.containerLeft.spinnerUp) {
				element.containerLeft.spinnerUp.deactivate();
				element.containerLeft.spinnerUp = null;
			}
			else if(sp > 0 && !element.containerLeft.spinnerUp) {
				this.createSpinnersForNode(id, side,"up");
			}
			
			if (sp*3 + 4 > il) {
				if (element.containerLeft.spinnerDown) {
					element.containerLeft.spinnerDown.deactivate();
					element.containerLeft.spinnerDown = null;
				}
			}
			else {
				if (!element.containerLeft.spinnerDown && il>3) {
					this.createSpinnersForNode(id, side,"down");
				}
			}
		}
		else {
			var sp = element.containerRight.spinnerPosition;
			var ol = element.containerRight.outputs.length;
			
			if(sp == 0 && element.containerRight.spinnerUp) {
				element.containerRight.spinnerUp.deactivate();
				element.containerRight.spinnerUp = null;
			}
			else if(sp > 0 && !element.containerRight.spinnerUp) {
				this.createSpinnersForNode(id, side,"up");
			}
			
			if (sp*3 + 4 > ol) {
				if (element.containerRight.spinnerDown) {
					element.containerRight.spinnerDown.deactivate();
					element.containerRight.spinnerDown = null;
				}
			}
			else {
				if (!element.containerRight.spinnerDown  && ol>3) {
					this.createSpinnersForNode(id, side,"down");
				}
			}
		}
	},
	
	
	// SECTION: Zoom Diagram	
	/**
	 * Zoom has been triggered, adjust the parameter adornments to the new size
	 */
	zoomTriggered : function() {
		for ( var pn in this.visualizationModel.adornedElementsByIdMap) {
			
			this._spinParameters(pn, "left", "neutral");
			
			this._spinParameters(pn, "right", "neutral");
		}
		
		for ( var pn in this.visualizationModel.adornedTagElements) {
			var tempElement = this.visualizationModel.adornedTagElements[pn];
			if (tempElement.tags.length > 0) {
				var tn = tempElement.tags[0];
				var index = tn.indexOf("::");
				var tg = tn.substring(0,index);
				var tnn = tn.substring(index+2);
				this.collapseExpandedTags(pn, tg, tnn, false);
			}

		}
	},
	
	// SECTION: Tag adornment
	/**
	 * Adorn the node with a tage
	 * @param tagParmeter the tag name being adorned
	 * Note: it can be either a single tag object or a collection of tags (used for view refreshes)
	 */
	adornDiagramWithTag: function(tagParmeter) {
		if (dojo.isArray(tagParmeter)) {
			
			this.visualizationModel.cleanTags();
			
			for (var i=0;i<tagParmeter.length; i++) {
				this._adornDiagramWithTag(tagParmeter[i]);
			}
		}
		else {
			this._adornDiagramWithTag(tagParmeter);
		}
	},

	/**
	 * Internal function to perform the tag adornment to the diagram
	 * @param tagParmeter the tag object
	 */
	_adornDiagramWithTag: function(tagParmeter) {
	
		tagParmeter.tagName = this.makeStringSafe(tagParmeter.tagName);
		tagParmeter.groupName = this.makeStringSafe(tagParmeter.groupName);
		this.visualizationModel.addTag(tagParmeter);

		var groupName = tagParmeter.groupName;
		var tagName = tagParmeter.tagName;
	
		var nodesToAdorn = tagParmeter.nodes;
		var colour = tagParmeter.colour;
		for ( var j = 0; j < nodesToAdorn.length; j++) {
			var activityId = nodesToAdorn[j];
			this._adornTag(activityId, groupName, tagName, colour, false, true);
		}
	},

	/**
	 * Add the tag adornment to a specific node wihtin the diagram
	 * @param activityId - the id of the node
	 * @param groupName - the name of the tag group
	 * @param tagName - the name of the tage value
	 * @param colour - the colour of the tag
	 * @param expanded - whether the tags are expanded or collapsed
	 * @param tagUpDirection - the direction to expand the tags (up or down). Usually it's up unless we're at the top of the diagram and room doesn't exist
	 */
	_adornTag : function(activityId, groupName, tagName, colour, expanded, tagUpDirection) {

		// Sanity check, can't adorn a node without an id
		if (!activityId)
			return;

		if (this.visualizationModel.adornedTagElements[activityId] == null) {
			var elementTags = {};
			elementTags.tags=[];
			elementTags.expanded = false;
			this.visualizationModel.adornedTagElements[activityId] = elementTags;
		}
		
		var tagKey = groupName + "::" + tagName;
		
		var tagPosition = this.getTagPosition(activityId, tagKey);
		this.visualizationModel.adornedTagElements[activityId].tags[this.visualizationModel.adornedTagElements[activityId].tags.length] = tagKey;
		
		// Find the node to adorn.
		var diagram = this.ilogDiagrammer;
		var node = this._getNodeById(diagram, activityId);

		var gfx = diagram.getGraphElementFromDataItem(node);

		var dictionaryData = this.getAdornerForTag(colour,tagName,tagPosition, expanded, tagUpDirection);

		var a = null;
		if (expanded) {
			a = diagram.createAdorner(TagAdornerExpanded,dictionaryData, "TagAdornerExpanded");
		}
		else {
			a = diagram.createAdorner(TagAdornerCollapsed,dictionaryData, "TagAdornerCollapsed");
		}
		
		a.tagName = tagName;
		a.groupName = groupName;
		a.setAdorned(gfx);
		a.elementId=activityId;
		a.moveToBack();
		
		// Bidi Support
//		var tagDisplayName = this.truncateText(tagName, 13);
//		var dir =  com.ibm.bpm.wpd.util.BidiUtils.getDirValue(tagDisplayName);
//		if (dir=="rtl") {
//			this.modifyAdornTextDir(a, tagDisplayName, "right"); 
//		}

		this.visualizationModel.adornedElementsTagMap[tagKey].adornedNodes[this.visualizationModel.adornedElementsTagMap[tagKey].adornedNodes.length] = a;

	},
	
	/**
	 * Get the tag position for the given tag name
	 * @param activityId - the id of the node
	 * @param tagName - the name of the tag
	 * @returns
	 */
	getTagPosition: function(activityId, tagName) {

		var tTag = this.visualizationModel.adornedTagElements[activityId].tags;
		for (var i=0;i<tTag.length;i++) {
			if (tTag[i] == tagName)
				return i;
		}
		
		return tTag.length;
		
	},
	
	/**
	 * Gets the dictionary for the given tag info
	 * @param colour - the colour fo the tag
	 * @param tagName - the name of the tag
	 * @param tagPosition - the position of the tag
	 * @param expanded - whether the tage is expanded or collapsed
	 * @param tagUpDirection - the direction on the epxand
	 * @returns {dojox.collections.Dictionary}
	 */
	getAdornerForTag: function(colour,tagName, tagPosition, expanded, tagUpDirection) {
		
		tagName = this.truncateText(tagName, 13);
		
		var aDictionary = new dojox.collections.Dictionary();
		
		
		var tWidth = 80 * this.zoomLevel;
		var tHeight = 18 * this.zoomLevel;
		var vOffset = 5 * tagPosition;
		var hOffset = 5 * tagPosition;
		var spacerOffset = 0;
		var adornerHandleName = "TagHandleCollapsed";
		var adornerName = "TagAdornerCollapsed";
		
		var fontSize = 10*this.zoomLevel;
		
		
		if (expanded) {
			vOffset = (18 * tagPosition);
			hOffset = 0;
			spacerOffset = 2 * tagPosition;
			adornerHandleName = "TagHandleExpanded";
			adornerName = "TagAdornerExpanded";
		}
		
		if (tagUpDirection) {
			aDictionary.add(adornerName, "[{"
				+ " dojoAttachPoint:'button'," + " template:'"
				+ adornerHandleName + "',"
				+ " transform: { dx: {{elemBaseLeft}} + " + hOffset
				+ " , dy: {{elemBaseTop}} - " + (20 * this.zoomLevel + vOffset + spacerOffset)
				+ " }" + "}]");
		}
		else {
			aDictionary.add(adornerName, "[{"
					+ " dojoAttachPoint:'button'," + " template:'"
					+ adornerHandleName + "',"
					+ " transform: { dx: {{elemBaseLeft}} + " + hOffset
					+ " , dy: {{elemBaseTop}} + " + (20 * this.zoomLevel + vOffset + spacerOffset - 40)
					+ " }" + "}]");
		}

	aDictionary.add(adornerHandleName, "["
			+ "{ shape: { type: 'rect', width: " + tWidth + ", height: " + tHeight + "}, " +
					//"stroke: {type: 'stroke', color: 'black', width: 4}, fill: " + this._getRGBForColourString(colour, true) + "}," +
			"stroke: {type: 'stroke', color:" + this._getRGBForColourString(colour, true) + ", width: 2}, fill:{r: 255, g: 255, b:255, a: 1}}," +
			"{ shape: { type: 'text', width: " + tWidth + ", height: " + tHeight + ", x:0, y:" + (13 * this.zoomLevel) + ", text: '"
			+ tagName + "', align: 'start'}, font:{type:'font',weight: 'normal', size: '" + fontSize + "pt', family:'arial'}," + 
			" fill: {r: 0, g: 0, b: 0, a: 1} }]");

		return aDictionary;

	
	},
	
	/**
	 * Expand the collapsed tags
	 * @param elementId - the id of the node
	 */
	expandCollapsedTags: function(elementId) {
		// Find the adorners on the element.
		var tTags = this.visualizationModel.adornedTagElements[elementId].tags;
		
		var tagLength = tTags.length;
		var node = this._getNodeById(this.ilogDiagrammer, elementId);
		var expandUp = true;

		var swimlane = this._getNodeByName(this.ilogDiagrammer,node.lane[0]);
		
		var yPos = node.y[0];
		if (swimlane)
			yPos = yPos + swimlane.y[0];
		
		// Determine if we have enough room to expand up. The yPosition on the node is releative to that in it's swimlane (not the overall diagram).
		// Use the swimlane yPosition in combination with the node yPosition.
		if (yPos < tagLength * 20)
			expandUp = false;
		
		var tempTags = {};
		// Move the tags
		for (var i=0; i<tTags.length;i++) {
			
			var name=tTags[i];
			var nodes = this.visualizationModel.adornedElementsTagMap[name].adornedNodes;
			var tNodes = [];
			for (var j=0;j<nodes.length;j++) {
				var tAdorner = nodes[j];
				var tt = tAdorner.elementId;
				if (tt == elementId) {
					tempTags[name] = this.visualizationModel.adornedElementsTagMap[name].data.colour;
					tAdorner.deactivate();
				}
				else {
					tNodes[tNodes.length] = tAdorner;
				}
			}
			
			this.visualizationModel.adornedElementsTagMap[name].adornedNodes = tNodes;
		}
		
		// Reset the tags to add them
		this.visualizationModel.adornedTagElements[elementId].tags = [];
		
		// Add the new ones
		for (var tn in tempTags) {
			var index = tn.indexOf("::");
			var tg = tn.substring(0,index);
			var tnn = tn.substring(index+2);
			this._adornTag(elementId, tg, tnn, tempTags[tn], true, expandUp);
		}
		
		
	},
	
	/**
	 * Collapse expanded tags. Bring to front all existing tags on nodes elsewhere
	 * @param elementId - the id of the node
	 * @param groupName - the group name that was selected
	 * @param tagName - the tag name that was selected
	 * @param scramble - move tags on other nodes as well
	 */
	collapseExpandedTags: function(elementId, groupName, tagName, scramble) {
		// Find the adorners on the element.
		var tTags = this.visualizationModel.adornedTagElements[elementId].tags;
		
		var tagKey = groupName + "::" + tagName;
		
		var tempTags = [];
		var elementIds = {};

		// Move the tags
		for (var i=0; i<tTags.length;i++) {
			
			var name=tTags[i];
			
			if (i==0 && name==tagKey)
				scramble = false;
			
			var nodes = this.visualizationModel.adornedElementsTagMap[name].adornedNodes;
			var tNodes = [];
			for (var j=0;j<nodes.length;j++) {
				var tAdorner = nodes[j];
				var tt = tAdorner.elementId;
				if (tt == elementId) {
					var tempTag = {};
					tempTag.name = name;
					tempTag.colour = this.visualizationModel.adornedElementsTagMap[name].data.colour;
					
					// Need to regig the order
					if (name == tagKey)
						tempTags[0] = tempTag;
					else {
						if (tempTags.length == 0)
							tempTags[tempTags.length +1] = tempTag;
						else
							tempTags[tempTags.length] = tempTag;
					}
					
					tAdorner.deactivate();
				}
				else {
					if (name == tagKey)
						elementIds[tt] = "a";
					tNodes[tNodes.length] = tAdorner;
				}
			}
			this.visualizationModel.adornedElementsTagMap[name].adornedNodes = tNodes;
		}
		
		// Reset the tags to add them
		this.visualizationModel.adornedTagElements[elementId].tags = [];
		
		// Add the new ones
		for (var i = 0; i < tempTags.length; i++) {
			var index = tempTags[i].name.indexOf("::");
			var tg = tempTags[i].name.substring(0,index);
			var tnn = tempTags[i].name.substring(index+2);
			this._adornTag(elementId, tg,tnn, tempTags[i].colour, false, true);
		}
		
		if (scramble) {
			for (var elemId in elementIds)
				this.collapseExpandedTags(elemId, groupName, tagName, false);
		}
		
	},

	/**
	 * Remove the tag name from the nodes
	 * @param tagParameter
	 */
	removeTagAdornment : function(tagParameter) {
		
		tagParameter.tagName = this.makeStringSafe(tagParameter.tagName);
		tagParameter.groupName = this.makeStringSafe(tagParameter.groupName);
		
		var nodesToRedraw = this.visualizationModel.removeTag(tagParameter);
		for (var key in nodesToRedraw) {
			var groupName = nodesToRedraw[key];
			var index = groupName.indexOf("::");
			var tagName = groupName.substring(index+2);
			groupName = groupName.substring(0,index);
			this.collapseExpandedTags(key, groupName, tagName, false);
		}
	},
	
	/**
	 * Return the diagram node for the given id
	 * @param diagram
	 * @param id
	 * @returns
	 */
	_getNodeById : function(diagram, id) {

		// Get the list of links available as an
		// ItemFileReadStore
		var nodes = diagram.nodesStore;

		// Query the store for a given link
		var node;
		nodes.fetchItemByIdentity({
			identity : id,
			onItem : function(item, request) {
				node = item;
			}
		});

		return node;

	},
	
	_getNodeByName: function(diagram, name) {
		var nodes = diagram.nodesStore._arrayOfAllItems;
		for(var i=0;i<nodes.length;i++) {
			
			/* Perfrom a check to see if the label actually exists. It appears that if you add a phase, it'll add a node to the
			nodestore without a label. However, the order in which you add the phases in importantant to get this to fail (as it
			might find what it's looking for before it hits it*/
			if (nodes[i].label && nodes[i].label[0] == name)
				return nodes[i];
		}
		return null;
	},
	
	/**
	 * Call with toggling between the tags and parameters in the navigator. Clean up any elements as necessary
	 * @param type
	 */
	toggleAdornmentType: function(type) {
		if (type == "parameter")
			this.visualizationModel.cleanTags();
		else
			this.visualizationModel.cleanParameters();
	},

	// SECTION: Subclassed functions
	
	/**
	 * Superclass function assumes node is create via a datastore, doens't work for programtically created nodes. If the item is set call the
	 * superclass funection (as that was created via a datastore, otherwise it's a dynamic markup
	 */
	createNode : function(item, template, graph, diagram) {
		if (item)
			return this.inherited(arguments);
	},

	/**
	 * Superclass function assumes node is create via a datastore, doens't work for programtically created nodes. If the item is set call the
	 * superclass funection (as that was created via a datastore, otherwise it's a dynamic markup
	 */
	onNodeCreated : function(node) {
		if (node._data)
			this.inherited(arguments);
	},
	
	/**
	 * Subclass to only allow drill down on subprocesses. We don't want to drill into non subprocesses
	 * @param e
	 */
	drillDown: function(e) {
		
		if ( this._data )		
			var data = this._data.item;
		else{
			//Each transparent <div> keeps its originating item in the name field
			var id = e.target.getAttribute( 'name' );
			var data = this.IDtoItemMap.item( id );
		}
		
		if (data.poType!="BPD" || data.bpmn2TaskType == "CalledProcess")
			return;
		
		var params = {};
		
		if ( data.poId )
	    	params.documentId = dojo.isArray(data.poId) ? data.poId[0] : data.poId;
	    	
	    if ( data.label )
		    params.label = dojo.isArray(data.label) ? data.label[0] : data.label;
	
	    if ( data.snapshotId )
	    	params.snapshotId = dojo.isArray(data.snapshotId) ? data.snapshotId[0] : data.snapshotId;
	    
	    if ( data.branchId )
	    	params.branchId = dojo.isArray(data.branchId) ? data.branchId[0] : data.branchId;
	    
	    if (data.bpmn2TaskType) {
	    	if (bpmnTaskType == 'CalledProcess')
	    		return;
	    	
    		var bpmnTaskType = dojo.isArray(data.bpmn2TaskType) ? data.bpmn2TaskType[0] : data.bpmn2TaskType;
    		if (bpmnTaskType == 'SubProcess' || bpmnTaskType == 'Event_SubProcess') {
	    		params.subProcParentId = dojo.isArray(data.id) ? data.id[0] : data.id;
	    	}
    	}
	   
	    dojo.publish("com.ibm.bpm.wpd.visualization.VisualizationViewer::openProcess",[ params ]);
	},
	
	iDtoItemMap: function(id) {
		for (var i=0;i<this.data.items.length;i++) {
			var tid = this.data.items[i].id;
			tid = dojo.isArray(tid) ? tid[0] : tid;
			if (tid == id)
				return this.data.items[i];
		}
	},
	
	/**
	 * Return the node types that we can drill into
	 * @param implType
	 * @returns {Boolean}
	 */
	drillableType: function(implType) {
		  return( implType == 'SubProcess' || implType == 'Event_SubProcess');
	},
	
	_getEditorParams: function(store, linkStore, overviewEnabled) {
		var params =
		{
            nodesStore: store,
            nodesQuery:{id:'*'},
            nodesQueryOptions: {
                deep: true
            },
			childBinding: "children",
			successorsBinding: "successors",
			xBinding: "x",
			yBinding: "y",
			createLinksForHierarchy: false,
			createSubgraphFunction: dojo.hitch( this, 'createSwimLane' ),
			createNodeFunction: dojo.hitch( this, 'createNode' ),
			
			linkLayoutActive: false,
			linksStore:linkStore,
			linksQuery:{id:'*'},
//            linksGraphBinding: "gfx",
//            linksGraphProperties: "intermediatePoints",
            startNodeBinding: "start",
            endNodeBinding: "end",
            selectionMode: ibm_ilog.diagram.Selection.dataStore_multi,
  			nodeMenu: null,
				
			linkStyle: {shapeType: ibm_ilog.diagram.LinkShapeType.OrthogonalEditable, strokeWidth:1, strokeColor: 'gray' },
			onLinkCreated: dojo.hitch( this, 'onLinkCreated' ),
			onNodeCreated: dojo.hitch( this, 'onNodeCreated' ),
			
			linkTemplateFunction: dojo.hitch( this, "getLinkTemplate" ),
			
			nodeTemplateFunction: dojo.hitch( this, 'getNodeTemplate' ),
			onLoaded: function() { 
				ibm_ilog.diagram.util.Batch.endBatch();
				try{  
					//BPM Social 'Coach Toolbar Preview' requires it's owning window to be notified of document loading
					dojo.publish("com.ibm.bpm.social.Participants.loaded");
				} catch (e) {}
			},
			overview : overviewEnabled,
			move: false
		};
		
		return params;
	},
	
	/**
	 * Key has been pressed
	 * @param e
	 */
	_keyPressed: function(e) {
		
		var keyPressedValue = e.which ? e.which : e.keyCode;
		
		//Nothing has focus
		if (!this.ilogDiagrammer.getFocusedElement())
			return;
		
		var data = this.ilogDiagrammer.getFocusedElement()._data;
		if (!data)
			return;
		
		data=data.item.id[0];
		
		// Node doesn't have any adorners
		if (!this.visualizationModel.adornedElementsByIdMap[data] && !this.visualizationModel.adornedTagElements[data] && keyPressedValue!="110")
			return;
		
		// first determine what is pressed
		if (this.keyStroke == null) {
			// We're looking for l or r
			if (keyPressedValue == "108" && this.visualizationModel.adornedElementsByIdMap[data].containerLeft.inputs)
				this.keyStroke = "l";
			else if (keyPressedValue == "114" && this.visualizationModel.adornedElementsByIdMap[data].containerRight.outputs)
				this.keyStroke = "r";
			else if (keyPressedValue == "101" && this.visualizationModel.adornedTagElements[data].tags) {
				this.keyStroke = "e";
				this.expandCollapsedTags(data);
			}
			else if (keyPressedValue == "110")
				this.keyStroke = "n";
		}
		else {
			if (this.keyStroke == "l") {
				if (keyPressedValue == "117") {
					// Move left side up
					this._spinParameters(data, "left", "up");
				}
				else if (keyPressedValue == "100") {
					// Move left side down
					this._spinParameters(data, "left", "down");
				}
				else if (keyPressedValue == "49") {

					// Expand/collapse left slot1
					var ad = this.visualizationModel.adornedElementsByIdMap[data].containerLeft.visibleInputs[0];
					if (ad) {
						this._adornWithLabel(ad.parameterName);
					}
				}
				else if (keyPressedValue == "50") {
					// Expand/collapse left slot2
					var ad = this.visualizationModel.adornedElementsByIdMap[data].containerLeft.visibleInputs[1];
					if (ad) {
						this._adornWithLabel(ad.parameterName);
					}
				}
				else if (keyPressedValue == "51") {
					// Expand/collapse left slot3
					var ad = this.visualizationModel.adornedElementsByIdMap[data].containerLeft.visibleInputs[2];
					if (ad) {
						this._adornWithLabel(ad.parameterName);
					}
				}
			}
			else if (this.keyStroke == "r") {
					
				if (keyPressedValue == "117") {
					// Move left side up
					this._spinParameters(data, "right", "up");
				}
				else if (keyPressedValue == "100") {
					// Move left side down
					this._spinParameters(data, "right", "down");
				}
				else if (keyPressedValue == "49") {
					// Expand/collapse left slot1
					var ad = this.visualizationModel.adornedElementsByIdMap[data].containerRight.visibleOutputs[0];
					if (ad) {
						this._adornWithLabel(ad.parameterName);
					}
				}
				else if (keyPressedValue == "50") {
					// Expand/collapse left slot2
					var ad = this.visualizationModel.adornedElementsByIdMap[data].containerRight.visibleOutputs[1];
					if (ad) {
						this._adornWithLabel(ad.parameterName);
					}
				}
				else if (keyPressedValue == "51") {
					// Expand/collapse left slot3
					var ad = this.visualizationModel.adornedElementsByIdMap[data].containerRight.visibleOutputs[2];
					if (ad) {
						this._adornWithLabel(ad.parameterName);
					}
				}
			}
			else if (this.keyStroke == "e") {
				if (keyPressedValue > 48 && keyPressedValue < 59) {
					var slot = keyPressedValue - 49;
					var value = "";
					// We have one in the slot
					if (this.visualizationModel.adornedTagElements[data].tags[slot])
						value = this.visualizationModel.adornedTagElements[data].tags[slot];
					else 
						value = this.visualizationModel.adornedTagElements[data].tags[0];
					
					var index = value.indexOf("::");
					this.collapseExpandedTags(data, value.substring(0,index), value.substring(index+2), true);
				}
				
			}
			else if (this.keyStroke  == "n" && keyPressedValue == "116")
				this._toggleNavigationStyle();

			this.keyStroke = null;
		}
	},
	
	
	// SECTION: widget initialization
	
	/**
	 * Enable viewer to listen to the label expand event
	 */
	addListener: function() {
		var localVar = this;
		this.listener = dojo.subscribe("bpmnviewerlite::expandlabel", function(notification) {
			localVar._adornWithLabel(notification);
		});
		
		this.listener1 = dojo.subscribe("bpmnviewerlite::spinparameters", function(notification) {
			localVar._spinParameters(notification.elementId,notification.side,notification.spinnerDirection);
		});
		
		this.listener2 = dojo.subscribe("bpmnviewerlite::expandtags", function(notification) {

			if (notification.expand)
				localVar.expandCollapsedTags(notification.elementId);
			else
				localVar.collapseExpandedTags(notification.elementId, notification.groupName, notification.tagName, true);
		});
		
	},
	
	/**
	 * Disable the viewer from listening to the label expand event
	 */
	removeListener: function() {
		dojo.unsubscribe(this.listener);
		dojo.unsubscribe(this.listener1);
		dojo.unsubscribe(this.listener2);
	},
	
	/**
	 * Create the adorners that we will need to the visualation
	 * of the diagram
	 */
	createAdorner : function() {

		var myVariable = this;

		// Parameter adornment
		ibm_ilog.diagram.gfxDeclaration(dojo.declare(this.PARAMETER_HANDLE,ibm_ilog.diagram.adorners.AdornerHandle, {
			activate : function() {
				this.clickHandle = this.connect("onclick",this._adorner,"clickmsg");
			},

			deactivate : function() {
				this.disconnect(this.clickHandle);
				try {					
					if (this.children.length>0)
						this.remove(this.children[0]);
				} catch (err) {
					console.log('Error clearing marker: ' + err.message);
				}
			}

		}));
		
		ibm_ilog.diagram.gfxDeclaration(dojo.declare(this.PARAMETER_ADORNER,ibm_ilog.diagram.adorners.AdornerWithHandles, {

			initialize : function() {
				this.inherited(arguments);
				this._createHandle(ParameterHandle, "button");
			},
		
			clickmsg : function() {
				// Fire a label expand event. Which we ourselves will listen to. We're doing this
				// as we can't call the function in this class directly. The function that is called
				// is the instance of the class when this adorner is registered. If a new instance of
				// the class is create (i.e. drilldown) the subprocess expand will end up being called
				// on the main processes instance.
				debugger;
				dojo.publish("bpmnviewerlite::expandlabel",[ this.parameterName ]);
			},
		
			deactivate : function() {
				this.inherited(arguments);
				this._deactivateHandle("button");
			}
		
		}));
		
		// Spinner adornment
		ibm_ilog.diagram.gfxDeclaration(dojo.declare('SpinnerHandle',ibm_ilog.diagram.adorners.AdornerHandle, {
			activate : function() {
				this.clickHandle = this.connect("onclick",this._adorner,"clickmsg");
			},

			deactivate : function() {
				this.disconnect(this.clickHandle);
				try {
					this.remove(this.children[0]);
				} catch (err) {
					console.log('Error clearing marker: ' + err.message);
				}
			}

		}));
		
		ibm_ilog.diagram.gfxDeclaration(dojo.declare('SpinnerAdorner',ibm_ilog.diagram.adorners.AdornerWithHandles, {
			initialize : function() {
				this.inherited(arguments);
				this._createHandle(SpinnerHandle, "button");
			},
		
			clickmsg : function() {
				//myVariable._spinParameters(this.elementId, this.side, this.spinnerDirection);
				params = {};
				params.elementId = this.elementId;
				params.side = this.side;
				params.spinnerDirection = this.spinnerDirection;
				
				dojo.publish("bpmnviewerlite::spinparameters",[ params]);
			},
		
			deactivate : function() {
				this.inherited(arguments);
				this._deactivateHandle("button");
			}
		
		}));
		
		
		// Tag adornment
		ibm_ilog.diagram.gfxDeclaration(dojo.declare('TagHandleCollapsed',ibm_ilog.diagram.adorners.AdornerHandle, {
			activate : function() {
				//this.expandTags = this.connect("onmouseover",this._adorner,"expandTags");
				this.expandTags = this.connect("onclick",this._adorner,"expandTags");
			},

			deactivate : function() {
				this.disconnect(this.expandTags);
				try {
					this.remove(this.children[0]);
				} catch (err) {
					console.log('Error clearing marker: ' + err.message);
				}
			}

		}));
		
		ibm_ilog.diagram.gfxDeclaration(dojo.declare('TagHandleExpanded',ibm_ilog.diagram.adorners.AdornerHandle, {
			activate : function() {
				this.clickHandle = this.connect("onclick",this._adorner,"collapseTags");
			},

			deactivate : function() {
				this.disconnect(this.clickHandle);
				try {
					this.remove(this.children[0]);
				} catch (err) {
					console.log('Error clearing marker: ' + err.message);
				}
			}

		}));
		
		ibm_ilog.diagram.gfxDeclaration(dojo.declare('TagAdornerCollapsed',ibm_ilog.diagram.adorners.AdornerWithHandles, {

			initialize : function() {
				this.inherited(arguments);
				this._createHandle(TagHandleCollapsed, "button");
			},
		
			expandTags : function() {
				var params = {};
				params.elementId = this.elementId;
				params.expand = true;
				dojo.publish("bpmnviewerlite::expandtags",[ params]);
				//myVariable.expandCollapsedTags(this.elementId);
			},
		
			deactivate : function() {
				this.inherited(arguments);
				this._deactivateHandle("button");
			}
		}));
		
		ibm_ilog.diagram.gfxDeclaration(dojo.declare('TagAdornerExpanded',ibm_ilog.diagram.adorners.AdornerWithHandles, {

			initialize : function() {
				this.inherited(arguments);
				this._createHandle(TagHandleExpanded, "button");
			},
		
			collapseTags : function() {
				var params = {};
				params.elementId = this.elementId;
				params.groupName = this.groupName;
				params.tagName = this.tagName;
				params.expand = false;
				dojo.publish("bpmnviewerlite::expandtags",[ params]);
				//myVariable.collapseExpandedTags(this.elementId, this.groupName, this.tagName, true);
			},
		
			deactivate : function() {
				this.inherited(arguments);
				this._deactivateHandle("button");
			}
		}));

	},
	

	// SECTION: General widget functions
	
	makeStringSafe: function(value) {
		value = value.replace(/\\/g, '\\\\').replace(/\n/g, '\\n');
		value = value.replace(/'/g, "\\'");
		
		if (value.charAt(0) == '"' && value.slice(-1) == '"') {
			value = '"' + value.slice(1, -1).replace(/"/g, '\\"') + '"';
		} else if (value.charAt(0) == "'" && value.slice(-1) == "'") {
			value = "'" + value.slice(1, -1).replace(/'/g, "\\'") + "'";
		}
		 return value;
	},
	
	truncateText: function(value, limit) {
		if (this.isDoubleByte(value))
			limit = Math.floor(limit/2);
		
		if (value.length < limit)
			return value;
		
		return value.substring(0, limit-3) + "...";
	},
	
	isDoubleByte: function(str) {
		for (var i = 0, n = str.length; i < n; i++) {
	        if (str.charCodeAt( i ) > 255) {
	        	return true; 
	        }
	    }
	    return false;
	},
	
	/**
	 * Support resize on the viewer to its children widgets
	 * Currently Chrome has problem resizing and redraw the diagram.  So we skip Chrome for now.
	 */
	resize: function() {
		//if (!dojo.isChrome) {
			if (this.mainBC) {
				this.mainBC.resize();
				if (this.ilogDiagrammer) {
					this.ilogDiagrammer.resize();
				}
			}
		//}
	},
	
	_toggleNavigationStyle: function() {
		if (this.navigationStyle == 'graph') {
			this.ilogDiagrammer._keyInteractors.get(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId).setNavigationStyle(ibm_ilog.diagram.interactors.Navigator.TreeStyle);
			this.navigationStyle = 'tree';
		}
		else {
			this.ilogDiagrammer._keyInteractors.get(ibm_ilog.diagram.interactors.Navigator.KeyInteractorId).setNavigationStyle(ibm_ilog.diagram.interactors.Navigator.GraphStyle);
			this.navigationStyle = 'graph';
		}
	},
	
	
	/**
	 * Bidi Support
	 * Modify the text direction given a node.
	 * @param node
	 */
	modifyAdornTextDir: function(node, value, activityPosition) {
		if (node.rawNode) {
			for (var i=0; i<node.rawNode.childNodes.length; i++) {
				if (node.rawNode.childNodes[i]) {
					var gNode = node.rawNode.childNodes[i];
					if ("g"==gNode.tagName) {
						for (var j=0; j<gNode.childNodes.length; j++) {
							var g2Node = gNode.childNodes[j];
							if ("g"==g2Node.tagName) {
								for (var k=0; k<g2Node.childNodes.length; k++) {
									var textNode = g2Node.childNodes[k];
									if ("text"==textNode.tagName) {
										// find the text node
										var isIERTL = false;
										if (dojo.isIE) {
											isIERTL = com.ibm.bpm.wpd.util.BidiUtils.hasNonLatinChar(value);
											if (isIERTL) // only set dir if it is useful
												textNode.setAttribute("dir", "rtl");
										}
										else
											textNode.setAttribute("direction", "rtl");
										if (dojo.isChrome || dojo.isSafari || isIERTL) {
											if (activityPosition=="left")
												textNode.setAttribute("text-anchor", "start");
											else
											if (activityPosition=="right")
												textNode.setAttribute("text-anchor", "end");
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
});