dojo.provide("com.ibm.bpm.wpd.visualization.VisualizationViewer");


dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.bpm.wpd.core.model.shellstate.ShellStateModel");
dojo.require("com.ibm.bpm.wpd.core.control.WpdRestHandler");
dojo.require("com.ibm.bpm.wpd.core.model.viewstate.ViewStateModel");
dojo.require("com.ibm.bpm.wpd.document.bpd.view.BPMNViewer");
dojo.require("com.ibm.bpm.wpd.visualization.BPMNViewerLite");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojox.layout.ExpandoPane");
dojo.require("com.ibm.bpm.wpd.visualization.NavigationPane");
dojo.require("dijit.layout.StackContainer");
dojo.require("com.ibm.bpm.wpd.core.view.ViewerContentPane");
dojo.require("com.ibm.bpm.wpd.core.breadcrumb.view.WebViewerBreadcrumb");
dojo.require("com.ibm.bpm.wpd.visualization.model.SessionManagement");
dojo.require("com.ibm.bpm.wpd.base.errorhandling.WpdErrorDialog");

dojo.declare("com.ibm.bpm.wpd.visualization.VisualizationViewer", [ dijit._Widget, dijit._Templated ], {
		
		templatePath : dojo.moduleUrl("com.ibm.bpm.wpd.visualization","templates/VisualizationViewer.html"),
		
		widgetsInTemplate : true,
		
		
		documentId : null,
		documentName : null, 
		assetType : null,
		branchId : null,
		snapshotId : null,
		
		_subscriptionHandles : null,
		
		_currentViewer:null,
		
		_openedViews:null,
		
		breadcrumb : null,
		breadcrumbList:null,
		navigator: null,
		
		drillDownInProgress: false,
					
		constructor : function (params) {
			this.documentId = params.documentId;
			//this.documentName=params.documentName;
			this.assetType=params.assetType;
			this.branchId=params.branchId;
			this.snapshotId=params.snapshotId;
			this._subscriptionHandles = [];
			this._openedViews = [];
			breadcrumbList = [];
				
		},
		
		postCreate : function() {
			
			this._querySnapshotId();
			
			// Move up top as I could be triggering breadcrumb events on a reload before it's drawn
			breadcrumb = new com.ibm.bpm.wpd.core.breadcrumb.view.WebViewerBreadcrumb({className:"viewerBreadcrumbBar"}, this.breadcrumbContainer);
			this._setupListeners();

			var bc = com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().getBreadcrumb();
			
			// if bc is set we have refresh situation. If the length is 1 we know we haven't drilled down so nothing is require
			// if it's greater than 1 we've drilled down. In this case build the breadcrumb up to the last element and them build it.
			if (bc.length > 1) {
				
				// Build the breadcrumb.
				for (var i=0;i<bc.length - 1;i++) {
					var entry = bc[i];
					var separatorIndex = entry.indexOf(":");
					this._fireUpdateBreadcrumbEvent(entry.substring(0,separatorIndex),entry.substring(separatorIndex + 1,entry.length), this.snapshotId, this.branchId);
				}
				
				var entry = bc[bc.length - 1];
				var separatorIndex = entry.indexOf(":");
				// On the last one load the diagram
				this.loadProcessDiagram(entry.substring(0,separatorIndex), this.snapshotId, entry.substring(separatorIndex + 1,entry.length),this.branchId, true, true);
			}
			else
				this.loadProcessDiagram(this.documentId, this.snapshotId, null,this.branchId, true, true);
			
			this.visBorderContainer.startup();
		},
		
		/**
		 * When browsing the branch tip, used to query the latest tip snapshot. This snapshot id will be used to keep the pieces in sync (navigator
		 * subprocess navigation). While we can include the tip snapshot id as part of the url, we do need to requery it on a refresh in the event
		 * the user made changes to the process they would like to see. Only a browser refresh will pick up the new changes
		 * NOTE: This is a synchornous query (as opposed to the others which are asynchronous) 
		 */
		_querySnapshotId: function() {
			if (!this.snapshotId || this.snapshotId == null || this.snapshotId == "null") {
				com.ibm.bpm.wpd.core.control.WpdRestHandler.getInstance().performRequest("get", 
					"/v1/tipsnapshotid/" + this.branchId, dojo.hitch(this, function(response) {
						this.snapshotId = response.data.snapshotId;
					}), dojo.hitch(this, function(response) {
						com.ibm.bpm.wpd.base.errorhandling.WpdErrorDialog.showError(response);
					}),null, true);
			}
		},
		
		_setupListeners: function() {
			var localVar = this;
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::adornParameter", function(notification) {
				// Looking for the format
				/*{"name":"My first input",
				 * "colour":"red",
				 * "type":"a type",   either input, output or private
				 * "nodes":[{"id":"bpdid:5cd499bd441308ca:37d49876:12e27ee26e3:-67db","position":"left"},
				 * {"id":"bpdid:5cd499bd441308ca:37d49876:12e27ee26e3:-67c6","position":"left"},
				 * {"id":"bpdid:5cd499bd441308ca:37d49876:12e27ee26e3:-6796","position":"left"}]}*/
				// Do the current viewer first as it's the visable one
				localVar._currentViewer.adornDigramWithParameter(notification);
			}));
			
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::removeAdornedParameter", function(notification) {
				// Looking for the format
				/*{"name":"My first input"}*/
				if (!localVar.drillDownInProgress) {
					localVar._currentViewer.removeParameterAdornment(notification);
				}
			}));
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::adornTag", function(notification) {
				localVar._currentViewer.adornDiagramWithTag(notification);	
			}));
			
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::removeAdornedTag", function(notification) {
				localVar._currentViewer.removeTagAdornment(notification);
			}));
			
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::openProcess", function(notification) {
				localVar.loadProcessDiagram(notification.documentId, notification.snapshotId, notification.label, notification.branchId, false, true);
			}));
			
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::addBreadcrumbEntry", this, this.modelChanged));
			
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::breadcrumbEntryClicked", this, function(notification) {
				localVar.switchToProcessDiagram(notification.documentId);
			}));
			this._subscriptionHandles.push(dojo.subscribe("com.ibm.bpm.wpd.visualization.VisualizationViewer::toggleViewer", this, function(notification) {
				localVar._currentViewer.toggleAdornmentType(notification);
			}));
		},
		
		modelChanged: function(notification){		
			 var iconPath = "images/process-app-16.png";	
			 viewModel = {name : notification.label, type:"app", id: notification.documentId};			 
			 breadcrumb.addBreadcrumbEntry(viewModel, this._handleBreadcrumbClickEvent, true, iconPath, this._handleBreadcrumbKeyboardClickEvent);
			 breadcrumbList[breadcrumbList.length] = viewModel; 	 
		},
		
		_handleBreadcrumbKeyboardClickEvent:function(event){
			if (event.charOrCode == dojo.keys.ENTER){
				var params = {};
				var div = event.currentTarget;
				div = div.parentNode;
				var model = breadcrumb._divToModel[div.id];
				params.documentId = model.id;
				var elementsToRemove = 0;
			
				for ( var i = 0; i < breadcrumbList.length-1; i++ ) {
				 	if ( breadcrumbList[i].id == params.documentId ) {
			 			for ( var j=i+1; j < breadcrumbList.length; j++) {	
			 				var viewModelToRemove = {name : breadcrumbList[j].name, type: breadcrumbList[j].type, id: breadcrumbList[j].id};		 		
			 				breadcrumb.removeBreadcrumbEntry(viewModelToRemove);
			 				elementsToRemove++;
			 			}
			 			breadcrumbList.splice(i+1,elementsToRemove);
			 			break;
			 		}
				 }
				breadcrumb._setStyleForSelectedBreadcrumb(model);
				dojo.publish("com.ibm.bpm.wpd.visualization.VisualizationViewer::breadcrumbEntryClicked",[ params ]);
			}
		},		
		
		_handleBreadcrumbClickEvent:function(event){
			var params = {};

			var div = event.currentTarget;
			div = div.parentNode;
			var model = breadcrumb._divToModel[div.id];
			params.documentId = model.id;
			var elementsToRemove = 0;
			
			for ( var i = 0; i < breadcrumbList.length-1; i++ ) {
			 	if ( breadcrumbList[i].id == params.documentId ) {
			 		for ( var j=i+1; j < breadcrumbList.length; j++) {	
			 			var viewModelToRemove = {name : breadcrumbList[j].name, type: breadcrumbList[j].type, id: breadcrumbList[j].id};		 		
			 			breadcrumb.removeBreadcrumbEntry(viewModelToRemove);
			 			elementsToRemove++;
			 		}
			 		breadcrumbList.splice(i+1,elementsToRemove);
			 		break;
			 	}
			 }
			breadcrumb._setStyleForSelectedBreadcrumb(model);
			dojo.publish("com.ibm.bpm.wpd.visualization.VisualizationViewer::breadcrumbEntryClicked",[ params ]);
		},		
		
		/**
		 * Unsubscrible the event handles and clean up the viewer
		 */
		cleanup:function(){	
			for ( var i = 0; i < this._subscriptionHandles.length; i++) {
				if(!!this._subscriptionHandles[i]){
					dojo.unsubscribe(this._subscriptionHandles[i]);
				}
			}
			
			// Should clean up?
			_currentViewer = null;
		},
		
		postMixInProperties: function()
		{
	        dojo.requireLocalization("com.ibm.bpm.wpd.visualization", "VisualizationViewerResources");
			this.messages = dojo.i18n.getLocalization("com.ibm.bpm.wpd.visualization", "VisualizationViewerResources");
		},
		
		/**
		 * Load the base process diagram used by the process viewer. Currently map a direct REST call
		 */
		loadProcessDiagram: function(_documentId, _snapshotId, _name, _branchId, triggerNavigationLoad, addbreadcrumb) {
			
			// Would like to do a sync request so I don't have to put the navigation pane here, but for some reason the process view remains hidden if I do
			com.ibm.bpm.wpd.core.control.WpdRestHandler.getInstance().performRequest("get", 
					"/v1/visual/processModel/" + _documentId + "?snapshotId=" + _snapshotId, dojo.hitch(this, function(response) {
						this.addToProcessViewStack(response, _documentId, _name, _snapshotId, _branchId, addbreadcrumb);
						if (triggerNavigationLoad) {
							// Trigger the viewer to create the adorners. We only really need to do this once.
							// Move from the diaplayyAll of the bpmn viewer to here
							this._currentViewer.createAdorner();
							
							var navObj = {documentId: _documentId, branchId:_branchId, snapshotId:_snapshotId};
							this.navigator = new com.ibm.bpm.wpd.visualization.NavigationPane(navObj, this.navigation);
						}
						else {
							this._updateNavigator(_documentId);
						}
					}), dojo.hitch(this, function(response) {
						com.ibm.bpm.wpd.base.errorhandling.WpdErrorDialog.showError(response);
						
					}));
			
		},
		
		switchToProcessDiagram: function(documentId) {
			var tempOpenedView = [];
			var viewFound = false;
			for (var i=0; i< this._openedViews.length;i++) {
				if (this._openedViews[i].viewStateModel.documentId == documentId) {
					viewFound=true;
					tempOpenedView[tempOpenedView.length] = this._openedViews[i];
					
					// Disable the event listeners form the viewer we're leaving
					if (this._currentViewer)
						this._currentViewer.removeListener();
					
					this._currentViewer=this._openedViews[i].viewer;
					
					// Enable the event listeners to the viewer we're entering
					this._currentViewer.addListener();
					// Open this vsm
					this.processViewer.selectChild(this._openedViews[i]);
					
				}
				else {
					if (viewFound) {
						// close the remainder
						this._openedViews[i].viewer.destroy();
					}
					else {
						//Add back to temp collection
						tempOpenedView[tempOpenedView.length] = this._openedViews[i];
					}
					
				}
			}
			
			// We have a reload and it hasn't been loaded
			if (!viewFound) {
				this.loadProcessDiagram(documentId, this.snapshotId, "", this.branchId, false, false);
				
				//destroy the old views
				for (var i=0; i< tempOpenedView.length;i++) {
					tempOpenedView[i].viewer.destroy();
				}
			}
			else {
				this._openedViews = tempOpenedView;
			}
			
			// Clean the breadcrumb cache
			com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().removeFromBreadcrumb(documentId);
			this._updateNavigator(documentId);
		},
		
		_updateNavigator: function(documentId) {
			
			// Set the drill down, breadcrumb navigation is in process. If it is we don't want to
			// trigger the remove event from the session as that will wipe out whether nodes are
			// expanded or not.
			this.drillDownInProgress = true;
			this.navigator.updatePane(documentId,this.snapshotId);
			this.drillDownInProgress = false;
		}, 
		
		/**
		 * First event to update the breadcrumb
		 * @param documentId - the unique id of the process or sub activity
		 * @param label - the name of the process or sub activity
		 * @param snapshotId - the snapshot of the current loaded diagram
		 * @param branchId - the branch of the current laoded diagram
		 */
		_fireUpdateBreadcrumbEvent: function(documentId, label,snapshotId, branchId) {
			var params = {};
			
			if (documentId)
		    	params.documentId = documentId;
		    	
		    if (label)
			    params.label = label;
		
		    if (snapshotId)
		    	params.snapshotId = snapshotId;
		    
		    if (branchId)
		    	params.branchId = branchId;
		    
		    dojo.publish("com.ibm.bpm.wpd.visualization.VisualizationViewer::addBreadcrumbEntry",[ params ]);
		},
		
		/**
		 * Create the process viewer based on the results from the REST call. Add the proces viewer to a stacked pane and collection of viewers
		 * This will allow easier navigation with the breadcrumb
		 * @param response
		 */
		addToProcessViewStack : function (response,pdocumentId, plabel, psnapshotId, pbranchId, addbreadcrumb){
			var documentObj;		
			documentObj = response.data;	

			if (plabel == null)
				plabel=documentObj.name;
			// Create the visual model for the process doc
			var vsmObj = {documentName: this.documentName, documentType:"BPDDocType", documentId:this.documentId, document:documentObj};
			vsmObj.branchOrSnapshotId = this.snapshotId;
			var vsm = new com.ibm.bpm.wpd.core.model.viewstate.ViewStateModel();
			vsm.name = plabel;
			vsm.documentId = pdocumentId;
			vsm.type = "BPDDocType";
			vsm.branchOrSnapshotId = psnapshotId;
			vsm.setDocumentModel(documentObj);
			
			
			// Add the visual to a stacked container
			var viewerCp = new com.ibm.bpm.wpd.core.view.ViewerContentPane({ parseOnLoad: false, style: "height:98%;width:900px;"});
			var viewerAttachPoint = document.createElement("div");
			viewerAttachPoint.style.height = "100%";
			viewerAttachPoint.style.width = "100%";
			if ( dojo.version.minor < 5 )
				viewerCp.attr("content", viewerAttachPoint);
			else
				viewerCp.set("content", viewerAttachPoint);
			this.processViewer.addChild(viewerCp);
			this.processViewer.selectChild(viewerCp);
			
			// The latest version of Chrome has introduced an issue where the height is being set to 0 when the content pane is being added
			// Unfortunately I don't know when Chrome introduced this issue. For now check that if the height is 0 and Chrome is being used
			// force it to the approximate size it should be. Weill continue to investigate
			if (dojo.isChrome && viewerCp.domNode.style.height == "0px") {
				var width = this.processViewer._borderBox.w;
				var workaroundHeight= width * 0.62;
				viewerCp.domNode.style.height = workaroundHeight + "px";
			}
			
			this.processViewer.resize();
			
			// Disable the event listeners from the viewer we're leaving
			if (this._currentViewer)
				this._currentViewer.removeListener();
			this._currentViewer = new com.ibm.bpm.wpd.visualization.BPMNViewerLite({viewStateModel:vsm}, viewerAttachPoint);
			
			// Enable the event listeners to the viewer we're entering
			this._currentViewer.addListener();
			viewerCp.viewStateModel = vsm;
			viewerCp.viewer = this._currentViewer;
			
			// Add current view to list of views
			this._openedViews[this._openedViews.length] = viewerCp;
			
			if (addbreadcrumb) {
				// Fire event to update the breadcrumb
				this._fireUpdateBreadcrumbEvent(pdocumentId, plabel, psnapshotId, pbranchId);
				com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance().addToBreadcrumb(plabel,pdocumentId);
			}
			
		}

});