dojo.provide("com.ibm.bpm.wpd.visualization.model.BPMNViewerVisualizationModel");

dojo.require("com.ibm.bpm.wpd.core.model.MObject");

dojo.declare("com.ibm.bpm.wpd.visualization.model.BPMNViewerVisualizationModel", [com.ibm.bpm.wpd.core.model.MObject], {

	adornedElementsByParameterMap : null,
	adornedElementsByIdMap : null,
	// This will be the new one
	adornedElementsTagMap: {},
	adornedTagElements:{},
	
	CONTAINER_LEFT_SIDE:"left",
	CONTAINER_RIGHT_SIDE:"right",
	CONTAINER_SPINNER_UP: "up",
	CONTAINER_SPINNER_DOWN: "down",
	
	constructor: function() {
		this.adornedElementsByParameterMap = {};
		this.adornedElementsByIdMap = {};
		this.adornedElementsTagMap = {};
		this.adornedTagElements = {};	
	},
	
	/**
	 * Add an adornment parameter to the model
	 * @param aParameter - parameter to be added
	 */
	addParameter: function(aParameter) {
		var tempVariable = {};
		tempVariable.data = aParameter;
		tempVariable.adornedNodeLabelsExpanded = false;

		this.adornedElementsByParameterMap[aParameter.name] = tempVariable;		
	},
	
	/**
	 * Add the icon adorner for a given parameter. Creates a container value for the side of the element should it not already exist
	 * @param id - the id of the element being adorned
	 * @param adornment - the adornment icon
	 * @param side - the side of the element that contains the adornment (either left or right)
	 * @param parameterName - the name of the parameter
	 */
	addParameterIconAdornment: function(id, adornment, side, parameterName) {
		
		// Lazy initialize the map
		if (!this.adornedElementsByIdMap[id] || this.adornedElementsByIdMap[id]==null) {
			var tempVariable = {};
			tempVariable.containerRight={};
			tempVariable.containerRight.outputs=[];
			tempVariable.containerRight.visibleOutputs=[];
			tempVariable.containerRight.spinnerPosition = 0;
			tempVariable.containerRight.spinnerUp = null;
			tempVariable.containerRight.spinnerDown = null;
			tempVariable.containerLeft={};
			tempVariable.containerLeft.inputs=[];
			tempVariable.containerLeft.visibleInputs=[];
			tempVariable.containerLeft.spinnerPosition = 0;
			tempVariable.containerLeft.spinnerUp = null;
			tempVariable.containerLeft.spinnerDown = null;
			
			this.adornedElementsByIdMap[id] = tempVariable;
		}
		
		var tempParameterItem = this.adornedElementsByIdMap[id];

		if (side == this.CONTAINER_LEFT_SIDE) {
			var containerSide = tempParameterItem.containerLeft;
			containerSide.inputs[containerSide.inputs.length] = parameterName;
			
			if (adornment != null) {
				var slotPosition = this.getSlotPosition(containerSide.visibleInputs);
				
				var slotValue = {};
				slotValue.iconAdornment = adornment;
				slotValue.nameAdornment=null;
				slotValue.parameterName = parameterName;
				
				containerSide.visibleInputs[slotPosition] = slotValue;
			}
		}
		else {
			var containerSide = tempParameterItem.containerRight;
			containerSide.outputs[containerSide.outputs.length] = parameterName;
			
			if (adornment != null) {
				var slotPosition = this.getSlotPosition(containerSide.visibleOutputs);
				
				var slotValue = {};
				slotValue.iconAdornment = adornment;
				slotValue.nameAdornment=null;
				slotValue.parameterName = parameterName;
				
				containerSide.visibleOutputs[slotPosition] = slotValue;
			}
		}

		
	},
	
	/**
	 * Add a spinner adornments to the container of the diagram.
	 * @param id - The element id which will contain the spinner
	 * @param side - The side of the element which will contain the spinner
	 * @param direction - the spinner position (top or bottom)
	 * @param spinner - the spinner adornment
	 */
	addSpinner: function(id, side,direction, spinner) {
		if (side == this.CONTAINER_LEFT_SIDE) {
			if (direction == this.CONTAINER_SPINNER_UP)
				this.adornedElementsByIdMap[id].containerLeft.spinnerUp = spinner;
			else
				this.adornedElementsByIdMap[id].containerLeft.spinnerDown = spinner;
		}
		else {
			if (direction == this.CONTAINER_SPINNER_UP)
				this.adornedElementsByIdMap[id].containerRight.spinnerUp = spinner;
			else
				this.adornedElementsByIdMap[id].containerRight.spinnerDown = spinner;
		}
	},
	
	/**
	 * Add a label adornment for a given icon adorment
	 * @param id - The element id
	 * @param parameterName- The name of the parameter being expanded
	 * @param side - The side where the adornment occurs (left or right)
	 * @param adornment - The label adornment
	 */
	addParameterLabelAdornment: function(id, parameterName, side, adornment, elementHeightPosition) {
		
		// In theory should never happen
		if (!this.adornedElementsByIdMap[id] || this.adornedElementsByIdMap[id]==null) {
			return;
		}
		
		this.adornedElementsByParameterMap[parameterName].adornedNodeLabelsExpanded = true;
		
		var tempParameterItem = this.adornedElementsByIdMap[id];
		
		if (side == this.CONTAINER_LEFT_SIDE) {
			var containerSide = tempParameterItem.containerLeft;
			//for (var i=0; i<containerSide.visibleInputs.length;i++) {
				//if (containerSide.visibleInputs[i].parameterName == parameterName){
					containerSide.visibleInputs[elementHeightPosition].nameAdornment=adornment;
				//	break;
				//}
			//}
		}
		else {
			var containerSide = tempParameterItem.containerRight;
			//for (var i=0; i<containerSide.visibleOutputs.length;i++) {
				//if (containerSide.visibleOutputs[i].parameterName == parameterName){
					containerSide.visibleOutputs[elementHeightPosition].nameAdornment=adornment;
					//break;
				//}
			//}
		}
		
	},
	
	/**
	 * Collect the adornment information for a given parameter name
	 * @param parameterName - The name of the parameter
	 * @returns Will return a collection of nodes in the format
	 * {nodes:[{id:"element id",position:"adornment slot position",side:"element side"},...]
	 */
	getAdornmentInfoViaParameterName: function(parameterName) {
		
		var tempItem = {};
		tempItem.colour = this.adornedElementsByParameterMap[parameterName].data.colour;
		tempItem.adornedNodeLabelsExpanded = this.adornedElementsByParameterMap[parameterName].adornedNodeLabelsExpanded;
		tempItem.nodes = [];
		
		var tNodes = this.adornedElementsByParameterMap[parameterName].data.nodes;
		
		var visitedNodesLeft = {};
		var visitedNodesRight = {};
		
		
		for (var i=0;i<tNodes.length;i++) {
			var tn = this.adornedElementsByIdMap[tNodes[i].id];
			if (tNodes[i].position == this.CONTAINER_LEFT_SIDE) {
				
				if (visitedNodesLeft[tNodes[i].id])
					continue;
				
				visitedNodesLeft[tNodes[i].id] = "v";
				
				for (var j=0;j<tn.containerLeft.visibleInputs.length;j++) {
					if (tn.containerLeft.visibleInputs[j].parameterName == parameterName) {
						var nodeInfo = {};
						nodeInfo.side = this.CONTAINER_LEFT_SIDE;
						nodeInfo.position=j;
						nodeInfo.id = tNodes[i].id;
							
						tempItem.nodes[tempItem.nodes.length] = nodeInfo;
						//break;
					}
				}
			}
			else {
				
				if (visitedNodesRight[tNodes[i].id])
					continue;
				
				visitedNodesRight[tNodes[i].id] = "v";
				
				for (var j=0;j<tn.containerRight.visibleOutputs.length;j++) {
					if (tn.containerRight.visibleOutputs[j].parameterName == parameterName) {
						var nodeInfo = {};
						nodeInfo.side = this.CONTAINER_RIGHT_SIDE;
						nodeInfo.position=j;
						nodeInfo.id = tNodes[i].id;
							
						tempItem.nodes[tempItem.nodes.length] = nodeInfo;
						//break;
					}
				}
			}
		}
		
		return tempItem;
	},
	
	adornmentExists: function(name) {
		if (this.adornedElementsByParameterMap[name])
			return true;
		
		return false;
	},
	
	getAdornmentParameters: function() {
		var result = {};
		for (var name in this.adornedElementsByParameterMap) {
			result[name] = this.adornedElementsByParameterMap[name];
		}
		return result;
	},
	
	/**
	 * Close all the adornments for a given parameter name
	 * @param parameterName
	 */
	closeNameAdormentsForParameterName: function(parameterName) {
		var tNodes = this.adornedElementsByParameterMap[parameterName].data.nodes;
		for (var i=0;i<tNodes.length;i++) {
			var tn = this.adornedElementsByIdMap[tNodes[i].id];
			for (var j=0;j<tn.containerLeft.visibleInputs.length;j++) {
				if (tn.containerLeft.visibleInputs[j].parameterName == parameterName) {
					if (tn.containerLeft.visibleInputs[j].nameAdornment) {
						tn.containerLeft.visibleInputs[j].nameAdornment.deactivate();
						tn.containerLeft.visibleInputs[j].nameAdornment = null;
						/*var ab = tn.containerLeft.visibleInputs[j].nameAdornment.button;
						var tp = ab._template;
						tp = tp.replace("width: 100, height: 18","width: 2, height: 18");
						tp = tp.replace("width: 100, height: 18","width: 2, height: 18");
						ab._template = tp;
						ab.applyTemplate(tp);*/
						//break;
					}
				}
			}
			
			for (var j=0;j<tn.containerRight.visibleOutputs.length;j++) {
				if (tn.containerRight.visibleOutputs[j].parameterName == parameterName) {
					if (tn.containerRight.visibleOutputs[j].nameAdornment) {
						tn.containerRight.visibleOutputs[j].nameAdornment.deactivate();
						tn.containerRight.visibleOutputs[j].nameAdornment = null;
						//break;
					}	
				}
			}
		}
		
		this.adornedElementsByParameterMap[parameterName].adornedNodeLabelsExpanded = false;
	},
	
	/**
	 * Get the next 3 positions for the parameters on a given element id
	 * @param id - id of the element
	 * @param side - the side of the element
	 * @param spinnerDirection - direct the spinner was clicked (up or down)
	 * @returns
	 */
	getNextParameters: function(id,side,spinnerDirection) {
		var tempNodes = this.adornedElementsByIdMap[id];
		var currentPosition = 0;
		var nodesCollection = null;
		var nextThree = [];
		
		if (side == this.CONTAINER_LEFT_SIDE) {
			nodesCollection = tempNodes.containerLeft.inputs;
			currentPosition = tempNodes.containerLeft.spinnerPosition;
		}
		else {
			nodesCollection = tempNodes.containerRight.outputs;
			currentPosition = tempNodes.containerRight.spinnerPosition;
		}
		
		// If up and spinner position is 0, we can't do
		if (spinnerDirection == this.CONTAINER_SPINNER_UP && currentPosition ==0)
			return null;
		
		// if down and spinner position > size-3, we can't do
		//if (spinnerDirection == this.CONTAINER_SPINNER_DOWN && currentPosition > nodesCollection.length-4)
		if (spinnerDirection == this.CONTAINER_SPINNER_DOWN && (currentPosition*3) > nodesCollection.length-4)
			return null;
		
		
//		if (spinnerDirection == this.CONTAINER_SPINNER_DOWN) {
//			nextThree[0] = nodesCollection[currentPosition + 1];
//			nextThree[1] = nodesCollection[currentPosition + 2];
//			nextThree[2] = nodesCollection[currentPosition + 3];
//		}
//		else if (spinnerDirection == this.CONTAINER_SPINNER_UP) {
//			nextThree[0] = nodesCollection[currentPosition - 1];
//			nextThree[1] = nodesCollection[currentPosition];
//			nextThree[2] = nodesCollection[currentPosition + 1];
//		}
//		else {
//			nextThree[0] = nodesCollection[currentPosition];
//			nextThree[1] = nodesCollection[currentPosition + 1];
//			nextThree[2] = nodesCollection[currentPosition + 2];
//		}
		
		if (spinnerDirection == this.CONTAINER_SPINNER_DOWN)
			currentPosition = (currentPosition + 1) * 3;
		else if (spinnerDirection == this.CONTAINER_SPINNER_UP)
			currentPosition = (currentPosition - 1) * 3;
		else {
			// a node was deleted. If our current position is greater we have lost that last node from the spinner. We might need to decrement it
			if (!(currentPosition * 3 < nodesCollection.length) && currentPosition != 0) {
				currentPosition = currentPosition - 1;
				
				if (side == this.CONTAINER_LEFT_SIDE) {
					this.adornedElementsByIdMap[id].containerLeft.spinnerPosition = this.adornedElementsByIdMap[id].containerLeft.spinnerPosition - 1;
				}
				else {
					this.adornedElementsByIdMap[id].containerRight.spinnerPosition = this.adornedElementsByIdMap[id].containerRight.spinnerPosition - 1;
				}
			}
			currentPosition = currentPosition * 3;
		}
			
		
		nextThree[0] = nodesCollection[currentPosition];
		nextThree[1] = nodesCollection[currentPosition + 1];
		nextThree[2] = nodesCollection[currentPosition + 2];
		
		return nextThree;
	},
	
	/**
	 * Close the adornments for a given element
	 * @param id - id of the element
	 * @param side - side of the element
	 */
	clearParameterContainer: function(id,side) {
		var tempNodes = this.adornedElementsByIdMap[id];
		var nodesCollection = null;
		if (side == this.CONTAINER_LEFT_SIDE) {
			nodesCollection = tempNodes.containerLeft.visibleInputs;
			if (tempNodes.containerLeft.spinnerUp) {
				tempNodes.containerLeft.spinnerUp.deactivate();
				tempNodes.containerLeft.spinnerUp = null;
			}
			if (tempNodes.containerLeft.spinnerDown) {
				tempNodes.containerLeft.spinnerDown.deactivate();
				tempNodes.containerLeft.spinnerDown = null;
			}
		}
		else {
			nodesCollection = tempNodes.containerRight.visibleOutputs;
			if (tempNodes.containerRight.spinnerUp) {
				tempNodes.containerRight.spinnerUp.deactivate();
				tempNodes.containerRight.spinnerUp = null;
			}
			if (tempNodes.containerRight.spinnerDown) {
				tempNodes.containerRight.spinnerDown.deactivate();
				tempNodes.containerRight.spinnerDown = null;
			}
		}
		
		for (var i=0;i<nodesCollection.length;i++) {
			
			nodesCollection[i].iconAdornment.deactivate();
			if (nodesCollection[i].nameAdornment)
				nodesCollection[i].nameAdornment.deactivate();
		}
	
	},
	
	/**
	 * For a given node update the visible parameter adornment collect and spinner position
	 * @param id
	 * @param side
	 * @param newVisible
	 * @param spinnerDirection
	 */
	updateVisibleNodes: function(id,side,newVisible,spinnerDirection) {
		var tempNodes = this.adornedElementsByIdMap[id];
		if (side == this.CONTAINER_LEFT_SIDE) {
			nodesCollection = tempNodes.containerLeft.visibleInputs = newVisible;
			if (spinnerDirection == this.CONTAINER_SPINNER_DOWN)
				tempNodes.containerLeft.spinnerPosition++;
			else if (spinnerDirection == this.CONTAINER_SPINNER_UP)
				tempNodes.containerLeft.spinnerPosition--;
				
		}
		else {
			nodesCollection = tempNodes.containerRight.visibleOutputs = newVisible;
			if (spinnerDirection == this.CONTAINER_SPINNER_DOWN)
				tempNodes.containerRight.spinnerPosition++;
			else  if (spinnerDirection == this.CONTAINER_SPINNER_UP)
				tempNodes.containerRight.spinnerPosition--;
		}
	},
	
	/**
	 * Find the next available position in the container to add an adornment to
	 * @param id
	 * @param side
	 * @returns - int of the adornment position
	 */
	getAdornedElementPosition : function(id, side) {
		if (!this.adornedElementsByIdMap[id])
			return 0;

		// We need the visible elements for now
		if (side == this.CONTAINER_LEFT_SIDE) {
			return this._checkPositions(this.adornedElementsByIdMap[id].containerLeft.visibleInputs);
		}

		return this._checkPositions(this.adornedElementsByIdMap[id].containerRight.visibleOutputs);	
	},
	
	needsDownSpinner : function(id, side) {

		if (!this.adornedElementsByIdMap[id])
			return 0;
		//element.containerLeft.spinnerDown && il>3
		// We need the visible elements for now
		if (side == this.CONTAINER_LEFT_SIDE) {
			var tlength = this._checkPositions(this.adornedElementsByIdMap[id].containerLeft.inputs);
			var lc = this.adornedElementsByIdMap[id].containerLeft;
			return (!lc.spinnerDown && lc.spinnerPosition*3+3 < tlength);
		}
		
		var tlength = this._checkPositions(this.adornedElementsByIdMap[id].containerRight.outputs);
		var rc = this.adornedElementsByIdMap[id].containerRight;
		return (!rc.spinnerDown && rc.spinnerPosition*3+3 < tlength);		
	},
	
	_checkPositions : function(positonArray) {
		return positonArray.length;
		
	},
	
	/**
	 * Reset the model to it's initial state
	 */
	reset: function() {
		this.adornedElementsByParameterMap = {};
		this.adornedElementsByIdMap = {};
	},
	
	/**
	 * Remove the parameter from the model
	 * @param parameterName
	 * @returns a collection of elements that will need to be redrawn
	 * In the form [{id:"element id",side:"element side"}]
	 */
	removeParameterAdornment : function(parameterName) {

		var redrawNodes = [];
		if (!this.adornedElementsByParameterMap[parameterName])
			return redrawNodes;
		var tNodes = this.adornedElementsByParameterMap[parameterName].data.nodes;
		for (var i=0;i<tNodes.length;i++) {
			var tn = this.adornedElementsByIdMap[tNodes[i].id];
			if (tNodes[i].position == this.CONTAINER_LEFT_SIDE) {
				// Can remove the loop
				var tempResult = this._removeFromCollection(tn.containerLeft.inputs, parameterName);
				this.adornedElementsByIdMap[tNodes[i].id].containerLeft.inputs = tempResult[0];
				
				// Do I need to update the counter
				if (tempResult[1] < tn.containerLeft.spinnerPosition)
					this.adornedElementsByIdMap[tNodes[i].id].containerLeft.spinnerPosition --;
				
				if (this.adornedElementsByIdMap[tNodes[i].id].containerLeft.inputs.length < 4
						&& this.adornedElementsByIdMap[tNodes[i].id].containerLeft.spinnerUp != null) {
					this.adornedElementsByIdMap[tNodes[i].id].containerLeft.spinnerUp.deactivate();
					this.adornedElementsByIdMap[tNodes[i].id].containerLeft.spinnerUp = null;
					
					if (this.adornedElementsByIdMap[tNodes[i].id].containerLeft.spinnerDown) {
						this.adornedElementsByIdMap[tNodes[i].id].containerLeft.spinnerDown.deactivate();
						this.adornedElementsByIdMap[tNodes[i].id].containerLeft.spinnerDown = null;
					}
					
				}
						
				if (tn.containerLeft.visibleInputs.length < 4) {
					var nodeToRedraw = {};
					nodeToRedraw.id= tNodes[i].id;
					nodeToRedraw.side=tNodes[i].position;
					redrawNodes[redrawNodes.length] = nodeToRedraw;
				}
				else {
					for (var j=0;j<tn.containerLeft.visibleInputs.length;j++) {
						
						if (tn.containerLeft.visibleInputs[j] != null && tn.containerLeft.visibleInputs[j].parameterName == parameterName) {
							var nodeToRedraw = {};
							nodeToRedraw.id= tNodes[i].id;
							nodeToRedraw.side=tNodes[i].position;
							redrawNodes[redrawNodes.length] = nodeToRedraw;
							break;
						}
					}
				}
			}
			else {
				var tempResult = this._removeFromCollection(tn.containerRight.outputs, parameterName);
				this.adornedElementsByIdMap[tNodes[i].id].containerRight.outputs = tempResult[0];
				
				if (tempResult[1] < tn.containerRight.spinnerPosition)
					this.adornedElementsByIdMap[tNodes[i].id].containerRight.spinnerPosition --;
				
				if (this.adornedElementsByIdMap[tNodes[i].id].containerRight.outputs.length < 4
						&& this.adornedElementsByIdMap[tNodes[i].id].containerRight.spinnerUp != null) {
					this.adornedElementsByIdMap[tNodes[i].id].containerRight.spinnerUp.deactivate();
					this.adornedElementsByIdMap[tNodes[i].id].containerRight.spinnerUp = null;
					
					if (this.adornedElementsByIdMap[tNodes[i].id].containerRight.spinnerDown) {
						this.adornedElementsByIdMap[tNodes[i].id].containerRight.spinnerDown.deactivate();
						this.adornedElementsByIdMap[tNodes[i].id].containerRight.spinnerDown = null;
					}
					
				}
					
				
				if (tn.containerRight.visibleOutputs.length < 4) {
					var nodeToRedraw = {};
					nodeToRedraw.id= tNodes[i].id;
					nodeToRedraw.side=tNodes[i].position;
					redrawNodes[redrawNodes.length] = nodeToRedraw;
				}
				else {
					for (var j=0;j<tn.containerRight.visibleOutputs.length;j++) {
						
						if (tn.containerRight.visibleOutputs[j] != null && tn.containerRight.visibleOutputs[j].parameterName == parameterName) {
							var nodeToRedraw = {};
							nodeToRedraw.id= tNodes[i].id;
							nodeToRedraw.side=tNodes[i].position;
							redrawNodes[redrawNodes.length] = nodeToRedraw;
							break;
						}
					}
				}
			}
		}
	
		this.adornedElementsByParameterMap[parameterName]=null;
		
		return redrawNodes;
		
	},
	
	_removeFromCollection: function(aCollection, parameterName) {
		var newCollection = [];
		var removedPosition = 0;
		for (var i=0;i<aCollection.length;i++) {
			if (aCollection[i] != parameterName) {
				newCollection[newCollection.length] = aCollection[i];
			}
			else {
				removedPosition = i;
			}
		}
		
		return [newCollection,removedPosition];
	},
	
	getSlotPosition: function(slots) {
		for (var i = 0;i<slots.length;i++) {
			if (slots[i] == null)
				return i;
		}
		return slots.length;
		
	},
	
	
	// TAG functions
	addTag: function(aTag) {
		var tempVariable = {};
		tempVariable.data = aTag;
		tempVariable.adornedNodes = [];

		if (aTag.groupName)
			this.adornedElementsTagMap[aTag.groupName+"::"+aTag.tagName] = tempVariable;
		else
			this.adornedElementsTagMap[aTag.name] = tempVariable;		
	},
	
	removeTag: function(aTag) {
		var key="";
		if (aTag.tagName)
			key = aTag.groupName+"::"+aTag.tagName;
		else
			key = aTag.name;
		
		var variableData = this.adornedElementsTagMap[key];

		if (!variableData)
			return;
		
		// First deactivate
		for ( var j = 0; j < variableData.adornedNodes.length; j++) {
			variableData.adornedNodes[j].deactivate();
		}
		
		// Second clean up adornedElementsTagMap
		var tc = {};
		for (var keys in this.adornedElementsTagMap) {
			if (keys != key)
				tc[keys] = this.adornedElementsTagMap[keys];
		}

		this.adornedElementsTagMap = tc;
		
		var nodesToRedraw = {};

		// Finally clean up any nodes
		for (var keys in this.adornedTagElements) {
			var tags = this.adornedTagElements[keys].tags;
			var ttags = [];
			for (var i = 0;i<tags.length;i++) {
				if (tags[i] != key)
					ttags[ttags.length] = tags[i];
			}
			
			if (ttags.length > 0 && tags.length != ttags.length)
				nodesToRedraw[keys] = ttags[0];
				
			this.adornedTagElements[keys].tags = ttags;
		}
		
		return nodesToRedraw;
	},
	
	cleanTags: function() {
		this.adornedTagElements = {};
		
		var adornCollection = [];
		
		for (var id in this.adornedElementsTagMap) {
			var element = this.adornedElementsTagMap[id];
			if (element && element.adornedNodes) {
				for (var i=0;i < element.adornedNodes.length;i++) {
					adornCollection.push(element.adornedNodes[i]);
				}
			}
		}
		
		for (var i = 0; i<adornCollection.length;i++) {
			adornCollection[i].deactivate();
		}
		
		this.adornedElementsTagMap = {};
	},

	cleanParameters: function() {
		this.adornedElementsByParameterMap = {};
		
		// Loop through all the parameter and confirm they're cleaned up.
		var adornCollection = [];
		
		for (var id in this.adornedElementsByIdMap) {
			var element = this.adornedElementsByIdMap[id];
			
			if (element.containerLeft) {
				if (element.containerLeft.visibleInputs) {
					for (var i=0;i < element.containerLeft.visibleInputs.length;i++) {
						adornCollection.push(element.containerLeft.visibleInputs[i].iconAdornment);
						if (element.containerLeft.visibleInputs[i].nameAdornment)
							adornCollection.push(element.containerLeft.visibleInputs[i].nameAdornment);
					}
				}
				
				if (element.containerLeft.spinnerUp)
					adornCollection.push(element.containerLeft.spinnerUp);
				
				if (element.containerLeft.spinnerDown)
					adornCollection.push(element.containerLeft.spinnerDown);
			}
			
			if (element.containerRight) {
				if (element.containerRight.visibleOutputs) {
					for (var i=0;i < element.containerRight.visibleOutputs.length;i++) {
						adornCollection.push(element.containerRight.visibleOutputs[i].iconAdornment);
						if (element.containerRight.visibleOutputs[i].nameAdornment)
							adornCollection.push(element.containerRight.visibleOutputs[i].nameAdornment);
					}
				}
				
				if (element.containerRight.spinnerUp)
					adornCollection.push(element.containerRight.spinnerUp);
				
				if (element.containerRight.spinnerDown)
					adornCollection.push(element.containerRight.spinnerDown);
			}
		}
		
		for (var i = 0; i<adornCollection.length;i++) {
			adornCollection[i].deactivate();
		}
		
		this.adornedElementsByIdMap = {};
		
	}
	
	
});