dojo.provide("com.ibm.bpm.wpd.visualization.model.SessionManagement");

dojo.require("com.ibm.bpm.wpd.core.model.MObject");

dojo.declare("com.ibm.bpm.wpd.visualization.model.SessionManagement", [com.ibm.bpm.wpd.core.model.MObject], {
	
	
	PARAMETER_EXPANDED:"E",
	PARAMETER_COLLPASED:"C",
	
	_instance:null,
	_sessionInfo: {},
	

	/**
	 * The constructor is a private constructor. To be called by the getInstance only. Look for the session in the cookie
	 * if it exist read it out, otherwise create an empty sessionInfo
	 */
	constructor: function() {
		// First time being call, see if the session exists. If so retrieve it
		
		var tempSessionInfo = null;
		
		if (this._isOlderBrowser()) {
			var cookieInfo = document.cookie;
			
			var t = cookieInfo.indexOf("visualization");
			if (t != -1) {
				var tString = cookieInfo.substring(t+14, cookieInfo.length);
				t = tString.indexOf(";");
				tempSessionInfo = tString.substring(0,t);
			}
		}
		else {
			tempSessionInfo = window.sessionStorage.getItem("visualization");
		}
		
		
		
		if (tempSessionInfo) 
			this._sessionInfo = dojo.fromJson(tempSessionInfo);
		else {
			this._sessionInfo.visualSelection="parameter"; // or "tag"
			this._sessionInfo.parameters={};
			this._sessionInfo.tags = [];
			this._sessionInfo.breadcrumb = [];
			this._sessionInfo.checkedBoxes = []; // for the parent boxes
		}
	},
	
	/**
	 * Get the visual selection type
	 * @return type
	 */
	getVisualSelection: function() {
		return this._sessionInfo.visualSelection; 
	},
	
	/**
	 * Set the visual selection type
	 * @param type
	 */
	setVisualSelection: function(type) {
		this._sessionInfo.visualSelection = type; 
		this._saveSession();
	},

	/**
	 * Add a parameter to the session. By default the parameter will will collapsed
	 * @param name
	 */
	addParameter: function(name) {
		if (!this._sessionInfo.parameters[name])
			this.updateParameterStatus(name, this.PARAMETER_COLLPASED);
	},
	
	/**
	 * Remove the parameter from the session
	 * @param name
	 */
	removeParameter: function(name) {
		var tp = {};
		for (var param in this._sessionInfo.parameters) {
			if (param != name)
				tp[param] = this._sessionInfo.parameters[param];
		}
		this._sessionInfo.parameters = tp;
		this._saveSession();
	},
	
	/**
	 * Update the parameter state within the session. Currently there are 2 states expanded and collpased
	 * @param name
	 * @param status
	 */
	updateParameterStatus: function(name, status) {
		this._sessionInfo.parameters[name] = status;
		this._saveSession();
	},
	
	/**
	 * Get the parameters currently stored within the session
	 * @returns {Array}
	 */
	getParameterNames: function() {
		var result = [];
		for (var pname in this._sessionInfo.parameters) {
			result[result.length] = pname;
		}
		return result;
	},
	
	/**
	 * Return the parameter currently stored in the sesison along with their states
	 * @returns
	 */
	getParametersWithValues: function() {
		return this._sessionInfo.parameters;
	},
	
	/**
	 * Returns the status of the parameter
	 */
	getParameterStatus: function(name) {
		return this._sessionInfo.parameters[name];
	},
	
	/**
	 * Add a tag name to the session
	 * @param tag
	 * @param group
	 */
	addTag: function(tag, group) {
		for (var i=0; i<this._sessionInfo.tags.length;i++) {
			if (this._sessionInfo.tags[i].tag == tag &&
				this._sessionInfo.tags[i].group == group)
				return;
		}
		this._sessionInfo.tags[this._sessionInfo.tags.length] = {tag:tag, group:group};
		this._saveSession();
	},
	
	/**
	 * Remove the tag name form the session
	 * @param tag
	 * @param group
	 */
	removeTag: function(tag, group) {
		
		var tempTags = [];
		for (var i=0; i<this._sessionInfo.tags.length;i++) {
			if (this._sessionInfo.tags[i].tag != tag ||
				this._sessionInfo.tags[i].group != group)
				tempTags[tempTags.length] = this._sessionInfo.tags[i];
		}
		
		this._sessionInfo.tags = tempTags;
		this._saveSession();
		
	},
	
	/**
	 * Return a collection of tag and group names
	 * @returns
	 */
	getTags: function() {
		return this._sessionInfo.tags;
	}, 
	
	/**
	 * Add the breadcrumb to the sesison path
	 * @param name
	 * @param id
	 */
	addToBreadcrumb: function(name,id) {
		// First check to see if the breadcrumb exists. On a refresh we are forced to reload some of the diagrams, so we don't want to add
		// them to the breadcrumb again. Since it's not possible to have circular references in our breadcrumb (unlike the asset list) this
		// should be fine
		for (var i=0; i<this._sessionInfo.breadcrumb.length;i++) {
			if (this._sessionInfo.breadcrumb[i].indexOf(id) != -1) {
				// It's possible the user changed the name
				this._sessionInfo.breadcrumb[i] = id+":"+name;
				this._saveSession();
				return;
			}
		}
		
		
		this._sessionInfo.breadcrumb[this._sessionInfo.breadcrumb.length] = id+":"+name;
		this._saveSession();
	},
	
	/**
	 * Remove the breadcrumb from the session path.
	 * @param id
	 */
	removeFromBreadcrumb: function(id) {
		var tempBreadCrumb = [];
		for (var i=0; i<this._sessionInfo.breadcrumb.length;i++) {
			tempBreadCrumb[tempBreadCrumb.length] = this._sessionInfo.breadcrumb[i];
			if (this._sessionInfo.breadcrumb[i].indexOf(id) != -1)
				break;
		}
		
		this._sessionInfo.breadcrumb = tempBreadCrumb;
		
		this._saveSession();
	},
	
	/**
	 * Get the session breadcrumb path, include name and id
	 * @returns
	 */
	getBreadcrumb: function() {
		return this._sessionInfo.breadcrumb;
	},
	
	/**
	 * Add a checked box name to the session
	 * @param name
	 */
	addCheckedbox: function(name) {
		var exist = false;
		for (var i=0; i<this._sessionInfo.checkedBoxes.length;i++) {
			if (this._sessionInfo.checkedBoxes[i] == name) {
				exist = true;
				break;
			}
		}
		if (!exist) {
			this._sessionInfo.checkedBoxes[this._sessionInfo.checkedBoxes.length] = name;
			this._saveSession();
		}
	},
	
	/**
	 * Remove the checked box name from the session
	 * @param name
	 */
	removeCheckedbox: function(name) {		
		var tempCheckedBoxes = [];
		for (var i=0; i<this._sessionInfo.checkedBoxes.length;i++) {
			if (this._sessionInfo.checkedBoxes[i] != name)
				tempCheckedBoxes[tempCheckedBoxes.length] = this._sessionInfo.checkedBoxes[i];
		}		
		this._sessionInfo.checkedBoxes = tempCheckedBoxes;
		this._saveSession();
		
	},
	
	/**
	 * Remove all checked box name from the session
	 */
	clearCheckedbox:  function() {
		this._sessionInfo.checkedBoxes = [];
		this._saveSession();
	},
	
	/**
	 * Return a collection of checked box names
	 * @returns
	 */
	getCheckedboxes: function() {
		return this._sessionInfo.checkedBoxes;
	}, 
	
	/**
	 * Reset the session. Should we ever use this? Maybe on a refresh, after we're at a stable state?
	 */
	resetSession: function() {
		this._sessionInfo.visualSelection="parameter";
		this._sessionInfo.parameters={};
		this._sessionInfo.tags = [];
		this._sessionInfo.breadcrumb = [];
		this._sessionInfo.checkedBoxes = [];
		
		this._saveSession();
	},
	
	/**
	 * To be called internally only. Used to save the session information after each change to the session info
	 */
	_saveSession: function() {
		// On older browsers we need to use the cookie method
		if (this._isOlderBrowser()) {
			document.cookie = "visualization=" + dojo.toJson(this._sessionInfo) + ";";
		}
		else{
			// Use the session storage.
			window.sessionStorage.setItem("visualization",dojo.toJson(this._sessionInfo));
		}
	},
	
	_isOlderBrowser: function() {
		//if (dojo.isIE < 8 || dojo.isFF < 3.5 || dojo.isSafari < 4)
		// Remove the ie check as ie is rotten
		if (dojo.isFF < 3.5 || dojo.isSafari < 4)
			return true;
		
		return false;
	},
	
	memoryUsage: function() {
		return dojo.toJson(this._sessionInfo).length;
	}
});

/**
 * Create an instance of the session
 */
com.ibm.bpm.wpd.visualization.model.SessionManagement.getInstance = function() {	
	if (!dojo.global.BPM.WPD_SessionManagement)
		dojo.global.BPM.WPD_SessionManagement = new com.ibm.bpm.wpd.visualization.model.SessionManagement();
	
	return dojo.global.BPM.WPD_SessionManagement;
};