/** Worker-compatible dojo base
 *
 * 
 * We can not use dojo.js since it assumes a UI thread browser environment. 
 * We strip down the services into separate calls, and override dojo.require()
 * to the needs of the WebWorker.
 * 
 * @author AV
 */

function _loadDojo(dojoDir){
	if (typeof this.console == "undefined") {
	
		this.console = {};
		
		var logKinds = ["log", "info", "debug", "warn", "error", "dir"];
		
		for (var i = 0; i < logKinds.length; ++i) {
			console[logKinds[i]] = function(){
			};
		}
	}
	
	if (!this._isDojo17) {
		importScripts(dojoDir + "dojo/_base/_loader/bootstrap.js"); // defines dojo object
		dojo._hasResource = {}; // needed when a dojo build is used
		importScripts(dojoDir + "dojo/_base/_loader/loader.js"); // defines require and provide
		dojo.require = function(moduleName, omitModuleCheck){
			// summary:
			//		A redefined (and simplified) version of dojo.require that works in a Web Worker.
			
			var module = this._loadedModules[moduleName];
			if (module) 
				return module;
			
			var relpath = this._getModuleSymbols(moduleName).join("/");
			var resourcePath = dojoDir + "dojo/" + relpath + ".js";
			
			importScripts(resourcePath);
			
			return this._loadedModules[moduleName];
		};
		
		/** Load other base services we need. 
		 * We can use the just-redefined require().
		 *
		 * @todo AV: better way to do this?
		 */
		dojo.require("dojo._base.connect");
	}
	else {
	
		djConfig = {
			baseUrl: dojoDir + "/dojo",
			has: {
				"host-browser": 0,
				"dom": 0,
				"dojo-has-api": 1,
				"dojo-xhr-factory": 0,
				"dojo-inject-api": 1,
				"dojo-timeout-api": 0,
				"dojo-trace-api": 1,
				"dojo-loader-catches": 1,
				"dojo-dom-ready-api": 0,
				"dojo-dom-ready-plugin": 0,
				"dojo-ready-api": 1,
				"dojo-error-api": 1,
				"dojo-publish-privates": 1,
				"dojo-gettext-api": 1,
				"dojo-sniff": 0,
				"dojo-loader": 1,
				"dojo-boot": 1,
				"dojo-test-xd": 0,
				"dojo-test-sniff": 0
			}
		};
		
		importScripts(dojoDir + "dojo/dojo.js");
		
		listen = dojo.require("dojo.listen");
		
		// simplified dojo.connect/disconnect, we cannot import the full connect.js
		// since it includes DOM code for mouse events etc.
		dojo.connect = function(obj, event, context, method, dontFix){
			var a = arguments, args = [], i = 0;
			args.push(typeof a[0] == "string" ? null : a[i++], a[i++]);
			var a1 = a[i + 1];
			args.push(typeof a1 == "string" || typeof a1 == "function" ? a[i++] : null, a[i++]);
			for (var l = a.length; i < l; i++) {
				args.push(a[i]);
			}
			return dojo._connect.apply(this, args);
		};
		
		dojo._connect = function(obj, event, context, method, dontFix){
			if (typeof event == "string" && event.substring(0, 2) == "on") {
				event = event.substring(2);
			}
			else 
				if (!obj || !(obj.addEventListener || obj.attachEvent)) {
					return aspect.after(obj || dojo.global, event, dojo.hitch(context, method), true);
				}
			if (!obj) {
				obj = dojo.global;
			}
			return listen(obj, event, dojo.hitch(context, method), dontFix);
		};
		
		dojo.disconnect = function(handle){
			if (handle) {
				handle.cancel();
			}
		};
	}
	
	dojo.require("dojo._base.lang");
	dojo.require("dojo._base.array");
	dojo.require("dojo._base.declare");
	dojo.require("dojo._base.json");
}
