/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define([
"../main",
"dojo/_base/declare",
"dojo/_base/connect",
"dojox/gfx/matrix",
"./HudComponent",
"../overview/Overview",
"../overview/OverviewZoomInteractor",
"../overview/OverviewPanInteractor",
"../interactors/InteractorManager",
"../util/Geometry"
], function(
iid,
declare,
connect,
matrix,
HudComponent,
Overview,
OverviewZoomInteractor,
OverviewPanInteractor,
InteractorManager,
g
){

/*=====
var HudComponent = ibm_ilog.diagram.widget.HudComponent;
=====*/

var OverviewHudComponent =
	declare('ibm_ilog.diagram.widget.OverviewHudComponent', HudComponent, {

		//
		//	summary:
		//		A Viewport component holding an overview.
		//

		//
		//	_interactors: ibm_ilog.diagram.interactors.InteractorManager
		//		The InteractorManager used for the interactions in the component
		//
		_interactors: null,

		//
		//	_background: dojox.gfx.Rect
		//		The background Rect shape.
		//
		_background: null,

		//
		//	_size: dojox.gfx.Rect
		//		The background Rect shape.
		//
		_size: g.Size(0, 0),

		//
		//	_overview: ibm_ilog.diagram.overview.Overview
		//		The Overview object used in the component.
		//
		_overview: null,

		constructor: function (viewport, content, renderer, size) {
			//
			//	summary:
			//		Construct the component in the given viewport, with the given renderer, with the given size.
			//
			this._size = size;
			this._shape.setTransform(matrix.translate(iid._invisibleLocation, iid._invisibleLocation));
			this._background = this._shape.createRect(size).setFill([0, 0, 0, 0.5]);

			this._overview = new Overview(viewport, content, renderer, this._shape.createGroup(), this._size);
			this._viewport = viewport;
			this._update();

			this._interactors = new InteractorManager();
			this._interactors.add("pan", new OverviewPanInteractor().initialize(viewport, this._overview, viewport.getSurface()));
			this._interactors.add("zoom", new OverviewZoomInteractor().initialize(viewport, this._overview, viewport.getSurface()));
			this._interactors.switchTo("pan", "zoom");

			connect.connect(viewport, "onClientSizeChanged", this, "_update");
			connect.connect(viewport, "onLayoutHud", this, "_update");
		},
		
		
		setPan: function( b ) {
			//
			//	summary:
			//		Activate or deactivate panning in the overview
			//
			this._interactors.get("pan").setEnabled(b);
		},
		
		getPan: function() {
			//
			//	Returns:
			//		True if the receiver is activated for panning
			//
			this._interactors.get("pan").getEnabled();
		},
		
		setZoom: function( b ) {
			//
			//	summary:
			//		Activate or deactivate zooming in the overview
			//
			this._interactors.get("zoom").setEnabled(b);
		},
		
		getZoom: function() {
			//
			//	Returns:
			//		True if the receiver is activated for zooming
			//
			this._interactors.get("zoom").getEnabled();
		},
		
		getOverview: function() {
			//
			//	returns: ibm_ilog.diagram.overview.Overview
			//			The Overview object
			//
			return this._overview;
		},

		_enable: function () {
			//
			//	summary:
			//			Enables the component and displays it.
			//
			this.inherited(arguments);
			this._overview.setEnabled(true);
			this._update();
		},

		_disable: function () {
			//
			//	summary:
			//			Disables the component and hides it.
			//
			this._overview.setEnabled(false);
			this.inherited(arguments);
		},

		setSize: function (size) {
			//
			//	summary:
			//		Resizes the component
			//
			this._size = size;
			this._update();
		},

		getSize: function () {
			//
			//	summary:
			//		Returns the size of the component.
			//
			return this._size;
		},

		_update: function () {
			//
			//	summary:
			//		Updates the size and location of the component after resize of the 
			//		viewport or the overview component.
			//
			if (this._enabled && this._viewport.isHudEnabled()) {
				var sz = this._size;
				var hsz = this._viewport.getHudSize();
				this._overview.setSize(sz);
				this._background.setShape(sz);

				var location = { y: hsz.height - sz.height, x:0 };
				
				if(!this._viewport.isRtl()){
					location.x = hsz.width - sz.width;
				}

				this._shape.setTransform(matrix.translate(location));
			}
		}

	});

return OverviewHudComponent;

});
