/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define([
"dojo/_base/declare",
"dojo/_base/connect",
"dojox/gfx/matrix",
"./HudComponent",
"./ScrollbarInteractor",
"../util/Geometry"
], function(
declare,
connect,
matrix,
HudComponent,
ScrollbarInteractor,
g
){

/*=====
var HudComponent = ibm_ilog.diagram.widget.HudComponent;
=====*/

var ScrollbarHudComponent =
	declare('ibm_ilog.diagram.widget.ScrollbarHudComponent', HudComponent, {
		
		_tWidth: 16,

		_tPos: 0,

		_minHandleLength: 16, 
		
		_handleTrackPos: 0,

		_handleTrackLength: 16,
		
		_handleShape: {y:0,height:0},

		_setHorizontal:function() {
			this._posName="x";
			this._lenName="width";
			this._tPosName="y";
			this._tLenName="height";
		},
		
		_setVertical:function() {
			this._posName="y";
			this._lenName="height";
			this._tPosName="x";
			this._tLenName="width";
		},

		isHorizontal:function() {
			return this._posName=="x";
		},
		
		isVertical:function() {
			return this._posName=="y";
		},

		constructor: function (viewport, /*boolean*/ horizontal ) {
			
			if(horizontal) {
				this._setHorizontal();
			} else {
				this._setVertical();
			}			
			
			this._clientSizeChanged();

			this._shape.setTransform(matrix.translate(0, 0));

			var trackShape = {};
			trackShape[this._posName] = this._handleTrackPos;
			trackShape[this._lenName] = this._handleTrackLength;
			trackShape[this._tPosName] = this._tPos;
			trackShape[this._tLenName] = this._tWidth; // roundy shape
			this._track = this._shape.createRect(trackShape).setFill([0, 0, 0, 0.5]);

			var shape = {};
			shape[this._tPosName] = this._tPos;
			shape[this._tLenName] = this._tWidth; // roundy shape
			this._handle = this._shape.createRect(shape).setFill([0xAA,0xD7,0xFF,1]);

			this._updateLayout();
			
			connect.connect(viewport, "onViewRectChanged", this, "_updateLayout");
			connect.connect(viewport, "onClientSizeChanged", this, "_clientSizeChanged");
			connect.connect(viewport, "onClientSizeChanged", this, "_updateLayout");

			this._interactor = new ScrollbarInteractor().initialize(viewport,this);
			this._interactor.activate();

		},
		
		_clientSizeChanged: function() {
			var cz = this._viewport.getClientSize();
			this._handleTrackLength = cz[this._lenName]-this._handleTrackPos- this._tWidth;
			this._tPos = cz[this._tLenName] - this._tWidth;
		},
		
		_handleArea: function() {
			var L = this._lenName, P = this._posName, tP = this._tPosName;
			var vr = this._viewport.getViewRect();
			var cr = g.addRect(this._viewport.getContentRect(),vr);
			var r = {};
			
			var hl = vr[L]/cr[L]*this._handleTrackLength;
			if(hl<this._minHandleLength) {
				hl = this._minHandleLength;
				this._ratio = (cr[L]-vr[L])/(this._handleTrackLength-hl);
			} else { 
				this._ratio = cr[L]/this._handleTrackLength;
			}
			r[L] = hl;
			r[P] = (vr[P]-cr[P])/this._ratio;
			r[tP] = this._tPos;
			return r;
		},
		
		getRatio: function() {
			return this._ratio;
		},

		_updateLayout: function () {
			//
			//	summary:
			//		Corrects the position of the component after a resize of the viewport.
			//
			var L = this._lenName, P = this._posName; 
			var tL = this._tLenName, tP = this._tPosName; 

			if (this._enabled && this._viewport.isHudEnabled()) {
				var sh = this._handleArea();
				
				if( this._handleShape[L]  != sh[L] ||
					  this._handleShape[P]  != sh[P] || 
					  this._handleShape[tL] != sh[tL] || 
					  this._handleShape[tP] != sh[tP])
				{
					this._handleShape = sh;
					this._handle.setShape(sh);

					sh = {};
					sh[L]=this._handleTrackLength;
					sh[tP]=this._tPos;
					this._track.setShape(sh);
				}
			}
		}
	});

return ScrollbarHudComponent;

});
