/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define([
"dojo/_base/declare",
"../interactors/DragInteractor",
"../util/Geometry"
], function(
declare,
DragInteractor,
g
){

/*=====
var DragInteractor = ibm_ilog.diagram.interactors.DragInteractor;
=====*/

var ScrollbarInteractor =
declare("ibm_ilog.diagram.widget.ScrollbarInteractor", [DragInteractor], {

	initialize: function( /*ibm_ilog.diagram.widget.Viewport*/viewport, /*ibm_ilog.diagram.widget.ScrollbarHudComponent*/ scrollbar ) {
		this._viewport = viewport;
		this._scrollbar = scrollbar;
		this._initialize();
		
		return this;
	},
	
	_declareStates: function () {
		this._declareState("idle", ["fit","start"]);
		this._declareState("active", ["first", "move", "end", "cancelOnDragStart", "cancelOnSelectStart"]);
	},
	
	getDefaultConnections: function() {
		var cs = this.inherited(arguments);
		cs.fit = {
				src: this._getInitialEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_fitContents",
				filter: this._buildInputFilter({shift:true,button:0}),
				gfxConnect:true
			};
		return cs;
	},

	_getInitialEventSource: function() {
		return this._scrollbar._handle; 
	},
	
	_fitContents: function(e) {
		this._viewport.setViewRect(this._viewport.getContentRect());
	},

	_dragPreStart: function(e) {
		var v = this._viewport;
		this.inherited(arguments);
		this._firstViewRect = v.getViewRect();
	},
    
	_dragMove: function(e) {
    this.inherited(arguments);

    var P = this._scrollbar._posName;
    // scale delta to zoom level to get delta in model coords
    var delta = {};
    delta[P] = this._totalDelta[P];
    var panDelta = g.mulPoint(delta,this._scrollbar.getRatio());
		
    this._viewport.setViewRect( g.moveRect(this._firstViewRect,panDelta), {instant:true} );
  },
	
	_dragEnd: function(e) {
		//
		//	summary:
		//		Terminates the current panning process.
		//
		var v = this._viewport;
		var dragged = this.hasDragged; // inherited will clear this;
		this.inherited(arguments); // clears hadDragged
		if(!dragged) {
			this._viewport.centerAt(this.eventViewportLocation(e));
		}
	}
    
});

return ScrollbarInteractor;

});
