/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define(["../main", 
		"dojo/_base/declare", 
		"dojo/_base/lang", 
		"dojo/_base/sniff", 
		"dojo/_base/connect", 
		"dojo/_base/fx", 
		"dojo/_base/window", 
		"dojo/dom-geometry",
		"dojo/dom-attr",
		"dojo/dom-class", 
		"dojo/dom-style", 
		"dojox/gfx", 
		"dojox/gfx/matrix", 
		"../_base/dojoext", 
		"../util/DeferredInitialization", 
		"../util/Geometry", 
		"../util/ErrorReporter"], 
		function(iid, declare, lang, has, connect, fx, window, domg, doma, domClass, domstyle, gfx, matrix, dojex, DeferredInitialization, g, R){

	var EmptyRect = g.EmptyRect, Size = g.Size;
	
	/*=====
	 var DeferredInitialization = ibm_ilog.diagram.util.DeferredInitialization;
	 =====*/
	var Viewport = declare('ibm_ilog.diagram.widget.Viewport', [DeferredInitialization], {
		//	summary:
		//		A viewport window to a rectangular area of the diagram that is drawn on the browser.
		//		The drawn area is usually controlled by two scrollbars, horizontal and vertical.
		//
		//	description:
		//
		//		The Viewport class is an abstract class that provides basic infrastructure over which
		//		to construct concrete Diagram viewports.
		//
		//		A Viewport object is a collaborator of the Diagram widget that abstracts this notion 
		//		of viewport. The Diagram widget can then deal with the the viewport independently of 
		//		how it is actually implemented. 
		//
		//		The main responsibility of the viewport object is to control the current viewable 
		//		rectangle of the Diagram and report updates to it.
		//	
		//		The following concept definitions help understand the Viewport API:
		//			
		//		-	Content Rectangle    The area inside which the viewport can move, mostly based 
		//								 on the content bounding box.
		//
		//		-	View Rectangle       The rectangular portion of the content that is currently 
		//			                     visible in the viewport.
		//
		//		-	Client Size          The size of the UI space that allows the contents to be 
		//			                     viewed.
		//
		//	authors: AV
		
		_isIBMDiagramViewport: true,
		
		// ========================================================================================================
		//
		//	BASIC COLLABORATORS AND INITIALIZATION:
		//
		//	
		//
		//
		//	_diagram:
		//		The associated Diagram widget.
		//
		_diagram: null,
		
		//
		//	_viewportMatrix:
		//		The transformation matrix applied to the content contents for displaying the in the
		//		viewport. Used to transform model coordinates to view coordinates.
		//
		_viewportMatrix: matrix.identity,
		
		//
		//	_viewportMatrix:
		//		The inverse of the transformation matrix applied to the content contents for 
		//		displaying the in the viewport. Used to transform view coords into model coords.
		//
		_viewportMatrixInverse: matrix.identity,
		
		
		templateString: function(){
			//
			//	summary:
			//		This method is must return the template that the Diagram widget must use 
			//		in order to use the receiver as viewport.
			//
			//	diagram: ibm_ilog.diagram.widget.Diagram
			//		The diagram
			//
			R.error("SubclassResponsibility", "templateString");
		},
		
		//
		//	_needsHudRelayout:
		//		Flag to indicate that we need to relayout the hud and probably resize the surface also.
		//
		_needsHudRelayout: false,
		
		initialize: function( /*ibm_ilog.diagram.widget.Diagram*/diagram){
			//
			//	summary:
			//		Constructs a Viewport for collaborating with the diagram.
			//
			//	diagram: ibm_ilog.diagram.widget.Diagram
			//		The diagram
			//
			this._beginInit();
			
			this._hudComponents = [];
			
			this._diagram = diagram;
			
			this._createSurface();
			
			this._surface.whenLoaded(this, function(){
				this.setViewRect(this._clientSize, {
					instant: true
				});
				this._endInit();
			});
		},
		
		_getSurfaceSize: function(){
			return this.getHudSize();
		},
		
		_updateSurface: function(){
			var hz = this._getSurfaceSize();
			
			if (!has("ie") && !iid.isCanvas) {
				this.getSurface().setDimensions(hz.width + "px", hz.height + "px");
			} else {
				this.getSurface().setDimensions(hz.width, hz.height);
			}
			if (doma.get(document.body, "dir") == "rtl"){
				var offset = this._getSurfaceXOffset();
				var prt = this.getSurface()._parent;				
				domstyle.set(prt, "left", offset+"px");
			};
		},
		
		_getSurfaceXOffset:function(){
			return 0;
		},
		
		_eventBackground: null,
		
		
		_eventfulContent: null,
		
		getEventSourceBackground: function(){
			return this._underlayLayer;
		},
		
		getEventSource: function(){
			return this._eventfulContent;
		},
		
		_createEventSourceShape: function(){
			this._eventBackground = this._underlayLayer.createRect();
			if (domClass.contains(window.doc.body, "dijit_a11y")) // high contrast
				this._eventBackground.setFill([238, 238, 238, 1]);
			else this._eventBackground.setFill([0, 0, 0, 0.0001]);
			this._updateEventSourceShape();
		},
		
		getBackgroundShape: function(){
			return this._eventBackground;
		},
		
		_updateEventSourceShape: function(){
			//	
			//	Updates the event source shape size to the current viewport navigable area
			//
			if (this._eventBackground) {
				var s = this.getHudSize();
				s.width = s.width || 1; // bug in IE6 if width == 0
				s.height = s.height || 1; // bug in IE6 if height == 0
				this._eventBackground.setShape(s);
			}
		},
		
		eventClientLocation: function(e){
			//
			//	Returns the client location of a mouse event generated by the event source
			//
			var r = dojex.coords(this._surface._parent, true);
			if ((e.type !== "undefined") &&
			(e.type === "touchstart" ||
			e.type === "touchmove" ||
			e.type === "touchend" ||
			e.type === "gesturechange")) {
				return {
					x: e.touches[0].pageX - r.x,
					y: e.touches[0].pageY - r.y
				};
				
			}
			return {
				x: e.pageX - r.x,
				y: e.pageY - r.y
			};
		},
		
		
		eventContentLocation: function(e){
			//
			//	Returns the content location of a mouse event generated by the event source
			//
			return this.pointClientToContent(this.eventClientLocation(e));
		},
		
		//
		//	The GFX rendering surface
		//
		_surface: null,
		
		_computeInitialSize: function(){
			var box = domg.getContentBox(this._diagram.domNode);
			var parentBox = null;
			if (!box.w || !box.h) // only if needed 
				parentBox = this._diagram.getParent() ? domg.getContentBox(this._diagram.getParent().containerNode) : {};
			
			// Note the fallback on the height of box for the width and on width of box for the height. 
			// This is intentional. Comment from Abel:
			// It's really a very improbable fallback case; where if only one axis is specified or 
			// can be inferred; then we assume the surface to be a square. You can remove it if you
			// prefer a less confusing logic there. This fallback case has never been used in 
			// any test/sample
			return Size(box.w || parentBox.w || box.h || 256, box.h || parentBox.h || box.w || 256);
		},
		
		_createSurface: function(){
		
			var csz = this._computeInitialSize();
			
			if (!has("ie")) {
				this._surface = gfx.createSurface(this._diagram.canvasNode, csz.width + "px", csz.height + "px");
			} else {
				this._surface = gfx.createSurface(this._diagram.canvasNode, csz.width, csz.height);
			}
			
			this._surface.whenLoaded(this, function(){
			
				//_surface
				//	_hudLayer                 <- top-level HUD components
				//	_eventfulContent          <- content event source
				//		_overlayLayer           <- non-transformed overlay shapes (adorners, marquee, etc.)
				//		_transformedLayer       <- viewport transformation
				//			_transformedOverlay   <- non-content overlays (effects, controls, etc.)
				//			_content              <- content (Graph)
				//			_transformedUnderlay  <- non-content underlays (background, backlights)
				//	  _underlayLayer          <- non-transformed underlay shapes (background, grid, etc.)
				//	
				
				this._eventfulContent = this._surface.createGroup();
				
				// the viewport node is used for transforming the content to be shown into the viewport
				this._transformedLayer = this._eventfulContent.createGroup();
				
				// the overlay layer is for drawing adorners or other event-requiring stuff
				this._overlayLayer = this._eventfulContent.createGroup();
				this._overlayLayer.moveToFront();
				
				// the ui node is for drawing content that is not affected by the viewport transform
				this._hudLayer = this._surface.createGroup();
				this._hudLayer.moveToFront();
				
				// the content node is for user content that is affected by the viewport transformation
				this._content = this._transformedLayer.createGroup();
				
				this._transformedOverlay = this._transformedLayer.createGroup();
				this._transformedOverlay.moveToFront();
				
				this._transformedUnderlay = this._transformedLayer.createGroup();
				this._transformedUnderlay.moveToBack();
				
				this._underlayLayer = this._eventfulContent.createGroup();
				this._underlayLayer.moveToBack();
				
				this._createEventSourceShape();
				
				connect.connect(this._content, "_onChanged", this, "_onContentChangedNotification");
				
				this.setSize(csz, false);
			});
		},
		
		getSurface: function(){
			return this._surface;
		},
		
		getHudLayer: function(){
			return this._hudLayer;
		},
		
		getOverlayLayer: function(){
			return this._overlayLayer;
		},
		
		getUnderlayLayer: function(){
			return this._underlayLayer;
		},
		
		getContent: function(){
			return this._content;
		},
		
		_showOverlays: function(v){
			if (v && this._overlayLayer.getParent() == null) {
				this._eventfulContent.add(this._overlayLayer);
				this._overlayLayer.moveToFront();
			} else if (!v) {
				this._eventfulContent.remove(this._overlayLayer);
			}
		},
		
		_setViewportTransform: function( /*dojox.gfx.matrix*/zt){
			//
			//	returns:
			//		the transformation used for zooming
			//
			//	tags:
			//		private
			//
			this._transformedLayer.setTransform(zt);
		},
		
		getViewportTransform: function(){
			return this._viewportMatrix;
		},
		
		getViewportInverseTransform: function(){
			return this._viewportMatrixInverse;
		},
		
		destroy: function(){
			//
			//	summary:
			//		Gracefully destroys the receiver
			//
			if (this._refreshContentsTimer) {
				clearTimeout(this._refreshContentsTimer);
			}
			if (this._surface) {
				this._surface.destroy();
			}
		},
		
		
		// ========================================================================================================
		//
		//	HUD (Heads-up display)
		//
		_hudComponents: null,
		
		//
		//	_hudEnabled: 
		//		True if hud is enabled
		//
		_hudEnabled: true,
		
		//
		//	_hudZoomingOverlay: 
		//		True if hud zooming overlay effect is enabled
		//
		_hudZoomingOverlay: false,
		
		//
		//
		//	_hudTargetView:
		//		The visible Rect representing the target viewport
		//
		_hudTargetView: null,
		
		registerHudComponent: function(component){
			this._hudComponents.push(component);
		},
		
		setHudZoomFeedbackEnabled: function( /*Boolean*/b){
			this._hudZoomingOverlay = b;
		},
		
		isHudZoomFeedbackEnabled: function(){
			return this._hudZoomingOverlay;
		},
		
		setHudEnabled: function( /*Boolean*/b){
			if (b != this._hudEnabled) {
				if (b) {
					this._surface.add(this._hudLayer);
					this._hudLayer.moveToFront();
					this._hudEnabled = true;
					//this._activateHud();
				} else {
					//this._deactivateHud();
					this._hudEnabled = false;
					this._surface.remove(this._hudLayer);
				}
			}
			return this;
		},
		
		isHudEnabled: function(){
			return this._hudEnabled;
		},
		
		_activateHud: function(){
			//		dojo.forEach(this._hudComponents,function(c){
			//			c.setEnabled(true);
			//		});
		},
		
		_deactivateHud: function(){
			//		dojo.forEach(this._hudComponents,function(c){
			//			c.setEnabled(false);
			//		});
			//		if(this._hudTargetView) {
			//			this._hudLayer.remove(this._hudTargetView);
			//		}
		},
		
		_hudClearZoomingOverlay: function(){
			if (this._hudTargetView) {
				this._transformedOverlay.remove(this._hudTargetView);
			}
		},
		
		_hudCreateZoomingOverlay: function(){
			if (this._hudEnabled && this._hudZoomingOverlay && this._animOrigin.width !== this._animTarget.width) {
				var w = Math.min(this._animTarget.width, this._animTarget.height) * 0.10;
				var len = w * 0.50;
				var p = this._animTarget.scalePivot;
				var x1 = p.x - w;
				var x2 = p.x + w;
				var y1 = p.y - w;
				var y2 = p.y + w;
				
				this._hudTargetView = this._transformedOverlay.createGroup();
				var style = {
					color: "black",
					width: 1 / this._zoom
				};
				this._hudTargetView.createPolyline({
					points: [{
						x: x1,
						y: y1 + len
					}, {
						x: x1,
						y: y1
					}, {
						x: x1 + len,
						y: y1
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x2 - len,
						y: y1
					}, {
						x: x2,
						y: y1
					}, {
						x: x2,
						y: y1 + len
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x2,
						y: y2 - len
					}, {
						x: x2,
						y: y2
					}, {
						x: x2 - len,
						y: y2
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x1 + len,
						y: y2
					}, {
						x: x1,
						y: y2
					}, {
						x: x1,
						y: y2 - len
					}]
				}).setStroke(style);
			}
		},
		
		
		
		
		// ========================================================================================================
		//
		//	CONTENT RECTANGLE:
		//
		//	The area inside which the viewport can move, mostly based on the content bounding box.
		//
		//	The content rectangle (or navigable area) is computed from two values, the content 
		//	bounding box and the current client size, and it is obtained by the following method:
		//	A percentage of the current client size, defined by _airFactor is added to the
		//	borders of the content bounding box.
		//	
		//	The two main events that change the content rectangle are:
		//		- a change in the content
		//			(_onContentBoundingBoxChanged)
		//		- a change in the size of the viewport
		//			(setClientSize)
		
		//
		//	_contentBoundingBox:
		//		The area inside which we can move
		//
		_contentBoundingBox: EmptyRect,
		
		//
		//	_contentRect:
		//		The area inside which we can move
		//
		_contentRect: EmptyRect,
		
		//
		//	_airFactor:
		//		The minimum navigable space to the sides of the current content bounds as a percentage
		//		with respect to the current model bounding box.
		//
		_airFactor: 0.0,
		
		
		_getContentAir: function(){
			return g.sizeLongestSide(this._contentBoundingBox) * this._airFactor;
		},
		
		//
		//	_refreshContentsTimer:
		//		The timer scheduled by the refreshContentNotification request
		//
		_refreshContentsTimer: null,
		
		//
		//	_refreshContentsInterval:
		//		Refresh rate used for monitoring changes in the content.
		//
		_refreshContentsInterval: 50,
		
		setRefreshContentsSuspended: function( /*Boolean*/suspended){
			if (this._refreshContentsSuspended !== suspended) {
				this._refreshContentsSuspended = suspended;
				if (suspended) {
					clearTimeout(this._refreshContentsTimer);
					this._refreshContentsTimer = null;
				} else {
					this._refreshContents();
				}
			}
			
		},
		
		getRefreshContentsSuspended: function(){
			return this._refreshContentsSuspended;
		},
		
		setRefreshContentsInverval: function( /*Number*/msInterval){
			this._refreshContentsInterval = msInterval;
			if (msInterval < 0 && this._refreshContentsTimer) {
				clearTimeout(this._refreshContentsTimer);
				this._refreshContentsTimer = null;
			}
			return this;
		},
		
		getRefreshContentsInterval: function(){
			return this._refreshContentsInterval;
		},
		
		_onContentChangedNotification: function(){
			//
			//	summary:
			//		Sets up a timer to give the originating process time to finish with all the changes
			//
			if (!this._refreshContentsSuspended && this._refreshContentsInterval >= 0 && !this._refreshContentsTimer) {
				this._refreshContentsTimer = setTimeout(lang.hitch(this, "_refreshContents", this._refreshContents), this._refreshContentsInterval);
			}
		},
		
		refresh: function(){
			//
			// 	summary:
			//		Forces a refresh contents procedure, where the content rect is updated and consequently 
			//		associated scrollbars and other features. 
			//
			this._refreshContents();
		},
		
		_refreshContents: function(){
			//
			//	summary:
			//		Callback executed after the contentChangedNotification timer is finished.
			//
			if (!this._refreshContentsSuspended) {
				this._refreshContentsTimer = null;
				this._onContentBoundingBoxChanged();
			}
		},
		
		setAirFactor: function( /*Number*/factor){
			//
			//	summary:
			//		Sets the content air percentage with respect to the client size
			//
			this._airFactor = factor;
			
			if (this._computeContentRect()) {
				this._onViewRectConstraintsChanged();
			}
			return this;
		},
		
		
		getAirFactor: function(){
			//
			//	summary:
			//		Returns the content air percentage with respect to the client size
			//
			return this._airFactor;
		},
		
		
		getContentRect: function(){
			//
			//	summary:
			//		The current content size in pixels, which defines the available navigation area.
			//
			//	description:
			//		This value is updated by content BB change notifications (see _onContentBoundingBoxChanged)
			//
			//
			return this._contentRect;
		},
		
		
		_onContentBoundingBoxChanged: function(){
			//
			//	summary:
			//		The function is executed when the content of the content has changed.
			//
			//	returns:
			//		True if the change affected content bounding box.
			//
			//	tags:
			//		private
			//
			this._contentBoundingBox = this._content.getBoundingBox() || EmptyRect;
			
			if (this._computeContentRect()) {
				this._onViewRectConstraintsChanged()
			}
		},
		
		
		_computeContentRect: function(){
			//
			//	summary:
			//		Updates the navigable area
			//
			//	tags:
			//		private
			//
			var newContentRect = g.expandRect(this._contentBoundingBox, this._getContentAir());
			
			if (!g.equalRect(this._contentRect, newContentRect)) {
				this._contentRect = newContentRect;
				this.onContentRectChanged();
				return true;
			} else {
				return false;
			}
		},
		
		onContentRectChanged: function(){
			//
			//	summary:
			//		Event triggered when the content rect has changed.
			//
		},
		
		_constraintToContent: function( /*dojox.gfx.Rectangle*/r){
			//
			//	summary:
			//		Applies a constraining policy on view rectangle r, returning the result.
			//
			//	tags:
			//		private
			//
			var cx = this._contentRect.x;
			var cy = this._contentRect.y;
			var cw = this._contentRect.width;
			var ch = this._contentRect.height;
			var w = r.width;
			var h = r.height;
			var x = r.x;
			var y = r.y;
			
			// Correct aspect ratio of r enlarging shorter sides
			var csz = this.getClientSize();
			var vrat = g.getRectRatio(csz);
			var rrat = g.getRectRatio(r);
			if (rrat > vrat) {
				h = w / vrat;
			} else if (rrat < vrat) {
				w = h * vrat;
			}
			
			var pivotdelta;
			if (this._minZoom > 0 && csz.width / w < this._minZoom) {
				// TODO move this into a geometric pivot-scaling of rectangles
				pivotdelta = r.scalePivot ? g.mulPoint(g.subPoint(r, r.scalePivot), (csz.width / w) / this._minZoom) : {
					x: 0,
					y: 0
				};
				
				x = r.scalePivot.x + pivotdelta.x;
				y = r.scalePivot.y + pivotdelta.y;
				
				w = csz.width / this._minZoom;
				h = csz.height / this._minZoom;
			}
			
			if (this._maxZoom > 0 && csz.width / w > this._maxZoom) {
				// TODO move this into a geometric pivot-scaling of rectangles
				pivotdelta = r.scalePivot ? g.mulPoint(g.subPoint(r, r.scalePivot), (csz.width / w) / this._maxZoom) : {
					x: 0,
					y: 0
				};
				
				x = r.scalePivot.x + pivotdelta.x;
				y = r.scalePivot.y + pivotdelta.y;
				
				w = csz.width / this._maxZoom;
				h = csz.height / this._maxZoom;
			}
			
			return {
				x: x,
				y: y,
				width: w,
				height: h,
				scalePivot: r.scalePivot
			};
		},
		
		// ========================================================================================================
		//
		//	CLIENT SIZE:
		//
		//		The size of the UI space that allows the contents to be viewed.
		//
		//
		//	_clientSize:
		//		The size of current viewport
		//
		_clientSize: EmptyRect,
		
		//
		//	_resizeMode: Number
		//		Current resize mode of the vieport, one of:
		//		| 1: Keep zoom level, expand area
		//		| 2: Keep viewable area, change zoom level. Longest side defines zoom level.
		//
		_resizeMode: 1,
		
		setResizeMode: function( /*Number*/m){
			this._resizeMode = m;
		},
		
		getResizeMode: function(){
			return this._resizeMode;
		},
		
		getClientSize: function(){
			//
			//	summary:
			//		Returns the current viewport client size
			//
			return {
				width: this._clientSize.width,
				height: this._clientSize.height
			};
		},
		
		
		setClientSize: function( /*Size*/sz){
			//
			//	summary:
			//		Sets the Client Size
			//
			R.error("SubclassResponsibility", "setClientSize");
		},
		
		
		onClientSizeChanged: function(){
			//
			//	summary:
			//		The viewport has been resized (implies the related onSourceChanged event)
			//
			this.onLayoutHud();
			
			// Possible HUD information: this is a test
			//this._updateHudZoom();
			this._updateEventSourceShape();
		},
		
		getHudSize: function(){
			return this.getClientSize();
		},
		
		setSize: function( /*Size*/sz){
			//
			//	summary:
			//		Sets the Viewport Size
			//
			R.error("SubclassResponsibility", "setSize");
		},
		
		// ========================================================================================================
		//
		//	VIEW RECTANGLE:
		//
		//	The rectangle that represents which part of the model is currently seen from the viewport.
		//
		//	
		//
		//	_viewRect:
		//		The bounds of current content
		//
		_viewRect: EmptyRect,
		
		//
		//	_animate:
		//		Boolean flag to indicate that viewport movements are animated by default.
		//
		_animate: false,
		
		//
		//	_animLength:
		//		The length of the animation in milliseconds
		//
		_animLength: 200,
		
		//
		//	_animSteps:
		//		The number of updates to the viewport during the animation
		//
		_animSteps: 10,
		
		//
		//	_animation:
		//		The animation object (while animating).
		//
		_animation: null,
		
		//
		//	_animOrigin:
		//		The animation origin (while animating).
		//
		_animOrigin: null,
		
		//
		//	_animTarget:
		//		The animation target view rectangle (while animating).
		//
		_animTarget: null,
		
		getAnimLength: function(){
			return this._animLength;
		},
		
		setAnimLength: function( /*Number*/milliseconds){
			this._animLength = milliseconds;
			return this;
		},
		
		getAnimSteps: function(){
			return this._animSteps;
		},
		
		setAnimSteps: function( /*Number*/steps){
			this._animSteps = steps;
			return this;
		},
		
		setAnimated: function( /*Boolean*/animated){
			this._animated = animated;
			return this;
		},
		
		getAnimated: function(){
			return this._animated;
		},
		
		getViewRect: function(){
			//
			//	summary:
			//		The currently displayed area in diagram coordinates.
			//
			return this._viewRect;
		},
		
		_cloneRect: function(r){
			//	[AV] note: this is 7x-9x faster than the dojo.clone equivalent or than iterating attributes:
			//		var r, rect = {x:4,y:7,width:10,height:123};
			//		var start = new Date().getTime();
			//		for(var i = 0; i<10000; i++) { r = dojo.clone(rect); }
			//		var end = new Date().getTime();
			//		var start2 = new Date().getTime();
			//		for(var i = 0; i<10000; i++) { r = V._cloneRect(rect); }
			//		var end2 = new Date().getTime();
			//		console.log("dojo.clone",end-start,"_cloneRect",end2-start2);
			var cr = {};
			if (r.x !== undefined) 
				cr.x = r.x;
			if (r.y !== undefined) 
				cr.y = r.y;
			if (r.width !== undefined) 
				cr.width = r.width;
			if (r.height !== undefined) 
				cr.height = r.height;
			if (r.scalePivot !== undefined) 
				cr.scalePivot = r.scalePivot;
			return cr;
		},
		
		prepareViewRect: function( /*dojox.gfx.Rectangle*/r, /*dojox.gfx.Rectangle*/ viewRect /*=current*/){
			//
			//	summary:
			//		Pre-apply corrections to r that would otherwise be applied at the moment r is set as 
			//		the View Rectangle
			//
			//	description:
			//		Some corrections may be applied to r before assigning it 
			//		as the current View Rectangle:
			//
			//    	- if width xor height are set, set the other based on current viewport aspect ratio
			//
			//    	- for each axis:
			//    		- if origin is not defined and length is, make origin so that center is preserved
			//    		- if origin is defined but width is not, keep current length
			//
			//    	- if both width and height are set but the aspect ratio is not correct, correct short 
			//    	  side to match correct aspect ratio and maintain visibility of desired area.
			//
			//    	- if r goes beyond height or width, constrain location to make r within
			//
			//    	- if r lies before the origin, set location to origin
			//
			//    	- apply zoom constraints
			//
			viewRect = viewRect || this._viewRect;
			r = this._cloneRect(r);
			r.scalePivot = r.scalePivot || g.getRectCenter(r) || g.getRectCenter(viewRect);
			
			if (r.width === undefined && r.height !== undefined) {
				r.width = r.height * (g.getRectRatio(this.getClientSize()));
			}
			
			if (r.height === undefined && r.width !== undefined) {
				r.height = r.width / (g.getRectRatio(this.getClientSize()));
			}
			
			r = g.rectCenteredOverride(viewRect, r);
			r = this._constraintToContent(r);
			
			return r;
			
		},
		
		
		stopAnimation: function(){
			if (this._animation) {
				this._animation.stop();
				this._animation = null;
			}
		},
		
		_startViewRectAnimation: function( /*dojox.gfx.Rectangle*/viewRect){
			this._animOrigin = this._viewRect;
			this._animTarget = viewRect;
			this._hudCreateZoomingOverlay();
			var animation = new fx.Animation({
				duration: this._animLength,
				rate: this._animLength / this._animSteps,
				curve: [0, 100],
				onStop: lang.hitch(this, function(v){
					animation = null;
					this._animTarget = null;
					this._animOrigin = null;
					this._hudClearZoomingOverlay();
				}),
				onAnimate: lang.hitch(this, function(v){
					if (this._animation === animation) {
						// this is checked because animation.stop() is 
						// not working when fired from within the first 
						// step.
						var completion = v / 100;
						this._viewRect = g.standarizeRect(g.rectBlend(this._animOrigin, this._animTarget, completion));
						this._applyViewRect(true, true);
						this.onViewRectChanged();
						if (v == 100) {
							this.stopAnimation();
						}
					} else {
						// Try to stop the animation again
						animation.stop();
					}
				})
			});
			this._animation = animation;
			this._animation.play();
		},
		
		setViewRect: function( /*dojox.gfx.Rectangle*/r, /*Obj?*/ options){
			//
			//	summary:
			//		Sets the currently displayed area.
			//
			//	description:
			//		Some corrections are applied to r. See prepareViewRect.
			//
			// 	options:
			//		.animate
			//			animation of the viewport
			//		.instant
			//			cancel animated viewport
			//		.forceApply
			//			causes the view rect to be applied even if it has not changed
			//    .center
			//      causes the view rect to be centered
			//
			options = options || {};
			if (!r) {
				return;
			}
			var oldR = g.cloneRect(r);
			r = this.prepareViewRect(r);
			if (options.center) {
				if (oldR.width <= r.width) {
					r.x -= (r.width - oldR.width) / 2;
				}
				if (oldR.height <= r.height) {
					r.y -= (r.height - oldR.height) / 2;
				}
			}
			
			var sizeChanged = !g.rectEqualSize(this._viewRect, r, 1 / this._zoom);
			var originChanged = !g.rectEqualOrigin(this._viewRect, r, 1 / this._zoom);
			
			this.stopAnimation();
			
			if (sizeChanged || originChanged || options.forceApply) {
				if (!(this._animated || options.animate) || options.instant) {
					this._viewRect = g.standarizeRect(r);
					this._applyViewRect(sizeChanged, originChanged);
					this.onViewRectChanged();
				} else {
					this._startViewRectAnimation(r);
				}
			}
			
			return this._viewRect;
		},
		
		_applyViewRect: function( /*Boolean*/sizeChanged, /*Boolean*/ originChanged){
			this._computeZoom();
			
			//
			//	Subclasses must extend with the required actions.
			//
		},
		
		clientToContent: function(){
			//
			//	summary:
			//		returns a matrix that transforms a client coordinate into a content coordinate
			//
			return this._viewportMatrixInverse;
		},
		
		pointClientToContent: function( /*Point*/p){
			//
			//	summary:
			//		transforms a client coordinate into a content coordinate
			//
			return this._viewportMatrixInverse.transformPoint(p);
		},
		
		contentToClient: function(){
			//
			//	summary:
			//		returns a matrix that transforms a content coordinate into a client coordinate
			//
			return this._viewportMatrix;
		},
		
		pointContentToClient: function( /*Point*/p){
			//
			//	summary:
			//		transforms a content coordinate into a client coordinate
			//
			return this._viewportMatrix.transformPoint(p);
		},
		
		onLayoutHud: function(){
			//this._updateHudZoom();
			this._updateSurface();
			this._needsHudRelayout = false;
		},
		
		onViewRectChanged: function onViewRectChanged(){
			//
			//	summary:
			//		The view rectangle has been updated. 
			//
			// scrollbars may have reappeared/disappeared 
			if (this._needsHudRelayout) {
				this.onLayoutHud();
				this._needsHudRelayout = false;
			}
		},
		
		
		_onViewRectConstraintsChanged: function(){
			//
			//	summary:
			//		The view rectangle constraints have been updated.
			//
			this.setViewRect(this._viewRect, {
				forceApply: true,
				instant: true
			});
		},
		
		
		// ========================================================================================================
		//
		//	ZOOM:
		//
		//	Viewport zoom is defined from the relation between the viewport size and the view rectangle.
		//
		//		
		
		//
		//	_zoom:
		//		Current zoom level factor (larger is closer)
		//
		_zoom: 1,
		
		
		//
		//	_maxZoom:
		//		Defines how much the view rectangle can zoom in. Unlimited if 0.
		//	
		_maxZoom: 0,
		
		
		//
		//	_minZoom:
		//		Defines how much the view rectangle can zoom out. If 0, the limit is based on the contentRectangle.
		//	
		_minZoom: 0,
		
		
		setMaxZoom: function( /*Number*/z){
			//
			//	summary:
			//		Sets a maximum zoom and updates the view rectangle based on new constraints.
			//
			if (z < 0) { // normalize
				z = 0;
			}
			
			if (z > 0 && this._minZoom > 0 && z < this._minZoom) {
				z = this._maxZoom;
				R.warn("MaxZoomBelowMin");
			}
			this._maxZoom = z;
			this._onViewRectConstraintsChanged();
			return this;
		},
		
		
		getMinZoom: function(){
			//
			//	summary:
			//		Returns the current minimum zoom constraint.
			//
			return this._minZoom;
		},
		
		
		getMaxZoom: function(){
			//
			//	summary:
			//		Returns the current maximum zoom constraint.
			//
			return this._maxZoom;
		},
		
		
		setMinZoom: function( /*Number*/z){
			//
			//	summary:
			//		Sets a minimum zoom and updates the view rectangle based on new constraints.
			//
			if (z < 0) { // normalize
				z = 0;
			}
			
			if (z > 0 && this._maxZoom > 0 && z > this._maxZoom) {
				z = this._maxZoom;
				R.warn("MinZoomAboveMax");
			}
			this._minZoom = z;
			this._onViewRectConstraintsChanged();
		},
		
		
		setZoom: function( /*Number*/zoom, options){
			//
			//	summary:
			//		This method is a helper that actually controls setSourceRect expanding or shrinking
			//		the view rectangle size to achieve the zoom effect.
			//
			//	options:
			//		.animate
			//			animation of the viewport
			//		.instant
			//			cancel animated viewport
			//		.forceApply
			//			causes the view rect to be applied even if it has not changed
			//
			this.setViewRect({
				height: (this.getClientSize().height / zoom)
			}, options);
			return this;
		},
		
		
		getZoom: function(){
			//
			//	summary:
			//		Returns the current zoom level based on the current view rectangle and content size.
			//
			return this._zoom;
		},
		
		
		_computeZoom: function(){
			//
			//	summary:
			//		Computes and stores the current zoom factor by comparing viewport size and view rectangle size.
			//
			var newZoom = this._zoom;
			if (this._viewRect.width) {
				var vp = this.getClientSize();
				newZoom = Math.min(vp.width / this._viewRect.width, vp.height / this._viewRect.height);
			}
			this._zoom = newZoom;
		},
		
		
		
		
		
		// ========================================================================================================
		//
		//	MISCELANEOUS SERVICES:
		//
		//
		//	_focusAirFactor:
		//		The minimum space reserved for the view rect on the visualizeArea operation.
		//		Expressed in terms of a percentage of the size of the area.
		//
		_visualizeAirFactor: 1.50,
		
		
		visualizeArea: function( /*dojox.gfx.Rectangle*/r, /*{expandCurrent,airFactor}*/ options){
			//
			//	summary:
			//		Makes the r rectangle visible, givin it an extra air between the 
			//		area and the borders of the viewport if possible. The view may be 
			//		zoomed out if required, but it is never zoomed in.
			//
			//	r:
			//		The area to make visible.
			//
			//	airFactor:
			//		Percent of the area that is reserved as air. Overrides the default air 
			//		factor defined bu the member _focusAirFactor.
			//
			options = options || {};
			var vr = this._viewRect;
			var airFactor = (options.airFactor !== undefined) ? (1 + 2 * options.airFactor) : this._visualizeAirFactor;
			
			if (options.expandCurrent) {
				this.setViewRect(g.addRect(g.expandRect(r, Math.max(r.width, r.height) * (airFactor - 1) / 2), vr), options);
			} else {
				var minSize = {
					width: Math.max(vr.width, r.width * airFactor),
					height: Math.max(vr.height, r.height * airFactor)
				};
				var centered = g.rectCenteredAt(vr, g.getRectCenter(r));
				this.setViewRect(g.rectCenteredOverride(centered, minSize), options);
			}
		},
		
		centerAt: function( /*Point*/p, options){
			//
			//	summary:
			//		Centers the viewport at the requested point, without changing the zoom.
			//
			this.setViewRect(g.rectCenteredAt(this.getViewRect(), p), options);
		},
		
		zoomAt: function( /*Point*/p, /*Number*/ z, options){
			//
			//	summary:
			//		Scales the current zooms by a given factor z, using point p as pivot.
			//
			var vr = this.getViewRect(); // current viewport pos
			var rp = g.subPoint(p, vr); // relative cp
			var newViewRect = g.subPoint(p, g.mulPoint(rp, 1 / z));
			newViewRect.height = vr.height / z;
			newViewRect.scalePivot = p;
			
			this.setViewRect(newViewRect, options);
		},
		
		reset: function(){
			//
			//	TODO: reset the receiver to its initial state
			//
		},
		
		_dumpState: function(){
			console.log("zoom:" + g._round(this._zoom, 3) + " view:" + g.printRect(this._viewRect) + " cnt:" + g.printRect(this._contentRect));
		},
		
		toString: function(){
			return "[Viewport]";
		},
		
		getDiagram: function(){
			return this._diagram;
		},
		
		bringIntoView: function(/* ibm_ilog.diagram.Node */node, /* Number? */ margin){
			//
			//  summary:
			//     translate the view just enough so that a given element is fully visible
			//
			
			var vr = this.getViewRect();
			var bb = node.getBounds(this.getDiagram().getGraph());
			if (margin === undefined) 
				margin = 10;
			
			bb = g.cloneRect(bb);
			bb.x -= margin;
			bb.y -= margin;
			bb.width += 2 * margin;
			bb.height += 2 * margin;
			
			var newvr = {
				x: vr.x,
				y: vr.y
			};
			newvr.x = Math.max(newvr.x, bb.x + bb.width - vr.width);
			newvr.y = Math.max(newvr.y, bb.y + bb.height - vr.height);
			newvr.x = Math.min(newvr.x, bb.x);
			newvr.y = Math.min(newvr.y, bb.y);
			
			this.setViewRect(newvr);
		}
		
		
	});
	
	return Viewport;
	
});
