/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define([
"../main",
"dojo/_base/declare",
"dojo/_base/connect",
"dojox/gfx/matrix",
"./HudComponent",
"../util/Geometry"
], function(
iid,
declare,
connect,
matrix,
HudComponent,
g
){

var Point = g.Point;

/*=====
var HudComponent = ibm_ilog.diagram.widget.HudComponent;
=====*/

var ZoomHudComponent =
	declare('ibm_ilog.diagram.widget.ZoomHudComponent', HudComponent, {
		//
		//	summary:
		//		A viewport component that displays the current zoom factor.
		//

		//
		//	_currentZoom:
		//		The current zoom factor being displayed at the component
		//
		_currentZoom: 0,

		constructor: function (viewport) {
			this._shape.setTransform(matrix.translate(iid._invisibleLocation, iid._invisibleLocation));
			this._shape.createRect({
				width: 30,
				height: 20
			}).setFill([0, 0, 0, 0.5]);
			this._shape.createText({
				x: 15,
				y: 15,
				align: "middle",
				text: "x" + g._round(this._zoom, 1)
			}).setFont({
				size: "10pt",
				family: "sans-serif"
			}).setStroke("white").setFill("white");
			this._updateLayout();
			this._updateValue();

			connect.connect(viewport, "onLayoutHud", this, "_updateLayout");
			connect.connect(viewport, "onViewRectChanged", this, "_updateValue");
			//connect.connect(viewport,"onClientSizeChanged",this,"_updateLayout");
		},

		_enable: function () {
			//
			//	summary:
			//			Enables the component and displays it.
			//
			this.inherited(arguments);
			this._updateLayout();
			this._updateValue();
		},

		_updateValue: function () {
			//
			//	summary:
			//		Updates the value displayed at the component
			//
			if (this._enabled && this._viewport.isHudEnabled() && (this._viewport.getZoom() !== this._currentZoom)) {
				this._currentZoom = this._viewport.getZoom();
				this._shape.children[1].setShape({
					text: "x" + g._round(this._currentZoom, 1)
				});
			}
		},

		_updateLayout: function () {
			//
			//	summary:
			//		Corrects the position of the component after a resize of the viewport.
			//
			if (this._enabled && this._viewport.isHudEnabled()) {
				var hudSize = this._viewport.getHudSize();
				var location;
				if(!this._viewport.isRtl()) {
					location = Point(hudSize.width - 30, 0);
				} else {
					location = Point(0,0);
				}
				this._shape.setTransform(matrix.translate(location));
			}
		}
	});

return ZoomHudComponent;

});
