/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define([
"dojo/_base/lang",
"dojo/_base/array",
"dojo/_base/sniff",
"dojo/_base/connect",
"dojo/dom-construct",
"dojo/dom-attr",
"dijit/Tooltip",
"dijit/focus",
"dojo/number",
"../interactors/PanInteractor",
"../interactors/ZoomInteractor",
"../interactors/SelectionInteractor",
"../interactors/MoveInteractor",
"../interactors/MarqueeSelectionInteractor",
"../interactors/WheelPanInteractor",
"../interactors/AdornerFocusInteractor",
"../interactors/FocusInteractor",
"../interactors/KeyboardInteractorManager",
"../interactors/Navigator",
"../interactors/SelectionKeyInteractor",
"../interactors/MoveKeyInteractor",
"../interactors/PanKeyInteractor",
"../interactors/ZoomKeyInteractor",
"../interactors/SwitchModeInteractor",
"../adorners/Adorner",
"../adorners/LinkAdorner",
"../adorners/AnnotationAdorner",
"../util/GraphUtil"
], function(
lang,
array,
has,
connect,
domctr,
domattr,
Tooltip,
focus,
number,
PanInteractor,
ZoomInteractor,
SelectionInteractor,
MoveInteractor,
MarqueeSelectionInteractor,
WheelPanInteractor,
AdornerFocusInteractor,
FocusInteractor,
KeyboardInteractorManager,
Navigator,
SelectionKeyInteractor,
MoveKeyInteractor,
PanKeyInteractor,
ZoomKeyInteractor,
SwitchModeInteractor,
Adorner,
LinkAdorner,
AnnotationAdorner,
gu
){

// This class contains the Diagram code that is conditionnally loaded
// only on desktop (non-mobile) platforms.

	var _defaultWaiLabelFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	var _defaultWaiDescriptionFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	// Extender object to add functionality on desktop (i.e., non-mobile) platforms:
	
	var _Diagram = {
		
		// =======================================================================
		//
		// INTERACTORS:
		//
		
		_initializeInteractors: function() {
			//
			//	summary:
			//		Initializes the interactors used by the Diagram
			//
			//	tags:
			//		protected
			//
			
			this._interactors.add("pan", new PanInteractor().initialize(this));
			this._interactors.add("select", new SelectionInteractor().initialize(this._graph, this.getSelection(), this._viewport.getBackgroundShape()));
			this._interactors.add("zoom", new ZoomInteractor().initialize(this));
			
			this._interactors.add("move", new MoveInteractor().initialize(this._selection, this.getViewport(), {
				doMove: this.moveFunction,
				isMovable: this.isMovableFunction
			}));
			this._interactors.add("marquee", s = new MarqueeSelectionInteractor().initialize(this.getViewport(), this.getSelection()));
			this._interactors.add("wheelPan", new WheelPanInteractor().initialize(this));
			this._interactors.add("adorners", new AdornerFocusInteractor().initialize(this));
			this._interactors.add("focus", new FocusInteractor().initialize(this));
			
			this._setupInteractors(["pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus"]);
			
			connect.connect(this._graph, "onLayoutAnimationStart", this, function() {
				// TODO focusedElement also
				this.setNodeAdornersFocus(null);
				if (this._getNodeAdornersFocusAttr()) {
					this._setNodeAdornersFocusAttr(false);
					
					var h = connect.connect(this._graph, "onLayoutAnimationEnd", this, function() {
						connect.disconnect(h);
						this._setNodeAdornersFocusAttr(true);
					});
				}
			});
		},
		
		enterStandardMode: function() {
			//
			//	summary:
			//		TODO TBD. Probably to be renamed also
			//    	
			this._interactors.switchTo("pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus");
		},	
		
		//
		//  move: [markup] boolean
		//      enable or disable moving entities
		//
		move: false,
		
		_setMoveAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getMoveAttr();
			if (old !== b) {
				this.move = b;
				this._interactors.get("move").setEnabled(b);
				this._firePropertyChanged('move', old, b);
			}
			return this;
		},
		
		_getMoveAttr: function() {
			return this._viewportReady ? this._interactors.get("move").getEnabled() : false;
		},
		
		// =======================================================================
		//
		// KEY INTERACTORS:
		//
		_keyInteractors: null,
		
		_initializeKeyInteractors: function() {
			//
			//	tags:
			//		private
			//
			this._keyInteractors = new KeyboardInteractorManager(this);
			
			this._keyInteractors.add(Navigator.KeyInteractorId, new Navigator().initialize(this));
			this._keyInteractors.add(SelectionKeyInteractor.KeyInteractorId, new SelectionKeyInteractor().initialize(this));
			this._keyInteractors.add(PanKeyInteractor.KeyInteractorId, new PanKeyInteractor().initialize(this));
			this._keyInteractors.add(ZoomKeyInteractor.KeyInteractorId, new ZoomKeyInteractor().initialize(this));
			this._keyInteractors.add(MoveKeyInteractor.KeyInteractorId, new MoveKeyInteractor().initialize(this));
			var edition = new SwitchModeInteractor();
			this._keyInteractors.add(SwitchModeInteractor.KeyInteractorId, edition.initialize(this));
			edition.enterNavigation();
			this._keyInteractors.activate();
		},
		
		getKeyInteractor: function(id) {
			// summary: Gets the key interactor with the specified id.
			return this._keyInteractors.get(id);
		},
	
		assignKeyInteractorInputs: function(iid, assignments) {
			// summary: Changes assignments of keys for a given interactor.
			// example: Change the multi-selection key to Shift+Spacebar:
			//			var assignment = {
			//				multiSelect:{
			//					hotKey: dojo.keys.SPACE,
            //					filter: {alt:false,shift:true}
            //					}
            //				};
			//			diagram.assignKeyInteractorInputs("Selection",assignment);
			this.getKeyInteractor(iid).assignInputs(assignments);
		},
		
		getKeyboardEventSource: function() {
			var wai = this._getWaiNode(this._graph);
			return wai || this.focusNode || this.domNode;
		},
		
		// =======================================================================
		//
		//	Adorners
		//
	
		_initializeAdorners: function() {
			var v = this.getViewport();
			
			this._adornerTemplates = ibm_ilog.diagram.templates.defaultAdornerTemplates.clone();
			
			v.whenLoaded(lang.hitch(this, function() {
				this._adornerLayer = v.getOverlayLayer().createGroup();
			}));
		},
		
		
		_createAdorners: function() {
			this._focusedNodeAdorner = this.createAdorner(Adorner, this._adornerTemplates, "NodeFocusAdorner");
			
			this._focusedLinkAdorner = this.createAdorner(LinkAdorner, this._adornerTemplates, "LinkFocusAdorner");
	
		},
		
		// =======================================================================
		//
		//	WAI-ARIA / A11Y
		//
		
		tabIndex: "-1",
		
		//
		//	disableGraphWai: boolean
		//		Disables the infrastructure used for assigning WAI roles. (Initialization-only attribute).
		//        
		disableGraphWai: false,
		
		//
		//	enableGraphWaiDescribedBy: boolean
		//		Used to enable description for wai structure
		//
		enableGraphWaiDescribedBy: false,
		
		//
		//	waiNodeLabelFunction: Function?
		//		This function is used to compute the WAI label text for a node.
		//
		waiNodeLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiLinkLabelFunction: Function?
		//		This function is used to compute the WAI label text for a link.
		//
		waiLinkLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiSubgraphLabelFunction: Function?
		//		This function is used to compute the WAI label text for a subgraph.
		//
		waiSubgraphLabelFunction: _defaultWaiLabelFunction,
		
		//	waiGraphLabelFunction: Function?
		//		This function is used to compute the WAI label for the graph.
		//
		waiGraphLabelFunction: function(graph) {
			return "Graph";
		},
		
		
		
		//
		//	waiNodeRoleFunction: Function?
		//		This function is used to compute the WAI role for a node.
		//
		waiNodeRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiLinkRoleFunction: Function?
		//		This function is used to compute the WAI role for a link.
		//
		waiLinkRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiSubgraphRoleFunction: Function?
		//		This function is used to compute the WAI role for a subgraph.
		//
		waiSubgraphRoleFunction: function(ge) {
			return "group";
		},
		
		//	waiGraphRoleFunction: Function?
		//		This function is used to compute the WAI role for the graph.
		//
		waiGraphRoleFunction: function(graph) {
			return "list";
		},
		
		
		
		//
		//	waiNodeDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a node.
		//
		waiNodeDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiLinkDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a link.
		//
		waiLinkDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiSubgraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a subgraph.
		//
		waiSubgraphDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//	waiGraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description for the graph.
		//
		waiGraphDescriptionFunction: function(graph) {
			return "";
		},
		
		
		
		_bindWaiNode: function( /*ibm_ilog.diagram.GraphElement|ibm_ilog.diagram.Graph*/ge, /*div*/ waiNode, /*label*/ waiDesc) {
			//
			//	summary:	
			//		Adds private WAI information to graph element and the WAI node. 
			//
			//	tags:
			//		private
			//
			ge.__diagramWaiNode = waiNode;
			ge.__diagramWaiDesc = waiDesc;
			waiNode.__graphElement = ge;
		},
		
		_onWaiFocus: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has received focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_onWaiBlur: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has lost focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_waiInvisibleStyle: {
			position: 'absolute',
			left: '0px',
			top: '0px',
			width: '100%',
			height: '100%',
			overflow:"visible"
		},
		
		_initializeGraphWai: function() {
			//
			//	summary:
			//		Initializes the infrastructure to support WAI screen reading on the graph elements.
			//
			//	tags:
			//		private
			//
			
			// when the widget receives focus, forward to appropiate WAI node 
			this._connections.push(connect.connect(this.domNode, "onfocus", this, "_onDiagramFocus"));
			
			// create root node at a hidden position
			var waiRoot = domctr.create('div', {style:{
				position: 'absolute',
				left: '0px',
				top: '0px',
				width: '100%',
				height: '100%',
				overflow: "visible",
				'z-index': "-2"
			}}, this.domNode, "first");
			
			this._createWaiNode(this._graph, this.id, waiRoot, this.waiGraphRoleFunction(this._graph));
			
			this._graph.__diagramWaiNode.setAttribute('tabIndex','0');
			
			this._setWaiState(this._graph.__diagramWaiNode, "label", this.waiGraphLabelFunction(this._graph));
			if (this.enableGraphWaiDescribedBy) {
				this._graph.__diagramWaiDesc.innerHTML = this.waiGraphDescriptionFunction(this._graph);
			}
			
			// connect to onfocus/onblur
			if (!has("ie")) {
				waiRoot.addEventListener("focus", lang.hitch(this, "_onWaiFocus"), true /*capturing phase*/);
				waiRoot.addEventListener("blur", lang.hitch(this, "_onWaiBlur"), true /*capturing phase*/);
			} else {
				// use focusin focusout which DOES bubble up (IE does NOT support capturing phase)
				this._connections.push(connect.connect(waiRoot, "focusin", lang.hitch(this, "_onWaiFocus")));
				this._connections.push(connect.connect(waiRoot, "focusout", lang.hitch(this, "_onWaiBlur")));
			}
			
			this._connections.push(connect.connect(this, "onNodeCreated", this, "_registerNodeWai"));
			this._connections.push(connect.connect(this, "onSubgraphCreated", this, "_registerSubgraphWai"));
			this._connections.push(connect.connect(this, "onLinkCreated", this, "_registerLinkWai"));
			
			this._connections.push(connect.connect(this, "onFocusedElementChanged", this, "_updateElementWaiFocus"));
			//
			this._connections.push(connect.connect(this.getSelection(), "onSelectionChanged", this, "_updateSelectedState"));
			
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/beforeDisposing', this, "_destroyWaiNode"));
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/refreshWaiLabel', this, "_updateWaiLabel"));
			
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/labelChanged', this, "_updateWaiLabel"));
			
			this._subscriptions.push(connect.subscribe("/ibm_diagram/Subgraph/collapsedChanged", this, "_updateSubgraphCollapsed"));
		},
		
		_getWaiNode: function(e) {
			//
			//	summary:
			//		Returns the WAI node associated with a graph element.
			//
			//	tags:
			//		private
			//
			return e.__diagramWaiNode;
		},
		
		_onDiagramFocus: function(subgraph) {
			//
			//	summary:
			//		Used when the widget receives HTML focus. Forwards it to the current focused element.
			//
			//	tags:
			//		private
			//
			//	TODO: check this is needed (maybe tabTndex is always enough).
			//
			var wainode = this._getWaiNode(this._focusedElement|| this._graph)
			if(wainode) {
				focus.focus(wainode);
			}
		},
		
		_updateSubgraphCollapsed: function(subgraph) {
			//
			//	summary:
			//		Used when a subgraph collapsed state has changed. Updates the corresponding WAI property. 
			//
			//	tags:
			//		private
			//
			var wainode = this._getWaiNode(subgraph);
			if(wainode) {
				this._setWaiState(wainode, "expanded", !subgraph.isCollapsed());
			}
		},
		
		_updateSelectedState: function(selected, deselected) {
			//
			//	summary:
			//		Used a graph element is selected or deselected. Updates the correspondig WAI attribute.
			//
			//	tags:
			//		private
			//
			var selection = this.getSelection();
			array.forEach(deselected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						this._setWaiState(wai, "selected", false);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
			array.forEach(selected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						this._setWaiState(wai, "selected", true);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
		},
		
		_updateElementWaiFocus: function(old, current) {
			//
			//	summary:
			// 		Change wai focus to node associated with the current focused element position
			//
			//	tags:
			//		private
			//
			var oldWai = (old && this._getWaiNode(old)) || this._getWaiNode(this._graph);
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);

			//	TODO: [av] try aria-activedescendant instead?

			oldWai.setAttribute("tabIndex", "-1");
			curWai.setAttribute("tabIndex", "0");

			focus.focus(curWai);
		},

		_htmlRefocus: function() {
			//
			//	summary:
			//		Set the HTML focus into the appropiate component based on the current widget state.
			//
			var current = this.getFocusedElement();  
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);
			focus.focus(curWai);
		},
		
		_getFunctionTriad: function(pre, post) {
			//
			//	summary:
			// 		helper. returns a {N:L:S:} for use with GraphUtil.dispatchClass().
			//
			//	tags:
			//		private
			//
			if (pre && pre.length) {
				return {
					N: this[pre + "Node" + post + "Function"],
					L: this[pre + "Link" + post + "Function"],
					S: this[pre + "Subgraph" + post + "Function"]
				};
			} else {
				return {
					N: this["node" + post + "Function"],
					L: this["link" + post + "Function"],
					S: this["subgraph" + post + "Function"]
				};
			}
		},
		
		_updateWaiDescription: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI description text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiDescription) {
				ge.__diagramWaiDescription.innerHTML = gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Description"));
			}
		},
		
		_updateWaiLabel: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI label text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiNode) {
				this._setWaiState(ge.__diagramWaiNode, 'label', gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Label")));
			}
		},
		
		_createWaiNode: function(element, id, waiParent, role) {
			//
			//	summary:
			//		create node div for WAI
			//
			//	tags:
			//		private
			//
			var waiNodeId = id + "_waiNode";
			var waiLabelId = id + "_waiLabel";
			var waiDescId = id + "_waiDesc";
			
			var atts = lang.mixin({
				id: waiNodeId,
				tabIndex: -1
			}, {style:this._waiInvisibleStyle});
			
			var geWaiNode = domctr.create("div", atts, waiParent);
			var geWaiDesc;
			if (this.enableGraphWaiDescribedBy) {
				geWaiDesc = domctr.create("label", lang.mixin({
					id: waiDescId,
					'for': waiNodeId
				}, {style:this._waiInvisibleStyle}), waiParent);
			}
			
			domattr.set(geWaiNode, "role", role);
			
			if (this.enableGraphWaiDescribedBy) {
				this._setWaiState(geWaiNode, 'describedBy', waiDescId);
			}
			
			this._bindWaiNode(element, geWaiNode, geWaiDesc);
		},
		
		_registerGraphElementWai: function(ge, role) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Graph Element
			//
			//	tags:
			//		private
			//
			
			// get div parent of parent
			
			var container = ge.getParent(), parentWaiNode;
			
			if (!container || !container.__diagramWaiNode) {
				// R.warn("MissingWaiNode", container);
				return;
			}
			
			parentWaiNode = container.__diagramWaiNode;
			
			// create node div for WAI
			this._createWaiNode(ge, ge.getId(), parentWaiNode, role);
			
			this._updateWaiLabel(ge);
			this._updateWaiDescription(ge);
		},
		
		_destroyWaiNode: function(ge) {
			if (ge.__diagramWaiLabel) {
				domctr.destroy(ge.__diagramWaiLabel);
			}
			if (ge.__diagramWaiNode) {
				domctr.destroy(ge.__diagramWaiNode);
			}
		},
		
		_registerLinkWai: function(link) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Link
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(link, this.waiLinkRoleFunction());
			
		},
		
		_registerNodeWai: function(node) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Node
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(node, this.waiNodeRoleFunction());
			
		},
		
		_registerSubgraphWai: function(subgraph) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Subgraph
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(subgraph, this.waiSubgraphRoleFunction());
			
			this._updateSubgraphCollapsed(subgraph);
			subgraph.getGraph().__diagramWaiNode = subgraph.__diagramWaiNode;
			
		},
		
		_setWaiState: function(node, state, value){
			node.setAttribute("aria-"+state, value);
		}
	};
	
	return _Diagram;

});
