/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../circular/galg/ClusteringGraph",	// #5
  "../../internalutil/GraphModelUtil",	// #6
  "../../internalutil/HashTable",	// #7
  "../../internalutil/TranslateUtil",	// #8
  "../../IIterator"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    ClusteringGraph,	// #5
    GraphModelUtil,	// #6
    HashTable,	// #7
    TranslateUtil,	// #8
    IIterator	// #9
    ) {

var CLAutoClustering = declare("ibm_ilog.graphlayout.circular.clayout.CLAutoClustering", null, 
{
  // summary:
  //		The algorithm to calculate the graph of clusters automatically from
  //		the topology. 
  // description:
  //		The algorithm to calculate the graph of clusters automatically from
  //		the topology. 
  //		Specified clusters are respected if possible, but each node can only
  //		belong to one cluster. Nodes that have no clusters are partitioned as
  //		needed.   
  // tags:
  //		private
  
  _model: null,
  _graphOfClusters: null,
  _counter: null,

  constructor : function(graphOfClusters, model)
  {
    this._graphOfClusters = graphOfClusters;
    this._model = model;
  },
  
  run : function(layout)
  {
    // summary:
    //		Runs the algorithm. 
    // description:
    //		Runs the algorithm. 
    //		If clusters the nodes and adds the corresponding clusters to the
    //		graph of clusters. It does not handle the links in graph of
    //		clusters.
    // tags:
    //		private
    
    var cgraph = this.createClusteringGraph();
    if (layout.isClusterByBiconnectivity()) {
      cgraph.clusterBiconnectedComponents();
    }
    cgraph.splitLargeClusters(layout.getMaximumClusterSize(), layout.getMaximumNumberOfIterationsToReachMaxClusterSize());
    cgraph.clusterHighDegreeNodes(layout.getHighDegreeStarClusterSize());
    cgraph.combineSmallClusters(layout.getMinimumClusterSize());

    return cgraph; // ibm_ilog.graphlayout.circular.galg.ClusteringGraph
  },

  getClusterId : function(node)
  {
    // summary:
    //		Returns the cluster id of the node. 
    // description:
    //		Returns the cluster id of the node. 
    //		Automatic clustering does not support nodes that have multiple
    //		cluster ids, hence it returns just one id, or null if there is none.
    // tags:
    //		private
    
    return null; // ibm_ilog.graphlayout.circular.CircularClusterId
  },

  getStarCenter : function(node)
  {
    // summary:
    //		Returns whether the node is specified to be star center. 
    // description:
    //		Returns whether the node is specified to be star center. 
    //		This can be TAG&#95;TRUE or TAG&#95;FALSE.
    // tags:
    //		private
    
    return CLAutoClustering.TAG_FALSE; // int
  },

  getIndex : function(node)
  {
    // summary:
    //		Returns the index if specified of the node within the cluster.
    // tags:
    //		private
    
    return -1;
    
  },

  createClusteringGraph : function()
  {
    // summary:
    //		Creates the clustering graph.
    // tags:
    //		private
    
    var cgraph = new ClusteringGraph();
    var clusterIdToNodeArrayList = new HashTable();

    var e = new TranslateUtil.ArrayIterator(this._model.getNodes());

    while (e.hasNext())
    {
      var node = e.next();
      cgraph.addNode(node);
      var id = this.getClusterId(node);

      if (id != null)
      {
        var nodesInCluster = clusterIdToNodeArrayList.item(id);

        if (nodesInCluster == null)
        {
          nodesInCluster = new Array();
          clusterIdToNodeArrayList.add(id, nodesInCluster);
        }

        nodesInCluster.push(node);
      }
    }

    e = new TranslateUtil.ArrayIterator(this._model.getLinks());

    while (e.hasNext())
    {
      var link = e.next();
      var from = this._model.getFrom(link);
      var toobj = this._model.getTo(link);
      cgraph.addLink(link, from, toobj);
    }

    var iter = clusterIdToNodeArrayList.getIterator();

    while (iter.hasNext())
    {
      var entry = iter.next();
      var nodes = entry.value;
      this.createClusteringGraphForNodes(nodes, cgraph);
    }

    cgraph.refresh();
    return cgraph; // ibm_ilog.graphlayout.circular.galg.ClusteringGraph
  },

  createClusteringGraphForNodes : function(nodes, cgraph)
  {
    var n = nodes.length;
    if (n <= 1) {
      return;
    }
    var cnode = cgraph.collapse(new CLAutoClustering.InternalNodeIterator(cgraph,  new TranslateUtil.ArrayIterator(nodes)), false);

    for (var i = 0; i < n; i++)
    {
      var node = nodes[i];

      if (this.getStarCenter(node) == CLAutoClustering.TAG_TRUE)
      {
        cgraph.setStarCenter(cnode, cgraph.getInternalNode(node));
      }
    }
  }

}
);

lang.mixin(CLAutoClustering,
{
  TAG_FALSE : null,
  TAG_TRUE : null,
  _fields_init : function(c)
  {
    c = c || CLAutoClustering;
    
    c.TAG_FALSE = 0;
    c.TAG_TRUE = 1;
  }
  
}
);

CLAutoClustering.InternalNodeIterator = declare( IIterator, 
{
  _cgraph: null,
  _nodes: null,

  constructor : function(cgraph, nodes)
  {
    this._cgraph = cgraph;
    this._nodes = nodes;
  },
  
  hasNext : function()
  {
    // summary:
    // tags:
    //		private
    
    return this._nodes.hasNext(); // Boolean
  },

  next : function()
  {
    // summary:
    // tags:
    //		private
    
    var node = this._nodes.next();
    return this._cgraph.getInternal(node); // Object
  },

  remove : function()
  {
    // summary:
    // tags:
    //		private
    
  }

}
);
(function(){ CLAutoClustering._fields_init(); })();

return CLAutoClustering;
});
