/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../circular/clayout/CLANode",	// #5
  "../../circular/galg/BiconnAlgorithm",	// #6
  "../../circular/galg/BiconnModel",	// #7
  "../../internalutil/GraphModelUtil",	// #8
  "../../internalutil/PriorityQueue",	// #9
  "../../internalutil/TranslateUtil"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    CLANode,	// #5
    BiconnAlgorithm,	// #6
    BiconnModel,	// #7
    GraphModelUtil,	// #8
    PriorityQueue,	// #9
    TranslateUtil	// #10
    ) {

var CLCrossingReduction = declare("ibm_ilog.graphlayout.circular.clayout.CLCrossingReduction", null, 
{
  // summary:
  //		The algorithm to calculate an ordering of the nodes of a cluster so
  //		that the number of links crossings is reduced. 
  // description:
  //		The algorithm to calculate an ordering of the nodes of a cluster so
  //		that the number of links crossings is reduced. 
  //		Thr crossing reduction is only a heuristic and does not necessarily
  //		produce the minimal result. Note that star center are only taken into
  //		account by circular sifting as postprocessing step. The
  //		outerplanarity is anyway done in a way so that star center crossings
  //		are seldom.  It works on CLANode, CLCluster and CLLink.   
  // tags:
  //		private
  
  _clusters: null,

  _maxIntraSiftingIterations : null,

  _maxInterSiftingIterations : null,

  _haveNodesWithSpecifiedIndex : null,

  _clustersHavePositions: null,

  _considerInterClusterLinks: null,

  _intraWeight : null,
  _interWeight : null,
  _mixedWeight : null,

  constructor : function(clusters)
  {
    this._maxIntraSiftingIterations = 10;
    this._maxInterSiftingIterations = 10;
    this._haveNodesWithSpecifiedIndex = false;
    this._intraWeight = 1;
    this._interWeight = 1;
    this._mixedWeight = 1;
    this._clusters = clusters;
  },
  
  setCrossWeights : function(intraWeight, interWeight, mixedWeight)
  {
    this._intraWeight = intraWeight;
    this._interWeight = interWeight;
    this._mixedWeight = mixedWeight;
  },

  run : function(clustersHavePositions, considerInterClusterLinks)
  {
    // summary:
    //		Runs the crossing reduction.
    // tags:
    //		private
    
    this._clustersHavePositions = clustersHavePositions;
    this._considerInterClusterLinks = considerInterClusterLinks;

    for (var i = 0; i < this._clusters.length; i++)
      this.orderCluster(this._clusters[i]);
  },

  finalStep : function(cluster, nodes)
  {
    // summary:
    //		The final step for each cluster. 
    // description:
    //		The final step for each cluster. 
    //		At that time point, the cluster must have its position already.
    // tags:
    //		private
    
    if (this._clustersHavePositions)
    {
      var clNodes = nodes;
      var startAngle = cluster.getBestRotationAngle(clNodes);
      cluster.placeNodes(clNodes, startAngle);
      cluster.saveOrdering(nodes);
      cluster.setLaidOut(true);
    }
    else
    {
      cluster.saveOrdering(nodes);
    }
  },

  orderCluster : function(cluster)
  {
    if (this.ensureSpecifiedIndexe(cluster))
    {
      return;
    }

    if (cluster.getOrdering() == null)
    {
      if (this.outerPlanar(cluster))
      {
      }
      else
      {
        this.circularInsert(cluster);
      }
    }

    if (this._clustersHavePositions && this._considerInterClusterLinks)
    {
      this.circularInterLinksSiftingVectNodes(cluster, cluster.getOrdering());
    }
  },

  ensureSpecifiedIndexe : function(cluster)
  {
    // summary:
    //		Orders the nodes according to specified indexe. 
    // description:
    //		Orders the nodes according to specified indexe. 
    //		Returns true if all nodes have specified indexe.
    // tags:
    //		private
    
    var n = cluster.size();
    var nodes = CLCrossingReduction.getNodes(cluster);
    var numNodesWithSpecIndex = 0;

    for (var i = 0; i < n; i++)
    {
      var node = nodes[i];
      var specIndex = node.getSpecIndex();

      if (specIndex >= 0)
      {
        node.setIndex(specIndex);
        numNodesWithSpecIndex++;
      }
    }

    this._haveNodesWithSpecifiedIndex = numNodesWithSpecIndex >= 2;

    if (numNodesWithSpecIndex == n)
    {
      nodes.sort(CLCrossingReduction.compareNodeIndex);
      this.finalStep(cluster, nodes);
      return true; // Boolean
    }

    return false; // Boolean
  },

  circularInsert : function(cluster)
  {
    // summary:
    //		The idea is that we successively add nodes into the circle. 
    // description:
    //		The idea is that we successively add nodes into the circle. 
    //		The already placed nodes form a continuous area of the circle. Hence
    //		a new node can only be placed at the left side or at the right side
    //		of the continuous area. In order to decide which side, we calculate
    //		how many crossings the current node would cause to links that go
    //		from placed nodes to unplaced nodes. The order in which we choose
    //		nodes to be inserted is relevant. We choose as next node of the
    //		sequence the unplaced nodes that has the most links to placed nodes,
    //		and from these the one that has the least number of links to
    //		unplaced nodes.
    // tags:
    //		private
    
    var n = cluster.size();
    var nodes = CLCrossingReduction.getNodes(cluster);
    var closingLinks = TranslateUtil.newArray(n, 0);

    var leftIndex = 0;
    var rightIndex = 0;
    var queue = new CLCrossingReduction.PrioQueue(n);

    var numNodesWithSpecIndex = 0;
    var i = 0;
    var node = null;

    for (i = 0; i < n; i++)
    {
      node = nodes[i];
      node.setPrioQueueIndex(-1);
      node.setPlacedNeighbors(0);
      node.setUnplacedNeighbors(node.getNumberOfIntraLinks());
      var specIndex = node.getSpecIndex();

      if (specIndex >= 0)
      {
        node.setIndex(specIndex);
        numNodesWithSpecIndex++;
      }
      else
      {
        node.setIndex(-1);
      }
    }

    var e1 = null;
    var e2 = null;
    var link = null;
    var other = null;

    if (numNodesWithSpecIndex > 0)
    {
      nodes.sort(CLCrossingReduction.compareNodeIndex);
      rightIndex = 0;
      leftIndex = n - numNodesWithSpecIndex - 1;

      for (i = leftIndex + 1; i < n; i++)
      {
        node = nodes[i];
        e2 = node.getIntraLinks();

        while (e2.hasNext())
        {
          link = e2.next();
          other = link.getCLAOtherEnd(node);
          other.incrPlacedNeighbors();
          other.decrUnplacedNeighbors();
        }
      }
    }

    for (i = 0; i < n; i++)
    {
      node = nodes[i];
      if (node.getIndex() < 0) {
        queue.add(node);
      }
    }

    while (!queue.isEmpty())
    {
      node = queue.popMin();
      node.setPrioQueueIndex(-1);

      if (leftIndex == rightIndex)
      {
        nodes[leftIndex] = node;
        node.setIndex(leftIndex);
        leftIndex--;
        rightIndex++;
      }
      else
      {
        e1 = node.getIntraLinks();

        while (e1.hasNext())
        {
          link = e1.next();
          other = link.getCLAOtherEnd(node);
          if (other.getIndex() >= 0) {
            closingLinks[other.getIndex()]++;
          }
        }

        var numRightCrossings = 0;
        var sumClosingLinks = 0;
        i = (leftIndex + 1) % n;

        while (i != rightIndex)
        {
          numRightCrossings += (sumClosingLinks * nodes[i].getUnplacedNeighbors());
          sumClosingLinks += closingLinks[i];
          i++;
          if (i >= n) {
            i = 0;
          }
        }

        var numLeftCrossings = 0;
        sumClosingLinks = 0;
        i = rightIndex - 1;
        if (i < 0) {
          i = n - 1;
        }
        while (i != leftIndex)
        {
          numLeftCrossings += (sumClosingLinks * nodes[i].getUnplacedNeighbors());
          sumClosingLinks += closingLinks[i];
          closingLinks[i] = 0;

          i--;
          if (i < 0) {
            i = n - 1;
          }
        }

        if (numLeftCrossings < numRightCrossings)
        {
          nodes[leftIndex] = node;
          node.setIndex(leftIndex);
          leftIndex--;
        }
        else
        {
          nodes[rightIndex] = node;
          node.setIndex(rightIndex);
          rightIndex++;
        }
      }

      e2 = node.getIntraLinks();

      while (e2.hasNext())
      {
        link = e2.next();
        other = link.getCLAOtherEnd(node);
        other.incrPlacedNeighbors();
        other.decrUnplacedNeighbors();
        queue.updateObject(other);
      }

      if (leftIndex < 0) {
        leftIndex = n - 1;
      }
      if (rightIndex >= n) {
        rightIndex = 0;
      }
    }

    if (this._maxIntraSiftingIterations > 0) {
      this.nodesCircularSiftingVectNodes(nodes);
    }
    this.finalStep(cluster, nodes);
  },

  nodesCircularSiftingVectNodes : function(nodes)
  {
    // summary:
    //		
    // tags:
    //		private
    
    if (this._maxIntraSiftingIterations == 0) {
      return;
    }
    var numCrossings = CLCrossingReduction.getClusterIntraCrossings(nodes);
    if (numCrossings == 0) {
      return;
    }

    var aux = TranslateUtil.newArray(nodes.length, null);
    TranslateUtil.arrayCopy(nodes, 0, aux, 0, nodes.length);
    var counter = this._maxIntraSiftingIterations;

    var oldNumCrossings = numCrossings + 1;

    while (numCrossings < oldNumCrossings && counter > 0)
    {
      counter--;
      oldNumCrossings = numCrossings;
      numCrossings = this.circularSiftingOneRound(nodes, aux, oldNumCrossings);
      if (numCrossings == 0) {
        return;
      }
    }
  },

  circularSiftingOneRound : function(nodes, backup, numCrossings)
  {
    // summary:
    //		Performs one round of circular sifting. 
    // description:
    //		Performs one round of circular sifting. 
    //		Returns the new number of crossings.  
    // tags:
    //		private
    
    for (var i = 0; i < backup.length; i++)
      numCrossings = this.circularSiftingOneNode(backup[i], nodes, numCrossings);
    return numCrossings; // int
  },

  circularSiftingOneNode : function(node, nodes, numCrossings)
  {
    // summary:
    //		The sifting of one node. 
    // description:
    //		The sifting of one node. 
    //		If we have nodes with specified index, then these nodes are already
    //		in order when starting the sifting. We are allowed to place free
    //		nodes in between. Hence, if we never move nodes with specified
    //		index, the nodes of specified index remain in order.  
    // tags:
    //		private
    
    if (node.getSpecIndex() >= 0) {
      return numCrossings; // int
    }

    var n = nodes.length;
    var start = node.getIndex();
    var end = start - 1;
    if (end < 0) {
      end = nodes.length - 1;
    }
    var a = start;
    var b;
    var crossingsAB;
    var crossingsBA;
    var bestIndex = start;
    var bestNumCrossings = numCrossings;
    var aux;

    while (a != end)
    {
      b = (a + 1) % nodes.length;
      crossingsAB = CLCrossingReduction.getIntraCrossings(nodes[a], nodes[b], n);
      aux = nodes[a];
      nodes[a] = nodes[b];
      nodes[b] = aux;
      nodes[a].setIndex(a);
      nodes[b].setIndex(b);
      crossingsBA = CLCrossingReduction.getIntraCrossings(nodes[a], nodes[b], n);
      numCrossings += crossingsBA - crossingsAB;

      if (numCrossings < bestNumCrossings)
      {
        bestIndex = b;
        bestNumCrossings = numCrossings;
      }

      a = b;
    }

    var i = 0;

    if (a < bestIndex)
    {
      bestIndex--;

      if (a != bestIndex)
      {
        for (i = a; i < bestIndex; i++)
        {
          nodes[i] = nodes[i + 1];
          nodes[i].setIndex(i);
        }

        nodes[bestIndex] = node;
        node.setIndex(bestIndex);
      }
    }
    else if (a > bestIndex)
    {
      for (i = a; i > bestIndex; i--)
      {
        nodes[i] = nodes[i - 1];
        nodes[i].setIndex(i);
      }

      nodes[bestIndex] = node;
      node.setIndex(bestIndex);
    }

    return bestNumCrossings; // int
  },

  _bestStartAngle: null,
  _bestRotationPenalty: null,
  _bestIntraCrossingPenalty: null,
  _bestInterCrossingPenalty: null,

  circularInterLinksSiftingVectNodes : function(cluster, nodes)
  {
    // summary:
    //		
    // tags:
    //		private
    
    if (this._maxInterSiftingIterations == 0) {
      return;
    }
    cluster.setLaidOut(true);
    cluster.fillLinkArrays();

    if (!cluster.hasExtraLinks()) {
      return;
    }

    var clNodes = nodes;
    this._bestStartAngle = cluster.getBestRotationAngle(clNodes);
    cluster.placeNodes(clNodes, this._bestStartAngle);
    this._bestRotationPenalty = cluster.getRotationPenalty(true);
    this._bestIntraCrossingPenalty = this._intraWeight * CLCrossingReduction.getClusterIntraCrossings(nodes);
    this._bestInterCrossingPenalty = cluster.getCrossingPenalty(0.0, this._interWeight, this._mixedWeight);

    if (this._bestIntraCrossingPenalty == 0.0 && this._bestInterCrossingPenalty == 0.0 && this._bestRotationPenalty == 0.0) {
      return;
    }
    this.mirror(clNodes);
    var startAngle = cluster.getBestRotationAngle(clNodes);
    cluster.placeNodes(clNodes, startAngle);
    var rotationPenalty = cluster.getRotationPenalty(true);
    var intraCrossingPenalty = this._intraWeight * CLCrossingReduction.getClusterIntraCrossings(nodes);
    var interCrossingPenalty = cluster.getCrossingPenalty(0.0, this._interWeight, this._mixedWeight);

    var crossingPenalty = intraCrossingPenalty + interCrossingPenalty;
    var bestCrossingPenalty = this._bestIntraCrossingPenalty + this._bestInterCrossingPenalty;

    if ((crossingPenalty < bestCrossingPenalty) || ((crossingPenalty == bestCrossingPenalty) && (rotationPenalty < this._bestRotationPenalty)))
    {
      this._bestStartAngle = startAngle;
      this._bestRotationPenalty = rotationPenalty;
      this._bestIntraCrossingPenalty = intraCrossingPenalty;
      this._bestInterCrossingPenalty = interCrossingPenalty;

      if (this._bestIntraCrossingPenalty == 0.0 && this._bestInterCrossingPenalty == 0.0 && this._bestRotationPenalty == 0.0) {
        return;
      }
    }
    else
    {
      this.mirror(clNodes);
      CLCrossingReduction.ensureIndexe(nodes);
      cluster.placeNodes(clNodes, this._bestStartAngle);
    }

    var aux = TranslateUtil.newArray(nodes.length, null);
    TranslateUtil.arrayCopy(nodes, 0, aux, 0, nodes.length);
    var counter = this._maxInterSiftingIterations;

    var changed = true;

    while (changed && counter > 0)
    {
      counter--;
      changed = this.circularInterLinksSiftingOneRound(cluster, nodes, aux);

      if (this._bestIntraCrossingPenalty == 0.0 && this._bestInterCrossingPenalty == 0.0 && this._bestRotationPenalty == 0.0) {
        return;
      }
    }

    cluster.cleanLinkArrays();
  },

  circularInterLinksSiftingOneRound : function(cluster, nodes, backup)
  {
    // summary:
    //		Performs one round of circular sifting with intercluster links. 
    // description:
    //		Performs one round of circular sifting with intercluster links. 
    //		Returns whether anything has changed.  
    // tags:
    //		private
    
    var changed = false;
    var i = 0;

    for (i = 0; i < backup.length; i++)
    {
      if (backup[i].hasExtraLinks()) {
        changed = this.circularInterLinksSiftingOneNode(cluster, backup[i], nodes) || changed;
      }
    }

    if (changed)
    {
      for (i = 0; i < backup.length; i++)
      {
        if (!backup[i].hasExtraLinks()) {
          changed = this.circularInterLinksSiftingOneNode(cluster, backup[i], nodes) || changed;
        }
      }
    }

    return changed; // Boolean
  },

  circularInterLinksSiftingOneNode : function(cluster, node, nodes)
  {
    // summary:
    //		The sifting of one node with intercluster links. 
    // description:
    //		The sifting of one node with intercluster links. 
    //		If we have nodes with specified index, then these nodes are already
    //		in order when starting the sifting. We are allowed to place free
    //		nodes in between. Hence, if we never move nodes with specified
    //		index, the nodes of specified index remain in order.   
    // tags:
    //		private
    
    if (node.getSpecIndex() >= 0) {
      return false; // Boolean
    }

    var n = nodes.length;
    var start = node.getIndex();
    var end = start - 1;
    if (end < 0) {
      end = nodes.length - 1;
    }
    var a = start;
    var b;
    var crossingsAB;
    var crossingsBA;
    var bestIndex = start;
    var startAngle = this._bestStartAngle;
    var rotationPenalty = this._bestRotationPenalty;
    var intraCrossingPenalty = this._bestIntraCrossingPenalty;
    var interCrossingPenalty = this._bestInterCrossingPenalty;
    var crossingPenalty;
    var bestCrossingPenalty = intraCrossingPenalty + interCrossingPenalty;
    var changed = false;

    var aux;
    var clNodes = nodes;

    while (a != end)
    {
      b = (a + 1) % nodes.length;
      crossingsAB = CLCrossingReduction.getIntraCrossings(nodes[a], nodes[b], n);
      aux = nodes[a];
      nodes[a] = nodes[b];
      nodes[b] = aux;
      nodes[a].setIndex(a);
      nodes[b].setIndex(b);
      crossingsBA = CLCrossingReduction.getIntraCrossings(nodes[a], nodes[b], n);

      intraCrossingPenalty += this._intraWeight * (crossingsBA - crossingsAB);

      startAngle = cluster.getBestRotationAngle(clNodes);
      cluster.placeNodes(clNodes, startAngle);
      rotationPenalty = cluster.getRotationPenalty(true);
      interCrossingPenalty = cluster.getCrossingPenalty(0.0, this._interWeight, this._mixedWeight);
      crossingPenalty = intraCrossingPenalty + interCrossingPenalty;

      if ((crossingPenalty < bestCrossingPenalty) || ((crossingPenalty == bestCrossingPenalty) && (rotationPenalty < this._bestRotationPenalty)))
      {
        this._bestRotationPenalty = rotationPenalty;
        this._bestStartAngle = startAngle;
        this._bestIntraCrossingPenalty = intraCrossingPenalty;
        this._bestInterCrossingPenalty = interCrossingPenalty;
        bestCrossingPenalty = crossingPenalty;
        bestIndex = b;

        changed = true;
      }

      a = b;
    }

    if (a != bestIndex)
    {
      var i = a;

      while (i != bestIndex)
      {
        var j = i - 1;
        if (j < 0) {
          j = n - 1;
        }
        nodes[i] = nodes[j];
        nodes[i].setIndex(i);
        i = j;
      }

      nodes[bestIndex] = node;
      node.setIndex(bestIndex);
      cluster.placeNodes(clNodes, this._bestStartAngle);
    }

    return changed; // Boolean
  },

  mirror : function(nodes)
  {
    // summary:
    //		Mirrors, i.e. 
    // description:
    //		Mirrors, i.e. 
    //		reverses the nodes.
    // tags:
    //		private
    
    var n = nodes.length;
    var m = Math.floor(n / 2);

    for (var i = 0; i < m; i++)
    {
      var node = nodes[i];
      nodes[i] = nodes[n - 1 - i];
      nodes[n - 1 - i] = node;
    }
  },

  outerPlanar : function(cluster)
  {
    // summary:
    //		Returns true if the cluster is outerplanar. 
    // description:
    //		Returns true if the cluster is outerplanar. 
    //		In this case, it also orders the node to achieve a planar embedding.
    //		If it returns false, the cluster is not laid out.
    // tags:
    //		private
    
    if (this._haveNodesWithSpecifiedIndex) {
      return false; // Boolean
    }

    var nodes = CLCrossingReduction.getNodes(cluster);

    for (var i = 0; i < nodes.length; i++)
      nodes[i].setIndex(0);

    var alg = new BiconnAlgorithm();
    var rootBlocks = alg.getBlockTrees(new CLCrossingReduction.ClusterBiconnModel(cluster));

    var index = 0;
    index = this.layoutBlocks(rootBlocks, index, alg);

    if (index < 0)
    {
      return false; // Boolean
    }

    nodes.sort(CLCrossingReduction.compareNodeIndex);
    if (cluster.getStarCenter() != null)
    {
      this.nodesCircularSiftingVectNodes(nodes);
    }

    this.finalStep(cluster, nodes);

    return true; // Boolean
  },

  layoutBlocks : function(rootBlocks, index, alg)
  {
    // summary:
    //		Layout a forest of block trees of biconnected components. 
    // description:
    //		Layout a forest of block trees of biconnected components. 
    //		Returns the next free node index. Returns -1, if any component is
    //		not outerplanar.
    // tags:
    //		private
    
    if (rootBlocks == null) {
      return index; // int
    }
    var iter =  new TranslateUtil.ArrayIterator(rootBlocks);

    while (iter.hasNext())
    {
      var block = iter.next();
      index = this.layoutBlockTree(block, index, alg);
      if (index < 0) {
        break;
      }
    }

    return index; // int
  },

  layoutBlockTree : function(block, index, alg)
  {
    // summary:
    //		Layout a block tree of biconnected components. 
    // description:
    //		Layout a block tree of biconnected components. 
    //		Returns the next free node index. Returns -1, if any component is
    //		not outerplanar.
    // tags:
    //		private
    
    var nodes = this.getNodesOfBiconnBlock(block);
    if (!(this.biconnectedOuterPlanar(nodes))) {
      return -1; // int
    }
    var i = 0;
    var auxindex = 0;

    for (i = 0; i < nodes.length; i++)
      nodes[i].setIndex(i);
    nodes = this.getOrderedNodes(block);

    if (block.isRootPure())
    {
      auxindex = index++;
      index = this.layoutBlocks(alg.getReachableBiconnectivityBlocks(nodes[0]), index, alg);
      nodes[0].setIndex(auxindex);
      if (index < 0) {
        return index; // int
      }
    }

    for (i = 1; i < nodes.length; i++)
    {
      auxindex = index++;
      index = this.layoutBlocks(alg.getReachableBiconnectivityBlocks(nodes[i]), index, alg);
      nodes[i].setIndex(auxindex);
      if (index < 0) {
        return index; // int
      }
    }

    return index; // int
  },

  biconnectedOuterPlanar : function(nodes)
  {
    // summary:
    //		Returns true if the array of input nodes form a biconnected,
    //		outerplanar component. 
    // description:
    //		Returns true if the array of input nodes form a biconnected,
    //		outerplanar component. 
    //		In this case, it also orders the node so that the embedding is
    //		planar.
    // tags:
    //		private
    
    var aux = TranslateUtil.newArray(nodes.length, null);
    TranslateUtil.arrayCopy(nodes, 0, aux, 0, nodes.length);
    var okay = this.biconnectedOuterPlanarImpl(aux);

    if (okay)
    {
      TranslateUtil.arrayCopy(aux, 0, nodes, 0, nodes.length);
    }

    for (var i = 0; i < nodes.length; i++)
      nodes[i].clearOuterPlanarData();
    return okay; // Boolean
  },

  biconnectedOuterPlanarImpl : function(nodes)
  {
    // summary:
    //		Outerplanar embedding of a biconnected component. 
    // description:
    //		Outerplanar embedding of a biconnected component. 
    //		Notice that it does not work if the set of input nodes is not a
    //		biconnected component. Each outerplanar biconnected component has at
    //		least one node with exactly two neighbors (a 2-node). If we cut this
    //		node and connect its 2 neighbors by an edge, the remaining graph
    //		must also be biconnected outerplanar. Hence the embedding/planarity
    //		test works recursively. We cut 2-nodes until the remaining is a pair
    //		of nodes. (In fact, the smallest biconnected nontrivial outerplanar
    //		graph is a triangle, and cutting the last 2-node of a triangle
    //		results in the pair of nodes). If at any time point we find no
    //		2-node, the graph is not biconnected outerplanar. Returns true if
    //		the graph is biconnected outerplanar.
    // tags:
    //		private
    
    var nodeSet = new Array();
    var i = 0;

    for (i = 0; i < nodes.length; i++)
      nodes[i].setIndex(-27);

    for (i = 0; i < nodes.length; i++)
    {
      nodeSet.push(nodes[i]);
      var e = nodes[i].getIntraLinks();

      while (e.hasNext())
      {
        var link = e.next();

        if (link.getCLASource() == nodes[i] && link.getCLATarget().getIndex() == -27)
        {
          CLANode.addRealEdge(link.getCLASource(), link.getCLATarget());
        }
      }
    }

    var node;

    while (nodeSet.length > 2)
    {
      node = this.getDegree2Node(nodeSet);
      if (node == null) {
        return false; // Boolean
      }
      TranslateUtil.removeArrayObject(nodeSet, node);
      var neighbor1 = null;
      var neighbor2 = null;
      var nr = node.getNumRealNeighbors();

      if (nr > 0)
      {
        neighbor1 = node.getRealNeighbor(0);
        if (CLANode.setOrderingNeighbors(node, neighbor1) == false) {
          return false; // Boolean
        }
        if (nr > 1)
        {
          neighbor2 = node.getRealNeighbor(1);
          if (CLANode.setOrderingNeighbors(node, neighbor2) == false) {
            return false; // Boolean
          }
        }
      }

      var na = node.getNumArtificialNeighbors();

      if (na > 0)
      {
        if (neighbor1 == null) {
          neighbor1 = node.getArtificialNeighbor(0);
        }
        else {
          neighbor2 = node.getArtificialNeighbor(0);
        }
        if (na > 1)
        {
          neighbor2 = node.getArtificialNeighbor(1);
        }
      }

      CLANode.removeEdge(neighbor1, neighbor2);
      CLANode.addArtificialEdge(neighbor1, neighbor2);

      CLANode.removeEdge(neighbor1, node);
      CLANode.removeEdge(neighbor2, node);
    }

    node = null;

    for (i = 0; i < nodes.length; i++)
    {
      if (nodes[i].getNumOrderingNeighbors() == 1)
      {
        node = nodes[i];
        break;
      }
    }

    if (node == null) {
      node = nodes[0];
    }

    var prev = null;
    var index = 0;

    while (index < nodes.length)
    {
      nodes[index] = node;

      if (node == null)
      {
        return false; // Boolean
      }

      node.setIndex(index);
      node = node.getOtherOrderingNeighbor(prev);
      prev = nodes[index];
      index++;
    }

    return true; // Boolean
  },

  getDegree2Node : function(nodeSet)
  {
    // summary:
    //		Returns the node with degree 2 from the node set.
    // tags:
    //		private
    
    var n = nodeSet.length;

    for (var i = 0; i < n; i++)
    {
      var node = nodeSet[i];
      if (node.getDegree() == 2) {
        return node; // ibm_ilog.graphlayout.circular.clayout.CLANode
      }
    }

    return null; // ibm_ilog.graphlayout.circular.clayout.CLANode
  },

  getNodesOfBiconnBlock : function(block)
  {
    // summary:
    //		Returns the nodes of a biconnected block. 
    // description:
    //		Returns the nodes of a biconnected block. 
    //		The resulting array contains also the root cut vertex of this block.
    //		 
    // tags:
    //		private
    
    var allNodes = block.getAllNodes();

    var result = TranslateUtil.newArray(allNodes.length, null);
    return TranslateUtil.copyArrayInto(allNodes,  result); // Array
  },

  getOrderedNodes : function(block)
  {
    // summary:
    //		Returns the nodes of a biconnected block ordered according to index.
    //		
    // description:
    //		Returns the nodes of a biconnected block ordered according to index.
    //		
    //		The array is cyclic rotated, so that the root cut vertex is the node
    //		with index 0.
    // tags:
    //		private
    
    var blockRoot = block.getRoot();
    var result = this.getNodesOfBiconnBlock(block);

    result.sort(CLCrossingReduction.compareNodeIndex);
    if (blockRoot != null)
    {
      var count = result.length;

      for (var i = 0; i < result.length; i++)
      {
        if (result[i] == blockRoot) {
          count = 0;
        }
        else {
          count++;
        }
        result[i].setIndex(count);
      }
      result.sort(CLCrossingReduction.compareNodeIndex);
    }

    return result; // Array
  }

}
);

lang.mixin(CLCrossingReduction,
{

  getNodes : function(cluster)
  {
    var nodes = TranslateUtil.newArray(cluster.size(), null);
    var e = cluster.getNodes();
    var i = 0;

    while (e.hasNext())
      nodes[i++] = e.next();
    return nodes; // Array
  },

  intraCrossFast : function(s1, t1, k, l)
  {
    var s2;
    var t2;

    if (k < l)
    {
      s2 = k;
      t2 = l;
    }
    else
    {
      s2 = l;
      t2 = k;
    }

    return (s1 < s2 && s2 < t1 && t1 < t2) || (s2 < s1 && s1 < t2 && t2 < t1); // Boolean
  },

  getClusterIntraCrossings : function(nodes)
  {
    CLCrossingReduction.ensureIndexe(nodes);
    var n = nodes.length;
    var numCrossings = 0;
    var openLinks = TranslateUtil.newArray(n, 0);
    var closingLinks = TranslateUtil.newArray(n, 0);
    var minClosingLinkIndex;
    var hasStarCenter = false;
    var node = null;
    var link = null;
    var e = null;
    var i = 0;
    var j = 0;
    var s = 0;
    var t = 0;

    for (i = 0; i < n; i++)
    {
      node = nodes[i];
      if (node.hasLinkToStarCenter()) {
        hasStarCenter = true;
      }
      e = node.getIntraLinks();
      minClosingLinkIndex = i;

      while (e.hasNext())
      {
        link = e.next();
        s = link.getCLASource().getIndex();
        t = link.getCLATarget().getIndex();

        if (s < i)
        {
          openLinks[s]--;
          closingLinks[s]++;
          if (s < minClosingLinkIndex) {
            minClosingLinkIndex = s;
          }
        }
        else if (t < i)
        {
          openLinks[t]--;
          closingLinks[t]++;
          if (t < minClosingLinkIndex) {
            minClosingLinkIndex = t;
          }
        }
        else if (s != t) {
          openLinks[i]++;
        }
      }

      var sumOpen = 0;

      for (j = i - 1; j >= minClosingLinkIndex; j--)
      {
        numCrossings += sumOpen * closingLinks[j];
        closingLinks[j] = 0;
        sumOpen += openLinks[j];
      }
    }

    if (hasStarCenter)
    {
      var halfCircle = 0.5 * n;
      var openLinksWithLessThanHalfCircle = 0;
      var diff = 0;

      for (i = 0; i < n; i++)
      {
        node = nodes[i];
        e = node.getIntraLinks();

        while (e.hasNext())
        {
          link = e.next();
          s = link.getCLASource().getIndex();
          t = link.getCLATarget().getIndex();
          diff = Math.abs(s - t);

          if (diff < halfCircle)
          {
            if (s < i) {
              openLinksWithLessThanHalfCircle--;
            }
            else if (t < i) {
              openLinksWithLessThanHalfCircle--;
            }
            else if (s != t) {
              openLinksWithLessThanHalfCircle++;
            }
          }
          else if (diff > halfCircle)
          {
            if (s < i) {
              openLinksWithLessThanHalfCircle++;
            }
            else if (t < i) {
              openLinksWithLessThanHalfCircle++;
            }
          }
        }

        if (node.hasLinkToStarCenter()) {
          numCrossings += openLinksWithLessThanHalfCircle;
        }
      }

      for (i = 0; i < n; i++)
      {
        if (openLinksWithLessThanHalfCircle == 0) {
          break;
        }
        node = nodes[i];
        e = node.getIntraLinks();

        while (e.hasNext())
        {
          link = e.next();
          s = link.getCLASource().getIndex();
          t = link.getCLATarget().getIndex();
          diff = Math.abs(s - t);

          if (diff > halfCircle)
          {
            if (i < s || i < t) {
              openLinksWithLessThanHalfCircle--;
            }
          }
        }

        if (node.hasLinkToStarCenter()) {
          numCrossings += openLinksWithLessThanHalfCircle;
        }
      }
    }

    return numCrossings; // int
  },

  getIntraCrossings : function(a, b, numNodes)
  {
    if (a == b) {
      return 0; // int
    }

    var na = a.getNumberOfIntraLinks();
    var nb = b.getNumberOfIntraLinks();
    var numCrossings = 0;
    var i = 0;
    var j = 0;

    for (i = 0; i < na; i++)
    {
      var linka = a.getIntraLink(i);
      var s1 = linka.getCLASource().getIndex();
      var t1 = linka.getCLATarget().getIndex();

      if (s1 > t1)
      {
        s1 = t1;
        t1 = linka.getCLASource().getIndex();
      }

      for (j = 0; j < nb; j++)
      {
        var linkb = b.getIntraLink(j);

        if (linka != linkb)
        {
          if (CLCrossingReduction.intraCrossFast(s1, t1, linkb.getCLASource().getIndex(), linkb.getCLATarget().getIndex())) {
            numCrossings++;
          }
        }
      }
    }

    var halfCircle = 0.5 * numNodes;
    var link = null;
    var s = 0;
    var t = 0;
    var k = 0;
    var diff = 0;

    if (a.hasLinkToStarCenter())
    {
      k = a.getIndex();

      for (i = 0; i < nb; i++)
      {
        link = b.getIntraLink(i);
        s = link.getCLASource().getIndex();
        t = link.getCLATarget().getIndex();
        diff = Math.abs(s - t);

        if (diff < halfCircle)
        {
          if ((s < k && k < t) || (t < k && k < s)) {
            numCrossings++;
          }
        }
        else if (diff > halfCircle)
        {
          if ((s > k || k > t) && (t > k || k > s)) {
            numCrossings++;
          }
        }
      }
    }

    if (b.hasLinkToStarCenter())
    {
      k = b.getIndex();

      for (i = 0; i < na; i++)
      {
        link = a.getIntraLink(i);
        s = link.getCLASource().getIndex();
        t = link.getCLATarget().getIndex();
        diff = Math.abs(s - t);

        if (diff < halfCircle)
        {
          if ((s < k && k < t) || (t < k && k < s)) {
            numCrossings++;
          }
        }
        else if (diff > halfCircle)
        {
          if ((s > k || k > t) && (t > k || k > s)) {
            numCrossings++;
          }
        }
      }
    }

    return numCrossings; // int
  },

  ensureIndexe : function(nodes)
  {
    for (var i = 0; i < nodes.length; i++)
      nodes[i].setIndex(i);
  },

  checkIndexe : function(nodes)
  {
    for (var i = 0; i < nodes.length; i++)
      if (nodes[i].getIndex() != i) {
        return false; // Boolean
      }
    return true; // Boolean
  },

  compareNodeIndex : function(o1, o2)
  {
    return ((o1).getIndex() - (o2).getIndex()); // int
  }

}
);

CLCrossingReduction.PrioQueue = declare(PriorityQueue, 
{
  N: null,

  constructor : function(n)
  {
    this.N = n;
    this.init(n);
  },
  
  getValue : function(obj)
  {
    var node = obj;
    return -this.N * node.getPlacedNeighbors() - (this.N - 1 - node.getUnplacedNeighbors()); // Number
  },

  getIndex : function(obj)
  {
    return (obj).getPrioQueueIndex(); // int
  },

  setIndex : function(obj, i)
  {
    (obj).setPrioQueueIndex(i);
  },

  updateObject : function(obj)
  {
    var k = this.getIndex(obj);
    if (k < 0) {
      return;
    }
    this.update(k);
  }

}
);
CLCrossingReduction.ClusterBiconnModel = declare( BiconnModel, 
{
  _cluster: null,

  constructor : function(cluster)
  {
    this._cluster = cluster;
  },
  
  getNodes : function()
  {
    return this._cluster.getNodes();
  },

  getLinks : function()
  {
    return this._cluster.getIntraLinks();
  },

  getSource : function(link)
  {
    return (link).getSource(); // Object
  },

  getTarget : function(link)
  {
    return (link).getTarget(); // Object
  },

  getLinksFrom : function(node)
  {
    return (node).getIntraLinksFrom(); // ibm_ilog.graphlayout.IIterator
  },

  getLinksTo : function(node)
  {
    return (node).getIntraLinksTo(); // ibm_ilog.graphlayout.IIterator
  },

  setDFSNumber : function(node, number)
  {
    (node).setDFSNumber(number);
  },

  getDFSNumber : function(node)
  {
    return (node).getDFSNumber(); // int
  },

  setCutVertex : function(node, cutVertex)
  {
    (node).setCutVertex(cutVertex);
  },

  isCutVertex : function(node)
  {
    return (node).isCutVertex(); // Boolean
  },

  setVisited : function(link, visited)
  {
    (link).setVisited(visited);
  },

  isVisited : function(link)
  {
    return (link).isVisited(); // Boolean
  }

}
);

return CLCrossingReduction;
});
