/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../circular/galg/BiconnBlock",	// #5
  "../../circular/galg/SimpleGraphUtil",	// #6
  "../../internalutil/GraphModelUtil",	// #7
  "../../internalutil/HashTable",	// #8
  "../../internalutil/LogResUtil",	// #9
  "../../internalutil/TranslateUtil"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    BiconnBlock,	// #5
    SimpleGraphUtil,	// #6
    GraphModelUtil,	// #7
    HashTable,	// #8
    LogResUtil,	// #9
    TranslateUtil	// #10
    ) {

var BiconnAlgorithm = declare("ibm_ilog.graphlayout.circular.galg.BiconnAlgorithm", null, 
{
  // summary:
  //		The algorithm to calculate the biconnected components of a
  //		traversable graph. 
  // description:
  //		The algorithm to calculate the biconnected components of a
  //		traversable graph. 
  //		<p>
  //		A set of nodes is a biconnected component if after removing one
  //		arbitrary single node from the graph, the set of nodes remains
  //		connected. For the connectivity, the direction of the links does not
  //		play any role. If after removing a node from a graph the graph is
  //		split into two parts that are not connected, then the node is called
  //		<em>cut vertex</em>. If after removing a link from a graph the graph
  //		is split into two parts that are not connected, then the link is
  //		called <em>cut edge</em>. A cut edge is a link that has two cut
  //		vertices as end nodes.   
  //		</p>
  //		<p>
  //		The biconnected components of a graph form a forest of block trees. A
  //		block is a set of nodes that are biconnected. All nodes that are not
  //		cut vertices belong only to one block. Cut vertices may belong to one
  //		or more blocks. In particular, a cut vertex that has no incident cut
  //		edge (i.e. all incident links are not cut edges) must belongs to
  //		multiple blocks.  
  //		</p>
  //		<p>
  //		A block is connected to another block if:  
  //		<ul>
  //		<li> there is a cut vertex that belongs to both blocks, or</li>  
  //		<li> there is a cut edge whose source belong to the one block and
  //		whose target belongs to the other block.</li>  
  //		</ul>
  //		Due to the construction of the block tree by the algorithm, the block
  //		tree is delivered as a directed tree. That is, each block in the
  //		block tree has a parent, except for the root of the block tree. Each
  //		block except the root block has a special cut vertex that connects
  //		the block to the parent block (i.e. either this cut vertex is in both
  //		blocks or it has an incident cut edge that goes to a node of the
  //		parent block). This cut vertex is called root of the block.  
  //		</p>
  //		<p>
  //		The algorithm delivers the forest of block trees. It allows to query
  //		which nodes are cut vertices and which cut vertex is the root of a
  //		block. The algorithm further allows to test whether a graph is
  //		biconnected.   
  //		</p>
  // tags:
  //		private
  
  constructor : function()
  {
    
  },

  _model: null,
  _root: null,
  _done: null,
  _dfsNumber: null,
  _activeNodes: null,
  _roots: null,
  _preferredTraversalStarters: null,
  _biconnBlocksPerNode: null,

  attach : function(model)
  {
    // summary:
    //		Attaches the algorithm to the model that describes the graph.
    // tags:
    //		private
    
    this._model = model;
    this._done = false;
    this._preferredTraversalStarters = null;
  },

  run : function()
  {
    // summary:
    //		Runs the biconnectivity algorithm. 
    // description:
    //		Runs the biconnectivity algorithm. 
    //		The model must be attached first.  
    // tags:
    //		private
    
    if (this._model == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6018E");
    }

    this._biconnBlocksPerNode = new HashTable();
    this._done = true;
    this._roots = this.getBlockTreesImpl();
  },

  isBiconnected : function(model)
  {
    // summary:
    //		Tests whether the graph is biconnected. 
    // description:
    //		Tests whether the graph is biconnected. 
    //		This method attaches the input model and runs the biconnectivity
    //		algorithm.  
    // model: BiconnModel
    //		The model that describes the graph
    // tags:
    //		private
    
    var roots = this.getBlockTrees(model);

    if (roots.length > 1) {
      return false; // Boolean
    }
    var root =  new TranslateUtil.ArrayIterator(roots).next();
    return (root.getChildren().length == 0); // Boolean
  },

  getBlockTrees : function(model)
  {
    // summary:
    //		Returns the forest of block trees of biconnected components of the
    //		graph. 
    // description:
    //		Returns the forest of block trees of biconnected components of the
    //		graph. 
    //		This method attaches the input model and runs the biconnectivity
    //		algorithm.  
    // model: BiconnModel
    //		The model that describes the graph
    // returns:
    //		The collection of root blocks of the block tree. This collection
    //		contains elements of type 'BiconnBlock'.
    // tags:
    //		private
    
    this.attach(model);
    this.run();
    return this.getAllBlockTrees(); // Array
  },

  getAllBlockTrees : function()
  {
    // summary:
    //		Returns the forest of block trees of biconnected components of the
    //		graph. 
    // description:
    //		Returns the forest of block trees of biconnected components of the
    //		graph. 
    //		This method does not run the algorithm. You must run the algorithm
    //		yourself before calling this method. It returns always the block
    //		trees of the previous run of the algorithm.  
    // returns:
    //		The collection of root blocks of the block tree. This collection
    //		contains elements of type 'BiconnBlock'.
    // tags:
    //		private
    
    if (this._model == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6018E");
    }

    if (!this._done)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6059E");
    }

    return this._roots; // Array
  },

  getBiconnectedComponents : function(model)
  {
    // summary:
    //		Returns the biconnected components of the graph described by the
    //		input model. 
    // description:
    //		Returns the biconnected components of the graph described by the
    //		input model. 
    //		This method attaches the input model and runs the biconnectivity
    //		algorithm.  
    // model: BiconnModel
    //		The model that describes the graph.
    // returns:
    //		A collection of blocks of type 'BiconnBlock'.
    // tags:
    //		private
    
    var roots = this.getBlockTrees(model);
    var components = new Array();

    var _for_each_array_1 = roots;
    for (var _for_each_index_1 = 0; _for_each_index_1 < _for_each_array_1.length; _for_each_index_1++){
      var blockObj = _for_each_array_1[_for_each_index_1];
      this.collectBlocks(blockObj, components);
    }
    
    return components; // Array
  },

  getAllBiconnectedComponents : function()
  {
    // summary:
    //		Returns all the biconnected components. 
    // description:
    //		Returns all the biconnected components. 
    //		This method does not run the algorithm. You must run the algorithm
    //		yourself before calling this method. It returns always the block
    //		trees of the previous run of the algorithm.  
    // returns:
    //		A collection of blocks of type 'BiconnBlock'.
    // tags:
    //		private
    
    var roots = this.getAllBlockTrees();
    var components = new Array();

    var _for_each_array_2 = roots;
    for (var _for_each_index_2 = 0; _for_each_index_2 < _for_each_array_2.length; _for_each_index_2++){
      var blockObj = _for_each_array_2[_for_each_index_2];
      this.collectBlocks(blockObj, components);
    }
    
    return components; // Array
  },

  collectBlocks : function(block, components)
  {
    // summary:
    //		Collect the biconnected components of a graph from the block tree.
    // tags:
    //		private
    
    components.push(block);

    var _for_each_array_3 = block.getChildren();
    for (var _for_each_index_3 = 0; _for_each_index_3 < _for_each_array_3.length; _for_each_index_3++){
      var blockObj = _for_each_array_3[_for_each_index_3];
      this.collectBlocks(blockObj, components);
    }
    
  },

  isCutVertex : function(node)
  {
    // summary:
    //		Returns 'true' if the input node is a cut vertex. 
    // description:
    //		Returns <code>true</code> if the input node is a cut vertex. 
    //		This information is only available after running the algorithm.  
    // node: Object
    //		A node of the attached model.
    // returns:
    //		whether the node is a cut vertex.
    // tags:
    //		private
    
    if (this._model == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6018E");
    }

    if (!this._done)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6059E");
    }

    return this._model.isCutVertex(node); // Boolean
  },

  isCutEdge : function(link)
  {
    // summary:
    //		Returns 'true' if the input link is a cut edge. 
    // description:
    //		Returns <code>true</code> if the input link is a cut edge. 
    //		This information is only available after running the algorithm.  
    // link: Object
    //		A link of the attached model.
    // returns:
    //		whether the link is a cut edge.
    // tags:
    //		private
    
    if (this._model == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6018E");
    }

    if (!this._done)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6059E");
    }

    return this._model.isCutVertex(this._model.getSource(link)) && this._model.isCutVertex(this._model.getTarget(link)); // Boolean
  },

  getReachableBiconnectivityBlocks : function(node)
  {
    // summary:
    //		Returns the biconnectivity blocks reachable from this node. 
    // description:
    //		Returns the biconnectivity blocks reachable from this node. 
    //		This is only relevant for cut vertices. More precisely, it returns:
    //		1) all biconnectivity blocks reachable over a cut edge from this
    //		node that have their root cut vertex in their body. 2) all
    //		biconnectivity blocks that have this node as the (unique) root cut
    //		vertex but
    //		don't contain the node in the body (i.e., the root is not pure).  Notice that a cut vertix may belong to multiple blocks. For one of these blocks, it belongs to the pure nodes (the body if the block). This block is not returned here. For all other blocks that it belongs to, it is the root cut vertex not belonging to the pure nodes. These blocks are returned here. Additionally, those blocks are returned that don't
    //		contain the cut vertex but are connected by a cut edge to the cut
    //		vertex.  
    //		<p>
    //		The method returns <code>null</code> if the input node is not a cut
    //		vertex.  
    //		</p>
    // tags:
    //		private
    
    if (this._model == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6018E");
    }

    if (!this._done)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6059E");
    }

    var blocks = this._biconnBlocksPerNode.item(node);
    if (blocks == null) {
      return null; // Array
    }
    return blocks; // Array
  },

  addStartNode : function(node)
  {
    // summary:
    //		Adds a node that should be used as start node of the biconnectivity
    //		handling. 
    // description:
    //		Adds a node that should be used as start node of the biconnectivity
    //		handling. 
    //		The biconnectivity blocks of the start nodes become the roots of the
    //		block trees.  
    //		<p>
    //		It is not necessary to specify start nodes in order to run the
    //		algorithm. If no start nodes are given, the root block of each block
    //		tree will be randomly choosen. If start nodes are given, the root
    //		block of a block tree will contain a start node whenever possible.  
    //		</p>
    //		<p>
    //		You must attach a model first before specifying the start nodes.
    //		</p>
    // tags:
    //		private
    
    if (this._model == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6018E");
    }

    if (this._preferredTraversalStarters == null) {
      this._preferredTraversalStarters = new Array();
    }
    this._preferredTraversalStarters.push(node);
  },

  getBlockTreesImpl : function()
  {
    // summary:
    //		Calculates and returns the forest of block trees.
    // tags:
    //		private
    
    this._activeNodes = new Array();
    var blocks = new Array();

    var iter = this._model.getNodes();
    var node = null;

    while (iter.hasNext())
    {
      node = iter.next();
      this._model.setDFSNumber(node, -1);
      this._model.setCutVertex(node, false);
    }

    var link = null;
    iter = this._model.getLinks();

    while (iter.hasNext())
    {
      link = iter.next();
      this._model.setVisited(link, false);
    }

    if (this._preferredTraversalStarters != null)
    {
      var _for_each_array_4 = this._preferredTraversalStarters;
      for (var _for_each_index_4 = 0; _for_each_index_4 < _for_each_array_4.length; _for_each_index_4++)
      {
        var nodeStarter = _for_each_array_4[_for_each_index_4];
        if (this._model.getDFSNumber(nodeStarter) < 0)
        {
          this._dfsNumber = 1;
          this.traverse(null, nodeStarter);
          this.buildBlockTree(this._root);
          blocks.push(this._root);
        }
      }
    }

    iter = this._model.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();

      if (this._model.getDFSNumber(node) < 0)
      {
        this._dfsNumber = 1;
        this.traverse(null, node);
        this.buildBlockTree(this._root);
        blocks.push(this._root);
      }
    }

    return blocks; // Array
  },

  traverse : function(parent, node)
  {
    // summary:
    //		Traverse the graph by DFS to calculate the biconnected components. 
    // description:
    //		Traverse the graph by DFS to calculate the biconnected components. 
    //		Returns the lowpoint number of the subtree. The lowpoint number is
    //		the smallest DFS number of a node reachable from the input node.
    // tags:
    //		private
    
    var lowPoint = this._dfsNumber;
    var nodeDFSNumber = lowPoint;
    this._model.setDFSNumber(node, nodeDFSNumber);
    this._dfsNumber++;
    this._activeNodes.push(node);

    var posInActive = this._activeNodes.length + 1;

    var n = 0;
    var iter = SimpleGraphUtil.getIncidentLinks(this._model, node);

    while (iter.hasNext())
    {
      var link = iter.next();
      var child = SimpleGraphUtil.getOppositeNode(this._model, node, link);
      if (child == parent) {
        continue;
      }
      n++;
      this._model.setVisited(link, true);
      var childDFSNumber = this._model.getDFSNumber(child);

      if (childDFSNumber >= 0)
      {
        if (childDFSNumber < nodeDFSNumber) {
          lowPoint = Math.min(lowPoint, childDFSNumber);
        }
      }
      else
      {
        posInActive = this._activeNodes.length + 1;
        var childLowPoint = this.traverse(node, child);

        if (childLowPoint >= nodeDFSNumber)
        {
          this.collectBlock(node, parent, posInActive, lowPoint);
        }
        else
        {
          lowPoint = Math.min(lowPoint, childLowPoint);
        }
      }
    }

    if (n == 0)
    {
      this.collectBlock(node, parent, posInActive, lowPoint);
    }

    return lowPoint; // int
  },

  collectBlock : function(root, parent, childPosInActive, lowPoint)
  {
    // summary:
    //		Collect all nodes that belong to this biconnectivity block.  
    // root: Object
    //		The root cut vertex of the block
    // parent: Object
    //		The parent of the cut vertex in the DFS
    // childPosInActive: int
    //		The position of the first body node of the block
    //		in activeNodes. The entire tail of activeNodes belongs to the
    //		block.
    // lowPoint: int
    //		Current low point of root.
    // tags:
    //		private
    
    var rootBelongsToBody = true;

    if (lowPoint < this._model.getDFSNumber(root)) {
      rootBelongsToBody = false;
    }
    else
    {
      var iter = SimpleGraphUtil.getIncidentLinks(this._model, root);

      while (iter.hasNext())
      {
        var link = iter.next();

        if (!this._model.isVisited(link))
        {
          rootBelongsToBody = false;
          break;
        }
      }
    }

    var bodyIsEmpty = (this._activeNodes.length - childPosInActive < 0);
    if (rootBelongsToBody) {
      bodyIsEmpty = false;
    }
    if (bodyIsEmpty) {
      return;
    }

    var block = new BiconnBlock();

    while (this._activeNodes.length >= childPosInActive)
      block.addNode(this._activeNodes.pop());
    block.setRoot(root);

    if (this._model.getDFSNumber(root) == 1)
    {
      if (!rootBelongsToBody) {
        this._model.setCutVertex(root, true);
      }
    }
    else {
      this._model.setCutVertex(root, true);
    }
    if (rootBelongsToBody)
    {
      block.addNode(root);

      this._activeNodes.pop();

      if (parent != null)
      {
        this.addBiconnectivityBlock(parent, block);
        this._model.setCutVertex(parent, true);
      }
    }
    else
    {
      this.addBiconnectivityBlock(root, block);
    }

    this._root = block;
  },

  addBiconnectivityBlock : function(node, block)
  {
    // summary:
    //		If this is a cutvertex, add a biconnectivity block reachable from
    //		this node.
    // tags:
    //		private
    
    var blocksObj = this._biconnBlocksPerNode.item(node);
    var blocks = null;

    if (blocksObj == null)
    {
      blocks = new Array();
      this._biconnBlocksPerNode.add(node, blocks);
    }
    else
    {
      blocks = blocksObj;
    }

    blocks.push(block);
  },

  buildBlockTree : function(block)
  {
    // summary:
    //		Build the block tree from the biconnectivity blocks.
    // tags:
    //		private
    
    var pureNodes = block.getPureNodes();

    var _for_each_array_5 = pureNodes;
    for (var _for_each_index_5 = 0; _for_each_index_5 < _for_each_array_5.length; _for_each_index_5++)
    {
      var node = _for_each_array_5[_for_each_index_5];
      var blocks = this.getReachableBiconnectivityBlocks(node);
      if (blocks == null) {
        continue;
      }
      var _for_each_array_6 = blocks;
      for (var _for_each_index_6 = 0; _for_each_index_6 < _for_each_array_6.length; _for_each_index_6++)
      {
        var childBlockObj = _for_each_array_6[_for_each_index_6];
        var childBlock = childBlockObj;
        childBlock.setParent(block);
        this.buildBlockTree(childBlock);
      }
    }
  }

}
);

return BiconnAlgorithm;
});
