/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../internalutil/GraphModelUtil",	// #5
  "../../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var BiconnBlock = declare("ibm_ilog.graphlayout.circular.galg.BiconnBlock", null, 
{
  // summary:
  //		This class represents a block of biconnected nodes. 
  // description:
  //		This class represents a block of biconnected nodes. 
  //		The biconnectivity algorithm calculates the blocks of biconnected
  //		components of a graph. Each block is a set of nodes which have the
  //		following property: When you remove one node from the block, then the
  //		remaining nodes of the same block stay connected.  
  //		<p>
  //		A cut vertex of the graph is a node that has the following property:
  //		If this node is removed from the graph, then the graph is split into
  //		two components that are not anymore connected. A cut vertex can be
  //		considered as the connection between two blocks. Hence, a cut vertex
  //		can belong to many blocks. The blocks with their connections form a
  //		block tree. Each inner block in the block tree has one root cut
  //		vertex, which is the connection to the previous block in the block
  //		tree.  
  //		</p>
  //		<p>
  //		The literature knows the notion of <em>cut edge</em>. A cut edge is
  //		simply a link that has a cut vertex as source and a cut vertex as
  //		target.   
  //		</p>
  // tags:
  //		private
  
  constructor : function()
  {
    
  this._nodes = new Array();
  this._root = null;
  this._rootIsInside = false;
  this._children = new Array();},

  _nodes : null,

  _root : null,

  _rootIsInside : null,

  _parent: null,

  _children : null,

  getParent : function()
  {
    // summary:
    //		Returns the parent block of this block. 
    // description:
    //		Returns the parent block of this block. 
    //		All biconnectivity blocks form a block tree.
    // tags:
    //		private
    
    return this._parent; // ibm_ilog.graphlayout.circular.galg.BiconnBlock
  },

  getChildren : function()
  {
    // summary:
    //		Returns the child block of this block. 
    // description:
    //		Returns the child block of this block. 
    //		All biconnectivity blocks form a block tree. Modifying the returned
    //		collection modifies the block.
    // tags:
    //		private
    
    return this._children; // Array
  },

  setParent : function(block)
  {
    // summary:
    //		Sets the parent of the block.
    // tags:
    //		private
    
    if (this._parent == block) {
      return;
    }
    if (this._parent != null) {
      TranslateUtil.removeArrayObject(this._parent._children, this);
    }
    this._parent = block;
    if (this._parent != null) {
      this._parent._children.push(this);
    }
  },

  addNode : function(node)
  {
    // summary:
    //		Adds a node to the block.
    // tags:
    //		private
    
    this._nodes.push(node);
    if (node == this._root) {
      this._rootIsInside = true;
    }
  },

  setRoot : function(node)
  {
    // summary:
    //		Sets the root cut vertex.
    // tags:
    //		private
    
    this._root = node;
  },

  getRoot : function()
  {
    // summary:
    //		Returns the root cut vertex of the block. 
    // description:
    //		Returns the root cut vertex of the block. 
    //		The root cut vertex is a node that belong to this block, and
    //		removing the cut vertex splits the parent block from this block.
    //		Hence, the root might also belong to the parent block. If the root
    //		is pure, it does not belong to the parent block but only to this
    //		block.
    // tags:
    //		private
    
    return this._root; // Object
  },

  isRootPure : function()
  {
    // summary:
    //		Returns 'true' if the root belongs to the pure nodes of
    //		the block. 
    // description:
    //		Returns <code>true</code> if the root belongs to the pure nodes of
    //		the block. 
    //		Returns <code>false</code> if the root belongs to this block and to
    //		the parent block. In this case, the root is not stored in the pure
    //		nodes of this block but in the pure nodes of some ancestor block.  
    // tags:
    //		private
    
    return this._rootIsInside; // Boolean
  },

  getPureNodes : function()
  {
    // summary:
    //		Returns the pure nodes of the block. 
    // description:
    //		Returns the pure nodes of the block. 
    //		The root is excluded if the root is a cut vertex that also belongs
    //		to the parent block. Modifying the returned collection modifies the
    //		block.
    // tags:
    //		private
    
    return this._nodes; // Array
  },

  getAllNodes : function()
  {
    // summary:
    //		Returns the nodes of the block including the root. 
    // description:
    //		Returns the nodes of the block including the root. 
    //		This method returns all nodes that belong to the block, even if all
    //		cut vertices also belong to other blocks. Modifying the returned
    //		collection may modify the block.
    // tags:
    //		private
    
    if (this._rootIsInside || this._root == null) {
      return this.getPureNodes(); // Array
    }
    var result = (this._nodes).concat([]);
    result.push(this._root);
    return result; // Array
  },

  getAllNodesArray : function()
  {
    // summary:
    //		Returns the nodes of the block including the root. 
    // description:
    //		Returns the nodes of the block including the root. 
    //		This method returns all nodes that belong to the block, even if all
    //		cut vertices also belong to other blocks.
    // tags:
    //		private
    
    var n = this._nodes.length;

    if (this._rootIsInside || this._root == null)
    {
      return this._nodes.concat([]); // Array
    }
    else
    {
      var m = n;
      n++;
      var result = TranslateUtil.newArray(n, null);
      result[0] = this._root;

      for (var i = 0; i < m; i++)
        result[i + 1] = this._nodes[i];
      return result; // Array
    }
  }

}
);

return BiconnBlock;
});
