/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../circular/galg/ClusteringGraph",	// #5
  "../../internalutil/GraphModelUtil",	// #6
  "../../internalutil/LogResUtil",	// #7
  "../../internalutil/PriorityQueue",	// #8
  "../../internalutil/RandomUtil",	// #9
  "../../internalutil/TranslateUtil"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    ClusteringGraph,	// #5
    GraphModelUtil,	// #6
    LogResUtil,	// #7
    PriorityQueue,	// #8
    RandomUtil,	// #9
    TranslateUtil	// #10
    ) {

var KLAlgorithm = declare("ibm_ilog.graphlayout.circular.galg.KLAlgorithm", null, 
{
  // summary:
  //		The algorithm to split a node set into two evenly sized clusters. 
  // description:
  //		The algorithm to split a node set into two evenly sized clusters. 
  //		The algorithm idea is originally by Kernigham and Lin, here in the
  //		variant by Fiduccia and Mattheyses, which is faster, even though I
  //		did not really tune it to linear time.   
  // tags:
  //		private
  
  constructor : function()
  {
    
  },

  _graph: null,
  _cluster1: null,
  _cluster2: null,
  _cluster1BorderNodes: null,
  _cluster2BorderNodes: null,
  _moveNodes: null,
  _cutWeight: null,

  attach : function(graph)
  {
    // summary:
    //		Attaches the algorithm to the model that describes the graph.
    // tags:
    //		private
    
    this._graph = graph;
  },

  run : function(maxNumberRounds)
  {
    // summary:
    //		Runs the biconnectivity algorithm. 
    // description:
    //		Runs the biconnectivity algorithm. 
    //		The model must be attached first.  
    // tags:
    //		private
    
    if (this._cluster1 == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6060E");
    }

    if (this._cluster2 == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6060E");
    }

    if (this._cluster1.length + this._cluster2.length <= 1) {
      return;
    }
    if (maxNumberRounds <= 0) {
      maxNumberRounds = 1;
    }
    this.initializeGainAndLoss();

    for (var i = 0; i < maxNumberRounds; i++)
    {
      if (!this.round()) {
        return;
      }
    }
  },

  createInitialPartitionFromIterator : function(nodes)
  {
    // summary:
    //		Creates the initial partition.  
    // nodes: IIterator
    //		The nodes to be partitioned
    // tags:
    //		private
    
    var nodesList = new Array();

    while (nodes.hasNext())
      nodesList.push(nodes.next());
    createInitialPartition(nodesList);
  },

  createInitialPartition : function(nodes)
  {
    // summary:
    //		Creates the initial partition.  
    // nodes: Array
    //		The nodes to be partitioned
    // tags:
    //		private
    
    var rand = new RandomUtil(0);
    var bestCluster1 = null;
    var bestCluster2 = null;
    var bestCutWeight = /*Number.MAX_VALUE*/2147483647;

    for (var i = 0; i < 5; i++)
    {
      var cutWeight = this.createRandomPartition(nodes, rand);

      if (cutWeight < bestCutWeight)
      {
        bestCutWeight = cutWeight;
        bestCluster1 = this._cluster1;
        bestCluster2 = this._cluster2;
      }

      if (cutWeight == 0) {
        break;
      }
    }

    this.setInitialPartition(bestCluster1, bestCluster2);
  },

  setInitialPartition : function(cluster1, cluster2)
  {
    // summary:
    //		Sets the initial partitioning. 
    // description:
    //		Sets the initial partitioning. 
    //		Both hashsets must contain disjoint sets of nodes of the clustering
    //		graph.
    // tags:
    //		private
    
    this._cluster1 = cluster1;
    this._cluster2 = cluster2;
  },

  getPartition1 : function()
  {
    // summary:
    //		Returns the final first partition. 
    // description:
    //		Returns the final first partition. 
    //		Can only be used after the algorithm has run.
    // tags:
    //		private
    
    return this._cluster1; // Array
  },

  getPartition2 : function()
  {
    // summary:
    //		Returns the final second partition. 
    // description:
    //		Returns the final second partition. 
    //		Can only be used after the algorithm has run.
    // tags:
    //		private
    
    return this._cluster2; // Array
  },

  getCutWeight : function()
  {
    // summary:
    //		Returns the cut weight for the current partition.
    // tags:
    //		private
    
    var cutWeight = 0;
    var iter =  new TranslateUtil.ArrayIterator(this._cluster1);

    while (iter.hasNext())
    {
      var node = iter.next();
      cutWeight += this._graph.computeLinkWeight(node, this._cluster2);
    }

    return cutWeight; // int
  },

  createRandomPartition : function(nodes, rand)
  {
    // summary:
    //		Creates an initial random partition.  
    // nodes: Array
    //		The nodes to be partitioned
    // rand: RandomUtil
    //		A random function.
    // returns:
    //		the cut weight
    // tags:
    //		private
    
    this._cluster1 = new Array();
    this._cluster2 = new Array();
    var n = nodes.length;
    var iter =  new TranslateUtil.ArrayIterator(nodes);

    while (iter.hasNext())
    {
      var node = iter.next();

      if (this._cluster1.length >= Math.floor(n / 2))
      {
        TranslateUtil.addSetObject(this._cluster2, node);
      }
      else if (this._cluster2.length >= Math.floor(n / 2))
      {
        TranslateUtil.addSetObject(this._cluster1, node);
      }
      else if (rand.nextBoolean())
      {
        TranslateUtil.addSetObject(this._cluster1, node);
      }
      else
      {
        TranslateUtil.addSetObject(this._cluster2, node);
      }
    }

    return this.getCutWeight(); // int
  },

  initializeGainAndLoss : function()
  {
    // summary:
    //		Initialize the gains and losses and puts border nodes into the
    //		priority queues. 
    // description:
    //		Initialize the gains and losses and puts border nodes into the
    //		priority queues. 
    //		Why only border nodes? Because this keeps the priority queues small,
    //		hence fast. If we have no border nodes anymore, it means two
    //		disconnected parts are separated into &#95;cluster1 and &#95;cluster2, hence
    //		we do not have to worry about the situation where we find no gain
    //		ainmore. This will be optimal anyway. It assumes that &#95;cluster1 and
    //		&#95;cluster2 are already initialized.
    // tags:
    //		private
    
    this._cluster1BorderNodes = new KLAlgorithm.PriorityQueue();
    this._cluster2BorderNodes = new KLAlgorithm.PriorityQueue();
    var node = null;
    var iter =  new TranslateUtil.ArrayIterator(this._cluster1);

    while (iter.hasNext())
    {
      node = iter.next();
      node.gain = this._graph.computeLinkWeight(node, this._cluster2);
      node.loss = this._graph.computeLinkWeight(node, this._cluster1);
    }

    iter =  new TranslateUtil.ArrayIterator(this._cluster2);

    while (iter.hasNext())
    {
      node = iter.next();
      node.gain = this._graph.computeLinkWeight(node, this._cluster1);
      node.loss = this._graph.computeLinkWeight(node, this._cluster2);
    }

    var n = this._cluster1.length + this._cluster2.length;
    this._moveNodes = TranslateUtil.newArray(n, null);
  },

  round : function()
  {
    // summary:
    //		Do one round of the Fiduccia-Mattheyes algorithm. 
    // description:
    //		Do one round of the Fiduccia-Mattheyes algorithm. 
    //		It tries to find two subsets of both clusters that can be exchanged
    //		so that the cutweight gets smaller. For this, it always swaps those
    //		pairs of nodes with the highest gain-loss value until all nodes are
    //		swapped. Then it backtracks to the situation where we had the best
    //		cutweight.
    // tags:
    //		private
    
    var node;

    if (this._cluster1.length < this._cluster2.length)
    {
      var aux = this._cluster2;
      this._cluster2 = this._cluster1;
      this._cluster1 = aux;
    }

    this._cluster1BorderNodes.init(this._cluster1.length);
    this._cluster2BorderNodes.init(this._cluster2.length);

    this._cutWeight = 0;
    var iter =  new TranslateUtil.ArrayIterator(this._cluster1);

    while (iter.hasNext())
    {
      node = iter.next();
      node.prioQueueIndex = -1;
      if (node.gain > 0) {
        this._cluster1BorderNodes.add(node);
      }
      this._cutWeight += node.gain;
    }

    iter =  new TranslateUtil.ArrayIterator(this._cluster2);

    while (iter.hasNext())
    {
      node = iter.next();
      node.prioQueueIndex = -1;
      if (node.gain > 0) {
        this._cluster2BorderNodes.add(node);
      }
    }

    if (this._cutWeight == 0) {
      return false; // Boolean
    }

    var index = 0;
    var bestCutWeight = this._cutWeight;
    var bestIndex = 0;
    var even = (this._cluster1.length == this._cluster2.length);

    while (!this._cluster1BorderNodes.isEmpty() && !this._cluster2BorderNodes.isEmpty())
    {
      node = this._cluster1BorderNodes.popMin();
      this._moveNodes[index++] = node;
      this._cutWeight -= this.moveNodeAndLock(node, this._cluster1, this._cluster2, this._cluster1BorderNodes, this._cluster2BorderNodes);

      if (!even)
      {
        if (this._cutWeight < bestCutWeight)
        {
          bestCutWeight = this._cutWeight;
          bestIndex = index;
        }
      }

      node = this._cluster2BorderNodes.popMin();
      this._moveNodes[index++] = node;
      this._cutWeight -= this.moveNodeAndLock(node, this._cluster2, this._cluster1, this._cluster2BorderNodes, this._cluster1BorderNodes);

      if (this._cutWeight < bestCutWeight)
      {
        bestCutWeight = this._cutWeight;
        bestIndex = index;
      }
    }

    while (index > bestIndex)
    {
      node = this._moveNodes[--index];
      this._cutWeight -= this.moveNodeAndLock(node, this._cluster1, this._cluster2, null, null);
      if (index == bestIndex) {
        break;
      }
      node = this._moveNodes[--index];
      this._cutWeight -= this.moveNodeAndLock(node, this._cluster2, this._cluster1, null, null);
    }

    if (this._cutWeight != bestCutWeight) {
      throw new Error("Bad1"); // Boolean
    }
    return bestIndex > 0; // Boolean
  },

  moveNodeAndLock : function(node, cluster1, cluster2, cluster1BorderNodes, cluster2BorderNodes)
  {
    // summary:
    //		Moves the node from cluster1 to cluster2 and updates gain and loss
    //		values. 
    // description:
    //		Moves the node from cluster1 to cluster2 and updates gain and loss
    //		values. 
    //		It also updates the priority queues, if they are not null. Returns
    //		the weight gain.
    // tags:
    //		private
    
    TranslateUtil.removeArrayObject(cluster1, node);
    TranslateUtil.addSetObject(cluster2, node);
    this.markedLocked(node);
    var link = null;
    var target = null;
    var w = 0;

    var iter = this._graph.getLinksFrom(node);

    while (iter.hasNext())
    {
      link = iter.next();
      w = this._graph.getLinkWeight(link);
      target = this._graph.getTarget(link);

      if ((arr.indexOf(cluster1, target)>=0))
      {
        target.gain += w;
        target.loss -= w;

        if (cluster1BorderNodes != null && target.gain > 0 && !this.isLocked(target))
        {
          if (this.isInClusterBorderNodes(target)) {
            cluster1BorderNodes.updateNode(target);
          }
          else {
            cluster1BorderNodes.add(target);
          }
        }
      }
      else if ((arr.indexOf(cluster2, target)>=0))
      {
        target.gain -= w;
        target.loss += w;

        if (cluster2BorderNodes != null && target.gain > 0 && !this.isLocked(target))
        {
          if (this.isInClusterBorderNodes(target)) {
            cluster2BorderNodes.updateNode(target);
          }
          else {
            cluster2BorderNodes.add(target);
          }
        }
      }
    }

    iter = this._graph.getLinksTo(node);
    var source = null;

    while (iter.hasNext())
    {
      link = iter.next();
      w = this._graph.getLinkWeight(link);
      source = this._graph.getSource(link);

      if ((arr.indexOf(cluster1, source)>=0))
      {
        source.gain += w;
        source.loss -= w;

        if (cluster1BorderNodes != null && source.gain > 0 && !this.isLocked(source))
        {
          if (this.isInClusterBorderNodes(source)) {
            cluster1BorderNodes.updateNode(source);
          }
          else {
            cluster1BorderNodes.add(source);
          }
        }
      }
      else if ((arr.indexOf(cluster2, source)>=0))
      {
        source.gain -= w;
        source.loss += w;

        if (cluster2BorderNodes != null && source.gain > 0 && !this.isLocked(source))
        {
          if (this.isInClusterBorderNodes(source)) {
            cluster2BorderNodes.updateNode(source);
          }
          else {
            cluster2BorderNodes.add(source);
          }
        }
      }
    }

    var h = node.loss;
    node.loss = node.gain;
    node.gain = h;
    return node.loss - node.gain; // int
  },

  markedLocked : function(node)
  {
    // summary:
    //		Mark a node as locked.
    // tags:
    //		private
    
    node.prioQueueIndex = -2;
  },

  isLocked : function(node)
  {
    // summary:
    //		Check whether a node is locked.
    // tags:
    //		private
    
    return (node.prioQueueIndex == -2); // Boolean
  },

  isInClusterBorderNodes : function(node)
  {
    // summary:
    //		Check whether a node is in the priority queue.
    // tags:
    //		private
    
    return (node.prioQueueIndex >= 0); // Boolean
  }

}
);

lang.mixin(KLAlgorithm,
{
}
);

KLAlgorithm.PriorityQueue = declare(PriorityQueue, 
{

  constructor : function()
  {
    
  },

  getValue : function(obj)
  {
    var node = obj;
    return node.loss - node.gain; // Number
  },

  setIndex : function(obj, i)
  {
    var node = obj;
    node.prioQueueIndex = i;
  },

  updateNode : function(node)
  {
    // summary:
    //		Update a node after its value has changed. 
    // description:
    //		Update a node after its value has changed. 
    //		
    // tags:
    //		private
    
    this.update(node.prioQueueIndex);
  },

  remove : function(obj)
  {
    // summary:
    //		Removed the object from the queue. 
    // description:
    //		Removed the object from the queue. 
    //		Still O(log n).
    // tags:
    //		private
    
    var node = obj;
    node.gain = /*Number.MIN_VALUE*/(-2147483648);
    this.update(node.prioQueueIndex);
    var min = this.popMin();

    if (min != node) {
      throw new Error("Bug2");
    }
  }

}
);

return KLAlgorithm;
});
