/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../forcedirected/SolarSystemNode",	// #5
  "../internalutil/genericgrapher/AbstractGraph",	// #6
  "../internalutil/genericgrapher/AbstractLink",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/LogResUtil",	// #9
  "../internalutil/TranslateUtil"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    SolarSystemNode,	// #5
    AbstractGraph,	// #6
    AbstractLink,	// #7
    GraphModelUtil,	// #8
    LogResUtil,	// #9
    TranslateUtil	// #10
    ) {

var GraphCoarseningAlgo = declare("ibm_ilog.graphlayout.forcedirected.GraphCoarseningAlgo", null, 
{
  // summary:
  //		Coarsens the graph for the FMMM algo. 
  // description:
  //		Coarsens the graph for the FMMM algo. 
  //		
  // tags:
  //		private
  
  constructor : function()
  {
  },
  
  createCoarserGraph : function (a0, a1, a2)
  {
    if(arguments.length == 2)
      return this.createCoarserGraph_overload_2(a0, a1);
    else if(arguments.length == 3)
      return this.createCoarserGraph_overload_3(a0, a1, a2);
  },
  
  createCoarserGraph_overload_2 : function(graph, graphModel)
  {
    var suns = GraphCoarseningAlgo.createSolarSystemPartition(graph, graphModel);
    var coarserGraph = this.createCoarserGraph(graph, suns, graphModel);
    return coarserGraph; // ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractGraph
  },

  createCoarserGraph_overload_3 : function(graph, suns, graphModel)
  {
    // summary:
    //		Creates a coarser graph from the input graph according to its
    //		partitioning into solar systems. 
    // description:
    //		Creates a coarser graph from the input graph according to its
    //		partitioning into solar systems. 
    //		
    // graph: AbstractGraph
    //		The input graph
    // suns: Array
    //		The set of suns identified in the input graph.
    // graphModel: AbstractGraphLayoutModel
    //		
    // returns:
    //		The coarser graph.
    // tags:
    //		private
    
    var initialNumberOfNodes = graph.getNodesCount();
    var coarserGraph = new AbstractGraph(graphModel);

    var nSuns = suns.length;
    var mNode;
    var pNode;
    var sNode;
    var sunData;
    var planets;

    for (var i = 0; i < nSuns; i++)
    {
      sNode = suns[i];
      var node = new SolarSystemNode(sNode, sNode.boundingBox());
      coarserGraph.addNode(node);
      sunData = sNode.getUserData();
      sunData.setSolarSystemNode(node);
      planets = sunData.getPlanets();

      var _for_each_array_1 = planets;
      for (var _for_each_index_1 = 0; _for_each_index_1 < _for_each_array_1.length; _for_each_index_1++)
      {
        var opNode = _for_each_array_1[_for_each_index_1];
        pNode = opNode;
        var planetData = pNode.getUserData();
        planetData.setSolarSystemNode(node);
        var mNodes = planetData.getMoons();

        if (mNodes != null)
        {
          var _for_each_array_2 = mNodes;
          for (var _for_each_index_2 = 0; _for_each_index_2 < _for_each_array_2.length; _for_each_index_2++)
          {
            var omNode = _for_each_array_2[_for_each_index_2];
            mNode = omNode;
            var moonData = mNode.getUserData();
            moonData.setSolarSystemNode(node);
          }
        }
      }
    }

    for (var ji = 0; ji < nSuns; ji++)
    {
      sNode = suns[ji];
      sunData = sNode.getUserData();
      var solarSystemNode = sunData.getSolarSystemNode();

      var neighbors = graph.getNeighbors(sNode);

      while (neighbors.hasNext())
      {
        var neighbor = neighbors.next();
        if (neighbor != sNode) {
          this.addLinkIfNeeded(solarSystemNode, neighbor, coarserGraph);
        }
      }

      planets = sunData.getPlanets();

      var _for_each_array_3 = planets;
      for (var _for_each_index_3 = 0; _for_each_index_3 < _for_each_array_3.length; _for_each_index_3++)
      {
        var poNode = _for_each_array_3[_for_each_index_3];
        pNode = poNode;
        neighbors = graph.getNeighbors(pNode);

        while (neighbors.hasNext())
        {
          var pNodeNeighbor = neighbors.next();
          if (pNodeNeighbor != pNode && pNodeNeighbor != sNode) {
            this.addLinkIfNeeded(solarSystemNode, pNodeNeighbor, coarserGraph);
          }
        }

        var pData = pNode.getUserData();
        var moonNodes = pData.getMoons();

        if (moonNodes != null)
        {
          var _for_each_array_4 = moonNodes;
          for (var _for_each_index_4 = 0; _for_each_index_4 < _for_each_array_4.length; _for_each_index_4++)
          {
            var moNode = _for_each_array_4[_for_each_index_4];
            mNode = moNode;
            neighbors = graph.getNeighbors(mNode);

            while (neighbors.hasNext())
            {
              var nbor = neighbors.next();
              if (nbor != mNode && nbor != pNode && nbor != sNode) {
                this.addLinkIfNeeded(solarSystemNode, nbor, coarserGraph);
              }
            }
          }
        }
      }
    }

    if (coarserGraph.getNodesCount() < initialNumberOfNodes) {
      return coarserGraph; // ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractGraph
    }
    LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6757E",  initialNumberOfNodes);
    return null; // ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractGraph
  },

  addLinkIfNeeded : function(firstNodeSolarSystemNode, secondNode, coarserGraph)
  {
    var secondNodeData = secondNode.getUserData();
    if (secondNodeData == null) {
      return;
    }

    var secondNodeSolarSystemNode = secondNodeData.getSolarSystemNode();

    if (secondNodeSolarSystemNode != firstNodeSolarSystemNode && !coarserGraph.isLinkBetween(firstNodeSolarSystemNode, secondNodeSolarSystemNode))
    {
      var link = new AbstractLink(firstNodeSolarSystemNode, secondNodeSolarSystemNode);
      coarserGraph.addLink(link);
    }
  }

}
);

lang.mixin(GraphCoarseningAlgo,
{

  createSolarSystemPartition : function(graph, graphModel)
  {
    var nodes = graph.getNodes();
    var nNodes = graph.getNodesCount();
    var candidatesArray = TranslateUtil.newArray(nNodes, null);
    var counter = 0;

    while (nodes.hasNext())
    {
      var node = nodes.next();
      candidatesArray[counter++] = node;
    }

    var comp = new GraphCoarseningAlgo.GCANodeDegreeComparator(graph);

    candidatesArray.sort(lang.hitch(comp, comp.compare));
    var candidates = new Array();

    for (var i = 0; i < nNodes; i++)
      candidates.push(candidatesArray[i]);
    var suns = new Array();
    var pNode;

    while (!(candidates.length==0))
    {
      var index = candidates.length - 1;
      var sNode = candidates[index];

      GraphCoarseningAlgo.registerSun(sNode);
      TranslateUtil.removeArrayObject(candidates, sNode);
      suns.push(sNode);
      var neighbors = graph.getNeighbors(sNode);

      while (neighbors.hasNext())
      {
        pNode = neighbors.next();

        if (pNode == sNode)
        {
          LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6750E");
        }

        if (pNode.getType() == SolarSystemNode.MOON_NODE)
        {
          TranslateUtil.removeArrayObject(candidates, pNode);

          GraphCoarseningAlgo.registerPlanet(pNode, sNode);
        }
      }

      neighbors = graph.getNeighbors(sNode);

      while (neighbors.hasNext())
      {
        pNode = neighbors.next();

        if (pNode == sNode)
        {
          LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6750E");
        }

        if (pNode.getType() != SolarSystemNode.PLANET_NODE)
        {
          LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6751E");
        }

        var neighborsOfPlanet = graph.getNeighbors(pNode);

        while (neighborsOfPlanet.hasNext())
        {
          var ssNode = neighborsOfPlanet.next();

          if (ssNode == pNode)
          {
            LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6752E");
          }

          if (ssNode.getType() == SolarSystemNode.MOON_NODE)
          {
            TranslateUtil.removeArrayObject(candidates, ssNode);
          }
        }
      }
    }

    nodes = graph.getNodes();

    while (nodes.hasNext())
    {
      pNode = nodes.next();
      var type = pNode.getType();

      if (type == SolarSystemNode.MOON_NODE)
      {
        var mNode = pNode;
        var neighborsOfMoon = graph.getNeighbors(mNode);
        var foundPlanetForThisMoon = false;

        while (neighborsOfMoon.hasNext())
        {
          var neighbor = neighborsOfMoon.next();

          if (neighbor.getType() == SolarSystemNode.PLANET_NODE || neighbor.getType() == SolarSystemNode.PLANET_WITH_MOON_NODE)
          {
            GraphCoarseningAlgo.registerMoon(mNode, neighbor);

            foundPlanetForThisMoon = true;
            break;
          }
        }

        if (!foundPlanetForThisMoon)
        {
          var params = [mNode ];
          LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6753E", params);
        }
      }
    }

    return suns; // Array
  },

  registerSun : function(sNode)
  {
    sNode.setType(SolarSystemNode.SUN_NODE);

    sNode.setUserData(new GraphCoarseningAlgo.SunData());
  },

  registerPlanet : function(pNode, sNode)
  {
    pNode.setType(SolarSystemNode.PLANET_NODE);
    pNode.setUserData(new GraphCoarseningAlgo.PlanetData());
    var sunData = sNode.getUserData();

    if (sunData == null)
    {
      sunData = new GraphCoarseningAlgo.SunData(pNode);
      sNode.setUserData(sunData);
    }
    else
    {
      sunData.addPlanet(pNode);
    }
  },

  registerMoon : function(mNode, pNode)
  {
    if (pNode.getUserData() == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6754E");
    }

    if (pNode.getType() != SolarSystemNode.PLANET_NODE && pNode.getType() != SolarSystemNode.PLANET_WITH_MOON_NODE)
    {
      LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6755E");
    }

    pNode.setType(SolarSystemNode.PLANET_WITH_MOON_NODE);
    mNode.setType(SolarSystemNode.MOON_NODE);
    mNode.setUserData(new GraphCoarseningAlgo.MoonData());
    var planetData = pNode.getUserData();

    if (planetData != null)
    {
      planetData.addMoon(mNode);
    }
    else
    {
      var params = [pNode ];
      LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6756E", params);
    }
  },

  getSolarSystemNode : function(node)
  {
    var data = node.getUserData();
    return data.getSolarSystemNode(); // ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractNode
  },

  getPlanets : function(sunNode)
  {
    var sunData = sunNode.getUserData();
    return sunData.getPlanets(); // Array
  },

  getMoons : function(pNode)
  {
    var planetData = pNode.getUserData();
    return planetData.getMoons(); // Array
  }

}
);

GraphCoarseningAlgo.SolarSystemData = declare( null, 
{

  constructor : function()
  {
    
  },
  _node: null,

  setSolarSystemNode : function(node)
  {
    if (node == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ForceDirectedLayout ", "graphlayout.expert.message.6758E");
    }

    this._node = node;
  },

  getSolarSystemNode : function()
  {
    if (this._node == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6759E");
    }

    return this._node; // ibm_ilog.graphlayout.internalutil.genericgrapher.AbstractNode
  }

}
);
GraphCoarseningAlgo.SunData = declare(GraphCoarseningAlgo.SolarSystemData, 
{
  // summary:
  //		The solar system to which the current node is connected.
  // tags:
  //		private
  
  _pNodes : null,

  constructor : function(pNode )
  {
    this._pNodes = new Array();    if (pNode)
      this.addPlanet(pNode);
                              
  },
  
  addPlanet : function(pNode)
  {
    if (pNode == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ForceDirectedLayout ", "graphlayout.expert.message.6760E");
    }

    this._pNodes.push(pNode);
  },

  getPlanets : function()
  {
    // summary:
    //		Returns the planets of this sun. 
    // description:
    //		Returns the planets of this sun. 
    //		Can be empty (sun without planets).
    // tags:
    //		private
    
    return this._pNodes; // Array
  }

}
);
GraphCoarseningAlgo.PlanetData = declare(GraphCoarseningAlgo.SolarSystemData, 
{
  _mNodes: null,

  constructor : function(

)
  {
  },
  
  addMoon : function(mNode)
  {
    if (this._mNodes == null) {
      this._mNodes = new Array();
    }
    this._mNodes.push(mNode);
  },

  getMoons : function()
  {
    return this._mNodes; // Array
  }

}
);
GraphCoarseningAlgo.MoonData = declare(GraphCoarseningAlgo.SolarSystemData, 
{

  constructor : function()
  {
  }
  
}
);
GraphCoarseningAlgo.GCANodeDegreeComparator = declare( null, 
{
  _graph: null,

  constructor : function(graph)
  {
    this._graph = graph;
  },
  
  compare : function(node1, node2)
  {
    var degree1 = this._graph.getNodeLinksCount(node1);
    var degree2 = this._graph.getNodeLinksCount(node2);
    if (degree1 < degree2) {
      return 1; // int
    }
    else if (degree1 == degree2) {
      return 0; // int
    }
    else {
      return -1; // int
    }
  }

}
);

return GraphCoarseningAlgo;
});
