/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../forcedirected/GraphCoarseningAlgo",	// #5
  "../forcedirected/SolarSystemNode",	// #6
  "../forcedirected/TopLevelAbstractGraphAdapter",	// #7
  "../internalutil/genericgrapher/AbstractGraph",	// #8
  "../internalutil/genericgrapher/AbstractGraphAdapter",	// #9
  "../internalutil/genericgrapher/AbstractLink",	// #10
  "../internalutil/GraphModelUtil",	// #11
  "../internalutil/LogResUtil",	// #12
  "../internalutil/TranslateUtil"	// #13
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphCoarseningAlgo,	// #5
    SolarSystemNode,	// #6
    TopLevelAbstractGraphAdapter,	// #7
    AbstractGraph,	// #8
    AbstractGraphAdapter,	// #9
    AbstractLink,	// #10
    GraphModelUtil,	// #11
    LogResUtil,	// #12
    TranslateUtil	// #13
    ) {

var MultilevelGraphPartitioningAlgo = declare("ibm_ilog.graphlayout.forcedirected.MultilevelGraphPartitioningAlgo", null, 
{
  // summary:
  //		Partitions the graph for the FMM algo.  
  // tags:
  //		private
  
  constructor : function()
  {
  },
  
  run : function(graphModel, graphs, models)
  {
    // summary:
    //		Creates the multilevel partitioning of the graph described by the
    //		given model. 
    // description:
    //		Creates the multilevel partitioning of the graph described by the
    //		given model. 
    //		The method fills the lists <code>graphs</code> and
    //		<code>models</code>.
    
    var graph = new AbstractGraph(graphModel);
    graphs.push(graph);

    models.push(new TopLevelAbstractGraphAdapter(graph, graphModel));
    var coarseningAlgo = new GraphCoarseningAlgo();
    this.fill(graph, graphModel);

    while (graph.getNodesCount() > 10)
    {
      graph = coarseningAlgo.createCoarserGraph(graph, graphModel);
      if (graph.getNodesCount() < 3 || graph.getLinksCount() < 3) {
        break;
      }
      graphs.push(graph);
      models.push(new AbstractGraphAdapter(graph));
    }
  },

  fill : function(graph, graphModel)
  {
    // summary:
    //		Creates an abstract graph from a given graph model. 
    // description:
    //		Creates an abstract graph from a given graph model. 
    //		The following links are filtered out: - Self-links. - Multilinks
    //		(only one link between the same pair of nodes).
    // tags:
    //		private
    
    var nodes = new TranslateUtil.ArrayIterator(graphModel.getNodes());

    while (nodes.hasNext())
    {
      var node = nodes.next();
      var abstractNode = new SolarSystemNode(node, GraphModelUtil.getObjectBounds(graphModel, node));
      graphModel.setParameter(node, MultilevelGraphPartitioningAlgo.ABSTRACT_NODE, abstractNode);
      graph.addNode(abstractNode);
    }

    var links = new TranslateUtil.ArrayIterator(graphModel.getLinks());

    while (links.hasNext())
    {
      var link = links.next();
      var from = graphModel.getFrom(link);
      var toobj = graphModel.getTo(link);

      if (from != toobj)
      {
        var abstractFrom = graphModel.getParameter(from, MultilevelGraphPartitioningAlgo.ABSTRACT_NODE);
        var abstractTo = graphModel.getParameter(toobj, MultilevelGraphPartitioningAlgo.ABSTRACT_NODE);

        if (abstractFrom == abstractTo)
        {
          LogResUtil.logAndThrowRuntimeExc(" ForceDirectedLayout ", "graphlayout.expert.message.6761E");
        }

        if (!graph.isLinkBetween(abstractFrom, abstractTo))
        {
          var abstractLink = new AbstractLink(abstractFrom, abstractTo);

          graph.addLink(abstractLink);
          abstractLink.setUserData(link);
          abstractLink.setOriginalObject(link);
        }
      }
    }

    nodes = new TranslateUtil.ArrayIterator(graphModel.getNodes());

    while (nodes.hasNext())
    {
      var no = nodes.next();
      graphModel.setParameter(no, MultilevelGraphPartitioningAlgo.ABSTRACT_NODE, null);
    }
  }

}
);

lang.mixin(MultilevelGraphPartitioningAlgo,
{
  ABSTRACT_NODE : null,
  _fields_init : function(c)
  {
    c = c || MultilevelGraphPartitioningAlgo;
    
    c.ABSTRACT_NODE = "__abstractNode";
  }
  
}
);

(function(){ MultilevelGraphPartitioningAlgo._fields_init(); })();

return MultilevelGraphPartitioningAlgo;
});
