/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphLayoutData",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/LogResUtil",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../_GraphLayout"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphLayoutData,	// #5
    GraphModelUtil,	// #6
    LogResUtil,	// #7
    TranslateUtil,	// #8
    GraphLayout	// #9
    ) {

var RegularizationAlgo = declare("ibm_ilog.graphlayout.forcedirected.RegularizationAlgo", null, 
{
  // summary:
  //		This class produces a quite uniform distribution of nodes inside a
  //		regular polygon of fixed nodes. 
  // description:
  //		This class produces a quite uniform distribution of nodes inside a
  //		regular polygon of fixed nodes. 
  //		It is a kind of minimization of the square of the distances between
  //		nodes. In other words, it puts each non-fixed node at the "center" of
  //		its neighbors.   
  // tags:
  //		private
  /*=====
  // SLOW_GOOD: int
  //		Slow node placement algorithm. 
  //		It provides more uniformity of the node distribution inside the
  //		exterior cycle, but is slightly slower.
  SLOW_GOOD : null,
  // QUICK_BAD: int
  //		Fast node placement algorithm. 
  //		It provides less uniformity of the node distribution inside the
  //		exterior cycle, but is slightly faster.
  QUICK_BAD : null,
  =====*/
  
  _topologyData: null,
  _positionData: null,

  _xSum : null,
  _ySum : null,
  _xSumUp : null,
  _ySumUp : null,
  _sumDown: null,

  _mode : null,

  constructor : function()
  {
    this._xSum = 0;
    this._ySum = 0;
    this._xSumUp = 0;
    this._ySumUp = 0;
    this._mode = RegularizationAlgo.QUICK_BAD;
  },
  
  getMode : function()
  {
    return this._mode; // int
  },

  computeXYbarycenter : function(nodeId)
  {
    // summary:
    //		Computes _x, _y = the position of nodeId as a barycenter of his 
    //		neighbors.
    // tags:
    //		private
    
    var newX;
    var newY;
    this.computeSumXYNeighbours(nodeId);

    newX = RegularizationAlgo.BARYCENTER_COEF * (this._xSum / this._topologyData.getNumberOfNeighbours(nodeId)) + this._positionData.getXofNode(nodeId) * RegularizationAlgo.BARYCENTER_COEF;
    newY = RegularizationAlgo.BARYCENTER_COEF * (this._ySum / this._topologyData.getNumberOfNeighbours(nodeId)) + this._positionData.getYofNode(nodeId) * RegularizationAlgo.BARYCENTER_COEF;
    this._positionData.recordMove(Math.abs(this._positionData.getXofNode(nodeId) - newX), Math.abs(this._positionData.getYofNode(nodeId) - newY));
    this._positionData.setNodePosition(nodeId, newX, newY);
  },

  computeXYcenter : function(nodeId)
  {
    // summary:
    //		Computes _x, _y = the position of nodeId at the center of its
    //		neighbors using another formula which gives a slower convergence
    //		speed, but a better quality of the results. 
    // description:
    //		Computes &#95;x, &#95;y = the position of nodeId at the center of its
    //		neighbors using another formula which gives a slower convergence
    //		speed, but a better quality of the results. 
    //		We compute the center of the smallest disc including all the
    //		neighbors.
    // tags:
    //		private
    
    var newX;
    var newY;

    this.computeSumUpDownNeighbours(nodeId, this._positionData.getXofNode(nodeId), this._positionData.getYofNode(nodeId));

    if (this._sumDown > 0.00001)
    {
      newX = this._positionData.getXofNode(nodeId) + this._xSumUp / (4 * this._sumDown);
      newY = this._positionData.getYofNode(nodeId) + this._ySumUp / (4 * this._sumDown);
      this._positionData.recordMove(Math.abs(this._positionData.getXofNode(nodeId) - newX), Math.abs(this._positionData.getYofNode(nodeId) - newY));
      this._positionData.setNodePosition(nodeId, newX, newY);
    }
  },

  computeSumXYNeighbours : function(nodeId)
  {
    // summary:
    //		Computes _xSum = the sum of the x coordinate of all neighbors of
    //		nodeId idem for _ySum.
    // tags:
    //		private
    
    this._xSum = 0;
    this._ySum = 0;

    var n = this._topologyData.getNumberOfNeighbours(nodeId);

    for (var i = 0; i < n; i++)
    {
      var neighborId = this._topologyData.getNeighbour(nodeId, i);
      this._xSum += this._positionData.getXofNode(neighborId);
      this._ySum += this._positionData.getYofNode(neighborId);
    }
  },

  computeSumUpDownNeighbours : function(nodeId, x, y)
  {
    // summary:
    //		Computes _xSum = the sum of the x coordinate of all neighbours of
    //		nodeId. 
    // description:
    //		Computes &#95;xSum = the sum of the x coordinate of all neighbours of
    //		nodeId. 
    //		Idem for &#95;ySum  
    // nodeId: int
    //		The Id of the current node
    // x: Number
    //		the x-coordinate of the node
    // y: Number
    //		the y-coordinate of the node
    // tags:
    //		private
    
    this._xSumUp = 0;
    this._ySumUp = 0;
    this._sumDown = 0;
    var dist = 0;
    var distX = 0;
    var distY = 0;

    var n = this._topologyData.getNumberOfNeighbours(nodeId);

    for (var neighb = 0; neighb < n; neighb++)
    {
      var neighborId = this._topologyData.getNeighbour(nodeId, neighb);
      distX = this._positionData.getXofNode(neighborId) - this._positionData.getXofNode(nodeId);
      distY = this._positionData.getYofNode(neighborId) - this._positionData.getYofNode(nodeId);
      dist = distX * distX + distY * distY;
      dist = dist * dist;
      this._xSumUp += distX * dist;
      this._ySumUp += distY * dist;
      this._sumDown += dist;
    }
  },

  regularize : function(graphModel, fixedNodeInterface)
  {
    // summary:
    //		Performs the regularization.  
    // returns:
    //		The number of iterations done.
    
    var msg;

    if (graphModel == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ForceDirectedLayout ", "graphlayout.expert.message.6012E");
    }

    if (fixedNodeInterface == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ForceDirectedLayout ", "graphlayout.expert.message.6763E");
    }

    var nodeId;
    var numberOfIterationsDone = 0;
    var regLayout = new RegularizationAlgo.RegularizationLayout();
    regLayout.attach(graphModel);
    var layoutData = new GraphLayoutData(regLayout, fixedNodeInterface);
    layoutData.beforeLayout(true, true, false, false);
    this._topologyData = layoutData.getTopologicalData();
    this._positionData = layoutData.getPositionData();

    if (!this._positionData.identifyFixedAndMoveableNodes(true, false))
    {
      LogResUtil.logAndThrowGraphLayoutExc(" ForceDirectedLayout ", "graphlayout.expert.message.6764E");
    }

    var maxIter = RegularizationAlgo.MAX_NUMBER_OF_ITER;
    var firstIndexOfMoveableNodes = this._positionData.getFirstIndexOfMoveableNodes();
    var numberOfMoveableNodes = this._positionData.getNumberOfMoveableNodes();
    if (numberOfMoveableNodes == 0) {
      return 0; // int
    }

    var nodePlacementAlgo = this.getMode();

    for (var iter = 1; iter <= maxIter; iter++)
    {
      numberOfIterationsDone = iter;
      this._positionData.resetMaxMovementCounter();

      for (var k = firstIndexOfMoveableNodes; k < firstIndexOfMoveableNodes + numberOfMoveableNodes; k++)
      {
        nodeId = this._positionData.getNodeId(k);

        switch (nodePlacementAlgo)
        {
          case RegularizationAlgo.SLOW_GOOD:
            
            this.computeXYbarycenter(nodeId);
            break;
          case RegularizationAlgo.QUICK_BAD:
            this.computeXYcenter(nodeId);
            break;
          default:
            var params = [this.getMode() ];
            LogResUtil.logAndThrowGraphLayoutExc(" ForceDirectedLayout ", "graphlayout.expert.message.6765E", params);
        }
      }

      if (this._positionData.getMaxMovement() <= RegularizationAlgo.CONVERGENCE_THRESHOLD) {
        break;
      }
      if (this.isNodesPlacementTimeElapsed()) {
        break;
      }
    }

    this._positionData.placeNodesAtPosition(false);

    return numberOfIterationsDone; // int
  },

  isNodesPlacementTimeElapsed : function()
  {
    return false; // Boolean
  }

}
);

lang.mixin(RegularizationAlgo,
{
  CONVERGENCE_THRESHOLD : null,

  BARYCENTER_COEF : null,
  MAX_NUMBER_OF_ITER : null,

  SLOW_GOOD : null,

  QUICK_BAD : null,
  _fields_init : function(c)
  {
    c = c || RegularizationAlgo;
    
    c.CONVERGENCE_THRESHOLD = 1;
    c.BARYCENTER_COEF = -0.24;
    c.MAX_NUMBER_OF_ITER = 50;
    c.SLOW_GOOD = 0;
    c.QUICK_BAD = 1;
  }
  
}
);

RegularizationAlgo.RegularizationLayout = declare(GraphLayout, 
{

  constructor : function()
  {
    
  },
  copy : function()
  {
    return null; // ibm_ilog.graphlayout.GraphLayout
  },

  layout : function(redraw)
  {
    // summary:
    // tags:
    //		private
    
  }

}
);

RegularizationAlgo.RegularizationLayout._getGlobalProperties = function()
{
  // tags:
  //		private
  return GraphLayout._getGlobalProperties();
};

RegularizationAlgo.RegularizationLayout._getNodeProperties = function()
{
  // tags:
  //		private
  return GraphLayout._getNodeProperties();
};

RegularizationAlgo.RegularizationLayout._getLinkProperties = function()
{
  // tags:
  //		private
  return GraphLayout._getLinkProperties();
};

(function(){ RegularizationAlgo._fields_init(); })();

return RegularizationAlgo;
});
