/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HEnum",	// #5
  "../hierarchical/HGraphAlgorithm",	// #6
  "../hierarchical/HSegment",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HEnum,	// #5
    HGraphAlgorithm,	// #6
    HSegment,	// #7
    GraphModelUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var AnchorPointOptimization = declare("ibm_ilog.graphlayout.hierarchical.AnchorPointOptimization", HGraphAlgorithm, 
{
  // summary:
  //		AnchorPointOptimization allows to optimize the anchor points of
  //		segments. 
  // description:
  //		AnchorPointOptimization allows to optimize the anchor points of
  //		segments. 
  //		The problem is mainly this: Given the actual anchorpoints in sorted
  //		form, and the wanted anchorpoints and a minimal distance between the
  //		anchor points, change as many actual anchor points as possible to the
  //		wanted anchor points without destroying the sorting. Example: the
  //		actual anchor points are 2 4 7 10 13 the wanted anchor points are 1 5
  //		3 11 11 and the minimal distance is 2, then the best solution is 1 5
  //		7 11 13  Note we cannot set 7 to 3 because it would violate the
  //		sorting. Also, we cannot set 13 to 11 because it would violate the
  //		minimal distance condition.  
  // tags:
  //		private
  
  _numberOfSegments : null,
  _segments: null,
  _portNumbers: null,
  _actAnchors: null,
  _wantedAnchors: null,
  _minDist: null,

  _epsilon : null,

  constructor : function(graph)
  {
    this._numberOfSegments = 0;
    this._epsilon = 0.00001;
    this.init(graph);
  },
  
  initFromSegments : function(iter, numberOfSegments, fromSide)
  {
    // summary:
    //		Initialize with the segments at either from or to side.  
    // tags:
    //		private
    
    this._numberOfSegments = numberOfSegments;
    if (this._numberOfSegments == 0) {
      return;
    }
    var segment;
    var i = 0;
    var dir = this.getGraph().getLevelFlow();
    var minSegmentDist = this.getGraph().getMinDistBetweenLinks(dir);
    var thickness;
    if (minSegmentDist <= 0.0) {
      this._epsilon = 0.0;
    }
    this._segments = TranslateUtil.newArray(numberOfSegments, null);
    this._actAnchors = TranslateUtil.newArray(numberOfSegments, 0);
    this._wantedAnchors = TranslateUtil.newArray(numberOfSegments, 0);
    this._minDist = TranslateUtil.newArray(numberOfSegments, 0);
    this._portNumbers = TranslateUtil.newArray(numberOfSegments, 0);

    while (iter.hasNext())
    {
      segment = iter.next();
      thickness = segment.getOwnerLink().getThickness();
      this._segments[i] = segment;
      this._portNumbers[i] = 0;
      if (fromSide) {
        this._actAnchors[i] = segment.getRealFromCoord(dir);
      }
      else {
        this._actAnchors[i] = segment.getRealToCoord(dir);
      }
      this._wantedAnchors[i] = segment.getWantedAnchorCoord();
      this._minDist[i] = 0.5 * (thickness + minSegmentDist);
      i++;
    }
  },

  initFromFromAndToSegments : function(fromSegIter, toSegIter, numberOfSegments, fromSide)
  {
    // summary:
    //		Initialize with the segments that have port numbers at from and to
    //		side, and with the segments that have no port numbers at the
    //		specified side. 
    // description:
    //		Initialize with the segments that have port numbers at from and to
    //		side, and with the segments that have no port numbers at the
    //		specified side. 
    //		The parameter numberOfSegments is the upper limit of how many
    //		segments we have.  
    // tags:
    //		private
    
    this._numberOfSegments = numberOfSegments;
    if (this._numberOfSegments == 0) {
      return;
    }
    var segment;
    var fromSegment;
    var toSegment;
    var i = 0;
    var dir = this.getGraph().getLevelFlow();
    var coord;
    var minSegmentDist = this.getGraph().getMinDistBetweenLinks(dir);
    var thickness;
    var portNumber;
    if (minSegmentDist <= 0.0) {
      this._epsilon = 0.0;
    }
    this._segments = TranslateUtil.newArray(numberOfSegments, null);
    this._portNumbers = TranslateUtil.newArray(numberOfSegments, 0);
    this._actAnchors = TranslateUtil.newArray(numberOfSegments, 0);
    this._wantedAnchors = TranslateUtil.newArray(numberOfSegments, 0);
    this._minDist = TranslateUtil.newArray(numberOfSegments, 0);

    if (fromSegIter.hasNext()) {
      fromSegment = fromSegIter.next();
    }
    else {
      fromSegment = null;
    }
    if (toSegIter.hasNext()) {
      toSegment = toSegIter.next();
    }
    else {
      toSegment = null;
    }
    while (fromSegment != null || toSegment != null)
    {
      if (fromSegment == null) {
        segment = toSegment;
      }
      else if (toSegment == null) {
        segment = fromSegment;
      }
      else if (fromSegment.getRealFromCoord(dir) < toSegment.getRealToCoord(dir)) {
        segment = fromSegment;
      }
      else {
        segment = toSegment;
      }
      if (segment == fromSegment)
      {
        coord = segment.getRealFromCoord(dir);
        if (fromSegIter.hasNext()) {
          fromSegment = fromSegIter.next();
        }
        else {
          fromSegment = null;
        }
        portNumber = segment.getFromPortNumber();
        if (portNumber == HEnum.UNSPECIFIED && !fromSide) {
          continue;
        }
        this._portNumbers[i] = portNumber - HEnum.UNSPECIFIED;
      }
      else
      {
        coord = segment.getRealToCoord(dir);
        if (toSegIter.hasNext()) {
          toSegment = toSegIter.next();
        }
        else {
          toSegment = null;
        }
        portNumber = segment.getToPortNumber();
        if (portNumber == HEnum.UNSPECIFIED && fromSide) {
          continue;
        }
        this._portNumbers[i] = -portNumber + HEnum.UNSPECIFIED;
      }

      thickness = segment.getOwnerLink().getThickness();
      this._segments[i] = segment;
      this._actAnchors[i] = coord;
      this._wantedAnchors[i] = segment.getWantedAnchorCoord();
      this._minDist[i] = 0.5 * (thickness + minSegmentDist);
      i++;
    }

    this._numberOfSegments = i;
  },

  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this._segments = null;
    this._actAnchors = null;
    this._wantedAnchors = null;
    this._minDist = null;
  },

  run : function()
  {
    // summary:
    //		Run the algorithm.
    // tags:
    //		private
    
    if (this._numberOfSegments == 0) {
      return;
    }
    var i;
    var j;
    var maxIndex = this._numberOfSegments - 1;
    var changed = true;
    var errorFactor = (1.0 - this._epsilon);

    if (this._wantedAnchors[0] < this._actAnchors[0]) {
      this._actAnchors[0] = this._wantedAnchors[0];
    }
    if (this._wantedAnchors[maxIndex] > this._actAnchors[maxIndex]) {
      this._actAnchors[maxIndex] = this._wantedAnchors[maxIndex];
    }
    while (changed)
    {
      changed = false;

      for (i = 1; i <= maxIndex; i++)
      {
        if (this._wantedAnchors[i] < this._actAnchors[i])
        {
          j = i - 1;

          while (j >= 0 && !this.isBlocked(i, j))
            j--;

          if (j < 0 || this._wantedAnchors[i] - this._actAnchors[j] >= errorFactor * (this._minDist[i] + this._minDist[j]))
          {
            this._actAnchors[i] = this._wantedAnchors[i];
            changed = true;
          }
        }
      }

      for (i = maxIndex - 1; i >= 0; i--)
      {
        if (this._wantedAnchors[i] > this._actAnchors[i])
        {
          j = i + 1;

          while (j < maxIndex && !this.isBlocked(i, j))
            j++;

          if (j > maxIndex || this._actAnchors[j] - this._wantedAnchors[i] >= errorFactor * (this._minDist[i] + this._minDist[j]))
          {
            this._actAnchors[i] = this._wantedAnchors[i];
            changed = true;
          }
        }
      }
    }

    for (i = 0; i < this._numberOfSegments; i++)
      this._segments[i].setWantedAnchorCoord(this._actAnchors[i]);
  },

  isBlocked : function(i, j)
  {
    // summary:
    //		Returns true if the segment i is blocked by segment j. 
    // description:
    //		Returns true if the segment i is blocked by segment j. 
    //		The field &#95;portNumbers contains the negative port number if the
    //		segment goes towards upper level, and the positive port number if
    //		the segment goes towards lower level. A segment is blocked by
    //		another segment, if both go in the same direction, or if the one
    //		towards upper level has a higher port number than the one towards
    //		lower level.
    // tags:
    //		private
    
    if (this._portNumbers[i] * this._portNumbers[j] >= 0) {
      return true; // Boolean
    }
    if (this._portNumbers[i] < 0)
    {
      if (this._portNumbers[j] <= -this._portNumbers[i]) {
        return true; // Boolean
      }
    }
    else
    {
      if (this._portNumbers[i] <= -this._portNumbers[j]) {
        return true; // Boolean
      }
    }

    return false; // Boolean
  }

}
);

return AnchorPointOptimization;
});
