/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HLevelAlgorithm",	// #5
  "../hierarchical/HSegment",	// #6
  "../internalutil/ArrayStableSort",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLevelAlgorithm,	// #5
    HSegment,	// #6
    ArrayStableSort,	// #7
    GraphModelUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm;
=====*/

var CalcCrossings = declare("ibm_ilog.graphlayout.hierarchical.CalcCrossings", HLevelAlgorithm, 
{
  // summary:
  //		CalcCrossings is an algorithm the calculates the number of crossings
  //		of the segments between two layers with respect to the current
  //		ordering of the nodes within the layers. 
  // description:
  //		CalcCrossings is an algorithm the calculates the number of crossings
  //		of the segments between two layers with respect to the current
  //		ordering of the nodes within the layers. 
  //		This is a sweepline algorithm with complexit O(#segments log
  //		#segments + #crossings) with sorting, which is usually better than
  //		the simple O(#segments^2) algorithm.  
  // tags:
  //		private
  
  _numCrossings: null,
  _segmentList: null,
  _outSegmentSort : null,
  _inSegmentSort : null,
  _withIntergraphLinks: null,

  constructor : function (a0)
  {
    // summary:
    //		The constructor.
    
    if(arguments.length == 0)
      this._constructor_overload_CalcCrossings_0();
    else if(arguments.length == 1)
      this._constructor_overload_CalcCrossings_1(a0);
  },
  
  _constructor_overload_CalcCrossings_0 : function()
  {

    this._outSegmentSort = new CalcCrossings.OutSegmentSort();
    this._inSegmentSort = new CalcCrossings.InSegmentSort();
  },
  
  _constructor_overload_CalcCrossings_1 : function(withIntergraphLinks )
  {
    this._outSegmentSort = new CalcCrossings.OutSegmentSort();
    this._inSegmentSort = new CalcCrossings.InSegmentSort();
    this._withIntergraphLinks = withIntergraphLinks;
  },
  
  init : function(upperLevel, lowerLevel)
  {
    // summary:
    //		Initializes the algorithm with the actual levels.
    // tags:
    //		private
    
    this.inherited("init", arguments, [upperLevel, lowerLevel]);
    this._segmentList = null;
  },

  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._segmentList = null;
  },

  run : function()
  {
    // summary:
    //		Start the algorithm.
    // tags:
    //		private
    
    this.getUpperLevel().updateInfoRecursivelly(this._withIntergraphLinks);
    this.getLowerLevel().updateInfoRecursivelly(this._withIntergraphLinks);

    var i = 0;
    var iter;
    if (this._withIntergraphLinks) {
      iter = this.getUpperLevel().getSegmentsFrom(true);
    }
    else {
      iter = this.getUpperLevel().getNormalSegmentsFrom();
    }
    while (iter.hasNext())
    {
      iter.next();
      i++;
    }

    if (i == 0)
    {
      this._numCrossings = 0;
      return;
    }

    var segments = TranslateUtil.newArray(i, null);
    i = 0;
    if (this._withIntergraphLinks) {
      iter = this.getUpperLevel().getSegmentsFrom(true);
    }
    else {
      iter = this.getUpperLevel().getNormalSegmentsFrom();
    }
    while (iter.hasNext())
      segments[i++] = iter.next();

    this._outSegmentSort.sort(segments);

    this.makeSegmentList(segments);

    this._inSegmentSort.sort(segments);

    this.countCrossings(segments);
  },

  getNumberOfCrossings : function(upperLevel, lowerLevel)
  {
    // summary:
    //		Returns the number of crossings.
    // tags:
    //		private
    
    this.init(upperLevel, lowerLevel);
    this.run();
    return this._numCrossings; // int
  },

  countCrossings : function(segments)
  {
    // summary:
    //		This is the real crossing calculation mechanism. 
    // description:
    //		This is the real crossing calculation mechanism. 
    //		It is a plane sweep. It is described in Bently, Ottmann, IEEE Trans.
    //		on Computers, C 28 No 9, pp. 643-647, 1979, or in any standard
    //		algorithm book that deals with plane sweep. This implementation here
    //		is a little bit different, so you may not see the sweeping anymore,
    //		but you get the idea why it works from there.
    // tags:
    //		private
    
    this._numCrossings = 0;

    for (var i = 0; i < segments.length; i++)
      this._numCrossings += this.getCrossingsAndRemove(segments[i]);
  },

  makeSegmentList : function(segments)
  {
    // summary:
    //		Convert a sorted array of segments into a list of segments.
    // tags:
    //		private
    
    if (segments.length == 0)
    {
      this._segmentList = null;
      return;
    }

    var last = this._segmentList = new CalcCrossings.ListCell();
    last.segment = segments[0];

    for (var i = 1; i < segments.length; i++)
    {
      last.next = new CalcCrossings.ListCell();
      last = last.next;
      last.segment = segments[i];
    }
  },

  getCrossingsAndRemove : function(segment)
  {
    // summary:
    //		Search the segment and removes it from the list. 
    // description:
    //		Search the segment and removes it from the list. 
    //		It returns the number of cells before the cell that contains the
    //		segment, which is also the number of crossings that this segment
    //		causes.
    // tags:
    //		private
    
    var prev = null;
    var act = this._segmentList;
    var numCrossings = 0;

    while (act.segment != segment)
    {
      prev = act;
      act = act.next;
      numCrossings++;
    }

    if (prev != null) {
      prev.next = act.next;
    }
    else {
      this._segmentList = act.next;
    }
    return numCrossings; // int
  }

}
);

lang.mixin(CalcCrossings,
{
}
);

CalcCrossings.ListCell = declare( null, 
{

  constructor : function()
  {
    
  },
  segment: null,
  next: null
}
);
CalcCrossings.OutSegmentSort = declare(ArrayStableSort, 
{

  constructor : function()
  {
  },
  
  compare : function(o1, o2)
  {
    var s1 = o1;
    var s2 = o2;
    var v1 = s1.getCrossRedFromPosition();
    var v2 = s2.getCrossRedFromPosition();
    if (v1 < v2) {
      return true; // Boolean
    }
    if (v1 > v2) {
      return false; // Boolean
    }
    return (s1.getCrossRedToPosition() <= s2.getCrossRedToPosition()); // Boolean
  }

}
);
CalcCrossings.InSegmentSort = declare(ArrayStableSort, 
{

  constructor : function()
  {
  },
  
  compare : function(o1, o2)
  {
    var s1 = o1;
    var s2 = o2;
    var v1 = s1.getCrossRedToPosition();
    var v2 = s2.getCrossRedToPosition();
    if (v1 < v2) {
      return true; // Boolean
    }
    if (v1 > v2) {
      return false; // Boolean
    }
    return (s1.getCrossRedFromPosition() <= s2.getCrossRedFromPosition()); // Boolean
  }

}
);

return CalcCrossings;
});
