/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/leveling/HLVEdge",	// #5
  "../hierarchical/leveling/HLVGraph",	// #6
  "../hierarchical/leveling/HLVNode",	// #7
  "../hierarchical/leveling/LevelingAlgorithm",	// #8
  "../hierarchical/makeacyclic/MakeAcyclicAlgorithm",	// #9
  "../hierarchical/HEnum",	// #10
  "../hierarchical/HGraphAlgorithm",	// #11
  "../hierarchical/HPercentages",	// #12
  "../hierarchical/HierarchicalExtremityConstraint",	// #13
  "../hierarchical/HierarchicalGroupSpreadConstraint",	// #14
  "../hierarchical/HierarchicalLevelRangeConstraint",	// #15
  "../hierarchical/HierarchicalNodeGroup",	// #16
  "../hierarchical/HierarchicalRelativeLevelConstraint",	// #17
  "../hierarchical/HierarchicalSameLevelConstraint",	// #18
  "../internalutil/AlignmentOptions",	// #19
  "../internalutil/GraphModelUtil",	// #20
  "../internalutil/TranslateUtil"	// #21
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLVEdge,	// #5
    HLVGraph,	// #6
    HLVNode,	// #7
    LevelingAlgorithm,	// #8
    MakeAcyclicAlgorithm,	// #9
    HEnum,	// #10
    HGraphAlgorithm,	// #11
    HPercentages,	// #12
    HierarchicalExtremityConstraint,	// #13
    HierarchicalGroupSpreadConstraint,	// #14
    HierarchicalLevelRangeConstraint,	// #15
    HierarchicalNodeGroup,	// #16
    HierarchicalRelativeLevelConstraint,	// #17
    HierarchicalSameLevelConstraint,	// #18
    AlignmentOptions,	// #19
    GraphModelUtil,	// #20
    TranslateUtil	// #21
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var CalcLevelingAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.CalcLevelingAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		CalcLevelingAlgorithm assigns level numbers to the hierarchical
  //		nodes. 
  // description:
  //		CalcLevelingAlgorithm assigns level numbers to the hierarchical
  //		nodes. 
  //		It respects the level constraints. It tries to assign levels such
  //		that the majority of the links flow from lower numbered levels to
  //		higher numbered levels. It works for cyclic graphs by automatically
  //		breaking the cycles.  
  //		<p>
  //		The actual work is performed by the leveling graph. All we do here is
  //		just creating the leveling graph, and then start the leveling
  //		algorithm, which does all the real work on the leveling graph, and
  //		finally to move the resulting level numbers back into the
  //		hierarchical nodes.  
  //		</p>
  //		<p>
  //		The reason why we use a separate leveling graph is because the
  //		constraints also imply leveling edges. Furthermore, certain links are
  //		ignored when the incremental mode is on. Therefore the leveling graph
  //		doesn't correspond one-to-one to the hierarchical graph.  
  //		</p>
  //		<p>
  //		 
  //		</p>
  // tags:
  //		private
  
  _levelingGraph: null,
  _sameLevelFlags: null,
  _levelingNodes: null,
  _constraintManager: null,
  _incrementalLayout: null,
  _needFirstBypassLevel: null,
  _northExtremityNode: null,
  _southExtremityNode: null,
  _northExtremityLevelingNode: null,
  _southExtremityLevelingNode: null,

  _levelingStrategy: null,
  _handleIntergraphLinks: null,

  constructor : function(graph, constraintManager)
  {
    this.init(graph);
    this._levelingGraph = new HLVGraph();
    var numNodes = graph.getNumberOfNodes();
    this._sameLevelFlags = TranslateUtil.newArray(numNodes, 0);
    this._levelingNodes = TranslateUtil.newArray(numNodes, null);
    this._constraintManager = constraintManager;
    this._incrementalLayout = graph.isIncrementalMode();
    this._needFirstBypassLevel = false;
  },
  
  setLevelingStrategy : function(strategy)
  {
    // summary:
    //		Sets the options of the algorithm.
    // tags:
    //		private
    
    this._levelingStrategy = strategy;
  },

  setHandleIntergraphLinks : function(flag)
  {
    // summary:
    //		Sets whether the algorithm handles intergraph links.
    // tags:
    //		private
    
    this._handleIntergraphLinks = flag;
  },

  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._levelingGraph = null;
    this._sameLevelFlags = null;
    this._levelingNodes = null;
    this._constraintManager = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var incrementalLevels = new Array();
    var graph = this.getGraph();

    var numNodes = graph.getNumberOfNodes();
    var numLinks = graph.getNumberOfLinks();
    var estimatedCycles = Math.floor((numLinks - numNodes) / 5);
    if (estimatedCycles < 0) {
      estimatedCycles = 0;
    }
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_PARTITIONING], estimatedCycles + 50 + Math.floor(numNodes * 1 / 15), false);

    graph.updateNodeIDs();

    var e = this._constraintManager.getGroups();

    while (e.hasNext())
      (e.next()).initializeBeforeLeveling();

    this.initLevelingNodeIndex();
    if (this._incrementalLayout) {
      this.handleIncrementalLeveling(incrementalLevels);
    }
    this.handleNorthSouthExtremityConstraintsPass1();
    this.handleSameLevelConstraints();
    this.createLevelingNodes();

    if (this._northExtremityNode != null) {
      this._northExtremityLevelingNode = this.getNodeLevelingNode(this._northExtremityNode);
    }
    else {
      this._northExtremityLevelingNode = null;
    }
    if (this._southExtremityNode != null) {
      this._southExtremityLevelingNode = this.getNodeLevelingNode(this._southExtremityNode);
    }
    else {
      this._southExtremityLevelingNode = null;
    }
    if (!this._incrementalLayout) {
      this.handleGroupSpreadConstraints();
    }
    this.handleNorthSouthExtremityConstraintsPass2();

    this.handleRelativeLevelConstraints();
    this.handleLinks();
    this.handleIntergraphLinks();
    if (this._incrementalLayout) {
      this.handleIncrementalLevelingOrder(incrementalLevels);
    }

    var mkAcyclAlg = new MakeAcyclicAlgorithm(this.getLevelingGraph(), graph.getLayout(), this.getPercController());
    mkAcyclAlg.run();

    this.getLevelingGraph().updatePropagationFlags();
    this.handleSpecLevelIndex();
    this.handleLevelRangeConstraints();

    mkAcyclAlg.clean();
    var crLevelAlg = new LevelingAlgorithm(this.getLevelingGraph(), graph.getLayout(), this.getPercController());
    crLevelAlg.setLevelingStrategy(this._levelingStrategy);
    crLevelAlg.run();
    crLevelAlg.clean();

    this.transferResult();
    this.checkBypassLevel();
    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  getLevelingGraph : function()
  {
    // summary:
    //		Returns the leveling graph.
    // tags:
    //		private
    
    return this._levelingGraph; // ibm_ilog.graphlayout.hierarchical.leveling.HLVGraph
  },

  getNodeLevelingNode : function(node)
  {
    // summary:
    //		Returns the leveling node of a hierarchical node.  
    // tags:
    //		private
    
    node = this.getGraph().getOwnedNode(node);
    return this._levelingNodes[node.getNumID()]; // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  },

  getOriginalNodeLevelingNode : function(node)
  {
    // summary:
    //		Returns the leveling node of an original node.  
    // tags:
    //		private
    
    return this._levelingNodes[this.getGraph().getNode(node).getNumID()]; // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  },

  needFirstBypassLevel : function()
  {
    // summary:
    //		Returns 'true' if we need a level before level 0, because
    //		there are two nodes at level 0 that have a link that bypasses the
    //		nodes at the north side.
    // tags:
    //		private
    
    return this._needFirstBypassLevel; // Boolean
  },

  initLevelingNodeIndex : function()
  {
    // summary:
    //		Returns the HNode that belongs to this graph. 
    // description:
    //		Returns the HNode that belongs to this graph. 
    //		When doing a recursive layout, the input node may be an inner node 
    //		of a subgraph. Find the ancestor that is in this graph. We use this
    //		only for// intergraph links, hence it works since intergraph links
    //		are always// in the common ancestor of both end nodes.//
    //		--------------------------------------------------------------------------//
    //		First, we have to evaluate all "same-level" constraints. This is
    //		just a// classical union-find problem. Complexity is nearly linear,
    //		see standard// Tarjan algo. in any good algorithm book
    //		(it's precisely O(n a(n, m)) for// n nodes and m unions, where a is the inverse Ackerman function which grows// notoriously slowly such that we can assume a(n, m) <= 5 for all cases).// Nodes have number IDs from 0 to n. So we simply allocate an array// &#95;sameLevelFlags such that://   &#95;sameLevelFlags[i] == i        if the index of the leveling node for//                                  node with numID i is i//   &#95;sameLevelFlags[i] == j (< i) if the index of the leveling node for//                                  node with numID i is actually the index//                                  of the leveling node for the node with//                                  numID j/** Initialize the union-find problem that ensures the same level. Initially, we assume each node with numID i has the leveling node with index i.
    // tags:
    //		private
    
    var numNodes = this.getGraph().getNumberOfNodes();

    for (var i = 0; i < numNodes; i++)
      this._sameLevelFlags[i] = i;
  },

  unionLevelNodeIndex : function(node1, node2)
  {
    // summary:
    //		Ensures that the input nodes are at the same level, by making sure
    //		that both have the same index for the leveling node. 
    // description:
    //		Ensures that the input nodes are at the same level, by making sure
    //		that both have the same index for the leveling node. 
    //		This is just the classical "Union" operation from the union-find
    //		problem.
    // tags:
    //		private
    
    var i1 = this.findLevelingNodeIndex(node1);
    var i2 = this.findLevelingNodeIndex(node2);

    if (i1 < i2) {
      this._sameLevelFlags[i2] = i1;
    }
    else if (i1 > i2) {
      this._sameLevelFlags[i1] = i2;
    }
  },

  findLevelingNodeIndex : function(node)
  {
    // summary:
    //		Finds the index of the leveling node for a hierarchical node.
    // tags:
    //		private
    
    return this.findLevelingNodeIndexImpl(node.getNumID()); // int
  },

  findLevelingNodeIndexImpl : function(numID)
  {
    // summary:
    //		Finds the index of the leveling node for a hierarchical node with
    //		index numID.  
    // tags:
    //		private
    
    var i = numID;

    while (this._sameLevelFlags[i] != i)
      i = this._sameLevelFlags[i];

    this._sameLevelFlags[numID] = i;
    return i; // int
  },

  createLevelingNodes : function()
  {
    // summary:
    //		Create the necessary leveling nodes.
    // tags:
    //		private
    
    var j;

    for (var i = 0; i < this._levelingNodes.length; i++)
    {
      if (this._sameLevelFlags[i] == i)
      {
        this._levelingNodes[i] = new HLVNode();
        this.getLevelingGraph().addNode(this._levelingNodes[i]);
      }
      else
      {
        j = this.findLevelingNodeIndexImpl(i);

        this._levelingNodes[i] = this._levelingNodes[j];
      }
    }

    this._sameLevelFlags = null;
  },

  handleSameLevelConstraints : function()
  {
    // summary:
    //		Handle all same level constraints, and group spread constraints with
    //		spead 0. 
    // description:
    //		Handle all same level constraints, and group spread constraints with
    //		spead 0. 
    //		For these, we force that they belong to the same level by ensuring
    //		that the same leveling node is used. This is performed before the
    //		leveling nodes are created.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var e = this._constraintManager.getConstraints();
    var constraint;

    while (e.hasNext())
    {
      constraint = e.next();
      if (!constraint.isValidForLayout()) {
        continue;
      }
      if (constraint instanceof HierarchicalSameLevelConstraint)
      {
        var slc = constraint;
        var node1 = graph.getNode(slc.getFirstNode());
        var node2 = graph.getNode(slc.getSecondNode());

        if (!this._incrementalLayout || node1.isMarkedForIncremental() || node2.isMarkedForIncremental()) {
          this.unionLevelNodeIndex(node1, node2);
        }
      }
      else if (constraint instanceof HierarchicalGroupSpreadConstraint)
      {
        var gsc = constraint;
        var g = gsc.getGroup();

        if (gsc.getSpreadSize() == 0 && g != null)
        {
          if (!this._incrementalLayout || g.size() - this.getMarkedForIncrementalSize(g) <= 1)
          {
            var node = null;
            var e2 = gsc.getGroup().elements();

            while (e2.hasNext())
            {
              if (node == null) {
                node = graph.getNode(e2.next());
              }
              else {
                this.unionLevelNodeIndex(node, graph.getNode(e2.next()));
              }
            }
          }
        }
      }
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  getMarkedForIncrementalSize : function(g)
  {
    // summary:
    //		Returns the number of nodes of the group that are marked for
    //		incremental leveling.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var numMarkedNodes = 0;
    var e = g.elements();

    while (e.hasNext())
    {
      var node = graph.getNode(e.next());
      if (node.isMarkedForIncremental()) {
        numMarkedNodes++;
      }
    }

    return numMarkedNodes; // int
  },

  handleGroupSpreadConstraints : function()
  {
    // summary:
    //		Handles all group spread constraints except those with spread 0. 
    // description:
    //		Handles all group spread constraints except those with spread 0. 
    //		This is performed before any other edge is added to the leveling
    //		graph, because we need to have ALL propagation group start nodes
    //		before adding any edge to the leveling graph.
    // tags:
    //		private
    
    var e = this._constraintManager.getConstraints();
    var constraint;

    while (e.hasNext())
    {
      constraint = e.next();
      if (!constraint.isValidForLayout()) {
        continue;
      }
      if (constraint instanceof HierarchicalGroupSpreadConstraint)
      {
        var c = constraint;
        var g = c.getGroup();
        var spreadSize = c.getSpreadSize();

        if (spreadSize > 0 && (!this._incrementalLayout || g.size() - this.getMarkedForIncrementalSize(g) <= 1))
        {
          var entryNode = this.getGroupEntryNode(g, 10000.0);
          var exitNode = this.getGroupExitNode(g, 10000.0);
          entryNode.connectToOtherGroupNode(exitNode, spreadSize);
          exitNode.connectToOtherGroupNode(entryNode, -spreadSize);
        }
      }
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  handleRelativeLevelConstraints : function()
  {
    // summary:
    //		Handles all relative level constraints. 
    // description:
    //		Handles all relative level constraints. 
    //		This is performed after the group spread constraints are performed,
    //		before the graph is made acyclic.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var e = this._constraintManager.getConstraints();
    var constraint;
    var source;
    var target;
    var ls;
    var hs;
    var sourceMark;
    var targetMark;
    var p;

    while (e.hasNext())
    {
      constraint = e.next();
      if (!constraint.isValidForLayout()) {
        continue;
      }
      if (constraint instanceof HierarchicalRelativeLevelConstraint)
      {
        var c = constraint;
        ls = c.getLowerSubject();
        hs = c.getHigherSubject();
        sourceMark = targetMark = true;
        p = c.getPriority();

        if (ls instanceof HierarchicalNodeGroup) {
          source = this.getGroupExitNode(ls, p);
        }
        else
        {
          source = this.getOriginalNodeLevelingNode(ls);
          sourceMark = graph.getNode(ls).isMarkedForIncremental();
        }

        if (hs instanceof HierarchicalNodeGroup) {
          target = this.getGroupEntryNode(hs, p);
        }
        else
        {
          target = this.getOriginalNodeLevelingNode(hs);
          targetMark = graph.getNode(hs).isMarkedForIncremental();
        }

        this.addEdge(source, target, sourceMark, targetMark, p, 1);
      }
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  handleLinks : function (a0, a1)
  {
    // summary:
    //		Handles all normal links. 
    // description:
    //		Handles all normal links. 
    //		This is basically the same mechanism as for relative level
    //		constraints.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.handleLinks_overload_0();
    else if(arguments.length == 2)
      return this.handleLinks_overload_2(a0, a1);
  },
  
  handleLinks_overload_0 : function()
                                    
  {
    this.handleLinks(this.getGraph().getLinks(), 1);
  },

  handleIntergraphLinks : function()
  {
    // summary:
    //		Handles all intergraph links. 
    // description:
    //		Handles all intergraph links. 
    //		This is basically the same mechanism as for relative level
    //		constraints.
    // tags:
    //		private
    
    if (!this._handleIntergraphLinks) {
      return;
    }
    this.handleLinks(this.getGraph().getIntergraphLinks(), 0.0001);
  },

  handleLinks_overload_2 : function(iter, prioFactor)
  {
    // summary:
    //		Common code for normal and intergraph links.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var link;
    var fromPortSide;
    var toPortSide;
    var minSpan;
    var source;
    var target;

    while (iter.hasNext())
    {
      link = iter.next();
      fromPortSide = link.getFromPortSide();
      toPortSide = link.getToPortSide();

      if ((fromPortSide == HEnum.NORTH && toPortSide != HEnum.NORTH) || (fromPortSide != HEnum.SOUTH && toPortSide == HEnum.SOUTH))
      {
        source = this.getNodeLevelingNode(link.getTo());
        target = this.getNodeLevelingNode(link.getFrom());
        minSpan = 1;
      }
      else if ((fromPortSide == HEnum.NORTH && toPortSide == HEnum.NORTH) || (fromPortSide == HEnum.SOUTH && toPortSide == HEnum.SOUTH))
      {
        source = target = null;
        minSpan = 0;
      }
      else
      {
        source = this.getNodeLevelingNode(link.getFrom());
        target = this.getNodeLevelingNode(link.getTo());
        minSpan = 1;
      }

      this.addEdge(source, target, graph.getOwnedNode(link.getFrom()).isMarkedForIncremental(), graph.getOwnedNode(link.getTo()).isMarkedForIncremental(),
          prioFactor * link.getStartSegment().getPriority(), minSpan);
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  addEdge : function(source, target, sourceMarkedForIncremental, targetMarkedForIncremental, priority, minSpan)
  {
    // summary:
    //		Auxiliary method that adds the leveling edge, if the incremental
    //		mode is disabled, or if either source or target are marked for
    //		incremental recalculation.
    // tags:
    //		private
    
    if (target == this._northExtremityLevelingNode)
    {
      target = source;
      source = this._northExtremityLevelingNode;
    }

    if (source == this._southExtremityLevelingNode)
    {
      source = target;
      target = this._southExtremityLevelingNode;
    }

    if (source != null && target != null && source != target)
    {
      if (!this._incrementalLayout || sourceMarkedForIncremental || targetMarkedForIncremental)
      {
        if (priority > 10000.0) {
          priority = 10000.0;
        }
        this.getLevelingGraph().addEdgeWithGroupCheck(new HLVEdge(priority, minSpan), source, target);
      }
    }
  },

  handleNorthSouthExtremityConstraintsPass1 : function()
  {
    // summary:
    //		Handle all north or south extremity constraints. 
    // description:
    //		Handle all north or south extremity constraints. 
    //		This is the first pass, where all nodes with such constraints are
    //		put into the same level. If incremental mode is enabled, we do this
    //		only for the nodes that are marked for incremental recalculation. We
    //		use two special leveling node: one for the first level, and another
    //		for the last level. We force all north extremity nodes to belong to
    //		the same level by ensuring that the same leveling node is used. Dito
    //		for all south extremity nodes. This is performed before the leveling
    //		nodes are created.
    // tags:
    //		private
    
    this._northExtremityNode = null;
    this._southExtremityNode = null;
    var graph = this.getGraph();
    var node;
    var e = this._constraintManager.getConstraints();
    var constraint;

    while (e.hasNext())
    {
      constraint = e.next();
      if (!constraint.isValidForLayout()) {
        continue;
      }
      if (constraint instanceof HierarchicalExtremityConstraint)
      {
        var c = constraint;

        if (c.getSide() == AlignmentOptions.NORTH)
        {
          node = graph.getNode(c.getNode());

          if (node.isMarkedForIncremental() || !this._incrementalLayout)
          {
            if (this._northExtremityNode == null) {
              this._northExtremityNode = node;
            }
            else {
              this.unionLevelNodeIndex(node, this._northExtremityNode);
            }
          }
        }
        else if (c.getSide() == AlignmentOptions.SOUTH)
        {
          node = graph.getNode(c.getNode());

          if (node.isMarkedForIncremental() || !this._incrementalLayout)
          {
            if (this._southExtremityNode == null) {
              this._southExtremityNode = node;
            }
            else {
              this.unionLevelNodeIndex(node, this._southExtremityNode);
            }
          }
        }
      }
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  handleNorthSouthExtremityConstraintsPass2 : function()
  {
    // summary:
    //		Handle all north or south extremity constraints. 
    // description:
    //		Handle all north or south extremity constraints. 
    //		This is the second pass, where we put leveling edges between the
    //		extremity leveling nodes and all other nodes. This is performed
    //		after the group spread constraints are performed, before the graph
    //		is made acyclic.
    // tags:
    //		private
    
    var levelingGraph = this.getLevelingGraph();
    var node;
    var iter;
    var edge;

    if (this._northExtremityNode != null)
    {
      var northNode = this.getNodeLevelingNode(this._northExtremityNode);
      iter = levelingGraph.getNodes(false);

      while (iter.hasNext())
      {
        node = iter.next();

        if (node != northNode)
        {
          edge = new HLVEdge(10000.0, 1);
          levelingGraph.addEdgeBetweenNodes(edge, northNode, node);
        }
      }
    }

    if (this._southExtremityNode != null)
    {
      var southNode = this.getNodeLevelingNode(this._southExtremityNode);
      iter = levelingGraph.getNodes(false);

      while (iter.hasNext())
      {
        node = iter.next();

        if (node != southNode)
        {
          edge = new HLVEdge(10000.0, 1);
          levelingGraph.addEdgeWithGroupCheck(edge, node, southNode);
        }
      }
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  handleLevelRangeConstraints : function()
  {
    // summary:
    //		Handles all level range constraints. 
    // description:
    //		Handles all level range constraints. 
    //		This is performed after the level graph was made acyclic and the
    //		propagation flags are up to date, because this may trigger a
    //		propagation.
    // tags:
    //		private
    
    var e = this._constraintManager.getConstraints();
    var constraint;
    var graph = this.getGraph();

    while (e.hasNext())
    {
      constraint = e.next();
      if (!constraint.isValidForLayout()) {
        continue;
      }
      if (constraint instanceof HierarchicalLevelRangeConstraint)
      {
        var c = constraint;
        var minLevel = c.getMinLevel();
        var maxLevel = c.getMaxLevel();
        var subject = c.getSubject();

        if (subject instanceof HierarchicalNodeGroup)
        {
          var e2 = (subject).elements();

          while (e2.hasNext())
            this.setMinMaxLevel(graph.getNode(e2.next()), minLevel, maxLevel);
        }
        else {
          this.setMinMaxLevel(graph.getNode(subject), minLevel, maxLevel);
        }
      }
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  handleSpecLevelIndex : function()
  {
    // summary:
    //		Handle specified level indexe. 
    // description:
    //		Handle specified level indexe. 
    //		This is basically the same mechanism as for level range constraints.
    //		This is performed after the level graph was made acyclic and the
    //		propagation flags are up to date, because this may trigger a
    //		propagation.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();
    var node;
    var levelNumber;

    while (iter.hasNext())
    {
      node = iter.next();
      levelNumber = node.getSpecLevelNumber();
      this.setMinMaxLevel(node, levelNumber, levelNumber);
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  setMinMaxLevel : function(node, minLevel, maxLevel)
  {
    // summary:
    //		Sets the min and max level of a node. 
    // description:
    //		Sets the min and max level of a node. 
    //		Minlevel and maxlevel constraints are ignored if the node is laid
    //		out incrementally.
    // tags:
    //		private
    
    if (!this._incrementalLayout || node.isMarkedForIncremental())
    {
      var levelingNode = this.getNodeLevelingNode(node);
      if (minLevel >= 0) {
        levelingNode.setMinLevelNumber(minLevel);
      }
      if (maxLevel >= 0) {
        levelingNode.setMaxLevelNumber(maxLevel);
      }
    }
  },

  getGroupEntryNode : function(g, priority)
  {
    // summary:
    //		Returns the group entry node. 
    // description:
    //		Returns the group entry node. 
    //		If none is there, it creates a new one and returns it. It also adds
    //		all the auxiliary edges with minspan 0 from the entry node to the
    //		group nodes.
    // tags:
    //		private
    
    if (g.getEntryNode() == null)
    {
      var groupEntryNode = new HLVNode();
      this.getLevelingGraph().addNode(groupEntryNode);
      g.setEntryNode(groupEntryNode);

      var e = g.elements();
      var groupNode;
      var edge;

      while (e.hasNext())
      {
        groupNode = this.getOriginalNodeLevelingNode(e.next());
        edge = new HLVEdge(0.0, 0);
        edge.makeUnbreakable();
        this.getLevelingGraph().addEdgeBetweenNodes(edge, groupEntryNode, groupNode);
      }
    }
    else {
      g.getEntryNode().setMinOutPriority(priority);
    }
    return g.getEntryNode(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  },

  getGroupExitNode : function(g, priority)
  {
    // summary:
    //		Returns the group exit node. 
    // description:
    //		Returns the group exit node. 
    //		If none is there, it creates a new one and returns it. It also adds
    //		all the auxiliary edges with minspan 0 from the group nodes to the
    //		exit.
    // tags:
    //		private
    
    if (g.getExitNode() == null)
    {
      var groupExitNode = new HLVNode();
      this.getLevelingGraph().addNode(groupExitNode);
      g.setExitNode(groupExitNode);

      var e = g.elements();
      var groupNode;
      var edge;

      while (e.hasNext())
      {
        groupNode = this.getOriginalNodeLevelingNode(e.next());
        edge = new HLVEdge(0.0, 0);
        edge.makeUnbreakable();
        this.getLevelingGraph().addEdgeBetweenNodes(edge, groupNode, groupExitNode);
      }
    }
    else {
      g.getExitNode().setMinInPriority(priority);
    }
    return g.getExitNode(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  },

  transferResult : function()
  {
    // summary:
    //		Transfers the resulting level numbers into the hierarchical nodes.
    // tags:
    //		private
    
    var levelNumber;
    var iter = this.getGraph().getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      levelNumber = this.getNodeLevelingNode(node).getLevelNumber();
      node.setLevelNumber(levelNumber);
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  checkBypassLevel : function()
  {
    // summary:
    //		Check whether we need a bypass level before the first level or after
    //		the last level.
    // tags:
    //		private
    
    this._needFirstBypassLevel = false;
    var iter = this.getGraph().getLinks();
    var link;

    while (iter.hasNext())
    {
      link = iter.next();
      if (link.getFrom().getLevelNumber() == 0 && link.getFromPortSide() == HEnum.NORTH) {
        this._needFirstBypassLevel = true;
      }
      if (link.getTo().getLevelNumber() == 0 && link.getToPortSide() == HEnum.NORTH) {
        this._needFirstBypassLevel = true;
      }
      if (this._needFirstBypassLevel) {
        return;
      }
    }

    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
  },

  handleIncrementalLeveling : function(incrLevels)
  {
    // summary:
    //		Partition the nodes that are not marked for incremental complete
    //		recalculation according to their coordinates. 
    // description:
    //		Partition the nodes that are not marked for incremental complete
    //		recalculation according to their coordinates. 
    //		This fills the vector <code>incrLevels</code> with objects of type
    //		<code>IncrementalLevel</code>. Each incremental level contains all
    //		the nodes that belong to it. All these nodes are set to be at the
    //		same level (they use the same leveling node). Note that nodes marked
    //		for incremental are not contained in any of the incremental levels.
    //		The incremental levels are ordered in the vector according to
    //		increasing cordinate in flow direction.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var dir = graph.getEdgeFlow();
    var levelRange = 0.5 * graph.getMinDistBetweenNodes(dir);

    var nodeArray = graph.getNodesSortedByCoordForIncremental(dir);
    this.layoutStepPerformed();

    var node;
    var level;
    var bestLevel;
    var distToLevel;
    var distToBestLevel;
    var indexFirstRelevantLevel = 0;
    var j;

    for (var i = 0; i < nodeArray.length; i++)
    {
      node = nodeArray[i];

      if (!node.isMarkedForIncremental())
      {
        node.setSpecLevelNumber(-1);
        node.setSpecPositionInLevel(-1);
        bestLevel = null;
        distToBestLevel = 0.0;

        for (j = indexFirstRelevantLevel; j < incrLevels.length; j++)
        {
          level = incrLevels[j];

          if (level.mayContain(node, levelRange))
          {
            distToLevel = level.getReferenceCoord() - node.getCenter(dir);
            if (distToLevel < 0.0) {
              distToLevel = -distToLevel;
            }
            if (bestLevel == null)
            {
              bestLevel = level;
              distToBestLevel = distToLevel;
            }
            else if (distToLevel < distToBestLevel)
            {
              bestLevel = level;
              distToBestLevel = distToLevel;
            }
          }
          else {
            indexFirstRelevantLevel = j + 1;
          }
        }

        if (bestLevel != null) {
          bestLevel.addNode(node);
        }
        else {
          incrLevels.push(new CalcLevelingAlgorithmIncrementalLevel(this, node));
        }
      }

      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  handleIncrementalLevelingOrder : function(incrementalLevels)
  {
    // summary:
    //		Make sure that the incremental levels are placed in the correct
    //		order. 
    // description:
    //		Make sure that the incremental levels are placed in the correct
    //		order. 
    //		We achieve this by creating leveling edges of high priority between
    //		the leveling nodes that represent the incremental levels.
    // tags:
    //		private
    
    var e =  new TranslateUtil.ArrayIterator(incrementalLevels);
    var levelingGraph = this.getLevelingGraph();
    var prevLevelingNode = null;
    var levelingNode;

    while (e.hasNext())
    {
      levelingNode = this.getNodeLevelingNode((e.next()).getReferenceNode());

      if (prevLevelingNode != null)
      {
        var edge = new HLVEdge(10000.0, 1);
        edge.makeUnbreakable();
        levelingGraph.addEdgeWithGroupCheck(edge, prevLevelingNode, levelingNode);
      }

      prevLevelingNode = levelingNode;
    }

    this.getPercController().addPoints(3);
    this.layoutStepPerformed();
  }

}
);
CalcLevelingAlgorithmIncrementalLevel = declare( null, 
{
  // summary:
  //		An incremental hierarchical level is an auxiliary data structure
  //		needed for incremental layout. 
  // description:
  //		An incremental hierarchical level is an auxiliary data structure
  //		needed for incremental layout. 
  //		The real nodes are partitioned according to their coordinates into
  //		different incremental levels. All nodes of the same incremental level
  //		are handled similar as a "same-level-constraint", they are forced to
  //		the same level using the same leveling node for all these
  //		hierarchical nodes.
  
  __outerThis: null,

  _referenceNode: null,

  _dir: null,
  _numberOfNodes: null,

  _sumMinCoords: null,
  _sumMaxCoords: null,
  _sumCenterCoords: null,

  constructor : function(input__outerThis, node)
  {
    this.__outerThis = input__outerThis;

    var graph = node.getOwnerGraph();
    this._dir = graph.getEdgeFlow();
    var size = node.getOldSizeInEdgeFlow();
    var coord = node.getCenter(this._dir) - 0.5 * size;
    this._sumMinCoords = coord;
    coord += size;
    this._sumMaxCoords = coord;
    this._sumCenterCoords = node.getCenter(this._dir);
    this._numberOfNodes = 1;
    this._referenceNode = node;
  },
  
  addNode : function(node)
  {
    // summary:
    //		Adds a node to this incremental level.
    // tags:
    //		private
    
    var size = node.getOldSizeInEdgeFlow();
    var coord = node.getCenter(this._dir) - 0.5 * size;
    this._sumMinCoords += coord;
    coord += size;
    this._sumMaxCoords += coord;
    this._sumCenterCoords += node.getCenter(this._dir);
    this._numberOfNodes++;

    this.__outerThis.unionLevelNodeIndex(node, this._referenceNode);
  },

  getReferenceCoord : function()
  {
    // summary:
    //		Returns the reference line of this level. 
    // description:
    //		Returns the reference line of this level. 
    //		The area +/- levelRange around this reference line belongs to this
    //		level.
    // tags:
    //		private
    
    return (this._sumCenterCoords / this._numberOfNodes); // Number
  },

  getReferenceNode : function()
  {
    // summary:
    //		Returns the reference node for this level.
    // tags:
    //		private
    
    return this._referenceNode; // ibm_ilog.graphlayout.hierarchical.HNode
  },

  mayContain : function(node, levelRange)
  {
    // summary:
    //		Returns true if the input node may belong to this incremental level.
    //		
    // description:
    //		Returns true if the input node may belong to this incremental level.
    //		
    //		Since a node may belong to many incremental levels at the same time,
    //		this is not yet the final decision whether the node belongs to this
    //		level.
    // tags:
    //		private
    
    var size = node.getOldSizeInEdgeFlow();
    var nodeMinCoord = node.getCenter(this._dir) - 0.5 * size;
    var nodeMaxCoord = nodeMinCoord + size;

    if (this._sumMinCoords / this._numberOfNodes <= nodeMaxCoord && nodeMinCoord <= this._sumMaxCoords / this._numberOfNodes) {
      return true; // Boolean
    }

    var levelRangeMin = this.getReferenceCoord() - levelRange;
    var levelRangeMax = levelRangeMin + 2 * levelRange;
    if (levelRangeMin <= nodeMaxCoord && nodeMinCoord <= levelRangeMax) {
      return true; // Boolean
    }
    return false; // Boolean
  }

}
);

return CalcLevelingAlgorithm;
});
