/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HEnum",	// #5
  "../hierarchical/HGraphAlgorithm",	// #6
  "../hierarchical/HSegmentSetMarker",	// #7
  "../internalutil/ArrayStableSort",	// #8
  "../internalutil/GraphModelUtil",	// #9
  "../internalutil/TranslateUtil"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HEnum,	// #5
    HGraphAlgorithm,	// #6
    HSegmentSetMarker,	// #7
    ArrayStableSort,	// #8
    GraphModelUtil,	// #9
    TranslateUtil	// #10
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var CrossRedSelfLinkAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.CrossRedSelfLinkAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		Since self-links are represented as orthogonal links, this is nearly
  //		the same algorithm as in OrthRoutingLevelAlgorithm. 
  // description:
  //		Since self-links are represented as orthogonal links, this is nearly
  //		the same algorithm as in OrthRoutingLevelAlgorithm. 
  //		It orders the orthogonal self-link segments to minimize the
  //		crossings. At that time point, the real orthogonal segments
  //		don't need to exist yet, but we need to know where they start and end. That means for self-links that the connector points must already be known. This class only builds the interval graph and orders the segments within the interval graph. It does not calculate the final bends yet.  
  // tags:
  //		private
  
  _segmentMarker: null,
  _numNormalSegments: null,

  _intervals: null,
  _startIntervals: null,
  _endIntervals: null,

  _segmentDir: null,

  _minCoord: null,

  _maxCoord: null,

  _side: null,

  _node: null,

  _minDistNodeLink: null,
  _minDistLinkLink: null,

  constructor : function(graph)
  {
    this._segmentMarker = new HSegmentSetMarker(graph, false);
    this._numNormalSegments = graph.getNumberOfNormalSegments();
    this._intervals = TranslateUtil.newArray(graph.getNumberOfNormalSegments() + graph.getNumberOfIntergraphSegments(), null);

    this.init(graph);

    this.getGraph().updateSegmentIDs();
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    if (this._segmentMarker != null) {
      this._segmentMarker.clean();
    }
    this._segmentMarker = null;
    this._intervals = null;
    this._startIntervals = null;
    this._endIntervals = null;
  },

  initNodeSide : function(node, side)
  {
    // summary:
    //		Initialize the algorithm for a given node and size.  
    // tags:
    //		private
    
    this._node = node;
    this._side = side;
    var graph = this.getGraph();
    var xdir = graph.getLevelFlow();
    var ydir = 1 - xdir;

    switch (side)
    {
      case HEnum.EAST:
      case HEnum.WEST:
        this._segmentDir = ydir;
        this._minCoord = node.getCoord(ydir);
        this._maxCoord = node.getCoord(ydir) + node.getSize(ydir);
        this._minDistNodeLink = graph.getMinDistBetweenNodeAndLink(xdir);
        this._minDistLinkLink = graph.getMinDistBetweenLinks(xdir);
        break;
      case HEnum.NORTH:
      case HEnum.SOUTH:
        this._segmentDir = xdir;
        this._minCoord = node.getCoord(xdir);
        this._maxCoord = node.getCoord(xdir) + node.getSize(xdir);
        this._minDistNodeLink = graph.getMinDistBetweenNodeAndLink(ydir);
        this._minDistLinkLink = graph.getMinDistBetweenLinks(ydir);
        break;
    }
  },

  run : function()
  {
    // summary:
    //		Start the algorithm. 
    // description:
    //		Start the algorithm. 
    //		Can only be done after init.
    // tags:
    //		private
    
    if (this._node.getSelfLinkInfo().getSelfLinksCount() == 0) {
      return;
    }
    this.calcIntervalGraph();

    this.calcSegmentDistances();

    this.cleanSegmentIntervals();
  },

  createSegmentIntervals : function()
  {
    // summary:
    //		Create all segment intervals needed between this upper and lower
    //		level. 
    // description:
    //		Create all segment intervals needed between this upper and lower
    //		level. 
    //		The intervals are stored in the field &#95;intervals at index position
    //		of the first segment of each interval, and again in the fields
    //		called &#95;startIntervals and &#95;endIntervals, which we need for the
    //		plane sweep later.
    // tags:
    //		private
    
    var selfLinkInfo = this._node.getSelfLinkInfo();
    var interval;
    var zone0Links = selfLinkInfo.getSelfLinks2(this._side, 0);
    var zone1Links = selfLinkInfo.getSelfLinks2(this._side, 1);
    var bypassLinks = selfLinkInfo.getBypassSelfLinks(this._side);

    var n = zone0Links.length + zone1Links.length + bypassLinks.length;
    this._startIntervals = TranslateUtil.newArray(n, null);
    this._endIntervals = TranslateUtil.newArray(n, null);

    var i = 0;
    var j = 0;

    for (j = 0; j < zone0Links.length; j++)
    {
      interval = new CrossRedSelfLinkAlgorithmSegmentInterval(this, zone0Links[j], true);
      this._startIntervals[i] = interval;
      this._endIntervals[i] = interval;
      i++;
    }

    for (j = 0; j < zone1Links.length; j++)
    {
      interval = new CrossRedSelfLinkAlgorithmSegmentInterval(this, zone1Links[j], false);
      this._startIntervals[i] = interval;
      this._endIntervals[i] = interval;
      i++;
    }

    for (j = 0; j < bypassLinks.length; j++)
    {
      interval = new CrossRedSelfLinkAlgorithmSegmentInterval(this, bypassLinks[j], false);
      this._startIntervals[i] = interval;
      this._endIntervals[i] = interval;
      i++;
    }
  },

  cleanSegmentIntervals : function()
  {
    // summary:
    //		Clean all segment intervals needed between this upper and lower
    //		level. 
    // description:
    //		Clean all segment intervals needed between this upper and lower
    //		level. 
    //		Not really necessary, but nice for the memory.
    // tags:
    //		private
    
    for (var i = 0; i < this._startIntervals.length; i++)
    {
      this._intervals[this.getIntervalID(this._startIntervals[i].segment)] = null;
      this._startIntervals[i].clean();
    }

    this._startIntervals = this._endIntervals = null;
  },

  calcSegmentDistances : function()
  {
    // summary:
    //		Calculates the distance of the segments from the node border. 
    // description:
    //		Calculates the distance of the segments from the node border. 
    //		Note that this is always a positive distance. Additionally it fills
    //		the nodes self-link info with the necessary margin.
    // tags:
    //		private
    
    var roots = new Array();
    var interval;
    var largerInterval;
    var dist;
    var maxMargin;
    var thickness;
    var e;

    maxMargin = 0.0;

    for (var i = 0; i < this._startIntervals.length; i++)
    {
      interval = this._startIntervals[i];
      thickness = this._startIntervals[i].getThickness();
      interval.setDistFromBorder(this._minDistNodeLink + 0.5 * thickness);
      if (maxMargin < this._minDistNodeLink + thickness) {
        maxMargin = this._minDistNodeLink + thickness;
      }
      if (interval.nrSmallerIntervals == 0) {
        roots.push(interval);
      }
    }

    while (!(roots.length==0))
    {
      interval = roots.pop();
      dist = interval.getDistFromBorder() + 0.5 * interval.getThickness() + this._minDistLinkLink;
      e =  new TranslateUtil.ArrayIterator(interval.largerIntervals);

      while (e.hasNext())
      {
        largerInterval = e.next();
        thickness = largerInterval.getThickness();
        largerInterval.setDistFromBorderAtLeastTo(dist + 0.5 * thickness);
        largerInterval.nrSmallerIntervals--;
        if (maxMargin < dist + thickness) {
          maxMargin = dist + thickness;
        }
        if (largerInterval.nrSmallerIntervals == 0) {
          roots.push(largerInterval);
        }
      }
    }

    roots = null;
    this._node.getSelfLinkInfo().setAdditionalSpace(this._side, maxMargin);
  },

  calcIntervalGraph : function()
  {
    // summary:
    //		Calculates the interval graph.
    // tags:
    //		private
    
    this.createSegmentIntervals();
    new CrossRedSelfLinkAlgorithmSegmentIntervalLowerBoundSort().sort(this._startIntervals);
    new CrossRedSelfLinkAlgorithmSegmentIntervalHigherBoundSort().sort(this._endIntervals);

    var startIndex = 0;
    var endIndex = 0;
    var maxIndex = this._startIntervals.length;
    var interval;

    while (startIndex < maxIndex || endIndex < maxIndex)
    {
      if (startIndex < maxIndex && endIndex < maxIndex)
      {
        if (this._startIntervals[startIndex].getLowerCoord() <= this._endIntervals[endIndex].getHigherCoord())
        {
          interval = this._startIntervals[startIndex++];
          this.resolveConflicts(interval);
          this.makeActive(interval);
        }
        else
        {
          interval = this._endIntervals[endIndex++];
          this.makeInactive(interval);
        }
      }
      else if (startIndex < maxIndex)
      {
        interval = this._startIntervals[startIndex++];
        this.resolveConflicts(interval);
        this.makeActive(interval);
      }
      else
      {
        interval = this._endIntervals[endIndex++];
        this.makeInactive(interval);
      }
    }
  },

  resolveConflicts : function(inputInterval)
  {
    // summary:
    //		Resolves conflicts between intervals by deciding for each currently
    //		active, conflicting interval whether it is smaller or larger than
    //		the input interval, which is about to become active. 
    // description:
    //		Resolves conflicts between intervals by deciding for each currently
    //		active, conflicting interval whether it is smaller or larger than
    //		the input interval, which is about to become active. 
    //		This actually sets the interval handles to larger intervals in the
    //		vector largerIntervals.
    // tags:
    //		private
    
    var activeInterval;
    var iter = this.getActiveIntervals();

    while (iter.hasNext())
    {
      activeInterval = iter.next();

      if (this.haveConflict(activeInterval, inputInterval))
      {
        if (this.isSmaller(activeInterval, inputInterval))
        {
          activeInterval.largerIntervals.push(inputInterval);
          inputInterval.nrSmallerIntervals++;
        }
        else
        {
          inputInterval.largerIntervals.push(activeInterval);
          activeInterval.nrSmallerIntervals++;
        }
      }
    }
  },

  haveConflict : function(i1, i2)
  {
    // summary:
    //		Returns true if both active segments interval are in conflict. 
    // description:
    //		Returns true if both active segments interval are in conflict. 
    //		The interval i1 is already active, the interval i2 becomes active
    //		right now.
    // tags:
    //		private
    
    if (i1.intervalType == CrossRedSelfLinkAlgorithm.HL || i1.intervalType == CrossRedSelfLinkAlgorithm.LL) {
      if (i2.intervalType == CrossRedSelfLinkAlgorithm.HH || i2.intervalType == CrossRedSelfLinkAlgorithm.HL) {
        return true; // Boolean
      }
    }
    if (i1.intervalType == CrossRedSelfLinkAlgorithm.HH || i1.intervalType == CrossRedSelfLinkAlgorithm.LH) {
      if (i2.intervalType == CrossRedSelfLinkAlgorithm.LH || i2.intervalType == CrossRedSelfLinkAlgorithm.LL) {
        return true; // Boolean
      }
    }
    return CrossRedSelfLinkAlgorithm.overlaps(i1, i2); // Boolean
  },

  isSmaller : function(i1, i2)
  {
    // summary:
    //		Returns true if the first segment interval needs a smaller layer
    //		than the second segment interval in orthogonal routing.
    // tags:
    //		private
    
    if (i1.intervalType != i2.intervalType)
    {
      if (i1.intervalType == CrossRedSelfLinkAlgorithm.HH) {
        return true; // Boolean
      }
      if (i2.intervalType == CrossRedSelfLinkAlgorithm.HH) {
        return false; // Boolean
      }
      if (i1.intervalType == CrossRedSelfLinkAlgorithm.LL) {
        return false; // Boolean
      }
      if (i2.intervalType == CrossRedSelfLinkAlgorithm.LL) {
        return true; // Boolean
      }
      if (i1.intervalType == CrossRedSelfLinkAlgorithm.HL) {
        return true; // Boolean
      }
      return false; // Boolean
    }

    if (i1.intervalType == CrossRedSelfLinkAlgorithm.HH)
    {
      if (i1.higherCoord < i2.higherCoord) {
        return true; // Boolean
      }
      if (i1.higherCoord > i2.higherCoord) {
        return false; // Boolean
      }
      if (i1.lowerCoord > i2.lowerCoord) {
        return true; // Boolean
      }
      if (i1.lowerCoord < i2.lowerCoord) {
        return false; // Boolean
      }
      return this.getIntervalID(i1.segment) < this.getIntervalID(i2.segment); // Boolean
    }
    else if (i1.intervalType == CrossRedSelfLinkAlgorithm.LL)
    {
      var i1NeedsSmaller = false;
      var testSide = HEnum.EAST;

      switch (this._side)
      {
        case HEnum.SOUTH:
          testSide = HEnum.EAST;
          i1NeedsSmaller = false;
          break;
        case HEnum.NORTH:
          testSide = HEnum.EAST;
          i1NeedsSmaller = true;
          break;
        case HEnum.WEST:
          testSide = HEnum.NORTH;
          i1NeedsSmaller = true;
          break;
        case HEnum.EAST:
          testSide = HEnum.NORTH;
          i1NeedsSmaller = false;
          break;
      }
      var link1 = i1.segment.getOwnerLink();
      var link2 = i2.segment.getOwnerLink();
      var c1;
      var c2;
      if (link1.getFromPortSide() == testSide) {
        c1 = link1.getFromCoord(1 - this._segmentDir);
      }
      else {
        c1 = link1.getToCoord(1 - this._segmentDir);
      }
      if (link2.getFromPortSide() == testSide) {
        c2 = link2.getFromCoord(1 - this._segmentDir);
      }
      else {
        c2 = link2.getToCoord(1 - this._segmentDir);
      }
      if (c1 < c2) {
        return i1NeedsSmaller; // Boolean
      }
      if (c1 > c2) {
        return !i1NeedsSmaller; // Boolean
      }
      testSide = (testSide + 2) % 4;
      if (link1.getFromPortSide() == testSide) {
        c1 = link1.getFromCoord(1 - this._segmentDir);
      }
      else {
        c1 = link1.getToCoord(1 - this._segmentDir);
      }
      if (link2.getFromPortSide() == testSide) {
        c2 = link2.getFromCoord(1 - this._segmentDir);
      }
      else {
        c2 = link2.getToCoord(1 - this._segmentDir);
      }
      if (c1 < c2) {
        return i1NeedsSmaller; // Boolean
      }
      if (c1 > c2) {
        return !i1NeedsSmaller; // Boolean
      }
      return this.getIntervalID(i1.segment) < this.getIntervalID(i2.segment); // Boolean
    }
    else if (i1.intervalType == CrossRedSelfLinkAlgorithm.LH)
    {
      if (i2.higherCoord > i1.higherCoord || i2.higherCoord == i1.higherCoord && i2.lowerCoord > i1.lowerCoord) {
        return true; // Boolean
      }
      return false; // Boolean
    }
    else
    {
      if (i2.lowerCoord < i1.lowerCoord || i2.lowerCoord == i1.lowerCoord && i2.higherCoord < i1.higherCoord) {
        return true; // Boolean
      }
      return false; // Boolean
    }
  },

  makeActive : function(i)
  {
    // summary:
    //		Adds an interval to the set of active segment intervals.
    // tags:
    //		private
    
    this._segmentMarker.insert(i.segment);
  },

  makeInactive : function(i)
  {
    // summary:
    //		Removes an interval from the set of active segment intervals.
    // tags:
    //		private
    
    this._segmentMarker.discard(i.segment);
  },

  getActiveIntervals : function()
  {
    // summary:
    //		Returns the active segments.
    // tags:
    //		private
    
    return new CrossRedSelfLinkAlgorithmAnonClass_1(this); // ibm_ilog.graphlayout.hierarchical.CrossRedSelfLinkAlgorithm.IntervalIterator
  },

  getFromCoord : function(segment, dir)
  {
    // summary:
    //		Returns the from coordinate of the segment in the coordinate system
    //		of this graph.
    // tags:
    //		private
    
    return segment.getRawFromCoord(dir); // Number
  },

  getToCoord : function(segment, dir)
  {
    // summary:
    //		Returns the to coordinate of the segment in the coordinate system of
    //		this graph.
    // tags:
    //		private
    
    return segment.getRawToCoord(dir); // Number
  },

  getIntervalID : function(segment)
  {
    // summary:
    //		Returns the ID of a segment.
    // tags:
    //		private
    
    if (segment.isIntergraphEdge()) {
      return this._numNormalSegments + segment.getNumID(); // int
    }
    return segment.getNumID(); // int
  }

}
);

lang.mixin(CrossRedSelfLinkAlgorithm,
{
  HL : null,
  LH : null,
  LL : null,
  HH : null,

  overlaps : function(a, b)
  {
    return a.lowerCoord <= b.higherCoord && a.higherCoord >= b.lowerCoord; // Boolean
  },

  _fields_init : function(c)
  {
    c = c || CrossRedSelfLinkAlgorithm;
    if(HGraphAlgorithm._fields_init)
      HGraphAlgorithm._fields_init(c);
    
    c.HL = 0;
    c.LH = 1;
    c.LL = 2;
    c.HH = 3;
  }
  
}
);

CrossRedSelfLinkAlgorithmSegmentInterval = declare( null, 
{
  __outerThis: null,
  intervalType: null,
  lowerCoord: null,
  higherCoord: null,
  segment: null,
  nrSmallerIntervals: null,
  largerIntervals: null,

  constructor : function(input__outerThis, link, isSubZone0)
  {
    this.__outerThis = input__outerThis;
    var dir = this.__outerThis._segmentDir;
    var c1;
    var c2;
    var delta;
    var thickness = link.getThickness();
    var fromPortSide = link.getFromPortSide();
    var toPortSide = link.getToPortSide();

    if (fromPortSide == this.__outerThis._side && toPortSide == this.__outerThis._side)
    {
      this.intervalType = CrossRedSelfLinkAlgorithm.HH;
      c1 = this.__outerThis.getFromCoord(link.getStartSegment(), dir);
      c2 = this.__outerThis.getToCoord(link.getEndSegment(), dir);
      this.lowerCoord = Math.min(c1, c2) - 0.5 * thickness;
      this.higherCoord = Math.max(c1, c2) + 0.5 * thickness;
      this.segment = link.getStartSegment().getTo().getIntergraphOpposite(link.getStartSegment());
    }
    else if (fromPortSide == this.__outerThis._side)
    {
      this.intervalType = (isSubZone0 ? CrossRedSelfLinkAlgorithm.LH : CrossRedSelfLinkAlgorithm.HL);
      delta = ((isSubZone0 ? 0.5 : -0.5)) * thickness;
      c1 = this.__outerThis.getFromCoord(link.getStartSegment(), dir) + delta;
      c2 = (isSubZone0 ? this.__outerThis._minCoord : this.__outerThis._maxCoord);
      this.lowerCoord = Math.min(c1, c2);
      this.higherCoord = Math.max(c1, c2);
      this.segment = link.getStartSegment().getTo().getIntergraphOpposite(link.getStartSegment());
    }
    else if (toPortSide == this.__outerThis._side)
    {
      this.intervalType = (isSubZone0 ? CrossRedSelfLinkAlgorithm.LH : CrossRedSelfLinkAlgorithm.HL);
      delta = ((isSubZone0 ? 0.5 : -0.5)) * thickness;
      c1 = this.__outerThis.getToCoord(link.getEndSegment(), dir) + delta;
      c2 = (isSubZone0 ? this.__outerThis._minCoord : this.__outerThis._maxCoord);
      this.lowerCoord = Math.min(c1, c2);
      this.higherCoord = Math.max(c1, c2);
      this.segment = link.getEndSegment().getFrom().getIntergraphOpposite(link.getEndSegment());
    }
    else
    {
      this.intervalType = CrossRedSelfLinkAlgorithm.LL;
      c1 = this.__outerThis._minCoord;
      c2 = this.__outerThis._maxCoord;
      this.lowerCoord = Math.min(c1, c2);
      this.higherCoord = Math.max(c1, c2);
      var auxs = link.getStartSegment();
      var auxn = auxs.getTo();
      auxs = auxn.getIntergraphOpposite(auxs);
      auxn = auxs.getOpposite(auxn);
      this.segment = auxn.getIntergraphOpposite(auxs);
    }

    this.__outerThis._intervals[this.__outerThis.getIntervalID(this.segment)] = this;
    this.nrSmallerIntervals = 0;
    this.largerIntervals = new Array();
  },
  
  clean : function()
  {
    this.segment = null;
    this.largerIntervals = null;
  },

  getLowerCoord : function()
  {
    return this.lowerCoord; // Number
  },

  getHigherCoord : function()
  {
    return this.higherCoord; // Number
  },

  getThickness : function()
  {
    return this.segment.getOwnerLink().getThickness(); // Number
  },

  setDistFromBorder : function(dist)
  {
    this.segment.setOrthogonalBend(dist);
  },

  setDistFromBorderAtLeastTo : function(dist)
  {
    if (dist > this.segment.getOrthogonalBend()) {
      this.setDistFromBorder(dist);
    }
  },

  getDistFromBorder : function()
  {
    return this.segment.getOrthogonalBend(); // Number
  },

  debugPrint : function()
  {
    switch (this.intervalType)
    {
      case CrossRedSelfLinkAlgorithm.LL:
        trace("LL");
        break;
      case CrossRedSelfLinkAlgorithm.HH:
        trace("HH");
        break;
      case CrossRedSelfLinkAlgorithm.HL:
        trace("HL");
        break;
      case CrossRedSelfLinkAlgorithm.LH:
        trace("LH");
        break;
    }
    trace("" + this.getLowerCoord() + " .. " + this.getHigherCoord());
  }

}
);
CrossRedSelfLinkAlgorithmSegmentIntervalLowerBoundSort = declare(ArrayStableSort, 
{

  constructor : function()
  {
  },
  
  compare : function(o1, o2)
  {
    return (o1).getLowerCoord() <= (o2).getLowerCoord(); // Boolean
  }

}
);
CrossRedSelfLinkAlgorithmSegmentIntervalHigherBoundSort = declare(ArrayStableSort, 
{

  constructor : function()
  {
  },
  
  compare : function(o1, o2)
  {
    return (o1).getHigherCoord() <= (o2).getHigherCoord(); // Boolean
  }

}
);
CrossRedSelfLinkAlgorithm.IntervalIterator = declare(null, 
{
  hasNext : function()
  {
  },
  
  next : function()
  {
  }
  
});
/**
 * @private
 */
CrossRedSelfLinkAlgorithmAnonClass_1 = declare(CrossRedSelfLinkAlgorithm.IntervalIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  i: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.i = this.__outerThis._segmentMarker.getElements();
  },

  hasNext : function()
  {
    return this.i.hasNext(); // Boolean
  },

  next : function()
  {
    var segment = this.i.next();

    return this.__outerThis._intervals[this.__outerThis.getIntervalID(segment)]; // ibm_ilog.graphlayout.hierarchical.CrossRedSelfLinkAlgorithmSegmentInterval
  }

});

(function(){ CrossRedSelfLinkAlgorithm._fields_init(); })();

return CrossRedSelfLinkAlgorithm;
});
