/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/makeacyclic/HMAEdge",	// #5
  "../hierarchical/makeacyclic/HMAGraph",	// #6
  "../hierarchical/makeacyclic/HMANode",	// #7
  "../hierarchical/makeacyclic/MakeAcyclicAlgorithm",	// #8
  "../hierarchical/CrossingReductionAlgorithm",	// #9
  "../hierarchical/HPercentages",	// #10
  "../hierarchical/HSwimLane",	// #11
  "../internalutil/GraphModelUtil",	// #12
  "../internalutil/TranslateUtil"	// #13
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HMAEdge,	// #5
    HMAGraph,	// #6
    HMANode,	// #7
    MakeAcyclicAlgorithm,	// #8
    CrossingReductionAlgorithm,	// #9
    HPercentages,	// #10
    HSwimLane,	// #11
    GraphModelUtil,	// #12
    TranslateUtil	// #13
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var CrossingReductionAlgorithm = ibm_ilog.graphlayout.hierarchical.CrossingReductionAlgorithm;
=====*/

var CrossRedSwimLaneAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.CrossRedSwimLaneAlgorithm", CrossingReductionAlgorithm, 
{
  // summary:
  //		CrossRedSwimLaneAlgorithm performs the additional crossing reduction
  //		when we have swimlanes. 
  // description:
  //		CrossRedSwimLaneAlgorithm performs the additional crossing reduction
  //		when we have swimlanes. 
  //		It is performed after the normal crossing reduction. It makes sure
  //		that the nodes of the swimlanes are in subsequent positions, and thet
  //		the swimlanes are not globally intertwined.  
  // tags:
  //		private
  
  _dummySwimLanes: null,

  constructor : function(graph)
  {
    this._checkBestPositions = false;
    this._numberOfSweeps = 1;
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var points = 3 * (graph.getNumberOfLevels() - 1) + 2;
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_CROSS_SWIMLANEREDUCTION], points, false);
    this.calcSwimLaneOrder();
    this.allocateDummySwimLanes();
    this.calcSwimLanesForFreeNodes();
    this.updateInfoInLevels();
    this._usePortBaryCenter = false;

    if (graph.getNumberOfLevels() == 1) {
      this.treatForwardLevel(null, graph.getFirstLevel());
    }
    else
    {
      this.sweepForward();
      this.sweepBackward();
      this.sweepForward();
    }

    this.calcCrossings(true);
    this.storeLevelPositionsInNodes();
  },

  calcSwimLaneOrder : function()
  {
    // summary:
    //		Calculate a global ordering of the swimlanes.
    // tags:
    //		private
    
    if (this.hasFullPositionNumbers()) {
      this.setOrderingNumberFromPositionNumbers();
    }
    else if (this.hasSwimLanesAtAllNodes())
    {
      this.calcOrderingNumbersFromCrossings();
      this.verifySpecifiedLanePositionIndex();
    }
    else
    {
      this.calcOrderingNumbersFromNeighboring();
      this.verifySpecifiedLanePositionIndex();
    }

    this.setFinalOrderingNumbers();
  },

  calcOrderingNumbersFromCrossings : function()
  {
    // summary:
    //		Calculate the undirected ordering graph, and derive the ordering
    //		numbers a sequential order with smallest penalty. 
    // description:
    //		Calculate the undirected ordering graph, and derive the ordering
    //		numbers a sequential order with smallest penalty. 
    //		The penalty is: if a link goes from swimlane A to swimlane B, and
    //		there are n swimlanes in between, then the penalty is "n". The
    //		penalty is for instance 0 if all links remain in the same swimlane
    //		or go to the direct neighbor swimlane. This mechanism is used when
    //		all nodes have swimlanes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var orderingGraph = new HMAGraph();
    var nodes = TranslateUtil.newArray(graph.getNumberOfSwimLanes(), null);
    var i = 0;

    var lane;
    var e = graph.getSwimLanes();

    while (e.hasNext())
    {
      lane = e.next();
      nodes[i] = new HMANode();
      lane.setOrderingNode(nodes[i]);
      orderingGraph.addNode(nodes[i]);
      i++;
    }

    var linkIter = graph.getLinks();
    var link;
    var node1;
    var node2;
    var edge;

    while (linkIter.hasNext())
    {
      link = linkIter.next();
      node1 = link.getFrom().getSwimLane().getOrderingNode();
      node2 = link.getTo().getSwimLane().getOrderingNode();

      if (node1 != node2)
      {
        edge = orderingGraph.getEdgeBetween(node1, node2, false);
        if (edge != null) {
          edge.setPriority(edge.getPriority() + 1);
        }
        else {
          orderingGraph.addEdgeBetweenNodes(new HMAEdge(1.0), node1, node2);
        }
      }
    }

    for (i = 0; i < nodes.length; i++)
      nodes[i].setOrderingNumber(i);

    var changed = true;
    var maxIter = nodes.length + 2;

    while (changed && maxIter > 0)
    {
      maxIter--;
      changed = false;

      for (i = 0; i < nodes.length - 1; i++)
      {
        if (this.needsSwapFromCrossings(nodes[i], nodes[i + 1]))
        {
          node1 = nodes[i];
          nodes[i] = nodes[i + 1];
          nodes[i + 1] = node1;
          nodes[i].setOrderingNumber(i);
          nodes[i + 1].setOrderingNumber(i + 1);
          changed = true;
        }
      }

      if (!changed)
      {
        for (i = 0; i < nodes.length - 2; i++)
        {
          switch (this.needsRotateFromCrossings(nodes[i], nodes[i + 1], nodes[i + 2]))
          {
            case 1:
              
              node1 = nodes[i];
              nodes[i] = nodes[i + 2];
              nodes[i + 2] = nodes[i + 1];
              nodes[i + 1] = node1;
              nodes[i].setOrderingNumber(i);
              nodes[i + 1].setOrderingNumber(i + 1);
              nodes[i + 2].setOrderingNumber(i + 2);
              changed = true;
              break;
            case -1:
              node1 = nodes[i];
              nodes[i] = nodes[i + 1];
              nodes[i + 1] = nodes[i + 2];
              nodes[i + 2] = node1;
              nodes[i].setOrderingNumber(i);
              nodes[i + 1].setOrderingNumber(i + 1);
              nodes[i + 2].setOrderingNumber(i + 2);
              changed = true;
              break;
          }
        }
      }
    }

    this.transferOrderingNumbersFromOrderingGraph();
  },

  needsSwapFromCrossings : function(node1, node2)
  {
    // summary:
    //		Returns true if the two input node should swap the positions.
    // tags:
    //		private
    
    var changeOfPenalty = 0;
    var index = node1.getOrderingNumber();
    var iter = node1.getHMAEdges();
    var edge;
    var otherNode;

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getHMAOpposite(node1);

      if (otherNode != node2)
      {
        if (otherNode.getOrderingNumber() < index) {
          changeOfPenalty += edge.getPriority();
        }
        else {
          changeOfPenalty -= edge.getPriority();
        }
      }
    }

    index = node2.getOrderingNumber();
    iter = node2.getHMAEdges();

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getHMAOpposite(node2);

      if (otherNode != node1)
      {
        if (otherNode.getOrderingNumber() < index) {
          changeOfPenalty -= edge.getPriority();
        }
        else {
          changeOfPenalty += edge.getPriority();
        }
      }
    }

    return (changeOfPenalty < 0); // Boolean
  },

  needsRotateFromCrossings : function(node1, node2, node3)
  {
    // summary:
    //		Returns 1 if the three input nodes should be rotated clockwise, -1
    //		of they should be rotated anticlockwise and 0 if they should not
    //		change.
    // tags:
    //		private
    
    var changeOfPenaltyClockwise = 0;
    var changeOfPenaltyAntiClockwise = 0;
    var index = node1.getOrderingNumber();
    var iter = node1.getHMAEdges();
    var edge;
    var otherNode;

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getHMAOpposite(node1);

      if (otherNode == node2)
      {
        changeOfPenaltyAntiClockwise += edge.getPriority();
      }
      else if (otherNode == node3)
      {
        changeOfPenaltyClockwise -= edge.getPriority();
        changeOfPenaltyAntiClockwise -= edge.getPriority();
      }
      else
      {
        if (otherNode.getOrderingNumber() < index)
        {
          changeOfPenaltyClockwise += edge.getPriority();
          changeOfPenaltyAntiClockwise += 2 * edge.getPriority();
        }
        else
        {
          changeOfPenaltyClockwise -= edge.getPriority();
          changeOfPenaltyAntiClockwise -= 2 * edge.getPriority();
        }
      }
    }

    index = node2.getOrderingNumber();
    iter = node2.getHMAEdges();

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getHMAOpposite(node2);

      if (otherNode == node3)
      {
        changeOfPenaltyClockwise += edge.getPriority();
      }
      else if (otherNode != node1)
      {
        if (otherNode.getOrderingNumber() < index)
        {
          changeOfPenaltyClockwise += edge.getPriority();
          changeOfPenaltyAntiClockwise -= edge.getPriority();
        }
        else
        {
          changeOfPenaltyClockwise -= edge.getPriority();
          changeOfPenaltyAntiClockwise += edge.getPriority();
        }
      }
    }

    index = node3.getOrderingNumber();
    iter = node3.getHMAEdges();

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getHMAOpposite(node3);

      if (otherNode != node1 && otherNode != node2)
      {
        if (otherNode.getOrderingNumber() < index)
        {
          changeOfPenaltyClockwise -= 2 * edge.getPriority();
          changeOfPenaltyAntiClockwise -= edge.getPriority();
        }
        else
        {
          changeOfPenaltyClockwise += 2 * edge.getPriority();
          changeOfPenaltyAntiClockwise += edge.getPriority();
        }
      }
    }

    if (changeOfPenaltyClockwise < changeOfPenaltyAntiClockwise)
    {
      if (changeOfPenaltyClockwise < 0) {
        return 1; // int
      }
    }
    else if (changeOfPenaltyAntiClockwise < 0) {
      return -1; // int
    }
    return 0; // int
  },

  calcOrderingNumbersFromNeighboring : function()
  {
    // summary:
    //		Calculate the ordering graph, and derive the ordering numbers from a
    //		topological sorting of the ordering graph. 
    // description:
    //		Calculate the ordering graph, and derive the ordering numbers from a
    //		topological sorting of the ordering graph. 
    //		This works as follows: We allocate a HMAGraph with one node per
    //		swimlane. If there is a node of swimlane A directly to the left of a
    //		node of swimlane B, we add a link from A to B in the HMAGraph,
    //		because it indicates that A better be left of B. Finally we make the
    //		HMAGraph acyclic and get the ordering numbers from the topological
    //		sorting. This mechanism is used when not all nodes have swimlanes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var orderingGraph = new HMAGraph();
    var lane;
    var e = graph.getSwimLanes();

    while (e.hasNext())
    {
      lane = e.next();
      lane.setOrderingNode(new HMANode());
      orderingGraph.addNode(lane.getOrderingNode());
    }

    var lastLane;
    var levelIter = graph.getLevels();
    var level;
    var nodeIter;

    while (levelIter.hasNext())
    {
      level = levelIter.next();
      nodeIter = level.getNodes();
      lastLane = null;

      while (nodeIter.hasNext())
      {
        lane = nodeIter.next().getSwimLane();

        if (lane != null)
        {
          if (lane != lastLane && lastLane != null)
          {
            orderingGraph.addEdgeBetweenNodes(new HMAEdge(1.0), lastLane.getOrderingNode(), lane.getOrderingNode());
          }

          lastLane = lane;
        }
      }
    }

    var orderAlg = new MakeAcyclicAlgorithm(orderingGraph, this.getGraph().getLayout(), null);
    orderAlg.setTopologicalOrdering(true);
    orderAlg.run();

    this.transferOrderingNumbersFromOrderingGraph();
  },

  transferOrderingNumbersFromOrderingGraph : function()
  {
    // summary:
    //		Transfer the result and clean up.
    // tags:
    //		private
    
    var e = this.getGraph().getSwimLanes();
    var lane;

    while (e.hasNext())
    {
      lane = e.next();
      lane.setOrderingNumber(lane.getOrderingNode().getOrderingNumber());
      lane.setOrderingNode(null);
    }
  },

  setOrderingNumberFromPositionNumbers : function()
  {
    // summary:
    //		Sets the ordering number of all swimlanes from the position indexes.
    //		
    // description:
    //		Sets the ordering number of all swimlanes from the position indexes.
    //		
    //		This assumes all swimlanes have position numbers.
    // tags:
    //		private
    
    var lane;
    var e = this.getGraph().getSwimLanes();

    while (e.hasNext())
    {
      lane = e.next();
      lane.setOrderingNumber(lane.getSpecPositionIndex());
    }
  },

  verifySpecifiedLanePositionIndex : function()
  {
    // summary:
    //		Verifies that the swimlanes has ordering numbers that are consistent
    //		to the specified position indexes. 
    // description:
    //		Verifies that the swimlanes has ordering numbers that are consistent
    //		to the specified position indexes. 
    //		This is only used if some swimlanes have position numbers and others
    //		don't, because it is slower.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var e1 = graph.getSwimLanes();
    var e2;
    var lane;
    var otherLane;
    var actIndex;
    var desiredIndex;

    while (e1.hasNext())
    {
      lane = e1.next();
      desiredIndex = lane.getSpecPositionIndex();
      actIndex = lane.getOrderingNumber();

      if (actIndex != desiredIndex && desiredIndex >= 0)
      {
        e2 = graph.getSwimLanes();

        while (e2.hasNext())
        {
          otherLane = e2.next();

          if (otherLane.getOrderingNumber() == desiredIndex)
          {
            otherLane.setOrderingNumber(actIndex);
            break;
          }
        }

        lane.setOrderingNumber(desiredIndex);
      }
    }
  },

  setFinalOrderingNumbers : function()
  {
    // summary:
    //		Final assignment of even ordering numbers. 
    // description:
    //		Final assignment of even ordering numbers. 
    //		This makes the ordering numbers even and subsequent.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var lanes = graph.getSortedSwimLanes();

    var n = graph.getNumberOfSwimLanes();

    for (var i = 0; i < n; i++)
    {
      lanes[i].setOrderingNumber(2 + 2 * i);
      lanes[i].setCalcPositionIndex(i);
    }
  },

  hasFullPositionNumbers : function()
  {
    // summary:
    //		Check whether all swimlanes have specified position indexes.
    // tags:
    //		private
    
    var lane;
    var e = this.getGraph().getSwimLanes();

    while (e.hasNext())
    {
      lane = e.next();
      if (lane.getSpecPositionIndex() < 0) {
        return false; // Boolean
      }
    }

    return true; // Boolean
  },

  hasSwimLanesAtAllNodes : function()
  {
    // summary:
    //		Check whether all nodes in the graph have swimlanes.
    // tags:
    //		private
    
    var levelIter = this.getGraph().getLevels();
    var nodeIter;

    while (levelIter.hasNext())
    {
      nodeIter = levelIter.next().getNodes();

      while (nodeIter.hasNext())
      {
        if (nodeIter.next().getSwimLane() == null) {
          return false; // Boolean
        }
      }
    }

    return true; // Boolean
  },

  calcSwimLanesForFreeNodes : function()
  {
    // summary:
    //		Calculate the swimlane for nodes that
    //		don't belong to a swimlane yet. 
    // description:
    //		Calculate the swimlane for nodes that don't belong to a swimlane
    //		yet. 
    //		These dummy swimlanes get odd ordering numbers so that they are
    //		between the specified swimlanes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getNodes();
    var node;
    var marker;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getSwimLane() != null) {
        node.setMarker(node.getSwimLane().getOrderingNumber());
      }
      else {
        node.setMarker(-1);
      }
    }

    iter = graph.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getMarker() == -1) {
        this.propagateMarker(node, this.getPropagationSwimLaneMarker(node));
      }
    }

    this.optimizeSwimLanesForFreeNodes();

    iter = graph.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.getSwimLane() == null)
      {
        if (node.getMarker() == -1) {
          node.setMarker(1);
        }
        else if (node.isDummyNode())
        {
          marker = node.getOwnerLink().getTo().getMarker();

          if (marker % 2 == 1) {
            node.setMarker(marker);
          }
          else
          {
            marker = node.getOwnerLink().getFrom().getMarker();
            if (marker % 2 == 1) {
              node.setMarker(marker);
            }
          }
        }
      }
    }

    iter = graph.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getSwimLane() == null) {
        node.setSwimLane(this.getDummySwimLane(node.getMarker()));
      }
    }
  },

  optimizeSwimLanesForFreeNodes : function()
  {
    // summary:
    //		Optimize the swimlanes for free nodes. 
    // description:
    //		Optimize the swimlanes for free nodes. 
    //		At this time point, the swimlane ordering number is stores as marker
    //		in each node. The nodes with odd markers have no specified
    //		swimlanes. For these nodes, we iteratively change the marker and
    //		hope that the process converges.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var changed = true;
    var maxIter = 10;

    while (changed && maxIter > 0)
    {
      changed = false;
      maxIter--;
      var iter = graph.getNodes();
      var node;
      var oldSwimLane;
      var newSwimLane;

      while (iter.hasNext())
      {
        node = iter.next();

        if (node.getSwimLane() == null)
        {
          oldSwimLane = node.getMarker();
          newSwimLane = this.getSwimLaneMarker(node);

          if (oldSwimLane != newSwimLane)
          {
            node.setMarker(newSwimLane);
            changed = true;
          }
        }
      }
    }
  },

  getPropagationSwimLaneMarker : function(node)
  {
    // summary:
    //		Returns the swimlane marker of a connected component of nodes that
    //		don't have a maker yet and start at the input node.
    // tags:
    //		private
    
    this._numNodesForPropagation = 0;
    this._sumMarkersForPropagation = 0.0;
    this.calcPropagationSwimLaneMarker(node);
    if (this._numNodesForPropagation == 0) {
      return -1; // int
    }
    var result = Math.floor(((this._sumMarkersForPropagation + 0.0001) / this._numNodesForPropagation));

    if (result % 2 == 0) {
      result++;
    }
    return result; // int
  },

  _numNodesForPropagation: null,
  _sumMarkersForPropagation: null,

  calcPropagationSwimLaneMarker : function(node)
  {
    // summary:
    //		This does the real work.
    // tags:
    //		private
    
    if (node.getMarker() != -1) {
      return;
    }
    node.setMarker(0);
    var iter = node.getSegments();
    var othernode;

    while (iter.hasNext())
    {
      othernode = iter.next().getOpposite(node);

      if (othernode.getMarker() == -1) {
        this.calcPropagationSwimLaneMarker(othernode);
      }
      else if (othernode.getMarker() > 0)
      {
        this._sumMarkersForPropagation += othernode.getMarker();
        this._numNodesForPropagation++;
      }
    }
  },

  propagateMarker : function(node, marker)
  {
    // summary:
    //		Set the marker to the input node and to all nodes that are connected
    //		to the input node, that don't have a marker yet.
    // tags:
    //		private
    
    if (marker <= 0) {
      return;
    }
    if (node.getMarker() > 0) {
      return;
    }
    node.setMarker(marker);
    var iter = node.getSegments();

    while (iter.hasNext())
      this.propagateMarker(iter.next().getOpposite(node), marker);
  },

  getSwimLaneMarker : function(node)
  {
    // summary:
    //		Calculate the marker for a node from the markers of its neighbors.
    // tags:
    //		private
    
    var marker = 0;
    var numNodes = 0;
    var iter = node.getSegments();
    var othernode;

    while (iter.hasNext())
    {
      othernode = iter.next().getOpposite(node);

      if (othernode.getMarker() != -1)
      {
        marker += othernode.getMarker();
        numNodes++;
      }
    }

    if (numNodes == 0) {
      return -1; // int
    }
    var result = Math.floor(((marker + 0.0001) / numNodes));

    if (result % 2 == 0) {
      result++;
    }
    return result; // int
  },

  allocateDummySwimLanes : function()
  {
    // summary:
    //		Allocate the dummy swimlanes for the free nodes that
    //		don't have any swimlane yet.
    // tags:
    //		private
    
    var n = this.getGraph().getNumberOfSwimLanes();
    this._dummySwimLanes = TranslateUtil.newArray(n + 1, null);
  },

  getDummySwimLane : function(i)
  {
    // summary:
    //		Returns the dummy swimlane for a node with ordering number i. 
    // description:
    //		Returns the dummy swimlane for a node with ordering number i. 
    //		This dummy swimlane represents the free area between the swimlane
    //		with ordering i-1 and i+1. The number i must be odd, because the
    //		swim lane numbers are even.
    // tags:
    //		private
    
    if (i < 2) {
      i = 0;
    }
    if (Math.floor((i - 1) / 2) >= this._dummySwimLanes.length) {
      i = this._dummySwimLanes.length - 1;
    }
    else {
      i = Math.floor((i - 1) / 2);
    }
    if (this._dummySwimLanes[i] == null)
    {
      this._dummySwimLanes[i] = this.getGraph().addDummySwimLane();
      this._dummySwimLanes[i].setOrderingNumber(2 * i + 1);
    }

    return this._dummySwimLanes[i]; // ibm_ilog.graphlayout.hierarchical.HSwimLane
  },

  initForwardFirstLevel : function(firstLevel)
  {
    // summary:
    //		Initializes the first level (with level number n) in a backward
    //		sweep.
    // tags:
    //		private
    
    this.verifySwimLanes(firstLevel);
    firstLevel.storeLevelPositionsInNodes(1, false);
  },

  initBackwardFirstLevel : function(firstLevel)
  {
    // summary:
    //		Initializes the first level (with level number n) in a backward
    //		sweep.
    // tags:
    //		private
    
    this.verifySwimLanes(firstLevel);
    firstLevel.storeLevelPositionsInNodes(1, false);
  },

  treatForwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number 1 ... 
    // description:
    //		Treats a level (with level number 1 ... 
    //		n) in a forward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i-1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.inherited("treatForwardLevel", arguments, [prevLevel, level]);
    this.verifySwimLanes(level);
    level.storeLevelPositionsInNodes(1, false);
  },

  treatBackwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number n-1 ... 
    // description:
    //		Treats a level (with level number n-1 ... 
    //		0) in a backward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i+1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.inherited("treatBackwardLevel", arguments, [prevLevel, level]);
    this.verifySwimLanes(level);
    level.storeLevelPositionsInNodes(1, false);
  },

  verifySwimLanes : function(level)
  {
    // summary:
    //		Verifies that the node ordering is consistent with the swimlane
    //		ordering.
    // tags:
    //		private
    
    if (level.getNodesField() == null) {
      return;
    }
    this.calcAveragePositionOfSwimLanes(level);
    this.calcSwimLaneSortValue(level);
    this._nodeSortAlg.sort(level.getNodesField());

    var e = this.getGraph().getSwimLanes();

    while (e.hasNext())
      (e.next()).moveToNextLevelDuringCrossingReduction();
  },

  calcAveragePositionOfSwimLanes : function(level)
  {
    // summary:
    //		Calculaye the average position of the swimlanes with respect to the
    //		input level.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var e = graph.getSwimLanes();
    var lane;

    while (e.hasNext())
    {
      lane = e.next();
      lane.setAvgPositionActLevel(0);
      lane.setMarker(0);
    }

    var iter = level.getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      lane = node.getSwimLane();

      if (lane != null)
      {
        lane.addMarker(1);
        lane.addAvgPositionActLevel(node.getPositionInLevel());
      }
    }

    e = graph.getSwimLanes();

    while (e.hasNext())
    {
      lane = e.next();
      if (lane.getMarker() > 0) {
        lane.setAvgPositionActLevel(lane.getAvgPositionActLevel() / lane.getMarker());
      }
      else {
        lane.setAvgPositionActLevel(-1);
      }
    }
  },

  calcSwimLaneSortValue : function(level)
  {
    // summary:
    //		Calculate the swimlane sorting value. 
    // description:
    //		Calculate the swimlane sorting value. 
    //		This also sets the swimlane ordering value into the marker of the
    //		node. This is needed when treating the next level.
    // tags:
    //		private
    
    var factor = level.getNodesCount() + 5;
    var iter = level.getNodes();
    var node;
    var lane;
    var val;

    while (iter.hasNext())
    {
      node = iter.next();
      lane = node.getSwimLane();

      if (lane != null) {
        val = lane.getOrderingNumber();
      }
      else
      {
        val = this.getFreeNodeOrderingValue(node);
        node.setSwimLane(this.getDummySwimLane(val));
      }

      node.setSortValue(factor * val + node.getPositionInLevel());
    }
  },

  getFreeNodeOrderingValue : function(node)
  {
    // summary:
    //		Returns the dummy ordering number of a node that does not belong to
    //		any swimlane. 
    // description:
    //		Returns the dummy ordering number of a node that does not belong to
    //		any swimlane. 
    //		Since all swimlanes have even ordering numbers, this returns an odd
    //		ordering number. The goal is to find a good number so that the links
    //		remain short.
    // tags:
    //		private
    
    var laneBetween = 0;
    var sum = 0;
    var numNodes = 0;
    var testLane = null;
    var position = node.getPositionInLevel();

    var iter = node.getSegments();
    var othernode;

    while (iter.hasNext())
    {
      othernode = iter.next().getOpposite(node);

      if (othernode.getSwimLane() != null)
      {
        sum += othernode.getSwimLane().getOrderingNumber();
        numNodes++;
        testLane = othernode.getSwimLane();
      }
    }

    if (numNodes > 0)
    {
      laneBetween = Math.floor(((sum + 0.000001) / numNodes));

      if (laneBetween % 2 == 0 && laneBetween * numNodes == sum && testLane != null && testLane.getOrderingNumber() == laneBetween)
      {
        if (testLane.getAvgPositionActLevel() != -1)
        {
          if (testLane.getAvgPositionActLevel() < position) {
            return testLane.getOrderingNumber() + 1; // int
          }
          else {
            return testLane.getOrderingNumber() - 1; // int
          }
        }
        else if (node.getSegmentsToCount() > 0)
        {
          if (testLane.getAvgPositionPrevLevel() < node.getSortValue()) {
            return testLane.getOrderingNumber() + 1; // int
          }
          else {
            return testLane.getOrderingNumber() - 1; // int
          }
        }
      }
      else
      {
        if (laneBetween % 2 == 0) {
          return laneBetween + 1; // int
        }
        else {
          return laneBetween; // int
        }
      }
    }
    else
    {
      laneBetween = 1;
      var bestAvgPosition = -1;
      var e = this.getGraph().getSwimLanes();

      while (e.hasNext())
      {
        testLane = e.next();

        if (testLane.getAvgPositionActLevel() < position && testLane.getAvgPositionActLevel() > bestAvgPosition)
        {
          laneBetween = testLane.getOrderingNumber() + 1;
          bestAvgPosition = testLane.getAvgPositionActLevel();
        }
      }
    }

    return laneBetween; // int
  }

}
);

return CrossRedSwimLaneAlgorithm;
});
