/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/relpositioning/HRPSolving",	// #5
  "../hierarchical/CalcCrossings",	// #6
  "../hierarchical/HLevelSweepAlgorithm",	// #7
  "../hierarchical/HNodeSort",	// #8
  "../hierarchical/HPercentages",	// #9
  "../internalutil/GraphModelUtil",	// #10
  "../internalutil/TranslateUtil"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HRPSolving,	// #5
    CalcCrossings,	// #6
    HLevelSweepAlgorithm,	// #7
    HNodeSort,	// #8
    HPercentages,	// #9
    GraphModelUtil,	// #10
    TranslateUtil	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelSweepAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelSweepAlgorithm;
=====*/

var CrossingReductionAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.CrossingReductionAlgorithm", HLevelSweepAlgorithm, 
{
  // summary:
  //		CrossingReductionAlgorithm traverses the levels and performs crossing
  //		reduction at each level.  
  // tags:
  //		private
  
  _nodeSortAlg: null,
  _solveAlg: null,
  _usePortBaryCenter : null,
  _useMedianCrossingValue: null,
  calcCrossAlg : null,
  _numberOfSweeps: null,
  _checkBestPositions: null,
  _bestNumberOfCrossings: null,
  _traversalNumber: null,

  constructor : function(graph)
  {
    this._usePortBaryCenter = false;
    this.calcCrossAlg = new CalcCrossings();
    this.init(graph);
    var layout = graph.getLayout();
    this._nodeSortAlg = new HNodeSort();
    this._solveAlg = new HRPSolving(null, layout, graph.getPercController());
    this._numberOfSweeps = layout.getNumberOfLinkCrossingSweeps();
    this._checkBestPositions = layout.isBacktrackCrossingReductionEnabled();
    this._useMedianCrossingValue = layout.isMedianCrossingValueEnabled();
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._nodeSortAlg = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var points = (this._numberOfSweeps + 3) * (graph.getNumberOfLevels() - 1) + 2;
    if (graph.hasPorts() || graph.hasFixedLinkSides()) {
      points += 2 * (graph.getNumberOfLevels() - 1);
    }
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_CROSS_REDUCTION], points, false);
    this.unwindDisconnectedParts();
    this.updateInfoInLevels();
    this._bestNumberOfCrossings = /*Number.MAX_VALUE*/2147483647;

    var atLeastOneSweepDone = false;

    if (graph.hasPorts() || graph.hasFixedLinkSides())
    {
      this._usePortBaryCenter = true;
      this.sweepForward();
      atLeastOneSweepDone = true;
      if (!this.isOptimal()) {
        this.sweepBackward();
      }
    }

    this._usePortBaryCenter = false;

    if (graph.getNumberOfLevels() == 1) {
      this.treatForwardLevel(null, graph.getFirstLevel());
    }
    else
    {
      for (var i = 0; i < this._numberOfSweeps; i++)
      {
        if (atLeastOneSweepDone && this.isOptimal()) {
          break;
        }
        if (i % 2 == 0) {
          this.sweepForward();
        }
        else {
          this.sweepBackward();
        }
        atLeastOneSweepDone = true;
      }
    }

    this.restoreBestPositions();
    this.calcCrossings(true);
    this.storeLevelPositionsInNodes();
  },

  unwindDisconnectedParts : function()
  {
    // summary:
    //		This makes already sure that disconnected parts are not intermixed. 
    // description:
    //		This makes already sure that disconnected parts are not intermixed. 
    //		It traverses the connected parts of the graph and assigns sorting
    //		numbers so that when sorting the levels with these numbers,
    //		disconnected parts are not intermixed.
    // tags:
    //		private
    
    this._traversalNumber = 1;
    var graph = this.getGraph();
    var niter = graph.getNodes();
    var node;

    while (niter.hasNext())
    {
      niter.next().setSortValue(-1);
    }

    niter = graph.getNodes();

    while (niter.hasNext())
    {
      node = niter.next();
      if (node.getSortValue() < 0) {
        this.traverseConnectedComponent(node);
      }
    }

    var liter = this.getGraph().getLevels();
    var level;

    while (liter.hasNext())
    {
      level = liter.next();
      if (level.getNodesField() != null) {
        this._nodeSortAlg.sort(level.getNodesField());
      }
      this.layoutStepPerformed();
    }
  },

  traverseConnectedComponent : function(node)
  {
    // summary:
    //		Traverse the component connected to the input node and assign
    //		continuous sorting numbers to these components.
    // tags:
    //		private
    
    var stack = new Array();
    node.setSortValue(this._traversalNumber++);
    stack.push(node);

    while (!(stack.length==0))
    {
      node = stack.pop();
      var iter = node.getSegmentsFrom();
      var segment;
      var child;

      while (iter.hasNext())
      {
        segment = iter.next();
        child = segment.getTo();

        if (child.getSortValue() < 0)
        {
          child.setSortValue(this._traversalNumber++);
          stack.push(child);
        }
      }

      iter = node.getSegmentsTo();

      while (iter.hasNext())
      {
        segment = iter.next();
        child = segment.getFrom();

        if (child.getSortValue() < 0)
        {
          child.setSortValue(this._traversalNumber++);
          stack.push(child);
        }
      }
    }
  },

  sweepForward : function()
  {
    // summary:
    //		Performs a sweep forwards over the levels.
    // tags:
    //		private
    
    this.inherited("sweepForward", arguments, []);
    this.checkForBestPositions();
  },

  sweepBackward : function()
  {
    // summary:
    //		Performs a sweep forwards over the levels.
    // tags:
    //		private
    
    this.inherited("sweepBackward", arguments, []);
    this.checkForBestPositions();
  },

  verifySpecifiedPositionIndex : function(level)
  {
    // summary:
    //		Verified for the nodes with specified position index that  they are
    //		at specified position.
    // tags:
    //		private
    
    var maxIndex = level.getNodesCount();

    if (maxIndex > 0 && level.hasNodesWithSpecPos())
    {
      var iter = level.getNodes();
      var node;
      var otherNode;
      var actIndex = 0;
      var desiredIndex;
      var i;
      var maxIterations = level.getNodesCount() + 1;

      while (iter.hasNext())
      {
        node = iter.next();

        desiredIndex = node.getSpecPositionInLevel();
        i = maxIterations;

        while (desiredIndex != actIndex && desiredIndex >= 0 && desiredIndex < maxIndex && i > 0)
        {
          i--;
          otherNode = level.getNodesField()[desiredIndex];
          level.getNodesField()[desiredIndex] = node;
          level.getNodesField()[actIndex] = otherNode;
          node = otherNode;
          desiredIndex = node.getSpecPositionInLevel();
        }

        actIndex++;
      }
    }
  },

  calcPredBaryCenter : function(level)
  {
    // summary:
    //		Calculates the predecessor bary value of the nodes in the level. 
    // description:
    //		Calculates the predecessor bary value of the nodes in the level. 
    //		It presumes that the position number of the nodes in the previous
    //		level are set.
    // tags:
    //		private
    
    var numNodes = level.getNodesCount();
    var iter = level.getNodes();
    var node;

    if (this._usePortBaryCenter)
    {
      while (iter.hasNext())
        iter.next().calcPredBaryCenterForPorts(false);
    }
    else
    {
      while (iter.hasNext())
        iter.next().calcPredBaryCenter(false, numNodes);
    }

    if (this._useMedianCrossingValue)
    {
      var factor = ((numNodes > 0 ? 0.00001 / numNodes : 0.00001));
      iter = level.getNodes();

      while (iter.hasNext())
        iter.next().addPredMedian(factor);
    }

    if (level != this.getGraph().getFirstLevel())
    {
      iter = level.getNodes();
      var oldSortValue = 0.0;

      while (iter.hasNext())
      {
        node = iter.next();

        if (node.getSegmentsToCount() == 0)
        {
          node.setSortValue(oldSortValue);
        }
        else {
          oldSortValue = node.getSortValue();
        }
      }

      iter = level.getNodesBackward();

      while (iter.hasNext())
      {
        node = iter.next();
        if (node.getSegmentsToCount() == 0) {
          node.setSortValue((oldSortValue + node.getSortValue()) / 2.0);
        }
        else {
          oldSortValue = node.getSortValue();
        }
      }
    }
  },

  calcSuccBaryCenter : function(level)
  {
    // summary:
    //		Calculates the successor bary value of the nodes in the level. 
    // description:
    //		Calculates the successor bary value of the nodes in the level. 
    //		It presumes that the position number of the nodes in the next level
    //		are set.
    // tags:
    //		private
    
    var numNodes = level.getNodesCount();
    var iter = level.getNodes();
    var node;

    if (this._usePortBaryCenter)
    {
      while (iter.hasNext())
        iter.next().calcSuccBaryCenterForPorts(false);
    }
    else
    {
      while (iter.hasNext())
        iter.next().calcSuccBaryCenter(false, numNodes);
    }

    if (this._useMedianCrossingValue)
    {
      var factor = ((numNodes > 0 ? 0.00001 / numNodes : 0.00001));
      iter = level.getNodes();

      while (iter.hasNext())
        iter.next().addSuccMedian(factor);
    }

    if (level != this.getGraph().getLastLevel())
    {
      iter = level.getNodes();
      var oldSortValue = 0.0;

      while (iter.hasNext())
      {
        node = iter.next();

        if (node.getSegmentsFromCount() == 0)
        {
          node.setSortValue(oldSortValue);
        }
        else {
          oldSortValue = node.getSortValue();
        }
      }

      iter = level.getNodesBackward();

      while (iter.hasNext())
      {
        node = iter.next();
        if (node.getSegmentsFromCount() == 0) {
          node.setSortValue((oldSortValue + node.getSortValue()) / 2.0);
        }
        else {
          oldSortValue = node.getSortValue();
        }
      }
    }
  },

  processConstraints : function(level)
  {
    // summary:
    //		Process the relative position constraints and side by side
    //		constraints for this level.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var constraintNetwork = level.getConstraintNetwork();
    if (constraintNetwork == null) {
      return;
    }
    this._solveAlg.initGraph(constraintNetwork);

    var iter = level.getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      graph.getConstraintNode(node).setBarycenter(node.getSortValue(), 1.0);
    }

    this._solveAlg.run();

    iter = level.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();
      node.setSortValue(graph.getConstraintNode(node).getPositionNumber());
    }
  },

  initForwardFirstLevel : function(firstLevel)
  {
    // summary:
    //		Initializes the first level (with level number 0) in a forward
    //		sweep.
    // tags:
    //		private
    
    if (firstLevel.getConstraintNetwork() != null)
    {
      this.storePositionAsSortValue(firstLevel);
      this.processConstraints(firstLevel);
      if (firstLevel.getNodesField() != null) {
        this._nodeSortAlg.sort(firstLevel.getNodesField());
      }
    }

    this.verifySpecifiedPositionIndex(firstLevel);
    firstLevel.storeLevelPositionsInNodes(1, false);
  },

  initBackwardFirstLevel : function(firstLevel)
  {
    // summary:
    //		Initializes the first level (with level number n) in a backward
    //		sweep.
    // tags:
    //		private
    
    if (firstLevel.getConstraintNetwork() != null)
    {
      this.storePositionAsSortValue(firstLevel);
      this.processConstraints(firstLevel);
      if (firstLevel.getNodesField() != null) {
        this._nodeSortAlg.sort(firstLevel.getNodesField());
      }
    }

    this.verifySpecifiedPositionIndex(firstLevel);
    firstLevel.storeLevelPositionsInNodes(1, false);
  },

  storePositionAsSortValue : function(level)
  {
    // summary:
    //		Sets the sort value to the position within the level. 
    // description:
    //		Sets the sort value to the position within the level. 
    //		For the first level in a downward sweep, and the last level in an
    //		upward sweep, we sort them according to the current positions,
    //		because these nodes have no pred/succ bary center. Normally this
    //		would be unnecessary because it
    //		doesn't change anything. But if we have constraints, the sorting is necessary, because the constraints may change the positions, and the constraint solver requires that the sort value is set to something for the unconstraint nodes.
    // tags:
    //		private
    
    var iter = level.getNodes();
    var i = 1;

    while (iter.hasNext())
      iter.next().setSortValue(i++);
  },

  treatForwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number 1 ... 
    // description:
    //		Treats a level (with level number 1 ... 
    //		n) in a forward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i-1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.calcPredBaryCenter(level);
    this.processConstraints(level);
    if (level.getNodesField() != null) {
      this._nodeSortAlg.sort(level.getNodesField());
    }
    this.verifySpecifiedPositionIndex(level);
    level.storeLevelPositionsInNodes(1, false);
  },

  treatBackwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number n-1 ... 
    // description:
    //		Treats a level (with level number n-1 ... 
    //		0) in a backward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i+1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.calcSuccBaryCenter(level);
    this.processConstraints(level);
    if (level.getNodesField() != null) {
      this._nodeSortAlg.sort(level.getNodesField());
    }
    this.verifySpecifiedPositionIndex(level);
    level.storeLevelPositionsInNodes(1, false);
  },

  checkForBestPositions : function()
  {
    // summary:
    //		Check whether the new position is better than the previous best
    //		position.
    // tags:
    //		private
    
    if (!this._checkBestPositions) {
      return;
    }
    var nrCrossings = this.calcCrossings(false);

    if (nrCrossings < this._bestNumberOfCrossings)
    {
      this.saveBestPositions();
      this._bestNumberOfCrossings = nrCrossings;
    }
  },

  isOptimal : function()
  {
    // summary:
    //		Returns true when the optimum is reached.
    // tags:
    //		private
    
    return this._checkBestPositions && (this._bestNumberOfCrossings == 0); // Boolean
  },

  saveBestPositions : function()
  {
    // summary:
    //		Save the current position of the nodes as best position.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().saveBestPosition();
  },

  restoreBestPositions : function()
  {
    // summary:
    //		Restore the best position of the nodes.
    // tags:
    //		private
    
    if (!this._checkBestPositions) {
      return;
    }
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().restoreBestPosition();
    var leveliter = this.getGraph().getLevels();
    var level;

    while (leveliter.hasNext())
    {
      level = leveliter.next();
      if (level.getNodesField() != null) {
        this._nodeSortAlg.sort(level.getNodesField());
      }
      level.storeLevelPositionsInNodes(1, false);
    }
  },

  storeLevelPositionsInNodes : function()
  {
    // summary:
    //		Store the final position numbers of the nodes in the levels.
    // tags:
    //		private
    
    var iter = this.getGraph().getLevels();

    while (iter.hasNext())
    {
      iter.next().storeLevelPositionsInNodes(0, false);
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  updateInfoInLevels : function()
  {
    // summary:
    //		Updates the info stored in the levels.
    // tags:
    //		private
    
    var iter = this.getGraph().getLevels();

    while (iter.hasNext())
    {
      iter.next().updateInfo();
      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  },

  calcCrossings : function(storeInLevels)
  {
    // summary:
    //		Calculate the number of crossings in the graph.
    // tags:
    //		private
    
    var iter = this.getGraph().getLevels();
    var prevlevel = null;
    var level;
    var numCrossings = 0;
    var h;

    while (iter.hasNext())
    {
      level = iter.next();

      if (prevlevel != null)
      {
        h = this.calcCrossAlg.getNumberOfCrossings(prevlevel, level);
        numCrossings += h;
        if (storeInLevels) {
          prevlevel.setNumberOfCrossings(h);
        }
      }

      prevlevel = level;
    }

    return numCrossings; // int
  }

}
);

return CrossingReductionAlgorithm;
});
