/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HLevelAlgorithm",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLevelAlgorithm,	// #5
    GraphModelUtil,	// #6
    TranslateUtil	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm;
=====*/

var CrossingReductionSwapAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.CrossingReductionSwapAlgorithm", HLevelAlgorithm, 
{
  // summary:
  //		This level algorithm is a fine tuning phase for the crossing
  //		reduction. 
  // description:
  //		This level algorithm is a fine tuning phase for the crossing
  //		reduction. 
  //		It checks what happens when neighbored nodes are swapped. The
  //		algorithm works only on a proper hierarchy and requires that the node
  //		position numbers are up to date.  
  // tags:
  //		private
  
  _withIntergraphSegments: null,

  constructor : function(withIntergraphSegments )
  {
    this._withIntergraphSegments = withIntergraphSegments || false ;
  },
  
  init : function(upperLevel, lowerLevel)
  {
    // summary:
    //		Initializes the algorithm with the actual levels.
    // tags:
    //		private
    
    this.inherited("init", arguments, [upperLevel, lowerLevel]);
  },

  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  run : function()
  {
    // summary:
    //		Start the algorithm.
    // tags:
    //		private
    
    var upperLevel = this.getUpperLevel();
    var lowerLevel = this.getLowerLevel();

    if (lowerLevel.getNumberOfCrossings() > 0 || (upperLevel != null && upperLevel.getNumberOfCrossings() > 0)) {
      this.checkAndSwap(lowerLevel);
    }
  },

  checkAndSwap : function(level)
  {
    // summary:
    //		This method does the real work. 
    // description:
    //		This method does the real work. 
    //		It is only called if there are really some crossings.
    // tags:
    //		private
    
    if (level.getConstraintNetwork() != null)
    {
      return;
    }

    var n = level.getNodesCount();
    var node1;
    var node2;
    var changed = true;

    while (changed)
    {
      changed = false;

      for (var i = 0; i < n - 1; i++)
      {
        node1 = level.getNode(i);
        node2 = level.getNode(i + 1);

        if (this.needsSwap(node1, node2))
        {
          changed = true;
          CrossingReductionSwapAlgorithm.swap(node1, node2);
        }
      }
    }
  },

  needsSwap : function(node1, node2)
  {
    // summary:
    //		Returns true if swapping both neighbored input nodes reduces the
    //		number of crossings.
    // tags:
    //		private
    
    if (node1.getSpecPositionInLevel() >= 0) {
      return false; // Boolean
    }
    if (node2.getSpecPositionInLevel() >= 0) {
      return false; // Boolean
    }
    if (node1.getSwimLane() != node2.getSwimLane()) {
      return false; // Boolean
    }
    var numCrossingsBeforeSwap = 0;
    var numCrossingsAfterSwap = 0;
    var iter1;
    var iter2;
    var pos1;
    var pos2;
    if (this._withIntergraphSegments) {
      iter1 = node1.getAllSegmentsFrom(true, true, true, false);
    }
    else {
      iter1 = node1.getSegmentsFrom();
    }
    while (iter1.hasNext())
    {
      pos1 = iter1.next().getCrossRedToPosition();
      if (this._withIntergraphSegments) {
        iter2 = node2.getAllSegmentsFrom(true, true, true, false);
      }
      else {
        iter2 = node2.getSegmentsFrom();
      }
      while (iter2.hasNext())
      {
        pos2 = iter2.next().getCrossRedToPosition();
        if (pos2 < pos1) {
          numCrossingsBeforeSwap++;
        }
        if (pos2 > pos1) {
          numCrossingsAfterSwap++;
        }
      }
    }

    if (this._withIntergraphSegments) {
      iter1 = node1.getAllSegmentsTo(true, true, true, false);
    }
    else {
      iter1 = node1.getSegmentsTo();
    }
    while (iter1.hasNext())
    {
      pos1 = iter1.next().getCrossRedFromPosition();
      if (this._withIntergraphSegments) {
        iter2 = node2.getAllSegmentsTo(true, true, true, false);
      }
      else {
        iter2 = node2.getSegmentsTo();
      }
      while (iter2.hasNext())
      {
        pos2 = iter2.next().getCrossRedFromPosition();
        if (pos2 < pos1) {
          numCrossingsBeforeSwap++;
        }
        if (pos2 > pos1) {
          numCrossingsAfterSwap++;
        }
      }
    }

    return (numCrossingsBeforeSwap > numCrossingsAfterSwap); // Boolean
  }

}
);

lang.mixin(CrossingReductionSwapAlgorithm,
{

  swap : function(node1, node2)
  {
    var field = node1.getLevel().getNodesField();
    var position1 = node1.getPositionInLevel();
    var position2 = node2.getPositionInLevel();

    field[position1] = node2;
    field[position2] = node1;
    node1.setPositionInLevel(position2);
    node2.setPositionInLevel(position1);
  }

}
);

return CrossingReductionSwapAlgorithm;
});
