/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HEnum",	// #5
  "../hierarchical/HLevelSweepAlgorithm",	// #6
  "../hierarchical/HNodeSort",	// #7
  "../hierarchical/HPercentages",	// #8
  "../hierarchical/HSegmentBufferedIterator",	// #9
  "../internalutil/GraphModelUtil",	// #10
  "../internalutil/TranslateUtil"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HEnum,	// #5
    HLevelSweepAlgorithm,	// #6
    HNodeSort,	// #7
    HPercentages,	// #8
    HSegmentBufferedIterator,	// #9
    GraphModelUtil,	// #10
    TranslateUtil	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelSweepAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelSweepAlgorithm;
=====*/

var EastWestExpansionAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.EastWestExpansionAlgorithm", HLevelSweepAlgorithm, 
{
  // summary:
  //		EastWestExpansionAlgorithm introduces auxiliary nodes for the links
  //		with east-side or west-side port specification. 
  // description:
  //		EastWestExpansionAlgorithm introduces auxiliary nodes for the links
  //		with east-side or west-side port specification. 
  //		This is done after crossing reduction. By that it also resolves
  //		EAST&#95;OR&#95;WEST specifications and updates the level structure. Example:
  //		------        ----    ------    ---- |      |      becomes    |west| 
  //		|   |  |east| | node |                 |aux |  | node |  |aux | |    
  //		 |                 |node|  |      |  |node| ------                  
  //		----    ------    ---- (west)// | \(east)        (south)||       
  //		|(south) |(south) //  |  \                    ||        |        |   
  //		 //   |(south)                ||        |        |  The connector
  //		calculation later is able to handle these auxiliary nodes such that
  //		they visually disappear, and the final result will look like this: 
  //		------ &#95;&#95;&#95;&#95;&#95;|      |&#95;&#95;&#95;&#95;&#95; |  &#95;&#95;&#95;| node |     | | |   |      |     | |
  //		|    ------      | | |       |        | | |       |        |     Note
  //		that for intergraph links, the aux nodes are always in the same graph
  //		as the original node, since they must be in the same level as the
  //		original node.  
  // tags:
  //		private
  
  _nodeSortAlg: null,
  _numberAuxNodesInActLevel: null,
  _numberAuxNodes: null,
  _prevLevel: null,
  _actLevel: null,
  _nextLevel: null,

  constructor : function(graph)
  {
    this.init(graph);
    this._nodeSortAlg = new HNodeSort();
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._nodeSortAlg = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_EAST_WEST_EXPANSION], 2 * (graph.getNumberOfLevels() - 1), false);

    graph.updateLevelIDs();

    this._numberAuxNodes = 0;
    this.sweepForward();

    if (this._numberAuxNodes > 0)
    {
      this.sweepBackward();
    }
  },

  initForwardFirstLevel : function(level)
  {
    this._prevLevel = null;
    this._actLevel = level;
    var nextLevelNum = level.getNumID() + 1;
    if (this.getGraph().getNumberOfLevels() > nextLevelNum) {
      this._nextLevel = this.getGraph().getLevel(nextLevelNum);
    }
    else {
      this._nextLevel = null;
    }
    this.treatForwardLevel();
  },

  treatForwardLevel : function (a0, a1)
  {
    if(arguments.length == 2)
      return this.treatForwardLevel_overload_2(a0, a1);
    else if(arguments.length == 0)
      return this.treatForwardLevel_overload_0();
  },
  
  treatForwardLevel_overload_2 : function(prevLevel, level)
  {
    this._prevLevel = prevLevel;
    this._actLevel = level;
    var nextLevelNum = level.getNumID() + 1;
    if (this.getGraph().getNumberOfLevels() > nextLevelNum) {
      this._nextLevel = this.getGraph().getLevel(nextLevelNum);
    }
    else {
      this._nextLevel = null;
    }
    this.treatForwardLevel();
  },

  treatForwardLevel_overload_0 : function()
  {
    // summary:
    //		Treats one level.
    // tags:
    //		private
    
    if (this._prevLevel == null && this._nextLevel == null) {
      return;
    }
    if (this._actLevel.getNodesCount() == 0) {
      return;
    }
    this._numberAuxNodesInActLevel = 0;

    var iter = this._actLevel.getNodes();
    var node;

    while (iter.hasNext())
      this.resolveEastWestPortSides(iter.next());

    iter = this._actLevel.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();
      if (this.needsEastPortAuxNode(node)) {
        this._numberAuxNodesInActLevel++;
      }
      if (this.needsWestPortAuxNode(node)) {
        this._numberAuxNodesInActLevel++;
      }
    }

    this._numberAuxNodes += this._numberAuxNodesInActLevel;

    if (this._numberAuxNodesInActLevel > 0)
    {
      this._actLevel.setCapacity(this._actLevel.getNodesCount() + this._numberAuxNodesInActLevel);
    }
  },

  resolveEastWestPortSides : function(node)
  {
    // summary:
    //		Resolves the east-or-west port side specifications of segments
    //		incident to this node, and checks whether an east-west auxiliary is
    //		necessary.
    // tags:
    //		private
    
    node.setMarker(0);
    if (node.isDummyNode()) {
      return;
    }
    var iter = node.getAllSegmentsFrom(true, true, false, false);
    var segment;
    var portSide;
    var prio;
    var baryCenter;
    node.calcSuccBaryCenter(true, 0);
    baryCenter = node.getSortValue();

    while (iter.hasNext())
    {
      segment = iter.next();
      portSide = segment.getFromPortSide();

      if (portSide == HEnum.EAST_OR_WEST)
      {
        if (segment.getCrossRedToPosition() < baryCenter) {
          portSide = HEnum.WEST;
        }
        else {
          portSide = HEnum.EAST;
        }
        segment.setFromPortSide(portSide);
      }

      if (portSide == HEnum.EAST)
      {
        if (this.isEastWestLink(segment.getOwnerLink())) {
          this.markHasEastWestLinkToEast(node);
        }
        else {
          this.markHasNormalLinkToEast(node);
        }
      }
      else if (portSide == HEnum.WEST)
      {
        if (this.isEastWestLink(segment.getOwnerLink())) {
          this.markHasEastWestLinkToWest(node);
        }
        else {
          this.markHasNormalLinkToWest(node);
        }
      }
    }

    node.calcPredBaryCenter(true, 0);
    baryCenter = node.getSortValue();
    iter = node.getAllSegmentsTo(true, true, false, false);

    while (iter.hasNext())
    {
      segment = iter.next();
      portSide = segment.getToPortSide();

      if (portSide == HEnum.EAST_OR_WEST)
      {
        if (segment.getCrossRedFromPosition() < baryCenter) {
          portSide = HEnum.WEST;
        }
        else {
          portSide = HEnum.EAST;
        }
        segment.setToPortSide(portSide);
      }

      if (portSide == HEnum.EAST)
      {
        if (this.isEastWestLink(segment.getOwnerLink())) {
          this.markHasEastWestLinkToEast(node);
        }
        else {
          this.markHasNormalLinkToEast(node);
        }
      }
      else if (portSide == HEnum.WEST)
      {
        if (this.isEastWestLink(segment.getOwnerLink())) {
          this.markHasEastWestLinkToWest(node);
        }
        else {
          this.markHasNormalLinkToWest(node);
        }
      }
    }

    node.correctPortNumbers();
  },

  initBackwardFirstLevel : function(level)
  {
    this._nextLevel = null;
    this._actLevel = level;
    var prevLevelNum = level.getNumID() - 1;
    if (prevLevelNum >= 0) {
      this._prevLevel = this.getGraph().getLevel(prevLevelNum);
    }
    else {
      this._prevLevel = null;
    }
    this.treatBackwardLevel();
  },

  treatBackwardLevel : function (a0, a1)
  {
    if(arguments.length == 2)
      return this.treatBackwardLevel_overload_2(a0, a1);
    else if(arguments.length == 0)
      return this.treatBackwardLevel_overload_0();
  },
  
  treatBackwardLevel_overload_2 : function(nextLevel, level)
  {
    this._nextLevel = nextLevel;
    this._actLevel = level;
    var prevLevelNum = level.getNumID() - 1;
    if (prevLevelNum >= 0) {
      this._prevLevel = this.getGraph().getLevel(prevLevelNum);
    }
    else {
      this._prevLevel = null;
    }
    this.treatBackwardLevel();
  },

  treatBackwardLevel_overload_0 : function()
  {
    // summary:
    //		Treats one level.
    // tags:
    //		private
    
    if (this._prevLevel == null && this._nextLevel == null) {
      return;
    }
    if (this._actLevel.getNodesCount() == 0) {
      return;
    }

    var lastNode = this._actLevel.getNode(this._actLevel.getNodesCount() - 1);

    var iter = this._actLevel.getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      this.createAuxNode(node);

      if (node == lastNode) {
        break;
      }
    }

    this._nodeSortAlg.sort(this._actLevel.getNodesField());

    this._actLevel.markInfoOutOfDate();
    this._actLevel.updateInfo();
  },

  createAuxNode : function(node)
  {
    // summary:
    //		Creates the auxiliary nodes for this node.
    // tags:
    //		private
    
    node.setSortValue(node.getPositionInLevel());
    if (node.isDummyNode()) {
      return;
    }

    var needsEastPortAuxNode = this.needsEastPortAuxNode(node);
    var needsWestPortAuxNode = this.needsWestPortAuxNode(node);

    var eastAuxNode = null;
    var westAuxNode = null;

    if (needsEastPortAuxNode)
    {
      eastAuxNode = node.createEastPortAuxNode();
      this._actLevel.add(eastAuxNode);
    }

    if (needsWestPortAuxNode)
    {
      westAuxNode = node.createWestPortAuxNode();
      this._actLevel.add(westAuxNode);
    }

    if (eastAuxNode == null && westAuxNode == null) {
      return;
    }

    var graph = this.getGraph();
    var iter;
    iter = new HSegmentBufferedIterator(node.getAllSegmentsFrom(true, true, false, false));
    var segment;
    var portSide;

    while (iter.hasNext())
    {
      segment = iter.next();
      portSide = segment.getFromPortSide();

      if (portSide == HEnum.EAST && eastAuxNode != null)
      {
        graph.removeEdge(segment);
        graph.addEdgeBetweenNodes(segment, eastAuxNode, segment.getTo());
        segment.setFromPortSide(HEnum.SOUTH);
      }
      else if (portSide == HEnum.WEST && westAuxNode != null)
      {
        graph.removeEdge(segment);
        graph.addEdgeBetweenNodes(segment, westAuxNode, segment.getTo());
        segment.setFromPortSide(HEnum.SOUTH);
      }
    }

    iter = new HSegmentBufferedIterator(node.getAllSegmentsTo(true, true, false, false));

    while (iter.hasNext())
    {
      segment = iter.next();
      portSide = segment.getToPortSide();

      if (portSide == HEnum.EAST && eastAuxNode != null)
      {
        graph.removeEdge(segment);
        graph.addEdgeBetweenNodes(segment, segment.getFrom(), eastAuxNode);
        segment.setToPortSide(HEnum.NORTH);
      }
      else if (portSide == HEnum.WEST && westAuxNode != null)
      {
        graph.removeEdge(segment);
        graph.addEdgeBetweenNodes(segment, segment.getFrom(), westAuxNode);
        segment.setToPortSide(HEnum.NORTH);
      }
    }

    if (eastAuxNode != null) {
      eastAuxNode.correctEastWestAuxSize();
    }
    if (westAuxNode != null) {
      westAuxNode.correctEastWestAuxSize();
    }
  },

  markHasNormalLinkToEast : function(node)
  {
    node.setMarker(node.getMarker() | EastWestExpansionAlgorithm.HAS_BP_LINK_TO_EAST);
  },

  markHasNormalLinkToWest : function(node)
  {
    node.setMarker(node.getMarker() | EastWestExpansionAlgorithm.HAS_BP_LINK_TO_WEST);
  },

  markHasEastWestLinkToEast : function(node)
  {
    node.setMarker(node.getMarker() | EastWestExpansionAlgorithm.HAS_EW_LINK_TO_EAST);
  },

  markHasEastWestLinkToWest : function(node)
  {
    node.setMarker(node.getMarker() | EastWestExpansionAlgorithm.HAS_EW_LINK_TO_WEST);
  },

  hasNormalLinkToEast : function(node)
  {
    return (node.getMarker() & EastWestExpansionAlgorithm.HAS_BP_LINK_TO_EAST) != 0; // Boolean
  },

  hasNormalLinkToWest : function(node)
  {
    return (node.getMarker() & EastWestExpansionAlgorithm.HAS_BP_LINK_TO_WEST) != 0; // Boolean
  },

  hasEastWestLinkToEast : function(node)
  {
    return (node.getMarker() & EastWestExpansionAlgorithm.HAS_EW_LINK_TO_EAST) != 0; // Boolean
  },

  hasEastWestLinkToWest : function(node)
  {
    return (node.getMarker() & EastWestExpansionAlgorithm.HAS_EW_LINK_TO_WEST) != 0; // Boolean
  },

  needsEastPortAuxNode : function(node)
  {
    // summary:
    //		Returns 'true' if the input node needs an east node.
    // tags:
    //		private
    
    if (this.hasNormalLinkToEast(node)) {
      return true; // Boolean
    }
    if (this.hasEastWestLinkToEast(node))
    {
      var eastNode = node.getLevel().getNode(node.getPositionInLevel() + 1);
      if (eastNode.isEastWestPortAuxNode()) {
        return true; // Boolean
      }
      if (this.hasNormalLinkToWest(eastNode)) {
        return true; // Boolean
      }
    }

    return false; // Boolean
  },

  needsWestPortAuxNode : function(node)
  {
    // summary:
    //		Returns 'true' if the input node needs a west node.
    // tags:
    //		private
    
    if (this.hasNormalLinkToWest(node)) {
      return true; // Boolean
    }
    if (this.hasEastWestLinkToWest(node))
    {
      var westNode = node.getLevel().getNode(node.getPositionInLevel() - 1);
      if (westNode.isEastWestPortAuxNode()) {
        return true; // Boolean
      }
      if (this.hasNormalLinkToEast(westNode)) {
        return true; // Boolean
      }
    }

    return false; // Boolean
  },

  isEastWestLink : function(link)
  {
    // summary:
    //		Returns 'true' if the input link is an east-west link. 
    // description:
    //		Returns <code>true</code> if the input link is an east-west link. 
    //		That means the link can be drawn straight between two neighbored
    //		nodes. This is nearly the same code as in
    //		EastWestLinkRemovalAlgorithm.
    // tags:
    //		private
    
    var from = link.getFrom();
    var toobj = link.getTo();

    if (from.getOwnerGraph() != toobj.getOwnerGraph()) {
      return false; // Boolean
    }
    if (from.getLevelNumber() != toobj.getLevelNumber()) {
      return false; // Boolean
    }
    var diff = (from.getPositionInLevel() - toobj.getPositionInLevel());
    if (diff != 1 && diff != -1) {
      return false; // Boolean
    }
    if (from.isEastWestPortAuxNode()) {
      return false; // Boolean
    }
    if (toobj.isEastWestPortAuxNode()) {
      return false; // Boolean
    }
    if (link.isFromPortNumberSpecified()) {
      return false; // Boolean
    }
    if (link.isToPortNumberSpecified()) {
      return false; // Boolean
    }

    var westNodeSide = ((diff < 0 ? link.getOrigFromPortSide() : link.getOrigToPortSide()));
    var eastNodeSide = ((diff < 0 ? link.getOrigToPortSide() : link.getOrigFromPortSide()));
    if (westNodeSide != HEnum.UNSPECIFIED && westNodeSide != HEnum.EAST && westNodeSide != HEnum.EAST_OR_WEST) {
      return false; // Boolean
    }
    if (eastNodeSide != HEnum.UNSPECIFIED && eastNodeSide != HEnum.WEST && eastNodeSide != HEnum.EAST_OR_WEST) {
      return false; // Boolean
    }
    return true; // Boolean
  }

}
);

lang.mixin(EastWestExpansionAlgorithm,
{
  HAS_BP_LINK_TO_EAST : null,
  HAS_BP_LINK_TO_WEST : null,
  HAS_EW_LINK_TO_EAST : null,
  HAS_EW_LINK_TO_WEST : null,
  _fields_init : function(c)
  {
    c = c || EastWestExpansionAlgorithm;
    if(HLevelSweepAlgorithm._fields_init)
      HLevelSweepAlgorithm._fields_init(c);
    
    c.HAS_BP_LINK_TO_EAST = 1;
    c.HAS_BP_LINK_TO_WEST = 2;
    c.HAS_EW_LINK_TO_EAST = 4;
    c.HAS_EW_LINK_TO_WEST = 8;
  }
  
}
);

(function(){ EastWestExpansionAlgorithm._fields_init(); })();

return EastWestExpansionAlgorithm;
});
