/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HGraphAlgorithm",	// #5
  "../hierarchical/HLink",	// #6
  "../hierarchical/HPercentages",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/HashTable",	// #9
  "../internalutil/TranslateUtil"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HGraphAlgorithm,	// #5
    HLink,	// #6
    HPercentages,	// #7
    GraphModelUtil,	// #8
    HashTable,	// #9
    TranslateUtil	// #10
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var EastWestLinkRoutingAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.EastWestLinkRoutingAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		EastWestLinkRoutingAlgorithm routes those links that were removed by
  //		the EastWestLinkRemovalAlgorithm. 
  // description:
  //		EastWestLinkRoutingAlgorithm routes those links that were removed by
  //		the EastWestLinkRemovalAlgorithm. 
  //		These are links between pairs of neighbored nodes. It is called after
  //		the nodes go their final positions (that is, after orthogonal link
  //		conflict resultion and everything else that may change node
  //		positions).  
  //		<p>
  //		For straight and polyline links, the task is fairly simple: we
  //		calculate the connection points of the links and keep them straight.
  //		For orthogonal links, we must introduce two dummy nodes and carefully
  //		resolve conflicts between segments if there are multilink bundles.
  //		The conflict resolution is basically the same algorithm as
  //		OrthRoutingLevelAlgorithm, however here I choose a different
  //		implementation that is optimized for small numbers of links.
  //		Multilink bundles usually have only a few links (my guess: less than
  //		8).  
  //		</p>
  //		<p>
  //		Note that east-west links are never intergraph links.  
  //		</p>
  // tags:
  //		private
  
  _edgeFlowDir: null,
  _levelFlowDir: null,
  _neighborLinksAligned: null,
  _eastlinkTable : null,

  constructor : function(graph)
  {
    this._eastlinkTable = new HashTable();
    this.init(graph);
    this._edgeFlowDir = graph.getEdgeFlow();
    this._levelFlowDir = graph.getLevelFlow();
    this._neighborLinksAligned = graph.getLayout().isNeighborLinksAligned();
  },
  
  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var numLinks = graph.getNumberOfEastWestLinks();
    var hasOrthogonalLinks = graph.hasOrthogonalLinks();
    var percController = this.getPercController();
    percController.startStep(this.getPercentages().steps[HPercentages.STEP_EAST_WEST_ROUTING], 4, false);
    if (numLinks == 0) {
      return;
    }

    var iter = graph.getEastWestLinks();
    var link;
    iter = graph.getEastWestLinks();

    while (iter.hasNext())
    {
      link = iter.next();

      graph.addEdge(link.getStartSegment());
    }

    iter = graph.getEastWestLinks();

    while (iter.hasNext())
    {
      link = iter.next();
      link.getFrom().setMarker(1);
      link.getTo().setMarker(1);
    }

    percController.addPoints(1);
    this.layoutStepPerformed();
    this.calcEastSideLinkNumbers();
    percController.addPoints(1);
    this.layoutStepPerformed();
    this.calcConnectionPoints();
    percController.addPoints(1);
    this.layoutStepPerformed();
    if (hasOrthogonalLinks) {
      this.makeLinksOrthogonal();
    }
    percController.addPoints(1);
    this.layoutStepPerformed();
  },

  calcEastSideLinkNumbers : function()
  {
    // summary:
    //		Calculate how many east-west links are on the east side of each
    //		node. 
    // description:
    //		Calculate how many east-west links are on the east side of each
    //		node. 
    //		We
    //		don't need to know it at the west side, because we can look at the west neighbor's
    //		east side for that number. Afterwards, the following holds: - the
    //		marker field of each node is the number of links going towards east
    //		plus one - the multilink index of each link is the index (1 ... n)
    //		of the link in the link bundle. If we look at the west node of the
    //		link, then the index is smaller than the marker field of the west
    //		node.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getEastWestLinks();
    var link;
    var westNode;
    var numEastLinks;

    while (iter.hasNext())
    {
      link = iter.next();
      westNode = this.getWestNode(link);

      numEastLinks = westNode.getMarker();
      link.setMultiLinkIndex(numEastLinks);
      westNode.setMarker(numEastLinks + 1);
    }
  },

  calcConnectionPoints : function (a0)
  {
    // summary:
    //		Calculate the connection points for all east west links.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcConnectionPoints_overload_0();
    else if(arguments.length == 1)
      return this.calcConnectionPoints_overload_1(a0);
  },
  
  calcConnectionPoints_overload_0 : function()
                                              
  {
    var graph = this.getGraph();
    var iter = graph.getEastWestLinks();

    while (iter.hasNext())
    {
      this.calcConnectionPoints(iter.next());
    }
  },

  calcConnectionPoints_overload_1 : function(link)
  {
    // summary:
    //		Calculate the connection points for the input link. 
    // description:
    //		Calculate the connection points for the input link. 
    //		At this time point, the link is straight.
    // tags:
    //		private
    
    if (link.isFromSideFixed()) {
      link.setFromCoordWhenFixed();
    }
    if (link.isToSideFixed()) {
      link.setToCoordWhenFixed();
    }
    if (link.isFromSideFixed() && link.isToSideFixed()) {
      return;
    }
    var xdir = this._levelFlowDir;
    var ydir = this._edgeFlowDir;
    var linknum = link.getMultiLinkIndex();
    var numLinks;
    var from = link.getFrom();
    var toobj = link.getTo();

    if (from.getPositionInLevel() < toobj.getPositionInLevel())
    {
      numLinks = from.getMarker();
      link.setFromCoordWhenFree(xdir, from.getCoord(xdir) + from.getSize(xdir));
      link.setToCoordWhenFree(xdir, toobj.getCoord(xdir));
    }
    else
    {
      numLinks = toobj.getMarker();
      link.setFromCoordWhenFree(xdir, from.getCoord(xdir));
      link.setToCoordWhenFree(xdir, toobj.getCoord(xdir) + toobj.getSize(xdir));
    }

    var fromMinY = from.getCoord(ydir);
    var fromMaxY = from.getCoord(ydir) + from.getSize(ydir);
    var toMinY = toobj.getCoord(ydir);
    var toMaxY = toobj.getCoord(ydir) + toobj.getSize(ydir);
    var f = linknum / numLinks;

    if (this._neighborLinksAligned)
    {
      if (fromMinY <= toMinY && fromMaxY >= toMaxY)
      {
        fromMinY = toMinY;
        fromMaxY = toMaxY;
      }
      else if (toMinY <= fromMinY && toMaxY >= fromMaxY)
      {
        toMinY = fromMinY;
        toMaxY = fromMaxY;
      }
    }

    link.setFromCoordWhenFree(ydir, fromMinY + f * (fromMaxY - fromMinY));
    link.setToCoordWhenFree(ydir, toMinY + f * (toMaxY - toMinY));
  },

  makeLinksOrthogonal : function()
  {
    // summary:
    //		Make all east-west links orthogonal.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getEastWestLinks();
    var link;
    var westNode;
    var numEastLinks;

    while (iter.hasNext())
    {
      link = iter.next();
      if (!link.isOrthogonal()) {
        continue;
      }
      westNode = this.getWestNode(link);

      numEastLinks = westNode.getMarker() - 1;

      if (numEastLinks == 1) {
        this.makeOrthogonal(link);
      }
      else
      {
        var obj = this._eastlinkTable.item(westNode);
        var v = null;

        if (obj == null)
        {
          v = new Array();
          this._eastlinkTable.add(westNode, v);
        }
        else
        {
          v = obj;
        }

        v.push(link);
      }
    }

    var e = new TranslateUtil.ArrayIterator(this._eastlinkTable.getValueList());

    while (e.hasNext())
      this.makeBundleOrthogonal(e.next());
  },

  makeOrthogonal : function (a0, a1)
  {
    // summary:
    //		Make one simple link orthogonal. 
    // description:
    //		Make one simple link orthogonal. 
    //		The link is not part of a link bundle.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.makeOrthogonal_overload_1(a0);
    else if(arguments.length == 2)
      return this.makeOrthogonal_overload_2(a0, a1);
  },
  
  makeOrthogonal_overload_1 : function(link)
                                            
  {
    var xdir = this._levelFlowDir;
    var ydir = this._edgeFlowDir;
    var middle = 0.5 * (link.getFromCoord(xdir) + link.getToCoord(xdir));
    this.makeOrthogonal(link, middle);
  },

  makeBundleOrthogonal : function(linksVector)
  {
    // summary:
    //		Make a bundle of multilinks orthogonal. 
    // description:
    //		Make a bundle of multilinks orthogonal. 
    //		The input vector is the vector of links between the same pair of
    //		nodes. This is a simple O(n^3) algorithm. Speed
    //		doesn't matter because the multilink bundles usually have only few links.  
    // tags:
    //		private
    
    var e;
    var link;
    var xdir = this._levelFlowDir;
    var ydir = this._edgeFlowDir;
    var changed;
    var min;
    var max;
    var minDelta;
    var min1;
    var min2;
    var max1;
    var max2;
    var i;
    var j;

    e =  new TranslateUtil.ArrayIterator(linksVector);
    link = null;

    while (e.hasNext())
    {
      link = e.next();
      link.setMultiLinkIndex(0);
    }

    min = link.getFromCoord(xdir);
    max = link.getToCoord(xdir);

    if (min > max)
    {
      max = min;
      min = link.getToCoord(xdir);
    }

    minDelta = Math.min(link.getFrom().getSize(ydir), link.getTo().getSize(ydir));
    minDelta = 0.9 * minDelta / (linksVector.length + 1);

    var links;

    links = TranslateUtil.newArray(linksVector.length, null);
    TranslateUtil.copyArrayInto(linksVector, links);

    for (i = 0; i < links.length; i++)
    {
      min1 = Math.min(links[i].getFromCoord(ydir), links[i].getToCoord(ydir));
      max1 = Math.min(links[i].getFromCoord(ydir), links[i].getToCoord(ydir));

      for (j = i + 1; j < links.length; j++)
      {
        min2 = Math.min(links[j].getFromCoord(ydir), links[j].getToCoord(ydir));
        max2 = Math.min(links[j].getFromCoord(ydir), links[j].getToCoord(ydir));

        if (min2 < min1 || (min1 == min2 && max2 < max1))
        {
          link = links[j];
          links[j] = links[i];
          links[i] = link;
          min1 = min2;
          max1 = max2;
        }
      }
    }

    changed = true;
    var aux;

    while (changed)
    {
      changed = this.resolveConflicts(links, minDelta, true);
      aux = this.resolveConflicts(links, minDelta, false);
      changed = aux || changed;
    }

    var numberOfOrthogonalRows = 0;

    for (i = 0; i < links.length; i++)
      if (numberOfOrthogonalRows < links[i].getMultiLinkIndex()) {
        numberOfOrthogonalRows = links[i].getMultiLinkIndex();
      }
    numberOfOrthogonalRows++;

    for (i = 0; i < links.length; i++)
      this.makeOrthogonal(links[i], min + (max - min) * (links[i].getMultiLinkIndex() + 2) / (numberOfOrthogonalRows + 3));
  },

  resolveConflicts : function(links, minDelta, forwardSweep)
  {
    // summary:
    //		Sweep over the links and resolve conflicts. 
    // description:
    //		Sweep over the links and resolve conflicts. 
    //		The algorithm works also for all other flow directions, but for
    //		simplicity, assume top-down layout in the explanations: A conflict
    //		is if two links have the same multilink index but their
    //		xdir-interval is overlapping. The x-dir interval is [
    //		min(fromx,tox), max(fromx,tox)+minDelta ].  In a forwards sweep, we
    //		solve conflicts by lifting type 1 links to the next orthogonal row,
    //		and create this shape: &#95;&#95;&#95;&#95;&#95;&#95;&#95; &#95;&#95;&#95;&#95;&#95;  |&#95;&#95;&#95; &#95;&#95;&#95;  |&#95;&#95;&#95;&#95;&#95; |&#95;&#95;&#95;&#95;&#95;&#95;&#95; In a
    //		backward sweep, we solve conflicts by lifting type 2 link to the
    //		next orthogonal row, and create this shape: &#95;&#95;&#95;&#95;&#95;&#95;&#95; &#95;&#95;&#95;|  &#95;&#95;&#95;&#95;&#95;
    //		&#95;&#95;&#95;&#95;&#95;|  &#95;&#95;&#95; &#95;&#95;&#95;&#95;&#95;&#95;&#95;|
    // tags:
    //		private
    
    var changed = false;
    var i;
    var j;
    var step;
    var start;
    var end;
    var link1;
    var link2;
    var min1;
    var max1;
    var min2;
    var max2;
    var type1;
    var type2;
    var xdir = this._levelFlowDir;
    var ydir = this._edgeFlowDir;
    var typeToMove;

    if (forwardSweep)
    {
      step = 1;
      start = 0;
      end = links.length;
      typeToMove = 1;
    }
    else
    {
      step = -1;
      start = links.length - 1;
      end = -1;
      typeToMove = 2;
    }

    for (i = start; i != end; i += step)
    {
      link1 = links[i];
      min1 = Math.min(link1.getFromCoord(ydir), link1.getToCoord(ydir));
      max1 = Math.max(link1.getFromCoord(ydir), link1.getToCoord(ydir));
      if (min1 == max1) {
        continue;
      }
      type1 = this.getLinkType(link1, xdir, ydir);

      for (j = i + step; j != end; j += step)
      {
        link2 = links[j];
        min2 = Math.min(link2.getFromCoord(ydir), link2.getToCoord(ydir));
        max2 = Math.max(link2.getFromCoord(ydir), link2.getToCoord(ydir));
        if (min2 == max2) {
          continue;
        }
        type2 = this.getLinkType(link2, xdir, ydir);

        if (link1.getMultiLinkIndex() == link2.getMultiLinkIndex())
        {
          if (min1 <= min2 && min2 <= max1 + minDelta)
          {
            if (type1 == 1 && type1 == typeToMove)
            {
              link1.setMultiLinkIndex(link1.getMultiLinkIndex() + 1);
              changed = true;
            }
            else if (type2 == typeToMove)
            {
              link2.setMultiLinkIndex(link2.getMultiLinkIndex() + 1);
              changed = true;
            }
          }
          else if (min2 <= min1 && min1 <= max2 + minDelta)
          {
            if (type2 == 1 && type2 == typeToMove)
            {
              link2.setMultiLinkIndex(link2.getMultiLinkIndex() + 1);
              changed = true;
            }
            else if (type1 == typeToMove)
            {
              link1.setMultiLinkIndex(link1.getMultiLinkIndex() + 1);
              changed = true;
            }
          }
        }
      }
    }

    return changed; // Boolean
  },

  makeOrthogonal_overload_2 : function(link, orthSegmentCoord)
  {
    // summary:
    //		This really makes a link orthogonal.
    // tags:
    //		private
    
    var xdir = this._levelFlowDir;
    var ydir = this._edgeFlowDir;
    if (link.getFromCoord(ydir) == link.getToCoord(ydir)) {
      return;
    }
    link.addTwoDummyNodesAtEastWestLink();
    var dummyNode1 = link.getStartSegment().getTo();
    var dummyNode2 = link.getEndSegment().getFrom();

    dummyNode1.setCoord(ydir, link.getFromCoord(ydir));
    dummyNode2.setCoord(ydir, link.getToCoord(ydir));
    dummyNode1.setCoord(xdir, orthSegmentCoord);
    dummyNode2.setCoord(xdir, orthSegmentCoord);
  },

  getLinkType : function(link, xdir, ydir)
  {
    // summary:
    //		Returns the type of a link. 
    // description:
    //		Returns the type of a link. 
    //		The type is 1 or 2. Type1:  &#95;&#95;        Type2:    &#95;&#95; |&#95;&#95;           
    //		&#95;&#95;|
    // tags:
    //		private
    
    if (link.getFromCoord(xdir) < link.getToCoord(xdir))
    {
      if (link.getFromCoord(ydir) < link.getToCoord(ydir)) {
        return 1; // int
      }
      else {
        return 2; // int
      }
    }
    else
    {
      if (link.getFromCoord(ydir) < link.getToCoord(ydir)) {
        return 2; // int
      }
      else {
        return 1; // int
      }
    }
  },

  getWestNode : function(link)
  {
    // summary:
    //		Returns the west node of the link.
    // tags:
    //		private
    
    if (link.getFrom().getPositionInLevel() < link.getTo().getPositionInLevel()) {
      return link.getFrom(); // ibm_ilog.graphlayout.hierarchical.HNode
    }
    else {
      return link.getTo(); // ibm_ilog.graphlayout.hierarchical.HNode
    }
  }

}
);

return EastWestLinkRoutingAlgorithm;
});
