/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HLevelSweepAlgorithm",	// #5
  "../hierarchical/HSegmentSort",	// #6
  "../internalutil/GraphModelUtil",	// #7
  "../internalutil/HashTable",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLevelSweepAlgorithm,	// #5
    HSegmentSort,	// #6
    GraphModelUtil,	// #7
    HashTable,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelSweepAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelSweepAlgorithm;
=====*/

var ForkPointAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.ForkPointAlgorithm", HLevelSweepAlgorithm, 
{
  // summary:
  //		ForkPointAlgorithm traverses the levels and calculates the "from" and
  //		"to" fork points of the links if fork link shapes are enabled. 
  // description:
  //		ForkPointAlgorithm traverses the levels and calculates the "from" and
  //		"to" fork points of the links if fork link shapes are enabled. 
  //		This must be done after the links have their final end points. The
  //		fork points are stored in the links. During orthogonal routing, the
  //		fork points are used as start points of the links, and later two
  //		additional segments are routed from the start of the link to the fork
  //		point. This will give the typical fork shape.  
  //		<p>
  //		Note that a link may have a fork at the start side and at the end
  //		side. Therefore both sides influence each other, hence we need
  //		several sweeps  to calculate the final fork shapes.  
  //		</p>
  // tags:
  //		private
  
  _levelFlowDirection: null,
  _useFromFork: null,
  _useToFork: null,
  _prefForkAxisLength: null,
  _linkDist: null,
  _fromForks : null,
  _toForks : null,
  _sortAlg : null,
  _recursiveMode: null,
  _forkingLinks: null,
  _forkingDummies: null,

  constructor : function(graph)
  {
    this._fromForks = new Array();
    this._toForks = new Array();
    this._sortAlg = new HSegmentSort();
    this.init(graph);
    this._levelFlowDirection = graph.getLevelFlow();
    this._useFromFork = graph.getLayout().isFromFork();
    this._useToFork = graph.getLayout().isToFork();
    this._prefForkAxisLength = graph.getLayout().getPreferredForkAxisLength();
    this._linkDist = graph.getMinDistBetweenLinks(this._levelFlowDirection);

    this._recursiveMode = graph.getLayout().isRecursiveLayoutMode();
    this._forkingLinks = new Array();
    this._forkingDummies = new HashTable();
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    if (!this._useFromFork && !this._useToFork) {
      return;
    }
    this.sweepForward();
  },

  treatForwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number 1 ... 
    // description:
    //		Treats a level (with level number 1 ... 
    //		n) in a forward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i-1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.calcInitialForkPoints(prevLevel, level);
    var changed = true;
    var counter = 0;

    while (changed && counter < 20)
    {
      counter++;
      changed = this.correctForkPoints();
    }

    this.markNeedsFork(prevLevel, true);
    this.markNeedsFork(level, false);
  },

  treatBackwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number n-1 ... 
    // description:
    //		Treats a level (with level number n-1 ... 
    //		0) in a backward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i+1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
  },

  calcInitialForkPoints : function (a0, a1, a2)
  {
    // summary:
    //		Calculate the initial fork points.
    // tags:
    //		private
    
    if(arguments.length == 2)
      return this.calcInitialForkPoints_overload_2(a0, a1);
    else if(arguments.length == 3)
      return this.calcInitialForkPoints_overload_3(a0, a1, a2);
  },
  
  calcInitialForkPoints_overload_2 : function(fromLevel, toLevel)
                                                                
  {
    this.calcInitialForkPoints(fromLevel, this._fromForks, true);
    this.calcInitialForkPoints(toLevel, this._toForks, false);
  },

  calcInitialForkPoints_overload_3 : function(level, datas, fromSide)
  {
    // summary:
    //		Calculate the initial fork points.
    // tags:
    //		private
    
    this.collectForkData(level, datas, fromSide);

    this.sortForkDataArray(datas, fromSide);

    var e =  new TranslateUtil.ArrayIterator(datas);
    var data;
    var width;
    var leftWidth;
    var coord;
    var delta;

    while (e.hasNext())
    {
      data = e.next();
      delta = 0;

      if (data.leftLimit < data.forkCoordinate)
      {
        delta = this._linkDist;

        while (data.leftLimit + 1.5 * delta >= data.forkCoordinate)
          delta = 0.5 * delta;
      }

      data.leftLimit = data.leftLimit + delta;
      delta = 0;

      if (data.rightLimit > data.forkCoordinate)
      {
        delta = this._linkDist;

        while (data.rightLimit - 1.5 * delta <= data.forkCoordinate)
          delta = 0.5 * delta;
      }

      data.rightLimit = data.rightLimit - delta;
      width = (data.segments.length - 1) * this._prefForkAxisLength;
      data.leftUsed = data.forkCoordinate - 0.5 * width;
      data.rightUsed = data.forkCoordinate + 0.5 * width;
      if (data.leftUsed < data.leftLimit) {
        data.leftUsed = data.leftLimit;
      }
      if (data.rightUsed > data.rightLimit) {
        data.rightUsed = data.rightLimit;
      }
    }

    var prevData = null;
    e =  new TranslateUtil.ArrayIterator(datas);

    while (e.hasNext())
    {
      data = e.next();

      if (prevData != null)
      {
        delta = (data.forkCoordinate - prevData.forkCoordinate) / 5;
        if (delta > this._linkDist) {
          delta = this._linkDist;
        }
        if (prevData.rightUsed + delta >= data.leftUsed)
        {
          var w1 = prevData.rightUsed - prevData.forkCoordinate;
          var w2 = data.forkCoordinate - data.leftUsed;

          if (w1 + w2 > 0)
          {
            width = data.forkCoordinate - prevData.forkCoordinate - delta;
            prevData.rightUsed = prevData.forkCoordinate + width * w1 / (w1 + w2);
            data.leftUsed = data.forkCoordinate - width * w2 / (w1 + w2);
          }
        }
      }

      prevData = data;
    }

    e =  new TranslateUtil.ArrayIterator(datas);

    while (e.hasNext())
    {
      data = e.next();
      width = this._prefForkAxisLength * data.segments.length;
      leftWidth = 0.5 * width;
      if (data.forkCoordinate - leftWidth < data.leftUsed) {
        leftWidth = data.forkCoordinate - data.leftUsed;
      }
      if (data.forkCoordinate - leftWidth + width > data.rightUsed) {
        width = data.rightUsed - data.forkCoordinate + leftWidth;
      }
      var n = data.segments.length - 1;
      var i = 0;
      data.delta = width / n;
      var e1 =  new TranslateUtil.ArrayIterator(data.segments);

      while (e1.hasNext())
      {
        var segment = e1.next();
        coord = data.forkCoordinate - leftWidth + i * data.delta;
        this.setForkPoint(segment, fromSide, coord);
        i++;
      }
    }
  },

  collectForkData : function(level, datas, fromSide)
  {
    // summary:
    //		Collect and fill datas with the ForkData items.
    // tags:
    //		private
    
    var iter;
    var segment;
    var coord;
    var pendingSegment = null;
    var forkCoord = -/*Number.MAX_VALUE*/(3.4028235E38);
    var leftLimit = -/*Number.MAX_VALUE*/(3.4028235E38);
    var actForkData = null;
    datas.length = 0;
    var segments = new Array();
    iter = this.getSegments(level, fromSide);

    while (iter.hasNext())
    {
      segment = iter.next();
      segments.push(segment);

      if (fromSide)
      {
        coord = segment.getRealFromCoord(this._levelFlowDirection);
        segment.setFromForkPoint(coord);
        segment.setSortValue(coord);
      }
      else
      {
        coord = segment.getRealToCoord(this._levelFlowDirection);
        segment.setToForkPoint(coord);
        segment.setSortValue(coord);
      }
    }

    var array;

    array = segments;
    this._sortAlg.sort(array);

    for (var i = 0; i < array.length; i++)
    {
      segment = array[i];
      coord = segment.getSortValue();

      if (coord == forkCoord && this.forkAllowed(segment, fromSide))
      {
        if (actForkData == null)
        {
          actForkData = new ForkPointAlgorithm.ForkData();
          actForkData.forkCoordinate = forkCoord;
          actForkData.leftLimit = leftLimit;
          actForkData.rightLimit = /*Number.MAX_VALUE*/(3.4028235E38);
          actForkData.segments = new Array();

          if (pendingSegment != null)
          {
            actForkData.segments.push(pendingSegment);
            this.registerForkingSegment(pendingSegment, fromSide);
          }

          pendingSegment = null;
          datas.push(actForkData);
        }

        actForkData.segments.push(segment);
        this.registerForkingSegment(segment, fromSide);
      }
      else
      {
        leftLimit = forkCoord;
        forkCoord = coord;
        pendingSegment = segment;
        if (actForkData != null) {
          actForkData.rightLimit = coord;
        }
        actForkData = null;
      }
    }
  },

  forkAllowed : function(segment, fromSide)
  {
    // summary:
    //		Returns true if fork is allowed for the input segment.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var intergraphSegment = segment.isIntergraphEdge();
    var opp;

    if (fromSide)
    {
      var toNode = segment.getTo();

      if (toNode.isDummyNode() && toNode.getOwnerGraph() == graph)
      {
        opp = toNode.getIntergraphOpposite(segment);
        if (opp != null && opp.isIntergraphEdge() && toNode.getOwnerGraph() != opp.getOwnerGraph()) {
          return false; // Boolean
        }
      }
    }
    else
    {
      var fromNode = segment.getFrom();

      if (fromNode.isDummyNode() && fromNode.getOwnerGraph() == graph)
      {
        opp = fromNode.getIntergraphOpposite(segment);
        if (opp != null && opp.isIntergraphEdge() && fromNode.getOwnerGraph() != opp.getOwnerGraph()) {
          return false; // Boolean
        }
      }
    }

    if (fromSide)
    {
      if (!segment.canHaveFromFork()) {
        return false; // Boolean
      }
      if (segment.isReversed()) {
        return this._useToFork; // Boolean
      }
      else {
        return this._useFromFork; // Boolean
      }
    }
    else
    {
      if (!segment.canHaveToFork()) {
        return false; // Boolean
      }
      if (segment.isReversed()) {
        return this._useFromFork; // Boolean
      }
      else {
        return this._useToFork; // Boolean
      }
    }
  },

  sortForkDataArray : function(datas, fromSide)
  {
    // summary:
    //		Sort the segments in all fork
    //		data's according to the other coordinate than specified by fromSide.  
    // tags:
    //		private
    
    var e =  new TranslateUtil.ArrayIterator(datas);

    while (e.hasNext())
      this.sortForkData(e.next(), fromSide);
  },

  sortForkData : function(data, fromSide)
  {
    // summary:
    //		Sort the segments in this fork data according to the other
    //		coordinate than specified by fromSide.
    // tags:
    //		private
    
    var i;

    var array;

    array = data.segments;
    for (i = 0; i < array.length; i++)
    {
      var segment = array[i];
      if (fromSide) {
        segment.setSortValue(segment.getRealToCoord(this._levelFlowDirection));
      }
      else {
        segment.setSortValue(segment.getRealFromCoord(this._levelFlowDirection));
      }
    }

    this._sortAlg.sort(array);
  },

  correctForkPoints : function (a0, a1)
  {
    // summary:
    //		Correct the fork points, i.e. 
    // description:
    //		Correct the fork points, i.e. 
    //		if possible, move them to the other end point.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.correctForkPoints_overload_0();
    else if(arguments.length == 2)
      return this.correctForkPoints_overload_2(a0, a1);
  },
  
  correctForkPoints_overload_0 : function()
                                          
  {
    var changed = this.correctForkPoints(this._fromForks, true);
    var aux = this.correctForkPoints(this._toForks, false);
    changed = aux || changed;
    return changed; // Boolean
  },

  correctForkPoints_overload_2 : function(datas, fromSide)
  {
    // summary:
    // tags:
    //		private
    
    var changed = false;
    var e =  new TranslateUtil.ArrayIterator(datas);
    var data;
    var segment;
    var coord;
    var coord2;
    var otherCoord;
    var limit;
    var i;

    while (e.hasNext())
    {
      data = e.next();
      limit = data.leftLimit;

      for (i = 0; i < data.segments.length; i++)
      {
        if (i == data.segments.length - 1) {
          limit = Math.max(limit, data.forkCoordinate);
        }
        segment = data.segments[i];
        coord = coord2 = this.getForkPoint(segment, fromSide);
        otherCoord = this.getOppositeForkPoint(segment, !fromSide);

        if (coord >= otherCoord && otherCoord >= limit)
        {
          this.setForkPoint(segment, fromSide, otherCoord);
          coord2 = this.getForkPoint(segment, fromSide);
          if (coord != coord2) {
            changed = true;
          }
        }

        limit = coord2 + data.delta;
      }

      limit = data.rightLimit;

      for (i = data.segments.length - 1; i >= 0; i--)
      {
        if (i == 0) {
          limit = Math.min(limit, data.forkCoordinate);
        }
        segment = data.segments[i];
        coord = coord2 = this.getForkPoint(segment, fromSide);
        otherCoord = this.getOppositeForkPoint(segment, !fromSide);

        if (coord <= otherCoord && otherCoord <= limit)
        {
          this.setForkPoint(segment, fromSide, otherCoord);
          coord2 = this.getForkPoint(segment, fromSide);
          if (coord != coord2) {
            changed = true;
          }
        }

        limit = coord2 - data.delta;
      }
    }

    return changed; // Boolean
  },

  getForkPoint : function(segment, fromSide)
  {
    // summary:
    //		Returns the fork point.
    // tags:
    //		private
    
    if (fromSide) {
      return segment.getFromForkPoint(this._levelFlowDirection); // Number
    }
    else {
      return segment.getToForkPoint(this._levelFlowDirection); // Number
    }
  },

  getOppositeForkPoint : function(segment, fromSide)
  {
    // summary:
    //		Returns the fork point in the raw coordinate system of the opposite
    //		side. 
    // description:
    //		Returns the fork point in the raw coordinate system of the opposite
    //		side. 
    //		If fromSide is true, it returns the from fork point in the
    //		coordinate system of the to side, and vice versa. For normal
    //		segments, we just work on raw coordinates. For intergraph links
    //		inside this graph, the real coordinates of the link are the
    //		coordinates of the corresponding side, because we know at the
    //		corresponding side it a real node belonging to this graph. &#95;&#95;&#95;&#95;&#95;&#95;&#95; |
    //		 | N ---|-- N   | |&#95;&#95;&#95;&#95;&#95;&#95;&#95;| fork side   opposite side For intergraph
    //		links outside this graph, we can never run into this because if they
    //		end at a node inside this graph, there must be at least one other
    //		dummy node inside this graph. &#95;&#95;&#95;&#95;&#95;&#95;&#95; |       | |  N--D-|---N
    //		|&#95;&#95;&#95;&#95;&#95;&#95;&#95;| fork side   opposite side
    // tags:
    //		private
    
    return this.getForkPoint(segment, fromSide); // Number
  },

  setForkPoint : function(segment, fromSide, value)
  {
    // summary:
    //		Sets the fork point.
    // tags:
    //		private
    
    if (fromSide) {
      segment.setFromForkPoint(value);
    }
    else {
      segment.setToForkPoint(value);
    }
  },

  markNeedsFork : function(level, fromSide)
  {
    // summary:
    //		Marks which segments needs fork. 
    // description:
    //		Marks which segments needs fork. 
    //		This mark is needed by the orthogonal routing algorithm.
    
    var iter = this.getSegments(level, fromSide);
    var xdir = this._levelFlowDirection;

    while (iter.hasNext())
    {
      var segment = iter.next();
      var x0;
      var x1;

      if (fromSide)
      {
        x0 = segment.getRealFromCoord(xdir);
        x1 = segment.getFromForkPoint(xdir);
        segment.setNeedsFromFork(x0 != x1);
      }
      else
      {
        x0 = segment.getRealToCoord(xdir);
        x1 = segment.getToForkPoint(xdir);
        segment.setNeedsToFork(x0 != x1);
      }
    }
  },

  registerForkingSegment : function(segment, fromSide)
  {
    // summary:
    //		Registers a segment to participate in the fork.
    // tags:
    //		private
    
    TranslateUtil.addSetObject(this._forkingLinks, segment.getOwnerLink());

    if (fromSide)
    {
      var fromNode = segment.getFrom();
      if (fromNode.isDummyNode() && fromNode.getOwnerGraph() != segment.getOwnerGraph()) {
        this._forkingDummies.add(fromNode, new ForkPointAlgorithm.ForkDummyData(segment, true));
      }
    }
    else
    {
      var toNode = segment.getTo();
      if (toNode.isDummyNode() && toNode.getOwnerGraph() != segment.getOwnerGraph()) {
        this._forkingDummies.add(toNode, new ForkPointAlgorithm.ForkDummyData(segment, false));
      }
    }
  },

  swapForkAndEndPoints : function()
  {
    // summary:
    //		Swaps the fork points with the start and end points of all links. 
    // description:
    //		Swaps the fork points with the start and end points of all links. 
    //		This is used temporarily during orthogonal routing. After two
    //		switches, the original situation is again created.
    // tags:
    //		private
    
    if (!this._useFromFork && !this._useToFork) {
      return;
    }
    var graph = this.getGraph();

    var iter;

    iter = this._forkingDummies.getIterator();

    while (iter.hasNext())
    {
      var e = iter.next();
      var node = e.key;
      var data = e.value;
      this.swapForkingDummies(graph, node, data);
    }

    iter =  new TranslateUtil.ArrayIterator(this._forkingLinks);

    while (iter.hasNext())
    {
      var link = iter.next();
      link.swapForkAndEndPoints(this._levelFlowDirection);
    }
  },

  swapForkingDummies : function(graph, node, data)
  {
    if (data.valid)
    {
      node.setCoord(this._levelFlowDirection, data.oldCoord);
      data.valid = false;
    }
    else
    {
      data.oldCoord = node.getCoord(this._levelFlowDirection);
      data.valid = true;
      var h;

      if (data.fromForkSide)
      {
        h = data.link.getFromForkPoint();
        h = graph.getInverseTranslatedCoord(node, h, this._levelFlowDirection);
        node.setCoord(this._levelFlowDirection, h);
      }
      else
      {
        h = data.link.getToForkPoint();
        h = graph.getInverseTranslatedCoord(node, h, this._levelFlowDirection);
        node.setCoord(this._levelFlowDirection, h);
      }
    }
  },

  createForks : function (a0)
  {
    // summary:
    //		Route the fork at the links. 
    // description:
    //		Route the fork at the links. 
    //		This is called after the second swapForAndEndPoints. It makes the
    //		first and last segment orthogonal again and by this creates the
    //		fork.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.createForks_overload_0();
    else if(arguments.length == 1)
      return this.createForks_overload_1(a0);
  },
  
  createForks_overload_0 : function()
                                    
  {
    if (!this._useFromFork && !this._useToFork) {
      return;
    }
    var iter = this.getGraph().getLevels();

    while (iter.hasNext())
      this.createForks(iter.next());
  },

  createForks_overload_1 : function(level)
  {
    // summary:
    //		Route the fork at the links. 
    // description:
    //		Route the fork at the links. 
    //		This is called after the second swapForAndEndPoints. It makes the
    //		first and last segment orthogonal again and by this creates the
    //		fork.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var forkCoord = level.getFromForkCoord();
    var x0;
    var x1;
    var iter = this.getSegments(level, true);
    var segment;
    var link;
    var dummyNode;
    var xdir = this._levelFlowDirection;
    var ydir = 1 - xdir;
    var reversed;

    while (iter.hasNext())
    {
      segment = iter.next();
      link = segment.getOwnerLink();

      if (link.isOrthogonal())
      {
        reversed = segment.isReversed();
        x0 = segment.getRealFromCoord(xdir);
        x1 = segment.getFromForkPoint(xdir);

        if (x0 != x1)
        {
          dummyNode = link.addDummyNodeAtGraph(segment, graph);
          dummyNode.setCoord(xdir, ((reversed ? x0 : x1)));
          dummyNode.setCoord(ydir, forkCoord);
          dummyNode = link.addDummyNodeAtGraph(segment, graph);
          dummyNode.setCoord(xdir, ((reversed ? x1 : x0)));
          dummyNode.setCoord(ydir, forkCoord);
        }
      }
    }

    forkCoord = level.getToForkCoord();
    iter = this.getSegments(level, false);

    while (iter.hasNext())
    {
      segment = iter.next();
      link = segment.getOwnerLink();

      if (link.isOrthogonal())
      {
        reversed = segment.isReversed();
        x0 = segment.getRealToCoord(xdir);
        x1 = segment.getToForkPoint(xdir);

        if (x0 != x1)
        {
          dummyNode = link.addDummyNodeAtGraph(segment, graph);
          dummyNode.setCoord(xdir, ((reversed ? x1 : x0)));
          dummyNode.setCoord(ydir, forkCoord);
          dummyNode = link.addDummyNodeAtGraph(segment, graph);
          dummyNode.setCoord(xdir, ((reversed ? x0 : x1)));
          dummyNode.setCoord(ydir, forkCoord);
        }
      }
    }
  },

  getSegments : function(level, fromSide)
  {
    // summary:
    //		Returns the fork relevant segments.
    // tags:
    //		private
    
    if (this._recursiveMode)
    {
      if (fromSide) {
        return level.getSegmentsFrom(true); // ibm_ilog.graphlayout.hierarchical.HSegmentIterator
      }
      else {
        return level.getSegmentsTo(true); // ibm_ilog.graphlayout.hierarchical.HSegmentIterator
      }
    }
    else
    {
      if (fromSide) {
        return level.getNormalSegmentsFrom(); // ibm_ilog.graphlayout.hierarchical.HSegmentIterator
      }
      else {
        return level.getNormalSegmentsTo(); // ibm_ilog.graphlayout.hierarchical.HSegmentIterator
      }
    }
  }

}
);

lang.mixin(ForkPointAlgorithm,
{
}
);

ForkPointAlgorithm.ForkData = declare( null, 
{

  constructor : function()
  {
    
  },
  segments: null,
  forkCoordinate: null,
  leftLimit: null,
  rightLimit: null,
  leftUsed: null,
  rightUsed: null,
  delta: null

}
);
ForkPointAlgorithm.ForkDummyData = declare( null, 
{
  valid: null,
  fromForkSide: null,
  oldCoord: null,
  link: null,

  constructor : function(seg, fromSide)
  {
    this.valid = false;
    this.fromForkSide = (seg.isReversed() ? !fromSide : fromSide);
    this.oldCoord = 0;
    this.link = seg.getOwnerLink();
  }
  
}
);

return ForkPointAlgorithm;
});
