/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HGraphAlgorithm",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HGraphAlgorithm,	// #5
    GraphModelUtil,	// #6
    TranslateUtil	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var HLevelSweepAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.HLevelSweepAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		HLevelSweepAlgorithm traverses the levels forwards and backwards to
  //		perform a certain task.  
  // tags:
  //		private
  
  constructor : function()
  {
    
  },

  init : function(graph)
  {
    // summary:
    //		Initializes the algorithm with the hierarchical graph.
    // tags:
    //		private
    
    this.inherited("init", arguments, [graph]);
  },

  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  initForwardFirstLevel : function(firstLevel)
  {
    // summary:
    //		Initializes the first level (with level number 0) in a forward
    //		sweep.
    // tags:
    //		private
    
  },

  initBackwardFirstLevel : function(firstLevel)
  {
    // summary:
    //		Initializes the first level (with level number n) in a backward
    //		sweep.
    // tags:
    //		private
    
  },

  treatForwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number 1 ... 
    // description:
    //		Treats a level (with level number 1 ... 
    //		n) in a forward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i-1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    },
  
  treatBackwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number n-1 ... 
    // description:
    //		Treats a level (with level number n-1 ... 
    //		0) in a backward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i+1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    },
  
  sweepLevelForward : function(level)
  {
    // summary:
    //		Sweep forward for all subgraphs contained in the input level.  
    // tags:
    //		private
    
    if (level == null) {
      return;
    }
    var iter = level.getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getRepresentationGraph() != null) {
        this.sweepGraphForward(node.getRepresentationGraph());
      }
    }
  },

  sweepLevelBackward : function(level)
  {
    // summary:
    //		Sweep backward for all subgraphs contained in the input level.  
    // tags:
    //		private
    
    if (level == null) {
      return;
    }
    var iter = level.getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getRepresentationGraph() != null) {
        this.sweepGraphBackward(node.getRepresentationGraph());
      }
    }
  },

  sweepGraphForward : function(graph)
  {
    // summary:
    //		Sweep forward of a subgraph.  
    // tags:
    //		private
    
    var oldGraph = this.getGraph();
    this.init(graph);
    this.sweepForward();
    this.init(oldGraph);
  },

  sweepGraphBackward : function(graph)
  {
    // summary:
    //		Sweep backward of a subgraph.  
    // tags:
    //		private
    
    var oldGraph = this.getGraph();
    this.init(graph);
    this.sweepBackward();
    this.init(oldGraph);
  },

  sweepForward : function()
  {
    // summary:
    //		Performs a sweep forwards over the levels.
    
    var graph = this.getGraph();

    if (graph.getNumberOfLevels() > 1)
    {
      var iter = graph.getLevels();
      var prevLevel;
      var level;

      prevLevel = iter.next();
      this.initForwardFirstLevel(prevLevel);

      while (iter.hasNext())
      {
        level = iter.next();
        this.treatForwardLevel(prevLevel, level);
        this.getPercController().addPoints(1);
        this.layoutStepPerformed();
        prevLevel = level;
      }
    }
  },

  sweepBackward : function()
  {
    // summary:
    //		Performs a sweep backwards over the levels.
    // tags:
    //		private
    
    var graph = this.getGraph();

    if (graph.getNumberOfLevels() > 1)
    {
      var iter = graph.getLevelsBackward();
      var prevLevel;
      var level;

      prevLevel = iter.prev();
      this.initBackwardFirstLevel(prevLevel);

      while (iter.hasPrev())
      {
        level = iter.prev();
        this.treatBackwardLevel(prevLevel, level);
        this.getPercController().addPoints(1);
        this.layoutStepPerformed();
        prevLevel = level;
      }
    }
  }

}
);

return HLevelSweepAlgorithm;
});
