/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HierarchicalConstraint",	// #5
  "../hierarchical/HierarchicalNodeGroup",	// #6
  "../internalutil/GraphModelUtil",	// #7
  "../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HierarchicalConstraint,	// #5
    HierarchicalNodeGroup,	// #6
    GraphModelUtil,	// #7
    TranslateUtil	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HierarchicalConstraint = ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint;
=====*/

var HierarchicalLevelRangeConstraint = declare("ibm_ilog.graphlayout.hierarchical.HierarchicalLevelRangeConstraint", HierarchicalConstraint, 
{
  // summary:
  //		The class 'LevelRangeConstraint'  is a constraint that
  //		forces the node to be placed at a level with index  within the
  //		specified range. 
  // description:
  //		The class <code>LevelRangeConstraint</code>  is a constraint that
  //		forces the node to be placed at a level with index  within the
  //		specified range. 
  //		<p>
  //		Nodes are organized in horizontal or vertical levels numbered from 0
  //		to "n". If the link flow direction is from top to bottom, the nodes
  //		with level index 0 are placed in the topmost level, and the nodes
  //		with higher level indexes are placed in the levels below. If the link
  //		flow direction is from left to right, the nodes with level index 0
  //		are placed in the leftmost level, and the nodes with higher level
  //		indexes are placed in the levels farther to the right.  
  //		</p>
  //		<p>
  //		For instance, a level range constraint with minimum index 5 and
  //		maximum index 7 causes the node to be placed at level with index 5,
  //		6, or 7. A level range with minimum index 6 and maximum index 6
  //		causes the node to be placed exactly at level 6. This has the same
  //		effect as specifying the node level index 6 by <code>
  //		HierarchicalLayout.setSpecNodeLevelIndex()</code>.   
  //		</p>
  //		<p>
  //		The constraint is evaluated, if the incremental mode is disabled. If
  //		the incremental mode is enabled, the constraint is only evaluated if
  //		the constraint node is marked for incremental recalculation, because
  //		otherwise, the incremental positions of the nodes are dominant.  
  //		</p>
  //		<p>
  //		This kind of constraint is dominated by same level constraints and
  //		group spread constraints, that is, two nodes cannot have different
  //		level ranges while being forced to the same level. The constraint has
  //		higher priority than any relative level constraints.  
  //		</p>
  //		<p>
  //		The automatic conflict resolution can handle conflicting constraints.
  //		However, to speedup the layout, it is recommended to specify
  //		constraints in a way such that there are no conflicts.   
  //		</p>
  //		<p>
  //		Note that empty levels are invisible after layout. For instance a
  //		node with level range constraint to be placed at level 2 may appear
  //		to be placed in the topmost level, if level 0 and 1
  //		don't contain any other nodes.  
  //		</p>
  //		<p>
  //		In recursive layout mode (see <code> HierarchicalLayout.setRecursiveLayoutMode()</code>), all nodes of the group of a level spread constraint must belong to the same subgraph, and the constraint must be installed at the layout instance that is attached to this subgraph. It is not possible to have a level spread constraint containing a group of nodes of different subgraphs.   
  //		</p>
  
  _nodeOrGroup: null,
  _minLevel: null,
  _maxLevel: null,

  constructor : function (a0, a1, a2)
  {
    // summary:
    //		Creates a new 'LevelRangeConstraint' . 
    // description:
    //		Creates a new <code>LevelRangeConstraint</code> . 
    //		<p>
    //		The following examples show how to use the constraint:   
    //		</p>
    //		<pre> layout.addConstraint(new LevelRangeConstraint(node1, 5, 7));
    //		layout.addConstraint(new LevelRangeConstraint(node2,
    //		                            5, HierarchicalLayout.UNSPECIFIED));
    //		layout.addConstraint(new LevelRangeConstraint(node3,
    //		                            HierarchicalLayout.UNSPECIFIED, 7)); </pre>
    //		This forces <tt>node1</tt> to be in level 5, 6, or 7, and
    //		<tt>node2</tt>  to be in a level with index greater or equal to 5,
    //		and <tt>node3</tt> to be in a level with index smaller or equal to
    //		7.  
    //		<p>
    //		  
    //		</p>
    // nodeOrGroup:
    //		A node, or a group of type ' HierarchicalNodeGroup', that
    //		should be placed in the specified level range.
    // minLevel:
    //		The lower bound of the level range, or '
    //		HierarchicalLayout.UNSPECIFIED' if no lower bound is needed.
    // maxLevel:
    //		The upper bound of the level range, or '
    //		HierarchicalLayout.UNSPECIFIED' if no upper bound is needed.
    
    if(arguments.length == 3)
      this._constructor_overload_HierarchicalLevelRangeConstraint_3(a0, a1, a2);
    else if(arguments.length == 1)
      this._constructor_overload_HierarchicalLevelRangeConstraint_1(a0);
  },
  
  _constructor_overload_HierarchicalLevelRangeConstraint_3 : function(nodeOrGroup , minLevel , maxLevel )
  {
    this.setPriority(0.9 * Number.MAX_VALUE);
    this._nodeOrGroup = nodeOrGroup;
    this.setLevelRange(minLevel, maxLevel);
  },
  
  _constructor_overload_HierarchicalLevelRangeConstraint_1 : function(source)
  {
    this._nodeOrGroup = source._nodeOrGroup;
    this._minLevel = source._minLevel;
    this._maxLevel = source._maxLevel;
  },
  
  copy : function()
  {
    // summary:
    //		Copies the constraint. 
    // description:
    //		Copies the constraint. 
    //		<p>
    //		</p>
    
    return new HierarchicalLevelRangeConstraint(this); // ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint
  },

  getSubject : function()
  {
    // summary:
    //		Returns the subject of this constraint. 
    // description:
    //		Returns the subject of this constraint. 
    //		This is either the constraint node, or the instance of <code>
    //		HierarchicalNodeGroup</code> that describes a group of nodes.
    
    return this._nodeOrGroup; // Object
  },

  getMinLevel : function()
  {
    // summary:
    //		Returns the minimum level index suitable for the subject of this
    //		constraint (that is, for the node or the group of nodes). 
    // description:
    //		Returns the minimum level index suitable for the subject of this
    //		constraint (that is, for the node or the group of nodes). 
    //		<p>
    //		 
    //		</p>
    
    return this._minLevel; // int
  },

  setMinLevel : function(minLevel)
  {
    // summary:
    //		Sets the minimum level index suitable for the subject of this
    //		constraint, that is, for the node or the group of nodes. 
    // description:
    //		Sets the minimum level index suitable for the subject of this
    //		constraint, that is, for the node or the group of nodes. 
    //		
    // minLevel: int
    //		The minimum level index suitable for the subject of this
    //		constraint.
    
    if (minLevel < 0) {
      this._minLevel = -1;
    }
    else
    {
      this._minLevel = minLevel;
      if (this._maxLevel >= 0) {
        this._maxLevel = Math.max(this._maxLevel, minLevel);
      }
    }
  },

  getMaxLevel : function()
  {
    // summary:
    //		Returns the maximum level index suitable for the subject of this
    //		constraint (that is, for the node or the group of nodes). 
    // description:
    //		Returns the maximum level index suitable for the subject of this
    //		constraint (that is, for the node or the group of nodes). 
    //		<p>
    //		 
    //		</p>
    
    return this._maxLevel; // int
  },

  setMaxLevel : function(maxLevel)
  {
    // summary:
    //		Sets the maximum level index suitable for the subject of this
    //		constraint, that is, for the node or the group of nodes. 
    // description:
    //		Sets the maximum level index suitable for the subject of this
    //		constraint, that is, for the node or the group of nodes. 
    //		
    // maxLevel: int
    //		The maximum level index suitable for the subject of this
    //		constraint.
    
    if (maxLevel < 0) {
      this._maxLevel = -1;
    }
    else
    {
      this._maxLevel = maxLevel;
      if (this._minLevel >= 0) {
        this._minLevel = Math.min(this._minLevel, maxLevel);
      }
    }
  },

  setLevelRange : function(minLevel, maxLevel)
  {
    // summary:
    //		Sets the level range suitable for the subject of this constraint
    //		(that is, for the node or the group of nodes). 
    // description:
    //		Sets the level range suitable for the subject of this constraint
    //		(that is, for the node or the group of nodes). 
    //		Examples:  
    //		<pre> 
    //		constraint1.setLevelRange(5, 7);
    //		constraint2.setLevelRange(5, HierarchicalLayout.UNSPECIFIED);
    //		constraint3.setLevelRange(HierarchicalLayout.UNSPECIFIED, 7); 
    //		</pre>
    //		The first statement forces the node or node group to be level 5, 6,
    //		or 7. The second statement forces the node or node group to be in a
    //		level with index greater or equal to 5. The third statement forces
    //		the node or node group to be in a level with index smaller or equal
    //		to 7.  
    //		<p>
    //		 
    //		</p>
    // minLevel: int
    //		The lower bound of the level range, or '
    //		HierarchicalLayout.UNSPECIFIED' if no lower bound is needed.
    // maxLevel: int
    //		The upper bound of the level range, or '
    //		HierarchicalLayout.UNSPECIFIED' if no upper bound is needed.
    
    if (maxLevel < 0)
    {
      this._minLevel = minLevel;
      this._maxLevel = -1;
    }
    else if (minLevel <= maxLevel)
    {
      this._minLevel = minLevel;
      this._maxLevel = maxLevel;
    }
    else
    {
      this._minLevel = maxLevel;
      this._maxLevel = minLevel;
    }
  },

  getFirstSubject : function()
  {
    // summary:
    //		Returns the "first" subject of the constraint. 
    // description:
    //		Returns the "first" subject of the constraint. 
    //		This means simply we can set it via <code>
    //		HierarchicalLevelRangeConstraint.setFirstSubject()</code>.
    // tags:
    //		private
    
    return this._nodeOrGroup; // Object
  },

  setFirstSubject : function(subject)
  {
    // summary:
    //		Sets the first subject of the constraint.
    // tags:
    //		private
    
    this._nodeOrGroup = subject;
  },

  validate : function(model)
  {
    // summary:
    //		Returns <tt>true</tt> if the constraint is valid in the input graph
    //		model.
    // tags:
    //		private
    
    return this.validateNodeOrGroup(model, this.getSubject()); // Boolean
  },

  validateForLayout : function(graph)
  {
    // summary:
    //		Sets the valid flag of the constraint during layout. 
    // description:
    //		Sets the valid flag of the constraint during layout. 
    //		The purpose is the same as validate, but this is used temporarily
    //		during layout while <code>
    //		HierarchicalLevelRangeConstraint.validate()</code> is used to remove
    //		invalid constraints completely.
    // tags:
    //		private
    
    this.setValidForLayout(true);
    this.validateNodeOrGroupForLayout(graph, this.getSubject());
  },

  actAfterAdd : function(manager)
  {
    // summary:
    //		Acts after the constraint was added to the constraint manager.
    // tags:
    //		private
    
    var subject = this.getSubject();
    if (subject instanceof HierarchicalNodeGroup) {
      manager.addGroup(subject);
    }
  }

}
);

lang.mixin(HierarchicalLevelRangeConstraint,
{
  serialVersionUID : null,
  _fields_init : function(c)
  {
    c = c || HierarchicalLevelRangeConstraint;
    if(HierarchicalConstraint._fields_init)
      HierarchicalConstraint._fields_init(c);
    
    c.serialVersionUID = 7247930267832682855;
  }
  
}
);

(function(){ HierarchicalLevelRangeConstraint._fields_init(); })();

return HierarchicalLevelRangeConstraint;
});
