/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HierarchicalConstraint",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HierarchicalConstraint,	// #5
    GraphModelUtil,	// #6
    TranslateUtil	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HierarchicalConstraint = ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint;
=====*/

var HierarchicalSameLevelConstraint = declare("ibm_ilog.graphlayout.hierarchical.HierarchicalSameLevelConstraint", HierarchicalConstraint, 
{
  // summary:
  //		The class 'SameLevelConstraint'  is a constraint that
  //		forces two nodes to be placed at the same level. 
  // description:
  //		The class <code>SameLevelConstraint</code>  is a constraint that
  //		forces two nodes to be placed at the same level. 
  //		<p>
  //		The constraint is evaluated, if the incremental mode is disabled. If
  //		the incremental mode is enabled, the constraint is only evaluated if
  //		at least one of the nodes is marked for incremental recalculation,
  //		because otherwise, the incremental positions of the nodes are
  //		dominant.  
  //		</p>
  //		<p>
  //		Among the constraints, this kind has the highest priority. If several
  //		conflicting constraints are specified (such as the node must be
  //		placed in a lower level than another node <i>and</i> at the same
  //		level as this other node), the constraints with lower priority are
  //		ignored during layout.  
  //		</p>
  //		<p>
  //		The automatic conflict resolution can handle conflicting constraints.
  //		However, to speedup the layout, it is recommended to specify
  //		constraints in a way such that there are no conflicts.   
  //		</p>
  //		<p>
  //		The same level constraint has the same priority as the group spread
  //		constraint, because both cannot be conflicting. In fact, there are
  //		two ways of forcing two nodes to the same level: use the same level
  //		constraint, or use a group spread constraint of a group of two nodes
  //		with spread size 0.  
  //		</p>
  //		<p>
  //		In recursive layout mode (see <code>
  //		HierarchicalLayout.setRecursiveLayoutMode()</code>), both nodes of
  //		the same level constraint must belong to the same subgraph, and the
  //		constraint must be installed at the layout instance that is attached
  //		to this subgraph. It is not possible to have a same level constraint
  //		containing nodes of different subgraphs.   
  //		</p>
  
  _node1: null,
  _node2: null,

  constructor : function (a0, a1)
  {
    // summary:
    //		Creates a new 'SameLevelConstraint' . 
    // description:
    //		Creates a new <code>SameLevelConstraint</code> . 
    //		<p>
    //		The following example shows how to use the constraint:   
    //		</p>
    //		<pre> layout.addConstraint(new SameLevelConstraint(node1, node2)); </pre>
    //		This forces <tt>node1</tt> to be placed at the same level as
    //		<tt>node2</tt>.   
    //		<p>
    //		  
    //		</p>
    // node1:
    //		The one node.
    // node2:
    //		The other node.
    
    if(arguments.length == 2)
      this._constructor_overload_HierarchicalSameLevelConstraint_2(a0, a1);
    else if(arguments.length == 1)
      this._constructor_overload_HierarchicalSameLevelConstraint_1(a0);
  },
  
  _constructor_overload_HierarchicalSameLevelConstraint_2 : function(node1 , node2 )
  {
    this.setPriority(Number.MAX_VALUE);
    this._node1 = node1;
    this._node2 = node2;
  },
  
  _constructor_overload_HierarchicalSameLevelConstraint_1 : function(source)
  {
    this._node1 = source._node1;
    this._node2 = source._node2;
  },
  
  copy : function()
  {
    // summary:
    //		Copies the constraint. 
    // description:
    //		Copies the constraint. 
    //		<p>
    //		</p>
    
    return new HierarchicalSameLevelConstraint(this); // ibm_ilog.graphlayout.hierarchical.HierarchicalConstraint
  },

  getFirstNode : function()
  {
    // summary:
    //		Returns the first node of this constraint.
    
    return this._node1; // Object
  },

  getSecondNode : function()
  {
    // summary:
    //		Returns the second node of this constraint.
    
    return this._node2; // Object
  },

  getFirstSubject : function()
  {
    // summary:
    //		Returns the "first" subject of the constraint. 
    // description:
    //		Returns the "first" subject of the constraint. 
    //		This means simply we can set it via <code>
    //		HierarchicalSameLevelConstraint.setFirstSubject()</code>.
    // tags:
    //		private
    
    return this._node1; // Object
  },

  setFirstSubject : function(subject)
  {
    // summary:
    //		Sets the first subject of the constraint.
    // tags:
    //		private
    
    this._node1 = subject;
  },

  getSecondSubject : function()
  {
    // summary:
    //		Returns the "second" subject of the constraint. 
    // description:
    //		Returns the "second" subject of the constraint. 
    //		This means simply we can set it via <code>
    //		HierarchicalSameLevelConstraint.setSecondSubject()</code>.
    // tags:
    //		private
    
    return this._node2; // Object
  },

  setSecondSubject : function(subject)
  {
    // summary:
    //		Sets the first subject of the constraint.
    // tags:
    //		private
    
    this._node2 = subject;
  },

  validate : function(model)
  {
    // summary:
    //		Returns <tt>true</tt> if the constraint is valid in the input graph
    //		model.
    // tags:
    //		private
    
    if (!this.validateNode(model, this.getFirstNode())) {
      return false; // Boolean
    }
    if (!this.validateNode(model, this.getSecondNode())) {
      return false; // Boolean
    }
    return true; // Boolean
  },

  validateForLayout : function(graph)
  {
    // summary:
    //		Sets the valid flag of the constraint during layout. 
    // description:
    //		Sets the valid flag of the constraint during layout. 
    //		The purpose is the same as validate, but this is used temporarily
    //		during layout while <code>
    //		HierarchicalSameLevelConstraint.validate()</code> is used to remove
    //		invalid constraints completely.
    // tags:
    //		private
    
    this.setValidForLayout(true);
    this.validateNodeForLayout(graph, this.getFirstNode());
    this.validateNodeForLayout(graph, this.getSecondNode());
  },

  actAfterAdd : function(manager)
  {
    // summary:
    //		Acts after the constraint was added to the constraint manager.
    // tags:
    //		private
    
  }

}
);

lang.mixin(HierarchicalSameLevelConstraint,
{
  serialVersionUID : null,
  _fields_init : function(c)
  {
    c = c || HierarchicalSameLevelConstraint;
    if(HierarchicalConstraint._fields_init)
      HierarchicalConstraint._fields_init(c);
    
    c.serialVersionUID = 6689456825437964877;
  }
  
}
);

(function(){ HierarchicalSameLevelConstraint._fields_init(); })();

return HierarchicalSameLevelConstraint;
});
