/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HGraphAlgorithm",	// #5
  "../hierarchical/HNode",	// #6
  "../internalutil/genericquadtree/GenericApplyObject",	// #7
  "../internalutil/genericquadtree/GenericIndexedSet",	// #8
  "../internalutil/genericquadtree/IGenericQuadtreeObject",	// #9
  "../internalutil/GraphModelUtil",	// #10
  "../internalutil/HashTable",	// #11
  "../internalutil/LinkStyles",	// #12
  "../internalutil/LogResUtil",	// #13
  "../internalutil/Point",	// #14
  "../internalutil/Rectangle",	// #15
  "../internalutil/TranslateUtil"	// #16
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HGraphAlgorithm,	// #5
    HNode,	// #6
    GenericApplyObject,	// #7
    GenericIndexedSet,	// #8
    IGenericQuadtreeObject,	// #9
    GraphModelUtil,	// #10
    HashTable,	// #11
    LinkStyles,	// #12
    LogResUtil,	// #13
    Point,	// #14
    Rectangle,	// #15
    TranslateUtil	// #16
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var LinkSegmentOptim = declare("ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim", HGraphAlgorithm, 
{
  // summary:
  //		Optimization of link segments. 
  // description:
  //		Optimization of link segments. 
  //		Shifts orthogonal segments in level flow direction to avoid bends.
  //		This algorithm works with nested graphs. It can only be done after
  //		all nested graphs are handled, for two reasons: 1) It required
  //		coordinates without shift
  //		delta's from subgraphs 2) It cannot handle partially routed intergraph links that would occur during a layout of a subgraph.  
  // tags:
  //		private
  
  _dir: null,
  _minDist : null,
  _quadtree: null,
  _linksToSegments: null,
  _eliminateStairCases: null,
  _eliminateBends: null,

  constructor : function(graph)
  {
    this._minDist = 10;
    this._oifc = null;
    this.init(graph);
    this._dir = graph.getLevelFlow();
    this._eliminateStairCases = true;
    this._eliminateBends = true;
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  setParameters : function(eliminateStairCases, eliminateBends)
  {
    // summary:
    //		Transfers the parameters of the algorithm.
    // tags:
    //		private
    
    this._eliminateStairCases = eliminateStairCases;
    this._eliminateBends = eliminateBends;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();

    var glstyle = graph.getLayout().getGlobalLinkStyle();
    if (glstyle == LinkStyles.NO_RESHAPE_STYLE || glstyle == LinkStyles.STRAIGHT_LINE_STYLE) {
      return;
    }
    if (glstyle == LinkStyles.POLYLINE_STYLE) {
      this._eliminateStairCases = false;
    }
    if (!this._eliminateStairCases && !this._eliminateBends) {
      return;
    }
    graph.setOffsetForMinDist(this._dir, 0);
    this._quadtree = new GenericIndexedSet(20, 20, this.getGenericQuadtreeObjectInterface());
    this._linksToSegments = new HashTable();
    this.registerInQuadTree(graph);
    this.eliminateUselessSegments();
    var stairCases = this.getStairCases();

    if (this._eliminateStairCases)
    {
      for (var i = 0; i < 4; i++)
      {
        if (!this.eliminateAllStaircases(stairCases)) {
          break;
        }
      }
    }

    if (this._eliminateBends)
    {
      for (var j = 0; j < 4; j++)
      {
        if (!this.eliminateNonorthogonalBend()) {
          break;
        }
      }
    }
  },

  registerInQuadTree : function(graph)
  {
    // summary:
    //		Register all nodes and links in the quadtree.
    // tags:
    //		private
    
    var niter = graph.getNodes();

    while (niter.hasNext())
    {
      var node = niter.next();

      if (!node.isDummyNode() && !node.isEastWestPortAuxNode())
      {
        this._quadtree.addObject(node);
      }
    }

    var liter = graph.getLinks();

    while (liter.hasNext())
    {
      this.insertLink(liter.next());
    }

    liter = graph.getIntergraphLinks();

    while (liter.hasNext())
    {
      this.insertLink(liter.next());
    }

    this.markExtremaFixed(graph);
    var giter = graph.getSubgraphs();

    while (giter.hasNext())
      this.registerInQuadTree(giter.next());
  },

  markExtremaFixed : function(graph)
  {
    // summary:
    //		Marks the extreme points of the links as fixed. 
    // description:
    //		Marks the extreme points of the links as fixed. 
    //		If we would not do that, the subgraphs could shift because they 
    //		could become smaller due to link optimizations.
    // tags:
    //		private
    
    var minX = /*Number.MAX_VALUE*/(3.4028235E38);
    var minY = /*Number.MAX_VALUE*/(3.4028235E38);
    var maxX = -/*Number.MAX_VALUE*/(3.4028235E38);
    var maxY = -/*Number.MAX_VALUE*/(3.4028235E38);
    var x;
    var y;
    var ht;
    var niter = graph.getNodes();

    while (niter.hasNext())
    {
      var node = niter.next();
      if (node.isDummyNode()) {
        continue;
      }
      if (node.isEastWestPortAuxNode()) {
        continue;
      }
      x = node.getCoord(this._dir);
      y = node.getCoord(1 - this._dir);
      if (x < minX) {
        minX = x;
      }
      if (x > maxX) {
        maxX = x;
      }
      if (y < minY) {
        minY = y;
      }
      if (y > maxY) {
        maxY = y;
      }
      x += node.getSize(this._dir);
      y += node.getSize(1 - this._dir);
      if (x < minX) {
        minX = x;
      }
      if (x > maxX) {
        maxX = x;
      }
      if (y < minY) {
        minY = y;
      }
      if (y > maxY) {
        maxY = y;
      }
    }

    var liter = graph.getLinks();
    var link;
    var seg;

    while (liter.hasNext())
    {
      link = liter.next();
      seg = this.getSegments(link);

      while (seg != null)
      {
        ht = 0.5 * seg.getThickness();
        x = seg.getStart().x;
        y = seg.getStart().y;
        if (x - ht < minX) {
          minX = x - ht;
        }
        if (x + ht > maxX) {
          maxX = x + ht;
        }
        if (y - ht < minY) {
          minY = y - ht;
        }
        if (y + ht > maxY) {
          maxY = y + ht;
        }
        x = seg.getEnd().x;
        y = seg.getEnd().y;
        if (x - ht < minX) {
          minX = x - ht;
        }
        if (x + ht > maxX) {
          maxX = x + ht;
        }
        if (y - ht < minY) {
          minY = y - ht;
        }
        if (y + ht > maxY) {
          maxY = y + ht;
        }
        seg = seg.getNext();
      }
    }

    liter = graph.getIntergraphLinks();

    while (liter.hasNext())
    {
      link = liter.next();
      seg = this.getSegments(link);

      while (seg != null)
      {
        ht = 0.5 * seg.getThickness();
        x = seg.getStart().x;
        y = seg.getStart().y;
        if (x - ht < minX) {
          minX = x - ht;
        }
        if (x + ht > maxX) {
          maxX = x + ht;
        }
        if (y - ht < minY) {
          minY = y - ht;
        }
        if (y + ht > maxY) {
          maxY = y + ht;
        }
        x = seg.getEnd().x;
        y = seg.getEnd().y;
        if (x - ht < minX) {
          minX = x - ht;
        }
        if (x + ht > maxX) {
          maxX = x + ht;
        }
        if (y - ht < minY) {
          minY = y - ht;
        }
        if (y + ht > maxY) {
          maxY = y + ht;
        }
        seg = seg.getNext();
      }
    }

    liter = graph.getLinks();

    while (liter.hasNext())
    {
      link = liter.next();
      seg = this.getSegments(link);

      while (seg != null)
      {
        ht = 0.5 * seg.getThickness();
        x = seg.getStart().x;
        y = seg.getStart().y;

        if (x - ht == minX || x + ht == maxX || y - ht == minY || y + ht == maxY)
        {
          seg.markStartFixed();
        }

        x = seg.getEnd().x;
        y = seg.getEnd().y;

        if (x - ht == minX || x + ht == maxX || y - ht == minY || y + ht == maxY)
        {
          seg.markEndFixed();
        }

        seg = seg.getNext();
      }
    }

    liter = graph.getIntergraphLinks();

    while (liter.hasNext())
    {
      link = liter.next();
      seg = this.getSegments(link);

      while (seg != null)
      {
        ht = 0.5 * seg.getThickness();
        x = seg.getStart().x;
        y = seg.getStart().y;

        if (x - ht == minX || x + ht == maxX || y - ht == minY || y + ht == maxY)
        {
          seg.markStartFixed();
        }

        x = seg.getEnd().x;
        y = seg.getEnd().y;

        if (x - ht == minX || x + ht == maxX || y - ht == minY || y + ht == maxY)
        {
          seg.markEndFixed();
        }

        seg = seg.getNext();
      }
    }
  },

  distSquareToSegment : function(px, py, sx, sy, ex, ey)
  {
    // summary:
    //		Returns the distance square of a point (px, py) to a segment.
    // tags:
    //		private
    
    var bestDistSquare = Number.MAX_VALUE;
    var distSquare;
    var dx;
    var dy;

    dx = px - sx;
    dy = py - sy;
    distSquare = dx * dx + dy * dy;
    if (distSquare < bestDistSquare) {
      bestDistSquare = distSquare;
    }
    dx = px - ex;
    dy = py - ey;
    distSquare = dx * dx + dy * dy;
    if (distSquare < bestDistSquare) {
      bestDistSquare = distSquare;
    }
    dx = ex - sx;
    dy = ey - sy;

    if (dx != 0 && Math.min(sx, ex) <= px && px <= Math.max(sx, ex))
    {
      var y = (px - sx) * dy / dx + sy;
      distSquare = (y - py) * (y - py);
      if (distSquare < bestDistSquare) {
        bestDistSquare = distSquare;
      }
    }

    if (dy != 0 && Math.min(sy, ey) <= py && py <= Math.max(sy, ey))
    {
      var x = (py - sy) * dx / dy + sx;
      distSquare = (x - px) * (x - px);
      if (distSquare < bestDistSquare) {
        bestDistSquare = distSquare;
      }
    }

    return bestDistSquare; // Number
  },

  eliminateUselessSegments : function (a0)
  {
    // summary:
    //		Eliminate all segments that are useless.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.eliminateUselessSegments_overload_0();
    else if(arguments.length == 1)
      return this.eliminateUselessSegments_overload_1(a0);
  },
  
  eliminateUselessSegments_overload_0 : function()
                                                  
  {
    var changed = false;
    var aux = false;

    var _for_each_array_1 = this._linksToSegments.getValueList();
    for (var _for_each_index_1 = 0; _for_each_index_1 < _for_each_array_1.length; _for_each_index_1++)
    {
      var segObj = _for_each_array_1[_for_each_index_1];
      var seg = segObj;
      aux = this.eliminateUselessSegments(seg);
      changed = aux || changed;
    }

    return changed; // Boolean
  },

  eliminateUselessSegments_overload_1 : function(seg)
  {
    // summary:
    //		Eliminate all segments that are useless.
    // tags:
    //		private
    
    var changed = false;
    var prev = seg;
    var next = prev.getNext();

    while (next != null)
    {
      if (Math.abs(prev.getStart().x - prev.getEnd().x) < 0.001 && Math.abs(next.getStart().x - next.getEnd().x) < 0.001)
      {
        this._quadtree.removeObject(prev);
        this._quadtree.removeObject(next);
        prev.deleteNext();
        this._quadtree.addObject(prev);
        next = prev.getNext();
        changed = true;
      }
      else if (Math.abs(prev.getStart().y - prev.getEnd().y) < 0.001 && Math.abs(next.getStart().y - next.getEnd().y) < 0.001)
      {
        this._quadtree.removeObject(prev);
        this._quadtree.removeObject(next);
        prev.deleteNext();
        this._quadtree.addObject(prev);
        next = prev.getNext();
        changed = true;
      }
      else
      {
        prev = next;
        next = prev.getNext();
      }
    }

    return changed; // Boolean
  },

  eliminateAllStaircases : function(staircases)
  {
    // summary:
    //		Eliminate staircase shapes.  
    // tags:
    //		private
    
    var changed = false;

    var _for_each_array_2 = staircases;
    for (var _for_each_index_2 = 0; _for_each_index_2 < _for_each_array_2.length; _for_each_index_2++)
    {
      var segObj = _for_each_array_2[_for_each_index_2];
      var seg = segObj;
      var counter = 0;

      while (this.eliminateStaircase(seg))
      {
        changed = true;
        counter++;
        if (counter == 10) {
          break;
        }
      }
    }

    return changed; // Boolean
  },

  getStairCases : function()
  {
    // summary:
    //		Returns the possible staircase segment chains.
    // tags:
    //		private
    
    var result = new Array();

    var _for_each_array_3 = this._linksToSegments.getValueList();
    for (var _for_each_index_3 = 0; _for_each_index_3 < _for_each_array_3.length; _for_each_index_3++)
    {
      var segObj = _for_each_array_3[_for_each_index_3];
      if (this.hasStaircase(segObj)) {
        result.push(segObj);
      }
    }

    return result; // Array
  },

  hasStaircase : function(seg)
  {
    // summary:
    //		Returns if the segment chain has a stair case.
    // tags:
    //		private
    
    var prev = seg;
    var middle = prev.getNext();
    if (middle == null) {
      return false; // Boolean
    }
    var next = middle.getNext();

    while (next != null)
    {
      var success = false;

      if (prev.isOrthogonal() && next.isOrthogonal())
      {
        if (!prev.isStartFixed() && !prev.isEndFixed())
        {
          return true; // Boolean
        }

        if (!success && !next.isStartFixed() && !next.isEndFixed())
        {
          return true; // Boolean
        }
      }

      prev = middle;
      middle = next;
      next = middle.getNext();
    }

    return false; // Boolean
  },

  eliminateStaircase : function(seg)
  {
    // summary:
    //		Eliminate all staircases in the segment chain.
    // tags:
    //		private
    
    var changed = false;
    var prev = seg;
    var middle = prev.getNext();
    if (middle == null) {
      return false; // Boolean
    }
    var next = middle.getNext();

    while (next != null)
    {
      var success = false;

      if (prev.isOrthogonal() && next.isOrthogonal())
      {
        if (!prev.isStartFixed() && !prev.isEndFixed())
        {
          success = this.tryMovePrevToNext(prev, middle, next);
        }

        if (!success && !next.isStartFixed() && !next.isEndFixed())
        {
          success = this.tryMoveNextToPrev(prev, middle, next);
        }
      }

      if (success)
      {
        middle = prev.getNext();
        next = ((middle == null ? null : middle.getNext()));
        changed = true;
      }
      else
      {
        prev = middle;
        middle = next;
        next = middle.getNext();
      }
    }

    return changed; // Boolean
  },

  tryMovePrevToNext : function(prev, middle, next)
  {
    // summary:
    //		Tries to move prev to next.
    // tags:
    //		private
    
    var x = next.getStart().x;

    if (this.allowSegAtPos(prev, x))
    {
      this._quadtree.removeObject(next);
      prev.setStart(x);
      prev.setEnd(x);
      prev.deleteNext();
      prev.deleteNext();
      this._quadtree.addObject(prev);

      if (prev.getPrev() != null)
      {
        this._quadtree.addObject(prev.getPrev());
      }

      return true; // Boolean
    }

    return false; // Boolean
  },

  tryMoveNextToPrev : function(prev, middle, next)
  {
    // summary:
    //		Tries to move next to prev.
    // tags:
    //		private
    
    var x = prev.getEnd().x;

    if (this.allowSegAtPos(next, x))
    {
      this._quadtree.removeObject(prev);
      next.setStart(x);
      next.setEnd(x);
      prev.deleteNext();
      prev.deleteNext();
      this._quadtree.addObject(prev);

      if (prev.getNext() != null)
      {
        this._quadtree.addObject(prev.getNext());
      }

      return true; // Boolean
    }

    return false; // Boolean
  },

  allowSegAtPos : function(seg, x)
  {
    // summary:
    //		Check whether the segment can be moved to the input position. 
    // description:
    //		Check whether the segment can be moved to the input position. 
    //		If it returns true, the seg and its previous segment and its next
    //		segment are outside the quadtree.
    // tags:
    //		private
    
    this._quadtree.removeObject(seg);
    var prev = seg.getPrev();
    var next = seg.getNext();
    if (prev != null) {
      this._quadtree.removeObject(prev);
    }
    if (next != null) {
      this._quadtree.removeObject(next);
    }
    var oldSX = seg.getStart().x;
    var oldEX = seg.getEnd().x;
    var link = seg.getLink();
    var oldSelfOverlap = 0;
    var oldCrossings = 0;
    oldCrossings += this.calcCrossings(seg);

    if (prev != null)
    {
      oldCrossings += this.calcCrossings(prev);
      oldSelfOverlap += this.getOverlapRange(prev, link.getFrom());
      oldSelfOverlap += this.getOverlapRange(prev, link.getTo());
    }

    if (next != null)
    {
      oldCrossings += this.calcCrossings(next);
      oldSelfOverlap += this.getOverlapRange(next, link.getFrom());
      oldSelfOverlap += this.getOverlapRange(next, link.getTo());
    }

    var prevHasOverlaps = false;
    var nextHasOverlaps = false;
    seg.setStart(x);
    seg.setEnd(x);
    var okay = !this.hasOverlap(seg);
    if (okay && prev != null) {
      okay = !this.hasOverlap(prev, link.getFrom(), link.getTo());
    }
    if (okay && next != null) {
      okay = !this.hasOverlap(next, link.getFrom(), link.getTo()) && okay;
    }
    if (okay)
    {
      var newSelfOverlap = 0;
      var newCrossings = 0;
      newCrossings += this.calcCrossings(seg);

      if (prev != null)
      {
        newCrossings += this.calcCrossings(prev);
        newSelfOverlap += this.getOverlapRange(prev, link.getFrom());
        newSelfOverlap += this.getOverlapRange(prev, link.getTo());
      }

      if (next != null)
      {
        newCrossings += this.calcCrossings(next);
        newSelfOverlap += this.getOverlapRange(next, link.getFrom());
        newSelfOverlap += this.getOverlapRange(next, link.getTo());
      }

      okay = (newCrossings <= oldCrossings) && (newSelfOverlap <= oldSelfOverlap);
    }

    if (!okay)
    {
      seg.setStart(oldSX);
      seg.setEnd(oldEX);
      this._quadtree.addObject(seg);
      if (seg.getPrev() != null) {
        this._quadtree.addObject(seg.getPrev());
      }
      if (seg.getNext() != null) {
        this._quadtree.addObject(seg.getNext());
      }
      return false; // Boolean
    }

    return true; // Boolean
  },

  eliminateNonorthogonalBend : function (a0)
  {
    // summary:
    //		Eliminate useless bends when the link is nonorthogonal.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.eliminateNonorthogonalBend_overload_0();
    else if(arguments.length == 1)
      return this.eliminateNonorthogonalBend_overload_1(a0);
  },
  
  eliminateNonorthogonalBend_overload_0 : function()
                                                    
  {
    var changed = false;

    var _for_each_array_4 = this._linksToSegments.getValueList();
    for (var _for_each_index_4 = 0; _for_each_index_4 < _for_each_array_4.length; _for_each_index_4++)
    {
      var segObj = _for_each_array_4[_for_each_index_4];
      var seg = segObj;
      if (seg.getLink().isOrthogonal()) {
        continue;
      }
      var counter = 0;

      while (this.eliminateNonorthogonalBend(seg))
      {
        changed = true;
        counter++;
        if (counter == 10) {
          break;
        }
      }
    }

    return changed; // Boolean
  },

  eliminateNonorthogonalBend_overload_1 : function(seg)
  {
    // summary:
    //		Eliminate all useless bends in the segment chain.
    // tags:
    //		private
    
    var changed = false;
    var prev = seg;
    var next = prev.getNext();

    while (next != null)
    {
      var success = false;

      if (!prev.isEndFixed() && !next.isStartFixed())
      {
        success = this.tryEliminateBend(prev, next);
      }

      if (success)
      {
        next = prev.getNext();
        changed = true;
      }
      else
      {
        prev = next;
        next = prev.getNext();
      }
    }

    return changed; // Boolean
  },

  tryEliminateBend : function(prev, next)
  {
    // summary:
    //		Tries to eliminate a bend.
    // tags:
    //		private
    
    this._quadtree.removeObject(prev);
    this._quadtree.removeObject(next);
    var link = prev.getLink();
    var oldCrossings = 0;
    oldCrossings += this.calcCrossings(prev);
    oldCrossings += this.calcCrossings(next);
    var oldSelfOverlap = 0;
    oldSelfOverlap += this.getOverlapRange(prev, link.getFrom());
    oldSelfOverlap += this.getOverlapRange(prev, link.getTo());
    oldSelfOverlap += this.getOverlapRange(next, link.getFrom());
    oldSelfOverlap += this.getOverlapRange(next, link.getTo());
    var oldx = prev.getEnd().x;
    var oldy = prev.getEnd().y;
    TranslateUtil.movePoint(prev.getEnd(), next.getEnd().x, next.getEnd().y);
    var newCrossings = this.calcCrossings(prev, next.getNext());
    var newSelfOverlap = 0;
    newSelfOverlap += this.getOverlapRange(prev, link.getFrom());
    newSelfOverlap += this.getOverlapRange(prev, link.getTo());

    if (newCrossings <= oldCrossings && newSelfOverlap <= 1.1 * oldSelfOverlap && !this.hasOverlap(prev, link.getFrom(), link.getTo()))
    {
      prev.deleteNext();
      this._quadtree.addObject(prev);
      return true; // Boolean
    }
    else
    {
      TranslateUtil.movePoint(prev.getEnd(), oldx, oldy);
      this._quadtree.addObject(prev);
      this._quadtree.addObject(next);
      return false; // Boolean
    }
  },

  hasOverlap : function(seg, ignore1 , ignore2 )
  {
    // summary:
    //		Checks whether the input segment overlaps any node that it is not
    //		allowed to overlap.
    // tags:
    //		private
    
    var r = seg.boundingBox();

    if (seg.isOrthogonal())
    {
      var delta = 0.5 * seg.getLink().getThickness();
      var dist1 = this.getGraph().getMinDistBetweenNodeAndLink(this._dir);
      var dist2 = this.getGraph().getMinDistBetweenLinks(this._dir);
      delta += Math.max(dist1, dist2);
      r.x -= delta;
      r.width += 2 * delta;
    }

    var list = this.getObjects(r);

    var _for_each_array_5 = list;
    for (var _for_each_index_5 = 0; _for_each_index_5 < _for_each_array_5.length; _for_each_index_5++)
    {
      var obj = _for_each_array_5[_for_each_index_5];
      var ignored = false;

      if (obj instanceof HNode)
      {
        var node = obj;
        if (node == ignore1) {
          ignored = true;
        }
        if (node == ignore2) {
          ignored = true;
        }
        if (ignore1 != null)
        {
          if (node.getEastPortAuxNode() == ignore1) {
            ignored = true;
          }
          if (node.getWestPortAuxNode() == ignore1) {
            ignored = true;
          }
        }

        if (ignore2 != null)
        {
          if (node.getEastPortAuxNode() == ignore2) {
            ignored = true;
          }
          if (node.getWestPortAuxNode() == ignore2) {
            ignored = true;
          }
        }

        if (!ignored && this.hasOverlapBetweenSegmentAndNode(seg, node))
        {
          return true; // Boolean
        }
      }
      else
      {
        var otherSeg = obj;
        var link = seg.getLink();

        if (seg.isIncident(link.getFrom()))
        {
          if (otherSeg.isIncident(link.getFrom())) {
            ignored = true;
          }
        }

        if (seg.isIncident(link.getTo()))
        {
          if (otherSeg.isIncident(link.getTo())) {
            ignored = true;
          }
        }

        if (!ignored && this.hasOverlapBetweenSegments(seg, otherSeg))
        {
          return true; // Boolean
        }
      }
    }

    return false; // Boolean
  },

  hasOverlapBetweenSegmentAndNode : function(seg, node)
  {
    // summary:
    //		Checks whether the segment overlaps the input node.  
    // tags:
    //		private
    
    if (seg.isIncident(node))
    {
      return false; // Boolean
    }

    var link = seg.getLink();

    if (this.isNestedInside(node, link.getFrom()))
    {
      return false; // Boolean
    }

    if (this.isNestedInside(node, link.getTo()))
    {
      return false; // Boolean
    }

    var p1 = seg.getStart();
    var p2 = seg.getEnd();
    var p3 = new Point();
    var p4 = new Point();
    var r = this.getGenericQuadtreeObjectInterface().boundingBox(node);

    if (seg.isOrthogonal())
    {
      var delta = 0.5 * seg.getLink().getThickness();
      delta += this.getGraph().getMinDistBetweenNodeAndLink(this._dir);

      if (delta > 0) {
        delta -= 0.5;
      }
      r.x -= delta;
      r.width += 2 * delta;
    }

    TranslateUtil.movePoint(p3, r.x, r.y);
    TranslateUtil.movePoint(p4, r.x + r.width, r.y);
    if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, false)) {
      return true; // Boolean
    }
    TranslateUtil.movePoint(p3, r.x + r.width, r.y + r.height);
    if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, false)) {
      return true; // Boolean
    }
    TranslateUtil.movePoint(p4, r.x, r.y + r.height);
    if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, false)) {
      return true; // Boolean
    }
    TranslateUtil.movePoint(p3, r.x, r.y);
    if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, false)) {
      return true; // Boolean
    }
    return false; // Boolean
  },

  hasOverlapBetweenSegments : function(seg1, seg2)
  {
    // summary:
    //		Checks whether two link segments overlaps.  
    // tags:
    //		private
    
    if (seg1.isOrthogonal() && seg2.isOrthogonal())
    {
      if (seg1.getPrev() == seg2) {
        return false; // Boolean
      }
      if (seg1.getNext() == seg2) {
        return false; // Boolean
      }
      if (seg1.getConnectingSegment(seg2) != null) {
        return false; // Boolean
      }
      var p1 = seg1.getStart();
      var p2 = seg1.getEnd();
      var p3 = new Point();
      var p4 = new Point();
      var delta = 0.5 * seg1.getLink().getThickness();
      delta += 0.5 * seg2.getLink().getThickness();
      delta += this.getGraph().getMinDistBetweenLinks(this._dir);

      if (delta > 0) {
        delta -= 0.5;
      }
      TranslateUtil.movePoint(p3, seg2.getStart().x - delta, seg2.getStart().y);
      TranslateUtil.movePoint(p4, seg2.getStart().x - delta, seg2.getEnd().y);
      if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, true)) {
        return true; // Boolean
      }
      TranslateUtil.movePoint(p3, seg2.getStart().x + delta, seg2.getEnd().y);
      if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, true)) {
        return true; // Boolean
      }
      TranslateUtil.movePoint(p4, seg2.getStart().x + delta, seg2.getStart().y);
      if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, true)) {
        return true; // Boolean
      }
      TranslateUtil.movePoint(p3, seg2.getStart().x - delta, seg2.getStart().y);
      if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, true)) {
        return true; // Boolean
      }
      return false; // Boolean
    }

    return false; // Boolean
  },

  getOverlapRange : function(seg, node)
  {
    // summary:
    //		Returns the overlap range between a node and a segment. 
    // description:
    //		Returns the overlap range between a node and a segment. 
    //		This is only used for end nodes of the link of the segment.
    // tags:
    //		private
    
    if (node.isEastWestPortAuxNode()) {
      node = node.getEastWestPortAuxOwner();
    }
    var p1 = seg.getStart();
    var p2 = seg.getEnd();
    var p3 = new Point();
    var p4 = new Point();
    var r = this.getGenericQuadtreeObjectInterface().boundingBox(node);
    var cp = TranslateUtil.newArray(4, null);
    var idx = 0;
    TranslateUtil.movePoint(p3, r.x, r.y);
    TranslateUtil.movePoint(p4, r.x + r.width, r.y);
    cp[idx] = LinkSegmentOptim.segmentsCrossPoint(p1, p2, p3, p4, true);
    if (cp[idx] != null) {
      idx++;
    }
    TranslateUtil.movePoint(p3, r.x + r.width, r.y + r.height);
    cp[idx] = LinkSegmentOptim.segmentsCrossPoint(p1, p2, p3, p4, true);
    if (cp[idx] != null) {
      idx++;
    }
    TranslateUtil.movePoint(p4, r.x, r.y + r.height);
    cp[idx] = LinkSegmentOptim.segmentsCrossPoint(p1, p2, p3, p4, true);
    if (cp[idx] != null) {
      idx++;
    }
    TranslateUtil.movePoint(p3, r.x, r.y);
    cp[idx] = LinkSegmentOptim.segmentsCrossPoint(p1, p2, p3, p4, true);
    if (cp[idx] != null) {
      idx++;
    }
    if (idx == 0) {
      return 0; // Number
    }
    if (idx == 1)
    {
      if (TranslateUtil.rectContains(r, p1.x, p1.y)) {
        cp[1] = p1;
      }
      else {
        cp[1] = p2;
      }
    }

    var dx = cp[0].x - cp[1].x;
    var dy = cp[0].y - cp[1].y;
    return Math.sqrt(dx * dx + dy * dy); // Number
  },

  isNestedInside : function(parent, child)
  {
    // summary:
    //		Checks whether the nodes are nested inside each other.
    // tags:
    //		private
    
    if (parent == child) {
      return false; // Boolean
    }
    while (child != parent && child != null)
    {
      child = child.getOwnerGraph().getRepresentationNode();
      if (child == parent) {
        return true; // Boolean
      }
    }

    return false; // Boolean
  },

  calcCrossings : function (a0, a1)
  {
    // summary:
    //		Calculates the number of crossing of the input segment with other
    //		segments.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.calcCrossings_overload_1(a0);
    else if(arguments.length == 2)
      return this.calcCrossings_overload_2(a0, a1);
  },
  
  calcCrossings_overload_1 : function(seg)
                                          
  {
    return this.calcCrossings(seg, null); // int
  },

  calcCrossings_overload_2 : function(seg, ignore

)
  {
    var r = seg.boundingBox();
    var list = this.getObjects(r);
    var count = 0;

    var _for_each_array_6 = list;
    for (var _for_each_index_6 = 0; _for_each_index_6 < _for_each_array_6.length; _for_each_index_6++)
    {
      var obj = _for_each_array_6[_for_each_index_6];
      if (obj instanceof LinkSegmentOptim.LinkSegment)
      {
        if (obj == ignore) {
          continue;
        }
        count += this.calcCrossingsImpl(seg, obj);
      }
    }

    return count; // int
  },

  calcCrossingsImpl : function(seg1, seg2)
  {
    // summary:
    //		Checks whether the two input segments cross.
    // tags:
    //		private
    
    if (seg1 == seg2) {
      return 0; // int
    }
    if (seg1.getPrev() == seg2) {
      return 0; // int
    }
    if (seg1.getNext() == seg2) {
      return 0; // int
    }
    var p1 = seg1.getStart();
    var p2 = seg1.getEnd();
    var p3 = seg2.getStart();
    var p4 = seg2.getEnd();
    if (LinkSegmentOptim.segmentsCross(p1, p2, p3, p4, false)) {
      return 1; // int
    }
    return 0; // int
  },

  getObjects : function(rect)
  {
    // summary:
    //		Returns all objects that touch the input rectangle.
    // tags:
    //		private
    
    var result = new Array();
    return this.addObjectsTo(rect, result); // Array
  },

  addObjectsTo : function(rect, result)
  {
    // summary:
    //		Adds all objects that touch the input rectangle to the input
    //		arraylist.  
    // tags:
    //		private
    
    this._quadtree.mapIntersects(rect, new LinkSegmentOptimAnonClass_1(), result);
    return result; // Array
  },

  insertLink : function(link)
  {
    // summary:
    //		Inserts a link into the quadtree.
    // tags:
    //		private
    
    link.optimize();
    var seg = this.getSegments(link);

    while (seg != null)
    {
      this._quadtree.removeObject(seg);
      seg = seg.getNext();
    }

    seg = this.createSegments(link);
    this._linksToSegments.add(link, seg);

    while (seg != null)
    {
      if (link.isFrozen())
      {
        seg.markStartFixed();
        seg.markEndFixed();
      }

      this._quadtree.addObject(seg);
      seg = seg.getNext();
    }
  },

  getSegments : function(link)
  {
    // summary:
    //		Returns the link segments of a link. 
    // description:
    //		Returns the link segments of a link. 
    //		Returns the start segment.
    // tags:
    //		private
    
    return this._linksToSegments.item(link); // ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim.LinkSegment
  },

  createSegments : function(link)
  {
    // summary:
    //		Create the link segments of a link. 
    // description:
    //		Create the link segments of a link. 
    //		Returns the start segment.
    // tags:
    //		private
    
    var start = null;
    var prev = null;
    var reversed = false;
    var s = link.getStartSegment();
    var d;

    if (s.getFrom().isDummyNode())
    {
      d = s.getFrom();
      reversed = true;
    }
    else if (s.getTo().isDummyNode())
    {
      d = s.getTo();
    }
    else
    {
      d = s.getTo();
    }

    var startSeg = s;

    while (d.isDummyNode())
    {
      if (!d.isInvalid())
      {
        prev = new LinkSegmentOptim.LinkSegment(this, startSeg, s, reversed, prev);
        if (start == null) {
          start = prev;
        }
        s = d.getIntergraphOpposite(s);
        startSeg = s;
      }
      else
      {
        s = d.getIntergraphOpposite(s);
      }

      d = s.getOpposite(d);
    }

    prev = new LinkSegmentOptim.LinkSegment(this, startSeg, s, reversed, prev);
    if (start == null) {
      start = prev;
    }
    return start; // ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim.LinkSegment
  },

  _oifc : null,
  getGenericQuadtreeObjectInterface : function()
  {
    if (this._oifc == null)
    {
      this._oifc = new LinkSegmentOptimAnonClass_2(this);
    }

    return this._oifc; // ibm_ilog.graphlayout.internalutil.genericquadtree.IGenericQuadtreeObject
  }

}
);

lang.mixin(LinkSegmentOptim,
{

  segmentsCross : function(p1, p2, p3, p4, excludeEnds)
  {
    return LinkSegmentOptim.segmentsCrossPoint(p1, p2, p3, p4, excludeEnds, false) != null; // Boolean
  },

  segmentsCrossPoint : function (a0, a1, a2, a3, a4, a5)
  {
    if(arguments.length == 5)
      return this.segmentsCrossPoint_overload_5(a0, a1, a2, a3, a4);
    else if(arguments.length == 6)
      return this.segmentsCrossPoint_overload_6(a0, a1, a2, a3, a4, a5);
  },
  
  segmentsCrossPoint_overload_5 : function(p1, p2, p3, p4, excludeEnds)
  {
    return LinkSegmentOptim.segmentsCrossPoint(p1, p2, p3, p4, excludeEnds, true); // Point
  },

  segmentsCrossPoint_overload_6 : function(p1, p2, p3, p4, excludeEnds, getCalcPoint

)
  {
    var dx1 = (p2.x - p1.x);
    var dy1 = (p2.y - p1.y);
    var dx2 = (p4.x - p3.x);
    var dy2 = (p4.y - p3.y);

    if (dx1 == 0.0 && dy1 == 0.0) {
      return null; // Point
    }
    if (dx2 == 0.0 && dy2 == 0.0) {
      return null; // Point
    }

    var dx1y2 = dx1 * dy2;
    var dx2y1 = dx2 * dy1;
    if (dx1y2 == dx2y1) {
      return null; // Point
    }

    var crossX = ((p1.x * dx2y1 - p3.x * dx1y2 + (p3.y - p1.y) * dx1 * dx2) / (dx2y1 - dx1y2));
    var crossY = ((p1.y * dx1y2 - p3.y * dx2y1 + (p3.x - p1.x) * dy1 * dy2) / (dx1y2 - dx2y1));

    if (Math.abs(dx1) > Math.abs(dy1))
    {
      if (crossX < Math.min(p1.x, p2.x) || Math.max(p1.x, p2.x) < crossX) {
        return null; // Point
      }
    }
    else
    {
      if (crossY < Math.min(p1.y, p2.y) || Math.max(p1.y, p2.y) < crossY) {
        return null; // Point
      }
    }

    if (Math.abs(dx2) > Math.abs(dy2))
    {
      if (crossX < Math.min(p3.x, p4.x) || Math.max(p3.x, p4.x) < crossX) {
        return null; // Point
      }
    }
    else
    {
      if (crossY < Math.min(p3.y, p4.y) || Math.max(p3.y, p4.y) < crossY) {
        return null; // Point
      }
    }

    if (excludeEnds)
    {
      if (crossX == p2.x && crossY == p2.y) {
        return null; // Point
      }
      if (crossX == p4.x && crossY == p4.y) {
        return null; // Point
      }
    }

    if (getCalcPoint) {
      return new Point(crossX, crossY); // Point
    }
    else {
      return p1; // Point
    }
  }

}
);

LinkSegmentOptim.LinkSegment = declare( null, 
{
  __outerThis: null,
  _link: null,
  _p0: null,
  _p1: null,
  _incident0: null,
  _incident1: null,
  _startSegment: null,
  _endSegment: null,
  _thickness: null,
  _startFixed: null,
  _endFixed: null,

  constructor : function(input__outerThis, s0, s1, rev, prev)
  {
    this.__outerThis = input__outerThis;
    this._link = s0.getOwnerLink();
    this._startSegment = s0;
    this._endSegment = s1;
    this._thickness = this._link.getRealThickness();
    this._incident1 = null;
    var p0Fixed;
    var p0;
    var p1Fixed;
    var p1;

    if (!s0.getFrom().isInvalid())
    {
      p0 = new Point(s0.getRealFromCoord(this.__outerThis._dir), s0.getRealFromCoord(1 - this.__outerThis._dir));
      p0Fixed = !s0.getFrom().isDummyNode();
    }
    else
    {
      p0 = new Point(s0.getRealToCoord(this.__outerThis._dir), s0.getRealToCoord(1 - this.__outerThis._dir));
      p0Fixed = !s0.getTo().isDummyNode();
      rev = false;
    }

    if (!s1.getTo().isInvalid())
    {
      p1 = new Point(s1.getRealToCoord(this.__outerThis._dir), s1.getRealToCoord(1 - this.__outerThis._dir));
      p1Fixed = !s1.getTo().isDummyNode();
    }
    else
    {
      p1 = new Point(s1.getRealFromCoord(this.__outerThis._dir), s1.getRealFromCoord(1 - this.__outerThis._dir));
      p1Fixed = !s1.getFrom().isDummyNode();
    }

    if (prev != null)
    {
      this._incident0 = prev;
      prev._incident1 = this;
      this._p0 = prev._p1;
      this._startFixed = false;

      if (TranslateUtil.pointEquals(this._p0, p0))
      {
        this._p1 = p1;
        this._endFixed = p1Fixed;
      }
      else
      {
        this._p1 = p0;
        this._endFixed = p0Fixed;
      }
    }
    else
    {
      if (rev)
      {
        this._p0 = p1;
        this._p1 = p0;
        this._startFixed = p1Fixed;
        this._endFixed = p0Fixed;
      }
      else
      {
        this._p0 = p0;
        this._p1 = p1;
        this._startFixed = p0Fixed;
        this._endFixed = p1Fixed;
      }
    }

    if (this._link.isOrthogonal())
    {
      if (Math.abs(this._p0.x - this._p1.x) < 0.001 && Math.abs(this._p0.y - this._p1.y) > 1)
      {
        this._p0.x = this._p1.x;
      }
    }
  },
  
  boundingBox : function()
  {
    // summary:
    //		Returns the bounding box of the link segment.
    // tags:
    //		private
    
    var rect = null;
    var minx = Math.min(this._p0.x, this._p1.x) - 0.5 * this._thickness;
    var maxx = Math.max(this._p0.x, this._p1.x) + 0.5 * this._thickness;
    var miny = Math.min(this._p0.y, this._p1.y) - 0.5 * this._thickness;
    var maxy = Math.max(this._p0.y, this._p1.y) + 0.5 * this._thickness;
    rect = new Rectangle(minx, miny, maxx - minx, maxy - miny);

    if (rect.width < 1)
    {
      rect.width = 1;
    }

    if (rect.height < 1)
    {
      rect.height = 1;
    }

    return rect; // Rectangle
  },

  getLink : function()
  {
    // summary:
    //		Returns the link that is represented by these segments.
    // tags:
    //		private
    
    return this._link; // ibm_ilog.graphlayout.hierarchical.HLink
  },

  isOrthogonal : function()
  {
    // summary:
    //		Returns whether the link segment is vertically orthogonal.
    // tags:
    //		private
    
    return this._p0.x == this._p1.x; // Boolean
  },

  isStartFixed : function()
  {
    // summary:
    //		Returns whether the link segment is fixed at the start point.
    // tags:
    //		private
    
    return this._startFixed; // Boolean
  },

  isEndFixed : function()
  {
    // summary:
    //		Returns whether the link segment is fixed at the end point.
    // tags:
    //		private
    
    return this._endFixed; // Boolean
  },

  markStartFixed : function()
  {
    // summary:
    //		Marks the segment start as fixed.
    // tags:
    //		private
    
    this._startFixed = true;
  },

  markEndFixed : function()
  {
    // summary:
    //		Marks the segment end as fixed.
    // tags:
    //		private
    
    this._endFixed = true;
  },

  getPrev : function()
  {
    // summary:
    //		Returns the previous segment. 
    // description:
    //		Returns the previous segment. 
    //		May return null.
    // tags:
    //		private
    
    return this._incident0; // ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim.LinkSegment
  },

  getNext : function()
  {
    // summary:
    //		Returns the next segment. 
    // description:
    //		Returns the next segment. 
    //		May return null.
    // tags:
    //		private
    
    return this._incident1; // ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim.LinkSegment
  },

  deleteNext : function()
  {
    // summary:
    //		Delete a segment.
    // tags:
    //		private
    
    var next = this.getNext();
    if (next == null) {
      return;
    }
    var dummy = this.getConnectingDummy(next);
    dummy.markInvalid();
    this._p1 = next._p1;
    this._incident1 = next.getNext();

    if (next.getNext() != null)
    {
      next.getNext()._incident0 = this;
      next.getNext()._p0 = this._p1;
    }

    this._endSegment = next._endSegment;
    this._endFixed = next._endFixed || this._endFixed;
  },

  getConnectingSegment : function(s)
  {
    // summary:
    //		Returns the intermdiate segment between two segments, if any. 
    // description:
    //		Returns the intermdiate segment between two segments, if any. 
    //		Returns null if both segments are not connected.
    // tags:
    //		private
    
    if (this.getNext() != null && this.getNext().getNext() == s) {
      return this.getNext(); // ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim.LinkSegment
    }
    if (this.getPrev() != null && this.getPrev().getPrev() == s) {
      return this.getPrev(); // ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim.LinkSegment
    }
    return null; // ibm_ilog.graphlayout.hierarchical.LinkSegmentOptim.LinkSegment
  },

  getConnectingDummy : function(s)
  {
    // summary:
    //		Returns the connecting dummy node between two segments.
    // tags:
    //		private
    
    if (this.getNext() == s)
    {
      if (this._endSegment.getTo().isDummyNode())
      {
        if (this._endSegment.getTo() == s._startSegment.getFrom()) {
          return this._endSegment.getTo(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
        if (this._endSegment.getTo() == s._startSegment.getTo()) {
          return this._endSegment.getTo(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
      }

      if (this._endSegment.getFrom().isDummyNode())
      {
        if (this._endSegment.getFrom() == s._startSegment.getFrom()) {
          return this._endSegment.getFrom(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
        if (this._endSegment.getFrom() == s._startSegment.getTo()) {
          return this._endSegment.getFrom(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
      }
    }

    if (this.getPrev() == s)
    {
      if (this._startSegment.getTo().isDummyNode())
      {
        if (this._startSegment.getTo() == s._endSegment.getFrom()) {
          return this._startSegment.getTo(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
        if (this._startSegment.getTo() == s._endSegment.getTo()) {
          return this._startSegment.getTo(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
      }

      if (this._startSegment.getFrom().isDummyNode())
      {
        if (this._startSegment.getFrom() == s._endSegment.getFrom()) {
          return this._startSegment.getFrom(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
        if (this._startSegment.getFrom() == s._endSegment.getTo()) {
          return this._startSegment.getFrom(); // ibm_ilog.graphlayout.hierarchical.HNode
        }
      }
    }

    return null; // ibm_ilog.graphlayout.hierarchical.HNode
  },

  isIncident : function(node)
  {
    // summary:
    //		Returns true if the input node is incident to this segment.
    // tags:
    //		private
    
    if (this._startSegment.getFrom() == node) {
      return true; // Boolean
    }
    if (this._startSegment.getTo() == node) {
      return true; // Boolean
    }
    if (this._endSegment.getFrom() == node) {
      return true; // Boolean
    }
    if (this._endSegment.getTo() == node) {
      return true; // Boolean
    }
    return false; // Boolean
  },

  getThickness : function()
  {
    // summary:
    //		Returns the thickness of the segment.
    // tags:
    //		private
    
    return this._thickness; // Number
  },

  setStart : function(x)
  {
    // summary:
    //		Sets the start point X coordinate.
    // tags:
    //		private
    
    if (this._startFixed) {
      return;
    }
    this._p0.x = x;
    if (!this._startSegment.getFrom().isInvalid()) {
      this._startSegment.getFromPoint()[this.__outerThis._dir] = x;
    }
    else {
      this._startSegment.getToPoint()[this.__outerThis._dir] = x;
    }
  },

  setEnd : function(x)
  {
    // summary:
    //		Sets the end point X coordinate.
    // tags:
    //		private
    
    if (this._endFixed) {
      return;
    }
    this._p1.x = x;
    if (!this._endSegment.getTo().isInvalid()) {
      this._endSegment.getToPoint()[this.__outerThis._dir] = x;
    }
    else {
      this._endSegment.getFromPoint()[this.__outerThis._dir] = x;
    }
  },

  getStart : function()
  {
    // summary:
    //		Returns the start point.
    // tags:
    //		private
    
    return this._p0; // Point
  },

  getEnd : function()
  {
    // summary:
    //		Returns the end point.
    // tags:
    //		private
    
    return this._p1; // Point
  }

}
);
/**
 * @private
 */
LinkSegmentOptimAnonClass_1 = declare(GenericApplyObject,
{
  // summary:
  //		For debugging: returns the length of the segment chain.
  // tags:
  //		private
  
  constructor : function()
  {
    
  },

  apply : function(obj, arg)
  {
    var r = arg;
    r.push(obj);
  }

});

/**
 * @private
 */
LinkSegmentOptimAnonClass_2 = declare(IGenericQuadtreeObject,
{
  // tags:
  //		private
  
  __outerThis : null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
  },

  boundingBox : function(object)
  {
    if (object instanceof HNode)
    {
      var node = object;
      var x;
      var y;
      var w;
      var h;

      if (node.getRepresentationGraph() != null)
      {
        x = node.getCoord(this.__outerThis._dir);
        y = node.getCoord(1 - this.__outerThis._dir);
        w = node.getSize(this.__outerThis._dir);
        h = node.getSize(1 - this.__outerThis._dir);
      }
      else
      {
        x = node.getCoord(this.__outerThis._dir) + node.getOffsetToDistTopLeft(this.__outerThis._dir);
        y = node.getCoord(1 - this.__outerThis._dir) + node.getOffsetToDistTopLeft(1 - this.__outerThis._dir);
        w = node.getDistSize(this.__outerThis._dir);
        h = node.getDistSize(1 - this.__outerThis._dir);
      }

      if (w < 1) {
        w = 1;
      }
      if (h < 1) {
        h = 1;
      }
      return new Rectangle(x, y, w, h); // Rectangle
    }
    else if (object instanceof LinkSegmentOptim.LinkSegment)
    {
      return (object).boundingBox(); // Rectangle
    }
    else
    {
      LogResUtil.logAndThrowRuntimeExc(" HierarchicalLayout ", "graphlayout.expert.message.6405E");
      return null; // Rectangle
    }
  }

});

return LinkSegmentOptim;
});
