/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HEnum",	// #5
  "../hierarchical/HGraphAlgorithm",	// #6
  "../hierarchical/HPercentages",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/LinkStyles",	// #9
  "../internalutil/TranslateUtil"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HEnum,	// #5
    HGraphAlgorithm,	// #6
    HPercentages,	// #7
    GraphModelUtil,	// #8
    LinkStyles,	// #9
    TranslateUtil	// #10
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var OptimizeLinksAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.OptimizeLinksAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		OptimizeLinksAlgorithm optimizes long links that contain several
  //		dummy nodes by conceptually removing dummy nodes. 
  // description:
  //		OptimizeLinksAlgorithm optimizes long links that contain several
  //		dummy nodes by conceptually removing dummy nodes. 
  //		The removing is not really done. The dummy nodes are merely marked as
  //		invalid. The final layout step that translates the hierarchical
  //		representation into the JViews representation will ignore such marked
  //		dummy nodes. This algorithm does not require a proper hierarchy
  //		structure with levels. But note that after this algorithm, the graph
  //		is not anymore a proper hierarchy. Therefore, this step should always
  //		be after the steps that use level sweeps.  
  // tags:
  //		private
  
  _multiLinkOptimizationEnabled : null,
  _multiLinkOptimizationOffset : null,
  _multiLinkOptimizationMaxSpread : null,

  constructor : function(graph)
  {
    this._multiLinkOptimizationEnabled = true;
    this._multiLinkOptimizationOffset = 5;
    this._multiLinkOptimizationMaxSpread = 50;
    this.init(graph);
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  setParameters : function(multiLinkOptimizationEnabled, multiLinkOptimizationOffset, multiLinkOptimizationMaxSpread)
  {
    // summary:
    //		Sets the parameters of the algorithm.
    // tags:
    //		private
    
    this._multiLinkOptimizationEnabled = multiLinkOptimizationEnabled;
    this._multiLinkOptimizationOffset = multiLinkOptimizationOffset;
    this._multiLinkOptimizationMaxSpread = multiLinkOptimizationMaxSpread;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_OPTIMIZE_LINKS], graph.getNumberOfLinks(), false);

    var glstyle = graph.getLayout().getGlobalLinkStyle();
    if (glstyle == LinkStyles.NO_RESHAPE_STYLE || glstyle == LinkStyles.STRAIGHT_LINE_STYLE) {
      return;
    }
    if (this._multiLinkOptimizationEnabled) {
      this.convertAllOverlappingMultilinksWithoutBends();
    }
    if (!this.hasThinDummyNodes()) {
      return;
    }
    this.translateThinDummyNodes();
    this.optimizeLinks();
  },

  convertAllOverlappingMultilinksWithoutBends : function()
  {
    // summary:
    //		Convert straight overlapping multilinks into bundles with bends.
    // tags:
    //		private
    
    var listOfMultilinkLists = new Array();
    this.collectMultilinksWithoutBends(listOfMultilinkLists);
    var iter =  new TranslateUtil.ArrayIterator(listOfMultilinkLists);

    while (iter.hasNext())
    {
      var links = iter.next();
      this.convertOverlappingMultilinksWithoutBends(links);
    }
  },

  convertOverlappingMultilinksWithoutBends : function(links)
  {
    // summary:
    //		Convert one link bundle of straight links into links with bends.
    // tags:
    //		private
    
    var n = links.length;
    var mx = 0;
    var my = 0;
    var dx = 0;
    var dy = 0;
    var dist = this._multiLinkOptimizationOffset;
    if (dist * n > this._multiLinkOptimizationMaxSpread) {
      dist = this._multiLinkOptimizationMaxSpread / n;
    }
    for (var i = 0; i < n; i++)
    {
      var link = links[i];

      if (i == 0)
      {
        var p0 = link.getRealFromPoint();
        var p1 = link.getRealToPoint();
        mx = 0.5 * (p0.x + p1.x);
        my = 0.5 * (p0.y + p1.y);
        dx = p1.y - p0.y;
        dy = p0.x - p1.x;
        var d = Math.sqrt(dx * dx + dy * dy);

        if (d > 0)
        {
          dx = dx / d;
          dy = dy / d;
        }
      }

      var dummy = link.addDummyNode(link.getStartSegment());
      dummy.setCoord(0, (mx + (i - 0.5 * (n - 1)) * dx * dist));
      dummy.setCoord(1, (my + (i - 0.5 * (n - 1)) * dy * dist));
      link.setFrozen(true);
    }
  },

  collectMultilinksWithoutBends : function(listOfMultilinkLists)
  {
    // summary:
    //		Collects all straight multilinks into the input list. 
    // description:
    //		Collects all straight multilinks into the input list. 
    //		At the end, the input list is filled with array lists of multilinks.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var niter = graph.getNodes();
    var siter;
    var segment;
    var otherNode;
    var aux;
    var link;
    var links;

    while (niter.hasNext())
    {
      var node = niter.next();
      if (node.isDummyNode()) {
        continue;
      }
      node.setAuxObject(/*Boolean.true*/true);
      siter = node.getSegmentsFrom();

      while (siter.hasNext())
      {
        segment = siter.next();
        otherNode = segment.getTo();
        if (otherNode.isDummyNode()) {
          continue;
        }
        if (otherNode.getAuxObject() == /*Boolean.true*/true) {
          continue;
        }
        otherNode.setAuxObject(null);
      }

      siter = node.getSegmentsTo();

      while (siter.hasNext())
      {
        segment = siter.next();
        otherNode = segment.getFrom();
        if (otherNode.isDummyNode()) {
          continue;
        }
        if (otherNode.getAuxObject() == /*Boolean.true*/true) {
          continue;
        }
        otherNode.setAuxObject(null);
      }

      siter = node.getSegmentsFrom();

      while (siter.hasNext())
      {
        segment = siter.next();
        otherNode = segment.getTo();
        if (otherNode.isDummyNode()) {
          continue;
        }
        aux = otherNode.getAuxObject();
        if (aux == /*Boolean.true*/true) {
          continue;
        }
        link = segment.getOwnerLink();
        if (link.getLinkStyle() == HEnum.ORTHOGONAL_LINK) {
          continue;
        }
        if (aux == null) {
          otherNode.setAuxObject(link);
        }
        else if (aux instanceof Array) {
          (aux).push(link);
        }
        else
        {
          links = new Array();
          links.push(aux);
          links.push(link);
          otherNode.setAuxObject(links);
          listOfMultilinkLists.push(links);
        }
      }

      siter = node.getSegmentsTo();

      while (siter.hasNext())
      {
        segment = siter.next();
        otherNode = segment.getFrom();
        if (otherNode.isDummyNode()) {
          continue;
        }
        aux = otherNode.getAuxObject();
        if (aux == /*Boolean.true*/true) {
          continue;
        }
        link = segment.getOwnerLink();
        if (link.getLinkStyle() == HEnum.ORTHOGONAL_LINK) {
          continue;
        }
        if (aux == null) {
          otherNode.setAuxObject(link);
        }
        else if (aux instanceof Array) {
          (aux).push(link);
        }
        else
        {
          links = new Array();
          links.push(aux);
          links.push(link);
          otherNode.setAuxObject(links);
          listOfMultilinkLists.push(links);
        }
      }
    }
  },

  hasThinDummyNodes : function()
  {
    // summary:
    //		Calculate how many of the thin dummy nodes exist.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.isDummyNode() && (node.getWidth() != 0.0 || node.getHeight() != 0.0))
      {
        return true; // Boolean
      }
    }

    return false; // Boolean
  },

  translateThinDummyNodes : function()
  {
    // summary:
    //		Translate the thin dummy nodes into two dummy nodes plus one segment
    //		in between.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getNodes();
    var node;
    var lastNode = graph.getLastNode();
    var dummyNode;
    var segment;
    var otherSegment;
    var index;
    var firstPoint;
    var secondPoint;

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.isDummyNode() && (node.getWidth() != 0.0 || node.getHeight() != 0.0))
      {
        if (node.getWidth() == 0.0) {
          index = 1;
        }
        else {
          index = 0;
        }
        if (node.getSegmentsFromCount() > 0)
        {
          segment = node.getFirstSegmentFrom();
          firstPoint = segment.getRealToCoord(index);
        }
        else if (node.getSegmentsToCount() > 0)
        {
          segment = node.getFirstSegmentTo();
          firstPoint = segment.getRealFromCoord(index);
        }
        else
        {
          segment = null;
          firstPoint = 0.0;
        }

        if (segment != null)
        {
          otherSegment = node.getOpposite(segment);
          if (otherSegment.getFrom() == node) {
            secondPoint = otherSegment.getRealToCoord(index);
          }
          else {
            secondPoint = otherSegment.getRealFromCoord(index);
          }
          dummyNode = segment.getOwnerLink().addDummyNode(segment);
          dummyNode.setCoord(0, node.getX());
          dummyNode.setCoord(1, node.getY());

          if (firstPoint < secondPoint)
          {
            dummyNode.setCoord(index, node.getCoord(index));
            node.setCoord(index, node.getCoord(index) + node.getSize(index));
          }
          else if (firstPoint > secondPoint)
          {
            dummyNode.setCoord(index, node.getCoord(index) + node.getSize(index));
          }
          else if (segment.isReversed())
          {
            dummyNode.setCoord(index, node.getCoord(index) + node.getSize(index));
          }
          else
          {
            dummyNode.setCoord(index, node.getCoord(index));
            node.setCoord(index, node.getCoord(index) + node.getSize(index));
          }

          node.setSize(index, 0.0);
        }
      }

      if (node == lastNode) {
        break;
      }
    }
  },

  optimizeLinks : function()
  {
    // summary:
    //		Optimize the links by marking dummy nodes as invalid if they are not
    //		needed.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var linkIter = graph.getLinks();
    var nodeIter;
    var link;
    var node;
    var segment;

    var point1;
    var point2A;
    var point2B;
    var point3;

    while (linkIter.hasNext())
    {
      link = linkIter.next();
      nodeIter = link.getNodes();
      segment = link.getStartSegment();

      if (segment.getTo().isDummyNode())
      {
        point1 = segment.getFromPoint();
        point2A = segment.getToPoint();
      }
      else
      {
        point1 = segment.getToPoint();
        point2A = segment.getFromPoint();
      }

      while (nodeIter.hasNext())
      {
        node = nodeIter.next();
        segment = node.getOpposite(segment);

        if (segment.getFrom() == node)
        {
          point2B = segment.getFromPoint();
          point3 = segment.getToPoint();
        }
        else
        {
          point2B = segment.getToPoint();
          point3 = segment.getFromPoint();
        }

        if (point2A == point2B)
        {
          if (point1[0] == point2A[0] && point2B[0] == point3[0] || point1[1] == point2A[1] && point2B[1] == point3[1])
          {
            node.markInvalid();
          }
          else {
            point1 = point2B;
          }
        }
        else {
          point1 = point2B;
        }
        point2A = point3;
      }

      this.getPercController().addPoints(1);
      this.layoutStepPerformed();
    }
  }

}
);

return OptimizeLinksAlgorithm;
});
