/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HLevelAlgorithm",	// #5
  "../hierarchical/HSegment",	// #6
  "../internalutil/GraphModelUtil",	// #7
  "../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLevelAlgorithm,	// #5
    HSegment,	// #6
    GraphModelUtil,	// #7
    TranslateUtil	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm;
=====*/

var OrthLinkRepositionAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.OrthLinkRepositionAlgorithm", HLevelAlgorithm, 
{
  // summary:
  //		OrthLinkRepositionAlgorithm is responsable for fixing position
  //		conflicts related to orthogonal links. 
  // description:
  //		OrthLinkRepositionAlgorithm is responsable for fixing position
  //		conflicts related to orthogonal links. 
  //		These conflicts occur if we have two segments that want to be
  //		converted into orthogonal and, e.g., in top down layout, and the one
  //		has exactly the same x coordinate at as "from" coordinate than the
  //		other as "to" coordinate. If this is the higher coordinate of both
  //		end points of the corresponding segment, we will have segment
  //		overlaps. As example, consider: A    B               A    B \  /     
  //		                     |&#95;&#95;&#95;&#95;| \/     which is transformed        | 
  //		which is unfortunately /\     into the original      &#95;&#95;&#95;&#95;&#95;| 
  //		ambiguous. /  \    picture:               |    | C    D              
  //		           C    D This algorithm tries to solve such ambiguities by
  //		moving the node D to the left or right. This solves the problem
  //		because the result will be A    B           A    B |&#95;&#95;  |          
  //		|&#95;&#95;&#95;&#95;|&#95; | |                | |  &#95;&#95;|&#95;|    or      &#95;&#95;&#95;&#95;| | |  |        
  //		    |      | C  D             C      D The algorithm assumes that we
  //		have a proper hierarchy structure, the adjacencies at the segments
  //		are sorted and the connector points are already calculated.  
  // tags:
  //		private
  
  _minDelta: null,
  _segments: null,

  _leftDelta: null,
  _rightDelta: null,

  constructor : function(graph)
  {
    var direction = graph.getLevelFlow();
    this._minDelta = graph.getMinDistBetweenLinks(direction);
  },
  
  init : function(upperLevel, lowerLevel)
  {
    // summary:
    //		Initializes the algorithm with the actual levels.
    // tags:
    //		private
    
    this.inherited("init", arguments, [upperLevel, lowerLevel]);
  },

  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._segments = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    this.createSegmentField();

    this.markDummyNodesOfStraightSegments();

    this.resolveAllConflicts();
    this._segments = null;
  },

  createSegmentField : function()
  {
    // summary:
    //		Fills the array _segments with the segments between the current
    //		upper and lower level. 
    // description:
    //		Fills the array &#95;segments with the segments between the current
    //		upper and lower level. 
    //		The segments are sorted according to the "from" point. The sort
    //		value of the segment contains the index of the segment in the field,
    //		for faster access.
    // tags:
    //		private
    
    var iter = this.getUpperLevel().getNormalSegmentsFrom();
    var segment;
    var i = 0;

    while (iter.hasNext())
    {
      iter.next();
      i++;
    }

    this._segments = TranslateUtil.newArray(i, null);

    iter = this.getUpperLevel().getNormalSegmentsFrom();
    i = 0;

    while (iter.hasNext())
    {
      segment = iter.next();
      this._segments[i] = segment;
      segment.setSortValue(i);
      i++;
    }
  },

  markDummyNodesOfStraightSegments : function()
  {
    // summary:
    //		Mark all dummy nodes that are part of a straight link. 
    // description:
    //		Mark all dummy nodes that are part of a straight link. 
    //		When we have to move these dummy nodes, we will try to keep them
    //		straight.
    // tags:
    //		private
    
    var iter = this.getLowerLevel().getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (this.isPartOfStraightSegment(node)) {
        node.setMarker(4711);
      }
      else {
        node.setMarker(0);
      }
    }
  },

  canCauseConflict : function(segment)
  {
    // summary:
    //		Returns true if this segment may cause a conflict at its "from"
    //		point.
    // tags:
    //		private
    
    if (!segment.getOwnerLink().isOrthogonal()) {
      return false; // Boolean
    }
    var dir = this.getLevelFlowDir();

    if (segment.getRealFromCoord(dir) > segment.getRealToCoord(dir) + 0.0002) {
      return true; // Boolean
    }
    return false; // Boolean
  },

  hasConflictAtSegment : function(segment)
  {
    // summary:
    //		Returns true if this segment causes a conflict at its "to" point. 
    // description:
    //		Returns true if this segment causes a conflict at its "to" point. 
    //		If true, it updates as side effect the values &#95;leftDelta and
    //		&#95;rightDelta which are the minimal amount to move the "to" node of
    //		this segment to the left or right to solve this conflict.   
    // tags:
    //		private
    
    if (!segment.getOwnerLink().isOrthogonal()) {
      return false; // Boolean
    }
    var dir = this.getLevelFlowDir();
    var toPoint = segment.getRealToCoord(dir);
    var hasConf = false;

    if (segment.getRealFromCoord(dir) < toPoint)
    {
      var minConflictCoord = toPoint - this._minDelta;
      var maxConflictCoord = toPoint + this._minDelta;
      var conflictCoord;
      var distToMinConflictCoord;
      var distToMaxConflictCoord;
      var index = Math.floor(segment.getSortValue()) + 1;
      var otherSegment;

      while (index < this._segments.length)
      {
        otherSegment = this._segments[index++];
        conflictCoord = otherSegment.getRealFromCoord(dir);
        distToMinConflictCoord = conflictCoord - minConflictCoord;
        if (distToMinConflictCoord <= 0.0) {
          continue;
        }
        distToMaxConflictCoord = maxConflictCoord - conflictCoord;
        if (distToMaxConflictCoord <= 0.0) {
          break;
        }
        if (this.canCauseConflict(otherSegment))
        {
          if (distToMaxConflictCoord > this._leftDelta) {
            this._leftDelta = distToMaxConflictCoord;
          }
          if (distToMinConflictCoord > this._rightDelta) {
            this._rightDelta = distToMinConflictCoord;
          }
          hasConf = true;
        }
      }
    }

    return hasConf; // Boolean
  },

  hasConflictAtNode : function(node)
  {
    // summary:
    //		Returns true if this node causes a conflict. 
    // description:
    //		Returns true if this node causes a conflict. 
    //		If true, it updates as side effect the values &#95;leftDelta and
    //		&#95;rightDelta which are the minimal amount to move this node to the
    //		left or right to solve this conflict.   
    // tags:
    //		private
    
    var iter = node.getSegmentsTo();
    var segment;
    this._leftDelta = 0.0;
    this._rightDelta = 0.0;
    var hasConf = false;

    while (iter.hasNext())
    {
      segment = iter.next();
      if (this.hasConflictAtSegment(segment)) {
        hasConf = true;
      }
    }

    return hasConf; // Boolean
  },

  resolveConflict : function(node, minCoord)
  {
    // summary:
    //		Resolved a conflict for one node. 
    // description:
    //		Resolved a conflict for one node. 
    //		minCoord is the minimal coordinate the node can be moved to.
    // tags:
    //		private
    
    var dir = this.getLevelFlowDir();
    if (node.isFixedForIncremental(dir)) {
      return;
    }
    var coord = node.getCoord(dir);
    var moveToLowerAllowed = true;
    var leftCoord;
    var rightCoord;

    var iterCount = 0;
    var notReady = true;
    var factor;

    while (this.hasConflictAtNode(node) && notReady)
    {
      iterCount++;

      leftCoord = coord - this._leftDelta;

      rightCoord = coord + this._rightDelta;

      factor = 1.0;

      while (leftCoord == coord && factor < 1.0E16)
      {
        factor *= 2.0;
        leftCoord = coord - this._leftDelta * factor;
      }

      factor = 1.0;

      while (rightCoord == coord && factor < 1.0E16)
      {
        factor *= 2.0;
        rightCoord = coord + this._rightDelta * factor;
      }

      if (moveToLowerAllowed && leftCoord >= minCoord) {
        this.moveNode(node, leftCoord);
      }
      else
      {
        notReady = this.moveNodes(node, rightCoord);
        moveToLowerAllowed = false;
      }

      coord = node.getCoord(dir);
      notReady = (iterCount < 1000) && notReady;
    }
  },

  resolveAllConflicts : function()
  {
    var dir = this.getLevelFlowDir();
    var iter = this.getLowerLevel().getNodes();
    var prevNode = null;
    var node;
    var minPos = -/*Number.MAX_VALUE*/(3.4028235E38);

    while (iter.hasNext())
    {
      node = iter.next();
      if (prevNode != null) {
        minPos += this.getMinDist(prevNode, node);
      }
      this.resolveConflict(node, minPos);
      minPos = node.getCoord(dir) + node.getSize(dir);
      prevNode = node;
    }
  },

  moveNodes : function(startNode, startPos)
  {
    // summary:
    //		Moves all nodes starting from the input node so that the start node
    //		is at startPos and the remaining nodes are still not operlapping in
    //		level flow direction. 
    // description:
    //		Moves all nodes starting from the input node so that the start node
    //		is at startPos and the remaining nodes are still not operlapping in
    //		level flow direction. 
    //		Note that this also updates the "from" and "to" points of segments.
    //		It returns true if we were able to move the nodes, and false
    //		otherwise.
    // tags:
    //		private
    
    var dir = this.getLevelFlowDir();
    var iter = this.getLowerLevel().getNodes();
    var prevNode = null;
    var node;
    var actPos = startPos;

    iter.init(startNode);

    while (iter.hasNext())
    {
      node = iter.next();
      if (prevNode != null) {
        actPos += this.getMinDist(prevNode, node);
      }
      if (node.getCoord(dir) < actPos)
      {
        if (node.isFixedForIncremental(dir))
        {
          return false; // Boolean
        }

        actPos += node.getSize(dir);
        prevNode = node;
      }
      else {
        break;
      }
    }

    prevNode = null;
    actPos = startPos;
    iter = this.getLowerLevel().getNodes();
    iter.init(startNode);

    while (iter.hasNext())
    {
      node = iter.next();
      if (prevNode != null) {
        actPos += this.getMinDist(prevNode, node);
      }
      if (node.getCoord(dir) < actPos)
      {
        this.moveNode(node, actPos);
        actPos += node.getSize(dir);
        prevNode = node;
      }
      else {
        break;
      }
    }

    return true; // Boolean
  },

  moveNode : function(node, coord)
  {
    // summary:
    //		Moves a node in level flow direction to the input coordinate. 
    // description:
    //		Moves a node in level flow direction to the input coordinate. 
    //		If the node is part of a straight link, it also tries to move the
    //		remaining dummy nodes of this link, so that it stays straight.
    // tags:
    //		private
    
    var needsStraighten = this.isPartOfStraightSegment(node) || (node.getLevel() == this.getLowerLevel() && node.getMarker() == 4711);
    node.setCoordWithLinkUpdate(this.getLevelFlowDir(), coord);
    if (needsStraighten) {
      this.straightenLink(node.getFirstSegmentFrom().getTo(), coord);
    }
  },

  isPartOfStraightSegment : function(node)
  {
    // summary:
    //		Returns true if the dummy node is in a segment that is straight. 
    // description:
    //		Returns true if the dummy node is in a segment that is straight. 
    //		This means the next dummy node of the same link is at the same
    //		position in level flow as the input node.
    // tags:
    //		private
    
    var dir = this.getLevelFlowDir();
    if (!node.isDummyNode()) {
      return false; // Boolean
    }
    if (node.getSegmentsFromCount() != 1) {
      return false; // Boolean
    }
    var nextNode = node.getFirstSegmentFrom().getTo();
    if (!nextNode.isDummyNode()) {
      return false; // Boolean
    }
    if (nextNode.getSegmentsToCount() != 1) {
      return false; // Boolean
    }
    return (nextNode.getCoord(dir) == node.getCoord(dir)); // Boolean
  },

  straightenLink : function(node, coord)
  {
    // summary:
    //		Try to straighten a link again. 
    // description:
    //		Try to straighten a link again. 
    //		We do this after we moved the start node, if the start node was a
    //		dummy node and node is the next dummy node of the same link, and the
    //		link was originally straight. So we try to make it straight again.
    // tags:
    //		private
    
    var dir = this.getLevelFlowDir();

    var prevNode = node.getPrevNeighbor();
    var nextNode = node.getNextNeighbor();

    if (prevNode != null)
    {
      if (coord < prevNode.getCoord(dir) + prevNode.getSize(dir) + this.getMinDist(prevNode, node)) {
        return;
      }
    }

    if (nextNode != null)
    {
      if (coord > nextNode.getCoord(dir) - node.getSize(dir) - this.getMinDist(node, nextNode)) {
        return;
      }
    }

    this.moveNode(node, coord);
  },

  getMinDist : function(node1, node2)
  {
    // summary:
    //		Returns the minimal distance between both input nodes.
    // tags:
    //		private
    
    return this.getGraph().getMinDist(this.getLevelFlowDir(), node1, node2); // Number
  }

}
);

return OrthLinkRepositionAlgorithm;
});
