/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HGraphAlgorithm",	// #5
  "../hierarchical/HLevelAlgApplicator",	// #6
  "../hierarchical/HPercentages",	// #7
  "../hierarchical/HSegmentBufferedIterator",	// #8
  "../hierarchical/OrthRoutingLevelAlgorithm",	// #9
  "../internalutil/GraphModelUtil",	// #10
  "../internalutil/TranslateUtil"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HGraphAlgorithm,	// #5
    HLevelAlgApplicator,	// #6
    HPercentages,	// #7
    HSegmentBufferedIterator,	// #8
    OrthRoutingLevelAlgorithm,	// #9
    GraphModelUtil,	// #10
    TranslateUtil	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var OrthRoutingAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.OrthRoutingAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		OrthRoutingAlgorithm traverses the levels and performs orthogonal
  //		routing of the segments. 
  // description:
  //		OrthRoutingAlgorithm traverses the levels and performs orthogonal
  //		routing of the segments. 
  //		Note that afterwards, the graph is not anymore a proper hierarchy.
  //		Therefore, this step should always be the last one of the steps that
  //		use level sweeps.  
  // tags:
  //		private
  
  _levelAlg: null,

  constructor : function(graph)
  {
    this.init(graph);
    this._levelAlg = new OrthRoutingLevelAlgorithm(graph);
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    if (this._levelAlg != null) {
      this._levelAlg.clean();
    }
    this._levelAlg = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_ORTH_ROUTING], graph.getNumberOfLevels() - 1, false);

    if (graph.getNumberOfNormalSegments() == 0 && graph.getNumberOfIntergraphSegments() == 0) {
      return;
    }
    new HLevelAlgApplicator(graph, this._levelAlg, false).run();

    this._levelAlg.clean();
    this._levelAlg = null;

    this.createOrthogonalLinkShapes();
  },

  createOrthogonalLinkShapes : function()
  {
    // summary:
    //		Reorganizes the graph so that the links consist completely of
    //		orthogonal segments. 
    // description:
    //		Reorganizes the graph so that the links consist completely of
    //		orthogonal segments. 
    //		It splits each segment into 3 segments, if the orthogonal bend field
    //		of the segment contains a valid split coordinate.
    // tags:
    //		private
    
    var graph = this.getGraph();

    var iter = new HSegmentBufferedIterator(graph.getAllSegments());
    var segment;
    var link;
    var dummyNode;
    var coord;
    var fromCoord;
    var toCoord;
    var edgeDir = graph.getEdgeFlow();
    var levelDir = graph.getLevelFlow();

    while (iter.hasNext())
    {
      segment = iter.next();
      coord = segment.getOrthogonalBend();

      if (coord != /*Number.MAX_VALUE*/(3.4028235E38))
      {
        fromCoord = graph.getTranslatedCoord(segment.getFrom(), segment.getFromPoint()[levelDir], levelDir);
        toCoord = graph.getTranslatedCoord(segment.getTo(), segment.getToPoint()[levelDir], levelDir);
        link = segment.getOwnerLink();
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setCoord(edgeDir, coord);
        if (segment.isReversed()) {
          dummyNode.setCoord(levelDir, fromCoord);
        }
        else {
          dummyNode.setCoord(levelDir, toCoord);
        }
        dummyNode = link.addDummyNodeAtGraph(segment, graph);
        dummyNode.setCoord(edgeDir, coord);
        if (segment.isReversed()) {
          dummyNode.setCoord(levelDir, toCoord);
        }
        else {
          dummyNode.setCoord(levelDir, fromCoord);
        }
      }
    }

    this.layoutStepPerformed();
  }

}
);

return OrthRoutingAlgorithm;
});
