/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HLevelAlgorithm",	// #5
  "../hierarchical/HNode",	// #6
  "../hierarchical/HSegment",	// #7
  "../hierarchical/HSegmentSetMarker",	// #8
  "../internalutil/ArrayStableSort",	// #9
  "../internalutil/GraphModelUtil",	// #10
  "../internalutil/TranslateUtil"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLevelAlgorithm,	// #5
    HNode,	// #6
    HSegment,	// #7
    HSegmentSetMarker,	// #8
    ArrayStableSort,	// #9
    GraphModelUtil,	// #10
    TranslateUtil	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm;
=====*/

var OrthRoutingLevelAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.OrthRoutingLevelAlgorithm", HLevelAlgorithm, 
{
  // summary:
  //		OrthRoutingLevelAlgorithm performs the first pass of a orthogonal
  //		routing for the segments that run between these levels. 
  // description:
  //		OrthRoutingLevelAlgorithm performs the first pass of a orthogonal
  //		routing for the segments that run between these levels. 
  //		It stores the orthogonal bend coordinate in the segment, but does not
  //		really create the orthogonal segments yet.  The algorithm assumes
  //		that we have a proper hierarchy structure, the adjacencies at the
  //		segments are sorted and the connector points are already calculated. 
  //		
  // tags:
  //		private
  
  _numberOfBends: null,
  _maxLayerNumber: null,

  _halfMinDelta: null,

  _segmentMarker: null,

  _intervals: null,
  _startIntervals: null,
  _endIntervals: null,
  _upperLevelHasForks: null,
  _lowerLevelHasForks: null,
  _numNormalSegments: null,

  constructor : function(graph)
  {
    this._numberOfBends = 0;
    this._maxLayerNumber = 0.0;
    this._segmentMarker = new HSegmentSetMarker(graph, false);
    this._numNormalSegments = graph.getNumberOfNormalSegments();
    this._intervals = TranslateUtil.newArray(graph.getNumberOfNormalSegments() + graph.getNumberOfIntergraphSegments(), null);
    this._halfMinDelta = 0.5 * graph.getMinDistBetweenLinks(graph.getLevelFlow());
  },
  
  init : function(upperLevel, lowerLevel)
  {
    // summary:
    //		Initializes the algorithm with the actual levels.
    // tags:
    //		private
    
    this.inherited("init", arguments, [upperLevel, lowerLevel]);
    this._maxLayerNumber = 0.0;
  },

  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    if (this._segmentMarker != null) {
      this._segmentMarker.clean();
    }
    this._segmentMarker = null;
    this._intervals = null;
    this._startIntervals = null;
    this._endIntervals = null;
  },

  run : function()
  {
    // summary:
    //		Start the algorithm.
    // tags:
    //		private
    
    this.getGraph().updateSegmentIDs();

    this.checkForkNodes();

    this.calcIntervalGraph();

    this.calcFinalLayerNumbers();

    this.cleanSegmentIntervals();

    this.shiftLowerLevelAndSetBendCoord();

    this.optimizeBypassDummyNodes();
  },

  getNumberOfBends : function()
  {
    // summary:
    //		Returns the number of additional bends needed for the orthogonal
    //		routing since allocation of this algorithm object.
    // tags:
    //		private
    
    return this._numberOfBends; // int
  },

  checkForkNodes : function()
  {
    var iter;
    var segment;
    var dir = this.getLevelFlowDir();
    this._upperLevelHasForks = false;
    this._lowerLevelHasForks = false;
    if (this.getGraph().getLayout().isFromFork() == false && this.getGraph().getLayout().isToFork() == false) {
      return;
    }
    iter = this.getUpperLevel().getSegmentsFrom(true);

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.needsFromFork())
      {
        this._upperLevelHasForks = true;
        break;
      }
    }

    iter = this.getLowerLevel().getSegmentsTo(true);

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.needsToFork())
      {
        this._lowerLevelHasForks = true;
        break;
      }
    }
  },

  createSegmentIntervals : function()
  {
    // summary:
    //		Create all segment intervals needed between this upper and lower
    //		level. 
    // description:
    //		Create all segment intervals needed between this upper and lower
    //		level. 
    //		The intervals are stored in the field &#95;intervals at index position
    //		of the first segment of each interval, and again in the fields
    //		called &#95;startIntervals and &#95;endIntervals, which we need for the
    //		plane sweep later.
    // tags:
    //		private
    
    var dir = this.getLevelFlowDir();

    var iter = this.getUpperLevel().getSegmentsFrom(true);
    var segment;
    var i = 0;

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.getFrom().isBypassNode())
      {
        if (segment.isReversed()) {
          i++;
        }
      }
      else if (segment.getTo().isBypassNode())
      {
        if (segment.isReversed()) {
          i++;
        }
      }
      else if (this.getFromCoord(segment, dir) != this.getToCoord(segment, dir) && segment.getOwnerLink().isOrthogonal()) {
        i++;
      }
    }

    this._startIntervals = TranslateUtil.newArray(i, null);
    this._endIntervals = TranslateUtil.newArray(i, null);

    iter = this.getUpperLevel().getSegmentsFrom(true);
    i = 0;
    var interval;

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.getFrom().isBypassNode())
      {
        if (segment.isReversed())
        {
          interval = new OrthRoutingLevelSegmentInterval(this, segment.getFrom());
          this._startIntervals[i] = interval;
          this._endIntervals[i] = interval;
          i++;
        }
      }
      else if (segment.getTo().isBypassNode())
      {
        if (segment.isReversed())
        {
          interval = new OrthRoutingLevelSegmentInterval(this, segment.getTo());
          this._startIntervals[i] = interval;
          this._endIntervals[i] = interval;
          i++;
        }
      }
      else if (this.getFromCoord(segment, dir) != this.getToCoord(segment, dir) && segment.getOwnerLink().isOrthogonal())
      {
        interval = new OrthRoutingLevelSegmentInterval(this, segment);
        this._startIntervals[i] = interval;
        this._endIntervals[i] = interval;
        i++;
      }
      else
      {
        segment.setOrthogonalBend(0.0);
      }
    }
  },

  cleanSegmentIntervals : function()
  {
    // summary:
    //		Clean all segment intervals needed between this upper and lower
    //		level. 
    // description:
    //		Clean all segment intervals needed between this upper and lower
    //		level. 
    //		Not really necessary, but nice for the memory.
    // tags:
    //		private
    
    for (var i = 0; i < this._startIntervals.length; i++)
    {
      this._intervals[this.getIntervalID(this._startIntervals[i].firstSegment)] = null;
      this._startIntervals[i].clean();
      this._startIntervals[i] = null;
      this._endIntervals[i] = null;
    }
  },

  calcFinalLayerNumbers : function()
  {
    // summary:
    //		Calculates the layer number for the segment intervals in
    //		_startIntervals.
    // tags:
    //		private
    
    var roots = new Array();
    var interval;
    var largerInterval;
    var layerNumber;
    var e;

    this._maxLayerNumber = 1.0;

    for (var i = 0; i < this._startIntervals.length; i++)
    {
      interval = this._startIntervals[i];
      interval.setLayerNumber(1.0);
      if (interval.nrSmallerIntervals == 0) {
        roots.push(interval);
      }
    }

    while (!(roots.length==0))
    {
      interval = roots.pop();
      layerNumber = interval.getLayerNumber() + 1.0;
      e =  new TranslateUtil.ArrayIterator(interval.largerIntervals);

      while (e.hasNext())
      {
        largerInterval = e.next();
        largerInterval.setLayerNumberAtLeastTo(layerNumber);
        largerInterval.nrSmallerIntervals--;
        if (layerNumber > this._maxLayerNumber) {
          this._maxLayerNumber = layerNumber;
        }
        if (largerInterval.nrSmallerIntervals == 0) {
          roots.push(largerInterval);
        }
      }
    }

    roots = null;
  },

  calcIntervalGraph : function()
  {
    // summary:
    //		Calculates the interval graph.
    // tags:
    //		private
    
    this.createSegmentIntervals();
    new OrthRoutingLevelSegmentIntervalLowerBoundSort().sort(this._startIntervals);
    new OrthRoutingLevelSegmentIntervalHigherBoundSort().sort(this._endIntervals);

    var startIndex = 0;
    var endIndex = 0;
    var maxIndex = this._startIntervals.length;
    var interval;

    while (startIndex < maxIndex || endIndex < maxIndex)
    {
      if (startIndex < maxIndex && endIndex < maxIndex)
      {
        if (this._startIntervals[startIndex].getLowerDeltaBound() <= this._endIntervals[endIndex].getHigherDeltaBound())
        {
          interval = this._startIntervals[startIndex++];
          this.resolveConflicts(interval);
          this.makeActive(interval);
        }
        else
        {
          interval = this._endIntervals[endIndex++];
          this.makeInactive(interval);
        }
      }
      else if (startIndex < maxIndex)
      {
        interval = this._startIntervals[startIndex++];
        this.resolveConflicts(interval);
        this.makeActive(interval);
      }
      else
      {
        interval = this._endIntervals[endIndex++];
        this.makeInactive(interval);
      }
    }
  },

  resolveConflicts : function(inputInterval)
  {
    // summary:
    //		Resolves conflicts between intervals by deciding for each currently
    //		active, conflicting interval whether it is smaller or larger than
    //		the input interval, which is about to become active. 
    // description:
    //		Resolves conflicts between intervals by deciding for each currently
    //		active, conflicting interval whether it is smaller or larger than
    //		the input interval, which is about to become active. 
    //		This actually sets the interval handles to larger intervals in the
    //		vector largerIntervals.
    // tags:
    //		private
    
    var activeInterval;
    var iter = this.getActiveIntervals();

    while (iter.hasNext())
    {
      activeInterval = iter.next();

      if (this.haveConflict(activeInterval, inputInterval))
      {
        if (this.isSmaller(activeInterval, inputInterval))
        {
          activeInterval.largerIntervals.push(inputInterval);
          inputInterval.nrSmallerIntervals++;
        }
        else
        {
          inputInterval.largerIntervals.push(activeInterval);
          activeInterval.nrSmallerIntervals++;
        }
      }
    }
  },

  haveConflict : function(i1, i2)
  {
    // summary:
    //		Returns true if both active segments interval are in conflict. 
    // description:
    //		Returns true if both active segments interval are in conflict. 
    //		The interval i1 is already active, the interval i2 becomes active
    //		right now.
    // tags:
    //		private
    
    if (i1.intervalType == OrthRoutingLevelAlgorithm.HL || i1.intervalType == OrthRoutingLevelAlgorithm.LL) {
      if (i2.intervalType == OrthRoutingLevelAlgorithm.HH || i2.intervalType == OrthRoutingLevelAlgorithm.HL) {
        return true; // Boolean
      }
    }
    if (i1.intervalType == OrthRoutingLevelAlgorithm.HH || i1.intervalType == OrthRoutingLevelAlgorithm.LH) {
      if (i2.intervalType == OrthRoutingLevelAlgorithm.LH || i2.intervalType == OrthRoutingLevelAlgorithm.LL) {
        return true; // Boolean
      }
    }
    return OrthRoutingLevelAlgorithm.overlaps(i1, i2); // Boolean
  },

  isSmaller : function(i1, i2)
  {
    // summary:
    //		Returns true if the first segment interval needs a smaller layer
    //		than the second segment interval in orthogonal routing.
    // tags:
    //		private
    
    if (i1.intervalType != i2.intervalType)
    {
      if (i1.intervalType == OrthRoutingLevelAlgorithm.HH) {
        return true; // Boolean
      }
      if (i2.intervalType == OrthRoutingLevelAlgorithm.HH) {
        return false; // Boolean
      }
      if (i1.intervalType == OrthRoutingLevelAlgorithm.LL) {
        return false; // Boolean
      }
      if (i2.intervalType == OrthRoutingLevelAlgorithm.LL) {
        return true; // Boolean
      }
      if (i1.intervalType == OrthRoutingLevelAlgorithm.HL) {
        return true; // Boolean
      }
      return false; // Boolean
    }

    if (i1.intervalType == OrthRoutingLevelAlgorithm.HH)
    {
      if (i1.higherCoord < i2.higherCoord) {
        return true; // Boolean
      }
      if (i1.higherCoord > i2.higherCoord) {
        return false; // Boolean
      }
      if (i1.lowerCoord > i2.lowerCoord) {
        return true; // Boolean
      }
      if (i1.lowerCoord < i2.lowerCoord) {
        return false; // Boolean
      }
      return this.getIntervalID(i1.firstSegment) < this.getIntervalID(i2.firstSegment); // Boolean
    }
    else if (i1.intervalType == OrthRoutingLevelAlgorithm.LL)
    {
      if (i1.lowerCoord < i2.lowerCoord) {
        return true; // Boolean
      }
      if (i1.lowerCoord > i2.lowerCoord) {
        return false; // Boolean
      }
      if (i1.higherCoord > i2.higherCoord) {
        return true; // Boolean
      }
      if (i1.higherCoord < i2.higherCoord) {
        return false; // Boolean
      }
      return this.getIntervalID(i1.firstSegment) < this.getIntervalID(i2.firstSegment); // Boolean
    }
    else if (i1.intervalType == OrthRoutingLevelAlgorithm.LH)
    {
      if (i2.higherCoord > i1.higherCoord || i2.higherCoord == i1.higherCoord && i2.lowerCoord > i1.lowerCoord) {
        return true; // Boolean
      }
      return false; // Boolean
    }
    else
    {
      if (i2.lowerCoord < i1.lowerCoord || i2.lowerCoord == i1.lowerCoord && i2.higherCoord < i1.higherCoord) {
        return true; // Boolean
      }
      return false; // Boolean
    }
  },

  makeActive : function(i)
  {
    // summary:
    //		Adds an interval to the set of active segment intervals.
    // tags:
    //		private
    
    this._segmentMarker.insert(i.firstSegment);
  },

  makeInactive : function(i)
  {
    // summary:
    //		Removes an interval from the set of active segment intervals.
    // tags:
    //		private
    
    this._segmentMarker.discard(i.firstSegment);
  },

  getActiveIntervals : function()
  {
    // summary:
    //		Returns the active segments.
    // tags:
    //		private
    
    return new OrthRoutingLevelAlgorithmAnonClass_1(this); // ibm_ilog.graphlayout.hierarchical.OrthRoutingLevelIntervalIterator
  },

  shiftLowerLevelAndSetBendCoord : function()
  {
    // summary:
    //		Finally, create enough space and assign the orthogonal bend
    //		coordinates. 
    // description:
    //		Finally, create enough space and assign the orthogonal bend
    //		coordinates. 
    //		This means: shift the lower level away from the upper level so that
    //		there is enough space for horizontal segment layers in between. Then
    //		assign the orthogonal bend coordinates.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var upperLevel = this.getUpperLevel();
    var lowerLevel = this.getLowerLevel();
    var dir = this.getEdgeFlowDir();
    var minPos = upperLevel.getCoord(dir) + upperLevel.getSize(dir);
    var maxPos = lowerLevel.getCoord(dir);
    var desiredMaxPos;
    var minDist = graph.getMinDistBetweenNodes(dir);
    var desiredDist = 0;
    var nodelinkdist = graph.getMinDistBetweenNodeAndLink(dir);
    var linkdist = graph.getMinDistBetweenLinks(dir);
    var levelJustification = graph.getLevelJustification();
    var hasFixedLevels = graph.hasFixedNodesInLevels(dir);

    var iter = upperLevel.getSegmentsFrom(true);
    var segment;
    var k;
    var thickness;
    var highestThickness = 0.0;
    var highestEndThickness = 0.0;

    while (iter.hasNext())
    {
      segment = iter.next();
      thickness = segment.getOwnerLink().getThickness();
      if (highestThickness < thickness) {
        highestThickness = thickness;
      }
      k = segment.getOrthogonalBend();

      if (k == 1.0 || k == this._maxLayerNumber)
      {
        if (highestEndThickness < thickness) {
          highestEndThickness = thickness;
        }
      }
    }

    nodelinkdist += 0.5 * highestEndThickness;
    linkdist += highestThickness;

    var startDist = nodelinkdist;
    var endDist = nodelinkdist;
    var forkSegLen = graph.getLayout().getMinForkSegmentLength();
    if (this._upperLevelHasForks) {
      startDist = Math.max(startDist, forkSegLen + linkdist);
    }
    if (this._lowerLevelHasForks) {
      endDist = Math.max(endDist, forkSegLen + linkdist);
    }
    if (this._maxLayerNumber > 0.0) {
      desiredDist = (this._maxLayerNumber - 1.0) * linkdist + startDist + endDist;
    }
    if (hasFixedLevels)
    {
      desiredMaxPos = minPos + desiredDist;

      if (desiredMaxPos <= maxPos) {
        startDist += 0.5 * (maxPos - desiredMaxPos);
      }
      else
      {
        if (maxPos < minPos)
        {
          maxPos = minPos;
          minPos = lowerLevel.getCoord(dir);
        }

        var n = this._maxLayerNumber + 1.0;
        if (this._upperLevelHasForks) {
          n++;
        }
        if (this._lowerLevelHasForks) {
          n++;
        }
        linkdist = (maxPos - minPos) / n;
        if (this._upperLevelHasForks) {
          startDist = 2 * linkdist;
        }
        else {
          startDist = linkdist;
        }
      }
    }
    else
    {
      desiredMaxPos = minPos + Math.max(minDist, desiredDist);
      lowerLevel.setFreeNodesCoordInEdgeFlowWithLinkUpdate(desiredMaxPos, levelJustification);
    }

    upperLevel.setFromForkCoord(minPos + startDist - linkdist);
    lowerLevel.setToForkCoord(minPos + startDist + this._maxLayerNumber * linkdist);

    iter = upperLevel.getSegmentsFrom(true);

    while (iter.hasNext())
    {
      segment = iter.next();
      k = segment.getOrthogonalBend();

      if (k > 0.0)
      {
        segment.setOrthogonalBend(minPos + startDist + (k - 1.0) * linkdist);
        this._numberOfBends += 2;
      }
      else {
        segment.setOrthogonalBend(/*Number.MAX_VALUE*/(3.4028235E38));
      }
    }
  },

  optimizeBypassDummyNodes : function()
  {
    // summary:
    //		Optimize the position of the dummy nodes between a reference segment
    //		and its referenced segment.
    // tags:
    //		private
    
    var iter = this.getUpperLevel().getNodes();
    var node;
    var segment;
    var dir = this.getEdgeFlowDir();

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.isUpperBypassNode())
      {
        segment = node.getFirstSegmentFrom();
        if (segment.getOrthogonalBend() != /*Number.MAX_VALUE*/(3.4028235E38)) {
          node.setCoord(dir, segment.getOrthogonalBend());
        }
      }
    }

    iter = this.getLowerLevel().getNodes();

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.isLowerBypassNode())
      {
        segment = node.getFirstSegmentTo();
        if (segment.getOrthogonalBend() != /*Number.MAX_VALUE*/(3.4028235E38)) {
          node.setCoord(dir, segment.getOrthogonalBend());
        }
      }
    }
  },

  getFromCoord : function(segment, dir)
  {
    // summary:
    //		Returns the from coordinate of the segment in the coordinate system
    //		of this graph.
    // tags:
    //		private
    
    return segment.getRealFromCoord(dir); // Number
  },

  getToCoord : function(segment, dir)
  {
    // summary:
    //		Returns the to coordinate of the segment in the coordinate system of
    //		this graph.
    // tags:
    //		private
    
    return segment.getRealToCoord(dir); // Number
  },

  getIntervalID : function(segment)
  {
    // summary:
    //		Returns the ID of a segment.
    // tags:
    //		private
    
    if (segment.isIntergraphEdge()) {
      return this._numNormalSegments + segment.getNumID(); // int
    }
    return segment.getNumID(); // int
  }

}
);

lang.mixin(OrthRoutingLevelAlgorithm,
{
  HL : null,
  LH : null,
  LL : null,
  HH : null,

  overlaps : function(a, b)
  {
    return a.lowerCoord <= b.higherCoord && a.higherCoord >= b.lowerCoord; // Boolean
  },

  _fields_init : function(c)
  {
    c = c || OrthRoutingLevelAlgorithm;
    if(HLevelAlgorithm._fields_init)
      HLevelAlgorithm._fields_init(c);
    
    c.HL = 0;
    c.LH = 1;
    c.LL = 2;
    c.HH = 3;
  }
  
}
);

OrthRoutingLevelSegmentInterval = declare( null, 
{
  __outerThis: null,
  intervalType: null,
  lowerCoord: null,
  higherCoord: null,
  firstSegment: null,
  secondSegment: null,
  nrSmallerIntervals: null,
  largerIntervals: null,

  constructor : function(input__outerThis, arg)
  {
    this.__outerThis = input__outerThis;
    if (arg instanceof HSegment) {
      this.initFromSegment(arg);
    }
    else if (arg instanceof HNode) {
      this.initFromNode(arg);
    }
  },
  
  initFromSegment : function(segment)
  {
    var dir = this.__outerThis.getLevelFlowDir();
    this.firstSegment = segment;
    this.secondSegment = null;
    var graph = segment.getOwnerGraph();
    var cf = this.__outerThis.getFromCoord(segment, dir);
    var ct = this.__outerThis.getToCoord(segment, dir);
    var thickness = segment.getOwnerLink().getThickness();

    if (cf < ct)
    {
      this.lowerCoord = cf - 0.5 * thickness;
      this.higherCoord = ct + 0.5 * thickness;
      this.intervalType = OrthRoutingLevelAlgorithm.HL;
    }
    else
    {
      this.lowerCoord = ct - 0.5 * thickness;
      this.higherCoord = cf + 0.5 * thickness;
      this.intervalType = OrthRoutingLevelAlgorithm.LH;
    }

    this.__outerThis._intervals[this.__outerThis.getIntervalID(segment)] = this;
    this.nrSmallerIntervals = 0;
    this.largerIntervals = new Array();
  },

  initFromNode : function(node)
  {
    var dir = this.__outerThis.getLevelFlowDir();
    var c1;
    var c2;

    if (node.isUpperBypassNode())
    {
      this.intervalType = OrthRoutingLevelAlgorithm.LL;
      this.firstSegment = node.getFirstSegmentFrom();
    }
    else
    {
      this.intervalType = OrthRoutingLevelAlgorithm.HH;
      this.firstSegment = node.getFirstSegmentTo();
    }

    this.secondSegment = node.getOpposite(this.firstSegment);

    if (this.intervalType == OrthRoutingLevelAlgorithm.HH)
    {
      c1 = this.__outerThis.getFromCoord(this.firstSegment, dir);
      c2 = this.__outerThis.getFromCoord(this.secondSegment, dir);
    }
    else
    {
      c1 = this.__outerThis.getToCoord(this.firstSegment, dir);
      c2 = this.__outerThis.getToCoord(this.secondSegment, dir);
    }

    if (c1 < c2)
    {
      this.lowerCoord = c1;
      this.higherCoord = c2;
    }
    else
    {
      this.lowerCoord = c2;
      this.higherCoord = c1;
    }

    this.__outerThis._intervals[this.__outerThis.getIntervalID(this.firstSegment)] = this;
    this.nrSmallerIntervals = 0;
    this.largerIntervals = new Array();
  },

  clean : function()
  {
    this.firstSegment = null;
    this.secondSegment = null;
    this.largerIntervals = null;
  },

  getLowerDeltaBound : function()
  {
    return this.lowerCoord - this.__outerThis._halfMinDelta; // Number
  },

  getHigherDeltaBound : function()
  {
    return this.higherCoord + this.__outerThis._halfMinDelta; // Number
  },

  setLayerNumber : function(layerNumber)
  {
    this.firstSegment.setOrthogonalBend(layerNumber);
    if (this.secondSegment != null) {
      this.secondSegment.setOrthogonalBend(layerNumber);
    }
  },

  setLayerNumberAtLeastTo : function(layerNumber)
  {
    if (layerNumber > this.firstSegment.getOrthogonalBend()) {
      this.setLayerNumber(layerNumber);
    }
  },

  getLayerNumber : function()
  {
    return this.firstSegment.getOrthogonalBend(); // Number
  }

}
);
OrthRoutingLevelSegmentIntervalLowerBoundSort = declare(ArrayStableSort, 
{

  constructor : function()
  {
  },
  
  compare : function(o1, o2)
  {
    return (o1).getLowerDeltaBound() <= (o2).getLowerDeltaBound(); // Boolean
  }

}
);
OrthRoutingLevelSegmentIntervalHigherBoundSort = declare(ArrayStableSort, 
{

  constructor : function()
  {
  },
  
  compare : function(o1, o2)
  {
    return (o1).getHigherDeltaBound() <= (o2).getHigherDeltaBound(); // Boolean
  }

}
);
OrthRoutingLevelIntervalIterator = declare(null, 
{
  hasNext : function()
  {
  },
  
  next : function()
  {
  }
  
});
/**
 * @private
 */
OrthRoutingLevelAlgorithmAnonClass_1 = declare(OrthRoutingLevelIntervalIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  i: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.i = this.__outerThis._segmentMarker.getElements();
  },

  hasNext : function()
  {
    return this.i.hasNext(); // Boolean
  },

  next : function()
  {
    var segment = this.i.next();

    return this.__outerThis._intervals[this.__outerThis.getIntervalID(segment)]; // ibm_ilog.graphlayout.hierarchical.OrthRoutingLevelSegmentInterval
  }

});

(function(){ OrthRoutingLevelAlgorithm._fields_init(); })();

return OrthRoutingLevelAlgorithm;
});
