/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HEnum",	// #5
  "../hierarchical/HLevelAlgorithm",	// #6
  "../hierarchical/HSegment",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HEnum,	// #5
    HLevelAlgorithm,	// #6
    HSegment,	// #7
    GraphModelUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelAlgorithm;
=====*/

var PolyLinkRepositionAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.PolyLinkRepositionAlgorithm", HLevelAlgorithm, 
{
  // summary:
  //		PolyLinkRepositionAlgorithm is responsable for fixing link
  //		overlappings because of to flat gradient. 
  // description:
  //		PolyLinkRepositionAlgorithm is responsable for fixing link
  //		overlappings because of to flat gradient. 
  //		This is only done for the links that are not orthogonal. Links
  //		starting at a small node may overlap a neighbored large node if the
  //		link gradient is too flat. In this case, we add an additional bend.
  //		Example:   &#95;/&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;                  / &#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;      |       
  //		    |                / |            |     /|            |            
  //		  |  |            |     / |            |  becomes      |  |          
  //		 |     .&#95;&#95;&#95;&#95;&#95;/. |            |         .&#95;&#95;&#95;&#95;&#95;|. |            |     |
  //		Node | | Large Node |         | Node | | Large Node |      Note the
  //		new bend in the link! The algorithm assumes that we have a proper
  //		hierarchy structure, the nodes are already placed, the adjacencies at
  //		the segments are sorted and the connector points are already
  //		calculated. This algorithm does not move any nodes and does not
  //		affect orthogonal links. It must run AFTER orthogonal link routing.
  //		For ease of explanation, we assume x-coordinate is the level flow
  //		direction and y-coordinate is the edge flow direction.  
  // tags:
  //		private
  
  _middleBetweenLevels: null,

  _minShiftOffset: null,

  _segments: null,

  _obstacleXCoordUpperLevel: null,
  _obstacleYCoordUpperLevel: null,
  _obstacleXCoordLowerLevel: null,
  _obstacleYCoordLowerLevel: null,
  _obstacleIsSegmentUpperLevel: null,
  _obstacleIsSegmentLowerLevel: null,

  _segmentsUpperLevelIndex: null,
  _segmentsLowerLevelIndex: null,

  constructor : function()
  {
  },
  
  init : function(upperLevel, lowerLevel)
  {
    // summary:
    //		Initializes the algorithm with the actual levels.
    // tags:
    //		private
    
    this.inherited("init", arguments, [upperLevel, lowerLevel]);

    var dir = this.getGraph().getEdgeFlow();
    this._minShiftOffset = 2.0;
    this._middleBetweenLevels = 0.5 * (upperLevel.getCoord(dir) + upperLevel.getSize(dir) + lowerLevel.getCoord(dir));
  },

  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._segments = null;
    this._obstacleXCoordUpperLevel = null;
    this._obstacleYCoordUpperLevel = null;
    this._obstacleXCoordLowerLevel = null;
    this._obstacleYCoordLowerLevel = null;
    this._obstacleIsSegmentUpperLevel = null;
    this._obstacleIsSegmentLowerLevel = null;
    this._segmentsUpperLevelIndex = null;
    this._segmentsLowerLevelIndex = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    this._minShiftOffset = this.getGraph().getMinDistBetweenNodeAndLink(this.getGraph().getEdgeFlow());
    this.generateSegmentIndexe();
    this.generateUpperLevelObstacleTable();
    this.generateLowerLevelObstacleTable();
    var changed = true;
    var aux = false;
    var oldMinShiftOffset;
    var numberOfRestarts = 0;
    var numberOfIterations = 0;

    while (changed)
    {
      numberOfIterations++;

      if (numberOfIterations > 100) {
        break;
      }
      if (numberOfIterations > 3) {
        this._minShiftOffset = 0.5 * this._minShiftOffset;
      }
      oldMinShiftOffset = this._minShiftOffset;
      changed = this.resolveConflicts();

      if (oldMinShiftOffset > this._minShiftOffset)
      {
        if (numberOfRestarts >= 3) {
          this._minShiftOffset = 0.0;
        }
        changed = true;
        this.fillUpperLevelObstacleTable();
        this.fillLowerLevelObstacleTable();
        numberOfRestarts++;
        numberOfIterations = 0;
      }
      else
      {
        aux = this.correctForkLinks();
        changed = aux || changed;
      }
    }

    this.createAllBends();
  },

  generateSegmentIndexe : function()
  {
    // summary:
    //		Generates the segment indexe. 
    // description:
    //		Generates the segment indexe. 
    //		Allocates the fields &#95;segmentsUpperLevelIndex and
    //		&#95;segmentsLowerLevelIndex. Each segment starting at the upper level
    //		or ending at the lower level has one index i. This index is stored
    //		in the sort value of the segment for faster access. For the segment
    //		with index i, the value &#95;segmentsUpperLevelIndex[i] is its index
    //		into &#95;obstacleX|YCoordUpperLevel, and the value
    //		&#95;segmentsLowerLevelIndex[i] is its index into
    //		&#95;obstacleX|YCoordLowerLevel. By this we can very fast retrieve how
    //		much a segment must be shifted away from the upper level, or from
    //		the lower level.
    // tags:
    //		private
    
    var iter;
    var segment;

    iter = this.getUpperLevel().getSegmentsFrom(true);

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.isGradientZeroInFlow()) {
        segment.setSortValue(-1.0);
      }
      else {
        segment.setSortValue(1.0);
      }
    }

    iter = this.getLowerLevel().getSegmentsTo(true);

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.isGradientZeroInFlow()) {
        segment.setSortValue(-1.0);
      }
      else {
        segment.setSortValue(1.0);
      }
    }

    var i = 0;
    iter = this.getUpperLevel().getSegmentsFrom(true);

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.getSortValue() >= 0.0)
      {
        i++;
        segment.setSortValue(-1.0);
      }
    }

    iter = this.getLowerLevel().getSegmentsTo(true);

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.getSortValue() >= 0.0)
      {
        i++;
        segment.setSortValue(-1.0);
      }
    }

    this._segmentsUpperLevelIndex = TranslateUtil.newArray(i, 0);
    this._segmentsLowerLevelIndex = TranslateUtil.newArray(i, 0);
    this._segments = TranslateUtil.newArray(i, null);

    for (i = 0; i < this._segmentsUpperLevelIndex.length; i++)
      this._segmentsUpperLevelIndex[i] = this._segmentsLowerLevelIndex[i] = -1;

    i = 0;
    iter = this.getUpperLevel().getSegmentsFrom(true);

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.getSortValue() < 0.0 && !segment.isGradientZeroInFlow())
      {
        segment.setSortValue(i);
        this._segments[i] = segment;
        i++;
      }
    }

    iter = this.getLowerLevel().getSegmentsTo(true);

    while (iter.hasNext())
    {
      segment = iter.next();

      if (segment.getSortValue() < 0.0 && !segment.isGradientZeroInFlow())
      {
        segment.setSortValue(i);
        this._segments[i] = segment;
        i++;
      }
    }
  },

  getUpperLevelIndex : function(segment)
  {
    // summary:
    //		Returns the index of the segment in the _obstacleX|YCoordUpperLevel
    //		field. 
    // description:
    //		Returns the index of the segment in the &#95;obstacleX|YCoordUpperLevel
    //		field. 
    //		Returns -1 if the segment does not start at the upper level, e.g.
    //		orthogonal bypass segments. Returns also -1 if the segment has
    //		gradient zero in flow direction.
    // tags:
    //		private
    
    var i = Math.floor(segment.getSortValue());
    if (i < 0 || i > this._segmentsUpperLevelIndex.length) {
      return -1; // int
    }
    return this._segmentsUpperLevelIndex[i]; // int
  },

  getLowerLevelIndex : function(segment)
  {
    // summary:
    //		Returns the index of the segment in the _obstacleX|YCoordLowerLevel
    //		field. 
    // description:
    //		Returns the index of the segment in the &#95;obstacleX|YCoordLowerLevel
    //		field. 
    //		Returns -1 if the segment does not end at the lower level, e.g.
    //		orthogonal bypass segments. Returns also -1 if the segment has
    //		gradient zero in flow direction.
    // tags:
    //		private
    
    var i = Math.floor(segment.getSortValue());
    if (i < 0 || i > this._segmentsLowerLevelIndex.length) {
      return -1; // int
    }
    return this._segmentsLowerLevelIndex[i]; // int
  },

  generateUpperLevelObstacleTable : function()
  {
    // summary:
    //		Generates the obstacle table for the upper level. 
    // description:
    //		Generates the obstacle table for the upper level. 
    //		The obstacle table is sorted according to the coordinate in level
    //		flow (here called &#95;obstacleXCoordUpperLevel). For each node, we
    //		generate two obstacle entries for both corner points that are
    //		adjacent to the level. For each segment, we generate one obstacle
    //		entry which is the start point of the segment. We store in
    //		&#95;segmentsUpperLevelIndex[i] the index of the corresponding entry of
    //		the segment with sort value i. Example: | Node |            * dummy
    //		node
    //		`------'             \ ^      ^            ^ \ link |      |            |   \ obstacles       obstacle \
    // tags:
    //		private
    
    var i = 2 * this.getUpperLevel().getNodesCount();
    var iter = this.getUpperLevel().getSegmentsFrom(true);
    var segment;

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.getSortValue() >= 0.0) {
        i++;
      }
    }

    this._obstacleXCoordUpperLevel = TranslateUtil.newArray(i, 0);
    this._obstacleYCoordUpperLevel = TranslateUtil.newArray(i, 0);
    this._obstacleIsSegmentUpperLevel = TranslateUtil.newArray(i, false);
    this.fillUpperLevelObstacleTable();
  },

  fillUpperLevelObstacleTable : function()
  {
    // summary:
    //		This actually fills the obstacle table for the upper level.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var ydir = this.getGraph().getEdgeFlow();
    var xdir = 1 - ydir;
    var index;
    var niter = this.getUpperLevel().getNodes();
    var node;
    var siter;
    var segment;
    var i = 0;

    while (niter.hasNext())
    {
      node = niter.next();

      if (!node.isEastWestPortAuxNode())
      {
        this._obstacleXCoordUpperLevel[i] = node.getCoord(xdir);
        this._obstacleYCoordUpperLevel[i] = node.getCoord(ydir) + node.getSize(ydir);
        this._obstacleIsSegmentUpperLevel[i] = false;
        i++;
      }

      siter = node.getAllSegmentsFrom(true, true, true, true);

      while (siter.hasNext())
      {
        segment = siter.next();
        index = Math.floor(segment.getSortValue());

        if (index >= 0)
        {
          this._segmentsUpperLevelIndex[index] = i;
          this._obstacleXCoordUpperLevel[i] = segment.getRealFromCoord(xdir);
          this._obstacleYCoordUpperLevel[i] = segment.getRealFromCoord(ydir);
          this._obstacleIsSegmentUpperLevel[i] = true;
          i++;
        }
      }

      if (!node.isEastWestPortAuxNode())
      {
        this._obstacleXCoordUpperLevel[i] = node.getCoord(xdir) + node.getSize(xdir);
        this._obstacleYCoordUpperLevel[i] = node.getCoord(ydir) + node.getSize(ydir);
        this._obstacleIsSegmentUpperLevel[i] = false;
        i++;
      }
    }
  },

  generateLowerLevelObstacleTable : function()
  {
    // summary:
    //		Generates the obstacle table for the lower level. 
    // description:
    //		Generates the obstacle table for the lower level. 
    //		The obstacle table is sorted according to the coordinate in level
    //		flow (here called &#95;obstacleXCoordLowerLevel). For each node, we
    //		generate two obstacle entries for both corner points that are
    //		adjacent to the level. For each segment, we generate one obstacle
    //		entry which is the start point of the segment. We store in
    //		&#95;segmentsLowerLevelIndex[i] the index of the corresponding entry of
    //		the segment with sort value i. Example: | Node |            * dummy
    //		node
    //		`------'             \ ^      ^            ^ \ link |      |            |   \ obstacles       obstacle \
    // tags:
    //		private
    
    var i = 2 * this.getLowerLevel().getNodesCount();
    var iter = this.getLowerLevel().getSegmentsTo(true);
    var segment;

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.getSortValue() >= 0.0) {
        i++;
      }
    }

    this._obstacleXCoordLowerLevel = TranslateUtil.newArray(i, 0);
    this._obstacleYCoordLowerLevel = TranslateUtil.newArray(i, 0);
    this._obstacleIsSegmentLowerLevel = TranslateUtil.newArray(i, false);
    this.fillLowerLevelObstacleTable();
  },

  fillLowerLevelObstacleTable : function()
  {
    // summary:
    //		This actually fills the obstacle table for the lower level.
    // tags:
    //		private
    
    var ydir = this.getGraph().getEdgeFlow();
    var xdir = 1 - ydir;
    var index;
    var niter = this.getLowerLevel().getNodes();
    var node;
    var siter;
    var segment;
    var i = 0;

    while (niter.hasNext())
    {
      node = niter.next();

      if (!node.isEastWestPortAuxNode())
      {
        this._obstacleXCoordLowerLevel[i] = node.getCoord(xdir);
        this._obstacleYCoordLowerLevel[i] = node.getCoord(ydir);
        this._obstacleIsSegmentLowerLevel[i] = false;
        i++;
      }

      siter = node.getAllSegmentsTo(true, true, true, true);

      while (siter.hasNext())
      {
        segment = siter.next();
        index = Math.floor(segment.getSortValue());

        if (index >= 0)
        {
          this._segmentsLowerLevelIndex[index] = i;
          this._obstacleXCoordLowerLevel[i] = segment.getRealToCoord(xdir);
          this._obstacleYCoordLowerLevel[i] = segment.getRealToCoord(ydir);
          this._obstacleIsSegmentLowerLevel[i] = true;
          i++;
        }
      }

      if (!node.isEastWestPortAuxNode())
      {
        this._obstacleXCoordLowerLevel[i] = node.getCoord(xdir) + node.getSize(xdir);
        this._obstacleYCoordLowerLevel[i] = node.getCoord(ydir);
        this._obstacleIsSegmentLowerLevel[i] = false;
        i++;
      }
    }
  },

  getUpperLevelYCoord : function(segment)
  {
    // summary:
    //		Returns the _obstacleYCoordUpperLevel of the input segment.
    // tags:
    //		private
    
    if (segment.isGradientZeroInFlow()) {
      return (0.0); // Number
    }
    var i = this.getUpperLevelIndex(segment);
    if (i < 0) {
      return (0.0); // Number
    }
    return this._obstacleYCoordUpperLevel[i]; // Number
  },

  getLowerLevelYCoord : function(segment)
  {
    // summary:
    //		Returns the _obstacleYCoordLowerLevel of the input segment.
    // tags:
    //		private
    
    if (segment.isGradientZeroInFlow()) {
      return (0.0); // Number
    }
    var i = this.getLowerLevelIndex(segment);
    if (i < 0) {
      return (0.0); // Number
    }
    return this._obstacleYCoordLowerLevel[i]; // Number
  },

  updateUpperLevelYCoord : function(segment, y)
  {
    // summary:
    //		Updates the _obstacleYCoordUpperLevel of the input segment. 
    // description:
    //		Updates the &#95;obstacleYCoordUpperLevel of the input segment. 
    //		This one should be increasing.
    // tags:
    //		private
    
    if (segment.isGradientZeroInFlow()) {
      return;
    }
    var i = this.getUpperLevelIndex(segment);
    if (i < 0) {
      return;
    }
    if (this._obstacleYCoordUpperLevel[i] < y) {
      this._obstacleYCoordUpperLevel[i] = y;
    }
  },

  updateLowerLevelYCoord : function(segment, y)
  {
    // summary:
    //		Updates the _obstacleYCoordLowerLevel of the input segment. 
    // description:
    //		Updates the &#95;obstacleYCoordLowerLevel of the input segment. 
    //		This one should be decreasing.
    // tags:
    //		private
    
    if (segment.isGradientZeroInFlow()) {
      return;
    }
    var i = this.getLowerLevelIndex(segment);
    if (i < 0) {
      return;
    }
    if (this._obstacleYCoordLowerLevel[i] > y) {
      this._obstacleYCoordLowerLevel[i] = y;
    }
  },

  resolveConflicts : function()
  {
    // summary:
    //		Solves all conflicts of all segments. 
    // description:
    //		Solves all conflicts of all segments. 
    //		It returns true if a conflict was found and solved.
    // tags:
    //		private
    
    var someConflictsSolved = false;
    var oldMinShiftOffset = this._minShiftOffset;

    for (var i = 0; i < this._segments.length; i++)
    {
      someConflictsSolved = this.resolveConflictsAtUpperLevel(this._segments[i]) || someConflictsSolved;
      someConflictsSolved = this.resolveConflictsAtLowerLevel(this._segments[i]) || someConflictsSolved;
      if (this._minShiftOffset < oldMinShiftOffset) {
        break;
      }
    }

    return someConflictsSolved; // Boolean
  },

  resolveConflictsAtUpperLevel : function(segment)
  {
    // summary:
    //		Solves the conflicts of one segment starting at the upper level. 
    // description:
    //		Solves the conflicts of one segment starting at the upper level. 
    //		It returns true if a conflict was found and solved.
    // tags:
    //		private
    
    if (segment.isGradientZeroInFlow()) {
      return false; // Boolean
    }
    var upperindex = this.getUpperLevelIndex(segment);
    if (upperindex < 0) {
      return false; // Boolean
    }
    var ydir = this.getGraph().getEdgeFlow();
    var xdir = 1 - ydir;
    var oldCoord = this._obstacleYCoordUpperLevel[upperindex];
    var x1 = segment.getRealFromCoord(xdir);
    var y1 = segment.getRealFromCoord(ydir);
    var x2 = segment.getRealToCoord(xdir);
    var y2 = segment.getRealToCoord(ydir);
    var lowerindex = this.getLowerLevelIndex(segment);
    if (lowerindex >= 0) {
      y2 = this._obstacleYCoordLowerLevel[lowerindex];
    }
    var increasing = (x2 > x1);
    var i = upperindex;

    while (true)
    {
      if (increasing)
      {
        i++;
        if (i >= this._obstacleYCoordUpperLevel.length) {
          break;
        }
        if (this._obstacleXCoordUpperLevel[i] >= x2) {
          break;
        }
      }
      else
      {
        i--;
        if (i < 0) {
          break;
        }
        if (this._obstacleXCoordUpperLevel[i] <= x2) {
          break;
        }
      }

      if (this.resolveUpperLevelConflict(segment, x1, y1, x2, y2, i)) {
        break;
      }
    }

    return (this._obstacleYCoordUpperLevel[upperindex] != oldCoord); // Boolean
  },

  resolveConflictsAtLowerLevel : function(segment)
  {
    // summary:
    //		Solves the conflicts of one segment starting at the lower level. 
    // description:
    //		Solves the conflicts of one segment starting at the lower level. 
    //		It returns true if a conflict was found and solved.
    // tags:
    //		private
    
    if (segment.isGradientZeroInFlow()) {
      return false; // Boolean
    }
    var lowerindex = this.getLowerLevelIndex(segment);
    if (lowerindex < 0) {
      return false; // Boolean
    }
    var ydir = this.getGraph().getEdgeFlow();
    var xdir = 1 - ydir;
    var oldCoord = this._obstacleYCoordLowerLevel[lowerindex];
    var x1 = segment.getRealToCoord(xdir);
    var y1 = segment.getRealToCoord(ydir);
    var x2 = segment.getRealFromCoord(xdir);
    var y2 = segment.getRealFromCoord(ydir);
    var upperindex = this.getUpperLevelIndex(segment);
    if (upperindex >= 0) {
      y2 = this._obstacleYCoordUpperLevel[upperindex];
    }
    var increasing = (x2 > x1);
    var i = lowerindex;

    while (true)
    {
      if (increasing)
      {
        i++;
        if (i >= this._obstacleYCoordLowerLevel.length) {
          break;
        }
        if (this._obstacleXCoordLowerLevel[i] >= x2) {
          break;
        }
      }
      else
      {
        i--;
        if (i < 0) {
          break;
        }
        if (this._obstacleXCoordLowerLevel[i] <= x2) {
          break;
        }
      }

      if (this.resolveLowerLevelConflict(segment, x1, y1, x2, y2, i)) {
        break;
      }
    }

    return (this._obstacleYCoordLowerLevel[lowerindex] != oldCoord); // Boolean
  },

  resolveUpperLevelConflict : function(segment, x1, y1, x2, y2, i)
  {
    // summary:
    //		Checks whether a segment of the upper level is in conflict with the
    //		i-th obstacle of the upper level. 
    // description:
    //		Checks whether a segment of the upper level is in conflict with the
    //		i-th obstacle of the upper level. 
    //		This updates the &#95;obstacleYCoordUpperLevel of the segment. As side
    //		effect, it also updates &#95;minShiftOffset if the shifting gets too
    //		large. It returns true if a restart is necessary because
    //		&#95;minShiftOffset has changed. The situation is this: y4 is to be
    //		calculated (x1,y1)  x3 |\  | | \ | obstacle |  \| y4\  | \|&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;
    //		y3 \\ (x2,y2)
    // tags:
    //		private
    
    var x3 = this._obstacleXCoordUpperLevel[i];
    var y3 = this._obstacleYCoordUpperLevel[i] + this._minShiftOffset;

    if (x3 >= x1 && x3 >= x2) {
      return false; // Boolean
    }
    if (x3 <= x1 && x3 <= x2) {
      return false; // Boolean
    }

    var xdir = this.getGraph().getLevelFlow();
    var ydir = 1 - xdir;
    var node = this.getGraph().getOwnedNode(segment.getFrom());
    var xmin = node.getCoord(xdir);
    var xmax = xmin + node.getSize(xdir);
    var ymax = node.getCoord(ydir) + node.getSize(ydir);

    if (this._obstacleIsSegmentUpperLevel[i] == true)
    {
      if (xmin <= x3 && x3 <= xmax && this._obstacleYCoordUpperLevel[i] <= ymax) {
        return false; // Boolean
      }
    }
    else
    {
      if (xmin <= x3 && x3 <= xmax) {
        return false; // Boolean
      }
    }

    var y4 = y2 - (y2 - y3) * (x2 - x1) / (x2 - x3);

    if (y4 > this._middleBetweenLevels)
    {
      this._minShiftOffset = 0.5 * this._minShiftOffset;
      return true; // Boolean
    }

    this.updateUpperLevelYCoord(segment, y4);
    return false; // Boolean
  },

  resolveLowerLevelConflict : function(segment, x1, y1, x2, y2, i)
  {
    // summary:
    //		Checks whether a segment of the lower level is in conflict with the
    //		i-th obstacle of the lower level. 
    // description:
    //		Checks whether a segment of the lower level is in conflict with the
    //		i-th obstacle of the lower level. 
    //		This updates the &#95;obstacleYCoordLowerLevel of the segment. As side
    //		effect, it also updates &#95;minShiftOffset if the shifting gets too
    //		large. It returns true if a restart is necessary because
    //		&#95;minShiftOffset has changed. The situation is this: y4 is to be
    //		calculated (x2,y2) // / / / &#95;/&#95;&#95;&#95;&#95;&#95;&#95;&#95; y3 y4/  | |  /| | / | obstacle
    //		|/  | (x1,y1)  x3
    // tags:
    //		private
    
    var x3 = this._obstacleXCoordLowerLevel[i];
    var y3 = this._obstacleYCoordLowerLevel[i] - this._minShiftOffset;

    if (x3 >= x1 && x3 >= x2) {
      return false; // Boolean
    }
    if (x3 <= x1 && x3 <= x2) {
      return false; // Boolean
    }

    var xdir = this.getGraph().getLevelFlow();
    var ydir = 1 - xdir;
    var node = this.getGraph().getOwnedNode(segment.getTo());
    var xmin = node.getCoord(xdir);
    var xmax = xmin + node.getSize(xdir);
    var ymin = node.getCoord(ydir);

    if (this._obstacleIsSegmentLowerLevel[i] == true)
    {
      if (xmin <= x3 && x3 <= xmax && this._obstacleYCoordLowerLevel[i] >= ymin) {
        return false; // Boolean
      }
    }
    else
    {
      if (xmin <= x3 && x3 <= xmax) {
        return false; // Boolean
      }
    }

    var y4 = y2 - (y2 - y3) * (x2 - x1) / (x2 - x3);

    if (y4 < this._middleBetweenLevels)
    {
      this._minShiftOffset = 0.5 * this._minShiftOffset;
      return true; // Boolean
    }

    this.updateLowerLevelYCoord(segment, y4);
    return false; // Boolean
  },

  correctForkLinks : function (a0, a1, a2, a3)
  {
    // summary:
    //		Correct the segments that start at the same coordinate. 
    // description:
    //		Correct the segments that start at the same coordinate. 
    //		This works because the &#95;obstacleYCoordUpper|LowerLevel are sorted in
    //		x-dir. Returns true if something changed.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.correctForkLinks_overload_0();
    else if(arguments.length == 4)
      return this.correctForkLinks_overload_4(a0, a1, a2, a3);
  },
  
  correctForkLinks_overload_0 : function()
                                          
  {
    var changed = false;

    changed = this.correctForkLinks(this._obstacleXCoordUpperLevel, this._obstacleYCoordUpperLevel, this._obstacleIsSegmentUpperLevel, true) || changed;
    changed = this.correctForkLinks(this._obstacleXCoordLowerLevel, this._obstacleYCoordLowerLevel, this._obstacleIsSegmentLowerLevel, false) || changed;

    return (changed); // Boolean
  },

  correctForkLinks_overload_4 : function(xarray, yarray, isSegment, increase)
  {
    // summary:
    //		Searches the areas in the sorted xarray where the values are equal. 
    // description:
    //		Searches the areas in the sorted xarray where the values are equal. 
    //		If increase is true, it sets yarray to the max of all yarray in such
    //		an area. If increase is false, it sets yarray to the min of all
    //		yarray in such an area. Returns true if something changed.
    // tags:
    //		private
    
    if (xarray.length == 0) {
      return (false); // Boolean
    }
    var changed = false;
    var prevX = xarray[0] - 1.0;
    var bestY;
    var startIndex = 0;
    var endIndex = 0;
    var j;

    for (var i = 0; i < xarray.length; i++)
    {
      if (xarray[i] == prevX) {
        endIndex = i;
      }
      else
      {
        if (endIndex > startIndex)
        {
          if (increase) {
            bestY = -/*Number.MAX_VALUE*/(3.4028235E38);
          }
          else {
            bestY = /*Number.MAX_VALUE*/(3.4028235E38);
          }
          for (j = startIndex; j <= endIndex; j++)
          {
            if (isSegment[j])
            {
              if (bestY < yarray[j] && increase) {
                bestY = yarray[j];
              }
              else if (bestY > yarray[j] && !increase) {
                bestY = yarray[j];
              }
            }
          }

          for (j = startIndex; j <= endIndex; j++)
            if (isSegment[j])
            {
              changed = (yarray[j] != bestY) || changed;
              yarray[j] = bestY;
            }
        }

        startIndex = endIndex = i;
        prevX = xarray[i];
      }
    }

    return (changed); // Boolean
  },

  createAllBends : function()
  {
    // summary:
    //		Create the bends of all segments.
    // tags:
    //		private
    
    for (var i = 0; i < this._segments.length; i++)
      this.createBends(this._segments[i]);
  },

  createBends : function(segment)
  {
    // summary:
    //		Bend creation for one segment.
    // tags:
    //		private
    
    if (segment.isGradientZeroInFlow()) {
      return;
    }
    var needsReclip = false;
    var upperIndex = this.getUpperLevelIndex(segment);
    var lowerIndex = this.getLowerLevelIndex(segment);
    var ydir = this.getGraph().getEdgeFlow();
    var xdir = 1 - ydir;
    var link = segment.getOwnerLink();
    var dummyNode = null;
    var yUpper = this.getUpperLevelYCoord(segment);
    var yLower = this.getLowerLevelYCoord(segment);

    if (upperIndex >= 0 && yUpper > segment.getRealFromCoord(ydir) && !segment.getFrom().isUpperBypassNode())
    {
      if (!segment.getFrom().isDummyNode()) {
        needsReclip = true;
      }
      dummyNode = link.addDummyNodeAtGraph(segment, this.getGraph());
      dummyNode.setCoord(xdir, this._obstacleXCoordUpperLevel[upperIndex]);
      dummyNode.setCoord(ydir, this._obstacleYCoordUpperLevel[upperIndex]);
      if (!segment.isReversed()) {
        segment = dummyNode.getIntergraphOpposite(segment);
      }
    }

    if (lowerIndex >= 0 && yLower < segment.getRealToCoord(ydir) && !segment.getTo().isLowerBypassNode())
    {
      if (!segment.getTo().isDummyNode()) {
        needsReclip = true;
      }
      dummyNode = link.addDummyNodeAtGraph(segment, this.getGraph());
      dummyNode.setCoord(xdir, this._obstacleXCoordLowerLevel[lowerIndex]);
      dummyNode.setCoord(ydir, this._obstacleYCoordLowerLevel[lowerIndex]);
    }

    if (this.getGraph().getConnectorStyle() == HEnum.CLIPPED_PINS && needsReclip)
    {
      var connectorAlg;
      connectorAlg = link.getFrom().getOwnerGraph().getConnectorAlgorithm();
      connectorAlg.calcFromClippedConnector(link);
      connectorAlg = link.getTo().getOwnerGraph().getConnectorAlgorithm();
      connectorAlg.calcToClippedConnector(link);
    }
  }

}
);

return PolyLinkRepositionAlgorithm;
});
