/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/CrossRedSelfLinkAlgorithm",	// #5
  "../hierarchical/HEnum",	// #6
  "../hierarchical/HGraphAlgorithm",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    CrossRedSelfLinkAlgorithm,	// #5
    HEnum,	// #6
    HGraphAlgorithm,	// #7
    GraphModelUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var SelfLinkAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.SelfLinkAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		SelfLinkAlgorithm handles everything related to self-links. 
  // description:
  //		SelfLinkAlgorithm handles everything related to self-links. 
  //		Self-links are removed at the beginning of layout and reinserted at
  //		the end of layout. All self-links are always routed orthogonally and
  //		use evenly spaced connector mode, simply because this looks best. All
  //		self-links have from port and to port sides.  
  // tags:
  //		private
  
  _removedSelfLinks : null,
  _nodesWithSelfLinks : null,

  constructor : function(graph)
  {
    this._removedSelfLinks = new Array();
    this._nodesWithSelfLinks = new Array();
    this.init(graph);
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
  },

  removeSelfLinks : function()
  {
    // summary:
    //		Remove all self-links.
    // tags:
    //		private
    
    var portAuxNodesToCorrect = new Array();
    var levelsOutOfDate = new Array();
    var graph = this.getGraph();
    var levelFlowDir = graph.getLevelFlow();

    var iter = graph.getLinks();

    while (iter.hasNext())
    {
      var link = iter.next();
      var from = link.getFrom();
      var toobj = link.getTo();
      var fromOwner = null;
      var toOwner = null;

      if (from.isEastWestPortAuxNode()) {
        fromOwner = from.getEastWestPortAuxOwner();
      }
      else {
        fromOwner = from;
      }
      if (toobj.isEastWestPortAuxNode()) {
        toOwner = toobj.getEastWestPortAuxOwner();
      }
      else {
        toOwner = toobj;
      }
      if (fromOwner == toOwner)
      {
        var owner = fromOwner;

        if (owner.getEastPortAuxNode() == from) {
          link.setFromPortSide(HEnum.EAST);
        }
        else if (owner.getWestPortAuxNode() == from) {
          link.setFromPortSide(HEnum.WEST);
        }
        if (owner.getEastPortAuxNode() == toobj) {
          link.setToPortSide(HEnum.EAST);
        }
        else if (owner.getWestPortAuxNode() == toobj) {
          link.setToPortSide(HEnum.WEST);
        }
        TranslateUtil.addSetObject(this._nodesWithSelfLinks, owner);
        this._removedSelfLinks.push(link);
        owner.registerSelfLink(link);
        this.calcSelfLinkDummyNodes(link);

        var ssegment = link.getStartSegment();
        var esegment = link.getEndSegment();

        if (from != owner)
        {
          TranslateUtil.addSetObject(portAuxNodesToCorrect, from);

          if (ssegment.getFrom() == from)
          {
            graph.removeEdge(ssegment);
            graph.addEdgeBetweenNodes(ssegment, owner, ssegment.getTo());
          }
          else if (ssegment.getTo() == from)
          {
            graph.removeEdge(ssegment);
            graph.addEdgeBetweenNodes(ssegment, ssegment.getFrom(), owner);
          }

          if (esegment.getTo() == from)
          {
            graph.removeEdge(esegment);
            graph.addEdgeBetweenNodes(esegment, esegment.getFrom(), owner);
          }
          else if (esegment.getTo() == from)
          {
            graph.removeEdge(esegment);
            graph.addEdgeBetweenNodes(esegment, owner, esegment.getTo());
          }
        }

        if (toobj != owner)
        {
          TranslateUtil.addSetObject(portAuxNodesToCorrect, toobj);

          if (ssegment.getFrom() == toobj)
          {
            graph.removeEdge(ssegment);
            graph.addEdgeBetweenNodes(ssegment, owner, ssegment.getTo());
          }
          else if (ssegment.getTo() == toobj)
          {
            graph.removeEdge(ssegment);
            graph.addEdgeBetweenNodes(ssegment, ssegment.getFrom(), owner);
          }

          if (esegment.getTo() == toobj)
          {
            graph.removeEdge(esegment);
            graph.addEdgeBetweenNodes(esegment, esegment.getFrom(), owner);
          }
          else if (esegment.getTo() == toobj)
          {
            graph.removeEdge(esegment);
            graph.addEdgeBetweenNodes(esegment, owner, esegment.getTo());
          }
        }

        var niter = link.getNodes();

        while (niter.hasNext())
        {
          var dummy = niter.next();
          if (dummy.getLevel() != null) {
            TranslateUtil.addSetObject(levelsOutOfDate, dummy.getLevel());
          }
        }

        graph.removeLink(link);
      }
    }

    var node;

    var iter2 =  new TranslateUtil.ArrayIterator(portAuxNodesToCorrect);

    while (iter2.hasNext())
    {
      node = iter2.next();

      if (node.getSegmentsFromCount() == 0 && node.getSegmentsToCount() == 0)
      {
        TranslateUtil.addSetObject(levelsOutOfDate, node.getLevel());
        graph.removeNode(node);
      }
      else
      {
        node.setSize(levelFlowDir, graph.getMinDistBetweenLinks(levelFlowDir));
        node.correctEastWestAuxSize();
      }
    }

    iter2 =  new TranslateUtil.ArrayIterator(levelsOutOfDate);

    while (iter2.hasNext())
    {
      var level = iter2.next();
      level.markInfoOutOfDate();
      level.updateInfo();
    }

    iter2 =  new TranslateUtil.ArrayIterator(this._nodesWithSelfLinks);

    while (iter2.hasNext())
    {
      node = iter2.next();
      node.getSelfLinkInfo().decideLinkSidesOfLinksBypassingNodes(node);
    }

    this.calcMinDistThicknessFactorAndOffsets();
  },

  reinsertSelfLinks : function (a0)
  {
    // summary:
    //		Reinsert all self-links.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.reinsertSelfLinks_overload_0();
    else if(arguments.length == 1)
      return this.reinsertSelfLinks_overload_1(a0);
  },
  
  reinsertSelfLinks_overload_0 : function()
                                          
  {
    this.reinsertSelfLinks(true);
  },

  reinsertSelfLinks_overload_1 : function(reroute)
  {
    // summary:
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter =  new TranslateUtil.ArrayIterator(this._removedSelfLinks);

    while (iter.hasNext())
    {
      var link = iter.next();
      graph.addLink(link);
      if (reroute) {
        this.routeSelfLink(link);
      }
    }
  },

  reRemoveSelfLinks : function()
  {
    // summary:
    //		Reremove all self-links.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter =  new TranslateUtil.ArrayIterator(this._removedSelfLinks);

    while (iter.hasNext())
    {
      var link = iter.next();
      graph.removeLink(link);
    }
  },

  calcSelfLinkDummyNodes : function(link)
  {
    // summary:
    //		Calculate enough dummy nodes per self-link. 
    // description:
    //		Calculate enough dummy nodes per self-link. 
    //		This also normalizes the self-link again.
    // tags:
    //		private
    
    var numDummyNodes = 3;
    var fromPortSide = link.getFromPortSide();
    var toPortSide = link.getToPortSide();
    if (fromPortSide == toPortSide) {
      numDummyNodes = 2;
    }
    if ((fromPortSide + 2) % 4 == toPortSide) {
      numDummyNodes = 4;
    }

    var reversedSegments = new Array();
    var dummy = null;
    var segment = link.getStartSegment();
    if (segment.getTo().isDummyNode()) {
      dummy = segment.getTo();
    }
    else if (segment.getFrom().isDummyNode()) {
      dummy = segment.getFrom();
    }
    while (segment != null)
    {
      if (segment.isReversed()) {
        TranslateUtil.addSetObject(reversedSegments, segment);
      }
      if (dummy != null)
      {
        segment = dummy.getIntergraphOpposite(segment);
        dummy = segment.getOpposite(dummy);
        if (!dummy.isDummyNode()) {
          dummy = null;
        }
      }
      else
      {
        segment = null;
      }
    }

    var siter =  new TranslateUtil.ArrayIterator(reversedSegments);

    while (siter.hasNext())
    {
      segment = siter.next();
      segment.reverse();
    }

    var n = link.getNumberOfNodes();

    while (n < numDummyNodes)
    {
      link.addDummyNode(link.getStartSegment());
      n++;
    }

    var iter = link.getNodes();

    while (iter.hasNext())
    {
      var node = iter.next();
      node.setSize(0, 0);
      node.setSize(1, 0);
      node.setCoord(0, 300);
      node.setCoord(1, 300);
    }
  },

  calcMinDistThicknessFactorAndOffsets : function()
  {
    // summary:
    //		Calculates the parameters for evenly spaced connectors.
    // tags:
    //		private
    
    var graph = this.getGraph();
    if (graph.getConnectorStyle() != HEnum.EVENLY_SPACED_PINS) {
      return;
    }
    var connectorAlg = graph.getConnectorAlgorithm();
    var iter =  new TranslateUtil.ArrayIterator(this._nodesWithSelfLinks);

    while (iter.hasNext())
    {
      var node = iter.next();
      connectorAlg.calcMinDistThicknessFactorAndOffsets(node, HEnum.NORTH);
      connectorAlg.calcMinDistThicknessFactorAndOffsets(node, HEnum.SOUTH);
      connectorAlg.calcMinDistThicknessFactorAndOffsets(node, HEnum.EAST);
      connectorAlg.calcMinDistThicknessFactorAndOffsets(node, HEnum.WEST);
    }
  },

  calcSelfLinkConnectors : function()
  {
    // summary:
    //		Calculates the self-link connectors.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var connectorAlg = graph.getConnectorAlgorithm();
    var iter =  new TranslateUtil.ArrayIterator(this._nodesWithSelfLinks);

    while (iter.hasNext())
    {
      var node = iter.next();
      connectorAlg.calcSelfLinkConnectors(node);
    }
  },

  calcSelfLinkDistances : function()
  {
    // summary:
    //		Calculates the self-link distances from the node border.
    // tags:
    //		private
    
    this.reinsertSelfLinks(false);
    var alg = new CrossRedSelfLinkAlgorithm(this.getGraph());
    var iter =  new TranslateUtil.ArrayIterator(this._nodesWithSelfLinks);

    while (iter.hasNext())
    {
      var node = iter.next();
      alg.initNodeSide(node, HEnum.EAST);
      alg.run();
      alg.initNodeSide(node, HEnum.WEST);
      alg.run();
      alg.initNodeSide(node, HEnum.NORTH);
      alg.run();
      alg.initNodeSide(node, HEnum.SOUTH);
      alg.run();
    }

    this.reRemoveSelfLinks();
  },

  routeSelfLink : function(link)
  {
    // summary:
    //		Routes a self-link. 
    // description:
    //		Routes a self-link. 
    //		The distances from the node border is in the orthogonal bend
    //		coordinate. The connection points of the link are already calculated
    //		and the node has its final position.
    // tags:
    //		private
    
    var node = link.getFrom();
    var dummy;
    var dir = 0;
    var borderCoord = 0;
    var dist;
    var sign = 1;
    var startsEastWest = false;
    var segment = link.getStartSegment();

    switch (link.getFromPortSide())
    {
      case HEnum.EAST:
        dir = this.getGraph().getLevelFlow();
        borderCoord = node.getCoord(dir) + node.getSize(dir);
        sign = 1;
        startsEastWest = true;
        break;
      case HEnum.WEST:
        dir = this.getGraph().getLevelFlow();
        borderCoord = node.getCoord(dir);
        sign = -1;
        startsEastWest = true;
        break;
      case HEnum.NORTH:
        dir = this.getGraph().getEdgeFlow();
        borderCoord = node.getCoord(dir);
        sign = -1;
        startsEastWest = false;
        break;
      case HEnum.SOUTH:
        dir = this.getGraph().getEdgeFlow();
        borderCoord = node.getCoord(dir) + node.getSize(dir);
        sign = 1;
        startsEastWest = false;
        break;
    }
    dir = 1 - dir;
    dummy = segment.getTo();
    dummy.setCoord(dir, link.getFromCoord(dir));
    segment = dummy.getIntergraphOpposite(segment);
    dist = segment.getOrthogonalBend();
    dir = 1 - dir;
    segment.getFrom().setCoord(dir, borderCoord + sign * dist);
    segment.getTo().setCoord(dir, borderCoord + sign * dist);
    segment = link.getEndSegment();

    switch (link.getToPortSide())
    {
      case HEnum.EAST:
        dir = this.getGraph().getLevelFlow();
        borderCoord = node.getCoord(dir) + node.getSize(dir);
        sign = 1;
        break;
      case HEnum.WEST:
        dir = this.getGraph().getLevelFlow();
        borderCoord = node.getCoord(dir);
        sign = -1;
        break;
      case HEnum.NORTH:
        dir = this.getGraph().getEdgeFlow();
        borderCoord = node.getCoord(dir);
        sign = -1;
        break;
      case HEnum.SOUTH:
        dir = this.getGraph().getEdgeFlow();
        borderCoord = node.getCoord(dir) + node.getSize(dir);
        sign = 1;
        break;
    }
    dir = 1 - dir;
    dummy = segment.getFrom();
    dummy.setCoord(dir, link.getToCoord(dir));
    segment = dummy.getIntergraphOpposite(segment);
    dist = segment.getOrthogonalBend();
    dir = 1 - dir;
    segment.getFrom().setCoord(dir, borderCoord + sign * dist);
    segment.getTo().setCoord(dir, borderCoord + sign * dist);

    if (Math.abs(link.getFromPortSide() - link.getToPortSide()) == 2)
    {
      if (startsEastWest)
      {
        dir = this.getGraph().getEdgeFlow();

        if (node.getSelfLinkInfo().isEastWestGoesNorth())
        {
          borderCoord = node.getCoord(dir);
          sign = -1;
        }
        else
        {
          borderCoord = node.getCoord(dir) + node.getSize(dir);
          sign = 1;
        }
      }
      else
      {
        dir = this.getGraph().getLevelFlow();

        if (node.getSelfLinkInfo().isNorthSouthGoesEast())
        {
          borderCoord = node.getCoord(dir) + node.getSize(dir);
          sign = 1;
        }
        else
        {
          borderCoord = node.getCoord(dir);
          sign = -1;
        }
      }

      var auxs = link.getStartSegment();
      var auxn = auxs.getTo();
      auxs = auxn.getIntergraphOpposite(auxs);
      auxn = auxs.getOpposite(auxn);
      segment = auxn.getIntergraphOpposite(auxs);
      dist = segment.getOrthogonalBend();
      segment.getFrom().setCoord(dir, borderCoord + sign * dist);
      segment.getTo().setCoord(dir, borderCoord + sign * dist);
    }
  }

}
);

return SelfLinkAlgorithm;
});
