/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HLevelSweepAlgorithm",	// #5
  "../hierarchical/HPercentages",	// #6
  "../hierarchical/HSegmentSort",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLevelSweepAlgorithm,	// #5
    HPercentages,	// #6
    HSegmentSort,	// #7
    GraphModelUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelSweepAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelSweepAlgorithm;
=====*/

var SortAdjByCoordAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.SortAdjByCoordAlgorithm", HLevelSweepAlgorithm, 
{
  // summary:
  //		SortAdjByCoordAlgorithm traverses the levels and sorts the
  //		adjacencies of the nodes at the north and south side of each wrt. 
  // description:
  //		SortAdjByCoordAlgorithm traverses the levels and sorts the
  //		adjacencies of the nodes at the north and south side of each wrt. 
  //		coordinates of connection points in level flow direction. Different
  //		to the class SortAdjacenciesAlgorithm, this sorting is done AFTER
  //		each link connection point is calculated (therefore the special
  //		handling of upper and lower bypass nodes is unnecessary here). It is
  //		necessary to sort the adjacencies again, because port index
  //		specifications may mess up the result of SortAdjacencyAlgorithm, but
  //		orthogonal routing strictly requires sorted adjacencies. As
  //		additional problem, segments of the same port index start at the same
  //		connection point. We need them sorted according to the position index
  //		of the node on the other side, to get a good result in orthogonal
  //		routing. So, the sort criteria is: a) from|toCoord(levelFlowDir) b)
  //		if from|toCoord(levelFlowDir) are equal, then
  //		segment.getOpposite(node).positionInLevel  When we come to here, all
  //		segments have already been sorted once. Therefore we can access
  //		getAllSortedSegments().  
  // tags:
  //		private
  
  _segmentSortAlg: null,
  _levelFlowDirection: null,

  constructor : function(graph)
  {
    this.init(graph);
    this._segmentSortAlg = new HSegmentSort();
    this._levelFlowDirection = graph.getLevelFlow();
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._segmentSortAlg = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var pts = 2 * (graph.getNumberOfLevels() - 1);
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_SORT_ADJ_BYCOORD], pts, false);
    this.sweepForward();
    this.sweepBackward();
  },

  sortFromSegments : function(level)
  {
    // summary:
    //		Sort all "from" segments in the level.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var nodeIter = level.getNodes();
    var node;
    var othernode;
    var segmentIter;
    var segment;
    var mindelta;
    var coord;

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();

      segmentIter = node.getAllSortedSegmentsFrom();

      while (segmentIter.hasNext())
      {
        segment = segmentIter.next();
        segment.setSortValue(segment.getRawFromCoord(this._levelFlowDirection));
      }

      node.sortFromSegments(this._segmentSortAlg);

      mindelta = this.getMinDelta(node, node.getAllSortedSegmentsFrom());

      segmentIter = node.getAllSortedSegmentsFrom();

      while (segmentIter.hasNext())
      {
        segment = segmentIter.next();
        coord = segment.getRawFromCoord(this._levelFlowDirection);
        othernode = graph.getOwnedNode(segment.getTo());

        if (othernode != null) {
          coord += mindelta / (othernode.getPositionInLevel() + 2);
        }
        segment.setSortValue(coord);
      }

      node.sortFromSegments(this._segmentSortAlg);
    }
  },

  sortToSegments : function(level)
  {
    // summary:
    //		Sort all "to" segments in the level.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var nodeIter = level.getNodes();
    var node;
    var othernode;
    var segmentIter;
    var segment;
    var mindelta;
    var coord;

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();

      segmentIter = node.getAllSortedSegmentsTo();

      while (segmentIter.hasNext())
      {
        segment = segmentIter.next();
        segment.setSortValue(segment.getRawToCoord(this._levelFlowDirection));
      }

      node.sortToSegments(this._segmentSortAlg);

      mindelta = this.getMinDelta(node, node.getAllSortedSegmentsTo());

      segmentIter = node.getAllSortedSegmentsTo();

      while (segmentIter.hasNext())
      {
        segment = segmentIter.next();
        coord = segment.getRawToCoord(this._levelFlowDirection);
        othernode = graph.getOwnedNode(segment.getFrom());

        if (othernode != null) {
          coord += mindelta / (othernode.getPositionInLevel() + 2);
        }
        segment.setSortValue(coord);
      }

      node.sortToSegments(this._segmentSortAlg);
    }
  },

  getMinDelta : function(node, iter)
  {
    // summary:
    //		Returns the smallest distance between two sort values that are not
    //		equal. 
    // description:
    //		Returns the smallest distance between two sort values that are not
    //		equal. 
    //		At this time point, the sort values are sorted.
    // tags:
    //		private
    
    var mindelta = node.getSize(this._levelFlowDirection) + 1.0;
    var prevcoord = node.getCoord(this._levelFlowDirection) - 1.0;
    var coord;

    while (iter.hasNext())
    {
      coord = iter.next().getSortValue();
      if (coord - prevcoord > 0.0 && coord - prevcoord < mindelta) {
        mindelta = coord - prevcoord;
      }
      prevcoord = coord;
    }

    return mindelta; // Number
  },

  treatForwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number 1 ... 
    // description:
    //		Treats a level (with level number 1 ... 
    //		n) in a forward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i-1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.sortFromSegments(prevLevel);
  },

  treatBackwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number n-1 ... 
    // description:
    //		Treats a level (with level number n-1 ... 
    //		0) in a backward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i+1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.sortToSegments(prevLevel);
  }

}
);

return SortAdjByCoordAlgorithm;
});
