/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HLevelSweepAlgorithm",	// #5
  "../hierarchical/HPercentages",	// #6
  "../hierarchical/HSegmentSort",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLevelSweepAlgorithm,	// #5
    HPercentages,	// #6
    HSegmentSort,	// #7
    GraphModelUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HLevelSweepAlgorithm = ibm_ilog.graphlayout.hierarchical.HLevelSweepAlgorithm;
=====*/

var SortAdjacenciesAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.SortAdjacenciesAlgorithm", HLevelSweepAlgorithm, 
{
  // summary:
  //		SortAdjacenciesAlgorithm traverses the levels and sorts the
  //		adjacencies of the nodes wrt. 
  // description:
  //		SortAdjacenciesAlgorithm traverses the levels and sorts the
  //		adjacencies of the nodes wrt. 
  //		node positions. Different to SortAdjByCoordAlgorithm, this is done
  //		before the connection points are calculated. Therefore we cannot sort
  //		according to connection point coordinates, but rather conversely, we
  //		want the connection point coordinates calculated after this sorting. 
  //		
  // tags:
  //		private
  
  _segmentSortAlg: null,
  _levelFlowDirection: null,
  _hasOrthogonalLinks: null,

  constructor : function(graph)
  {
    this.init(graph);
    this._segmentSortAlg = new HSegmentSort();
    this._levelFlowDirection = graph.getLevelFlow();
    this._hasOrthogonalLinks = graph.hasOrthogonalLinks();
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._segmentSortAlg = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var points = 2 * (graph.getNumberOfLevels() - 1);
    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_SORT_ADJACENCIES], points, false);
    this.sweepForward();
    this.sweepBackward();

    this.sweepForward();
  },

  calcFromSegmentSortValue : function(level)
  {
    // summary:
    //		Store the node position as sort value in the "from" segments.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var nodeIter = level.getNodes();
    var node;
    var prevNode = null;
    var segmentIter;
    var segment;
    var sortValueBase;
    var sortValueWidth;
    var delta;
    var i;
    var segmentsCount;
    var maxPort;
    var port;
    var reversePorts;

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();

      if (this._hasOrthogonalLinks && node.isUpperBypassNode()) {
        this.calcFromSegmentBypassSortValue(node);
      }
      else
      {
        if (prevNode != null && prevNode.getEastPortAuxNode() == node) {
          reversePorts = true;
        }
        else {
          reversePorts = false;
        }
        segmentIter = node.getAllSegmentsFrom(true, true, true, true);
        maxPort = 0;
        segmentsCount = 0;

        while (segmentIter.hasNext())
        {
          segment = segmentIter.next();

          if (segment.getFrom() == node)
          {
            port = segment.getFromPortNumber();
            if (port > maxPort) {
              maxPort = port;
            }
          }

          segmentsCount++;
        }

        sortValueBase = node.getCoord(this._levelFlowDirection);
        sortValueWidth = node.getSize(this._levelFlowDirection);
        delta = sortValueWidth / (segmentsCount + maxPort + 2);
        segmentIter = node.getAllSegmentsFrom(true, true, true, true);
        i = 0.0;

        while (segmentIter.hasNext())
        {
          segment = segmentIter.next();

          if (segment.getFrom() == node)
          {
            port = segment.getFromPortNumber();

            if (port >= 0)
            {
              if (reversePorts) {
                segment.setSortValue(sortValueBase + (segmentsCount + 1 + maxPort - port) * delta);
              }
              else {
                segment.setSortValue(sortValueBase + (segmentsCount + 1 + port) * delta);
              }
            }
            else
            {
              segment.setSortValue(sortValueBase + i * delta);
              i += 1.0;
            }
          }
          else
          {
            var coord = graph.getTranslatedCoord(segment.getFrom(), segment.getFrom().getCoord(this._levelFlowDirection), this._levelFlowDirection);
            segment.setSortValue(coord);
            i += 1.0;
          }
        }
      }

      prevNode = node;
    }
  },

  calcToSegmentSortValue : function(level)
  {
    // summary:
    //		Store the node position as sort value in the "to" segments.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var nodeIter = level.getNodes();
    var node;
    var prevNode = null;
    var segmentIter;
    var segment;
    var sortValueBase;
    var sortValueWidth;
    var delta;
    var i;
    var segmentsCount;
    var maxPort;
    var port;
    var reversePorts;

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();

      if (this._hasOrthogonalLinks && node.isLowerBypassNode()) {
        this.calcToSegmentBypassSortValue(node);
      }
      else
      {
        if (node.isEastWestPortAuxNode() && (prevNode == null || prevNode.getEastPortAuxNode() != node)) {
          reversePorts = true;
        }
        else {
          reversePorts = false;
        }
        segmentIter = node.getAllSegmentsTo(true, true, true, true);
        maxPort = 0;
        segmentsCount = 0;

        while (segmentIter.hasNext())
        {
          segment = segmentIter.next();

          if (segment.getTo() == node)
          {
            port = segment.getToPortNumber();
            if (port > maxPort) {
              maxPort = port;
            }
          }

          segmentsCount++;
        }

        sortValueBase = node.getCoord(this._levelFlowDirection);
        sortValueWidth = node.getSize(this._levelFlowDirection);
        delta = sortValueWidth / (segmentsCount + maxPort + 2);
        segmentIter = node.getAllSegmentsTo(true, true, true, true);
        i = 0.0;

        while (segmentIter.hasNext())
        {
          segment = segmentIter.next();

          if (segment.getTo() == node)
          {
            port = segment.getToPortNumber();

            if (port >= 0)
            {
              if (reversePorts) {
                segment.setSortValue(sortValueBase + (segmentsCount + 1 + maxPort - port) * delta);
              }
              else {
                segment.setSortValue(sortValueBase + (segmentsCount + 1 + port) * delta);
              }
            }
            else
            {
              segment.setSortValue(sortValueBase + i * delta);
              i += 1.0;
            }
          }
          else
          {
            var coord = graph.getTranslatedCoord(segment.getTo(), segment.getTo().getCoord(this._levelFlowDirection), this._levelFlowDirection);
            segment.setSortValue(coord);
            i += 1.0;
          }
        }
      }

      prevNode = node;
    }
  },

  correctSortValueAtMultiAuxPortNodes : function(level)
  {
    // summary:
    //		This fixes the sort value at the "from" side for links that are
    //		between pairs of aux nodes and have ports at both sides. 
    // description:
    //		This fixes the sort value at the "from" side for links that are
    //		between pairs of aux nodes and have ports at both sides. 
    //		In this case, we consider the  "to" port number rather than the
    //		"from" port number in order to avoid that multilinks between both
    //		aux nodes are twisted multiple times.   The problem is basically the
    //		following: Assume we have to east side links connect from port 0 to
    //		port 0, and from port 1 to port 1. If we
    //		don't do the correction here, the result would look like this &#95;&#95;&#95;                                          &#95;&#95;&#95; |   |----.                                   |   |----. |&#95;&#95;&#95;|----+---.        but with correction    |&#95;&#95;&#95;|----+---. |&#95;&#95;&#95;|&#95;&#95;&#95;     it will look like               |   | &#95;&#95;&#95;         |   |    this:                   &#95;&#95;&#95;     |   | |   |--------+---'
    //		|   |----'   | |&#95;&#95;&#95;|--------'               
    //		|&#95;&#95;&#95;|--------' It is important that we do this correction only at one side, i.e here at the "from" side, not at the "to" side. The input level is the level of nodes at the "to" side of the segments to be sorted.
    // tags:
    //		private
    
    var nodeIter = level.getNodes();
    var fromNode;
    var node;
    var prevNode = null;
    var segmentIter;
    var segment;
    var sortValueBase;
    var sortValueWidth;
    var delta;
    var segmentsCount;
    var maxPort;
    var fromPort;
    var toPort;
    var reversePorts;

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();

      if (node.isEastWestPortAuxNode())
      {
        if (prevNode == null || prevNode.getEastPortAuxNode() != node) {
          reversePorts = true;
        }
        else {
          reversePorts = false;
        }
        segmentIter = node.getSegmentsTo();
        maxPort = 0;

        while (segmentIter.hasNext())
        {
          toPort = segmentIter.next().getToPortNumber();
          if (toPort > maxPort) {
            maxPort = toPort;
          }
        }

        segmentIter = node.getSegmentsTo();

        while (segmentIter.hasNext())
        {
          segment = segmentIter.next();
          fromNode = segment.getFrom();
          fromPort = segment.getFromPortNumber();
          toPort = segment.getToPortNumber();

          if (fromPort >= 0 && toPort >= 0 && fromNode.isEastWestPortAuxNode())
          {
            sortValueBase = fromNode.getCoord(this._levelFlowDirection);
            sortValueWidth = fromNode.getSize(this._levelFlowDirection);
            segmentsCount = fromNode.getSegmentsFromCount();
            delta = sortValueWidth / (segmentsCount + maxPort + 2);
            if (reversePorts) {
              segment.setSortValue(sortValueBase + (segmentsCount + 1 + maxPort - toPort) * delta);
            }
            else {
              segment.setSortValue(sortValueBase + (segmentsCount + 1 + toPort) * delta);
            }
          }
        }
      }

      prevNode = node;
    }
  },

  calcFromSegmentBypassSortValue : function(node)
  {
    // summary:
    //		Calculate the sort value of a bypass dummy node for orthogonal
    //		layout.
    // tags:
    //		private
    
    var level = node.getLevel();
    var segment = node.getFirstSegmentFrom();
    var otherSegment = node.getOpposite(segment);
    var otherLevel = otherLevel = segment.getTo().getLevel();
    var levelSize = level.getSize(this._levelFlowDirection);
    var levelCoord = level.getCoord(this._levelFlowDirection);
    var otherLevelSize = otherLevel.getSize(this._levelFlowDirection);
    var otherLevelCoord = otherLevel.getCoord(this._levelFlowDirection);

    var minNormalSortValue = levelCoord;
    var maxNormalSortValue = levelCoord + levelSize;

    var coord = segment.getTo().getCoord(this._levelFlowDirection);
    var otherCoord = otherSegment.getTo().getCoord(this._levelFlowDirection);
    coord -= otherLevelCoord;
    otherCoord -= otherLevelCoord;

    var tieBreaker;
    if (levelSize != 0.0) {
      tieBreaker = (node.getCoord(this._levelFlowDirection) - levelCoord) / levelSize;
    }
    else {
      tieBreaker = 0.0;
    }
    if (coord < otherCoord)
    {
      segment.setSortValue(maxNormalSortValue + 1.0 + otherLevelSize + tieBreaker - otherCoord);
      otherSegment.setSortValue(minNormalSortValue - 1.0 - tieBreaker - coord);
    }
    else if (coord > otherCoord)
    {
      segment.setSortValue(minNormalSortValue - 1.0 - tieBreaker - otherCoord);
      otherSegment.setSortValue(maxNormalSortValue + 1.0 + otherLevelSize + tieBreaker - coord);
    }
    else
    {
      segment.setSortValue(node.getCoord(this._levelFlowDirection));
      otherSegment.setSortValue(node.getCoord(this._levelFlowDirection));
    }
  },

  calcToSegmentBypassSortValue : function(node)
  {
    // summary:
    //		Calculate the sort value of a bypass dummy node for orthogonal
    //		layout.
    // tags:
    //		private
    
    var level = node.getLevel();
    var segment = node.getFirstSegmentTo();
    var otherSegment = node.getOpposite(segment);
    var otherLevel = otherLevel = segment.getFrom().getLevel();
    var levelSize = level.getSize(this._levelFlowDirection);
    var levelCoord = level.getCoord(this._levelFlowDirection);
    var otherLevelSize = otherLevel.getSize(this._levelFlowDirection);
    var otherLevelCoord = otherLevel.getCoord(this._levelFlowDirection);

    var minNormalSortValue = levelCoord;
    var maxNormalSortValue = levelCoord + levelSize;

    var coord = segment.getFrom().getCoord(this._levelFlowDirection);
    var otherCoord = otherSegment.getFrom().getCoord(this._levelFlowDirection);
    coord -= otherLevelCoord;
    otherCoord -= otherLevelCoord;

    var tieBreaker;
    if (levelSize != 0.0) {
      tieBreaker = (node.getCoord(this._levelFlowDirection) - levelCoord) / levelSize;
    }
    else {
      tieBreaker = 0.0;
    }
    if (coord < otherCoord)
    {
      segment.setSortValue(maxNormalSortValue + 1.0 + otherLevelSize + tieBreaker - otherCoord);
      otherSegment.setSortValue(minNormalSortValue - 1.0 - tieBreaker - coord);
    }
    else if (coord > otherCoord)
    {
      segment.setSortValue(minNormalSortValue - 1.0 - tieBreaker - otherCoord);
      otherSegment.setSortValue(maxNormalSortValue + 1.0 + otherLevelSize + tieBreaker - coord);
    }
    else
    {
      segment.setSortValue(node.getCoord(this._levelFlowDirection));
      otherSegment.setSortValue(node.getCoord(this._levelFlowDirection));
    }
  },

  treatForwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number 1 ... 
    // description:
    //		Treats a level (with level number 1 ... 
    //		n) in a forward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i-1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.calcFromSegmentSortValue(prevLevel);
    this.correctSortValueAtMultiAuxPortNodes(level);
    var iter = level.getNodes();

    while (iter.hasNext())
      iter.next().sortToSegments(this._segmentSortAlg);
  },

  treatBackwardLevel : function(prevLevel, level)
  {
    // summary:
    //		Treats a level (with level number n-1 ... 
    // description:
    //		Treats a level (with level number n-1 ... 
    //		0) in a backward sweep.  
    // prevLevel: HLevel
    //		The level before the actual level (with level number
    //		i+1).
    // level: HLevel
    //		The actual level (with level number i).
    // tags:
    //		private
    
    this.calcToSegmentSortValue(prevLevel);

    var iter = level.getNodes();

    while (iter.hasNext())
      iter.next().sortFromSegments(this._segmentSortAlg);
  }

}
);

return SortAdjacenciesAlgorithm;
});
