/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HEnum",	// #5
  "../hierarchical/HGraphAlgorithm",	// #6
  "../hierarchical/HPercentages",	// #7
  "../internalutil/GraphModelUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HEnum,	// #5
    HGraphAlgorithm,	// #6
    HPercentages,	// #7
    GraphModelUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var StraightenChainAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.StraightenChainAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		StraightenChainAlgorithm straightens the chains of nodes with one
  //		incoming and one outgoing segment, and the long links that cross
  //		several levels. 
  // description:
  //		StraightenChainAlgorithm straightens the chains of nodes with one
  //		incoming and one outgoing segment, and the long links that cross
  //		several levels. 
  //		Notice that the dummy nodes of long links have just the described
  //		property: one incoming and one outgoing segment. Nevertheless, the
  //		algorithm is not restricted to dummy nodes. The algorithm requires
  //		the existence of a proper hierarchy (i.e. nodes in levels, dummy
  //		nodes so that segments have span 1, no dummy nodes are yet marked as
  //		invalid nodes). It should be used before calculating the final link
  //		shape (orthogonal or not). The algorithm can be applied in two
  //		variants: either it takes only the node center into account (then it
  //		can run inside the node positioning phase), which means it must run
  //		before the connection point calculation, or it takes the correct
  //		connection points into account, which means it must run after
  //		connection point calculation.  
  // tags:
  //		private
  
  _useConnectionPoints: null,

  _levelFlowDir: null,

  _intervalMinForStartNode: null,
  _intervalMaxForStartNode: null,
  _intervalMinForCurrentNode: null,
  _intervalMaxForCurrentNode: null,

  _offsetFromStartNode: null,
  _prefPosition1: null,
  _prefPosition2: null,

  _hasSwimLanes: null,

  constructor : function(graph, useConnectionPoints)
  {
    this.init(graph);
    this._useConnectionPoints = useConnectionPoints;
    this._levelFlowDir = graph.getLevelFlow();
    this._hasSwimLanes = false;
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this._hasSwimLanes = graph.hasSwimLanes();

    var points = 3;

    if (this._useConnectionPoints)
    {
      points = 2 * graph.getNumberOfLinks();
    }

    this.getPercController().startStep(this.getPercentages().steps[HPercentages.STEP_STRAIGHTEN_LINKS], points, false);
    this.updateLevels();
    this.straightenLongChains();
  },

  updateLevels : function()
  {
    // summary:
    //		Make sure that all levels are up to date.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getLevels();

    while (iter.hasNext())
      iter.next().updateInfo();
  },

  straightenLongChains : function()
  {
    // summary:
    //		Searches for long links that span more than 2 levels or for chains
    //		of nodes that have one input and one output link, and tries to
    //		straighten these chains.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var nodeIter;
    var node;
    var startNodes = new Array();

    var levelIter = graph.getLevels();

    while (levelIter.hasNext())
    {
      nodeIter = levelIter.next().getNodes();

      while (nodeIter.hasNext())
      {
        node = nodeIter.next();
        if (this.isLongChainStartNode(node)) {
          startNodes.push(node);
        }
      }
    }

    var maxIter = 2 * startNodes.length;
    var changed = true;
    var aux = false;

    while (changed && maxIter > 0)
    {
      maxIter--;
      changed = false;
      var e =  new TranslateUtil.ArrayIterator(startNodes);

      while (e.hasNext())
      {
        node = e.next();
        aux = this.straightenLongChain(node);
        changed = aux || changed;
      }
    }
  },

  straightenLongChain : function(startNode)
  {
    // summary:
    //		Straightens the chain that starts at the input node. 
    // description:
    //		Straightens the chain that starts at the input node. 
    //		If it is a link to be straightened, it is normally the first dummy
    //		node of the link. It returns true if it moved any node, false
    //		otherwise.
    // tags:
    //		private
    
    this.getPercController().addPoints(1);
    this.layoutStepPerformed();
    var changed = false;
    var aux = false;
    this.setStartInterval(startNode);
    var segment = this.getFirstSegmentFrom(startNode);
    if (segment == null) {
      return this.tryStraightenLongChainPart(startNode, null); // Boolean
    }

    var node = segment.getTo();

    while (this.isLongChainInnerNode(node))
    {
      if (!this.updateInterval(node))
      {
        aux = this.tryStraightenLongChainPart(startNode, node);
        changed = aux || changed;
        startNode = node;
        this.setStartInterval(node);
      }

      node = this.getFirstSegmentFrom(node).getTo();
    }

    if (this.isLongChainEndNode(node) && this.updateInterval(node))
    {
      aux = this.tryStraightenLongChainPart(startNode, null);
    }
    else
    {
      aux = this.tryStraightenLongChainPart(startNode, node);
    }

    changed = aux || changed;
    return changed; // Boolean
  },

  tryStraightenLongChainPart : function(startNode, endNode)
  {
    // summary:
    //		Trys to straighten the chain between the startNode (inclusive) and
    //		endNode (exclusive). 
    // description:
    //		Trys to straighten the chain between the startNode (inclusive) and
    //		endNode (exclusive). 
    //		It returns true if any node was moved, false otherwise.
    // tags:
    //		private
    
    var fromSegment = this.getFirstSegmentFrom(startNode);
    var toSegment;
    var offset1;
    var offset2;
    var best;
    var changed = false;

    if (fromSegment == null || fromSegment.getTo() == endNode)
    {
      toSegment = this.getFirstSegmentTo(startNode);

      if (toSegment != null)
      {
        if (fromSegment == null || toSegment.getOwnerLink().isOrthogonal())
        {
          best = this._prefPosition1;
          if (this._intervalMinForStartNode < best && best < this._intervalMaxForStartNode) {
            return this.moveNode(startNode, best); // Boolean
          }
        }
      }

      if (fromSegment != null)
      {
        if (toSegment == null || fromSegment.getOwnerLink().isOrthogonal())
        {
          best = this._prefPosition2;
          if (this._intervalMinForStartNode < best && best < this._intervalMaxForStartNode) {
            return this.moveNode(startNode, best); // Boolean
          }
        }
      }

      return false; // Boolean
    }

    best = this._prefPosition2;
    if (best < this._intervalMinForStartNode || best > this._intervalMaxForStartNode) {
      best = this._prefPosition1;
    }
    if (best < this._intervalMinForStartNode) {
      best = this._intervalMinForStartNode;
    }
    if (best > this._intervalMaxForStartNode) {
      best = this._intervalMaxForStartNode;
    }
    var node = startNode;
    var aux = false;

    while (node != endNode)
    {
      aux = this.moveNode(node, best);
      changed = aux || changed;

      if (changed && this._useConnectionPoints)
      {
        toSegment = this.getFirstSegmentTo(node);

        if (toSegment != null && !node.isDummyNode())
        {
          var diff = toSegment.getRealToCoord(this._levelFlowDir) - toSegment.getRealFromCoord(this._levelFlowDir);
          if (-0.00001 < diff && diff < 0.00001) {
            this.moveNode(node, node.getCoord(this._levelFlowDir) + diff);
          }
          diff = toSegment.getRealToCoord(this._levelFlowDir) - toSegment.getRealFromCoord(this._levelFlowDir);

          if (-0.00001 < diff && diff < 0.00001)
          {
            var point = toSegment.getToPoint();
            point[this._levelFlowDir] += diff;
          }
        }
      }

      fromSegment = this.getFirstSegmentFrom(node);
      if (fromSegment == null) {
        break;
      }
      node = fromSegment.getTo();
      offset1 = this.getConnectionPointOffset(fromSegment, false);
      offset2 = this.getConnectionPointOffset(fromSegment, true);
      best = best + offset2 - offset1;
    }

    return changed; // Boolean
  },

  moveNode : function(node, coord)
  {
    // summary:
    //		Move the node to the input coordinate. 
    // description:
    //		Move the node to the input coordinate. 
    //		Returns true if this changes something.
    // tags:
    //		private
    
    var oldCoord = node.getCoord(this._levelFlowDir);

    if (oldCoord != coord)
    {
      if (this._useConnectionPoints) {
        node.setCoordWithLinkUpdate(this._levelFlowDir, coord);
      }
      else {
        node.setCoord(this._levelFlowDir, coord);
      }

      var delta = oldCoord - node.getCoord(this._levelFlowDir);
      if (-0.1 > delta || delta > 0.1) {
        return true; // Boolean
      }
    }

    return false; // Boolean
  },

  setStartInterval : function(startNode)
  {
    // summary:
    //		Sets the interval of the start node of the chain.
    // tags:
    //		private
    
    var actMin = this.smallestPossibleCoordinate(startNode);
    var actMax = this.largestPossibleCoordinate(startNode);
    this._intervalMinForCurrentNode = actMin;
    this._intervalMaxForCurrentNode = actMax;
    this._intervalMinForStartNode = actMin;
    this._intervalMaxForStartNode = actMax;
    this._offsetFromStartNode = 0;
    this._prefPosition1 = this.getPrefStartPosition(startNode);
    this._prefPosition2 = this.getPrefEndPosition(startNode);
  },

  updateInterval : function(node)
  {
    // summary:
    //		Updates the interval of the start node of the chain so that the
    //		chain until the input node can be straight. 
    // description:
    //		Updates the interval of the start node of the chain so that the
    //		chain until the input node can be straight. 
    //		Returns true if the input node can be part of the staightened chain,
    //		otherwise false.
    // tags:
    //		private
    
    var segment = this.getFirstSegmentTo(node);
    var actMin = this.smallestPossibleCoordinate(node);
    var actMax = this.largestPossibleCoordinate(node);
    var offset = this.getConnectionPointOffset(segment, true) - this.getConnectionPointOffset(segment, false);
    this._offsetFromStartNode += offset;

    if (this._intervalMinForCurrentNode != -/*Number.MAX_VALUE*/(3.4028235E38)) {
      this._intervalMinForCurrentNode += offset;
    }
    if (this._intervalMaxForCurrentNode != /*Number.MAX_VALUE*/(3.4028235E38)) {
      this._intervalMaxForCurrentNode += offset;
    }
    if (actMax < this._intervalMinForCurrentNode || actMin > this._intervalMaxForCurrentNode) {
      return false; // Boolean
    }
    this._prefPosition2 = this.getPrefEndPosition(node) - this._offsetFromStartNode;

    if (this._intervalMinForCurrentNode < actMin)
    {
      this._intervalMinForStartNode = actMin - this._offsetFromStartNode;
      this._intervalMinForCurrentNode = actMin;
    }

    if (this._intervalMaxForCurrentNode > actMax)
    {
      this._intervalMaxForStartNode = actMax - this._offsetFromStartNode;
      this._intervalMaxForCurrentNode = actMax;
    }

    return true; // Boolean
  },

  getPrefStartPosition : function(node)
  {
    // summary:
    //		Calculate the preferred start position of the node. 
    // description:
    //		Calculate the preferred start position of the node. 
    //		The preferred start position means the position if the node is start
    //		node of a chain and is aligned with its predecessor node.
    // tags:
    //		private
    
    var segment = this.getFirstSegmentTo(node);
    if (segment == null) {
      return node.getCoord(this._levelFlowDir); // Number
    }
    return segment.getFrom().getCoord(this._levelFlowDir) + this.getConnectionPointOffset(segment, true) - this.getConnectionPointOffset(segment, false); // Number
  },

  getPrefEndPosition : function(node)
  {
    // summary:
    //		Calculate the preferred end position of the node. 
    // description:
    //		Calculate the preferred end position of the node. 
    //		The preferred start position means the position if the node is end
    //		node of a chain and is aligned with its successor node.
    // tags:
    //		private
    
    var segment = this.getFirstSegmentFrom(node);
    if (segment == null) {
      return node.getCoord(this._levelFlowDir); // Number
    }
    return segment.getTo().getCoord(this._levelFlowDir) + this.getConnectionPointOffset(segment, false) - this.getConnectionPointOffset(segment, true); // Number
  },

  getConnectionPointOffset : function(segment, fromSide)
  {
    // summary:
    //		Return the offset between node border and connection point of the
    //		segment in level flow direction.
    // tags:
    //		private
    
    var node = ((fromSide ? segment.getFrom() : segment.getTo()));
    var sideFixed = ((fromSide ? segment.isFromSideFixed() : segment.isToSideFixed()));
    var clippedConnector = (this.getGraph().getConnectorStyle() == HEnum.CLIPPED_PINS);
    if (!this._useConnectionPoints || (node.isDummyNode() && node.getSize(this._levelFlowDir) == 0) || (clippedConnector && !sideFixed && !node.isEastWestPortAuxNode())) {
      return 0.5 * node.getSize(this._levelFlowDir); // Number
    }

    var offset;
    if (fromSide) {
      offset = segment.getRealFromCoord(this._levelFlowDir);
    }
    else {
      offset = segment.getRealToCoord(this._levelFlowDir);
    }
    offset -= node.getCoord(this._levelFlowDir);
    return offset; // Number
  },

  smallestPossibleCoordinate : function(node)
  {
    // summary:
    //		Returns the smallest coordinate that can be used for the current
    //		node. 
    // description:
    //		Returns the smallest coordinate that can be used for the current
    //		node. 
    //		It assumes that the position in level is up-to-date.
    // tags:
    //		private
    
    var prevNode = node.getPrevNeighbor();

    if (prevNode != null)
    {
      var direction = this._levelFlowDir;
      var rc = prevNode.getCoord(direction) + prevNode.getSize(direction);
      rc += this.getMinDist(prevNode, node);
      return (rc); // Number
    }
    else {
      return -/*Number.MAX_VALUE*/(3.4028235E38); // Number
    }
  },

  largestPossibleCoordinate : function(node)
  {
    // summary:
    //		Returns the largest coordinate that can be used for the current
    //		node. 
    // description:
    //		Returns the largest coordinate that can be used for the current
    //		node. 
    //		It assumes that the position in level is up-to-date.
    // tags:
    //		private
    
    var nextNode = node.getNextNeighbor();

    if (nextNode != null)
    {
      var direction = this._levelFlowDir;
      var rc = nextNode.getCoord(direction);
      rc -= (this.getMinDist(node, nextNode) + node.getSize(direction));
      return (rc); // Number
    }
    else {
      return /*Number.MAX_VALUE*/(3.4028235E38); // Number
    }
  },

  getMinDist : function(node1, node2)
  {
    // summary:
    //		Returns the minimal distance between both input nodes.
    // tags:
    //		private
    
    return this.getGraph().getMinDist(this._levelFlowDir, node1, node2); // Number
  },

  getSegmentsFromCount : function(node)
  {
    // summary:
    //		Returns the number of successor segments.
    // tags:
    //		private
    
    if (this._hasSwimLanes)
    {
      var count = 0;
      var iter = node.getSegmentsFrom();

      while (iter.hasNext())
      {
        if (iter.next().getPriority() != 0) {
          count++;
        }
      }

      return count; // int
    }
    else {
      return node.getSegmentsFromCount(); // int
    }
  },

  getSegmentsToCount : function(node)
  {
    // summary:
    //		Returns the number of predecessor segments.
    // tags:
    //		private
    
    if (this._hasSwimLanes)
    {
      var count = 0;
      var iter = node.getSegmentsTo();

      while (iter.hasNext())
      {
        if (iter.next().getPriority() != 0) {
          count++;
        }
      }

      return count; // int
    }
    else {
      return node.getSegmentsToCount(); // int
    }
  },

  isLongChainStartNode : function(node)
  {
    // summary:
    //		Returns <tt>true</tt> if the node is the start node of a long chain.
    //		
    // description:
    //		Returns <tt>true</tt> if the node is the start node of a long chain.
    //		
    //		This means, it is movable, has at least one incident segment, no
    //		more than one incoming and outgoing segment, and its predecessor, if
    //		any, is not a inner node or start node of a long chain.
    // tags:
    //		private
    
    if (node.isFixedForIncremental(this._levelFlowDir)) {
      return false; // Boolean
    }
    var segmentFromCount = this.getSegmentsFromCount(node);
    var segmentToCount;
    var predecessor = null;

    if (this._hasSwimLanes)
    {
      segmentToCount = 0;
      var iter = node.getSegmentsTo();
      var segment;

      while (iter.hasNext())
      {
        segment = iter.next();

        if (segment.getPriority() != 0)
        {
          segmentToCount++;
          predecessor = segment.getFrom();
        }
      }
    }
    else
    {
      segmentToCount = node.getSegmentsToCount();
      if (segmentToCount > 0) {
        predecessor = node.getFirstSegmentTo().getFrom();
      }
    }

    switch (segmentToCount)
    {
      case 0:
        return (segmentFromCount == 1); // Boolean
      case 1:
        if (segmentFromCount > 1) {
          return false; // Boolean
        }
        if (this.isLongChainInnerNode(predecessor)) {
          return false; // Boolean
        }
        if (this.isLongChainStartNode(predecessor)) {
          return false; // Boolean
        }
        return true; // Boolean
      default:
        return false; // Boolean
    }
  },

  isLongChainEndNode : function(node)
  {
    // summary:
    //		Returns <tt>true</tt> if the node is the end node of a long chain. 
    // description:
    //		Returns <tt>true</tt> if the node is the end node of a long chain. 
    //		This means, it is movable, has at least one incident segment, no
    //		more than one incoming and outgoing segment, and its successor, if
    //		any, is not a inner node or end node of a long chain.
    // tags:
    //		private
    
    if (node.isFixedForIncremental(this._levelFlowDir)) {
      return false; // Boolean
    }
    var segmentFromCount;
    var segmentToCount = this.getSegmentsToCount(node);
    var successor = null;

    if (this._hasSwimLanes)
    {
      segmentFromCount = 0;
      var iter = node.getSegmentsFrom();
      var segment;

      while (iter.hasNext())
      {
        segment = iter.next();

        if (segment.getPriority() != 0)
        {
          segmentFromCount++;
          successor = segment.getTo();
        }
      }
    }
    else
    {
      segmentFromCount = node.getSegmentsFromCount();
      if (segmentFromCount > 0) {
        successor = node.getFirstSegmentFrom().getTo();
      }
    }

    switch (segmentFromCount)
    {
      case 0:
        return (segmentToCount == 1); // Boolean
      case 1:
        if (segmentToCount > 1) {
          return false; // Boolean
        }
        if (this.isLongChainInnerNode(successor)) {
          return false; // Boolean
        }
        if (this.isLongChainEndNode(successor)) {
          return false; // Boolean
        }
        return true; // Boolean
      default:
        return false; // Boolean
    }
  },

  isLongChainInnerNode : function(node)
  {
    // summary:
    //		Returns <tt>true</tt> if the node has one incoming and one outgoing
    //		segment.
    // tags:
    //		private
    
    if (node.isFixedForIncremental(this._levelFlowDir)) {
      return false; // Boolean
    }
    return (this.getSegmentsFromCount(node) == 1 && this.getSegmentsToCount(node) == 1); // Boolean
  },

  getFirstSegmentFrom : function(node)
  {
    // summary:
    //		Returns the segment that leads in the long chain from this node to
    //		the next node.
    // tags:
    //		private
    
    if (this._hasSwimLanes)
    {
      var iter = node.getSegmentsFrom();

      while (iter.hasNext())
      {
        var segment = iter.next();
        if (segment.getPriority() != 0) {
          return segment; // ibm_ilog.graphlayout.hierarchical.HSegment
        }
      }

      return null; // ibm_ilog.graphlayout.hierarchical.HSegment
    }
    else
    {
      if (node.getSegmentsFromCount() == 0) {
        return null; // ibm_ilog.graphlayout.hierarchical.HSegment
      }
      return node.getFirstSegmentFrom(); // ibm_ilog.graphlayout.hierarchical.HSegment
    }
  },

  getFirstSegmentTo : function(node)
  {
    // summary:
    //		Returns the segment that leads in the long chain from this node to
    //		the previous node.
    // tags:
    //		private
    
    if (this._hasSwimLanes)
    {
      var iter = node.getSegmentsTo();

      while (iter.hasNext())
      {
        var segment = iter.next();
        if (segment.getPriority() != 0) {
          return segment; // ibm_ilog.graphlayout.hierarchical.HSegment
        }
      }

      return null; // ibm_ilog.graphlayout.hierarchical.HSegment
    }
    else
    {
      if (node.getSegmentsToCount() == 0) {
        return null; // ibm_ilog.graphlayout.hierarchical.HSegment
      }
      return node.getFirstSegmentTo(); // ibm_ilog.graphlayout.hierarchical.HSegment
    }
  }

}
);

return StraightenChainAlgorithm;
});
