/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HGraphAlgorithm",	// #5
  "../hierarchical/HIntNodeMarker",	// #6
  "../internalutil/GraphModelUtil",	// #7
  "../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HGraphAlgorithm,	// #5
    HIntNodeMarker,	// #6
    GraphModelUtil,	// #7
    TranslateUtil	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HGraphAlgorithm = ibm_ilog.graphlayout.hierarchical.HGraphAlgorithm;
=====*/

var SwimLanePlacementAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.SwimLanePlacementAlgorithm", HGraphAlgorithm, 
{
  // summary:
  //		SwimLanePlacementAlgorithm is actually a collection of auxiliary
  //		methods that help when positioning swimlanes. 
  // description:
  //		SwimLanePlacementAlgorithm is actually a collection of auxiliary
  //		methods that help when positioning swimlanes. 
  //		The algorithm works on a proper hierarchy.  
  // tags:
  //		private
  
  _swimLanes: null,

  _actSwimLaneOrderingNumber: null,
  _maxSwimLaneOrderingNumber: null,
  _minSwimLaneOrderingNumber: null,

  _nodeShiftMarker: null,

  _visitedNodeIndexPhase1: null,
  _visitedNodeIndexPhase2: null,

  _minComponentLevelIndex: null,
  _maxComponentLevelIndex: null,

  _fixedNodesInShiftTogether: null,

  _useConnectionPoints : null,

  _levelFlowDirection: null,

  _additionalSwimLaneBorderOffset: null,

  constructor : function(graph)
  {
    this._useConnectionPoints = false;
    this.init(graph);

    this._swimLanes = null;
    if (graph.hasSwimLanes()) {
      this.preprocessSwimLanes();
    }
    this._nodeShiftMarker = new HIntNodeMarker(graph);
    this._visitedNodeIndexPhase1 = TranslateUtil.newArray(graph.getNumberOfLevels(), 0);
    this._visitedNodeIndexPhase2 = TranslateUtil.newArray(graph.getNumberOfLevels(), 0);
    this._levelFlowDirection = graph.getLevelFlow();
  },
  
  clean : function()
  {
    // summary:
    //		Clean the object after usage. 
    // description:
    //		Clean the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    if (this._nodeShiftMarker != null) {
      this._nodeShiftMarker.clean();
    }
    this._nodeShiftMarker = null;
    this._visitedNodeIndexPhase1 = null;
    this._visitedNodeIndexPhase2 = null;
    this._swimLanes = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm. 
    // description:
    //		Starts the algorithm. 
    //		It verifies that the swimlanes
    //		don't interleave and calculates the swimlane bounds.
    // tags:
    //		private
    
    this.calcSwimLaneBorders(true);
    this.calcSwimLaneBounds();
  },

  preprocessSwimLanes : function (a0)
  {
    // summary:
    //		Preprocess the swimlanes before doing the sweeps.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.preprocessSwimLanes_overload_0();
    else if(arguments.length == 1)
      return this.preprocessSwimLanes_overload_1(a0);
  },
  
  preprocessSwimLanes_overload_0 : function()
                                            
  {
    this.processSegmentsForSwimLanes();
    this.calcSwimLaneMarkers();
    var iter = this.getGraph().getLevels();

    while (iter.hasNext())
      iter.next().buildNodesFieldForSwimLanes(this._swimLanes);
  },

  processSegmentsForSwimLanes : function()
  {
    // summary:
    //		Set the priority of segments that cross swimlanes to 0.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getNormalSegments();
    var segment;

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.getFrom().getSwimLane() != segment.getTo().getSwimLane()) {
        segment.setPriority(0);
      }
    }
  },

  calcSwimLaneMarkers : function()
  {
    // summary:
    //		Determine for each swimlane whether it is fixed to the left or right
    //		and the minimal and maximal positions that are influenced from fixed
    //		nodes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this._swimLanes = graph.getSortedSwimLanes();
    var n = this._swimLanes.length;
    this._minSwimLaneOrderingNumber = this._swimLanes[0].getOrderingNumber();
    this._maxSwimLaneOrderingNumber = this._swimLanes[n - 1].getOrderingNumber();
    var i;
    var width;

    for (i = 0; i < n; i++)
    {
      width = this._swimLanes[i].getHigherMinMargin();
      if (i + 1 < this._swimLanes.length) {
        width += this._swimLanes[i + 1].getLowerMinMargin();
      }
      this._swimLanes[i].setMarker(i);
      this._swimLanes[i].setFreeToLeft(true);
      this._swimLanes[i].setFreeToRight(true);
      this._swimLanes[i].setMinFixed(/*Number.MAX_VALUE*/(3.4028235E38));
      this._swimLanes[i].setMaxFixed(-/*Number.MAX_VALUE*/(3.4028235E38));
      this._swimLanes[i].setMinInfluencedFromFixed(/*Number.MAX_VALUE*/(3.4028235E38));
      this._swimLanes[i].setMaxInfluencedFromFixed(-/*Number.MAX_VALUE*/(3.4028235E38));
      this._swimLanes[i].allocateBorderNodes(graph, width);
    }

    var iter = graph.getLevels();

    while (iter.hasNext())
      this.preprocessSwimLanes(iter.next());

    var isFree = true;

    for (i = 0; i < n; i++)
    {
      this._swimLanes[i].setFreeToLeft(isFree);
      isFree = (this._swimLanes[i].getMaxFixed() == -/*Number.MAX_VALUE*/(3.4028235E38)) && isFree;
    }

    isFree = true;

    for (i = n - 1; i >= 0; i--)
    {
      this._swimLanes[i].setFreeToRight(isFree);
      isFree = (this._swimLanes[i].getMinFixed() == /*Number.MAX_VALUE*/(3.4028235E38)) && isFree;
    }
  },

  preprocessSwimLanes_overload_1 : function(level)
  {
    // summary:
    //		Determine for each swimlane whether it is fixed to the left or right
    //		and the minimal and maximal positions. 
    // description:
    //		Determine for each swimlane whether it is fixed to the left or right
    //		and the minimal and maximal positions. 
    //		This does the work per level.
    // tags:
    //		private
    
    var dir = this._levelFlowDirection;

    var iter = level.getNodes();
    var node;
    var prevNode = null;
    var lane;
    var pos;
    var densestPosSinceLastFixed = -/*Number.MAX_VALUE*/(3.4028235E38);

    while (iter.hasNext())
    {
      node = iter.next();
      lane = node.getSwimLane();

      if (node.isFixedForIncremental(dir))
      {
        lane.updateMinFixed(node.getCoord(dir) - lane.getLowerMinMargin());
        pos = node.getCoord(dir) + node.getSize(dir) + lane.getHigherMinMargin();
        lane.updateMaxFixed(pos);
        lane.updateMaxInfluencedFromFixed(pos);
        densestPosSinceLastFixed = node.getCoord(dir) + node.getSize(dir);
      }
      else if (densestPosSinceLastFixed != -/*Number.MAX_VALUE*/(3.4028235E38))
      {
        if (prevNode != null) {
          densestPosSinceLastFixed += this.getMinDist(prevNode, node);
        }
        densestPosSinceLastFixed += node.getSize(dir);
        pos = densestPosSinceLastFixed + lane.getHigherMinMargin();
        lane.updateMaxInfluencedFromFixed(pos);
      }

      prevNode = node;
    }

    prevNode = null;
    iter = level.getNodesBackward();
    densestPosSinceLastFixed = /*Number.MAX_VALUE*/(3.4028235E38);

    while (iter.hasNext())
    {
      node = iter.next();
      lane = node.getSwimLane();

      if (node.isFixedForIncremental(dir))
      {
        pos = node.getCoord(dir) - lane.getLowerMinMargin();
        lane.updateMinInfluencedFromFixed(pos);
        densestPosSinceLastFixed = node.getCoord(dir);
      }
      else if (densestPosSinceLastFixed != /*Number.MAX_VALUE*/(3.4028235E38))
      {
        if (prevNode != null) {
          densestPosSinceLastFixed -= this.getMinDist(node, prevNode);
        }
        densestPosSinceLastFixed -= node.getSize(dir);
        pos = densestPosSinceLastFixed - lane.getLowerMinMargin();
        lane.updateMinInfluencedFromFixed(pos);
      }

      prevNode = node;
    }
  },

  calcSwimLaneBorders : function(finalrun)
  {
    // summary:
    //		Calculate the border cordinates of the swimlanes.
    // tags:
    //		private
    
    if (this._swimLanes == null) {
      return;
    }
    if (finalrun) {
      this._additionalSwimLaneBorderOffset = 0;
    }
    else {
      this._additionalSwimLaneBorderOffset = 3 * this.getGraph().getMinDistBetweenNodes(this._levelFlowDirection);
    }
    this.calcSwimLaneMinMaxNodePositions();

    for (var i = 0; i < this._swimLanes.length; i++)
    {
      if (this._swimLanes[i].isFreeToRight()) {
        this.handleFreeToRightSwimLane(i);
      }
      else if (this._swimLanes[i].isFreeToLeft()) {
        this.handleFreeToLeftSwimLane(i);
      }
      else {
        this.handleFixedSwimLane(i);
      }
    }

    this.evaluateShiftValues();
  },

  evaluateShiftValues : function()
  {
    // summary:
    //		Evaluate the shift values to shift the nodes of the swimlanes to
    //		their final positions.
    // tags:
    //		private
    
    var dir = this._levelFlowDirection;
    var levelIter = this.getGraph().getLevels();
    var nodeIter;
    var node;
    var lane;

    while (levelIter.hasNext())
    {
      nodeIter = levelIter.next().getNodes();

      while (nodeIter.hasNext())
      {
        node = nodeIter.next();
        lane = node.getSwimLane();
        this.moveNodeBy(node, lane.getShiftValue());
      }
    }
  },

  calcSwimLaneMinMaxNodePositions : function()
  {
    // summary:
    //		Calculate the minimal and maximal node position per swimlane.
    // tags:
    //		private
    
    for (var i = 0; i < this._swimLanes.length; i++)
    {
      this._swimLanes[i].setMinNodePosition(/*Number.MAX_VALUE*/(3.4028235E38));
      this._swimLanes[i].setMaxNodePosition(-/*Number.MAX_VALUE*/(3.4028235E38));
      this._swimLanes[i].setShiftValue(0);
    }

    var dir = this._levelFlowDirection;
    var levelIter = this.getGraph().getLevels();
    var nodeIter;
    var node;
    var lane;
    var pos;

    while (levelIter.hasNext())
    {
      nodeIter = levelIter.next().getNodes();

      while (nodeIter.hasNext())
      {
        node = nodeIter.next();
        lane = node.getSwimLane();
        pos = node.getCoord(dir);
        if (pos < lane.getMinNodePosition()) {
          lane.setMinNodePosition(pos);
        }
        pos += node.getSize(dir);
        if (pos > lane.getMaxNodePosition()) {
          lane.setMaxNodePosition(pos);
        }
      }
    }
  },

  handleFreeToRightSwimLane : function(actSwimLaneOrderingNumber)
  {
    // summary:
    //		Handles a swimlane whose right border can freely be shifted.
    // tags:
    //		private
    
    var dir = this._levelFlowDirection;
    var actSwimLane = this._swimLanes[actSwimLaneOrderingNumber];
    var maxPos = actSwimLane.getMaxNodePosition() + actSwimLane.getShiftValue() + this._additionalSwimLaneBorderOffset;
    actSwimLane.setBorderCoord(dir, maxPos);
    maxPos += actSwimLane.getBorderNode(0).getSize(dir);

    if (actSwimLaneOrderingNumber < this._swimLanes.length - 1)
    {
      var nextLane = this._swimLanes[actSwimLaneOrderingNumber + 1];
      var minPos = nextLane.getMinNodePosition() + nextLane.getShiftValue() - this._additionalSwimLaneBorderOffset;
      var shift = maxPos - minPos;

      for (var i = actSwimLaneOrderingNumber + 1; i < this._swimLanes.length; i++)
      {
        if (shift > 0) {
          this._swimLanes[i].addShiftValue(shift);
        }
        else if (this._swimLanes[i].isFreeToLeft()) {
          this._swimLanes[i].addShiftValue(shift);
        }
      }
    }
  },

  handleFreeToLeftSwimLane : function(actSwimLaneOrderingNumber)
  {
    // summary:
    //		Handles a swimlane whose left border can freely be shifted.
    // tags:
    //		private
    
    if (actSwimLaneOrderingNumber == 0) {
      return;
    }
    var dir = this._levelFlowDirection;
    var actSwimLane = this._swimLanes[actSwimLaneOrderingNumber];
    var prevSwimLane = this._swimLanes[actSwimLaneOrderingNumber - 1];
    var minPos = actSwimLane.getMinNodePosition() + actSwimLane.getShiftValue() - this._additionalSwimLaneBorderOffset;
    minPos -= prevSwimLane.getBorderNode(0).getSize(dir);
    prevSwimLane.setBorderCoord(dir, minPos);
    var maxPos = prevSwimLane.getMaxNodePosition() + prevSwimLane.getShiftValue() + this._additionalSwimLaneBorderOffset;
    var shift = maxPos - minPos;

    for (var i = actSwimLaneOrderingNumber - 1; i >= 0; i--)
    {
      if (shift > 0) {
        this._swimLanes[i].addShiftValue(-shift);
      }
      else if (this._swimLanes[i].isFreeToRight()) {
        this._swimLanes[i].addShiftValue(-shift);
      }
    }
  },

  handleFixedSwimLane : function(actSwimLaneOrderingNumber)
  {
    // summary:
    //		Handles a swimlane whose both border are restricted by fixed nodes.
    // tags:
    //		private
    
  },

  calcSwimLaneBounds : function()
  {
    // summary:
    //		Calculate the bounds of each swimlane in level flow direction.
    // tags:
    //		private
    
    if (this._swimLanes == null) {
      return;
    }
    var dir = this._levelFlowDirection;
    var minpos = /*Number.MAX_VALUE*/(3.4028235E38);
    var maxpos = -/*Number.MAX_VALUE*/(3.4028235E38);
    var pos;
    var i;
    var iter = this.getGraph().getLevels();
    var level;

    while (iter.hasNext())
    {
      level = iter.next();

      if (level.getNodesCount() > 0)
      {
        pos = level.getNode(0).getCoord(dir);
        if (pos < minpos) {
          minpos = pos;
        }
      }
    }

    var borderNode;

    for (i = 0; i < this._swimLanes.length - 1; i++)
    {
      borderNode = this._swimLanes[i].getBorderNode(0);
      minpos -= this._swimLanes[i].getLowerMinMargin();
      maxpos = borderNode.getCoord(dir) + this._swimLanes[i].getHigherMinMargin();
      this._swimLanes[i].setCoord(dir, minpos);
      this._swimLanes[i].setSize(dir, maxpos - minpos);
      minpos = borderNode.getCoord(dir) + borderNode.getSize(dir);
    }

    i = this._swimLanes.length - 1;
    borderNode = this._swimLanes[i].getBorderNode(0);
    minpos -= this._swimLanes[i].getLowerMinMargin();
    maxpos = borderNode.getCoord(dir) + this._swimLanes[i].getHigherMinMargin();
    this._swimLanes[i].setCoord(dir, minpos);
    this._swimLanes[i].setSize(dir, maxpos - minpos);
  },

  calcFinalSwimLaneBounds : function()
  {
    // summary:
    //		Calculate the final bounds of the swimlanes.
    // tags:
    //		private
    
    if (this._swimLanes == null) {
      return;
    }
    this.calcSwimLaneBorders(true);
    this.calcSwimLaneBounds();
    var i;
    var dir = this._levelFlowDirection;

    var maxSizePerUnit = 0;
    var units;
    var sizePerUnit = 0;

    for (i = 0; i < this._swimLanes.length; i++)
    {
      this._swimLanes[i].setShiftValue(0);
      units = this._swimLanes[i].getRelativeSize();

      if (units > 0)
      {
        sizePerUnit = this._swimLanes[i].getSize(dir) / units;
        if (sizePerUnit > maxSizePerUnit) {
          maxSizePerUnit = sizePerUnit;
        }
      }
    }

    if (maxSizePerUnit == 0) {
      return;
    }

    var minPos;
    var maxPos = this._swimLanes[0].getCoord(dir);
    var desiredSize;
    var shiftVal;

    for (i = 0; i < this._swimLanes.length; i++)
    {
      units = this._swimLanes[i].getRelativeSize();
      if (units > 0) {
        desiredSize = maxSizePerUnit * units;
      }
      else {
        desiredSize = this._swimLanes[i].getSize(dir);
      }
      minPos = this._swimLanes[i].getCoord(dir);
      shiftVal = maxPos - minPos + 0.5 * (desiredSize - this._swimLanes[i].getSize(dir));

      if (this._swimLanes[i].isFreeToRight())
      {
        this._swimLanes[i].addShiftValue(shiftVal);
        this._swimLanes[i].setCoord(dir, maxPos);
        this._swimLanes[i].setSize(dir, desiredSize);
        this._swimLanes[i].setBorderCoord(dir, maxPos + desiredSize - this._swimLanes[i].getHigherMinMargin());
      }

      maxPos = this._swimLanes[i].getCoord(dir) + this._swimLanes[i].getSize(dir);
    }

    i = this._swimLanes.length - 1;
    minPos = this._swimLanes[i].getCoord(dir) + this._swimLanes[i].getSize(dir);

    for (i = this._swimLanes.length - 1; i >= 0; i--)
    {
      units = this._swimLanes[i].getRelativeSize();
      if (units > 0) {
        desiredSize = maxSizePerUnit * units;
      }
      else {
        desiredSize = this._swimLanes[i].getSize(dir);
      }
      minPos -= desiredSize;
      maxPos = this._swimLanes[i].getCoord(dir);
      shiftVal = minPos - maxPos + 0.5 * (desiredSize - this._swimLanes[i].getSize(dir));

      if (this._swimLanes[i].isFreeToLeft())
      {
        this._swimLanes[i].addShiftValue(shiftVal);
        this._swimLanes[i].setCoord(dir, minPos);
        this._swimLanes[i].setSize(dir, desiredSize);
        this._swimLanes[i].setBorderCoord(dir, minPos + desiredSize - this._swimLanes[i].getHigherMinMargin());
      }

      minPos = this._swimLanes[i].getCoord(dir);
    }

    this.evaluateShiftValues();
  },

  shiftTogether : function(hasSwimLaneBorderNodes)
  {
    var changed = true;
    var numberOfIterations = 0;

    while (changed)
    {
      changed = false;
      numberOfIterations++;
      if (numberOfIterations > 5) {
        break;
      }
      for (var i = 0; i < this._visitedNodeIndexPhase1.length; i++)
      {
        this._visitedNodeIndexPhase1[i] = -1;
        this._visitedNodeIndexPhase2[i] = -1;
      }

      this._nodeShiftMarker.setAll(0);

      if (hasSwimLaneBorderNodes) {
        this._actSwimLaneOrderingNumber = this._minSwimLaneOrderingNumber;
      }
      else {
        this._actSwimLaneOrderingNumber = -1;
      }

      var rootNode = this.getRootNodeForShiftToLeft();
      var shiftDist;

      while (rootNode != null)
      {
        shiftDist = this.getShiftDistForShiftToLeft(rootNode);
        if (shiftDist > 0.0) {
          changed = true;
        }
        this.shiftToLeft(rootNode, shiftDist);
        rootNode = this.getRootNodeForShiftToLeft();
      }

      for (var j = 0; j < this._visitedNodeIndexPhase1.length; j++)
      {
        this._visitedNodeIndexPhase1[j] = this.getGraph().getLevel(j).getNodesCount();
        this._visitedNodeIndexPhase2[j] = this.getGraph().getLevel(j).getNodesCount();
      }

      this._nodeShiftMarker.setAll(0);

      if (hasSwimLaneBorderNodes) {
        this._actSwimLaneOrderingNumber = this._maxSwimLaneOrderingNumber;
      }
      else {
        this._actSwimLaneOrderingNumber = -1;
      }
      rootNode = this.getRootNodeForShiftToRight();

      while (rootNode != null)
      {
        shiftDist = this.getShiftDistForShiftToRight(rootNode);
        if (shiftDist > 0.0) {
          changed = true;
        }
        this.shiftToRight(rootNode, shiftDist);
        rootNode = this.getRootNodeForShiftToRight();
      }
    }
  },

  getRootNodeForShiftToLeft : function()
  {
    // summary:
    //		Returns the root node for the next shift to left. 
    // description:
    //		Returns the root node for the next shift to left. 
    //		This is the unvisited node with the smallest shift distance to a
    //		visited left neightbor. If we have swimlanes, it makes sure that we
    //		visit the swimlanes in the correct order.
    // tags:
    //		private
    
    var rootNode = this.getNodeWithSmallestShiftDistToLeft(0, this._visitedNodeIndexPhase1.length - 1, true, this._actSwimLaneOrderingNumber);
    if (rootNode != null || this._actSwimLaneOrderingNumber < 0) {
      return rootNode; // ibm_ilog.graphlayout.hierarchical.HNode
    }
    while (this._actSwimLaneOrderingNumber < this._maxSwimLaneOrderingNumber)
    {
      this.markSwimLaneBorderNodesVisited(this._actSwimLaneOrderingNumber, true);

      this._actSwimLaneOrderingNumber++;
      rootNode = this.getNodeWithSmallestShiftDistToLeft(0, this._visitedNodeIndexPhase1.length - 1, true, this._actSwimLaneOrderingNumber);
      if (rootNode != null) {
        return rootNode; // ibm_ilog.graphlayout.hierarchical.HNode
      }
    }

    return null; // ibm_ilog.graphlayout.hierarchical.HNode
  },

  getRootNodeForShiftToRight : function()
  {
    // summary:
    //		Returns the root node for the next shift to right. 
    // description:
    //		Returns the root node for the next shift to right. 
    //		This is the unvisited node with the smallest shift distance to a
    //		visited right neightbor. If we have swimlanes, it makes sure that we
    //		visit the swimlanes in the correct order.
    // tags:
    //		private
    
    var rootNode = this.getNodeWithSmallestShiftDistToRight(0, this._visitedNodeIndexPhase1.length - 1, true, this._actSwimLaneOrderingNumber);
    if (rootNode != null || this._actSwimLaneOrderingNumber < 0) {
      return rootNode; // ibm_ilog.graphlayout.hierarchical.HNode
    }
    while (this._actSwimLaneOrderingNumber > this._minSwimLaneOrderingNumber)
    {
      this.markSwimLaneBorderNodesVisited(this._actSwimLaneOrderingNumber - 1, false);

      this._actSwimLaneOrderingNumber--;
      rootNode = this.getNodeWithSmallestShiftDistToRight(0, this._visitedNodeIndexPhase1.length - 1, true, this._actSwimLaneOrderingNumber);
      if (rootNode != null) {
        return rootNode; // ibm_ilog.graphlayout.hierarchical.HNode
      }
    }

    return null; // ibm_ilog.graphlayout.hierarchical.HNode
  },

  markSwimLaneBorderNodesVisited : function(orderingNumber, toLeft)
  {
    // summary:
    //		Mark all border nodes of the input swimlane with input index as
    //		visited.
    // tags:
    //		private
    
    var lane = null;
    var i;

    for (i = 0; i < this._swimLanes.length; i++)
      if (this._swimLanes[i].getOrderingNumber() == orderingNumber)
      {
        lane = this._swimLanes[i];
        break;
      }

    if (lane == null) {
      return;
    }

    var borderNode;
    var position;

    for (i = 0; i < this._visitedNodeIndexPhase1.length; i++)
    {
      borderNode = lane.getBorderNode(i);
      position = borderNode.getPositionInLevel();

      if (toLeft)
      {
        if (this._visitedNodeIndexPhase1[i] < position) {
          this._visitedNodeIndexPhase1[i] = position;
        }
        if (this._visitedNodeIndexPhase2[i] < position) {
          this._visitedNodeIndexPhase2[i] = position;
        }
      }
      else
      {
        if (this._visitedNodeIndexPhase1[i] > position) {
          this._visitedNodeIndexPhase1[i] = position;
        }
        if (this._visitedNodeIndexPhase2[i] > position) {
          this._visitedNodeIndexPhase2[i] = position;
        }
      }
    }
  },

  getShiftDistForShiftToLeft : function(node)
  {
    // summary:
    //		Calculate the maximal allowed shift distance of the component
    //		starting at the node. 
    // description:
    //		Calculate the maximal allowed shift distance of the component
    //		starting at the node. 
    //		At that time point, the shift value of all unvisited nodes is -2.
    //		The shift value of all visited nodes of the same component is -1,
    //		and the shift value of all visited nodes of different components it
    //		larger or equal 0.
    // tags:
    //		private
    
    this._minComponentLevelIndex = this._visitedNodeIndexPhase1.length;
    this._maxComponentLevelIndex = -1;
    this._fixedNodesInShiftTogether = false;
    this.visitForCalcLevelRangeToLeft(node);
    if (this._fixedNodesInShiftTogether) {
      return (0.0); // Number
    }

    var referenceNode = this.getNodeWithSmallestShiftDistToLeft(this._minComponentLevelIndex, this._maxComponentLevelIndex, false, -1);
    if (referenceNode == null) {
      return (0.0); // Number
    }
    var dist = this.getShiftDistToLeft(referenceNode);
    return ((dist < 0.0 ? 0.0 : dist)); // Number
  },

  getShiftDistForShiftToRight : function(node)
  {
    // summary:
    //		Calculate the maximal allowed shift distance of the component
    //		starting at the node. 
    // description:
    //		Calculate the maximal allowed shift distance of the component
    //		starting at the node. 
    //		At that time point, the shift value of all unvisited nodes is -2.
    //		The shift value of all visited nodes of the same component is -1,
    //		and the shift value of all visited nodes of different components it
    //		larger or equal 0.
    // tags:
    //		private
    
    this._minComponentLevelIndex = this._visitedNodeIndexPhase1.length;
    this._maxComponentLevelIndex = -1;
    this._fixedNodesInShiftTogether = false;
    this.visitForCalcLevelRangeToRight(node);
    if (this._fixedNodesInShiftTogether) {
      return (0.0); // Number
    }

    var referenceNode = this.getNodeWithSmallestShiftDistToRight(this._minComponentLevelIndex, this._maxComponentLevelIndex, false, -1);
    if (referenceNode == null) {
      return (0.0); // Number
    }
    var dist = this.getShiftDistToRight(referenceNode);
    return ((dist < 0.0 ? 0.0 : dist)); // Number
  },

  getNodeWithSmallestShiftDistToLeft : function(minLevelIndex, maxLevelIndex, preferLeftestNode, swimLaneIndex)
  {
    // summary:
    //		Returns the unvisited node with the smallest shift distance to a
    //		visited left neightbor. 
    // description:
    //		Returns the unvisited node with the smallest shift distance to a
    //		visited left neightbor. 
    //		Only the nodes of the levels in the given level index range are
    //		considered. If the swimlane index is larger than 0, only the nodes
    //		in the given swimlane are considered.
    // tags:
    //		private
    
    var level;
    var node;
    var bestNode = null;
    var bestDist = /*Number.MAX_VALUE*/(3.4028235E38);
    var dist;
    var position;

    for (var i = minLevelIndex; i <= maxLevelIndex; i++)
    {
      level = this.getGraph().getLevel(i);
      position = this._visitedNodeIndexPhase2[i] + 1;

      if (position < level.getNodesCount())
      {
        node = level.getNode(position);

        if (swimLaneIndex < 0 || (node.getSwimLane() != null && node.getSwimLane().getOrderingNumber() == swimLaneIndex))
        {
          if (position == 0 && !preferLeftestNode) {
            dist = /*Number.MAX_VALUE*/(3.4028235E38);
          }
          else {
            dist = this.getShiftDistToLeft(node);
          }
          if (dist < bestDist)
          {
            bestNode = node;
            bestDist = dist;
          }
        }
      }
    }

    return bestNode; // ibm_ilog.graphlayout.hierarchical.HNode
  },

  getNodeWithSmallestShiftDistToRight : function(minLevelIndex, maxLevelIndex, preferRightestNode, swimLaneIndex)
  {
    // summary:
    //		Returns the unvisited node with the smallest shift distance to a
    //		visited right neightbor. 
    // description:
    //		Returns the unvisited node with the smallest shift distance to a
    //		visited right neightbor. 
    //		Only the nodes of the levels in the given level index range are
    //		considered. If the swimlane index is larger than 0, only the nodes
    //		in the given swimlane are considered.
    // tags:
    //		private
    
    var level;
    var node;
    var bestNode = null;
    var bestDist = /*Number.MAX_VALUE*/(3.4028235E38);
    var dist;
    var position;

    for (var i = minLevelIndex; i <= maxLevelIndex; i++)
    {
      level = this.getGraph().getLevel(i);
      position = this._visitedNodeIndexPhase2[i] - 1;

      if (position >= 0)
      {
        node = level.getNode(position);

        if (swimLaneIndex < 0 || (node.getSwimLane() != null && node.getSwimLane().getOrderingNumber() == swimLaneIndex))
        {
          if (position == level.getNodesCount() - 1 && !preferRightestNode) {
            dist = /*Number.MAX_VALUE*/(3.4028235E38);
          }
          else {
            dist = this.getShiftDistToRight(node);
          }
          if (dist < bestDist)
          {
            bestNode = node;
            bestDist = dist;
          }
        }
      }
    }

    return bestNode; // ibm_ilog.graphlayout.hierarchical.HNode
  },

  getShiftDistToLeft : function(node)
  {
    // summary:
    //		Returns the shift distance of the input node to its "left"
    //		neighbored node. 
    // description:
    //		Returns the shift distance of the input node to its "left"
    //		neighbored node. 
    //		Returns -1 if there is no "left" neighbor.
    // tags:
    //		private
    
    var level = node.getLevel();
    var index = node.getPositionInLevel();
    if (index <= 0) {
      return -1.0; // Number
    }
    var leftNode = level.getNode(index - 1);
    var dist = node.getCoord(this._levelFlowDirection) - leftNode.getCoord(this._levelFlowDirection) - leftNode.getSize(this._levelFlowDirection) - this.getMinDist(leftNode, node);
    if (dist < 0.0) {
      dist = 0.0;
    }
    return dist; // Number
  },

  getShiftDistToRight : function(node)
  {
    // summary:
    //		Returns the shift distance of the input node to its "right"
    //		neighbored node. 
    // description:
    //		Returns the shift distance of the input node to its "right"
    //		neighbored node. 
    //		Returns -1 if there is no "right" neighbor.
    // tags:
    //		private
    
    var level = node.getLevel();
    var index = node.getPositionInLevel();
    if (index >= level.getNodesCount() - 1) {
      return -1.0; // Number
    }
    var rightNode = level.getNode(index + 1);
    var dist = rightNode.getCoord(this._levelFlowDirection) - node.getCoord(this._levelFlowDirection) - node.getSize(this._levelFlowDirection) - this.getMinDist(node, rightNode);
    if (dist < 0.0) {
      dist = 0.0;
    }
    return dist; // Number
  },

  visitForCalcLevelRangeToLeft : function(node)
  {
    // summary:
    //		Calculate the maximal shift distance of the component starting at
    //		the node, in order to shift to the "left" side. 
    // description:
    //		Calculate the maximal shift distance of the component starting at
    //		the node, in order to shift to the "left" side. 
    //		This is phase 1 of the shift together mechanism. All unvisited nodes
    //		have node shift marker 0. All nodes that were visited by phase 1
    //		have node shift marker 1. All nodes that were visited by phase 2
    //		(and phase 1) have node shift marker 2.
    // tags:
    //		private
    
    if (this._nodeShiftMarker.get(node) >= 1) {
      return;
    }

    var stack = new Array();
    stack.push(node);
    this._nodeShiftMarker.set(node, 1);

    while (!(stack.length==0))
    {
      node = stack.pop();

      if (node.isFixedForIncremental(this._levelFlowDirection)) {
        this._fixedNodesInShiftTogether = true;
      }
      var level = node.getLevel();
      var levelIndex = node.getLevelNumber();
      var position = node.getPositionInLevel();
      var oldVisitedNodeIndex = this._visitedNodeIndexPhase1[levelIndex];

      if (levelIndex < this._minComponentLevelIndex) {
        this._minComponentLevelIndex = levelIndex;
      }
      if (levelIndex > this._maxComponentLevelIndex) {
        this._maxComponentLevelIndex = levelIndex;
      }
      if (oldVisitedNodeIndex < position) {
        this._visitedNodeIndexPhase1[levelIndex] = position;
      }
      for (var i = oldVisitedNodeIndex + 1; i < position; i++)
        this.pushOnStack(level.getNode(i), stack, 1);

      this.treatParentsAndChildren(node, stack, 1);
    }
  },

  visitForCalcLevelRangeToRight : function(node)
  {
    // summary:
    //		Calculate the maximal shift distance of the component starting at
    //		the node, in order to shift to the "right" side. 
    // description:
    //		Calculate the maximal shift distance of the component starting at
    //		the node, in order to shift to the "right" side. 
    //		This is phase 1 of the shift together mechanism. All unvisited nodes
    //		have node shift marker 0. All nodes that were visited by phase 1
    //		have node shift marker 1. All nodes that were visited by phase 2
    //		(and phase 1) have node shift marker 2.
    
    if (this._nodeShiftMarker.get(node) >= 1) {
      return;
    }

    var stack = new Array();
    stack.push(node);
    this._nodeShiftMarker.set(node, 1);

    while (!(stack.length==0))
    {
      node = stack.pop();

      if (node.isFixedForIncremental(this._levelFlowDirection)) {
        this._fixedNodesInShiftTogether = true;
      }
      var level = node.getLevel();
      var levelIndex = node.getLevelNumber();
      var position = node.getPositionInLevel();
      var oldVisitedNodeIndex = this._visitedNodeIndexPhase1[levelIndex];

      if (levelIndex < this._minComponentLevelIndex) {
        this._minComponentLevelIndex = levelIndex;
      }
      if (levelIndex > this._maxComponentLevelIndex) {
        this._maxComponentLevelIndex = levelIndex;
      }
      if (oldVisitedNodeIndex > position) {
        this._visitedNodeIndexPhase1[levelIndex] = position;
      }
      for (var i = position + 1; i < oldVisitedNodeIndex; i++)
        this.pushOnStack(level.getNode(i), stack, 1);

      this.treatParentsAndChildren(node, stack, 1);
    }
  },

  shiftToLeft : function(node, shiftValue)
  {
    // summary:
    //		Visit a node and shift together the node towards the "left" side. 
    // description:
    //		Visit a node and shift together the node towards the "left" side. 
    //		This is phase 2 of the shift together mechanism. All unvisited nodes
    //		have node shift marker 0. All nodes that were visited by phase 1
    //		have node shift marker 1. All nodes that were visited by phase 2
    //		have node shift marker 2. This avoids that any node is shifted
    //		twice.
    
    if (this._nodeShiftMarker.get(node) >= 2) {
      return;
    }

    var stack = new Array();
    stack.push(node);
    this._nodeShiftMarker.set(node, 2);

    while (!(stack.length==0))
    {
      node = stack.pop();

      if (shiftValue != 0.0) {
        node.shiftCoord(this._levelFlowDirection, -shiftValue);
      }
      var level = node.getLevel();
      var levelIndex = node.getLevelNumber();
      var position = node.getPositionInLevel();
      var oldVisitedNodeIndex = this._visitedNodeIndexPhase2[levelIndex];

      if (oldVisitedNodeIndex < position) {
        this._visitedNodeIndexPhase2[levelIndex] = position;
      }
      for (var i = oldVisitedNodeIndex + 1; i < position; i++)
        this.pushOnStack(level.getNode(i), stack, 2);

      this.treatParentsAndChildren(node, stack, 2);
    }
  },

  shiftToRight : function(node, shiftValue)
  {
    // summary:
    //		Visit a node and shift together the node towards the "right" side. 
    // description:
    //		Visit a node and shift together the node towards the "right" side. 
    //		This is phase 2 of the shift together mechanism. All unvisited nodes
    //		have node shift marker 0. All nodes that were visited by phase 1
    //		have node shift marker 1. All nodes that were visited by phase 2
    //		have node shift marker 2. This avoids that any node is shifted
    //		twice.
    // tags:
    //		private
    
    if (this._nodeShiftMarker.get(node) >= 2) {
      return;
    }

    var stack = new Array();
    stack.push(node);
    this._nodeShiftMarker.set(node, 2);

    while (!(stack.length==0))
    {
      node = stack.pop();

      if (shiftValue != 0.0) {
        node.shiftCoord(this._levelFlowDirection, shiftValue);
      }
      var level = node.getLevel();
      var levelIndex = node.getLevelNumber();
      var position = node.getPositionInLevel();
      var oldVisitedNodeIndex = this._visitedNodeIndexPhase2[levelIndex];

      if (oldVisitedNodeIndex > position) {
        this._visitedNodeIndexPhase2[levelIndex] = position;
      }
      for (var i = position + 1; i < oldVisitedNodeIndex; i++)
        this.pushOnStack(level.getNode(i), stack, 2);

      this.treatParentsAndChildren(node, stack, 2);
    }
  },

  getMinDist : function(node1, node2)
  {
    // summary:
    //		Returns the minimal distance between both input nodes.
    // tags:
    //		private
    
    return this.getGraph().getMinDist(this._levelFlowDirection, node1, node2); // Number
  },

  setUseConnectionPoints : function(flag)
  {
    // summary:
    //		Whether the algorithm has to respect connection points of links.
    // tags:
    //		private
    
    this._useConnectionPoints = flag;
  },

  moveNodeBy : function(node, offset)
  {
    // summary:
    //		Moves a node.
    // tags:
    //		private
    
    if (offset != 0)
    {
      var coord = node.getCoord(this._levelFlowDirection) + offset;
      if (this._useConnectionPoints) {
        node.setCoordWithLinkUpdate(this._levelFlowDirection, coord);
      }
      else {
        node.setCoord(this._levelFlowDirection, coord);
      }
    }
  },

  treatParentsAndChildren : function(node, stack, desired)
  {
    // summary:
    //		Push all parent and children of the input node on the stack, if
    //		their shift marker is smaller than the desired value. 
    // description:
    //		Push all parent and children of the input node on the stack, if
    //		their shift marker is smaller than the desired value. 
    //		Afterwards the shift marker is set to the desired value.
    // tags:
    //		private
    
    var iter = node.getSegments();
    var segment;

    while (iter.hasNext())
    {
      segment = iter.next();
      if (segment.getPriority() != 0) {
        this.pushOnStack(segment.getOpposite(node), stack, desired);
      }
    }
  },

  pushOnStack : function(node, stack, desired)
  {
    // summary:
    //		Push the input node on the stack if its shift marker is smaller than
    //		the desired value. 
    // description:
    //		Push the input node on the stack if its shift marker is smaller than
    //		the desired value. 
    //		Afterwards the shift marker is set to the desired value.
    // tags:
    //		private
    
    if (this._nodeShiftMarker.get(node) < desired)
    {
      this._nodeShiftMarker.set(node, desired);
      stack.push(node);
    }
  }

}
);

return SwimLanePlacementAlgorithm;
});
