/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/graphbase/HTBaseEdge",	// #5
  "../../hierarchical/graphbase/HTBaseGraphMember",	// #6
  "../../internalutil/GraphModelUtil",	// #7
  "../../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HTBaseEdge,	// #5
    HTBaseGraphMember,	// #6
    GraphModelUtil,	// #7
    TranslateUtil	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HTBaseGraphMember = ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseGraphMember;
=====*/

var HTBaseNode = declare("ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseNode", HTBaseGraphMember, 
{
  // summary:
  //		A topological base node. 
  // description:
  //		A topological base node. 
  //		This is the node base class for various special purpose graph models.
  //		It contains only the adjacency information.   
  // tags:
  //		private
  
  _firstInEdge: null,

  _firstOutEdge: null,

  _indegree: null,

  _outdegree: null,

  _insideRemoveNode: null,

  constructor : function()
  {
    this._firstInEdge = null;
    this._firstOutEdge = null;
    this._indegree = 0;
    this._outdegree = 0;
  },
  
  addOutEdge : function(edge)
  {
    // summary:
    //		Adds a new edge to the outgoing adjacencies of this node. 
    // description:
    //		Adds a new edge to the outgoing adjacencies of this node. 
    //		This auxiliary can only be called for a new edge that is freshly
    //		added to the graph, or after the edge ware removed via
    //		<code>edge.&#95;source.removeOutEdge(edge)</code>.
    // tags:
    //		private
    
    edge.insertBeforeOutEdge(this, this._firstOutEdge);
    this._firstOutEdge = edge;
    this._outdegree++;
  },

  addInEdge : function(edge)
  {
    // summary:
    //		Adds a new edge to the incoming adjacencies of this node. 
    // description:
    //		Adds a new edge to the incoming adjacencies of this node. 
    //		This auxiliary can only be called for a new edge that is freshly
    //		added to the graph, or after the edge ware removed via
    //		<code>edge.&#95;target.removeInEdge(edge)</code>.
    // tags:
    //		private
    
    edge.insertBeforeInEdge(this, this._firstInEdge);
    this._firstInEdge = edge;
    this._indegree++;
  },

  removeOutEdge : function(edge)
  {
    // summary:
    //		Removes the edge from the outgoing adjacencies of this node. 
    // description:
    //		Removes the edge from the outgoing adjacencies of this node. 
    //		This auxiliary can only be called to remove the edge from the graph
    //		completely. Otherwise, it leaves the edge in inconsistent state, and
    //		must be followed by <code>source.insertOutEdge(edge)</code> to make
    //		the state consistent again.
    // tags:
    //		private
    
    if (this._insideRemoveNode)
    {
      edge._nextOutEdge = null;
      edge._prevOutEdge = null;
      return;
    }

    if (this._firstOutEdge == edge) {
      this._firstOutEdge = edge.removeOutEdge();
    }
    else {
      edge.removeOutEdge();
    }
    this._outdegree--;
  },

  removeInEdge : function(edge)
  {
    // summary:
    //		Removes the edge from the incoming adjacencies of this node. 
    // description:
    //		Removes the edge from the incoming adjacencies of this node. 
    //		This auxiliary can only be called to remove the edge from the graph
    //		completely. Otherwise, it leaves the edge in inconsistent state, and
    //		must be followed by <code>target.insertInEdge(edge)</code> to make
    //		the state consistent again.
    // tags:
    //		private
    
    if (this._insideRemoveNode)
    {
      edge._nextInEdge = null;
      edge._prevInEdge = null;
      return;
    }

    if (this._firstInEdge == edge) {
      this._firstInEdge = edge.removeInEdge();
    }
    else {
      edge.removeInEdge();
    }
    this._indegree--;
  },

  containsOutEdge : function(edge)
  {
    // summary:
    //		Checks whether the input edge is contained in the outgoing edge
    //		list.
    // tags:
    //		private
    
    return ((edge == null ? false : edge.isContainedInOutgoing(this._firstOutEdge))); // Boolean
  },

  containsInEdge : function(edge)
  {
    // summary:
    //		Checks whether the input edge is contained in the incoming edge
    //		list.
    // tags:
    //		private
    
    return ((edge == null ? false : edge.isContainedInIncoming(this._firstInEdge))); // Boolean
  },

  sortOutEdges : function(sortAlg)
  {
    // summary:
    //		Sorts the outgoing edges.
    // tags:
    //		private
    
    if (this._firstOutEdge != null)
    {
      var array = this.getOutEdgesArray();

      sortAlg.sort(array);

      this.setOutEdges(array);
    }
  },

  sortInEdges : function(sortAlg)
  {
    // summary:
    //		Sort the incoming edges.
    // tags:
    //		private
    
    if (this._firstInEdge != null)
    {
      var array = this.getInEdgesArray();

      sortAlg.sort(array);

      this.setInEdges(array);
    }
  },

  setInEdgePosition : function(edge, loc)
  {
    // summary:
    //		Moves the edge (which must be in the incoming adjancy list of this
    //		node) to the position just after the input loc. 
    // description:
    //		Moves the edge (which must be in the incoming adjancy list of this
    //		node) to the position just after the input loc. 
    //		If the input loc is null, it moves to the very first position.
    // tags:
    //		private
    
    this._firstInEdge = this.setInEdgePositionCore(edge, loc, this._firstInEdge);
  },

  setInEdgePositionCore : function(edge, loc, firstInList)
  {
    // summary:
    //		This is the core of the method above. 
    // description:
    //		This is the core of the method above. 
    //		Returns the first in the in edge list.
    // tags:
    //		private
    
    if (edge._prevInEdge == loc) {
      return firstInList; // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
    }
    if (firstInList == edge) {
      firstInList = edge.removeInEdge();
    }
    else {
      edge.removeInEdge();
    }
    edge.insertAfterInEdge(this, loc);

    if (loc == null)
    {
      edge._nextInEdge = firstInList;
      firstInList = edge;
    }

    return firstInList; // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
  },

  setOutEdgePosition : function(edge, loc)
  {
    // summary:
    //		Moves the edge (which must be in the outgoing adjancy list of this
    //		node) to the position just after the input loc. 
    // description:
    //		Moves the edge (which must be in the outgoing adjancy list of this
    //		node) to the position just after the input loc. 
    //		If the input loc is null, it moves to the very first position.
    // tags:
    //		private
    
    this._firstOutEdge = this.setOutEdgePositionCore(edge, loc, this._firstOutEdge);
  },

  setOutEdgePositionCore : function(edge, loc, firstInList)
  {
    // summary:
    //		This is the core of the method above. 
    // description:
    //		This is the core of the method above. 
    //		Returns the first in the out edge list.
    // tags:
    //		private
    
    if (edge._prevOutEdge == loc) {
      return firstInList; // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
    }
    if (firstInList == edge) {
      firstInList = edge.removeOutEdge();
    }
    else {
      edge.removeOutEdge();
    }
    edge.insertAfterOutEdge(this, loc);

    if (loc == null)
    {
      edge._nextOutEdge = firstInList;
      firstInList = edge;
    }

    return firstInList; // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
  },

  getOutEdgesArray : function()
  {
    // summary:
    //		Fill an array with the outgoing edges.
    // tags:
    //		private
    
    var edges = TranslateUtil.newArray(this._outdegree, null);
    var edge = this._firstOutEdge;
    var i = 0;

    while (edge != null)
    {
      edges[i++] = edge;
      edge = edge._nextOutEdge;
    }

    return edges; // Array
  },

  getInEdgesArray : function()
  {
    // summary:
    //		Fill an array with the incoming edges.
    // tags:
    //		private
    
    var edges = TranslateUtil.newArray(this._indegree, null);
    var edge = this._firstInEdge;
    var i = 0;

    while (edge != null)
    {
      edges[i++] = edge;
      edge = edge._nextInEdge;
    }

    return edges; // Array
  },

  setOutEdges : function(array)
  {
    // summary:
    //		Rebuild the outgoing adjacencies from an array.
    // tags:
    //		private
    
    this._outdegree = array.length;

    if (this._outdegree > 0)
    {
      this._firstOutEdge = array[0];
      this._firstOutEdge._prevOutEdge = null;
      array[this._outdegree - 1]._nextOutEdge = null;
    }
    else {
      this._firstOutEdge = null;
    }
    for (var i = 1; i < this._outdegree; i++)
    {
      array[i]._prevOutEdge = array[i - 1];
      array[i - 1]._nextOutEdge = array[i];
    }
  },

  setInEdges : function(array)
  {
    // summary:
    //		Rebuild the incoming adjacencies from an array.
    // tags:
    //		private
    
    this._indegree = array.length;

    if (this._indegree > 0)
    {
      this._firstInEdge = array[0];
      this._firstInEdge._prevInEdge = null;
      array[this._indegree - 1]._nextInEdge = null;
    }
    else {
      this._firstInEdge = null;
    }
    for (var i = 1; i < this._indegree; i++)
    {
      array[i]._prevInEdge = array[i - 1];
      array[i - 1]._nextInEdge = array[i];
    }
  },

  getOutdegree : function()
  {
    // summary:
    //		Returns the number of outgoing edges.
    // tags:
    //		private
    
    return this._outdegree; // int
  },

  getIndegree : function()
  {
    // summary:
    //		Returns the number of incoming edges.
    // tags:
    //		private
    
    return this._indegree; // int
  },

  getDegree : function()
  {
    // summary:
    //		Returns the number of incident edges.
    // tags:
    //		private
    
    return this._indegree + this._outdegree; // int
  },

  getOppositeBaseEdge : function (a0, a1, a2)
  {
    // summary:
    //		Returns the opposite edge of the node. 
    // description:
    //		Returns the opposite edge of the node. 
    //		This assumes that the node has exactly two edges.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.getOppositeBaseEdge_overload_1(a0);
    else if(arguments.length == 3)
      return this.getOppositeBaseEdge_overload_3(a0, a1, a2);
  },
  
  getOppositeBaseEdge_overload_1 : function(edge)
                                                
  {
    return this.getOppositeBaseEdge(edge, this._firstInEdge, this._firstOutEdge); // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
  },

  getOppositeBaseEdge_overload_3 : function(edge, firstInEdge, firstOutEdge)
  {
    // summary:
    //		This is the implementation of the above method.
    // tags:
    //		private
    
    var e = firstInEdge;

    while (e != null)
    {
      if (e != edge) {
        return e; // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
      }
      e = e._nextInEdge;
    }

    e = firstOutEdge;

    while (e != null)
    {
      if (e != edge) {
        return e; // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
      }
      e = e._nextOutEdge;
    }

    return null; // ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseEdge
  },

  actBeforeRemovePath : function()
  {
    // summary:
    //		When a node is removed from a path.
    // tags:
    //		private
    
  }

}
);

return HTBaseNode;
});
