/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/leveling/HLVEdgeIterator",	// #5
  "../../hierarchical/leveling/HLVNodeIterator",	// #6
  "../../internalutil/GraphModelUtil",	// #7
  "../../internalutil/LogResUtil",	// #8
  "../../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HLVEdgeIterator,	// #5
    HLVNodeIterator,	// #6
    GraphModelUtil,	// #7
    LogResUtil,	// #8
    TranslateUtil	// #9
    ) {

var HLVBlock = declare("ibm_ilog.graphlayout.hierarchical.leveling.HLVBlock", null, 
{
  // summary:
  //		A block of nodes for the optimal node leveling strategy. 
  // description:
  //		A block of nodes for the optimal node leveling strategy. 
  //		In the optimal strategy, we form blocks of nodes if nodes lie dense.
  //		Then we shift the blocks if possible.   
  // tags:
  //		private
  
  _nodes : null,

  _edges : null,

  _maxShiftToLower: null,

  _maxShiftToHigher: null,

  _next: null,
  _prev: null,

  constructor : function (a0, a1)
  {
    // summary:
    //		The dummy start block.
    
    if(arguments.length == 0)
      this._constructor_overload_HLVBlock_0();
    else if(arguments.length == 2)
      this._constructor_overload_HLVBlock_2(a0, a1);
  },
  
  _constructor_overload_HLVBlock_0 : function()
  {

    this._nodes = new Array();
    this._edges = new Array();
    this._prev = this._next = null;
  },
  
  _constructor_overload_HLVBlock_2 : function(node , prev )
  {
    this._nodes = new Array();
    this._edges = new Array();
    this._nodes = new Array();
    this._edges = new Array();
    node.setBlock(this);
    this._nodes.push(node);
    this._maxShiftToLower = node.getLevelNumber() - node.getMinLevelNumber();
    this._maxShiftToHigher = node.getMaxLevelNumber() - node.getLevelNumber();
    if (this._maxShiftToLower < 0) {
      this._maxShiftToLower = 0;
    }
    if (this._maxShiftToHigher < 0) {
      this._maxShiftToHigher = 0;
    }
    var iter = node.getInAndOutEdges();

    while (iter.hasNext())
    {
      var edge = iter.next();
      if (edge.getHLVSource() != edge.getHLVTarget()) {
        this._edges.push(edge);
      }
    }

    this._prev = prev;
    this._next = null;
    prev._next = this;
  },
  
  getNext : function()
  {
    // summary:
    //		Returns the next in the list of blocks.
    // tags:
    //		private
    
    return this._next; // ibm_ilog.graphlayout.hierarchical.leveling.HLVBlock
  },

  getNodes : function()
  {
    // summary:
    //		Returns the nodes of the block.
    // tags:
    //		private
    
    return new HLVBlockAnonClass_1(this); // ibm_ilog.graphlayout.hierarchical.leveling.HLVNodeIterator
  },

  getEdges : function()
  {
    // summary:
    //		Returns the external edges of the block.
    // tags:
    //		private
    
    return new HLVBlockAnonClass_2(this); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdgeIterator
  },

  getMaxShiftToLower : function()
  {
    // summary:
    //		Returns the maximal shift value we can shift towards a lower
    //		coordinate.
    // tags:
    //		private
    
    return this._maxShiftToLower; // int
  },

  getMaxShiftToHigher : function()
  {
    // summary:
    //		Returns the maximal shift value we can shift towards a higher
    //		coordinate.
    // tags:
    //		private
    
    return this._maxShiftToHigher; // int
  },

  checkBlock : function()
  {
    // summary:
    //		Check whether the block can be unified with neighbored blocks. 
    // description:
    //		Check whether the block can be unified with neighbored blocks. 
    //		Updates the list of all blocks as well. Returns true if the block
    //		got combined with any other block.
    // tags:
    //		private
    
    if (this._prev == null) {
      return false; // Boolean
    }
    var blocksToAdd = new Array();
    var iter = this.getEdges();

    while (iter.hasNext())
    {
      var edge = iter.next();
      var s = edge.getHLVSource();
      var t = edge.getHLVTarget();
      var span = s.getLevelNumber() - t.getLevelNumber();
      if (span < 0) {
        span = -span;
      }
      if (span == 0 || span == edge.getMinSpan())
      {
        var otherBlock = ((s.getBlock() == this ? t.getBlock() : s.getBlock()));
        TranslateUtil.addSetObject(blocksToAdd, otherBlock);
      }
    }

    var added = false;

    var _for_each_array_1 = blocksToAdd;
    for (var _for_each_index_1 = 0; _for_each_index_1 < _for_each_array_1.length; _for_each_index_1++)
    {
      var blockObj = _for_each_array_1[_for_each_index_1];
      var block = blockObj;

      if (block != this)
      {
        this.addBlock(block);
        added = true;

        if (block._prev != null) {
          block._prev._next = block._next;
        }
        if (block._next != null) {
          block._next._prev = block._prev;
        }
      }
    }

    return added; // Boolean
  },

  addBlock : function(block)
  {
    // summary:
    //		Combines this block with the input block.
    // tags:
    //		private
    
    if (block == this) {
      return;
    }
    var niter = block.getNodes();

    while (niter.hasNext())
    {
      var node = niter.next();
      node.setBlock(this);
      this._nodes.push(node);
      this._maxShiftToLower = Math.min(this._maxShiftToLower, node.getLevelNumber() - node.getMinLevelNumber());
      this._maxShiftToHigher = Math.min(this._maxShiftToHigher, node.getMaxLevelNumber() - node.getLevelNumber());
    }

    if (this._maxShiftToLower < 0) {
      this._maxShiftToLower = 0;
    }
    if (this._maxShiftToHigher < 0) {
      this._maxShiftToHigher = 0;
    }
    var newEdges = new Array();
    var eiter = this.getEdges();
    var edge;

    while (eiter.hasNext())
    {
      edge = eiter.next();
      if (edge.getHLVSource().getBlock() != edge.getHLVTarget().getBlock()) {
        newEdges.push(edge);
      }
    }

    eiter = block.getEdges();

    while (eiter.hasNext())
    {
      edge = eiter.next();
      if (edge.getHLVSource().getBlock() != edge.getHLVTarget().getBlock()) {
        newEdges.push(edge);
      }
    }

    this._edges = newEdges;
  },

  shiftBlock : function (a0)
  {
    // summary:
    //		Shifts this block by a delta. 
    // description:
    //		Shifts this block by a delta. 
    //		Positive means to higher, negative means to lower.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.shiftBlock_overload_0();
    else if(arguments.length == 1)
      return this.shiftBlock_overload_1(a0);
  },
  
  shiftBlock_overload_0 : function()
                                    
  {
    if (this._prev == null) {
      return false; // Boolean
    }
    return this.shiftBlock(this.getShiftValue()); // Boolean
  },

  shiftBlock_overload_1 : function(delta)
  {
    // summary:
    //		Shifts this block by a delta. 
    // description:
    //		Shifts this block by a delta. 
    //		Positive means to higher, negative means to lower.
    // tags:
    //		private
    
    if (delta > 0 && delta > this._maxShiftToHigher) {
      delta = this._maxShiftToHigher;
    }
    if (delta < 0 && delta < -this._maxShiftToLower) {
      delta = -this._maxShiftToLower;
    }
    if (delta == 0) {
      return false; // Boolean
    }
    var iter = this.getNodes();
    this._maxShiftToLower = 10000;
    this._maxShiftToHigher = 10000;

    while (iter.hasNext())
    {
      var node = iter.next();
      node.setLevelNumber(node.getLevelNumber() + delta);
      this._maxShiftToLower = Math.min(this._maxShiftToLower, node.getLevelNumber() - node.getMinLevelNumber());
      this._maxShiftToHigher = Math.min(this._maxShiftToHigher, node.getMaxLevelNumber() - node.getLevelNumber());
    }

    return true; // Boolean
  },

  getShiftValue : function()
  {
    // summary:
    //		Calculate the best shift value. 
    // description:
    //		Calculate the best shift value. 
    //		Pretty much the same as Algorithm.calcBaryLevel, except that we
    //		calculate the delta, not the level.
    // tags:
    //		private
    
    var minDelta = /*Number.MIN_VALUE*/(-2147483648);
    var maxDelta = /*Number.MAX_VALUE*/2147483647;
    var span;
    var prio;
    var sumPrio = 0.0;
    var sumDelta = 0.0;
    var eiter = this.getEdges();

    while (eiter.hasNext())
    {
      var edge = eiter.next();
      var s = edge.getHLVSource();
      var t = edge.getHLVTarget();
      var otherNode = ((s.getBlock() == this ? t : s));
      if (s.getBlock() == this) {
        span = t.getLevelNumber() - s.getLevelNumber();
      }
      else {
        span = s.getLevelNumber() - t.getLevelNumber();
      }
      prio = edge.getPriority();
      sumPrio += prio;
      sumDelta += prio * span;

      if (!otherNode.isPropagationGroupEndNode() && !otherNode.isPropagationGroupStartNode())
      {
        if (span < 0)
        {
          span += edge.getMinSpan();
          if (span > minDelta) {
            minDelta = span;
          }
        }
        else
        {
          span -= edge.getMinSpan();
          if (span < maxDelta) {
            maxDelta = span;
          }
        }
      }
    }

    if (sumPrio == 0.0) {
      return 0; // int
    }
    var delta = Math.floor((sumDelta / sumPrio + 0.5));
    if (delta < minDelta) {
      return minDelta; // int
    }
    if (delta > maxDelta) {
      return maxDelta; // int
    }
    return delta; // int
  }

}
);
/**
 * @private
 */
HLVBlockAnonClass_1 = declare(HLVNodeIterator,
{
  // summary:
  //		For debugging.
  // tags:
  //		private
  
  __outerThis : null,

  index: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.index = 0;
  },

  init : function(node)
  {
    LogResUtil.logAndThrowRuntimeExc(" HierarchicalLayout ", "graphlayout.expert.message.6400E");
  },

  hasNext : function()
  {
    return this.index < this.__outerThis._nodes.length; // Boolean
  },

  hasPrev : function()
  {
    return false; // Boolean
  },

  next : function()
  {
    return this.__outerThis._nodes[this.index++]; // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  },

  prev : function()
  {
    return null; // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  }

});

/**
 * @private
 */
HLVBlockAnonClass_2 = declare(HLVEdgeIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  index: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.index = 0;
  },

  init : function(node)
  {
    LogResUtil.logAndThrowRuntimeExc(" HierarchicalLayout ", "graphlayout.expert.message.6400E");
  },

  hasNext : function()
  {
    return this.index < this.__outerThis._edges.length; // Boolean
  },

  hasPrev : function()
  {
    return false; // Boolean
  },

  next : function()
  {
    return this.__outerThis._edges[this.index++]; // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  },

  prev : function()
  {
    return null; // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  }

});

return HLVBlock;
});
