/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/graphbase/HTBaseAdjEdgeIterator",	// #5
  "../../hierarchical/graphbase/HTBaseInEdgeIterator",	// #6
  "../../hierarchical/graphbase/HTBaseOutEdgeIterator",	// #7
  "../../hierarchical/leveling/HLVEdgeIterator",	// #8
  "../../hierarchical/makeacyclic/HMANode",	// #9
  "../../internalutil/GraphModelUtil",	// #10
  "../../internalutil/TranslateUtil"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HTBaseAdjEdgeIterator,	// #5
    HTBaseInEdgeIterator,	// #6
    HTBaseOutEdgeIterator,	// #7
    HLVEdgeIterator,	// #8
    HMANode,	// #9
    GraphModelUtil,	// #10
    TranslateUtil	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HMANode = ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANode;
=====*/

var HLVNode = declare("ibm_ilog.graphlayout.hierarchical.leveling.HLVNode", HMANode, 
{
  // summary:
  //		A node for the algorithm to calculate a leveling of a graph. 
  // description:
  //		A node for the algorithm to calculate a leveling of a graph. 
  //		This node is used to calculate a level number (between 0 and max)
  //		with respect to various constraints.  A sketch of some potential
  //		situations:  
  //		<ul>
  //		<li> an unconstraint HNode corresponds to one representative HLVNode 
  //		
  //		<li> a HNode that is restricted to level range [min, max] has min and
  //		max as minimal and maximal level number in its representative HLVNode
  //		 
  //		<li> a HNode that is restricted to level k has minimal and maximal
  //		level number k in its representative HLVNode  
  //		<li> if a HNode that must be below another HNode, there is an HLVEdge
  //		with minspan 1 between both representativ HLVNodes.  
  //		<li> if a group of HNodes must be at the same level, they share the
  //		same representative HLVNode  
  //		<li> if a group of HNodes must be in level range [min, max], all
  //		their representative HLVNodes have min and max as minimal and maximal
  //		level number.   
  //		<li> if a group of HNodes must be below another node "A", it is
  //		represented in the following way: There is an artificial entry
  //		HLVNode "S" for the group, and a HLVEdge with minspan 0 from "S" to
  //		each of the representativ HLVNodes of the group. There is a HLVEdge
  //		with minspan 1 from the representativ HLVNode of "A" to "S".  
  //		<li> if a group of HNodes must be above another node "A", it is
  //		represented in the following way: There is an artificial exit HLVNode
  //		"E" for the group, and a HLVEdge with minspan 0 from each of the
  //		representativ HLVNodes of the group to "E". There is a HLVEdge with
  //		minspan 1 from "E" to the representativ HLVNode of "A".  
  //		<li> if a group "X" of nodes must be below another group "Y" of
  //		nodes, it is represented in the following way: There is an artificial
  //		entry HLVNode "XS" for the group X, and  and artificial exit HLVNode
  //		"YE" and a HLVEdge with minspan 1 from "YE" to "XS". Furthermore
  //		there are HLVEdges with minspan 0 as described in the previous two
  //		points.   
  //		<li> if a group of HNodes must be in a level range that is not wider
  //		than k (and k greater 0, otherwise it means "same level"), this is
  //		represented in the following way: There is an artificial entry
  //		HLVNode "S" and exit HLVNode "E" for the group, and HLVEdges with
  //		minspan 0 as described in the previously. All group nodes are marked
  //		to need forward propagation and backward propagation. The other group
  //		node of "S" points to "E" and of "E" points to "S". The dist to other
  //		group node of "S" is k, and of "E" is -k. For each HLVEdge from any
  //		node "Z" to a group node, we add an auxiliary edge with minspan
  //		-infinite and priority 0 from "Z" to "S".  
  //		</ul>
  //		Important: before we set any minimal or maximal or final level
  //		number, we need to have the graph complete and acyclic. We cannot add
  //		edges later on, this would mess up the mechanism.  
  // tags:
  //		private
  
  _minLevelNumber: null,
  _maxLevelNumber: null,

  _block: null,

  _otherGroupNode: null,

  _distToOtherGroupNode: null,

  _flags: null,

  constructor : function()
  {
    this.setOrderingNumber(HLVNode.UNSPECIFIED);
    this._minLevelNumber = HLVNode.MIN;
    this._maxLevelNumber = HLVNode.MAX;
    this._otherGroupNode = null;
    this._distToOtherGroupNode = 0;
    this._flags = 0;
  },
  
  getBlock : function()
  {
    // summary:
    //		Returns the block of the node in the optimal leveling strategy.
    
    return this._block; // ibm_ilog.graphlayout.hierarchical.leveling.HLVBlock
  },

  setBlock : function(block)
  {
    // summary:
    //		Sets the block of the node in the optimal leveling strategy.
    // tags:
    //		private
    
    this._block = block;
  },

  isPropagationGroupStartNode : function()
  {
    // summary:
    //		Returns true if the node is a start node of a node group that has a
    //		level range constraint which must be propagated.
    // tags:
    //		private
    
    return (this._distToOtherGroupNode > 0 && this._otherGroupNode != null); // Boolean
  },

  isPropagationGroupEndNode : function()
  {
    // summary:
    //		Returns true if the node is an end node of a node group that has a
    //		level range constraint which must be propagated.
    // tags:
    //		private
    
    return (this._distToOtherGroupNode < 0 && this._otherGroupNode != null); // Boolean
  },

  connectToOtherGroupNode : function(node, dist)
  {
    // summary:
    //		Connects this node to the other group node. 
    // description:
    //		Connects this node to the other group node. 
    //		This makes this node a propagation group start or end node.
    // tags:
    //		private
    
    this._otherGroupNode = node;
    this._distToOtherGroupNode = dist;
  },

  getLevelNumber : function()
  {
    // summary:
    //		Returns the actual level number.
    // tags:
    //		private
    
    return this.getOrderingNumber(); // int
  },

  getMinLevelNumber : function()
  {
    // summary:
    //		Returns the minimal possible level number.
    // tags:
    //		private
    
    return this._minLevelNumber; // int
  },

  getMaxLevelNumber : function()
  {
    // summary:
    //		Returns the maximal possible level number.
    // tags:
    //		private
    
    return this._maxLevelNumber; // int
  },

  limitLevelNumber : function(val)
  {
    // summary:
    //		Limits the value to be between minimal and maximal level number. 
    // description:
    //		Limits the value to be between minimal and maximal level number. 
    //		Returns the limited value.
    // tags:
    //		private
    
    if (val < this._minLevelNumber) {
      return this._minLevelNumber; // int
    }
    if (val > this._maxLevelNumber) {
      return this._maxLevelNumber; // int
    }
    return val; // int
  },

  setLevelNumber : function(val)
  {
    // summary:
    //		Sets the level number of the node. 
    // description:
    //		Sets the level number of the node. 
    //		We
    //		don't change min or max level number here, because we want to know about this range for later optimization steps.
    // tags:
    //		private
    
    this.setOrderingNumber(val);

    if (!this.isPropagationGroupStartNode()) {
      this.propagateForward(val);
    }
    this.propagateAfterSettingMinLevel(val);
    this.propagateAfterSettingMaxLevel(val);
  },

  setMinLevelNumber : function(val)
  {
    // summary:
    //		Increase the minimal level number to val.
    // tags:
    //		private
    
    if (this.getLevelNumber() == HLVNode.UNSPECIFIED && val <= this._maxLevelNumber && val > this._minLevelNumber)
    {
      this._minLevelNumber = val;
      this.propagateAfterSettingMinLevel(val);
    }
  },

  setMaxLevelNumber : function(val)
  {
    // summary:
    //		Decrease the maximal level number to val.
    // tags:
    //		private
    
    if (this.getLevelNumber() == HLVNode.UNSPECIFIED && val >= this._minLevelNumber && val < this._maxLevelNumber)
    {
      this._maxLevelNumber = val;
      this.propagateAfterSettingMaxLevel(val);
    }
  },

  getOutEdges : function()
  {
    // summary:
    //		Returns the outgoing edges of the node.
    // tags:
    //		private
    
    return new HLVOutEdgeIterator(this); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdgeIterator
  },

  getInEdges : function()
  {
    // summary:
    //		Returns the incoming edges of the node.
    // tags:
    //		private
    
    return new HLVInEdgeIterator(this); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdgeIterator
  },

  getInAndOutEdges : function()
  {
    // summary:
    //		Returns the incoming and outgoing edges of the node.
    // tags:
    //		private
    
    return new HLVAdjEdgeIterator(this); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdgeIterator
  },

  needsForwardPropagationToEndNodes : function()
  {
    // summary:
    //		Returns true if the node needs to forward propagate the min level
    //		number to the propagation group end nodes. 
    // description:
    //		Returns true if the node needs to forward propagate the min level
    //		number to the propagation group end nodes. 
    //		In fact, forward propagation is always necessary if the node has an
    //		outgoing edge to a group end node. We use this flag as cache, just
    //		to know faster whether such an edge exist without examining all
    //		edges.
    // tags:
    //		private
    
    return (this._flags & 1) != 0; // Boolean
  },

  useForwardPropagationToEndNodes : function()
  {
    // summary:
    //		Stores that the node needs forward propagation.
    // tags:
    //		private
    
    this._flags |= 1;
  },

  needsBackwardPropagationToStartNodes : function()
  {
    // summary:
    //		Returns true if the node needs to backward propagate the max level
    //		number to the propagation group start nodes. 
    // description:
    //		Returns true if the node needs to backward propagate the max level
    //		number to the propagation group start nodes. 
    //		In fact, backward propagation is always necessary if the node has an
    //		incoming edge from a group start node. We use this flag as cache,
    //		just to know faster whether such an edge exist without examining all
    //		edges.
    // tags:
    //		private
    
    return (this._flags & 2) != 0; // Boolean
  },

  useBackwardPropagationToStartNodes : function()
  {
    // summary:
    //		Stores that the node needs backward propagation.
    // tags:
    //		private
    
    this._flags |= 2;
  },

  updatePropagationFlags : function()
  {
    // summary:
    //		Updates the propagation flags. 
    // description:
    //		Updates the propagation flags. 
    //		This must be called once after the graph topology was made acyclic,
    //		but before any propagation is done.
    // tags:
    //		private
    
    var iter;

    if (this.isPropagationGroupStartNode())
    {
      iter = this.getOutEdges();

      while (iter.hasNext())
        iter.next().getHLVTarget().useBackwardPropagationToStartNodes();
    }

    if (this.isPropagationGroupEndNode())
    {
      iter = this.getInEdges();

      while (iter.hasNext())
        iter.next().getHLVSource().useForwardPropagationToEndNodes();
    }
  },

  propagateAfterSettingMinLevel : function(val)
  {
    // summary:
    //		Propagate after setting min level number.
    // tags:
    //		private
    
    if (this.isPropagationGroupStartNode()) {
      this.propagateForward(val);
    }
    else if (this.needsForwardPropagationToEndNodes()) {
      this.propagateForwardToEndNodes(val);
    }
    if (this.isPropagationGroupEndNode())
    {
      var startNode = this._otherGroupNode;
      val = val + this._distToOtherGroupNode;
      startNode.setMinLevelNumber(val);
    }
  },

  propagateAfterSettingMaxLevel : function(val)
  {
    // summary:
    //		Propagate after setting max level number.
    // tags:
    //		private
    
    if (this.isPropagationGroupEndNode()) {
      this.propagateBackward(val);
    }
    else if (this.needsBackwardPropagationToStartNodes()) {
      this.propagateBackwardToStartNodes(val);
    }
    if (this.isPropagationGroupStartNode())
    {
      var endNode = this._otherGroupNode;
      val = val + this._distToOtherGroupNode;
      endNode.setMaxLevelNumber(val);
    }
  },

  propagateForward : function(val)
  {
    // summary:
    //		Normal forward propagation of a level value.
    // tags:
    //		private
    
    var iter = this.getOutEdges();
    var edge;
    var node;
    var minSpan;

    while (iter.hasNext())
    {
      edge = iter.next();
      minSpan = edge.getMinSpan();
      node = edge.getHLVTarget();
      node.setMinLevelNumber(val + minSpan);
    }
  },

  propagateBackward : function(val)
  {
    // summary:
    //		Normal backward propagation of a level value.
    // tags:
    //		private
    
    var iter = this.getInEdges();
    var edge;
    var node;
    var minSpan;

    while (iter.hasNext())
    {
      edge = iter.next();
      minSpan = edge.getMinSpan();
      node = edge.getHLVSource();
      node.setMaxLevelNumber(val - minSpan);
    }
  },

  propagateForwardToEndNodes : function(val)
  {
    // summary:
    //		Forward propagation of a level value to the end node of the group.
    // tags:
    //		private
    
    var iter = this.getOutEdges();
    var edge;
    var node;
    var minSpan;

    while (iter.hasNext())
    {
      edge = iter.next();
      minSpan = edge.getMinSpan();
      node = edge.getHLVTarget();
      if (node.isPropagationGroupEndNode()) {
        node.setMinLevelNumber(val + minSpan);
      }
    }
  },

  propagateBackwardToStartNodes : function(val)
  {
    // summary:
    //		Backward propagation of a level value to the start node of the
    //		group.
    // tags:
    //		private
    
    var iter = this.getInEdges();
    var edge;
    var node;
    var minSpan;

    while (iter.hasNext())
    {
      edge = iter.next();
      minSpan = edge.getMinSpan();
      node = edge.getHLVSource();
      if (node.isPropagationGroupStartNode()) {
        node.setMaxLevelNumber(val - minSpan);
      }
    }
  }

}
);

lang.mixin(HLVNode,
{
  UNSPECIFIED : null,
  MIN : null,
  MAX : null,
  _fields_init : function(c)
  {
    c = c || HLVNode;
    if(HMANode._fields_init)
      HMANode._fields_init(c);
    
    c.UNSPECIFIED = -1;
    c.MIN = 0;
    c.MAX = /*Number.MAX_VALUE*/2147483647;
  }
  
}
);

HLVOutEdgeIterator = declare(HTBaseOutEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(node)
  {
    this.initSuper(node, false);
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  }

}
);

HLVInEdgeIterator = declare(HTBaseInEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(node)
  {
    this.initSuper(node, false);
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  }

}
);

HLVAdjEdgeIterator = declare(HTBaseAdjEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(node)
  {
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.leveling.HLVEdge
  }

}
);
(function(){ HLVNode._fields_init(); })();

return HLVNode;
});
