/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/graphbase/BaseAlgorithm",	// #5
  "../../hierarchical/leveling/HLVBlock",	// #6
  "../../hierarchical/leveling/HLVNode",	// #7
  "../../hierarchical/leveling/LevelingModes",	// #8
  "../../internalutil/GraphModelUtil",	// #9
  "../../internalutil/QuickSort",	// #10
  "../../internalutil/TranslateUtil"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    BaseAlgorithm,	// #5
    HLVBlock,	// #6
    HLVNode,	// #7
    LevelingModes,	// #8
    GraphModelUtil,	// #9
    QuickSort,	// #10
    TranslateUtil	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var BaseAlgorithm = ibm_ilog.graphlayout.hierarchical.graphbase.BaseAlgorithm;
=====*/

var LevelingAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.leveling.LevelingAlgorithm", BaseAlgorithm, 
{
  // summary:
  //		This algorithm assigns level numbers to the nodes. 
  // description:
  //		This algorithm assigns level numbers to the nodes. 
  //		It assumes that the input graph is acyclic. It tries to assign levels
  //		such that the majority of the edges flow into from lower numbered
  //		levels to higher numbered levels. It allows the leveling constraints
  //		as exmplained in HLVNode.java.   
  // tags:
  //		private
  
  _roots: null,
  _maxLevelNumber: null,

  _levelingStrategy: null,

  constructor : function(graph, layout, percController)
  {
    this.init(graph, layout, percController);
    this._roots = new Array();
    this._maxLevelNumber = 0;
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._roots = null;
  },

  setLevelingStrategy : function(strategy)
  {
    // summary:
    //		Sets the options of the algorithm.
    // tags:
    //		private
    
    this._levelingStrategy = strategy;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    this.calcInitialLevelNumbers();

    switch (this._levelingStrategy)
    {
      case LevelingModes.OPTIMAL:
        this.optimizeBlocks();
        this.pullToBaryLevel(5, true, true);
        break;
      case LevelingModes.SEMI_OPTIMAL:
        this.pullToHigherLevel(true, false);
        this.pullToLowerLevel(false, true);
        this.pullToBaryLevel(5, true, true);
        break;
      case LevelingModes.HIGHER_LEVELS:
        this.pullToHigherLevel(true, true);
        break;
      case LevelingModes.LOWER_LEVELS:
        this.pullToLowerLevel(true, true);
        break;
      case LevelingModes.SPREAD_OUT:
        this.pullToHigherLevel(false, true);
        this.pullToLowerLevel(true, false);
        this.pullToBaryLevel(5, true, false);
        break;
    }
  },

  calcInitialLevelNumbers : function()
  {
    // summary:
    //		Calculates an initial level assigment for the nodes by respecting
    //		which segments are removed by the algorithm that made the graph
    //		acyclic.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var node;
    var otherNode;
    var iter;
    var edge;
    var num;

    this.collectRoots();

    while (!(this._roots.length==0))
    {
      node = this._roots.pop();
      num = node.getMinLevelNumber();
      if (num < 0) {
        num = 0;
      }
      if (num > this._maxLevelNumber) {
        this._maxLevelNumber = num;
      }
      node.setLevelNumber(num);
      iter = node.getOutEdges();

      while (iter.hasNext())
      {
        edge = iter.next();
        otherNode = edge.getHLVTarget();
        graph.removeEdge(edge);
        if (otherNode == node) {
          continue;
        }
        if (otherNode.getIndegree() == 0) {
          this._roots.push(otherNode);
        }
      }

      graph.removeNode(node);
    }

    var nodeIter = graph.getNodes(false);

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();
      node.setLevelNumber(node.limitLevelNumber(0));
    }

    graph.restoreTemporaryRemovedNodes();
    graph.restoreTemporaryRemovedEdges(false);
    this.addPercPoints(10);
    this.layoutStepPerformed();
  },

  collectRoots : function()
  {
    // summary:
    //		Collects all root nodes and pushes them on the stack.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getNodes(false);
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getIndegree() == 0) {
        this._roots.push(node);
      }
    }

    this.layoutStepPerformed();
  },

  pullToHigherLevel : function(shiftRoots, shiftLeaves)
  {
    // summary:
    //		Optimizes the partitioning by moving the nodes as close as possible
    //		to their neighbors at higher numbered levels. 
    // description:
    //		Optimizes the partitioning by moving the nodes as close as possible
    //		to their neighbors at higher numbered levels. 
    //		This is needed if we start the roots always at level 0, but when two
    //		paths from different roots join, it might happen that one of the
    //		last link before the join is unnecessary long. Moving the whole
    //		branch to higher levels eliminates this problem.
    // tags:
    //		private
    
    var sortAlg = new LevelingAlgorithm.HLVNodeSort(this.getGraph());
    var nodeArray = sortAlg.getSortedNodes();
    var node;
    var num;

    for (var i = nodeArray.length - 1; i >= 0; i--)
    {
      node = nodeArray[i];
      num = this.calcHighestPossibleLevel(node, shiftRoots, shiftLeaves);
      num = node.limitLevelNumber(num);
      if (num > this._maxLevelNumber) {
        this._maxLevelNumber = num;
      }
      node.setLevelNumber(num);
    }

    this.addPercPoints(10);
    this.layoutStepPerformed();
  },

  pullToLowerLevel : function(shiftRoots, shiftLeaves)
  {
    // summary:
    //		Optimizes the partitioning by moving the nodes as close as possible
    //		to their neighbors at lower numbered levels. 
    // description:
    //		Optimizes the partitioning by moving the nodes as close as possible
    //		to their neighbors at lower numbered levels. 
    //		This is not needed when nodes are placed to the lowest possible
    //		level (which is usually the case). However, if nodes are restricted
    //		to certain levels, e.g. by constraints, this might be necessary.
    // tags:
    //		private
    
    var sortAlg = new LevelingAlgorithm.HLVNodeSort(this.getGraph());
    var nodeArray = sortAlg.getSortedNodes();
    var node;
    var num;

    for (var i = 0; i < nodeArray.length; i++)
    {
      node = nodeArray[i];
      num = this.calcLowestPossibleLevel(node, shiftRoots, shiftLeaves);
      num = node.limitLevelNumber(num);
      if (num > this._maxLevelNumber) {
        this._maxLevelNumber = num;
      }
      node.setLevelNumber(num);
    }

    this.addPercPoints(10);
    this.layoutStepPerformed();
  },

  pullToBaryLevel : function(maxIteration, roundUp, includeRootsAndLeaves)
  {
    // summary:
    //		Optimizes the partitioning by pulling the nodes closer to the
    //		barycenter with respect to level numbers.  
    // maxIteration: int
    //		The maximal number of iterations of the algorithm.
    // roundUp: Boolean
    //		<tt>true</tt> if it should round the barycenter value up.
    //		If the previous pass was <tt>pullToLowerLevel</tt>, it is
    //		recommended to round up; if is was <tt>pullToHigherLevel</tt>, it
    //		is recommended to round down.
    // includeRootsAndLeaves: Boolean
    //		<tt>true</tt> if it should also move roots
    //		and leaves, <tt>false</tt> if it should only move inner nodes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var somethingChanged = true;
    var actIteration = 0;
    var iter;
    var node;
    var optimalLevel;
    var roundOffset = 0.0;
    if (roundUp) {
      roundOffset = 0.5;
    }
    while (somethingChanged && actIteration < maxIteration)
    {
      somethingChanged = false;
      actIteration++;

      iter = graph.getNodes(false);

      while (iter.hasNext())
      {
        node = iter.next();

        if (includeRootsAndLeaves || !this.isRootOrLeaf(node))
        {
          optimalLevel = this.calcBaryLevel(node, roundOffset);
          optimalLevel = node.limitLevelNumber(optimalLevel);

          if (optimalLevel != node.getLevelNumber())
          {
            if (optimalLevel > this._maxLevelNumber) {
              this._maxLevelNumber = optimalLevel;
            }
            node.setLevelNumber(optimalLevel);
            somethingChanged = true;
          }
        }
      }

      this.addPercPoints(1);
      this.layoutStepPerformed();
    }
  },

  calcHighestPossibleLevel : function(node, shiftRoots, shiftLeaves)
  {
    // summary:
    //		Calculates the highest level for this node while respecting the
    //		minimal span of segments.
    // tags:
    //		private
    
    var maxLevel = this._maxLevelNumber + 1;
    var otherLevelNumber;
    var testLevel;
    var thisLevelNumber = node.getLevelNumber();
    var iter = node.getInAndOutEdges();
    var edge;
    var otherNode;
    var isRoot = true;

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getOpposite(node);
      otherLevelNumber = otherNode.getLevelNumber();

      if (!otherNode.isPropagationGroupStartNode())
      {
        if (otherLevelNumber == thisLevelNumber) {
          return thisLevelNumber; // int
        }
        else if (otherLevelNumber > thisLevelNumber)
        {
          testLevel = otherLevelNumber - edge.getMinSpan();
          if (testLevel < maxLevel && testLevel >= 0) {
            maxLevel = testLevel;
          }
        }
        else
        {
          isRoot = false;
        }
      }
    }

    if (maxLevel == this._maxLevelNumber + 1)
    {
      if (shiftLeaves) {
        return this._maxLevelNumber; // int
      }
      else {
        return thisLevelNumber; // int
      }
    }

    if (isRoot && !shiftRoots) {
      return thisLevelNumber; // int
    }
    return maxLevel; // int
  },

  calcLowestPossibleLevel : function(node, shiftRoots, shiftLeaves)
  {
    // summary:
    //		Calculates the lowest level for this node while respecting the
    //		minimal span of segments.
    // tags:
    //		private
    
    var minLevel = -1;
    var otherLevelNumber;
    var testLevel;
    var thisLevelNumber = node.getLevelNumber();
    var iter = node.getInAndOutEdges();
    var edge;
    var otherNode;
    var isLeaf = true;

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getOpposite(node);
      otherLevelNumber = otherNode.getLevelNumber();

      if (!otherNode.isPropagationGroupEndNode())
      {
        if (otherLevelNumber == thisLevelNumber) {
          return thisLevelNumber; // int
        }
        else if (otherLevelNumber < thisLevelNumber)
        {
          testLevel = otherLevelNumber + edge.getMinSpan();
          if (testLevel > minLevel) {
            minLevel = testLevel;
          }
        }
        else
        {
          isLeaf = false;
        }
      }
    }

    if (minLevel == -1)
    {
      if (shiftRoots) {
        return 0; // int
      }
      else {
        return thisLevelNumber; // int
      }
    }

    if (isLeaf && !shiftLeaves) {
      return thisLevelNumber; // int
    }
    return minLevel; // int
  },

  calcBaryLevel : function(node, roundOffset)
  {
    // summary:
    //		Calculates the barycenter level for this node while respecting the
    //		minimal span of segments.
    // tags:
    //		private
    
    var minLevel = 0;
    var maxLevel = this._maxLevelNumber;
    var otherLevelNumber;
    var testLevel;
    var prio;
    var sumPrio = 0.0;
    var sumLevel = 0.0;
    var baryLevel;
    var thisLevelNumber = node.getLevelNumber();
    var iter = node.getInAndOutEdges();
    var edge;
    var otherNode;

    while (iter.hasNext())
    {
      edge = iter.next();
      otherNode = edge.getOpposite(node);
      otherLevelNumber = otherNode.getLevelNumber();
      prio = edge.getPriority();
      sumPrio += prio;
      sumLevel += prio * otherLevelNumber;

      if (!otherNode.isPropagationGroupEndNode() && !otherNode.isPropagationGroupStartNode())
      {
        if (otherLevelNumber == thisLevelNumber) {
          return thisLevelNumber; // int
        }
        else if (otherLevelNumber < thisLevelNumber)
        {
          testLevel = otherLevelNumber + edge.getMinSpan();
          if (testLevel > minLevel) {
            minLevel = testLevel;
          }
        }
        else
        {
          testLevel = otherLevelNumber - edge.getMinSpan();
          if (testLevel < maxLevel && testLevel >= 0) {
            maxLevel = testLevel;
          }
        }
      }
    }

    if (sumPrio == 0.0) {
      return thisLevelNumber; // int
    }
    baryLevel = Math.floor((sumLevel / sumPrio + roundOffset));
    if (baryLevel < minLevel) {
      return minLevel; // int
    }
    if (baryLevel > maxLevel) {
      return maxLevel; // int
    }
    return baryLevel; // int
  },

  isRootOrLeaf : function(node)
  {
    // summary:
    //		Tests whether the node is a conceptual root or leaf. 
    // description:
    //		Tests whether the node is a conceptual root or leaf. 
    //		This variant is a bit slow since we have to handle reversed edges.
    // tags:
    //		private
    
    var isRoot = true;
    var isLeaf = true;
    var thisLevelNumber = node.getLevelNumber();
    var iter = node.getInAndOutEdges();
    var edge;
    var otherNode;
    var otherLevelNumber;

    while (iter.hasNext() && (isRoot || isLeaf))
    {
      edge = iter.next();
      otherNode = edge.getOpposite(node);
      otherLevelNumber = otherNode.getLevelNumber();
      if (otherLevelNumber < thisLevelNumber) {
        isRoot = false;
      }
      if (otherLevelNumber > thisLevelNumber) {
        isLeaf = false;
      }
    }

    return isRoot || isLeaf; // Boolean
  },

  optimizeBlocks : function()
  {
    // summary:
    //		Optimize the node levels by shifting blocks.
    // tags:
    //		private
    
    var start = new HLVBlock();
    var prev = start;

    var graph = this.getGraph();
    var nodeIter = graph.getNodes(false);

    while (nodeIter.hasNext())
    {
      prev = new HLVBlock(nodeIter.next(), prev);
    }

    var act;
    var changed = true;
    var aux = false;

    while (changed)
    {
      changed = false;
      act = start;

      while (act != null)
      {
        aux = act.checkBlock();
        changed = aux || changed;
        act = act.getNext();
      }
    }

    this.addPercPoints(1);
    this.layoutStepPerformed();

    changed = true;
    var counter = 0;

    while (changed && counter < 100)
    {
      changed = false;
      act = start;
      counter++;

      while (act != null)
      {
        var shifted = act.shiftBlock();

        if (shifted)
        {
          changed = true;
          act.checkBlock();
        }

        act = act.getNext();
      }

      this.addPercPoints(1);
      this.layoutStepPerformed();
    }
  },

  getGraph : function()
  {
    // summary:
    //		Returns the graph we are working on.
    // tags:
    //		private
    
    return this._graph; // ibm_ilog.graphlayout.hierarchical.leveling.HLVGraph
  }

}
);

LevelingAlgorithm.HLVNodeSort = declare(QuickSort, 
{
  // summary:
  
  _nodes: null,

  constructor : function(graph)
  {
    this._nodes = TranslateUtil.newArray(graph.getNumberOfNodes(), null);
    var iter = graph.getNodes(false);
    var i = 0;

    while (iter.hasNext())
      this._nodes[i++] = iter.next();
  },
  
  getSortedNodes : function()
  {
    this.sort();
    return this._nodes; // Array
  },

  sort : function()
  {
    // summary:
    // tags:
    //		private
    
    this.inherited("sort", arguments, [this._nodes.length]);
  },

  compare : function(loc1, loc2)
  {
    return (this._nodes[loc1].getLevelNumber() - this._nodes[loc2].getLevelNumber()); // int
  },

  swap : function(loc1, loc2)
  {
    var h = this._nodes[loc1];
    this._nodes[loc1] = this._nodes[loc2];
    this._nodes[loc2] = h;
  }

}
);

return LevelingAlgorithm;
});
