/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/graphbase/HTBaseAdjEdgeIterator",	// #5
  "../../hierarchical/graphbase/HTBaseInEdgeIterator",	// #6
  "../../hierarchical/graphbase/HTBaseNode",	// #7
  "../../hierarchical/graphbase/HTBaseOutEdgeIterator",	// #8
  "../../hierarchical/makeacyclic/HMAEdgeIterator",	// #9
  "../../internalutil/GraphModelUtil",	// #10
  "../../internalutil/TranslateUtil"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HTBaseAdjEdgeIterator,	// #5
    HTBaseInEdgeIterator,	// #6
    HTBaseNode,	// #7
    HTBaseOutEdgeIterator,	// #8
    HMAEdgeIterator,	// #9
    GraphModelUtil,	// #10
    TranslateUtil	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HTBaseNode = ibm_ilog.graphlayout.hierarchical.graphbase.HTBaseNode;
=====*/

var HMANode = declare("ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANode", HTBaseNode, 
{
  // summary:
  //		A node for the algorithm to make the graph acyclic. 
  // description:
  //		A node for the algorithm to make the graph acyclic. 
  //		This node maintains the sum of priorities per incoming or outgoing
  //		edge, and the number of incoming and outgoing unbreakable edges. We
  //		don't simulate unbreakable edges by priority==MAX&#95;VALUE because this would cause overflow problems when adding and removing unbreakable egdes.   
  // tags:
  //		private
  
  _inprio: null,
  _outprio: null,

  _numUnbreakableInEdges: null,
  _numUnbreakableOutEdges: null,

  _orderingNumber: null,

  constructor : function()
  {
    this._inprio = 0.0;
    this._outprio = 0.0;
    this._numUnbreakableInEdges = 0;
    this._numUnbreakableOutEdges = 0;
    this._orderingNumber = -1;
  },
  
  setOrderingNumber : function(number)
  {
    // summary:
    //		Sets the ordering number of the node. 
    // description:
    //		Sets the ordering number of the node. 
    //		This is only relevant if the graph gets topologically sorted.
    //		Otherwise, the number can be freely used.
    // tags:
    //		private
    
    this._orderingNumber = number;
  },

  getOrderingNumber : function()
  {
    // summary:
    //		Returns the ordering number of the node. 
    // description:
    //		Returns the ordering number of the node. 
    //		This is only relevant if the graph gets topologically sorted.
    // tags:
    //		private
    
    return this._orderingNumber; // int
  },

  getInPriority : function()
  {
    // summary:
    //		Returns the sum of priorities of incoming edges.
    // tags:
    //		private
    
    return this._inprio; // Number
  },

  getOutPriority : function()
  {
    // summary:
    //		Returns the sum of priorities of outgoing edges.
    // tags:
    //		private
    
    return this._outprio; // Number
  },

  getInUnbreakCount : function()
  {
    // summary:
    //		Returns the number of unbreakable incoming edges.
    // tags:
    //		private
    
    return this._numUnbreakableInEdges; // int
  },

  getOutUnbreakCount : function()
  {
    // summary:
    //		Returns the number of unbreakable outgoing edges.
    // tags:
    //		private
    
    return this._numUnbreakableOutEdges; // int
  },

  clearEdgeData : function()
  {
    // summary:
    //		Sets the data of incident edges (priorities, unbreakable edge count)
    //		back to 0,
    // tags:
    //		private
    
    this._inprio = this._outprio = 0.0;
    this._numUnbreakableInEdges = this._numUnbreakableOutEdges = 0;
  },

  addInEdgeData : function(edge)
  {
    // summary:
    //		Adds the data of the input incoming edge to this node.
    // tags:
    //		private
    
    this._inprio += edge.getPriority();
    if (edge.isUnbreakable()) {
      this._numUnbreakableInEdges++;
    }
  },

  addOutEdgeData : function(edge)
  {
    // summary:
    //		Adds the data of the input outgoing edge to this node.
    // tags:
    //		private
    
    this._outprio += edge.getPriority();
    if (edge.isUnbreakable()) {
      this._numUnbreakableOutEdges++;
    }
  },

  removeInEdgeData : function(edge)
  {
    // summary:
    //		Removes the data of the input incoming edge from this node.
    // tags:
    //		private
    
    this._inprio -= edge.getPriority();
    if (this._inprio < 0.0 || this.getIndegree() == 0) {
      this._inprio = 0.0;
    }
    if (edge.isUnbreakable()) {
      this._numUnbreakableInEdges--;
    }
  },

  removeOutEdgeData : function(edge)
  {
    // summary:
    //		Removes the data of the input outgoing edge from this node.
    // tags:
    //		private
    
    this._outprio -= edge.getPriority();
    if (this._outprio < 0.0 || this.getOutdegree() == 0) {
      this._outprio = 0.0;
    }
    if (edge.isUnbreakable()) {
      this._numUnbreakableOutEdges--;
    }
  },

  addOutEdge : function(edge)
  {
    // summary:
    //		Adds a new edge to the outgoing adjacencies of this node. 
    // description:
    //		Adds a new edge to the outgoing adjacencies of this node. 
    //		Overridden version of the base class method.
    // tags:
    //		private
    
    this.inherited("addOutEdge", arguments, [edge]);
    if ((this.getOwnerBaseGraph()).isNodeEdgeDataValid()) {
      this.addOutEdgeData(edge);
    }
  },

  addInEdge : function(edge)
  {
    // summary:
    //		Adds a new edge to the incoming adjacencies of this node. 
    // description:
    //		Adds a new edge to the incoming adjacencies of this node. 
    //		Overridden version of the base class method.
    // tags:
    //		private
    
    this.inherited("addInEdge", arguments, [edge]);
    if ((this.getOwnerBaseGraph()).isNodeEdgeDataValid()) {
      this.addInEdgeData(edge);
    }
  },

  removeOutEdge : function(edge)
  {
    // summary:
    //		Removes the edge from the outgoing adjacencies of this node. 
    // description:
    //		Removes the edge from the outgoing adjacencies of this node. 
    //		Overridden version of the base class method.
    // tags:
    //		private
    
    this.inherited("removeOutEdge", arguments, [edge]);
    if ((this.getOwnerBaseGraph()).isNodeEdgeDataValid()) {
      this.removeOutEdgeData(edge);
    }
  },

  removeInEdge : function(edge)
  {
    // summary:
    //		Removes the edge from the incoming adjacencies of this node. 
    // description:
    //		Removes the edge from the incoming adjacencies of this node. 
    //		Overridden version of the base class method.
    // tags:
    //		private
    
    this.inherited("removeInEdge", arguments, [edge]);
    if ((this.getOwnerBaseGraph()).isNodeEdgeDataValid()) {
      this.removeInEdgeData(edge);
    }
  },

  getOutEdges : function()
  {
    // summary:
    //		Returns the outgoing edges of the node.
    // tags:
    //		private
    
    return new HMAOutEdgeIterator(this); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdgeIterator
  },

  getInEdges : function()
  {
    // summary:
    //		Returns the incoming edges of the node.
    // tags:
    //		private
    
    return new HMAInEdgeIterator(this); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdgeIterator
  },

  getHMAEdges : function()
  {
    // summary:
    //		Returns the adjacent edges of the node.
    
    return new HMAAdjEdgeIterator(this); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdgeIterator
  },

  setMinOutPriority : function(minPriority)
  {
    // summary:
    //		Ensures the all outgoing edges have at least the minimum priority.
    // tags:
    //		private
    
    var iter = this.getOutEdges();
    var edge;

    while (iter.hasNext())
    {
      edge = iter.next();
      if (edge.getPriority() < minPriority) {
        edge.setPriority(minPriority);
      }
    }
  },

  setMinInPriority : function(minPriority)
  {
    // summary:
    //		Ensures the all incoming edges have at least the minimum priority.
    // tags:
    //		private
    
    var iter = this.getInEdges();
    var edge;

    while (iter.hasNext())
    {
      edge = iter.next();
      if (edge.getPriority() < minPriority) {
        edge.setPriority(minPriority);
      }
    }
  }

}
);

HMAOutEdgeIterator = declare(HTBaseOutEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(node)
  {
    this.initSuper(node, false);
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  }

}
);

HMAInEdgeIterator = declare(HTBaseInEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(node)
  {
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  }

}
);

HMAAdjEdgeIterator = declare(HTBaseAdjEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(node)
  {
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  }

}
);

return HMANode;
});
