/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/graphbase/BaseAlgorithm",	// #5
  "../../internalutil/GraphModelUtil",	// #6
  "../../internalutil/LayoutUtil",	// #7
  "../../internalutil/LogResUtil",	// #8
  "../../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    BaseAlgorithm,	// #5
    GraphModelUtil,	// #6
    LayoutUtil,	// #7
    LogResUtil,	// #8
    TranslateUtil	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var BaseAlgorithm = ibm_ilog.graphlayout.hierarchical.graphbase.BaseAlgorithm;
=====*/

var MakeAcyclicAlgorithm = declare("ibm_ilog.graphlayout.hierarchical.makeacyclic.MakeAcyclicAlgorithm", BaseAlgorithm, 
{
  // summary:
  //		This algorithm makes a graph acyclyc by removing or reversing certain
  //		edges. 
  // description:
  //		This algorithm makes a graph acyclyc by removing or reversing certain
  //		edges. 
  //		It works on a HMAGraph.  
  //		<p>
  //		Self loops are removed. They can be accessed by getSelfLoops()
  //		afterwards. One edge of each two cycle gets reversed. The reversed
  //		edges of two cycles can be accessed by getTwoCycles() afterwards.
  //		Other edges that belong to a cycle may get reversed as well. You can
  //		test whether an edge is reversed by testing edge.isReversed().   
  //		</p>
  // tags:
  //		private
  
  _selfLoops: null,
  _twoCycles: null,

  _roots: null,
  _leaves: null,

  _topologicalOrdering: null,

  constructor : function(graph, layout, percController)
  {
    this.init(graph, layout, percController);
    this._selfLoops = new Array();
    this._twoCycles = new Array();
    this._roots = new Array();
    this._leaves = new Array();
    this._topologicalOrdering = false;
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the object after usage. 
    // description:
    //		Cleans the object after usage. 
    //		It is not necessary to use this, but it speeds up garbage collection
    //		if we do it when we don't need the object anymore.
    // tags:
    //		private
    
    this.inherited("clean", arguments, []);
    this._selfLoops = null;
    this._twoCycles = null;
    this._roots = null;
    this._leaves = null;
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm.
    // tags:
    //		private
    
    this.getGraph().emptyTemporaryRemovedNodes();
    this.getGraph().emptyTemporaryRemovedEdges();
    this.removeSelfLoops();
    this.breakTwoCycles();
    this.breakRemainingCycles();

    if (this.isTopologicalOrdering()) {
      this.calcTopologicalOrder();
    }
  },

  removeSelfLoops : function()
  {
    // summary:
    //		Removes the self-loops.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getEdges(false);
    var edge;

    while (iter.hasNext())
    {
      edge = iter.next();

      if (edge.isSelfLoop())
      {
        graph.removeEdge(edge);
        this._selfLoops.push(edge);
      }
    }

    this.addPercPoints(1);
    this.layoutStepPerformed();
  },

  breakTwoCycles : function()
  {
    // summary:
    //		Removes the 2-cycles. 
    // description:
    //		Removes the 2-cycles. 
    //		A 2-cycles is a cycle of the length 2, e.g. A to B to A. To break
    //		the cycle, the one of both edges with the smaller priority needs to
    //		be removed.
    // tags:
    //		private
    
    var graph = this.getGraph();

    graph.clearNodeEdgeData();
    this.addPercPoints(1);
    this.layoutStepPerformed();

    var node;
    var otherNode;
    var edge;
    var nodeIter = graph.getNodes(false);
    var edgeIter;
    var outPrio;
    var inPrio;
    var outUnbreakCount;
    var inUnbreakCount;

    while (nodeIter.hasNext())
    {
      node = nodeIter.next();

      edgeIter = node.getOutEdges();

      while (edgeIter.hasNext())
      {
        edge = edgeIter.next();
        edge.getHMATarget().addInEdgeData(edge);
      }

      edgeIter = node.getInEdges();

      while (edgeIter.hasNext())
      {
        edge = edgeIter.next();
        edge.getHMASource().addOutEdgeData(edge);
      }

      edgeIter = node.getOutEdges();

      while (edgeIter.hasNext())
      {
        edge = edgeIter.next();
        otherNode = edge.getHMATarget();
        outPrio = otherNode.getOutPriority();
        inPrio = otherNode.getInPriority();
        outUnbreakCount = otherNode.getOutUnbreakCount();
        inUnbreakCount = otherNode.getInUnbreakCount();

        if (outPrio == 0.0 && outUnbreakCount == 0) {
          otherNode.clearEdgeData();
        }
        else
        {
          edge.markTwoCycle();
          if (outUnbreakCount > inUnbreakCount || outUnbreakCount == inUnbreakCount && outPrio >= inPrio) {
            this._twoCycles.push(edge);
          }
        }
      }

      edgeIter = node.getInEdges();

      while (edgeIter.hasNext())
        edgeIter.next().getHMASource().clearEdgeData();
    }

    this.addPercPoints(1);
    this.layoutStepPerformed();

    var e =  new TranslateUtil.ArrayIterator(this._twoCycles);

    while (e.hasNext())
    {
      edge = e.next();
      edge.markReversed();
      edge.reverse();
    }
  },

  breakRemainingCycles : function()
  {
    // summary:
    //		Breaks all cycles that are not self-loops nor 2-cycles. 
    // description:
    //		Breaks all cycles that are not self-loops nor 2-cycles. 
    //		The goal is to find the edges with smallest priority that belong to
    //		a cycle and break just these. This is unfortunately NP complete.
    //		Therefore, this is a heuristic.
    // tags:
    //		private
    
    var graph = this.getGraph();

    graph.recalcNodeEdgeData();
    this.addPercPoints(1);
    this.layoutStepPerformed();

    var iter = graph.getNodes(false);
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getIndegree() == 0) {
        this._roots.push(node);
      }
      else if (node.getOutdegree() == 0) {
        this._leaves.push(node);
      }
    }

    var testVal = graph.getNumberOfNodes() + 1;

    while (graph.getNumberOfNodes() > 0)
    {
      while (!(this._roots.length==0))
      {
        node = this._roots.pop();

        this.removeOutEdges(node);
        if (node.getOwnerBaseGraph() != null) {
          graph.removeNode(node);
        }
      }

      while (!(this._leaves.length==0))
      {
        node = this._leaves.pop();

        this.removeInEdges(node);
        if (node.getOwnerBaseGraph() != null) {
          graph.removeNode(node);
        }
      }

      if (graph.getNumberOfNodes() >= testVal)
      {
        LogResUtil.logAndThrowRuntimeExc(" HierarchicalLayout ", "graphlayout.expert.message.6401E");
      }

      testVal = graph.getNumberOfNodes();

      if (testVal > 0)
      {
        this.addPercPoints(1);
        this.layoutStepPerformed();
        node = this.searchBreakNode();
        this.reverseAndRemoveInEdges(node);
        this._roots.push(node);
      }
    }

    graph.restoreTemporaryRemovedNodes();
    graph.restoreTemporaryRemovedEdges(false);
    this.layoutStepPerformed();
  },

  searchBreakNode : function()
  {
    // summary:
    //		Searchs a break node in the set of remaining nodes. 
    // description:
    //		Searchs a break node in the set of remaining nodes. 
    //		At this time point, the remaining nodes are a network of cycles. The
    //		break node is the node whose incoming edges are removed to break the
    //		cycle. Since the break node becomes a root by that, all its
    //		remaining outgoing edges
    //		don't need to be removed as they cannot be part of any other cycle. The goal is to minimize the priority of edges that are removed to break cycles and to maximize the priority of edges that are not removed. Hence, we search a node with minimal to-priority and maximal from-priority.
    // tags:
    //		private
    
    var bestInPrio = 0.0;
    var bestOutPrio = 0.0;
    var bestUnbreakCount = 0;
    var bestNode = null;
    var inPrio = 0.0;
    var outPrio = 0.0;
    var unbreakCount = 0;
    var node;

    var iter = this.getGraph().getNodes(false);

    while (iter.hasNext())
    {
      node = iter.next();
      inPrio = node.getInPriority();
      outPrio = node.getOutPriority();
      unbreakCount = node.getInUnbreakCount();

      if (bestNode == null || unbreakCount < bestUnbreakCount || (unbreakCount == bestUnbreakCount && (inPrio < bestInPrio || (inPrio == bestInPrio && outPrio > bestOutPrio))))
      {
        bestNode = node;
        bestInPrio = inPrio;
        bestOutPrio = outPrio;
        bestUnbreakCount = unbreakCount;
      }
    }

    return bestNode; // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMANode
  },

  removeOutEdges : function(node)
  {
    // summary:
    //		Iterates over outgoing edges, and removes them. 
    // description:
    //		Iterates over outgoing edges, and removes them. 
    //		By this, target nodes of these edges might become new roots.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = node.getOutEdges();
    var otherNode;
    var edge;

    while (iter.hasNext())
    {
      edge = iter.next();

      otherNode = edge.getHMATarget();
      graph.removeEdge(edge);
      if (otherNode.getIndegree() == 0) {
        this._roots.push(otherNode);
      }
    }
  },

  removeInEdges : function(node)
  {
    // summary:
    //		Iterates over incoming edges, and removes them. 
    // description:
    //		Iterates over incoming edges, and removes them. 
    //		By this, source nodes of these edges might become new leaves.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = node.getInEdges();
    var otherNode;
    var edge;

    while (iter.hasNext())
    {
      edge = iter.next();

      otherNode = edge.getHMASource();
      graph.removeEdge(edge);
      if (otherNode.getOutdegree() == 0) {
        this._leaves.push(otherNode);
      }
    }
  },

  reverseAndRemoveInEdges : function(node)
  {
    // summary:
    //		Iterates over incoming edges, and cuts and removes them. 
    // description:
    //		Iterates over incoming edges, and cuts and removes them. 
    //		This actually breaks the cycles. By this, "from" nodes of these
    //		edges might become new leaves.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = node.getInEdges();
    var otherNode;
    var edge;

    while (iter.hasNext())
    {
      edge = iter.next();

      otherNode = edge.getHMASource();
      graph.removeEdge(edge);

      edge.markReversed();
      edge.reverse();
      if (otherNode.getOutdegree() == 0) {
        this._leaves.push(otherNode);
      }
    }
  },

  calcTopologicalOrder : function()
  {
    // summary:
    //		Calculate a topological ordering on the nodes. 
    // description:
    //		Calculate a topological ordering on the nodes. 
    //		The ordering number is available by <code>
    //		HMANode.getOrderingNumber()</code>. The ordering number is currently
    //		total, i.e. each node has a different number.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getNodes(false);
    var node;
    var nextOrderingNumber = 0;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getIndegree() == 0) {
        this._roots.push(node);
      }
    }

    while (!(this._roots.length==0))
    {
      node = this._roots.pop();
      node.setOrderingNumber(nextOrderingNumber++);
      this.removeOutEdges(node);
      graph.removeNode(node);
    }

    graph.restoreTemporaryRemovedNodes();
    graph.restoreTemporaryRemovedEdges(false);
  },

  getGraph : function()
  {
    // summary:
    //		Returns the graph we are working on.
    // tags:
    //		private
    
    return this._graph; // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraph
  },

  getSelfLoops : function()
  {
    // summary:
    //		Returns the removed self-loop edges.
    // tags:
    //		private
    
    if (this._selfLoops != null) {
      return  new TranslateUtil.ArrayIterator(this._selfLoops); // ibm_ilog.graphlayout.IIterator
    }
    else {
      return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
    }
  },

  getTwoCycles : function()
  {
    // summary:
    //		Returns the 2-cycle representation edges. 
    // description:
    //		Returns the 2-cycle representation edges. 
    //		From each 2-cycle, there is at least one representation edge here.
    // tags:
    //		private
    
    if (this._twoCycles != null) {
      return  new TranslateUtil.ArrayIterator(this._twoCycles); // ibm_ilog.graphlayout.IIterator
    }
    else {
      return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
    }
  },

  setTopologicalOrdering : function(flag)
  {
    // summary:
    //		Sets whether we we sort the nodes topologically after making the
    //		graph acyclic. 
    // description:
    //		Sets whether we we sort the nodes topologically after making the
    //		graph acyclic. 
    //		This is diabled by default. If enabled, the nodes get ordering
    //		numbers that represent the topological ordering.
    // tags:
    //		private
    
    this._topologicalOrdering = flag;
  },

  isTopologicalOrdering : function()
  {
    // summary:
    //		Returns true if we sort the nodes topologically after making
    //		acyclic.
    // tags:
    //		private
    
    return this._topologicalOrdering; // Boolean
  }

}
);

return MakeAcyclicAlgorithm;
});
