/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/graphbase/HTBaseEdgeIterator",	// #5
  "../../hierarchical/graphbase/HTBaseNodeIterator",	// #6
  "../../hierarchical/makeacyclic/HMAEdge",	// #7
  "../../hierarchical/makeacyclic/HMAEdgeIterator",	// #8
  "../../hierarchical/makeacyclic/HMAGraph",	// #9
  "../../hierarchical/relpositioning/HRPNode",	// #10
  "../../hierarchical/relpositioning/HRPNodeIterator",	// #11
  "../../internalutil/GraphModelUtil",	// #12
  "../../internalutil/TranslateUtil"	// #13
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HTBaseEdgeIterator,	// #5
    HTBaseNodeIterator,	// #6
    HMAEdge,	// #7
    HMAEdgeIterator,	// #8
    HMAGraph,	// #9
    HRPNode,	// #10
    HRPNodeIterator,	// #11
    GraphModelUtil,	// #12
    TranslateUtil	// #13
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var HMAGraph = ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAGraph;
=====*/

var HRPGraph = declare("ibm_ilog.graphlayout.hierarchical.relpositioning.HRPGraph", HMAGraph, 
{
  // summary:
  //		A graph for the algorithm to calculate the relative positions of
  //		nodes within each level. 
  // description:
  //		A graph for the algorithm to calculate the relative positions of
  //		nodes within each level. 
  //		For each level that has nodes with constraints, we need to create
  //		such a graph.  
  //		<p>
  //		This graph also contains the basic transformations to keep the node
  //		neighboring and group structures up to date. See the documentation of
  //		<code> HRPNode</code> at the beginning.   
  //		</p>
  // tags:
  //		private
  
  constructor : function()
  {
  },
  
  addEdgeForceLeft : function(priority, unbreakable, source, target)
  {
    // summary:
    //		Adds an edge to the graph that should ensure that the source node is
    //		left of the target node. 
    // description:
    //		Adds an edge to the graph that should ensure that the source node is
    //		left of the target node. 
    //		In fact, it only adds the edge between source and target if both
    //		have the same parent. Otherwise it adds the edge between those two
    //		ancestors whose parent is the common ancestor of source and target. 
    //		
    // tags:
    //		private
    
    if (source == target) {
      return;
    }
    if (source.getParent() != target.getParent())
    {
      source.mark();
      var commonAncestor = target.getMarkedAncestor();
      source.unmark();
      if (source == commonAncestor) {
        return;
      }
      if (target == commonAncestor) {
        return;
      }
      while (source.getParent() != commonAncestor)
        source = source.getParent();

      while (target.getParent() != commonAncestor)
        target = target.getParent();
    }

    if (source.isNeighbored())
    {
      if (!source.hasOrderedNeighbors())
      {
        var corner = source.getLeftmostOrExtremeNeighbor();
        var node = corner;
        var prev = null;
        var index;

        while (node != source && node != target)
        {
          index = node.getOppositeNeighborIndex(prev);
          prev = node;
          node = node.getNeighbor(index);
        }

        if (node == source)
        {
          corner.setRightNeighbor(corner.getOppositeNeighbor(null));
        }
        else
        {
          corner.setLeftNeighbor(corner.getOppositeNeighbor(null));
        }
      }
    }
    else
    {
      var edge = new HMAEdge(priority);
      if (unbreakable) {
        edge.makeUnbreakable();
      }
      this.addEdgeBetweenNodes(edge, source, target);
    }
  },

  addEdgeBetweenGroups : function(priority, source, target)
  {
    // summary:
    //		Add an edge between two disjoint groups of nodes that have the same
    //		parent. 
    // description:
    //		Add an edge between two disjoint groups of nodes that have the same
    //		parent. 
    //		We assume that the nodes are not neighbored. We
    //		don't need to add edges between neighbored nodes, only determine the ordering of the neighbor chain.  
    // tags:
    //		private
    
    if (source.length == 0) {
      return;
    }
    if (target.length == 0) {
      return;
    }
    var firstSourceNode = source[0];
    var parent = firstSourceNode.getParent();
    var sourceNode;
    var targetNode;
    var node;
    var edge;

    if (source.length == 1) {
      sourceNode = firstSourceNode;
    }
    else
    {
      sourceNode = new HRPNode(0);
      sourceNode.setParent(parent);
      this.addNode(sourceNode);
      var sourcee =  new TranslateUtil.ArrayIterator(source);

      while (sourcee.hasNext())
      {
        node = sourcee.next();

        edge = new HMAEdge(0.0);
        edge.makeUnbreakable();
        this.addEdgeBetweenNodes(edge, node, sourceNode);
      }
    }

    if (target.length == 1) {
      targetNode = target[0];
    }
    else
    {
      targetNode = new HRPNode(0);
      targetNode.setParent(parent);
      this.addNode(targetNode);
      var targete =  new TranslateUtil.ArrayIterator(target);

      while (targete.hasNext())
      {
        node = targete.next();

        edge = new HMAEdge(0.0);
        edge.makeUnbreakable();
        this.addEdgeBetweenNodes(edge, targetNode, node);
      }
    }

    this.addEdgeBetweenNodes(new HMAEdge(priority), sourceNode, targetNode);
  },

  init : function(westNode, eastNode)
  {
    // summary:
    //		Initialize the graph. 
    // description:
    //		Initialize the graph. 
    //		This introduces a dummy node as parent of all nodes to make sure
    //		that all nodes are connected into one tree. It also handles the
    //		unique east and west node, if they exist. This is performed before
    //		any other grouping is performed.
    // tags:
    //		private
    
    var numNodes = this.getNumberOfNodes();

    var root = new HRPNode(numNodes);
    var group = root;
    this.addNode(root);

    if (eastNode != null || westNode != null)
    {
      if (eastNode != null) {
        numNodes--;
      }
      if (westNode != null) {
        numNodes--;
      }
      group = new HRPNode(numNodes);
      this.addNode(group);
      group.setParent(root);
    }

    var iter = this.getNodes(false);
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node != root && node != group && node != eastNode && node != westNode) {
        node.setParent(group);
      }
    }

    if (westNode != null)
    {
      westNode.setParent(root);
      this.neighbor(westNode, group, true);
    }

    if (eastNode != null)
    {
      eastNode.setParent(root);
      this.neighbor(group, eastNode, true);
    }
  },

  canNeighbor : function(node1, node2, node1IsLeft)
  {
    // summary:
    //		Check whether node1 can be direct neighbor of node 2. 
    // description:
    //		Check whether node1 can be direct neighbor of node 2. 
    //		If <tt>node1IsLeft</tt> is true, the node1 wants to be left neighbor
    //		of node2. If it is false, then node1 can be left or right neighbor.
    
    if (node1IsLeft)
    {
      var node1RightNeighbor = node1.getRightNeighbor();
      var node2LeftNeighbor = node2.getLeftNeighbor();

      if (node1RightNeighbor == node2) {
        return true; // Boolean
      }
      if (node1.getLeftNeighbor() == node2) {
        return false; // Boolean
      }
      if (node1.hasOrderedNeighbors() && node1RightNeighbor != null) {
        return false; // Boolean
      }
      if (node2.hasOrderedNeighbors() && node2LeftNeighbor != null) {
        return false; // Boolean
      }
    }

    if (node1.getNeighbor(0) == node2) {
      return true; // Boolean
    }
    if (node1.getNeighbor(1) == node2) {
      return true; // Boolean
    }
    if (node1.getNeighbor(0) != null && node1.getNeighbor(1) != null) {
      return false; // Boolean
    }
    if (node2.getNeighbor(0) != null && node2.getNeighbor(1) != null) {
      return false; // Boolean
    }
    return !node1.isThisOrChainedNeighbor(node2); // Boolean
  },

  neighbor : function(node1, node2, node1IsLeft)
  {
    // summary:
    //		Neighbor a node with another node. 
    // description:
    //		Neighbor a node with another node. 
    //		If <tt>node1IsLeft</tt> is true, the node1 is left neighbor of
    //		node2. If it is false, then node1 can be left or right neighbor. We
    //		assume that the test is already performed whether this is possible.
    // tags:
    //		private
    
    node1.setNodeNeighbor(node2);
    if (node1IsLeft) {
      node2.setLeftNeighbor(node1);
    }
    else {
      node2.setNodeNeighbor(node1);
    }
  },

  assignPositions : function(position, leftmostCorner)
  {
    // summary:
    //		Assign position numbers to the neighbor chain.
    // tags:
    //		private
    
    var iter = leftmostCorner.getNeighborChain();

    while (iter.hasNext())
      iter.next().setPositionNumber(position++);
  },

  getNodes : function(backwards)
  {
    // summary:
    //		Returns the nodes of the graph.
    // tags:
    //		private
    
    return new HRPGraphNodeIterator(this, backwards); // ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNodeIterator
  },

  getEdges : function(backwards)
  {
    // summary:
    //		Returns the edges of the graph.
    // tags:
    //		private
    
    return new HRPGraphEdgeIterator(this, backwards); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdgeIterator
  }

}
);

HRPGraphEdgeIterator = declare(HTBaseEdgeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(graph, backwards)
  {
  },
  
  next : function()
  {
    return this.nextBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  },

  prev : function()
  {
    return this.prevBaseEdge(); // ibm_ilog.graphlayout.hierarchical.makeacyclic.HMAEdge
  }

}
);

HRPGraphNodeIterator = declare(HTBaseNodeIterator, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function(graph, backwards)
  {
  },
  
  next : function()
  {
    return this.nextBaseNode(); // ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode
  },

  prev : function()
  {
    return this.prevBaseNode(); // ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode
  }

}
);

return HRPGraph;
});
