/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/relpositioning/HRPNode",	// #5
  "../../internalutil/GraphModelUtil",	// #6
  "../../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HRPNode,	// #5
    GraphModelUtil,	// #6
    TranslateUtil	// #7
    ) {

var HRPOrdering = declare("ibm_ilog.graphlayout.hierarchical.relpositioning.HRPOrdering", null, 
{
  // summary:
  //		This class handles the fact that one group of nodes must be at lower
  //		position than another group of nodes. 
  // description:
  //		This class handles the fact that one group of nodes must be at lower
  //		position than another group of nodes. 
  //		This is achieved by adding ordering edges between the lower
  //		positioned and the higher positioned nodes. Precisely: we add only
  //		edges between nodes that have the same parent. Hence we do the
  //		following:  
  //		<p>
  //		We color all paths from leaves of the lower positioned group with
  //		blue, and all path from leaves of the higher positioned group with
  //		red. If a blue and a red path join, we color them violet.  
  //		</p>
  //		<p>
  //		Then, for each node that was colored, we add edges from the blue
  //		children to the violet children (if any), and from the violet
  //		children to the red children (if any). If there are no violet
  //		children, we add edges directly between the blue and the red
  //		children.  
  //		</p>
  //		<p>
  //		This mechanism imposes the correct ordering edges on the graph, if no
  //		node has more than one violet child. The argumentation is inductive: 
  //		<ol>
  //		 
  //		<li> If there is no violet child, the edges between blue and red
  //		children assures that the blue children are before the red children. 
  //		
  //		<li> If there is a violet child, then the violet subtree consists of
  //		red and blue nodes, and for these, the ordering holds by induction
  //		assumption. Hence, the edges from blue to violet ensure that the blue
  //		nodes are before the nodes of the violet subtree, and, by induction
  //		assumption, the blue nodes of the violet subtree are before the red
  //		nodes of the violet subtree, and the edges from the violet child to
  //		the red nodes ensure that the blue and red nodes of the violet
  //		subtree are before the red children, hence we can summarize: all blue
  //		nodes are ensured to be before the red nodes. 
  //		</ol>
  //		However, if a node has two violet children, the whole argumentation
  //		doesn't work, and the constraint that the blue nodes are before the red nodes is impossible, because it means that there are blue and red nodes intermixed in two subtrees. In order to satisfy the constraint, we would have to break the subtrees into a blue and a red part, but we cannot do this, because it would violate a side by side constraint (remember: each subtree represents a side by side constraint).  
  //		<p>
  //		Nevertheless, if we have two violet children, we still <i>partially</i> satisfy the constraint if all blue nodes are before all violet nodes before all red nodes.  
  //		</p>
  //		<p>
  //		If all children of a node are neighbored, we don't add any edge
  //		between them, but simply impose a ordering on the neighbor chain.
  //		Again, we try to achieve that blue is before violet, which is before
  //		red.  
  //		</p>
  //		<p>
  //		  
  //		</p>
  // tags:
  //		private
  
  _graph: null,

  constructor : function(graph)
  {
    this.init(graph);
  },
  
  init : function(graph)
  {
    // summary:
    //		Initializes the algorithm.
    // tags:
    //		private
    
    this._graph = graph;
  },

  run : function(priority, source, target)
  {
    // summary:
    //		Starts all ordering. 
    // description:
    //		Starts all ordering. 
    //		This is the main entry point, after initialization.  
    // priority: Number
    //		The priority of the constraint.
    // source: Array
    //		The vector of nodes of type 'HRPNode' that need
    //		to be at lower postion.
    // target: Array
    //		The vector of nodes of type 'HRPNode' that need
    //		to be at higher postion.
    // tags:
    //		private
    
    if (source.length == 0 || target.length == 0) {
      return;
    }
    this.markForOrdering( new TranslateUtil.ArrayIterator(source), HRPOrdering.BLUE);
    this.markForOrdering( new TranslateUtil.ArrayIterator(target), HRPOrdering.RED);

    this.collectRedBlueVioletChildren( new TranslateUtil.ArrayIterator(source));
    this.collectRedBlueVioletChildren( new TranslateUtil.ArrayIterator(target));

    this.createOrdering( new TranslateUtil.ArrayIterator(source), priority);
    this.createOrdering( new TranslateUtil.ArrayIterator(target), priority);
  },

  markForOrdering : function(e, color)
  {
    // summary:
    //		Marks all nodes of the input enumeration with one color, and
    //		propagate the color along the ancestor path.
    // tags:
    //		private
    
    while (e.hasNext())
      (e.next()).markForOrdering(color, false);
  },

  collectRedBlueVioletChildren : function(e)
  {
    // summary:
    //		Collects all blue, red and violet children for nodes of the input
    //		enumeration and for their ancestors.
    // tags:
    //		private
    
    var node;

    while (e.hasNext())
    {
      node = e.next();

      node.getParent().updateOrderingInfoFromChild(node);
    }
  },

  createOrdering : function(e, priority)
  {
    // summary:
    //		Tries to ensure the ordering by adding ordering edges between blue
    //		and violet and red children. 
    // description:
    //		Tries to ensure the ordering by adding ordering edges between blue
    //		and violet and red children. 
    //		This also unmarks the nodes again.
    // tags:
    //		private
    
    while (e.hasNext())
      this.createNodeOrdering(priority, e.next());
  },

  createNodeOrdering : function(priority, node)
  {
    // summary:
    //		Creates the ordering between the blue and violet and red children of
    //		the input node and all ancestors.  
    // tags:
    //		private
    
    while (node != null)
    {
      if (node.getColor() == HRPOrdering.WHITE) {
        return;
      }
      if (node.getNumberOfOrderingChildren() == 0)
      {
        node.unmarkForOrdering();
        node = node.getParent();
        continue;
      }

      var blueNodes = node.getBlueChildren();
      var redNodes = node.getRedChildren();
      var violetNodes = node.getVioletChildren();

      var child = null;
      if (blueNodes.length > 0) {
        child = blueNodes[0];
      }
      else if (redNodes.length > 0) {
        child = redNodes[0];
      }
      else {
        child = violetNodes[0];
      }
      if (child.isNeighbored()) {
        this.createNeighborOrdering(priority, child);
      }
      else
      {
        if (violetNodes.length > 0)
        {
          this._graph.addEdgeBetweenGroups(priority, blueNodes, violetNodes);
          this._graph.addEdgeBetweenGroups(priority, violetNodes, redNodes);
        }
        else {
          this._graph.addEdgeBetweenGroups(priority, blueNodes, redNodes);
        }
      }

      node.unmarkForOrdering();
      node = node.getParent();
    }
  },

  createNeighborOrdering : function(priority, node)
  {
    // summary:
    //		Imposes an ordering on the neighbor chain, such that the majority of
    //		blue nodes are at the left side and the majority of red nodes are at
    //		the right side. 
    // description:
    //		Imposes an ordering on the neighbor chain, such that the majority of
    //		blue nodes are at the left side and the majority of red nodes are at
    //		the right side. 
    //		Currently we ignore the priority. Maybe one day we want to reorder
    //		an ordered neighbor chain, if the priority of this constraint is
    //		higher than the priority that imposed the ordering.
    // tags:
    //		private
    
    if (node.hasOrderedNeighbors()) {
      return;
    }
    var corner = node.getLeftmostOrExtremeNeighbor();
    var i = 0;
    var minBlue = /*Number.MAX_VALUE*/2147483647;
    var maxBlue = -1;
    var minRed = /*Number.MAX_VALUE*/2147483647;
    var maxRed = -1;
    var minVio = /*Number.MAX_VALUE*/2147483647;
    var maxVio = -1;

    var iter = corner.getNeighborChain();

    while (iter.hasNext())
    {
      switch (iter.next().getColor())
      {
        case HRPOrdering.BLUE:
          maxBlue = i;
          break;
        case HRPOrdering.RED:
          maxRed = i;
          break;
        case HRPOrdering.VIOLET:
          maxVio = i;
          break;
      }
      i++;
    }

    i--;

    while (iter.hasPrev())
    {
      switch (iter.prev().getColor())
      {
        case HRPOrdering.BLUE:
          minBlue = i;
          break;
        case HRPOrdering.RED:
          minRed = i;
          break;
        case HRPOrdering.VIOLET:
          minVio = i;
          break;
      }
      i--;
    }

    i = ((maxBlue > -1 ? 1 : 0)) + ((maxRed > -1 ? 1 : 0)) + ((maxVio > -1 ? 1 : 0));

    if (i >= 2)
    {
      if (maxBlue < minVio && maxBlue < minRed && maxVio < minRed)
      {
        corner.setRightNeighbor(corner.getOppositeNeighbor(null));
      }
      else if (maxRed < minVio && maxRed < minBlue && maxVio < minBlue)
      {
        corner.setLeftNeighbor(corner.getOppositeNeighbor(null));
      }
    }
  }

}
);

lang.mixin(HRPOrdering,
{
  WHITE : null,
  BLUE : null,
  RED : null,
  VIOLET : null,
  _fields_init : function(c)
  {
    c = c || HRPOrdering;
    
    c.WHITE = HRPNode.WHITE;
    c.BLUE = HRPNode.BLUE;
    c.RED = HRPNode.RED;
    c.VIOLET = HRPNode.VIOLET;
  }
  
}
);

(function(){ HRPOrdering._fields_init(); })();

return HRPOrdering;
});
