/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/graphbase/BaseAlgorithm",	// #5
  "../../internalutil/GraphModelUtil",	// #6
  "../../internalutil/PriorityQueue",	// #7
  "../../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    BaseAlgorithm,	// #5
    GraphModelUtil,	// #6
    PriorityQueue,	// #7
    TranslateUtil	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var BaseAlgorithm = ibm_ilog.graphlayout.hierarchical.graphbase.BaseAlgorithm;
=====*/

var HRPSolving = declare("ibm_ilog.graphlayout.hierarchical.relpositioning.HRPSolving", BaseAlgorithm, 
{
  // summary:
  //		This algorithm solves the constraint network. 
  // description:
  //		This algorithm solves the constraint network. 
  //		This means, it assigns a position number to each node such that: 
  //		<ol>
  //		 
  //		<li> The nodes that have a common parent are ordered. If a node has
  //		position number i and represents k real nodes, then the next sibling
  //		has position number i+k. </li>  
  //		<li> If a node has position number i and represents k real nodes, all
  //		position numbers in its subtree are between i and i+k. </li>  
  //		<li> All leaves that represent a real node have disjoint position
  //		numbers. </li>  
  //		<li> The nodes of a neighbor chain cover a continuous range of the
  //		position numbers. </li>  
  //		<li> If there is an edge between two nodes, the source node has a
  //		lower position number than the higher nodes. </li>  
  //		<li> Unconstraint nodes have position numbers such that they are
  //		ordered after increasing barycenter value. That is, if there is a
  //		choice for somenodes which of them node has the lower position
  //		number, then the barycenter value acts as tie breaker. </li> 
  //		</ol>
  //		The last rule means that for the unconstraint nodes, we try to
  //		achieve the fewest link crossings by using the barycenter heuristic.
  //		This is what we do as well if the graph is completely unconstraint.  
  //		<p>
  //		The calling sequence is (for each level that has a HRPGraph as
  //		constraint network) as follows: 
  //		<ol>
  //		 
  //		<li> initialize via <code>init(graph)</code>. </li>   
  //		<li> transfer the barycenter values via <code>node.setBarycenter(a,
  //		b)</code> to all HRPNodes of that level. </li>   
  //		<li> call <code>run()</code> to solve the system. </li>  
  //		<li> transfer the position numbers from the HRPNodes to the HNodes,
  //		and sort the HNodes accordingly. </li> 
  //		</ol>  
  //		</p>
  //		<p>
  //		  
  //		</p>
  // tags:
  //		private
  
  _rootOfNesting: null,
  _queue: null,

  constructor : function(graph, layout, percController)
  {
    this.init(graph, layout, percController);
    this._queue = new HRPSolvePriorityQueue();
  },
  
  initGraph : function(graph)
  {
    // summary:
    //		Initializes the algorithm.  
    // tags:
    //		private
    
    this._graph = graph;
    this._rootOfNesting = null;

    var iter = this.getGraph().getNodes(false);

    while (iter.hasNext())
      iter.next().init();

    this._queue.init(this.getGraph().getNumberOfNodes());
  },

  run : function()
  {
    // summary:
    //		Starts the algorithm, after all barycenter values are set for each
    //		constraint nodes.
    // tags:
    //		private
    
    this.collectStartNodes();
    this.createOrderingChain();
    this.getGraph().restoreTemporaryRemovedNodes();
    this.getGraph().restoreTemporaryRemovedEdges(false);
    this.storePositionNumbers(this._rootOfNesting, 0);
    this.addPercPoints(1);
    this.layoutStepPerformed();
  },

  getGraph : function()
  {
    // summary:
    //		Returns the graph we are working on.
    // tags:
    //		private
    
    return this._graph; // ibm_ilog.graphlayout.hierarchical.relpositioning.HRPGraph
  },

  collectStartNodes : function()
  {
    // summary:
    //		Search all start nodes and puts them into the priority queue. 
    // description:
    //		Search all start nodes and puts them into the priority queue. 
    //		Start nodes are the nodes without incoming edges.
    //		Don't confuse this with the nesting structure that is implemented by the parent-child relationship. As side effect, we also determine the root of the nesting structure, that is the node that has no parent.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var iter = graph.getNodes(false);
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getIndegree() == 0) {
        this._queue.add(node);
      }
      if (node.getParent() == null) {
        this._rootOfNesting = node;
      }
    }

    this.addPercPoints(1);
    this.layoutStepPerformed();
  },

  createOrderingChain : function()
  {
    // summary:
    //		Creates the ordering chain, i.e. 
    // description:
    //		Creates the ordering chain, i.e. 
    //		calculates the "FirstChildInOrder" and the "SiblingInOrder" fields.
    //		This enables us to calculate the position number later by simply
    //		traversing the graph via these fields. It handles the entire graph
    //		at once, independent of the nesting structure.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var node;
    var otherNode;
    var parent;
    var edge;

    while (!this._queue.isEmpty())
    {
      node = this._queue.popMin();
      parent = node.getParent();

      if (parent != null)
      {
        if (node.isNeighbored()) {
          this.createOrderingForNeighborChain(node);
        }
        else {
          parent.appendToOrder(node);
        }
      }

      var iter = node.getOutEdges();

      while (iter.hasNext())
      {
        edge = iter.next();
        otherNode = edge.getHMATarget();
        graph.removeEdge(edge);
        if (otherNode == node) {
          continue;
        }
        if (otherNode.getIndegree() == 0) {
          this._queue.add(otherNode);
        }
      }

      graph.removeNode(node);
    }

    this.addPercPoints(1);
    this.layoutStepPerformed();
  },

  createOrderingForNeighborChain : function(node)
  {
    // summary:
    //		Calculates the ordering for a neighbor chain. 
    // description:
    //		Calculates the ordering for a neighbor chain. 
    //		We handle all nodes of the ordering chain at once. If the chain has
    //		ordered neighbors, the order is clear and unique. If the chain
    //		doesn't have ordered neighbors, we have to decide which corner is the left or right one. Apart from that, the created order must reflect the neighbor order of the chain.
    // tags:
    //		private
    
    var parent = node.getParent();

    if (parent.getFirstChildInOrder() != null) {
      return;
    }
    var corner = node.getLeftmostOrExtremeNeighbor();
    var iter;

    if (corner.hasOrderedNeighbors())
    {
      iter = corner.getNeighborChain();

      while (iter.hasNext())
        parent.appendToOrder(iter.next());
    }
    else
    {
      var baryArray = TranslateUtil.newArray(parent.getNumberOfChildren(), 0);
      var i = 0;
      iter = corner.getNeighborChain();

      while (iter.hasNext())
        baryArray[i++] = iter.next().getBaryCenter();
      var numInversions = this.numberOfInversions(baryArray);
      var numNonInversions = this.numberOfNonInversions(baryArray);

      if (numInversions < numNonInversions)
      {
        iter = corner.getNeighborChain();

        while (iter.hasNext())
          parent.appendToOrder(iter.next());
      }
      else
      {
        iter = corner.getExtremeNeighbor(corner.getOppositeNeighborIndex(null)).getNeighborChain();

        while (iter.hasNext())
          parent.appendToOrder(iter.next());
      }
    }
  },

  numberOfInversions : function(array)
  {
    // summary:
    //		Calculates the number of inversions in the input array. 
    // description:
    //		Calculates the number of inversions in the input array. 
    //		This is the O(n^2) variant, but we expect the array to be small, so
    //		speed doesn't matter.
    // tags:
    //		private
    
    var num = 0;

    for (var i = 0; i < array.length; i++)
      for (var j = i + 1; j < array.length; j++)
        if (array[i] > array[j]) {
          num++;
        }
    return num; // int
  },

  numberOfNonInversions : function(array)
  {
    // summary:
    //		Calculates the number of noninversions in the input array. 
    // description:
    //		Calculates the number of noninversions in the input array. 
    //		This is the O(n^2) variant, but we expect the array to be small, so
    //		speed doesn't matter.
    // tags:
    //		private
    
    var num = 0;

    for (var i = 0; i < array.length; i++)
      for (var j = i + 1; j < array.length; j++)
        if (array[i] < array[j]) {
          num++;
        }
    return num; // int
  },

  storePositionNumbers : function(node, number)
  {
    // summary:
    //		After creating the ordering chain, this final step traverses the
    //		graph and stores the final ordering numbers.
    // tags:
    //		private
    
    while (node != null)
    {
      node.setPositionNumber(number);
      this.storePositionNumbers(node.getFirstChildInOrder(), number);
      number += node.getNumberOfRepresentedNodes();
      node = node.getSiblingInOrder();
    }
  }

}
);

HRPSolvePriorityQueue = declare(PriorityQueue, 
{
  // summary:
  //		
  // tags:
  //		private
  
  constructor : function()
  {
    
  },

  getValue : function(obj)
  {
    // summary:
    //		For the priority queue.
    // tags:
    //		private
    
    return (obj).getBaryCenter(); // Number
  }

}
);

return HRPSolving;
});
