/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../../hierarchical/relpositioning/HRPNode",	// #5
  "../../internalutil/GraphModelUtil",	// #6
  "../../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HRPNode,	// #5
    GraphModelUtil,	// #6
    TranslateUtil	// #7
    ) {

var HRPTransform = declare("ibm_ilog.graphlayout.hierarchical.relpositioning.HRPTransform", null, 
{
  // summary:
  //		This class contains the collection of transformation that we need to
  //		do when creating a new group. 
  // description:
  //		This class contains the collection of transformation that we need to
  //		do when creating a new group. 
  //		These transformations keep the invariants of the graph up to date.  
  //		<p>
  //		The scenario is the following: We want to constraint a group "V" of
  //		nodes to be placed close together. This is, of course, not always
  //		possible. Therefore, we first color the constraint graph: the leaves
  //		that represent the groups of nodes "V" become black (all other nodes
  //		are white). An inner node that has only black children becomes black,
  //		and an inner node that has black and white children becomes graytone
  //		(actually, we distinguish between gray, blackwhite and whiteblack,
  //		see HRPColoring.java).  
  //		</p>
  //		<p>
  //		The task of the transformations is now to ensure that the black
  //		leaves are placed close together, i.e. without white leaves in
  //		between. Thus, we split gray nodes into two directly neighbored
  //		nodes: the one containing all the white children, the other
  //		containing all the black children. If we have a neighbored chain, we
  //		ensure that the one end remains black, that is, if we have to append
  //		black nodes, we put them to the black end, if we have to append white
  //		node, we put them to the white end.  
  //		</p>
  //		<p>
  //		The main entry point is run(V) for the group "V" of nodes to be
  //		placed close together.  
  //		</p>
  //		<p>
  //		  
  //		</p>
  // tags:
  //		private
  
  _graph: null,
  _numMarkedLeaves: null,

  constructor : function(graph)
  {
    this.init(graph);
  },
  
  init : function(graph)
  {
    // summary:
    //		Initializes the algorithm.
    // tags:
    //		private
    
    this._graph = graph;
    this._numMarkedLeaves = 0;
  },

  run : function(nodes)
  {
    // summary:
    //		Starts all transformations. 
    // description:
    //		Starts all transformations. 
    //		This is the main entry point. We assume that the graph is already
    //		colored.  
    // nodes: Array
    //		The vector of nodes of type 'HRPNode' that need
    //		to be grouped together.
    // tags:
    //		private
    
    if (nodes.length <= 1) {
      return;
    }
    this._numMarkedLeaves = nodes.length;

    var e =  new TranslateUtil.ArrayIterator(nodes);
    var node;

    while (e.hasNext())
    {
      node = e.next();

      this.transform(node);
      this.trace("\n");
    }

    e =  new TranslateUtil.ArrayIterator(nodes);

    while (e.hasNext())
    {
      node = e.next();
      node.unmarkForColoring();
    }
  },

  transform : function(node)
  {
    // summary:
    //		Perform the transformations for the input node. 
    // description:
    //		Perform the transformations for the input node. 
    //		This is mutually recursive and propagates from leaf to root.
    // tags:
    //		private
    
    var parent = node.getParent();

    if (this.isGroupComplete(node)) {
      return;
    }
    if (parent == null) {
      return;
    }
    if (parent.getColor() == HRPTransform.BLACK) {
      this.transform(parent);
    }
    else if (node.isNeighbored())
    {
      var grandparent = parent.getParent();

      if (parent.getNumberOfGraytoneChildren() != 0) {
        return;
      }
      if (this.isGroupComplete(parent)) {
        return;
      }
      if (grandparent == null) {
        return;
      }
      var blackcorner = node.getExtremeNeighbor(0);
      var othercorner = node.getExtremeNeighbor(1);

      if (blackcorner.getColor() != HRPTransform.BLACK)
      {
        var n = blackcorner;
        blackcorner = othercorner;
        othercorner = n;
      }

      if (grandparent.getNumberOfChildren() == 1) {
        this.transformSingleChild(parent);
      }
      if (parent.isNeighbored()) {
        this.transformSubchain(parent, blackcorner, othercorner);
      }
      else {
        this.transformFreeGrayChain(parent, blackcorner, othercorner);
      }
    }
    else
    {
      if (parent.getNumberOfChildren() == 1) {
        this.transformSingleChild(node);
      }
      this.transformFreeBlackChild(node);
    }
  },

  getOrCreateBlackSplitNode : function(node)
  {
    // summary:
    //		Returns, and, if necessary, creates the black split node for a node.
    //		
    // description:
    //		Returns, and, if necessary, creates the black split node for a node.
    //		
    //		Precisely, we need the black split node if there is at least one
    //		black child and either any white child, or if there are two graytone
    //		child.
    // tags:
    //		private
    
    var blackSplitNode = node.getBlackSplitNode();
    if (blackSplitNode != null) {
      return blackSplitNode; // ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode
    }
    this.createBlackSplitNode(node);
    return node.getBlackSplitNode(); // ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode
  },

  createBlackSplitNode : function(node)
  {
    // summary:
    //		Creates a black split node for a node that has not only black
    //		children. 
    // description:
    //		Creates a black split node for a node that has not only black
    //		children. 
    //		Precisely, we need the black split node if there is at least one
    //		black child and either any white child, or if there are two graytone
    //		child.
    // tags:
    //		private
    
    var blackSplitNode = node;

    if (node.getNumberOfBlackChildren() > 0)
    {
      if (this.isGroupComplete(node) || (node.getNumberOfGraytoneChildren() == 1 && node.getNumberOfWhiteChildren() > 0))
      {
        blackSplitNode = this.newNode();
        blackSplitNode.setColor(HRPTransform.BLACK);
        blackSplitNode.allocColorExtension();
        blackSplitNode.setParent(node);
      }
      else if (node.getNumberOfGraytoneChildren() == 0)
      {
        if (!node.isNeighbored())
        {
          var auxNode = this.newNode();

          auxNode.setColor(node.getColor());

          auxNode.allocColorExtension();
          auxNode.updateMarkedInfoOnAdd(node);
          auxNode.setParent(node.getParent());
          node.setParent(auxNode);
          blackSplitNode = this.newNode();
          blackSplitNode.allocColorExtension();
          blackSplitNode.setColor(HRPTransform.BLACK);
          blackSplitNode.setParent(auxNode);
        }
        else
        {
          blackSplitNode = this.newNode();
          blackSplitNode.allocColorExtension();
          blackSplitNode.setColor(HRPTransform.BLACK);
          blackSplitNode.setParent(node.getParent());
        }

        var index = node.getIndexOfBlackSideNeighbor();
        var n = node.getNeighbor(index);
        node.setNeighbor(index, blackSplitNode);
        blackSplitNode.setNeighbor(1 - index, node);
        blackSplitNode.copyOrdering(node);

        if (n != null)
        {
          blackSplitNode.setNeighbor(index, n);
          n.replaceNeighbor(node, blackSplitNode);
        }
      }
    }

    node.setBlackSplitNode(blackSplitNode);
  },

  transformSingleChild : function(node)
  {
    // summary:
    //		The node is the single child of its parent. 
    // description:
    //		The node is the single child of its parent. 
    //		We can replace the parent by the child.
    // tags:
    //		private
    
    var parent = node.getParent();
    if (parent == null) {
      return;
    }
    this.trace("T1 ");

    this.replaceNode(parent, node);
    this.removeNode(parent);

    if (node.getParent() != null && node.getParent().getNumberOfChildren() == 1) {
      this.transformSingleChild(node);
    }
  },

  transformSubchain : function(node, blackcorner, othercorner)
  {
    // summary:
    //		The node is the root of a chained group. 
    // description:
    //		The node is the root of a chained group. 
    //		There is a black corner and another corner in the chain. If the node
    //		is gray, the other corner is white, but if the node is black, the
    //		other corner is black, too. The node has neighbors. Hence we need to
    //		integrate the children chain into the node chain.
    // tags:
    //		private
    
    this.trace("T2 ");

    var leftNeighbor = node.getLeftNeighbor();
    var rightNeighbor = node.getRightNeighbor();

    this.setParentOfChain(blackcorner, node.getParent());

    var index = node.getIndexOfBlackSideNeighbor();
    var blackSideNeighbor = node.getNeighbor(index);
    var otherSideNeighbor = node.getNeighbor(1 - index);

    if (blackSideNeighbor != null)
    {
      blackcorner.replaceNeighbor(null, blackSideNeighbor);
      blackSideNeighbor.replaceNeighbor(node, blackcorner);
    }

    if (otherSideNeighbor != null)
    {
      othercorner.replaceNeighbor(null, otherSideNeighbor);
      otherSideNeighbor.replaceNeighbor(node, othercorner);
    }

    this.removeNode(node);

    if (leftNeighbor != null) {
      leftNeighbor.ensureOrderedNeighborsIfNecessary();
    }
    else if (rightNeighbor != null) {
      rightNeighbor.ensureOrderedNeighborsIfNecessary();
    }
    blackcorner.ensureOrderedNeighborsIfNecessary();
    othercorner.ensureOrderedNeighborsIfNecessary();

    this.transform(blackcorner);
  },

  transformFreeBlackChild : function(node)
  {
    // summary:
    //		The node is an unneighbored black child of a parent. 
    // description:
    //		The node is an unneighbored black child of a parent. 
    //		We move the node to the black split node of the parent, if any.
    // tags:
    //		private
    
    var parent = node.getParent();
    if (parent == null) {
      return;
    }

    var blackSplitNode = this.getOrCreateBlackSplitNode(parent);

    if (blackSplitNode != parent)
    {
      this.trace("Move Black (T3,T6,T8) ");

      node.setParent(blackSplitNode);

      if (blackSplitNode.getParent() == parent) {
        blackSplitNode.updateMarkedInfoOnAdd(node);
      }
      else if (blackSplitNode.getParent() == parent.getParent())
      {
        parent.updateMarkedInfoOnRemove(node);
        blackSplitNode.updateMarkedInfoOnAdd(node);
      }
      else
      {
        blackSplitNode.getParent().updateMarkedInfoOnAdd(node);
        blackSplitNode.updateMarkedInfoOnAdd(node);
      }

      if (parent.getNumberOfBlackChildren() == 0)
      {
        if (blackSplitNode.getNumberOfChildren() == 1)
        {
          this.transformSingleChild(node);

          if (parent.getColor() != HRPTransform.WHITE) {
            parent.setBlackSplitNode(node);
          }
          this.transform(node);
        }
        else {
          this.transform(blackSplitNode);
        }
      }
    }
  },

  transformFreeGrayChain : function(node, blackcorner, othercorner)
  {
    // summary:
    //		The node is the parent of a chained group. 
    // description:
    //		The node is the parent of a chained group. 
    //		There is a black corner and another corner in the chain.
    // tags:
    //		private
    
    var parent = node.getParent();
    if (parent == null) {
      return;
    }

    var blackSplitNode = this.getOrCreateBlackSplitNode(parent);
    var otherblackcorner;
    var othernode;

    if (blackSplitNode == parent)
    {
      if (parent.getNumberOfGraytoneChildren() == 1)
      {
        var attachNode;

        if (parent.getNumberOfBlackChildren() > 0)
        {
          attachNode = blackcorner;
          this.trace("T4 ");
        }
        else
        {
          attachNode = othercorner;
          this.trace("T5 ");
        }

        parent.updateMarkedInfoOnRemove(node);
        node.updateMarkedInfoOnAdd(parent);
        this.replaceNode(parent, node);
        parent.setParent(node);
        parent.setNeighbor(0, attachNode);
        parent.setNeighbor(1, null);
        attachNode.replaceNeighbor(null, parent);
        attachNode.ensureOrderedNeighborsIfNecessary();

        if (node.isNeighbored() && !this.isGroupComplete(node))
        {
          if (attachNode == blackcorner) {
            this.transformSubchain(node, parent, othercorner);
          }
          else {
            this.transformSubchain(node, blackcorner, parent);
          }
        }
        else {
          this.transform(blackcorner);
        }
      }
      else
      {
        if (parent.getGrayChainBlackCorner() == null) {
          parent.setGrayChainBlackCorner(blackcorner);
        }
        else
        {
          this.trace("T7 ");
          otherblackcorner = parent.getGrayChainBlackCorner();
          othernode = otherblackcorner.getParent();
          othernode.updateMarkedInfoOnAdd(node);
          this.setParentOfChain(blackcorner, othernode);
          otherblackcorner.replaceNeighbor(null, blackcorner);
          blackcorner.replaceNeighbor(null, otherblackcorner);
          otherblackcorner.ensureOrderedNeighborsIfNecessary();
          blackcorner.ensureOrderedNeighborsIfNecessary();
          this.removeNode(node);

          if (parent.getNumberOfWhiteChildren() == 0) {
            this.transformSingleChild(node);
          }
        }
      }
    }
    else
    {
      if (parent.getNumberOfGraytoneChildren() == 1)
      {
        this.trace("T6 ");
        node.updateMarkedInfoOnAdd(blackSplitNode);
        blackSplitNode.setParent(node);
        blackSplitNode.setNeighbor(0, blackcorner);
        blackcorner.replaceNeighbor(null, blackSplitNode);
        blackcorner.ensureOrderedNeighborsIfNecessary();

        if (!this.isGroupComplete(parent))
        {
          this.trace("T5inT6 ");
          parent.updateMarkedInfoOnRemove(node);
          node.updateMarkedInfoOnAdd(parent);
          this.replaceNode(parent, node);
          parent.setParent(node);
          parent.setNeighbor(0, othercorner);
          parent.setNeighbor(1, null);
          othercorner.replaceNeighbor(null, parent);
          othercorner.ensureOrderedNeighborsIfNecessary();

          if (node.isNeighbored()) {
            this.transformSubchain(node, blackSplitNode, parent);
          }
          else {
            this.transform(blackSplitNode);
          }
        }
      }
      else
      {
        if (parent.getGrayChainBlackCorner() == null) {
          parent.setGrayChainBlackCorner(blackcorner);
        }
        else
        {
          this.trace("T8 ");
          otherblackcorner = parent.getGrayChainBlackCorner();
          othernode = otherblackcorner.getParent();
          othernode.updateMarkedInfoOnAdd(node);
          othernode.updateMarkedInfoOnAdd(blackSplitNode);
          this.setParentOfChain(blackcorner, othernode);
          blackSplitNode.setParent(othernode);
          blackSplitNode.setNeighbor(0, blackcorner);
          blackSplitNode.setNeighbor(1, otherblackcorner);
          otherblackcorner.replaceNeighbor(null, blackSplitNode);
          blackcorner.replaceNeighbor(null, blackSplitNode);
          otherblackcorner.ensureOrderedNeighborsIfNecessary();
          blackcorner.ensureOrderedNeighborsIfNecessary();
          this.removeNode(node);

          if (parent.getNumberOfWhiteChildren() == 0) {
            this.transformSingleChild(node);
          }
        }
      }
    }
  },

  newNode : function()
  {
    // summary:
    //		Create a new node.
    // tags:
    //		private
    
    var node = new HRPNode(0);
    this._graph.addNode(node);
    return node; // ibm_ilog.graphlayout.hierarchical.relpositioning.HRPNode
  },

  removeNode : function(node)
  {
    // summary:
    //		Removes a node.
    // tags:
    //		private
    
    this._graph.removeNode(node);
    node.dispose();
  },

  isGroupComplete : function(node)
  {
    // summary:
    //		Returns true if the group is complete at the input node. 
    // description:
    //		Returns true if the group is complete at the input node. 
    //		That means, all nodes that belong to the group to be formed are
    //		descendent of the input node.
    // tags:
    //		private
    
    return (node.getNumberOfMarkedLeaves() == this._numMarkedLeaves); // Boolean
  },

  setParentOfChain : function(chainNode, parent)
  {
    // summary:
    //		Set the parent for a chain of neighbored nodes.  
    // chainNode: HRPNode
    //		The leftmost or rightmost node of the neighbor chain.
    // parent: HRPNode
    //		The parent to be set.
    // tags:
    //		private
    
    var iter = chainNode.getNeighborChain();

    while (iter.hasNext())
      iter.next().setParent(parent);
  },

  replaceNode : function(source, target)
  {
    // summary:
    //		Replace the original node by the new node. 
    // description:
    //		Replace the original node by the new node. 
    //		The source node is not automatically set free. It is the
    //		responsability of the caller to either remove the source node, or to
    //		change parent and both neighbors of the source node afterwards.  
    // source: HRPNode
    //		The source node to be replaced.
    // target: HRPNode
    //		The replacing target node.
    // tags:
    //		private
    
    var n0 = source.getNeighbor(0);
    var n1 = source.getNeighbor(1);
    target.setNeighbor(0, n0);
    target.setNeighbor(1, n1);
    target.setParent(source.getParent());
    target.copyOrdering(source);
    if (n0 != null) {
      n0.replaceNeighbor(source, target);
    }
    if (n1 != null) {
      n1.replaceNeighbor(source, target);
    }
  },

  trace : function(msg)
  {
    // summary:
    //		Trace the tansformation.
    // tags:
    //		private
    
  }

}
);

lang.mixin(HRPTransform,
{
  BLACK : null,
  WHITE : null,
  _fields_init : function(c)
  {
    c = c || HRPTransform;
    
    c.BLACK = HRPNode.BLACK;
    c.WHITE = HRPNode.WHITE;
  }
  
}
);

(function(){ HRPTransform._fields_init(); })();

return HRPTransform;
});
