/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var ArrayStableSort = declare("ibm_ilog.graphlayout.internalutil.ArrayStableSort", null, 
{
  // summary:
  //		A generic sort algorithm for arrays. 
  // description:
  //		A generic sort algorithm for arrays. 
  //		We
  //		can't use the one in internalutils, because it is not stable, i.e., two elements with equal key might change their positions. This is deadly for the crossing reduction, therefore we need a stable quicksort there.
  
  constructor : function()
  {
  },
  
  sort : function(a)
  {
    // summary:
    //		Sorts the input array of objects. 
    // description:
    //		Sorts the input array of objects. 
    //		This sort is quaranteed to be <i>stable</i>: equal elements will not
    //		be reordered as a result of the sort.
    
    var n = a.length;
    if (n == 1) {
      return;
    }
    if (n == 2)
    {
      if (!this.compare(a[0], a[1]))
      {
        var aux1 = a[0];
        a[0] = a[1];
        a[1] = aux1;
      }

      return;
    }

    var b = TranslateUtil.newArray(n, null);
    var runs = TranslateUtil.newArray(n + 2, 0);
    var aux;
    var aux2;

    var i = 0;
    var j = 0;
    var s = 0;
    var lo;
    var hi;

    while (i < n)
    {
      do
      {
        aux = a[i++];
      }
      while (i < n && this.compare(aux, a[i]));
      runs[j++] = i;

      s = i;

      while (i < n && this.compare(a[i], aux))
      {
        if (!this.compare(aux, a[i]))
        {
          if (s < i - 1)
          {
            lo = s;
            hi = i - 1;

            while (lo < hi)
            {
              aux2 = a[lo];
              a[lo] = a[hi];
              a[hi] = aux2;
              lo++;
              hi--;
            }
          }

          s = i;
        }

        aux = a[i++];
      }

      if (s < i - 1)
      {
        lo = s;
        hi = i - 1;

        while (lo < hi)
        {
          aux2 = a[lo];
          a[lo] = a[hi];
          a[hi] = aux2;
          lo++;
          hi--;
        }
      }

      runs[j++] = i;
    }

    runs[j] = n;

    var done = false;

    while (!done)
    {
      var first = this.mergeruns(a, b, runs, n);
      var second = this.mergeruns(b, a, runs, n);
      done = first || second;
    }
  },

  mergeruns : function(src, dst, runs, n)
  {
    // summary:
    //		Merge bitonic sequences into ascending or descending sequences. 
    // description:
    //		Merge bitonic sequences into ascending or descending sequences. 
    //		By this, the data gets copied from src to dst. It also updates the
    //		run array to have the markers for the new bitonic sequences.  
    // src: Array
    //		The original data
    // dst: Array
    //		The target array that will contain the sorted sequences
    // runs: Array
    //		contains the indexe were each ascending and descending
    //		sequence starts.
    // n: int
    //		The size of the source array
    // returns:
    //		true if dst is sorted at the end. Or with other words, if the
    //		input src contains only one bitonic sequence.
    // tags:
    //		private
    
    var i = 0;
    var j = 0;
    var k = 0;
    var r = 0;
    var m;
    var ascending = true;

    while (i < n)
    {
      k = i;
      m = runs[j++];
      i = runs[j++];
      this.merge(src, dst, k, i - 1, m, ascending);
      ascending = !ascending;
      runs[r++] = i;
    }

    runs[r] = n;
    return k == 0; // Boolean
  },

  merge : function(src, dst, lo, hi, mid, asc)
  {
    // summary:
    //		Merge a bitonic sequence into a sorted sequence.  
    // src: Array
    //		The original data
    // dst: Array
    //		The target array that will contain the sorted sequences
    // lo: int
    //		The lowest index where the sequence starts.
    // hi: int
    //		The highest index where the sequence ends (inclusive).
    // mid: int
    //		The index where the descending part starts (inclusive).
    // asc: Boolean
    //		Whether the result should be sorted ascending or descending.
    // tags:
    //		private
    
    var k = (asc ? lo : hi);
    var c = (asc ? 1 : -1);
    var i = lo;
    var j = hi;

    while (i < mid && j >= mid)
    {
      if (this.compare(src[i], src[j]))
      {
        dst[k] = src[i++];
      }
      else
      {
        dst[k] = src[j--];
      }

      k += c;
    }

    while (i < mid)
    {
      dst[k] = src[i++];
      k += c;
    }

    while (j >= mid)
    {
      dst[k] = src[j--];
      k += c;
    }
  },

  compare : function(o1, o2)
  {
    // summary:
    //		Compare two elements. 
    // description:
    //		Compare two elements. 
    //		For ascendent sort, must return true if element(loc1)
    //		<= element(loc2), false otherwise. For descendent sort,  must return true if element(loc1) >=
    //		element(loc2), false otherwise.
    // tags:
    //		protected
    }
  
}
);

return ArrayStableSort;
});
