/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutParametersUtil",	// #6
  "../internalutil/PositionData",	// #7
  "../internalutil/TopologicalData",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutParametersUtil,	// #6
    PositionData,	// #7
    TopologicalData,	// #8
    TranslateUtil	// #9
    ) {

var GraphLayoutData = declare("ibm_ilog.graphlayout.internalutil.GraphLayoutData", null, 
{
  // summary:
  //		A class to store data structures concerning the topology of the graph
  //		(nodes/links by integer nodeId/linkId, what linkId connects two given
  //		nodeIds...) and concerning the positions of the nodes (what
  //		coordinates of a nodeId...). 
  // description:
  //		A class to store data structures concerning the topology of the graph
  //		(nodes/links by integer nodeId/linkId, what linkId connects two given
  //		nodeIds...) and concerning the positions of the nodes (what
  //		coordinates of a nodeId...). 
  //		This class is used internally by UniformLengthEdgesLayout,
  //		SpringEmbedderLayout and TopologicalMeshLayout.
  
  _layout: null,
  _positionData: null,
  _topologicalData: null,
  _fixedNodeInterface: null,

  constructor : function(layout, fixedNodeInterface )
  {
    this._layout = layout || null ;
    this._fixedNodeInterface = fixedNodeInterface;
  },
  
  getPositionData : function()
  {
    // summary:
    //		Returns the PositionData instance.
    
    return this._positionData; // ibm_ilog.graphlayout.internalutil.PositionData
  },

  getTopologicalData : function()
  {
    // summary:
    //		Returns the TopologicalData instance.
    
    return this._topologicalData; // ibm_ilog.graphlayout.internalutil.TopologicalData
  },

  beforeLayout : function(needsLinksBetweenNodes, needsNeighbours, oneConnectedComp, memorySavings)
  {
    // summary:
    //		Performs some extra operations before the layout.  
    // needsLinksBetweenNodes: Boolean
    //		True if it must create a data structure
    //		for the links.
    // needsNeighbours: Boolean
    //		True if it must create a data structure for the
    //		neighbours.
    // oneConnectedComp: Boolean
    //		true if we work only on the connected component
    //		which contains the starting node, false otherwise.
    // memorySavings: Boolean
    //		True if it should try to minimize the memory
    //		consumption.
    
    if (!this._layout.isStructureUpToDate() || this._topologicalData == null)
    {
      this.updateTopologicalData(oneConnectedComp);
      if (needsLinksBetweenNodes) {
        this._topologicalData.computeWhatLinkBetweenTwoNodes(memorySavings);
      }
      if (needsNeighbours) {
        this._topologicalData.initializeNeighbours();
      }
    }

    if (!this._layout.isStructureUpToDate() || !this._layout.isGeometryUpToDate() || this._positionData == null) {
      this.updatePositionData();
    }
  },

  updateTopologicalData : function(oneConnectedComp)
  {
    // summary:
    //		Before redoing the layout, if the structure (topology) of the graph
    //		has changed, they are no more up to date in comparison with  the
    //		data stored in TopologicalData. 
    // description:
    //		Before redoing the layout, if the structure (topology) of the graph
    //		has changed, they are no more up to date in comparison with  the
    //		data stored in TopologicalData. 
    //		Hence, we need to rebuild this data.  
    // needsLinksBetweenNodes:
    //		True if it must create a data structure
    //		for the links.
    // needsNeighbours:
    //		True if it must create a data structure for the
    //		neighbours.
    // oneConnectedComp: Boolean
    //		true if we work only on the connected component
    //		which contains the starting node, false otherwise.
    // memorySavings:
    //		True if it should try to minimize the memory
    //		consumption.
    // tags:
    //		private
    
    if (this._topologicalData == null) {
      this._topologicalData = new TopologicalData(this._layout, this, oneConnectedComp);
    }
    else {
      this._topologicalData.initialize(oneConnectedComp);
    }
  },

  updatePositionData : function()
  {
    // summary:
    //		Before redoing the layout, if the positions of the nodes  were
    //		changed, they are no more up to date in comparison with  the data
    //		stored in PositionData. 
    // description:
    //		Before redoing the layout, if the positions of the nodes  were
    //		changed, they are no more up to date in comparison with  the data
    //		stored in PositionData. 
    //		Hence, we need to rebuild this data.
    // tags:
    //		private
    
    if (this._positionData == null) {
      this._positionData = new PositionData(this._layout, this);
    }
    else {
      this._positionData.initialize();
    }
  },

  setIntIdentifier : function(nodeOrLink, id)
  {
    // summary:
    //		Sets the unique int identifier associated with the node or link.
    
    if (id < 0) {
      id = -1;
    }
    LayoutParametersUtil.setNodeOrLinkProperty(this._layout, nodeOrLink, null, false, false, GraphLayoutData.NUMBER_PROPERTY, ((id < 0 ? null : (id))));
  },

  getIntIdentifier : function(nodeOrLink)
  {
    // summary:
    //		Returns the unique int identifier associated with the node or link. 
    // description:
    //		Returns the unique int identifier associated with the node or link. 
    //		Returns -1 if none was stored.
    
    var value = LayoutParametersUtil.getNodeOrLinkParameter(this._layout, nodeOrLink, GraphLayoutData.NUMBER_PROPERTY);

    if(typeof(value) == "number")
      return value;
    return -1; // int
  },

  isFixed : function(nodeId)
  {
    // summary:
    //		Returns 'true' if 'node' is a node contained 
    //		in the grapher managed by the layout instance and if it is a fixed
    //		node,  or 'false' otherwise. 
    // description:
    //		Returns <code>true</code> if <code>node</code> is a node contained 
    //		in the grapher managed by the layout instance and if it is a fixed
    //		node,  or <code>false</code> otherwise. 
    //		<p>
    //		For the TopologicalMeshLayout, the exterior nodes are also
    //		considered as fixed.
    //		</p>
    
    if (nodeId < 0 || nodeId >= this._topologicalData.getNumberOfNodes()) {
      return false; // Boolean
    }
    if (this._topologicalData == null) {
      return false; // Boolean
    }
    var node = this._topologicalData.getNode(nodeId);
    return ((this._fixedNodeInterface != null) ? this._fixedNodeInterface.isFixed(node) : (this._layout.isPreserveFixedNodes() && this._layout.isFixed(node))); // Boolean
  }

}
);

lang.mixin(GraphLayoutData,
{
  NUMBER_PROPERTY : null,
  _fields_init : function(c)
  {
    c = c || GraphLayoutData;
    
    c.NUMBER_PROPERTY = "__GraphLayoutNumber";
  }
  
}
);

(function(){ GraphLayoutData._fields_init(); })();

return GraphLayoutData;
});
