/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    TranslateUtil	// #7
    ) {

var GraphModelDFS = declare("ibm_ilog.graphlayout.internalutil.GraphModelDFS", null, 
{
  // summary:
  //		A depth-first search in a graph model. 
  // description:
  //		A depth-first search in a graph model.
  
  _graphModel: null,
  _propName: null,

  constructor : function(graphModel, propName)
  {
    var msg;

    if (graphModel == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" GraphLayout ", "graphlayout.expert.message.6078E");
    }

    if (propName == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" GraphLayout ", "graphlayout.expert.message.6079E");
    }

    this._graphModel = graphModel;
    this._propName = propName;
  },
  
  execute : function(startNode, clean)
  {
    // summary:
    //		Executes the DFS in 'graphModel' from
    //		'startNode'. 
    // description:
    //		Executes the DFS in <code>graphModel</code> from
    //		<code>startNode</code>. 
    //		IMPORTANT NOTE: with clean at true, ALL the nodes and links of the
    //		graph model are visited one more time, in order to remove the
    //		property used for marking. If the DFS is restarted several times
    //		from different starting points on the same graph, this would be much
    //		too costly. In such cases, you should pass false for clean and do
    //		the cleaning by your self, in an unique browse at the end.  
    // startNode: Object
    //		the starting node
    // clean: Boolean
    //		if true, the property objects are guarantied to be removed.
    
    if (startNode == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" GraphLayout ", "graphlayout.expert.message.6080E");
    }

    try
    {
      this.dfsVisit(startNode);
    }
    finally
    {
      if (clean) {
        this.clean();
      }
    }
  },

  dfsVisit : function(node)
  {
    // summary:
    //		Recursive depth-first search starting at 'node'.
    // tags:
    //		private
    
    this.startVisit(node);
    this.markNode(node);
    var links = new TranslateUtil.ArrayIterator(GraphModelUtil.getLinks(this._graphModel, node));

    while (links.hasNext())
    {
      var nextLink = links.next();

      if (!this.isMarkedLink(nextLink))
      {
        this.markLink(nextLink);
        var nextNode = GraphModelUtil.getOpposite(this._graphModel, nextLink, node);

        if (!this.isMarkedNode(nextNode))
        {
          this.traverseDiscovery(nextLink, node);
          if (!this.isDone()) {
            this.dfsVisit(nextNode);
          }
        }
        else {
          this.traverseBack(nextLink, node);
        }
      }
    }

    this.finishVisit(node);
  },

  markNode : function(node)
  {
    // summary:
    //		Marks 'node' as visited. 
    // description:
    //		Marks <code>node</code> as visited. 
    //		Subclass can override this method and <code>
    //		GraphModelDFS.isMarkedNode()</code> to implement a different marking
    //		mechanism.  
    // tags:
    //		protected
    
    this._graphModel.setParameter(node, this._propName, node);
  },

  isMarkedNode : function(node)
  {
    // summary:
    //		Returns 'true' if 'node' has been visited and
    //		'false' otherwise.  
    // tags:
    //		protected
    
    return this._graphModel.getParameter(node, this._propName) != null; // Boolean
  },

  markLink : function(link)
  {
    // summary:
    //		Marks 'link' as visited. 
    // description:
    //		Marks <code>link</code> as visited. 
    //		Subclass can override this method and <code>
    //		GraphModelDFS.isMarkedLink()</code> to implement a different marking
    //		mechanism.  
    // tags:
    //		protected
    
    this._graphModel.setParameter(link, this._propName, link);
  },

  isMarkedLink : function(link)
  {
    // summary:
    //		Returns 'true' if 'link' has been visited and
    //		'false' otherwise.  
    // tags:
    //		protected
    
    return this._graphModel.getParameter(link, this._propName) != null; // Boolean
  },

  clean : function()
  {
    // summary:
    //		Cleans the property that has been set on the nodes and links.
    // tags:
    //		protected
    
    if (this._graphModel != null)
    {
      var e = new TranslateUtil.ArrayIterator(GraphModelUtil.getNodesAndLinks(this._graphModel));

      while (e.hasNext())
        this._graphModel.setParameter(e.next(), this._propName, null);
    }
  },

  startVisit : function(node)
  {
    // summary:
    //		This method is called when starts the visit of 'node'. 
    // description:
    //		This method is called when starts the visit of <code>node</code>. 
    //		The default implementation does nothing.
    // tags:
    //		protected
    
  },

  finishVisit : function(node)
  {
    // summary:
    //		This method is called when ends the visit of 'node'. 
    // description:
    //		This method is called when ends the visit of <code>node</code>. 
    //		The default implementation does nothing.
    // tags:
    //		protected
    
  },

  traverseDiscovery : function(link, fromNode)
  {
    // summary:
    //		This method is called when a link is traversed. 
    // description:
    //		This method is called when a link is traversed. 
    //		The default implementation does nothing.
    // tags:
    //		protected
    
  },

  traverseBack : function(link, fromNode)
  {
    // summary:
    //		This method is called when a link is traversed back. 
    // description:
    //		This method is called when a link is traversed back. 
    //		The default implementation does nothing.
    // tags:
    //		protected
    
  },

  isDone : function()
  {
    // summary:
    //		Returns 'true' if the traversal must be stopped. 
    // description:
    //		Returns <code>true</code> if the traversal must be stopped. 
    //		The default implementation returns always <code>false</code>. You
    //		can override this method to implement an eary stopping mechanism.
    // tags:
    //		protected
    
    return false; // Boolean
  }

}
);

return GraphModelDFS;
});
