/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var PercCompleteController = declare("ibm_ilog.graphlayout.internalutil.PercCompleteController", null, 
{
  // summary:
  //		This class is a controller the percentage complete feature of
  //		algorithms that require multiple complex steps. 
  // description:
  //		This class is a controller the percentage complete feature of
  //		algorithms that require multiple complex steps. 
  //		It can be used as well if the real number of steps is not known from
  //		the beginning, but only an estimation is known.  
  //		<p>
  //		Simplest example: Assume the layout algo has 3 steps that iterate
  //		through all nodes and do something. The first step takes 10% of the
  //		overall time, and the other both take 60% and 30%. Note that the sum
  //		must be 100%. Then you use the controller for instance inside a
  //		layout class <code>XYZLayout</code> in this way:  
  //		</p>
  //		<pre> 
  //		controller.setStepEstimation(3);
  //		controller.startStep(10.0f, numberOfNodes);
  //		foreach node {
  //		  ... do something for one node ...
  //		  controller.addPoints(1);
  //		  ...
  //		  controller.notifyPercentageComplete(this);
  //		  layoutStepPerformed(); 
  //		}
  //		
  //		controller.startStep(60.0f, numberOfNodes);
  //		foreach node {
  //		  ... do something for one node ...
  //		  controller.addPoints(1);
  //		  ...
  //		  controller.notifyPercentageComplete(this);
  //		  layoutStepPerformed(); 
  //		}
  //		
  //		controller.startFinalStep(30.0f, numberOfNodes);
  //		foreach node {
  //		  ... do something for node with index i ...
  //		  controller.addPoints(1);
  //		  ...
  //		  controller.notifyPercentageComplete(this);
  //		  layoutStepPerformed(); 
  //		}
  //		controller.stopAll();
  //		</code> 
  //		<p>
  //		As result, the layout report will show an increase of the percentage
  //		of completion that is approximately linear in time.
  //		You need the <code>controller.notifyPercentageComplete(this)</code>
  //		only exactly before the <code>layoutStepPerformed()</code>, because it
  //		transfers the data to the layout report. The layout report is for customers
  //		accessible only in the layout listeners that are called by
  //		<code>layoutStepPerformed()</code>, and outside the layout run.
  //		<p>
  //		It is not necessary to have a <code>layoutStepPerformed()</code> per
  //		<code>controller.addPoint(1)</code>, but it is also nonsense to
  //		do <i>n</i> times <code>controller.addPoint(1)</code> if we are sure that
  //		there is no <code>layoutStepPerformed()</code> in between. Instead, you can
  //		call <code>controller.addPoint(n)</code> as one statement.
  //		<p>
  //		You can change your estimation about the number of steps, or of the number
  //		of points any time during the layout run. As earlier you do this in the
  //		layout run, as smoother will be the percentage increase. In particular, you
  //		should <i>not</i> change your estimation about the step number after the
  //		final step has started, and about the point number after the last point
  //		of one step is given.
  //		You should also avoid changing the estimation too dramatically, e.g. if
  //		your first estimation was 10 steps, and after the 9th step you realize
  //		that there are 1000 steps more to go, then we are in trouble. 
  //		<p>
  //		The easiest way to get an estimation of the percentage amount of each single
  //		step is to uncomment the <code>debugPrintTime()</code> statements and
  //		to try it with large graphs. If it prints always time a, b, c, then the
  //		step percentages are a*100/(a+b+c)%, b*100/(a+b+c)%, c*100/(a+b+c)%.
  //		<p>
  //		
  //		
  //		
  // tags:
  //		private
  
  _numberOfStepsNeeded: null,
  _numberOfStepsDone: null,
  _percentageOfPreviousSteps: null,
  _percentageOfCurrentStepFromAll: null,
  _numberOfPointsNeeded: null,
  _numberOfPointsDone: null,

  constructor : function()
  {
    this.init();
  },
  
  init : function()
  {
    // summary:
    //		Reinitialize the percentage complete controller. 
    // description:
    //		Reinitialize the percentage complete controller. 
    //		This should be called at the start of layout, if this is not a
    //		fresh, unused controller.
    // tags:
    //		private
    
    this._numberOfStepsNeeded = 0;
    this._numberOfStepsDone = 0;
    this._percentageOfPreviousSteps = 0.0;
    this._percentageOfCurrentStepFromAll = 0.0;
    this._numberOfPointsNeeded = 0;
    this._numberOfPointsDone = 0;
  },

  notifyPercentageComplete : function(layout)
  {
    // summary:
    //		Notify layout about the percentage complete.
    // tags:
    //		private
    
    var perc = this._percentageOfPreviousSteps + this._percentageOfCurrentStepFromAll * this._numberOfPointsDone / this._numberOfPointsNeeded;

    layout.increasePercentageComplete(Math.floor(perc));
  },

  setStepEstimation : function(numberOfStepsNeeded)
  {
    // summary:
    //		Sets the estimated number of steps needed for layout. 
    // description:
    //		Sets the estimated number of steps needed for layout. 
    //		This estimation can be changed during layout any time.
    // tags:
    //		private
    
    if (numberOfStepsNeeded > 0)
    {
      this._percentageOfPreviousSteps *= this._numberOfStepsNeeded / numberOfStepsNeeded;
      this._numberOfStepsNeeded = numberOfStepsNeeded;
    }
  },

  incrementStepEstimation : function(delta)
  {
    // summary:
    //		Increment the estimated number of steps needed for layout. 
    // description:
    //		Increment the estimated number of steps needed for layout. 
    //		This estimation can be changed during layout any time.
    // tags:
    //		private
    
    this.setStepEstimation(this._numberOfStepsNeeded + delta);
  },

  setPointEstimation : function(numberOfPointsNeeded)
  {
    // summary:
    //		Sets the estimated number of points needed for the current step. 
    // description:
    //		Sets the estimated number of points needed for the current step. 
    //		This estimation can be changed during layout any time.
    // tags:
    //		private
    
    this._numberOfPointsNeeded = numberOfPointsNeeded;
  },

  incrementPointEstimation : function(delta)
  {
    // summary:
    //		Increment the estimated number of points needed for the current
    //		step. 
    // description:
    //		Increment the estimated number of points needed for the current
    //		step. 
    //		This estimation can be changed during layout any time.
    // tags:
    //		private
    
    this._numberOfPointsNeeded += delta;
  },

  startStep : function(percentageOfStep, pointsOfStep, finalStep)
  {
    // summary:
    //		Starts a new step. 
    // description:
    //		Starts a new step. 
    //		It registers the layout step with the input estimation about the
    //		percentage this step needs compared to all steps, and the number of
    //		points this step needs to be completed.
    // tags:
    //		private
    
    this._numberOfStepsDone++;
    this._percentageOfPreviousSteps += this._percentageOfCurrentStepFromAll;

    if (!finalStep)
    {
      if (this._numberOfStepsDone >= this._numberOfStepsNeeded) {
        this.setStepEstimation(this._numberOfStepsNeeded + 1);
      }
    }

    this._percentageOfCurrentStepFromAll = percentageOfStep;
    this.setPointEstimation(pointsOfStep);
    this._numberOfPointsDone = 0;
  },

  stopAll : function()
  {
    // summary:
    //		Stop all.
    // tags:
    //		private
    
    this._numberOfStepsDone++;
    this._percentageOfPreviousSteps += this._percentageOfCurrentStepFromAll;
  },

  addPoints : function(delta)
  {
    // summary:
    //		Increments the number of points by delta.
    // tags:
    //		private
    
    this._numberOfPointsDone += delta;
    if (this._numberOfPointsDone > this._numberOfPointsNeeded) {
      this._numberOfPointsDone = this._numberOfPointsNeeded;
    }
  },

  getPoints : function()
  {
    // summary:
    //		returns the current number of points.
    // tags:
    //		private
    
    return this._numberOfPointsDone; // int
  }

}
);

return PercCompleteController;
});
