/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/Point",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    Point,	// #7
    Rectangle,	// #8
    TranslateUtil	// #9
    ) {

var PositionData = declare("ibm_ilog.graphlayout.internalutil.PositionData", null, 
{
  // summary:
  //		Contains some data structures related to the position of nodes. 
  // description:
  //		Contains some data structures related to the position of nodes. 
  //		
  // tags:
  //		private
  
  _layout: null,
  _graphModel: null,
  _layoutData: null,
  _topologicalData: null,
  _nx : null,
  _ny : null,
  _nbbox: null,

  _vectNodesPositionX: null,
  _vectNodesPositionY: null,

  _maxDelta : null,
  _atLeastOneNodeMoved : null,

  _vectNodesIds: null,
  _firstIndexOfMoveableNodes : null,
  _firstIndexOfFixedNodes : null,
  _numberOfMoveableNodes : null,
  _numberOfFixedNodes : null,

  _vectFixedNodesFlags: null,

  constructor : function(layout, layoutData)
  {
    this._nx = 0;
    this._ny = 0;
    this._maxDelta = 0;
    this._atLeastOneNodeMoved = false;
    this._firstIndexOfMoveableNodes = 0;
    this._firstIndexOfFixedNodes = 0;
    this._numberOfMoveableNodes = 0;
    this._numberOfFixedNodes = 0;
    this._layout = layout;
    this._layoutData = layoutData;
    this.initialize();
  },
  
  initialize : function()
  {
    this._graphModel = this._layout.getGraphModel();
    this._topologicalData = this._layoutData.getTopologicalData();
    this._atLeastOneNodeMoved = false;
    var numberOfNodes = this._topologicalData.getNumberOfNodes();

    if (this._vectNodesIds == null || numberOfNodes != this._vectNodesIds.length)
    {
      this._vectNodesPositionX = TranslateUtil.newArray(numberOfNodes, 0);
      this._vectNodesPositionY = TranslateUtil.newArray(numberOfNodes, 0);

      this._vectNodesIds = TranslateUtil.newArray(numberOfNodes, 0);
    }

    this.computeVectNodesPosition();
  },

  getVectNodesPositionX : function()
  {
    return this._vectNodesPositionX; // Array
  },

  getVectNodesPositionY : function()
  {
    return this._vectNodesPositionY; // Array
  },

  isAtLeastOneNodeMoved : function()
  {
    return this._atLeastOneNodeMoved; // Boolean
  },

  getXofNode : function(nodeId)
  {
    // summary:
    //		Returns the x coordinate of a node.
    // tags:
    //		private
    
    return this._vectNodesPositionX[nodeId]; // Number
  },

  getYofNode : function(nodeId)
  {
    // summary:
    //		Returns the y coordinate of a node.
    // tags:
    //		private
    
    return this._vectNodesPositionY[nodeId]; // Number
  },

  setNodePosition : function(nodeId, x, y)
  {
    // summary:
    //		Sets the x and y coordinates of a node.
    // tags:
    //		private
    
    this._vectNodesPositionX[nodeId] = x;
    this._vectNodesPositionY[nodeId] = y;
  },

  getDistX : function(nodeId1, nodeId2)
  {
    return this._vectNodesPositionX[nodeId1] - this._vectNodesPositionX[nodeId2]; // Number
  },

  getDistY : function(nodeId1, nodeId2)
  {
    return this._vectNodesPositionY[nodeId1] - this._vectNodesPositionY[nodeId2]; // Number
  },

  getDistPointX : function(nodeId, point)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return this._vectNodesPositionX[nodeId] - point.x; // Number
  },

  getDistPointY : function(nodeId, point)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return this._vectNodesPositionY[nodeId] - point.y; // Number
  },

  updateNodePositions : function()
  {
    // summary:
    //		Updates the node positions. 
    // description:
    //		Updates the node positions. 
    //		This method can only be called after the initialization of
    //		PositionData while the graph  structure did not change. It is more
    //		efficient to call it than to reconstruct from scratch.
    // tags:
    //		private
    
    this.computeVectNodesPosition();
  },

  computeVectNodesPosition : function()
  {
    var nodes = this._topologicalData.getNodes();
    if (nodes == null) {
      return;
    }
    var node;
    var nodeId;

    for (var i = 0; i < nodes.length; i++)
    {
      node = nodes[i];

      nodeId = this._layoutData.getIntIdentifier(node);

      this.computeNodePosition(node);
      this._vectNodesPositionX[nodeId] = this._nx;
      this._vectNodesPositionY[nodeId] = this._ny;
    }
  },

  computeNodePosition : function(node)
  {
    this._nbbox = GraphModelUtil.getObjectBounds(this._graphModel, node);
    this._nx = this._nbbox.x + this._nbbox.width * 0.5;
    this._ny = this._nbbox.y + this._nbbox.height * 0.5;
  },

  resetMaxMovementCounter : function()
  {
    // summary:
    //		Must be called BEFORE each iteration of a layout algorithm that
    //		needs to compute the maximum move of the nodes at each iteration in
    //		order to detect convergence.
    // tags:
    //		private
    
    this._maxDelta = 0;
  },

  getMaxMovement : function()
  {
    // summary:
    //		Returns the maximum move of a node.
    // tags:
    //		private
    
    return this._maxDelta; // Number
  },

  recordMove : function(deltaX, deltaY)
  {
    if (deltaX < 0) {
      deltaX = -deltaX;
    }
    if (deltaY < 0) {
      deltaY = -deltaY;
    }
    if (deltaX > this._maxDelta) {
      this._maxDelta = deltaX;
    }
    if (deltaY > this._maxDelta) {
      this._maxDelta = deltaY;
    }
  },

  getNumberOfMoveableNodes : function()
  {
    return this._numberOfMoveableNodes; // int
  },

  getNumberOfFixedNodes : function()
  {
    return this._numberOfFixedNodes; // int
  },

  getFirstIndexOfMoveableNodes : function()
  {
    return this._firstIndexOfMoveableNodes; // int
  },

  getFirstIndexOfFixedNodes : function()
  {
    return this._firstIndexOfFixedNodes; // int
  },

  getNodeId : function(k)
  {
    return this._vectNodesIds[k]; // int
  },

  identifyFixedAndMoveableNodes : function(hasAlwaysFixedNodes, storeFixedNodesFlags)
  {
    // summary:
    //		Records the fixed and moveable node ids. 
    // description:
    //		Records the fixed and moveable node ids. 
    //		Both are stored in the same array.  
    // hasAlwaysFixedNodes: Boolean
    //		With Topological Mesh Layout there are always
    //		fixed nodes, even if the preservation of fixed nodes is not
    //		enabled. This is because the nodes in the outer cycle are fixed
    //		during the second step of TML. The argument is true for TML and
    //		false for the others
    // storeFixedNodesFlags: Boolean
    //		If true, it stores a vector of flags
    //		allowing to quickly know if nodeId corresponds to a fixed node or
    //		not. This is optional because some layouts
    //		don't need it, they simply browse fixed (or non-fixed) nodes.
    // returns:
    //		true if ok, false if an error has occurred
    // tags:
    //		private
    
    this._numberOfMoveableNodes = 0;
    this._firstIndexOfFixedNodes = 0;
    this._numberOfFixedNodes = 0;
    if (this._layout == null) {
      return false; // Boolean
    }
    if (this._topologicalData == null) {
      return false; // Boolean
    }
    var numberOfNodes = this._topologicalData.getNumberOfNodes();
    if (numberOfNodes < 1) {
      return false; // Boolean
    }

    var preserveFixedNodes = this._layout.isPreserveFixedNodes();
    var needsStoringFixedNodesFlags = false;
    var nodeId;

    if (preserveFixedNodes || hasAlwaysFixedNodes)
    {
      if (storeFixedNodesFlags)
      {
        needsStoringFixedNodesFlags = true;

        if (this._vectFixedNodesFlags == null || numberOfNodes != this._vectFixedNodesFlags.length) {
          this._vectFixedNodesFlags = TranslateUtil.newArray(numberOfNodes, false);
        }
      }

      for (nodeId = 0; nodeId < numberOfNodes; nodeId++)
      {
        if (this._layoutData.isFixed(nodeId))
        {
          this._vectNodesIds[this._numberOfFixedNodes++] = nodeId;
          if (needsStoringFixedNodesFlags) {
            this._vectFixedNodesFlags[nodeId] = true;
          }
        }
      }
    }

    this._firstIndexOfMoveableNodes = this._firstIndexOfFixedNodes + this._numberOfFixedNodes;

    for (nodeId = 0; nodeId < numberOfNodes; nodeId++)
    {
      if (!this._layoutData.isFixed(nodeId))
      {
        this._vectNodesIds[this._firstIndexOfMoveableNodes + this._numberOfMoveableNodes++] = nodeId;
        if (needsStoringFixedNodesFlags) {
          this._vectFixedNodesFlags[nodeId] = false;
        }
      }
    }

    return true; // Boolean
  },

  hasBeenIdentifiedAsFixed : function(nodeId)
  {
    if (this._numberOfMoveableNodes == 0) {
      return true; // Boolean
    }
    if (this._numberOfFixedNodes == 0) {
      return false; // Boolean
    }
    return this._vectFixedNodesFlags[nodeId]; // Boolean
  },

  getNodesBarycenter : function()
  {
    // summary:
    //		Returns the barycenter of the nodes. 
    // description:
    //		Returns the barycenter of the nodes. 
    //		Notice that all the nodes are considered, including fixed.
    // tags:
    //		private
    
    var numberOfNodes = this._topologicalData.getNumberOfNodes();

    if (numberOfNodes < 1)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6084E",  numberOfNodes);
    }

    var xSum = 0;
    var ySum = 0;

    for (var nodeId = 0; nodeId < numberOfNodes; nodeId++)
    {
      xSum += this._vectNodesPositionX[nodeId];
      ySum += this._vectNodesPositionY[nodeId];
    }

    return new Point(xSum / numberOfNodes, ySum / numberOfNodes); // Point
  },

  rescale : function(factor)
  {
    // summary:
    //		Rescales with the given factor around the barycenter.  
    // factor: Number
    //		The scale factor.
    // tags:
    //		private
    
    if (factor == 0 || factor == 1) {
      return;
    }
    var x = 0;
    var y = 0;
    var barycenter = this.getNodesBarycenter();
    var nNodes = this._topologicalData.getNumberOfNodes();

    for (var nodeId = 0; nodeId < nNodes; nodeId++)
    {
      x = this.getXofNode(nodeId);
      y = this.getYofNode(nodeId);
      this.setNodePosition(nodeId, barycenter.x + (x - barycenter.x) * factor, barycenter.y + (y - barycenter.y) * factor);
    }
  },

  getNodesBoundingBox : function()
  {
    // summary:
    //		Returns the bounding box of the centers of the nodes. 
    // description:
    //		Returns the bounding box of the centers of the nodes. 
    //		Notice that all the nodes are considered, including fixed.
    // tags:
    //		private
    
    var numberOfNodes = this._topologicalData.getNumberOfNodes();

    if (numberOfNodes < 1)
    {
      LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6084E",  numberOfNodes);
    }

    var first = true;
    var bbox = new Rectangle();
    var point = new Point();

    for (var nodeId = 0; nodeId < numberOfNodes; nodeId++)
    {
      TranslateUtil.movePoint(point, this._vectNodesPositionX[nodeId], this._vectNodesPositionY[nodeId]);

      if (first)
      {
        TranslateUtil.moveRect(bbox, point.x, point.y);
        first = false;
      }
      else {
        TranslateUtil.addPoint(bbox, point .x, point .y);
      }
    }

    return bbox; // Rectangle
  },

  placeNodesAtPosition : function(redraw, centerPoint )
  {
    // summary:
    //		Moves all the nodes at the stored positions. 
    // description:
    //		Moves all the nodes at the stored positions. 
    //		Useful when you want to move all nodes in one operation.  
    // redraw: Boolean
    //		Redraw argument.
    // centerPoint: Point
    //		This is the point where the barycenter of the nodes
    //		should be after placement. If null, no recentering is done.
    // returns:
    //		false if at least one move could not be done, true otherwise
    // tags:
    //		private
    
    var dx = 0;
    var dy = 0;
    var currentCenterPoint = (centerPoint != null ? this.getNodesBarycenter() : null);

    if (currentCenterPoint != null)
    {
      dx = centerPoint.x - currentCenterPoint.x;
      dy = centerPoint.y - currentCenterPoint.y;
    }

    var wellDone = true;
    var maxIndex = this._firstIndexOfMoveableNodes + this._numberOfMoveableNodes;
    var nodeId;

    for (var moveableNodeId = this._firstIndexOfMoveableNodes; moveableNodeId < maxIndex; moveableNodeId++)
    {
      nodeId = this._vectNodesIds[moveableNodeId];
      if (!this.moveNode(nodeId, this.getXofNode(nodeId) + dx, this.getYofNode(nodeId) + dy, redraw)) {
        wellDone = false;
      }
    }

    return wellDone; // Boolean
  },

  moveNodes : function(redraw, point)
  {
    // summary:
    //		Moves all the nodes at the specified position. 
    // description:
    //		Moves all the nodes at the specified position. 
    //		Useful when you want to move all nodes in one operation.  
    // redraw: Boolean
    //		Redraw argument.
    // point: Point
    //		This is the point where the barycenter of the nodes should
    //		be after placement. If null, no recentering is done.
    // returns:
    //		false if at least one move could not be done, true otherwise
    // tags:
    //		private
    
    var x = point.x;
    var y = point.y;
    var wellDone = true;
    var maxIndex = this._firstIndexOfMoveableNodes + this._numberOfMoveableNodes;
    var nodeId;

    for (var moveableNodeId = this._firstIndexOfMoveableNodes; moveableNodeId < maxIndex; moveableNodeId++)
    {
      nodeId = this._vectNodesIds[moveableNodeId];
      if (!this.moveNode(nodeId, x, y, redraw)) {
        wellDone = false;
      }
    }

    return wellDone; // Boolean
  },

  moveNode : function(nodeId, x, y, redraw)
  {
    // summary:
    //		Useful if you want to move nodes one by one (currently done only in
    //		TMLdrawing) return false if it could not move the object at the
    //		desired position (the current version returns always true) and
    //		returns true otherwise. 
    // description:
    //		Useful if you want to move nodes one by one (currently done only in
    //		TMLdrawing) return false if it could not move the object at the
    //		desired position (the current version returns always true) and
    //		returns true otherwise. 
    //		The method is public because also called from
    //		topologicalmesh.TMLdrawing.
    // tags:
    //		private
    
    var node = this._topologicalData.getNode(nodeId);
    var nodeBBox = GraphModelUtil.getObjectBounds(this._graphModel, node);
    this._graphModel.moveNode(node, x - nodeBBox.width * 0.5, y - nodeBBox.height * 0.5);
    this._atLeastOneNodeMoved = true;

    this.setNodePosition(nodeId, x, y);
    return true; // Boolean
  }

}
);

return PositionData;
});
