/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var PriorityQueue = declare("ibm_ilog.graphlayout.internalutil.PriorityQueue", null, 
{
  // summary:
  //		An abstract priority queue. 
  // description:
  //		An abstract priority queue. 
  //		It allows to insert objects in O(log n) and to get the smallest
  //		element in O(1). It can remove the smallest element in O(log n).
  
  constructor : function()
  {
    
  this._numElements = 0;},

  _queue: null,

  _numElements : null,

  init : function(size)
  {
    // summary:
    //		Initializes the priority queue. 
    // description:
    //		Initializes the priority queue. 
    //		This must be called before the first object is added.  
    // size: int
    //		The maximum number of objects that need to be stored in the
    //		queue. If more objects are stored in the queue, the queue
    //		automatically resized itself.
    
    this._queue = TranslateUtil.newArray(((size > 0 ? size : 1)), null);
    this._numElements = 0;
  },

  clear : function()
  {
    // summary:
    //		Removes all elements from the priority queue. 
    // description:
    //		Removes all elements from the priority queue. 
    //		This should be called to avoid memory leaks when the queue
    //		isn't empty but is temporarily not needed anymore.
    // tags:
    //		private
    
    if (this._queue == null) {
      return;
    }
    for (var i = 0; i <= this._numElements; i++)
      this._queue[i] = null;
    this._numElements = 0;
  },

  resize : function()
  {
    // summary:
    //		Resizes the priority queue.
    // tags:
    //		private
    
    var h = TranslateUtil.newArray(2 * this._queue.length, null);

    for (var i = 0; i < this._numElements; i++)
      h[i] = this._queue[i];
    this._queue = h;
  },

  size : function()
  {
    // summary:
    //		Returns the number of elements stored in the queue.
    
    return this._numElements; // int
  },

  isEmpty : function()
  {
    // summary:
    //		Returns 'true' if the queue is empty.
    
    return this._numElements == 0; // Boolean
  },

  getValue : function(obj)
  {
    // summary:
    //		Returns the comparison value of the element.  
    // obj: Object
    //		The object of the queue.
    },
  
  setIndex : function(obj, i)
  {
    // summary:
    //		Tells the object its current index. 
    // description:
    //		Tells the object its current index. 
    //		Can be overridden. The default implementation does nothing.
    
  },

  getMin : function()
  {
    // summary:
    //		Return the smallest element of the priority queue.
    
    return this._queue[0]; // Object
  },

  popMin : function()
  {
    // summary:
    //		Return the smallest element and removes it from the priority queue.
    
    var obj = this.getMin();
    this.removeMin();
    return obj; // Object
  },

  add : function(obj)
  {
    // summary:
    //		Adds an object to the queue.  
    // obj: Object
    //		The new object.
    
    if (this._numElements == this._queue.length) {
      this.resize();
    }
    this.setIndex(obj, this._numElements);
    this._queue[this._numElements++] = obj;
    if (this._numElements > 1) {
      this.validateInvariantsUpwards(this._numElements - 1);
    }
  },

  removeMin : function()
  {
    // summary:
    //		Removes the smallest object from the priority queue.
    
    if (this.isEmpty()) {
      return;
    }
    this._queue[0] = this._queue[this._numElements - 1];
    this._queue[--this._numElements] = null;
    if (this._queue[0] != null) {
      this.setIndex(this._queue[0], 0);
    }
    if (this._numElements > 1) {
      this.validateInvariantsDownwards(0);
    }
  },

  update : function(index)
  {
    // summary:
    //		Update when the value of the object at given index has changed. 
    // description:
    //		Update when the value of the object at given index has changed. 
    //		This method comes handy to use this data structure instead of a
    //		fibonacci heap.
    
    this.validateInvariantsDownwards(index);
    if (index != 0) {
      this.validateInvariantsUpwards(index);
    }
  },

  validateInvariantsUpwards : function(k)
  {
    // summary:
    //		Establishes the heap invariant assuming the heap satisfies the
    //		invariant except possibly for the leaf indexed by k. 
    // description:
    //		Establishes the heap invariant assuming the heap satisfies the
    //		invariant except possibly for the leaf indexed by k. 
    //		This promotes the leave node upwards until the heap invariant is
    //		valid again. It needs time O(log &#95;numElements).
    // tags:
    //		private
    
    var parentIndex;
    var val = this.getValue(this._queue[k]);
    var aux;

    while (k > 0)
    {
      parentIndex = Math.floor((k - 1) / 2);
      if (this.getValue(this._queue[parentIndex]) <= val) {
        break;
      }
      aux = this._queue[parentIndex];
      this._queue[parentIndex] = this._queue[k];
      this._queue[k] = aux;
      this.setIndex(this._queue[parentIndex], parentIndex);
      this.setIndex(this._queue[k], k);
      k = parentIndex;
    }
  },

  validateInvariantsDownwards : function(k)
  {
    // summary:
    //		Establishes the heap invariant in the subheap rooted at k, which is
    //		assumed to satisfy the heap invariant except possibly for node k
    //		itself. 
    // description:
    //		Establishes the heap invariant in the subheap rooted at k, which is
    //		assumed to satisfy the heap invariant except possibly for node k
    //		itself. 
    //		This promotes the root node downwards until the heap invariant is
    //		valid again. It needs time O(log &#95;numElements).
    // tags:
    //		private
    
    var childIndex = 2 * k + 1;
    var val = this.getValue(this._queue[k]);
    var childVal;
    var childVal2;
    var aux;

    while (childIndex < this._numElements)
    {
      childVal = this.getValue(this._queue[childIndex]);

      if (childIndex + 1 < this._numElements)
      {
        childVal2 = this.getValue(this._queue[childIndex + 1]);

        if (childVal > childVal2)
        {
          childIndex++;
          childVal = childVal2;
        }
      }

      if (val <= childVal) {
        break;
      }
      aux = this._queue[childIndex];
      this._queue[childIndex] = this._queue[k];
      this._queue[k] = aux;
      this.setIndex(this._queue[childIndex], childIndex);
      this.setIndex(this._queue[k], k);
      k = childIndex;
      childIndex = 2 * k + 1;
    }
  }

}
);

return PriorityQueue;
});
