/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelDFS",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/HashTable",	// #7
  "../internalutil/KeyDataInt",	// #8
  "../internalutil/ListOfIntegers",	// #9
  "../internalutil/LogResUtil",	// #10
  "../internalutil/Rectangle",	// #11
  "../internalutil/TranslateUtil"	// #12
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelDFS,	// #5
    GraphModelUtil,	// #6
    HashTable,	// #7
    KeyDataInt,	// #8
    ListOfIntegers,	// #9
    LogResUtil,	// #10
    Rectangle,	// #11
    TranslateUtil	// #12
    ) {

var TopologicalData = declare("ibm_ilog.graphlayout.internalutil.TopologicalData", null, 
{
  // summary:
  //		A class that stores information about the topology of the graph and
  //		allows a quick access to it. 
  // description:
  //		A class that stores information about the topology of the graph and
  //		allows a quick access to it. 
  //		
  // tags:
  //		private
  
  _layout: null,

  _layoutData: null,
  _graphModel: null,
  _numberOfNodes : null,
  _numberOfLinks : null,
  _noMoveableNode : null,

  _vectNeighbours: null,

  _vectNodes: null,
  _vectLinks: null,

  _whatLinkBetweenTwoNodes: null,

  _neighbors : null,

  constructor : function(layout, layoutData, oneConnectedComp)
  {
    this._numberOfNodes = 0;
    this._numberOfLinks = 0;
    this._noMoveableNode = false;
    this._neighbors = new HashTable();
    this._layout = layout;
    this._layoutData = layoutData;
    this.initialize(oneConnectedComp);
  },
  
  initialize : function(oneConnectedComp)
  {
    this._graphModel = this._layout.getGraphModel();
    this.computeVectNodesAndLinks(oneConnectedComp);
  },

  initializeNeighbours : function()
  {
    if (this._vectNeighbours == null || this._vectNeighbours.length != this._numberOfNodes) {
      this._vectNeighbours = TranslateUtil.newArray(this._numberOfNodes, null);
    }
    var node;
    var numberOfNeighbours;
    var neigh;

    for (var nodeId = 0; nodeId < this._numberOfNodes; nodeId++)
    {
      node = this.getNode(nodeId);
      if (node == null) {
        continue;
      }
      numberOfNeighbours = GraphModelUtil.getNodeDegree(this._graphModel, node);
      if (numberOfNeighbours < 0) {
        continue;
      }
      this._vectNeighbours[nodeId] = new ListOfIntegers(numberOfNeighbours);

      this._neighbors.clear();
      var neighbors = new TranslateUtil.ArrayIterator(GraphModelUtil.getNeighbors(this._graphModel, node));

      while (neighbors.hasNext())
      {
        neigh = neighbors.next();

        if (neigh != null && neigh != node && !this._neighbors.containsKey(neigh))
        {
          this._vectNeighbours[nodeId].append(this._layoutData.getIntIdentifier(neigh));

          this._neighbors.add(neigh, neigh);
        }
      }
    }
  },

  getNeighbour : function(nodeId, neighbourId)
  {
    return this._vectNeighbours[nodeId].get(neighbourId); // int
  },

  getNumberOfNeighbours : function(nodeId)
  {
    return this._vectNeighbours[nodeId].length(); // int
  },

  getNodes : function()
  {
    // summary:
    //		Returns the array of nodes that is internally stored by this class. 
    // description:
    //		Returns the array of nodes that is internally stored by this class. 
    //		Do not modify it!
    // tags:
    //		private
    
    return this._vectNodes; // Array
  },

  getNode : function(nodeId)
  {
    return this._vectNodes[nodeId]; // Object
  },

  getNumberOfNodes : function()
  {
    return this._numberOfNodes; // int
  },

  getLink : function(linkId)
  {
    return this._vectLinks[linkId]; // Object
  },

  getNumberOfLinks : function()
  {
    return this._numberOfLinks; // int
  },

  countNodesAndLinks : function()
  {
    // summary:
    //		Must be done ONE TIME for each different graph (in terms of
    //		topology) or each time you perform the semi-automatic layout (to
    //		detect new fixed/moveable nodes). 
    // description:
    //		Must be done ONE TIME for each different graph (in terms of
    //		topology) or each time you perform the semi-automatic layout (to
    //		detect new fixed/moveable nodes). 
    //		Used only for Uniform and Spring. Not for TML, which uses a DFS to
    //		put the numbers.
    // tags:
    //		private
    
    this._numberOfNodes = 0;
    this._numberOfLinks = 0;
    var node;

    for (var nodes = new TranslateUtil.ArrayIterator(this._graphModel.getNodes()); nodes.hasNext();)
    {
      node = nodes.next();

      this._layoutData.setIntIdentifier(node, this._numberOfNodes++);
    }

    var link;

    for (var links = new TranslateUtil.ArrayIterator(this._graphModel.getLinks()); links.hasNext();)
    {
      link = links.next();
      this._layoutData.setIntIdentifier(link, this._numberOfLinks++);
    }
  },

  getLinkId : function(nodeId1, nodeId2)
  {
    // summary:
    //		Returns the linkId between two nodesIds. 
    // description:
    //		Returns the linkId between two nodesIds. 
    //		It only considers one link betw. the two nodes, chosen  arbitrarily
    //		among the multiple links (if any).  Returns a negative value if no
    //		link betw. these nodes. Used only by Spring Embedder,
    //		UniformLengthEdges and TML.
    // tags:
    //		private
    
    return this._whatLinkBetweenTwoNodes.get(TopologicalData.computeUniqueIndex(nodeId1, nodeId2, this._numberOfNodes)); // int
  },

  isLinkBetween : function(nodeId1, nodeId2)
  {
    return this._whatLinkBetweenTwoNodes.get(TopologicalData.computeUniqueIndex(nodeId1, nodeId2, this._numberOfNodes)) >= 0; // Boolean
  },

  computeVectNodesAndLinks : function(oneConnectedComp)
  {
    // summary:
    //		Fills arrays of nodes and links, for quick, indexed access.  
    // oneConnectedComp: Boolean
    //		true if we work only on the connected component
    //		which contains the starting node, false otherwise
    // tags:
    //		private
    
    if (oneConnectedComp) {
      this.computeVectNodesAndLinksOneConnectedComp();
    }
    else {
      this.computeVectNodesAndLinksGlobal();
    }
  },

  computeVectNodesAndLinksOneConnectedComp : function()
  {
  },

  computeVectNodesAndLinksGlobal : function()
  {
    this.countNodesAndLinks();

    this._vectNodes = TranslateUtil.newArray(this._numberOfNodes, null);
    this._vectLinks = TranslateUtil.newArray(this._numberOfLinks, null);
    var nodeId = 0;
    var linkId = 0;

    var checkFixedNodes = this._layout.supportsPreserveFixedNodes() && this._layout.isPreserveFixedNodes();
    var noMoveableNode = true;
    var nodes = new TranslateUtil.ArrayIterator(this._graphModel.getNodes());
    var node;
    var msg;

    while (nodes.hasNext())
    {
      node = nodes.next();

      if (checkFixedNodes && noMoveableNode && !this._layout.isFixed(node)) {
        noMoveableNode = false;
      }
      if (nodeId < this._numberOfNodes) {
        this._vectNodes[nodeId++] = node;
      }
      else
      {
        LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6087E");
      }
    }

    if (checkFixedNodes) {
      this.setNoMoveableNode(noMoveableNode);
    }
    else {
      this.setNoMoveableNode(false);
    }
    var links = new TranslateUtil.ArrayIterator(this._graphModel.getLinks());
    var link;

    while (links.hasNext())
    {
      link = links.next();

      if (linkId < this._numberOfLinks) {
        this._vectLinks[linkId++] = link;
      }
      else
      {
        LogResUtil.logAndThrowRuntimeExc(" GraphLayout ", "graphlayout.expert.message.6088E");
      }
    }
  },

  setNoMoveableNode : function(noMoveableNode)
  {
    this._noMoveableNode = noMoveableNode;
  },

  isNoMoveableNode : function()
  {
    return this._noMoveableNode; // Boolean
  },

  computeWhatLinkBetweenTwoNodes : function(memorySavings)
  {
    this._whatLinkBetweenTwoNodes = new KeyDataInt(this._numberOfNodes * this._numberOfNodes, false);

    var nodeId1;
    var nodeId2;
    var link;
    var from;
    var toobj;

    for (var linkId = 0; linkId < this._numberOfLinks; linkId++)
    {
      link = this._vectLinks[linkId];
      from = this._graphModel.getFrom(link);
      toobj = this._graphModel.getTo(link);
      nodeId1 = this._layoutData.getIntIdentifier(from);
      nodeId2 = this._layoutData.getIntIdentifier(toobj);

      this._whatLinkBetweenTwoNodes.put(TopologicalData.computeUniqueIndex(nodeId1, nodeId2, this._numberOfNodes), linkId);
    }
  },

  getModifiedLayoutRegion : function(layoutRegion)
  {
    // summary:
    //		Computes and returns the biggest rectangle such as: if the center of
    //		the nodes is moved somewhere inside it, we can be sure that no part
    //		of a node will be outside the layout region.
    // tags:
    //		private
    
    var maxWidth = 0;
    var maxHeight = 0;
    var node;
    var bbox;
    var nNodes = this.getNumberOfNodes();

    for (var i = 0; i < nNodes; i++)
    {
      node = this.getNode(i);
      bbox = GraphModelUtil.getObjectBounds(this._graphModel, node);
      if (bbox.width > maxWidth) {
        maxWidth = bbox.width;
      }
      if (bbox.height > maxHeight) {
        maxHeight = bbox.height;
      }
    }

    return new Rectangle(layoutRegion.x + maxWidth * 0.5, layoutRegion.y + maxHeight * 0.5, layoutRegion.width - maxWidth, layoutRegion.height - maxHeight); // Rectangle
  }

}
);

lang.mixin(TopologicalData,
{

  computeUniqueIndex : function(nodeId1, nodeId2, numberOfNodes)
  {
    return (nodeId1 >= nodeId2 ? (nodeId2 + nodeId1 * numberOfNodes) : (nodeId1 + nodeId2 * numberOfNodes)); // int
  }

}
);

TopologicalData.ConnectedComponentTraversal = declare(GraphModelDFS, 
{
  __outerThis: null,

  _vectNodes2: null,
  _vectLinks2: null,
  _nNodes : null,
  _nLinks : null,

  constructor : function(input__outerThis, graphModel, temporaryPropName)
  {
    this.__outerThis = input__outerThis;
    this._nNodes = 0;
    this._nLinks = 0;
  },
  
  execute : function(startNode, vectNodes, vectLinks)
  {
    // summary:
    // tags:
    //		private
    
    this._vectNodes2 = vectNodes;
    this._vectLinks2 = vectLinks;
    this._nNodes = 0;
    this._nLinks = 0;
    this.inherited("execute", arguments, [startNode, true]);
  },

  markNode : function(node)
  {
    this.inherited("markNode", arguments, [node]);
    this._vectNodes2.push(node);

    this.__outerThis._layoutData.setIntIdentifier(node, this._nNodes++);
  },

  markLink : function(link)
  {
    this.inherited("markLink", arguments, [link]);
    this._vectLinks2.push(link);

    this.__outerThis._layoutData.setIntIdentifier(link, this._nLinks++);
  },

  clean : function()
  {
    // summary:
    //		Overridden version that cleans the temporary property on the stored
    //		nodes and links, instead of cleaning all the nodes/links from the
    //		graph as the base class.
    // tags:
    //		private
    
    if (this._graphModel == null) {
      return;
    }
    var n = this._vectNodes2.length;
    var i;

    for (i = 0; i < n; i++)
      this._graphModel.setParameter(this._vectNodes2[i], this._propName, null);
    n = this._vectLinks2.length;

    for (i = 0; i < n; i++)
      this._graphModel.setParameter(this._vectLinks2[i], this._propName, null);
  }

}
);

return TopologicalData;
});
