/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/QuickSort",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../longlink/LLTerminationCandAlgorithm",	// #9
  "../Direction"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    QuickSort,	// #7
    TranslateUtil,	// #8
    LLTerminationCandAlgorithm,	// #9
    Direction	// #10
    ) {

var LLDirectRouteAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLDirectRouteAlgorithm", null, 
{
  // summary:
  //		The algorithm to calculate link routes if the style is DIRECT_STYLE. 
  // description:
  //		The algorithm to calculate link routes if the style is DIRECT&#95;STYLE. 
  //		Given all termination point candidates for all direct links, all we
  //		need to do is to choose the termination points wisely, i.e. such that
  //		the direct links
  //		don't overlap or cross each other. We do this per node, i.e. for all direct links at one node, we choose their order, and by that their termination points. We use a simple heuristic for this. Note that this alg. is by far not as intelligent as its counterpart in ShortLinkLayout. We did it only for compatibility, not because it is useful.  
  // tags:
  //		private
  
  _layout: null,

  _node: null,

  _occupiedMarker: null,
  _firstFreeOccupiedMarkerSlot: null,

  constructor : function(layout, grid)
  {
    this._layout = layout;
    this._node = null;
    this._occupiedMarker = null;
    this._firstFreeOccupiedMarkerSlot = -1;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._occupiedMarker = null;

    this._layout = null;
    this._node = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getNode : function()
  {
    // summary:
    //		Returns the reference node.
    // tags:
    //		private
    
    return (this._node); // Object
  },

  getTerminationPoints : function(link)
  {
    // summary:
    //		Get the termination point array of the link.
    // tags:
    //		private
    
    if (link.getFromNode() == this.getNode()) {
      return link.getStartPoints(); // Array
    }
    else {
      return link.getEndPoints(); // Array
    }
  },

  setTerminationPoint : function(link, index)
  {
    // summary:
    //		Set the actual termination point of the link.
    // tags:
    //		private
    
    if (link.getFromNode() == this.getNode())
    {
      link.setActStartCandidate(index);
    }
    else
    {
      link.setActEndCandidate(index);
    }
  },

  route : function(node, links)
  {
    // summary:
    //		Route all direct links that are incident to this node. 
    // description:
    //		Route all direct links that are incident to this node. 
    //		This only calculates the actual termination point of the links in
    //		the input vector that corresponds to the end where the input node
    //		is. That is, for links that start at this node, it calculate
    //		actStartPoint, and for links that end at this node, it calculate
    //		actEndPoint.  Self loops are never in the link vector.
    // tags:
    //		private
    
    this._node = node;

    var e =  new TranslateUtil.ArrayIterator(links);
    var link;

    while (e.hasNext())
    {
      link = e.next();
      link.setPreferedSide(this.getLinkSide(link));
    }

    this.calcLinksAtSide(Direction.TOP, links);
    this.calcLinksAtSide(Direction.BOTTOM, links);
    this.calcLinksAtSide(Direction.LEFT, links);
    this.calcLinksAtSide(Direction.RIGHT, links);
    return (true); // Boolean
  },

  calcLinksAtSide : function(side, links)
  {
    // summary:
    //		Assign termination points for the links that want to go to the
    //		specified side.
    // tags:
    //		private
    
    var linksAtSide = new Array();
    var e =  new TranslateUtil.ArrayIterator(links);
    var link;

    while (e.hasNext())
    {
      link = e.next();
      if (link.getPreferedSide() == side) {
        linksAtSide.push(link);
      }
    }

    if (linksAtSide.length == 0) {
      return;
    }

    var correction = 0.0;
    var dir = 1;
    var sign = -1;
    var middleOffset = 0;
    var incrOnPositive = -1;
    var incrOnNegative = 0;

    switch (side)
    {
      case Direction.RIGHT:
        correction = -Math.PI;
        dir = 1;
        sign = 1;
        middleOffset = 1;
        incrOnPositive = 0;
        incrOnNegative = 1;
        break;
      case Direction.TOP:
        correction = -0.5 * Math.PI;
        dir = 0;
        sign = 1;
        middleOffset = 1;
        incrOnPositive = 0;
        incrOnNegative = 1;
        break;
      case Direction.BOTTOM:
        correction = -1.5 * Math.PI;
        dir = 0;
        sign = -1;
        middleOffset = 0;
        incrOnPositive = -1;
        incrOnNegative = 0;
        break;
    }
    
    e =  new TranslateUtil.ArrayIterator(linksAtSide);

    while (e.hasNext())
    {
      link = e.next();

      this.calcRadialSortValue(link, correction);
    }

    var sortAlg = new LLDirectRouteAlgorithm.LinkSort(this, linksAtSide);
    sortAlg.sort();
    sortAlg.dispose();

    this.clearOccupied(linksAtSide.length);

    var middle = Math.floor((linksAtSide.length - middleOffset) / 2);
    link = linksAtSide[middle];
    this.setTerminationPoint(link, 0);
    this.markOccupied(sign * this.getTerminationPoints(link)[0].getIndex(dir));
    var incr = incrOnPositive + incrOnNegative;
    var leftMostIndex = sign * this.getTerminationPoints(link)[0].getIndex(dir);
    var rightMostIndex = sign * this.getTerminationPoints(link)[0].getIndex(dir);
    var penalty;
    var bestPenalty;
    var bestPointIndex;
    var bestIsOccupied;
    var coordIndex;
    var points;

    var i;
    var j;

    for (i = 1; i < linksAtSide.length; i++)
    {
      link = linksAtSide[middle + incr];
      points = this.getTerminationPoints(link);
      bestPenalty = 0;
      bestPointIndex = 0;
      bestIsOccupied = true;

      if (incr > 0)
      {
        for (j = 0; j < points.length; j++)
        {
          if (points[j].getSide() == side)
          {
            coordIndex = sign * points[j].getIndex(dir);
            penalty = coordIndex - rightMostIndex;

            if (this.isOccupied(coordIndex))
            {
              if (!bestIsOccupied) {
                continue;
              }
            }
            else if (bestIsOccupied)
            {
              bestPenalty = penalty;
              bestPointIndex = j;
              bestIsOccupied = false;
            }

            if (penalty >= 0)
            {
              if (bestPenalty <= 0 || penalty < bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
            else
            {
              if (bestPenalty <= 0 && penalty > bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
          }
        }

        incr = incrOnPositive - incr;
      }
      else
      {
        for (j = 0; j < points.length; j++)
        {
          if (points[j].getSide() == side)
          {
            coordIndex = sign * points[j].getIndex(dir);
            penalty = leftMostIndex - coordIndex;

            if (this.isOccupied(coordIndex))
            {
              if (!bestIsOccupied) {
                continue;
              }
            }
            else if (bestIsOccupied)
            {
              bestPenalty = penalty;
              bestPointIndex = j;
              bestIsOccupied = false;
            }

            if (penalty >= 0)
            {
              if (bestPenalty <= 0 || penalty < bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
            else
            {
              if (bestPenalty <= 0 && penalty > bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
          }
        }

        incr = incrOnNegative - incr;
      }

      coordIndex = sign * points[bestPointIndex].getIndex(dir);
      this.setTerminationPoint(link, bestPointIndex);
      this.markOccupied(coordIndex);
      if (coordIndex > rightMostIndex) {
        rightMostIndex = coordIndex;
      }
      if (coordIndex < leftMostIndex) {
        leftMostIndex = coordIndex;
      }
    }
  },

  getLinkSide : function(link)
  {
    // summary:
    //		Determines for a single link at which border it wants to go.
    // tags:
    //		private
    
    var node = this.getNode();
    var startRect = link.getFromRect();
    var endRect = link.getToRect();
    var layout = this.getLayout();
    var minStartLen = layout.getMinStartSegmentLength();
    var minEndLen = layout.getMinEndSegmentLength();
    var pref = TranslateUtil.newArray(4, 0);
    var aux = TranslateUtil.newArray(4, 0);
    var terminationPoints;
    pref[0] = pref[1] = pref[2] = pref[3] = 0;

    if (link.getFromNode() == node)
    {
      terminationPoints = link.getStartPoints();

      if (link.getFromPoint() != null)
      {
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.x - minStartLen, startRect.x + startRect.width + minStartLen, endRect.x - minEndLen, endRect.x + endRect.width + minEndLen, 1, 3, 
                pref, aux);
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.y - minStartLen, startRect.y + startRect.height + minStartLen, endRect.y - minEndLen, endRect.y + endRect.height + minEndLen, 0, 2,
            pref, aux);
      }
    }
    else
    {
      terminationPoints = link.getEndPoints();

      if (link.getToPoint() != null)
      {
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.x - minStartLen, startRect.x + startRect.width + minStartLen, endRect.x - minEndLen, endRect.x + endRect.width + minEndLen, 1, 3, aux,
            pref);
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.y - minStartLen, startRect.y + startRect.height + minStartLen, endRect.y - minEndLen, endRect.y + endRect.height + minEndLen, 0, 2,
            aux, pref);
      }
    }

    if (terminationPoints.length > 1) {
      terminationPoints[0].sortAccordingToSide(terminationPoints, pref);
    }
    return (terminationPoints[0].getSide()); // int
  },

  calcRadialSortValue : function(link, correction)
  {
    // summary:
    //		Calculate the sort value as the angle of the ray starting at the
    //		reference node and going through the center of the other end node. 
    // description:
    //		Calculate the sort value as the angle of the ray starting at the
    //		reference node and going through the center of the other end node. 
    //		The input correction value allows use to rotate the coordinate
    //		system such that the 0-axis always is opposite to the current side.
    // tags:
    //		private
    
    var startX;
    var startY;
    var endX;
    var endY;
    var rect;

    if (link.getFromNode() == this.getNode())
    {
      rect = link.getFromRect();
      startX = rect.x + 0.5 * rect.width;
      startY = rect.y + 0.5 * rect.height;
      rect = link.getToRect();
      endX = rect.x + 0.5 * rect.width;
      endY = rect.y + 0.5 * rect.height;

      if (link.getFromPoint() != null)
      {
        startX = link.getFromPoint().x;
        startY = link.getFromPoint().y;
      }

      if (link.getToPoint() != null)
      {
        endX = link.getToPoint().x;
        endY = link.getToPoint().y;
      }
    }
    else
    {
      rect = link.getToRect();
      startX = rect.x + 0.5 * rect.width;
      startY = rect.y + 0.5 * rect.height;
      rect = link.getFromRect();
      endX = rect.x + 0.5 * rect.width;
      endY = rect.y + 0.5 * rect.height;

      if (link.getToPoint() != null)
      {
        startX = link.getToPoint().x;
        startY = link.getToPoint().y;
      }

      if (link.getFromPoint() != null)
      {
        endX = link.getFromPoint().x;
        endY = link.getFromPoint().y;
      }
    }

    var angle = LayoutUtil.angle(startX, startY, endX, endY);
    angle += correction;
    angle = LayoutUtil.correctAngle(angle);
    link.setSortValue(angle);
  },

  clearOccupied : function(numberOfMarkers)
  {
    // summary:
    //		Clear all occupied marker.
    // tags:
    //		private
    
    this._occupiedMarker = TranslateUtil.newArray(numberOfMarkers, 0);
    this._firstFreeOccupiedMarkerSlot = 0;
  },

  markOccupied : function(index)
  {
    // summary:
    //		Mark the index as occupied.
    // tags:
    //		private
    
    this._occupiedMarker[this._firstFreeOccupiedMarkerSlot++] = index;
  },

  isOccupied : function(index)
  {
    // summary:
    //		Returns true if the index is occupied.
    // tags:
    //		private
    
    for (var i = 0; i < this._firstFreeOccupiedMarkerSlot; i++)
      if (this._occupiedMarker[i] == index) {
        return (true); // Boolean
      }
    return (false); // Boolean
  }

}
);

LLDirectRouteAlgorithm.LinkSort = declare(QuickSort, 
{
  // summary:
  //		Inner auxiliary class to sort a vector of links according to the sort
  //		value.
  
  __outerThis: null,

  _links: null,

  constructor : function(input__outerThis, links)
  {
    this.__outerThis = input__outerThis;
    this._links = links;
  },
  
  dispose : function()
  {
    this._links = null;
  },

  sort : function()
  {
    // summary:
    // tags:
    //		private
    
    this.inherited("sort", arguments, [this._links.length]);
  },

  compare : function(loc1, loc2)
  {
    var link1 = this._links[loc1];
    var link2 = this._links[loc2];

    if (link1.getSortValue() == link2.getSortValue())
    {
      var link1Point;
      var link2Point;
      if (link1.getFromNode() == this.__outerThis.getNode()) {
        link1Point = link1.getActEndPoint();
      }
      else {
        link1Point = link1.getActStartPoint();
      }
      if (link2.getFromNode() == this.__outerThis.getNode()) {
        link2Point = link2.getActEndPoint();
      }
      else {
        link2Point = link2.getActStartPoint();
      }
      if (link1Point == null || link2Point == null) {
        return 0; // int
      }
      switch (link1Point.getSide())
      {
        case Direction.TOP:
          return (link2Point.getIndex(0) - link1Point.getIndex(0)); // int
        case Direction.BOTTOM:
          return (link1Point.getIndex(0) - link2Point.getIndex(0)); // int
        case Direction.LEFT:
          return (link1Point.getIndex(1) - link2Point.getIndex(1)); // int
        case Direction.RIGHT:
          return (link2Point.getIndex(1) - link1Point.getIndex(1)); // int
      }
      return 0; // int
    }

    if (link1.getSortValue() < link2.getSortValue()) {
      return -1; // int
    }
    else {
      return 1; // int
    }
  },

  swap : function(loc1, loc2)
  {
    var link1 = this._links[loc1];
    var link2 = this._links[loc2];
    this._links[loc2] =   link1;
    this._links[loc1] =   link2;
  }

}
);

return LLDirectRouteAlgorithm;
});
