/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../longlink/LLGridLine"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    LLGridLine	// #7
    ) {

var LLGrid = declare("ibm_ilog.graphlayout.longlink.LLGrid", null, 
{
  // summary:
  //		The routing grid for the link routing algorithm. 
  // description:
  //		The routing grid for the link routing algorithm. 
  //		<p>
  //		It stores the information which grid points are free to route a line
  //		and which are used. Some grid points may be free only in certain
  //		routing directions. During a routing, it also stores the backtrack
  //		distance per grid point for the routing. The backtrack distance is
  //		the distance a routing needs to be larger than the shortest direct
  //		routing. For instance to route from point (x0, y0) to point (x1, y1),
  //		the direct distance is D = abs(x1- x0) + abs(y1 - y0). The backtrack
  //		distance of a routing that has orthogonal length L is B = L - D. We
  //		use the grid to find a routing with the shortest backtrack distance.
  //		In order to manage the grid points effectively, we use the grid line
  //		data structure. This reduces the memory overhead compared to storing
  //		each single grid point, because often large areas of neighbored grid
  //		points behave absolutely identical.  
  //		</p>
  // tags:
  //		private
  
  _baseCoord: null,

  _gridDist: null,

  _minDist: null,

  _numberOfGridPoints: null,

  _gridLinesX: null,
  _gridLinesY: null,

  _dirtyLines: null,

  _dummyLine: null,

  _immediateReorganizationNeeded: null,

  constructor : function()
  {
  },
  
  init : function(baseCoordX, baseCoordY, gridDistX, gridDistY, minDistX, minDistY, gridSizeX, gridSizeY)
  {
    // summary:
    //		Initializes the grid. 
    // description:
    //		Initializes the grid. 
    //		This must be called after construction, before the grid can be used.
    //		It is also possible to reinitialize the grid this way without
    //		allocating a new one.
    // tags:
    //		private
    
    this._baseCoord = TranslateUtil.newArray(2, 0);
    this._baseCoord[0] = baseCoordX;
    this._baseCoord[1] = baseCoordY;
    this._gridDist = TranslateUtil.newArray(2, 0);
    this._gridDist[0] = (gridDistX > 1.0 ? gridDistX : 1.0);
    this._gridDist[1] = (gridDistY > 1.0 ? gridDistY : 1.0);
    this._minDist = TranslateUtil.newArray(2, 0);
    this._minDist[0] = (minDistX > 1.0 ? minDistX : 1.0);
    this._minDist[1] = (minDistY > 1.0 ? minDistY : 1.0);
    this._numberOfGridPoints = TranslateUtil.newArray(2, 0);
    this._numberOfGridPoints[0] = gridSizeX;
    this._numberOfGridPoints[1] = gridSizeY;
    this._gridLinesX = TranslateUtil.newArray(gridSizeX, null);
    this._gridLinesY = TranslateUtil.newArray(gridSizeY, null);
    var i;

    for (i = 0; i < gridSizeX; i++)
      this._gridLinesX[i] = new LLGridLine(gridSizeY - 1, i, false);

    for (i = 0; i < gridSizeY; i++)
      this._gridLinesY[i] = new LLGridLine(gridSizeX - 1, i, true);
    this._dummyLine = new LLGridLine(1, -1, false);
    this._dirtyLines = this._dummyLine;
    this._immediateReorganizationNeeded = false;
  },

  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._baseCoord = null;
    this._gridDist = null;
    this._minDist = null;
    var i;

    for (i = 0; i < this._numberOfGridPoints[0]; i++)
      this._gridLinesX[i].dispose();

    for (i = 0; i < this._numberOfGridPoints[1]; i++)
      this._gridLinesY[i].dispose();
    this._numberOfGridPoints = null;
    this._dummyLine.dispose();

    this._dummyLine = null;
    this._dirtyLines = null;
  },

  isImmediateReorganizationNeeded : function()
  {
    // summary:
    //		Returns true if the grid needs immediate reorganization.
    // tags:
    //		private
    
    return (this._immediateReorganizationNeeded); // Boolean
  },

  setImmediateReorganizationNeeded : function(flag)
  {
    // summary:
    //		Sets whether the grid needs immediate reorganization.
    // tags:
    //		private
    
    this._immediateReorganizationNeeded = flag;
  },

  getMinCoord : function(dir)
  {
    // summary:
    //		Returns the smaller coordinate of the grid. 
    // description:
    //		Returns the smaller coordinate of the grid. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._baseCoord[dir]); // Number
  },

  getMaxCoord : function(dir)
  {
    // summary:
    //		Returns the larger coordinate of the rectangle. 
    // description:
    //		Returns the larger coordinate of the rectangle. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._baseCoord[dir] + this._numberOfGridPoints[dir] * this._gridDist[dir]); // Number
  },

  getGridCoord : function(dir, i)
  {
    // summary:
    //		Returns the coordinate of the ith grid line. 
    // description:
    //		Returns the coordinate of the ith grid line. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._baseCoord[dir] + i * this._gridDist[dir]); // Number
  },

  getGridIndex : function(dir, coord, larger)
  {
    // summary:
    //		Returns the grid index corresponding to a coordinate. 
    // description:
    //		Returns the grid index corresponding to a coordinate. 
    //		Direction 0 means x, and 1 means y. If larger is true, it returns an
    //		index with larger or equal coordinate, otherwise an index with
    //		smaller or equal coordinate.
    // tags:
    //		private
    
    var resf = (coord - this.getMinCoord(dir)) / this._gridDist[dir];
    if (larger) {
      return (Math.floor(Math.ceil(resf))); // int
    }
    return (Math.floor(Math.floor(resf))); // int
  },

  getClosestGridIndex : function(dir, coord)
  {
    // summary:
    //		Returns the closest grid index corresponding to a coordinate. 
    // description:
    //		Returns the closest grid index corresponding to a coordinate. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    var index1 = this.getGridIndex(dir, coord, false);
    var index2 = this.getGridIndex(dir, coord, true);
    var d1 = coord - this.getGridCoord(dir, index1);
    var d2 = this.getGridCoord(dir, index2) - coord;
    if (d1 < d2) {
      return (index1); // int
    }
    return (index2); // int
  },

  getGridIndexAtMinDist : function(dir, coord, larger)
  {
    // summary:
    //		Returns the grid index corresponding to a coordinate, with at least
    //		minDist larger or smaller. 
    // description:
    //		Returns the grid index corresponding to a coordinate, with at least
    //		minDist larger or smaller. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    var resf = (coord - this.getMinCoord(dir)) / this._gridDist[dir];
    var res;

    if (larger)
    {
      res = Math.floor(Math.floor(resf));

      while (this.getGridCoord(dir, res) - coord < this._minDist[dir])
        res++;
    }
    else
    {
      res = Math.floor(Math.ceil(resf));

      while (coord - this.getGridCoord(dir, res) < this._minDist[dir])
        res--;
    }

    return (res); // int
  },

  getNumberOfGridLines : function(dir)
  {
    // summary:
    //		Returns the number of grid lines. 
    // description:
    //		Returns the number of grid lines. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._numberOfGridPoints[dir]); // int
  },

  getGridDist : function(dir)
  {
    // summary:
    //		Returns the distance between gridlines. 
    // description:
    //		Returns the distance between gridlines. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._gridDist[dir]); // Number
  },

  getGridLine : function(dir, i)
  {
    // summary:
    //		Returns the ith grid line in direction dir.
    // tags:
    //		private
    
    if (dir == 0) {
      return this._gridLinesX[i]; // ibm_ilog.graphlayout.longlink.LLGridLine
    }
    return this._gridLinesY[i]; // ibm_ilog.graphlayout.longlink.LLGridLine
  },

  searchClosestSegmentAlive : function(index0, index1, dir, larger)
  {
    // summary:
    //		Returns the closest segment that away from the start point given by
    //		index0 and index1. 
    // description:
    //		Returns the closest segment that away from the start point given by
    //		index0 and index1. 
    //		The direction dir is the direction of index0. If dir is 0, it
    //		searches in horizontal direction, if it is 1, it searches in
    //		vertical direction. If larger is true, it searches a larger segment.
    //		The returned segment can either be horizontally or vertically
    //		oriented. Preferably it is oriented same to the search direction.
    // tags:
    //		private
    
    if (index0 < 0 || index0 >= this._numberOfGridPoints[dir]) {
      return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    if (index1 < 0 || index1 >= this._numberOfGridPoints[1 - dir]) {
      return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }

    var seg1 = this.getGridLine(1 - dir, index1).searchClosestSegmentAlive(this, index0, larger);
    var seg2;
    var startIndex;
    var endIndex;
    var maxIndex = this._numberOfGridPoints[dir];

    if (larger)
    {
      startIndex = (index0 > 0 ? index0 : 0);
      endIndex = maxIndex;
    }
    else
    {
      startIndex = (index0 < maxIndex ? index0 : maxIndex - 1);
      endIndex = -1;
    }

    if (seg1 != null)
    {
      if (index0 < seg1.getStartIndex()) {
        endIndex = seg1.getStartIndex();
      }
      else if (index0 > seg1.getEndIndex()) {
        endIndex = seg1.getEndIndex();
      }
      else {
        return (seg1); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
    }

    var i;

    if (larger)
    {
      for (i = startIndex; i < endIndex; i++)
      {
        seg2 = this.getGridLine(dir, i).searchSegment(index1);
        if (seg2 != null && !seg2.isDeadlock(this, 0, /*Number.MAX_VALUE*/2147483647)) {
          return (seg2); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
      }
    }
    else
    {
      for (i = startIndex; i > endIndex; i--)
      {
        seg2 = this.getGridLine(dir, i).searchSegment(index1);
        if (seg2 != null && !seg2.isDeadlock(this, 0, /*Number.MAX_VALUE*/2147483647)) {
          return (seg2); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
      }
    }

    return (seg1); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  getMinGridIndexAtSide : function(rect, dir, minCornerOffset)
  {
    // summary:
    //		Returns the smallest grid index at the given side of the rectangle.
    // tags:
    //		private
    
    var coord = LLGrid.getMinCoord(rect, dir) + minCornerOffset;
    var size = LLGrid.getSize(rect, dir);

    if (size < 2 * minCornerOffset)
    {
      var a = this.getGridIndex(dir, coord - minCornerOffset, true);
      var b = this.getGridIndex(dir, coord - minCornerOffset + size, false);
      if (a < b) {
        return (Math.floor((a + b) / 2)); // int
      }
      return (a); // int
    }

    return (this.getGridIndex(dir, coord, true)); // int
  },

  getMaxGridIndexAtSide : function(rect, dir, minCornerOffset)
  {
    // summary:
    //		Returns the largest grid index at the given side of the rectangle.
    // tags:
    //		private
    
    var coord = LLGrid.getMaxCoord(rect, dir) - minCornerOffset;
    var size = LLGrid.getSize(rect, dir);

    if (size < 2 * minCornerOffset)
    {
      var a = this.getGridIndex(dir, coord + minCornerOffset - size, true);
      var b = this.getGridIndex(dir, coord + minCornerOffset, false);
      if (a < b) {
        return (Math.floor((a + b) / 2)); // int
      }
      return (b); // int
    }

    return (this.getGridIndex(dir, coord, false)); // int
  },

  markDirty : function(gridLine)
  {
    // summary:
    //		Marks a grid line as dirty.
    // tags:
    //		private
    
    if (!gridLine.isDirty())
    {
      gridLine.setNextDirty(this._dirtyLines);
      this._dirtyLines = gridLine;
    }
  },

  reorganizeAllBeforeSearch : function()
  {
    // summary:
    //		Reorganize the complete grid after setting all obstacles, before
    //		starting a search.
    // tags:
    //		private
    
    var maxNrGPX = this._numberOfGridPoints[0];
    var maxNrGPY = this._numberOfGridPoints[1];
    var i;

    for (i = 0; i < maxNrGPX; i++)
      this._gridLinesX[i].reorganizeBeforeSearch(false);

    for (i = 0; i < maxNrGPY; i++)
      this._gridLinesY[i].reorganizeBeforeSearch(false);
    this._dirtyLines = this._dummyLine;
  },

  reorganizeDirtyBeforeSearch : function()
  {
    // summary:
    //		Reorganize the dirty grid lines before the next routing. 
    // description:
    //		Reorganize the dirty grid lines before the next routing. 
    //		This can be faster if the previous route
    //		didn't cover the complete grid. It also clears the grid.
    // tags:
    //		private
    
    var gridLine = this._dirtyLines;
    var nextGridLine;

    while (gridLine != this._dummyLine)
    {
      nextGridLine = gridLine.getNextDirty();
      gridLine.reorganizeBeforeSearch(false);
      gridLine = nextGridLine;
    }

    this._dirtyLines = this._dummyLine;
  },

  clearDirty : function()
  {
    // summary:
    //		Clears the dirty grid lines before the next routing. 
    // description:
    //		Clears the dirty grid lines before the next routing. 
    //		This can be faster if the previous route
    //		didn't cover the complete grid. It can be called if reorganization is not necessary because no new obstacles were set on the grid, but a previous route kept info data on the grid. This info data is removed.
    // tags:
    //		private
    
    var gridLine = this._dirtyLines;
    var nextGridLine;

    while (gridLine != this._dummyLine)
    {
      nextGridLine = gridLine.getNextDirty();
      gridLine.clear();
      gridLine = nextGridLine;
    }

    this._dirtyLines = this._dummyLine;
  },

  isWithoutObstacles : function(dir, index, limits)
  {
    // summary:
    //		Returns true if the area from limits[0] to limits[1] is
    //		obstacle-free on grid link with index. 
    // description:
    //		Returns true if the area from limits[0] to limits[1] is
    //		obstacle-free on grid link with index. 
    //		If dir is 0, the free segment should be horizontal, otherwise
    //		vertical.
    // tags:
    //		private
    
    var line = this.getGridLine(1 - dir, index);
    if (line == null) {
      return (false); // Boolean
    }
    var segment = line.searchSegment(limits[0]);
    if (segment == null) {
      return (false); // Boolean
    }
    if (segment.getStartIndex() > limits[1]) {
      return (false); // Boolean
    }
    if (segment.getEndIndex() < limits[1]) {
      return (false); // Boolean
    }
    return (true); // Boolean
  },

  isWithinRectObstacle : function(xindex, yindex, rect)
  {
    // summary:
    //		Returns true if the input grid index coordinate is within the input
    //		rectangle obstacle.
    // tags:
    //		private
    
    var gridLineX0 = this.getGridIndexAtMinDist(0, rect.x, false);
    var gridLineX1 = this.getGridIndexAtMinDist(0, rect.x + rect.width, true);
    var gridLineY0 = this.getGridIndexAtMinDist(1, rect.y, false);
    var gridLineY1 = this.getGridIndexAtMinDist(1, rect.y + rect.height, true);

    gridLineX0++;
    gridLineX1--;
    gridLineY0++;
    gridLineY1--;
    return (gridLineX0 <= xindex && xindex <= gridLineX1 && gridLineY0 <= yindex && yindex <= gridLineY1); // Boolean
  },

  removeObstacle : function(dir, index, limits, link, i)
  {
    // summary:
    //		Remove an obstacle that is currently coverd by the segment i of the
    //		input link. 
    // description:
    //		Remove an obstacle that is currently coverd by the segment i of the
    //		input link. 
    //		If dir is 0, the segment should be horizontal, otherwise vertical.
    //		The obstacle is a line segment at grid line with index, and the area
    //		of that grid line from limits[0] to limits[1] must be given free.
    // tags:
    //		private
    
    if (link.getSegmentDir(i) == dir)
    {
      var line = this.getGridLine(1 - dir, index);
      var seg;
      var notDone = true;

      if (line != null)
      {
        var min = Math.min(limits[0], limits[1]);
        var max = Math.max(limits[0], limits[1]);

        if (link.getSegments()[i] >= 0)
        {
          if (!link.startsAtObstacle(i))
          {
            seg = line.searchSegment(min - 1);

            if (seg != null)
            {
              seg.setEndIndex(max);
              notDone = false;
            }
          }

          if (notDone && !link.endsAtObstacle(i))
          {
            seg = line.searchSegment(max + 1);

            if (seg != null)
            {
              seg.setStartIndex(min);
              notDone = false;
            }
          }

          if (notDone) {
            line.addSegment(min, max);
          }
        }
        else
        {
          if (!link.endsAtObstacle(i))
          {
            seg = line.searchSegment(min - 1);

            if (seg != null)
            {
              seg.setEndIndex(max);
              notDone = false;
            }
          }

          if (notDone && !link.startsAtObstacle(i))
          {
            seg = line.searchSegment(max + 1);

            if (seg != null)
            {
              seg.setStartIndex(min);
              notDone = false;
            }
          }

          if (notDone) {
            line.addSegment(min, max);
          }
        }

        this.markDirty(line);
      }
    }
  },

  setRectObstacle : function(rect)
  {
    // summary:
    //		Store in the grid the information about a rectangular obstacle.  
    // tags:
    //		private
    
    var gridLineX0 = this.getGridIndexAtMinDist(0, rect.x, false);
    var gridLineX1 = this.getGridIndexAtMinDist(0, rect.x + rect.width, true);
    if (gridLineX0 >= gridLineX1) {
      return;
    }
    var gridLineY0 = this.getGridIndexAtMinDist(1, rect.y, false);
    var gridLineY1 = this.getGridIndexAtMinDist(1, rect.y + rect.height, true);
    if (gridLineY0 >= gridLineY1) {
      return;
    }
    gridLineX0++;
    gridLineX1--;
    gridLineY0++;
    gridLineY1--;
    if (gridLineX0 < 0) {
      gridLineX0 = 0;
    }
    if (gridLineY0 < 0) {
      gridLineY0 = 0;
    }
    if (gridLineX1 >= this._numberOfGridPoints[0]) {
      gridLineX1 = this._numberOfGridPoints[0] - 1;
    }
    if (gridLineY1 >= this._numberOfGridPoints[1]) {
      gridLineY1 = this._numberOfGridPoints[1] - 1;
    }
    this.storeRectObstacle(gridLineX0, gridLineX1, gridLineY0, gridLineY1);
  },

  setPointObstacle : function(point)
  {
    // summary:
    //		Store in the grid the information about a point obstacle.  
    // tags:
    //		private
    
    var gridLineX0;
    var gridLineX1;
    var gridLineY0;
    var gridLineY1;

    var coord = point.x;
    gridLineX0 = Math.floor(Math.ceil((coord - this.getMinCoord(0)) / this._gridDist[0]));

    while (coord - this.getGridCoord(0, gridLineX0) < this._minDist[0])
      gridLineX0--;
    gridLineX1 = gridLineX0;

    while (this.getGridCoord(0, gridLineX1) - coord < this._minDist[0])
      gridLineX1++;
    coord = point.y;
    gridLineY0 = Math.floor(Math.ceil((coord - this.getMinCoord(1)) / this._gridDist[1]));

    while (coord - this.getGridCoord(1, gridLineY0) < this._minDist[1])
      gridLineY0--;
    gridLineY1 = gridLineY0;

    while (this.getGridCoord(1, gridLineY1) - coord < this._minDist[1])
      gridLineY1++;

    gridLineX0++;
    gridLineX1--;
    gridLineY0++;
    gridLineY1--;
    if (gridLineX0 < 0) {
      gridLineX0 = 0;
    }
    if (gridLineY0 < 0) {
      gridLineY0 = 0;
    }
    if (gridLineX1 >= this._numberOfGridPoints[0]) {
      gridLineX1 = this._numberOfGridPoints[0] - 1;
    }
    if (gridLineY1 >= this._numberOfGridPoints[1]) {
      gridLineY1 = this._numberOfGridPoints[1] - 1;
    }
    this.storeRectObstacle(gridLineX0, gridLineX1, gridLineY0, gridLineY1);
  },

  storeRectObstacle : function(gridLineX0, gridLineX1, gridLineY0, gridLineY1)
  {
    // summary:
    //		Store a rectangular obstacle that reached from gridLineX0 to
    //		gridLineX1, and from gridLineY0 to gridLineY1. 
    // description:
    //		Store a rectangular obstacle that reached from gridLineX0 to
    //		gridLineX1, and from gridLineY0 to gridLineY1. 
    //		This is used for rectangles and points.
    // tags:
    //		private
    
    var i;

    if (gridLineY1 >= gridLineY0 - 1) {
      for (i = gridLineX0; i <= gridLineX1; i++)
        this._gridLinesX[i].setObstacle(this, gridLineY0, gridLineY1);
    }
    if (gridLineX1 >= gridLineX0 - 1) {
      for (i = gridLineY0; i <= gridLineY1; i++)
        this._gridLinesY[i].setObstacle(this, gridLineX0, gridLineX1);
    }
  },

  setObstacle : function(p1, p2)
  {
    // summary:
    //		Store in the grid the information about a line obstacle.
    // tags:
    //		private
    
    this.setPointObstacle(p1);
    if (p1.x == p2.x && p1.y == p2.y) {
      return;
    }
    this.setPointObstacle(p2);

    var isHorizontal = false;
    var isVertical = false;
    var dx = (p2.x - p1.x);
    var dy = (p2.y - p1.y);
    var adx = (dx > 0.0 ? dx : -dx);
    var ady = (dy > 0.0 ? dy : -dy);

    if (ady == 0.0) {
      isHorizontal = true;
    }
    else if (adx == 0.0) {
      isVertical = true;
    }
    else if (adx > ady)
    {
      var slopea = dy / dx;
      if (-0.2 < slopea && slopea < 0.2) {
        isHorizontal = true;
      }
    }
    else
    {
      var slopeb = dx / dy;
      if (-0.2 < slopeb && slopeb < 0.2) {
        isVertical = true;
      }
    }

    if (!isHorizontal && !isVertical) {
      return;
    }

    var gridLineX0;
    var gridLineX1;
    var gridLineY0;
    var gridLineY1;
    var coord = (dx > 0.0 ? p1.x : p2.x);
    gridLineX0 = Math.floor(Math.floor((coord - this.getMinCoord(0)) / this._gridDist[0]));

    while (coord - this.getGridCoord(0, gridLineX0) >= this._minDist[0])
      gridLineX0++;
    coord = (dx > 0.0 ? p2.x : p1.x);
    gridLineX1 = Math.floor(Math.ceil((coord - this.getMinCoord(0)) / this._gridDist[0]));

    while (this.getGridCoord(0, gridLineX1) - coord >= this._minDist[0])
      gridLineX1--;
    coord = (dy > 0.0 ? p1.y : p2.y);
    gridLineY0 = Math.floor(Math.floor((coord - this.getMinCoord(1)) / this._gridDist[1]));

    while (coord - this.getGridCoord(1, gridLineY0) >= this._minDist[1])
      gridLineY0++;
    coord = (dy > 0.0 ? p2.y : p1.y);
    gridLineY1 = Math.floor(Math.ceil((coord - this.getMinCoord(1)) / this._gridDist[1]));

    while (this.getGridCoord(1, gridLineY1) - coord >= this._minDist[1])
      gridLineY1--;
    if (gridLineX0 < 0) {
      gridLineX0 = 0;
    }
    if (gridLineY0 < 0) {
      gridLineY0 = 0;
    }
    if (gridLineX1 >= this._numberOfGridPoints[0]) {
      gridLineX1 = this._numberOfGridPoints[0] - 1;
    }
    if (gridLineY1 >= this._numberOfGridPoints[1]) {
      gridLineY1 = this._numberOfGridPoints[1] - 1;
    }

    var i;
    var j;

    for (i = gridLineX0; i <= gridLineX1; i++)
      for (j = gridLineY0; j <= gridLineY1; j++)
        if (this.checkGridPointForLine(i, j, p1, p2)) {
          if (isHorizontal) {
            this._gridLinesY[j].setObstacle(this, i, i);
          }
          else {
            this._gridLinesX[i].setObstacle(this, j, j);
          }
        }
  },

  setGridLineObstacle : function(isHorizontal, index, limits)
  {
    // summary:
    //		Store in the grid the information about a horizontal or vertical
    //		line obstacle that is given in grid index coordinates (rather than
    //		in real coordinates). 
    // description:
    //		Store in the grid the information about a horizontal or vertical
    //		line obstacle that is given in grid index coordinates (rather than
    //		in real coordinates). 
    //		The obstacle is on grid line given by index, and reaches from
    //		limits[0] to limits[1].
    // tags:
    //		private
    
    if (isHorizontal) {
      this._gridLinesY[index].setObstacle(this, limits[0], limits[1]);
    }
    else {
      this._gridLinesX[index].setObstacle(this, limits[0], limits[1]);
    }
  },

  checkGridPointForLine : function(i, j, p1, p2)
  {
    // summary:
    //		Check whether the grid point (i, j) is closer to line p1, p2 than
    //		minDist.
    // tags:
    //		private
    
    var x = this.getGridCoord(0, i);
    var y = this.getGridCoord(1, j);
    var minx;
    var maxx;
    var miny;
    var maxy;

    if (p1.x < p2.x)
    {
      minx = p1.x;
      maxx = p2.x;
    }
    else
    {
      minx = p2.x;
      maxx = p1.x;
    }

    if (p1.y < p2.y)
    {
      miny = p1.y;
      maxy = p2.y;
    }
    else
    {
      miny = p2.y;
      maxy = p1.y;
    }

    var dx = (p2.x - p1.x);
    var dy = (p2.y - p1.y);

    var fx;
    var fy;

    if (dx == 0.0)
    {
      fx = /*Number.MAX_VALUE*/(3.4028235E38);
      fy = p1.x;
    }
    else if (dy == 0.0)
    {
      fx = p1.y;
      fy = /*Number.MAX_VALUE*/(3.4028235E38);
    }
    else
    {
      fx = p1.y + (x - p1.x) * dy / dx;
      fy = p1.x + (y - p1.y) * dx / dy;
    }

    var d;

    if (minx <= fy && fy <= maxx)
    {
      d = (fy - x);
      if (d < 0.0) {
        d = -d;
      }
      if (d < this._minDist[0]) {
        return (true); // Boolean
      }
    }

    if (miny <= fx && fx <= maxy)
    {
      d = (fx - y);
      if (d < 0.0) {
        d = -d;
      }
      if (d < this._minDist[1]) {
        return (true); // Boolean
      }
    }

    return (false); // Boolean
  },

  ensureBorder : function(n)
  {
    // summary:
    //		Ensure that the grid has at least "n" empty grid lines at each
    //		border. 
    // description:
    //		Ensure that the grid has at least "n" empty grid lines at each
    //		border. 
    //		It returns true if something changed.
    // tags:
    //		private
    
    var maxNrGPX = this._numberOfGridPoints[0];
    var maxNrGPY = this._numberOfGridPoints[1];
    if (maxNrGPX == 0) {
      return (false); // Boolean
    }
    if (maxNrGPY == 0) {
      return (false); // Boolean
    }
    var newXGridLinesBefore = 0;
    var newXGridLinesAfter = 0;
    var newYGridLinesBefore = 0;
    var newYGridLinesAfter = 0;
    var i;

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPX || !this._gridLinesX[i].isFree(maxNrGPY - 1))
      {
        newXGridLinesBefore = n - i;
        break;
      }
    }

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPX || !this._gridLinesX[maxNrGPX - 1 - i].isFree(maxNrGPY - 1))
      {
        newXGridLinesAfter = n - i;
        break;
      }
    }

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPY || !this._gridLinesY[i].isFree(maxNrGPX - 1))
      {
        newYGridLinesBefore = n - i;
        break;
      }
    }

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPY || !this._gridLinesY[maxNrGPY - 1 - i].isFree(maxNrGPX - 1))
      {
        newYGridLinesAfter = n - i;
        break;
      }
    }

    var newMaxNrGPX = maxNrGPX + newXGridLinesBefore + newXGridLinesAfter;
    var newMaxNrGPY = maxNrGPY + newYGridLinesBefore + newYGridLinesAfter;
    if (newMaxNrGPX == maxNrGPX && newMaxNrGPY == maxNrGPY) {
      return (false); // Boolean
    }
    var table;
    var seg;

    for (i = 0; i < maxNrGPX; i++)
      this._gridLinesX[i].shiftIndex(newXGridLinesBefore, newYGridLinesBefore);

    for (i = 0; i < maxNrGPY; i++)
      this._gridLinesY[i].shiftIndex(newYGridLinesBefore, newXGridLinesBefore);

    if (newXGridLinesBefore + newXGridLinesAfter != 0)
    {
      table = TranslateUtil.newArray(newMaxNrGPX, null);

      for (i = 0; i < newXGridLinesBefore; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPY - 1, i, false);
        this.markDirty(table[i]);
      }

      for (i = 0; i < maxNrGPX; i++)
        table[i + newXGridLinesBefore] = this._gridLinesX[i];

      for (i = maxNrGPX + newXGridLinesBefore; i < newMaxNrGPX; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPY - 1, i, false);
        this.markDirty(table[i]);
      }

      this._gridLinesX = table;
      this._numberOfGridPoints[0] = newMaxNrGPX;
      this._baseCoord[0] -= newXGridLinesBefore * this._gridDist[0];
    }

    if (newYGridLinesBefore + newYGridLinesAfter != 0)
    {
      table = TranslateUtil.newArray(newMaxNrGPY, null);

      for (i = 0; i < newYGridLinesBefore; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPX - 1, i, true);
        this.markDirty(table[i]);
      }

      for (i = 0; i < maxNrGPY; i++)
        table[i + newYGridLinesBefore] = this._gridLinesY[i];

      for (i = maxNrGPY + newYGridLinesBefore; i < newMaxNrGPY; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPX - 1, i, true);
        this.markDirty(table[i]);
      }

      this._gridLinesY = table;
      this._numberOfGridPoints[1] = newMaxNrGPY;
      this._baseCoord[1] -= newYGridLinesBefore * this._gridDist[1];
    }

    if (newXGridLinesBefore != 0)
    {
      for (i = 0; i < newMaxNrGPY; i++)
      {
        seg = this._gridLinesY[i].searchSegment(newXGridLinesBefore);
        if (seg == null) {
          this._gridLinesY[i].addSegment(0, newXGridLinesBefore - 1);
        }
        else {
          seg.setStartIndex(0);
        }
      }
    }

    if (newXGridLinesAfter != 0)
    {
      for (i = 0; i < newMaxNrGPY; i++)
      {
        seg = this._gridLinesY[i].searchSegment(newMaxNrGPX - newXGridLinesAfter - 1);
        if (seg == null) {
          this._gridLinesY[i].addSegment(newMaxNrGPX - newXGridLinesAfter, newMaxNrGPX - 1);
        }
        else {
          seg.setEndIndex(newMaxNrGPX - 1);
        }
      }
    }

    if (newYGridLinesBefore != 0)
    {
      for (i = 0; i < newMaxNrGPX; i++)
      {
        seg = this._gridLinesX[i].searchSegment(newYGridLinesBefore);
        if (seg == null) {
          this._gridLinesX[i].addSegment(0, newYGridLinesBefore - 1);
        }
        else {
          seg.setStartIndex(0);
        }
      }
    }

    if (newYGridLinesAfter != 0)
    {
      for (i = 0; i < newMaxNrGPX; i++)
      {
        seg = this._gridLinesX[i].searchSegment(newMaxNrGPY - newYGridLinesAfter - 1);
        if (seg == null) {
          this._gridLinesX[i].addSegment(newMaxNrGPY - newYGridLinesAfter, newMaxNrGPY - 1);
        }
        else {
          seg.setEndIndex(newMaxNrGPY - 1);
        }
      }
    }

    return (true); // Boolean
  }

}
);

lang.mixin(LLGrid,
{

  getMinCoord : function(rect, dir)
  {
    return (dir == 0 ? rect.x : rect.y); // Number
  },

  getMaxCoord : function(rect, dir)
  {
    return (dir == 0 ? rect.x + rect.width : rect.y + rect.height); // Number
  },

  getSize : function(rect, dir)
  {
    return (dir == 0 ? rect.width : rect.height); // Number
  }

}
);

return LLGrid;
});
