/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../longlink/LLGridSegment"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    LLGridSegment	// #7
    ) {

var LLGridLine = declare("ibm_ilog.graphlayout.longlink.LLGridLine", null, 
{
  // summary:
  //		A grid line is owned by the routing grid (LLGrid) and consists of a
  //		list of grid segments. 
  // description:
  //		A grid line is owned by the routing grid (LLGrid) and consists of a
  //		list of grid segments. 
  //		<p>
  //		The grid line can be horizontal or vertical. The crossing point
  //		between a horizontal and a vertical grid line is a grid point, but we
  //		don't have any special data structure for each single grid point in the LLGrid data structure.   
  //		</p>
  //		<p>
  //		Routing is done along grid lines. Therefore, the grid line is split into grid segments that are usable and grid segments that are not usable i.e., invalid, because covered by another object. In fact, we remove the invalid segments before routing (they only occur during the creation of the grid lines) in order to make the algorithm fast. So, during routing, the grid lines contain only segments that are usable.  
  //		</p>
  //		<p>
  //		If a grid line has many segments, we organize the segments as binary tree in order to find them faster. Otherwise we organze the segments as linear list.  
  //		</p>
  //		<p>
  //		For details, see class LLGrid.  
  //		</p>
  // tags:
  //		private
  
  _segments: null,

  _index: null,

  _isHorizontal: null,

  _isBinTree: null,

  _nextDirty: null,

  constructor : function(maxGridPointIndex, index, isHorizontal)
  {
    this._index = index;
    this._isHorizontal = isHorizontal;
    this._isBinTree = false;

    this._segments = new LLGridSegment(this, 0, maxGridPointIndex, null);
    this._nextDirty = null;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.convertToList();
    var seg = this._segments;
    var aux;

    while (seg != null)
    {
      aux = seg.getNext1();
      seg.dispose();
      seg = aux;
    }

    this._segments = null;

    this._nextDirty = null;
  },

  isHorizontal : function()
  {
    // summary:
    //		Returns true if the grid line is horizontal.
    // tags:
    //		private
    
    return (this._isHorizontal); // Boolean
  },

  getIndex : function()
  {
    // summary:
    //		Returns the index of the grid line.
    // tags:
    //		private
    
    return (this._index); // int
  },

  isDirty : function()
  {
    // summary:
    //		Returns true if the grid line is dirty, i.e. 
    // description:
    //		Returns true if the grid line is dirty, i.e. 
    //		needs to be reorganized before we start the next routing. Note that
    //		the dirty list is so organized such that all real dirty grid lines
    //		have a next line, i.e. the last real grid line points to a dummy
    //		next grid line. Therefore it is enough to check the next pointer.
    // tags:
    //		private
    
    return (this._nextDirty != null); // Boolean
  },

  getNextDirty : function()
  {
    // summary:
    //		Returns the next dirty grid line.
    // tags:
    //		private
    
    return (this._nextDirty); // ibm_ilog.graphlayout.longlink.LLGridLine
  },

  setNextDirty : function(next)
  {
    // summary:
    //		Sets the next dirty grid line.
    // tags:
    //		private
    
    this._nextDirty = next;
  },

  clear : function()
  {
    // summary:
    //		Clear all segments before routing.
    // tags:
    //		private
    
    if (this._segments != null) {
      this._segments.clearRecursive();
    }
    this._nextDirty = null;
  },

  reorganizeBeforeSearch : function(keepDirty)
  {
    // summary:
    //		Reorganize the grid line such that the grid segments for a binary
    //		tree. 
    // description:
    //		Reorganize the grid line such that the grid segments for a binary
    //		tree. 
    //		This also clears all segments before the next routing. Afterwards,
    //		the grid line is not anymore dirty.
    // tags:
    //		private
    
    this.convertToList();

    var seg = this._segments;
    var aux;

    while (seg != null && seg.isInvalid())
    {
      aux = seg.getNext1();
      seg.dispose();
      seg = aux;
    }

    this._segments = seg;
    var numSegments = 0;

    while (seg != null)
    {
      seg.clear();
      seg.removeInvalidNext();
      numSegments++;
      seg = seg.getNext1();
    }

    if (numSegments >= 8) {
      this.convertToBinTree(numSegments);
    }
    if (!keepDirty) {
      this._nextDirty = null;
    }
  },

  convertToList : function()
  {
    // summary:
    //		Convert the binary tree of the grid line into a linear list.
    // tags:
    //		private
    
    if (this._isBinTree)
    {
      var nextseg = this._segments;
      this._segments = null;
      this.listConversionTraversal(nextseg);
      this._isBinTree = false;
    }
  },

  convertToBinTree : function(numSegments)
  {
    // summary:
    //		Convert the segment list of the grid line into a binary tree.
    // tags:
    //		private
    
    if (numSegments > 0 && !this._isBinTree)
    {
      var segTab = TranslateUtil.newArray(numSegments, null);
      var seg = this._segments;
      var i = 0;

      while (seg != null)
      {
        segTab[i++] = seg;
        seg = seg.getNext1();
      }

      this._segments = this.bintreeConversionTraversal(segTab, 0, numSegments - 1);
      this._isBinTree = true;
    }
  },

  listConversionTraversal : function(segment)
  {
    // summary:
    //		Traverse the binary tree to convert it to a linear list. 
    // description:
    //		Traverse the binary tree to convert it to a linear list. 
    //		It always prepends in in-order to the &#95;segments.
    // tags:
    //		private
    
    if (segment == null) {
      return;
    }
    this.listConversionTraversal(segment.getNext1());
    var nextseg = this._segments;
    this._segments = segment;
    this.listConversionTraversal(segment.getNext0());
    segment.setNexts(null, nextseg);
  },

  bintreeConversionTraversal : function(segTab, startIndex, endIndex)
  {
    // summary:
    //		Traverse the list to convert it to a binary tree. 
    // description:
    //		Traverse the list to convert it to a binary tree. 
    //		It returns the root of the tree.
    // tags:
    //		private
    
    if (endIndex < startIndex) {
      return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    var middle = Math.floor((startIndex + endIndex) / 2);
    var seg = segTab[middle];
    seg.setNexts(this.bintreeConversionTraversal(segTab, startIndex, middle - 1), this.bintreeConversionTraversal(segTab, middle + 1, endIndex));
    return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  shiftIndex : function(ownDelta, segmentDelta)
  {
    // summary:
    //		Shift indexe by delta.
    // tags:
    //		private
    
    this._index += ownDelta;
    if (segmentDelta != 0 && this._segments != null) {
      this._segments.shiftIndex(segmentDelta);
    }
  },

  isFree : function(maxGridPoint)
  {
    // summary:
    //		Returns true if the grid line is completely free, i.e. 
    // description:
    //		Returns true if the grid line is completely free, i.e. 
    //		without obstacles.
    // tags:
    //		private
    
    if (this._segments == null) {
      return (false); // Boolean
    }
    if (this._segments.getStartIndex() > 0) {
      return (false); // Boolean
    }
    if (this._segments.getEndIndex() < maxGridPoint) {
      return (false); // Boolean
    }
    return (true); // Boolean
  },

  addSegment : function(startIndex, endIndex)
  {
    // summary:
    //		Add a new segment to this grid line.
    // tags:
    //		private
    
    var seg = new LLGridSegment(this, startIndex, endIndex, null);

    if (this._segments == null)
    {
      this._segments = seg;
    }
    else if (this._isBinTree)
    {
      var otherseg = this.searchClosestSegment(startIndex, false);

      if (otherseg != null)
      {
        seg.setNexts(null, otherseg.getNext1());
        otherseg.setNexts(otherseg.getNext0(), seg);
      }
      else
      {
        otherseg = this.searchClosestSegment(endIndex, true);

        otherseg.setNexts(seg, otherseg.getNext1());
      }
    }
    else
    {
      var predseg = this.searchClosestSegment(startIndex, false);

      if (predseg != null)
      {
        seg.setNexts(null, predseg.getNext1());
        predseg.setNexts(null, seg);
      }
      else
      {
        seg.setNexts(null, this._segments);
        this._segments = seg;
      }
    }
  },

  searchSegment : function(i)
  {
    // summary:
    //		Search the segment of this line that has index i within its range.
    // tags:
    //		private
    
    var seg = this._segments;

    if (this._isBinTree)
    {
      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (i < seg.getStartIndex()) {
          seg = seg.getNext0();
        }
        else {
          seg = seg.getNext1();
        }
      }
    }
    else
    {
      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        seg = seg.getNext1();
      }
    }

    return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchClosestSegment : function(i, larger)
  {
    // summary:
    //		Search the closest segment to index i. 
    // description:
    //		Search the closest segment to index i. 
    //		It either has index i within its range, or is larger or smaller that
    //		i, depending on the flag "larger".
    // tags:
    //		private
    
    var seg = this._segments;

    if (this._isBinTree)
    {
      var bestSeg = null;

      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (larger == true && i < seg.getStartIndex()) {
          bestSeg = seg;
        }
        if (larger == false && seg.getEndIndex() < i) {
          bestSeg = seg;
        }
        if (i < seg.getStartIndex()) {
          seg = seg.getNext0();
        }
        else {
          seg = seg.getNext1();
        }
      }

      return (bestSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    else
    {
      var prevSeg = null;

      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (i < seg.getStartIndex())
        {
          if (larger) {
            return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
          else {
            return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
        }

        prevSeg = seg;
        seg = seg.getNext1();
      }

      if (!larger) {
        return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
    }

    return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchClosestSegmentAlive : function(grid, i, larger)
  {
    // summary:
    //		Search the closest segment to index i that is not a deadlock. 
    // description:
    //		Search the closest segment to index i that is not a deadlock. 
    //		It either has index i within its range, or is larger or smaller that
    //		i, depending on the flag "larger".
    // tags:
    //		private
    
    var seg = this._segments;
    var minIndex;
    var maxIndex;

    if (larger)
    {
      minIndex = i;
      maxIndex = /*Number.MAX_VALUE*/2147483647;
    }
    else
    {
      minIndex = 0;
      maxIndex = i;
    }

    if (this._isBinTree)
    {
      if (larger)
      {
        while (seg != null)
        {
          if (seg.getEndIndex() >= i) {
            break;
          }
          seg = seg.getNext1();
        }
      }
      else
      {
        while (seg != null)
        {
          if (seg.getStartIndex() <= i) {
            break;
          }
          seg = seg.getNext0();
        }
      }

      return this.searchClosestSegmentAliveBinTree(grid, i, larger, minIndex, maxIndex, seg, null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    else
    {
      var prevSeg = null;

      while (seg != null && seg.isDeadlock(grid, minIndex, maxIndex))
        seg = seg.getNext1();

      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (i < seg.getStartIndex())
        {
          if (larger) {
            return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
          else {
            return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
        }

        prevSeg = seg;
        seg = seg.getNext1();

        while (seg != null && seg.isDeadlock(grid, minIndex, maxIndex))
          seg = seg.getNext1();
      }

      if (!larger) {
        return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
    }

    return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchClosestSegmentAliveBinTree : function(grid, i, larger, minIndex, maxIndex, seg, bestSeg)
  {
    // summary:
    //		Search the closest segment to index i that is not a deadlock in the
    //		binary tree.
    // tags:
    //		private
    
    if (seg == null) {
      return bestSeg; // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    if (!seg.isDeadlock(grid, minIndex, maxIndex))
    {
      if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
        return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
      if (larger)
      {
        if (i < seg.getStartIndex() && (bestSeg == null || seg.getStartIndex() < bestSeg.getStartIndex())) {
          bestSeg = seg;
        }
      }
      else
      {
        if (seg.getEndIndex() < i && (bestSeg == null || bestSeg.getEndIndex() < seg.getEndIndex())) {
          bestSeg = seg;
        }
      }
    }

    bestSeg = this.searchClosestSegmentAliveBinTree(grid, i, larger, minIndex, maxIndex, seg.getNext0(), bestSeg);
    bestSeg = this.searchClosestSegmentAliveBinTree(grid, i, larger, minIndex, maxIndex, seg.getNext1(), bestSeg);
    return bestSeg; // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchOrthogonalSegment : function(grid, i)
  {
    // summary:
    //		Returns the grid segment that intersects this grid line at index
    //		position i.
    // tags:
    //		private
    
    var dir;
    if (this._isHorizontal) {
      dir = 0;
    }
    else {
      dir = 1;
    }
    return (grid.getGridLine(dir, i).searchSegment(this._index)); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  calcApproxCrossings : function(grid, start, end)
  {
    // summary:
    //		Calculates the approximate number of link crossings (line obstacle
    //		crossings) of a segment ranging from start to end on this grid line.
    //		
    // description:
    //		Calculates the approximate number of link crossings (line obstacle
    //		crossings) of a segment ranging from start to end on this grid line.
    //		
    //		It is precise if all line obstacles are on the grid. But if a line
    //		obstacle is not orthogonal, or if two line obstacles overlap at the
    //		same grid line, the result may be slightly wrong. But still good
    //		enough to serve as cost value for the heuristic. Of course this
    //		makes sense only if there is a segment covering from start to end on
    //		this grid line.
    // tags:
    //		private
    
    var numCrossings = 0;
    var min = ((start < end ? start : end));
    var max = ((start > end ? start : end));

    for (var i = min; i < max; i++)
    {
      if (this.searchOrthogonalSegment(grid, i) == null) {
        numCrossings++;
      }
    }

    return (numCrossings); // int
  },

  setObstacle : function(grid, obstacleStartIndex, obstacleEndIndex)
  {
    // summary:
    //		Mark that there is an obstacle in the grid line.
    // tags:
    //		private
    
    var seg = this._segments;
    grid.markDirty(this);
    var hasInvalids = false;

    if (this._isBinTree)
    {
      hasInvalids = this.setObstacleRec(this._segments, obstacleStartIndex, obstacleEndIndex);
    }
    else
    {
      var segInv;

      while (seg != null && seg.getStartIndex() <= obstacleEndIndex)
      {
        seg.setObstacle(obstacleStartIndex, obstacleEndIndex);
        segInv = seg.isInvalid();
        hasInvalids = segInv || hasInvalids;
        seg = seg.getNext1();
      }
    }

    if (hasInvalids && grid.isImmediateReorganizationNeeded()) {
      this.reorganizeBeforeSearch(true);
    }
  },

  setObstacleRec : function(segment, obstacleStartIndex, obstacleEndIndex)
  {
    // summary:
    //		In case of a binary tree, this recursive method does the real work. 
    // description:
    //		In case of a binary tree, this recursive method does the real work. 
    //		It returns true if any segment becomes invalid during setting
    //		obstacles.
    // tags:
    //		private
    
    if (segment == null) {
      return (false); // Boolean
    }
    segment.setObstacle(obstacleStartIndex, obstacleEndIndex);
    var hasInvalids = segment.isInvalid();
    var result;

    if (segment.isInvalid() || segment.getStartIndex() > obstacleStartIndex)
    {
      result = this.setObstacleRec(segment.getNext0(), obstacleStartIndex, obstacleEndIndex);
      hasInvalids = result || hasInvalids;
    }

    if (segment.isInvalid() || segment.getEndIndex() < obstacleEndIndex)
    {
      result = this.setObstacleRec(segment.getNext1(), obstacleStartIndex, obstacleEndIndex);
      hasInvalids = result || hasInvalids;
    }

    return (hasInvalids); // Boolean
  }

}
);

return LLGridLine;
});
