/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var LLGridSegment = declare("ibm_ilog.graphlayout.longlink.LLGridSegment", null, 
{
  // summary:
  //		A grid segment of a grid line of the routing grid. 
  // description:
  //		A grid segment of a grid line of the routing grid. 
  //		<p>
  //		It represents an area where a link can be routed along the grid line.
  //		During a routing, it also stores the minimal number of segments
  //		needed to reach this segment, the backtrack distance of this segment
  //		(i.e. the amount we needed to go away from the target in order to
  //		bypass obstacles to reach the target), the area of the segment where
  //		the backtrack distance is valid (which corresponds to the visited
  //		part of the segment). For details, see class LLGrid.  
  //		</p>
  // tags:
  //		private
  
  _gridLine: null,

  _startIndex: null,
  _endIndex: null,

  _backtrackDist: null,

  _minSegmentNumber: null,

  _btStartIndex: null,
  _btEndIndex: null,

  _next0: null,
  _next1: null,

  _btnext: null,

  constructor : function(owner, startIndex, endIndex, next)
  {
    this._gridLine = owner;
    this._startIndex = startIndex;
    this._endIndex = endIndex;
    this._backtrackDist = /*Number.MAX_VALUE*/2147483647;
    this._btStartIndex = /*Number.MAX_VALUE*/2147483647;
    this._btEndIndex = -1;
    this._minSegmentNumber = /*Number.MAX_VALUE*/2147483647;
    this._next0 = null;
    this._next1 = next;
    this._btnext = null;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._next0 = null;
    this._next1 = null;
    this._btnext = null;
    this._gridLine = null;
  },

  getGridLine : function()
  {
    // summary:
    //		Returns the owner grid line of this segment.
    // tags:
    //		private
    
    return (this._gridLine); // ibm_ilog.graphlayout.longlink.LLGridLine
  },

  getStartIndex : function()
  {
    // summary:
    //		Returns the start index of the segment, i.e. 
    // description:
    //		Returns the start index of the segment, i.e. 
    //		the index of the grid point where the segment starts. If it is a
    //		horizontal segment, this is an x-index while the own index of the
    //		grid line is an y index, otherwise convers.
    // tags:
    //		private
    
    return (this._startIndex); // int
  },

  getEndIndex : function()
  {
    // summary:
    //		Returns the end index of the segment, i.e. 
    // description:
    //		Returns the end index of the segment, i.e. 
    //		the index of the grid point where the segment ends. If it is a
    //		horizontal segment, this is an x-index while the own index of the
    //		grid line is an y index, otherwise convers.
    // tags:
    //		private
    
    return (this._endIndex); // int
  },

  setStartIndex : function(index)
  {
    // summary:
    //		Sets the start index.
    // tags:
    //		private
    
    this._startIndex = index;
  },

  setEndIndex : function(index)
  {
    // summary:
    //		Sets the end index.
    // tags:
    //		private
    
    this._endIndex = index;
  },

  getNext1 : function()
  {
    // summary:
    //		Returns the next segment in the same gridline, or null.
    // tags:
    //		private
    
    return (this._next1); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  getNext0 : function()
  {
    // summary:
    //		Returns the next segment in the same gridline, if the grid line is
    //		organized as binary tree, or null.
    // tags:
    //		private
    
    return (this._next0); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  setNexts : function(seg0, seg1)
  {
    // summary:
    //		Sets the both next pointer during reorganizing the gridline into a
    //		binary tree, or back into a list.
    // tags:
    //		private
    
    this._next0 = seg0;
    this._next1 = seg1;
  },

  isHorizontal : function()
  {
    // summary:
    //		Returns true if the segment is horizontal.
    // tags:
    //		private
    
    return (this.getGridLine().isHorizontal()); // Boolean
  },

  isDeadlock : function(grid, minIndex, maxIndex)
  {
    // summary:
    //		Returns true if the segment is an immediate deadlock withing
    //		minIndex and maxIndex, i.e. 
    // description:
    //		Returns true if the segment is an immediate deadlock withing
    //		minIndex and maxIndex, i.e. 
    //		if there is no other segment to continue.
    // tags:
    //		private
    
    if (this.isInvalid()) {
      return (true); // Boolean
    }
    var line = this.getGridLine();
    if (minIndex < this._startIndex) {
      minIndex = this._startIndex;
    }
    if (maxIndex > this._endIndex) {
      maxIndex = this._endIndex;
    }
    for (var i = minIndex; i <= maxIndex; i++)
      if (line.searchOrthogonalSegment(grid, i) != null) {
        return (false); // Boolean
      }
    return (true); // Boolean
  },

  isInvalid : function()
  {
    // summary:
    //		Returns true if the segment is not valid anymore. 
    // description:
    //		Returns true if the segment is not valid anymore. 
    //		This happens if a segment gets completely overlapped by an obstacle.
    // tags:
    //		private
    
    return (this._startIndex == -1 && this._endIndex == -1); // Boolean
  },

  removeInvalidNext : function()
  {
    // summary:
    //		Removes the next segment if it is invalid. 
    // description:
    //		Removes the next segment if it is invalid. 
    //		This should only be called while the gridline is a list.
    // tags:
    //		private
    
    var seg = this._next1;
    var aux;

    while (seg != null && seg.isInvalid())
    {
      aux = seg.getNext1();
      seg.dispose();
      seg = aux;
    }

    this._next1 = seg;
  },

  setObstacle : function(obstacleStartIndex, obstacleEndIndex)
  {
    // summary:
    //		Mark that there is an obstacle in the segment. 
    // description:
    //		Mark that there is an obstacle in the segment. 
    //		If it is in the middle of the segment, this splits the segment into
    //		two. This even works correct if obstacleStartIndex =
    //		obstacleEndIndex + 1, which happens e.g., for objects that are in
    //		beetwen two grid lines. This theoretically works if the grid line is
    //		a list or a binary tree,  but it will make the binary tree
    //		unbalanced,
    // tags:
    //		private
    
    if (this.isInvalid()) {
      return;
    }
    if (obstacleStartIndex > this._endIndex) {
      return;
    }
    if (obstacleEndIndex < this._startIndex) {
      return;
    }
    if (obstacleStartIndex <= this._startIndex)
    {
      if (this._endIndex <= obstacleEndIndex)
      {
        this._startIndex = -1;
        this._endIndex = -1;
      }
      else {
        this._startIndex = obstacleEndIndex + 1;
      }
    }
    else if (this._endIndex <= obstacleEndIndex)
    {
      this._endIndex = obstacleStartIndex - 1;
    }
    else
    {
      var seg = new LLGridSegment(this._gridLine, obstacleEndIndex + 1, this._endIndex, this._next1);
      this._next1 = seg;
      this._endIndex = obstacleStartIndex - 1;
    }
  },

  canBacktrack : function()
  {
    // summary:
    //		Returns true if the segment is a backtrack segment. 
    // description:
    //		Returns true if the segment is a backtrack segment. 
    //		This is the case, if not the complete segment is already covered by
    //		a valid backtrack distance.
    // tags:
    //		private
    
    return (this._btStartIndex > this._startIndex || this._btEndIndex < this._endIndex); // Boolean
  },

  getBacktrackStartIndex : function()
  {
    // summary:
    //		Returns the start index of the area where the current backtrack
    //		distance is valid or smaller. 
    // description:
    //		Returns the start index of the area where the current backtrack
    //		distance is valid or smaller. 
    //		In the remaining area, the real backtrack distance is larger than
    //		the stored one.
    // tags:
    //		private
    
    return (this._btStartIndex); // int
  },

  getBacktrackEndIndex : function()
  {
    // summary:
    //		Returns the end index of the area where the current backtrack
    //		distance is valid or smaller. 
    // description:
    //		Returns the end index of the area where the current backtrack
    //		distance is valid or smaller. 
    //		In the remaining area, the real backtrack distance is larger than
    //		the stored one.
    // tags:
    //		private
    
    return (this._btEndIndex); // int
  },

  setBacktrackStartIndex : function(startIndex)
  {
    // summary:
    //		Sets the start index of the area where the current backtrack
    //		distance is valid or smaller.
    // tags:
    //		private
    
    this._btStartIndex = startIndex;
  },

  setBacktrackEndIndex : function(endIndex)
  {
    // summary:
    //		Sets the end index of the area where the current backtrack distance
    //		is valid or smaller.
    // tags:
    //		private
    
    this._btEndIndex = endIndex;
  },

  isOnBacktrackStack : function()
  {
    // summary:
    //		Returns true if the segment is on the backtrack stack. 
    // description:
    //		Returns true if the segment is on the backtrack stack. 
    //		Note that the backtrack stack is so organized such that all real
    //		segments have a next segment, i.e. the last real segment points to a
    //		dummy next segment. Therefore it is enough to check the next
    //		pointer.
    // tags:
    //		private
    
    return (this._btnext != null); // Boolean
  },

  getNextBacktrackSegement : function()
  {
    // summary:
    //		Returns the next segment in the stack of backtrack segments, or
    //		null, if it's not on the backtrack stack.
    // tags:
    //		private
    
    return (this._btnext); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  setNextBacktrackSegement : function(next)
  {
    // summary:
    //		Sets the next segment in the stack of backtrack segments.
    // tags:
    //		private
    
    this._btnext = next;
  },

  getBacktrackDist : function()
  {
    // summary:
    //		Returns the stored backtrack distance of the segment.
    // tags:
    //		private
    
    return (this._backtrackDist); // int
  },

  setBacktrackDist : function(dist)
  {
    // summary:
    //		Set the backtrack distance of the segment.
    // tags:
    //		private
    
    this._backtrackDist = dist;
  },

  calcBacktrackDist : function(i)
  {
    // summary:
    //		Returns the real backtrack distance at index position i.
    // tags:
    //		private
    
    var backtrackDist = this.getBacktrackDist();
    if (backtrackDist == /*Number.MAX_VALUE*/2147483647) {
      return /*Number.MAX_VALUE*/2147483647; // int
    }
    if (i < this._startIndex || i > this._endIndex) {
      return /*Number.MAX_VALUE*/2147483647; // int
    }
    if (i < this._btStartIndex) {
      return (backtrackDist + this._btStartIndex - i); // int
    }
    if (i > this._btEndIndex) {
      return (backtrackDist + i - this._btEndIndex); // int
    }
    return (backtrackDist); // int
  },

  getMinSegmentNumber : function()
  {
    // summary:
    //		Returns the minimal number of segments necessary to reach this
    //		segment.
    // tags:
    //		private
    
    return (this._minSegmentNumber); // int
  },

  setMinSegmentNumber : function(segNumber)
  {
    // summary:
    //		Sets the minimal number of segments necessary to reach this segment.
    // tags:
    //		private
    
    this._minSegmentNumber = segNumber;
  },

  updateMinSegmentNumber : function(segNumber)
  {
    // summary:
    //		Updates the minimal number of segments necessary to reach this
    //		segment.
    // tags:
    //		private
    
    if (segNumber < this._minSegmentNumber)
    {
      this._minSegmentNumber = segNumber;
    }
  },

  clear : function()
  {
    // summary:
    //		Clear this segment before routing.
    // tags:
    //		private
    
    this._btnext = null;
    this._backtrackDist = /*Number.MAX_VALUE*/2147483647;
    this._btStartIndex = /*Number.MAX_VALUE*/2147483647;
    this._btEndIndex = -1;
    this._minSegmentNumber = /*Number.MAX_VALUE*/2147483647;
  },

  clearRecursive : function()
  {
    // summary:
    //		Clear recursively all segments reaching from this segment.
    // tags:
    //		private
    
    this.clear();
    if (this._next0 != null) {
      this._next0.clearRecursive();
    }
    if (this._next1 != null) {
      this._next1.clearRecursive();
    }
  },

  shiftIndex : function(delta)
  {
    // summary:
    //		Shifts the segment indexe by delta.
    // tags:
    //		private
    
    if (!this.isInvalid())
    {
      this._startIndex += delta;
      this._endIndex += delta;
      if (this._btStartIndex != /*Number.MAX_VALUE*/2147483647) {
        this._btStartIndex += delta;
      }
      if (this._btEndIndex != -1) {
        this._btEndIndex += delta;
      }
    }

    if (this._next0 != null) {
      this._next0.shiftIndex(delta);
    }
    if (this._next1 != null) {
      this._next1.shiftIndex(delta);
    }
  },

  searchOrthogonalSegment : function(grid, i)
  {
    // summary:
    //		Returns the grid segment that intersects this grid line at index
    //		position i.
    // tags:
    //		private
    
    return (this.getGridLine().searchOrthogonalSegment(grid, i)); // ibm_ilog.graphlayout.longlink.LLGridSegment
  }

}
);

return LLGridSegment;
});
