/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/HashTable",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../longlink/LLIncrementalLink",	// #8
  "../longlink/LLIncrementalNode"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    HashTable,	// #6
    TranslateUtil,	// #7
    LLIncrementalLink,	// #8
    LLIncrementalNode	// #9
    ) {

var LLIncrementalData = declare("ibm_ilog.graphlayout.longlink.LLIncrementalData", null, 
{
  // summary:
  //		Data container for nodes and links when using incremental mode. 
  // description:
  //		Data container for nodes and links when using incremental mode. 
  //		We need this data in order to figure out which links have changed and
  //		must be rerouted, and where links must start or end. The data is
  //		updated when incremental is switched on, and after layout is
  //		performed when incremental is already on.  
  // tags:
  //		private
  
  _nodeData: null,
  _linkData: null,

  constructor : function()
  {
    this._nodeData = null;
    this._linkData = null;
  },
  
  update : function(layout)
  {
    // summary:
    //		Updates the incremental data for the layout algorithm.
    // tags:
    //		private
    
    var model = layout.getGraphModel();
    if (model == null) {
      return;
    }
    var numNodes = model.getNodesCount();
    var numLinks = model.getLinksCount();
    this._nodeData = new HashTable();
    this._linkData = new HashTable();
    var obj;
    var e = new TranslateUtil.ArrayIterator(model.getNodes());

    while (e.hasNext())
    {
      obj = e.next();
      this._nodeData.add(obj, new LLIncrementalNode(layout, obj));
      layout.addStepCount(1);
    }

    e = new TranslateUtil.ArrayIterator(model.getLinks());

    while (e.hasNext())
    {
      obj = e.next();
      this._linkData.add(obj, new LLIncrementalLink(layout, obj));
      layout.addStepCount(1);
    }
  },

  getNodeData : function(node)
  {
    // summary:
    //		Returns the incremental data for a node.
    // tags:
    //		private
    
    if (this._nodeData == null) {
      return (null); // ibm_ilog.graphlayout.longlink.LLIncrementalNode
    }
    return this._nodeData.item(node); // ibm_ilog.graphlayout.longlink.LLIncrementalNode
  },

  getLinkData : function(link)
  {
    // summary:
    //		Returns the incremental data for a link.
    // tags:
    //		private
    
    if (this._linkData == null) {
      return (null); // ibm_ilog.graphlayout.longlink.LLIncrementalLink
    }
    return this._linkData.item(link); // ibm_ilog.graphlayout.longlink.LLIncrementalLink
  },

  prepare : function(layout, grid)
  {
    // summary:
    //		Prepares the data before the next layout.
    // tags:
    //		private
    
    var model = layout.getGraphModel();
    if (model == null) {
      return;
    }
    if (this._nodeData != null)
    {
      var nodeDataToBeRemoved = new Array();
      var e;
      var inode;

      e = new TranslateUtil.ArrayIterator(this._nodeData.getValueList());

      while (e.hasNext())
      {
        inode = e.next();
        if (model.isNode(inode.getNodeObject())) {
          inode.prepare(layout, grid);
        }
        else {
          nodeDataToBeRemoved.push(inode);
        }
        layout.addStepCount(1);
      }

      e =  new TranslateUtil.ArrayIterator(nodeDataToBeRemoved);

      while (e.hasNext())
      {
        inode = e.next();
        this._nodeData.remove(inode.getNodeObject());
      }
    }
  },

  clear : function(link)
  {
    // summary:
    //		Clears the incremental data such that the input link will be
    //		rerouted next time (even if it hasn't changed).
    // tags:
    //		private
    
    var ilink = this.getLinkData(link);
    if (ilink != null) {
      ilink.clear();
    }
  },

  cleanNode : function(node)
  {
    // summary:
    //		Cleans the node data when the node is removed from the graph.
    // tags:
    //		private
    
    if (this._nodeData != null) {
      this._nodeData.remove(node);
    }
  },

  cleanLink : function(link)
  {
    // summary:
    //		Cleans the link data when the link is removed from the graph.
    // tags:
    //		private
    
    if (this._linkData != null) {
      this._linkData.remove(link);
    }
  }

}
);

return LLIncrementalData;
});
