/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var LLIncrementalLink = declare("ibm_ilog.graphlayout.longlink.LLIncrementalLink", null, 
{
  // summary:
  //		Data for links when using incremental mode. 
  // description:
  //		Data for links when using incremental mode. 
  //		We need this data in order to figure out which links have changed and
  //		must be rerouted. The data is updated when incremental is switched
  //		on, and after layout is performed when incremental is already on.  
  // tags:
  //		private
  
  _linkObject: null,

  _points: null,

  _numPoints: null,

  _ifromNode: null,
  _itoNode: null,

  _fromFixed: null,
  _toFixed: null,

  _newFromPoint: null,
  _newToPoint: null,

  constructor : function(layout, link)
  {
    this._linkObject = link;
    this.update(layout);
  },
  
  update : function(layout)
  {
    // summary:
    //		Updates the link points and data that define the link.
    // tags:
    //		private
    
    var model = layout.getGraphModel();
    var idata = layout.getIncrementalData();
    var linkObject = this.getLinkObject();
    this._points = model.getLinkPoints(linkObject);
    this._numPoints = this._points.length;
    this._ifromNode = idata.getNodeData(model.getFrom(linkObject));
    this._itoNode = idata.getNodeData(model.getTo(linkObject));
    this._fromFixed = layout.isFromPointFixed(linkObject);
    this._toFixed = layout.isToPointFixed(linkObject);

    if (layout.isIncrementalConnectionPreserving())
    {
      this._newFromPoint = this.getFromPoint();
      this._newToPoint = this.getToPoint();
    }
    else
    {
      this._newFromPoint = null;
      this._newToPoint = null;
    }

    if (this._numPoints >= 2)
    {
      if (this._ifromNode != null) {
        this._ifromNode.updateLink(this, this._points[0], true, this.isHorizontal(this._points[0], this._points[1]));
      }
      if (this._itoNode != null) {
        this._itoNode.updateLink(this, this._points[this._numPoints - 1], false, this.isHorizontal(this._points[this._numPoints - 1], this._points[this._numPoints - 2]));
      }
    }
  },

  clear : function()
  {
    // summary:
    //		Clears the incremental data such that this link will be rerouted
    //		next time (even if it hasn't changed).
    // tags:
    //		private
    
    if (this._numPoints > 0) {
      this._numPoints = -this._numPoints;
    }
  },

  isHorizontal : function(p1, p2)
  {
    // summary:
    //		Returns 'true' if a link segment is "more" horizontal
    //		than vertical. 
    // description:
    //		Returns <code>true</code> if a link segment is "more" horizontal
    //		than vertical. 
    //		This is just a heuristic, that also works for links that are not
    //		perfectly orthogonal.
    // tags:
    //		private
    
    var dx = p1.x - p2.x;
    var dy = p1.y - p2.y;
    if (dx < 0.0) {
      dx = -dx;
    }
    if (dy < 0.0) {
      dy = -dy;
    }
    return (dx >= dy); // Boolean
  },

  needsReroute : function(model, grid)
  {
    // summary:
    //		Returns 'true' if the link must be rerouted. 
    // description:
    //		Returns <code>true</code> if the link must be rerouted. 
    //		This is the case if one of the following holds: - its incremental
    //		data is incomplete (thas is, e.g., the link is new) - its "from" or
    //		"to" node has changed - any of its link points has changed - the
    //		link overlaps any obstacle - the link is not orthogonal
    // tags:
    //		private
    
    var fromNode = this.getFromNode();
    var toNode = this.getToNode();
    var linkObject = this.getLinkObject();
    if (fromNode == null) {
      return (true); // Boolean
    }
    if (toNode == null) {
      return (true); // Boolean
    }
    if (fromNode.getNodeObject() != model.getFrom(linkObject)) {
      return (true); // Boolean
    }
    if (toNode.getNodeObject() != model.getTo(linkObject)) {
      return (true); // Boolean
    }
    if (fromNode.needsReroute()) {
      return (true); // Boolean
    }
    if (toNode.needsReroute()) {
      return (true); // Boolean
    }

    var actPoints = model.getLinkPoints(linkObject);
    var actNumPoints = actPoints.length;

    if (actNumPoints != this._numPoints) {
      return (true); // Boolean
    }

    var i;

    for (i = 0; i < actNumPoints; i++)
      if (!TranslateUtil.pointEquals(actPoints[i], this._points[i])) {
        return (true); // Boolean
      }
    if (actNumPoints < 2) {
      return (true); // Boolean
    }

    var p1;
    var p2;
    var limits = TranslateUtil.newArray(2, 0);
    var index;
    var sign;
    var deltax;
    var deltay;

    for (i = 0; i < actNumPoints - 1; i++)
    {
      p1 = actPoints[i];
      p2 = actPoints[i + 1];
      deltax = p1.x - p2.x;
      deltay = p1.y - p2.y;

      if (deltax == 0.0)
      {
        index = grid.getClosestGridIndex(0, p1.x);

        deltax = grid.getGridCoord(0, index) - p1.x;

        if ((i > 0 && i < actNumPoints - 2) || (i == 0 && !this.isFromPointFixed()) || (i == actNumPoints - 2 && !this.isToPointFixed()))
        {
          if (deltax > 0.5 || deltax < -0.5)
          {
            return (true); // Boolean
          }
        }

        limits[0] = grid.getClosestGridIndex(1, p1.y);
        limits[1] = grid.getClosestGridIndex(1, p2.y);
        sign = ((limits[0] <= limits[1] ? 1 : -1));

        if (i == 0)
        {
          var recta = fromNode.getOldRect();

          while (grid.isWithinRectObstacle(index, limits[0], recta))
            limits[0] += sign;
        }

        if (i == actNumPoints - 2)
        {
          var rectb = toNode.getOldRect();

          while (grid.isWithinRectObstacle(index, limits[1], rectb))
            limits[1] -= sign;
        }

        if (!grid.isWithoutObstacles(1, index, limits))
        {
          return (true); // Boolean
        }
      }
      else if (deltay == 0)
      {
        index = grid.getClosestGridIndex(1, p1.y);

        deltay = grid.getGridCoord(1, index) - p1.y;

        if ((i > 0 && i < actNumPoints - 2) || (i == 0 && !this.isFromPointFixed()) || (i == actNumPoints - 2 && !this.isToPointFixed()))
        {
          if (deltay > 0.5 || deltay < -0.5)
          {
            return (true); // Boolean
          }
        }

        limits[0] = grid.getClosestGridIndex(0, p1.x);
        limits[1] = grid.getClosestGridIndex(0, p2.x);
        sign = ((limits[0] <= limits[1] ? 1 : -1));

        if (i == 0)
        {
          var rectc = fromNode.getOldRect();

          while (grid.isWithinRectObstacle(limits[0], index, rectc))
            limits[0] += sign;
        }

        if (i == actNumPoints - 2)
        {
          var rectd = toNode.getOldRect();

          while (grid.isWithinRectObstacle(limits[1], index, rectd))
            limits[1] -= sign;
        }

        if (!grid.isWithoutObstacles(0, index, limits))
        {
          return (true); // Boolean
        }
      }
      else
      {
        return (true); // Boolean
      }
    }

    return (false); // Boolean
  },

  getLinkObject : function()
  {
    // summary:
    //		Returns the corresponding link.
    // tags:
    //		private
    
    return this._linkObject; // Object
  },

  getFromNode : function()
  {
    // summary:
    //		Returns the incremental data of the "from" node.
    // tags:
    //		private
    
    return this._ifromNode; // ibm_ilog.graphlayout.longlink.LLIncrementalNode
  },

  getToNode : function()
  {
    // summary:
    //		Returns the incremental data of the "to" node.
    // tags:
    //		private
    
    return this._itoNode; // ibm_ilog.graphlayout.longlink.LLIncrementalNode
  },

  getFromPoint : function()
  {
    // summary:
    //		Returns the "from" point of the previous route.
    // tags:
    //		private
    
    return this._points[0]; // Point
  },

  getToPoint : function()
  {
    // summary:
    //		Returns the "to" point of the previous route.
    // tags:
    //		private
    
    var n = ((this._numPoints > 0 ? this._numPoints : -this._numPoints));
    return this._points[n - 1]; // Point
  },

  isFromPointFixed : function()
  {
    // summary:
    //		Returns true if the "from" point was fixed.
    // tags:
    //		private
    
    return this._fromFixed; // Boolean
  },

  isToPointFixed : function()
  {
    // summary:
    //		Returns true if the "to" point was fixed.
    // tags:
    //		private
    
    return this._toFixed; // Boolean
  },

  setNewFromPoint : function(p)
  {
    // summary:
    //		Sets the new "from" point for the next route.
    // tags:
    //		private
    
    this._newFromPoint = p;
  },

  setNewToPoint : function(p)
  {
    // summary:
    //		Sets the new "to" point for the next route.
    // tags:
    //		private
    
    this._newToPoint = p;
  },

  getNewFromPoint : function()
  {
    // summary:
    //		Returns the new "from" point if the end node has moved.
    // tags:
    //		private
    
    return this._newFromPoint; // Point
  },

  getNewToPoint : function()
  {
    // summary:
    //		Returns the new "to" point if the end node has moved.
    // tags:
    //		private
    
    return this._newToPoint; // Point
  }

}
);

return LLIncrementalLink;
});
