/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Point",	// #6
  "../internalutil/QuickSort",	// #7
  "../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Point,	// #6
    QuickSort,	// #7
    TranslateUtil	// #8
    ) {

var LLIncrementalNode = declare("ibm_ilog.graphlayout.longlink.LLIncrementalNode", null, 
{
  // summary:
  //		Data for nodes when using incremental mode. 
  // description:
  //		Data for nodes when using incremental mode. 
  //		We need this data in order to figure out which links have changed and
  //		must be rerouted. The data is updated when incremental is switched
  //		on, and after layout is performed when incremental is already on.  
  // tags:
  //		private
  
  _nodeObject: null,

  _oldRect: null,
  _newRect: null,

  _topLinks: null,
  _bottomLinks: null,
  _leftLinks: null,
  _rightLinks: null,

  constructor : function(layout, node)
  {
    this._nodeObject = node;
    this._topLinks = new Array();
    this._bottomLinks = new Array();
    this._leftLinks = new Array();
    this._rightLinks = new Array();
    this.update(layout);
  },
  
  update : function(layout)
  {
    // summary:
    //		Updates the link points that define the link.
    // tags:
    //		private
    
    this._oldRect = layout.getNodeBoxForConnection(this._nodeObject);
    this._newRect = null;
  },

  updateLink : function(link, p, fromSide, isHorizontal)
  {
    // summary:
    //		Updates the information about an incident link.
    // tags:
    //		private
    
    if (isHorizontal)
    {
      if (p.x >= this._oldRect.x + 0.5 * this._oldRect.width) {
        this._rightLinks.push(new LLIncrementalNode.LinkPointData(link, p.y, fromSide));
      }
      else {
        this._leftLinks.push(new LLIncrementalNode.LinkPointData(link, p.y, fromSide));
      }
    }
    else
    {
      if (p.y >= this._oldRect.y + 0.5 * this._oldRect.height) {
        this._bottomLinks.push(new LLIncrementalNode.LinkPointData(link, p.x, fromSide));
      }
      else {
        this._topLinks.push(new LLIncrementalNode.LinkPointData(link, p.x, fromSide));
      }
    }
  },

  sortLinks : function(v)
  {
    // summary:
    //		Sorts the vector of link point
    //		data's according to the data coordinate.
    // tags:
    //		private
    
    if (v.length > 1)
    {
      var sortAlg = new LLIncrementalNode.LinkPointDataSortAlg(v);
      sortAlg.sort(v.length);
      sortAlg.dispose();
    }
  },

  reassignLinkCoords : function(v, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist)
  {
    // summary:
    //		Reassign link termination coordinates for the links in the link
    //		point data vector. 
    // description:
    //		Reassign link termination coordinates for the links in the link
    //		point data vector. 
    //		The range for the links at the side of the node is minCoord to
    //		maxCoord. Aligned to the grid, it is minGridCoord to maxGridCoord.
    //		If there are enough grid lines available, it chooses the termination
    //		points at the grid lines. Otherwise, it divides the available range
    //		evenly and chooses termination points away from the grid lines.
    // tags:
    //		private
    
    var sz = v.length;
    if (sz == 0) {
      return;
    }

    var coord;
    var delta;

    if ((sz - 1) * gridDist <= maxGridCoord - minGridCoord)
    {
      var k = Math.floor(Math.floor((maxGridCoord - minGridCoord) / gridDist + 0.5));
      k = Math.floor((k - (sz - 1)) / 2);
      coord = minGridCoord + k * gridDist;
      delta = gridDist;
    }
    else
    {
      coord = minCoord;
      delta = (maxCoord - minCoord) / (sz - 1);
    }

    var e =  new TranslateUtil.ArrayIterator(v);
    var d;

    while (e.hasNext())
    {
      d = e.next();
      d._coord = coord;
      coord += delta;
    }
  },

  setNewTerminationPoint : function(lpdata, topBottom)
  {
    // summary:
    //		Sets the input point as new termination point to the link.
    // tags:
    //		private
    
    var x;
    var y;
    var link = lpdata._link;

    if (lpdata._fromSide)
    {
      x = link.getFromPoint().x;
      y = link.getFromPoint().y;
    }
    else
    {
      x = link.getToPoint().x;
      y = link.getToPoint().y;
    }

    var perc;

    if (topBottom)
    {
      perc = (y - this._oldRect.y - 0.5 * this._oldRect.height) / this._oldRect.height;
      y = this._newRect.y + 0.5 * this._newRect.height + perc * this._newRect.height;
      x = lpdata._coord;
    }
    else
    {
      perc = (x - this._oldRect.x - 0.5 * this._oldRect.width) / this._oldRect.width;
      x = this._newRect.x + 0.5 * this._newRect.width + perc * this._newRect.width;
      y = lpdata._coord;
    }

    if (lpdata._fromSide) {
      link.setNewFromPoint(new Point(x, y));
    }
    else {
      link.setNewToPoint(new Point(x, y));
    }
  },

  setNewTerminationPoints : function(v, topBottom)
  {
    // summary:
    //		Sets all input point as new termination point to the links in the
    //		link point data vector.
    // tags:
    //		private
    
    var e =  new TranslateUtil.ArrayIterator(v);
    var d;

    while (e.hasNext())
    {
      d = e.next();
      this.setNewTerminationPoint(d, topBottom);
    }
  },

  prepare : function(layout, grid)
  {
    // summary:
    //		Prepares the incremental node data before the next layout.
    // tags:
    //		private
    
    if (this._newRect == null)
    {
      this._newRect = layout.getNodeBoxForConnection(this._nodeObject);

      if (layout.isIncrementalConnectionPreserving() && this.needsReroute())
      {
        this.sortLinks(this._rightLinks);
        this.sortLinks(this._leftLinks);
        this.sortLinks(this._topLinks);
        this.sortLinks(this._bottomLinks);
        var minCOffs = layout.getMinNodeCornerOffset();
        var minCoord = this._newRect.y + minCOffs;
        var maxCoord = this._newRect.y + this._newRect.height - minCOffs;
        var minGridIndex = grid.getMinGridIndexAtSide(this._newRect, 1, minCOffs);
        var maxGridIndex = grid.getMaxGridIndexAtSide(this._newRect, 1, minCOffs);
        var minGridCoord = grid.getGridCoord(1, minGridIndex);
        var maxGridCoord = grid.getGridCoord(1, maxGridIndex);
        var gridDist = grid.getGridDist(1);
        this.reassignLinkCoords(this._rightLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        this.reassignLinkCoords(this._leftLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        minCoord = this._newRect.x + minCOffs;
        maxCoord = this._newRect.x + this._newRect.width - minCOffs;
        minGridIndex = grid.getMinGridIndexAtSide(this._newRect, 0, minCOffs);
        maxGridIndex = grid.getMaxGridIndexAtSide(this._newRect, 0, minCOffs);
        minGridCoord = grid.getGridCoord(0, minGridIndex);
        maxGridCoord = grid.getGridCoord(0, maxGridIndex);
        gridDist = grid.getGridDist(0);
        this.reassignLinkCoords(this._topLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        this.reassignLinkCoords(this._bottomLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        this.setNewTerminationPoints(this._rightLinks, false);
        this.setNewTerminationPoints(this._leftLinks, false);
        this.setNewTerminationPoints(this._topLinks, true);
        this.setNewTerminationPoints(this._bottomLinks, true);
      }
    }
  },

  needsReroute : function()
  {
    // summary:
    //		Returns 'true' if all incident links must be rerouted
    //		because the node moved.
    // tags:
    //		private
    
    if (this._oldRect == null) {
      return true; // Boolean
    }

    var delta;
    delta = this._newRect.x - this._oldRect.x;
    if (delta != 0) {
      return true; // Boolean
    }
    delta = this._newRect.y - this._oldRect.y;
    if (delta != 0) {
      return true; // Boolean
    }
    delta = this._newRect.width - this._oldRect.width;
    if (delta != 0) {
      return true; // Boolean
    }
    delta = this._newRect.height - this._oldRect.height;
    if (delta != 0) {
      return true; // Boolean
    }
    return (false); // Boolean
  },

  getNodeObject : function()
  {
    // summary:
    //		Returns the corresponding node.
    // tags:
    //		private
    
    return this._nodeObject; // Object
  },

  getOldRect : function()
  {
    // summary:
    //		Returns the bounding rectangle from the previous layout.
    // tags:
    //		private
    
    return this._oldRect; // Rectangle
  }

}
);

LLIncrementalNode.LinkPointData = declare( null, 
{
  // summary:
  //		Inner auxiliary class to combine a link with its termination
  //		coordinate.
  
  _link: null,
  _coord: null,
  _fromSide: null,

  constructor : function(link, coord, fromSide)
  {
    this._link = link;
    this._coord = coord;
    this._fromSide = fromSide;
  }
  
}
);

LLIncrementalNode.LinkPointDataSortAlg = declare(QuickSort, 
{
  // summary:
  //		Inner auxiliary class to sort the link point data vectors according
  //		to the coord.
  
  _v: null,

  constructor : function(v)
  {
    this._v = v;
  },
  
  dispose : function()
  {
  },

  compare : function(loc1, loc2)
  {
    var d1 = this._v[loc1];
    var d2 = this._v[loc2];
    if (d1._coord == d2._coord) {
      return 0; // int
    }
    else if (d1._coord < d2._coord) {
      return -1; // int
    }
    else {
      return 1; // int
    }
  },

  swap : function(loc1, loc2)
  {
    var d1 = this._v[loc1];
    var d2 = this._v[loc2];
    this._v[loc2] =   d1;
    this._v[loc1] =   d2;
  }

}
);

return LLIncrementalNode;
});
