/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Point",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../longlink/LLGridSegment",	// #8
  "../longlink/LLTerminationPoint",	// #9
  "../Direction"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Point,	// #6
    TranslateUtil,	// #7
    LLGridSegment,	// #8
    LLTerminationPoint,	// #9
    Direction	// #10
    ) {

var LLLinkRouteAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLLinkRouteAlgorithm", null, 
{
  // summary:
  //		The algorithm to perform one single routing. 
  // description:
  //		The algorithm to perform one single routing. 
  //		<p>
  //		It traverses the grid from start grid point until it reaches the end
  //		grid point and stores the necessary backtrack distance. This is like
  //		calculating shortest distances in a graph. The it searches one path
  //		backwards from the end point to the start point by passing only over
  //		decreasing backtrack distance points. TODO: needs review  
  //		</p>
  // tags:
  //		private
  
  _layout: null,

  _grid: null,

  _points: null,

  _crossingsInStraightOrOneBendRoute: null,

  _backtrackDist: null,

  _firstValidBacktrackDist: null,

  _backtrackSegments: null,

  _startIndex: null,
  _endIndex: null,

  _startSegment: null,
  _endSegment: null,

  _forbiddenMinIndexAtStart: null,
  _forbiddenMaxIndexAtStart: null,

  _forbiddenMinIndexAtEnd: null,
  _forbiddenMaxIndexAtEnd: null,

  _dummySegment: null,

  _searchExhaustive : null,

  _maxBacktrackDist: null,

  _minAdditionalBacktrackDist : null,

  _elapseTime: null,

  _topPoints: null,
  _bottomPoints: null,
  _leftPoints: null,
  _rightPoints: null,

  constructor : function(layout, grid)
  {
    this._searchExhaustive = false;
    this._minAdditionalBacktrackDist = 5;
    this._layout = layout;
    this._grid = grid;
    this._startIndex = TranslateUtil.newArray(2, 0);
    this._endIndex = TranslateUtil.newArray(2, 0);
    this._points = new Array();
    this._dummySegment = new LLGridSegment(null, 0, 0, null);
    this._searchExhaustive = layout.isExhaustiveSearching();
    this._maxBacktrackDist = layout.getMaxBacktrack();
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._points = null;
    this._dummySegment.dispose();
    this._topPoints = null;
    this._bottomPoints = null;
    this._leftPoints = null;
    this._rightPoints = null;

    this._grid = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getPoints : function()
  {
    // summary:
    //		Returns the calculated points.
    // tags:
    //		private
    
    return (this._points); // Array
  },

  getGrid : function()
  {
    // summary:
    //		Returns the routing grid.
    // tags:
    //		private
    
    return (this._grid); // ibm_ilog.graphlayout.longlink.LLGrid
  },

  routeStraight : function(link)
  {
    // summary:
    //		Route a link from its start node to its end node straight. 
    // description:
    //		Route a link from its start node to its end node straight. 
    //		It returns true on success.
    // tags:
    //		private
    
    var layout = this.getLayout();
    this._elapseTime = new Date().getTime() + layout.getAllowedTimePerLink();

    link.cleanRoute();
    this._points.length = 0;
    if (this.routeBestStraight(link)) {
      return (true); // Boolean
    }
    return (false); // Boolean
  },

  route : function(link)
  {
    // summary:
    //		Route a link from its start node to its end node. 
    // description:
    //		Route a link from its start node to its end node. 
    //		It returns true on success.
    // tags:
    //		private
    
    var layout = this.getLayout();
    this._elapseTime = new Date().getTime() + layout.getAllowedTimePerLink();

    link.cleanRoute();
    this._points.length = 0;

    if (layout.isStraightRouteEnabled())
    {
      if (this.routeBestOneBend(link)) {
        return (true); // Boolean
      }
    }

    if (link.isSelfLoop())
    {
      if (this.routeSelfLoopOptimized(link)) {
        return (true); // Boolean
      }
    }

    var startPointCand = link.getStartPoints();
    var endPointCand = link.getEndPoints();
    var numberStartPointCands = startPointCand.length;
    var numberEndPointCands = endPointCand.length;

    var maxSPIndex = 0;
    var maxEPIndex = 0;
    var actStartPoint;
    var actEndPoint;
    var prevStartPoint = null;
    var prevEndPoint = null;
    var isLastStart;
    var isLastEnd;
    var i;

    while (true)
    {
      if (maxSPIndex < numberStartPointCands)
      {
        if (maxEPIndex < numberEndPointCands)
        {
          if (startPointCand[maxSPIndex].getPenalty() < endPointCand[maxEPIndex].getPenalty())
          {
            actStartPoint = startPointCand[maxSPIndex];
            maxSPIndex++;
            isLastStart = (maxSPIndex == numberStartPointCands);

            if (maxEPIndex > 0 && this.needNewSearch(link, actStartPoint, prevStartPoint))
            {
              prevEndPoint = null;

              for (i = 0; i < maxEPIndex; i++)
              {
                actEndPoint = endPointCand[i];
                isLastEnd = (i == numberEndPointCands - 1);

                if (this.needNewSearch(link, actEndPoint, prevEndPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
                {
                  prevStartPoint = actStartPoint;
                  prevEndPoint = actEndPoint;

                  if (this.searchRoute(actStartPoint, actEndPoint, true))
                  {
                    link.setActStartAndEndCandidates(maxSPIndex - 1, i);
                    link.storeRoute(this.getPoints());
                    return (true); // Boolean
                  }
                }
              }
            }
          }
          else
          {
            actEndPoint = endPointCand[maxEPIndex];
            maxEPIndex++;
            isLastEnd = (maxEPIndex == numberEndPointCands);

            if (maxSPIndex > 0 && this.needNewSearch(link, actEndPoint, prevEndPoint))
            {
              prevStartPoint = null;

              for (i = 0; i < maxSPIndex; i++)
              {
                actStartPoint = startPointCand[i];
                isLastStart = (i == numberStartPointCands - 1);

                if (this.needNewSearch(link, actStartPoint, prevStartPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
                {
                  prevStartPoint = actStartPoint;
                  prevEndPoint = actEndPoint;

                  if (this.searchRoute(actStartPoint, actEndPoint, true))
                  {
                    link.setActStartAndEndCandidates(i, maxEPIndex - 1);
                    link.storeRoute(this.getPoints());
                    return (true); // Boolean
                  }
                }
              }
            }
          }
        }
        else
        {
          actStartPoint = startPointCand[maxSPIndex];
          maxSPIndex++;
          isLastStart = (maxSPIndex == numberStartPointCands);

          if (maxEPIndex > 0 && this.needNewSearch(link, actStartPoint, prevStartPoint))
          {
            prevEndPoint = null;

            for (i = 0; i < maxEPIndex; i++)
            {
              actEndPoint = endPointCand[i];
              isLastEnd = (i == numberEndPointCands - 1);

              if (this.needNewSearch(link, actEndPoint, prevEndPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
              {
                prevStartPoint = actStartPoint;
                prevEndPoint = actEndPoint;

                if (this.searchRoute(actStartPoint, actEndPoint, true))
                {
                  link.setActStartAndEndCandidates(maxSPIndex - 1, i);
                  link.storeRoute(this.getPoints());
                  return (true); // Boolean
                }
              }
            }
          }
        }
      }
      else if (maxEPIndex < numberEndPointCands)
      {
        actEndPoint = endPointCand[maxEPIndex];
        maxEPIndex++;
        isLastEnd = (maxEPIndex == numberEndPointCands);

        if (maxSPIndex > 0 && this.needNewSearch(link, actEndPoint, prevEndPoint))
        {
          prevStartPoint = null;

          for (i = 0; i < maxSPIndex; i++)
          {
            actStartPoint = startPointCand[i];
            isLastStart = (i == numberStartPointCands - 1);

            if (this.needNewSearch(link, actStartPoint, prevStartPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
            {
              prevStartPoint = actStartPoint;
              prevEndPoint = actEndPoint;

              if (this.searchRoute(actStartPoint, actEndPoint, true))
              {
                link.setActStartAndEndCandidates(i, maxEPIndex - 1);
                link.storeRoute(this.getPoints());
                return (true); // Boolean
              }
            }
          }
        }
      }
      else
      {
        break;
      }
    }

    return (false); // Boolean
  },

  routeSelfLoopOptimized : function(link)
  {
    // summary:
    //		Route self loops optimized. 
    // description:
    //		Route self loops optimized. 
    //		Self loops prefer two neighbored termination points at the same
    //		side. However, the order of the termination points in the start and
    //		end candidate arrays is such that self loops are preferably routed
    //		between different sides. To overcome this deficiency, we test here
    //		all pairs of termination point candidates that have minimal penalty,
    //		belong to the same side and are directly neighbored. If we find such
    //		a pair, we route there and return true. If we
    //		don't find such a pair that is successful, then we return false, which implies that we use the normal mechanism then.
    // tags:
    //		private
    
    var startPointCand = link.getStartPoints();
    var endPointCand = link.getEndPoints();
    var numberStartPointCands = startPointCand.length;
    var numberEndPointCands = endPointCand.length;

    if (numberStartPointCands == 0 || numberEndPointCands == 0) {
      return (false); // Boolean
    }
    if (numberStartPointCands == 1 && numberEndPointCands == 1) {
      return (false); // Boolean
    }

    var actSPIndex = 0;
    var actEPIndex = 0;
    var actStartPoint;
    var actEndPoint;
    var dir;
    var bestStartPointPenalty = startPointCand[0].getPenalty();
    var bestEndPointPenalty = endPointCand[0].getPenalty();

    for (actSPIndex = 0; actSPIndex < numberStartPointCands; actSPIndex++)
    {
      actStartPoint = startPointCand[actSPIndex];

      if (actStartPoint.getPenalty() > bestStartPointPenalty) {
        break;
      }
      for (actEPIndex = 0; actEPIndex < numberEndPointCands; actEPIndex++)
      {
        actEndPoint = endPointCand[actEPIndex];

        if (actEndPoint.getPenalty() > bestEndPointPenalty) {
          break;
        }
        if (actStartPoint.getSide() == actEndPoint.getSide())
        {
          if (actStartPoint.getSide() == Direction.TOP || actStartPoint.getSide() == Direction.BOTTOM) {
            dir = 0;
          }
          else {
            dir = 1;
          }
          if (actStartPoint.getIndex(dir) + 1 == actEndPoint.getIndex(dir))
          {
            if (this.nonzeroRoute(actStartPoint, actEndPoint, false, false))
            {
              if (this.searchRoute(actStartPoint, actEndPoint, true))
              {
                link.setActStartAndEndCandidates(actSPIndex, actEPIndex);
                link.storeRoute(this.getPoints());
                return (true); // Boolean
              }
            }
          }
        }
      }
    }

    return (false); // Boolean
  },

  nonzeroRoute : function(p1, p2, p1IsLast, p2IsLast)
  {
    // summary:
    //		Returns true if start and end point are not identical, or if they
    //		are identical but there are no others. 
    // description:
    //		Returns true if start and end point are not identical, or if they
    //		are identical but there are no others. 
    //		This check is needed to avoid that self loops are routed as one
    //		point (i.e. a path with zero length).
    // tags:
    //		private
    
    if (p1.getIndex(0) == p2.getIndex(0) && p1.getIndex(1) == p2.getIndex(1))
    {
      if (!p1IsLast) {
        return (false); // Boolean
      }
      if (!p2IsLast) {
        return (false); // Boolean
      }
    }

    return (true); // Boolean
  },

  needNewSearch : function(link, newPoint, oldPoint)
  {
    // summary:
    //		Returns true if we need to try a search starting (or ending) at the
    //		new point. 
    // description:
    //		Returns true if we need to try a search starting (or ending) at the
    //		new point. 
    //		We tried already a search from old point which was unsuccessful. If
    //		the old point is reachable from new point, then the search starting
    //		from new point must be unsuccessful as well, otherwise the old point
    //		would have been reachable by passing the new point.
    // tags:
    //		private
    
    if (link.isSelfLoop()) {
      return (true); // Boolean
    }
    if (oldPoint == null) {
      return (true); // Boolean
    }
    if (this._maxBacktrackDist < 20) {
      return (true); // Boolean
    }
    var result = true;
    var oldMaxBacktrack = this._maxBacktrackDist;
    var oldMinAddBacktrack = this._minAdditionalBacktrackDist;
    var oldSearchExhaustive = this._searchExhaustive;

    this._maxBacktrackDist = 10;
    this._minAdditionalBacktrackDist = 0;
    this._searchExhaustive = false;
    if (this.searchRoute(newPoint, oldPoint, false)) {
      result = false;
    }
    this._maxBacktrackDist = oldMaxBacktrack;
    this._minAdditionalBacktrackDist = oldMinAddBacktrack;
    this._searchExhaustive = oldSearchExhaustive;
    return (result); // Boolean
  },

  searchRoute : function(startPoint, endPoint, storeBends)
  {
    // summary:
    //		Routes a link from start point to end point.
    // tags:
    //		private
    
    this._grid.clearDirty();
    this._points.length = 0;

    this._startIndex[0] = startPoint.getIndex(0);
    this._startIndex[1] = startPoint.getIndex(1);
    this._endIndex[0] = endPoint.getIndex(0);
    this._endIndex[1] = endPoint.getIndex(1);

    var startDir = 1;
    var endDir = 1;
    this._startSegment = startPoint.getSegment();
    this._endSegment = endPoint.getSegment();
    if (this._startSegment == null) {
      return (false); // Boolean
    }
    if (this._endSegment == null) {
      return (false); // Boolean
    }
    this._forbiddenMinIndexAtStart = startPoint.getForbiddenMinIndex();
    this._forbiddenMaxIndexAtStart = startPoint.getForbiddenMaxIndex();
    this._forbiddenMinIndexAtEnd = endPoint.getForbiddenMinIndex();
    this._forbiddenMaxIndexAtEnd = endPoint.getForbiddenMaxIndex();
    if (this._startSegment.isHorizontal()) {
      startDir = 0;
    }
    if (this._endSegment.isHorizontal()) {
      endDir = 0;
    }
    if (this._startSegment == this._endSegment)
    {
      return (true); // Boolean
    }

    var segStart = this._startSegment.getStartIndex();
    var segEnd = this._startSegment.getEndIndex();

    if (this._forbiddenMaxIndexAtStart >= segStart && this._forbiddenMinIndexAtStart <= segEnd)
    {
      if (segStart < this._forbiddenMinIndexAtStart) {
        this._startSegment.setBacktrackStartIndex(this._forbiddenMinIndexAtStart);
      }
      else {
        this._startSegment.setBacktrackStartIndex(segStart);
      }
      if (segEnd > this._forbiddenMaxIndexAtStart) {
        this._startSegment.setBacktrackEndIndex(this._forbiddenMaxIndexAtStart);
      }
      else {
        this._startSegment.setBacktrackEndIndex(segEnd);
      }
      this._startSegment.setBacktrackDist(0);
      this._grid.markDirty(this._startSegment.getGridLine());
    }

    segStart = this._endSegment.getStartIndex();
    segEnd = this._endSegment.getEndIndex();

    if (this._forbiddenMaxIndexAtEnd >= segStart && this._forbiddenMinIndexAtEnd <= segEnd)
    {
      if (segStart < this._forbiddenMinIndexAtEnd) {
        this._endSegment.setBacktrackStartIndex(this._forbiddenMinIndexAtEnd);
      }
      else {
        this._endSegment.setBacktrackStartIndex(segStart);
      }
      if (segEnd > this._forbiddenMaxIndexAtEnd) {
        this._endSegment.setBacktrackEndIndex(this._forbiddenMaxIndexAtEnd);
      }
      else {
        this._endSegment.setBacktrackEndIndex(segEnd);
      }
      this._endSegment.setBacktrackDist(0);
      this._grid.markDirty(this._endSegment.getGridLine());
    }

    if (this.searchForwardWithBacktrack(this._startSegment, this._startIndex[startDir], startDir))
    {
      if (storeBends)
      {
        this._endSegment.setMinSegmentNumber(LLLinkRouteAlgorithm.MAXVAL);
        this._endSegment.setBacktrackDist(LLLinkRouteAlgorithm.MAXVAL);
        this.searchBackward(this._endSegment, this._endIndex[endDir], endDir);
      }

      return (true); // Boolean
    }

    return (false); // Boolean
  },

  searchForwardWithBacktrack : function(startSegment, startPointIndex, dir)
  {
    // summary:
    //		Search from the start segment to the end segment with backtrack. 
    // description:
    //		Search from the start segment to the end segment with backtrack. 
    //		Backtrack means, if we
    //		don't reach the end segment in the shortest possible distance, allow to move away from the end point by the backtrack distance in order to bypass obstacles. The startSegment is a segment in direction "dir", which is 0 if it is a horizontal segment and 1 if it is a vertical segment. The startPointIndex is the index of the start point of this segment. It returns true if the end segment is found.
    // tags:
    //		private
    
    this._firstValidBacktrackDist = -1;
    this._backtrackDist = 0;
    this._backtrackSegments = this._dummySegment;
    var result = this.searchForward(startSegment, startPointIndex, dir, 0);
    if (result == true) {
      this._firstValidBacktrackDist = 0;
    }
    if (result == true && this._searchExhaustive == false && this._minAdditionalBacktrackDist == 0) {
      return (true); // Boolean
    }
    while (this._backtrackDist < this._maxBacktrackDist && new Date().getTime() <= this._elapseTime)
    {
      this._backtrackDist++;

      var seg = this._backtrackSegments;
      var predseg = null;
      var nrsegs = 0;

      while (seg != this._dummySegment)
      {
        if (!seg.canBacktrack())
        {
          if (predseg == null)
          {
            this._backtrackSegments = seg.getNextBacktrackSegement();
            seg.setNextBacktrackSegement(null);
            seg = this._backtrackSegments;
          }
          else
          {
            predseg.setNextBacktrackSegement(seg.getNextBacktrackSegement());
            seg.setNextBacktrackSegement(null);
            seg = predseg.getNextBacktrackSegement();
          }
        }
        else
        {
          nrsegs++;
          predseg = seg;
          seg = predseg.getNextBacktrackSegement();
        }
      }

      if (nrsegs == 0) {
        break;
      }

      var btsegments = TranslateUtil.newArray(nrsegs, null);
      var btlowerIndex = TranslateUtil.newArray(nrsegs, 0);
      var btupperIndex = TranslateUtil.newArray(nrsegs, 0);
      var nextseg;
      var i = 0;
      seg = this._backtrackSegments;

      while (seg != this._dummySegment)
      {
        nextseg = seg.getNextBacktrackSegement();
        seg.setNextBacktrackSegement(null);
        btsegments[i] = seg;
        btlowerIndex[i] = seg.getBacktrackStartIndex() - 1;
        btupperIndex[i] = seg.getBacktrackEndIndex() + 1;
        i++;
        seg = nextseg;
      }

      this._backtrackSegments = this._dummySegment;

      for (i = 0; i < nrsegs; i++)
      {
        seg = btsegments[i];
        dir = ((seg.isHorizontal() ? 0 : 1));

        if (seg.canBacktrack() && seg.getStartIndex() <= btlowerIndex[i] && seg.getBacktrackStartIndex() > btlowerIndex[i])
        {
          result = this.searchForward(seg, btlowerIndex[i], dir, LLLinkRouteAlgorithm.MAXVAL) || result;

          if (result == true)
          {
            if (this._firstValidBacktrackDist == -1) {
              this._firstValidBacktrackDist = this._backtrackDist;
            }
            if (this._searchExhaustive == false && this._firstValidBacktrackDist + this._minAdditionalBacktrackDist <= this._backtrackDist) {
              break;
            }
          }
        }

        if (seg.canBacktrack() && seg.getEndIndex() >= btupperIndex[i] && seg.getBacktrackEndIndex() < btupperIndex[i])
        {
          result = this.searchForward(seg, btupperIndex[i], dir, LLLinkRouteAlgorithm.MAXVAL) || result;

          if (result == true)
          {
            if (this._firstValidBacktrackDist == -1) {
              this._firstValidBacktrackDist = this._backtrackDist;
            }
            if (this._searchExhaustive == false && this._firstValidBacktrackDist + this._minAdditionalBacktrackDist <= this._backtrackDist) {
              break;
            }
          }
        }
      }

      if (result == true && this._searchExhaustive == false && this._firstValidBacktrackDist + this._minAdditionalBacktrackDist <= this._backtrackDist) {
        break;
      }
    }

    return (result); // Boolean
  },

  searchForward : function(segment, gridPointIndex, dir, minSegmentNr)
  {
    // summary:
    //		Search from the input segment to the end segment. 
    // description:
    //		Search from the input segment to the end segment. 
    //		The input segment is a segment in direction "dir", which is 0 if it
    //		is a horizontal segment and 1 if it is a vertical segment. The grid
    //		point index is the index where we reached the segment. It returns
    //		true if the end segment is found without any backtrack, i.e. without
    //		moving further away from the end point, otherwise false.
    // tags:
    //		private
    
    if (this.isForbidden(gridPointIndex, segment)) {
      return (false); // Boolean
    }
    if (segment == null) {
      return (false); // Boolean
    }
    var grid = this.getGrid();
    var gridLine = segment.getGridLine();
    var ownIndex = gridLine.getIndex();
    var segStart = segment.getStartIndex();
    var segEnd = segment.getEndIndex();

    if (gridPointIndex < segStart || gridPointIndex > segEnd) {
      return (false); // Boolean
    }

    var newAreaStart = -1;
    var newAreaEnd = -1;

    var targetIndex = this._endIndex[dir];
    var oldBacktrackDist = segment.calcBacktrackDist(gridPointIndex);
    var oldBacktrackStart = segment.getBacktrackStartIndex();
    var oldBacktrackEnd = segment.getBacktrackEndIndex();

    if (oldBacktrackDist >= this._backtrackDist)
    {
      if (gridPointIndex <= this._endIndex[dir])
      {
        newAreaStart = gridPointIndex;
        newAreaEnd = ((targetIndex < segEnd ? targetIndex : segEnd));
      }
      else
      {
        newAreaStart = ((targetIndex > segStart ? targetIndex : segStart));
        newAreaEnd = gridPointIndex;
      }

      if (oldBacktrackDist == LLLinkRouteAlgorithm.MAXVAL)
      {
        segment.setBacktrackStartIndex(newAreaStart);
        segment.setBacktrackEndIndex(newAreaEnd);
      }
      else
      {
        if (newAreaStart < oldBacktrackStart) {
          segment.setBacktrackStartIndex(newAreaStart);
        }
        else {
          newAreaStart = oldBacktrackEnd + 1;
        }
        if (newAreaEnd > oldBacktrackEnd) {
          segment.setBacktrackEndIndex(newAreaEnd);
        }
        else {
          newAreaEnd = oldBacktrackStart - 1;
        }
      }

      segment.updateMinSegmentNumber(minSegmentNr);
      segment.setBacktrackDist(this._backtrackDist);
      grid.markDirty(gridLine);
    }

    if (segment == this._endSegment) {
      return (!this.isForbidden(gridPointIndex, segment)); // Boolean
    }
    if (segment.canBacktrack() && !segment.isOnBacktrackStack())
    {
      segment.setNextBacktrackSegement(this._backtrackSegments);
      this._backtrackSegments = segment;
    }

    var result = false;

    if (newAreaStart >= 0 && newAreaEnd >= 0 && newAreaStart <= newAreaEnd)
    {
      minSegmentNr = segment.getMinSegmentNumber() + 1;
      var seg;

      if (targetIndex <= newAreaStart)
      {
        for (var ia = newAreaStart; ia <= newAreaEnd; ia++)
        {
          if (!this.isForbidden(ia, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, ia);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }
        }
      }
      else if (targetIndex >= newAreaEnd)
      {
        for (var ib = newAreaEnd; ib >= newAreaStart; ib--)
        {
          if (!this.isForbidden(ib, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, ib);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }
        }
      }
      else
      {
        var i = targetIndex;
        var j = targetIndex + 1;

        while (i >= newAreaStart || j <= newAreaEnd)
        {
          if (i >= newAreaStart && !this.isForbidden(i, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, i);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }

          if (j <= newAreaEnd && !this.isForbidden(j, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, j);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }

          i--;
          j++;
        }
      }
    }

    return (result); // Boolean
  },

  searchBackward : function(segment, gridPointIndex, dir)
  {
    // summary:
    //		Search from the input segment to the start segment. 
    // description:
    //		Search from the input segment to the start segment. 
    //		Here we visit only the area of the segment that is marked with the
    //		backtrack distance. By that we are sure to reach the start node
    //		without more backtrack then the marked distance. The input segment
    //		is a segment in direction "dir", which is 0 if it is a horizontal
    //		segment and 1 if it is a vertical segment. The grid point index is
    //		the index where we reached the segment.
    // tags:
    //		private
    
    var ownIndex = segment.getGridLine().getIndex();

    if (segment == this._startSegment) {
      return;
    }

    var grid = this.getGrid();
    var i = gridPointIndex;
    var j = gridPointIndex + 1;
    var start = segment.getStartIndex();
    var end = segment.getEndIndex();
    var actMinSegmentNumber;
    var bestIndexToContinue = -1;
    var bestMinSegmentNumberToContinue = LLLinkRouteAlgorithm.MAXVAL;
    var bestSegToContinue = null;
    var seg;

    while (i >= start || j <= end)
    {
      if (i >= start && !this.isForbidden(i, segment))
      {
        seg = segment.searchOrthogonalSegment(grid, i);

        if (seg != null)
        {
          actMinSegmentNumber = seg.getMinSegmentNumber();

          if (!this.isForbidden(ownIndex, seg) && actMinSegmentNumber < bestMinSegmentNumberToContinue)
          {
            bestIndexToContinue = i;
            bestMinSegmentNumberToContinue = actMinSegmentNumber;
            bestSegToContinue = seg;
          }
        }
      }

      if (j <= end && !this.isForbidden(j, segment))
      {
        seg = segment.searchOrthogonalSegment(grid, j);

        if (seg != null)
        {
          actMinSegmentNumber = seg.getMinSegmentNumber();

          if (!this.isForbidden(ownIndex, seg) && actMinSegmentNumber < bestMinSegmentNumberToContinue)
          {
            bestIndexToContinue = j;
            bestMinSegmentNumberToContinue = actMinSegmentNumber;
            bestSegToContinue = seg;
          }
        }
      }

      i--;
      j++;
    }

    if (dir == 1) {
      this._points.push(new Point(ownIndex, bestIndexToContinue));
    }
    else {
      this._points.push(new Point(bestIndexToContinue, ownIndex));
    }
    this.searchBackward(bestSegToContinue, ownIndex, 1 - dir);
  },

  isForbidden : function(index, segment)
  {
    // summary:
    //		Returns true if the index is in the forbidden area of the segment. 
    // description:
    //		Returns true if the index is in the forbidden area of the segment. 
    //		This is mainly used to test that we
    //		don't traverse the area of the start or end segment that would violate the minimal start/end segment length.
    // tags:
    //		private
    
    if (segment == this._startSegment && index >= this._forbiddenMinIndexAtStart && index <= this._forbiddenMaxIndexAtStart) {
      return (true); // Boolean
    }
    if (segment == this._endSegment && index >= this._forbiddenMinIndexAtEnd && index <= this._forbiddenMaxIndexAtEnd) {
      return (true); // Boolean
    }
    return (false); // Boolean
  },

  routeBestStraight : function(link)
  {
    // summary:
    //		Search the best straight route. 
    // description:
    //		Search the best straight route. 
    //		It returns true on success and false on failure.
    // tags:
    //		private
    
    if (link.isSelfLoop()) {
      return (false); // Boolean
    }

    var iter;

    if (link.getStartPoints().length > 3 && link.getEndPoints().length > 3)
    {
      this.divideTerminationPointsBySide(link.getStartPoints());
      var startTopPoints = this._topPoints;
      var startBottomPoints = this._bottomPoints;
      var startLeftPoints = this._leftPoints;
      var startRightPoints = this._rightPoints;

      this.divideTerminationPointsBySide(link.getEndPoints());
      var endTopPoints = this._topPoints;
      var endBottomPoints = this._bottomPoints;
      var endLeftPoints = this._leftPoints;
      var endRightPoints = this._rightPoints;
      iter = new LLTerminationPointIterator(startTopPoints, endBottomPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      iter = new LLTerminationPointIterator(startBottomPoints, endTopPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      iter = new LLTerminationPointIterator(startLeftPoints, endRightPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      iter = new LLTerminationPointIterator(startRightPoints, endLeftPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      return (false); // Boolean
    }
    else
    {
      iter = new LLTerminationPointIterator(link.getStartPoints(), link.getEndPoints());
      return (this.iterateAndRouteBestStraight(iter, link)); // Boolean
    }
  },

  iterateAndRouteBestStraight : function(iter, link)
  {
    // summary:
    //		Iterates over all combination of start and end points, using the
    //		input iterator, and search the best straight route. 
    // description:
    //		Iterates over all combination of start and end points, using the
    //		input iterator, and search the best straight route. 
    //		It returns true on success and false on failure.
    // tags:
    //		private
    
    var routeFound = false;
    var bestStartCandIndex = 0;
    var bestEndCandIndex = 0;
    var bestCrossings = LLLinkRouteAlgorithm.MAXVAL;
    var penalty;
    var lastPenalty = LLLinkRouteAlgorithm.MAXVAL;
    var startPoint;
    var endPoint;
    var bestStartPoint = null;
    var bestEndPoint = null;
    var startPointIndex = 0;
    var endPointIndex = 0;
    var i;

    while (iter.hasNext())
    {
      startPoint = iter.getStartPoint();
      endPoint = iter.getEndPoint();

      penalty = Math.max(startPoint.getPenalty(), endPoint.getPenalty());
      if (penalty > lastPenalty) {
        break;
      }
      lastPenalty = penalty;

      if (this.searchStraightRoute(startPoint, endPoint))
      {
        for (i = 0; i < link.getStartPoints().length; i++)
        {
          if (link.getStartPoints()[i] == startPoint)
          {
            startPointIndex = i;
            break;
          }
        }

        for (i = 0; i < link.getEndPoints().length; i++)
        {
          if (link.getEndPoints()[i] == endPoint)
          {
            endPointIndex = i;
            break;
          }
        }

        if (this._crossingsInStraightOrOneBendRoute == 0)
        {
          link.setActStartAndEndCandidates(startPointIndex, endPointIndex);
          link.storeRoute(null);
          return (true); // Boolean
        }

        if (this._crossingsInStraightOrOneBendRoute < bestCrossings)
        {
          bestCrossings = this._crossingsInStraightOrOneBendRoute;
          bestStartCandIndex = startPointIndex;
          bestEndCandIndex = endPointIndex;
          bestStartPoint = startPoint;
          bestEndPoint = endPoint;
          routeFound = true;
        }
      }

      iter.next();
      if (new Date().getTime() > this._elapseTime) {
        break;
      }
    }

    if (routeFound)
    {
      this.searchStraightRoute(bestStartPoint, bestEndPoint);
      link.setActStartAndEndCandidates(bestStartCandIndex, bestEndCandIndex);
      link.storeRoute(null);
    }

    return (routeFound); // Boolean
  },

  routeBestOneBend : function(link)
  {
    // summary:
    //		Search the best one bend route. 
    // description:
    //		Search the best one bend route. 
    //		It returns true on success and false on failure.
    // tags:
    //		private
    
    if (link.isSelfLoop()) {
      return (false); // Boolean
    }
    var routeFound = false;
    var bestStartCandIndex = 0;
    var bestEndCandIndex = 0;
    var bestCrossings = LLLinkRouteAlgorithm.MAXVAL;
    var penalty;
    var lastPenalty = LLLinkRouteAlgorithm.MAXVAL;
    var iter = new LLTerminationPointIterator(link.getStartPoints(), link.getEndPoints());
    var startPoint;
    var endPoint;
    var bestStartPoint = null;
    var bestEndPoint = null;

    while (iter.hasNext())
    {
      startPoint = iter.getStartPoint();
      endPoint = iter.getEndPoint();

      penalty = Math.max(startPoint.getPenalty(), endPoint.getPenalty());
      if (penalty > lastPenalty) {
        break;
      }
      lastPenalty = penalty;

      if (this.searchOneBendRoute(startPoint, endPoint))
      {
        if (this._crossingsInStraightOrOneBendRoute == 0)
        {
          link.setActStartAndEndCandidates(iter.getStartPointIndex(), iter.getEndPointIndex());
          link.storeRoute(this.getPoints());
          return (true); // Boolean
        }

        if (this._crossingsInStraightOrOneBendRoute < bestCrossings)
        {
          bestCrossings = this._crossingsInStraightOrOneBendRoute;
          bestStartCandIndex = iter.getStartPointIndex();
          bestEndCandIndex = iter.getEndPointIndex();
          bestStartPoint = startPoint;
          bestEndPoint = endPoint;
          routeFound = true;
        }
      }

      iter.next();
      if (new Date().getTime() > this._elapseTime) {
        break;
      }
    }

    if (routeFound)
    {
      this.searchOneBendRoute(bestStartPoint, bestEndPoint);
      link.setActStartAndEndCandidates(bestStartCandIndex, bestEndCandIndex);
      link.storeRoute(this.getPoints());
    }

    return (routeFound); // Boolean
  },

  searchStraightRoute : function(startPoint, endPoint)
  {
    // summary:
    //		Routes a link from start point to end point straight. 
    // description:
    //		Routes a link from start point to end point straight. 
    //		It returns true on success and false on failure. It also stores the
    //		number ofr crossings with line obstacles for this route in
    //		&#95;crossingsInStraightOrOneBendRoute;
    // tags:
    //		private
    
    var startSegment = startPoint.getSegment();
    var endSegment = endPoint.getSegment();

    if (startSegment == endSegment && startSegment != null)
    {
      this._points.length = 0;

      var line = startSegment.getGridLine();
      this._crossingsInStraightOrOneBendRoute = line.calcApproxCrossings(this.getGrid(), startPoint.getIndex(((line.isHorizontal() ? 0 : 1))), endPoint.getIndex(((line.isHorizontal() ? 0 : 1))));
      return true; // Boolean
    }

    return (false); // Boolean
  },

  searchOneBendRoute : function(startPoint, endPoint)
  {
    // summary:
    //		Routes a link from start point to end point with just one bend. 
    // description:
    //		Routes a link from start point to end point with just one bend. 
    //		straight. It returns true on success and false on failure. It also
    //		stores the number ofr crossings with line obstacles for this route
    //		in &#95;crossingsInStraightOrOneBendRoute;
    // tags:
    //		private
    
    var startSegment = startPoint.getSegment();
    var endSegment = endPoint.getSegment();
    if (startSegment == null) {
      return (false); // Boolean
    }
    if (endSegment == null) {
      return (false); // Boolean
    }
    var startLine = startSegment.getGridLine();
    var endLine = endSegment.getGridLine();

    if (startLine.isHorizontal() == endLine.isHorizontal()) {
      return (false); // Boolean
    }
    var c1 = endLine.getIndex();
    var c2 = startLine.getIndex();

    if (startSegment.getStartIndex() > c1 || startSegment.getEndIndex() < c1) {
      return (false); // Boolean
    }
    if (endSegment.getStartIndex() > c2 || endSegment.getEndIndex() < c2) {
      return (false); // Boolean
    }
    if (startPoint.getForbiddenMinIndex() <= c1 && startPoint.getForbiddenMaxIndex() >= c1) {
      return (false); // Boolean
    }
    if (endPoint.getForbiddenMinIndex() <= c2 && endPoint.getForbiddenMaxIndex() >= c2) {
      return (false); // Boolean
    }
    this._points.length = 0;
    if (startSegment.isHorizontal()) {
      this._points.push(new Point(c1, c2));
    }
    else {
      this._points.push(new Point(c2, c1));
    }

    var c0 = startPoint.getIndex(((startLine.isHorizontal() ? 0 : 1)));
    this._crossingsInStraightOrOneBendRoute = startLine.calcApproxCrossings(this.getGrid(), c0, c1);
    c0 = endPoint.getIndex(((endLine.isHorizontal() ? 0 : 1)));
    this._crossingsInStraightOrOneBendRoute += endLine.calcApproxCrossings(this.getGrid(), c0, c2);
    return (true); // Boolean
  },

  divideTerminationPointsBySide : function(points)
  {
    // summary:
    //		Extracts the termination points from the array of termination points
    //		according to the side. 
    // description:
    //		Extracts the termination points from the array of termination points
    //		according to the side. 
    //		The result is in &#95;topPoints, &#95;bottomPoints, &#95;leftPoints and
    //		&#95;rightPoints.
    // tags:
    //		private
    
    var i;
    var nrTopPoints = 0;
    var nrBottomPoints = 0;
    var nrLeftPoints = 0;
    var nrRightPoints = 0;

    for (i = 0; i < points.length; i++)
    {
      switch (points[i].getSide())
      {
        case Direction.TOP:
          nrTopPoints++;
          break;
        case Direction.BOTTOM:
          nrBottomPoints++;
          break;
        case Direction.LEFT:
          nrLeftPoints++;
          break;
        case Direction.RIGHT:
          nrRightPoints++;
          break;
      }
    }

    this._topPoints = TranslateUtil.newArray(nrTopPoints, null);
    this._bottomPoints = TranslateUtil.newArray(nrBottomPoints, null);
    this._leftPoints = TranslateUtil.newArray(nrLeftPoints, null);
    this._rightPoints = TranslateUtil.newArray(nrRightPoints, null);
    nrTopPoints = 0;
    nrBottomPoints = 0;
    nrLeftPoints = 0;
    nrRightPoints = 0;

    for (i = 0; i < points.length; i++)
    {
      switch (points[i].getSide())
      {
        case Direction.TOP:
          this._topPoints[nrTopPoints++] = points[i];
          break;
        case Direction.BOTTOM:
          this._bottomPoints[nrBottomPoints++] = points[i];
          break;
        case Direction.LEFT:
          this._leftPoints[nrLeftPoints++] = points[i];
          break;
        case Direction.RIGHT:
          this._rightPoints[nrRightPoints++] = points[i];
          break;
      }
    }
  }

}
);

lang.mixin(LLLinkRouteAlgorithm,
{
  MAXVAL : null,

  _fields_init : function(c)
  {
    c = c || LLLinkRouteAlgorithm;
    
    c.MAXVAL = /*Number.MAX_VALUE*/2147483647;
  }
  
}
);

LLTerminationPointIterator = declare( null, 
{
  startPointCand: null,
  endPointCand: null,
  numberStartPointCands: null,
  numberEndPointCands: null,
  maxSPIndex: null,
  maxEPIndex: null,
  actSPIndex: null,
  actEPIndex: null,
  iterateAtStart: null,

  constructor : function(s, e)
  {
    this.startPointCand = s;
    this.endPointCand = e;
    this.numberStartPointCands = this.startPointCand.length;
    this.numberEndPointCands = this.endPointCand.length;
    this.maxSPIndex = 0;
    this.maxEPIndex = 0;
    this.actSPIndex = 0;
    this.actEPIndex = 0;
    this.iterateAtStart = true;
  },
  
  hasNext : function()
  {
    return (this.maxSPIndex < this.numberStartPointCands && this.maxEPIndex < this.numberEndPointCands); // Boolean
  },

  getStartPoint : function()
  {
    return (this.startPointCand[this.actSPIndex]); // ibm_ilog.graphlayout.longlink.LLTerminationPoint
  },

  getEndPoint : function()
  {
    return (this.endPointCand[this.actEPIndex]); // ibm_ilog.graphlayout.longlink.LLTerminationPoint
  },

  getStartPointIndex : function()
  {
    return (this.actSPIndex); // int
  },

  getEndPointIndex : function()
  {
    return (this.actEPIndex); // int
  },

  next : function()
  {
    if (this.iterateAtStart)
    {
      if (this.actSPIndex < this.maxSPIndex) {
        this.actSPIndex++;
      }
      else {
        this.incrementMaxStartOrEnd();
      }
    }
    else
    {
      if (this.actEPIndex < this.maxEPIndex) {
        this.actEPIndex++;
      }
      else {
        this.incrementMaxStartOrEnd();
      }
    }
  },

  incrementMaxStartOrEnd : function()
  {
    this.maxSPIndex++;
    this.maxEPIndex++;

    if (this.maxSPIndex < this.numberStartPointCands)
    {
      if (this.maxEPIndex < this.numberEndPointCands)
      {
        if (this.startPointCand[this.maxSPIndex].getPenalty() < this.endPointCand[this.maxEPIndex].getPenalty())
        {
          this.actSPIndex = this.maxSPIndex;
          this.iterateAtStart = false;
          this.maxEPIndex--;
          this.actEPIndex = 0;
        }
        else if (this.startPointCand[this.maxSPIndex].getPenalty() > this.endPointCand[this.maxEPIndex].getPenalty())
        {
          this.actEPIndex = this.maxEPIndex;
          this.iterateAtStart = true;
          this.maxSPIndex--;
          this.actSPIndex = 0;
        }
        else if (this.startPointCand[this.maxSPIndex].getOrder() < this.endPointCand[this.maxEPIndex].getOrder())
        {
          this.actSPIndex = this.maxSPIndex;
          this.iterateAtStart = false;
          this.maxEPIndex--;
          this.actEPIndex = 0;
        }
        else
        {
          this.actEPIndex = this.maxEPIndex;
          this.iterateAtStart = true;
          this.maxSPIndex--;
          this.actSPIndex = 0;
        }
      }
      else
      {
        this.actSPIndex = this.maxSPIndex;
        this.iterateAtStart = false;
        this.maxEPIndex--;
        this.actEPIndex = 0;
      }
    }
    else if (this.maxEPIndex < this.numberEndPointCands)
    {
      this.actEPIndex = this.maxEPIndex;
      this.iterateAtStart = true;
      this.maxSPIndex--;
      this.actSPIndex = 0;
    }
  }

}
);
(function(){ LLLinkRouteAlgorithm._fields_init(); })();

return LLLinkRouteAlgorithm;
});
