/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../longlink/LLTerminationPoint",	// #7
  "../Direction"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    LLTerminationPoint,	// #7
    Direction	// #8
    ) {

var LLTerminationCandAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLTerminationCandAlgorithm", null, 
{
  // summary:
  //		The algorithm to find the possible start and end points of a single
  //		link. 
  // description:
  //		The algorithm to find the possible start and end points of a single
  //		link. 
  //		If the start point is not fixed, it searches a field of candidates
  //		for start and end points along the border of the "from" and "to"
  //		node. Even if the start point is fixed, this algorithm is needed to
  //		calculate all the auxiliary information needed for the class
  //		LLTerminationPoint.  
  // tags:
  //		private
  
  _layout: null,

  _grid: null,

  _resPoints: null,
  _numberResPoints: null,

  _fallbackPoint: null,

  constructor : function(layout, grid)
  {
    this._layout = layout;
    this._grid = grid;
    this._resPoints = null;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._layout = null;
    this._grid = null;
    this._resPoints = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getGrid : function()
  {
    // summary:
    //		Returns the routing grid.
    // tags:
    //		private
    
    return (this._grid); // ibm_ilog.graphlayout.longlink.LLGrid
  },

  run : function(link)
  {
    // summary:
    //		Find all start and end point candidates for the input link. 
    // description:
    //		Find all start and end point candidates for the input link. 
    //		It stores the candidates in the link.  
    // tags:
    //		private
    
    var layout = this.getLayout();

    var fromPref = TranslateUtil.newArray(4, 0);
    var toPref = TranslateUtil.newArray(4, 0);
    var i;

    for (i = 0; i < 4; i++)
      fromPref[i] = toPref[i] = 0;
    var fromRect = link.getFromRect();
    var toRect = link.getToRect();
    var fromPoint = link.getFromPoint();
    var toPoint = link.getToPoint();

    if ((fromPoint == null || toPoint == null) && !link.isSelfLoop()) {
      this.calcSidePreferences(fromRect, toRect, fromPref, toPref);
    }

    var minLen = layout.getMinStartSegmentLength();
    this._fallbackPoint = null;
    if (fromPoint == null) {
      this.generateTerminationPoints(fromRect, fromPref, link, link.getFromNode(), minLen, true);
    }
    else {
      this.searchSingleTerminationPoint(fromPoint.x, fromPoint.y, fromRect, minLen);
    }
    link.setStartPoints(this._resPoints, this._numberResPoints);

    minLen = layout.getMinEndSegmentLength();
    if (toPoint == null) {
      this.generateTerminationPoints(toRect, toPref, link, link.getToNode(), minLen, false);
    }
    else {
      this.searchSingleTerminationPoint(toPoint.x, toPoint.y, toRect, minLen);
    }
    link.setEndPoints(this._resPoints, this._numberResPoints);
  },

  generateTerminationPoints : function(terminationRect, sidePref, link, node, minLen, origin)
  {
    // summary:
    //		Generate all potential termination points if no fixed start/end
    //		point is given. 
    // description:
    //		Generate all potential termination points if no fixed start/end
    //		point is given. 
    //		The result is in the array &#95;resPoint, while &#95;numberResPoints is the
    //		first free index in this array (which means accordingly, the number
    //		of points in the array). The array is ordered according to penalties
    //		(and if the penalty is equal, according to the
    //		point's internal order).
    // tags:
    //		private
    
    this._fallbackPoint = null;

    var minCOffs = this.getLayout().getMinNodeCornerOffset();
    var grid = this.getGrid();
    var startHIndex = grid.getMinGridIndexAtSide(terminationRect, 0, minCOffs);
    var endHIndex = grid.getMaxGridIndexAtSide(terminationRect, 0, minCOffs);
    var numberOfHPoints = Math.max(endHIndex - startHIndex + 1, 0);
    var startVIndex = grid.getMinGridIndexAtSide(terminationRect, 1, minCOffs);
    var endVIndex = grid.getMaxGridIndexAtSide(terminationRect, 1, minCOffs);
    var numberOfVPoints = Math.max(endVIndex - startVIndex + 1, 0);

    this._resPoints = TranslateUtil.newArray(2 * numberOfHPoints + 2 * numberOfVPoints, null);
    this._numberResPoints = 0;

    this.searchTerminationPoints(terminationRect.y, -minLen, Direction.TOP, startHIndex, numberOfHPoints, sidePref[0], link, node, origin);

    this.searchTerminationPoints(terminationRect.y + terminationRect.height, minLen, Direction.BOTTOM, startHIndex, numberOfHPoints, sidePref[2], link, node, origin);

    this.searchTerminationPoints(terminationRect.x, -minLen, Direction.LEFT, startVIndex, numberOfVPoints, sidePref[1], link, node, origin);

    this.searchTerminationPoints(terminationRect.x + terminationRect.width, minLen, Direction.RIGHT, startVIndex, numberOfVPoints, sidePref[3], link, node, origin);

    if (this._numberResPoints > 1)
    {
      this._resPoints[0].sortAccordingToPenalty(this._resPoints);
    }

    if (this._numberResPoints == 0 && this._fallbackPoint != null) {
      this._resPoints[this._numberResPoints++] = this._fallbackPoint;
    }
  },

  searchTerminationPoints : function(terminationCoord, minLen, side, minGridIndex, numberOfGridLines, sidePreference, link, node, origin)
  {
    // summary:
    //		Search the termination points for a routing at a given side when the
    //		start/end points are not fixed. 
    // description:
    //		Search the termination points for a routing at a given side when the
    //		start/end points are not fixed. 
    //		The result is in the array &#95;resPoint where &#95;numberResPoints points
    //		to the first free index. Note that the array must be allocated
    //		before. The side is Direction (Left, Top, Right, Bottom). The object
    //		where the link terminates covers the grid from grid line
    //		"minGridIndex" to "minGridIndex + numberOfGridLines". The effective
    //		end point of the link is at terminationCoord. Side preference is a
    //		value between 0 and 4 to indicate how favourable this side is. This
    //		is needed to calculate a good order for the termination points. The
    //		flag "origin" is true if we search start points of the link, and
    //		false if we search end points of the link. termination Coord |      
    //		        side = Right V         | minGridIndex -> ,------.         |
    //		| node |&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;| link |      | mGI + nOGL   ->
    //		`------'^        ^ \      / minLen
    // tags:
    //		private
    
    if (numberOfGridLines <= 0) {
      return;
    }

    var larger;
    var dir;

    switch (side)
    {
      case Direction.RIGHT:
        dir = 0;
        larger = true;
        break;
      case Direction.LEFT:
        dir = 0;
        larger = false;
        break;
      case Direction.BOTTOM:
        dir = 1;
        larger = true;
        break;
      case Direction.TOP:
      default:
        dir = 1;
        larger = false;
        break;
    }

    var grid = this.getGrid();
    var terminationIndex = grid.getGridIndexAtMinDist(dir, terminationCoord, larger);
    var terminationIndexAtMinLen = grid.getGridIndex(dir, terminationCoord + minLen, larger);
    var nrGridLines0 = grid.getNumberOfGridLines(dir);
    var nrGridLines1 = grid.getNumberOfGridLines(1 - dir);

    if (larger)
    {
      if (terminationIndexAtMinLen < terminationIndex) {
        terminationIndexAtMinLen = terminationIndex;
      }
    }
    else
    {
      if (terminationIndexAtMinLen > terminationIndex) {
        terminationIndexAtMinLen = terminationIndex;
      }
    }

    if (terminationIndexAtMinLen < 0) {
      return;
    }
    if (terminationIndexAtMinLen >= nrGridLines0) {
      return;
    }

    var i;
    var index0;
    var index1;
    var incr = 0;
    var order = sidePreference * this._resPoints.length;
    var forbiddenMinIndex;
    var forbiddenMaxIndex;
    var penalty = 0;
    var overlapPenalty = terminationIndexAtMinLen - terminationIndex;
    if (overlapPenalty < 0) {
      overlapPenalty = -overlapPenalty;
    }
    var mid = minGridIndex + Math.floor((numberOfGridLines - 1) / 2);
    var segment;

    for (i = 0; i < numberOfGridLines; i++)
    {
      index1 = mid + incr;

      if (index1 < 0 || index1 >= nrGridLines1)
      {
        if (incr <= 0) {
          incr = -incr + 1;
        }
        else {
          incr = -incr;
        }
        continue;
      }

      penalty = 0;
      segment = grid.getGridLine(1 - dir, index1).searchSegment(terminationIndex);

      if (segment == null) {
        penalty = overlapPenalty;
      }
      else if (segment.getStartIndex() > terminationIndexAtMinLen || segment.getEndIndex() < terminationIndexAtMinLen)
      {
        penalty = overlapPenalty;
        if (larger) {
          penalty -= (segment.getEndIndex() - terminationIndex);
        }
        else {
          penalty -= (terminationIndex - segment.getStartIndex());
        }
        if (penalty <= 0) {
          penalty = 1;
        }
      }

      segment = grid.getGridLine(1 - dir, index1).searchClosestSegment(terminationIndexAtMinLen, larger);

      if (segment != null)
      {
        forbiddenMinIndex = -1;
        forbiddenMaxIndex = -1;

        if (segment.getStartIndex() > terminationIndexAtMinLen)
        {
          index0 = segment.getStartIndex();
          penalty += index0 - terminationIndexAtMinLen;
        }
        else if (segment.getEndIndex() < terminationIndexAtMinLen)
        {
          index0 = segment.getEndIndex();
          penalty += terminationIndexAtMinLen - index0;
        }
        else
        {
          index0 = terminationIndexAtMinLen;

          if (larger)
          {
            forbiddenMinIndex = terminationIndex;
            forbiddenMaxIndex = terminationIndexAtMinLen - 1;
            if (forbiddenMaxIndex < forbiddenMinIndex) {
              forbiddenMinIndex = forbiddenMaxIndex;
            }
          }
          else
          {
            forbiddenMinIndex = terminationIndexAtMinLen + 1;
            forbiddenMaxIndex = terminationIndex;
            if (forbiddenMaxIndex < forbiddenMinIndex) {
              forbiddenMaxIndex = forbiddenMinIndex;
            }
          }
        }

        penalty =
          this.getLayout().getTerminationPointPenalty(link.getLinkObject(), node, origin, grid.getGridCoord(0, ((dir == 0 ? index0 : index1))), grid.getGridCoord(1, ((dir == 1 ? index0 : index1))),
              side, penalty);

        if (penalty == LLTerminationCandAlgorithm.MAXVAL)
        {
          if (this._fallbackPoint == null) {
            this._fallbackPoint = new LLTerminationPoint(index0, index1, forbiddenMinIndex, forbiddenMaxIndex, dir, 0, order, side, segment);
          }
        }
        else
        {
          this._resPoints[this._numberResPoints++] = new LLTerminationPoint(index0, index1, forbiddenMinIndex, forbiddenMaxIndex, dir, penalty, order, side, segment);
        }
      }

      if (incr <= 0) {
        incr = -incr + 1;
      }
      else {
        incr = -incr;
      }
      order++;
    }
  },

  searchSingleTerminationPoint : function(x, y, terminationRect, minSegDist)
  {
    // summary:
    //		Search a single appropriate termination point if the fixed
    //		coordinates (x, y) are given as start/end point. 
    // description:
    //		Search a single appropriate termination point if the fixed
    //		coordinates (x, y) are given as start/end point. 
    //		Since (x, y) may not be a grid point, and the first/last segment
    //		needs length "minSegDist", we need a grid point that is minSegDist
    //		from (x, y) away. The result is in the array &#95;resPoint where
    //		&#95;numberResPoints points to the first free index. Different to
    //		searchTerminationPoints, the array must NOT be allocated before,
    //		because we know it is just one point.
    // tags:
    //		private
    
    var grid = this.getGrid();
    var xindex;
    var yindex;
    var left = terminationRect.x;
    var right = terminationRect.x + terminationRect.width;
    var top = terminationRect.y;
    var bottom = terminationRect.y + terminationRect.height;

    var bestXIndex = 0;
    var bestYIndex = 0;
    var bestForbiddenMinIndex = -1;
    var bestForbiddenMaxIndex = -1;

    var bestSide = Direction.LEFT;
    var testIndex;
    var testForbMinIndex;
    var testForbMaxIndex;

    var testSide;
    var bestDistToBorder = LLTerminationCandAlgorithm.FMAXVAL;
    var bestDistToSeg = LLTerminationCandAlgorithm.FMAXVAL;
    var h1;
    var h2;
    var bestSegment = null;
    var seg;
    var needOrthDummySeg = false;

    testSide = Direction.LEFT;
    xindex = grid.getGridIndex(0, x - minSegDist, false);
    yindex = grid.getClosestGridIndex(1, y);

    if (x <= right)
    {
      h1 = left - x;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (y < top || bottom < y) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(xindex, yindex, 0, false);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (!seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > xindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < xindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = xindex;
        testForbMinIndex = xindex + 1;
        testForbMaxIndex = grid.getGridIndex(0, x, false);
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMaxIndex = testForbMinIndex;
        }
      }

      h2 = grid.getGridCoord(0, testIndex) - (x - minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = testIndex;
        bestYIndex = yindex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = !seg.isHorizontal();
      }
    }

    testSide = Direction.RIGHT;
    xindex = grid.getGridIndex(0, x + minSegDist, true);

    if (x >= left)
    {
      h1 = right - x;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (y < top || bottom < y) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(xindex, yindex, 0, true);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (!seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > xindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < xindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = xindex;
        testForbMinIndex = grid.getGridIndex(0, x, true);
        testForbMaxIndex = xindex - 1;
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMinIndex = testForbMaxIndex;
        }
      }

      h2 = grid.getGridCoord(0, testIndex) - (x + minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = testIndex;
        bestYIndex = yindex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = !seg.isHorizontal();
      }
    }

    testSide = Direction.TOP;
    xindex = grid.getClosestGridIndex(0, x);
    yindex = grid.getGridIndex(1, y - minSegDist, false);

    if (y <= bottom)
    {
      h1 = top - y;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (x < left || right < x) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(yindex, xindex, 1, false);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > yindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < yindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = yindex;
        testForbMinIndex = yindex + 1;
        testForbMaxIndex = grid.getGridIndex(1, y, false);
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMaxIndex = testForbMinIndex;
        }
      }

      h2 = grid.getGridCoord(1, testIndex) - (y - minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = xindex;
        bestYIndex = testIndex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = seg.isHorizontal();
      }
    }

    testSide = Direction.BOTTOM;
    yindex = grid.getGridIndex(1, y + minSegDist, true);

    if (y >= top)
    {
      h1 = bottom - y;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (x < left || right < x) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(yindex, xindex, 1, true);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > yindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < yindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = yindex;
        testForbMinIndex = grid.getGridIndex(1, y, true);
        testForbMaxIndex = yindex - 1;
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMinIndex = testForbMaxIndex;
        }
      }

      h2 = grid.getGridCoord(1, testIndex) - (y + minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = xindex;
        bestYIndex = testIndex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = seg.isHorizontal();
      }
    }

    if (bestSegment != null)
    {
      if (needOrthDummySeg)
      {
        var line;

        if (bestSegment.isHorizontal())
        {
          line = grid.getGridLine(0, bestXIndex);

          line.addSegment(bestYIndex, bestYIndex);
          bestSegment = line.searchSegment(bestYIndex);
        }
        else
        {
          line = grid.getGridLine(1, bestYIndex);

          line.addSegment(bestXIndex, bestXIndex);
          bestSegment = line.searchSegment(bestXIndex);
        }
      }

      this._resPoints = TranslateUtil.newArray(1, null);
      this._resPoints[0] = new LLTerminationPoint(bestXIndex, bestYIndex, bestForbiddenMinIndex, bestForbiddenMaxIndex, 0, 0, 0, bestSide, bestSegment);
      this._numberResPoints = 1;
    }
    else
    {
      this._numberResPoints = 0;
    }
  },

  calcSidePreferences : function(startRect, endRect, startPref, endPref)
  {
    // summary:
    //		Calculate the side preferences. 
    // description:
    //		Calculate the side preferences. 
    //		The result is in the arrays startPref[4] and endPref[4]. The sides
    //		are numbered: top = 0, left = 1, bottom = 2, right = 3.
    // tags:
    //		private
    
    var layout = this.getLayout();
    var minStartLen = layout.getMinStartSegmentLength();
    var minEndLen = layout.getMinEndSegmentLength();
    LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.x - minStartLen, startRect.x + startRect.width + minStartLen, endRect.x - minEndLen, endRect.x + endRect.width + minEndLen, 1, 3,
        startPref, endPref);
    LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.y - minStartLen, startRect.y + startRect.height + minStartLen, endRect.y - minEndLen, endRect.y + endRect.height + minEndLen, 0, 2,
        startPref, endPref);
  }

}
);

lang.mixin(LLTerminationCandAlgorithm,
{
  MAXVAL : null,
  FMAXVAL : null,

  calcSidePrefInternal : function(startMin, startMax, endMin, endMax, lowerIndex, upperIndex, startPref, endPref)
  {
    var startMid = 0.5 * (startMin + startMax);
    var endMid = 0.5 * (endMin + endMax);

    if (startMid <= endMid)
    {
      if (startMin >= endMin)
      {
        startPref[lowerIndex] = 3;
        startPref[upperIndex] = 2;
        endPref[lowerIndex] = 3;
        endPref[upperIndex] = 2;
      }
      else if (startMax >= endMax)
      {
        startPref[lowerIndex] = 2;
        startPref[upperIndex] = 3;
        endPref[lowerIndex] = 2;
        endPref[upperIndex] = 3;
      }
      else if (startMax >= endMin)
      {
        startPref[lowerIndex] = 3;
        startPref[upperIndex] = 2;
        endPref[lowerIndex] = 2;
        endPref[upperIndex] = 3;
      }
      else
      {
        startPref[lowerIndex] = 4;
        startPref[upperIndex] = 1;
        endPref[lowerIndex] = 1;
        endPref[upperIndex] = 4;
      }
    }
    else
    {
      if (endMin >= startMin)
      {
        startPref[lowerIndex] = 3;
        startPref[upperIndex] = 2;
        endPref[lowerIndex] = 3;
        endPref[upperIndex] = 2;
      }
      else if (endMax >= startMax)
      {
        startPref[lowerIndex] = 2;
        startPref[upperIndex] = 3;
        endPref[lowerIndex] = 2;
        endPref[upperIndex] = 3;
      }
      else if (endMax >= startMin)
      {
        startPref[lowerIndex] = 2;
        startPref[upperIndex] = 3;
        endPref[lowerIndex] = 3;
        endPref[upperIndex] = 2;
      }
      else
      {
        startPref[lowerIndex] = 1;
        startPref[upperIndex] = 4;
        endPref[lowerIndex] = 4;
        endPref[upperIndex] = 1;
      }
    }
  },

  _fields_init : function(c)
  {
    c = c || LLTerminationCandAlgorithm;
    
    c.MAXVAL = /*Number.MAX_VALUE*/2147483647;
    c.FMAXVAL = /*Number.MAX_VALUE*/(3.4028235E38);
  }
  
}
);

(function(){ LLTerminationCandAlgorithm._fields_init(); })();

return LLTerminationCandAlgorithm;
});
