/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/LogResUtil",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/TranslateUtil",	// #9
  "../shortlink/_SLBundlesModes"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    LogResUtil,	// #7
    Rectangle,	// #8
    TranslateUtil,	// #9
    _SLBundlesModes	// #10
    ) {

var _CostComputer = declare("ibm_ilog.graphlayout.shortlink._CostComputer", null, 
{
  // summary:
  //		This class is in charge of the computation of the cost of each
  //		solution. 
  // description:
  //		This class is in charge of the computation of the cost of each
  //		solution. 
  //		
  // tags:
  //		private
  
  _linkLinkPenalty: null,
  _linkNodePenalty: null,

  _linkBundlesMode: null,
  _tempRect1 : null,
  _tempRect2 : null,
  _tempRect3 : null,

  constructor : function()
  {
    this._tempRect1 = new Rectangle();
    this._tempRect2 = new Rectangle();
    this._tempRect3 = new Rectangle();
  },
  
  clean : function()
  {
  },

  update : function(layout)
  {
    // summary:
    //		Updates parameters from the current values on the layout.
    // tags:
    //		private
    
    var shortLayout = layout.getShortLinkLayout();
    this._linkLinkPenalty = shortLayout.getLinkToLinkCrossingPenalty();
    this._linkNodePenalty = shortLayout.getLinkToNodeCrossingPenalty();
    this._linkBundlesMode = shortLayout.getLinkBundlesMode();
  },

  getLinkCost : function (a0, a1, a2, a3, a4)
  {
    // summary:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    // description:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    //		Only nodes and links inside the link region are checked.  
    // global: Boolean
    //		true means that the method is called for every link and we
    //		are interested in the total number of crossings (cross betw two
    //		links is counted just once). If false, all the cossings for this
    //		link are counted. PACKAGE BECAUSE CALLED FROM ShortLinkAlgorithm
    // tags:
    //		private
    
    if(arguments.length == 4)
      return this.getLinkCost_overload_4(a0, a1, a2, a3);
    else if(arguments.length == 5)
      return this.getLinkCost_overload_5(a0, a1, a2, a3, a4);
  },
  
  getLinkCost_overload_4 : function(graphModel, interGraphLinksRoutingModel, linkData, global)
                                                                                              
  {
    return this.getLinkCost(graphModel, interGraphLinksRoutingModel, linkData, false, global); // Number
  },

  getLinkCost_overload_5 : function(graphModel, interGraphLinksRoutingModel, linkData, onlyLinkNodeOverlaps, global)
  {
    // summary:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    // description:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    //		Only nodes and links inside the link region are checked.  
    // global: Boolean
    //		true means that the method is called for every link and we
    //		are interested in the total number of crossings (cross betw two
    //		links is counted just once). If false, all the crossings for this
    //		link are counted. PACKAGE BECAUSE CALLED FROM ShortLinkAlgorithm
    // tags:
    //		private
    
    if (linkData == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6601E");
    }

    var crossNode = 0;
    var crossLink = 0;

    crossNode += (this.getLinkAdjacentNodeCost(linkData, interGraphLinksRoutingModel, true) + this.getLinkAdjacentNodeCost(linkData, interGraphLinksRoutingModel, false));

    var intersectingObjects = linkData.getIntersectingObjects();
    var nObjects = (intersectingObjects == null ? 0 : intersectingObjects.length);

    if (nObjects > 0)
    {
      var index = linkData.getIndex();
      var nodeOrLinkData;
      var indexOtherLink;

      for (var i = 0; i < nObjects; i++)
      {
        nodeOrLinkData = intersectingObjects[i];
        indexOtherLink = 0;

        if (nodeOrLinkData.isNode()) {
          crossNode += this.getLinkNodeCost(interGraphLinksRoutingModel, linkData, nodeOrLinkData);
        }
        else if (!onlyLinkNodeOverlaps)
        {
          indexOtherLink = (nodeOrLinkData).getIndex();

          if (!global || (index < indexOtherLink)) {
            crossLink += this.getLinkLinkCost(linkData, nodeOrLinkData);
          }
        }
      }
    }

    var cost = crossLink * this._linkLinkPenalty + crossNode * this._linkNodePenalty;

    if (cost != 0 && !onlyLinkNodeOverlaps)
    {
      var nSlaves = linkData.getNumberOfIndividualLinks();
      if (nSlaves > 1) {
        cost *= nSlaves;
      }
    }

    return cost; // Number
  },

  getLinkAdjacentNodeCost : function(linkData, interGraphLinksRoutingModel, origin)
  {
    // summary:
    //		Computes the cost of the crossings between linkData and its adjacent
    //		node on the origin/dest nodes.
    // tags:
    //		private
    
    var nodeData;

    if (interGraphLinksRoutingModel != null)
    {
      nodeData = linkData.getConnectionNode(origin);

      if (
        linkData.isInterGraphLink() &&
          (interGraphLinksRoutingModel.fromEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink) || interGraphLinksRoutingModel.toEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink)))
        {
          if ((origin ? linkData.isQuasiSelfInterGraphLinkOrigin() : linkData.isQuasiSelfInterGraphLinkDestination()))
          {
            TranslateUtil.reshapeFromRect(this._tempRect1, linkData.getLinkShape().boundingBox());

            return (TranslateUtil.rectContainsRect(linkData.getConnectionNode(origin).boundingBox, this._tempRect1) ? 0 : 1); // Number
          }
        }
    }

    var crossNode = 0;
    var nodeSide = linkData.getNodeSide(origin);
    var point0 = linkData.getPointAt(origin, 0);
    var point1 = linkData.getPointAt(origin, 1);

    if (nodeSide.getPointRadialDelta(point0, point1) < 0)
    {
      crossNode++;
    }
    else
    {
      nodeData = linkData.getConnectionNode(origin);
      var oppositeNodeData = linkData.getConnectionNode(!origin);
      if (linkData.isOverlapping() && (TranslateUtil.rectContainsRect(oppositeNodeData.boundingBox, nodeData.boundingBox) || TranslateUtil.rectContainsRect(nodeData.boundingBox,
            oppositeNodeData.boundingBox))) {
        return 0; // Number
      }
      var point2 = linkData.getPointAt(origin, 2);

      if (linkData.getCurrentShapeType().isOrthogonal())
      {
        var nBends = linkData.getCurrentShapeType().getNumberOfBends();

        if (linkData.isOverlapping() && nBends >= 1)
        {
          if (this.intersects(point0, point1, 0, oppositeNodeData.boundingBox, false, false)) {
            crossNode++;
          }
        }

        if (nBends >= 2)
        {
          var point3 = linkData.getPointAt(origin, 3);
          if (this.intersects(point2, point3, 0, nodeData.boundingBox, false, false)) {
            crossNode++;
          }
        }
      }
      else
      {
        if (this.intersects(point1, point2, 0, nodeData.boundingBox, false, false)) {
          crossNode++;
        }
        if (nodeSide.getPointRadialDelta(point1, point2) < 0)
        {
          crossNode += _CostComputer.SMALL_PENALTY;
        }
      }
    }

    return crossNode; // Number
  },

  getLinkNodeCost : function(interGraphLinksRoutingModel, linkData, nodeData)
  {
    // summary:
    //		Returns the cost associated to the crossings between the link
    //		'link' and the node 'node'. 
    // description:
    //		Returns the cost associated to the crossings between the link
    //		<code>link</code> and the node <code>node</code>. 
    //		The cost is 0 if there are no crossings. Note that the link is
    //		checked for intersection with the link connection rectangle of the
    //		node.  
    //		<p>
    //		Note that the cost is multiplied by the number of slave links in
    //		getLinkCost.
    //		</p>
    // tags:
    //		private
    
    var msg;

    if (linkData == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6601E");
    }

    if (interGraphLinksRoutingModel != null)
    {
      if (
        linkData.isInterGraphLink() &&
          (interGraphLinksRoutingModel.fromEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink) || interGraphLinksRoutingModel.toEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink))) {
          return 0.0; // Number
      }
    }

    var shape = linkData.getLinkShape();

    if (shape == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6602E");
    }

    var bbox = nodeData.boundingBox;
    var halfWidth = linkData.getLinkWidth() * 0.5;
    TranslateUtil.reshapeFromRect(this._tempRect1, shape.boundingBox());
    if (halfWidth > 0) {
      TranslateUtil.inflateRect(this._tempRect1, halfWidth, halfWidth);
    }
    if (!_CostComputer.isOverlap(this._tempRect1, bbox)) {
      return 0; // Number
    }

    var limit = shape.getNumberOfPoints() - 2;

    for (var i = 0; i <= limit; i++)
    {
      if (this.intersects(shape.getPointAt(i), shape.getPointAt(i + 1), halfWidth, bbox, i == 0, i == limit)) {
        return 1; // Number
      }
    }

    return 0; // Number
  },

  getLinkLinkCost : function(linkData1, linkData2)
  {
    // summary:
    //		Computes the cost associated with eventual crossings betw. 
    // description:
    //		Computes the cost associated with eventual crossings betw. 
    //		2 links.
    // tags:
    //		private
    
    var shape1 = linkData1.getLinkShape();
    var shape2 = linkData2.getLinkShape();
    var halfWidth1 = linkData1.getLinkWidth() * 0.5;
    var halfWidth2 = linkData2.getLinkWidth() * 0.5;
    TranslateUtil.reshapeFromRect(this._tempRect1, shape1.boundingBox());
    TranslateUtil.reshapeFromRect(this._tempRect2, shape2.boundingBox());
    TranslateUtil.inflateRect(this._tempRect1, halfWidth1, halfWidth1);
    TranslateUtil.inflateRect(this._tempRect2, halfWidth2, halfWidth2);
    if (!_CostComputer.isOverlap(this._tempRect1, this._tempRect2)) {
      return 0; // Number
    }

    var limit1 = shape1.getNumberOfPoints() - 2;
    var limit2 = shape2.getNumberOfPoints() - 2;
    var from1;
    var to1;
    var from2;
    var to2;
    var bothLinksOrthogonal = linkData1.isOrthogonal() && linkData2.isOrthogonal();
    var cross = 0;
    var startIndex1 = 0;
    var startIndex2 = 0;

    if (this._linkBundlesMode == _SLBundlesModes.NO_BUNDLE)
    {
      var originPointOnSameNodeSide = linkData1.getFromNode() == linkData2.getFromNode() && linkData1.getFromNodeSide() == linkData2.getFromNodeSide() &&
        !((linkData1.isFromPointFixed() || linkData2.isFromPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(0), shape2.getPointAt(0)));

      var destinationPointOnSameNodeSide = linkData1.getToNode() == linkData2.getToNode() && linkData1.getToNodeSide() == linkData2.getToNodeSide() &&
        !((linkData1.isToPointFixed() || linkData2.isToPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(limit1 + 1), shape2.getPointAt(limit2 + 1)));

      var originPoint1OnSameNodeSideAsDestinationPoint2 = !originPointOnSameNodeSide && linkData1.getFromNode() == linkData2.getToNode() && linkData1.getFromNodeSide() == linkData2.getToNodeSide() &&
        !((linkData1.isFromPointFixed() || linkData2.isToPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(0), shape2.getPointAt(limit2 + 1)));

      var destinationPoint1OnSameNodeSideAsOriginPoint2 = !destinationPointOnSameNodeSide && linkData1.getToNode() == linkData2.getFromNode() &&
        linkData1.getToNodeSide() == linkData2.getFromNodeSide() && !((linkData1.isToPointFixed() || linkData2.isFromPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(limit1 + 1)
          , shape2.getPointAt(0)));

      if (
        !
          ((originPointOnSameNodeSide && destinationPointOnSameNodeSide) ||
              (originPoint1OnSameNodeSideAsDestinationPoint2 && destinationPoint1OnSameNodeSideAsOriginPoint2) && (limit1 <= 1 && limit2 <= 1)))
        {
          if (originPointOnSameNodeSide)
          {
            startIndex1 = 1;
            startIndex2 = 1;
          }
          else if (originPoint1OnSameNodeSideAsDestinationPoint2)
          {
            startIndex1 = 1;
            limit2--;
          }

          if (destinationPointOnSameNodeSide)
          {
            limit1--;
            limit2--;
          }
          else if (destinationPoint1OnSameNodeSideAsOriginPoint2)
          {
            limit1--;
            startIndex2 = 1;
          }
        }
    }

    for (var i = startIndex1; i <= limit1 && cross == 0; i++)
    {
      from1 = shape1.getPointAt(i);
      to1 = shape1.getPointAt(i + 1);

      this.computeBounds(this._tempRect1, from1, to1, i == 0, i == limit1, halfWidth1);

      if (_CostComputer.isOverlap(this._tempRect1, this._tempRect2))
      {
        for (var j = startIndex2; j <= limit2 && cross == 0; j++)
        {
          from2 = shape2.getPointAt(j);
          to2 = shape2.getPointAt(j + 1);

          if (bothLinksOrthogonal)
          {
            this.computeBounds(this._tempRect3, from2, to2, j == 0, j == limit2, halfWidth2);
            if (_CostComputer.isOverlap(this._tempRect1, this._tempRect3)) {
              cross++;
            }
          }
          else if (LayoutUtil.intersects(from1.x, from1.y, to1.x, to1.y, from2.x, from2.y, to2.x, to2.y)) {
            cross++;
          }
        }
      }
    }

    return cross; // Number
  },

  computeBounds : function(rect, fromPoint, toPoint, isFromPointConnectionPoint, isToPointConnectionPoint, halfLineWidth)
  {
    // summary:
    //		Stores in 'rect' the bounds of a thick segment.
    // tags:
    //		private
    
    var width = toPoint.x - fromPoint.x;
    var height = toPoint.y - fromPoint.y;

    if (!isFromPointConnectionPoint && !isToPointConnectionPoint)
    {
      TranslateUtil.reshapeRect(rect, (fromPoint.x <= toPoint.x ? fromPoint.x : toPoint.x), (fromPoint.y <= toPoint.y ? fromPoint.y : toPoint.y),
        ((width >= 0 ? width : -width)), ((height >= 0 ? height : -height)));

      TranslateUtil.inflateRect(rect, halfLineWidth, halfLineWidth);
    }
    else
    {
      var xMin;
      var xMax;
      var yMin;
      var yMax;
      var horizontalSegment = ((height < 0 ? -height : height)) < ((width < 0 ? -width : width));

      if (width > 0)
      {
        xMin = fromPoint.x;
        xMax = toPoint.x;
        if (!horizontalSegment || !isFromPointConnectionPoint) {
          xMin -= halfLineWidth;
        }
        if (!horizontalSegment || !isToPointConnectionPoint) {
          xMax += halfLineWidth;
        }
      }
      else
      {
        xMin = toPoint.x;
        xMax = fromPoint.x;
        if (!horizontalSegment || !isToPointConnectionPoint) {
          xMin -= halfLineWidth;
        }
        if (!horizontalSegment || !isFromPointConnectionPoint) {
          xMax += halfLineWidth;
        }
      }

      var verticalSegment = !horizontalSegment;

      if (height > 0)
      {
        yMin = fromPoint.y;
        yMax = toPoint.y;
        if (!verticalSegment || !isFromPointConnectionPoint) {
          yMin -= halfLineWidth;
        }
        if (!verticalSegment || !isToPointConnectionPoint) {
          yMax += halfLineWidth;
        }
      }
      else
      {
        yMin = toPoint.y;
        yMax = fromPoint.y;
        if (!verticalSegment || !isToPointConnectionPoint) {
          yMin -= halfLineWidth;
        }
        if (!verticalSegment || !isFromPointConnectionPoint) {
          yMax += halfLineWidth;
        }
      }

      TranslateUtil.reshapeRect(rect, xMin, yMin, xMax - xMin, yMax - yMin);
    }
  },

  intersects : function(from, toobj, halfWidth, rect, firstPoint, lastPoint)
  {
    // summary:
    //		Checks whether Computes the number of crossings between the segment
    //		from-to and rect.
    // tags:
    //		private
    
    if (from.x != toobj.x && from.y != toobj.y)
    {
      var xMaxRect = rect.x + rect.width;
      var yMaxRect = rect.y + rect.height;

      if (from.x < rect.x && toobj.x < rect.x || from.x > xMaxRect && toobj.x > xMaxRect || from.y < rect.y && toobj.y < rect.y || from.y > yMaxRect && toobj.y > yMaxRect) {
        return false; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, rect.x, rect.y, xMaxRect, rect.y)) {
        return true; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, xMaxRect, rect.y, xMaxRect, yMaxRect)) {
        return true; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, xMaxRect, yMaxRect, rect.x, yMaxRect)) {
        return true; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, rect.x, yMaxRect, rect.x, rect.y)) {
        return true; // Boolean
      }
    }
    else
    {
      var width = toobj.x - from.x;
      var height = toobj.y - from.y;
      if (width == 0 && height == 0) {
        return false; // Boolean
      }
      if (width < 0) {
        width = -width;
      }
      if (height < 0) {
        height = -height;
      }
      if (halfWidth > 0)
      {
        if (firstPoint)
        {
          if (width == 0)
          {
            if (from.y <= toobj.y) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, from.y, 2 * halfWidth, height + halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, toobj.y - halfWidth, 2 * halfWidth, height + halfWidth);
            }
          }
          else
          {
            if (from.x <= toobj.x) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x, from.y - halfWidth, width + halfWidth, 2 * halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, toobj.x - halfWidth, toobj.y - halfWidth, width + halfWidth, 2 * halfWidth);
            }
          }
        }
        else if (lastPoint)
        {
          if (width == 0)
          {
            if (from.y <= toobj.y) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, from.y + halfWidth, 2 * halfWidth, height - halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, toobj.y, 2 * halfWidth, height - halfWidth);
            }
          }
          else
          {
            if (from.x <= toobj.x) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x + halfWidth, from.y - halfWidth, width - halfWidth, 2 * halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, toobj.x, toobj.y - halfWidth, width - halfWidth, 2 * halfWidth);
            }
          }
        }
        else
        {
          if (width == 0)
          {
            if (from.y <= toobj.y) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, from.y + halfWidth, 2 * halfWidth, height);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, toobj.y - halfWidth, 2 * halfWidth, height);
            }
          }
          else
          {
            if (from.x <= toobj.x) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x + halfWidth, from.y - halfWidth, width, 2 * halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, toobj.x - halfWidth, toobj.y - halfWidth, width, 2 * halfWidth);
            }
          }
        }
      }
      else
      {
        TranslateUtil.reshapeRect(this._tempRect1, (from.x <= toobj.x ? from.x : toobj.x), (from.y <= toobj.y ? from.y : toobj.y), width, height);
      }

      return _CostComputer.isOverlap(this._tempRect1, rect); // Boolean
    }

    return false; // Boolean
  }

}
);

lang.mixin(_CostComputer,
{
  SMALL_PENALTY : null,

  isOverlap : function(rect1, rect2)
  {
    return (rect1.x < rect2.x + rect2.width && rect2.x < rect1.x + rect1.width && rect1.y < rect2.y + rect2.height && rect2.y < rect1.y + rect1.height); // Boolean
  },

  _fields_init : function(c)
  {
    c = c || _CostComputer;
    
    c.SMALL_PENALTY = 0.01;
  }
  
}
);

(function(){ _CostComputer._fields_init(); })();

return _CostComputer;
});
