/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    TranslateUtil	// #7
    ) {

var _IncidentLinksRefiner = declare("ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner", null, 
{
  // summary:
  //		This class is used to refine the links incident to a node. 
  // description:
  //		This class is used to refine the links incident to a node. 
  //		This consists in moving some segments of the links in order to ovoid
  //		overlaps and crossings between the links. Each layout instance uses
  //		only one instance of it for each link style.  
  // tags:
  //		extension
  
  _layout: null,

  constructor : function(layout)
  {
    this._layout = layout;
  },
  
  getLayoutAlgorithm : function()
  {
    return this._layout; // ibm_ilog.graphlayout.shortlink._ShortLinkAlgorithm
  },

  clean : function()
  {
    // summary:
    //		Cleans the data stored by this class.
    // tags:
    //		private
    
  },

  refine : function(vectLinks, nodeData, nodeSide, linkOffset, minFinalSegmentLength, reversedOrder, withFixedConnectionPoints)
  {
    // summary:
    //		Final refinement of the link layout. 
    // description:
    //		Final refinement of the link layout. 
    //		Refines a set of links connected to <code>nodeSide</code> side of
    //		<code>node</code>. Once the type of shape for each link and the
    //		order of the connection of all the links incident to a given side of
    //		a node are choosen, the second and, eventually, the third segment of
    //		an orthogonal link can be moved a little by this refinement
    //		procedure. A different procedure, that can move the two intermediate
    //		points, is used for direct style links. The purpose of the
    //		refinement is to avoid overlapping segments for links that are
    //		incident to the same side of the same node. (Currently, nothing is
    //		done for links that are not incident to the same side of the same
    //		node.) CAUTION: it is mandatory to call this.sort before this.refine
    //		!  
    // withFixedConnectionPoints: Boolean
    //		true if at least one link in vectLinks
    //		has fixed connection points or is fixed. In such a case, the link
    //		may not be ordered according to their relative orientation toward
    //		the node side.
    // tags:
    //		extension
    },
  
  refineLinksOrthogonal : function(link1, link2, origin1, origin2, nodeSide, goingLeft, linkOffset, stopIfDifferentDirection)
  {
    // summary:
    //		Reshapes the second segment of link2 to avoid crossings with link1,
    //		if necessary.  
    // goingLeft: Boolean
    //		'true' if both links are expected to do a
    //		left-turn, and 'false' if they are expected to do a
    //		right-turn
    // link1: _SLLinkData
    //		previous link (at left of link2)
    //		link2: 
    //		           _SLLinkData
    //		current link (at right of link1)
    // returns:
    //		true 'true' if both links have the
    //		'goingLeft' orientation, and
    //		'false'otherwise.
    // tags:
    //		protected
    
    if (link1 == link2 || link2.isSameSideSelfLink() || link2.isFixed())
    {
      return true; // Boolean
    }

    var nPoints1 = link1.getLinkShape().getNumberOfPoints();
    var nBends2 = link2.getLinkShape().getNumberOfPoints() - 2;

    if (nBends2 < 1)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6603E");
    }

    var point11 = link1.getPointAt(origin1, 1);
    var point12 = (nPoints1 > 2 ? link1.getPointAt(origin1, 2) : point11);
    var point21 = link2.getPointAt(origin2, 1);
    var point22 = link2.getPointAt(origin2, 2);
    var goingLeft1 = nodeSide.getTangentDelta(point11, point12) >= 0;
    var goingLeft2 = nodeSide.getTangentDelta(point21, point22) >= 0;
    if (nBends2 < 2) {
      return (goingLeft ? (goingLeft1 && goingLeft2) : (!goingLeft1 && !goingLeft2)); // Boolean
    }

    var firstLinkIsSameSideSelfLink = link1.isSameSideSelfLink();
    var okDirection = (goingLeft ? (firstLinkIsSameSideSelfLink || goingLeft1 && goingLeft2) : (firstLinkIsSameSideSelfLink || !goingLeft1 && !goingLeft2));

    if (!okDirection && (stopIfDifferentDirection || (goingLeft != goingLeft2 || !goingLeft)))
    {
      return false; // Boolean
    }

    link2.shapePointsModified();
    var nBends1 = link1.getLinkShape().getNumberOfPoints() - 2;
    var point13 = null;
    var point14 = null;

    if (nBends1 >= 2)
    {
      point13 = link1.getPointAt(origin1, 3);
      if (nBends1 >= 3) {
        point14 = link1.getPointAt(origin1, 4);
      }
    }

    var point23 = null;
    var point24 = null;
    var point25 = null;

    if (nBends2 >= 2)
    {
      point23 = link2.getPointAt(origin2, 3);

      if (nBends2 >= 3)
      {
        point24 = link2.getPointAt(origin2, 4);
        if (nBends2 >= 4) {
          point25 = link2.getPointAt(origin2, 5);
        }
      }
    }

    var sameNodeAndSideDestination = link1.getConnectionNode(!origin1) == link2.getConnectionNode(!origin2) && link1.getNodeSide(!origin1) == link2.getNodeSide(!origin2);
    nodeSide.refineLinksOrthogonalWithPoints(this._layout, point11, point12, point13, point14, point21, point22, point23, point24, point25, link1.getLinkWidth(), link2.getLinkWidth(), goingLeft1,
        goingLeft2, firstLinkIsSameSideSelfLink, nBends1, nBends2, link2.boundingBox, link2.getNodeSide(!origin2), linkOffset, sameNodeAndSideDestination, true);

    return true; // Boolean
  }

}
);

return _IncidentLinksRefiner;
});
