/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_SLBundlesModes"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _SLBundlesModes	// #7
    ) {

var _MultiLinkBundleLayouter = declare("ibm_ilog.graphlayout.shortlink._MultiLinkBundleLayouter", null, 
{
  // summary:
  //		This class performs the layout of a bundle of multilinks. 
  // description:
  //		This class performs the layout of a bundle of multilinks. 
  //		The mechanism is the following: For each set of multilinks, the
  //		LinkData of one of the links is flagged as being the "master" link of
  //		the bundle, while the others are "slaves". The master stores the list
  //		of slaves. During the optimization, only the master link is
  //		considered. At the end of the layout, or in animated mode, the real
  //		links need to be lay out according to the computed shape of the
  //		master link. One single instance is used by each instance of
  //		ShortLinkLayout.  
  // tags:
  //		private
  
  constructor : function()
  {
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the data stored by this class.
    // tags:
    //		private
    
  },

  layout : function (a0, a1, a2, a3, a4, a5, a6)
  {
    // summary:
    //		Gives the final shape to a master link together with its slave
    //		links. 
    // description:
    //		Gives the final shape to a master link together with its slave
    //		links. 
    //		Used in "sameShapeForMultipleLinks" mode only.
    // tags:
    //		private
    
    if(arguments.length == 7)
      return this.layout_overload_7(a0, a1, a2, a3, a4, a5, a6);
    else if(arguments.length == 6)
      return this.layout_overload_6(a0, a1, a2, a3, a4, a5);
  },
  
  layout_overload_7 : function(sllayout, graphModel, masterLink, bundleMode, linkOffset, minFinalSegmentLength, redraw)
                                                                                                                      
  {
    var slaveLinks = masterLink.getIndividualLinks();
    if (slaveLinks == null) {
      return;
    }
    var nLinks = slaveLinks.length;
    if (nLinks == 0) {
      return;
    }
    this.layout(sllayout, graphModel, masterLink, bundleMode, linkOffset, redraw);
  },

  layout_overload_6 : function(layout, graphModel, masterLink, bundleMode, linkOffset, redraw)
  {
    // summary:
    //		Lays out the bundle of links. 
    // description:
    //		Lays out the bundle of links. 
    //		For left orientation: ------------ --------- | startLink------ | | |
    //		| | --------- |       | --------- And for right orientation:
    //		------------ |  --------- |  |  ------startLink |  |  | --------- | 
    //		     | ---------
    // tags:
    //		private
    
    var slaveLinks = masterLink.getIndividualLinks();
    var nLinks = slaveLinks.length;
    var slaveLink;
    var masterShape;

    var masterLinkWidth = 0;
    var masterLinkWidthOrig = 0;
    var masterLinkWidthDest = 0;
    var masterLinkOffsetOrig = 0;
    var masterLinkOffsetDest = 0;
    var masterFrom = null;
    var masterTo = null;
    var masterBends = null;
    var masterNBends = 0;
    var onLeftSide = false;
    var halfLinkWidth = 0;
    var previousHalfLinkWidth = 0;
    var offset = 0;
    var offsetOrig = 0;
    var offsetDest = 0;
    var onlyFirstAndLastSeg = bundleMode != _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT;

    var copyPointLocations = onlyFirstAndLastSeg;

    if (bundleMode != _SLBundlesModes.NO_BUNDLE)
    {
      masterFrom = masterLink.getConnectionPoint(true);
      masterTo = masterLink.getConnectionPoint(false);
      masterShape = masterLink.getLinkShape();
      masterBends = masterShape.getIntermediatePoints();
      masterNBends = masterShape.getNumberOfPoints() - 2;
      masterLinkWidth = masterLink.getLinkWidth();
      masterLinkWidthOrig = masterLink.getLinkWidthWithOffset(masterLink.getOriginLinkOffset());
      masterLinkWidthDest = masterLink.getLinkWidthWithOffset(masterLink.getDestinationLinkOffset());
      masterLinkOffsetOrig = masterLink.getOriginLinkOffset();
      masterLinkOffsetDest = masterLink.getDestinationLinkOffset();

      onLeftSide = masterLink.isGoingLeft(true);
    }

    for (var i = 0; i < nLinks; i++)
    {
      slaveLink = slaveLinks[i];

      slaveLink.prepareForBundleLayout(masterLink, copyPointLocations);

      if (bundleMode != _SLBundlesModes.NO_BUNDLE)
      {
        halfLinkWidth = slaveLink.getLinkWidth() * 0.5;

        if (i == 0)
        {
          offset = masterLinkWidth * 0.5 - halfLinkWidth;
          offsetOrig = masterLinkWidthOrig * 0.5 - halfLinkWidth;
          offsetDest = masterLinkWidthDest * 0.5 - halfLinkWidth;
        }
        else
        {
          offset -= (linkOffset + halfLinkWidth + previousHalfLinkWidth);
          offsetOrig -= (masterLinkOffsetOrig + halfLinkWidth + previousHalfLinkWidth);
          offsetDest -= (masterLinkOffsetDest + halfLinkWidth + previousHalfLinkWidth);
        }

        previousHalfLinkWidth = halfLinkWidth;
        slaveLink.applyOffset(masterFrom, masterTo, masterBends, masterNBends, offset, offsetOrig, offsetDest, onLeftSide, masterLink.getFromNode() != slaveLink.getFromNode(), onlyFirstAndLastSeg);
      }

      layout.finalShapeLink(graphModel, slaveLink, false, redraw);
    }
  }

}
);

return _MultiLinkBundleLayouter;
});
