/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../tree/TGraphAlgorithm",	// #8
  "../tree/TLayoutModes"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    GraphModelUtil,	// #6
    TranslateUtil,	// #7
    TGraphAlgorithm,	// #8
    TLayoutModes	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var TGraphAlgorithm = ibm_ilog.graphlayout.tree.TGraphAlgorithm;
=====*/

var TAutoTipOver = declare("ibm_ilog.graphlayout.tree.TAutoTipOver", TGraphAlgorithm, 
{
  // summary:
  //		This class calls contains the support for determining the local tip
  //		over specification automatically to optimize the aspect ratio. 
  // description:
  //		This class calls contains the support for determining the local tip
  //		over specification automatically to optimize the aspect ratio. 
  //		To calculate the optimal result, it would need exponential time.
  //		Therefore this here is just a heuristic that tries a few hard coded
  //		approaches and chooses the best one. (See at the end in comments the
  //		code that would test all exponentially many possibilities).  
  // tags:
  //		private
  
  _aspectRatio: null,

  _bestArea: null,
  _maxDepth: null,

  constructor : function(graph)
  {
    this._aspectRatio = graph.getAspectRatio();

    if (graph.getFlowDirection() % 2 == 0) {
      this._aspectRatio = 1 / this._aspectRatio;
    }
    this._bestArea = Number.MAX_VALUE;
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.inherited("disposeIt", arguments, []);
  },

  getAspectRatio : function()
  {
    // summary:
    //		Returns the desired aspect ratio.
    // tags:
    //		private
    
    return this._aspectRatio; // Number
  },

  getMaxDepth : function()
  {
    // summary:
    //		Returns the maximal depth of the tree.
    // tags:
    //		private
    
    return this._maxDepth; // int
  },

  isBetterAspectRatio : function(width, hight)
  {
    // summary:
    //		Determines whether the input aspect ratio is better than the current
    //		best aspect ratio. 
    // description:
    //		Determines whether the input aspect ratio is better than the current
    //		best aspect ratio. 
    //		Returns <code>true</code> if the current situation is better than
    //		the best previous situation.
    // tags:
    //		private
    
    var a;

    if (width == 0.0 || hight == 0.0) {
      return (false); // Boolean
    }
    else if (width / hight < this.getAspectRatio()) {
      a = hight * hight * this.getAspectRatio();
    }
    else {
      a = width * width / this.getAspectRatio();
    }
    if (a < this._bestArea)
    {
      this._bestArea = a;
      return (true); // Boolean
    }
    else {
      return (false); // Boolean
    }
  },

  storeOriginalAlignmentStyle : function()
  {
    // summary:
    //		Stores the current alignment style as original alignment style for
    //		all nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().storeOriginalAlignmentStyle();
  },

  restoreOriginalAlignmentStyle : function()
  {
    // summary:
    //		Restores the original alignment style for all nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().restoreOriginalAlignmentStyle();
  },

  storeBestAlignmentStyle : function()
  {
    // summary:
    //		Stores the current alignment style as best alignment style for all
    //		nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().storeBestAlignmentStyle();
  },

  restoreBestAlignmentStyle : function()
  {
    // summary:
    //		Restores the best alignment style for all nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().restoreBestAlignmentStyle();
  },

  doLayout : function()
  {
    // summary:
    //		Perform one layout and check aspect ratio afterwards. 
    // description:
    //		Perform one layout and check aspect ratio afterwards. 
    //		After that it restores the original situation again.
    // tags:
    //		private
    
    var boundingBox;
    var graph = this.getGraph();
    graph.calcCoords();
    var a = graph.calcBoundingBox();
    boundingBox = graph.calcBoundingBox();
    var w = boundingBox.width;
    var h = boundingBox.height;
    if (this.isBetterAspectRatio(w, h)) {
      this.storeBestAlignmentStyle();
    }
    this.restoreOriginalAlignmentStyle();
  },

  run : function(mode)
  {
    // summary:
    //		Performs several layouts with different local tip-over alignment
    //		strategies. 
    // description:
    //		Performs several layouts with different local tip-over alignment
    //		strategies. 
    //		It determines which one has the best space characteristics at given
    //		aspect ration. Finally, it sets to the best layout.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this._maxDepth = graph.getMaxDepth();

    this.storeOriginalAlignmentStyle();

    var numberOfTries = 3;

    switch (mode)
    {
      case AlignmentOptions.TIP_OVER:
        numberOfTries = 8;
        break;
      case TLayoutModes.TIP_LEAVES_OVER:
      case TLayoutModes.TIP_ROOTS_OVER:
        numberOfTries = this._maxDepth + 3;
        break;
      case TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER:
        numberOfTries = Math.floor(this._maxDepth * (this._maxDepth - 1) / 2) + 3;
        break;
    }
    
    this.startStep(0.0, 1);
    this.setStepEstimation(6 + numberOfTries * 2);
    graph.changePercParameterForAutoTipOverLayout(numberOfTries);

    this.doLayout();
    if (!this.mayContinue()) {
      return;
    }
    if (this.tipAllOver()) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    switch (mode)
    {
      case AlignmentOptions.TIP_OVER:
        this.fastTipOver();
        break;
      case TLayoutModes.TIP_LEAVES_OVER:
        this.fromLeavesToRoot();
        break;
      case TLayoutModes.TIP_ROOTS_OVER:
        this.fromRootToLeaves();
        break;
      case TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER:
        this.fromRootAndLeavesToInner();
        break;
    }
    
    this.restoreBestAlignmentStyle();
    graph.calcCoords();
  },

  tipAllOver : function()
  {
    // summary:
    //		Tip all nodes over. 
    // description:
    //		Tip all nodes over. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;
    var graph = this.getGraph();
    var node;
    var iter = graph.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.canAutoTipOver())
      {
        changed = true;
        node.setAlignmentStyle(AlignmentOptions.TIP_OVER);
      }
    }

    return (changed); // Boolean
  },

  fromLeavesToRoot : function (a0)
  {
    // summary:
    //		Tips over the leaves and their ancestors up to a certain level. 
    // description:
    //		Tips over the leaves and their ancestors up to a certain level. 
    //		It tries all levels.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.fromLeavesToRoot_overload_0();
    else if(arguments.length == 1)
      return this.fromLeavesToRoot_overload_1(a0);
  },
  
  fromLeavesToRoot_overload_0 : function()
                                          
  {
    var graph = this.getGraph();

    for (var i = 0; i < this.getMaxDepth(); i++)
    {
      if (this.fromLeavesToRoot(i)) {
        this.doLayout();
      }
      else {
        graph.correctPercentageForTipOver();
      }
      if (!this.mayContinue()) {
        return;
      }
    }
  },

  fromLeavesToRoot_overload_1 : function(tipOverLevel)
  {
    // summary:
    //		Tip over all nodes at depth starting at tip-over-level. 
    // description:
    //		Tip over all nodes at depth starting at tip-over-level. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    var iter = this.getGraph().getRoots();

    while (iter.hasNext())
    {
      changed = this.tipOverAtDepth(iter.next(), 0, -1, tipOverLevel) || changed;
    }

    return (changed); // Boolean
  },

  fromRootToLeaves : function (a0)
  {
    // summary:
    //		Tips over the root and their descendents up to a certain level. 
    // description:
    //		Tips over the root and their descendents up to a certain level. 
    //		It tries all levels.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.fromRootToLeaves_overload_0();
    else if(arguments.length == 1)
      return this.fromRootToLeaves_overload_1(a0);
  },
  
  fromRootToLeaves_overload_0 : function()
                                          
  {
    var graph = this.getGraph();

    for (var i = 0; i < this.getMaxDepth(); i++)
    {
      if (this.fromRootToLeaves(i)) {
        this.doLayout();
      }
      else {
        graph.correctPercentageForTipOver();
      }
      if (!this.mayContinue()) {
        return;
      }
    }
  },

  fromRootToLeaves_overload_1 : function(tipOverLevel)
  {
    // summary:
    //		Tip over all nodes at depth lower than tip-over-level. 
    // description:
    //		Tip over all nodes at depth lower than tip-over-level. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    var iter = this.getGraph().getRoots();

    while (iter.hasNext())
    {
      changed = this.tipOverAtDepth(iter.next(), 0, tipOverLevel, this.getMaxDepth() + 1) || changed;
    }

    return (changed); // Boolean
  },

  fromRootAndLeavesToInner : function (a0, a1)
  {
    // summary:
    //		Tips over the root and their descendents up to a certain level, and
    //		the leaves and their ancestors starting from a certain level. 
    // description:
    //		Tips over the root and their descendents up to a certain level, and
    //		the leaves and their ancestors starting from a certain level. 
    //		It tries all levels.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.fromRootAndLeavesToInner_overload_0();
    else if(arguments.length == 2)
      return this.fromRootAndLeavesToInner_overload_2(a0, a1);
  },
  
  fromRootAndLeavesToInner_overload_0 : function()
                                                  
  {
    var graph = this.getGraph();
    var i;
    var j;

    for (i = -1; i < this.getMaxDepth() - 1; i++)
    {
      for (j = i + 2; j <= this.getMaxDepth(); j++)
      {
        if (this.fromRootAndLeavesToInner(i, j)) {
          this.doLayout();
        }
        else {
          graph.correctPercentageForTipOver();
        }
        if (!this.mayContinue()) {
          return;
        }
      }
    }
  },

  fromRootAndLeavesToInner_overload_2 : function(tipOverRootLevel, tipOverLeavesLevel)
  {
    // summary:
    //		Tip over all nodes at depth 0 to tipOverRootLevel and at depth
    //		tipOverLeavesLevel to maxDepth. 
    // description:
    //		Tip over all nodes at depth 0 to tipOverRootLevel and at depth
    //		tipOverLeavesLevel to maxDepth. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    var iter = this.getGraph().getRoots();

    while (iter.hasNext())
    {
      changed = this.tipOverAtDepth(iter.next(), 0, tipOverRootLevel, tipOverLeavesLevel) || changed;
    }

    return (changed); // Boolean
  },

  fastTipOver : function()
  {
    // summary:
    //		Fast compromise. 
    // description:
    //		Fast compromise. 
    //		Doesn't try all levels, but only the first and second root and leaf level.
    // tags:
    //		private
    
    var graph = this.getGraph();
    if (this.fromLeavesToRoot(this.getMaxDepth() - 1)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromLeavesToRoot(this.getMaxDepth() - 2)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromRootToLeaves(0)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromRootToLeaves(1)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromRootAndLeavesToInner(0, this.getMaxDepth() - 1)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
  },

  tipOverAtDepth : function(node, depth, tipOverRootLevel, tipOverLeavesLevel)
  {
    // summary:
    //		Tip over all nodes at depth lower than 'tipOverRootLevel'
    //		or higher than 'tipOverLeavesLevel'. 
    // description:
    //		Tip over all nodes at depth lower than <code>tipOverRootLevel</code>
    //		or higher than <code>tipOverLeavesLevel</code>. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    if (node.canAutoTipOver())
    {
      if (depth >= tipOverLeavesLevel || depth <= tipOverRootLevel)
      {
        node.setAlignmentStyle(AlignmentOptions.TIP_OVER);
        changed = true;
      }
    }

    for (var i = 0; i < node.getNumberOfChildren(); i++)
    {
      changed = this.tipOverAtDepth(node.getChild(i), depth + 1, tipOverRootLevel, tipOverLeavesLevel) || changed;
    }

    return (changed); // Boolean
  }

}
);

return TAutoTipOver;
});
