/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/Point",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/TranslateUtil",	// #9
  "../tree/TAngleSpacingModes",	// #10
  "../tree/TBalloonRadiusModes",	// #11
  "../tree/TGraphAlgorithm",	// #12
  "../tree/TNode"	// #13
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    Point,	// #7
    Rectangle,	// #8
    TranslateUtil,	// #9
    TAngleSpacingModes,	// #10
    TBalloonRadiusModes,	// #11
    TGraphAlgorithm,	// #12
    TNode	// #13
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var TGraphAlgorithm = ibm_ilog.graphlayout.tree.TGraphAlgorithm;
=====*/

var TBalloon = declare("ibm_ilog.graphlayout.tree.TBalloon", TGraphAlgorithm, 
{
  // summary:
  //		This class is the Balloon Tree layout. 
  // description:
  //		This class is the Balloon Tree layout. 
  //		It runs after calculating the spanning tree, and is independent from
  //		the normal tree layout in TGraph. It does not support subtree blocks
  //		or recursive layout. In principle we could extend it one day to
  //		support recursive layout, but this is currently not done. The
  //		original implementation came from JLoox by Fabien HERTSCHUH e.a.  
  // tags:
  //		private
  
  _initialAngle : null,
  _generalAngleRange : null,
  _leafAngleRange : null,
  _rootAngleRange : null,
  _minimumRadius : null,

  _angleSpacing : null,
  _useNodeSizes : null,

  _radiusMode : null,
  _minNodeSpacing : null,
  _aspectRatio : null,

  compareNodeSurface : function(o1, o2)
  {
    // summary:
    //		Compares two tree nodes based on their surface.  
    // o1: Object
    //		First TNode
    // o2: Object
    //		Second TNode
    // tags:
    //		private
    
    var node1 = o1;
    var node2 = o2;
    var s1 = node1.getBTSubtreeSurface();
    var s2 = node2.getBTSubtreeSurface();

    return TranslateUtil.compareNumber(s1, s2); // int
  },

  constructor : function(graph)
  {
    this._initialAngle = 0;
    this._generalAngleRange = 360;
    this._leafAngleRange = 360;
    this._rootAngleRange = 360;
    this._minimumRadius = 20;
    this._angleSpacing = TBalloon.FAST_PROPORTIONAL;
    this._useNodeSizes = true;
    this._radiusMode = TBalloonRadiusModes.VARIABLE_RADIUS;
    this._minNodeSpacing = 10;
    this._aspectRatio = 1;
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.inherited("disposeIt", arguments, []);
  },

  setParameters : function(angleSpacing, radiusMode, minNodeSpacing, minRadius, aspectRatio, initialAngle, rootAngleRange, innerAngleRange, leafAngleRange, useNodeSizes)
  {
    // summary:
    //		Set the layout parameters.
    // tags:
    //		private
    
    this._angleSpacing = angleSpacing;
    this._radiusMode = radiusMode;
    this._minNodeSpacing = minNodeSpacing;
    this._minimumRadius = minRadius;
    this._aspectRatio = aspectRatio;
    this._initialAngle = initialAngle;
    this._rootAngleRange = rootAngleRange;
    this._generalAngleRange = innerAngleRange;
    this._leafAngleRange = leafAngleRange;
    this._useNodeSizes = useNodeSizes;
    if (this.getGraph().getFlowDirection() == 2) {
      this._initialAngle += 180;
    }
  },

  run : function()
  {
    // summary:
    //		Places all nodes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this.startStep(graph.getPercRadialOrBalloon(), 2 * graph.getNumberOfNodes());
    var coordBase = 0;
    var sdir = graph.getDirTowardsSiblings();
    var offset = graph.getMinDistBetweenBranches(sdir);
    var iter = graph.getRoots();

    while (iter.hasNext())
      coordBase = this.calcBalloonLayout(iter.next(), coordBase, 0.0) + 2.0 * offset;
  },

  calcBalloonLayout : function(root, border0, border1)
  {
    // summary:
    //		Performs a balloon tree layout for the tree starting at input root. 
    //		
    // root: TNode
    //		The root node.
    // border0: Number
    //		The top left border of this tree (dir0).
    // border1: Number
    //		The top left border of this tree (dir1).
    // returns:
    //		The maximal coordinate of the layout in direction dir0.
    // tags:
    //		private
    
    this.incrPercPointEstimation(20);

    var graph = this.getGraph();

    var dir0 = graph.getDirTowardsSiblings();
    var dir1 = 1 - dir0;

    root.setBTCenter(0, 0.0);
    root.setBTCenter(1, 0.0);
    if (!this.mayContinue()) {
      return 0; // Number
    }
    this.calcSubtreeSurfaces(root);

    this.addPercPoints(10);
    this.layoutStepPerformed();

    root.setBTCenter(0, 0);
    root.setBTCenter(1, 0);
    if (!this.mayContinue()) {
      return 0; // Number
    }
    this.computeRadiuses(root);

    var sizeRatioX = Math.max(1.0, this._aspectRatio);
    var sizeRatioY = Math.max(1.0, 1 / this._aspectRatio);
    if (!this.mayContinue()) {
      return 0; // Number
    }
    this.calculateAbsolutePosition(root, this._initialAngle, sizeRatioX, sizeRatioY);

    if (!this.mayContinue()) {
      return 0; // Number
    }
    var bounds = TranslateUtil.newArray(2, 0);
    bounds[0] = root.getMinCoord(dir0);
    bounds[1] = root.getMaxCoord(dir0);
    this.calcSubtreeBounds(root, bounds, dir0);

    var dx;
    var dy;

    if (dir0 == 0)
    {
      dx = border0 - bounds[0];
      dy = border1;
      bounds[0] += dx;
      bounds[1] += dx;
    }
    else
    {
      dx = border1;
      dy = border0 - bounds[0];
      bounds[0] += dy;
      bounds[1] += dy;
    }

    if (!this.mayContinue()) {
      return 0; // Number
    }
    graph.shiftSubtreeBy(root, dx, dy);

    this.addPercPoints(10);
    this.layoutStepPerformed();
    return (bounds[1]); // Number
  },

  computeRadiuses : function(root)
  {
    // summary:
    //		Compute the radius needed for a node not to interfere with its
    //		siblings (and its subtrees).
    // tags:
    //		private
    
    var w = root.getBTSize(0);
    var h = root.getBTSize(1);
    var radius = 0.5 * Math.sqrt(w * w + h * h);
    if (this._angleSpacing == TBalloon.PROPORTIONAL) {
      this.computeRadiusesSlowPropAngleSpacing(root);
    }
    else {
      this.computeRadiusesNormAngleSpacing(root, 180, this._initialAngle, radius);
    }
  },

  computeRadiusesSlowPropAngleSpacing : function(node)
  {
    // summary:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    // description:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    //		This is used for the slow variant of proportional angle spacing.
    // tags:
    //		private
    
    this.addPercPoints(1);
    this.layoutStepPerformed();

    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var nodeHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
    var s = nodeHalfDiag + 0.5 * this._minNodeSpacing;
    var size = node.getNumberOfVirtualChildren();
    if (size == 0) {
      return;
    }
    var childSubtreeBounds = TranslateUtil.newArray(size, null);

    var iter = node.getVirtualChildren();
    var child;
    var i = 0;

    while (iter.hasNext())
    {
      child = iter.next();
      this.computeRadiusesSlowPropAngleSpacing(child);

      childSubtreeBounds[i++] = this.getSubtreeBounds(child, 0);
    }

    var sumChildAngleRange = this.getAngleRange(node);
    var totalSurface = 0;
    var surface;
    var children = TranslateUtil.newArray(size, null);
    i = 0;
    iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      child = iter.next();
      surface = childSubtreeBounds[i].height;
      if (surface == 0) {
        surface = 1;
      }
      child.setBTSubtreeSurface(surface);
      totalSurface += surface;
      children[i++] = child;
    }

    children.sort(lang.hitch(this,this.compareNodeSurface));
    var remainingAngle = sumChildAngleRange;
    var deltaStartAngle = (360 - sumChildAngleRange) / 2;

    for (var j = size - 1; j >= 0; j--)
    {
      child = children[j];
      var childrenSurface = child.getBTSubtreeSurface();
      var maxAngleRange;

      if (size == 1)
      {
        maxAngleRange = 360;
      }
      else
      {
        maxAngleRange = 180;
      }

      var range = (remainingAngle * childrenSurface) / totalSurface;
      child.setBTAngleRange(Math.min(maxAngleRange, range));
      remainingAngle -= child.getBTAngleRange();
      totalSurface -= childrenSurface;
    }

    var minimumRadius = this._minimumRadius;
    var radius;
    var beta;
    iter = node.getVirtualChildren();
    i = 0;
    var currentAngle = 180 + deltaStartAngle;

    while (iter.hasNext())
    {
      child = iter.next();

      if (this._useNodeSizes)
      {
        w = child.getBTSize(0);
        h = child.getBTSize(1);
        var childHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
        minimumRadius = this._minimumRadius + nodeHalfDiag + childHalfDiag;
      }

      var angleRange = child.getBTAngleRange();
      var rangleRange = (angleRange)/180.0*Math.PI;
      var H1 = Math.abs(childSubtreeBounds[i].y);
      var H2 = childSubtreeBounds[i].height - H1;
      var angle = 0;
      var Raux;

      if (H1 == 0)
      {
        angle = 0;
        if (angleRange >= 89) {
          Raux = 0;
        }
        else {
          Raux = H2 / Math.tan(rangleRange);
        }
      }
      else if (H2 == 0)
      {
        angle = angleRange;
        if (angleRange >= 89) {
          Raux = 0;
        }
        else {
          Raux = H1 / Math.tan(rangleRange);
        }
      }
      else
      {
        if (angleRange >= 179)
        {
          angle = angleRange / 2;
          Raux = 0;
        }
        else
        {
          angle = TBalloon.solveTanEquation(rangleRange, H1 / H2);
          Raux = H1 / Math.tan(angle);
          angle = (angle)*180.0/Math.PI;
        }
      }

      radius = Math.max(nodeHalfDiag, Raux) - childSubtreeBounds[i].x;
      radius = Math.max(radius, minimumRadius);
      child.setBTRadius(radius);
      child.setBTAngle(currentAngle + angleRange - angle);
      currentAngle += child.getBTAngleRange();
      i++;
    }

    this.optimizeRadius(node, childSubtreeBounds, true);
  },

  computeRadiusesNormAngleSpacing : function(node, angleRange, initialAngle, parentHalfDiag)
  {
    // summary:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    // description:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    //		This is used for constant angle spacing and proportional angle
    //		spacing. Returns the bounds of the entire subtree starting at node
    //		relative to the node center, i.e. the bounds are exactly returned
    //		rectangle if the node center is placed at (0,0). In reality, the
    //		node center will be placed somewhere else, but at that time point we
    //		don't know yet where.
    // tags:
    //		private
    
    this.addPercPoints(1);
    this.layoutStepPerformed();
    node.setBTAngle(initialAngle);

    if (angleRange > 180.0)
    {
      angleRange = 180;
    }

    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var w2 = w + this._minNodeSpacing;
    var h2 = h + this._minNodeSpacing;
    var subtreeBounds = new Rectangle(-w2 / 2, -h2 / 2, w2, h2);
    var nodeHalfDiag = 0.5 * Math.sqrt(w * w + h * h);

    var sumChildAngleRange = this.getAngleRange(node);
    var currentAngle = initialAngle - (sumChildAngleRange / 2);
    var size = node.getNumberOfVirtualChildren();
    var iter;
    var child;

    var i = 0;

    if (this._angleSpacing == TBalloon.REGULAR)
    {
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        iter.next().setBTAngleRange((sumChildAngleRange / size));
      }
    }
    else
    {
      var totalSurface = 0;
      var children = TranslateUtil.newArray(size, null);
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        totalSurface += Math.sqrt(child.getBTSubtreeSurface());
        children[i++] = child;
      }

      children.sort(lang.hitch(this,this.compareNodeSurface));
      var remainingAngle = sumChildAngleRange;

      for (var j = size - 1; j >= 0; j--)
      {
        child = children[j];
        var childrenSurface = Math.sqrt(child.getBTSubtreeSurface());
        var maxAngleRange;

        if (size == 1)
        {
          maxAngleRange = 360;
        }
        else
        {
          maxAngleRange = 180;
        }

        var range = (remainingAngle * childrenSurface) / totalSurface;
        child.setBTAngleRange(Math.min(maxAngleRange, range));
        remainingAngle -= child.getBTAngleRange();
        totalSurface -= childrenSurface;
      }
    }

    var childSubtreeBounds = TranslateUtil.newArray(size, null);
    iter = node.getVirtualChildren();
    i = 0;

    while (iter.hasNext())
    {
      child = iter.next();
      currentAngle += (child.getBTAngleRange() / 2);
      childSubtreeBounds[i++] = this.computeRadiusesNormAngleSpacing(child, child.getBTAngleRange(), currentAngle, nodeHalfDiag);
      currentAngle += (child.getBTAngleRange() / 2);
    }

    this.optimizeRadius(node, childSubtreeBounds, false);

    iter = node.getVirtualChildren();
    i = 0;

    while (iter.hasNext())
    {
      child = iter.next();

      var ca = child.getBTAngle();
      var cr = child.getBTRadius();
      var childCenterX = cr * Math.cos((ca)/180.0*Math.PI);
      var childCenterY = -cr * Math.sin((ca)/180.0*Math.PI);
      var childSubtreeCenterX = childCenterX + TranslateUtil.getRectCenterX(childSubtreeBounds[i]);
      var childSubtreeCenterY = childCenterY + TranslateUtil.getRectCenterY(childSubtreeBounds[i]);
      var childSubtreeWidth = childSubtreeBounds[i].width;
      var childSubtreeHeight = childSubtreeBounds[i].height;
      TranslateUtil.addPoint(subtreeBounds, childSubtreeCenterX - (childSubtreeWidth / 2), childSubtreeCenterY - (childSubtreeHeight / 2));
      TranslateUtil.addPoint(subtreeBounds, childSubtreeCenterX + (childSubtreeWidth / 2), childSubtreeCenterY + (childSubtreeHeight / 2));
      i++;
    }

    var minimumRadius = this.getMinRadius(subtreeBounds, initialAngle, parentHalfDiag, nodeHalfDiag);
    node.setBTMinRadius(minimumRadius);
    var radius = 0;
    var alpha1 = initialAngle - (angleRange / 2);
    var alpha2 = initialAngle + (angleRange / 2);
    var sinInitialAngle = Math.sin((initialAngle)/180.0*Math.PI);
    var cosInitialAngle = Math.cos((initialAngle)/180.0*Math.PI);

    var offsets = 
      [
        new Point(subtreeBounds.x, subtreeBounds.y),
        new Point(subtreeBounds.x, TranslateUtil.getRectMaxY(subtreeBounds)),
        new Point(TranslateUtil.getRectMaxX(subtreeBounds), TranslateUtil.getRectMaxY(subtreeBounds)),
        new Point(TranslateUtil.getRectMaxX(subtreeBounds), subtreeBounds.y)
       ];
    var sinAlpha1 = Math.sin((alpha1)/180.0*Math.PI);
    var cosAlpha1 = Math.cos((alpha1)/180.0*Math.PI);
    var D = sinInitialAngle * cosAlpha1 - cosInitialAngle * sinAlpha1;
    var T = 0;

    if (D != 0)
    {
      for (i = 0; i < 4; i++)
      {
        T = offsets[i].y * cosAlpha1 + offsets[i].x * sinAlpha1;
        radius = Math.max(radius, T / D);
      }
    }

    var sinAlpha2 = Math.sin((alpha2)/180.0*Math.PI);
    var cosAlpha2 = Math.cos((alpha2)/180.0*Math.PI);
    D = sinInitialAngle * cosAlpha2 - cosInitialAngle * sinAlpha2;

    if (D != 0)
    {
      for (i = 0; i < 4; i++)
      {
        T = offsets[i].y * cosAlpha2 + offsets[i].x * sinAlpha2;
        radius = Math.max(radius, T / D);
      }
    }

    var radiusCorrection = 0;
    node.setBTRadius(Math.max(radius + radiusCorrection, minimumRadius));
    return subtreeBounds; // Rectangle
  },

  optimizeRadius : function(node, childSubtreeBounds, recalcSubtreeBounds)
  {
    // summary:
    //		Optimize the radius for the children of the input node. 
    // description:
    //		Optimize the radius for the children of the input node. 
    //		At that time point, all children have an angle and a radius. It
    //		assumes that the angle of node and of all its children is already
    //		stored in the node. The radius of the children must also be stored
    //		already, except for the node itself. The childSubtreeBounds must be
    //		relative to the child locations, i.e. they assume that the children
    //		are at (0,0).
    // tags:
    //		private
    
    var child;
    var iter;
    var maxChildRadius = 0;
    var maxMinChildRadius = 0;
    var i;

    if (this._radiusMode == TBalloonRadiusModes.UNIFORM_RADIUS)
    {
      var maxRadius = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        maxRadius = Math.max(maxRadius, child.getBTRadius());
      }

      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        child.setBTRadius(maxRadius);
      }

      return;
    }
    else if (this._radiusMode == TBalloonRadiusModes.UNIFORM_LEAVES_RADIUS || this._radiusMode == TBalloonRadiusModes.OPTIMIZED_UNIFORM_LEAVES_RADIUS)
    {
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (child.getNumberOfChildren() == 0)
        {
          maxChildRadius = Math.max(maxChildRadius, child.getBTRadius());
          maxMinChildRadius = Math.max(maxMinChildRadius, child.getBTMinRadius());
        }
      }

      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        if (child.getNumberOfChildren() == 0) {
          child.setBTRadius(maxChildRadius);
        }
      }
    }

    if (this._radiusMode == TBalloonRadiusModes.UNIFORM_RADIUS || this._radiusMode == TBalloonRadiusModes.UNIFORM_LEAVES_RADIUS || this._radiusMode == TBalloonRadiusModes.VARIABLE_RADIUS) {
      return;
    }
    if (recalcSubtreeBounds)
    {
      var w = node.getBTSize(0);
      var h = node.getBTSize(1);
      var nodeHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
      maxMinChildRadius = 0;
      i = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        w = child.getBTSize(0);
        h = child.getBTSize(1);
        var childHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
        childSubtreeBounds[i] = this.getSubtreeBounds(child, child.getBTAngle());
        child.setBTMinRadius(this.getMinRadius(childSubtreeBounds[i], child.getBTAngle(), nodeHalfDiag, childHalfDiag));
        if (child.getNumberOfChildren() == 0) {
          maxMinChildRadius = Math.max(maxMinChildRadius, child.getBTMinRadius());
        }
        i++;
      }
    }

    var handled = TranslateUtil.newArray(node.getNumberOfVirtualChildren(), false);
    var minRadius;
    var actRadius;
    var testRadius;

    if (this._radiusMode == TBalloonRadiusModes.OPTIMIZED_UNIFORM_LEAVES_RADIUS)
    {
      minRadius = maxMinChildRadius;
      actRadius = maxChildRadius;

      while (actRadius > minRadius && actRadius - minRadius > 1)
      {
        testRadius = (minRadius + actRadius) / 2;

        iter = node.getVirtualChildren();

        while (iter.hasNext())
        {
          child = iter.next();

          if (child.getNumberOfChildren() == 0)
          {
            child.setBTRadius(testRadius);
          }
        }

        i = 0;
        var overlaps = false;
        iter = node.getVirtualChildren();

        while (iter.hasNext() && !overlaps)
        {
          child = iter.next();

          if (child.getNumberOfChildren() == 0)
          {
            overlaps = this.overlapsSubtree(node, i, child.getBTAngle(), testRadius, childSubtreeBounds);
          }

          i++;
        }

        if (overlaps)
        {
          iter = node.getVirtualChildren();

          while (iter.hasNext())
          {
            child = iter.next();

            if (child.getNumberOfChildren() == 0)
            {
              child.setBTRadius(actRadius);
            }
          }

          minRadius = testRadius;
        }
        else
        {
          actRadius = testRadius;
        }
      }

      i = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (child.getNumberOfChildren() == 0)
        {
          handled[i] = true;
        }

        i++;
      }
    }

    while (true)
    {
      var bestIndex = -1;
      var bestChild = null;
      var bestRadius = Number.MAX_VALUE;
      var bestChildAngle = 0;
      i = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (!handled[i] && child.getBTRadius() < bestRadius)
        {
          bestChild = child;
          bestRadius = child.getBTRadius();
          bestChildAngle = child.getBTAngle();
          bestIndex = i;
        }

        i++;
      }

      if (bestChild == null) {
        return;
      }
      minRadius = bestChild.getBTMinRadius();
      actRadius = bestRadius;

      while (actRadius > minRadius && actRadius - minRadius > 1)
      {
        testRadius = (minRadius + actRadius) / 2;

        if (this.overlapsSubtree(node, bestIndex, bestChildAngle, testRadius, childSubtreeBounds))
        {
          minRadius = testRadius;
        }
        else
        {
          bestChild.setBTRadius(testRadius);
          actRadius = testRadius;
        }
      }

      handled[bestIndex] = true;
    }
  },

  overlapsSubtree : function(node, index, childAngle, childRadius, childSubtreeBounds)
  {
    // summary:
    //		Tests whether the subtree of input index overlaps with the bounds of
    //		any other subtree in the current radius setting.  
    // tags:
    //		private
    
    var testRect1 = new Rectangle();
    var testRect2 = new Rectangle();
    testRect1.x = childSubtreeBounds[index].x;
    testRect1.y = childSubtreeBounds[index].y;
    testRect1.width = childSubtreeBounds[index].width;
    testRect1.height = childSubtreeBounds[index].height;
    testRect1.x += (childRadius * Math.cos((childAngle)/180.0*Math.PI));
    testRect1.y += (-childRadius * Math.sin((childAngle)/180.0*Math.PI));

    var cr = 100000;
    var ca = node.getBTAngle();
    var dx = (cr * Math.cos((ca + 180)/180.0*Math.PI));
    var dy = (-cr * Math.sin((ca + 180)/180.0*Math.PI));
    if (node.getImmediateParent() != null && this.overlapsLine(testRect1, 0, 0, dx, dy)) {
      return true; // Boolean
    }
    var i = 0;
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();

      if (i != index)
      {
        cr = child.getBTRadius();
        ca = child.getBTAngle();
        dx = (cr * Math.cos((ca)/180.0*Math.PI));
        dy = (-cr * Math.sin((ca)/180.0*Math.PI));
        testRect2.x = childSubtreeBounds[i].x + dx;
        testRect2.y = childSubtreeBounds[i].y + dy;
        testRect2.width = childSubtreeBounds[i].width;
        testRect2.height = childSubtreeBounds[i].height;
        if (TranslateUtil.rectIntersects(testRect1, testRect2)) {
          return true; // Boolean
        }
        if (this.overlapsLine(testRect1, 0, 0, dx, dy)) {
          return true; // Boolean
        }
      }

      i++;
    }

    return false; // Boolean
  },

  getMinRadius : function(subtreeBounds, angle, parentHalfDiag, nodeHalfDiag)
  {
    // summary:
    //		Compute the minimal radius of the node.
    // tags:
    //		private
    
    while (angle > 360)
      angle -= 360;

    while (angle < 0)
      angle += 360;
    var rangle = (angle)/180.0*Math.PI;
    var minimumRadius = this._minimumRadius;

    if (this._useNodeSizes)
    {
      minimumRadius += parentHalfDiag + nodeHalfDiag;
    }

    var dx;
    var dy;
    var sinAlpha;
    var cosAlpha;
    var M;

    if (angle < 1)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag - subtreeBounds.x);
    }
    else if (angle < 89)
    {
      dx = -subtreeBounds.x;
      dy = subtreeBounds.height + subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else if (angle < 91)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + subtreeBounds.height + subtreeBounds.y);
    }
    else if (angle < 179)
    {
      dx = subtreeBounds.width + subtreeBounds.x;
      dy = subtreeBounds.height + subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else if (angle < 181)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + subtreeBounds.width + subtreeBounds.x);
    }
    else if (angle < 269)
    {
      dx = subtreeBounds.width + subtreeBounds.x;
      dy = -subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else if (angle < 271)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag - subtreeBounds.y);
    }
    else if (angle < 359)
    {
      dx = -subtreeBounds.x;
      dy = -subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag - subtreeBounds.x);
    }

    return minimumRadius; // Number
  },

  calculateAbsolutePosition : function(node, initialAngle, sizeRatioX, sizeRatioY)
  {
    // summary:
    //		Computes absolute positions of nodes from angles and radiuses
    //		computed beforehand.
    // tags:
    //		private
    
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();
      var currentAngle = child.getBTAngle();

      if (this._angleSpacing == TBalloon.PROPORTIONAL)
      {
        currentAngle += initialAngle;
      }

      var newX = (node.getBTCenter(0) + (child.getBTRadius() * sizeRatioX * Math.cos((currentAngle)/180.0*Math.PI)));
      var newY = (node.getBTCenter(1) - (child.getBTRadius() * sizeRatioY * Math.sin((currentAngle)/180.0*Math.PI)));
      child.setBTCenter(0, newX);
      child.setBTCenter(1, newY);
      this.calculateAbsolutePosition(child, currentAngle, sizeRatioX, sizeRatioY);
    }

    this.addPercPoints(1);
    this.layoutStepPerformed();
  },

  getSubtreeBounds : function(node, initialAngle)
  {
    // summary:
    //		Computes the bounds of the subtree for angle spacing slow
    //		proportional. 
    // description:
    //		Computes the bounds of the subtree for angle spacing slow
    //		proportional. 
    //		In this case, the angle of the link from node to child is stored in
    //		the child as offset to the parent angle.
    // tags:
    //		private
    
    var result = new Rectangle();
    this.addSubtreeBounds(node, 0, 0, initialAngle, result);
    result.x -= this._minNodeSpacing / 2;
    result.y -= this._minNodeSpacing / 2;
    result.width += this._minNodeSpacing;
    result.height += this._minNodeSpacing;

    if (result.width == 0) {
      result.width = 0.1;
    }
    if (result.height == 0) {
      result.height = 0.1;
    }
    return result; // Rectangle
  },

  addSubtreeBounds : function(node, nodeX, nodeY, initialAngle, bbox)
  {
    // summary:
    //		This does the real work.
    // tags:
    //		private
    
    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var r = Math.max(w, h) / 2;
    var newX;
    var newY;
    TranslateUtil.addPoint(bbox, nodeX - r, nodeY - r);
    TranslateUtil.addPoint(bbox, nodeX + r, nodeY + r);
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();
      var currentAngle = initialAngle + child.getBTAngle();
      newX = (nodeX + (child.getBTRadius() * Math.cos((currentAngle)/180.0*Math.PI)));
      newY = (nodeY - (child.getBTRadius() * Math.sin((currentAngle)/180.0*Math.PI)));
      this.addSubtreeBounds(child, newX, newY, currentAngle, bbox);
    }
  },

  calcSubtreeBounds : function(node, bounds, dir)
  {
    // summary:
    //		Calculate the bounds of the subtree.
    // tags:
    //		private
    
    bounds[0] = Math.min(bounds[0], node.getMinCoord(dir));
    bounds[1] = Math.max(bounds[1], node.getMaxCoord(dir));
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      this.calcSubtreeBounds(iter.next(), bounds, dir);
    }
  },

  calcSubtreeSurfaces : function(node)
  {
    // summary:
    //		This does the work for one subtree starting at the input node. 
    // description:
    //		This does the work for one subtree starting at the input node. 
    //		It returns the surface of the subtree, and additionally stores all
    //		surfaces of all subsubtrees in the nodes. These are used as
    //		heuristical value to estimate the area of the balloons for subtrees
    //		that have that node as root.  We use it to determine what angle
    //		range a subtree can occupy.
    // tags:
    //		private
    
    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var surface = Math.sqrt(w * h);
    if (!this._useNodeSizes) {
      surface = 1;
    }
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();
      surface += this.calcSubtreeSurfaces(child);
    }

    node.setBTSubtreeSurface(surface);
    return surface; // Number
  },

  getAngleRange : function(node)
  {
    // summary:
    //		Returns the angle raange granted to children of the input node to
    //		place themselves.
    // tags:
    //		private
    
    if (node.getImmediateParent() == null)
    {
      return this._rootAngleRange; // Number
    }
    else if (node.hasOnlyLeaves())
    {
      return this._leafAngleRange; // Number
    }
    else
    {
      return this._generalAngleRange; // Number
    }
  },

  overlapsLine : function(rect, x0, y0, x1, y1)
  {
    // summary:
    //		Check if the input rectangle overlaps the line segment.  
    // tags:
    //		private
    
    if (TranslateUtil.rectContains(rect, x0, y0)) {
      return true; // Boolean
    }
    if (TranslateUtil.rectContains(rect, x1, y1)) {
      return true; // Boolean
    }
    var h = rect.height;
    var w = rect.width;
    if (LayoutUtil.intersects(rect.x, rect.y, rect.x, rect.y + h, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    if (LayoutUtil.intersects(rect.x + w, rect.y, rect.x + w, rect.y + h, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    if (LayoutUtil.intersects(rect.x, rect.y, rect.x + w, rect.y, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    if (LayoutUtil.intersects(rect.x, rect.y + h, rect.x + w, rect.y + h, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    return false; // Boolean
  }

}
);

lang.mixin(TBalloon,
{
  REGULAR : null,

  FAST_PROPORTIONAL : null,

  PROPORTIONAL : null,

  solveTanEquation : function(Z, F)
  {
    var limit = Math.PI / 2;
    var x = F * Z / (1 + F);
    if (x >= limit || Z - x >= limit) {
      x = Z / 2;
    }
    var bestX = x;
    var bestD = Math.abs(Math.tan(x) / Math.tan(Z - x) - F);
    var delta = Z / 3;

    while (delta > 0.0001)
    {
      x = x - delta;

      for (var i = 0; i < 2; i++)
      {
        if (x >= 0 && x < Z && x < limit && Z - x < limit)
        {
          var D = Math.abs(Math.tan(x) / Math.tan(Z - x) - F);

          if (D < bestD)
          {
            bestD = D;
            bestX = x;
          }
        }

        x += 2 * delta;
      }

      x = bestX;
      delta = delta / 2;
    }

    return x; // Number
  },

  _fields_init : function(c)
  {
    c = c || TBalloon;
    if(TGraphAlgorithm._fields_init)
      TGraphAlgorithm._fields_init(c);
    
    c.REGULAR = TAngleSpacingModes.REGULAR;
    c.FAST_PROPORTIONAL = TAngleSpacingModes.FAST_PROPORTIONAL;
    c.PROPORTIONAL = TAngleSpacingModes.PROPORTIONAL;
  }
  
}
);

(function(){ TBalloon._fields_init(); })();

return TBalloon;
});
