/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var TPath = declare("ibm_ilog.graphlayout.tree.TPath", null, 
{
  // summary:
  //		A path for tree layout is needed to describe the left, right, top or
  //		bottom border of a subtree. 
  // description:
  //		A path for tree layout is needed to describe the left, right, top or
  //		bottom border of a subtree. 
  //		Fow instance when we compare two subtrees to determine how much the
  //		right subtree must be shifted in order not to overlap the left
  //		subtree, we basically compare the left border of the right subtree
  //		with the right border of the left subtree. The coordinate stored in a
  //		path cell is in the coordinate system local to the corresponding node
  //		(ignoring the subtree correction value stored in the node). Hence,
  //		the coordinate of two cells of a border path might be in completely
  //		different coordinate systems. We additional store the coordinate
  //		correction value which indicates how much we have to shift the
  //		coordinate system of the next cell compared to the coordinate system
  //		of the actual cell. Along the path, the sum of coordinate correction
  //		values of previous cells plus the local coordinate of the actual cell
  //		is the coordinate in the global coordinate system. This holds for the
  //		x-direction (index 0) as well as for the y direction (index 1).
  //		Therefore we store coordinates and coordinate correction values as
  //		array of 2 elements. We also store for each child of the current root
  //		node the first path cell of the subtree that is visible at the
  //		corresponding border, and we maintain a backpointer from this path
  //		cell to that child node, which is the root of the subtree the node
  //		corresponding to the path cell belongs to. This is needed for the
  //		case that two borders touch each other and must be shifted, because
  //		then we have fast (O(1)) access to the corresponding child whose
  //		subtree causes the shifting. The siblings of this child (together
  //		with their subtrees) must be shifted.  
  // tags:
  //		private
  
  _node: null,

  _subtreeRoot: null,

  _next: null,

  _last: null,

  _startCoord: null,
  _endCoord: null,

  _coordCorrection: null,

  constructor : function(node, isLeftOrTop, dir)
  {
    this._next = null;
    this._last = this;
    this._startCoord = TranslateUtil.newArray(2, 0);
    this._endCoord = TranslateUtil.newArray(2, 0);
    this._coordCorrection = TranslateUtil.newArray(2, 0);
    this._coordCorrection[0] = 0.0;
    this._coordCorrection[1] = 0.0;
    this._subtreeRoot = null;
    this._node = node;
    this.update(isLeftOrTop, dir);
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    if (this._subtreeRoot != null)
    {
      if (this._subtreeRoot.getFirstVisibleAtBorder(0, 0) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(0, 0, null);
      }
      if (this._subtreeRoot.getFirstVisibleAtBorder(0, 1) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(0, 1, null);
      }
      if (this._subtreeRoot.getFirstVisibleAtBorder(1, 0) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(1, 0, null);
      }
      if (this._subtreeRoot.getFirstVisibleAtBorder(1, 1) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(1, 1, null);
      }
    }

    this._subtreeRoot = null;

    this._startCoord = null;
    this._endCoord = null;
    this._coordCorrection = null;
    this._next = null;
    this._last = null;
    this._node = null;
  },

  update : function(isLeftOrTop, dir)
  {
    // summary:
    //		Updates the local coordinates of this cell.  
    // isLeftOrTop: Boolean
    //		is 'true' if this is a left or top border.
    // dir: int
    //		is 0 for left or right border and 1 for top or bottom border.
    // tags:
    //		private
    
    if (isLeftOrTop) {
      this._endCoord[dir] = this._node.getMinCoord(dir);
    }
    else {
      this._endCoord[dir] = this._node.getMaxCoord(dir);
    }
    this._startCoord[dir] = this._endCoord[dir];
    this._startCoord[1 - dir] = this._node.getMinCoord(1 - dir);
    this._endCoord[1 - dir] = this._node.getMaxCoord(1 - dir);
  },

  getNode : function()
  {
    // summary:
    //		Returns the current node.
    // tags:
    //		private
    
    return (this._node); // ibm_ilog.graphlayout.tree.TNode
  },

  getNext : function()
  {
    // summary:
    //		Returns the next path cell.
    // tags:
    //		private
    
    return (this._next); // ibm_ilog.graphlayout.tree.TPath
  },

  getLast : function()
  {
    // summary:
    //		Returns the last path cell.
    // tags:
    //		private
    
    return (this._last); // ibm_ilog.graphlayout.tree.TPath
  },

  getSubtreeRoot : function()
  {
    // summary:
    //		Returns the subtree root. 
    // description:
    //		Returns the subtree root. 
    //		Thats the root of the subtree this path belongs to. Because subtrees
    //		at the right side might be smaller than subtrees on the left side
    //		(in flow direction bottom), the combined border path might contain
    //		path segments that belong to different subtrees: P             The
    //		right border of P consists of parts originated C1    C2  C2       
    //		from C1, C2, and C3. The topmost border cell in / \   / \ /&#95;\       
    //		the combinde border of P originated from C1 or C2 /   \ /&#95;&#95;&#95;\       
    //		   have C2 or C2 as subtree root. /&#95;&#95;&#95;&#95;&#95;\      Needed because when a
    //		right subtree shifts away from the left subtree, some subtrees
    //		between left and right subtree gain space. With the subtree root, it
    //		is O(1) to find those subtrees.
    // tags:
    //		private
    
    return (this._subtreeRoot); // ibm_ilog.graphlayout.tree.TNode
  },

  setSubtreeRoot : function(root)
  {
    // summary:
    //		Sets the subtree root.
    // tags:
    //		private
    
    this._subtreeRoot = root;
  },

  getStartCoord : function(index)
  {
    // summary:
    //		Returns the start coordinate (local to the
    //		node's coordinate system). 
    // description:
    //		Returns the start coordinate (local to the node's coordinate
    //		system). 
    //		If index is 0, it is the x-coordinate, if index is 1, it is the
    //		y-coordinate.
    // tags:
    //		private
    
    return (this._startCoord[index]); // Number
  },

  setStartCoord : function(index, coord)
  {
    // summary:
    //		Sets the start coordinate. 
    // description:
    //		Sets the start coordinate. 
    //		If index is 0, it sets the x-coordinate, if index is 1, then the
    //		y-coordinate.
    // tags:
    //		private
    
    this._startCoord[index] = coord;
  },

  getEndCoord : function(index)
  {
    // summary:
    //		Returns the end coordinate (local to the node's coordinate system). 
    // description:
    //		Returns the end coordinate (local to the node's coordinate system). 
    //		If index is 0, it is the x-coordinate, if index is 1, it is the
    //		y-coordinate.
    // tags:
    //		private
    
    return (this._endCoord[index]); // Number
  },

  setEndCoord : function(index, coord)
  {
    // summary:
    //		Sets the end coordinate. 
    // description:
    //		Sets the end coordinate. 
    //		If index is 0, it sets the x-coordinate, if index is 1, then the
    //		y-coordinate.
    // tags:
    //		private
    
    this._endCoord[index] = coord;
  },

  getLastCellEndCoord : function(index)
  {
    // summary:
    //		Returns the end coordinate of the last cell.
    // tags:
    //		private
    
    var p = this;

    while (p.getNext() != null)
      p = p.getNext();
    return p.getEndCoord(index); // Number
  },

  shiftCoord : function(index, dist)
  {
    // summary:
    //		Shifts the coordinate. 
    // description:
    //		Shifts the coordinate. 
    //		If index is 0, it shifts the x-coordinate, if index is 1, then the
    //		y-coordinate.
    // tags:
    //		private
    
    this._startCoord[index] += dist;
    this._endCoord[index] += dist;
  },

  getCoordCorrection : function(index)
  {
    // summary:
    //		Returns the coordinate correction. 
    // description:
    //		Returns the coordinate correction. 
    //		If index is 0, it is the correction in x-direction, if index is 1,
    //		it is the correction in y-direction.
    // tags:
    //		private
    
    return (this._coordCorrection[index]); // Number
  },

  setCoordCorrection : function(index, coordCorrection)
  {
    // summary:
    //		Sets the coordinate correction. 
    // description:
    //		Sets the coordinate correction. 
    //		If index is 0, it sets the correction in x-direction, if index is 1,
    //		it sets the correction in y-direction.
    // tags:
    //		private
    
    this._coordCorrection[index] = coordCorrection;
  },

  shiftCoordCorrection : function(index, dist)
  {
    // summary:
    //		Shifts the coordinate correction. 
    // description:
    //		Shifts the coordinate correction. 
    //		If index is 0, it shifts the x-coordinate correction, if index is 1,
    //		then the y-coordinate correction.
    // tags:
    //		private
    
    this._coordCorrection[index] += dist;
  },

  cutPath : function(cutCell)
  {
    // summary:
    //		Cuts the path at the input cell (exclusively) and returns the
    //		remaining after the cutCell as indiviual path.
    // tags:
    //		private
    
    var tail = cutCell._next;
    if (tail != null) {
      tail._last = this._last;
    }
    cutCell._next = null;
    this._last = cutCell;
    return tail; // ibm_ilog.graphlayout.tree.TPath
  },

  append : function(path, last)
  {
    // summary:
    //		Appends a path. 
    // description:
    //		Appends a path. 
    //		Important that this cell is the current root of the path. The input
    //		cell need not to be a root, but the additional parameter
    //		<code>last</code> needs to point to the last cell of the input path.
    // tags:
    //		private
    
    this._last._next = path;
    this._last = last;
    path._last = null;
  },

  swap : function(path)
  {
    // summary:
    //		Swaps this path with the input path. 
    // description:
    //		Swaps this path with the input path. 
    //		This path looks afterwards exactly like the input path, and the
    //		input path exactly like this path.
    // tags:
    //		private
    
    var h = this._node;
    this._node = path.getNode();
    path._node = h;
    h = this._subtreeRoot;
    this._subtreeRoot = path.getSubtreeRoot();
    path._subtreeRoot = h;
    var x = this._startCoord[0];
    this._startCoord[0] = path.getStartCoord(0);
    path._startCoord[0] = x;
    x = this._startCoord[1];
    this._startCoord[1] = path.getStartCoord(1);
    path._startCoord[1] = x;
    x = this._endCoord[0];
    this._endCoord[0] = path.getEndCoord(0);
    path._endCoord[0] = x;
    x = this._endCoord[1];
    this._endCoord[1] = path.getEndCoord(1);
    path._endCoord[1] = x;
    x = this._coordCorrection[0];
    this._coordCorrection[0] = path.getCoordCorrection(0);
    path._coordCorrection[0] = x;
    x = this._coordCorrection[1];
    this._coordCorrection[1] = path.getCoordCorrection(1);
    path._coordCorrection[1] = x;
    var p = this._next;
    this._next = path.getNext();
    path._next = p;
    p = this._last;
    if (path.getLast() == path) {
      this._last = this;
    }
    else {
      this._last = path.getLast();
    }
    if (p == this) {
      path._last = path;
    }
    else {
      path._last = p;
    }
  },

  reverse : function(dir)
  {
    // summary:
    //		Reverses the path. 
    // description:
    //		Reverses the path. 
    //		Start and end coordinates in given dir are inverted and swapped. The
    //		next pointer is reversed. This makes only sense when the coordinate
    //		correction is 0. Returns the new first cell.
    // tags:
    //		private
    
    var newLast = this;
    var act = this;
    var prev = null;
    var next;
    var h;

    while (act != null)
    {
      h = act._startCoord[dir];
      act._startCoord[dir] = -act._endCoord[dir];
      act._endCoord[dir] = -h;
      next = act.getNext();
      act._next = prev;
      prev = act;
      act = next;
    }

    prev._last = newLast;
    return prev; // ibm_ilog.graphlayout.tree.TPath
  },

  eliminateCoordinateCoorection : function()
  {
    // summary:
    //		Eliminate the coordinate correction at a border.
    // tags:
    //		private
    
    var x = 0.0;
    var y = 0.0;
    var cell = this;

    while (cell != null)
    {
      cell.shiftCoord(0, x);
      cell.shiftCoord(1, y);
      x += cell.getCoordCorrection(0);
      y += cell.getCoordCorrection(1);
      cell.setCoordCorrection(0, 0.0);
      cell.setCoordCorrection(1, 0.0);
      cell = cell.getNext();
    }
  },

  getMaxCoord : function(dir, westSide)
  {
    // summary:
    //		Returns the maximal coordinate in given direction. 
    // description:
    //		Returns the maximal coordinate in given direction. 
    //		If westSide, it returns only the maximal coordinate in given
    //		direction of those cells that are in opposite direction before 0, if
    //		east side then after 0. This is used in tip-over-east-west to check
    //		the bottom border. The children laid out at the west side are then
    //		the children with x
    //		< 0, and the children at the east side are those with x > 0
    //		(assuming flow direction to the bottom).
    // tags:
    //		private
    
    var max = -/*Number.MAX_VALUE*/(3.4028235E38);
    var x = 0.0;
    var y = 0.0;
    var oppositeDir = 1 - dir;
    var cell = this;

    while (cell != null)
    {
      cell.shiftCoord(0, x);
      cell.shiftCoord(1, y);
      x += cell.getCoordCorrection(0);
      y += cell.getCoordCorrection(1);
      cell.setCoordCorrection(0, 0.0);
      cell.setCoordCorrection(1, 0.0);

      if (westSide)
      {
        if (cell.getStartCoord(oppositeDir) < 0) {
          max = Math.max(max, cell.getStartCoord(dir));
        }
        if (cell.getEndCoord(oppositeDir) < 0) {
          max = Math.max(max, cell.getEndCoord(dir));
        }
      }
      else
      {
        if (cell.getStartCoord(oppositeDir) > 0) {
          max = Math.max(max, cell.getStartCoord(dir));
        }
        if (cell.getEndCoord(oppositeDir) > 0) {
          max = Math.max(max, cell.getEndCoord(dir));
        }
      }

      cell = cell.getNext();
    }

    return max; // Number
  },

  cutForEastWest : function(oppositeDir, westSide)
  {
    // summary:
    //		Cuts this path into two parts. 
    // description:
    //		Cuts this path into two parts. 
    //		On the west side, this path contains this head cell and all its
    //		cells that are before 0 in opposite direction. On the east side,
    //		this path contains this head cell and all its cells that are after 0
    //		in opposite direction. Returns the other path that was cut out. Used
    //		in east-west tip over on the top and bottom border.
    // tags:
    //		private
    
    this.eliminateCoordinateCoorection();
    var head = this;
    var tail = head.cutPath(head);
    if (tail == null) {
      return null; // ibm_ilog.graphlayout.tree.TPath
    }
    var prevCell = null;
    var cell = tail;

    while (cell != null)
    {
      if (cell.getEndCoord(oppositeDir) > 0) {
        break;
      }
      prevCell = cell;
      cell = cell.getNext();
    }

    var eastPart = ((prevCell == null ? tail : tail.cutPath(prevCell)));
    var westPart = ((prevCell == null ? null : tail));

    if (westSide)
    {
      if (westPart != null) {
        this.append(westPart, westPart.getLast());
      }
      return eastPart; // ibm_ilog.graphlayout.tree.TPath
    }
    else
    {
      if (eastPart != null) {
        this.append(eastPart, eastPart.getLast());
      }
      return westPart; // ibm_ilog.graphlayout.tree.TPath
    }
  },

  combineForEastWest : function(otherPath, westSide)
  {
    // summary:
    //		This is the inverse of cutForEastWest. 
    // description:
    //		This is the inverse of cutForEastWest. 
    //		It combines this path with the input path again.
    // tags:
    //		private
    
    if (otherPath == null) {
      return;
    }
    if (westSide)
    {
      this.append(otherPath, otherPath.getLast());
    }
    else
    {
      var tail = this.cutPath(this);
      this.append(otherPath, otherPath.getLast());
      if (tail != null) {
        this.append(tail, tail.getLast());
      }
    }
  },

  disposeNextUntil : function(lastCell)
  {
    // summary:
    //		Delete a path from the next cell up to the input last cell
    //		(exclusively). 
    // description:
    //		Delete a path from the next cell up to the input last cell
    //		(exclusively). 
    //		Just to speed up garbage collection in java.
    // tags:
    //		private
    
    if (lastCell == this) {
      return;
    }
    var cell = this._next;
    var nextCell;

    while (cell != null && cell != lastCell)
    {
      nextCell = cell.getNext();
      cell.dispose();
      cell = nextCell;
    }

    this._next = null;
    this._last = this;
  }

}
);

return TPath;
});
