/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/QuickSort",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../tree/TGraphAlgorithm"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    GraphModelUtil,	// #6
    QuickSort,	// #7
    TranslateUtil,	// #8
    TGraphAlgorithm	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var TGraphAlgorithm = ibm_ilog.graphlayout.tree.TGraphAlgorithm;
=====*/

var TSpanTree = declare("ibm_ilog.graphlayout.tree.TSpanTree", TGraphAlgorithm, 
{
  // summary:
  //		This algorithm class contains the mechanism to calculate the spanning
  //		tree, if the graph is not a tree. 
  // description:
  //		This algorithm class contains the mechanism to calculate the spanning
  //		tree, if the graph is not a tree. 
  //		Note that at the beginning, we
  //		don't know which are the roots and which the leaves, and which are parent and which are children. Therefore, each link from A to B is represented by a child entry B in node A and as a child entry A in node B, even though it may later turn out that either one (or even both) are not in the spanning tree. Further note that there are east-west neighbors, which are stored as normal children as well as in the east-west-neighbor field. During calculation of the spanning tree, these east-west neighbors are high priority. The task is: 1) figure out what are the roots of each component (Note: we can work with disconnected components. The roots are placed in the node array as the first nodes. 2) what is the spanning tree. Remove all child entries of links that are not part of the spanning tree. Reverse all child entries of links that are part of the spanning tree in reverse direction. 3) what is the order of the children (i.e. which is the first child and which the last child). We derive the order from the coordinates, to support incremental stablility.  
  // tags:
  //		private
  
  _firstUnvisitedIndex: null,

  _intergraphLinkConnectivity: null,

  constructor : function(graph, checkIntergraphLinks)
  {
    this._firstUnvisitedIndex = 0;
    this._intergraphLinkConnectivity = checkIntergraphLinks;
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.inherited("disposeIt", arguments, []);
  },

  run : function()
  {
    // summary:
    //		Calculates the children-parents relationship of the nodes. 
    // description:
    //		Calculates the children-parents relationship of the nodes. 
    //		By a DFS, it basically calculates a spanning tree of the graph.
    // tags:
    //		private
    
    var i;
    var graph = this.getGraph();
    var numOfNodes = graph.getNumberOfNodes();
    var node;

    this.startStep(graph.getPercForSpanTree(), 7 * numOfNodes);

    for (i = 0; i < numOfNodes; i++)
    {
      node = graph.getNode(i);
      node.calcRootNodePreference();
      node.setIndex(i);
      node.sortChildren(graph, true);
      this.addPercPoints(1);
      this.layoutStepPerformed();
    }

    this._firstUnvisitedIndex = 0;
    var invisibleRoot = graph.getInvisibleRoot();
    if (invisibleRoot != null) {
      this._firstUnvisitedIndex = 1;
    }
    node = this.searchRootNode();
    var numRoots = 0;

    while (node != null)
    {
      numRoots++;

      while (node != null)
      {
        this.calcSpanningTree(node);
        node = this.searchIntergraphConnectivityContinuation();
      }

      node = this.searchRootNode();
    }

    this.resortRootNodes();

    if (invisibleRoot != null)
    {
      if (numRoots > 1)
      {
        invisibleRoot.setChildrenCapacity(numRoots);

        for (i = 1; i < numOfNodes && graph.getNode(i).getImmediateParent() == null; i++)
        {
          node = graph.getNode(i);
          invisibleRoot.addChild(node);
          node.setOriginalParent(invisibleRoot);
        }
      }
      else if (numOfNodes > 1)
      {
        graph.swapPosition(invisibleRoot, 1);
      }

      invisibleRoot.setImmediateParent(null);
    }

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();

    var isRadial = graph.isRadialLayout();

    for (i = 0; i < numOfNodes && graph.getNode(i).getImmediateParent() == null; i++)
    {
      node = graph.getNode(i);

      if (isRadial)
      {
        node.setEastWestNeighbor(0, null);
        node.setEastWestNeighbor(1, null);
      }

      node.validateEastWestNeighbor(isRadial);
      node.validateAssociates(isRadial);
    }

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();

    this.rearrangeTipOverBothSides();
    this.layoutStepPerformed();
    numOfNodes = graph.getNumberOfNodes();

    for (i = 0; i < numOfNodes; i++)
    {
      graph.getNode(i).sortChildren(graph, false);

      this.addPercPoints(1);
      this.layoutStepPerformed();
    }

    graph.initSiblingChains();

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();

    graph.calcMaxDepth();

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();
  },

  runRecursive : function()
  {
    // summary:
    //		Calculates the children-parents relationship of the nodes. 
    // description:
    //		Calculates the children-parents relationship of the nodes. 
    //		Same as run, but this does it for all nested TGraphs.
    // tags:
    //		private
    
    this.run();
    var iter = this.getGraph().getNodesThatAreSubgraphs();

    while (iter.hasNext())
    {
      var subgraphNode = iter.next();
      var subgraph = subgraphNode.getRepresentationGraph();
      if (subgraph == null) {
        continue;
      }
      var alg = new TSpanTree(subgraph, true);
      alg.runRecursive();
      alg.disposeIt();
    }
  },

  searchRootNode : function()
  {
    // summary:
    //		Searches the best unvisited root node. 
    // description:
    //		Searches the best unvisited root node. 
    //		It returns null if all nodes were visited.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var numberOfNodes = graph.getNumberOfNodes();
    var bestRootPreference = 0;
    var bestNode = null;
    var node;

    for (var i = this._firstUnvisitedIndex; i < numberOfNodes; i++)
    {
      node = graph.getNode(i);

      if (bestNode == null || node.getRootNodePreference() > bestRootPreference)
      {
        bestNode = node;
        bestRootPreference = node.getRootNodePreference();
      }
    }

    return bestNode; // ibm_ilog.graphlayout.tree.TNode
  },

  searchIntergraphConnectivityContinuation : function()
  {
    // summary:
    //		In intergraph connectivity mode, search a node that is connected to
    //		the already visited part with an intergraph connectivity. 
    // description:
    //		In intergraph connectivity mode, search a node that is connected to
    //		the already visited part with an intergraph connectivity. 
    //		Add this intergraph connectivity as child.
    // tags:
    //		private
    
    if (!this._intergraphLinkConnectivity) {
      return null; // ibm_ilog.graphlayout.tree.TNode
    }
    var graph = this.getGraph();
    var numberOfNodes = graph.getNumberOfNodes();
    var bestRootPreference = 0;
    var bestNode = null;
    var bestNodeVisitedParent = null;
    var node;
    var visitedParent;

    for (var i = 0; i < this._firstUnvisitedIndex; i++)
    {
      visitedParent = graph.getNode(i);
      var n = visitedParent.getNumberOfIntergraphConnectivityChildren();

      for (var j = 0; j < n; j++)
      {
        node = visitedParent.getIntergraphConnectivityChild(j);

        if (node.isVisited(this._firstUnvisitedIndex))
        {
          visitedParent.removeIntergraphConnectivityChild(node);
          node.removeIntergraphConnectivityChild(visitedParent);
          j--;
          n = visitedParent.getNumberOfIntergraphConnectivityChildren();
        }
        else
        {
          if (bestNode == null || node.getRootNodePreference() > bestRootPreference)
          {
            bestNode = node;
            bestNodeVisitedParent = visitedParent;
            bestRootPreference = node.getRootNodePreference();
          }
        }
      }
    }

    if (bestNode != null)
    {
      bestNodeVisitedParent.addChild(bestNode);
      bestNode.setOriginalParent(bestNodeVisitedParent);
      bestNodeVisitedParent.removeIntergraphConnectivityChild(bestNode);
      bestNode.removeIntergraphConnectivityChild(bestNodeVisitedParent);
    }

    return bestNode; // ibm_ilog.graphlayout.tree.TNode
  },

  calcSpanningTree : function(root)
  {
    // summary:
    //		Calculates the spanning tree by a BFS starting at the input root
    //		node.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var actIndex = this._firstUnvisitedIndex;
    var i;
    var node;
    var child;

    graph.swapPosition(root, this._firstUnvisitedIndex++);

    while (actIndex < this._firstUnvisitedIndex)
    {
      node = graph.getNode(actIndex++);

      this.addPercPoints(1);
      this.layoutStepPerformed();

      for (i = 0; i < node.getNumberOfChildren(); i++)
      {
        child = node.getChild(i);

        if (!child.isVisited(this._firstUnvisitedIndex))
        {
          child.setOriginalParent(node);
          graph.swapPosition(child, this._firstUnvisitedIndex++);
        }
        else
        {
          node.removeChild(child);
          i--;
        }
      }
    }
  },

  resortRootNodes : function()
  {
    // summary:
    //		Make sure that all the root nodes are first in the nodes array. 
    // description:
    //		Make sure that all the root nodes are first in the nodes array. 
    //		By that, we have later faster access to the root nodes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var numberOfNodes = graph.getNumberOfNodes();
    var firstNonRootNodeIndex = 0;
    var node;
    var i = 0;

    for (i = 0; i < numberOfNodes; i++)
    {
      node = graph.getNode(i);
      node.cleanIntergraphConnectivityChildren();
      if (node.getImmediateParent() == null) {
        graph.swapPosition(node, firstNonRootNodeIndex++);
      }
    }

    var invisibleRoot = graph.getInvisibleRoot();
    var min = ((invisibleRoot == null ? -Number.MAX_VALUE : -Number.MAX_VALUE / 2));
    var hasNodeComparator = graph.getLayout().getNodeComparator() != null;

    for (i = 0; i < firstNonRootNodeIndex; i++)
    {
      node = graph.getNode(i);
      if (node == invisibleRoot) {
        node.setSortValue(-Number.MAX_VALUE);
      }
      else {
        node.setSortValue(Math.max(min, (hasNodeComparator ? node.getSpecChildrenOrder() : node.orthogonalOrder(graph, false))));
      }
    }

    var sortAlg = new TSpanTree.RootSort(graph, firstNonRootNodeIndex);
    sortAlg.sortRoots();
    sortAlg.dispose();
  },

  rearrangeTipOverBothSides : function()
  {
    // summary:
    //		Rearrange the graph such that we have dummy nodes and east west
    //		neighbors that simulate the tip-over-both-sides alignment.
    // tags:
    //		private
    
    var graph = this.getGraph();
    if (graph.isRadialLayout()) {
      return;
    }
    if (graph.isBalloonLayout()) {
      return;
    }

    var numberOfNodes = graph.getNumberOfNodes();
    var nrDummyNodes = 0;
    var i;

    for (i = 0; i < numberOfNodes; i++)
      nrDummyNodes += this.calcTipOverBothSidesDummyNodes(graph.getNode(i));
    if (nrDummyNodes > 0) {
      graph.increaseNodeCapacity(nrDummyNodes);
    }
    for (i = 0; i < numberOfNodes; i++)
      this.createTipOverBothSidesDummyNodes(graph.getNode(i));
  },

  calcTipOverBothSidesDummyNodes : function(node)
  {
    // summary:
    //		Calculate how many dummy nodes are necessary if the input node has
    //		tip-over-both-sides alignment.
    // tags:
    //		private
    
    var numChild = node.getNumberOfChildren();
    var numAssoc = node.getNumberAssociates();
    var numEastAssoc = node.getNumberEastAssociates();
    var numWestAssoc = node.getNumberWestAssociates();
    var numEWAssoc = numAssoc - numEastAssoc - numWestAssoc;
    if (node.getEastWestNeighbor(0) != null) {
      numChild--;
    }
    if (node.getEastWestNeighbor(1) != null) {
      numChild--;
    }
    if (numChild <= 1)
    {
      if (numAssoc == 0) {
        return 0; // int
      }
    }

    var ewmax = Math.max(numEastAssoc, numWestAssoc);
    var ewmin = Math.min(numEastAssoc, numWestAssoc);
    var remain = Math.max(0, numEWAssoc - ewmax + ewmin);
    if (!node.isTipOverBothSides()) {
      return ewmax + Math.floor((remain + 1) / 2); // int
    }
    return ewmax + Math.floor((remain + 1) / 2) + Math.floor((numChild - numAssoc + 1) / 2); // int
  },

  createTipOverBothSidesDummyNodes : function (a0, a1, a2, a3, a4)
  {
    // summary:
    //		Create the dummy nodes necessary if the input node has
    //		tip-over-both-sides alignment.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.createTipOverBothSidesDummyNodes_overload_1(a0);
    else if(arguments.length == 5)
      return this.createTipOverBothSidesDummyNodes_overload_5(a0, a1, a2, a3, a4);
  },
  
  createTipOverBothSidesDummyNodes_overload_1 : function(node)
                                                              
  {
    var numberOfChildren = node.getNumberOfChildren();
    var numberChildrenNoEastWest = numberOfChildren;
    if (node.getEastWestNeighbor(0) != null) {
      numberChildrenNoEastWest--;
    }
    if (node.getEastWestNeighbor(1) != null) {
      numberChildrenNoEastWest--;
    }
    var numAssoc = node.getNumberAssociates();

    if (numberChildrenNoEastWest <= 1)
    {
      if (node.isTipOverBothSides())
      {
        node.setAlignmentStyle(AlignmentOptions.CENTER);
      }

      if (numAssoc == 0) {
        return;
      }
    }

    if (!node.isTipOverBothSides() && numAssoc == 0) {
      return;
    }

    var assocChildren = new Array();
    var tipChildren = new Array();
    var remainingChildren = new Array();
    var eastWestNeighbors = new Array();
    var child;
    var i = 0;

    for (i = 0; i < numberOfChildren; i++)
    {
      child = node.getChild(i);
      if (node.hasEastWestNeighbor(child)) {
        eastWestNeighbors.push(child);
      }
      else if (node.hasAssociate(child)) {
        assocChildren.push(child);
      }
      else if (node.isTipOverBothSides()) {
        tipChildren.push(child);
      }
      else {
        remainingChildren.push(child);
      }
    }

    if (assocChildren.length + tipChildren.length == 0) {
      return;
    }
    node.sortChildrenForTipOverBothSides(this.getGraph(), assocChildren);
    node.sortChildrenForTipOverBothSides(this.getGraph(), tipChildren);

    var n1 = 1 + eastWestNeighbors.length;
    var n2 = 2 + ((tipChildren.length == 0 ? remainingChildren.length : 1));
    var n3 = 2 + remainingChildren.length;
    var last;
    node.setChildrenCapacity(n1);

    var n = eastWestNeighbors.length;

    for (i = 0; i < n; i++)
    {
      child = eastWestNeighbors[i];
      node.addChild(child);
      child.setImmediateParent(node);
    }

    last = this.createTipOverBothSidesDummyNodes(node, assocChildren, n2, 0, true);
    last = this.createTipOverBothSidesDummyNodes(last, tipChildren, n3, 0, false);

    if (last != null)
    {
      last.setAlignmentStyle(node.getAlignmentStyle());
      node.setAlignmentStyle(AlignmentOptions.CENTER);
    }

    n = remainingChildren.length;

    for (i = 0; i < n; i++)
    {
      child = remainingChildren[i];
      last.addChild(child);
      child.setImmediateParent(last);
    }

    if (node != last) {
      node.setLastDummyTipOverBothSides(last);
    }
  },

  createTipOverBothSidesDummyNodes_overload_5 : function(parent, children, capacityLastDummy, startAtSide, useChildAlignment)
  {
    // summary:
    //		This creates the T-shape and returns the last dummy node, or the
    //		parent, if no dummy nodes are needed. 
    // description:
    //		This creates the T-shape and returns the last dummy node, or the
    //		parent, if no dummy nodes are needed. 
    //		Note that the parent has always already the correct children
    //		capacity. P | C --D-- C | C --D-- C
    // tags:
    //		private
    
    var numberOfChildren = children.length;
    if (numberOfChildren == 0) {
      return parent; // ibm_ilog.graphlayout.tree.TNode
    }

    var west = new Array();
    var east = new Array();
    var i;
    var child;

    if (useChildAlignment)
    {
      for (i = 0; i < numberOfChildren; i++)
      {
        child = children[i];

        if (child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_WEST)
        {
          west.push(child);
        }
        else if (child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_EAST)
        {
          east.push(child);
        }
      }
    }

    for (i = 0; i < numberOfChildren; i++)
    {
      child = children[i];

      if (useChildAlignment && child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_WEST)
      {
      }
      else if (useChildAlignment && child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_EAST)
      {
      }
      else if (west.length < east.length)
      {
        west.push(child);
      }
      else if (west.length > east.length)
      {
        east.push(child);
      }
      else if (startAtSide == 0)
      {
        west.push(child);
      }
      else
      {
        east.push(child);
      }
    }

    var graph = this.getGraph();
    var actDummy;
    var nextDummy = null;
    var lastDummy = parent;
    var size = 0;
    var last = false;
    var maxI = Math.max(east.length, west.length);
    var dirTowardsLeaves = graph.getDirTowardsLeaves();
    actDummy = graph.createDummyNode();
    parent.addChild(actDummy);
    actDummy.setImmediateParent(parent);
    actDummy.setAlignmentStyle(AlignmentOptions.CENTER);
    i = 0;

    while (i < maxI)
    {
      if (i == maxI - 1) {
        last = true;
      }
      lastDummy = actDummy;
      actDummy.setChildrenCapacity((last ? capacityLastDummy : 3));

      if (i < west.length)
      {
        child = west[i];
        size = child.getSize(dirTowardsLeaves);
        if (size > actDummy.getSize(dirTowardsLeaves)) {
          actDummy.setSize(dirTowardsLeaves, size);
        }
        actDummy.setSize(1 - dirTowardsLeaves, 0);
        child.setImmediateParent(actDummy);
        actDummy.addChild(child);
        actDummy.setEastWestNeighbor(0, child);
      }

      if (i < east.length)
      {
        child = east[i];
        size = child.getSize(dirTowardsLeaves);
        if (size > actDummy.getSize(dirTowardsLeaves)) {
          actDummy.setSize(dirTowardsLeaves, size);
        }
        actDummy.setSize(1 - dirTowardsLeaves, 0);
        child.setImmediateParent(actDummy);
        actDummy.addChild(child);
        actDummy.setEastWestNeighbor(1, child);
      }

      i++;

      if (!last)
      {
        nextDummy = graph.createDummyNode();
        actDummy.addChild(nextDummy);
        nextDummy.setImmediateParent(actDummy);
        actDummy.setAlignmentStyle(AlignmentOptions.CENTER);
      }

      actDummy = nextDummy;
    }

    return lastDummy; // ibm_ilog.graphlayout.tree.TNode
  }

}
);
TSpanTree.RootSort = declare(QuickSort, 
{
  // summary:
  //		Class to sort the root nodes.
  
  _numberOfElements: null,
  _graph: null,

  constructor : function(graph, numRoots)
  {
    this._graph = graph;
    this._numberOfElements = numRoots;
  },
  
  sortRoots : function()
  {
    // summary:
    //		
    // tags:
    //		private
    
    this.inherited("sort", arguments, [this._numberOfElements]);
  },

  dispose : function()
  {
  },

  compare : function(loc1, loc2)
  {
    var node1 = this._graph.getNode(loc1);
    var node2 = this._graph.getNode(loc2);
    if (node1.getSortValue() < node2.getSortValue()) {
      return -1; // int
    }
    if (node1.getSortValue() > node2.getSortValue()) {
      return 1; // int
    }
    return 0; // int
  },

  swap : function(loc1, loc2)
  {
    var node = this._graph.getNode(loc2);
    this._graph.swapPosition(node, loc1);
  }

}
);

return TSpanTree;
});
