/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "./internalutil/LogResUtil",	// #5
  "./internalutil/TranslateUtil",	// #6
  "./DefaultLayoutProvider",	// #7
  "./GraphLayout",	// #8
  "./RLayoutModes",	// #9
  "./MultipleLayout"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    LogResUtil,	// #5
    TranslateUtil,	// #6
    DefaultLayoutProvider,	// #7
    GraphLayout,	// #8
    RLayoutModes,	// #9
    MultipleLayout	// #10
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var GraphLayout = ibm_ilog.graphlayout.GraphLayout;
=====*/

var RecursiveLayout = declare("ibm_ilog.graphlayout.RecursiveLayout", GraphLayout, 
{
  // summary:
  //		The main class for the Recursive Layout algorithm. 
  // description:
  //		The main class for the Recursive Layout algorithm. 
  //		<p>
  //		This is not a layout algorithm but rather a facility to perform
  //		layouts recursively in a nested graph. Unless otherwise mentioned,
  //		the normal layout algorithms such as <code>
  //		ibm&#95;ilog.graphlayout.hierarchical.HierarchicalLayout</code>, <code>
  //		ibm&#95;ilog.graphlayout.tree.TreeLayout</code>, <code>
  //		ibm&#95;ilog.graphlayout.MultipleLayout</code>, and so forth work on a
  //		flat graph, that is, they lay out the attached graph but not the
  //		subgraphs of the attached graph. The Recursive Layout is different:
  //		it traverses the nesting structure starting from the attached graph
  //		and applies a layout recursively on all subgraphs. It can be tailored
  //		for which sublayout has to be applied to which subgraph.   
  //		</p>
  //		<p>
  //		There are basically two scenarios:  
  //		<ul>
  //		<li> The same layout style must be applied to all subgraphs.</li>  
  //		<li> An individual layout style must be applied to each
  //		subgraphs.</li>  
  //		</ul>
  //		<b>First scenario: Same layout style everywhere</b>  
  //		</p>
  //		<p>
  //		The following example shows the first scenario:   
  //		</p>
  //		<pre>
  //		GraphLayout layout = new RecursiveLayout(new TreeLayout());
  //		layout.attach(topLevelGrapher);
  //		LayoutReport report = layout.performLayout(true, true); </pre>
  //		In this case, a tree layout is performed recursively to the top-level
  //		graph and to each subgraph. All layouts are performed with the same
  //		global layout parameters as for the layout instance passed as an
  //		argument to the constructor of <code>RecursiveLayout</code>, which is
  //		called its <i>reference layout</i>. You can access the reference
  //		layout instance to change the global layout parameters:   
  //		<pre>
  //		TreeLayout treeLayout = (TreeLayout)layout.getReferenceLayout();
  //		treeLayout.setFlowDirection(Direction.Left); </pre>
  //		Internally, a clone of the reference instance is created for each
  //		subgraph. This clone remains attached as long as the Recursive Layout
  //		is attached to the top-level graph. Before layout is performed, the
  //		global layout parameters are copied from the reference instance to
  //		each clone. If you need to set layout parameters for individual nodes
  //		and links, you have to access the layout instance of the subgraph
  //		that owns the node or link:   
  //		<pre>
  //		TreeLayout treeLayout = (TreeLayout)layout.getLayout(subgraph);
  //		treeLayout.setAlignment(node, TreeLayout.TIP&#95;OVER); </pre>     
  //		<p>
  //		In the typical case, when you use instances of <code>Graphic</code>
  //		as nodes and instances of <code>Grapher</code> as subgraphs, it is:  
  //		</p>
  //		<pre>
  //		TreeLayout treeLayout = (TreeLayout)layout.getLayout(node.getGraphicBag());
  //		treeLayout.setAlignment(node, TreeLayout.TIP&#95;OVER); </pre>   
  //		<p>
  //		<b>Second scenario: Different layout styles at different
  //		subgraphs</b>  
  //		</p>
  //		<p>
  //		The following example shows the second scenario: Each subgraph should
  //		be laid out by a different layout style or with individual layout
  //		parameters. In this case, we need a layout provider (see <code>
  //		ibm&#95;ilog.graphlayout.ILayoutProvider</code>) that specifies which
  //		layout instance is used for which subgraph.   
  //		</p>
  //		<pre>
  //		GraphLayout layout = new RecursiveLayout(layoutProvider);
  //		layout.attach(topLevelGrapher);
  //		LayoutReport report = layout.performLayout(true, true); </pre>
  //		The layout provider returns a different layout instance for each
  //		subgraph. For example, it may return a grid layout for the top-level
  //		graph, but a tree layout for one subgraph and a bus layout for
  //		another subgraph of the top-level graph. Furthermore, each layout can
  //		have different global layout parameters. You can implement your own
  //		application-specific layout provider, or you can use the predefined
  //		<code> ibm&#95;ilog.graphlayout.DefaultLayoutProvider</code> or <code>
  //		RecursiveLayoutProvider</code> that allows you to specify the
  //		preferred layout of each subgraph. If neither a reference layout nor
  //		a layout provider is given to the Recursive Layout, an internal
  //		default layout provider of class <code>RecursiveLayoutProvider</code>
  //		is used. This allows you to specify the layouts of subgraphs in a
  //		very convenient way:   
  //		<pre>
  //		RecursiveLayout layout = new RecursiveLayout();
  //		layout.attach(topLevelGrapher);
  //		// specify the layout of the top-level graph
  //		layout.setLayout(null, new GridLayout());
  //		// specify the layout of subgraphs
  //		layout.setLayout(subgraph1, new TreeLayout());
  //		layout.setLayout(subgraph2, new BusLayout());
  //		// perform the layout
  //		LayoutReport report = layout.performLayout(true, true); </pre>
  //		In this scenario, there is no reference layout. All layout parameters
  //		of different subgraphs are independent. You access the layout
  //		instance of each individual subgraph in order to change global layout
  //		parameters for this subgraph as well as parameters of nodes and links
  //		of the subgraph. For instance, if the attached top-level graph
  //		contains two subgraphs, and node1 belongs to subgraph1 and node2
  //		belongs to subgraph2, you can set individual global and local layout
  //		parameters in this way:   
  //		<pre>
  //		// access the layout of the top-level graph
  //		GridLayout gridLayout = (GridLayout)layout.getLayout(null);
  //		gridLayout.setLayoutMode(GridLayout.TILE&#95;TO&#95;COLUMNS);
  //		// access the layouts of the subgraph
  //		TreeLayout treeLayout = (TreeLayout)layout.getLayout(subgraph1);
  //		treeLayout.setFlowDirection(Direction.Left);
  //		treeLayout.setAlignment(node1, TreeLayout.TIP&#95;OVER);
  //		BusLayout busLayout = (BusLayout)layout.getLayout(subgraph2);
  //		busLayout.setOrdering(BusLayout.ORDER&#95;BY&#95;HEIGHT);
  //		busLayout.setBus(node2);
  //		</pre>    
  //		<p>
  //		<b>Notes:</b>  
  //		<ul>
  //		<li> The normal layout algorithms such as <code>
  //		ibm&#95;ilog.graphlayout.hierarchical.HierarchicalLayout</code>, <code>
  //		ibm&#95;ilog.graphlayout.tree.TreeLayout</code>, <code>
  //		ibm&#95;ilog.graphlayout.MultipleLayout</code>, and so forth work only on
  //		a flat graph, but for convenience, you can apply <code>
  //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code> with the
  //		<code>traverse</code> flag set to <code>true</code>. This does not
  //		mean that the normal layout instance works on the entire nested graph
  //		including the subgraphs in this case. In fact, internally it uses a
  //		Recursive Layout in reference layout mode, that is, a clone of the
  //		top-level graph layout instance is created for each subgraph, and
  //		these clones are applied recursively to the nested graph. Each clone
  //		still treats only a flat graph.</li>  
  //		<li> It is not possible to create a Recursive Layout that has a
  //		Recursive Layout as reference layout:   
  //		<pre>
  //		layout = new RecursiveLayout(new RecursiveLayout()); </pre>
  //		This calls the copy constructor of the Recursive Layout, but does not
  //		create a Recursive Layout in reference layout mode. A Recursive
  //		Layout inside another Recursive Layout would traverse the nested
  //		graph and perform the Recursive Layout for each subgraph, which,
  //		however, itself would recursively traverse the nested graph again.
  //		This would just be waste of run time.</li>  
  //		<li> The layout event listeners installed at the Recursive Layout
  //		receive layout events whenever the layout of a subgraph fires a
  //		layout event. These events, however, do not contain the information
  //		on which sublayout was started or finished. If you need this
  //		information, you should install an event listener at all
  //		sublayouts.</li>  
  //		<li> If you want to combine <code>MultipleLayout</code> and
  //		<code>RecursiveLayout</code>, you should use <code>
  //		RecursiveMultipleLayout</code>, in particular when you use explicitly
  //		specified labeling models. The Recursive Layout that has a Multiple
  //		Layout as the reference layout works as long as default labeling
  //		models (instances of <code>
  //		ibm&#95;ilog.graphlayout.label.DefaultLabelingModel</code>) are used. The
  //		Recursive Multiple Layout is more flexible; it is in principle a
  //		Recursive Layout that has instances of Multiple Layout as sublayouts
  //		for the subgraphs.  Additionally, the Recursive Multiple Layout
  //		ensures that the correct labeling models (clones of the specified
  //		labeling model) are created in the nested graph. </li>  
  //		</ul>  
  //		</p>
  /*=====
  // REFERENCE_LAYOUT_MODE: int
  //		Layout mode with reference layout.
  REFERENCE_LAYOUT_MODE : null,
  // INTERNAL_PROVIDER_MODE: int
  //		Layout mode with internal layout provider.
  INTERNAL_PROVIDER_MODE : null,
  // SPECIFIED_PROVIDER_MODE: int
  //		Layout mode with explicitly specified provider.
  SPECIFIED_PROVIDER_MODE : null,
  =====*/
  
  constructor : function()
  {
    
  this.DEFAULT_RECURSIVE_ALLOWED_TIME = 1000000000;},

  _layoutProvider: null,
  _referenceLayout: null,

  _mode: null,
  _subgraphCorrection: null,

  _force: null,
  _subgraphsToLayout: null,
  _actLayout: null,
  _startTime: null,
  _stoppedBeforeCompletion: null,
  _oldAllowedTime: null,

  _basePercentage: null,
  _numSubLayouts: null,

  _usePercentageCompletionSubstitute: null,
  _numStepsInSublayout: null,

  _isRecursiveLayout: true,
  DEFAULT_RECURSIVE_ALLOWED_TIME : null,

  constructor : function(layout, layoutProvider)
  {
  // summary:
  //        Creates a new instance of the Recursive Layout algorithm that allows
  //        you to apply layouts to the entire nested graph.
  // description:
  //        If a layout instance is provided, this constructor creates a new
  //        recursive layout instance using the layout as a reference. In this
  //        case, the recursive layout mode is REFERENCE_LAYOUT_MODE.
  //        If the given layout is a recursive layout, this constructor creates
  //        a new recursive layout instance by copying the given one.
  //        
  //        If a layout instance is not provided, but a layout provider is passed
  //        as parameter, the recursive layout instance will have a layout mode
  //        set to SPECIFIED_PROVIDER_MODE. Therefore using the
  //        layout instances as specified by the layout provider.
  //        
  //        If neither the layout nor the layout provider are specified,
  //        the recursive layout instance created is set to INTERNAL_LAYOUT_MODE.
  // layout: ibm_ilog.graphlayout.GraphLayout:
  //        If the given layout is a recursive layout, this constructor
  //        creates a new layout instance by copying the given one. If this is another
  //        layout, this constructor creates a new instance of the Recursive Layout
  //        algorithm that allows you to apply the reference layout to the entire nested graph.
  // layoutProvider: ibm_ilog.graphlayout.ILayoutProvider:
  //        Allows to specify a different layout style or individual
  //        layout parameters to each subgraph of the nested graph. When this parameter is
  //        specified, the layout mode of this Recursive Layout is SPECIFIED_PROVIDER_MODE.
    if (layout instanceof GraphLayout &&
        layoutProvider instanceof GraphLayout){
      layout = new MultipleLayout(layout, layoutProvider);
      layoutProvider = null;
    }
    if (layout != null) {
      if (layout instanceof RecursiveLayout) {
        // Copy source layout
        this.copyParameters(layout);
        this.setParametersUpToDate(false);
      } else {
        // Use given layout as reference
        this.setLayoutMode(RecursiveLayout.REFERENCE_LAYOUT_MODE, layout, null);
      }
    } else {
      if (layoutProvider != null) {
        this.setLayoutMode(RecursiveLayout.SPECIFIED_PROVIDER_MODE, null, layoutProvider);
      } else {
        this.setLayoutMode(RecursiveLayout.INTERNAL_PROVIDER_MODE, null, null);
      }
    }
  },
  
  init : function()
  {
    // summary:
    //		Initializes instance variables. 
    // description:
    //		Initializes instance variables. 
    //		<p>
    //		You should not call this method directly. The method is called
    //		internally by the constructor without arguments and by the copy
    //		constructor. The method must be overridden by subclasses that need
    //		to initialize additional instance variables.
    //		</p>
    // tags:
    //		protected
    
    this.inherited("init", arguments, []);
    this._subgraphCorrection = null;
    this.DEFAULT_RECURSIVE_ALLOWED_TIME = 100000000;
    this.setAllowedTime(this.DEFAULT_RECURSIVE_ALLOWED_TIME);
  },

  copy : function()
  {
    // summary:
    //		Copies the layout instance. 
    // description:
    //		Copies the layout instance. 
    //		<p>
    //		This method copies the layout instance by calling the copy
    //		constructor. Note that the parameters which are specific to a node
    //		or a link are not copied. It depends on the layout mode what is
    //		copied.  
    //		<ul>
    //		<li> <code> RecursiveLayout.REFERENCE&#95;LAYOUT&#95;MODE</code> - The
    //		reference layout is copied deeply.</li>  
    //		<li> <code> RecursiveLayout.INTERNAL&#95;PROVIDER&#95;MODE</code> - The
    //		information on which subgraph uses which sublayout is not
    //		copied.</li>  
    //		<li> <code> RecursiveLayout.SPECIFIED&#95;PROVIDER&#95;MODE</code> - The
    //		specified layout provider is copied, but not deeply.</li>  
    //		</ul>  
    //		</p>
    // returns:
    //		A copy of the layout instance.
    
    return new RecursiveLayout(this); // ibm_ilog.graphlayout.GraphLayout
  },

  copyParameters : function(source)
  {
    // summary:
    //		Copies the parameters from a given layout instance. 
    // description:
    //		Copies the parameters from a given layout instance. 
    //		Note that the parameters which are specific to a node or a link are
    //		not copied. It depends on the layout mode what is copied:  
    //		<ul>
    //		<li> <code> RecursiveLayout.REFERENCE&#95;LAYOUT&#95;MODE</code> - The
    //		reference layout is copied deeply.</li>  
    //		<li> <code> RecursiveLayout.INTERNAL&#95;PROVIDER&#95;MODE</code> - The
    //		information on which subgraph uses which sublayout is not
    //		copied.</li>  
    //		<li> <code> RecursiveLayout.SPECIFIED&#95;PROVIDER&#95;MODE</code> - The
    //		specified layout provider is copied, but not deeply.</li>  
    //		</ul>  
    // source: GraphLayout
    //		The layout instance from which the parameters are copied.
    
    this.inherited("copyParameters", arguments, [source]);

    if (source instanceof RecursiveLayout)
    {
      var rsource = source;

      this._savePreferredLayouts = rsource._savePreferredLayouts;
      this._loadLayoutMode = rsource._loadLayoutMode;

      this._subgraphCorrection = rsource._subgraphCorrection;
      this._mode = rsource.getLayoutMode();

      switch (this._mode)
      {
        case RecursiveLayout.REFERENCE_LAYOUT_MODE:
          this.copyReferenceLayout(rsource._referenceLayout);

          this.initDefaultLayoutProvider();

          break;
        case RecursiveLayout.INTERNAL_PROVIDER_MODE:
          this.initRecursiveLayoutProvider();

          break;
        case RecursiveLayout.SPECIFIED_PROVIDER_MODE:
          break;
      }
    }
  },

  initDefaultLayoutProvider : function()
  {
    // summary:
    // tags:
    //		private
    
  },

  initRecursiveLayoutProvider : function()
  {
    // summary:
    // tags:
    //		private
    
  },

  setLayoutMode : function(mode, referenceLayout, layoutProvider)
  {
    // summary:
    //		Set the layout mode. 
    // description:
    //		Set the layout mode. 
    //		This is nonpublic. It is used for the save/loadParameters mechanism.
    // tags:
    //		private
    
    if (this.getGraphModel() != null) {
      this.beforeDetach();
    }
    var msg = null;
    this._mode = mode;

    switch (mode)
    {
      case RecursiveLayout.REFERENCE_LAYOUT_MODE:
        if (referenceLayout == null)
        {
          LogResUtil.logAndThrowIllegalArgExc(" RecursiveLayout ", "graphlayout.expert.message.6677E");
        }

        if (referenceLayout instanceof RecursiveLayout)
        {
          LogResUtil.logAndThrowIllegalArgExc(" RecursiveLayout ", "graphlayout.expert.message.6678E");
        }

        this._referenceLayout = referenceLayout;

        this.initDefaultLayoutProvider();

        break;
      case RecursiveLayout.INTERNAL_PROVIDER_MODE:
        this._referenceLayout = null;

        this.initRecursiveLayoutProvider();

        break;
      case RecursiveLayout.SPECIFIED_PROVIDER_MODE:
        if (layoutProvider == null)
        {
          LogResUtil.logAndThrowIllegalArgExc(" RecursiveLayout ", "graphlayout.expert.message.6676E");
        }

        this._referenceLayout = null;
        this._layoutProvider = layoutProvider;
        break;
    }
    if (this.getGraphModel() != null) {
      this.afterAttach(this.getGraphModel());
    }
  },

  getLayoutMode : function()
  {
    // summary:
    //		Returns the layout mode of the Recursive Layout. 
    // description:
    //		Returns the layout mode of the Recursive Layout. 
    //		The possible values are:  
    //		<ul>
    //		<li> <code> RecursiveLayout.REFERENCE&#95;LAYOUT&#95;MODE</code> - The same
    //		layout style with the same global layout parameters is applied to
    //		all subgraphs of the nested graph.</li>  
    //		<li> <code> RecursiveLayout.INTERNAL&#95;PROVIDER&#95;MODE</code> - The
    //		layout is applied using an internal recursive layout provider. The
    //		layout styles of individual subgraphs can be specified by <code>
    //		RecursiveLayout.setLayout()</code>.</li>  
    //		<li> <code> RecursiveLayout.SPECIFIED&#95;PROVIDER&#95;MODE</code> - The
    //		layout is applied using an explicitly specified layout
    //		provider.</li>  
    //		</ul>  
    // returns:
    //		The layout mode.
    
    return this._mode; // int
  },

  getLayoutProvider : function()
  {
    // summary:
    //		Returns the layout provider. 
    // description:
    //		Returns the layout provider. 
    //		<p>
    //		If the reference layout mode is used, this method returns an
    //		internally created instance of <code>
    //		ibm&#95;ilog.graphlayout.DefaultLayoutProvider</code>.  
    //		</p>
    //		<p>
    //		If the internal provider mode is used, this method returns an
    //		internally created instance of <code>
    //		RecursiveLayoutProvider</code>.  
    //		</p>
    //		<p>
    //		In both reference layout and internal provider modes, you should not
    //		manipulate the internally created layout provider directly.  
    //		</p>
    //		<p>
    //		If the specified provider mode is used, it returns the specified
    //		layout provider.  
    //		</p>
    // returns:
    //		The layout provider, which is the instance that is responsible
    //		for specifying the layout instance to be used for laying out each
    //		graph.
    
    return this._layoutProvider; // ibm_ilog.graphlayout.ILayoutProvider
  },

  getReferenceLayout : function()
  {
    // summary:
    //		Returns the reference layout if the reference layout mode is used;
    //		returns 'null' otherwise. 
    // description:
    //		Returns the reference layout if the reference layout mode is used;
    //		returns <code>null</code> otherwise. 
    //		<p>
    //		In reference layout mode, the reference layout is used to lay out
    //		the top-level graph. Clones of the reference layout are used to lay
    //		out the subgraphs. The entire nested graph is laid out with the
    //		global layout parameters of the reference layout. If you change
    //		global layout parameters of the reference layout, this will affect
    //		the layout of subgraphs as well, because the global layout
    //		parameters are copied from the reference layout to the layouts of
    //		the subgraphs.  
    //		</p>
    // returns:
    //		The reference layout if the reference layout mode is used.
    
    return this._referenceLayout; // ibm_ilog.graphlayout.GraphLayout
  },

  copyReferenceLayout : function(sourceReferenceLayout)
  {
    // summary:
    //		Copies the input reference layout.
    // tags:
    //		private
    
    if (sourceReferenceLayout == this._referenceLayout) {
      return;
    }
    if (sourceReferenceLayout == null)
    {
      if (this._referenceLayout != null) {
        this._referenceLayout.detach();
      }
      this._referenceLayout = null;
    }
    else
    {
      if (this._referenceLayout == null) {
        this._referenceLayout = sourceReferenceLayout.copy();
      }
      else if (sourceReferenceLayout.declaredClass == this._referenceLayout.declaredClass) {
        this._referenceLayout.copyParameters(sourceReferenceLayout);
      }
      else
      {
        var tmodel = this._referenceLayout.getGraphModel();
        this._referenceLayout.detach();
        this._referenceLayout = sourceReferenceLayout.copy();
        if (tmodel != null) {
          this._referenceLayout.attach(tmodel);
        }
      }
    }
  },

  setLayout : function(subgraph, layout)
  {
    // summary:
    //		Sets the layout instance to be used for the input subgraph. 
    // description:
    //		Sets the layout instance to be used for the input subgraph. 
    //		If <code>null</code> is passed as the subgraph, the input layout is
    //		used for the top-level graph. If <code>null</code> is passed as the
    //		layout, no layout will be performed for the corresponding subgraph. 
    //		
    //		<p>
    //		You can use this method only in internal provider mode. You must
    //		attach a grapher or graph model first before using this method. Each
    //		subgraph must get a different layout instance, that is, the layout
    //		instances cannot be shared among different subgraphs. The input
    //		layout is automatically attached and detached by the Recursive
    //		Layout as needed.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // layout: GraphLayout
    //		The layout instance used to lay out the subgraph. This
    //		should be a new layout instance that is not used anywhere else.
    
    this.setLayout(subgraph, layout, true, false);
  },

  getLayout : function(subgraph)
  {
    // summary:
    //		Returns the layout instance to be used for the input subgraph. 
    // description:
    //		Returns the layout instance to be used for the input subgraph. 
    //		If <code>null</code> is passed as the subgraph, the layout instance
    //		of the top-level graph is returned. You can use this method in all
    //		layout modes. You must attach a  graph model first before using this
    //		method. The method may return <code>null</code> if no layout should
    //		be performed for the subgraph.  
    //		<p>
    //		In reference layout mode, changing global layout parameters of the
    //		layout instances of subgraphs is useless, because during layout, the
    //		global parameters of the reference layout are used for each
    //		subgraph.  
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // returns:
    //		The layout instance used to lay out the subgraph.
    
    var model = this.getGraphModel();

    if (model == null)
    {
      var name = "getLayout";
      var params = [name ];
      LogResUtil.logAndThrowRuntimeExc(" RecursiveLayout ", "graphlayout.expert.message.6680E", params);
    }

    var provider = this.getLayoutProvider();
    var submodel;
    var result;

    if (subgraph == null || subgraph == this.getGraphModel())
    {
      submodel = model;

      var flag = TranslateUtil.GraphModelData.getData(model).internalGraphModelChecking;

      try
      {
        TranslateUtil.GraphModelData.setInternalGraphModelChecking(model, false);
        result = this.getCleanLayout(submodel, provider.getGraphLayout(submodel));
      }
      finally
      {
        TranslateUtil.GraphModelData.setInternalGraphModelChecking(model, flag);
      }
    }
    else
    {
      submodel = model.getGraphModel(subgraph);
      result = this.getCleanLayout(submodel, provider.getGraphLayout(submodel));
    }

    return result; // ibm_ilog.graphlayout.GraphLayout
  },

  getLayouts : function(preOrder)
  {
    // summary:
    //		Returns the instances of 'GraphLayout' for the nested
    //		graph encapsulated by the graph model of this layout instance. 
    // description:
    //		Returns the instances of <code>GraphLayout</code> for the nested
    //		graph encapsulated by the graph model of this layout instance. 
    //		You can use this method in all layout modes. You must attach a 
    //		graph model before using this method.  
    //		<p>
    //		This method returns the layout instance for the top-level graph and
    //		recursively for all subgraphs. The order of the enumeration can be
    //		preorder (that is, the layout of the parent graph comes before the
    //		layout of the subgraphs) or postorder (that is, the layout of the
    //		subgraphs comes before the layout of the parent graph).  
    //		</p>
    //		<p>
    //		In reference layout mode, changing global layout parameters of the
    //		layout instances of subgraphs is useless, because during layout, the
    //		global parameters of the reference layout are used for each
    //		subgraph.  
    //		</p>
    // preOrder: Boolean
    //		If 'true', the layout instances are returned
    //		in preorder, otherwise in postorder.
    // returns:
    //		The instances of 'GraphLayout' for the nested
    //		graph encapsulated by the graph model of this layout instance.
    
    return this.getLayoutsImpl(preOrder); // ibm_ilog.graphlayout.IIterator
  },

  getCleanLayout : function(model, layout)
  {
    // summary:
    //		Cleans the mess in reference layout mode. 
    // description:
    //		Cleans the mess in reference layout mode. 
    //		Normally this is not necessary, if people would follow the
    //		documentation. But just to be sure nobody has messed up in
    //		REFERENCE&#95;LAYOUT&#95;MODE, we make sure that the layout has the same
    //		global parameters as the reference layout.
    // tags:
    //		private
    
    if (this.getLayoutMode() == RecursiveLayout.REFERENCE_LAYOUT_MODE)
    {
      if (layout == null || !layout.declaredClass == this.getReferenceLayout().declaredClass)
      {
        if (model == null && layout != null) {
          model = layout.getGraphModel();
        }
        layout = this.getReferenceLayout().copy();
        var provider = this.getLayoutProvider();
        if (model != null) {
          provider.setPreferredLayout(model, layout);
        }
      }
      else if (layout != this.getReferenceLayout()) {
        layout.copyParameters(this.getReferenceLayout());
      }
    }

    return layout; // ibm_ilog.graphlayout.GraphLayout
  },

  attach : function(graphModel)
  {
    // summary:
    //		Allows you to specify the top-level graph model of the nested graph
    //		you want to lay out. 
    // description:
    //		Allows you to specify the top-level graph model of the nested graph
    //		you want to lay out. 
    //		<p>
    //		In addition to the functionality of the base class, the Recursive
    //		Layout prepares and attaches the reference layout in layout mode
    //		<code> RecursiveLayout.REFERENCE&#95;LAYOUT&#95;MODE</code>.  
    //		</p>
    // graphModel: AbstractGraphLayoutModel
    //		The graph model.
    
    this.inherited("attach", arguments, [graphModel]);
    this.afterAttach(graphModel);
  },

  afterAttach : function(graphModel)
  {
    // summary:
    //		In reference layout mode, attach the reference layout.
    // tags:
    //		private
    
    if (this.getLayoutMode() == RecursiveLayout.REFERENCE_LAYOUT_MODE)
    {
      var refLayout = this.getReferenceLayout();
      var provider = this.getLayoutProvider();

      var flag = TranslateUtil.GraphModelData.getData(graphModel).internalGraphModelChecking;

      try
      {
        TranslateUtil.GraphModelData.setInternalGraphModelChecking(graphModel, false);
        provider.setPreferredLayout(graphModel, refLayout);
      }
      finally
      {
        TranslateUtil.GraphModelData.setInternalGraphModelChecking(graphModel, flag);
      }

      if (refLayout.getGraphModel() != graphModel)
      {
        if (refLayout.getGraphModel() != null) {
          refLayout.detach();
        }
        refLayout.attach(graphModel);
      }
    }
  },

  detach : function()
  {
    // summary:
    //		Detaches the graph model from the layout instance. 
    // description:
    //		Detaches the graph model from the layout instance. 
    //		When you attach a new graph model to the layout instance, you do not
    //		need to detach the old graph model because this is done
    //		automatically when you call <code> RecursiveLayout.attach()</code>.
    //		The <code>detach</code> method performs cleaning operations on the
    //		graph model. In addition to the cleaning operations in the base
    //		class, the Recursive Layout detaches the sublayouts of subgraphs. In
    //		internal provider mode, it also cleans all settings of layout
    //		instances for subgraphs.  
    //		<p>
    //		Note that you must call this method when you no longer need the
    //		layout instance. Otherwise, some objects may not be garbage
    //		collected.  
    //		</p>
    
    this.beforeDetach();
    this.inherited("detach", arguments, []);
  },

  beforeDetach : function()
  {
    // summary:
    //		Detach the sublayouts if necessary.
    // tags:
    //		private
    
    var model = this.getGraphModel();

    if (model != null)
    {
      var provider = this.getLayoutProvider();

      if (provider != null && provider instanceof DefaultLayoutProvider) {
        (provider).detachLayouts(model, true);
      }
      else
      {
        var e = this.getLayoutsImpl(true);

        while (e.hasNext())
        {
          var layout = e.next();
          if (layout != null && layout.getGraphModel() != null) {
            layout.detach();
          }
        }
      }
    }
  },

  performLayout : function(force, traverse)
  {
    // summary:
    //      This is different from the implementation of the base class; the Recursive
    //      Layout always ignores the <code>traverse</code> flag because the layout is
    //      always done recursively.
    this._force = force;
    return this.inherited("performLayout", arguments, [force, false]);
  },
  
  performSubgraphLayout : function(subgraph, force, traverse)
  {
  // summary:
  //        Starts the layout algorithm with a subgraph.
  // description:
  //        Layout is performed for <code>subgraph</code> and, the <code>traverse</code>
  //        flag is set to <code>true</code> recursively for all subgraphs
  //        contained in this <code>subgraph</code>. If the <code>subgraph</code> is set
  //        to <code>null</code>, the layout is started at the top level graph.
  // subgraph:
  //        The subgraph to start the layout.
  // force:
  //        If <code>true</code>, the method <code>isLayoutNeeded</code>
  //        is not called. No check is made to determine if it is necessary
  //        to perform the layout.
  // traverse:
  //        If <code>true</code>, the layout is applied recursively
  //        to all subgraphs contained in the subgraph.
  //        Otherwise, it is only applied on the input subgraph.
  // returns:
  //        The report with information about the execution of
  //        the graph layout in the given subgraph.
    this.collectSubgraphsToLayout(subgraph, traverse);
    this.performLayout(force);
    this._subgraphsToLayout = null;
    return this.getLayoutReport();
  },
  
  performSublayout : function(subgraph, layout, force)
  {
  // summary:
  //        Starts the input layout algorithm.
  // description:
  //        This method is used when this layout controls the input layout as
  //        a sublayout.
  //        Layout classes can override this method if changes are needed with
  //        respect to the way in which the input layout is started.
  //        You should not call this method directly.
  //        In addition to the functionality of the base class, the Recursive
  //        Layout prepares the sublayouts according to the layout mode.
  // subgraph:
  //        The subgraph if used during nested layout, or <code>null</code>.
  // layout:
  //        The sublayout to be performed.
  // force:
  //        If <code>true</code>, no check is made to determine if
  //        it is necessary to perform the layout.
  // returns:
  //        The layout code. See <code>GraphLayoutReport.getCode()</code>.
    if(this._callSuper_performSublayout)
      return this.inherited("performSublayout", arguments, [subgraph, layout, force, true]);
    return this.performSublayoutImpl(subgraph, layout, force, true);
  },
  
  mayContinue : function()
  {
    // summary:
    //		Returns true if layout should continue.
    // tags:
    //		private
    
    if (this._stoppedBeforeCompletion) {
      return false; // Boolean
    }
    if (this.isLayoutTimeElapsed() || this.isStoppedImmediately())
    {
      this._stoppedBeforeCompletion = true;
      return false; // Boolean
    }

    return true; // Boolean
  },

  setSubgraphCorrectionInterface : function(ifc)
  {
    // summary:
    //		Sets the correction strategy for subgraphs. 
    // description:
    //		Sets the correction strategy for subgraphs. 
    //		The method <code> ISubgraphCorrection.correct()</code> of the
    //		subgraph correction interface is called immediately after the layout
    //		of the subgraph has finished. This allows you, for instance, to
    //		correct the position of the subgraph after its contents have been
    //		laid out.  
    //		<p>
    //		If all subgraphs are laid out completely, and the layout mode of all
    //		layouts in the nested graph is not incremental, then you do not need
    //		to install any subgraph correction interface. However, in the
    //		following situations, it is useful to install a subgraph correction
    //		interface:  
    //		<ul>
    //		<li> The subgraph is laid out, but its parent graph is not laid out
    //		(because no layout is specified for the parent graph).</li>  
    //		<li> The subgraph is laid out, but the subgraph is specified as
    //		fixed inside the parent graph (for instance by <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.isFixed()</code>).</li>  
    //		<li> The subgraph and its parent graph are laid out, but the layout
    //		of the parent graph uses an incremental mode that analyzes the old
    //		position of the subgraph (see for instance <code>
    //		ibm&#95;ilog.graphlayout.tree.TreeLayout.setIncrementalMode()</code> or
    //		<code>
    //		ibm&#95;ilog.graphlayout.hierarchical.HierarchicalLayout.setIncrementalMode()</code>).
    //		</li>  
    //		</ul>
    //		If subgraphs of type <code> ibm&#95;ilog.diagram.Graph</code> are used,
    //		the layout of the subgraph affects the bounding box of the subgraph.
    //		The subgraph may appear at a completely different position after
    //		layout, even though no layout of the parent graph was performed that
    //		contains the subgraph as a node, and even though <code>
    //		ibm&#95;ilog.diagram.Graph.moveObject()</code> was never called for the
    //		subgraph. The reason for this effect is simply the internal bounding
    //		box mechanism of <code> ibm&#95;ilog.diagram.Graph</code>. This effect
    //		is very unfortunate in the situations that were described above.
    //		There are two predefined subgraph correction strategies that help to
    //		eliminate this effect: <code>
    //		SubgraphCorrectionBarycenterFixed</code> and <code>
    //		SubgraphCorrectionBoundsFixed</code>.   
    //		</p>
    
    if (this._subgraphCorrection != ifc)
    {
      this._subgraphCorrection = ifc;
      this.onParameterChanged("SubgraphCorrectionInterface");
    }
  },

  getSubgraphCorrectionInterface : function()
  {
    // summary:
    //		Returns the subgraph correction interface that is applied after the
    //		nodes and links of the subgraph were laid out. 
    // description:
    //		Returns the subgraph correction interface that is applied after the
    //		nodes and links of the subgraph were laid out. 
    //		<p>
    //		It returns <code>null</code> if none is specified.  
    //		</p>
    
    return this._subgraphCorrection; // ibm_ilog.graphlayout.ISubgraphCorrection
  },

  setAllowedTime : function(time)
  {
    // summary:
    //		Sets the upper limit for the duration of the layout algorithm. 
    // description:
    //		Sets the upper limit for the duration of the layout algorithm. 
    //		<p>
    //		The default value for Recursive layout is <code>1000000000</code>
    //		(1000000 seconds). The allowed time is additionally limited by the
    //		allowed time parameter on the sublayouts.  
    //		</p>
    // time: Number
    //		The allowed time in milliseconds.
    
    this.inherited("setAllowedTime", arguments, [time]);
  },

  getAllowedTime : function()
  {
    // summary:
    //		Returns the currently allowed time for the layout algorithm in
    //		milliseconds. 
    // description:
    //		Returns the currently allowed time for the layout algorithm in
    //		milliseconds. 
    //		The allowed time is additionally limited by the allowed time
    //		parameter on the sublayouts. The effective allowed time is the
    //		minimum of the sum of allowed times for for the sublayouts and the
    //		value returned as allowed time for this Recursive layout.  
    //		<p>
    //		Note that the method <code>performLayout</code> does not
    //		automatically stop the layout when the allowed time is exceeded. It
    //		only delegates  the stop to the sublayouts, and it is entirely the
    //		responsibility of the implementation of the method <code>
    //		RecursiveLayout.layout()</code> on the sublayouts to do this.  
    //		</p>
    
    return (false ? this.DEFAULT_RECURSIVE_ALLOWED_TIME : this.inherited("getAllowedTime", arguments, [])); // Number
  },

  setMinBusyTime : function(time)
  {
    // summary:
    //		Sets the minimal time the layout algorithm can be busy between two
    //		calls of '
    //		ibm_ilog.graphlayout.GraphLayout.onLayoutStepPerformed()' when
    //		the method '
    //		ibm_ilog.graphlayout.GraphLayout.callLayoutStepPerformedIfNeeded()'
    //		is used. 
    // description:
    //		Sets the minimal time the layout algorithm can be busy between two
    //		calls of <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.onLayoutStepPerformed()</code> when
    //		the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.callLayoutStepPerformedIfNeeded()</code>
    //		is used. 
    //		In reference layout mode, it also calls <code>setMinBusyTime</code> 
    //		at the reference layout, hence all layouts of subgraphs will use
    //		this parameter setting. If a graph is attached and the layout mode
    //		is the internal or specified provider mode, it traverses the nesting
    //		hierarchy and calls <code>setMinBusyTime</code>  for all layout
    //		instances of subgraphs. However, if you insert subgraphs after
    //		calling this method, or if you change the layout instances of
    //		subgraphs in your specified provider or via <code>
    //		RecursiveLayout.setLayout()</code> in the internal provider after
    //		calling this method, the new layouts of these subgraphs may have a
    //		different setting.  
    //		<p>
    //		Warning: In internal or specified provider mode, this method does
    //		not throw an exception when no graph model is yet attached. However,
    //		if no graph model is yet attached, the method cannot traverse any
    //		nesting hierarchy.  
    //		</p>
    //		<p>
    //		The default value is <code>50</code> (milliseconds). 
    //		</p>
    
    this.inherited("setMinBusyTime", arguments, [time]);

    if (this.getLayoutMode() == RecursiveLayout.REFERENCE_LAYOUT_MODE) {
      this.getReferenceLayout().setMinBusyTime(time);
    }
    else if (this.getGraphModel() != null)
    {
      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        var layout = e.next();
        if (layout != null) {
          layout.setMinBusyTime(time);
        }
      }
    }
  },

  setLayoutRunning : function(running, fromParents)
  {
    // summary:
    //		Sets whether layout is running. 
    // description:
    //		Sets whether layout is running. 
    //		This method is part of IBM ILOG Dojo Diagrammer internals. You
    //		should not use it.   
    // running: Boolean
    //		Whether layout is running or not.
    // fromParents: Boolean
    //		Whether this notification is from a parent layout or
    //		from this layout instance.
    
    this.inherited("setLayoutRunning", arguments, [running, fromParents]);

    if (this.getGraphModel() != null)
    {
      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        var layout = e.next();
        if (layout != null) {
          layout.setLayoutRunning(running, true);
        }
      }
    }
  },

  contentsChanged : function(event)
  {
    // summary:
    //		Called when the structure or the geometry of the graph changes. 
    // description:
    //		Called when the structure or the geometry of the graph changes. 
    //		This method is the implementation of the <code>
    //		ibm&#95;ilog.graphlayout.GraphModelListener</code> interface.  
    // event: GraphLayoutModelEvent
    //		The event which describes the change that has occurred in
    //		the graph model.
    
    this.contentsChangedImpl(event.graphModel, event);
  },

  setInputCheckEnabled : function(enable)
  {
    // summary:
    //		Sets whether checks are enabled for the  nodes, links, and/or labels
    //		  provided as arguments for the different methods of this layout
    //		algorithm and its sublayout algorithms. 
    // description:
    //		Sets whether checks are enabled for the  nodes, links, and/or labels
    //		  provided as arguments for the different methods of this layout
    //		algorithm and its sublayout algorithms. 
    //		<p>
    //		In reference layout mode, it also calls
    //		<code>setInputCheckEnabled</code>  at the reference layout, hence
    //		all layouts of subgraphs will use this parameter setting. If a graph
    //		is attached and the layout mode is the internal or specified
    //		provider mode, it traverses the nesting hierarchy and calls
    //		<code>setInputCheckEnabled</code>  for all layout instances of
    //		subgraphs. However, if you insert subgraphs after calling this
    //		method, or if you change the layout instances of subgraphs in your
    //		specified provider or via <code> RecursiveLayout.setLayout()</code>
    //		in the internal provider after calling this method, the new layouts
    //		of these subgraphs may have a different setting.  
    //		</p>
    //		<p>
    //		Warning: In internal or specified provider mode, this method does
    //		not throw an exception when no graph model is yet attached. However,
    //		if no graph model is yet attached, the method cannot traverse any
    //		nesting hierarchy.  
    //		</p>
    //		<p>
    //		It is enabled by default. 
    //		</p>
    
    this.inherited("setInputCheckEnabled", arguments, [enable]);

    if (this.getLayoutMode() == RecursiveLayout.REFERENCE_LAYOUT_MODE) {
      this.getReferenceLayout().setInputCheckEnabled(enable);
    }
    else if (this.getGraphModel() != null)
    {
      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        var layout = e.next();
        if (layout != null) {
          layout.setInputCheckEnabled(enable);
        }
      }
    }
  },

  stopImmediately : function()
  {
    // summary:
    //		Stops the running layout algorithm as soon as possible. 
    // description:
    //		Stops the running layout algorithm as soon as possible. 
    //		This works only if all layouts of all subgraphs support stopping the
    //		layout immediately. If the currently running sublayout of a subgraph
    //		does not support this feature, the sublayout will run to completion,
    //		but the layouts of other subgraphs will not be started.   
    //		<p>
    //		This method can be used if multiple threads are used for layout and
    //		GUI control. The GUI control thread calls this method to notify the
    //		layout thread that the layout run must be stopped. The layout
    //		algorithm will perform some final cleanup operations before
    //		terminating. Therefore, the layout thread will continue until the
    //		cleanup operations are finished. The GUI thread, however, returns
    //		immediately from this method.   
    //		</p>
    //		<p>
    //		If the layout algorithm is stopped before completion, the result
    //		code of the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code>.  
    //		</p>
    // returns:
    //		'true' if the algorithm can be stopped at this time.
    
    var result = this.inherited("stopImmediately", arguments, []);

    if (this.getGraphModel() != null)
    {
      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        var layout = e.next();

        if (layout != null && layout.isLayoutRunning())
        {
          result = layout.stopImmediately() && result;
        }
      }
    }

    return (result); // Boolean
  },

  setAutoCheckAppropriateLinksEnabled : function(enable)
  {
    // summary:
    //		Sets whether the layout checks whether links are appropriate. 
    // description:
    //		Sets whether the layout checks whether links are appropriate. 
    //		This is internally used. You should not use it.  
    // returns:
    //		The previous value.
    
    var e = this.getLayoutsImpl(true);

    while (e.hasNext())
    {
      var layout = e.next();
      if (layout != null) {
        layout.setAutoCheckAppropriateLinksEnabled(enable);
      }
    }

    return this.inherited("setAutoCheckAppropriateLinksEnabled", arguments, [enable]); // Boolean
  },

  isParametersUpToDate : function()
  {
    // summary:
    //		Returns 'false' if at least one parameter was modified
    //		since the last time the layout was successfully performed on the
    //		same graph or if the layout has never been performed successfully on
    //		the same graph. 
    // description:
    //		Returns <code>false</code> if at least one parameter was modified
    //		since the last time the layout was successfully performed on the
    //		same graph or if the layout has never been performed successfully on
    //		the same graph. 
    //		It returns <code>true</code> if no changes occurred.  
    //		<p>
    //		The method returns <code>false</code> if the parameters of one of
    //		the sublayouts is out-of-date.  
    //		</p>
    
    if (this.getGraphModel() != null)
    {
      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        var layout = e.next();
        if (layout != null && !layout.isParametersUpToDate()) {
          return false; // Boolean
        }
      }
    }

    return this.inherited("isParametersUpToDate", arguments, []); // Boolean
  },

  isStructureUpToDate : function()
  {
    // summary:
    //		Returns 'false' if at least one modification occurred in
    //		the structure of the graph since the last time the layout was
    //		successfully performed on the same graph or if the layout has never
    //		been performed successfully on the same graph. 
    // description:
    //		Returns <code>false</code> if at least one modification occurred in
    //		the structure of the graph since the last time the layout was
    //		successfully performed on the same graph or if the layout has never
    //		been performed successfully on the same graph. 
    //		Returns <code>true</code> if no changes occurred.  
    //		<p>
    //		The method returns <code>false</code> if the structure of one of the
    //		sublayouts is out-of-date.  
    //		</p>
    
    if (this.getGraphModel() != null)
    {
      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        var layout = e.next();
        if (layout != null && !layout.isStructureUpToDate()) {
          return false; // Boolean
        }
      }
    }

    return this.inherited("isStructureUpToDate", arguments, []); // Boolean
  },

  isGeometryUpToDate : function()
  {
    // summary:
    //		Returns 'false' if at least one node or link was moved or
    //		reshaped since the last time the layout was successfully performed
    //		on the same graph or if the layout has never been performed
    //		successfully on the same graph. 
    // description:
    //		Returns <code>false</code> if at least one node or link was moved or
    //		reshaped since the last time the layout was successfully performed
    //		on the same graph or if the layout has never been performed
    //		successfully on the same graph. 
    //		It returns <code>true</code> if no changes occurred.  
    //		<p>
    //		The method returns <code>false</code> if the geometry of one of the
    //		sublayouts is out-of-date.  
    //		</p>
    
    if (this.getGraphModel() != null)
    {
      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        var layout = e.next();
        if (layout != null && !layout.isGeometryUpToDate()) {
          return false; // Boolean
        }
      }
    }

    return this.inherited("isGeometryUpToDate", arguments, []); // Boolean
  },

  supportsPercentageComplete : function()
  {
    // summary:
    //		Indicates whether the layout class can estimate the percentage of
    //		completion during the run of the layout. 
    // description:
    //		Indicates whether the layout class can estimate the percentage of
    //		completion during the run of the layout. 
    //		If all layouts of subgraphs support this feature, the percentage
    //		value is calculated from the sublayouts.  
    //		<p>
    //		If some sublayouts do not support this feature, the Recursive Layout
    //		algorithm can indicate by discrete percentage values how many
    //		layouts of subgraphs are already finished. For instance, if the
    //		nesting hierarchy contains five graphs, the percentage will increase
    //		in fixed steps by 20%.  
    //		</p>
    //		<p>
    //		The calculated percentage values are not very precise.  
    //		</p>
    
    return true; // Boolean
  },

  supportsStopImmediately : function()
  {
    // summary:
    //		Indicates whether the layout class can immediately interrupt the
    //		current run of the layout in a controlled way. 
    // description:
    //		Indicates whether the layout class can immediately interrupt the
    //		current run of the layout in a controlled way. 
    //		<p>
    //		In principle, the Recursive Layout supports this feature. If,
    //		however, the sublayout of a subgraph is running and does not support
    //		this feature, then the sublayout runs to completion first before the
    //		entire nested layout is stopped.  
    //		</p>
    //		<p>
    //		If the layout algorithm is stopped before completion, the result
    //		code of the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code>.  
    //		</p>
    
    return true; // Boolean
  },

  supportsAllowedTime : function()
  {
    // summary:
    //		Indicates whether the layout class can stop the layout computation
    //		when a user-defined allowed time is exceeded. 
    // description:
    //		Indicates whether the layout class can stop the layout computation
    //		when a user-defined allowed time is exceeded. 
    //		<p>
    //		In principle, the Recursive Layout supports this feature. If,
    //		however, the sublayout of a subgraph is running and does not support
    //		this feature, then the sublayout runs to completion first before the
    //		entire nested layout is stopped when the allowed time has elapsed.  
    //		</p>
    //		<p>
    //		If the layout algorithm is stopped before completion, the result
    //		code of the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code>.  
    //		</p>
    
    return true; // Boolean
  }

}
);

lang.mixin(RecursiveLayout,
{
  REFERENCE_LAYOUT_MODE : null,

  INTERNAL_PROVIDER_MODE : null,

  SPECIFIED_PROVIDER_MODE : null,
  _fields_init : function(c)
  {
    c = c || RecursiveLayout;
    if(GraphLayout._fields_init)
      GraphLayout._fields_init(c);
    
    c.REFERENCE_LAYOUT_MODE = RLayoutModes.REFERENCE_LAYOUT_MODE;
    c.INTERNAL_PROVIDER_MODE = RLayoutModes.INTERNAL_PROVIDER_MODE;
    c.SPECIFIED_PROVIDER_MODE = RLayoutModes.SPECIFIED_PROVIDER_MODE;
  }
  
}
);

RecursiveLayout._getGlobalProperties = function()
{
  // tags:
  //		private
  return GraphLayout._getGlobalProperties();
};

RecursiveLayout._getNodeProperties = function()
{
  // tags:
  //		private
  return GraphLayout._getNodeProperties();
};

RecursiveLayout._getLinkProperties = function()
{
  // tags:
  //		private
  return GraphLayout._getLinkProperties();
};

(function(){ RecursiveLayout._fields_init(); })();

return RecursiveLayout;
});
