/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "./internalutil/LayoutUtil",	// #5
  "./internalutil/LogResUtil",	// #6
  "./internalutil/TranslateUtil",	// #7
  "./MultipleLayout",	// #8
  "./RecursiveLayout"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    LayoutUtil,	// #5
    LogResUtil,	// #6
    TranslateUtil,	// #7
    MultipleLayout,	// #8
    RecursiveLayout	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var RecursiveLayout = ibm_ilog.graphlayout.RecursiveLayout;
=====*/

var RecursiveMultipleLayout = declare("ibm_ilog.graphlayout.RecursiveMultipleLayout", RecursiveLayout, 
{
  // summary:
  //		The main class for the Recursive Multiple Layout algorithm. 
  // description:
  //		The main class for the Recursive Multiple Layout algorithm. 
  //		<p>
  //		This is not a layout algorithm but rather a facility to perform
  //		multiple layouts recursively in a nested graph. In principle, this is
  //		just the combination of Recursive Layout and  Multiple Layout.   
  //		</p>
  //		<p>
  //		Unless otherwise mentioned, the normal layout algorithms such as
  //		<code> ibm&#95;ilog.graphlayout.hierarchical.HierarchicalLayout</code>,
  //		<code> ibm&#95;ilog.graphlayout.tree.TreeLayout</code>, <code>
  //		ibm&#95;ilog.graphlayout.MultipleLayout</code>, and so forth work on a
  //		flat graph, that is, they lay out the attached graph but not the
  //		subgraphs of the attached graph. Similarly, label layout algorithms
  //		work only on flat managers, not on nested graphs. The Recursive
  //		Multiple Layout is different: it traverses the nesting structure
  //		starting from the attached graph and recursively applies a first
  //		graph layout, then a second graph layout, and  finally a label layout
  //		on all subgraphs. It can be tailored for which sublayouts have to be
  //		applied to which subgraph.  
  //		</p>
  //		<p>
  //		The Recursive Multiple Layout is a Recursive Layout (see <code>
  //		RecursiveLayout</code>) where all layouts of subgraphs are Multiple
  //		Layouts (see <code> ibm&#95;ilog.graphlayout.MultipleLayout</code>).  
  //		</p>
  //		<p>
  //		For instance, assume that you want to apply to each subgraph a Tree
  //		layout, then a Link layout for the nontree links, and finally  a
  //		Label layout for the link labels. It would be unfortunate to call
  //		<code> ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>
  //		multiple times to achieve this, because the tree layout of the parent
  //		graph  would be finished before the link layout of its subgraph has
  //		started; hence the link layout of the subgraph invalidates the tree
  //		layout of the parent graph again. To avoid this effect, the   Tree
  //		layout, Link layout, and Label layout   of the subgraph should be
  //		finished before any layout of the parent graph has started. The
  //		following examples shows how to achieve this by using the Recursive
  //		Multiple Layout class. There are basically two scenarios:  
  //		<ul>
  //		<li> The same set of layout algorithms must be applied to all
  //		subgraphs.</li>  
  //		<li> An individual layout style must be applied to each
  //		subgraph.</li>  
  //		</ul>
  //		<b>First scenario: Same layout styles everywhere</b> 
  //		</p>
  //		<p>
  //		To perform a Tree Layout, a Link Layout, and an Annealing Label
  //		Layout on a nested graph, call:  
  //		</p>
  //		<pre>
  //		GraphLayout layout = new MultipleRecursiveLayout(
  //		                              new TreeLayout(),
  //		                              new LinkLayout(),
  //		                              new AnnealingLabelLayout());
  //		layout.attach(grapher);
  //		layout.performLayout(true, true); </pre>
  //		If you simply want to apply a label layout in a nested grapher, call:
  //		 
  //		<pre>
  //		GraphLayout layout = new MultipleRecursiveLayout(
  //		                              new AnnealingLabelLayout());
  //		layout.attach(grapher);
  //		layout.performLayout(true, true);
  //		</pre>
  //		In this case, each subgraph is treated bottom-up. First a tree layout
  //		is performed on the subgraph. Then a link layout is performed on the
  //		subgraph, finally the label layout is performed  on the subgraph. All
  //		subgraphs of the nested graph including the top-level graph are
  //		treated in this way. All layouts are performed with the same global
  //		layout parameters as the layout instances passed as arguments to the
  //		constructor of <code>RecursiveMultipleLayout</code>, which are called
  //		the <i>reference layouts</i>.  You can access the reference layout
  //		instances to change the global  layout parameters:   
  //		<pre>
  //		TreeLayout treeLayout =
  //		    (TreeLayout)layout.getFirstReferenceGraphLayout();
  //		treeLayout.setFlowDirection(Direction.Left);
  //		LinkLayout linkLayout =
  //		    (LinkLayout)layout.getSecondReferenceGraphLayout();
  //		linkLayout.setLinkOffset(5f);
  //		AnnealingLabelLayout labelLayout =
  //		    (AnnealingLabelLayout)layout.getReferenceLabelLayout();
  //		labelLayout.setObstacleOffset(20f); </pre>
  //		Internally, a clone of the reference instance is created for each
  //		subgraph. This clone remains attached as long as the Recursive
  //		Multiple Layout is attached to the top-level graph. Before layout is
  //		performed, the global layout parameters are copied from the reference
  //		instance to each clone. If you need to set layout parameters for
  //		individual  nodes, links, or labels,   you have to access the layout
  //		instance of the subgraph that owns  the node, link, or label:     
  //		<pre>
  //		TreeLayout treeLayout = (TreeLayout)layout.getFirstGraphLayout(subgraph);
  //		treeLayout.setAlignment(node, TreeLayout.TIP&#95;OVER);
  //		LinkLayout linkLayout = (LinkLayout)layout.getSecondGraphLayout(subgraph);
  //		linkLayout.setLinkStyle(link, LinkLayout.ORTHOGONAL&#95;STYLE);
  //		AnnealingLabelLayout labelLayout = (AnnealingLabelLayout)layout.getLabelLayout(subgraph);
  //		labelLayout.setLabelDescriptor(label, descriptor); </pre>      
  //		<p>
  //		In the typical case, when you use instances of <code>Graphic</code>
  //		as nodes and instances of <code>Grapher</code> as subgraphs, it is:  
  //		</p>
  //		<pre>
  //		TreeLayout treeLayout = (TreeLayout)layout.getFirstGraphLayout(node.getGraphicBag());
  //		treeLayout.setAlignment(node, TreeLayout.TIP&#95;OVER);
  //		... </pre>   
  //		<p>
  //		<b>Second scenario: Different layout styles at different
  //		subgraphs</b> 
  //		</p>
  //		<p>
  //		The following example shows the second scenario: Each subgraph should
  //		be laid out by a different layout style or with individual layout
  //		parameters. In this case, there are no reference layouts. You specify
  //		the layouts of subgraphs in a very convenient way:   
  //		</p>
  //		<pre>
  //		RecursiveMultipleLayout layout = new RecursiveMultipleLayout();
  //		layout.attach(topLevelGrapher);
  //		// specify the layout of the top-level graph
  //		layout.setLayout(null, new GridLayout(),
  //		                       new LinkLayout(),
  //		                       new RandomLabelLayout()); 
  //		// specify the layout of subgraphs
  //		layout.setLayout(subgraph1, new TreeLayout(),
  //		                            new LinkLayout(),
  //		                            new AnnealingLabelLayout()); 
  //		layout.setLayout(subgraph2, new HierarchicalLayout(),
  //		                            new LinkLayout(),
  //		                            new AnnealingLabelLayout()); 
  //		// perform the layout
  //		LayoutReport report = layout.performLayout(true, true); </pre>
  //		In this scenario, all layout parameters of different subgraphs are
  //		independent. You access the layout instance of each individual
  //		subgraph in the same way as when you have a reference layout. This
  //		allows you to change global layout parameters for this subgraph as
  //		well as parameters of nodes , links, or labels  of the subgraph.  
  //		<p>
  //		<b>Notes:</b>  
  //		<ul>
  //		<li> The Recursive Multiple Layout does not support the specified
  //		provider mode of <code> RecursiveLayout</code>.</li>  
  //		<li> It is not allowed to create a Recursive Layout that contains
  //		another Recursive Layout:   
  //		<pre>
  //		layout = new RecursiveLayout(new RecursiveMultipleLayout());
  //		layout.performLayout(true, true);
  //		</pre>
  //		This would traverse the nested graph and perform the Recursive Layout
  //		for each subgraph, which, however, itself would recursively traverse
  //		the nested graph again. This would just be waste of run time.</li>   
  //		<li> The layout event listeners installed at the Recursive Multiple
  //		Layout receive layout events whenever the layout of a subgraph fires
  //		a layout event. These events, however, do not contain the information
  //		on which sublayout was started or finished. If you need this
  //		information, you should install an event listener at all
  //		sublayouts.</li>   
  //		<li> The reference labeling model cannot be saved to named properties
  //		via <code>
  //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.saveParametersToNamedProperties()</code>.
  //		</li>   
  //		</ul>   
  //		</p>
  /*=====
  // ACTIVE: int
  //		The sublayout is active for all subgraphs.
  ACTIVE : null,
  // INACTIVE: int
  //		The sublayout is inactive for all subgraphs.
  INACTIVE : null,
  // MIXED: int
  //		The sublayout can be active or inactive for individual subgraphs.
  MIXED : null,
  =====*/
  
  _firstLayoutActive: null,
  _secondLayoutActive: null,

  _oldFirstLayoutActive: null,
  _oldSecondLayoutActive: null,

  constructor : function (a0, a1)
  {
    if(arguments.length == 2)
      this._constructor_overload_RecursiveMultipleLayout_2(a0, a1);
    else if(arguments.length == 1)
      this._constructor_overload_RecursiveMultipleLayout_1(a0);
  },
  
  _constructor_overload_RecursiveMultipleLayout_2 : function(referenceLayout1 , referenceLayout2 )
  {
    if (referenceLayout1 && referenceLayout1._isGraphLayout && 
       referenceLayout2 && referenceLayout2._isGraphLayout)
      this.getReferenceLayout().setSilentAttach(true);
  },
  
  init : function()
  {
    // summary:
    //		Initializes instance variables. 
    // description:
    //		Initializes instance variables. 
    //		<p>
    //		You should not call this method directly. The method is called
    //		internally by the constructor without arguments and by the copy
    //		constructor. The method must be overridden by subclasses that need
    //		to initialize  additional instance variables.
    //		</p>
    // tags:
    //		protected
    
    this.inherited("init", arguments, []);

    this._referenceLabelingModel = null;
    this._firstLayoutActive = RecursiveMultipleLayout.ACTIVE | (RecursiveMultipleLayout.ACTIVE << RecursiveMultipleLayout.SHIFT_FOR_AUTOLAYOUT);
    this._secondLayoutActive = RecursiveMultipleLayout.ACTIVE | (RecursiveMultipleLayout.ACTIVE << RecursiveMultipleLayout.SHIFT_FOR_AUTOLAYOUT);

    this._labelLayoutActive = RecursiveMultipleLayout.ACTIVE | (RecursiveMultipleLayout.ACTIVE << RecursiveMultipleLayout.SHIFT_FOR_AUTOLAYOUT);
  },

  _constructor_overload_RecursiveMultipleLayout_1 : function(source)
  {
  },
  
  copy : function()
  {
    // summary:
    //		Copies the layout instance. 
    // description:
    //		Copies the layout instance. 
    //		<p>
    //		This method copies the layout instance by calling the copy
    //		constructor. Note that the parameters which are specific to a node
    //		or a link are not copied.  The reference labeling model is not
    //		copied. It depends on the layout mode what is copied:  
    //		<ul>
    //		<li> <code> RecursiveMultipleLayout.REFERENCE&#95;LAYOUT&#95;MODE</code> -
    //		The reference layouts are copied deeply.</li>  
    //		<li> <code> RecursiveMultipleLayout.INTERNAL&#95;PROVIDER&#95;MODE</code> -
    //		The information which subgraph uses which sublayouts is not
    //		copied.</li>  
    //		</ul>  
    //		</p>
    // returns:
    //		A copy of the layout instance.
    
    return new RecursiveMultipleLayout(this); // ibm_ilog.graphlayout.GraphLayout
  },

  copyParameters : function(source)
  {
    // summary:
    //		Copies the parameters from a given layout instance. 
    // description:
    //		Copies the parameters from a given layout instance. 
    //		Note that the parameters which are specific to a node or a link are
    //		not copied.  The reference labeling model is not copied. It depends
    //		on the layout mode what is copied:  
    //		<ul>
    //		<li> <code> RecursiveMultipleLayout.REFERENCE&#95;LAYOUT&#95;MODE</code> -
    //		The reference layouts are copied deeply.</li>  
    //		<li> <code> RecursiveMultipleLayout.INTERNAL&#95;PROVIDER&#95;MODE</code> -
    //		The information which subgraph uses which sublayouts is not
    //		copied.</li>  
    //		</ul>  
    // source: GraphLayout
    //		The layout instance from which the parameters are copied.
    
    this.inherited("copyParameters", arguments, [source]);

    if (source instanceof RecursiveMultipleLayout)
    {
      var rsource = source;
      this._firstLayoutActive = rsource._firstLayoutActive;
      this._secondLayoutActive = rsource._secondLayoutActive;
    }
  },

  getFirstReferenceGraphLayout : function()
  {
    // summary:
    //		Returns the first reference graph layout, if the reference layout
    //		mode is used. 
    // description:
    //		Returns the first reference graph layout, if the reference layout
    //		mode is used. 
    //		Returns <code>null</code> otherwise.  
    //		<p>
    //		In reference layout mode, the reference layout is used to lay out
    //		the top-level graph. Clones of the reference layout are used to lay
    //		out the subgraphs. The entire nested graph is laid out with the
    //		global layout parameters of the reference layout. If you change
    //		global layout parameters of the reference layout, this will affect
    //		the layout of subgraphs as well, because the global layout
    //		parameters are copied from the reference layout to the layouts of
    //		the subgraphs.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    var refLayout = this.getReferenceLayout();
    if (refLayout != null) {
      return refLayout.getFirstGraphLayout(); // ibm_ilog.graphlayout.GraphLayout
    }
    return null; // ibm_ilog.graphlayout.GraphLayout
  },

  getSecondReferenceGraphLayout : function()
  {
    // summary:
    //		Returns the second reference graph layout, if the reference layout
    //		mode is used. 
    // description:
    //		Returns the second reference graph layout, if the reference layout
    //		mode is used. 
    //		Returns <code>null</code> otherwise.  
    //		<p>
    //		In reference layout mode, the reference layout is used to lay out
    //		the top-level graph. Clones of the reference layout are used to lay
    //		out the subgraphs. The entire nested graph is laid out with the
    //		global layout parameters of the reference layout. If you change
    //		global layout parameters of the reference layout, this will affect
    //		the layout of subgraphs as well, because the global layout
    //		parameters are copied from the reference layout to the layouts of
    //		the subgraphs.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    var refLayout = this.getReferenceLayout();
    if (refLayout != null) {
      return refLayout.getSecondGraphLayout(); // ibm_ilog.graphlayout.GraphLayout
    }
    return null; // ibm_ilog.graphlayout.GraphLayout
  },

  setLayout : function(subgraph, layout)
  {
    // summary:
    //		Sets the layout instance to be used for the input subgraph. 
    // description:
    //		Sets the layout instance to be used for the input subgraph. 
    //		The input layout must be an instance of <code>MultipleLayout</code>.
    //		If <code>null</code> is passed as the subgraph, the input layout is
    //		used for the top-level graph. If <code>null</code> is passed as the
    //		layout, no layout will be performed for the corresponding subgraph. 
    //		
    //		<p>
    //		You can use this method only in internal provider mode. You must
    //		attach a grapher or graph model first before using this method. Each
    //		subgraph must get a different layout instance, that is, the layout
    //		instances cannot be shared among different subgraphs. The input
    //		layout is automatically attached and detached by the Recursive
    //		Multiple Layout as needed.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // layout: GraphLayout
    //		The layout instance used to lay out the subgraph. This
    //		can be 'null', or must be an instance of
    //		'MultipleLayout'.
    
    if(this._callSuper_setLayout){
      this.inherited("setLayout", arguments);
      return;
    }

    if (layout && !(layout instanceof MultipleLayout))
      throw new Error("An instance of MultipleLayout is expected at RecursiveMultipleLayout(Object, GraphLayout)");

    var mlayout = layout;
    if (mlayout != null) {
      mlayout.setSilentAttach(true);
    }
    this.inherited("setLayout", arguments, [subgraph, layout,  true,  false]);
  },

  setLayoutImpl : function(subgraph, layout, detachPrevious , traverse )
  {
    // summary:
    //		Sets the layout instance to be used for the input subgraph. 
    // description:
    //		Sets the layout instance to be used for the input subgraph. 
    //		The input layout must be an instance of <code>MultipleLayout</code>.
    //		If the <code>traverse</code> flag is <code>true</code>, it traverses
    //		the nested graph starting from the input graph and sets a clone of
    //		the input layout recursively for each subgraphs. Notice that if you
    //		insert subgraphs after calling this method, the new subgraphs have
    //		no layout yet assigned.  
    //		<p>
    //		If <code>null</code> is passed as the subgraph, the input layout is
    //		used for the top-level graph, and the traversal, if any, starts at
    //		the  top-level graph. If <code>null</code> is passed as the layout,
    //		no layout will be performed for the corresponding subgraphs.  
    //		</p>
    //		<p>
    //		You can use this method only in internal provider mode. Each
    //		subgraph must get a different layout instance, that is, the layout
    //		instances cannot be shared among different subgraphs. You must
    //		attach a grapher or graph model first before using this method. The
    //		input layout is automatically attached and detached by the Recursive
    //		Multiple Layout as needed.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // layout: GraphLayout
    //		The layout instance used to lay out the subgraph. This
    //		must be an instance of 'MultipleLayout'.
    // detachPrevious: Boolean
    //		If 'true', the layout instance
    //		previously specified as the preferred layout of the subgraph (if
    //		any) is detached.
    // traverse: Boolean
    //		If 'true', clones of the input layout are
    //		recursively used for all current subgraphs of the input graph.
    
    if (layout != null && !(layout instanceof MultipleLayout))
    {
      LogResUtil.logAndThrowRuntimeExc(" RecursiveMultipleLayout ", "graphlayout.expert.message.6681E");
    }

    var mlayout = layout;
    if (mlayout != null) {
      mlayout.setSilentAttach(true);
    }    this._callSuper_performLayout = true;
    this.setLayout(subgraph, layout, detachPrevious, traverse);
    delete this._callSuper_performLayout;
    
  },

  setMultipleLayout : function(subgraph, layout1 , layout2 )
  {
    // summary:
    //		Sets the layout instances to be used for the input subgraph. 
    // description:
    //		Sets the layout instances to be used for the input subgraph. 
    //		If <code>null</code> is passed as the subgraph, the input layout is
    //		used for the top-level graph. Internally, a Multiple Layout is
    //		created that contains the input layouts as sublayouts.  
    //		<p>
    //		When the Recursive Multiple Layout is performed,
    //		<code>layout1</code> is applied first and <code>layout2</code>
    //		second to the input subgraph.  
    //		</p>
    //		<p>
    //		You can use this method only in internal provider mode. You must
    //		attach a grapher or graph model before using this method. Each
    //		subgraph must get different layout instances, that is, the layout
    //		instances cannot be shared among different subgraphs. The input
    //		layouts are automatically attached and detached by the recursive
    //		multiple layout as needed.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // layout1: GraphLayout
    //		The first graph layout used to lay out the subgraph.
    // layout2: GraphLayout
    //		The second graph layout used to lay out the subgraph.
    
    var layout = new MultipleLayout(layout1, layout2);
    layout.setSilentAttach(true);
    this._callSuper_performLayout = true;
    this.setLayout(subgraph, layout);
    delete this._callSuper_performLayout;
  },

  getFirstGraphLayout : function(subgraph)
  {
    // summary:
    //		Returns the layout instance to be used for the input subgraph as the
    //		first layout. 
    // description:
    //		Returns the layout instance to be used for the input subgraph as the
    //		first layout. 
    //		If <code>null</code> is passed as the subgraph, the layout instance
    //		of the top-level graph is returned. You can use this method in all
    //		layout modes. You must attach a grapher or graph model first before
    //		using this method. The method may return <code>null</code> if no
    //		layout should be performed for the subgraph.  
    //		<p>
    //		In reference layout mode, changing global layout parameters of the
    //		layout instances of subgraphs is useless, because during layout, the
    //		global parameters of the first reference graph layout are used for
    //		each subgraph.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // returns:
    //		The layout instance used as the first layout for the subgraph.
    
    var layout = this.getLayout(subgraph);
    if (layout != null) {
      return layout.getFirstGraphLayout(); // ibm_ilog.graphlayout.GraphLayout
    }
    return null; // ibm_ilog.graphlayout.GraphLayout
  },

  getSecondGraphLayout : function(subgraph)
  {
    // summary:
    //		Returns the layout instance to be used for the input subgraph as the
    //		second layout. 
    // description:
    //		Returns the layout instance to be used for the input subgraph as the
    //		second layout. 
    //		If <code>null</code> is passed as the subgraph, the layout instance
    //		of the top-level graph is returned. You can use this method in all
    //		layout modes. You must attach a grapher or graph model first before
    //		using this method. The method may return <code>null</code> if no
    //		layout should be performed for the subgraph.  
    //		<p>
    //		In reference layout mode, changing global layout parameters of the
    //		layout instances of subgraphs is useless, because during layout, the
    //		global parameters of the second reference graph layout are used for
    //		each subgraph.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // returns:
    //		The layout instance used as the second layout for the subgraph.
    
    var layout = this.getLayout(subgraph);
    if (layout != null) {
      return layout.getSecondGraphLayout(); // ibm_ilog.graphlayout.GraphLayout
    }
    return null; // ibm_ilog.graphlayout.GraphLayout
  },

  getLayout : function(subgraph)
  {
    // summary:
    //		Returns the layout instance to be used for the input subgraph. 
    // description:
    //		Returns the layout instance to be used for the input subgraph. 
    //		Here, it always returns an instance of <code>MultipleLayout</code>.
    //		If <code>null</code> is passed as the subgraph, the layout instance
    //		of the top-level graph is returned. You can use this method in all
    //		layout modes. You must attach a grapher or graph model first before
    //		using this method. The method may return <code>null</code> if no
    //		layout should be performed for the subgraph.  
    //		<p>
    //		In reference layout mode, changing global layout parameters of the
    //		layout instances of subgraphs is useless, because during layout, the
    //		global parameters of the reference layout are used for each
    //		subgraph.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // subgraph: Object
    //		The subgraph or 'null'.
    // returns:
    //		The Multiple Layout instance used to lay out the subgraph.
    
    var layout = this.inherited("getLayout", arguments, [subgraph]);
    return layout; // ibm_ilog.graphlayout.GraphLayout
  },

  getLayoutsImpl : function(preOrder)
  {
    // summary:
    //		Returns the instances of 'MultipleLayout' for the nested
    //		graph encapsulated by the graph model of this layout instance. 
    // description:
    //		Returns the instances of <code>MultipleLayout</code> for the nested
    //		graph encapsulated by the graph model of this layout instance. 
    //		You can use this method in all layout modes. You must attach a
    //		grapher or graph model first before using this method.  
    //		<p>
    //		It returns the layout instance for the top-level graph, and
    //		recursively for all subgraphs. The order of the enumeration can be
    //		preorder (that is, the layout of the parent graph comes before the
    //		layout of the subgraphs) or postorder (that is, the layout of the
    //		subgraphs comes before the layout of the parent graph).  
    //		</p>
    //		<p>
    //		In reference layout mode, changing global layout parameters of the
    //		layout instances of subgraphs is useless, because during layout, the
    //		global parameters of the reference layout are used for each
    //		subgraph.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // preOrder: Boolean
    //		If 'true', the layout instances are returned
    //		in preorder, otherwise in postorder.
    // tags:
    //		private
    
    var model = this.getGraphModel();

    if (model == null)
    {
      var name = "getLayouts";
      var params = [name ];
      LogResUtil.logAndThrowRuntimeExc(" RecursiveMultipleLayout ", "graphlayout.expert.message.6680E", params);
    }

    var rootModel = LayoutUtil.getRootModel(model);
    return this.inherited("getLayoutsImpl", arguments, [preOrder]); // ibm_ilog.graphlayout.IIterator
  },

  attach : function(graphModel)
  {
    // summary:
    //		Allows you to specify the top-level graph model of the nested graph
    //		you want to lay out. 
    // description:
    //		Allows you to specify the top-level graph model of the nested graph
    //		you want to lay out. 
    //		In addition to the functionality of the base class, the Recursive
    //		Multiple Layout prepares and attaches the reference layouts in
    //		layout mode <code>
    //		RecursiveMultipleLayout.REFERENCE&#95;LAYOUT&#95;MODE</code>.  If a label
    //		layout algorithm is used but no reference labeling model was
    //		specified before, it tries to use an <code>
    //		ibm&#95;ilog.graphlayout.label.DefaultLabelingModel</code>.  
    //		<p>
    //		  
    //		</p>
    // graphModel: AbstractGraphLayoutModel
    //		The graph model.
    
    this.inherited("attach", arguments, [graphModel]);
  },

  detach : function()
  {
    // summary:
    //		Detaches the graph model from the layout instance. 
    // description:
    //		Detaches the graph model from the layout instance. 
    //		When you attach a new graph model to the layout instance, you do not
    //		need  to detach the old graph model because this is done
    //		automatically when you call <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.attach()</code>. The
    //		<code>detach</code> method performs cleaning operations on the graph
    //		model. In addition to the cleaning operations in the base class, the
    //		Recursive Multiple Layout detaches the sublayouts of subgraphs and
    //		disposes of the labeling models including the reference labeling
    //		model. In internal provider mode, it also cleans all settings of
    //		layout instances for subgraphs.  
    //		<p>
    //		Note that you must call this method when you no longer need the
    //		layout instance. Otherwise, some objects may not be garbage
    //		collected.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    this.inherited("detach", arguments, []);
  },

  propagateFirstGraphLayoutActive : function(active)
  {
    // summary:
    //		Sets whether the first graph layout of all subgraphs is active. 
    // description:
    //		Sets whether the first graph layout of all subgraphs is active. 
    //		This allows you to temporarily disable the first graph layouts. The
    //		first graph layouts are active by default. However, if you want to
    //		perform a layout using only the second graph layouts or the label
    //		layouts of the subgraphs,  you can set the first layout inactive. 
    //		In this case, none of the first graph layouts of subgraphs are
    //		performed  when calling <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code> on the 
    //		Recursive Multiple layout instance.  
    //		<p>
    //		In reference layout mode, it calls <code>
    //		ibm&#95;ilog.graphlayout.MultipleLayout.setFirstGraphLayoutActive()</code>
    //		at the reference layout, which is in this case a Multiple Layout. If
    //		a graph is attached and the layout mode is the internal provider
    //		mode, it traverses the nesting hierarchy and calls  <code>
    //		ibm&#95;ilog.graphlayout.MultipleLayout.setFirstGraphLayoutActive()</code>
    //		for all Multiple Layout instances of subgraphs. However, if you
    //		insert subgraphs after calling this method, or if you change the
    //		layout instances of subgraphs in the internal provider after calling
    //		this method, the new layouts of these subgraphs may have a different
    //		setting.  
    //		</p>
    //		<p>
    //		This method throws a runtime exception when no graph model is yet
    //		attached.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // active: Boolean
    //		'true' if the first graph layout is active, or
    //		'false' otherwise.
    
    var l;

    if (this.getLayoutMode() == RecursiveLayout.REFERENCE_LAYOUT_MODE)
    {
      l = this.getReferenceLayout();
      l.setFirstGraphLayoutActive(active);
    }
    else
    {
      if (this.getGraphModel() == null)
      {
        var name = "propagateFirstGraphLayoutActive";
        var params = [name ];
        LogResUtil.logAndThrowRuntimeExc(" RecursiveMultipleLayout ", "graphlayout.expert.message.6680E", params);
      }

      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        l = e.next();
        if (l != null) {
          l.setFirstGraphLayoutActive(active);
        }
      }
    }
  },

  propagateSecondGraphLayoutActive : function(active)
  {
    // summary:
    //		Sets whether the second graph layout of all subgraphs is active. 
    // description:
    //		Sets whether the second graph layout of all subgraphs is active. 
    //		This allows you to temporarily disable the second graph layouts. The
    //		second graph layouts are active by default. To perform a layout
    //		using only the first graph layouts  or the label layouts of the
    //		subgraphs,  set the second layout to be inactive.  In this case,
    //		none of the second graph layouts of  subgraphs are performed when
    //		calling <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code> on the
    //		Recursive Multiple layout instance.  
    //		<p>
    //		In reference layout mode,  <code>
    //		ibm&#95;ilog.graphlayout.MultipleLayout.setSecondGraphLayoutActive()</code>
    //		is called  at the reference layout, which is in this case a Multiple
    //		Layout. If a graph is attached and the layout mode is the internal
    //		provider mode, it traverses the nesting hierarchy and calls  <code>
    //		ibm&#95;ilog.graphlayout.MultipleLayout.setSecondGraphLayoutActive()</code>
    //		for all Multiple Layout instances of subgraphs. If you insert
    //		subgraphs after calling this method, or if you change the layout
    //		instances of subgraphs in the internal provider after calling this
    //		method, the new layouts of these subgraphs may have a different
    //		setting.  
    //		</p>
    //		<p>
    //		This method throws a runtime exception when no graph model is yet
    //		attached.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // active: Boolean
    //		Set to 'true' if the second graph layout is 
    //		active.
    
    var l;

    if (this.getLayoutMode() == RecursiveLayout.REFERENCE_LAYOUT_MODE)
    {
      l = this.getReferenceLayout();
      l.setSecondGraphLayoutActive(active);
    }
    else
    {
      if (this.getGraphModel() == null)
      {
        var name = "propagateSecondGraphLayoutActive";
        var params = [name ];
        LogResUtil.logAndThrowRuntimeExc(" RecursiveMultipleLayout ", "graphlayout.expert.message.6680E", params);
      }

      var e = this.getLayoutsImpl(true);

      while (e.hasNext())
      {
        l = e.next();
        if (l != null) {
          l.setSecondGraphLayoutActive(active);
        }
      }
    }
  },

  setFirstGraphLayoutActive : function(value)
  {
    // summary:
    //		Sets whether the first graph layout of all subgraphs is active. 
    // description:
    //		Sets whether the first graph layout of all subgraphs is active. 
    //		This holds for calls of <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code> , except for
    //		calls that were triggered by automatic layout. See <code>
    //		RecursiveLayout.setAutoLayout()</code> for  more. Possible values
    //		are:  
    //		<ul>
    //		<li> <code> RecursiveMultipleLayout.ACTIVE</code>: the first graph
    //		layout is active for all subgraphs. </li>  
    //		<li> <code> RecursiveMultipleLayout.INACTIVE</code>: the first graph
    //		layout is inactive for all subgraphs. </li>  
    //		<li> <code> RecursiveMultipleLayout.MIXED</code>: it is unspecified
    //		whether the first layout is active or inactive for all subgraphs.
    //		Instead, it is possible to specify for each subgraph individually by
    //		calling <code>
    //		ibm&#95;ilog.graphlayout.MultipleLayout.setFirstGraphLayoutActive()</code>
    //		to control whether the first graph layout should be active or
    //		inactive. </li>  
    //		</ul>  
    //		<p>
    //		 
    //		</p>
    // value: int
    //		Sets whether the first graph layout is active.
    
    if (value != RecursiveMultipleLayout.ACTIVE && value != RecursiveMultipleLayout.INACTIVE && value != RecursiveMultipleLayout.MIXED)
    {
      var name = "firstGraphLayoutActive";
      var params = [value, name ];
      LogResUtil.logAndThrowIllegalArgExc(" RecursiveMultipleLayout ", "graphlayout.expert.message.6682E", params);
    }

    if (value != this.getFirstGraphLayoutActive())
    {
      this._firstLayoutActive &= ~RecursiveMultipleLayout.AMASK;
      this._firstLayoutActive |= value;
      this.onParameterChanged("FirstGraphLayoutActive");
    }
  },

  getFirstGraphLayoutActive : function()
  {
    // summary:
    //		Returns whether the first graph layout of all subgraphs is active. 
    // description:
    //		Returns whether the first graph layout of all subgraphs is active. 
    //		This holds for calls of <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>, except for
    //		calls triggered by automatic layout. See <code>
    //		RecursiveLayout.setAutoLayout()</code> for more  information.  
    // returns:
    //		If the first graph layout of all subgraphs is active, '
    //		RecursiveMultipleLayout.ACTIVE' is returned. If the first
    //		graph layout of all subgraphs is
    //		inactive, ' RecursiveMultipleLayout.INACTIVE' is
    //		returned. Otherwise ' RecursiveMultipleLayout.MIXED'
    //		is returned.
    
    return this._firstLayoutActive & RecursiveMultipleLayout.AMASK; // int
  },

  setSecondGraphLayoutActive : function(value)
  {
    // summary:
    //		Sets whether the second graph layout of all subgraphs is active. 
    // description:
    //		Sets whether the second graph layout of all subgraphs is active. 
    //		This holds for calls to <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>, except
    //		calls triggered by automatic layout (see <code>
    //		RecursiveLayout.setAutoLayout()</code>). Possible values are:  
    //		<ul>
    //		<li> <code> RecursiveMultipleLayout.ACTIVE</code>: the second graph
    //		layout is active for all subgraphs. </li>  
    //		<li> <code> RecursiveMultipleLayout.INACTIVE</code>: the second
    //		graph layout is inactive for all subgraphs. </li>  
    //		<li> <code> RecursiveMultipleLayout.MIXED</code>: it is unspecified
    //		whether the second layout is active or inactive for all subgraphs.
    //		Instead, it is possible to specify for each subgraph individually by
    //		calling <code>
    //		ibm&#95;ilog.graphlayout.MultipleLayout.setSecondGraphLayoutActive()</code>
    //		to control whether the second graph layout should be active or
    //		inactive. </li>  
    //		</ul>  
    //		<p>
    //		 
    //		</p>
    // value: int
    //		Sets whether the second graph layout is active.
    
    if (value != RecursiveMultipleLayout.ACTIVE && value != RecursiveMultipleLayout.INACTIVE && value != RecursiveMultipleLayout.MIXED)
    {
      var name = "secondGraphLayoutActive";
      var params = [value, name ];
      LogResUtil.logAndThrowIllegalArgExc(" RecursiveMultipleLayout ", "graphlayout.expert.message.6682E", params);
    }

    if (value != this.getSecondGraphLayoutActive())
    {
      this._secondLayoutActive &= ~RecursiveMultipleLayout.AMASK;
      this._secondLayoutActive |= value;
      this.onParameterChanged("SecondGraphLayoutActive");
    }
  },

  getSecondGraphLayoutActive : function()
  {
    // summary:
    //		Tests if the second graph layout of all subgraphs is active. 
    // description:
    //		Tests if the second graph layout of all subgraphs is active. 
    //		This holds for calls to <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>, except for
    //		calls triggered by automatic layout (see <code>
    //		RecursiveLayout.setAutoLayout()</code>).  
    // returns:
    //		If the second graph layout of all subgraphs is active, '
    //		RecursiveMultipleLayout.ACTIVE' is returned. If the second
    //		graph layout of all subgraphs
    //		is inactive, ' RecursiveMultipleLayout.INACTIVE' is
    //		returned. Otherwise '
    //		RecursiveMultipleLayout.MIXED' is returned.
    
    return this._secondLayoutActive & RecursiveMultipleLayout.AMASK; // int
  }

}
);

lang.mixin(RecursiveMultipleLayout,
{
  ACTIVE : null,

  INACTIVE : null,

  MIXED : null,
  AMASK : null,
  SHIFT_FOR_AUTOLAYOUT : null,
  _fields_init : function(c)
  {
    c = c || RecursiveMultipleLayout;
    if(RecursiveLayout._fields_init)
      RecursiveLayout._fields_init(c);
    
    c.ACTIVE = 1;
    c.INACTIVE = 2;
    c.MIXED = 0;
    c.AMASK = RecursiveMultipleLayout.ACTIVE | RecursiveMultipleLayout.INACTIVE;
    c.SHIFT_FOR_AUTOLAYOUT = 2;
  }
  
}
);

RecursiveMultipleLayout._getGlobalProperties = function()
{
  // tags:
  //		private
  return RecursiveLayout._getGlobalProperties();
};

RecursiveMultipleLayout._getNodeProperties = function()
{
  // tags:
  //		private
  return RecursiveLayout._getNodeProperties();
};

RecursiveMultipleLayout._getLinkProperties = function()
{
  // tags:
  //		private
  return RecursiveLayout._getLinkProperties();
};

(function(){ RecursiveMultipleLayout._fields_init(); })();

return RecursiveMultipleLayout;
});
