/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define([
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../hierarchical/HierarchicalNodeGroup",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../IIterator"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    HierarchicalNodeGroup,	// #5
    TranslateUtil,	// #6
    IIterator	// #7
    ) {

var ConstraintManager = declare("ibm_ilog.graphlayout.hierarchical.ConstraintManager", null, 
{
  // summary:
  //		The class 'ConstraintManager' manages the constraints and
  //		groups of nodes. 
  // description:
  //		The class <code>ConstraintManager</code> manages the constraints and
  //		groups of nodes. 
  //		<p>
  //		 
  //		</p>
  // tags:
  //		private
  
  _firstGroup : null,
  _numberOfGroups : null,
  _firstConstraint : null,
  _numberOfConstraints : null,

  constructor : function()
  {
    this._firstGroup = null;
    this._numberOfGroups = 0;
    this._firstConstraint = null;
    this._numberOfConstraints = 0;
  },
  
  copy : function()
  {
    // summary:
    //		Creates a (deep) copy of the constraint manager. 
    // description:
    //		Creates a (deep) copy of the constraint manager. 
    //		Note that it is not a perfect copy because the order of the groups
    //		and constraints get reversed. But in our context, this
    //		doesn't matter, because we happen to always apply it twice where necessary, and by that the original order is obtained.
    // tags:
    //		private
    
    var mgr = new ConstraintManager();

    var e = this.getGroups();

    while (e.hasNext())
      mgr.addGroup((e.next()).copy());

    e = this.getConstraints();
    var constraint;
    var subject;

    while (e.hasNext())
    {
      constraint = (e.next()).copy();
      subject = constraint.getFirstSubject();
      if (subject instanceof HierarchicalNodeGroup) {
        constraint.setFirstSubject((subject).getRecentCopy());
      }
      subject = constraint.getSecondSubject();
      if (subject instanceof HierarchicalNodeGroup) {
        constraint.setSecondSubject((subject).getRecentCopy());
      }
      mgr.addConstraint(constraint);
    }

    return mgr; // ibm_ilog.graphlayout.hierarchical.ConstraintManager
  },

  addGroup : function(group)
  {
    // summary:
    //		Adds the node group to the constraint manager.
    // tags:
    //		private
    
    if (group == null) {
      return;
    }
    if (group.getOwner() == this) {
      return;
    }
    if (group.getOwner() != null) {
      group.getOwner().removeGroup(group);
    }
    group.setNext(this._firstGroup);
    group.setOwner(this);
    this._firstGroup = group;
    this._numberOfGroups++;
  },

  removeGroup : function(group)
  {
    // summary:
    //		Removes a node group from the constraint manager.
    // tags:
    //		private
    
    if (group == null) {
      return;
    }
    if (group.getOwner() != this) {
      return;
    }
    if (this._firstGroup == group) {
      this.removeNextGroup(null);
    }
    else
    {
      var prev = this._firstGroup;

      while (prev != null && prev.getNext() != group)
        prev = prev.getNext();
      if (prev != null) {
        this.removeNextGroup(prev);
      }
    }
  },

  removeNextGroup : function(prevGroup)
  {
    // summary:
    //		Removes the next group from the constraint manager. 
    // description:
    //		Removes the next group from the constraint manager. 
    //		If the input is null, it removes the first group.
    // tags:
    //		private
    
    var groupToRemove = null;
    if (prevGroup == null) {
      groupToRemove = this._firstGroup;
    }
    else {
      groupToRemove = prevGroup.getNext();
    }
    if (groupToRemove != null && groupToRemove.getOwner() == this)
    {
      if (prevGroup == null) {
        this._firstGroup = groupToRemove.getNext();
      }
      else {
        prevGroup.setNext(groupToRemove.getNext());
      }
      groupToRemove.setNext(null);
      groupToRemove.setOwner(null);
      this._numberOfGroups--;
    }
  },

  getNumberOfGroups : function()
  {
    // summary:
    //		Returns the number of groups stores in this constraint manager.
    // tags:
    //		private
    
    return this._numberOfGroups; // int
  },

  getGroups : function()
  {
    // summary:
    //		Returns the node groups stored in this constraint manager.
    // tags:
    //		private
    
    return new ConstraintManagerAnonClass_1(this); // ibm_ilog.graphlayout.IIterator
  },

  validateGroups : function(model)
  {
    // summary:
    //		Validates all node groups. 
    // description:
    //		Validates all node groups. 
    //		This removes all groups that are no longer valid because all their
    //		nodes do not belong anymore to the graph model.
    // tags:
    //		private
    
    var e = this.getGroups();
    var prev = null;
    var group;

    while (e.hasNext())
    {
      group = e.next();
      if (!group.validate(model)) {
        this.removeNextGroup(prev);
      }
      else {
        prev = group;
      }
    }
  },

  addConstraint : function(constraint)
  {
    // summary:
    //		Adds the constraint to the constraint manager.
    // tags:
    //		private
    
    if (constraint == null) {
      return;
    }
    constraint.setNext(this._firstConstraint);
    this._firstConstraint = constraint;
    this._numberOfConstraints++;
    constraint.actAfterAdd(this);
  },

  removeConstraint : function(constraint)
  {
    // summary:
    //		Removes a constraint from the constraint manager.
    // tags:
    //		private
    
    if (constraint == null) {
      return;
    }
    if (this._firstConstraint == constraint) {
      this.removeNextConstraint(null);
    }
    else
    {
      var prev = this._firstConstraint;

      while (prev != null && prev.getNext() != constraint)
        prev = prev.getNext();
      if (prev != null) {
        this.removeNextConstraint(prev);
      }
    }
  },

  removeNextConstraint : function(prevConstraint)
  {
    // summary:
    //		Removes the next constraint from the constraint manager. 
    // description:
    //		Removes the next constraint from the constraint manager. 
    //		If the input is null, it removes the first constraint.
    // tags:
    //		private
    
    var constraintToRemove = null;
    if (prevConstraint == null) {
      constraintToRemove = this._firstConstraint;
    }
    else {
      constraintToRemove = prevConstraint.getNext();
    }
    if (constraintToRemove != null)
    {
      if (prevConstraint == null) {
        this._firstConstraint = constraintToRemove.getNext();
      }
      else {
        prevConstraint.setNext(constraintToRemove.getNext());
      }
      constraintToRemove.setNext(null);
      this._numberOfConstraints--;
    }
  },

  getNumberOfConstraints : function()
  {
    // summary:
    //		Returns the number of constraints stores in this constraint manager.
    // tags:
    //		private
    
    return this._numberOfConstraints; // int
  },

  getConstraints : function()
  {
    // summary:
    //		Returns the constraints stored in this constraint manager.
    // tags:
    //		private
    
    return new ConstraintManagerAnonClass_2(this); // ibm_ilog.graphlayout.IIterator
  },

  validateConstraints : function(model)
  {
    // summary:
    //		Validates all constraints. 
    // description:
    //		Validates all constraints. 
    //		This removes all constraints that are no longer valid because all
    //		their nodes do not belong anymore to the graph model.
    // tags:
    //		private
    
    var e = this.getConstraints();
    var prev = null;
    var constraint;

    while (e.hasNext())
    {
      constraint = e.next();
      if (!constraint.validate(model)) {
        this.removeNextConstraint(prev);
      }
      else {
        prev = constraint;
      }
    }
  },

  beforeLayout : function(graph)
  {
    // summary:
    //		Called before layout.
    // tags:
    //		private
    
    var e = this.getGroups();

    while (e.hasNext())
      (e.next()).beforeLayout(graph);
    e = this.getConstraints();

    while (e.hasNext())
      (e.next()).validateForLayout(graph);
  },

  afterLayout : function(graph)
  {
    // summary:
    //		Called after layout.
    // tags:
    //		private
    
    var e = this.getGroups();

    while (e.hasNext())
      (e.next()).afterLayout(graph);
  }

}
);

lang.mixin(ConstraintManager,
{
  serialVersionUID : null,
  _fields_init : function(c)
  {
    c = c || ConstraintManager;
    
    c.serialVersionUID = 375937290235458;
  }
  
}
);

/**
 * @private
 */
ConstraintManagerAnonClass_1 = declare(IIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  actGroup: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.actGroup = this.__outerThis._firstGroup;
  },

  hasMoreElements : function()
  {
    return this.actGroup != null; // Boolean
  },

  nextElement : function()
  {
    if (this.actGroup == null) {
      throw new Error("no more groups"); // Object
    }
    var g = this.actGroup;
    this.actGroup = g.getNext();
    return g; // Object
  }

});

/**
 * @private
 */
ConstraintManagerAnonClass_2 = declare(IIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  actConstraint: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.actConstraint = this.__outerThis._firstConstraint;
  },

  hasMoreElements : function()
  {
    return this.actConstraint != null; // Boolean
  },

  nextElement : function()
  {
    if (this.actConstraint == null) {
      throw new Error("no more constraints"); // Object
    }
    var c = this.actConstraint;
    this.actConstraint = c.getNext();
    return c; // Object
  }

});

(function(){ ConstraintManager._fields_init(); })();

return ConstraintManager;
});
