/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

//>>built
require({cache:{
'ibm_ilog/diagram/interactors/MoveKeyInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/MoveKeyInteractor", [
"dojo/_base/declare",
"dojo/_base/array",
"dojo/keys",
"require",
"dojox/gfx/matrix",
"./Interactor",
"../Selection",
"../util/Geometry",
"../util/GraphUtil",
"../util/Batch"
],
function(
declare,
array,
keys,
require,
m,
Interactor,
Selection,
g,
gu,
Batch
){

// The following classes are loaded dynamically if the editor modules are loaded.

var MultipleAction = null;
var UndoActionList = null;
var EditingUtils = null;
var _classesRequired = false;

var _requireClasses = function()
{
	if(!_classesRequired){
		// no try/catch, if we are in the editor these classes must be there.
		MultipleAction = require("../editor/undo/MultipleAction");
		UndoActionList = require("../editor/undo/UndoActionList");
		EditingUtils = require("../editor/EditingUtils");
	}
	_classesRequired = true;
}

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var MoveKeyInteractor =
declare("ibm_ilog.diagram.interactors.MoveKeyInteractor", [Interactor], {
	//summary:
	//		This Interactor manages the graphElements movement through the keyboard interaction.
	//		This Interactor have 8 connections: moveLeft, moveRigth, moveUp, moveDown, fineMoveLeft, fineMoveRigth, fineMoveUp, fineMoveDown.
	
	_diagram: null,
	_increment: null,
	_fineIncrement: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.		
		this._diagram = diagram;
		this.setIncrement(10);
		this.setFineIncrement(1);
		return this._initialize();
	},
	setIncrement: function(increment){
		// summary: 
		//		sets the move increment
		// increment: integer
		//		the increment to be set
		this._increment = increment;
	},
	getIncrement: function(){
		// summary: 
		//		gets the move increment
		return this._increment;
	},
	setFineIncrement: function(increment){
		// summary: 
		//		sets the move fine increment
		// increment: integer
		//		the fine increment to be set
		this._fineIncrement = increment;
	},
	getFineIncrement: function(){
		// summary: 
		//		gets the move fine increment
		return this._fineIncrement;
	},
	moveUp: function(e){
		// summary:
	    //		This method moves the selected elements up
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:-this._increment});
	},
	moveDown: function(e){
		// summary:
	    //		This method moves the selected elements down
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:this._increment});
	},
	moveRight: function(e){
		// summary:
	    //		This method moves the selected elements rigth
	    // e: Event object
	    //		the event to be treated.
		this._move({x:this._increment,y:0});
	},
	moveLeft: function(e){
		// summary:
	    //		This method moves the selected elements left
	    // e: Event object
	    //		the event to be treated.
		this._move({x:-this._increment,y:0});
	},
	fineMoveUp: function(e){
		// summary:
	    //		This method finely moves the selected elements up
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:-this._fineIncrement});
	},
	fineMoveDown: function(e){
		// summary:
	    //		This method finely moves the selected elements down
	    // e: Event object
	    //		the event to be treated.
		this._move({x:0,y:this._fineIncrement});
	},
	fineMoveRight: function(e){
		// summary:
	    //		This method finely moves the selected elements rigth
	    // e: Event object
	    //		the event to be treated.
		this._move({x:this._fineIncrement,y:0});
	},
	fineMoveLeft: function(e){
		// summary:
	    //		This method finely moves the selected elements left
	    // e: Event object
	    //		the event to be treated.
		this._move({x:-this._fineIncrement,y:0});
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return MoveKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			moveLeft: {
				hotKey: keys.LEFT_ARROW,
				connectTo: "moveLeft",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveLeft: {
				hotKey: null,
				connectTo: "fineMoveLeft",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, moveRight: {
				hotKey: keys.RIGHT_ARROW,
				connectTo: "moveRight",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveRight: {
				hotKey: null,
				connectTo: "fineMoveRight",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, moveUp: {
				hotKey: keys.UP_ARROW,
				connectTo: "moveUp",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveUp: {
				hotKey: null,
				connectTo: "fineMoveUp",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, moveDown: {
				hotKey: keys.DOWN_ARROW,
				connectTo: "moveDown",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, fineMoveDown: {
				hotKey: null,
				connectTo: "fineMoveDown",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}
		};
	},
	_move: function(delta){
		var targetSet = this._buildTargetSet();
		
		if(targetSet.length == 0){
			return false;
		}
		
		// check move limits defined on the Graph if any:
		array.forEach(targetSet,function(s){
			var p = s.e.getParent();
			if (p && p.onElementMoving) {
				var oldx = delta.x;
				var oldy = delta.y;
				p.onElementMoving.call(p, s.e, s.t1, delta);
			}
		},this);
		
		array.forEach(targetSet,function(s){
			this._doMove(s.e,s.t1,delta);
		},this);
		
		Batch.endBatch(); // update links immediately for better visual result
		
		if(this._diagram._isIBMDiagramEditor){
			var um = this._diagram.getUndoManager();
			um.addAction(this._createUndoAction(targetSet));
		}
		return true;
	},
	_doMove: function(graphElement,originalTransform,delta) {
		//	summary:
		//		Concrete the movement to a graph element.
		//	graphElement:
		//		The graph element to move.
		//	originalTransform:
		//		The matrix representing the location from which the delta is computed.
		//	delta:
		//		The ammount to translate the entity from its original location.
		graphElement.setTransform(m.multiply(originalTransform,m.translate(delta)));
	},
	_isMovable: function(ge) {
		return !(ge._isIBMDiagramLink);
	},
	getTransformToElement: function(e) {
		//	returns:
		//		a transform from viewport coords to element coords
		var rm = e._getRealMatrix();
		return (rm && rm.inverse());
	},
	_buildTargetSet: function() {
		//
		//	summary:
		//		Builds the set of entities that are going to be affected by the drag.
		//
		//	description:
		//		Grab Nodes and Subgraphs from the _selection object. Their original 
		//    transforms are saved to be used in the drag process. We also store 
		//		viewport-to-element transforms since they are not likely to change
		//		while dragging (would require modifying the transformation of some
		//		of the parents of the selected shape). This might change if we plan
		//		to provide pan-while-moving.
		//

		var selection = this._diagram.getSelection();
		var s = selection.fastGet();
		var gs = [];
		
		s.forEach(function(e){
			var ge = selection._asGraphElement(e);
			if(ge && this._isMovable(ge)) {
				// no link moving for now
				gs.push(ge);
			}
		},this);
		
		//if no selection, add the focused element
		if(gs.length == 0){
			var c = this._diagram.getFocusedElement();
			if(c && this._isMovable(c)){
				gs.push(c);
			}
		}
		
		var targetSet = array.map(gu.maximals(gs),function(e){
			return { t1:e.getTransform(), e:e, t:this.getTransformToElement(e) };
		},this);
		
		return targetSet;
	},
	_createUndoAction: function(targetSet){
		// summary:
		//		creates the undo action, based on the targetSet
		
		_requireClasses();
		
		var mAction = new MultipleAction(UndoActionList.Move);
		
		array.forEach(targetSet, function(s){
			var action = EditingUtils.createMoveUndoAction(s.e, s.t1, s.e.getTransform());
			mAction.addAction(action);
		});
		
		return mAction;
	}
});

MoveKeyInteractor.KeyInteractorId = "Move";

return MoveKeyInteractor;

});

},
'ibm_ilog/diagram/layer_base':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm_ilog/diagram/layer_base_dojo_requires,ibm_ilog/diagram/base,ibm_ilog/diagram/_base/diagext,ibm_ilog/diagram/_base/dojoext,ibm_ilog/diagram/_base/gfxext,ibm_ilog/diagram/data/dtlbinding,ibm_ilog/diagram/gfxlayout,ibm_ilog/diagram/gfxlayout/_base,ibm_ilog/diagram/gfxlayout/CanvasLayout,ibm_ilog/diagram/gfxlayout/Engine,ibm_ilog/diagram/gfxlayout/FillLayout,ibm_ilog/diagram/gfxlayout/GridLayout,ibm_ilog/diagram/gfxlayout/Layout,ibm_ilog/diagram/gfxlayout/StackLayout,ibm_ilog/diagram/gfxwidget/GfxWidget,ibm_ilog/diagram/Graph,ibm_ilog/diagram/GraphElement,ibm_ilog/diagram/graphlayout/_base,ibm_ilog/diagram/graphlayout/Animation,ibm_ilog/diagram/graphlayout/GraphAdapter,ibm_ilog/diagram/graphlayout/_LayoutProvider,ibm_ilog/diagram/graphlayout/LayoutRunner,ibm_ilog/diagram/graphlayout/ServerSideLayout,ibm_ilog/diagram/graphlayout/webworker/_GraphAdapter,ibm_ilog/diagram/graphlayout/WebWorkerLayout,ibm_ilog/diagram/graphlayout/webworker/_LayoutProvider,ibm_ilog/diagram/graphlayout/webworker/_matrix,ibm_ilog/diagram/interactors/AdornerFocusInteractor,ibm_ilog/diagram/interactors/DragInteractor,ibm_ilog/diagram/interactors/FocusInteractor,ibm_ilog/diagram/interactors/Interactor,ibm_ilog/diagram/interactors/InteractorManager,ibm_ilog/diagram/interactors/KeyboardInteractorManager,ibm_ilog/diagram/interactors/MarqueeSelectionInteractor,ibm_ilog/diagram/interactors/MoveInteractor,ibm_ilog/diagram/interactors/MoveKeyInteractor,ibm_ilog/diagram/interactors/Navigator,ibm_ilog/diagram/interactors/PanInteractor,ibm_ilog/diagram/interactors/PanKeyInteractor,ibm_ilog/diagram/interactors/SelectionInteractor,ibm_ilog/diagram/interactors/SelectionKeyInteractor,ibm_ilog/diagram/interactors/SwitchModeInteractor,ibm_ilog/diagram/interactors/WheelPanInteractor,ibm_ilog/diagram/interactors/ZoomInteractor,ibm_ilog/diagram/interactors/ZoomKeyInteractor,ibm_ilog/diagram/Link,ibm_ilog/diagram/Node,ibm_ilog/diagram/overview/Overview,ibm_ilog/diagram/overview/OverviewGfxRenderer,ibm_ilog/diagram/overview/OverviewInteractor,ibm_ilog/diagram/overview/OverviewPanInteractor,ibm_ilog/diagram/overview/OverviewRenderer,ibm_ilog/diagram/overview/OverviewTemplateRenderer,ibm_ilog/diagram/overview/OverviewTopLevelTemplateRenderer,ibm_ilog/diagram/overview/OverviewWidget,ibm_ilog/diagram/overview/OverviewZoomInteractor,ibm_ilog/diagram/Port,ibm_ilog/diagram/Selectable,ibm_ilog/diagram/Selection,ibm_ilog/diagram/style,ibm_ilog/diagram/Subgraph,ibm_ilog/diagram/SwimLane,ibm_ilog/diagram/templates/editor_templates,ibm_ilog/diagram/templates/templates,ibm_ilog/diagram/templating,ibm_ilog/diagram/util/Batch,ibm_ilog/diagram/util/ClippingUtil,ibm_ilog/diagram/util/DeferredInitialization,ibm_ilog/diagram/util/ErrorReporter,ibm_ilog/diagram/util/Geometry,ibm_ilog/diagram/util/GlobalInitialization,ibm_ilog/diagram/util/GraphUtil,ibm_ilog/diagram/util/HandleSet,ibm_ilog/diagram/util/LimitedStack,ibm_ilog/diagram/util/MultilineText,ibm_ilog/diagram/util/PrintUtil,ibm_ilog/diagram/util/Serializer,ibm_ilog/diagram/widget/DataConnector,ibm_ilog/diagram/widget/Diagram,ibm_ilog/diagram/widget/_Diagram,ibm_ilog/diagram/widget/DivViewport,ibm_ilog/diagram/widget/GfxViewport,ibm_ilog/diagram/widget/HudComponent,ibm_ilog/diagram/widget/OverviewHudComponent,ibm_ilog/diagram/widget/ScrollbarHudComponent,ibm_ilog/diagram/widget/ScrollbarInteractor,ibm_ilog/diagram/widget/Viewport,ibm_ilog/diagram/widget/ZoomHudComponent,ibm_ilog/diagram/adorners/Adorner,ibm_ilog/diagram/adorners/AdornerHandle,ibm_ilog/diagram/adorners/AdornerWithHandles,ibm_ilog/diagram/adorners/AnnotationAdorner,ibm_ilog/diagram/adorners/HighlightedHandle,ibm_ilog/diagram/adorners/LinkAdorner,ibm_ilog/diagram/adorners/MoveAnnotationInteractor,ibm_ilog/diagram/adorners/TemplatedOverlay"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
dojo.provide('ibm_ilog.diagram.layer_base');

dojo.require('ibm_ilog.diagram.layer_base_dojo_requires');

dojo.require('ibm_ilog.diagram.base');
dojo.require('ibm_ilog.diagram._base.diagext');
dojo.require('ibm_ilog.diagram._base.dojoext');
dojo.require('ibm_ilog.diagram._base.gfxext');
dojo.require('ibm_ilog.diagram.data.dtlbinding');
dojo.require('ibm_ilog.diagram.gfxlayout');
dojo.require('ibm_ilog.diagram.gfxlayout._base');
dojo.require('ibm_ilog.diagram.gfxlayout.CanvasLayout');
dojo.require('ibm_ilog.diagram.gfxlayout.Engine');
dojo.require('ibm_ilog.diagram.gfxlayout.FillLayout');
dojo.require('ibm_ilog.diagram.gfxlayout.GridLayout');
dojo.require('ibm_ilog.diagram.gfxlayout.Layout');
dojo.require('ibm_ilog.diagram.gfxlayout.StackLayout');
dojo.require('ibm_ilog.diagram.gfxwidget.GfxWidget');
dojo.require('ibm_ilog.diagram.Graph');
dojo.require('ibm_ilog.diagram.GraphElement');
dojo.require('ibm_ilog.diagram.graphlayout._base');
dojo.require("ibm_ilog.diagram.graphlayout.Animation");
dojo.require("ibm_ilog.diagram.graphlayout.GraphAdapter");
dojo.require("ibm_ilog.diagram.graphlayout._LayoutProvider");
dojo.require("ibm_ilog.diagram.graphlayout.LayoutRunner");
dojo.require('ibm_ilog.diagram.graphlayout.ServerSideLayout');
dojo.require("ibm_ilog.diagram.graphlayout.webworker._GraphAdapter");
dojo.require('ibm_ilog.diagram.graphlayout.WebWorkerLayout');
dojo.require("ibm_ilog.diagram.graphlayout.webworker._LayoutProvider");
dojo.require("ibm_ilog.diagram.graphlayout.webworker._matrix");
dojo.require('ibm_ilog.diagram.interactors.AdornerFocusInteractor');
dojo.require('ibm_ilog.diagram.interactors.DragInteractor');
dojo.require('ibm_ilog.diagram.interactors.FocusInteractor');
dojo.require('ibm_ilog.diagram.interactors.Interactor');
dojo.require('ibm_ilog.diagram.interactors.InteractorManager');
dojo.require('ibm_ilog.diagram.interactors.KeyboardInteractorManager');
dojo.require('ibm_ilog.diagram.interactors.MarqueeSelectionInteractor');
dojo.require('ibm_ilog.diagram.interactors.MoveInteractor');
dojo.require('ibm_ilog.diagram.interactors.MoveKeyInteractor');
dojo.require('ibm_ilog.diagram.interactors.Navigator');
dojo.require('ibm_ilog.diagram.interactors.PanInteractor');
dojo.require('ibm_ilog.diagram.interactors.PanKeyInteractor');
dojo.require('ibm_ilog.diagram.interactors.SelectionInteractor');
dojo.require('ibm_ilog.diagram.interactors.SelectionKeyInteractor');
dojo.require('ibm_ilog.diagram.interactors.SwitchModeInteractor');
dojo.require('ibm_ilog.diagram.interactors.WheelPanInteractor');
dojo.require('ibm_ilog.diagram.interactors.ZoomInteractor');
dojo.require('ibm_ilog.diagram.interactors.ZoomKeyInteractor');
dojo.require('ibm_ilog.diagram.Link');
dojo.require('ibm_ilog.diagram.Node');
dojo.require('ibm_ilog.diagram.overview.Overview');
dojo.require('ibm_ilog.diagram.overview.OverviewGfxRenderer');
dojo.require('ibm_ilog.diagram.overview.OverviewInteractor');
dojo.require('ibm_ilog.diagram.overview.OverviewPanInteractor');
dojo.require('ibm_ilog.diagram.overview.OverviewRenderer');
dojo.require('ibm_ilog.diagram.overview.OverviewTemplateRenderer');
dojo.require('ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer');
dojo.require('ibm_ilog.diagram.overview.OverviewWidget');
dojo.require('ibm_ilog.diagram.overview.OverviewZoomInteractor');
dojo.require('ibm_ilog.diagram.Port');
dojo.require('ibm_ilog.diagram.Selectable');
dojo.require('ibm_ilog.diagram.Selection');
dojo.require('ibm_ilog.diagram.style');
dojo.require('ibm_ilog.diagram.Subgraph');
dojo.require("ibm_ilog.diagram.SwimLane");
dojo.require('ibm_ilog.diagram.templates.editor_templates');
dojo.require('ibm_ilog.diagram.templates.templates');
dojo.require('ibm_ilog.diagram.templating');
dojo.require('ibm_ilog.diagram.util.Batch');
dojo.require('ibm_ilog.diagram.util.ClippingUtil');
dojo.require('ibm_ilog.diagram.util.DeferredInitialization');
dojo.require('ibm_ilog.diagram.util.ErrorReporter');
dojo.require('ibm_ilog.diagram.util.Geometry');
dojo.require('ibm_ilog.diagram.util.GlobalInitialization');
dojo.require('ibm_ilog.diagram.util.GraphUtil');
dojo.require('ibm_ilog.diagram.util.HandleSet');
dojo.require("ibm_ilog.diagram.util.LimitedStack");
dojo.require("ibm_ilog.diagram.util.MultilineText");
dojo.require('ibm_ilog.diagram.util.PrintUtil');
dojo.require('ibm_ilog.diagram.util.Serializer');
dojo.require('ibm_ilog.diagram.widget.DataConnector');
dojo.require('ibm_ilog.diagram.widget.Diagram');
dojo.require('ibm_ilog.diagram.widget._Diagram');
dojo.require('ibm_ilog.diagram.widget.DivViewport');
dojo.require('ibm_ilog.diagram.widget.GfxViewport');
dojo.require('ibm_ilog.diagram.widget.HudComponent');
dojo.require('ibm_ilog.diagram.widget.OverviewHudComponent');
dojo.require('ibm_ilog.diagram.widget.ScrollbarHudComponent');
dojo.require('ibm_ilog.diagram.widget.ScrollbarInteractor');
dojo.require('ibm_ilog.diagram.widget.Viewport');
dojo.require('ibm_ilog.diagram.widget.ZoomHudComponent');
dojo.require('ibm_ilog.diagram.adorners.Adorner');
dojo.require('ibm_ilog.diagram.adorners.AdornerHandle');
dojo.require('ibm_ilog.diagram.adorners.AdornerWithHandles');
dojo.require('ibm_ilog.diagram.adorners.AnnotationAdorner');
dojo.require('ibm_ilog.diagram.adorners.HighlightedHandle');
dojo.require('ibm_ilog.diagram.adorners.LinkAdorner');
dojo.require('ibm_ilog.diagram.adorners.MoveAnnotationInteractor');
dojo.require('ibm_ilog.diagram.adorners.TemplatedOverlay');

});

},
'ibm_ilog/diagram/adorners/HighlightedHandle':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/HighlightedHandle", [
"../main",
"dojo/_base/declare",
"dojo/_base/array",
"./AdornerHandle",
"../util/HandleSet"
],
function(
iid,
declare,
array,
AdornerHandle,
HandleSet
){

/*=====
var HandleSet = ibm_ilog.diagram.util.HandleSet;
var AdornerHandle = ibm_ilog.diagram.adorners.AdornerHandle;
=====*/

var gfxCon = HandleSet.gfxConnector;
var gfxDis = HandleSet.gfxDisconnector;
	
var HighlightedHandle =
	iid.gfxDeclaration(declare('ibm_ilog.diagram.adorners.HighlightedHandle', [AdornerHandle, HandleSet],{
		
		_interactors: null,
		
		_state: "",
		
		_hovered: false,
		
		_active: false,
		
		constructor: function(){
			this._interactors = [];
		},
		
		initialize: function(adorner,template) {
			this._adorner = adorner;
			this._template = template;
			this.applyTemplate(this._template);
			
			this.remove(this.stNormal);
			this.remove(this.stHighlighted);
			this.remove(this.stActive);
			
			this._active = false;
			this._hovered = false;
			
			this.refreshState();
		},

		interactionBegun: function(interactor) {
			this._active = true;
			this.refreshState();
		},

		interactionEnded: function(interactor) {
			this._active = false;
			this.refreshState();
		},

		changeState: function(stId) {
			if(this._state !== stId) {
				if(this[this._state])
					this.remove(this[this._state]);
				//console.log(stId)
				this._state = stId;
				if(this[this._state])
					this.add(this[this._state]);
			}
		},
		
		addInteractor: function(interactor){
			this._interactors.push(interactor);
			interactor.setManager(this._adorner._diagram._interactors);
		},

		refreshState: function() {
			if(this._active) {
				this.changeState("stActive");
			} else {
				if(this._hovered) {
					this.changeState("stHighlighted");
				} else {
					this.changeState("stNormal");
				}
			}
		},

		onMouseOver: function(){
			this._hovered = true;
			this.refreshState();
		},
		
		onMouseOut: function(){
			this._hovered = false;
			this.refreshState();
		},		
		
		activate:function() {
			this._active = false;
			this._hovered = false;
			this.changeState("stNormal");
			array.forEach(this._interactors,function(item){item.activate();});
			this._connect("hoverStart",this,"onmouseover",this,"onMouseOver",gfxCon,gfxDis);
			this._connect("hoverEnd",this,"onmouseout",this,"onMouseOut",gfxCon,gfxDis);
		},
		
		deactivate:function() {
			array.forEach(this._interactors,function(item){item.deactivate();});
			this.changeState("");
			this._disconnect("hoverStart");
			this._disconnect("hoverEnd");
		}
	}));
	
	/* This must be added in each class that implements this class
	 * 
	 * HighlightedHandle.templateId = "TemplateName";
	 */

return HighlightedHandle;

});

},
'ibm_ilog/diagram/Node':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/Node", ["./main",
"dojo/_base/declare", "dojo/_base/array", "dojo/_base/config",
"dojox/gfx", "dojox/gfx/matrix",
"./GraphElement", "./templating", "./templates/templates", "./gfxlayout",
"./util/Geometry", "./util/ClippingUtil"
],
function(iid, declare, array, config, gfx, matrix, GraphElement, templating, templates, gfxlayout, g, ClippingUtil){
	
	var Rect = g.Rect, EmptyRect = g.EmptyRect, Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	
/*=====
var GraphElement = ibm_ilog.diagram.GraphElement;
=====*/

var Node = 
declare("ibm_ilog.diagram.Node", [GraphElement], {
    // summary:
    //		A Node is a graphic object that can be connected to other nodes using Link objects.
    
    // Allows to test the class faster that testing declaredClass
    _isIBMDiagramNode: true,

  	//
  	//	TID: String
  	//		Graph element Type ID, used to distinguish basic graph 
    //		element types:
    //	
    //			N = Node
    //			S = Subgraph
    //			L = Link
    //	
  	//		Usually used as a very fast index into maps based on 
    //		ibm_diagram graph element types.
  	//
    TID: 'N',
    
    _ports: null,
 
    constructor: function() {
        if (config.useGfxLayout) {
            this.setLayout(new gfxlayout.GridLayout());
        }
    },
    
    //---------- Disposal ----------
    
    _dispose: function() {
        // summary:
        //		Dispose this Node.
        // description:
        //		This function should be called when the node is completely deleted, that is, when it is removed
        //		from its parent graph and it will never be used again.
        
        array.forEach(this.getLinks(), function(link) {
            if (link.getStartNode() == this) 
                link.setStartPort(null);
            if (link.getEndNode() == this) 
                link.setEndPort(null);
        });
    },
    
    // ---------- Ports management: ----------
    
    addPort: function(/*ibm_ilog.diagram.PortBase*/port) {
        // summary:
        //		Adds a port to this node.
        // port: ibm_ilog.diagram.PortBase:
        //		The port to add.
        if (!this._ports) 
            this._ports = [port];
        else 
            this._ports.push(port);
        port._setOwner(this);
    },
    
    removePort: function(/*ibm_ilog.diagram.PortBase*/port) {
        // summary:
        //		Removes a port from this node.
        // port: ibm_ilog.diagram.PortBase:
        //		The port to remove.
        if (this._ports) {
            var index = array.indexOf(this._ports, port);
            if (index >= 0) {
                port._setOwner(null);
                if (this._ports.length == 1) 
                    this._ports = null;
                else 
                    this._ports.splice(index, 1);
            }
        }
    },
    
    getPorts: function() {
        // summary:
        //		Returns the ports of this node.
        // returns:
        //		An array of PortBase objects.
        // description:
        //		Note that this method always returns a newly allocated array.
        var ports = [];
        if (this._ports) {
            for (var i = 0; i < this._ports.length; i++) 
                ports.push(this._ports[i]);
        }
        return ports;
    },
    
    _invalidateLinks: function() {
        // tag: private
        // summary:
        //		Invalidates the shape of all the links connected to this node. 
        // description:
        //		This method is internal and should not be called directly by application code.
        if (this._ports) {
            for (var i = 0; i < this._ports.length; i++) {
                this._ports[i].invalidateLinks();
            }
        }
    },
    
	_setParent: function(parent, matrix)
	{
		GraphElement.prototype._setParent.call(this, parent, matrix);
        if (this._ports) {
            for (var i = 0; i < this._ports.length; i++) {
				var p = this._ports[i];
				if(p._links){
					for(var j = 0; j < p._links.length; j++)
						p._links[j]._forceCheckIntergraphLink();
				}
            }
        }
	},
	
    // ---------- Get connected links: ----------
    
    getLinks: function(starting, ending) {
        // summary: 
        //		Gets the links whose start and/or end nodes are equal to this node.
        // description:
        //		If starting is not false, the method returns all the links whose start nodes are this node.
        //		If ending is not false, the method returns all the links whose end nodes are this node.
        // starting: Boolean:
        //		If this argument is not false (e.g., true or undefined), links whose start nodes are equal to this node are returned.
        // ending: Boolean:
        //		If this argument is not false (e.g., true or undefined), links whose end nodes are equal to this node are returned.
        // returns:
        //		An array of ibm_ilog.diagram.Link objects.
        // example:
        //		node.getLinks()
        //		node.getLinks(true, true)
        //		node.getLinks(true)
        //				 These 3 calls return all the links connected to the node.
        // example:
        //		node.getLinks(true, false)
        //				 This call returns the links starting at the node.
        // example:
        //		node.getLinks(false)
        //		node.getLinks(false, true)
        //				 These 2 calls return the links ending at the node.
        // example:
        //		node.getLinks(false, false) 
        //				This call always returns an empty array.
        
        var result = [];
        if (this._ports !== null && (starting != false || ending != false)) {
            array.forEach(this._ports, function(p) {
                array.forEach(p.getLinks(), function(link) {
                    if ((starting != false && link.getStartPort() === p) ||
                    (ending != false && link.getEndPort() === p)) {
                        if (array.indexOf(result, link) < 0) 
                            result.push(link);
                    }
                }, this);
            }, this);
        }
        return result; // Array
    },
    
    // ---------- Base shape: ----------
    
    getBaseShape: function() {
        // summary:
        //		Gets the base shape of the node.
        // description:
        //		The base shape for various purposes, for example links will typically
        //		be connected to the base shape instead of the overall bounding rectangle of the node.
        //		By default, the base shape is the shape contained in the node template
        //		whose dojoAttachPoint property is set to "baseShape". In the default template,
        //		this is a rounded rectangle. If not baseshape exists in the template, the
        //		Node itself is returned.
        
        return this.baseShape || this;
    },
    
    // ---------- Bounds: ----------
    
    getBounds: function(/*dojox.gfx.Group*/parent) {
        // summary:
        //		Returns the bounds of this node in the coordinate space of the specified parent.
        // parent: dojox.gfx.Group
        //		The parent object that determines the coordinate space
        //		the bounds are relative to. If null or undefined, the bounds of the node
        //		relative to its parent are returned.
        // returns:
        //		A rectangle (i.e. an object with x/y/width/height properties) containing
        //		the bounds of the node in the specified coordinate space.

        var rect = this.getLocalBoundingBox();
        if (rect && parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            rect = t.transformRectangle(rect);
        }
        return rect;
    },
    
    setBounds: function(rect, /*dojox.gfx.Group*/ parent) {
        // summary:
        //		Changes the transform of this node so that its bounds will be a given rectangle,
        //		in the coordinate space of the specified parent.
        // rect:
        //		A rectangle (i.e. an object with x/y/width/height properties) containing
        //		the new bounds of the node in the specified coordinate space.
        // parent: dojox.gfx.Group
        //		The parent object that determines the coordinate space
        //		the bounds are relative to. If null or undefined, the bounds of the node
        //		relative to its parent are changed.
        
        if (parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            t = t.inverse();
            rect = t.transformRectangle(rect);
        }
        
        if (config.useGfxLayout) {
            // if this Node has a layout, set its clientRectangle
            if (this.getLayout()) {
                this.setExplicitSize(rect);
				this.move(rect.x, rect.y, parent);
            // maybe it contains a panel as only child
            } else if (this.children.length === 1 && this.children[0].children && this.children[0].getLayout()) {
                this.children[0].setExplicitSize(rect);
            // fallback: uses transform
            } else {
                var oldBounds = this.getLocalBoundingBox();
                if (oldBounds === null) {
                    this.applyLeftTransform(matrix.translate(rect.x, rect.y));
                } else if (rect.x != oldBounds.x ||
                rect.y != oldBounds.y ||
                rect.width != oldBounds.width ||
                rect.height != oldBounds.height) {
                    this.applyLeftTransform(matrix.createFromRectangles(oldBounds, rect));
                }
            }
        } else {
                var oldBounds = this.getLocalBoundingBox();
                if (oldBounds === null) {
                    this.applyLeftTransform(matrix.translate(rect.x, rect.y));
                } else if (rect.x != oldBounds.x ||
                           rect.y != oldBounds.y ||
                           rect.width != oldBounds.width ||
                           rect.height != oldBounds.height) {
                    this.applyLeftTransform(matrix.createFromRectangles(oldBounds, rect));
                }
        }
    },
    
    allowsResize: function() {
    	return false;
    },

    minimumSize: Size(10,10),
    
    move: function(/*number*/x, /*number*/ y, /*dojox.gfx.Group*/ parent) {
        // summary:
        //		Moves the node to the specified position.
        // description:
        //		This function changes the node's transform so that the top-left of its bounds
        //		(in the parent coordinate space) will be the specified 'x'and 'y' arguments
        
        
        if (parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            t = t.inverse();
            var p = {
                x: x,
                y: y
            };
            p = t.transformPoint(p);
            x = p.x;
            y = p.y;
        }

        var oldBounds = this.getLocalBoundingBox();
        if (oldBounds) {
            if (x != oldBounds.x || y != oldBounds.y) 
                this.applyLeftTransform(matrix.translate(x - oldBounds.x, y - oldBounds.y));
        } else {
            this.applyLeftTransform(matrix.translate(x, y));
        }
    },
    
    onBoundsChanged: function() {
        // summary:
        //		This event is called to signal that the bounds of the node have changed.
    },
    
    _onChanged: function(shape, reason) {
    	// This method is called a lot, keep it short and fast :).
    	
    	// <inlining this call from BoundingBoxFix at diagext.js>
    	// 
    	// this.inherited(arguments);
    	this._bbCacheValid = false;
    	// <end inline>

    	if(!Node._ChangesInvalidateLinksDisabled) {
    		this._invalidateLinks();
    	}
    	this.onBoundsChanged(shape,reason); // bounds probably changed
    },
    
    hitTest: function(p,graph,tolerance){
    	// Summary:
    	//		return if the given point hits the element
    	
    	if(tolerance == null){
    		tolerance = 0;
    	}
    	var bounds = this.getBounds(graph);
    	if(!bounds) {
    		return false;
    	}
    	var bb = g.expandRect(bounds,tolerance);
    	if(g.containsPoint(bb,p)) {
    		var shape = this.getBaseShape();
    		if(shape != this){
    			var referencePoint = g.Point(0,0);
    			var clipConnectionPoint = this._getExternalPoint(bb);
    			// This solution only works with closed shapes
    			var completeResult = {};
    			ClippingUtil.clipToShape(this,graph,shape,bounds,referencePoint, clipConnectionPoint, p,completeResult);
    			// If the point is inside the node, the number of intersection must be even
    			if( (completeResult.numberOfResults%2) == 1){
					return true;
				}else{
					return false;
				}
    		}else{
    			return true;
    		}
    	}
    	return false;
    },
    
    _getExternalPoint: function(bounds){
    	var margin = 30;//this margin is to ensure that the point is outside the node shape (as the bb is not correctly on path shapes, this is big)
    	return g.Point(bounds.x-margin,bounds.y-margin);
    },
    
    __eod: undefined
   
});

// Declare bindable properties
templating.declareBindableProperty(Node, "label", "");
templating.declareBindableProperty(Node, "wrap", true);
templating.declareBindableProperty(Node, "annotation", ""); // used by annotation adorners
templating.declareBindableProperty(Node, "tooltip", ""); // used to generate a tooltip on hover
templating.declareBindableProperty(Node, "backgroundColor", "#ADD7FF");
templating.declareBindableProperty(Node, "selectedBackgroundColor", "#0000FF");
templating.declareBindableProperty(Node, "borderColor", "#808080");
templating.declareBindableProperty(Node, "textColor", "#000000");
templating.declareBindableProperty(Node, "selectedTextColor", "#FFFFFF");
templating.declareBindableProperty(Node, "data", null);
templating.declareBindableProperty(Node, "textDir", "");

// Default node template
Node.defaultTemplate = iid.declareTemplate(templates.defaultNodeTemplate);

Node.nodeType = gfx.Group.nodeType;
Node.defaultShape = {
    type: 'node'
};

return Node;

});

},
'ibm_ilog/diagram/Selectable':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/Selectable", ["dojo/_base/declare", "dojox/collections/ArrayList"],
function(declare, ArrayList){

var Selectable =
declare("ibm_ilog.diagram.Selectable", [], {
	// summary:
	//		Selectable class group all the functionality to store and manage the selection state and style of a component.
	//      Link and Node must inherit this class in order to be selectable.
	
	// Allows to test the class faster that testing declaredClass
    _isIBMDiagramSelectable: true,
    
	// Internal selection state
	_selected: false,
	// enabled for select
	_selectable: true,
    // the list of styled
	_styledChildren : null,
   
    isSelected: function() {
	// summary:
	//		returns the selected state
	// returns:
	//      true if selected
		return this._selected;
	},
	setSelected: function(selected) {
		// summary:
		//		sets the selected state
		// selected:
		//		the new selection state
		// returns:
		//		true if the selection state could be changed
		this._selected = selected;
		this._changeSelectionStyle(selected);
		return true;
	},
	toggleSelected: function(){
		// summary:
		//		toggle the selection state
		// returns:
		//		true if the selection state could be changed
		var s = !this._selected;
		return this.setSelected(s);
	},
	setSelectable: function(selectable){
		// summary:
		//		enable or disable selecting
		// enabled:
		// 		true to enable selecting
		this._selectable = selectable;
	},
	isSelectable: function(){
		// summary:
		//		return if the selectable can be selected
		return this._selectable;
	},
	_changeSelectionStyle: function(selected){
		// summary:
		//		change the selection style
		// selected:
		//		the new selection state
		if(this._styledChildren == null){
			this._styledChildren = new ArrayList();
			this._findAllStyledElements();
		}
		var iterator = this._styledChildren.getIterator();
		while(!iterator.atEnd()){
			var shape = iterator.get();
			this._applySelectedStyle(shape,selected);
		}
	},
	
	_applySelectedStyle: function(shape,selected){
		// summary:
		//		change the selection style of the given shape
		// shape:
		//		the shape to be styled
		// selected:
		//		the new selection state	
		if(selected){
			if(!shape.onSelectionStyle || shape.__selectedStyleChanged){ // Added to fix Defect 344
				if(shape.__selectedStyleChanged){
					this._applyStyle(shape,shape['standardStyle']);
				}
				shape.standardStyle = this._getStandardStyle(shape,shape['selectedStyle']);
				this._applyStyle(shape,shape['selectedStyle']);
				shape.onSelectionStyle = true;
			}
		}else{
			this._applyStyle(shape,shape['standardStyle']);
			shape.standardStyle = null;
			shape.onSelectionStyle = false;
		}
		delete shape.__selectedStyleChanged;
	},
	_findAllStyledElements: function(){
		this._findStyledChildren(this.children);
	},
	_findStyledChildren: function(childrenArray){
		// summary:
		//		find all the children that have to be styled when selecting, and save the standard style
		for (var c in childrenArray){
			var child = childrenArray[c];
			if(!Selectable.Util.isSelectableInstance(child)) {
					if (child['selectedStyle']) {
						this._styledChildren.add(child);
					}
					if (child['children']) {
						this._findStyledChildren(child['children']);
					}
			}
		}
	},
	//TODO this function should be in a tool
	_applyStyle:function(shape, style){
		// summary:
		//		sets a new style to the desired element
		// shape:
		//		the shape to apply the style
		// style:
		//      the new style to be applied
		for (var p in style) {
			if (style.hasOwnProperty(p)){
				var setter = 'set' + p.charAt(0).toUpperCase() + p.substring(1);
				if (shape[setter]) {
					shape[setter](style[p]);
				}
			}
		}
	},
	_getStandardStyle:function(shape, style){
		// summary:
		//		gets the standard style of the given style element
		// shape:
		//		the shape to get the style
		// style:
		//      the style from where to get the elements
		var simpleStyle = {};
		for (var p in style) {
			if (style.hasOwnProperty(p)){
				var getter = 'get' + p.charAt(0).toUpperCase() + p.substring(1);
				if (shape[getter]) {
					simpleStyle[p] = shape[getter]();
				}
			}
		}
		return simpleStyle;
	}
});


Selectable.Util = {
	isSelectableInstance: function(ge){
		// Summary:
		//		this function returns if the given element is an instance of Selectable.
		//		It is a workaround to avoid using isInstanceOf (not supported on dojo 1.3.2)
		if(ge['_isIBMDiagramSelectable']){
			return true;
		}
		return false;
	}
}

return Selectable;

});

},
'ibm_ilog/diagram/adorners/AdornerWithHandles':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/AdornerWithHandles", [
"../main",
"dojo/_base/declare",
"dojox/collections/ArrayList",
"./Adorner"
],
function(
iid,
declare,
ArrayList,
Adorner
){

/*=====
var Adorner = ibm_ilog.diagram.adorners.Adorner;
=====*/

var sup = Adorner.prototype;

var a = iid.gfxDeclaration(declare('ibm_ilog.diagram.adorners.AdornerWithHandles', [Adorner], {

	//   
	//	_handles:
	//		The AdornerHandles created for the Adorner
	//
	_handles: null,
	
	initialize: function () {
		
		this._handles = new ArrayList();
		sup.initialize.apply(this,arguments);
	},

	activate: function () {
		//
		// 	summary:
		//		activates all the adorner handles
		//
		this._handles.forEach(function (e) {
			e.activate();
		});
	},

	deactivate: function () {
		//
		// 	summary:
		//		deactivates all the adorner handles
		//
		this._handles.forEach(function (e) {
			e.deactivate();
		});
	},

	_activateHandle: function (attachPointName) {
		if (this[attachPointName] !== undefined) {
			this[attachPointName].activate();
		}
	},

	_deactivateHandle: function (attachPointName) {
		if (this[attachPointName] !== undefined) {
			this[attachPointName].deactivate();
		}
	},

	_createHandle: function (clazz, attachPointName) {
		if (this[attachPointName]) {
			var template = this._templatePool.item(this[attachPointName].template || clazz.templateId);
			var handle = this[attachPointName].createObject(clazz);
			handle.initialize(this, template);
			this[attachPointName] = handle;
			this._handles.add(handle);
			return handle;
		} else {
			return null;
		}
	}
	
}));

return a;

});

},
'ibm_ilog/diagram/util/GlobalInitialization':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/GlobalInitialization", ["dojo/_base/lang", "dojo/_base/connect"],
function(lang, connect){
	
	// TODO: Change to this when AMD conversion is complete:
	// var gi = {};
	var gi = lang.getObject("ibm_ilog.diagram.util.GlobalInitialization", true);
	
	gi._initializing = 0;
	
	gi.then = function(cb) {
		//
		//	summary:
		//		Executes cb after all registered entities have initialized.
		//
		if(!gi._initializing) {
			cb();
		} else {
			var h = connect.connect(gi,"_finishedInit",function(){
				cb();
				connect.disconnect(h);
			});
		}
	};
	
	gi.beginInit = function() {
		//
		//	summary:
		//		Notify an entity begins initialization.
		//
		
		gi._initializing++;
	};
	
	gi._finishedInit = function() {

	};

	gi.endInit = function() {
		//
		//	summary:
		//		Notify an entity just finished initialization.
		//
		gi._initializing--;
		if(!gi._initializing) {
			gi._finishedInit();
		}
	};
	
	return gi;
});

},
'ibm_ilog/diagram/overview/OverviewInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewInteractor", [
"dojo/_base/declare",
"../_base/dojoext",
"../interactors/Interactor"
],
function(
declare,
dojex,
Interactor
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var OverviewInteractor =
declare("ibm_ilog.diagram.overview.OverviewInteractor", [Interactor],
{
		//
		//	summary:
		//		Abstract class with the basic behavior of an interactor which handles input into an Overview.
		//
		
		//
		//	_viewport: ibm_ilog.diagram.widget.Viewport
		//		The diagram viewport
		//
		_viewport: null,

		//
		//	_viewport: ibm_ilog.diagram.overview
		//		The overview
		//
		_overview: null,
		
		//
		//	_surface: dojox.gfx.Surface
		//		The surface
		//
		_surface: null,

		initialize: function( /*ibm_ilog.diagram.widget.Viewport*/viewport, /*ibm_ilog.diagram.overview*/ overview, /*dojox.gfx.Surface*/ surface ) {
			this._viewport = viewport;
			this._overview = overview;
			this._surface = surface;
			this._initialize();
			return this;
		},
		
		eventSurfaceLocation: function (e) {
			//
			//	Returns the client location of a mouse event generated by the event source
			//
			var r = dojex.coords(this._surface._parent, true);
			return {
				x: e.pageX - r.x,
				y: e.pageY - r.y
			};
		},		

		eventViewportLocation: function( /*Event*/ e ) {
			//
			//	summary:
			//		Returns the event location in content coordinates
			//
			var loc = this.eventSurfaceLocation(e);
			return this._overview.clientToContent().transformPoint(loc);
		}

});

return OverviewInteractor;	

});

},
'ibm_ilog/diagram/PortBase':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/PortBase", ["dojo/_base/declare", "dojo/_base/array", "dojox/gfx/matrix",
"./util/Geometry", "./util/ClippingUtil"],
function(declare, array, matrix, Geometry, ClippingUtil){

var PortBase = declare("ibm_ilog.diagram.PortBase", null, {
	// summary:
	//		The PortBase class represents the base class for ports.
	// description:
	//		A port defines a connection point between a Link and a Node.
	//		The PortBase class is intended as the base class for subclasses that
	//		define different types of ports. PortBase instances should not be created
	//		directly, only its subclasses should be used.
	
	_owner: null,
	_links: null,
	_clipOnShape: null,
	
	getOwner: function() {
		// summary:
		//		Gets the owner Node of this port.
		// returns:
		//		The Node to which this port belongs.
		return this._owner;
	},
	
	_setOwner: function(owner) {
		// summary:
		//		Sets the owner Node of this port.
		// owner: ibm_ilog.diagram.Node:
		//		The new owner of this port.
		// description:
		//		This method is internal and should not be called directly by application code.
		
		if (owner != this._owner) {
			this._owner = owner;
			this.invalidateLinks();
		}
	},
	
	getConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary: 
		//		Gets the connection point and the origin point for this port.
		// link: ibm_ilog.diagram.Link:
		//		The Link that is (or will be) connected to this port.
		// referencePoint:
		//		The opposite point of the segment connected to this port.
		// parent:
		//		The parent object (usually a Graph) that determines the coordinate space
		//		the connection point is relative to.
		// connectionPoint:
		//		An object (with 'x' and 'y' properties) into which the connection point will be stored.
		//		The connection point is the point where the link will start or end.
		// originPoint:
		//		An object (with 'x' and 'y' properties) into which the origin point will be stored.
		//		The origin point is the point towards which the link will 'aim' (typically the center of the node),
		//		and is used to determine the shape of orthogonal links for example.
		// returns:
		//		The 'connectionPoint' argument.
		// description:
		//		This is the central method of the PortBase class and is called by the Link object
		//		to get its start and end points. This method calls computeConnectionPoint, which is
		//		the method that actually calculates the connection point in the PortBase subclasses.
		//		Then, getConnectionPoint applies the clipping if getClipOnShape() is true.
		//		So, getConnectionPoint should not be overridden in subclasses, computeConnectionPoint
		//		is the method that is overridden instead.
		this.computeConnectionPoint(link, parent, referencePoint, connectionPoint, originPoint, start);
		if(this.getClipOnShape()){
			var p = this._clipToShape(link,parent,referencePoint, connectionPoint, originPoint, start);
			connectionPoint.x = p.x;
			connectionPoint.y = p.y;
		}
		return connectionPoint;
	},	
	
	computeConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary: 
		//		Computes the connection point and the origin point for a given subclass of PortBase.
		// link: ibm_ilog.diagram.Link:
		//		The Link that is (or will be) connected to this port.
		// referencePoint:
		//		The opposite point of the segment connected to this port.
		// parent:
		//		The parent object (usually a Graph) that determines the coordinate space
		//		the connection point is relative to.
		// connectionPoint:
		//		An object (with 'x' and 'y' properties) into which the connection point will be stored.
		//		The connection point is the point where the link will start or end.
		// originPoint:
		//		An object (with 'x' and 'y' properties) into which the origin point will be stored.
		//		The origin point is the point towards which the link will 'aim' (typically the center of the node),
		//		and is used to determine the shape of orthogonal links for example.
		// returns:
		//		The 'connectionPoint' argument.
		// description:
		//		This method is called by PortBase.getConnectionPoint, and should be overridden in
		//		PortBase subclasses to actually implement the calculation of the connection point.
		//		Clipping on the node's shape is implemented by getConnectionPoint, so
		//		computeConnectionPoint does not have to care about it.
		//		The base implementation for the PortBase class returns the center of the node for both
		//		the connection point and the origin point.
		
		var bounds = this.getConnectionBounds(parent);
		connectionPoint.x = originPoint.x = bounds.x + bounds.width / 2;
		connectionPoint.y = originPoint.y = bounds.y + bounds.height / 2;
		return connectionPoint;
	},
	
	getConnectionBounds: function(parent) {
		// summary:
		//		Gets the bounds of the owner node's base shape used to compute the connection point.
		// parent:
		//		The parent object (usually a Graph) that determines the coordinate space
		//		the bounds are relative to.
		// returns:
		//		A rectangle (with 'x', 'y', 'width' and 'height' properties) containing the bounds
		//		used to compute the connection point.
		// description:
		//		This method returns the bounds that will be used by the port to compute the connection and origin points.
		//		The default implementation returns the bounds of the base shape returned by getBaseShape().
		//		This method	can be overridden to return different bounds.
		
		if(!parent)
			parent = this._owner.getParent();
		
		var baseShape = this.getBaseShape();
        var rect = baseShape.getBoundingBox();
        var m = matrix;
        // t1: transform from surface -> base shape
        var t1 = baseShape._getRealMatrix() || m.identity;
        // t2: transform from surface -> parent
        var t2 = parent._getRealMatrix() || m.identity;
        var t = m.multiply(t2.inverse(), t1);
		rect = t.transformRectangle(rect);
        return rect;
	},
	
	needsReferencePoint: function() {
		// summary:
		//		Indicates whether or not this port needs a reference point to compute the connection point.
		// returns:
		//		True if this port needs a reference point to compute the connection point,
		//		or false otherwise.
		// description:
		//		This method is used by a PortBase subclass implementation to signal whether or not
		//		it needs (and uses) the referencePoint argument passed to the getConnectionPoint method.
		//		A subclass must return true if it actually uses the referencePoint, or false if it does
		//		not use it. If needsReferencePoint returns true, the caller of the getConnectionPoint method
		//		(usually the Link object) must pass a valid referencePoint. If needsReferencePoint returns false,
		//		the PortBase subclass guarantees that the referencePoint will never be used or referenced,
		//		so the caller may pass an invalid point, or a null or undefined object.
		//		By default the method returns true if getClipOnShape() returns true, of false otherwise.
		//		PortBase subclasses that always need a reference point must override this method and return true.
		
		return this._clipOnShape || false;
	},
	
	invalidateLinks: function() {
		// summary:
		//		Invalidates the shape of all the links connected to this port. 
		// description:
		//		Invalidates the shape of all the links connected to this port. 
		//		This method must be called by PortBase subclasses when a change of the port
		//		causes the connection point to change.
		if(this._links)
			for(var i = 0; i < this._links.length; i++)
				this._links[i].invalidateLinkShape();
				
	},
	
	_addLink: function(link) {
		// summary:
		//		Adds a link to the list of links connected to this port.
		// description:
		//		This method is internal and should not be called directly by application code.
		if(!this._links)
			this._links = [ link ];
		else
			this._links.push(link);
	},
	
	_removeLink: function(link) {
		// summary:
		//		Removes a link from the list of links connected to this port.
		// description:
		//		This method is internal and should not be called directly by application code.
		if (this._links) {
			var index = array.indexOf(this._links, link);
			if (index >= 0) {
				if(this._links.length == 1)
					this._links = null;
				else
					this._links.splice(index, 1);
			}
		}
	},
	
	getLinks: function() {
		// summary:
		//		Gets the links connected to this port.
		// returns:
		//		An array of Links connected to this port.
		// description:
		//		Note that this method always returns a newly allocated array.
		var links = [];
		if (this._links) {
			for(var i = 0; i < this._links.length; i++)
			links.push(this._links[i]);
		}
		return links;
	},
	
	getLinksCount: function() {
		// summary:
		//		Gets the number of links connected to this port.
		
		return this._links ? this._links.length : 0;
	},

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////   CLIPPING TOOLS ///////////////////////////////////////////////////////////////////	
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

	setClipOnShape: function(clipOnShape){
		// summary:
		//		Determines whether the connection point is "clipped" on the base shape of the node.
		// description:
		//		If clipOnShape is true, the getConnectionPoint method will extend the segment defined
		//		by the reference point and the connection point so that the connection point is on the
		//		outline of the node's shape, so that the connection point always "touches" the node shape
		//		even if the node is not rectangular (for example, if the node has an elliptical of polygonal shape).
		//		If clipOnShape is false, the getConnectionPoint always returns the return of computeConnectionPoint
		//		without modifying it, and the point may not be exactly on the boundary on the node shape.
		// clipOnShape:
		//		If true, the connection point is clipped on the node's shape, otherwise the connection point
		//		is the point returned by computeConnectionPoint.
		
		if (clipOnShape != this._clipOnShape) {
			this._clipOnShape = clipOnShape;
			this.invalidateLinks();
		}
	},
	
	getClipOnShape: function(){
		// summary:
		//		Gets the flag that determines whether the connection point is "clipped" on the base shape of the node.
		// description:
		//		If clipOnShape is true, the getConnectionPoint method will extend the segment defined
		//		by the reference point and the connection point so that the connection point is on the
		//		outline of the node's shape, so that the connection point always "touches" the node shape
		//		even if the node is not rectangular (for example, if the node has an elliptical of polygonal shape).
		//		If clipOnShape is false, the getConnectionPoint always returns the return of computeConnectionPoint
		//		without modifying it, and the point may not be exactly on the boundary on the node shape.
		// returns:
		//		True if the connection point is clipped on the node's shape, or false otherwise.
		
		return this._clipOnShape;
	},
	
	getBaseShape : function() {
		// summary:
		//		Gets the shape contained in the owner node's template to which links should be connected.
		// description:
		//		The base shape is used by getConnectionBounds to compute the default connection bounds,
		//		and also when clipOnShape is true to clip the connection point if the base shape is
		//		non-rectangular.
		//		By default the base shape is determined by calling the getBaseShape() of the owner node,
		//		but this can be overridden to connect links to a different template part of the node.
		
		return this._owner.getBaseShape();
	},
	
	_clipToShape: function(link,parent,referencePoint, connectionPoint, originPoint, start){
		
		var node = this._owner;
		var shape = this.getBaseShape();
		var bounds = this.getConnectionBounds(parent);
		
		if(shape == node){
			var shape = Geometry.Rect(bounds.x,bounds.y,bounds.width,bounds.height);
		}
			
		var resClipPoint = ClippingUtil.clipToShape(node,parent,shape,bounds,originPoint, connectionPoint, referencePoint)

		return resClipPoint?resClipPoint:connectionPoint;
	}
		
});

return PortBase;

});

},
'ibm_ilog/diagram/overview/OverviewPanInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewPanInteractor", [
"dojo/_base/declare",
"./OverviewInteractor",
"../interactors/DragInteractor",
"../util/Geometry",
"../util/GraphUtil"
],
function(
declare,
OverviewInteractor,
DragInteractor,
g,
gu
){

// Note: OverviewInteractor must be last in the superclass array since it overrides initialize()

/*=====
var DragInteractor = ibm_ilog.diagram.interactors.DragInteractor;
var OverviewInteractor = ibm_ilog.diagram.overview.OverviewInteractor;
=====*/

var OverviewPanInteractor =
declare("ibm_ilog.diagram.overview.OverviewPanInteractor", [DragInteractor, OverviewInteractor], {
	//
	//	summary:
	//		An OverviewPanInteractor pans the viewport when the user drags on 
	//		the overview and centers the viewport on a point when the user clicks at 
	//		that point.	//

	_dragCursor: "move",


	//
	//	_constrain: Boolean
	//		Constrains view rectangle movements to the viewport's content area
	//
	_constrain: true,
	
	setConstrain: function( /*boolean*/ b ) {
		this._constrain = b;
	},
	
	getConstrain: function() {
		return this._constrain;
	},
	
	_initialize: function() {
		this.inherited(arguments);
		this._cursorStyle = gu.getSurface(this._overview._output).rawNode.style;
	},
	
	_declareStates: function () {
		this._declareState("idle", ["fit","start","mouseenter","mouseleave"]);
		this._declareState("active", ["first", "move", "end", "cancelOnDragStart", "cancelOnSelectStart","mouseenter","mouseleave"]);
	},
	
	getDefaultConnections: function() {
		var cs = this.inherited(arguments);
		cs.fit = {
				src: this._getInitialEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_fitContents",
				filter: this._buildInputFilter({shift:true,button:0}),
				gfxConnect:true
			};
		cs.mouseenter = {
				src: this._getInitialEventSource(),
				srcEvt: "onmouseenter",
				connectTo: "_hover",
				gfxConnect:true
			};
		cs.mouseleave = {
				src: this._getInitialEventSource(),
				srcEvt: "onmouseleave",
				connectTo: "_leave",
				gfxConnect:true
			};
		return cs;
	},
	
	_updateCursor: function() {
		if(this._hovered || this._state._id=="active") {
			this._showDragCursor();
		} else {
			this._hideDragCursor();
		}
	},
	
	_hover: function() {
		this._hovered = true;
		this._updateCursor();
	},
	
	_leave: function() {
		this._hovered = false;
		this._updateCursor();
	},

	_getInitialEventSource: function() {
		return this._overview.getEventSource(); 
	},
	
	_fitContents: function(e) {
		this._viewport.setViewRect(this._viewport.getContentRect());
	},

	_dragPreStart: function(e) {
		this._cursorStyle = this._cursorStyle || gu.getSurface(this._overview._output);
		var v = this._viewport;
		this.inherited(arguments);
		this._firstViewRect = v.getViewRect();
	},
    
	_dragMove: function(e) {
    this.inherited(arguments);

		// scale delta to zoom level to get delta in model coords
		var panDelta = g.mulPoint(this._totalDelta,1/this._overview.getZoom());
		
		var newViewRect = this._constrainedViewRect(g.moveRect(this._firstViewRect,panDelta));
		
		this._viewport.setViewRect( newViewRect, {instant:true} );
  },
	
	_dragEnd: function(e) {
		//
		//	summary:
		//		Terminates the current panning process.
		//
		var v = this._viewport;
		var dragged = this.hasDragged; // inherited will clear this;
		this.inherited(arguments); // clears hadDragged
		if(!dragged) {
			
			var newViewRect = this._constrainedViewRect(g.rectCenteredAt(this._viewport.getViewRect(), this.eventViewportLocation(e)));

			this._viewport.setViewRect( newViewRect );
		}
	},

  _constrainedViewRect: function( viewRect ) {
	  if(this._constrain) {
			viewRect = g.constrainRect(viewRect,g.addRect(this._viewport.getContentRect(),this._viewport.getViewRect()));
		}
	  return viewRect;
  }

});

return OverviewPanInteractor;

});

},
'ibm_ilog/diagram/interactors/Interactor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/Interactor", [
"../main",
"dojo/_base/declare", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/sniff",
"dojox/collections/Dictionary", "dojox/gfx",
"../_base/dojoext", "../util/HandleSet", "../util/ErrorReporter", "../Selectable"
],
function(
iid,
declare, lang, arr, has,
Dictionary, gfx,
dojex, HandleSet, R, Selectable
){
	
var InteractorState =
declare("ibm_ilog.diagram.interactors.InteractorState", [], {
	//
	//    summary:
	//        Represents a state of an interactor in terms of
	//      the connections are active during it.
	//
	_connections: null,

	_id: "",

	_interactor: null,

	constructor: function (interactor, id, connections) {
		this._interactor = interactor;
		this._id = id;
		this._connections = connections;
	},
	
	addConnection: function(c) {
		this._connections.push(c);
	},

	_forEachId: function (fname, s) {
		arr.forEach(this._connections, s[fname], s);
	},

	enter: function () {
		this._forEachId("_doConnect", this._interactor);
	},

	leave: function (host) {
		this._forEachId("_disconnect", this._interactor);
	}
});

var gfxCon = HandleSet.gfxConnector;
var gfxDis = HandleSet.gfxDisconnector;

/*=====
var HandleSet = ibm_ilog.diagram.util.HandleSet;
=====*/

var Interactor =
declare("ibm_ilog.diagram.interactors.Interactor", [HandleSet], {
	//
	//    summary:
	//        The interactor defines the API and common functionality for all the event management interactors.
	//
	//    description:
	//        The 
	//

	//
	//    The map with the connections used by the interactor
	//
	_connections: null,

	//
	//    _enabled:
	//        temporarily ignore events, but keeps dojo.connections
	//
	_enabled: true,

	//
	//    _manager:
	//        the mouse interactor manager which coordinates the use of the interactor
	//
	_manager: null,
	
	//
	//    _keyManager:
	//        the keyboard interactor manager which coordinates the use of the interactor
	//
	_keyManager: null,
	
	//
	//    _states:
	//        the states declared for this interactor
	//
	_states: null,

	//
	//    _state:
	//        the current state of the interactor
	//
	_state: null,

	//
	//    _hotKeys:
	//        the current hotKeys of the interactor
	//			format
	//			{ keyCode1:[connectionId1,connectionId2], keyCode2:[connectionId2] }
	//
	_hotKeys: null,

	_declareState: function (id, cids) {
		//
		//    summary:
		//        declares a state of the interactor
		//
		this._states.add(id, new InteractorState(this, id, cids));
	},

	_goStateId: function (stateId) {
		//
		//    summary:
		//        enters the state with the given id
		//
		this._goState(this._states.item(stateId));
	},

	_goState: function (s) {
		//
		//    summary:
		//        enters the state s
		//
		if (this._state !== s) {
			if (this._state) {
				this._state.leave(this);
			}
			this._state = s;
			if (this._state) {
				this._state.enter(this);
			}
		}
	},

	_declareStates: function () {
		//
		//    summary:
		//        subclasses must override this method for declaring interactor states
		//
	},

	setManager: function (manager) {
		this._manager = manager;
	},
	
	setKeyManager: function (keyManager) {
		this._keyManager = keyManager;
		this.registerHotKeys();
	},
	
	_initialize: function () {
		//
		//    summary:
		//        basic internal initialization
		//
		this.setConnections(this.getDefaultConnections());
		this._states = new Dictionary();
		this.initializeFilters();
		this._hotKeys = {};
		this._declareStates();
		return this;
	},

	_doConnect: function (id) {
		//
		//    summary:
		//        connects the connection declared with the given id, used by state transitions
		//
		var cdata = this._connections[id];
		if (cdata && !cdata.hotKey) {
			var target = cdata.target || this;
			if (cdata.connectTo in target) {
				var filteredHandler = cdata.filteredHandler;
				try{
					this._connect(id, cdata.src, cdata.srcEvt, this, filteredHandler, cdata.gfxConnect && gfxCon, cdata.gfxConnect && gfxDis);
				} catch(e) {
					R.warn("EventConnectionFailed", cdata.srcEvt);
				}
			} else {
				R.error("InvalidInteractorEvent", cdata.connectTo);
			}
		} else {
			R.error("InvalidInteractorId",id);
		}
	},
	
	keyPressed: function(ids,e){
		var managed = false;
		for(var index in ids){
			var cdata = this._connections[ids[index]];
			if(cdata.hotKey){
				managed = cdata.filteredHandler(e) || managed;
			}
		}
		return managed;
	},
	
	initializeFilters: function () {
		//
		//    summary:
		//       
		//
		for(var id in this._connections){
			var cdata = this._connections[id];
			var target = cdata.target || this;
			if (cdata.connectTo in target) {
				var filter = this._applyFilter(target, cdata.connectTo, cdata.filter);
				cdata.filteredHandler = filter;
			} else {
				R.error("InvalidInteractorEvent", cdata.connectTo);
			}
		}
	},

	registerHotKeys: function(){
		if(this._keyManager){
			this._keyManager.clearHotKeys(this.getKeyInteractorId(),this._hotKeys);
			this._hotKeys = {};
			for(var id in this._connections){
				var cdata = this._connections[id];
				var hk = cdata.hotKey;
				if(!(hk in this._hotKeys)){
					this._hotKeys[hk] = [];
				}
				this._hotKeys[hk].push(id);
			}
			this._keyManager.setHotKeys(this.getKeyInteractorId(),this._hotKeys);
		}
	},
	
	setEnabled: function (b) {
		if (this._enabled != b) {
			this._enabled = b;
			if (b) {
				this.onEnabled();
			}
			if (!b) {
				this.onDisabled();
			}
		}
	},

	getEnabled: function () {
		return this._enabled;
	},

	onEnabled: function () {},
	onDisabled: function () {},

	activate: function () {
		//
		// summary:
		//        Activates "idle" state
		//
		this._goStateId("idle");
	},

	deactivate: function () {
		//
		// summary:
		//        Disconnects all connections by going to the null state
		//
		this._goState(null);

		// shut down other temporary connections
		this._disconnectAll();
	},

	_fconnect: function (id, source, event, obj, handlerName) {
		//
		// summary:
		//        Makes a temporary connection with the standard enabled-only filter
		//
		switch (arguments.length) {
		case 5:
			return this._connect(id, source, event, obj, this._applyFilter(obj, handlerName));
		default:
			R.error("InvalidNumberOfArguments", arguments, 5);
		}
	},

	toString: function () {
		return this.declaredClass.split(".").pop();
	},

	setConnections: function (connections) {
		//
		//    summary:
		//        sets a new connections configuration
		//
		this._connections = connections;
	},

	overrideConnections: function (connections) {
		//
		//    summary:
		//        Override the default event map with the given handlers
		//
		this._connections = lang.clone(this._connections);
		lang.mixin(this._connections, connections);
	},

	getConnections: function () {
		//
		//    summary:
		//        return the eventMap
		//
		return this._connections;
	},

	getDefaultConnections: function () {
		//
		//    summary:
		//        This function has to be overriden by subclasses, and used to return the default event map for the Interactor
		//
		return {};
	},

	reactivate: function () {
		//
		// summary:
		//        release all the handles and reconnect.
		//
		this.deactivate();
		this.activate();
	},

	_basicBuildInputFilter: function(key,value) {
		if(value===undefined)
			R.error("InvalidInputFilterSpec",key);
		switch(key) {
		case "condition":
			return value;
		case "button":
			switch(value) {
			case 0:
				return this.eventButtonIsLeft;
			case 1:
				return this.eventButtonIsRight;
			case 2:
				return this.eventButtonIsMiddle;
			default:
				R.error("InvalidInputFilterSpec",key);
			}
		case "alt":
			return function(e) {
				return e.altKey === value;
			};
		case "ctrl":
			return function(e) {
				return e.ctrlKey === value;
			};
		case "shift":
			return function(e) {
				return e.shiftKey === value;
			};
		case "disabled":
			return function(e) {
				return false;
			};
		default:
			R.error("InvalidInputFilterSpec",key);
		}
	},
	
	_buildInputFilter: function(spec) {
		var filter = null;
		for(s in spec) {
			var newFilter = this._basicBuildInputFilter(s,spec[s]);  
			if(spec.hasOwnProperty(s)) {
				if(filter) {
					(function(oldFilter,newFilter) { // fresh scope
						filter = function(e) {
							return oldFilter(e) && newFilter(e);
						};
					})(filter,newFilter);
				} else {
					filter = newFilter;
				}
			}
		};
		return filter;
	},
	
	_assignEventInput: function(id,spec) {
		if('filter' in spec){
			this._connections[id].filter = this._buildInputFilter(spec.filter);
		}
		if('hotKey' in spec){
			this._connections[id].hotKey = spec.hotKey;
		}
	},	
	
	assignInputs: function(assignments) {
		for(id in assignments) {
			if(this._connections.hasOwnProperty(id)) {
				this._assignEventInput(id,assignments[id]);
			} else {
				R.error("InvalidInteractorEvent",id);
			}
		}
		this.initializeFilters();
		this.registerHotKeys();
		if (this._state !== null) {
			this.reactivate();
		}
	},
	
	_applyFilter: function (scope, handlerName, filter) {
		//
		// summary:
		//        if the filter function return true, calls the internal handler given as parameter
		// methodName:
		//        the handler to be called
		// filter:
		//         the filter function to be evaluated
		//
		var thiz = this;
		if (filter) {
			return function (e) {
				if (thiz._enabled && filter(e)) {
					scope[handlerName](e);
					return true;
				}
				return false;
			};
		} else {
			return function (e) {
				if (thiz._enabled) {
					scope[handlerName].apply(scope, arguments);
					return true;
				}
				return false;
			};
		}
	},

	_getGraphElementFromEventInRoot: function (event, root) {
		// Summary:
		//        Returns the GraphElement (i.e. Node or Link) containing the target of the specified event. 
		// event:
		//        The event.
		// root:
		//        The root GFX shape containing the graph elements.
		
		var gfxTarget = event.gfxTarget;
		if (gfxTarget) {
			// We are using a Dojo that adds a gfxTarget field to the event (Dojo 1.6+):
			// use this and look upwards to find the GraphElement.
			var ge = gfxTarget; // a GFX shape
			if (Selectable.Util.isSelectableInstance(ge)) {
				return ge;
			}
			if (!ge.getParent) 
				return null;
			for (var elem = ge.getParent(); elem && elem.getParent; elem = elem.getParent()) {
				if (Selectable.Util.isSelectableInstance(elem)) {
					return elem;
				}
				if (elem === root) 
					break;
			}
		}
		else {
			// We are using an older Dojo: use the old method, that is, search recursively
			// top-to-bottom from the root to find the GFX shape whose event source (= HTML node)
			// is the event target. This does not work for Canvas, so for Canvas we must be in
			// the case above (which means Canvas is not support for "old" Dojo versions).
			return this._getGraphElementFromHtml(event.target, root);
		}
		return null;
	},

	_getGraphElementFromHtml: function(html, root) {
		// Summary:
		//        Returns the GraphElement (i.e. Node or Link) containing the specified (HTML) event target.
		// html: Node
		//        The event target (an HTML node).
		// root:
		//        The root GFX shape containing the graph elements.
		
		// fix for touch event
		// on iOS the event target on a gfx text is a TextNode no the SVGText
		if (!iid.isSilverlight && html.nodeType == 3) 
			html = html.parentNode;
		
		if (gfx.equalSources(root.getEventSource(), html)) {
			return root;
		} else {
			if (root['children']) {
				for (var index in root.children) {
					var ge = this._getGraphElementFromHtml(html, root.children[index]);
					if (ge) {
						if (Selectable.Util.isSelectableInstance(ge)) {
							return ge;
						}
						else {
							return root;
						}
					}
				}
			}
		}
		
		return null;
	},

	__eod: undefined
});

//////////////////////////////////////////////////////////////////////////////
//
//    DOM Events Helper Functions
//
// dojo 1.4 has this already, but we need to support 1.3.2
if (has("ie") < 9) {
	if(gfx.renderer == 'vml') {
		lang.extend(Interactor, {
			eventButtonIsLeft: function (e) {
				return e.button & 1;
			}, eventButtonIsMiddle: function (e) {
				return e.button & 4;
			}, eventButtonIsRight: function (e) {
				return e.button & 2;
			}
		});
	} else {
		if(gfx.renderer == 'silverlight') {
			lang.extend(Interactor, {
				eventButtonIsLeft: function (e) {
					return true;
				}, eventButtonIsMiddle: function (e) {
					return false;
				}, eventButtonIsRight: function (e) {
					return false;
				}
			});
		}
	}
} else {
	lang.extend(Interactor, {
		eventButtonIsLeft: function (e) {
			return e.button == 0;
		}, eventButtonIsMiddle: function (e) {
			return e.button == 1;
		}, eventButtonIsRight: function (e) {
			return e.button == 2;
		}
	});
}

if (has("mozilla")) {
	lang.extend(Interactor, {
		eventNameWheel: "DOMMouseScroll",
		eventWheelAmmount: function (e) {
			return -e.detail;
		}
	});
} else {
	lang.extend(Interactor, {
		eventNameWheel: "onmousewheel",
		eventWheelAmmount: function (e) {
			return e.wheelDelta;
		}
	});
}

lang.extend(Interactor, {
	eventPageLocation: function (e) {
		//
		//    summary:
		//        returns pointer position associated with the given event
		//
		return {
			x: e.pageX,
			y: e.pageY
		};
	}, eventClientLocation: function (e) {
		//
		//    summary:
		//        returns pointer position associated with the given event
		//
		var r = dojex.coords(e.currentTarget);
		return {
			x: e.clientX - r.x,
			y: e.clientY - r.y
		};
	}
});

return Interactor;

});
},
'ibm_ilog/diagram/interactors/InteractorManager':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/InteractorManager", [
"dojo/_base/declare",
"dojo/_base/array",
"dojox/collections/Dictionary",
"dojox/collections/Set",
"dojox/collections/ArrayList",
"./Interactor",
"../util/ErrorReporter"
],
function(
declare,
arr,
Dictionary,
Set,
ArrayList,
Interactor,
R
){

var InteractorManager =
declare("ibm_ilog.diagram.interactors.InteractorManager", [], {
	//
	//	summary:
	//		Organizes interactors by id and is responible for distinguishing 
	//		the currently active interactors.
	//
	
	//
	//	_interactors: 
	//		Registered interactors
	_interactors:null,
	
	//
	//	_stack: 
	//		Holds active interaction (currently only one level is used)
	_stack:null,
	
	//
	//	_base: 
	//		Idle interactors, which are active when _stack is empty
	_base:null,
	
	constructor:function(){
		//
		//	summary:
		//		creates a new instance of InteractorManager
		//
		this._interactors = new Dictionary();
		this._base = this._gatherInteractors([]);
		this._stack = [];
	},
	
	add:function(id,interactor){
		//
		//	summary:
		//		register a new interactor with the given id
		//
		this._interactors.add(id,interactor);
		interactor._manager = this;
	},

	get:function(id){
		//
		//	summary:
		//		Returns the interactor registered with the given id
		//
		var interactor = this._interactors.item(id);
		if(!interactor) {
			R.error("InvalidInteractorId",id);
		}
		return interactor;
	},
	
	top:function(){
		//
		//	summary:
		//		Returns the top level of the stack
		//
		return this._stack[this._stack.length-1];
	},

	activeInteractors:function(){
		//
		//	summary:
		//		Returns a set with the currently active interactors
		//
		return this._stack.length?this.top().interactors:this._base;
	},

	_createStackObject:function(interactors){
		//
		//	summary:
		//		Returns a new stack object (also used as handle for popping)
		//
		return {interactors:interactors, active:true, index:this._stack.length};
	},

	_activate: function() {
		//
		//	summary:
		//		Activate the active set (currently: either _base or _stack[0])
		//
		this.activeInteractors().forEach(function(i){
			i.activate();
		},this);
	},

	_deactivate: function() {
		//
		//	summary:
		//		Deactivate the active set (currently: either _base or _stack[0])
		//
		this.activeInteractors().forEach(function(i){
			i.deactivate();
		},this);
	},
	
	_gatherInteractors: function( /*array*/ maybeNames ) {
		//
		//	summary:
		//		Gather a set of interactor that are listed explicitly of 
		//      by name into a dojox.collections.Set
		//
		var interactors = new ArrayList();
		arr.forEach(maybeNames,function(i){
			if(typeof i=="string") {
				interactors.add(this.get(i));
			} else if (i instanceof Interactor) {
				interactors.add(i);
			} else {
				R.error("UnexpectedArgument");
			}
		},this);
		return interactors;
	},	
	
	switchTo:function(interactor1,interactor2,etc){
		//
		//	summary:
		//		Changes the set of idle interactors that are ready to be used. 
		//		Has immediate effect only when there are no elements in the active _stack.
		//
		var interactors = this._gatherInteractors(arguments); 

		if(!this._stack.length) {
			this._deactivate();
			this._base = interactors;
			this._activate();
		} else {
			this._base = interactors;
		}
	},
	
	push:function(interactor1,interactor2,etc){
		//
		//	summary:
		//		Push a set of interactors into the active stack. (Although currently only 1 is supported)
		//
		if(this._stack.length) {
			R.error("RestrictedToOneInteractor");
		}
		
		var interactors = this._gatherInteractors(arguments);
		var keep = [];
		var old = this.activeInteractors();
		var so = this._createStackObject(interactors);
		this._stack.push(so);
		var n = this.activeInteractors();
		Set.difference(old,n).forEach(function(i){
			i.deactivate();
		});
		
		return this.top(); // this is the handle for deactivation
	},

	pop:function(h){
		//
		//	summary:
		//		Deactivates and eventually pops the interactors previously pushed on the active stack
		//
		if(h.active) {
			h.active = false;
			if(h == this.top()) {
				var old = this.activeInteractors();
				var i = h.index;
				while(i>=0 && !this._stack[i].active) {
					this._stack.pop();
					i--;
				}
				var n = this.activeInteractors();

				Set.difference(n,old).forEach(function(i){
					i.activate();
				});
			}
		}
	},
	
	stackLength: function(){
		return this._stack.length;
	},

	__eod:undefined

});

return InteractorManager;

});
},
'ibm_ilog/diagram/widget/Viewport':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/Viewport", ["../main", 
		"dojo/_base/declare", 
		"dojo/_base/lang", 
		"dojo/_base/sniff", 
		"dojo/_base/connect", 
		"dojo/_base/fx", 
		"dojo/_base/window", 
		"dojo/dom-geometry",
		"dojo/dom-attr",
		"dojo/dom-class", 
		"dojo/dom-style", 
		"dojox/gfx", 
		"dojox/gfx/matrix", 
		"../_base/dojoext", 
		"../util/DeferredInitialization", 
		"../util/Geometry", 
		"../util/ErrorReporter"], 
		function(iid, declare, lang, has, connect, fx, window, domg, doma, domClass, domstyle, gfx, matrix, dojex, DeferredInitialization, g, R){

	var EmptyRect = g.EmptyRect, Size = g.Size;
	
	/*=====
	 var DeferredInitialization = ibm_ilog.diagram.util.DeferredInitialization;
	 =====*/
	var Viewport = declare('ibm_ilog.diagram.widget.Viewport', [DeferredInitialization], {
		//	summary:
		//		A viewport window to a rectangular area of the diagram that is drawn on the browser.
		//		The drawn area is usually controlled by two scrollbars, horizontal and vertical.
		//
		//	description:
		//
		//		The Viewport class is an abstract class that provides basic infrastructure over which
		//		to construct concrete Diagram viewports.
		//
		//		A Viewport object is a collaborator of the Diagram widget that abstracts this notion 
		//		of viewport. The Diagram widget can then deal with the the viewport independently of 
		//		how it is actually implemented. 
		//
		//		The main responsibility of the viewport object is to control the current viewable 
		//		rectangle of the Diagram and report updates to it.
		//	
		//		The following concept definitions help understand the Viewport API:
		//			
		//		-	Content Rectangle    The area inside which the viewport can move, mostly based 
		//								 on the content bounding box.
		//
		//		-	View Rectangle       The rectangular portion of the content that is currently 
		//			                     visible in the viewport.
		//
		//		-	Client Size          The size of the UI space that allows the contents to be 
		//			                     viewed.
		//
		//	authors: AV
		
		_isIBMDiagramViewport: true,
		
		// ========================================================================================================
		//
		//	BASIC COLLABORATORS AND INITIALIZATION:
		//
		//	
		//
		//
		//	_diagram:
		//		The associated Diagram widget.
		//
		_diagram: null,
		
		//
		//	_viewportMatrix:
		//		The transformation matrix applied to the content contents for displaying the in the
		//		viewport. Used to transform model coordinates to view coordinates.
		//
		_viewportMatrix: matrix.identity,
		
		//
		//	_viewportMatrix:
		//		The inverse of the transformation matrix applied to the content contents for 
		//		displaying the in the viewport. Used to transform view coords into model coords.
		//
		_viewportMatrixInverse: matrix.identity,
		
		
		templateString: function(){
			//
			//	summary:
			//		This method is must return the template that the Diagram widget must use 
			//		in order to use the receiver as viewport.
			//
			//	diagram: ibm_ilog.diagram.widget.Diagram
			//		The diagram
			//
			R.error("SubclassResponsibility", "templateString");
		},
		
		//
		//	_needsHudRelayout:
		//		Flag to indicate that we need to relayout the hud and probably resize the surface also.
		//
		_needsHudRelayout: false,
		
		initialize: function( /*ibm_ilog.diagram.widget.Diagram*/diagram){
			//
			//	summary:
			//		Constructs a Viewport for collaborating with the diagram.
			//
			//	diagram: ibm_ilog.diagram.widget.Diagram
			//		The diagram
			//
			this._beginInit();
			
			this._hudComponents = [];
			
			this._diagram = diagram;
			
			this._createSurface();
			
			this._surface.whenLoaded(this, function(){
				this.setViewRect(this._clientSize, {
					instant: true
				});
				this._endInit();
			});
		},
		
		_getSurfaceSize: function(){
			return this.getHudSize();
		},
		
		_updateSurface: function(){
			var hz = this._getSurfaceSize();
			
			if (!has("ie") && !iid.isCanvas) {
				this.getSurface().setDimensions(hz.width + "px", hz.height + "px");
			} else {
				this.getSurface().setDimensions(hz.width, hz.height);
			}
			if (doma.get(document.body, "dir") == "rtl"){
				var offset = this._getSurfaceXOffset();
				var prt = this.getSurface()._parent;				
				domstyle.set(prt, "left", offset+"px");
			};
		},
		
		_getSurfaceXOffset:function(){
			return 0;
		},
		
		_eventBackground: null,
		
		
		_eventfulContent: null,
		
		getEventSourceBackground: function(){
			return this._underlayLayer;
		},
		
		getEventSource: function(){
			return this._eventfulContent;
		},
		
		_createEventSourceShape: function(){
			this._eventBackground = this._underlayLayer.createRect();
			if (domClass.contains(window.doc.body, "dijit_a11y")) // high contrast
				this._eventBackground.setFill([238, 238, 238, 1]);
			else this._eventBackground.setFill([0, 0, 0, 0.0001]);
			this._updateEventSourceShape();
		},
		
		getBackgroundShape: function(){
			return this._eventBackground;
		},
		
		_updateEventSourceShape: function(){
			//	
			//	Updates the event source shape size to the current viewport navigable area
			//
			if (this._eventBackground) {
				var s = this.getHudSize();
				s.width = s.width || 1; // bug in IE6 if width == 0
				s.height = s.height || 1; // bug in IE6 if height == 0
				this._eventBackground.setShape(s);
			}
		},
		
		eventClientLocation: function(e){
			//
			//	Returns the client location of a mouse event generated by the event source
			//
			var r = dojex.coords(this._surface._parent, true);
			if ((e.type !== "undefined") &&
			(e.type === "touchstart" ||
			e.type === "touchmove" ||
			e.type === "touchend" ||
			e.type === "gesturechange")) {
				return {
					x: e.touches[0].pageX - r.x,
					y: e.touches[0].pageY - r.y
				};
				
			}
			return {
				x: e.pageX - r.x,
				y: e.pageY - r.y
			};
		},
		
		
		eventContentLocation: function(e){
			//
			//	Returns the content location of a mouse event generated by the event source
			//
			return this.pointClientToContent(this.eventClientLocation(e));
		},
		
		//
		//	The GFX rendering surface
		//
		_surface: null,
		
		_computeInitialSize: function(){
			var box = domg.getContentBox(this._diagram.domNode);
			var parentBox = null;
			if (!box.w || !box.h) // only if needed 
				parentBox = this._diagram.getParent() ? domg.getContentBox(this._diagram.getParent().containerNode) : {};
			
			// Note the fallback on the height of box for the width and on width of box for the height. 
			// This is intentional. Comment from Abel:
			// It's really a very improbable fallback case; where if only one axis is specified or 
			// can be inferred; then we assume the surface to be a square. You can remove it if you
			// prefer a less confusing logic there. This fallback case has never been used in 
			// any test/sample
			return Size(box.w || parentBox.w || box.h || 256, box.h || parentBox.h || box.w || 256);
		},
		
		_createSurface: function(){
		
			var csz = this._computeInitialSize();
			
			if (!has("ie")) {
				this._surface = gfx.createSurface(this._diagram.canvasNode, csz.width + "px", csz.height + "px");
			} else {
				this._surface = gfx.createSurface(this._diagram.canvasNode, csz.width, csz.height);
			}
			
			this._surface.whenLoaded(this, function(){
			
				//_surface
				//	_hudLayer                 <- top-level HUD components
				//	_eventfulContent          <- content event source
				//		_overlayLayer           <- non-transformed overlay shapes (adorners, marquee, etc.)
				//		_transformedLayer       <- viewport transformation
				//			_transformedOverlay   <- non-content overlays (effects, controls, etc.)
				//			_content              <- content (Graph)
				//			_transformedUnderlay  <- non-content underlays (background, backlights)
				//	  _underlayLayer          <- non-transformed underlay shapes (background, grid, etc.)
				//	
				
				this._eventfulContent = this._surface.createGroup();
				
				// the viewport node is used for transforming the content to be shown into the viewport
				this._transformedLayer = this._eventfulContent.createGroup();
				
				// the overlay layer is for drawing adorners or other event-requiring stuff
				this._overlayLayer = this._eventfulContent.createGroup();
				this._overlayLayer.moveToFront();
				
				// the ui node is for drawing content that is not affected by the viewport transform
				this._hudLayer = this._surface.createGroup();
				this._hudLayer.moveToFront();
				
				// the content node is for user content that is affected by the viewport transformation
				this._content = this._transformedLayer.createGroup();
				
				this._transformedOverlay = this._transformedLayer.createGroup();
				this._transformedOverlay.moveToFront();
				
				this._transformedUnderlay = this._transformedLayer.createGroup();
				this._transformedUnderlay.moveToBack();
				
				this._underlayLayer = this._eventfulContent.createGroup();
				this._underlayLayer.moveToBack();
				
				this._createEventSourceShape();
				
				connect.connect(this._content, "_onChanged", this, "_onContentChangedNotification");
				
				this.setSize(csz, false);
			});
		},
		
		getSurface: function(){
			return this._surface;
		},
		
		getHudLayer: function(){
			return this._hudLayer;
		},
		
		getOverlayLayer: function(){
			return this._overlayLayer;
		},
		
		getUnderlayLayer: function(){
			return this._underlayLayer;
		},
		
		getContent: function(){
			return this._content;
		},
		
		_showOverlays: function(v){
			if (v && this._overlayLayer.getParent() == null) {
				this._eventfulContent.add(this._overlayLayer);
				this._overlayLayer.moveToFront();
			} else if (!v) {
				this._eventfulContent.remove(this._overlayLayer);
			}
		},
		
		_setViewportTransform: function( /*dojox.gfx.matrix*/zt){
			//
			//	returns:
			//		the transformation used for zooming
			//
			//	tags:
			//		private
			//
			this._transformedLayer.setTransform(zt);
		},
		
		getViewportTransform: function(){
			return this._viewportMatrix;
		},
		
		getViewportInverseTransform: function(){
			return this._viewportMatrixInverse;
		},
		
		destroy: function(){
			//
			//	summary:
			//		Gracefully destroys the receiver
			//
			if (this._refreshContentsTimer) {
				clearTimeout(this._refreshContentsTimer);
			}
			if (this._surface) {
				this._surface.destroy();
			}
		},
		
		
		// ========================================================================================================
		//
		//	HUD (Heads-up display)
		//
		_hudComponents: null,
		
		//
		//	_hudEnabled: 
		//		True if hud is enabled
		//
		_hudEnabled: true,
		
		//
		//	_hudZoomingOverlay: 
		//		True if hud zooming overlay effect is enabled
		//
		_hudZoomingOverlay: false,
		
		//
		//
		//	_hudTargetView:
		//		The visible Rect representing the target viewport
		//
		_hudTargetView: null,
		
		registerHudComponent: function(component){
			this._hudComponents.push(component);
		},
		
		setHudZoomFeedbackEnabled: function( /*Boolean*/b){
			this._hudZoomingOverlay = b;
		},
		
		isHudZoomFeedbackEnabled: function(){
			return this._hudZoomingOverlay;
		},
		
		setHudEnabled: function( /*Boolean*/b){
			if (b != this._hudEnabled) {
				if (b) {
					this._surface.add(this._hudLayer);
					this._hudLayer.moveToFront();
					this._hudEnabled = true;
					//this._activateHud();
				} else {
					//this._deactivateHud();
					this._hudEnabled = false;
					this._surface.remove(this._hudLayer);
				}
			}
			return this;
		},
		
		isHudEnabled: function(){
			return this._hudEnabled;
		},
		
		_activateHud: function(){
			//		dojo.forEach(this._hudComponents,function(c){
			//			c.setEnabled(true);
			//		});
		},
		
		_deactivateHud: function(){
			//		dojo.forEach(this._hudComponents,function(c){
			//			c.setEnabled(false);
			//		});
			//		if(this._hudTargetView) {
			//			this._hudLayer.remove(this._hudTargetView);
			//		}
		},
		
		_hudClearZoomingOverlay: function(){
			if (this._hudTargetView) {
				this._transformedOverlay.remove(this._hudTargetView);
			}
		},
		
		_hudCreateZoomingOverlay: function(){
			if (this._hudEnabled && this._hudZoomingOverlay && this._animOrigin.width !== this._animTarget.width) {
				var w = Math.min(this._animTarget.width, this._animTarget.height) * 0.10;
				var len = w * 0.50;
				var p = this._animTarget.scalePivot;
				var x1 = p.x - w;
				var x2 = p.x + w;
				var y1 = p.y - w;
				var y2 = p.y + w;
				
				this._hudTargetView = this._transformedOverlay.createGroup();
				var style = {
					color: "black",
					width: 1 / this._zoom
				};
				this._hudTargetView.createPolyline({
					points: [{
						x: x1,
						y: y1 + len
					}, {
						x: x1,
						y: y1
					}, {
						x: x1 + len,
						y: y1
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x2 - len,
						y: y1
					}, {
						x: x2,
						y: y1
					}, {
						x: x2,
						y: y1 + len
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x2,
						y: y2 - len
					}, {
						x: x2,
						y: y2
					}, {
						x: x2 - len,
						y: y2
					}]
				}).setStroke(style);
				this._hudTargetView.createPolyline({
					points: [{
						x: x1 + len,
						y: y2
					}, {
						x: x1,
						y: y2
					}, {
						x: x1,
						y: y2 - len
					}]
				}).setStroke(style);
			}
		},
		
		
		
		
		// ========================================================================================================
		//
		//	CONTENT RECTANGLE:
		//
		//	The area inside which the viewport can move, mostly based on the content bounding box.
		//
		//	The content rectangle (or navigable area) is computed from two values, the content 
		//	bounding box and the current client size, and it is obtained by the following method:
		//	A percentage of the current client size, defined by _airFactor is added to the
		//	borders of the content bounding box.
		//	
		//	The two main events that change the content rectangle are:
		//		- a change in the content
		//			(_onContentBoundingBoxChanged)
		//		- a change in the size of the viewport
		//			(setClientSize)
		
		//
		//	_contentBoundingBox:
		//		The area inside which we can move
		//
		_contentBoundingBox: EmptyRect,
		
		//
		//	_contentRect:
		//		The area inside which we can move
		//
		_contentRect: EmptyRect,
		
		//
		//	_airFactor:
		//		The minimum navigable space to the sides of the current content bounds as a percentage
		//		with respect to the current model bounding box.
		//
		_airFactor: 0.0,
		
		
		_getContentAir: function(){
			return g.sizeLongestSide(this._contentBoundingBox) * this._airFactor;
		},
		
		//
		//	_refreshContentsTimer:
		//		The timer scheduled by the refreshContentNotification request
		//
		_refreshContentsTimer: null,
		
		//
		//	_refreshContentsInterval:
		//		Refresh rate used for monitoring changes in the content.
		//
		_refreshContentsInterval: 50,
		
		setRefreshContentsSuspended: function( /*Boolean*/suspended){
			if (this._refreshContentsSuspended !== suspended) {
				this._refreshContentsSuspended = suspended;
				if (suspended) {
					clearTimeout(this._refreshContentsTimer);
					this._refreshContentsTimer = null;
				} else {
					this._refreshContents();
				}
			}
			
		},
		
		getRefreshContentsSuspended: function(){
			return this._refreshContentsSuspended;
		},
		
		setRefreshContentsInverval: function( /*Number*/msInterval){
			this._refreshContentsInterval = msInterval;
			if (msInterval < 0 && this._refreshContentsTimer) {
				clearTimeout(this._refreshContentsTimer);
				this._refreshContentsTimer = null;
			}
			return this;
		},
		
		getRefreshContentsInterval: function(){
			return this._refreshContentsInterval;
		},
		
		_onContentChangedNotification: function(){
			//
			//	summary:
			//		Sets up a timer to give the originating process time to finish with all the changes
			//
			if (!this._refreshContentsSuspended && this._refreshContentsInterval >= 0 && !this._refreshContentsTimer) {
				this._refreshContentsTimer = setTimeout(lang.hitch(this, "_refreshContents", this._refreshContents), this._refreshContentsInterval);
			}
		},
		
		refresh: function(){
			//
			// 	summary:
			//		Forces a refresh contents procedure, where the content rect is updated and consequently 
			//		associated scrollbars and other features. 
			//
			this._refreshContents();
		},
		
		_refreshContents: function(){
			//
			//	summary:
			//		Callback executed after the contentChangedNotification timer is finished.
			//
			if (!this._refreshContentsSuspended) {
				this._refreshContentsTimer = null;
				this._onContentBoundingBoxChanged();
			}
		},
		
		setAirFactor: function( /*Number*/factor){
			//
			//	summary:
			//		Sets the content air percentage with respect to the client size
			//
			this._airFactor = factor;
			
			if (this._computeContentRect()) {
				this._onViewRectConstraintsChanged();
			}
			return this;
		},
		
		
		getAirFactor: function(){
			//
			//	summary:
			//		Returns the content air percentage with respect to the client size
			//
			return this._airFactor;
		},
		
		
		getContentRect: function(){
			//
			//	summary:
			//		The current content size in pixels, which defines the available navigation area.
			//
			//	description:
			//		This value is updated by content BB change notifications (see _onContentBoundingBoxChanged)
			//
			//
			return this._contentRect;
		},
		
		
		_onContentBoundingBoxChanged: function(){
			//
			//	summary:
			//		The function is executed when the content of the content has changed.
			//
			//	returns:
			//		True if the change affected content bounding box.
			//
			//	tags:
			//		private
			//
			this._contentBoundingBox = this._content.getBoundingBox() || EmptyRect;
			
			if (this._computeContentRect()) {
				this._onViewRectConstraintsChanged()
			}
		},
		
		
		_computeContentRect: function(){
			//
			//	summary:
			//		Updates the navigable area
			//
			//	tags:
			//		private
			//
			var newContentRect = g.expandRect(this._contentBoundingBox, this._getContentAir());
			
			if (!g.equalRect(this._contentRect, newContentRect)) {
				this._contentRect = newContentRect;
				this.onContentRectChanged();
				return true;
			} else {
				return false;
			}
		},
		
		onContentRectChanged: function(){
			//
			//	summary:
			//		Event triggered when the content rect has changed.
			//
		},
		
		_constraintToContent: function( /*dojox.gfx.Rectangle*/r){
			//
			//	summary:
			//		Applies a constraining policy on view rectangle r, returning the result.
			//
			//	tags:
			//		private
			//
			var cx = this._contentRect.x;
			var cy = this._contentRect.y;
			var cw = this._contentRect.width;
			var ch = this._contentRect.height;
			var w = r.width;
			var h = r.height;
			var x = r.x;
			var y = r.y;
			
			// Correct aspect ratio of r enlarging shorter sides
			var csz = this.getClientSize();
			var vrat = g.getRectRatio(csz);
			var rrat = g.getRectRatio(r);
			if (rrat > vrat) {
				h = w / vrat;
			} else if (rrat < vrat) {
				w = h * vrat;
			}
			
			var pivotdelta;
			if (this._minZoom > 0 && csz.width / w < this._minZoom) {
				// TODO move this into a geometric pivot-scaling of rectangles
				pivotdelta = r.scalePivot ? g.mulPoint(g.subPoint(r, r.scalePivot), (csz.width / w) / this._minZoom) : {
					x: 0,
					y: 0
				};
				
				x = r.scalePivot.x + pivotdelta.x;
				y = r.scalePivot.y + pivotdelta.y;
				
				w = csz.width / this._minZoom;
				h = csz.height / this._minZoom;
			}
			
			if (this._maxZoom > 0 && csz.width / w > this._maxZoom) {
				// TODO move this into a geometric pivot-scaling of rectangles
				pivotdelta = r.scalePivot ? g.mulPoint(g.subPoint(r, r.scalePivot), (csz.width / w) / this._maxZoom) : {
					x: 0,
					y: 0
				};
				
				x = r.scalePivot.x + pivotdelta.x;
				y = r.scalePivot.y + pivotdelta.y;
				
				w = csz.width / this._maxZoom;
				h = csz.height / this._maxZoom;
			}
			
			return {
				x: x,
				y: y,
				width: w,
				height: h,
				scalePivot: r.scalePivot
			};
		},
		
		// ========================================================================================================
		//
		//	CLIENT SIZE:
		//
		//		The size of the UI space that allows the contents to be viewed.
		//
		//
		//	_clientSize:
		//		The size of current viewport
		//
		_clientSize: EmptyRect,
		
		//
		//	_resizeMode: Number
		//		Current resize mode of the vieport, one of:
		//		| 1: Keep zoom level, expand area
		//		| 2: Keep viewable area, change zoom level. Longest side defines zoom level.
		//
		_resizeMode: 1,
		
		setResizeMode: function( /*Number*/m){
			this._resizeMode = m;
		},
		
		getResizeMode: function(){
			return this._resizeMode;
		},
		
		getClientSize: function(){
			//
			//	summary:
			//		Returns the current viewport client size
			//
			return {
				width: this._clientSize.width,
				height: this._clientSize.height
			};
		},
		
		
		setClientSize: function( /*Size*/sz){
			//
			//	summary:
			//		Sets the Client Size
			//
			R.error("SubclassResponsibility", "setClientSize");
		},
		
		
		onClientSizeChanged: function(){
			//
			//	summary:
			//		The viewport has been resized (implies the related onSourceChanged event)
			//
			this.onLayoutHud();
			
			// Possible HUD information: this is a test
			//this._updateHudZoom();
			this._updateEventSourceShape();
		},
		
		getHudSize: function(){
			return this.getClientSize();
		},
		
		setSize: function( /*Size*/sz){
			//
			//	summary:
			//		Sets the Viewport Size
			//
			R.error("SubclassResponsibility", "setSize");
		},
		
		// ========================================================================================================
		//
		//	VIEW RECTANGLE:
		//
		//	The rectangle that represents which part of the model is currently seen from the viewport.
		//
		//	
		//
		//	_viewRect:
		//		The bounds of current content
		//
		_viewRect: EmptyRect,
		
		//
		//	_animate:
		//		Boolean flag to indicate that viewport movements are animated by default.
		//
		_animate: false,
		
		//
		//	_animLength:
		//		The length of the animation in milliseconds
		//
		_animLength: 200,
		
		//
		//	_animSteps:
		//		The number of updates to the viewport during the animation
		//
		_animSteps: 10,
		
		//
		//	_animation:
		//		The animation object (while animating).
		//
		_animation: null,
		
		//
		//	_animOrigin:
		//		The animation origin (while animating).
		//
		_animOrigin: null,
		
		//
		//	_animTarget:
		//		The animation target view rectangle (while animating).
		//
		_animTarget: null,
		
		getAnimLength: function(){
			return this._animLength;
		},
		
		setAnimLength: function( /*Number*/milliseconds){
			this._animLength = milliseconds;
			return this;
		},
		
		getAnimSteps: function(){
			return this._animSteps;
		},
		
		setAnimSteps: function( /*Number*/steps){
			this._animSteps = steps;
			return this;
		},
		
		setAnimated: function( /*Boolean*/animated){
			this._animated = animated;
			return this;
		},
		
		getAnimated: function(){
			return this._animated;
		},
		
		getViewRect: function(){
			//
			//	summary:
			//		The currently displayed area in diagram coordinates.
			//
			return this._viewRect;
		},
		
		_cloneRect: function(r){
			//	[AV] note: this is 7x-9x faster than the dojo.clone equivalent or than iterating attributes:
			//		var r, rect = {x:4,y:7,width:10,height:123};
			//		var start = new Date().getTime();
			//		for(var i = 0; i<10000; i++) { r = dojo.clone(rect); }
			//		var end = new Date().getTime();
			//		var start2 = new Date().getTime();
			//		for(var i = 0; i<10000; i++) { r = V._cloneRect(rect); }
			//		var end2 = new Date().getTime();
			//		console.log("dojo.clone",end-start,"_cloneRect",end2-start2);
			var cr = {};
			if (r.x !== undefined) 
				cr.x = r.x;
			if (r.y !== undefined) 
				cr.y = r.y;
			if (r.width !== undefined) 
				cr.width = r.width;
			if (r.height !== undefined) 
				cr.height = r.height;
			if (r.scalePivot !== undefined) 
				cr.scalePivot = r.scalePivot;
			return cr;
		},
		
		prepareViewRect: function( /*dojox.gfx.Rectangle*/r, /*dojox.gfx.Rectangle*/ viewRect /*=current*/){
			//
			//	summary:
			//		Pre-apply corrections to r that would otherwise be applied at the moment r is set as 
			//		the View Rectangle
			//
			//	description:
			//		Some corrections may be applied to r before assigning it 
			//		as the current View Rectangle:
			//
			//    	- if width xor height are set, set the other based on current viewport aspect ratio
			//
			//    	- for each axis:
			//    		- if origin is not defined and length is, make origin so that center is preserved
			//    		- if origin is defined but width is not, keep current length
			//
			//    	- if both width and height are set but the aspect ratio is not correct, correct short 
			//    	  side to match correct aspect ratio and maintain visibility of desired area.
			//
			//    	- if r goes beyond height or width, constrain location to make r within
			//
			//    	- if r lies before the origin, set location to origin
			//
			//    	- apply zoom constraints
			//
			viewRect = viewRect || this._viewRect;
			r = this._cloneRect(r);
			r.scalePivot = r.scalePivot || g.getRectCenter(r) || g.getRectCenter(viewRect);
			
			if (r.width === undefined && r.height !== undefined) {
				r.width = r.height * (g.getRectRatio(this.getClientSize()));
			}
			
			if (r.height === undefined && r.width !== undefined) {
				r.height = r.width / (g.getRectRatio(this.getClientSize()));
			}
			
			r = g.rectCenteredOverride(viewRect, r);
			r = this._constraintToContent(r);
			
			return r;
			
		},
		
		
		stopAnimation: function(){
			if (this._animation) {
				this._animation.stop();
				this._animation = null;
			}
		},
		
		_startViewRectAnimation: function( /*dojox.gfx.Rectangle*/viewRect){
			this._animOrigin = this._viewRect;
			this._animTarget = viewRect;
			this._hudCreateZoomingOverlay();
			var animation = new fx.Animation({
				duration: this._animLength,
				rate: this._animLength / this._animSteps,
				curve: [0, 100],
				onStop: lang.hitch(this, function(v){
					animation = null;
					this._animTarget = null;
					this._animOrigin = null;
					this._hudClearZoomingOverlay();
				}),
				onAnimate: lang.hitch(this, function(v){
					if (this._animation === animation) {
						// this is checked because animation.stop() is 
						// not working when fired from within the first 
						// step.
						var completion = v / 100;
						this._viewRect = g.standarizeRect(g.rectBlend(this._animOrigin, this._animTarget, completion));
						this._applyViewRect(true, true);
						this.onViewRectChanged();
						if (v == 100) {
							this.stopAnimation();
						}
					} else {
						// Try to stop the animation again
						animation.stop();
					}
				})
			});
			this._animation = animation;
			this._animation.play();
		},
		
		setViewRect: function( /*dojox.gfx.Rectangle*/r, /*Obj?*/ options){
			//
			//	summary:
			//		Sets the currently displayed area.
			//
			//	description:
			//		Some corrections are applied to r. See prepareViewRect.
			//
			// 	options:
			//		.animate
			//			animation of the viewport
			//		.instant
			//			cancel animated viewport
			//		.forceApply
			//			causes the view rect to be applied even if it has not changed
			//    .center
			//      causes the view rect to be centered
			//
			options = options || {};
			if (!r) {
				return;
			}
			var oldR = g.cloneRect(r);
			r = this.prepareViewRect(r);
			if (options.center) {
				if (oldR.width <= r.width) {
					r.x -= (r.width - oldR.width) / 2;
				}
				if (oldR.height <= r.height) {
					r.y -= (r.height - oldR.height) / 2;
				}
			}
			
			var sizeChanged = !g.rectEqualSize(this._viewRect, r, 1 / this._zoom);
			var originChanged = !g.rectEqualOrigin(this._viewRect, r, 1 / this._zoom);
			
			this.stopAnimation();
			
			if (sizeChanged || originChanged || options.forceApply) {
				if (!(this._animated || options.animate) || options.instant) {
					this._viewRect = g.standarizeRect(r);
					this._applyViewRect(sizeChanged, originChanged);
					this.onViewRectChanged();
				} else {
					this._startViewRectAnimation(r);
				}
			}
			
			return this._viewRect;
		},
		
		_applyViewRect: function( /*Boolean*/sizeChanged, /*Boolean*/ originChanged){
			this._computeZoom();
			
			//
			//	Subclasses must extend with the required actions.
			//
		},
		
		clientToContent: function(){
			//
			//	summary:
			//		returns a matrix that transforms a client coordinate into a content coordinate
			//
			return this._viewportMatrixInverse;
		},
		
		pointClientToContent: function( /*Point*/p){
			//
			//	summary:
			//		transforms a client coordinate into a content coordinate
			//
			return this._viewportMatrixInverse.transformPoint(p);
		},
		
		contentToClient: function(){
			//
			//	summary:
			//		returns a matrix that transforms a content coordinate into a client coordinate
			//
			return this._viewportMatrix;
		},
		
		pointContentToClient: function( /*Point*/p){
			//
			//	summary:
			//		transforms a content coordinate into a client coordinate
			//
			return this._viewportMatrix.transformPoint(p);
		},
		
		onLayoutHud: function(){
			//this._updateHudZoom();
			this._updateSurface();
			this._needsHudRelayout = false;
		},
		
		onViewRectChanged: function onViewRectChanged(){
			//
			//	summary:
			//		The view rectangle has been updated. 
			//
			// scrollbars may have reappeared/disappeared 
			if (this._needsHudRelayout) {
				this.onLayoutHud();
				this._needsHudRelayout = false;
			}
		},
		
		
		_onViewRectConstraintsChanged: function(){
			//
			//	summary:
			//		The view rectangle constraints have been updated.
			//
			this.setViewRect(this._viewRect, {
				forceApply: true,
				instant: true
			});
		},
		
		
		// ========================================================================================================
		//
		//	ZOOM:
		//
		//	Viewport zoom is defined from the relation between the viewport size and the view rectangle.
		//
		//		
		
		//
		//	_zoom:
		//		Current zoom level factor (larger is closer)
		//
		_zoom: 1,
		
		
		//
		//	_maxZoom:
		//		Defines how much the view rectangle can zoom in. Unlimited if 0.
		//	
		_maxZoom: 0,
		
		
		//
		//	_minZoom:
		//		Defines how much the view rectangle can zoom out. If 0, the limit is based on the contentRectangle.
		//	
		_minZoom: 0,
		
		
		setMaxZoom: function( /*Number*/z){
			//
			//	summary:
			//		Sets a maximum zoom and updates the view rectangle based on new constraints.
			//
			if (z < 0) { // normalize
				z = 0;
			}
			
			if (z > 0 && this._minZoom > 0 && z < this._minZoom) {
				z = this._maxZoom;
				R.warn("MaxZoomBelowMin");
			}
			this._maxZoom = z;
			this._onViewRectConstraintsChanged();
			return this;
		},
		
		
		getMinZoom: function(){
			//
			//	summary:
			//		Returns the current minimum zoom constraint.
			//
			return this._minZoom;
		},
		
		
		getMaxZoom: function(){
			//
			//	summary:
			//		Returns the current maximum zoom constraint.
			//
			return this._maxZoom;
		},
		
		
		setMinZoom: function( /*Number*/z){
			//
			//	summary:
			//		Sets a minimum zoom and updates the view rectangle based on new constraints.
			//
			if (z < 0) { // normalize
				z = 0;
			}
			
			if (z > 0 && this._maxZoom > 0 && z > this._maxZoom) {
				z = this._maxZoom;
				R.warn("MinZoomAboveMax");
			}
			this._minZoom = z;
			this._onViewRectConstraintsChanged();
		},
		
		
		setZoom: function( /*Number*/zoom, options){
			//
			//	summary:
			//		This method is a helper that actually controls setSourceRect expanding or shrinking
			//		the view rectangle size to achieve the zoom effect.
			//
			//	options:
			//		.animate
			//			animation of the viewport
			//		.instant
			//			cancel animated viewport
			//		.forceApply
			//			causes the view rect to be applied even if it has not changed
			//
			this.setViewRect({
				height: (this.getClientSize().height / zoom)
			}, options);
			return this;
		},
		
		
		getZoom: function(){
			//
			//	summary:
			//		Returns the current zoom level based on the current view rectangle and content size.
			//
			return this._zoom;
		},
		
		
		_computeZoom: function(){
			//
			//	summary:
			//		Computes and stores the current zoom factor by comparing viewport size and view rectangle size.
			//
			var newZoom = this._zoom;
			if (this._viewRect.width) {
				var vp = this.getClientSize();
				newZoom = Math.min(vp.width / this._viewRect.width, vp.height / this._viewRect.height);
			}
			this._zoom = newZoom;
		},
		
		
		
		
		
		// ========================================================================================================
		//
		//	MISCELANEOUS SERVICES:
		//
		//
		//	_focusAirFactor:
		//		The minimum space reserved for the view rect on the visualizeArea operation.
		//		Expressed in terms of a percentage of the size of the area.
		//
		_visualizeAirFactor: 1.50,
		
		
		visualizeArea: function( /*dojox.gfx.Rectangle*/r, /*{expandCurrent,airFactor}*/ options){
			//
			//	summary:
			//		Makes the r rectangle visible, givin it an extra air between the 
			//		area and the borders of the viewport if possible. The view may be 
			//		zoomed out if required, but it is never zoomed in.
			//
			//	r:
			//		The area to make visible.
			//
			//	airFactor:
			//		Percent of the area that is reserved as air. Overrides the default air 
			//		factor defined bu the member _focusAirFactor.
			//
			options = options || {};
			var vr = this._viewRect;
			var airFactor = (options.airFactor !== undefined) ? (1 + 2 * options.airFactor) : this._visualizeAirFactor;
			
			if (options.expandCurrent) {
				this.setViewRect(g.addRect(g.expandRect(r, Math.max(r.width, r.height) * (airFactor - 1) / 2), vr), options);
			} else {
				var minSize = {
					width: Math.max(vr.width, r.width * airFactor),
					height: Math.max(vr.height, r.height * airFactor)
				};
				var centered = g.rectCenteredAt(vr, g.getRectCenter(r));
				this.setViewRect(g.rectCenteredOverride(centered, minSize), options);
			}
		},
		
		centerAt: function( /*Point*/p, options){
			//
			//	summary:
			//		Centers the viewport at the requested point, without changing the zoom.
			//
			this.setViewRect(g.rectCenteredAt(this.getViewRect(), p), options);
		},
		
		zoomAt: function( /*Point*/p, /*Number*/ z, options){
			//
			//	summary:
			//		Scales the current zooms by a given factor z, using point p as pivot.
			//
			var vr = this.getViewRect(); // current viewport pos
			var rp = g.subPoint(p, vr); // relative cp
			var newViewRect = g.subPoint(p, g.mulPoint(rp, 1 / z));
			newViewRect.height = vr.height / z;
			newViewRect.scalePivot = p;
			
			this.setViewRect(newViewRect, options);
		},
		
		reset: function(){
			//
			//	TODO: reset the receiver to its initial state
			//
		},
		
		_dumpState: function(){
			console.log("zoom:" + g._round(this._zoom, 3) + " view:" + g.printRect(this._viewRect) + " cnt:" + g.printRect(this._contentRect));
		},
		
		toString: function(){
			return "[Viewport]";
		},
		
		getDiagram: function(){
			return this._diagram;
		},
		
		bringIntoView: function(/* ibm_ilog.diagram.Node */node, /* Number? */ margin){
			//
			//  summary:
			//     translate the view just enough so that a given element is fully visible
			//
			
			var vr = this.getViewRect();
			var bb = node.getBounds(this.getDiagram().getGraph());
			if (margin === undefined) 
				margin = 10;
			
			bb = g.cloneRect(bb);
			bb.x -= margin;
			bb.y -= margin;
			bb.width += 2 * margin;
			bb.height += 2 * margin;
			
			var newvr = {
				x: vr.x,
				y: vr.y
			};
			newvr.x = Math.max(newvr.x, bb.x + bb.width - vr.width);
			newvr.y = Math.max(newvr.y, bb.y + bb.height - vr.height);
			newvr.x = Math.min(newvr.x, bb.x);
			newvr.y = Math.min(newvr.y, bb.y);
			
			this.setViewRect(newvr);
		}
		
		
	});
	
	return Viewport;
	
});

},
'ibm_ilog/diagram/overview/OverviewRenderer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewRenderer", [
"dojo/_base/declare",
"../util/ErrorReporter"
],
function(
declare,
R
){

var OverviewRenderer =
declare('ibm_ilog.diagram.overview.OverviewRenderer',null, {
	
	//
	//	summary:
	//		The abstract superclass of overview renderers. This class may only exist
	//		for documentation purposes only.
	//

	render: function( /*Object*/content, /*dojox.gfx.Group*/ output ) {
		R.error("SubclassResponsibility","render");
	}
		
});

return OverviewRenderer;

});

},
'ibm_ilog/diagram/util/MultilineText':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/MultilineText", ["../base", "dojo/_base/lang", "dojo/_base/config", "dojox/gfx/_base", "dojox/gfx/matrix", "dojox/gfx/shape"],
function(iid, lang, config, gfx, matrix, shape){
	
/*=====
var gfx = dojox.gfx;
=====*/

var MultilineText = iid.gfxDeclare("ibm_ilog.diagram.util.MultilineText", [ gfx.Group ], {
	// summary:
	//		A multi-line text shape for GFX.
	// description:
	//		The MultilineText shape is similar to a Text shape, except that it can display multiple lines of text.
	//		<p>
	//		The shape object of the MultilineText accepts the same attributes as a plain Text shape
	//		("text", "x", "y", "align", etc). The "type" property must be "mltext".
	//		</p>
	//		<p>
	//		The MultilineText shape also accepts an additional "separator" attribute, that
	//		defines the line separator. The default separator is "\n". You can specify a simple string, or
	//		a RegExp as separator (for example, "[ \t]+"). When the text contains one or more
	//		occurrences of the line separator, the text is broken into several lines. The first line is placed
	//		normally according to the x and y properties of the shape, and subsequent lines are stacked vertically
	//		below the first line, that is, the baseline of the second line is at (x, y+h), where h is the font height.
	//		You can also specify an additional spacing using the "spacing" property of the shape (0 by default).
	//		</p>
	//		<p>
	//		The MultilineText supports simple text wrapping. To enable text wrapping, set the "wrap" property of the shape to true.
	//		When text wrapping is enabled, the text will be automatically broken into lines if necessary to fit the width of the text.
	//		This will work only if the width of the shape is constrained in one of the following ways:
	//		<ul>
	//		<li>You can specify the "width" property of the shape to a fixed width.</li>
	//		<li>The shape can be contained in a Group that has a GFX layout, and that constrains the width of the shape.
	//		For example, if the MultilineText shape is contained in a Group with a GridLayout with a fixed column width,
	//		then the text will be wrapped to fit the column width. (Note: the width determined by the GFX layout has precedence
	//		over the "width" shape property.)</li>
	//		</ul>
	//		In addition to wrapping text, you can also wrap single words if they do not fit the text width. To do this,
	//		set the "wordWrap" property of the shape to true (word-wrapping is false by default).
	//		</p>
	//		<p>
	//		<strong>Note:</strong> The built-in text wrapping method is a simplistic convenience placeholder that only breaks lines at space characters.
	//		To wrap text correctly according to globalization rules, you will need to provide a more complete implementation.
	//		To do this, you should override the <code>split</code> and <code>join</code> methods (to use different word breaking rules)
	//		or the <code>wrap</code> method (to completely replace the text wrapping algorithm), through a <code>dojo.extend</code> call.
	//		</p>
	//		<p>
	//		The MultilineText shape is actually a GFX Group shape containing Text shapes that display the lines.
	//		Each Text line has the same attributes as the MultilineText shape: align, font, fill, etc. For example,
	//		if the MultilineText has align: 'left', then all the lines will be aligned on the left. Similarly,
	//		calling setFont() on the MultilineText shape sets the font of all the lines.
	//		</p>
	
	_isIBMDiagramMultilineText: true,
	
	_font: null,
	
	_lastWrapWidth: -1,
	
	textDir: "",
	
	setTextDir: function(/*String*/textdir) {
	   // summary:
	   //       set text direction property, need for runtime change of text direction  
	   
	   this.textDir = textdir; 
	},	
	
	constructor: function(){
		// summary:
		//		Creates an empty MultilineText instance.
		
		this.shape = lang.clone(iid.util.MultilineText.defaultMultilineText);
	},
	
	setShape: function(shape){
		// summary:
		//		Sets the attributes of the MultilineText shape.
		// description:
		//		The <code>shape</code> parameter is an object with the following properties:
		//		<table style="border-collapse:collapse;" border="1">
		//			<tr>
		//				<th><b>Property</b></th>	<th><b>Description</b></th>
		//			</tr>
		//			<tr>
		//				<td>type</td>				<td>must be <code>'mltext'</code></td>
		//			</tr>
		//			<tr>
		//				<td>text</td>				<td>The text string to display.</td>
		//			</tr>
		//			<tr>
		//				<td>x</td>					<td>Defines the horizontal position of the text.</td>
		//			</tr>
		//			<tr>
		//				<td>y</td>					<td>Defines the vertical position of the baseline of the first line of text.</td>
		//			</tr>
		//			<tr>
		//				<td>align</td>				<td>The horizontal alignment (see dojox.gfx.Text).</td>
		//			</tr>
		//			<tr>
		//				<td>decoration</td>			<td>(see dojox.gfx.Text)</td>
		//			</tr>
		//			<tr>
		//				<td>rotated</td>			<td>(see dojox.gfx.Text) Not supported for multi-line text.</td>
		//			</tr>
		//			<tr>
		//				<td>kerning</td>			<td>(see dojox.gfx.Text)</td>
		//			</tr>
		//			<tr>
		//				<td>separator</td>			<td>The line separator character or regular expression.</td>
		//			</tr>
		//			<tr>
		//				<td>spacing</td>			<td>The vertical line spacing.</td>
		//			</tr>
		//			<tr>
		//				<td>wrap</td>				<td>If <code>true</code>, enables text wrapping. The value can also be a custom text wrapping function.</td>
		//			</tr>
		//			<tr>
		//				<td>wordWrap</td>			<td>If <code>true</code>, enables word-wrapping, that is, breaking single-word lines that do not fit the specified width.</td>
		//			</tr>
		//			<tr>
		//				<td>width</td>				<td>Specifies a default width for text wrapping. Can be overridden by a GFX layout.</td>
		//			</tr>
		//		</table>
		// shape: Object
		//		A hash that contains the properties of the shape.
		
		if(this._noSetShape) return this; // see comment in _createLine before this.createText...
		if(iid._inContainerAdd) return this;
		
		var sh = this.shape = lang.mixin(lang.clone(iid.util.MultilineText.defaultMultilineText), shape);
		
		this._lastWrapWidth = -1;
		
		if(sh.wrap && sh.width > 0)
			this._doWrap(sh.width);
		else {
			var sep = sh.separator;
			if(sep.length > 1)
				sep = new RegExp(sep, "g");

			var lines = sh.text.split(sep);

			this._createLines(lines);
		}
		// propagate changes and invalidate
		iid._base.gfxext.onChangePatch.call(this, 'setShape');
		return this;
	},
	
	_createLines: function(lines)
	{
		// summary:
		//		Creates the Text shapes displaying the lines of text.
		
		if(this.children && this.children.length > 0)
			this.clear();
		
		var sh = this.shape;
		
		var y = sh.y;
		for(var i = 0; i < lines.length; i++){
			var textObj = this._createLine(lines[i], y);
			if(textObj){
				var bb = textObj.getBoundingBox();
				if(bb)
					y += bb.height + sh.spacing;
			}
		}
	},
	
	_createLine: function(text, y)
	{
		var sh = this.shape;
		
		var textShape = lang.clone(gfx.defaultText);
		for(var p in textShape)
			if(p != "type" && sh[p])
				textShape[p] = sh[p];
		textShape.y = y;
		textShape.text = text;
		
		if (config.isBidiInDojoDiagrammer && sh.textDir)                            
			textShape.textDir = sh.textDir;	
		
		// This is a workaround for a very nasty bug with Dojo 1.6:
		// when adding a text child, Dojo now resets the shape and text of all other text children,
		// but it does so using a dojox.gfx.utils.forEach, which iterates also on the multiline shape itself
		// (because the forEach just tests if the shape has a getFont method, which the parent multiline shape
		// happens to have too...).
		// So to avoid this we disable setShape by a flag.
		this._noSetShape = true;
		var textObj = this.createText(textShape);
		delete this._noSetShape;
		if(textObj){
			// copy font and fill:
			var s;
			s = this.getFont(); if(s) textObj.setFont(s);
			s = this.getFill(); if(s) textObj.setFill(s);
		}	
		return textObj;
	},
	
	setFont: function(font){
		// summary: 
		//		Sets the font for text.
		// font: Object
		//		A font object (see dojox.gfx.defaultFont) or a font string
		
		if(iid._inContainerAdd) return this;
		
		this._font = font;
		
		var y = this.shape.y;
		for(var i = 0; i < this.children.length; i++){
			this.children[i].setFont(font);
		}
		// propagate changes and invalidate
		iid._base.gfxext.onChangePatch.call(this, 'setShape');
		return this;
	},
	
	getFont: function(){
		// summary:
		//		Returns the current font object or null.
		
		return this._font;
	},
	
	setFill: function(fill){
		// summary: 
		//		Sets the color for text.
		// fill: Object
		//		A fill object (see dojox.gfx.defaultFill) or a color string.
		
		this.inherited(arguments);
		for(var i = 0; i < this.children.length; i++)
			this.children[i].setFill(fill);
	},
	
	_computeBoundingBox: function(){
		// summary: Better fix than the old getBoundingBox override, since the previous fix
		//		did not store the bbox in the cache, so direct calls to _computeBoundingBox (from gfxlayout/_base)
		//		still returned null...
		gfx.Group.prototype._computeBoundingBox.call(this);
		if(!this._bbCache){
			this._bbCache = { x:0, y:0, width:1, height:1 };
			var lbb = { x:0, y:0, width:1, height:1 };
			var t = this.getTransform();
			this._lbbCache = t ? t.transformRectangle(lbb) : lbb;
		}
	},
	
	_computePreferredSize: function(/*Size*/proposedSize) {
		// summary:
		//     Computes the preferred size of this shape. 
		// description:
		//     This method wraps the text if required.
		
		var width = proposedSize.width;
		if(width <= 0)
			width = this.shape.width;

    	if(this.shape.wrap && width > 0)
    		this._doWrap(width);
		
    	var sz = this.inherited(arguments);
    	
    	return sz;
	},
	
    applyLayout: function(/*Rectangle*/rect) {
    	// summary:
    	//		Layout this text in the given rectangle.
    	
    	if (this.shape.wrap) {
			var w = rect.width;
			var t = this.getTransform();
			if(t && !t.isTranslation())
				w = t.inverse().transformRectangle(rect).width;
			this._doWrap(w);
		}
    	
    	var x = rect.x;
    	var y = rect.y;
        var oldBounds = this.getLocalBoundingBox();
        if (oldBounds){
        	// horizontal alignment:
        	switch(this.shape.align){
        	case 'middle':
        		x += (rect.width - oldBounds.width)/2;
        		break;
        	case 'end':
        		x += (rect.width - oldBounds.width);
        		break;
        	}
        	// vertical alignment:
        	switch(this.valign || 'stretch'){
        	case 'center':
        	case 'stretch':
        		y += (rect.height - oldBounds.height)/2;
        		break;
        	case 'bottom':
        		y += rect.height - oldBounds.height;
        		break;
        	}
        	
            this.applyLeftTransform(matrix.translate(x-oldBounds.x, y-oldBounds.y));
        } else {
            this.setTransform(matrix.translate(x, y));
        }
    },
    
    _doWrap: function(width)
    {
    	if(width == this._lastWrapWidth)
    		return;
   		this._lastWrapWidth = width; // may be reset to -1 below if we see that bboxes can't be computed
    	
    	if(this.shape.text == ""){
    		this._createLines([""]);
    		return;
    	}
    	
		var wrap = this.shape.wrap;
		
		if(wrap){
			try {
				if(this.suspendInvalidate) this.suspendInvalidate();
				this._noOnChanged=true;
				var lines = this.wrap(this.shape.text, width);
				this._createLines(lines);
			} catch(e){
			} finally {
				if(this.resumeInvalidate) this.resumeInvalidate(false);
				delete this._noOnChanged;
				this._bbCacheValid = false;
			}
		}
    },
    
    wrap: function(text, width)
    {
		// summary:
		//		Breaks the text into lines to fit the specified width.
		// description:
		//		This method implements a simplistic text wrapping algorithm that breaks 
		//		the specified text into lines to fit the specified width.
		//		This simple implementation breaks the text into words by calling the
		//		<code>split</code> and <code>join</code> methods.
		//		When using Dojo Diagrammer in a globalized application, you should override
		//		the default algorithm (through a <code>dojo.extend</code> call) and provide a more complete implementation.
		//		You can just override the <code>split</code> and <code>join</code> methods if this fits your needs, 
		//		or you can completely override the <code>wrap</code> method if necessary.
		// text: string
		//		The text to break into lines fitting the specified width.
		// width: Number
		//		The width in which the wrapped text should fit.
		// returns:
		//		An array of strings representing the wrapped lines of text.
		
        var lines = [];
        var sh = this.shape;
        
        // Keep "hard" line breaks: split the text into paragraphs first,
        // then wrap each paragraph separately.
		var sep = sh.separator;
		if(sep.length > 1)
			sep = new RegExp(sep, "g");
		var paragraphs = text.split(sep);
		
		var tmpText = null;
		
		for(var pi = 0; pi < paragraphs.length; pi++){
			// split into words
			var words = this.split(paragraphs[pi]);
			if (words.length > 0) {
				var line = null;
				// join words into lines until max width is exceeded
				var i0 = 0;
				tmpText = this._createLine("", 0);
				var bb;
				for (var i = 0; i < words.length; i++) {
					var word = words[i];
					if (!line) {
						line = word;
						// Do word-wrapping?
						if (sh.wordWrap) {
							tmpText.setShape({
								text: line
							});
							bb = tmpText.getBoundingBox();
							if(!bb || (bb.width == 0 && bb.height == 0)) this._lastWrapWidth = -1;
							if (bb && bb.width > width) {
								var cutIndex = this.next(word, 0);
								if (cutIndex > 0) {
									for (var j = this.next(word, cutIndex); j >= 0; j = this.next(word, j)) {
										var w = word.substring(0, j);
										tmpText.setShape({
											text: w
										});
										bb = tmpText.getBoundingBox();
										if(!bb || (bb.width == 0 && bb.height == 0)) this._lastWrapWidth = -1;
										if (bb && bb.width > width) 
											break;
										cutIndex = j;
									}
									lines.push(word.substring(0, cutIndex));
									words[i] = word.substring(cutIndex);
									line = null;
									i--;
								}
							}
						}
					} else {
						var newline = this.join(words.slice(i0, i + 1));
						tmpText.setShape({
							text: newline
						});
						bb = tmpText.getBoundingBox();
						if(!bb || (bb.width == 0 && bb.height == 0)) this._lastWrapWidth = -1;
						if (bb && bb.width > width) {
							lines.push(line);
							i0 = i;
							line = null;
							i--;
						}
						else {
							line = newline;
						}
					}
				}
				if (line) 
					lines.push(line);
			}
		}
        
		if(tmpText) this.remove(tmpText);
		
        return lines; // Array
    },
    
    split: function(text)
    {
		// summary:
		//		Splits the text using a space character delimiter.
		// description:
		//		The default method used to break the text into words for text wrapping.
		//		This default implementation splits the text using a space character delimiter.
		//		When using Dojo Diagrammer in a globalized application, you should override
		//		the default algorithm (through a <code>dojo.extend</code> call) and provide a more complete implementation.
		//		You can just override the <code>split</code> and <code>join</code> methods if this fits your needs, 
		//		or you can completely override the <code>wrap</code> method if necessary.
		// text: string
		//		The text to split into words.
		// returns:
		//		An array of strings representing the possible line break positions.
		
    	return text.split(" "); // Array
    },
    
    join: function(words)
    {
		// summary:
		//		Joins the words using a space character delimiter.
		// description:
		//		The default method used to join word into lines for text wrapping.
		//		This default implementation joins the words using a space character delimiter.
		//		When using Dojo Diagrammer in a globalized application, you should override
		//		the default algorithm (through a <code>dojo.extend</code> call) and provide a more complete implementation.
		//		You can just override the <code>split</code> and <code>join</code> methods if this fits your needs, 
		//		or you can completely override the <code>wrap</code> method if necessary.
		// words: Array
		//		The array of words to join into a line string.
		// returns:
		//		The specified words joined into a string.
		
    	return words.join(" "); // string
    },
	
	next: function(word, index)
	{
		// summary:
		//		Returns the next character in the word after index where word-wrapping is allowed.
		// description:
		//		This method is called during word-wrapping, to get the next character position where word-wrapping would be allowed.
		//		The default implementation just increments the index. You can override this method to forbid word breaks on particular
		//		characters, or for globalization.
		// word: string
		//		The word that is being wrapped.
		// index: int
		//		The start index from which the next character if required.
		// returns:
		//		The index of the next character where a word break may occur, or -1 if the end of the string was reached.
		
		index++;
		return index < word.length ? index : -1; // int
	}
});

iid.util.MultilineText._nextId = 1;
iid.util.MultilineText.defaultMultilineText = lang.mixin(lang.clone(gfx.defaultText),
										{ type: 'mltext', separator: '\n', spacing: 0, wrap: false, wordWrap: false, width: -1 });
iid.util.MultilineText.nodeType = gfx.Group.nodeType;

//
// Extends createShape to support our own shape types
//

function extendCreateShape(shapeType) {
    var method = "createShape";
    var old = shapeType.prototype[method];
    shapeType.prototype[method] = function() {
        var ret = old.apply(this, arguments);
        if (!ret) {
            var sh = arguments[0];
            switch (sh.type) {
                case 'mltext':
                    ret = this.createObject(iid.util.MultilineText, sh);
                    if (config.isBidiInDojoDiagrammer) {
                        var sp = this;
    				    while (sp) {
    					    if (sp.textDir) {
    						    ret.textDir = sp.textDir;
    						    break;
    					    }
    					    else if(sp.getTextDir) {
    						    ret.textDir = sp.getTextDir();
    						    break;
    					    }
    					    if (!sp.parent)
    						    break;
    					    sp = sp.getParent();
    				    }
                    }                    
                    break;
            }
        }
        return ret;
    };
}
extendCreateShape(gfx.Surface);
extendCreateShape(gfx.Group);

return MultilineText;

});

},
'ibm_ilog/graphlayout/internalutil/FixedPoint':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// #######################################
// ######## NOT Generated by J2JS ########
// ###### DO NOT DELETE THIS FILE! #######
// #######################################

// We do not generate this class automatically since there is no Point class in Dojo
// that we could use as a base class for FixedPoint. Points are just objects with 
// x and y properties.

// Helper comment for the Dojo apidoc build tool:

/*=====
var Point = ibm_ilog.graphlayout.internalutil.Point;
=====*/

define("ibm_ilog/graphlayout/internalutil/FixedPoint", ["dojo/_base/declare", "./Point"],
	function(declare, Point) {
return declare('ibm_ilog.graphlayout.internalutil.FixedPoint', Point, 
{
  // summary:
  //		This is an internal subclass of Point that is used to represent fixed
  //		connection points of links. 
});
});

},
'ibm_ilog/diagram/interactors/DragInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/DragInteractor", [
"../main",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/event",
"dojox/gfx/matrix",
"./Interactor",
"../util/Geometry"
],
function(
iid,
declare,
lang,
event,
m,
Interactor,
g
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var DragInteractor =
declare("ibm_ilog.diagram.interactors.DragInteractor", [Interactor], {

	//
	//	_viewport: ibm_ilog.diagram.widget.Diagram
	//		The viewport where the drag occurs
	//
	_viewport: null,

	//
	//	hasDragged: Boolean
	//		true if last drag start was followed by a move
	//
	hasDragged: false,

	//
	//	_transform: [dojox.gfx.matrix.Matrix2D]
	//		Transform used to transform UI coordinate system into moveable object's system
	//
	_transform: null,

	//
	//	_last: 
	//		Last event coordinate
	//		
	_last: null, 

	//
	//	_first: 
	//		The event coordinate at which the drag started
	//		
	_first: null,

	//
	//	_firstViewportT:
	//		The viewport transform at the point where _first was defined or updated.
	//
	_firstViewportT: null,
		
	//
	//	_current: 
	//		Current event coordinate
	//		
	_current: null,
	
	//
	//	_delta: 
	//		The difference between _current and _last
	//		
	_delta: null,
	
	//
	//	_dragCursor: 
	//		Optional cursor ID set while dragging
	//		
	_dragCursor: null,
	
	//
	//	_cursorDom: 
	//		The DOM style that controls the cursor we will update
	//		
	_cursorStyle: null,

	
	_declareStates: function () {
		var idleStates = ["start"];
		if (iid.mobileEnabled)
			idleStates.push("touchStart");
		this._declareState("idle", idleStates);
		this._declareState("active", ["first", "move", "end", "cancelOnDragStart", "cancelOnSelectStart"]);
		if (iid.mobileEnabled)
			this._declareState("activeTouch", ["touchFirst","touchMove","touchEnd"]);
	},

	_initialize: function () {
		if (!this._viewport) {
			this._getEventCoord = this.__getEventCoordNoViewport;
			this._cursorStyle = this._cursorStyle || document.body.style;
		} else {
			this._cursorStyle = this._cursorStyle || this._viewport.getDiagram().domNode.style;
		}
		this._last = g.Point(0, 0);
		this._first = this._last;
		this._current = this._last;
		this.inherited(arguments);

	},

	initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
		if (diagram._isIBMDiagramDiagram) {
			this._viewport = diagram.getViewport();
		} else if (diagram._isIBMDiagramViewport) {
			this._viewport = diagram;
		}
		this._initialize();
		return this;
	},

	_getInitialEventSource: function () {
		return this._viewport.getEventSource();
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		var defaultConnections = {
			start: {
				src: this._getInitialEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_dragPreStart",
				filter: this._buildInputFilter({button:0,ctrl:false,shift:false}),
				gfxConnect: true
			}, first: {
				src: document,
				srcEvt: "onmousemove",
				connectTo: "_dragFirstMove"
			}, move: {
				src: document,
				srcEvt: "onmousemove",
				connectTo: "_dragMove"
			}, end: {
				src: document,
				srcEvt: "onmouseup",
				connectTo: "_dragEnd"
			}, cancelOnDragStart: {
				src: document,
				srcEvt: "ondragstart",
				target: dojo,
				connectTo: "stopEvent"
			}, cancelOnSelectStart: {
				src: document,
				srcEvt: "onselectstart",
				target: dojo,
				connectTo: "stopEvent"
			}
		};
		if (iid.mobileEnabled){
			var mobileDefaultConnections = {
				touchStart : {
					src: this._getInitialEventSource(),
					srcEvt: "ontouchstart",
					connectTo: "_dragTouchPreStart",
					gfxConnect: true
				}, touchFirst: {
					src: document,
					srcEvt: "ontouchmove",
					connectTo: "_dragTouchFirstMove"
				},  touchMove : {
					src: document,
					srcEvt: "ontouchmove",
					connectTo: "_dragTouchMove"
				}, touchEnd : {
					src: document,
					srcEvt: "ontouchend",
					connectTo: "_dragTouchEnd"
				}
			};
			lang.mixin(defaultConnections,mobileDefaultConnections);
		};
		return defaultConnections;
	},

	activate: function () {
		//
		//	summary:
		//		Activates the receiver by connecting it to its sources of events.
		//
		this._goStateId("idle");
		this.inherited(arguments);
	},

	_dragPreStart: function (e) {
		//
		//	summary:
		//		Starts the drag process by storing initial coordinates for pointer and view rectangle
		//
		if (this._state._id !== "idle") console.warn("Unexpected active dragging.");

		if (dojox.gfx.renderer !== "silverlight")
			event.fix(e);
		
		this._goStateId("active");
		this._startDragGesture(e);
		this._updateCursor();
	},

	_startDragGesture: function(e){
		this._firstViewportT = this._viewport ? this._viewport.getViewportInverseTransform() : null;
		this._last = this._getEventCoord(e);
		this._first = this._last;
		this._current = this._last;
		this._delta = {
			x: 0,
			y: 0
		};
		this._totalDelta = this._delta;
		
	},

	_showDragCursor: function() {
		if(this._dragCursor && this._cursorStyle && this._cursorBackup == null) {
			this._cursorBackup = this._cursorStyle.cursor;
			this._cursorStyle.cursor = this._dragCursor;
		}
	},

	_hideDragCursor: function() {
		if(this._cursorBackup!==null) {
			if (this._cursorStyle)
				this._cursorStyle.cursor = this._cursorBackup;
			this._cursorBackup = null;
		}
	},
	
	_updateCursor:function() {
		if(this._state._id=="active") {
			this._showDragCursor();
		} else {
			this._hideDragCursor();
		}
	},
	
	_dragFirstMove: function (e) {
		if (this._manager) {
			this._activationHandle = this._manager.push(this);
		}
		this._disconnect("first");
		this.hasDragged = true;
	},

	_dragMove: function (e) {
		if (dojox.gfx.renderer !== "silverlight")
			event.fix(e);
		this._dragGesture(e);
	},

	_dragGesture: function (e){
		this._last = this._current;
		this._current = this._getEventCoord(e);
		this._delta = {
			x: this._current.x - this._last.x,
			y: this._current.y - this._last.y
		};
		this._totalDelta = {
			x: this._current.x - this._first.x,
			y: this._current.y - this._first.y
		};
		
	},
	_dragEnd: function (e) {
		if (dojox.gfx.renderer !== "silverlight")
			event.fix(e);

		if (this._activationHandle) {
			this._manager.pop(this._activationHandle);
			this._activationHandle = null;
		}
		this._goState(null);

		this.hasDragged = false;
		this._goStateId("idle");
		
		this._updateCursor();
	},

	_getEventCoord: function (e) {
		var pl = this._viewport.eventClientLocation(e);
		if (this._transform) {
			pl = m.multiplyPoint(this._transform, pl);
		}
		return pl;
	},

	__getEventCoordNoViewport: function (e) {
		var pl = this.eventPageLocation(e);
		if (this._transform) {
			pl = m.multiplyPoint(this._transform, pl);
		}
		return pl;
	},
	_dragTouchPreStart: function (e) {
		if (dojox.gfx.renderer !== "silverlight")
			event.fix(e);
		e.preventDefault();		
		//
		//	summary:
		//		Starts the drag process by storing initial coordinates for pointer and view rectangle
		//
		if (this._state._id !== "idle") console.warn("Unexpected active dragging.");

		
		this._goStateId("activeTouch");
		this._startDragGesture(e);
		this._updateCursor();
	},	
	
	_dragTouchFirstMove: function (e) {
		if (dojox.gfx.renderer !== "silverlight")
			event.fix(e);
		e.preventDefault();		
		
		this._disconnect("touchFirst");
		this.hasDragged = true;
	},

	_dragTouchMove: function (e) {
		if (dojox.gfx.renderer !== "silverlight")
			event.fix(e);
		e.preventDefault();		
		// if more than one finger don't do anything
		if (e.touches.length > 1){
			return false;
		}
		
		this._dragGesture(e);
		
	},	
	_dragTouchEnd: function (e) {
		if (dojox.gfx.renderer !== "silverlight")
			event.fix(e);
		if (e.touches.length > 0){
			// one finger left, restart the gesture
			this._startDragGesture(e);
			return false;
		}
		if (this._activationHandle) {
			this._manager.pop(this._activationHandle);
			this._activationHandle = null;
		}
		this._goState(null);

		this.hasDragged = false;
		this._goStateId("idle");
		
		this._updateCursor();

		
	}	


});

return DragInteractor;

});
},
'ibm_ilog/diagram/Subgraph':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/Subgraph", [
"./main",
"dojo/_base/config", "dojo/_base/array", "dojo/_base/lang", "dojo/_base/declare",
"dojo/_base/sniff", "dojo/_base/connect", "dojo/fx",
"dojox/gfx", "dojox/gfx/fx", "dojox/gfx/utils", "dojox/gfx/matrix",
"./GraphElement", "./Node", "./templating", "./templates/templates",
"./gfxlayout",
"dojo/has!config-useGfxLayout?./gfxlayout/CanvasLayout:",
"./util/Geometry", "./util/ErrorReporter"
],
function(iid,
config, array, lang, declare,
has, connect, fx,
gfx, gfxfx, gfxutils, matrix,
GraphElement, Node, templating, templates,
gfxlayout,
CanvasLayout,
g, ErrorReporter){

/*=====
Node = ibm_ilog.diagram.Node;
=====*/

var Subgraph =
declare("ibm_ilog.diagram.Subgraph", [Node], {
    // summary:
    //		A Subgraph is a Node that contains a nested Graph.
    // description:
    //		Subgraph objects are used to implemented nested graphs, that is, graphs contained in graphs.
    //		A Subgraph can be collapsed (i.e., its nested graph is hidden and replaced by a plain node)
    //		or expanded (i.e., its nested graph is fully displayed).
    
    // Allows to test the class faster that testing declaredClass
    _isIBMDiagramSubgraph: true,

  	//
  	//	TID: String
  	//		Graph element Type ID, used to distinguish basic graph 
    //		element types:
    //	
    //			N = Node
    //			S = Subgraph
    //			L = Link
    //	
  	//		Usually used as a very fast index into maps based on 
    //		ibm_diagram graph element types.
  	//
    TID: 'S',

    // "attach points" (aka template parts),
    // initialized automatically when reading the template.
    _graph: null,
    _collapsedGroup: null,
    _expandedGroup: null,
    _expandButton: null,
    _collapseButton: null,
    
    _collapsed: false,
    
    _fixedPosition: null,
    _animate: true,
    
    allowsResize: function() {
    	return true;
    },
    
    dispose: function(options) {
    	//	summary:
			//		Dispose this GraphElement.
			//	description:
			//		This function should be called when the GraphElement is completely deleted, that is, when it is removed
			//		from its parent graph and it will never be used again.
			//	options:
    	//		options.noRecurse:
			//			Do not recurse children
    	if(!options || !options.noRecurse) {
	    	array.forEach(this._graph.children, function(c){
	    		c.dispose(options);
	    	});
    	}
    	GraphElement.prototype.dispose.apply(this,arguments);
    },

    _subgraphInitialize: function() {
        // summary:
        //		Performs post-creation initializations such as connecting events, etc.
        // tags:
        //		private
        
        this._fixedPosition = {
            x: 0,
            y: 0
        };
        
        this.baseShape = this._expandedBaseShape;
        
        if (this._graph) {
            this._graph._owningSubgraph = this;
            if (config.useGfxLayout)
                this._graph.setLayout(new CanvasLayout());
        }
        this._hide(this._collapsedGroup);
        
		try { // throws errors in Silverlight for example
			if (this._expandedBaseShape) {
				this._expandedBaseShape.connect("ondblclick", this, this._collapseButtonClicked);
				if (iid.mobileEnabled) 
					this._expandedBaseShape.connect("ontouchend", this, this._collapseButtonDoubleTap);
			}
			if (this.titleShape) {
				this.titleShape.connect("ondblclick", this, this._collapseButtonClicked);
				if (iid.mobileEnabled) 
					this.titleShape.connect("ontouchend", this, this._collapseButtonDoubleTap);
			}
			
			if (this._collapsedBaseShape) {
				this._collapsedBaseShape.connect("ondblclick", this, this._expandButtonClicked);
				if (iid.mobileEnabled) 
					this._collapsedBaseShape.connect("ontouchend", this, this._expandButtonDoubleTap);
			}
			if (this.textShape) {
				this.textShape.connect("ondblclick", this, this._expandButtonClicked);
				if (iid.mobileEnabled) 
					this.textShape.connect("ontouchend", this, this._expandButtonDoubleTap);
				
			}
		} catch(err){}
        // setup mouse listeners on expand/collapse buttons:
        if (this._expandButton) {
            this._expandButton.connect("onclick", this, this._expandButtonClicked);
        	if (iid.mobileEnabled) {
				try {
					this._expandButton.connect("ontouchend", this, this._expandButtonTouch);
				} catch (err) {}
			}
        }
        if (this._collapseButton) {
            this._collapseButton.connect("onclick", this, this._collapseButtonClicked);
        	if (iid.mobileEnabled) {
				try {
					this._collapseButton.connect("ontouchend", this, this._collapseButtonTouch);
				} catch(err){}
			}
        }
		
        if (!config.useGfxLayout)
		  this._doLayout();
    },
    
    applyTemplate: function(/*String||Object*/template, /*Object||ibm_ilog.diagram.data.dtlbinding.BoundContext*/context) {
        // summary:
        //     Applies the specified template on this subgraph.
        
        var c = this.inherited(arguments);
        if (c)
            this._subgraphInitialize();
        return c;
    },
    
    getGraph: function() {
        // summary:
        //		Gets the nested graph contained in this Subgraph.
        // returns: ibm_ilog.diagram.Graph
        //		The Graph object contained in this Subgraph.
        return this._graph;
    },
    
    setAnimate: function(/*Boolean*/value){
        // summary:
        //     Sets whether the expand/collapse operation is animated. The default value is true.
        // value: Boolean
    	//		Indicates whether the expand/collapse operation is animated.
        this._animate = value;
        return this;
    },
    
    getAnimate: function() {
        // summary:
        //     Returns whether the expand/collapse operation is animated.
        return this._animate;
    },
    
    isCollapsed: function() {
        // summary:
        //		Gets the collapsed state of this Subgraph.
        // returns: Boolean:
        //		True if the subgraph is collapsed, or false if the subgraph is expanded.
        return this._collapsed;
    },
    setCollapsed: function(/*Boolean*/collapsed, /*Boolean*/animate) {
        // summary:
        //		Collapses or expands this subgraph.
        // collapsed: Boolean:
        //		If true, the subgraph is collapsed, otherwise it is expanded.
        // animate: Boolean
    	//		If specified, indicates whether the expand/collapse operation is animated.
    	//		If this argument is not specified, the <code>getAnimate()</code> flag is used instead.
        
        this._setCollapsed(collapsed, arguments.length == 2 ? animate : this._animate);
        return this;
    },
    _setCollapsed: function(/*Boolean*/v, /*Boolean*/animate) {
        // summary:
        //		Internal method to collapse/expand the subgraph.
        // description:
        //		This method does nothing if v is equal to isCollapsed().
        // v: Boolean:
        //		True to collapse, false to expand.
        // animate: Boolean:
        //		True to animate.
        // tags:
        //		private
        
        if (v != this._collapsed) {

					if(this._animating)
						return;
						
			  if(!this.canCollapse())
				return;
					
		      if (config.useGfxLayout) {
		          this.suspendInvalidate();
		      }
		  
		      this._collapsed = v;
		
		      this.baseShape = v ? this._collapsedBaseShape : this._expandedBaseShape;
		      
		      var toHide, toShow;
		      if (this._collapsed) {
		          toHide = this._expandedGroup;
		          toShow = this._collapsedGroup;
		      } else {
		          toHide = this._collapsedGroup;
		          toShow = this._expandedGroup;
		      }
		      
		      var bbox = toHide.getLocalBoundingBox();
		      
		      this._show(toShow);
		
		      this._realign(toShow, bbox);
			
		      if (animate && !iid.isVml) {
		        var anim = fx.combine([this.__animateShowHide(toShow, true), this.__animateShowHide(toHide, false)]);
		        connect.connect(anim, "onAnimate", this, this._invalidateIntergraphLinks);
						this._animating = true;
		        this.onCollapsedChanged();
		          
		             	connect.connect(anim, "onEnd", this, function(){
							if (config.useGfxLayout) {
								this.resumeInvalidate(true);
								gfxlayout.doPendingValidate(this);
							}
							delete this._animating;
			        this.onCollapsedAnimated();
						});
		                         
		        anim.play();
		      } else {
		        this.onCollapsedChanged();
		        
		        this._hide(toHide);
		        
		        this._invalidateIntergraphLinks();
		
		        if (config.useGfxLayout) {
		            this.resumeInvalidate(true);
					gfxlayout.doPendingValidate(this);
		        }
		        
		      }
		      this._changeSelectionStyle(this.isSelected());
        }
    },
    
	canCollapse: function() {
		// summary:
		//		Indicates if this Subgraph can be collapsed.
		// description:
		//		Indicates if this Subgraph can be collapsed.
		//		This method always returns true for the Subgraph class,
		//		it can be overridden in subclasses that do not support collapsing.
		// returns:
		//		True if this Subgraph can be collapsed, false otherwise.
		
		return true; // Boolean
	},
	
    getFixedPosition: function() {
        // summary:
        //		Gets the fixed position used to realign the subgraph when it is expanded/collapsed.
        // returns:
        //		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the subgraph width/height.
        return this._fixedPosition;
    },
    
    setFixedPosition: function(position) {
        // summary:
        //		Sets a fixed position used to realign the subgraph when it is expanded/collapsed.
        // description:
        //		When a subgraph is expanded or collapsed, it is automatically realigned so that its position
        //		in the new state (expanded or collapsed) matches the position in the previous state.
        //		This method specifies how the subgraph will be realigned by defining a fixed position relative
        //		to the subgraph width/height.
        //		For example, setFixedPosition({x:0, y:0}) (the default value)
        //		realigns the subgraph so that its top-left corner stays at the same position,
        //		and setFixedPosition({x:0.5, y:0.5})
        //		realigns the subgraph so that its center stays at the same position.
        // position:
        //		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the subgraph width/height.
        
        if (typeof(position) != 'object' || !('x' in position) || !('y' in position)) 
            ErrorReporter.error("UnexpectedArgument", this, "setFixedPosition", "position", position);
        
        this._fixedPosition.x = position.x;
        this._fixedPosition.y = position.y;
        return this;
    },
    
    
    _realign: function(group, bbox) {
        // summary:
        //		Realigns the position of the shown group
        //		based on the bbox of the hidden group
        // tag: private
        
        var bb = group.getLocalBoundingBox();
        if (!bb) return;
        var newx = bbox.x + this._fixedPosition.x * (bbox.width - bb.width);
        var newy = bbox.y + this._fixedPosition.y * (bbox.height - bb.height);
        if (bb.x != newx || bb.y != newy) 
            group.applyLeftTransform(matrix.translate(newx - bb.x, newy - bb.y));
    },
	
    _saveStyles: function(group) {
        // summary:
        //		Workaround for IE: removing a shape clears all its styles.
        // tags:
        //		private
        gfxutils.forEach(group, function(shape) {
            if (shape.setFill) {
                shape.oldFill = shape.getFill();
            }
            if (shape.setStroke) {
                shape.oldStroke = shape.getStroke();
            }
            if (shape.setFont) {
                shape.oldFont = shape.getFont();
            }
            if (shape.setShape) {
                shape.oldShape = shape.getShape();
            }
        }, null);
    },
    
    _restoreStyles: function(group) {
        // summary:
        //		Workaround for IE: removing a shape clears all its styles.
        // tags:
        //		private
        gfxutils.forEach(group, function(shape) {
            try {
                if (config.useGfxLayout)
                    shape.suspendInvalidate();
                
                if (shape.oldFill) {
                    shape.setFill(shape.oldFill);
                }
                if (shape.oldStroke) {
                    shape.setStroke(shape.oldStroke);
                }
                if (shape.oldFont) {
                    shape.setFont(shape.oldFont);
                }
                if (shape.oldShape) {
					if('text' in shape.oldShape) // text may have been edited
						shape.oldShape.text = shape.getShape().text;
                    shape.setShape(shape.oldShape);
                }
            } finally {
                if (config.useGfxLayout) {
                    if (shape.oldShape && 'text' in shape.oldShape) 
                        shape._explicitSize = null;
                    shape.resumeInvalidate(true);
                }
            }
        }, null);
    },
    
    onCollapsedAnimated: function() {
      // summary:
      //		Called when the subgraph is about to be been expanded or collapsed.
    	connect.publish("/ibm_diagram/Subgraph/collapsedAnimated",[this]);
    },

    onCollapsedChanged: function() {
        // summary:
        //		Called when the subgraph has been expanded or collapsed.
    	connect.publish("/ibm_diagram/Subgraph/collapsedChanged",[this]);
    },
    
    __animateShowHide: function(group, show) {
        // summary:
        //		Creates and starts the expand/collapse animations.
        // tags:
        //		private
        var anim;
        var startScale, endScale;
        if (show) {
            startScale = 0.1;
            endScale = 1;
        } else {
            startScale = 1;
            endScale = 0.1;
        }
        var r = group.getLocalBoundingBox();
        var cx = r.x + r.width * this._fixedPosition.x;
        var cy = r.y + r.height * this._fixedPosition.y;
        anim = gfxfx.animateTransform({
            shape: group,
            duration: 500,
            transform: [{
                name: "scaleAt",
                start: [startScale, startScale, cx, cy],
                end: [endScale, endScale, cx, cy]
            }, {
                name: "original"
            }]
        });
        if (!show) {
			group._transformBeforeAnim = group.getTransform();
            connect.connect(anim, "onEnd", this, lang.hitch(this, "__endHide", group));
        }
        return anim;
    },
    
    __endHide: function(group) {
        // summary:
        //		Called at the end of the expand/collapse animation.
        // tags:
        //		private
        group.setTransform(group._transformBeforeAnim);
		delete group._transformBeforeAnim;
        // call _hide() after setTransform (crash under IE)
        this._hide(group);
    },
    
    _hide: function(toHide) {
        if (toHide && toHide.getParent()) {
            if ((has("chrome") && iid.isSvg) || iid.isVml) {
                this._saveStyles(toHide);
            }
            toHide._collapsedParent = toHide.getParent();
            toHide._collapsedParent.remove(toHide);
        }
    },
    
    _show: function(toShow) {
        if (toShow && !toShow.getParent()) {
            toShow._collapsedParent.add(toShow);
            toShow._collapsedParent = undefined;
            if ((has("chrome")  && iid.isSvg) || iid.isVml) {
                this._restoreStyles(toShow);
            }
        }
    },
    
    _expandButtonClicked: function(e) {
        // summary:
        //		Expand button click callback.
        // tags:
        //		private
        this._setCollapsed(false, this._animate);
    },
    _collapseButtonClicked: function(e) {
        // summary:
        //		Collapse button click callback.
        // tags:
        //		private
        this._setCollapsed(true, this._animate);
    },
    
    _expandButtonTouch: function(e) {
        // summary:
        //		Expand button click callback.
        // tags:
        //		private
        this._setCollapsed(false, this._animate);
        if (e.preventDefault)
          e.preventDefault();
        
    },
    _expandButtonDoubleTap: function(e) {
        // summary:
        //		Expand button click callback.
        // tags:
        //		private
        
  		if(event.touches.length == 0){
  			var delay = new Date().getTime() - this._lastUp;
  			if(this._lastUp && (delay < 500)){
  				this._setCollapsed(false, this._animate);
  				delete this._lastUp;
  			}else{
  				this._lastUp = new Date().getTime();
  			}   
  		}			
      if (e.preventDefault)
        e.preventDefault();
        
    },
    _collapseButtonTouch: function(e) {
        // summary:
        //		Collapse button click callback.
        // tags:
        //		private
        this._setCollapsed(true, this._animate);
        if (e.preventDefault)
          e.preventDefault();
    },
    
    _collapseButtonDoubleTap: function(e) {
        // summary:
        //		Collapse button click callback.
        // tags:
        //		private
  		if(event.touches.length == 0){
  			var delay = new Date().getTime() - this._lastUp;
  			if(this._lastUp && (delay < 500)){
  				this._setCollapsed(true, this._animate);
  				delete this._lastUp;
  			}else{
  				this._lastUp = new Date().getTime();
  			}   
  		}			
      if (e.preventDefault)
        e.preventDefault();
        
    },
    
	getCollapsedPort: function(link, port) {
		// summary:
		//		Gets a port to which the specified intergraph link will be connected when this
		//		subgraph is collapsed.
		// link: ibm_ilog.diagram.Link:
		//		The intergraph link connected to a node contained in this subgraph.
		// port: ibm_ilog.diagram.PortBase:
		//		The port to which the intergraph link is normally connected when this subgraph is expanded.
		// description:
		//		When a subgraph is collapsed, intergraph links connected to nodes contained in the subgraph
		//		are automatically reconnected to a port of the collapsed subgraph itself.
		//		This method lets you customize which port will be used for this.
		//		If this method returns null (the default), an AutomaticPort will be used by default.
		
		return null;
	},
	
	propagateTextDir: function(/*String*/textdir) {
        // summary: 
        //      propagate a new text direction to children (nodes, links)
        // textdir: String, text direction 
	 
        if (config.isBidiInDojoDiagrammer) {
	   	    array.forEach(this.getGraph().children, function(n) {
	   	        if (n.setTextDir){
	   	            n.setTextDir(textdir);
	   	            if (n instanceof Subgraph)
	   	                n.propagateTextDir(textdir);
	   	        }    
	   	    });
        }	   	    
	},
	
    // 
    // Internal layout (i.e. layout the parts of the template)
    // 
    
    // TODO: should not be hard-coded...
    _graphPosition: null,
    _headerHeight: 20,
    _margin: 2,
    
    layoutShape: function( /*Object*/target, forceDoLayout){
        var c = target._collapsedGroup, e = target._expandedGroup, s, h;
        if (this._collapsed) {
            s = c;
            h = e;
        } else {
            s = e;
            h = c;
        }
        
        target._graphPosition = {x:0,y:0};
        if (forceDoLayout)
            this._doLayout(target);

        if(this._collapsed) {
            var hbb = h.getLocalBoundingBox();
            this._realign(s, hbb);
        }
        if (h) {
            h.getParent().remove(h);
        }        
    },
    
    applyLayout : function(rect){
        if (config.useGfxLayout) {
            Node.prototype.applyLayout.apply(this, arguments);
            // update the subgraph location depending on the graph content bbox
            // (_noTranslate: flag set in the SwimLane subclass)
            if (this.getLayout() && !this.isCollapsed() && !this._noTranslate && this._graph) {
                var graphBB = this._graph.getBoundingBox();
                if (!graphBB) // empty
                    return;
                var t = this._graph.getShapeToContainerTransform(this) || matrix.identity;
                var subBB = this.getBoundingBox();
                var r = t.transformRectangle(graphBB);
                var r2 = subBB;
                var dx =  graphBB.x - (r.x-r2.x);
                var dy =  graphBB.y - (r.y-r2.y);
                t = this._expandedGroup.getTransform();
                if (this._oldSubgraphTransform)
                    t = matrix.multiply(this._oldSubgraphTransform.inverse(), t);
                this._oldSubgraphTransform = matrix.translate(dx, dy);
                t = matrix.multiply(t, this._oldSubgraphTransform);
                this._settingTransformInApplyLayout = true;
                this._expandedGroup.suspendInvalidate();
				this._expandedGroup.setTransform(t);
                this._expandedGroup.resumeInvalidate();
                delete this._settingTransformInApplyLayout;
            }
            
			// realign collapsed group after layout
			if(this.getLayout() && this.isCollapsed()){
				this._realign(this._collapsedGroup, this._expandedGroup.getLocalBoundingBox());
			}
        }
    },
    
    _doLayout: function( /*Object*/target ) {
        // summary:
        //		Layout the part of the subgraph template according to the new bounds of the graph.
        
        //if (/*config.useGfxLayout*/this.getLayout())
        //    return;
        
        target = target || this;
        
        // get new graph bounds
        var graphBounds = this._graph.getLocalBoundingBox();
        if (!graphBounds) {
            graphBounds = {x:0, y:0, width:80, height:40};
    	} else {
            graphBounds = g.cloneRect(graphBounds);
        }
        if (target._header) {
            if (target._graphPosition) {
                var dx = graphBounds.x - target._graphPosition.x;
                var dy = graphBounds.y - target._graphPosition.y;
                target._header.applyLeftTransform(matrix.translate(dx, dy));
            } else {
            	target._graphPosition = {};
            }
            target._graphPosition.x = graphBounds.x;
            target._graphPosition.y = graphBounds.y;
        }
        
        var gm = this.getGraphMargin();
        graphBounds.x -= gm;
        graphBounds.y -= gm;
        graphBounds.width += 2 * gm;
        graphBounds.height += 2 * gm;
        
		var shape;
		
        // move/resize graph background rect
        if (target._graphBackground) {
            shape = target._graphBackground.getShape();
            shape.x = graphBounds.x;
            shape.y = graphBounds.y;
            shape.width = graphBounds.width;
            shape.height = graphBounds.height;
            target._graphBackground.setShape(shape);
        }
        
        // move/resize overall background rect
        var bg = target._expandedBaseShape || target._background; // try also _background for compatibility...
        if (bg) {
            shape = bg.getShape();
            shape.x = graphBounds.x - this._margin;
            shape.y = graphBounds.y - this._headerHeight;
            shape.width = graphBounds.width + 2 * this._margin;
            shape.height = graphBounds.height + this._headerHeight + this._margin;
            bg.setShape(shape);
        }
    },
    
    _graphChanged: function() {
        if (config.useGfxLayout) {
            if (this.getLayout()) 
                this.invalidate();
            else
                this._doLayout();
        } else {
            this._doLayout();
        }
        
    },
    
	setExplicitSize: function(sz)
	{
		// summary:
		//		Does nothing, Subgraphs cannot have an explicit size, their size is determined by their children.
		
		if(this._isIBMDiagramSwimLane)
			this.inherited(arguments);
	},
	
    // 
    // Intergraph links invalidation:
    // (intergraph links are added to the subgraph's _intergraphLinks array by the Link,
    // when setting the start/end ports).
    // 
    
    _onChanged: function(shape, reason) {
    	// This method is called a lot, keep it short and fast :).

    	//	[av] this.inherited(arguments); replaced by:
		Node.prototype._onChanged.apply(this,arguments);
        
    	if(!Node._ChangesInvalidateLinksDisabled) {
	    	if (shape == this && reason == "setTransform") {
	            this._invalidateIntergraphLinks();
	        }
    	}
    },
    
    _invalidateIntergraphLinks: function() {
        if (this._intergraphLinks) {
            array.forEach(this._intergraphLinks, function(link) {
                link.invalidateLinkShape();
            });
        }
    },
    
    getIntergraphLinks: function(){
    	// summary:
        //		return all the intergraph links
    	// return array of ibm_ilog.diagram.Link
        return this._intergraphLinks || [];
    },
    _findAllStyledElements: function() {
    	// summary:
        //		find all the styled elements, from collapsed and expanded groups
    	// override Selectable method
        if(this._collapsedGroup!=null){
        	this._findStyledChildren(this._collapsedGroup.children);
        }
        if(this._expandedGroup!=null){
        	this._findStyledChildren(this._expandedGroup.children);
        }
    },
    
    _applySelectedStyle: function(shape,selected){
    	// summary:
        //		apply the appropriate style to the shape, verifying if it is visible 
    	//      (some styles cannot be applied if no surface is associated )
    	// override Selectable method
    	if(this._isVisible(shape)){
    		this.inherited(arguments);
    	}
    },
    
    _isVisible: function(shape){
    	// summary:
        //		verify if the shape belongs to the shown group
    	var parent = shape.getParent();
    	while(parent!=null){
    		if(parent==this){
    			return true;
    		}
    		parent = parent.getParent();
    	}
    	return false;
    }
    
});

templating.declareBindableProperty(Subgraph, "headerColor", "#6699FF");
templating.declareBindableProperty(Subgraph, "selectedHeaderColor", "#0000FF");
templating.declareBindableProperty(Subgraph, "graphBackgroundColor", "#f8f8f8");
templating.declareBindableProperty(Subgraph, "graphMargin", 10);
templating.declareBindableProperty(Subgraph, "textDir", "");

// The default template for Subgraphs
Subgraph.defaultTemplate = iid.declareTemplate(templates.defaultSubgraphTemplate);

Subgraph.nodeType = gfx.Group.nodeType;
Subgraph.defaultShape = {
    type: 'subgraph'
};

return Subgraph;

});

},
'ibm_ilog/diagram/util/ClippingUtil':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/ClippingUtil", [
  "dojo/_base/lang",	// #1 (for dojo.clone)
  "dojox/gfx/matrix",	// #2 
  "./Geometry" 			// #3
  ],
  function(
    lang,		// #1 (for dojo.clone)
    matrix,		// #2
    Geometry	// #3
    ) {
	
	// TODO: Change to this when AMD conversion is complete:
	// var gu = {...
	var gu = lang.setObject("ibm_ilog.diagram.util.ClippingUtil", {
			
    getClippedPoint: function(bbox/*Rectangle*/,
                              p1/*Point*/, p2/*Point*/) { 
		// Summary:
	    // 		Returns the clipped point at rectangle (node bounding box), assuming 
	    // 		that p1 (attach point for a link in a graph) is inside the bounding box,
	    // 		and p2 (control point for a link) is outside that bounding box.
	    // bbox: 
		//		The rectangle (bounding box).
	    // p1: 
		//		The point (attach point) to which the unclipped line (link)
		//		is attached.
	    // p2:
		//		The auxiliary control point. For the purpose of
	    //      clipping, the line (link) can be considered as a ray that 
	    //      starts at the attach point and goes through the control point.
	    // return:
		//		The clipped point.
		//

      var intersectionPoints = [];
      var numIntersections = this.lineIntersectsRect(p1, p2, bbox, intersectionPoints);      
      // choose the result from the intersections
      return this.bestClipPointOnRay(p1, p2, intersectionPoints, numIntersections);
    },
			
	bestClipPointOnRay: function(startPoint/*Point*/, controlPoint/*Point*/,intersectionPoints/*Points Array*/,numIntersections){
		// Summary:
	    // 		Selects the best of n intersection points of a ray for clipping.
	    // 		If any intersection point is on the ray, it returns the one that is
	    // 		farthest from the start of the ray.
	    // 		If no intersection point is on the ray, it returns the one that is closest
	    // 		to the starting point of the ray.
	    // startPoint: 
		//		The starting point of the ray.
	    // controlPoint: 
		//		Another point of the ray.
	    // intersectionPoints:
		//		An array of intersection points. Typically,
	    //      these are the intersection points of a line through starting and
	    //      control points and the border shape of a node.
	    // numIntersections:
		//		The number of valid intersections in the array.
	    // return:
		//		The best of the intersection points.
		//
		      switch (numIntersections) {
		        case 0:
		          return {x:startPoint.x,y:startPoint.y};
		        case 1:
		          return intersectionPoints[0];
		      }
		      // case that we have 2 ore more intersections. Search the best one.
		      
		      var anyOnRay = false;
		      var closestPoint = null;
		      var farthestPoint = null;
		      var dx;
		      var dy;
		      var distSquare;
		      var closestDistSquare = Number.MAX_VALUE;
		      var farthestDistSquare = -1;
		      
		      for (var i = 0; i < numIntersections; i++) {
		        if (intersectionPoints[i] == null) continue;
		        dx = intersectionPoints[i].x - startPoint.x;
		        dy = intersectionPoints[i].y - startPoint.y;
		        distSquare = dx * dx + dy * dy;
		        if (this.pointOnRay(startPoint, controlPoint, intersectionPoints[i])) {
		          anyOnRay = true;
		          if (distSquare > farthestDistSquare) {
		            farthestDistSquare = distSquare;
		            farthestPoint = intersectionPoints[i];
		          }
		        }
		        else if (!anyOnRay && distSquare <= closestDistSquare) {
		          closestDistSquare = distSquare;
		          closestPoint = intersectionPoints[i];
		        }
		      }
		      
		      return anyOnRay ? farthestPoint : closestPoint;
		    },
		    
		    pointOnRay: function(p1/*Point*/, p2/*Point*/,testPoint/*Point*/){
		    	// Summary:
			    // 		Returns <code>true</code> if the input test point is on a ray.
			    // p1:
			    //		The start point of the ray.
			    // p2: 
			    // 		Another point on the ray.
			    // testPoint: 
			    //	 	The point to be tested.
		      var dx = (p2.x > p1.x ? p2.x - p1.x : p1.x - p2.x);
		      var dy = (p2.y > p1.y ? p2.y - p1.y : p1.y - p2.y);
		      
		      // compare in x or y direction. Choose the direction where the points
		      // have the larger span, to avoid rounding errors
		      if (dx > dy) {
		        if (p1.x < p2.x && p1.x <= testPoint.x)
		          return true;
		        if (p1.x > p2.x && p1.x >= testPoint.x)
		          return true;
		      }
		      else {
		        if (p1.y < p2.y && p1.y <= testPoint.y)
		          return true;
		        if (p1.y > p2.y && p1.y >= testPoint.y)
		          return true;
		      }
		      return false;
		    },
		    
 
		    
		    lineIntersectsSegment: function(p1/*Point*/,p2/*Point*/, p3/*Point*/, p4/*Point*/, res){
		    	// Summary:
			    // 		Calculates the intersection between a line and a line segment.
			    // p1: 
			    // 		 A point on the line.
			    // p2: 
			    // 		 Another point on the line.
			    // p3: 
			    // 		 The first end point of the line segment.
			    // p4: 
			    // 		 The second end point of the line segment.
			    // res: 
			    // 		 Container for the resulting intersection point.
			    // return: 
			    // 		 <code>true</code> if the line intersects the segment
			    //         at a unique point.
		      if (this.pointsEquals(p3,p4)) {
		        // test whether p3 is on the line p1-p2
		        if ((p3.x - p1.x) * (p2.y - p1.y) == (p2.x - p1.x) * (p3.y - p1.y)) {
		          res.x = p3.x;
		          res.y = p3.y;
		          return true;
		        }
		      }
		      else
		        if (this.GetLineIntersection(p1, p2, p3, p4, res) != null) {
		          var dx = (p4.x > p3.x ? p4.x - p3.x : p3.x - p4.x);
		          var dy = (p4.y > p3.y ? p4.y - p3.y : p3.y - p4.y);
		          
		          // compare in x or y direction. Choose the direction where the points
		          // have the larger span, to avoid rounding errors.
		          
		          if (dx > dy) {
		            if (p3.x <= res.x && res.x <= p4.x ||
		              p4.x <= res.x && res.x <= p3.x)
		              return true;
		          }
		          else {
		            if (p3.y <= res.y && res.y <= p4.y ||
		              p4.y <= res.y && res.y <= p3.y)
		              return true;
		          }
		        }
		      return false;
		    },
		    
		    
		    // Checking intersection with a polyline isn't that easy.
		    // First, we check the intersection with all polygon segments.
		    // However, it may happen that a line hits exactly the corner point
		    // between two adjacent polygon segments. Due to rounding errors, the
		    // intersection point may appear outside of both polygon segments (this
		    // happens frequently if the one segment is nearly horizontal and the other
		    // is nearly vertical. Therefore we need a special corner test.
		    //
		    // Example,    Due to rounding errors,
		    //             GetLineIntersction \
		    //             may return          \
		    //             this point ------->  X
		    //             which appears         +----------|
		    //             to be outside         |\
		    //             of both polygon       | \
		    //             segments              |  \
		    //                                   |   \
		    //                                   |    \ line
		    //                                   -     \
		    //
		    // If a line doesn't cross adjacent segments (a,b) and (b,c), but crosses
		    // the segment (a,c), we know that it MUST cross the polygon a-b-c at a
		    // point close to b, but we had rounding errors during the calculation that
		    // makes the calculated point appear outside (a,b) and (b,c). In this case,
		    // we report b as the intersection point.
		    

		    lineIntersectsPolyPoints: function(p1/*Point*/, p2/*Point*/, points, closed, res){
			    // Calculates the intersections between a line and a polyline or polygon.
			    // p1: 
			    // 		  One point of the line.
			    // p2: 
			    // 		  Another point on the line.
			    // points: 
			    // 		  The points of the polyline or polygon.
			    // closed: 
			    // 		  <code>true</code> if it is a polygon,
			    //        <code>false</code> if it is a polyline.
			    // res: 
			    // 		  Container for the resulting intersection points.
			    //            This container must must be large enough to get all
			    //            intersection points.
			    // return: 
			    // 		  The number of intersections.
		      var numIntersections = 0;
		      var pa = null;
		      var pb = null;
		      var pc = null;
		      var intersectsFirstSegment = false;
		      var intersectsAB = false;
		      var intersectsBC = false;
		      var prevCrossPoint = null;
		      var crossPoint = {};
		      var pab = {x:0,y:0};
		      var pbc = {x:0,y:0};
		      
		      for (var i = 0; i < points.length; i++) {
		        pc = points[i];
		        // segment test
		        if (pb != null) {
		          if (this.lineIntersectsSegment(p1, p2, pb, pc, crossPoint)) {
		            if (prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,crossPoint)) {
		              res[numIntersections++] = prevCrossPoint = crossPoint;
		              crossPoint = {};
		              intersectsBC = true;
		              if (i == 1)
		                intersectsFirstSegment = true;
		            }
		          }
		        }
		        // corner test
		        if (pa != null && !intersectsAB && !intersectsBC) {
		          pab.x = 0.5 * (pa.x + pb.x);
		          pab.y = 0.5 * (pa.y + pb.y);
		          pbc.x = 0.5 * (pb.x + pc.x);
		          pbc.y = 0.5 * (pb.y + pc.y);
		          if (this.lineIntersectsSegment(p1, p2, pab, pbc, crossPoint)) {
		            if (prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,pb))
		              res[numIntersections++] = prevCrossPoint = pb;
		          }
		        }
		        
		        pa = pb;
		        pb = pc;
		        intersectsAB = intersectsBC;
		      }
		      
		      var firstCrossPoint = (numIntersections > 0 ? res[0] : null);
		      
		      if (points.length > 2 && closed) {
		        pc = points[0];
		        // segment test
		        if (this.lineIntersectsSegment(p1, p2, pb, pc, crossPoint)) {
		          if ((prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,crossPoint)) &&
		            (firstCrossPoint == null || !this.pointsEquals(firstCrossPoint,crossPoint))) {
		              res[numIntersections++] = crossPoint;
		              crossPoint = {};
		              intersectsBC = true;
		            }
		        }
		        // and two more corner tests
		        if (!intersectsAB && !intersectsBC) {
		          pab.x = 0.5 * (pa.x + pb.x);
		          pab.y = 0.5 * (pa.y + pb.y);
		          pbc.x = 0.5 * (pb.x + pc.x);
		          pbc.y = 0.5 * (pb.y + pc.y);
		          if (this.lineIntersectsSegment(p1, p2, pab, pbc, crossPoint)) {
		            if ((prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,pb)) &&
		              (firstCrossPoint == null || !this.pointsEquals(firstCrossPoint,pb)))
		              res[numIntersections++] = prevCrossPoint = pb;
		          }
		        }
		        pa = points[0];
		        if (!intersectsFirstSegment && !intersectsBC) {
		          pab.x = 0.5 * (pa.x + pc.x);
		          pab.y = 0.5 * (pa.y + pc.y);
		          pbc.x = 0.5 * (pb.x + pc.x);
		          pbc.y = 0.5 * (pb.y + pc.y);
		          if (this.lineIntersectsSegment(p1, p2, pab, pbc, crossPoint)) {
		            if ((prevCrossPoint == null || !this.pointsEquals(prevCrossPoint,pc)) &&
		              (firstCrossPoint == null || !this.pointsEquals(firstCrossPoint,pc)))
		              res[numIntersections++] = pc;
		          }
		        }
		      }
		      
		      return numIntersections;
		    },
		    
    	lineIntersectsRect: function(p1/*Point*/, p2/*Point*/, 
                                 	 rect/*Rectangle*/,
                                 	 res) {
		// Summary:
		// 		Calculates the intersections between a line and a rectangle.
		// p1: 
		// 		One point of the line.
		// p2: 
		// 		Another point on the line.
		// rect: 
		// 		The rectangle to test.
		// res: 
		// 		Container of two for the resulting intersection points.
		// return: 
		// 		 The number of intersections.

      	// get the corner points of the bounding box
		var pts = [];
		pts[0] = {x:rect.x, y:rect.y};
		pts[1] = {x:rect.x + rect.width, y:pts[0].y};
		pts[2] = {x:pts[1].x, y:rect.y + rect.height};
		pts[3] = {x:rect.x, y:pts[2].y};
      
      	// get the intersections between the line through attach and control
      	// point and the polygon defined by pts.
      
      	return this.lineIntersectsPolyPoints(p1, p2, pts, true, res);
    	},
		    
		    lineIntersectsEllipse:function(p1/*Point*/, p2/*Point*/, rect/*dojox.gfx.Rectangle*/, res){
		    	// Summary:
			    // 		Calculates both intersections between a line and an ellipse.
			    // p1: 
			    // 		One point of the line.
			    // p2: 
			    // 		Another point on the line.
			    // rect: 
			    // 		The bounding box of the ellipse.
			    // res: 
			    // 		Container of two for the resulting intersection points.
			    // return: 
			    // 		 The number of intersections.
		      var dx = p2.x - p1.x;
		      var dy = p2.y - p1.y;
		      var a = 0.5 * rect.width;
		      var b = 0.5 * rect.height;
		      var cx = rect.x + a;
		      var cy = rect.y + b;
		      var x;
		      var y;
		      var f;
		      var g;
		      var z;
		      var sqrtz;
		      
		      if (dx == 0.0 && dy == 0.0)
		        return 0;
		      
		      if ((dx > 0.0 ? dx : -dx) > (dy > 0.0 ? dy : -dy)) {
		        f = dy/dx;
		        g = p1.y - cy - f * (p1.x - cx);
		        z = b * b + a * a * f * f - g * g;
		        if (z < 0.0)
		          return 0;
		        
		        if (z == 0.0) {
		          x = - f * g * a * a / (b * b + a * a * f * f);
		          y = f * x + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          return 1;
		        }
		        else {
		          sqrtz = Math.sqrt(z);
		          x = (a * b * sqrtz - f * g * a * a) / (b * b + a * a * f * f);
		          y = f * x + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          x = (- a * b * sqrtz - f * g * a * a) / (b * b + a * a * f * f);
		          y = f * x + g;
		          res[1] = {x:(x + cx),y:(y + cy)};
		          return 2;
		        }
		      }
		      else {
		        f = dx/dy;
		        g = p1.x - cx - f * (p1.y - cy);
		        z = a * a + b * b * f * f - g * g;
		        if (z < 0.0)
		          return 0;
		        
		        if (z == 0.0) {
		          y = - f * g * b * b / (a * a + b * b * f * f);
		          x = f * y + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          return 1;
		        }
		        else {
		          sqrtz = Math.sqrt(z);
		          y = (a * b * sqrtz - f * g * b * b) / (a * a + b * b * f * f);
		          x = f * y + g;
		          res[0] = {x:(x + cx),y:(y + cy)};
		          y = (- a * b * sqrtz - f * g * b * b) / (a * a + b * b * f * f);
		          x = f * y + g;
		          res[1] = {x:(x + cx),y:(y + cy)};
		          return 2;
		        }
		      }
		    },
		    
		    lineIntersectsArc:function(p1/*Point*/, p2/*Point*/,rect/*dojox.gfx.Rectangle*/, startAngle, deltaAngle,res){
		    	// Summary:
		    	// 		Calculates both intersections between a line and an arc (a partial ellipse).
			    // p1: 
			    // 		 One point of the line.
			    // p2: 
			    // 		 Another point on the line.
			    // rect: 
			    // 		 The bounding box of the complete ellipse.
			    // startAngle: 
			    // 		 The angle of the starting point in degrees (0 .. 360).
			    // deltaAngle: 
			    // 		 The angle range of the arc from the starting point to
			    //                   the end point in degrees (0 .. 360).
			    // res: 
			    // 		 Container of two for the resulting intersection points.
			    // return: 
			    // 		 The number of intersections.
		      var tempRes = [];
		      var numPoints = this.lineIntersectsEllipse(p1, p2, rect, tempRes);
		      
		      var i;
		      var j = 0;
		      for (i = 0; i < numPoints; i++) {
		        if (this.ellipticAngle(rect, tempRes[i], startAngle) <= deltaAngle){
		        	j++;
		        	res.push(tempRes[i]);
		        }
		      }
		      
		      return j;
		    },
		    
		    lineIntersectsPartialEllipse: function(p1/*Point*/, p2/*Point*/,rx,ry, startPoint,endPoint,fa,fs,alpha,res){
		    	//Summary:
		    	// 		Calculates both intersections between a line and a partial ellipse.
		    	var center = this.ellipseEndpointToCenterParameterization(startPoint,endPoint,fa,fs,rx,ry,alpha);
		    	var rect = {x:center.x-(rx), y:center.y-(ry), width:rx*2, height:ry*2};
		    	
		    	var pp1 = this._rotatePointAt(p1,center,-alpha);
		    	var pp2 = this._rotatePointAt(p2,center,-alpha);
		    	
		    	var angle;
		    	var delta;
		    	if(fs==1){
		    		angle = this.ellipticAngle(rect, endPoint, 0);
		    		delta = this.ellipticAngle(rect, startPoint, angle);
		    	}else{
		    		angle = this.ellipticAngle(rect, startPoint, 0);
		    		delta = this.ellipticAngle(rect, endPoint, angle);
		    	}
		    	angle = angle + alpha;
		    	var length = res.length
		    	this.lineIntersectsArc(pp1, pp2,rect, angle, delta,res);
		    	if((res.length - length) >= 1){
		    		res[res.length-1] = this._rotatePointAt(res[res.length-1],center,alpha);
		    	}
		    	return res.length - length;
		    	
		    },
		    
		    _rotatePointAt: function(point,center,angle){
		    	var _m = matrix;
		    	var mRot = _m.rotategAt(angle,center);
		    	return _m.multiplyPoint(mRot,point);
		    },
		    
		    
		    lineIntersectsQuadSpline: function(p1/*Point*/, p2 /*Point*/,c/*Array*/,startIndex,res){
		    	// Summary:
		    	// 		Calculates the two intersections between a line and a quadratic Bezier spline.
			    // p1:
		    	// 		 One point of the line.
			    // p2:
		    	// 		 Another point on the line.
			    // c:
		    	// 		  An array of control points of the spline.
			    // startIndex:
		    	// 		 The index of the first valid control point in the array <code>c</code>.
			    //       There must be three valid control points of the spline in the array.
			    // res:
		    	// 		 Container of two for the resulting intersection points.
			    // return:
		    	// 		 The number of intersections.
		    	
		    	
		       // the spline parameters
		      var cx = c[startIndex].x;
		      var bx = 2 * (c[startIndex + 1].x - cx);
		      var ax = c[startIndex + 2].x - bx - cx;
		      var cy = c[startIndex].y;
		      var by = 2 * (c[startIndex + 1].y - cy);
		      var ay = c[startIndex + 2].y - by - cy;
		      
		      // the line parameters
		      var DistX = p2.x - p1.x;
		      var DistY = p2.y - p1.y;
		      
		      // is it a line? if not, return no solution
		      if (DistX == 0.0 && DistY == 0.0)
		        return 0;
		      
		      // the spline formula is (with t = 0..1):
		      //
		      //     x =  ax * t^2 + bx * t + cx
		      //     y =  ay * t^2 + by * t + cy
		      //
		      // calculate all parameters t of intersection points
		      
		      var numSolutions;
		      var t = [0,0];
		      var f;
		      var g;
		      
		      if ((DistX > 0.0 ? DistX : -DistX) > (DistY > 0.0 ? DistY : -DistY)) {
		        f = DistY/DistX;
		        g = p1.y - f * p1.x;
		        numSolutions = this.calcQuadSolution(f*ax-ay, f*bx-by, f*cx-cy+g, t);
		      }
		      else {
		        f = DistX/DistY;
		        g = p1.x - f * p1.y;
		        numSolutions = this.calcQuadSolution(f*ay-ax, f*by-bx, f*cy-cx+g, t);
		      }
		      
		      // Only those solutions between 0 and 1 are relevant
		      var numPoints = 0;
		      for (var i = 0; i < numSolutions; i++) {
		        if (0 <= t[i] && t[i] <= 1) {
		          res[numPoints++] ={x:(ax * t[i] * t[i] + bx * t[i] + cx),y:(ay * t[i] * t[i] + by * t[i] + cy)};
		        }
		      }
		      
		      return numPoints;
		    },
		   
		   lineIntersectsCubicSpline: function(p1/*Point*/, p2/*Point*/, c/*Array*/, startIndex, res){
			    // Summary:
			    // 		Calculates all three intersections between a line and a cubic Bezier spline.
			    // p1:
		    	// 		 One point of the line.
			    // p2:
		    	// 		 Another point on the line.
			    // c:
		    	// 		  An array of control points of the spline.
			    // startIndex:
		    	// 		 The index of the first valid control point in the array <code>c</code>.
			    //                   There must be four valid control points of the spline in the array.
			    // res:
		    	// 		 Container of three for the resulting intersection points.
			    // return:
		    	// 		 The number of intersections.
		    	
		    	// the spline parameters
		      var dx = c[startIndex].x;
		      var cx = 3 * (c[startIndex + 1].x - dx);
		      var bx = 3 * (c[startIndex + 2].x - c[startIndex + 1].x) - cx;
		      var ax = c[startIndex + 3].x - bx - cx - dx;
		      var dy = c[startIndex].y;
		      var cy = 3 * (c[startIndex + 1].y - dy);
		      var by = 3 * (c[startIndex + 2].y - c[startIndex + 1].y) - cy;
		      var ay = c[startIndex + 3].y - by - cy - dy;
		      
		      // the line parameters
		      var DistX = p2.x - p1.x;
		      var DistY = p2.y - p1.y;
		      
		      // is it a line? if not, return no solution
		      if (DistX == 0.0 && DistY == 0.0)
		        return 0;
		      
		      // the spline formula is (with t = 0..1):
		      //
		      //     x =  ax * t^3 + bx * t^2 + cx * t + dx
		      //     y =  ay * t^3 + by * t^2 + cy * t + dy
		      //
		      // calculate all parameters t of intersection points
		      
		      var numSolutions;
		      var t = [0,0,0];
		      var f;
		      var g;
		      
		      if ((DistX > 0.0 ? DistX : -DistX) > (DistY > 0.0 ? DistY : -DistY)) {
		        f = DistY/DistX;
		        g = p1.y - f * p1.x;
		        numSolutions = this.calcCubicSolution(f*ax-ay, f*bx-by, f*cx-cy, f*dx-dy+g, t);
		      }
		      else {
		        f = DistX/DistY;
		        g = p1.x - f * p1.y;
		        numSolutions = this.calcCubicSolution(f*ay-ax, f*by-bx, f*cy-cx, f*dy-dx+g, t);
		      }
		      
		      // Only those solutions between 0 and 1 are relevant
		      var numPoints = 0;
		      for (var i = 0; i < numSolutions; i++) {
		        if (0 <= t[i] && t[i] <= 1)
		          res[numPoints++] ={x:((ax * t[i] * t[i] * t[i] + bx * t[i] * t[i] + cx * t[i] + dx)),y:((ay * t[i] * t[i] * t[i] + by * t[i] * t[i] + cy * t[i] + dy))};
		      }
		      
		      return numPoints;
		    },

		    GetLineIntersection: function( p1LA/*Point*/, p2LA/*Point*/,p1LB/*Point*/, p2LB/*Point*/,result){
		    	// Summary:
			    // 		Returns the intersection point between two lines.
			    //  p1LA:
			    // 		 One point on the first line.
			    //  p2LA:
			    // 		 Another point on the first line.
			    //  p1LB:
			    // 		 One point on the second line.
			    //  p2LB:
			    // 		 Another point on the second line.
			    //  result:
			    // 		 If not <code>null</code>, this point is used a container
			    //               for the return value and is filled with the intersection
			    //               point if the lines intersect.
			    //               If <code>null</code>, a new point is allocated internally
			    //               and returned, if the lines intersect.
			    // return:
			    // 		 The unique intersection point, or <code>null</code> if the two lines are parallel.
			    
		      return this.GetLineIntersection1( p1LA.x, p1LA.y, p2LA.x, p2LA.y,p1LB.x, p1LB.y, p2LB.x, p2LB.y,result);
		    },
		    
		    
		    GetLineIntersection1: function(ax1, ay1, ax2, ay2, bx1, by1, bx2, by2, result){
		    	// Summary:
		    	// 		Returns the intersection point between two lines.
			    //  ax1:
			    // 		  The x coordinate of the first point on the first line.
			    //  ay1:
			    // 		  The y coordinate of the first point on the first line.
			    //  ax2:
			    // 		  The x coordinate of the second point on the first line.
			    //  ay2:
			    // 		  The y coordinate of the second point on the first line.
			    //  bx1:
			    // 		  The x coordinate of the first point on the second line.
			    //  by1:
			    // 		  The y coordinate of the first point on the second line.
			    //  bx2:
			    // 		  The x coordinate of the second point on the second line.
			    //  by2:
			    // 		  The y coordinate of the second point on the second line.
			    //  result:
			    // 		  If not <code>null</code>, this point is used a container
			    //               for the return value and is filled with the intersection
			    //               point if the lines intersect.
			    //               If <code>null</code>, a new point is allocated internally
			    //               and returned, if the lines intersect.
			    // return:
			    // 		  The unique intersection point, or <code>null</code> if the two lines are parallel.
		    	
		      var dx1 = ax2 - ax1;
		      var dy1 = ay2 - ay1;
		      var dx2 = bx2 - bx1;
		      var dy2 = by2 - by1;
		      
		      // if cross determinantes are equal, the segments are parallel
		      var dx1y2 = dx1 * dy2;
		      var dx2y1 = dx2 * dy1;
		      
		      if (dx1y2 == dx2y1)
		        return null;
		      
		      // for precision, first handle vertical case, then general case
		      if (ax1 == ax2)
		        result.x = ax1;
		      else if (bx1 == bx2)
		        result.x = bx1;
		      else
		        result.x = (ax1 * dx2y1 - bx1 * dx1y2 + (by1  - ay1) * dx1 * dx2) / (dx2y1 - dx1y2);
		      
		      // for precision, first handle horizontal case, then general case
		      if (ay1 == ay2)
		        result.y = ay1;
		      else if (by1 == by2)
		        result.y = by1;
		      else
		        result.y =  (ay1 * dx1y2 - by1 * dx2y1 + (bx1 - ax1) * dy1 * dy2) / (dx1y2 - dx2y1);
		      
		      return result;
		    },
		    
		 // --------------------------------------------------------------------------
		 // Auxiliaries related to arcs / ellipses
		    
		    ellipticAngle: function(rect/*dojox.gfx.Rectangle*/, p/*Point*/, startAngle){
		    // Summary:
		    // 		Returns the elliptic angle of point p relative to the center of rect where
		    // 		the rect is considered the bounding box of an ellipse and startAngle
		    // 		indicates the origin of the elliptic scale.
		    //  rect:
	    	// 		 The bounding box of the complete ellipse.
		    //  p:
	    	// 		 The point whose elliptic angle is calculated.
		    //  startAngle:
	    	// 		 The angle that defines the 0 point.
		    // return:
	    	// 		 The elliptic angle (relative to startAngle). The value is between 0 and 360.
		      var dx = p.x - rect.x - rect.width / 2;
		      var dy = rect.y + rect.height / 2 - p.y;
		      var x = dx / rect.width;
		      var y = dy / rect.height;
		      var a = 180.0 / Math.PI *  Math.atan2(y, x);
		      a -= startAngle;
		      while (a < 0.0) a += 360.0;
		      while (a > 360.0) a -= 360.0;
		      return a;
		    },
		    
		    retrieveArcPointFromAngle: function(rect/*dojox.gfx.Rectangle*/, angle){
		    // Summary:
	    	// 		Returns the point on an arc from an elliptic angle.
		    //  rect:
	    	// 		 The bounding box of the complete ellipse.
		    //  angle:
	    	// 		 The elliptic angle to be considered.
		    //  p:
	    	// 		 Container for the resulting point.	
		    	
		      angle = angle * Math.PI / 180.0;
		      var p ={};
		      p.x = rect.x + 0.5 * rect.width * (1 + Math.cos(angle));
		      p.y = rect.y + 0.5 * rect.height * (1 - Math.sin(angle));
		      return p;
		    },
		    
		    
		    arcStartPoint: function(rect/*dojox.gfx.Rectangle*/, startAngle){
		    // Summary:
		    // 		Returns the starting point of an arc (a partial ellipse).
		    //  rect:
		    // 		 The bounding box of the complete ellipse.
		    //  startAngle:
			// 		  The angle of the starting point in degrees (0 .. 360).
		    // return:
			// 		  The starting point.
		    	
		      var angle = startAngle;
		      var p = retrieveArcPointFromAngle(rect, angle);
		      
		      // now, EllipticAngle should return 0.0. However, due to rounding, it
		      // may return a value representing an angle close to 0.0. We can accept
		      // angles slightly larger than 0 degree, but not angles slightly smaller
		      // than 0 degree because they would create gaps between the arc and the
		      // point. Note that an angle smaller than 0 degree is calculated as
		      // a value around 359.
		      
		      var testAngle = ellipticAngle(rect, p, startAngle);
		      
		      if (testAngle > 180.0) {
		        // search a valid angle by choosing a delta offset
		        var delta = 0.0;
		        while (testAngle > 180.0) {
		          delta += 0.004;
		          angle = startAngle + delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // try to reduce the delta offset until the angle gets invalid
		        var bestDelta = delta;
		        while (testAngle < 180.0) {
		          bestDelta = delta;
		          delta = delta/8;
		          angle = startAngle + delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // now get the resulting point from the last valid angle
		        angle = startAngle + bestDelta;
		        p = retrieveArcPointFromAngle(rect, angle);
		      }
		      return p;
		    },
		    
		    
		    arcEndPoint: function(rect/*dojox.gfx.Rectangle*/, startAngle, deltaAngle){
		    // Summary:
		    // 		Returns the end point of an arc (a partial ellipse).
		    // rect:
		    // 		  The bounding box of the complete ellipse.
		    // startAngle:
		    // 		  The angle of the starting point in degrees (0 .. 360).
		    // deltaAngle:
		    // 		  The angle range of the arc from the starting point to the end point in degrees (0 .. 360).
		    // return:
		    // 		  The end point.
		    	
		      if (deltaAngle == 0.0)
		        return arcStartPoint(rect, startAngle);
		      
		      var angle = startAngle + deltaAngle;
		      var p = retrieveArcPointFromAngle(rect, angle);
		      
		      // now, EllipticAngle should return deltaAngle. However, due to rounding,
		      // it may return a value representing an angle close to deltaAngle. We can
		      // accept angles slightly smaller than deltaAngle, but not angles slightly
		      // larger than deltaAngle because they would create gaps between the arc
		      // and the point.
		      
		      var testAngle = ellipticAngle(rect, p, startAngle);
		      
		      if (testAngle > deltaAngle) {
		        // search a valid angle by choosing a delta offset
		        var delta = 0.0;
		        while (testAngle > deltaAngle) {
		          delta += 0.004;
		          angle = startAngle + deltaAngle - delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // try to reduce the delta offset until the angle gets invalid
		        var bestDelta = delta;
		        while (testAngle < deltaAngle) {
		          bestDelta = delta;
		          delta = delta/8;
		          angle = startAngle + deltaAngle - delta;
		          p = retrieveArcPointFromAngle(rect, angle);
		          testAngle = ellipticAngle(rect, p, startAngle);
		        }
		        // now get the resulting point from the last valid angle
		        angle = startAngle + deltaAngle - bestDelta;
		        p = retrieveArcPointFromAngle(rect, angle);
		      }
		      return p;
		    },
		    
		    pointsEquals: function(p1,p2){
		    	return p1.x==p2.x && p1.y==p2.y;
		    },
		    
		    calcLinearSolution: function( a, b, result){
		    // Summary:
		    // 		 Calculates the solution of the equation a * x + b = 0.
		    //  a:
		    // 		 Parameter of the equation.
		    //  b:
		    // 		 Parameter of the equation.
		    //  result:
		    // 		 Container for the result. The array must have size 1.
		    // return:
		    // 		  The number of solutions. Return -1 if the equation is redundant, that is, if every x is a solution.
		    
		    	
		      if (a == 0.0) {
		        if (b == 0.0) return -1;
		        return 0;
		      }
		      result[0] = -b / a;
		      return 1;
		    },	
		    
		    calcQuadSolution: function( a, b, c, result) {
		    // Summary:
		    // 		 Calculates the solution of the equation a * x^2 + b * x + c = 0.
		    //  a:
		    // 		  Parameter of the equation.
		    //  b:
		    // 		  Parameter of the equation.
		    //  c:
		    // 		  Parameter of the equation.
		    //  result:
		    // 		  Container for the result. The array must have size 2.
		    // return:
		    // 		  The number of solutions. Return -1 if the equation is redundant, that is, if every x is a solution.
		    	
		    	
		      if (a == 0.0)
		        return this.calcLinearSolution(b, c, result);
		      
		      // solution is (+/- sqrt(b^2 - 4ac) - b)/(2a)
		      
		      var D = b * b - 4 * a * c;
		      if (D < 0) return 0;
		      if (D == 0) {
		        result[0] = -b / (2 * a);
		        return 1;
		      }
		      var sqrtD = Math.sqrt(D);
		      result[0] = (sqrtD - b) / (2 * a);
		      result[1] = (-sqrtD - b) / (2 * a);
		      return 2;
		    },	  
		    
		    calcCubicSolution: function(a, b, c, d, result){
		    // Summary:
		    // 		 Calculates the solution of the equation a * x^3 + b * x^2 + c * x + d = 0.
		    // a:
		    // 		  Parameter of the equation.
		    // b:
		    // 		  Parameter of the equation.
		    // c:
		    // 		  Parameter of the equation.
		    // d:
		    // 		  Parameter of the equation.
		    // result:
		    // 		  Container for the result. The array must have size 3.
		    // return:
		    // 		  The number of solutions. Return -1 if the equation is redundant, that is, if every x is a solution.
		    	
		    	
		      if (a == 0.0)
		        return this.calcQuadSolution(b, c, d, result);
		      
		      // We use a variant of the Cardan-Tartaglia formula (see any good
		      // mathematics formula collectione, e.g., Bronstein 1985, pp. 131-132)
		      
		      var oneThird = 1.0 / 3.0;
		      var z = b / (3 * a);
		      var r = 3 * a * a;
		      var p = (3 * a * c - b * b) / r;
		      var q = (2 * b * b * b - 9 * a * b * c + 9 * r * d) / (9 * r * a);
		      var sgn;
		      
		      // now, solve y^3 + p * y + q = 0  and x = y - z
		      
		      if (p == 0.0) {
		        // 1 solution: y = - q^(1/3)
		        sgn = (q >= 0.0 ? 1.0 : -1.0);
		        result[0] = -sgn * Math.pow(sgn * q, oneThird) - z;
		        return 1;
		      }
		      
		      var cubicPThird = p * p * p / 27;
		      var squareQHalf = q * q / 4;
		      var D = cubicPThird + squareQHalf;
		      
		      if (D < 0) {
		        // The formula of Cardan, Bombelli, and others.
		        // The background is basically the same as below, except that we
		        // need to get the square root of a negative number and hence end up
		        // in the complex number world. Luckily, the i-part of the complex
		        // numbers eliminate each other in the end result.
		        
		        // 3 solutions
		        var rho = Math.sqrt(-cubicPThird);
		        var phi = Math.acos(-q / (2 * rho));
		        sgn = (rho >= 0.0 ? 1.0 : -1.0);
		        var f = 2 * sgn * Math.pow(sgn * rho, oneThird);
		        result[0] = f * Math.cos(phi / 3) - z;
		        result[1] = f * Math.cos((phi + 2 * Math.PI) / 3) - z;
		        result[2] = f * Math.cos((phi + 4 * Math.PI) / 3) - z;
		        return 3;
		      } else {
		        // The original Cardan-Tartaglia formula. The background is this:
		        // (u + v)^3 - 3uv(u + v) - u^3 - v^3 = 0
		        // hence y = (u+v) is a solution iff p=-3uv and u^3+v^3+q = 0
		        // With v=-p/(3u) follows u^3-p^3/(27u^3)+q = 0
		        // that is u^6 + q u^3 - p^3/27 = 0
		        // With u1 = u^3 follows u1^2 + q u1 - p^3/27 = 0.
		        // This quadratic formula can be solved with the standard mechanism to u1,
		        // hence u and v can be obtained. The second solution can be found by
		        // factorizing the first solution.
		        
		        var u1 = (q > 0
		          ? // Avoid rounding errors due to cancellation
		          cubicPThird / (Math.sqrt(D) + q / 2)
		          : Math.sqrt(D) - q / 2);
		        sgn = (u1 >= 0.0 ? 1.0 : -1.0);
		        var u = sgn * Math.pow(sgn * u1, oneThird);
		        
		        // u cannot be zero, because p != 0. Only if p = 0 follows D = q/2
		        // hence u1 = 0 hence u = 0. But just to be sure of rounding problems,
		        // we repeat:
		        
		        if (u == 0.0) {
		          // see above, case p==0.0
		          sgn = (q >= 0.0 ? 1.0 : -1.0);
		          result[0] = -sgn * Math.pow(sgn * q, oneThird) - z;
		          return 1;
		        }
		        
		        var v = -p / (3 * u);
		        if (u == v) {
		          // 2 solutions
		          result[0] = u + v - z;
		          result[1] = -0.5 * (u + v) - z;
		          return 2;
		        } else {
		          // 1 solution
		          result[0] = u + v - z;
		          return 1;
		        }
		      }
		    },	  
		    
		    // --------------------------------------------------------------------------
			// Auxiliaries related to matrix
		    
		    multiplyVectorial: function(matrix,vector){
		    // Summary:
		    //		multiplies a 2x2 matrix towards a 2x1 vector
		    	var rta = [0,0];
		    	for(var i=0;i<2;i++){
		    		for(var j=0;j<2;j++){
		    			rta[i] = rta[i] + ( matrix[i][j] * vector[j]);
		    		}
		    	}
		    	return rta;
		    },
		    multiplyScalar: function(vector,scalar){
			    // Summary:
			    //		multiplies a matrix towards a scalar
			    	var rta = lang.clone(vector);
			    	for(var i=0;i<vector.length;i++){
			    			rta[i] = vector[i]* scalar;
			    	}
			    	return rta;
			},
			add: function(v1,v2){
			    // Summary:
			    //		adds a 2x2 matrix towards a 2x2 matrix
			    	var rta = [0,0];
			    		for(var j=0;j<2;j++){
			    			rta[j] = v1[j] + v2[j];
			    		}
			    	return rta;
			    },
			
			ellipseEndpointToCenterParameterization: function(p1,p2,fa,fs,rx,ry,alphaDegrees){
				//Summary:
			    //		Change the parameterization of an ellipse, from endpoint to center.
			    var	alpha = this.toRadians(alphaDegrees); 
				
				//Step 1: Compute (x1', y1') according to the formula
				var m1 = [[Math.cos(alpha),Math.sin(alpha)],[-Math.sin(alpha),Math.cos(alpha)]];
				var v1 = [(p1.x-p2.x)/2,(p1.y-p2.y)/2];
				var pt = this.multiplyVectorial(m1,v1);
				
				//Step 2: Compute (cX', cY') according to the formula
				var t1 = Math.pow(rx,2)*Math.pow(ry,2);
				var t2 = Math.pow(rx,2)*Math.pow(pt[1],2);
				var t3 = Math.pow(ry,2)*Math.pow(pt[0],2);
				var sqrt = Math.pow((t1 - t2 - t3)/(t2 + t3),(1/2));
				var v2 =[rx*pt[1]/ry,-ry*pt[0]/rx];
				var ct = this.multiplyScalar(v2,sqrt);//(cX', cY')
				
				//the - sign is chosen if fa = fs   
				if(fa==fs){
					ct = this.multiplyScalar(ct,-1);
				}
				
				//Step 3: Compute (cX, cY) from (cX', cY')
				var m2 = [[Math.cos(alpha),-Math.sin(alpha)],[Math.sin(alpha),Math.cos(alpha)]];
				var v3 = [(p1.x+p2.x)/2,(p1.y+p2.y)/2];
				var center = this.add(this.multiplyVectorial(m2,ct),v3);
				
				return {x:center[0],y:center[1]};
				
			},
			ellipticalAngle: function(startPoint,endPoint,center){
				var v1 = {x:startPoint.x-center.x,y:startPoint.y-center.y};
				var v2 = {x:endPoint.x-center.x,y:endPoint.y-center.y};
				return vectorsAngle(v1,v2);
			},
			vectorsAngle: function(v1,v2){ 
				var t1 = v1.x*v2.x+v1.y*v2.y;
				var t2 = this.vectorLenght(v1)*this.vectorLenght(v2);
				return Math.acos(t1/t2);
			},
			vectorLenght: function(v){
				return Math.pow(Math.pow(v.x,2)+Math.pow(v.y,2),1/2);
			},
			toDegrees: function(radian){
				return (360*radian)/(2*Math.PI);
			},
			toRadians: function(degrees){
				return (degrees*2*Math.PI)/360;
			},
			
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////    GFX SHAPES CLIPPING FUNCTIONS     ///////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			
		clipToShape: function(node,graph,shape,bounds,unusedPoint, connectionPoint, referencePoint,completeResult){	
				
				var result = [],resultPoint,numberOfResults = 0, clipConnectionPoint = {}, points, poli, 
					s, stype = shape.getShape ? shape.getShape().type : null; 
				clipConnectionPoint.x = connectionPoint.x;
				clipConnectionPoint.y = connectionPoint.y;
				
				if(stype === 'rect'){
					s = shape.getShape();
					points = [{x:s.x,y:s.y},{x:s.x+s.width,y:s.y},{x:s.x+s.width,y:s.y+s.height},{x:s.x,y:s.y+s.height},{x:s.x,y:s.y}];
					poli = this._transformPoints(shape/*node*/,graph,points);
					numberOfResults = this.lineIntersectsPolyPoints(referencePoint,clipConnectionPoint,poli,true,result);
				}else if(stype === 'circle' || stype === 'ellipse'){
					numberOfResults = this.lineIntersectsEllipse(referencePoint,clipConnectionPoint,bounds,result);
				}else if(stype === 'polyline'){
					points = shape.getShape().points;
					poli = this._transformPoints(shape/*node*/,graph,points);
					numberOfResults = this.lineIntersectsPolyPoints(referencePoint,clipConnectionPoint,poli,true,result);
				}else if(stype === 'path'){
					numberOfResults = this._clipOnPath(node,graph,shape,unusedPoint, clipConnectionPoint, referencePoint,result);
				}else if (shape instanceof Object && shape.width && shape.height){
					// workaround to deal with raw rectangle
					s = shape;
					points = [{x:s.x,y:s.y},{x:(s.x+s.width),y:s.y},{x:(s.x+s.width),y:(s.y+s.height)},{x:s.x,y:(s.y+s.height)},{x:s.x,y:s.y}];
					numberOfResults = this.lineIntersectsPolyPoints(referencePoint,clipConnectionPoint,points,true,result);
				}
				
				if(completeResult){
					completeResult.points = this._filterPoints(result,clipConnectionPoint, referencePoint);
					completeResult.numberOfResults = completeResult.points.length;
				}
				
				if(numberOfResults>0){
					// FIXME using the original results because the bounding box is not being calculated correctly
					//var resultPoint = this.bestClipPointOnRay(referencePoint,clipConnectionPoint,completeResult.points,completeResult.numberOfResults);
					
					resultPoint = this.bestClipPointOnRay(clipConnectionPoint,referencePoint,result,numberOfResults);
					return resultPoint;
				}else{
					return null;
				}
			},
			
			_filterPoints: function(points,p1,p2){
				var g = Geometry;
				var result = [];
				var rect = g.createRect(p1,p2);
				for(pn in points){
					var p = points[pn];
					if(g.containsPoint(rect,p)){
						result.push(p);
					}
				}
				return result;
			},
			
			_transformPoints: function(node,graph,points){
				// Summary:
				//		transform the given points, to the absolute graph coordinates
				
				var result = [];
				for(pn in points){
					var p = points[pn];
					p = this._transformPoint(node,graph,p);
					result.push(p);
				}
				return result;
			},
			
			_transformPoint: function(node,graph,point){
				// Summary:
				//		transform the given point, to the absolute graph coordinates
				var _m = matrix;
				//var p2 = _m.multiplyPoint(node.getTransform(), point);
				var p2 =  _m.multiplyPoint(node.getShapeToContainerTransform(graph), point);
				return p2;
			},
			
			_clipOnPath: function(node,graph,shape,unusedPoint, connectionPoint, referencePoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Path shape.
				var segments = shape.segments;
				var lastPoint = this._transformPoint(shape/*node*/,graph,{x:0,y:0});
				var reflectionControlPoint = {};
				var firstPointPath = this._transformPoint(shape/*node*/,graph,{x:0,y:0});
				for(var index in segments){
					var segment = segments[index];
					var action = segment.action;
					var args = segment.args;
						if( action == 'M'){ //absolute move
							lastPoint = {x:segments[index].args[0],y:segments[index].args[1]};
							reflectionControlPoint = {}; 
							firstPointPath = lastPoint; continue;}
						if( action == 'm'){ //relative move
							lastPoint =  this._changeToAbsolute( {x:segments[index].args[0],y:segments[index].args[1]},lastPoint);
							reflectionControlPoint = {}; 
							firstPointPath = lastPoint; continue;}
						if( action == 'Z' || action == 'z' ){ // close path
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,firstPointPath,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'L' ){//absolute line
							var p2 = {x:args[0],y:args[1]};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'l' ){//relative line
							var p2 = this._changeToAbsolute({x:args[0],y:args[1]},lastPoint);
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'H' ){//horizontal absolute line
							var p2 =  {x:args[0],y:lastPoint.y};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'h' ){//horizontal relative line
							var p2 = {x:args[0]+lastPoint.x,y:lastPoint.y};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'V' ){//vertical absolute line
							var p2 = {x:lastPoint.x,y:args[0]};
							lastPoint = this._clipOnLinearPath(shape/*node*/,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'v' ){//vertical relative line
							p2 = {x:lastPoint.x,y:args[0]+lastPoint.y};
							lastPoint = this._clipOnLinearPath(shape/*node*/,graph,lastPoint,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections);
							continue;}
						if( action == 'C' || action == 'c' || action == 'S' || action == 's' ){ // cubic bezier
							lastPoint = this._clipOnCubicPath(shape/*node*/,graph,segment, connectionPoint, referencePoint, lastPoint, reflectionControlPoint, intersections);
							continue;}
						if( action == 'Q' || action == 'q' || action == 'T' || action == 't'){ // quadratic bezier
							lastPoint = this._clipOnQuadraticPath(shape/*node*/,graph,segment, connectionPoint, referencePoint, lastPoint, reflectionControlPoint, intersections);
							continue;}
						if( action == 'A' || action == 'a' ){ 
							lastPoint = this._clipOnArcPath(shape/*node*/,graph,segment, connectionPoint, referencePoint, lastPoint, reflectionControlPoint, intersections);
							continue;}
				}
				return intersections.length;
			},
			
			_clipOnLinearPath: function(node,graph,p1,p2,connectionPoint,referencePoint,reflectionControlPoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Linear Path shape.
				var crossPoint = {};
				
				if (this.lineIntersectsSegment(connectionPoint,referencePoint,this._transformPoint(node,graph,p1), this._transformPoint(node,graph,p2), crossPoint)) {
		            intersections.push(crossPoint);
				}
				reflectionControlPoint.action = 'l';
				return p2;
			},
			
			_clipOnQuadraticPath: function(node,graph,segment,connectionPoint,
																		referencePoint,lastPoint,reflectionControlPoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Quadratic Bezier Path shape.
				var args = segment.args;
				var result = [];
				var p1;
				var p2;
				
				// DIFFERENCE BETWEEN Q (normal) AND T (smooth) 
				if( segment.action.toLowerCase() == 'q'){
					p1 = {x:args[0],y:args[1]};
					p2 = {x:args[2],y:args[3]};
				}else{ // case T
					p2 = {x:args[0],y:args[1]};
					if( reflectionControlPoint != {} && reflectionControlPoint.action.toLowerCase() == 'q' ){
						p1 = {x:lastPoint.x+reflectionControlPoint.x, y:lastPoint.y+reflectionControlPoint.y};
					}else{
						p1 = {x:lastPoint.x, y:lastPoint.y};
					}
				}
				
				// RELATIVE Transform
				if( segment.action == 'q'){
					p1 = this._changeToAbsolute(p1,lastPoint);
					p2 = this._changeToAbsolute(p2,lastPoint);
				}
				if( segment.action == 't'){
					p2 = this._changeToAbsolute(p2,lastPoint);
				}
				
				// set reflection point
				reflectionControlPoint.x = p2.x-p1.x;
				reflectionControlPoint.y = p2.y-p1.y;
				reflectionControlPoint.action = 'q';
				
				var points = [lastPoint,p1,p2];
				var controlPoints = this._transformPoints(node,graph,points);
				var n = this.lineIntersectsQuadSpline(referencePoint,connectionPoint,controlPoints,0,result);
				for(var index in result){
					intersections.push(result[index]);
				}
				return p2;
			},
			
			_clipOnCubicPath: function(node,graph,segment,connectionPoint,referencePoint,
																lastPoint,reflectionControlPoint,intersections){
				// Summary:
				//		return the connection point, clipping to a Cubic Bezier Path shape.
				
				var args = segment.args;
				var result = [];
				var p1;
				var p2;
				var p3;
				if( segment.action.toLowerCase() == 'c'){
					p1 = {x:args[0],y:args[1]};
					p2 = {x:args[2],y:args[3]};
					p3 = {x:args[4],y:args[5]};
				}else{ // case T
					p2 = {x:args[0],y:args[1]};
					p3 = {x:args[2],y:args[3]};
					if( reflectionControlPoint != {} && reflectionControlPoint.action.toLowerCase() == 'c' ){
						p1 = {x:lastPoint.x+reflectionControlPoint.x, y:lastPoint.y+reflectionControlPoint.y};
					}else{
						p1 = {x:lastPoint.x, y:lastPoint.y};
					}	
				}
				
				if( segment.action == 'c'){
					p1 = this._changeToAbsolute(p1,lastPoint);
					p2 = this._changeToAbsolute(p2,lastPoint);
					p3 = this._changeToAbsolute(p3,lastPoint);
				}
				if( segment.action == 's'){
					p2 = this._changeToAbsolute(p2,lastPoint);
					p3 = this._changeToAbsolute(p3,lastPoint);
				}
				
				reflectionControlPoint.x = p3.x-p2.x;
				reflectionControlPoint.y = p3.y-p2.y;
				reflectionControlPoint.action = 'c';
				
				var points = [lastPoint,p1,p2,p3];
				var controlPoints = this._transformPoints(node,graph,points);
				var n = this.lineIntersectsCubicSpline(referencePoint,connectionPoint,controlPoints,0,result);
				for(var index in result){
					intersections.push(result[index]);
				}
				return p3;
			},
			
			_clipOnArcPath:function(node,graph,segment, connectionPoint, referencePoint, 
															lastPoint, reflectionControlPoint, intersections){
				// Summary:
				//		return the connection point, clipping to a partial arc Path.
				var args = segment.args;
				var p2 = {x:args[5],y:args[6]};
				var rx = args[0];
				var ry = args[1];
				var	xAxisRotation = args[2];
				var largeArcFlag = args[3]; 
				var sweepFlag =	args[4];
				
				if(segment.action == 'a'){
					p2 = this._changeToAbsolute(p2,lastPoint);
				}
				var result = [];
				var n = this.lineIntersectsPartialEllipse(referencePoint,connectionPoint,rx,ry, 
					this._transformPoint(node,graph,lastPoint),
					this._transformPoint(node,graph,p2),largeArcFlag,sweepFlag,xAxisRotation,result);
				for(var index in result){
					intersections.push(result[index]);
				}
				
				reflectionControlPoint.action = 'a';
				
				return p2;
				
			},
			
			_changeToAbsolute: function(point, lastPoint){
				// Summary:
				//		return the given point, changing its relative coordinates to absolute, according to the last point.
				return {x: point.x+lastPoint.x, y:point.y+lastPoint.y}
			},		
			
			_eof:function(){}
	});
	
  return gu;
});

},
'ibm_ilog/diagram/SwimLane':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/SwimLane", [
"./main",
"dojo/_base/config", "dojo/_base/declare", "dojo/_base/lang",
"dojox/gfx", "dojox/gfx/matrix",
"./templating", "./templates/templates", "./Subgraph",
"./gfxlayout/CanvasLayout",
"./util/Batch", "./util/ErrorReporter"
],
function(iid,
config, declare, lang,
gfx, matrix,
templating, templates, Subgraph, CanvasLayout, Batch, ErrorReporter){

/*=====
Subgraph = ibm_ilog.diagram.Subgraph;
=====*/

var SwimLane =
declare("ibm_ilog.diagram.SwimLane", [Subgraph], {
	// summary:
	//		Represents a swim lane in a Dojo Diagrammer graph.
	// description:
	//		The <code>SwimLane</code> object is a special type of <code>Subgraph</code>
	//		used to divide a diagram into horizontal or vertical "stripes" that typically
	//		represent the different participants in a business process or workflow diagram.
	//		The differences with a plain <code>Subgraph</code> object are:
	//		<ul>
	//		  <li>
	//		    The swim lane has its own position and size, it does not adapt automatically to its contents.
	//		  </li>
	//		  <li>
	//		    The position and orientation of the title bar can be changed using the <code>setVertical()</code>
	//		    and <code>setRightToLeft()</code> methods.
	//		  </li>
	//		  <li>
	//		    Nested swim lanes (that is, swim lanes contained in another swim lane) can be automatically
	//		    stacked inside their parent lane (see <code>setStackSublanes</code>).
	//		  </li>
	//		</ul>
	
	_vertical: false,
	_rightToLeft: false,
	_stackSublanes: true,
	
	_isIBMDiagramSwimLane: true,
	_flattenForGL: true,
	_noTranslate: true,
	
	constructor: function()
	{
		if(!config.useGfxLayout)
			ErrorReporter.error("SwimLanesRequireGfxLayout");
		this._explicitSize = { width: 100, height: 100};
	},
	
    applyTemplate: function(/*String||Object*/template, /*Object||ibm_ilog.diagram.data.dtlbinding.BoundContext*/context) {
        // summary:
        //     Applies the specified template on this subgraph.
        
        var c = this.inherited(arguments);
        this._template = template;
        return c;
    },
	
	_subgraphInitialize: function()
	{
		this.inherited(arguments);
		if(this._graph){
			this._graph.setLayout(new _SwimLaneLayout());
			this._graph._flattenForGL = true; // we mark both the subgraph and its graph for efficiency
			var g = this._graph.getParent();
			if(g.declaredClass == gfx.Group.prototype.declaredClass)
				g.setLayout(new _GroupLayout());
		}
	},
	
	setVertical: function(vertical)
	{
		// summary:
		//		Sets the orientation of the swim lane.
		// description:
		//		Sets the orientation of the swim lane.
		// vertical: Boolean:
		//		If <code>true</code>, the title of the swim lane will be displayed at the top of the graph.
		//		If <code>false</code>, the title of the swim lane will be displayed (vertically)
		//		on the left (if <code>isRightToLeft()</code> is <code>false</code>) or
		//		on the right (if <code>isRightToLeft()</code> is <code>true</code>)
		//		of the graph.
		
		if(vertical != this._vertical){
			this._vertical = vertical;
			this.applyTemplate(this._template);
		}
        return this;
	},
	
	getVertical: function()
	{
		// summary:
		//		Gets the orientation of the swim lane.
		// description:
		//		Gets the orientation of the swim lane (see <code>setVertical</code>).
		
		return this._vertical;
	},
	
	setRightToLeft: function(rightToLeft)
	{
		// summary:
		//		Sets the horizontal direction of the swim lane.
		// description:
		//		Sets the horizontal direction of the swim lane.
		//		
		//		This property has an effect only if <code>isVertical()</code> is <code>false</code>.
		// rightToLeft: Boolean:
		//		If <code>false</code>, the title of the swim lane will be displayed on the left of the graph.
		//		If <code>rightToLeft</code> is <code>true</code>, the title of the swim lane
		//		will be displayed on the right of the graph.
		
		if(rightToLeft != this._rightToLeft){
			this._rightToLeft = rightToLeft;
			this.applyTemplate(this._template);
		}
        return this;
	},
	
	getRightToLeft: function()
	{
		// summary:
		//		Gets the horizontal direction of the swim lane.
		// description:
		//		Gets the horizontal direction of the swim lane (see <code>setRightToLeft()</code>.
		
		return this._rightToLeft;
	},
	
	setStackSublanes: function(stack)
	{
		// summary:
		//		Sets the sub-lane stacking mode.
		// description:
		//		Sets the sub-lane stacking mode.
		//		
		//		Sub-lanes (that is, <code>SwimLane</code> objects contained in this <code>SwimLane</code>)
		//		are stacked vertically if <code>getVertical()</code> is <code>false</code>, or
		//		horizontally if <code>getVertical()</code> is <code>true</code>.
		// stack: Boolean:
		//		If <code>true</code>, sub-lanes will be stacked, otherwise they will be left to their
		//		current position and size.
		
		if(stack != this._stackSublanes){
			this._stackSublanes = stack;
			this.invalidate();
		}
        return this;
	},
	
	getStackSublanes: function()
	{
		// summary:
		//		Gets the sub-lane stacking mode.
		// description:
		//		Gets the sub-lane stacking mode (see <code>setStackSublanes()</code>).
		
		return this._stackSublanes;
	},
	
	getNested: function()
	{
		// summary:
		//		Returns true if this swim lane is a nested lane contained in another swim lane.
		
		var parent = this.getParent();
		return parent && parent._owningSubgraph && parent._owningSubgraph._isIBMDiagramSwimLane;
	},
	
	canCollapse: function() {
		// summary:
		//		Returns false if this swim lane is nested.
		// returns:
		//		True if this SwimLane is not nested, false if it is nested.
		
		return !this.getNested(); // Boolean
	},
	
    applyLayout: function(rect)
	{
		this.inherited(arguments);
		if (this._graphRect) {
			var gr = this._graphRect.getBoundingBox();
			var t = this._graphRect.getShapeToContainerTransform(this._graph);
			gr = t.transformRectangle(gr);
			this._graph.setMoveLimits(gr);
		}
	}
});

// The default template for SwimLanes
SwimLane.defaultTemplate = iid.declareTemplate(templates.defaultSwimLaneTemplate);

SwimLane.nodeType = gfx.Group.nodeType;
SwimLane.defaultShape = {
    type: 'swimlane'
};

templating.declareBindableProperty(SwimLane, "poolSpacing", 20);

/*=====
var CanvasLayout = ibm_ilog.diagram.gfxlayout.CanvasLayout;
=====*/

var _SwimLaneLayout =
declare('ibm_ilog.diagram._SwimLaneLayout', CanvasLayout, {
	// summary:
	//		A private subclass of CanvasLayout used in swim lanes to stack sub-lanes.
	// tags:
	//		private
	
    layoutChildren: function(contBounds) {
        var children = this._container.children, count = children.length, o, i;
        if (count === 0)
            return;
        var sl = this._container._owningSubgraph;
        var pos = 0;
        for (i=0; i<count;++i) {
            var child  = children[i];
            // Process only panels (ie a Group with a Layout)
            if (!child.children || !child.getLayout())
                continue;
			var sz = child._isIBMDiagramSwimLane ?
						child.getExplicitSize() :
						child.getPreferredSize();
            var x = 0, y = 0, w = sz.width, h = sz.height;
            // Stack sub-lanes if required.
            if(sl && sl._stackSublanes && child._isIBMDiagramSwimLane){
				var graphRect = sl._graphRect || sl._graphBackground;
           		var gbb = graphRect.getBoundingBox();
            	if(sl._vertical){
            		x = pos;
            		h = gbb.height;
            		pos += w;
            	} else {
            		y = pos;
            		w = gbb.width;
            		pos += h;
            	}
            	child.setExplicitSize({width:w, height:h});
            	var gt = sl._graph.getTransform();
            	if(gt){
            		var p = {x:x, y:y};
            		p = gt.inverse().transformPoint(p);
            		x = p.x; y = p.y;
            	}
            	//child.move(x, y);
				// move the top-left point of the child lane's base shape instead,
				// for the case when nodes are outside of the swim lane
				var baseShape = child.getBaseShape();
		        var rect = baseShape.getBoundingBox();
				var t = baseShape.getShapeToContainerTransform(this._container);
				rect = t.transformRectangle(rect);
				var dx = x - rect.x;
				var dy = y - rect.y;
				if(dx !== 0 || dy !== 0)
					child.applyLeftTransform(matrix.translate(dx, dy));
            }
            child.layout({x:0,y:0,width:w,height:h});
        }
    }
});

var _GroupLayout =
declare('ibm_ilog.diagram._GroupLayout', CanvasLayout, {
	// summary:
	//		A private subclass of Layout used in swim lanes...
	// tags:
	//		private
	
    layoutChildren: function(contBounds) {
        var children = this._container.children, count = children.length, o, i;
        if (count === 0)
            return;
        for (i=0; i<count;++i) {
            var child  = children[i];
            // Process only panels (ie a Group with a Layout)
            if (!child.children || !child.getLayout())
                continue;
            var sz = child.getPreferredSize();
            child.layout({x:0,y:0,width:sz.width,height:sz.height});
        }
    }
});

lang.mixin(SwimLane, {
	
	// automaticHierarchicalLayout: boolean
	//		Controls whether swim lanes are automatically laid out when a Hierarchical layout
	//		is performed on the graph. If the flag is set to false, the beforeHierarchicalLayout/afterHierarchicalLayout
	//		methods are called respectively before and after the layout, to let applications implement their own layout.
	automaticHierarchicalLayout: true,
	
	beforeHierarchicalLayout: function(graph, hl){
		// summary:
		//		Called if automaticHierarchicalLayout is set to false before a Hierarchical layout
		//		is performed on the graph.
		// description:
		//		Called if automaticHierarchicalLayout is set to false before a Hierarchical layout
		//		is performed on the graph. This method must return an  array of swim lanes, that will
		//		be passed back to afterHierarchicalLayout.
		// graph: ibm_ilog.diagram.Graph
		//		The toplevel graph on which the layout is performed.
		// hl: ibm_ilog.graphlayout.hierarchical.HierarchicalLayout
		//		The hierarchical layout that is being performed.
		// returns:
		//		An array of Swimlane objects that will be passed back to afterHierarchicalLayout.
		return []; // Array
	},
	
	afterHierarchicalLayout: function(graph, hl, swimlanes){
		// summary:
		//		Called if automaticHierarchicalLayout is set to false after a Hierarchical layout
		//		is performed on the graph.
		// description:
		//		Called if automaticHierarchicalLayout is set to false after a Hierarchical layout
		//		is performed on the graph. This method should perform a custom layout on the swim lanes.
		// graph: ibm_ilog.diagram.Graph
		//		The toplevel graph on which the layout is performed.
		// hl: ibm_ilog.graphlayout.hierarchical.HierarchicalLayout
		//		The hierarchical layout that is being performed.
		// swimlanes: Array
		//		The array of SwimLane obejcts that was returned by beforeHierarchicalLayout.
	},
	
	_beforeLayout: function(graph, hl, Direction, HierarchicalNodeGroup, HierarchicalSwimLaneConstraint)
	{
		// summary:
		//		Creates HierarchicalSwimLaneConstraint objects for all swim lanes in the graph.
		
		if (!this.automaticHierarchicalLayout) {
			return this.beforeHierarchicalLayout(graph, hl);
		}
		
		// collect all swim lanes
		var swimlanes = [];
		this._getSwimLanes(graph, swimlanes);
		
		var n = swimlanes.length;
		
		if(n == 0)
			return null;
		
		var l = swimlanes[0];

		var dir = l._vertical ?
				Direction.BOTTOM :
					(l._rightToLeft ?
							Direction.LEFT :
								Direction.RIGHT);
		
		if(dir == Direction.RIGHT)
			swimlanes.reverse();
		
		hl.setFlowDirection(dir);
		
		// create a swim lane constraint for each lane
		for(var i = 0; i < n; i++){
			l = swimlanes[i];
			
			// add all nodes in a node group
			var g = new HierarchicalNodeGroup();
			var children = l._graph.children;
			for(var j = 0; j < children.length; j++){
				var node = children[j];
				if(node._isIBMDiagramNode && !node._isIBMDiagramSwimLane)
					g.add(node);
			}
			
			// compute margins
			var m = l.getGraphMargin();
			var wm = m;
			var em = m;
			if(l._isFirstLaneInPool || l._isLastLaneInPool){
				m = l.getPoolSpacing()/2;
				if(l._vertical || l._rightToLeft){
					if(l._isFirstLaneInPool)
						wm += m;
					if(l._isLastLaneInPool)
						em += m;
				} else {
					if(l._isFirstLaneInPool)
						em += m;
					if(l._isLastLaneInPool)
						wm += m;
				}
			}
			
			// create the constraint
			var c = new HierarchicalSwimLaneConstraint(g, 0, i, wm, em);
			l._hierarchicalConstraint = c;
			hl.addConstraint(c);
		}
		
		// return the swim lane (or null if empty)
		return swimlanes.length > 0 ? swimlanes : null;
	},
	
	_getSwimLanes: function(graph, list)
	{
		var children = graph.children;
		var n = children.length;
		for(var i = 0; i < n; i++){
			var child = children[i];
			if(child._isIBMDiagramSwimLane){
				// expand all swim lanes, collapsed lanes are not well supported by graph layout.
				if(child.isCollapsed())
					child.setCollapsed(false, false);
				var old = list.length;
				// first see if it has nested swim lanes in it:
				if (child._graph)
					this._getSwimLanes(child._graph, list);
				if(list.length == old) // no: return it in the list.
					list.push(child);
				else {
					// don't stack lanes automatically during layout
					child._oldStackSublanes = child._stackSublanes;
					child.setStackSublanes(false);
				}
				// mark the first and last lane/sublane inside toplevel lanes,
				// so we can add spacing margins
				if(!child.getParent()._owningSubgraph){
					list[old]._isFirstLaneInPool = true;
					list[list.length-1]._isLastLaneInPool = true;
				}
			}
		}
		
		return list;
	},
	
	_afterLayout: function(graph, hl, swimlanes)
	{
		// summary:
		//		Updates the position/size of all swim lanes according to info computed by the hierarchical layout.
		
		if (!this.automaticHierarchicalLayout) {
			this.afterHierarchicalLayout(graph, hl, swimlanes);
			return;
		}
		
		// First update the swim lanes that actually contain nodes
		// (those that correspond to constraints)
		var n = swimlanes.length;
		var i, l;
		for(i = 0; i < n; i++){
			l = swimlanes[i];
			var c = l._hierarchicalConstraint;
			var bb = c.getCalcBoundingBox();
			
			// add north/south margins, and remove extra west/east margin
			// corresponding to pool spacing.
			var m = l.getGraphMargin();
			var wm = c.getWestMinMargin() - m;
			var em = c.getEastMinMargin() - m;
			if(l._vertical){
				bb.y -= m;
				bb.height += 2*m;
				bb.x += wm;
				bb.width -= wm + em;
			} else {
				bb.x -= m;
				bb.width += 2*m;
				bb.y += l._rightToLeft ? wm : em;
				bb.height -= wm + em;
			}
			this._moveResizeLane(graph, l, bb);
		}
		
		// make sure all sublanes have the correct size already...
		Batch.endBatch();
		
		// Now, update swim lanes that contain other swim lanes,
		// and that must just fit their children lanes
		this._updateParentLanes(graph);
		
		// cleanup
		for(i = 0; i < n; i++){
			l = swimlanes[i];
			hl.removeConstraint(l._hierarchicalConstraint);
			delete l._hierarchicalConstraint;
		}
	},
	
	_moveResizeLane: function(graph, sl, rect)
	{
		// summary:
		//		Moves/resizes a lane so that its 'graph rect' becomes rect.
		//		Rect is given in 'graph' coordinates.
		
		// we will work in the coord system of the lane, so transform
		// the rect (which is passed in graph coords) to this system:
		var t = sl.getTransformToContainer(graph);
		if(t)
			rect = t.inverse().transformRectangle(rect);
		
		// get current lane's subgraph rect, to compute header width/height
		var graphRect = sl._graphRect || sl._graphBackground;
		var gr = graphRect.getBoundingBox();
        var t = graphRect.getShapeToContainerTransform(sl) || matrix.identity;
		gr = t.transformRectangle(gr);

		// current lane bounds: we do not use the real bounds including all nodes,
		// because at this point nodes can be completely ouside of the lane, 
		// instead we get the current explicit size and the current transform...
		t = sl.getTransform();
		var bb = {
				x: t ? t.dx : 0,
				y: t ? t.dy : 0,
				width : sl.getExplicitSize().width || sl.getBounds().width,
				height : sl.getExplicitSize().height || sl.getBounds().height
		};
		
		// lane translation to get to the desired position
		// (taking into account subgraph offsets)
		var dx = rect.x - gr.x - bb.x;
		var dy = rect.y - gr.y - bb.y;
		
		// resize lane (also taking into account diff between graph rect and lane bbox)
		var sz = bb;
		var w = rect.width + sz.width - gr.width;
		var h = rect.height + sz.height - gr.height;
		sl.setExplicitSize({width:w, height:h});
		
		// translate the lane,
		sl.applyLeftTransform(matrix.translate(dx, dy));
		
		// and inversely translate its graph
		// so that nodes still appear at the same position (computed by the layout)
		
		sl._graph.applyLeftTransform(matrix.translate(-dx, -dy));
	},
	
	_updateParentLanes: function(graph)
	{
		var children = graph.children;
		var n = children.length;
		for(var i = 0; i < n; i++){
			var child = children[i];
			if(child._isIBMDiagramSwimLane && !child._hierarchicalConstraint){
				var graphBounds = child._graph.getBoundingBox();
				t = child._graph.getShapeToContainerTransform(graph);
				if(t)
					graphBounds = t.transformRectangle(graphBounds);
				this._moveResizeLane(graph, child, graphBounds);
				this._updateParentLanes(child._graph);
				child.setStackSublanes(child._oldStackSublanes);
				delete child._oldStackSublanes;
			}
		}
	}
});

return SwimLane;

});


},
'ibm_ilog/diagram/adorners/Adorner':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/Adorner", [
"../main",
"dojo/_base/declare",
"dojo/_base/config",
"dojo/_base/lang",
"dojo/_base/connect",
"dojox/gfx/matrix",
"../templating",
"./TemplatedOverlay",
"../util/HandleSet",
"../util/GraphUtil",
"../util/Geometry",
"../util/ErrorReporter"
],
function(
iid,
declare,
config,
lang,
connect,
m,
templating,
TemplatedOverlay,
HandleSet,
gu,
g,
R
){

var __invisibleLocation = g.Rect(iid._invisibleLocation, iid._invisibleLocation, iid._invisibleSize, iid._invisibleSize);

/*=====
var TemplatedOverlay = ibm_ilog.diagram.adorners.TemplatedOverlay;
var HandleSet = ibm_ilog.diagram.util.HandleSet;
=====*/

var a = iid.gfxDeclaration(declare('ibm_ilog.diagram.adorners.Adorner', [TemplatedOverlay, HandleSet], {
	//
	//	summary:
	//		A UI overlay associated with a graph entity.
	//
	//	description:
	//		The Adorner object provides a graphical decoration for a
	//		graph entity. The decoration is composed of a graphical 
	//		representation given by the adorner template and also by a set of smaller 
	//		AdornerHandle entities.
	//		Each adorner usually defines a fixed set of handles which can be used. AdornerHandles 
	//		usually provide an interactive service to the user. For example, the handle 
	//		can connect to the onmousedown event of its shapes to perform some kind of 
	//		action on the adorned. If this action leads to an interaction process that 
	//		is not instantaneous, then the AdornerHandle can push an Interactor object 
	//		into the interactors stack, blocking other conflicting interactors 
	//		temporarily while the action is performed. For example, during the creation 
	//		of links some interactions may be disabled.
	//
	//	construction:
	//		Adorners don't use parameterized constructors. Instead, they rely on an 
	//		initialize method. For example:
	//
	//			new ibm_ilog.diagram.editor.ResizeAdorner().initialize(diagram,
	//		                                                    templatePool,
	//			                                                  "ResizeAdorner0");
	//
	//		The reason for this is that the creation is implemented with a template 
	//		method, and thus the template code implemented in the superclass needs the 
	//		methods already overriden. Currently dojo executes superclass constructors 
	//		before overriding methods redeclared in the subclass. 
	//
	//	TODO:
	//    . update extendCreateShape for using deserialization
	//		. change individual props for composite: elemBounds instead of elemLeft, elemTop, elemW, elemH
	//

	//   
	//	_adorned:
	//		The adorned element
	//
	_adorned: null,

	//   
	//	_changeConnections:
	//		The connections handles of the current connections to change events related to the bound shape
	//
	_changeConnections: [],

	//   
	//	_elementBounds:
	//		The bounding box of the bound element, in Graph coordinates
	//
	_elementBounds: null,

	//   
	//	_viewport:
	//		The viewport at which the Adorner resides
	//
	_viewport: null,

	//   
	//	_templatePool:
	//		The dictionary used for assigning templates to the AdornerHandles the Adorner
	//
	_templatePool: null,

	//   
	//	_diagram:
	//		The diagram at which the Adorner resides
	//
	_diagram: null,

	//   
	_onEGDisposeConn: null,

	initialize: function (diagram, templatePool, templateId) {
		//
		//	summary:
		//		Initializes an Adorner after construction.
		//
		//	diagram:
		//		The Diagram Dijit
		//
		//	templatePool:
		//		The template pool. The adorner gets its template from this pool using the 
		//		templateId argument, and also for instantiating its handles with the 
		//		templates from the pool.
		//
		//	templateId:
		//		the Id for the adorner template in the pool
		//
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._templatePool = templatePool;
		
		this._elementBoundsChanged();
		
        if (config.isBidiInDojoDiagrammer) {
		    var tDir = diagram._graph.textDir;          
		    if (tDir)                                   
			    this.setTextDir(tDir);
		}

		var template = templatePool.item(templateId);
		this.applyTemplate(template);
		
		//Added to remove the adorner if binded element is removed
		this._onEGDisposeConn = connect.connect(this._diagram,'beforeGraphElementDispose',this,'_onElementDisposed');
	},

	getAdorned: function () {
		//
		// 	summary:
		//		gets the bounded element
		//
		return this._adorned;
	},

	getViewport: function () {
		//
		// 	summary:
		//		gets the viewport
		//
		return this._viewport;
	},
	
	getDiagram: function () {
		//
		// 	summary:
		//		gets the diagram
		//
		return this._diagram;
	},

	activate: function () {
		//
		// 	summary:
		//		activates the adorner
		//
	},

	deactivate: function () {
		//
		// 	summary:
		//		deactivates the adorner
		//
	},

	_invalidated: 0,
	_invalidationCallbackH: null,
	_invalidationCallback: function() {
		//
		//	summary:
		//		The callback used in the _invalidate() setTimeout. Also called by forceUpdate();
		//
		this._elementBoundsChanged();
		this._invalidationCallbackH = null;
		this._invalidated = 0;
	},
	
	_invalidate: function() {
		//
		// 	summary:
		//		Requests invalidation of the adorner shape. 
		//	description:
		//		Usually based on a change notification. The update is batched with a setTimeout(0).
		//		_invalidate() is used for changes that are reported a lot or those that don't really
		//		need an immediate update.
		//		forceUpdate() can be used to clear the timeout and update immediately.
		//
		if(!this._invalidated) {
			var thiz = this;
			this._invalidationCallbackH = setTimeout(lang.hitch(this,"_invalidationCallback"),0);
		}
		this._invalidated++;
	},
	
	forceUpdate: function() {
		//
		//	summary:
		//		Used to clear the timeout and update the adorner shape immediately.
		//
		if(this._invalidationCallbackH) {
			clearTimeout(this._invalidationCallbackH);
		}
		this._invalidationCallback();
	},
	
	_reconnectNotifications: function () {
		//
		// 	summary:
		//		Reconnect to change notifications related to the bound element. 
		//

		if (this._adorned && !this._isConnected("v")) {
			this._connect("v", this._viewport, "onViewRectChanged", this,"_updateLayout");
		} else if(!this._adorned) {
			this._disconnect("v");
		}
		var cc = this._changeConnections;
		for (var i = 0; i < cc.length; i++) {
			connect.disconnect(cc[i]);
		}
		this._changeConnections = cc = [];
		var e = this._adorned;
		if (e) {
			gu.doAncestors(e, function (sg) {
				if (sg._isIBMDiagramSubgraph) {
					var gr = sg.getGraph();
					cc.push(connect.connect(sg, "onBoundsChanged", this, function (s, reason) {
						this._invalidate();
					}));
					cc.push(connect.connect(sg, "onCollapsedChanged", this, function () {
						this._invalidate();
					}));
					cc.push(connect.connect(sg, "onCollapsedAnimated", this, function () {
						this._invalidate();
					}));
					cc.push(connect.connect(sg, "_setParent", this, function () {
						this._reconnectNotifications();
						this._invalidate();
					}));
				}
			}, this);

			cc.push(connect.connect(e, "_setParent", this, function () {
				if(this._adorned && this._adorned._no_reconnect_on_add) return; // to avoid an infinite event loop
				this._reconnectNotifications();
				this._invalidate();
			}));

			// last shape can receive child updates
			cc.push(connect.connect(e, "onBoundsChanged", this, "forceUpdate"));
		}
	},

	setAdorned: function (element) {
		//
		//	summary:
		//		apply the adorner to the given element
		//
		if (this._adorned !== element) {
			if (this._adorned) {
				this.deactivate();
			}
			this._adorned = element;

			this._reconnectNotifications();
			this._elementBoundsChanged();

			if (this._adorned) {
				this.activate();
			}
		}
	},
	
	_required: function() {
		return this._adorned && gu.isStableVisible(this._adorned); 
	},

	_setFallbackBounds: function() {
		this._elementBounds = null;
		this._elementBaseBounds = null;
	},
	
	_computeElementBoundsChanged: function () {
		if(this._required()) {
			this._elementBounds = this._adorned.getBounds(this._diagram.getGraph().getParent());
			var bs = this._adorned.baseShape;
			if(bs) {
			  var pm = this._diagram.getGraph()._getRealMatrix().inverse();
			  var cm = bs._getRealMatrix() || m.identity;
			  var t = m.multiply(pm,cm);
			  this._elementBaseBounds = t.transformRectangle(bs.getBoundingBox());
			} else {
				this._elementBaseBounds = null;
			}
		} else {
			this._setFallbackBounds();
		}
	},

	_elementBoundsChanged: function () {
		this._computeElementBoundsChanged();
		this._updateLayout();
	},

	_updateLayout: function () {

		var bb = __invisibleLocation, bbb = __invisibleLocation;
		if (this._required()) {
			// There is an adorned, and so this._elementBounds has meaningful data 
			var vt = this._viewport.contentToClient();
			bb = vt.transformRectangle(this._elementBounds);
			bbb = vt.transformRectangle(this._elementBaseBounds || this._elementBounds);
		}
		this.startDTLBatch();
		
		this.setElemWidth(bb.width);
		this.setElemHeight(bb.height);
		this.setElemRight(bb.x + bb.width);
		this.setElemBottom(bb.y + bb.height);
		this.setElemLeft(bb.x);
		this.setElemTop(bb.y);
		this.setElemCenterX(bb.width / 2);
		this.setElemCenterY(bb.height / 2);
		this.setElemBaseWidth(bbb.width);
		this.setElemBaseHeight(bbb.height);
		this.setElemBaseRight(bbb.x + bbb.width);
		this.setElemBaseBottom(bbb.y + bbb.height);
		this.setElemBaseLeft(bbb.x);
		this.setElemBaseTop(bbb.y);
		this.setElemBaseCenterX(bbb.width / 2);
		this.setElemBaseCenterY(bbb.height / 2);

		this.endDTLBatch();
	},
	_onElementDisposed: function(graphElement){
		// Summary:
		//		this method is called when a graph element is 
		if(this._adorned && graphElement===this._adorned){
			this.setAdorned(null);
		}
	},
	
	destroy: function(){
		// summary:
		//		removes the diagram connection
		connect.disconnect(this._onEGDisposeConn);
	},

	// ==============================================================
	//	These are copied from ibm_ilog.diagram.Node to support owning Ports/Links
	//
	getBounds: function(/*dojox.gfx.Group*/parent) { 
	  // summary:
	  //		Returns the bounds of this node in the coordinate space of the specified parent.
	  // parent: dojox.gfx.Group
	  //		The parent object that determines the coordinate space
	  //		the bounds are relative to. If null or undefined, the bounds of the node
	  //		relative to its parent are returned.
	  // returns:
	  //		A rectangle (i.e. an object with x/y/width/height properties) containing
	  //		the bounds of the node in the specified coordinate space.
	
		/*
		  var rect = this.getLocalBoundingBox();
		  if (rect && parent && parent != this.getParent()) {
		      var t = this.getTransformToContainer(parent);
		      rect = t.transformRectangle(rect);
		  }
		  */
		
		// Changed by Eric: the previous implementation seems wrong, it returns {x: ibm_ilog.diagram._invisibleLocation, y: ibm_ilog.diagram._invisibleLocation, etc }...
		// (looks like the _invisibleLocation thing?...)
		// The following, using the bounds of the baseShape, works better.
		// Abel, please check that this will always work...
			
		if(!parent)
			parent = this.getParent();
		
		var baseShape = this.getBaseShape();
		var rect = baseShape.getBoundingBox();
		// t1: transform from surface -> base shape
		var t1 = baseShape._getRealMatrix() || m.identity;
		// t2: transform from surface -> parent
		var t2 = parent._getRealMatrix() || m.identity;
		var t = m.multiply(t2.inverse(), t1);
		rect = t.transformRectangle(rect);
		
		return rect;
	},
	
	getBaseShape: function() {
		return this.baseShape;
	},
	
	onChanged: function() {
		
	},
	
	_onChanged: function() {
		this.onChanged();
	}

}));

templating.declareBindableProperty(a, "elemBaseLeft", null);
templating.declareBindableProperty(a, "elemBaseTop", null);
templating.declareBindableProperty(a, "elemBaseBottom", null);
templating.declareBindableProperty(a, "elemBaseRight", null);
templating.declareBindableProperty(a, "elemBaseWidth", null);
templating.declareBindableProperty(a, "elemBaseHeight", null);
templating.declareBindableProperty(a, "elemBaseCenterX", null);
templating.declareBindableProperty(a, "elemBaseCenterY", null);

templating.declareBindableProperty(a, "elemLeft", null);
templating.declareBindableProperty(a, "elemTop", null);
templating.declareBindableProperty(a, "elemBottom", null);
templating.declareBindableProperty(a, "elemRight", null);
templating.declareBindableProperty(a, "elemWidth", null);
templating.declareBindableProperty(a, "elemHeight", null);
templating.declareBindableProperty(a, "elemCenterX", null);
templating.declareBindableProperty(a, "elemCenterY", null);
templating.declareBindableProperty(a, "textDir", null);      

return a;

});

},
'ibm_ilog/diagram/widget/GfxViewport':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/GfxViewport", [
"dojo/_base/declare",
"dojo/_base/sniff",
"dojo/_base/window",
"dojo/_base/connect",
"dojo/dom-attr",
"dojox/gfx/matrix",
"./Viewport",
"./ScrollbarHudComponent",
"../util/HandleSet",
"../util/Geometry",
"../util/ErrorReporter"
], function(
declare,
has,
window,
connect,
doma,
matrix,
Viewport,
ScrollbarHudComponent,
HandleSet,
g,
R
){

var Size = g.Size;

/*=====
var Viewport = ibm_ilog.diagram.widget.Viewport;
var HandleSet = ibm_ilog.diagram.util.HandleSet;
=====*/

var GfxViewport =
declare('ibm_ilog.diagram.widget.GfxViewport', [Viewport, HandleSet], {
	//
	//	summary:
	//		This is a Viewport class based on an HTML <div> which overflows with scrollbars.
	//
	//	description:
	//
	//		DivViewport depends on a div declared at Diagram.html with attachPoint=scrollbarsNode.
	//		This div encloses the diagram Surface. 
	//		
	//		The diagram Surface is extended to enclose the whole diagram tightly. The 
	//		div is constrained to the desired client size using its width and height 
	//		properties. The view rectangle is then associated with the div scroll 
	//		positions, scrollTop and scrollLeft. Setting the view rectangle then 
	//		updates scrollTop and scrollLeft.
	//
	//		The zoom level is derived from the view rectangle, but is mapped entirely 
	//		different than its position. Instead, the zoom controls a scale transformation 
	//		of the entire Diagram. If the view rectangle is small, the Diagram is zoomed 
	//		in to fit the requested portion into the viewport window. With this also the 
	//		Surface is scaled, as it is required that the entire Graph fits the surface 
	//		exactly so that the scrollTop/scrollLeft properties work.
	//		
	//	note:
	//		the zooming service required from the Diagram object should be at a UI 
	//		level and should not affect, for example, the bouding box of the the Graph 
	//		objects. One option is to define an intermediate node between the Surface root 
	//		and the Graph node, which controls this transformation.
	//
		
	_hScrollbarVisible: true,
	
	_vScrollbarVisible: true,

	templateString: function() {
		return "<div style='overflow:hidden; position:relative'>"+
		        	"<div dojoAttachPoint='canvasNode' style='z-index:1; position:absolute; top:0px; left:0px'></div>"+
		        "</div>";
	},
	
	//
	//	_updatingScrollbars:
	//		true when scrollbar position is being updated and we will ignore related events
	//		useful in chrome since it reports events immediately.
	//
	_updatingScrollbars: false,

	getHudSize: function() {
		var hsz = this.getClientSize();
//		var sbw = 16;//this.getScrollbarWidth();
//		if(!this._hScrollbarVisible) {
//			hsz.height+=sbw;
//		}
//		if(!this._vScrollbarVisible) {
//			hsz.width+=sbw;
//		}
		return hsz;
	},
	
	updateScrollbarVisibilityFlags: function() {
  	if(this._hScrollbarVisible != (this._viewRect.width<this._contentRect.width)) {
  		this._hScrollbarVisible = (this._viewRect.width<this._contentRect.width);
  		this._needsHudRelayout = true;
  	}
  	if(this._vScrollbarVisible != (this._viewRect.height<this._contentRect.height)) {
  		this._vScrollbarVisible = (this._viewRect.height<this._contentRect.height);
  		this._needsHudRelayout = true;
  	}
	},

	_onContentBoundingBoxChanged: function(){
		//
		//	summary:
		//		The function is executed when the content of the content has changed.
		//
		//	returns:
		//		True if the change affected content bounding box.
		//
		//	tags:
		//		private
		//
		this.inherited(arguments)
	},

	initialize: function( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		//
		//	summary:
		//		Initializes a Viewport for collaborating with the diagram.
		//
		//	diagram: ibm_ilog.diagram.widget.Diagram
		//		The diagram
		//
//		this._scrollbars = diagram.scrollbarsNode;		
//		this._scrollbarsContent = diagram.scrollbarsContentNode;
		
		this.inherited(arguments);

		this._surface.whenLoaded(this, function () {
			new ScrollbarHudComponent(this,true);
			new ScrollbarHudComponent(this,false);
		});

		//this._connect("onscroll",this._scrollbars,"onscroll","_onScroll");
		
	},

//	_onScroll: function(e) {
//		if(!this._updatingScrollbars) {
//			var p = Point(this._scrollbars.scrollLeft,this._scrollbars.scrollTop);
//			
//			// The following condition traps bouncing events generated by 
//			// the receiver updating the scrollbar programatically
//			
//			if(!g.equalPoint(this._scrollCache,p)) {
//				var q = this._pointScrollToModel(p);
//				this.setViewRect(q,{instant:true});
//			}
//		}
//	},	

	setClientSize: function( /*Size*/ sz ){
		//
		//	summary:
		//		Sets the client Size
		//
		this.setSize({width:sz.width,height:sz.height});
	},
	
	setSize: function( /*Size*/ sz ) {
		//
		//	summary:
		//		Sets the Viewport Size
		//
		var style = {style:{width:sz.width+"px",height:sz.height+"px"}};

		doma.set(this._diagram.domNode,style);
		//dojo.attr(this._scrollbars,style);
		this._updateClientSize(Size(sz.width,sz.height));
		this._updateSurface();
	},
	

	_updateClientSize: function( /*Size*/ sz ) {
		//
		//	summary:
		//		Updates viewportRect from the div attributes
		//
		//	tags:
		//		private
		//
		var vpH = sz.height ;//|| this._scrollbars.clientHeight;
		var vpW = sz.width ;//|| this._scrollbars.clientWidth;
		
		var newClientSize = { width:vpW, height:vpH };


		if(!g.equalRect(this._clientSize,newClientSize)) {

			this._clientSize = newClientSize;
			
			this._computeContentRect();

			switch(this._resizeMode) {
			case 1: // keep zoom level
				this.setViewRect({
					x:this._viewRect.x,
					y:this._viewRect.y,
					width:(newClientSize.width/this.getZoom())},{instant:true});
				break;
			case 2:
				this._onViewRectConstraintsChanged();
				break;
			default:
				R.error("InvalidViewportResizeMode",this._resizeMode)
			}
			
			this.onClientSizeChanged();

		}
	},

	_pointContentToScroll: function(p) {
		//
		//	summary:
		//		Transforms a point from content coordinates to UI coordinates.
		//
		return g.subPoint(this._viewportMatrix.transformPoint(p),this._transformedContentRect);
	},
	
	_pointScrollToModel: function(p) {
		//
		//	summary:
		//		Transforms a point from UI coordinates to content coordinates.
		//
		return this._viewportMatrixInverse.transformPoint(g.addPoint(this._transformedContentRect,p));
	},	

	
	onViewRectChanged: function onViewRectChanged(){
		this.updateScrollbarVisibilityFlags();	
		this.inherited(arguments);
	},

	_applyViewRect: function( /*Boolean*/ sizeChanged, /*Boolean*/ originChanged ){
		//
		//	summary:
		//		Updates the surface size and _transformedContentRect based on the _viewRect and
		//		_contentRect. Called after a change in zoom or Graph content.
		//
		//	tags:
		//		private
		//
		
		this.inherited(arguments);
		
		// create transformation, apply if to Graph, and compute new transformedContentRect
		var zt = matrix.scale(this._zoom);
		var tt = matrix.translate(g.negPoint(this._viewRect)); // uses rect origin
		var t  = matrix.multiply(zt,tt);

		this._transformedContentRect = t.transformRectangle(this._contentRect);
		
		var scrollContentSize = g.floorSize(g.growRect(this._transformedContentRect,this.getClientSize()));
		      // this._transformedContentRect;
			 	  // [AV] replace by this go back to tight surface mode

		this._viewportMatrix = t;
		this._viewportMatrixInverse = t.inverse();

		this._setViewportTransform(t);
	},
	
	_getSurfaceSize: function () {
		return this._clientSize;
	}

});

return GfxViewport;

});

},
'ibm_ilog/graphlayout/hierarchical/HierarchicalNodeGroup':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/hierarchical/HierarchicalNodeGroup", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/LayoutUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    LayoutUtil,	// #5
    TranslateUtil	// #6
    ) {

var HierarchicalNodeGroup = declare("ibm_ilog.graphlayout.hierarchical.HierarchicalNodeGroup", null, 
{
  // summary:
  //		The class 'NodeGroup' allows to create a group of nodes
  //		that is subject to a constraint. 
  // description:
  //		The class <code>NodeGroup</code> allows to create a group of nodes
  //		that is subject to a constraint. 
  //		<p>
  //		A node group itself is not a constraint. But some constraints allow
  //		node groups as input rather than single nodes. For instance, it is
  //		possible to force a group of nodes to be placed in a higher level
  //		than another group of nodes. Or it is possible to force all nodes of
  //		the group to be placed no more than a certain number of levels apart.
  //		 
  //		</p>
  
  _owner: null,
  _next: null,
  _recentCopy: null,
  _groupEntryNode: null,
  _groupExitNode: null,
  _validForLayout: null,
  _nodes: null,
  _originalNodes: null,

  constructor : function (a0)
  {
    if(arguments.length == 0)
      this._constructor_overload_HierarchicalNodeGroup_0();
    else if(arguments.length == 1)
      this._constructor_overload_HierarchicalNodeGroup_1(a0);
  },
  
  _constructor_overload_HierarchicalNodeGroup_0 : function()
  {
    this._constructor_overload_HierarchicalNodeGroup_1(null);
  },
  
  _constructor_overload_HierarchicalNodeGroup_1 : function(nodes )
  {
    if (nodes && nodes.declaredClass == HierarchicalNodeGroup.prototype.declaredClass)
      nodes = nodes._nodes;
    this._owner = null;
    this._next = null;
    if (nodes == null) {
      this._nodes = null;
    }
    else {
      this._nodes = nodes.concat([]);
    }
  },
  
  copy : function()
  {
    // summary:
    //		Copies the group.
    // tags:
    //		private
    
    this._recentCopy = new HierarchicalNodeGroup(this);
    return this._recentCopy; // ibm_ilog.graphlayout.hierarchical.HierarchicalNodeGroup
  },

  add : function(node)
  {
    // summary:
    //		Adds a node to the node group. 
    // description:
    //		Adds a node to the node group. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    
    if (this._nodes == null) {
      this._nodes = new Array();
    }
    this._nodes.push(node);
  },

  remove : function(node)
  {
    // summary:
    //		Removes a node from the node group. 
    // description:
    //		Removes a node from the node group. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    
    while (this.contains(node))
    {
      TranslateUtil.removeArrayObject(this._nodes, node);
    }
  },

  size : function()
  {
    // summary:
    //		Returns the number of nodes in the node group.  
    // returns:
    //		The number of nodes in this group.
    
    if (this._nodes == null) {
      return 0; // int
    }
    return this._nodes.length; // int
  },

  elements : function()
  {
    // summary:
    //		Returns the nodes in the node group.  
    // returns:
    //		The nodes in the node group.
    
    if (this._nodes == null) {
      return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
    }
    return  new TranslateUtil.ArrayIterator(this._nodes); // ibm_ilog.graphlayout.IIterator
  },

  contains : function(node)
  {
    // summary:
    //		Returns 'true' if the node belongs to this group.  
    // node: Object
    //		The node instance.
    // returns:
    //		'true' if the node belongs to this group.
    
    if (this._nodes == null) {
      return false; // Boolean
    }
    return (arr.indexOf(this._nodes, node)>=0); // Boolean
  },

  isValidForLayout : function()
  {
    // summary:
    //		Returns 'true' if the constraint is valid for layout.
    // tags:
    //		private
    
    return this._validForLayout; // Boolean
  },

  beforeLayout : function(graph)
  {
    // summary:
    //		Called before the layout really starts.
    // tags:
    //		private
    
    if (this._nodes == null)
    {
      this._validForLayout = false;
      return;
    }

    this._originalNodes = this._nodes;

    var allValid = true;
    var e =  new TranslateUtil.ArrayIterator(this._nodes);
    var node;

    while (e.hasNext() && allValid)
    {
      node = e.next();
      if (graph.getNode(node) == null) {
        allValid = false;
      }
    }

    if (!allValid)
    {
      var v = new Array();
      e =  new TranslateUtil.ArrayIterator(this._nodes);

      while (e.hasNext())
      {
        node = e.next();
        if (graph.getNode(node) != null) {
          v.push(node);
        }
      }

      this._nodes = v;
    }

    this._validForLayout = this._nodes.length > 0;
  },

  afterLayout : function(graph)
  {
    // summary:
    //		Called after the layout is finished.
    // tags:
    //		private
    
    this._nodes = this._originalNodes;
    this._originalNodes = null;
  },

  getNext : function()
  {
    // summary:
    //		Returns the next constraint.
    // tags:
    //		private
    
    return this._next; // ibm_ilog.graphlayout.hierarchical.HierarchicalNodeGroup
  },

  setNext : function(next)
  {
    // summary:
    //		Set the next constraint.
    // tags:
    //		private
    
    this._next = next;
  },

  getOwner : function()
  {
    // summary:
    //		Returns the owner of the constraint.
    // tags:
    //		private
    
    return this._owner; // ibm_ilog.graphlayout.hierarchical.ConstraintManager
  },

  setOwner : function(manager)
  {
    // summary:
    //		Sets the owner of the constraint.
    // tags:
    //		private
    
    this._owner = manager;
  },

  getRecentCopy : function()
  {
    // summary:
    //		Returns the recent copy of the node group. 
    // description:
    //		Returns the recent copy of the node group. 
    //		Needed for the copy mechanism of the constraint manager.
    // tags:
    //		private
    
    return this._recentCopy; // ibm_ilog.graphlayout.hierarchical.HierarchicalNodeGroup
  },

  initializeBeforeLeveling : function()
  {
    // summary:
    //		Initialize the group entry and exit node before performing a
    //		leveling.
    // tags:
    //		private
    
    this._groupEntryNode = this._groupExitNode = null;
  },

  getEntryNode : function()
  {
    // summary:
    //		Returns the group entry node in the leveling graph.
    // tags:
    //		private
    
    return this._groupEntryNode; // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  },

  setEntryNode : function(node)
  {
    // summary:
    //		Sets the group entry node in the leveling graph.
    // tags:
    //		private
    
    this._groupEntryNode = node;
  },

  getExitNode : function()
  {
    // summary:
    //		Returns the group exit node in the leveling graph.
    // tags:
    //		private
    
    return this._groupExitNode; // ibm_ilog.graphlayout.hierarchical.leveling.HLVNode
  },

  setExitNode : function(node)
  {
    // summary:
    //		Sets the group exit node in the leveling graph.
    // tags:
    //		private
    
    this._groupExitNode = node;
  },

  validate : function(model)
  {
    // summary:
    //		Validates the group with respect to the graph model. 
    // description:
    //		Validates the group with respect to the graph model. 
    //		This removes all objects that are not nodes in the graph model. It
    //		also eliminates duplicates. It returns <code>true</code> if the
    //		group is afterwards still valid.
    // tags:
    //		private
    
    if (this._nodes == null) {
      return false; // Boolean
    }
    var v = new Array();

    var e = this.elements();
    var node;

    while (e.hasNext())
    {
      node = e.next();
      if (model.isNode(node) && !(arr.indexOf(v, node)>=0)) {
        v.push(node);
      }
    }

    this._nodes = v;
    return (this._nodes.length > 0); // Boolean
  }

}
);

lang.mixin(HierarchicalNodeGroup,
{
  serialVersionUID : null,
  _fields_init : function(c)
  {
    c = c || HierarchicalNodeGroup;
    
    c.serialVersionUID = 7289437835687213568;
  }
  
}
);

(function(){ HierarchicalNodeGroup._fields_init(); })();

return HierarchicalNodeGroup;
});

},
'ibm_ilog/diagram/_base/diagext':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/_base/diagext", ["dojo/_base/lang", "dojo/has", "dojox/gfx", "dojox/gfx/shape", "dojox/gfx/matrix", "../util/Geometry"],
function(lang, has, gfx, shape, matrix, geom){

	// TODO:
	// For now we must create a global object since there are still references to ibm_ilog.diagram._base.gfxext
	// in the rest of the code, but when AMD migration is finished all global references should be replaced
	// by module references and this will be replaced by a simple object creation:
	// var de = {};
	var de = lang.getObject("ibm_ilog.diagram._base.diagext", true);
	
    //
	// Add transform functions to all shapes:
	//
	lang.extend(shape.Shape, {
		// summary:
		//		Provides additional methods on GFX shapes to compute transforms.
		// tags:
		//		private
		
		getTransformToContainer: function(/*dojox.gfx.Group*/container){
			// summary:
			//        Gets the transform from the parent coordinate system
			//        to the specified container coordinate system.
			// container: dojox.gfx.Group:
			//        the target container.
			
			if (container === null) {
				//TODO: ErrorReporter
				throw {
					'name': 'ArgumentNullException',
					message: "Null parameter: container"
				};
			}
			var m = matrix;
			if (container === this) {
				return m.identity;
			}
			var t1 = m.identity;
			// does not take into account shape transform
			var p = this.getParent();
			// TODO: no need to go up till the surface. Should stop to 
			// 1st common ancestor. check perf gain ?
			if (p !== null) {
				t1 = p._getRealMatrix() || m.identity;
			}
			var t2 = container._getRealMatrix() || m.identity;
			var t = m.multiply(t2.inverse(), t1);
			return t;
		},
		
		getShapeToContainerTransform: function(container){
			// summary:
			//        Gets the transform from this shape coordinate system
			//        to the specified container coordinate system.
			// description:
			//        This method takes into account the shape transform.
			// container: dojox.gfx.Group:
			//        the target container.
			if (container === null) {
				//TODO: ErrorReporter
				throw {
					'name': 'ArgumentNullException',
					message: "Null parameter: container"
				};
			}
			var m = matrix;
			if (container === this) {
				return m.identity;
			}
			// TODO: no need to go up till the surface. Should stop to 
			// 1st common ancestor. check perf gain ? 
			// t1: transform from surface -> shape
			var t1 = this._getRealMatrix() || m.identity;
			// t2: transform from surface -> container
			var t2 = container._getRealMatrix() || m.identity;
			return m.multiply(t2.inverse(), t1);
		}
	});
	
	//
	// BoundingBox fix
	//
	
	var eq = geom.equalRect;
	
	de.BoundingBoxFix = {
		// summary:
		//		Fixes/provides additional methods on GFX groups for bounding box computations.
		// tags:
		//		private
		
		_computeBoundingBox: function(){
			var bb = null;
			var cs = this.children;
			var ncs = this.children.length;
			var c;
			for (var i = 0; i < ncs; ++i) {
				var c = cs[i];
				var cbb = c.getBoundingBox();
				if (!cbb) {
					continue;
				}
				var ct = c.getTransform();
				if (ct && !ct.isIdentity()) {
					cbb = ct.transformRectangle(cbb);
				}
				bb = bb ? geom.addRect(bb, cbb) : cbb;
			}
			
			this._bbCache = bb;
			if (bb) {
				var t = this.getTransform() || matrix.identity;
				if (t) 
					bb = t.transformRectangle(bb);
			}
			this._lbbCache = bb;
		},
		
		_prepareBoundingBoxCache: function(){
			var oldbb = this._bbCache;
			var oldlbb = this._lbbCache;
			
			if (!oldbb || !this._bbCacheValid) {
				this._computeBoundingBox();
				this._bbCacheValid = true;
			}
		},
		
		getBoundingBox: function(){
			// summary:
			//        Gets the bounding box of this dojox.gfx.Group. Returns null if the group is empty.
			// description:
			//        This method returns the global children bounding box without
			//        taken into account the transform of this Group.
			this._prepareBoundingBoxCache();
			
			return this._bbCache;
		},
		
		_onChanged: function(source, reason){
			this._bbCacheValid = false;
			// 	[av]
			// 	this method became an "inlined inherited" at:
			//	- Node.js
			//	- Graph.js
		},
		
		getLocalBoundingBox: function(){
			// summary:
			//        Gets the bounding box of this dojox.gfx.Group in the local coordinate system.
			// description:
			//        This method returns the global children bounding box 
			//        taken into account the transform of this Group.        
			this._prepareBoundingBoxCache();
			
			return this._lbbCache;
		}
	};
	
	lang.extend(gfx.Group, de.BoundingBoxFix);
	
	return de;
});

},
'ibm_ilog/diagram/gfxlayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout", [
"dojo/has!config-useGfxLayout?./gfxlayout/_base:",
"dojo/has!config-useGfxLayout?./gfxlayout/Layout:",
"dojo/has!config-useGfxLayout?./gfxlayout/Engine:",
"dojo/has!config-useGfxLayout?./gfxlayout/GridLayout:",
"dojo/has!config-useGfxLayout?./gfxlayout/StackLayout:",
"dojo/has!config-useGfxLayout?./gfxlayout/FillLayout:"
],
function(gfxlayout, Layout, Engine, GridLayout, StakcLayout, FillLayout){
	return gfxlayout;
});

},
'ibm_ilog/diagram/layer_base_dojo_requires':function(){
// wrapped by build app
define(["dijit","dojo","dojox"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
dojo.provide('ibm_ilog.diagram.layer_base_dojo_requires');

dojo["require"]('dijit._base');
dojo["require"]('dijit.dijit');
dojo["require"]('dijit.form.Textarea');
dojo["require"]('dijit.Tooltip');
dojo["require"]('dojo.number');
dojo["require"]('dojo.parser');
dojo["require"]('dojo._base.array');
dojo["require"]('dojo._base.connect');
dojo["require"]('dojo._base.declare');
dojo["require"]('dojo._base.json');
dojo["require"]('dojo._base.lang');
dojo["require"]('dojo.data.api.Identity');
dojo["require"]('dojo.data.api.Notification');
dojo["require"]('dojo.data.api.Read');
dojo["require"]('dojo.data.api.Write');
dojo["require"]('dojo.fx');
dojo["require"]('dojox.collections.ArrayList');
dojo["require"]('dojox.collections._base');
dojo["require"]('dojox.collections.Dictionary');
dojo["require"]('dojox.collections.Set');
dojo["require"]('dojox.collections.Stack');
dojo["require"]('dojox.dtl._base');
dojo["require"]('dojox.dtl.Context');
dojo["require"]('dojox.gfx');
dojo["require"]('dojox.gfx.fx');
dojo["require"]('dojox.gfx.matrix');
dojo["require"]('dojox.gfx.move');
dojo["require"]('dojox.gfx.shape');
dojo["require"]('dojox.gfx.utils');
dojo["require"]('dojox.lang.functional.array');
dojo["require"]('dojox.lang.functional.fold');

});

},
'ibm_ilog/diagram/overview/OverviewGfxRenderer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewGfxRenderer", [
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/array",
"dojox/gfx",
"dojox/gfx/utils",
"./OverviewRenderer"
],
function(
declare,
lang,
array,
g,
gu,
OverviewRenderer
){

/*=====
var OverviewRenderer = ibm_ilog.diagram.overview.OverviewRenderer;
=====*/

var OverviewGfxRenderer =
declare('ibm_ilog.diagram.overview.OverviewGfxRenderer', OverviewRenderer, {
	
	// static
	_transformSerialize: function transformSerialize(
			/* dojox.gfx.Surface || dojox.gfx.Shape */ object,
			/* Function | String */                    transform
		)
	{
		var t = {}, isSurface = object instanceof g.Surface;

		var transformChildren = function() {
			var mapf = function(x){ return transformSerialize(x,transform); };
			var notz = function(x){ return x!=null; };
			return array.filter(array.map(object.children,mapf),notz);
		};
		
		if(isSurface) {
			return transformChildren();	// Array
		} else {
			return transform(object,transformChildren);
		}

	},

	
	// static
	_shapeCopy: function(copy,shape,getChildren) {
		var v;
		if(!(shape instanceof g.Surface) &&
	       !(shape instanceof g.Group)){		
			copy.shape = shape.getShape();
		} else {
			copy.children = getChildren();	
		}
		
		if(shape.getTransform){
			v = shape.getTransform();
			if(v){ copy.transform = v; }
		}
		if(shape.getStroke){
			v = shape.getStroke();
			if(v){ copy.stroke = v; }
		}
		if(shape.getFill){
			v = shape.getFill();
			if(v){ copy.fill = v; }
		}
		if(shape.getFont){
			v = shape.getFont();
			if(v){ copy.font = v; }
		}
	},


	// static
	stdShapeFilter: function(filter) {
		return function (shape,getChildren) {
				if(filter(shape)) {
					var copy = {};
					this._shapeCopy(copy,shape,getChildren);
					return copy;
				} else {
					return null;
				}
			};
	},
	
	// static
	overviewFilter0: function( object ) {
		return true;
	},

	// static
	overviewFilter1: function( object ) {
		var t = {}, v, isSurface = object instanceof g.Surface;
		if(isSurface || object instanceof g.Group){
			return true;
		} else {
			return object.getShape().type=="rect";
		}
	},
	
	// static
	overviewFilter2: function( object ) {
		var t = {}, v, isSurface = object instanceof g.Surface;
		if(isSurface || object instanceof g.Group){
			return true;
		} else {
			return object.getShape().type=="polyline";
		}
	},

	// static
	overviewTransform3: function(shape,getChildren) {
		var isSurface = shape instanceof g.Surface;
		var isGroup   = shape instanceof g.Group;
		if(isSurface || isGroup || shape.getShape().type=="rect" ){
			var copy = {};
			this._shapeCopy(copy,shape,getChildren);
			if(shape.getShape().type=="rect") {
				copy.fill = {r:0,g:0,b:0,a:1}; // build fill by hand because destination objects are empty after "serialization"
			}
			return copy;
		} else {
			return null;
		}
	},
	
	constructor: function( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		this._diagram = diagram;
	},

	render: function( /*dojox.gfx.Group*/ output ) {

		var nodefilter = nodefilter || lang.hitch(this,this.overviewTransform3);
		
		var graphJson = this._transformSerialize(this._diagram.getGraph(),nodefilter);
		
		gu.deserialize(output, graphJson);
		
	}
});

return OverviewGfxRenderer;

});

},
'ibm_ilog/diagram/Link':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/Link", ["./main",
"dojo/_base/declare", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/json",
"dojox/gfx", "dojox/gfx/matrix",
"./GraphElement", "./Port", "./templating", "./templates/templates",
"./util/Geometry", "./util/Batch", "./util/ErrorReporter"
],
function(iid, declare, lang, array, json, gfx, gfxm, GraphElement, port, templating, templates, Geometry, Batch, ErrorReporter){

var LinkShapeType = {
    Straight: 1,
    Orthogonal: 2,
    OrthogonalEditable: 3,
    Oblique: 4,
    Free: 5
};
iid.LinkShapeType = LinkShapeType;

/*=====
var GraphElement = ibm_ilog.diagram.GraphElement;
=====*/

var Link = 
declare("ibm_ilog.diagram.Link", [GraphElement], {
    // summary:
    //		A Link is a graphic object that connects two Node objects.
    // description:
    //		The link is a templatable object that connects two nodes.
    //		A link is created by calling graph.createLink(template), where 'graph' is an ibm_ilog.diagram.Graph instance,
    //		and 'template' is a GFX definition of the shapes that make up the link.
    //		The template must contain at least a GFX shape that will represent the polyline shape of the link.
    //		This shape must have a "dojoAttachPoint: '_path'" property.
    //		The template can contain one or two shapes representing the start and/or end arrows
    //		of the link, with "dojoAttachPoint: '_startArrow'" and "dojoAttachPoint: '_endArrow'" properties.
	//		The template can also contain decorations, that is, shapes that will placed placed automatically
	//		along the link's path. Decorations are identified by a isDecoration property which must be set to true.
	//		Additional properties can be specified to define how a decoration is placed and aligned on the path.
	//		See the getDecorations() method for more details.
    
	// Allows to test the class faster that testing declaredClass
	_isIBMDiagramLink: true,
	
	//
	//	TID: String
	//		Graph element Type ID, used to distinguish basic graph 
  //		element types:
  //	
  //			N = Node
  //			S = Subgraph
  //			L = Link
  //	
	//		Usually used as a very fast index into maps based on 
  //		ibm_diagram graph element types.
	//
  TID: 'L',
	
    // properties
    _startPort: null,
    _endPort: null,
    _startPoint: {
        x: 0,
        y: 0
    },
    _endPoint: {
        x: 100,
        y: 0
    },
    _intermediatePoints: null,
    _shapeType: LinkShapeType.Free,
    _nodeDistance: 20,
    _hitTolerance: 0,
    
    // template parts
    _path: null,
    _startArrow: null,
    _endArrow: null,
    
    // internal state	
    _pathPoints: null,
    _pathPointsValid: false,
    _selectionPath: null,
	
	_isIntergraphLinkAtStart: false,
	_isIntergraphLinkAtEnd: false,

	_curveTension: 0,
	
    constructor: function() {
        // summary: Constructor.

        this._startPoint = {
            x: 0,
            y: 0
        };
        this._endPoint = {
            x: 100,
            y: 0
        };
    },
    
    //---------- Disposal ----------
    
    _dispose: function() {
        // summary:
        //		Dispose this Link.
        // description:
        //		This function should be called when the link is completely deleted, that is, when it is removed
        //		from its parent graph and it will never be used again.
        
        this.setStartPort(null);
        this.setEndPort(null);
    },
    
    invalidate:function(){},
    applyLayout:function(){},
    
    //---------- Start/end ports ----------
    
    getStartPort: function() {
        // summary:
        //		Gets the start port of the link.
        // returns:
        //		The PortBase object that defines the start point of the link.
        return this._startPort;
    },
    
    setStartPort: function(port) {
        // summary:
        //		Sets the start port of the link.
        // port: ibm_ilog.diagram.PortBase:
        //		The PortBase object that defines the start point of the link.
        if (this._startPort != port) {
            if (this._startPort)
				this._startPort._removeLink(this);
			this._checkIntergraphLink(this._startPort, port, true);
            this._startPort = port;
            if (this._startPort)
				this._startPort._addLink(this);
            this.invalidateLinkShape();
        }
        return this;
    },
    
    getEndPort: function() {
        // summary:
        //		Gets the end port of the link.
        // returns:
        //		The PortBase object that defines the end point of the link.
        return this._endPort;
    },
    
    setEndPort: function(port) {
        // summary:
        //		Sets the end port of the link.
        // port: ibm_ilog.diagram.PortBase:
        //		The PortBase object that defines the end point of the link.
        if (this._endPort != port) {
			if (this._endPort) 
				this._endPort._removeLink(this);
			this._checkIntergraphLink(this._endPort, port, false);
			this._endPort = port;
			if (this._endPort)
				this._endPort._addLink(this);
			this.invalidateLinkShape();
		}
        return this;
    },
	
    _forceCheckIntergraphLink: function(){
		this._isIntergraphLinkAtStart = this._forceCheckIntergraphLink1(this.getStartPort(),this._isIntergraphLinkAtStart);
		this._isIntergraphLinkAtEnd = this._forceCheckIntergraphLink1(this.getEndPort(),this._isIntergraphLinkAtEnd);
    },
    
    _forceCheckIntergraphLink1: function(port,isIntergraphLink){
    	if(port){
    		if(isIntergraphLink){
    			this._checkIntergraphLink1(port, true);
    		}
    		return this._checkIntergraphLink1(port, false);
		}
    	return isIntergraphLink;
    },
    
	_checkIntergraphLink: function(oldPort, newPort, start){
		// check cases when we know this is not necessary:
		if(oldPort && newPort && oldPort._owner == newPort._owner)
			return;
		// remove link from _integraphLinks arrays for old port:
		var isIntergraphLink = start ? this._isIntergraphLinkAtStart : this._isIntergraphLinkAtEnd;
		if (oldPort && isIntergraphLink)
			this._checkIntergraphLink1(oldPort, true);
		// add link from _integraphLinks arrays for new port:
		if (newPort) {
			isIntergraphLink = this._checkIntergraphLink1(newPort, false);
			if(start)
				this._isIntergraphLinkAtStart = isIntergraphLink;
			else
				this._isIntergraphLinkAtEnd = isIntergraphLink;
		}
	},
		
	_checkIntergraphLink1: function(port, cleanup) {

		// Is this an intergraph link?
		// (definition = link parent is an ancestor or port owner parent)
		var node = port.getOwner();
		var linkParent = this.getParent();
		if(node && linkParent){
			var nodeParent = node.getParent();
			if(nodeParent && nodeParent != linkParent){
				for(var p = nodeParent.getParent(); p && p.getParent; p = p.getParent()){
					if(p == linkParent){
						// OK, yes, this is an intergraph link.
						// record the link in the _intergraphLinks arrays of all subgraphs
						// between the node parent and the link parent
						for (var sub = nodeParent.getParent(); sub != linkParent; sub = sub.getParent()) {
							if (sub._isIBMDiagramSubgraph) {
								if (cleanup) {
									if (sub._intergraphLinks) {
										var index = array.indexOf(sub._intergraphLinks, this);
										if (index >= 0) {
											if(sub._intergraphLinks.length == 1)
												sub._intergraphLinks = undefined;
											else
												sub._intergraphLinks.splice(index, 1);
										}
									}
								}
								else {
									if (!sub._intergraphLinks) 
										sub._intergraphLinks = [this];
									else 
										if (array.indexOf(sub._intergraphLinks, this) < 0) 
											sub._intergraphLinks.push(this);
								}
							}
						}
						return true;
					}
				}
			}
		}
		return false;
	},
	
	_setParent: function(parent, matrix)
	{
		GraphElement.prototype._setParent.call(this, parent, matrix);
		this._forceCheckIntergraphLink();
	},
    
    //---------- Start/end nodes ----------
    
    getStartNode: function() {
        // summary:
        //		Gets the start node of the link.
        // returns:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a start port has been specified, this function returns the owner of the start port,
        //		otherwise it returns null.
        
        if (this._startPort) 
            return this._startPort.getOwner();
        else 
            return null;
    },
    
    setStartNode: function(/*ibm_ilog.diagram.Node*/node) {
        // summary:
        //		Sets the start node of the link.
        // node:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a start port has been specified, this function returns the owner of the start port,
        //		otherwise it returns null.
        
        if (node) {
            if (!this._startPort || this._startPort.getOwner() != node) {
                this.setStartPort(this._getOrCreateDefaultPort(node));
            }
        } else {
            this.setStartPort(null);
        }
        return this;
    },
    
    getEndNode: function() {
        // summary:
        //		Gets the end node of the link.
        // returns:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a end port has been specified, this function returns the owner of the end port,
        //		otherwise it returns null.
        
        if (this._endPort) 
            return this._endPort.getOwner();
        else 
            return null;
    },
    
    setEndNode: function(/*ibm_ilog.diagram.Node*/node) {
        // summary:
        //		Sets the end node of the link.
        // node:
        //		A ibm_ilog.diagram.Node object.
        // description:
        //		If a end port has been specified, this function returns the owner of the end port,
        //		otherwise it returns null.
        
        if (node) {
            if (!this._endPort || this._endPort.getOwner() != node) {
                this.setEndPort(this._getOrCreateDefaultPort(node));
            }
        } else {
            this.setEndPort(null);
        }
        return this;
    },
    
    _getOrCreateDefaultPort: function(/*ibm_ilog.diagram.Node*/node) {
        // tag: private
        // summary:
        //		Looks for an existing default port on the specified node
		///		(the default port is identified by the _isDefaultPort flag),
        //		if none exists create and add one.
        // node: ibm_ilog.diagram.Node:
        //		The node in which the default port should be looked up or created.
        
        var port;
        var ports = node.getPorts();
        for (var i = 0; i < ports.length; i++) {
            var port = ports[i];
            if (port._isDefaultPort) {
                return port;
            }
        }
        port = this._createDefaultPort(node);
		port._isDefaultPort = true;
        node.addPort(port);
        return port;
    },
	
	_createDefaultPort: function(/*ibm_ilog.diagram.Node*/node) {
        // tag: private
        // summary:
        //		Called by _getOrCreateDefaultPort to create the default port if none exists.
		//		By default, creates a new AutomaticPort.
        // node: ibm_ilog.diagram.Node:
        //		The node in which the default port should be created.
		return new port.AutomaticPort();
	},
    
    //---------- Start/end points ----------
    
    getFallbackStartPoint: function() {
        // summary:
        //		Gets the start point of the link.
        // returns:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The start point is used only when no start port is specified.
        return this._startPoint;
    },
    
    setFallbackStartPoint: function(point) {
        // summary:
        //		Sets the start point of the link.
        // point:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The start point is used only when no start port is specified.
        
        if (typeof(point) != 'object' || !('x' in point) || !('y' in point)) 
            ErrorReporter.error("UnexpectedArgument", this, "setFallbackStartPoint", "point", point);
        
        if (point.x != this._startPoint.x ||
        point.y != this._startPoint.y) {
            this._startPoint.x = point.x;
            this._startPoint.y = point.y;
            if (!this._startPort) 
                this.invalidateLinkShape();
            
        }
        return this;
    },
    
    getFallbackEndPoint: function() {
        // summary:
        //		Gets the end point of the link.
        // returns:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The end point is used only when no end port is specified.
        return this._endPoint;
    },
    
    setFallbackEndPoint: function(point) {
        // summary:
        //		Sets the end point of the link.
        // point:
        //		An object with 'x' and 'y' properties.
        // description:
        //		The end point is used only when no end port is specified.
        
        if (typeof(point) != 'object' || !('x' in point) || !('y' in point)) 
            ErrorReporter.error("UnexpectedArgument", this, "setFallbackEndPoint", "point", point);
        
        if (point.x != this._endPoint.x ||
        point.y != this._endPoint.y) {
            this._endPoint.x = point.x;
            this._endPoint.y = point.y;
            if (!this._endPort) 
                this.invalidateLinkShape();
        }
        return this;
    },
    
    //---------- Intermediate points ----------
    
    getIntermediatePoints: function() {
        // summary:
        //		Gets the intermediate points of the link.
        // returns:
        //		An array of objects with 'x' and 'y' properties.
        // description:
        //		The intermediate points define the additional points of the link (i.e., the points between the start and end points,
        //		which are always determined by the setStartPort()/setEndPort() or setFallbackStartPoint/setFallbackEndPoint() functions)
        //		when the link shape type is ibm_ilog.diagram.LinkShapeType.Free or ibm_ilog.diagram.LinkShapeType.OrthogonalEditable.
        //		For other shape types, the intermediate points are ignored since the link path is computed automatically.
        
        if (this._intermediatePoints !== null) {
            return lang.clone(this._intermediatePoints);
        } else {
            return [];
        }
    },
    
    setIntermediatePoints: function(points) {
        // summary:
        //		Sets the intermediate points of the link.
        // points:
        //		An array of objects with 'x' and 'y' properties
        //		(can be null to remove all intermediate points).
        // description:
        //		The intermediate points define the additional points of the link (i.e., the points between the start and end points,
        //		which are always determined by the setStartPort()/setEndPort() or setFallbackStartPoint/setFallbackEndPoint() functions)
        //		when the link shape type is ibm_ilog.diagram.LinkShapeType.Free or ibm_ilog.diagram.LinkShapeType.OrthogonalEditable.
        //		For other shape types, the intermediate points are ignored since the link path is computed automatically.
        
        if (points !== null) {
            this._intermediatePoints = lang.clone(points);
        } else {
            this._intermediatePoints = null;
        }
        if (this._shapeType == LinkShapeType.Free ||
        		this._shapeType == LinkShapeType.OrthogonalEditable) 
            this.invalidateLinkShape(); // in the other shapeTypes the interm points are not used
        return this;            
    },
    
    //---------- Shape type ----------
    
    getShapeType: function() {
        // summary:
        //		Gets the shape type of this link.
        // returns:
        //		One of the values defined in the ibm_ilog.diagram.LinkShapeType enumeration.
        // description:
        //		The shape type determines if and how the link automatically computes its shape
        //		based on its start and end port or points, and possibly its intermediate points.
        
        return this._shapeType;
    },
    
    setShapeType: function(/*int*/shapeType) {
        // summary:
        //		Sets the shape type of this link.
        // shapeType: int:
        //		One of the values defined in the ibm_ilog.diagram.LinkShapeType enumeration.
        // description:
        //		The shape type determines if and how the link automatically computes its shape
        //		based on its start and end port or points, and possibly its intermediate points.
        
        if (shapeType != this._shapeType) {
            this._shapeType = shapeType;
            this.invalidateLinkShape();
        }
        return this;
    },
    
    //---------- Node distance ----------
    
    getNodeDistance: function() {
        // summary:
        //		Gets the minimum distance between this link and its start or end nodes when the link is orthogonal.
        // returns:
        //		A distance in pixels.
        // description:
        //		This property is used when the link shape type is Orthogonal, OrthogonalEditable or Oblique,
        //		and determines the minimum distance kept between the link and its start or end nodes.
        //		The default distance is 20.
        
        return this._nodeDistance;
    },
    
    setNodeDistance: function(/*number*/distance) {
        // summary:
        //		Sets the minimum distance between this link and its start or end nodes when the link is orthogonal.
        // distance: number:
        //		A distance in pixels.
        // description:
        //		This property is used when the link shape type is Orthogonal, OrthogonalEditable or Oblique,
        //		and determines the minimum distance kept between the link and its start or end nodes.
        
        if (distance != this._nodeDistance) {
            this._nodeDistance = distance;
            if (this._shapeType == LinkShapeType.Orthogonal ||
            this._shapeType == LinkShapeType.OrthogonalEditable ||
            this._shapeType == LinkShapeType.Oblique) 
                this.invalidateLinkShape();
        }
        return this;
    },
    
    //---------- Curve tension (splines) ----------
	
	setCurveTension: function(/*number*/t) {
		// summary:
		//		Sets the curve tension to draw the link as a spline.
		// t: number:
		//		The curve tension, between 0 and 1.
		// description:
		//		This properties allows to draw curved links. By default, the curve tension is 0, which draws
		//		a plain polyline (not curved). If the curve tension is set to a value greater than 0, the
		//		link is drawn as a smooth spline that goes through the start point, then through the middle
		//		of each link segment (as computed according to the link shape type), and then to the end point.
		//		The curve tension determines the shape of the curve: small values will produce sharper bends,
		//		larger values will produce a smoother curve.
		//		The curve tension must be greater than 0 and less than 1
		//		(0.5 usually provides nicest results).
		//
		//		NOTE: If the _path part of the link template was a GFX Polyline shape, setting the curve tension to
		//			  a value greater than 0 automatically replaces it by a GFX Path shape. 
		
		if(t < 0) t = 0;
		if(t > 1) t = 1;
		if(t != this._curveTension){
			this._curveTension = t;
			// if the path is a polyline, replace it by a GFX Path.
			if (t > 0 && !(this._path instanceof gfx.Path)) {
				var stroke, selectedStyle, parent;
				if (this._path) {
					stroke = this._path.getStroke();
					selectedStyle = this._path.selectedStyle;
					parent = this._path.getParent();
					// Added to solve problem when link is selected
					standardStyle = this._path.standardStyle;
					if(this._styledChildren)
						this._styledChildren.remove(this._path);
					this._path.removeShape();
				}
				this._path = (parent || this).createPath();
				if(stroke)
					this._path.setStroke(stroke);
				if(selectedStyle)
					this._path.selectedStyle = selectedStyle;
				// Added to solve problem when link is selected
				if(standardStyle)
					this._path.standardStyle = standardStyle;
				if(this._styledChildren && this._path.standardStyle)
					this._styledChildren.add(this._path);
			}
			this.invalidateLinkShape();
		}
        return this;
	},
	
	getCurveTension: function() {
		// summary:
		//		Gets the curve tension.
		// returns: number:
		//		The curve tension.
		// description:
		//		See setCurveTension.
		
		return this._curveTension;
	},
    //---------- Path ----------

    layoutShape: function(target) {
    	//
    	//	_DiagramTemplated API:
    	//	
    	//		for externally instantiated templates
    	//
    	this._updatePath(target);
    },
    
    _updatePath: function(target) {
    	
    	var target = target || this;
    
        var points = this.applyPathToArrows(target);
    	this.applyPath(target._path,points,target._curveTension);
		// decorations:
		
		this._placeDecorations(target);
        this._updateSelectionPath(points,target,target._curveTension);
    },
    
    //-------------------------------------------------------	
    applyPathToArrows: function(target){
    	
    	var points = this._pathPoints;
        
        // arrows:
        if (points && points.length >= 2) {
            if ((target._startArrow && target._startArrow.getParent()) ||
            (target._endArrow && target._endArrow.getParent())) {
                points = lang.clone(points); // end points will be modified
            }
            if (target._startArrow && target._startArrow.getParent()) {
            	this._updateArrow(target._startArrow, points[0], points[1],target);
            }
            if (target._endArrow && target._endArrow.getParent()) {
            	this._updateArrow(target._endArrow, points[points.length - 1], points[points.length - 2],target);
            }
        }
        return points;
    },
    
    applyPath: function(path,points,curveTension){
    	
        if (path instanceof gfx.Polyline) {
            path.setShape(points);
			path.bbox = null; // patch for GFX bug...
        } else {
            path.setShape("");
            if (points && points.length > 0) {
                path.moveTo(points[0].x, points[0].y);
				if (curveTension > 0 && points.length > 2) {
					var x = points[0].x;
					var y = points[0].y;
					var t = 1 - curveTension/2;
					for (var i = 0; i < points.length - 2; i++) {
						// first control point:
						var p1 = points[i + 1];
						var x1 = x + (p1.x - x) * t;
						var y1 = y + (p1.y - y) * t;
						// Compute next curve point:
						var p2 = points[i + 2];
						if (i < points.length - 3) {
							x = p1.x + (p2.x - p1.x) / 2;
							y = p1.y + (p2.y - p1.y) / 2;
						}
						else {
							x = p2.x;
							y = p2.y;
						}
						// second control point:
						var x2 = x + (p1.x - x) * t;
						var y2 = y + (p1.y - y) * t;
						path.curveTo(x1, y1, x2, y2, x, y);
					}
				}
				else {
					// polyline:
					for (var i = 1; i < points.length; i++) {
						path.lineTo(points[i].x, points[i].y);
					}
				}
            }
        }
	},
    //---------- Mouse tolerance ----------
    
    _updateSelectionPath: function(points,target,curveTension) {
    	var target = target || this;
        if (target._hitTolerance == 0) {
            if (target._selectionPath != null) {
            	target.remove(target._selectionPath);
            	target._selectionPath = null;
            }
        } else {
            this._updatePathShape(target,curveTension);
            this.applyPath(target._selectionPath,points,curveTension);
        }
    },
    
    _updatePathShape: function(target,curveTension){
		
    	if (target._selectionPath != null) {
        	if( ((curveTension > 0) && !(this._selectionPath instanceof gfx.Path)) || 
        	    ((curveTension == 0) && (this._selectionPath instanceof gfx.Path)) ){
        			target.remove(target._selectionPath);
        			target._selectionPath = null;
        	}
        }
    	if (target._selectionPath == null) {
    		if (curveTension > 0) {
    			target._selectionPath = target.createPath();
    		}else{
    			target._selectionPath = target.createPolyline();
    		}
    		target._selectionPath.setStroke(json.fromJson("{color: [255,255,255,0.001],width: " + this._hitTolerance + "}"));
    	}
	},
    
    getHitTolerance: function() {
        // Summary:
        //		gets the mouse tolerance.
        
        return this._hitTolerance;
    },
    
    setHitTolerance: function(tolerance) {
        // Summary: 
        //		sets the mouse tolerance. This value affects all the mouse events and the tolerance to consider it is over a link.
        //		The recomended values goes from 5 to 10 (bigger --> bigger tolerance).
        //		If 0, no tolerance is applied.
        
        this._hitTolerance = tolerance;
        this.invalidateLinkShape();
        return this;
    },
    
    //---------- Arrows ----------
    
    isStartArrowVisible: function() {
        // summary:
        //		Gets the visibility of the start arrow.
        // returns:
        //		True if the start arrow is defined in the link template and visible.
        //		False if the start arrow is either not defined in the link template or invisible.
        
        return (this._startArrow && this._startArrow.getParent()) ? true : false;
    },
    
    setStartArrowVisible: function(v) {
        // summary:
        //		Makes the start arrow visible or invisible.
        // v: Boolean:
        //		If no start arrow is defined in the link template
        //		(that is, if the template contains no shape with "dojoAttachPoint: '_startArrow'"),
        //		this method does nothing.
        //		Otherwise, if 'v' is true, the start arrow is shown (i.e., added to the link),
        //		and if 'v' is false, the start arrow is hidden (i.e. removed from the link).
        
        if (this._startArrow) {
            if (v) {
                if (!this._startArrow.getParent()) {
                    this.add(this._startArrow);
		            this.invalidateLinkShape();
                }
            } else {
                if (this._startArrow.getParent()) {
                    this.remove(this._startArrow);
		            this.invalidateLinkShape();
                }
            }
        }
        return this;
    },
    
    isEndArrowVisible: function() {
        // summary:
        //		Gets the visibility of the end arrow.
        // returns:
        //		True if the end arrow is defined in the link template and visible.
        //		False if the end arrow is either not defined in the link template or invisible.
        return (this._endArrow && this._endArrow.getParent()) ? true : false;
    },
    
    setEndArrowVisible: function(v) {
        // summary:
        //		Makes the end arrow visible or invisible.
        // v: Boolean:
        //		If no end arrow is defined in the link template
        //		(that is, if the template contains no shape with "dojoAttachPoint: '_endArrow'"),
        //		this method does nothing.
        //		Otherwise, if 'v' is true, the end arrow is shown (i.e., added to the link),
        //		and if 'v' is false, the end arrow is hidden (i.e. removed from the link).
        
        if (this._endArrow) {
            if (v) {
                if (!this._endArrow.getParent()) {
                    this.add(this._endArrow);
		            this.invalidateLinkShape();
                }
            } else {
                if (this._endArrow.getParent()) {
                    this.remove(this._endArrow);
		            this.invalidateLinkShape();
                }
            }
        }
        return this;
    },
    
    
    _updateArrow: function(arrow, endPoint, nextPoint,target) {
    	var target = target || this;
        // tag: private
        var _m = gfxm;
        var dx = nextPoint.x - endPoint.x;
        var dy = nextPoint.y - endPoint.y;
		// hack for VML: because of roundings in transforms,
		// almost vertical/horizontal segments cause exceptions...
		if(dx > -0.01 && dx < 0.01) dx = 0;
		if(dy > -0.01 && dy < 0.01) dy = 0;
        var a = Math.atan2(-dy, -dx);
        var transforms = [_m.translate(endPoint.x, endPoint.y), _m.rotate(a)];
        var stroke = target._path.getStroke();
        if (stroke && stroke.width > 2) {
            transforms.push(_m.scale(1 + (stroke.width - 2) / 4)); // nicer than linear scaling...
        }
		arrow.setTransform(transforms);
		
        // shorten the segment so that the polyline's end does not show at the tip of the arrow
        var l = Math.sqrt(dx * dx + dy * dy);
        if (l > 0) {
            stroke = target._path.getStroke();
            var w = stroke ? stroke.width : 2;
            var r = w / l;
            endPoint.x += r * dx;
            endPoint.y += r * dy;
        }
    },
    
    //---------- Points computation ----------
    
    getPoints: function() {
        // summary:
        //		Gets the actual points of the link.
        // description:
        //		This function calls validateLinkShape() and returns the points that have been computed.
        
        this.validateLinkShape();
        return lang.clone(this._pathPoints);
    },
    
    invalidateLinkShape: function() {
        // summary:
        //		Invalidates the shape of this link.
        // description:
        //		This function ensures that the shape of the link will be recomputed before the next redisplay of the link.
        //		This function is called automatically when the start or end port change, or their owner nodes change,
        //		or when a property of the link that determines the shape (such as the intermediate points) is changed.
        //		In most cases you do not need to call this function explicitly.
        
        this._pathPointsValid = false;
        Batch.add(this); // arrange so that _endBatch() is called soon
    },
    
    _endBatch: function() {
        // summary:
        //		This method will be called when invalidateLinkShape() has been called, and calls validateLinkShape().
        // tags:
        //		private
        
        this.validateLinkShape();
    },
    
    validateLinkShape: function() {
        // summary:
        //		Recomputes the shape of this link.
        // description:
        //		This function is called automatically when invalidateLinkShape() has been called, and before the
        //		link is redisplayed. 
        //		In most cases you do not need to call this function explicitly.
        
        if (!this._pathPointsValid) {
            this._computeLinkPoints();
            this._updatePath();
            this._pathPointsValid = true;
        }
    },
    
    revalidateLinkShape: function() {
        // summary:
        //		Recomputes the shape of this link.
        
    	this._pathPointsValid = false;
    	this.validateLinkShape();
    },

    _getParent: function() {
        // tag: private
        var p = this.getParent();
        if (p && !p.getParent) {
            p = null;
        }
        return p;
    },
    
    _computeLinkPoints: function() {
        // tag: private
        var _m = gfxm;
        var startPoint = this.getFallbackStartPoint();
        var endPoint = this.getFallbackEndPoint();
        var startOrigin = startPoint;
        var endOrigin = endPoint;
        var cp, op, p, result;
        
        var parent = this._getParent();
        
        if (parent !== null) {
			
			var startPort = this._isIntergraphLinkAtStart ? this._getVisiblePort(this._startPort) : this._startPort;
			var endPort = this._isIntergraphLinkAtEnd ? this._getVisiblePort(this._endPort) : this._endPort;
			
            var t = _m.invert(this.getTransform());
            
            if (startPort !== null &&
            startPort.getOwner() !== null) {
                p = {
                    x: 0,
                    y: 0
                };
                
                if (startPort.needsReferencePoint()) {
                    p = this._getReferencePoint(false, startPort, endPort);
                    if (endPort === null) {
                        p = _m.multiplyPoint(this.getTransform(), p);
                    }
                }
                cp = {};
                op = {};
                result = startPort.getConnectionPoint(this, parent, p, cp, op, true);
                startPoint = _m.multiplyPoint(t, cp);
                startOrigin = _m.multiplyPoint(t, op);
            }
            
            if (endPort !== null &&
            endPort.getOwner() !== null) {
                p = {
                    x: 0,
                    y: 0
                };
                
                if (endPort.needsReferencePoint()) {
                    p = this._getReferencePoint(true, startPort, endPort);
                    if (startPort === null) {
                        p = _m.multiplyPoint(this.getTransform(), p);
                    }
                }
                cp = {};
                op = {};
                result = endPort.getConnectionPoint(this, parent, p, cp, op, false);
                endPoint = _m.multiplyPoint(t, cp);
                endOrigin = _m.multiplyPoint(t, op);
            }
        }
        
        this._setLinkPoints(startPoint, startOrigin, endPoint, endOrigin, startPort, endPort);
    },
	
	_getVisiblePort: function(port) {
		if(!port)
			return null;
		var owner = port.getOwner();
		if(!owner)
			return null;
		var topMostCollapsedNode = null;
		var linkParent = this.getParent();
		for(var p = owner.getParent(); p && p.getParent && p != linkParent; p = p._collapsedParent || p.getParent()){
			if(p._collapsedParent){
				for (var pp = p._collapsedParent; pp && pp.getParent; pp = pp.getParent()) {
					if (pp._isIBMDiagramSubgraph) {
						topMostCollapsedNode = pp;
						break;
					}
				}
			}
		}
		if (topMostCollapsedNode) {
			var collapsedPort = topMostCollapsedNode.getCollapsedPort(this, port);
			if(collapsedPort){
				if(!collapsedPort.getOwner())
					topMostCollapsedNode.addPort(collapsedPort);
				return collapsedPort;
			} 
			return this._getOrCreateDefaultPort(topMostCollapsedNode);
		}
		return port;
	},
	
    _getReferencePoint: function(start, startPort, endPort) {
        // tag: private
        var _m = gfxm;
        var points = this._intermediatePoints;
        if ((this._shapeType === LinkShapeType.Free ||
        this._shapeType === LinkShapeType.OrthogonalEditable) &&
        points !== null &&
        points.length > 0) {
            var p = start ? points[points.length - 1] : points[0];
			var t = this.getTransform();
			if(t)
            	return _m.multiplyPoint(t, p);
			else
				return p;
        } else {
            var port = start ? startPort : endPort;
            if (port !== null && port.getOwner() !== null && !port.needsReferencePoint()) {
                cp = {};
                op = {};
                return port.getConnectionPoint(this, this.getParent(), null, cp, op, start);
            } else {
                var obj = start ? (startPort !== null ? startPort.getOwner() : null) : (endPort !== null ? endPort.getOwner() : null);
                if (obj !== null && obj.getParent() !== null) {
                    // Note by E.T
                    // testing obj.Parent != null is not enough because
                    // link from or to still may be not in same hierarchy
                    // but it fixes some basic cases.
                    var relBounds;
                    try {
                        relBounds = obj.getBounds(this.getParent());
                    } catch (Exception) {
                        // port owner not in same hierarchy.(for example not added yet)
						relBounds = null;
                    }
					if (relBounds != null) {
						return {
							x: relBounds.x + relBounds.width / 2,
							y: relBounds.y + relBounds.height / 2
						};
					} else {
						if (start) {
							return this.getFallbackStartPoint();
						}
						else {
							return this.getFallbackEndPoint();
						}
					}
                    
                } else {
                    if (start) {
                        return this.getFallbackStartPoint();
                    } else {
                        return this.getFallbackEndPoint();
                    }
                }
            }
        }
    },
    
    _setLinkPoints: function(startPoint, startOrigin, endPoint, endOrigin, startPort, endPort) {
        // tag: private
        
        var i;
        
        this._pathPoints = null;
        
        if (Math.abs(startPoint.x - endPoint.x) <= _constants._epsilon) {
            endPoint.x = startPoint.x;
        }
        if (Math.abs(startPoint.y - endPoint.y) <= _constants._epsilon) {
            endPoint.y = startPoint.y;
        }
        var points = [];
        switch (this._shapeType) {
            case LinkShapeType.Free:
                points.push(startPoint);
                if (this._intermediatePoints !== null) {
                    points.push.apply(points, this._intermediatePoints);
                }
                points.push(endPoint);
                break;
            case LinkShapeType.Straight:
                points.push(startPoint, endPoint);
                break;
            case LinkShapeType.Orthogonal:
            case LinkShapeType.OrthogonalEditable:
            case LinkShapeType.Oblique:
                // TODO (what?)
                
                var startDirection = !this._samePoint(startPoint, startOrigin) ? this._getDirection(startOrigin, startPoint) : this._getDirection(startPoint, endPoint);
                var endDirection = !this._samePoint(endPoint, endOrigin) ? this._getDirection(endOrigin, endPoint) : this._getDirection(endPoint, startPoint);
                
                points.push(startPoint);
                
                var startBounds, endBounds;
                var parent = this._getParent();
                if (parent !== null) {
                    if (startPort !== null) {
                        startBounds = startPort.getConnectionBounds(parent);
                    /*
                     } else if (this.getStart() !== null) {
                     startBounds = this.getStart().getBounds(parent);
                     */
                    } else {
                        startBounds = {
                            x: startPoint.x,
                            y: startPoint.y,
                            width: 0,
                            height: 0
                        };
                    }
                    if (endPort !== null) {
                        endBounds = endPort.getConnectionBounds(parent);
                    /*
                     } else if (this.getEnd() !== null) {
                     endBounds = this.getEnd().getBounds(parent);
                     */
                    } else {
                        endBounds = {
                            x: endPoint.x,
                            y: endPoint.y,
                            width: 0,
                            height: 0
                        };
                    }
                } else {
                    startBounds = {
                        x: startPoint.x,
                        y: startPoint.y,
                        width: 0,
                        height: 0
                    };
                    endBounds = {
                        x: endPoint.x,
                        y: endPoint.y,
                        width: 0,
                        height: 0
                    };
                }
                
                if (this._shapeType === LinkShapeType.OrthogonalEditable &&
                this._intermediatePoints !== null &&
                this._intermediatePoints.length > 0) {
                    var keepExisting = true;
                    
                    // Check that the existing segments are orthogonal.
                    //
                    if (keepExisting) {
                        for (i = 0; i < this._intermediatePoints.length - 1; i++) {
                            if (this._intermediatePoints[i].x != this._intermediatePoints[i + 1].x &&
                            this._intermediatePoints[i].y != this._intermediatePoints[i + 1].y) {
                                keepExisting = false;
                                break;
                            }
                        }
                    }
                    
                    if (keepExisting) {
                        // Keep the existing points,
                        // and just adjust the first/last segments.
                        //
                        points.push.apply(points, this._intermediatePoints);
                        //                        for each(var p in this._intermediatePoints) {
                        //							points.push(p);
                        //						}
                        //
                        points.push(endPoint);
                        
                        // adjust start:
                        //
                        this._adjustEndPoint(points, 0, 1);
                        
                        // adjust end:
                        //
                        this._adjustEndPoint(points, points.length - 1, -1);
                    } else {
                        this._computeOrthogonal(startPoint, startDirection, startBounds, endPoint, endDirection, endBounds, points);
                        points.push(endPoint);
                    }
                } else {
                    this._computeOrthogonal(startPoint, startDirection, startBounds, endPoint, endDirection, endBounds, points);
                    points.push(endPoint);
                }
                
                if (this._shapeType === LinkShapeType.Oblique) {
                
                    for (i = 0; i < points.length - 3; i++) {
                        var p1 = points[i];
                        var p2 = points[i + 1];
                        var p3 = points[i + 2];
                        var p4 = points[i + 3];
                        if (p1.x === p2.x && p3.x === p4.x) {
                            var dx = Math.abs(p3.x - p1.x) / 2;
                            var dy1 = p2.y - p1.y;
                            var dy2 = p4.y - p3.y;
                            if (((dy1 > 0 && dy2 > 0) || (dy1 < 0 && dy2 < 0)) &&
                            Math.abs(dy1) >= dx &&
                            Math.abs(dy2) >= dx) {
                                if (dy1 > 0) {
                                    points[i + 1] = {
                                        x: p2.x,
                                        y: p2.y - dx
                                    };
                                    points[i + 2] = {
                                        x: p3.x,
                                        y: p3.y + dx
                                    };
                                } else {
                                    points[i + 1] = {
                                        x: p2.x,
                                        y: p2.y + dx
                                    };
                                    points[i + 2] = {
                                        x: p3.x,
                                        y: p3.y - dx
                                    };
                                }
                            }
                        } else if (p1.y === p2.y && p3.y === p4.y) {
                            var dy = Math.abs(p3.y - p1.y) / 2;
                            var dx1 = p2.x - p1.x;
                            var dx2 = p4.x - p3.x;
                            if (((dx1 > 0 && dx2 > 0) || (dx1 < 0 && dx2 < 0)) &&
                            Math.abs(dx1) >= dy &&
                            Math.abs(dx2) >= dy) {
                                if (dx1 > 0) {
                                    points[i + 1] = {
                                        x: p2.x - dy,
                                        y: p2.y
                                    };
                                    points[i + 2] = {
                                        x: p3.x + dy,
                                        y: p3.y
                                    };
                                } else {
                                    points[i + 1] = {
                                        x: p2.x + dy,
                                        y: p2.y
                                    };
                                    points[i + 2] = {
                                        x: p3.x - dy,
                                        y: p3.y
                                    };
                                }
                            }
                        }
                    }
                    
                }
                
                break;
            default:
                break;
        }
        
        this._pathPoints = points;
    },
    
    _samePoint: function(p1, p2) {
        return (Math.abs(p1.x - p2.x) <= _constants._epsilon) &&
        (Math.abs(p1.y - p2.y) <= _constants._epsilon);
    },
    
    _getDirection: function(originPoint, connectionPoint) {
        // tag: private
        var dx = connectionPoint.x - originPoint.x;
        var dy = connectionPoint.y - originPoint.y;
        if (Math.abs(dx) >= Math.abs(dy)) {
            // Horizontal:
            if (dx > 0) {
                return _constants._right;
            } else {
                return _constants._left;
            }
        } else {
            // Vertical:
            if (dy > 0) {
                return _constants._bottom;
            } else {
                return _constants._top;
            }
        }
    },
    
    _adjustEndPoint: function(points, i, d) {
        // tag: private
        
        var p0 = points[i]; // The start or end point.
        var p1 = points[i + d]; // The point just next to the start/end point.
        // Do we need to adjust? Only if the first/last segment is not orthogonal.
        if (Math.abs(p0.x - p1.x) > _constants._epsilon && Math.abs(p0.y - p1.y) > _constants._epsilon) {
            // We must guess if the first/last segment must be horizontal or vertical.
            // For this, we cannot just look at the first segment, because the
            // first/last point may have been already modified by the interactor.
            // So, we look at the next segment, and we say that the first/last
            // segment must be orthogonal to it.
            //
            var p2 = points[i + d + d]; // The second point of the next segment.
            var nextSegmentIsHorizontal;
            var dx = Math.abs(p1.x - p2.x);
            var dy = Math.abs(p1.y - p2.y);
            if (dx < _constants._epsilon && dx < _constants._epsilon && points.length > 3) {
                // The next segment is null: this can happen if points
                // get aligned during interaction. In that case, we look at
                // the segment after the next segment.
                p1 = p2;
                p2 = points[i + d + d + d];
                dx = Math.abs(p1.x - p2.x);
                dy = Math.abs(p1.y - p2.y);
                nextSegmentIsHorizontal = dx < dy;
            } else {
                nextSegmentIsHorizontal = dx > dy;
            }
            
            if (nextSegmentIsHorizontal) {
                points[i + d] = {
                    x: p0.x,
                    y: p1.y
                };
            } else {
                points[i + d] = {
                    x: p1.x,
                    y: p0.y
                };
            }
        }
    },
    
    _rotatePosition: function(position, count) {
        // tag: private
        return (position + count) % 4;
    },
    
    _rotatePoint: function(p, count) {
        // tag: private
        var tmp;
        switch (count) {
            case 1:
                tmp = p.x;
                p.x = -p.y;
                p.y = tmp;
                break;
            case 2:
                p.x = -p.x;
                p.y = -p.y;
                break;
            case 3:
                tmp = p.x;
                p.x = p.y;
                p.y = -tmp;
                break;
        }
        return p;
    },
    _rotateRect: function(r, count) {
        // tag: private
        var tmp;
        switch (count) {
            case 1:
                tmp = r.x;
                r.x = -r.y - r.height;
                r.y = tmp;
                tmp = r.width;
                r.width = r.height;
                r.height = tmp;
                break;
            case 2:
                r.x = -r.x - r.width;
                r.y = -r.y - r.height;
                break;
            case 3:
                tmp = r.x;
                r.x = r.y;
                r.y = -tmp - r.width;
                tmp = r.width;
                r.width = r.height;
                r.height = tmp;
                break;
        }
        return r;
    },
    
    _computeOrthogonal: function(startPoint, startDirection, startBounds, endPoint, endDirection, endBounds, points) {
        // tag: private
        
        // First, rotate everything so that the start point points to the top.
        //
        var rotateCount = 0;
        switch (startDirection) {
            case _constants._top:
                break;
            case _constants._bottom:
                rotateCount = 2;
                break;
            case _constants._left:
                rotateCount = 1;
                break;
            case _constants._right:
                rotateCount = 3;
                break;
        }
        
        startPoint = this._rotatePoint(lang.clone(startPoint), rotateCount);
        startDirection = this._rotatePosition(startDirection, rotateCount);
        startBounds = this._rotateRect(lang.clone(startBounds), rotateCount);
        endPoint = this._rotatePoint(lang.clone(endPoint), rotateCount);
        endDirection = this._rotatePosition(endDirection, rotateCount);
        endBounds = this._rotateRect(lang.clone(endBounds), rotateCount);
        
        // OK, now we start to enumerate all directions for the end point.
        //
        var x, y;
        switch (endDirection) {
            case _constants._top:
                if (endBounds.y + endBounds.height < startBounds.y) {
                    if ((endBounds.x > startPoint.x) || (endBounds.x + endBounds.width < startPoint.x)) {
                        //     ___          ___
                        //    |   |        |   |
                        //    |   E   or   E   |
                        //    |                |
                        //    S                S
                        
                        y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    } else {
                        //     ___          ___
                        //    |   |        |   |
                        //    |   E   or   E   |
                        //    |__            __|
                        //       |          |
                        //       S          S
                        
                        y = (startPoint.y + endBounds.y + endBounds.height) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        if (endPoint.x > startPoint.x) {
                            x = endBounds.x - this._nodeDistance;
                        } else {
                            x = endBounds.x + endBounds.width + this._nodeDistance;
                        }
                        points.push({
                            x: x,
                            y: y
                        });
                        y = endBounds.y - this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    }
                } else if (endBounds.y > startBounds.y + startBounds.height) {
                    if ((endPoint.x > startBounds.x + startBounds.width) || (endPoint.x < startBounds.x)) {
                        //     ___          ___
                        //    |   |        |   |
                        //    S   |   or   |   S
                        //        E        E
                        
                        y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    } else {
                        //     __          __
                        //    |  |        |  |
                        //    S  |   or   |  S
                        //      _|        |_
                        //     |            |
                        //     E            E
                        
                        y = startBounds.y - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        if (endPoint.x < startPoint.x) {
                            x = startBounds.x - this._nodeDistance;
                        } else {
                            x = startBounds.x + startBounds.width + this._nodeDistance;
                        }
                        points.push({
                            x: x,
                            y: y
                        });
                        y = (endPoint.y + startBounds.y + startBounds.height) / 2;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    }
                } else {
                    //     _          _
                    //    | |        | |
                    //    S E   or   E S
                    
                    y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    points.push({
                        x: endPoint.x,
                        y: y
                    });
                }
                break;
            case _constants._bottom:
                if (endPoint.y < startPoint.y) {
                    if (endPoint.x !== startPoint.x) // straight link!...
                    {
                        //           E        E
                        //     ______|   or   |______
                        //    |                      |
                        //    S                      S
                        
                        y = (startPoint.y + endPoint.y) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        points.push({
                            x: endPoint.x,
                            y: y
                        });
                    }
                } else if ((endBounds.x > startBounds.x + startBounds.width) ||
                (endBounds.x + endBounds.width < startBounds.x)) {
                    //     ____                    ____
                    //    |    |    E   or   E    |    |
                    //    S    |____|        |____|    S
                    
                    y = startBounds.y - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    if (endBounds.x > startBounds.x + startBounds.width) {
                        x = (endBounds.x + startBounds.x + startBounds.width) / 2;
                    } else {
                        x = (endBounds.x + endBounds.width + startBounds.x) / 2;
                    }
                    points.push({
                        x: x,
                        y: y
                    });
                    y = endBounds.y + endBounds.height + this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: endPoint.x,
                        y: y
                    });
                } else {
                    //     ______          ______
                    //    |      |        |      |
                    //    S      |   or   |      S
                    //        E  |        |  E
                    //        |__|        |__|
                    
                    y = Math.min(startBounds.y, endBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    if (endPoint.x > startPoint.x) {
                        x = Math.max(startBounds.x + startBounds.width, endBounds.x + endBounds.width) + this._nodeDistance;
                    } else {
                        x = Math.min(startBounds.x, endBounds.x) - this._nodeDistance;
                    }
                    points.push({
                        x: x,
                        y: y
                    });
                    y = endBounds.y + endBounds.height + this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: endPoint.x,
                        y: y
                    });
                }
                break;
            case _constants._left:
                if (endBounds.x < startPoint.x) {
                    if (endBounds.y + endBounds.height < startPoint.y) {
                        //    _E
                        //   |___
                        //       |
                        //       S
                        
                        y = (startPoint.y + endBounds.y + endBounds.height) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = endBounds.x - this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    } else {
                        //     _____
                        //    |     |
                        //    |_E   |
                        //          S
                        
                        y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = Math.min(endBounds.x, startBounds.x) - this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    }
                } else if (endPoint.y < startBounds.y) {
                    //     ____E
                    //    |
                    //    |
                    //    S
                    
                    points.push({
                        x: startPoint.x,
                        y: endPoint.y
                    });
                } else if (endBounds.x > startBounds.x + startBounds.width) {
                    //     ____
                    //    |    |
                    //    S    |___E
                    
                    y = startBounds.y - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = (startBounds.x + startBounds.width + endBounds.x) / 2;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                } else {
                    //     ___
                    //    |   |
                    //    |   S
                    //    |__E
                    
                    y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = Math.min(endBounds.x, startBounds.x) - this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                }
                break;
            case _constants._right:
                if (endBounds.x + endBounds.width > startPoint.x) {
                    if (endBounds.y + endBounds.height < startPoint.y) {
                        //      E_
                        //     ___|
                        //    |
                        //    S
                        
                        y = (startPoint.y + endBounds.y + endBounds.height) / 2;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = endBounds.x + endBounds.width + this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    } else {
                        //     _____
                        //    |     |
                        //    |   E_|
                        //    S
                        
                        y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                        points.push({
                            x: startPoint.x,
                            y: y
                        });
                        x = Math.max(endBounds.x + endBounds.width, startBounds.x + startBounds.width) + this._nodeDistance;
                        points.push({
                            x: x,
                            y: y
                        });
                        points.push({
                            x: x,
                            y: endPoint.y
                        });
                    }
                } else if (endPoint.y < startBounds.y) {
                    //    E____
                    //         |
                    //         |
                    //         S
                    
                    points.push({
                        x: startPoint.x,
                        y: endPoint.y
                    });
                } else if (endBounds.x + endBounds.width < startBounds.x) {
                    //          ____
                    //         |    |
                    //    E____|    S
                    
                    y = startBounds.y - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = (startBounds.x + endBounds.x + endBounds.width) / 2;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                } else {
                    //     ___
                    //    |   |
                    //    S   |
                    //     E__|
                    
                    y = Math.min(endBounds.y, startBounds.y) - this._nodeDistance;
                    points.push({
                        x: startPoint.x,
                        y: y
                    });
                    x = Math.max(endBounds.x + endBounds.width, startBounds.x + startBounds.width) + this._nodeDistance;
                    points.push({
                        x: x,
                        y: y
                    });
                    points.push({
                        x: x,
                        y: endPoint.y
                    });
                }
                break;
        }
        
        // Rotate the computed points back to the original orientation.
        //
        if (rotateCount !== 0) {
            rotateCount = 4 - rotateCount;
            for (var i = 1; i < points.length; i++) {
                var p = points[i];
                p = this._rotatePoint(p, rotateCount);
                points[i] = p;
            }
        }
    },
    _changeSelectionStyle: function(selected) {
        this.inherited(arguments);
        this.invalidateLinkShape();
    },
	
	// Link decorations:
	
	getDecorations: function() {
		// summary:
		//		Gets an array containing the decorations of the link.
		// returns: Array:
		//		An array of GFX shapes.
		// description:
		//		The link can contain shapes called "decorations". Decorations are automatically
		//		placed along the link path. To be considered as a decoration, a child shape
		//		must have the "isDecoration" set to true. The decoration can have the following properties
		//		that define their placement on the link:
		//		- position: A number that defines the position of the decoration along the link. The meaning
		//				of the position depends on the positionType property.
		//				The default position is 0.5, and 
		//				The default positionType is ibm_ilog.diagram.LinkDecorationPositionType.Relative,
		//				so by default a decoration is placed at the middle of the link.
		//		- positionType: Defines the way the position is interpreted. Possible values are:
		//			ibm_ilog.diagram.LinkDecorationPositionType.Relative:
		//				The position property is a ratio between 0 and 1 that defines a relative position on the
		//				link (0 = start of the link, 1 = of the link, 0.5 = middle of the link, etc).
		//			ibm_ilog.diagram.LinkDecorationPositionType.AnsoluteFromStart:
		//				The position is a fixed length that defines an offset from the start of the link
		//				(going forward from the start to the end of the link).
		//			ibm_ilog.diagram.LinkDecorationPositionType.AnsoluteFromMiddle:
		//				The position is a fixed length that defines an offset from the middle of the link
		//				(going forward from the start to the end of the link).
		//			ibm_ilog.diagram.LinkDecorationPositionType.AnsoluteFromEnd:
		//				The position is a fixed length that defines an offset from the end of the link
		//				(going backward from the end to the start of the link).
		//				The default positionType is ibm_ilog.diagram.LinkDecorationPositionType.Relative,
		//				and the default position is 0.5,
		//				so by default a decoration is placed at the middle of the link.
		//		- orthogonalAlignment: Defines the alignment of the decoration orthogonally to the link path.
		//				Possible values are:
		//			ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Left:
		//				The decoration is placed on the "left" of the link, looking towards the end of the link
		//				(for example, if the link is horizontal and pointing to the right, the decoration will be
		//				above the link). The spacing between the decoration and the link is defined by the
		//				distance property.
		//			ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center:
		//				The decoration is centered on the link.
		//			ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Start:
		//				The decoration is placed on the "right" of the link, looking towards the end of the link
		//				(for example, if the link is horizontal and pointing to the right, the decoration will be
		//				below the link). The spacing between the decoration and the link is defined by the
		//				distance property.
		//				The default orthogonalAlignment is ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Center.
		//		- parallelAlignment: Defines the alignment of the decoration along the link.
		//				Possible values are:
		//			ibm_ilog.diagram.LinkDecorationParallelAlignment.Start:
		//				The side of the decoration that is closest to the start of the link is aligned
		//				on the point defined by the position and positionType.
		//			ibm_ilog.diagram.LinkDecorationParallelAlignment.Middle:
		//				The middle of the decoration is aligned
		//				on the point defined by the position and positionType.
		//			ibm_ilog.diagram.LinkDecorationParallelAlignment.Start:
		//				The side of the decoration that is closest to the end of the link is aligned
		//				on the point defined by the position and positionType.
		//				The default parallelAlignment is ibm_ilog.diagram.LinkDecorationParellelAlignment.Middle.
		//		- distance: Defines the distance between the decoration and the link when
		//				orthogonalAlignment is ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Left
		//				or ibm_ilog.diagram.LinkDecorationOrthogonalAlignment.Right.
		//				The default distance is 0.
		//		- autoRotate: If autoRotate is true, the decoration is rotated according to the orientation
		//				of the link segment that contains the decoration position.
		//				By default, autoRotate is false.

		var decorations = [];
		
		var children = this.children;
		var childrenCount = children.length;
		
		for (var childIndex = 0; childIndex < childrenCount; childIndex++) {
			var item = children[childIndex];
			if (item.isDecoration) {
				decorations.push(item);
			}
		}
		
		return decorations;
	},
	
	_placeDecorations: function(target){
		// summary:
		//		Moves (and possibly rotates) a decoration along the link's path.
		// tags:
		//		private
		
		target = target || this;
		
		var points = null;
		var totalLength = 0;
		
		var children = target.children;
		var childrenCount = children.length;
		
		for (var childIndex = 0; childIndex < childrenCount; childIndex++) {
			var item = children[childIndex];
			if (item.isDecoration) {
				
				if (!points) {
					points = target._pathPoints;
					// compute total length of link
					for (var i = 0; i < points.length - 1; i++) 
						totalLength += Geometry.getDistance(points[i], points[i + 1]);
				}
				
				// get placement parameters and set default values
				
				var position = (item.position !== undefined) ? item.position : 0.5;
				var positionType = item.positionType || iid.LinkDecorationPositionType.Relative;
				var positionLength = 0;
				var autoRotate = item.autoRotate;
				var distance = item.distance || 0;
				var orthogonalAlignment = item.orthogonalAlignment || iid.LinkDecorationOrthogonalAlignment.Center;
				var parallelAlignment = item.parallelAlignment || iid.LinkDecorationParallelAlignment.Middle;
				
				switch (positionType) {
					case iid.LinkDecorationPositionType.Relative:
						positionLength = totalLength * position;
						break;
					case iid.LinkDecorationPositionType.AbsoluteFromStart:
						positionLength = position;
						break;
					case iid.LinkDecorationPositionType.AbsoluteFromMiddle:
						positionLength = totalLength / 2 + position;
						break;
					case iid.LinkDecorationPositionType.AbsoluteFromEnd:
						positionLength = totalLength - position;
						break;
				}
				
				var currentLength = 0;
				var i;
				for (i = 0; i < points.length - 1; i++) {
					currentLength += Geometry.getDistance(points[i], points[i + 1]);
					if (positionLength < currentLength) 
						break;
				}
				if (i == points.length - 1) 
					i--;
				
				var d = Geometry.getDistance(points[i], points[i + 1]);
				
				var p;
				var dx, dy;
				
				if (d == 0) {
					p = {
						x: points[i].x,
						y: points[i].y
					};
					dx = 0;
					dy = 0;
				}
				else {
					var e = currentLength - positionLength;
					dx = points[i + 1].x - points[i].x;
					dy = points[i + 1].y - points[i].y;
					p = {
						x: points[i + 1].x - (dx * e) / d,
						y: points[i + 1].y - (dy * e) / d
					};
				}
				
				// Compute final position based on alignments and distance:
				//
				
				item.setTransform(null);
				
				var bounds = item.getBoundingBox() || {height: 0, width: 0, x:0, y:0};
				
				var rotationTransform = null;
				
				if (dy == 0 && !autoRotate) {
					// Horizontal segment:
					//
					var dir = dx >= 0 ? 1 : -1;
					switch (orthogonalAlignment) {
						case iid.LinkDecorationOrthogonalAlignment.Center:
							break;
						case iid.LinkDecorationOrthogonalAlignment.Left:
							p.y -= dir * (distance + bounds.height / 2);
							break;
						case iid.LinkDecorationOrthogonalAlignment.Right:
							p.y += dir * (distance + bounds.height / 2);
							break;
					}
					switch (parallelAlignment) {
						case iid.LinkDecorationParallelAlignment.Middle:
							break;
						case iid.LinkDecorationParallelAlignment.Start:
							p.x += dir * (bounds.width / 2);
							break;
						case iid.LinkDecorationParallelAlignment.End:
							p.x -= dir * (bounds.width / 2);
							break;
					}
				}
				else 
					if (dx == 0 && !autoRotate) {
						// Vertical segment:
						//
						dir = dy >= 0 ? 1 : -1;
						switch (orthogonalAlignment) {
							case iid.LinkDecorationOrthogonalAlignment.Center:
								break;
							case iid.LinkDecorationOrthogonalAlignment.Left:
								p.x += dir * (distance + bounds.width / 2);
								break;
							case iid.LinkDecorationOrthogonalAlignment.Right:
								p.x -= dir * (distance + bounds.width / 2);
								break;
						}
						switch (parallelAlignment) {
							case iid.LinkDecorationParallelAlignment.Middle:
								break;
							case iid.LinkDecorationParallelAlignment.Start:
								p.y += dir * (bounds.height / 2);
								break;
							case iid.LinkDecorationParallelAlignment.End:
								p.y -= dir * (bounds.height / 2);
								break;
						}
					}
					else {
						// Oblique segment, or AutoRotate == true
						//
						
						var angle = Math.atan2(dy, dx);
						var t = gfxm.rotateAt(-angle, p);
						var pbounds;
						if (autoRotate) 
							pbounds = lang.clone(bounds);
						else 
							pbounds = t.transformRectangle(bounds);
						
						
						switch (orthogonalAlignment) {
							case iid.LinkDecorationOrthogonalAlignment.Center:
								break;
							case iid.LinkDecorationOrthogonalAlignment.Left:
								p.y -= distance + pbounds.height / 2;
								break;
							case iid.LinkDecorationOrthogonalAlignment.Right:
								p.y += distance + pbounds.height / 2;
								break;
						}
						switch (parallelAlignment) {
							case iid.LinkDecorationParallelAlignment.Middle:
								break;
							case iid.LinkDecorationParallelAlignment.Start:
								p.x += pbounds.width / 2;
								break;
							case iid.LinkDecorationParallelAlignment.End:
								p.x -= pbounds.width / 2;
								break;
						}
						
						t = t.inverse();
						p = t.transformPoint(p);
						
						if (autoRotate) {
							var pi = Math.PI;
							if (angle > pi / 2 || angle < -pi / 2) {
								if (angle < 0) 
									angle += pi;
								else 
									angle -= pi;
								t = gfxm.rotateAt(angle, {
									x: t.dx,
									y: t.dy
								});
							}
							item.setTransform(t);
							bounds = t.transformRectangle(bounds);
						}
					}
				
				// OK, move the decoration now, 'p' contains the location of the center of the decoration.
				
				dx = p.x - (bounds.x + bounds.width / 2)
				dy = p.y - (bounds.y + bounds.height / 2);
				
				if (dx != 0 || dy != 0) 
					item.applyLeftTransform(gfxm.translate(dx, dy));
			}
		}
	},
	
// ---------- Bounds: ----------
    
    _onChanged: function(shape, reason) {
    	this._bbCacheValid = false;
    	if(this._explicitSize)
    		this._explicitSize = null;
	},
	
	_computePreferredSize: function(proposedSize){
		this._noOnChanged = true;
		this.validateLinkShape();
		delete this._noOnChanged;
    	this._bbCacheValid = false;
    	if(this._explicitSize)
    		this._explicitSize = null;
		return GraphElement.prototype._computePreferredSize.call(this, proposedSize);
	},
	
    getBounds: function(/*dojox.gfx.Group*/parent) {
        // summary:
        //		Returns the bounds of this node in the coordinate space of the specified parent.
        // parent: dojox.gfx.Group
        //		The parent object that determines the coordinate space
        //		the bounds are relative to. If null or undefined, the bounds of the node
        //		relative to its parent are returned.
        // returns:
        //		A rectangle (i.e. an object with x/y/width/height properties) containing
        //		the bounds of the node in the specified coordinate space.
        
        var rect = this.getLocalBoundingBox();
        if (rect && parent && parent != this.getParent()) {
            var t = this.getTransformToContainer(parent);
            rect = t.transformRectangle(rect);
        }
        return rect;
    },
    
    hitTest: function(p,graph,tolerance){
    	// Summary:
    	//		return if the given point hits the element
    	var g = Geometry;
    	
    	if(tolerance == null){
    		tolerance = this._hitTolerance;
    	}
    	
    	var bb = this.getBounds(graph);
    	if(!bb) {
    		return false;
    	}
    	bb = g.expandRect(bb,tolerance);
    	if(g.containsPoint(bb,p)) {
    		var path = this._path;
    		if (path instanceof gfx.Polyline) {
    			var t = this.getShapeToContainerTransform(graph).inverse();
    			p = t.transformPoint(p);
    			if(this._getPathDistance(path,p)<tolerance){
    				return true;
    			}
    		}else{
    			// TODO if the link is not a polyline, use just the bounding box
    			return true;
    		}
    	}
    	return false;
    },
    
    _getPathDistance: function(path,p){
    	// Summary:
    	//		calculates the distance from the polyline to the given point
    	//
    	var g = Geometry;
    	var points = path.getShape().points;
    	var distance = Number.MAX_VALUE;
    	for(var i=1;i<points.length;i++){
    		var a = points[i-1];
    		var b = points[i];
    		var d = g.getPointToSegmentDistance(a,b,p);
    		distance = Math.min(distance,d);
    	}
    	return distance;
    }
});

var LinkDecorationPositionType = {
	Relative: 1,
	AbsoluteFromStart: 2,
	AbsoluteFromMiddle: 3,
	AbsoluteFromEnd: 4
};
iid.LinkDecorationPositionType = LinkDecorationPositionType;

var LinkDecorationParallelAlignment = {
	Start: 1,
	Middle: 2,
	End: 3
};
iid.LinkDecorationParallelAlignment = LinkDecorationParallelAlignment;

var LinkDecorationOrthogonalAlignment = {
	Left: 1,
	Center: 2,
	Right: 3
};
iid.LinkDecorationOrthogonalAlignment = LinkDecorationOrthogonalAlignment;

var _constants = {
    _top: 0,
    _right: 1,
    _bottom: 2,
    _left: 3,
    
    _epsilon: 0.001
};

Link.nodeType = gfx.Group.nodeType;
Link.defaultShape = {
    type: 'link'
};

// Declare bindable properties
templating.declareBindableProperty(Link, "strokeColor", "#000000");
templating.declareBindableProperty(Link, "strokeWidth", 2);
templating.declareBindableProperty(Link, "selectedStrokeColor", "#0000FF");
templating.declareBindableProperty(Link, "selectedStrokeWidth", 3);
templating.declareBindableProperty(Link, "data", null);
templating.declareBindableProperty(Link, "textDir", ""); 

Link.defaultTemplate = iid.declareTemplate(templates.defaultLinkTemplate);

Link.LinkShapeType = LinkShapeType; // for easier access from module value

return Link;

});

},
'ibm_ilog/diagram/gfxlayout/Engine':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout/Engine", ["dojo/_base/array", "./_base", "../util/Batch"],
function(array, gfxlayout, Batch){

//
// public interface to the layout engine singleton.
//

gfxlayout.postInvalidate = function(/*dojox.gfx.Group*/obj) {
    // summary:
    //     Posts an asynchronous validation request for the specified Panel.
    gfxlayout.Engine.invalidate(/*dojox.gfx.Shape*/obj);
};

gfxlayout.doPendingValidate=function(/*dojox.gfx.Group*/obj){
    // summary:
    //     Processes immediately any validation request registered for the specified panel.
    gfxlayout.Engine.doPendingValidate(obj);
};

//
// Layout engine (Singleton).
// 
gfxlayout.Engine = (function() {
    // the singleton instance
    var instance = null;
    
    function Ctor() {
        
        // the validation requests queue
        var _queue = [];
        
        // Posts an asynchronous validation request for the specified shape.
        this.invalidate = function(/*dojox.gfx.Shape*/obj) {
            
            // ensures the shape is in the hierarchy
            if (this._isOrphan(obj)) {
                return;
            }
            // find the top-level parent
            var topPnl = obj;
            while (topPnl.getParent() &&
                   topPnl.getParent().getLayout &&
                   topPnl.getParent().getLayout()) {
                topPnl = topPnl.getParent();
            }
            // no -panel- found (ie Group with a layout mgr)
            if (!topPnl.getLayout || !topPnl.getLayout()) {
                return;
            }
            // invalidate() invoked during the layout process ?
            if (topPnl.getLayout().inLayout()) 
                return;
            // if the top panel found is already in the pending queue,
            // do nothing
            if (array.some(_queue, function(item) {
                if (item === topPnl) 
                    return true; // stop loop
                })) {
                return; // already in the queue, do nothing
            }
            // add it to the pending validation queue
            _queue.push(topPnl);
            // add a layout() request to the Batch
            Batch.add(this);
        };
        
        // Processes immediately the validation requests registered for the specified shape.
        this.doPendingValidate=function(/*dojox.gfx.Shape*/obj){
            var topPnl = obj;
            while (topPnl.getParent() &&
                   topPnl.getParent().getLayout()) {
                topPnl = topPnl.getParent();
            }
            if (topPnl && topPnl.getLayout()){
                if (topPnl.getLayout().inLayout()) 
                    return;
                // look up the panel in the queue
                var ql = _queue.length;
				for (var i=0;i<ql;++i){
                    if (_queue[i] === topPnl) {
                        _queue[i] = null;
                        // Layout it
                        this._layout(topPnl);
                        break;
                    }
                }
            }            
        };
        
        // Invoked by the Batch to trigger the pending layout() request
        this._endBatch = function() {
            this.layout();
        };
        
        this.layout = function() {
            // init a new pending queue
            var queue = _queue;
            _queue = [];
			var fl = this._layout;
            var ql = queue.length;
            for (var i = 0; i < ql; ++i) {
                var panel = queue[i];
                if (!panel) // may have been removed by a doPendingValidate()
                    continue;
                fl.call(this, panel);
            }
        };
        
        this._layout = function(/*dojox.gfx.Group*/panel){
            //console.log("Engine layout");
            // is it still in the hierarchy ? may have been removed since then
            if (this._isOrphan(panel)) {
                return;
            }
            // get the explicit size of the top level panel to validate.
            // by default it's w=-1 (auto) and h=-1 (auto)
            var size = panel.getExplicitSize();
            // computes children size
            size = panel.computePreferredSize(size);
            // layout the panel
            panel.layout({x:0, y:0, width:size.width, height:size.height});
        };
        
        this._isOrphan = function(shape){
            // a Surface does not have getParent() method.
            for (var p = shape.getParent();p && p.getParent; p=p.getParent()){
            }
            return p === null;
        };
        
    };
    
    return new function() {
        if (instance == null) {
            instance = new Ctor();
            instance.constructor = null;
        }
        return instance;
    };
})();

	return gfxlayout;
	
});

},
'ibm_ilog/diagram/graphlayout/WebWorkerLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/WebWorkerLayout", [
"require",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/config",
"dojo/_base/json",
"dojo/_base/window",
"dojo/_base/url",
"./ServerSideLayout"
],
function(
require,
declare,
lang,
config,
json,
window,
_Url,
ServerSideLayout
){

/*=====
var _AsynchronousLayout = ibm_ilog.diagram.graphlayout._AsynchronousLayout;
=====*/

var WebWorkerLayout =
declare('ibm_ilog.diagram.graphlayout.WebWorkerLayout', ServerSideLayout._AsynchronousLayout, 
{
	// summary:
	//		Executes a graph layout asynchronously in a Web Worker.
	// description:
	//		The WebWorkerLayout class is similar to the ServerSideLayout class,
	//		but instead of sending a request to a REST service on a remote server,
	//		it executes the graph layout on the client asynchronously using an HTML 5 Web Worker.
	//		This class can be used only if the browser supports web workers (which is not
	//		the case of Internet Explorer, for example).
	//		
	//		The main advantage of executing a graph layout in a Web Worker is that the browser UI
	//		is not blocked while the layout is executed. On the other hand, executing the
	//		layout in a worker thread has a cost, mainly due to the initialization of the
	//		worker which must load its own version of Dojo and the graph layout code before the layout
	//		can be executed. In addition, the graph topology is sent to the web worker and
	//		returned back to the calling thread by serializing the whole graph as a JSON string,
	//		which takes some additional time for big graphs.
	//		
	//		A WebWorkerLayout object is constructed with a Graph object.
	//		To execute a layout, the user must select the algorithm with the standard 
	//		client-side GraphLayout API:
	//		
	//		graph.setNodeLayout(x);
	//		graph.setLinkLayout(y);
	//		
	//		After this, the WebWorkerLayout object can be used to execute these 
	//		algorithms in a web worker by calling:
	//		
	//		webWorkerLayout.layout();
	//		
	//		This will obtain the currently configured layouts on the graph object,
	//		including optional settings for laying out internal subgraphs, gather
	//		their configured parameters and requesting the equivalent layout in the
	//		web worker.
	//
	//		Each time a layout is requested, the graph and the layout settings are
	//		serialized into a JSON object sent in the request.
	//
	//		The layout() method returns a dojo.Deferred which is signaled after the
	//		execution is complete.
	//		
	//		The web worker is reused if possible when the same WebWorkerLayout object
	//		is called several times on the same graph. The first layout execution will take more time
	//		due to the fact that the worker must load Dojo and the graph layout code.
	//		Subsequent layout executions will be faster.
	//		
	//		The WebWorkerLayout class cannot be used if Dojo is loaded from a CDN,
	//		so you must have a local copy of Dojo on your web site.
	
	_worker: null,
	
	_debugPrefix: "WebWorkerLayout: ",
	
	// dojoDir: String
	//		The parent directory of dojo/, relative to the ibm_ilog/diagram/graphlayout/webworker/ directory.
	//		This will be passed to the Web Worker thread to load Dojo.
	//		If not specified, this will be computed from the document URL and the dojo.baseUrl.
	dojoDir: null,
	
	_start: function(data) {
		//
		//	summary:
		//		Creates the Worker object to execute the layout, and applies the results.
		//	returns:
		//		a dojo.Deferred to signal when the process finishes.
		//

		// the worker is reused if already created
		if(!this._worker){
			// create and setup worker
			// first, compute the relative URL from the main application to the _Worker.js:
			this._worker = new Worker(require.toUrl('ibm_ilog/diagram/graphlayout/webworker/_Worker.js'));
			this._worker.onmessage = lang.hitch(this, function(e){
				
				var data = json.fromJson(e.data);
				if (data._error) {
					this._onError(data);
					this._worker = null; // don't reuse worker after an error...
				} else if(data._info) { console.log("[worker says:] " + data.message);
				} else {
					this._laidout = data;
					this._preLayoutDeferred.addCallback(lang.hitch(this,"_onComplete"));
				}
			});
			this._worker.onerror = lang.hitch(this, function(e){
				this._onError({ _error: true, responseText: e.message, workerError: e });
			});
			
			// Compute urls of dojo, ibm_ilog/graphlayout and ibm_ilog/diagram

			var baseUrl = this._getWorkerUrl(config.baseUrl);
			var graphlayoutUrl = this._getWorkerUrl(require.toUrl("ibm_ilog/graphlayout"));
			var diagramUrl = this._getWorkerUrl(require.toUrl("ibm_ilog/diagram"));

			this._worker.postMessage(baseUrl);
			this._worker.postMessage(graphlayoutUrl);
			this._worker.postMessage(diagramUrl);
			
		}
		
		// Now send the graph data:
		this._worker.postMessage(data);
	},
	
	_getWorkerUrl: function(url){
		return new _Url(window.doc.location.href, url).uri;
	},
	
	_stop: function()
	{
		if(this._worker){
			this._worker.terminate();
			this._worker = null; // cannot be reused after terminate()
		}
	}
});

return WebWorkerLayout;

});


},
'ibm_ilog/graphlayout/AbstractGraphLayoutModel':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// #######################################
// ######## NOT Generated by J2JS ########
// ###### DO NOT DELETE THIS FILE! #######
// #######################################

define("ibm_ilog/graphlayout/AbstractGraphLayoutModel", ["dojo/_base/declare", "dojo/_base/array", "./internalutil/LinkStyles"], 
	function(declare, arr, linkStyles) {
	return declare('ibm_ilog.graphlayout.AbstractGraphLayoutModel', null, {
    _statics: {
        _uniqueIdCounter: -1,
        _rootModelPropertyName: "__RootModel"
    },
    _attachedLayouts: null,
    _nodeOrLinkProperties: null,
    _properties: null,
    _rootModel: null,
    _isDisposed: null,
    _subgraphModelPropertyName: null,

    constructor: function() {
        this._isDisposed = false;
        var id = this._statics._uniqueIdCounter++;
        this.initPropertyNames(id);
    },
    initPropertyNames: function(id) {
        this._subgraphModelPropertyName = this.getParameterName("__SubgraphModel", id);
    },
    getParameterName: function(name, id) {
        var n = name + id;
        return n;
    },
    dispose: function() {
        if (!this._isDisposed) {
            this.disposeChildren();
            this.disposeThis();
        }
    },
    disposeThis: function() {
        if (this._isDisposed) {
            return;
        }
        this._isDisposed = true;
        this.detachLayouts();
        this.cleanSubmodelProperty();
        this._nodeOrLinkProperties = null;
        this._properties = null;
        this._rootModel = null;
    },
    cleanSubmodelProperty: function() {
        var rootModel = this.getRoot();
        if (rootModel !== null) {
            var parentModel = getParent();
            if (parentModel !== null) {
                var subgraphs = parentModel.getSubgraphs();
                var subModel;
                arr.forEach(subgraphs, function(subgraph) {
                    subModel = rootModel.getStoredGraphModel(subgraph);
                    if (subModel === this) {
                        rootModel.removeStoredGraphModel(subgraph);
                    }
                }, this);
            }
        }
    },
    detachLayouts: function() {
        if (this._attachedLayouts === null) {
            return;
        }
        var layouts = this._attachedLayouts;
        var n = layouts.length;
        for (var i = 0; i < n; i++) {
            var layout = layouts[i];
            if (layout.getGraphModel() === this) {
                layout.detach();
            }
        }
    },
    disposeChildren: function() {
        if (this._isDisposed) {
            return;
        }

        var rootModel = this.getRoot() !== null ? this.getRoot() : this;
        var subgraphs = this.getSubgraphs();
        var subGraphModel;
        arr.forEach(subgraphs, function(subgraph) {
            subGraphModel = rootModel.getStoredGraphModel(subgraph);
            if (subGraphModel !== null) {
                subGraphModel.dispose();
            }
            rootModel.removeStoredGraphModel(subgraph);
        });
    },
    isDisposed: function() { return this._isDisposed; },
    onAttach: function(layout) {
        if (this._attachedLayouts === null) {
            this._attachedLayouts = [];
        }
        if (arr.indexOf(this._attachedLayouts, layout) < 0) {
            this._attachedLayouts.push(layout);
        }
    },
    onDetach: function(layout) {
        if (this._attachedLayouts === null) {
            return;
        }
		var index = arr.indexOf(this._attachedLayouts, layout);
		if(index >= 0)
			this._attachedLayouts.splice(index, 1);
        if (this._attachedLayouts.length === 0) {
            this._attachedLayouts = null;
        }
    },
    getParent: function() { },
    getRoot: function() { return this._rootModel; },
    setRootModel: function(rootModel) { this._rootModel = rootModel; },
    storeGraphModel: function(subgraph, graphModel) {
        if (graphModel === this) {
            throw { name: 'ArgumentException', message: "The root model cannot be reused for subgraphs" };
        }
        this.setParameter(subgraph, this._subgraphModelPropertyName, graphModel);
        if (graphModel !== null) {
            graphModel.setRootModel(this);
        }
    },
    removeStoredGraphModel: function(subgraph) {
        this.setParameter(subgraph, this._subgraphModelPropertyName, null);
    },
    getStoredGraphModel: function(subgraph) {
        return this.getParameter(subgraph, this._subgraphModelPropertyName);
    },
    getGraphModel: function(subgraph) {
        if (subgraph === null) {
            throw { name: 'ArgumentException', message: "subgraph cannot be null" };
        }
        if (this.isDisposed()) {
            throw { name: 'Exception', message: "Called on disposed model: " + this };
        }
        var rootModel = this.getRoot();
        if (rootModel !== null) {
            if (rootModel.isDisposed()) {
                throw { name: 'Exception', message: "root model of " + this + " is disposed" };
            }
            return rootModel.getGraphModel(subgraph);
        }
        var graphModel = this.getStoredGraphModel(subgraph) || null;
        if (graphModel === null) {
            this.setParameter(subgraph, this._statics._rootModelPropertyName, this);
            graphModel = this.createGraphModel(subgraph);
            this.setParameter(subgraph, this._statics._rootModelPropertyName, null);
            if (graphModel === null) {
                throw { name: 'Exception', message: "createGraphModel returned null on " + this +
                          " for subgraph: " + subgraph
                };
            }
            if (graphModel.isDisposed()) {
                throw { name: 'Exception', message: "createGraphModel returned a disposed model: " +
                          graphModel + " on " + this +
                          " for subgraph: " + subgraph
                };
            }
			var parentGraphModel = graphModel.getParent();
            if (parentGraphModel !== null && !parentGraphModel.isNode(subgraph)) {
                throw { name: 'ArgumentException', message: "The subgraph must be a node in the parent graph model" };
            }
            this.storeGraphModel(subgraph, graphModel);
        } else if (graphModel.isDisposed()) {
            throw { name: 'Exception', message: "The graph model " + graphModel + " stored for " +
                            subgraph + " on " + this + " is disposed"
            };
        }
        return graphModel;
    },
    createGraphModel: function(subgraph) { },
    getNodes: function() { },
    getLinks: function() { },
    getSubgraphs: function() { return []; },
    getIntergraphLinks: function() { return []; },
    getLinksFrom: function(node) { },
    getLinksTo: function(node) { },
    getNodesCount: function() { return this.getNodes().length; },
    getLinksCount: function() { return this.getLinks().length; },
    getIntergraphLinksCount: function() { return this.getIntergraphLinks().length; },
    getSubgraphsCount: function() { return this.getSubgraphs().length; },
    getLinksFromCount: function(node) { return this.getLinksFrom(node).length; },
    getLinksToCount: function(node) { return this.getLinksTo(node).length; },
    getFrom: function(link) { },
    getTo: function(link) { },
    isNode: function(o) { },
    isLink: function(o) { },
    isIntergraphLink: function(o) { return false; },
    isSubgraph: function(o) { return false; },
    isLayoutNeeded: function(layout) { return true; },
    beforeLayout: function(layout) { },
    afterLayout: function() { },
	
    setParameter: function(nodeOrLink, key, val) {
        if (arguments.length == 3) {
            if (val) {
                nodeOrLink[key] = val;
            } else {
                delete nodeOrLink[key];
            }
        } else {
			val = key;
			key = nodeOrLink;
            if (val) {
                this[key] = val;
            } else {
                delete this[key];
            }
        }
    },
    getParameter: function(nodeOrLink, key) {
        if (arguments.length == 2) {
            return nodeOrLink[key];
        } else {
			key = nodeOrLink;
            return this[key];
        }
    },
	
    contentsChanged: function(sender, args) { },

    moveNode: function(node, x, y) { },
    boundingBox: function(nodeOrLink) { },
    getLinkPoints: function(link) { },
	getLinkPointAt: function(link, index) { return this.getLinkPoints()[index]; },
    reshapeLink:function(link, style, fromPoint, fromPointMode,
                         points, startIndex, length,
                         toPoint, toPointMode) { },
    hasMoveableConnectionPoint: function(link, origin) { return true; },
    hasPinnedConnectionPoint: function(link, origin) { return false; },
    getLinkWidth: function(link) { return 1; },
    
	setContentsAdjusting : function() {},
	
	saveAndEnableViewCoordinates: function(rootModel){},
	restoreAndDisableViewCoordinates: function(){},
	
	getTransform: function(subModel) { return null; }
});
});
},
'ibm_ilog/diagram/widget/OverviewHudComponent':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/OverviewHudComponent", [
"../main",
"dojo/_base/declare",
"dojo/_base/connect",
"dojox/gfx/matrix",
"./HudComponent",
"../overview/Overview",
"../overview/OverviewZoomInteractor",
"../overview/OverviewPanInteractor",
"../interactors/InteractorManager",
"../util/Geometry"
], function(
iid,
declare,
connect,
matrix,
HudComponent,
Overview,
OverviewZoomInteractor,
OverviewPanInteractor,
InteractorManager,
g
){

/*=====
var HudComponent = ibm_ilog.diagram.widget.HudComponent;
=====*/

var OverviewHudComponent =
	declare('ibm_ilog.diagram.widget.OverviewHudComponent', HudComponent, {

		//
		//	summary:
		//		A Viewport component holding an overview.
		//

		//
		//	_interactors: ibm_ilog.diagram.interactors.InteractorManager
		//		The InteractorManager used for the interactions in the component
		//
		_interactors: null,

		//
		//	_background: dojox.gfx.Rect
		//		The background Rect shape.
		//
		_background: null,

		//
		//	_size: dojox.gfx.Rect
		//		The background Rect shape.
		//
		_size: g.Size(0, 0),

		//
		//	_overview: ibm_ilog.diagram.overview.Overview
		//		The Overview object used in the component.
		//
		_overview: null,

		constructor: function (viewport, content, renderer, size) {
			//
			//	summary:
			//		Construct the component in the given viewport, with the given renderer, with the given size.
			//
			this._size = size;
			this._shape.setTransform(matrix.translate(iid._invisibleLocation, iid._invisibleLocation));
			this._background = this._shape.createRect(size).setFill([0, 0, 0, 0.5]);

			this._overview = new Overview(viewport, content, renderer, this._shape.createGroup(), this._size);
			this._viewport = viewport;
			this._update();

			this._interactors = new InteractorManager();
			this._interactors.add("pan", new OverviewPanInteractor().initialize(viewport, this._overview, viewport.getSurface()));
			this._interactors.add("zoom", new OverviewZoomInteractor().initialize(viewport, this._overview, viewport.getSurface()));
			this._interactors.switchTo("pan", "zoom");

			connect.connect(viewport, "onClientSizeChanged", this, "_update");
			connect.connect(viewport, "onLayoutHud", this, "_update");
		},
		
		
		setPan: function( b ) {
			//
			//	summary:
			//		Activate or deactivate panning in the overview
			//
			this._interactors.get("pan").setEnabled(b);
		},
		
		getPan: function() {
			//
			//	Returns:
			//		True if the receiver is activated for panning
			//
			this._interactors.get("pan").getEnabled();
		},
		
		setZoom: function( b ) {
			//
			//	summary:
			//		Activate or deactivate zooming in the overview
			//
			this._interactors.get("zoom").setEnabled(b);
		},
		
		getZoom: function() {
			//
			//	Returns:
			//		True if the receiver is activated for zooming
			//
			this._interactors.get("zoom").getEnabled();
		},
		
		getOverview: function() {
			//
			//	returns: ibm_ilog.diagram.overview.Overview
			//			The Overview object
			//
			return this._overview;
		},

		_enable: function () {
			//
			//	summary:
			//			Enables the component and displays it.
			//
			this.inherited(arguments);
			this._overview.setEnabled(true);
			this._update();
		},

		_disable: function () {
			//
			//	summary:
			//			Disables the component and hides it.
			//
			this._overview.setEnabled(false);
			this.inherited(arguments);
		},

		setSize: function (size) {
			//
			//	summary:
			//		Resizes the component
			//
			this._size = size;
			this._update();
		},

		getSize: function () {
			//
			//	summary:
			//		Returns the size of the component.
			//
			return this._size;
		},

		_update: function () {
			//
			//	summary:
			//		Updates the size and location of the component after resize of the 
			//		viewport or the overview component.
			//
			if (this._enabled && this._viewport.isHudEnabled()) {
				var sz = this._size;
				var hsz = this._viewport.getHudSize();
				this._overview.setSize(sz);
				this._background.setShape(sz);

				var location = { y: hsz.height - sz.height, x:0 };
				
				if(!this._viewport.isRtl()){
					location.x = hsz.width - sz.width;
				}

				this._shape.setTransform(matrix.translate(location));
			}
		}

	});

return OverviewHudComponent;

});

},
'ibm_ilog/diagram/data/StoreApi':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/data/StoreApi", ["dojo/_base/lang", "dojo/aspect", "dojo/_base/array", "dojo/_base/Deferred"], 
function(lang, aspect, array, Deferred){
	var storeApi = {};
	
	// TODO: Change to this when AMD conversion is complete:
	var iidd = lang.getObject("ibm_ilog.diagram.data", true);
	iidd.StoreApi = storeApi;
	
	var eventMapApi1 = {
		// StoreApi events | dojo.data.api events
		'onNew': 'onNew',
		'onRemove': 'onDelete',
		'onUpdate': 'onSet',
		'onSave': 'save',
		'onRevert': 'revert'
	}
	var DojoData = {
		bridge: function(store){
		},
		// this = the store
		getValue: function(item, prop, defaultValue){
			return this.getValue(item, prop);
		},
		getValues: function(item, prop){
			return this.getValues(item, prop);
		},
		setValue: function(item, prop, value){
			this.setValue(item, prop, value);
		},
		getFeatures: function(){
			var f = this.getFeatures();
			return {
				canRead: f["dojo.data.api.Read"],
				identity: f["dojo.data.api.Identity"],
				canWrite: f["dojo.data.api.Write"],
				canNotify: f["dojo.data.api.Notification"]
			}
		},
		isItem: function(item){
			return this.isItem(item);
		},
		fetchItemByIdentity: function(args){
			return this.fetchItemByIdentity(args);
		},
		getLabelAttributes: function(item){
			return this.getLabelAttributes(item);
		},
		getLabel: function(item){
			return this.getLabel(item);
		},
		isItemLoaded: function(item){
			return this.isItemLoaded(item);
		},
		fetch: function(args){
			return this.fetch(args);
		},
		loadItem: function(args){
			return this.loadItem(args);
		},
		listen: function(args){
			var h = [];
			for (var p in args) {
				if (p !== 'scope' && args.hasOwnProperty(p)) {
					h.push(aspect.after(this, eventMapApi1[p], lang.hitch(args.scope, args[p]), true));
				}
			}
			return {
				remove: function(){
					removeListenHandlers(h);
				}
			};
		}
	};
	
	var DojoStore = {
		bridge: function(store){
			store.__iid_dataAPIObservers = {};
		},
		// this = the store
		getValue: function(item, prop, defaultValue){
			return prop in item ? item[prop] : defaultValue;
		},
		getValues: function(item, prop){
			var val = item[prop];
			return val instanceof Array ? val : val === undefined ? [] : [val];
		},
		setValue: function(item, prop, value){
			item[prop] = value;
		},
		getFeatures: function(){
			return {
				canRead: !!this.get,
				identity: true,
				canWrite: !!this.put,
				canNotify: !!this.notify
			};
		},
		isItem: function(item){
			return item && !(item instanceof Array || typeof item == 'array') && !(item instanceof Date);
		},
		fetchItemByIdentity: function(args){
			var item;
			Deferred.when(this.get(args.identity), function(result){
				item = result;
				if (args.onItem) 
					args.onItem.call(args.scope, result);
			}, function(error){
				if (args.onError) 
					args.onError.call(args.scope, error);
			});
			return item;
		},
		getLabelAttributes: function(item){
			return [this.labelProperty];
		},
		getLabel: function(item){
			return this.labelProperty ? item[this.labelProperty] : undefined;
		},
		isItemLoaded: function(item){
			return true; // ?
		},
		fetch: function(args){
			args = lang.delegate(args, args && args.queryOptions);
			var self = this;
			var scope = args.scope || self;
			var query = args.query;
			var results = this.query(query, args);
			Deferred.when(results.total, function(totalCount){
				Deferred.when(results, function(results){
					if (typeof results.observe === 'function') {
						results.observe(lang.hitch(self, observableListener), true);
					}
					if (args.onComplete) {
						args.onComplete.call(scope, results, args);
					}
					return results;
				}, errorHandler);
			});
			function errorHandler(error){
				if (args.onError) {
					args.onError.call(scope, error, args);
				}
			}
		},
		loadItem: function(args){
			// TODO: how to know it has been loaded ?...
			var item = args.item;
			if (item && args.onItem) {
				args.onItem.call(args.scope, args.item);
			}
			return item;
		},
		listen: function(args){
			if (typeof this.notify !== 'function') 
				return;
			var h = [], f;
			for (var p in args) {
				if (p !== 'scope' && args.hasOwnProperty(p)) {
					if (!this.__iid_dataAPIObservers[p]) 
						this.__iid_dataAPIObservers[p] = [];
					pushHandler(this.__iid_dataAPIObservers[p], args[p], args.scope, h);
				}
			}
			return {
				remove: function(){
					removeListenHandlers(h);
				}
			}
		}
	};
	
	var removeListenHandlers = function(h){
		array.forEach(h, function(i){
			i.remove();
		});
	};
	
	var pushHandler = function(dest, f, scope, h){
		(function(){
			f = lang.hitch(scope, f);
			dest.push(f);
			h.push({
				remove: function(){
					arrayRemove(dest, array.indexOf(dest, f));
				}
			});
		})();
	};
	
	var arrayRemove = function(array, from, to){
		var rest = array.slice((to || from) + 1 || array.length);
		array.length = from < 0 ? array.length + from : from;
		return array.push.apply(array, rest);
	};
	
	var observableListener = function(object, removedFrom, insertedInto){
		if (removedFrom == -1 && this.__iid_dataAPIObservers.onNew && this.__iid_dataAPIObservers.onNew.length > 0) {
			array.forEach(this.__iid_dataAPIObservers.onNew, function(l){
				l(object);
			});
		} else if (insertedInto == -1 && this.__iid_dataAPIObservers.onRemove && this.__iid_dataAPIObservers.onRemove.length > 0) {
			array.forEach(this.__iid_dataAPIObservers.onRemove, function(l){
				l(object);
			});
		} else if (insertedInto !== -1 && removedFrom !== -1 && this.__iid_dataAPIObservers.onUpdate && this.__iid_dataAPIObservers.onUpdate.length > 0) {
			array.forEach(this.__iid_dataAPIObservers.onUpdate, function(l){
				l(object);
			});
		}
		
	};
	
	
	lang.mixin(storeApi, {
		bridge: function(store){
			store._iid_dataAPI = typeof store.get === 'function' ? DojoStore : DojoData;
			store._iid_dataAPI.bridge(store);
		},
		dispose: function(store){
			store._iid_dataAPI = null;
			delete store._iid_dataAPI;
		},
		fromDojoStore: function(store){
			return typeof store.get === 'function';
		},
		fromDojoData: function(store){
			return typeof store.getFeatures === 'function';
		},
		getValue: function(store, item, p){
			return store._iid_dataAPI.getValue.call(store, item, p);
		},
		getValues: function(store, item, p){
			return store._iid_dataAPI.getValues.call(store, item, p);
		},
		setValue: function(store, item, p, value){
			store._iid_dataAPI.setValue.call(store, item, p, value);
		},
		getIdentity: function(store, item){
			return store.getIdentity(item);
		},
		getFeatures: function(store){
			return store._iid_dataAPI.getFeatures.call(store);
		},
		isItem: function(store, item){
			return store._iid_dataAPI.isItem.call(store, item);
		},
		fetchItemByIdentity: function(store, args){
			return store._iid_dataAPI.fetchItemByIdentity.call(store, args);
		},
		getLabelAttributes: function(store, item){
			return store._iid_dataAPI.getLabelAttributes.call(store, item);
		},
		getLabel: function(store, item){
			return store._iid_dataAPI.getLabel.call(store, item);
		},
		isItemLoaded: function(store, item){
			return store._iid_dataAPI.isItemLoaded.call(store, item);
		},
		fetch: function(store, args){
			return store._iid_dataAPI.fetch.call(store, args);
		},
		loadItem: function(store, args){
			return store._iid_dataAPI.loadItem.call(store, args);
		},
		listen: function(store, args){
			return store._iid_dataAPI.listen.call(store, args);
		}
		
	});
	return storeApi;
});

},
'ibm_ilog/diagram/interactors/AdornerFocusInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/AdornerFocusInteractor", [
"dojo/_base/declare", "dojo/_base/connect", "dojo/_base/event",
"./Interactor",
"../util/Geometry", "../util/GraphUtil"
],
function(
declare, connect, event,
Interactor,
g, gu
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var AdornerFocusInteractor =
declare("ibm_ilog.diagram.interactors.AdornerFocusInteractor", [Interactor], {
	//
	//	summary:
	//		This interactor sets the diagram widget adorners focus 
	//		according to current mouse location. 
	//
	
	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//
	_diagram: null,

	_nodeMargin: 15,
	_linkMargin: 15,

	_lowCpuMode: null,

	_lastPosition: null,
	
	initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._initialize();
		this._lowCpuMode = this._diagram.isLowCpuMode();
		
		connect.connect(this._diagram, 'lowCpuModeChange', this, function () {
			this._lowCpuMode = this._diagram.isLowCpuMode();
			this.deactivate();
			this._initialize();
		});
		
		this._lastPosition = {
			x: 0,
			y: 0
		};
		return this;
	},

	_declareStates: function () {
		if (this._lowCpuMode == true) {
			this._declareState("idle", ["nodeover", "linkover", "subgraphover", "testfreecursor","onLeaveSurface"]);
		} else {
			this._declareState("idle", ["detectOnOver","onLeaveSurface"]);
		}
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		return {
			nodeover: {
				src: this._diagram,
				srcEvt: "onNodeMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			linkover: {
				src: this._diagram,
				srcEvt: "onLinkMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			subgraphover: {
				src: this._diagram,
				srcEvt: "onSubgraphMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			testfreecursor: {
				gfxConnect: true,
				src: this._viewport.getEventSource(),
				srcEvt: "onmousemove",
				connectTo: "_testFreeCursor"
			},
			detectOnOver: {
				src: this._viewport.getEventSource(),
				srcEvt: "onmousemove",
				connectTo: "_detectOnOver",
				gfxConnect: true
			},
			onLeaveSurface: {
				src: this._diagram.domNode,
				srcEvt: "onmouseleave",
				connectTo: "_onLeaveSurface"
			}
		};
	},
	
	_onLeaveSurface: function(e) {
		// Check if we are completely outside of the surface,
		// in that case we get no move event so we must free the adorners.
		var out = false;
		var l = this._viewport.eventClientLocation(e);
		if(l.x <= 0 || l.y <= 0){
			out = true;
		} else {
			var d = this._viewport.getClientSize();
			if(l.x >= d.width || l.y >= d.height)
				out = true;
		}
		if(out){
			this._freeCursor();
			this._freeLinkCursor();
		}
	},

	_setNodeAdornersFocus: function (ge, e) {
		// Summary:
		//		Sets the appropriate cursor, according to the ge element. Release the previous cursor, id applied.
		//		
		var cursorIsFree = this._diagram.getNodeAdornersFocus() == null;
		if (!ge._isIBMDiagramLink && (cursorIsFree || gu.isContainedBy(ge, this._diagram.getNodeAdornersFocus()))) {
			this._freeLinkCursor(null);
			this._diagram.setNodeAdornersFocus(ge);
			event.stop(e);
		} else if (ge._isIBMDiagramLink) {
			this._freeCursor(null);
			this._diagram.setLinkAdornersFocus(ge);
			event.stop(e);
		}
	},

	onEnabled: function () {
		this._goStateId("idle");
	},

	onDisabled: function () {
		this._goState(null);
		this._freeCursor();
	},

	_freeCursor: function (e) {
		// Summary:
		//		Free the node/subgraph cursor
		//
		this._diagram.setNodeAdornersFocus(null);
	},

	_freeLinkCursor: function (e) {
		// Summary:
		//		Free the link cursor
		//
		this._diagram.setLinkAdornersFocus(null);
	},

	_testFreeCursor: function (e) {
		// Summary:
		//		Free the cursor testing the bounding box (adding a margin)
		//
		var D = this._viewport.getDiagram();
		var c = this._diagram.getNodeAdornersFocus();
		if (c) {
			var outOfBounds;
			var bb = g.expandRect(c.getBounds(this._viewport.getSurface()), this._nodeMargin);
			var l = this._viewport.eventClientLocation(e);
			outOfBounds = !g.containsPoint(bb, l);
			if (D.mustFreeAdornersFocus(c, p, this._nodeMargin, outOfBounds)) {
				this._freeCursor();
			}
		}
		c = this._diagram.getLinkAdornersFocus();
		if (c) {
			var V = this._viewport;
			var p = V.eventContentLocation(e);
			outOfBounds = !c.hitTest(p, D.getGraph(), this._linkMargin);
			if (D.mustFreeAdornersFocus(c, p, this._linkMargin, outOfBounds)) {
				this._freeLinkCursor();
			}
		}
	},

	_detectOnOver: function (e) {
		//added to increase performance
		var current = {
			x: e.pageX,
			y: e.pageY
		};
		var delta = {
			x: Math.abs(current.x - this._lastPosition.x),
			y: Math.abs(current.y - this._lastPosition.y)
		};
		var minDelta = 2;
		if (delta.x < minDelta && delta.y < minDelta) {			
			return;
		}
		this._testFreeCursor(e);
		var p = this._viewport.eventContentLocation(e);
		var ges = this._diagram.getGraph().hitTest(p, 4);
		var ge = this._changeFocus(ges);
		if (ge) {
			this._setNodeAdornersFocus(ge, e);
		}
		this._lastPosition = {
			x: current.x,
			y: current.y
		};

	},
	_changeFocus: function (ges) {
		if (ges.length > 0) {
			var D = this._viewport.getDiagram();
			var anyCursor = this._diagram.getNodeAdornersFocus() || this._diagram.getLinkAdornersFocus();
			var changeAdornersFocus = true;
			// Adrian: now no longer depends on the selection state.
			//   Cf. Defect 2111.
			if (anyCursor && (/* anyCursor.isSelected() || */ anyCursor._isIBMDiagramLink)) {
				changeAdornersFocus = false;
			}

			var newge = ges[0];
			/* Adrian: now no longer depends on the selection state.
			   Cf. Defect 2111.
			for (var index in ges) {
				if (ges[index].isSelected()) {
					newge = ges[index];
					break;
				}
			}
			*/
			/* Adrian: Old comment already present before my changes for defect 2111.
			var newge = null;
			for(var index in ges){
				if(ges[index].isSelected()){
					newge = ges[index];
					break;
				}else{
					if(!newge && ges[index]._isIBMDiagramNode){
						newge = ges[index];
					}
				}
			}
			newge = newge?newge:ges[0];
			*/
			// Adrian: now no longer depends on the selection state.
			// Cf. defect 2111
			if (newge._isIBMDiagramLink /* && !newge.isSelected() */) {
				if (anyCursor && anyCursor._isIBMDiagramNode && !gu.isContainedBy(newge, anyCursor)) {
					changeAdornersFocus = false;
				}
			}

			return D.changeAdornersFocus(anyCursor, newge, ges, changeAdornersFocus);
		}
		return null;
	}
});

return AdornerFocusInteractor;

});

},
'ibm_ilog/diagram/overview/OverviewWidget':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewWidget", [
"../main",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/array",
"dojo/_base/sniff",
"dojo/_base/connect",
"dojo/dom-construct",
"dojo/dom-attr",
"dojo/dom-geometry",
"dijit/_Widget",
"dijit/registry",
"dojox/gfx",
"./Overview",
"./OverviewRenderer",
"./OverviewPanInteractor",
"./OverviewZoomInteractor",
"./OverviewTemplateRenderer",
"../interactors/InteractorManager",
"../util/DeferredInitialization",
"../util/ErrorReporter",
"../util/Geometry"
],
function(
iid,
declare,
lang,
array,
has,
connect,
domConstruct,
domAttr,
domGeometry,
_Widget,
registry,
gfx,
Overview,
OverviewRenderer,
OverviewPanInteractor,
OverviewZoomInteractor,
OverviewTemplateRenderer,
InteractorManager,
DeferredInitialization,
R,
g
){

var Size = g.Size;

/*=====
var DeferredInitialization = ibm_ilog.diagram.util.DeferredInitialization;
=====*/

var OverviewWidget =
declare('ibm_ilog.diagram.overview.OverviewWidget', [_Widget, DeferredInitialization],
{

	// summary:
	//		A widget that shows the overview of a Diagram.
	//
	// markup example:
	// |	<div dojoType="ibm_ilog.diagram.widget.Diagram" jsId="myDiagram"></div>
	// |	<div dojoType="ibm_ilog.diagram.overview.OverviewWidget" diagram="myDiagram"></div>
	//
	// markup example (custom renderer):
	//
	// |	<div dojoType="ibm_ilog.diagram.widget.Diagram" jsId="myDiagram"></div>
	// |	<div dojoType="ibm_ilog.diagram.overview.OverviewTemplateRenderer" graph="myDiagram.getGraph()" node="myNodeTemplate" jsId="renderer"></div>
	// |	<div dojoType="ibm_ilog.diagram.overview.OverviewWidget" diagram="myDiagram" renderer="renderer"></div>
	//
	
	//
	//	diagram: ibm_ilog.diagram.widget.Diagram
	//
	diagram: null,

	//
	//	renderer: ibm_ilog.diagram.overview.OverviewRenderer
	//		The renderer used for generating the overview image.
	//
	renderer: null,

	//
	//	viewRectFill: String|Object
	//		The fill of the rectangle used to represent the view rectangle
	//
	viewRectFill: null,
	
	//
	//	viewRectStroke: String|Object
	//		The stroke of the rectangle used to represent the view rectangle
	//
	viewRectStroke: null,

	//	
	//	_diagram: [private] ibm_ilog.diagram.widget.Diagram
	//		The diagram Widget
	//
	_diagram: null,
	
	//
	//	_overview: [private] ibm_ilog.diagram.overview.Overview
	//		The overview displayed in the widget surface
	//
	_overview: null,
	
  //
	//	_overviewGroup: [private] dojox.gfx.Group
	//		The group where the overview is placed in the surface
	//
	_overviewGroup: null,

	//
	//	_surface: dojox.gfx.Surface
	//		The surface
	//
	_surface: null,
	
    // ========================================================================================================
    //
    // WIDGET LIFECYCLE:
    //
    //	The following group methods define the process of construction, initialization, deinitialization and 
    //	related. These are mostly hooks into the Dijit framework.
    //

	templateString:  "<div dojoAttachPoint='canvasNode' style='overflow:hidden'></div>",

	buildRendering: function(){
		// there are no variables in the template so we can cache the DOM tree
		var node = domConstruct.toDom(this.templateString);
		if (node.nodeType != 1) {
			throw new Error("Invalid template: " + this.templateString);
		}
		this.domNode = node;
		var nodes = node.getElementsByTagName("*");
		for (var i = -1; i < nodes.length; i++) {
			var n = (i == -1) ? node : nodes[i];
			var attachPoint = n.getAttribute("dojoAttachPoint");
			this[attachPoint] = n;
		}
		
		//this.inherited(arguments);
		
		this._fillContent(this.srcNodeRef);
		
	},
	
	_fillContent: function(/*DomNode*/ source){
		// summary:
		//	Relocate source contents to templated container node.
		//	this.containerNode must be able to receive children, or exceptions will be thrown.
		// tags:
		//	protected
		var dest = this.containerNode;
		if (source && dest) {
			while (source.hasChildNodes()) {
				dest.appendChild(source.firstChild);
			}
		}
	},
	
	postMixInProperties: function() {
	
		this._beginInit();

    	this.inherited(arguments);

    	if(this.diagram._isIBMDiagramDiagram) {
    		this._diagram = this.diagram;
		} else if( (typeof (this.diagram)) == "string") {
			this._diagram = registry.byId(this.diagram);
		} else if(this.diagram instanceof HTMLElement) {
	    	this._diagram = registry.byNode(this.diagram);
		} else {
			R.error("OverviewDiagramMissing",this.diagram);
		}
    	
	},
	
	postCreate: function() {
	
		this.inherited(arguments);

    // compute initial size
    var csz = this._computeSize();
	
    if(!has("ie")) {
			this._surface = gfx.createSurface(this.canvasNode,csz.width+"px",csz.height+"px");
		} else {
			this._surface = gfx.createSurface(this.canvasNode,csz.width,csz.height);
		}        

		this._surface.whenLoaded(this,function(){

    	// create a group for hosting the overview
      this._overviewGroup = this._surface.createGroup();
		        
			this._diagram.whenLoaded(lang.hitch(this,function(){

			    // create a defaut renderer if the user has not defined one
	        this.renderer = this.renderer || new OverviewTemplateRenderer();
		        
			    // create an overview for our viewport with the just constructed arguments
	        this._overview = new Overview(this._diagram.getViewport(),this._diagram.getGraph(),this.renderer,this._overviewGroup,csz);
	    	
	        this._overview.setInteractorManager(this.diagram.getInteractorManager());
	        
	        if(this.viewRectStroke) {
	        	this._overview.setViewRectStroke(this.viewRectStroke);
	        }
	        
	        if(this.viewRectFill) {
	        	this._overview.setViewRectFill(this.viewRectFill);
	        }
	        
			    // create an interactor manager for the overview interactors
		    	this._interactors = new InteractorManager();
		    	
			    // create the pan interactor (which handles pan and cenderAt)
		    	this._interactors.add("pan",new OverviewPanInteractor().initialize(this._diagram.getViewport(),this._overview,this._surface));
	
		    	// create the zoom interactor
		    	this._interactors.add("zoom",new OverviewZoomInteractor().initialize(this._diagram.getViewport(),this._overview,this._surface));
	
		    	this._setupInteractors(["pan","zoom"]);
		    	
		    	// start the interactors
		    	this._interactors.switchTo("pan","zoom");
				
			    if (has("ie") > 7 && iid.isVml){
					// fix overview interaction issue in ie8 (ie8 document mode)
					var panInter = this._interactors.get("pan"),
						zoomInter = this._interactors.get("zoom");
					this._ie8Handler = [
						connect.connect(this._overview, "__ieBeforeEvtSrcChange", this, function(){
							array.forEach([panInter,zoomInter],function(inter){
								if (inter._state)
									inter._state.leave();
							});
						}),
						connect.connect(this._overview, "__ieAfterEvtSrcChange", this, function(){
							array.forEach([panInter,zoomInter],function(inter){
								if (inter._state)
									inter._state.enter();
							});
						}) 
					];
				}

		    	this._endInit();
	
		    }));
	    });
	},
	
	//
	//  pan: [markup] boolean
	//      enable or disable panning
	//
	pan: true,
	
	_setPanAttr: function( /*Boolean*/b ) {
		if (!this._viewportReady) 
			// at creation time, interactors are initialized in the postCreate stage.
			// Since customSetter are called before postCreate(), skip it in this case.
			return; 
		var old = this._getPanAttr(); // use the getter as the interactor may have been configured directly
		if (old !== b) {
			this.pan = b;
			this._interactors.get('pan').setEnabled(b);
			this._firePropertyChanged('pan', old, b);
		}
		return this;
	},
	
	_getPanAttr: function() {
		return this._viewportReady ? this._interactors.get("pan").getEnabled() : true;
	},
	
	//
	//  constrainPan: [markup] boolean
	//		Constrains view rectangle movements to the overview area
	//
	constrainPan: true,
	
	_setConstrainPanAttr: function( /*Boolean*/b ) {
		if (!this._viewportReady) 
			// at creation time, interactors are initialized in the postCreate stage.
			// Since customSetter are called before postCreate(), skip it in this case.
			return; 
		var old = this._getPanAttr(); // use the getter as the interactor may have been configured directly
		if (old !== b) {
			this.constrainPan = b;
			this._interactors.get('pan').setConstrain(b);
			this._firePropertyChanged('constrainPan', old, b);
		}
		return this;
	},
	
	_getConstrainPanAttr: function() {
		return this._viewportReady ? this._interactors.get("pan").getConstrain() : true;
	},

	
	//
	//  zoom: [markup] boolean
	//      enable or disable zooming
	//
	zoom: true,
	
	_setZoomAttr: function( /*Boolean*/b ) {
		if (!this._viewportReady) 
			// at creation time, interactors are initialized in the postCreate stage.
			// Since customSetter are called before postCreate(), skip it in this case.
			return; 
		var old = this._getZoomAttr();
		if (old !== b) {
			this.zoom = b;
			this._interactors.get("zoom").setEnabled(b);
			this._firePropertyChanged('zoom', old, b);
		}
		return this;
	},
	
	_getZoomAttr: function() {
		return this._viewportReady ? this._interactors.get("zoom").getEnabled() : true;
	},
	
	_setupInteractors: function(ids) {
		array.forEach(ids, function(i) {
			this._interactors.get(i).setEnabled(this[i]);
		}, this);
		
		this._interactors.get("pan").setConstrain(this.constrainPan);
		
	},

	destroy: function() {

		this._interactors.switchTo();
		this._interactors = null;

		this._overview = null;

		this._surface.remove(this._overviewGroup);
		this._overviewGroup = null;

		this._surface.destroy();
		this._surface = null;
		
		if (this._ie8Handler){
			array.forEach(this._ie8Handler, function(l){
				connect.disconnect(l);
			});
		}
        
        this.inherited(arguments);

	},

	resize: function( /*{w:h:}*/ sz ) {
		//
		//	summary:
		//		resize accordingly when our container specifies so
		//
		this.whenLoaded(lang.hitch(this,function(){
			var style = {width:sz.w+"px",height:sz.h+"px"};
			domAttr.set(this.domNode,{"style":style});
			if(!has("ie")) {
				this._surface.setDimensions(style.width,style.height);
			} else {
				this._surface.setDimensions(sz.w,sz.h);
			}
			this._overview.setSize({width:sz.w,height:sz.h});
		}));
	},
	
    // ========================================================================================================

	_computeSize: function() {
		//
		//	summary:
		//		decide what size to use for initializing the overview
		//
		var b = domGeometry.getContentBox(this.domNode);
		var c = this.getParent()?domGeometry.getContentBox(this.getParent().containerNode):{};
		return Size(b.w || c.w || b.h || 256, b.h || c.h || b.w || 256);
	},
	
	getParent: function(){
		// summary:
		//		Returns the parent widget of this widget, assuming the parent
		//		specifies isContainer
		var parent = registry.getEnclosingWidget(this.domNode.parentNode);
		return parent && parent.isContainer ? parent : null;
	},
	
	getDiagram: function() {
		return this._diagram;
	},
	
	setDiagram: function(diagram) {
		if(this._diagram!=diagram) {
			this._diagram = diagram;
			var o = this._overview;
			if(o){
				var enabled = o.isEnabled(); 
				o.setEnabled(false);
				o.setContent(diagram.getGraph());
				o.setViewport(diagram.getViewport());
				o.setEnabled(enabled);
				o.generateContent();
			}
		}
	},

	getOverview: function() {
		return this._overview;
	},	

	getRenderer: function() {
		return this.renderer;
	},
	
	setRenderer: function(r) {
		this._overview.setRenderer(r);
		this._overview.generateContent();
	}
});	

return OverviewWidget;

});

},
'ibm_ilog/diagram/util/Geometry':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/Geometry", [
	"dojo/_base/lang",
	"dojo/has!config-diagramForMobile?:dojo/number",	// #1 (for dojo.number.round)
	"./ErrorReporter"	// #2
	],
	function(
		lang,
		NumberUtils, // #1
		ErrorReporter	// #2
		) {

	// TODO: Change to this when AMD conversion is complete:
	// var g = {};
	var g = lang.getObject("ibm_ilog.diagram.util.Geometry", true);
	
	//
	//	Representation for the empty rectangle
	//		
	g.EmptyRect = { x: 0, y: 0, width: 0, height: 0 };

	
  g.isEmptyRect = function( /*dojox.gfx.Rectangle*/ r ) {
	//	returns:
	//		true when r is the empty rectangle.
      return r.x === 0 && r.y === 0 && r.width === 0 && r.height === 0;
  };

  g.equalRect = function( /*dojox.gfx.Rectangle*/ r1, /*dojox.gfx.Rectangle*/ r2) {
  	//	returns:
  	//		true when the two rects are equal
      return (r1.x === r2.x) &&
         (r1.y === r2.y) &&
         (r1.width === r2.width) &&
         (r1.height === r2.height);
  };

	g.cloneRect = function ( /*dojox.gfx.Rectangle*/ r) {
    	return {
    		x:r.x,
    		y:r.y,
    		width:r.width,
    		height:r.height
    		};
		};
		
  g.Rect = function(x,y,w,h) {
  	//	x: Number
  	//	y: Number
  	//	w: Number
  	//	h: Number
  	var r = {};

  	if(x!==undefined) {
  		r.x = x;
  	}
  	
  	if(y!==undefined) {
  		r.y = y;
  	}
  	
  	if(w!==undefined) {
  		r.width = w;
  	}
  	
  	if(h!==undefined) {
  		r.height = h;
  	}
  	
  	return r;
  };
  
  g.standarizeRect = function(r) {
  	//	r:	dojox.gfx.Rectangle
  	return {
  		x:r.x||0,
  		y:r.y||0,
  		width:r.width||r.w||0,
  		height:r.height||r.h||0
  		};
  };
  
  g.rectAssign = function(a,b) {
  	a.x = b.x;
  	a.y = b.y;
  	a.width = b.width;
  	a.height = b.height;
  };
  
  g.rectIncomplete = function( r ) {
  	//	r:	dojox.gfx.Rectangle
      return r.x===undefined || r.y===undefined || r.width===undefined || r.height===undefined;
  };
  
  g.getRectCenter = function( r ) {
  	//	r:	dojox.gfx.Rectangle
  	//	returns:
  	//		a point equal to the center of the rectangle r
      if(g.rectIncomplete(r)) {
      	return null;
      } else {
      	return { x: r.x + r.width / 2, y: r.y + r.height / 2 };
      }
  };

  g.rectCenteredAt = function( r, p, center ) {
  	//	r:	dojox.gfx.Rectangle
  	//	p:	dojox.gfx.Point
  	//	center: dojox.gfx.Point? 
  	//	returns:
  	//		a copy of r but centered at p
  	center = center || { x:r.width / 2, y:r.height / 2 };
      return { x: p.x - center.x, y: p.y - center.y, width:r.width, height:r.height};
  };

  g.rectIntersects = function(rect1,rect2) {
      return (rect1.x < (rect2.x + rect2.width)) &&
             (rect2.x < (rect1.x + rect1.width)) &&
             (rect1.y < (rect2.y + rect2.height)) &&
             (rect2.y < (rect1.y + rect1.height));        
  };
  
  g.addRect = function( /*dojox.gfx.Rectangle*/ a, /*dojox.gfx.Rectangle*/ b) {
  	//	returns:
  	//		a rectangle representing the addition of the two given.
      if (a === null && b === null) {
          return null;
      }
      if (a === null && b !== null) {
          return b;
      }
      if (b === null) {
          return a;
      }
      var single1 = Math.min(a.x, b.x);
      var single2 = Math.max(a.x + a.width, b.x + b.width);
      var single3 = Math.min(a.y, b.y);
      var single4 = Math.max(a.y + a.height, b.y + b.height);
      return { x: single1, y: single3, width: single2 - single1, height: single4 - single3 };
  };
  
  g.createRect = function(p1,p2){
  	var x = Math.min(p2.x,p1.x);
      var y = Math.min(p2.y,p1.y);
      var width = Math.abs(p2.x-p1.x);
      var height = Math.abs(p2.y-p1.y);
  	return {x:x,y:y,width:width,height:height};
  };
  
  g.containsRect = function(cr,r){
  	if(cr.x<=r.x && cr.y<=r.y 
  			&& (cr.width+cr.x)>=(r.x+r.width) && (cr.height+cr.y)>=(r.y+r.height)){
  		return true;
  	}
  	return false;
  };
  
  g.clipRect = function(cr,r){
  	// cr: clipping rectangle
  	// r: clipee
  	var res = {
  		x:Math.min(Math.max(cr.x,r.x),cr.x+cr.width),
  		y:Math.min(Math.max(cr.y,r.y),cr.y+cr.height)
  	};
  	
  	res.width = Math.max(0,Math.min(cr.x+cr.width,r.x+r.width)-res.x);
  	res.height= Math.max(0,Math.min(cr.y+cr.height,r.y+r.height)-res.y);

  	return res;
  };

  g.moveRect = function( /*dojox.gfx.Rectangle*/ r, /*dojox.gfx.Point*/ delta ) {
  	return { x: (r.x||0)+delta.x,
		     y: (r.y||0)+delta.y,
		 width: r.width,
		height: r.height };
  };
  
  g.relocateRect = function( /*dojox.gfx.Rectangle*/ r, /*dojox.gfx.Point*/ p ) {
  	return { x: p.x,
		     y: p.y,
		 width: r.width,
		height: r.height };
  };
  
  g.growRect = function( /*dojox.gfx.Rectangle*/ r, /*ibm_ilog.diagram.util.Geometry.Size*/ minSize ) {
  	return { x: r.x,
	         y: r.y,
	     width: Math.max(r.width,minSize.width),
	    height: Math.max(r.height,minSize.height) };
	};

	g.expandRect = function(rect,l,t,r,b) {
		switch(arguments.length) {
		case 2:
			t = r = b = l;
			break;
		case 3:
			b = t;
			r = l;
			break;
		case 5:
			break;
		default:
			ErrorReporter.error("InvalidNumberOfArguments",arguments,2,3,5);
			
			break;
		}
    	return { x: rect.x-l,
		         y: rect.y-t,
		     width: rect.width+l+r,
		    height: rect.height+t+b };
	};
	
	g.printRect = function(r) {
		var f = function (n){ return g._round(n,1); };
		return f(r.width)+" x "+f(r.height)+": ["+f(r.x)+","+f(r.x+r.width)+"] x ["+f(r.y)+","+f(r.y+r.height)+"]";
	};
    
	g.rectEqualSize = function(r1,r2,resolution) {
  	return Math.abs(r1.width - r2.width) < resolution && Math.abs(r1.height - r2.height) < resolution;
  };
  
  g.rectEqualOrigin = function(r1,r2,resolution) {
  	return Math.abs(r1.x - r2.x) < resolution && Math.abs(r1.y - r2.y) < resolution;
  };
  
  g._rangeCenteredOverride = function(result,oldv,newv,x,l) {
  	//
  	//	summary:    	
  	//		Returns new range based on the overriding of a base range with override values from newr.
  	//		Used as a helper function for rectCenteredOverride.
  	//
  	//	tags:
  	//		private
  	//
  	//	description:
  	//		Receives an incomplete range newr and assigns its values to oldr, returning the result. 
  	//		It does not change the original range. The semantics are fairly straighforward except for
  	//		one case: if the newr specifies a new [w] and does not specify the corresponding
  	//		[x], then the resulting rectangle is resized centered, instead of keeping its original 
  	//		location.
  	//
  	//	returns: ibm_ilog.diagram.Rectangle
  	//		The result 
  	//	result: ibm_ilog.diagram.Rectangle
  	//		Where the result will be stored 
  	//	newr: ibm_ilog.diagram.Rectangle
  	//		The incomplete rectangle that holds the new values
  	//	oldr: ibm_ilog.diagram.Rectangle
  	//		The fake asignee it will not be changed
  	//	x:	String
  	//		The name of the attribute representing the range origin
  	//	w:	String
  	//		The name of the attribute representing the range length
  	//
	if( newv[l] === undefined ) {
    	if( newv[x] === undefined ) {
    		result[x]=oldv[x];
    	} else {
  			result[x]=newv[x];
    	}
  		result[l]=oldv[l];
	} else {
    	if( newv[x] === undefined ) {
    		// keeps range center
    		result[x]=oldv[x]-(newv[l]-oldv[l])/2;
			
    		// TODO use pivot instead:
    		//
			//if(!newv.scalePivot)
			//	newv.scalePivot = {};
			//var p = newv.scalePivot[x] = newv.scalePivot[x] || (newv[x]+(newv[l]/2));
			//
			//result[x]= p-(newv[w]/oldv[w])*(p-oldv[x]);

			
    	} else {
  			result[x]=newv[x];
    	}
		result[l]=newv[l];
	}    	
  	return result;
  };
  
  g.rectCenteredOverride = function(oldr,newr) {
  	//
  	//	summary:    	
  	//		Returns new rectangle based on the overriding of a base rectangle with override values from newr.
  	//
  	//	description:
  	//		Receives an incomplete rectangle newr and assigns its values to oldr, returning the result. 
  	//		It does not change the original rectangle. The semantics are fairly straighforward except for
  	//		one case: if the newr specifies a new width or height and does not specify the corresponding
  	//		x or y, then the resulting rectangle is resized centered, instead of keeping its original 
  	//		location.
  	//
  	//	newr:
  	//		The incomplete rectangle that holds the new values
  	//	oldr:
  	//		The fake asignee it will not be changed
  	//
  	var result = newr.scalePivot?{scalePivot:newr.scalePivot}:{};
  	this._rangeCenteredOverride(result,oldr,newr,'y','height');
  	this._rangeCenteredOverride(result,oldr,newr,'x','width');
  	return result;
  };
  
  g.constrainRect = function(r,area,resize) {
  	var w = resize?Math.min(r.width,area.width):r.width;
  	var h = resize?Math.min(r.height,area.height):r.height;
		return {
			x:      Math.max(Math.min(r.x,area.x+area.width-w),area.x),
			y:      Math.max(Math.min(r.y,area.y+area.height-h),area.y),
			width:  w,
			height: h
		};
  };
  
  g.visitMembers = function(obj,visitor) {
  	// probably should be moved to a generic set of utils
	var r = {};
	for( _i in obj ) {
		r[_i]=visitor(obj[_i]);
	}
	return r;
  };
  
  g.getRectRatio = function(r) {
  	return 	r.width/r.height;
  };
  
  g.rectBlend = function(r1,r2,factor) {
	var cx = r1.x + (r2.x - r1.x) * factor;
	var cy = r1.y + (r2.y - r1.y) * factor;
	var cwidth = r1.width + (r2.width - r1.width) * factor;
	var cheight = r1.height + (r2.height - r1.height) * factor;
	return {x:cx,y:cy,width:cwidth,height:cheight};
  };
  
  g.containsPoint = function(r,p) {
  	return r.x<=p.x && r.y<=p.y && p.x<=(r.x+r.width) && p.y<=(r.y+r.height);
  };

  
/*=====
	dojo.declare("ibm_ilog.diagram.util.Geometry.Size", null, {
		// summary:
		//		An object with 'width' and 'height' properties used as argument for various ibm_ilog.diagram.util.Geometry methods.
		
		constructor: function( width, height ) {
			// width: Number
			// height: Number
			this.width = width;
			this.height = height;
		}
	});
=====*/
    
  g.printSize = function(p) {
  	//	p: ibm_ilog.diagram.util.Geometry.Size
  	return "("+p.width+","+p.height+")"
  };

  g.sizeLongestSide = function( /*ibm_ilog.diagram.util.Geometry.Size*/ sz ) {
  	return Math.max(sz.width,sz.height);
  };

  g.visitSize = function( /*Function*/ op, /*ibm_ilog.diagram.util.Geometry.Size*/ size ) {
  	return op(size.width,size.height);
  };

  g.isNullSize = function( /*dojox.gfx.Rectangle*/ r ) {
  	//
  	//	summary:
	//		returns true when r is a "null" rectangle: width and height are 0 or undefined
  	//
      return !r.width && !r.height;
  };

  g.Size = function(width,height) {
  	return {width:width,height:height};
  };

  g.equalSize = function(p1, p2) {
  	//	returns:
  	//		true when the two rects are equal
      return (p1.width === p2.width) &&
         (p1.height === p2.height);
  };
  
  g.floorSize = function( sz ) {
  	return {width:Math.floor(sz.width),height:Math.floor(sz.height)};
  };

  g.mulSize = function(p1, s) {
    return { width: p1.width*s, height: p1.height*s };
  };

  g.Point = function( /*Number*/ x, /*Number*/ y ) {
  	// x: Number
  	// y: Number
  	// returns: dojox.gfx.Point
  	return {x:x,y:y};
  };

  g.floorPoint = function( p ) {
  	// p: dojox.gfx.Point
  	// returns: dojox.gfx.Point
  	return {x:Math.floor(p.x),y:Math.floor(p.y)};
  };
  
  g.printPoint = function(p) {
  	// p: dojox.gfx.Point
  	// returns: dojox.gfx.Point
  	return "("+p.x+","+p.y+")"
  };

  g.InvalidPoint = function() {
  	// returns: dojox.gfx.Point
  	return { x: Number.NEGATIVE_INFINITY, y: Number.POSITIVE_INFINITY };
  };

  g.EmptyPoint = function() {
  	// returns: dojox.gfx.Point
  	return { x: NaN, y: NaN };
  };
  
  g.PointZero = function() {
  	// returns: dojox.gfx.Point
  	return { x: 0, y: 0 };
  };
  
  g.getDistance = function(p1, p2) {
  	// p1: dojox.gfx.Point
  	// p2: dojox.gfx.Point
  	// returns: Number
      var a = p2.x - p1.x;
      var b = p2.y - p1.y;
      return Math.sqrt(a * a + b * b);
  };
  
  g.getPointToSegmentDistance = function(A,B,C){
  	// Summary:
  	//		return the distance between a segment and a point
  	//		A and B are the segment points
  	
  	// special case where A = B
  	if(this.equalPoint(A,B)){
  		return this.getDistance(A,C);
  	}
  	
  	var u = ((C.x-A.x)*(B.x-A.x)+(C.y-A.y)*(B.y-A.y))/(Math.pow(B.x-A.x,2)+Math.pow(B.y-A.y,2));
  	
  	// P is the nearest point of the segment line
  	var P = {};
  	
  	if(u>1){
  		// the point P is over the point B - distance = B-C
  		return this.getDistance(B,C);
  	}else{
  		if(u<0){
  			// the point P is behind the point A - distance = A-C
  			return this.getDistance(A,C);
  		}else{
  			// the point P is in between A and B - distance = C-P
  			P.x = A.x + u * ( B.x - A.x);
  			P.y = A.y + u * ( B.y - A.y);
  			return this.getDistance(P,C);
  		}
  	}
  };
  
  g.isInvalidPoint = function(p) {
  	// p: dojox.gfx.Point
  	// returns: Boolean
      return p.x === Number.NEGATIVE_INFINITY && p.y === Number.POSITIVE_INFINITY;
  };
  
  g.isEmptyPoint = function(p) {
  	// p: dojox.gfx.Point
  	// returns: Boolean
      return isNaN(p.x) && isNaN(p.y);
  };

  g.addPoint = function(p1, p2) {
  	//	p1: dojox.gfx.Point
  	//	p2: dojox.gfx.Point
  	//	returns: dojox.gfx.Point
      return { x: p1.x+p2.x, y: p1.y+p2.y };
  };

  g.subPoint = function(p1, p2) {
  	//	p1: dojox.gfx.Point
  	//	p2: dojox.gfx.Point
  	//	returns: dojox.gfx.Point
      return { x: p1.x-p2.x, y: p1.y-p2.y };
  };

  g.negPoint = function(p) {
  	//	p: dojox.gfx.Point
  	//	returns: dojox.gfx.Point
      return { x: -p.x, y: -p.y };
  };
  
  g.mulPoint = function(p1, s) {
  	//	p: dojox.gfx.Point
  	//	s: Number
  	//	returns: dojox.gfx.Point
      return { x: p1.x*s, y: p1.y*s };
  };

  g.equalPoint = function(p1, p2) {
  	//	p1: dojox.gfx.Point
  	//	p2: dojox.gfx.Point
  	//	returns: Boolean
  	//		true when the two rects are equal
    return (p1.x === p2.x) &&
       (p1.y === p2.y);
  };

//
//	MISCELANEOUS OPERATIONS
//
    
  g.getScale = function(x, y) {
  	//	x: Number
  	//	y: Number
  	//	returns: Number
		if (x === 0) {
		    return y;
		} else if (y === 0) {
		    return x;
		} else {
		    return Math.sqrt(x * x + y * y);
		}
  };
  
  g.getStrokeWidth = function(stroke, t) {
		var w = stroke.width || 1;
		var zoom = Math.min(t.xx, t.yy);
		var realWidth = w * zoom;
		//            if (_minWidth >= 0f && realWidth < _minWidth)
		//                realWidth = _minWidth;
		//            if (_maxWidth >= 0f && realWidth > _maxWidth)
		//                realWidth = _maxWidth;
		w = (zoom === 0) ? 0 : (realWidth / zoom);
		return w;
  };

  g._round = function(value, places) {
		if(NumberUtils && NumberUtils.round)
			return NumberUtils.round(value, places); // Number
		return (+value).toFixed(places)*1; // Number
  };
	
  return g;
});

},
'ibm_ilog/diagram/graphlayout/webworker/_LayoutProvider':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/webworker/_LayoutProvider", [
"require",
"dojo/_base/declare",
"ibm_ilog/graphlayout/ILayoutProvider",
"ibm_ilog/graphlayout/MultipleLayout",
"./_GraphAdapter"
], function(
require,
declare,
ILayoutProvider,
MultipleLayout,
_GraphAdapter
){

var layoutClasses = {
    circular: 		"ibm_ilog.graphlayout.circular.CircularLayout",
    forcedirected: 	"ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout",
    grid: 			"ibm_ilog.graphlayout.grid.GridLayout",
    hierarchical: 	"ibm_ilog.graphlayout.hierarchical.HierarchicalLayout",
    longlink: 		"ibm_ilog.graphlayout.longlink.LongLinkLayout",
    random: 		"ibm_ilog.graphlayout.random.RandomLayout",
    shortlink: 		"ibm_ilog.graphlayout.shortlink.ShortLinkLayout",
    tree: 			"ibm_ilog.graphlayout.tree.TreeLayout"
};

/*=====
var ILayoutProvider = ibm_ilog.graphlayout.ILayoutProvider;
=====*/

var _LayoutProvider =
declare("ibm_ilog.diagram.graphlayout.webworker._LayoutProvider", ILayoutProvider, {
  
  getGraphLayout : function(graphModel)
  {
     var graph = graphModel._graph;
     var nodeLayout = this._getLayout(graph, graph.nodeLayout);
     var linkLayout = this._getLayout(graph, graph.linkLayout);
     var res = null;
     if (nodeLayout == null && linkLayout == null)
       res = null; // This means no layout at all for the current graph model
     else if (nodeLayout == null)
       res = linkLayout;
     else if (linkLayout == null)
       res = nodeLayout;
     else
        res = new MultipleLayout(nodeLayout, linkLayout);
     
     if (res != null && res.getGraphModel() == null) 
       res.attach(new _GraphAdapter(graph));

     this._setObjProperties(graph.nodes, nodeLayout, linkLayout);
     this._setObjProperties(graph.links, nodeLayout, linkLayout);
     
     return res;
  },
  
  _getLayout : function(graph, layoutObj)
  {
    if(!layoutObj)
      return null;
    
    // already created and cached?
    var layout = layoutObj._layout;
    if(layout)
      return layout;
    
    var layoutClass = layoutClasses[layoutObj.algorithm];
	var constr;
	// loader MUST be in symchronous mode!
    require([layoutClass.replace(/\./g, '/')], function(c) { constr = c; });
    layout = new constr();
    
    // store for reuse
    layoutObj._layout = layout;
    
    // set layout properties
    this._setProperties(layout, layoutObj);
    
    // create hierarchical constraints
    if(layoutObj.constraints)
      this._createConstraints(layout, layoutObj.constraints, graph);
    
    return layout;
  },
  
  _setObjProperties : function(a, nodeLayout, linkLayout)
  {
    if(a){
      for(var i = 0; i < a.length; i++){
        var obj = a[i];
        if(nodeLayout && obj.parentNodeLayout)
          this._setProperties(nodeLayout, obj.parentNodeLayout, obj);
        if(linkLayout && obj.parentLinkLayout)
          this._setProperties(linkLayout, obj.parentLinkLayout, obj);
      }
    }
  },
  
  _setProperties: function(layout, props, obj)
  {
    var defltObj = {};
    for(var p in props){
      if(p == "algorithm" || p == "constraints" || p in defltObj)
        continue;
      var setter = "set" + p.charAt(0).toUpperCase() + p.substr(1);
      var f = layout[setter];
      if(typeof(f) == "function"){
        var v = props[p];
        // convert points and rectangles
        if(v instanceof Array){
          if(v.length == 2)
            v = { x:v[0], y:v[1] };
          else if(v.length == 4)
            v = { x:v[0], y:v[1], width:v[2], height:v[3] };
        }
        if(obj)
          f.call(layout, obj, v);
        else
          f.call(layout, v);
      }
    }
  },
  
  _createConstraints: function(layout, constraints, graph)
  {
    // find the id -> node hash, stored in the toplevel graph object
    var idToNode = graph._idToNode;
    while(!idToNode){
      graph = graph._parent;
      idToNode = graph._idToNode;
    }

    var ca = [];

    for(var i = 0; i < constraints.length; i++){
      var obj = constraints[i];
      var c = null;
      var className = "ibm_ilog.graphlayout.hierarchical.Hierarchical" + obj.name;
	  // loader MUST be in symchronous mode!
 	  var constr;
      require([className.replace(/\./g, '/')], function(c) { constr = c; });
      switch(obj.name){
      case "LevelRangeConstraint":
        c = new constr(this._getNodeGroup(obj.nodes, idToNode), obj.minLevel, obj.maxLevel);
        break;
      case "GroupSpreadConstraint":
        c = new constr(this._getNodeGroup(obj.nodes, idToNode), obj.spreadSize);
        break;
      case "RelativeLevelConstraint":
        c = new constr(this._getNodeGroup(obj.lowerNodes, idToNode), this._getNodeGroup(obj.higherNodes, idToNode), obj.priority);
        break;
      case "ExtremityConstraint":
        c = new constr(this._getNode(obj.node, idToNode), obj.side);
        break;
      case "RelativePositionConstraint":
        c = new constr(this._getNodeGroup(obj.lowerNodes, idToNode), this._getNodeGroup(obj.higherNodes, idToNode), obj.priority);
        break;
      case "SameLevelConstraint":
        c = new constr(this._getNode(obj.firstNode, idToNode), this._getNode(obj.secondNode, idToNode));
        break;
      case "SideBySideConstraint":
        c = new constr(this._getNodeGroup(obj.nodes, idToNode), obj.priority);
        break;
      case "SwimLaneConstraint":
        c = new constr(this._getNodeGroup(obj.nodes, idToNode), obj.relativeSize, obj.specPositionIndex, obj.westMinMargin, obj.eastMinMargin);
        break;
      }
      
      if(c)
        ca.push(c);
    }
    
    // constraints are sent in reverse order, so add them in reverse order too.
    for(var i = ca.length - 1; i >= 0; i--)
      layout.addConstraint(ca[i]);
  },
  
  _getNodeGroup: function(nodes, idToNode)
  {
  	var constr;
    require(["ibm_ilog/graphlayout/hierarchical/HierarchicalNodeGroup"], function(c) { constr = c; });
    var group = new constr();
    for(var i = 0; i < nodes.length; i++)
      group.add(idToNode[nodes[i].id]);
    return group;
  },
  
  _getNode: function(obj, idToNode)
  {
    return idToNode[obj.id];
  }
});

return _LayoutProvider;

});


},
'ibm_ilog/diagram/graphlayout/webworker/_GraphAdapter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/webworker/_GraphAdapter", [
"require",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/array",
"dojox/gfx/matrix",
"ibm_ilog/graphlayout/AbstractGraphLayoutModel",
"ibm_ilog/graphlayout/internalutil/GraphModelUtil"
], function(
require,
declare,
lang,
array,
matrix,
AbstractGraphLayoutModel,
GraphModelUtil
){

/*=====
var AbstractGraphLayoutModel = ibm_ilog.graphlayout.AbstractGraphLayoutModel;
=====*/

var _GraphAdapter =
declare("ibm_ilog.diagram.graphlayout.webworker._GraphAdapter", AbstractGraphLayoutModel,
{
	// summary:
	//		The implementation of the graph model interface to layout the contents of a Graph.
	
	_graph: null, // a JS object actually
	
	getGraph: function(){
		return this._graph;
	},
	
	constructor: function(graph, layout) {
		this._graph = graph;
		this._originatingLayout = layout;
	},
	
    isNode: function(o) {
		return o._isNode;
	},
	
    getNodes: function() { 
    	return this._graph.nodes;
	},
	
    isLink: function(o) {
    	return o._isLink;
	},
	
    getLinks: function() { 
    	return array.filter(this._graph.links, function(o){ return o._isLink; });
	},

    isIntergraphLink: function(o) {
    	return o._isIntergraphLink;
	},
	
    getIntergraphLinks: function() { 
    	return array.filter(this._graph.links, function(o){ return o._isIntergraphLink; });
	},
	
    isSubgraph: function(o) {
    	return o._isSubgraph;
	},
	
    getSubgraphs: function(){
    	return array.filter(this._graph.nodes, function(o){ return o._isSubgraph; });
	},
	
    getLinksFrom: function(node) { 
		var linksFrom = node._fromLinks; 
		if (!linksFrom)
			return [];
		return array.filter(linksFrom, function(o) { return o._isLink; }); 
	},
	
    getLinksTo: function(node) { 
		var linksTo = node._toLinks; 
		if (!linksTo)
			return [];
		return array.filter(linksTo, function(o) { return o._isLink; }); 
	},
	
    getFrom: function(link) { return link._from; },
    getTo: function(link) { return link._to; },
	
	getAllLinks : function() {
		// summary:
		//		Gets all the links and intergraph links contained in the graph associated with this graph model.
		// description:
		//		Gets all the links and intergraph links contained in the graph associated with this graph model.
		//		This method is used only in the ibm_ilog.diagram.graphlayout.Animation class, it is not part of the
		//		base graph model interface.
    	return array.filter(this._graph.links, function(o){ return o._isLink || o._isIntergraphLink; });
	},
	
    boundingBox: function(nodeOrLink){
    	var b = nodeOrLink.b;
    	if(b)
    		return { x:b[0], y:b[1], width:b[2], height:b[3] };
   		else
       		return { x:0, y:0, width:0, height:0 };
	},
	
    moveNode: function(node, x, y) {
    	node.b[0] = x;
    	node.b[1] = y;
	},
	
    getLinkPoints: function(link) {
    	var points = [];
    	var p;
    	
    	if(link.fp)
    		p = { x:link.fp[0], y: link.fp[1] };
    	else
    		p = { x:link._from.b[0]+link._from.b[2]/2, y:link._from.b[1]+link._from.b[3]/2 };
    	points.push(p);
    	
    	if(link.ip)
    		for(var i = 0; i < link.ip.length; i++)
    			points.push({ x:link.ip[i][0], y:link.ip[i][1] });
    	
    	if(link.tp)
    		p = { x:link.tp[0], y: link.tp[1] };
    	else
    		p = { x:link._to.b[0]+link._to.b[2]/2, y:link._to.b[1]+link._to.b[3]/2 };
    	points.push(p);
    	
    	return points;
	},
	
    reshapeLink: function(link, style, fromPoint, fromPointMode, points, startIndex, length, toPoint, toPointMode){
		/* DEBUG. To active it, also activate the DEBUG code in WebWorkerLayout.
		self.postMessage({
        type: "debug",
        message: "reshapeLink..."
    });
    */
    
		// start point:
		switch(fromPointMode){
			case GraphModelUtil.ReshapeLinkMode.IGNORE:
			case GraphModelUtil.ReshapeLinkMode.FIX:
				break;
			case GraphModelUtil.ReshapeLinkMode.MOVE:
				link.fp = [ fromPoint.x, fromPoint.y ];
				link.fromMoved = true;
			 	break;
			case GraphModelUtil.ReshapeLinkMode.CLIP:
			 	delete link.fp;
			 	break;
			case GraphModelUtil.ReshapeLinkMode.MOVE_AND_CLIP:
				link.fp = [ fromPoint.x, fromPoint.y ];
				link.fromMoved = true;
			 	break;
		}
		
		// end point:
		switch(toPointMode){
			case GraphModelUtil.ReshapeLinkMode.IGNORE:
			case GraphModelUtil.ReshapeLinkMode.FIX:
				break;
			case GraphModelUtil.ReshapeLinkMode.MOVE:
				link.tp = [ toPoint.x, toPoint.y ];
				link.toMoved = true;
			 	break;
			case GraphModelUtil.ReshapeLinkMode.CLIP:
			 	delete link.tp;
			 	break;
			case GraphModelUtil.ReshapeLinkMode.MOVE_AND_CLIP:
				link.tp = [ toPoint.x, toPoint.y ];
				link.toMoved = true;
			 	break;
		}
		
		// intermediate points:
		
		if(length > 0){
			link.ip = [];
			for(var i = startIndex; i < startIndex + length; i++){
				link.ip.push([ points[i].x, points[i].y ]);
			}
		} else {
			delete link.ip;
		}
	},
	
    hasMoveableConnectionPoint: function(link, origin) {
    	return true;
	},
	
    hasPinnedConnectionPoint: function(link, origin) { 
    	return origin ? link.sp : link.ep;
	},
	
    getLinkWidth: function(link) { return link.w; },
	
    createGraphModel: function(subgraph) { 
		return new _GraphAdapter(subgraph);
	},
	
	getParent: function() {
		var root = this.getRoot();
		if(root == null || root == this)
			return null;
		var parentGraph = this._graph._parent;
		if (parentGraph) {
			if(parentGraph == root._graph)
				return root;
			return root.getGraphModel(parentGraph);
		}
		return null;
	},
	
    setOriginatingLayout: function(layout) {
        this._originaringLayout = layout;
    },
    
    getOriginatingLayout: function() {
        return this._originaringLayout;
    },
	
	getTransform: function(subModel)
	{
		var m = subModel._graph.m;
		var t;
		if(m)
			t = { xx:m[0], xy:m[1], yx:m[2], yy:m[3], dx:m[4], dy:m[5] };
		else
			t = { xx:1, xy:0, yx:0, yy:1, dx:0, dy:0 };
		return new matrix.Matrix2D(t);
	}
});

var HierarchicalLayout, HierarchicalSwimLaneConstraint;

lang.mixin(_GraphAdapter, {
	
	prepareGraph: function(graph)
	{
		var idToNode = {};
		_GraphAdapter._prepareNodes(graph, idToNode);
		_GraphAdapter._prepareLinks(graph, idToNode);
		graph._idToNode = idToNode; // for later use by constraints
	},
	
	_prepareNodes: function(graph, idToNode)
	{
		var i;
		if(graph.nodes){
			for(i = 0; i < graph.nodes.length; i++){
				var node = graph.nodes[i];
				idToNode[node.id] = node;
				node._isNode = true;
				node._parent = graph;
				if(node.gb){
					node._isSubgraph = true;
					_GraphAdapter._prepareNodes(node, idToNode);
				}
			}
		}
	},
	
	_prepareLinks: function(graph, idToNode)
	{
		var i;
		if(graph.links){
			for(i = 0; i < graph.links.length; i++){
				var link = graph.links[i];

				link._from = idToNode[link.f];
				link._to = idToNode[link.t];

				if(!link._from._fromLinks) link._from._fromLinks = [];
				link._from._fromLinks.push(link);

				if(!link._to._toLinks) link._to._toLinks = [];
				link._to._toLinks.push(link);

				link._isIntergraphLink =
					link._from._parent != graph ||
					link._to._parent != graph;
				link._isLink = !link._isIntergraphLink;
			}
		}
		if(graph.nodes){
			for(i = 0; i < graph.nodes.length; i++){
				var node = graph.nodes[i];
				if(node._isSubgraph)
					_GraphAdapter._prepareLinks(node, idToNode);
			}
		}
	},
	
	cleanupGraph: function(graph)
	{
		// Add calculated bboxes of swim lanes if hierarchical
		
		if(HierarchicalLayout === undefined){
			try {
				HierarchicalLayout = require("ibm_ilog/graphlayout/hierarchical/HierarchicalLayout");
				HierarchicalSwimLaneConstraint = require("ibm_ilog/graphlayout/hierarchical/HierarchicalSwimLaneConstraint");
			} catch(err){
				HierarchicalLayout = null;
				HierarchicalSwimLaneConstraint = null;
			}
		}
		
		if(graph.nodeLayout &&
		   graph.nodeLayout._layout &&
		   HierarchicalLayout && HierarchicalSwimLaneConstraint &&
		   graph.nodeLayout._layout instanceof HierarchicalLayout){
		   	var bboxes = [];
		   	var constraints = graph.nodeLayout._layout.getConstraints();
			while(constraints.hasNext()){
				var c = constraints.next();
				if(c instanceof HierarchicalSwimLaneConstraint){
					var bbox = c.getCalcBoundingBox();
					bboxes.push([ bbox.x, bbox.y, bbox.width, bbox.height ]);
				}
			}
			if(bboxes.length > 0)
				graph.swimLaneCalcBBoxes = bboxes;
		}
		
		// clear cached layout instances
		if(graph.nodeLayout)
			delete graph.nodeLayout._layout;
		if(graph.linkLayout)
			delete graph.linkLayout._layout;
		
		delete graph._idToNode;
		
		// clear all node/link properties,
		// including those stored by _prepareNodes/Links
		var i;
		var p;
		if(graph.nodes){
			for(i = 0; i < graph.nodes.length; i++){
				var node = graph.nodes[i];
				if(node._isSubgraph)
					_GraphAdapter.cleanupGraph(node);
				for(p in node)
					if(p.charAt(0) == "_")
						delete node[p];

			}
		}
		if(graph.links){
			for(i = 0; i < graph.links.length; i++){
				var link = graph.links[i];
				if (!link.fromMoved)
					delete link.fp;
			  if (!link.toMoved)
					delete link.tp;
				for(p in link)
					if(p.charAt(0) == "_")
						delete link[p];
			}
		}
	}
});

return _GraphAdapter;

});

},
'ibm_ilog/diagram/style':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/style", ["dojo/_base/lang"],
function(lang){

	// TODO: Change to this when AMD conversion is complete:
	// var ids = {};
	var ids = lang.getObject("ibm_ilog.diagram.style", true);
        
    function getSetter(o, p) {
        var uname = p.length > 1 ? (p.substr(0, 1).toUpperCase()) + p.substr(1) : p.toUpperCase();
        var set = lang.getObject('set' + uname, false, o);
        return set;
    };
    
    ids.applyTo = function(/*Object*/mixin, /*Object*/target) {
        // summary: applies the specified style to the given target.
        // mixin: Object: 
        //            A dictionary of property-value pairs that define global properties to apply on the specified target.
        //            Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
        //            for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
        //            the node). If no setters are found, the property is set to the target as a simple field.
        // target: Object: the target instance.
         
        if (!mixin || !target) 
            return;
        for (var p in mixin) {
            var setter = getSetter(target, p);
            if (setter) {
                var val = mixin[p];
                setter.call(target, val);
            } else {
                // no setter -> add as a simple attribute
				target[p]=val; 
            }
        }
    };
   
   return ids; 
});

},
'ibm_ilog/diagram/adorners/AdornerHandle':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/AdornerHandle", [
"../main",
"dojo/_base/declare",
"dojox/gfx/utils",
"./TemplatedOverlay"
],
function(
iid,
declare,
gfxu,
TemplatedOverlay
){

/*=====
var TemplatedOverlay = ibm_ilog.diagram.adorners.TemplatedOverlay;
=====*/

var AdornerHandle =
	iid.gfxDeclaration(declare('ibm_ilog.diagram.adorners.AdornerHandle', TemplatedOverlay,{
		
		initialize: function(adorner,template) {
			this._adorner = adorner;
			this._template = template;
			this.applyTemplate(this._template);
		},
		getAdorner: function(){
			return this._adorner;
		},
		getViewport: function() {
			return this._adorner.getViewport();
		},
		
		add: function(toShow){
			// Summary:
			//		if IE browser, restore the saved Styles
			this.inherited(arguments); 
			if (iid.isVml) {
	             this._restoreStyles(toShow);
	         }
		},
		
		remove: function(toHide){
			// Summary:
			//		if IE browser, saves the Styles before removing the elements from its parent
			//		Workaround for IE: removing a shape clears all its styles.
			if (iid.isVml) {
	                this._saveStyles(toHide);
	         }
			 this.inherited(arguments);
		},
		
		_saveStyles: function(group) {
	        // summary:
	        //		Workaround for IE: removing a shape clears all its styles.
	        // tags:
	        //		private
	        gfxu.forEach(group, function(shape) {
	            if (shape.setFill) {
	                shape.oldFill = shape.getFill();
	            }
	            if (shape.setStroke) {
	                shape.oldStroke = shape.getStroke();
	            }
	            if (shape.setFont) {
	                shape.oldFont = shape.getFont();
	            }
	            if (shape.setFont) {
	                shape.oldFont = shape.getFont();
	            }
	            if (shape.setShape) {
	                shape.oldShape = shape.getShape();
	            }
	        }, null);
	    },
	    
	    _restoreStyles: function(group) {
	        // summary:
	        //		Workaround for IE: removing a shape clears all its styles.
	        // tags:
	        //		private
	        gfxu.forEach(group, function(shape) {
	            if (shape.oldFill) {
	                shape.setFill(shape.oldFill);
	            }
	            if (shape.oldStroke) {
	                shape.setStroke(shape.oldStroke);
	            }
	            if (shape.oldFont) {
	                shape.setFont(shape.oldFont);
	            }
	            if (shape.oldShape) {
	                shape.setShape(shape.oldShape);
	            }
	        }, null);
	    }
		
	}));

return AdornerHandle;

});


},
'ibm_ilog/diagram/gfxlayout/GridLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout/GridLayout", ["dojo/_base/lang", "dojo/_base/declare", "./Layout"],
function(lang, declare, Layout){

/*=====
var Layout = ibm_ilog.diagram.gfxlayout.Layout;
=====*/

var GridLayout =
declare('ibm_ilog.diagram.gfxlayout.GridLayout', Layout, {
    // summary:
    //      A layout class that arranges children of a container as a grid.
    // description:
    //      The GridLayout class arranges children of a container as grid elements.
    //      <p>The grid structure is defined by the 'rows' and 'cols' constructor parameters. 
    //      These properties are arrays of descriptor objects that defined how the column or row
    //      size is computed. These descriptor objects support the following properties:
    //      'width': if the container width (resp. height) is not fixed, then the column width (resp. row height)
    //      depends on the value of this property. A value of -1 means the dimension will accomodate to the maximum
    //      children dimension. Any other positive number sets the column width (resp. row height).
    //      'weight': if the container width (resp. height) is fixed, this property defines how the space 
    //      remaining after all the fixed width's (resp. height's) have been substracted from the 
    //      container width (resp. height) is distributed accross the columns (resp. the rows).
    //      <p>
    //      The GridLayout class defines the paddingLeft, paddingRight, paddingTop and paddingBottom properties
    //      to specificy the internal margins of the panel from which the children are positionned. A convenient global padding
    //      property is also defined to set the same padding in all directions.
    //      <p>Children in the grid are placed into the grid cells according to the 'column' and 'row' properties of the children. 
    //      When these properties are not specified on a shape, the shape is automatically placed in the grid according
    //      to its order in the container children list. The direction of the placement flow is defined via the 'horizontalFlow'
    //      property, which is 'true' by default. For example, assuming the 'horizontalFlow' property is set to 'true' the children
    //      do not defined any specific row/col index, the first child will be placed at the (0,0) cell, the 2nd one at the (1,0) cell, and so on.
    //      <p>
    //      A child can span over multiple rows or columns depending on their 'colspan' or 'rowspan' properties. These properties indicates
    //      the number of columns or rows the child must overlap. A value of '-1' means the child should span over all the remaining columns or rows.
    //      <p>
    //      Children are arranged within the computed grid cell according to their 'valign' 
    //      (one of the 'top', 'center', 'bottom' and 'stretch' values), 'halign' (one of the 'left', 'center', 'right' and 'stretch' values) 
    //      and their 'margins' properties.
    
    // the rows and columns descriptor
    rows: null,
    cols: null,
    // padding
    padding: 0,
    paddingLeft: 0,
    paddingRight: 0,
    paddingTop: 0,
    paddingBottom: 0,
    
    horizontalFlow: true,
    
    constructor: function(args) {
        this.rows = this.rows || [];
        this.cols = this.cols || [];
        if (this.rows.length === 0) 
            this.rows[0] = {
                weight: 1,
                height: -1
            };
        if (this.cols.length === 0) 
            this.cols[0] = {
                weight: 1,
                width: -1
            };
        if (args && 'padding' in args) 
            this._setPadding(args.padding, false);
    },
	
	getConfig : function() {
		// summary:
		// 		Gets the current configuration of this layout instance.
		// Description:
		//		This method returns a descriptor of this layout current configuration
		//		as well as the type information so that it can be passed to the Group.setLayout() method
		//		(for example to make a copy the layout of a Group). 
		// returns: Object
		//	An object that represents the layout current configuration.
		var arg ={};
		arg.type = 'ibm_ilog.diagram.gfxlayout.GridLayout';
		if (this.padding)
			arg.padding = this.padding;
		else {
			arg.paddingLeft = this.paddingLeft;
			arg.paddingRight = this.paddingRight;
			arg.paddingTop = this.paddingTop;
			arg.paddingBottom = this.paddingBottom;
		}
		arg.rows = lang.clone(this.rows);
		arg.cols = lang.clone(this.cols);
		arg.horizontalFlow = this.horizontalFlow;
		return arg;	/*Object*/	
	},
    
    setHorizontalFlow: function(/*Boolean*/value) {
        // summary:
        //     Sets whether children with no specified 'row' or 'column' properties
        //     are arranged horizontally or vertically in the grid. 
        //  value: Boolean: the automatic flow direction. 
        
        if (value !== this.horizontalFlow) {
            var old = this.horizontalFlow;
            this.horizontalFlow = value;
            this.onLayoutPropertyChanged('horizontalFlow', old, value);
        }
    },
	
	getHorizontalFlow:function(){ 
		return this.horizontalFlow; 
	},
    
    setPadding: function(value) {
        // summary:
        //     Sets the padding of this layout.
        // description:
        //     This property is used to set the internal margins of this layout.
        //     The value can either be a 4-elements array whose items match
        //     respectively the left, top, right and bottom paddings ; or one integer
        //     that is applied to all paddings.
        this._setPadding(value, true);
    },
    _setPadding: function(value, notify) {
        if (value instanceof Array) {
            var l = value.length, oldL = this.paddingLeft, oldR = this.paddingRight, oldT = this.paddingTop, oldB = this.paddingBottom;
            if (l > 0) 
                this.paddingLeft = value[0];
            if (l > 1) 
                this.paddingTop = value[1];
            if (l > 2) 
                this.paddingRight = value[2];
            if (l > 3) 
                this.paddingBottom = value[3];
            if (notify &&
            (this.paddlingLeft !== oldL ||
            this.paddingTop !== oldT ||
            this.paddingBottom !== oldB ||
            this.paddingRight !== oldR)) {
            
                this.onLayoutPropertyChanged();
            }
        } else {
            var old = this.padding;
            this.padding = this.paddingLeft = this.paddingRight = this.paddingBottom = this.paddingTop = value;
            if (notify && (value !== old)) {
                this.onLayoutPropertyChanged();
            }
        }
    },
    
    adjustBBox: function(/*dojox.gfx.Rectangle*/bbox) {
        // summary:
        //     Returns the container bbox adjusted according to the constraints of this layout.
        // bbox: Rectangle: the bounding box of the associated container.
        
        bbox.x -= this.paddingLeft;
        bbox.y -= this.paddingTop;
        bbox.width += this.paddingLeft + this.paddingRight;
        bbox.height += this.paddingTop + this.paddingBottom;
        return bbox;
    },
    
    _getColIdx: function(shape, shapeIdx) {
        var c = 0;
        if ('column' in shape) 
            c = shape.column;
        else if (this.horizontalFlow) 
            c = shapeIdx % this.cols.length;
        else 
            c = Math.floor(shapeIdx / this.rows.length);
        return Math.min(c, this.cols.length - 1);
    },
    
    _getRowIdx: function(shape, shapeIdx) {
        var r = 0;
        if ('row' in shape) 
            r = shape.row;
        else if (this.horizontalFlow) 
            r = Math.floor(shapeIdx / this.cols.length);
        else 
            r = shapeIdx % this.rows.length;
        return Math.min(r, this.rows.length - 1);
    },
    
    _getRowDesc: function(rowIdx) {
        return rowIdx < this.rows.length ? this.rows[rowIdx] : null;
    },
    
    _getColDesc: function(colIdx) {
        return colIdx < this.cols.length ? this.cols[colIdx] : null;
    },
    
    computePreferredSize: function(proposedSize) {
    
        var children = this._container.children, count = children.length, i = 0, r = 0, c = 0, sh, sb, rDesc, cDesc, rm, cm;
        var rowsMetric = [], colsMetric = [];
        
        for (i = 0; i < this.rows.length; ++i) {
            rDesc = this.rows[i];
            rm = {};
            rm.height = 0;
            if ('height' in rDesc && rDesc.height !== -1) 
                rm.height = rDesc.height;
            rowsMetric.push(rm);
        }
        for (i = 0; i < this.cols.length; ++i) {
            cDesc = this.cols[i];
            cm = {};
            cm.width = 0;
            if ('width' in cDesc && cDesc.width !== -1) {
                cm.width = cDesc.width;
            }
            colsMetric.push(cm);
        }
        
        /*
         var totalFixedW = 0, totalWeightW = 0, totalFixedH = 0, totalWeightH = 0;
         
         for (i = 0; i < this.rows.length; ++i) {
         rDesc = this.rows[i];
         rm = {};
         rm.count = 0;
         rm.weight = -1;
         if ('weight' in rDesc) {
         var weight = rDesc.weight;
         rm.weight = weight;
         totalWeightH += weight;
         }
         rm.height = -1;
         if ('height' in rDesc && rDesc.height !== -1)
         rm.height = rDesc.height;
         rowsMetric.push(rm);
         }
         for (i = 0; i < this.cols.length; ++i) {
         cDesc = this.cols[i];
         cm = {};
         cm.count = 0;
         cm.weight = -1;
         if ('weight' in cDesc) {
         var weight = cDesc.weight;
         cm.weight = weight;
         totalWeightW += weight;
         }
         cm.width = -1;
         if ('width' in cDesc && cDesc.width !== -1) {
         cm.width = cDesc.width;
         }
         colsMetric.push(cm);
         }
         */
        for (i = 0; i < count; ++i) {
            sh = children[i];
            c = this._getColIdx(sh, i);
            r = this._getRowIdx(sh, i);
            // get the corresponding column/row descriptors
            rDesc = this.rows[r];
            cDesc = this.cols[c];
            var sz = {};
            sz.width = ('width' in cDesc && cDesc.width !== -1) ? cDesc.width : -1;
            sz.height = ('height' in rDesc && rDesc.height !== -1) ? rDesc.height : -1;
            sb = sh.computePreferredSize(sz);
			// take into account the shape width only if it is not an horizontal filler
			if (!sh.hfiller && 'width' in cDesc) {
                // if column width not set, set to the maximum preferred width
                if (cDesc.width === -1) {
                    var w = sb.width;
                    if (sh.margins) {
                        w = this._inflateH(w, sh.margins);
                    }
                    if (w > colsMetric[c].width) 
                        colsMetric[c].width = w;
                } else {
                    colsMetric[c].width = cDesc.width;
                }
            }
			// take into account the shape height only if it is not a vertical filler
            if (!sh.vfiller && 'height' in rDesc) {
                // if not fixed height, get the maximum preferred height 
                if (rDesc.height === -1) {
                    var h = sb.height;
                    if (sh.margins) {
                        h = this._inflateV(h, sh.margins);
                    }
                    if (h > rowsMetric[r].height) 
                        rowsMetric[r].height = h;
                } else {
                    rowsMetric[r].height = rDesc.height;
                }
            }
        }
        
        /*
         for (i = 0; i < count; ++i) {
         sh = children[i];
         c = this._getColIdx(sh, i);
         r = this._getRowIdx(sh, i);
         // get the corresponding column/row metrics
         rm = rowsMetric[r];
         cm = colsMetric[c];
         rDesc = this.rows[r];
         cDesc = this.cols[c];
         ++cm.count;
         ++rm.count;
         var sz = {};
         sz.width = ('width' in cDesc && cDesc.width !== -1) ? cDesc.width : -1;
         sz.height = ('height' in rDesc && rDesc.height !== -1) ? rDesc.height : -1;
         sb = sh.computePreferredSize(sz);
         // if column width not set, set to the maximum preferred width
         // (except if a weight has been specified, in this case the width
         // will be computed later when the total fixed width will be known
         if (this.cols[c].width === -1 && cm.weight === -1) {
         var w = sb.width;
         if (sh.margins)
         w = this._inflateH(w, sh.margins);
         if (w > cm.width)
         cm.width = w;
         }
         // if not fixed height, get the maximum preferred height
         if (this.rows[r].height === -1 && rm.weight === -1) {
         var h = sb.height;
         if (sh.margins)
         h = this._inflateV(h, sh.margins);
         if (h > rm.height)
         rm.height = h;
         }
         }
         // here, we have all fixed size computing done.
         // now we can compute the 'weighted' sizes
         if (totalWeightW > 0) {
         // Compute columns width
         for (i = 0; i < colsMetric.length; ++i) {
         cm = colsMetric[i];
         // if a width has been computed previously for this column, sum it
         if (cm.width !== -1) {
         totalFixedW += cm.width;
         } else { // width not set yet, and column not empty
         if (cm.weight === -1 && cm.count > 0) {
         cm.weight = 1;
         ++totalWeightW;
         } else if (cm.weight !== -1 && cm.count === 0) {
         // the column is empty, should not be included in the total
         --totalWeightW;
         }
         }
         }
         var remainingW = Math.max(0, contBounds.width - totalFixedW);
         // then compute the 'weighted' width
         if (totalWeightW > 0) {
         var unitW = remainingW / totalWeightW;
         for (i = 0; i < colsMetric.length; ++i) {
         cm = colsMetric[i];
         if (cm.width === -1 && cm.count > 0) {
         cDesc = this.cols[i];
         var weight = cDesc.weight || 1;
         cm.width = cm.weight * unitW;
         }
         }
         }
         }
         if (totalWeightH > 0) {
         
         for (i = 0; i < rowsMetric.length; ++i) {
         rm = rowsMetric[i];
         // if a height has been computed previously for this row, sum it
         if (rm.height !== -1) {
         totalFixedH += rm.height;
         } else {
         if (rm.weight === -1) {
         rm.weight = 1;
         totalWeightH += 1;
         }
         }
         }
         var remainingH = Math.max(0, contBounds.height - totalFixedH);
         if (totalWeightH > 0) {
         var unitH = remainingH / totalWeightH;
         for (i = 0; i < rowsMetric.length; ++i) {
         rm = rowsMetric[i];
         if (rm.height === -1) {
         rDesc = this.rows[i];
         var weight = rDesc.weight || 1;
         rm.height = rm.weight * unitH;
         }
         }
         }
         }
         if (totalWeightH > 0 || totalWeightW > 0) {
         for (i = 0; i < count; ++i) {
         sh = children[i];
         c = this._getColIdx(sh, i);
         r = this._getRowIdx(sh, i);
         // get the corresponding column/row metrics
         rm = rowsMetric[r];
         cm = colsMetric[c];
         var sz = {
         width: cm.width,
         height: rm.height
         };
         sb = sh.getPreferredSize(sz);
         
         }
         }
         */
        var sz = {
            width: 0,
            height: 0
        };
        if (proposedSize.width === -1) {
        
            for (i = 0; i < colsMetric.length; ++i) 
                sz.width += colsMetric[i].width;
        } else {
            sz.width = proposedSize.width;
        }
        if (proposedSize.height === -1) {
            for (i = 0; i < rowsMetric.length; ++i) 
                sz.height += rowsMetric[i].height;
        } else {
            sz.height = proposedSize.height;
        }
        sz.width += this.paddingLeft + this.paddingRight;
        sz.height += this.paddingTop + this.paddingBottom;
        
        return sz;
        
    },
    
    layoutChildren: function(contBounds) {
    
        contBounds = lang.clone(contBounds);
        var children = this._container.children, count = children.length, i = 0, r = 0, c = 0, sh, sb, rDesc, cDesc, cm, rm;
        var rowsMetric = [], colsMetric = [];
        var totalFixedW = 0, totalWeightW = 0, totalFixedH = 0, totalWeightH = 0;
        
        for (i = 0; i < this.rows.length; ++i) {
            rDesc = this.rows[i];
            rm = {};
            rm.count = 0;
            rm.weight = -1;
            if ('weight' in rDesc) {
                var weight = rDesc.weight;
                rm.weight = weight;
                totalWeightH += weight;
            }
            rm.height = -1;
            if ('height' in rDesc && rDesc.height !== -1) 
                rm.height = rDesc.height;
            rowsMetric.push(rm);
        }
        for (i = 0; i < this.cols.length; ++i) {
            cDesc = this.cols[i];
            cm = {};
            cm.count = 0;
            cm.weight = -1;
            if ('weight' in cDesc) {
                var weight = cDesc.weight;
                cm.weight = weight;
                totalWeightW += weight;
            }
            cm.width = -1;
            if ('width' in cDesc && cDesc.width !== -1) {
                cm.width = cDesc.width;
            }
            colsMetric.push(cm);
        }
        
        contBounds.width = Math.max(0, contBounds.width - this.paddingLeft - this.paddingRight);
        contBounds.height = Math.max(0, contBounds.height - this.paddingTop - this.paddingBottom);
        for (i = 0; i < count; ++i) {
            sh = children[i];
            sb = sh.getPreferredSize();
            c = this._getColIdx(sh, i);
            r = this._getRowIdx(sh, i);
            // get the corresponding column/row metrics
            rm = rowsMetric[r];
            cm = colsMetric[c];
            ++cm.count;
            ++rm.count;
            // if column width not set, set to the maximum preferred width
            // (except if a weight has been specified, in this case the width
            // will be computed later when the total fixed width will be known
            if (this.cols[c].width === -1 && cm.weight === -1) {
                var w = sb.width;
                if (sh.margins) 
                    w = this._inflateH(w, sh.margins);
                if (w > cm.width) 
                    cm.width = w;
            }
            // if not fixed height, get the maximum preferred height 
            if (this.rows[r].height === -1 && rm.weight === -1) {
                var h = sb.height;
                if (sh.margins) 
                    h = this._inflateV(h, sh.margins);
                if (h > rm.height) 
                    rm.height = h;
            }
        }
        // Compute columns width
        for (i = 0; i < colsMetric.length; ++i) {
            cm = colsMetric[i];
            // if a width has been computed previously for this column, sum it
            if (cm.width !== -1) {
                totalFixedW += cm.width;
            } else { // width not set yet, and column not empty
                if (cm.weight === -1 && cm.count > 0) {
                    cm.weight = 1;
                    ++totalWeightW;
                } else if (cm.weight !== -1 && cm.count === 0) {
                    // the column is empty, should not be included in the total
                    --totalWeightW;
                }
            }
        }
        var remainingW = Math.max(0, contBounds.width - totalFixedW);
        // then compute the 'weighted' width
        if (totalWeightW > 0) {
            var unitW = remainingW / totalWeightW;
            for (i = 0; i < colsMetric.length; ++i) {
                cm = colsMetric[i];
                if (cm.width === -1 && cm.count > 0) {
                    cDesc = this.cols[i];
                    var weight = cDesc.weight || 1;
                    cm.width = cm.weight * unitW;
                }
            }
        }
        
        for (i = 0; i < rowsMetric.length; ++i) {
            rm = rowsMetric[i];
            // if a height has been computed previously for this row, sum it
            if (rm.height !== -1) {
                totalFixedH += rm.height;
            } else {
                if (rm.weight === -1) {
                    rm.weight = 1;
                    totalWeightH += 1;
                }
            }
        }
        var remainingH = Math.max(0, contBounds.height - totalFixedH);
        if (totalWeightH > 0) {
            var unitH = remainingH / totalWeightH;
            for (i = 0; i < rowsMetric.length; ++i) {
                rm = rowsMetric[i];
                if (rm.height === -1) {
                    rDesc = this.rows[i];
                    var weight = rDesc.weight || 1;
                    rm.height = rm.weight * unitH;
                }
            }
        }
        // compute cell coord
        var currentX = /*contBounds.x + */ this.paddingLeft;
        for (i = 0; i < colsMetric.length; ++i) {
            cm = colsMetric[i];
            cm.x = currentX;
            currentX += cm.width;
        }
        var currentY = /*contBounds.y + */ this.paddingTop;
        for (i = 0; i < rowsMetric.length; ++i) {
            rm = rowsMetric[i];
            rm.y = currentY;
            currentY += rm.height;
        }
        // arrange children
        for (i = 0; i < count; ++i) {
            var slotBounds = {}, sh = children[i];
            c = this._getColIdx(sh, i);
            r = this._getRowIdx(sh, i);
            cm = colsMetric[c];
            rm = rowsMetric[r];
            slotBounds.x = cm.x;
            slotBounds.width = cm.width;
            slotBounds.y = rm.y;
            slotBounds.height = rm.height;
            // span
            var span = sh.colspan;
            if (span) {
                this._spanSlot(span, c, slotBounds, 'width', colsMetric);
            }
            span = sh.rowspan;
            if (span) {
                this._spanSlot(span, r, slotBounds, 'height', rowsMetric);
            }
            this.placeChildInSlot(sh, slotBounds, true, true);
        }
    },
    
    _spanSlot: function(span, dimIdx, slotBounds, dim, dimMetric) {
        if (span === -1) 
            span = dimMetric.length - dimIdx;
        if (span > 1) {
            var end = dimIdx + span;
            for (++dimIdx; dimIdx < end; ++dimIdx) {
                if (dimIdx < dimMetric.length) {
                    var dm = dimMetric[dimIdx];
                    slotBounds[dim] += dm[dim];
                }
            }
        }
        return slotBounds;
    }
});

return GridLayout;

});

},
'ibm_ilog/diagram/adorners/LinkAdorner':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/LinkAdorner", [
"../main",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/array",
"dojo/_base/connect",
"dojo/_base/event",
"dojox/gfx",
"../templating",
"./TemplatedOverlay",
"./Adorner",
"./AdornerWithHandles",
"../util/Geometry",
"../util/ErrorReporter"
],
function(
iid,
declare,
lang,
array,
connect,
event,
gfx,
templating,
TemplatedOverlay,
Adorner,
AdornerWithHandles,
g,
R
){

/*=====
var AdornerWithHandles = ibm_ilog.diagram.adorners.AdornerWithHandles;
=====*/

var par = Adorner.prototype;

var LinkAdorner =
iid.gfxDeclaration(declare('ibm_ilog.diagram.adorners.LinkAdorner', [AdornerWithHandles], {
	//
	//	summary:
	//		A UI overlay associated with a link graph entity.
	//
	//	description:
	//		The LinkAdorner object provides a graphical decoration for a
	//		link graph entity. See Adorner for further
	//
	_elementPath: null,
	_echoConnections: null,
	
	initialize: function (diagram, templatePool, templateId) {
		// Summary:
		//		initialize the adorner. Adds some mouse listeners to mouseup and mousedownto push the events to the links
		this.inherited(arguments);
		this._echoConnect();
	},
	
	_buildEchoConnection: function(eventName){
		try {
			this._echoConnections.push(this._path.connect(eventName, this, 'echoEvent'));
		} catch(e) {
			R.warn("EventConnectionFailed",eventName);
		}
	},
	
	_echoConnect: function(){
		if(this._echoConnections){
			array.forEach(this._echoConnections,function(conn){connect.disconnect(conn);});
		}
		this._echoConnections = [];
		if (this['_path']) {
			this._buildEchoConnection('onmousedown');
			this._buildEchoConnection('onmouseup');
			this._buildEchoConnection('onclick');
			this._buildEchoConnection('oncontextmenu');
		}
	},
	
	echoEvent: function (e) {
		// Summary:
		//		Push the recieved event to the binded link
		 if (e.initMouseEvent) {        // Firefox, Google Chrome, Safari, Opera
			  var evt = document.createEvent('MouseEvents');
			  evt.initMouseEvent(e.type, e.bubbles, e.cancelable, e.view, e.detail, e.screenX, e.screenY, e.clientX, e.clientY, e.ctrlKey, e.altKey, e.shiftKey, e.metaKey, e.button, e.relatedTarget);
			  this.getAdorned().rawNode.dispatchEvent(evt);
		 }else{
			  if (document.createEventObject) {        //Internet Explorer
                  var mouseEvent = document.createEventObject (e);
                  mouseEvent.button = e.button;
                  this.getAdorned().rawNode.fireEvent('on'+e.type,mouseEvent);
              }
		}
		event.stop(e);
	},
	
	_reconnectNotifications: function () {
		//
		// 	summary:
		//		Reconnect to change notifications related to the bound element. 
		//
		this.inherited(arguments);
		var e = this._adorned;
		if (e) {
			this._changeConnections.push(connect.connect(e, "_updatePath", this, "_elementBoundsChanged"));
		}
	},

	_setFallbackBounds: function() {
		this._elementPath = [{
			x: iid._invisibleLocation,
			y: iid._invisibleLocation
		}, {
			x: iid._invisibleLocation,
			y: iid._invisibleLocation
		}]; // by default construct adorner out of sight 
		par._setFallbackBounds.apply(this,arguments);
	},
	
	_computeElementBoundsChanged: function () {
		//
		// 	summary:
		//		gets the Element Path from the bounded element
		//
		if (this._required()) {
			this._elementPath = this._adorned._pathPoints;
		}
		par._computeElementBoundsChanged.apply(this,arguments);
	},

	_updateLayout: function () {
		
		this.startDTLBatch();
		this.inherited(arguments);
		var path = lang.clone(this._elementPath);
		if (this._required()) {
			var v = this._viewport,
				z = v.getZoom(),
				vr = v.getViewRect();
			var t = this._adorned.getShapeToContainerTransform(this._diagram.getGraph().getParent());

			path[0] = t.transformPoint(path[0]);
			path[0] = g.moveRect(path[0], g.negPoint(vr));
			path[0].x = path[0].x * z;
			path[0].y = path[0].y * z;

			path[path.length - 1] = t.transformPoint(path[path.length - 1]);
			path[path.length - 1] = g.moveRect(path[path.length - 1], g.negPoint(vr));
			path[path.length - 1].x = path[path.length - 1].x * z;
			path[path.length - 1].y = path[path.length - 1].y * z;
		}

		this.setLinkStartX(path[0].x);
		this.setLinkStartY(path[0].y);
		this.setLinkEndX(path[path.length - 1].x);
		this.setLinkEndY(path[path.length - 1].y);
		this.endDTLBatch();

		//update path
		if (this['_path'] != undefined) {
			if (this.getAdorned() != null) {
				var transformationFunction = lang.hitch(this, function (point, list) {
					var v = this._viewport,
						z = v.getZoom(),
						vr = v.getViewRect();
					var t = this._adorned.getShapeToContainerTransform(v._diagram.getGraph().getParent());
					point = t.transformPoint(point);
					point = g.moveRect(point, g.negPoint(vr));
					point.x = point.x * z;
					point.y = point.y * z;
					return point;
				});
				var curveTension = this.getAdorned().getCurveTension();
				this._updatePathShape(curveTension);
				var points = array.map(this.getAdorned()._pathPoints,transformationFunction);
				this.getAdorned().applyPath(this._path, points,curveTension);
			} else {
				if (this._path instanceof gfx.Polyline) {
					this._path.setShape(path);
					this._path.bbox = null; // patch for GFX bug...
				} else {
					this._path.setShape("");
				}
			}
		}
	},
	
	_updatePathShape: function(curveTension){
		if (curveTension > 0 && !(this._path instanceof gfx.Path)) {
			var stroke, selectedStyle, parent;
			if (this._path) {
				stroke = this._path.getStroke();
				parent = this._path.getParent();
				this._path.removeShape();
			}
			this._path = (parent || this).createPath();
			this._echoConnect();
			if(stroke)
				this._path.setStroke(stroke);
		}
	}

}));

templating.declareBindableProperty(LinkAdorner, "linkStartX", null);
templating.declareBindableProperty(LinkAdorner, "linkStartY", null);
templating.declareBindableProperty(LinkAdorner, "linkEndX", null);
templating.declareBindableProperty(LinkAdorner, "linkEndY", null);

return LinkAdorner;

});

},
'ibm_ilog/diagram/widget/_Diagram':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/_Diagram", [
"dojo/_base/lang",
"dojo/_base/array",
"dojo/_base/sniff",
"dojo/_base/connect",
"dojo/dom-construct",
"dojo/dom-attr",
"dijit/Tooltip",
"dijit/focus",
"dojo/number",
"../interactors/PanInteractor",
"../interactors/ZoomInteractor",
"../interactors/SelectionInteractor",
"../interactors/MoveInteractor",
"../interactors/MarqueeSelectionInteractor",
"../interactors/WheelPanInteractor",
"../interactors/AdornerFocusInteractor",
"../interactors/FocusInteractor",
"../interactors/KeyboardInteractorManager",
"../interactors/Navigator",
"../interactors/SelectionKeyInteractor",
"../interactors/MoveKeyInteractor",
"../interactors/PanKeyInteractor",
"../interactors/ZoomKeyInteractor",
"../interactors/SwitchModeInteractor",
"../adorners/Adorner",
"../adorners/LinkAdorner",
"../adorners/AnnotationAdorner",
"../util/GraphUtil"
], function(
lang,
array,
has,
connect,
domctr,
domattr,
Tooltip,
focus,
number,
PanInteractor,
ZoomInteractor,
SelectionInteractor,
MoveInteractor,
MarqueeSelectionInteractor,
WheelPanInteractor,
AdornerFocusInteractor,
FocusInteractor,
KeyboardInteractorManager,
Navigator,
SelectionKeyInteractor,
MoveKeyInteractor,
PanKeyInteractor,
ZoomKeyInteractor,
SwitchModeInteractor,
Adorner,
LinkAdorner,
AnnotationAdorner,
gu
){

// This class contains the Diagram code that is conditionnally loaded
// only on desktop (non-mobile) platforms.

	var _defaultWaiLabelFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	var _defaultWaiDescriptionFunction = function(ge) {
		if (ge.getLabel) {
			return ge.getLabel();
		} else {
			return ge.toString();
		}
	};
	
	// Extender object to add functionality on desktop (i.e., non-mobile) platforms:
	
	var _Diagram = {
		
		// =======================================================================
		//
		// INTERACTORS:
		//
		
		_initializeInteractors: function() {
			//
			//	summary:
			//		Initializes the interactors used by the Diagram
			//
			//	tags:
			//		protected
			//
			
			this._interactors.add("pan", new PanInteractor().initialize(this));
			this._interactors.add("select", new SelectionInteractor().initialize(this._graph, this.getSelection(), this._viewport.getBackgroundShape()));
			this._interactors.add("zoom", new ZoomInteractor().initialize(this));
			
			this._interactors.add("move", new MoveInteractor().initialize(this._selection, this.getViewport(), {
				doMove: this.moveFunction,
				isMovable: this.isMovableFunction
			}));
			this._interactors.add("marquee", s = new MarqueeSelectionInteractor().initialize(this.getViewport(), this.getSelection()));
			this._interactors.add("wheelPan", new WheelPanInteractor().initialize(this));
			this._interactors.add("adorners", new AdornerFocusInteractor().initialize(this));
			this._interactors.add("focus", new FocusInteractor().initialize(this));
			
			this._setupInteractors(["pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus"]);
			
			connect.connect(this._graph, "onLayoutAnimationStart", this, function() {
				// TODO focusedElement also
				this.setNodeAdornersFocus(null);
				if (this._getNodeAdornersFocusAttr()) {
					this._setNodeAdornersFocusAttr(false);
					
					var h = connect.connect(this._graph, "onLayoutAnimationEnd", this, function() {
						connect.disconnect(h);
						this._setNodeAdornersFocusAttr(true);
					});
				}
			});
		},
		
		enterStandardMode: function() {
			//
			//	summary:
			//		TODO TBD. Probably to be renamed also
			//    	
			this._interactors.switchTo("pan", "select", "zoom", "move", "marquee", "wheelPan", "adorners", "focus");
		},	
		
		//
		//  move: [markup] boolean
		//      enable or disable moving entities
		//
		move: false,
		
		_setMoveAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getMoveAttr();
			if (old !== b) {
				this.move = b;
				this._interactors.get("move").setEnabled(b);
				this._firePropertyChanged('move', old, b);
			}
			return this;
		},
		
		_getMoveAttr: function() {
			return this._viewportReady ? this._interactors.get("move").getEnabled() : false;
		},
		
		// =======================================================================
		//
		// KEY INTERACTORS:
		//
		_keyInteractors: null,
		
		_initializeKeyInteractors: function() {
			//
			//	tags:
			//		private
			//
			this._keyInteractors = new KeyboardInteractorManager(this);
			
			this._keyInteractors.add(Navigator.KeyInteractorId, new Navigator().initialize(this));
			this._keyInteractors.add(SelectionKeyInteractor.KeyInteractorId, new SelectionKeyInteractor().initialize(this));
			this._keyInteractors.add(PanKeyInteractor.KeyInteractorId, new PanKeyInteractor().initialize(this));
			this._keyInteractors.add(ZoomKeyInteractor.KeyInteractorId, new ZoomKeyInteractor().initialize(this));
			this._keyInteractors.add(MoveKeyInteractor.KeyInteractorId, new MoveKeyInteractor().initialize(this));
			var edition = new SwitchModeInteractor();
			this._keyInteractors.add(SwitchModeInteractor.KeyInteractorId, edition.initialize(this));
			edition.enterNavigation();
			this._keyInteractors.activate();
		},
		
		getKeyInteractor: function(id) {
			// summary: Gets the key interactor with the specified id.
			return this._keyInteractors.get(id);
		},
	
		assignKeyInteractorInputs: function(iid, assignments) {
			// summary: Changes assignments of keys for a given interactor.
			// example: Change the multi-selection key to Shift+Spacebar:
			//			var assignment = {
			//				multiSelect:{
			//					hotKey: dojo.keys.SPACE,
            //					filter: {alt:false,shift:true}
            //					}
            //				};
			//			diagram.assignKeyInteractorInputs("Selection",assignment);
			this.getKeyInteractor(iid).assignInputs(assignments);
		},
		
		getKeyboardEventSource: function() {
			var wai = this._getWaiNode(this._graph);
			return wai || this.focusNode || this.domNode;
		},
		
		// =======================================================================
		//
		//	Adorners
		//
	
		_initializeAdorners: function() {
			var v = this.getViewport();
			
			this._adornerTemplates = ibm_ilog.diagram.templates.defaultAdornerTemplates.clone();
			
			v.whenLoaded(lang.hitch(this, function() {
				this._adornerLayer = v.getOverlayLayer().createGroup();
			}));
		},
		
		
		_createAdorners: function() {
			this._focusedNodeAdorner = this.createAdorner(Adorner, this._adornerTemplates, "NodeFocusAdorner");
			
			this._focusedLinkAdorner = this.createAdorner(LinkAdorner, this._adornerTemplates, "LinkFocusAdorner");
	
		},
		
		// =======================================================================
		//
		//	WAI-ARIA / A11Y
		//
		
		tabIndex: "-1",
		
		//
		//	disableGraphWai: boolean
		//		Disables the infrastructure used for assigning WAI roles. (Initialization-only attribute).
		//        
		disableGraphWai: false,
		
		//
		//	enableGraphWaiDescribedBy: boolean
		//		Used to enable description for wai structure
		//
		enableGraphWaiDescribedBy: false,
		
		//
		//	waiNodeLabelFunction: Function?
		//		This function is used to compute the WAI label text for a node.
		//
		waiNodeLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiLinkLabelFunction: Function?
		//		This function is used to compute the WAI label text for a link.
		//
		waiLinkLabelFunction: _defaultWaiLabelFunction,
		
		//
		//	waiSubgraphLabelFunction: Function?
		//		This function is used to compute the WAI label text for a subgraph.
		//
		waiSubgraphLabelFunction: _defaultWaiLabelFunction,
		
		//	waiGraphLabelFunction: Function?
		//		This function is used to compute the WAI label for the graph.
		//
		waiGraphLabelFunction: function(graph) {
			return "Graph";
		},
		
		
		
		//
		//	waiNodeRoleFunction: Function?
		//		This function is used to compute the WAI role for a node.
		//
		waiNodeRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiLinkRoleFunction: Function?
		//		This function is used to compute the WAI role for a link.
		//
		waiLinkRoleFunction: function(ge) {
			return "listitem";
		},
		
		//
		//	waiSubgraphRoleFunction: Function?
		//		This function is used to compute the WAI role for a subgraph.
		//
		waiSubgraphRoleFunction: function(ge) {
			return "group";
		},
		
		//	waiGraphRoleFunction: Function?
		//		This function is used to compute the WAI role for the graph.
		//
		waiGraphRoleFunction: function(graph) {
			return "list";
		},
		
		
		
		//
		//	waiNodeDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a node.
		//
		waiNodeDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiLinkDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a link.
		//
		waiLinkDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//
		//	waiSubgraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description text for a subgraph.
		//
		waiSubgraphDescriptionFunction: _defaultWaiDescriptionFunction,
		
		//	waiGraphDescriptionFunction: Function?
		//		This function is used to compute the WAI description for the graph.
		//
		waiGraphDescriptionFunction: function(graph) {
			return "";
		},
		
		
		
		_bindWaiNode: function( /*ibm_ilog.diagram.GraphElement|ibm_ilog.diagram.Graph*/ge, /*div*/ waiNode, /*label*/ waiDesc) {
			//
			//	summary:	
			//		Adds private WAI information to graph element and the WAI node. 
			//
			//	tags:
			//		private
			//
			ge.__diagramWaiNode = waiNode;
			ge.__diagramWaiDesc = waiDesc;
			waiNode.__graphElement = ge;
		},
		
		_onWaiFocus: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has received focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_onWaiBlur: function(e) {
			//
			//	summary:
			//		Occurs when a WAI node has lost focus browser focus.
			//		Used for debugging for now.
			//
			//	tags:
			//		private
			//
		},
		
		_waiInvisibleStyle: {
			position: 'absolute',
			left: '0px',
			top: '0px',
			width: '100%',
			height: '100%',
			overflow:"visible"
		},
		
		_initializeGraphWai: function() {
			//
			//	summary:
			//		Initializes the infrastructure to support WAI screen reading on the graph elements.
			//
			//	tags:
			//		private
			//
			
			// when the widget receives focus, forward to appropiate WAI node 
			this._connections.push(connect.connect(this.domNode, "onfocus", this, "_onDiagramFocus"));
			
			// create root node at a hidden position
			var waiRoot = domctr.create('div', {style:{
				position: 'absolute',
				left: '0px',
				top: '0px',
				width: '100%',
				height: '100%',
				overflow: "visible",
				'z-index': "-2"
			}}, this.domNode, "first");
			
			this._createWaiNode(this._graph, this.id, waiRoot, this.waiGraphRoleFunction(this._graph));
			
			this._graph.__diagramWaiNode.setAttribute('tabIndex','0');
			
			this._setWaiState(this._graph.__diagramWaiNode, "label", this.waiGraphLabelFunction(this._graph));
			if (this.enableGraphWaiDescribedBy) {
				this._graph.__diagramWaiDesc.innerHTML = this.waiGraphDescriptionFunction(this._graph);
			}
			
			// connect to onfocus/onblur
			if (!has("ie")) {
				waiRoot.addEventListener("focus", lang.hitch(this, "_onWaiFocus"), true /*capturing phase*/);
				waiRoot.addEventListener("blur", lang.hitch(this, "_onWaiBlur"), true /*capturing phase*/);
			} else {
				// use focusin focusout which DOES bubble up (IE does NOT support capturing phase)
				this._connections.push(connect.connect(waiRoot, "focusin", lang.hitch(this, "_onWaiFocus")));
				this._connections.push(connect.connect(waiRoot, "focusout", lang.hitch(this, "_onWaiBlur")));
			}
			
			this._connections.push(connect.connect(this, "onNodeCreated", this, "_registerNodeWai"));
			this._connections.push(connect.connect(this, "onSubgraphCreated", this, "_registerSubgraphWai"));
			this._connections.push(connect.connect(this, "onLinkCreated", this, "_registerLinkWai"));
			
			this._connections.push(connect.connect(this, "onFocusedElementChanged", this, "_updateElementWaiFocus"));
			//
			this._connections.push(connect.connect(this.getSelection(), "onSelectionChanged", this, "_updateSelectedState"));
			
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/beforeDisposing', this, "_destroyWaiNode"));
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/refreshWaiLabel', this, "_updateWaiLabel"));
			
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/labelChanged', this, "_updateWaiLabel"));
			
			this._subscriptions.push(connect.subscribe("/ibm_diagram/Subgraph/collapsedChanged", this, "_updateSubgraphCollapsed"));
		},
		
		_getWaiNode: function(e) {
			//
			//	summary:
			//		Returns the WAI node associated with a graph element.
			//
			//	tags:
			//		private
			//
			return e.__diagramWaiNode;
		},
		
		_onDiagramFocus: function(subgraph) {
			//
			//	summary:
			//		Used when the widget receives HTML focus. Forwards it to the current focused element.
			//
			//	tags:
			//		private
			//
			//	TODO: check this is needed (maybe tabTndex is always enough).
			//
			var wainode = this._getWaiNode(this._focusedElement|| this._graph)
			if(wainode) {
				focus.focus(wainode);
			}
		},
		
		_updateSubgraphCollapsed: function(subgraph) {
			//
			//	summary:
			//		Used when a subgraph collapsed state has changed. Updates the corresponding WAI property. 
			//
			//	tags:
			//		private
			//
			var wainode = this._getWaiNode(subgraph);
			if(wainode) {
				this._setWaiState(wainode, "expanded", !subgraph.isCollapsed());
			}
		},
		
		_updateSelectedState: function(selected, deselected) {
			//
			//	summary:
			//		Used a graph element is selected or deselected. Updates the correspondig WAI attribute.
			//
			//	tags:
			//		private
			//
			var selection = this.getSelection();
			array.forEach(deselected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						this._setWaiState(wai, "selected", false);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
			array.forEach(selected, function(e) {
				var ge = selection._asGraphElement(e);
				if (ge) {
					var wai = this._getWaiNode(ge);
					if (wai) {
						this._setWaiState(wai, "selected", true);
					} else {
						// R.warn("MissingWaiNode", e);
					}
				}
			}, this);
		},
		
		_updateElementWaiFocus: function(old, current) {
			//
			//	summary:
			// 		Change wai focus to node associated with the current focused element position
			//
			//	tags:
			//		private
			//
			var oldWai = (old && this._getWaiNode(old)) || this._getWaiNode(this._graph);
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);

			//	TODO: [av] try aria-activedescendant instead?

			oldWai.setAttribute("tabIndex", "-1");
			curWai.setAttribute("tabIndex", "0");

			focus.focus(curWai);
		},

		_htmlRefocus: function() {
			//
			//	summary:
			//		Set the HTML focus into the appropiate component based on the current widget state.
			//
			var current = this.getFocusedElement();  
			var curWai = (current && this._getWaiNode(current)) || this._getWaiNode(this._graph);
			focus.focus(curWai);
		},
		
		_getFunctionTriad: function(pre, post) {
			//
			//	summary:
			// 		helper. returns a {N:L:S:} for use with GraphUtil.dispatchClass().
			//
			//	tags:
			//		private
			//
			if (pre && pre.length) {
				return {
					N: this[pre + "Node" + post + "Function"],
					L: this[pre + "Link" + post + "Function"],
					S: this[pre + "Subgraph" + post + "Function"]
				};
			} else {
				return {
					N: this["node" + post + "Function"],
					L: this["link" + post + "Function"],
					S: this["subgraph" + post + "Function"]
				};
			}
		},
		
		_updateWaiDescription: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI description text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiDescription) {
				ge.__diagramWaiDescription.innerHTML = gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Description"));
			}
		},
		
		_updateWaiLabel: function(ge) {
			//
			//	summary:
			// 		Regenerates the WAI label text of a graph element.
			//
			//	tags:
			//		private
			//
			if (ge && ge.__diagramWaiNode) {
				this._setWaiState(ge.__diagramWaiNode, 'label', gu.dispatchClass(ge, arguments, this, this._getFunctionTriad("wai", "Label")));
			}
		},
		
		_createWaiNode: function(element, id, waiParent, role) {
			//
			//	summary:
			//		create node div for WAI
			//
			//	tags:
			//		private
			//
			var waiNodeId = id + "_waiNode";
			var waiLabelId = id + "_waiLabel";
			var waiDescId = id + "_waiDesc";
			
			var atts = lang.mixin({
				id: waiNodeId,
				tabIndex: -1
			}, {style:this._waiInvisibleStyle});
			
			var geWaiNode = domctr.create("div", atts, waiParent);
			var geWaiDesc;
			if (this.enableGraphWaiDescribedBy) {
				geWaiDesc = domctr.create("label", lang.mixin({
					id: waiDescId,
					'for': waiNodeId
				}, {style:this._waiInvisibleStyle}), waiParent);
			}
			
			domattr.set(geWaiNode, "role", role);
			
			if (this.enableGraphWaiDescribedBy) {
				this._setWaiState(geWaiNode, 'describedBy', waiDescId);
			}
			
			this._bindWaiNode(element, geWaiNode, geWaiDesc);
		},
		
		_registerGraphElementWai: function(ge, role) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Graph Element
			//
			//	tags:
			//		private
			//
			
			// get div parent of parent
			
			var container = ge.getParent(), parentWaiNode;
			
			if (!container || !container.__diagramWaiNode) {
				// R.warn("MissingWaiNode", container);
				return;
			}
			
			parentWaiNode = container.__diagramWaiNode;
			
			// create node div for WAI
			this._createWaiNode(ge, ge.getId(), parentWaiNode, role);
			
			this._updateWaiLabel(ge);
			this._updateWaiDescription(ge);
		},
		
		_destroyWaiNode: function(ge) {
			if (ge.__diagramWaiLabel) {
				domctr.destroy(ge.__diagramWaiLabel);
			}
			if (ge.__diagramWaiNode) {
				domctr.destroy(ge.__diagramWaiNode);
			}
		},
		
		_registerLinkWai: function(link) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Link
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(link, this.waiLinkRoleFunction());
			
		},
		
		_registerNodeWai: function(node) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Node
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(node, this.waiNodeRoleFunction());
			
		},
		
		_registerSubgraphWai: function(subgraph) {
			//
			//	summary:
			//		Register a div node for Wai roles for a recently created Subgraph
			//
			//	tags:
			//		private
			//
			
			this._registerGraphElementWai(subgraph, this.waiSubgraphRoleFunction());
			
			this._updateSubgraphCollapsed(subgraph);
			subgraph.getGraph().__diagramWaiNode = subgraph.__diagramWaiNode;
			
		},
		
		_setWaiState: function(node, state, value){
			node.setAttribute("aria-"+state, value);
		}
	};
	
	return _Diagram;

});

},
'ibm_ilog/diagram/interactors/PanInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/PanInteractor", [
"dojo/_base/declare",
"./DragInteractor",
"../util/Geometry"
],
function(
declare,
DragInteractor,
g
){

/*=====
var DragInteractor = ibm_ilog.diagram.interactors.DragInteractor;
=====*/

var PanInteractor =
declare("ibm_ilog.diagram.interactors.PanInteractor", [DragInteractor], {
	//
	//	summary:
	//		A PanInteractor controls the process of viewport panning
	//
	//
	//	_panViewStart: Rect
	//		initial position of the view rectangle when the current pan process started
	//	
	_panViewLast: null,

	_dragCursor: "move",

	_getInitialEventSource: function () {
		return this._viewport.getEventSource();
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		var cs = this.inherited(arguments);
		cs.start.filter = this._buildInputFilter({shift:true,button:0});
		return cs;
	},

	_dragPreStart: function (e) {

		this.inherited(arguments);

		this._firstViewRect = this._viewport.getViewRect();

		var v = this._viewport;
	},

	_dragMove: function (e) {

		this.inherited(arguments);

		var v = this._viewport;

		var viewMovement = g.mulPoint(this._totalDelta, -1 / v.getZoom()); // scale delta to zoom level to get delta in model coords
		this._panViewLast = v.setViewRect(g.moveRect(this._firstViewRect, viewMovement), {instant:true});
	},

	_dragEnd: function (e) {
		var v = this._viewport;

		this.inherited(arguments);
	},

	_dragTouchPreStart: function (e) {

		this.inherited(arguments);

		this._firstViewRect = this._viewport.getViewRect();

	},

	_dragTouchMove: function (e) {
		this.inherited(arguments);
		if (e.touches.length > 1){
			return false;
		}
		
		var v = this._viewport;

		var viewMovement = g.mulPoint(this._totalDelta, -1 / v.getZoom()); // scale delta to zoom level to get delta in model coords
		this._panViewLast = v.setViewRect(g.moveRect(this._firstViewRect, viewMovement), {instant:true});
	},

	_dragTouchEnd: function (e) {
		this.inherited(arguments);
		if (e.touches.length > 0){
			this._firstViewRect = this._viewport.getViewRect();			
			return false;
		}
	},

	_setOnShapes: function ( /*Boolean*/ b) {
		// [AV] TODO this method needs refactoring
		if (b) {
			// background and shapes
			this._connections.start.src = this._viewport.getEventSource();
		} else {
			// only background
			this._connections.start.src = this._viewport.getEventSourceBackground();
		}
	},

	setOnShapes: function ( /*Boolean*/ b) {
		this._setOnShapes(b);
		this.reactivate();
	}
});

return PanInteractor;

});

},
'ibm_ilog/diagram/widget/Diagram':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/Diagram", [
"../main",
"require",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/array",
"dojo/_base/json",
"dojo/_base/config",
"dojo/_base/connect",
"dojo/dom-geometry",
"dojo/dom-construct",
"dojo/dom-style",
"dijit/_Widget",
"dijit/registry",
"dojox/gfx/matrix",
"dojox/gfx/utils",
"dojox/collections/ArrayList",
"dojox/collections/Dictionary",
"../_base/dojoext",
"./DataConnector",
"./DivViewport",
"../overview/OverviewTopLevelTemplateRenderer",
"../widget/OverviewHudComponent",
"../widget/ZoomHudComponent",
"../interactors/PanInteractor",
"../interactors/ZoomInteractor",
"../interactors/SelectionInteractor",
"../interactors/InteractorManager",
"../data/dtlbinding",
"../util/ErrorReporter",
"../util/DeferredInitialization",
"../util/Geometry",
"../util/GraphUtil",
"../style",
"../Graph",
"../Node",
"../Link",
"../Subgraph",
"../Selection",
"../Selectable",
"dojo/has!config-isBidiInDojoDiagrammer?../util/BidiUtil:",
"dojo/has!config-diagramForMobile?:./_Diagram",
"../data/StoreApi"
], function(
iid,
require,
declare,
lang,
array,
json,
config,
connect,
domgeo,
domctr,
domstyle,
_Widget,
registry,
matrix,
gfxutils,
ArrayList,
Dictionary,
dojoext,
DataConnector,
DivViewport,
OverviewTopLevelTemplateRenderer,
OverviewHudComponent,
ZoomHudComponent,
PanInteractor,
ZoomInteractor,
SelectionInteractor,
InteractorManager,
dtlbinding,
R,
DeferredInitialization,
g,
gu,
_style,
Graph,
Node,
Link,
Subgraph,
Selection,
Selectable,
BidiUtil,
_Diagram,
StoreApi
){

	var Size = g.Size;
	
	var Tooltip;
	
	var _emptyEntityHandler = function(entity, event) {
	};
	
/*=====
var _Widget = dijit._Widget;
var DataConnector = ibm_ilog.diagram.widget.DataConnector;
var DeferredInitialization = ibm_ilog.diagram.util.DeferredInitialization;
=====*/

	var Diagram =
	declare('ibm_ilog.diagram.widget.Diagram', [_Widget, DataConnector, DeferredInitialization], {
		// summary:
		//		The Diagram widget is the main entry point to the Dojo Diagrammer component.
		// description:
		//		The purpose of the Dojo Diagrammer component is to display <em>graphs</em> (or <em>diagrams</em>)
		//		(that is, graphical representations consisting of nodes connected by links)
		//		in a Web browser page. The Dojo Diagrammer component is fully implemented in JavaScript
		//		and is based on the Dojo Toolkit. Dojo Diagrammer uses DojoX GFX to render the diagram.
		//		<p>
		//		The Diagram widget is the Dijit widget that is commonly used as an entry point to Dojo Diagrammer,
		//		and provides the following main features:
		//		<ul>
		//		<li><strong>Data store connection</strong>
		//		The Diagram widget can load a Dojo data store that implements the dojo.data.api.Read interface.
		//		For this, the data store must be set as the 'nodesStore' attribute of the Diagram.
		//		Each item of the data store will typically be displayed as a node in the graph.
		//		The appearance of the nodes is defined by a <em>template</em>, specified using the 'nodeTemplate'
		//		attribute (or the nodeTemplateFunction to define different templates according to each item).
		//		The Diagram widget will also create links to represent the relationships between nodes.
		//		These relationships must be defined by specifying one or several of the following attributes:
		//		childBinding, parentBinding, successorsBinding, predecessorsBinding. For example, if
		//		successorsBinding attribute is set to "nextItems", the value if the "nextItems" property
		//		of the data items is expected to be an array of identifiers that define
		//		the set of items to which the current item should be connected by links.
		//		</li>
		//		<li><strong>Viewport (scrolling and zooming)</strong>
		//		If the diagram is larger than the client area of the Diagram, it can be scrolled using the
		//		scrollbars or using the mouse (the default interaction for panning is Shift-drag).
		//		The diagram can also be zoomed or unzoomed (the default interaction is Control-mouse wheel).
		//		Alternately, all viewport actions can be done by code through the viewport object, accessible by the getViewport()
		//		method.
		//		</li>
		//		<li><strong>Selection</strong>
		//		Nodes and links in the diagram can be selected. The set of selected objects can be accessed and modified
		//		through the selection object accessible by the getSelection() method.
		//		The appearance of selected objects can be customized through their template.
		//		</li>
		//		<li><strong>Event handling</strong>
		//		The Diagram handles mouse and keyboard events through objects called "interactors". Examples of interactors are
		//		the SelectionInteractor, the PanInteractor, the ZoomInteractor, etc. The predefined interactors can be
		//		enabled or disabled using attributes of the Diagram widget: 'select', 'zoom', 'pan', etc.
		//		In addition, the Diagram widget provides events to easily connect to mouse or key events on all the
		//		nodes or links of the diagram: onNodeClick, onNodeMouseDown, onLinkClick, etc.
		//		</li>
		//		<li><strong>Accessibility</strong>
		//		The Diagram widget provides accessibility services: the diagram can be described by a screen reader,
		//		and the user can navigate through the nodes and links using the keyboard.
		//		</li>
		//		<li><strong>Touch-enablement and mobile support</strong>
		//		The Diagram widget reacts to touch events on mobile platforms (or other touch-enabled devices):
		//		single touch-and-scroll pans the diagram, two-fingers pinch zooms the diagram.
		//		Dojo Diagrammer can be used with dojox.mobile.
		//		In addition, to optimize download size, some features are disabled on mobile:
		//		key events, focus adorners, accessibility, etc. You can explicitly disable these features by setting
		//		the 'diagramForMobile' flag to true in djConfig/dojoConfig. If diagramForMobile is not defined,
		//		the Diagram widget tries to auto-detect some well-known mobile user agents. Note that diagramForMobile
		//		is also set to true if the dojox.mobile module is loaded before the Diagram widget is loaded.
		//		You can set diagramForMobile to false explicitly to enable all features even on mobiles.
		//		</li>
		//		</ul>
		//		The Diagram widget uses the following basic graphic objects behind the scene to display the diagram:
		//		<ul>
		//		<li><strong>Graph</strong> (class ibm&#95;ilog.diagram.Graph): A subclass of dojox.gfx.Group that contains nodes and links. </li>
		//		<li><strong>Node</strong> (class ibm&#95;ilog.diagram.Node): Represents a node in a Graph, the contents of a Node are defined by a template. </li>
		//		<li><strong>Link</strong> (class ibm&#95;ilog.diagram.Link): Represents a link between two nodes. </li>
		//		<li><strong>Subgraph</strong> (ibm&#95;ilog.diagram.Subgraph): Represents a Node that contains a nested Graph. </li>
		//		</ul>
		//		It is possible to use the Dojo Diagramer component without using a Diagram widget.
		//		For this, you must create a GFX surface (or use an existing one), and create a Graph in it using surface.createGraph().
		//		You can then create nodes, links and subgraphs using the methods createNode, createLink and createSubgraph of the Graph
		//		object.
		
		_isIBMDiagramDiagram: true,
		
		_graph: null,
		
		_firePropertyChanged: function(pname, oldValue, newValue) {
			if (this.isInitialized()) {
				this.onPropertyChanged(pname, oldValue, newValue);
			}
		},
		
		onPropertyChanged: function(/*String*/name, /*Object*/ oldValue, /*Object*/ newValue) {
			// summary:
			//     An extension point invoked when a property has changed.
			// name: String: the property name.
			// oldValue: Object: the old property value.
			// newValue: Object: the new property value.
		},
		
		// Public properties
		
		// loadOnCreate: Boolean
		//		Indicates whether data are loaded automatically at the end of the widget creation phase (provided all required parameters have been set). 
		//		The default value is true. If set to false,	it requires a explicit call to Diagram.load() to load the data and build the graph.
		loadOnCreate: true,
		
		// nodesStore: dojo.data.api.Read 
		//		A data store that provides data item to be represented as Nodes.
		//		Must implement at least the dojo.data.api.Read and dojo.data.api.Identity interfaces.
		nodesStore: null,
		
		_setNodesStoreAttr: function(value) {
			var old = this._getNodesStore();
			if (old !== value) {
				this.nodesStore = this._setNodesStore(value);
				this._firePropertyChanged('nodesStore', old, value);
			}
			return this;
		},
		
		// An object providing the query criteria for the nodes data store.
		nodesQuery: null, // cast the value as an Object
		
		_setNodesQueryAttr: function(value) {
			// use the DataConnector getter to get the real old value and not the one set in the mixin() step
			var old = this._getNodesQuery(); 
			this.nodesQuery = this._setNodesQuery(value);
			this._firePropertyChanged('nodesQuery', old, value);
			return this;
		},
		
		// nodesQueryOptions: Object
		//                    An object containing additional options for the node query.
		nodesQueryOptions: {}, // cast the value as an Object
		_setNodesQueryOptionsAttr: function(value) {
			// use the DataConnector getter to get the real old value and not the one set in the mixin() step
			var old = this._getNodesQueryOptions();
			this.nodesQueryOptions = this._setNodesQueryOptions(value);
			this._firePropertyChanged('nodesQueryOptions', old, value);
			return this;
		},
		
		_setBindingAttr: function(value, pname) {
			if (value) {
				var func = lang.getObject(value); // global scope
				// We don't want "xBinding" treated as variable name only a Function or a String
				if (typeof(func) == "function") {
					value = func;
				}
			}
		    var uname = (pname.substr(0, 1).toUpperCase()) + pname.substr(1);
			// _s and _g refer to the corresponding functions defined in the DataConnector impl.
			// which all starts like '_setFoo'
			var _s = '_set' + uname;
			var _g = '_get' + uname;
			var old = this[_g]();
			if (old !== value) {
				this[pname] = this[_s](value);
				this._firePropertyChanged(pname, old, value);
			}
			return this;
		},
		
		// childBinding: String
		//     A string that indicates either the data item field name 
		//     defining the "child" relationship or a function name that returns the
		//     origin of a link for the specified data item.
		childBinding: "", // cast the value as a String
		
		_setChildBindingAttr: function(value) {
			// We don't want "xBinding" treated as variable name only a Function or a String
			return this._setBindingAttr(value, 'childBinding');
		},
		
		// parentBinding: String
		//     A string that indicates either the data item field name defining the
		//     "parent" relationship or a function name that returns the destination 
		//     of a link for the specified data item.
		parentBinding: "", // cast the value as a String
		
		_setParentBindingAttr: function(parentBinding) {
			return this._setBindingAttr(parentBinding, 'parentBinding');
		},
		
		// predecessorsBinding: String
		//     A string that indicates either the data item field name 
		//     defining the predecessors or a function name that returns the
		//     origins of a link for the specified data item.
		predecessorsBinding: "", // cast the value as a String
		
		_setPredecessorsBindingAttr: function(predecessorsBinding) {
			return this._setBindingAttr(predecessorsBinding, 'predecessorsBinding');
		},
		
		// successorsBinding: String
		//     A string that indicates either the data item field name defining the
		//     successors or a function name that returns the destination 
		//     of a link for the specified data item.
		successorsBinding: "", // cast the value as a String

		_setSuccessorsBindingAttr: function(successorsBinding) {
			return this._setBindingAttr(successorsBinding, 'successorsBinding');
		},
		
		// xBinding: String
		//     A string that indicates either the data item field name defining the
		//     "x location" associated with a nodesStore data item or a named function that 
		//     returns the "x location" associated with a nodesStore data item.
		xBinding: "", // cast the value as a String
		
		_setXBindingAttr: function(xBinding) {
			return this._setBindingAttr(xBinding, 'xBinding');
		},
		
		// yBinding: String
		//     A string that indicates either the data item field name defining the
		//     "y location" associated with a nodesStore data item or a named function that 
		//     returns the "y location" associated with a nodesStore data item.
		yBinding: "", // cast the value as a String

		_setYBindingAttr: function(yBinding) {
			return this._setBindingAttr(yBinding, 'yBinding');
		},
		
		// nodesGraphBinding: String
		//     A string that indicates either the data item field name defining the
		//     "node graphic data" associated with a nodeStore data item or a named function that 
		//     returns the "node graphic data" associated with a linksStore data item.
		nodesGraphBinding: "", // cast the value as a String

		_setNodesGraphBindingAttr: function(nodesGraphBinding) {
			return this._setBindingAttr(nodesGraphBinding, 'nodesGraphBinding');
		},
		
		// nodesGraphProperties: String
		//           A comma-separated list of properties to save if nodesGraphBinding is specified.
		nodesGraphProperties: "",
		
		_setNodesGraphPropertiesAttr: function(value) {
			var old = this._getNodesGraphProperties();
			if (old !== value) {
				this.nodesGraphProperties = this._setNodesGraphProperties(value);
				this._firePropertyChanged('nodesGraphProperties', old, value);
			}
			return this;
		},
		
		// centerOnLocation: Boolean
		//     If true, the values provided by xBinding/yBinding atributes represent the center of the node.
		//     If false, the values provided by xBinding/yBinding atributes represent the top left-hand corner of the node.
		centerOnLocation: false,
		
		_setCenterOnLocationAttr: function(value) {
			var old = this._getCenterOnLocation();
			if (old !== value) {
				this.centerOnLocation = this._setCenterOnLocation(value);
				this._firePropertyChanged('centerOnLocation', old, value);
			}
			return this;
		},
		
		// linksStore: dojo.data.api.Read
		//             A data store that provides data item to be represented as Links when the
		//             nodes relationship cannot be implicitly deducted from the nodes store data structure.
		//             Must implement at least the dojo.data.api.Read and
		//             dojo.data.api.Identity interfaces.
		linksStore: null,
		
		_setLinksStoreAttr: function(value) {
			var old = this._getLinksStore();
			if (old !== value) {
				this.linksStore = this._setLinksStore(value);
				this._firePropertyChanged('linksStore', old, value);
			}
			return this;
		},
		
		// linksQuery: Object
		//     An object providing the query criteria for the links datas store.		
		linksQuery: null, // cast the value as a String
		
		_setLinksQueryAttr: function(value) {
			var old = this._getLinksQuery();
			this.linksQuery = this._setLinksQuery(value);
			this._firePropertyChanged('linksQuery', old, value);
			return this;
		},
		
		// linksQueryOptions: Object
		//                    An object containing additional options for the links query.
		linksQueryOptions: {}, // cast the value as an Object

		_setLinksQueryOptionsAttr: function(value) {
			var old = this._getLinksQueryOptions();
			this.linksQueryOptions = this._setLinksQueryOptions(value);
			this._firePropertyChanged('linksQueryOptions', old, value);
			return this;
		},
		
		// linksGraphBinding: String
		//     A string that indicates either the data item field name defining the
		//     "link graphic data" associated with a linksStore data item or a named function that 
		//     returns the "link graphic data" associated with a linksStore data item.
		linksGraphBinding: "", // cast the value as a String
		
		_setLinksGraphBindingAttr: function(linksGraphBinding) {
			return this._setBindingAttr(linksGraphBinding, 'linksGraphBinding');
		},
		
		// linksGraphProperties: String
		//     A comma-separated list of properties to save if linksGraphBinding is specified.
		linksGraphProperties: "",
		
		_setLinksGraphPropertiesAttr: function(value) {
			var old = this._getLinksGraphProperties();
			if (old !== value) {
				this.linksGraphProperties = this._setLinksGraphProperties(value);
				this._firePropertyChanged('linksGraphProperties', old, value);
			}
			return this;
		},
		
		// startNodeBinding: String
		//              A string that indicates either the data item field name 
		//              defining the "start" of a link or a function name that returns the
		//              origin of a link for the specified data item.
		startNodeBinding: "start", // cast the value as a String and set the default as field with name of "start"
		
		_setStartNodeBindingAttr: function(startNodeBinding) {
			return this._setBindingAttr(startNodeBinding, 'startNodeBinding');
		},

		// endNodeBinding: String
		//     A string that indicates either the data item field name defining the
		//     "end" of a link or a function name that returns the destination 
		//     of a link for the specified data item.
		endNodeBinding: "end", // cast the value as a String and set the default as field with name of "end"

		_setEndNodeBindingAttr: function(endNodeBinding) {
			return this._setBindingAttr(endNodeBinding, 'endNodeBinding');
		},
		
		// createLinksForHierarchy: Boolean
		//     If true, the hierarchy is rendered using nodes and links.
		//     If false, it is rendered using subgraphs and nodes
		createLinksForHierarchy: true,
		
		_setCreateLinksForHierarchyAttr: function(value) {
			var old = this._getCreateLinksForHierarchy();
			if (old !== value) {
				this.createLinksForHierarchy = this._setCreateLinksForHierarchy(value);
				this._firePropertyChanged('createLinksForHierarchy', old, value);
			}
			return this;
		},
		
		// textDir: String
		//     A string that indicates the text direction of displayed text
		textDir: "",   
		
		_setTextDirAttr: function(value) {
		    if (config.isBidiInDojoDiagrammer) {
		        if (this._graph)
		            this._graph.setTextDir(value);
	  	        if (this._annotations) {
		           array.forEach(this._annotations,function(a) {
				        if (a.setTextDir)
				            a.setTextDir(value);
			            });
    	        }    
		    }
		},		
		
		//////////////////////////////////////////////////////////////////////////////
		//	Viewport Properties
		//

		viewportClass: "DivViewport",
		
		//
		//	viewportAirFactor: [markup] Boolean
		//		The minimum navigable space to the sides of the current content bounds as a percentage
		//		with respect to the current model bounding box.
		viewportAirFactor: 0.0,
		_setViewportAirFactorAttr: function(value) {
			return this._setViewportAttr("AirFactor",value); 
		},
		_getViewportAirFactorAttr: function(){
			return this._getViewportAttr("AirFactor");
		},

		//
		// viewportAnimated: [markup] Boolean
		//		Boolean flag to indicate that viewport movements are animated by default.
		viewportAnimated: false,
		_setViewportAnimatedAttr: function(value) {
			return this._setViewportAttr("Animated",value); 
		},
		_getViewportAnimatedAttr: function(){
			return this._getViewportAttr("Animated");
		},

		//
		// viewportMaxZoom: [markup] Number
		//		Defines how much the view rectangle can zoom in. Unlimited if 0.
		viewportMaxZoom: 0,
		_setViewportMaxZoomAttr: function(value) {
			return this._setViewportAttr("MaxZoom",value); 
		},
		_getViewportMaxZoomAttr: function(){
			return this._getViewportAttr("MaxZoom");
		},

		//
		//	viewportMinZoom: [markup] Number
		//		Defines how much the view rectangle can zoom out. If 0, the limit is based on the contentRectangle.
		//
		viewportMinZoom: 0,
		_setViewportMinZoomAttr: function(value) {
			return this._setViewportAttr("MinZoom",value); 
		},
		_getViewportMinZoomAttr: function(){
			return this._getViewportAttr("MinZoom");
		},

		//
		//	viewportResizeMode: [markup] Number
		//		Resize mode of the vieport. See Viewport.js
		//
		viewportResizeMode: 1,
		_setViewportResizeModeAttr: function(value) {
			return this._setViewportAttr("ResizeMode",value); 
		},
		_getViewportResizeModeAttr: function(){
			return this._getViewportAttr("ResizeMode");
		},

		
		
		// Initialize viewport properties.
		// The method is called during the postCreate stage to apply the markup properties
		// that have been deferred (viewport is not fully initialized when the custom
		// setters are invoked by the dijit attributes engine).
		_initializeViewportProperties: function(){
			if (this.overviewSize != undefined)
				this._setOverviewSizeAttr(this.overviewSize);
			if (this.overviewPan != undefined)
				this._setOverviewPanAttr(this.overviewPan);
			if (this.overviewZoom!= undefined)
				this._setOverviewZoomAttr(this.overviewZoom);
			if (this.showZoomFactor)
				this._setShowZoomFactorAttr(this.showZoomFactor);
			if (this.showZoomFeedback)
				this._setShowZoomFeedbackAttr(this.showZoomFeedback);
			if (this.overview)
				this._setOverviewAttr(this.overview);
			if (!this.hud)
				this._setHudAttr(this.hud);
		},

		
		
		//
		//	overviewSize: Size
		//		The size of the embedded Overview
		//		
		overviewSize: Size(100, 100),
		
		_setOverviewSizeAttr: function(value) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewSizeAttr();
			this.overviewSize = value;
			if (this._hudOverview) {
				this._hudOverview.setSize(value);
			}
			this._firePropertyChanged('overviewSize', old, value);
			return this;
		},
		
		_getOverviewSizeAttr:function(){
			return this._hudOverview ? this._hudOverview.getSize() : this.overviewSize;
		},
		
		
		//
		//	overviewPan: boolean
		//		Activate or deactivate panning in the hud overview
		//		
		overviewPan: true,
		
		_setOverviewPanAttr: function(value) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewPanAttr();
			this.overviewPan = value;
			if (this._hudOverview) {
				this._hudOverview.setPan(value);
			}
			this._firePropertyChanged('overviewPan', old, value);
			return this;
		},
		
		_getOverviewPanAttr:function(){
			return this._hudOverview ? this._hudOverview.getPan() : this.overviewPan;
		},

		
		//
		//	overviewZoom: boolean
		//		Activate or deactivate panning in the hud overview
		//		
		overviewZoom: true,
		
		_setOverviewZoomAttr: function(value) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewZoomAttr();
			this.overviewZoom = value;
			if (this._hudOverview) {
				this._hudOverview.setZoom(value);
			}
			this._firePropertyChanged('overviewZoom', old, value);
			return this;
		},
		
		_getOverviewZoomAttr:function(){
			return this._hudOverview ? this._hudOverview.getZoom() : this.overviewZoom;
		},

		
		//
		//	showZoomFactor: [markup] Boolean
		//		Display the current zoom factor
		//		
		showZoomFactor: false,
		
		_setShowZoomFactorAttr: function(enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getShowZoomFactorAttr();
			if (old !== enabled) {
				this.showZoomFactor = enabled;
				if (!this._hudZoom) {
					this._hudZoom = new ZoomHudComponent(this._viewport);
				}
				this._hudZoom.setEnabled(enabled);
				this._firePropertyChanged('showZoomFactor', old, enabled);
			}
			return this;
		},
		
		_getShowZoomFactorAttr: function() {
			return this._hudZoom ? this._hudZoom.isEnabled() : false;
		},
		
		
		//
		//	showZoomFeedback: [markup] Boolean
		//		Display a visual feedback while zooming with animation 
		//		
		showZoomFeedback: false,
		
		_setShowZoomFeedbackAttr: function(/*Boolean*/enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getShowZoomFeedbackAttr();
			if (old !== enabled) {
				this.showZoomFeedback = enabled;
				this._viewport.setHudZoomFeedbackEnabled(enabled);
				this._firePropertyChanged('showZoomFeedback', old, enabled);
			}
			return this;
		},
		
		_getShowZoomFeedbackAttr: function() {
			return this._viewportReady ? this._viewport.isHudZoomFeedbackEnabled() : false;
		},
		
		
		
		//
		//  overview: [markup] Boolean
		//      Use the embedded overview hud component
		//      
		overview: false,
		
		_setOverviewAttr: function(/*Boolean*/enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewAttr();
			if (old !== enabled) {
				this.overview = enabled;
				if (!this._hudOverview) {
					var renderer = this.overviewRenderer || new OverviewTopLevelTemplateRenderer();
					this._hudOverview = new OverviewHudComponent(this._viewport, this._graph, renderer, this.overviewSize);
					this._hudOverview.getOverview().setInteractorManager(this._interactors);
					this._hudOverview.setPan(this.overviewPan);
					this._hudOverview.setZoom(this.overviewZoom);
				}
				this._hudOverview.setEnabled(enabled);
				this._firePropertyChanged('overview', old, enabled);
			}
			return this;
		},
		
		_getOverviewAttr: function() {
			return this._hudOverview ? this._hudOverview.isEnabled() : false; 
		},


		
		//
		//  overviewRenderer: ibm_ilog.diagram.overview.OverviewRenderer
		//      Use renderer used in the embedded overview
		//      
		overviewRenderer: false,
		
		_setOverviewRendererAttr: function(/*ibm_ilog.diagram.overview.OverviewRenderer*/renderer) {
			if (!this._viewportReady)
				return;
			var old = this._getOverviewRendererAttr();
			if (old !== renderer) {
				this.overviewRenderer = renderer;
				if (this._hudOverview) {
					this._hudOverview.getOverview().setRenderer(renderer);
				}
				this._firePropertyChanged('overviewRenderer', old, renderer);
			}
			return this;
		},
		
		_getOverviewRendererAttr: function() {
			return this._hudOverview ? this._hudOverview.getOverview().getRenderer() : this.overviewRenderer; 
		},
		
		
		//
		//  hud: [markup] Boolean
		//      Use to disable the entire viewport hud.
		//      
		
		hud: true,
		
		_setHudAttr: function(enabled) {
			if (!this._viewportReady)
				return;
			var old = this._getHudAttr();
			if (old !== enabled) {
				this.hud = enabled;
				this._viewport.setHudEnabled(enabled);
				this._firePropertyChanged('hud', old, enabled);
			}
			return this;
		},
		
		_getHudAttr: function() {
			return this._viewportReady ? this._viewport.isHudEnabled() : true;
		},

		
		
		_setViewportAttr: function(uname, value) {
			if (!this._viewportReady)
				return;
			var old = this['_getViewport'+uname+'Attr']();
			if (old !== value) {
				var pname = 'viewport'+uname;
				this[pname] = value;
				this._viewport['set'+uname](value);
				this._firePropertyChanged(pname, old, value);
			}
			return this;
		},

		_getViewportAttr: function(uname) {
			var pname = 'viewport'+uname;
			if (!this._viewportReady) {
				return this[pname];
			} else {
				return this._viewport['get'+uname]();
			}
		},
		
		//////////////////////////////////////////////////////////////////////////////
		//  Interactor Properties
		//

		//
		//  pan: [markup] boolean
		//      enable or disable panning
		//
		pan: true,
		
		_setPanAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getPanAttr(); // use the getter as the interactor may have been configured directly
			if (old !== b) {
				this.pan = b;
				this._interactors.get('pan').setEnabled(b);
				this._firePropertyChanged('pan', old, b);
			}
			return this;
		},
		
		_getPanAttr: function() {
			return this._viewportReady ? this._interactors.get("pan").getEnabled() : true;
		},

		
		
		//
		//  pan: [markup] boolean
		//      enable or disable panning if clicking on shapes.
		//
		panOnShapes:false,
		
		_setPanOnShapesAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return;
			var old = this._getPanOnShapesAttr();
			if (old !== b) {
				this.panOnShapes = b;
				this._interactors.get("pan").setActOnShapes(b);
				this._firePropertyChanged('panOnShapes', old, b);
			}
			return this; 
		},
		
		_getPanOnShapesAttr: function( /*boolean*/b) {
			return this._viewportReady ? this._interactors.get("pan").getActOnShapes(b) : false;
		},
		
		
		
		
		//
		//  zoom: [markup] boolean
		//      enable or disable zooming
		//
		zoom: true,
		
		_setZoomAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getZoomAttr();
			if (old !== b) {
				this.zoom = b;
				this._interactors.get("zoom").setEnabled(b);
				this._firePropertyChanged('zoom', old, b);
			}
			return this;
		},
		
		_getZoomAttr: function() {
			return this._viewportReady ? this._interactors.get("zoom").getEnabled() : true;
		},
		
		
		
		
		//
		//  select: [markup] boolean
		//      enable or disable selecting entities
		//
		select: true,

		_setSelectAttr: function( /*Boolean*/b) {
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getSelectAttr();
			if (old !== b) {
				this.select = b;
				this._interactors.get("select").setEnabled(b);
				this._firePropertyChanged('select', old, b);
			}
			return this;
		},
		_getSelectAttr: function() {
			return this._viewportReady ? this._interactors.get("select").getEnabled() : true;
		},
		
		
		//
		//  marquee: [markup] boolean
		//      enable or disable marquee selection
		//
		marquee: true,
		
		_setMarqueeAttr: function( /*Boolean*/b) {
			if(!_Diagram) return;
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getMarqueeAttr();
			if (old !== b) {
				this.marquee = b;
				this._interactors.get("marquee").setEnabled(b);
				this._firePropertyChanged('marquee', old, b);
			}
			return this;
		},
		
		_getMarqueeAttr: function() {
			if(!_Diagram) return false;
			return this._viewportReady ? this._interactors.get("marquee").getEnabled() : true;
		},
		
		
		
		//
		//  wheelPan: [markup] boolean
		//      enable or disable scrolling with the mouse wheel (vertical panning)
		//
		wheelPan: true,
		
		_setWheelPanAttr: function( /*Boolean*/b) {
			if(!_Diagram) return;
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getWheelPanAttr();
			if (old !== b) {
				this.wheelPan = b;
				this._interactors.get("wheelPan").setEnabled(b);
				this._firePropertyChanged('wheelPan', old, b);
			}
			return this;
		},
		
		_getWheelPanAttr: function() {
			if(!_Diagram) return false;
			return this._viewportReady ? this._interactors.get("wheelPan").getEnabled() : true;
		},
		
		
		
		//
		//	adorners: [markup] boolean
		//		enable or disable adorners focus following mouse pointer
		//
		adorners: true,
		
		_setNodeAdornersFocusAttr: function( /*Boolean*/b) {
			if(!_Diagram) return;
			if (!this._viewportReady) 
				// at creation time, interactors are initialized in the postCreate stage.
				// Since customSetter are called before postCreate(), skip it in this case.
				return; 
			var old = this._getNodeAdornersFocusAttr();
			if (old !== b) {
				this.adorners = b;
				this._interactors.get("adorners").setEnabled(b);
				this._firePropertyChanged('adorners', old, b);
			}
			return this;
		},
		
		_getNodeAdornersFocusAttr: function() {
			if(!_Diagram) return false;
			return this._viewportReady ? this._interactors.get("adorners").getEnabled() : true;
		},
		
		//
		//  moveFunction: Function
		//      This function is an optional callback to configure how the MoveInteractor applies the movement deltas on entities.
		//      function(graphElement,originalTransform,delta) {
		//          //  summary:
		//          //      Concrete the movement to a graph element.
		//          //  graphElement:
		//          //      The graph element to move.
		//          //  originalTransform:
		//          //      The matrix representing the location from which the delta is computed.
		//          //  delta:
		//          //      The ammount to translate the entity from its original location.
		//      },  
		//
		moveFunction: null,
		
		//
		//  isMovableFunction: Function
		//      This function is an optional callback to configure how the MoveInteractor tests movability of elements
		//      function(graphElement) {
		//          //  summary:
		//          //      Return true if the given graph element is allowed to move
		//          //  graphElement:
		//          //      The graph element to test.
		//      }
		//
		isMovableFunction: null,
		
		
		//////////////////////////////////////////////////////////////////////////////
		// Events properties
		
		//
		//  nodeMenu:
		//      An optional dijit.Menu for nodes (property)
		//
		nodeMenu: null,
		
		//
		//  linkMenu:
		//      An optional dijit.Menu for links (property)
		//
		linkMenu: null,
		
		//
		//  subgraphMenu:
		//      An optional dijit.Menu for subgraphs (property)
		//
		subgraphMenu: null,
		
		//
		//  backMenu:
		//      An optional dijit.Menu for the diagram background (property)
		//
		backMenu: null,
		
		//////////////////////////////////////////////////////////////////////////////
		// Styling Properties
		
		// nodeStyle: Object: 
		//     A dictionary of property-value pairs that define global properties to apply on nodes.
		//     Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
		//     for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
		//     the node).
		nodeStyle: null,
		
		_setNodeStyleAttr: function(value) {
			var old = this._created ? this.nodeStyle : null;
			this.nodeStyle = value;
			this._firePropertyChanged('nodeStyle', old, value);
			return this;
		},
		
		// linkStyle: Object: 
		//     A dictionary of property-value pairs that define global properties to apply on links.
		//     Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
		//     for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
		//     the node).
		linkStyle: null,
		
		_setLinkStyleAttr: function(value) {
			var old = this._created ? this.linkStyle : null;
			this.linkStyle = value;
			this._firePropertyChanged('linkStyle', old, value);
			return this;
		},
		
		// subgraphStyle: Object: 
		//     A dictionary of property-value pairs that define global properties to apply on subgraphs.
		//     Properties must obey the Java get/set naming convention in order to be taken into account (i.e.
		//     for a property 'myProperty', there must be a getMyProperty and a setMyProperty methods defined on
		//     the node).
		subgraphStyle: null,
		
		_setSubgraphStyleAttr: function(value) {
			var old = this._created ? this.subgraphStyle : null;
			this.subgraphStyle = value;
			this._firePropertyChanged('subgraphStyle', old, value);
			return this;
		},
		
		//////////////////////////////////////////////////////////////////////////////
		// Templating Properties
		
		_setTemplateField: function(field, template) {
			var _field = '_' + field;
			this[_field] = this[field] = template;
			if (template) {
				if (typeof(template) == "string" && template.length > 0) {
					// if no DTL, convert it to plain JS object to prevent useless costly parsing
					if (!template.match(/{{|{%/)) 
						this[_field] = json.fromJson(template);
				}
				if (!(this[_field] instanceof dtlbinding.GfxTemplate)) 
					this[_field] = new dtlbinding.GfxTemplate(this[_field]);
			}
		},
		
		// nodeTemplate: String:
		//    A description object that defines the graphical representation of nodes.
		nodeTemplate: '',
		
		_setNodeTemplateAttr: function(template) {
			var old = this._created ? this.nodeTemplate : '';
			this._setTemplateField('nodeTemplate', template);
			this._firePropertyChanged('nodeTemplate', old, template);
			return this;
		},
		
		// nodeTemplateFunction: Function
		//    an optional function that computes the node template based on the item.
		//    The function receives the following parameters:
		//    item : the data item to create node from
		//    diagram : a reference to this diagram widget.
		nodeTemplateFunction: null,
		
		// createNodeFunction: Function
		//    An optional function that creates the ibm_ilog.diagram.Node instance for a given data
		//    item.
		//    The function receives the following parameters:
		//    item : the data item associated with this node.
		//    templateObj : A template object.
		//    graph : the target graph to create the node from. 
		//    diagram : a reference to this diagram widget.
		createNodeFunction: null,
		
		
		// subgraphTemplate: String:
		//    A description object that defines the graphical representation of subgraphs.
		subgraphTemplate: '',
		
		_setSubgraphTemplateAttr: function(/*String||Object*/template) {
			var old = this._created ? this.subgraphTemplate : '';
			this._setTemplateField('subgraphTemplate', template);
			this._firePropertyChanged('subgraphTemplate', old, template);
			return this;
		},
		
		// subgraphTemplateFunction: Function
		//    an optional function that computes the subgraph template based on the item.
		//    The function receives the following parameters:
		//    item : the data item to create subgraph from.
		//    diagram : a reference to this diagram widget.
		subgraphTemplateFunction: null,
		
		// createSubgraphFunction: Function
		//    An optional function that creates the ibm_ilog.diagram.Subgraph instance for a given data
		//    item.
		//    The function receives the following parameters:
		//    item : associated with this node.
		//    templateObj : the template object.
		//    graph : the target graph to create the subgraph from.
		//    diagram : a reference to this diagram widget.
		createSubgraphFunction: null,
		
		// linkTemplate: String:
		//    A description object that defines the graphical representation of links.
		linkTemplate: '',
		
		_setLinkTemplateAttr: function(/*String||Object*/template) {
			var old = this._created ? this.linkTemplate : '';
			this._setTemplateField('linkTemplate', template);
			this._firePropertyChanged('linkTemplate', old, template);
			return this;
		},
		
		// linkTemplateFunction: Function
		//    an optional function that computes the link template dynamically based on the item.
		//    The function receives the following parameters:
		//    linkItem : the link data item, if any.
		//    startNode: the link start node.
		//    endNode: the link end node.
		//    diagram : a reference to this diagram widget.
		linkTemplateFunction: null,
		
		// createLinkFunction: Function
		//    An optional function that creates the ibm_ilog.diagram.Link instance for a given data
		//    item.  
		//    The function receives the following parameters:
		//    linkItem : the link data item, if any.
		//    startNode: the link start node.
		//    endNode: the link end node.
		//    templateObj : the template object.
		//    graph : the parent graph of the link.
		//    diagram : a reference to this diagram widget.
		createLinkFunction: null,
		
		
		//////////////////////////////////////////////////////////////////////////////
		// Graph GraphLayout Properties
		
		_initializeLayoutProps:function(){
			if (this.nodeLayout)
				this._setNodeLayoutAttr(this.nodeLayout);
			if (this.automaticNodeLayout)
				this._setAutomaticNodeLayoutAttr(this.automaticNodeLayout);
			if (!this.nodeLayoutActive)
				this._setNodeLayoutActiveAttr(this.nodeLayoutActive);
			if (this.linkLayout)
				this._setLinkLayoutAttr(this.linkLayout);
			if (this.automaticLinkLayout)
				this._setAutomaticLinkLayoutAttr(this.automaticLinkLayout);
			if (!this.linkLayoutActive)
				this._setLinkLayoutActiveAttr(this.linkLayoutActive);
		},
		
		// summary: ibm_ilog.graphlayout.GraphLayout :
		//    The graph layout algorithm that will be executed on the nodes of this diagram.
		nodeLayout: null,
		
		_setNodeLayoutAttr: function(/*ibm_ilog.graphlayout.GraphLayout*/layout) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.getNodeLayout();
			if (old !== layout) {
				this.nodeLayout = layout;
				this._graph.setNodeLayout(layout);
				this._firePropertyChanged('nodeLayout', old, layout);
			}
			return this;
		},
		
		_getNodeLayoutAttr: function() {
			return this._viewportReady ? this._graph.getNodeLayout() : null;
		},
		
		//		Determines if the node layout must be executed automatically
		//		whenever the graph changes.
		automaticNodeLayout: false,
		
		_setAutomaticNodeLayoutAttr: function(/*boolean*/automatic) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isAutomaticNodeLayout();
			if (old !== automatic) {
				this.automaticNodeLayout = automatic;
				this._graph.setAutomaticNodeLayout(automatic);
				this._firePropertyChanged('automaticNodeLayout', old, automatic);
			}
			return this;
		},
		
		_getAutomaticNodeLayoutAttr: function() {
			return this._viewportReady ? this._graph.isAutomaticNodeLayout() : false;
		},
		
		// summary:
		//		Activates or deactivates the graph layout specified by setNodeLayout().
		nodeLayoutActive:true,
		
		_setNodeLayoutActiveAttr: function(/*boolean*/active) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isNodeLayoutActive();
			if (old !== active){
				this.nodeLayoutActive = active;
				this._graph.setNodeLayoutActive(active);
				this._firePropertyChanged('nodeLayoutActive', old, active);
			}
			return this;
		},
		
		_getNodeLayoutActiveAttr: function() {
			return this._viewportReady ? this._graph.isNodeLayoutActive() : true;
		},
		
		// summary: ibm_ilog.graphlayout.GraphLayout :
		//    The graph layout algorithm that will be executed on the links of this diagram.
		linkLayout: null,
				
		_setLinkLayoutAttr: function(/*ibm_ilog.graphlayout.GraphLayout*/layout) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.getLinkLayout();
			if (old !== layout) {
				this.linkLayout = layout;
				this._graph.setLinkLayout(layout);
				this._firePropertyChanged('linkLayout', old, layout);
			}
			return this;
		},

		_getLinkLayoutAttr: function() {
			return this._viewportReady ? this._graph.getLinkLayout() : null;
		},

		//		Determines if the link layout must be executed automatically
		//		whenever the graph changes.
		automaticLinkLayout: false,
		
		_setAutomaticLinkLayoutAttr: function(/*boolean*/automatic) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isAutomaticLinkLayout();
			if (old !== automatic) {
				this.automaticLinkLayout = automatic;
				this._graph.setAutomaticLinkLayout(automatic);
				this._firePropertyChanged('automaticLinkLayout', old, automatic);
			}
			return this;
		},
		
		_getAutomaticLinkLayoutAttr: function() {
			return this._viewportReady ? this._graph.isAutomaticLinkLayout() : false;
		},
		
		// summary:
		//		Activates or deactivates the link layout specified by setLinkLayout().
		linkLayoutActive: true,
		
		_setLinkLayoutActiveAttr: function(/*boolean*/active) {
			if (!this._viewportReady)
				return; 
			var old = this._graph.isLinkLayoutActive();
			if (old !== active) {
				this.linkLayoutActive = active;
				this._graph.setLinkLayoutActive(active);
				this._firePropertyChanged('linkLayoutActive', old, active);
			}
			return this;
		},
		
		_getLinkLayoutActiveAttr: function() {
			return this._viewportReady ? this._graph.isLinkLayoutActive() : true;
		},
		
		// END OF PROPERTIES
		// =======================================================================
		
		
		// =======================================================================
		//
		// WIDGET LIFECYCLE:
		//
		//  The following group methods define the process of construction, initialization, deinitialization and 
		//  related. These are mostly hooks into the Dijit framework.
		//
		
		_connections: null,
		_subscriptions: null,
		
		constructor: function() {
			this._connections = [];
			this._subscriptions = [];
		},
		
		postMixInProperties: function() {
			this.inherited(arguments);
			
			this._beginInit();
			
			// Create Viewport, which defines the templateString to use.
			this._createViewport();
			
			// Create Selection object
			this._createSelection();
			
			// register node events
			this._initializeNodeEvents();
			
			// initialize annotations infrastructure
			this._initializeAnnotations();
			
			// initialize adorner infrastructure
			this._initializeAdorners();

			// create interactor manager
			this._interactors = new InteractorManager();

		},
		
		postCreate: function() {
		
			this.inherited(arguments);
			
			this._initializeViewport();

			this.getViewport().whenLoaded(lang.hitch(this, function() {
				if (this["backMenu"]) {
					try {
						var thiz = this;
						this._viewport.getEventSourceBackground().connect("oncontextmenu", function(e) {
							thiz.lastMenuTarget = thiz._graph;
							thiz.lastMenuEvent = e;
						});
						this["backMenu"].bindDomNode(this._viewport.getEventSourceBackground().getEventSource());
					} catch (e) {
						R.warn("EventConnectionFailed", "oncontextmenu");
					}
				}
				
				this._graph = this.getViewport().getContent().createGraph();
				this._graph._diagram = this;
				if (config.isBidiInDojoDiagrammer)
				    this._graph.textDir = this.textDir;							
				
				if (!this.disableGraphWai) {
					this._initializeGraphWai();
				} else {
					this.set('tabIndex','0');
				}
				
				this._connections.push(connect.connect(this._graph, "onLayoutAnimationStart", this, function() {
					this._viewport.setRefreshContentsSuspended(true);
				}));
				
				this._connections.push(connect.connect(this._graph, "onLayoutAnimationEnd", this, function() {
					this._viewport.setRefreshContentsSuspended(false);
				}));
				
				this._viewportReady = true;
				
				// initialize viewport properties
				this._initializeViewportProperties();
				
				// initialize graph graphlayout properties
				this._initializeLayoutProps();
				
				// initialize interactors
				this._initializeInteractors();
				
				// initialize key interactors
				this._initializeKeyInteractors();
				
				this.enterStandardMode();
				
				// If there was a nodeStore defined on the widget then draw     
				if (this.loadOnCreate && this._getNodesStore()) {
					this.load();
				}
				
				this.postPostCreate();
				
				this._endInit();
			}));
		},
		
		postPostCreate: function() {
			//  summary:
			//      Hook executed just after Diagram postCreate. Used by DiagramEditor subclass.
		},
		
		
		onLoaded: function() {
			//  summary:
			//      Invoked when the diagram has loaded the data. 
		},
		
		resize: function(info) {
		  var doResize = true;
			// No size info passed: use the widget's marginBox
			if(!info || !info.w || !info.h || info.w <= 0 || info.h <= 0){ 
			  info = domgeo.getMarginBox(this.domNode);
			  doResize = false;
			}
			
			// We get negative sizes sometimes... (e.g. on Chrome/Canvas)
			if (!info || !info.w || !info.h || info.w <= 0 || info.h <= 0) 				
  	  	return;
	    
			// We get _viewport.getSurface() being null sometimes... (e.g. on IE8)
			if (this._viewport.getSurface()) {
				this._viewport.setSize({
					width: info.w,
					height: info.h
				},doResize);
			}
		},
		
		destroy: function() {
		
			if (this._interactors) {
				this._interactors.switchTo();
			}
			
			array.forEach(this._connections, connect.disconnect, dojo);
			
			array.forEach(this._subscriptions, connect.unsubscribe, dojo);
			
			this._destroyDataConnector();

			if (this._graph) {
				this._graph.clear(true); // true = call dispose() on children
			}
			this._graph = null;
			
			if (this._viewport) {
				this._viewport.destroy();
			}
			
			this.inherited(arguments);
		},
		
		reset: function() {
			// clear the selection
			this.getSelection().clear();
			// reset the dataconnector and the associated graphical shapes created from the stores
			this._reset();
			
			// clear any remaining shapes (those not created from datastores)
			this._graph.clear(/*dispose*/true);
			
			// Defect 1875
			// The layout instances do cleanup work when notified that the content of the
			// graph has changed. Hence, let's notify the layout:
			var nodeLayout = this._graph.getNodeLayout();
			var linkLayout = this._graph.getLinkLayout();
			var graphModelEvent = null;
			if (nodeLayout || linkLayout) {
				// We do not need a dojo.require for GraphLayoutModelEvent, because this is 
				// required by GraphLayout itself. Hence, if a layout instance is already
				// set on the graph, the event class has already been required.
				var GraphLayoutModelEvent = require("ibm_ilog/graphlayout/GraphLayoutModelEvent"); 
				graphModelEvent = new GraphLayoutModelEvent();
				graphModelEvent.setType(GraphLayoutModelEvent.STRUCTURE_CHANGED);
			}
			if (nodeLayout)
				nodeLayout.contentsChanged(graphModelEvent);
		  if (linkLayout)
				linkLayout.contentsChanged(graphModelEvent);
				
			this._viewport.reset();
			this.enterStandardMode();
			
			return this;
		},
		
		
		// =======================================================================
		//
		//	WAI-ARIA / A11Y
		//
		
		// moved to _Diagram.js
		
		disableGraphWai: true,

		// =======================================================================
		//
		//  VIEWPORT:
		//
		//      The viewport is an abstraction used to control the currently visualized area of the model.
		//
		//      see:
		//      widget.Viewport, widget.DivViewport
		//
		//
		
		//
		//  _viewport: ibm_ilog.diagram.widget.Viewport
		//      The viewport
		//
		_viewport: null,
		
		getViewport: function() {
			// summary:
			//     Gets the Viewport instance.
			return this._viewport;
		},
		
		visualizeBounds: function( /*Node|Link|Subgraph*/graphic, /*{expandCurrent:false,airFactor}*/ options) {
			//
			//	summary:
			//		Makes the bounds of a graph object visible, givin it an extra air between the 
			//		area and the borders of the viewport if possible. The view may be zoomed out 
			//		if required, but it is never zoomed in.
			//
			//	graphic:
			//		The object to make visible
			//
			//	options:
			//		Viewport options kwArgs
			//
			var b = graphic.getBounds(this.getGraph());
			if (!b) {
				return;
			}
			this._viewport.visualizeArea(b, options);
		},
		
		centerOnNode: function( /*Node|Link*/nodeOrLink, /*{animate}*/ options) {
			//
			//	summary:
			//		Centers the viewport on an object of the graph. Does not change the zoom level.
			//
			nodeOrLink = this.asGraphElement(nodeOrLink);
			var b = nodeOrLink.getBounds(this.getGraph());
			if (!b) 
				return;
			if (nodeOrLink._isIBMDiagramNode) {
				this._viewport.centerAt(g.getRectCenter(b), options);
			} else {
				if (nodeOrLink._isIBMDiagramLink) {
					//
					//	TODO: temporary, correct this when it's ready
					//
					this._viewport.centerAt(g.getRectCenter(b), options);
				}
			}
			
		},
		
		_createViewport: function() {
			//
			//	Creates the Viewport object
			//
			//	tags:
			//		private
			//
			
			// if an exotic viewport class is specified, we assume it is already loaded so a simple require should get it.
			this._viewport = new (this.viewportClass == "DivViewport" ? DivViewport : require("./"+this.viewportClass))();
			this.templateString = this._viewport.templateString();
			
			
		},
		
		buildRendering: function(){
			// there are no variables in the template so we can cache the DOM tree
			var node = domctr.toDom(this.templateString);
			if(node.nodeType != 1){
				throw new Error("Invalid template: " + this.templateString);
			}
			this.domNode = node;
			var nodes = node.getElementsByTagName("*");
			for (var i = -1; i < nodes.length; i++){
				var n = (i == -1) ? node : nodes[i];
				var attachPoint = n.getAttribute("dojoAttachPoint");
				if (attachPoint == 'canvasNode') {
					domstyle.set(n, "overflow", 'hidden');
				}
				this[attachPoint]= n;
			}
			
			//this.inherited(arguments);
			
			this._fillContent(this.srcNodeRef);
			
		},
		
		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},
		
		
		_initializeViewport: function() {
			//
			//	Creates the Viewport object
			//
			//	tags:
			//		private
			//
			this._viewport.initialize(this);
			
			this._viewport.setAirFactor(this.viewportAirFactor);
			this._viewport.setAnimated(this.viewportAnimated);
			this._viewport.setMaxZoom(this.viewportMaxZoom);
			this._viewport.setMinZoom(this.viewportMinZoom);
			this._viewport.setResizeMode(this.viewportResizeMode);
			
		},
		
		createAdorner: function(clazz, templates, rootId) {
			var a = this._adornerLayer.createObject(clazz);
			a.initialize(this, templates, rootId);
			return a;
		},
		
		destroyAdorner: function(a) {
			a.destroy();
			a.getParent().remove(a);
		},
		
	  fitToContents: function(margin, zoomOutOnly) {
	    this.getGraph().fitToContents(margin, zoomOutOnly);
	  },
	  
		// =======================================================================
		//
		// INTERACTORS:
		//
		//		An interactor is an abstraction of a user interface process. Each interactor controls some kind
		//		process. It is responsible for making it available and has all the UI logic pertinent to it.
		//
		//		The interactors are managed by an InteractionManager, which organizes them by id and is responible
		//		for distinguishing the currently active interactors.
		//
		//		The diagram widget uses interactors to provide the users with ways to interact with the Graph
		//		surface. It uses a PanInteractor to provide the user with a mode in which the surface
		//		can be navigating by grabbing it with the left mouse button and zoomed using the mouse wheel.
		//		It also uses a SelectionInteractor to provide the user the ability of selecting nodes by 
		//		left-clicking them.
		//
		//		see:
		//		event.Interactor, event.PanInteractor, event.SelectionInteractor, event.InteractorManager
		//
		
		//
		//	_interactors: ibm_ilog.diagram.interactors.InteractorManager
		//		The interactor manager
		//
		_interactors: null,
		
		
		_setupInteractors: function(ids) {
			array.forEach(ids, function(i) {
				this._interactors.get(i).setEnabled(this[i]);
			}, this);
		},
		
		_initializeInteractors: function() {
			//
			//	summary:
			//		Initializes the interactors used by the Diagram
			//
			//	tags:
			//		protected
			//
			
			this._interactors.add("pan", new PanInteractor().initialize(this));
			this._interactors.add("select", new SelectionInteractor().initialize(this._graph, this.getSelection(), this._viewport.getBackgroundShape()));
			this._interactors.add("zoom", new ZoomInteractor().initialize(this));
			
			this._setupInteractors(["pan", "select", "zoom"]);
		},
		
		startup: function() {
			this.inherited(arguments);
			
			//	[av] adorner creation fails on certain renderers/browsers if this is done before "startup".
			//this._createAdorners();
			this._viewport.whenLoaded(lang.hitch(this, this._createAdorners));
			
			this._loadAnnotations();
		},
		
		getInteractor: function(id) {
			return this._interactors.get(id);
		},
		
		getInteractorManager: function() {
			return this._interactors;
		},

		enterStandardMode: function() {
			//
			//	summary:
			//		TODO TBD. Probably to be renamed also
			//    	
			this._interactors.switchTo("pan", "select", "zoom");
		},
		
		assignInteractorInputs: function(iid, assignments) {
			this.getInteractor(iid).assignInputs(assignments);
		},
		
		setMarqueeMode: function( /*int*/b) {
			this._interactors.get("marquee").setMode(b);
		},
		
		enableMarqueePartialSelection: function( /*boolean*/b) {
			this._interactors.get("marquee").setPartialSelectionEnabled(b);
		},

		// =======================================================================
		//
		// KEY INTERACTORS:
		//
		
		// moved to _Diagram.js
		
		_initializeKeyInteractors: function() {
		},
		
		// =======================================================================
		//
		//	GRAPH MOUSE EVENTS:
		//
		//		The user can connect to input events related to entities through the events defined in this 
		//		section. These are mapped to the corresponding DOM events when they occur on the corresponding
		//		entity (Node or Link).
		//
		
		onNodeContextMenu: function(n, e) {
			this.lastMenuTarget = n;
			this.lastMenuEvent = e;
		},
		onNodeClick: _emptyEntityHandler,
		onNodeDblClick: _emptyEntityHandler,
		onNodeMouseUp: _emptyEntityHandler,
		onNodeMouseDown: _emptyEntityHandler,
		onNodeMouseMove: _emptyEntityHandler,
		onNodeMouseEnter: _emptyEntityHandler,
		onNodeMouseLeave: _emptyEntityHandler,
		onNodeMouseOver: _emptyEntityHandler,
		onNodeMouseOut: _emptyEntityHandler,
		// Touch events for Node
		onNodeTouchStart: function (ge,event){
			this.onNodeMouseDown(ge,event);
		},
		onNodeTouchEnd: function (ge,event){
			this.onNodeMouseUp(ge,event);
			this.onNodeClick(ge,event);
			if(event.touches.length == 0){
				var delay = new Date().getTime() - this._lastUp;
				if(this._lastUp && (delay < 500)){
					this.onNodeDblClick(ge,event);
					delete this._lastUp;
				}else{
					this._lastUp = new Date().getTime();
				}   
			}			
		},
		onNodeTouchMove: function (ge,event){
			this.onNodeMouseMove(ge,event);
		},
		
		onLinkContextMenu: function(l, e) {
			this.lastMenuTarget = l;
			this.lastMenuEvent = e;
		},
		onLinkClick: _emptyEntityHandler,
		onLinkDblClick: _emptyEntityHandler,
		onLinkMouseUp: _emptyEntityHandler,
		onLinkMouseDown: _emptyEntityHandler,
		onLinkMouseMove: _emptyEntityHandler,
		onLinkMouseEnter: _emptyEntityHandler,
		onLinkMouseLeave: _emptyEntityHandler,
		onLinkMouseOver: _emptyEntityHandler,
		onLinkMouseOut: _emptyEntityHandler,
		
		// Touch events for Link
    onLinkTouchStart: function (ge,event){
      this.onLinkMouseDown(ge,event);
    },
    onLinkTouchEnd: function (ge,event){
        this.onLinkMouseUp(ge,event);
        this.onLinkClick(ge,event);
        if(event.touches.length == 0){
            var delay = new Date().getTime() - this._lastUp;
            if(this._lastUp && (delay < 500)){
                this.onLinkDblClick(ge,event);
                delete this._lastUp;
            }else{
                this._lastUp = new Date().getTime();
            }   
        }           
    },
    onLinkTouchMove: function (ge,event){
        this.onLinkMouseMove(ge,event);
    },
  
		
		onSubgraphContextMenu: function(s, e) {
			this.lastMenuTarget = s;
			this.lastMenuEvent = e;
		},
		onSubgraphClick: _emptyEntityHandler,
		onSubgraphDblClick: _emptyEntityHandler,
		onSubgraphMouseUp: _emptyEntityHandler,
		onSubgraphMouseDown: _emptyEntityHandler,
		onSubgraphMouseMove: _emptyEntityHandler,
		onSubgraphMouseEnter: _emptyEntityHandler,
		onSubgraphMouseLeave: _emptyEntityHandler,
		onSubgraphMouseOver: _emptyEntityHandler,
		onSubgraphMouseOut: _emptyEntityHandler,

    // Touch events for subGraph
    onSubgraphTouchStart: function (ge,event){
      this.onSubgraphMouseDown(ge,event);
    },
    onSubgraphTouchEnd: function (ge,event){
        this.onSubgraphMouseUp(ge,event);
        this.onSubgraphClick(ge,event);
        if(event.touches.length == 0){
            var delay = new Date().getTime() - this._lastUp;
            if(this._lastUp && (delay < 500)){
                this.onSubgraphDblClick(ge,event);
                delete this._lastUp;
            }else{
                this._lastUp = new Date().getTime();
            }   
        }           
    },
    onSubgraphTouchMove: function (ge,event){
        this.onSubgraphMouseMove(ge,event);
    },
  
        
		
		//
		//	lastMenuTarget:
		//		The entity on which the last/current menu was opened.
		//
		lastMenuTarget: null,
		
		//
		//	linkMenu:
		//		An optional dijit.Menu for links (property)
		//
		_initializeNodeEvents: function() {
			//
			// TODO analyze and maybe replace this connection method by
			// the deferred method used in _Widget.
			//
			// May also be replaced by a single connection plus a linear search on the geometry.
			//
			//	tags:
			//		private
			//
			
			var eventNames = ["ContextMenu", "Click", "DblClick", "MouseUp", "MouseDown", "MouseMove", "MouseEnter", "MouseLeave", "MouseOver", "MouseOut"];
			if (iid.mobileEnabled){
				eventNames.push("TouchStart","TouchMove","TouchEnd");
			}
			var lcEventNames = array.map(eventNames, function(e) {
				return e.toLowerCase();
			});
			
			var self = this;
			
			// Connect to node events
			
			array.forEach(["Node", "Link", "Subgraph"], function(get) {
				var lget = get.toLowerCase();
				var evtPrefix = "on" + get;
				var menuName = lget + "Menu";
				this._connections.push(connect.connect(this, "_on" + get + "Created", this, function(ge) {
					for (var i = 0; i < eventNames.length; i++) {
						(function() {
							var eventName = evtPrefix + eventNames[i];
							try {
								ge.connect("on" + lcEventNames[i], self, function(e) {
									return (this[eventName])(ge, e);
								});
							} catch (e) {
								R.warn("EventConnectionFailed", "on" + lcEventNames[i]);
							}
						})();
					}
					if (this[menuName]) {
						try {
							this[menuName].bindDomNode(ge.getEventSource());
						} catch(e){}
					}
				}));
				
			}, this);
			
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/beforeDisposing', this, function(graphElement) {
				this.beforeGraphElementDispose(graphElement);
			}));
			
			this._subscriptions.push(connect.subscribe('/ibm_diagram/GraphElement/afterDisposing', this, function(graphElement) {
				this.afterGraphElementDispose(graphElement);
			}));
			
		},
		
		beforeGraphElementDispose: function(graphElement) {
			//
			//	summary:
			//		this method is called before any graph element is disposed
		},
		
		afterGraphElementDispose: function(graphElement) {
			//
			//	summary:
			//		this method is called after any graph element is disposed
		},
		
		// =======================================================================
		//
		//	SELECTION:
		//
		//		The interface to the currently selected nodes/dataitems.
		//
		//		see:
		//		widget.SelectionInteractor, widget.Selection
		//
		//
		
		
		//
		//	_selection: ibm_ilog.diagram.Selection
		//		The selection object
		//
		_selection: null,
		
		getSelection: function() {
			//
			//	summary:
			//		Return the selection object
			//    	
			return this._selection;
		},
		
		// bindable property that defines the selection mode
		// See: ibm_diagram_Selection
		selectionMode: null,
		
		_createSelection: function() {
			//
			//	summary:
			//		Creates the selection object
			//    	
			if (this.selectionMode == null) {
				this.selectionMode = Selection.standard;
			}
			this._selection = new Selection(this);
			
		},
		
		
		// =======================================================================
		//
		//	ADORNERS FOCUS:
		//
		//		The interface to manage focus on an entity for operation
		//
		//		see:
		//		widget.AdornerFocusInteractor
		//
		//

		//
		//  _nodeAdornersFocus: ibm_ilog.diagram.Node|ibm_ilog.diagram.Subgraph
		//      The currently adorner-focused node/subgraph 
		//
		_nodeAdornersFocus: null,

		//
		//	getAssociatedNodeAdornersFunction:
		//		User-customizable function which must return the adorners associated with the adorner focus on the given node.
		//
		getAssociatedNodeAdornersFunction: function(node) {
			return [];
		},
		
		//
		//	getAssociatedLinkAdornersFunction:
		//		User-customizable function which must return the adorners associated with the adorner focus on the given link.
		//
		getAssociatedLinkAdornersFunction: function(link) {
			return [];
		},

		//
		//	getAssociatedSubgraphAdornersFunction:
		//		User-customizable function which must return the adorners associated with the adorner focus on the given subgraph.
		//
		getAssociatedSubgraphAdornersFunction: function(sg) {
			return [];
		},

		getNodeAdornersFocus: function() {
			//
			//	summary:
			//		Returns the current node adorners focus.
			//			
			return this._nodeAdornersFocus;
		},
		
		setNodeAdornersFocus: function(ge) {
			//
			//	summary:
			//		Sets the node adorners focus on a graph element. Used by the adorners focus interactor;
			//			
			var oldValue = this._nodeAdornersFocus;
			this._nodeAdornersFocus = ge;
			if (oldValue != ge) {
				this.onNodeAdornersFocusChanged(oldValue, ge);
			}
			return this;
		},
		
		onNodeAdornersFocusChanged: function(oldValue, newValue) {
			//
			//	summary:
			//		event that occurs when the adorners focus has changed.
			//			
			var adorners = [];
			if(newValue) {
				if(newValue._isIBMDiagramSubgraph) {
					adorners = this.getAssociatedSubgraphAdornersFunction(newValue);
				} else {
					adorners = this.getAssociatedNodeAdornersFunction(newValue);
				}
			}
			array.forEach(this._activeNodeAdorners,function(g){
				g.setAdorned(null);
			},this);
			
			this._activeNodeAdorners = adorners;

			array.forEach(this._activeNodeAdorners,function(g){
				g.setAdorned(newValue);
			},this);
		},

		
		//
		//  _nodeAdornersFocus: ibm_ilog.diagram.Link
		//      The currently adorner-focused link 
		//
		_linkAdornersFocus: null,
		
		getLinkAdornersFocus: function() {
			//
			//	summary:
			//		Returns the current link adorners focus.
			//			
			return this._linkAdornersFocus;
		},
		
		setLinkAdornersFocus: function(ge) {
			//
			//	summary:
			//		Sets the link adorners focus on a graph element. Used by the adorners focus interactor;
			//			
			var oldValue = this._linkAdornersFocus;
			this._linkAdornersFocus = ge;
			if (oldValue != ge) {
				this.onLinkAdornersFocusChanged(oldValue, ge);
			}
			return this;
		},
		onLinkAdornersFocusChanged: function(oldValue, newValue) {
			//
			//	summary:
			//		event that occurs when the link adorners focus has changed.
			//			
			var adorners = []; 
			if(newValue) {
				adorners = this.getAssociatedLinkAdornersFunction(newValue);
			}
			
			array.forEach(this._activeLinkAdorners,function(g){
				g.setAdorned(null);
			},this);
			
			this._activeLinkAdorners = adorners;

			array.forEach(this._activeLinkAdorners,function(g){
				g.setAdorned(newValue);
			},this);
		},

		mustFreeAdornersFocus: function(graphElement, point, margin, outOfBounds) {
			// Summary:
			//		This function is called from the adorners focus interactor to decide whether to free the adorners focus or not.
			
			// This special case is added to be able to modify intermediate point on spline links
			if (outOfBounds && graphElement._isIBMDiagramLink && graphElement.isSelected() && graphElement.getCurveTension() > 0) {
				return false;
			}
			return outOfBounds;
		},
		
		changeAdornersFocus: function(focus, newFocus, possibleCursors, mustchangeAdornersFocus) {
			// Summary:
			//		This function is called from the adorners focus interactor to decide whether to change the adorners focus or not.
			// return: 
			//		the new gfx to apply the cursor, or null if not need to change. 	
			if (mustchangeAdornersFocus) {
				return newFocus;
			} else {
				return null;
			}
		},
		
		// _lowCpuMode: Enables the use of some low cpu consuming functions.
		// Be aware that some of them have less precision than the high cpu consumption.
		_lowCpuMode: false,
		
		isLowCpuMode: function() {
			//	Summary:
			//		return true if the low cpu mode is enabled
			//
			return this._lowCpuMode;
		},
		
		
		enableLowCpuMode: function(enable) {
			//	Summary:
			//		enables or disables the low cpu mode
			//
			this._lowCpuMode = enable;
			this.lowCpuModeChanged();
		},
		
		lowCpuModeChanged: function() {
			//	Summary:
			//		this function is called when the low cpu mode is changed
			//
		},

		// =======================================================================
		//
		//	Adorners
		//

		// moved to _Diagram.js
		
		_initializeAdorners: function() {
		},
		
		_createAdorners: function() {
		},
		
		// =======================================================================
		//
		//	FOCUSED ELEMENT:
		//
		//		The interface to manage focus on an entity for operation and navigation
		//
		//		see:
		//		widget.FocusInteractor
		//
		//

		focus: true,

		//
		//  _focusedElement: /*GraphElement*/
		//      The graph element currently focused
		//
		_focusedElement: null,
		
		_focusedNodeAdorner: null,
		
		_focusedLinkAdorner: null,

		_currentFocusAdorner: null,
		
		getFocusedElement: function() {
			return this._focusedElement;
		},
		
		getFocusAdorner: function(ge) {
			var node = this._focusedNodeAdorner;
			var link = this._focusedLinkAdorner;
			
			return {N:node,S:node,L:link}[ge.TID];
		},
		
		setFocusedElement: function(ge) {
			var oldValue = this._focusedElement;
			this._focusedElement = ge;
			
			var adorner = null;
			if(ge) {
				adorner = this.getFocusAdorner(ge);
			}
			
			if (adorner) {
				adorner.setAdorned(ge);
			}
			if(adorner!=this._currentFocusAdorner && this._currentFocusAdorner) {
				this._currentFocusAdorner.setAdorned(null);
			}
			this._currentFocusAdorner = adorner;
			
			if (oldValue != ge) {
				this.onFocusedElementChanged(oldValue, ge);
			}
			
			// [av] make sure we get focus when user clicks the diagram.
			// the standard capture is hidden because the only element with
			// tabIndex 0 may be hidden under several layers.
			this._htmlRefocus();

			return this;
		},
		
		onFocusedElementChanged: function(oldValue, newValue) {
		},
		
		// =======================================================================
		//
		//	Layout
		//
		
		
		performGraphLayout: function() {
		// summary:
		//		Executes the graph layout and/or the link layout specified by the setNodeLayout()
		//		and setLinkLayout methods.
		// description:
		//		If a non-null graph layout has been specified by calling setNodeLayout(),
		//		and if the graph layout is active (that is, unless setNodeLayoutActive(false) has been called),
		//		then the graph layout algorithm is performed on the contents of this graph.
		//		Then, if a non-null link layout has been specified by calling setLinkLayout(),
		//		and if the link layout is active (that is, unless setLinkLayoutActive(false) has been called),
		//		then the link layout algorithm is performed on the contents of this graph.
			
			this._graph.performGraphLayout();
		},
		
		//
		// Input Events
		//
		
		// =======================================================================
		//
		//	DATA CONNECTION:
		//
		//		The interface to manage attributes related to the node and links data stores
		//
		//		see:
		//		widget.DataConnector
		//
		// =======================================================================
		
		load: function() {
			// summary: 
			//        loads the data stores and creates the diagram.
			
			// Temporary
			this._load();
		},
		
		// =======================================================================
		
		//
		// Templating
		//    
		
		_nodeTemplate: null,
		_subgraphTemplate: null,
		_linkTemplate: null,
		
		createTemplatedShape: function(/*Function*/shapeType, /*String||Object?*/template, /*ibm_ilog.diagram.Graph?*/onGraph, /*Object?*/style, /*dojo.data.api.Read?*/store, /*Object?*/item) {
			// summary:
			// 		Creates an instance of the specified type fully initialized.
			// description:
			//     This method creates an instance of the specified type and populates it with the specified template.
			//     If the template defines bindings to the shape or the data item (if any), bindings are resolved.
			// shapeType: Function
			//		The type of the instance.
			// template: String||Object
			//    a template defining the shape contents. If no template is specified, the default Subgraph template will be used. Optional.
			// onGraph: ibm_ilog.diagram.Graph
			//    the parent graph. Optional.
			// store : dojo.data.api.Read
			//	a data store. Optional.
			// item: dojo.data.api.Item:
			//    a data item associated with this shape. Optional.
			
			onGraph = onGraph || this._graph;
			// Creates the node instance
			var shape = onGraph.createObject(shapeType);
			// Set the data context
			if (store && item) {
				shape.setData(new dtlbinding.DataContext(item, store));
			}
			// apply the style
			if (style) {
				_style.applyTo(style, shape);
			}
			// Apply the template (and resolve bindings, if any)
			onGraph._applyTemplate(shape, template, null, null);
			return shape;
		},
		
		_resolveTemplate: function(func, prop, def) {
			var template = null;
			// is there a function to return a template ?
			if (func) {
				// Should return a String to enable DTL engine. An Object means no binding.
				template = func();
			}
			if (!template) {
				// let's try the property 
				template = prop;
			}
			if (!template) {
				//let's use the default one
				template = def;
			}
			return template;
		},
		
		// a private hook to use for internal notification, as the public onNodeCreated
		// might be changed by the user *after* the widget initialization (in this case,
		// the internal "dojo.connect('onNodeCreated')" would listen the old method.
		_onNodeCreated: function(node, diagram) {
			this.onNodeCreated(node, diagram);
		},
		
		onNodeCreated: function(node, diagram) {
			// summary:
			//    Invoked when the specified node has been created.
			// node: ibm_ilog.diagram.Node: the new node instance.
			// diagram: ibm_ilog.diagram.widget.Diagram: the diagram instance that fired the event.
		},
		
		resolveNodeTemplate: function(/*dojo.data.Item*/item) {
			// summary:
			//    Returns the template to represent the specified node item.
			//    item: dojo.data.Item: the node data item.
			
			var func = this.nodeTemplateFunction ? lang.partial(this.nodeTemplateFunction, item, this) : null;
			return this._resolveTemplate(func, this._nodeTemplate, Node.defaultTemplate);
		},
		
		createNode: function(/*Object or String*/template, /*dojo.data.api.Item*/ item, /*ibm_ilog.diagram.Graph*/ onGraph, /*Function?*/ nodeType) {
			// summary:
			//        Creates a Node that represents the specified data item.
			// template: Object or String 
			//		The template to use for this node. Optional.
			// item: dojo.data.api.Item 
			//		a data item associated with this node. Optional.
			// onGraph: ibm_ilog.diagram.Graph 
			//		The parent graph of the node. If null, the top-level graph is used. Optional.
			// nodeType: Function
			//		The type of the node instance. Optional.
			// returns:
			//        A ibm_ilog.diagram.Node instance.
			
			
			if (!onGraph) {
				onGraph = this._graph;
			}
			
			template = template || this.resolveNodeTemplate(item);
			
			var node = null;
			// let's try a custom node factory
			if (this.createNodeFunction) {
				node = this.createNodeFunction.call(null, item, template, onGraph, this);
				if (node && (!(node._isIBMDiagramNode))) {
					R.error("UnexpectedReturnValueType", this, createNodeFunction, "ibm_diagam.Node", typeof(node));
				}
				
			}
			if (!node) {
				nodeType = nodeType || Node;
				node = this.createTemplatedShape(nodeType, template, onGraph, this.nodeStyle, this.nodesStore, item);
			}
			
			// set Label property
			if (item) {
				
				var text = StoreApi.getLabel(this.nodesStore, item);
				if (text) {
					node.setLabel(text);
				}
				
			}
			
			// TODO: should we pass the store and item ? 
			this._onNodeCreated(node, this);
			
			return node;
		},
		
		// a private hook to use for internal notification, as the public onNodeCreated
		// might be changed by the user *after* the widget initialization (in this case,
		// the internal "dojo.connect('onNodeCreated')" would listen the old method.
		_onSubgraphCreated: function(subgraph, diagram) {
			this.onSubgraphCreated(subgraph, diagram);
		},
		
		onSubgraphCreated: function(/*ibm_ilog.diagram.Subgraph*/subgraph, /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
			// summary:
			//    Invoked when the specified subgraph object has been created and initialized. 
			//    The default implementation does nothing.
			// subgraph: ibm_ilog.diagram.Subgraph: the new subgraph instance.
			// diagram: ibm_ilog.diagram.widget.Diagram: the diagram instance that fired the event.
		},
		
		resolveSubgraphTemplate: function(item) {
			// summary:
			//    Returns the template to represent the specified subgraph item.
			
			var func = this.subgraphTemplateFunction ? lang.partial(this.subgraphTemplateFunction, item, this) : null;
			return this._resolveTemplate(func, this._subgraphTemplate, Subgraph.defaultTemplate);
		},
		
		createSubgraph: function(/*Object or String*/template, /*dojo.data.api.Item*/ item, /*ibm_ilog.diagram.Graph*/ onGraph, /*Function?*/ nodeType) {
			// summary:
			//        Creates a Subgraph that represents the specified data item.
			// template: Object or String The template to use for this subgraph. Optional.
			// item: dojo.data.api.Item a data item associated with this subgraph. Optional.
			// onGraph: ibm_ilog.diagram.Graph The parent graph of the subgraph. If null, the top-level graph is used. Optional.
			// nodeType: Function
			//		The type of the node instance. Optional.
			// returns:
			//        A ibm_ilog.diagram.Subgraph instance.
			
			if (!onGraph) {
				onGraph = this._graph;
			}
			
			template = template || this.resolveSubgraphTemplate(item);
			
			var subgraph = null;
			// let's try a custom subgraph factory
			if (this.createSubgraphFunction) {
				subgraph = this.createSubgraphFunction.call(null, item, template, onGraph, this);
				if (subgraph && (!(subgraph._isIBMDiagramSubgraph))) {
					R.error("UnexpectedReturnValueType", this, createSubgraphFunction, "ibm_diagam.Subgraph", typeof(subgraph));
				}
			}
			if (!subgraph) {
				nodeType = nodeType || Subgraph;
				subgraph = this.createTemplatedShape(nodeType, template, onGraph, this.subgraphStyle, this.nodesStore, item);
			}
			
			// set Label property
			if (item) {
				var text = StoreApi.getLabel(this.nodesStore,item);
				if (text) {
					subgraph.setLabel(text);
				}
			}
			
			
			this._onSubgraphCreated(subgraph, this);
			
			return subgraph;
		},
		
		connectNodes: function(/* ibm_ilog.diagram.Node */start, /* ibm_ilog.diagram.Node */ end, /*String||Object*/ template, /*dojo.data.Item*/ linkItem) {
			// summary:
			//        Connects the specified nodes.
			//        This method creates a ibm_ilog.diagram.Link instance invoking the createLink() method
			//        and connects its extremities to the specified nodes. Optionally, one may specify
			//        the linksStore item associated with this link.
			// start: ibm_ilog.diagram.Node : the link start node.
			// end: ibm_ilog.diagram.Node: the link end node.
			// template: Object or String : the template to use. Optional.
			// linkItem: dojo.data.Item: the data item corresponding to this link, if any. Optional.
			// returns: ibm_ilog.diagram.Link
			
			if (!linkItem) {
				linkItem = null;
			}
			
			var link = this.createLink(start, end, template, linkItem);
			if (!link.getStartPort()) {
				link.setStartNode(start);
			}
			if (!link.getEndPort()) {
				link.setEndNode(end);
			}
			return link;
		},
		
		// a private hook to use for internal notification, as the public onNodeCreated
		// might be changed by the user *after* the widget initialization (in this case,
		// the internal "dojo.connect('onNodeCreated')" would listen the old method.
		_onLinkCreated: function(link, start, end, diagram) {
			this.onLinkCreated(link, start, end, diagram);
		},
		
		onLinkCreated: function(/*ibm_ilog.diagram.Link*/link, /*ibm_ilog.diagram.Node*/ start, /*ibm_ilog.diagram.Node*/ end, /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
			// summary:
			//    Invoked when the specified link has been created.
			// link: ibm_ilog.diagram.Link: the new link instance.
			// start: ibm_ilog.diagram.Node: the link start node, if any.
			// end: ibm_ilog.diagram.Node: the link end node, if any.
			// diagram: ibm_ilog.diagram.widget.Diagram: the diagram instance that fired the event.
		
		},
		
		resolveLinkTemplate: function(/*dojo.data.Item*/linkItem,/*ibm_ilog.diagram.Node*/ start, /*ibm_ilog.diagram.Node*/ end) {
			// summary:
			//    Returns the link template to represent the specified relationship.
			// linkItem: dojo.data.Item: the data item corresponding to this link, if any.
			// start: ibm_ilog.diagram.Node : the link start node.
			// end: ibm_ilog.diagram.Node: the link end node.
			
			var func = this.linkTemplateFunction ? lang.partial(this.linkTemplateFunction, linkItem, start, end, this) : null;
			return this._resolveTemplate(func, this._linkTemplate, Link.defaultTemplate);
		},
		
		createLink: function(/* ibm_ilog.diagram.Node */start, /* ibm_ilog.diagram.Node */ end, /*Object or String*/ template, /*dojo.data.Item*/ linkItem, /*Function?*/ linkType) {
			// summary:
			//        Creates a Link to connect the specified nodes. Optionally, one may specify
			//        the linksStore item associated with this link. 
			// description: This method only creates the Link instance and does not connect the nodes. Caller may invoke
			//        the Diagram.connectNodes() method instead, which invokes this method and perfom the connection.
			// start: ibm_ilog.diagram.Node
			//		the link start node.
			// end: ibm_ilog.diagram.Node
			//		the link end node.
			// template: Object or String
			//		the template to use. Optional.
			// linkItem: dojo.data.Item
			//		the data item corresponding to this link, if any. Optional.
			// nodeType: Function
			//		The type of the node instance. Optional.
			// returns:
			//        a ibm_ilog.diagram.Link instance.
			
			if (!linkItem) {
				linkItem = null;
			}
			
			template = template || this.resolveLinkTemplate(linkItem, start, end);
			
			// let's try a custom factory
			var link = null;
			var graph = gu.lowestCommonAncestor(start, end);
			if (this.createLinkFunction) {
				link = this.createLinkFunction.call(null, linkItem, start, end, template, graph, this);
				if (link && (!(link._isIBMDiagramLink))) {
					R.error("UnexpectedReturnValueType", this, createLinkFunction, "ibm_diagam.Link", typeof(link));
				}
			}
			if (!link) {
				linkType = linkType || Link;
				link = this.createTemplatedShape(linkType, template, graph, this.linkStyle, this.linksStore, linkItem);
			}
			this._onLinkCreated(link, start, end, this);
			return link;
		},
		
		
		//
		// Data Binding
		//
		
		getBoundItem: function(nodeOrLink) {
			// summary: 
			//     Returns the data item, if any, associated with the specified node or link.
			// nodeOrLink: ibm_ilog.diagram.Node || ibm_ilog.diagram.Link || ibm_ilog.diagram.Subgraph: 
			//     the node, link or subgraph instance.
			
			var data = nodeOrLink.getData();
			return data ? data.item : null;
		},
		
		//------------------
		// Image export API
		//------------------
		
		toSvg: function(/*Function*/onComplete, /*Function*/ onError, /*Boolean*/ all){
			// summary:
			//     Export the graph as an SVG string.
			// description:
			//		Export the graph as an SVG string. This features requires Dojo 1.4.x or greater.
			// onComplete: Function
			//		The callback that receives the resulting SVG.
			// onError: Function
			//		The function called when the generation fails.
			// all: Boolean
			//		Indicates whether the whole graph should be visible in the resulting svg. If false, only the visible area is displayed. The default value is false. 
			
			var svg, gr = this._graph;
			var oldhud = this._getHudAttr();
			// hide hud and adorners
			this._setHudAttr(false);
			this.getViewport()._showOverlays(false);
			
			// patch so that gfx (de)serialization does not crash on MultilineText deserialization :
			// The gfx serialize() method does not save the 'shape' property of a Group instance, but
			// will save the others 'default' gfx properties (fill, stroke, font...), so MultilineText instances
			// are serialized as Group instance :
			// { children : [....], transform:..., font: ... }.  (so, no shape:{type:'ml'}).
			// Therefore, when the json is deserialized, a Group instance is created, and setFont() is invoked
			// (which fails because Group.setFont does not exist).
			// Patch (Hack!!!!): we delete the getFont/setFont accessors so that the font is not serialized
			var MultilineText = null;
			if (!iid.isSvg) {
				try {
					MultilineText = require("ibm_ilog/diagram/util/MultilineText");
					var getFont = MultilineText.prototype.getFont;
					delete MultilineText.prototype.getFont;
				} catch (err) {
				}
			}
			svg = gfxutils.toSvg(gu.getSurface(gr));
			svg.addCallbacks(lang.hitch(this, function(result){
				// restore font accessors
				if (!iid.isSvg && MultilineText) {
					MultilineText.prototype.getFont = getFont;
				}
				if (all) {
					var topg = this.getViewport().getEventSource();
					var bbox = gr.getBoundingBox();
					var tlt = this.getViewport()._transformedLayer.getTransform();
					var inv = tlt.inverse();
					var m = matrix.multiply(matrix.translate(-bbox.x, -bbox.y), inv);
					// neutralize the transformedLayer transform
					result = result.replace(/(<g)/, '$1 transform="matrix(' + m.xx + ',' + m.xy + ',' + m.yx + ',' + m.yy + ',' + (m.dx) + ',' + (m.dy) + ')"'); // resize surface to graph content
					// resize viewport size
					var e = /(<svg\b[^>]*)(\bwidth=\s*"\d*\w*")([^>]*)(\bheight=\s*"\d*\w*")([^>]*>)/g;
					var bb = "$1width=\"" + Math.round(bbox.width) + "px\"$3height=\"" + Math.round(bbox.height) + "px\"$5";
					result = result.replace(e, bb);
				}
				// restore state
				this._setHudAttr(oldhud);
				this.getViewport()._showOverlays(true);
				onComplete.call(null, result);
			}), lang.hitch(this, function(result){
				// restore font accessors
				if (!iid.isSvg && MultilineText) {
					MultilineText.prototype.getFont = getFont;
				}
				// restore state
				this._setHudAttr(oldhud);
				this.getViewport()._showOverlays(true);
				if (onError) {
					onError.call(null, result);
				}
			}));
		},
		
		
		//
		// Misc
		//
		
		getParent: function(){
			// summary:
			//		Returns the parent widget of this widget, assuming the parent
			//		specifies isContainer
			var parent = registry.getEnclosingWidget(this.domNode.parentNode);
			return parent && parent.isContainer ? parent : null;
		},
		
		getSurface: function() {
			// summary: 
			//        Returns the dojox.gfx.Surface object displaying the graph.
			// returns:
			//        A dojox.gfx.Surface instance. 
			return this._viewport.getSurface();
		},
		
		getGraph: function() {
			// summary:
			//        Returns the ibm_ilog.diagram.Graph instance that displays the graph.
			// returns:
			//        An ibm_ilog.diagram.Graph instance. 
			return this._graph;
		},
		
		getGraphTransform: function() {
			// summary:
			//        Returns the graph transform.
			// returns:
			//        A dojox.gfx.matrix.Matrix2D instance that represents the current graph transform.
			return this._graph.getTransform() || matrix.identity;
		},
		
		getGraphToDiagramTransform: function() {
			// summary:
			//		Returns the matrix used to transform coordinates of objects contained
			//		in this diagram's graph (returned by <code>getGraph()</code>) to coordinates
			//		relative to this Diagram object.
			// returns:
			//		A dojox.gfx.matrix.Matrix2D instance.
			
			return this._graph._getRealMatrix() || matrix.identity; // dojox.gfx.matrix.Matrix2D
		},
		
		getGraphToClientTransform: function() {
			// summary:
			//		Returns the matrix used to transform coordinates of objects contained
			//		in this diagram's graph (returned by <code>getGraph()</code>) to coordinates
			//		relative to the client browser's page.
			// returns:
			//		A dojox.gfx.matrix.Matrix2D instance.
			
			var t = this.getGraphToDiagramTransform();
			var rect = this.domNode.getBoundingClientRect();
			t = matrix.multiply(matrix.translate(rect.left, rect.top), t);
			return t; // dojox.gfx.matrix.Matrix2D
		},
		
		getNodeRect: function(node, inClient) {
			// summary:
			//		Returns the bounding rectangle of a node of the graph,
			//		in client (i.e., browser page) coordinates, or relative to the Diagram.
			// description:
			//		This method computes the bounding rectangle of a node contained in this Diagram widget.
			//		The returned rectangle is either relative to the whole browser page (if inClient is true or omitted),
			//		or relative to the bounds of the Diagram widget (if inClient is false).
			// node: ibm_ilog.diagram.Node
			//		A Node of the Graph displayed in this Diagram.
			// inClient: Boolean
			//		If true (the default), the returned rectangle is relative to the whole browser page.
			//		If false, the returned rectangle is relative to the bounds of the Diagram widget.
			// returns:
			//		An object with x, y, width and height properties.
			
			var bb = node.getBounds(this._graph);
			var t = (arguments.length == 1 || inClient) ? this.getGraphToClientTransform() : this.getGraphToDiagramTransform();
			bb = t.transformRectangle(bb);
			return bb; // Object
		},
		
		getGraphElementFromDataItem: function(item) {
			// summary:
			//        Gets the GFX object, for the given Data Item.
			var node = this._nodesStore ? this._getNodeFromStoreItem(item) : null;
			return node ? node : (this._linksStore ? this._getLinkFromStoreItem(item) : null);
		},
		
		asGraphElement: function(itemOrGraphElement) {
			// Summary:
			// 		return the gfx that corresponds to the given item, checking if it is not a gfx already
			if (Selectable.Util.isSelectableInstance(itemOrGraphElement)) {
				return itemOrGraphElement;
			} else {
				return this.getGraphElementFromDataItem(itemOrGraphElement);
			}
		},
		
		useDataStore: function() {
			// Summary:
			//		return if the Diagram is using node store.
			return this.nodesStore != null;
		},
		
		// =======================================================================
		//
		//  ANNOTATIONS:
		//
		
		//
		//	annotationsEnabled:
		//		Used to disable annotations.
		//
		annotationsEnabled: true,
		
		//
		//	createAnnotationAdornerFunction:
		//		Allows to specify what adorner to create for annotating the given GraphElement
		//
		createAnnotationAdornerFunction: function(ge, kwArgs) {
			//
			//	ge: the graph element
			//
			var layer = this._annotationLayer;
			
			var AnnotationAdorner = require("../adorners/AnnotationAdorner"); // may throw error if not required explicitly...
			var target = layer.createObject(AnnotationAdorner.AnnotationTargetAdorner); // Attention!! dojo
			target.initialize(this, this._adornerTemplates, "AnnotationTargetAdorner");
			
			var annotation = layer.createObject(AnnotationAdorner);
			annotation.initialize(this, this._adornerTemplates, "AnnotationAdorner");
			
			var link = layer.createObject(Link);
			link.applyTemplate(this._adornerTemplates.item("AnnotationLink"));
			
			link.setShapeType(Link.LinkShapeType.Orthogonal);
			
			var port = require("../Port");
			var s = new port.AutomaticPort();
			var e = new port.AutomaticPort();
			
			s._setOwner(annotation);
			e._setOwner(target);
			
			link.setStartPort(s);
			link.setEndPort(e);
			
			target.setLink(link);
			annotation.setLink(link, target);
			
			return annotation;
		},
		
		//
		//	destroyAnnotationAdornerFunction:
		//		Counterpart to the createAnnotationAdornerFunction, this functions destroyed the previously created annotation.
		//
		destroyAnnotationAdornerFunction: function(annotation) {
			var layer = this._annotationLayer;
			layer.remove(annotation.getLink());
			this.destroyAdorner(annotation.getTarget());
			this.destroyAdorner(annotation);
		},

		//
		//	hasAnnotationFunction:
		//		Allows to specify when the adorner needs an annotation adorner created
		//
		hasAnnotationFunction: function(ge) {
			//
			//	ge: the graph element
			//
			return ge.getAnnotation() != "";
		},
		
		//
		//	loadAnnotationsFunction:
		//		Allows to create the annotations already associated with data already present in the ge.
		//
		loadAnnotationsFunction: function(ge) {
			//
			//	ge: the graph element
			//
		},
		
		_loadAnnotations: function() {
			//
			//	summary:
			//		Allows the user to create annotations based on ge/item data. This 
			//		method is executed after widget initialization is completed. 
			//
			gu.forEach(this._graph, this.loadAnnotationsFunction, this);
		},
		
		_annotations: null,
		
		_initializeAnnotations: function() {
			var v = this.getViewport();
			
			v.whenLoaded(lang.hitch(this, function() {
				this._annotations = new Dictionary();
				this._annotationLayer = v.getOverlayLayer().createGroup();
			}));
		},
		
		getAnnotationList: function(ge) {
			//
			//	returns:
			//		A dojox.collections.ArrayList with the annotations created for the graph element.
			//	note:
			//		Creates an empty list if none is registered yet.
			//
			var id = ge.getId();
			var ans;
			if (this._annotations.contains(id)) {
				ans = this._annotations.item(id);
			} else {
				this._annotations.add(id, ans = (new ArrayList()));
			}
			return ans;
		},
		
		addAnnotation: function(ge, kwArgs) {
			//
			//	ge: ibm_ilog.diagram.GraphElement
			//		The entity being annotated
			//
			if (this.annotationsEnabled) {
				var a = this.createAnnotationAdornerFunction(ge, kwArgs);
				
				var ans = this.getAnnotationList(ge);
				
				ans.add(a);
				
				a.setAdorned(ge);
			}
		},
		
		removeAnnotation: function(annotation) {
			var ans = this.getAnnotationList(annotation.getAdorned());
			ans.remove(annotation);
			this.destroyAnnotationAdornerFunction(annotation);
		},
		
		// =======================================================================
		//
		//  TOOLTIPS:
		//
		
		// On mobile platforms, these functions can be used only if the application
		// explicitly requires dijit.Tooltip. On desktop, dijit.Tooltip is required
		// in _Diagram.js so the application has nothing to do.
		
		//
		//	getTooltipContentFunction:
		//		Callback function used to build the tooltip content associated with 
		//		the given graph element.  
		//
		getTooltipContentFunction: function(ge) {
			return ge.getTooltip ? ge.getTooltip() : "";
		},

		//
		//	getTooltipAroundPlacementFunction:
		//		Callback function used to compute the graph element bounding box used
		//		to compute the tooltip placement.
		//
		getTooltipAroundPlacementFunction: function(ge) {
			var r = dojoext.coords(this.domNode, true), bb;
			bb = this._viewport.contentToClient().transformRectangle(ge.getBounds(this._graph));
			bb = g.moveRect(g.clipRect(g.standarizeRect(this._viewport.getClientSize()),bb),r);
			bb = { x:bb.x, y:bb.y, w:bb.width, h:bb.height }; // rename w/h for Dojo 1.7/2.0
			return bb;
		},

		showTooltip: function(ge, innerHTML, dontHideOnChange) {
			//
			//	summary:
			//		Opens the dijit master tooltip on a graph element.
			//
			//	description:
			//		Opens a tooltip for a graph element. The content is get using the 
			//		getTooltipContentFunction function. Optionally, 
			//		the content can be overriden using the 'innerHTML' argument.
			//
			//	ge: ibm_ilog.diagram.GraphElement
			//		the graph element
			//
			//	innerHTML: String?
			//		optional argument that overrides the content of the tooltip.
			//
			//	dontHideOnChange:  boolean
			//		option to disable auto-hiding of the tooltip then the element reports changes
			//

			if(Tooltip === undefined){
				try {
					Tooltip = require("dijit/Tooltip");
				} catch(err){
					Tooltip = null;
				}
			}
			if(!Tooltip)
				return;
			
			this.hideTooltip();
			
			innerHTML = innerHTML || this.getTooltipContentFunction(ge);
			if (innerHTML.length) {
				if (innerHTML.length) {

					var bb = this.getTooltipAroundPlacementFunction(ge);
					
					//Set text direction in tooltip
					if (config.isBidiInDojoDiagrammer) {  
					    var UCC = "";  
	                    var textdir = this.textDir;
	                    if (textdir == "auto")
	                        textdir = BidiUtil.checkDir(innerHTML);
	                    if (textdir == "rtl")
	                        UCC  = "\u202B"; //RLE
	                    else if (textdir == "ltr")
	                        UCC = "\u202A"; //LRE
	                    Tooltip.show(UCC+innerHTML, bb);      
    				} else	
						Tooltip.show(innerHTML, bb);
						
					this._tooltipHandle = bb;
					this._tooltipTarget = ge;

					if(!dontHideOnChange) {
						this._tooltipHideOnChange = connect.connect(ge,"_onChanged",this,function() {
							this.hideTooltip();
							connect.disconnect(this._tooltipHideOnChange);
						});
					}
				}
			}
		},
		
		hideTooltip: function() {
			//
			//	summary:
			//		Hides the dijit master tooltip, if currently in use by the Diagram.
			//
			
			if(!Tooltip)
				return;
			
			if(this._tooltipHandle) {
				Tooltip.hide(this._tooltipHandle);
				if(this._tooltipHideOnChange) {
					connect.disconnect(this._tooltipHideOnChange);
				}
				this._tooltipHandle = null;
				this._tooltipTarget= null;
			}
		},
		
		//
		//	_tooltipTarget: ibm_ilog.diagram.GraphElement?
		//		The current toolip target.
		//
		_tooltipOwner: null,
		
		//
		//	_tooltipHandle: Object?
		//		The tooltip handle, used for hiding.
		//
		_tooltipHandle: null,
		
		//
		//	_tooltipHideOnChange: Object?
		//		The tooltip hide on change connection handle.
		//
		_tooltipHideOnChange: null
	});
	
	// Add _Diagram features to Diagram if not on mobile.
	if(_Diagram)
		lang.extend(Diagram, _Diagram);
	
	return Diagram;
		
});

},
'ibm_ilog/diagram/_base/dojoext':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/_base/dojoext", ["dojo/_base/lang", "dojo/dom", "dojo/_base/html"],
function(lang, dom, html){

	// TODO:
	// For now we must create a global object since there are still references to ibm_ilog.diagram._base.gfxext
	// in the rest of the code, but when AMD migration is finished all global references should be replaced
	// by module references and this will be replaced by a simple object creation:
	// var de = {};
	var de = lang.getObject("ibm_ilog.diagram._base.dojoext", true);
	
	// Dojo patches and cross-version helper functions
	
	// dojo.coords marked with dojo.deprecated in 1.7
	de.coords = function(node, includeScroll){
		var n = dom.byId(node), mb = html.marginBox(n), abs = html.position(n, includeScroll);
		mb.x = abs.x;
		mb.y = abs.y;
		return mb;
	};
	
	return de;
});

},
'ibm_ilog/diagram/util/LimitedStack':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/LimitedStack", ["dojo/_base/declare", "dojo/_base/array", "dojox/collections/_base"],
function(declare, array, collections){

var LimitedStack = declare("ibm_ilog.diagram.util.LimitedStack", [], {
	//
	// Summary:
	//		this class is similar to dojox.collections.Stack, but a maximun size can be set.
	//		When the maxium size is reached, the oldest elements are discarded.
	
		_stack: null,
		_maxSize: null,
		count: null,
		constructor: function(arr,maximumSize){
			this._stack= [];
			this.setMaxSize(maximumSize);
			if(arr){
				this._stack=this._stack.concat(arr);
			}
			this.count=this._stack.length;
		},
		clear: function(){
			//	summary
			//	Clear the internal array and reset the count
			this._stack=[];
			this.count=this._stack.length;
		},
		clone: function(){
			//	summary
			//	Create and return a clone of this Stack
			return new LimitedStack(this._stack);
		},
		contains: function(/* object */o){
			//	summary
			//	check to see if the stack contains object o
			for (var i=0; i<this._stack.length; i++){
				if (this._stack[i] == o){
					return true;	//	bool
				}
			}
			return false;	//	bool
		},
		copyTo: function(/* array */ arr, /* int */ i){
			//	summary
			//	copy the stack into array arr at index i
			arr.splice(i,0,this._stack);
		},
		forEach: function(/* function */ fn, /* object? */ scope){
			//	summary
			//	functional iterator, following the mozilla spec.
			array.forEach(this._stack, fn, scope);
		},
		getIterator: function(){
			//	summary
			//	get an iterator for this collection
			return new collections.Iterator(this._stack);	//	dojox.collections.Iterator
		},
		peek: function(){
			//	summary
			//	Return the next item without altering the stack itself.
			return this._stack[(this._stack.length-1)];	//	object
		},
		pop: function(){
			//	summary
			//	pop and return the next item on the stack
			var r=this._stack.pop();
			this.count=this._stack.length;
			return r;	//	object
		},
		push: function(/* object */ o){
			//	summary
			//	Push object o onto the stack
			//  If the maximum size is reached, the first object is removed (the older object)
			this.count=this._stack.push(o);
			if(this._maxSize && this.count>this._maxSize){
				this._stack.shift();
				this.count--;
			}
		},
		toArray: function(){
			//	summary
			//	create and return an array based on the internal collection
			return [].concat(this._stack);	//	array
		},
		setMaxSize: function(maxSize){
			// summary:
			//		sets maximum size of the stack. If null or 0, no maximum size is set and behaves as a traditional stack.
			this._maxSize = maxSize==0?null:maxSize;
			if(this._maxSize){
				while(this.count>this._maxSize){
					this._stack.shift();
					this.count--;
				}
			}
		},
		getMaxSize: function(){
			// summary:
			//		gets maximum size of the stack.
			return this._maxSize;
		}
});

return LimitedStack;

});
},
'ibm_ilog/diagram/widget/HudComponent':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/HudComponent", [
"dojo/_base/declare"
], function(
declare
){

var HudComponent =
declare('ibm_ilog.diagram.widget.HudComponent', null, {
	//
	//	summary:
	//		Superclass for Viewport overlaid components.
	//	
	
	//
	//	_layer: dojox.gfx.Group
	//			Viewport layer at which the component is created.
	//
	_layer: null,

	//
	//	_enabled: boolean
	//			True if enabled and visible.
	//
	_enabled: true,

	//
	//	_shape: dojox.gfx.Group
	//			Main shape of the component
	//
	_shape: null,

	constructor: function (viewport) {
		this._enabled = true;
		this._viewport = viewport;
		this._layer = viewport._hudLayer;
		this._shape = this._layer.createGroup();

		viewport.registerHudComponent(this);
	},

	_enable: function () {
		//
		//	summary:
		//			Enables the component and displays it.
		//
		this._enabled = true;
		this._layer.add(this._shape);
	},

	_disable: function () {
		//
		//	summary:
		//			Disables the component and hides it.
		//
		this._layer.remove(this._shape);
		this._enabled = false;
	},

	setEnabled: function ( /*boolean*/ enabled ) {
		//
		//	summary:
		//			Shows or hides the component, disabling possible connections and overhead while hidden.
		//
		if (!this._enabled && enabled) {
			this._enable();
		} else if (this._enabled && !enabled) {
			this._disable();
		}
	},

	isEnabled: function () {
		//
		//	summary:
		//			Returns true of the component is active and visible.
		//
		return this._enabled;
	}

});

return HudComponent;

});

},
'ibm_ilog/diagram/widget/DataConnector':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/DataConnector", ["dojo/_base/lang", "dojo/_base/declare", "dojo/_base/array", "dojo/_base/json", "dojo/_base/connect", "dojox/collections/ArrayList", "dojox/collections/Dictionary", "../Port", "../util/ErrorReporter", "../data/StoreApi"], function(lang, declare, array, json, connect, ArrayList, Dictionary, port, R, StoreApi){

	var nodesBindingProperties = ["_successorsBinding", "_predecessorsBinding", "_parentBinding", "_childBinding", "_nodesGraphBinding", "_xBinding", "_yBinding"];
	var linksBindingProperties = ["_startNodeBinding", "_endNodeBinding", "_linksGraphBinding"];
	
	/*
	 * @author John Kelly
	 *
	 */
	var DataConnector = declare('ibm_ilog.diagram.widget.DataConnector', null, {
		//
		//	summary: 
		//
		//		The Data Connector class encapsulates the relationship
		//		between data stores, nodes and links.
		//
		//	description:
		//		
		//		Based on information provided through setters, the Data Connector reads in
		//      information from one or two data stores, then builds a model of the relationship
		//      between the items. When the model is complete, the Data Connector creates
		//      Subgraphs, Nodes and Links as appropriate.
		//
		//      There may be up to two stores involved. 
		//      - A nodesStore whose items represent subgraphs or nodes.
		//      - A linksStore whose items describe the relationship between nodeStore items.
		//      Both stores must, at least, support both dojo.data.api.Read and dojo.data.api.Identity
		//      data store features, as defined at http://docs.dojocampus.org/dojo/data
		//
		//      The basic flow is as follows:
		//      1.Based on the nodesStore query and queryOptions provided, load all matching items
		//        into the model.
		//      2.If defined, load in items from the linksStore. Each item will then be examined for 
		//        attributes "start" and "end". The values of these attributes are the identities 
		//        of items in the nodesStore. This inter-item relationship information is added to the
		//        model. If any the items referred to has not already been loaded from the 
		//        nodesStore it is done so now.
		//      3.Based on any to/from relationship functions or fields defined, add inter-item 
		//        relationship information to the model.If any the items referred to has not 
		//        already been loaded from the nodesStore it is done so now. Repeat step 3 until
		//        all items that have been loaded have had there inter-item relationships resolved.
		//      4.Now the model is fully populated, any final information needed can be deduced directly
		//        from the model.
		//      5.Based on information in the model, and options provided to the class instance, Subgraphs
		//        Nodes and links are created.
		//
		//	unit test:
		//
		//      The associated unit testcase is ibm_diagram/tests/widget/DataConnector.html
		//  
		
		// Private properties   
		
		// ------------------------------------
		// Node-related
		// ------------------------------------
		
		//
		// The validated nodes query
		//
		_nodesQuery: null,
		
		//
		// The validated nodes query options object
		//
		_nodesQueryOptions: null,
		
		// A string that indicates either the data item field name defining the child
		// relationship or a function name that returns the origin of a link 
		// for the specified data item    
		_childBinding: null,
		_childBindingFunction: null,
		
		// A string that indicates either the data item field name 
		// defining the parent relationship or a function name that returns
		// the destination of a link for the specified data item.    
		_parentBinding: null,
		_parentBindingFunction: null,
		
		// A string that indicates either the data item field name defining the from
		// relationship or a function name that returns the origin of a link 
		// for the specified data item    
		_predecessorsBinding: null,
		_predecessorsBindingFunction: null,
		
		// A string that indicates either the data item field name 
		// defining the to relationship or a function name that returns
		// the destination of a link for the specified data item.    
		_successorsBinding: null,
		_successorsBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "x location" associated with a nodesStore data item or a named function that 
		// returns the "x location" associated with a nodesStore data item.    
		_xBinding: null,
		_xBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "y location" associated with a nodesStore data item or a named function that 
		// returns the "y location" associated with a nodesStore data item.    
		_yBinding: null,
		_yBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "node graph data" associated with a nodesStore data item or a named function that 
		// returns the "node graph data" associated with a nodesStore data item.    
		_nodesGraphBinding: null,
		_nodesGraphBindingFunction: null,
		
		// A string that indicates either the data item field name defining the
		// "link graph data" associated with a linksStore data item or a named function that 
		// returns the "link graph data" associated with a linksStore data item.    
		_linksGraphBinding: null,
		_linksGraphBindingFunction: null,
		
		// A boolean to determine what xBinding/yBinding values represent
		//  - If true, the values provided by xBinding/yBinding atributes represent the center of the node.
		//  - If false, the values provided by xBinding/yBinding atributes represent the top left-hand corner of the node.
		_centerOnLocation: false,
		
		//
		// An array of node object functions which will be used to persist
		// graphical information to the store, if required.
		//
		
		_nodesGraphFuncMap: null,
		_nodesGraphProperties: null,
		
		//
		// The validated node data store
		//
		_nodesStore: null,
		
		//
		// Flag to indicate how hierarchical data should be represented visually.
		// If true, it will represent the nodesStore items as ibm_ilog.diagram.Node instances 
		// connected by links. If false, it will represent non-leaf items as ibm_ilog.diagram.Subgraph 
		// instances and leaf items as ibm_ilog.diagram.Node instances.
		_createLinksForHierarchy: true,
		
		// Defines which of the dojo.data.api that the store supports
		// See http://docs.dojocampus.org/dojo/data/api
		_nodesStoreFeatures: null,
		
		//
		// the map of store items to shapes nodes
		//
		_model: null,
		
		// ------------------------------------
		// Link-related
		// ------------------------------------
		
		//
		// An array of link object functions which will be used to persist
		// graphical information to the store, if required.
		//
		_linksGraphFuncMap: null,
		_linksGraphProperties: null,
		
		//
		// The validated link data store
		//
		_linksStore: null,
		
		// Defines which of the dojo.data.api that the store supports
		// See http://docs.dojocampus.org/dojo/data/api
		_linksStoreFeatures: null,
		
		//
		// The validated links query object
		//
		_linksQuery: null,
		
		//
		// The validated links query options object
		//
		_linksQueryOptions: null,
		
		// A string that indicates either the data item field name defining the start
		// item in a link or a function name that returns the origin of a link 
		// for the specified data item    
		_startNodeBinding: null,
		_startNodeBindingFunction: null,
		
		// A string that indicates either the data item field name 
		// defining the end item in a link or a function name that returns
		// the destination of a link for the specified data item.    
		_endNodeBinding: null,
		_endNodeBindingFunction: null,
		
		//
		// Array of notification handles
		_nodesStoreConnectHandles: null,
		_linksStoreConnectHandles: null,
		
		
		constructor: function(){
			// summary: 
			//     Construct and instance of ibm_ilog.diagram.widget.DataConnector and 
			//     initialise it.              
			// tags:
			//	    private      
			this._initAttributes();
			this._startNodeBinding = DataConnector.StartLinkFieldname;
			this._endNodeBinding = DataConnector.EndLinkFieldname;
			this._model = new _DataModel();
			
		},
		
		_initAttributes: function(){
			// summary: 
			//     initialise attributes              
			// tags:
			//	    private      
			this._nodesStoreConnectHandles = null;
			this._linksStoreConnectHandles = null;
			this._createLinksForHierarchy = true;
			this._nodesQuery = null;
			this._nodesQueryOptions = null;
			this._parentBinding = null;
			this._parentBindingFunction = null;
			this._childBinding = null;
			this._childBindingFunction = null;
			this._predecessorsBinding = null;
			this._predecessorsBindingFunction = null;
			this._successorsBinding = null;
			this._successorsBindingFunction = null;
			this._xBinding = null;
			this._xBindingFunction = null;
			this._yBinding = null;
			this._yBindingFunction = null;
			this._nodesGraphBinding = null;
			this._nodesGraphBindingFunction = null;
			this._centerOnLocation = false;
			this._nodesGraphFuncMap = null;
			this._nodesGraphProperties = "";
			this._nodesStore = null;
			this._nodesStoreFeatures = null;
			this._linksGraphFuncMap = null;
			this._linksGraphProperties = "";
			this._linksStore = null;
			this._linksStoreFeatures = null;
			this._linksQuery = null;
			this._linksQueryOptions = null;
			this._linksGraphBinding = null;
			this._linksGraphBindingFunction = null;
			this._startNodeBinding = null;
			this._startNodeBindingFunction = null;
			this._endNodeBinding = null;
			this._endNodeBindingFunction = null;
		},
		
		_load: function(){
			// summary: 
			//     Validate options, loads data and creates the graph
			// tags:
			//	    private 
			if (this._isValidModel()) {
				this._reset();
				this._buildModelFromStores();
			}
		},
		
		_destroyDataConnector: function(){
			// summary: 
			//     Clear up all resources used by the DataConnector
			// tags:
			//	    private 
			this._reset();
			delete this._model;
			this._initAttributes();
		},
		
		_reset: function(){
			// summary: 
			//     Reset the current collection of nodes and links.
			// tags:
			//	    private 
			
			//this._model.initLinks();
			//this._model.initNodes();
			this._clearNodes();
		},
		
		_clearNodes: function(){
			// summary: 
			//     Clear the current collection of nodes created from the nodes store.
			// tags:
			//	    private
			
			// clear graphical shapes			
			this._model.forEachEntry(function(e){
				// remove all links attached to this node (both linksStore and implicits)
				var links = e.node.getLinks();
				array.forEach(links, function(link){
					this._linkDispose(link);
				}, this);
				// and dispose the node
				this._nodeDispose(e.node);
			}, this);
			// clear data model
			this._model.initNodes();
			// also the links since all the links shapes have been removed
			this._model.initLinks();
		},
		
		_clearLinks: function(){
			// summary: 
			//     Clear the current collection of links created from the link store
			// tags:
			//	    private
			
			this._model.forEachLinkEntry(function(e){
				var link = e.link;
				var ld = link.getData();
				if (ld && ld.store && ld.store === this._linksStore) {
					this._linkDispose(link);
				}
			}, this);
			this._model.initLinks();
		},
		
		
		// ===============================================================
		// Internal child/parent binding functions
		// ===============================================================
		
		_childBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item){
			// summary: 
			//     Find the items which are associated with the childBinding field.
			// tags:
			//	    private 
			return StoreApi.getValues(nodesStore, item, this._childBinding);
		},
		
		_parentBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item){
			// summary: 
			//     Find the items which are associated with the parentBinding field.
			// tags:
			//	    private   
			return StoreApi.getValues(nodesStore, item, this._parentBinding);
		},
		
		
		// ===============================================================
		// Internal successors/from binding functions
		// ===============================================================
		
		_successorsBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item){
			// summary: 
			//     Find the items which are associated with the successorsBinding field.
			// tags:
			//	    private 
			return StoreApi.getValues(nodesStore, item, this._successorsBinding);
		},
		
		_predecessorsBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item){
			// summary: 
			//     Find the items which are associated with the predecessorsBinding field.
			// tags:
			//	    private   
			return StoreApi.getValues(nodesStore, item, this._predecessorsBinding);
		},
		
		// ===============================================================
		// Internal x/y binding functions
		// ===============================================================
		
		_xBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item,/* new value */ value){
			// summary: 
			//     Get/Set the value of the xBinding field.
			// tags:
			//	    private
			var retVal;
			if (value) {
				retVal = StoreApi.setValue(nodesStore, item, this._xBinding, value);
			} else {
				retVal = StoreApi.getValue(nodesStore, item, this._xBinding);
				if (!retVal) {
					retVal = 0;
				}
			}
			return retVal;
		},
		
		_yBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item, /* new value */ value){
			// summary: 
			//     Get/Set the value of the yBinding field.
			// tags:
			//	    private   
			var retVal;
			if (value) {
				retVal = StoreApi.setValue(nodesStore, item, this._yBinding, value);
			} else {
				retVal = StoreApi.getValue(nodesStore, item, this._yBinding);
				if (!retVal) {
					retVal = 0;
				}
				
			}
			return retVal;
			
		},
		
		// ===============================================================
		// Internal nodes/links graphical binding functions
		// ===============================================================
		
		_graphBindingByNodesStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item,/* new value */ value){
			// summary: 
			//     Get/Set the value of the nodesGraphBinding field.
			// tags:
			//	    private
			var retVal;
			if (value) {
				var jvalue = json.toJson(value);
				retVal = StoreApi.setValue(nodesStore, item, this._nodesGraphBinding, jvalue);
			} else {
				var retValj = StoreApi.getValue(nodesStore, item, this._nodesGraphBinding);
				retVal = json.fromJson(retValj);
			}
			return retVal;
		},
		
		_graphBindingByLinksStoreFieldName: function(/* dojo.data.api.Read */nodesStore, /* store item */ item, /* new value */ value){
			// summary: 
			//     Get/Set the value of the linksGraphBinding field.
			// tags:
			//	    private   
			var retVal;
			if (value) {
				var jvalue = json.toJson(value);
				retVal = StoreApi.setValue(nodesStore, item, this._linksGraphBinding, jvalue);
			} else {
				var retValj = StoreApi.getValue(nodesStore, item, this._linksGraphBinding);
				retVal = json.fromJson(retValj);
			}
			return retVal;
		},
		
		// ===============================================================
		// Setters and Getters
		// ===============================================================
		
		_setCreateLinksForHierarchy: function(/* Boolean */value){
			// summary: 
			//     Sets the render mode.
			// tags:
			//	    private 
			this._createLinksForHierarchy = value;
			return value;
		},
		
		_getCreateLinksForHierarchy: function(){
			// summary: 
			//     Returns the render mode.
			// tags:
			//	    private 
			return this._createLinksForHierarchy;
		},
		
		_setNodesStore: function(/* dojo.data.api.Read */value){
			// summary: 
			//     Sets the nodes store. The nodes store provides data item to be 
			//     represented as nodes in the diagram. It must implement, at least, 
			//     the dojo.data.api.Read and dojo.data.api.Indentity interfaces.
			// tags:
			//	    private 
			
			// If there is already a store associated already...
			if (this._nodesStore) {
				// clear graphical shapes
				this._clearNodes();
				// drop notifications from it
				this._unwireNodesStore();
				this._nodesStoreFeatures = null;
				StoreApi.dispose(this._nodesStore);
			}
			
			if (value) {
				StoreApi.bridge(value);
				this._nodesStoreFeatures = StoreApi.getFeatures(value);
				if (this._isSupportedStore(this._nodesStoreFeatures)) {
					this._nodesStore = value;
				} else {
					StoreApi.dispose(value);
					this._nodesStore = null;
					this._nodesStoreFeatures = null;
				}
			} else {
				this._nodesStore = null;
			}
			
			// wire notifications in 
			if (this._nodesStore) {
				this._wireNodesStore();
			}
			
			return this._nodesStore;
		},
		
		_getNodesStore: function(){
			// summary: 
			//     Returns the nodes store.
			// tags:
			//	    private 
			return this._nodesStore;
		},
		
		_setLinksStore: function(/* dojo.data.api.Read */value){
			// summary: 
			//     Sets the links store. The links store provides data item to be represented
			//     as links in the diagram. It must implement at least the dojo.data.api.Read and
			//     dojo.data.api.Indentity interfaces. If no links store is specified, links
			//     are created implicitly from the nodes data store via the connections 
			//     definition as defined by the predecessorsBinding/successorsBinding properties.
			// tags:
			//	    private 
			
			// If there is already a store associated already......
			if (this._linksStore) {
				// clear graphical shapes
				this._clearLinks();
				// drop notifications from it
				this._unwireLinksStore();
				this._linksStoreFeatures = null;
				StoreApi.dispose(this._linksStore);
			}
			
			if (value) {
				StoreApi.bridge(value);
				this._linksStoreFeatures = StoreApi.getFeatures(value);
				if (this._isSupportedStore(this._linksStoreFeatures)) {
					this._linksStore = value;
				} else {
					StoreApi.dispose(value);
					this._linksStore = null;
					this._linksStoreFeatures = null;
				}
			} else {
				this._linksStore = null;
			}
			// wire notifications in 
			if (this._linksStore) {
				this._wireLinksStore();
			}
			
			
			return this._linksStore;
		},
		
		_getLinksStore: function(){
			// summary: 
			//     Returns the links store.
			// tags:
			//	    private 
			return this._linksStore;
		},
		
		_setNodesQuery: function(value){
			// summary: 
			//     Sets the nodes query.
			// tags:
			//	    private 
			var validatedQuery = null;
			if (!value || typeof value === 'string' || value instanceof Object) {
				validatedQuery = this._nodesQuery = value;
			} else {
				R.error("CannotUseAStringForAQuery");
			}
			return validatedQuery;
		},
		
		_getNodesQuery: function(){
			// summary: 
			//     Returns the nodes query.
			// tags:
			//	    private 
			return this._nodesQuery;
		},
		
		_setNodesQueryOptions: function(value){
			// summary: 
			//     Sets the nodes query options.
			// tags:
			//	    private 
			var validatedQueryOptions = null;
			validatedQueryOptions = this._nodesQueryOptions = value;
			return validatedQueryOptions;
		},
		
		_getNodesQueryOptions: function(){
			// summary: 
			//     Returns the nodes query options.
			// tags:
			//	    private 
			return this.validatedQueryOptions;
		},
		
		_setChildBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover child items.
			// tags:
			//	    private 
			this._childBinding = value;
			if (this._childBinding instanceof Function) {
				this._childBindingFunction = value;
			} else {
				if (!this._childBinding || this._childBinding === "") {
					this._childBindingFunction = null;
				} else {
					this._childBindingFunction = this._childBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getChildBinding: function(){
			// summary: 
			//     Returns the childBinding.
			// tags:
			//	    private 
			return this._childBinding;
		},
		
		_setParentBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover parent items.
			// tags:
			//	    private 
			this._parentBinding = value;
			if (this._parentBinding instanceof Function) {
				this._parentBindingFunction = value;
			} else {
				if (!this._parentBinding || this._parentBinding === "") {
					this._parentBindingFunction = null;
				} else {
					this._parentBindingFunction = this._parentBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getParentBinding: function(){
			// summary: 
			//     Returns the parentBinding.
			// tags:
			//	    private 
			return this._parentBinding;
		},
		
		_setPredecessorsBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover items predecessors.
			// tags:
			//	    private 
			this._predecessorsBinding = value;
			if (this._predecessorsBinding instanceof Function) {
				this._predecessorsBindingFunction = value;
			} else {
				if (!this._predecessorsBinding || this._predecessorsBinding === "") {
					this._predecessorsBindingFunction = null;
				} else {
					this._predecessorsBindingFunction = this._predecessorsBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getPredecessorsBinding: function(){
			// summary: 
			//     Returns the predecessorsBinding.
			// tags:
			//	    private 
			return this._predecessorsBinding;
		},
		
		_setSuccessorsBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover items to link TO.
			// tags:
			//	    private 
			this._successorsBinding = value;
			if (this._successorsBinding instanceof Function) {
				this._successorsBindingFunction = value;
			} else {
				if (!this._successorsBinding || this._successorsBinding === "") {
					this._successorsBindingFunction = null;
				} else {
					this._successorsBindingFunction = this._successorsBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getSuccessorsBinding: function(){
			// summary: 
			//     Returns the successorsBinding.
			// tags:
			//	    private 
			return this._successorsBinding;
		},
		
		_setXBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover an items x position.
			// tags:
			//	    private 
			this._xBinding = value;
			if (this._xBinding instanceof Function) {
				this._xBindingFunction = value;
			} else {
				if (!this._xBinding || this._xBinding === "") {
					this._xBindingFunction = null;
				} else {
					this._xBindingFunction = this._xBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getXBinding: function(){
			// summary: 
			//     Returns the xBinding.
			// tags:
			//	    private 
			return this._xBinding;
		},
		
		_setYBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover an items y position.
			// tags:
			//	    private 
			this._yBinding = value;
			if (this._yBinding instanceof Function) {
				this._yBindingFunction = value;
			} else {
				if (!this._yBinding || this._yBinding === "") {
					this._yBindingFunction = null;
				} else {
					this._yBindingFunction = this._yBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getYBinding: function(){
			// summary: 
			//     Returns the yBinding.
			// tags:
			//	    private 
			return this._yBinding;
		},
		
		_setNodesGraphBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover an node item's graph data.
			// tags:
			//	    private 
			this._nodesGraphBinding = value;
			if (this._nodesGraphBinding instanceof Function) {
				this._nodesGraphBindingFunction = value;
			} else {
				if (!this._nodesGraphBinding || this._nodesGraphBinding === "") {
					this._nodesGraphBindingFunction = null;
				} else {
					this._nodesGraphBindingFunction = this._graphBindingByNodesStoreFieldName;
				}
			}
			return value;
		},
		
		_getNodesGraphBinding: function(){
			// summary: 
			//     Returns the _nodesGraphBinding.
			// tags:
			//	    private 
			return this._nodesGraphBinding;
		},
		
		_setNodesGraphProperties: function(nodesGraphProperties){
			// summary: 
			//     Set the list of nodes properties to save
			// tags:
			//	    private
			this._nodesGraphProperties = nodesGraphProperties;
			// Set map to null so it can be-evaluated the next time it it needed.
			this._nodesGraphFuncMap = null;
		},
		
		_getNodesGraphProperties: function(){
			// summary: 
			//     Get the list of nodes properties
			// tags:
			//	    private            
			return this._nodesGraphProperties;
		},
		
		_setCenterOnLocation: function(value){
			// summary: 
			//     Sets the centerOnLocation value.
			// tags:
			//	    private 
			this._centerOnLocation = value;
			return value;
		},
		
		_getCenterOnLocation: function(value){
			// summary: 
			//     Returns the centerOnLocation value.
			// tags:
			//	    private 
			return this._centerOnLocation;
		},
		
		
		_setLinksQuery: function(value){
			// summary: 
			//     Sets the links query.
			// tags:
			//	    private 
			var validatedQuery = null;
			if (!value || typeof value === 'string' || value instanceof Object) {
				validatedQuery = this._linksQuery = value;
			} else {
				R.error("CannotUseAStringForAQuery");
			}
			return validatedQuery;
		},
		
		_getLinksQuery: function(){
			// summary: 
			//     Returns the links query.
			// tags:
			//	    private 
			return this._linksQuery;
		},
		
		_setLinksQueryOptions: function(value){
			// summary: 
			//     Sets the links query options.
			// tags:
			//	    private 
			var validatedQueryOptions = null;
			validatedQueryOptions = this._linksQueryOptions = value;
			return validatedQueryOptions;
		},
		
		_getLinksQueryOptions: function(){
			// summary: 
			//     Returns the links query options.
			// tags:
			//	    private 
			return this.validatedQueryOptions;
		},
		
		_setLinksGraphBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover an link item's graph data.
			// tags:
			//	    private 
			this._linksGraphBinding = value;
			if (this._linksGraphBinding instanceof Function) {
				this._linksGraphBindingFunction = value;
			} else {
				if (!this._linksGraphBinding || this._linksGraphBinding === "") {
					this._linksGraphBindingFunction = null;
				} else {
					this._linksGraphBindingFunction = this._graphBindingByLinksStoreFieldName;
				}
			}
			return value;
		},
		
		_getLinksGraphBinding: function(){
			// summary: 
			//     Returns the _linksGraphBinding.
			// tags:
			//	    private 
			return this._linksGraphBinding;
		},
		
		_setLinksGraphProperties: function(linksGraphProperties){
			// summary: 
			//     Set the list of links properties to save
			// tags:
			//	    private
			
			this._linksGraphProperties = linksGraphProperties;
			// Set map to null so it can be-evaluated the next time it it needed.
			this._linksGraphFuncMap = null;
			
		},
		
		_getLinksGraphProperties: function(){
			// summary: 
			//     Get the list of links properties
			// tags:
			//	    private
			return this._linksGraphProperties;
		},
		
		_setStartNodeBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover an item which starts a link.
			// tags:
			//	    private 
			this._startNodeBinding = value;
			if (this._startNodeBinding instanceof Function) {
				this._startNodeBindingFunction = value;
			}
			return value;
		},
		
		_getStartNodeBinding: function(){
			// summary: 
			//     Returns the startNodeBinding.
			// tags:
			//	    private 
			return this._startNodeBinding;
		},
		
		_setEndNodeBinding: function(value){
			// summary: 
			//     Sets the information to tell us now to discover an item which ends a link.
			// tags:
			//	    private 
			this._endNodeBinding = value;
			if (this._endNodeBinding instanceof Function) {
				this._endNodeBindingFunction = value;
			}
			return value;
		},
		
		_getEndNodeBinding: function(){
			// summary: 
			//     Returns the endNodeBinding.
			// tags:
			//	    private 
			return this._endNodeBinding;
		},
		
		// ===============================================================
		// Validation functions
		// ===============================================================
		
		_isValidModel: function(){
			// summary: 
			//     Validates if we have enough information available
			//     to build a coherent collection of nodes and links
			// tags:
			//	    private 
			var isValid = false;
			
			if (this._nodesStore) {
				isValid = true;
			} else {
				R.error("NodesStoreRequired");
			}
			return isValid;
		},
		
		
		_isSupportedStore: function(features){
			// summary:
			//     Validates if the store features presented are enough to
			//     provide minimum level of functionality.
			// tags:
			//	    private 
			var isSupported = false;
			
			// ensure required interfaces are supported by the model
			if (!features.identity) {
				R.error("StoresMustSupportIdentity");
			} else {
				if (!features.canRead) {
					R.error("StoresMustSupportRead");
				} else {
					isSupported = true;
				}
			}
			return isSupported;
		},
		
		// ===============================================================
		// Node store loading functions
		// ===============================================================
		
		_buildModelFromStores: function(){
			// summary: 
			//     If nodesStore available and query supplied, load items based on the 
			//     nodesQuery and nodesQueryOptions. Then attempt to 
			//     resolve  connections from links store.
			// tags:
			//	    private 
			
			if (this._nodesStore && this._nodesQuery) {
				StoreApi.fetch(this._nodesStore, {
					query: this._nodesQuery,
					queryOptions: this._nodesQueryOptions,
					scope: this,
					onError: function(error, request){
						this.onNodesStoreFetchError.apply(this, arguments);
					},
					onComplete: function(items, request){
						for (var i = 0; i < items.length; i++) {
							this._addStoreItemToModel(items[i]);
						}
						this._resolveConnectionsFromLinksStore();
					}
				});
			} else {
				this._resolveConnectionsFromLinksStore();
			}
		},
		
		onStoreFetchError: function(error, request){
		
		},
		
		onNodesStoreFetchError: function(error, request){
			// if not on debugAtAllCosts, we won't see this
			R.warn("UnexpectedErrorOnNodesStoreFetch", error);
			this.onStoreFetchError.apply(this, arguments);
		},
		
		onLinksStoreFetchError: function(error, request){
			// if not on debugAtAllCosts, we won't see this
			R.warn("UnexpectedErrorOnLinksStoreFetch", error);
			this.onStoreFetchError.apply(this, arguments);
		},
		
		_loadNodesStoreItem: function(itemToLoad){
			// summary: 
			//     Load individual item from the nodesStore
			//     and add it to the model.
			// tags:
			//	    private 
			StoreApi.loadItem(this._nodesStore, {
				item: itemToLoad,
				scope: this,
				onError: function(error, request){
					throw error;
				},
				onItem: function(item, request){
					this._addStoreItemToModel(item);
				}
			});
		},
		
		_loadNodesStoreItemById: function(ident){
			// summary: 
			//     Load individual item from the nodesStore
			//     by it's identity and add it to the model.
			// tags:
			//	    private 
			StoreApi.fetchItemByIdentity(this._nodesStore, {
				identity: ident,
				scope: this,
				onError: function(error, request){
					this.onNodesStoreFetchError.apply(this, arguments);
				},
				onItem: function(item, request){
					this._addStoreItemToModel(item);
				}
			});
		},
		
		// ===============================================================
		// Link store loading functions
		// ===============================================================
		_getStartLinkItemId: function(linkStoreItem){
			// tags:
			//	    private 
			var startLinkItemId = null;
			
			if (this._startNodeBindingFunction) {
				var startLinkItem = this._startNodeBindingFunction.call(this, this._linksStore, linkStoreItem, this._nodesStore);
				startLinkItemId = StoreApi.getIdentity(this._nodesStore, startLinkItem);
			} else {
				startLinkItemId = StoreApi.getValue(this._linksStore, linkStoreItem, this._startNodeBinding);
			}
			return startLinkItemId;
		},
		
		_getEndLinkItemId: function(linkStoreItem){
			// tags:
			//	    private 
			var endLinkItemId = null;
			
			if (this._endNodeBindingFunction) {
				var endLinkItem = this._endNodeBindingFunction.call(this, this._linksStore, linkStoreItem, this._nodesStore);
				endLinkItemId = StoreApi.getIdentity(this._nodesStore, endLinkItem);
			} else {
				endLinkItemId = StoreApi.getValue(this._linksStore, linkStoreItem, this._endNodeBinding);
			}
			return endLinkItemId;
		},
		
		_resolveConnectionsFromLinksStore: function(){
			// summary: 
			//     If linksStore available, load items based on the 
			//     linksQuery and linksQueryOptions. Add information
			//     about the item relationships into the model, Then 
			//     attempt to resolve connections from nodes store.
			// tags:
			//	    private 
			if (this._linksStore) {
				StoreApi.fetch(this._linksStore, {
					query: this._linksQuery,
					queryOptions: this._linksQueryOptions,
					scope: this,
					onError: function(error, request){
						this.onLinksStoreFetchError.apply(this, arguments);
					},
					onComplete: function(items, request){
					
						for (var i = 0; i < items.length; i++) {
							var currentLinkStoreItem = items[i];
							var linkEntry = this._addLinkStoreItemToModel(currentLinkStoreItem);
							linkEntry.startId = this._getStartLinkItemId(currentLinkStoreItem);
							linkEntry.endId = this._getEndLinkItemId(currentLinkStoreItem);
							
							// If we have enough information to describe a link then
							// update the model. 
							if (linkEntry.startId !== null) {
								linkEntry.start = this._model.getEntry(linkEntry.startId);
							}
							if (linkEntry.endId !== null) {
								linkEntry.end = this._model.getEntry(linkEntry.endId);
							}
							
							if (!linkEntry.start || !linkEntry.end) {
								linkEntry.pending = true;
							}
						}
						this._resolveConnectionsFromNodesStore();
					}
				});
			} else {
				this._resolveConnectionsFromNodesStore();
			}
		},
		
		// ===============================================================
		// NodeStore Notification-related functions
		// ===============================================================
		
		_wireNodesStore: function(){
			// summary: 
			//     If the nodesStore supports notification, register to be informed of any changes
			// tags:
			//	    private  
			if (this._nodesStore) {
				var features = this._nodesStoreFeatures, args;
				if (features.canNotify) {
					args = {
						scope: this,
						onNew: this._onNewNodeItem,
						onUpdate: this._onSetNodeItem,
						onRemove: this._onDeleteNodeItem
					};
					/*
					
					 this._nodesStoreConnectHandles = new ArrayList();
					
					 this._nodesStoreConnectHandles.add(connect.connect(this._nodesStore, "onNew", this, "_onNewNodeItem"));
					
					 this._nodesStoreConnectHandles.add(connect.connect(this._nodesStore, "onSet", this, "_onSetNodeItem"));
					
					 this._nodesStoreConnectHandles.add(connect.connect(this._nodesStore, "onDelete", this, "_onDeleteNodeItem"));
					
					 */
					
				}
				
				if (features.canWrite) {
					if (!args) {
						args = {
							scope: this
						}
					}
					lang.mixin(args, {
						onSave: this._onSaveNodesStore,
						onRevert: this._onRevertNodesStore
					})
					/*
					 this._nodesStoreConnectHandles.add(connect.connect(this._nodesStore, "save", this, "_onSaveNodesStore"));
					 this._nodesStoreConnectHandles.add(connect.connect(this._nodesStore, "revert", this, "_onRevertNodesStore"));
					 */
					// the connections to _onChanged and onBoundsChanged node events
					this._onNodeChangedHandlers = {};
				}
				if (args) {
					var h = StoreApi.listen(this._nodesStore, args);
					this._nodesStoreConnectHandles = new ArrayList(h);
				}
				
			}
		},
		
		_unwireNodesStore: function(){
			// summary:
			//     unregister any nodesStore notifications we may have set.
			// tags:
			//	    private 
			if (this._nodesStoreConnectHandles && this._nodesStoreConnectHandles.count > 0) {
				this._nodesStoreConnectHandles.forEach(function(entry){
					//connect.disconnect(entry);
					entry.remove();
				}, this);
				this._nodesStoreConnectHandles = null;
			}
			// the content of _onNodeChangedHandlers is disposed on a per-node basis, in _nodeDispose()
			this._onNodeChangedHandlers = null;
		},
		
		_connectItemsInExistingDiagram: function(parentItem, childItem, isHierarchicalConnection){
			// summary:
			//     utility function which connects a new node into an existing diagram.
			// tags:
			//	    private 
			
			// console.info("_connectItemsInExistingDiagram");
			// console.info("\tparentItem=" + parentItem ? this._nodesStore.getIdentity(parentItem) : "null");
			// console.info("\childItem=" + childItem ? this._nodesStore.getIdentity(childItem) : "null");
			
			var parentNode = this._getNodeFromStoreItem(parentItem);
			var childNode = this._getNodeFromStoreItem(childItem);
			
			
			if (this._createLinksForHierarchy || !isHierarchicalConnection) {
				// console.info("_createLinksForHierarchy=true");
				if (!childNode) {
					var entry = this._addStoreItemToModel(childItem);
					childNode = entry.node = this.createNode(null, childItem);
					this._applyNodeStoreItemGraphics(entry.node);
				}
				if (!parentNode) {
					var entry = this._addStoreItemToModel(parentItem);
					parentNode = entry.node = this.createNode(null, parentItem);
					this._applyNodeStoreItemGraphics(entry.node);
				}
				var link = this.connectNodes(parentNode, childNode);
			} else {
				// console.info("_createLinksForHierarchy=false");
				if (!childNode) {
				
					if (parentNode && parentNode._isIBMDiagramSubgraph) {
						// console.info("parentNode is Subgraph");
						var entry = this._addStoreItemToModel(childItem);
						childNode = entry.node = this.createNode(null, childItem, parentNode.getGraph());
						this._applyNodeStoreItemGraphics(entry.node);
					} else {
						// console.info("parentNode is Node");
						// parent node needs to change from node to subgraph
						//
						var entry = this._addStoreItemToModel(childItem);
						var parentSubgraph = this.createSubgraph(null, parentItem, parentNode.getParent());
						// Set this subgraph in the the model
						this._setNodeFromStoreItem(parentItem, parentSubgraph);
						childNode = entry.node = this.createNode(null, childItem, parentSubgraph.getGraph());
						// then re-wire linked node to the new subgraph                        
						array.forEach(parentNode.getLinks(true, false), function(link){
							var port = new port.AutomaticPort();
							parentSubgraph.addPort(port);
							link.setStartPort(port);
						});
						array.forEach(parentNode.getLinks(false, true), function(link){
							var port = new port.AutomaticPort();
							parentSubgraph.addPort(port);
							link.setEndPort(port);
						});
						// dispose of the old parentNode
						this._nodeDispose(parentNode, parentItem);
						
						// position child node, then parent subgraph
						this._applyNodeStoreItemGraphics(childNode);
						this._applyNodeStoreItemGraphics(parentSubgraph);
						
					}
				} else {
					// Situation does not occur in current function usage..
				}
			}
		},
		
		_onNodeChangedHandlers: null,
		
		_addOnChangedHandler: function(node, event, handler){
			var h = connect.connect(node, event, this, handler);
			var hl = this._onNodeChangedHandlers[node.getId()];
			if (!hl) {
				hl = [];
				this._onNodeChangedHandlers[node.getId()] = hl;
			}
			hl.push(h);
		},
		
		_clearOnChangedHandlers: function(node){
			if (!this._onNodeChangedHandlers) 
				return;
			var id = node.getId(), hl = this._onNodeChangedHandlers[id];
			if (!hl) 
				return;
			array.forEach(hl, function(h){
				connect.disconnect(h);
			});
			hl.length = 0;
			delete this._onNodeChangedHandlers[id];
		},
		
		// ===============================================================
		//  Function to extract/store graphical information in the nodes 
		//  store itself
		// ===============================================================
		_applyNodeStoreItemGraphics: function(node){
			// summary:
			//     Applies any graphical information to a node/subgraph
			//     that is stored in the nodesStore item, specifically..
			//     (a) TBD from the specified function
			//     (b) x/y positioning.
			// tags:
			//	    private 
			if (this._nodesGraphBindingFunction || this._xBindingFunction || this._yBindingFunction) {
			
				var item = node.getData().item, isWrite = this._nodesStoreFeatures.canWrite;
				
				if (this._nodesGraphBindingFunction) {
					var gfxInfo = this._nodesGraphBindingFunction.call(this, this._nodesStore, item);
					if (gfxInfo) {
						this._applyProperties(node, gfxInfo);
					}
					
					// If store supports write then monitor node for changes
					if (isWrite) {
						this._addOnChangedHandler(node, '_onChanged', function(){
							if (this._nodesGraphBindingFunction && node.getData && node.getData()) {
							
								var nitem = node.getData().item;
								if (nitem && StoreApi.isItem(this._nodesStore, nitem)) {
									try {
										this._internalSet = true;
										
										var gfxInfo = this._extractProperties(node, this._nodesGraphProperties);
										this._nodesGraphBindingFunction.call(this, this._nodesStore, nitem, gfxInfo);
									} finally {
										this._internalSet = false;
									}
								}
							}
						});
					}
				}
				if (this._xBindingFunction || this._yBindingFunction) {
				
					var posx = 0;
					var posy = 0;
					
					if (this._xBindingFunction) {
						posx = this._xBindingFunction.call(this, this._nodesStore, item);
					}
					if (this._yBindingFunction) {
						posy = this._yBindingFunction.call(this, this._nodesStore, item);
					}
					if (this._centerOnLocation) {
						var bounds = node.getBounds();
						posx = posx - (bounds.width / 2);
						posy = posy - (bounds.height / 2);
					}
					
					node.move(posx, posy);
					
					// If store supports write then monitor node for changes
					if (isWrite) {
						this._addOnChangedHandler(node, "onBoundsChanged", function(shape, reason){
							if (reason === "setTransform" && shape.getData && shape.getData()) {
							
								var posx = 0;
								var posy = 0;
								
								var bounds = shape.getBounds();
								
								if (this._centerOnLocation) {
									posx = bounds.x + (bounds.width / 2);
									posy = bounds.y + (bounds.height / 2);
								} else {
									posx = bounds.x;
									posy = bounds.y;
								}
								
								var sitem = shape.getData().item;
								if (sitem && StoreApi.isItem(this._nodesStore, sitem)) {
									if (this._xBindingFunction) {
										// read the current x value and only write new one if they are different
										var curposx = this._xBindingFunction.call(this, this._nodesStore, sitem);
										if (curposx !== posx) {
											try {
												this._internalSet = true;
												this._xBindingFunction.call(this, this._nodesStore, sitem, posx);
											} finally {
												this._internalSet = false;
												
											}
										}
									}
									if (this._yBindingFunction) {
										// read the current y value and only write new one if they are different
										var curposy = this._yBindingFunction.call(this, this._nodesStore, sitem);
										if (curposy !== posy) {
											try {
												this._internalSet = true;
												this._yBindingFunction.call(this, this._nodesStore, sitem, posy);
											} finally {
												this._internalSet = false;
											}
										}
									}
								}
							}
						});
					}
				}
			}
		},
		
		_connectItems: function(item, filteritem){
			// summary:
			//     Resolve and links through to/from/parent/child bindings
			//     filternode can be specified to limit resolution 
			//     between just the "item" node and a filter node
			// tags:
			//	    private 
			if (this._successorsBindingFunction) {
				var connectingItems = this._successorsBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					array.forEach(connectingItems, function(connectingItem){
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(item, connectingItem, false);
						}
					}, this);
				}
			}
			
			if (this._predecessorsBindingFunction) {
				var connectingItems = this._predecessorsBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					array.forEach(connectingItems, function(connectingItem){
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(connectingItem, item, false);
						}
					}, this);
				}
			}
			
			if (this._parentBindingFunction) {
				var connectingItems = this._parentBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					array.forEach(connectingItems, function(connectingItem){
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(connectingItem, item, true);
						}
					}, this);
				}
			}
			
			if (this._childBindingFunction) {
				var connectingItems = this._childBindingFunction.call(this, this._nodesStore, item);
				if (connectingItems) {
					array.forEach(connectingItems, function(connectingItem){
						if (!filteritem || filteritem === connectingItem) {
							this._connectItemsInExistingDiagram(item, connectingItem, true);
						}
					}, this);
				}
			}
			
		},
		
		acceptNewItem:function(item, isNode){
			// summary:
			//		Returns whether the specified new item should be processed. 
			// description:
			//		This method is invoked when a new item has been added to the data store.
			//		The default implementation returns true, meaning that a corresponding Node or Link
			//		instance is created for every new items.
			// item: Object
			//		the new data item.
			// isNode: Boolean
			//		Whether the item represents a Node or a Link item.
			// returns:
			//		A Boolean indicating whether the specified new item should be processed.
			return true; // Boolean
		},
		
		_onNewNodeItem: function(newItem, parentInfo){
			// summary:
			//		called when a new item is added to the nodesStore.
			// tags:
			//		private 
			
			// console.info("DataConnector._onNewNodeItem id=" + this._nodesStore.getIdentity(newItem) + ",parentInfo=" + (parentInfo ? this._nodesStore.getIdentity(parentInfo.item) : "no parent"));
			
			if (!this.acceptNewItem(newItem, true /* true = node*/)){
				return;
			}
			
			// connect this item to others, possibly creating a node/subgraph in the process
			this._connectItems(newItem, null);
			
			// Check if a parent has been defined
			if (parentInfo) {
				var parentNode = this._getNodeFromStoreItem(parentInfo.item);
				if (parentNode) {
					this._connectItems(parentInfo.item, newItem);
				}
			}
			
			// if a node/subgraph is not yet created for this item. then create one
			if (!this._getNodeFromStoreItem(newItem)) {
				var entry = this._addStoreItemToModel(newItem);
				entry.node = this.createNode(null, newItem);
				this._applyNodeStoreItemGraphics(entry.node);
			}
			
			// Check if this item completes any pending links
			// Create links from lists in the link model 
			var nodeEntry = this._model.getEntry(StoreApi.getIdentity(this._nodesStore, newItem));
			this._model.forEachPendingLinkEntry(function(linkEntry, nodeEntry){
				if (!linkEntry.start) {
					if (nodeEntry.id === linkEntry.startId) {
						linkEntry.start = nodeEntry;
					}
					
				}
				if (!linkEntry.end) {
					if (nodeEntry.id === linkEntry.endId) {
						linkEntry.end = nodeEntry;
					}
				}
				if (linkEntry.start && linkEntry.end) {
					// We have found a pending link which can be completed with this item 
					linkEntry.link = this.connectNodes(linkEntry.start.node, linkEntry.end.node, null, linkEntry.item);
					this._applyLinkStoreItemGraphics(linkEntry.link);
					linkEntry.pending = false;
				}
			}, nodeEntry, this);
		},
		
		_onDeleteNodeItem: function(item){
			// summary:
			//     called when an item is deleted from the nodesStore.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onDeleteNodeItem id=" + this._nodesStore.getIdentity(item));
			// We first need to iteratively process any child items, as these
			// would not have been loaded if this potential parent 
			// not existed originally. Unfortunately we can't use the _childBindingFunction
			// to achieve this, as the datastore considers that the item currently being 
			// processed has been deleted. We'll need to find the children from the model instead.
			var id = StoreApi.getIdentity(this._nodesStore, item);
			var shape = this._getNodeFromStoreItem(item);
			var dc = this;
			if (shape) {
				array.forEach(shape.getLinks(), function(link){
					dc._linkDispose(link);
				});
				this._nodeDispose(shape, item);
			}
			this._model.removeEntry(id);
			
		},
		
		_getExtraElements: function(Array1, Array2){
			// tags:
			//	    private 
			var extras = [];
			for (var i = 0; i < Array2.length; i++) {
				var arrlen = Array1.length;
				var j = 0;
				for (j = 0; j < arrlen; j++) {
					if (Array2[i] === Array1[j]) {
						break;
					}
				}
				if (j === arrlen) {
					// this element was not found
					extras.push(Array2[i]);
				}
			}
			return extras;
		},
		
		_onSetNodeItem: function(modifiedItem, attribute, oldValue, newValue){
			// summary:
			//     called when an item in nodesStore changes.
			//     This code is only interested in changes in attributes related to
			//     - store item relationships.
			//     - store item label.
			//     - x/y position
			//
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSetNodeItem id=" + this._nodesStore.getIdentity(modifiedItem) + ",attribute=" + attribute + ",old=" + oldValue + ",new=" + newValue);
			
			if (this._internalSet) 
				return;
			
			if (StoreApi.fromDojoStore(this._nodesStore)) {
				var entry = this._model.getEntry(StoreApi.getIdentity(this._nodesStore, modifiedItem));
				if (entry._bindingBackup) {
					var bo = this._getChangedNodeBindingValues(entry, true /*update backup val with new val*/);
					if (bo) {
						attribute = bo.p;
						oldValue = bo.o;
						newValue = bo.n;
					}
				}
			}
			
			var shape = this._getNodeFromStoreItem(modifiedItem);
			if (shape) {
			
				var oldState;
				var newState;
				
				if (StoreApi.isItem(this._nodesStore, oldValue)) {
					oldState = [oldValue];
				} else {
					oldState = oldValue || [];
				}
				
				if (StoreApi.isItem(this._nodesStore, newValue)) {
					newState = [newValue];
				} else {
					newState = newValue || [];
				}
				
				if (oldState instanceof Array && newState instanceof Array) {
					var newItems = this._getExtraElements(oldState, newState);
					var deletedItems = this._getExtraElements(newState, oldState);
					// Look to remove links between this shape and the deleted items
					// console.info("deletedItems.length=" + deletedItems.length);
					for (var i = 0; i < deletedItems.length; i++) {
						var shape2 = this._getNodeFromStoreItem(deletedItems[i]);
						// console.info("\titem=" + i + " shape2=" + shape2 + " item=" + deletedItems[i].name);
						var connections = shape.getLinks();
						for (var j = 0; j < connections.length; j++) {
							var link = connections[j];
							if (link.getEndNode() === shape2) {
								this._linkDispose(link);
							}
						}
						connections = shape2.getLinks();
						for (var j = 0; j < connections.length; j++) {
							var link = connections[j];
							if (link.getEndNode() === shape) {
								this._linkDispose(link);
							}
						}
					}
					// console.info("newItems.length=" + newItems.length);
					// Add links between this shape and the new item					
					for (var i = 0; i < newItems.length; i++) {
						this._connectItems(newItems[i], modifiedItem);
						this._connectItems(modifiedItem, newItems[i]);
					}
					
				}
				
				// Update Node label
				// check if the updated attribute is involved in the label
				var attr = StoreApi.getLabelAttributes(this._nodesStore, modifiedItem);
				if (attr) {
					for (var i = 0; i < attr.length; ++i) {
						if (attr[i] === attribute) {
							var label = StoreApi.getLabel(this._nodesStore, modifiedItem);
							shape.setLabel(newValue);
							break;
						}
					}
				}
				
				// Update Node position
				this._applyNodeStoreItemGraphics(shape);
			}
		},
		
		_onSaveNodesStore: function(){
			// summary:
			//     called after the nodesStore has been saved.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSaveNodesStore");
			this.reset();
			this._load();
		},
		
		_onRevertNodesStore: function(){
			// summary:
			//     called after the nodesStore has reverted back to the last "saved" state.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onRevertNodesStore");
			this.reset();
			this._load();
		},
		
		// ===============================================================
		// LinkStore Notification-related functions
		// ===============================================================
		
		_wireLinksStore: function(){
			// summary: 
			//     If the nodesStore supports notification, register to be informed of any changes
			// tags:
			//	    private    
			if (this._linksStore) {
				var features = this._linksStoreFeatures, args;
				if (features.canNotify) {
					args = {
						scope: this,
						onNew: this._onNewLinkItem,
						onUpdate: this._onSetLinkItem,
						onRemove: this._onDeleteLinkItem
					};
					/*
					
					 this._linksStoreConnectHandles = new ArrayList();
					
					 this._linksStoreConnectHandles.add(connect.connect(this._linksStore, "onNew", this, "_onNewLinkItem"));
					
					 this._linksStoreConnectHandles.add(connect.connect(this._linksStore, "onSet", this, "_onSetLinkItem"));
					
					 this._linksStoreConnectHandles.add(connect.connect(this._linksStore, "onDelete", this, "_onDeleteLinkItem"));
					
					 */
					
				}
				
				if (features.canWrite) {
					if (!args) {
						args = {
							scope: this
						}
					}
					lang.mixin(args, {
						onSave: this._onSaveLinksStore,
						onRevert: this._onRevertLinksStore
					})
					/*
					 this._linksStoreConnectHandles.add(connect.connect(this._linksStore, "save", this, "_onSaveLinksStore"));
					 this._linksStoreConnectHandles.add(connect.connect(this._linksStore, "revert", this, "_onRevertLinksStore"));
					 */
					this._onLinkShapeHandlers = {};
				}
				if (args) {
					var h = StoreApi.listen(this._linksStore, args);
					this._linksStoreConnectHandles = new ArrayList(h);
				}
				
			}
		},
		
		_unwireLinksStore: function(){
			// summary:
			//     unregister any nodesStore notifications we may have set.
			// tags:
			//	    private 
			if (this._linksStoreConnectHandles && this._linksStoreConnectHandles.count > 0) {
				this._linksStoreConnectHandles.forEach(function(entry){
					//connect.disconnect(entry);
					entry.remove();
				}, this);
				this._linksStoreConnectHandles = null;
			}
			this._onLinkShapeHandlers = null;
		},
		
		_onNewLinkItem: function(newitem, parentInfo){
			// summary:
			//     called when a new item added to the linksStore.
			// tags:
			//	    private 

			if (!this.acceptNewItem(newitem, false /* false = link*/)){
				return;
			}
			
			var id = StoreApi.getIdentity(this._linksStore, newitem);
			// console.info("DataConnector._onNewLinkItem id=" + id + " parentInfo=" + parentInfo);
			var linkEntry = this._addLinkStoreItemToModel(newitem);
			linkEntry.startId = this._getStartLinkItemId(newitem);
			linkEntry.endId = this._getEndLinkItemId(newitem);
			if (linkEntry.startId !== null && linkEntry.endId !== null) {
				linkEntry.start = this._model.getEntry(linkEntry.startId);
				linkEntry.end = this._model.getEntry(linkEntry.endId);
				if (linkEntry.start && linkEntry.end) {
					linkEntry.pending = false;
					linkEntry.link = this.connectNodes(linkEntry.start.node, linkEntry.end.node, null, newitem);
					this._applyLinkStoreItemGraphics(linkEntry.link);
				} else {
					linkEntry.pending = true;
				}
			}
		},
		
		_onDeleteLinkItem: function(item){
			// summary:
			//     called when an item is deleted from the linksStore.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onDeleteLinkItem id=" + id);
			var link = this._getLinkFromStoreItem(item);
			this._linkDispose(link, item);
		},
		
		_onSetLinkItem: function(item, attribute, oldValue, newValue){
			// summary:
			//     called when an item in linksStore changes. 
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSetLinkItem id=" + this._linksStore.getIdentity(item) + ",attribute=" + attribute + ",old=" + oldValue + ",new=" + newValue);
			
			if (this._internalSet) 
				return;
			
			var id = StoreApi.getIdentity(this._linksStore, item);
			var linkEntry = this._model.getLinkEntry(id);
			if (linkEntry) {
				var oldStartLinkItemId = linkEntry.startId;
				var oldEndLinkItemId = linkEntry.endId;
				var newStartLinkItemId = this._getStartLinkItemId(item);
				var newEndLinkItemId = this._getEndLinkItemId(item);
				
				if ((oldStartLinkItemId !== newStartLinkItemId) || (oldEndLinkItemId !== newEndLinkItemId)) {
					// The start and/or end point for this link has been changed
					this._linkDispose(linkEntry.link, item);
					linkEntry.link = null;
					
					if (oldStartLinkItemId !== newStartLinkItemId) {
						linkEntry.startId = newStartLinkItemId;
						linkEntry.start = this._model.getEntry(newStartLinkItemId);
					}
					if (oldEndLinkItemId !== newEndLinkItemId) {
						linkEntry.endId = newEndLinkItemId;
						linkEntry.end = this._model.getEntry(newEndLinkItemId);
					}
					
					if (linkEntry.start && linkEntry.end) {
						linkEntry.pending = false;
						linkEntry.link = this.connectNodes(linkEntry.start.node, linkEntry.end.node, null, item);
						this._applyLinkStoreItemGraphics(linkEntry.link);
					} else {
						linkEntry.pending = true;
					}
				}
			}
		},
		
		_onSaveLinksStore: function(){
			// summary:
			//     called after the linksStore has been saved.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onSaveLinksStore");
			this.reset();
			this._load();
		},
		
		_onRevertLinksStore: function(){
			// summary:
			//     called after the linksStore has reverted back to the last "saved" state.
			//     we simply request a complete redraw of the diagram.
			// tags:
			//	    private 
			
			// console.info("DataConnector._onRevertLinksStore");
			this.reset();
			this._load();
		},
		
		// a simple hashtable (ie {} ) for invalidateLinkShape connect handlers
		_onLinkShapeHandlers: null,
		
		_addLinkShapeHandler: function(link, handler){
			var h = connect.connect(link, 'invalidateLinkShape', this, handler);
			this._onLinkShapeHandlers[link.getId()] = h;
		},
		
		_clearLinkShapeHandlers: function(link){
			if (!this._onLinkShapeHandlers) 
				return;
			var id = link.getId(), h = this._onLinkShapeHandlers[id];
			if (h) 
				connect.disconnect(h);
			delete this._onNodeChangedHandlers[id];
		},
		
		// ===============================================================
		//  Function to extract/store graphical information in the Links 
		//  datastore itself
		// ===============================================================
		_applyLinkStoreItemGraphics: function(link){
			// summary:
			//     Applies any graphical information to a node/subgraph
			//     that is stored in the nodesStore item, specifically..
			//     (a) TBD from the specified function
			//     (b) x/y positioning.
			// tags:
			//	    private 
			
			if (link.getData) {
			
				var item = link.getData().item;
				
				if (this._linksGraphBindingFunction) {
					var gfxInfo = this._linksGraphBindingFunction.call(this, this._linksStore, item);
					if (gfxInfo) {
						this._applyProperties(link, gfxInfo);
					}
					
					// If store supports write then monitor link for changes
					if (this._linksStoreFeatures.canWrite) {
						this._addLinkShapeHandler(link, function(){
							if (this._linksGraphBindingFunction && link.getData()) {
								var litem = link.getData().item;
								if (litem && StoreApi.isItem(this._linksStore, litem)) {
									try {
										this._internalSet = true;
										var gfxInfo = this._extractProperties(link, this._linksGraphProperties);
										this._linksGraphBindingFunction.call(this, this._linksStore, litem, gfxInfo);
									} finally {
										this._internalSet = false;
									}
								}
							}
						});
					}
				}
			}
		},
		
		// ===============================================================
		// Graphics-properties-related functions
		// ===============================================================
		
		_getPropFuncMap: function(o, proplist){
			// summary: 
			//     For a particular list of properties return a map 
			//     of property names to function names
			// tags:
			//	    private         
			var funcMap = o._isIBMDiagramNode ? this._nodesGraphFuncMap : this._linksGraphFuncMap;
			if (!funcMap) {
				var props = proplist.split(",");
				funcMap = {};
				for (var i = 0; i < props.length; i++) {
					var getName = "get" + props[i].charAt(0).toUpperCase() + props[i].substring(1);
					if (o[getName] && (o[getName] instanceof Function)) {
						funcMap[props[i]] = getName;
					} else {
						var isName = "is" + props[i].charAt(0).toUpperCase() + props[i].substring(1);
						if (o[isName] && (o[isName] instanceof Function)) {
							funcMap[props[i]] = isName;
						}
					}
				}
				
				if (o._isIBMDiagramNode) 
					this._nodesGraphFuncMap = funcMap;
				else this._linksGraphFuncMap = funcMap;
			}
			return funcMap;
		},
		
		_extractProperties: function(o, props){
			// summary: 
			//     Extracts a subset of properties of an object, given a 
			//     list of getter functions. Function names are assumed to
			//     be of the form isXXX or getXXX
			// tags:
			//	    private 
			var extracted = {};
			
			var funcMap = this._getPropFuncMap(o, props);
			for (var pname in funcMap) {
				extracted[pname] = o[funcMap[pname]]();
			}
			return extracted;
		},
		
		_applyProperties: function(o, props){
			// summary: 
			//     Applies properties to an object. Setter functions are 
			//     determined, based on the property names.
			//     for property name xxxx, setter function is setXxxx.
			// tags:
			//	    private 
			for (var prop in props) {
				if (props.hasOwnProperty(prop)) {
					var setfunc = 'set' + prop.charAt(0).toUpperCase() + prop.substring(1);
					if (o[setfunc] && (o[setfunc] instanceof Function)) {
						o[setfunc](props[prop]);
					}
				}
			}
		},
		
		// ===============================================================
		// Model-related functions
		// ===============================================================
		
		_addStoreItemToModel: function(item){
			// summary: 
			//     Add a store item into the model.
			// tags:
			//	    private 
			
			var id = StoreApi.getIdentity(this._nodesStore, item);
			
			var entry = this._model.getEntry(id);
			
			if (!entry) {
				entry = this._model.addEntry(id);
			}
			this._model.setEntryItem(entry, item);
			if (StoreApi.fromDojoStore(this._nodesStore)) {
				this._backupNodeBindingValues(entry);
			}
			
			this._completeModelConstruction();
			
			return entry;
		},
		
		_addLinkStoreItemToModel: function(item){
			// summary: 
			//     Add a store item into the model.
			// tags:
			//	    private 
			
			var id = StoreApi.getIdentity(this._linksStore, item);
			
			var entry = this._model.getLinkEntry(id);
			
			if (!entry) {
				entry = this._model.addLinkEntry(id);
			}
			this._model.setLinkEntryItem(entry, item);
			return entry;
		},
		
		_backupNodeBindingValues: function(entry){
			this._backupBindingValues(this._nodesStore, entry, nodesBindingProperties)
		},
		
		_backupLinkBindingValues: function(entry){
			this._backupBindingValues(this._linksStore, entry, linksBindingProperties)
		},
		
		_backupBindingValues:function(store, entry, properties){
			entry._bindingBackup = {};
			array.forEach(properties, function(p){
				var v = this[p];
				if (typeof v !== 'undefined' && !(this[p] instanceof Function)) {
					entry._bindingBackup[p] = StoreApi.getValue(store, entry.item, v);
				}
			}, this);
		},
		_getChangedNodeBindingValues: function(entry, update){
		
			return this._getChangedBindingValues(this._nodesStore, entry, nodesBindingProperties, update);
		},
		
		_getChangedLinkBindingValues: function(entry, update){
			return this._getChangedBindingValues(this._linksStore, entry, linksBindingProperties, update);
		},
		
		_getChangedBindingValues: function(store, entry, properties, update){
			if (entry._bindingBackup) {
				var t = properties;
				for (var i = 0; i < t.length; ++i) {
					var v = this[t[i]], val;
					if (typeof v !== 'undefined' &&
						!(this[t[i]] instanceof Function) &&
						entry._bindingBackup[t[i]] !== (val = StoreApi.getValue(store, entry.item, v))) {
						var result = {
							p: t[i], // changed attribute
							o: entry._bindingBackup[t[i]], // old val
							n: val //new val
						};
						if (update) {
							entry._bindingBackup[t[i]] = val;
						}
						return result;
					}
				};
			}
		},
		
		_completeModelConstruction: function(){
			// summary: 
			//     Called after the all model entries have been created
			// tags:
			//	    private 
			
			// When all model entries have been creates and their items loaded..            
			if (this._model.isModelComplete()) {
				//                
				// Assumptions made for hierarchies
				// - An entry with an empty to-list is a leaf node 
				// - The from-list length will by not longer than 1
				
				this._model.forEachEntry(function(entry){
					var childlist = entry.childlist;
					var parentlist = entry.parentlist;
					if (childlist.length > 0) {
					
						for (var t = 0; t < childlist.length; t++) {
							var childEntry = this._model.getEntry(childlist[t]);
							childEntry.parent = entry;
							entry.children.push(childEntry);
						}
					}
					if (parentlist.length === 1) {
						var parentEntry = this._model.getEntry(parentlist[0]);
						entry.parent = parentEntry;
						parentEntry.children.push(entry);
					}
				}, this);
				
				this._onModelCreated(this._model);
				this._createShapesFromModel();
			}
		},
		
		_onModelCreated: function(/* ibm_ilog.diagram.widget._DataModel */model){
			// summary: 
			//     Function which is called when the model is complete.
			// tags:
			//	    private 
		
			// model.dumpToConsole();
		},
		
		_resolveConnectionsFromNodesStore: function(){
			// summary: 
			//     Attempt to resolve (discover relationship information for) 
			//     nodesStore items. Add information about the item relationships
			//     into the model. Repeat until all are resolved.
			//     Note that item-connecting information can be provided either
			//     by registered binding functions or from the link store.
			// tags:
			//	    private        
			var unresolved = [];
			
			this._model.forEachUnresolvedEntry(function(entry){
			
				var currentItem = entry.item;
				
				if (this._parentBindingFunction) {
					var connectingItems = this._parentBindingFunction.call(this, this._nodesStore, currentItem);
					array.forEach(connectingItems, function(connectingItem){
						var ident = StoreApi.getIdentity(this._nodesStore, connectingItem);
						
						if (this._model.containsEntry(ident)) {
							entry.parentlist.push(ident);
						}
					}, this);
				}
				
				if (this._childBindingFunction) {
					var connectingItems = this._childBindingFunction.call(this, this._nodesStore, currentItem);
					array.forEach(connectingItems, function(connectingItem){
						var ident = StoreApi.getIdentity(this._nodesStore, connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.childlist.push(ident);
							
						}
					}, this);
				}
				
				if (this._successorsBindingFunction) {
					var connectingItems = this._successorsBindingFunction.call(this, this._nodesStore, currentItem);
					array.forEach(connectingItems, function(connectingItem){
						var ident = StoreApi.getIdentity(this._nodesStore, connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.tolist.push(ident);
						}
					}, this);
				}
				
				if (this._predecessorsBindingFunction) {
					var connectingItems = this._predecessorsBindingFunction.call(this, this._nodesStore, currentItem);
					array.forEach(connectingItems, function(connectingItem){
						var ident = StoreApi.getIdentity(this._nodesStore, connectingItem);
						if (this._model.containsEntry(ident)) {
							entry.fromlist.push(ident);
						}
					}, this);
				}
				
				entry.resolved = true;
			}, this);
			
			// If all items are resolved and the model is fully populated, otherwise add 
			// new entries in the model 
			if (unresolved.length === 0) {
				this._model.setAllEntriesAdded();
				this._completeModelConstruction();
			} else {
				for (var i = 0; i < unresolved.length; i++) {
					if (StoreApi.isItemLoaded(this._nodesStore, unresolved[i])) {
						this._addStoreItemToModel(unresolved[i]);
					} else {
						this._loadNodesStoreItem(unresolved[i]);
					}
				}
				this._resolveConnectionsFromNodesStore();
			}
		},
		
		// ===============================================================
		// Shape mapping functions
		// ===============================================================
		
		_getNodeFromStoreItem: function(item){
			// summary:
			//        Gets the GFX object, for the given Node Data Item.
			// tags:
			//	    private 
			
			var node = null;
			var id = StoreApi.getIdentity(this._nodesStore, item);
			var nodeModelEntry = this._model.getEntry(id);
			if (nodeModelEntry) {
				node = nodeModelEntry.node;
			}
			return node;
		},
		
		_setNodeFromStoreItem: function(item, node){
			// summary:
			//        Sets the GFX object, associating it with the given Node Data Item.
			// tags:
			//	    private 
			
			var id = StoreApi.getIdentity(this._nodesStore, item);
			var nodeModelEntry = this._model.getEntry(id);
			if (nodeModelEntry) {
				nodeModelEntry.node = node;
			}
		},
		
		
		_getLinkFromStoreItem: function(item){
			// summary:
			//        Gets the GFX object, for the given Link Data Item.
			// tags:
			//	    private 
			var link = null;
			var id = StoreApi.getIdentity(this._linksStore, item);
			var linkModelEntry = this._model.getLinkEntry(id);
			if (linkModelEntry) {
				link = linkModelEntry.link;
			}
			return link;
		},
		
		// ===============================================================
		// Nodes, Subgraphs and Links creation
		// ===============================================================
		_createLinksFromModel: function(){
			// summary: 
			//     Based on options provided, create explicit links objects
			//     associated with the nodeStore items and their
			//     relationships.
			// tags:
			//	    private 
			
			// Create links from lists in the node model 
			this._model.forEachEntry(function(entry){
				var currentNode;
				var childlist = entry.childlist;
				var parentlist = entry.parentlist;
				var tolist = entry.tolist;
				var fromlist = entry.fromlist;
				if (this._createLinksForHierarchy) {
				
					if (childlist.length + parentlist.length > 0) {
						currentNode = entry.node;
						var link;
						for (var t = 0; t < childlist.length; t++) {
							var childNode = this._model.getEntry(childlist[t]).node;
							link = this.connectNodes(currentNode, childNode);
						}
						for (var f = 0; f < parentlist.length; f++) {
							var parentNode = this._model.getEntry(parentlist[f]).node;
							link = this.connectNodes(parentNode, currentNode);
						}
					}
				}
				if (tolist.length + fromlist.length > 0) {
					currentNode = entry.node;
					var link;
					for (var t = 0; t < tolist.length; t++) {
						var toNode = this._model.getEntry(tolist[t]).node;
						link = this.connectNodes(currentNode, toNode);
					}
					for (var f = 0; f < fromlist.length; f++) {
						var fromNode = this._model.getEntry(fromlist[f]).node;
						link = this.connectNodes(fromNode, currentNode);
					}
				}
			}, this);
			
			// Create links from lists in the link model 
			this._model.forEachCompleteLinkEntry(function(entry){
				entry.link = this.connectNodes(entry.start.node, entry.end.node, null, entry.item);
				this._applyLinkStoreItemGraphics(entry.link);
			}, this);
			
		},
		
		_createShapesFromModel: function(){
			// summary: 
			//     Based on options provided, create graphical objects
			//     associated with the nodeStore items and their
			//     relationships.
			// tags:
			//	    private 
			if (this._createLinksForHierarchy) {
				// Create nodes
				this._model.forEachEntry(function(entry){
					entry.node = this.createNode(null, entry.item);
					this._applyNodeStoreItemGraphics(entry.node);
				}, this);
				this._createLinksFromModel();
			} else {
				// Create subgraphs/nodes
				this._model.forEachTopEntry(function(entry){
					if (entry.isLeaf()) {
						entry.node = this.createNode(null, entry.item);
						this._applyNodeStoreItemGraphics(entry.node);
					} else {
						entry.node = this.createSubgraph(null, entry.item);
						this.createChildGraphics(entry);
						this._applyNodeStoreItemGraphics(entry.node);
					}
				}, this);
				this._createLinksFromModel();
			}
			
			this.onLoaded();
		},
		
		createChildGraphics: function(parent){
			// summary: 
			//     For hierarchical data, recursively go through the tree
			//     and create subgraphs and nodes accordingly
			// tags:
			//	    private         
			var children = parent.children;
			for (var i = 0; i < children.length; i++) {
				var child = children[i];
				if (child.isLeaf()) {
					child.node = this.createNode(null, child.item, parent.node.getGraph());
					this._applyNodeStoreItemGraphics(child.node);
				} else {
					child.node = this.createSubgraph(null, child.item, parent.node.getGraph());
				}
				
				this.createChildGraphics(child);
			}
			this._applyNodeStoreItemGraphics(parent.node);
		},
		
		// ===============================================================
		// Nodes and Links disposal
		// ===============================================================
		_nodeDispose: function(node, item){
			// summary:
			//     dispose of a node. 
			// tags:
			//	    private 
			
			if (node) {
				this._clearOnChangedHandlers(node);
				node.removeShape();
				node.dispose();
			}
		},
		
		_linkDispose: function(link, item){
			// summary:
			//     dispose of a link. 
			// tags:
			//	    private 
			
			if (link) {
				this._clearLinkShapeHandlers(link);
				link.removeShape();
				link.dispose();
			}
		}
		
	});
	
	//
	//	ibm_ilog.diagram.widget.DataConnector.StartLinkFieldname: String
	//		The name of a field in a linksStore item which defines the start node of a link
	//      The value associated with this field is a String which represents the 
	//      identity of an item in the nodesStore.
	//
	DataConnector.StartLinkFieldname = "start";
	
	//
	//	ibm_ilog.diagram.widget.DataConnector.EndLinkFieldname: String
	//		The name of a field in a linksStore item which defines the end node of  a link
	//      The value associated with this field is a String which represents the 
	//      identity of an item in the nodesStore.
	//
	DataConnector.EndLinkFieldname = "end";
	
	var _DataModel = declare('ibm_ilog.diagram.widget._DataModel', null, {
		//
		//	summary:
		//
		//		A _DataModel is a collection of _NodeModelEntry items.
		//
		//	description:
		//		
		//		The _DataModel is a half-way house between the store and canvas.
		// tags:
		//	    private 
		
		// nmodel : dojox.collections.Dictionary
		//          A collection of entries, each representing a nodesStore item, associated 
		//          graphical objects and its relationships between other nodes
		//          It is not considered complete until all entries have been added
		//          and the item for each entry has loaded, I.e. no pending loads  
		nmodel: null,
		
		// lmodel : dojox.collections.Dictionary
		//          A collection of entries, each representing a linksStore item, associated 
		//          graphical objects  
		lmodel: null,
		
		// pendingLoads : Integer
		//                Number of model entries which don't have associated items 
		pendingLoads: 0,
		
		// allEntriesAdded : Boolean
		//                When true, all datastores have been parsed and model entries
		//                have been created.
		
		allEntriesAdded: false,
		
		// initialLoadComplete : Boolean
		//                When true, it means all items loaded and associated graphics
		//                have been created.
		
		initialLoadComplete: false,
		
		constructor: function(){
			// summary:
			//     Creates an instance of a data model
			//
			// tags:
			//	    private 
			this.nmodel = new Dictionary();
			this.lmodel = new Dictionary();
			this.init();
		},
		
		init: function(){
			// tags:
			//	    private 
			
			this.initNodes();
			this.initLinks();
			
			
			this.pendingLoads = 0;
			
			
		},
		
		initNodes: function(){
			// tags:
			//	    private 
			array.forEach(this.nmodel.getKeyList(), function(key){
				this.nmodel.remove(key);
			}, this);
			this.nmodel.clear();
			this.allEntriesAdded = false;
			this.initialLoadComplete = false;
		},
		
		initLinks: function(){
			// tags:
			//	    private 
			array.forEach(this.lmodel.getKeyList(), function(key){
				this.lmodel.remove(key);
			}, this);
			this.lmodel.clear();
		},
		
		isModelComplete: function(){
			// summary:
			//     Check if the model is "complete" - I.e. all entries have been added
			//     and each entry has an associated item. 
			//     We only want this to return true for the first call.
			// tags:
			//	    private 
			var isComplete = (this.pendingLoads === 0 && this.allEntriesAdded && !this.initialLoadComplete);
			if (isComplete) {
				this.initialLoadComplete = true;
			}
			return isComplete;
		},
		
		setAllEntriesAdded: function(){
			// summary:
			//     Indicate to the model that all entries have been added.
			// tags:
			//	    private 
			this.allEntriesAdded = true;
		},
		
		incrPendingLoads: function(){
			// summary:
			//     Increase the count of item loads pending.
			// tags:
			//	    private 
			this.pendingLoads++;
		},
		
		decrPendingLoads: function(){
			// summary:
			//     Decrease the count of item loads pending.
			// tags:
			//	    private            
			this.pendingLoads--;
		},
		
		clear: function(){
			// summary:
			//     Clears the data model
			// tags:
			//	    private 
			this.init();
		},
		
		getEntry: function(id){
			// summary:
			//     Get an entry, based on the nodeStore Identity
			// tags:
			//	    private         
			return this.nmodel.item(id);
		},
		
		getLinkEntry: function(id){
			// summary:
			//     Get an entry, based on the linksStore Identity
			// tags:
			//	    private         
			return this.lmodel.item(id);
		},
		
		containsEntry: function(id){
			// summary:
			//     Enquire if there is an entry in the data model which relates
			//     to a specified nodeStore Identity
			// tags:
			//	    private  
			return this.nmodel.containsKey(id);
		},
		
		addEntry: function(id){
			// summary:
			//     Creates a _NodeModelEntry, associates it with a specified
			//     nodeStore Identity then adds it to the model
			// tags:
			//	    private    
			var entry = new _NodeModelEntry();
			
			this.nmodel.add(id, entry);
			entry.id = id;
			
			// Track the number of item loads pending
			this.incrPendingLoads();
			return entry;
		},
		
		removeEntry: function(id){
			// summary:
			//     Removes the _NodeModelEntry, associated with a specified
			//     nodeStore Identity, from model
			// tags:
			//	    private    
			var entry = this.getEntry(id);
			this.nmodel.remove(id);
		},
		
		addLinkEntry: function(id){
			// summary:
			//     Creates a _LinkModelEntry, associates it with a specified
			//     linkStore Identity then adds it to the model
			// tags:
			//	    private    
			var entry = new _LinkModelEntry();
			
			this.lmodel.add(id, entry);
			entry.id = id;
			return entry;
		},
		
		removeLinkEntry: function(id){
			// summary:
			//     Removes the _LinkModelEntry, associated with a specified
			//     linkStore Identity, from model
			// tags:
			//	    private    
			var entry = this.getLinkEntry(id);
			this.lmodel.remove(id);
		},
		
		
		
		setEntryItem: function(entry, item){
			// summary:
			//     Associates and entry with a datastore item
			//     and tracks the number of item loads pending.   
			//     IT IS VERY IMPORTANT THAT setEntryItem (rather than entry.item = item;)
			//     IS USED SO THE MODEL CAN KEEP TRACK OF WHICH ENTRIES DON'T HAVE ITEMS
			// tags:
			//	    private         
			entry.item = item;
			this.decrPendingLoads();
		},
		
		setLinkEntryItem: function(entry, item){
			// summary:
			//     Associates and entry with a datastore item
			// tags:
			//	    private     
			entry.item = item;
		},
		
		forEachLinkEntry: function(func, scope){
			// summary:
			//     Loops through each link entry in the model, calling a named function, in
			//     the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)"
			// tags:
			//	    private     
			var e = this.lmodel.getIterator();
			while (e.get()) {
				func.call(scope, e.element.value);
			}
		},
		
		forEachCompleteLinkEntry: function(func, scope){
			// summary:
			//     Loops through each link entry in the model which is not 'pending', calling
			//     a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)".
			// tags:
			//	    private     
			var e = this.lmodel.getIterator();
			while (e.get()) {
				if (!e.element.value.pending) {
					func.call(scope, e.element.value);
				}
			}
		},
		
		forEachPendingLinkEntry: function(func, nodeEntry, scope){
			// summary:
			//     Loops through each link entry in the model which is 'pending', calling
			//     a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)".
			// tags:
			//	    private     
			var e = this.lmodel.getIterator();
			while (e.get()) {
				if (e.element.value.pending) {
					func.call(scope, e.element.value, nodeEntry);
				}
			}
		},
		
		forEachEntry: function(func, scope){
			// summary:
			//     Loops through each entry in the model, calling a named function, in
			//     the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)"
			// tags:
			//	    private     
			var e = this.nmodel.getIterator();
			while (e.get()) {
				func.call(scope, e.element.value);
			}
		},
		
		forEachTopEntry: function(func, scope){
			// summary:
			//     Loops through each entry in the model which have no parents, 
			//     calling a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)"
			// tags:
			//	    private     
			var e = this.nmodel.getIterator();
			while (e.get()) {
				if (!e.element.value.parent) {
					func.call(scope, e.element.value);
				}
			}
		},
		
		forEachUnresolvedEntry: function(func, scope){
			// summary:
			//     Loops through each entry in the model which is not marked as resolved, calling
			//     a named function, in the context of the specified scope.
			//     The prototype of the function must be of the form "funct(entry)".
			// tags:
			//	    private     
			var e = this.nmodel.getIterator();
			while (e.get()) {
				if (!e.element.value.resolved) {
					func.call(scope, e.element.value);
				}
			}
		},
		
		dumpToConsole: function(){
			// summary:
			//     Debug facility to dump the contents of the data model to the console.
			// tags:
			//	    private 
			
			// console.info("Model dump =================================Nodes and their relationships");
			
			this.forEachEntry(function(entry){
				var leafStatus = entry.isLeaf() ? "(+leaf)" : "";
				var itemStatus = entry.item ? "(+item)" : "";
				var resolvedStatus = entry.resolved ? "(+resolved)" : "";
				// console.info("Item - " + entry.id + " " + itemStatus + " " + resolvedStatus + " " + leafStatus);
				var childlist = entry.childlist;
				for (var t = 0; t < childlist.length; t++) {
					// console.info("       Children : " + childlist[t]);
				}
				var parentlist = entry.parentlist;
				for (var f = 0; f < parentlist.length; f++) {
					// console.info("       Parent   : " + parentlist[f]);
				}
				
				var tolist = entry.tolist;
				for (var t = 0; t < tolist.length; t++) {
					// console.info("       To       : " + tolist[t]);
				}
				var fromlist = entry.fromlist;
				for (var f = 0; f < fromlist.length; f++) {
					// console.info("       From     : " + fromlist[f]);
				}
			}, this);
			
			// console.info("Model dump =================================Links");
			this.forEachLinkEntry(function(entry){
				// console.info("Item - " + entry.id);
			}, this);
		}
	});
	
	var _NodeModelEntry = declare('ibm_ilog.diagram.widget._NodeModelEntry', null, {
		//
		//	summary:
		//
		//		A DataModelEntry collects attributes related to a nodesStore item.
		//
		//	description:
		//		
		//		When completely populated, it forms a logical representation of a
		//      nodeStore item and its relationship with other items.
		
		id: "",
		item: null,
		node: null,
		resolved: false,
		childlist: [],
		parentlist: [],
		tolist: [],
		fromlist: [],
		parent: null,
		children: [],
		
		
		constructor: function(){
			// summary:
			//     Creates an instance of a data model entry.
			// tags:
			//	    private 
			this.id = "";
			this.item = null;
			this.node = null;
			this.resolved = false;
			this.childlist = [];
			this.parentlist = [];
			this.fromlist = [];
			this.tolist = [];
			this.parent = null;
			this.children = [];
		},
		
		isLeaf: function(){
			return (this.children.length === 0);
		}
		
	});
	
	var _LinkModelEntry = declare('ibm_ilog.diagram.widget._LinkModelEntry', null, {
		//
		//	summary:
		//
		//		A DataModelEntry collects attributes related to a linksStore item.
		//
		//	description:
		//		
		//		When completely populated, it forms a logical representation of a
		//      nodeStore item and its relationship with other items.
		
		id: "",
		item: null,
		link: null,
		startId: null,
		start: null,
		endId: null,
		end: null,
		pending: false,
		
		constructor: function(){
			// summary:
			//     Creates an instance of a data model entry.
			// tags:
			//	    private 
			this.id = "";
			this.item = null;
			this.link = null;
			this.startId = null;
			this.start = null;
			this.endId = null;
			this.end = null;
			this.pending = false;
		}
		
	});
	
	return DataConnector;
});

},
'ibm_ilog/diagram/templates/editor_templates':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/templates/editor_templates", [
"dojo/_base/lang",
"dojox/collections/Dictionary",
"dijit/form/Textarea",
"../data/dtlbinding"
],
function(lang, Dictionary, Textarea, dtlbinding){

	// TODO: Change to this when AMD conversion is complete:
	// var t = {};
	var t = lang.getObject("ibm_ilog.diagram.templates", true);
	
    t.defaultResizeAdornerTemplate = [{
        dojoAttachPoint: 'adornerRoot',
        transform: {
            dx: $bind("{{elemBaseLeft}}"),
            dy: $bind("{{elemBaseTop}}")
        },
        children: [{
            dojoAttachPoint: 'hndTL',
            transform: {
                dx: -7,
                dy: -7,
                xx: 1,
                yy: 1
            }
        }, {
            dojoAttachPoint: 'hndTR',
            transform: {
                dx: $bind("{{elemBaseWidth}}+7"),
                dy: -7,
                xx: -1,
                yy: 1
            }
        }, {
            dojoAttachPoint: 'hndBL',
            transform: {
                dx: -7,
                dy: $bind("{{elemBaseHeight}}+7"),
                xx: 1,
                yy: -1
            }
        }, {
            dojoAttachPoint: 'hndBR',
            transform: {
                dx: $bind("{{elemBaseWidth}}+7"),
                dy: $bind("{{elemBaseHeight}}+7"),
                xx: -1,
                yy: -1
            }
        }]
    }];

    t.defaultResizeHandleTemplate = [{
        dojoAttachPoint: 'stNormal',
        children: [{
            shape: {
                type: 'rect',
                x: 0,
                y: 0,
                width: 10,
                height: 10
            },
            fill: [0, 0, 0, 0.0001]
        }, {
            shape: {
                type: 'line',
                x1: 0,
                y1: 0,
                x2: 10,
                y2: 0
            },
            stroke: {
                color: [0, 120, 255, 0.5],
                width: 2
            }
        }, {
            shape: {
                type: 'line',
                x1: 0,
                y1: 0,
                x2: 0,
                y2: 10
            },
            stroke: {
                color: [0, 120, 255, 0.5],
                width: 2
            }
        }]
    }, {
        dojoAttachPoint: 'stHighlighted',
        children: [{
            shape: {
                type: 'rect',
                x: 0,
                y: 0,
                width: 10,
                height: 10
            },
            fill: [0, 0, 0, 0.0001]
        }, {
            shape: {
                type: 'line',
                x1: 0,
                y1: 0,
                x2: 10,
                y2: 0
            },
            stroke: {
                color: [0, 120, 255],
                width: 2
            }
        }, {
            shape: {
                type: 'line',
                x1: 0,
                y1: 0,
                x2: 0,
                y2: 10
            },
            stroke: {
                color: [0, 120, 255],
                width: 2
            }
        }]
    }, {
        dojoAttachPoint: 'stActive',
        children: [{
            shape: {
                type: 'line',
                x1: 0,
                y1: 0,
                x2: 10,
                y2: 0
            },
            stroke: {
                color: [255, 0, 120, 0.3],
                width: 2
            }
        }, {
            shape: {
                type: 'line',
                x1: 0,
                y1: 0,
                x2: 0,
                y2: 10
            },
            stroke: {
                color: [255, 0, 120, 0.3],
                width: 2
            }
        }]
    }];
    t.defaultResizeHandleTemplate.noDtl = true;

    t.defaultConnectionAdornerTemplate = [{
        dojoAttachPoint: 'adornerRoot',
        transform: {
            dx: $bind("{{elemBaseLeft}}"),
            dy: $bind("{{elemBaseTop}}")
        },
        children: [{
            dojoAttachPoint: 'hndT',
            transform: {
                dx: $bind("{{elemBaseCenterX}}-10"),
                dy: 5,
                xx: 1,
                yy: -1
            }
        }, {
            dojoAttachPoint: 'hndR',
            transform: {
                dx: $bind("{{elemBaseWidth}}-5"),
                dy: $bind("{{elemBaseCenterY}}+10"),
                xx: 0,
                xy: 1,
                yx: -1,
                yy: 0
            }
        }, {
            dojoAttachPoint: 'hndB',
            transform: {
                dx: $bind("{{elemBaseCenterX}}-10"),
                dy: $bind("{{elemBaseHeight}}-5"),
                xx: 1,
                yy: 1
            }
        }, {
            dojoAttachPoint: 'hndL',
            transform: {
                dx: 5,
                dy: $bind("{{elemBaseCenterY}}-10"),
                xx: 0,
                xy: -1,
                yx: 1,
                yy: 0
            }
        }]
    }];

    t.defaultConnectionHandleTemplate = [{
        dojoAttachPoint: 'stNormal',
        children: [{
            shape: {
                type: 'path',
                path: 'M 1,10 L 19,10 L 10,18 z e'
            },
            fill: [0, 120, 255, 0.5]
        }, {
            shape: {
                type: 'path',
                path: 'M 0,10 L 20,10 L 15,0 L 5,0 z e'
            },
            fill: [255, 255, 255, 0.001]
        }]
    }, {
        dojoAttachPoint: 'stHighlighted',
        children: [{
            shape: {
                type: 'path',
                path: 'M 1,10 L 19,10 L 10,18 z e'
            },
            fill: [0, 120, 255, 1]
        }, {
            shape: {
                type: 'circle',
                cx: 10,
                cy: 10,
                r: 10
            },
            fill: [0, 120, 255, 0.001]
        }, {
            shape: {
                type: 'path',
                path: 'M 0,10 L 20,10 L 15,0 L 5,0 z e'
            },
            fill: [255, 255, 255, 0.001]
        }]
    }, {
        dojoAttachPoint: 'stActive',
        children: [{
            shape: {
                type: 'path',
                path: 'M 1,10 L 19,10 L 10,18 z e'
            },
            fill: [255, 0, 120, 0.3]
        }]
    }];
    t.defaultConnectionHandleTemplate.noDtl = true;
    
    t.defaultConnectionTargetHandleTemplate = [{
        dojoAttachPoint: 'stNormal',
        children: [{
            shape: {
                type: 'path',
                path: 'M 1,10 L 19,10 L 10,18 z e'
            },
            fill: [0, 120, 255, 0.5]
        }, {
            shape: {
                type: 'circle',
                cx: 10,
                cy: 10,
                r: 10
            },
            fill: [255, 255, 255, 0.001]
        }]
    }, {
        dojoAttachPoint: 'stHighlighted',
        children: [{
            shape: {
                type: 'path',
                path: 'M 1,10 L 19,10 L 10,18 z e'
            },
            fill: [255, 0, 120, 0.3]
        }, {
            shape: {
                type: 'circle',
                cx: 10,
                cy: 10,
                r: 10
            },
            fill: [255, 0, 120, 0.001]
        }]
    }, {
        dojoAttachPoint: 'stActive',
        children: [{
            shape: {
                type: 'path',
                path: 'M 1,10 L 19,10 L 10,18 z e'
            },
            fill: [255, 0, 120, 0.3]
        }]
    }];
    t.defaultConnectionTargetHandleTemplate.noDtl = true;

    t.defaultLinkConnectionAdornerTemplate = [{
        dojoAttachPoint: 'adornerRoot',
        children: [{
            dojoAttachPoint: '_path',
            shape: {
                type: 'polyline',
                points: [{
                    x: 0,
                    y: 0
                }, {
                    x: 0,
                    y: 0
                }]
            },
            stroke: {
                width: 6,
                color: [255, 0, 255, 0.15]
            }
        },{
            dojoAttachPoint: 'start',
            transform: {
                dx: $bind("{{linkStartX}}"),
                dy: $bind("{{linkStartY}}")
            }
        }, {
            dojoAttachPoint: 'end',
            transform: {
                dx: $bind("{{linkEndX}}"),
                dy: $bind("{{linkEndY}}")
            }
        }]
    }];

    t.defaultLinkConnectionHandleTemplate = [{
        dojoAttachPoint: 'stNormal',
        children: [{
            shape: {
                type: 'circle',
                cx: 0,
                cy: 0,
                r: 5
            },
            fill: [0, 120, 255, 0.3]
        }]
    }, {
        dojoAttachPoint: 'stHighlighted',
        children: [{
            shape: {
                type: 'circle',
                cx: 0,
                cy: 0,
                r: 5
            },
            fill: [0, 120, 255, 0.7]
        }]
    }, {
        dojoAttachPoint: 'stActive',
        children: [{
            shape: {
                type: 'circle',
                cx: 0,
                cy: 0,
                r: 5
            },
            fill: [255, 0, 255, 0.5]
        }]
    }];
    t.defaultLinkConnectionHandleTemplate.noDtl = true;

    t.defaultTextEditAdornerTemplate = [{
        dojoAttachPoint: 'textbox',
        transform: {
            dx: $bind("{{compLeft}}-2"),
            dy: $bind("{{compTop}}-2")
        },
        height: $bind("{{compHeight}}+4"),
        width: $bind("{{compWidth}}+4"),
        fontPxSize: $bind("{{refFontPxSize}}"),
        fontMinPxSize: 12,
        minWidth: 40,
        shape: {
            type: 'widget',
            dojoType: 'dijit.form.Textarea',
            tabIndex: -1,
            scrollOnFocus: 0,
            role: 'presentation',
            style: 'text-align:center;font-size:10pt;font-family:sans-serif'
        }
    }];

    t.defaultLinkIntermediatePointsAdornerTemplate = [{
        dojoAttachPoint: 'adornerRoot',
        children: [{
            dojoAttachPoint: '_path',
            shape: {
                type: 'polyline',
                points: [{
                    x: 0,
                    y: 0
                }, {
                    x: 0,
                    y: 0
                }]
            },
            stroke: {
                width: 7,
                color: [255, 0, 150, 0.1]
            }
        }, {
            dojoAttachPoint: 'currentPointGroup',
            children: []
        }, {
            dojoAttachPoint: 'addPointGroup',
            children: []
        }]
    }];
    t.defaultLinkIntermediatePointsAdornerTemplate.noDtl = true;
    
    t.defaultLinkIntermediatePointsHandleTemplate = [{
        dojoAttachPoint: 'stNormal',
        transform: {
            dx: $bind("{{intPointPosX}}"),
            dy: $bind("{{intPointPosY}}")
        },
        children: [{
            shape: {
                type: 'path',
                path: 'M -7,0 L 0,-7 L 7,0 L 0,7 z e'
            },
            fill: [0, 120, 255, 0.3]
        }]
    }, {
        dojoAttachPoint: 'stHighlighted',
        transform: {
            dx: $bind("{{intPointPosX}}"),
            dy: $bind("{{intPointPosY}}")
        },
        children: [{
            shape: {
                type: 'path',
                path: 'M -7,0 L 0,-7 L 7,0 L 0,7 z e'
            },
            fill: [0, 120, 255, 0.7]
        }]
    }, {
        dojoAttachPoint: 'stActive',
        transform: {
            dx: $bind("{{intPointPosX}}"),
            dy: $bind("{{intPointPosY}}")
        },
        children: [{
            shape: {
                type: 'path',
                path: 'M -7,0 L 0,-7 L 7,0 L 0,7 z e'
            },
            fill: [255, 0, 255, 0.5]
        }]
    }];
    
     t.defaultLinkIntermediatePointsAddHandleTemplate = [{
        dojoAttachPoint: 'stNormal',
        transform: {
            dx: $bind("{{addPointPosX}}"),
            dy: $bind("{{addPointPosY}}")
        },
        children: [{
       	 	shape: {
        		type: 'circle',
        		cx: 0,
        		cy: 0,
        		r: 6
    		},
            fill: [0, 120, 255, 0.5]
        },{
            shape: {
            type: 'path',
            path: 'M 0,-4 L 0,4 M -4,0 L 4,0 e'
        	},
        	stroke: {
                width: 2,
                color: [255, 255, 255, 1]
            }
        }]
    }, {
        dojoAttachPoint: 'stHighlighted',
        transform: {
            dx: $bind("{{addPointPosX}}"),
            dy: $bind("{{addPointPosY}}")
        },
        children: [{
       	 	shape: {
        		type: 'circle',
        		cx: 0,
        		cy: 0,
        		r: 6
    		},
            fill: [0, 120, 255, 0.9]
        },{
            shape: {
            type: 'path',
            path: 'M 0,-4 L 0,4 M -4,0 L 4,0 e'
        	},
        	stroke: {
                width: 2,
                color: [255, 255, 255, 1]
            }
        }]
    }, {
        dojoAttachPoint: 'stActive',
        transform: {
            dx: $bind("{{addPointPosX}}"),
            dy: $bind("{{addPointPosY}}")
        },
        children: [{
        	 shape: {
            	type: 'circle',
            	cx: 0,
            	cy: 0,
            	r: 6
        	},
            fill: [255, 0, 255, 0.7]
        },{
            shape: {
            type: 'path',
            path: 'M 0,-4 L 0,4 M -4,0 L 4,0 e'
        	},
        	stroke: {
                width: 2,
                color: [255, 255, 255, 1]
            }
        }]
    }];
    
    t.defaultAdornerTemplates = t.defaultAdornerTemplates || new Dictionary();
    
    t.defaultAdornerTemplates.add("TextEditAdorner", t.defaultTextEditAdornerTemplate);
    t.defaultAdornerTemplates.add("ResizeAdorner", t.defaultResizeAdornerTemplate);
    t.defaultAdornerTemplates.add("ResizeHandle", t.defaultResizeHandleTemplate);
    t.defaultAdornerTemplates.add("ConnectionHandle", t.defaultConnectionHandleTemplate);
    t.defaultAdornerTemplates.add("ConnectionAdorner", t.defaultConnectionAdornerTemplate);
    t.defaultAdornerTemplates.add("ConnectionTargetHandle", t.defaultConnectionTargetHandleTemplate);
    t.defaultAdornerTemplates.add("ConnectionTargetAdorner", t.defaultConnectionAdornerTemplate);
    
    t.defaultAdornerTemplates.add("LinkConnectionAdorner", t.defaultLinkConnectionAdornerTemplate);
    t.defaultAdornerTemplates.add("LinkConnectionHandle", t.defaultLinkConnectionHandleTemplate);
    
    t.defaultAdornerTemplates.add("LinkIntermediatePointsHandle", t.defaultLinkIntermediatePointsHandleTemplate);
    t.defaultAdornerTemplates.add("LinkIntermediatePointsAdorner", t.defaultLinkIntermediatePointsAdornerTemplate);
    t.defaultAdornerTemplates.add("LinkIntermediatePointsAddHandle",t.defaultLinkIntermediatePointsAddHandleTemplate);
	
	return t;
});

},
'ibm_ilog/diagram/interactors/MarqueeSelectionInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/MarqueeSelectionInteractor", [
"dojo/_base/declare",
"dojo/_base/event",
"dojo/_base/array",
"dojo/_base/json",
"dojox/collections/ArrayList",
"./Interactor",
"./DragInteractor",
"../Selection",
"../Selectable",
"../util/Geometry"
],
function(
declare,
event,
arr,
json,
ArrayList,
Interactor,
DragInteractor,
Selection,
Selectable,
g
){

var superclass = DragInteractor.prototype;
	
/*=====
var DragInteractor = ibm_ilog.diagram.interactors.DragInteractor;
=====*/

var MarqueeSelectionInteractor =
declare("ibm_ilog.diagram.interactors.MarqueeSelectionInteractor", [DragInteractor], {
	//summary:
	//		This function provides the functionality to manage marquee selection events on links and nodes.
	_selection: null,
	_background: null,
	_marqueePointerStart: null,
	_marquee:null,
	_marqueeStyle: json.fromJson("{'fill':'rgba(0, 0, 255, 0.1)' , 'stroke':{'width': '1', 'color':'#0000FF'}}"),
	_marqueeMode: null,
	_partial: null,
	_mustClear: null,
	_declareStates: function() {
	// Summary:
	//		Add the crtl start to the original states
	this.inherited(arguments);	
		this._declareState("idle",["start","startCtrl"]);
	},
	
	getDefaultConnections:function(){
		//
		//	summary:
		//		returns the default event mapping
		//
    		var map = this.inherited(arguments);
    		map.startCtrl = {src:this._getInitialEventSource(),srcEvt:"onmousedown",connectTo:"_dragPreStart",filter:this._buildInputFilter({button:0,ctrl:true}),gfxConnect:true};
    		return map;
    },
    
	initialize: function(viewport,selection) {
		// summary:
		//		creates a new instance, sets the viewport appropriate elements from where to connect and set the default event mapping.
		//		If the given root is not a viewport, the standard selection is activated
		if(selection==null){	
			this._selection = new Selection();
		}else{
			this._selection = selection; 
		}
		this._background = viewport.getEventSourceBackground();
		this._marqueeMode = MarqueeSelectionInteractor.topParent;
		this._partial = true;
		this._viewport = viewport;
		this._initialize();
		return this;
	},
	
	_getInitialEventSource: function() {
		return this._background;
	},
	
	setMarqueeStyle: function(style/*a style json object*/){
    	//summary:
		//		set the marquee style
    	this._marqueeStyle = style;
    },
    
    getMode: function(){
    	//summary:
		//		gets the marquee mode
    	return this._marqueeMode;
    },
    
    setMode: function(mode){
    	//summary:
		//		sets the marquee mode
    	this._marqueeMode = mode;
    },
    isPartialSelectionEnabled: function(){
    	//summary:
		//		gets if the marquee selects partial bounded elements.
    	return this._partial;
    },
    
    setPartialSelectionEnabled: function(partial){
    	//summary:
		//		sets if the marquee selects partial bounded elements.
    	this._partial = partial;
    },
    _dragPreStart: function(e) {
		//	summary:
		//		Starts the marquee selection process by storing initial coordinates for pointer and view rectangle
		//
    	this.inherited(arguments);
    	if(!e.ctrlKey) { // ask for ctrl key
    		this._mustClear = true;
    	}else{
    		this._mustClear = false;
    	}
		this._marqueePointerStart = this._viewport.eventContentLocation(e);
			
    },
    
    _dragMove: function(e) {
		//
		//	summary:
		//		When the receiver is _marqueeActive, draw the corresponding marquee on the viewport UI layer.
    	//
    	// this.inherited(arguments); 
		// replaced by the following for performance:
		superclass._dragMove.apply(this, arguments);
		
    	var p = this._viewport.eventContentLocation(e); // current mouse coord
        
        var r = g.createRect(this._marqueePointerStart,p);
        
        var transformedRect = this._transformRect(r);
        transformedRect.width = transformedRect.width || 1; // zero crash vml
        transformedRect.height= transformedRect.height|| 1; // zero crash vml
        	
        if(this._marquee==null){
        	this._marquee = this._viewport.getOverlayLayer().createRect( transformedRect );
        	this._applyStyle(this._marquee,this._marqueeStyle);
        }else{
        	this._marquee.setShape(transformedRect);
        }
        
        
    },
   
	_dragEnd: function(e) {
		//
		//	summary:
		//		Terminates the current marquee selection process.
    	//
    	var dragged = this.hasDragged;
		this.inherited(arguments);
    	if(dragged){
    		var p2 = this._viewport.eventContentLocation(e);
    		this._selectBoundedElements(this._marqueePointerStart,p2);
    		this._viewport.getOverlayLayer().remove(this._marquee);
    		this._marquee = null;
    	}
    },
    
    _selectBoundedElements: function(p1,p2){
    	// Summary:
    	//		select all the elements that are completely bounded by the rectangle delimited by the given points
    	
    	var rect = g.createRect(p1,p2);
    	var graph = this._viewport._diagram.getGraph();
    	var toBeAdded = [];
    	this._selectElements(graph,graph,rect,toBeAdded);
    	this._selection.add(toBeAdded,this._mustClear);
    },
    
    _selectElements: function(parentGraph,graph,rect,toBeAdded){
    	var anyAdded = false;
    	for(var n in graph.children){
    		var c = graph.children[n];
    		var b = c.getBounds(parentGraph);
    		
    		var isBounded = this._evaluatesBounded(parentGraph,c,b,rect);
    		var isTouched = this._partial && this._evaluatesTouched(parentGraph,c,b,rect);
    		var isSubgraph = c._isIBMDiagramSubgraph && !c.isCollapsed();
    		
    		var shouldBeIncluded = isBounded||isTouched;
    		
    		switch(this._marqueeMode){
    			case MarqueeSelectionInteractor.topLevelOnly:{ 
    				if(shouldBeIncluded){
    					this._addToSelection(c,toBeAdded);
    					anyAdded = true;}
    				break;}
    			case MarqueeSelectionInteractor.topParent:{ 
    				if( (isSubgraph && isBounded) || (!isSubgraph && shouldBeIncluded)){
    					this._addToSelection(c,toBeAdded);
    					anyAdded = true;
    				}else {if(isSubgraph){
    					anyAdded = this._selectElements(parentGraph,c.getGraph(),rect,toBeAdded) || anyAdded;
    					if(!anyAdded && shouldBeIncluded){
    						this._addToSelection(c,toBeAdded);
    						anyAdded = true;
    					}
    				}}
    				break;}
    			case MarqueeSelectionInteractor.all:{ 
    				if(shouldBeIncluded){
    					this._addToSelection(c,toBeAdded);
    				anyAdded = true;}
    				if(isSubgraph){
    					anyAdded = this._selectElements(parentGraph,c.getGraph(),rect,toBeAdded) || anyAdded;}
    				break;}
    		}
    	}
    	return anyAdded;
    },
    _evaluatesBounded: function(parentGraph,element,bound,rect){
    	return bound && g.containsRect(rect,bound);
    },
    _evaluatesTouched: function (parentGraph,element,bound,rect){
    	if(element._isIBMDiagramLink){
    		var points = element._pathPoints;
    		var t = element.getShapeToContainerTransform(parentGraph);
    		var included = false;
    		arr.forEach(points,function(p){
    				included = included || g.containsPoint(rect,t.transformPoint(p));
    		},this);
    		return included;
    	}else{
    		return bound && g.rectIntersects(bound,rect);
    	}
    },
    _addToSelection: function(element,toBeAdded){
    	if(this._selection.accept(element) && element.isSelectable()){
    		toBeAdded.push(element);
    	}
    },
  //TODO this function should be in a tool
    _transformRect: function(bb/*rectangle*/) {
    	//
		//	summary:
		//		transform the rectangle to absolute UI coordinates
    	//
    	var v = this._viewport, z = v.getZoom(), vr = v.getViewRect();
    	bb = g.moveRect(bb,g.negPoint(vr)); 		
    	bb.width = bb.width*z;
    	bb.height = bb.height*z;
    	bb.x = bb.x*z;
    	bb.y = bb.y*z;
		return bb;
	},
    //TODO this function should be in a tool
	_applyStyle:function(shape, style){
		// summary:
		//		sets a new style to the desired element
		// shape:
		//		the shape to apply the style
		// style:
		//      the new style to be applied
		for (var p in style) {
			if (style.hasOwnProperty(p)){
				var setter = 'set' + p.charAt(0).toUpperCase() + p.substring(1);
				if (shape[setter]) {
					shape[setter](style[p]);
				}
			}
		}
	}

});

// ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topLevelOnly: number
//		in this mode, only the top level elements (the ones that are created in the graph) are selected.
//		This is the default value.
MarqueeSelectionInteractor.topLevelOnly = 0;

// ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.topParent: number
//		in this mode, only the highest parent of the selected elements is selected, but not its children. 
//		For example, if a complete subgraph is bounded, just the subgraph is selected, but not its children.
//		In the case the subgraph is not completely bounded by the marquee, its bounded children are selected.
MarqueeSelectionInteractor.topParent = 1;

//ibm_ilog.diagram.interactors.MarqueeSelectionInteractor.all: number
//		in this mode all the bounding elements are selected.
MarqueeSelectionInteractor.all = 2;

return MarqueeSelectionInteractor;

});
},
'ibm_ilog/diagram/graphlayout/GraphAdapter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/GraphAdapter", [
"dojo/_base/declare",
"dojox/gfx/matrix",
"ibm_ilog/graphlayout/AbstractGraphLayoutModel",
"ibm_ilog/graphlayout/internalutil/GraphModelUtil",
"ibm_ilog/graphlayout/internalutil/HashTable",
"../Port",
"../Link",
"../util/Batch"
],
function(
declare,
matrix,
AbstractGraphLayoutModel,
GraphModelUtil,
HashTable,
port,
Link,
Batch
){

var ReshapeLinkStyle = GraphModelUtil.ReshapeLinkStyle;
var ReshapeLinkMode = GraphModelUtil.ReshapeLinkMode;
var BasicPort = port.BasicPort;

/*=====
var AbstractGraphLayoutModel = ibm_ilog.graphlayout.AbstractGraphLayoutModel;
=====*/

var GraphAdapter =
declare("ibm_ilog.diagram.graphlayout.GraphAdapter", AbstractGraphLayoutModel, {
	// summary:
	//		The implementation of the graph model interface to layout the contents of a Graph.
	
	_graph: null,
	
	_global: null, // work in global surface coordinates
	
	getGraph: function(){
		return this._graph;
	},
	
	constructor: function(graph, layout) {
		this._graph = graph;
		this._originatingLayout = layout;
	},

	/* old implementation (no swim lanes)...
	_filter: function(f)
	{
        var children = this._graph.children;
        var list = dojoilter(children, f, this);
        return list;
	},
	
	_isGraphChild: function(obj) {
		return obj.getParent() == this._graph;
	},
	*/
	
	// new implementation: swim lanes are "flattened"
	_filterChildren: function(f)
	{
		return this._filterChildrenImpl(this._graph.children, f, []);
	},
	
	_filterChildrenImpl: function(children, f, list)
	{
        var n = children.length;
        for(var i = 0; i < n; i++){
        	var child = children[i];
        	if(child._flattenForGL && !child._collapsed){
        		this._filterChildrenImpl(child._graph.children, f, list);
        	} else {
        		if(f.call(this, child))
        			list.push(child);
        	}
        }
        return list;
	},
	
	_isGraphChild: function(obj) {
		var parent = obj.getParent();
		while(parent && parent._flattenForGL){
			var subgraph = parent._owningSubgraph;
			if(subgraph)
				parent = subgraph.getParent();
			else
				break;
		}
		return parent == this._graph;
	},

    isNode: function(o) {
		return o._isIBMDiagramNode && !(o._flattenForGL && !o._collapsed) && this._isGraphChild(o);
	},
	
	_isNodeOrLink: function(o) { 
		return this.isNode(o) || this._isLink(o);		
	},
	
    getNodes: function() {
    	return this._filterChildren(this.isNode);
	},
	
    isLink: function(o) {
		if (o._isIBMDiagramLink && this._isGraphChild(o)) {
			var start = this._getVisibleStartNode(o);
			if (!start) 
				return false;
			var end = this._getVisibleEndNode(o);
			if (!end) 
				return false;
			return this._isGraphChild(start) && this._isGraphChild(end)
		}
		return false;
	},
	
    getLinks: function() { 
    	return this._filterChildren(this.isLink);
	},

    isIntergraphLink: function(o) { 
		if (o._isIBMDiagramLink && this._isGraphChild(o)) {
			var start = this._getVisibleStartNode(o);
			if (!start) 
				return false;
			var end = this._getVisibleEndNode(o);
			if (!end) 
				return false;
			return !this._isGraphChild(start) || !this._isGraphChild(end);
		}
		return false;
	},
	
    getIntergraphLinks: function() { 
    	return this._filterChildren(this.isIntergraphLink);
	},
	
    isSubgraph: function(o) { 
		return o._isIBMDiagramSubgraph && !(o._flattenForGL && !o._collapsed) && this._isGraphChild(o);
	},
	
    getSubgraphs: function(){
    	return this._filterChildren(this.isSubgraph);
	},
	
    getLinksFrom: function(node) { 
		var linksFrom = node.getLinks(true, false); 
		return dojo.filter(linksFrom, this.isLink, this)
	},
	
    getLinksTo: function(node) { 
		var linksTo = node.getLinks(false, true);
		return dojo.filter(linksTo, this.isLink, this)
	},
	
    getFrom: function(link) { return this._getVisibleStartNode(link); },
    getTo: function(link) { return this._getVisibleEndNode(link); },
	
	_getVisibleStartNode: function(link)
	{
		// summary:
		//		Gets the visible start node of a link, taking into account collapsed subgraphs.
		// tags:
		//		private
		var port = link._startPort;
		if (link._isIntergraphLinkAtStart)
			port = link._getVisiblePort(port);
		if(port)
			return port._owner;
		return null;
	},
	
	_getVisibleEndNode: function(link)
	{
		// summary:
		//		Gets the visible end node of a link, taking into account collapsed subgraphs.
		// tags:
		//		private
		var port = link._endPort;
		if (link._isIntergraphLinkAtEnd)
			port = link._getVisiblePort(port);
		if(port)
			return port._owner;
		return null;
	},
	
	getNodesAndAllLinks : function() {
		// summary:
		//		Gets the nodes, the links and the intergraph links contained in the graph associated with this graph model.
		// description:
		//		Gets the nodes, the links and the intergraph links contained in the graph associated with this graph model.
		//		This method is used only in the ibm_ilog.diagram.graphlayout.ServerSideLayout class, it is not part of the
		//		base graph model interface.
    	return this._filterChildren(this._isNodeOrLink);
	},
	
	getAllLinks : function() {
		// summary:
		//		Gets all the links and intergraph links contained in the graph associated with this graph model.
		// description:
		//		Gets all the links and intergraph links contained in the graph associated with this graph model.
		//		This method is used only in the ibm_ilog.diagram.graphlayout.Animation class, it is not part of the
		//		base graph model interface.
    	return this._filterChildren(this._isLink);
	},
	
    _isLink: function(o) { 
		return o._isIBMDiagramLink && this._isGraphChild(o) && 
			this._getVisibleStartNode(o) && this._getVisibleEndNode(o);
	},
	
    boundingBox: function(nodeOrLink){
		var rect;
		if (nodeOrLink._isIBMDiagramNode) {
			rect = nodeOrLink.getBounds(this._graph);
		}
		else {
			rect = nodeOrLink.getLocalBoundingBox();
			if (this._graph != nodeOrLink.getParent()) {
				var t = nodeOrLink.getTransformToContainer(this._graph);
				rect = t.transformRectangle(rect);
			}
		}
		if(this._global){
			var p1 = {x:rect.x, y:rect.y};
			var p2 = {x:rect.x+rect.width, y:rect.y+rect.height};
			p1 = this._localToGlobal(p1);
			p2 = this._localToGlobal(p2);
			rect.x = p1.x; rect.y = p1.y;
			rect.width = p2.x - p1.x;
			rect.height = p2.y - p1.y;
		}
		return rect;
	},
	
    moveNode: function(node, x, y) {
		if(this._global){
			var p = {x:x, y:y};
			p = this._globalToLocal(p);
			x = p.x; y = p.y;
		}
		node.move(x, y, this._graph); 
	},
	
    getLinkPoints: function(link) {
		var points = link.getPoints();
		if(this._global){
			for(var i = 0; i < points.length; i++)
				points[i] = this._localToGlobal(points[i]);
		} else if(link.getParent() != this._graph){
			var t = link.getShapeToContainerTransform(this._graph);
			for(var i = 0; i < points.length; i++)
				points[i] = t.transformPoint(points[i]);
		}
		return points;
	},
	
	getLinkPointAt: function(link, index) {
		// Faster than the default implementation in AbstractGraphLayoutModel.
		// Could be even faster if we'd want to provide Link.getPointAt.
		var points = link.getPoints();
		var point = points[index];
		if(this._global){
			 point = this._localToGlobal(point);
		} else if(link.getParent() != this._graph){
			var t = link.getShapeToContainerTransform(this._graph);
			point = t.transformPoint(point);
		}
		return point;
	},
	
    reshapeLink: function(link, style, fromPoint, fromPointMode, points, startIndex, length, toPoint, toPointMode){
		
		link.setTransform(null); // in case link was translated...
		
		if(this._global){
			if(fromPoint)
				fromPoint = this._globalToLocal(fromPoint);
			if(toPoint)
				toPoint = this._globalToLocal(toPoint);
			if(points){
				for(var i = 0; i < length; i++){
					points[startIndex+i] = this._globalToLocal(points[startIndex+i]);
				}
			}
		} else if(link.getParent() != this._graph){
			var t = link.getShapeToContainerTransform(this._graph);
			t = t.inverse();
			if(fromPoint)
				fromPoint = t.transformPoint(fromPoint);
			if(toPoint)
				toPoint = t.transformPoint(toPoint);
			if(points){
				for(var i = 0; i < length; i++){
					points[startIndex+i] = t.transformPoint(points[startIndex+i]);
				}
			}
		}
		
		// set link style:
		switch (style) {
			case ReshapeLinkStyle.IGNORE:
				break;
			case ReshapeLinkStyle.STRAIGHT:
				link.setShapeType(Link.LinkShapeType.Straight);
				break;
            case ReshapeLinkStyle.DIRECT:
                link.setShapeType(Link.LinkShapeType.Free);
                break;
			case ReshapeLinkStyle.ORTHOGONAL:
				link.setShapeType(length > 0 ? Link.LinkShapeType.OrthogonalEditable :
											   Link.LinkShapeType.Straight);
				break;
			case ReshapeLinkStyle.POLYLINE:
				link.setShapeType(Link.LinkShapeType.Free);
				break;
		}
		
		var parent = link.getParent();
		
		// start point:
		switch(fromPointMode){
			case ReshapeLinkMode.IGNORE:
			case ReshapeLinkMode.FIX:
				break;
			case ReshapeLinkMode.MOVE:
			 	this._movePort(link, true, fromPoint, false, parent);
			 	break;
			case ReshapeLinkMode.CLIP:
			 	this._movePort(link, true, null, true, parent);
			 	break;
			case ReshapeLinkMode.MOVE_AND_CLIP:
			 	this._movePort(link, true, fromPoint, true, parent);
			 	break;
		}
		
		// end point:
		switch(toPointMode){
			case ReshapeLinkMode.IGNORE:
			case ReshapeLinkMode.FIX:
				break;
			case ReshapeLinkMode.MOVE:
			 	this._movePort(link, false, toPoint, false, parent);
			 	break;
			case ReshapeLinkMode.CLIP:
			 	this._movePort(link, false, null, true, parent);
			 	break;
			case ReshapeLinkMode.MOVE_AND_CLIP:
			 	this._movePort(link, false, toPoint, true, parent);
			 	break;
		}
		
		// intermediate points:
		
		if(length > 0){
			if(startIndex != 0 || length != points.length)
				points = points.slice(startIndex, length - startIndex);
		} else {
			points = null;
		}
		link.setIntermediatePoints(points);
	},
	
	_movePort : function(link, start, point, clip, parent) {
		var port = start ? link.getStartPort() : link.getEndPort();
		// special case for intergraph links and collapsed subgraphs:
		var collapsed = false;
		if ((start && link._isIntergraphLinkAtStart) ||
		(!start && link._isIntergraphLinkAtEnd)) {
			var collapsedPort = link._getVisiblePort(port);
			if (collapsedPort != port) {
				// the link is an intergraph link,
				// and its start/end node is in a collapsed subgraph:
				// use the collapsed port instead.
				collapsed = true;
				port = collapsedPort;
			}
		}
		var node = port.getOwner();
		
		// can we reuse the same port?
		// AV: instead of checking that the port is our BasicPort, we could
		// check model.hasMoveableConnectionPoint. This would allow people 
		// to create their own movable ports as direct subclasses of PortBase,
		// while overriding accordingly the default implementation of
		// model.hasMoveableConnectionPoint. However, this use-case is unlikely
		// (people can always subclass BasicPort), and the current implementation
		// is faster (avoids to retrieve again the port). So let's stay with it. 
		if(!port._isIBMDiagramBasicPort || port.getLinksCount() > 1){
			// no, create a new one.
			port = new BasicPort();
			node.addPort(port);
			// special case for intergraph links and collapsed subgraphs:
			if (collapsed) {
				// the link is an intergraph link,
				// and its start/end node is in a collapsed subgraph:
				// we must not replace the actual start/end port of the link,
				// instead, we change the getCollapsedPort function of the subgraph
				// so that it will return the new BasicPort for this link.
				if (!node._collapsedPorts) {
					// first time we do this for this subgraph: initialize the hash table
					// and replace the function
					node._collapsedPorts = new HashTable();
					node.getCollapsedPort = function(link, port){
						return this._collapsedPorts.item(link);
					};
				}
				// remember to return the new BasicPort for this link.
				node._collapsedPorts.add(link, port);
			}
			else {
				if (start) 
					link.setStartPort(port);
				else 
					link.setEndPort(port);
			}
		}
		
		if (point) {
			var bounds = port.getConnectionBounds(parent);
			port.setPosition({
				x: (point.x - bounds.x) / bounds.width,
				y: (point.y - bounds.y) / bounds.height
			});
		}
		else {
			port.setPosition({
				x: 0.5,
				y: 0.5
			});
		}
		
		port.setOffset({x:0, y:0});
		port.setClipOnShape(clip);
	},
	
    hasMoveableConnectionPoint: function(link, origin) {
		var port = origin ? link._startPort : link._endPort;
		if (origin ? link._isIntergraphLinkAtStart : link._isIntergraphLinkAtEnd) {
			port = link._getVisiblePort(port);
		}
		// AVA: In case there wouldn't be a guarantee that in any situation
		// link._startPort / _getVisiblePort do return a (non-null) port, we would need 
		// to modify GraphAdapter such that it creates the port on the fly if necessary. 
		// But apparently there is always a port.
		return !port ||
		  // Testing _isIBMDiagramBasicPort just as GraphAdapter._movePort. Needed 
		  // because port.setPosition/isMovable are both defined on 
		  // BasicPort, not on its parent portBase.  
		  (port._isIBMDiagramBasicPort && port.isMovable());
	},
	
    hasPinnedConnectionPoint: function(link, origin) { 
		// Note that hasPinned =! !hasMoveable.
		// Whenever the port is *not* a BasicPort, GraphAdapter replaces it with a BasicPort.
		// So it considers it as movable. 
		// We identify user's intention to "pin" a port when the port is
		// a BasicPort with port.movable == false.
		
		var port = origin ? link._startPort : link._endPort;
		if (origin ? link._isIntergraphLinkAtStart : link._isIntergraphLinkAtEnd) {
			port = link._getVisiblePort(port);
		}
		// AVA: In case there wouldn't be a guarantee that in any situation
		// link._startPort / _getVisiblePort do return a (non-null) port, we would need 
		// to modify GraphAdapter such that it creates the port on the fly if necessary. 
		// But apparently there is always a port.
		return port &&
		  // Testing _isIBMDiagramBasicPort just as GraphAdapter._movePort. Needed 
		  // because port.setPosition/isMovable are both defined on 
		  // BasicPort, not on its parent portBase.  
		  port._isIBMDiagramBasicPort && !port.isMovable();
	},
	
    getLinkWidth: function(link) { return link.getStrokeWidth(); },
	
    createGraphModel: function(subgraph) { 
		var submodel = new GraphAdapter(subgraph.getGraph());
		//submodel.setRootModel(this.getRoot() || this);
		return submodel;
	},
	
	getParent: function() {
		var root = this.getRoot();
		if(root == null || root == this)
			return null;
		if (this._graph._owningSubgraph) {
			var parentGraph = this._graph._owningSubgraph.getParent();
			if(parentGraph == root._graph)
				return root;
			var parentSubgraph = parentGraph._owningSubgraph;
			if (parentSubgraph) 
				return root.getGraphModel(parentSubgraph);
			else 
				return root;
		}
		return null;
	},
	
    setOriginatingLayout: function(layout) {
        this._originaringLayout = layout;
    },
    getOriginatingLayout: function() {
        return this._originaringLayout;
    },
	
	afterLayout: function()
	{
		// Make sure subgraphs have their final size at the end of the layout,
		// for recursive layout. TODO: check if we can do better...
		if (this._graph._owningSubgraph) {
			Batch.endBatch();
            // do not call _doLayout explicitly as the layout is now handled
            // by the gfxlayout if it applies.
            if (dojo.config.useGfxLayout && this._graph._owningSubgraph.getLayout())
                return;
            this._graph._owningSubgraph._graphChanged();
		}
	},
	
	_localToGlobal : function(p)
	{
		var m = this._graph._getRealMatrix();
		if (m) {
			var m2 = this._global._getRealMatrix();
			m2 = matrix.invert(m2);
			m = matrix.multiply(m2, m);
			p = matrix.multiplyPoint(m, p);
		}
		return p;
	},
	
	_globalToLocal : function(p)
	{
		var m = this._graph._getRealMatrix();
		if (m) {
			var m2 = this._global._getRealMatrix();
			m = matrix.invert(m);
			m = matrix.multiply(m, m2);
			p = matrix.multiplyPoint(m, p);
		}
		return p;
	},
	
	saveAndEnableViewCoordinates: function(rootModel)
	{
		this._oldGlobal = this._global;
		this._global = rootModel._graph;
	},
	
	restoreAndDisableViewCoordinates: function()
	{
		this._global = this._oldGlobal;
	},
	
	getTransform: function(subModel)
	{
		return subModel._graph.getShapeToContainerTransform(this._graph);
	}	
});

return GraphAdapter;

});

},
'ibm_ilog/diagram/util/ErrorReporter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/ErrorReporter", ["dojo/_base/lang", "dojo/json",
"dojo/has!config-diagramForMobile?:dojo/i18n", "dojo/has!config-diagramForMobile?:dojo/string"],
function(lang, json, i18n, string){

	// TODO:
	// For now we must create a global object since there are still references to ibm_ilog.diagram.util.ErrorReporter
	// in the rest of the code, but when AMD migration is finished all global references should be replaced
	// by module references and this will be replaced by a simple object creation:
	// var e = {};
	var e = lang.getObject("ibm_ilog.diagram.util.ErrorReporter", true);

	var B;

	e.getLocalizedString = function(id){
		// summary:
		//		Returns the localized string for the specified identifier.
		// description:
		//		This function normally fetches the specified string (using the dojo.i18n API)
		//		from the package name "ibm_ilog.diagram" and the bundle name "ErrorReporter".
		//		On mobile platforms (that is, if dojo.config.diagramForMobile is true),
		//		dojo.i18n is not used (to reduce the size of downloaded code), so the id is
		//		returned unchanged. To localize messages on mobile, the application should
		//		replace this function (ibm_ilog.diagram.util.ErrorReporter.getLocalizedString)
		//		by a function that requests the localized string from the server.
		if(!i18n)
			return id;
		try { // workaround for i18n bug in dojo 1.7...
			if (!B) 
				B = i18n.getLocalization("ibm_ilog.diagram", "ErrorReporter");
			return B[id];
		} catch(error){
			return id;
		}
	};
	
	e._errors = {};
	
	e.error = function(id) {
		throw this.buildError.apply(this,arguments);
	};

	e.warn = function(id) {
		console.warn(this.buildDescription.apply(this,arguments));
	};

	e.buildError = function(id) {
		var e = new Error(this.buildDescription.apply(this,arguments));
		e._isIBMDiagramError = true;
		var ed = this._errors[id];
		if(ed!==undefined) {
			e.id = ed.id;
			e.errorCode = ed.errorCode; 
		}
		return e;
	};
	
	e.buildDescription = function(id) {
		var err = this._errors[id];
		if(!err) {
			err = this._errors.InvalidErrorId;
		}
		var description = err.descriptor.apply(this,arguments);
		return err.errorCode+ ": " + description + " ("+err.id+")";
	};

	
	e._errorAtObject = function(obj,args) {
		// TODO [AV] obj is ignored for now
		this.error.apply(this,args);
	};
	
	var toKw = function(args,names,start) {
		var kwArgs = {}
		start = start || 0;
		for(var i=0;i<names.length;i++) {
			kwArgs[names[i]] = args[start+i];
		}
		return kwArgs;
	};
	
	var substitute  = function(s, args) {
		if(string)
			return string.substitute(s, args);
		return s;
	};
	
	var buildStandardDescriptor = function(/*Array*/kwArgNames) {
		// 
		//	summary:
		//		Builds a standard error descriptor function.
		//
		//	description:
		//		This function takes names for the positional user arguments 
		//		in the ErrorReporter.error() call so that they can be referenced
		//		from the I18N string templates easier.
		//
		//	kwArgNames: Array
		//		list of the argument names for using them in the call to 
		//		string.substitute
		//
		return function(id) {
			return substitute(e.getLocalizedString(id),toKw(arguments,kwArgNames,1));
		};

	};
	
	e.declareError = function(errorCode,id,descriptor) {
		// 
		//	summary:
		//		Error declaration helper
		//
		//	errorCode: String
		//		The IBM standard error code.
		//
		//	id: String
		//		An alphabetic error name used to reference it from the 
		//		code; making it more readable.
		//
		//	descriptor: String|Array|Function
		//		The error message or message generator.
		//	
		//		It can be three: 
		//		* message itself (Non-I18Nized)
		//		* a function which handles the R.error() arguments directly
		//		* an array naming the arguments of the R.error() call; for 
		//		  which a function is built that calls string.substitute 
		//		  with these names for I18N. (We call this the standard descriptor)  
		//
		var error = {};
		error.errorCode = errorCode;
		error.id = id;
		if(typeof(descriptor) == "function") {
			error.descriptor = descriptor;
		} else if(descriptor instanceof Array) { 
			error.descriptor = buildStandardDescriptor(descriptor);
		} else {
			error.descriptor = function() {
				return ""+descriptor;
			};
		}
		if(this._errors[id]!==undefined) {
			throw new Error(substitute(e.getLocalizedString("ErrorIdAlreadyDeclared"),id));
		}
		this._errors[id] = error;
	};
	
	//
	// ERROR DECLARATIONS
	//	

	var errorTable =
	[

	 
	 	//-------------------------------------------------------
		// ErrorReporter internals
	 	//
		{	name: "Unknown" },
		{ name: "InvalidErrorId", args:["errorId"] },
		{	name: "ErrorIdAlreadyDeclared", args:["errorId"] },

		//-------------------------------------------------------
		// Generic-JavaScript
		//
		{ name: "SubclassResponsibility", args:["methodName"] },
	 	{
		 	name: "NullPointerException",
			descriptor: function(id, objectName, where) {
				return where?
						substitute(e.getLocalizedString("NullPointerException2"),{objectName:objectName,where:where}):
						substitute(e.getLocalizedString("NullPointerException"),{objectName:objectName});
			}
		},
		{
		 	name: "Wrapped",
		 	descriptor: function(id, process, error) {
		 		if(error.message) {
					error = error.message;
				}
				return substitute(e.getLocalizedString(id),{process:process,error:error});
			}
		},
		{
		 	name: "UnexpectedArgument",
		 	descriptor: function(id, sender, functionName, argumentName, actualArgument) {
				// summary:
				// 		Throws an error indicating that an argument is not of the expected type or range. 
				// sender:
				//		The object that raised the error.
				// functionName:
				//		The name of the function in which the error occurred.
				// argumentName:
				//		The name of the argument that received a wrong value.
				// actualArgument:
				//		The argument value that was passed and detected as unexpected.
				return substitute(e.getLocalizedString(id),toKw(arguments,["sender","functionName","argumentName","actualArgument"],1));
			}
		},
		{
		 	name: "InvalidNumberOfArguments",
		 	descriptor: function(id, args) {				
				return substitute(e.getLocalizedString(id),{
					numberOfArgs: args.length,
					options: Array.prototype.slice.call(arguments,4).toString()});
			}
		},
		{
			name:"UnexpectedReturnValueType", args: ["sender", "functionName", "expectedType", "actualType"]
		},
		
		
		// ------------------------------------------------------
		// Dojo-version
		//
		{ name:"UnsupportedDojoFeature", args:["version"] },
		
		
		
		
		//-------------------------------------------------------
		// Generic-Graph/Diagram
		//
		{	name: "GraphElementExpected" },
		
		
		
		
		
		//-------------------------------------------------------
		// DataStore
		//
		{	name: "NotInDataStore" },
		{	name: "CannotUseAStringForAQuery" },
		{	name: "NodesStoreRequired" },
		{	name: "StoresMustSupportIdentity" },
		{	name: "StoresMustSupportRead" },
		{
			name: "UnexpectedErrorOnNodesStoreFetch",
			descriptor: function(id, error){
				if(error.message) {			
					error = error.message;
				}
				return substitute(e.getLocalizedString(id),{errorString:error});
			}
		},
		{
			name: "UnexpectedErrorOnLinksStoreFetch",
			descriptor: function(id, error){
				if(error.message) {			
					error = error.message;
				}
				return substitute(e.getLocalizedString(id),{errorString:error});
			}
		},

		
		
		
		//-------------------------------------------------------
		// Overview
		//
		{	name: "OverviewDiagramMissing", args: ["diagram"] },

		
		
		//-------------------------------------------------------
		// WAI
		//
		{	name: "MissingWaiNode", args: ["element"] },

		
		
		//-------------------------------------------------------
		// ServerSideLayout
		//
		{
			name: "UnsupportedAlgorithm",
			descriptor: function(id,algorithm){
				return substitute(e.getLocalizedString(id),{algorithmClass:algorithm.declaredClass});
			}
		},


		
		//-------------------------------------------------------
		// Interactors
		//
		{	name: "RestrictedToOneInteractor" },
		{
		 	name: "InvalidInteractorId",
			descriptor: function(id, interactorId) {
				return substitute(e.getLocalizedString(id),{interactorId:json.stringify(interactorId)});
			}
		},
		{	name: "InvalidInteractorEvent", args: ["name"] },
		{	name: "InvalidInputFilterSpec", args: ["name"] },



		
		//-------------------------------------------------------
		// UI events/connections
		//
		{	name: "ConnectionStillActive", args: ["id"]	},
		{	name: "EventConnectionFailed", args: ["eventName"] },

		
		
		
		//-------------------------------------------------------
		// Viewport
		//
		{	name: "MaxZoomBelowMin" },
		{	name: "MinZoomAboveMax" },

		
		//-------------------------------------------------------
		// Swim lanes
		//
		{	name: "SwimLanesRequireGfxLayout" },
		

		//-------------------------------------------------------
		// User-defined functions
		//
		{
			name: "DropUserFunctionError",
			descriptor: function(id, error){
				if(error.message) {			
					error = error.message;
				}
				return substitute(e.getLocalizedString(id),{error:error});
			}
		},
		{
			name: "UndoUserActionError",
			descriptor: function(id, action, error){
				if(error.message) {
					error = error.message;
				}
				return substitute(e.getLocalizedString(id),{action:action,error:error});
			}
		},

		
		
		
		//-------------------------------------------------------
		//
		{	name: "LASTERRORCODE" }
	];
	
	//	
	//	Iterate the table and declare the errors.
	//
	
	for( var i = 0; i<errorTable.length; i++ ) {
		var code = "CWZDD"+(2000+i)+"E"; // Should we better put this on the table and take care of possible errors/repetitions/maintainance problems/etc.?
		// AVA: yes, it think it would be better. My experience: I got 
		// CWZDD2016E: A node or link layout algorithm is needed. 
		// Then, to debug, I grepped for "2016" everywhere and found nothing...
		var ed = errorTable[i];
		e.declareError(code,ed.name,ed.descriptor||ed.args||ed.description||[]);
	}
	
	return e;
});

},
'ibm_ilog/diagram/adorners/AnnotationAdorner':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/AnnotationAdorner", [
"../main",
"dojo/_base/declare",
"dojo/_base/connect",
"dojox/collections/ArrayList",
"dojox/collections/Dictionary",
"../templating",
"./Adorner",
"./MoveAnnotationInteractor"
],
function(
iid,
declare,
connect,
ArrayList,  // not used directly here, but used in Diagram to manage annotations
Dictionary, // not used directly here, but used in Diagram to manage annotations
templating,
Adorner,
MoveAnnotationInteractor
){

/*=====
var Adorner = ibm_ilog.diagram.adorners.Adorner;
=====*/

// Note on Dictionary/ArrayList: we require them here, so that on mobile we just have to tell users
// to require this module to use annotations.

var AnnotationTargetAdorner =
iid.gfxDeclaration(declare('ibm_ilog.diagram.adorners.AnnotationTargetAdorner', [Adorner], {

	_link: null,
	
	setLink: function(link) {
		this._link = link;
	},
	
	_onChange: function() {
		if(this._link) {
			this._link.revalidateLinkShape(); // immediate
			this._link.invalidateLinkShape(); // .. and failproof
		}
	}
}));

var sup = Adorner.prototype;

var a = iid.gfxDeclaration(declare('ibm_ilog.diagram.adorners.AnnotationAdorner', [Adorner], {
	//
	//	summary:
	//		An Adorner which displays node annotations
	//
	
	//
	//	annotationChangedEvent:
	//		The event to watch for associated with the annotation changing its value. 
	//
	annotationChangedEvent: "setAnnotation", 
	
	//
	//	annotationGetter:
	//		The method to invoke in the adorned entity to get the annotation. This is used in 
	//		the default implementation of refreshAnnotation. 
	//
	annotationGetter: "getAnnotation",
	
	
	_moveInteractor: null,
	
	initialize: function() {
		this.inherited(arguments);
		this._moveInteractor = new MoveAnnotationInteractor();
		this._moveInteractor.initialize(this._viewport,this);
		this._moveInteractor.setManager(this._diagram._interactors);
		
	},
	
	_reconnectNotifications: function() {
		//
		//	Add an additional connection to watch for "annotation changed" events.
		//
		Adorner.prototype._reconnectNotifications.apply(this,arguments);
		var e = this._adorned;
		if (e) {
			this.refreshAnnotation();
			this._changeConnections.push(connect.connect(e,this.annotationChangedEvent,this,"refreshAnnotation"));
		}
	},
	
	refreshAnnotation: function(){
		//
		//	summary:
		//		Update receiver with a new annotation
		//	description:
		//		Current implementation relies on the template referencing 
		//		the "annotation" bindable property of the receiver.
		//		Can be overriden by subclasses to produce specific 
		//		ways of	rendering the annotation like word wrap, etc.
		//
		this.setAnnotation(this._adorned[this.annotationGetter]());
	},

	activate: function () {
		this._moveInteractor.activate();
	},

	deactivate: function () {
		this._moveInteractor.deactivate();
	},
	
	_link: null,
	_target: null,

	_updateLayout: function () {
		sup._updateLayout.apply(this,arguments);
	},
	
	setLink: function(link,target) {
		this._target = target;
		this._link = link;
	},
	
	getLink: function() {
		return this._link;
	},
	
	getTarget: function() {
		return this._target;
	},
	
	setAdorned: function(adorned) {
		this.inherited(arguments);
		if(this._target) {
			this._target.setAdorned(adorned);
		}
	},
	
	getBaseShape: function() {
		return this.baseShape;
	},
	
	destroy: function() {
		sup.destroy.apply(this,arguments);
	},
	
	_onChanged: function() {
		if(this._link) {
			this._link.revalidateLinkShape(); // immediate
			this._link.invalidateLinkShape(); // .. and failproof
		}
	}
	
}));

templating.declareBindableProperty(a, "annotation", "");

a.AnnotationTargetAdorner = AnnotationTargetAdorner;

return a;

});

},
'ibm_ilog/diagram/gfxlayout/Layout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout/Layout", ["dojo/_base/lang", "dojo/_base/declare", "./Engine"],
function(lang, declare, Engine){

var Layout = declare('ibm_ilog.diagram.gfxlayout.Layout', null, {

    // the Group attached to this layout
    _container: null,
    
    
    // summary:
    //     The base class for Layout implementations.
    // description:
    //     A Layout instance is responsible for arranging the children of a GFX container
    //     according to positioning and sizing constraints.
    
    constructor: function(args) {
        // summary:
        //     Creates and initializes a new ibm_ilog.diagram.gfxlayout.Layout instance.
        // args : Object : a mixin set of parameters.
        
        lang.mixin(this, args);
    },
	
	getConfig:function(){
		// summary:
		// Gets the current configuration of this layout instance.
		// returns: Object
		//	An object that represents the layout current configuration.
		return {}; /*Object*/
	},
    
    _attach: function(/*dojox.gfx.Group*/container) {
        // summary:
        //     Associates this layout to the specified container.
        // container: dojox.gfx.Group:
        //     The container to associate.
        
        this._container = container;
    },
    
    _detach: function() {
        // summary:
        //     Detach this layout from its associated container.
        
        this._container = null;
    },
    
    onLayoutPropertyChanged: function() {
        // summary:
        //     Fires a layout property changed event.
        // description:
        //     This method should be called by layout subclasses when a property change needs
        //     the layout to be refreshed. 
        
        this.invalidate();
    },
    
    invalidate: function() {
        // summary:
        //     Invalidates this layout.
        // description:
        //     This method post a layout request to the layout engine.
        
        if (this.__inLayout) 
            return;
        //ibm_ilog.diagram.gfxlayout.postInvalidate(this._container);
        this._container.invalidate();
    },
    
    
    computePreferredSize: function(proposedSize) {
        // summary:
        //     compute the preferred size of the associated container according to the layout strategy.
        
        return proposedSize;
    },
    
    adjustBBox : function(/*Rectangle*/bbox){
        // summary:
        //     Adjusts the specified rectangle according to the constraints of this layout.
        return bbox;
    },

    inLayout:function(){
        // summary:
        //     Indicates whether this layout is in a layout session.
        return this.__inLayout; 
    },
    
    doLayout: function(/*Rect*/contBounds) {
        // summary:
        //     Applies this layout to the associated container.
        
        try {
            this.__inLayout = true;
            this.layoutChildren(contBounds);        
        } finally {
            this.__inLayout = false;
        }
    },
    
    layoutChildren: function(/*Rect*/contBounds) {
        // summary:
        //     Applies this layout strategy to the associated container within the specified bounds.
        //     Subclass must override this method to implement their own layout logic. 
    },
    
    placeChildInSlot: function(/*dojox.gfx.Shape*/shape, /*dojox.gfx.Rectangle*/slotBounds, /*Boolean*/hAlign, /*Boolean*/vAlign) {
        // summary:
        //     Places the specified object in the given slotBounds.
        // description:
        //     This method should be invoked by layout implementation to place a shape within
        //     the given slot bounds, taking into account the vertical and horizontal alignment
        //     contraints of the shape (specified by means of the 'valign' and 'halign' properties
        //     on the shape object) and the margins, if any (specified by means of the 'margins'
        //     property.
        // shape : dojox.gfx.Shape: the shape.
        // slotBounds : Rectangle : the slot bounds (i.e the bounds where the shape must lie in)
        // hAlign: Boolean : indicates whether the shape must be aligned horizontally according to
        //      the value of its halign property.
        // vAlign: Boolean : indicates whether the shape must be aligned vertically according to
        //      the value of its valign property.
        
        var checkW = true, checkH = true, shapeBounds = shape.getPreferredSize();
        if (shape.margins){
            slotBounds = this._applyMargins(slotBounds, shape.margins);
        }
        if (hAlign) {
            if (!vAlign) {
                shapeBounds.y = slotBounds.y;
                shapeBounds.height = slotBounds.height;
            }
            var halign = shape.halign || 'stretch';
            switch (halign) {
                case 'left':
                    shapeBounds.x = slotBounds.x;
					// the width should not be greater than the slot width
					shapeBounds.width = Math.min(shapeBounds.width, slotBounds.width);
                    break;
                case 'center':
					// the width should not be greater than the slot width
					shapeBounds.width = Math.min(shapeBounds.width, slotBounds.width);
                    shapeBounds.x = slotBounds.x + (slotBounds.width - shapeBounds.width) / 2;
                    break;
                case 'right':
					// the width should not be greater than the slot width
					shapeBounds.width = Math.min(shapeBounds.width, slotBounds.width);
                    shapeBounds.x = slotBounds.x + slotBounds.width - shapeBounds.width;
                    break;
                default:
                case 'stretch':
                    shapeBounds.width = slotBounds.width;
			        shapeBounds = shape._checkMinMaxSize(shapeBounds);
                    shapeBounds.x = slotBounds.x + (slotBounds.width - shapeBounds.width)/2;
                    checkW = false;
					break;
            }
        }
        if (vAlign){
            if (!hAlign) {
                shapeBounds.x = slotBounds.x;
                shapeBounds.width = slotBounds.width;
            }
            var valign = shape.valign || 'stretch';
            switch (valign) {
                case 'top':
                    shapeBounds.y = slotBounds.y;
					// the height should not be greater than the slot height
					shapeBounds.height = Math.min(shapeBounds.height, slotBounds.height);
                    break;
                case 'center':
					// the height should not be greater than the slot height
					shapeBounds.height = Math.min(shapeBounds.height, slotBounds.height);
                    shapeBounds.y = slotBounds.y + (slotBounds.height - shapeBounds.height) / 2;
                    break;
                case 'bottom':
					// the height should not be greater than the slot height
					shapeBounds.height = Math.min(shapeBounds.height, slotBounds.height);
                    shapeBounds.y = slotBounds.y + slotBounds.height - shapeBounds.height;
                    break;
                default:
                case 'stretch':
                    shapeBounds.height = slotBounds.height;
			        shapeBounds = shape._checkMinMaxSize(shapeBounds);
                    shapeBounds.y = slotBounds.y + (slotBounds.height - shapeBounds.height)/2;
					checkH = false;
                    break;
            }
        }
        // check Min/Max size
		if (checkW || checkH)
        	shapeBounds = shape._checkMinMaxSize(shapeBounds);
        
        shape.layout(shapeBounds);
    },
    
    // margins stuff
    
    _inflateH: function(/*Number*/val, /*Object||Array*/margins) {
        // summary:
        //     Enlarge the specified value by the given horizontal margins.
        // val : Number : the value to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the left and right margins are expected to be at index 0 and 2 respectively.
        
        if (margins instanceof Array) {
            // left and right are indexes 0 and 2;
            if (margins.length > 0) 
                val += margins[0];
            if (margins.length > 2) 
                val += margins[2];
        } else {
            val += 2 * margins;
        }
        return val;
    },
    
    _inflateV: function(/*Number*/val, /*Object||Array*/margins) {
        // summary:
        //     Enlarge the specified value by the given vertical margins.
        // val : Number : the value to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the top and bottom margins are expected to be at index 1 and 3 respectively.

        if (margins instanceof Array) {
            // top and bottom are indexes 1 and 3;
            if (margins.length > 1) 
                val += margins[1];
            if (margins.length > 3) 
                val += margins[3];
        } else {
            val += 2 * margins;
        }
        return val;
    },
    
    _inflateRect:function(/*dojox.gfx.Rectangle*/rect, /*Object||Array*/margins){
        // summary:
        //     Enlarge the specified rect by the given margins.
        // rect: Rectangle: the rectangle to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the top and bottom margins are expected to be at index 1 and 3 respectively, and
        //          the left and right margins are expected to be at index 0 and 2 respectively.
        
        rect = lang.clone(rect);
        if (margins instanceof Array) {
            // left
            if (margins.length > 0) {
                rect.x -= margins[0];
                rect.width += margins[0];
            }
            // top
            if (margins.length > 1) {
                rect.y -= margins[1];
                rect.height += margins[1];
            }
            // right
            if (margins.length > 2) {
                rect.width += margins[2];
            }
            // bottom
            if (margins.length > 3) {
                rect.height += margins[3];
            }
        } else {
            rect.x -= margins;
            rect.width += 2 * margins;
            rect.y -= margins;
            rect.height += 2 * margins;
        }
        return rect;
    },
        
    _applyMargins: function(/*dojox.gfx.Rectangle*/rect, /*Object||Array*/margins) {
        // summary:
        //     Applies the specified margins to given rect. This operation is the
        //     opposite of the _inflateRect method.
        // rect: Rectangle: the rectangle to inflate.
        // margins: Object || Array: the margins. If the parameter is an array,
        //          the top and bottom margins are expected to be at index 1 and 3 respectively, and
        //          the left and right margins are expected to be at index 0 and 2 respectively.
        
        rect = lang.clone(rect);
        if (margins instanceof Array) {
            // left
            if (margins.length > 0) {
                rect.x += margins[0];
                rect.width -= margins[0];
            }
            // top
            if (margins.length > 1) {
                rect.y += margins[1];
                rect.height -= margins[1];
            }
            // right
            if (margins.length > 2) {
                rect.width -= margins[2];
            }
            // bottom
            if (margins.length > 3) {
                rect.height -= margins[3];
            }
        } else {
            rect.x += margins;
            rect.width -= 2 * margins;
            rect.y += margins;
            rect.height -= 2 * margins;
        }
        if (rect.width<0)
            rect.width = 0;
        if (rect.height<0)
            rect.height = 0; 
        return rect;
    }

    
});

return Layout;

});

},
'ibm_ilog/diagram/interactors/WheelPanInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/WheelPanInteractor", [
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/event",
"dojox/gfx/matrix",
"dojox/gfx/fx",
"./Interactor",
"../util/Geometry"
],
function(
declare,
lang,
event,
matrix,
gfxfx,
Interactor,
g
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var WheelPanInteractor =
declare("ibm_ilog.diagram.interactors.WheelPanInteractor", [Interactor], {
	//
	//	summary:
	//		A WheelPanInteractor controls the process of panning the viewport by the mouse wheel
	//

	
	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//		The diagram to pan
	//
	_diagram: null,
	
	//
	//	_incrementFactor: Number
	//		The amount at which pan is moved at each discrete step
	//
	_incrementFactor: 40,
	
	_declareStates: function() {
		this._declareState("idle",["wheelPan"]);
	},
	
	activate: function() {
		this._goStateId("idle");
	},
	
	setIncrementFactor: function( /*Number*/ factor ) {
		//
		//	summary:
		//		Sets the amount at which pan is moved at each discrete operation
		//
		this._incrementFactor = factor;
	},
	
	getIncrementFactor: function() {
		//
		//	summary:
		//		Returns the amount at which pan is moved at each discrete operation
		//
		return this._incrementFactor;
	},
	
    initialize: function( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		//
		//	summary:
		//		Wheel Pan interactor constructor, based on a target Diagram widget.
		//
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._initialize();
		return this;
	},
    
    getDefaultConnections:function(){
		//
		//	summary:
		//		Returns the default event mapping
		//
    	return {
    		wheelPan:{src:this._diagram.domNode,srcEvt:this.eventNameWheel,connectTo:"_wheelPan",filter:this._buildInputFilter({ctrl:false,shift:false})}
    	};
    },
    
    _wheelPan: function( /*Event*/ e ) {
		//
		//	summary:
		//		Handles the wheel mouse event and dispatches the appropriate pan operations
		//
    	
    	var V = this._viewport;
		var delta = this._incrementFactor;
		
		if(this.eventWheelAmmount(e)<0) {
			delta = -delta;
		}

		var viewMovement = g.mulPoint(g.Point(0,delta),-1/V.getZoom()); // scale delta to zoom level to get delta in model coords
		
		V.setViewRect( g.moveRect(V.getViewRect(),viewMovement), {instant:true} );

		if (e.preventDefault) {
			e.preventDefault();
		}
		//	TODO: do we need this?
		e.returnValue = false;

    }
});

return WheelPanInteractor;

});

},
'ibm_ilog/diagram/Graph':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/Graph", ["./base",
"dojo/_base/declare", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/config",
"require",
"dojox/gfx/_base", "dojox/gfx/matrix",
"./Node", "./Link", "./Subgraph",
"./util/ErrorReporter", "./util/Batch", "./graphlayout/LayoutRunner", "./util/Geometry"],
function(iid, declare, lang, array, config, require, gfx, matrix, Node, Link, Subgraph, ErrorReporter, Batch, LayoutRunner, Geometry){

/*=====
var gfx = dojox.gfx;
var LayoutRunner = ibm_ilog.diagram.graphlayout.LayoutRunner;
=====*/

	var Graph = declare("ibm_ilog.diagram.Graph", [gfx.Group, LayoutRunner], {
		// summary:
		//		The Graph object acts as a container for Node and Link objects. 
		
		// Allows to test the class faster that testing declaredClass
		_isIBMDiagramGraph: true,
		
		// diagram widget if this graph is embedded in a Diagram widget
		_diagram: null,
		
		constructor: function(){
			var id = iid._IdGenerator.nextId(this).toString();
			this.getId = function(){
				return id;
			};
		},
		
		createNode: function(/*String || Object*/template, context){
			// summary:
			//		Creates a new node in this Graph.
			// template:
			//		The template of the new node. If not specified, the default node template is used.
			//      If null, then an empty node is created.
			// returns:
			//		The new node.
			
			var node = this._createShape(Node, template, Node.defaultTemplate, context);
			return node;
		},
		
		createLink: function(/*String || Object*/template, context){
			// summary:
			//		Creates a new link in this Graph.
			// description:
			//      The description defined in the template parameter must specify which shape 
			//      in the template defines the link path by means of the dojoAttachPoint attribute set to '_path'.
			//      In addition, the optional start and end arrow shapes can be defined in the template
			//      object and in this case must be declared using the corresponding '_startArrow' and '_endArrow'
			//      dojoAttachPoint's. 
			// template:
			//		The template of the new link. If not specified, the default link template is used.
			// returns:
			//		The new link.
			
			var link = this._createShape(Link, template, Link.defaultTemplate, context);
			return link;
		},
		
		createSubgraph: function(/*String || Object*/template, context){
			// summary:
			//		Creates a new subgraph in this Graph.
			// template:
			//		The template of the new subgraph. If not specified or null, the default subgraph template is used.
			// returns:
			//		The new subgraph.
			
			var subgraph = this._createShape(Subgraph, template, Subgraph.defaultTemplate, context);
			return subgraph;
		},
		
		createSwimLane: function(/*String || Object*/template, context){
			// summary:
			//		Creates a new swim lane in this Graph.
			// template:
			//		The template of the new swim lane. If not specified or null, the default swim lane template is used.
			// returns:
			//		The new swim lane.
			
			var swimlane = this._createShape(ibm_ilog.diagram.SwimLane, template, ibm_ilog.diagram.SwimLane.defaultTemplate, context);
			return swimlane;
		},
		
		_createShape: function(type, template, defaultTemplate, context){
			var shape = this.createObject(type);
			this._applyTemplate(shape, template, defaultTemplate, context);
			return shape;
		},
		
		_applyTemplate: function(shape, template, defaultTemplate, gfxContext){
			if (config.isBidiInDojoDiagrammer) 
				this._propogateTextDir(shape);
			if (template === null) // null-> empty shape 
				return;
			if (!template) { // undefined-> default template fallback
				template = defaultTemplate;
			}
			if (!template) {
				ErrorReporter.error("UnexpectedArgument", this, "createSubgraph", "template", template);
			}
			shape.applyTemplate(template, gfxContext);
			shape._template = template; // [AV] temporary, TBD with Patrick
			return shape;
		},
		
		_propogateTextDir: function(shape){
			if (config.isBidiInDojoDiagrammer) {
				var tDir = this.textDir, gr = this;
				while (!tDir) {
					if (!gr.getParent) 
						break;
					gr = gr.getParent();
					tDir = gr.textDir;
				}
				if (tDir && shape.setTextDir) 
					shape.setTextDir(tDir);
			}
		},
		
		onChanged: function(){
			// summary:
			//		This event is called to signal that the contents of the graph have changed.
		},
		
		_onChanged: function(shape, reason){
			// This method is called a lot, keep it short and fast :).
			
			// <inlining this call from BoundingBoxFix at diagext.js>
			// 
			// this.inherited(arguments);
			
			this._bbCacheValid = false;
			
			// <end inline>
			
			// TODO: aggregate changes
			this.onChanged();
			
			if (this._owningSubgraph) 
				this._owningSubgraph._graphChanged();
			
			// Auto-layout:
			// We do an auto-layout only if a node changes (basically when a node is moved),
			// to avoid infinite loops: node move -> auto-layout -> link change -> _onChanged -> etc
			if (shape._isIBMDiagramNode) 
				this._checkAutoLayout();
		},
		
		clear: function(/*boolean*/dispose){
			// summary: 
			//     removes all shapes from a group/surface
			// dispose: boolean :
			//     Indicates whether the graph contents should be disposed. The default
			//     value is true.
			
			if (typeof(dispose) === 'undefined') 
				dispose = true;
			if (dispose) {
				array.forEach(this.children, function(s){
					if (s.dispose && typeof(s.dispose) == "function") 
						s.dispose();
				}, this);
			}
			this.inherited(arguments);
		},
		
		textDir: "", //text direction property
		setTextDir: function(/*String*/textdir){
			// summary: 
			//      set text direction to graph, propagate a new text direction to children (subgraphs, nodes, links)
			// textdir: String, text direction 
			
			if (config.isBidiInDojoDiagrammer) {
				if (this.textDir == textdir) 
					return;
				this.textDir = textdir;
				array.forEach(this.children, function(n){
					if (n.setTextDir) {
						if (n instanceof Subgraph) 
							n.propagateTextDir(textdir);
						n.setTextDir(textdir);
					}
				});
			}
		},
		
		getTextDir: function(){
			// summary: 
			//      returns text direction of the graph	
			return this.textDir;
		},
		
		//------------------
		// Fit to contents
		//------------------
		
		fitToContents: function(margin, zoomOutOnly){
			// summary:
			//		Changes the transform of this Graph so that the whole graph is visible in its parent surface.
			// margin: number:
			//		A margin (in pixels) to leave around the graph contents.
			// zoomOutOnly: boolean:
			//		If true, the graph will only be zoomed out, that is, if its contents are smaller than the surface,
			//			the graph will only be centered and not zoomed in.
			//		If false, the graph will be zoomed in or out to fit the size of the parent surface.
			if (this._diagram) {
				var graphBounds = this.getBoundingBox();
				var viewBounds = this._diagram.getViewport().getViewRect();
				if (zoomOutOnly) {
					if (!margin) 
						margin = 10;
					if (graphBounds.width + 2 * margin <= viewBounds.width && graphBounds.height + 2 * margin <= viewBounds.height) {
						var r = {};
						r.x = graphBounds.x - (viewBounds.width - graphBounds.width) / 2 + margin;
						r.y = graphBounds.y - (viewBounds.height - graphBounds.height) / 2 + margin;
						r.width = viewBounds.width;
						r.height = viewBounds.height;
						this._diagram.getViewport().setViewRect(r);
						return;
					}
				}
				this._diagram.getViewport().setViewRect(Geometry.expandRect(graphBounds, margin), {
					center: true
				});
			}
			else 
				this.setTransform(this.getFitToContentsTransform(margin, zoomOutOnly));
		},
		
		getFitToContentsTransform: function(margin, zoomOutOnly){
			// summary:
			//		Computes the transform to apply to this Graph so that the whole graph is visible in its parent surface.
			// returns: dojox.gfx.matrix.Matrix2D:
			//		The transform to apply to the graph. If the parent of the graph is not a GFX Surface,
			//		the identity matrix is returned.
			// margin: number:
			//		A margin (in pixels) to leave around the graph contents.
			// zoomOutOnly: boolean:
			//		If true, the graph will only be zoomed out, that is, if its contents are smaller than the surface,
			//			the graph will only be centered and not zoomed in.
			//		If false, the graph will be zoomed in or out to fit the size of the parent surface.
			
			var surface;
			for (surface = this.getParent(); surface != null; surface = surface.getParent ? surface.getParent() : null) {
				if (!surface.getParent) 
					break;
			}
			if (!surface) 
				return matrix.identity;
			
			Batch.endBatch(); // update links
			if (!margin) 
				margin = 10;
			
			var surfaceSize = surface.getDimensions();
			
			var graphBounds = this.getBoundingBox();
			var viewBounds = {
				x: margin,
				y: margin,
				width: surfaceSize.width - 2 * margin,
				height: surfaceSize.height - 2 * margin
			};
			
			var t;
			if (!zoomOutOnly || graphBounds.width > viewBounds.width || graphBounds.height > viewBounds.height) 
				t = matrix.createFromRectangles(graphBounds, viewBounds, true);
			else 
				t = matrix.translate((viewBounds.x + viewBounds.width / 2) - (graphBounds.x + graphBounds.width / 2), (viewBounds.y + viewBounds.height / 2) - (graphBounds.y + graphBounds.height / 2));
			
			return t;
		},
		
		
		//------------------
		// Hit testing API
		//------------------
		
		hitTest: function(p, tolerance){
			// Summary:
			//		find the diagram elements bellow the given point
			//		The resulting array is order by the z-order of the elements
			var elements = [];
			if (tolerance == null) {
				tolerance = 2;
			}
			this._hitTest(this, p, elements, tolerance);
			return elements;
		},
		
		_hitTest: function(graph, p, elements, tolerance){
			for (var i = graph.children.length - 1; i >= 0; i--) {
				// we iterate in reverse order to return the last elements first (z-order)
				var c = graph.children[i];
				if (c.hitTest && typeof(c.hitTest) == "function") {
					if (c.hitTest(p, this.getParent(), tolerance)) {
						if (c._isIBMDiagramSubgraph && !c.isCollapsed()) {
							this._hitTest(c.getGraph(), p, elements, tolerance);
						}
						elements.push(c);
					}
				}
			}
		},
		
		//------------------------------------
		// Shortcuts to enable selection/move
		//------------------------------------
		
		setSelect: function(select, callback){
			// summary:
			//		Enables or disables interactive selection on this Graph.
			// select: Boolean
			//		If true, the user will be able to select nodes by clicking on them.
			//		If false, selection will be disabled.
			// callback: Function
			//		(Optional) A function that will be called when the Select interactor is actually loaded.
			//		If not already loaded, the Select interactor may be loaded asynchronously by the AMD loader
			//		so this callback can be used to be notified when the interactor is loaded.
			//		The callback is called only if the select argument is true.
			
			if (select != this._select) {
				this._select = select;
				if (select) {
					if (!this._selInteractor) {
						require(["./interactors/SelectionInteractor"], lang.hitch(this, function(SelectionInteractor){
							this._selInteractor = new SelectionInteractor().initialize(this);
							this._selInteractor.activate();
							if (callback) 
								callback.call(this);
						}));
					}
					else {
						this._selInteractor.activate();
						if (callback) 
							callback.call(this);
					}
				}
				else {
					if (this._selInteractor) {
						this._selInteractor.deactivate();
					}
				}
			}
			return this;
		},
		
		isSelect: function(){
			// summary:
			//		Returns true if selection is enabled, or false if selection is disabled.
			
			return this._select || false; // Boolean
		},
		
		getSelection: function(){
			// summary:
			//		Returns the selection object, if selection is enabled in the graph (isSelect == true).
			if (this.isSelect() && this._selInteractor) {
				return this._selInteractor.getSelection();
			}
			return null;
		},
		
		setMove: function(move, callback){
			// summary:
			//		Enables or disables interactive move on this Graph.
			// description:
			//		Enables or disables interactive move on this Graph.
			//		Note that enabling move also enables select automatically.
			// move: Boolean
			//		If true, the user will be able to move nodes by dragging them.
			//		If false, move will be disabled.
			// callback: Function
			//		(Optional) A function that will be called when the Move interactor is actually loaded.
			//		If not already loaded, the Move interactor may be loaded asynchronously by the AMD loader
			//		so this callback can be used to be notified when the interactor is loaded.
			//		The callback is called only if the move argument is true.

			if (move != this._move) {
				this._move = move;
				if (move) {
					this.setSelect(true, lang.hitch(this, function(){
						if (!this._moveInteractor) {
							require(["./interactors/MoveInteractor"], lang.hitch(this, function(MoveInteractor){
								this._moveInteractor = new MoveInteractor().initialize(this._selInteractor.getSelection(), this);
								this._moveInteractor.activate();
								if (callback) 
									callback.call(this);
							}));
						}
						else {
							this._moveInteractor.activate();
							if (callback) 
								callback.call(this);
						}
					}));
				}
				else {
					if (this._moveInteractor) {
						this._moveInteractor.deactivate();
					}
				}
			}
			return this;
		},
		
		isMove: function(){
			// summary:
			//		Returns true if move is enabled, or false if move is disabled.
			
			return this._move || false;
		},
		
		//----------------------------------------
		// Shortcuts to get nodes/links/subgraphs
		//----------------------------------------
		
		getNodes: function(){
			// summary:
			//		Returns the nodes (that is, instances of ibm_ilog.diagram.Node) contained in this Graph.
			
			return array.filter(this.children, function(item){
				return item._isIBMDiagramNode;
			}); // Array
		},
		
		getLinks: function(){
			// summary:
			//		Returns the links (that is, instances of ibm_ilog.diagram.Link) contained in this Graph.
			
			return array.filter(this.children, function(item){
				return item._isIBMDiagramLink;
			}); // Array
		},
		
		getSubgraphs: function(){
			// summary:
			//		Returns the subgraphs (that is, instances of ibm_ilog.diagram.Subgraph) contained in this Graph.
			
			return array.filter(this.children, function(item){
				return item._isIBMDiagramSubgraph;
			}); // Array
		},
		
		//--------------
		// Check moves:
		//--------------
		
		onElementMoving: function(element, transform, delta){
			// summary:
			//		Called when an element is about to be moved by an interactor.
			// description:
			//		This event lets a Graph forbid or limit node moves.
			//		The event handler can modify the delta argument:
			//		if delta.x or delta.y are modified, the move is limited to the new delta
			//		(and other objects moved together with the specified element will also be limited accordingly).
			//		The default implementation checks the limits set by setMoveLimits(rect) (null by default -> no limit).
			// element: ibm_ilog.diagram.GraphElement
			//		The element being moved
			// transform: dojox.gfx.matrix.Matrix2D
			//		The original transform of the element.
			// delta: Point
			//		An object with x and y properties specifying the move delta.
			//		This argument can be modified by event handlers to forbid of limit the move.
			
			var limits = this.getMoveLimits();
			if (limits) {
				// delta is in untransformed element coords, so convert the limits accordingly...
				if (transform) 
					limits = transform.inverse().transformRectangle(limits);
				var bb = element.getBoundingBox();
				if (bb.x + delta.x + bb.width > limits.x + limits.width) 
					delta.x = limits.x + limits.width - bb.x - bb.width;
				if (bb.y + delta.y + bb.height > limits.y + limits.height) 
					delta.y = limits.y + limits.height - bb.y - bb.height;
				if (bb.x + delta.x < limits.x) 
					delta.x = limits.x - bb.x;
				if (bb.y + delta.y < limits.y) 
					delta.y = limits.y - bb.y;
			}
		},
		
		_moveLimits: null,
		
		setMoveLimits: function(rect){
			// summary:
			//		Sets the move limits for this Graph.
			// description:
			//		This method lets you limit interactive node moves to a given rectangle in the Graph.
			// rect: Rectangle
			//		A rectangle (x/y/width/height) that specifies the region where nodes
			//		are allowed to be moved. If null, node moves are not limited.
			
			this._moveLimits = rect;
		},
		
		getMoveLimits: function(){
			// summary:
			//		Gets the move limits for this Graph.
			// description:
			//		See setMoveLimits().
			// returns: Rectangle
			//		A rectangle (x/y/width/height) that specifies the region where nodes
			//		are allowed to be moved. If null, node moves are not limited.
			//		The default is null (no limit).
			
			return this._moveLimits;
		},
		
		getOwningSubgraph: function(){
			// summary:
			//		Returns the Subgraph object that is displaying this Graph.
			// description:
			//		Returns the <code>Subgraph</code> object that is displaying this <code>Graph</code>.
			//		If this <code>Graph</code is not contained in a <code>Subgraph</code>,
			//		returns <code>null</code> or <code>undefined</code>.
			// returns:
			//		The Subgraph object that is displaying this Graph.
			return this._owningSubgraph; // ibm_ilog.diagram.Subgraph
		},
		
		//--------------
		// toString()...
		//--------------
		
		toString: function(){
			return "Graph[" + this.children.length + "]" + (this._owningSubgraph ? (" at " + this._owningSubgraph.toString()) : "");
		}
	});
	
	Graph.nodeType = gfx.Group.nodeType;
	Graph.defaultShape = {
		type: 'graph'
	};
	
	// Extend the GFX Surface and Group objects with a createGraph function:
	
	var graphCreator = {
		createGraph: function(){
			var graph = this.createObject(Graph);
			return graph;
		}
	};
	lang.extend(gfx.Surface, graphCreator);
	lang.extend(gfx.Group, graphCreator);
	
	return Graph;
});

},
'ibm_ilog/diagram/util/Batch':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/Batch", ["dojo/_base/lang"], function(lang){

	// Automatic batch mechanism to group possibly costly changes in Diagram objects.
	
	// TODO: Change to this when AMD conversion is complete:
	// var batch = {};
	var batch = lang.getObject("ibm_ilog.diagram.util.Batch", true);
	
	// the current batch
	var _batch = [];
	
	// the timer that will call endBatch
	var _timer = null;
	
	batch.add = function(obj){
		// summary:
		//		Adds the specified object to the current batch.
		// obj:
		//		The object to add to the current batch.
		// description:
		//		This method checks whether the specified object is already contained in the current
		//		batch, and if not adds it. When the first object is added, a timer is started so as
		//		to have the endBatch() function called automatically as soon as the current event loop
		//		is finished.
		//		This method is fast and can be called many times on an object without performance issues.
		
		if (!obj._inBatch) {
			_batch.push(obj);
			obj._inBatch = true;
			if (!_timer) {
				_timer = setTimeout(batch.endBatch, 0);
			}
		}
	};
	
	batch.endBatch = function(){
		// summary:
		//		Calls the _endBatch() method of all objects in the current batch.
		// description:
		//		This function is normally called automatically by the batch timer.
		//		You can call it explicitly if you want to refresh all the objects immediately.
		//		It is also possible to have endBatch() called when a specified method of a specified
		//		class is called, using endBatchOn(type, method).
		
		try {
			// Objects may be batched while we are here, so we save the current batch list
			// in a local variable and allocate a new "public" batch list immediately.
			// We loop until the new batch list is empty.
			// TODO: test for infinite loops?
			var batch = _batch;
			_batch = [];
			while (batch.length > 0) {
				for (var i = 0; i < batch.length; i++) {
					var obj = batch[i];
					if (obj._inBatch) {
						try {
							obj._endBatch();
						}
						finally {
							obj._inBatch = false;
						}
					}
				}
				batch = _batch;
				_batch = [];
			}
		}
		finally {
			_timer = null;
		}
	};
	
	batch.endBatchOn = function(type, method){
		// summary:
		//		Ensures that endBatch() is called when the specified method
		//		of the specified class is called.
		// description:
		//		You can call endBatchOn(type, method) to make sure all objects are refreshed immediately
		//		when the specified method is called.
		
		var old = type.prototype[method];
		type.prototype[method] = function(){
			var ret = old.apply(this, arguments);
			batch.endBatch();
			return ret;
		}
	};
	
	return batch;
});

},
'ibm_ilog/diagram/graphlayout/webworker/_matrix':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/webworker/_matrix", ["dojo/_base/lang", "dojox/gfx/matrix"], function(lang, matrix){

lang.extend(matrix.Matrix2D, {
	inverse: function() {
		// summary:
		//        Returns the inverse transformation.
		return matrix.invert(this);
	},
	isIdentity: function() {
		// summary:
		//        Indicates whether this transform corresponds to the identity operation.
		return this.xy === 0 && this.yx === 0 && this.xx === 1 && this.yy === 1 && this.dx === 0 && this.dy === 0;
	},
	transformPoint: function(/* dojox.gfx.Point */p) {
		// summary:
		//        Applies this transformation to the specified point.
		return matrix.multiplyPoint(this, p);
	},
	transformRectangle: function(/* dojox.gfx.Rectangle */rect) {
		// summary:
		//        Applies the transformation to a rectangle.
		// description:
		//        The method applies the transformation on all corners of the
		//        rectangle and returns the smallest rectangle enclosing the 4 transformed
		//        points.            
		rect = rect ||
		{
			x: 0,
			y: 0,
			width: 0,
			height: 0
		};
		if (this.isIdentity()) {
			return {
				'x': rect.x,
				'y': rect.y,
				'width': rect.width,
				'height': rect.height
			};
		}
		var m = matrix;
		var p0 = m.multiplyPoint(this, rect.x, rect.y);
		var p1 = m.multiplyPoint(this, rect.x, rect.y + rect.height);
		var p2 = m.multiplyPoint(this, rect.x + rect.width, rect.y);
		var p3 = m.multiplyPoint(this, rect.x + rect.width, rect.y + rect.height);
		var minx = Math.min(p0.x, Math.min(p1.x, Math.min(p2.x, p3.x)));
		var miny = Math.min(p0.y, Math.min(p1.y, Math.min(p2.y, p3.y)));
		var maxx = Math.max(p0.x, Math.max(p1.x, Math.max(p2.x, p3.x)));
		var maxy = Math.max(p0.y, Math.max(p1.y, Math.max(p2.y, p3.y)));
		var r = {};
		r.x = minx;
		r.y = miny;
		r.width = maxx - minx;
		r.height = maxy - miny;
		return r;
	}
});

return matrix;

});


},
'ibm_ilog/graphlayout/internalutil/Point':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// #######################################
// ######## NOT Generated by J2JS ########
// ###### DO NOT DELETE THIS FILE! #######
// #######################################

// ------------------------------------------------------------------------
// Utility methods used in the code generated by J2JS.
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/internalutil/Point", ["dojo/_base/declare"],
	function(declare) {
return declare('ibm_ilog.graphlayout.internalutil.Point', null, {
	x : 0,
	y : 0,
	
	constructor : function(x, y)
	{
		this.x = x || 0;
		this.y = y || 0;
	}
});
});
},
'ibm_ilog/diagram/AutomaticPort':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/AutomaticPort", ["dojo/_base/declare",
"./PortBase"],
function(declare, PortBase){

/*=====
var PortBase = ibm_ilog.diagram.PortBase;
=====*/

var AutomaticPort = declare("ibm_ilog.diagram.AutomaticPort", PortBase, {
	// summary:
	//		The AutomaticPort class represents a port that connects a link to the middle of the
	//		closest side of a node.
	
	needsReferencePoint: function(){
		// summary:
		//		Returns true to indicate that the automatic port needs a reference point
		//		to compute the connection point.
		return true;
	},
		
	computeConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary:
		//		Implements the calculation of the connection and origin points for the AutomaticPort class.
		// description:
		//		Computes the connection point as the middle of the side of the node that is closest
		//		to the reference point.
		//		The origin point is always the center of the node.
		
		var bounds = this.getConnectionBounds(parent);
		var cx = bounds.x + bounds.width/2;
		var cy = bounds.y + bounds.height/2;
		originPoint.x = cx;
		originPoint.y = cy;
		connectionPoint.x = cx;
		connectionPoint.y = cy;
		if (bounds.width > 0 && bounds.height > 0) {
			var dx = referencePoint.x - cx;
			var dy = referencePoint.y - cy;
			var refRatio = dx != 0 ? (dy > 0 ? dy : -dy) / (dx > 0 ? dx : -dx) : 2;
			if (refRatio > 1) {
				// above or below: determine which from sign of dy
				connectionPoint.y = bounds.y;
				if (dy > 0) 
					connectionPoint.y += bounds.height;
			}
			else {
				// left or right: determine which from sign of dx
				connectionPoint.x = bounds.x;
				if (dx > 0) 
					connectionPoint.x += bounds.width;
			}
		}
		return connectionPoint;
	}
});

return AutomaticPort;

});

},
'ibm_ilog/diagram/interactors/ZoomKeyInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/ZoomKeyInteractor", [
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/sniff",
"dojo/keys",
"./Interactor",
"../util/Geometry"
],
function(
declare,
lang,
has,
keys,
Interactor,
Geometry
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var ZoomKeyInteractor =
declare("ibm_ilog.diagram.interactors.ZoomKeyInteractor", [Interactor], {
	//summary:
	//		This Interactor manages the ViewPort Zoom through the keyboard interaction.
	//		This Interactor have 2 connections: zoomIn, zoomOut.
	
	_diagram: null,
	_incrementFactor: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.				
		this._diagram = diagram;
		this.setIncrementFactor(1.5);
		return this._initialize();
	},
	setIncrementFactor: function(incrementFactor){
		// summary: 
		//		sets the zoom increment factor
		// increment: integer
		//		the increment factor to be set
		this._incrementFactor = incrementFactor;
	},
	getIncrementFactor: function(){
		// summary: 
		//		gets the zoom increment factor
		// return: integer
		return this._incrementFactor;
	},
	zoomOut: function(e){
		// summary:
	    //		This method zooms the diagram out
	    // e: Event object
	    //		the event to be treated.
		this._zoom(1/this._incrementFactor);
	},
	zoomIn: function(e){
		// summary:
	    //		This method zooms the diagram in
	    // e: Event object
	    //		the event to be treated.
		this._zoom(this._incrementFactor);
	},
	zoomReset: function(e){
		// summary:
	    //		This method reset the diagram zoom (set to 1)
	    // e: Event object
	    //		the event to be treated.
		var v = this._diagram.getViewport();
		var zoom = 1/v.getZoom();
		this._zoom(zoom);
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return ZoomKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		var connections = {
			zoomIn: {
				hotKey: keys.NUMPAD_PLUS,
				connectTo: "zoomIn",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, zoomOut: {
				hotKey: keys.NUMPAD_MINUS,
				connectTo: "zoomOut",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, zoomReset: {
				hotKey: keys.NUMPAD_0,
				connectTo: "zoomReset",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, zoomReset2: {
				hotKey: 48, // The non-numpad 0 key - always different from numpad 0
				connectTo: "zoomReset",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}
		};
		if(!has("ff")){ // on all browsers except FF, the non-numpad +/- keys have different keycodes
			lang.mixin(connections,	{
			zoomIn2: {
				hotKey: 187,
				connectTo: "zoomIn",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}, zoomOut2: {
				hotKey: 189,
				connectTo: "zoomOut",
				filter: this._buildInputFilter({ctrl:true,shift:false})
			}
			});
		}
		return connections;
	},
	_zoom: function(zoomFactor){
		var v = this._diagram.getViewport();
		var ge = this._diagram.getFocusedElement();
		 
		if(ge){
			var point = Geometry.getRectCenter(ge.getBounds(this._diagram.getGraph()));
			this._zoomAt(v,point,zoomFactor);
		}else{
			var currentZoom = v.getZoom();
	        var zoom = currentZoom * zoomFactor;
			v.setZoom(zoom, {
				instant: true
			});
		}
		
		
	},
	_zoomAt: function(viewport, /*Point*/ p, /*Number*/z ) {
		//
		//	summary:
		//		Performs a zoom operation maitaining a pivot point p in the same 
    	//		fixed position relative to the viewport.
		//
    	var cp = viewport.pointClientToContent(p); // content p    	
    	viewport.zoomAt(cp,z);
    }
});

ZoomKeyInteractor.KeyInteractorId = "Zoom";

return ZoomKeyInteractor;

});

},
'ibm_ilog/diagram/util/HandleSet':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/HandleSet", ["dojo/_base/declare", "dojo/_base/array", "dojo/_base/connect", "dojox/collections/Dictionary", "./ErrorReporter"],
function(declare, array, connect, Dictionary, R){

var HandleSet = declare("ibm_ilog.diagram.util.HandleSet", null, {

	_connectionHandles: null,

	constructor: function () {
		this._connectionHandles = new Dictionary();
	},

	_connect: function (id, source, eventName, target, handlerName, connector, disconnector) {
		//
		//	summary:
		//		dojo.connects and stores handle in the receiver with given id
		//
		var h;
		connector = connector || connect.connect;
		disconnector = disconnector || connect.disconnect;
		if (this._connectionHandles.containsKey(id)) {
			R.warn("ConnectionStillActive", id);
		} else {
			switch (arguments.length) {
			case 5:
			case 7:
				h = connector(source, eventName, target, handlerName);
				break;
			default:
				R.error("InvalidNumberOfArguments", arguments, 5, 7);
				break;
			}
		}

		if (h) {
			this._addHandle(id, h, source, disconnector);
		}
	},

	_isConnected: function (id) {
		//
		//	summary:
		//		dojo.connects and stores handle in the receiver with given id
		//
		return this._connectionHandles.containsKey(id);
	},

	_disconnect: function (id) {
		//
		//	summary:
		//		dojo.connects and stores handle in the receiver with given id
		//
		var handle = this._connectionHandles.item(id);
		if (this._connectionHandles.remove(id)) {
			handle.disconnector(handle.h, handle.src);
		}
	},

	_addHandle: function ( /*String*/ id, /*dojo handler*/ handle, /*Obj*/ src, /*Function*/ disconnector) {
		//
		// summary:
		//		add a handle to the handles list
		// handle:
		//		the handle to be added
		//
		this._connectionHandles.add(id, {
			h: handle,
			src: src,
			disconnector: disconnector
		});
	},


	_disconnectAll: function () {
		//
		// summary:
		//		release all the handles.
		//
		array.forEach(this._connectionHandles.getValueList(), function (handle) {
			handle.disconnector(handle.h, handle.src);
		});
		this._connectionHandles.clear();
	}
});

HandleSet.gfxConnector = function(src,evt,object,handler) {
	return src.connect(evt,object,handler);
};

HandleSet.gfxDisconnector = function(handle,src) {
	return src.disconnect(handle);
};

return HandleSet;

});
},
'ibm_ilog/diagram/interactors/ZoomInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/ZoomInteractor", [
"../main",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/sniff",
"./Interactor"
],
function(
iid,
declare,
lang,
has,
Interactor
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var ZoomInteractor =
declare("ibm_ilog.diagram.interactors.ZoomInteractor", [Interactor], {
	//
	//	summary:
	//		A ZoomInteractor controls the process of viewport panning and zooming
	//

	
	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//		The diagram to pan
	//
	_diagram: null,
	
	//
	//	_incrementFactor: Number
	//		The factor at which the zoom is incremented at each discrete step
	//
	_incrementFactor: 1.5,
	
	_p0X:0,_p0Y:0,_p1X:0,_p1Y:0,

	_startDistance: 0,
	
	_declareStates: function() {
		var idleStates = ["wheelZoom"];
		if (iid.mobileEnabled)
			idleStates.push("touchStart");
		this._declareState("idle",idleStates);
		if (iid.mobileEnabled)
			this._declareState("activeTouch",["touchMove","touchEnd"]);
	},
	
	activate: function() {
		this._goStateId("idle");
	},
	
	setIncrementFactor: function( /*Number*/ factor ) {
		//
		//	summary:
		//		Sets the increment factor for zooming discrete operations
		//
		this._incrementFactor = factor;
	},
	
	getIncrementFactor: function() {
		//
		//	summary:
		//		Returns the increment factor for zooming discrete operations
		//
		return this._incrementFactor;
	},
	
    initialize: function( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		//
		//	summary:
		//		Zoom interactor constructor, based on a target Diagram widget.
		//
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._initialize();
		return this;
	},
    
    getDefaultConnections:function(){
		//
		//	summary:
		//		Returns the default event mapping
		//
    	var defaultConnections = {
    		wheelZoom:{
    			src:this._diagram.domNode,
    			srcEvt:this.eventNameWheel,
    			connectTo:"_wheelZoom",
    			filter:this._buildInputFilter(has("webkit")?{shift:true}:{ctrl:true})}
    	};
    	
		if (iid.mobileEnabled){
			var mobileDefaultConnections = {
				touchStart : {
					src: this._diagram.domNode,
					srcEvt: "ontouchstart",
					connectTo: "_touchStart"
				}, touchMove :{
					src: this._diagram.domNode,
					srcEvt: "ontouchmove",
					connectTo: "_touchMove"
				}, touchEnd: {
					src: this._diagram.domNode,
					srcEvt: "ontouchend",
					connectTo: "_touchEnd"
				}

			};
			lang.mixin(defaultConnections,mobileDefaultConnections);
		};
    	return defaultConnections;
    },
    
    _wheelZoom: function( /*Event*/ e ) {
		//
		//	summary:
		//		Handles the wheel mouse event and dispatchs the appropriate zooming operations
		//

		var v = this._incrementFactor;
		
		if(this.eventWheelAmmount(e)<0) {
			v = 1/v;
		}

		this._zoomAt(this._viewport.eventClientLocation(e),v);

		if (e.preventDefault) {
			e.preventDefault();
		}
		//	TODO: do we need this?
		e.returnValue = false;

    },
    
    _zoomAt: function( /*Point*/ p, /*Number*/z ) {
		//
		//	summary:
		//		Performs a zoom operation maitaining a pivot point p in the same 
    	//		fixed position relative to the viewport.
		//
    	var V = this._viewport;
    	var cp = V.pointClientToContent(p); // content p    	
    	V.zoomAt(cp,z);
    },
    
    _touchStart: function (/*Event*/ e) {
    	e.preventDefault();
    	if (e.touches.length != 2)
    		return false;
    	
    	this._p0X = e.touches[0].pageX;
    	this._p0Y = e.touches[0].pageY;
    	this._p1X = e.touches[1].pageX;
    	this._p1Y = e.touches[1].pageY;
    	
    	this._startDistance = Math.sqrt(Math.pow(this._p1X - this._p0X,2) + Math.pow(this._p1Y - this._p0Y,2));
    	
		this._goStateId("activeTouch");
		
    },
    _touchMove: function (/*Event*/ e) {
    	e.preventDefault();
    	if (e.touches.length < 2)
    		return false;
    	
    	this._p0X = e.touches[0].pageX;
    	this._p0Y = e.touches[0].pageY;
    	this._p1X = e.touches[1].pageX;
    	this._p1Y = e.touches[1].pageY;
    	
    	var distance = Math.sqrt(Math.pow(this._p1X - this._p0X,2) + Math.pow(this._p1Y - this._p0Y,2));
    	var scale = distance / this._startDistance;
    	this._startDistance = distance;
    	
    	var p = {
        		pageX : (this._p0X + this._p1X)/2,
        		pageY : (this._p0Y + this._p1Y)/2
        	};
        	
    	this._zoomAt(this._viewport.eventClientLocation(p),scale);
    	
    },
    _touchEnd: function (/*Event*/ e) {
    	e.preventDefault();
    	if (e.touches.length < 2)
    		this._goStateId("idle");
    	return false;
    	
    }

    
});

return ZoomInteractor;

});

},
'ibm_ilog/diagram/graphlayout/Animation':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/Animation", ["dojo/_base/declare", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/fx",
"../Node", "../Link", "../util/Batch"],
function(declare, lang, array, fx, Node, Link, Batch){

var Animation =
declare("ibm_ilog.diagram.graphlayout.Animation", null, {
	// summary:
	//		An internal class that handles graph layout animation.
	// tags:
	//		private
	
	_graph: null,
	_fitTransform: null,
	_nodes: null,
	_links: null,
	
	constructor: function(graph, recursive)
	{
		// tags:
		//		private
		this._graph = graph;
		this._nodes = this._getObjects(true, recursive);
		this._links = this._getObjects(false, recursive);
	},
	
	_getObjects: function(nodes, recursive){
		// summary:
		//		Gets all the nodes or links to animate.
		// tags:
		//		private
		var layout = this._graph._rml;
		if (layout) {
			var model = layout.getGraphModel();
			// Note: we use getAllLinks, which is specific to GrapherAdapter:
			// in the Diagram code we assume the model is always a GrapherAdapter.
			var objs = nodes ? model.getNodes() : model.getAllLinks();
			if(recursive)
				objs = this._concat(objs, this._getSubgraphObjects(model, model, nodes));
			return objs;
		}
		return [];
	},
	
	_getSubgraphObjects: function(root, model, nodes) {
		// summary:
		//		Gets all the nodes or links contained in subgraphs.
		// tags:
		//		private
		var objs = [];
		var subgraphs = model.getSubgraphs();
		for(var i = 0; i < subgraphs.length; i++){
			var submodel = root.getGraphModel(subgraphs[i]);
			objs = this._concat(objs, nodes ? submodel.getNodes() : submodel.getAllLinks());
			objs = this._concat(objs, this._getSubgraphObjects(root, submodel, nodes));
		}
		return objs;
	},
	
	_concat : function(a1, a2){
		if(a2 && a2.length > 0)
			return a1.concat(a2);
		else
			return a1;
	},
	
    beforeLayout: function(){
		// tags:
		//		private
		
		// stop previous animation:
		if (this._graph._glAnimation) {
			this._graph._glAnimation.stop();
			delete this._graph._glAnimation;
		}
		
		// save all node/link geometry:
		array.forEach(this._nodes, function(node){
			var bounds = node.getBounds();
			node._oldPosition = {
				x: bounds.x,
				y: bounds.y
			};
		});
		array.forEach(this._links, function(link){
			link._oldPoints = link.getPoints();
		});
	},
	
    afterLayout: function(){
		// tags:
		//		private
		
		// update links:
		Batch.endBatch();
		
		// get new node/link geometry:
		array.forEach(this._nodes, function(node){
			var bounds = node.getBounds();
			node._newPosition = {
				x: bounds.x,
				y: bounds.y
			};
		});
		array.forEach(this._links, function(link){
			link._newPoints = link.getPoints();
			// animate link points only if nodes have moved.
			link._newShapeType = link.getShapeType();
			link.setShapeType(Link.LinkShapeType.Free);
			if (link._newPoints.length != link._oldPoints.length) {
				// add points to the array that has less points by interpolation
				var more, less;
				if (link._newPoints.length > link._oldPoints.length) {
					more = link._newPoints;
					less = link._oldPoints;
				}
				else {
					more = link._oldPoints;
					less = link._newPoints;
				}
				// avg number of points to add on each segment
				var n = Math.floor((more.length - less.length) / (less.length - 1));
				for (var i = 0; i < less.length - 1; i++) {
					var p1 = less[i];
					var p2 = less[i + 1];
					// last segment: add exactly the number of points needed
					if (i == less.length - 2) 
						n = more.length - less.length;
					for (var j = 1; j <= n; j++) {
						var p = {
							x: p1.x + j * (p2.x - p1.x) / (n + 1),
							y: p1.y + j * (p2.y - p1.y) / (n + 1)
						};
						less.splice(j, 0, p);
					}
					i += n;
				}
			}
			link._newIntermediatePoints = link.getIntermediatePoints();
		});
		var anim = new fx.Animation({
			duration: 1000,
			rate: 20,
			curve: [0, 100],
            beforeBegin: function(){
                // disable _onChanged events during animation
            	// [av] TODO: maybthis flag changes should be moved into a global flag stack
            	Node._ChangesInvalidateLinksDisabled = true;                 
            },
			onAnimate: lang.hitch(this, function(v){
				var completion = v / 100;
				array.forEach(this._nodes, function(node){
					var p1 = node._oldPosition;
					var p2 = node._newPosition;
					node.move(p1.x + completion * (p2.x - p1.x), p1.y + completion * (p2.y - p1.y));
				});
				array.forEach(this._links, function(link){
					var oldp = link._oldPoints;
					var newp = link._newPoints;
					if (oldp && newp && oldp.length == newp.length) {
						var count = oldp.length;
						var ip = [];
						for (var i = 1; i < count - 1; i++) {
							var p1 = oldp[i];
							var p2 = newp[i];
							ip.push({
								x: p1.x + completion * (p2.x - p1.x),
								y: p1.y + completion * (p2.y - p1.y)
							});
						}
						link.setIntermediatePoints(ip);
					}
				});
				
				Batch.endBatch();
				
				if (this._graph.isAutoFitOnLayout() && this._mustSetFitTransform) {
					this._graph.setTransform(this._fitTransform);
					delete this._mustSetFitTransform;
				}
				
				this._graph.onLayoutAnimate(v);
			}),
			onEnd: lang.hitch(this, function(e){
                // re-enable the _onChanged events
            	// [av] TODO: maybe this flag changes should be moved into a global flag stack
                delete Node._ChangesInvalidateLinksDisabled;
				array.forEach(this._nodes, function(node){
					node._oldPosition = undefined;
					node._newPosition = undefined;
				});
				array.forEach(this._links, function(link){
					if (link._newIntermediatePoints) 
						link.setIntermediatePoints(link._newIntermediatePoints);
					if (link._newShapeType) 
						link.setShapeType(link._newShapeType);
					link._newShapeType = undefined;
					link._oldPoints = undefined;
					link._newPoints = undefined;
					link._newIntermediatePoints = undefined;
				});
				this._graph.onLayoutAnimationEnd();
				delete this._graph._glAnimation;
			})
		});
		
		if (this._graph.isAutoFitOnLayout()) {
			this._fitTransform = this._graph.getFitToContentsTransform();
			this._mustSetFitTransform = true;
		}
		
		this._graph.onLayoutAnimationStart();
		this._graph._glAnimation = anim;
		anim.play();
	}
});

return Animation;

});

},
'ibm_ilog/diagram/GraphElement':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/GraphElement", ["./base",
"dojo/_base/lang", "dojo/_base/array", "dojo/_base/declare", "dojo/_base/connect",
"dojox/gfx/_base",
"./Selectable", "./templating"],
function(iid, lang, array, declare, connect, gfx, Selectable, templating){

/*=====
var gfx = dojox.gfx;
var Selectable = ibm_ilog.diagram.Selectable;
var templating = ibm_ilog.diagram.templating;
=====*/

var GraphElement =
declare("ibm_ilog.diagram.GraphElement", [gfx.Group, Selectable, templating._DiagramTemplated], {
  	//
	// summary:
  	//		An element that can be contained in a Graph.
	//
    
	_gfxConnections: null,
	_dojoConnections: null,
	
	//
	//	_IBMDiagramType: String
	//		Graph element type, used to distinguish basic graph element types: node, subgraph, link. 
	//		Used to quickly index maps based on graph element types.
	//
	_IBMDiagramType: undefined,
	
	_isIBMDiagramGraphElement: true,
	
	toString: function() {
    	return this.declaredClass.split(".").pop()+" "+this.getId()+(this.getLabel?(": "+this.getLabel()):"");
    },
    constructor: function() {
        var id = iid._IdGenerator.nextId(this).toString();
        this.getId = function() {
            return id;
        };
        this._gfxConnections = [];
		this._dojoConnections =  [ connect.connect(this,"setLabel",this,function(){
        	connect.publish('/ibm_diagram/GraphElement/labelChanged', [this]);
        }) ];
    },
    dispose: function(options) {
    	//	summary:
			//		Dispose this GraphElement.
			//	description:
			//		This function should be called when the GraphElement is completely deleted, that is, when it is removed
			//		from its parent graph and it will never be used again.
			//	options:
    	//		options.noRecurse:
			//			Do not recurse children
    	this.beforeDisposing(this);
    	connect.publish('/ibm_diagram/GraphElement/beforeDisposing',[this]);
    	this._dispose();
    	this.afterDisposing(this);
    	connect.publish('/ibm_diagram/GraphElement/afterDisposing',[this]);
    	//free connections
    	this.disconnectAll();
    },
    beforeDisposing: function(graphElement) {
    	// summary:
        //		This function is called before disposing the GraphElement
    },
    afterDisposing: function(graphElement) {
    	// summary:
        //		This function is called after disposing the GraphElement
    	// description:
        //		This function free all possible listeners
    	
    	// free possible listeners
        this._disposeBoundCountext();
    },
    connect: function() {
    	// summary:
        //		override gfx.connect to add the handle to the internal list
    	var conn = this.inherited(arguments);
    	this._gfxConnections.push(conn);
    	return conn;
    },
    addConnection: function(connection){
    	// summary:
		//		Records a connection to an event on this graph element so it can be disconnected at dispose time.
		// description:
        //		This method should be called to record the result of every dojo.connect call done to the graph element that
    	//		should be disconnected on dispose. Do not call this method when a call to this.connect() is done (that is,
		// 		a GFX event connection), since this is already done internally.
    	this._dojoConnections.push(connection);
		
		// PR: you should NOT called this method for dojo.connect handler. Indeed, the
		// disconnectAll() method calls this.disconnect() which does not always relies on dojo.disconnect,
		// depending on the renderer. In general: you should not mix dojo.connect/disconnect with shape.connect/disconnect.
		//
		// ==> This method should be removed.
		// [Eric] changed the doc to explain this, and created 2 different lists (one for GFX events and one for dojo connect/on)
    },
    disconnectAll: function() {
    	// summary:
        //		This function is disconnect every connect attached to this element
    	array.forEach(this._gfxConnections,lang.hitch(this,function(conn){
    		this.disconnect(conn);
    		delete conn;
    	}));
    	array.forEach(this._dojoConnections,lang.hitch(this,function(conn){
    		conn.remove();
			delete conn;
    	}));
    	this._gfxConnections = [];
    	this._dojoConnections = [];
    },
    _dispose: function(graphElement) {
    	// summary:
        //		this function should be overriden by any GraphElement subclass that needs any specific disposing action.
    }
    
});

GraphElement.byShape = function(shape) {
	// summary:
	//		Returns the GraphElement that contains the specified shape, or null.
	
	for(var p = shape; p && p.getParent; p = p.getParent())
		if(p instanceof GraphElement)
			return p;
	return null;
};

return GraphElement;

});

},
'ibm_ilog/diagram/interactors/SelectionInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/SelectionInteractor", [
"../main",
"dojo/_base/declare",
"./Interactor",
"../Selection"
],
function(
iid,
declare,
Interactor,
Selection
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var SelectionInteractor =
declare("ibm_ilog.diagram.interactors.SelectionInteractor", [Interactor], {
	//summary:
	//		This function provides the functionality to manage selection events on links and nodes.
	//the selection object
	_selection: null,

	//the event generation root
	_root: null,

	initialize: function (root, selection, background) {
		// summary:
		//		creates a new instance, sets the graph element from where to connect and set the default event mapping
		if (selection == null) {
			this._selection = new Selection();
		} else {
			this._selection = selection;
		}
		this._root = root;
		this._background = background;
		this._initialize();
		if(background) {
			this._states.item("idle").addConnection("clear");
		}
		return this;
	},

	_declareStates: function () {
		var idleStates = ["select", "toggle","add"];
		if (iid.mobileEnabled)
			idleStates.push("touchSelect");
		this._declareState("idle", idleStates);
		this._declareState("active", ["move", "end"]);
		this._declareState("activeTouch", ["touchEnd"]);
	},

	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			select: {
				src: this._root,
				srcEvt: "onmousedown",
				connectTo: "_selectPress",
				filter: this._buildInputFilter({ctrl:false}),
				gfxConnect:true
			}, clear: {
				src: this._background,
				srcEvt: "onmousedown",
				connectTo: "_backgroundPress",
				filter: this._buildInputFilter({ctrl:false}),
				gfxConnect:true
			}, toggle: {
				src: this._root,
				srcEvt: "onmousedown",
				connectTo: "_selectionTogglePress",
				filter: this._buildInputFilter({ctrl:true,button:0}),
				gfxConnect:true
			}, add: {
				src: this._root,
				srcEvt: "onmousedown",
				connectTo: "_selectionAddPress",
				filter: this._buildInputFilter({ctrl:true,button:1}), // right button does not toggle out a selected entity
				gfxConnect:true
			}, move: {
				src: document,
				srcEvt: "onmousemove",
				connectTo: "_mouseMoved"
			}, end: {
				src: document,
				srcEvt: "onmouseup",
				connectTo: "_selectRelease"
			}, touchSelect: {
				src: this._root,
				srcEvt: "touchstart",
				connectTo: "_touchSelectPress",
				gfxConnect:true
			}, touchEnd: {
				src: document,
				srcEvt: "touchend",
				connectTo: "_touchSelectRelease"
			}
		};
	},
	
	_getGraphElementFromEvent: function (e) {
		// Summary:
		//		return the selection object
		var ge = this._getGraphElementFromEventInRoot(e, this._root);
		return ge !== this._root ? ge : null;
	},
	
	getSelection: function () {
		// Summary:
		//		return the selection object
		return this._selection;
	},

	//
	//	_focusedEntity: dojox.gfx.shape
	//		The entity focused during a selection operation (null if none)
	//
	_focusedEntity: null,


	_selectPress: function (event) {
		// summary:
		//		clear the selection and select the given element.
		// not multiselect
		this._focusedEntity = this._getGraphElementFromEvent(event);
		if (this._focusedEntity !== null && !this._focusedEntity.isSelected() && this._focusedEntity.isSelectable()  && this._selection.accept(this._focusedEntity)) {
			this._selection.add(this._focusedEntity,true);
		} else {
			this._goStateId("active");
		}
	},

	_touchSelectPress: function (event) {
		//console.log('touchSelectPress');
		event.preventDefault();
		if (event.touches.length > 1)
			return false;
		//event.target = event.touches[0].target;
		// summary:
		//		clear the selection and select the given element.
		// not multiselect
		this._focusedEntity = this._getGraphElementFromEvent(event);
		if (this._focusedEntity !== null && !this._focusedEntity.isSelected() && this._focusedEntity.isSelectable()  && this._selection.accept(this._focusedEntity)) {
			this._selection.add(this._focusedEntity,true);
		} else {
			this._goStateId("activeTouch");
		}
	},

	_mouseMoved: function (e) {
		this._goStateId("idle");
	},

	_selectRelease: function (event) {
		this._goState(null);

		if (!this._focusedEntity) {
			this._selection.clear();
		} else {
			if(this._focusedEntity.isSelectable() && this._selection.accept(this._focusedEntity)){
				this._selection.add(this._focusedEntity,true);
			}
		}
		this._focusedEntity = null;
		this._goStateId("idle");
	},

	_touchSelectRelease: function (event) {
		if (event.touches.length > 0){
			return false;
		}
		this._goState(null);

		if (!this._focusedEntity) {
			this._selection.clear();
		} else {
			if(this._focusedEntity.isSelectable() && this._selection.accept(this._focusedEntity)){
				this._selection.add(this._focusedEntity,true);
			}
		}
		this._focusedEntity = null;
		this._goStateId("idle");
	},

	_selectionTogglePress: function (event,noRemove) {
		// summary:
		//		adds the given element to the selection (if not selected) or remove it from the selection (if selected)
		// multiselect
		this._focusedEntity = this._getGraphElementFromEvent(event);
		if (this._focusedEntity != null) {
			if (!this._focusedEntity.isSelected() && this._focusedEntity.isSelectable() && this._selection.accept(this._focusedEntity)) {
				this._selection.add(this._focusedEntity);
			} else {
				if(!noRemove) {
					this._selection.remove(this._focusedEntity);
				}
			}
		}
	},
	_selectionAddPress: function (event) {
		this._selectionTogglePress(event,true);
	},
	
	_backgroundPress: function(event) {
		this._focusedEntity = null;
		this._goStateId("active"); // will clear if released, cancelled if moved
	}

});

return SelectionInteractor;

});

},
'ibm_ilog/diagram/main':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/main", ["dojo/_base/lang", "dojo/has"], function(lang, has) {
	// summary:
	//		This module is the main module of Dojo Diagrammer, used by other modules to get a root object (ibm_ilog.diagram).
	
	// Create or get the root ibm_ilog.diagram object. When AMD migration is complete,
	// this *should* be the only place where we explicitly reference ibm_ilog.diagram
	// (apart from dojo.declare() calls, which also create global variables as a side effect).
	//
	var diagram = lang.getObject("ibm_ilog.diagram", true);
	
	/*=====	
	ibm_ilog.diagram.version = function(){
		// summary:
		//		Version number of Dojo Diagrammer.
		// major: Integer
		//		Major version.
		// minor: Integer
		//		Minor version.
		// fixpack: Integer
		//		Fix Pack version.
		// ifix: Integer
		//		iFix version.
		// flag: String
		//		Descriptor flag.
		this.major = 0;
		this.minor = 0;
		this.fixpack = 0;
		this.ifix = 0;
		this.flag = "";
	}
	=====*/
	diagram.version = {
		major: 1, minor: 1, fixpack: 2, ifix: 0, flag: "",
		toString: function(){
			var v = diagram.version;
			return v.major + "." + v.minor + "." + v.fixpack + "." + v.ifix + v.flag;	// String
		}
	};
	
	return diagram;
	
});

},
'ibm_ilog/diagram/templating':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/templating", ["dojo/_base/lang", "dojo/_base/array", "dojo/_base/declare",
"./_base/gfxext", "./data/dtlbinding", "./util/ErrorReporter"],
function(lang, array, declare, ge, idb, R){

	// TODO: Change to this when AMD conversion is complete:
	// var idt = {};
	var idt = lang.getObject("ibm_ilog.diagram.templating", true);
	
	lang.mixin(idt, {
	
		instantiate: function(/*dojox.gfx.Surface || dojox.gfx.Group */parent, /*Object*/ template, bindings) {
			// summary:
			//        Instantiate the specified template.
			// description:
			//        A template object describes both the contents of a shape and its
			//        appearance properties. The description object must conform to the
			//        gfx serialization specification. In addition, it supports the concept
			//        of attach point that exists in dijit with the dojoAttachPoint attribute.
			//        When this method is invoked, an instance of the template is created 
			//        and initialized. If attach points are defined in the template, the 
			//        corresponding properties are defined on the instance.
			// parent: dojox.gfx.Surface || dojox.gfx.Group:
			//        The container of the new shape.
			// template: Object :
			//        The description object.
			try {
				// flag used to skip _onChanged notification 
				parent._noOnChanged = true;
				// no validation during the deserialization
				if (parent.suspendInvalidate) 
					parent.suspendInvalidate();
				
				var t = idt._deserialize(parent, template, bindings);
				// create dojoAttachPoint fields
				if (t instanceof Array) {
					array.forEach(t, function(s) {
						idt._makeAttachPoint(s, parent);
					});
				} else {
					idt._makeAttachPoint(t, parent);
				}
			} finally {
				delete parent._noOnChanged;
				// trigger layout
				if (parent.resumeInvalidate) 
					parent.resumeInvalidate(true);
				ge.propagateChange(parent, null);
			}
			return t;
		},
		
		_makeAttachPoint: function(s, root) {
			root = root || s;
			if (s._attachPoint) {
				root[s._attachPoint] = s;
				// track attachPoints to be able to dispose them 
				if (!('_attachPoints' in root)) 
					root._attachPoints = [];
				root._attachPoints.push(s._attachPoint);
				delete s._attachPoint;
			}
			// Group ?
			if ('children' in s) {
				array.forEach(s.children, function(c) {
					idt._makeAttachPoint(c, root);
				});
			}
		},
		
		_deserialize: function(/* dojox.gfx.Surface || dojox.gfx.Group */parent,/* Object */ object, bindings) {
			// summary:
			//        Deserializes the specified template object.
			// template: Object : the description object to deserialize.
			// returns:
			//        A gfx shape that represents the specified description.
			
			if (object instanceof Array) {
				return array.map(object, function(s) {
					return idt._deserialize(parent, s, bindings);
				}); // Array
			}
			try {
				var shape = ("shape" in object) ? parent.createShape(object.shape) : parent.createGroup();
			} catch (e) {
				R.error("Wrapped", "templating.js # _deserialize # createShape/group", e);
			}
			if (bindings && bindings.length > 0) {
				// TODO: bindings is not used here. Replace with a bool
				idt.resolveBindingRefs(shape, object, bindings);
			}
			try {
				if (shape.suspendInvalidate) 
					shape.suspendInvalidate();
				// flag used to skip _onChanged notification 
				shape._noOnChanged = true;
				for (var p in object) {
					if (p == 'shape') 
						continue;
					else if (p == "transform") {
						shape.setTransform(object.transform);
					} else if (p == "stroke") {
						shape.setStroke(object.stroke);
					} else if (p == "fill") {
						shape.setFill(object.fill);
					} else if (p == "font") {
						shape.setFont(object.font);
					} else if (p == "children") {
						array.forEach(object.children, function(c) {
							idt._deserialize(shape, c, bindings);
						});
					} else if (p == "selectedStyle") {
						shape.selectedStyle = object.selectedStyle;
					} else if (p == "dojoAttachPoint") {
						shape._attachPoint = object.dojoAttachPoint;
					} else { // extended attribute
						// first check if a setter exists
						var n = p.substr(0, 1).toUpperCase() + p.substr(1);
						var setter = shape['set' + n];
						if (typeof(setter) == "function") {
							setter.call(shape, object[p]);
						} else {
							// fallback
							shape[p] = object[p];
						}
					}
				}
			} finally {
				delete shape._noOnChanged;
				if (shape.resumeInvalidate) 
					shape.resumeInvalidate(false); // false = do not trigger a validate()
				ge.propagateChange(shape, null);
			}
			return shape; // dojox.gfx.Shape
		},
		
		resolveBindingRefs: function(shape, o, bindings, path) {
			path = path || [];
			for (var p in o) {
				if (p && p != "children") {
					if (p.indexOf("_binding_") !== -1) {
						var b = o[p];
						b.target = shape;
						b.path = lang.clone(path);
						// not needed anymore
						delete o[p];
					} else if (typeof(o[p]) == "object" || typeof(o[p]) == "function") {
						var npath = lang.clone(path);
						npath.push(p);
						this.resolveBindingRefs(shape, o[p], bindings, npath);
					}
				}
			}
		}
		
	});
	
	//
	// _DiagramTemplated
	//
	var _DiagramTemplated = declare("ibm_ilog.diagram.templating._DiagramTemplated", null, {
	
		applyTemplate: function(template, context) {
			// summary:
			//    Applies the specified template on this instance.
			// template: Object or string :
			//    the template to apply.
			// context: Object : the runtime context. Optional. If not specified, the runtime context is this shape instance.
			if (template) {
				if (this.children.length > 0) {
					this.clear();
				}
				this._clearAttachPoints();
				this._disposeBoundCountext();
				var dtlTemplate = template instanceof idb.GfxTemplate ? template : new idb.GfxTemplate(template);
				this._boundContext = context || new idb.BoundContext(this, dtlTemplate.useDtl);
				try {
				
					if (this.suspendInvalidate) 
						this.suspendInvalidate();
					var content = dtlTemplate.render(this._boundContext, this);
					this.onTemplateApplied(template);
				} catch (e) {
					this.onTemplateError(template, e);
					R.error("Wrapped", "_DiagramTemplated.applyTemplate", e);
				} finally {
					if (this.resumeInvalidate) 
						this.resumeInvalidate(true);
				}
				return content;
			}
			return null;
		},
		
		onTemplateApplied: function(/*Object || String*/template) {
			// summary:
			//      Invoked when the template has been applied.
			// template: Object or string :
			//    the template to apply.
		},
		
		onTemplateError: function(/*Object || String*/template, /*Exception*/ e) {
			// summary:
			//      Invoked if an error is raised when applying the specified template .
			// template: Object or string :
			//    the template to apply.
		},
		
		_disposeBoundCountext: function() {
			if (this._boundContext && this._boundContext.dispose) {
				this._boundContext.dispose();
			}
		},
		
		_clearAttachPoints: function() {
			if (this._attachPoints) {
				array.forEach(this._attachPoints, function(p) {
					delete this[p];
				}, this);
				this._attachPoints.length = 0;
				delete this._attachPoints;
			}
		},
		
		layoutShape: function( /*{shape}*/target) {
			//
			//	No layout needed by default.
			//
		},
		
		shallowInstantiateAt: function(target, template) {
			//
			//	Instantiate a shallow version of the node, only visible shape is
			//	generated, otherwise the result has no unneeded members. This means
			//	the result may not be an instance of the class of the receiver.
			//
			if (template) {
					var dtlTemplate = template instanceof idb.GfxTemplate ? template : new idb.GfxTemplate(template);
					var context = new idb.BoundContext(this, dtlTemplate.useDtl);
					var content = dtlTemplate.render(context, target);
			}
			
			return target;
			
		},
		
		onPropertyChanged: function(property, oldValue, newValue) {
			// summary:
			//		Called when a bindable property has changed.
			// description:
			//		Bindable properties are properties defined on a Node or a Link
			//		and that can be used in templates using bindings.
		},
		
		_getBindableProperty: function(property) {
			// summary:
			//		The generic method that gets a bindable property.
			// tags:
			//		private
			return this["_" + property];
		},
		
		_setBindableProperty: function(property, newValue) {
			// summary:
			//		The generic method that sets a bindable property.
			// tags:
			//		private
			var oldValue = this["_" + property];
			if (newValue != oldValue) {
				this["_" + property] = newValue;
				this.onPropertyChanged(property, oldValue, newValue);
			}
		},
		
		startDTLBatch: function() {
			if (this._boundContext) {
				this._boundContext.startBatch();
			}
		},
		
		endDTLBatch: function() {
			if (this._boundContext) {
				this._boundContext.endBatch();
			}
		}
	});
	
	// Declares a bindable property by extending a class with the following:
	//	- the field that contains the value (initialized to the default value)
	//	- the setXyx and getXyz methods that get and set the value
	idt.declareBindableProperty = function(type, property, defaultValue) {
		// summary:
		//		Declares a bindable property for an object type.
		// description:
		//		Bindable properties are properties of a graphic object (Node, Link or Subgraph)
		//		that can be used to define bindings in templates.
		// type:
		//		The type to which the property must be added.
		// property:
		//		The name of the property. If the property name is 'foo', declaring the bindable property
		//		will extend the specified type with two methods: getFoo() and a setFoo(newValue).
		//		In the template of the object, you can define bindings like: text: '{{foo}}'.
		//		The initial value of the bindable property will be used when creating the object, and
		//		when the property is changed the template will be updated automatically.
		// defaultValue:
		//		The default value of the property.
		var extension = {};
		// value field
		extension["_" + property] = defaultValue;
		var uname = (property.substr(0, 1).toUpperCase()) + property.substr(1);
		// getter function
		extension["get" + uname] = function() {
			return this._getBindableProperty(property);
		};
		// setter function
		extension["set" + uname] = function(newValue) {
			this._setBindableProperty(property, newValue);
			return this;
		};
		// OK extend now.
		lang.extend(type, extension);
	};
	
	// See comment in data/dtlbinding.js (GfxTemplate.render function)
	lang.extend(idb.GfxTemplate, {
		_instantiate: idt.instantiate
	});
	
	idt._DiagramTemplated = _DiagramTemplated;
	
	return idt;
});

},
'ibm_ilog/diagram/interactors/SwitchModeInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/SwitchModeInteractor", [
"dojo/_base/declare",
"dojo/_base/array",
"dojo/keys",
"./Interactor",
"./MoveKeyInteractor",
"./Navigator"
],
function(
declare,
array,
keys,
Interactor,
MoveKeyInteractor,
Navigator
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var SwitchModeInteractor =
declare("ibm_ilog.diagram.interactors.SwitchModeInteractor", [Interactor], {
	//summary:
	//		This Interactor manages the UndoManager through the keyboard interaction.
	//		This Interactor have 3 connections: switchEdition, enterEdition, enterNavigation.
	
	_diagram: null,
	_onNavigation: null,
	_navigationInteractorIds: null,
	_editionInteractorIds: null,
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.		
		this._diagram = diagram;
		this._navigationInteractorIds = [Navigator.KeyInteractorId];
		this._editionInteractorIds = [MoveKeyInteractor.KeyInteractorId];
		return this._initialize();
	},
	switchEdition: function(e){
		// summary:
	    //		switch between edition and navigation mode.
		if(this._onNavigation){
			this.enterEdition();
		}else{
			this.enterNavigation();
		}
	},
	enterEdition: function(){
		// summary:
	    //		enter edition mode
		this._setInteractorsEnabled(this._navigationInteractorIds,false);
		this._setInteractorsEnabled(this._editionInteractorIds,true);
		this._onNavigation = false;
	},
	enterNavigation: function(){
		// summary:
	    //		enter navigation mode
		this._setInteractorsEnabled(this._navigationInteractorIds,true);
		this._setInteractorsEnabled(this._editionInteractorIds,false);
		this._onNavigation = true;
	},
	addNavigationInteractorId: function(id){
		// summary:
		//		adds an interactor id, that will be enabled when entering on navigation mode
		this._navigationInteractorIds.push(id);
	},
	addEditionInteractorId: function(id){
		// summary:
		//		adds an interactor id, that will be enabled when entering on edition mode
		this._editionInteractorIds.push(id);
	},
	_setInteractorsEnabled: function(interactorsIds,enabled){
		array.forEach(interactorsIds,
				function(id){
						this._keyManager.get(id).setEnabled(enabled);
						},this);
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return SwitchModeInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			enterNavigation: {
				hotKey: null,
				connectTo: "enterNavigation",
				filter: this._buildInputFilter({})
			}, enterEdition: {
				hotKey: null,
				connectTo: "enterEdition",
				filter: this._buildInputFilter({})
			}, switchEdition: {
				hotKey: keys.F9,
				connectTo: "switchEdition"
			}
		};
	}
});

SwitchModeInteractor.KeyInteractorId = "EnterEdition";

return SwitchModeInteractor;

});

},
'ibm_ilog/diagram/graphlayout/_base':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/_base", [
"dojo/_base/lang",
"./LayoutRunner",
"./Animation",
"./_LayoutProvider",
"ibm_ilog/graphlayout/RecursiveMultipleLayout",
"./GraphAdapter"
],
function(
lang,
LayoutRunner,
Animation,
_LayoutProvider,
RecursiveMultipleLayout,
GraphAdapter
){
	
	// The purpose of this module is to avoid loading graph layout classes if no graph layout is used in the app.
	// This works as follows: if graph layout is used, at least one graph layout class will be loaded.
	// All graph layout classes are subclasses of GraphLayout.js, which has this module (ibm_ilog/diagram/graphlayout/_base)
	// in its reference list. When this module is loaded, it will mixin some graph layout class pointers into
	// LayoutRunner, which then be able to use them (at runtime).
		
	return lang.mixin(LayoutRunner, {
		Animation: Animation,
		_LayoutProvider: _LayoutProvider,
		RecursiveMultipleLayout: RecursiveMultipleLayout,
		GraphAdapter: GraphAdapter
	});
});

},
'ibm_ilog/diagram/adorners/MoveAnnotationInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/MoveAnnotationInteractor", [
"../main",
"dojo/_base/declare",
"dojox/gfx/matrix",
"../interactors/DragInteractor"
],
function(
iid,
declare,
m,
DragInteractor
){

/*=====
var DragInteractor = ibm_ilog.diagram.interactors.DragInteractor;
=====*/

var MoveAnnotationInteractor =
declare("ibm_ilog.diagram.adorners.MoveAnnotationInteractor", [DragInteractor], {

	initialize: function ( viewport, annotation ) {
		//
		//	summary:
		//		Initializes the interactor to work with the given handle
		//
		this._annotation = annotation;
		this._viewport = viewport;
		this._initialize();
		return this;
	},
	
	_getInitialEventSource: function () {
		return this._annotation;
	},

	_refreshTransform: function () {
		this._transform = this._annotation._adorned._getRealMatrix().inverse();
	},

	activate: function () {
		this.inherited(arguments);
	},

	_dragPreStart: function () {
		this._refreshTransform();
		this._connect("updateTransforms", this._viewport, "onViewRectChanged", this, "_refreshTransform");

		this.inherited(arguments);
	},

	_dragEnd: function () {
		this._disconnect("updateTransforms");
		var dragged = this.hasDragged, action, D;
		this.inherited(arguments);
	},

	_dragMove: function () {
		this.inherited(arguments);
		var t = this._annotation.getTransform();
		t = m.multiply(t,m.translate(this._delta))
		this._annotation.setTransform(t);
	}

});

return MoveAnnotationInteractor;

});

},
'ibm_ilog/diagram/interactors/FocusInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/FocusInteractor", [
"dojo/_base/declare",
"dojo/_base/event",
"./Interactor"
],
function(
declare,
event,
Interactor
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var FocusInteractor =
declare("ibm_ilog.diagram.interactors.FocusInteractor", [Interactor], {
	//
	//	summary:
	//

	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//
	_diagram: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//
	_graph: null,

	initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram ) {
		this._diagram = diagram;
		this._graph = diagram.getGraph();
		this._viewport = diagram.getViewport();
		this._initialize();
		return this;
	},

	_declareStates: function () {
		this._declareState("idle", ["changeFocus"]);
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		return {
			changeFocus: {
				src: this._viewport.getEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_changeFocus",
				gfxConnect: true
			}
		};
	},

	_getGraphElementFromEvent: function (e) {
		// Summary:
		//		return the selection object
		var ge = this._getGraphElementFromEventInRoot(e, this._graph);
		return ge !== this._graph ? ge : null;
	},
	
	_changeFocus: function (e) {

		this._diagram.setFocusedElement(this._getGraphElementFromEvent(e));

		event.stop(e);
	},

	onEnabled: function () {
		this._goStateId("idle");
	},

	onDisabled: function () {
		this._goState(null);
	}
	
});

return FocusInteractor;

});
},
'ibm_ilog/diagram/data/dtlbinding':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/data/dtlbinding", ["../main", "dojo/_base/declare", "dojo/_base/lang", "dojo/_base/array",
"dojo/_base/connect", "dojo/_base/config", "dojo/_base/json",
"dojox/dtl/_base", "dojox/dtl/Context", "dojox/collections/Dictionary",
"../util/ErrorReporter","../data/StoreApi"
],
function(iid, declare, lang, array, connect, config, json, dd, Context, Dictionary, R, StoreApi){

	// Note: we use dojo/_base/json which accepts non-strict JSON (i.e. no quotes needed around property names)
	//		 it would be nice to use dojo/json but we would have to convert all our templates to strict JSON...
	
	// TODO: Change to this when AMD conversion is complete:
	// var idb = {...
	var idb = lang.setObject("ibm_ilog.diagram.data.dtlbinding", {
		//dtlExp: /[\s{,]([\w'"]*):["'\s]*{{(.+?)}}[\s'"]*|[\s{,][\w]*:\s*['"][^'"]*{{[^'"]*}}[^'"]*['"]|{%.+?%}/g,
		//dtlExp: /([\s*{,])([\w\s'"]*):([^\{\{,]*{{(([^'"|]*)[^'"]*)}}[A-Za-z\s'"]*)|{%.+?%}/g,
		dtlExp: /([{,])(\s*['"]?([\w]*)['"]?\s*:([^\{\{,]*{{([^|}]+)[^}}]*}}[^,}]*))/g,
		//arrays:                  ([{,])(\s*['"]?([\w]*)['"]?:([^\{\{]*{{([^|}]+)[^}}]*}}[^,}\]]*))
		itemPropExp: /{{([^\.]*)[\.]?([^|}]*)\|?[^}]*\}{2}/,
		replaceExp: "$1_binding_$3:{destPName:'$3',srcPName:'$5',value:$4},$2",
		_get: function(key) {
			if (this.length) {
				return (this[0] instanceof idb.DataContext) ? this[0].get(key) : this[0][key];
			}
		}
	});
	
	$bind = function(expr) {
		var f = function() {
			return expr;
		};
		f.__is_idb = true;
		return f;
	};
	
	var dumpTpl = function(obj) {
		var recurse = arguments.callee, objtype = typeof obj;
		if (objtype == "number" || objtype == "boolean") {
			return obj + "";
		}
		if (obj === null) {
			return "null";
		}
		if (objtype == "string") {
			return json._escapeString(obj);
		}
		if (objtype == "function") {
			return obj.__is_idb ? obj.call() : null;
		}
		if (obj instanceof Array) {
			var res = array.map(obj, function(o) {
				var val = recurse(o);
				return val;
			});
			return '[' + res.join(',') + ']';
		}
		var output = [], key, sep = "", newLine = "";
		for (key in obj) {
			var keyStr, val;
			keyStr = json._escapeString(key);
			val = recurse(obj[key]);
			if (typeof val != "string") {
				// skip non-serializable values
				continue;
			}
			output.push(newLine + keyStr + ":" + sep + val);
		}
		return "{" + output.join("," + sep) + newLine + "}"; // String
	};
	
	
	var _handlersMap = new Dictionary();
	
	idb.connectShapeChanges = function(shape, ctx) {
		var handlers = _handlersMap.item(shape.getId());
		if (!handlers) {
			handlers = [];
			_handlersMap.add(shape.getId(), handlers);
		}
		var entry = {
			context: ctx,
			h: connect.connect(shape, 'onPropertyChanged', ctx, onShapeChanged)
		};
		handlers.push(entry);
	};
	
	idb.disconnectShapeChanges = function(/*ibm_ilog.diagram.data.dtlbinding.BoundContext*/ctx) {
		if (!ctx._shape) 
			return;
		var handlers = _handlersMap.item(ctx._shape.getId());
		if (handlers) {
			array.every(handlers, function(e, idx) {
				if (e.context === ctx) {
					connect.disconnect(e.h);
					handlers.splice(idx, 1);
					return false;
				}
				return true;
			});
		}
	};
	
	function onShapeChanged(property, oldValue, newValue) {
		var ctx = this;
		if (ctx.__changeBatchLevel) {
			ctx.__changeBatchQueue.push(property);
		} else {
			updateShapeBoundProperty(ctx, property, true);
		}
	};
	
	var _storesList = [];
	
	function _findStoreEntry(store) {
		var entry = null;
		array.every(_storesList, function(e, idx) {
			if (e.store === store) {
				entry = e;
				entry.idx = idx;
				return false;
			}
			return true;
		});
		return entry;
	};
	
	idb.connectStoreChanges = function(item, store, /*ibm_ilog.diagram.data.dtlbinding.BoundContext*/ ctx) {
		if (!StoreApi.getFeatures(store).canNotify) {
			return;
		}
		var entry = _findStoreEntry(store);
		if (!entry) {
			entry = {};
			entry.store = store;
			//entry.handler = connect.connect(store, "onSet", store, onStoreChanged);
			entry.handler = StoreApi.listen(store, {onUpdate: onStoreChanged, scope:store});
			// a map between the item identity and the associated context
			entry.ctxMap = new Dictionary();
			_storesList.push(entry);
		}
		var identity = StoreApi.getIdentity(store, item);
		// TODO: is it really a 1-1 relationship ? Not the case if two diagrams are connected to the same store in the same page.
		entry.ctxMap.add(identity, ctx);
	};
	
	idb.disconnectStoreChanges = function(/*ibm_ilog.diagram.data.dtlbinding.BoundContext*/ctx) {
		if (!ctx._dataCtx) 
			return;
		var store = ctx._dataCtx.store;
		if (!store) 
			return;
		var entry = _findStoreEntry(store);
		if (entry) {
			var item = ctx._dataCtx.item;
			var identity = StoreApi.getIdentity(store, item);
			entry.ctxMap.remove(identity);
			// no more entries for this store ?
			if (entry.ctxMap.count === 0) {
				// then disconnect the store
				//connect.disconnect(entry.handler);
				entry.handler.remove();
				// and remove the entry from the list
				_storesList.splice(entry.idx, 1);
			}
		}
	};
	
	function onStoreChanged(item, attribute, oldV, newV) {
		// Here, this = store;        
		var store = this;
		var entry = _findStoreEntry(store);
		if (!entry) 
			return; // should never happen
		var identity = StoreApi.getIdentity(store, item);
		var ctx = entry.ctxMap.item(identity);
		if (!ctx) 
			return;
		updateShapeBoundProperty(ctx, attribute);
	};
	
	var	dataPropExp = /{{\s*(data.[\w.]*)/;
	function getBindingPropertyValue(/*DataContext*/ctx, binding) {
		// check if the binding is a {{data.something...}}
		var	isDataProp = dataPropExp.test(binding.value);
		return isDataProp ? StoreApi.getValue(ctx.store,ctx.item,binding.srcPName) : undefined;		
	}
	
	function updateShapeBoundProperty(ctx, property, fromShape) {
		var bindings = ctx._bindings,
		// indicates if store is dojo.store implementation
			isDojoStore = ctx._dataCtx && StoreApi.fromDojoStore(ctx._dataCtx.store);
		
		if (!property && isDojoStore){ // a store and no property name -> dojo.store notification
			// find the property that changed. To avoid conflict with same-name prop ({{foo}} vs {{data.foo}}
			// ensures it is a *data.* prefix
			for (var i = 0; i < bindings.length; ++i) {
				var b = bindings[i];
				// check if the property has changed ?
				if (b.propValue !== getBindingPropertyValue(ctx._dataCtx, b)){
					property = b.srcPName;
					break;
				}
			}
		}
		for (var i = 0; i < bindings.length; ++i) {
			var binding = bindings[i];
			
			// TODO: uncomment this and demo does not render correctly the "Doctor's office" nodes. (multiline text not wrapped).
			// Either a invalidate() is missing, or a bug elsewhere, but it should work and the code below enabled.
			// 
			/*
			// skip the binding if the notif is from a shapeChanged and the binding is a {{data.xxx}}			
			if (fromShape && dataPropExp.test(binding.value))
				continue;
			*/
			
			if (binding.srcPName === property) {
				var shape = binding.target;
				// update the binding property value;
				if (isDojoStore) {
					binding.propValue = getBindingPropertyValue(ctx._dataCtx, binding);
				}
				var tpl = new dd.Template(binding.value, true);
				var v = tpl.render(ctx);
				v = _fromJson(v);
				var s0 = binding.path.length > 0 ? binding.path[0] : binding.destPName;
				var pname = (s0.substr(0, 1).toUpperCase()) + s0.substr(1);
				var mainGetter = lang.getObject('get' + pname, false, shape);
				if (!mainGetter) {
					continue;
				}
				// get a ref to the current toplevel property value. eg mainProp = shape.getStroke();
				var mainProp = mainGetter.call(shape);
				// go down the properties path and stop before the last element (e.g. mainProp.color)                    
				if (binding.path.length > 0) {
					var o = binding.path.length > 1 ? lang.getObject(binding.path.slice(1).join('.'), false, mainProp) : mainProp;
					// set the bound field to the new value (e.g: mainProp.color.r = newValue)
					o[binding.destPName] = v;
				} else {
					mainProp = v;
				}
				// finally call the setter with the updated mainProp object (e.g: shape.setStroke(mainProp))
				var mainSetter = lang.getObject('set' + pname, false, shape);
				if (mainSetter) {
				    if (config.isBidiInDojoDiagrammer &&  property === "textDir")
				        mainSetter.call(shape, mainProp, property);
				    else    				
					    mainSetter.call(shape, mainProp);
				}
			}
		}
	};
	
	// [av] TODO this is a generic array.contains implementation, maybe shoube placed somewhere else if needed.
	var __contains = function(a, v) {
		var i = a.length;
		while (i--) {
			if (a[i] === v) {
				return true;
			}
		}
		return false;
	};
	
	var __getChange = function(changes, shape, pname) {
		var c;
		for (var i = 0; i < changes.length; i++) {
			c = changes[i];
			if (c.shape == shape && c.pname == pname) {
				return c;
			}
		}
		return null;
	};
	
	function updateShapeBoundProperties(ctx, properties) {
	
		var complexChanges = [];
		var simpleChanges = [];
		
		array.forEach(ctx._bindings, function(binding) {
			if (__contains(properties, binding.srcPName)) {
				var shape = binding.target;
				var tpl = new dd.Template(binding.value, true);
				var v = tpl.render(ctx);
				v = _fromJson(v);
				var s0 = binding.path.length > 0 ? binding.path[0] : binding.destPName;
				var pname = (s0.substr(0, 1).toUpperCase()) + s0.substr(1);
				
				// go down the properties path and stop before the last element (e.g. mainProp.color)                    
				if (binding.path.length > 0) {
				
					var mainGetter = lang.getObject('get' + pname, false, shape);
					if (!mainGetter) {
						return;
					}
					
					var c = __getChange(complexChanges, shape, pname);
					if (!c) {
						c = {
							shape: shape,
							pname: pname,
							value: mainGetter.call(shape)
						};
						complexChanges.push(c);
					}
					
					var o = binding.path.length > 1 ? lang.getObject(binding.path.slice(1).join('.'), false, c.value) : c.value;
					// set the bound field to the new value (e.g: value.color.r = newValue)
					o[binding.destPName] = v;
				} else {
					simpleChanges.push({
						shape: shape,
						pname: pname,
						value: v
					});
				}
			};
		});
		
		array.forEach(complexChanges.concat(simpleChanges), function(c) {
			var mainSetter = lang.getObject('set' + c.pname, false, c.shape);
			if (mainSetter) {
				mainSetter.call(c.shape, c.value);
			}
		});
	};
	
	function _fromJson(v) {
		v = v.replace(/\\/g, '\\\\').replace(/\n/g, '\\n');
		if (v.charAt(0) == '"' && v.slice(-1) == '"') {
			v = '"' + v.slice(1, -1).replace(/"/g, '\\"') + '"';
		} else if (v.charAt(0) == "'" && v.slice(-1) == "'") {
			v = "'" + v.slice(1, -1).replace(/'/g, "\\'") + "'";
		}
		return json.fromJson(v);
	};
	
	idb.BoundContext = lang.extend(function(shape, listen) {
		this._shape = shape;
		// we don't mixin this with shape because we don't want the context methods (like dispose())
		// to be replaced by same-name shape methods.
		Context.call(this, {});
		this._listen = listen;
		if (this._shape) {
			if (this._listen) {
				idb.connectShapeChanges(shape, this);
				if (shape.getData) {
					var dataCtx = shape.getData();
					if (dataCtx) {
						this._dataCtx = dataCtx;
						idb.connectStoreChanges(dataCtx.item, dataCtx.store, this);
					}
				}
			}
			this.setThis(this._shape); // to resolve function call
		}
	}, Context.prototype, {
	
		get: function(key, otherwise) {
			if (this._shape) {
				var val;
				// special case: 'context': to be able to reference the shape in a template filter (as parameter)
				// { prop : {{ foo | myfilter:context }} } 
				// where myfilter:function(val, arg)  <- arg= this._shape
				if (key == 'context') 
					return this._shape;
				// first try a getKey()
				var pname = key.slice(0, 1).toUpperCase() + key.slice(1);
				var getter = lang.getObject('get' + pname, false, this._shape);
				if (getter && typeof(getter) == "function") {
					val = getter.call(this._shape);
					return val;
				}
				if (key in this._shape) {
					return this._shape[key];
				}
				// delegate to optional dict
				return Context.prototype.get.apply(this, arguments);
			}
		},
		dispose: function() {
			if (this._listen && this._shape) {
				idb.disconnectShapeChanges(this);
			}
			delete this._shape;
			if (this._listen) {
				idb.disconnectStoreChanges(this);
			}
			if (this._bindings) {
				this._bindings.length = 0;
			}
			delete this._bindings;
		},
		setBindings: function(bindings) {
			this._bindings = bindings;
			// iterate over the bindins prop 
			if (this._dataCtx && StoreApi.fromDojoStore(this._dataCtx.store)){
				for (var i=0; i<bindings.length;++i){
					var b = bindings[i];
					b.propValue = getBindingPropertyValue(this._dataCtx, b);
				}
			}
		},
		endBatch: function() {
			this.__changeBatchLevel--;
			if (!this.__changeBatchLevel) {
				try {
					updateShapeBoundProperties(this, this.__changeBatchQueue);
				} catch (e) {
					R.error("Wrapped", "BoundContext # endBatch", e);
				} finally {
					delete this.__changeBatchQueue;
				}
			}
		},
		startBatch: function() {
			if (!this.__changeBatchLevel) {
				this.__changeBatchQueue = [];
				this.__changeBatchLevel = 1;
			} else {
				this.__changeBatchLevel++;
			}
		}
		
	});
	
	function processTemplate(template) {
		var tpl = template, isStr = (typeof(template) == "string"), parse = isStr, useDtl = false;
		// a plain JS object -> dump to string except if it has no bindings
		if (!isStr && !template.noDtl) {
			// serialize the template object to string
			tpl = dumpTpl(template); /*String*/
			parse = true;
		}
		if (parse) {
			if (tpl.length > 0) {
				// detect whether dtl is used
				// need to use indexOf as the regexp may return result for str such as "{shape:{..}}"
				if (tpl.indexOf('{{') > 0 || tpl.indexOf("{%") > 0) {
					// some dtl markups have been found
					useDtl = true;
					var result = tpl.replace(idb.dtlExp, idb.replaceExp);
					if (result) {
						// escape the dtl expression so that it can be 'dejsonified'
						var valueExp = /(value):([^}}]*}}[^,}'"]*['"]*)/
						var values = result.match(valueExp);
						while (values) {
							var s = values[2];
							s = "'" + escape(s) + "'";
							result = result.replace(valueExp, "$1:" + s);
							values = result.match(valueExp);
						}
						tpl = result;
					}
				} else { // no DTL detects
					// no DTL and was a plain JS ? mark the template object for next passes
					if (!isStr) {
						tpl = template;
						template.noDtl = true;
					}
				}
			} else { // empty string -> make it plain object
				tpl = {};
				tpl.noDtl = true;
			}
		}
		return {
			tpl: tpl,
			useDtl: useDtl
		};
	};
	
	iid.declareTemplate = function(template) {
		var r = processTemplate(template);
		return new idb.GfxTemplate(r.tpl, r.useDtl);
	};
	
	idb.GfxTemplate = lang.extend(function(template, useDtl) {
		// 2 arguments:  the ctor expects in this case a preprocessed template
		// as first parameter, and whether it uses dtl as 2nd parameter.
		//
		// 1 argument : the ctor expects a raw template.
		
		this.template = null;
		this.useDtl = false;
		var i;
		
		if (arguments.length > 1) {
			this.useDtl = useDtl;
			if (useDtl) {
				this.template = new dd.Template(template, true);
			} else {
				this.template = template; /*String || Object*/
			}
		} else {
			var r = processTemplate(template);
			this.useDtl = r.useDtl;
			if (this.useDtl) {
				this.template = new dd.Template(r.tpl, true);
			} else {
				this.template = r.tpl; /*String || Object*/
			}
		}
	}, {
		render: function(context, parent) {
			// summary:
			//    Renders this template as a gfx shape.
			// context: dojox.dtl.Context:
			//    The binding context.
			// parent: dojox.gfx.Surface || dojox.gfx.Group
			//    The gfx parent.
			// returns: dojox.gfx.Shape
			//    The shape corresponding to the template definition. 
			
			var v;
			if (this.useDtl) {
				v = this.template.render(context); /*String*/
			} else {
				v = this.template; /* String || object */
			}
			var o, bindings;
			if (typeof(v) == "string") {
				o = _toObject(v);
				if (this.useDtl) 
					bindings = this._reduceBindings(o);
			} else {
				o = v;
			}
			// _instantiate is actually ibm_ilog.diagram.templating.instantiate,
			// but we cannot depend in templating because it would cause a circular dependency,
			// The _instantiate function is added to GfxTemplate by the templating module. 
			var s = this._instantiate(parent, o, bindings);
			if (bindings && context.setBindings) 
				context.setBindings(bindings);
			return s;
		},
		
		_reduceBindings: function(o, bindings) {
			var bindings = bindings || [];
			for (var p in o) {
				if (p.indexOf("_binding_") !== -1) {
					var b = o[p];
					b.value = unescape(b.value);
					b.idx = bindings.length;
					var pos = b.srcPName.indexOf('.');
					if (pos != -1) {
						b.srcPName = b.srcPName.slice(pos + 1);
					}
					bindings.push(b);
				} else if (typeof(o[p]) == "object" || typeof(o[p]) == "function") {
					this._reduceBindings(o[p], bindings);
				}
			}
			return bindings;
		}
	});
	
	function _toObject(v){
		var filtered = [], c = 0;
		v = v.replace(/(:[\s]*")(\s*\b[^"]*)(")/g, function(m, p1, p2, p3){
			m = p2.replace(/\\/g, '\\\\').replace(/\n/g, '\\n');
			return p1 + m + p3;
		});
		v = v.replace(/(:[\s]*')(\s*\b[^']*)(')/g, function(m, p1, p2, p3){
			m = p2.replace(/\\/g, '\\\\').replace(/\n/g, '\\n');
			return p1 + m + p3;
		});
		return json.fromJson(v);
	}
	
	var DataContext = declare('ibm_ilog.diagram.data.DataContext', null, {
		constructor: function(item, store) {
			this.item = item;
			this.store = store;
		},
		
		getValue: function(/*String*/attribute, /*value?*/ defaultValue) {
			// summary:
			//     Returns a single attribute value. This method is a shortcut to
			//     this.store.getValue(item, attribute, defaultValue).
			
			if (this.store && this.item) {
				return StoreApi.getValue(this.store, item, attribute, defaultValue);
			}
			return null;
		},
		
		get: function(key) {
			// summary:
			//     Internal method. Should not be called directly.
			
			var store = this.store;
			var item = this.item;
			if (key == "getLabel") {
				return StoreApi.getLabel(store, item);
			} else if (key == "getAttributes") {
				return StoreApi.getAttributes(store, item);
			} else if (key == "getIdentity") {
				if (StoreApi.getFeatures(store).identity) {
					return StoreApi.getIdentity(store, item);
				}
				return "Store has no identity API";
			} else {
				var values = StoreApi.getValues(store, item, key);
				if (!values) {
					return;
				}
				if (!(values instanceof Array)) {
					return new DataContext(values, store);
				}
				
				values = array.map(values, function(value) {
					if ((typeof(value) == "object" || typeof(value) == "function") && StoreApi.isItem(store, value)) {
						return new DataContext(value, store);
					}
					return value;
				});
				values.get = idb._get;
				return values;
			}
		}
	});
	
	idb.DataContext = DataContext;
	
	DataContext.prototype.get.safe = true;
	
	return idb;
});

},
'ibm_ilog/diagram/BasicPort':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/BasicPort", ["dojo/_base/declare",
"./PortBase",
"./util/ErrorReporter"],
function(declare, PortBase, ErrorReporter){

/*=====
var PortBase = ibm_ilog.diagram.PortBase;
=====*/

var BasicPort = declare("ibm_ilog.diagram.BasicPort", PortBase, {
	// summary:
	//		The BasicPort class represents ports that define fixed connection points
	//		relative to the rectangular bounds of a node.
	//		The 'position' property is an object with 'x' and 'y' properties representing
	//		ratios (between 0 and 1) of the width and height of the node. For example,
	//		the position { x: 0.5, y: 0.5} defines a connection point at the center of a node, and
	//		the position { x: 1, y: 0 } defines a connection point at the top-right of a node.
	//		The 'offset' property (another x/y object, but in pixels this time) can be used
	//		to alter slightly this relative position.
	
	// Allows to test the class faster that testing declaredClass
	_isIBMDiagramBasicPort: true,
	
	_position: null,
	_offset: null,
	_movable: true,
	
    constructor: function() {
		// summary:
		//		Creates a new BasicPort with a default position set to the center of the node.
		// description:
		//		The default position is { x: 0.5, y: 0.5 }, and the default offset is { x: 0, y: 0 }.
		this._position = { x: 0.5, y: 0.5 };
		this._offset = { x: 0, y: 0 };
	},
	
	getPosition: function() {
		// summary:
		//		Gets the position of this port.
		// returns:
		//		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the node width/height.
		return this._position;
	},
	
	setPosition: function(position) {
		// summary:
		//		Sets the position of this port.
		// position:
		//		An object with 'x' and 'y' properties representing a ratio (between 0 and 1) of the node width/height.
		
		if(typeof(position) != 'object' || !('x' in position) || !('y' in position))
			ErrorReporter.error("UnexpectedArgument",this, "setPosition", "position", position);
		
		if (position.x != this._position.x ||
		    position.y != this._position.y) {
			this._position.x = position.x;
			this._position.y = position.y;
			this.invalidateLinks();
		}
	},
		
    isMovable: function() {
        // summary:
        //		Gets the movable state of this port.
        // returns:
        //		True if graph layout and user interaction are allowed to change the position 
	    //      of the port.
        //		False if graph layout and user interaction are not allowed to change the 
		//      position of the port.
        
		// AVA: the design of the class hierarchy for Ports is different in  
		// Diagram for Flex in the sense that the base class (PortBase) defines
		// a "move" function (default implementation is empty). Hence, the functions
		// "set/isMovable" plus the marker "supportsMove" are also defined at the
		// base class. This allows the code that uses the port to treat all ports in
		// a generic way, without the need to check the port class.
		// Here, the function which allows to move the port ("setPosition") exists
		// only on BasicPort, not on PortBase. Therefore, we cannot place "set/isMovable"
		// and "supportsMove" at PortBase. Not a big deal, but maybe it would be worth
		// to review the design.
		// With the current design, Diagram.flex' "supportsMove" makes no sense here,
		// therefore I only added "set/isMovable".  
        return this._movable ? true : false;
    },
    
    setMovable: function(m) {
        // summary:
        //		Sets the movable state of this port. The ports are movable by default.
        // m: Boolean:
        //		If 'm' is true, graph layout and user interaction are allowed to change the position 
	    //      of the port.
        //		If 'm' is false, graph layout and user interaction are not allowed to change the 
		//      position of the port.
        
        this._movable = m;
    },
	
	getOffset: function() {
		// summary:
		//		Gets the offset of this port.
		// returns:
		//		An object with 'x' and 'y' properties representing horizontal/vertical offsets in pixels.
		return this._offset;
	},
	
	setOffset: function(offset) {
		// summary:
		//		Sets the offset of this port.
		// offset:
		//		An object with 'x' and 'y' properties representing horizontal/vertical offsets in pixels.
		
		if(typeof(offset) != 'object' || !('x' in offset) || !('y' in offset))
			ErrorReporter.error("UnexpectedArgument",this, "setOffset", "offset", offset);
		
		if (offset.x != this._offset.x ||
		    offset.y != this._offset.y) {
			this._offset.x = offset.x;
			this._offset.y = offset.y;
			this.invalidateLinks();
		}
	},
	
	computeConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		// summary:
		//		Implements the calculation of the connection and origin points for the BasicPort class.
		// description:
		//		The connection point is computed based on the position and offset properties.
		//		The origin point is always the center of the node.
		
		var bounds = this.getConnectionBounds(parent);
		connectionPoint.x = bounds.x + bounds.width * this._position.x;
		connectionPoint.y = bounds.y + bounds.height * this._position.y;
		originPoint.x = bounds.x + bounds.width/2;
		originPoint.y = bounds.y + bounds.height/2;
		connectionPoint.x += this._offset.x;
		connectionPoint.y += this._offset.y;
		return connectionPoint;
	}
});

return BasicPort;

});


},
'ibm_ilog/diagram/widget/ZoomHudComponent':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/ZoomHudComponent", [
"../main",
"dojo/_base/declare",
"dojo/_base/connect",
"dojox/gfx/matrix",
"./HudComponent",
"../util/Geometry"
], function(
iid,
declare,
connect,
matrix,
HudComponent,
g
){

var Point = g.Point;

/*=====
var HudComponent = ibm_ilog.diagram.widget.HudComponent;
=====*/

var ZoomHudComponent =
	declare('ibm_ilog.diagram.widget.ZoomHudComponent', HudComponent, {
		//
		//	summary:
		//		A viewport component that displays the current zoom factor.
		//

		//
		//	_currentZoom:
		//		The current zoom factor being displayed at the component
		//
		_currentZoom: 0,

		constructor: function (viewport) {
			this._shape.setTransform(matrix.translate(iid._invisibleLocation, iid._invisibleLocation));
			this._shape.createRect({
				width: 30,
				height: 20
			}).setFill([0, 0, 0, 0.5]);
			this._shape.createText({
				x: 15,
				y: 15,
				align: "middle",
				text: "x" + g._round(this._zoom, 1)
			}).setFont({
				size: "10pt",
				family: "sans-serif"
			}).setStroke("white").setFill("white");
			this._updateLayout();
			this._updateValue();

			connect.connect(viewport, "onLayoutHud", this, "_updateLayout");
			connect.connect(viewport, "onViewRectChanged", this, "_updateValue");
			//connect.connect(viewport,"onClientSizeChanged",this,"_updateLayout");
		},

		_enable: function () {
			//
			//	summary:
			//			Enables the component and displays it.
			//
			this.inherited(arguments);
			this._updateLayout();
			this._updateValue();
		},

		_updateValue: function () {
			//
			//	summary:
			//		Updates the value displayed at the component
			//
			if (this._enabled && this._viewport.isHudEnabled() && (this._viewport.getZoom() !== this._currentZoom)) {
				this._currentZoom = this._viewport.getZoom();
				this._shape.children[1].setShape({
					text: "x" + g._round(this._currentZoom, 1)
				});
			}
		},

		_updateLayout: function () {
			//
			//	summary:
			//		Corrects the position of the component after a resize of the viewport.
			//
			if (this._enabled && this._viewport.isHudEnabled()) {
				var hudSize = this._viewport.getHudSize();
				var location;
				if(!this._viewport.isRtl()) {
					location = Point(hudSize.width - 30, 0);
				} else {
					location = Point(0,0);
				}
				this._shape.setTransform(matrix.translate(location));
			}
		}
	});

return ZoomHudComponent;

});

},
'ibm_ilog/diagram/adorners/TemplatedOverlay':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/adorners/TemplatedOverlay", [
"../main",
"dojo/_base/declare",
"dojox/gfx",
"../templating"
],
function(
iid,
declare,
gfx,
templating
){

/*=====
var templating = ibm_ilog.diagram.templating;
=====*/

var TemplatedOverlay =
    declare('ibm_ilog.diagram.adorners.TemplatedOverlay',[gfx.Group, templating._DiagramTemplated],{
    	
    	constructor: function() {
	        var id = iid._IdGenerator.nextId(this).toString();
	        this.getId = function() {
	            return id;
	        }
    	}

    });
    
return TemplatedOverlay;
	
});

},
'ibm_ilog/diagram/gfxlayout/StackLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout/StackLayout", ["dojo/_base/lang", "dojo/_base/declare", "./Layout"],
function(lang, declare, Layout){

/*=====
var Layout = ibm_ilog.diagram.gfxlayout.Layout;
=====*/

var StackLayout =
declare('ibm_ilog.diagram.gfxlayout.StackLayout', Layout, {
    // summary:
    //      A layout class that arranges children of a container as a horizontal or vertical stack.
    // description:
    //      The StackLayout class arranges children of a container as a horizontal or vertical stack
    //      depending on the value of its horizontal property. By default, a StackLayout has a vertical flow.
    //      An horizontal stack layout align its children vertically in the computed layout slot according to
    //      their 'valign' (one of the 'top', 'center', 'bottom' and 'stretch' values) and their 'margins' properties.
    //      Respectively, a vertical stack layout align its children horizontally in the computed layout slot according to
    //      their 'halign' (one of the 'left', 'center', 'right' and 'stretch' values) and their 'margins' properties.
    //      The StackLayout class defines the paddingLeft, paddingRight, paddingTop and paddingBottom properties
    //      to specify the internal margins of the panel from which the children are positioned. A convenient global padding
    //      property is also defined to set the same padding in all directions.
    //      Space between children can be adjusted using the gap property.
    
    // orientation
    horizontal: false,
    // padding
    padding: 0,
    paddingLeft: 0,
    paddingRight: 0,
    paddingTop: 0,
    paddingBottom: 0,
    // margin between items
    gap: 0,
    
    constructor: function(args) {
        if (args && 'padding' in args) {
            // you cannot set both padding and paddingXXX at the same time.
            this._setPadding(args.padding, false);
        }
    },
	
	getConfig: function() {
		// summary:
		// 		Gets the current configuration of this layout instance.
		// Description:
		//		This method returns a descriptor of this layout current configuration
		//		as well as the type information so that it can be passed to the Group.setLayout() method
		//		(for example to make a copy the layout of a Group). 
		// returns: Object
		//	An object that represents the layout current configuration.
		var arg = {};
		arg.type = 'ibm_ilog.diagram.gfxlayout.StackLayout';
		if (this.padding) 
			arg.padding = this.padding;
		else {
			arg.paddingLeft = this.paddingLeft;
			arg.paddingRight = this.paddingRight;
			arg.paddingTop = this.paddingTop;
			arg.paddingBottom = this.paddingBottom;
		}
		arg.gap = this.gap;
		arg.horizontal = this.horizontal;
		return arg; /*Object*/
	},
    
    setGap: function(/*Integer*/value) {
        // summary:
        //     Sets the items gap.
        //  value: Integer: the gap between each items. 
        if (value !== this.gap) {
            var old = this.gap;
            this.gap = value;
            this.onLayoutPropertyChanged('gap', old, value);
        }
    },
    
    setHorizontal: function(/*Boolean*/value) {
        // summary:
        //     Sets the stack orientation.
        // value: Boolean: true for an horizontal layout, false otherwise.
        
        if (this.horizontal !== value) {
            var old = this.horizontal;
            this.horizontal = value;
            this.onLayoutPropertyChanged('horizontal', old, value);
        }
    },
    
    setPadding: function(value) {
        // summary:
        //     Sets the padding of this layout.
        // description:
        //     This property is used to set the internal margins of this layout.
        //     The value can either be a 4-elements array whose items match
        //     respectively the left, top, right and bottom paddings ; or one integer
        //     that is applied to all paddings.
        this._setPadding(value, true);
    },
    _setPadding: function(value, notify) {
        if (value instanceof Array) {
            var l = value.length, oldL = this.paddingLeft, oldR = this.paddingRight, oldT = this.paddingTop, oldB = this.paddingBottom;
            if (l > 0) 
                this.paddingLeft = value[0];
            if (l > 1) 
                this.paddingTop = value[1];
            if (l > 2) 
                this.paddingRight = value[2];
            if (l > 3) 
                this.paddingBottom = value[3];
            if (notify &&
            (this.paddlingLeft !== oldL ||
            this.paddingTop !== oldT ||
            this.paddingBottom !== oldB ||
            this.paddingRight !== oldR)) {
            
                this.onLayoutPropertyChanged();
            }
        } else {
            var old = this.padding;
            this.padding = this.paddingLeft = this.paddingRight = this.paddingBottom = this.paddingTop = value;
            if (notify && (value !== old)) {
                this.onLayoutPropertyChanged();
            }
        }
    },
    
    adjustBBox: function(/*dojox.gfx.Rectangle*/bbox) {
        // summary:
        //     Returns the container bbox adjusted according to the constraints of this layout.
        // bbox: Rectangle: the bounding box of the associated container.
        
        bbox.x -= this.paddingLeft;
        bbox.y -= this.paddingTop;
        bbox.width += this.paddingLeft + this.paddingRight;
        bbox.height += this.paddingTop + this.paddingBottom;
        return bbox;
    },
    
    computePreferredSize: function(/*Size*/proposedSize) {
        // summary:
        //     Computes the preferred size of the associated container according to the layout strategy.
        
        var csz = {}, children = this._container.children, count = children.length, maxW = 0, maxH = 0, i, bounds = [];
        
        for (i = 0; i < count; ++i) {
            var s = children[i];
            var sb = s.computePreferredSize(proposedSize);            
			if (s.margins) {
                sb.width = this._inflateH(sb.width, s.margins);
                sb.height = this._inflateV(sb.height, s.margins);
            }
            if (!s.hfiller && sb.width > maxW) 
                maxW = sb.width;
            if (!s.vfiller && sb.height > maxH) 
                maxH = sb.height;
            bounds.push(lang.clone(sb));
        }
        
        if (proposedSize.width !== -1 && proposedSize.height !== -1) {
            return proposedSize;
        }
        if (proposedSize.height !== -1) 
            maxH = proposedSize.height;
        if (proposedSize.width !== -1) 
            maxW = proposedSize.width;
        var lastCoord = 0;
        if (this.horizontal) {
            for (i = 0; i < count; ++i) {
                if (children[i].hfiller)
					continue;
                var sb = bounds[i];
				lastCoord += sb.width;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
            csz.width = lastCoord;
            csz.height = maxH;
        } else {
            for (i = 0; i < count; ++i) {
                if (children[i].vfiller)
					continue;
                var sb = bounds[i];
                lastCoord += sb.height;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
            csz.height = lastCoord;
            csz.width = maxW;
        }
        csz.width += this.paddingLeft + this.paddingRight;
        csz.height += this.paddingTop + this.paddingBottom;
        return csz;
    },
    
    layoutChildren: function(/*Size*/contBounds) {
        // summary:
        //     Lays out the children of the associated container.
        
        var children = this._container.children, count = children.length, i;
        contBounds = lang.clone(contBounds);
        
        contBounds.width = Math.max(0, contBounds.width - this.paddingLeft - this.paddingRight);
        contBounds.height = Math.max(0, contBounds.height - this.paddingTop - this.paddingBottom);
        var lastCoord;
        if (this.horizontal) {
            lastCoord = /*contBounds.x +*/ this.paddingLeft;
            for (i = 0; i < count; ++i) {
                var s = children[i];
                var sb = s.getPreferredSize();
                var slot = {};
                slot.x = lastCoord;
                slot.y = /*contBounds.y + */ this.paddingTop;
                slot.height = contBounds.height;
				slot.width = sb.width;
                if (s.margins) 
                    slot.width = this._inflateH(slot.width, s.margins);
                // the elements in the stack must be clipped 
				if (slot.width + lastCoord > contBounds.width)
					slot.width = contBounds.width - lastCoord;
                this.placeChildInSlot(s, slot, false, true);
                lastCoord += slot.width;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
        } else {
            lastCoord = /*contBounds.y +*/ this.paddingTop;
            for (i = 0; i < count; ++i) {
                var s = children[i];
                var sb = s.getPreferredSize();
                var slot = {};
                slot.x = /*contBounds.x +*/ this.paddingLeft;
                slot.y = lastCoord;
                slot.width = contBounds.width;
                slot.height = sb.height;
                if (s.margins) 
                    slot.height = this._inflateV(slot.height, s.margins);
                // the elements in the stack must be clipped 
				if (slot.height + lastCoord > contBounds.height)
					slot.height = contBounds.height - lastCoord;
                this.placeChildInSlot(s, slot, true, false);
                lastCoord += slot.height;
                if (i < count - 1) 
                    lastCoord += this.gap;
            }
        }
    }
});

return StackLayout;

});

},
'ibm_ilog/diagram/Selection':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/Selection", [
"dojo/_base/declare", "dojo/_base/lang", "dojo/_base/array", "dojo/_base/connect",
"dojox/collections/ArrayList",
"./Selectable",
"./util/GraphUtil", "./util/ErrorReporter"],
function(declare, lang, array, connect, ArrayList, Selectable, gu, R){

var Selection = 
declare("ibm_ilog.diagram.Selection", [], {
	
	_diagram: null,
	_selectionStrategy: null,
	_multiSelectionEnabled: null,
	_nodeSelectionEnabled: null,
	_linkSelectionEnabled: null,
	
	constructor: function(diagram,options) {
		// summary:
		//		creates a new instance
		this._multiSelectionEnabled = true;
		this._nodeSelectionEnabled = true;
		this._linkSelectionEnabled = true;
		this._diagram = diagram;	
		options = options || diagram || {selectionMode:Selection.standard};
		this.instantiateSelectionStrategy(diagram,options);
	},
	
	instantiateSelectionStrategy: function(diagram,options){
		var useIndividualConnect = !diagram!=null;
		
		switch( options.selectionMode){
			case Selection.standard:{
				if(diagram && diagram.useDataStore()){
					this._selectionStrategy = new StoreSelectionStrategy(this,useIndividualConnect);
				}else{
					this._selectionStrategy = new GFXSelectionStrategy(this,useIndividualConnect);
				}
				this.setMultiSelectionEnabled(true);
				break;}
			case Selection.gfx_single: {
				this._selectionStrategy = new GFXSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(false);
				break;}
			case Selection.gfx_multi: {
				this._selectionStrategy = new GFXSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(true);
				break;}
			case Selection.dataStore_single: {
				this._selectionStrategy = new StoreSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(false);
				break;}
			case Selection.dataStore_multi: {
				this._selectionStrategy = new StoreSelectionStrategy(this,useIndividualConnect);
				this.setMultiSelectionEnabled(true);
				break;}		
		}
		if(!useIndividualConnect){
			connect.connect(diagram,'beforeGraphElementDispose',this,'onElementDisposed');
		}
		
	},
	add: function(elements,forceClear){
		// summary:
		// 		sets the desired as selected, and add it to the selected list (if not in
		// 		the list yet). Also connect the element to react to the selectableChanged method
		// elements: a single element or an array of elements
		var removed = [];
		if((!this._multiSelectionEnabled && this.get().count>0) || forceClear == true){
			removed = this._clear();
		}
		if(!(elements instanceof Array)){
			elements = [elements];
		}
		var added = [];
		for( itemIndex in elements ){ 
		 		var item = elements[itemIndex];
				var rta = this._selectionStrategy.add(item);
				if(rta){
					added.push(rta);
				}		
		 }
		
		if(added.length>0 || removed.length>0){
			this.onSelectionChanged(added,removed);
			return true;
		}else{
			return false;
		}
	},
	remove: function(elements,isDisposed){
		// summary:
		// 		sets the desired as unselected, and remove it from the selected list (if in
		// 		the list).
		// elements: a single element or an array of elements
		if(!(elements instanceof Array)){
			elements = [elements];
		}
		var removed = [];
		for( itemIndex in elements ){ 
		 		var item = elements[itemIndex];
				var rta = this._selectionStrategy.remove(item,isDisposed);
				if(rta){
					removed.push(item);
				}		
		 }
		
		if(removed.length>0){
			!isDisposed && this.onSelectionChanged([],removed);
			return true;
		}else{
			return false;
		}
	},
	_clear: function(){
		// summary:
		// sets the selection state of all the elements in the selected list as
		// false, and clear the list. Return an array with the removed elements
		var removed = this._selectionStrategy.get().toArray();
		this._selectionStrategy.clear();
		return removed;
	},
	clear: function() {
		// summary:
		// sets the selection state of all the elements in the selected list as
		// false, and clear the list. If any element is removed from the selection, a notification is thrown.
		var removed = this._clear();
		if(removed.length>0){
			this.onSelectionChanged([],removed);
		}
	},
	get: function(){
		// summary:
		// gets the ArrayList of selected elements
		return this._selectionStrategy.get().clone();
	},
	fastGet: function(){
		// summary:
		// gets the ArrayList of selected elements
		return this._selectionStrategy.get();
	},
	onSelectionChanged: function(added,removed){
		
	},
	getMultiSelectionEnabled: function(){
		// summary:
		// gets if multi-Selection is enabled (true by default)
		return this._multiSelectionEnabled;
	},
	setMultiSelectionEnabled: function(enabled){
		// summary:
		// sets if multi-Selection is enabled (true by default)
		this._multiSelectionEnabled = enabled;
	},
	getNodeSelectionEnabled: function(){
		// summary:
		// gets if Node Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		return this._nodeSelectionEnabled;
	},
	setNodeSelectionEnabled: function(enabled){
		// summary:
		// sets if node Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		this._nodeSelectionEnabled = enabled;
	},
	getLinkSelectionEnabled: function(){
		// summary:
		// gets if link Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		return this._linkSelectionEnabled;
	},
	setLinkSelectionEnabled: function(enabled){
		// summary:
		// sets if link Selection is enabled (true by default)
		// this enablement only affects the accept(graphelement) return value.
		this._linkSelectionEnabled = enabled;
	},
	_getDataItem: function(element){
		if(Selectable.Util.isSelectableInstance(element)){
			return this._diagram.getBoundItem(element);
		}else{
			return element;
		}
	},
	_asGraphElement: function(element){
		if(!Selectable.Util.isSelectableInstance(element)){
			return this._diagram.getGraphElementFromDataItem(element);
		}else{
			return element;
		}
	},
	accept: function(element){
		// Summary:
		//		This method return true if the element can be added to the selection 
		//		eg1: an element with no data item cannot be added to a StoreSelectionStrategy
		//		eg: a link is trying to be added to the selection but link selection is disabled
		var ge = this._asGraphElement(element);
		return ((ge._isIBMDiagramNode && this._nodeSelectionEnabled) || (ge._isIBMDiagramLink && this._linkSelectionEnabled)) && this._selectionStrategy.accept(element);
	},
	onElementDisposed: function(graphElement) {
		this.remove(graphElement,true);
	},
	selectAll: function(graph,options) {
		//
		//	summary: 
		//		select all elements in a graph 
		//	graph:
		//		the graph containing all the items to be selected
		//	options:
		//		options.noCheck: if true, skip checking each item is accepted by the receiver
		//
		graph = graph || this._diagram.getGraph();
		
		var selectables = gu.allGraphElements(graph.children).toArray()
	
		if(!options || !options.noCheck) {
			selectables = array.filter(selectables,lang.hitch(this,"accept"));
		}
		this.add(selectables,true);
	},	
	__eod: undefined
});

var AbstractSelectionStrategy =
declare("ibm_ilog.diagram.AbstractSelectionStrategy", [], {
	_selectedHandle: null,
	_selectedElements: null,
	_selection: null,
	_individualConnect: null,
	
	constructor: function(selection,useIndividualConnect) {
		this._selection = selection;
		this._individualConnect = useIndividualConnect;
		this._selectedElements = new ArrayList();
		if(this._individualConnect){
			this._selectedHandle = new ArrayList();
		}
	},
	add: function(element){
	
	},
	remove: function(element,isDisposed){
		
	},
	disposeRemove: function(element){
		
	},
	get: function(){
		return this._selectedElements;
	},
	clear: function(){
		while(this._selectedElements.count>0){
			var element = this._selectedElements.item(this._selectedElements.count-1);
			this.remove(element);
		}
	},
	_addElement: function(element){
		this._selectedElements.add(element);
		var ge = this._selection._asGraphElement(element);
		if (this._individualConnect) {
			this._selectedHandle.add(connect.connect(ge, 'beforeDisposing', this._selection, 'onElementDisposed'));
		}
	},
	_removeElement: function(element){
		// summary:
		//		remove the elements from the list, disconnect it and remove the handle.
			var index =  this._selectedElements.indexOf(element);
			this._selectedElements.removeAt(index);
			if (this._individualConnect) {
				connect.disconnect(this._selectedHandle.item(index));
				this._selectedHandle.removeAt(index);
			}
	}
});

/*=====
var AbstractSelectionStrategy = ibm_ilog.diagram.AbstractSelectionStrategy;
=====*/

var GFXSelectionStrategy =
declare("ibm_ilog.diagram.GFXSelectionStrategy", [AbstractSelectionStrategy], {
	add: function(element){
		var ge = this._selection._asGraphElement(element);
		if(!this._selectedElements.contains(ge)){
			if(ge.setSelected(true)){
				this._addElement(ge);
				return ge;
			}
		}
		return null;
	},
	remove: function(element,isDisposed){
		var ge = this._selection._asGraphElement(element);
		if (this._selectedElements.contains(ge)) {
			if(!isDisposed){
				ge.setSelected(false);
			}
			this._removeElement(element);
			return ge;
		}
		return null;
	},
	accept: function(element){
		// Summary:
		//		This method return true if the element can be added to the selection 
		//		(eg: an element with no data item cannot be added to a StoreSelectionStrategy)
		
		return true;
	}
});

var StoreSelectionStrategy =
declare("ibm_ilog.diagram.StoreSelectionStrategy", [AbstractSelectionStrategy], {
	add: function(element){
		var ge = this._selection._asGraphElement(element);
		var dataItem = this._selection._getDataItem(element);
		if(dataItem == null){
			R.error("NotInDataStore");
		}
		if(!this._selectedElements.contains(dataItem)){
			if(ge == null || ge.setSelected(true)){
				this._addElement(dataItem);
				var dataItem = this._selection._getDataItem(element);
				return dataItem;	
			}
		}
		return null;
	},
	remove: function(element,isDisposed){
		var ge = this._selection._asGraphElement(element);
		var dataItem = this._selection._getDataItem(element);
		if(dataItem == null){
			R.error("NotInDataStore");
		}
		if (this._selectedElements.contains(dataItem)) {
			if (ge != null && !isDisposed) {
				ge.setSelected(false);
			}
			this._removeElement(dataItem);
			return dataItem;
		}
		return null;
	},
	accept: function(element){
		// Summary:
		//		This method return true if the element can be added to the selection 
		//		(eg: an element with no data item cannot be added to a StoreSelectionStrategy)
		
		var dataItem = this._selection._getDataItem(element);
		if(dataItem == null){
			return false;
		}
		return true;
	}
});

Selection.standard = 0;
Selection.dataStore_single = 1;
Selection.dataStore_multi = 2;
Selection.gfx_single = 3;
Selection.gfx_multi = 4;

return Selection;

});

},
'ibm_ilog/diagram/gfxlayout/FillLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout/FillLayout", ["dojo/_base/declare", "./Layout"],
function(declare, Layout){

/*=====
var Layout = ibm_ilog.diagram.gfxlayout.Layout;
=====*/

var FillLayout =
declare('ibm_ilog.diagram.gfxlayout.FillLayout', Layout, {
	
	getConfig:function(){
		// summary:
		// Gets the current configuration of this layout instance.
		// returns: Object
		//	An object that represents the layout current configuration.
		return {type:'ibm_ilog.diagram.gfxlayout.FillLayout'}; /*Object*/
	},

    computePreferredSize: function(proposedSize) {
        var children = this._container.children, count = children.length, o, i, maxW = 0, maxH = 0;
        for (i = 0; i < count; ++i) {
            if (children[i].filler) 
                continue;
            var sz = children[i].computePreferredSize(proposedSize);
            if (sz.width > maxW) 
                maxW = sz.width;
            if (sz.height > maxH)
                maxH = sz.height;
        }
        return {
            width: proposedSize.width === -1 ? maxW : proposedSize.width,
            height: proposedSize.height === -1 ? maxH : proposedSize.height
        }
    },
    
    layoutChildren: function(contBounds) {
        var children = this._container.children, count = children.length, o, i;
        for (i = 0; i < count; ++i) {
            children[i].layout(contBounds);
        }
    }
});

return FillLayout;

});

},
'ibm_ilog/diagram/gfxlayout/_base':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout/_base", ["../main",
"dojo/_base/lang", "dojo/_base/config", "dojo/_base/sniff", "dojo/dom-style",
"dojox/gfx", "dojox/gfx/matrix",
"../_base/gfxext", "../_base/diagext",
"../util/Geometry", "../util/ErrorReporter",
"dojo/has!config-isBidiInDojoDiagrammer?../util/BidiUtil:"],
function(iid, lang, config, has, domstyle, gfx, matrix, gfxext, diagext, Geometry, ErrorReporter, BidiUtil){

	// TODO: Change to this when AMD conversion is complete:
	// var gfxlayout = {};
	var gfxlayout = lang.getObject("ibm_ilog.diagram.gfxlayout", true);
	
	// ensures the explicitSize is updated when the font is changed.
	gfxext.patchMethod(gfx.Text, "setFont", null, function(){
		this.__shapeSet();
	});
	
	// Returns the size of the given string when rendered by the specified Text shape.
	var getTextSize = function(/*dojox.gfx.Text*/sh, /*String*/ txt){
		if (!txt || txt.length == 0) 
			return null;
		if (iid.isSvg && !iid.isSvgWeb) {
			if (!sh.rawNode.firstChild) 
				return null;
			//Fix for svg in FF. When the text starts with lamed, tzadik, may be other letters, the text width after Bidi conversion is smaller, than 
			//the width of the text before convertion, we have to apply Bidi conversion on the text before measurement, to prevent 
			//additional truncation
			if (config.isBidiInDojoDiagrammer && sh.getShape().__beforeBidi && has("ff")) {
				var shape = lang.clone(sh.getShape());
				shape.text = txt;
				shape = BidiUtil.bidiFormat.call(sh, shape);
				txt = shape.text;
			}
			// summary: get the text width in pixels
			var rawNode = sh.rawNode, oldParent = rawNode.parentNode, _measurementNode = rawNode.cloneNode(true);
			_measurementNode.style.visibility = "hidden";
			// solution to the "orphan issue" in FF
			var _height = 0, _width = 0, _x = 0, _y = 0;
			_measurementNode.firstChild.nodeValue = txt;
			oldParent.appendChild(_measurementNode);
			try {
				var bbox = _measurementNode.getBBox();
				_width = parseInt(bbox.width, 10);
				_height = parseInt(bbox.height, 10);
							
			} catch (e){return null;}
			oldParent.removeChild(_measurementNode);
			return {
				width: _width,
				height: _height
			};
		} else if (iid.isVml) {
			var oldShape = sh.getShape();
			
			var rawNode = sh.rawNode, _display = rawNode.style.display;
			sh._suspendInvalidate = true;
			sh.setShape({
				text: txt
			});
			sh.getTextWidth();
			var _width = gfx.pt2px(parseFloat(rawNode.currentStyle.width));
			var _height = gfx.pt2px(parseFloat(rawNode.currentStyle.height));
			// in VML, the width/height we get are in view coordinates
			var m = sh._getRealMatrix();
			if (m && !m.isIdentity()) {
				var sz = {
					x: 0,
					y: 0,
					width: _width,
					height: _height
				};
				sz = m.inverse().transformRectangle(sz);
				_width = sz.width;
				_height = sz.height;
			}
			sh.setShape(oldShape);
			sh._suspendInvalidate = false;
			return {
				width: _width,
				height: _height
			};
			
			/*
			 var rawNode = sh.rawNode, oldParent = rawNode.parentNode,
			 _measurementNode = rawNode.cloneNode(true);
			 oldParent.appendChild(_measurementNode);
			 _measurementNode.style.display = "inline";
			 // find text path
			 var t = null, c = _measurementNode.childNodes;
			 for(var i = 0; i < c.length; ++i){
			 var tag = c[i].tagName;
			 if(tag == "textpath"){
			 t = c[i];
			 }
			 }
			 t.string = txt;
			 t.on = true;
			 
			 
			 var bcr = _measurementNode.getBoundingClientRect();
			 console.log((bcr.right - bcr.left) + " " + (bcr.bottom - bcr.top));
			 return {x:bcr.left, y:bcr.top, width: bcr.width || (bcr.right - bcr.left), height: bcr.height || (bcr.bottom - bcr.top)};
			 */
		} else if (iid.isSilverlight) {
			var oldtext = sh.rawNode.text;
			sh.rawNode.text = txt;
			var bb = {
				width: sh.rawNode.actualWidth,
				height: sh.rawNode.actualHeight
			};
			sh.rawNode.text = oldtext;
			return bb;
		} else if (sh.getTextWidth) {
			if(iid.isSvgWeb && !sh.parent)
				return null;
			// this works for Canvas
			var oldtext = sh.shape.text;
			sh.shape.text = txt;
			var w = sh.getTextWidth();
			sh.shape.text = oldtext;
			var font = sh.getFont();
			var fz = font ? font.size : gfx.defaultFont.size;
			var h = gfx.normalizedLength(fz);
			return {
				width: w,
				height: h
			};
		}
	};
	
	//
	// An extension to gfx Shape classes to add features required by the layout system.
	//
	gfxlayout._LayoutShapeExt = {
	
		_explicitSize: null,
		
		// Called when a setShape (or any methods that modify the bbox) is invoked.
		__shapeSet: function(){
			if (this._suspendInvalidate) {
				return;
			}
			// getBoundingBox may return null (for ex when the Text is not in the hierarchy yet)
			var bbox = this.getBoundingBox();
			// in Chrome, a (0,0,0,0) bbox is returned for undetermined node state instead of null.
			// So we treat such a bbox as null to force the _expliciteSize computation later
			// (see defect 2105)
			if (bbox && bbox.x === 0 && bbox.y === 0 && bbox.width === 0 && bbox.height === 0)
				bbox = null;
			this._explicitSize = bbox;
		},
		
		getExplicitSize: function(){
			// summary:
			//     Returns the explicit size of this shape.
			// description: 
			//     This method returns the size as computed by the getBoundingBox() method
			//     of the shape explicit geometry (that is, the geometry explicitely set and
			//     not the geometry computed by the layout).
			return this._explicitSize;
		},
		
		getPreferredSize: function(){
			// summary:
			// returns the preferred size as computed by the computePreferredSize() method.
			
			return {
				width: this._preferredSize.width,
				height: this._preferredSize.height
			};
		},
		
		computePreferredSize: function(/*Size*/proposedSize){
			// summary:
			//     Computes the preferred size of this shape according to the specified proposed size,
			//     and ensures it is conformed with the minimum and maximum size of this shape.
			// proposedSize: Size: the proposed size.        
			
			var sz = this._computePreferredSize(proposedSize);
			this._preferredSize = this._checkMinMaxSize(sz);
			return {
				width: this._preferredSize.width,
				height: this._preferredSize.height
			};
		},
		
		/*protected*/
		_computePreferredSize: function(/*Size*/proposedSize){
			// summary:
			//     Computes the preferred size of this shape. 
			// description:
			//     This method is invoked by the public computePreferredSize() method. Subclasses should override
			//     this method for specific implementation.
			
			var rect = this._explicitSize;
			if (!rect) 
				rect = this._explicitSize = this.getBoundingBox();
			if (!rect) 
				rect = Geometry.EmptyRect;
			var t = this.getTransform();
			if (t) {
				// do not modify rect as it breaks under IE9
				var rt = {
					x: rect.x || 0,
					y: rect.y || 0,
					width: rect.width,
					height: rect.height
				}
				rect = t.transformRectangle(rt);
			}
			return {
				width: rect.width,
				height: rect.height
			};
		},
		
		_checkMinMaxSize: function(/*Size*/bounds){
			// summary:
			//     Ensures the specified bounds fits the minimum and maximum size.
			
			return this._checkMaxSize(this._checkMinSize(bounds));
		},
		
		_checkMaxSize: function(bounds){
			// summary:
			//     Ensures the specified bounds fits the maximum size.
			
			var sz = this.maximumSize;
			if (sz && sz.height) {
				bounds.height = Math.min(sz.height, bounds.height);
			}
			if (sz && sz.width) {
				bounds.width = Math.min(sz.width, bounds.width);
			}
			return bounds;
		},
		
		_checkMinSize: function(bounds){
			// summary:
			//     Ensures the specified bounds fits the minimum size.
			
			var sz = this.minimumSize;
			if (sz && sz.height) {
				bounds.height = Math.max(sz.height, bounds.height);
			}
			if (sz && sz.width) {
				bounds.width = Math.max(sz.width, bounds.width);
			}
			return bounds;
		},
		
		//_suspendInvalidate: false,
		_suspendInvalidate: 0,
		
		suspendInvalidate: function(){
			// summary:
			//    Suspends the invalidate propagation to the parent hierarchy.
			
			//this._suspendInvalidate = true;
			++this._suspendInvalidate;
		},
		
		resumeInvalidate: function(/*Boolean*/invalidate){
			// summary:
			//     Resumes the invalidate propagation to the parent hierarchy.
			// invalidate: Boolean: Indicates whether a call to invalidate() should be done.
			
			//this._suspendInvalidate = false;
			
			--this._suspendInvalidate;
			if (this._suspendInvalidate < 0) {
				this._suspendInvalidate = 0;
				// TODO: throw Error
			}
			if (invalidate && !this._suspendInvalidate) 
				//if (invalidate) 
				this.invalidate();
		},
		
		invalidate: function(){
			// summary:
			//     Invalidates this shape.
			// description:
			//     Call this method when a change occurs on this shape that could impact its
			//     bounding box. The default implementation propagates the invalidation to
			//     the parent hierarchy.
			if (this._suspendInvalidate) 
				return;
			var p = this.getParent();
			if (p && p.invalidate) {
				// indicates the size must be recomputed for all the parent layout chain
				p.invalidate();
			}
		},
		
		layout: function(/*Rectangle*/rect){
			// summary:
			//     Lays out this shape into the specified rectangle.
			
			try {
				this.suspendInvalidate();
				this.applyLayout(rect);
			} finally {
				this.resumeInvalidate();
			}
		},
		
		/*protected*/
		applyLayout: function(/*Rectangle*/rect){
			// summary:
			//     Invoked by the public layout() method to layout this shape into this specified rect.
			//     Subclasses should override this method to implement their own logic. The default
			//     implementation computes the transform needed to fit the specified rect.
			var t = this.getTransform() || matrix.identity;
			// 'disable' the transform set previously by the layout
			// so here, t is all the transforms except the one from the layout
			if (this.__oldLayoutTransform && !this.__oldLayoutTransform.isIdentity()) {
				t = matrix.multiply(this.__oldLayoutTransform.inverse(), t);
			}
			var bbox = this.getBoundingBox();
			if (t && bbox) 
				bbox = t.transformRectangle(bbox);
			if (bbox) {
				this.__oldLayoutTransform = matrix.createFromRectangles(bbox, rect);
				t = matrix.multiply(this.__oldLayoutTransform,t);
				this.setTransform(t);
			} else {
				var dx = rect.x, dy = rect.y;
				if (dx != 0 || dy != 0) {
					this.__oldLayoutTransform = matrix.translate(dx, dy);
					t = matrix.multiply(this.__oldLayoutTransform, t);
					this.setTransform(t);
				} else {
					this.__oldLayoutTransform = null;
				}
				
			}
		}
	};
	
	lang.extend(gfx.shape.Shape, gfxlayout._LayoutShapeExt);
	
	//
	// A fix for missing gfx.Text.getBoundingBox
	//
	
	gfxlayout._ellipsis = String.fromCharCode(8230);
	
	var _truncate = function(txt, width){
		var t = txt, c = txt.length, tw = getTextSize(this, t);
		tw = tw ? tw.width : 0;
		while (c > 0 && tw > width) {
			// drop 2 chars
			c -= 2;
			// and count ellipsis
			t = txt.slice(0, c) + gfxlayout._ellipsis;
			tw = getTextSize(this, t);
			tw = tw ? tw.width : 0;
		}
		// t='...' and still > width
		if (tw > width) 
			t = '';
		return t;
	};
	
	lang.extend(gfx.Text, {
	
		_truncateMode: 'default',
		
		_truncatesFn: _truncate,
		
		setTruncateMode: function(mode){
			// summary:
			// 		Sets the method used to truncate the text.
			// description:
			//		This method is called by the applyLayout() function when the text set on this shape
			//		is wider than the available space so that it fits into the specified space. By default,
			//		the text is truncated with ellipsis as much as needed to fit the available width. You can
			//		specify a custom function or disable the behavior. The custom Function should obey the
			//		following specification:
			//		function foo(text, width) {
			//			// summary: Returns the new text.
			//			// text: String : the text to truncate.
			//			// width: Number: the slot width.
			//			// returns : the truncated text.
			//		}
			// mode: null||'default'||Function
			//		null to disable, 'default' to enable the default mode, or a Function for a custom implementation.
			
			if (mode != this._truncateMode) {
				var old = this._truncateMode;
				this._truncateMode = mode;
				if (!mode) {
					this._truncatesFn = null;
				} else if (mode == 'default') {
					this._truncatesFn = _truncate;
				} else if (typeof(mode) == "function") {
					this._truncatesFn = mode;
				}
				this.invalidate();
			}
		},
		
		getTruncateMode: function(){
			// summary:
			//		Returns the truncate mode.
			
			return this._truncateMode;
		},
		
		applyLayout: function(rect){
			// fix for #1855: on IE, when the text is changed, setShape calls setTransform internally,
			// this will trigger a layout before __setShape was called, thus before ___realtext was set to the new text.
			// So, to know if __realtext is valid, we also check the sh.text against a __disptext (displayed text) property,
			// that we set (below in this method) to the actually displayed (truncated) text.
			var sh = this.getShape(), nsh = {}, txt = "";
			//fix for regression bugs #3282 and 3287, bidi defects, on SVG (FF) and Silverlight, (not VML), the bidi text in subgraph displayed reversed
			//if the text was Bidi transformed,  for truncation, we should use the text before transformation (__realtext, but __disptext could not be set yet)
			if (config.isBidiInDojoDiagrammer && sh.__beforeBidi && (iid.isSilverlight || (iid.isSvg && has("ff")))) {
				txt = this.__realtext ? this.__realtext : sh.text;
    		} else {		
			    txt = (this.__realtext && this.__disptext == sh.text) ? this.__realtext : sh.text;
			}
			var sz = getTextSize(this, txt), sbbox = this.getBoundingBox();
			// if bbox is null (means not in the dom or hidden)
			// do nothing
			if (!sbbox) 
				return;
			
			// If the text is transformed, first apply the inverse transform to the rect:
			var tr = this.getTransform();
			if (tr && !tr.isIdentity()) 
				rect = tr.inverse().transformRectangle(rect);
			
			// Compute the text origin
			
			var h = sz.height, w = sz.width;
			
			switch (sh.align) {
				case 'start':
					nsh.x = rect.x;
					break;
				case 'middle':
					nsh.x = rect.x + rect.width / 2;
					break;
				case 'end':
					nsh.x = rect.x + rect.width;
			}
			// Patch for Canvas: if dir="rtl", text is displayed rtl relative to the align position
			// (I guess maybe SVG should do the same, but the fact is that it does not.
			//  Maybe one day we'll need to do the same for SVG?...)
			if(iid.isCanvas && (sh.align == 'start' || sh.align == 'end')){
				// go up to the GFX Surface to find a DOM node
				for(var p = this.getParent(); p; p = p.getParent()){
					if(p.declaredClass == gfx.Surface.prototype.declaredClass){
						// Look at the computed 'direction' style,
						// seems the easier way to get the inherited direction value...
						var style = domstyle.getComputedStyle(p.rawNode);
						if(style && style.direction == "rtl"){
							if(sh.align == 'start')
								nsh.x += rect.width;
							else
								nsh.x -= rect.width;
						}
						break;
					}
				}
			}
			var ascent = sh.y - sbbox.y;
			nsh.y = rect.y + (rect.height - h) / 2 + ascent;
			
			// check if text is larger than the given rect. 
			// In this case: cut the text and add ellipsis
			if (this._truncatesFn && txt && txt.length > 0 && Math.round(rect.width) > 0 && w > rect.width) {
				txt = this._truncatesFn.call(this, txt, rect.width);
			}
			nsh.text = txt;
			this.__disptext = txt;
			if (config.isBidiInDojoDiagrammer) {
				if (sh.textDir) 
					nsh.textDir = sh.textDir;
				if (sh.ceType) 
					nsh.ceType = sh.ceType;
			}
			this.__setShapeInt = true;
			try {
				this.setShape(nsh);
			} finally {
				this.__setShapeInt = false;
			}
		},
		
		__shapeSet: function(callee){
			if (!this.__setShapeInt && callee === 'setShape') {
				if (config.isBidiInDojoDiagrammer) {
					var tt = this.getShape().__beforeBidi;
					this.__realtext = tt ? tt : this.getShape().text;
				} else 
					this.__realtext = this.getShape().text;
			}
			
			// under IE, we need to reset the explicitSize, as the text bbox may be computed
			// in the createShape() call, and at this time, the _suspendInvalidate is not called yet
			// and the node might be in an incomplete state.	
			if (iid.isVml && this._suspendInvalidate) {
				this._explicitSize = null;
				return;
			}
			gfxlayout._LayoutShapeExt.__shapeSet.call(this);
		}
	});
	
	lang.extend(gfx.Rect, {
		applyLayout: function(rect){
			var t = this.getTransform();
			if (t && !t.isIdentity()) {
				//TODO: if t is translation:=> do a resize instead
				var oldBounds = this.getBoundingBox();
				oldBounds = t.transformRectangle(oldBounds);
				this.applyLeftTransform(matrix.createFromRectangles(oldBounds, rect));
			} else {
				this.setShape(rect);
			}
		}
	});
	
	lang.extend(gfx.Ellipse, {
		applyLayout: function(rect){
			var t = this.getTransform();
			//TODO: if t is translation:=> do a resize instead
			if (t && !t.isIdentity()) {
				var oldBounds = this.getBoundingBox();
				oldBounds = t.transformRectangle(oldBounds);
				this.applyLeftTransform(matrix.createFromRectangles(oldBounds, rect));
			} else {
				var sh = {
					cx: rect.x + rect.width / 2,
					cy: rect.y + rect.height / 2,
					rx: rect.width / 2,
					ry: rect.height / 2
				};
				this.setShape(sh);
			}
		}
	});
	
	lang.extend(gfx.Circle, {
		applyLayout: function(rect){
			var t = this.getTransform();
			//TODO: if t is translation:=> do a resize instead
			if (t && !t.isIdentity()) {
				var oldBounds = this.getBoundingBox();
				oldBounds = t.transformRectangle(oldBounds);
				this.applyLeftTransform(matrix.createFromRectangles(oldBounds, rect));
			} else {
				var sh = {
					cx: rect.x + rect.width / 2,
					cy: rect.y + rect.height / 2,
					r: Math.min(rect.width / 2, rect.height / 2)
				};
				this.setShape(sh);
			}
		}
	});
	
	//
	// An extension to gfx Group to add layout capabilities.
	//
	gfxlayout._PanelExt = {
	
		_ibmd_layoutMgr: null,
		
		__shapeSet: function(){
		
		},
		
		setLayout: function(/*ibm_ilog.diagram.gfxlayout.Layout || Object*/layoutOpt){
			// summary:
			//     Sets the layout manager for this container.
			// layoutOpt: ibm_ilog.diagram.gfxlayout.Layout || layoutOpt:
			//     Either a layout instance or a plain JS object that describes the layout to instantiate and its parameters.
			//     In case it is a plain JS object, the object must contain a 'type' property of type String that gives the
			//     name of the Layout subclass to instantiate. The 'layoutOpt' parameter is then passed to the constructor to
			//     configure the layout.
			
			if (this._ibmd_layoutMgr) 
				this._ibmd_layoutMgr._detach();
			var layout = null;
			// layoutOpt is either a Layout instance or a plain JS object that describes the class and its parameters.
			if (layoutOpt instanceof gfxlayout.Layout) {
				layout = layoutOpt;
			} else if (layoutOpt) {
				var t = layoutOpt.type;
				if (!t) 
					ErrorReporter.error("UnexpectedArgument", this, "setLayout", "layoutOpt.type", layoutOpt);
				var type = lang.getObject(t, false);
				if (typeof(type) == "function") {
					layout = new type(layoutOpt);
				} else {
					ErrorReporter.error("UnexpectedArgument", this, "setLayout", "layoutOpt.type", layoutOpt);
				}
			}
			this._ibmd_layoutMgr = layout;
			if (this._ibmd_layoutMgr) {
				this._ibmd_layoutMgr._attach(this);
				this._ibmd_layoutMgr.invalidate();
			}
		},
		
		getLayout: function(){
			// summary:
			//     Gets the layout manager set on this container.
			
			return this._ibmd_layoutMgr; /*ibm_ilog.diagram.gfxlayout.Layout*/
		},
		
		_needsLayout: false,
		
		needsLayout: function(){
			// summary:
			//     Marks this container as needed to be layed out the next pass.
			this._needsLayout = true;
		},
		
		_oldLayoutRect: null,
		
		applyLayout: function(/*Rectangle*/rect){
			// summary: Lays out this container.
			// rect: Rectangle: 
			
			if (this._ibmd_layoutMgr) {
				// check a layout is needed: either because computePreferredSize as marked it as needed, or
				// if the rect is different from the previous one.
				var doIt = this._needsLayout || !this._oldLayoutRect ||
				this._oldLayoutRect.width !== rect.width ||
				this._oldLayoutRect.height !== rect.height;
				if (doIt) {
					this._oldLayoutRect = lang.clone(rect);
					try {
						// Delegate to the layout manager.
						// Layout is done relative to the parents (ie. (0,0) )
						this._ibmd_layoutMgr.doLayout({
							x: 0,
							y: 0,
							width: rect.width,
							height: rect.height
						});
					} finally {
						this._needsLayout = false;
					}
				}
				// then positioned this container to the right coord
				//
				// remove the previous transform set by the layout
				var t = this.getTransform() || matrix.identity;
				// 'disable' the transform set previously by the layout
				// so here, t is all the transforms except the one from the layout
				if (this.__oldLayoutTransform && !this.__oldLayoutTransform.isIdentity()) {
					t = matrix.multiply(this.__oldLayoutTransform.inverse(), t);
				}
				var bbox = this.getBoundingBox();
				if (t && bbox) 
					bbox = t.transformRectangle(bbox);
				var dx = rect.x, dy = rect.y;
				if (dx != 0 || dy != 0) {
					this.__oldLayoutTransform = matrix.translate(dx, dy);
					t = matrix.multiply(this.__oldLayoutTransform,t);
					var oldt = this.getTransform();
					if (!(oldt != null && oldt.isSame(t))) 
						this.setTransform(t);
				} else {
					this.__oldLayoutTransform = null;
				}
			} else {
				gfxlayout._LayoutShapeExt.applyLayout.call(this, rect);
			}
		},
		
		_recomputeSize: false,
		
		invalidate: function(){
			// summary:
			//     Invalidates this shape.
			
			if (this._suspendInvalidate) 
				return;
			var p = this.getParent();
			var lmgr = this._ibmd_layoutMgr;
			// do nothing if a layout is running or no parent and no layout        
			if ((!lmgr && !p) || (lmgr && lmgr.inLayout())) 
				return;
			// marks the panel so that its preferred size needs to be recomputed the next pass.
			this._recomputeSize = true;
			// propagate to the parent hierarchy
			if (p && p._ibmd_layoutMgr) 
				p.invalidate();
			else if (lmgr) 
				// no parent or no parent panel, invalidate this one
				gfxlayout.postInvalidate(this);
		},
		
		_oldProposedSize: null,
		
		_computePreferredSize: function(proposedSize){
			// summary:
			//     Computes the preferred size of this panel.
			
			var sz;
			// empty ? nothing to do
			if (this.children.length == 0) 
				return this.getExplicitSize();
			if (this._ibmd_layoutMgr) {
				// recompute the size only if it has been explicitely marked as needed
				// or if the proposed size is different from the previous one.
				if (this._explicitSize) {
					proposedSize = this._explicitSize;
				}
				if (this._recomputeSize === true) {
					try {
						sz = this._computeLayoutPreferredSize(proposedSize);
						this.needsLayout();
					} finally {
						this._recomputeSize = false;
					}
				} else if (!this._oldProposedSize ||
				proposedSize.width !== this._oldProposedSize.width ||
				proposedSize.height !== this._oldProposedSize.height) {
				
					sz = this._computeLayoutPreferredSize(proposedSize);
					this.needsLayout();
				} else {
					// do not recompute the size.
					sz = this.getPreferredSize();
				}
			} else {
				// call bbox
				sz = gfxlayout._LayoutShapeExt._computePreferredSize.call(this, proposedSize);
			}
			return sz;
		},
		
		_computeLayoutPreferredSize: function(proposedSize){
			// summary:
			//     Computes the preferred size with the layout.
			
			this._oldProposedSize = {
				width: proposedSize.width,
				height: proposedSize.height
			};
			var sz = this._ibmd_layoutMgr.computePreferredSize(proposedSize);
			var t = this.getTransform();
			if (t) {
				var rect = {
					x: 0,
					y: 0,
					width: sz.width,
					height: sz.height
				};
				rect = t.transformRectangle(rect);
				sz.width = rect.width;
				sz.height = rect.height;
			}
			return sz;
		},
		
		getExplicitSize: function(){
			// summary:
			//     Returns the explicit size of this container.
			
			if (!this._explicitSize) {
				this._explicitSize = {
					width: -1, // = auto
					height: -1 // = auto
				};
			}
			return {
				width: this._explicitSize.width,
				height: this._explicitSize.height
			};
		},
		
		setExplicitSize: function(size){
			// summary:
			//     Sets the explicit size of this shape.
			
			this._explicitSize = {
				width: size.width,
				height: size.height
			};
			this.invalidate();
		},
		
		_computeBoundingBox: function(){
			// if it has been marked as modified, ensure it is validated before computing the bounding box
			if (this._recomputeSize && this._ibmd_layoutMgr) 
				gfxlayout.doPendingValidate(this);
			var bbox = diagext.BoundingBoxFix._computeBoundingBox.call(this);
			if (bbox && this._ibmd_layoutMgr) {
				return this._ibmd_layoutMgr.adjustBBox(bbox);
			} else {
				return bbox;
			}
		}
	};
	
	lang.extend(gfx.Group, gfxlayout._PanelExt);
	
	return gfxlayout;
});

},
'ibm_ilog/diagram/templates/templates':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/templates/templates", [
"../main",
"dojo/_base/lang", "dojo/_base/config",
"dojox/collections/Dictionary",
"../util/MultilineText",
"../data/dtlbinding"
],
function(iid, lang, config, Dictionary, MultilineText, dtlbinding){

	// TODO: Change to this when AMD conversion is complete:
	// var t = {};
	var t = lang.getObject("ibm_ilog.diagram.templates", true);
	
if (config.useGfxLayout) {

	// NODE - GFX-LAYOUT TEMPLATE

t.defaultNodeTemplate = [{
    dojoAttachPoint: 'baseShape',
    shape:{
        width:80, height:40, r: (iid.isVML ? "0" : "5"),
        type:'rect'
    },
    fill: '{{backgroundColor}}',
    stroke: {'color': '{{borderColor}}', 'width': 2 },
    selectedStyle: {
        fill: '{{selectedBackgroundColor}}'
    }
},{
    dojoAttachPoint: 'textShape', 
    margins:[5,2,5,2],
    shape:{
        type:'mltext',
        text:'{{label}}', 
        wrap: $bind('{{wrap}}'), width: 70,
        align:'middle'
    }, 
    editable: {
        id:'label',
        editor:'textbox',
        border:2,
        minRegionSize:{width:50,height:15},
        regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
        layoutProperties: {margins:[5,5,5,5], valign: 'center', halign: 'center'}
    },
    fill:'{{textColor}}', 
    font:{type:'font', size:'10pt', family:'sans-serif'},
    selectedStyle: {
        fill: '{{selectedTextColor}}'
    }
}];

t.defaultOverviewNodeTemplate = [{
  dojoAttachPoint: 'baseShape',
  shape:{
      width:80, height:40, r: (iid.isVml ? "0" : "5"),
      type:'rect'
  },
  fill: '{{backgroundColor}}',
  stroke: {'color': '{{borderColor}}', 'width': 2 }
}];

} else {

	// NODE - NON GFX-LAYOUT TEMPLATE

t.defaultNodeTemplate = [{
    dojoAttachPoint: 'baseShape',
    shape: {
        x: 0,
        y: 0,
        width: 80,
        height: 40,
        r: 5,
        type: 'rect'
    },
    fill: '{{backgroundColor}}',
    stroke: {
        'color': '{{borderColor}}',
        'width': 2
    },
    selectedStyle: {
        fill: '{{selectedBackgroundColor}}'
    }
}, {
   dojoAttachPoint: 'textShape',
   shape: {
       type: 'text',
       text: '{{label}}',
       x: 40,
       y: 24,
       align: 'middle'
   },
   editable: {
       id:'label',
       editor:'textbox',
       border:2,
       regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
       baseRegion:{x:20,width:40,y:11,height:15}
   },
   fill: '{{textColor}}',
   font: {
       type: 'font',
       size: '10pt',
       family: 'sans-serif'
       },
   selectedStyle: {
       fill: '{{selectedTextColor}}'
   }
}];

t.defaultOverviewNodeTemplate = [{
  dojoAttachPoint: 'baseShape',
  shape: {
      x: 0,
      y: 0,
      width: 80,
      height: 40,
      r: 5,
      type: 'rect'
  },
  fill: '{{backgroundColor}}',
  stroke: {
      'color': '{{borderColor}}',
      'width': 2
  }
}];

}

t.defaultLinkTemplate = [{
    dojoAttachPoint: '_path',
    shape: {
        type: 'polyline',
        points: [{
            x: 0,
            y: 0
        }, {
            x: 100,
            y: 0
        }]
    },
    stroke: {
        width: $bind("{{strokeWidth}}"),
        color: '{{strokeColor}}'
    },
    selectedStyle: {
        stroke: {
            width: $bind("{{selectedStrokeWidth}}"),
            color: '{{selectedStrokeColor}}'
        }
    }
}, {
    dojoAttachPoint: '_endArrow',
    shape: {
        type: 'polyline',
        points: [{
            x: -8,
            y: 4
        }, {
            x: 0,
            y: 0
        }, {
            x: -8,
            y: -4
        }]
    },
    fill: '{{strokeColor}}',
    selectedStyle: {
        fill: '{{selectedStrokeColor}}'
    }
}];

t.defaultOverviewLinkTemplate = [{
  dojoAttachPoint: '_path',
  shape: {
      type: 'polyline',
      points: [{
          x: 0,
          y: 0
      }, {
          x: 100,
          y: 0
      }]
  },
  stroke: {
      width: $bind("{{strokeWidth}}"),
      color: '{{strokeColor}}'
  }
}];

if (config.useGfxLayout) {

	// SUBGRAPH - GFX-LAYOUT TEMPLATE

    t.defaultSubgraphTemplate = [{
        dojoAttachPoint: '_expandedGroup',
        layout: {
            type: 'ibm_ilog.diagram.gfxlayout.GridLayout',
            rows: [{
                height: -1
            }, {
                height: -1
            }]
        },
        children: [{
            dojoAttachPoint: '_expandedBaseShape',
            row: 0,
            rowspan: -1,
            shape: {
                type: 'rect',
                width: 0,
                height: 0,
                r: 4
            },
            fill: '{{headerColor}}',
            selectedStyle: {
                fill: '{{selectedHeaderColor}}'
            },
            stroke: '#404040'
        }, {
			hfiller:true,
            dojoAttachPoint: '_header',
            row: 0,
            margins: [2, 0, 2, 2],
            layout: {
                type: 'ibm_ilog.diagram.gfxlayout.StackLayout',
                horizontal: true,
                gap: 2
            },
            children: [{
                dojoAttachPoint: '_collapseButton',
                valign: 'center',
                layout: {
                    type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
                },
                children: [{
                    shape: {
                        type: 'rect',
                        width: 12,
                        height: 12,
                        r: 4
                    },
                    fill: '#808080'
                }, {
                    valign: 'center',
                    halign: 'center',
                    shape: {
                        type: 'line',
                        x1: 3,
                        y1: 6,
                        x2: 9,
                        y2: 6
                    },
                    stroke: {
                        color: 'white',
                        width: 2
                    }
                }]
            }, {
            	layout: {
					type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
				},
				children:[{
                dojoAttachPoint: 'titleShape',
				valign: 'center', halign: 'left',
                shape: {
                    type: 'text',
                    text: '{{label}}',
                    align: 'start'
                },
                fill: '{{textColor}}',
                font: {
                    type: 'font',
                    size: '10pt',
                    family: 'sans-serif'
                },
                editable: {
                    id: 'label',
                    editor: 'textbox',
                    options: {
                        style: {
                            'text-align': 'left'
                        }
                    },
                    border: 2,
			        regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
    		    	layoutProperties: { valign: 'center', halign: 'left'}
                },
                selectedStyle: {
                    fill: '{{selectedTextColor}}'
                }
				}]
            }]
        }, {
            dojoAttachPoint: '_graphBackground',
            row: 1,
            margins: 2,
            col: 0,
            shape: {
                type: 'rect',
                width: 0,
                height: 0
            },
            fill: '{{graphBackgroundColor}}'
        }, {
            dojoAttachPoint: '_graph',
            row: 1,
            margins: $bind('{{graphMargin}}'),
			minimumSize: { width: 80, height: 40 },
            col: 0,
            shape: {
                type: 'graph'
            }
        }]
    }, {
        dojoAttachPoint: '_collapsedGroup',
        layout: {
            type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
        },
        children: [{
            dojoAttachPoint: '_collapsedBaseShape',
            shape: {
                width: 80,
                height: 40,
                r: 5,
                type: 'rect'
            },
            fill: '{{backgroundColor}}',
            stroke: {
                'width': 2
            },
            selectedStyle: {
                fill: '{{selectedBackgroundColor}}'
            }
        }, {
            dojoAttachPoint: 'textShape',
            margins: 5,
            shape: {
                type: 'mltext',
                text: '{{label}}',
		        wrap: $bind('{{wrap}}'), width: 70,
                align: 'middle'
            },
            fill: '{{textColor}}',
            selectedStyle: {
                fill: '{{selectedTextColor}}'
            },
    		editable: {
        		id:'label',
    		    editor:'textbox',
    		    border:2,
    		    regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}},
    		    layoutProperties: {margins:[5,5,5,5], valign: 'center', halign: 'center'}
    		},
            font: {
                type: 'font',
                size: '10pt',
                family: 'sans-serif'
            }
        }, {
            margins: 4,
            valign: 'top',
            halign: 'left',
            dojoAttachPoint: '_expandButton',
            layout: {
                type: 'ibm_ilog.diagram.gfxlayout.GridLayout'
            },
            children: [{
                shape: {
                    type: 'rect',
                    width: 12,
                    height: 12,
                    r: 4
                },
                fill: '#808080'
            }, {
                valign: 'center',
                halign: 'center',
                shape: {
                    type: 'line',
                    x1: 3,
                    y1: 6,
                    x2: 9,
                    y2: 6
                },
                stroke: 'white'
            }, {
                valign: 'center',
                halign: 'center',
                shape: {
                    type: 'line',
                    x1: 6,
                    y1: 3,
                    x2: 6,
                    y2: 9
                },
                stroke: {
                    color: 'white',
                    width: 2
                }
            }]
        }]
    }];
    
} else {

	// SUBGRAPH - NON GFX-LAYOUT TEMPLATE

	t.defaultSubgraphTemplate = [{
        dojoAttachPoint: '_expandedGroup',
        children: [{
            dojoAttachPoint: '_expandedBaseShape',
           shape: {
               type: 'rect',
               r: 5
           },
           fill: '{{headerColor}}',
           selectedStyle: {
               fill: '{{selectedHeaderColor}}'
           },
           stroke: '#404040'
       }, {
           dojoAttachPoint: '_graphBackground',
           shape: {
               type: 'rect'
           },
           fill: '{{graphBackgroundColor}}'
       }, {
            dojoAttachPoint: '_graph',
            shape: {
                type: 'graph'
            }
        }, {
           dojoAttachPoint: '_header',
           transform: { dx: -8, dy: -26 },
           children: [{
               dojoAttachPoint: '_collapseButton',
               children: [{
                   shape: {
                       type: 'rect',
                       x: 0,
                       y: 0,
                       width: 12,
                       height: 12,
                       r: 4
                   },
                   fill: '#808080'
               }, {
                   shape: {
                       type: 'line',
                       x1: 3,
                       y1: 6,
                       x2: 9,
                       y2: 6
                   },
                   stroke: {
                       color: 'white',
                       width: 2
                   }
               }]
           }, {
               dojoAttachPoint: 'titleShape',
               shape: {
                   type: 'text',
                   text: '{{label}}',
                   x: 15,
                   y: 10,
                   align: 'start'
               },
               fill: '{{textColor}}',
               font: {
                   type: 'font',
                   size: '10pt',
                   family: 'sans-serif'
               },
               editable: { 
                   id:'label', 
                   editor:'textbox', 
                   options:{style:{'text-align':'left'}},
                   border:2,
                   baseRegion:{x:20,width:40,y:0,height:15}
               },
               selectedStyle: {
                   fill: '{{selectedTextColor}}'
               }
           }]
       }]
    }, {
        dojoAttachPoint: '_collapsedGroup',
        children: [{
            dojoAttachPoint: '_collapsedBaseShape',
            shape: {
                x: 0,
                y: 0,
                width: 80,
                height: 40,
                r: 5,
                type: 'rect'
            },
            fill: '{{backgroundColor}}',
            stroke: {
                'width': 2
            },
            selectedStyle: {
                fill: '{{selectedBackgroundColor}}'
            }
        }, {
            dojoAttachPoint: 'textShape',
            shape: {
                type: 'text',
                text: '{{label}}',
                x: 40,
                y: 24,
                align: 'middle'
            },
            fill: '{{textColor}}',
            selectedStyle: {
                fill: '{{selectedTextColor}}'
            },
            editable:{
               id:'label',
               editor:'textbox',
               border:2,
               baseRegion:{x:20,width:40,y:11,height:15}
           },
            font: {
                type: 'font',
                size: '10pt',
                family: 'sans-serif'
            }
        }, {
            dojoAttachPoint: '_expandButton',
           transform: { dx:2, dy:2 },
            children: [{
                shape: {
                    type: 'rect',
                    x: 0,
                    y: 0,
                    width: 12,
                    height: 12,
                    r: 4
                },
                fill: '#808080'
            }, {
                shape: {
                    type: 'line',
                    x1: 3,
                    y1: 6,
                    x2: 9,
                    y2: 6
                },
                stroke: 'white'
            }, {
                shape: {
                    type: 'line',
                    x1: 6,
                    y1: 3,
                    x2: 6,
                    y2: 9
                },
                stroke: {
                    color: 'white',
                    width: 2
                }
            }]
        }]
    }];

}

t.defaultOverviewExpandedSubgraphTemplate = [{
	dojoAttachPoint: 'baseShape',
	shape: {
		type: 'rect'
	},
	fill: '{{graphBackgroundColor}}',
	stroke: '#404040'
}];

t.defaultOverviewCollapsedSubgraphTemplate = [{
	dojoAttachPoint: 'baseShape',
	shape: {
		type: 'rect'
	},
	fill: '{{backgroundColor}}',
	stroke: '#404040'
}];


t.defaultNodeFocusAdornerTemplate = [{
	dojoAttachPoint: 'adornerRoot',
	children:[{
   	dojoAttachPoint: 'baseShape',
   	shape: {
       x: $bind("{{elemBaseLeft}}-3"),
       y: $bind("{{elemBaseTop}}-3"),
       width: $bind("{{elemBaseWidth}}+6"),
       height: $bind("{{elemBaseHeight}}+6"),
       type: 'rect'
		},
		stroke: {
			color: 'black',
			style: 'ShortDot'
		}
	}]
}];

t.defaultAnnotationTargetAdornerTemplate = [{
	dojoAttachPoint: 'adornerRoot',
	children:[{
	   	dojoAttachPoint: 'baseShape',
	   	shape: {
	       x: $bind("{{elemBaseLeft}}"),
	       y: $bind("{{elemBaseTop}}"),
	       width: $bind("{{elemBaseWidth}}"),
	       height: $bind("{{elemBaseHeight}}"),
	       type: 'rect'
		}
	}]
}];

if(config.useGfxLayout) {

	// ANNOTATION - GFX-LAYOUT TEMPLATE

	t.defaultAnnotationAdornerTemplate = [{
		dojoAttachPoint: 'adornerRoot',
    transform:{dx: $bind("{{elemBaseRight}}+15"),dy: $bind("{{elemBaseTop}}-25")},
		layout:{ type:'ibm_ilog.diagram.gfxlayout.GridLayout'},
		children:[{
		   	dojoAttachPoint: 'baseShape',
		   	shape: {
		       width: 1,
		       height: 1,
		       type: 'rect'
			},
			stroke: {
				color: [0,0,0,1],
				style: 'ShortDot',
				width: 1
			},
			fill: [222,222,150,0.75]
		},{
		   dojoAttachPoint: 'textShape',
			valign:'center',
			halign:'center',
			margins:4,
		   shape: {
		       type: 'text',
		       text: '{{annotation|safe}}',
		       x: 5,
		       y: 15,
		       align: 'middle'
		   },
		   fill: [0,0,0],
		   font: {
		       type: 'font',
		       size: '10pt',
		       family: 'sans-serif'
		   }
		}
		]
	}];
} else {
	
	// ANNOTATION - NON GFX-LAYOUT TEMPLATE
	
	t.defaultAnnotationAdornerTemplate = [{
		dojoAttachPoint: 'adornerRoot',
    transform:{dx: $bind("{{elemBaseRight}}+15"),dy: $bind("{{elemBaseTop}}-25")},
		children:[{
	   	dojoAttachPoint: 'baseShape',
	   	shape: {
	       width: 150,
	       height: 20,
	       type: 'rect'
			},
			stroke: {
				color: [120,120,0,1],
				style: 'ShortDot',
				width: 2
			},
			fill: [120,120,0,0.5]
		},{
		   dojoAttachPoint: 'textShape',
		   shape: {
		       type: 'text',
		       text: '{{annotation|safe}}',
		       x: 5,
		       y: 15,
		       align: 'left'
		   },
		   fill: [0,0,0],
		   font: {
		       type: 'font',
		       size: '10pt',
		       family: 'sans-serif'
		   }
		}
		]
	}];
}	

t.defaultAnnotationLinkTemplate = [{
    dojoAttachPoint: '_path',
    shape: {
        type: 'polyline',
        points: [{
            x: 0,
            y: 0
        }, {
            x: 100,
            y: 0
        }]
    },
		stroke: {
			color: [0,0,0,1],
			style: 'ShortDot',
			width: 1
		},
    selectedStyle: {
        stroke: {
            width: $bind("{{selectedStrokeWidth}}"),
            color: '{{selectedStrokeColor}}'
        }
    }
}];

t.defaultLinkFocusAdornerTemplate = [{
	dojoAttachPoint: 'adornerRoot',
	children:[{
	    dojoAttachPoint: '_path',
	    shape: {
	        type: 'polyline',
	        points: [{
	            x: 0,
	            y: 0
	        }, {
	            x: 0,
	            y: 0
	        }]
	    },
	    stroke: {
			 width: 5,
	        color: [255,0,255,0.5]
	    }
	}]
}];
t.defaultLinkFocusAdornerTemplate.noDtl = true;

t.defaultAdornerTemplates = t.defaultAdornerTemplates || new Dictionary();

t.defaultAdornerTemplates.add("NodeFocusAdorner",t.defaultNodeFocusAdornerTemplate);
t.defaultAdornerTemplates.add("LinkFocusAdorner",t.defaultLinkFocusAdornerTemplate);
t.defaultAdornerTemplates.add("AnnotationAdorner",t.defaultAnnotationAdornerTemplate);
t.defaultAdornerTemplates.add("AnnotationTargetAdorner",t.defaultAnnotationTargetAdornerTemplate);
t.defaultAdornerTemplates.add("AnnotationLink",t.defaultAnnotationLinkTemplate);

t.defaultSwimLaneTemplate = "[{" +
"    dojoAttachPoint: '_expandedGroup'," +
"    layout:{" +
"       type:'ibm_ilog.diagram.gfxlayout.GridLayout'," +
" {% if vertical %}" +
"       rows:[{height:-1},{}]" +
" {% else %}" +
"  {% if rightToLeft %}" +
"       cols:[{},{width:-1}]" +
"  {% else %}" +
"       cols:[{width:-1},{}]" +
"  {% endif %}" +
" {% endif %}" +
"    }," +
"    children: [{" +
"           dojoAttachPoint: '_expandedBaseShape'," +
" {% if vertical %}" +
"           row:0, rowspan:2," +
" {% else %}" +
"           column:0, colspan:2," +
" {% endif %}" +
"           shape:{"+
"               type:'rect', width:0, height:0"+
"           },"+
"           fill:'{{headerColor}}'," +
"           selectedStyle: { "+
"               fill: '{{selectedHeaderColor}}'"+
"           }," +
"           stroke:'#404040'" +
"       }, {" +
"           dojoAttachPoint: '_header'," +
" {% if vertical %}" +
"           row:0, margins:[2,0,2,2]," +
"  {% if nested %}" +
"   		minimumSize: { width:-1, height:16 }," +
"  {% endif %}" +
" {% else %}" +
"  {% if rightToLeft %}" +
"           column:1, margins:[2,2,0,2]," +
"  {% else %}" +
"           column:0, margins:[0,2,2,2]," +
"  {% endif %}" +
"  {% if nested %}" +
"   		minimumSize: { width:16, height:-1 }," +
"  {% endif %}" +
" {% endif %}" +
"           layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'" +
" {% if not nested %}" +
"               ," +
"  {% if vertical %}" +
"               horizontalFlow:true, cols:[{width:-1},{}]" +
"  {% else %}" +
"               horizontalFlow:false, rows:[{height:-1},{}]" +
"  {% endif %}" +
" {% endif %}" +
"           }," +
"           children:["+
" {% if not nested %}" +
"             {" +
"               dojoAttachPoint: '_collapseButton',"+
" {% if vertical %}" +
"               valign:'center', halign:'left'," +
" {% else %}" +
"               halign:'center', valign:'top'," +
" {% endif %}" +
"               layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
"               children: [{" +
"                   shape: {" +
"                       type: 'rect'," +
"                       x: 0," +
"                       y: 0," +
"                       width: 12," +
"                       height: 12" +
"                   }," +
"                   fill: '#808080'" +
"               },{ " +
"                   valign:'center',halign:'center'," +
"                   shape: {" +
"                       type: 'line'," +
"                       x1: 3," +
"                       y1: 6," +
"                       x2: 9," +
"                       y2: 6" +
"                   }," +
"                   stroke: {" +
"                       color: 'white'," +
"                       width: 2" +
"                   }" +
"               }]" +
"           },"+
" {% endif %}" +
"           {" +
"            	layout: {" +
"					type: 'ibm_ilog.diagram.gfxlayout.GridLayout'" +
"				}," +
"				children:[{" +
"               dojoAttachPoint: 'titleShape'," +
" {% if vertical %}" +
"               valign:'center', halign:'stretch'," +
" {% else %}" +
"               halign:'center', valign:'stretch'," +
"  {% if rightToLeft %}" +
"           	transform: {xx: 0, xy: -1, yx: 1, yy: 0}," +
"  {% else %}" +
"           	transform: {xx: 0, xy: 1, yx: -1, yy: 0}," +
"  {% endif %}" +
" {% endif %}" +
"               shape: {" +
"                   type: 'text'," +
"                   text: '{{label}}'," +
" {% if vertical %}" +
"                   align: 'middle'" +
" {% else %}" +
"  {% if rightToLeft %}" +
"                   align: 'middle'" +
"  {% else %}" +
"                   align: 'middle'" +
"  {% endif %}" +
" {% endif %}" +
"               }," +
"               fill: '{{textColor}}'," +
"               font: {" +
"                   type: 'font'," +
"                   size: '10pt'," +
"                   family: 'sans-serif'" +
"               }," +
"               editable: { " +
"                   id:'label', " +
"                   editor:'textbox', " +
"                   options:{style:{'text-align':'left'}}," +
"                   border:2," +
"			        regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}}," +
"    		    	layoutProperties: {" +
" {% if vertical %}" +
"               valign:'center', halign:'left'" +
" {% else %}" +
"               halign:'center', valign:'top'" +
" {% endif %}" +
"                   }" +
"               },"+
"               selectedStyle: {" +
"                  fill: '{{selectedTextColor}}'" +
"               }" +
"				}]" +
"           }]" +
"       }, {" +
"           dojoAttachPoint: '_graphRect'," +
" {% if vertical %}" +
"           row:1," +
" {% else %}" +
"  {% if not rightToLeft %}" +
"           column:1," +
"  {% endif %}" +
" {% endif %}" +
"           shape:{"+
"               type:'rect',"+
"               width:0,height:0"+
"           }" +
"       }, {" +
"           dojoAttachPoint: '_graphBackground'," +
" {% if vertical %}" +
"           row:1," +
" {% else %}" +
"  {% if rightToLeft %}" +
"           column:0," +
"  {% else %}" +
"           column:1," +
"  {% endif %}" +
" {% endif %}" +
"           margins:1," +
"           shape:{"+
"               type:'rect',"+
"               width:0,height:0"+
"           }," +
"           fill:'{{graphBackgroundColor}}'" +
"       }, {" +
" {% if vertical %}" +
"           row:1, column:0," +
" {% else %}" +
"  {% if rightToLeft %}" +
"           column:0," +
"  {% else %}" +
"           column:1," +
"  {% endif %}" +
"           row:0," +
" {% endif %}" +
"           margins:0," +
"           halign:'left', valign:'top'," +
"           children: [{" +
"               shape:{"+
"                   type:'rect',"+
"                   x:0,y:0,width:0,height:0"+
"               }" +
"           }, {" +
"               dojoAttachPoint: '_graph'," +
"               shape:{"+
"                   type:'graph'"+
"               }" +
"           }]" +
"       }]" +
"   }, {" +
"   dojoAttachPoint: '_collapsedGroup'," +
"   layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
"   children:[{"+ 
"     layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
" {% if vertical %}" +
"     halign:'left'," +
" {% else %}" +
"     valign:'top'," +
" {% endif %}" +
"     children:[{"+ 
"		 dojoAttachPoint: '_collapsedBaseShape'," +
"        shape: {" +
"            x: 0," +
"            y: 0," +
"            width: 80," +
"            height: 40," +
"            r: 5," +
"            type: 'rect'" +
"        }," +
"        fill: '{{backgroundColor}}'," +
"        stroke: {" +
"            'width': 2" +
"        }," +
"        selectedStyle: {" +
"            fill: '{{selectedBackgroundColor}}'" +
"        }" +
"   }," +
"   {" +
"        dojoAttachPoint: 'textShape'," +
"        margins:5," +
"        shape: {" +
"            type: 'mltext'," +
"            text: '{{label}}'," +
"            x: 40," +
"            y: 24," +
"	         wrap: {{wrap}}, width: 70," +
"            align: 'middle'" +
"        }," +
"        fill: '{{textColor}}'," +
"        selectedStyle: {" +
"            fill: '{{selectedTextColor}}'" +
"        }," +
"        editable:{" +
"           id:'label'," +
"           editor:'textbox'," +
"           border:2," +
"   		regionStyle:{selectedStyle:{stroke:{color:'white',style:'ShortDot'}}}," +
"   		layoutProperties: {margins:[5,5,5,5], valign: 'center', halign: 'center'}" +
"        },"+
"        font: {" +
"            type: 'font'," +
"            size: '10pt'," +
"            family: 'sans-serif'" +
"        }" +
"   }," +
"   {" +
"       margins:4," +
"       valign:'top'," +
" {% if vertical %}" +
"       halign:'left'," +
" {% else %}" +
"  {% if rightToLeft %}" +
"       halign:'right'," +
"  {% else %}" +
"       halign:'left'," +
"  {% endif %}" +
" {% endif %}" +
"       dojoAttachPoint: '_expandButton'," +
"       layout:{type:'ibm_ilog.diagram.gfxlayout.GridLayout'}," +
"       children:[{" +
"            shape: {" +
"                type: 'rect'," +
"                x: 0," +
"                y: 0," +
"                width: 12," +
"                height: 12," +
"                r: 4" +
"            }," +
"            fill: '#808080'" +
"        }, {" +
"            valign:'center',halign:'center'," +
"            shape: {" +
"                type: 'line'," +
"                x1: 3," +
"                y1: 6," +
"                x2: 9," +
"                y2: 6" +
"            }," +
"            stroke: 'white'" +
"        }, {" +
"           valign:'center',halign:'center'," +
"           shape: {" +
"                type: 'line'," +
"                x1: 6," +
"                y1: 3," +
"                x2: 6," +
"                y2: 9" +
"            }," +
"            stroke: {" +
"                color: 'white'," +
"                width: 2" +
"            }" +
"       }" +
"       ]" +
"     }" +
"     ]" +
"   }" +
"   ]" +
"}" +
"]";

return t;

});

},
'ibm_ilog/diagram/graphlayout/LayoutRunner':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/LayoutRunner", [
"dojo/_base/declare", "dojo/_base/config", "dojo/_base/connect",
"require",
"../gfxlayout/Engine", "../util/Batch"
], function(declare, config, connect, require, gfxlayout, Batch){

// The following classes are (tentatively) loaded dynamically when we perform a layout,
// they are used when we do a hierarchical layout and the diagram contains swim lanes.

var SwimLane = null;
var HierarchicalLayout = null;
var HierarchicalNodeGroup = null;
var HierarchicalSwimLaneConstraint = null;
var Direction = null;
var _classesRequired = false;

var _requireClasses = function()
{
	if(!_classesRequired){
		try {
			HierarchicalLayout = require("ibm_ilog/graphlayout/hierarchical/HierarchicalLayout");
			HierarchicalNodeGroup = require("ibm_ilog/graphlayout/hierarchical/HierarchicalNodeGroup");
			HierarchicalSwimLaneConstraint = require("ibm_ilog/graphlayout/hierarchical/HierarchicalSwimLaneConstraint");
			Direction = require("ibm_ilog/graphlayout/Direction");
			SwimLane = require("../SwimLane");
		} catch(err){}
	}
	_classesRequired = true;
}

var LayoutRunner = 
declare("ibm_ilog.diagram.graphlayout.LayoutRunner", null, {
	// summary:
	//		This class provides the graph layout API for a Graph object.
	// tags:
	//		private
	
	_nodeLayout: null,
	_linkLayout: null,
	_nodeLayoutActive: true,
	_linkLayoutActive: true,
	_automaticNodeLayout: false,
	_automaticLinkLayout: false,
	_animatedLayout: false,
	_autoFitOnLayout: false,
	_rml: null,
	_graphAdapter: null,
	_layoutProvider: null,
	
	setNodeLayout: function(/*ibm_ilog.graphlayout.GraphLayout*/layout){
		// summary:
		//		Sets the graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// layout: ibm_ilog.graphlayout.GraphLayout:
		//		The graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		if (layout != this._nodeLayout) {
			this._detachLayout(this._nodeLayout, true);
			this._nodeLayout = layout;
			this._attachLayout(this._nodeLayout, true);
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	getNodeLayout: function(){
		// summary:
		//		Gets the graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// returns: ibm_ilog.graphlayout.GraphLayout:
		//		The graph layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		return this._nodeLayout;
	},
	
	setNodeLayoutActive: function(active){
		// summary:
		//		Activates or deactivates the graph layout specified by setNodeLayout().
		// active: boolean:
		//		True to activate the graph layout, false to deactivate it.
		// description:
		//		If the graph layout is active, it will be performed when performGraphLayout() is called.
		//		For example, you can call setNodeLayoutActive(false) if you want to perform only a link layout.
		
		if (active != this._nodeLayoutActive) {
			this._nodeLayoutActive = active;
			// Update the multiple layout instance
			var rml = this._rml;
			if (rml != null) {
				var ml = rml.getLayout(null);
				ml.setFirstGraphLayoutActive(active);
			}
			// We must refresh the sub-layouts because they may be an inherited copy of this one.
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	isNodeLayoutActive: function(){
		// summary:
		//		Returns true if the graph layout is active, or false otherwise.
		// returns: boolean:
		//		True if the graph layout is active, false otherwise.
		
		return this._nodeLayoutActive;
	},
	
	setAutomaticNodeLayout: function(automatic){
		// summary:
		//		Determines if the graph layout must be executed automatically
		//		whenever the graph changes.
		// automatic: boolean:
		//		If true, the graph layout will be executed automatically whenever the graph changes.
		//		If false, the graph layout must be executed by calling performGraphLayout explicitly.
		
		if (automatic != this._automaticNodeLayout) {
			this._automaticNodeLayout = automatic;
			if (automatic) 
				this._checkAutoLayout();
		}
	},
	
	isAutomaticNodeLayout: function(){
		// summary:
		//		Returns the flag specifying if the graph layout must be executed automatically
		//		whenever the graph changes.
		// returns: Boolean
		//		True if the graph layout will be executed automatically whenever the graph changes.
		//		False if the graph layout is executed only by calling performGraphLayout explicitly.
		return this._automaticNodeLayout;
	},
	
	setLinkLayout: function(/*ibm_ilog.graphlayout.GraphLayout*/layout){
		// summary:
		//		Sets the link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// layout: ibm_ilog.graphlayout.GraphLayout:
		//		The link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		if (layout != this._linkLayout) {
			this._detachLayout(this._linkLayout, false);
			this._linkLayout = layout;
			this._attachLayout(this._linkLayout, false);
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	getLinkLayout: function(){
		// summary:
		//		Gets the link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		// returns: ibm_ilog.graphlayout.GraphLayout:
		//		The link layout algorithm that will be executed on the contents of this graph
		//		when performGraphLayout is called.
		
		return this._linkLayout;
	},
	
	setLinkLayoutActive: function(active){
		// summary:
		//		Activates or deactivates the link layout specified by setLinkLayout().
		// active: boolean:
		//		True to activate the link layout, false to deactivate it.
		// description:
		//		If the link layout is active, it will be performed when performGraphLayout() is called.
		//		For example, you can call setLinkLayoutActive(false) if you want to perform only a graph layout.
		
		if (active != this._linkLayoutActive) {
			this._linkLayoutActive = active;
			// Update the multiple layout instance
			var rml = this._rml;
			if (rml != null) {
				var ml = rml.getLayout(null);
				ml.setSecondGraphLayoutActive(active);
			}
			// We must refresh the sub-layouts because they may be an inherited copy of this one.
			this._invalidateSubLayouts();
			this._checkAutoLayout();
		}
	},
	
	isLinkLayoutActive: function(){
		// summary:
		//		Returns true if the link layout is active, or false otherwise.
		// returns: boolean:
		//		True if the link layout is active, false otherwise.
		
		return this._linkLayoutActive;
	},
	
	setAutomaticLinkLayout: function(automatic){
		// summary:
		//		Determines if the link layout must be executed automatically
		//		whenever the graph changes.
		// automatic: boolean:
		//		If true, the link layout will be executed automatically whenever the graph changes.
		//		If false, the link layout must be executed by calling performGraphLayout explicitly.
		
		if (automatic != this._automaticLinkLayout) {
			this._automaticLinkLayout = automatic;
			if (automatic) 
				this._checkAutoLayout();
		}
	},
	
	isAutomaticLinkLayout: function(){
		// summary:
		//		Returns the flag specifying if the link layout must be executed automatically
		//		whenever the graph changes.
		// returns Boolean:
		//		True if the link layout will be executed automatically whenever the graph changes.
		//		False if the link layout is executed only by calling performGraphLayout explicitly.
		return this._automaticLinkLayout;
	},
	
	setAnimatedLayout: function(animated){
		// summary:
		//		Determines if an animation must be displayed after a graph layout has been performed.
		// animated: boolean:
		//		If true, an animation moves the nodes and reshapes the links from their position/shape
		//		before the layout to their position/shape after the layout.
		
		this._animatedLayout = animated;
	},
	
	isAnimatedLayout: function(){
		// summary:
		//		Returns the flags that determines if an animation must be displayed after a graph layout has been performed.
		// returns: boolean:
		//		True if graph layout is animated, false otherwise.
		
		return this._animatedLayout;
	},
	
	setAutoFitOnLayout: function(autoFit){
		// summary:
		//		Determines if fitToContents is called after a graph layout is performed.
		// autoFit: boolean:
		//		If true, fitToContents() is automatically called to show all the graph after a graph layout is performed.
		
		this._autoFitOnLayout = autoFit;
	},
	
	isAutoFitOnLayout: function(){
		// summary:
		//		Returne the boolean that determines if fitToContents is called after a graph layout is performed.
		// returns: boolean:
		//		True if fitToContents() is automatically called to show all the graph after a graph layout is performed.
		return this._autoFitOnLayout;
	},
	
	performGraphLayout: function(/*boolean*/recursive){
		// summary:
		//		Executes the graph layout and/or the link layout specified by the setNodeLayout()
		//		and setLinkLayout methods.
		// description:
		//		If a non-null graph layout has been specified by calling setNodeLayout(),
		//		and if the graph layout is active (that is, unless setNodeLayoutActive(false) has been called),
		//		then the graph layout algorithm is performed on the contents of this graph.
		//		Then, if a non-null link layout has been specified by calling setLinkLayout(),
		//		and if the link layout is active (that is, unless setLinkLayoutActive(false) has been called),
		//		then the link layout algorithm is performed on the contents of this graph.
		// recursive: boolean:
		//		If true, the graph layout is executed recursively on all Subgraphs contained in this Graph.
		//		If false, the graph layout is executed only on this Graph.
		
		if(arguments.length == 0)
			recursive = true;
		this._performGraphLayout(false, recursive);
	},
	
	onLayoutAnimationStart: function(){
		// summary:
		//		Called when the graph layout animation starts, if isAnimatedGraphLayout() is true.
		this._inLayoutAnimation = true;
	},
	
	onLayoutAnimate: function(v){
		// summary:
		//		Called on each step of the graph layout animation, if isAnimatedGraphLayout() is true.
		// v: number:
		//		A value between 0 and 100 specifying the progress of the animation.
	},
	
	onLayoutAnimationEnd: function(){
		// summary:
		//		Called when the graph layout animation ends, if isAnimatedGraphLayout() is true.
		this._inLayoutAnimation = false;
	},
	
	_performGraphLayout: function(auto, recursive){
	
		if (!this._rml) 
			return;
		
		// summary:
		//		Internal function that actually performs the layout.
		// auto: boolean:
		//		True if an auto-layout must be performed, false if an explicit layout is performed.
		//		If auto = true, the _automaticNodeLayout/_automaticLinkLayout flags are tested
		//		 to see which layout is in automatic mode.
		// recursive: boolean:
		//		If true, the graph layout is executed recursively on all Subgraphs contained in this Graph.
		//		If false, the graph layout is executed only on this Graph.
		
		var autoGraph = this._automaticNodeLayout;
		var autoLink = this._automaticLinkLayout;
		var animated = this._animatedLayout;
		
		// clear the auto graph layout flags to avoid infinite loops:
		// node move -> _onChange -> _checkAutoLayout -> auto-layout -> node move -> etc
		this._automaticNodeLayout = false;
		this._automaticLinkLayout = false;
		
        if (config.useGfxLayout){
            gfxlayout.Engine.layout();
        }
        
		var animation;
		if (this._animatedLayout) {
			animation = new LayoutRunner.Animation(this, recursive);
			animation.beforeLayout(recursive);
		}
		
		
		var swimLanes;

		// See if Hierarchical layout and SwimLane classes are loaded
		_requireClasses();
		
		 // Testing SwimLane below is enough since _requireClasses also require the hierarchical classes,
		 // so if SwimLane is there then all hierarchical classes + Direction must be there too.
		if (SwimLane &&
			this._nodeLayout &&
			this._nodeLayout instanceof HierarchicalLayout) {
			swimLanes = SwimLane._beforeLayout(this, this._nodeLayout, Direction, HierarchicalNodeGroup, HierarchicalSwimLaneConstraint);
		}
		
		if (this._rml) {
			
			this._refreshSubLayouts();
			
			var layout;
			if (recursive) 
				layout = this._rml;
			else 
				layout = this._rml.getLayout(null);
			layout.performLayout(true);
		}
		
		if (this._animatedLayout) {
			animation.afterLayout(recursive);
			animation = null;
		}
		
		if (!auto) 
			Batch.endBatch();
		
		if(swimLanes)
			SwimLane._afterLayout(this, this._nodeLayout, swimLanes);
		
		if (this._autoFitOnLayout && !this._animatedLayout) // animation will do the fit
			this.fitToContents();
		
		this._automaticNodeLayout = autoGraph;
		this._automaticLinkLayout = autoLink;
	},
	
	_attachLayout: function(layout, first){
		// tags:
		//		private 
		
		if (!layout) 
			return;
		
		var rml = this._rml;
		if (!rml) {
			var provider = new LayoutRunner._LayoutProvider(this);
			rml = new LayoutRunner.RecursiveMultipleLayout(null, provider);
			rml.setFirstGraphLayoutActive(LayoutRunner.RecursiveMultipleLayout.MIXED);
			rml.setSecondGraphLayoutActive(LayoutRunner.RecursiveMultipleLayout.MIXED);
			rml.attach(new LayoutRunner.GraphAdapter(this, rml));
			this._rml = rml;
		}
		
		var ml = rml.getLayout(null);
		
		if (first) {
			ml.setFirstGraphLayout(layout);
			ml.setFirstGraphLayoutActive(this._nodeLayoutActive);
		}
		else {
			ml.setSecondGraphLayout(layout);
			ml.setSecondGraphLayoutActive(this._linkLayoutActive);
		}
		
		// Before 22 april 2011, it was: 
		// layout.attach(new ibm_ilog.diagram.graphlayout.GraphAdapter(this, layout));
		// Fix for the configuration of local parameters (e.g. Hierarchical constraints) in 
		// subgraphs:
		layout.attach(rml.getGraphModel());
		
		layout._graphParameterChangedConnect =
			connect.connect(layout, "onParameterChanged", this, this._onLayoutParameterChanged);
	},
	
	_detachLayout: function(layout, first){
		// tags:
		//		private
		
		if (!layout) 
			return;
		
		var rml = this._rml;
		if (rml) {
			var ml = rml.getLayout(null);
			if (first) 
				ml.setFirstGraphLayout(null);
			else 
				ml.setSecondGraphLayout(null);
			
			if (!this._nodeLayout && !this._linkLayout) {
				rml.detach();
				this._rml = null;
			}
		}
		
		if(layout._graphParameterChangedConnect)
			connect.disconnect(layout._graphParameterChangedConnect);
	},
	
	_onLayoutParameterChanged: function(parameterName){
		// summary:
		//		Invalidates subgraph layouts when a layout property is changed.
		
		if (this._inLayoutParameterChanged || this._inAutoLayout) 
			return;
		if (parameterName == "ParametersUpToDate" || // User changed some other property.
				// Constraints of Hierarchical layout are not copied from parent
				// layout to inherited layout, hence we do not need to invalidate 
				// sublayouts when constraints are added to a parent layout.
				parameterName == "Constraints")
			return;
			
		try {
			this._inLayoutParameterChanged = true;
			
			var rml = this._rml;
			if (rml != null && !rml.isLayoutRunning()) {
				var ml = rml.getLayout(null);
				if (ml != null && !ml.isLayoutRunning()) {
					// OK, we are not in auto-layout, no layout is running,
					// we are not called recursively, we can do our stuff.					
					this._invalidateSubLayouts();
				}
			}
		}
		finally {
			delete this._inLayoutParameterChanged;
		}
	},
	
	_getParentGraph: function(){
		if (this._owningSubgraph) 
			return this._owningSubgraph.getParent();
		else 
			return null;
	},
	
	_invalidateSubLayouts: function(){
		for (var p = this; p != null; p = p._getParentGraph()) 
			p._sublayoutsInvalid = true;
	},
	
	_refreshSubLayouts: function(){
		// summary:
		//		Clears the layouts allocated by the layout provider for the subgraphs of this
		//		container. This method must be called every time a sub-layout is subject to change,
		//		that is:
		//		- when the toplevel layout itself changes (i.e., the GraphLayout.LinkLayout properties),
		//		- when a property of a toplevle layout changes,
		//		- when the Graph/LinkLayoutActive properties change
		//		- when a subgraph is added (because the sublayout must be attached immediately
		//		  if we are in auto-layout, so that the layout is done is something changes in
		//		  the subgraph.
		
		if (!this._sublayoutsInvalid) 
			return;
		delete this._sublayoutsInvalid;
		
		var rml = this._rml;
		if (rml != null) {
			var model = rml.getGraphModel();
			
			// Call the recursive method to go down into the subgraphs of this
			// container.
			//
			this._refreshSubLayoutsRec(rml.getLayoutProvider(), model, model);
		}
		
		var parent = this._getParentGraph();
		if (parent != null) 
			parent._refreshSubLayouts();
	},
	
	_refreshSubLayoutsRec: function(provider, topmodel, model){
		// summary:
		//		Recursive helper method for the previous method.
		
		var subgraphs = model.getSubgraphs();
		for (var i = 0; i < subgraphs.length; i++) {
			var submodel = topmodel.getGraphModel(subgraphs[i]);
			provider.setPreferredLayout(submodel, null);
			provider.getGraphLayout(submodel);
			this._refreshSubLayoutsRec(provider, topmodel, submodel);
		}
	},
    
	_checkAutoLayout: function(){
		// summary:
		//		Checks if an auto-layout must be performed and, if yes, schedule it.
		
		if (!this._inLayoutAnimation &&
		((this._nodeLayout && this._automaticNodeLayout && this._nodeLayoutActive) ||
		(this._linkLayout && this._automaticLinkLayout && this._linkLayoutActive))) {
			Batch.add(this);
		}
	},
	
	_endBatch: function(){
	
		var rml = this._rml;
		if (!rml) 
			return;
		
		try {
			if (!this._automaticNodeLayout) 
				rml.setFirstGraphLayoutActive(LayoutRunner.RecursiveMultipleLayout.INACTIVE);
			if (!this._automaticLinkLayout) 
				rml.setSecondGraphLayoutActive(LayoutRunner.RecursiveMultipleLayout.INACTIVE);
	
			this._inAutoLayout = true;
			
			this._performGraphLayout(true, true);
			
		}
		finally {
			this._inAutoLayout = false;
			
			if (!this._automaticNodeLayout) 
				rml.setFirstGraphLayoutActive(LayoutRunner.RecursiveMultipleLayout.MIXED);
			if (!this._automaticLinkLayout) 
				rml.setSecondGraphLayoutActive(LayoutRunner.RecursiveMultipleLayout.MIXED);
		}
	},
	
	_getActualLayout: function(first){
		// tags:
		//		private
		var toplevelGraph = null;
		for (var graph = this; graph != null; graph = graph._getParentGraph()) 
			toplevelGraph = graph;
		if (toplevelGraph && toplevelGraph._rml) {
			toplevelGraph._refreshSubLayouts();
			var ml = toplevelGraph._rml.getLayout(this._owningSubgraph);
			if(ml){
				if(first)
					return ml.getFirstGraphLayout();
				else
					return ml.getSecondGraphLayout();
			}
		}
		return null;
	},
	
	_getActualNodeLayout: function(){
		// summary:
		//		Gets the node layout that is actually executed on this graph.
		//		The actual layout can be inherited from another graph higher in the hierarchy.
		// tags:
		//		private
		return this._getActualLayout(true);
	},
	
	_getActualLinkLayout: function(toplevelGraph){
		// summary:
		//		Gets the link layout that is actually executed on this graph.
		//		The actual layout can be inherited from another graph higher in the hierarchy.
		// tags:
		//		private
		return this._getActualLayout(false);
	}
});

return LayoutRunner;

});

},
'ibm_ilog/diagram/overview/OverviewZoomInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewZoomInteractor", [
"dojo/_base/declare",
"./OverviewInteractor"
],
function(
declare,
OverviewInteractor
){

/*=====
var OverviewInteractor = ibm_ilog.diagram.overview.OverviewInteractor;
=====*/

var OverviewZoomInteractor =
declare("ibm_ilog.diagram.overview.OverviewZoomInteractor", [OverviewInteractor], {
	//
	//	summary:
	//		A ZoomInteractor controls the process of viewport panning and zooming
	//
	//	_incrementFactor: Number
	//		The factor at which the zoom is incremented at each discrete step
	//
	_incrementFactor: 1.5,



	setIncrementFactor: function ( /*Number*/ factor) {
		//
		//	summary:
		//		Sets the increment factor for zooming discrete operations
		//
		this._incrementFactor = factor;
	},

	getIncrementFactor: function () {
		//
		//	summary:
		//		Returns the increment factor for zooming discrete operations
		//
		return this._incrementFactor;
	},

	_declareStates: function () {
		this._declareState("idle", ["wheelZoom"]);
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		Returns the default event mapping
		//
		return {
			wheelZoom: {
				src: this._overview.getEventSource(),
				srcEvt: this.eventNameWheel,
				connectTo: "_wheelZoom",
				filter: this._noneFilter,
				gfxConnect: true
			}
		};
	},

	_wheelZoom: function ( /*Event*/ e) {
		//
		//	summary:
		//		Handles the wheel mouse event and dispatchs the appropriate zooming operations
		//
		var v = this._incrementFactor;

		if (this.eventWheelAmmount(e) < 0) {
			v = 1 / v;
		}

		this._viewport.setZoom(this._viewport.getZoom() * v);

		if (e.preventDefault) {
			e.preventDefault();
		}

		//	TODO: do we need this?
		e.returnValue = false;
	}

});

return OverviewZoomInteractor;

});

},
'ibm_ilog/diagram/overview/OverviewTemplateRenderer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewTemplateRenderer", [
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/array",
"./OverviewRenderer",
"../util/GraphUtil"
],
function(
declare,
lang,
array,
OverviewRenderer,
gu
){

/*=====
var OverviewRenderer = ibm_ilog.diagram.overview.OverviewRenderer;
=====*/

var OverviewTemplateRenderer =
declare('ibm_ilog.diagram.overview.OverviewTemplateRenderer', OverviewRenderer, {

	//
	//	summary:
	//		Uses custom templates to generate the overview image.
	//
	//	description:
	//		Iterates the given graph and reinstantiates the elements in the graph 
	//		with new templates. If no templates are specified, default ones are used.
	//	
	
	//
	//	node:  Function
	//		The node template function for the overview.
	//
	node: function(node){
		return node._template;
	},
	
	//
	//	link:  Function
	//		The link template function for the overview.
	//
	link: function(link){
		return link._template;
	},

	//
	//	subgraph:  Function
	//		The subgraph template function for the overview.
	//
	subgraph: function(subgraph){
		return subgraph._template;
	},

	//
	//	stripTexts: Boolean
	//		Optional removal of the texts from the generated shapes.
	//
	stripTexts: true,
	
	//
	//	stripBitmaps: Boolean
	//		Optional removal of the bitmaps from the generated shapes
	//
	stripBitmaps: true,

	constructor: function( kwArgs ) {
		if(kwArgs) {
			lang.mixin(this,kwArgs);
		}
		
		this._toFunction("node");
		this._toFunction("link");
		this._toFunction("subgraph");

	},
	
	_toFunction: function(name) {
		if(typeof this[name] != "function") {
			var value = this[name];
			this[name] = function(){
				return value;
			};
		}
	},

	_cloneGroup: function(parent,g) {
		var ng = parent.createGroup();
		if(g.getTransform){
			var t = g.getTransform();
			if(t) {
				ng.setTransform(t);
			}
		}
		return ng;
	},

	_stripShapes: function(childrenArray){
		var remove = [];
		array.forEach(childrenArray,function(child){
			if(!child.TID) {
				if (child.shape && ((this.stripTexts && child.shape.type=="text") || (this.stripBitmaps && child.shape.type=="image") )) {
					remove.push(child);
				} else {
					if (child.children) {
						this._stripShapes(child.children);
					}
				}
			}
		},this);
		array.forEach(remove,function(child){
			child.getParent().remove(child);
		},this);
	},

	
	render: function( graph, /*dojox.gfx.Group*/ output ) {
		
		var sub = function(ps,ns,g,b) {
			if(g.isCollapsed()) {
				ns.clear(); // TODO: cut iteration earlier instead of removing result of collapsed subgraphs
			}
			var subrep=ns.createGroup();
			subrep = g.shallowInstantiateAt(subrep,this.subgraph(g));
            g.layoutShape(subrep,true);
			subrep.moveToBack();
      if(this.stripTexts || this.stripBitmaps) {
        this._stripShapes(subrep.children);
    }
//
//	Draft of future version of this method:
//
//			ps.remove(ns);
//			var subrep=this.cloneGroup(ps,g);
//			subrep = g.shallowInstantiateAt(subrep,this.subgraph(g));
//			subrep._graph.add(ns);
//	        g.layoutShape(subrep);
//			subrep._graph.moveToFront();
		};

		var instantiator = function(templateF) {
			return function(ps,g){
				var t = templateF(g);
				var r = g.shallowInstantiateAt(this._cloneGroup(ps,g),t);
	            g.layoutShape(r,true);
	      if(this.stripTexts || this.stripBitmaps) {
	          this._stripShapes(r.children);
	      }
				return r;
			};
		};

		gu.fold(
			graph,
			output, // state
			gu.folders({
				sub:   sub,
				node:  instantiator(this.node),
				link:  instantiator(this.link),
				cBase: null,
				accGroup: this._cloneGroup,
				accGraph: this._cloneGroup,
				accSubgraph: this._cloneGroup
			}),
			this
		);

	}
});

return OverviewTemplateRenderer;

});


},
'ibm_ilog/diagram/overview/OverviewTopLevelTemplateRenderer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/OverviewTopLevelTemplateRenderer", [
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/array",
"dojo/_base/config",
"dojox/gfx/matrix",
"../templates/templates",
"./OverviewRenderer",
"../util/GraphUtil"
],
function(
declare,
lang,
array,
config,
m,
templates,
OverviewRenderer,
gu
){

/*=====
var OverviewRenderer = ibm_ilog.diagram.overview.OverviewRenderer;
=====*/

var OverviewTopLevelTemplateRenderer =
declare('ibm_ilog.diagram.overview.OverviewTopLevelTemplateRenderer', OverviewRenderer, {

	//
	//	summary:
	//		Uses custom templates to generate the overview image.
	//
	//	description:
	//		Iterates the given graph and reinstantiates the elements in the graph 
	//		with new templates. If no templates are specified, default ones are used.
	//	
	
	//
	//	node:  Function
	//		The node template function for the overview.
	//
	node: templates.defaultOverviewNodeTemplate,
	
	//
	//	link:  Function
	//		The link template function for the overview.
	//
	link: templates.defaultOverviewLinkTemplate,

	//
	//	expandedSubgraph:  Function
	//		The expanded subgraph template function for the overview.
	//
	expandedSubgraph: templates.defaultOverviewExpandedSubgraphTemplate,

	//
	//	collapsedSubgraph:  Function
	//		The collapsed subgraph template function for the overview.
	//
	collapsedSubgraph: templates.defaultOverviewCollapsedSubgraphTemplate,

	//
	//	stripTexts: Boolean
	//		Optional removal of the texts from the generated shapes.
	//
	stripTexts: true,

	constructor: function( kwArgs ) {
		if(kwArgs) {
			lang.mixin(this,kwArgs);
		}
		
		this._toFunction("node");
		this._toFunction("link");
		this._toFunction("expandedSubgraph");
		this._toFunction("collapsedSubgraph");

	},
	
	_toFunction: function(name) {
		if(typeof this[name] != "function") {
			var value = this[name];
			this[name] = function(){
				return value;
			};
		}
	},

	templateFor: function(ge) {
		return gu.dispatchClass(ge,arguments,this,{
			N:this.node,
			S:function(ge) {
				return ge.isCollapsed()?this.collapsedSubgraph():this.expandedSubgraph();
			},
      L:this.link
		});
	},
	
	renderGraphElement: function(ge) {
		
		var template = this.templateFor(ge);

		var ng = this._output.createGroup();
		
		if(ge.getTransform){
			var t = ge.getTransform();
			if(t) {
				ng.setTransform(t);
			}
		}

		var r = ge.shallowInstantiateAt(ng,template);
		
		if(!ge._isIBMDiagramLink){
			var oldBB = r.getLocalBoundingBox();
			var newBB = ge.getLocalBoundingBox();
		
			if(config.useGfxLayout) {
				r.layout(newBB);
			} else {
				r.applyLeftTransform( m.createFromRectangles( oldBB, newBB ));
			}
		} else {
    	ge.layoutShape(r);
		}
	},

	render: function( graph, /*dojox.gfx.Group*/ output ) {
		
		graph = graph || this.graph;
		
		this._output = output;
		
		array.forEach(graph.children,this.renderGraphElement,this);

	}
});

return OverviewTopLevelTemplateRenderer;

});

},
'ibm_ilog/diagram/interactors/SelectionKeyInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/SelectionKeyInteractor", [
"dojo/_base/declare",
"dojo/keys",
"./Interactor"
],
function(
declare,
keys,
Interactor
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var SelectionKeyInteractor =
declare("ibm_ilog.diagram.interactors.SelectionKeyInteractor", [Interactor], {
	//summary:
	//		This Interactor manages the Selection through the keyboard interaction.
	//		This Interactor have 4 connections: singleSelect, multiSelect, selectAll and clearSelection.		
	
	_diagram: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.				
		this._diagram = diagram;
		return this._initialize();
	},
	
	getSelection: function () {
		// Summary:
		//		return the selection object.
		// return: ibm_ilog.diagram.Selection
		return this._diagram.getSelection();
	},
	
	getKeyInteractorId: function(){
		// Summary:
		//		return the interactor Id.
		return SelectionKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			singleSelect: {
				hotKey:keys.SPACE,
				connectTo: "singleSelect",
				filter: this._buildInputFilter({ctrl:false})
			}, multiSelect: {
				hotKey:keys.SPACE,
				connectTo: "multiSelect",
				filter: this._buildInputFilter({ctrl:true})
			}, selectAll: {
				hotKey:65,/*letter a*/
				connectTo: "selectAll",
				filter: this._buildInputFilter({ctrl:true})
			}, clearSelection: {
				hotKey:keys.ESCAPE,
				connectTo: "clearSelection"
			}
		};
	},
	singleSelect: function(e){
		// summary:
	    //		This method clears the selection and adds the focused element to the selection.
	    // e: Event object
	    //		the event to be treated.
		var sel = this.getSelection();
		var ge = this._diagram.getFocusedElement();
		if(ge && sel.accept(ge)){
			sel.add(ge,true);
		}
	},
	multiSelect: function(e){
		// summary:
	    //		This method clears the adds the focused element to the selection or removed it if selected.
	    // e: Event object
	    //		the event to be treated.
		var sel = this.getSelection();
		var ge = this._diagram.getFocusedElement();
		if(ge){
			if(ge.isSelected()){
				sel.remove(ge);
			}else{
				if(sel.accept(ge)){
					sel.add(ge,false);
				}
			}
		}
	},
	selectAll: function(e){
		// summary:
	    //		This method adds all the graph elements to the selection
	    // e: Event object
	    //		the event to be treated.
		var graph = this._diagram.getGraph();
		this.getSelection().selectAll(graph);
	},
	clearSelection: function(e){
		// summary:
	    //		This method clears the selection
	    // e: Event object
	    //		the event to be treated.
		var sel = this.getSelection();
		sel.clear();
	}
});

SelectionKeyInteractor.KeyInteractorId = "Selection";

return SelectionKeyInteractor;


});


},
'ibm_ilog/diagram/graphlayout/_LayoutProvider':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/_LayoutProvider", [
"dojo/_base/declare",
"ibm_ilog/graphlayout/DefaultLayoutProvider",
"ibm_ilog/graphlayout/MultipleLayout",
"ibm_ilog/graphlayout/internalutil/TranslateUtil"
],
function(
declare,
DefaultLayoutProvider,
MultipleLayout,
TranslateUtil
){
	
/*=====
var DefaultLayoutProvider = ibm_ilog.graphlayout.DefaultLayoutProvider;
=====*/

var _LayoutProvider =
declare("ibm_ilog.diagram.graphlayout._LayoutProvider", DefaultLayoutProvider, {
	// summary:
	//		The layout provider that determines the layout to perform for each graph or subgraph.
	// tags:
	//		private

	constructor: function(graph){
		this._graph = graph;
	},
	
	getGraphLayout: function(graphModel){
	
		// Did we already create a layout for this container?
		// If yes, reuse it. This means that, whenever the layout must be
		// recreated (for example, because a property of the toplevel layout
		// is modified), we must clear all the layouts allocated by this provider
		// so that they are recreated next time. This is done by the refreshSubLayouts()
		// method.
		//
		var ml = this.getPreferredLayout(graphModel);
		if (ml) 
			return ml;
		
		// OK, no layout to reuse: create a new one, which must be a MultipleLayout
		// whose first layout is the GraphLayout property of this container or one
		// of its ancestors, and the second layout is the LinkLayout property.
		//
		// We determine which graph/link layout to use by looking
		// up the container hierarchy, and take the first non-null GraphLayout/LinkLayout
		// property.
		//
		var nodeLayout = null;
		var linkLayout = null;
		var nodeLayoutActive = true;
		var linkLayoutActive = true;
		
		var adapter;
		
		for (var adapter = graphModel; adapter != null; adapter = adapter.getParent()) {
		
			var graph = adapter.getGraph();
			
			// The Graph/LinkLayout is active iff the Graph/LinkLayoutActive
			// flag is true for all ancestors.
			//
			if (!graph._nodeLayoutActive) 
				nodeLayoutActive = false;
			if (!graph._linkLayoutActive) 
				linkLayoutActive = false;
			
			// Take the first non-null Graph/LinkLayout up in the hierarchy.
			//
			if (!nodeLayout && graph._nodeLayout) 
				nodeLayout = graph._nodeLayout;
			if (!linkLayout && graph._linkLayout) 
				linkLayout = graph._linkLayout;
			if (graph == this._graph) 
				break;
		}
		
		// We must copy the layouts for sub-graphs, but not for the container
		// that owns them, because the layout holds the node/link extended properties.
		//
		var copy = graphModel == null || graphModel._graph != this._graph;
		if (copy) {
			if (nodeLayout) 
				nodeLayout = nodeLayout.copy();
			if (linkLayout) 
				linkLayout = linkLayout.copy();
		}
		
		// Create and attach the MultipleLayout. This will attach the first and second layouts
		// automatically.
		//
		ml = new MultipleLayout(nodeLayout, linkLayout);
		
		var data = TranslateUtil.GraphModelData.getData(graphModel);
		data.setInternalGraphModelChecking(false);
		ml.attach(graphModel);
		this.setPreferredLayout(graphModel, ml);
		data.setInternalGraphModelChecking(false);
		ml.setFirstGraphLayoutActive(nodeLayoutActive);
		ml.setSecondGraphLayoutActive(linkLayoutActive);
		
		return ml;
	}
});

return _LayoutProvider;

});

},
'ibm_ilog/diagram/_base/gfxext':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/_base/gfxext", ["../main", "dojo/_base/lang", "dojo/_base/declare", "dojo/_base/config", "dojo/_base/array",
"dojo/_base/html", "dojo/_base/window", "dojo/_base/sniff", "dojo/dom",
"dojox/gfx", "dojox/gfx/shape", "dojox/gfx/utils", "dojox/gfx/matrix",
"dojo/has!config-isBidiInDojoDiagrammer?../util/BidiUtil:"],
function(iid, lang, declare, config, arr, html, win, has, dom, g, gs, gfxu, matrix, BidiUtil){

	// TODO:
	// For now we must create a global object since there are still references to ibm_ilog.diagram._base.gfxext
	// in the rest of the code, but when AMD migration is finished all global references should be replaced
	// by module references and this will be replaced by a simple object creation:
	// var ge = {};
	var ge = lang.getObject("ibm_ilog.diagram._base.gfxext", true);
	
	var renderer = g.renderer;
	switch (renderer) {
		case 'vml':
			iid.isVml = true;
			break;
		case 'svg':
			iid.isSvg = true;
			if (g.svg.useSvgWeb) {              
				iid.isSvgWeb = true;
			}			
			break;
		case 'silverlight':
			iid.isSilverlight = true;
			break;
		case 'canvas':
		case 'canvasWithEvents':
			iid.isCanvas = true;
			break;
	}
	
	iid.mobileEnabled = iid.isSilverlight ? false : true;
	
	// VML Container.add fix for Dojo >= 1.6:
	if (iid.isVml) {
		// Surface/Group.add in Dojo 1.6+ will call setShape and setFont recursively
		// on all children of the Group. This causes trouble in our code because we
		// trap these methods to notify changes in parents. To avoid this, we set
		// a global flag saying that we are inside add, and we should not propagate
		// notifications up. The flag is also checked in the MultilineText object
		// to avoid re-wrapping text.
		// Update in DD 1.1.1: Container.add is really too slow, we patch it to get acceptable perfs.
		// This can be turned off by dojoConfig { patchGfxAdd: false; }
		// Update: this is now done directly in GFX (config.fixVmlAdd) but wee keep this flag
		// just in case, because our fix should be slightly faster than GFX's patch
		// (we do not update at all any style while GFX updates the fill/stroke of the shape)
		var patchGfxAdd = config.patchGfxAdd;
		var oldAdd = !patchGfxAdd ? g.vml.Surface.prototype.add :
		function(shape){
			if (this != shape.getParent()) {
				this.rawNode.appendChild(shape.rawNode);
				gs.Container.add.call(this, shape);
			}
		};
		var myAdd = function(shape){
			var oldInAdd = iid._inContainerAdd;
			iid._inContainerAdd = true;
			oldAdd.call(this, shape);
			iid._inContainerAdd = oldInAdd;
			return this; // self
		};
		
		g.vml.Surface.prototype.add = myAdd;
		g.vml.Group.prototype.add = myAdd;
	}
	
	//
	// uniqueId generator
	//
	var _currentId = 0;
	iid._IdGenerator = {};
	iid._IdGenerator.nextId = function(obj) {
		return ++_currentId;
	};
	
	// "invisible location" values:
	if(!iid.isVml){
		iid._invisibleLocation = -10000;
		iid._invisibleSize = 1010;
	} else {
		iid._invisibleLocation = 100000;
		iid._invisibleSize = 10;
	}
	
	ge.patchMethod = function(/*Object*/type, /*String*/ method, /*Function*/ execBefore, /*Function*/ execAfter) {
		// summary:
		//     Patches the specified method of the given type so that the 'execBefore' (resp. 'execAfter') function is 
		//     called before (resp. after) invoking the legacy implementation.
		// description:
		//     The execBefore function is invoked with the following parameter:
		//     execBefore(method, arguments) where 'method' is the patched method name and 'arguments' the arguments received
		//     by the legacy implementation.
		//     The execAfter function is invoked with the following parameter:
		//     execBefore(method, returnValue, arguments) where 'method' is the patched method name, 'returnValue' the value
		//     returned by the legacy implementation and 'arguments' the arguments received by the legacy implementation.
		// type: Object: the type to patch.
		// method: String: the method name.
		// execBefore: Function: the function to execute before the legacy implementation. 
		// execAfter: Function: the function to execute after the legacy implementation.
		var old = type.prototype[method];
		type.prototype[method] = function() {
			var callee = method;
			if (execBefore) 
				execBefore.call(this, callee, arguments);
			var ret = old.apply(this, arguments);
			if (execAfter) 
				ret = execAfter.call(this, callee, ret, arguments) || ret;
			return ret;
		};
	};
	
	// SVGWeb fix
	if(iid.isSvgWeb){
		var svg = g.svg;
		var oldcs = svg.createSurface;
		svg.createSurface = function(parentNode, width, height){
			var s = oldcs.call(this, parentNode, width, height);
			s._parent = dom.byId(parentNode);
			return s;
		};
		if(svg != g)
			g.createSurface = svg.createSurface;
	}
	
	//
	// Silverlight event conversion fix
	// Fixes:
	//  1.  e.target should be mapped to EventArgs.Source
	//  2.  e.pageX/pageY are added so that it does not break the interactions code
	// COO: Code borrowed from dojox/gfx/silverlight.js
	if (iid.isSilverlight) {
		
		lang.extend(g.Group, {
			_setFillAttr : function() {}
		});
		
		
	    var surfaces = g.silverlight.surfaces;
		var mouseFix = function(s, a) {
			var ev = {
				target: s,
				currentTarget: s,
				preventDefault: function() {
				},
				stopPropagation: function() {
				}
			};
			if (a) {
				try {
					// BEGIN PATCH
					if (a.source) // source is defined from Silverlight 2+
						ev.target = a.source;
					// END PATCH
					ev.ctrlKey = a.ctrl;
					ev.shiftKey = a.shift;
					var p = a.getPosition(null);
					ev.x = ev.offsetX = ev.layerX = p.x;
					ev.y = ev.offsetY = ev.layerY = p.y;
					// calculate clientX and clientY
					var parent = s.getHost().parentNode;//surfaces[s.getHost().content.root.name];
					var t = html.position(parent);
					ev.clientX = t.x + p.x;
					ev.clientY = t.y + p.y;
					// BEGIN PATCH 2
					var d = (win.doc.documentElement && win.doc.documentElement.scrollLeft != null) ? win.doc.documentElement : win.doc.body;
					ev.pageX = ev.clientX + d.scrollLeft;
					ev.pageY = ev.clientY + d.scrollTop;
					// END PATCH 2
				} catch (e) {
					// squelch bugs in MouseLeave's implementation
				}
			}
			return ev;
		};
		var keyFix = function(s, a) {
			var ev = {
				keyCode: a.platformKeyCode,
				ctrlKey: a.ctrl,
				shiftKey: a.shift
			};
			// BEGIN PATCH
			if (a.source) // source is defined from Silverlight 2+
				ev.target = a.source;
			// END PATCH
			return ev;
		};
		var eventNames = {
			onclick: {
				name: "MouseLeftButtonUp",
				fix: mouseFix
			},
			onmouseenter: {
				name: "MouseEnter",
				fix: mouseFix
			},
			onmouseleave: {
				name: "MouseLeave",
				fix: mouseFix
			},
			onmouseover: {
				name: "MouseEnter",
				fix: mouseFix
			},
			onmouseout: {
				name: "MouseLeave",
				fix: mouseFix
			},
			onmousedown: {
				name: "MouseLeftButtonDown",
				fix: mouseFix
			},
			onmouseup: {
				name: "MouseLeftButtonUp",
				fix: mouseFix
			},
			onmousemove: {
				name: "MouseMove",
				fix: mouseFix
			},
			onkeydown: {
				name: "KeyDown",
				fix: keyFix
			},
			onkeyup: {
				name: "KeyUp",
				fix: keyFix
			}
		};
		var eventsProcessing = {
			connect: function(name, object, method) {
				var token, n = name in eventNames ? eventNames[name] : {
					name: name,
					fix: function() {
						return {};
					}
				};
				if (arguments.length > 2) {
					token = this.getEventSource().addEventListener(n.name, function(s, a) {
						lang.hitch(object, method)(n.fix(s, a));
					});
				} else {
					token = this.getEventSource().addEventListener(n.name, function(s, a) {
						object(n.fix(s, a));
					});
				}
				return {
					name: n.name,
					token: token
				};
			},
			disconnect: function(token) {
				this.getEventSource().removeEventListener(token.name, token.token);
			}
		};
		lang.extend(g.silverlight.Shape, eventsProcessing);
		lang.extend(g.silverlight.Surface, eventsProcessing);
		// COO END
	}
	
	//
	// Extends createShape to support our own shape types
	//
	
	var extendCreateShape = function(callee, ret, args) {
		if (!ret) {
			var sh = args[0];
			switch (sh.type) {
				case 'graph':
					ret = this.createObject(iid.Graph);
					break;
				case 'node':
					ret = this.createObject(iid.Node);
					break;
				case 'link':
					ret = this.createObject(iid.Link);
					break;
				case 'subgraph':
					ret = this.createObject(iid.Subgraph);
					break;
				case 'swimlane':
					ret = this.createObject(iid.SwimLane);
					break;
			}
		}
		return ret;
	};
	ge.patchMethod(g.Surface, "createShape", null, extendCreateShape);
	ge.patchMethod(g.Group, "createShape", null, extendCreateShape);
	
	var vmlFixList = [];
	
	var needsVmlFix = function(instance) {
		for (var i = 0; i < vmlFixList.length; ++i) {
			if (instance instanceof vmlFixList[i]) {
				return true;
			}
		}
		return false;
	};
	
	iid.gfxDeclaration = function(ctor) {
		ctor.nodeType = g.Group.nodeType;
		vmlFixList.push(ctor);
		return ctor;
	};
	
	iid.gfxDeclare = function(name, superclass, declarations) {
		//
		//	NOTE: warning, this may break dojo doc tools
		//
		var ctor = declare.apply(dojo, arguments);
		return iid.gfxDeclaration(ctor);
	};
	
	//
	// Fix VML createObject to perform additional initializations
	// performed in VML createObject implementation.
	//
	
	if (iid.isVml) {
		var fixVMLCreateObject = function(callee, ret, args) {
			var shape = ret;
			if ((iid.Node && (shape instanceof iid.Node)) ||
			(iid.Graph && (shape instanceof iid.Graph)) ||
			(iid.Subgraph && (shape instanceof iid.Subgraph)) ||
			(iid.Link && (shape instanceof iid.Link)) ||
			(needsVmlFix(shape))) {
			
				// Call Creator._overrideSize
				var node = shape.rawNode;
				this._overrideSize(node);
				
				// Need to do vml-specific createGroup tasks
				// see dojox.gfx.vml.js/createGroup()
				var r = shape.rawNode.ownerDocument.createElement("v:rect");
				r.style.left = r.style.top = 0;
				r.style.width = shape.rawNode.style.width;
				r.style.height = shape.rawNode.style.height;
				r.filled = r.stroked = "f";
				shape.rawNode.appendChild(r);
				shape.bgNode = r;
			}
		};
		ge.patchMethod(g.Surface, 'createObject', null, fixVMLCreateObject);
		ge.patchMethod(g.Group, 'createObject', null, fixVMLCreateObject);
	}
	
	//
	// Change notifications
	//
	
	ge.propagateChange = function(source, reason) {
		for (var shape = source; shape && !shape._noOnChanged && shape.getParent; shape = shape.getParent()) {
			if (shape._onChanged) 
				shape._onChanged(source, reason);
		}
	};
	
	(function() {
		var fixvml = iid.isVml;
		// a function used to propagate the change notification to the hierarchy
		var onChangeFunc = function(callee, ret, args) {
			if(iid._inContainerAdd) return;
			if (config.useGfxLayout) {
				this.__shapeSet(callee);
				// no need to invalidate top-level panels that are translated.
				var p = this.getParent();
				if (callee !== 'setTransform' || (p && p.getLayout && p.getLayout())) 
					this.invalidate();
			}
			ge.propagateChange(this, callee);
		};
		// make it accessible from outside (used in MultilineText as it overrides the setShape method
		// and therefore delete the patch
		ge.onChangePatch = onChangeFunc;
		 		
		// a patched function for vml to be called before the original function 
		var vmlFixRawNodePath = function() {
			if (!this.rawNode.path) 
				this.rawNode.path = {};
		};
		
		//Bidi support functions
		if (config.isBidiInDojoDiagrammer) {
		    //apply Bidi format on the text, using text direction and complex expression properties
    	    var bidiUpdateText = function(callee, args) {      
			    var sh = args[0];
			    if (sh === undefined){
				    return;
			    }
			    //for case the user call setShape()with argument containing a new textDir but not text, like text.setShape({textDir: "ltr"});
			    if (sh.text === undefined) {
				    if (this.getShape().__beforeBidi)
					    sh.text = this.getShape().__beforeBidi;
				    else {
					    if (this.getShape().text === undefined)
						    return;
					    sh.text = this.getShape().text;
				    }	
			    }
			    if (sh.textDir === undefined) {
			        if (sh.ceType) {
			           this.getShape().textDir = "ltr"; 
			        }
				    if (this.getShape().textDir === undefined) 
					    return;
				    sh.textDir = this.getShape().textDir; //for case like applyLayout(), when textDir is not set in argument shape.
			    }
                if (typeof(sh.text) != "string") {
				    sh.text = sh.text.toString();
                }
			    if (sh.__stopBidi || sh.text.length < 2) { //no need to apply bidi format
				    if (sh.__stopBidi) //flag __stopBidi can be used by user to prevent multiple transformation of the same text
					    delete sh.__stopBidi;
				    return;
			    }
			    
			    //store original, non processed text in __beforeBidi field
			    if (args[1] === undefined) {  
			    	if (sh.__beforeBidi) {    
			            this.getShape().__old = sh.__beforeBidi; //fix for print (deserialization), to fix second transformation of the same text during createText
			        }
			        sh.__beforeBidi = sh.text;
			    }  
			    else if (sh.__beforeBidi) // text direction was changed after the text was processed
			        sh.text = sh.__beforeBidi;  
			    
			    //need for additional/future shape processing, like applyLayout() 
                this.getShape().__beforeBidi = sh.__beforeBidi;  
			    
			    //complex expression case
			    if (sh.ceType) {
                    sh = BidiUtil.complExp.call(this,sh);
                    this.getShape().ceType = sh.ceType;		            
                    return true;
			    }
			    sh = BidiUtil.bidiFormat.call(this,sh);
			    return true;
		    };
      		
      		//set text direction to the shape, get if from argument, or
      		//from the parents, if possible.  
      		//then we call setShape with shape containing textDir parameter
      		var bidiUpdateShape = function(calee, textShape, args) {			
			    var sh = textShape.getShape();
			    var bidi = false;
			    if (!sh.textDir && args[0] && args[0].textDir)
				    sh.textDir = args[0].textDir;
    		  				
			    //trying to get textDir form parents			
			    if (!sh.textDir) {
				    bidi = true;
				    var sp = textShape;
				    var i = 0;
				    while (sp) {
					    if (sp.textDir) {
						    sh.textDir = sp.textDir;
						    break;
					    }
					    else if(sp.getTextDir) {
						    sh.textDir = sp.getTextDir();
						    break;
					    }
					    if (!sp.parent)
						    break;
					    sp = sp.getParent();
					    i++;
				    }
			    }
       			if (sh.__old){ //fix for print (deserialization), to fix second transformation of the same text during createText
    			    sh.text = sh.__old;
    			    delete sh.__old;
    			    if (args[0] && args[0].__beforeBidi)
    			        delete args[0].__beforeBidi;
    			    bidi = true;
    			} 			
			    if (sh.textDir && bidi) {
				    if (args[0] && args[0].ceType) //parameter for complex expression, for case, textDir was not set explicitly in the template
				        sh.ceType = args[0].ceType;	
				    textShape.setShape(sh);
			    }
    		
			    return textShape;
		    };	
		 }   
		ge.patchMethod(g.Rect, "setShape", null, onChangeFunc);
		ge.patchMethod(g.Circle, "setShape", null, onChangeFunc);
		ge.patchMethod(g.Ellipse, "setShape", null, onChangeFunc);
		ge.patchMethod(g.Line, "setShape", fixvml ? vmlFixRawNodePath : null, onChangeFunc);
		ge.patchMethod(g.Polyline, "setShape", fixvml ? vmlFixRawNodePath : null, onChangeFunc);
		ge.patchMethod(g.Path, "setShape", fixvml ? vmlFixRawNodePath : null);
		ge.patchMethod(g.Image, "setShape", null, onChangeFunc);
		ge.patchMethod(g.Text, "setShape", config.isBidiInDojoDiagrammer ? bidiUpdateText : null, onChangeFunc);
		ge.patchMethod(g.TextPath, "setShape", null, onChangeFunc);
		
		ge.patchMethod(g.Rect, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.Circle, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.Ellipse, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.Line, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.Polyline, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.Image, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.Text, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.TextPath, "setTransform", null, onChangeFunc);
		
		ge.patchMethod(g.Group, "setTransform", null, onChangeFunc);
		ge.patchMethod(g.Group, "add", null, onChangeFunc);
		ge.patchMethod(g.Group, "remove", null, onChangeFunc);
		ge.patchMethod(g.Group, "clear", null, onChangeFunc);
		ge.patchMethod(g.Group, "createText", null, config.isBidiInDojoDiagrammer ? bidiUpdateShape : null);  		
		
		ge.patchMethod(g.Text, "setFont", null, onChangeFunc);
		
	})();
	
	//
	// dojox.gfx.matrix.Matrix2D extensions
	//
	
	lang.mixin(matrix, {
		createFromRectangles: function(/* dojox.gfx.Rectangle */content, /*dojox.gfx.Rectangle */ viewport, /*Boolean*/ keepAspectRatio) {
			// summary:
			//        Creates a dojo.gfx.Matrix2d instance that allows to transform
			//        a rectangle so that it is visible in a viewport rectangle.
			// content:
			//        The content rectangle to display in the viewport.
			// viewport:
			//        The destination viewport.
			// keepAspectRatio:
			//		  If true, the resulting matrix preserves the aspect ratio, 
			//			so applying it to the content rectangle may not give exactly the viewport rectangle.
			//		  If false, the resulting matrix does not preserve the aspect ratio, 
			//			and it exactly transforms the content rectangle into the viewport rectangle.
			var sx = (Math.abs(viewport.width - content.width) < 0.001) ? 1 : viewport.width / content.width;
			var sy = (Math.abs(viewport.height - content.height) < 0.001) ? 1 : viewport.height / content.height;
			var t = new matrix.Matrix2D();
			if (keepAspectRatio) {
				var s = Math.min(sx, sy);
				t.xx = t.yy = s;
				t.dx = (viewport.x + viewport.width / 2 - s * (content.x + content.width / 2));
				t.dy = (viewport.y + viewport.height / 2 - s * (content.y + content.height / 2));
			} else {
				t.xx = sx;
				t.yy = sy;
				t.dx = viewport.x - sx * content.x;
				t.dy = viewport.y - sy * content.y;
			}
			t.xy = 0;
			t.yx = 0;
			return t;
		}
	});
	lang.extend(matrix.Matrix2D, {
		inverse: function() {
			// summary:
			//        Returns the inverse transformation.
			return matrix.invert(this);
		},
		isScale: function() {
			// summary:
			//        Indicates whether this transform corresponds to a scaling operation.
			return this.xy === 0 && this.yx === 0;
		},
		isTranslation: function() {
			// summary:
			//        Indicates whether this transform corresponds to a translation operation.
			return this.xy === 0 && this.yx === 0 && this.xx === 1 && this.yy === 1;
		},
		isIdentity: function() {
			// summary:
			//        Indicates whether this transform corresponds to the identity operation.
			return this.xy === 0 && this.yx === 0 && this.xx === 1 && this.yy === 1 && this.dx === 0 && this.dy === 0;
		},
		isRotation: function() {
			// summary:
			//        Indicates whether this transform corresponds to a rotation operation.
			return (this.xy === 0 && this.yx === 0) || (this.yx * this.xx + this.xy * this.yy === 0);
		},
		isInvertible: function() {
			// summary:
			//        Indicates whether this transform is invertible.
			return this.xx * this.yy - this.yx * this.xy !== 0;
		},
		isSame: function(t) {
			// summary:
			//        Indicates whether this transform and the specified transform are the same.
			return this.xx === t.xx && this.xy === t.xy && this.yx === t.yx &&
			this.yy === t.yy &&
			this.dx === t.dx &&
			this.dy === t.dy;
		},
		transformPoints: function(points) {
			// summary:
			//        Applies this transformation to an array of points.
			return arr.map(points, function(p) {
				return matrix.multiplyPoint(this, p);
			}, this);
		},
		transformPoint: function(/* dojox.gfx.Point */p) {
			// summary:
			//        Applies this transformation to the specified point.
			return matrix.multiplyPoint(this, p);
		},
		transformRectangle: function(/* dojox.gfx.Rectangle */rect) {
			// summary:
			//        Applies the transformation to a rectangle.
			// description:
			//        The method applies the transformation on all corners of the
			//        rectangle and returns the smallest rectangle enclosing the 4 transformed
			//        points.            
			rect = rect ||
			{
				x: 0,
				y: 0,
				width: 0,
				height: 0
			};
			if (this.isIdentity()) {
				return {
					'x': rect.x,
					'y': rect.y,
					'width': rect.width,
					'height': rect.height
				};
			}
			var m = matrix;
			var p0 = m.multiplyPoint(this, rect.x, rect.y);
			var p1 = m.multiplyPoint(this, rect.x, rect.y + rect.height);
			var p2 = m.multiplyPoint(this, rect.x + rect.width, rect.y);
			var p3 = m.multiplyPoint(this, rect.x + rect.width, rect.y + rect.height);
			var minx = Math.min(p0.x, p1.x, p2.x, p3.x);
			var miny = Math.min(p0.y, p1.y, p2.y, p3.y);
			var maxx = Math.max(p0.x, p1.x, p2.x, p3.x);
			var maxy = Math.max(p0.y, p1.y, p2.y, p3.y);
			var r = {};
			r.x = minx;
			r.y = miny;
			r.width = maxx - minx;
			r.height = maxy - miny;
			return r;
		},
		transformVector: function(/* dojox.gfx.Point || dojox.gfx.Size */p) {
			// summary:
			//        Applies the transformation to a vector.
			var x, y, r = {};
			if (this.isIdentity()) {
				return p;
			}
			if ('x' in p && 'y' in p) { // point
				x = p.x;
				y = p.y;
				r.x = (x * this.xx) + (y * this.xy);
				r.y = (x * this.yx) + (y * this.yy);
			} else if ('width' in p && 'height' in p) { // size
				x = p.width;
				y = p.height;
				r.width = (x * this.xx) + (y * this.xy);
				r.height = (x * this.yx) + (y * this.yy);
			}
			return r;
		}
	});
	
	// dojox.gfx.Text.getBoundingBox fix
	_isOrphan = function(shape) {
		// a Surface does not have getParent() method.
		/*
		 for (var p = shape.getParent();p && p.getParent; p=p.getParent()){
		 }
		 return p === null;
		 */
		var p = shape.rawNode;
		while (p && p !== document.body) {
			if (p.style && p.style.display === 'none') 
				return true;
			p = p.parentNode;
		}
		return p !== document.body;
	};
	
	lang.extend(g.Text, {
		getBoundingBox: function() {
			var sh = this.getShape(), sz;
			sz = this.__getTextBBox(sh.text);
			return sz;
		},
		
		__getTextBBox: function(s) {
			if (!s || s.length == 0) 
				return null;
			if (iid.isSvg && !iid.isSvgWeb) {
				// try/catch the FF native getBBox error. cheaper than walking up in the DOM
				// hierarchy to check the conditions (bench show /10 )
				try {
					return this.rawNode.getBBox();
				} catch (e) {
					return null;
				}
			} else if (iid.isVml) {
				// mandatory fix or IE crash in the vgx.dll (vml impl.)
				if (_isOrphan(this)) {
					return null;
				}
				var rawNode = this.rawNode, _display = rawNode.style.display;
				rawNode.style.display = "inline";
				
				// [av] added ||0 to avoid the IE crashes.
				// TODO remove ||0 and solve NaNs from "auto" settings in IE properly
				var w = g.pt2px(parseFloat(rawNode.currentStyle.width)) || 0;
				var h = g.pt2px(parseFloat(rawNode.currentStyle.height)) || 0;
				var sz = {
					x: 0,
					y: 0,
					width: w,
					height: h
				};
				// in VML, the width/height we get are in view coordinates
				var m = this._getRealMatrix();
				if (m && !m.isIdentity()) {
					sz = m.inverse().transformRectangle(sz);
				}
				// It's impossible to get the x/y from the currentStyle.left/top,
				// because all negative coordinates are 'clipped' to 0.
				// (x:0 + translate(-100) -> x=0
				this._computeLocation(sz);
				rawNode.style.display = _display;
				return sz;
			} else if (iid.isSilverlight) {
				var bb = {
					width: this.rawNode.actualWidth,
					height: this.rawNode.actualHeight
				};
				return this._computeLocation(bb,0.75); // 0.75: rough approx. of ascent...	
			} else if (this.getTextWidth) {
				if(iid.isSvgWeb && !this.parent)
					return null;
				var w = this.getTextWidth();
				var font = this.getFont();
				var fz = font ? font.size : g.defaultFont.size;
				var h = g.normalizedLength(fz);
				// TODO check if 0,0 is ok or needs to be corrected.
				sz = {
					width: w,
					height: h
				};
				this._computeLocation(sz, 0.85); // 0.85: rough approx. of ascent...
				return sz;
			}
		},
		
		_computeLocation: function(sz, coef) {
			var width = sz.width, height = sz.height, sh = this.getShape(), align = sh.align;
			switch (align) {
				case 'end':
					sz.x = sh.x - width;
					break;
				case 'middle':
					sz.x = sh.x - width / 2;
					break;
				case 'start':
				default:
					sz.x = sh.x;
					break;
			}
			coef = coef || 1;
			sz.y = sh.y - height*coef; // rough approximation of the ascent!...
			return sz;
		}
	});
	
	//
	// selectedStyle property
	//
	
	var SelectedStyleMixin = {
		setSelectedStyle:function(style) {
			this.selectedStyle = style;
			this.onSelectedStyleChanged();
		},
		getSelectedStyle:function() {
			return this.selectedStyle;
		},
		onSelectedStyleChanged: function() {
			var ge = iid.GraphElement.byShape(this);
			if(ge) {
				this.__selectedStyleChanged = true; // flag to force re-applying of selected style
				ge._changeSelectionStyle(ge.isSelected());
			}
		},
		__eod: undefined
	};
	lang.extend(gs.Shape, SelectedStyleMixin);
	
	// VML/IE8: patch for cases where surface is created outside of the DOM.
	//
	if(iid.isVml && has("ie") >= 8){
		var _patchMethod = function(method, field){
			var shapeProto = g.vml.Shape.prototype;
			var old = shapeProto[method];
			shapeProto[method] = function(a){
				var s = this.getParent();
				while(s && s.getParent) s = s.getParent();
				if(s && _isOrphan(s)){
					this[field] = a;
					if(s){
						if(s._parent && !s._timer){
							//console.log("watching orphan surface...");
							s._timer = setTimeout(lang.hitch(s, _timer), 100);
						}
						return this;
					} else {
						return old.call(this, a); // no surface?
					}
				} else {
					return old.call(this, a);
				}
			};
		};
		var _timer = function(){
			var s = this;
			if(!s._parent){
				//console.log("surface disposed.");
				delete s._timer;
			} else if(_isOrphan(s)){
				// still not added: continue waiting
				//console.log("surface still orphan: continue waiting...");
				s._timer = setTimeout(lang.hitch(s, _timer), 100);
			} else {
				delete s._timer;
				//console.log("surface added: fixing");
				// remove/add surface node (don't know why exactly but hey that's VML...)
				s._parent.removeChild(s.clipNode);
				s._parent.appendChild(s.clipNode);
				// reset all shapes and styles on all shapes
				gfxu.forEach(s, function(shape){
					if(shape != s){
						if(shape.setShape) shape.setShape(shape.shape);
						if(shape.setFill) shape.setFill(shape.fillStyle);
						if(shape.setStroke) shape.setStroke(shape.strokeStyle);
					}
				}, s);
			}
		};
		_patchMethod("setShape", "shape");
		_patchMethod("setFill", "fillStyle");
		_patchMethod("setStroke", "strokeStyle");
	}
	
	return ge;
});

},
'ibm_ilog/diagram/Port':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/Port", ["./PortBase", "./BasicPort", "./AutomaticPort"],
function(PortBase, BasicPort, AutomaticPort){
	return {
		PortBase: PortBase,
		BasicPort: BasicPort,
		AutomaticPort: AutomaticPort
	};
});

},
'ibm_ilog/diagram/gfxwidget/GfxWidget':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxwidget/GfxWidget", [
"require",
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/connect",
"dojo/dom-style",
"dojox/gfx"
], function(
require,
declare,
lang,
connect,
style,
gfx
){

/*=====
var gfx = dojox.gfx;
=====*/

var GfxWidget =
declare('ibm_ilog.diagram.gfxwidget.GfxWidget', [gfx.Rect], {
	// summary:
	//		A custom GFX shape that allows to put a Dijit widget in a Dojo Diagrammer node or link.
	// description:
	//		The <code>GfxWidget</code> class is a custom GFX shape that is intended to be used in
	//		Dojo Diagrammer node and link templates to integrate Dijit widgets in nodes or links.
	//		The widget type can be specified either by specifying the <code>dojoType</code> property
	//		of the shape, or by passing a function as the <code>createWidget</code> property of the shape.
	//		<p>
	//		Note that this special shape has a number of limitations:
	//		<ul>
	//		<li>The widget does not zoom when the graph is zoomed (only its position will be updated).</li>
	//		<li>The stacking order of widgets may be inconsistent with the order of their parent group shapes.</li>
	//		<li>If the shape is used outside of a Diagram widget, the HTML element that contains the surface
	//		must have its <code>position</code> style set to <code>absolute</code>.</li>
	//		</ul>
	//		</p>
	//	example:
	//		See the widgets/widgets.html sample for examples.
	
	_widget:null,
	
	constructor: function(){
		// summary:
		//		Creates an empty GfxWidget instance.
		// description:
		//		You must call <code>setShape</code> to specify the widget class and attributes.
		
		this.shape = lang.clone(GfxWidget.defaultWidget);
		this.bbox = null;
	},
	
	setShape: function(shape){
		// summary:
		//		Sets the attributes of the GfxWidget shape.
		// description:
		//		The <code>shape</code> parameter is an object with the following properties:
		//		<table style="border-collapse:collapse;" border="1">
		//			<tr>
		//				<th><b>Property</b></th>	<th><b>Description</b></th>
		//			</tr>
		//			<tr>
		//				<td>type</td>				<td>must be <code>'widget'</code></td>
		//			</tr>
		//			<tr>
		//				<td>dojoType</td>			<td>The type of the widget, for example <code>'dijit.form.Button'</code>. If <code>createWidget</code> is specified, dojoType is ignored.</td>
		//			</tr>
		//			<tr>
		//				<td>createWidget</td>		<td>A function that is called to create the widget. The function will be called with the <code>GfxWidget</code> instance as argument.</td>
		//			</tr>
		//			<tr>
		//				<td>x,y</td>				<td>The position of the widget.</td>
		//			</tr>
		//			<tr>
		//				<td>width,height</td>		<td>The size of the shape. If no size is specified, the size of the widget is used.</td>
		//			</tr>
		//			<tr>
		//				<td><em>widget attribute</em></td>		<td>Any other property is passed to the widget using an <code>attr()</code> call. For example, <code>label: 'Click Here'</code> to set the label of a <code>Button</code>.</td>
		//			</tr>
		//		</table>
		// shape: Object
		//		A hash that contains the properties of the widget.
		
		shape = lang.mixin(lang.clone(GfxWidget.defaultWidget), shape);
		
		if (shape) {
			if (!this._widget) {
				var w = null;
				
				if (typeof(shape.createWidget) == 'function') {
					w = shape.createWidget.call(null, this);
					this._setupWidget(w);
				} else if (shape.dojoType) {
					require([shape.dojoType.replace(/\./g, "/")], lang.hitch(this, function(c){ // may be asynchronous
						w = new c(shape);
						this._setupWidget(w);
						this._setWidgetAttrs(shape)
					}));
					this.shape = shape;
					this.bbox = null;
					return this;
				}
			}
			
			this._setWidgetAttrs(shape)
		}
		
		this.shape = shape;
		this.bbox = null;
		
		return this;
	},
	
	_setupWidget: function(w) {
		if (w) {
			this._widget = w;
			w._gfxShape = this;
			w.startup();
			var n = w.domNode;
			n.style.position = "absolute";
		}
	},

	_setWidgetAttrs: function(shape){
		if (this._widget) {
			for (var p in shape) {
				if (!(p in GfxWidget.defaultWidget)) {
					this._widget.set(p, shape[p]);
				}
			}
		}
	},
	
	prepare: function(container) {
		//this._container = container;
		//this._container.appendChild(this._widget.domNode);
	},
	
	_setParent: function(parent){
		var ret = this.inherited(arguments);
		
		for (var p = this.getParent(); p != null; p = p.getParent ? p.getParent() : null) {
			if(p instanceof gfx.Surface){
				p._parent.appendChild(this._widget.domNode);
				break;
			} else {
				connect.connect(p, "setTransform", this, this._updateWidgetPosition);
				if(p._isIBMDiagramLink)
					connect.connect(p, "validateLinkShape", this, this._updateWidgetPosition);
			}
		}
		
		this._updateWidgetPosition();
		
		return ret;
	},


	_minHeight: 0,
	
	_minWidth: 0,

	_minFontPxSize: 0,

	setWidth: function(w) {
		if(this._minWidth && w<this._minWidth) {
			w = this._minWidth;
		}
		style.set(this._widget.domNode,"width",w+"px");		
	},

	setHeight: function(h) {
		if(this._minHeight && h<this._minHeight) {
			h = this._minHeight;
		}
		style.set(this._widget.domNode,"minHeight",h+"px"); // hack for textarea
		style.set(this._widget.domNode,"height",h+"px");
	},

	setMinWidth: function(w) {
		this._minWidth = w;
		if(w && this.getWidth()<w) {
			this.setWidth(w);
		}
	},

	setMinHeight: function(h) {
		this._minHeight = h;
		if(h && this.getHeight()<h) {
			this.setHeight(h);
		}
	},

	getMinWidth: function() {
		return this._minWidth || 0;
	},

	getMinHeight: function() {
		return this._minHeight || 0;
	},
	
	setFontPxSize: function(sz) {
		if(this._minFontPxSize && sz<this._minFontPxSize) {
			sz = this._minFontPxSize;
		}
		style.set(this._widget.domNode,"fontSize",sz+"px");
	},
	
	setFontMinPxSize: function(sz) {
		this._minFontPxSize = sz;
		if(sz && this.getFontPxSize()<sz) {
			this.setFontPxSize(sz);
		}
	},


	getFontMinPxSize: function() {
		return this._minFontPxSize || 0;
	},

	getWidth: function() {
		return gfx.normalizedLength(style.get(this._widget.domNode,"width"));
	},

	getHeight: function() {
		return gfx.normalizedLength(style.get(this._widget.domNode,"height"));
	},

	getFontPxSize: function() {
		return gfx.pt2px(parseFloat(style.get(this._widget.domNode,"font-size")));
	},	

	setTransform: function(t){
		// summary:
		//		Overridden from the base GFX shape class.
		// tags:
		//		private
		
		this.inherited(arguments);
		this._updateWidgetPosition();
		return this;
	},
	
	getBoundingBox: function(){
		// summary:
		//		Overridden from the base GFX shape class.
		// tags:
		//		private
		
		if (!this.bbox) {
			if (this.shape.width && this.shape.height) {
				this.bbox = {
					x: this.shape.x,
					y: this.shape.y,
					width: this.shape.width,
					height: this.shape.height
				};
			}
			else {
				var rect = this._widget.domNode.getBoundingClientRect();
				this.bbox = {
					x: this.shape.x,
					y: this.shape.y,
					width: rect.right - rect.left,
					height: rect.bottom - rect.top
				};
			}
		}
		return this.bbox;
	},
	
	_updateWidgetPosition: function(){
		var box = this.getBoundingBox();
		var t = this._getRealMatrix();
		if(t)
			box = t.transformRectangle(box);
		this._widget.domNode.style.left = box.x + "px";
		this._widget.domNode.style.top = box.y + "px";
	}
});

GfxWidget._nextId = 1;
GfxWidget.defaultWidget = { type:'widget', x:0, y:0, width:0, height:0, r:0, dojoType:'', createWidget:'' };
GfxWidget.nodeType = gfx.Rect.nodeType;

GfxWidget.byWidget = function(widget){
	// summary:
	//		Returns the GfxWidget instance that created the specified widget.
	
	return widget._gfxShape || null;
};

//
// Extends createShape to support our own shape types
//

function extendCreateShape(shapeType) {
    var method = "createShape";
    var old = shapeType.prototype[method];
    shapeType.prototype[method] = function() {
        var ret = old.apply(this, arguments);
        if (!ret) {
            var sh = arguments[0];
            switch (sh.type) {
                case 'widget':
                    ret = this.createObject(GfxWidget, sh);
                    break;
            }
        }
        return ret;
    };
}
extendCreateShape(gfx.Surface);
extendCreateShape(gfx.Group);

return GfxWidget;

});

},
'ibm_ilog/diagram/base':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/base", ["./_base/gfxext", "./_base/diagext", "./_base/dojoext"],
function(){
	return ibm_ilog.diagram;
});

},
'ibm_ilog/diagram/widget/DivViewport':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/DivViewport", ["dojo/_base/declare", 
		"dojo/_base/sniff", 
		"dojo/_base/window", 
		"dojo/_base/connect", 
		"dojo/dom-attr", 
		"dojo/dom-style", 
		"dojox/gfx/matrix", 
		"dijit/registry", 
		"./Viewport", 
		"../util/HandleSet", 
		"../util/Geometry", 
		"../util/ErrorReporter"], 
		function(declare, has, window, connect, doma, domstyle, matrix, registry, Viewport, HandleSet, g, R){

	var Size = g.Size, PointZero = g.PointZero, Point = g.Point;
	
	function getScrollBarWidth(){
		var a = document.createElement('p');
		a.style.width = "100%";
		a.style.height = "200px";
		var b = document.createElement('div');
		b.style.position = "absolute";
		b.style.top = "0px";
		b.style.left = "0px";
		b.style.visibility = "hidden";
		b.style.width = "200px";
		b.style.height = "150px";
		b.style.overflow = "hidden";
		b.appendChild(a);
		document.body.appendChild(b);
		var w1 = a.offsetWidth;
		b.style.overflow = 'scroll';
		var w2 = a.offsetWidth;
		if (w1 == w2) 
			w2 = b.clientWidth;
		document.body.removeChild(b);
		return (w1 - w2);
	};
	
	/*=====
	 var Viewport = ibm_ilog.diagram.widget.Viewport;
	 var HandleSet = ibm_ilog.diagram.util.HandleSet;
	 =====*/
	var DivViewport = declare('ibm_ilog.diagram.widget.DivViewport', [Viewport, HandleSet], {
		//
		//	summary:
		//		This is a Viewport class based on an HTML <div> which overflows with scrollbars.
		//
		//	description:
		//
		//		DivViewport depends on a div declared at Diagram.html with attachPoint=scrollbarsNode.
		//		This div encloses the diagram Surface. 
		//		
		//		The diagram Surface is extended to enclose the whole diagram tightly. The 
		//		div is constrained to the desired client size using its width and height 
		//		properties. The view rectangle is then associated with the div scroll 
		//		positions, scrollTop and scrollLeft. Setting the view rectangle then 
		//		updates scrollTop and scrollLeft.
		//
		//		The zoom level is derived from the view rectangle, but is mapped entirely 
		//		different than its position. Instead, the zoom controls a scale transformation 
		//		of the entire Diagram. If the view rectangle is small, the Diagram is zoomed 
		//		in to fit the requested portion into the viewport window. With this also the 
		//		Surface is scaled, as it is required that the entire Graph fits the surface 
		//		exactly so that the scrollTop/scrollLeft properties work.
		//		
		//	note:
		//		the zooming service required from the Diagram object should be at a UI 
		//		level and should not affect, for example, the bouding box of the the Graph 
		//		objects. One option is to define an intermediate node between the Surface root 
		//		and the Graph node, which controls this transformation.
		//
		
		//
		//	The DOMNode of the div to which we bound
		//
		_scrollbars: null,
		
		_scrollbarsContent: null,
		
		_scrollbarWidth: 0, // TODO compute this
		_scrollCache: PointZero,
		
		_hScrollbarVisible: true,
		
		_vScrollbarVisible: true,
		
		templateString: function(){
			return "<div style='overflow:hidden; position:relative;' tabindex='-1' role='presentation'>" +
			"<div dojoAttachPoint='scrollbarsNode' style='z-index:0; position:absolute; top:0px; left:0px; overflow:auto' tabindex='-1' role='presentation'>" +
			"<div dojoAttachPoint='scrollbarsContentNode' style='visibility:hidden' tabindex='-1' role='presentation'></div>" +
			"</div>" +
			"<div dojoAttachPoint='canvasNode' style='z-index:1; position:absolute; top:0px; left:0px'></div>" +
			"</div>";
		},
		
		//
		//	_updatingScrollbars:
		//		true when scrollbar position is being updated and we will ignore related events
		//		useful in chrome since it reports events immediately.
		//
		_updatingScrollbars: false,
		
		_computeScrollbarWidth: function(){
			return (this._scrollbarWidth = getScrollBarWidth());
		},
		
		getScrollbarWidth: function(){
			return this._scrollbarWidth || this._computeScrollbarWidth();
		},
		
		getHudSize: function(){
			var hsz = this.getClientSize();
			var sbw = this.getScrollbarWidth();
			if (!this._hScrollbarVisible) {
				hsz.height += sbw;
			}
			if (!this._vScrollbarVisible) {
				hsz.width += sbw;
			}
			return hsz;
		},
		
		updateScrollbarVisibilityFlags: function(){
			var hv = (this._scrollbars.scrollWidth > this._scrollbars.clientWidth);//(this._viewRect.width<this._contentRect.width);
			if (this._hScrollbarVisible != hv) {
				this._hScrollbarVisible = hv;
				this._needsHudRelayout = true;
			}
			var vv = (this._scrollbars.scrollHeight > this._scrollbars.clientHeight);//(this._viewRect.height<this._contentRect.height);
			if (this._vScrollbarVisible != vv) {
				this._vScrollbarVisible = vv;
				this._needsHudRelayout = true;
			}
		},
		_getSurfaceXOffset:function(){
			return this.isRtl() ? (this._vScrollbarVisible ? this.getScrollbarWidth(): 0) : 0;
		},
		
		_onContentBoundingBoxChanged: function(){
			//
			//	summary:
			//		The function is executed when the content of the content has changed.
			//
			//	returns:
			//		True if the change affected content bounding box.
			//
			//	tags:
			//		private
			//
			this.inherited(arguments)
		},
		_canvasNode:null,
		initialize: function( /*ibm_ilog.diagram.widget.Diagram*/diagram){
			//
			//	summary:
			//		Initializes a Viewport for collaborating with the diagram.
			//
			//	diagram: ibm_ilog.diagram.widget.Diagram
			//		The diagram
			//
			this._scrollbars = diagram.scrollbarsNode;
			this._scrollbarsContent = diagram.scrollbarsContentNode;
			this._canvasNode = diagram.canvasNode;
			this.inherited(arguments);
			
			this._connect("onscroll", this._scrollbars, "onscroll", this, "_onScroll");
			if (!diagram.getParent()) 
				this._connect("onresize", has("ie") ? diagram.domNode : window.global, "onresize", this, "_onResize");
			
			var parent = registry.getEnclosingWidget(diagram.domNode.parentNode);
			
			// we are in dojox.mobile
			if (parent && typeof(parent.onAfterTransitionIn) == "function") {
				connect.connect(parent, "onAfterTransitionIn", this, "_onAfterTransitionIn");
				
			};
		},
		
		_debug: false,
		
		_onScroll: function(e){
			if (!this._updatingScrollbars) {
				var p = Point(this._scrollbars.scrollLeft, this._scrollbars.scrollTop);
				
				if (this._debug) {
					console.log("cached:", g.printRect(this._scrollCache), ", scroll:", g.printPoint(p));
				}
				
				// The following condition traps bouncing events generated by 
				// the receiver updating the scrollbar programatically
				
				if (!g.equalPoint(this._scrollCache, p)) {
					var q = this._pointScrollToModel(p);
					this.setViewRect(q, {
						instant: true
					});
				}
			}
		},
		
		_onResize: function(e){
			if (!this._diagram.domNode) 
				return; // widget has probably been removed from the DOM... TODO: remove the listener?
			var csz = this._computeInitialSize();
			this.setSize(csz, false);
		},
		
		_onAfterTransitionIn: function(){
			this.updateScrollbarVisibilityFlags();
			var csz = this._computeInitialSize();
			this.setSize(csz, false);
			
		},
		
		setClientSize: function( /*Size*/sz){
			//
			//	summary:
			//		Sets the client Size
			//
			this.setSize({
				width: sz.width + this.getScrollbarWidth(),
				height: sz.height + this.getScrollbarWidth()
			});
		},
		
		setSize: function( /*Size*/sz){
			//
			//	summary:
			//		Sets the Viewport Size
			//
			var sbw = this.getScrollbarWidth();
			
			var style = {
				style: {
					width: sz.width + "px",
					height: sz.height + "px"
				}
			};
			
			if (arguments.length < 2 || arguments[1]) 
				doma.set(this._diagram.domNode, style);
			doma.set(this._scrollbars, style);
			
			this._updateClientSize(Size(sz.width - sbw, sz.height - sbw));
			this._updateSurface();
		},
		
		
		_updateClientSize: function( /*Size*/sz){
			//
			//	summary:
			//		Updates viewportRect from the div attributes
			//
			//	tags:
			//		private
			//
			var vpH = sz.height || this._scrollbars.clientHeight;
			var vpW = sz.width || this._scrollbars.clientWidth;
			
			var newClientSize = {
				width: vpW,
				height: vpH
			};
			
			
			if (!g.equalRect(this._clientSize, newClientSize)) {
			
				this._clientSize = newClientSize;
				
				this._computeContentRect();
				
				switch (this._resizeMode) {
					case 1: // keep zoom level
						// Note that we do NOT set the height attribute. Explanation from Abel:
						// setViewRect can handle missing variables and they are 
						// inferred. We never want to change the aspect ratio, so this is a 
						// way of specifying priority: "take whatever height you need, but 
						// set width to what I give you"
						this.setViewRect({
							x: this._viewRect.x,
							y: this._viewRect.y,
							width: (newClientSize.width / this.getZoom())
						}, {
							instant: true
						});
						break;
					case 2:
						this._onViewRectConstraintsChanged();
						break;
					default:
						R.error("InvalidViewportResizeMode", this._resizeMode)
				}
				
				this.onClientSizeChanged();
				
			}
		},
		
		_pointContentToScroll: function(p){
			var r = g.subPoint(this._viewportMatrix.transformPoint(p), this._transformedContentRect);
			if (has("ff") && this._scrollCache && this.isRtl()) {
				//https://bugzilla.mozilla.org/show_bug.cgi?id=383026
				var corr = this._scrollCache.width - this._viewportMatrix.transformRectangle(this._viewRect).width;
				r.x = r.x - corr;
			}
			return r;
		},
		
		isRtl: function(){
			//	summary:
			//		returns true if running on RTL mode.
			//	note:
			//		Peller said in the forum that RTL is supported only if set at the body. This is a dijit limitation.
			return doma.get(window.doc.body, "dir") == "rtl";
		},
		
		_pointScrollToModel: function(p){
			if (has("ff") && this._scrollCache && this.isRtl()) {
				//https://bugzilla.mozilla.org/show_bug.cgi?id=383026
				var corr = this._scrollCache.width - this._viewportMatrix.transformRectangle(this._viewRect).width;
				p.x = corr + p.x;
			}
			return this._viewportMatrixInverse.transformPoint(g.addPoint(this._transformedContentRect, p));
		},
		
		onViewRectChanged: function onViewRectChanged(){
			this.updateScrollbarVisibilityFlags();
			this.inherited(arguments);
		},
		
		_applyViewRect: function( /*Boolean*/sizeChanged, /*Boolean*/ originChanged){
			//
			//	summary:
			//		Updates the surface size and _transformedContentRect based on the _viewRect and
			//		_contentRect. Called after a change in zoom or Graph content.
			//
			//	tags:
			//		private
			//
			
			if (!this.__initialized) {
				return;
			}
			
			this.inherited(arguments);
			
			// create transformation, apply if to Graph, and compute new transformedContentRect
			var zt = matrix.scale(this._zoom);
			var tt = matrix.translate(g.negPoint(this._viewRect)); // uses rect origin
			var t = matrix.multiply(zt, tt);
			
			this._transformedContentRect = t.transformRectangle(g.addRect(this._viewRect, this._contentRect));
			
			var scrollContentSize = g.floorSize(g.growRect(this._transformedContentRect, this.getClientSize()));
			// this._transformedContentRect;
			// [AV] replace by this go back to tight surface mode
			
			this._viewportMatrix = t;
			this._viewportMatrixInverse = t.inverse();
			
			var scrollCache = g.floorPoint(this._pointContentToScroll(this._viewRect));
			
			scrollCache.width = scrollContentSize.width;
			scrollCache.height = scrollContentSize.height;
			
			this._updatingScrollbars = true; // needed for chrome
			//this._disconnect("onscroll"); // does not seem to work for avoiding the event generated 
			// update in the following lines
			
			// "if" to avoid updating in the wrong order (shrinking width while position requires it, and viceversa.)
			// and to avoid updating unless strictly necessary
			if (scrollCache.width >= this._scrollCache.width) {
				if (this._scrollCache.width != scrollCache.width) {
					this._scrollbarsContent.style.width = scrollCache.width + "px";
				}
				if ((this._scrollCache.x) != (scrollCache.x)) {
					this._scrollbars.scrollLeft = scrollCache.x;
				}
			} else {
				if (this._scrollCache.x != scrollCache.x) {
					this._scrollbars.scrollLeft = scrollCache.x;
				}
				if (this._scrollCache.width != scrollCache.width) {
				
					this._scrollbarsContent.style.width = scrollCache.width + "px";
				}
			}
			
			if (scrollCache.height >= this._scrollCache.height) {
				if (this._scrollCache.height != scrollCache.height) {
					this._scrollbarsContent.style.height = scrollCache.height + "px";
				}
				if (this._scrollCache.y != scrollCache.y) {
					this._scrollbars.scrollTop = scrollCache.y;
				}
			} else {
				if (this._scrollCache.y != scrollCache.y) {
					this._scrollbars.scrollTop = scrollCache.y;
				}
				if (this._scrollCache.height != scrollCache.height) {
					this._scrollbarsContent.style.height = scrollCache.height + "px";
				}
			}
			
			scrollCache.x = this._scrollbars.scrollLeft; // sometimes the assignment fails, not setting the correct value
			scrollCache.y = this._scrollbars.scrollTop; // this gets the real into the cache
			this._scrollCache = scrollCache;
			
			this._updatingScrollbars = false;
			
			// give the change to the DOM to process the recent 
			// updates so we don't get their notifications
			//		setTimeout(dojo.hitch(this,function(){
			//			this._connect("onscroll",this._scrollbars,"onscroll",this,"_onScroll");
			//		}),1);
			
			this._setViewportTransform(t);
		}
		
		
	});
	
	
	return DivViewport;
	
});

},
'ibm_ilog/diagram/widget/ScrollbarInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/ScrollbarInteractor", [
"dojo/_base/declare",
"../interactors/DragInteractor",
"../util/Geometry"
], function(
declare,
DragInteractor,
g
){

/*=====
var DragInteractor = ibm_ilog.diagram.interactors.DragInteractor;
=====*/

var ScrollbarInteractor =
declare("ibm_ilog.diagram.widget.ScrollbarInteractor", [DragInteractor], {

	initialize: function( /*ibm_ilog.diagram.widget.Viewport*/viewport, /*ibm_ilog.diagram.widget.ScrollbarHudComponent*/ scrollbar ) {
		this._viewport = viewport;
		this._scrollbar = scrollbar;
		this._initialize();
		
		return this;
	},
	
	_declareStates: function () {
		this._declareState("idle", ["fit","start"]);
		this._declareState("active", ["first", "move", "end", "cancelOnDragStart", "cancelOnSelectStart"]);
	},
	
	getDefaultConnections: function() {
		var cs = this.inherited(arguments);
		cs.fit = {
				src: this._getInitialEventSource(),
				srcEvt: "onmousedown",
				connectTo: "_fitContents",
				filter: this._buildInputFilter({shift:true,button:0}),
				gfxConnect:true
			};
		return cs;
	},

	_getInitialEventSource: function() {
		return this._scrollbar._handle; 
	},
	
	_fitContents: function(e) {
		this._viewport.setViewRect(this._viewport.getContentRect());
	},

	_dragPreStart: function(e) {
		var v = this._viewport;
		this.inherited(arguments);
		this._firstViewRect = v.getViewRect();
	},
    
	_dragMove: function(e) {
    this.inherited(arguments);

    var P = this._scrollbar._posName;
    // scale delta to zoom level to get delta in model coords
    var delta = {};
    delta[P] = this._totalDelta[P];
    var panDelta = g.mulPoint(delta,this._scrollbar.getRatio());
		
    this._viewport.setViewRect( g.moveRect(this._firstViewRect,panDelta), {instant:true} );
  },
	
	_dragEnd: function(e) {
		//
		//	summary:
		//		Terminates the current panning process.
		//
		var v = this._viewport;
		var dragged = this.hasDragged; // inherited will clear this;
		this.inherited(arguments); // clears hadDragged
		if(!dragged) {
			this._viewport.centerAt(this.eventViewportLocation(e));
		}
	}
    
});

return ScrollbarInteractor;

});

},
'ibm_ilog/diagram/widget/ScrollbarHudComponent':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/widget/ScrollbarHudComponent", [
"dojo/_base/declare",
"dojo/_base/connect",
"dojox/gfx/matrix",
"./HudComponent",
"./ScrollbarInteractor",
"../util/Geometry"
], function(
declare,
connect,
matrix,
HudComponent,
ScrollbarInteractor,
g
){

/*=====
var HudComponent = ibm_ilog.diagram.widget.HudComponent;
=====*/

var ScrollbarHudComponent =
	declare('ibm_ilog.diagram.widget.ScrollbarHudComponent', HudComponent, {
		
		_tWidth: 16,

		_tPos: 0,

		_minHandleLength: 16, 
		
		_handleTrackPos: 0,

		_handleTrackLength: 16,
		
		_handleShape: {y:0,height:0},

		_setHorizontal:function() {
			this._posName="x";
			this._lenName="width";
			this._tPosName="y";
			this._tLenName="height";
		},
		
		_setVertical:function() {
			this._posName="y";
			this._lenName="height";
			this._tPosName="x";
			this._tLenName="width";
		},

		isHorizontal:function() {
			return this._posName=="x";
		},
		
		isVertical:function() {
			return this._posName=="y";
		},

		constructor: function (viewport, /*boolean*/ horizontal ) {
			
			if(horizontal) {
				this._setHorizontal();
			} else {
				this._setVertical();
			}			
			
			this._clientSizeChanged();

			this._shape.setTransform(matrix.translate(0, 0));

			var trackShape = {};
			trackShape[this._posName] = this._handleTrackPos;
			trackShape[this._lenName] = this._handleTrackLength;
			trackShape[this._tPosName] = this._tPos;
			trackShape[this._tLenName] = this._tWidth; // roundy shape
			this._track = this._shape.createRect(trackShape).setFill([0, 0, 0, 0.5]);

			var shape = {};
			shape[this._tPosName] = this._tPos;
			shape[this._tLenName] = this._tWidth; // roundy shape
			this._handle = this._shape.createRect(shape).setFill([0xAA,0xD7,0xFF,1]);

			this._updateLayout();
			
			connect.connect(viewport, "onViewRectChanged", this, "_updateLayout");
			connect.connect(viewport, "onClientSizeChanged", this, "_clientSizeChanged");
			connect.connect(viewport, "onClientSizeChanged", this, "_updateLayout");

			this._interactor = new ScrollbarInteractor().initialize(viewport,this);
			this._interactor.activate();

		},
		
		_clientSizeChanged: function() {
			var cz = this._viewport.getClientSize();
			this._handleTrackLength = cz[this._lenName]-this._handleTrackPos- this._tWidth;
			this._tPos = cz[this._tLenName] - this._tWidth;
		},
		
		_handleArea: function() {
			var L = this._lenName, P = this._posName, tP = this._tPosName;
			var vr = this._viewport.getViewRect();
			var cr = g.addRect(this._viewport.getContentRect(),vr);
			var r = {};
			
			var hl = vr[L]/cr[L]*this._handleTrackLength;
			if(hl<this._minHandleLength) {
				hl = this._minHandleLength;
				this._ratio = (cr[L]-vr[L])/(this._handleTrackLength-hl);
			} else { 
				this._ratio = cr[L]/this._handleTrackLength;
			}
			r[L] = hl;
			r[P] = (vr[P]-cr[P])/this._ratio;
			r[tP] = this._tPos;
			return r;
		},
		
		getRatio: function() {
			return this._ratio;
		},

		_updateLayout: function () {
			//
			//	summary:
			//		Corrects the position of the component after a resize of the viewport.
			//
			var L = this._lenName, P = this._posName; 
			var tL = this._tLenName, tP = this._tPosName; 

			if (this._enabled && this._viewport.isHudEnabled()) {
				var sh = this._handleArea();
				
				if( this._handleShape[L]  != sh[L] ||
					  this._handleShape[P]  != sh[P] || 
					  this._handleShape[tL] != sh[tL] || 
					  this._handleShape[tP] != sh[tP])
				{
					this._handleShape = sh;
					this._handle.setShape(sh);

					sh = {};
					sh[L]=this._handleTrackLength;
					sh[tP]=this._tPos;
					this._track.setShape(sh);
				}
			}
		}
	});

return ScrollbarHudComponent;

});

},
'ibm_ilog/diagram/gfxlayout/CanvasLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/gfxlayout/CanvasLayout", ["dojo/_base/declare", "dojox/gfx/matrix", "./Layout", "../util/Geometry"],
function(declare, matrix, Layout, Geometry){

/*=====
var Layout = ibm_ilog.diagram.gfxlayout.Layout;
=====*/

var CanvasLayout =
declare('ibm_ilog.diagram.gfxlayout.CanvasLayout', Layout, {

	getConfig:function(){
		// summary:
		// Gets the current configuration of this layout instance.
		// returns: Object
		//	An object that represents the layout current configuration.
		return {type:'ibm_ilog.diagram.gfxlayout.CanvasLayout'}; /*Object*/
	},

	computePreferredSize: function(proposedSize) {
		var children = this._container.children, count = children.length, i, bbox, sz;
		for (i = 0; i < count; ++i) {
			var child = children[i];
			sz = child.computePreferredSize({
				width: -1,
				height: -1
			});
			var rect = child.getBoundingBox();
			if (!rect) 
				continue;
			var t = child.getTransform();
			
			if (child.__oldLayoutTransform) {
				t = matrix.multiply(child.__oldLayoutTransform.inverse(), t);
			}
			
			if (t && !t.isIdentity()) 
				rect = t.transformRectangle(rect);
			rect.width = sz.width;
			rect.height = sz.height;
			bbox = bbox ? Geometry.addRect(rect, bbox) : rect;
		}
		return {
			width: bbox ? bbox.width : -1,
			height: bbox ? bbox.height : -1
		};
	},
	
	layoutChildren: function(contBounds) {
		var children = this._container.children, count = children.length, o, i;
		if (count === 0) 
			return;
		for (i = 0; i < count; ++i) {
			var child = children[i];
			// Process only panels (ie a Group with a Layout)
			if (!child.children || !child.getLayout()) 
				continue;
			var sz = child.getPreferredSize();
			child.layout({
				x: 0,
				y: 0,
				width: sz.width,
				height: sz.height
			});
		}
		// translate the graph so that the top-left corner of its content bbox
		// matches the layout origin (contBounds)
		var r = this._container.getBoundingBox();
		if (!r) 
			return;
		var t = this._container.getTransform();
		if (this._oldCanvasTransform) 
			t = matrix.multiply(this._oldCanvasTransform.inverse(), t);
		this._oldCanvasTransform = matrix.translate(contBounds.x - r.x, contBounds.y - r.y);
		t = matrix.multiply(t, this._oldCanvasTransform);
		this._container.setTransform(t);
	}
});

return CanvasLayout;

});

},
'ibm_ilog/diagram/overview/Overview':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/overview/Overview", [
"../main",
"dojo/_base/declare",
"dojo/_base/lang",
"dojox/gfx/matrix",
"../util/HandleSet",
"../util/Geometry"
],
function(
iid,
declare,
lang,
matrix,
HandleSet,
g
){

var Size = g.Size;

/*=====
var HandleSet = ibm_ilog.diagram.util.HandleSet;
=====*/

var Overview =
declare('ibm_ilog.diagram.overview.Overview', [HandleSet], {
//		summary:
//			Draws the overview as a GFX group.
//		
//		architecture:
//			The overview is a mini-diagram with no zooming or panning options that always shows the 
//			entire diagram, like a map.
//	
//			An Overview object draws a map-like representation of the contents of a viewport. It can be used 
//			through the OverviewWidget or directly (for example for using it to draw a component of the Viewport HUD).
//	
//			An Overview relies on an OverviewRenderer which encapsulates the algorithm for generating 
//			the overview image. Different algorithms provide different representation.
//	
//		view rectangle:
//			The Overview also represents the current location of the Viewport with a GFX Rectangle. 
//			Whenever the viewport is repositioned or zoomed, the Viewport object will notify the event, to 
//			which the Overview will react updating a rectangle shape.
//	
//		navigation:
//			See pointers on how the user could interact with the Overview, see the subclasses of OverviewInteractor
//			and how the Overview widget uses them.
//	

	
	//	
	//	_diagram: ibm_ilog.diagram.widget.Viewport
	//		The viewport
	//
	_viewport: null,
	
	//	
	//	_diagram: ibm_ilog.diagram.overview.OverviewRenderer
	//		The overview renderer
	//
	_renderer: null,

	//	
	//	_output: dojox.gfx.Group
	//		The insertion point into the Gfx Surface
	//
	_output: null,
	
	//	
	//	_render: dojox.gfx.Group
	//		Where the overview image is generated
	//
	_render: null,

	//	
	//	_content: Object
	//		The content renderer (usually the Graph).
	//
	_content: null,

	//	
	//	_hud: dojox.gfx.Group
	//		Where the overlay shapes are drawn (e.g. the view rectangie)
	//
	_hud: null,

	//	
	//	_enabled: boolean
	//		When false, the receiver hides its contents and ignores change events. 
	//
	_enabled: true,	

	setEnabled: function( /*boolean*/ enabled ) {
		if(!enabled && this._enabled) {
			this._output.remove(this._hud);
			this._render.clear();
			this._output.remove(this._render);
			if(this._timerHandle) {
				clearTimeout(this._timerHandle);
				this._timerHandle = null;
			}
			this._unsubscribe();
		} else if(enabled && !this._enabled) {
			this._output.add(this._hud);
			this._output.add(this._render);
			this._subscribe();
		}
		this._enabled = enabled;
	},
	
	isEnabled: function() {
			return this._enabled;
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//	Initialization
	//

	constructor: function( /*ibm_ilog.diagram.widget.Viewport*/ viewport,
			                                        /*Object*/ content,
	             /*ibm_ilog.diagram.overview.OverviewRenderer*/ renderer, 
	                                   /*dojox.gfx.Group*/ output, 
	                                              /*Size*/ size )
	{
		//
		//	summary:
		//		Constructs an Overview for observing the given viewport, whose contents will be drawn in an area 
		//		of the given size using the given renderer, and places it at the output gfx Group.
		//
		this._viewport = viewport;
		this._renderer = renderer;
		this._content = content;
		this._size = size;
		this._output = output;
		
		this._render = output.createGroup();

		// create hud group after render group so it is at the front
		this._hud = output.createGroup();
		this._createViewRectShape();
		this._createEventSourceShape();

		this._subscribe();
	},
	
	setContent: function( /*Object*/ content) {
		this._content = content;
	},
	
	getContent: function() {
		return this._content;
	},

	setViewport: function( /*Object*/ viewport) {
		this._viewport = viewport;
	},
	
	getViewport: function() {
		return this._viewport;
	},

	_unsubscribe: function () {
		this._disconnectAll();
	},
	
	_subscribe: function () {
  	this._connect("contentChanged",this._viewport.getContent(),"_onChanged",this,"_onContentChangedNotification");
  	this._connect("viewRectChanged",this._viewport,"onViewRectChanged",this,"_updateViewRectShape");
  	this._onContentChangedNotification();
  	this._updateViewRectShape();
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//
	//	Overview operation
	//

	generateContent: function() {

		//
		//	Draws the overview picture
		//
		
		var render = this._beginRender();
		
		this._renderer.render(this._content,render);
		
		this._endRender(render);
		
		this._adjustContent();

	},

	_beginRender: function() {
		if(this._debug) {
			return this._render;
		} else {
			var render = this._output.createGroup();
			var cr = this._viewport.getContentRect();
	
			// we render offscreen, and move it when it's done
			render.setTransform(matrix.translate(iid._invisibleLocation,iid._invisibleLocation));
			render.moveToBack();
			return render;
		}
	},
	
	geViewport: function() {
		return this._viewport;
	},
	
	_endRender: function( render ) {
		if(!this._debug) {
			if(this._render) {
				this._output.remove(this._render);
			}
			this._render = render;
			render.setTransform();
		}
	},

	setRenderer: function( /*ibm_ilog.diagram.overview.OverviewRenderer*/ renderer ) {
		this._renderer = renderer;
	},
	
	getRenderer: function() {
		//
		//	summary:
		//		Returns the current renderer object used at the receiver.
		//
		return this._renderer;
	},

	//	
	//	_zoom:
	//		The current zoom factor applied to the contents displayed by the
	//		overview to be fit in the current overview size.
	//
	_zoom: 1,
	
	getZoom: function() {
		return this._zoom;
	},

	//	
	//	_size: 
	//		The size of the overview
	//
	_size: Size(0,0),
	
	getSize: function() {
		return this._size;
	},

	setSize: function( /*Size*/ sz ) {
	    this._size = sz;
		this._adjustContent();
	},
	
	_adjustContent: function() {
		
		var aabb = this._viewport.getContentRect();
		if(aabb.width && aabb.height) {
			var sz = this._size;
			var xr = sz.width/aabb.width;
			var yr = sz.height/aabb.height;
			var cent = { x: 0, y:0 };
	
			if(xr<yr) {
				this._zoom = xr;
				cent.y = (aabb.width*(sz.height/sz.width)-aabb.height)/2;
			} else {
				this._zoom = yr;
				cent.x = (aabb.height*(sz.width/sz.height)-aabb.width)/2;
			}
	
			var zt = matrix.scale(this._zoom);
			var tt = matrix.translate({x:-aabb.x+cent.x,y:-aabb.y+cent.y}); // uses rect origin
			var t  = matrix.multiply(zt,tt);
			this._render.setTransform(t);
			
			this._updateEventSourceShape();
			this._updateViewRectShape();
		}
		this.onContentReady();
		
	},
	
	onContentReady: function() {
		
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	_viewRectShape: 
	//		The GFX rectangle used to represent the view rectangle
	//
	_viewRectShape: null,

	//	
	//	_viewRectShapeFill:
	//		The fill of the view rect shape
	//	
	_viewRectFill:[0xAA,0xD7,0xFF,0.2],
	
	setViewRectFill: function(fill) {
		this._viewRectFill = fill;
		if(this._viewRectShape) {
			this._viewRectShape.setFill(fill);
		}
	},

	//	
	//	_viewRectShapeStroke:
	//		The stroke for the border of the view rect shape
	//
	_viewRectStroke:{color:[0xAA,0xD7,0xFF]},

	setViewRectStroke: function(stroke) {
		this._viewRectStroke = stroke;
		this._updateViewRectShape();
	},
	
	
	_createViewRectShape: function() {
		this._viewRectShape = this._hud.createRect();
		this._viewRectShape.setFill(this._viewRectFill);
		this._updateViewRectShape();
	},

	_updateViewRectShape: function() {
		//
		//	Updates the view rectangle
		//
		if(this._viewRectShape) {
			var t = this._render.getTransform();
			if(t) {
				var vr = g.clipRect(g.standarizeRect(this._size),t.transformRectangle(this._viewport.getViewRect()));
				this._viewRectShape.setShape(vr);
				this._viewRectShape.setStroke({color:this._viewRectStroke.color,width:(this._viewRectStroke.width||1)});

			}
		}
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	_eventSourceShape:
	//		A shape used as an event generator for interaction. Current implementation uses a invisible rectangle 
	//		(created with alpha 0.0001) that catches mouse events in the entire group area.
	//
	_eventSourceShape: null,

	getEventSource: function() {
		return this._eventSourceShape;
	},

	_createEventSourceShape: function() {
		this._eventSourceShape = this._hud.createRect();
		this._eventSourceShape.setFill([0,0,0,0.0001]);
		this._updateEventSourceShape();
	},
	
	_updateEventSourceShape: function() {
		//	
		//	Updates the event source shape size to the current viewport navigable area
		//
		if(this._eventSourceShape) {
			// the hooks are required on ie8 to reset the interactor connections (in the OverviewWidget).
			// because Rect.setShape changes the rawNode. So the handlers are lost.
			this.__ieBeforeEvtSrcChange();
			var shape = this._eventSourceShape.getShape();
			var r = g.standarizeRect(this._size);
			if (r.width !== shape.width || r.height !== shape.height)
				this._eventSourceShape.setShape(r);
			this.__ieAfterEvtSrcChange();
		}
	},
	__ieBeforeEvtSrcChange:function(){},
	__ieAfterEvtSrcChange:function(){},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	_timerHandle:
	//		The timer scheduled by the refreshContentNotification request
	//
	_timerHandle:null,
	
	//
	//	_timerInterval:
	//		Refresh rate used for monitoring changes in the content.
	//
	_timerInterval: 1500,

	//
	//	_timerWaitForInteractions: boolean
	//		if true, the rendering timer waits for interactions to finish
	//
	_timerWaitForInteractions: true,

	setWaitForInteractions: function( bool ) {
		this._timerWaitForInteractions = bool ;
	},
	
	getWaitForInteractions: function() {
		return this._timerWaitForInteractions;
	},

	//
	//	_interactorManager: ibm_ilog.diagram.interactors.InteractorManager
	//		the interactor manager to wait for
	//
	_interactorManager: null,

	setInteractorManager: function( interactorManager ) {
		this._interactorManager = interactorManager;
	},
	
	getInteractorManager: function( interactorManager ) {
		return this._interactorManager;
	},

	setRefreshContentsInverval: function( /*Number*/ msInterval ) {
		this._timerInterval = msInterval;
		if(msInterval<0 && this._timerHandle) {
			clearTimeout(this._timerHandle);
			this._timerHandle = null;
		}
	},
	
	getRefreshContentsInverval: function() {
		return this._timerInterval;
	},

	_onContentChangedNotification: function(){
		//
		//	summary:
		//		Sets up a timer to give the originating process time to finish with all the changes
		//
		if(this._timerInterval>=0 && !this._timerHandle) {
			this._timerHandle = 
				setTimeout(lang.hitch(this,this._timer),this._timerInterval);
		}
	},
	
	_timer: function() {
		//
		//	summary:
		//		Callback executed after the contentChangedNotification timer is finished.
		//
		this._timerHandle = null;
		if(this._timerWaitForInteractions && this._interactorManager && this._interactorManager.stackLength()) {
			this._onContentChangedNotification();
		} else {
			this.generateContent();
		}
	},

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	
	//	clientToContent:
	//		
	//
	clientToContent: function() {
		return this._render._getRealMatrix().inverse();
	},


	contentToClient: function() {
		return this._render._getRealMatrix();
	}
});

return Overview;

});

},
'ibm_ilog/graphlayout/internalutil/GraphModelUtil':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
///////////////////////////////////////////////////////////////////////////////
// IBM ILOG Dojo Diagrammer 1.0
///////////////////////////////////////////////////////////////////////////////

// #######################################
// ######## NOT Generated by J2JS ########
// ###### DO NOT DELETE THIS FILE! #######
// #######################################

// ------------------------------------------------------------------------
// Utility methods used in the code generated by J2JS.
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/internalutil/GraphModelUtil", [
	"dojo/_base/lang",  // #1
	"dojo/_base/array", // #2
	"./FixedPoint", // #3 
	"./LinkStyles", // #4
	"./LogResUtil", // #5
	"../internalutil/TranslateUtil"	// #6
	],
	function(
		lang, // #1
		arr,  // #2
		FixedPoint, // #3 
		LinkStyles, // #4
		LogResUtil, // #5
		TranslateUtil // #6
		) {
		
	var graphlayoutNS = lang.getObject("ibm_ilog.graphlayout", true);
	var graphlayoutinternalNS = lang.getObject("ibm_ilog.graphlayout.internalutil", true);
	
	var ReshapeLinkMode = {
   		IGNORE: 0,
   		MOVE: 1,
   		FIX: 2,
   		CLIP: 3,
   		MOVE_AND_CLIP: 4
	};
	
	var ReshapeLinkStyle = {
    	IGNORE: LinkStyles.NO_RESHAPE_STYLE,
    	STRAIGHT: LinkStyles.STRAIGHT_LINE_STYLE,
    	ORTHOGONAL: LinkStyles.ORTHOGONAL_STYLE,
    	POLYLINE: LinkStyles.POLYLINE_STYLE,
    	DIRECT: LinkStyles.DIRECT_STYLE
	};

var GraphModelUtil = {
	
	getObjectBounds : function(model, obj)
	{
		var r = model.boundingBox(obj);
		if (r) 
			return {
				x: r.x,
				y: r.y,
				width: r.width,
				height: r.height
			};
		else 
			return null;
	},
	
    reshapeLink : function(model, layout, link, linkStyle, fromPoint, points, 
													startIndex, pointsLen, toPoint)
	{
	// summary:    
    //		Reshapes the points of a link.
	
	
	  var clip = layout.isLinkClipping();
	  
      var fromPointMode;
      if (fromPoint instanceof FixedPoint) {
        fromPointMode = ReshapeLinkMode.FIX;
      } else if (fromPoint != null) {
        fromPointMode = clip ? ReshapeLinkMode.MOVE_AND_CLIP : ReshapeLinkMode.MOVE;
      } else {
        fromPointMode = clip ? ReshapeLinkMode.CLIP : ReshapeLinkMode.IGNORE;
      }

      var toPointMode;
      if (toPoint instanceof FixedPoint) {
        toPointMode = ReshapeLinkMode.FIX;
      } else if (toPoint != null) {
        toPointMode = clip ? ReshapeLinkMode.MOVE_AND_CLIP : ReshapeLinkMode.MOVE;
      } else {
        toPointMode = clip ? ReshapeLinkMode.CLIP : ReshapeLinkMode.IGNORE;
      }
      
      var style;
      switch (linkStyle) {                
        case LinkStyles.ORTHOGONAL_STYLE:
        case LinkStyles.THREE_BENDS_ORTHOGONAL_STYLE:
        case LinkStyles.TWO_BENDS_ORTHOGONAL_STYLE:
          style = ReshapeLinkStyle.ORTHOGONAL;
          break;
        case LinkStyles.POLYLINE_STYLE:
          style = ReshapeLinkStyle.POLYLINE;
          break;
        case LinkStyles.DIRECT_STYLE:
          style = ReshapeLinkStyle.DIRECT;
          break;
        case LinkStyles.STRAIGHT_LINE_STYLE:
        default:
          style = ReshapeLinkStyle.STRAIGHT;
          break;
      }
      model.reshapeLink(link, style, 
                        fromPoint, fromPointMode,
                        points, startIndex, pointsLen, 
                        toPoint, toPointMode);
    },
	
    deleteIntermediatePointsOnLinks : function(graphModel, layout)
	{                                
    // summary:
    //		Deletes all the intermediate points (if any) of all the links.
     
      var link;
  	  var links = graphModel.getLinks();
	  	var i;
      if (layout.supportsPreserveFixedLinks()) {
        var isPreserveFixedLinks = layout.isPreserveFixedLinks();
        for (i = 0; i < links.length; i++) {
		  		link = links[i];
          if (!(isPreserveFixedLinks && layout.isFixed(link)))
            GraphModelUtil.reshapeLink(graphModel, 
							layout, link, LinkStyles.STRAIGHT_LINE_STYLE, 
							null, null, 0, 0, null);
        }
      } else {
        for (i = 0; i < links.length; i++) {
					link = links[i];
					GraphModelUtil.reshapeLink(graphModel, 
						layout, link, LinkStyles.STRAIGHT_LINE_STYLE, 
						null, null, 0, 0, null);
				}
      }
    },
	
	getNodesAndLinks : function(graphModel)
	{
		var nodes = graphModel.getNodes();
		var links = graphModel.getLinks();
		var result = [];
		var i;
		
		for(i = 0; i < nodes.length; i++)
			result.push(nodes[i]);
		for(i = 0; i < links.length; i++)
			result.push(links[i]);
		
		return result;
	},
	
	getLinks : function(graphModel, node)
	{
		var linksFrom = graphModel.getLinksFrom(node);
		var linksTo = graphModel.getLinksTo(node);
		var result = [];
		var i;
		
		for(i = 0; i < linksFrom.length; i++)
			result.push(linksFrom[i]);
		for(i = 0; i < linksTo.length; i++)
			result.push(linksTo[i]);
		
		return result;
	},

	getLinksCount : function(graphModel, node)
	{
		return graphModel.getLinksFromCount(node) + graphModel.getLinksToCount(node);
	},
	
	getOpposite: function(model, link, node)
	{
		if (node == model.getFrom(link)) 
			return model.getTo(link);
		if (node == model.getTo(link)) 
			return model.getFrom(link);
		LogResUtil.logAndThrowIllegalArgExc("GraphModelUtil", "graphlayout.expert.message.6010E");
	},
	
    getNeighbors : function(model, node)
	{
      var result = [];
      var neighbor;
      var link;
	  var links;
      var i;
      // OPTIMIZEME?
      
	  links = model.getLinksFrom(node);
      for(i = 0; i < links.length; i++) {
	  	link = links[i];
        neighbor = GraphModelUtil.getOpposite(model, link, node);
        if (neighbor != null && neighbor != node)
          result.push(neighbor);
      }
      
      for(i = 0; i < links.length; i++) {
	  	link = links[i];
        neighbor = GraphModelUtil.getOpposite(model, link, node);
        if (neighbor != null && neighbor != node)
          result.push(neighbor);
      }
      return result;
    },
	
    getNodeDegree : function(model, node)
	{
      var result = [];
      var neighbor;
      var link;
	  var links;
      var i;
      // OPTIMIZEME?
      
	  links = model.getLinksFrom(node);
      for(i = 0; i < links.length; i++) {
	  	link = links[i];
        neighbor = GraphModelUtil.getOpposite(model, link, node);
        if (neighbor != node && arr.indexOf(result, neighbor) == -1)
          result.push(neighbor);
      }
      
      for(i = 0; i < links.length; i++) {
	  	link = links[i];
        neighbor = GraphModelUtil.getOpposite(model, link, node);
        if (neighbor != node && arr.indexOf(result, neighbor) == -1)
          result.push(neighbor);
      }
      return result.length;
    },
	
	getModelBounds : function(model)
	{
		var nodes = model.getNodes();
		var links = model.getLinks();
		var bbox = null;
		var bb;
		var i;
		
		for (i = 0; i < nodes.length; i++) {
			bb = model.boundingBox(nodes[i]);
			if(bbox == null)
				bbox = { x: bb.x, y: bb.y, width: bb.width, height: bb.height };
			else
				TranslateUtil.addRect(bbox, bb);
		}
		for(i = 0; i < links.length; i++){
			bb = model.boundingBox(links[i]);
			if(bbox == null)
				bbox = { x: bb.x, y: bb.y, width: bb.width, height: bb.height };
			else
				TranslateUtil.addRect(bbox, bb);
		}
		
		if(bbox)
			return bbox;
		else
			return { x:0, y:0, width:0, height:0 };
	},
	
	move: function(model, x, y){
		var currentBBox = GraphModelUtil.getModelBounds(model);
		var dx = x - currentBBox.x;
		var dy = y - currentBBox.y;
		if (dx == 0 && dy == 0) 
			return;
		
		var nds = model.getNodes();
		if (nds.length == 0) 
			return;
		
		// Get the link points before any movement
		// because the end points position depend on
		// the position of the nodes
		var lks = model.getLinks();
		var link;
		var lprop;
		var lindex = 0;
		var linkPoints;
		var linksArray = null;
		
		// Get the link points before node movement
		// so that end points are correct.
		if (lks.length > 0) 
			linksArray = new Array(lks.length);
		var i;
		for (i = 0; i < lks.length; i++) {
			var link = lks[i];
			linkPoints = model.getLinkPoints(link);
			lprop = new Object();
			lprop.link = link;
			lprop.linkPoints = linkPoints;
			linksArray[lindex] = lprop;
			lindex++;
		}
		
		// First move nodes
		for (i = 0; i < nds.length; i++) {
			var node = nds[i];
			var bbox = model.boundingBox(node);
			model.moveNode(node, bbox.x + dx, bbox.y + dy);
		}
		
		// Reshape the link if the link is not straight.
		
		if (linksArray) {
			for (i = 0; i < linksArray.length; i++) {
				var lprop = linksArray[i];
				link = lprop.link;
				linkPoints = lprop.linkPoints;
				if (linkPoints != null /* AV: NO LONGER && (linkPoints.length > 2) */) {
					// Translate link points
					var lastIndex = linkPoints.length - 1;
					var j;
					for (j = 0; j < linkPoints.length; j++) {
						var p = linkPoints[j];
						p.x = p.x + dx;
						p.y = p.y + dy;
					}
					
					model.reshapeLink(link, ReshapeLinkStyle.IGNORE, null, 
						ReshapeLinkMode.IGNORE, linkPoints, 1, lastIndex - 1, null, 
						ReshapeLinkMode.IGNORE);
				}
			}
		}
	}
};

graphlayoutNS.ReshapeLinkMode = ReshapeLinkMode;
graphlayoutNS.ReshapeLinkStyle = ReshapeLinkStyle;
	
// also store internal enums in GraphModelUtil so they are accessible from the module value
GraphModelUtil.ReshapeLinkStyle = ReshapeLinkStyle;
GraphModelUtil.ReshapeLinkMode = ReshapeLinkMode;

graphlayoutinternalNS.GraphModelUtil = GraphModelUtil;

return GraphModelUtil;
});
},
'ibm_ilog/diagram/util/GraphUtil':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/GraphUtil", ["dojo/_base/lang", "dojo/_base/array", "dojo/_base/window", "dojox/gfx", "dojox/gfx/matrix", "dojox/lang/functional/lambda", "dojox/lang/functional/fold",
"dojox/collections/Set", "dojox/collections/ArrayList",
"./ErrorReporter", "./Geometry"],
function(lang, array, window, gfx, matrix, df, fold, dxcs, ArrayList, R, g){

	var foldl = df.foldl;
	
	// TODO: Change to this when AMD conversion is complete:
	// var gu = {};
	var gu = lang.getObject("ibm_ilog.diagram.util.GraphUtil", true);

gu.allGraphElements = function collectChildren(roots,result) {
	if(!result) {
		result = new ArrayList();
	}
	array.forEach(roots, function(e){
		if(e._isIBMDiagramGraphElement && !result.contains(e)) {
			result.add(e);
			var g;
			if(e._isIBMDiagramSubgraph && (g=e.getGraph().children)) {
				gu.allGraphElements(g,result);
			}
		}
	});
	return result;
};

gu.maximals = function maximals(gs) {
	//
	//	summary:
	//		Returns only the maximal containers for an array of graph elements
	//	s: Array
	//		the array of graph elements
	//

	var r = [];
	var subgraphs = [];

	array.forEach(gs,function(e){
		if(e._isIBMDiagramSubgraph) {
			subgraphs.push(e);
		}
	});
	
	array.forEach(gs,function(e){
		var keep = true;
		
		for( var i=0; keep && i < subgraphs.length; ++i ) {
			var s = subgraphs[i];
			if(s!=e && gu.isContainedBy(e,s)) {
				keep = false;
			}
		}
		if(keep) {
			r.push(e);
		}
		
	},this);
	
	return r;
};

gu.getOwningSubgraph = function getOwningSubgraph(ge) {
	for (var p = gu.safeGetParent(ge); p !== null && p.declaredClass != 'dojox.gfx.Surface'; p = gu.safeGetParent(p)) {
		if (p._isIBMDiagramSubgraph) {
			return p;
		}
	}
	return null;
};

gu.isStableVisible = function(ge){
	//
	//	returns:
	//		true of the graph element is visible in the surface
	//
  for (var p = ge.getParent();p && p.getParent; p=p.getParent()){
  	if(p._isIBMDiagramSubgraph && p._animating) {
  		return false;
  	}
  }
  return p !== null;
};

var _pairLowestCommonAncestor = function _pairLowestCommonAncestor(ga, gb) {
	var gaa;
	if(ga==gb) {
		return ga;
	}
	if (gu.isContainedBy(gb, ga)) {
		return ga;
	}
	if (gu.isContainedBy(ga, gb)) {
		return gb;
	}
	if(gaa = gu.getOwningSubgraph(ga)) {
		return _pairLowestCommonAncestor(gu.safeGetParent(gaa),gb);
	} else {
		return ga;
	}
};

gu.lowestCommonAncestor = function lowestCommonAncestor() {
	var elements = arguments;
	if(arguments[0] instanceof Array) {
		elements = arguments[0];
	}
	var a = elements[0].getParent();
	for(var i=1;i<elements.length;i++) {
		a = _pairLowestCommonAncestor(a,gu.safeGetParent(elements[i]));
	}
	return a;
};

gu.applyStyle = function(shape,style) {
	if(style.fill) {
		shape.setFill(style.fill);
	}
	if(style.stroke) {
		shape.setStroke(style.stroke);
	}
	if(style.font) {
		shape.setStroke(style.font);
	}
	if(style.selectedStyle) {
		shape.selectedStyle = style.selectedStyle;
	}
	if(style.shape) {
		shape.setShape(style.shape);
	}
	if(style.editable && style.editable.style) {
		shape.editable = shape.editable || {};
		shape.editable.style = style.editable.style;
	}
};

gu.getSurface= function(ge) {
	while(ge && ge.getParent) {
		ge = ge.getParent();
	};
	return ge;
};

gu.forEach = function(ge,f,scope) {
	//
	// Simple forEach style iteration through Graph entities
	//	
	scope = scope || window.global;
	if(ge) {
		var isSurface = ge instanceof gfx.Surface;
		var isSubgraph = ge._isIBMDiagramSubgraph;
		var isNode = ge._isIBMDiagramNode;
		var isLink = ge._isIBMDiagramLink;
		var isGroup = ge instanceof gfx.Group;
		
		if( isSurface || isLink || isNode || isSubgraph ) {
			f.call(scope,ge);
		}
		
		var subs = {};
		
		if( isSubgraph ) {
			subs = ge.getGraph().children;
		} else if( isGroup || isSurface ) {
			subs = ge.children;
		}

		array.forEach(subs,function(ge){
			gu.forEach(ge,f,scope);
		});
	}
};

gu.isContainedBy = function(element,container) {
	while(element){
		if(element==container) {
			return true;
		}
		element = gu.safeGetParent(element);
	}
	return false;
};

gu.safeGetParent = function(ge){
	if(ge.getParent && ge.getParent()){
		return ge.getParent();
	}else{
		if(ge._collapsedParent) {
			return ge._collapsedParent;
		}
	}
	return null;
};

gu.doAncestors = function(shape,cb,scope) {
	scope=scope||window.global;
	for(; shape && gu.safeGetParent(shape); shape = gu.safeGetParent(shape)){
		cb.call(scope,gu.safeGetParent(shape));
	}
};

gu.folders = function(kwArgs) {
	var ignore = function(){};
	return lang.mixin({
		graph: ignore,
		group: ignore,
		sub:   ignore,
		node:  ignore,
		link:  ignore,
		other: ignore,
		child: ignore,
		cBase: null,
		accGraph: ignore,
		accSubgraph: ignore,
		accGroup: ignore		
	},
		kwArgs
	);
};

gu.fold = function fold(e,ps,folders,scope,stopRecursionAtCollapsedSub) {
	//
	//	summary:
	//		Graph data structure fold function. --Functional Programming--
	//
	//	description:
	//		Recurses a Graphic offering callbacks to the user at each structural joint. In addition
	//		to the standard folder functions for each kind of entity, this function supports
	//		a state that is built forward on composite nodes [ for processing root-to-leaf instead
	//		of only folding leaf-to-root ].
	//      Also, it allows optionally to stop the recursion at the level of collapsed subgraphs,
	//      which are treated as a node.
	//
	//	types:
	//		G == Graphic
	//		FG == folded graphic
	//		FC == folded child collection
	//		ST == forward-built state
	//
	//	e:  /*G*/
	//		The Graphic to recurse
	//
	//	ps: /*ST*/
	//		the initial parent state
	//
	//	folders: {
	//
	//		graph: function( ST, ST, G, FG ){ return FG; },
	//			Fold a Graph
	//
	//		group: function( ST, ST, G, FG ){ return FG; },
	//			Fold a Group
	//
	//		sub:   function( ST, ST, G, FG ){ return FG; },
	//			Fold a Subgraph (receives folded inner Graph)
	//
	//		node:  function( ST, G ){ return FG; },
	//			Fold a Node
	//
	//		link:  function( ST, G ){ return FG; },
	//			Fold a Group node
	//
	//		other: function( ST, G ){ return FG; },
	//			Fold an unknown node (eventual gfx.Rects, etc.)
	//
	//		child: function( ST, FC, FG ){ return FC; },
	//			Fold a folded Graphic FG into a folded child-collection FC, returning the new collection
	//
	//		cBase: FC
	//			The initially empty folded child collection (constant)
	//
	//		accSubgraph: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//		accGraph: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//		accGroup: function( ST, G ){ return ST; }
	//			The forward-state accumulator, builds a new ST given a newly encountered G
	//
	//	}
	//
	//  stopRecursionAtCollapsedSub: 
	//      If true, the recursion stops at the level of collapsed subgraphs, which are
	//      treated as a node. 
	//
	//	NOTE: unstable API
	//
	scope = scope || window.global;
	
	var foldChildren = function(c,ns,stopRecursionAtCollapsedSub) {
			return foldl(c,
				function(z,j){
					return folders.child.call(scope,ns,z,gu.fold(j,ns,folders,scope,stopRecursionAtCollapsedSub));
				},folders.cBase);
	};
	
	if(e._isIBMDiagramSubgraph) { 
	   if (stopRecursionAtCollapsedSub && e.isCollapsed()) {
	     return folders.node.call(scope,ps,e);
	   } else {
	   	 var ns = folders.accSubgraph.call(scope,ps,e); 
	     return folders.sub.call(scope,ps,ns,e,gu.fold(e.getGraph(),ns,folders,scope,stopRecursionAtCollapsedSub));	   	
	   } 
	}
	else if(e._isIBMDiagramNode)     { return folders.node.call(scope,ps,e); }
	else if(e._isIBMDiagramLink)     { return folders.link.call(scope,ps,e); }
	else if(e._isIBMDiagramGraph)    { var ns = folders.accGraph.call(scope,ps,e); 
	                                             return folders.graph.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
	else if(e instanceof gfx.Group)            { var ns = folders.accGroup.call(scope,ps,e); 
	                                             return folders.group.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
	else { return folders.other.call(scope,ps,e); }
};

///////////////////////////////////////////////////////////////////////////////
//
// 	fold example 1: logXml(graph)
//

gu.logXml = function(g) {
	
	var acc = function(indent,e) {
		console.log(indent+"<"+e.declaredClass.slice(12)+" id="+(e.getId?e.getId():"?")+">");
		return indent+" ";
	};
	
	var fold = function(parIndent,selfIndent,g) {
		console.log(parIndent+"</"+g.declaredClass.slice(12)+">");
	};
	
	var leaf = function(parIndent,e) {
		console.log(parIndent+"<"+e.declaredClass.slice(12)+" id="+(e.getId?e.getId():"?")+"/>");
	};
	
	var ignore = function(){};

	gu.fold(
		g,
		"", // accumulated forward-state is the indentation string
		{
			graph: fold,
			group: fold,
			sub:   fold,
			node:  leaf,
			link:  leaf,
			other: ignore,
			child: ignore,
			cBase: null,
			accGraph: acc,
			accGroup: acc,
			accSubgraph: acc
		},
		this
	);
};

///////////////////////////////////////////////////////////////////////////////
//
// 	fold example 2: countNodes(graph)
//

gu.countNodes = function(g) {
	
	var add = function(st,siblingSum,childSum) {
		return siblingSum+childSum;
	};
	
	var one = function(){ return 1; };

	var zero = function(){ return 0; };

	var pass = function(ps,ns,g,fc){ return fc; };

	var nothing = function(){};

	return gu.fold(
		g,
		undefined, // forward state not used
		{
			graph: pass,
			group: pass,
			sub:   pass,
			node:  one,
			link:  zero,
			other: zero,
			child: add,
			cBase: 0,
			accGraph: nothing,
			accGroup: nothing,
			accSubgraph: nothing
		},
		this
	);
};


gu.dispatchClass = function(ge,args,context,handlers) {
	return handlers[ge.TID].apply(context,args);
};

gu._howGotHere = function(shape) {
	//
	//	summary:
	//		Debugging helper. Dumps the offsets contributed by 
	//		each transformation in the ancestors of a shape.
	//
	var r, t;
	var print = g.printPoint;
	
	while(shape) {
		r = shape._getRealMatrix();
		
		if(t = shape.getTransform && shape.getTransform()) {
			var p = {x:t.dx / t.xx, y:t.dy/t.yy};
			var d = null; 
			
			var p1 = gu.safeGetParent(shape);
			p1 = p1 && p1._getRealMatrix && p1._getRealMatrix();
			p1 = p1 || matrix.identity;
			p1 = p1 && p1.transformPoint({x:0,y:0});
			
			var p2 = r.transformPoint({x:0,y:0});
			
			console.log( p1?print({x:p2.x-p1.x,y:p2.y-p1.y}):"(-,-)", print(p));
		} else {
			console.log( "--" );
		}
		
		shape = gu.safeGetParent(shape);
	}
};

return gu;

});

},
'ibm_ilog/diagram/interactors/Navigator':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/Navigator", [
"dojo/_base/declare",
"dojo/_base/connect",
"dojo/keys",
"dojox/collections/Stack",
"./Interactor"
],
function(
declare,
connect,
keys,
Stack,
Interactor
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var Navigator =
declare("ibm_ilog.diagram.interactors.Navigator",Interactor,{
	//summary:
	//		This Interactor manages the naviagation of the graph through the keyboard interaction.
	//		This Interactor have 5 connections: changeNavigatorStyle, clearFocusedElement, backward, forward, previous, next.
	
	_diagram: null,
	_strategy: null,
	_strategyNumber: null,
	
	initialize: function(diagram){
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.		
		this._diagram = diagram;
		this.setNavigationStyle(Navigator.TreeStyle);
		return this._initialize();
	},
	changeNavigatorStyle: function(e){
		// summary:
		//		switches the navigation style
		this.setNavigationStyle((this._strategyNumber+1)%2);
	},
	clearFocusedElement: function(){
		// summary:
		//		clears the focus.
		this._diagram.setFocusedElement(null);
	},
	next: function(e){
		// Summary:
		//		apply the next operation according to the navigation style.
		this._strategy.next(e);
	},
	previous: function(e){
		// Summary:
		//		apply the previous operation according to the navigation style.
		this._strategy.previous(e);
	},
	forward: function(e){
		// Summary:
		//		apply the forward operation according to the navigation style.
		this._strategy.forward(e);
	},
	backward: function(e){
		// Summary:
		//		apply the backward operation according to the navigation style.
		this._strategy.backward(e);
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return Navigator.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			backward: {
				hotKey: keys.LEFT_ARROW,	
				connectTo: "backward",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, forward: {
				hotKey: keys.RIGHT_ARROW,	
				connectTo: "forward",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, previous: {
				hotKey: keys.UP_ARROW,	
				connectTo: "previous",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, next: {
				hotKey: keys.DOWN_ARROW,	
				connectTo: "next",
				filter: this._buildInputFilter({ctrl:false,shift:false})
			}, changeNavigatorStyle: {
				hotKey: keys.F8,	
				connectTo: "changeNavigatorStyle"
			},clearFocusedElement: {
				hotKey: keys.ESCAPE,	
				connectTo: "clearFocusedElement"
			}
		};
	},
	setNavigationStyle: function(style){
		// Summary:
		//		sets the navigation style.
		// style: number
		//		one of the navigation styles
		this._strategyNumber = style;
		switch(style){
			case Navigator.TreeStyle: this._strategy = new TreeNavigator(this._diagram);break;
			case Navigator.GraphStyle: this._strategy = new GraphNavigator(this._diagram);break;
			default: this._strategy = new NullNavigator(this._diagram); this._strategyNumber = Navigator.NullStyle;break;
		}
	}
});

var NullNavigator =
declare("ibm_ilog.diagram.interactors.NullNavigator",null,{
	//summary:
	//		This class implements the disabled Navigation style. Also works as the superclass for others Navigation-Styles
	next: function(e){
		// Summary:
		//		do Nothing
	},
	previous: function(e){
		// Summary:
		//		do Nothing
	},
	forward: function(e){
		// Summary:
		//		do Nothing
	},
	backward: function(e){
		// Summary:
		//		do Nothing
	},
	_getFirstNode: function(){
		var children = this._diagram.getGraph().children;
		for(var i = 0;i<children.length;i++){
			if(children[i]._isIBMDiagramNode){
				return children[i];
			}
		}
		return null;
	},
	_getLastNode: function(){
		var children = this._diagram.getGraph().children;
		for(var i = (children.length-1);i>=0;i--){
			if(children[i]._isIBMDiagramNode){
				return children[i];
			}
		}
		return null;
	},
	_getChildIndex: function(array,element){
		for(index in array){
			if(element == array[index]){
				return parseInt(index);
			}
		}
		return -1;
	},
	_getVisibleParent: function(ge){
		// summary: 
		//		gets the lowest level visible element. If the Graph element is visible, return the same argument.
		// return: ibm_ilog.diagram.GraphElement
		var topMostCollapsedNode = null;
		var graph = this._diagram.getGraph();
		for(var p = ge.getParent(); p && p.getParent && p != graph; p = p._collapsedParent || p.getParent()){
			if(p._collapsedParent){
				for (var pp = p._collapsedParent; pp && pp.getParent; pp = pp.getParent()) {
					if (pp._isIBMDiagramSubgraph) {
						topMostCollapsedNode = pp;
						break;
					}
				}
			}
		}
		return topMostCollapsedNode || ge;
	},
	setFocusedElement: function(ge){
		// Summary:
		//		sets the ge to the diagram
		this._diagram.setFocusedElement(ge);
	}
});

/*=====
var NullNavigator = ibm_ilog.diagram.interactors.NullNavigator;
=====*/

var GraphNavigator =
declare("ibm_ilog.diagram.interactors.GraphNavigator",[NullNavigator],{
	//summary:
	//		This class implements the Graph Navigation style. The navigation is done througth 
	//		connections and record the path to enable backward navigation.
	_diagram: null,
	_stack: null,
	_clearOnNotification: null,
	_connections: null,
	
	constructor: function(diagram){
		this._diagram = diagram;
		this._stack = new Stack();
		this._clearOnNotification = true;
		this._connections = [];
		this._connections.push(connect.connect(this._diagram,'onFocusedElementChanged',this, function(){
			if(this._clearOnNotification){
				this.resetStack();
			}
		}));
	},
	next: function(e){
		// description:
		//		If Node focused: focus the first link of the focused node, and remember the 
		//		focused node as being the owner of the links being navigated.
		//		If Link focused: focus the previous link of previously focused node 
		//		(the node whose links are being navigated). If the currently focused link is 
		//		the first of the list, it does nothing.
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._nextConnection(ge);
		}
	},
	previous: function(e){
		// description:
		//		If Node focused: focus the last link of the focused node and remember 
		//		the focused node as being the owner of the links being navigated.
		//		If Link focused: focus the next link of previously focused node (the node 
		//		whose links are being navigated). If the currently focused link is the last 
		//		of the list, it does nothing.		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._previousConnection(ge);
		}
	},
	forward: function(e){
		// description:
		//		If Node focused: focus the last link that was followed backwards (if any) 
		//		to get to that node. It is then possible to keep on browsing other links of 
		//		the previous node that was focused before ending up on the current one (navigating backwards).
		//		If Link focused: focus the endNode of the currently focused link, if any.		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._followConnectionForward(ge);
		}
	},
	backward: function(e){
		// Summary:
		//		If Node focused: focus the last link that was followed forward (if any) to 
		//		get to that node. It is then possible to keep on browsing other links of the 
		//		previous node that was focused before ending up on the current one (navigating forward).
		//		If Link focused: focus the startNode of the currently focused link, if any.		
		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._followConnectionBackwards(ge);
		}
	},
	_validateFocusedElement: function(ge){
		if(ge){
			var vp = this._getVisibleParent(ge);
			if(vp == ge){
				return true;
			}else{
				this.resetStack();
				this.setFocusedElement(vp);
				return false;
			}
		}else{
			this.resetStack();
			this.setFocusedElement(this._getFirstNode());
			return false;
		}
	},
	getFocusedElement: function(){
		// summary: 
		//		gets the focused GraphElement from the diagram
		// return: ibm_ilog.diagram.GraphElement
		return this._diagram.getFocusedElement();
	},
	_followConnectionBackwards: function(ge){
		if(ge._isIBMDiagramNode){
			if(this._stack.peek() && this._stack.peek()._isIBMDiagramLink){
				this.setFocusedElement(this._stack.pop());
			}
		}else{
			var node = this._getLinkedNode(ge,true);
			this.setFocusedElement(node);
			if(this._stack.peek() == node){
				this._stack.pop();
			}else{
				this._stack.push(ge);
			}
		}
	},
	_followConnectionForward: function(ge){
		if(ge._isIBMDiagramNode){
			if(this._stack.peek() && this._stack.peek()._isIBMDiagramLink){
				this.setFocusedElement(this._stack.pop());
			}
		}else{
			var node = this._getLinkedNode(ge,false);
			this.setFocusedElement(node);
			if(this._stack.peek() == node){
				this._stack.pop();
			}else{
				this._stack.push(ge);
			}
		}		
	},
	_nextConnection: function(ge){
		if(ge._isIBMDiagramNode){
			var links = this._getElementLinks(ge);
			if(links.length>0){
				this.setFocusedElement(links[0]);
				this._stack.push(ge);
			}
		}else{
			var link = this._getFollowingLink(ge,1);
			if(link){
				this.setFocusedElement(link);
			}
		}
	},
	_previousConnection: function(ge){
		if(ge._isIBMDiagramNode){
			var links = this._getElementLinks(ge);
			if(links.length>0){
				this.setFocusedElement(links[links.length-1]);
				this._stack.push(ge);
			}
		}else{
			var link = this._getFollowingLink(ge,-1);
			if(link){
				this.setFocusedElement(link);
			}
		}
	},
	resetStack: function(){
		// summary:
		//		clears the navigation stack
		this._stack.clear();
	},
	_getFollowingLink: function(ge,increment){
		var links = this._getElementLinks(this._stack.peek());
		var index = this._getChildIndex(links,ge); 
		for(var i = index+increment;i>=0 && i<links.length;i=i+increment){
			return links[i];
		}
		return null;
	},
	setFocusedElement: function(ge){
		// summary:
		//		calls the superclass method, but disable the clearNotification
		this._clearOnNotification = false;
		this.inherited(arguments);
		this._clearOnNotification = true;
	},
	_getLinkedNode: function(link,start){
		var port;
		if(start){
			port = link.getStartPort();
		}else{
			port = link.getEndPort();
		}
		port = link._getVisiblePort(port);
		return port.getOwner();
	},
	_getElementLinks: function(ge){
		var result = ge.getLinks();
		if(ge._isIBMDiagramSubgraph){
			result = result.concat(ge.getIntergraphLinks());
		}
		return result;
	}
});

var TreeNavigator =
declare("ibm_ilog.diagram.interactors.TreeNavigator",[NullNavigator],{
	//summary:
	//		This class implements the Tree Navigation style.
	_diagram: null,
	constructor: function(diagram){
		this._diagram = diagram;
	},
	next: function(e){
		// description:
		//		focus the next sibling of the currently selected Node. If there is no such sibling, 
		//		it focus the next sibling of the parent of the currently selected node, provided that the 
		//		parent it not the root graph. In this case, it does nothing.
				
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this.nextNode(ge);
		}
	},
	previous: function(e){
		// description:
		//		focus the next sibling of the currently selected Node. If there is no such sibling, 
		//		it focus the next sibling of the parent of the currently selected node, provided that the 
		//		parent it not the root graph. In this case, it does nothing.	
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this.previousNode(ge);
		}
	},
	forward: function(e){
		// description:
		//		opens the currently selected Node if it is a closed Subgraph. 
		//		If it is an opened Subgraph, or if it is not a Subgraph, it does nothing.		
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._openNode(ge);
		}
	},
	backward: function(e){
		// description:
		//		closes the currently selected Node if it is an opened Subgraph. If is is a closed subgraph,
		//		or if it is not a Subgraph, it focus the parent of the currently selected node, provided 
		//		that the parent it not the root graph. In this case, it does nothing.	
		var ge = this.getFocusedElement();
		if(this._validateFocusedElement(ge)){
			this._closeNode(ge);
		}
	},
	_validateFocusedElement: function(ge){
		if(ge){
			var vp = this._getVisibleParent(ge);
			if(vp == ge){
				return true;
			}else{
				this.setFocusedElement(vp);
				return false;
			}
		}else{
			this.setFocusedElement(this._getFirstNode());
			return false;
		}
	},
	nextNode: function(ge){
		this._nextNode(ge,1);
	},
	previousNode: function(ge){
		this._nextNode(ge,-1);
	},
	_nextNode: function(c,increment){
		if(c){
			if(c._isIBMDiagramNode){
				var p = c.getParent();
				var next = this._getFollowingNode(p.children,c,increment);
				if(next){
					this.setFocusedElement(next);
				}else{
					this._nextNode(p._owningSubgraph,increment);
				}
			}
		}else{
			if(increment>0){
				this.setFocusedElement(this._getFirstNode());
			}else{
				this.setFocusedElement(this._getLastNode());
			}
		}
	},
	_openNode: function(ge){
		if(ge && ge._isIBMDiagramSubgraph){
			if(ge.isCollapsed() && ge.canCollapse()){
				ge.setCollapsed(false);	
			}else{
				var children = ge.getGraph().children;
				if(children.length>0){
					this.setFocusedElement(children[0]);
					return;
				}
			}
		}
	},
	_closeNode: function(ge){
		var ge = this.getFocusedElement();
		if(ge && ge._isIBMDiagramNode){
			if(ge._isIBMDiagramSubgraph && !ge.isCollapsed() && ge.canCollapse()){
				ge.setCollapsed(true);	
			}else{
				var parent = ge.getParent()._owningSubgraph;
				if(parent){
					this.setFocusedElement(parent);
				}
			}
		}
	},
	getFocusedElement: function(){
		// summary: 
		//		gets the focused Node from the diagram
		// return: ibm_ilog.diagram.Node
		var ge = this._diagram.getFocusedElement();
		if(ge && !ge._isIBMDiagramLink) {
			return ge;
		}
		return null;
	},
	_getFollowingNode: function(array,element,increment){
		var index = this._getChildIndex(array,element); 
		for(var i = index+increment;i>=0 && i<array.length;i=i+increment){
			if(array[i]._isIBMDiagramNode){
				return array[i];
			}
		}
		return null;
	}
});
// null navigation style
Navigator.NullStyle = -1;
// tree navigation style
Navigator.TreeStyle = 0;
// graph navigation style
Navigator.GraphStyle = 1;

// the interactor Id
Navigator.KeyInteractorId = "Navigator";

return Navigator;

});

},
'ibm_ilog/diagram/interactors/PanKeyInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/PanKeyInteractor", [
"dojo/_base/declare",
"dojo/keys",
"./Interactor",
"../util/Geometry"
],
function(
declare,
keys,
Interactor,
g
){

/*=====
var Interactor = ibm_ilog.diagram.interactors.Interactor;
=====*/

var PanKeyInteractor =
declare("ibm_ilog.diagram.interactors.PanKeyInteractor", [Interactor], {
	//summary:
	//		This Interactor manages the ViewPort Pan movement through the keyboard interaction.
	//		This Interactor have 4 connections: panLeft, panRigth, panUp, panDown.
	
	_diagram: null,
	_increment: null,
	
	initialize: function (diagram) {
	// summary:
    //		initialize the new instance.
    // diagram: ibm_ilog.diagram.widget.Diagram
    //		the associated diagram.				
		this._diagram = diagram;
		this.setIncrement(10);
		return this._initialize();
	},
	setIncrement: function(increment){
		// summary: 
		//		sets the pan increment
		// increment: integer
		//		the increment to be set
		this._increment = increment;
	},
	getIncrement: function(){
		// summary: 
		//		gets the pan increment
		return this._increment;
	},
	panUp: function(e){
		// summary:
	    //		This method pans the diagram up
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:0,y:-this._increment});
	},
	panDown: function(e){
		// summary:
	    //		This method pans the diagram down
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:0,y:this._increment});
	},
	panRight: function(e){
		// summary:
	    //		This method pans the diagram right
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:this._increment,y:0});
	},
	panLeft: function(e){
		// summary:
	    //		This method pans the diagram left
	    // e: Event object
	    //		the event to be treated.
		this._pan({x:-this._increment,y:0});
	},
	getKeyInteractorId: function(){
		// summary:
	    //		returns the Interactor Id
		return PanKeyInteractor.KeyInteractorId;
	},
	getDefaultConnections: function () {
		// summary:
		//		returns the default event mapping
		
		return {
			panLeft: {
				hotKey: keys.LEFT_ARROW,	
				connectTo: "panLeft",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panRight: {
				hotKey: keys.RIGHT_ARROW,	
				connectTo: "panRight",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panUp: {
				hotKey: keys.UP_ARROW,	
				connectTo: "panUp",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}, panDown: {
				hotKey: keys.DOWN_ARROW,	
				connectTo: "panDown",
				filter: this._buildInputFilter({ctrl:false,shift:true})
			}
		};
	},
	_pan: function(delta){
		var v = this._diagram.getViewport();
		var viewMovement = g.mulPoint(delta, -1 / v.getZoom()); // scale delta to zoom level to get delta in model coords
		var firstViewRect = v.getViewRect();
		v.setViewRect(g.moveRect(firstViewRect, viewMovement), {instant:true});
	}
});

PanKeyInteractor.KeyInteractorId = "Pan";

return PanKeyInteractor;

});

},
'ibm_ilog/diagram/interactors/KeyboardInteractorManager':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/KeyboardInteractorManager", [
"dojo/_base/declare",
"dojo/_base/lang",
"dojo/_base/connect",
"dojo/_base/event",
"./Interactor"
],
function(
declare,
lang,
connect,
event,
Interactor
){

var KeyboardInteractorManager =
declare("ibm_ilog.diagram.interactors.KeyboardInteractorManager", [], {
	// summary:
	//		This class manage the keyInteractor, by connection to the Diagram onkeydown event and pushing it to the appropriate registered Interactors.
	// The associated diagram
	_diagram: null,

	// the event connection
	_connection: null,

	// a dictionary of keyInteractors, by id
	_keyInteractors: null,

	//a map of hoKeys with the keyInteractors connections to be launch in each case
	// format:
	//			{ 65: { interactorId1:[connection1,connection2] , interactorId2:[connection3] },
	//			  70: { interactorId2:[connection4] } }
	_hotKeys: null,

	constructor: function (diagram) {
		// summary:
		//		Creates a new instance.
		// diagram: ibm_ilog.diagram.widget.Diagram
		//		the associated diagram.
		this._diagram = diagram;
		this._keyInteractors = {};
		this._hotKeys = {};
	},
	activate: function () {
		// summary:
		//		Activate the Manager, creating the connection.
		if (!this._connection) {
			this._connection = connect.connect(this._diagram.getKeyboardEventSource(), "onkeydown", this, "keyPressed");
		}
	},
	deactivate: function () {
		// summary:
		//		Deactivate the Manager, deleting the connection.
		connect.disconnect(this._connection);
		this._connection = null;
	},
	add: function (id, keyInteractor) {
		// summary:
		//		Adds a new keyInteractor.
		// id: String
		//		the keyIntearctor id.		
		// keyInteractor: ibm_ilog.diagram.interactors.Interactor
		//		the keyIntearctor to be added.		
		this._keyInteractors[id] = keyInteractor;
		keyInteractor.setKeyManager(this);
	},
	get: function (id) {
		// summary:
		//		Gets a keyInteractor by its id.
		// id: String
		//		the keyIntearctor id.
		// return: ibm_ilog.diagram.interactors.Interactor
		return this._keyInteractors[id];
	},
	remove: function (id) {
		// summary:
		//		Removes a keyInteractor by its id.
		// id: String
		//		the keyIntearctor id.
		delete(this._keyInteractors[id]);
	},
	keyPressed: function (e) {
		// summary:
		//		This method is called when a key is pressed in the diagram.
		// e: Event object
		//		the event to be treated.
		// return boolean
		var wasHandled = false;

		if (e.keyCode in this._hotKeys) {
			for (var accId in this._hotKeys[e.keyCode]) {
				wasHandled = this._keyInteractors[accId].keyPressed(this._hotKeys[e.keyCode][accId], e) || wasHandled;
			}
		}

		// If handled, stop the event and prevent default
		if (wasHandled) {
			event.stop(e);
			if (e.preventDefault) {
				e.preventDefault();
			}
		}
		return wasHandled;
	},

	setHotKeys: function (id, hotKeys) {
		// summary:
		//		This method is called to register the hotkeys to the manager. Ideally, this is called inside the Interactors at initialization time.
		// id: String
		//		the keyIntearctor id.
		// hotKeys: JS object
		//		a dictionary with the hotKeys to be register and the connections to be registered in each one. Format: {key1:[conn1,conn2], key2:[conn4]}
		for (var key in hotKeys) {
			if (!(key in this._hotKeys)) {
				this._hotKeys[key] = {};
			}
			this._hotKeys[key][id] = lang.clone(hotKeys[key]);
		}
	},
	clearHotKeys: function (id, hotKeys) {
		// summary:
		//		This method is called to clear the hotkeys in the manager. Ideally, this is called inside the Interactors when trying to unregister it for removal or reconfiguration.
		// id: String
		//		the keyIntearctor id.
		// hotKeys: JS object
		//		a dictionary with the hotKeys to be register and the connections to be registered in each one. Format: {key1:[conn1,conn2], key2:[conn4]}
		for (var key in hotKeys) {
			delete this._hotKeys[key][id];
		}
	},

	__eod: undefined
});

return KeyboardInteractorManager;

});

},
'ibm_ilog/diagram/util/Serializer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/Serializer", ["dojo/_base/declare", "dojo/_base/array", "dojox/collections/Dictionary", "./GraphUtil", "./ErrorReporter"],
function(declare, array, Dictionary, gu, R){

/*=====
ibm_ilog.diagram.util.__SubgraphSerializationInfo = function(){
	//	summary:
	//		Output of a subgraph deserialization with information for the 
	//		deserialization procedure. 
	//	created: Object
	//		The newly created object corresponding to the deserialization of the 
	//		subgraph. To be stored in the dictionary passed through all deserialization
	//		callbacks by it's old id. 
	//	container: Object
	//		The container that has been created while deserializing the subgraph.
	this.created = created;
	this.container = container;
};
=====*/

/*=====
ibm_ilog.diagram.util.__NodeSerializationInfo = function(){
	//	summary:
	//		Output of a node deserialization with information for the 
	//		deserialization procedure. 
	//	created: Object?
	//		The newly created object corresponding to the deserialization of the 
	//		node. To be stored in the dictionary passed through all deserialization
	//		callbacks by it's old id. 
	this.created = created;
};
=====*/

/*=====
ibm_ilog.diagram.util.__LinkSerializationInfo = function() {
	//	summary:
	//		Output of a link deserialization with information for the 
	//		deserialization procedure. 
	//	created: Object?
	//		The newly created object corresponding to the deserialization of the 
	//		link. To be stored in the dictionary passed through all deserialization
	//		callbacks by it's old id. 
	this.created = created;
};
=====*/

/*=====
ibm_ilog.diagram.util.__SerializerArgs = function(){
	//	getId: Function?
	//		This is used for identifying serialized graph elements through 
	//		the serialization/deserialization process.
	//	serializeNodeData: Function?
 	//		Callback used for serializing a node. The user must use this function
 	//		for creating the appropiate representation for a node which will later
 	//		be used by the corresponding deserialization function.
	//	serializeLinkData: Function?
 	//		Callback used for serializing a link. The user must use this function
 	//		for creating the appropiate representation for a link which will later
 	//		be used by the corresponding deserialization function.
	//	serializeSubgraphData: Function?
 	//		Callback used for serializing a subgraph. The user must use this function
 	//		for creating the appropiate representation for a subgraph which will later
 	//		be used by the corresponding deserialization function.
	//	deserializeNodeData: Function?
 	//		Deserialize a node from the serialized data into the given container.
	//	deserializeLinkData: Function?
 	//		Deserialize a link from the serialized data into the given container.
	//	deserializeSubgraphData: Function?
 	//		Deserialize a subgraph from the serialized data into the given container.
	
	// [av] TODO the following doc style is copied from dojo.xhr but is not currently working.
	this.getId = function(ge) {
		//	ge: ibm_ilog.diagram.GraphElement
	}
	
	this.serializeNodeData  = function(node){
		//	node: ibm_ilog.diagram.Node
		//	returns: ibm_ilog.diagram.util.__NodeSerializationInfo
	}
	 
	this.serializeLinkData  = function(link){
		//	link: ibm_ilog.diagram.Link
		//	returns: ibm_ilog.diagram.util.__LinkSerializationInfo
	};
	
	this.serializeSubgraphData  = function(subgraph){
		//	subgraph: ibm_ilog.diagram.Subgraph
		//	returns: ibm_ilog.diagram.util.__SubgraphSerializationInfo
	};
	 
	this.deserializeNodeData = function(serializedData,newByOldId,container){
		//	serializedData:	Object
		//		This is the serialized representation of the node to be deserialized.
	 	//	newByOldId: dojox.collections.Dictionary
	 	//		This is a dictionary of the newly deserialized objects accessible by
	 	//		their original id. This is used for cloning relations between the 
	 	//		original objects.
	 	//	container: Object
	 	//		The container into which to deserialize the node. This objects are 
	 	//		completely managed by the user callbacks.
	};
	 
	this.deserializeLinkData = function(serializedData,newByOldId,container){
		//	serializedData:	Object
		//		This is the serialized representation of the link to be deserialized.
	 	//	newByOldId: dojox.collections.Dictionary
	 	//		This is a dictionary of the newly deserialized objects accessible by
	 	//		their original id. This is used for cloning relations between the 
	 	//		original objects.
	 	//	container: Object?
	 	//		The container into which to deserialize the link. This objects are 
	 	//		completely managed by the user callbacks.
	};
	 
	this.deserializeSubgraphData = function(serializedData,newByOldId,container){
		//	serializedData:	Object
		//		This is the serialized representation of the subgraph to be deserialized.
	 	//	newByOldId: dojox.collections.Dictionary
	 	//		This is a dictionary of the newly deserialized objects accessible by
	 	//		their original id. This is used for cloning relations between the 
	 	//		original objects.
	 	//	container: Object
	 	//		The container into which to deserialize the subgraph. This objects are 
	 	//		completely managed by the user callbacks.
	};

};
=====*/

var Serializer = declare("ibm_ilog.diagram.util.Serializer",null,{
	//
	//	summary:
	//		This class is a template-based serialization procedure for GraphElements. Provided
	//		a specific set of user callbacks, the instances of this class can be used to 
	//		execute the serialization of a network of entities and its corresponding 
	//		deserialization.
	//
	//	see:
	//		ibm_ilog.diagram.util.__SerializerArgs

	constructor: function( /*ibm_ilog.diagram.util.__SerializerArgs*/ kwArgs ) {
		this._kwArgs = kwArgs;
		this._newByOldId = new Dictionary();
	},	
	
	testApiDoc: function( /*ibm_ilog.diagram.util.__SerializerArgs*/ kwArgs ) {
	
	},
	
	serialize: function(geSet) {
		//
		//	summary:
		//		Serializes a set of graph elements.
		//	geSet: Array
		//		Array of graph elements to serialize. Deserialization occurs once 
		//		for each regardlessly of the relationships between them.
		//		See also ibm_ilog.diagram.util.GraphUtil.maximals for achieving a variant
		//		which removes duplicated copying.
		//	returns: Array
		//		The serialized representation for each of the elements
		//		
		var c = this._kwArgs;
				
		var geById = new Dictionary();
		
		var dispatchers = {
			N:function(ge) { return c.serializeNodeData(ge); },
			L:function(ge) { return c.serializeLinkData(ge); },
			S:function(ge) { return c.serializeSubgraphData(ge); }
		};
		
		array.forEach(geSet,function(geRoot){
			gu.forEach(geRoot,function(ge){
				var id = c.getId(ge);
				if(!geById.contains(id)) {
					geById.add(id,gu.dispatchClass(ge,arguments,this,dispatchers));
				}
			},this);
		},this);

		var node = function(ps,ge) {
			return {
				node:1,
				id: c.getId(ge),
				data: geById.item(c.getId(ge))
			};
		};

		var link = function(ps,ge) {
			return {
				link:1,
				id: c.getId(ge),
				data: geById.item(c.getId(ge))
			};
		};

		var graph = function(ps,ns,ge,fc) {
			// the serialization of a graph is the 
			// already serialized child collection "fc"
			return fc;
		};
		
		var child = function(ps,fc,fe) {
			// add serialized child "fe" to the serialized 
			// child collection "fc" and return 
			var nc = [];
			for(var x in fc) {
				if(x) {
					nc.push(fc[x]);
				}
			}
			nc.push(fe);
			return nc;
		};
		
		var sub = function(ps,ns,ge,fg) {
			// data + serialized children collection 
			return {
				subgraph:1,
				id: c.getId(ge),
				data:geById.item(c.getId(ge)),
				children:fg
			};
		};
	
		var r = [];
		array.forEach(geSet,function(maximalGE){

			r.push(gu.fold(
				maximalGE,
				undefined,
				gu.folders({
					node: node,
					sub:  sub,
					link: link,
					graph: graph,
					cBase: [],
					child: child
				}),
				this
			));
		},this);
		
		return r;
		
	},
	
	deserialize: function(s,container) {
		//
		//	summary:
		//		Deserialize a set of elements into a container
		//	s: Array
		//		The serialized representations of the elements to deserialize
		//	container: Object
		//		The container into which to create the elements.
		//

		var result = array.map(s,function(s){
			return this._deserializeElement(s,container);
		},this);
		this._newByOldId.clear();
		return result;
	},

	_deserializeElement: function(se,container) {
		//
		//	summary:
		//		Deserialize a graph element into a container
		//	se: Array
		//		The serialized representation of the element to deserialize
		//	container: Object
		//		The container into which to create the elements.
		//
		var c = this._kwArgs;
		var dict = this._newByOldId;
		var newge;
		
	    if(se.node) {
		    newge = c.deserializeNodeData(se.data,dict,container);
		    if(newge) {
		    	dict.add(se.id,newge);
		    }
	    }
	    
	    if(se.subgraph) {
	    	newge = c.deserializeSubgraphData(se.data,dict,container);
	    	if(newge) {
			    dict.add(se.id,newge);
				array.forEach(se.children,function(e){
					this._deserializeElement(e,newge);
				},this);
	    	}
		}

		if(se.link) {
		    newge = c.deserializeLinkData(se.data,dict,container);
		    if(newge) {
		    	dict.add(se.id,newge);
		    }
		}
		return newge;
	}	
});

return Serializer;
});



},
'ibm_ilog/diagram/util/PrintUtil':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/PrintUtil", ["dojo/_base/kernel", "dojox", "dojo/_base/declare", "dojo/_base/lang", "dojox/gfx", "dojox/gfx/matrix", "dojox/gfx/utils", "./MultilineText", "./ErrorReporter"],
function(dojo, dojox, declare, lang, gfx, matrix, gfxu, MultilineText, R){

	/*=====
	ibm_ilog.diagram.util.__PrintUtilArgs = function(){
		//	getTargetSurfaceDiv: Function?
		//		This is used to get the HTML div tag where the target surfaces are going to be drawn
		//	postProcessing: Function?
	 	//		This function is called after every target surface is created, to customize any part necessary
		
		// [av] TODO the following doc style is copied from dojo.xhr but is not currently working.
		getTargetSurfaceDiv = function(row,column) {
			// [Mandatory]
			// row: the row number
			// column: the column number
			// return: the div tag where the surface will be drawn
		},
		postProcessing= function(row,column) {
			// [optional]
			// row: the row number
			// column: the column number
			// surface: the the drawn surface
			// group: the complete gfx.group drawn (typically the complete graph)
		},
	};
=====*/
	
var PrintUtil = declare("ibm_ilog.diagram.util.PrintUtil",null,{
	constructor: function( /*ibm_ilog.diagram.util.__PrintUtilArgs*/ kwArgs ) {
		this._kwArgs = kwArgs;
	},
	printGraph: function(graph,bounding,width,height,pagesWidth,pagesHeight,paperColor){
		// Summary:
		// 		Separates the given graph boundingBox into pagesWidth horizontal surfaces and pagesHeight vertical surfaces.
		//
		// graph:
		// 		the graph to be separated into multiple surfaces
		// bounding:
		// 		[Rect] indicates the area to be show. If missing, the complete graph is shown
		// width:
		// 		the width in pixels of every single surface
		// height:
		// 		the height in pixels of every single surface
		// pagesWidth:
		// 		the amount of horizontal surfaces into be split
		// pagesHeight:
		// 		the amount of vertical surfaces into be split
		// paperColor:
		//		[Color] this paperColor is used to hide not required part of the graph, when the target area has different aspect 
		//		ratio than the source bounding. If missing, white is used as default color.
		
		if (!gfxu.toSvg) {
			R.error("UnsupportedDojoFeature", "1.4.0");
		}
		
		// patch so that gfx (de)serialization does not crash on MultilineText deserialization :
		// The gfx serialize() method does not save the 'shape' property of a Group instance, but
		// will save the others 'default' gfx properties (fill, stroke, font...), so MultilineText instances
		// are serialized as Group instance :
		// { children : [....], transform:..., font: ... }.  (so, no shape:{type:'ml'}).
		// Therefore, when the json is deserialized, a Group instance is created, and setFont() is invoked
		// (which fails because Group.setFont does not exist).
		// Patch (Hack!!!!): we delete the getFont/setFont accessors so that the font is not serialized 
		var getFont = MultilineText.prototype.getFont;
		delete MultilineText.prototype.getFont;
		
		bounding = bounding || graph.getBoundingBox();
		paperColor = paperColor || 'white';
		// Calculate the zoom and necessary to fit and center
		var tMatrix = this._calculateTransformToFit(bounding, width, height, pagesHeight, pagesWidth);
		//Fix to avoid null bounding [defect 725]
		bounding = bounding || {x:0,y:0,width:0,height:0};
		var userFunc = this._kwArgs;
		for (var r = 0; r < pagesHeight; r++) {
			for (var c = 0; c < pagesWidth; c++) {
				var div = userFunc.getTargetSurfaceDiv(r,c);
				
				//Option 1: with this lines the code fails on div.appendChild(tempDiv);
				/*
				var tempDiv = dojo.create("div");
				div.appendChild(tempDiv);
				var surf = dojox.gfx.createSurface(tempDiv,width,height);
				*/
				
				
				//Option 2: This Line fails on vml.js line 1036: r.appendChild(b); (V:GROUP append V:RECT)
				var surf = gfx.createSurface(div,width,height);
				
				//Create temp surface to render object to and render.
				//Option 3: This Line fails on vml.js line 1036: r.appendChild(b); (V:GROUP append V:RECT)
				/*
				var divWindow = this._getWindow(div.ownerDocument);
				this._setDojoScope(divWindow);
				var surf = divWindow[dojox._scopeName].gfx.createSurface(div, width, height);
				*/
				
				surf.whenLoaded(lang.hitch(this, function(){
					var obj = gfxu.serialize(graph);
					var group = surf.createGroup();
					group.applyLeftTransform(tMatrix);
					group.applyLeftTransform(matrix.translate(width * c * -1, height * r * -1));
					gfxu.deserialize(group, obj);
					if(userFunc.postProcessing){
						userFunc.postProcessing(r,c,surf,group);
					}
					this._cropToFit(group,bounding,r,c,pagesHeight,pagesWidth,width,height,tMatrix.xx,paperColor);
				}));
			}
		}
		
		// restore font accessors
		MultilineText.prototype.getFont = getFont;
	},
	/* this code seems unused...
	_getWindow:function(anyDocument){
		var divWindow = anyDocument.defaultView ||  anyDocument.parentWindow;
		return divWindow;
	},
	_setDojoScope: function(window){
		
		window[dojox._scopeName] = this._getWindow(document)[dojox._scopeName];
		window[dojo._scopeName] = this._getWindow(document)[dojo._scopeName];
	},
	*/
	printViewport: function(diagram,width,height,pagesWidth,pagesHeight,paperColor){
		// Summary:
		// 		Separates the given diagram viewport into pagesWidth horizontal surfaces and pagesHeight vertical surfaces.
		// diagram:
		// 		the diagram which viewport will be separated into multiple surfaces
		// width:
		// 		the width in pixels of every single surface
		// height:
		// 		the height in pixels of every single surface
		// pagesWidth:
		// 		the amount of horizontal surfaces into be split
		// pagesHeight:
		// 		the amount of vertical surfaces into be split
		// paperColor:
		//		[Color] this paperColor is used to hide not required part of the graph, when the target area has different aspect 
		//		ratio than the source bounding. If missing, white is used as default color.
		var graph = diagram.getGraph();
		var bounding = diagram.getViewport().getViewRect();
		this.printGraph(graph,bounding,width,height,pagesWidth,pagesHeight,paperColor);
	},
	
	_calculateTransformToFit: function(printBB,dimx, dimy, h, w){
		// Summary:
		// 		Calculates and returns the Transformation necessary to fit the pintBB area in the printable area,
		//		compound by the merge of (dimx * w) X (dimy * h)
		
		//Fix to avoid null transformation [defect 725]
		if(!printBB){
			return matrix.identity;
		}
		
		if (printBB.width && printBB.height) {
			var sz = {
				width: dimx * w,
				height: dimy * h
			};
			var xr = sz.width / printBB.width;
			var yr = sz.height / printBB.height;
			var cent = {
				x: 0,
				y: 0
			};
			
			var zoom;
			if (xr < yr) {
				zoom = xr;
				cent.y = (printBB.width * (sz.height / sz.width) - printBB.height) / 2;
			}
			else {
				zoom = yr;
				cent.x = (printBB.height * (sz.width / sz.height) - printBB.width) / 2;
			}
			
			var zt = matrix.scale(zoom);
			var tt = matrix.translate({
				x: - printBB.x + cent.x,
				y: - printBB.y + cent.y
			});
			
			var t = matrix.multiply(zt, tt);
			
			return t;
		}
	},
	_cropToFit: function(group,bounding,r,c,pagesHeight,pagesWidth,width,height,zoom,paperColor){
		// Summary:
		// 		Creates rectangles over the surrounding surfaces, to limit the printed area to the given bounding 
		//		(hiding the parts that are out of the bounding when the target aspect ratio do not match the original aspect ratio)
		var rectx = Math.max((bounding.width),(width * pagesWidth/zoom));
		var recty = Math.max((bounding.height),(height * pagesHeight/zoom));
		
		if(r==0){
			group.createRect({ x: bounding.x, y: bounding.y-recty, width: rectx, height: recty }).setFill(paperColor);
		}
		if(c==0){
			group.createRect({ x: bounding.x-rectx, y: bounding.y, width: rectx, height: recty }).setFill(paperColor);
		}
		if(r==(pagesHeight-1)){
			group.createRect({ x: bounding.x, y: bounding.y+bounding.height, width: rectx, height: recty }).setFill(paperColor);
		}
		if(c==(pagesWidth-1)){
			group.createRect({ x: bounding.x+bounding.width, y: bounding.y, width: rectx, height: recty }).setFill(paperColor);
		}
	}
});

return PrintUtil;

});
},
'ibm_ilog/diagram/graphlayout/ServerSideLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/graphlayout/ServerSideLayout", [
	"dojo/_base/declare",
	"dojo/_base/config",
	"dojo/_base/lang",
	"dojo/_base/json",
	"dojo/_base/array",
	"dojo/_base/window",
	"dojo/_base/Deferred",
	"dojo/_base/xhr",
	"require",
	"dojox/lang/functional/array",
	"dojox/gfx",
	"dojox/collections/Dictionary",
	"ibm_ilog/graphlayout/IIterator",
	"ibm_ilog/graphlayout/internalutil/GraphModelUtil",
	"./GraphAdapter",
	"../util/GraphUtil",
	"../util/ErrorReporter",
	"../util/Geometry",
	"../util/Batch",
	"../gfxlayout/Engine",
	"dojo/has!graph-layout-without-diagram?:ibm_ilog/diagram/graphlayout/_base"
	],
	function(
		declare,
		config,
		lang,
		json,
		array,
		win,
		Deferred,
		xhr,
		require,
		df,
		gfx,
		Dictionary,
		IIterator,
		GraphModelUtil,
		GraphAdapter,
		gu,
		R,
		g,
		Batch,
		gfxlayout
	) {

// AVA: TODO OPTIMIZEME "ip"
    
    var isServerSideExtended = function(instance) {
        return instance.__serverSideEnabled__ === true;
    };
    
	var _requireCache = {};
	var _require = function(name) {
		// summary: Cache-enhanced version of require() used to dynamically load classes.
		var c = _requireCache[name];
		if(c === undefined){
			try {
				c = require(name);
			} catch(err){
				c = null;
			}
			_requireCache[name] = c;
		}
		return c;
	};
	
    var commonLayoutExtension = {
        __serverSideEnabled__: true,
        
        getServerLayoutGlobalParameters: function() {
            return {};
        },
        
        getLayoutGlobalDescriptor: function() {
            var descriptor = {
                algorithm: this.id
            };
            var globalParameters = this.getServerLayoutGlobalParameters();
            if (globalParameters !== null) {
                lang.mixin(descriptor, globalParameters);
            }
            return json.toJson(descriptor);
        },
        
        getLayoutNodeDescriptor: function(node) {
            var nodeParameters = this.getServerLayoutNodeParameters(node);
            if (nodeParameters === null) {
                return null;
            }
            var descriptor = {};
            lang.mixin(descriptor, nodeParameters);
            return json.toJson(descriptor);
        },
        
        getLayoutLinkDescriptor: function(link) {
            var linkParameters = this.getServerLayoutLinkParameters(link);
            if (linkParameters === null) {
                return null;
            }
            var descriptor = {};
            lang.mixin(descriptor, linkParameters);
            return json.toJson(descriptor);
        },
        
        _capitalize: function(w) {
            return w.substring(0, 1).toUpperCase() + w.substring(1);
        },
        
        _isRectType: function(value, defaultValue) {
            return (value !== null && value !== undefined && value.width !== undefined) ||
            (defaultValue !== null && defaultValue !== undefined && defaultValue.width !== undefined);
        },
        
        _isPointType: function(value, defaultValue) {
            // Must be called only for parameters that have been tested already against _isRectType.
            return (value !== null && value !== undefined && value.x !== undefined) ||
            (defaultValue !== null && defaultValue !== undefined && defaultValue.x !== undefined);
        },
        
        _forceSavingParameter: function(paramName, layoutName) {
            // A few global parameters have different default values in JViews than in Dojo.
            // This must be always be put in json.
            // The default value comes from J2JS metadata embedded in the java
            // layouts, for instance:
            // @j2js jsinspectable (name="linkClipping",defaultValue="false")
            // The alternate solution for handling the cases of different defaults would be
            // that the metadata declares the default for the JViews layouts, not for the
            // dojo ones. But doing so would introduce a mess in the metadata and would 
            // make it harder to be reused for other purposes, such as providing a customization
            // GUI for the Dojo layouts.
            return ("forcedirected" == layoutName &&
            		(paramName == "respectNodeSizes" ||
            		 paramName == "maxRepeatForConvergence" ||
            		 paramName == "layoutMode")) ||
            	  ("circular" == layoutName &&
             		 paramName == "clusteringMode") ||
            	  ("hierarchical" == layoutName &&
             		 paramName == "recursiveLayoutMode") ||
								// This param holds for several algos (BasicLinkStyleLayout and its
								// subclasses ForceDirected, Circular and Random). No real need to
								// check the layoutname. 
								(paramName == "connectLinksToNodeCenters");
        },
        
        _isDefaultValue: function(value, defaultValue, isRectParam, isPointParam) {
            if (value === null || value === undefined) {
                // if both null or undefined, then the param has the default value
                return defaultValue === null || defaultValue === undefined;
            } else if (defaultValue === null || defaultValue === undefined) {
                return value === null || value === undefined;
            } else {
            	// okay, both value and defaultValue are not null and are not undefined
               	if (isRectParam) {
                	return value.x == defaultValue.x && value.y == defaultValue.y &&
                     	value.width == defaultValue.width &&
                        value.height == defaultValue.height;
                } else if (isPointParam) {
                    return value.x == defaultValue.x && value.y == defaultValue.y;
                } else {
                 	// okay, it is a parameter of type int/float/long/double/boolean
                    return value == defaultValue;
                }
      	    }
        },
        
        _addServerParam: function(params, name, defaultValue, layoutName, getterArg) {
            // getterArg : the argument to be passed to the getter of the layout
            // parameter. This argument must be passed for local (per-node/link)
            // layout parameters (the node/link itself). For global parameters
            // it is not passed.
            // layoutName : the identifier of the layout algorithm. 
            //
            // Returns true if the parameter has been added, false otherwise.
            
						var isCircularClusterIds = "circular" == layoutName && "clusterIds" == name;
						
						var getterPrefix;
						var value = null;
						if (!isCircularClusterIds) {
							getterPrefix = (typeof(defaultValue) == "boolean") ? "is" : "get";
							value = this[getterPrefix + this._capitalize(name)](getterArg);
						} else {
							// Do not use CircularLayout.getClusterIds() because this triggers the 
							// clusterization which would fail with the reduced layout code for SSL.
							// Note that we are only interested in the specified clusters, not in
							// those computed by the automatic clustering.
							value = this._getClusterIds(this);
						}
			
            var isRectParam = this._isRectType(value, defaultValue);
            var isPointParam = isRectParam ? false : this._isPointType(value, defaultValue);
            
            if (this._forceSavingParameter(name, layoutName) ||
            		!this._isDefaultValue(value, defaultValue, isRectParam, isPointParam)) {
                var isHierarchicalConstraints = false; // case of HierarchicalLayout's constraints.
                var isCircularClusterIds = false; // case of CircularLayout's cluster ids.
                
                if (value !== null) {
                    if (isRectParam) {
                        value = [value.x, value.y, value.width, value.height];
                    } else if (isPointParam) {
                        value = [value.x, value.y];
                    } else if ("hierarchical" == layoutName) {
											if ("constraints" == name &&
					  							value instanceof IIterator) {
					  						this._addServerHierarchicalConstraintsParam(params, value);
					  						isHierarchicalConstraints = true;
					  					} else if ("recursiveLayoutMode" == name && 
																 value && // shortcut
											           this.getGraphModel().getSubgraphsCount() == 0) {
												// The client-side Dojo HierarchicalLayout automatically disables
												// recursive mode for flat graphs. We must configure the server-side
												// layout consistently. 
												// Also, the recursive mode of HierarchicalLayout does not support swimlanes
												// (documented limitation in Dojo and JViews). Thanks to the internal
												// flattening of the graph model in presence of swimlanes, the graph model
												// is flat for nested graphs containing swimlanes, and this is how the
												// client-side Dojo HierarchicalLayout deactivates the recursive mode for
												// swimlanes. By counting here the subgraphs we ensure the same holds for
												// server-side layout.
												value = false;
											}
                    } else if ("circular" == layoutName && "clusterIds" == name) {
                        this._addServerCircularClusterIdsParam(params, value);
                        isCircularClusterIds = true;
                    }
                } // else keep the value at null
                
                if (!(isHierarchicalConstraints || isCircularClusterIds)) {
                    params[name] = value;
                } // else constraints and cluster ids are handled differently
                
                return true;
            }
            return false;
        },
        
				// =====================================================================
				// Hierarchical layout
				
        _addServerHierarchicalConstraintsParam: function(params, value) {
            // value : the value of hierarchicalLayout.getConstraints() parameter. 
            if (!value) {
                return false;
            }
            var e = value;
            var constraint;
            var allConstraintsValue = [];
            var constraintValue;
            
						// Analysis of the dojo.require situation:
						// This method is only called if the user has configured an instance of
						// of HierarchicalLayout on the client graph. Hence, the class HierarchicalLayout
						// is already loaded. Which implies that ibm_ilog.graphlayout.hierarchical is already 
						// loaded. Hence no need for a dojo.require for it.
						// Also, a client-side constraint can be of a certain type only if an instance of
						// this constraint type has been specified by the user. Hence, again, we do not
						// need a dojo.require for the constraint class. We only need to check 
						// that the constraint class is not undefined before using instanceof.
			
			// AMD conversion note: the explanations above still apply, we call require() to check whether
			// each class is already loaded or not.
			
			var HierarchicalLevelRangeConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalLevelRangeConstraint");
			var HierarchicalGroupSpreadConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalGroupSpreadConstraint");
			var HierarchicalRelativeLevelConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalRelativeLevelConstraint");
			var HierarchicalExtremityConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalExtremityConstraint");
			var HierarchicalRelativePositionConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalRelativePositionConstraint");
			var HierarchicalSameLevelConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalSameLevelConstraint");
			var HierarchicalSideBySideConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalSideBySideConstraint");
			var HierarchicalSwimLaneConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalSwimLaneConstraint");
			
            while (e.hasNext()) {
                constraint = e.next();
                constraintValue = null;
                if (HierarchicalLevelRangeConstraint &&
				    				constraint instanceof HierarchicalLevelRangeConstraint) {
                    var nodesValue1 = this._getConstraintNodes(constraint.getSubject());
                    constraintValue = {
                        name: "LevelRangeConstraint",
                        minLevel: constraint.getMinLevel(),
                        maxLevel: constraint.getMaxLevel(),
                        nodes: nodesValue1
                    };
                } else if (HierarchicalGroupSpreadConstraint &&
						   							constraint instanceof HierarchicalGroupSpreadConstraint) {
                	var nodesValue2 = this._getConstraintNodes(constraint.getGroup());
                    constraintValue = {
                    	name: "GroupSpreadConstraint",
                        spreadSize: constraint.getSpreadSize(),
                        nodes: nodesValue2
                    };
               	} else if (HierarchicalRelativeLevelConstraint &&
				         						constraint instanceof HierarchicalRelativeLevelConstraint) {
                 	var lowerNodesValue1 = this._getConstraintNodes(constraint.getLowerSubject());
                    var higherNodesValue1 = this._getConstraintNodes(constraint.getHigherSubject());
                    	constraintValue = {
                        name: "RelativeLevelConstraint",
                        lowerNodes: lowerNodesValue1,
                        higherNodes: higherNodesValue1
                    };
                } else if (HierarchicalExtremityConstraint &&
				 		   							constraint instanceof HierarchicalExtremityConstraint) {
                    var nodeValue = this._getConstraintNodes(constraint.getNode());
                    constraintValue = {
                    	name: "ExtremityConstraint",
                        side: constraint.getSide(),
                        node: nodeValue
                    };
                } else if (HierarchicalRelativePositionConstraint &&
						   							constraint instanceof HierarchicalRelativePositionConstraint) {
                    var lowerNodesValue2 = this._getConstraintNodes(constraint.getLowerSubject());
                    var higherNodesValue2 = this._getConstraintNodes(constraint.getHigherSubject());
                    constraintValue = {
                    	name: "RelativePositionConstraint",
                        lowerNodes: lowerNodesValue2,
                        higherNodes: higherNodesValue2
                    };
				} else if (HierarchicalSameLevelConstraint &&
						   constraint instanceof HierarchicalSameLevelConstraint) {
					var firstNodeValue = this._getConstraintNodes(constraint.getFirstNode());
					var secondNodeValue = this._getConstraintNodes(constraint.getSecondNode());
					constraintValue = {
						name: "SameLevelConstraint",
						firstNode: firstNodeValue,
						secondNode: secondNodeValue
					};
				} else if (HierarchicalSideBySideConstraint &&
						   constraint instanceof HierarchicalSideBySideConstraint) {
					var nodesValue3 = this._getConstraintNodes(constraint.getGroup());
					constraintValue = {
						name: "SideBySideConstraint",
						nodes: nodesValue3
					};
				} else if (HierarchicalSwimLaneConstraint &&
						   constraint instanceof HierarchicalSwimLaneConstraint) {
					var nodesValue4 = this._getConstraintNodes(constraint.getGroup());
					constraintValue = {
						name: "SwimLaneConstraint",
						relativeSize: constraint.getRelativeSize(),
						specPositionIndex: constraint.getSpecPositionIndex(),
						eastMinMargin: constraint.getEastMinMargin(),
						westMinMargin: constraint.getWestMinMargin(),
						nodes: nodesValue4
					};
				} 
           if (constraintValue !== null) {
					// Properties that exist on all constraints:
					constraintValue.priority = constraint.getPriority();
					
					// Add it
                    allConstraintsValue.push(constraintValue);
                }
            }
            
            var atLeastOneConstraint = allConstraintsValue.length > 0;
            if (atLeastOneConstraint) {
                params.constraints = allConstraintsValue;
            }
            
            return atLeastOneConstraint;
        },
        
        _getConstraintNodes: function(constraintSubject) {
            // constraintSubject: a node or a HierarchicalNodeGroup
            // Returns an array of objects representing the nodes that are subject
            // to a hierarchical constraint.
            if (constraintSubject === null) {
                return null; // empty constraint
            }
            if (constraintSubject._isIBMDiagramNode) {
                return [{id: constraintSubject.getId()}];
            } // else it is a HierachicalNodeGroup
            var nodesValue = [];
            var elements = constraintSubject.elements();
            var node;
            while (elements.hasNext()) {
                node = elements.next();
                nodesValue.push({
                    id: node.getId()
                });
            }
            return nodesValue;
        },
				
				// ==================================================================================
				// Circular layout
				
				_getClusterIds: function(circularLayout) {
				   	var nodes = circularLayout.getGraphModel().getNodes();
						var n = nodes.length;
						var nodeClusterIds;
						var clusterIds = null; 
						var node;
						var clusterId;
						var ClusterIdCollection = _require("ibm_ilog/graphlayout/circular/clayout/ClusterIdCollection");
        		for (var i = 0; i < n; i++) {
							node = nodes[i];
							nodeClusterIds = circularLayout.getNodeClusterIds(node);
							var nClusterIds = nodeClusterIds != null ? nodeClusterIds.length : 0;
							for (var j = 0; j < nClusterIds; j++) {
			  					clusterId = clusterIds[j];
								if (clusterIds == null)
									clusterIds = new ClusterIdCollection(clusterId);
								else
								  clusterIds.add(clusterId);
			  			}								
						}
						return clusterIds != null ? clusterIds.getClusterIds() : [];
				},
        
        _addServerCircularClusterIdsParam: function(params, value) {
            // value : the value of circularLayout.getClusterIds() parameter. 
            if (!value) {
                return false;
            }
            var clusterId;
            var allClusterIdsValue = [];
            var clusterIdValue;
            var clusterIndex = 0;
            
						// Analysis of the dojo.require situation:
						// This method is only called if the user has configured an instance of
						// of CircularLayout on the client graph. Hence, the class CircularLayout
						// is already loaded. Which implies that ibm_ilog.graphlayout.circular is already 
						// loaded. Hence no need for a dojo.require for it.
						// Also, a client-side cluster identifier can be of a certain type only if an 
						// instance of it has been specified by the user. Hence, again, we do not
						// need a dojo.require for the cluster identifier class. We only need to check 
						// that the cluster identifier class is not undefined before using instanceof.
			
			// AMD conversion node: we now call require("classname") to test if the class is loaded.
			
			var CircularClusterName = _require("ibm_ilog/graphlayout/circular/CircularClusterName");
			var CircularClusterNumber = _require("ibm_ilog/graphlayout/circular/CircularClusterNumber");
			
            for (var i = 0; i < value.length; i++) {
                clusterId = value[i];
                clusterIdValue = null;
                var nodesDescription = [];
                var isClusterName = false;
                var isClusterNumber = false;
                if (CircularClusterName && 
										clusterId instanceof CircularClusterName) {
                    isClusterName = true;
                } else if (CircularClusterNumber &&
						   							clusterId instanceof CircularClusterNumber) {
									isClusterNumber = true;
                }
                if (isClusterName || isClusterNumber) {
                    var nodesValue = this.getClusterNodes(i);
                    for (var j = 0; j < nodesValue.length; j++) {
                        var node = nodesValue[j];
                        var index = this.getIndex(node, clusterId);
                        var isStarCenter = this.isStarCenter(node);
                        var obj = {
                            id: node.getId()
                        };
                        if (index >= 0) {
                            obj.index = index;
                        }
                        if (isStarCenter) {
                            obj.starCenter = true;
                        }
                        nodesDescription.push(obj);
                    }
                    clusterIdValue = {};
                    if (isClusterName) {
                        clusterIdValue.name = clusterId.getName();
                    } else {
                        clusterIdValue.number = clusterId.getNumber();
                    }
                    clusterIdValue.nodes = nodesDescription;
                }
                if (clusterIdValue.nodes) {
                    allClusterIdsValue.push(clusterIdValue);
                }
            }
            
            var atLeastOneClusterId = allClusterIdsValue.length > 0;
            if (atLeastOneClusterId) {
                params.clusterIds = allClusterIdsValue;
            }
            
            return atLeastOneClusterId;
        },
        
        _addServerGlobalParam: function(params, name, defaultValue, layoutName) {
            // Returns true if the parameter has been added, false otherwise.
            return this._addServerParam(params, name, defaultValue, layoutName);
        },
        
        _addServerNodeOrLinkParam: function(nodeOrLink, params, name, defaultValue, layoutName) {
            // Returns true if the parameter has been added, false otherwise.
            return this._addServerParam(params, name, defaultValue, layoutName, nodeOrLink);
        },
        
        _getServerGlobalParameters: function(globalProperties, layoutName) {
            var params = {};
            if (globalProperties) {
                var n = globalProperties.length;
                var param;
                for (var i = 0; i < n; i++) {
                    param = globalProperties[i];
                    this._addServerGlobalParam(params, param.name, param.defaultValue, layoutName);
                }
            }
            return params;
        },
        
        _getServerNodeOrLinkParameters: function(nodeOrLink, nodeOrLinkProperties, layoutName) {
            if (nodeOrLinkProperties) {
                var n = nodeOrLinkProperties.length;
                if (n === 0) {
                    return null;
                }
                var params = {};
                var param;
                // For local (per-node/link) parameters, we skip completely the dump to
                // json of these parameters if there is none with non-default value.
                // (For global parameters it goes differently, because we anyway need to
                // store the type of the algo.) 
                var atLeastOne = false;
                for (var i = 0; i < n; i++) {
                    param = nodeOrLinkProperties[i];
                    if (this._addServerNodeOrLinkParam(nodeOrLink, params, param.name, param.defaultValue, layoutName)) {
                        atLeastOne = true;
                    }
                }
                return atLeastOne ? params : null;
            } else {
                return null;
            }
        },
        
        _getServerLayoutGlobalParametersImpl: function(globalProperties, layoutName) {
            if (globalProperties === null) {
                return null;
            }
            return this._getServerGlobalParameters(globalProperties, layoutName);
        },
        
        _getServerLayoutNodeOrLinkParametersImpl: function(nodeOrLink, nodeOrLinkProperties, layoutName) {
            if (nodeOrLinkProperties === null || nodeOrLinkProperties.length === 0) {
                return null;
            }
            return this._getServerNodeOrLinkParameters(nodeOrLink, nodeOrLinkProperties, layoutName);
        }
    };
    
	var markServerSideExtended = function(instance) {
        lang.extend(instance.constructor, commonLayoutExtension);
    };
	
    var extendLayout = function(layout) {
        if (layout && !isServerSideExtended(layout)) {
            markServerSideExtended(layout);
			/*
            if (layoutExtensions[layout.declaredClass] === undefined) {
                R.error("UnsupportedAlgorithm", layout);
            } else {
                dojo.extend(layout.constructor, layoutExtensions[layout.declaredClass]);
            }
            */
			var layoutClass = layout.constructor;
			var id = layout.declaredClass.split('.')[2];
			lang.extend(layoutClass, {
				id: id,
				
				getServerLayoutGlobalParameters: function(){
					var globalProperties = layoutClass._getGlobalProperties();
					return this._getServerLayoutGlobalParametersImpl(globalProperties, id);
				},
				
				getServerLayoutNodeParameters: function(node){
					var nodeProperties = layoutClass._getNodeProperties();
					return this._getServerLayoutNodeOrLinkParametersImpl(node, nodeProperties, id);
				},
				
				getServerLayoutLinkParameters: function(link){
					var linkProperties = layoutClass._getLinkProperties();
					return this._getServerLayoutNodeOrLinkParametersImpl(link, linkProperties, id);
				}
			});
        }
    };
    
	// [Eric] moved here from GraphUtil.js, to avoid a dependency from GraphUtil to graphlayout.
	var foldGraphModel = function (e,ps,folders,scope,stopRecursionAtCollapsedSub) {
		//
		//	summary:
		//		Graph model data structure fold function. --Functional Programming--
		//
		//	description:
		//		Recurses a graph layout model offering callbacks to the user at each structural joint. In addition
		//		to the standard folder functions for each kind of entity, this function supports
		//		a state that is built forward on composite nodes [ for processing root-to-leaf instead
		//		of only folding leaf-to-root ].
		//      Also, it allows optionally to stop the recursion at the level of collapsed subgraphs,
		//      which are treated as a node.
		//
		//	types:
		//		G == Graph layout model (subclass of ibm_ilog.graphlayout.AbstractGraphLayoutModel)
		//		FG == folded graph layout model
		//		FC == folded child collection
		//		ST == forward-built state
		//
		//	e:  /*G*/
		//		The graph layout model (subclass of ibm_ilog.graphlayout.AbstractGraphLayoutModel) to recurse
		//
		//	ps: /*ST*/
		//		the initial parent state
		//
		//	folders: {
		//
		//		graph: function( ST, ST, G, FG ){ return FG; },
		//			Fold a graph layout model (ibm_ilog.graphlayout.AbstractGraphLayoutModel)
		//
		//		group: function( ST, ST, G, FG ){ return FG; },
		//			Fold a Group
		//
		//		sub:   function( ST, ST, G, FG ){ return FG; },
		//			Fold a Subgraph (receives folded inner Graph)
		//
		//		node:  function( ST, G ){ return FG; },
		//			Fold a Node
		//
		//		link:  function( ST, G ){ return FG; },
		//			Fold a Group node
		//
		//		other: function( ST, G ){ return FG; },
		//			Fold an unknown node (eventual gfx.Rects, etc.)
		//
		//		child: function( ST, FC, FG ){ return FC; },
		//			Fold a folded AbstractGraphLayoutModel FG into a folded child-collection FC, returning the new collection
		//
		//		cBase: FC
		//			The initially empty folded child collection (constant)
		//
		//		accSubgraph: function( ST, G ){ return ST; }
		//			The forward-state accumulator, builds a new ST given a newly encountered G
		//
		//		accGraph: function( ST, G ){ return ST; }
		//			The forward-state accumulator, builds a new ST given a newly encountered G
		//
		//		accGroup: function( ST, G ){ return ST; }
		//			The forward-state accumulator, builds a new ST given a newly encountered G
		//
		//	}
		//
		//  stopRecursionAtCollapsedSub: 
		//      If true, the recursion stops at the level of collapsed subgraphs, which are
		//      treated as a node. 
		//
		scope = scope || win.global;
	
		var foldChildren = function(c,ns,stopRecursionAtCollapsedSub) {
			return df.foldl(c,
				function(z,j){
					return folders.child.call(scope,ns,z,foldGraphModel(j,ns,folders,scope,stopRecursionAtCollapsedSub));
				},folders.cBase);
		};
	
		if(e._isIBMDiagramSubgraph) { 
	   		if (stopRecursionAtCollapsedSub && e.isCollapsed()) {
	     		return folders.node.call(scope,ps,e);
	   		} else {
				var ns = folders.accSubgraph.call(scope,ps,e); 
				return folders.sub.call(scope,ps,ns,e,foldGraphModel(new GraphAdapter(e.getGraph()),ns,folders,scope,stopRecursionAtCollapsedSub));	   	
			} 
		}
		else if(e._isIBMDiagramNode)     { return folders.node.call(scope,ps,e); }
		else if(e._isIBMDiagramLink)     { return folders.link.call(scope,ps,e); }
		else if(e._isIBMDiagramGraph)    { var ns = folders.accGraph.call(scope,ps,e); 
	                                             return folders.graph.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
    	else if(e instanceof GraphAdapter)    { var ns = folders.accGraph.call(scope,ps,e); 
	                                             return folders.graph.call(scope,ps,ns,e,foldChildren(e.getNodesAndAllLinks(),ns,stopRecursionAtCollapsedSub)); }												 
		else if(e instanceof gfx.Group)            { var ns = folders.accGroup.call(scope,ps,e); 
	                                             return folders.group.call(scope,ps,ns,e,foldChildren(e.children,ns,stopRecursionAtCollapsedSub)); }
		else { return folders.other.call(scope,ps,e); }
	};

    var _AsynchronousLayout =
	declare('ibm_ilog.diagram.graphlayout._AsynchronousLayout', null, {
        //
        //	summary:
        //		This class executes a layout algorithm asynchronously, and is the base class for the
        //		ServerSideLayout and WebWorkerLayout classes.
        //		
        //		An _AsynchronousLayout object is constructed with a Graph object.
        //		To execute a layout, the user must select the algorithm with the standard 
        //		client-side GraphLayout API:
        //
        //		graph.setNodeLayout(x);
        //		graph.setLinkLayout(y);
        //	
        //		After this, the _AsynchronousLayout object can be used to execute these 
        //		algorithms on the server or in a web worker by calling:
        //
        //		asynchronousLayout.layout();
        //
        //		This will obtain the currently configured layouts on the graph object,
        //		including optional settings for laying out internal subgraphs, gather
        //		their configured parameters and requesting the equivalent layout on the
        //		server or in the web worker.
        //
        //		Each time a layout is requested, the graph and the layout settings are
        //		serialized into a JSON object sent in the request.
        //
        //		The layout() method returns a dojo.Deferred which is signaled after the
        //		execution is complete.
        
        _localParametersEnabled: true,
        
        _running: false,
        
        _debugPrefix: "_AsynchronousLayout: ",
        
        constructor: function( /*Graph*/graph) {
            //
            //	summary: Creates and configures an AsynchronousLayout object.
            //	
            //  graph: Graph
            //		The graph to be laid out.
            //
            this._graph = graph;
            this._graphModel = new GraphAdapter(graph);
        },
        
        setLocalParametersEnabled: function(enabled) {
            //	summary:
            //		Sets whether the local (per node and per link) layout parameters 
            //		are sent to the server-side layout service or to the web worker.
            //	enable: Boolean
            //		'true' to enable the local parameters or 'false' to disable it.
            this._localParametersEnabled = enabled === true ? true : false;
        },
        
        isLocalParametersEnabled: function() {
            //	summary:
            //		Returns 'true' if the local (per node and per link) layout parameters are enabled.
            //	returns:
            //		'true' if the local (per node and per link) layout parameters are enabled. 
            return this._localParametersEnabled; // Boolean
        },
        
        _serializeMatrix: function(matrix) {
            //
            //	summary: Serializes a dojox.gfx.matrix in the JSON format needed for the protocol.
            //	
            return '[' +
            			matrix.xx + ',' + matrix.xy + ',' +
            			matrix.yx + ',' + matrix.yy + ',' +
            			matrix.dx + ',' + matrix.dy + 
					']';
        },
        
        _serializeBounds: function(rect) {
            //
            //	summary: Serializes a rectangle in the JSON format needed for the protocol.
            //	
            return rect === null ? 'null' : '[' +
            	rect.x + ',' + rect.y + ',' +
            	rect.width + ',' + rect.height +
				']';
        },
        
        _serializePoint: function(point) {
            //
            //	summary: Serializes a point in the JSON format needed for the protocol.
            //	
            return point === null ? 'null' : '[' +
            		point.x + ',' + point.y + ']';
        },
        
        _serializePoints: function(points) {
            //
            //	summary: Serializes a list of points in the JSON format needed for the protocol.
            //	
            var result = '[';
            var n = points.length;
            for (var i = 0; i < n; i++) {
                result += (i ? ',' : '') + this._serializePoint(points[i]);
            }
            return result + ']';
        },
        
        _serializeNode: function(node) {
            //
            //	summary: Serializes a node in the JSON format needed for the protocol.
            //	
            return '{' +
				'"id":"' + node.getId() + '",' +
            	'"b":' + this._serializeBounds(node.getBounds()) +
            	this._serializeLayoutNodeOrLinkParameters(node, false) +
            	'}';
        },
        
        _serializeLink: function(link) {
            //
            //	summary: Serializes a link in the JSON format needed for the protocol.
            //	
            var points = link.getPoints();
            var to = points.pop();
            var from = points.shift();
            
            var pinnedAtStartNode = this._graphModel.hasPinnedConnectionPoint(link, true);
            var pinnedAtEndNode = this._graphModel.hasPinnedConnectionPoint(link, false);
            
            return '{' +
            	'"id":"' + link.getId() + '",' +
            	'"f":"' + this._graphModel.getFrom(link).getId() + '",' +
            	'"t":"' + this._graphModel.getTo(link).getId() + '",' +
            	'"fp":' + this._serializePoint(from) + ',' +
            	'"tp":' + this._serializePoint(to) + ',' +
            	'"ip":' + this._serializePoints(points) + "," +
            	'"w":' + link.getStrokeWidth() + 
				(pinnedAtStartNode ? ("," + '"sp":true') : "") +
            	(pinnedAtEndNode ? ("," + '"ep":true') : "") +
            	this._serializeLayoutNodeOrLinkParameters(link, true) +
            	'}';
        },
        
        _serializeModel: function(graph) {
            //
            //	summary: Serializes the Graph in the JSON format needed for the protocol.		
            //
            
            var acc = function(s, ge) {
                return {
                    links: [],
                    nodes: []
                }; // new s for folding children
            };
            
            var accGroup = function(s, ge) {
                return s;
            };
            
            var fsub = function(s, ns, sg, fc) {
                var sgg = sg.getGraph();
                // The local bounds of the graph inside the subgraph
                var b = this._serializeBounds(sg.getBounds());
                
                // The local bounds of the graph inside the subgraph
                var sggBB = sgg.getLocalBoundingBox();
                
                var transformToParent = null;
				 
                // The bounds of the graph inside the subgraph
                // (to allow the server-side to determine the subgraph margins) 
                var sgt = sg.getTransform();
                if (sgt) {
                    sggBB = sgt.transformRectangle(sggBB);
										transformToParent = this._serializeMatrix(sgt);
                }
                
                var gb = this._serializeBounds(sggBB);
                              
                var serializedSub = '{' +
                	'"id":"' + sg.getId() + '",' +
                	this._serializeLayoutGlobalParameters(sgg) +
                	'"b":' + b + ',' + '"gb":' + gb + ',' +
										(transformToParent ? 
                	  ('"m":' + transformToParent + ',') : '') +
                	'"nodes":[' + fc.nodes.join(',') + '],' +
                	'"links":[' + fc.links.join(',') + ']' +
                	this._serializeLayoutNodeOrLinkParameters(sg, false) +
                	'}';
                s.nodes.push(serializedSub);
            };
            
            var fgraph = function(s, ns, ge, fc) {
                return ns;
            };
            
            var flink = function(s, ge) {
            	if (ge.getStartNode() !== null && ge.getEndNode() !== null) {
								s.links.push(this._serializeLink(ge)); // put in s
							}
              // else it is a pending link which must be filtered out
              // from the graph sent to the server-side layout
            };
            
            var fnode = function(s, ge) {
                s.nodes.push(this._serializeNode(ge)); // put in s
            };
            
            var ignore = function() {
            };
            
            var pass = function(s, ns, ge, fg) {
                return fg;
            };
            
            var sg = foldGraphModel(this._graphModel, null, gu.folders({
                graph: fgraph,
                sub: fsub,
                node: fnode,
                link: flink,
                accGraph: acc,
                accSubgraph: acc,
                accGroup: accGroup
            }), this, true /*stopRecursionAtCollapsedSubgraph*/);
            
            return '{' +
            this._serializeLayoutGlobalParameters(graph) +
            	'"nodes":[' + sg.nodes.join(',') + '],' +
            	'"links":[' + sg.links.join(',') + ']' +
            	'}';
        },
        
        _serializeLayoutGlobalParameters: function(graph) {
            var nodeLayout = graph._getActualNodeLayout();
            var linkLayout = graph._getActualLinkLayout();
            
            extendLayout(nodeLayout);
            extendLayout(linkLayout);
            return (nodeLayout ? ('"nodeLayout":' + 
										nodeLayout.getLayoutGlobalDescriptor() + ',') : "") +
            	(linkLayout ? ('"linkLayout":' + 
								linkLayout.getLayoutGlobalDescriptor() + ',') : "");
        },
        
        _serializeLayoutNodeOrLinkParameters: function(nodeOrLink, isLink) {
            if (!this.isLocalParametersEnabled()) {
                return '';
            }
            var graph = nodeOrLink.getParent();
            var nodeLayout = graph._getActualNodeLayout();
            var linkLayout = graph._getActualLinkLayout();
            var nodeLayoutNodeOrLinkDescriptor = null;
            var linkLayoutNodeOrLinkDescriptor = null;
            if (nodeLayout) {
                extendLayout(nodeLayout);
                nodeLayoutNodeOrLinkDescriptor = isLink ? 
					nodeLayout.getLayoutLinkDescriptor(nodeOrLink) : 
					nodeLayout.getLayoutNodeDescriptor(nodeOrLink);
            }
            if (linkLayout) {
                extendLayout(linkLayout);
                linkLayoutNodeOrLinkDescriptor = isLink ? 
					linkLayout.getLayoutLinkDescriptor(nodeOrLink) : 
					linkLayout.getLayoutNodeDescriptor(nodeOrLink);
            }
            return (nodeLayoutNodeOrLinkDescriptor !== null ? 
					(',"parentNodeLayout":' + nodeLayoutNodeOrLinkDescriptor) : "") +
            	(linkLayoutNodeOrLinkDescriptor !== null ? 
					(',"parentLinkLayout":' + linkLayoutNodeOrLinkDescriptor) : "");
        },
        
        layout: function() {
            //        	
            //	summary:
            //		Requests the layout and applies the results.
            //	returns:
            //		a dojo.Deferred to signal when the process finishes.
            //
            
            var graph = this._graphModel.getGraph();
            
			// Analysis of dojo.require situation:
			// We only need to execute SwimLane._beforeLayout in case:
			// 1) A instance of HierarchicalLayout is installed by the user on the graph, and
			// 2) An instance of SwimLane is used as subgraph.
			// Hence, if these 2 classes are undefined, we can safely skip it.
			// No dojo.require needed.
						
			// AMD conversion node: we now call require("classname") to test if the class is loaded.
			
			var HierarchicalLayout = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalLayout");
			var HierarchicalNodeGroup = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalNodeGroup");
			var HierarchicalSwimLaneConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalSwimLaneConstraint");
			var Direction = _require("ibm_ilog/graphlayout/Direction");
			var SwimLane = _require("../SwimLane");
			if (HierarchicalLayout && HierarchicalNodeGroup && SwimLane &&
				graph._nodeLayout && graph._nodeLayout instanceof HierarchicalLayout) {
				if (config.useGfxLayout) {
					gfxlayout.Engine.layout();
				}
				this._swimLanes = SwimLane._beforeLayout(graph, graph._nodeLayout, Direction, HierarchicalNodeGroup, HierarchicalSwimLaneConstraint);
			}
            
            var data = this._serializeModel(graph);
            
            this._preLayoutDeferred = new Deferred();
            this._layoutDeferred = new Deferred();
            
            this._running = true;
            this._start(data);
            
            // console.log(this._debugPrefix + "Sent");
			
            this._preLayout();
            
            return this._layoutDeferred; // dojo.Deferred
        },
        
        _preLayout: function() {
            //
            //	summary: Begins preparation for receiving the layout results in parallel
            //	         with the request sent to the server or the web worker.
            //	
            this._nodeIndex = new Dictionary();
            this._linkIndex = new Dictionary();
            
            foldGraphModel(this._graphModel, null, gu.folders({
                node: function(s, n) {
                    this._nodeIndex.add(n.getId(), n);
                },
                sub: function(s, ns, n, fc) {
                    this._nodeIndex.add(n.getId(), n);
                },
                link: function(s, l) {
                    this._linkIndex.add(l.getId(), l);
                }
            }), this);
            
            this._preLayoutDeferred.callback();
        },
        
        _layoutGraph: function(laidOutGraph) {
        
		    /*
            var logTr = function(ge) {
                var tr = ge.getTransform();
                console.log("\"", ge.toString(), tr ? ("\" (" + tr.dx + "," + tr.dy + ")") : 
							"", ge._isIBMDiagramSubgraph ? g.printRect(ge.getGraph().getBoundingBox()) : "");
            };
            */
			
            //
            //	summary: Applies the layout result.
            //	
            var gm = null, ls = this._linkIndex, ns = this._nodeIndex;
            var graph;
            
            if (laidOutGraph.id) {
                graph = ns.item(laidOutGraph.id).getGraph();
            } else {
                graph = this._graph;
            }
            
            var adapter = new GraphAdapter(graph);
            
            var getRect = function(field) {
                return field ? {
                    x: field[0],
                    y: field[1],
                    width: field[2],
                    height: field[3]
                } : null;
            };
			
            var maybeRect = function(field) {
                return field ? getRect(field) : null;
            };
            
            if (laidOutGraph.swimLaneCalcBBoxes) {
                // HierarchicalLayout.getConstraints returns the constraints in their
                // *reversed* insertion order. Thanks to that, we can rely on the fact that the 
                // calculated bboxes of swimlane constraints stored by the server-side
                // layout in JSON are ordered similarly to the client-side constraints. 
                var constraints = graph.getNodeLayout().getConstraints();
                array.forEach(laidOutGraph.swimLaneCalcBBoxes, function(calcBBox) {
                    var constraint = null;
					var HierarchicalSwimLaneConstraint = _require("ibm_ilog/graphlayout/hierarchical/HierarchicalSwimLaneConstraint");
                    while (constraints.hasNext() &&
                    	   (constraint === null ||
						    // No dojo.require needed. If the class is undefined, the constraint can't be
						    // of this type. 
                    		!(HierarchicalSwimLaneConstraint &&
							  constraint instanceof HierarchicalSwimLaneConstraint))) {
                        // skip any constraint which is not a swimlane constraint
                        constraint = constraints.next();
                    }
                    if (constraint !== null) { // we found a swimlane constraint
                        var bbox = maybeRect(calcBBox);
                        constraint.setCalcBoundingBox(bbox);
                    }
                }, this);
            }
            
            array.forEach(laidOutGraph.nodes, function(laidOutNode) {
                var n = ns.item(laidOutNode.id);
                if (n) {
                    if (laidOutNode.nodes) {
                        // FIRST: lay out subgraph contents first to get final content bounds
                        this._layoutGraph(laidOutNode);
                        
                        // ALSO: we need to update internal links to have correct bounds:
                        Batch.endBatch();
                    }
                    adapter.moveNode(n, laidOutNode.b[0], laidOutNode.b[1]);

                    // logTr(n);
                }
            }, this);
            
            var getPoint = function(field) {
                return field ? {
                    x: field[0],
                    y: field[1]
                } : null;
            };
			
            var maybePoint = function(field) {
                return field ? getPoint(field) : null;
            };
            
            array.forEach(laidOutGraph.links, function(laidOutLink) {
                var l = ls.item(laidOutLink.id);
                
                var fp = maybePoint(laidOutLink.fp);
                var tp = maybePoint(laidOutLink.tp);
                var ip = df.map(laidOutLink.ip || [], getPoint);
                
				// Analysis of dojo.require situation:
				// ibm_ilog.graphlayout.ReshapeLinkStyle and ibm_ilog.graphlayout.ReshapeLinkMode are
				// both defined in AbstractGraphLayoutModel, which is the parent class of 
				// GraphAdapter. We have already a dojo.require for GraphAdapter. Hence, 
				// no need of dojo.require for ibm_ilog.graphlayout.ReshapeLinkStyle and 
				// ibm_ilog.graphlayout.ReshapeLinkMode.
                adapter.reshapeLink(l, GraphModelUtil.ReshapeLinkStyle.POLYLINE, 
									fp, GraphModelUtil.ReshapeLinkMode.MOVE_AND_CLIP, ip, 0, ip.length, 
									tp, GraphModelUtil.ReshapeLinkMode.MOVE_AND_CLIP);
            }, this);
        },
        
        _onComplete: function(data) {
            //
            //	summary: The server or the web worker has returned the results of the layout. 
            //           The receiver is about to apply the results to the Graph object.
            //           When ready, the deferred object returned by layout() is signaled.
            //	
            
            // console.log(this._debugPrefix + "Response received, applying...");
            // console.dir(this._laidout);
            
            this._layoutGraph(this._laidout);
            
            if (this._swimLanes) {
                Batch.endBatch();
                var graph = this._graphModel.getGraph();
				var SwimLane = _require("../SwimLane");
                SwimLane._afterLayout(graph, graph._nodeLayout, this._swimLanes);
                this._swimLanes = null; // cleanup for gc
            }
            
            this._running = false;
            this._layoutDeferred.callback();
            
            // console.log(this._debugPrefix + "Response applied.");
        },
        
        _onError: function(error) {
            //
            //	summary: A error occurred while trying to execute the layout algorithm. 
            //	
            
            // console.log(this._debugPrefix + "Error. ", error.responseText);
            
            this._running = false;
            this._layoutDeferred.errback(error);
        },
        
        stop: function() {
            // summary:
            //		Stops the execution of the layout.
            // description:
            //		If a graph layout is currently running,	calling this method will stop 
			// 		the graph layout execution.
            //		No result is applied and the graph stays unchanged.
            
            if (this._running) {
                this._stop();
                this._running = false;
            }
        },
        
        isRunning: function() {
            // summary:
            //		Returns true if the layout is currently running.
            return this._running;
        },
        
        _start: function(data) {
            // summary:
            //		Starts the asynchronous layout. This method must be redefined in subclasses.
        },
        
        _stop: function() {
            // summary:
            //		Stops the current layout. This method must be redefined in subclasses.
        }
    });
    
/*=====
var _AsynchronousLayout = ibm_ilog.diagram.graphlayout._AsynchronousLayout;
=====*/

	var ServerSideLayout =
    declare('ibm_ilog.diagram.graphlayout.ServerSideLayout', _AsynchronousLayout, {
        //
        //	summary:
        //		This class executes a layout algorithm on a server. 
        //		
        //		A ServerSideLayout object is constructed with the Graph object and the
        //		URL where the layout service is located.
        //		To execute a layout, the user must select the algorithm with the standard 
        //		client-side GraphLayout API:
        //
        //		graph.setNodeLayout(x);
        //		graph.setLinkLayout(y);
        //	
        //		After this, the ServerSideLayout object can be used to execute these 
        //		algorithms on the server by calling:
        //
        //		serverSideLayout.layout();
        //
        //		This will obtain the currently configured layouts on the graph object,
        //		including optional settings for laying out internal subgraphs, gather
        //		their configured parameters and requesting the equivalent layout on the
        //		server.
        //
        //		Each time a layout is requested, the graph and the layout settings are
        //		serialized into a JSON object sent in the request.
        //
        //		The layout() method returns a dojo.Deferred which is signaled after the
        //		execution is complete.
        
        //	_url: string
        //		The URL where the layout service is located
        _url: "/dojo-diagrammer-server/graphlayout",
        
        _timeout: undefined,
        
        _debugPrefix: "ServerSideLayout: ",
        
        constructor: function(/*Graph*/graph, /*string*/url) {
            //
            //	summary: Creates and configures a ServerSideLayout object.
            //	
            //  graph: Graph
            //		The graph to be laid out.
            //	url: String
            //		The URL where the server is located.
            //
            this._url = url || this._url;
        },
        
        setUrl: function(url) {
            //	summary:
            //		Set the location of the layout service at the server.
            //  url: String
            //		The URL of the layout service at the server.
            //   
            this._url = url;
        },
        
        getUrl: function() {
            //	summary:
            //		Returns the location of the layout service at the server.
            //  returns:
            //		The location of the layout service at the server.
            return this._url; // String
        },
        
        setTimeout: function(msTimeout) {
            //	summary:
            //		Sets the timeout for the request sent to the server. The unit is milliseconds.
            //  msTimeout: int
            //		The timeout (in milliseconds) for the request sent to the server.
            this._timeout = msTimeout;
        },
        
        getTimeout: function() {
            //	summary:
            //		Returns the current timeout for the request sent to the server.
            //  returns:
            //		The timeout (in milliseconds) for the request sent to the server.
            return this._timeout; // int
        },
        
        _start: function(data) {
            var layoutModelKW = {
            
                url: this._url,
                
                postData: data,
                
                headers: lang.mixin({
                    "Content-Type": "application/json"
                }, {}),
                
                load: lang.hitch(this, function(response, ioArgs) {
                    this._laidout = json.fromJson(response);
                    this._preLayoutDeferred.addCallback(lang.hitch(this, "_onComplete"));
                }),
                
                error: lang.hitch(this, function(data, ioArgs) {
                    if (data.dojoType != 'cancel') { 
                        this._onError(data);
					}
                })
            };
            
            if (this._timeout !== undefined) {
                layoutModelKW.timeout = this._timeout;
            }
            
            // console.log(this._debugPrefix + "Sending xhrPost");
            // console.dir(json.fromJson(data));
			
            this._postDeferred = xhr.post(layoutModelKW);
        },
        
        _stop: function() {
            if (this._postDeferred && this._postDeferred.cancel) { 
                this._postDeferred.cancel();
			}
        }
    });
   
   ServerSideLayout._AsynchronousLayout = _AsynchronousLayout;
   
   return ServerSideLayout;
});

},
'ibm_ilog/diagram/interactors/MoveInteractor':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/interactors/MoveInteractor", [
"dojo/_base/declare",
"dojo/_base/array",
"dojo/_base/sniff",
"require",
"dojox/gfx/matrix",
"dojox/gfx/utils",
"./DragInteractor",
"../Selection",
"../util/Geometry",
"../util/GraphUtil",
"../util/Batch"
],
function(
declare,
array,
has,
require,
m,
gfxu,
DragInteractor,
Selection,
g,
gu,
Batch
){

/*=====
ibm_ilog.diagram.__MoveInteractorArgs = function() {
	//	doMove: Function?
	//		Concrete the movement to a graph element.
	//	isMovable: Function?
	//		Return true if the given graph element is allowed to move
	this.doMove = function(graphElement,originalTransform,delta) {
		//	summary:
		//		Concrete the movement to a graph element.
		//	graphElement:
		//		The graph element to move.
		//	originalTransform:
		//		The matrix representing the location from which the delta is computed.
		//	delta:
		//		The amount to translate the entity from its original location.
	}
	this.isMovable = function(graphElement) {
		//	summary:
		//		Return true if the given graph element is allowed to move
		//	graphElement:
		//		The graph element to test.
	}
};	
=====*/

var superclass = DragInteractor.prototype;

// The following classes are loaded dynamically if the editor modules are loaded.

var MultipleAction = null;
var UndoActionList = null;
var EditingUtils = null;
var _classesRequired = false;

var _requireClasses = function()
{
	if(!_classesRequired){
		// no try/catch, if we are in the editor these classes must be there.
		MultipleAction = require("../editor/undo/MultipleAction");
		UndoActionList = require("../editor/undo/UndoActionList");
		EditingUtils = require("../editor/EditingUtils");
	}
	_classesRequired = true;
}

/*=====
var DragInteractor = ibm_ilog.diagram.interactors.DragInteractor;
=====*/

var MoveInteractor =
declare("ibm_ilog.diagram.interactors.MoveInteractor", [DragInteractor], {
	//
	//	summary:
	//		A MoveInteractor allows the user to move the currently selected entities.
	//
	
	//
	//	_selection: ibm_ilog.diagram.Selection
	//		The selection, object, whose entities are to be moved by the operation
	//
	_selection: null,

	//
	//	_viewport: ibm_ilog.diagram.widget.Diagram
	//		The viewport witht the content to move
	//
	_viewport: null,

	//
	//	_content: dojox.gfx.Group
	//		The root of the elements to make moveable, used as the source of events.
	//
	_content: null,

	//
	//	_targetSet: array
	//		The elements that are going to be moved.
	//
	_targetSet: null,
	
	//
	//	_dragCursor: string
	//		id of the cursor used while dragging
	//
	_dragCursor: "move",
		
	initialize: function( /*ibm_ilog.diagram.Selection*/ selection, /*ibm_ilog.diagram.Viewport|ibm_ilog.diagram.Graph*/ viewportOrContent, /*ibm_ilog.diagram.__MoveInteractorArgs*/ callbacks ) {
		//
		//	summary:
		//		Constructs the MoveInteractor
		//

		if(viewportOrContent._isIBMDiagramDiagram) {
			this._viewport = viewportOrContent.getViewport();
			this._content = this._viewport.getContent();
		} else if(viewportOrContent._isIBMDiagramViewport) {
			this._viewport = viewportOrContent;
			this._content = this._viewport.getContent();
		} else if(viewportOrContent._isIBMDiagramGraph) {
			this._content = gu.getSurface(viewportOrContent);
			this._cursorStyle = this._content.getEventSource().style; 
		} else {
			R.error("UnexpectedArgument");
		}

		if(callbacks) {
			if(callbacks.doMove) {
				this._doMove = callbacks.doMove;
			}
			
			if(callbacks.isMovable) {
				this._isMovable = callbacks.isMovable;
			}
		}
		this._selection = selection;
		this._initialize();
		if(has("ie") < 8) this._ie7 = true;
		return this;
	},

	_getInitialEventSource: function() {
		return this._content; 
	},
	
	activate: function() {
		//
		//	summary:
		//		Activates the receiver by connecting it to its sources of events.
		//
		this._fconnect("selection",this._selection,"onSelectionChanged",this,"_onSelectionChanged");
		this.inherited(arguments);
		if(this._enabled) {
			this._buildTargetSet();
		}
	},
	
	onEnabled: function() {
		this._buildTargetSet();
	},
	
	_onSelectionChanged: function() {
		this._buildTargetSet();
	},

	_isMovable: function(ge) {
		return true;
	},
	
	_buildTargetSet: function() {
		//
		//	summary:
		//		Builds the set of entities that are going to be affected by the drag.
		//
		//	description:
		//		Grab Nodes and Subgraphs from the _selection object. Their original 
		//    transforms are saved to be used in the drag process. We also store 
		//		viewport-to-element transforms since they are not likely to change
		//		while dragging (would require modifying the transformation of some
		//		of the parents of the selected shape). This might change if we plan
		//		to provide pan-while-moving.
		//

		var s = this._selection.fastGet();
		var gs = [];
		
		s.forEach(function(e){
			var ge = this._selection._asGraphElement(e);
			if(ge && this._isMovable(ge)) {
				// no link moving for now
				gs.push(ge);
			}
		},this);
		
		this._targetSet = array.map(gu.maximals(gs),function(e){
			return { t1:e.getTransform(), e:e, t:this.getTransformToElement(e) };
		},this);
		
	},
	
	getTransformToElement: function(e) {
		//	returns:
		//		a transform from viewport coords to element coords
		var rm = e._getRealMatrix();
		return (rm && rm.inverse());
	},

	_getGraphElementFromEvent: function (e) {
		//	summary:
		//		return the selection object
		var ge = this._getGraphElementFromEventInRoot(e, this._content);
		return ge !== this._content ? ge : null;
	},

	_dragPreStart: function(e) {
		
		var starter = this._getGraphElementFromEvent(e);
		if(starter && !starter._isIBMDiagramLink && starter.isSelectable()) {
			//
			// [AV]
			// We should not be able to click on a non-selectable/moveable entity and 
			// start the drag from there, even thouh other entities in the target set
			// may be moveable.
			// To avoid this, we peek what's right below the position where we started
			// the drag. It should be a node or subgraph, and should be selectable.
			//
			// note: We may have not selected the to-be-dragged entity yet. It might 
			//       be selected in this same click right after this handler. So we can't
			//       test for isSelected() here.
			// 
		
			this.inherited(arguments);
			this._doDragPreStart(e);
		}
	},
	
  _dragTouchPreStart: function (e) {
    e.target = e.touches[0].target;
    var starter = this._getGraphElementFromEvent(e);
    if(starter && !starter._isIBMDiagramLink && starter.isSelectable()) {
      //
      // [AV]
      // We should not be able to click on a non-selectable/moveable entity and 
      // start the drag from there, even thouh other entities in the target set
      // may be moveable.
      // To avoid this, we peek what's right below the position where we started
      // the drag. It should be a node or subgraph, and should be selectable.
      //
      // note: We may have not selected the to-be-dragged entity yet. It might 
      //       be selected in this same click right after this handler. So we can't
      //       test for isSelected() here.
      // 
      this.inherited(arguments);
      
      this._doDragPreStart(e);
    }
    
  },

  _doDragPreStart: function (e){
    
    array.forEach(this._targetSet,function(s){
      s.t = this.getTransformToElement(s.e);
      s.t1 = s.e.getTransform();
    },this);
    
    if(this._viewport) {
      // Monitor viewport transform changes while dragging, so that we can update 
      // the viewport-to-element transforms.
      this._connect("updateTransforms",this._viewport,"onViewRectChanged",this,"_viewportMoved");
    }
    
  },
  
	_viewportMoved: function(){
		var oldit = this._firstViewportT || m.identity;
		var newt = this._viewport.getViewportTransform();
		
		// compute the new location of the first click in client coords.
		this._first = newt.transformPoint(oldit.transformPoint(this._first));
		
		array.forEach(this._targetSet,function(s){
			s.t = this.getTransformToElement(s.e);
		},this);
		this._firstViewportT = this._viewport.getViewportInverseTransform();
	},

	_doMove: function(graphElement,originalTransform,delta) {
		//	summary:
		//		Concrete the movement to a graph element.
		//	graphElement:
		//		The graph element to move.
		//	originalTransform:
		//		The matrix representing the location from which the delta is computed.
		//	delta:
		//		The ammount to translate the entity from its original location.
		graphElement.setTransform(m.multiply(originalTransform,m.translate(delta)));
		// workaround for IE7 bug when selecting several objects quickly
		if(this._ie7 && !graphElement._isIBMDiagramSubgraph){
			gfxu.forEach(graphElement, function(s){
				if(s.setShape && !s._isIBMDiagramMultilineText)
					s.setShape(s.getShape());
			});
		}
	},
	
	_dragMove: function(e) {
		//
		//	summary:
		//		Applies the current movement delta to the selected objects.
		//
		//	notes:
		//		It ends up transforming the UI coords to model coords and back to scrollbars coords,
		//		May need to be refactored if this is a performance bottleneck
		//
		
		// this.inherited(arguments); 
		// replaced by the following for performance:
		superclass._dragMove.apply(this, arguments);
		//
		
    this._doDragMove(e);
	},
	
  _dragTouchMove: function (e) {
    //
    //  summary:
    //    Applies the current movement delta to the selected objects.
    //
    //  notes:
    //    It ends up transforming the UI coords to model coords and back to scrollbars coords,
    //    May need to be refactored if this is a performance bottleneck
    //
    
    // this.inherited(arguments); 
    // replaced by the following for performance:
    superclass._dragTouchMove.apply(this, arguments);
    //
    this._doDragMove(e);
  },	
  
  _doDragMove: function (e){
    // check move limits defined on the Graph if any:
    var current = this._current;
    array.forEach(this._targetSet,function(s){
      var delta = g.subPoint(m.multiplyPoint(s.t,current),m.multiplyPoint(s.t,this._first));
      var p = s.e.getParent();
      if (p && p.onElementMoving) {
        var oldx = delta.x;
        var oldy = delta.y;
        p.onElementMoving.call(p, s.e, s.t1, delta);
        if (delta.x != oldx || delta.y != oldy)
          current = m.multiplyPoint(s.t.inverse(),g.addPoint(delta, m.multiplyPoint(s.t,this._first)));
      }
    },this);
    
    array.forEach(this._targetSet,function(s){
      var delta = g.subPoint(m.multiplyPoint(s.t,current),m.multiplyPoint(s.t,this._first));
      this._doMove(s.e,s.t1,delta);
    },this);
    
    Batch.endBatch(); // update links immediately for better visual result
    
  },
	_dragEnd: function(e) {
		var dragged = this.hasDragged;
		this.inherited(arguments);
    this._doDragEnd(dragged);
	},
	
  _dragTouchEnd: function (e) {
    var dragged = this.hasDragged;
    this.inherited(arguments);
    this._doDragEnd(dragged);
  },
  
  _doDragEnd: function (dragged){
    this._disconnect("updateTransforms");
    
    if(dragged && this._viewport && this._viewport.getDiagram()._isIBMDiagramEditor){
      var um = this._viewport.getDiagram().getUndoManager();
      um.addAction(this._createUndoAction());
    }
    
  },
  
	_createUndoAction: function(){
		
		_requireClasses();
		
		var mAction = new MultipleAction(UndoActionList.Move);
		
		array.forEach(this._targetSet, function(s){
			var action = EditingUtils.createMoveUndoAction(s.e, s.t1, s.e.getTransform());
			mAction.addAction(action);
		});
			
		return mAction;
	}
});

return MoveInteractor;

});

},
'ibm_ilog/diagram/util/DeferredInitialization':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
define("ibm_ilog/diagram/util/DeferredInitialization", ["dojo/_base/declare", "dojo/_base/connect", "./GlobalInitialization"],
function(declare, connect, gi){
	
	var di = declare("ibm_ilog.diagram.util.DeferredInitialization",[],{
		//
		//	summary:
		//		Provide the users of the class with a whenLoaded(cb) API for
		//		waiting for instances to finish initialization. Also subscribes
		//		the instance to the global initialization coordinator for global
		//		synchrinzation.
		//
		__initialized: false,
		
		_beginInit: function() {
			//
			//	summary:
			//		Begins initialization process.
			//
			this.__initialized = false;
	    	gi.beginInit();
	    },
	    
	    _endInit: function() {
			//
			//	summary:
			//		Finishes initialization process, subsequently executing all 
	    	//		registered callbacks.
			//
			this.__initialized = true;
	    	gi.endInit();
	    },
        
        isInitialized: function(){
            // summary:
            //     Returns whether the initialization process is done.
            return this.__initialized;
        },
	    
	    _defaultErrorHandler: function() {
	    	console.log(arguments);
	    },
	    
	    whenLoaded: function(cb,eb) {
			//
			//	summary:
			//		Registers a callback to be executed after the instance has 
	    	//		been initialized, or executes immediately if already initialized.
			//
	    	if(this.__initialized) {
	    		cb();
	    	} else {
		    	var h = connect.connect(this,"_endInit",function(){
		    		cb();
		    		connect.disconnect(h);
		    	});
	    	}
	    	return this;
	    }
    
	});
	
	return di;
});

}}});

require(["dojo/i18n"], function(i18n){
i18n._preloadLocalizations("ibm_ilog/diagram/nls/ibm_diagram_base", []);
});
define("ibm_ilog/diagram/ibm_diagram_base", [], 1);
