/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

//>>built
require({cache:{
'ibm_ilog/graphlayout/longlink/LLGrid':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLGrid", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../longlink/LLGridLine"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    LLGridLine	// #7
    ) {

var LLGrid = declare("ibm_ilog.graphlayout.longlink.LLGrid", null, 
{
  // summary:
  //		The routing grid for the link routing algorithm. 
  // description:
  //		The routing grid for the link routing algorithm. 
  //		<p>
  //		It stores the information which grid points are free to route a line
  //		and which are used. Some grid points may be free only in certain
  //		routing directions. During a routing, it also stores the backtrack
  //		distance per grid point for the routing. The backtrack distance is
  //		the distance a routing needs to be larger than the shortest direct
  //		routing. For instance to route from point (x0, y0) to point (x1, y1),
  //		the direct distance is D = abs(x1- x0) + abs(y1 - y0). The backtrack
  //		distance of a routing that has orthogonal length L is B = L - D. We
  //		use the grid to find a routing with the shortest backtrack distance.
  //		In order to manage the grid points effectively, we use the grid line
  //		data structure. This reduces the memory overhead compared to storing
  //		each single grid point, because often large areas of neighbored grid
  //		points behave absolutely identical.  
  //		</p>
  // tags:
  //		private
  
  _baseCoord: null,

  _gridDist: null,

  _minDist: null,

  _numberOfGridPoints: null,

  _gridLinesX: null,
  _gridLinesY: null,

  _dirtyLines: null,

  _dummyLine: null,

  _immediateReorganizationNeeded: null,

  constructor : function()
  {
  },
  
  init : function(baseCoordX, baseCoordY, gridDistX, gridDistY, minDistX, minDistY, gridSizeX, gridSizeY)
  {
    // summary:
    //		Initializes the grid. 
    // description:
    //		Initializes the grid. 
    //		This must be called after construction, before the grid can be used.
    //		It is also possible to reinitialize the grid this way without
    //		allocating a new one.
    // tags:
    //		private
    
    this._baseCoord = TranslateUtil.newArray(2, 0);
    this._baseCoord[0] = baseCoordX;
    this._baseCoord[1] = baseCoordY;
    this._gridDist = TranslateUtil.newArray(2, 0);
    this._gridDist[0] = (gridDistX > 1.0 ? gridDistX : 1.0);
    this._gridDist[1] = (gridDistY > 1.0 ? gridDistY : 1.0);
    this._minDist = TranslateUtil.newArray(2, 0);
    this._minDist[0] = (minDistX > 1.0 ? minDistX : 1.0);
    this._minDist[1] = (minDistY > 1.0 ? minDistY : 1.0);
    this._numberOfGridPoints = TranslateUtil.newArray(2, 0);
    this._numberOfGridPoints[0] = gridSizeX;
    this._numberOfGridPoints[1] = gridSizeY;
    this._gridLinesX = TranslateUtil.newArray(gridSizeX, null);
    this._gridLinesY = TranslateUtil.newArray(gridSizeY, null);
    var i;

    for (i = 0; i < gridSizeX; i++)
      this._gridLinesX[i] = new LLGridLine(gridSizeY - 1, i, false);

    for (i = 0; i < gridSizeY; i++)
      this._gridLinesY[i] = new LLGridLine(gridSizeX - 1, i, true);
    this._dummyLine = new LLGridLine(1, -1, false);
    this._dirtyLines = this._dummyLine;
    this._immediateReorganizationNeeded = false;
  },

  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._baseCoord = null;
    this._gridDist = null;
    this._minDist = null;
    var i;

    for (i = 0; i < this._numberOfGridPoints[0]; i++)
      this._gridLinesX[i].dispose();

    for (i = 0; i < this._numberOfGridPoints[1]; i++)
      this._gridLinesY[i].dispose();
    this._numberOfGridPoints = null;
    this._dummyLine.dispose();

    this._dummyLine = null;
    this._dirtyLines = null;
  },

  isImmediateReorganizationNeeded : function()
  {
    // summary:
    //		Returns true if the grid needs immediate reorganization.
    // tags:
    //		private
    
    return (this._immediateReorganizationNeeded); // Boolean
  },

  setImmediateReorganizationNeeded : function(flag)
  {
    // summary:
    //		Sets whether the grid needs immediate reorganization.
    // tags:
    //		private
    
    this._immediateReorganizationNeeded = flag;
  },

  getMinCoord : function(dir)
  {
    // summary:
    //		Returns the smaller coordinate of the grid. 
    // description:
    //		Returns the smaller coordinate of the grid. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._baseCoord[dir]); // Number
  },

  getMaxCoord : function(dir)
  {
    // summary:
    //		Returns the larger coordinate of the rectangle. 
    // description:
    //		Returns the larger coordinate of the rectangle. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._baseCoord[dir] + this._numberOfGridPoints[dir] * this._gridDist[dir]); // Number
  },

  getGridCoord : function(dir, i)
  {
    // summary:
    //		Returns the coordinate of the ith grid line. 
    // description:
    //		Returns the coordinate of the ith grid line. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._baseCoord[dir] + i * this._gridDist[dir]); // Number
  },

  getGridIndex : function(dir, coord, larger)
  {
    // summary:
    //		Returns the grid index corresponding to a coordinate. 
    // description:
    //		Returns the grid index corresponding to a coordinate. 
    //		Direction 0 means x, and 1 means y. If larger is true, it returns an
    //		index with larger or equal coordinate, otherwise an index with
    //		smaller or equal coordinate.
    // tags:
    //		private
    
    var resf = (coord - this.getMinCoord(dir)) / this._gridDist[dir];
    if (larger) {
      return (Math.floor(Math.ceil(resf))); // int
    }
    return (Math.floor(Math.floor(resf))); // int
  },

  getClosestGridIndex : function(dir, coord)
  {
    // summary:
    //		Returns the closest grid index corresponding to a coordinate. 
    // description:
    //		Returns the closest grid index corresponding to a coordinate. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    var index1 = this.getGridIndex(dir, coord, false);
    var index2 = this.getGridIndex(dir, coord, true);
    var d1 = coord - this.getGridCoord(dir, index1);
    var d2 = this.getGridCoord(dir, index2) - coord;
    if (d1 < d2) {
      return (index1); // int
    }
    return (index2); // int
  },

  getGridIndexAtMinDist : function(dir, coord, larger)
  {
    // summary:
    //		Returns the grid index corresponding to a coordinate, with at least
    //		minDist larger or smaller. 
    // description:
    //		Returns the grid index corresponding to a coordinate, with at least
    //		minDist larger or smaller. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    var resf = (coord - this.getMinCoord(dir)) / this._gridDist[dir];
    var res;

    if (larger)
    {
      res = Math.floor(Math.floor(resf));

      while (this.getGridCoord(dir, res) - coord < this._minDist[dir])
        res++;
    }
    else
    {
      res = Math.floor(Math.ceil(resf));

      while (coord - this.getGridCoord(dir, res) < this._minDist[dir])
        res--;
    }

    return (res); // int
  },

  getNumberOfGridLines : function(dir)
  {
    // summary:
    //		Returns the number of grid lines. 
    // description:
    //		Returns the number of grid lines. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._numberOfGridPoints[dir]); // int
  },

  getGridDist : function(dir)
  {
    // summary:
    //		Returns the distance between gridlines. 
    // description:
    //		Returns the distance between gridlines. 
    //		Direction 0 means x, and 1 means y.
    // tags:
    //		private
    
    return (this._gridDist[dir]); // Number
  },

  getGridLine : function(dir, i)
  {
    // summary:
    //		Returns the ith grid line in direction dir.
    // tags:
    //		private
    
    if (dir == 0) {
      return this._gridLinesX[i]; // ibm_ilog.graphlayout.longlink.LLGridLine
    }
    return this._gridLinesY[i]; // ibm_ilog.graphlayout.longlink.LLGridLine
  },

  searchClosestSegmentAlive : function(index0, index1, dir, larger)
  {
    // summary:
    //		Returns the closest segment that away from the start point given by
    //		index0 and index1. 
    // description:
    //		Returns the closest segment that away from the start point given by
    //		index0 and index1. 
    //		The direction dir is the direction of index0. If dir is 0, it
    //		searches in horizontal direction, if it is 1, it searches in
    //		vertical direction. If larger is true, it searches a larger segment.
    //		The returned segment can either be horizontally or vertically
    //		oriented. Preferably it is oriented same to the search direction.
    // tags:
    //		private
    
    if (index0 < 0 || index0 >= this._numberOfGridPoints[dir]) {
      return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    if (index1 < 0 || index1 >= this._numberOfGridPoints[1 - dir]) {
      return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }

    var seg1 = this.getGridLine(1 - dir, index1).searchClosestSegmentAlive(this, index0, larger);
    var seg2;
    var startIndex;
    var endIndex;
    var maxIndex = this._numberOfGridPoints[dir];

    if (larger)
    {
      startIndex = (index0 > 0 ? index0 : 0);
      endIndex = maxIndex;
    }
    else
    {
      startIndex = (index0 < maxIndex ? index0 : maxIndex - 1);
      endIndex = -1;
    }

    if (seg1 != null)
    {
      if (index0 < seg1.getStartIndex()) {
        endIndex = seg1.getStartIndex();
      }
      else if (index0 > seg1.getEndIndex()) {
        endIndex = seg1.getEndIndex();
      }
      else {
        return (seg1); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
    }

    var i;

    if (larger)
    {
      for (i = startIndex; i < endIndex; i++)
      {
        seg2 = this.getGridLine(dir, i).searchSegment(index1);
        if (seg2 != null && !seg2.isDeadlock(this, 0, /*Number.MAX_VALUE*/2147483647)) {
          return (seg2); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
      }
    }
    else
    {
      for (i = startIndex; i > endIndex; i--)
      {
        seg2 = this.getGridLine(dir, i).searchSegment(index1);
        if (seg2 != null && !seg2.isDeadlock(this, 0, /*Number.MAX_VALUE*/2147483647)) {
          return (seg2); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
      }
    }

    return (seg1); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  getMinGridIndexAtSide : function(rect, dir, minCornerOffset)
  {
    // summary:
    //		Returns the smallest grid index at the given side of the rectangle.
    // tags:
    //		private
    
    var coord = LLGrid.getMinCoord(rect, dir) + minCornerOffset;
    var size = LLGrid.getSize(rect, dir);

    if (size < 2 * minCornerOffset)
    {
      var a = this.getGridIndex(dir, coord - minCornerOffset, true);
      var b = this.getGridIndex(dir, coord - minCornerOffset + size, false);
      if (a < b) {
        return (Math.floor((a + b) / 2)); // int
      }
      return (a); // int
    }

    return (this.getGridIndex(dir, coord, true)); // int
  },

  getMaxGridIndexAtSide : function(rect, dir, minCornerOffset)
  {
    // summary:
    //		Returns the largest grid index at the given side of the rectangle.
    // tags:
    //		private
    
    var coord = LLGrid.getMaxCoord(rect, dir) - minCornerOffset;
    var size = LLGrid.getSize(rect, dir);

    if (size < 2 * minCornerOffset)
    {
      var a = this.getGridIndex(dir, coord + minCornerOffset - size, true);
      var b = this.getGridIndex(dir, coord + minCornerOffset, false);
      if (a < b) {
        return (Math.floor((a + b) / 2)); // int
      }
      return (b); // int
    }

    return (this.getGridIndex(dir, coord, false)); // int
  },

  markDirty : function(gridLine)
  {
    // summary:
    //		Marks a grid line as dirty.
    // tags:
    //		private
    
    if (!gridLine.isDirty())
    {
      gridLine.setNextDirty(this._dirtyLines);
      this._dirtyLines = gridLine;
    }
  },

  reorganizeAllBeforeSearch : function()
  {
    // summary:
    //		Reorganize the complete grid after setting all obstacles, before
    //		starting a search.
    // tags:
    //		private
    
    var maxNrGPX = this._numberOfGridPoints[0];
    var maxNrGPY = this._numberOfGridPoints[1];
    var i;

    for (i = 0; i < maxNrGPX; i++)
      this._gridLinesX[i].reorganizeBeforeSearch(false);

    for (i = 0; i < maxNrGPY; i++)
      this._gridLinesY[i].reorganizeBeforeSearch(false);
    this._dirtyLines = this._dummyLine;
  },

  reorganizeDirtyBeforeSearch : function()
  {
    // summary:
    //		Reorganize the dirty grid lines before the next routing. 
    // description:
    //		Reorganize the dirty grid lines before the next routing. 
    //		This can be faster if the previous route
    //		didn't cover the complete grid. It also clears the grid.
    // tags:
    //		private
    
    var gridLine = this._dirtyLines;
    var nextGridLine;

    while (gridLine != this._dummyLine)
    {
      nextGridLine = gridLine.getNextDirty();
      gridLine.reorganizeBeforeSearch(false);
      gridLine = nextGridLine;
    }

    this._dirtyLines = this._dummyLine;
  },

  clearDirty : function()
  {
    // summary:
    //		Clears the dirty grid lines before the next routing. 
    // description:
    //		Clears the dirty grid lines before the next routing. 
    //		This can be faster if the previous route
    //		didn't cover the complete grid. It can be called if reorganization is not necessary because no new obstacles were set on the grid, but a previous route kept info data on the grid. This info data is removed.
    // tags:
    //		private
    
    var gridLine = this._dirtyLines;
    var nextGridLine;

    while (gridLine != this._dummyLine)
    {
      nextGridLine = gridLine.getNextDirty();
      gridLine.clear();
      gridLine = nextGridLine;
    }

    this._dirtyLines = this._dummyLine;
  },

  isWithoutObstacles : function(dir, index, limits)
  {
    // summary:
    //		Returns true if the area from limits[0] to limits[1] is
    //		obstacle-free on grid link with index. 
    // description:
    //		Returns true if the area from limits[0] to limits[1] is
    //		obstacle-free on grid link with index. 
    //		If dir is 0, the free segment should be horizontal, otherwise
    //		vertical.
    // tags:
    //		private
    
    var line = this.getGridLine(1 - dir, index);
    if (line == null) {
      return (false); // Boolean
    }
    var segment = line.searchSegment(limits[0]);
    if (segment == null) {
      return (false); // Boolean
    }
    if (segment.getStartIndex() > limits[1]) {
      return (false); // Boolean
    }
    if (segment.getEndIndex() < limits[1]) {
      return (false); // Boolean
    }
    return (true); // Boolean
  },

  isWithinRectObstacle : function(xindex, yindex, rect)
  {
    // summary:
    //		Returns true if the input grid index coordinate is within the input
    //		rectangle obstacle.
    // tags:
    //		private
    
    var gridLineX0 = this.getGridIndexAtMinDist(0, rect.x, false);
    var gridLineX1 = this.getGridIndexAtMinDist(0, rect.x + rect.width, true);
    var gridLineY0 = this.getGridIndexAtMinDist(1, rect.y, false);
    var gridLineY1 = this.getGridIndexAtMinDist(1, rect.y + rect.height, true);

    gridLineX0++;
    gridLineX1--;
    gridLineY0++;
    gridLineY1--;
    return (gridLineX0 <= xindex && xindex <= gridLineX1 && gridLineY0 <= yindex && yindex <= gridLineY1); // Boolean
  },

  removeObstacle : function(dir, index, limits, link, i)
  {
    // summary:
    //		Remove an obstacle that is currently coverd by the segment i of the
    //		input link. 
    // description:
    //		Remove an obstacle that is currently coverd by the segment i of the
    //		input link. 
    //		If dir is 0, the segment should be horizontal, otherwise vertical.
    //		The obstacle is a line segment at grid line with index, and the area
    //		of that grid line from limits[0] to limits[1] must be given free.
    // tags:
    //		private
    
    if (link.getSegmentDir(i) == dir)
    {
      var line = this.getGridLine(1 - dir, index);
      var seg;
      var notDone = true;

      if (line != null)
      {
        var min = Math.min(limits[0], limits[1]);
        var max = Math.max(limits[0], limits[1]);

        if (link.getSegments()[i] >= 0)
        {
          if (!link.startsAtObstacle(i))
          {
            seg = line.searchSegment(min - 1);

            if (seg != null)
            {
              seg.setEndIndex(max);
              notDone = false;
            }
          }

          if (notDone && !link.endsAtObstacle(i))
          {
            seg = line.searchSegment(max + 1);

            if (seg != null)
            {
              seg.setStartIndex(min);
              notDone = false;
            }
          }

          if (notDone) {
            line.addSegment(min, max);
          }
        }
        else
        {
          if (!link.endsAtObstacle(i))
          {
            seg = line.searchSegment(min - 1);

            if (seg != null)
            {
              seg.setEndIndex(max);
              notDone = false;
            }
          }

          if (notDone && !link.startsAtObstacle(i))
          {
            seg = line.searchSegment(max + 1);

            if (seg != null)
            {
              seg.setStartIndex(min);
              notDone = false;
            }
          }

          if (notDone) {
            line.addSegment(min, max);
          }
        }

        this.markDirty(line);
      }
    }
  },

  setRectObstacle : function(rect)
  {
    // summary:
    //		Store in the grid the information about a rectangular obstacle.  
    // tags:
    //		private
    
    var gridLineX0 = this.getGridIndexAtMinDist(0, rect.x, false);
    var gridLineX1 = this.getGridIndexAtMinDist(0, rect.x + rect.width, true);
    if (gridLineX0 >= gridLineX1) {
      return;
    }
    var gridLineY0 = this.getGridIndexAtMinDist(1, rect.y, false);
    var gridLineY1 = this.getGridIndexAtMinDist(1, rect.y + rect.height, true);
    if (gridLineY0 >= gridLineY1) {
      return;
    }
    gridLineX0++;
    gridLineX1--;
    gridLineY0++;
    gridLineY1--;
    if (gridLineX0 < 0) {
      gridLineX0 = 0;
    }
    if (gridLineY0 < 0) {
      gridLineY0 = 0;
    }
    if (gridLineX1 >= this._numberOfGridPoints[0]) {
      gridLineX1 = this._numberOfGridPoints[0] - 1;
    }
    if (gridLineY1 >= this._numberOfGridPoints[1]) {
      gridLineY1 = this._numberOfGridPoints[1] - 1;
    }
    this.storeRectObstacle(gridLineX0, gridLineX1, gridLineY0, gridLineY1);
  },

  setPointObstacle : function(point)
  {
    // summary:
    //		Store in the grid the information about a point obstacle.  
    // tags:
    //		private
    
    var gridLineX0;
    var gridLineX1;
    var gridLineY0;
    var gridLineY1;

    var coord = point.x;
    gridLineX0 = Math.floor(Math.ceil((coord - this.getMinCoord(0)) / this._gridDist[0]));

    while (coord - this.getGridCoord(0, gridLineX0) < this._minDist[0])
      gridLineX0--;
    gridLineX1 = gridLineX0;

    while (this.getGridCoord(0, gridLineX1) - coord < this._minDist[0])
      gridLineX1++;
    coord = point.y;
    gridLineY0 = Math.floor(Math.ceil((coord - this.getMinCoord(1)) / this._gridDist[1]));

    while (coord - this.getGridCoord(1, gridLineY0) < this._minDist[1])
      gridLineY0--;
    gridLineY1 = gridLineY0;

    while (this.getGridCoord(1, gridLineY1) - coord < this._minDist[1])
      gridLineY1++;

    gridLineX0++;
    gridLineX1--;
    gridLineY0++;
    gridLineY1--;
    if (gridLineX0 < 0) {
      gridLineX0 = 0;
    }
    if (gridLineY0 < 0) {
      gridLineY0 = 0;
    }
    if (gridLineX1 >= this._numberOfGridPoints[0]) {
      gridLineX1 = this._numberOfGridPoints[0] - 1;
    }
    if (gridLineY1 >= this._numberOfGridPoints[1]) {
      gridLineY1 = this._numberOfGridPoints[1] - 1;
    }
    this.storeRectObstacle(gridLineX0, gridLineX1, gridLineY0, gridLineY1);
  },

  storeRectObstacle : function(gridLineX0, gridLineX1, gridLineY0, gridLineY1)
  {
    // summary:
    //		Store a rectangular obstacle that reached from gridLineX0 to
    //		gridLineX1, and from gridLineY0 to gridLineY1. 
    // description:
    //		Store a rectangular obstacle that reached from gridLineX0 to
    //		gridLineX1, and from gridLineY0 to gridLineY1. 
    //		This is used for rectangles and points.
    // tags:
    //		private
    
    var i;

    if (gridLineY1 >= gridLineY0 - 1) {
      for (i = gridLineX0; i <= gridLineX1; i++)
        this._gridLinesX[i].setObstacle(this, gridLineY0, gridLineY1);
    }
    if (gridLineX1 >= gridLineX0 - 1) {
      for (i = gridLineY0; i <= gridLineY1; i++)
        this._gridLinesY[i].setObstacle(this, gridLineX0, gridLineX1);
    }
  },

  setObstacle : function(p1, p2)
  {
    // summary:
    //		Store in the grid the information about a line obstacle.
    // tags:
    //		private
    
    this.setPointObstacle(p1);
    if (p1.x == p2.x && p1.y == p2.y) {
      return;
    }
    this.setPointObstacle(p2);

    var isHorizontal = false;
    var isVertical = false;
    var dx = (p2.x - p1.x);
    var dy = (p2.y - p1.y);
    var adx = (dx > 0.0 ? dx : -dx);
    var ady = (dy > 0.0 ? dy : -dy);

    if (ady == 0.0) {
      isHorizontal = true;
    }
    else if (adx == 0.0) {
      isVertical = true;
    }
    else if (adx > ady)
    {
      var slopea = dy / dx;
      if (-0.2 < slopea && slopea < 0.2) {
        isHorizontal = true;
      }
    }
    else
    {
      var slopeb = dx / dy;
      if (-0.2 < slopeb && slopeb < 0.2) {
        isVertical = true;
      }
    }

    if (!isHorizontal && !isVertical) {
      return;
    }

    var gridLineX0;
    var gridLineX1;
    var gridLineY0;
    var gridLineY1;
    var coord = (dx > 0.0 ? p1.x : p2.x);
    gridLineX0 = Math.floor(Math.floor((coord - this.getMinCoord(0)) / this._gridDist[0]));

    while (coord - this.getGridCoord(0, gridLineX0) >= this._minDist[0])
      gridLineX0++;
    coord = (dx > 0.0 ? p2.x : p1.x);
    gridLineX1 = Math.floor(Math.ceil((coord - this.getMinCoord(0)) / this._gridDist[0]));

    while (this.getGridCoord(0, gridLineX1) - coord >= this._minDist[0])
      gridLineX1--;
    coord = (dy > 0.0 ? p1.y : p2.y);
    gridLineY0 = Math.floor(Math.floor((coord - this.getMinCoord(1)) / this._gridDist[1]));

    while (coord - this.getGridCoord(1, gridLineY0) >= this._minDist[1])
      gridLineY0++;
    coord = (dy > 0.0 ? p2.y : p1.y);
    gridLineY1 = Math.floor(Math.ceil((coord - this.getMinCoord(1)) / this._gridDist[1]));

    while (this.getGridCoord(1, gridLineY1) - coord >= this._minDist[1])
      gridLineY1--;
    if (gridLineX0 < 0) {
      gridLineX0 = 0;
    }
    if (gridLineY0 < 0) {
      gridLineY0 = 0;
    }
    if (gridLineX1 >= this._numberOfGridPoints[0]) {
      gridLineX1 = this._numberOfGridPoints[0] - 1;
    }
    if (gridLineY1 >= this._numberOfGridPoints[1]) {
      gridLineY1 = this._numberOfGridPoints[1] - 1;
    }

    var i;
    var j;

    for (i = gridLineX0; i <= gridLineX1; i++)
      for (j = gridLineY0; j <= gridLineY1; j++)
        if (this.checkGridPointForLine(i, j, p1, p2)) {
          if (isHorizontal) {
            this._gridLinesY[j].setObstacle(this, i, i);
          }
          else {
            this._gridLinesX[i].setObstacle(this, j, j);
          }
        }
  },

  setGridLineObstacle : function(isHorizontal, index, limits)
  {
    // summary:
    //		Store in the grid the information about a horizontal or vertical
    //		line obstacle that is given in grid index coordinates (rather than
    //		in real coordinates). 
    // description:
    //		Store in the grid the information about a horizontal or vertical
    //		line obstacle that is given in grid index coordinates (rather than
    //		in real coordinates). 
    //		The obstacle is on grid line given by index, and reaches from
    //		limits[0] to limits[1].
    // tags:
    //		private
    
    if (isHorizontal) {
      this._gridLinesY[index].setObstacle(this, limits[0], limits[1]);
    }
    else {
      this._gridLinesX[index].setObstacle(this, limits[0], limits[1]);
    }
  },

  checkGridPointForLine : function(i, j, p1, p2)
  {
    // summary:
    //		Check whether the grid point (i, j) is closer to line p1, p2 than
    //		minDist.
    // tags:
    //		private
    
    var x = this.getGridCoord(0, i);
    var y = this.getGridCoord(1, j);
    var minx;
    var maxx;
    var miny;
    var maxy;

    if (p1.x < p2.x)
    {
      minx = p1.x;
      maxx = p2.x;
    }
    else
    {
      minx = p2.x;
      maxx = p1.x;
    }

    if (p1.y < p2.y)
    {
      miny = p1.y;
      maxy = p2.y;
    }
    else
    {
      miny = p2.y;
      maxy = p1.y;
    }

    var dx = (p2.x - p1.x);
    var dy = (p2.y - p1.y);

    var fx;
    var fy;

    if (dx == 0.0)
    {
      fx = /*Number.MAX_VALUE*/(3.4028235E38);
      fy = p1.x;
    }
    else if (dy == 0.0)
    {
      fx = p1.y;
      fy = /*Number.MAX_VALUE*/(3.4028235E38);
    }
    else
    {
      fx = p1.y + (x - p1.x) * dy / dx;
      fy = p1.x + (y - p1.y) * dx / dy;
    }

    var d;

    if (minx <= fy && fy <= maxx)
    {
      d = (fy - x);
      if (d < 0.0) {
        d = -d;
      }
      if (d < this._minDist[0]) {
        return (true); // Boolean
      }
    }

    if (miny <= fx && fx <= maxy)
    {
      d = (fx - y);
      if (d < 0.0) {
        d = -d;
      }
      if (d < this._minDist[1]) {
        return (true); // Boolean
      }
    }

    return (false); // Boolean
  },

  ensureBorder : function(n)
  {
    // summary:
    //		Ensure that the grid has at least "n" empty grid lines at each
    //		border. 
    // description:
    //		Ensure that the grid has at least "n" empty grid lines at each
    //		border. 
    //		It returns true if something changed.
    // tags:
    //		private
    
    var maxNrGPX = this._numberOfGridPoints[0];
    var maxNrGPY = this._numberOfGridPoints[1];
    if (maxNrGPX == 0) {
      return (false); // Boolean
    }
    if (maxNrGPY == 0) {
      return (false); // Boolean
    }
    var newXGridLinesBefore = 0;
    var newXGridLinesAfter = 0;
    var newYGridLinesBefore = 0;
    var newYGridLinesAfter = 0;
    var i;

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPX || !this._gridLinesX[i].isFree(maxNrGPY - 1))
      {
        newXGridLinesBefore = n - i;
        break;
      }
    }

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPX || !this._gridLinesX[maxNrGPX - 1 - i].isFree(maxNrGPY - 1))
      {
        newXGridLinesAfter = n - i;
        break;
      }
    }

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPY || !this._gridLinesY[i].isFree(maxNrGPX - 1))
      {
        newYGridLinesBefore = n - i;
        break;
      }
    }

    for (i = 0; i < n; i++)
    {
      if (i >= maxNrGPY || !this._gridLinesY[maxNrGPY - 1 - i].isFree(maxNrGPX - 1))
      {
        newYGridLinesAfter = n - i;
        break;
      }
    }

    var newMaxNrGPX = maxNrGPX + newXGridLinesBefore + newXGridLinesAfter;
    var newMaxNrGPY = maxNrGPY + newYGridLinesBefore + newYGridLinesAfter;
    if (newMaxNrGPX == maxNrGPX && newMaxNrGPY == maxNrGPY) {
      return (false); // Boolean
    }
    var table;
    var seg;

    for (i = 0; i < maxNrGPX; i++)
      this._gridLinesX[i].shiftIndex(newXGridLinesBefore, newYGridLinesBefore);

    for (i = 0; i < maxNrGPY; i++)
      this._gridLinesY[i].shiftIndex(newYGridLinesBefore, newXGridLinesBefore);

    if (newXGridLinesBefore + newXGridLinesAfter != 0)
    {
      table = TranslateUtil.newArray(newMaxNrGPX, null);

      for (i = 0; i < newXGridLinesBefore; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPY - 1, i, false);
        this.markDirty(table[i]);
      }

      for (i = 0; i < maxNrGPX; i++)
        table[i + newXGridLinesBefore] = this._gridLinesX[i];

      for (i = maxNrGPX + newXGridLinesBefore; i < newMaxNrGPX; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPY - 1, i, false);
        this.markDirty(table[i]);
      }

      this._gridLinesX = table;
      this._numberOfGridPoints[0] = newMaxNrGPX;
      this._baseCoord[0] -= newXGridLinesBefore * this._gridDist[0];
    }

    if (newYGridLinesBefore + newYGridLinesAfter != 0)
    {
      table = TranslateUtil.newArray(newMaxNrGPY, null);

      for (i = 0; i < newYGridLinesBefore; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPX - 1, i, true);
        this.markDirty(table[i]);
      }

      for (i = 0; i < maxNrGPY; i++)
        table[i + newYGridLinesBefore] = this._gridLinesY[i];

      for (i = maxNrGPY + newYGridLinesBefore; i < newMaxNrGPY; i++)
      {
        table[i] = new LLGridLine(newMaxNrGPX - 1, i, true);
        this.markDirty(table[i]);
      }

      this._gridLinesY = table;
      this._numberOfGridPoints[1] = newMaxNrGPY;
      this._baseCoord[1] -= newYGridLinesBefore * this._gridDist[1];
    }

    if (newXGridLinesBefore != 0)
    {
      for (i = 0; i < newMaxNrGPY; i++)
      {
        seg = this._gridLinesY[i].searchSegment(newXGridLinesBefore);
        if (seg == null) {
          this._gridLinesY[i].addSegment(0, newXGridLinesBefore - 1);
        }
        else {
          seg.setStartIndex(0);
        }
      }
    }

    if (newXGridLinesAfter != 0)
    {
      for (i = 0; i < newMaxNrGPY; i++)
      {
        seg = this._gridLinesY[i].searchSegment(newMaxNrGPX - newXGridLinesAfter - 1);
        if (seg == null) {
          this._gridLinesY[i].addSegment(newMaxNrGPX - newXGridLinesAfter, newMaxNrGPX - 1);
        }
        else {
          seg.setEndIndex(newMaxNrGPX - 1);
        }
      }
    }

    if (newYGridLinesBefore != 0)
    {
      for (i = 0; i < newMaxNrGPX; i++)
      {
        seg = this._gridLinesX[i].searchSegment(newYGridLinesBefore);
        if (seg == null) {
          this._gridLinesX[i].addSegment(0, newYGridLinesBefore - 1);
        }
        else {
          seg.setStartIndex(0);
        }
      }
    }

    if (newYGridLinesAfter != 0)
    {
      for (i = 0; i < newMaxNrGPX; i++)
      {
        seg = this._gridLinesX[i].searchSegment(newMaxNrGPY - newYGridLinesAfter - 1);
        if (seg == null) {
          this._gridLinesX[i].addSegment(newMaxNrGPY - newYGridLinesAfter, newMaxNrGPY - 1);
        }
        else {
          seg.setEndIndex(newMaxNrGPY - 1);
        }
      }
    }

    return (true); // Boolean
  }

}
);

lang.mixin(LLGrid,
{

  getMinCoord : function(rect, dir)
  {
    return (dir == 0 ? rect.x : rect.y); // Number
  },

  getMaxCoord : function(rect, dir)
  {
    return (dir == 0 ? rect.x + rect.width : rect.y + rect.height); // Number
  },

  getSize : function(rect, dir)
  {
    return (dir == 0 ? rect.width : rect.height); // Number
  }

}
);

return LLGrid;
});

},
'ibm_ilog/graphlayout/shortlink/_OrthoWith4BendsPositionArrays':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_OrthoWith4BendsPositionArrays", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_LinkShapeType"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _LinkShapeType	// #7
    ) {

var _OrthoWith4BendsPositionArrays = declare("ibm_ilog.graphlayout.shortlink._OrthoWith4BendsPositionArrays", null, 
{
  // summary:
  //		Contains the static arrays which define the candidate orthogonal
  //		shapes according to the relative position of the end-nodes. 
  // description:
  //		Contains the static arrays which define the candidate orthogonal
  //		shapes according to the relative position of the end-nodes. 
  //		<p>
  //		This is the version with 4-bend shapes (to be used when either a side
  //		filter is specified or the link has fixed connection points).  
  //		</p>
  // tags:
  //		private
  
  constructor : function()
  {
    
  }

}
);

lang.mixin(_OrthoWith4BendsPositionArrays,
{
  FROM_BL_TO_TR_POSITION_ARRAY : null,
  FROM_BR_TO_TL_POSITION_ARRAY : null,
  FROM_TL_TO_BR_POSITION_ARRAY : null,
  FROM_TR_TO_BL_POSITION_ARRAY : null,
  FROM_L_TO_R_POSITION_ARRAY : null,
  FROM_R_TO_L_POSITION_ARRAY : null,
  FROM_B_TO_T_POSITION_ARRAY : null,
  FROM_T_TO_B_POSITION_ARRAY : null,
  _fields_init : function(c)
  {
    c = c || _OrthoWith4BendsPositionArrays;
    
    c.FROM_BL_TO_TR_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.FourBendLRmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.FourBendBTmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.FourBendLRmiddle.instance,
        _LinkShapeType.FourBendBTmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ]
     ];
    c.FROM_BR_TO_TL_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.FourBendBTmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.FourBendRLmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.FourBendRLmiddle.instance,
        _LinkShapeType.FourBendBTmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ]
     ];
    c.FROM_TL_TO_BR_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.FourBendTBmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.FourBendLRmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.FourBendLRmiddle.instance,
        _LinkShapeType.FourBendTBmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ]
     ];
    c.FROM_TR_TO_BL_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.FourBendTBmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.FourBendRLmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.FourBendRLmiddle.instance,
        _LinkShapeType.FourBendTBmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ]
     ];
    c.FROM_L_TO_R_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.FourBendTBmiddle.instance,
        _LinkShapeType.FourBendBTmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ]
     ];
    c.FROM_R_TO_L_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.FourBendTBmiddle.instance,
        _LinkShapeType.FourBendBTmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance
       ],
      
      [
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance
       ]
     ];
    c.FROM_B_TO_T_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.FourBendRLmiddle.instance,
        _LinkShapeType.FourBendLRmiddle.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.FourBendBTright.instance,
        _LinkShapeType.FourBendBTleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ]
     ];
    c.FROM_T_TO_B_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.FourBendRLmiddle.instance,
        _LinkShapeType.FourBendLRmiddle.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ],
      
      [
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.FourBendTBright.instance,
        _LinkShapeType.FourBendTBleft.instance,
        _LinkShapeType.FourBendRLtop.instance,
        _LinkShapeType.FourBendRLbottom.instance,
        _LinkShapeType.FourBendLRtop.instance,
        _LinkShapeType.FourBendLRbottom.instance
       ]
     ];
  }
  
}
);

(function(){ _OrthoWith4BendsPositionArrays._fields_init(); })();

return _OrthoWith4BendsPositionArrays;
});

},
'ibm_ilog/graphlayout/shortlink/_SLNodeOrLinkData':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_SLNodeOrLinkData", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Rectangle",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Rectangle,	// #6
    TranslateUtil	// #7
    ) {

var _SLNodeOrLinkData = declare("ibm_ilog.graphlayout.shortlink._SLNodeOrLinkData", null, 
{
  // summary:
  //		The base class of the data structures for nodes and links. 
  // description:
  //		The base class of the data structures for nodes and links. 
  //		An instance of one of the subclasses is (temporarily) attached to
  //		each original node and link.   
  // tags:
  //		extension
  
  boundingBox : null,

  nodeOrLink: null,

  constructor : function(nodeOrLink)
  {
    this.boundingBox = new Rectangle();
    this.nodeOrLink = nodeOrLink;
  },
  
  isIntersectingObjectValid : function(linkData)
  {
    // summary:
    //		Returns true if the intersection between this NodeOrLinkData and a
    //		LinkData has to be taken into consideration. 
    // description:
    //		Returns true if the intersection between this NodeOrLinkData and a
    //		LinkData has to be taken into consideration. 
    //		Returns  false otherwise. This method is used to avoid unnecessary
    //		crossing tests.
    // tags:
    //		extension
    },
  
  isNode : function()
  {
    // summary:
    //		Returns true if this is a node, false otherwise.
    // tags:
    //		extension
    },
  
  isLink : function()
  {
    // summary:
    //		Returns true if this is a link, false otherwise.
    // tags:
    //		extension
    }
  
}
);

return _SLNodeOrLinkData;
});

},
'ibm_ilog/graphlayout/longlink/LLFallbackRouteAlgorithm':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLFallbackRouteAlgorithm", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Point",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../longlink/LLGrid",	// #8
  "../longlink/LLTerminationCandAlgorithm",	// #9
  "../longlink/LLTerminationPoint",	// #10
  "../Direction"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Point,	// #6
    TranslateUtil,	// #7
    LLGrid,	// #8
    LLTerminationCandAlgorithm,	// #9
    LLTerminationPoint,	// #10
    Direction	// #11
    ) {

var LLFallbackRouteAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLFallbackRouteAlgorithm", null, 
{
  // summary:
  //		The algorithm to calculate a fallback solution for one link if all
  //		other tries failed. 
  // description:
  //		The algorithm to calculate a fallback solution for one link if all
  //		other tries failed. 
  //		The fallback solution ignores crossings and overlapping nodes. It is
  //		a very simple (and, in the context of surrounding links and nodes,
  //		probably ugly) route. It however fits on the grid.  
  // tags:
  //		private
  
  _layout: null,

  _grid: null,

  _points: null,

  constructor : function(layout, grid)
  {
    this._layout = layout;
    this._grid = grid;
    this._points = new Array();
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._points = null;

    this._layout = null;
    this._grid = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getPoints : function()
  {
    // summary:
    //		Returns the calculated points.
    // tags:
    //		private
    
    return (this._points); // Array
  },

  getGrid : function()
  {
    // summary:
    //		Returns the routing grid.
    // tags:
    //		private
    
    return (this._grid); // ibm_ilog.graphlayout.longlink.LLGrid
  },

  route : function(link)
  {
    // summary:
    //		Route a link from its start node to its end node. 
    // description:
    //		Route a link from its start node to its end node. 
    //		This is a simple route that ignores overlappings and crossings. It
    //		returns true on success.
    // tags:
    //		private
    
    var layout = this.getLayout();

    link.cleanRoute();
    this._points.length = 0;

    var startPointCand = link.getStartPoints();
    var endPointCand = link.getEndPoints();
    var numberStartPointCands = startPointCand.length;
    var numberEndPointCands = endPointCand.length;
    var startRect = link.getFromRect();
    var endRect = link.getToRect();
    var startPref = TranslateUtil.newArray(4, 0);
    var endPref = TranslateUtil.newArray(4, 0);
    var minStartLen = layout.getMinStartSegmentLength();
    var minEndLen = layout.getMinEndSegmentLength();
    var grid = this.getGrid();

    if (numberStartPointCands <= 0 || numberEndPointCands <= 0)
    {
      startPref[0] = startPref[1] = startPref[2] = startPref[3] = 0;
      endPref[0] = endPref[1] = endPref[2] = endPref[3] = 0;
      LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.x - minStartLen, startRect.x + startRect.width + minStartLen, endRect.x - minEndLen, endRect.x + endRect.width + minEndLen, 1, 3,
          startPref, endPref);
      LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.y - minStartLen, startRect.y + startRect.height + minStartLen, endRect.y - minEndLen, endRect.y + endRect.height + minEndLen, 0, 2,
          startPref, endPref);
    }

    var bestPref = -1;
    var bestIndex = 0;

    if (numberStartPointCands <= 0)
    {
      startPointCand = TranslateUtil.newArray(1, null);

      for (var ix = 0; ix < 4; ix++)
      {
        if (startPref[ix] > bestPref)
        {
          bestPref = startPref[ix];
          bestIndex = ix;
        }
      }

      switch (bestIndex)
      {
        case 0:
          startPointCand[0] =
            new LLTerminationPoint(grid.getGridIndex(0, startRect.x + 0.5 * startRect.width, true), grid.getGridIndex(1, startRect.y - minStartLen, false), 0, 0, 0, 0, 0, Direction.TOP, null);
          break;
        case 1:
          startPointCand[0] =
            new LLTerminationPoint(grid.getGridIndex(0, startRect.x - minStartLen, false), grid.getGridIndex(1, startRect.y + 0.5 * startRect.width, true), 0, 0, 0, 0, 0, Direction.LEFT, null);
          break;
        case 2:
          startPointCand[0] =
            new LLTerminationPoint(grid.getGridIndex(0, startRect.x + startRect.width + minStartLen, true), grid.getGridIndex(1, startRect.y + 0.5 * startRect.width, true), 0, 0, 0, 0, 0,
                Direction.RIGHT, null);
          break;
        default:
          startPointCand[0] =
            new LLTerminationPoint(grid.getGridIndex(0, startRect.x + 0.5 * startRect.width, true), grid.getGridIndex(1, startRect.y + startRect.width + minStartLen, true), 0, 0, 0, 0, 0,
                Direction.BOTTOM, null);
          break;
      }
      link.setStartPoints(startPointCand, 1);
      startPointCand = link.getStartPoints();
      numberStartPointCands = startPointCand.length;
    }

    if (numberEndPointCands <= 0)
    {
      endPointCand = TranslateUtil.newArray(1, null);

      bestPref = -1;
      bestIndex = 0;

      for (var i = 0; i < 4; i++)
      {
        if (endPref[i] > bestPref)
        {
          bestPref = endPref[i];
          bestIndex = i;
        }
      }

      switch (bestIndex)
      {
        case 0:
          endPointCand[0] = new LLTerminationPoint(grid.getGridIndex(0, endRect.x + 0.5 * endRect.width, true), grid.getGridIndex(1, endRect.y - minEndLen, false), 0, 0, 0, 0, 0, Direction.TOP, null);
          break;
        case 1:
          endPointCand[0] = new LLTerminationPoint(grid.getGridIndex(0, endRect.x - minEndLen, false), grid.getGridIndex(1, endRect.y + 0.5 * endRect.width, true), 0, 0, 0, 0, 0, Direction.LEFT, null);
          break;
        case 2:
          endPointCand[0] =
            new LLTerminationPoint(grid.getGridIndex(0, endRect.x + endRect.width + minEndLen, true), grid.getGridIndex(1, endRect.y + 0.5 * endRect.width, true), 0, 0, 0, 0, 0, Direction.RIGHT, null);
          break;
        default:
          endPointCand[0] =
            new LLTerminationPoint(grid.getGridIndex(0, endRect.x + 0.5 * endRect.width, true), grid.getGridIndex(1, endRect.y + endRect.width + minEndLen, true), 0, 0, 0, 0, 0, Direction.BOTTOM, null);
          break;
      }
      link.setEndPoints(endPointCand, 1);
      endPointCand = link.getEndPoints();
      numberEndPointCands = endPointCand.length;
    }

    var startPoint = link.getStartPoints()[0];
    var endPoint = link.getEndPoints()[0];
    link.setActStartAndEndCandidates(0, 0);

    if (link.isSelfLoop() && startPoint.getIndex(0) == endPoint.getIndex(0) && startPoint.getIndex(1) == endPoint.getIndex(1))
    {
      if (numberEndPointCands >= 2)
      {
        endPoint = link.getEndPoints()[1];
        link.setActStartAndEndCandidates(0, 1);
      }
      else if (numberStartPointCands >= 2)
      {
        startPoint = link.getStartPoints()[1];
        link.setActStartAndEndCandidates(1, 0);
      }
    }

    switch (startPoint.getSide())
    {
      case Direction.TOP:
        switch (endPoint.getSide())
        {
          case Direction.TOP:
            this.routeSameSide(link, startPoint, endPoint, 1, false);
            break;
          case Direction.BOTTOM:
            this.routeOppositeSide(link, startPoint, endPoint, 1, false);
            break;
          case Direction.LEFT:
            this.routeAdjacentSide(link, startPoint, endPoint, 1, false, false);
            break;
          case Direction.RIGHT:
            this.routeAdjacentSide(link, startPoint, endPoint, 1, false, true);
            break;
        }
        break;
      case Direction.BOTTOM:
        switch (endPoint.getSide())
        {
          case Direction.TOP:
            this.routeOppositeSide(link, startPoint, endPoint, 1, true);
            break;
          case Direction.BOTTOM:
            this.routeSameSide(link, startPoint, endPoint, 1, true);
            break;
          case Direction.LEFT:
            this.routeAdjacentSide(link, startPoint, endPoint, 1, true, false);
            break;
          case Direction.RIGHT:
            this.routeAdjacentSide(link, startPoint, endPoint, 1, true, true);
            break;
        }
        break;
      case Direction.LEFT:
        switch (endPoint.getSide())
        {
          case Direction.TOP:
            this.routeAdjacentSide(link, startPoint, endPoint, 0, false, false);
            break;
          case Direction.BOTTOM:
            this.routeAdjacentSide(link, startPoint, endPoint, 0, false, true);
            break;
          case Direction.LEFT:
            this.routeSameSide(link, startPoint, endPoint, 0, false);
            break;
          case Direction.RIGHT:
            this.routeOppositeSide(link, startPoint, endPoint, 0, false);
            break;
        }
        break;
      case Direction.RIGHT:
        switch (endPoint.getSide())
        {
          case Direction.TOP:
            this.routeAdjacentSide(link, startPoint, endPoint, 0, true, false);
            break;
          case Direction.BOTTOM:
            this.routeAdjacentSide(link, startPoint, endPoint, 0, true, true);
            break;
          case Direction.LEFT:
            this.routeOppositeSide(link, startPoint, endPoint, 0, true);
            break;
          case Direction.RIGHT:
            this.routeSameSide(link, startPoint, endPoint, 0, true);
            break;
        }
        break;
    }
    link.storeRoute(this.getPoints());
    return (true); // Boolean
  },

  routeSameSide : function(link, startPoint, endPoint, dir, largerSide)
  {
    // summary:
    //		Route between same sides (top to top, bottom to bottom, etc). 
    // description:
    //		Route between same sides (top to top, bottom to bottom, etc). 
    //		dir is 0, if it is the left or right side, and 1 if it is top or
    //		bottom side. The flag largerSide is true if it is the side at the
    //		larger coordinates (i.e. the bottom or right side), and false if it
    //		is the side at the smaller coordinates (the left or top side).
    // tags:
    //		private
    
    var grid = this.getGrid();
    var odir = 1 - dir;

    var startIndex0 = startPoint.getIndex(odir);
    var startIndex1 = startPoint.getIndex(dir);
    var endIndex0 = endPoint.getIndex(odir);
    var endIndex1 = endPoint.getIndex(dir);

    var fromRect = link.getFromRect();
    var toRect = link.getToRect();
    var startMinIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMinCoord(fromRect, odir), false);
    var startMaxIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMaxCoord(fromRect, odir), true);
    var endMinIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMinCoord(toRect, odir), false);
    var endMaxIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMaxCoord(toRect, odir), true);

    var tSideIndex;
    if (largerSide) {
      tSideIndex = Math.max(grid.getGridIndexAtMinDist(dir, LLGrid.getMaxCoord(fromRect, dir), true), grid.getGridIndexAtMinDist(dir, LLGrid.getMaxCoord(toRect, dir), true));
    }
    else {
      tSideIndex = Math.min(grid.getGridIndexAtMinDist(dir, LLGrid.getMinCoord(fromRect, dir), false), grid.getGridIndexAtMinDist(dir, LLGrid.getMinCoord(toRect, dir), false));
    }
    if (
      link.isSelfLoop() || startIndex1 == endIndex1 || (largerSide && startIndex1 >= tSideIndex && endIndex1 >= tSideIndex) || (!largerSide && startIndex1 <= tSideIndex && endIndex1 <= tSideIndex) ||
        ((startIndex0 <= endMinIndex || startIndex0 >= endMaxIndex) && (endIndex0 <= startMinIndex || endIndex0 >= startMaxIndex)))
      {
        var connectionLevel;
        if (largerSide) {
          connectionLevel = Math.max(startIndex1, endIndex1);
        }
        else {
          connectionLevel = Math.min(startIndex1, endIndex1);
        }
        this.addPoint(odir, endIndex0, connectionLevel);
        this.addPoint(odir, startIndex0, connectionLevel);
      }
    else
    {
      var bypassLevel;

      if ((largerSide && startIndex1 > endIndex1) || (!largerSide && startIndex1 < endIndex1))
      {
        if (endIndex0 > Math.floor((startMinIndex + startMaxIndex) / 2)) {
          bypassLevel = startMaxIndex;
        }
        else {
          bypassLevel = startMinIndex;
        }
      }
      else
      {
        if (startIndex0 > Math.floor((endMinIndex + endMaxIndex) / 2)) {
          bypassLevel = endMaxIndex;
        }
        else {
          bypassLevel = endMinIndex;
        }
      }

      this.addPoint(odir, endIndex0, endIndex1);
      this.addPoint(odir, bypassLevel, endIndex1);
      this.addPoint(odir, bypassLevel, startIndex1);
      this.addPoint(odir, startIndex0, startIndex1);
    }
  },

  routeOppositeSide : function(link, startPoint, endPoint, dir, startLargerSide)
  {
    // summary:
    //		Route between opposite sides (top to bottom, left to right, etc). 
    // description:
    //		Route between opposite sides (top to bottom, left to right, etc). 
    //		dir is 0, if it is the left or right side, and 1 if it is top or
    //		bottom side. The flag startLargerSide is true if the link starts at
    //		the side at the larger coordinates (i.e. the bottom or right side),
    //		which means that it ends at the side at the smaller coordinates
    //		(i.e., the top or left side). The flag is false if the link starts
    //		at the side at the smaller coordinates (the left or top side) and
    //		ends at the side at the larger coordinates (the bottom or right
    //		side).
    // tags:
    //		private
    
    var grid = this.getGrid();
    var odir = 1 - dir;

    var startIndex0 = startPoint.getIndex(odir);
    var startIndex1 = startPoint.getIndex(dir);
    var endIndex0 = endPoint.getIndex(odir);
    var endIndex1 = endPoint.getIndex(dir);

    var fromRect = link.getFromRect();
    var toRect = link.getToRect();
    var startMinBorder = grid.getGridIndexAtMinDist(dir, LLGrid.getMinCoord(fromRect, dir), false);
    var startMaxBorder = grid.getGridIndexAtMinDist(dir, LLGrid.getMaxCoord(fromRect, dir), true);
    var endMinBorder = grid.getGridIndexAtMinDist(dir, LLGrid.getMinCoord(toRect, dir), false);
    var endMaxBorder = grid.getGridIndexAtMinDist(dir, LLGrid.getMaxCoord(toRect, dir), true);

    if (startIndex0 == endIndex0 && ((startLargerSide && startMaxBorder <= endMinBorder + 1) || (!startLargerSide && startMinBorder >= endMaxBorder - 1)))
    {
      return;
    }
    else if ((startLargerSide && startIndex1 <= endIndex1) || (!startLargerSide && startIndex1 >= endIndex1))
    {
      if (startIndex0 == endIndex0)
      {
        return;
      }
      else
      {
        var middlep = Math.floor((startIndex1 + endIndex1) / 2);
        this.addPoint(odir, endIndex0, middlep);
        this.addPoint(odir, startIndex0, middlep);
      }
    }
    else
    {
      var startMinIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMinCoord(fromRect, odir), false);
      var startMaxIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMaxCoord(fromRect, odir), true);
      var endMinIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMinCoord(toRect, odir), false);
      var endMaxIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMaxCoord(toRect, odir), true);

      var middle = Math.floor((Math.min(startMaxIndex, endMaxIndex) + Math.max(startMinIndex, endMinIndex)) / 2);

      if (middle >= Math.min(startMaxIndex, endMaxIndex))
      {
        this.addPoint(odir, endIndex0, endIndex1);
        this.addPoint(odir, middle, endIndex1);
        this.addPoint(odir, middle, startIndex1);
        this.addPoint(odir, startIndex0, startIndex1);
      }
      else
      {
        var startLevel = startIndex1;
        var endLevel = endIndex1;
        var bypassLevel;
        var bypassLevel1 = Math.max(startMaxIndex, endMaxIndex);
        var bypassLevel2 = Math.min(startMinIndex, endMinIndex);

        if (2 * bypassLevel1 - startIndex0 - endIndex0 < startIndex0 + endIndex0 - 2 * bypassLevel2)
        {
          bypassLevel = bypassLevel1;

          if (startMaxIndex < endMaxIndex)
          {
            if (startLargerSide && endMaxBorder > startLevel) {
              startLevel = endMaxBorder;
            }
            else if (!startLargerSide && endMinBorder < startLevel) {
              startLevel = endMinBorder;
            }
          }
          else
          {
            if (!startLargerSide && startMaxBorder > endLevel) {
              endLevel = startMaxBorder;
            }
            else if (startLargerSide && startMinBorder < endLevel) {
              endLevel = startMinBorder;
            }
          }
        }
        else
        {
          bypassLevel = bypassLevel2;

          if (startMaxIndex > endMaxIndex)
          {
            if (startLargerSide && endMaxBorder > startLevel) {
              startLevel = endMaxBorder;
            }
            else if (!startLargerSide && endMinBorder < startLevel) {
              startLevel = endMinBorder;
            }
          }
          else
          {
            if (!startLargerSide && startMaxBorder > endLevel) {
              endLevel = startMaxBorder;
            }
            else if (startLargerSide && startMinBorder < endLevel) {
              endLevel = startMinBorder;
            }
          }
        }

        this.addPoint(odir, endIndex0, endLevel);
        this.addPoint(odir, bypassLevel, endLevel);
        this.addPoint(odir, bypassLevel, startLevel);
        this.addPoint(odir, startIndex0, startLevel);
      }
    }
  },

  routeAdjacentSide : function(link, startPoint, endPoint, dir, startLargerSide, endLargerSide)
  {
    // summary:
    //		Route between adjacent sides (top to right, right to bottom, etc). 
    // description:
    //		Route between adjacent sides (top to right, right to bottom, etc). 
    //		dir is 0, if the start point is at the left or right side and the
    //		end point is at top or bottom side, and 1 if the start point is at
    //		top or bottom side and the end point is at left or right side. The
    //		flag startLargerSide is true if the link starts at the side at the
    //		larger coordinates (i.e. the bottom or right side). It is false if
    //		the link starts at the side at the smaller coordinates (the left or
    //		top side). Similar with the flag endLargerSide and the end point.
    // tags:
    //		private
    
    var grid = this.getGrid();
    var odir = 1 - dir;

    var startIndex0 = startPoint.getIndex(odir);
    var startIndex1 = startPoint.getIndex(dir);
    var endIndex0 = endPoint.getIndex(odir);
    var endIndex1 = endPoint.getIndex(dir);

    if (
      ((startLargerSide && endIndex1 >= startIndex1) || (!startLargerSide && endIndex1 <= startIndex1)) && ((endLargerSide && startIndex0 >= endIndex0) || (!endLargerSide && startIndex0 <= endIndex0)))
      {
        this.addPoint(odir, startIndex0, endIndex1);
      }
    else
    {
      var middlePoint0 = endIndex0;
      var middlePoint1 = startIndex1;

      var fromRect = link.getFromRect();
      var startMinIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMinCoord(fromRect, odir), false);
      var startMaxIndex = grid.getGridIndexAtMinDist(odir, LLGrid.getMaxCoord(fromRect, odir), true);

      var testLevel;

      if (endIndex0 > startMinIndex && endIndex0 < startMaxIndex)
      {
        if (startLargerSide)
        {
          testLevel = grid.getGridIndexAtMinDist(dir, LLGrid.getMaxCoord(fromRect, dir), true);

          if (endIndex1 <= testLevel)
          {
            if (endLargerSide) {
              middlePoint0 = startMaxIndex;
            }
            else {
              middlePoint0 = startMinIndex;
            }
          }
        }
        else
        {
          testLevel = grid.getGridIndexAtMinDist(dir, LLGrid.getMinCoord(fromRect, dir), false);

          if (endIndex1 >= testLevel)
          {
            if (endLargerSide) {
              middlePoint0 = startMaxIndex;
            }
            else {
              middlePoint0 = startMinIndex;
            }
          }
        }
      }

      var toRect = link.getToRect();
      var endMinIndex = grid.getGridIndexAtMinDist(dir, LLGrid.getMinCoord(toRect, dir), false);
      var endMaxIndex = grid.getGridIndexAtMinDist(dir, LLGrid.getMaxCoord(toRect, dir), true);

      if (startIndex1 > endMinIndex && startIndex1 < endMaxIndex)
      {
        if (endLargerSide)
        {
          testLevel = grid.getGridIndexAtMinDist(odir, LLGrid.getMaxCoord(toRect, odir), true);

          if (startIndex0 <= testLevel)
          {
            if (startLargerSide) {
              middlePoint1 = endMaxIndex;
            }
            else {
              middlePoint1 = endMinIndex;
            }
          }
        }
        else
        {
          testLevel = grid.getGridIndexAtMinDist(odir, LLGrid.getMinCoord(toRect, odir), false);

          if (startIndex0 >= testLevel)
          {
            if (startLargerSide) {
              middlePoint1 = endMaxIndex;
            }
            else {
              middlePoint1 = endMinIndex;
            }
          }
        }
      }

      this.addPoint(odir, middlePoint0, endIndex1);
      this.addPoint(odir, middlePoint0, middlePoint1);
      this.addPoint(odir, startIndex0, middlePoint1);
    }
  },

  addPoint : function(dir, indexInDir, otherIndex)
  {
    // summary:
    //		Adds a point to the list of points. 
    // description:
    //		Adds a point to the list of points. 
    //		Note that the points must be added in reverse order, because
    //		LLLink.storeRoute expects it that way.
    // tags:
    //		private
    
    if (dir == 0) {
      this._points.push(new Point(indexInDir, otherIndex));
    }
    else {
      this._points.push(new Point(otherIndex, indexInDir));
    }
  }

}
);

return LLFallbackRouteAlgorithm;
});

},
'ibm_ilog/graphlayout/shortlink/_SLNodeSide':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_SLNodeSide", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/LogResUtil",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../Direction"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    LogResUtil,	// #7
    TranslateUtil,	// #8
    Direction	// #9
    ) {

var _SLNodeSide = declare("ibm_ilog.graphlayout.shortlink._SLNodeSide", null, 
{
  // summary:
  //		A NodeSide, by its subclasses, represents each side of a
  //		(rectangular) node: top, bottom etc. 
  // description:
  //		A NodeSide, by its subclasses, represents each side of a
  //		(rectangular) node: top, bottom etc. 
  //		It contains methods for laying out the connection points of the links
  //		in the appropriate order (to avoid unnecessary crossings).  
  //		<p>
  //		In JViews
  //		<= 5.0, one layout instance used exactly one instance for each node side. Starting with JViews >=
  //		5.5, we use an instance for each node. A performance analysis showed
  //		that this allows a speed-up, because it avoids the need to rebuild on
  //		the fly the list of links contained in each node side.  
  // tags:
  //		extension
  
  _vectLinksOrig: null,
  _vectLinksDest: null,
  _vectNonFixedLinks: null,

  _vectFixedLinks: null,

  _isReversedDirection : null,
  _directionFlag: null,
  _layoutUpToDate : null,

  constructor : function(reversedDirection )
  {
    this._isReversedDirection = false;
    this._layoutUpToDate = false;
    this._isReversedDirection = reversedDirection;
  },
  
  getSide : function()
  {
    // summary:
    //		Returns an int that represents the side.
    // tags:
    //		extension
    },
  
  getOppositeSide : function()
  {
    // summary:
    //		Returns an int that represents the opposite side.
    // tags:
    //		extension
    },
  
  getDirection : function()
  {
    // summary:
    //		Returns an int that represents the direction of the flow of links
    //		connected to this node side. 
    // description:
    //		Returns an int that represents the direction of the flow of links
    //		connected to this node side. 
    //		This is generally the same as getNodeSide, except for
    //		"quasi-self-intergraph links", which have the reversed direction.
    // tags:
    //		private
    
    return (this._isReversedDirection ? this.getOppositeSide() : this.getSide()); // int
  },

  getOppositeDirection : function()
  {
    // summary:
    //		Returns an int that represents the opposite direction.
    // tags:
    //		private
    
    return (this._isReversedDirection ? this.getSide() : this.getOppositeSide()); // int
  },

  moveToDefaultConnectionPoint : function(point, rect)
  {
    // summary:
    //		Moves a point to the default connection point of the node side. 
    // description:
    //		Moves a point to the default connection point of the node side. 
    //		By default, it is the middle of the link connection rect.  
    // point:
    //		the connection point
    // rect:
    //		the link connection rect of the node on which the link is
    //		connected
    },
  
  clean : function()
  {
    // summary:
    //		Cleans this node side object.
    // tags:
    //		private
    
    if (this._vectLinksOrig != null) {
      this._vectLinksOrig.length = 0;
    }
    if (this._vectLinksDest != null) {
      this._vectLinksDest.length = 0;
    }
    if (this._vectFixedLinks != null) {
      this._vectFixedLinks.length = 0;
    }
    if (this._vectNonFixedLinks != null) {
      this._vectNonFixedLinks.length = 0;
    }
    this._layoutUpToDate = false;
  },

  setLayoutUpToDate : function(uptodate)
  {
    this._layoutUpToDate = uptodate;
  },

  addLink : function(linkData, origin)
  {
    if (linkData.isFixed() || (origin && linkData.isFromPointFixed()) || (!origin && linkData.isToPointFixed()))
    {
      if (this._vectFixedLinks == null) {
        this._vectFixedLinks = new Array();
      }
      this._vectFixedLinks.push(linkData);
    }
    else
    {
      if (this._vectNonFixedLinks == null) {
        this._vectNonFixedLinks = new Array();
      }
      this._vectNonFixedLinks.push(linkData);
    }

    this._layoutUpToDate = false;
  },

  removeLink : function(linkData, origin)
  {
    var removed = false;

    if (linkData.isFixed() || (origin && linkData.isFromPointFixed()) || (!origin && linkData.isToPointFixed()))
    {
      if (this._vectFixedLinks != null) {
        removed = TranslateUtil.removeArrayObject(this._vectFixedLinks, linkData);
      }
    }
    else
    {
      if (this._vectNonFixedLinks != null) {
        removed = TranslateUtil.removeArrayObject(this._vectNonFixedLinks, linkData);
      }
    }

    this._layoutUpToDate = false;
    return removed; // Boolean
  },

  isGoingTowardOrigin : function(linkData, origin)
  {
    var point1 = linkData.getPointAt(origin, 1);
    var point2 = linkData.getPointAt(origin, 2);
    var tangentDelta = this.getTangentDelta(point1, point2);

    if (tangentDelta > 0) {
      return true; // Boolean
    }
    else if (tangentDelta < 0) {
      return false; // Boolean
    }
    else
    {
      this._directionFlag = !this._directionFlag;
      return this._directionFlag; // Boolean
    }
  },

  layout : function(layout, sideLayouter, linkSorter, linksRefiner, connectionBoxInterface, nodeData, linkOffset, minFinalSegmentLength, bundleMode)
  {
    // summary:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    // description:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    //		The side of the node is determined by the actual subclass of of
    //		NodeSide on which we call the method.
    // tags:
    //		private
    
    if (!this._layoutUpToDate)
    {
      this.fillVectLinks();
      sideLayouter.layout(layout, linkSorter, linksRefiner, connectionBoxInterface, nodeData, this, this._vectLinksOrig, this._vectLinksDest, this._vectFixedLinks, linkOffset, minFinalSegmentLength,
          bundleMode);
      this._layoutUpToDate = true;
    }
  },

  refine : function(layout, sideLayouter, linksRefiner, nodeData, linkOffset, minFinalSegmentLength, bundleMode)
  {
    // summary:
    //		Refines.
    // tags:
    //		private
    
    sideLayouter.refine(linksRefiner, this._vectLinksOrig, this._vectLinksDest, this._vectFixedLinks, nodeData, this, linkOffset, minFinalSegmentLength, bundleMode);
  },

  fillVectLinks : function()
  {
    // summary:
    //		
    // tags:
    //		private
    
    if (this._vectLinksOrig != null) {
      this._vectLinksOrig.length = 0;
    }
    if (this._vectLinksDest != null) {
      this._vectLinksDest.length = 0;
    }
    if (this._vectNonFixedLinks == null) {
      return;
    }
    var nLinks = this._vectNonFixedLinks.length;

    for (var i = 0; i < nLinks; i++)
      this.fillVectLinksOrigDest(this._vectNonFixedLinks[i], nLinks);
  },

  fillVectLinksOrigDest : function(linkData, nLinks)
  {
    // summary:
    //		
    // nLinks: int
    //		the total number of non fixed links. 
    //		description:
    // nLinks: int
    //		the total number of non fixed links. 
    //		We use it to allocate
    //		the size of _vectLinksOrig/Dest.
    // tags:
    //		private
    
    var origin = linkData.isOrigin(this);

    if (linkData.isSameSideSelfLink() || this.isGoingTowardOrigin(linkData, origin))
    {
      if (this._vectLinksOrig == null) {
        this._vectLinksOrig = new Array();
      }
      this._vectLinksOrig.push(linkData);
    }
    else
    {
      if (this._vectLinksDest == null) {
        this._vectLinksDest = new Array();
      }
      this._vectLinksDest.push(linkData);
    }

    this._layoutUpToDate = false;
  },

  computeConnectionPoint : function(linkConnectionRect, linkData, origin, tangentialDistForConnectionPoint)
  {
    var shape = linkData.getLinkShape();
    var connectionPoint = (origin ? shape.getFrom() : shape.getTo());
    var tanCoordOfConnectionPoint = this.getMinTangentialCoord(linkConnectionRect) + tangentialDistForConnectionPoint;
    var radCoordOfConnectionPoint = this.getRectRadialCoord(linkConnectionRect);
    this.setTangentialCoord(connectionPoint, tanCoordOfConnectionPoint);
    this.setRadialCoord(connectionPoint, radCoordOfConnectionPoint);

    var nBends = shape.getNumberOfPoints() - 2;

    if (nBends >= 1)
    {
      var adjacentPoint = shape.getPointAtFromEndPoint(origin, 1);

      this.setTangentialCoord(adjacentPoint, tanCoordOfConnectionPoint);
    }
  },

  compareOrthogonalLinks : function(layout, link1, link2, origin1, origin2)
  {
    // summary:
    //		Decides about the order of the connection of two links on this side.
    //		
    // description:
    //		Decides about the order of the connection of two links on this side.
    //		
    //		Returns true if the links are in the correct order and false
    //		otherwise.
    // tags:
    //		private
    
    var point11 = link1.getPointAt(origin1, 1);
    var point12 = link1.getPointAt(origin1, 2);
    var point21 = link2.getPointAt(origin2, 1);
    var point22 = link2.getPointAt(origin2, 2);
    var goingLeft1 = this.getTangentDelta(point11, point12) >= 0;
    var goingLeft2 = this.getTangentDelta(point21, point22) >= 0;

    if (goingLeft1)
    {
      if (!goingLeft2) {
        return true; // Boolean
      }
    }
    else if (goingLeft2) {
      return false; // Boolean
    }

    var destination1 = !origin1;
    var destination2 = !origin2;
    var oppositeNode1 = link1.getConnectionNode(destination1);
    var oppositeSide = link1.getNodeSide(destination1);

    if (
      link2.getConnectionNode(destination2) == oppositeNode1 && (oppositeSide == link2.getNodeSide(destination2)) &&
        (link1.getCurrentShapeType().getNumberOfBends() == link2.getCurrentShapeType().getNumberOfBends()))
      {
        var oppositeConnPoint1 = link1.getConnectionPoint(destination1);
        var oppositeConnPoint2 = link2.getConnectionPoint(destination2);
        if (TranslateUtil.pointEquals(oppositeConnPoint1, oppositeConnPoint2)) {
          return true; // Boolean
        }
        else {
          return this.compareLinksByOppositeDirection(oppositeSide.getDirection(), oppositeConnPoint1, oppositeConnPoint2); // Boolean
        }
      }

    var nPoints1 = link1.getLinkShape().getNumberOfPoints();
    var nPoints2 = link2.getLinkShape().getNumberOfPoints();

    if (nPoints1 < 3 || nPoints2 < 3)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6631E");
    }

    if (nPoints1 == 3 && nPoints2 == 3) {
      return this.compareLinksByOppositeDirection(oppositeSide.getDirection(), link1.getConnectionPoint(destination1), link2.getConnectionPoint(destination2)); // Boolean
    }
    if (nPoints1 == 3 && nPoints2 >= 4)
    {
      var point23 = link2.getPointAt(origin2, 3);
      if (this.getPointRadialDelta(point22, point23) < 0 && this.getPointRadialDelta(point11, point21) < 0) {
        return !goingLeft1; // Boolean
      }
    }
    else if (nPoints1 >= 4 && nPoints2 == 3)
    {
      var point13 = link1.getPointAt(origin1, 3);
      if (this.getPointRadialDelta(point12, point13) < 0 && this.getPointRadialDelta(point21, point11) < 0) {
        return goingLeft1; // Boolean
      }
    }

    var firstLink;
    var secondLink;
    var firstLinkPoints = layout.getTempPoints1();
    var secondLinkPoints = layout.getTempPoints2();
    var firstWidth;
    var secondWidth;
    var firstOrigin;
    var secondOrigin;
    var firstNPoints;
    var secondNPoints;

    if (goingLeft1)
    {
      firstLink = link1;
      secondLink = link2;
      firstOrigin = origin1;
      secondOrigin = origin2;
      firstNPoints = nPoints1;
      secondNPoints = nPoints2;
    }
    else
    {
      firstLink = link2;
      secondLink = link1;
      firstOrigin = origin2;
      secondOrigin = origin1;
      firstNPoints = nPoints2;
      secondNPoints = nPoints1;
    }

    firstLink.copyPointLocations(firstLinkPoints, firstOrigin);
    secondLink.copyPointLocations(secondLinkPoints, secondOrigin);
    firstWidth = firstLink.getLinkWidth();
    secondWidth = secondLink.getLinkWidth();
    var firstPoint3 = null;
    var firstPoint4 = null;

    if (firstNPoints >= 4)
    {
      firstPoint3 = firstLinkPoints[3];
      if (firstNPoints >= 5) {
        firstPoint4 = firstLinkPoints[4];
      }
    }

    var secondPoint3 = null;
    var secondPoint4 = null;
    var secondPoint5 = null;

    if (secondNPoints >= 4)
    {
      secondPoint3 = secondLinkPoints[3];

      if (secondNPoints >= 5)
      {
        secondPoint4 = secondLinkPoints[4];
        if (secondNPoints >= 6) {
          secondPoint5 = secondLinkPoints[5];
        }
      }
    }

    var dangerousRefinement = this.refineLinksOrthogonalWithPoints(layout, firstLinkPoints[1], firstLinkPoints[2], firstPoint3, firstPoint4, secondLinkPoints[1], secondLinkPoints[2], secondPoint3,
        secondPoint4, secondPoint5, firstWidth, secondWidth, goingLeft1, goingLeft2, firstLink.isSameSideSelfLink(), firstNPoints - 2, secondNPoints - 2, secondLink.boundingBox,
        link2.getNodeSide(destination2), _SLNodeSide.SMALL_DELTA, false, true);

    if (dangerousRefinement) {
      return false; // Boolean
    }
    return !this.intersectsOrthogonalWithParallelStart(layout, firstLinkPoints, secondLinkPoints, firstNPoints, secondNPoints, goingLeft1); // Boolean
  },

  intersectsOrthogonalWithParallelStart : function(layout, points1, points2, nPoints1, nPoints2, goingLeft1)
  {
    // summary:
    //		Utility that checks if two orthogonal polylines intersects. 
    // description:
    //		Utility that checks if two orthogonal polylines intersects. 
    //		The assumption is made that both polylines start with either
    //		horizontal, or vertical segments. This allows to reduce the number
    //		of segment-to-segment crossing check, because we
    //		don't need to check two horizontal, or two vertical segments.
    // tags:
    //		private
    
    var limit1 = nPoints1 - 1;
    var limit2 = nPoints2 - 1;
    var from1;
    var to1;

    var from2 = layout.getTempPoint1();
    var to2 = layout.getTempPoint2();

    var smallDelta = (goingLeft1 ? -_SLNodeSide.SMALL_DELTA : _SLNodeSide.SMALL_DELTA);

    for (var i = 0; i < limit1; i++)
    {
      from1 = points1[i];
      to1 = points1[i + 1];

      for (var j = ((i % 2 == 0 ? 1 : 0)); j < limit2; j += 2)
      {
        TranslateUtil.setPointLocation(from2, points2[j]);

        TranslateUtil.setPointLocation(to2, points2[j + 1]);
        this.translateLeft(from2, smallDelta);
        this.translateLeft(to2, smallDelta);
        if (LayoutUtil.intersectsOrthogonal(from1.x, from1.y, to1.x, to1.y, from2.x, from2.y, to2.x, to2.y)) {
          return true; // Boolean
        }
      }
    }

    return false; // Boolean
  },

  refineLinksOrthogonalWithPoints : function(layout, point11, point12, point13, point14, point21, point22, point23, point24, point25, linkWidth1, linkWidth2, goingLeft1, goingLeft2,
    firstLinkIsSameSideSelfLink, nBends1, nBends2, linkRegion2, oppositeSide, linkOffset, sameOpposite, checkDangerousRefinement)
  {
    // summary:
    //		Refines a link in respect to the exact shape of its predecesor.  
    // returns:
    //		true if the refinement requires that the link goes outside its
    //		linkRegion. If the link goes outside the linkRegion, some crossings
    //		or overlaps may be missed by the CostComputer...
    
    var halfWidth1 = linkWidth1 * 0.5;
    var halfWidth2 = linkWidth2 * 0.5;
    var delta = linkOffset + halfWidth1 + halfWidth2;
    var overlappingSeg12 = this.overlappingTangentially(point11, point12, point21, point22, halfWidth1, halfWidth2);
    var tempPoint = layout.getTempPoint1();

    if (overlappingSeg12)
    {
      TranslateUtil.setPointLocation(tempPoint, point11);
      this.translateOutside(tempPoint, delta);

      if (this.getPointRadialDelta(point21, tempPoint) > 0)
      {
        if (point23 == null && checkDangerousRefinement) {
          return true; // Boolean
        }
        if (nBends2 == 2 && this.getPointRadialDelta(point23, point21) < 0 && this.getPointRadialDelta(tempPoint, point23) < halfWidth2)
        {
          if (checkDangerousRefinement) {
            return true; // Boolean
          }
          TranslateUtil.setPointLocation(tempPoint, point23);

          this.translateOutside(tempPoint, -halfWidth2);
        }

        this.setRadialCoord(point21, this.getPointRadialCoord(tempPoint));
        this.setRadialCoord(point22, this.getPointRadialCoord(tempPoint));

        if (checkDangerousRefinement && !TranslateUtil.rectContains(linkRegion2, point21.x, point21.y)) {
          return true; // Boolean
        }
      }

      if (nBends2 >= 3 && point13 != null && goingLeft1 == goingLeft2)
      {
        var rad23Seg1 = this.getPointRadialDelta(point12, point13);
        var rad23Seg2 = this.getPointRadialDelta(point22, point23);

        if (((rad23Seg1 < 0 && rad23Seg2 < 0) || (rad23Seg1 > 0 && rad23Seg2 > 0)) && this.overlappingRadially(point12, point13, point22, point23, halfWidth1, halfWidth2))
        {
          TranslateUtil.setPointLocation(tempPoint, point12);
          this.translateLeft(tempPoint, ((goingLeft2 ? ((rad23Seg1 <= 0 ? delta : -delta)) : ((rad23Seg1 >= 0 ? delta : -delta)))));
          var tanDelta = this.getTangentDelta(point22, tempPoint);
          var needsRefinement = false;

          if (goingLeft2)
          {
            if (rad23Seg1 <= 0)
            {
              if (tanDelta > 0) {
                needsRefinement = true;
              }
            }
            else if (tanDelta < 0) {
              needsRefinement = true;
            }
          }
          else
          {
            if (rad23Seg1 <= 0)
            {
              if (tanDelta < 0) {
                needsRefinement = true;
              }
            }
            else if (tanDelta > 0) {
              needsRefinement = true;
            }
          }

          if (needsRefinement)
          {
            var tan34Seg2 = this.getTangentDelta(point23, point24);
            var newTan34Seg2 = this.getTangentDelta(tempPoint, point24);

            if ((newTan34Seg2 >= 0 && tan34Seg2 >= 0 && !(nBends2 == 3 && newTan34Seg2 < halfWidth2)) || (newTan34Seg2 <= 0 && tan34Seg2 < 0 && !(nBends2 == 3 && -newTan34Seg2 < halfWidth2)))
            {
              var tanCoord = this.getTangentialCoord(tempPoint);
              this.setTangentialCoord(point22, tanCoord);
              this.setTangentialCoord(point23, tanCoord);

              if (checkDangerousRefinement && !TranslateUtil.rectContains(linkRegion2, point22.x, point22.y)) {
                return true; // Boolean
              }
            }
          }

          if (nBends2 >= 4 && point14 != null)
          {
            if (this.overlappingTangentially(point13, point14, point23, point24, halfWidth1, halfWidth2))
            {
              TranslateUtil.setPointLocation(tempPoint, point13);

              var tan34Seg1 = this.getTangentDelta(point13, point14);
              this.translateOutside(tempPoint,
                  ((goingLeft2
                      ? ((rad23Seg1 >= 0 ? ((tan34Seg1 >= 0 ? delta : -delta)) : ((tan34Seg1 >= 0 ? delta : -delta))))
                      : ((rad23Seg1 >= 0 ? ((tan34Seg1 <= 0 ? delta : -delta)) : ((tan34Seg1 <= 0 ? delta : -delta)))))));
              var radDelta = this.getPointRadialDelta(point23, tempPoint);

              if ((goingLeft2 && ((tan34Seg1 >= 0 && radDelta > 0) || (tan34Seg1 <= 0 && radDelta < 0))) || (!goingLeft2 && ((tan34Seg1 >= 0 && radDelta < 0) || (tan34Seg1 <= 0 && radDelta > 0))))
              {
                if (nBends2 == 4)
                {
                  var seg45Length = this.getPointRadialDelta(point24, point25);
                  var seg45LengthNew = this.getPointRadialDelta(tempPoint, point25);

                  if ((seg45Length < 0 && seg45LengthNew >= 0) || (seg45Length > 0 && seg45LengthNew <= 0) || (Math.abs(seg45Length) >= halfWidth2 && Math.abs(seg45LengthNew) < halfWidth2))
                  {
                    if (checkDangerousRefinement) {
                      return true; // Boolean
                    }
                    TranslateUtil.setPointLocation(tempPoint, point25);
                    oppositeSide.translateOutside(tempPoint, halfWidth2);
                  }
                }

                var radCoord = this.getPointRadialCoord(tempPoint);

                this.setRadialCoord(point23, radCoord);
                this.setRadialCoord(point24, radCoord);

                if (checkDangerousRefinement && !TranslateUtil.rectContains(linkRegion2, point23.x, point23.y)) {
                  return true; // Boolean
                }
              }
            }
          }
        }
      }
    }

    return false; // Boolean
  },

  overlappingRadially : function(point1, point2, point3, point4, halfWidth1, halfWidth2)
  {
    // summary:
    //		Returns true if the segment point1-point2 overlaps the segment
    //		point3-point4 in the radial direction wrt. 
    // description:
    //		Returns true if the segment point1-point2 overlaps the segment
    //		point3-point4 in the radial direction wrt. 
    //		this node side. For instance, the following segments do overlap
    //		radially wrt. the left side: ----- ---- while the following
    //		don't overlap radially wrt. the left side: ---- ----
    // tags:
    //		private
    
    return LayoutUtil.overlappingCoords(this.getPointRadialCoord(point1), this.getPointRadialCoord(point2), this.getPointRadialCoord(point3), this.getPointRadialCoord(point4), halfWidth1, halfWidth2); // Boolean
  },

  overlappingTangentially : function(point1, point2, point3, point4, halfWidth1, halfWidth2)
  {
    // summary:
    //		Returns true if the segment point1-point2 overlaps the segment
    //		point3-point4 in the radial direction wrt. 
    // description:
    //		Returns true if the segment point1-point2 overlaps the segment
    //		point3-point4 in the radial direction wrt. 
    //		this node side. For instance, the following segments do overlap
    //		radially wrt. the top side: ----- ---- while the following
    //		don't overlap radially wrt. the top side: ---- ----
    // tags:
    //		private
    
    return LayoutUtil.overlappingCoords(this.getTangentialCoord(point1), this.getTangentialCoord(point2), this.getTangentialCoord(point3), this.getTangentialCoord(point4), halfWidth1, halfWidth2); // Boolean
  },

  compareLinksByOppositeDirection : function(oppositeDirection, refPoint1, refPoint2)
  {
    // summary:
    //		Decides the order of the connection of two links that have the same
    //		category number and the same opposite side. 
    // description:
    //		Decides the order of the connection of two links that have the same
    //		category number and the same opposite side. 
    //		If the method returns true, link1 will be "before" link2.  
    // refPoint1: Point
    //		The reference point of link1 - usually the connection
    //		point on the opposite node
    // refPoint2: Point
    //		The reference point of link2
    // tags:
    //		extension
    },
  
  isOnLeft : function(goingLeft)
  {
    // summary:
    //		Computes the category number for a link from the view point of a
    //		given end node. 
    // description:
    //		Computes the category number for a link from the view point of a
    //		given end node. 
    //		Computes the "category" number of a link connected to a given node.
    //		For instance, for links connected to the TOP side of a node: Cat 1
    //		(currently not used) Cat 2           Cat 4 --------- |    | |      
    //		| |       | --------- Cat 1           Cat 5 The "category" numbers
    //		are ordered wrt. the optimal connection order on the node side. E.g.
    //		links of category 1 must be connected "before" (= on the left, for
    //		TOP side) links of category 2 etc.  
    // sideMiddlePoint:
    //		The point in the middle of the node side
    // refPoint:
    //		The reference point of the link - usually the connection
    //		point on the opposite node
    // tags:
    //		extension
    },
  
  getSidePosition : function(side)
  {
    // summary:
    //		Returns Direction.Left if 'side' is on the left of this
    //		side, Direction.Right if it is on the right, etc. 
    // description:
    //		Returns Direction.Left if <code>side</code> is on the left of this
    //		side, Direction.Right if it is on the right, etc. 
    //		Of course, left/right are relatively to this node side. This is
    //		currently used for only one purpuse: to determine whether the
    //		connection point of a 3-bend self-link should be placed on the left
    //		or on the right of a given node side. This depends on the connection
    //		side at the opposite extremity of the 3-bend self-link. Notice that
    //		self-links cannot be "quasi-self-intergraph links", hence the method
    //		will not be called for quasi-self-intergraph links. But anyway, the
    //		result is correct also for reversed directions of the node side.
    // tags:
    //		extension
    },
  
  getMinRadialCoord : function(rect)
  {},
  
  getMaxRadialCoord : function(rect)
  {},
  
  getMinTangentialCoord : function(rect)
  {},
  
  getMaxTangentialCoord : function(rect)
  {},
  
  getRectRadialCoord : function(linkConnectionRect)
  {
    // summary:
    //		For TOP, returns rect.y, for BOTTOM (rect.y + rect.height) etc.  
    // tags:
    //		extension
    },
  
  getPointRadialDelta : function(point1, point2)
  {
    // summary:
    //		Returns the radial delta of two points (usually, two points of a
    //		link).  
    // tags:
    //		private
    
    return this.getRadialDelta(this.getPointRadialCoord(point1), this.getPointRadialCoord(point2)); // Number
  },

  getRadialDelta : function(radCoord1, radCoord2)
  {
    // summary:
    //		Returns the radial delta of two radial coordinates.
    // tags:
    //		protected
    },
  
  getRectRadialDelta : function(point, rect)
  {
    // summary:
    //		Returns the radial delta of a point wrt. 
    // description:
    //		Returns the radial delta of a point wrt. 
    //		the radial coordinate of a rectangle.  
    // tags:
    //		private
    
    return this.getRadialDelta(this.getPointRadialCoord(point), this.getRectRadialCoord(rect)); // Number
  },

  getTangentDelta : function(point1, point2)
  {
    // summary:
    //		Returns the tangent delta of two points (usually, two points of a
    //		link). 
    // description:
    //		Returns the tangent delta of two points (usually, two points of a
    //		link). 
    //		Note that getTangentDelta() >= 0 is used to detect the direction of
    //		the turn
    // tags:
    //		protected
    },
  
  translateOutside : function(point, radialDelta)
  {
    // summary:
    //		Moves 'point' radially (in respect to the node side
    //		orientation) with 'radialDelta'.
    // tags:
    //		protected
    },
  
  translateLeft : function(point, tanDelta)
  {
    // summary:
    //		Moves 'point' tangentially (in respect to the node side
    //		orientation) with 'tanDelta', such that the tangential
    //		coord is reduced.
    // tags:
    //		protected
    },
  
  copyRadialCoord : function(source, dest)
  {
    // summary:
    //		Copies the x- or y- coordinate of 'source' (in the radial
    //		direction wrt. 
    // description:
    //		Copies the x- or y- coordinate of <code>source</code> (in the radial
    //		direction wrt. 
    //		the node side) to <code>dest</code>.
    //		It's the x- coord for left and right sides, and y-coord for top and bottom sides.
    // tags:
    //		protected
    },
  
  setRadialCoord : function(point, radCoord)
  {
    // summary:
    //		Sets the x- or y- coordinate of a point in the radial direction wrt.
    //		
    // description:
    //		Sets the x- or y- coordinate of a point in the radial direction wrt.
    //		
    //		the node side.
    //		It's the x- coord for left and right sides, and y-coord for top and bottom sides.
    // tags:
    //		protected
    },
  
  getPointRadialCoord : function(point)
  {
    // summary:
    //		Returns the x- or y- coordinate of a point in the radial diretion
    //		wrt. 
    // description:
    //		Returns the x- or y- coordinate of a point in the radial diretion
    //		wrt. 
    //		the node side.
    //		It's the x- coord for left and right sides, and y-coord for top and bottom sides.  
    // tags:
    //		protected
    },
  
  setTangentialCoord : function(point, tanCoord)
  {
    // summary:
    //		Sets the x- or y- coordinate of a point in the tangential direction
    //		wrt. 
    // description:
    //		Sets the x- or y- coordinate of a point in the tangential direction
    //		wrt. 
    //		the node side.
    //		It's the y- coord for left and right sides, and x-coord for top and bottom sides.
    // tags:
    //		protected
    },
  
  getTangentialCoord : function(point)
  {
    // summary:
    //		Returns the x- or y- coordinate of a point in the tangential
    //		diretion wrt. 
    // description:
    //		Returns the x- or y- coordinate of a point in the tangential
    //		diretion wrt. 
    //		the node side.
    //		It's the y- coord for left and right sides, and x-coord for top and bottom sides.
    // tags:
    //		protected
    }
  
}
);

lang.mixin(_SLNodeSide,
{
  SMALL_DELTA : null,
  _fields_init : function(c)
  {
    c = c || _SLNodeSide;
    
    c.SMALL_DELTA = 0.01;
  }
  
}
);

_SLNodeSide.SLTopNodeSide = declare(_SLNodeSide, 
{

  constructor : function (a0)
  {
    // summary:
    
    if(arguments.length == 0)
      this._constructor_overload__SLNodeSide_SLTopNodeSide_0();
    else if(arguments.length == 1)
      this._constructor_overload__SLNodeSide_SLTopNodeSide_1(a0);
  },
  
  _constructor_overload__SLNodeSide_SLTopNodeSide_0 : function()
  {

  },
  
  _constructor_overload__SLNodeSide_SLTopNodeSide_1 : function(reversedDirection )
  {
  },
  
  getSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.TOP; // int
  },

  getOppositeSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.BOTTOM; // int
  },

  moveToDefaultConnectionPoint : function(point, rect)
  {
    TranslateUtil.movePoint(point, rect.x + rect.width * 0.5, rect.y);
  },

  compareLinksByOppositeDirection : function(oppositeDirection, point1, point2)
  {
    // summary:
    //		Decides on the order of two links (say, link1 and link2) that are
    //		both connected to the same side (top, bottom, right or left) on the
    //		opposite nodes. 
    // description:
    //		Decides on the order of two links (say, link1 and link2) that are
    //		both connected to the same side (top, bottom, right or left) on the
    //		opposite nodes. 
    //		(They may have the same opposite node - case of multilinks), or
    //		different nodes.)  
    // oppositeDirection: int
    //		The direction of the links on the opposite node
    // point1: Point
    //		the "reference point" on link1
    // point2: Point
    //		the "reference point" on link2
    // returns:
    //		true if the order "link2 after link1" is correct, false
    //		otherwise
    // tags:
    //		private
    
    var res = false;

    switch (oppositeDirection)
    {
      case Direction.TOP:
        res = point1.x >= point2.x;
        break;
      case Direction.BOTTOM:
        res = point1.x <= point2.x;
        break;
      case Direction.LEFT:
        res = point1.y <= point2.y;
        break;
      case Direction.RIGHT:
        res = point1.y >= point2.y;
        break;
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6630E",  oppositeDirection);
    }
    return (this._isReversedDirection ? !res : res); // Boolean
  },

  isOnLeft : function(goingLeft)
  {
    return goingLeft; // Boolean
  },

  getSidePosition : function(side)
  {
    // summary:
    // tags:
    //		private
    
    return side; // int
  },

  getMinRadialCoord : function(rect)
  {
    return rect.y; // Number
  },

  getMaxRadialCoord : function(rect)
  {
    return rect.y + rect.height; // Number
  },

  getMinTangentialCoord : function(rect)
  {
    return rect.x; // Number
  },

  getMaxTangentialCoord : function(rect)
  {
    return rect.x + rect.width; // Number
  },

  getRectRadialCoord : function(linkConnectionRect)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return linkConnectionRect.y; // Number
  },

  getRadialDelta : function(radCoord1, radCoord2)
  {
    return (this._isReversedDirection ? radCoord2 - radCoord1 : radCoord1 - radCoord2); // Number
  },

  getTangentDelta : function(point1, point2)
  {
    // summary:
    // tags:
    //		private
    
    return point1.x - point2.x; // Number
  },

  translateOutside : function(point, radialDelta)
  {
    // summary:
    // tags:
    //		private
    
    if (this._isReversedDirection) {
      radialDelta = -radialDelta;
    }
    point.y -= radialDelta;
  },

  translateLeft : function(point, tanDelta)
  {
    // summary:
    // tags:
    //		private
    
    point.x -= tanDelta;
  },

  copyRadialCoord : function(source, dest)
  {
    // summary:
    // tags:
    //		private
    
    dest.y = source.y;
  },

  setRadialCoord : function(point, radCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.y = radCoord;
  },

  getPointRadialCoord : function(point)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return point.y; // Number
  },

  setTangentialCoord : function(point, tanCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.x = tanCoord;
  },

  getTangentialCoord : function(point)
  {
    // summary:
    // tags:
    //		private
    
    return point.x; // Number
  }

}
);
_SLNodeSide.SLBottomNodeSide = declare(_SLNodeSide, 
{

  constructor : function (a0)
  {
    // summary:
    
    if(arguments.length == 0)
      this._constructor_overload__SLNodeSide_SLBottomNodeSide_0();
    else if(arguments.length == 1)
      this._constructor_overload__SLNodeSide_SLBottomNodeSide_1(a0);
  },
  
  _constructor_overload__SLNodeSide_SLBottomNodeSide_0 : function()
  {

  },
  
  _constructor_overload__SLNodeSide_SLBottomNodeSide_1 : function(reversedOrientation )
  {
  },
  
  getSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.BOTTOM; // int
  },

  getOppositeSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.TOP; // int
  },

  moveToDefaultConnectionPoint : function(point, rect)
  {
    TranslateUtil.movePoint(point, rect.x + rect.width * 0.5, rect.y + rect.height);
  },

  compareLinksByOppositeDirection : function(oppositeDirection, point1, point2)
  {
    // summary:
    // tags:
    //		private
    
    var res = false;

    switch (oppositeDirection)
    {
      case Direction.TOP:
        res = point1.x <= point2.x;
        break;
      case Direction.BOTTOM:
        res = point1.x >= point2.x;
        break;
      case Direction.LEFT:
        res = point1.y >= point2.y;
        break;
      case Direction.RIGHT:
        res = point1.y <= point2.y;
        break;
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6630E",  oppositeDirection);
    }
    return (this._isReversedDirection ? !res : res); // Boolean
  },

  isOnLeft : function(goingLeft)
  {
    return !goingLeft; // Boolean
  },

  getSidePosition : function(side)
  {
    // summary:
    // tags:
    //		private
    
    switch (side)
    {
      case Direction.LEFT:
      case Direction.RIGHT:
        return side; // int
      case Direction.TOP:
        return Direction.BOTTOM; // int
      case Direction.BOTTOM:
        return Direction.TOP; // int
      default:
        return Direction.NONE; // int
    }
  },

  getMinRadialCoord : function(rect)
  {
    return rect.y; // Number
  },

  getMaxRadialCoord : function(rect)
  {
    return rect.y + rect.height; // Number
  },

  getMinTangentialCoord : function(rect)
  {
    return rect.x; // Number
  },

  getMaxTangentialCoord : function(rect)
  {
    return rect.x + rect.width; // Number
  },

  getRectRadialCoord : function(linkConnectionRect)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return linkConnectionRect.y + linkConnectionRect.height; // Number
  },

  getRadialDelta : function(radCoord1, radCoord2)
  {
    return (this._isReversedDirection ? radCoord1 - radCoord2 : radCoord2 - radCoord1); // Number
  },

  getTangentDelta : function(point1, point2)
  {
    // summary:
    // tags:
    //		private
    
    return point1.x - point2.x; // Number
  },

  translateOutside : function(point, radialDelta)
  {
    // summary:
    // tags:
    //		private
    
    if (this._isReversedDirection) {
      radialDelta = -radialDelta;
    }
    point.y += radialDelta;
  },

  translateLeft : function(point, tanDelta)
  {
    // summary:
    // tags:
    //		private
    
    point.x -= tanDelta;
  },

  copyRadialCoord : function(source, dest)
  {
    // summary:
    // tags:
    //		private
    
    dest.y = source.y;
  },

  setRadialCoord : function(point, radCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.y = radCoord;
  },

  getPointRadialCoord : function(point)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return point.y; // Number
  },

  setTangentialCoord : function(point, tanCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.x = tanCoord;
  },

  getTangentialCoord : function(point)
  {
    // summary:
    // tags:
    //		private
    
    return point.x; // Number
  }

}
);
_SLNodeSide.SLLeftNodeSide = declare(_SLNodeSide, 
{

  constructor : function (a0)
  {
    // summary:
    
    if(arguments.length == 0)
      this._constructor_overload__SLNodeSide_SLLeftNodeSide_0();
    else if(arguments.length == 1)
      this._constructor_overload__SLNodeSide_SLLeftNodeSide_1(a0);
  },
  
  _constructor_overload__SLNodeSide_SLLeftNodeSide_0 : function()
  {

  },
  
  _constructor_overload__SLNodeSide_SLLeftNodeSide_1 : function(reversedOrientation )
  {
  },
  
  getSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.LEFT; // int
  },

  getOppositeSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.RIGHT; // int
  },

  moveToDefaultConnectionPoint : function(point, rect)
  {
    TranslateUtil.movePoint(point, rect.x, rect.y + rect.height * 0.5);
  },

  compareLinksByOppositeDirection : function(oppositeDirection, point1, point2)
  {
    // summary:
    // tags:
    //		private
    
    var res = false;

    switch (oppositeDirection)
    {
      case Direction.TOP:
        res = point1.x <= point2.x;
        break;
      case Direction.BOTTOM:
        res = point1.x >= point2.x;
        break;
      case Direction.LEFT:
        res = point1.y >= point2.y;
        break;
      case Direction.RIGHT:
        res = point1.y <= point2.y;
        break;
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6630E",  oppositeDirection);
    }
    return (this._isReversedDirection ? !res : res); // Boolean
  },

  isOnLeft : function(goingLeft)
  {
    return !goingLeft; // Boolean
  },

  getSidePosition : function(side)
  {
    // summary:
    // tags:
    //		private
    
    switch (side)
    {
      case Direction.LEFT:
        return Direction.TOP; // int
      case Direction.RIGHT:
        return Direction.BOTTOM; // int
      case Direction.TOP:
        return Direction.LEFT; // int
      case Direction.BOTTOM:
        return Direction.RIGHT; // int
      default:
        return Direction.NONE; // int
    }
  },

  getMinRadialCoord : function(rect)
  {
    return rect.x; // Number
  },

  getMaxRadialCoord : function(rect)
  {
    return rect.x + rect.width; // Number
  },

  getMinTangentialCoord : function(rect)
  {
    return rect.y; // Number
  },

  getMaxTangentialCoord : function(rect)
  {
    return rect.y + rect.height; // Number
  },

  getRectRadialCoord : function(linkConnectionRect)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return linkConnectionRect.x; // Number
  },

  getRadialDelta : function(radCoord1, radCoord2)
  {
    return (this._isReversedDirection ? radCoord2 - radCoord1 : radCoord1 - radCoord2); // Number
  },

  getTangentDelta : function(point1, point2)
  {
    // summary:
    // tags:
    //		private
    
    return point1.y - point2.y; // Number
  },

  translateOutside : function(point, radialDelta)
  {
    // summary:
    // tags:
    //		private
    
    if (this._isReversedDirection) {
      radialDelta = -radialDelta;
    }
    point.x -= radialDelta;
  },

  translateLeft : function(point, tanDelta)
  {
    // summary:
    // tags:
    //		private
    
    point.y -= tanDelta;
  },

  copyRadialCoord : function(source, dest)
  {
    // summary:
    // tags:
    //		private
    
    dest.x = source.x;
  },

  setRadialCoord : function(point, radCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.x = radCoord;
  },

  getPointRadialCoord : function(point)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return point.x; // Number
  },

  setTangentialCoord : function(point, tanCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.y = tanCoord;
  },

  getTangentialCoord : function(point)
  {
    // summary:
    // tags:
    //		private
    
    return point.y; // Number
  }

}
);
_SLNodeSide.SLRightNodeSide = declare(_SLNodeSide, 
{

  constructor : function (a0)
  {
    // summary:
    
    if(arguments.length == 0)
      this._constructor_overload__SLNodeSide_SLRightNodeSide_0();
    else if(arguments.length == 1)
      this._constructor_overload__SLNodeSide_SLRightNodeSide_1(a0);
  },
  
  _constructor_overload__SLNodeSide_SLRightNodeSide_0 : function()
  {

  },
  
  _constructor_overload__SLNodeSide_SLRightNodeSide_1 : function(reversedOrientation )
  {
  },
  
  getSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.RIGHT; // int
  },

  getOppositeSide : function()
  {
    // summary:
    // tags:
    //		private
    
    return Direction.LEFT; // int
  },

  moveToDefaultConnectionPoint : function(point, rect)
  {
    TranslateUtil.movePoint(point, rect.x + rect.width, rect.y + rect.height * 0.5);
  },

  compareLinksByOppositeDirection : function(oppositeDirection, point1, point2)
  {
    // summary:
    // tags:
    //		private
    
    var res = false;

    switch (oppositeDirection)
    {
      case Direction.TOP:
        res = point1.x >= point2.x;
        break;
      case Direction.BOTTOM:
        res = point1.x <= point2.x;
        break;
      case Direction.LEFT:
        res = point1.y <= point2.y;
        break;
      case Direction.RIGHT:
        res = point1.y >= point2.y;
        break;
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6630E",  oppositeDirection);
    }
    return (this._isReversedDirection ? !res : res); // Boolean
  },

  isOnLeft : function(goingLeft)
  {
    return goingLeft; // Boolean
  },

  getSidePosition : function(side)
  {
    // summary:
    // tags:
    //		private
    
    switch (side)
    {
      case Direction.LEFT:
        return Direction.BOTTOM; // int
      case Direction.RIGHT:
        return Direction.TOP; // int
      case Direction.TOP:
        return Direction.LEFT; // int
      case Direction.BOTTOM:
        return Direction.RIGHT; // int
      default:
        return Direction.NONE; // int
    }
  },

  getMinRadialCoord : function(rect)
  {
    return rect.x; // Number
  },

  getMaxRadialCoord : function(rect)
  {
    return rect.x + rect.width; // Number
  },

  getMinTangentialCoord : function(rect)
  {
    return rect.y; // Number
  },

  getMaxTangentialCoord : function(rect)
  {
    return rect.y + rect.height; // Number
  },

  getRectRadialCoord : function(linkConnectionRect)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return linkConnectionRect.x + linkConnectionRect.width; // Number
  },

  getRadialDelta : function(radCoord1, radCoord2)
  {
    return (this._isReversedDirection ? radCoord1 - radCoord2 : radCoord2 - radCoord1); // Number
  },

  getTangentDelta : function(point1, point2)
  {
    // summary:
    // tags:
    //		private
    
    return point1.y - point2.y; // Number
  },

  translateOutside : function(point, radialDelta)
  {
    // summary:
    // tags:
    //		private
    
    if (this._isReversedDirection) {
      radialDelta = -radialDelta;
    }
    point.x += radialDelta;
  },

  translateLeft : function(point, tanDelta)
  {
    // summary:
    // tags:
    //		private
    
    point.y -= tanDelta;
  },

  copyRadialCoord : function(source, dest)
  {
    // summary:
    // tags:
    //		private
    
    dest.x = source.x;
  },

  setRadialCoord : function(point, radCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.x = radCoord;
  },

  getPointRadialCoord : function(point)
  {
    // summary:
    //		
    // tags:
    //		private
    
    return point.x; // Number
  },

  setTangentialCoord : function(point, tanCoord)
  {
    // summary:
    // tags:
    //		private
    
    point.y = tanCoord;
  },

  getTangentialCoord : function(point)
  {
    // summary:
    // tags:
    //		private
    
    return point.y; // Number
  }

}
);
(function(){ _SLNodeSide._fields_init(); })();

return _SLNodeSide;
});

},
'ibm_ilog/graphlayout/shortlink/_OrthogonalLinkShapeProducer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_OrthogonalLinkShapeProducer", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../shortlink/_LinkShapeProducer",	// #8
  "../shortlink/_LinkShapeType",	// #9
  "../shortlink/_OrthoWith4BendsPositionArrays",	// #10
  "../shortlink/_OrthoWithout4BendsPositionArrays",	// #11
  "../Direction"	// #12
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    TranslateUtil,	// #7
    _LinkShapeProducer,	// #8
    _LinkShapeType,	// #9
    _OrthoWith4BendsPositionArrays,	// #10
    _OrthoWithout4BendsPositionArrays,	// #11
    Direction	// #12
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _LinkShapeProducer = ibm_ilog.graphlayout.shortlink._LinkShapeProducer;
=====*/

var _OrthogonalLinkShapeProducer = declare("ibm_ilog.graphlayout.shortlink._OrthogonalLinkShapeProducer", _LinkShapeProducer, 
{
  // summary:
  //		A class that is used to produce candidate shapes for the links. 
  // description:
  //		A class that is used to produce candidate shapes for the links. 
  //		This class is used by the short link layout algorithm.  
  // tags:
  //		private
  
  constructor : function(layout)
  {
  },
  
  getMaxNumberOfBends : function()
  {
    // summary:
    //		Returns the maximum number of bends that the producer can produce. 
    // description:
    //		Returns the maximum number of bends that the producer can produce. 
    //		The default implementation returns always 3. You can override this
    //		method if your subclass supports more bends.
    // tags:
    //		private
    
    return _OrthogonalLinkShapeProducer.MAX_NUMBER_OF_BENDS; // int
  },

  getTopToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _OrthogonalLinkShapeProducer.TOP_TOP_SIDE_ARRAY; // Array
  },

  getTopToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return ((fromRect.y - toRect.y - toRect.height > 2 * bypassDistance) ? _OrthogonalLinkShapeProducer.TOP_BOTTOM_SIDE_2and4BENDS_ARRAY : _OrthogonalLinkShapeProducer.TOP_BOTTOM_SIDE_4BENDS_ARRAY); // Array
  },

  getTopToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.TOP_LEFT_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.TOP_LEFT_SIDE_ARRAY); // Array
  },

  getTopToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.TOP_RIGHT_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.TOP_RIGHT_SIDE_ARRAY); // Array
  },

  getBottomToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return ((toRect.y - fromRect.y - fromRect.height > 2 * bypassDistance)
      ? _OrthogonalLinkShapeProducer.BOTTOM_TOP_SIDE_2and4BENDS_ARRAY
      : _OrthogonalLinkShapeProducer.BOTTOM_TOP_SIDE_4BENDS_ARRAY); // Array
  },

  getBottomToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _OrthogonalLinkShapeProducer.BOTTOM_BOTTOM_SIDE_ARRAY; // Array
  },

  getBottomToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.BOTTOM_LEFT_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.BOTTOM_LEFT_SIDE_ARRAY); // Array
  },

  getBottomToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.BOTTOM_RIGHT_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.BOTTOM_RIGHT_SIDE_ARRAY); // Array
  },

  getLeftToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.LEFT_TOP_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.LEFT_TOP_SIDE_ARRAY); // Array
  },

  getLeftToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.LEFT_BOTTOM_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.LEFT_BOTTOM_SIDE_ARRAY); // Array
  },

  getLeftToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _OrthogonalLinkShapeProducer.LEFT_LEFT_SIDE_ARRAY; // Array
  },

  getLeftToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return ((toRect.x - fromRect.x - fromRect.width > 2 * bypassDistance) ? _OrthogonalLinkShapeProducer.LEFT_RIGHT_SIDE_2and4BENDS_ARRAY : _OrthogonalLinkShapeProducer.LEFT_RIGHT_SIDE_4BENDS_ARRAY); // Array
  },

  getRightToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.RIGHT_TOP_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.RIGHT_TOP_SIDE_ARRAY); // Array
  },

  getRightToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _OrthogonalLinkShapeProducer.RIGHT_BOTTOM_SIDE_OVERLAP_ARRAY : _OrthogonalLinkShapeProducer.RIGHT_BOTTOM_SIDE_ARRAY); // Array
  },

  getRightToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return ((toRect.x - fromRect.x - fromRect.width > 2 * bypassDistance) ? _OrthogonalLinkShapeProducer.RIGHT_LEFT_SIDE_2and4BENDS_ARRAY : _OrthogonalLinkShapeProducer.RIGHT_LEFT_SIDE_4BENDS_ARRAY); // Array
  },

  getRightToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _OrthogonalLinkShapeProducer.RIGHT_RIGHT_SIDE_ARRAY; // Array
  },

  getAllQuasiSelfInterGraphLinkShapeTypes : function()
  {
    // summary:
    //		Returns all the shape types for quasi-self-intergraph-links. 
    // description:
    //		Returns all the shape types for quasi-self-intergraph-links. 
    //		Used Only when the node side filtering or side-constraints have
    //		rejected all the shapes obtained by the fine-grain choice depending
    //		on the relative position of the end-nodes.
    // tags:
    //		protected
    
    return _OrthogonalLinkShapeProducer.ALL_QUASI_SELF_INTERGRAPH_LINK_TYPES; // Array
  },

  getFromBottomLeftToTopRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		Returns an array of shape types and computes the link region. 
    // description:
    //		Returns an array of shape types and computes the link region. 
    //		The selection of shapes depends on the relative position of the
    //		nodes and eventually on various settings. shapeTT ----------- |     
    //		   | |     ----- shapeLL--- |--------|   |------------------ |   |
    //		-------| T |--------         |shapeBR (not used) |   | |  ----|   | 
    //		     |      | |   | |  |   -----       |         | |   | |  |    | |
    //		       |  | |  ----- |sRL | |        |         | |  |   |--    | |  
    //		    |         | ---| F |-------shapeRB   |         | |  
    //		|--------|---------shapeRR  | -----        |                  | | | 
    //		 |                  | | -----------shapeBB           |
    //		--------------------------------
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(toRect.x - 2 * bypassDistance > fromRect.x + fromRect.width, fromRect.y - 2 * bypassDistance > toRect.y + toRect.height,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_BL_TO_TR_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_BL_TO_TR_POSITION_ARRAY)); // Array
  },

  getFromTopLeftToBottomRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(toRect.x - 2 * bypassDistance > fromRect.x + fromRect.width, toRect.y - 2 * bypassDistance > fromRect.y + fromRect.height,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_TL_TO_BR_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_TL_TO_BR_POSITION_ARRAY)); // Array
  },

  getFromLeftToRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		shapeTT --------- |       | -----   ----- | F |   | T | -----  
    //		----- |       | --------- shapeBB
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(toRect.x - 2 * bypassDistance > fromRect.x + fromRect.width,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_L_TO_R_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_L_TO_R_POSITION_ARRAY)); // Array
  },

  getFromBottomRightToTopLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | T | ----- ----- | F | -----
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(fromRect.x - 2 * bypassDistance > toRect.x + toRect.width, fromRect.y - 2 * bypassDistance > toRect.y + toRect.height,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_BR_TO_TL_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_BR_TO_TL_POSITION_ARRAY)); // Array
  },

  getFromTopRightToBottomLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(fromRect.x - 2 * bypassDistance > toRect.x + toRect.width, toRect.y - 2 * bypassDistance > fromRect.y + fromRect.height,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_TR_TO_BL_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_TR_TO_BL_POSITION_ARRAY)); // Array
  },

  getFromRightToLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		-----   ----- | T |   | F | -----   -----
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(toRect.x + toRect.width + 2 * bypassDistance < fromRect.x,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_R_TO_L_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_R_TO_L_POSITION_ARRAY)); // Array
  },

  getFromBottomToTopPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | T | ----- ----- | F | -----
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(toRect.y + toRect.height + 2 * bypassDistance < fromRect.y,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_B_TO_T_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_B_TO_T_POSITION_ARRAY)); // Array
  },

  getFromTopToBottomPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- ---| F |----- sh2 |  -----    | |   -----   | shape 1 ----| T
    //		|---- -----
    // tags:
    //		protected
    
    return this.getSelectedShapeTypes(toRect.y - 2 * bypassDistance > fromRect.y + fromRect.height,
        this.getPositionArray(linkData, _OrthoWith4BendsPositionArrays.FROM_T_TO_B_POSITION_ARRAY, _OrthoWithout4BendsPositionArrays.FROM_T_TO_B_POSITION_ARRAY)); // Array
  },

  getPositionArray : function(linkData, arrayWith4Bend, arrayWithout4Bend)
  {
    return ((this._nodeSideFilter != null || linkData.hasFixedConnectionPoint()) ? arrayWith4Bend : arrayWithout4Bend); // Array
  },

  getSelectedShapeTypes : function (a0, a1, a2)
  {
    // summary:
    // tags:
    //		private
    
    if(arguments.length == 2)
      return this.getSelectedShapeTypes_overload_2(a0, a1);
    else if(arguments.length == 3)
      return this.getSelectedShapeTypes_overload_3(a0, a1, a2);
  },
  
  getSelectedShapeTypes_overload_2 : function(condition, shapeTypesArray)
                                                                        
  {
    return (condition ? shapeTypesArray[0] : shapeTypesArray[1]); // Array
  },

  getSelectedShapeTypes_overload_3 : function(horizontalCondition, verticalCondition, shapeTypesArray)
  {
    if (horizontalCondition) {
      return (verticalCondition ? shapeTypesArray[3] : shapeTypesArray[1]); // Array
    }
    else if (verticalCondition) {
      return shapeTypesArray[2]; // Array
    }
    else {
      return shapeTypesArray[0]; // Array
    }
  },

  isEnoughSpace : function(linkData, shapeType, subgraphBox, nodeBox, bypassDistance, enoughSpaceOnLeft, enoughSpaceOnRight, enoughSpaceOnTop, enoughSpaceOnBottom)
  {
    // summary:
    //		Returns true if
    //		there's enough space on the node side, and returns false otherwise. 
    // description:
    //		Returns true if there's enough space on the node side, and returns
    //		false otherwise. 
    //		Overridden in OrthogonalLinkShapeProducer to add more checks.
    // tags:
    //		protected
    
    if (shapeType.isOrthogonal() && shapeType.getNumberOfBends() == 1)
    {
      var nodeSide = (linkData.isQuasiSelfInterGraphLinkOrigin() ? shapeType.getToSide(linkData) : shapeType.getFromSide(linkData));

      switch (nodeSide)
      {
        case Direction.LEFT:
          enoughSpaceOnLeft = nodeBox.x >= subgraphBox.x + subgraphBox.width * 0.5 + bypassDistance;
          break;
        case Direction.RIGHT:
          enoughSpaceOnRight = nodeBox.x + nodeBox.width <= subgraphBox.x + subgraphBox.width * 0.5 - bypassDistance;
          break;
        case Direction.TOP:
          enoughSpaceOnTop = nodeBox.y >= subgraphBox.y + subgraphBox.height * 0.5 + bypassDistance;
          break;
        case Direction.BOTTOM:
          enoughSpaceOnBottom = nodeBox.y + nodeBox.height <= subgraphBox.y + subgraphBox.height * 0.5 - bypassDistance;
          break;
        default:
          
          LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6630E",  nodeSide);
      }
    }

    return this.inherited("isEnoughSpace", arguments, [linkData, shapeType, subgraphBox, nodeBox, bypassDistance, enoughSpaceOnLeft, enoughSpaceOnRight, enoughSpaceOnTop, enoughSpaceOnBottom]); // Boolean
  }

}
);

lang.mixin(_OrthogonalLinkShapeProducer,
{
  MAX_NUMBER_OF_BENDS : null,

  TOP_TOP_SIDE_ARRAY : null,
  TOP_BOTTOM_SIDE_2and4BENDS_ARRAY : null,
  TOP_BOTTOM_SIDE_4BENDS_ARRAY : null,
  TOP_LEFT_SIDE_ARRAY : null,
  TOP_RIGHT_SIDE_ARRAY : null,
  BOTTOM_TOP_SIDE_2and4BENDS_ARRAY : null,
  BOTTOM_TOP_SIDE_4BENDS_ARRAY : null,
  BOTTOM_BOTTOM_SIDE_ARRAY : null,
  BOTTOM_LEFT_SIDE_ARRAY : null,
  BOTTOM_RIGHT_SIDE_ARRAY : null,
  LEFT_TOP_SIDE_ARRAY : null,
  LEFT_BOTTOM_SIDE_ARRAY : null,
  LEFT_LEFT_SIDE_ARRAY : null,
  LEFT_RIGHT_SIDE_2and4BENDS_ARRAY : null,
  LEFT_RIGHT_SIDE_4BENDS_ARRAY : null,
  RIGHT_TOP_SIDE_ARRAY : null,
  RIGHT_BOTTOM_SIDE_ARRAY : null,
  RIGHT_LEFT_SIDE_2and4BENDS_ARRAY : null,
  RIGHT_LEFT_SIDE_4BENDS_ARRAY : null,
  RIGHT_RIGHT_SIDE_ARRAY : null,

  TOP_LEFT_SIDE_OVERLAP_ARRAY : null,
  TOP_RIGHT_SIDE_OVERLAP_ARRAY : null,
  BOTTOM_LEFT_SIDE_OVERLAP_ARRAY : null,
  BOTTOM_RIGHT_SIDE_OVERLAP_ARRAY : null,
  LEFT_TOP_SIDE_OVERLAP_ARRAY : null,
  LEFT_BOTTOM_SIDE_OVERLAP_ARRAY : null,
  RIGHT_TOP_SIDE_OVERLAP_ARRAY : null,
  RIGHT_BOTTOM_SIDE_OVERLAP_ARRAY : null,

  ALL_QUASI_SELF_INTERGRAPH_LINK_TYPES : null,
  _fields_init : function(c)
  {
    c = c || _OrthogonalLinkShapeProducer;
    if(_LinkShapeProducer._fields_init)
      _LinkShapeProducer._fields_init(c);
    
    c.MAX_NUMBER_OF_BENDS = 4;
    c.TOP_TOP_SIDE_ARRAY = [_LinkShapeType.TwoBendTT.instance, _LinkShapeType.FourBendTTleft.instance, _LinkShapeType.FourBendTTright.instance ];
    c.TOP_BOTTOM_SIDE_2and4BENDS_ARRAY = [_LinkShapeType.TwoBendTB.instance, _LinkShapeType.FourBendTBright.instance, _LinkShapeType.FourBendTBleft.instance ];
    c.TOP_BOTTOM_SIDE_4BENDS_ARRAY = [_LinkShapeType.FourBendTBright.instance, _LinkShapeType.FourBendTBleft.instance ];
    c.TOP_LEFT_SIDE_ARRAY = [_LinkShapeType.OneBendTL.instance, _LinkShapeType.ThreeBendTL.instance ];
    c.TOP_RIGHT_SIDE_ARRAY = [_LinkShapeType.OneBendTR.instance, _LinkShapeType.ThreeBendTR.instance ];
    c.BOTTOM_TOP_SIDE_2and4BENDS_ARRAY = [_LinkShapeType.TwoBendBT.instance, _LinkShapeType.FourBendBTright.instance, _LinkShapeType.FourBendBTleft.instance ];
    c.BOTTOM_TOP_SIDE_4BENDS_ARRAY = [_LinkShapeType.FourBendBTright.instance, _LinkShapeType.FourBendBTleft.instance ];
    c.BOTTOM_BOTTOM_SIDE_ARRAY = [_LinkShapeType.TwoBendBB.instance, _LinkShapeType.FourBendBBleft.instance, _LinkShapeType.FourBendBBright.instance ];
    c.BOTTOM_LEFT_SIDE_ARRAY = [_LinkShapeType.OneBendBL.instance, _LinkShapeType.ThreeBendBL.instance ];
    c.BOTTOM_RIGHT_SIDE_ARRAY = [_LinkShapeType.OneBendBR.instance, _LinkShapeType.ThreeBendBR.instance ];
    c.LEFT_TOP_SIDE_ARRAY = [_LinkShapeType.OneBendLT.instance, _LinkShapeType.ThreeBendLT.instance ];
    c.LEFT_BOTTOM_SIDE_ARRAY = [_LinkShapeType.OneBendLB.instance, _LinkShapeType.ThreeBendLB.instance ];
    c.LEFT_LEFT_SIDE_ARRAY = [_LinkShapeType.TwoBendLL.instance, _LinkShapeType.FourBendLLleft.instance, _LinkShapeType.FourBendLLright.instance ];
    c.LEFT_RIGHT_SIDE_2and4BENDS_ARRAY = [_LinkShapeType.TwoBendLR.instance, _LinkShapeType.FourBendLRmiddle.instance, _LinkShapeType.FourBendLRtop.instance, _LinkShapeType.FourBendLRbottom.instance ];
    c.LEFT_RIGHT_SIDE_4BENDS_ARRAY = [_LinkShapeType.FourBendLRmiddle.instance, _LinkShapeType.FourBendLRtop.instance, _LinkShapeType.FourBendLRbottom.instance ];
    c.RIGHT_TOP_SIDE_ARRAY = [_LinkShapeType.OneBendRT.instance, _LinkShapeType.ThreeBendRT.instance ];
    c.RIGHT_BOTTOM_SIDE_ARRAY = [_LinkShapeType.OneBendRB.instance, _LinkShapeType.ThreeBendRB.instance ];
    c.RIGHT_LEFT_SIDE_2and4BENDS_ARRAY = [_LinkShapeType.TwoBendRL.instance, _LinkShapeType.FourBendRLmiddle.instance, _LinkShapeType.FourBendRLtop.instance, _LinkShapeType.FourBendRLbottom.instance ];
    c.RIGHT_LEFT_SIDE_4BENDS_ARRAY = [_LinkShapeType.FourBendRLmiddle.instance, _LinkShapeType.FourBendRLtop.instance, _LinkShapeType.FourBendRLbottom.instance ];
    c.RIGHT_RIGHT_SIDE_ARRAY = [_LinkShapeType.TwoBendRR.instance, _LinkShapeType.FourBendRRleft.instance, _LinkShapeType.FourBendRRright.instance ];
    c.TOP_LEFT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendTL_OVERLAP.instance ];
    c.TOP_RIGHT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendTR_OVERLAP.instance ];
    c.BOTTOM_LEFT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendBL_OVERLAP.instance ];
    c.BOTTOM_RIGHT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendBR_OVERLAP.instance ];
    c.LEFT_TOP_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendLT_OVERLAP.instance ];
    c.LEFT_BOTTOM_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendLB_OVERLAP.instance ];
    c.RIGHT_TOP_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendRT_OVERLAP.instance ];
    c.RIGHT_BOTTOM_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendRB_OVERLAP.instance ];
    c.ALL_QUASI_SELF_INTERGRAPH_LINK_TYPES = 
    [
      
      _LinkShapeType.OneBendTL.instance,
      _LinkShapeType.OneBendTR.instance,
      _LinkShapeType.OneBendBL.instance,
      _LinkShapeType.OneBendBR.instance,
      _LinkShapeType.OneBendLB.instance,
      _LinkShapeType.OneBendLT.instance,
      _LinkShapeType.OneBendRB.instance,
      _LinkShapeType.OneBendRT.instance,
      _LinkShapeType.TwoBendLL.instance,
      _LinkShapeType.TwoBendRR.instance,
      _LinkShapeType.TwoBendTT.instance,
      _LinkShapeType.TwoBendBB.instance,
      _LinkShapeType.TwoBendBT.instance,
      _LinkShapeType.TwoBendTB.instance,
      _LinkShapeType.TwoBendRL.instance,
      _LinkShapeType.TwoBendLR.instance,
      _LinkShapeType.ThreeBendLT.instance,
      _LinkShapeType.ThreeBendLB.instance,
      _LinkShapeType.ThreeBendRB.instance,
      _LinkShapeType.ThreeBendRT.instance,
      _LinkShapeType.ThreeBendTL.instance,
      _LinkShapeType.ThreeBendTR.instance,
      _LinkShapeType.ThreeBendBR.instance,
      _LinkShapeType.ThreeBendBL.instance
     ];
  }
  
}
);

(function(){ _OrthogonalLinkShapeProducer._fields_init(); })();

return _OrthogonalLinkShapeProducer;
});

},
'ibm_ilog/graphlayout/shortlink/_SLLinkData':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_SLLinkData", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/Point",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/ReshapeModes",	// #9
  "../internalutil/TranslateUtil",	// #10
  "../shortlink/_LinkShape",	// #11
  "../shortlink/_SLBundlesModes",	// #12
  "../shortlink/_SLNodeOrLinkData"	// #13
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    Point,	// #7
    Rectangle,	// #8
    ReshapeModes,	// #9
    TranslateUtil,	// #10
    _LinkShape,	// #11
    _SLBundlesModes,	// #12
    _SLNodeOrLinkData	// #13
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _SLNodeOrLinkData = ibm_ilog.graphlayout.shortlink._SLNodeOrLinkData;
=====*/

var _SLLinkData = declare("ibm_ilog.graphlayout.shortlink._SLLinkData", _SLNodeOrLinkData, 
{
  // summary:
  //		An object to store data about links. 
  // description:
  //		An object to store data about links. 
  //		The LinkData objects are stored in a property (temporarily) attached
  //		to the origin links.   
  // tags:
  //		private
  
  _intersectingObjects: null,

  _linkIndex : null,
  _linkShape: null,

  _linkWidth: null,
  _fromNodeData: null,
  _toNodeData: null,
  _fromNodeSide: null,
  _toNodeSide: null,

  _masterLink: null,

  _flags: null,

  _shapeTypes: null,

  _currentShapeType: null,

  constructor : function(layout, link, firstSlaveLink )
  {
    this._linkIndex = _SLLinkData.INITIAL_INDEX;
    this.update(layout, link, firstSlaveLink);
  },
  
  preamble:function(){
      return [arguments[1]];
  },
  isMaster : function()
  {
    // summary:
    //		Returns true if this is a master link. 
    // description:
    //		Returns true if this is a master link. 
    //		For the base class, returns always false.
    // tags:
    //		private
    
    return false; // Boolean
  },

  update : function(layout, link, firstSlaveLink)
  {
    // summary:
    //		Must be called before any use of an instance inside a layout
    //		session.
    
    var graphModel = layout.getGraphModel();

    this._linkWidth = graphModel.getLinkWidth(this.nodeOrLink);
    var incrementalMode = layout.isIncrementalMode();
    var linkOffset = layout.getLinkOffset();
    var fixed = layout.isFixedOrNoReshape(link);
    var fromPointFixed = layout.isFromPointFixed(link);
    var toPointFixed = layout.isToPointFixed(link);

    this.setOverlapping(false);
    this.setQuasiSelfInterGraphLinkOrigin(false);
    this.setQuasiSelfInterGraphLinkDestination(false);

    if (incrementalMode)
    {
      var fromNodeSide = this.getFromNodeSide();
      var toNodeSide = this.getToNodeSide();

      if (fromNodeSide == null || toNodeSide == null)
      {
        var masterLinkData = this.getMasterLink();

        if (masterLinkData != null)
        {
          fromNodeSide = masterLinkData.getFromNodeSide();
          toNodeSide = masterLinkData.getToNodeSide();
        }
      }

      this._fromNodeSide = fromNodeSide;
      this._toNodeSide = toNodeSide;
    }

    this.cleanMasterSlaveInfo(linkOffset, firstSlaveLink);
    this.setFixed(fixed);
    this.setFromPointFixed(fromPointFixed);
    this.setToPointFixed(toPointFixed);

    this.setFromSideFixed(false);
    this.setToSideFixed(false);

    this.setShapeTypeFixed(false);
    this.setAddedInFromNodeSide(false);
    this.setAddedInToNodeSide(false);
    this.shapePointsModified();

    if (incrementalMode)
    {
      this.updateIncrementalLinkReshapeRelatedData(layout);

      if (this.isShapeTypeFixed() && this._linkShape != null)
      {
        this.updateNodeSides(true, false);

        this.updateInterGraphLinkInfo(layout);
        return;
      }
    }

    fixed = this.isFixed();
    fromPointFixed = this.isFromPointFixed();
    toPointFixed = this.isToPointFixed();

    if (!this.isShapeTypeFixed())
    {
      this._currentShapeType = null;

      this.shapeTypeModified();
    }

    var fromNode = graphModel.getFrom(this.nodeOrLink);
    var toNode = graphModel.getTo(this.nodeOrLink);
    this._fromNodeData = layout.getOrCreateNodeData(graphModel, fromNode, false);
    this._toNodeData = layout.getOrCreateNodeData(graphModel, toNode, false);
    var fromRect = this._fromNodeData.getLinkConnectionRect();
    var toRect = this._toNodeData.getLinkConnectionRect();

    var fromPoint = ((fixed || fromPointFixed) ? graphModel.getLinkPointAt(this.nodeOrLink, 0) : new Point(fromRect.x + fromRect.width * 0.5, fromRect.y + fromRect.height * 0.5));
    var toPoint = ((fixed || toPointFixed)
      ? graphModel.getLinkPointAt(this.nodeOrLink, graphModel.getLinkPoints(this.nodeOrLink).length - 1)
      : new Point(toRect.x + toRect.width * 0.5, toRect.y + toRect.height * 0.5));

    this.updateInterGraphLinkInfo(layout);

    if (fixed)
    {
      this.boundingBox = GraphModelUtil.getObjectBounds(graphModel, this.nodeOrLink);

      this._linkShape = new _LinkShape(layout, this, fromPoint, toPoint);
    }
    else
    {
      this._linkShape = new _LinkShape(layout.getLinkShapeProducer(link).getMaxNumberOfBends());

      this._linkShape.setFrom(fromPoint);
      this._linkShape.setTo(toPoint);

      this.updateNodeSides(incrementalMode, false);
    }
  },

  updateIncrementalLinkReshapeRelatedData : function(layout)
  {
    // summary:
    //		WARNING: To be called only in incremental mode.
    // tags:
    //		private
    
    if (!this.hasBeenAlreadyLaidOut()) {
      return;
    }

    var linkReshapeMode = (this.hasSameEndNodeBoxesAsPreviously() ? layout.getIncrementalUnmodifiedLinkReshapeMode(this.nodeOrLink) : layout.getIncrementalModifiedLinkReshapeMode(this.nodeOrLink));

    switch (linkReshapeMode)
    {
      case ReshapeModes.FIXED_SHAPE_TYPE_MODE:
        if (this._currentShapeType != null)
        {
          this.setIndex(/*Number.MAX_VALUE*/2147483647);

          this.setShapeTypeFixed(true);
        }

        break;
      case ReshapeModes.FREE_MODE:
        break;
      case ReshapeModes.FIXED_NODE_SIDES_MODE:
        this.setFromSideFixed(true);
        this.setToSideFixed(true);
        break;
      case ReshapeModes.FIXED_CONNECTION_POINTS_MODE:
        this.setFromPointFixed(true);
        this.setToPointFixed(true);
        break;
      case ReshapeModes.FIXED_MODE:
        this.setFixed(true);
        break;
      default:
        var params = [linkReshapeMode ];
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6604E", params);
    }
  },

  copyIncrementalData : function(linkData)
  {
    // summary:
    //		Copies from linkData information needed for incremental layout.
    // tags:
    //		protected
    
    this._currentShapeType = linkData.getCurrentShapeType();
    this._fromNodeData = linkData.getFromNode();
    this._toNodeData = linkData.getToNode();
    this._fromNodeSide = linkData.getNodeSide(true);
    this._toNodeSide = linkData.getNodeSide(false);
    if (this._fromNodeSide != null) {
      this._fromNodeSide.clean();
    }
    if (this._toNodeSide != null) {
      this._toNodeSide.clean();
    }
  },

  updateNodeSides : function(incrementalMode, sameShapeForMultipleLinks)
  {
    if (!incrementalMode || (sameShapeForMultipleLinks && this.isSlave())) {
      return;
    }

    var nodeSide = this.getFromNodeSide();

    if (nodeSide != null)
    {
      nodeSide.addLink(this, true);
      this.setAddedInFromNodeSide(true);
    }

    if (!this.isSameSideSelfLink())
    {
      nodeSide = this.getToNodeSide();

      if (nodeSide != null)
      {
        nodeSide.addLink(this, false);
        this.setAddedInToNodeSide(true);
      }
    }
  },

  cleanMasterSlaveInfo : function(linkOffset, firstSlaveLink)
  {
    this.setSlave(false);
    this._masterLink = null;
  },

  updateInterGraphLinkInfo : function(layout)
  {
    this.setInterGraphLink(false);
    this.setQuasiSelfInterGraphLinkOrigin(false);
    this.setQuasiSelfInterGraphLinkDestination(false);
    var intergraphModel = layout.getInterGraphLinksRoutingModel();

    if (intergraphModel != null)
    {
      var isInterGraphLink = layout.getOriginalGraphModel().isIntergraphLink(intergraphModel.getOriginal(this.nodeOrLink));
      this.setInterGraphLink(isInterGraphLink);

      if (isInterGraphLink)
      {
        if (intergraphModel.nodeInsideGraph(this._fromNodeData.nodeOrLink, this._toNodeData.nodeOrLink)) {
          this.setQuasiSelfInterGraphLinkDestination(true);
        }
        else if (intergraphModel.nodeInsideGraph(this._toNodeData.nodeOrLink, this._fromNodeData.nodeOrLink)) {
          this.setQuasiSelfInterGraphLinkOrigin(true);
        }
      }
    }
  },

  hasBeenAlreadyLaidOut : function()
  {
    // summary:
    //		Returns true if the link has been laid out in the previous run. 
    // description:
    //		Returns true if the link has been laid out in the previous run. 
    //		In fact, it checks whether the necessary info for handling the link
    //		incrementally has been stored.
    // tags:
    //		private
    
    return (this._fromNodeSide != null && this._toNodeSide != null && this._fromNodeData != null && this._toNodeData != null); // Boolean
  },

  hasSameEndNodeBoxesAsPreviously : function()
  {
    // summary:
    //		Tells if the bbox of the origin and destination nodes is the same as
    //		in the previous layout. 
    // description:
    //		Tells if the bbox of the origin and destination nodes is the same as
    //		in the previous layout. 
    //		WARNING: call it only if canKeepCurrentShapeType returns true!!
    // tags:
    //		private
    
    return this._fromNodeData.hasSameBoxesAsPreviously() && this._toNodeData.hasSameBoxesAsPreviously(); // Boolean
  },

  setNodeSides : function (a0, a1, a2)
  {
    if(arguments.length == 2)
      return this.setNodeSides_overload_2(a0, a1);
    else if(arguments.length == 3)
      return this.setNodeSides_overload_3(a0, a1, a2);
  },
  
  setNodeSides_overload_2 : function(fromNodeSide, toNodeSide)
  {
    this.setNodeSides(fromNodeSide, toNodeSide, false);
  },

  setNodeSides_overload_3 : function(fromNodeSide, toNodeSide, forceAddition

)
  {
    if (fromNodeSide == null || toNodeSide == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6605E");
    }

    var addedInFromNodeSide = this.isAddedInFromNodeSide();
    var addedInToNodeSide = this.isAddedInToNodeSide();
    if ((fromNodeSide == this._fromNodeSide && toNodeSide == this._toNodeSide && !forceAddition) && (addedInFromNodeSide && addedInToNodeSide)) {
      return;
    }
    if (this.isSlave())
    {
      this._fromNodeSide = fromNodeSide;
      this._toNodeSide = toNodeSide;

      return;
    }

    var somethingChanged = false;
    var wasSameSideSelfLink = false;
    var isNowSameSideSelfLink = false;

    if (this.isSelfLink())
    {
      if (this._fromNodeSide == this._toNodeSide) {
        wasSameSideSelfLink = true;
      }
      if (fromNodeSide == toNodeSide) {
        isNowSameSideSelfLink = true;
      }
    }

    if (this._fromNodeSide != fromNodeSide || !addedInFromNodeSide)
    {
      if (this._fromNodeSide != null) {
        this._fromNodeSide.removeLink(this, true);
      }
      this._fromNodeSide = fromNodeSide;

      this._fromNodeSide.addLink(this, true);
      this.setAddedInFromNodeSide(true);
      somethingChanged = true;
    }

    if (this._toNodeSide != toNodeSide || !addedInToNodeSide)
    {
      if (this._toNodeSide != null && !wasSameSideSelfLink) {
        this._toNodeSide.removeLink(this, false);
      }
      this._toNodeSide = toNodeSide;

      if (!isNowSameSideSelfLink) {
        this._toNodeSide.addLink(this, false);
      }
      this.setAddedInToNodeSide(true);
      somethingChanged = true;
    }

    if (somethingChanged)
    {
      if (this._linkShape != null) {
        this._linkShape.invalidateBBox();
      }
      this.setShapeTypeUpToDate(false);
    }
  },

  getFromNodeSide : function()
  {
    return this._fromNodeSide; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function()
  {
    return this._toNodeSide; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  computeAllowedShapes : function(layout, shapeProducer)
  {
    // summary:
    //		Computes the set of shape types that are allowed for this link. 
    // description:
    //		Computes the set of shape types that are allowed for this link. 
    //		Called only once per layout.
    // tags:
    //		private
    
    var types = shapeProducer.getAllowedLinkShapes(layout, this);

    if (types == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6606E");
    }

    this.setShapeTypes(types);
  },

  computeLinkRegion : function(minFinalSegmentLength, linkOffset, linkBundlesMode)
  {
    // summary:
    //		Computes the link region. 
    // description:
    //		Computes the link region. 
    //		When computing the "link region", the bbox of the origin and
    //		destination nodes is enlarged in order to take into account the fact
    //		that the link may go outside this bbox with an amount which depends
    //		on the minFinalSegmentLength value, but also on the number of links
    //		that may be connected on a node side, the bundling policy etc. To be
    //		able to compute the link region BEFORE knowing the final shape of
    //		all the links, we take a rough estimation, which can be
    //		overestimated but should never be underestimated. Notice that
    //		overestimating the link region means computing more link-to-link and
    //		link-to-node intersection (that is, computing some unnecessary
    //		intersections), while underestimating means eventually decreasing
    //		the quality of the layout, because some link-to-link or link-to-node
    //		intersections
    //		won't be taken into account. The choice is made to overestimate. Notice also that this method uses infos stored on the node (total width of incident links) which is <b>not</b> available at the creation time of the link data. This is why this method is not called from the constructor.
    // tags:
    //		private
    
    if (this.boundingBox == null) {
      this.boundingBox = new Rectangle();
    }
    var fromRect = this.getFromNode().getLinkConnectionRect();
    var toRect = this.getToNode().getLinkConnectionRect();

    var linkRegion = this.boundingBox;

    var xMin = (fromRect.x <= toRect.x ? fromRect.x : toRect.x);
    var yMin = (fromRect.y <= toRect.y ? fromRect.y : toRect.y);
    var xMax = ((fromRect.x + fromRect.width) >= (toRect.x + toRect.width) ? fromRect.x + fromRect.width : toRect.x + toRect.width);
    var yMax = ((fromRect.y + fromRect.height) >= (toRect.y + toRect.height) ? fromRect.y + fromRect.height : toRect.y + toRect.height);

    if (this.isFromPointFixed())
    {
      var fromPoint = this.getConnectionPoint(true);
      if (fromPoint.x < xMin) {
        xMin = fromPoint.x;
      }
      else if (fromPoint.x > xMax) {
        xMax = fromPoint.x;
      }
      if (fromPoint.y < yMin) {
        yMin = fromPoint.y;
      }
      else if (fromPoint.y > yMax) {
        yMax = fromPoint.y;
      }
    }

    if (this.isToPointFixed())
    {
      var toPoint = this.getConnectionPoint(false);
      if (toPoint.x < xMin) {
        xMin = toPoint.x;
      }
      else if (toPoint.x > xMax) {
        xMax = toPoint.x;
      }
      if (toPoint.y < yMin) {
        yMin = toPoint.y;
      }
      else if (toPoint.y > yMax) {
        yMax = toPoint.y;
      }
    }

    var fromNode = this.getFromNode();
    var toNode = this.getToNode();

    var additional = minFinalSegmentLength;

    if (linkBundlesMode != _SLBundlesModes.NO_BUNDLE)
    {
      var additionalFrom = fromNode.getTotalIncidentLinkWidth() + (fromNode.getDegree() - 1) * linkOffset;
      var additionalTo = toNode.getTotalIncidentLinkWidth() + (toNode.getDegree() - 1) * linkOffset;
      additional += ((additionalFrom >= additionalTo ? additionalFrom : additionalTo));
    }

    if (additional != 0)
    {
      xMin -= additional;
      yMin -= additional;
      xMax += additional;
      yMax += additional;
    }

    TranslateUtil.reshapeRect(linkRegion, xMin, yMin, xMax - xMin, yMax - yMin);
  },

  isNode : function()
  {
    // summary:
    //		Returns true if this is a node, false otherwise.
    // tags:
    //		private
    
    return false; // Boolean
  },

  isLink : function()
  {
    // summary:
    //		Returns true if this is a link, false otherwise.
    // tags:
    //		private
    
    return true; // Boolean
  },

  getLinkWidth : function()
  {
    return this._linkWidth; // Number
  },

  getLinkWidthWithOffset : function(linkOffset )
  {
    // summary:
    //		In "sameShapeForMultipleLinks" mode, if this link is a bundle, the
    //		method returns the total width of the bundle, for a given link
    //		offset. 
    // description:
    //		In "sameShapeForMultipleLinks" mode, if this link is a bundle, the
    //		method returns the total width of the bundle, for a given link
    //		offset. 
    //		Otherwise, returns the normal width.  
    // tags:
    //		private
    
    return this._linkWidth; // Number
  },

  getLinkShape : function()
  {
    return this._linkShape; // ibm_ilog.graphlayout.shortlink._LinkShape
  },

  getFromNode : function()
  {
    return this._fromNodeData; // ibm_ilog.graphlayout.shortlink._SLNodeData
  },

  getToNode : function()
  {
    return this._toNodeData; // ibm_ilog.graphlayout.shortlink._SLNodeData
  },

  setOverlapping : function(overlapping)
  {
    if (overlapping) {
      this._flags |= _SLLinkData.OVERLAPPING_NODES;
    }
    else {
      this._flags &= ~_SLLinkData.OVERLAPPING_NODES;
    }
  },

  isOverlapping : function()
  {
    // summary:
    //		Returns true if the link connects overlapping nodes. 
    // description:
    //		Returns true if the link connects overlapping nodes. 
    //		More exactly, if the rectangles defined by the link connection rect,
    //		enlarged with the corresponding fixed connection point if any,
    //		overlap. Returns false otherwise.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.OVERLAPPING_NODES) != 0; // Boolean
  },

  setInterGraphLink : function(s)
  {
    if (s) {
      this._flags |= _SLLinkData.INTERGRAPH_LINK;
    }
    else {
      this._flags &= ~_SLLinkData.INTERGRAPH_LINK;
    }
  },

  isInterGraphLink : function()
  {
    // summary:
    //		Returns true if the link is an intergraph link, and false otherwise.
    //		
    // description:
    //		Returns true if the link is an intergraph link, and false otherwise.
    //		
    //		We use this flag in CostComputer, to avoid calling
    //		SubgraphData.from/toEndsInside again and again for normal links.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.INTERGRAPH_LINK) != 0; // Boolean
  },

  setQuasiSelfInterGraphLinkOrigin : function(s)
  {
    if (s) {
      this._flags |= _SLLinkData.QUASI_SELF_INTERGRAPH_LINK_ORIGIN;
    }
    else {
      this._flags &= ~_SLLinkData.QUASI_SELF_INTERGRAPH_LINK_ORIGIN;
    }
  },

  isQuasiSelfInterGraphLinkOrigin : function()
  {
    // summary:
    //		Returns true if the link goes from some node or subgraph to one of
    //		the parents graphs of the origin node. 
    // description:
    //		Returns true if the link goes from some node or subgraph to one of
    //		the parents graphs of the origin node. 
    //		Returns false otherwise.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.QUASI_SELF_INTERGRAPH_LINK_ORIGIN) != 0; // Boolean
  },

  setQuasiSelfInterGraphLinkDestination : function(s)
  {
    if (s) {
      this._flags |= _SLLinkData.QUASI_SELF_INTERGRAPH_LINK_DESTINATION;
    }
    else {
      this._flags &= ~_SLLinkData.QUASI_SELF_INTERGRAPH_LINK_DESTINATION;
    }
  },

  isQuasiSelfInterGraphLinkDestination : function()
  {
    // summary:
    //		Returns true if the link goes from a parent graph of the destination
    //		node to a destination node which is a node or a subgraph contained
    //		(directly or indirectly) in the graph which is the destination node.
    //		
    // description:
    //		Returns true if the link goes from a parent graph of the destination
    //		node to a destination node which is a node or a subgraph contained
    //		(directly or indirectly) in the graph which is the destination node.
    //		
    //		Returns false otherwise.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.QUASI_SELF_INTERGRAPH_LINK_DESTINATION) != 0; // Boolean
  },

  setFromPointFixed : function(fixed)
  {
    if (fixed) {
      this._flags |= _SLLinkData.FROM_POINT_FIXED;
    }
    else {
      this._flags &= ~_SLLinkData.FROM_POINT_FIXED;
    }
  },

  isFromPointFixed : function()
  {
    // summary:
    //		Returns true if the from point of the link is fixed, false
    //		otherwise.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.FROM_POINT_FIXED) != 0; // Boolean
  },

  setToPointFixed : function(fixed)
  {
    if (fixed) {
      this._flags |= _SLLinkData.TO_POINT_FIXED;
    }
    else {
      this._flags &= ~_SLLinkData.TO_POINT_FIXED;
    }
  },

  isToPointFixed : function()
  {
    return (this._flags & _SLLinkData.TO_POINT_FIXED) != 0; // Boolean
  },

  hasFixedConnectionPoint : function()
  {
    return this.isFixed() || this.isFromPointFixed() || this.isToPointFixed(); // Boolean
  },

  setShapeTypeFixed : function(fixed)
  {
    if (fixed) {
      this._flags |= _SLLinkData.SHAPE_TYPE_FIXED;
    }
    else {
      this._flags &= ~_SLLinkData.SHAPE_TYPE_FIXED;
    }
  },

  isShapeTypeFixed : function()
  {
    // summary:
    //		Returns true if the from point of the link is fixed, false
    //		otherwise.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.SHAPE_TYPE_FIXED) != 0; // Boolean
  },

  setFromSideFixed : function(fixed)
  {
    if (fixed) {
      this._flags |= _SLLinkData.FROM_SIDE_FIXED;
    }
    else {
      this._flags &= ~_SLLinkData.FROM_SIDE_FIXED;
    }
  },

  isFromSideFixed : function()
  {
    // summary:
    //		Returns true if the from side of the link is fixed, false otherwise.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.FROM_SIDE_FIXED) != 0; // Boolean
  },

  setToSideFixed : function(fixed)
  {
    if (fixed) {
      this._flags |= _SLLinkData.TO_SIDE_FIXED;
    }
    else {
      this._flags &= ~_SLLinkData.TO_SIDE_FIXED;
    }
  },

  isToSideFixed : function()
  {
    return (this._flags & _SLLinkData.TO_SIDE_FIXED) != 0; // Boolean
  },

  setAddedInFromNodeSide : function(added)
  {
    if (added) {
      this._flags |= _SLLinkData.ADDED_IN_FROM_NODE_SIDE;
    }
    else {
      this._flags &= ~_SLLinkData.ADDED_IN_FROM_NODE_SIDE;
    }
  },

  isAddedInFromNodeSide : function()
  {
    return (this._flags & _SLLinkData.ADDED_IN_FROM_NODE_SIDE) != 0; // Boolean
  },

  setAddedInToNodeSide : function(added)
  {
    if (added) {
      this._flags |= _SLLinkData.ADDED_IN_TO_NODE_SIDE;
    }
    else {
      this._flags &= ~_SLLinkData.ADDED_IN_TO_NODE_SIDE;
    }
  },

  isAddedInToNodeSide : function()
  {
    return (this._flags & _SLLinkData.ADDED_IN_TO_NODE_SIDE) != 0; // Boolean
  },

  setSlave : function(slave)
  {
    // summary:
    //		Sets this links as a slave link. 
    // description:
    //		Sets this links as a slave link. 
    //		(For multilinks.)
    // tags:
    //		protected
    
    if (slave) {
      this._flags |= _SLLinkData.IS_SLAVE;
    }
    else {
      this._flags &= ~_SLLinkData.IS_SLAVE;
    }
  },

  isSlave : function()
  {
    // summary:
    //		Returns true if this is a slave link. 
    // description:
    //		Returns true if this is a slave link. 
    //		(For multilinks.)
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.IS_SLAVE) != 0; // Boolean
  },

  setFixed : function(fixed)
  {
    // summary:
    //		Sets this link as a fixed or as a non-fixed link. 
    // description:
    //		Sets this link as a fixed or as a non-fixed link. 
    //		(Notice that a fixed link has necessarily its connection points
    //		fixed, while a non-fixed link may have its connection points fixed
    //		or non-fixed.
    // tags:
    //		private
    
    this._linkIndex = (fixed ? _SLLinkData.FIXED_LINK_INDEX : _SLLinkData.INITIAL_INDEX);
  },

  isFixed : function()
  {
    // summary:
    //		Returns true if this is a fixed link.
    // tags:
    //		private
    
    return this._linkIndex == _SLLinkData.FIXED_LINK_INDEX; // Boolean
  },

  isOrthogonal : function()
  {
    // summary:
    //		Returns true if the shape of the link is known to be orthogonal,
    //		false otherwise.
    // tags:
    //		private
    
    if (this._linkShape == null || this._currentShapeType == null) {
      return false; // Boolean
    }
    return this._currentShapeType.isOrthogonal(); // Boolean
  },

  setIndex : function(index)
  {
    // summary:
    //		Sets the index.
    // tags:
    //		private
    
    this._linkIndex = index;
  },

  getIndex : function()
  {
    // summary:
    //		Returns the index.
    // tags:
    //		private
    
    return this._linkIndex; // int
  },

  setShapeTypes : function(shapeTypes)
  {
    this._shapeTypes = shapeTypes;
    if (shapeTypes != null) {
      this.setCurrentShapeType(shapeTypes[0]);
    }
    this.setShapeTypeUpToDate(false);
  },

  getShapeTypes : function()
  {
    return this._shapeTypes; // Array
  },

  getShapeType : function(index)
  {
    if (this._shapeTypes == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6607E");
    }

    if (index >= this._shapeTypes.length)
    {
      var params = [index, this._shapeTypes.length ];
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6608E", params);
    }

    return this._shapeTypes[index]; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getShapeTypesCount : function()
  {
    return (this._shapeTypes != null ? this._shapeTypes.length : 0); // int
  },

  setCurrentShapeType : function(shapeType)
  {
    if (this._currentShapeType != shapeType)
    {
      this._currentShapeType = shapeType;
      if (this._linkShape != null) {
        this._linkShape.invalidateBBox();
      }
      this.setShapeTypeUpToDate(false);
    }
  },

  getCurrentShapeType : function()
  {
    return this._currentShapeType; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  prepareForBundleLayout : function(masterLinkData, copyPointLocations)
  {
    // summary:
    //		Copies the shape from the master link. 
    // description:
    //		Copies the shape from the master link. 
    //		(For multilinks.)
    // tags:
    //		private
    
    var reversed = masterLinkData.getFromNode() != this._fromNodeData;

    if (this._linkShape == null) {
      this._linkShape = new _LinkShape(masterLinkData.getLinkShape(), reversed, copyPointLocations);
    }
    else {
      this._linkShape.copyFrom(masterLinkData.getLinkShape(), reversed, copyPointLocations);
    }
    this._linkIndex = masterLinkData.getIndex();
    this._currentShapeType = masterLinkData.getCurrentShapeType();
    this._shapeTypes = masterLinkData.getShapeTypes();
    var fromNodeSide;
    var toNodeSide;

    if (reversed)
    {
      this._currentShapeType = this._currentShapeType.getReversedShapeType();
      fromNodeSide = masterLinkData.getToNodeSide();
      toNodeSide = masterLinkData.getFromNodeSide();
    }
    else
    {
      fromNodeSide = masterLinkData.getFromNodeSide();
      toNodeSide = masterLinkData.getToNodeSide();
    }

    this.setNodeSides(fromNodeSide, toNodeSide);
    this.setShapeTypeUpToDate(false);
  },

  isGoingLeft : function(origin)
  {
    // summary:
    //		Returns true if the link has left orientation wrt to its
    //		origin/destination, and false otherwise.
    // tags:
    //		private
    
    var nPoints = this._linkShape.getNumberOfPoints();

    if (nPoints < 2)
    {
      var params = [nPoints ];
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6609E", params);
    }

    var index1;
    var index2;

    if (nPoints >= 3)
    {
      if (origin)
      {
        index1 = 1;
        index2 = 2;
      }
      else
      {
        index1 = nPoints - 2;
        index2 = nPoints - 3;
      }
    }
    else
    {
      if (origin)
      {
        index1 = 0;
        index2 = 1;
      }
      else
      {
        index1 = nPoints - 1;
        index2 = nPoints - 2;
      }
    }

    var point1 = this.getPointAt(origin, index1);
    var point2 = this.getPointAt(origin, index2);
    var nodeSide = this.getNodeSide(origin);
    return nodeSide.getTangentDelta(point1, point2) >= 0; // Boolean
  },

  applyOffset : function(masterFromPoint, masterToPoint, masterBendPoints, nBends, offset, offsetOrig, offsetDest, goingLeftOrigin, reversed, onlyFirstAndLastSeg)
  {
    // summary:
    //		Applies an offset to the points of this link, such that the points
    //		are located at a given offset to 'bendPoints'.  
    // masterBendPoints: Array
    //		the master array of points
    // nBends: int
    //		the number of points (starting from index 0) to which we
    //		want to apply the offset
    // offset: Number
    //		the value of the offset
    // goingLeftOrigin: Boolean
    //		if true, the offset is applied on the left side
    //		of the points, looking from the first point. Otherwise, the offset
    //		is applied on the right side.
    // reversed: Boolean
    //		if true, the master has reversed orientation wrt this
    //		link
    // onlyFirstAndLastSeg: Boolean
    //		If true, only the first and last segments are
    //		processed
    // tags:
    //		private
    
    if (masterBendPoints != null)
    {
      var lnkNumberOfPoints = this._linkShape.getNumberOfPoints();

      if (lnkNumberOfPoints != (nBends + 2))
      {
        var params1 = [lnkNumberOfPoints, (nBends + 2) ];
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6610E", params1);
      }

      if (masterBendPoints.length < nBends)
      {
        var params2 = [masterBendPoints.length, nBends ];
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6611E", params2);
      }
    }
    else {
      nBends = this._linkShape.getNumberOfPoints() - 2;
    }
    var onLeftSide = goingLeftOrigin;

    var origin = !reversed;
    var orthogonalShape = this._currentShapeType.isOrthogonal();
    var firstPoint = this.getPointAt(origin, 0);
    var secondPoint = this.getPointAt(origin, 1);
    var beforeLastPoint = this.getPointAt(origin, nBends);
    var lastPoint = this.getPointAt(origin, nBends + 1);
    var secondMasterPoint = null;
    var beforeLastMasterPoint = null;

    if (masterFromPoint != null)
    {
      TranslateUtil.setPointLocation(firstPoint, masterFromPoint);
      beforeLastMasterPoint = masterBendPoints[nBends - 1];
      TranslateUtil.setPointLocation(beforeLastPoint, beforeLastMasterPoint);
      secondMasterPoint = masterBendPoints[0];
    }

    _SLLinkData.applyOrthogonalOffset(firstPoint, secondPoint, masterFromPoint, secondMasterPoint, offsetOrig, onLeftSide);

    if (orthogonalShape && !onlyFirstAndLastSeg)
    {
      var i;

      if (masterBendPoints != null) {
        for (i = 1; i < nBends; i++)
          _SLLinkData.applyOrthogonalOffset(this.getPointAt(origin, i), this.getPointAt(origin, i + 1), masterBendPoints[i - 1], masterBendPoints[i], offset, onLeftSide);
      }
      else {
        for (i = 1; i < nBends; i++)
          _SLLinkData.applyOrthogonalOffset(this.getPointAt(origin, i), this.getPointAt(origin, i + 1), null, null, offset, onLeftSide);
      }
    }

    _SLLinkData.applyOrthogonalOffset(beforeLastPoint, lastPoint, beforeLastMasterPoint, masterToPoint, offsetDest, onLeftSide);

    if (!orthogonalShape)
    {
      _SLLinkData.applyDirectOffsetCorrection(firstPoint, secondPoint, beforeLastPoint, secondMasterPoint, this.getNodeSide(origin), offsetOrig);
      _SLLinkData.applyDirectOffsetCorrection(lastPoint, beforeLastPoint, secondPoint, beforeLastMasterPoint, this.getNodeSide(!origin), offsetDest);
    }

    this.shapePointsModified();
  },

  setMasterLink : function(masterLink)
  {
    this._masterLink = masterLink;
  },

  getMasterLink : function()
  {
    return this._masterLink; // ibm_ilog.graphlayout.shortlink._SLLinkData
  },

  getNumberOfIndividualLinks : function()
  {
    // summary:
    //		Returns the number of slave links of this link. 
    // description:
    //		Returns the number of slave links of this link. 
    //		The default implementation returns always 1. Overridden in
    //		MultiLinkData.
    // tags:
    //		private
    
    return 1; // int
  },

  setOriginLinkOffset : function(offset)
  {
    // summary:
    //		The default implementation does nothing. 
    // description:
    //		The default implementation does nothing. 
    //		Overridden in MultiLinkData. Having it on the base class allows us
    //		to avoid casts to MultiLinkData.
    // tags:
    //		private
    
  },

  setDestinationLinkOffset : function(offset)
  {
    // summary:
    //		The default implementation does nothing. 
    // description:
    //		The default implementation does nothing. 
    //		Overridden in MultiLinkData. Having it on the base class allows us
    //		to avoid casts to MultiLinkData.
    // tags:
    //		private
    
  },

  getNodeSide : function(origin)
  {
    // summary:
    //		Returns the node side to which the link is connected, either at
    //		origin, or at destination.
    // tags:
    //		private
    
    return (origin ? this._fromNodeSide : this._toNodeSide); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getSide : function(origin)
  {
    // summary:
    //		Returns the direction of the node side to which the link is
    //		connected at the origin or the destination.
    // tags:
    //		private
    
    return this.getNodeSide(origin).getSide(); // int
  },

  getOppositeSide : function(origin)
  {
    // summary:
    //		Returns the direction that represents the side on the extremity of
    //		the link which is opposite to 'nodeData'. 
    // description:
    //		Returns the direction that represents the side on the extremity of
    //		the link which is opposite to <code>nodeData</code>. 
    //		If the link is a self-link, nodeSide is used to determine the
    //		orientation.
    // tags:
    //		private
    
    return this.getNodeSide(!origin).getSide(); // int
  },

  getConnectionNode : function(origin)
  {
    // summary:
    //		Returns the node to which the link is connected, either at origin or
    //		at destination.
    // tags:
    //		private
    
    return (origin ? this._fromNodeData : this._toNodeData); // ibm_ilog.graphlayout.shortlink._SLNodeData
  },

  copyPointLocations : function(points, origin)
  {
    var limit = this._linkShape.getNumberOfPoints();

    for (var i = 0; i < limit; i++)
      TranslateUtil.setPointLocation(points[i], this.getPointAt(origin, i));
  },

  setShapeTypeUpToDate : function(uptodate)
  {
    if (uptodate) {
      this._flags |= _SLLinkData.SHAPE_TYPE_UP_TO_DATE;
    }
    else
    {
      this._flags &= ~_SLLinkData.SHAPE_TYPE_UP_TO_DATE;
      this.setShapePointsUpToDate(false);
    }
  },

  isShapeTypeUpToDate : function()
  {
    // summary:
    //		Returns true if the type of shape of this link (may) have been
    //		changed since the last time '
    //		_SLLinkData.restoreCurrentShape()' has been called. 
    // description:
    //		Returns true if the type of shape of this link (may) have been
    //		changed since the last time <code>
    //		&#95;SLLinkData.restoreCurrentShape()</code> has been called. 
    //		The change of the type of shape basically implies the (eventual)
    //		change of the node side to which the link connects.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.SHAPE_TYPE_UP_TO_DATE) != 0; // Boolean
  },

  setShapePointsUpToDate : function(uptodate)
  {
    if (uptodate) {
      this._flags |= _SLLinkData.SHAPE_POINTS_UP_TO_DATE;
    }
    else {
      this._flags &= ~_SLLinkData.SHAPE_POINTS_UP_TO_DATE;
    }
  },

  isShapePointsUpToDate : function()
  {
    // summary:
    //		Returns true if the points of this link (may) have been moved since
    //		the last time ' _SLLinkData.restoreCurrentShape()' has
    //		been called.
    // tags:
    //		private
    
    return (this._flags & _SLLinkData.SHAPE_POINTS_UP_TO_DATE) != 0; // Boolean
  },

  shapePointsModified : function()
  {
    // summary:
    //		Tells the LinkData that its LinkShape has been modified wrt. 
    // description:
    //		Tells the LinkData that its LinkShape has been modified wrt. 
    //		the last restored shape.
    // tags:
    //		private
    
    this.setShapePointsUpToDate(false);
    if (this._linkShape != null) {
      this._linkShape.invalidateBBox();
    }
  },

  shapeTypeModified : function()
  {
    // summary:
    //		Tells the LinkData that its LinkShape has been modified wrt. 
    // description:
    //		Tells the LinkData that its LinkShape has been modified wrt. 
    //		the last restored shape.
    // tags:
    //		private
    
    this.setShapeTypeUpToDate(false);
  },

  restoreCurrentShape : function(layout, minFinalSegment, linkOffset, reversed)
  {
    // summary:
    //		Restores the points of the shape according to the currently selected
    //		shape type.
    
    if (this.isFixed()) {
      return;
    }
    if (this.isSlave())
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6613E");
    }

    if (this._currentShapeType == null) {
      this.computeAllowedShapes(layout, layout.getLinkShapeProducer(this.nodeOrLink));
    }

    var shapeType = (reversed ? this._currentShapeType.getReversedShapeType() : this._currentShapeType);

    if (!this.isShapeTypeUpToDate())
    {
      this.setNodeSides(shapeType.getFromNodeSide(this), shapeType.getToNodeSide(this));
      this.setShapeTypeUpToDate(true);
    }

    if (!this.isShapePointsUpToDate())
    {
      shapeType.computeIntermediatePoints(layout, this, minFinalSegment, reversed);

      this.setShapePointsUpToDate(true);
    }
  },

  getPointAt : function(origin, index)
  {
    // summary:
    //		Returns the point at a given index, starting either from the origin,
    //		or from the destination. 
    // description:
    //		Returns the point at a given index, starting either from the origin,
    //		or from the destination. 
    //		Index 0 = connection point on origin/dest node Index 1 = next point
    //		... Index n-1 = connection point on dest/orig node
    // tags:
    //		private
    
    return this._linkShape.getPointAtFromEndPoint(origin, index); // Point
  },

  getConnectionPoint : function(origin)
  {
    // summary:
    //		Returns the connection point either at the origin, or at the
    //		destination.
    // tags:
    //		private
    
    return (origin ? this._linkShape.getFrom() : this._linkShape.getTo()); // Point
  },

  isConnectionPointFixed : function(origin)
  {
    // summary:
    //		Returns true if the connection point on the origin or destination
    //		node is fixed.
    // tags:
    //		private
    
    return this.isFixed() || ((origin ? this.isFromPointFixed() : this.isToPointFixed())); // Boolean
  },

  isOrigin : function (a0, a1)
  {
    // summary:
    //		Given one of the end-nodes (argument 'nodeData', returns
    //		true if this end-node is the origin node and false otherwise. 
    // description:
    //		Given one of the end-nodes (argument <code>nodeData</code>, returns
    //		true if this end-node is the origin node and false otherwise. 
    //		If the link is a self-link, it uses nodeSide (that is, the
    //		connection side on the end-node) to discriminate. WARNING: nodeData
    //		and nodeSide MUST correspond to one of the end-nodes. Otherwise, the
    //		result is undetermined.
    // tags:
    //		private
    
    if(arguments.length == 2)
      return this.isOrigin_overload_2(a0, a1);
    else if(arguments.length == 1)
      return this.isOrigin_overload_1(a0);
  },
  
  isOrigin_overload_2 : function(nodeData, nodeSide)
                                                    
  {
    return (this.isSelfLink() ? this._fromNodeSide == nodeSide : this._fromNodeData == nodeData); // Boolean
  },

  isOrigin_overload_1 : function(nodeSide)
  {
    if (this._fromNodeData == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6614E");
    }

    return this._fromNodeSide == nodeSide; // Boolean
  },

  getIntersectingObjects : function()
  {
    // summary:
    //		Returns the nodes and links that this link may intersect.
    // tags:
    //		private
    
    return this._intersectingObjects; // Array
  },

  setIntersectingObjects : function(vect)
  {
    // summary:
    //		Sets the nodes and links that this link may intersect. 
    // description:
    //		Sets the nodes and links that this link may intersect. 
    //		You can pass <code>null</code>.
    // tags:
    //		private
    
    this._intersectingObjects = vect;
  },

  isIntersectingObjectValid : function(linkData)
  {
    // summary:
    //		Returns true if the intersection between this LinkData and another
    //		LinkData has to be taken into consideration. 
    // description:
    //		Returns true if the intersection between this LinkData and another
    //		LinkData has to be taken into consideration. 
    //		Returns false otherwise.
    // tags:
    //		private
    
    if (this.nodeOrLink != null && this.nodeOrLink == linkData.nodeOrLink) {
      return false; // Boolean
    }
    if (linkData == this)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6615E");
    }

    return true; // Boolean
  },

  isSelfLink : function()
  {
    // summary:
    //		Returns true if this link is a self-link, false otherwise.
    // tags:
    //		private
    
    return this._fromNodeData.nodeOrLink == this._toNodeData.nodeOrLink; // Boolean
  },

  isSameSideSelfLink : function()
  {
    // summary:
    //		Returns true if this link is a self-link with both origin and
    //		destination currently connected to the same side (of the same node).
    //		
    // description:
    //		Returns true if this link is a self-link with both origin and
    //		destination currently connected to the same side (of the same node).
    //		
    //		Returns false otherwise.
    // tags:
    //		private
    
    return this._fromNodeData.nodeOrLink == this._toNodeData.nodeOrLink && this._fromNodeSide == this._toNodeSide; // Boolean
  },

  getSideOfPoint : function(point, origin)
  {
    // summary:
    //		Returns the node side to which the connection point of a link
    //		incident to to this node (either as origin or as destination) is the
    //		closest.
    // tags:
    //		private
    
    return (origin ? this._fromNodeData.getSideOfPoint(point, this.isQuasiSelfInterGraphLinkOrigin()) : this._toNodeData.getSideOfPoint(point, this.isQuasiSelfInterGraphLinkDestination())); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_SLLinkData,
{
  INITIAL_INDEX : null,
  FIXED_LINK_INDEX : null,
  FROM_POINT_FIXED : null,
  TO_POINT_FIXED : null,
  FROM_SIDE_FIXED : null,
  TO_SIDE_FIXED : null,
  SHAPE_TYPE_FIXED : null,
  IS_SLAVE : null,
  SHAPE_TYPE_UP_TO_DATE : null,
  SHAPE_POINTS_UP_TO_DATE : null,
  ADDED_IN_FROM_NODE_SIDE : null,
  ADDED_IN_TO_NODE_SIDE : null,
  OVERLAPPING_NODES : null,

  INTERGRAPH_LINK : null,
  QUASI_SELF_INTERGRAPH_LINK_ORIGIN : null,
  QUASI_SELF_INTERGRAPH_LINK_DESTINATION : null,

  applyOrthogonalOffset : function(point1, point2, masterPoint1, masterPoint2, offset, onLeftSide)
  {
    var dw = 0;
    var dh = 0;

    if (masterPoint1 != null)
    {
      dw = masterPoint2.x - masterPoint1.x;
      dh = masterPoint2.y - masterPoint1.y;
    }
    else
    {
      dw = point2.x - point1.x;
      dh = point2.y - point1.y;
    }

    if (dw == 0)
    {
      if (dh == 0) {
        TranslateUtil.setPointLocation(point2, point1);
      }
      else
      {
        if ((onLeftSide && dh <= 0) || (!onLeftSide && dh > 0)) {
          offset = -offset;
        }
        if (masterPoint1 == null)
        {
          point1.x += offset;
          point2.x += offset;
        }
        else
        {
          point1.x = (masterPoint1.x + offset);
          point2.x = (masterPoint2.x + offset);
          point2.y = masterPoint2.y;
        }
      }
    }
    else if (dh == 0)
    {
      if ((onLeftSide && dw >= 0) || (!onLeftSide && dw < 0)) {
        offset = -offset;
      }
      if (masterPoint1 == null)
      {
        point1.y += offset;
        point2.y += offset;
      }
      else
      {
        point1.y = (masterPoint1.y + offset);
        point2.y = (masterPoint2.y + offset);
        point2.x = masterPoint2.x;
      }
    }
    else
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6612E");
    }
  },

  applyDirectOffsetCorrection : function(point0, point1, point2, masterPoint1, nodeSide, offset)
  {
    if (offset < 0) {
      offset = -offset;
    }
    var tanPoint1 = nodeSide.getTangentialCoord(point1);
    var tanDelta = nodeSide.getTangentialCoord(point2) - tanPoint1;
    var radDelta = nodeSide.getPointRadialCoord(point2) - nodeSide.getPointRadialCoord(point1);
    if (tanDelta == 0 && radDelta == 0) {
      return;
    }
    var length = Math.sqrt(tanDelta * tanDelta + radDelta * radDelta);
    if (length < 0.0000001) {
      return;
    }
    var k = offset / length;
    var deltaRadial = k * tanDelta;

    var tanMasterPoint1 = nodeSide.getTangentialCoord(masterPoint1);
    if (tanPoint1 > tanMasterPoint1) {
      deltaRadial = -deltaRadial;
    }
    nodeSide.translateOutside(point1, deltaRadial);
  },

  _fields_init : function(c)
  {
    c = c || _SLLinkData;
    if(_SLNodeOrLinkData._fields_init)
      _SLNodeOrLinkData._fields_init(c);
    
    c.INITIAL_INDEX = -1;
    c.FIXED_LINK_INDEX = -2;
    c.FROM_POINT_FIXED = 1 << 0;
    c.TO_POINT_FIXED = 1 << 1;
    c.FROM_SIDE_FIXED = 1 << 2;
    c.TO_SIDE_FIXED = 1 << 3;
    c.SHAPE_TYPE_FIXED = 1 << 4;
    c.IS_SLAVE = 1 << 5;
    c.SHAPE_TYPE_UP_TO_DATE = 1 << 6;
    c.SHAPE_POINTS_UP_TO_DATE = 1 << 7;
    c.ADDED_IN_FROM_NODE_SIDE = 1 << 8;
    c.ADDED_IN_TO_NODE_SIDE = 1 << 9;
    c.OVERLAPPING_NODES = 1 << 10;
    c.INTERGRAPH_LINK = 1 << 11;
    c.QUASI_SELF_INTERGRAPH_LINK_ORIGIN = 1 << 12;
    c.QUASI_SELF_INTERGRAPH_LINK_DESTINATION = 1 << 13;
  }
  
}
);

(function(){ _SLLinkData._fields_init(); })();

return _SLLinkData;
});

},
'ibm_ilog/graphlayout/longlink/LLIncrementalNode':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLIncrementalNode", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Point",	// #6
  "../internalutil/QuickSort",	// #7
  "../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Point,	// #6
    QuickSort,	// #7
    TranslateUtil	// #8
    ) {

var LLIncrementalNode = declare("ibm_ilog.graphlayout.longlink.LLIncrementalNode", null, 
{
  // summary:
  //		Data for nodes when using incremental mode. 
  // description:
  //		Data for nodes when using incremental mode. 
  //		We need this data in order to figure out which links have changed and
  //		must be rerouted. The data is updated when incremental is switched
  //		on, and after layout is performed when incremental is already on.  
  // tags:
  //		private
  
  _nodeObject: null,

  _oldRect: null,
  _newRect: null,

  _topLinks: null,
  _bottomLinks: null,
  _leftLinks: null,
  _rightLinks: null,

  constructor : function(layout, node)
  {
    this._nodeObject = node;
    this._topLinks = new Array();
    this._bottomLinks = new Array();
    this._leftLinks = new Array();
    this._rightLinks = new Array();
    this.update(layout);
  },
  
  update : function(layout)
  {
    // summary:
    //		Updates the link points that define the link.
    // tags:
    //		private
    
    this._oldRect = layout.getNodeBoxForConnection(this._nodeObject);
    this._newRect = null;
  },

  updateLink : function(link, p, fromSide, isHorizontal)
  {
    // summary:
    //		Updates the information about an incident link.
    // tags:
    //		private
    
    if (isHorizontal)
    {
      if (p.x >= this._oldRect.x + 0.5 * this._oldRect.width) {
        this._rightLinks.push(new LLIncrementalNode.LinkPointData(link, p.y, fromSide));
      }
      else {
        this._leftLinks.push(new LLIncrementalNode.LinkPointData(link, p.y, fromSide));
      }
    }
    else
    {
      if (p.y >= this._oldRect.y + 0.5 * this._oldRect.height) {
        this._bottomLinks.push(new LLIncrementalNode.LinkPointData(link, p.x, fromSide));
      }
      else {
        this._topLinks.push(new LLIncrementalNode.LinkPointData(link, p.x, fromSide));
      }
    }
  },

  sortLinks : function(v)
  {
    // summary:
    //		Sorts the vector of link point
    //		data's according to the data coordinate.
    // tags:
    //		private
    
    if (v.length > 1)
    {
      var sortAlg = new LLIncrementalNode.LinkPointDataSortAlg(v);
      sortAlg.sort(v.length);
      sortAlg.dispose();
    }
  },

  reassignLinkCoords : function(v, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist)
  {
    // summary:
    //		Reassign link termination coordinates for the links in the link
    //		point data vector. 
    // description:
    //		Reassign link termination coordinates for the links in the link
    //		point data vector. 
    //		The range for the links at the side of the node is minCoord to
    //		maxCoord. Aligned to the grid, it is minGridCoord to maxGridCoord.
    //		If there are enough grid lines available, it chooses the termination
    //		points at the grid lines. Otherwise, it divides the available range
    //		evenly and chooses termination points away from the grid lines.
    // tags:
    //		private
    
    var sz = v.length;
    if (sz == 0) {
      return;
    }

    var coord;
    var delta;

    if ((sz - 1) * gridDist <= maxGridCoord - minGridCoord)
    {
      var k = Math.floor(Math.floor((maxGridCoord - minGridCoord) / gridDist + 0.5));
      k = Math.floor((k - (sz - 1)) / 2);
      coord = minGridCoord + k * gridDist;
      delta = gridDist;
    }
    else
    {
      coord = minCoord;
      delta = (maxCoord - minCoord) / (sz - 1);
    }

    var e =  new TranslateUtil.ArrayIterator(v);
    var d;

    while (e.hasNext())
    {
      d = e.next();
      d._coord = coord;
      coord += delta;
    }
  },

  setNewTerminationPoint : function(lpdata, topBottom)
  {
    // summary:
    //		Sets the input point as new termination point to the link.
    // tags:
    //		private
    
    var x;
    var y;
    var link = lpdata._link;

    if (lpdata._fromSide)
    {
      x = link.getFromPoint().x;
      y = link.getFromPoint().y;
    }
    else
    {
      x = link.getToPoint().x;
      y = link.getToPoint().y;
    }

    var perc;

    if (topBottom)
    {
      perc = (y - this._oldRect.y - 0.5 * this._oldRect.height) / this._oldRect.height;
      y = this._newRect.y + 0.5 * this._newRect.height + perc * this._newRect.height;
      x = lpdata._coord;
    }
    else
    {
      perc = (x - this._oldRect.x - 0.5 * this._oldRect.width) / this._oldRect.width;
      x = this._newRect.x + 0.5 * this._newRect.width + perc * this._newRect.width;
      y = lpdata._coord;
    }

    if (lpdata._fromSide) {
      link.setNewFromPoint(new Point(x, y));
    }
    else {
      link.setNewToPoint(new Point(x, y));
    }
  },

  setNewTerminationPoints : function(v, topBottom)
  {
    // summary:
    //		Sets all input point as new termination point to the links in the
    //		link point data vector.
    // tags:
    //		private
    
    var e =  new TranslateUtil.ArrayIterator(v);
    var d;

    while (e.hasNext())
    {
      d = e.next();
      this.setNewTerminationPoint(d, topBottom);
    }
  },

  prepare : function(layout, grid)
  {
    // summary:
    //		Prepares the incremental node data before the next layout.
    // tags:
    //		private
    
    if (this._newRect == null)
    {
      this._newRect = layout.getNodeBoxForConnection(this._nodeObject);

      if (layout.isIncrementalConnectionPreserving() && this.needsReroute())
      {
        this.sortLinks(this._rightLinks);
        this.sortLinks(this._leftLinks);
        this.sortLinks(this._topLinks);
        this.sortLinks(this._bottomLinks);
        var minCOffs = layout.getMinNodeCornerOffset();
        var minCoord = this._newRect.y + minCOffs;
        var maxCoord = this._newRect.y + this._newRect.height - minCOffs;
        var minGridIndex = grid.getMinGridIndexAtSide(this._newRect, 1, minCOffs);
        var maxGridIndex = grid.getMaxGridIndexAtSide(this._newRect, 1, minCOffs);
        var minGridCoord = grid.getGridCoord(1, minGridIndex);
        var maxGridCoord = grid.getGridCoord(1, maxGridIndex);
        var gridDist = grid.getGridDist(1);
        this.reassignLinkCoords(this._rightLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        this.reassignLinkCoords(this._leftLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        minCoord = this._newRect.x + minCOffs;
        maxCoord = this._newRect.x + this._newRect.width - minCOffs;
        minGridIndex = grid.getMinGridIndexAtSide(this._newRect, 0, minCOffs);
        maxGridIndex = grid.getMaxGridIndexAtSide(this._newRect, 0, minCOffs);
        minGridCoord = grid.getGridCoord(0, minGridIndex);
        maxGridCoord = grid.getGridCoord(0, maxGridIndex);
        gridDist = grid.getGridDist(0);
        this.reassignLinkCoords(this._topLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        this.reassignLinkCoords(this._bottomLinks, minCoord, maxCoord, minGridCoord, maxGridCoord, gridDist);
        this.setNewTerminationPoints(this._rightLinks, false);
        this.setNewTerminationPoints(this._leftLinks, false);
        this.setNewTerminationPoints(this._topLinks, true);
        this.setNewTerminationPoints(this._bottomLinks, true);
      }
    }
  },

  needsReroute : function()
  {
    // summary:
    //		Returns 'true' if all incident links must be rerouted
    //		because the node moved.
    // tags:
    //		private
    
    if (this._oldRect == null) {
      return true; // Boolean
    }

    var delta;
    delta = this._newRect.x - this._oldRect.x;
    if (delta != 0) {
      return true; // Boolean
    }
    delta = this._newRect.y - this._oldRect.y;
    if (delta != 0) {
      return true; // Boolean
    }
    delta = this._newRect.width - this._oldRect.width;
    if (delta != 0) {
      return true; // Boolean
    }
    delta = this._newRect.height - this._oldRect.height;
    if (delta != 0) {
      return true; // Boolean
    }
    return (false); // Boolean
  },

  getNodeObject : function()
  {
    // summary:
    //		Returns the corresponding node.
    // tags:
    //		private
    
    return this._nodeObject; // Object
  },

  getOldRect : function()
  {
    // summary:
    //		Returns the bounding rectangle from the previous layout.
    // tags:
    //		private
    
    return this._oldRect; // Rectangle
  }

}
);

LLIncrementalNode.LinkPointData = declare( null, 
{
  // summary:
  //		Inner auxiliary class to combine a link with its termination
  //		coordinate.
  
  _link: null,
  _coord: null,
  _fromSide: null,

  constructor : function(link, coord, fromSide)
  {
    this._link = link;
    this._coord = coord;
    this._fromSide = fromSide;
  }
  
}
);

LLIncrementalNode.LinkPointDataSortAlg = declare(QuickSort, 
{
  // summary:
  //		Inner auxiliary class to sort the link point data vectors according
  //		to the coord.
  
  _v: null,

  constructor : function(v)
  {
    this._v = v;
  },
  
  dispose : function()
  {
  },

  compare : function(loc1, loc2)
  {
    var d1 = this._v[loc1];
    var d2 = this._v[loc2];
    if (d1._coord == d2._coord) {
      return 0; // int
    }
    else if (d1._coord < d2._coord) {
      return -1; // int
    }
    else {
      return 1; // int
    }
  },

  swap : function(loc1, loc2)
  {
    var d1 = this._v[loc1];
    var d2 = this._v[loc2];
    this._v[loc2] =   d1;
    this._v[loc1] =   d2;
  }

}
);

return LLIncrementalNode;
});

},
'ibm_ilog/graphlayout/layer_link':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm_ilog/graphlayout/shortlink/ShortLinkLayout,ibm_ilog/graphlayout/longlink/LongLinkLayout"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
dojo.provide('ibm_ilog.graphlayout.layer_link');

dojo.require('ibm_ilog.graphlayout.shortlink.ShortLinkLayout');
dojo.require('ibm_ilog.graphlayout.longlink.LongLinkLayout');

// The following modules will be recursively included by the modules above
//dojo.require('ibm_ilog.graphlayout.longlink.ILongLinkTerminationPointFilter');
//dojo.require('ibm_ilog.graphlayout.longlink.LLCrossPoint');
//dojo.require('ibm_ilog.graphlayout.longlink.LLCrossRedAlgorithm');
//dojo.require('ibm_ilog.graphlayout.longlink.LLDirectRouteAlgorithm');
//dojo.require('ibm_ilog.graphlayout.longlink.LLFallbackRouteAlgorithm');
//dojo.require('ibm_ilog.graphlayout.longlink.LLGraph');
//dojo.require('ibm_ilog.graphlayout.longlink.LLGrid');
//dojo.require('ibm_ilog.graphlayout.longlink.LLGridLine');
//dojo.require('ibm_ilog.graphlayout.longlink.LLGridSegment');
//dojo.require('ibm_ilog.graphlayout.longlink.LLIncrementalData');
//dojo.require('ibm_ilog.graphlayout.longlink.LLIncrementalLink');
//dojo.require('ibm_ilog.graphlayout.longlink.LLIncrementalNode');
//dojo.require('ibm_ilog.graphlayout.longlink.LLLink');
//dojo.require('ibm_ilog.graphlayout.longlink.LLLinkRouteAlgorithm');
//dojo.require('ibm_ilog.graphlayout.longlink.LLTerminationCandAlgorithm');
//dojo.require('ibm_ilog.graphlayout.longlink.LLTerminationPoint');
//dojo.require('ibm_ilog.graphlayout.longlink.LongLinkLayoutReport');
//dojo.require('ibm_ilog.graphlayout.shortlink._BestNeighborOptimization');
//dojo.require('ibm_ilog.graphlayout.shortlink._CombinatorialOptimization');
//dojo.require('ibm_ilog.graphlayout.shortlink._CostComputer');
//dojo.require('ibm_ilog.graphlayout.shortlink._DirectIncidentLinksRefiner');
//dojo.require('ibm_ilog.graphlayout.shortlink._DirectIncidentLinksSorter');
//dojo.require('ibm_ilog.graphlayout.shortlink._DirectLinkShapeProducer');
//dojo.require('ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner');
//dojo.require('ibm_ilog.graphlayout.shortlink._IncidentLinksSorter');
//dojo.require('ibm_ilog.graphlayout.shortlink._LinkLayoutBestNeighborOptimization');
//dojo.require('ibm_ilog.graphlayout.shortlink._LinkShape');
//dojo.require('ibm_ilog.graphlayout.shortlink._LinkShapeProducer');
//dojo.require('ibm_ilog.graphlayout.shortlink._LinkShapeType');
//dojo.require('ibm_ilog.graphlayout.shortlink._MultiLinkBundleLayouter');
//dojo.require('ibm_ilog.graphlayout.shortlink._NodeSideLayouter');
//dojo.require('ibm_ilog.graphlayout.shortlink._OrthogonalIncidentLinksRefiner');
//dojo.require('ibm_ilog.graphlayout.shortlink._OrthogonalIncidentLinksSorter');
//dojo.require('ibm_ilog.graphlayout.shortlink._OrthogonalLinkShapeProducer');
//dojo.require('ibm_ilog.graphlayout.shortlink._OrthoWith4BendsPositionArrays');
//dojo.require('ibm_ilog.graphlayout.shortlink._OrthoWithout4BendsPositionArrays');
//dojo.require('ibm_ilog.graphlayout.shortlink._ShortLinkAlgorithm');
//dojo.require('ibm_ilog.graphlayout.shortlink._SLLinkData');
//dojo.require('ibm_ilog.graphlayout.shortlink._SLMultiLinkData');
//dojo.require('ibm_ilog.graphlayout.shortlink._SLNodeData');
//dojo.require('ibm_ilog.graphlayout.shortlink._SLNodeOrLinkData');
//dojo.require('ibm_ilog.graphlayout.shortlink._SLNodeSide');
//dojo.require('ibm_ilog.graphlayout.shortlink._SLBundlesModes');

});

},
'ibm_ilog/graphlayout/shortlink/_OrthogonalIncidentLinksRefiner':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_OrthogonalIncidentLinksRefiner", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../shortlink/_IncidentLinksRefiner"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    TranslateUtil,	// #7
    _IncidentLinksRefiner	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _IncidentLinksRefiner = ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner;
=====*/

var _OrthogonalIncidentLinksRefiner = declare("ibm_ilog.graphlayout.shortlink._OrthogonalIncidentLinksRefiner", _IncidentLinksRefiner, 
{
  // summary:
  //		A subclass specialized for orthogonal link style. 
  // description:
  //		A subclass specialized for orthogonal link style. 
  //		
  // tags:
  //		private
  
  constructor : function(layout)
  {
  },
  
  refine : function(vectLinks, nodeData, nodeSide, linkOffset, minFinalSegmentLength, reversedOrder, withFixedConnectionPoints)
  {
    // summary:
    //		Final refinement of the link layout. 
    // description:
    //		Final refinement of the link layout. 
    //		Once the type of shape for each link and the order of the connection
    //		of all the links incident to a given side of a node are choosen, the
    //		second and, eventually, the third segment of an orthogonal link can
    //		be moved a little by this refinement procedure. The purpose is to
    //		avoid overlapping segments for links that are incident to the same
    //		side of the same node. (Currently, nothing is done for links that
    //		are not incident to the same side of the same node.)  
    //		<p>
    //		Note that, currently, this refinement procedure concerns only the
    //		orthogonal links style. In the future, something could be done for
    //		direct links also (placement of the first points of the links on an
    //		arc, as TGO does.)
    //		</p>
    // tags:
    //		private
    
    var nLinks = (vectLinks != null ? vectLinks.length : 0);
    if (nLinks < 2) {
      return;
    }
    var limit = nLinks - 1;
    var link1;
    var link2;
    var expectedOrientation;
    var stopIfDifferentOrientation = !withFixedConnectionPoints;

    for (var i = 0; i < limit; i++)
    {
      link1 = LayoutUtil.elementAt(vectLinks, i, nLinks, reversedOrder);
      link2 = LayoutUtil.elementAt(vectLinks, i + 1, nLinks, reversedOrder);
      if (link2.isFixed()) {
        continue;
      }
      expectedOrientation = this.refineLinksOrthogonal(link1, link2, link1.isOrigin(nodeData, nodeSide), link2.isOrigin(nodeData, nodeSide), nodeSide, true, linkOffset, stopIfDifferentOrientation);
      if (!expectedOrientation && stopIfDifferentOrientation) {
        break;
      }
    }

    for (var j = limit; j > 0; j--)
    {
      link1 = LayoutUtil.elementAt(vectLinks, j, nLinks, reversedOrder);
      link2 = LayoutUtil.elementAt(vectLinks, j - 1, nLinks, reversedOrder);
      if (link2.isFixed()) {
        continue;
      }
      expectedOrientation = this.refineLinksOrthogonal(link1, link2, link1.isOrigin(nodeData, nodeSide), link2.isOrigin(nodeData, nodeSide), nodeSide, false, linkOffset, stopIfDifferentOrientation);
      if (!expectedOrientation && stopIfDifferentOrientation) {
        break;
      }
    }
  }

}
);

return _OrthogonalIncidentLinksRefiner;
});

},
'ibm_ilog/graphlayout/shortlink/_CostComputer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_CostComputer", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/LogResUtil",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/TranslateUtil",	// #9
  "../shortlink/_SLBundlesModes"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    LogResUtil,	// #7
    Rectangle,	// #8
    TranslateUtil,	// #9
    _SLBundlesModes	// #10
    ) {

var _CostComputer = declare("ibm_ilog.graphlayout.shortlink._CostComputer", null, 
{
  // summary:
  //		This class is in charge of the computation of the cost of each
  //		solution. 
  // description:
  //		This class is in charge of the computation of the cost of each
  //		solution. 
  //		
  // tags:
  //		private
  
  _linkLinkPenalty: null,
  _linkNodePenalty: null,

  _linkBundlesMode: null,
  _tempRect1 : null,
  _tempRect2 : null,
  _tempRect3 : null,

  constructor : function()
  {
    this._tempRect1 = new Rectangle();
    this._tempRect2 = new Rectangle();
    this._tempRect3 = new Rectangle();
  },
  
  clean : function()
  {
  },

  update : function(layout)
  {
    // summary:
    //		Updates parameters from the current values on the layout.
    // tags:
    //		private
    
    var shortLayout = layout.getShortLinkLayout();
    this._linkLinkPenalty = shortLayout.getLinkToLinkCrossingPenalty();
    this._linkNodePenalty = shortLayout.getLinkToNodeCrossingPenalty();
    this._linkBundlesMode = shortLayout.getLinkBundlesMode();
  },

  getLinkCost : function (a0, a1, a2, a3, a4)
  {
    // summary:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    // description:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    //		Only nodes and links inside the link region are checked.  
    // global: Boolean
    //		true means that the method is called for every link and we
    //		are interested in the total number of crossings (cross betw two
    //		links is counted just once). If false, all the cossings for this
    //		link are counted. PACKAGE BECAUSE CALLED FROM ShortLinkAlgorithm
    // tags:
    //		private
    
    if(arguments.length == 4)
      return this.getLinkCost_overload_4(a0, a1, a2, a3);
    else if(arguments.length == 5)
      return this.getLinkCost_overload_5(a0, a1, a2, a3, a4);
  },
  
  getLinkCost_overload_4 : function(graphModel, interGraphLinksRoutingModel, linkData, global)
                                                                                              
  {
    return this.getLinkCost(graphModel, interGraphLinksRoutingModel, linkData, false, global); // Number
  },

  getLinkCost_overload_5 : function(graphModel, interGraphLinksRoutingModel, linkData, onlyLinkNodeOverlaps, global)
  {
    // summary:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    // description:
    //		Counts the crossings between link and the other links and nodes
    //		excepting extremity nodes of the link. 
    //		Only nodes and links inside the link region are checked.  
    // global: Boolean
    //		true means that the method is called for every link and we
    //		are interested in the total number of crossings (cross betw two
    //		links is counted just once). If false, all the crossings for this
    //		link are counted. PACKAGE BECAUSE CALLED FROM ShortLinkAlgorithm
    // tags:
    //		private
    
    if (linkData == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6601E");
    }

    var crossNode = 0;
    var crossLink = 0;

    crossNode += (this.getLinkAdjacentNodeCost(linkData, interGraphLinksRoutingModel, true) + this.getLinkAdjacentNodeCost(linkData, interGraphLinksRoutingModel, false));

    var intersectingObjects = linkData.getIntersectingObjects();
    var nObjects = (intersectingObjects == null ? 0 : intersectingObjects.length);

    if (nObjects > 0)
    {
      var index = linkData.getIndex();
      var nodeOrLinkData;
      var indexOtherLink;

      for (var i = 0; i < nObjects; i++)
      {
        nodeOrLinkData = intersectingObjects[i];
        indexOtherLink = 0;

        if (nodeOrLinkData.isNode()) {
          crossNode += this.getLinkNodeCost(interGraphLinksRoutingModel, linkData, nodeOrLinkData);
        }
        else if (!onlyLinkNodeOverlaps)
        {
          indexOtherLink = (nodeOrLinkData).getIndex();

          if (!global || (index < indexOtherLink)) {
            crossLink += this.getLinkLinkCost(linkData, nodeOrLinkData);
          }
        }
      }
    }

    var cost = crossLink * this._linkLinkPenalty + crossNode * this._linkNodePenalty;

    if (cost != 0 && !onlyLinkNodeOverlaps)
    {
      var nSlaves = linkData.getNumberOfIndividualLinks();
      if (nSlaves > 1) {
        cost *= nSlaves;
      }
    }

    return cost; // Number
  },

  getLinkAdjacentNodeCost : function(linkData, interGraphLinksRoutingModel, origin)
  {
    // summary:
    //		Computes the cost of the crossings between linkData and its adjacent
    //		node on the origin/dest nodes.
    // tags:
    //		private
    
    var nodeData;

    if (interGraphLinksRoutingModel != null)
    {
      nodeData = linkData.getConnectionNode(origin);

      if (
        linkData.isInterGraphLink() &&
          (interGraphLinksRoutingModel.fromEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink) || interGraphLinksRoutingModel.toEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink)))
        {
          if ((origin ? linkData.isQuasiSelfInterGraphLinkOrigin() : linkData.isQuasiSelfInterGraphLinkDestination()))
          {
            TranslateUtil.reshapeFromRect(this._tempRect1, linkData.getLinkShape().boundingBox());

            return (TranslateUtil.rectContainsRect(linkData.getConnectionNode(origin).boundingBox, this._tempRect1) ? 0 : 1); // Number
          }
        }
    }

    var crossNode = 0;
    var nodeSide = linkData.getNodeSide(origin);
    var point0 = linkData.getPointAt(origin, 0);
    var point1 = linkData.getPointAt(origin, 1);

    if (nodeSide.getPointRadialDelta(point0, point1) < 0)
    {
      crossNode++;
    }
    else
    {
      nodeData = linkData.getConnectionNode(origin);
      var oppositeNodeData = linkData.getConnectionNode(!origin);
      if (linkData.isOverlapping() && (TranslateUtil.rectContainsRect(oppositeNodeData.boundingBox, nodeData.boundingBox) || TranslateUtil.rectContainsRect(nodeData.boundingBox,
            oppositeNodeData.boundingBox))) {
        return 0; // Number
      }
      var point2 = linkData.getPointAt(origin, 2);

      if (linkData.getCurrentShapeType().isOrthogonal())
      {
        var nBends = linkData.getCurrentShapeType().getNumberOfBends();

        if (linkData.isOverlapping() && nBends >= 1)
        {
          if (this.intersects(point0, point1, 0, oppositeNodeData.boundingBox, false, false)) {
            crossNode++;
          }
        }

        if (nBends >= 2)
        {
          var point3 = linkData.getPointAt(origin, 3);
          if (this.intersects(point2, point3, 0, nodeData.boundingBox, false, false)) {
            crossNode++;
          }
        }
      }
      else
      {
        if (this.intersects(point1, point2, 0, nodeData.boundingBox, false, false)) {
          crossNode++;
        }
        if (nodeSide.getPointRadialDelta(point1, point2) < 0)
        {
          crossNode += _CostComputer.SMALL_PENALTY;
        }
      }
    }

    return crossNode; // Number
  },

  getLinkNodeCost : function(interGraphLinksRoutingModel, linkData, nodeData)
  {
    // summary:
    //		Returns the cost associated to the crossings between the link
    //		'link' and the node 'node'. 
    // description:
    //		Returns the cost associated to the crossings between the link
    //		<code>link</code> and the node <code>node</code>. 
    //		The cost is 0 if there are no crossings. Note that the link is
    //		checked for intersection with the link connection rectangle of the
    //		node.  
    //		<p>
    //		Note that the cost is multiplied by the number of slave links in
    //		getLinkCost.
    //		</p>
    // tags:
    //		private
    
    var msg;

    if (linkData == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6601E");
    }

    if (interGraphLinksRoutingModel != null)
    {
      if (
        linkData.isInterGraphLink() &&
          (interGraphLinksRoutingModel.fromEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink) || interGraphLinksRoutingModel.toEndsInside(linkData.nodeOrLink, nodeData.nodeOrLink))) {
          return 0.0; // Number
      }
    }

    var shape = linkData.getLinkShape();

    if (shape == null)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6602E");
    }

    var bbox = nodeData.boundingBox;
    var halfWidth = linkData.getLinkWidth() * 0.5;
    TranslateUtil.reshapeFromRect(this._tempRect1, shape.boundingBox());
    if (halfWidth > 0) {
      TranslateUtil.inflateRect(this._tempRect1, halfWidth, halfWidth);
    }
    if (!_CostComputer.isOverlap(this._tempRect1, bbox)) {
      return 0; // Number
    }

    var limit = shape.getNumberOfPoints() - 2;

    for (var i = 0; i <= limit; i++)
    {
      if (this.intersects(shape.getPointAt(i), shape.getPointAt(i + 1), halfWidth, bbox, i == 0, i == limit)) {
        return 1; // Number
      }
    }

    return 0; // Number
  },

  getLinkLinkCost : function(linkData1, linkData2)
  {
    // summary:
    //		Computes the cost associated with eventual crossings betw. 
    // description:
    //		Computes the cost associated with eventual crossings betw. 
    //		2 links.
    // tags:
    //		private
    
    var shape1 = linkData1.getLinkShape();
    var shape2 = linkData2.getLinkShape();
    var halfWidth1 = linkData1.getLinkWidth() * 0.5;
    var halfWidth2 = linkData2.getLinkWidth() * 0.5;
    TranslateUtil.reshapeFromRect(this._tempRect1, shape1.boundingBox());
    TranslateUtil.reshapeFromRect(this._tempRect2, shape2.boundingBox());
    TranslateUtil.inflateRect(this._tempRect1, halfWidth1, halfWidth1);
    TranslateUtil.inflateRect(this._tempRect2, halfWidth2, halfWidth2);
    if (!_CostComputer.isOverlap(this._tempRect1, this._tempRect2)) {
      return 0; // Number
    }

    var limit1 = shape1.getNumberOfPoints() - 2;
    var limit2 = shape2.getNumberOfPoints() - 2;
    var from1;
    var to1;
    var from2;
    var to2;
    var bothLinksOrthogonal = linkData1.isOrthogonal() && linkData2.isOrthogonal();
    var cross = 0;
    var startIndex1 = 0;
    var startIndex2 = 0;

    if (this._linkBundlesMode == _SLBundlesModes.NO_BUNDLE)
    {
      var originPointOnSameNodeSide = linkData1.getFromNode() == linkData2.getFromNode() && linkData1.getFromNodeSide() == linkData2.getFromNodeSide() &&
        !((linkData1.isFromPointFixed() || linkData2.isFromPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(0), shape2.getPointAt(0)));

      var destinationPointOnSameNodeSide = linkData1.getToNode() == linkData2.getToNode() && linkData1.getToNodeSide() == linkData2.getToNodeSide() &&
        !((linkData1.isToPointFixed() || linkData2.isToPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(limit1 + 1), shape2.getPointAt(limit2 + 1)));

      var originPoint1OnSameNodeSideAsDestinationPoint2 = !originPointOnSameNodeSide && linkData1.getFromNode() == linkData2.getToNode() && linkData1.getFromNodeSide() == linkData2.getToNodeSide() &&
        !((linkData1.isFromPointFixed() || linkData2.isToPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(0), shape2.getPointAt(limit2 + 1)));

      var destinationPoint1OnSameNodeSideAsOriginPoint2 = !destinationPointOnSameNodeSide && linkData1.getToNode() == linkData2.getFromNode() &&
        linkData1.getToNodeSide() == linkData2.getFromNodeSide() && !((linkData1.isToPointFixed() || linkData2.isFromPointFixed()) && !TranslateUtil.pointEquals(shape1.getPointAt(limit1 + 1)
          , shape2.getPointAt(0)));

      if (
        !
          ((originPointOnSameNodeSide && destinationPointOnSameNodeSide) ||
              (originPoint1OnSameNodeSideAsDestinationPoint2 && destinationPoint1OnSameNodeSideAsOriginPoint2) && (limit1 <= 1 && limit2 <= 1)))
        {
          if (originPointOnSameNodeSide)
          {
            startIndex1 = 1;
            startIndex2 = 1;
          }
          else if (originPoint1OnSameNodeSideAsDestinationPoint2)
          {
            startIndex1 = 1;
            limit2--;
          }

          if (destinationPointOnSameNodeSide)
          {
            limit1--;
            limit2--;
          }
          else if (destinationPoint1OnSameNodeSideAsOriginPoint2)
          {
            limit1--;
            startIndex2 = 1;
          }
        }
    }

    for (var i = startIndex1; i <= limit1 && cross == 0; i++)
    {
      from1 = shape1.getPointAt(i);
      to1 = shape1.getPointAt(i + 1);

      this.computeBounds(this._tempRect1, from1, to1, i == 0, i == limit1, halfWidth1);

      if (_CostComputer.isOverlap(this._tempRect1, this._tempRect2))
      {
        for (var j = startIndex2; j <= limit2 && cross == 0; j++)
        {
          from2 = shape2.getPointAt(j);
          to2 = shape2.getPointAt(j + 1);

          if (bothLinksOrthogonal)
          {
            this.computeBounds(this._tempRect3, from2, to2, j == 0, j == limit2, halfWidth2);
            if (_CostComputer.isOverlap(this._tempRect1, this._tempRect3)) {
              cross++;
            }
          }
          else if (LayoutUtil.intersects(from1.x, from1.y, to1.x, to1.y, from2.x, from2.y, to2.x, to2.y)) {
            cross++;
          }
        }
      }
    }

    return cross; // Number
  },

  computeBounds : function(rect, fromPoint, toPoint, isFromPointConnectionPoint, isToPointConnectionPoint, halfLineWidth)
  {
    // summary:
    //		Stores in 'rect' the bounds of a thick segment.
    // tags:
    //		private
    
    var width = toPoint.x - fromPoint.x;
    var height = toPoint.y - fromPoint.y;

    if (!isFromPointConnectionPoint && !isToPointConnectionPoint)
    {
      TranslateUtil.reshapeRect(rect, (fromPoint.x <= toPoint.x ? fromPoint.x : toPoint.x), (fromPoint.y <= toPoint.y ? fromPoint.y : toPoint.y),
        ((width >= 0 ? width : -width)), ((height >= 0 ? height : -height)));

      TranslateUtil.inflateRect(rect, halfLineWidth, halfLineWidth);
    }
    else
    {
      var xMin;
      var xMax;
      var yMin;
      var yMax;
      var horizontalSegment = ((height < 0 ? -height : height)) < ((width < 0 ? -width : width));

      if (width > 0)
      {
        xMin = fromPoint.x;
        xMax = toPoint.x;
        if (!horizontalSegment || !isFromPointConnectionPoint) {
          xMin -= halfLineWidth;
        }
        if (!horizontalSegment || !isToPointConnectionPoint) {
          xMax += halfLineWidth;
        }
      }
      else
      {
        xMin = toPoint.x;
        xMax = fromPoint.x;
        if (!horizontalSegment || !isToPointConnectionPoint) {
          xMin -= halfLineWidth;
        }
        if (!horizontalSegment || !isFromPointConnectionPoint) {
          xMax += halfLineWidth;
        }
      }

      var verticalSegment = !horizontalSegment;

      if (height > 0)
      {
        yMin = fromPoint.y;
        yMax = toPoint.y;
        if (!verticalSegment || !isFromPointConnectionPoint) {
          yMin -= halfLineWidth;
        }
        if (!verticalSegment || !isToPointConnectionPoint) {
          yMax += halfLineWidth;
        }
      }
      else
      {
        yMin = toPoint.y;
        yMax = fromPoint.y;
        if (!verticalSegment || !isToPointConnectionPoint) {
          yMin -= halfLineWidth;
        }
        if (!verticalSegment || !isFromPointConnectionPoint) {
          yMax += halfLineWidth;
        }
      }

      TranslateUtil.reshapeRect(rect, xMin, yMin, xMax - xMin, yMax - yMin);
    }
  },

  intersects : function(from, toobj, halfWidth, rect, firstPoint, lastPoint)
  {
    // summary:
    //		Checks whether Computes the number of crossings between the segment
    //		from-to and rect.
    // tags:
    //		private
    
    if (from.x != toobj.x && from.y != toobj.y)
    {
      var xMaxRect = rect.x + rect.width;
      var yMaxRect = rect.y + rect.height;

      if (from.x < rect.x && toobj.x < rect.x || from.x > xMaxRect && toobj.x > xMaxRect || from.y < rect.y && toobj.y < rect.y || from.y > yMaxRect && toobj.y > yMaxRect) {
        return false; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, rect.x, rect.y, xMaxRect, rect.y)) {
        return true; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, xMaxRect, rect.y, xMaxRect, yMaxRect)) {
        return true; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, xMaxRect, yMaxRect, rect.x, yMaxRect)) {
        return true; // Boolean
      }
      if (LayoutUtil.intersects(from.x, from.y, toobj.x, toobj.y, rect.x, yMaxRect, rect.x, rect.y)) {
        return true; // Boolean
      }
    }
    else
    {
      var width = toobj.x - from.x;
      var height = toobj.y - from.y;
      if (width == 0 && height == 0) {
        return false; // Boolean
      }
      if (width < 0) {
        width = -width;
      }
      if (height < 0) {
        height = -height;
      }
      if (halfWidth > 0)
      {
        if (firstPoint)
        {
          if (width == 0)
          {
            if (from.y <= toobj.y) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, from.y, 2 * halfWidth, height + halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, toobj.y - halfWidth, 2 * halfWidth, height + halfWidth);
            }
          }
          else
          {
            if (from.x <= toobj.x) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x, from.y - halfWidth, width + halfWidth, 2 * halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, toobj.x - halfWidth, toobj.y - halfWidth, width + halfWidth, 2 * halfWidth);
            }
          }
        }
        else if (lastPoint)
        {
          if (width == 0)
          {
            if (from.y <= toobj.y) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, from.y + halfWidth, 2 * halfWidth, height - halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, toobj.y, 2 * halfWidth, height - halfWidth);
            }
          }
          else
          {
            if (from.x <= toobj.x) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x + halfWidth, from.y - halfWidth, width - halfWidth, 2 * halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, toobj.x, toobj.y - halfWidth, width - halfWidth, 2 * halfWidth);
            }
          }
        }
        else
        {
          if (width == 0)
          {
            if (from.y <= toobj.y) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, from.y + halfWidth, 2 * halfWidth, height);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, from.x - halfWidth, toobj.y - halfWidth, 2 * halfWidth, height);
            }
          }
          else
          {
            if (from.x <= toobj.x) {
              TranslateUtil.reshapeRect(this._tempRect1, from.x + halfWidth, from.y - halfWidth, width, 2 * halfWidth);
            }
            else {
              TranslateUtil.reshapeRect(this._tempRect1, toobj.x - halfWidth, toobj.y - halfWidth, width, 2 * halfWidth);
            }
          }
        }
      }
      else
      {
        TranslateUtil.reshapeRect(this._tempRect1, (from.x <= toobj.x ? from.x : toobj.x), (from.y <= toobj.y ? from.y : toobj.y), width, height);
      }

      return _CostComputer.isOverlap(this._tempRect1, rect); // Boolean
    }

    return false; // Boolean
  }

}
);

lang.mixin(_CostComputer,
{
  SMALL_PENALTY : null,

  isOverlap : function(rect1, rect2)
  {
    return (rect1.x < rect2.x + rect2.width && rect2.x < rect1.x + rect1.width && rect1.y < rect2.y + rect2.height && rect2.y < rect1.y + rect1.height); // Boolean
  },

  _fields_init : function(c)
  {
    c = c || _CostComputer;
    
    c.SMALL_PENALTY = 0.01;
  }
  
}
);

(function(){ _CostComputer._fields_init(); })();

return _CostComputer;
});

},
'ibm_ilog/graphlayout/shortlink/_IncidentLinksSorter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_IncidentLinksSorter", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/QuickSort",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../shortlink/_SLBundlesModes",	// #8
  "../Direction"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    QuickSort,	// #6
    TranslateUtil,	// #7
    _SLBundlesModes,	// #8
    Direction	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var QuickSort = ibm_ilog.graphlayout.internalutil.QuickSort;
=====*/

var _IncidentLinksSorter = declare("ibm_ilog.graphlayout.shortlink._IncidentLinksSorter", QuickSort, 
{
  // summary:
  //		This class is used to sort the links incident to a node. 
  // description:
  //		This class is used to sort the links incident to a node. 
  //		A layout instance uses only one instance of class for each link
  //		style.   
  // tags:
  //		extension
  
  _vectLinks: null,

  _node: null,

  _side: null,

  _layout: null,

  constructor : function(layout)
  {
    this._layout = layout;
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the data stored by this class.
    // tags:
    //		private
    
    this._side = null;
    this._vectLinks = null;
    this._node = null;
  },

  sort : function(vectLinks, node, side, bundleMode)
  {
    // summary:
    //		Performs the sort.
    
    if (bundleMode != _SLBundlesModes.NO_BUNDLE)
    {
      var nLinks = (vectLinks != null ? vectLinks.length : 0);

      if (nLinks > 0)
      {
        this._vectLinks = vectLinks;
        this._node = node;
        this._side = side;
        this.inherited("sort", arguments, [nLinks]);
      }
    }
  },

  compare : function(loc1, loc2)
  {
    // summary:
    //		Compares two elements. 
    // description:
    //		Compares two elements. 
    //		Returns 0 if both elements are equal. Returns a number less than 0
    //		if the first element should come before the second element, and a
    //		number larger than 0 if the first element should come after the
    //		second element.
    // tags:
    //		protected
    
    if (this.compare1(loc1, loc2)) {
      return -1; // int
    }
    return 1; // int
  },

  compare1 : function(loc1, loc2)
  {
    // summary:
    //		Compare two elements. 
    // description:
    //		Compare two elements. 
    //		This is used when computing the order of the nodes connected to the
    //		same side of a node.
    // tags:
    //		private
    
    var link1 = this._vectLinks[loc1];
    var link2 = this._vectLinks[loc2];
    var origin1 = link1.isOrigin(this._node, this._side);
    var origin2 = link2.isOrigin(this._node, this._side);
    var connectionFixed1 = link1.isConnectionPointFixed(origin1);
    var connectionFixed2 = link2.isConnectionPointFixed(origin2);

    var position;

    if (connectionFixed1 && connectionFixed2) {
      return this._side.getTangentDelta(link1.getConnectionPoint(origin1), link2.getConnectionPoint(origin2)) <= 0; // Boolean
    }
    else if (link1.isSelfLink())
    {
      if (link1.isSameSideSelfLink()) {
        return true; // Boolean
      }
      else if (!link2.isSelfLink())
      {
        position = this._side.getSidePosition(link1.getNodeSide(!origin1).getDirection());
        if (position == Direction.LEFT) {
          return true; // Boolean
        }
        else if (position == Direction.RIGHT) {
          return false; // Boolean
        }
      }
    }
    else if (link2.isSelfLink())
    {
      if (link2.isSameSideSelfLink()) {
        return false; // Boolean
      }
      else if (!link1.isSelfLink())
      {
        position = this._side.getSidePosition(link2.getNodeSide(!origin2).getDirection());
        if (position == Direction.LEFT) {
          return false; // Boolean
        }
        else if (position == Direction.RIGHT) {
          return true; // Boolean
        }
      }
    }

    return this.compareLinks(link1, link2, origin1, origin2, this._side); // Boolean
  },

  compareLinks : function(link1, link2, origin1, origin2, nodeSide)
  {
    // summary:
    //		Compare two non-self-links that
    //		don't have (both of them) fixed connection points. 
    // description:
    //		Compare two non-self-links that don't have (both of them) fixed
    //		connection points. 
    //		This is used when computing the order of the nodes connected to the 
    //		same side of a node.
    // tags:
    //		protected
    },
  
  swap : function(loc1, loc2)
  {
    // summary:
    //		Swap two elements. 
    // description:
    //		Swap two elements. 
    //		(Method coming from the class QuickSort.)
    // tags:
    //		protected
    
    var link = this._vectLinks[loc1];
    this._vectLinks[loc1] =   this._vectLinks[loc2];
    this._vectLinks[loc2] =   link;
  }

}
);

return _IncidentLinksSorter;
});

},
'ibm_ilog/graphlayout/longlink/LLIncrementalLink':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLIncrementalLink", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var LLIncrementalLink = declare("ibm_ilog.graphlayout.longlink.LLIncrementalLink", null, 
{
  // summary:
  //		Data for links when using incremental mode. 
  // description:
  //		Data for links when using incremental mode. 
  //		We need this data in order to figure out which links have changed and
  //		must be rerouted. The data is updated when incremental is switched
  //		on, and after layout is performed when incremental is already on.  
  // tags:
  //		private
  
  _linkObject: null,

  _points: null,

  _numPoints: null,

  _ifromNode: null,
  _itoNode: null,

  _fromFixed: null,
  _toFixed: null,

  _newFromPoint: null,
  _newToPoint: null,

  constructor : function(layout, link)
  {
    this._linkObject = link;
    this.update(layout);
  },
  
  update : function(layout)
  {
    // summary:
    //		Updates the link points and data that define the link.
    // tags:
    //		private
    
    var model = layout.getGraphModel();
    var idata = layout.getIncrementalData();
    var linkObject = this.getLinkObject();
    this._points = model.getLinkPoints(linkObject);
    this._numPoints = this._points.length;
    this._ifromNode = idata.getNodeData(model.getFrom(linkObject));
    this._itoNode = idata.getNodeData(model.getTo(linkObject));
    this._fromFixed = layout.isFromPointFixed(linkObject);
    this._toFixed = layout.isToPointFixed(linkObject);

    if (layout.isIncrementalConnectionPreserving())
    {
      this._newFromPoint = this.getFromPoint();
      this._newToPoint = this.getToPoint();
    }
    else
    {
      this._newFromPoint = null;
      this._newToPoint = null;
    }

    if (this._numPoints >= 2)
    {
      if (this._ifromNode != null) {
        this._ifromNode.updateLink(this, this._points[0], true, this.isHorizontal(this._points[0], this._points[1]));
      }
      if (this._itoNode != null) {
        this._itoNode.updateLink(this, this._points[this._numPoints - 1], false, this.isHorizontal(this._points[this._numPoints - 1], this._points[this._numPoints - 2]));
      }
    }
  },

  clear : function()
  {
    // summary:
    //		Clears the incremental data such that this link will be rerouted
    //		next time (even if it hasn't changed).
    // tags:
    //		private
    
    if (this._numPoints > 0) {
      this._numPoints = -this._numPoints;
    }
  },

  isHorizontal : function(p1, p2)
  {
    // summary:
    //		Returns 'true' if a link segment is "more" horizontal
    //		than vertical. 
    // description:
    //		Returns <code>true</code> if a link segment is "more" horizontal
    //		than vertical. 
    //		This is just a heuristic, that also works for links that are not
    //		perfectly orthogonal.
    // tags:
    //		private
    
    var dx = p1.x - p2.x;
    var dy = p1.y - p2.y;
    if (dx < 0.0) {
      dx = -dx;
    }
    if (dy < 0.0) {
      dy = -dy;
    }
    return (dx >= dy); // Boolean
  },

  needsReroute : function(model, grid)
  {
    // summary:
    //		Returns 'true' if the link must be rerouted. 
    // description:
    //		Returns <code>true</code> if the link must be rerouted. 
    //		This is the case if one of the following holds: - its incremental
    //		data is incomplete (thas is, e.g., the link is new) - its "from" or
    //		"to" node has changed - any of its link points has changed - the
    //		link overlaps any obstacle - the link is not orthogonal
    // tags:
    //		private
    
    var fromNode = this.getFromNode();
    var toNode = this.getToNode();
    var linkObject = this.getLinkObject();
    if (fromNode == null) {
      return (true); // Boolean
    }
    if (toNode == null) {
      return (true); // Boolean
    }
    if (fromNode.getNodeObject() != model.getFrom(linkObject)) {
      return (true); // Boolean
    }
    if (toNode.getNodeObject() != model.getTo(linkObject)) {
      return (true); // Boolean
    }
    if (fromNode.needsReroute()) {
      return (true); // Boolean
    }
    if (toNode.needsReroute()) {
      return (true); // Boolean
    }

    var actPoints = model.getLinkPoints(linkObject);
    var actNumPoints = actPoints.length;

    if (actNumPoints != this._numPoints) {
      return (true); // Boolean
    }

    var i;

    for (i = 0; i < actNumPoints; i++)
      if (!TranslateUtil.pointEquals(actPoints[i], this._points[i])) {
        return (true); // Boolean
      }
    if (actNumPoints < 2) {
      return (true); // Boolean
    }

    var p1;
    var p2;
    var limits = TranslateUtil.newArray(2, 0);
    var index;
    var sign;
    var deltax;
    var deltay;

    for (i = 0; i < actNumPoints - 1; i++)
    {
      p1 = actPoints[i];
      p2 = actPoints[i + 1];
      deltax = p1.x - p2.x;
      deltay = p1.y - p2.y;

      if (deltax == 0.0)
      {
        index = grid.getClosestGridIndex(0, p1.x);

        deltax = grid.getGridCoord(0, index) - p1.x;

        if ((i > 0 && i < actNumPoints - 2) || (i == 0 && !this.isFromPointFixed()) || (i == actNumPoints - 2 && !this.isToPointFixed()))
        {
          if (deltax > 0.5 || deltax < -0.5)
          {
            return (true); // Boolean
          }
        }

        limits[0] = grid.getClosestGridIndex(1, p1.y);
        limits[1] = grid.getClosestGridIndex(1, p2.y);
        sign = ((limits[0] <= limits[1] ? 1 : -1));

        if (i == 0)
        {
          var recta = fromNode.getOldRect();

          while (grid.isWithinRectObstacle(index, limits[0], recta))
            limits[0] += sign;
        }

        if (i == actNumPoints - 2)
        {
          var rectb = toNode.getOldRect();

          while (grid.isWithinRectObstacle(index, limits[1], rectb))
            limits[1] -= sign;
        }

        if (!grid.isWithoutObstacles(1, index, limits))
        {
          return (true); // Boolean
        }
      }
      else if (deltay == 0)
      {
        index = grid.getClosestGridIndex(1, p1.y);

        deltay = grid.getGridCoord(1, index) - p1.y;

        if ((i > 0 && i < actNumPoints - 2) || (i == 0 && !this.isFromPointFixed()) || (i == actNumPoints - 2 && !this.isToPointFixed()))
        {
          if (deltay > 0.5 || deltay < -0.5)
          {
            return (true); // Boolean
          }
        }

        limits[0] = grid.getClosestGridIndex(0, p1.x);
        limits[1] = grid.getClosestGridIndex(0, p2.x);
        sign = ((limits[0] <= limits[1] ? 1 : -1));

        if (i == 0)
        {
          var rectc = fromNode.getOldRect();

          while (grid.isWithinRectObstacle(limits[0], index, rectc))
            limits[0] += sign;
        }

        if (i == actNumPoints - 2)
        {
          var rectd = toNode.getOldRect();

          while (grid.isWithinRectObstacle(limits[1], index, rectd))
            limits[1] -= sign;
        }

        if (!grid.isWithoutObstacles(0, index, limits))
        {
          return (true); // Boolean
        }
      }
      else
      {
        return (true); // Boolean
      }
    }

    return (false); // Boolean
  },

  getLinkObject : function()
  {
    // summary:
    //		Returns the corresponding link.
    // tags:
    //		private
    
    return this._linkObject; // Object
  },

  getFromNode : function()
  {
    // summary:
    //		Returns the incremental data of the "from" node.
    // tags:
    //		private
    
    return this._ifromNode; // ibm_ilog.graphlayout.longlink.LLIncrementalNode
  },

  getToNode : function()
  {
    // summary:
    //		Returns the incremental data of the "to" node.
    // tags:
    //		private
    
    return this._itoNode; // ibm_ilog.graphlayout.longlink.LLIncrementalNode
  },

  getFromPoint : function()
  {
    // summary:
    //		Returns the "from" point of the previous route.
    // tags:
    //		private
    
    return this._points[0]; // Point
  },

  getToPoint : function()
  {
    // summary:
    //		Returns the "to" point of the previous route.
    // tags:
    //		private
    
    var n = ((this._numPoints > 0 ? this._numPoints : -this._numPoints));
    return this._points[n - 1]; // Point
  },

  isFromPointFixed : function()
  {
    // summary:
    //		Returns true if the "from" point was fixed.
    // tags:
    //		private
    
    return this._fromFixed; // Boolean
  },

  isToPointFixed : function()
  {
    // summary:
    //		Returns true if the "to" point was fixed.
    // tags:
    //		private
    
    return this._toFixed; // Boolean
  },

  setNewFromPoint : function(p)
  {
    // summary:
    //		Sets the new "from" point for the next route.
    // tags:
    //		private
    
    this._newFromPoint = p;
  },

  setNewToPoint : function(p)
  {
    // summary:
    //		Sets the new "to" point for the next route.
    // tags:
    //		private
    
    this._newToPoint = p;
  },

  getNewFromPoint : function()
  {
    // summary:
    //		Returns the new "from" point if the end node has moved.
    // tags:
    //		private
    
    return this._newFromPoint; // Point
  },

  getNewToPoint : function()
  {
    // summary:
    //		Returns the new "to" point if the end node has moved.
    // tags:
    //		private
    
    return this._newToPoint; // Point
  }

}
);

return LLIncrementalLink;
});

},
'ibm_ilog/graphlayout/longlink/LLLinkRouteAlgorithm':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLLinkRouteAlgorithm", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Point",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../longlink/LLGridSegment",	// #8
  "../longlink/LLTerminationPoint",	// #9
  "../Direction"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Point,	// #6
    TranslateUtil,	// #7
    LLGridSegment,	// #8
    LLTerminationPoint,	// #9
    Direction	// #10
    ) {

var LLLinkRouteAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLLinkRouteAlgorithm", null, 
{
  // summary:
  //		The algorithm to perform one single routing. 
  // description:
  //		The algorithm to perform one single routing. 
  //		<p>
  //		It traverses the grid from start grid point until it reaches the end
  //		grid point and stores the necessary backtrack distance. This is like
  //		calculating shortest distances in a graph. The it searches one path
  //		backwards from the end point to the start point by passing only over
  //		decreasing backtrack distance points. TODO: needs review  
  //		</p>
  // tags:
  //		private
  
  _layout: null,

  _grid: null,

  _points: null,

  _crossingsInStraightOrOneBendRoute: null,

  _backtrackDist: null,

  _firstValidBacktrackDist: null,

  _backtrackSegments: null,

  _startIndex: null,
  _endIndex: null,

  _startSegment: null,
  _endSegment: null,

  _forbiddenMinIndexAtStart: null,
  _forbiddenMaxIndexAtStart: null,

  _forbiddenMinIndexAtEnd: null,
  _forbiddenMaxIndexAtEnd: null,

  _dummySegment: null,

  _searchExhaustive : null,

  _maxBacktrackDist: null,

  _minAdditionalBacktrackDist : null,

  _elapseTime: null,

  _topPoints: null,
  _bottomPoints: null,
  _leftPoints: null,
  _rightPoints: null,

  constructor : function(layout, grid)
  {
    this._searchExhaustive = false;
    this._minAdditionalBacktrackDist = 5;
    this._layout = layout;
    this._grid = grid;
    this._startIndex = TranslateUtil.newArray(2, 0);
    this._endIndex = TranslateUtil.newArray(2, 0);
    this._points = new Array();
    this._dummySegment = new LLGridSegment(null, 0, 0, null);
    this._searchExhaustive = layout.isExhaustiveSearching();
    this._maxBacktrackDist = layout.getMaxBacktrack();
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._points = null;
    this._dummySegment.dispose();
    this._topPoints = null;
    this._bottomPoints = null;
    this._leftPoints = null;
    this._rightPoints = null;

    this._grid = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getPoints : function()
  {
    // summary:
    //		Returns the calculated points.
    // tags:
    //		private
    
    return (this._points); // Array
  },

  getGrid : function()
  {
    // summary:
    //		Returns the routing grid.
    // tags:
    //		private
    
    return (this._grid); // ibm_ilog.graphlayout.longlink.LLGrid
  },

  routeStraight : function(link)
  {
    // summary:
    //		Route a link from its start node to its end node straight. 
    // description:
    //		Route a link from its start node to its end node straight. 
    //		It returns true on success.
    // tags:
    //		private
    
    var layout = this.getLayout();
    this._elapseTime = new Date().getTime() + layout.getAllowedTimePerLink();

    link.cleanRoute();
    this._points.length = 0;
    if (this.routeBestStraight(link)) {
      return (true); // Boolean
    }
    return (false); // Boolean
  },

  route : function(link)
  {
    // summary:
    //		Route a link from its start node to its end node. 
    // description:
    //		Route a link from its start node to its end node. 
    //		It returns true on success.
    // tags:
    //		private
    
    var layout = this.getLayout();
    this._elapseTime = new Date().getTime() + layout.getAllowedTimePerLink();

    link.cleanRoute();
    this._points.length = 0;

    if (layout.isStraightRouteEnabled())
    {
      if (this.routeBestOneBend(link)) {
        return (true); // Boolean
      }
    }

    if (link.isSelfLoop())
    {
      if (this.routeSelfLoopOptimized(link)) {
        return (true); // Boolean
      }
    }

    var startPointCand = link.getStartPoints();
    var endPointCand = link.getEndPoints();
    var numberStartPointCands = startPointCand.length;
    var numberEndPointCands = endPointCand.length;

    var maxSPIndex = 0;
    var maxEPIndex = 0;
    var actStartPoint;
    var actEndPoint;
    var prevStartPoint = null;
    var prevEndPoint = null;
    var isLastStart;
    var isLastEnd;
    var i;

    while (true)
    {
      if (maxSPIndex < numberStartPointCands)
      {
        if (maxEPIndex < numberEndPointCands)
        {
          if (startPointCand[maxSPIndex].getPenalty() < endPointCand[maxEPIndex].getPenalty())
          {
            actStartPoint = startPointCand[maxSPIndex];
            maxSPIndex++;
            isLastStart = (maxSPIndex == numberStartPointCands);

            if (maxEPIndex > 0 && this.needNewSearch(link, actStartPoint, prevStartPoint))
            {
              prevEndPoint = null;

              for (i = 0; i < maxEPIndex; i++)
              {
                actEndPoint = endPointCand[i];
                isLastEnd = (i == numberEndPointCands - 1);

                if (this.needNewSearch(link, actEndPoint, prevEndPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
                {
                  prevStartPoint = actStartPoint;
                  prevEndPoint = actEndPoint;

                  if (this.searchRoute(actStartPoint, actEndPoint, true))
                  {
                    link.setActStartAndEndCandidates(maxSPIndex - 1, i);
                    link.storeRoute(this.getPoints());
                    return (true); // Boolean
                  }
                }
              }
            }
          }
          else
          {
            actEndPoint = endPointCand[maxEPIndex];
            maxEPIndex++;
            isLastEnd = (maxEPIndex == numberEndPointCands);

            if (maxSPIndex > 0 && this.needNewSearch(link, actEndPoint, prevEndPoint))
            {
              prevStartPoint = null;

              for (i = 0; i < maxSPIndex; i++)
              {
                actStartPoint = startPointCand[i];
                isLastStart = (i == numberStartPointCands - 1);

                if (this.needNewSearch(link, actStartPoint, prevStartPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
                {
                  prevStartPoint = actStartPoint;
                  prevEndPoint = actEndPoint;

                  if (this.searchRoute(actStartPoint, actEndPoint, true))
                  {
                    link.setActStartAndEndCandidates(i, maxEPIndex - 1);
                    link.storeRoute(this.getPoints());
                    return (true); // Boolean
                  }
                }
              }
            }
          }
        }
        else
        {
          actStartPoint = startPointCand[maxSPIndex];
          maxSPIndex++;
          isLastStart = (maxSPIndex == numberStartPointCands);

          if (maxEPIndex > 0 && this.needNewSearch(link, actStartPoint, prevStartPoint))
          {
            prevEndPoint = null;

            for (i = 0; i < maxEPIndex; i++)
            {
              actEndPoint = endPointCand[i];
              isLastEnd = (i == numberEndPointCands - 1);

              if (this.needNewSearch(link, actEndPoint, prevEndPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
              {
                prevStartPoint = actStartPoint;
                prevEndPoint = actEndPoint;

                if (this.searchRoute(actStartPoint, actEndPoint, true))
                {
                  link.setActStartAndEndCandidates(maxSPIndex - 1, i);
                  link.storeRoute(this.getPoints());
                  return (true); // Boolean
                }
              }
            }
          }
        }
      }
      else if (maxEPIndex < numberEndPointCands)
      {
        actEndPoint = endPointCand[maxEPIndex];
        maxEPIndex++;
        isLastEnd = (maxEPIndex == numberEndPointCands);

        if (maxSPIndex > 0 && this.needNewSearch(link, actEndPoint, prevEndPoint))
        {
          prevStartPoint = null;

          for (i = 0; i < maxSPIndex; i++)
          {
            actStartPoint = startPointCand[i];
            isLastStart = (i == numberStartPointCands - 1);

            if (this.needNewSearch(link, actStartPoint, prevStartPoint) && this.nonzeroRoute(actStartPoint, actEndPoint, isLastStart, isLastEnd))
            {
              prevStartPoint = actStartPoint;
              prevEndPoint = actEndPoint;

              if (this.searchRoute(actStartPoint, actEndPoint, true))
              {
                link.setActStartAndEndCandidates(i, maxEPIndex - 1);
                link.storeRoute(this.getPoints());
                return (true); // Boolean
              }
            }
          }
        }
      }
      else
      {
        break;
      }
    }

    return (false); // Boolean
  },

  routeSelfLoopOptimized : function(link)
  {
    // summary:
    //		Route self loops optimized. 
    // description:
    //		Route self loops optimized. 
    //		Self loops prefer two neighbored termination points at the same
    //		side. However, the order of the termination points in the start and
    //		end candidate arrays is such that self loops are preferably routed
    //		between different sides. To overcome this deficiency, we test here
    //		all pairs of termination point candidates that have minimal penalty,
    //		belong to the same side and are directly neighbored. If we find such
    //		a pair, we route there and return true. If we
    //		don't find such a pair that is successful, then we return false, which implies that we use the normal mechanism then.
    // tags:
    //		private
    
    var startPointCand = link.getStartPoints();
    var endPointCand = link.getEndPoints();
    var numberStartPointCands = startPointCand.length;
    var numberEndPointCands = endPointCand.length;

    if (numberStartPointCands == 0 || numberEndPointCands == 0) {
      return (false); // Boolean
    }
    if (numberStartPointCands == 1 && numberEndPointCands == 1) {
      return (false); // Boolean
    }

    var actSPIndex = 0;
    var actEPIndex = 0;
    var actStartPoint;
    var actEndPoint;
    var dir;
    var bestStartPointPenalty = startPointCand[0].getPenalty();
    var bestEndPointPenalty = endPointCand[0].getPenalty();

    for (actSPIndex = 0; actSPIndex < numberStartPointCands; actSPIndex++)
    {
      actStartPoint = startPointCand[actSPIndex];

      if (actStartPoint.getPenalty() > bestStartPointPenalty) {
        break;
      }
      for (actEPIndex = 0; actEPIndex < numberEndPointCands; actEPIndex++)
      {
        actEndPoint = endPointCand[actEPIndex];

        if (actEndPoint.getPenalty() > bestEndPointPenalty) {
          break;
        }
        if (actStartPoint.getSide() == actEndPoint.getSide())
        {
          if (actStartPoint.getSide() == Direction.TOP || actStartPoint.getSide() == Direction.BOTTOM) {
            dir = 0;
          }
          else {
            dir = 1;
          }
          if (actStartPoint.getIndex(dir) + 1 == actEndPoint.getIndex(dir))
          {
            if (this.nonzeroRoute(actStartPoint, actEndPoint, false, false))
            {
              if (this.searchRoute(actStartPoint, actEndPoint, true))
              {
                link.setActStartAndEndCandidates(actSPIndex, actEPIndex);
                link.storeRoute(this.getPoints());
                return (true); // Boolean
              }
            }
          }
        }
      }
    }

    return (false); // Boolean
  },

  nonzeroRoute : function(p1, p2, p1IsLast, p2IsLast)
  {
    // summary:
    //		Returns true if start and end point are not identical, or if they
    //		are identical but there are no others. 
    // description:
    //		Returns true if start and end point are not identical, or if they
    //		are identical but there are no others. 
    //		This check is needed to avoid that self loops are routed as one
    //		point (i.e. a path with zero length).
    // tags:
    //		private
    
    if (p1.getIndex(0) == p2.getIndex(0) && p1.getIndex(1) == p2.getIndex(1))
    {
      if (!p1IsLast) {
        return (false); // Boolean
      }
      if (!p2IsLast) {
        return (false); // Boolean
      }
    }

    return (true); // Boolean
  },

  needNewSearch : function(link, newPoint, oldPoint)
  {
    // summary:
    //		Returns true if we need to try a search starting (or ending) at the
    //		new point. 
    // description:
    //		Returns true if we need to try a search starting (or ending) at the
    //		new point. 
    //		We tried already a search from old point which was unsuccessful. If
    //		the old point is reachable from new point, then the search starting
    //		from new point must be unsuccessful as well, otherwise the old point
    //		would have been reachable by passing the new point.
    // tags:
    //		private
    
    if (link.isSelfLoop()) {
      return (true); // Boolean
    }
    if (oldPoint == null) {
      return (true); // Boolean
    }
    if (this._maxBacktrackDist < 20) {
      return (true); // Boolean
    }
    var result = true;
    var oldMaxBacktrack = this._maxBacktrackDist;
    var oldMinAddBacktrack = this._minAdditionalBacktrackDist;
    var oldSearchExhaustive = this._searchExhaustive;

    this._maxBacktrackDist = 10;
    this._minAdditionalBacktrackDist = 0;
    this._searchExhaustive = false;
    if (this.searchRoute(newPoint, oldPoint, false)) {
      result = false;
    }
    this._maxBacktrackDist = oldMaxBacktrack;
    this._minAdditionalBacktrackDist = oldMinAddBacktrack;
    this._searchExhaustive = oldSearchExhaustive;
    return (result); // Boolean
  },

  searchRoute : function(startPoint, endPoint, storeBends)
  {
    // summary:
    //		Routes a link from start point to end point.
    // tags:
    //		private
    
    this._grid.clearDirty();
    this._points.length = 0;

    this._startIndex[0] = startPoint.getIndex(0);
    this._startIndex[1] = startPoint.getIndex(1);
    this._endIndex[0] = endPoint.getIndex(0);
    this._endIndex[1] = endPoint.getIndex(1);

    var startDir = 1;
    var endDir = 1;
    this._startSegment = startPoint.getSegment();
    this._endSegment = endPoint.getSegment();
    if (this._startSegment == null) {
      return (false); // Boolean
    }
    if (this._endSegment == null) {
      return (false); // Boolean
    }
    this._forbiddenMinIndexAtStart = startPoint.getForbiddenMinIndex();
    this._forbiddenMaxIndexAtStart = startPoint.getForbiddenMaxIndex();
    this._forbiddenMinIndexAtEnd = endPoint.getForbiddenMinIndex();
    this._forbiddenMaxIndexAtEnd = endPoint.getForbiddenMaxIndex();
    if (this._startSegment.isHorizontal()) {
      startDir = 0;
    }
    if (this._endSegment.isHorizontal()) {
      endDir = 0;
    }
    if (this._startSegment == this._endSegment)
    {
      return (true); // Boolean
    }

    var segStart = this._startSegment.getStartIndex();
    var segEnd = this._startSegment.getEndIndex();

    if (this._forbiddenMaxIndexAtStart >= segStart && this._forbiddenMinIndexAtStart <= segEnd)
    {
      if (segStart < this._forbiddenMinIndexAtStart) {
        this._startSegment.setBacktrackStartIndex(this._forbiddenMinIndexAtStart);
      }
      else {
        this._startSegment.setBacktrackStartIndex(segStart);
      }
      if (segEnd > this._forbiddenMaxIndexAtStart) {
        this._startSegment.setBacktrackEndIndex(this._forbiddenMaxIndexAtStart);
      }
      else {
        this._startSegment.setBacktrackEndIndex(segEnd);
      }
      this._startSegment.setBacktrackDist(0);
      this._grid.markDirty(this._startSegment.getGridLine());
    }

    segStart = this._endSegment.getStartIndex();
    segEnd = this._endSegment.getEndIndex();

    if (this._forbiddenMaxIndexAtEnd >= segStart && this._forbiddenMinIndexAtEnd <= segEnd)
    {
      if (segStart < this._forbiddenMinIndexAtEnd) {
        this._endSegment.setBacktrackStartIndex(this._forbiddenMinIndexAtEnd);
      }
      else {
        this._endSegment.setBacktrackStartIndex(segStart);
      }
      if (segEnd > this._forbiddenMaxIndexAtEnd) {
        this._endSegment.setBacktrackEndIndex(this._forbiddenMaxIndexAtEnd);
      }
      else {
        this._endSegment.setBacktrackEndIndex(segEnd);
      }
      this._endSegment.setBacktrackDist(0);
      this._grid.markDirty(this._endSegment.getGridLine());
    }

    if (this.searchForwardWithBacktrack(this._startSegment, this._startIndex[startDir], startDir))
    {
      if (storeBends)
      {
        this._endSegment.setMinSegmentNumber(LLLinkRouteAlgorithm.MAXVAL);
        this._endSegment.setBacktrackDist(LLLinkRouteAlgorithm.MAXVAL);
        this.searchBackward(this._endSegment, this._endIndex[endDir], endDir);
      }

      return (true); // Boolean
    }

    return (false); // Boolean
  },

  searchForwardWithBacktrack : function(startSegment, startPointIndex, dir)
  {
    // summary:
    //		Search from the start segment to the end segment with backtrack. 
    // description:
    //		Search from the start segment to the end segment with backtrack. 
    //		Backtrack means, if we
    //		don't reach the end segment in the shortest possible distance, allow to move away from the end point by the backtrack distance in order to bypass obstacles. The startSegment is a segment in direction "dir", which is 0 if it is a horizontal segment and 1 if it is a vertical segment. The startPointIndex is the index of the start point of this segment. It returns true if the end segment is found.
    // tags:
    //		private
    
    this._firstValidBacktrackDist = -1;
    this._backtrackDist = 0;
    this._backtrackSegments = this._dummySegment;
    var result = this.searchForward(startSegment, startPointIndex, dir, 0);
    if (result == true) {
      this._firstValidBacktrackDist = 0;
    }
    if (result == true && this._searchExhaustive == false && this._minAdditionalBacktrackDist == 0) {
      return (true); // Boolean
    }
    while (this._backtrackDist < this._maxBacktrackDist && new Date().getTime() <= this._elapseTime)
    {
      this._backtrackDist++;

      var seg = this._backtrackSegments;
      var predseg = null;
      var nrsegs = 0;

      while (seg != this._dummySegment)
      {
        if (!seg.canBacktrack())
        {
          if (predseg == null)
          {
            this._backtrackSegments = seg.getNextBacktrackSegement();
            seg.setNextBacktrackSegement(null);
            seg = this._backtrackSegments;
          }
          else
          {
            predseg.setNextBacktrackSegement(seg.getNextBacktrackSegement());
            seg.setNextBacktrackSegement(null);
            seg = predseg.getNextBacktrackSegement();
          }
        }
        else
        {
          nrsegs++;
          predseg = seg;
          seg = predseg.getNextBacktrackSegement();
        }
      }

      if (nrsegs == 0) {
        break;
      }

      var btsegments = TranslateUtil.newArray(nrsegs, null);
      var btlowerIndex = TranslateUtil.newArray(nrsegs, 0);
      var btupperIndex = TranslateUtil.newArray(nrsegs, 0);
      var nextseg;
      var i = 0;
      seg = this._backtrackSegments;

      while (seg != this._dummySegment)
      {
        nextseg = seg.getNextBacktrackSegement();
        seg.setNextBacktrackSegement(null);
        btsegments[i] = seg;
        btlowerIndex[i] = seg.getBacktrackStartIndex() - 1;
        btupperIndex[i] = seg.getBacktrackEndIndex() + 1;
        i++;
        seg = nextseg;
      }

      this._backtrackSegments = this._dummySegment;

      for (i = 0; i < nrsegs; i++)
      {
        seg = btsegments[i];
        dir = ((seg.isHorizontal() ? 0 : 1));

        if (seg.canBacktrack() && seg.getStartIndex() <= btlowerIndex[i] && seg.getBacktrackStartIndex() > btlowerIndex[i])
        {
          result = this.searchForward(seg, btlowerIndex[i], dir, LLLinkRouteAlgorithm.MAXVAL) || result;

          if (result == true)
          {
            if (this._firstValidBacktrackDist == -1) {
              this._firstValidBacktrackDist = this._backtrackDist;
            }
            if (this._searchExhaustive == false && this._firstValidBacktrackDist + this._minAdditionalBacktrackDist <= this._backtrackDist) {
              break;
            }
          }
        }

        if (seg.canBacktrack() && seg.getEndIndex() >= btupperIndex[i] && seg.getBacktrackEndIndex() < btupperIndex[i])
        {
          result = this.searchForward(seg, btupperIndex[i], dir, LLLinkRouteAlgorithm.MAXVAL) || result;

          if (result == true)
          {
            if (this._firstValidBacktrackDist == -1) {
              this._firstValidBacktrackDist = this._backtrackDist;
            }
            if (this._searchExhaustive == false && this._firstValidBacktrackDist + this._minAdditionalBacktrackDist <= this._backtrackDist) {
              break;
            }
          }
        }
      }

      if (result == true && this._searchExhaustive == false && this._firstValidBacktrackDist + this._minAdditionalBacktrackDist <= this._backtrackDist) {
        break;
      }
    }

    return (result); // Boolean
  },

  searchForward : function(segment, gridPointIndex, dir, minSegmentNr)
  {
    // summary:
    //		Search from the input segment to the end segment. 
    // description:
    //		Search from the input segment to the end segment. 
    //		The input segment is a segment in direction "dir", which is 0 if it
    //		is a horizontal segment and 1 if it is a vertical segment. The grid
    //		point index is the index where we reached the segment. It returns
    //		true if the end segment is found without any backtrack, i.e. without
    //		moving further away from the end point, otherwise false.
    // tags:
    //		private
    
    if (this.isForbidden(gridPointIndex, segment)) {
      return (false); // Boolean
    }
    if (segment == null) {
      return (false); // Boolean
    }
    var grid = this.getGrid();
    var gridLine = segment.getGridLine();
    var ownIndex = gridLine.getIndex();
    var segStart = segment.getStartIndex();
    var segEnd = segment.getEndIndex();

    if (gridPointIndex < segStart || gridPointIndex > segEnd) {
      return (false); // Boolean
    }

    var newAreaStart = -1;
    var newAreaEnd = -1;

    var targetIndex = this._endIndex[dir];
    var oldBacktrackDist = segment.calcBacktrackDist(gridPointIndex);
    var oldBacktrackStart = segment.getBacktrackStartIndex();
    var oldBacktrackEnd = segment.getBacktrackEndIndex();

    if (oldBacktrackDist >= this._backtrackDist)
    {
      if (gridPointIndex <= this._endIndex[dir])
      {
        newAreaStart = gridPointIndex;
        newAreaEnd = ((targetIndex < segEnd ? targetIndex : segEnd));
      }
      else
      {
        newAreaStart = ((targetIndex > segStart ? targetIndex : segStart));
        newAreaEnd = gridPointIndex;
      }

      if (oldBacktrackDist == LLLinkRouteAlgorithm.MAXVAL)
      {
        segment.setBacktrackStartIndex(newAreaStart);
        segment.setBacktrackEndIndex(newAreaEnd);
      }
      else
      {
        if (newAreaStart < oldBacktrackStart) {
          segment.setBacktrackStartIndex(newAreaStart);
        }
        else {
          newAreaStart = oldBacktrackEnd + 1;
        }
        if (newAreaEnd > oldBacktrackEnd) {
          segment.setBacktrackEndIndex(newAreaEnd);
        }
        else {
          newAreaEnd = oldBacktrackStart - 1;
        }
      }

      segment.updateMinSegmentNumber(minSegmentNr);
      segment.setBacktrackDist(this._backtrackDist);
      grid.markDirty(gridLine);
    }

    if (segment == this._endSegment) {
      return (!this.isForbidden(gridPointIndex, segment)); // Boolean
    }
    if (segment.canBacktrack() && !segment.isOnBacktrackStack())
    {
      segment.setNextBacktrackSegement(this._backtrackSegments);
      this._backtrackSegments = segment;
    }

    var result = false;

    if (newAreaStart >= 0 && newAreaEnd >= 0 && newAreaStart <= newAreaEnd)
    {
      minSegmentNr = segment.getMinSegmentNumber() + 1;
      var seg;

      if (targetIndex <= newAreaStart)
      {
        for (var ia = newAreaStart; ia <= newAreaEnd; ia++)
        {
          if (!this.isForbidden(ia, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, ia);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }
        }
      }
      else if (targetIndex >= newAreaEnd)
      {
        for (var ib = newAreaEnd; ib >= newAreaStart; ib--)
        {
          if (!this.isForbidden(ib, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, ib);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }
        }
      }
      else
      {
        var i = targetIndex;
        var j = targetIndex + 1;

        while (i >= newAreaStart || j <= newAreaEnd)
        {
          if (i >= newAreaStart && !this.isForbidden(i, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, i);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }

          if (j <= newAreaEnd && !this.isForbidden(j, segment))
          {
            seg = segment.searchOrthogonalSegment(grid, j);

            result = this.searchForward(seg, ownIndex, 1 - dir, minSegmentNr) || result;

            if (result == true && this._searchExhaustive == false) {
              break;
            }
          }

          i--;
          j++;
        }
      }
    }

    return (result); // Boolean
  },

  searchBackward : function(segment, gridPointIndex, dir)
  {
    // summary:
    //		Search from the input segment to the start segment. 
    // description:
    //		Search from the input segment to the start segment. 
    //		Here we visit only the area of the segment that is marked with the
    //		backtrack distance. By that we are sure to reach the start node
    //		without more backtrack then the marked distance. The input segment
    //		is a segment in direction "dir", which is 0 if it is a horizontal
    //		segment and 1 if it is a vertical segment. The grid point index is
    //		the index where we reached the segment.
    // tags:
    //		private
    
    var ownIndex = segment.getGridLine().getIndex();

    if (segment == this._startSegment) {
      return;
    }

    var grid = this.getGrid();
    var i = gridPointIndex;
    var j = gridPointIndex + 1;
    var start = segment.getStartIndex();
    var end = segment.getEndIndex();
    var actMinSegmentNumber;
    var bestIndexToContinue = -1;
    var bestMinSegmentNumberToContinue = LLLinkRouteAlgorithm.MAXVAL;
    var bestSegToContinue = null;
    var seg;

    while (i >= start || j <= end)
    {
      if (i >= start && !this.isForbidden(i, segment))
      {
        seg = segment.searchOrthogonalSegment(grid, i);

        if (seg != null)
        {
          actMinSegmentNumber = seg.getMinSegmentNumber();

          if (!this.isForbidden(ownIndex, seg) && actMinSegmentNumber < bestMinSegmentNumberToContinue)
          {
            bestIndexToContinue = i;
            bestMinSegmentNumberToContinue = actMinSegmentNumber;
            bestSegToContinue = seg;
          }
        }
      }

      if (j <= end && !this.isForbidden(j, segment))
      {
        seg = segment.searchOrthogonalSegment(grid, j);

        if (seg != null)
        {
          actMinSegmentNumber = seg.getMinSegmentNumber();

          if (!this.isForbidden(ownIndex, seg) && actMinSegmentNumber < bestMinSegmentNumberToContinue)
          {
            bestIndexToContinue = j;
            bestMinSegmentNumberToContinue = actMinSegmentNumber;
            bestSegToContinue = seg;
          }
        }
      }

      i--;
      j++;
    }

    if (dir == 1) {
      this._points.push(new Point(ownIndex, bestIndexToContinue));
    }
    else {
      this._points.push(new Point(bestIndexToContinue, ownIndex));
    }
    this.searchBackward(bestSegToContinue, ownIndex, 1 - dir);
  },

  isForbidden : function(index, segment)
  {
    // summary:
    //		Returns true if the index is in the forbidden area of the segment. 
    // description:
    //		Returns true if the index is in the forbidden area of the segment. 
    //		This is mainly used to test that we
    //		don't traverse the area of the start or end segment that would violate the minimal start/end segment length.
    // tags:
    //		private
    
    if (segment == this._startSegment && index >= this._forbiddenMinIndexAtStart && index <= this._forbiddenMaxIndexAtStart) {
      return (true); // Boolean
    }
    if (segment == this._endSegment && index >= this._forbiddenMinIndexAtEnd && index <= this._forbiddenMaxIndexAtEnd) {
      return (true); // Boolean
    }
    return (false); // Boolean
  },

  routeBestStraight : function(link)
  {
    // summary:
    //		Search the best straight route. 
    // description:
    //		Search the best straight route. 
    //		It returns true on success and false on failure.
    // tags:
    //		private
    
    if (link.isSelfLoop()) {
      return (false); // Boolean
    }

    var iter;

    if (link.getStartPoints().length > 3 && link.getEndPoints().length > 3)
    {
      this.divideTerminationPointsBySide(link.getStartPoints());
      var startTopPoints = this._topPoints;
      var startBottomPoints = this._bottomPoints;
      var startLeftPoints = this._leftPoints;
      var startRightPoints = this._rightPoints;

      this.divideTerminationPointsBySide(link.getEndPoints());
      var endTopPoints = this._topPoints;
      var endBottomPoints = this._bottomPoints;
      var endLeftPoints = this._leftPoints;
      var endRightPoints = this._rightPoints;
      iter = new LLTerminationPointIterator(startTopPoints, endBottomPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      iter = new LLTerminationPointIterator(startBottomPoints, endTopPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      iter = new LLTerminationPointIterator(startLeftPoints, endRightPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      iter = new LLTerminationPointIterator(startRightPoints, endLeftPoints);
      if (this.iterateAndRouteBestStraight(iter, link)) {
        return (true); // Boolean
      }
      return (false); // Boolean
    }
    else
    {
      iter = new LLTerminationPointIterator(link.getStartPoints(), link.getEndPoints());
      return (this.iterateAndRouteBestStraight(iter, link)); // Boolean
    }
  },

  iterateAndRouteBestStraight : function(iter, link)
  {
    // summary:
    //		Iterates over all combination of start and end points, using the
    //		input iterator, and search the best straight route. 
    // description:
    //		Iterates over all combination of start and end points, using the
    //		input iterator, and search the best straight route. 
    //		It returns true on success and false on failure.
    // tags:
    //		private
    
    var routeFound = false;
    var bestStartCandIndex = 0;
    var bestEndCandIndex = 0;
    var bestCrossings = LLLinkRouteAlgorithm.MAXVAL;
    var penalty;
    var lastPenalty = LLLinkRouteAlgorithm.MAXVAL;
    var startPoint;
    var endPoint;
    var bestStartPoint = null;
    var bestEndPoint = null;
    var startPointIndex = 0;
    var endPointIndex = 0;
    var i;

    while (iter.hasNext())
    {
      startPoint = iter.getStartPoint();
      endPoint = iter.getEndPoint();

      penalty = Math.max(startPoint.getPenalty(), endPoint.getPenalty());
      if (penalty > lastPenalty) {
        break;
      }
      lastPenalty = penalty;

      if (this.searchStraightRoute(startPoint, endPoint))
      {
        for (i = 0; i < link.getStartPoints().length; i++)
        {
          if (link.getStartPoints()[i] == startPoint)
          {
            startPointIndex = i;
            break;
          }
        }

        for (i = 0; i < link.getEndPoints().length; i++)
        {
          if (link.getEndPoints()[i] == endPoint)
          {
            endPointIndex = i;
            break;
          }
        }

        if (this._crossingsInStraightOrOneBendRoute == 0)
        {
          link.setActStartAndEndCandidates(startPointIndex, endPointIndex);
          link.storeRoute(null);
          return (true); // Boolean
        }

        if (this._crossingsInStraightOrOneBendRoute < bestCrossings)
        {
          bestCrossings = this._crossingsInStraightOrOneBendRoute;
          bestStartCandIndex = startPointIndex;
          bestEndCandIndex = endPointIndex;
          bestStartPoint = startPoint;
          bestEndPoint = endPoint;
          routeFound = true;
        }
      }

      iter.next();
      if (new Date().getTime() > this._elapseTime) {
        break;
      }
    }

    if (routeFound)
    {
      this.searchStraightRoute(bestStartPoint, bestEndPoint);
      link.setActStartAndEndCandidates(bestStartCandIndex, bestEndCandIndex);
      link.storeRoute(null);
    }

    return (routeFound); // Boolean
  },

  routeBestOneBend : function(link)
  {
    // summary:
    //		Search the best one bend route. 
    // description:
    //		Search the best one bend route. 
    //		It returns true on success and false on failure.
    // tags:
    //		private
    
    if (link.isSelfLoop()) {
      return (false); // Boolean
    }
    var routeFound = false;
    var bestStartCandIndex = 0;
    var bestEndCandIndex = 0;
    var bestCrossings = LLLinkRouteAlgorithm.MAXVAL;
    var penalty;
    var lastPenalty = LLLinkRouteAlgorithm.MAXVAL;
    var iter = new LLTerminationPointIterator(link.getStartPoints(), link.getEndPoints());
    var startPoint;
    var endPoint;
    var bestStartPoint = null;
    var bestEndPoint = null;

    while (iter.hasNext())
    {
      startPoint = iter.getStartPoint();
      endPoint = iter.getEndPoint();

      penalty = Math.max(startPoint.getPenalty(), endPoint.getPenalty());
      if (penalty > lastPenalty) {
        break;
      }
      lastPenalty = penalty;

      if (this.searchOneBendRoute(startPoint, endPoint))
      {
        if (this._crossingsInStraightOrOneBendRoute == 0)
        {
          link.setActStartAndEndCandidates(iter.getStartPointIndex(), iter.getEndPointIndex());
          link.storeRoute(this.getPoints());
          return (true); // Boolean
        }

        if (this._crossingsInStraightOrOneBendRoute < bestCrossings)
        {
          bestCrossings = this._crossingsInStraightOrOneBendRoute;
          bestStartCandIndex = iter.getStartPointIndex();
          bestEndCandIndex = iter.getEndPointIndex();
          bestStartPoint = startPoint;
          bestEndPoint = endPoint;
          routeFound = true;
        }
      }

      iter.next();
      if (new Date().getTime() > this._elapseTime) {
        break;
      }
    }

    if (routeFound)
    {
      this.searchOneBendRoute(bestStartPoint, bestEndPoint);
      link.setActStartAndEndCandidates(bestStartCandIndex, bestEndCandIndex);
      link.storeRoute(this.getPoints());
    }

    return (routeFound); // Boolean
  },

  searchStraightRoute : function(startPoint, endPoint)
  {
    // summary:
    //		Routes a link from start point to end point straight. 
    // description:
    //		Routes a link from start point to end point straight. 
    //		It returns true on success and false on failure. It also stores the
    //		number ofr crossings with line obstacles for this route in
    //		&#95;crossingsInStraightOrOneBendRoute;
    // tags:
    //		private
    
    var startSegment = startPoint.getSegment();
    var endSegment = endPoint.getSegment();

    if (startSegment == endSegment && startSegment != null)
    {
      this._points.length = 0;

      var line = startSegment.getGridLine();
      this._crossingsInStraightOrOneBendRoute = line.calcApproxCrossings(this.getGrid(), startPoint.getIndex(((line.isHorizontal() ? 0 : 1))), endPoint.getIndex(((line.isHorizontal() ? 0 : 1))));
      return true; // Boolean
    }

    return (false); // Boolean
  },

  searchOneBendRoute : function(startPoint, endPoint)
  {
    // summary:
    //		Routes a link from start point to end point with just one bend. 
    // description:
    //		Routes a link from start point to end point with just one bend. 
    //		straight. It returns true on success and false on failure. It also
    //		stores the number ofr crossings with line obstacles for this route
    //		in &#95;crossingsInStraightOrOneBendRoute;
    // tags:
    //		private
    
    var startSegment = startPoint.getSegment();
    var endSegment = endPoint.getSegment();
    if (startSegment == null) {
      return (false); // Boolean
    }
    if (endSegment == null) {
      return (false); // Boolean
    }
    var startLine = startSegment.getGridLine();
    var endLine = endSegment.getGridLine();

    if (startLine.isHorizontal() == endLine.isHorizontal()) {
      return (false); // Boolean
    }
    var c1 = endLine.getIndex();
    var c2 = startLine.getIndex();

    if (startSegment.getStartIndex() > c1 || startSegment.getEndIndex() < c1) {
      return (false); // Boolean
    }
    if (endSegment.getStartIndex() > c2 || endSegment.getEndIndex() < c2) {
      return (false); // Boolean
    }
    if (startPoint.getForbiddenMinIndex() <= c1 && startPoint.getForbiddenMaxIndex() >= c1) {
      return (false); // Boolean
    }
    if (endPoint.getForbiddenMinIndex() <= c2 && endPoint.getForbiddenMaxIndex() >= c2) {
      return (false); // Boolean
    }
    this._points.length = 0;
    if (startSegment.isHorizontal()) {
      this._points.push(new Point(c1, c2));
    }
    else {
      this._points.push(new Point(c2, c1));
    }

    var c0 = startPoint.getIndex(((startLine.isHorizontal() ? 0 : 1)));
    this._crossingsInStraightOrOneBendRoute = startLine.calcApproxCrossings(this.getGrid(), c0, c1);
    c0 = endPoint.getIndex(((endLine.isHorizontal() ? 0 : 1)));
    this._crossingsInStraightOrOneBendRoute += endLine.calcApproxCrossings(this.getGrid(), c0, c2);
    return (true); // Boolean
  },

  divideTerminationPointsBySide : function(points)
  {
    // summary:
    //		Extracts the termination points from the array of termination points
    //		according to the side. 
    // description:
    //		Extracts the termination points from the array of termination points
    //		according to the side. 
    //		The result is in &#95;topPoints, &#95;bottomPoints, &#95;leftPoints and
    //		&#95;rightPoints.
    // tags:
    //		private
    
    var i;
    var nrTopPoints = 0;
    var nrBottomPoints = 0;
    var nrLeftPoints = 0;
    var nrRightPoints = 0;

    for (i = 0; i < points.length; i++)
    {
      switch (points[i].getSide())
      {
        case Direction.TOP:
          nrTopPoints++;
          break;
        case Direction.BOTTOM:
          nrBottomPoints++;
          break;
        case Direction.LEFT:
          nrLeftPoints++;
          break;
        case Direction.RIGHT:
          nrRightPoints++;
          break;
      }
    }

    this._topPoints = TranslateUtil.newArray(nrTopPoints, null);
    this._bottomPoints = TranslateUtil.newArray(nrBottomPoints, null);
    this._leftPoints = TranslateUtil.newArray(nrLeftPoints, null);
    this._rightPoints = TranslateUtil.newArray(nrRightPoints, null);
    nrTopPoints = 0;
    nrBottomPoints = 0;
    nrLeftPoints = 0;
    nrRightPoints = 0;

    for (i = 0; i < points.length; i++)
    {
      switch (points[i].getSide())
      {
        case Direction.TOP:
          this._topPoints[nrTopPoints++] = points[i];
          break;
        case Direction.BOTTOM:
          this._bottomPoints[nrBottomPoints++] = points[i];
          break;
        case Direction.LEFT:
          this._leftPoints[nrLeftPoints++] = points[i];
          break;
        case Direction.RIGHT:
          this._rightPoints[nrRightPoints++] = points[i];
          break;
      }
    }
  }

}
);

lang.mixin(LLLinkRouteAlgorithm,
{
  MAXVAL : null,

  _fields_init : function(c)
  {
    c = c || LLLinkRouteAlgorithm;
    
    c.MAXVAL = /*Number.MAX_VALUE*/2147483647;
  }
  
}
);

LLTerminationPointIterator = declare( null, 
{
  startPointCand: null,
  endPointCand: null,
  numberStartPointCands: null,
  numberEndPointCands: null,
  maxSPIndex: null,
  maxEPIndex: null,
  actSPIndex: null,
  actEPIndex: null,
  iterateAtStart: null,

  constructor : function(s, e)
  {
    this.startPointCand = s;
    this.endPointCand = e;
    this.numberStartPointCands = this.startPointCand.length;
    this.numberEndPointCands = this.endPointCand.length;
    this.maxSPIndex = 0;
    this.maxEPIndex = 0;
    this.actSPIndex = 0;
    this.actEPIndex = 0;
    this.iterateAtStart = true;
  },
  
  hasNext : function()
  {
    return (this.maxSPIndex < this.numberStartPointCands && this.maxEPIndex < this.numberEndPointCands); // Boolean
  },

  getStartPoint : function()
  {
    return (this.startPointCand[this.actSPIndex]); // ibm_ilog.graphlayout.longlink.LLTerminationPoint
  },

  getEndPoint : function()
  {
    return (this.endPointCand[this.actEPIndex]); // ibm_ilog.graphlayout.longlink.LLTerminationPoint
  },

  getStartPointIndex : function()
  {
    return (this.actSPIndex); // int
  },

  getEndPointIndex : function()
  {
    return (this.actEPIndex); // int
  },

  next : function()
  {
    if (this.iterateAtStart)
    {
      if (this.actSPIndex < this.maxSPIndex) {
        this.actSPIndex++;
      }
      else {
        this.incrementMaxStartOrEnd();
      }
    }
    else
    {
      if (this.actEPIndex < this.maxEPIndex) {
        this.actEPIndex++;
      }
      else {
        this.incrementMaxStartOrEnd();
      }
    }
  },

  incrementMaxStartOrEnd : function()
  {
    this.maxSPIndex++;
    this.maxEPIndex++;

    if (this.maxSPIndex < this.numberStartPointCands)
    {
      if (this.maxEPIndex < this.numberEndPointCands)
      {
        if (this.startPointCand[this.maxSPIndex].getPenalty() < this.endPointCand[this.maxEPIndex].getPenalty())
        {
          this.actSPIndex = this.maxSPIndex;
          this.iterateAtStart = false;
          this.maxEPIndex--;
          this.actEPIndex = 0;
        }
        else if (this.startPointCand[this.maxSPIndex].getPenalty() > this.endPointCand[this.maxEPIndex].getPenalty())
        {
          this.actEPIndex = this.maxEPIndex;
          this.iterateAtStart = true;
          this.maxSPIndex--;
          this.actSPIndex = 0;
        }
        else if (this.startPointCand[this.maxSPIndex].getOrder() < this.endPointCand[this.maxEPIndex].getOrder())
        {
          this.actSPIndex = this.maxSPIndex;
          this.iterateAtStart = false;
          this.maxEPIndex--;
          this.actEPIndex = 0;
        }
        else
        {
          this.actEPIndex = this.maxEPIndex;
          this.iterateAtStart = true;
          this.maxSPIndex--;
          this.actSPIndex = 0;
        }
      }
      else
      {
        this.actSPIndex = this.maxSPIndex;
        this.iterateAtStart = false;
        this.maxEPIndex--;
        this.actEPIndex = 0;
      }
    }
    else if (this.maxEPIndex < this.numberEndPointCands)
    {
      this.actEPIndex = this.maxEPIndex;
      this.iterateAtStart = true;
      this.maxSPIndex--;
      this.actSPIndex = 0;
    }
  }

}
);
(function(){ LLLinkRouteAlgorithm._fields_init(); })();

return LLLinkRouteAlgorithm;
});

},
'ibm_ilog/graphlayout/shortlink/_ShortLinkAlgorithm':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_ShortLinkAlgorithm", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/genericquadtree/GenericApplyObject",	// #5
  "../internalutil/genericquadtree/GenericIndexedSet",	// #6
  "../internalutil/genericquadtree/IGenericQuadtreeObject",	// #7
  "../internalutil/ConnectionPointModes",	// #8
  "../internalutil/FixedPoint",	// #9
  "../internalutil/GraphModelUtil",	// #10
  "../internalutil/LayoutParametersUtil",	// #11
  "../internalutil/LinkStyles",	// #12
  "../internalutil/LogResUtil",	// #13
  "../internalutil/Point",	// #14
  "../internalutil/Rectangle",	// #15
  "../internalutil/SplineRouter",	// #16
  "../internalutil/SubgraphData",	// #17
  "../internalutil/TranslateUtil",	// #18
  "../longlink/LongLinkLayout",	// #19
  "../shortlink/_CostComputer",	// #20
  "../shortlink/_DirectIncidentLinksRefiner",	// #21
  "../shortlink/_DirectIncidentLinksSorter",	// #22
  "../shortlink/_DirectLinkShapeProducer",	// #23
  "../shortlink/_LinkLayoutBestNeighborOptimization",	// #24
  "../shortlink/_MultiLinkBundleLayouter",	// #25
  "../shortlink/_NodeSideLayouter",	// #26
  "../shortlink/_OrthogonalIncidentLinksRefiner",	// #27
  "../shortlink/_OrthogonalIncidentLinksSorter",	// #28
  "../shortlink/_OrthogonalLinkShapeProducer",	// #29
  "../shortlink/_SLBundlesModes",	// #30
  "../shortlink/_SLLinkData",	// #31
  "../shortlink/_SLMultiLinkData",	// #32
  "../shortlink/_SLNodeData",	// #33
  "../GraphLayoutReport"	// #34
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GenericApplyObject,	// #5
    GenericIndexedSet,	// #6
    IGenericQuadtreeObject,	// #7
    ConnectionPointModes,	// #8
    FixedPoint,	// #9
    GraphModelUtil,	// #10
    LayoutParametersUtil,	// #11
    LinkStyles,	// #12
    LogResUtil,	// #13
    Point,	// #14
    Rectangle,	// #15
    SplineRouter,	// #16
    SubgraphData,	// #17
    TranslateUtil,	// #18
    LongLinkLayout,	// #19
    _CostComputer,	// #20
    _DirectIncidentLinksRefiner,	// #21
    _DirectIncidentLinksSorter,	// #22
    _DirectLinkShapeProducer,	// #23
    _LinkLayoutBestNeighborOptimization,	// #24
    _MultiLinkBundleLayouter,	// #25
    _NodeSideLayouter,	// #26
    _OrthogonalIncidentLinksRefiner,	// #27
    _OrthogonalIncidentLinksSorter,	// #28
    _OrthogonalLinkShapeProducer,	// #29
    _SLBundlesModes,	// #30
    _SLLinkData,	// #31
    _SLMultiLinkData,	// #32
    _SLNodeData,	// #33
    GraphLayoutReport	// #34
    ) {

var _ShortLinkAlgorithm = declare("ibm_ilog.graphlayout.shortlink._ShortLinkAlgorithm", null, 
{
  // summary:
  //		This class is the main internal class for the Short Link Layout
  //		algorithm.  
  // tags:
  //		private
  
  _linkOffset: null,
  _minFinalSegment: null,

  _bundleMode: null,

  _animate: null,

  _redraw: null,

  _incrementalMode: null,

  _sameShapeForMultipleLinks: null,

  _globalLinkStyle: null,

  _globalOriginPointMode: null,

  _globalDestinationPointMode: null,

  _globalConnectorStyle: null,

  _globalIncrementalUnmodifiedLinkReshapeMode: null,

  _globalIncrementalModifiedLinkReshapeMode: null,
  _isPreserveFixedLinks: null,

  _multipleLinksFound: null,

  _stoppedBeforeCompletion: null,

  _costComputer: null,

  _linkSorter: null,

  _linkRefiner: null,

  _orthogonalShapeProducer: null,

  _directShapeProducer: null,

  _shapeProducer: null,

  _nodeSideLayouter: null,

  _bundleLayouter: null,

  _optimizer: null,

  _shortLinkLayout: null,

  _longLinkLayout: null,

  _vectNodesWithIncidentLinks: null,

  _vectNonFixedShapeLinksWithIntersecting: null,

  _vectNonFixedShapeLinksWithoutIntersecting: null,

  _vectNonFixedUniqueShapeLinks: null,

  _tempPoint1: null,
  _tempPoint2: null,
  _tempPoints1: null,
  _tempPoints2: null,
  _tempRect1: null,
  _tempRect2: null,

  _quadtree: null,

  _intersectApplyObject: null,

  _objectInterface: null,

  _originalGraphModel: null,

  _interGraphLinksRoutingModel: null,
  _duringLongLayout : null,

  constructor : function(layout)
  {
    this._duringLongLayout = false;
    this._shortLinkLayout = layout;
    this._tempPoint1 = new Point();
    this._tempPoint2 = new Point();
    var nPoints = _OrthogonalLinkShapeProducer.MAX_NUMBER_OF_BENDS + 2;
    this._tempPoints1 = TranslateUtil.newArray(nPoints, null);
    this._tempPoints2 = TranslateUtil.newArray(nPoints, null);
    var i;

    for (i = 0; i < this._tempPoints1.length; i++)
      this._tempPoints1[i] = new Point();

    for (i = 0; i < this._tempPoints2.length; i++)
      this._tempPoints2[i] = new Point();
    this._tempRect1 = new Rectangle();
    this._tempRect2 = new Rectangle();
    this._objectInterface = new _ShortLinkAlgorithm.QObjectInterface();
    this._orthogonalShapeProducer = new _OrthogonalLinkShapeProducer(this);
    this._directShapeProducer = new _DirectLinkShapeProducer(this);
    this._nodeSideLayouter = new _NodeSideLayouter();
    this._bundleLayouter = new _MultiLinkBundleLayouter();
    this._costComputer = new _CostComputer();
    this._originalGraphModel = this._shortLinkLayout.getGraphModel();
    this._longLinkLayout = new ShortLinkAlgorithmAnonClass_1(this);
    this._longLinkLayout.setParentLayout(this.getShortLinkLayout());
    this._longLinkLayout.setGlobalOriginPointMode(LongLinkLayout.MIXED_MODE);
    this._longLinkLayout.setGlobalDestinationPointMode(LongLinkLayout.MIXED_MODE);
    this._longLinkLayout.setPreserveFixedLinks(true);

    this._longLinkLayout.setInterGraphLinksMode(layout.isInterGraphLinksMode());
    this._longLinkLayout.setCombinedInterGraphLinksMode(layout.isCombinedInterGraphLinksMode());
  },
  
  getShortLinkLayout : function()
  {
    return this._shortLinkLayout; // ibm_ilog.graphlayout.shortlink.ShortLinkLayout
  },

  getGraphModel : function()
  {
    // summary:
    //		Returns the graph model currently attached to the layout instance. 
    // description:
    //		Returns the graph model currently attached to the layout instance. 
    //		This can be a SubgraphData.
    // tags:
    //		private
    
    return this._shortLinkLayout.getGraphModel(); // ibm_ilog.graphlayout.AbstractGraphLayoutModel
  },

  getOriginalGraphModel : function()
  {
    // summary:
    //		Returns the graph model originally attached to the layout instance.
    // tags:
    //		private
    
    return this._originalGraphModel; // ibm_ilog.graphlayout.AbstractGraphLayoutModel
  },

  getInterGraphLinksRoutingModel : function()
  {
    // summary:
    //		Returns the graph model internally built for routing intergraph
    //		links, or null if none.
    // tags:
    //		private
    
    return this._interGraphLinksRoutingModel; // ibm_ilog.graphlayout.internalutil.SubgraphData
  },

  isIncrementalMode : function()
  {
    return this._incrementalMode; // Boolean
  },

  getLinkOffset : function()
  {
    return this._linkOffset; // Number
  },

  detach : function()
  {
    if (this.getGraphModel() != null) {
      this.cleanInternalData();
    }
    if (this._longLinkLayout != null) {
      this._longLinkLayout.detach();
    }
  },

  getTempPoint1 : function()
  {
    // summary:
    //		Shared (per layout instance) internal point.
    // tags:
    //		private
    
    return this._tempPoint1; // Point
  },

  getTempPoint2 : function()
  {
    // summary:
    //		Shared (per layout instance) internal point.
    // tags:
    //		private
    
    return this._tempPoint2; // Point
  },

  getTempPoints1 : function()
  {
    // summary:
    //		Shared (per layout instance) internal points.
    // tags:
    //		private
    
    return this._tempPoints1; // Array
  },

  getTempPoints2 : function()
  {
    // summary:
    //		Shared (per layout instance) internal points.
    // tags:
    //		private
    
    return this._tempPoints2; // Array
  },

  getTempRect1 : function()
  {
    // summary:
    //		Shared (per layout instance) internal rect.
    // tags:
    //		private
    
    return this._tempRect1; // Rectangle
  },

  getTempRect2 : function()
  {
    // summary:
    //		Shared (per layout instance) internal rect.
    // tags:
    //		private
    
    return this._tempRect2; // Rectangle
  },

  cleanInternalData : function()
  {
    // summary:
    //		Gets called after each layout and when detaching.
    // tags:
    //		private
    
    this._optimizer = null;
    this._orthogonalShapeProducer.clean();
    this._directShapeProducer.clean();
    if (this._linkSorter != null) {
      this._linkSorter.clean();
    }
    if (this._linkRefiner != null) {
      this._linkRefiner.clean();
    }
    if (this._nodeSideLayouter != null) {
      this._nodeSideLayouter.clean();
    }
    if (this._bundleLayouter != null) {
      this._bundleLayouter.clean();
    }
    this._vectNodesWithIncidentLinks = null;
    this._vectNonFixedShapeLinksWithIntersecting = null;
    this._vectNonFixedShapeLinksWithoutIntersecting = null;
    this._vectNonFixedUniqueShapeLinks = null;
    this._quadtree = null;
    this._intersectApplyObject = null;
    this._originalGraphModel = null;
    this._interGraphLinksRoutingModel = null;
  },

  layout : function(redraw)
  {
    // summary:
    //		Computes the layout using the Short Link Layout algorithm. 
    // description:
    //		Computes the layout using the Short Link Layout algorithm. 
    //		
    // tags:
    //		private
    
    if (this._duringLongLayout) {
      return;
    }
    this._originalGraphModel = this.getGraphModel();
    this.update(redraw);

    if (this._shortLinkLayout.isInterGraphLinksMode())
    {
      if ((new TranslateUtil.ArrayIterator(this._originalGraphModel.getIntergraphLinks())).hasNext())
      {
        this._interGraphLinksRoutingModel = new SubgraphData();
        this._interGraphLinksRoutingModel.collectData(this.getGraphModel());
        this._interGraphLinksRoutingModel.setNormalLinksFixed(!this._shortLinkLayout.isCombinedInterGraphLinksMode());
        this._shortLinkLayout.setGraphModel(this._interGraphLinksRoutingModel);
      }
      else
      {
        if (!this._shortLinkLayout.isCombinedInterGraphLinksMode())
        {
          this._shortLinkLayout.getLayoutReport().setCode(GraphLayoutReport.LAYOUT_DONE);
          return;
        }
      }
    }

    try
    {
      this.updateBeforeLayout();
      this.doLayout(this.getGraphModel(), this._shortLinkLayout.getLayoutReport(), redraw);
    }
    finally
    {
      if (this._interGraphLinksRoutingModel != null) {
        this._interGraphLinksRoutingModel.clean();
      }
      this._shortLinkLayout.setGraphModel(this._originalGraphModel);

      this.cleanInternalData();
    }
  },

  doLayout : function(graphModel, layoutReport, redraw)
  {
    // summary:
    //		
    // tags:
    //		private
    
    if (this.mayContinue()) {
      this.computeConnectionPoints(this._sameShapeForMultipleLinks);
    }
    var numberOfIter = this._shortLinkLayout.getAllowedNumberOfIterations();

    if (numberOfIter > 0)
    {
      var allowedTime = this._shortLinkLayout.getAllowedTime();

      this.optimize(this._vectNonFixedShapeLinksWithIntersecting, numberOfIter, allowedTime);

      this.optimize(this._vectNonFixedShapeLinksWithoutIntersecting, 1, allowedTime);
      this._shortLinkLayout.callLayoutStepPerformedIfNeeded();

      this.computeConnectionPoints(this._sameShapeForMultipleLinks);

      this._shortLinkLayout.callLayoutStepPerformedIfNeeded();
    }

    var needsLongLinkLayout = this._shortLinkLayout.getLinkToNodeCrossingPenalty() > 0 && this._shortLinkLayout.isLinkOverlapNodesForbidden();

    this.finalShapeLinks(graphModel, !needsLongLinkLayout, true, redraw);

    if (needsLongLinkLayout && this.mayContinue())
    {
      this._duringLongLayout = true;

      this._shortLinkLayout.setGraphModel(this._originalGraphModel);

      if (this._interGraphLinksRoutingModel != null)
      {
        this._interGraphLinksRoutingModel.clean();
        this._interGraphLinksRoutingModel = null;
      }

      try
      {
        this.doLongLinkLayout(this._originalGraphModel, redraw);
      }
      finally
      {
        this._duringLongLayout = false;
      }
    }

    var gbls = this._shortLinkLayout.getGlobalLinkStyle();

    if (this.mayContinue() && this._shortLinkLayout.isSplineRoutingEnabled() && gbls != LinkStyles.NO_RESHAPE_STYLE)
    {
      var router = new ShortLinkAlgorithmAnonClass_2(this._shortLinkLayout, null, 0, this._shortLinkLayout.getMinSplineCurveSize(), this._shortLinkLayout.getMaxSplineCurveSize(),
          this._shortLinkLayout.getBalanceSplineCurveThreshold(), true, false, this);
      router.run(redraw, true);
    }

    layoutReport.setCode((this._stoppedBeforeCompletion ? GraphLayoutReport.STOPPED_AND_INVALID : GraphLayoutReport.LAYOUT_DONE));
  },

  doLongLinkLayout : function(graphModel, redraw)
  {
    // summary:
    //		As a post-processing step, executes the additional refinement of
    //		links that still overlap at least a node, using the LongLinklayout. 
    //		
    // graphModel: AbstractGraphLayoutModel
    //		
    // redraw: Boolean
    //		
    // tags:
    //		private
    
    if (this._longLinkLayout.getGraphModel() != graphModel)
    {
      var modelChecking = TranslateUtil.GraphModelData.getData(graphModel).internalGraphModelChecking;

      TranslateUtil.GraphModelData.setInternalGraphModelChecking(graphModel, false);
      this._longLinkLayout.attach(graphModel);

      TranslateUtil.GraphModelData.setInternalGraphModelChecking(graphModel, modelChecking);
    }

    this._longLinkLayout.setSplineRoutingEnabled(false);
    this._longLinkLayout.setAllowedTime(this._shortLinkLayout.getRemainingAllowedTime());
    this._longLinkLayout.performLayout(true);

    this.removeProperties();
  },

  getAuxiliaryLongLinkLayout : function()
  {
    return this._longLinkLayout; // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  optimize : function(vectLinks, numberOfIter, allowedTime)
  {
    // summary:
    //		Performs the optimization of the shape of a set of links.  
    // tags:
    //		private
    
    if (vectLinks == null || vectLinks.length == 0) {
      return;
    }
    this._optimizer = new _LinkLayoutBestNeighborOptimization(this, numberOfIter, allowedTime, vectLinks);
    this._optimizer.optimize();
    if (this._optimizer.isStoppedBeforeCompletion()) {
      this._stoppedBeforeCompletion = true;
    }
    this._optimizer = null;
  },

  update : function(redraw)
  {
    // summary:
    //		Updates according to the current values in ShortLinkLayout.
    // tags:
    //		private
    
    this._stoppedBeforeCompletion = false;
    this._incrementalMode = this._shortLinkLayout.isIncrementalMode();
    this._sameShapeForMultipleLinks = this._shortLinkLayout.isSameShapeForMultipleLinks();
    this._redraw = redraw;
    this._multipleLinksFound = false;
    this._linkOffset = this._shortLinkLayout.getLinkOffset();
    this._minFinalSegment = this._shortLinkLayout.getMinFinalSegmentLength();
    this._bundleMode = this._shortLinkLayout.getLinkBundlesMode();
    this._globalLinkStyle = this._shortLinkLayout.getGlobalLinkStyle();
    this._globalOriginPointMode = this._shortLinkLayout.getGlobalOriginPointMode();
    this._globalDestinationPointMode = this._shortLinkLayout.getGlobalDestinationPointMode();
    this._globalIncrementalUnmodifiedLinkReshapeMode = this._shortLinkLayout.getGlobalIncrementalUnmodifiedLinkReshapeMode();
    this._globalIncrementalModifiedLinkReshapeMode = this._shortLinkLayout.getGlobalIncrementalModifiedLinkReshapeMode();
    this._globalConnectorStyle = this._shortLinkLayout.getGlobalConnectorStyle();
    this._isPreserveFixedLinks = this._shortLinkLayout.isPreserveFixedLinks();
  },

  updateBeforeLayout : function()
  {
    // summary:
    //		Updates necessary before any layout. 
    // description:
    //		Updates necessary before any layout. 
    //		
    // tags:
    //		private
    
    if (this._interGraphLinksRoutingModel != null)
    {
      this._incrementalMode = false;
    }

    if (!this._shortLinkLayout.isParametersUpToDate()) {
      this._incrementalMode = false;
    }
    if (this._intersectApplyObject == null) {
      this._intersectApplyObject = new _ShortLinkAlgorithm.IntersectApplyObject();
    }
    this.initializeInternalData(this._shortLinkLayout.getGraphModel());

    this._linkSorter = this.makeIncidentLinksSorter();

    this._linkRefiner = this.makeIncidentLinksRefiner();
  },

  updateUtilities : function()
  {
    // summary:
    //		Updates information stored in various internal classes to which
    //		ShortLinkLayout delegates the layout job. 
    // description:
    //		Updates information stored in various internal classes to which
    //		ShortLinkLayout delegates the layout job. 
    //		Basically, these are parameters of ShortLinkLayout that are stored
    //		locally for efficiency reasons.
    // tags:
    //		private
    
    this.updateLinkShapeProducer();
    this._costComputer.update(this);
    this._nodeSideLayouter.update(this);
  },

  updateLinkShapeProducer : function()
  {
    this._orthogonalShapeProducer.update(this);
    this._directShapeProducer.update(this);

    switch (this._globalLinkStyle)
    {
      case LinkStyles.ORTHOGONAL_STYLE:
        this._shapeProducer = this._orthogonalShapeProducer;
        break;
      case LinkStyles.DIRECT_STYLE:
        this._shapeProducer = this._directShapeProducer;
        break;
      case LinkStyles.MIXED_STYLE:
        this._shapeProducer = null;
        break;
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6633E",  this._globalLinkStyle);
    }
  },

  getCostComputer : function()
  {
    // summary:
    //		Returns the CostComputer object.
    // tags:
    //		private
    
    return this._costComputer; // ibm_ilog.graphlayout.shortlink._CostComputer
  },

  getLinkNodeCost : function(graphModel, linkData)
  {
    // summary:
    //		Returns the cost induced by the current shape of linkData. 
    // description:
    //		Returns the cost induced by the current shape of linkData. 
    //		Only link-to-node overlaps are taken into account (and not
    //		link-to-link crossings). The linkToNodeCrossingPanalty is applied.
    //		This method is used for deciding whether applying the Long Link
    //		laytout is necessary for a particular link.
    // tags:
    //		private
    
    return this._costComputer.getLinkCost(graphModel, this._interGraphLinksRoutingModel, linkData, true, false); // Number
  },

  getLinkCost : function(graphModel, linkData, global)
  {
    // summary:
    //		Returns the cost induced by the current shape of linkData. 
    // description:
    //		Returns the cost induced by the current shape of linkData. 
    //		This only counts the intersections/crossings of linkData with other
    //		nodes/links.
    // tags:
    //		private
    
    return this._costComputer.getLinkCost(graphModel, this._interGraphLinksRoutingModel, linkData, false, global); // Number
  },

  isFixedOrNoReshape : function(link)
  {
    // summary:
    //		Returns 'true' if the link must not be reshaped.
    // tags:
    //		private
    
    if (!this._shortLinkLayout.isLayoutRunning()) {
      this.update(false);
    }
    if (this._interGraphLinksRoutingModel != null && this._interGraphLinksRoutingModel.isFixedLink(link)) {
      return true; // Boolean
    }
    return (this._isPreserveFixedLinks && this._shortLinkLayout.isFixed(link)) || this._globalLinkStyle == LinkStyles.NO_RESHAPE_STYLE ||
      (this._globalLinkStyle == LinkStyles.MIXED_STYLE && this._shortLinkLayout.getLinkStyle(link) == LinkStyles.NO_RESHAPE_STYLE); // Boolean
  },

  getConnectorStyle : function(node)
  {
    // summary:
    //		Returns the connector style for a particular node.
    // tags:
    //		private
    
    return (this._globalConnectorStyle == ConnectionPointModes.MIXED_MODE ? this._shortLinkLayout.getConnectorStyle(node) : this._globalConnectorStyle); // int
  },

  getIncrementalUnmodifiedLinkReshapeMode : function(link)
  {
    // summary:
    //		Returns the incremental link reshape mode for a particular
    //		unmodified link.
    // tags:
    //		private
    
    return (this._globalIncrementalUnmodifiedLinkReshapeMode == ConnectionPointModes.MIXED_MODE ? this._shortLinkLayout.getIncrementalUnmodifiedLinkReshapeMode(link) : 
        this._globalIncrementalUnmodifiedLinkReshapeMode); // int
  },

  getIncrementalModifiedLinkReshapeMode : function(link)
  {
    // summary:
    //		Returns the incremental link reshape mode for a particular modified
    //		link.
    // tags:
    //		private
    
    return (this._globalIncrementalModifiedLinkReshapeMode == ConnectionPointModes.MIXED_MODE
      ? this._shortLinkLayout.getIncrementalModifiedLinkReshapeMode(link)
      : this._globalIncrementalModifiedLinkReshapeMode); // int
  },

  notifyMixedLinkStyle : function()
  {
    // summary:
    //		Method called when the link style becomes "mixed".
    // tags:
    //		private
    
    this._shapeProducer = null;
  },

  getLinkShapeProducer : function(link)
  {
    // summary:
    //		Returns the link shape producer to be used for 'link'. 
    // description:
    //		Returns the link shape producer to be used for <code>link</code>. 
    //		When MIXED&#95;STYLE is NOT choosen, &#95;shapeProducer != null is returned.
    //		Otherwise, the method returns the appropriate producer depending on
    //		the individual link style option.   
    // tags:
    //		private
    
    if (this._shapeProducer != null) {
      return this._shapeProducer; // ibm_ilog.graphlayout.shortlink._LinkShapeProducer
    }
    switch (this._shortLinkLayout.getLinkStyle(link))
    {
      case LinkStyles.ORTHOGONAL_STYLE:
        return this._orthogonalShapeProducer; // ibm_ilog.graphlayout.shortlink._LinkShapeProducer
      case LinkStyles.DIRECT_STYLE:
      case LinkStyles.NO_RESHAPE_STYLE:
        return this._directShapeProducer; // ibm_ilog.graphlayout.shortlink._LinkShapeProducer
      default:
        var params = [this._shortLinkLayout.getLinkStyle(link), link ];
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6634E", params);
        return null; // ibm_ilog.graphlayout.shortlink._LinkShapeProducer
    }
  },

  makeIncidentLinksSorter : function()
  {
    // summary:
    //		Returns a new instance of the incident links sorter. 
    // description:
    //		Returns a new instance of the incident links sorter. 
    //		
    // tags:
    //		private
    
    switch (this._globalLinkStyle)
    {
      case LinkStyles.ORTHOGONAL_STYLE:
        return new _OrthogonalIncidentLinksSorter(this); // ibm_ilog.graphlayout.shortlink._IncidentLinksSorter
      case LinkStyles.DIRECT_STYLE:
      case LinkStyles.MIXED_STYLE:
        return new _DirectIncidentLinksSorter(this); // ibm_ilog.graphlayout.shortlink._IncidentLinksSorter
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6633E",  this._globalLinkStyle);
        return null; // ibm_ilog.graphlayout.shortlink._IncidentLinksSorter
    }
  },

  makeIncidentLinksRefiner : function()
  {
    // summary:
    //		Returns a new instance of the incident links refiner. 
    // description:
    //		Returns a new instance of the incident links refiner. 
    //		
    // tags:
    //		private
    
    switch (this._globalLinkStyle)
    {
      case LinkStyles.ORTHOGONAL_STYLE:
        return new _OrthogonalIncidentLinksRefiner(this); // ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner
      case LinkStyles.DIRECT_STYLE:
      case LinkStyles.MIXED_STYLE:
        return new _DirectIncidentLinksRefiner(this); // ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6633E",  this._globalLinkStyle);
        return null; // ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner
    }
  },

  initializeInternalData : function(graphModel)
  {
    // summary:
    //		Fixed links are not added to _vectNonFixedShapeLinksWithIntersecting
    //		(they are not decision variables), but fixed links can be
    //		intersecting objects of a link. 
    // description:
    //		Fixed links are not added to &#95;vectNonFixedShapeLinksWithIntersecting
    //		(they are not decision variables), but fixed links can be
    //		intersecting objects of a link. 
    //		Therefore, an intersecting object can not have the
    //		LINK&#95;INDEX&#95;PROPERTY.   
    // tags:
    //		private
    
    this.updateUtilities();

    var nLinks = graphModel.getLinksCount();
    var nNodes = graphModel.getNodesCount();

    if (this._vectNodesWithIncidentLinks == null) {
      this._vectNodesWithIncidentLinks = new Array();
    }
    else
    {
      TranslateUtil.noop();
      this._vectNodesWithIncidentLinks.length = 0;
    }

    if (this._vectNonFixedShapeLinksWithIntersecting == null) {
      this._vectNonFixedShapeLinksWithIntersecting = new Array();
    }
    else
    {
      TranslateUtil.noop();
      this._vectNonFixedShapeLinksWithIntersecting.length = 0;
    }

    if (this._vectNonFixedShapeLinksWithoutIntersecting == null) {
      this._vectNonFixedShapeLinksWithoutIntersecting = new Array();
    }
    else
    {
      TranslateUtil.noop();
      this._vectNonFixedShapeLinksWithoutIntersecting.length = 0;
    }

    if (this._vectNonFixedUniqueShapeLinks == null) {
      this._vectNonFixedUniqueShapeLinks = new Array();
    }
    else
    {
      TranslateUtil.noop();
      this._vectNonFixedUniqueShapeLinks.length = 0;
    }

    if (this._quadtree == null) {
      this._quadtree = new GenericIndexedSet(20, 20, this._objectInterface);
    }
    else {
      this._quadtree.deleteAll();
    }

    var nodes = new TranslateUtil.ArrayIterator(graphModel.getNodes());
    var node;
    var nodeData;

    while (nodes.hasNext())
    {
      node = nodes.next();
      nodeData = this.getOrCreateNodeData(graphModel, node, true);

      this._quadtree.addObject(nodeData);
      if (nodeData.getDegree() > 0) {
        this._vectNodesWithIncidentLinks.push(nodeData);
      }
    }

    var links = new TranslateUtil.ArrayIterator(graphModel.getLinks());
    var vectLinks = new Array();
    var link;
    var linkData;
    var i;

    while (links.hasNext())
    {
      link = links.next();
      linkData = this.getOrCreateLinkData(graphModel, link);
      vectLinks.push(linkData);
    }

    nLinks = vectLinks.length;

    if (this._sameShapeForMultipleLinks)
    {
      var newVectLinks = new Array();
      var linkToUse;
      var multiLinkData;

      for (i = 0; i < nLinks; i++)
      {
        linkData = vectLinks[i];
        linkData.computeLinkRegion(this._minFinalSegment, this._linkOffset, this._bundleMode);

        if (!linkData.isSlave())
        {
          multiLinkData = this.searchLinksWithSameEndPoints(graphModel, linkData);
          linkToUse = (multiLinkData == null ? linkData : multiLinkData);
          newVectLinks.push(linkToUse);
          this._quadtree.addObject(linkToUse);
        }
      }

      vectLinks = newVectLinks;

      newVectLinks = null;
    }
    else
    {
      for (i = 0; i < nLinks; i++)
      {
        linkData = vectLinks[i];
        linkData.computeLinkRegion(this._minFinalSegment, this._linkOffset, this._bundleMode);
        this._quadtree.addObject(linkData);
      }
    }

    var counter = 0;
    nLinks = vectLinks.length;

    for (i = 0; i < nLinks; i++)
    {
      linkData = vectLinks[i];

      if (linkData.isFixed()) {
        continue;
      }
      if (this._incrementalMode)
      {
        if (this._sameShapeForMultipleLinks && linkData.isSlave()) {
          this._multipleLinksFound = true;
        }
        if (linkData.isShapeTypeFixed())
        {
          this._vectNonFixedUniqueShapeLinks.push(linkData);
          linkData.setIndex(/*Number.MAX_VALUE*/2147483647);

          continue;
        }
      }

      linkData.restoreCurrentShape(this, this._minFinalSegment, this._linkOffset, false);

      if (linkData.getShapeTypesCount() > 1)
      {
        var intersectingObjects = this.computeIntersectingObjects(linkData);

        if (intersectingObjects != null) {
          this._vectNonFixedShapeLinksWithIntersecting.push(linkData);
        }
        else {
          this._vectNonFixedShapeLinksWithoutIntersecting.push(linkData);
        }
        linkData.setIntersectingObjects(intersectingObjects);
      }
      else {
        this._vectNonFixedUniqueShapeLinks.push(linkData);
      }
      linkData.setIndex(counter++);
    }

    this._shortLinkLayout.callLayoutStepPerformedIfNeeded();
  },

  getOrCreateNodeData : function(graphModel, node, update)
  {
    // summary:
    //		Gets or creates the NodeData corresponding to a node.  
    // update: Boolean
    //		If true, an NodeData already stored by the node is
    //		updated.
    // tags:
    //		private
    
    var nodeData = this.getNodeData(node);

    if (nodeData == null || nodeData.nodeOrLink != node)
    {
      if (!update)
      {
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6633E");
      }

      nodeData = new _SLNodeData(node);

      LayoutParametersUtil.setNodeOrLinkProperty(this._shortLinkLayout, node, null, false, false, _ShortLinkAlgorithm.NODE_DATA_PROPERTY, nodeData);
    }

    if (update) {
      nodeData.update(this.getNodeBox(graphModel, node), this.getLinkConnectionRectangle(graphModel, node), this.getConnectorStyle(node), graphModel, this._incrementalMode);
    }
    return nodeData; // ibm_ilog.graphlayout.shortlink._SLNodeData
  },

  getNodeData : function(node)
  {
    // summary:
    //		Gets the NodeData corresponding to a node, if one has been already
    //		stored. 
    // description:
    //		Gets the NodeData corresponding to a node, if one has been already
    //		stored. 
    //		Returns null otherwise.   
    // tags:
    //		private
    
    return LayoutParametersUtil.getNodeParameter(this._shortLinkLayout, node, _ShortLinkAlgorithm.NODE_DATA_PROPERTY); // ibm_ilog.graphlayout.shortlink._SLNodeData
  },

  getNodeBox : function(graphModel, node)
  {
    // summary:
    //		Returns the box of the node, to be taken into account for computing
    //		the crossings. 
    // description:
    //		Returns the box of the node, to be taken into account for computing
    //		the crossings. 
    //		
    // tags:
    //		private
    
    var nodeBoxInterface = this._shortLinkLayout.getNodeBoxInterface();
    if (nodeBoxInterface == null) {
      return GraphModelUtil.getObjectBounds(graphModel, node); // Rectangle
    }
    if (graphModel instanceof SubgraphData) {
      return (graphModel).getNodeBox(nodeBoxInterface, node); // Rectangle
    }
    return nodeBoxInterface.getBox(graphModel, node); // Rectangle
  },

  getLinkConnectionRectangle : function(graphModel, node)
  {
    // summary:
    //		Returns the rectangle on which the connection points lie on node.
    // tags:
    //		private
    
    var connectionBoxInterface = this._shortLinkLayout.getLinkConnectionBoxInterface();
    if (connectionBoxInterface == null) {
      return this.getNodeBox(graphModel, node); // Rectangle
    }
    if (graphModel instanceof SubgraphData) {
      return (graphModel).getNodeBox(connectionBoxInterface, node); // Rectangle
    }
    return connectionBoxInterface.getBox(graphModel, node); // Rectangle
  },

  searchLinksWithSameEndPoints : function(graphModel, linkData)
  {
    // summary:
    //		Searches for other links in the graph that have the same origin and
    //		destination (in any order) than 'link'.  
    // tags:
    //		private
    
    var multiLinkData = null;

    if (this.canBeMasterOrSlave(linkData))
    {
      var fromNode = linkData.getFromNode().nodeOrLink;

      multiLinkData = this.searchMultipleLinks(graphModel, (new TranslateUtil.ArrayIterator(graphModel.getLinksFrom(fromNode))), null, linkData);

      multiLinkData = this.searchMultipleLinks(graphModel, (new TranslateUtil.ArrayIterator(graphModel.getLinksTo(fromNode))), multiLinkData, linkData);
    }

    return multiLinkData; // ibm_ilog.graphlayout.shortlink._SLMultiLinkData
  },

  canBeMasterOrSlave : function(linkData)
  {
    // summary:
    //		Returns true if the link can be master or slave in a multilink
    //		bundle, false otherwise. 
    // description:
    //		Returns true if the link can be master or slave in a multilink
    //		bundle, false otherwise. 
    //		
    // tags:
    //		private
    
    return !(linkData.isFixed() || linkData.isFromPointFixed() || linkData.isToPointFixed()); // Boolean
  },

  searchMultipleLinks : function(graphModel, links, multiLinkData, linkData)
  {
    // summary:
    //		Searches inside the enumeration of links (always from or to links of
    //		'node') the one that have the same origin and destination
    //		node (in any order) than 'link'.  
    // links: IIterator
    //		an enumeration of links incident to 'fromNode'
    //		(we call once with from links, and once with to links).
    // multiLinkData: _SLMultiLinkData
    //		the instance of MultiLinkData into which the
    //		slave links, if any, will be added, or null.
    // linkData: _SLLinkData
    //		the link for which we search other links with the same
    //		origin and destination.
    // returns:
    //		the MultiLinkData containing the slave links, or null if none
    //		has been found.
    // tags:
    //		private
    
    var otherLink;
    var otherNode;
    var otherLinkData;

    while (links.hasNext())
    {
      otherLink = links.next();
      if (otherLink == linkData.nodeOrLink) {
        continue;
      }
      otherNode = GraphModelUtil.getOpposite(graphModel, otherLink, linkData.getFromNode().nodeOrLink);

      if (otherNode == linkData.getToNode().nodeOrLink)
      {
        otherLinkData = this.getLinkData(otherLink);

        if (!otherLinkData.isSlave() && this.canBeMasterOrSlave(otherLinkData))
        {
          if (multiLinkData == null)
          {
            multiLinkData = new _SLMultiLinkData(this, linkData, this._linkOffset);
            multiLinkData.computeLinkRegion(this._minFinalSegment, this._linkOffset, this._bundleMode);
          }

          multiLinkData.addSlave(otherLinkData);
          this._multipleLinksFound = true;
        }
      }
    }

    return multiLinkData; // ibm_ilog.graphlayout.shortlink._SLMultiLinkData
  },

  getOrCreateLinkData : function(graphModel, link)
  {
    // summary:
    //		Returns an LinkData for this link. 
    // description:
    //		Returns an LinkData for this link. 
    //		If it has not been already computed and stored for this link, the
    //		LinkData is computed and then stored in a property of the link.
    //		Otherwise, the method returns the old LinkData.   
    // tags:
    //		private
    
    var linkData = this.getLinkData(link);

    if (linkData == null || linkData.nodeOrLink != link)
    {
      linkData = new _SLLinkData(this, link);

      LayoutParametersUtil.setNodeOrLinkProperty(this._shortLinkLayout, link, null, false, false, _ShortLinkAlgorithm.LINK_DATA_PROPERTY, linkData);
    }
    else {
      linkData.update(this, link, null);
    }

    var fromNode = this.getOrCreateNodeData(graphModel, graphModel.getFrom(link), false);
    var toNode = this.getOrCreateNodeData(graphModel, graphModel.getTo(link), false);
    fromNode.setTotalIncidentLinkWidth(fromNode.getTotalIncidentLinkWidth() + linkData.getLinkWidth());
    toNode.setTotalIncidentLinkWidth(toNode.getTotalIncidentLinkWidth() + linkData.getLinkWidth());
    return linkData; // ibm_ilog.graphlayout.shortlink._SLLinkData
  },

  getLinkData : function(link)
  {
    // summary:
    //		Gets the LinkData corresponding to a link, if one has been already
    //		stored. 
    // description:
    //		Gets the LinkData corresponding to a link, if one has been already
    //		stored. 
    //		Returns null otherwise. Non-private for performance reasons.
    // tags:
    //		private
    
    return LayoutParametersUtil.getLinkParameter(this._shortLinkLayout, link, _ShortLinkAlgorithm.LINK_DATA_PROPERTY); // ibm_ilog.graphlayout.shortlink._SLLinkData
  },

  computeIntersectingObjects : function(linkData)
  {
    // summary:
    //		The method first computes the link region. 
    // description:
    //		The method first computes the link region. 
    //		Then, the link region is checked for intersection with the link
    //		region of all other links and with the bounding box of all the nodes
    //		(except the from- and to-node of the link). Each node or link for
    //		which there is intersection is stored in the vector that will be
    //		returned by this method. NOTE THAT THIS METHOD SHOULD NOT BE CALLED
    //		FOR FIXED LINKS: it would be a waste of time for nothing, since we
    //		do not need to compute an optimal shape for such links.  
    // returns:
    //		the objects that may intersect the link region of link; returns
    //		null if no such object.
    // tags:
    //		private
    
    var linkRegion = linkData.boundingBox;
    this._intersectApplyObject.init();
    this._quadtree.mapIntersects(linkRegion, this._intersectApplyObject, linkData);
    var intersectingObjects = this._intersectApplyObject.getIntersectingObjects();

    return intersectingObjects; // Array
  },

  reshapeLink : function(graphModel, linkData, shapeType, isLinkBundlingRequired)
  {
    // summary:
    //		Reshapes a link to a given shape type. 
    // description:
    //		Reshapes a link to a given shape type. 
    //		Package acess because called in LinkLayoutBestNeighborsOptimization 
    //		 
    // tags:
    //		private
    
    var oldFromSide = linkData.getFromNodeSide();
    var oldToSide = linkData.getToNodeSide();
    var newFromSide = shapeType.getFromNodeSide(linkData);
    var newToSide = shapeType.getToNodeSide(linkData);
    linkData.setCurrentShapeType(shapeType);

    linkData.restoreCurrentShape(this, this._minFinalSegment, this._linkOffset, false);
    if (isLinkBundlingRequired) {
      this.computeLinkConnectionPoints(linkData, oldFromSide, newFromSide, oldToSide, newToSide);
    }
  },

  finalShapeLinks : function(graphModel, removeProperties, slaveExpansion, redraw)
  {
    // summary:
    //		Gives to all the links their final shape, according to the optimal
    //		shape that has been computed. 
    // description:
    //		Gives to all the links their final shape, according to the optimal
    //		shape that has been computed. 
    //		With the same occasion, we remove some properties.  
    // tags:
    //		private
    
    this.finalShapeLinksFromVector(graphModel, this._vectNonFixedShapeLinksWithIntersecting, removeProperties, slaveExpansion, redraw);
    this.finalShapeLinksFromVector(graphModel, this._vectNonFixedShapeLinksWithoutIntersecting, removeProperties, slaveExpansion, redraw);
    this.finalShapeLinksFromVector(graphModel, this._vectNonFixedUniqueShapeLinks, removeProperties, slaveExpansion, redraw);
  },

  finalShapeLinksFromVector : function(graphModel, vectLinks, removeProperties, slaveExpansion, redraw)
  {
    // summary:
    //		Gives to the links stored in vectLinks their final shape.  
    // tags:
    //		private
    
    if (vectLinks == null) {
      return;
    }
    var linkData;
    var nLinks = vectLinks.length;

    for (var i = 0; i < nLinks; i++)
    {
      linkData = vectLinks[i];

      this.finalShapeLink(graphModel, linkData, slaveExpansion, redraw);
      if (removeProperties) {
        linkData.setIntersectingObjects(null);
      }
    }
  },

  removeProperties : function()
  {
    // summary:
    //		We call this when we cannot call finalShapeLinks with
    //		removeProperties at true.
    // tags:
    //		private
    
    this.removeVectorProperties(this._vectNonFixedShapeLinksWithIntersecting);
    this.removeVectorProperties(this._vectNonFixedShapeLinksWithoutIntersecting);
    this.removeVectorProperties(this._vectNonFixedUniqueShapeLinks);
  },

  removeVectorProperties : function(vectLinks)
  {
    // summary:
    //		
    // tags:
    //		private
    
    if (vectLinks == null) {
      return;
    }
    var nLinks = vectLinks.length;

    for (var i = 0; i < nLinks; i++)
      (vectLinks[i]).setIntersectingObjects(null);
  },

  finalShapeLink : function(graphModel, linkData, slaveExpansion, redraw)
  {
    // summary:
    //		Gives a link its final shape, according to the optimal shape that
    //		has been computed. 
    // description:
    //		Gives a link its final shape, according to the optimal shape that
    //		has been computed. 
    //		Not private because used in MultiLinkBundleLayouter.   
    // tags:
    //		private
    
    if (this._multipleLinksFound && slaveExpansion && linkData.isMaster())
    {
      if (linkData instanceof _SLMultiLinkData) {
        this._bundleLayouter.layout(this, graphModel, linkData, this._bundleMode, this._linkOffset, this._minFinalSegment, redraw);
      }
      else
      {
        var params = [linkData ];
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6636E", params);
      }
    }
    else
    {
      var shape = linkData.getLinkShape();

      var fromPoint = shape.getFrom();
      var toPoint = shape.getTo();

      if (linkData.isFromPointFixed() && fromPoint != null) {
        fromPoint = new FixedPoint(fromPoint .x, fromPoint .y);
      }
      if (linkData.isToPointFixed() && toPoint != null) {
        toPoint = new FixedPoint(toPoint .x, toPoint .y);
      }

      var linkStyle = this._globalLinkStyle;
      if (linkStyle == LinkStyles.MIXED_STYLE) {
        linkStyle = this._shortLinkLayout.getLinkStyle(linkData.nodeOrLink);
      }
      GraphModelUtil.reshapeLink(graphModel, this._shortLinkLayout, linkData.nodeOrLink, linkStyle, fromPoint, shape.getIntermediatePoints(), 0, shape.getNumberOfPoints() - 2, toPoint);
    }
  },

  computeConnectionPoints : function (a0, a1)
  {
    // summary:
    //		
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.computeConnectionPoints_overload_1(a0);
    else if(arguments.length == 2)
      return this.computeConnectionPoints_overload_2(a0, a1);
  },
  
  computeConnectionPoints_overload_1 : function(withSlaveLinks)
                                                              
  {
    this.computeConnectionPoints(withSlaveLinks, true);
  },

  computeConnectionPoints_overload_2 : function(withSlaveLinks, restoreShape )
  {
    // summary:
    //		Computes the connection points for all the links of the graph
    //		'graphModel'. 
    // description:
    //		Computes the connection points for all the links of the graph
    //		<code>graphModel</code>. 
    //		This method calls succesively
    //		<code>computeConnectionPoints(Object)</code> on each link. CAUTION :
    //		must be called only after initializeInternalData **********   
    // tags:
    //		private
    
    if (restoreShape)
    {
      this.restoreCurrentShape(this._vectNonFixedShapeLinksWithIntersecting, withSlaveLinks);
      this.restoreCurrentShape(this._vectNonFixedShapeLinksWithoutIntersecting, withSlaveLinks);
      this.restoreCurrentShape(this._vectNonFixedUniqueShapeLinks, withSlaveLinks);
    }

    if (this._bundleMode != _SLBundlesModes.NO_BUNDLE)
    {
      var nNodes = this._vectNodesWithIncidentLinks.length;
      var nodeData;

      var i;

      for (i = 0; i < nNodes; i++)
      {
        nodeData = this._vectNodesWithIncidentLinks[i];
        nodeData.layoutNodeSides(this, this._nodeSideLayouter, this._linkSorter, null, this._shortLinkLayout.getLinkConnectionBoxInterface(), this._linkOffset, this._minFinalSegment, this._bundleMode);
      }

      if (this._linkRefiner != null && this._bundleMode == _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT)
      {
        for (i = 0; i < nNodes; i++)
        {
          nodeData = this._vectNodesWithIncidentLinks[i];
          nodeData.refineNodeSides(this, this._nodeSideLayouter, this._linkRefiner, this._linkOffset, this._minFinalSegment, this._bundleMode);
        }
      }
    }

    this._shortLinkLayout.callLayoutStepPerformedIfNeeded();
  },

  restoreCurrentShape : function(vectLinks, withSlaveLinks)
  {
    // summary:
    //		Restores the currently computed shape of a vector of links. 
    // description:
    //		Restores the currently computed shape of a vector of links. 
    //		
    // tags:
    //		private
    
    var linkData;
    var nLinks = vectLinks.length;

    for (var i = 0; i < nLinks; i++)
    {
      linkData = vectLinks[i];
      if (!(withSlaveLinks && this._multipleLinksFound && linkData.isSlave())) {
        linkData.restoreCurrentShape(this, this._minFinalSegment, this._linkOffset, false);
      }
    }
  },

  computeLinkConnectionPoints : function(linkData, oldFromSide, newFromSide, oldToSide, newToSide)
  {
    // summary:
    
    if (this._bundleMode == _SLBundlesModes.NO_BUNDLE) {
      return;
    }

    var fromNodeData = linkData.getFromNode();
    var toNodeData = linkData.getToNode();
    var connectionBoxInterface = this._shortLinkLayout.getLinkConnectionBoxInterface();

    if (oldFromSide != null) {
      oldFromSide.layout(this, this._nodeSideLayouter, this._linkSorter, null, connectionBoxInterface, fromNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
    }
    if (newFromSide != oldFromSide) {
      newFromSide.layout(this, this._nodeSideLayouter, this._linkSorter, null, connectionBoxInterface, fromNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
    }
    if (oldToSide != null && (toNodeData != fromNodeData || (oldToSide != oldFromSide && oldToSide != newFromSide))) {
      oldToSide.layout(this, this._nodeSideLayouter, this._linkSorter, null, connectionBoxInterface, toNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
    }
    if (newToSide != oldToSide && (toNodeData != fromNodeData || (newToSide != oldFromSide && newToSide != newFromSide))) {
      newToSide.layout(this, this._nodeSideLayouter, this._linkSorter, null, connectionBoxInterface, toNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
    }
    if (this._linkRefiner != null && this._bundleMode == _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT)
    {
      if (oldFromSide != null) {
        oldFromSide.refine(this, this._nodeSideLayouter, this._linkRefiner, fromNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
      }
      if (newFromSide != oldFromSide) {
        newFromSide.refine(this, this._nodeSideLayouter, this._linkRefiner, fromNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
      }
      if (oldToSide != null && (toNodeData != fromNodeData || (oldToSide != oldFromSide && oldToSide != newFromSide))) {
        oldToSide.refine(this, this._nodeSideLayouter, this._linkRefiner, toNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
      }
      if (newToSide != oldToSide && (toNodeData != fromNodeData || (newToSide != oldFromSide && newToSide != newFromSide))) {
        newToSide.refine(this, this._nodeSideLayouter, this._linkRefiner, toNodeData, this._linkOffset, this._minFinalSegment, this._bundleMode);
      }
    }
  },

  isFromPointFixed : function(link)
  {
    // summary:
    //		Returns 'true' if the connection point of
    //		'link' on the origin node must be fixed. 
    // description:
    //		Returns <code>true</code> if the connection point of
    //		<code>link</code> on the origin node must be fixed. 
    //		Returns <code>false</code> otherwise.
    
    switch (this._globalOriginPointMode)
    {
      case ConnectionPointModes.FREE_MODE:
        return this.getGraphModel().hasPinnedConnectionPoint(link, true); // Boolean
      case ConnectionPointModes.FIXED_MODE:
        return true; // Boolean
      case ConnectionPointModes.MIXED_MODE:
        return (this._shortLinkLayout.getOriginPointMode(link) == ConnectionPointModes.FIXED_MODE ? true : this.getGraphModel().hasPinnedConnectionPoint(link, true)); // Boolean
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6637E",  this._globalOriginPointMode);
        return false; // Boolean
    }
  },

  isToPointFixed : function(link)
  {
    // summary:
    //		Returns 'true' if the connection point of
    //		'link' on the destination node must be fixed. 
    // description:
    //		Returns <code>true</code> if the connection point of
    //		<code>link</code> on the destination node must be fixed. 
    //		Returns <code>false</code> otherwise.
    // tags:
    //		private
    
    switch (this._globalDestinationPointMode)
    {
      case ConnectionPointModes.FREE_MODE:
        return this.getGraphModel().hasPinnedConnectionPoint(link, false); // Boolean
      case ConnectionPointModes.FIXED_MODE:
        return true; // Boolean
      case ConnectionPointModes.MIXED_MODE:
        return (this._shortLinkLayout.getDestinationPointMode(link) == ConnectionPointModes.FIXED_MODE ? true : this.getGraphModel().hasPinnedConnectionPoint(link, false)); // Boolean
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6638E",  this._globalDestinationPointMode);
        return false; // Boolean
    }
  },

  stopImmediately : function()
  {
    this._stoppedBeforeCompletion = true;
    if (this._optimizer != null) {
      this._optimizer.stopImmediately();
    }
  },

  mayContinue : function()
  {
    if (this._stoppedBeforeCompletion) {
      return false; // Boolean
    }
    var res = !this._shortLinkLayout.isLayoutTimeElapsedPackage();
    if (!res) {
      this._stoppedBeforeCompletion = true;
    }
    return res; // Boolean
  }

}
);

lang.mixin(_ShortLinkAlgorithm,
{
  LINK_DATA_PROPERTY : null,
  NODE_DATA_PROPERTY : null,
  _fields_init : function(c)
  {
    c = c || _ShortLinkAlgorithm;
    
    c.LINK_DATA_PROPERTY = "LinkData";
    c.NODE_DATA_PROPERTY = "NodeData";
  }
  
}
);

/**
 * @private
 */
ShortLinkAlgorithmAnonClass_1 = declare(LongLinkLayout,
{
  // tags:
  //		private
  
  __outerThis : null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
  },

  isFixed : function(link)
  {
    // summary:
    //		We assume the long link layout will only call it for links, never
    //		for nodes.
    // tags:
    //		private
    
    var longLinkLayoutModel = this.getGraphModel();

    if (longLinkLayoutModel instanceof SubgraphData)
    {
      link = (longLinkLayoutModel).getOriginal(link);
    }

    if (this.__outerThis.isFixedOrNoReshape(link)) {
      return true; // Boolean
    }
    var linkData = this.__outerThis.getLinkData(link);

    return linkData == null || this.__outerThis.getLinkNodeCost(this.__outerThis._originalGraphModel, linkData) <= 0; // Boolean
  },

  getOriginPointMode : function(link)
  {
    // summary:
    //		In practice, the long link layout will only call it for non-fixed
    //		links. 
    // description:
    //		In practice, the long link layout will only call it for non-fixed
    //		links. 
    //		Anyway, returning always FIXED&#95;MODE is okay.
    // tags:
    //		private
    
    return LongLinkLayout.FIXED_MODE; // int
  },

  getDestinationPointMode : function(link)
  {
    // summary:
    // tags:
    //		private
    
    return LongLinkLayout.FIXED_MODE; // int
  }

});

/**
 * @private
 */
ShortLinkAlgorithmAnonClass_2 = declare(SplineRouter,
{
  // tags:
  //		private
  
  __outerThis : null,

  constructor : function(arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
  },

  isReshapeAllowed : function(link)
  {
    if (this.__outerThis.isFixedOrNoReshape(link)) {
      return false; // Boolean
    }
    var filter = this.__outerThis.getShortLinkLayout().getSplineLinkFilter();

    if (filter != null)
    {
      var interGraphModel = this.__outerThis.getInterGraphLinksRoutingModel();

      if (interGraphModel != null)
      {
        if (!interGraphModel.isSpline(link, filter)) {
          return false; // Boolean
        }
      }
      else
      {
        if (!filter.isSpline(link)) {
          return false; // Boolean
        }
      }
    }

    return true; // Boolean
  }

});

_ShortLinkAlgorithm.QObjectInterface = declare( IGenericQuadtreeObject, 
{

  constructor : function()
  {
    
  },

  boundingBox : function(object)
  {
    return (object).boundingBox; // Rectangle
  }

}
);
_ShortLinkAlgorithm.IntersectApplyObject = declare( GenericApplyObject, 
{
  _intersectingObjects: null,

  constructor : function()
  {
  },
  
  init : function()
  {
    this._intersectingObjects = null;
  },

  getIntersectingObjects : function()
  {
    // summary:
    //		Specify the vector in which will be stored the links and nodes that
    //		can intersect a link.
    // tags:
    //		private
    
    return this._intersectingObjects; // Array
  },

  apply : function(obj, arg)
  {
    // summary:
    //		
    // obj: Object
    //		the NodeOrLinkData of the node/link that intersects our
    //		link, which is passed as the second argument ('link').
    // arg: Object
    //		the link for which we want to store all the other nodes and
    //		links that can intersect it.
    // tags:
    //		private
    
    var nodeOrLinkData = obj;
    var linkData = arg;

    if (nodeOrLinkData.isIntersectingObjectValid(linkData))
    {
      if (this._intersectingObjects == null) {
        this._intersectingObjects = new Array();
      }
      this._intersectingObjects.push(nodeOrLinkData);
    }
  }

}
);
(function(){ _ShortLinkAlgorithm._fields_init(); })();

return _ShortLinkAlgorithm;
});

},
'ibm_ilog/graphlayout/shortlink/_DirectLinkShapeProducer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_DirectLinkShapeProducer", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Point",	// #6
  "../internalutil/QuickSort",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../shortlink/_LinkShapeProducer",	// #9
  "../shortlink/_LinkShapeType"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Point,	// #6
    QuickSort,	// #7
    TranslateUtil,	// #8
    _LinkShapeProducer,	// #9
    _LinkShapeType	// #10
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _LinkShapeProducer = ibm_ilog.graphlayout.shortlink._LinkShapeProducer;
=====*/

var _DirectLinkShapeProducer = declare("ibm_ilog.graphlayout.shortlink._DirectLinkShapeProducer", _LinkShapeProducer, 
{
  // summary:
  //		A class that is used to produce candidate direct-link shapes for the
  //		links. 
  // description:
  //		A class that is used to produce candidate direct-link shapes for the
  //		links. 
  //		This class is used by the link layout algorithm.  
  // tags:
  //		private
  
  _linkOrderSort : null,

  constructor : function(layout)
  {
    this._linkOrderSort = new _DirectLinkShapeProducer.LinkOrderSort();
  },
  
  getMaxNumberOfBends : function()
  {
    // summary:
    //		Returns the maximum number of bends.
    // tags:
    //		private
    
    return 3; // int
  },

  getBypassDistance : function(linkData, globalBypassDistance)
  {
    // summary:
    //		Computes the bypass distance to be used for a particular link. 
    // description:
    //		Computes the bypass distance to be used for a particular link. 
    //		The default implementation from the base class adds the link width
    //		to the global bypass distance. But for direct links we get better
    //		results without adding the link width. Indeed, an increased
    //		(sometimes overestimated) bypass dist means an increased risk to get
    //		orthogonal shapes (which are used in direct link style for
    //		overlapping nodes), which are not pleasant in this style because of
    //		the mix of direct and orthogonal shapes.
    // tags:
    //		protected
    
    return globalBypassDistance; // Number
  },

  getTopToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.TOP_TOP_SIDE_ARRAY; // Array
  },

  getTopToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.TOP_BOTTOM_SIDE_ARRAY; // Array
  },

  getTopToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.TOP_LEFT_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.TOP_LEFT_SIDE_ARRAY); // Array
  },

  getTopToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.TOP_RIGHT_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.TOP_RIGHT_SIDE_ARRAY); // Array
  },

  getBottomToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.BOTTOM_TOP_SIDE_ARRAY; // Array
  },

  getBottomToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.BOTTOM_BOTTOM_SIDE_ARRAY; // Array
  },

  getBottomToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.BOTTOM_LEFT_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.BOTTOM_LEFT_SIDE_ARRAY); // Array
  },

  getBottomToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.BOTTOM_RIGHT_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.BOTTOM_RIGHT_SIDE_ARRAY); // Array
  },

  getLeftToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.LEFT_TOP_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.LEFT_TOP_SIDE_ARRAY); // Array
  },

  getLeftToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.LEFT_BOTTOM_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.LEFT_BOTTOM_SIDE_ARRAY); // Array
  },

  getLeftToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.LEFT_LEFT_SIDE_ARRAY; // Array
  },

  getLeftToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.LEFT_RIGHT_SIDE_ARRAY; // Array
  },

  getRightToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.RIGHT_TOP_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.RIGHT_TOP_SIDE_ARRAY); // Array
  },

  getRightToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return (isOverlapping ? _DirectLinkShapeProducer.RIGHT_BOTTOM_SIDE_OVERLAP_ARRAY : _DirectLinkShapeProducer.RIGHT_BOTTOM_SIDE_ARRAY); // Array
  },

  getRightToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.RIGHT_LEFT_SIDE_ARRAY; // Array
  },

  getRightToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {
    return _DirectLinkShapeProducer.RIGHT_RIGHT_SIDE_ARRAY; // Array
  },

  getAllQuasiSelfInterGraphLinkShapeTypes : function()
  {
    // summary:
    //		Returns all the shape types for quasi-self-intergraph-links. 
    // description:
    //		Returns all the shape types for quasi-self-intergraph-links. 
    //		Used Only when the node side filtering or side-constraints have
    //		rejected all the shapes obtained by the fine-grain choice depending
    //		on the relative position of the end-nodes.
    // tags:
    //		protected
    
    return _DirectLinkShapeProducer.ALL_QUASI_SELF_INTERGRAPH_LINK_TYPES; // Array
  },

  getFromBottomLeftToTopRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		Returns an array of shape types and computes the link region. 
    // description:
    //		Returns an array of shape types and computes the link region. 
    //		The selection of shapes depends on the relative position of the
    //		nodes and eventually on various settings. ----- -----------|   | /  
    //		        | T | /          --|   | |         /  ----- |        /    |
    //		| | ------------- | | |    /        / -----  /      / |   |--    / |
    //		F |---/ |   | -----
    // tags:
    //		protected
    
    return _DirectLinkShapeProducer.BOTTOM_LEFT_TO_TOP_RIGHT_POSITION_ARRAY; // Array
  },

  getFromTopLeftToBottomRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    
    return _DirectLinkShapeProducer.TOP_LEFT_TO_BOTTOM_RIGHT_POSITION_ARRAY; // Array
  },

  getFromLeftToRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		-----   ----- | F |   | T | -----   -----
    // tags:
    //		protected
    
    return ((toRect.x - 2 * bypassDistance > fromRect.x + fromRect.width) ? _DirectLinkShapeProducer.LEFT_TO_RIGHT_POSITION_ARRAY : _DirectLinkShapeProducer.CLOSE_LEFT_TO_RIGHT_POSITION_ARRAY); // Array
  },

  getFromBottomRightToTopLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | T | ----- ----- | F | -----
    // tags:
    //		protected
    
    return _DirectLinkShapeProducer.BOTTOM_RIGHT_TO_TOP_LEFT_POSITION_ARRAY; // Array
  },

  getFromTopRightToBottomLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    
    return _DirectLinkShapeProducer.TOP_RIGHT_TO_BOTTOM_LEFT_POSITION_ARRAY; // Array
  },

  getFromRightToLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		-----   ----- | T |   | F | -----   -----
    // tags:
    //		protected
    
    return ((fromRect.x - 2 * bypassDistance > toRect.x + toRect.width) ? _DirectLinkShapeProducer.RIGHT_TO_LEFT_POSITION_ARRAY : _DirectLinkShapeProducer.CLOSE_RIGHT_TO_LEFT_POSITION_ARRAY); // Array
  },

  getFromBottomToTopPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | T | ----- ----- | F | -----
    // tags:
    //		protected
    
    return ((toRect.y + toRect.height + 2 * bypassDistance < fromRect.y) ? _DirectLinkShapeProducer.BOTTOM_TO_TOP_POSITION_ARRAY : _DirectLinkShapeProducer.CLOSE_BOTTOM_TO_TOP_POSITION_ARRAY); // Array
  },

  getFromTopToBottomPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    
    return ((toRect.y - 2 * bypassDistance > fromRect.y + fromRect.height) ? _DirectLinkShapeProducer.TOP_TO_BOTTOM_POSITION_ARRAY : _DirectLinkShapeProducer.CLOSE_TOP_TO_BOTTOM_POSITION_ARRAY); // Array
  },

  sortShapeTypes : function(shapeTypes, linkData, minFinalSegmentLength, copyBeforeSort)
  {
    // summary:
    //		Overridden implementation of the method from LinkShapeProducer
    //		(which does nothing).
    // tags:
    //		protected
    
    var shapeTypesToSort = (copyBeforeSort ? this.copyShapeTypes(shapeTypes, shapeTypes.length) : shapeTypes);
    this._linkOrderSort.sort(shapeTypesToSort, linkData, minFinalSegmentLength);
    return shapeTypesToSort; // Array
  }

}
);

lang.mixin(_DirectLinkShapeProducer,
{
  TOP_TOP_SIDE_ARRAY : null,
  TOP_BOTTOM_SIDE_ARRAY : null,
  TOP_LEFT_SIDE_ARRAY : null,
  TOP_RIGHT_SIDE_ARRAY : null,
  BOTTOM_TOP_SIDE_ARRAY : null,
  BOTTOM_BOTTOM_SIDE_ARRAY : null,
  BOTTOM_LEFT_SIDE_ARRAY : null,
  BOTTOM_RIGHT_SIDE_ARRAY : null,
  LEFT_TOP_SIDE_ARRAY : null,
  LEFT_BOTTOM_SIDE_ARRAY : null,
  LEFT_LEFT_SIDE_ARRAY : null,
  LEFT_RIGHT_SIDE_ARRAY : null,
  RIGHT_TOP_SIDE_ARRAY : null,
  RIGHT_BOTTOM_SIDE_ARRAY : null,
  RIGHT_LEFT_SIDE_ARRAY : null,
  RIGHT_RIGHT_SIDE_ARRAY : null,

  TOP_LEFT_SIDE_OVERLAP_ARRAY : null,
  TOP_RIGHT_SIDE_OVERLAP_ARRAY : null,
  BOTTOM_LEFT_SIDE_OVERLAP_ARRAY : null,
  BOTTOM_RIGHT_SIDE_OVERLAP_ARRAY : null,
  LEFT_TOP_SIDE_OVERLAP_ARRAY : null,
  LEFT_BOTTOM_SIDE_OVERLAP_ARRAY : null,
  RIGHT_TOP_SIDE_OVERLAP_ARRAY : null,
  RIGHT_BOTTOM_SIDE_OVERLAP_ARRAY : null,

  BOTTOM_LEFT_TO_TOP_RIGHT_POSITION_ARRAY : null,
  TOP_LEFT_TO_BOTTOM_RIGHT_POSITION_ARRAY : null,
  LEFT_TO_RIGHT_POSITION_ARRAY : null,
  CLOSE_LEFT_TO_RIGHT_POSITION_ARRAY : null,
  BOTTOM_RIGHT_TO_TOP_LEFT_POSITION_ARRAY : null,
  TOP_RIGHT_TO_BOTTOM_LEFT_POSITION_ARRAY : null,
  RIGHT_TO_LEFT_POSITION_ARRAY : null,
  CLOSE_RIGHT_TO_LEFT_POSITION_ARRAY : null,
  BOTTOM_TO_TOP_POSITION_ARRAY : null,
  CLOSE_BOTTOM_TO_TOP_POSITION_ARRAY : null,
  TOP_TO_BOTTOM_POSITION_ARRAY : null,
  CLOSE_TOP_TO_BOTTOM_POSITION_ARRAY : null,

  ALL_QUASI_SELF_INTERGRAPH_LINK_TYPES : null,
  _fields_init : function(c)
  {
    c = c || _DirectLinkShapeProducer;
    if(_LinkShapeProducer._fields_init)
      _LinkShapeProducer._fields_init(c);
    
    c.TOP_TOP_SIDE_ARRAY = [_LinkShapeType.DirectTT.instance ];
    c.TOP_BOTTOM_SIDE_ARRAY = [_LinkShapeType.DirectTB.instance ];
    c.TOP_LEFT_SIDE_ARRAY = [_LinkShapeType.DirectTL.instance ];
    c.TOP_RIGHT_SIDE_ARRAY = [_LinkShapeType.DirectTR.instance ];
    c.BOTTOM_TOP_SIDE_ARRAY = [_LinkShapeType.DirectBT.instance ];
    c.BOTTOM_BOTTOM_SIDE_ARRAY = [_LinkShapeType.DirectBB.instance ];
    c.BOTTOM_LEFT_SIDE_ARRAY = [_LinkShapeType.DirectBL.instance ];
    c.BOTTOM_RIGHT_SIDE_ARRAY = [_LinkShapeType.DirectBR.instance ];
    c.LEFT_TOP_SIDE_ARRAY = [_LinkShapeType.DirectLT.instance ];
    c.LEFT_BOTTOM_SIDE_ARRAY = [_LinkShapeType.DirectLB.instance ];
    c.LEFT_LEFT_SIDE_ARRAY = [_LinkShapeType.DirectLL.instance ];
    c.LEFT_RIGHT_SIDE_ARRAY = [_LinkShapeType.DirectLR.instance ];
    c.RIGHT_TOP_SIDE_ARRAY = [_LinkShapeType.DirectRT.instance ];
    c.RIGHT_BOTTOM_SIDE_ARRAY = [_LinkShapeType.DirectRB.instance ];
    c.RIGHT_LEFT_SIDE_ARRAY = [_LinkShapeType.DirectRL.instance ];
    c.RIGHT_RIGHT_SIDE_ARRAY = [_LinkShapeType.DirectRR.instance ];
    c.TOP_LEFT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendTL_OVERLAP.instance ];
    c.TOP_RIGHT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendTR_OVERLAP.instance ];
    c.BOTTOM_LEFT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendBL_OVERLAP.instance ];
    c.BOTTOM_RIGHT_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendBR_OVERLAP.instance ];
    c.LEFT_TOP_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendLT_OVERLAP.instance ];
    c.LEFT_BOTTOM_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendLB_OVERLAP.instance ];
    c.RIGHT_TOP_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendRT_OVERLAP.instance ];
    c.RIGHT_BOTTOM_SIDE_OVERLAP_ARRAY = [_LinkShapeType.ThreeBendRB_OVERLAP.instance ];
    c.BOTTOM_LEFT_TO_TOP_RIGHT_POSITION_ARRAY = [_LinkShapeType.DirectTL.instance, _LinkShapeType.DirectRB.instance, _LinkShapeType.DirectTB.instance, _LinkShapeType.DirectRL.instance ];
    c.TOP_LEFT_TO_BOTTOM_RIGHT_POSITION_ARRAY = [_LinkShapeType.DirectRL.instance, _LinkShapeType.DirectRT.instance, _LinkShapeType.DirectBT.instance, _LinkShapeType.DirectBL.instance ];
    c.LEFT_TO_RIGHT_POSITION_ARRAY = [_LinkShapeType.DirectRL.instance, _LinkShapeType.DirectTT.instance, _LinkShapeType.DirectBB.instance ];
    c.CLOSE_LEFT_TO_RIGHT_POSITION_ARRAY = [_LinkShapeType.DirectTT.instance, _LinkShapeType.DirectBB.instance ];
    c.BOTTOM_RIGHT_TO_TOP_LEFT_POSITION_ARRAY = [_LinkShapeType.DirectLR.instance, _LinkShapeType.DirectTR.instance, _LinkShapeType.DirectTB.instance, _LinkShapeType.DirectLB.instance ];
    c.TOP_RIGHT_TO_BOTTOM_LEFT_POSITION_ARRAY = [_LinkShapeType.DirectLT.instance, _LinkShapeType.DirectBR.instance, _LinkShapeType.DirectBT.instance, _LinkShapeType.DirectLR.instance ];
    c.RIGHT_TO_LEFT_POSITION_ARRAY = [_LinkShapeType.DirectLR.instance, _LinkShapeType.DirectTT.instance, _LinkShapeType.DirectBB.instance ];
    c.CLOSE_RIGHT_TO_LEFT_POSITION_ARRAY = [_LinkShapeType.DirectTT.instance, _LinkShapeType.DirectBB.instance ];
    c.BOTTOM_TO_TOP_POSITION_ARRAY = [_LinkShapeType.DirectTB.instance, _LinkShapeType.DirectLL.instance, _LinkShapeType.DirectRR.instance ];
    c.CLOSE_BOTTOM_TO_TOP_POSITION_ARRAY = [_LinkShapeType.DirectLL.instance, _LinkShapeType.DirectRR.instance ];
    c.TOP_TO_BOTTOM_POSITION_ARRAY = [_LinkShapeType.DirectBT.instance, _LinkShapeType.DirectLL.instance, _LinkShapeType.DirectRR.instance ];
    c.CLOSE_TOP_TO_BOTTOM_POSITION_ARRAY = [_LinkShapeType.DirectLL.instance, _LinkShapeType.DirectRR.instance ];
    c.ALL_QUASI_SELF_INTERGRAPH_LINK_TYPES = 
    [
      
      _LinkShapeType.DirectLL.instance,
      _LinkShapeType.DirectRR.instance,
      _LinkShapeType.DirectTT.instance,
      _LinkShapeType.DirectBB.instance,
      _LinkShapeType.DirectLT.instance,
      _LinkShapeType.DirectLB.instance,
      _LinkShapeType.DirectRB.instance,
      _LinkShapeType.DirectRT.instance,
      _LinkShapeType.DirectTL.instance,
      _LinkShapeType.DirectTR.instance,
      _LinkShapeType.DirectBR.instance,
      _LinkShapeType.DirectBL.instance,
      _LinkShapeType.DirectBT.instance,
      _LinkShapeType.DirectTB.instance,
      _LinkShapeType.DirectRL.instance,
      _LinkShapeType.DirectLR.instance
     ];
  }
  
}
);

_DirectLinkShapeProducer.LinkOrderSort = declare(QuickSort, 
{

  constructor : function()
  {
    
  this._shapeTypes = null;
  this._lengthArray = null;
  this._p1 = new Point();
  this._p2 = new Point();},
  _shapeTypes : null,

  _lengthArray : null,

  _p1 : null,
  _p2 : null,

  sort : function(shapeTypes, linkData, minFinalSegmentLength)
  {
    // summary:
    // tags:
    //		private
    
    if (shapeTypes == null) {
      return;
    }
    var nShapes = shapeTypes.length;
    if (nShapes < 2) {
      return;
    }
    this._shapeTypes = shapeTypes;
    if (this._lengthArray == null || this._lengthArray.length < nShapes) {
      this._lengthArray = TranslateUtil.newArray(nShapes, 0);
    }
    var fromSide;
    var toSide;
    var shapeType;
    var fromRect = linkData.getFromNode().getLinkConnectionRect();
    var toRect = linkData.getToNode().getLinkConnectionRect();

    for (var i = 0; i < nShapes; i++)
    {
      shapeType = shapeTypes[i];

      fromSide = shapeType.getFromNodeSide(linkData);
      toSide = shapeType.getToNodeSide(linkData);

      fromSide.moveToDefaultConnectionPoint(this._p1, fromRect);
      fromSide.translateOutside(this._p1, minFinalSegmentLength);
      toSide.moveToDefaultConnectionPoint(this._p2, toRect);
      toSide.translateOutside(this._p2, minFinalSegmentLength);
      this._lengthArray[i] = (this._p2.x - this._p1.x) * (this._p2.x - this._p1.x) + (this._p2.y - this._p1.y) * (this._p2.y - this._p1.y);
    }

    this.inherited("sort", arguments, [nShapes]);
  },

  compare : function(loc1, loc2)
  {
    // summary:
    //		Compare the elements. 
    // description:
    //		Compare the elements. 
    //		Currently, the criteria used for sorting the shapes is the length of
    //		the intermediate segment of the link. This gives fairly close
    //		results in comparison with using an angle criteria.
    // tags:
    //		private
    
    if (this._lengthArray[loc1] == this._lengthArray[loc2]) {
      return 0; // int
    }
    else if (this._lengthArray[loc1] < this._lengthArray[loc2]) {
      return -1; // int
    }
    else {
      return 1; // int
    }
  },

  swap : function(loc1, loc2)
  {
    // summary:
    //		Swap two elements.
    // tags:
    //		private
    
    var shapeType = this._shapeTypes[loc1];
    this._shapeTypes[loc1] = this._shapeTypes[loc2];
    this._shapeTypes[loc2] = shapeType;

    var length = this._lengthArray[loc1];
    this._lengthArray[loc1] = this._lengthArray[loc2];
    this._lengthArray[loc2] = length;
  }

}
);
(function(){ _DirectLinkShapeProducer._fields_init(); })();

return _DirectLinkShapeProducer;
});

},
'ibm_ilog/graphlayout/longlink/LLIncrementalData':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLIncrementalData", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/HashTable",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../longlink/LLIncrementalLink",	// #8
  "../longlink/LLIncrementalNode"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    HashTable,	// #6
    TranslateUtil,	// #7
    LLIncrementalLink,	// #8
    LLIncrementalNode	// #9
    ) {

var LLIncrementalData = declare("ibm_ilog.graphlayout.longlink.LLIncrementalData", null, 
{
  // summary:
  //		Data container for nodes and links when using incremental mode. 
  // description:
  //		Data container for nodes and links when using incremental mode. 
  //		We need this data in order to figure out which links have changed and
  //		must be rerouted, and where links must start or end. The data is
  //		updated when incremental is switched on, and after layout is
  //		performed when incremental is already on.  
  // tags:
  //		private
  
  _nodeData: null,
  _linkData: null,

  constructor : function()
  {
    this._nodeData = null;
    this._linkData = null;
  },
  
  update : function(layout)
  {
    // summary:
    //		Updates the incremental data for the layout algorithm.
    // tags:
    //		private
    
    var model = layout.getGraphModel();
    if (model == null) {
      return;
    }
    var numNodes = model.getNodesCount();
    var numLinks = model.getLinksCount();
    this._nodeData = new HashTable();
    this._linkData = new HashTable();
    var obj;
    var e = new TranslateUtil.ArrayIterator(model.getNodes());

    while (e.hasNext())
    {
      obj = e.next();
      this._nodeData.add(obj, new LLIncrementalNode(layout, obj));
      layout.addStepCount(1);
    }

    e = new TranslateUtil.ArrayIterator(model.getLinks());

    while (e.hasNext())
    {
      obj = e.next();
      this._linkData.add(obj, new LLIncrementalLink(layout, obj));
      layout.addStepCount(1);
    }
  },

  getNodeData : function(node)
  {
    // summary:
    //		Returns the incremental data for a node.
    // tags:
    //		private
    
    if (this._nodeData == null) {
      return (null); // ibm_ilog.graphlayout.longlink.LLIncrementalNode
    }
    return this._nodeData.item(node); // ibm_ilog.graphlayout.longlink.LLIncrementalNode
  },

  getLinkData : function(link)
  {
    // summary:
    //		Returns the incremental data for a link.
    // tags:
    //		private
    
    if (this._linkData == null) {
      return (null); // ibm_ilog.graphlayout.longlink.LLIncrementalLink
    }
    return this._linkData.item(link); // ibm_ilog.graphlayout.longlink.LLIncrementalLink
  },

  prepare : function(layout, grid)
  {
    // summary:
    //		Prepares the data before the next layout.
    // tags:
    //		private
    
    var model = layout.getGraphModel();
    if (model == null) {
      return;
    }
    if (this._nodeData != null)
    {
      var nodeDataToBeRemoved = new Array();
      var e;
      var inode;

      e = new TranslateUtil.ArrayIterator(this._nodeData.getValueList());

      while (e.hasNext())
      {
        inode = e.next();
        if (model.isNode(inode.getNodeObject())) {
          inode.prepare(layout, grid);
        }
        else {
          nodeDataToBeRemoved.push(inode);
        }
        layout.addStepCount(1);
      }

      e =  new TranslateUtil.ArrayIterator(nodeDataToBeRemoved);

      while (e.hasNext())
      {
        inode = e.next();
        this._nodeData.remove(inode.getNodeObject());
      }
    }
  },

  clear : function(link)
  {
    // summary:
    //		Clears the incremental data such that the input link will be
    //		rerouted next time (even if it hasn't changed).
    // tags:
    //		private
    
    var ilink = this.getLinkData(link);
    if (ilink != null) {
      ilink.clear();
    }
  },

  cleanNode : function(node)
  {
    // summary:
    //		Cleans the node data when the node is removed from the graph.
    // tags:
    //		private
    
    if (this._nodeData != null) {
      this._nodeData.remove(node);
    }
  },

  cleanLink : function(link)
  {
    // summary:
    //		Cleans the link data when the link is removed from the graph.
    // tags:
    //		private
    
    if (this._linkData != null) {
      this._linkData.remove(link);
    }
  }

}
);

return LLIncrementalData;
});

},
'ibm_ilog/graphlayout/shortlink/_NodeSideLayouter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_NodeSideLayouter", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/ConnectorStyles",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/LayoutUtil",	// #7
  "../internalutil/LogResUtil",	// #8
  "../internalutil/SubgraphData",	// #9
  "../internalutil/TranslateUtil",	// #10
  "../shortlink/_SLBundlesModes"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    ConnectorStyles,	// #5
    GraphModelUtil,	// #6
    LayoutUtil,	// #7
    LogResUtil,	// #8
    SubgraphData,	// #9
    TranslateUtil,	// #10
    _SLBundlesModes	// #11
    ) {

var _NodeSideLayouter = declare("ibm_ilog.graphlayout.shortlink._NodeSideLayouter", null, 
{
  // summary:
  //		This class performs the layout of the connection points of the links
  //		connected to a node side. 
  // description:
  //		This class performs the layout of the connection points of the links
  //		connected to a node side. 
  //		Depending on the settings, it can also move other points of the link
  //		in order to avoid overlaps and crossings.  
  //		<p>
  //		One layout instance uses exactly one instance of this class.  
  //		</p>
  // tags:
  //		private
  
  _vectLinks: null,
  _evenlySpacedPinsMarginRatio: null,

  _totalWidth : null,
  _nSegments : null,

  constructor : function()
  {
    this._totalWidth = TranslateUtil.newArray(1, 0);
    this._nSegments = TranslateUtil.newArray(1, 0);
  },
  
  update : function(layout)
  {
    // summary:
    //		Updates parameters from the current values on the layout.
    // tags:
    //		private
    
    var shortLayout = layout.getShortLinkLayout();
    this._evenlySpacedPinsMarginRatio = shortLayout.getEvenlySpacedPinsMarginRatio();
  },

  clean : function()
  {
    // summary:
    //		Cleans the data stored by this class.
    // tags:
    //		private
    
    if (this._vectLinks != null) {
      this._vectLinks.length = 0;
    }
  },

  layout : function (a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11)
  {
    // summary:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    // description:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    //		The side of the node is determined by the actual subclass of of
    //		NodeSide on which we call the method.  
    // vectFixedLinks: Array
    //		Both fixed links and links with fixed connection
    //		point on 'nodeSide'
    // tags:
    //		private
    
    if(arguments.length == 12)
      return this.layout_overload_12(a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11);
    else if(arguments.length == 11)
      return this.layout_overload_11(a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
  },
  
  layout_overload_12 : function(layout, linkSorter, linkRefiner, connectionBoxInterface, nodeData, nodeSide, vectLinksOrig, vectLinksDest, vectFixedLinks, linkOffset, minFinalSegmentLength, 
      bundleMode)
                
  {
    var nOrig = (vectLinksOrig != null ? vectLinksOrig.length : 0);
    var nDest = (vectLinksDest != null ? vectLinksDest.length : 0);
    var nFixed = (vectFixedLinks != null ? vectFixedLinks.length : 0);

    if ((nOrig + nDest + nFixed) == 0) {
      return;
    }
    if (vectFixedLinks == null || vectFixedLinks.length == 0)
    {
      if (linkSorter != null)
      {
        linkSorter.sort(vectLinksOrig, nodeData, nodeSide, bundleMode);
        linkSorter.sort(vectLinksDest, nodeData, nodeSide, bundleMode);
      }

      var vectLinks = this.concatenate(vectLinksOrig, vectLinksDest);
      this.layoutWithoutFixedLinks(layout, linkRefiner, connectionBoxInterface, nodeData, nodeSide, vectLinks, linkOffset, minFinalSegmentLength, bundleMode);
    }
    else {
      this.layoutWithFixedLinks(layout, linkSorter, linkRefiner, connectionBoxInterface, nodeData, nodeSide, vectLinksOrig, vectLinksDest, vectFixedLinks, linkOffset, minFinalSegmentLength, 
              bundleMode);
    }
  },

  layoutWithoutFixedLinks : function(layout, linkRefiner, connectionBoxInterface, nodeData, nodeSide, vectLinks, linkOffset, minFinalSegmentLength, bundleMode)
  {
    // summary:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    // description:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    //		The side of the node is determined by the actual subclass of of
    //		NodeSide on which we call the method.
    // tags:
    //		private
    
    var nLinks = (vectLinks != null ? vectLinks.length : 0);
    if (nLinks == 0) {
      return;
    }
    if (nLinks == 1 && connectionBoxInterface == null)
    {
      var uniqueLinkData = vectLinks[0];
      if (!uniqueLinkData.isSameSideSelfLink() && !uniqueLinkData.isMaster()) {
        return;
      }
    }

    var linkConnectionRect = nodeData.getLinkConnectionRect();

    var sideLength = nodeSide.getMaxTangentialCoord(linkConnectionRect) - nodeSide.getMinTangentialCoord(linkConnectionRect);

    this.computeTotalWidthOfLinksWithoutOffsets(vectLinks, this._totalWidth, this._nSegments);
    var totalWidthWithoutOffsets = this._totalWidth[0];
    var nLineSegments = this._nSegments[0];

    linkOffset = this.adjustLinkOffsetIfAppropriate(totalWidthWithoutOffsets, nLineSegments, 2, sideLength, linkOffset, nodeData.getConnectorStyle());

    var totalWidthWithOffsets = this.computeTotalWidthOfLinks(totalWidthWithoutOffsets, nLineSegments, linkOffset);

    var initialDist = (sideLength - totalWidthWithOffsets) * 0.5;

    if (connectionBoxInterface != null)
    {
      var model = layout.getGraphModel();
      var node = nodeData.nodeOrLink;
      if (model instanceof SubgraphData) {
        initialDist += (model).getTangentialOffset(connectionBoxInterface, node, nodeSide.getSide());
      }
      else {
        initialDist += connectionBoxInterface.getTangentialOffset(model, node, nodeSide.getSide());
      }
    }

    this.layout(null, linkRefiner, vectLinks, nodeData, nodeSide, initialDist, linkOffset, minFinalSegmentLength, bundleMode, false, false);
  },

  layoutWithFixedLinks : function(layout, linkSorter, linkRefiner, connectionBoxInterface, nodeData, nodeSide, vectLinksOrig, vectLinksDest, vectFixedLinks, linkOffset, minFinalSegmentLength,
    bundleMode)
  {
    // summary:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    // description:
    //		Computes the connection points for all links incident to a given
    //		side of a node. 
    //		The side of the node is determined by the actual subclass of of
    //		NodeSide on which we call the method.
    // tags:
    //		private
    
    var nLinksOrig = (vectLinksOrig != null ? vectLinksOrig.length : 0);
    var nLinksDest = (vectLinksDest != null ? vectLinksDest.length : 0);
    var minTanCoordOfFixedLinks = /*Number.MAX_VALUE*/(3.4028235E38);

    var maxTanCoordOfFixedLinks = -/*Number.MAX_VALUE*/(3.4028235E38);

    var nFixed = (vectFixedLinks != null ? vectFixedLinks.length : 0);

    if (nFixed == 0)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6621E");
    }

    var linkData;
    var origin;
    var connectionPoint;
    var halfLinkWidth;

    if (bundleMode == _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT)
    {
      linkSorter.sort(vectFixedLinks, nodeData, nodeSide, bundleMode);

      if (nLinksOrig > 0)
      {
        linkData = vectFixedLinks[0];
        halfLinkWidth = linkData.getLinkWidth() * 0.5;
        origin = linkData.isOrigin(nodeData, nodeSide);
        connectionPoint = linkData.getConnectionPoint(origin);
        minTanCoordOfFixedLinks = nodeSide.getTangentialCoord(connectionPoint) - halfLinkWidth;
      }

      if (nLinksDest > 0)
      {
        linkData = vectFixedLinks[nFixed - 1];
        halfLinkWidth = linkData.getLinkWidth() * 0.5;
        origin = linkData.isOrigin(nodeData, nodeSide);
        connectionPoint = linkData.getConnectionPoint(origin);
        maxTanCoordOfFixedLinks = nodeSide.getTangentialCoord(connectionPoint) + halfLinkWidth;
      }
    }
    else
    {
      var tanCoord;
      var minTanCoord;
      var maxTanCoord;

      for (var i = 0; i < nFixed; i++)
      {
        linkData = vectFixedLinks[i];
        origin = linkData.isOrigin(nodeData, nodeSide);
        connectionPoint = linkData.getConnectionPoint(origin);
        tanCoord = nodeSide.getTangentialCoord(connectionPoint);
        halfLinkWidth = linkData.getLinkWidth() * 0.5;

        if (nLinksOrig > 0)
        {
          minTanCoord = tanCoord - halfLinkWidth;
          if (minTanCoord < minTanCoordOfFixedLinks) {
            minTanCoordOfFixedLinks = minTanCoord;
          }
        }

        if (nLinksDest > 0)
        {
          maxTanCoord = tanCoord + halfLinkWidth;
          if (maxTanCoord > maxTanCoordOfFixedLinks) {
            maxTanCoordOfFixedLinks = maxTanCoord;
          }
        }
      }
    }

    var linkConnectionRect = nodeData.getLinkConnectionRect();
    var linkOffsetOrig = linkOffset;
    var linkOffsetDest = linkOffset;

    this.computeTotalWidthOfLinksWithoutOffsets(vectLinksOrig, this._totalWidth, this._nSegments);
    var totalWidthWithoutOffsetsOrig = this._totalWidth[0];

    var nLineSegmentsOrig = this._nSegments[0] + 1;
    this.computeTotalWidthOfLinksWithoutOffsets(vectLinksDest, this._totalWidth, this._nSegments);
    var totalWidthWithoutOffsetsDest = this._totalWidth[0];

    var nLineSegmentsDest = this._nSegments[0] + 1;

    var minTanCoordOfSide = nodeSide.getMinTangentialCoord(linkConnectionRect);
    var maxTanCoordOfSide = nodeSide.getMaxTangentialCoord(linkConnectionRect);
    var initialDistOrig = 0;
    var initialDistDest = 0;

    var connectorStyle = nodeData.getConnectorStyle();

    if (nLinksOrig > 0)
    {
      var availableLengthOrig = minTanCoordOfFixedLinks - minTanCoordOfSide;

      if (availableLengthOrig < 0) {
        availableLengthOrig = 0;
      }
      linkOffsetOrig = this.adjustLinkOffsetIfAppropriate(totalWidthWithoutOffsetsOrig, nLineSegmentsOrig, 1, availableLengthOrig, linkOffsetOrig, connectorStyle);

      var totalWidthWithOffsetsOrig = this.computeTotalWidthOfLinks(totalWidthWithoutOffsetsOrig, nLineSegmentsOrig, linkOffsetOrig);
      initialDistOrig = minTanCoordOfFixedLinks - minTanCoordOfSide - totalWidthWithOffsetsOrig;
    }

    if (nLinksDest > 0)
    {
      var availableLengthDest = maxTanCoordOfSide - maxTanCoordOfFixedLinks;

      if (availableLengthDest < 0) {
        availableLengthDest = 0;
      }
      linkOffsetDest = this.adjustLinkOffsetIfAppropriate(totalWidthWithoutOffsetsDest, nLineSegmentsDest, 1, availableLengthDest, linkOffsetDest, connectorStyle);

      initialDistDest = maxTanCoordOfFixedLinks - minTanCoordOfSide + linkOffsetDest;
    }

    if (connectionBoxInterface != null)
    {
      var model = layout.getGraphModel();
      var node = nodeData.nodeOrLink;
      var offset;
      if (model instanceof SubgraphData) {
        offset = (model).getTangentialOffset(connectionBoxInterface, node, nodeSide.getSide());
      }
      else {
        offset = connectionBoxInterface.getTangentialOffset(model, node, nodeSide.getSide());
      }
      initialDistOrig += offset;
      initialDistDest += offset;
    }

    if (nLinksOrig > 0) {
      this.layout(null, null, vectLinksOrig, nodeData, nodeSide, initialDistOrig, linkOffsetOrig, minFinalSegmentLength, bundleMode, false, false);
    }
    if (nLinksDest > 0) {
      this.layout(null, null, vectLinksDest, nodeData, nodeSide, initialDistDest, linkOffsetDest, minFinalSegmentLength, bundleMode, false, false);
    }
    if (linkRefiner != null && bundleMode == _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT) {
      this.refine(linkRefiner, vectLinksOrig, vectFixedLinks, vectLinksDest, nodeData, nodeSide, linkOffset, minFinalSegmentLength, bundleMode);
    }
  },

  computeTotalWidthOfLinksWithoutOffsets : function(vectLinks, totalWidth, nLineSegments)
  {
    var nLinks = (vectLinks != null ? vectLinks.length : 0);

    if (nLinks == 0)
    {
      totalWidth[0] = 0;
      nLineSegments[0] = 0;
      return;
    }

    var totalWidthWithoutOffsets = 0;
    var nSegments = 0;
    var linkData;

    for (var i = 0; i < nLinks; i++)
    {
      linkData = vectLinks[i];

      if (linkData.isSameSideSelfLink())
      {
        totalWidthWithoutOffsets += (2 * linkData.getLinkWidthWithOffset(0));
        nSegments += 2 * linkData.getNumberOfIndividualLinks();
      }
      else
      {
        totalWidthWithoutOffsets += linkData.getLinkWidthWithOffset(0);
        nSegments += linkData.getNumberOfIndividualLinks();
      }
    }

    totalWidth[0] = totalWidthWithoutOffsets;
    nLineSegments[0] = nSegments;
  },

  adjustLinkOffsetIfAppropriate : function(totalWidthWithoutOffsets, nLineSegments, nMargins, availableLength, linkOffset, connectorStyle)
  {
    // summary:
    //		Adjusts the link offset in case of "evenly spaced pins" or if the
    //		total width if larger than the length of the border in case of
    //		"fixed offset pins".  
    // nMargins: int
    //		2 when no fixed connectoins (we may have to preserve a
    //		margin at both extremities of the node border) and 1 when laying
    //		out with fixed connections (only 1 margin to preserve)
    // tags:
    //		private
    
    if (linkOffset <= 0 || connectorStyle == ConnectorStyles.FIXED_OFFSET_PINS) {
      return linkOffset; // Number
    }
    if (totalWidthWithoutOffsets >= availableLength || nLineSegments == 1) {
      return 0; // Number
    }
    var adjustedNLineSegments = nLineSegments + nMargins * this._evenlySpacedPinsMarginRatio;
    var totalWidthWithOffsets = this.computeTotalWidthOfLinks(totalWidthWithoutOffsets, adjustedNLineSegments, linkOffset);
    if (connectorStyle == ConnectorStyles.AUTOMATIC_PINS && totalWidthWithOffsets <= availableLength) {
      return linkOffset; // Number
    }
    var delta = availableLength - totalWidthWithoutOffsets;

    return delta / (adjustedNLineSegments - 1); // Number
  },

  computeTotalWidthOfLinks : function(totalWidthWithoutOffsets, nLineSegments, linkOffset)
  {
    return (linkOffset != 0 ? (totalWidthWithoutOffsets + (nLineSegments - 1) * linkOffset) : totalWidthWithoutOffsets); // Number
  },

  refine : function(linkRefiner, vectLinksOrig, vectLinksDest, vectFixedLinks, nodeData, nodeSide, linkOffset, minFinalSegmentLength, bundleMode)
  {
    // summary:
    // tags:
    //		private
    
    if (linkRefiner != null && bundleMode == _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT)
    {
      var vectLinks = this.concatenate(vectLinksOrig, vectFixedLinks, vectLinksDest);

      if (vectLinks != null && vectLinks.length > 1) {
        linkRefiner.refine(vectLinks, nodeData, nodeSide, linkOffset, minFinalSegmentLength, false, (vectFixedLinks != null && vectFixedLinks.length > 0));
      }
    }
  },

  layout_overload_11 : function(linkSorter, linkRefiner, vectLinks, nodeData, nodeSide, initialDist, linkOffset, minFinalSegmentLength, bundleMode, reversedOrder, withFixedConnectionPoints)
  {
    // summary:
    //		Computes the connection points for the non-fixed links incident to
    //		'nodeData' and connected to its side specified by
    //		'nodeSide'. 
    // description:
    //		Computes the connection points for the non-fixed links incident to
    //		<code>nodeData</code> and connected to its side specified by
    //		<code>nodeSide</code>. 
    //		The first bend point is also adjusted, to keep the first segment
    //		orthogonal.  
    // nodeData: _SLNodeData
    //		the NodeData object corresponding to the node whose
    //		incident links must be laid out
    // reversedOrder: Boolean
    //		if 'true', the links are laid out in the
    //		reversed order of 'vectLinks'. Note that true is only
    //		passed together with null for linkSorter.
    // withFixedConnectionPoints: Boolean
    //		true if at least one link in vectLinks
    //		has fixed connection points or is fixed. In such a case, the link
    //		may not be ordered according to their relative orientation toward
    //		the node side.
    // tags:
    //		private
    
    var nLinks = (vectLinks != null ? vectLinks.length : 0);
    if (nLinks == 0) {
      return;
    }
    if (linkSorter != null) {
      linkSorter.sort(vectLinks, nodeData, nodeSide, bundleMode);
    }

    var tangentialDist = initialDist;
    var linkData;
    var halfLinkWidth;
    var origin;
    var linkConnectionRect = nodeData.getLinkConnectionRect();

    for (var j = 0; j < nLinks; j++)
    {
      linkData = LayoutUtil.elementAt(vectLinks, j, nLinks, reversedOrder);

      linkData.shapePointsModified();
      halfLinkWidth = linkData.getLinkWidthWithOffset(linkOffset) * 0.5;
      tangentialDist += halfLinkWidth;
      origin = linkData.isOrigin(nodeData, nodeSide);

      if (linkData.isSameSideSelfLink())
      {
        nodeSide.computeConnectionPoint(linkConnectionRect, linkData, true, tangentialDist);
        tangentialDist += 2 * halfLinkWidth + linkOffset;
        nodeSide.computeConnectionPoint(linkConnectionRect, linkData, false, tangentialDist);

        linkData.setOriginLinkOffset(linkOffset);
        linkData.setDestinationLinkOffset(linkOffset);
      }
      else
      {
        nodeSide.computeConnectionPoint(linkConnectionRect, linkData, origin, tangentialDist);

        if (origin) {
          linkData.setOriginLinkOffset(linkOffset);
        }
        else {
          linkData.setDestinationLinkOffset(linkOffset);
        }
      }

      tangentialDist += halfLinkWidth + linkOffset;
      linkData.shapePointsModified();
    }

    if (linkRefiner != null && bundleMode == _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT) {
      linkRefiner.refine(vectLinks, nodeData, nodeSide, linkOffset, minFinalSegmentLength, reversedOrder, withFixedConnectionPoints);
    }
  },

  concatenate : function (a0, a1, a2)
  {
    // summary:
    // tags:
    //		private
    
    if(arguments.length == 2)
      return this.concatenate_overload_2(a0, a1);
    else if(arguments.length == 3)
      return this.concatenate_overload_3(a0, a1, a2);
  },
  
  concatenate_overload_2 : function(vect1, vect2)
                                                
  {
    var vectLinks;

    if (vect1 == null) {
      vectLinks = vect2;
    }
    else if (vect2 == null) {
      vectLinks = vect1;
    }
    else
    {
      var n1 = vect1.length;
      var n2 = vect2.length;

      if (this._vectLinks == null) {
        this._vectLinks = new Array();
      }
      else
      {
        this._vectLinks.length = 0;

        TranslateUtil.noop();
      }

      var i;

      for (i = 0; i < n1; i++)
        this._vectLinks.push(vect1[i]);

      for (i = 0; i < n2; i++)
        this._vectLinks.push(vect2[i]);
      vectLinks = this._vectLinks;
    }

    return vectLinks; // Array
  },

  concatenate_overload_3 : function(vect1, vect2, vect3)
  {
    // summary:
    // tags:
    //		private
    
    if (vect1 == null) {
      return ((vect2 == null ? vect3 : this.concatenate(vect2, vect3))); // Array
    }
    else if (vect2 == null) {
      return ((vect3 == null ? vect1 : this.concatenate(vect1, vect3))); // Array
    }
    else if (vect3 == null) {
      return this.concatenate(vect1, vect2); // Array
    }
    else
    {
      var n1 = vect1.length;
      var n2 = vect2.length;
      var n3 = vect3.length;

      if (this._vectLinks == null) {
        this._vectLinks = new Array();
      }
      else
      {
        this._vectLinks.length = 0;

        TranslateUtil.noop();
      }

      var i = 0;

      for (i = 0; i < n1; i++)
        this._vectLinks.push(vect1[i]);

      for (i = 0; i < n2; i++)
        this._vectLinks.push(vect2[i]);

      for (i = 0; i < n3; i++)
        this._vectLinks.push(vect3[i]);
      return this._vectLinks; // Array
    }
  }

}
);

return _NodeSideLayouter;
});

},
'ibm_ilog/graphlayout/shortlink/_OrthoWithout4BendsPositionArrays':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_OrthoWithout4BendsPositionArrays", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_LinkShapeType"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _LinkShapeType	// #7
    ) {

var _OrthoWithout4BendsPositionArrays = declare("ibm_ilog.graphlayout.shortlink._OrthoWithout4BendsPositionArrays", null, 
{
  // summary:
  //		Contains the static arrays which define the candidate orthogonal
  //		shapes according to the relative position of the end-nodes. 
  // description:
  //		Contains the static arrays which define the candidate orthogonal
  //		shapes according to the relative position of the end-nodes. 
  //		<p>
  //		This is the version without 4-bend shapes (to be used when no side
  //		filter  and no fixed points).  
  //		</p>
  // tags:
  //		private
  
  constructor : function()
  {
    
  }

}
);

lang.mixin(_OrthoWithout4BendsPositionArrays,
{
  FROM_BL_TO_TR_POSITION_ARRAY : null,
  FROM_BR_TO_TL_POSITION_ARRAY : null,
  FROM_TL_TO_BR_POSITION_ARRAY : null,
  FROM_TR_TO_BL_POSITION_ARRAY : null,
  FROM_L_TO_R_POSITION_ARRAY : null,
  FROM_R_TO_L_POSITION_ARRAY : null,
  FROM_B_TO_T_POSITION_ARRAY : null,
  FROM_T_TO_B_POSITION_ARRAY : null,
  _fields_init : function(c)
  {
    c = c || _OrthoWithout4BendsPositionArrays;
    
    c.FROM_BL_TO_TR_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTL.instance
       ],
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendRB.instance
       ],
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance
       ],
      
      [
        _LinkShapeType.OneBendRB.instance,
        _LinkShapeType.OneBendTL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance
       ]
     ];
    c.FROM_BR_TO_TL_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance
       ],
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance
       ],
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance
       ],
      
      [
        _LinkShapeType.OneBendTR.instance,
        _LinkShapeType.OneBendLB.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendRB.instance
       ]
     ];
    c.FROM_TL_TO_BR_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance
       ],
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance
       ],
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance
       ],
      
      [
        _LinkShapeType.OneBendBL.instance,
        _LinkShapeType.OneBendRT.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance
       ]
     ];
    c.FROM_TR_TO_BL_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendLT.instance
       ],
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendBR.instance
       ],
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance
       ],
      
      [
        _LinkShapeType.OneBendLT.instance,
        _LinkShapeType.OneBendBR.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance
       ]
     ];
    c.FROM_L_TO_R_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance
       ],
      
      [
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendRL.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendTL.instance,
        _LinkShapeType.ThreeBendBL.instance
       ]
     ];
    c.FROM_R_TO_L_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance
       ],
      
      [
        _LinkShapeType.TwoBendBB.instance,
        _LinkShapeType.TwoBendTT.instance,
        _LinkShapeType.TwoBendLR.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendLT.instance,
        _LinkShapeType.ThreeBendLB.instance
       ]
     ];
    c.FROM_B_TO_T_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendTL.instance
       ],
      
      [
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendTB.instance,
        _LinkShapeType.ThreeBendLT_OVERLAP.instance,
        _LinkShapeType.ThreeBendRT_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR_OVERLAP.instance,
        _LinkShapeType.ThreeBendBL_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB.instance,
        _LinkShapeType.ThreeBendRB.instance,
        _LinkShapeType.ThreeBendTR.instance,
        _LinkShapeType.ThreeBendTL.instance
       ]
     ];
    c.FROM_T_TO_B_POSITION_ARRAY = 
    [
      
      [
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendLT.instance
       ],
      
      [
        _LinkShapeType.TwoBendRR.instance,
        _LinkShapeType.TwoBendLL.instance,
        _LinkShapeType.TwoBendBT.instance,
        _LinkShapeType.ThreeBendTR_OVERLAP.instance,
        _LinkShapeType.ThreeBendTL_OVERLAP.instance,
        _LinkShapeType.ThreeBendRB_OVERLAP.instance,
        _LinkShapeType.ThreeBendLB_OVERLAP.instance,
        _LinkShapeType.ThreeBendBR.instance,
        _LinkShapeType.ThreeBendBL.instance,
        _LinkShapeType.ThreeBendRT.instance,
        _LinkShapeType.ThreeBendLT.instance
       ]
     ];
  }
  
}
);

(function(){ _OrthoWithout4BendsPositionArrays._fields_init(); })();

return _OrthoWithout4BendsPositionArrays;
});

},
'ibm_ilog/graphlayout/longlink/LLCrossPoint':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLCrossPoint", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var LLCrossPoint = declare("ibm_ilog.graphlayout.longlink.LLCrossPoint", null, 
{
  // summary:
  //		An auxiliary data structure to specify a crossing point. 
  // description:
  //		An auxiliary data structure to specify a crossing point. 
  //		This data structure contains also all information we need to decide
  //		whether and how the crossing point can be reduced. The goal is to
  //		find two reducable crossing points of the same link, and reduce both
  //		of them, as for instance in: &#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;                        
  //		&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95; &#95;&#95;&#95;&#95;|&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;|&#95;&#95;&#95;&#95;&#95;                  |    
  //		&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;      | |    |         |     |  should become  |    |       
  //		 |     | A    C         D     B                 A    C         D    
  //		B   
  // tags:
  //		private
  
  _crossIndex: null,
  _hCrossSegMin: null,
  _hCrossSegMax: null,
  _vCrossSegMin: null,
  _vCrossSegMax: null,
  _crossSegIndexA: null,
  _crossSegIndexB: null,
  _distFromStartA: null,
  _distFromStartB: null,
  _type: null,
  _next: null,

  constructor : function()
  {
    this._crossIndex = TranslateUtil.newArray(2, 0);
    this._next = null;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    if (this._next != null) {
      this._next.dispose();
    }
    this._crossIndex = null;
    this._next = null;
  },

  getNext : function()
  {
    // summary:
    //		Returns the next cross point.
    // tags:
    //		private
    
    return (this._next); // ibm_ilog.graphlayout.longlink.LLCrossPoint
  },

  setNext : function(next)
  {
    // summary:
    //		Sets the next cross point.
    // tags:
    //		private
    
    this._next = next;
  },

  getCrossIndex : function(dir)
  {
    // summary:
    //		Returns the grid index of the cross point in given direction.
    // tags:
    //		private
    
    return (this._crossIndex[dir]); // int
  },

  getDistFromStartA : function()
  {
    // summary:
    //		Returns the distance from start node of link A to the crossing
    //		point.
    // tags:
    //		private
    
    return (this._distFromStartA); // int
  },

  getDistFromStartB : function()
  {
    // summary:
    //		Returns the distance from start node of link B to the crossing
    //		point.
    // tags:
    //		private
    
    return (this._distFromStartB); // int
  },

  getCrossSegIndexA : function()
  {
    // summary:
    //		Returns the index of the segment of link A where the crossing point
    //		is.
    // tags:
    //		private
    
    return (this._crossSegIndexA); // int
  },

  getCrossSegIndexB : function()
  {
    // summary:
    //		Returns the index of the segment of link B where the crossing point
    //		is.
    // tags:
    //		private
    
    return (this._crossSegIndexB); // int
  },

  getType : function()
  {
    // summary:
    //		Returns type of the cross point.
    // tags:
    //		private
    
    return (this._type); // int
  },

  makeUnfeasable : function()
  {
    // summary:
    //		Mark the crossing as nonfeasable.
    // tags:
    //		private
    
    this._type = -1;
  },

  isOppositeFlow : function()
  {
    // summary:
    //		Returns true if the crossing is between links in opposite flow.
    // tags:
    //		private
    
    return (this._type >= 8); // Boolean
  },

  getHorizontalTestSegment : function(limit)
  {
    // summary:
    //		Returns the y-index of the horizontal segment that must be free to
    //		eliminate the crossing. 
    // description:
    //		Returns the y-index of the horizontal segment that must be free to
    //		eliminate the crossing. 
    //		It also fills the input array limit[2] with the x bounds of the
    //		segment.
    // tags:
    //		private
    
    if (this._type % 2 == 0)
    {
      limit[0] = this._crossIndex[0] + 1;
      limit[1] = this._hCrossSegMax;
    }
    else
    {
      limit[0] = this._hCrossSegMin;
      limit[1] = this._crossIndex[0] - 1;
    }

    if ((this._type % 4) == 0 || (this._type % 4) == 1) {
      return (this._vCrossSegMin); // int
    }
    else {
      return (this._vCrossSegMax); // int
    }
  },

  getVerticalTestSegment : function(limit)
  {
    // summary:
    //		Returns the x-index of the vertical segment that must be free to
    //		eliminate the crossing. 
    // description:
    //		Returns the x-index of the vertical segment that must be free to
    //		eliminate the crossing. 
    //		It also fills the input array limit[2] with the y bounds of the
    //		segment.
    // tags:
    //		private
    
    if ((this._type % 4) == 0 || (this._type % 4) == 1)
    {
      limit[0] = this._vCrossSegMin;
      limit[1] = this._crossIndex[1] - 1;
    }
    else
    {
      limit[0] = this._crossIndex[1] + 1;
      limit[1] = this._vCrossSegMax;
    }

    if (this._type % 2 == 0) {
      return (this._hCrossSegMax); // int
    }
    else {
      return (this._hCrossSegMin); // int
    }
  },

  getHorizontalFreeSegment : function(limit)
  {
    // summary:
    //		Returns the y-index of the horizontal segment that can potentially
    //		be freed after elimination of the crossing. 
    // description:
    //		Returns the y-index of the horizontal segment that can potentially
    //		be freed after elimination of the crossing. 
    //		It also fills the input array limit[2] with the x bounds of the
    //		segment.
    // tags:
    //		private
    
    if (this._type % 2 == 0)
    {
      limit[0] = this._crossIndex[0] + 1;
      limit[1] = this._hCrossSegMax;
    }
    else
    {
      limit[0] = this._hCrossSegMin;
      limit[1] = this._crossIndex[0] - 1;
    }

    return (this._crossIndex[1]); // int
  },

  getVerticalFreeSegment : function(limit)
  {
    // summary:
    //		Returns the x-index of the vertical segment that can potentially be
    //		freed after elimination of the crossing. 
    // description:
    //		Returns the x-index of the vertical segment that can potentially be
    //		freed after elimination of the crossing. 
    //		It also fills the input array limit[2] with the y bounds of the
    //		segment.
    // tags:
    //		private
    
    if ((this._type % 4) == 0 || (this._type % 4) == 1)
    {
      limit[0] = this._vCrossSegMin;
      limit[1] = this._crossIndex[1] - 1;
    }
    else
    {
      limit[0] = this._crossIndex[1] + 1;
      limit[1] = this._vCrossSegMax;
    }

    return (this._crossIndex[0]); // int
  },

  calcBypassPoint : function(point)
  {
    // summary:
    //		Calculates the bypass point. 
    // description:
    //		Calculates the bypass point. 
    //		In order to reduce a crossing, the one link is routed over the
    //		bypass point, and the other is routed over the cross point.
    // tags:
    //		private
    
    switch (this._type % 4)
    {
      case 0:
        point[0] = this._hCrossSegMax;
        point[1] = this._vCrossSegMin;
        break;
      case 1:
        point[0] = this._hCrossSegMin;
        point[1] = this._vCrossSegMin;
        break;
      case 2:
        point[0] = this._hCrossSegMax;
        point[1] = this._vCrossSegMax;
        break;
      case 3:
        point[0] = this._hCrossSegMin;
        point[1] = this._vCrossSegMax;
        break;
    }
  },

  getHorizontalCrossContIndex : function()
  {
    // summary:
    //		Returns the x-index of the end point of a segment from which we have
    //		to route the segment horizontally to the cross point instead to the
    //		bypass point. 
    // description:
    //		Returns the x-index of the end point of a segment from which we have
    //		to route the segment horizontally to the cross point instead to the
    //		bypass point. 
    //		The y-index of the cross continuation point is of course the same as
    //		the crossing y-index.
    // tags:
    //		private
    
    switch (this._type % 4)
    {
      case 0:
      case 2:
        return (this._hCrossSegMin); // int
      case 1:
      case 3:
        return (this._hCrossSegMax); // int
    }
    return (0); // int
  },

  getVerticalCrossContIndex : function()
  {
    // summary:
    //		Returns the y-index of the end point of a segment from which we have
    //		to route the segment vertically to the cross point instead to the
    //		bypass point. 
    // description:
    //		Returns the y-index of the end point of a segment from which we have
    //		to route the segment vertically to the cross point instead to the
    //		bypass point. 
    //		The x-index of the cross continuation point is of course the same as
    //		the crossing x-index.
    // tags:
    //		private
    
    switch (this._type % 4)
    {
      case 0:
      case 1:
        return (this._vCrossSegMax); // int
      case 2:
      case 3:
        return (this._vCrossSegMin); // int
    }
    return (0); // int
  },

  getCrossContIndex : function(dir)
  {
    // summary:
    //		Returns the cross continuation index in given direction. 
    // description:
    //		Returns the cross continuation index in given direction. 
    //		0 means horizontal, 1 means vertical.
    // tags:
    //		private
    
    if (dir == 0) {
      return (this.getHorizontalCrossContIndex()); // int
    }
    else {
      return (this.getVerticalCrossContIndex()); // int
    }
  },

  needBypassIfFirst : function(link, crossSegIndex)
  {
    // summary:
    //		Returns true if the input link needs to route over the bypass point
    //		to reduce the crossing, assuming it is the first crossing. 
    // description:
    //		Returns true if the input link needs to route over the bypass point
    //		to reduce the crossing, assuming it is the first crossing. 
    //		If it is the second crossing to be reduced, just negate the result.
    //		The crossSegIndex is the index in the segment array of the link, of
    //		the segment that contributes to the crossing.
    // tags:
    //		private
    
    switch (this._type % 4)
    {
      case 0:
        if (link.getSegmentDir(crossSegIndex) == 0)
        {
          if (link.getSegments()[crossSegIndex] < 0) {
            return (true); // Boolean
          }
        }
        else
        {
          if (link.getSegments()[crossSegIndex] > 0) {
            return (true); // Boolean
          }
        }

        return (false); // Boolean
      case 1:
        if (link.getSegmentDir(crossSegIndex) == 0)
        {
          if (link.getSegments()[crossSegIndex] > 0) {
            return (true); // Boolean
          }
        }
        else
        {
          if (link.getSegments()[crossSegIndex] > 0) {
            return (true); // Boolean
          }
        }

        return (false); // Boolean
      case 2:
        if (link.getSegmentDir(crossSegIndex) == 0)
        {
          if (link.getSegments()[crossSegIndex] < 0) {
            return (true); // Boolean
          }
        }
        else
        {
          if (link.getSegments()[crossSegIndex] < 0) {
            return (true); // Boolean
          }
        }

        return (false); // Boolean
      case 3:
        if (link.getSegmentDir(crossSegIndex) == 0)
        {
          if (link.getSegments()[crossSegIndex] > 0) {
            return (true); // Boolean
          }
        }
        else
        {
          if (link.getSegments()[crossSegIndex] < 0) {
            return (true); // Boolean
          }
        }

        return (false); // Boolean
    }
    return (false); // Boolean
  },

  checkCrossing : function(segA1A2Index, distToA1, pA1, pA2, pA3, segB1B2Index, distToB1, pB1, pB2, pB3)
  {
    // summary:
    //		Determine the type of the crossing out of the corners of each 2
    //		segments of a link A and a link B, and set all internal data
    //		accordingly. 
    // description:
    //		Determine the type of the crossing out of the corners of each 2
    //		segments of a link A and a link B, and set all internal data
    //		accordingly. 
    //		After that, getType() will return -1 if it is not a reducable
    //		crossing, i.e. maybe not a crossing, or, if a crossing, then not
    //		reducable, and any other type value if it is potentially reducable.
    //		It returns true if a reducable crossing is found.
    // tags:
    //		private
    
    this._type = -1;

    if (pA1[0] == pA2[0])
    {
      if (pB1[0] == pB2[0])
      {
        if (pA1[1] < pB3[1] && pB3[1] < pA2[1])
        {
          this._crossIndex[0] = pA1[0];
          this._crossIndex[1] = pB3[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pB2[0] < pA1[0] && pA1[0] < pB3[0])
          {
            if (pB1[1] <= pB2[1] && pA2[0] <= pA3[0])
            {
              this._type = 3;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB3[0];
              this._vCrossSegMin = pA1[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + pA3[0] - pA2[0] + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB2[0] > pA1[0] && pA1[0] > pB3[0])
          {
            if (pB1[1] <= pB2[1] && pA3[0] <= pA2[0])
            {
              this._type = 6;
              this._hCrossSegMin = pB3[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA1[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + pB2[1] - pB1[1] + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pA1[1] > pB3[1] && pB3[1] > pA2[1])
        {
          this._crossIndex[0] = pA1[0];
          this._crossIndex[1] = pB3[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pB2[0] < pA1[0] && pA1[0] < pB3[0])
          {
            if (pB2[1] <= pB1[1] && pA2[0] <= pA3[0])
            {
              this._type = 1;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB3[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA1[1];
              this._distFromStartA = distToA1 + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + pB1[1] - pB2[1] + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB2[0] > pA1[0] && pA1[0] > pB3[0])
          {
            if (pB2[1] <= pB1[1] && pA3[0] <= pA2[0])
            {
              this._type = 4;
              this._hCrossSegMin = pB3[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA1[1];
              this._distFromStartA = distToA1 + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + pB1[1] - pB2[1] + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pB1[1] < pA3[1] && pA3[1] < pB2[1])
        {
          this._crossIndex[0] = pB1[0];
          this._crossIndex[1] = pA3[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index;

          if (pA2[0] < pB1[0] && pB1[0] < pA3[0])
          {
            if (pA1[1] <= pA2[1] && pB2[0] <= pB3[0])
            {
              this._type = 3;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA3[0];
              this._vCrossSegMin = pB1[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + pA2[1] - pA1[1] + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
          else if (pA2[0] > pB1[0] && pB1[0] > pA3[0])
          {
            if (pA1[1] <= pA2[1] && pB3[0] <= pB2[0])
            {
              this._type = 6;
              this._hCrossSegMin = pA3[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB1[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + pA2[1] - pA1[1] + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
        }
        else if (pB1[1] > pA3[1] && pA3[1] > pB2[1])
        {
          this._crossIndex[0] = pB1[0];
          this._crossIndex[1] = pA3[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index;

          if (pA2[0] < pB1[0] && pB1[0] < pA3[0])
          {
            if (pA2[1] <= pA1[1] && pB2[0] <= pB3[0])
            {
              this._type = 1;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA3[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB1[1];
              this._distFromStartA = distToA1 + pA1[1] - pA2[1] + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + this._vCrossSegMax - this._crossIndex[1];
            }
          }
          else if (pA2[0] > pB1[0] && pB1[0] > pA3[0])
          {
            if (pA2[1] <= pA1[1] && pB3[0] <= pB2[0])
            {
              this._type = 4;
              this._hCrossSegMin = pA3[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB1[1];
              this._distFromStartA = distToA1 + pA1[1] - pA2[1] + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + this._vCrossSegMax - this._crossIndex[1];
            }
          }
        }
      }
      else
      {
        if (pA1[1] < pB1[1] && pB1[1] < pA2[1])
        {
          this._crossIndex[0] = pA1[0];
          this._crossIndex[1] = pB1[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index;

          if (pB1[0] < pA1[0] && pA1[0] < pB2[0])
          {
            if (pB3[1] <= pB2[1] && pA3[0] <= pA2[0])
            {
              this._type = 10;
              this._hCrossSegMin = pB1[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA1[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB1[0] > pA1[0] && pA1[0] > pB2[0])
          {
            if (pB3[1] <= pB2[1] && pA2[0] <= pA3[0])
            {
              this._type = 15;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB1[0];
              this._vCrossSegMin = pA1[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pA1[1] > pB1[1] && pB1[1] > pA2[1])
        {
          this._crossIndex[0] = pA1[0];
          this._crossIndex[1] = pB1[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index;

          if (pB1[0] < pA1[0] && pA1[0] < pB2[0])
          {
            if (pB2[1] <= pB3[1] && pA3[0] <= pA2[0])
            {
              this._type = 12;
              this._hCrossSegMin = pB1[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA1[1];
              this._distFromStartA = distToA1 + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB1[0] > pA1[0] && pA1[0] > pB2[0])
          {
            if (pB2[1] <= pB3[1] && pA2[0] <= pA3[0])
            {
              this._type = 9;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB1[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA1[1];
              this._distFromStartA = distToA1 + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pB2[1] < pA3[1] && pA3[1] < pB3[1])
        {
          this._crossIndex[0] = pB3[0];
          this._crossIndex[1] = pA3[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pA2[0] < pB3[0] && pB3[0] < pA3[0])
          {
            if (pA2[1] <= pA1[1] && pB2[0] <= pB1[0])
            {
              this._type = 13;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA3[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB3[1];
              this._distFromStartA = distToA1 + pA1[1] - pA2[1] + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + pB1[0] - pB2[0] + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
          else if (pA2[0] > pB3[0] && pB3[0] > pA3[0])
          {
            if (pA2[1] <= pA1[1] && pB1[0] <= pB2[0])
            {
              this._type = 8;
              this._hCrossSegMin = pA3[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB3[1];
              this._distFromStartA = distToA1 + pA1[1] - pA2[1] + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + pB2[0] - pB1[0] + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
        }
        else if (pB2[1] > pA3[1] && pA3[1] > pB3[1])
        {
          this._crossIndex[0] = pB3[0];
          this._crossIndex[1] = pA3[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pA2[0] < pB3[0] && pB3[0] < pA3[0])
          {
            if (pA1[1] <= pA2[1] && pB2[0] <= pB1[0])
            {
              this._type = 11;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA3[0];
              this._vCrossSegMin = pB3[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + pA2[1] - pA1[1] + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + pB1[0] - pB2[0] + this._vCrossSegMax - this._crossIndex[1];
            }
          }
          else if (pA2[0] > pB3[0] && pB3[0] > pA3[0])
          {
            if (pA1[1] <= pA2[1] && pB1[0] <= pB2[0])
            {
              this._type = 14;
              this._hCrossSegMin = pA3[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB3[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + pA2[1] - pA1[1] + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + pB2[0] - pB1[0] + this._vCrossSegMax - this._crossIndex[1];
            }
          }
        }
      }
    }
    else
    {
      if (pB1[0] == pB2[0])
      {
        if (pA2[1] < pB3[1] && pB3[1] < pA3[1])
        {
          this._crossIndex[0] = pA3[0];
          this._crossIndex[1] = pB3[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pB2[0] < pA3[0] && pA3[0] < pB3[0])
          {
            if (pB2[1] <= pB1[1] && pA2[0] <= pA1[0])
            {
              this._type = 13;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB3[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA3[1];
              this._distFromStartA = distToA1 + pA1[0] - pA2[0] + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + pB1[1] - pB2[1] + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB2[0] > pA3[0] && pA3[0] > pB3[0])
          {
            if (pB2[1] <= pB1[1] && pA1[0] <= pA2[0])
            {
              this._type = 8;
              this._hCrossSegMin = pB3[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA3[1];
              this._distFromStartA = distToA1 + pA2[0] - pA1[0] + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + pB1[1] - pB2[1] + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pA2[1] > pB3[1] && pB3[1] > pA3[1])
        {
          this._crossIndex[0] = pA3[0];
          this._crossIndex[1] = pB3[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pB2[0] < pA3[0] && pA3[0] < pB3[0])
          {
            if (pB1[1] <= pB2[1] && pA2[0] <= pA1[0])
            {
              this._type = 11;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB3[0];
              this._vCrossSegMin = pA3[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + pA1[0] - pA2[0] + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + pB2[1] - pB1[1] + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB2[0] > pA3[0] && pA3[0] > pB3[0])
          {
            if (pB1[1] <= pB2[1] && pA1[0] <= pA2[0])
            {
              this._type = 14;
              this._hCrossSegMin = pB3[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA3[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + pA2[0] - pA1[0] + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + pB2[1] - pB1[1] + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pB1[1] < pA2[1] && pA2[1] < pB2[1])
        {
          this._crossIndex[0] = pB1[0];
          this._crossIndex[1] = pA1[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index;

          if (pA1[0] < pB1[0] && pB1[0] < pA2[0])
          {
            if (pA3[1] <= pA2[1] && pB3[0] <= pB2[0])
            {
              this._type = 10;
              this._hCrossSegMin = pA1[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB1[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
          else if (pA1[0] > pB1[0] && pB1[0] > pA2[0])
          {
            if (pA3[1] <= pA2[1] && pB2[0] <= pB3[0])
            {
              this._type = 15;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA1[0];
              this._vCrossSegMin = pB1[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
        }
        else if (pB1[1] > pA2[1] && pA2[1] > pB2[1])
        {
          this._crossIndex[0] = pB1[0];
          this._crossIndex[1] = pA1[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index;

          if (pA1[0] < pB1[0] && pB1[0] < pA2[0])
          {
            if (pA2[1] <= pA3[1] && pB3[0] <= pB2[0])
            {
              this._type = 12;
              this._hCrossSegMin = pA1[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB1[1];
              this._distFromStartA = distToA1 + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + this._vCrossSegMax - this._crossIndex[1];
            }
          }
          else if (pA1[0] > pB1[0] && pB1[0] > pA2[0])
          {
            if (pA2[1] <= pA3[1] && pB2[0] <= pB3[0])
            {
              this._type = 9;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA1[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB1[1];
              this._distFromStartA = distToA1 + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + this._vCrossSegMax - this._crossIndex[1];
            }
          }
        }
      }
      else
      {
        if (pA2[1] < pB1[1] && pB1[1] < pA3[1])
        {
          this._crossIndex[0] = pA3[0];
          this._crossIndex[1] = pB1[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index;

          if (pB1[0] < pA3[0] && pA3[0] < pB2[0])
          {
            if (pB2[1] <= pB3[1] && pA1[0] <= pA2[0])
            {
              this._type = 0;
              this._hCrossSegMin = pB1[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA3[1];
              this._distFromStartA = distToA1 + pA2[0] - pA1[0] + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB1[0] > pA3[0] && pA3[0] > pB2[0])
          {
            if (pB2[1] <= pB3[1] && pA2[0] <= pA1[0])
            {
              this._type = 5;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB1[0];
              this._vCrossSegMin = pA2[1];
              this._vCrossSegMax = pA3[1];
              this._distFromStartA = distToA1 + pA1[0] - pA2[0] + this._crossIndex[1] - this._vCrossSegMin;
              this._distFromStartB = distToB1 + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pA2[1] > pB1[1] && pB1[1] > pA3[1])
        {
          this._crossIndex[0] = pA3[0];
          this._crossIndex[1] = pB1[1];
          this._crossSegIndexA = segA1A2Index + 1;
          this._crossSegIndexB = segB1B2Index;

          if (pB1[0] < pA3[0] && pA3[0] < pB2[0])
          {
            if (pB3[1] <= pB2[1] && pA1[0] <= pA2[0])
            {
              this._type = 2;
              this._hCrossSegMin = pB1[0];
              this._hCrossSegMax = pB2[0];
              this._vCrossSegMin = pA3[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + pA2[0] - pA1[0] + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + this._crossIndex[0] - this._hCrossSegMin;
            }
          }
          else if (pB1[0] > pA3[0] && pA3[0] > pB2[0])
          {
            if (pB3[1] <= pB2[1] && pA2[0] <= pA1[0])
            {
              this._type = 7;
              this._hCrossSegMin = pB2[0];
              this._hCrossSegMax = pB1[0];
              this._vCrossSegMin = pA3[1];
              this._vCrossSegMax = pA2[1];
              this._distFromStartA = distToA1 + pA1[0] - pA2[0] + this._vCrossSegMax - this._crossIndex[1];
              this._distFromStartB = distToB1 + this._hCrossSegMax - this._crossIndex[0];
            }
          }
        }
        else if (pB2[1] < pA1[1] && pA1[1] < pB3[1])
        {
          this._crossIndex[0] = pB3[0];
          this._crossIndex[1] = pA1[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pA1[0] < pB3[0] && pB3[0] < pA2[0])
          {
            if (pA2[1] <= pA3[1] && pB1[0] <= pB2[0])
            {
              this._type = 0;
              this._hCrossSegMin = pA1[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB3[1];
              this._distFromStartA = distToA1 + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + pB2[0] - pB1[0] + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
          else if (pA1[0] > pB3[0] && pB3[0] > pA2[0])
          {
            if (pA2[1] <= pA3[1] && pB2[0] <= pB1[0])
            {
              this._type = 5;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA1[0];
              this._vCrossSegMin = pB2[1];
              this._vCrossSegMax = pB3[1];
              this._distFromStartA = distToA1 + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + pB1[0] - pB2[0] + this._crossIndex[1] - this._vCrossSegMin;
            }
          }
        }
        else if (pB2[1] > pA1[1] && pA1[1] > pB3[1])
        {
          this._crossIndex[0] = pB3[0];
          this._crossIndex[1] = pA1[1];
          this._crossSegIndexA = segA1A2Index;
          this._crossSegIndexB = segB1B2Index + 1;

          if (pA1[0] < pB3[0] && pB3[0] < pA2[0])
          {
            if (pA3[1] <= pA2[1] && pB1[0] <= pB2[0])
            {
              this._type = 2;
              this._hCrossSegMin = pA1[0];
              this._hCrossSegMax = pA2[0];
              this._vCrossSegMin = pB3[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + this._crossIndex[0] - this._hCrossSegMin;
              this._distFromStartB = distToB1 + pB2[0] - pB1[0] + this._vCrossSegMax - this._crossIndex[1];
            }
          }
          else if (pA1[0] > pB3[0] && pB3[0] > pA2[0])
          {
            if (pA3[1] <= pA2[1] && pB2[0] <= pB1[0])
            {
              this._type = 7;
              this._hCrossSegMin = pA2[0];
              this._hCrossSegMax = pA1[0];
              this._vCrossSegMin = pB3[1];
              this._vCrossSegMax = pB2[1];
              this._distFromStartA = distToA1 + this._hCrossSegMax - this._crossIndex[0];
              this._distFromStartB = distToB1 + pB1[0] - pB2[0] + this._vCrossSegMax - this._crossIndex[1];
            }
          }
        }
      }
    }

    return (this._type != -1); // Boolean
  },

  letSurvive : function(point)
  {
    // summary:
    //		Returns true if the input cross point is not on a segment that would
    //		be removed once the crossing is eliminated.
    // tags:
    //		private
    
    var x = point.getCrossIndex(0);
    var y = point.getCrossIndex(1);

    if (x == this._crossIndex[0])
    {
      if ((this._type % 4) == 0 || (this._type % 4) == 1)
      {
        if (this._vCrossSegMin + 1 <= y && y <= this._crossIndex[1] - 1) {
          return (false); // Boolean
        }
      }
      else
      {
        if (this._crossIndex[1] + 1 <= y && y <= this._vCrossSegMax - 1) {
          return (false); // Boolean
        }
      }
    }

    if (y == this._crossIndex[1])
    {
      if (this._type % 2 == 0)
      {
        if (this._crossIndex[0] + 1 <= x && x <= this._hCrossSegMax - 1) {
          return (false); // Boolean
        }
      }
      else
      {
        if (this._hCrossSegMin + 1 <= x && x <= this._crossIndex[0] - 1) {
          return (false); // Boolean
        }
      }
    }

    return (true); // Boolean
  }

}
);

return LLCrossPoint;
});

},
'ibm_ilog/graphlayout/longlink/LongLinkLayoutReport':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LongLinkLayoutReport", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../GraphLayoutReport"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    GraphLayoutReport	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var GraphLayoutReport = ibm_ilog.graphlayout.GraphLayoutReport;
=====*/

var LongLinkLayoutReport = declare("ibm_ilog.graphlayout.longlink.LongLinkLayoutReport", GraphLayoutReport, 
{
  // summary:
  //		The class used to store information about the behavior of the
  //		Orthogonal Long Link Layout algorithm.
  
  _numberOfFallbackLinks : null,

  constructor : function()
  {
    this._numberOfFallbackLinks = 0;
  },
  
  setNumberOfFallbackLinks : function(number)
  {
    // summary:
    //		Sets the number of fallback links.
    // tags:
    //		private
    
    this._numberOfFallbackLinks = number;
  },

  incrNumberOfFallbackLinks : function()
  {
    // summary:
    //		Increments the number of fallback links.
    // tags:
    //		private
    
    this._numberOfFallbackLinks += 1;
  },

  getNumberOfFallbackLinks : function()
  {
    // summary:
    //		Returns the number of links that could not be routed on the grid
    //		without obstacle overlaps. 
    // description:
    //		Returns the number of links that could not be routed on the grid
    //		without obstacle overlaps. 
    //		<p>
    //		A link may not be routed without overlaps if the allowed time to
    //		find a route for this link has elapsed (see <code>
    //		LongLinkLayout.getAllowedTimePerLink()</code>), or if the route
    //		would have to bypass too many obstacles (see <code>
    //		LongLinkLayout.getMaxBacktrack()</code>), or if there is no path on
    //		the grid from the start node to the end node that bypasses all
    //		obstacles.  
    //		</p>
    //		<p>
    //		If the fallback routing option was set to <code>true</code> by using
    //		<code> LongLinkLayout.setFallbackRouteEnabled()</code>, such links
    //		are routed by a simple fallback routing algorithm.  
    //		</p>
    // returns:
    //		The number of links that could not be routed on the grid without
    //		obstacle overlaps.
    
    return this._numberOfFallbackLinks; // int
  }

}
);

return LongLinkLayoutReport;
});

},
'ibm_ilog/graphlayout/shortlink/_SLNodeData':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_SLNodeData", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/Rectangle",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../shortlink/_SLNodeOrLinkData",	// #9
  "../shortlink/_SLNodeSide",	// #10
  "../Direction"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    Rectangle,	// #7
    TranslateUtil,	// #8
    _SLNodeOrLinkData,	// #9
    _SLNodeSide,	// #10
    Direction	// #11
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _SLNodeOrLinkData = ibm_ilog.graphlayout.shortlink._SLNodeOrLinkData;
=====*/

var _SLNodeData = declare("ibm_ilog.graphlayout.shortlink._SLNodeData", _SLNodeOrLinkData, 
{
  // summary:
  //		An object to data about nodes. 
  // description:
  //		An object to data about nodes. 
  //		An instance of NodeData is (temporarily) stored in each original
  //		node.   
  // tags:
  //		private
  
  _linkConnectionRect : null,
  _topSide: null,
  _bottomSide: null,
  _leftSide: null,
  _rightSide: null,
  _topSideQuasiSelfIG: null,
  _bottomSideQuasiSelfIG: null,
  _leftSideQuasiSelfIG: null,
  _rightSideQuasiSelfIG: null,

  _connectorStyle: null,

  _totalIncidentLinkWidth: null,

  _degree: null,

  _selfLinkIndex: null,

  _sameBoxesAsPreviously : null,

  constructor : function(node)
  {
    this._linkConnectionRect = new Rectangle();
    this._sameBoxesAsPreviously = false;
  },
  
  update : function(boundingBox, linkConnectionRect, connectorStyle, graphModel, incrementalMode)
  {
    // summary:
    //		Must be called before any use of an instance inside a layout
    //		session.
    // tags:
    //		private
    
    if (incrementalMode) {
      this._sameBoxesAsPreviously = this.boundingBox != null && TranslateUtil.rectEquals(this.boundingBox, boundingBox) && this._linkConnectionRect != null &&
      TranslateUtil.rectEquals(this._linkConnectionRect, linkConnectionRect);
    }
    TranslateUtil.reshapeFromRect(this.boundingBox, boundingBox);
    TranslateUtil.reshapeFromRect(this._linkConnectionRect, linkConnectionRect);
    this._connectorStyle = connectorStyle;
    this._totalIncidentLinkWidth = 0;
    this._degree = GraphModelUtil.getLinksCount(graphModel, this.nodeOrLink);

    this._selfLinkIndex = 0;
    this.cleanNodeSides();
  },

  hasSameBoxesAsPreviously : function()
  {
    return this._sameBoxesAsPreviously; // Boolean
  },

  getTopSide : function (a0, a1)
  {
    if(arguments.length == 2)
      return this.getTopSide_overload_2(a0, a1);
    else if(arguments.length == 1)
      return this.getTopSide_overload_1(a0);
  },
  
  getTopSide_overload_2 : function(forceCreation, isQuasiSelfInterGraphLink)
  {
    return (isQuasiSelfInterGraphLink ? this.getTopSideQuasiSelfIG(forceCreation) : this.getTopSide(forceCreation)); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getBottomSide : function (a0, a1)
  {
    if(arguments.length == 2)
      return this.getBottomSide_overload_2(a0, a1);
    else if(arguments.length == 1)
      return this.getBottomSide_overload_1(a0);
  },
  
  getBottomSide_overload_2 : function(forceCreation, isQuasiSelfInterGraphLink)
  {
    return (isQuasiSelfInterGraphLink ? this.getBottomSideQuasiSelfIG(forceCreation) : this.getBottomSide(forceCreation)); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getLeftSide : function (a0, a1)
  {
    if(arguments.length == 2)
      return this.getLeftSide_overload_2(a0, a1);
    else if(arguments.length == 1)
      return this.getLeftSide_overload_1(a0);
  },
  
  getLeftSide_overload_2 : function(forceCreation, isQuasiSelfInterGraphLink)
  {
    return (isQuasiSelfInterGraphLink ? this.getLeftSideQuasiSelfIG(forceCreation) : this.getLeftSide(forceCreation)); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getRightSide : function (a0, a1)
  {
    if(arguments.length == 2)
      return this.getRightSide_overload_2(a0, a1);
    else if(arguments.length == 1)
      return this.getRightSide_overload_1(a0);
  },
  
  getRightSide_overload_2 : function(forceCreation, isQuasiSelfInterGraphLink)
  {
    return (isQuasiSelfInterGraphLink ? this.getRightSideQuasiSelfIG(forceCreation) : this.getRightSide(forceCreation)); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getTopSide_overload_1 : function(forceCreation)
  {
    if (forceCreation && this._topSide == null) {
      this._topSide = new _SLNodeSide.SLTopNodeSide();
    }
    return this._topSide; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getBottomSide_overload_1 : function(forceCreation)
  {
    if (forceCreation && this._bottomSide == null) {
      this._bottomSide = new _SLNodeSide.SLBottomNodeSide();
    }
    return this._bottomSide; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getLeftSide_overload_1 : function(forceCreation)
  {
    if (forceCreation && this._leftSide == null) {
      this._leftSide = new _SLNodeSide.SLLeftNodeSide();
    }
    return this._leftSide; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getRightSide_overload_1 : function(forceCreation)
  {
    if (forceCreation && this._rightSide == null) {
      this._rightSide = new _SLNodeSide.SLRightNodeSide();
    }
    return this._rightSide; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getTopSideQuasiSelfIG : function(forceCreation)
  {
    if (forceCreation && this._topSideQuasiSelfIG == null) {
      this._topSideQuasiSelfIG = new _SLNodeSide.SLTopNodeSide(true);
    }
    return this._topSideQuasiSelfIG; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getBottomSideQuasiSelfIG : function(forceCreation)
  {
    if (forceCreation && this._bottomSideQuasiSelfIG == null) {
      this._bottomSideQuasiSelfIG = new _SLNodeSide.SLBottomNodeSide(true);
    }
    return this._bottomSideQuasiSelfIG; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getLeftSideQuasiSelfIG : function(forceCreation)
  {
    if (forceCreation && this._leftSideQuasiSelfIG == null) {
      this._leftSideQuasiSelfIG = new _SLNodeSide.SLLeftNodeSide(true);
    }
    return this._leftSideQuasiSelfIG; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getRightSideQuasiSelfIG : function(forceCreation)
  {
    if (forceCreation && this._rightSideQuasiSelfIG == null) {
      this._rightSideQuasiSelfIG = new _SLNodeSide.SLRightNodeSide(true);
    }
    return this._rightSideQuasiSelfIG; // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  cleanNodeSides : function()
  {
    if (this._topSide != null) {
      this._topSide.clean();
    }
    if (this._bottomSide != null) {
      this._bottomSide.clean();
    }
    if (this._leftSide != null) {
      this._leftSide.clean();
    }
    if (this._rightSide != null) {
      this._rightSide.clean();
    }
    if (this._topSideQuasiSelfIG != null) {
      this._topSideQuasiSelfIG.clean();
    }
    if (this._bottomSideQuasiSelfIG != null) {
      this._bottomSideQuasiSelfIG.clean();
    }
    if (this._leftSideQuasiSelfIG != null) {
      this._leftSideQuasiSelfIG.clean();
    }
    if (this._rightSideQuasiSelfIG != null) {
      this._rightSideQuasiSelfIG.clean();
    }
  },

  getConnectorStyle : function()
  {
    // summary:
    // tags:
    //		private
    
    return this._connectorStyle; // int
  },

  isNode : function()
  {
    // summary:
    //		Returns true if this is a node, false otherwise.
    // tags:
    //		private
    
    return true; // Boolean
  },

  isLink : function()
  {
    // summary:
    //		Returns true if this is a link, false otherwise.
    // tags:
    //		private
    
    return false; // Boolean
  },

  getLinkConnectionRect : function()
  {
    // summary:
    //		Returns the link connection rectangle.
    // tags:
    //		private
    
    return this._linkConnectionRect; // Rectangle
  },

  setDegree : function(degree)
  {
    this._degree = degree;
  },

  getDegree : function()
  {
    // summary:
    //		Returns the degree of the node.
    // tags:
    //		private
    
    return this._degree; // int
  },

  getTotalIncidentLinkWidth : function()
  {
    // summary:
    //		Returns the sum of the width of all the links incident to this node.
    // tags:
    //		private
    
    return this._totalIncidentLinkWidth; // Number
  },

  setTotalIncidentLinkWidth : function(width)
  {
    this._totalIncidentLinkWidth = width;
  },

  getSelfLinkIndex : function()
  {
    // summary:
    //		Returns the index which is used to distribute self-links to
    //		different node sides.
    // tags:
    //		private
    
    return this._selfLinkIndex; // int
  },

  incrementSelfLinkIndex : function(maxSelfIndex)
  {
    // summary:
    //		Passes to the next index. 
    // description:
    //		Passes to the next index. 
    //		We want to avoid that most of the self-links are placed on one
    //		particular node side. In other words, we want to distribute them
    //		more uniformly.
    // tags:
    //		private
    
    if (this._selfLinkIndex >= maxSelfIndex) {
      this._selfLinkIndex = 0;
    }
    else {
      this._selfLinkIndex++;
    }
  },

  isIntersectingObjectValid : function(linkData)
  {
    // summary:
    //		Returns true if the intersection between this NodeData and a
    //		LinkData has to be taken into consideration. 
    // description:
    //		Returns true if the intersection between this NodeData and a
    //		LinkData has to be taken into consideration. 
    //		Returns false otherwise.
    // tags:
    //		private
    
    return (linkData.getFromNode() != this && linkData.getToNode() != this); // Boolean
  },

  layoutNodeSides : function(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, linkOffset, minFinalSegmentLength, bundleMode)
  {
    // summary:
    //		Lays out the node sides.
    // tags:
    //		private
    
    if (this._topSide != null) {
      this._topSide.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._bottomSide != null) {
      this._bottomSide.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._leftSide != null) {
      this._leftSide.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._rightSide != null) {
      this._rightSide.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._topSideQuasiSelfIG != null) {
      this._topSideQuasiSelfIG.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._bottomSideQuasiSelfIG != null) {
      this._bottomSideQuasiSelfIG.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._leftSideQuasiSelfIG != null) {
      this._leftSideQuasiSelfIG.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._rightSideQuasiSelfIG != null) {
      this._rightSideQuasiSelfIG.layout(layout, nodeSideLayouter, linkSorter, linkRefiner, connectionBoxInterface, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
  },

  refineNodeSides : function(layout, nodeSideLayouter, linkRefiner, linkOffset, minFinalSegmentLength, bundleMode)
  {
    // summary:
    //		Refines the node sides.
    // tags:
    //		private
    
    if (this._topSide != null) {
      this._topSide.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._bottomSide != null) {
      this._bottomSide.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._leftSide != null) {
      this._leftSide.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._rightSide != null) {
      this._rightSide.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._topSideQuasiSelfIG != null) {
      this._topSideQuasiSelfIG.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._bottomSideQuasiSelfIG != null) {
      this._bottomSideQuasiSelfIG.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._leftSideQuasiSelfIG != null) {
      this._leftSideQuasiSelfIG.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
    if (this._rightSideQuasiSelfIG != null) {
      this._rightSideQuasiSelfIG.refine(layout, nodeSideLayouter, linkRefiner, this, linkOffset, minFinalSegmentLength, bundleMode);
    }
  },

  getSideOfPoint : function(point, isQuasiSelfInterGraphLink)
  {
    // summary:
    //		Returns the node side to which 'point' is the closest.  
    // isQuasiSelfInterGraphLink: Boolean
    //		True if all the following are true: -
    //		the point is the connection point of a link to this nodeData - this
    //		nodeData is represents a subgrapher in a SubgraphData model. - the
    //		link is an intergraph link connecting this nodeData with some node
    //		located inside the subgraph or in some sub(sub...)graph of it.
    //		False otherwise.
    // tags:
    //		private
    
    var side = _SLNodeData.getSideOfPoint(this.getLinkConnectionRect(), point);

    switch (side)
    {
      case Direction.TOP:
        return this.getTopSide(true, isQuasiSelfInterGraphLink); // ibm_ilog.graphlayout.shortlink._SLNodeSide
      case Direction.BOTTOM:
        return this.getBottomSide(true, isQuasiSelfInterGraphLink); // ibm_ilog.graphlayout.shortlink._SLNodeSide
      case Direction.LEFT:
        return this.getLeftSide(true, isQuasiSelfInterGraphLink); // ibm_ilog.graphlayout.shortlink._SLNodeSide
      case Direction.RIGHT:
        return this.getRightSide(true, isQuasiSelfInterGraphLink); // ibm_ilog.graphlayout.shortlink._SLNodeSide
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6630E",  side);
        return null; // ibm_ilog.graphlayout.shortlink._SLNodeSide
    }
  }

}
);

lang.mixin(_SLNodeData,
{

  getSideOfPoint : function(rect, point)
  {
    if (point.x <= rect.x) {
      return Direction.LEFT; // int
    }
    else if (point.x >= rect.x + rect.width) {
      return Direction.RIGHT; // int
    }
    else if (point.y <= rect.y) {
      return Direction.TOP; // int
    }
    else if (point.y >= rect.y + rect.height) {
      return Direction.BOTTOM; // int
    }

    var dxLeft = point.x - rect.x;
    var dxRight = rect.x + rect.width - point.x;
    var dyTop = point.y - rect.y;
    var dyBottom = rect.y + rect.height - point.y;

    if (dxLeft <= dxRight)
    {
      if (dyTop <= dyBottom) {
        return (dxLeft <= dyTop ? Direction.LEFT : Direction.TOP); // int
      }
      else {
        return (dxLeft <= dyBottom ? Direction.LEFT : Direction.BOTTOM); // int
      }
    }
    else
    {
      if (dyTop <= dyBottom) {
        return (dxRight <= dyTop ? Direction.RIGHT : Direction.TOP); // int
      }
      else {
        return (dxRight <= dyBottom ? Direction.RIGHT : Direction.BOTTOM); // int
      }
    }
  }

}
);

return _SLNodeData;
});

},
'ibm_ilog/graphlayout/shortlink/_MultiLinkBundleLayouter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_MultiLinkBundleLayouter", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_SLBundlesModes"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _SLBundlesModes	// #7
    ) {

var _MultiLinkBundleLayouter = declare("ibm_ilog.graphlayout.shortlink._MultiLinkBundleLayouter", null, 
{
  // summary:
  //		This class performs the layout of a bundle of multilinks. 
  // description:
  //		This class performs the layout of a bundle of multilinks. 
  //		The mechanism is the following: For each set of multilinks, the
  //		LinkData of one of the links is flagged as being the "master" link of
  //		the bundle, while the others are "slaves". The master stores the list
  //		of slaves. During the optimization, only the master link is
  //		considered. At the end of the layout, or in animated mode, the real
  //		links need to be lay out according to the computed shape of the
  //		master link. One single instance is used by each instance of
  //		ShortLinkLayout.  
  // tags:
  //		private
  
  constructor : function()
  {
  },
  
  clean : function()
  {
    // summary:
    //		Cleans the data stored by this class.
    // tags:
    //		private
    
  },

  layout : function (a0, a1, a2, a3, a4, a5, a6)
  {
    // summary:
    //		Gives the final shape to a master link together with its slave
    //		links. 
    // description:
    //		Gives the final shape to a master link together with its slave
    //		links. 
    //		Used in "sameShapeForMultipleLinks" mode only.
    // tags:
    //		private
    
    if(arguments.length == 7)
      return this.layout_overload_7(a0, a1, a2, a3, a4, a5, a6);
    else if(arguments.length == 6)
      return this.layout_overload_6(a0, a1, a2, a3, a4, a5);
  },
  
  layout_overload_7 : function(sllayout, graphModel, masterLink, bundleMode, linkOffset, minFinalSegmentLength, redraw)
                                                                                                                      
  {
    var slaveLinks = masterLink.getIndividualLinks();
    if (slaveLinks == null) {
      return;
    }
    var nLinks = slaveLinks.length;
    if (nLinks == 0) {
      return;
    }
    this.layout(sllayout, graphModel, masterLink, bundleMode, linkOffset, redraw);
  },

  layout_overload_6 : function(layout, graphModel, masterLink, bundleMode, linkOffset, redraw)
  {
    // summary:
    //		Lays out the bundle of links. 
    // description:
    //		Lays out the bundle of links. 
    //		For left orientation: ------------ --------- | startLink------ | | |
    //		| | --------- |       | --------- And for right orientation:
    //		------------ |  --------- |  |  ------startLink |  |  | --------- | 
    //		     | ---------
    // tags:
    //		private
    
    var slaveLinks = masterLink.getIndividualLinks();
    var nLinks = slaveLinks.length;
    var slaveLink;
    var masterShape;

    var masterLinkWidth = 0;
    var masterLinkWidthOrig = 0;
    var masterLinkWidthDest = 0;
    var masterLinkOffsetOrig = 0;
    var masterLinkOffsetDest = 0;
    var masterFrom = null;
    var masterTo = null;
    var masterBends = null;
    var masterNBends = 0;
    var onLeftSide = false;
    var halfLinkWidth = 0;
    var previousHalfLinkWidth = 0;
    var offset = 0;
    var offsetOrig = 0;
    var offsetDest = 0;
    var onlyFirstAndLastSeg = bundleMode != _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT;

    var copyPointLocations = onlyFirstAndLastSeg;

    if (bundleMode != _SLBundlesModes.NO_BUNDLE)
    {
      masterFrom = masterLink.getConnectionPoint(true);
      masterTo = masterLink.getConnectionPoint(false);
      masterShape = masterLink.getLinkShape();
      masterBends = masterShape.getIntermediatePoints();
      masterNBends = masterShape.getNumberOfPoints() - 2;
      masterLinkWidth = masterLink.getLinkWidth();
      masterLinkWidthOrig = masterLink.getLinkWidthWithOffset(masterLink.getOriginLinkOffset());
      masterLinkWidthDest = masterLink.getLinkWidthWithOffset(masterLink.getDestinationLinkOffset());
      masterLinkOffsetOrig = masterLink.getOriginLinkOffset();
      masterLinkOffsetDest = masterLink.getDestinationLinkOffset();

      onLeftSide = masterLink.isGoingLeft(true);
    }

    for (var i = 0; i < nLinks; i++)
    {
      slaveLink = slaveLinks[i];

      slaveLink.prepareForBundleLayout(masterLink, copyPointLocations);

      if (bundleMode != _SLBundlesModes.NO_BUNDLE)
      {
        halfLinkWidth = slaveLink.getLinkWidth() * 0.5;

        if (i == 0)
        {
          offset = masterLinkWidth * 0.5 - halfLinkWidth;
          offsetOrig = masterLinkWidthOrig * 0.5 - halfLinkWidth;
          offsetDest = masterLinkWidthDest * 0.5 - halfLinkWidth;
        }
        else
        {
          offset -= (linkOffset + halfLinkWidth + previousHalfLinkWidth);
          offsetOrig -= (masterLinkOffsetOrig + halfLinkWidth + previousHalfLinkWidth);
          offsetDest -= (masterLinkOffsetDest + halfLinkWidth + previousHalfLinkWidth);
        }

        previousHalfLinkWidth = halfLinkWidth;
        slaveLink.applyOffset(masterFrom, masterTo, masterBends, masterNBends, offset, offsetOrig, offsetDest, onLeftSide, masterLink.getFromNode() != slaveLink.getFromNode(), onlyFirstAndLastSeg);
      }

      layout.finalShapeLink(graphModel, slaveLink, false, redraw);
    }
  }

}
);

return _MultiLinkBundleLayouter;
});

},
'ibm_ilog/graphlayout/shortlink/_DirectIncidentLinksRefiner':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_DirectIncidentLinksRefiner", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../shortlink/_IncidentLinksRefiner"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    TranslateUtil,	// #7
    _IncidentLinksRefiner	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _IncidentLinksRefiner = ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner;
=====*/

var _DirectIncidentLinksRefiner = declare("ibm_ilog.graphlayout.shortlink._DirectIncidentLinksRefiner", _IncidentLinksRefiner, 
{
  // summary:
  //		A subclass specialized for direct link style. 
  // description:
  //		A subclass specialized for direct link style. 
  //		
  // tags:
  //		private
  
  constructor : function(layout)
  {
  },
  
  refine : function(vectLinks, nodeData, nodeSide, linkOffset, minFinalSegmentLength, reversedOrder, withFixedConnectionPoints)
  {
    // summary:
    //		Final refinement of the link layout. 
    // description:
    //		Final refinement of the link layout. 
    //		Once the type of shape for each link and the order of the connection
    //		of all the links incident to a given side of a node are choosen, the
    //		second and, eventually, the third segment of an orthogonal link can
    //		be moved a little by this refinement procedure. The purpose is to
    //		avoid overlapping segments for links that are incident to the same
    //		side of the same node. (Currently, nothing is done for links that
    //		are not incident to the same side of the same node.)  
    //		<p>
    //		Note that, currently, this refinement procedure concerns only the
    //		orthogonal links style. In the future, something could be done for
    //		direct links also (placement of the first points of the links on an
    //		arc, as TGO does.)
    //		</p>
    // tags:
    //		private
    
    var nLinks = (vectLinks == null ? 0 : vectLinks.length);
    if (nLinks < 2) {
      return;
    }

    var deltaRadCoord = 0;
    var isSameSideSelfLinkProcessed = false;
    var link;
    var point1 = null;
    var point2;
    var previousPoint1 = this.getLayoutAlgorithm().getTempPoint1();
    var tangentDelta;
    var tangentDeltaWithPreviousLink = 0;
    var linkBendHeight;
    var linkWidth;
    var linkWidth2;
    var isOrtho;
    var isSameSideSelfLink;
    var radialDelta;
    var refPoint = null;
    var value;
    var origin;
    var i = 0;

    for (i = 0; i < nLinks; i++)
    {
      link = LayoutUtil.elementAt(vectLinks, i, nLinks, reversedOrder);
      if (link.isFixed()) {
        continue;
      }
      isSameSideSelfLink = link.isSameSideSelfLink();

      if (isSameSideSelfLink)
      {
        if (isSameSideSelfLinkProcessed) {
          continue;
        }
        else {
          isSameSideSelfLinkProcessed = true;
        }
      }

      origin = link.isOrigin(nodeData, nodeSide);
      point1 = link.getPointAt(origin, 1);
      point2 = link.getPointAt(origin, 2);

      if (i > 0)
      {
        tangentDeltaWithPreviousLink = nodeSide.getTangentDelta(previousPoint1, point1);
        if (tangentDeltaWithPreviousLink < 0) {
          tangentDeltaWithPreviousLink = -tangentDeltaWithPreviousLink;
        }
      }

      TranslateUtil.setPointLocation(previousPoint1, point1);
      tangentDelta = nodeSide.getTangentDelta(point1, point2);
      if (refPoint == null) {
        refPoint = point1;
      }
      if (tangentDelta < 0 && !isSameSideSelfLink) {
        break;
      }
      isOrtho = !link.isFixed() && link.getCurrentShapeType().isOrthogonal();
      linkBendHeight = 0;
      linkWidth = link.getLinkWidth();

      if (isOrtho)
      {
        if (i < nLinks - 1)
        {
          var link2 = LayoutUtil.elementAt(vectLinks, i + 1, nLinks, reversedOrder);

          if (!link2.isFixed() && link2.getCurrentShapeType().isOrthogonal())
          {
            linkWidth2 = link2.getLinkWidth();
            this.refineLinksOrthogonal(link, link2, origin, link2.isOrigin(nodeData, nodeSide), nodeSide, true, linkOffset, true);
            linkBendHeight = (i > 0 ? (2 * linkOffset + linkWidth + linkWidth2) : (1.5 * linkOffset + 0.5 * linkWidth + linkWidth2));
            deltaRadCoord += linkBendHeight;
            continue;
          }
        }
        else {
          break;
        }
      }

      radialDelta = nodeSide.getPointRadialDelta(point1, point2);

      value = (radialDelta + Math.sqrt(radialDelta * radialDelta + tangentDelta * tangentDelta));

      linkBendHeight = (linkWidth + linkOffset) * (((value > 0.00001 || value < -0.00001) ? tangentDelta / value : 1));
      if (linkBendHeight < 0) {
        linkBendHeight = -linkBendHeight;
      }
      linkBendHeight = Math.min(linkBendHeight, ((i > 0 ? tangentDeltaWithPreviousLink * 4 : ((tangentDelta >= 0 ? tangentDelta : -tangentDelta)))));

      if (i > 0)
      {
        deltaRadCoord += (linkBendHeight * 0.5);

        if (!isSameSideSelfLink && !link.isFixed())
        {
          link.shapePointsModified();
          nodeSide.setRadialCoord(point1, nodeSide.getPointRadialCoord(refPoint));
          nodeSide.translateOutside(point1, deltaRadCoord);
          if (isOrtho) {
            nodeSide.setRadialCoord(point2, nodeSide.getPointRadialCoord(point1));
          }
          link.getLinkShape().invalidateBBox();
        }
      }

      deltaRadCoord += (linkBendHeight * 0.5);
    }

    deltaRadCoord = 0;
    tangentDeltaWithPreviousLink = 0;
    isSameSideSelfLinkProcessed = false;
    var maxIndex = nLinks - 1;
    refPoint = null;

    for (i = maxIndex; i >= 0; i--)
    {
      link = LayoutUtil.elementAt(vectLinks, i, nLinks, reversedOrder);
      if (link.isFixed()) {
        continue;
      }
      isSameSideSelfLink = link.isSameSideSelfLink();

      if (isSameSideSelfLink)
      {
        if (isSameSideSelfLinkProcessed) {
          continue;
        }
        else {
          isSameSideSelfLinkProcessed = true;
        }
      }

      origin = link.isOrigin(nodeData, nodeSide);
      point1 = link.getPointAt(origin, 1);
      point2 = link.getPointAt(origin, 2);

      if (i < maxIndex)
      {
        tangentDeltaWithPreviousLink = nodeSide.getTangentDelta(previousPoint1, point1);
        if (tangentDeltaWithPreviousLink < 0) {
          tangentDeltaWithPreviousLink = -tangentDeltaWithPreviousLink;
        }
      }

      TranslateUtil.setPointLocation(previousPoint1, point1);
      if (refPoint == null) {
        refPoint = point1;
      }
      tangentDelta = nodeSide.getTangentDelta(point1, point2);
      if (tangentDelta >= 0 && !isSameSideSelfLink) {
        break;
      }
      isOrtho = !link.isFixed() && link.getCurrentShapeType().isOrthogonal();
      linkBendHeight = 0;
      linkWidth = link.getLinkWidth();

      if (isOrtho)
      {
        if (i > 0)
        {
          var linkTwo = LayoutUtil.elementAt(vectLinks, i - 1, nLinks, reversedOrder);

          if (!linkTwo.isFixed() && linkTwo.getCurrentShapeType().isOrthogonal())
          {
            linkWidth2 = linkTwo.getLinkWidth();
            this.refineLinksOrthogonal(link, linkTwo, origin, linkTwo.isOrigin(nodeData, nodeSide), nodeSide, false, linkOffset, true);
            linkBendHeight = (i < maxIndex ? (2 * linkOffset + linkWidth + linkWidth2) : (1.5 * linkOffset + 0.5 * linkWidth + linkWidth2));
            deltaRadCoord += linkBendHeight;
            continue;
          }
        }
        else {
          break;
        }
      }

      radialDelta = nodeSide.getPointRadialDelta(point1, point2);

      value = (radialDelta + Math.sqrt(radialDelta * radialDelta + tangentDelta * tangentDelta));

      linkBendHeight = (linkWidth + linkOffset) * (((value > 0.00001 || value < -0.00001) ? -tangentDelta / value : 1));
      if (linkBendHeight < 0) {
        linkBendHeight = -linkBendHeight;
      }
      linkBendHeight = Math.min(linkBendHeight, ((i < maxIndex ? tangentDeltaWithPreviousLink * 4 : ((tangentDelta >= 0 ? tangentDelta : -tangentDelta)))));

      if (i < maxIndex)
      {
        deltaRadCoord += (linkBendHeight * 0.5);

        if (!isSameSideSelfLink && !link.isFixed())
        {
          link.shapePointsModified();
          nodeSide.setRadialCoord(point1, nodeSide.getPointRadialCoord(refPoint));
          nodeSide.translateOutside(point1, deltaRadCoord);
          if (isOrtho) {
            nodeSide.setRadialCoord(point2, nodeSide.getPointRadialCoord(point1));
          }
          link.getLinkShape().invalidateBBox();
        }
      }

      deltaRadCoord += (linkBendHeight * 0.5);
    }
  }

}
);

return _DirectIncidentLinksRefiner;
});

},
'ibm_ilog/graphlayout/longlink/LLCrossRedAlgorithm':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLCrossRedAlgorithm", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../longlink/LLCrossPoint",	// #7
  "../longlink/LLTerminationPoint"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    LLCrossPoint,	// #7
    LLTerminationPoint	// #8
    ) {

var LLCrossRedAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLCrossRedAlgorithm", null, 
{
  // summary:
  //		The algorithm performs a crossing reduction between two links. 
  // description:
  //		The algorithm performs a crossing reduction between two links. 
  //		It detecs that two links cross more than one time and tries to
  //		eliminate a pair of crossings.  
  //		<p>
  //		 
  //		</p>
  // tags:
  //		private
  
  _layout: null,

  _grid: null,

  _listOfCrossPoints: null,

  _crossing1: null,
  _crossing2: null,

  _bypassPoint1: null,
  _bypassPoint2: null,

  _segArray: null,
  _prevObstacleFlags: null,
  _nextObstacleFlags: null,

  _oldPrevObstacleFlags: null,
  _oldNextObstacleFlags: null,

  _otherPrevObstacleFlags: null,
  _otherNextObstacleFlags: null,

  constructor : function(layout, grid)
  {
    this._sp1 = TranslateUtil.newArray(2, 0);
    this._sp2 = TranslateUtil.newArray(2, 0);
    this._bp0 = TranslateUtil.newArray(2, 0);
    this._bp1 = TranslateUtil.newArray(2, 0);
    this._bp2 = TranslateUtil.newArray(2, 0);
    this._slimits = TranslateUtil.newArray(2, 0);
    this._blimits = TranslateUtil.newArray(2, 0);
    this._sgridCorr = TranslateUtil.newArray(2, 0);
    this._bgridCorr = TranslateUtil.newArray(2, 0);
    this._layout = layout;
    this._grid = grid;
    this._listOfCrossPoints = null;
    this._bypassPoint1 = TranslateUtil.newArray(2, 0);
    this._bypassPoint2 = TranslateUtil.newArray(2, 0);
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._grid = null;
    this._bypassPoint1 = null;
    this._bypassPoint2 = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getGrid : function()
  {
    // summary:
    //		Returns the routing grid.
    // tags:
    //		private
    
    return (this._grid); // ibm_ilog.graphlayout.longlink.LLGrid
  },

  run : function(link1, link2)
  {
    // summary:
    //		Starts the algorithm. 
    // description:
    //		Starts the algorithm. 
    //		If link1 and link2 cross several times, it tries to reduce the
    //		crossings by exchanging part of the links.
    // tags:
    //		private
    
    if (!link1.mayIntersectLink(link2)) {
      return (false); // Boolean
    }
    if (this.isStraightTerminationCrossing(link1, link2))
    {
      if (this.reduceStraightTerminationCrossing())
      {
        this.markAndPostProcess(link1, link2);
        return (true); // Boolean
      }
    }

    var counter = 0;
    var crossingReduced = false;
    var needContinue = true;
    var numCrossings = this.calcCrossings(link1, link2);

    while (needContinue)
    {
      needContinue = false;

      while (numCrossings > 1 && this.findPairOfReducableCrossings())
      {
        this.reduceCrossingPair(link1, link2);
        crossingReduced = true;

        numCrossings = this.calcCrossings(link1, link2);

        counter++;
        if (counter > 100) {
          break;
        }
      }

      if (numCrossings > 0)
      {
        if (
          link1.getFromNode() == link2.getFromNode() && this.canTerminate(link1, link2.getActStartPoint(), true) && this.canTerminate(link2, link1.getActStartPoint(), true) &&
            this.findSingleReducableCrossing(true))
          {
            this.reduceCrossingByTerminationExchange(link1, link2, true, true);

            crossingReduced = true;
            needContinue = true;

            numCrossings = this.calcCrossings(link1, link2);
          }
        else if (
          link1.getToNode() == link2.getToNode() && this.canTerminate(link1, link2.getActEndPoint(), false) && this.canTerminate(link2, link1.getActEndPoint(), false) &&
            this.findSingleReducableCrossing(true))
          {
            this.reduceCrossingByTerminationExchange(link1, link2, false, false);

            crossingReduced = true;
            needContinue = true;

            numCrossings = this.calcCrossings(link1, link2);
          }
        else if (
          link1.getFromNode() == link2.getToNode() && this.canTerminate(link1, link2.getActEndPoint(), true) && this.canTerminate(link2, link1.getActStartPoint(), false) &&
            this.findSingleReducableCrossing(false))
          {
            this.reduceCrossingByTerminationExchange(link1, link2, true, false);

            crossingReduced = true;
            needContinue = true;

            numCrossings = this.calcCrossings(link1, link2);
          }
        else if (
          link1.getToNode() == link2.getFromNode() && this.canTerminate(link1, link2.getActStartPoint(), false) && this.canTerminate(link2, link1.getActEndPoint(), true) &&
            this.findSingleReducableCrossing(false))
          {
            this.reduceCrossingByTerminationExchange(link1, link2, false, true);

            crossingReduced = true;
            needContinue = true;

            numCrossings = this.calcCrossings(link1, link2);
          }

        counter++;
        if (counter > 100) {
          break;
        }
      }
    }

    if (crossingReduced) {
      this.markAndPostProcess(link1, link2);
    }
    return (crossingReduced); // Boolean
  },

  markAndPostProcess : function(link1, link2)
  {
    // summary:
    //		Marks the links as changed.
    // tags:
    //		private
    
    link1.markLinkChanged();
    link2.markLinkChanged();
  },

  findPairOfReducableCrossings : function()
  {
    // summary:
    //		Search two crossings that are reducable. 
    // description:
    //		Search two crossings that are reducable. 
    //		It returns true if two crossings are found that fit to each other.
    //		It stores the crossings in &#95;crossing1 and &#95;crossing2.
    // tags:
    //		private
    
    var p1 = this._listOfCrossPoints;
    var p2;
    var forwardFlowA;
    var forwardFlowB;
    var okay;

    while (p1 != null)
    {
      if (p1.getType() != -1)
      {
        p2 = p1.getNext();

        while (p2 != null)
        {
          if (p2.getType() != -1)
          {
            okay = (p1.getCrossIndex(0) != p2.getCrossIndex(0) || p1.getCrossIndex(1) != p2.getCrossIndex(1));

            okay = p1.letSurvive(p2) && okay;
            okay = p2.letSurvive(p1) && okay;

            if (okay)
            {
              forwardFlowA = (p1.getDistFromStartA() < p2.getDistFromStartA());
              forwardFlowB = (p1.getDistFromStartB() < p2.getDistFromStartB());

              if (p1.isOppositeFlow() == p2.isOppositeFlow())
              {
                if (p1.isOppositeFlow() && forwardFlowA != forwardFlowB)
                {
                  this._crossing1 = p1;
                  this._crossing2 = p2;
                  return (true); // Boolean
                }
                else if (!p1.isOppositeFlow() && forwardFlowA == forwardFlowB)
                {
                  this._crossing1 = p1;
                  this._crossing2 = p2;
                  return (true); // Boolean
                }
              }
            }
          }

          p2 = p2.getNext();
        }
      }

      p1 = p1.getNext();
    }

    return (false); // Boolean
  },

  reduceCrossingPair : function(link1, link2)
  {
    // summary:
    //		Reduce the two crossings stored in _crossing1 and _crossing2.
    // tags:
    //		private
    
    var oldLink1Segments = link1.getSegments();
    var oldLink2Segments = link2.getSegments();

    this._crossing1.calcBypassPoint(this._bypassPoint1);
    this._crossing2.calcBypassPoint(this._bypassPoint2);

    var segmentIndexLink1Crossing1 = this._crossing1.getCrossSegIndexA();
    var segmentIndexLink1Crossing2 = this._crossing2.getCrossSegIndexA();
    var segmentIndexLink2Crossing1 = this._crossing1.getCrossSegIndexB();
    var segmentIndexLink2Crossing2 = this._crossing2.getCrossSegIndexB();

    var link1BypassAtCrossing1 = this._crossing1.needBypassIfFirst(link1, segmentIndexLink1Crossing1);
    var link1BypassAtCrossing2 = !this._crossing2.needBypassIfFirst(link1, segmentIndexLink1Crossing2);

    if (this._crossing1.getDistFromStartA() > this._crossing2.getDistFromStartA())
    {
      link1BypassAtCrossing1 = !link1BypassAtCrossing1;
      link1BypassAtCrossing2 = !link1BypassAtCrossing2;
    }

    var segsOfLink1BetweenCrossings;
    var segsOfLink2BetweenCrossings;
    var segsOfLink1BeforeAndAfterCrossings;
    var segsOfLink2BeforeAndAfterCrossings;

    if (segmentIndexLink1Crossing1 <= segmentIndexLink1Crossing2)
    {
      segsOfLink1BetweenCrossings = segmentIndexLink1Crossing2 - segmentIndexLink1Crossing1 + 1;
      segsOfLink1BeforeAndAfterCrossings = segmentIndexLink1Crossing1 + oldLink1Segments.length - segmentIndexLink1Crossing2 - 1;
    }
    else
    {
      segsOfLink1BetweenCrossings = segmentIndexLink1Crossing1 - segmentIndexLink1Crossing2 + 1;
      segsOfLink1BeforeAndAfterCrossings = segmentIndexLink1Crossing2 + oldLink1Segments.length - segmentIndexLink1Crossing1 - 1;
    }

    if (segmentIndexLink2Crossing1 <= segmentIndexLink2Crossing2)
    {
      segsOfLink2BetweenCrossings = segmentIndexLink2Crossing2 - segmentIndexLink2Crossing1 + 1;
      segsOfLink2BeforeAndAfterCrossings = segmentIndexLink2Crossing1 + oldLink2Segments.length - segmentIndexLink2Crossing2 - 1;
    }
    else
    {
      segsOfLink2BetweenCrossings = segmentIndexLink2Crossing1 - segmentIndexLink2Crossing2 + 1;
      segsOfLink2BeforeAndAfterCrossings = segmentIndexLink2Crossing2 + oldLink2Segments.length - segmentIndexLink2Crossing1 - 1;
    }

    var n = segsOfLink1BeforeAndAfterCrossings + segsOfLink2BetweenCrossings;
    if (link1BypassAtCrossing1) {
      n--;
    }
    else {
      n++;
    }
    if (link1BypassAtCrossing2) {
      n--;
    }
    else {
      n++;
    }
    if (link1BypassAtCrossing1 && link1BypassAtCrossing2 && segsOfLink2BetweenCrossings == 2) {
      n += 2;
    }
    var newLink1Segments = TranslateUtil.newArray(n, 0);
    var newLink1PrevObstacleFlags = TranslateUtil.newArray(n, false);
    var newLink1NextObstacleFlags = TranslateUtil.newArray(n, false);

    n = segsOfLink2BeforeAndAfterCrossings + segsOfLink1BetweenCrossings;
    if (link1BypassAtCrossing1) {
      n++;
    }
    else {
      n--;
    }
    if (link1BypassAtCrossing2) {
      n++;
    }
    else {
      n--;
    }
    if (!link1BypassAtCrossing1 && !link1BypassAtCrossing2 && segsOfLink1BetweenCrossings == 2) {
      n += 2;
    }
    var newLink2Segments = TranslateUtil.newArray(n, 0);
    var newLink2PrevObstacleFlags = TranslateUtil.newArray(n, false);
    var newLink2NextObstacleFlags = TranslateUtil.newArray(n, false);

    var segIndex;
    var index;
    var limits = TranslateUtil.newArray(2, 0);
    var grid = this.getGrid();
    index = this._crossing1.getHorizontalFreeSegment(limits);
    segIndex = this._crossing1.getCrossSegIndexA();
    grid.removeObstacle(0, index, limits, link1, segIndex);
    segIndex = this._crossing1.getCrossSegIndexB();
    grid.removeObstacle(0, index, limits, link2, segIndex);
    index = this._crossing1.getVerticalFreeSegment(limits);
    segIndex = this._crossing1.getCrossSegIndexA();
    grid.removeObstacle(1, index, limits, link1, segIndex);
    segIndex = this._crossing1.getCrossSegIndexB();
    grid.removeObstacle(1, index, limits, link2, segIndex);
    index = this._crossing2.getHorizontalFreeSegment(limits);
    segIndex = this._crossing2.getCrossSegIndexA();
    grid.removeObstacle(0, index, limits, link1, segIndex);
    segIndex = this._crossing2.getCrossSegIndexB();
    grid.removeObstacle(0, index, limits, link2, segIndex);
    index = this._crossing2.getVerticalFreeSegment(limits);
    segIndex = this._crossing2.getCrossSegIndexA();
    grid.removeObstacle(1, index, limits, link1, segIndex);
    segIndex = this._crossing2.getCrossSegIndexB();
    grid.removeObstacle(1, index, limits, link2, segIndex);

    this.fillSegmentsForCrossingPair(newLink1Segments, newLink1PrevObstacleFlags, newLink1NextObstacleFlags, link1, link2, true, link1BypassAtCrossing1, link1BypassAtCrossing2);

    this.fillSegmentsForCrossingPair(newLink2Segments, newLink2PrevObstacleFlags, newLink2NextObstacleFlags, link2, link1, false, !link1BypassAtCrossing1, !link1BypassAtCrossing2);

    link1.setSegments(newLink1Segments, newLink1PrevObstacleFlags, newLink1NextObstacleFlags);
    link2.setSegments(newLink2Segments, newLink2PrevObstacleFlags, newLink2NextObstacleFlags);

    index = this._crossing1.getHorizontalTestSegment(limits);
    grid.setGridLineObstacle(true, index, limits);
    index = this._crossing1.getVerticalTestSegment(limits);
    grid.setGridLineObstacle(false, index, limits);
    index = this._crossing2.getHorizontalTestSegment(limits);
    grid.setGridLineObstacle(true, index, limits);
    index = this._crossing2.getVerticalTestSegment(limits);
    grid.setGridLineObstacle(false, index, limits);

    this._crossing1.makeUnfeasable();
    this._crossing2.makeUnfeasable();

    if (link1BypassAtCrossing1 && link1BypassAtCrossing2 && segsOfLink2BetweenCrossings == 2) {
      link1.updateGrid();
    }
    if (!link1BypassAtCrossing1 && !link1BypassAtCrossing2 && segsOfLink1BetweenCrossings == 2) {
      link2.updateGrid();
    }
  },

  fillSegmentsForCrossingPair : function(segArray, prevObstacleFlags, nextObstacleFlags, fillLink, otherLink, fillLinkA, bypassC1, bypassC2)
  {
    // summary:
    //		Fills the new segment array of fillLink to elimitate the pair of
    //		crossings.
    // tags:
    //		private
    
    this._segArray = segArray;
    this._prevObstacleFlags = prevObstacleFlags;
    this._nextObstacleFlags = nextObstacleFlags;

    var fIndexC1;
    var fIndexC2;

    var fDistFromStartC1;
    var fDistFromStartC2;

    if (fillLinkA)
    {
      fIndexC1 = this._crossing1.getCrossSegIndexA();
      fIndexC2 = this._crossing2.getCrossSegIndexA();
      fDistFromStartC1 = this._crossing1.getDistFromStartA();
      fDistFromStartC2 = this._crossing2.getDistFromStartA();
    }
    else
    {
      fIndexC1 = this._crossing1.getCrossSegIndexB();
      fIndexC2 = this._crossing2.getCrossSegIndexB();
      fDistFromStartC1 = this._crossing1.getDistFromStartB();
      fDistFromStartC2 = this._crossing2.getDistFromStartB();
    }

    var minFIndex;
    var maxFIndex;
    var isReversed;

    if (fIndexC1 < fIndexC2)
    {
      minFIndex = fIndexC1;
      maxFIndex = fIndexC2;
      isReversed = false;
    }
    else
    {
      maxFIndex = fIndexC1;
      minFIndex = fIndexC2;

      if (fIndexC1 == fIndexC2)
      {
        if (fDistFromStartC1 <= fDistFromStartC2) {
          isReversed = false;
        }
        else {
          isReversed = true;
        }
      }
      else {
        isReversed = true;
      }
    }

    var firstBypass;
    var secondBypass;
    var firstBypassPoint;
    var secondBypassPoint;
    var firstCP;
    var secondCP;
    var otherIndexAtFirstCP;
    var otherIndexAtSecondCP;

    if (isReversed)
    {
      firstBypass = bypassC2;
      secondBypass = bypassC1;
      firstBypassPoint = this._bypassPoint2;
      secondBypassPoint = this._bypassPoint1;
      firstCP = this._crossing2;
      secondCP = this._crossing1;

      if (fillLinkA)
      {
        otherIndexAtFirstCP = this._crossing2.getCrossSegIndexB();
        otherIndexAtSecondCP = this._crossing1.getCrossSegIndexB();
      }
      else
      {
        otherIndexAtFirstCP = this._crossing2.getCrossSegIndexA();
        otherIndexAtSecondCP = this._crossing1.getCrossSegIndexA();
      }
    }
    else
    {
      firstBypass = bypassC1;
      secondBypass = bypassC2;
      firstBypassPoint = this._bypassPoint1;
      secondBypassPoint = this._bypassPoint2;
      firstCP = this._crossing1;
      secondCP = this._crossing2;

      if (fillLinkA)
      {
        otherIndexAtFirstCP = this._crossing1.getCrossSegIndexB();
        otherIndexAtSecondCP = this._crossing2.getCrossSegIndexB();
      }
      else
      {
        otherIndexAtFirstCP = this._crossing1.getCrossSegIndexA();
        otherIndexAtSecondCP = this._crossing2.getCrossSegIndexA();
      }
    }

    var oldSegArray = fillLink.getSegments();
    var otherSegArray = otherLink.getSegments();
    this._oldPrevObstacleFlags = fillLink.getPrevObstacleFlags();
    this._oldNextObstacleFlags = fillLink.getNextObstacleFlags();
    this._otherPrevObstacleFlags = otherLink.getPrevObstacleFlags();
    this._otherNextObstacleFlags = otherLink.getNextObstacleFlags();

    segArray[0] = 0;
    var i = this.transferSegments(oldSegArray, this._oldPrevObstacleFlags, this._oldNextObstacleFlags, 0, 0, minFIndex);

    i = this.handleFirstCrossing(fillLink.getSegmentDir(minFIndex), firstCP, secondCP, otherIndexAtFirstCP, otherIndexAtSecondCP, firstBypass, firstBypassPoint, i);

    var j;
    if (otherIndexAtFirstCP == otherIndexAtSecondCP) {
      j = otherIndexAtFirstCP;
    }
    else if (otherIndexAtFirstCP < otherIndexAtSecondCP) {
      j = otherIndexAtFirstCP + 1;
    }
    else {
      j = otherIndexAtFirstCP - 1;
    }
    i = this.transferSegments(otherSegArray, this._otherPrevObstacleFlags, this._otherNextObstacleFlags, i, j, otherIndexAtSecondCP);

    if (firstBypass && secondBypass && j == otherIndexAtSecondCP)
    {
      this._nextObstacleFlags[i] = false;
      this._prevObstacleFlags[i + 1] = false;
      i += 2;
    }

    i = this.handleSecondCrossing(fillLink.getSegmentDir(maxFIndex), secondCP, maxFIndex, otherIndexAtFirstCP, otherIndexAtSecondCP, secondBypass, secondBypassPoint, i);

    i = this.transferSegments(oldSegArray, this._oldPrevObstacleFlags, this._oldNextObstacleFlags, i, maxFIndex + 1, oldSegArray.length);

    prevObstacleFlags[0] = true;
    nextObstacleFlags[nextObstacleFlags.length - 1] = true;
  },

  transferSegments : function(srcSegArray, srcPrevObstacleFlags, srcNextObstacleFlags, i, start, end)
  {
    // summary:
    //		Transfer the area of srcSegArray between start (inclusive) and end
    //		(exclusive) to the _segArray starting at i. 
    // description:
    //		Transfer the area of srcSegArray between start (inclusive) and end
    //		(exclusive) to the &#95;segArray starting at i. 
    //		It returns the first index in &#95;segArray after the transfered area.
    //		It also transfers the same area of srcPrevObstacleFlags and
    //		srcNextObstacleFlags.
    // tags:
    //		private
    
    var j;

    if (start < end)
    {
      if (this._segArray[i] == 0)
      {
        this._prevObstacleFlags[i] = srcPrevObstacleFlags[start];
        this._nextObstacleFlags[i] = srcNextObstacleFlags[start];
        this._segArray[i++] = srcSegArray[start];
      }
      else
      {
        this._nextObstacleFlags[i] = srcNextObstacleFlags[start];
        this._segArray[i++] += srcSegArray[start];
      }

      for (j = start + 1; j < end; j++)
      {
        this._prevObstacleFlags[i] = srcPrevObstacleFlags[j];
        this._nextObstacleFlags[i] = srcNextObstacleFlags[j];
        this._segArray[i++] = srcSegArray[j];
      }

      if (i < this._segArray.length) {
        this._segArray[i] = 0;
      }
    }
    else if (start > end)
    {
      if (this._segArray[i] == 0)
      {
        this._prevObstacleFlags[i] = srcNextObstacleFlags[start];
        this._nextObstacleFlags[i] = srcPrevObstacleFlags[start];
        this._segArray[i++] = -srcSegArray[start];
      }
      else
      {
        this._nextObstacleFlags[i] = srcPrevObstacleFlags[start];
        this._segArray[i++] += -srcSegArray[start];
      }

      for (j = start - 1; j > end; j--)
      {
        this._prevObstacleFlags[i] = srcNextObstacleFlags[j];
        this._nextObstacleFlags[i] = srcPrevObstacleFlags[j];
        this._segArray[i++] = -srcSegArray[j];
      }

      if (i < this._segArray.length) {
        this._segArray[i] = 0;
      }
    }

    return (i); // int
  },

  handleFirstCrossing : function(dir, firstCP, secondCP, otherIndexAtFirstCP, otherIndexAtSecondCP, bypass, bypassPoint, i)
  {
    // summary:
    //		Transfer the necessary segments to _segArray of the link to be
    //		filled at the first cross point. 
    // description:
    //		Transfer the necessary segments to &#95;segArray of the link to be
    //		filled at the first cross point. 
    //		It also update &#95;nextObstacleFlags and &#95;prevObstacleFlags. The
    //		otherIndexAt... are the segment indexe of the other link (not to be
    //		filled) at the corresponding cross points. The flag bypass is true
    //		if the link to be filled must bypass the crossing, in which case the
    //		bypassPoint is the corresponding point at the first crossing. The
    //		flag dir is 0 if the actual link is horizontal at the first
    //		crossing, otherwise 1. It returns the first index in &#95;segArray after
    //		the crossing.
    // tags:
    //		private
    
    if (bypass)
    {
      i = this.handleBypassCrossing(dir, firstCP, bypassPoint, i);
    }
    else
    {
      var orthDir = 1 - dir;
      this._prevObstacleFlags[i] = this._oldPrevObstacleFlags[i];
      this._nextObstacleFlags[i] = false;
      this._segArray[i++] = firstCP.getCrossIndex(dir) - firstCP.getCrossContIndex(dir);

      if (otherIndexAtFirstCP == otherIndexAtSecondCP)
      {
        this._prevObstacleFlags[i] = false;
        this._nextObstacleFlags[i] = false;
        this._segArray[i++] = secondCP.getCrossIndex(orthDir) - firstCP.getCrossIndex(orthDir);
      }
      else
      {
        this._prevObstacleFlags[i] = false;
        if (otherIndexAtFirstCP < otherIndexAtSecondCP) {
          this._nextObstacleFlags[i] = this._otherNextObstacleFlags[otherIndexAtFirstCP];
        }
        else {
          this._nextObstacleFlags[i] = this._otherPrevObstacleFlags[otherIndexAtFirstCP];
        }
        this._segArray[i++] = firstCP.getCrossContIndex(orthDir) - firstCP.getCrossIndex(orthDir);
      }

      if (i < this._segArray.length) {
        this._segArray[i] = 0;
      }
    }

    return (i); // int
  },

  handleSecondCrossing : function(dir, secondCP, fillIndexAtSecondCP, otherIndexAtFirstCP, otherIndexAtSecondCP, bypass, bypassPoint, i)
  {
    // summary:
    //		Transfer the necessary segments to _segArray of the link to be
    //		filled at the second cross point. 
    // description:
    //		Transfer the necessary segments to &#95;segArray of the link to be
    //		filled at the second cross point. 
    //		It also update &#95;nextObstacleFlags and &#95;prevObstacleFlags. The
    //		otherIndexAt... are the segment indexe of the other link (not to be
    //		filled) at the corresponding cross points. The flag bypass is true
    //		if the link to be filled must bypass the crossing, in which case the
    //		bypassPoint is the corresponding point at the second crossing. The
    //		flag dir is 0 if the actual link is horizontal at the second
    //		crossing, otherwise 1. It returns the first index in &#95;segArray after
    //		the crossing.
    // tags:
    //		private
    
    if (bypass)
    {
      i = this.handleBypassCrossing(1 - dir, secondCP, bypassPoint, i);
    }
    else
    {
      var orthDir = 1 - dir;

      if (otherIndexAtFirstCP != otherIndexAtSecondCP)
      {
        if (this._segArray[i] == 0)
        {
          if (otherIndexAtFirstCP < otherIndexAtSecondCP) {
            this._prevObstacleFlags[i] = this._otherPrevObstacleFlags[otherIndexAtSecondCP];
          }
          else {
            this._prevObstacleFlags[i] = this._otherNextObstacleFlags[otherIndexAtSecondCP];
          }
        }

        this._nextObstacleFlags[i] = false;
        this._segArray[i++] += secondCP.getCrossIndex(orthDir) - secondCP.getCrossContIndex(orthDir);
      }

      this._prevObstacleFlags[i] = false;
      this._nextObstacleFlags[i] = this._oldNextObstacleFlags[fillIndexAtSecondCP];
      this._segArray[i++] = secondCP.getCrossContIndex(dir) - secondCP.getCrossIndex(dir);
      if (i < this._segArray.length) {
        this._segArray[i] = 0;
      }
    }

    return (i); // int
  },

  handleBypassCrossing : function(dir, crossPoint, bypassPoint, i)
  {
    // summary:
    //		Transfer the necessary segments to _segArray of the link to be
    //		filled if a cross point must be bypassed. 
    // description:
    //		Transfer the necessary segments to &#95;segArray of the link to be
    //		filled if a cross point must be bypassed. 
    //		It also update &#95;nextObstacleFlags and &#95;prevObstacleFlags. The
    //		bypassPoint is the corresponding point at the crossPoint. The flag
    //		dir is 0 if the actual link is horizontal at the second crossing,
    //		otherwise 1. It returns the first index in &#95;segArray after the
    //		crossing.
    // tags:
    //		private
    
    var orthDir = 1 - dir;

    i--;
    this._nextObstacleFlags[i] = true;
    this._segArray[i] += bypassPoint[orthDir] - crossPoint.getCrossIndex(orthDir);
    var line = this.getGrid().getGridLine(dir, bypassPoint[dir]);
    var seg;

    if (line != null)
    {
      seg = line.searchSegment(bypassPoint[orthDir]);

      if (seg != null)
      {
        if (this._segArray[i] < 0 && seg.getStartIndex() < bypassPoint[orthDir]) {
          this._nextObstacleFlags[i] = false;
        }
        else if (this._segArray[i] > 0 && seg.getEndIndex() > bypassPoint[orthDir]) {
          this._nextObstacleFlags[i] = false;
        }
      }
    }

    i++;
    this._prevObstacleFlags[i] = true;
    this._segArray[i] = crossPoint.getCrossIndex(dir) - bypassPoint[dir];
    line = this.getGrid().getGridLine(orthDir, bypassPoint[orthDir]);

    if (line != null)
    {
      seg = line.searchSegment(bypassPoint[dir]);

      if (seg != null)
      {
        if (this._segArray[i] > 0 && seg.getStartIndex() < bypassPoint[dir]) {
          this._prevObstacleFlags[i] = false;
        }
        else if (this._segArray[i] < 0 && seg.getEndIndex() > bypassPoint[dir]) {
          this._prevObstacleFlags[i] = false;
        }
      }
    }

    return (i); // int
  },

  canTerminate : function (a0, a1, a2, a3, a4, a5)
  {
    // summary:
    //		Test whether link can take the input point as termination point if
    //		this reduces a crossing. 
    // description:
    //		Test whether link can take the input point as termination point if
    //		this reduces a crossing. 
    //		We do such crossing reduction only if it
    //		doesn't change the penalty for the termination point.
    // tags:
    //		private
    
    if(arguments.length == 3)
      return this.canTerminate_overload_3(a0, a1, a2);
    else if(arguments.length == 6)
      return this.canTerminate_overload_6(a0, a1, a2, a3, a4, a5);
  },
  
  canTerminate_overload_3 : function(link, p, fromSide)
                                                      
  {
    return (this.canTerminate(link, p, fromSide, p.getIndex(0), p.getIndex(1), 0)); // Boolean
  },

  canTerminate_overload_6 : function(link, p, fromSide, coord0, coord1, dir)
  {
    // summary:
    //		Test whether link can take the point (x, y) which is neighbored to
    //		the input point p as termination point if this reduces a crossing. 
    // description:
    //		Test whether link can take the point (x, y) which is neighbored to
    //		the input point p as termination point if this reduces a crossing. 
    //		We do such crossing reduction only if it
    //		doesn't change the penalty for the termination point. The flag dir is 0 if coord0 is x-coordinate, otherwise it is 1.
    // tags:
    //		private
    
    var grid = this.getGrid();
    var penalty = ((fromSide ? link.getActStartPoint().getPenalty() : link.getActEndPoint().getPenalty()));

    if (fromSide)
    {
      if (link.getFromPoint() != null) {
        return (false); // Boolean
      }
    }
    else
    {
      if (link.getToPoint() != null) {
        return (false); // Boolean
      }
    }

    var x;
    var y;

    if (dir == 0)
    {
      x = grid.getGridCoord(0, coord0);
      y = grid.getGridCoord(1, coord1);
    }
    else
    {
      x = grid.getGridCoord(0, coord1);
      y = grid.getGridCoord(1, coord0);
    }

    return (this.getLayout().getTerminationPointPenalty(link.getLinkObject(), ((fromSide ? link.getFromNode() : link.getToNode())), fromSide, x, y, p.getSide(), p.getPenalty()) == penalty); // Boolean
  },

  findSingleReducableCrossing : function(sameDir)
  {
    // summary:
    //		Find a single crossing where the link flow are as specified in
    //		sameDir.
    // tags:
    //		private
    
    var p1 = this._listOfCrossPoints;

    while (p1 != null)
    {
      if (p1.getType() != -1)
      {
        if (p1.isOppositeFlow() == !sameDir)
        {
          this._crossing1 = p1;
          return (true); // Boolean
        }
      }

      p1 = p1.getNext();
    }

    return (false); // Boolean
  },

  reduceCrossingByTerminationExchange : function(link1, link2, exchangeAtLink1Start, exchangeAtLink2Start)
  {
    // summary:
    //		Eliminate one crossing by exchanging the termination points of both
    //		links at one side and reducing one crossing. 
    // description:
    //		Eliminate one crossing by exchanging the termination points of both
    //		links at one side and reducing one crossing. 
    //		The flag exchangeAtLink1Start is true if the termination point of
    //		link1 at its "from" side must be exchanged with the termination
    //		point of link2. The flag exchangeAtLink2Start is true if the
    //		termination point of link2 at its "from" side must be exchanged with
    //		the termination point of link1.
    // tags:
    //		private
    
    if (this._crossing1.getType() == -1) {
      return;
    }

    var sameDir = (exchangeAtLink1Start == exchangeAtLink2Start);

    var oldLink1Segments = link1.getSegments();
    var oldLink2Segments = link2.getSegments();

    this._crossing1.calcBypassPoint(this._bypassPoint1);

    var segmentIndexLink1Crossing = this._crossing1.getCrossSegIndexA();
    var segmentIndexLink2Crossing = this._crossing1.getCrossSegIndexB();

    var link1BypassAtCrossing = this._crossing1.needBypassIfFirst(link1, segmentIndexLink1Crossing);

    if (exchangeAtLink1Start) {
      link1BypassAtCrossing = !link1BypassAtCrossing;
    }

    var segsOfLink1;
    var segsOfLink2;

    if (exchangeAtLink1Start)
    {
      segsOfLink2 = segmentIndexLink1Crossing;
      segsOfLink1 = oldLink1Segments.length - segmentIndexLink1Crossing - 1;
    }
    else
    {
      segsOfLink1 = segmentIndexLink1Crossing;
      segsOfLink2 = oldLink1Segments.length - segmentIndexLink1Crossing - 1;
    }

    if (exchangeAtLink2Start)
    {
      segsOfLink1 += segmentIndexLink2Crossing;
      segsOfLink2 += oldLink2Segments.length - segmentIndexLink2Crossing - 1;
    }
    else
    {
      segsOfLink2 += segmentIndexLink2Crossing;
      segsOfLink1 += oldLink2Segments.length - segmentIndexLink2Crossing - 1;
    }

    if (!link1BypassAtCrossing) {
      segsOfLink1 += 2;
    }
    var newLink1Segments = TranslateUtil.newArray(segsOfLink1, 0);
    var newLink1PrevObstacleFlags = TranslateUtil.newArray(segsOfLink1, false);
    var newLink1NextObstacleFlags = TranslateUtil.newArray(segsOfLink1, false);

    if (link1BypassAtCrossing) {
      segsOfLink2 += 2;
    }
    var newLink2Segments = TranslateUtil.newArray(segsOfLink2, 0);
    var newLink2PrevObstacleFlags = TranslateUtil.newArray(segsOfLink2, false);
    var newLink2NextObstacleFlags = TranslateUtil.newArray(segsOfLink2, false);

    var segIndex;
    var index;
    var limits = TranslateUtil.newArray(2, 0);
    var grid = this.getGrid();
    index = this._crossing1.getHorizontalFreeSegment(limits);
    segIndex = this._crossing1.getCrossSegIndexA();
    grid.removeObstacle(0, index, limits, link1, segIndex);
    segIndex = this._crossing1.getCrossSegIndexB();
    grid.removeObstacle(0, index, limits, link2, segIndex);
    index = this._crossing1.getVerticalFreeSegment(limits);
    segIndex = this._crossing1.getCrossSegIndexA();
    grid.removeObstacle(1, index, limits, link1, segIndex);
    segIndex = this._crossing1.getCrossSegIndexB();
    grid.removeObstacle(1, index, limits, link2, segIndex);

    this.fillSegmentsForTerminationExchange(newLink1Segments, newLink1PrevObstacleFlags, newLink1NextObstacleFlags, link1, link2, segmentIndexLink1Crossing, segmentIndexLink2Crossing,
        link1BypassAtCrossing, exchangeAtLink1Start, sameDir);

    this.fillSegmentsForTerminationExchange(newLink2Segments, newLink2PrevObstacleFlags, newLink2NextObstacleFlags, link2, link1, segmentIndexLink2Crossing, segmentIndexLink1Crossing,
        !link1BypassAtCrossing, exchangeAtLink2Start, sameDir);

    var link1Point;
    var link2Point;
    var dummyPoint = new LLTerminationPoint(0, 0, 0, 0, 0, 0, 0, ibm_ilog.graphlayout.Direction.NONE, null);
    var gridCorrX = link1.getGridCorrection(0) - link2.getGridCorrection(0);
    var gridCorrY = link1.getGridCorrection(1) - link2.getGridCorrection(1);
    var link1StartDir = link1.getSegmentStartDir();
    var link2StartDir = link2.getSegmentStartDir();

    if (exchangeAtLink1Start)
    {
      link1Point = link1.getActStartPoint();
      if (sameDir) {
        link1StartDir = link2.getSegmentStartDir();
      }
      else {
        link1StartDir = link2.getSegmentEndDir();
      }
    }
    else {
      link1Point = link1.getActEndPoint();
    }
    if (exchangeAtLink2Start)
    {
      link2Point = link2.getActStartPoint();
      if (sameDir) {
        link2StartDir = link1.getSegmentStartDir();
      }
      else {
        link2StartDir = link1.getSegmentEndDir();
      }
    }
    else {
      link2Point = link2.getActEndPoint();
    }
    dummyPoint.copyData(link1Point, 0, 0);
    link1Point.copyData(link2Point, -gridCorrX, -gridCorrY);
    link2Point.copyData(dummyPoint, gridCorrX, gridCorrY);
    dummyPoint.dispose();

    link1.setSegments(newLink1Segments, newLink1PrevObstacleFlags, newLink1NextObstacleFlags);
    link2.setSegments(newLink2Segments, newLink2PrevObstacleFlags, newLink2NextObstacleFlags);

    link1.setSegmentStartDir(link1StartDir);
    link2.setSegmentStartDir(link2StartDir);

    index = this._crossing1.getHorizontalTestSegment(limits);
    grid.setGridLineObstacle(true, index, limits);
    index = this._crossing1.getVerticalTestSegment(limits);
    grid.setGridLineObstacle(false, index, limits);

    this._crossing1.makeUnfeasable();
  },

  fillSegmentsForTerminationExchange : function(segArray, prevObstacleFlags, nextObstacleFlags, fillLink, otherLink, fillSegIndexAtCrossing, otherSegIndexAtCrossing, bypassC, exchangeAtStart,
    bothLinkSameDir)
  {
    // summary:
    //		Fills the new segment array of fillLink to elimitate the crossings.
    // tags:
    //		private
    
    var i;

    this._segArray = segArray;
    this._prevObstacleFlags = prevObstacleFlags;
    this._nextObstacleFlags = nextObstacleFlags;

    var oldSegArray = fillLink.getSegments();
    var otherSegArray = otherLink.getSegments();
    this._oldPrevObstacleFlags = fillLink.getPrevObstacleFlags();
    this._oldNextObstacleFlags = fillLink.getNextObstacleFlags();
    this._otherPrevObstacleFlags = otherLink.getPrevObstacleFlags();
    this._otherNextObstacleFlags = otherLink.getNextObstacleFlags();

    segArray[0] = 0;

    if (exchangeAtStart)
    {
      if (bothLinkSameDir) {
        i = this.transferSegments(otherSegArray, this._otherPrevObstacleFlags, this._otherNextObstacleFlags, 0, 0, otherSegIndexAtCrossing);
      }
      else {
        i = this.transferSegments(otherSegArray, this._otherPrevObstacleFlags, this._otherNextObstacleFlags, 0, otherSegArray.length - 1, otherSegIndexAtCrossing);
      }
    }
    else
    {
      i = this.transferSegments(oldSegArray, this._oldPrevObstacleFlags, this._oldNextObstacleFlags, 0, 0, fillSegIndexAtCrossing);
    }

    var delta = ((bothLinkSameDir ? 1 : -1));

    if (exchangeAtStart)
    {
      i =
        this.handleSecondCrossing(fillLink.getSegmentDir(fillSegIndexAtCrossing), this._crossing1, fillSegIndexAtCrossing, otherSegIndexAtCrossing - delta, otherSegIndexAtCrossing, bypassC,
            this._bypassPoint1, i);
    }
    else
    {
      i = this.handleFirstCrossing(fillLink.getSegmentDir(fillSegIndexAtCrossing), this._crossing1, null, otherSegIndexAtCrossing, otherSegIndexAtCrossing + delta, bypassC, this._bypassPoint1, i);
    }

    if (exchangeAtStart)
    {
      i = this.transferSegments(oldSegArray, this._oldPrevObstacleFlags, this._oldNextObstacleFlags, i, fillSegIndexAtCrossing + 1, oldSegArray.length);
    }
    else
    {
      if (bothLinkSameDir) {
        i = this.transferSegments(otherSegArray, this._otherPrevObstacleFlags, this._otherNextObstacleFlags, i, otherSegIndexAtCrossing + 1, otherSegArray.length);
      }
      else {
        i = this.transferSegments(otherSegArray, this._otherPrevObstacleFlags, this._otherNextObstacleFlags, i, otherSegIndexAtCrossing - 1, -1);
      }
    }

    prevObstacleFlags[0] = true;
    nextObstacleFlags[nextObstacleFlags.length - 1] = true;
  },

  calcCrossings : function(link1, link2)
  {
    // summary:
    //		Find all reducable crossings between two links. 
    // description:
    //		Find all reducable crossings between two links. 
    //		It returns the number of crossings found.
    // tags:
    //		private
    
    var numberOfCrossings = 0;
    if (this._listOfCrossPoints != null) {
      this._listOfCrossPoints.dispose();
    }
    this._listOfCrossPoints = null;
    var segmentA1A2Index = 0;
    var distToA1 = 0;
    var point = TranslateUtil.newArray(2, 0);
    var pA1 = TranslateUtil.newArray(2, 0);
    var pA2 = TranslateUtil.newArray(2, 0);
    var pA3 = TranslateUtil.newArray(2, 0);
    var i = 2;
    var len;
    link1.getNextBend(point, 0);
    pA1[0] = point[0];
    pA1[1] = point[1];
    link1.getNextBend(point, 1);
    pA2[0] = point[0];
    pA2[1] = point[1];

    while (link1.getNextBend(point, i++))
    {
      pA3[0] = point[0];
      pA3[1] = point[1];
      numberOfCrossings += this.calcSegmentCrossings(link1, segmentA1A2Index, distToA1, pA1, pA2, pA3, link2);
      segmentA1A2Index++;
      len = pA1[0] - pA2[0] + pA1[1] - pA2[1];
      if (len < 0) {
        len = -len;
      }
      distToA1 += len;
      pA1[0] = pA2[0];
      pA1[1] = pA2[1];
      pA2[0] = pA3[0];
      pA2[1] = pA3[1];
    }

    return (numberOfCrossings); // int
  },

  calcSegmentCrossings : function(link1, segmentA1A2Index, distToA1, pA1, pA2, pA3, link2)
  {
    // summary:
    //		Finds all reducable crossings between the segment chain pA1-pA2-pA3
    //		of link1 and link2. 
    // description:
    //		Finds all reducable crossings between the segment chain pA1-pA2-pA3
    //		of link1 and link2. 
    //		It returns the number of crossings found.
    // tags:
    //		private
    
    var numberOfCrossings = 0;
    var segmentB1B2Index = 0;
    var distToB1 = 0;
    var point = TranslateUtil.newArray(2, 0);
    var pB1 = TranslateUtil.newArray(2, 0);
    var pB2 = TranslateUtil.newArray(2, 0);
    var pB3 = TranslateUtil.newArray(2, 0);
    var i = 2;
    var len;
    var crossPoint = new LLCrossPoint();
    var okay;
    var testIndex;
    var testLimits = TranslateUtil.newArray(2, 0);
    var grid = this.getGrid();
    link2.getNextBend(point, 0);
    pB1[0] = point[0];
    pB1[1] = point[1];
    link2.getNextBend(point, 1);
    pB2[0] = point[0];
    pB2[1] = point[1];

    while (link2.getNextBend(point, i++))
    {
      pB3[0] = point[0];
      pB3[1] = point[1];
      okay = false;

      if (crossPoint.checkCrossing(segmentA1A2Index, distToA1, pA1, pA2, pA3, segmentB1B2Index, distToB1, pB1, pB2, pB3))
      {
        okay = true;

        if (okay)
        {
          testIndex = crossPoint.getHorizontalTestSegment(testLimits);
          okay = grid.isWithoutObstacles(0, testIndex, testLimits);
        }

        if (okay)
        {
          testIndex = crossPoint.getVerticalTestSegment(testLimits);
          okay = grid.isWithoutObstacles(1, testIndex, testLimits);
        }

        if (okay)
        {
          testIndex = crossPoint.getHorizontalFreeSegment(testLimits);
          if (this.isForbidden(testIndex, testLimits, 0, link1) || this.isForbidden(testIndex, testLimits, 0, link2)) {
            okay = false;
          }
        }

        if (okay)
        {
          testIndex = crossPoint.getVerticalFreeSegment(testLimits);
          if (this.isForbidden(testIndex, testLimits, 1, link1) || this.isForbidden(testIndex, testLimits, 1, link2)) {
            okay = false;
          }
        }
      }

      if (okay)
      {
        numberOfCrossings++;
        crossPoint.setNext(this._listOfCrossPoints);
        this._listOfCrossPoints = crossPoint;
        crossPoint = new LLCrossPoint();
      }

      segmentB1B2Index++;
      len = pB1[0] - pB2[0] + pB1[1] - pB2[1];
      if (len < 0) {
        len = -len;
      }
      distToB1 += len;
      pB1[0] = pB2[0];
      pB1[1] = pB2[1];
      pB2[0] = pB3[0];
      pB2[1] = pB3[1];
    }

    crossPoint.dispose();
    return (numberOfCrossings); // int
  },

  isForbidden : function(i, limits, dir, link)
  {
    // summary:
    //		This method returns true if the input segment at grid line i in
    //		range given by limits touches or intersects the forbidden area of
    //		the link. 
    // description:
    //		This method returns true if the input segment at grid line i in
    //		range given by limits touches or intersects the forbidden area of
    //		the link. 
    //		The flag dir is 0 if the input segment is horizontal, otherwise 1.
    // tags:
    //		private
    
    var c = link.getGridCorrection(dir);

    var point = link.getActStartPoint();
    var a = 0;
    var b = 0;

    if (link.getSegmentStartDir() == dir)
    {
      if (point.getIndex(1 - dir) + link.getGridCorrection(1 - dir) == i)
      {
        a = point.getForbiddenMinIndex() + c - 1;
        b = point.getForbiddenMaxIndex() + c + 1;
        if (limits[0] <= b && limits[1] >= a) {
          return (true); // Boolean
        }
      }
    }

    point = link.getActEndPoint();

    if (link.getSegmentEndDir() == dir)
    {
      if (point.getIndex(1 - dir) + link.getGridCorrection(1 - dir) == i)
      {
        a = point.getForbiddenMinIndex() + c - 1;
        b = point.getForbiddenMaxIndex() + c + 1;
        if (limits[0] <= b && limits[1] >= a) {
          return (true); // Boolean
        }
      }
    }

    return (false); // Boolean
  },

  _straightLink: null,
  _bendLink: null,

  _exchangeAtStraightLinkStart: null,

  _exchangeAtBendLinkStart: null,

  _moveStraightToLower: null,

  _sp1 : null,
  _sp2 : null,
  _bp0 : null,
  _bp1 : null,
  _bp2 : null,

  _slimits : null,

  _blimits : null,

  _sgridCorr : null,
  _bgridCorr : null,

  isStraightTerminationCrossing : function(link1, link2)
  {
    // summary:
    //		Returns true if it is a reducable crossing of two links terminating
    //		at the same node, for which one link is straight. 
    // description:
    //		Returns true if it is a reducable crossing of two links terminating
    //		at the same node, for which one link is straight. 
    //		The crossing is only reducable if the links are movable.
    // tags:
    //		private
    
    if (link1.getSegments().length == 1)
    {
      if (link2.getSegments().length > 1)
      {
        this._straightLink = link1;
        this._bendLink = link2;
      }
      else {
        return (false); // Boolean
      }
    }
    else if (link2.getSegments().length == 1)
    {
      if (link1.getSegments().length > 1)
      {
        this._straightLink = link2;
        this._bendLink = link1;
      }
      else {
        return (false); // Boolean
      }
    }
    else {
      return (false); // Boolean
    }
    if (this._straightLink.getFromNode() == this._bendLink.getFromNode())
    {
      this._exchangeAtStraightLinkStart = true;
      this._exchangeAtBendLinkStart = true;
    }
    else if (this._straightLink.getToNode() == this._bendLink.getFromNode())
    {
      this._exchangeAtStraightLinkStart = false;
      this._exchangeAtBendLinkStart = true;
    }
    else if (this._straightLink.getFromNode() == this._bendLink.getToNode())
    {
      this._exchangeAtStraightLinkStart = true;
      this._exchangeAtBendLinkStart = false;
    }
    else if (this._straightLink.getToNode() == this._bendLink.getToNode())
    {
      this._exchangeAtStraightLinkStart = false;
      this._exchangeAtBendLinkStart = false;
    }
    else {
      return (false); // Boolean
    }

    var straightDir = this._straightLink.getSegmentStartDir();
    this._straightLink.getNextBend(this._sp1, 0);
    this._sp2[0] = this._sp1[0];
    this._sp2[1] = this._sp1[1];
    this._straightLink.getNextBend(this._sp2, 1);

    if (this._exchangeAtBendLinkStart)
    {
      if (straightDir == 1 - this._bendLink.getSegmentStartDir()) {
        return (false); // Boolean
      }
      this._bendLink.getNextBend(this._bp0, 0);
      this._bp1[0] = this._bp0[0];
      this._bp1[1] = this._bp0[1];
      this._bendLink.getNextBend(this._bp1, 1);
      this._bp2[0] = this._bp1[0];
      this._bp2[1] = this._bp1[1];
      this._bendLink.getNextBend(this._bp2, 2);
    }
    else
    {
      if (straightDir == 1 - this._bendLink.getSegmentEndDir()) {
        return (false); // Boolean
      }
      var numSegs = this._bendLink.getSegments().length;
      this._bendLink.getPrevBend(this._bp0, numSegs);
      this._bp1[0] = this._bp0[0];
      this._bp1[1] = this._bp0[1];
      this._bendLink.getPrevBend(this._bp1, numSegs - 1);
      this._bp2[0] = this._bp1[0];
      this._bp2[1] = this._bp1[1];
      this._bendLink.getPrevBend(this._bp2, numSegs - 2);
    }

    var bendDir = 1 - straightDir;

    if (this._bp1[straightDir] <= Math.min(this._sp1[straightDir], this._sp2[straightDir])) {
      return (false); // Boolean
    }
    if (this._bp1[straightDir] >= Math.max(this._sp1[straightDir], this._sp2[straightDir])) {
      return (false); // Boolean
    }
    if (this._sp1[bendDir] <= Math.min(this._bp1[bendDir], this._bp2[bendDir])) {
      return (false); // Boolean
    }
    if (this._sp1[bendDir] >= Math.max(this._bp1[bendDir], this._bp2[bendDir])) {
      return (false); // Boolean
    }
    this._moveStraightToLower = (this._bp1[bendDir] < this._bp2[bendDir]);

    this._sgridCorr[0] = this._straightLink.getGridCorrection(0);
    this._sgridCorr[1] = this._straightLink.getGridCorrection(1);
    this._bgridCorr[0] = this._bendLink.getGridCorrection(0);
    this._bgridCorr[1] = this._bendLink.getGridCorrection(1);

    this._slimits[0] = this._straightLink.getActStartPoint().getIndex(straightDir);
    this._slimits[1] = this._straightLink.getActEndPoint().getIndex(straightDir);
    this._slimits[0] += this._sgridCorr[straightDir];
    this._slimits[1] += this._sgridCorr[straightDir];

    if (this._exchangeAtBendLinkStart) {
      this._blimits[0] = this._bendLink.getActStartPoint().getIndex(straightDir);
    }
    else {
      this._blimits[0] = this._bendLink.getActEndPoint().getIndex(straightDir);
    }
    this._blimits[0] += this._bgridCorr[straightDir];
    this._blimits[1] = this._bp1[straightDir];
    return (true); // Boolean
  },

  reduceStraightTerminationCrossing : function()
  {
    // summary:
    //		Reduces a straight termination crossing. 
    // description:
    //		Reduces a straight termination crossing. 
    //		Returns true on success.
    // tags:
    //		private
    
    var numCrossings;
    var bestNumCrossings = /*Number.MAX_VALUE*/2147483647;
    var bestChoice = -1;
    var testGridIndexMoveStraight = TranslateUtil.newArray(1, 0);
    var testGridIndexMoveBend = TranslateUtil.newArray(1, 0);

    numCrossings = this.checkExchangeBendAndStraight();

    if (numCrossings < bestNumCrossings)
    {
      bestChoice = 0;
      bestNumCrossings = numCrossings;
    }

    numCrossings = this.checkMoveStraight(testGridIndexMoveStraight);

    if (numCrossings < bestNumCrossings)
    {
      bestChoice = 1;
      bestNumCrossings = numCrossings;
    }

    numCrossings = this.checkMoveBend(testGridIndexMoveBend);

    if (numCrossings < bestNumCrossings)
    {
      bestChoice = 2;
    }

    switch (bestChoice)
    {
      case 0:
        this.exchangeBendAndStraightLink();
        break;
      case 1:
        this.moveStraightLink(testGridIndexMoveStraight[0]);
        break;
      case 2:
        this.moveBendLink(testGridIndexMoveBend[0]);
        break;
    }
    return (bestNumCrossings < /*Number.MAX_VALUE*/2147483647); // Boolean
  },

  calcStraightBendTerminationNumberOfCrossings : function(sTermIndex, bTermIndex, bendSegIndex, bendSegLimit1, bendSegLimit2)
  {
    // summary:
    //		Calculates an approximate number of crossings of a straight link and
    //		a bend link in a specific situation. 
    // description:
    //		Calculates an approximate number of crossings of a straight link and
    //		a bend link in a specific situation. 
    //		If the links cross each other, then we count the crossing
    //		automatically twice. We also count the crossing with the own link,
    //		if the link is considered be moved (but
    //		isn't yet updated in the grid). So it is really only a coarse estimation. But good enough for the reduction heuristic. s2 | |       b3------------b2 |                     | s1                    b1 slimits specify the range from s1 to s2. blimits specify the range from b1 to b2. bendSegLimit1 and bendSegLimit2 specify the range from b2 to b3. sTermIndex is the index of segment (s1, s2) is bend direction. bTermIndex is the index of segment (b1, s2) is bend direction. bendSegIndex is the index of b2 and b3 is straight direction.
    // tags:
    //		private
    
    var grid = this.getGrid();
    var line;
    var straightDir = this._straightLink.getSegmentStartDir();
    var bendDir = 1 - straightDir;
    var numCrossings = 0;

    line = grid.getGridLine(bendDir, sTermIndex);
    if (line != null) {
      numCrossings += line.calcApproxCrossings(grid, this._slimits[0], this._slimits[1]);
    }
    line = grid.getGridLine(bendDir, bTermIndex);
    if (line != null) {
      numCrossings += line.calcApproxCrossings(grid, this._blimits[0], this._blimits[1]);
    }
    line = grid.getGridLine(straightDir, bendSegIndex);
    if (line != null) {
      numCrossings += line.calcApproxCrossings(grid, bendSegLimit1, bendSegLimit2);
    }
    return (numCrossings); // int
  },

  checkMoveStraight : function(gridIndexResult)
  {
    // summary:
    //		Check whether it is feasable to move the straight link to the other
    //		side of the bend link. 
    // description:
    //		Check whether it is feasable to move the straight link to the other
    //		side of the bend link. 
    //		It returns MAX&#95;VALUE if it is not feasable, otherwise it returns the
    //		best number of crossings when doing so. It fills the array field
    //		gridIndexResult[0] with the grid point in bendDir where the straight
    //		segment should be placed.
    // tags:
    //		private
    
    var straightDir = this._straightLink.getSegmentStartDir();
    var bendDir = 1 - straightDir;
    var actIndex = this._sp1[bendDir];
    var testIndex;
    var currentNumberCrossings = -1;
    var grid = this.getGrid();
    var startRect = this._straightLink.getFromRect();
    var endRect = this._straightLink.getToRect();
    var actStartPoint = this._straightLink.getActStartPoint();
    var actEndPoint = this._straightLink.getActEndPoint();
    var bestDeviation = 0.0;
    var bestNumberOfCrossings = /*Number.MAX_VALUE*/2147483647;
    var bestTPBDir = 0;
    var testNumberCrossings;
    var testDeviation;
    var minCOffs = this.getLayout().getMinNodeCornerOffset();
    var startIndex = Math.max(grid.getMinGridIndexAtSide(startRect, bendDir, minCOffs), grid.getMinGridIndexAtSide(endRect, bendDir, minCOffs));
    var endIndex = Math.min(grid.getMaxGridIndexAtSide(startRect, bendDir, minCOffs), grid.getMaxGridIndexAtSide(endRect, bendDir, minCOffs));
    var actStartLevelIndex = actStartPoint.getIndex(straightDir);
    var actEndLevelIndex = actEndPoint.getIndex(straightDir);

    for (testIndex = startIndex; testIndex <= endIndex; testIndex++)
    {
      if ((testIndex < actIndex && this._moveStraightToLower) || (testIndex > actIndex && !this._moveStraightToLower))
      {
        if (
          this._straightLink.isWithinFromNodeRange(bendDir, testIndex, minCOffs) && this._straightLink.isWithinToNodeRange(bendDir, testIndex, minCOffs) &&
            this.canTerminate(this._straightLink, actStartPoint, true, testIndex, actStartLevelIndex, bendDir) &&
            this.canTerminate(this._straightLink, actEndPoint, false, testIndex, actEndLevelIndex, bendDir))
          {
            if (grid.isWithoutObstacles(straightDir, testIndex, this._slimits))
            {
              if (currentNumberCrossings == -1) {
                currentNumberCrossings = this.calcStraightBendTerminationNumberOfCrossings(actIndex, this._bp0[bendDir], this._bp1[straightDir], this._bp1[bendDir], this._bp2[bendDir]);
              }
              testNumberCrossings = this.calcStraightBendTerminationNumberOfCrossings(testIndex, this._bp0[bendDir], this._bp1[straightDir], this._bp1[bendDir], this._bp2[bendDir]);

              testDeviation = Math.max(LLTerminationPoint.getDeviation(grid, startRect, testIndex, bendDir), LLTerminationPoint.getDeviation(grid, endRect, testIndex, bendDir));

              if (testNumberCrossings < currentNumberCrossings && (testNumberCrossings < bestNumberOfCrossings || (testNumberCrossings == bestNumberOfCrossings && testDeviation < bestDeviation)))
              {
                bestNumberOfCrossings = testNumberCrossings;
                bestDeviation = testDeviation;
                bestTPBDir = testIndex;
              }
            }
          }
      }
    }

    gridIndexResult[0] = bestTPBDir;
    return (bestNumberOfCrossings); // int
  },

  checkMoveBend : function(gridIndexResult)
  {
    // summary:
    //		Check whether it is feasable to move the bend link to the other side
    //		of the straight link. 
    // description:
    //		Check whether it is feasable to move the bend link to the other side
    //		of the straight link. 
    //		It returns MAX&#95;VALUE if it is not feasable, otherwise it returns the
    //		best number of crossings when doing so. It fills the array field
    //		gridIndexResult[0] with the grid point in bendDir where the bend
    //		segment should be placed.
    // tags:
    //		private
    
    var straightDir = this._straightLink.getSegmentStartDir();
    var bendDir = 1 - straightDir;
    var actIndex = this._bp0[bendDir];
    var testIndex;
    var currentNumberCrossings = -1;
    var rect;
    var grid = this.getGrid();
    var actPoint;

    if (this._exchangeAtBendLinkStart)
    {
      actPoint = this._bendLink.getActStartPoint();
      rect = this._bendLink.getFromRect();
    }
    else
    {
      actPoint = this._bendLink.getActEndPoint();
      rect = this._bendLink.getToRect();
    }

    var bestDeviation = 0.0;
    var bestNumberOfCrossings = /*Number.MAX_VALUE*/2147483647;
    var bestTPBDir = 0;
    var testNumberCrossings;
    var testDeviation;
    var aux = TranslateUtil.newArray(2, 0);
    var actLevelIndex = actPoint.getIndex(straightDir);
    var minCOffs = this.getLayout().getMinNodeCornerOffset();
    var startIndex = Math.max(grid.getMinGridIndexAtSide(rect, bendDir, minCOffs), grid.getMinGridIndexAtSide(rect, bendDir, minCOffs));
    var endIndex = Math.min(grid.getMaxGridIndexAtSide(rect, bendDir, minCOffs), grid.getMaxGridIndexAtSide(rect, bendDir, minCOffs));

    for (testIndex = startIndex; testIndex <= endIndex; testIndex++)
    {
      if ((this._moveStraightToLower && actIndex < testIndex && testIndex < this._bp2[bendDir]) || (!this._moveStraightToLower && actIndex > testIndex && testIndex > this._bp2[bendDir]))
      {
        if (
          this._bendLink.isWithinRectRange(rect, bendDir, testIndex, minCOffs) && this.canTerminate(this._bendLink, actPoint, this._exchangeAtBendLinkStart, testIndex, actLevelIndex, bendDir) &&
            grid.isWithoutObstacles(straightDir, testIndex, this._blimits))
          {
            if (testIndex < actIndex)
            {
              aux[0] = testIndex + 1;
              aux[1] = actIndex;
            }
            else
            {
              aux[0] = actIndex;
              aux[1] = testIndex - 1;
            }

            if (!this.isForbidden(this._bp1[straightDir], aux, bendDir, this._bendLink))
            {
              if (currentNumberCrossings == -1) {
                currentNumberCrossings = this.calcStraightBendTerminationNumberOfCrossings(this._sp1[bendDir], actIndex, this._bp1[straightDir], this._bp1[bendDir], this._bp2[bendDir]);
              }
              testNumberCrossings = this.calcStraightBendTerminationNumberOfCrossings(this._sp1[bendDir], testIndex, this._bp1[straightDir], testIndex, this._bp2[bendDir]);

              testDeviation = LLTerminationPoint.getDeviation(grid, rect, testIndex, bendDir);

              if (testNumberCrossings < currentNumberCrossings && (testNumberCrossings < bestNumberOfCrossings || (testNumberCrossings == bestNumberOfCrossings && testDeviation < bestDeviation)))
              {
                bestNumberOfCrossings = testNumberCrossings;
                bestDeviation = testDeviation;
                bestTPBDir = testIndex;
              }
            }
          }
      }
    }

    gridIndexResult[0] = bestTPBDir;
    return (bestNumberOfCrossings); // int
  },

  checkExchangeBendAndStraight : function()
  {
    // summary:
    //		Check whether it is feasable to exchange the bend link with the
    //		straight link. 
    // description:
    //		Check whether it is feasable to exchange the bend link with the
    //		straight link. 
    //		It returns MAX&#95;VALUE if it is not feasable, otherwise it returns the
    //		number of crossings when doing so.
    // tags:
    //		private
    
    var straightDir = this._straightLink.getSegmentStartDir();
    var bendDir = 1 - straightDir;

    var TPBDir = this._bp0[bendDir];
    var TPSDir;
    var minCOffs = this.getLayout().getMinNodeCornerOffset();

    if (this._exchangeAtStraightLinkStart)
    {
      TPSDir = this._slimits[1];
      if (!this._straightLink.isWithinToNodeRange(bendDir, TPBDir, minCOffs)) {
        return (/*Number.MAX_VALUE*/2147483647); // int
      }
      if (!this.canTerminate(this._bendLink, this._straightLink.getActStartPoint(), this._exchangeAtBendLinkStart)) {
        return (/*Number.MAX_VALUE*/2147483647); // int
      }
    }
    else
    {
      TPSDir = this._slimits[0];
      if (!this._straightLink.isWithinFromNodeRange(bendDir, TPBDir, minCOffs)) {
        return (/*Number.MAX_VALUE*/2147483647); // int
      }
      if (!this.canTerminate(this._bendLink, this._straightLink.getActEndPoint(), this._exchangeAtBendLinkStart)) {
        return (/*Number.MAX_VALUE*/2147483647); // int
      }
    }

    if (!this.canTerminate(this._straightLink, this._straightLink.getActStartPoint(), true, TPBDir, this._slimits[0], bendDir)) {
      return (/*Number.MAX_VALUE*/2147483647); // int
    }
    if (!this.canTerminate(this._straightLink, this._straightLink.getActEndPoint(), false, TPBDir, this._slimits[1], bendDir)) {
      return (/*Number.MAX_VALUE*/2147483647); // int
    }

    var grid = this.getGrid();
    var aux = TranslateUtil.newArray(2, 0);

    if (this._bp0[straightDir] < this._bp1[straightDir])
    {
      aux[1] = this._bp1[straightDir] + 1;
      aux[0] = Math.max(aux[1], TPSDir);
    }
    else
    {
      aux[1] = this._bp1[straightDir] - 1;
      aux[0] = Math.min(aux[1], TPSDir);
    }

    if (!grid.isWithoutObstacles(straightDir, TPBDir, aux)) {
      return (/*Number.MAX_VALUE*/2147483647); // int
    }
    if (this._sp1[bendDir] < this._bp0[bendDir])
    {
      aux[0] = this._sp1[bendDir] + 1;
      aux[1] = this._bp0[bendDir];
    }
    else
    {
      aux[0] = this._bp0[bendDir];
      aux[1] = this._sp1[bendDir] - 1;
    }

    if (this.isForbidden(this._bp1[straightDir], aux, bendDir, this._bendLink)) {
      return (/*Number.MAX_VALUE*/2147483647); // int
    }

    var numCrossings = this.calcStraightBendTerminationNumberOfCrossings(this._bp1[bendDir], this._sp1[bendDir], this._bp1[straightDir], this._sp1[bendDir], this._bp2[bendDir]);
    return (numCrossings); // int
  },

  moveStraightLink : function(TPBDir)
  {
    // summary:
    //		Move the straight link such that it is placed in bendDir at TPBDir.
    // tags:
    //		private
    
    var grid = this.getGrid();
    var straightDir = this._straightLink.getSegmentStartDir();
    var bendDir = 1 - straightDir;
    var limits = TranslateUtil.newArray(2, 0);
    limits[0] = this._sp1[straightDir];
    limits[1] = this._sp2[straightDir];
    grid.removeObstacle(straightDir, this._sp1[bendDir], limits, this._straightLink, 0);

    var TPBDirCorr = TPBDir - this._sgridCorr[bendDir];
    this._straightLink.getActStartPoint().setIndex(bendDir, TPBDirCorr);
    this._straightLink.getActEndPoint().setIndex(bendDir, TPBDirCorr);
    this._straightLink.updateGrid();
    grid.setRectObstacle(this._straightLink.getFromRect());
    grid.setRectObstacle(this._straightLink.getToRect());
  },

  moveBendLink : function(TPBDir)
  {
    // summary:
    //		Move the bend link such that it uses the termination point with
    //		input index.
    // tags:
    //		private
    
    var bendPoint;
    var obstacleFlags;
    var bSegIdx0;
    var bSegIdx1;
    var sign;

    if (this._exchangeAtBendLinkStart)
    {
      bendPoint = this._bendLink.getActStartPoint();
      bSegIdx0 = 0;
      bSegIdx1 = 1;
      obstacleFlags = this._bendLink.getPrevObstacleFlags();
      sign = 1;
    }
    else
    {
      bendPoint = this._bendLink.getActEndPoint();
      bSegIdx0 = this._bendLink.getSegments().length - 1;
      bSegIdx1 = this._bendLink.getSegments().length - 2;
      obstacleFlags = this._bendLink.getNextObstacleFlags();
      sign = -1;
    }

    var grid = this.getGrid();
    var straightDir = this._straightLink.getSegmentStartDir();
    var bendDir = 1 - straightDir;
    var limits = TranslateUtil.newArray(2, 0);
    limits[0] = this._bp0[straightDir];
    limits[1] = this._bp1[straightDir];
    grid.removeObstacle(straightDir, this._bp0[bendDir], limits, this._bendLink, bSegIdx0);
    limits[0] = this._bp1[bendDir];
    limits[1] = TPBDir;
    grid.removeObstacle(bendDir, this._bp1[straightDir], limits, this._bendLink, bSegIdx1);

    this._bendLink.getSegments()[bSegIdx1] += sign * (limits[0] - limits[1]);
    obstacleFlags[bSegIdx1] = false;
    bendPoint.setIndex(bendDir, TPBDir - this._bgridCorr[bendDir]);
    this._bendLink.updateGrid();
    grid.setRectObstacle(this._bendLink.getFromRect());
    grid.setRectObstacle(this._bendLink.getToRect());
  },

  exchangeBendAndStraightLink : function()
  {
    // summary:
    //		Exchange the termination point of the straight link and the bend
    //		link.
    // tags:
    //		private
    
    var dummyPoint = new LLTerminationPoint(0, 0, 0, 0, 0, 0, 0, ibm_ilog.graphlayout.Direction.NONE, null);
    var straightPoint;
    var bendPoint;
    var otherStraightPoint;
    var bSegIdx;
    var obstacleFlags;
    var TPSDir;
    var TPBDir;
    var sign;

    if (this._exchangeAtStraightLinkStart)
    {
      straightPoint = this._straightLink.getActStartPoint();
      otherStraightPoint = this._straightLink.getActEndPoint();
      TPSDir = this._slimits[1];
    }
    else
    {
      straightPoint = this._straightLink.getActEndPoint();
      otherStraightPoint = this._straightLink.getActStartPoint();
      TPSDir = this._slimits[0];
    }

    if (this._exchangeAtBendLinkStart)
    {
      bendPoint = this._bendLink.getActStartPoint();
      bSegIdx = 1;
      obstacleFlags = this._bendLink.getPrevObstacleFlags();
      sign = 1;
    }
    else
    {
      bendPoint = this._bendLink.getActEndPoint();
      bSegIdx = this._bendLink.getSegments().length - 2;
      obstacleFlags = this._bendLink.getNextObstacleFlags();
      sign = -1;
    }

    var grid = this.getGrid();
    var straightDir = this._straightLink.getSegmentStartDir();
    var bendDir = 1 - straightDir;
    var limits = TranslateUtil.newArray(2, 0);
    limits[0] = this._sp1[straightDir];
    limits[1] = this._sp2[straightDir];
    grid.removeObstacle(straightDir, this._sp1[bendDir], limits, this._straightLink, 0);

    limits[0] = this._bp1[bendDir];
    limits[1] = this._sp1[bendDir];
    grid.removeObstacle(bendDir, this._bp1[straightDir], limits, this._bendLink, bSegIdx);

    this._bendLink.getSegments()[bSegIdx] += sign * (limits[0] - limits[1]);
    obstacleFlags[bSegIdx] = false;

    TPBDir = this._bp0[bendDir];
    otherStraightPoint.setIndex(bendDir, TPBDir - this._sgridCorr[bendDir]);
    otherStraightPoint.setIndex(straightDir, TPSDir - this._sgridCorr[straightDir]);

    var gridCorrX = this._sgridCorr[0] - this._bgridCorr[0];
    var gridCorrY = this._sgridCorr[1] - this._bgridCorr[1];
    dummyPoint.copyData(straightPoint, 0, 0);
    straightPoint.copyData(bendPoint, -gridCorrX, -gridCorrY);
    bendPoint.copyData(dummyPoint, gridCorrX, gridCorrY);

    this._straightLink.updateGrid();
    this._bendLink.updateGrid();
    grid.setRectObstacle(this._straightLink.getFromRect());
    grid.setRectObstacle(this._straightLink.getToRect());
    grid.setRectObstacle(this._bendLink.getFromRect());
    grid.setRectObstacle(this._bendLink.getToRect());

    dummyPoint.dispose();
  }

}
);

return LLCrossRedAlgorithm;
});

},
'ibm_ilog/graphlayout/shortlink/_SLMultiLinkData':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_SLMultiLinkData", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_SLLinkData"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _SLLinkData	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _SLLinkData = ibm_ilog.graphlayout.shortlink._SLLinkData;
=====*/

var _SLMultiLinkData = declare("ibm_ilog.graphlayout.shortlink._SLMultiLinkData", _SLLinkData, 
{
  // summary:
  //		An object to store data about links. 
  // description:
  //		An object to store data about links. 
  //		The LinkData objects are stored in a property (temporarily) attached
  //		to the origin links.   
  // tags:
  //		private
  
  _linkOffset: null,
  _linkOffsetOrig: null,
  _linkOffsetDest: null,

  _individualLinks: null,

  constructor : function(layout, firstSlaveLink, linkOffset)
  {
    this.addSlave(firstSlaveLink);
  },
  
  preamble:function(){
      return [arguments[0], arguments[1].nodeOrLink, arguments[1]];
  },
  isMaster : function()
  {
    // summary:
    //		Returns true if this is a master link. 
    // description:
    //		Returns true if this is a master link. 
    //		For multilinks, overridden version that returns always true.
    // tags:
    //		private
    
    return true; // Boolean
  },

  cleanMasterSlaveInfo : function(linkOffset, firstSlaveLink)
  {
    this.inherited("cleanMasterSlaveInfo", arguments, [linkOffset, firstSlaveLink]);
    if (this._individualLinks != null) {
      this._individualLinks.length = 0;
    }
    this.setLinkOffset(linkOffset, true, true);

    this.copyIncrementalData(firstSlaveLink);
  },

  getLinkWidth : function()
  {
    return this.getLinkWidthWithOffset(this._linkOffset); // Number
  },

  getLinkWidthWithOffset : function(linkOffset )
  {
    // summary:
    //		In "sameShapeForMultipleLinks" mode, if this link is a bundle, the
    //		method returns the total width of the bundle, for a given link
    //		offset. 
    // description:
    //		In "sameShapeForMultipleLinks" mode, if this link is a bundle, the
    //		method returns the total width of the bundle, for a given link
    //		offset. 
    //		Otherwise, returns the normal width.  
    // tags:
    //		private
    
    if (linkOffset == 0) {
      return this._linkWidth; // Number
    }
    var nSlaves = this.getNumberOfIndividualLinks();
    return (nSlaves >= 2 ? (this._linkWidth + (nSlaves - 1) * linkOffset) : this._linkWidth); // Number
  },

  setLinkOffset : function(linkOffset, copyToOriginOffset, copyToDestinationOffset)
  {
    this._linkOffset = linkOffset;
    if (copyToOriginOffset) {
      this._linkOffsetOrig = linkOffset;
    }
    if (copyToDestinationOffset) {
      this._linkOffsetDest = linkOffset;
    }
  },

  setOriginLinkOffset : function(offset)
  {
    this._linkOffsetOrig = offset;
  },

  getOriginLinkOffset : function()
  {
    return this._linkOffsetOrig; // Number
  },

  setDestinationLinkOffset : function(offset)
  {
    this._linkOffsetDest = offset;
  },

  getDestinationLinkOffset : function()
  {
    return this._linkOffsetDest; // Number
  },

  addSlave : function(slaveLinkData)
  {
    // summary:
    //		Adds a slave link. 
    // description:
    //		Adds a slave link. 
    //		(This link is the master of <code>slaveLinkData</code>.)
    // tags:
    //		private
    
    if (this._individualLinks == null) {
      this._individualLinks = new Array();
    }
    slaveLinkData.setSlave(true);
    this._individualLinks.push(slaveLinkData);

    if (slaveLinkData.nodeOrLink != this.nodeOrLink) {
      this._linkWidth += slaveLinkData.getLinkWidth();
    }
    slaveLinkData.setMasterLink(this);
  },

  getIndividualLinks : function()
  {
    // summary:
    //		Returns the slave links of this (master) link.
    // tags:
    //		private
    
    return this._individualLinks; // Array
  },

  getNumberOfIndividualLinks : function()
  {
    // summary:
    //		Returns the number of slave links of this (master) link.
    // tags:
    //		private
    
    return (this._individualLinks == null ? 0 : this._individualLinks.length); // int
  }

}
);

return _SLMultiLinkData;
});

},
'ibm_ilog/graphlayout/shortlink/_IncidentLinksRefiner':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_IncidentLinksRefiner", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    TranslateUtil	// #7
    ) {

var _IncidentLinksRefiner = declare("ibm_ilog.graphlayout.shortlink._IncidentLinksRefiner", null, 
{
  // summary:
  //		This class is used to refine the links incident to a node. 
  // description:
  //		This class is used to refine the links incident to a node. 
  //		This consists in moving some segments of the links in order to ovoid
  //		overlaps and crossings between the links. Each layout instance uses
  //		only one instance of it for each link style.  
  // tags:
  //		extension
  
  _layout: null,

  constructor : function(layout)
  {
    this._layout = layout;
  },
  
  getLayoutAlgorithm : function()
  {
    return this._layout; // ibm_ilog.graphlayout.shortlink._ShortLinkAlgorithm
  },

  clean : function()
  {
    // summary:
    //		Cleans the data stored by this class.
    // tags:
    //		private
    
  },

  refine : function(vectLinks, nodeData, nodeSide, linkOffset, minFinalSegmentLength, reversedOrder, withFixedConnectionPoints)
  {
    // summary:
    //		Final refinement of the link layout. 
    // description:
    //		Final refinement of the link layout. 
    //		Refines a set of links connected to <code>nodeSide</code> side of
    //		<code>node</code>. Once the type of shape for each link and the
    //		order of the connection of all the links incident to a given side of
    //		a node are choosen, the second and, eventually, the third segment of
    //		an orthogonal link can be moved a little by this refinement
    //		procedure. A different procedure, that can move the two intermediate
    //		points, is used for direct style links. The purpose of the
    //		refinement is to avoid overlapping segments for links that are
    //		incident to the same side of the same node. (Currently, nothing is
    //		done for links that are not incident to the same side of the same
    //		node.) CAUTION: it is mandatory to call this.sort before this.refine
    //		!  
    // withFixedConnectionPoints: Boolean
    //		true if at least one link in vectLinks
    //		has fixed connection points or is fixed. In such a case, the link
    //		may not be ordered according to their relative orientation toward
    //		the node side.
    // tags:
    //		extension
    },
  
  refineLinksOrthogonal : function(link1, link2, origin1, origin2, nodeSide, goingLeft, linkOffset, stopIfDifferentDirection)
  {
    // summary:
    //		Reshapes the second segment of link2 to avoid crossings with link1,
    //		if necessary.  
    // goingLeft: Boolean
    //		'true' if both links are expected to do a
    //		left-turn, and 'false' if they are expected to do a
    //		right-turn
    // link1: _SLLinkData
    //		previous link (at left of link2)
    //		link2: 
    //		           _SLLinkData
    //		current link (at right of link1)
    // returns:
    //		true 'true' if both links have the
    //		'goingLeft' orientation, and
    //		'false'otherwise.
    // tags:
    //		protected
    
    if (link1 == link2 || link2.isSameSideSelfLink() || link2.isFixed())
    {
      return true; // Boolean
    }

    var nPoints1 = link1.getLinkShape().getNumberOfPoints();
    var nBends2 = link2.getLinkShape().getNumberOfPoints() - 2;

    if (nBends2 < 1)
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6603E");
    }

    var point11 = link1.getPointAt(origin1, 1);
    var point12 = (nPoints1 > 2 ? link1.getPointAt(origin1, 2) : point11);
    var point21 = link2.getPointAt(origin2, 1);
    var point22 = link2.getPointAt(origin2, 2);
    var goingLeft1 = nodeSide.getTangentDelta(point11, point12) >= 0;
    var goingLeft2 = nodeSide.getTangentDelta(point21, point22) >= 0;
    if (nBends2 < 2) {
      return (goingLeft ? (goingLeft1 && goingLeft2) : (!goingLeft1 && !goingLeft2)); // Boolean
    }

    var firstLinkIsSameSideSelfLink = link1.isSameSideSelfLink();
    var okDirection = (goingLeft ? (firstLinkIsSameSideSelfLink || goingLeft1 && goingLeft2) : (firstLinkIsSameSideSelfLink || !goingLeft1 && !goingLeft2));

    if (!okDirection && (stopIfDifferentDirection || (goingLeft != goingLeft2 || !goingLeft)))
    {
      return false; // Boolean
    }

    link2.shapePointsModified();
    var nBends1 = link1.getLinkShape().getNumberOfPoints() - 2;
    var point13 = null;
    var point14 = null;

    if (nBends1 >= 2)
    {
      point13 = link1.getPointAt(origin1, 3);
      if (nBends1 >= 3) {
        point14 = link1.getPointAt(origin1, 4);
      }
    }

    var point23 = null;
    var point24 = null;
    var point25 = null;

    if (nBends2 >= 2)
    {
      point23 = link2.getPointAt(origin2, 3);

      if (nBends2 >= 3)
      {
        point24 = link2.getPointAt(origin2, 4);
        if (nBends2 >= 4) {
          point25 = link2.getPointAt(origin2, 5);
        }
      }
    }

    var sameNodeAndSideDestination = link1.getConnectionNode(!origin1) == link2.getConnectionNode(!origin2) && link1.getNodeSide(!origin1) == link2.getNodeSide(!origin2);
    nodeSide.refineLinksOrthogonalWithPoints(this._layout, point11, point12, point13, point14, point21, point22, point23, point24, point25, link1.getLinkWidth(), link2.getLinkWidth(), goingLeft1,
        goingLeft2, firstLinkIsSameSideSelfLink, nBends1, nBends2, link2.boundingBox, link2.getNodeSide(!origin2), linkOffset, sameNodeAndSideDestination, true);

    return true; // Boolean
  }

}
);

return _IncidentLinksRefiner;
});

},
'ibm_ilog/graphlayout/shortlink/_SLBundlesModes':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_SLBundlesModes", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var _SLBundlesModes = declare("ibm_ilog.graphlayout.shortlink._SLBundlesModes", null, 
{
  // summary:
  //		This class contains all enumeration constants used for
  //		"linkBundlesMode"  parameter of ShortLinkLayout. 
  // description:
  //		This class contains all enumeration constants used for
  //		"linkBundlesMode"  parameter of ShortLinkLayout. 
  //		
  // tags:
  //		private
  
  constructor : function()
  {
    
  }

}
);

lang.mixin(_SLBundlesModes,
{
  NO_BUNDLE : null,
  FIRST_LAST_SEGMENT : null,
  IMPROVED_FIRST_LAST_SEGMENT : null,
  _fields_init : function(c)
  {
    c = c || _SLBundlesModes;
    
    c.NO_BUNDLE = 0;
    c.FIRST_LAST_SEGMENT = 1;
    c.IMPROVED_FIRST_LAST_SEGMENT = 2;
  }
  
}
);

(function(){ _SLBundlesModes._fields_init(); })();

return _SLBundlesModes;
});

},
'ibm_ilog/graphlayout/longlink/LLTerminationCandAlgorithm':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLTerminationCandAlgorithm", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../longlink/LLTerminationPoint",	// #7
  "../Direction"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    LLTerminationPoint,	// #7
    Direction	// #8
    ) {

var LLTerminationCandAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLTerminationCandAlgorithm", null, 
{
  // summary:
  //		The algorithm to find the possible start and end points of a single
  //		link. 
  // description:
  //		The algorithm to find the possible start and end points of a single
  //		link. 
  //		If the start point is not fixed, it searches a field of candidates
  //		for start and end points along the border of the "from" and "to"
  //		node. Even if the start point is fixed, this algorithm is needed to
  //		calculate all the auxiliary information needed for the class
  //		LLTerminationPoint.  
  // tags:
  //		private
  
  _layout: null,

  _grid: null,

  _resPoints: null,
  _numberResPoints: null,

  _fallbackPoint: null,

  constructor : function(layout, grid)
  {
    this._layout = layout;
    this._grid = grid;
    this._resPoints = null;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._layout = null;
    this._grid = null;
    this._resPoints = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getGrid : function()
  {
    // summary:
    //		Returns the routing grid.
    // tags:
    //		private
    
    return (this._grid); // ibm_ilog.graphlayout.longlink.LLGrid
  },

  run : function(link)
  {
    // summary:
    //		Find all start and end point candidates for the input link. 
    // description:
    //		Find all start and end point candidates for the input link. 
    //		It stores the candidates in the link.  
    // tags:
    //		private
    
    var layout = this.getLayout();

    var fromPref = TranslateUtil.newArray(4, 0);
    var toPref = TranslateUtil.newArray(4, 0);
    var i;

    for (i = 0; i < 4; i++)
      fromPref[i] = toPref[i] = 0;
    var fromRect = link.getFromRect();
    var toRect = link.getToRect();
    var fromPoint = link.getFromPoint();
    var toPoint = link.getToPoint();

    if ((fromPoint == null || toPoint == null) && !link.isSelfLoop()) {
      this.calcSidePreferences(fromRect, toRect, fromPref, toPref);
    }

    var minLen = layout.getMinStartSegmentLength();
    this._fallbackPoint = null;
    if (fromPoint == null) {
      this.generateTerminationPoints(fromRect, fromPref, link, link.getFromNode(), minLen, true);
    }
    else {
      this.searchSingleTerminationPoint(fromPoint.x, fromPoint.y, fromRect, minLen);
    }
    link.setStartPoints(this._resPoints, this._numberResPoints);

    minLen = layout.getMinEndSegmentLength();
    if (toPoint == null) {
      this.generateTerminationPoints(toRect, toPref, link, link.getToNode(), minLen, false);
    }
    else {
      this.searchSingleTerminationPoint(toPoint.x, toPoint.y, toRect, minLen);
    }
    link.setEndPoints(this._resPoints, this._numberResPoints);
  },

  generateTerminationPoints : function(terminationRect, sidePref, link, node, minLen, origin)
  {
    // summary:
    //		Generate all potential termination points if no fixed start/end
    //		point is given. 
    // description:
    //		Generate all potential termination points if no fixed start/end
    //		point is given. 
    //		The result is in the array &#95;resPoint, while &#95;numberResPoints is the
    //		first free index in this array (which means accordingly, the number
    //		of points in the array). The array is ordered according to penalties
    //		(and if the penalty is equal, according to the
    //		point's internal order).
    // tags:
    //		private
    
    this._fallbackPoint = null;

    var minCOffs = this.getLayout().getMinNodeCornerOffset();
    var grid = this.getGrid();
    var startHIndex = grid.getMinGridIndexAtSide(terminationRect, 0, minCOffs);
    var endHIndex = grid.getMaxGridIndexAtSide(terminationRect, 0, minCOffs);
    var numberOfHPoints = Math.max(endHIndex - startHIndex + 1, 0);
    var startVIndex = grid.getMinGridIndexAtSide(terminationRect, 1, minCOffs);
    var endVIndex = grid.getMaxGridIndexAtSide(terminationRect, 1, minCOffs);
    var numberOfVPoints = Math.max(endVIndex - startVIndex + 1, 0);

    this._resPoints = TranslateUtil.newArray(2 * numberOfHPoints + 2 * numberOfVPoints, null);
    this._numberResPoints = 0;

    this.searchTerminationPoints(terminationRect.y, -minLen, Direction.TOP, startHIndex, numberOfHPoints, sidePref[0], link, node, origin);

    this.searchTerminationPoints(terminationRect.y + terminationRect.height, minLen, Direction.BOTTOM, startHIndex, numberOfHPoints, sidePref[2], link, node, origin);

    this.searchTerminationPoints(terminationRect.x, -minLen, Direction.LEFT, startVIndex, numberOfVPoints, sidePref[1], link, node, origin);

    this.searchTerminationPoints(terminationRect.x + terminationRect.width, minLen, Direction.RIGHT, startVIndex, numberOfVPoints, sidePref[3], link, node, origin);

    if (this._numberResPoints > 1)
    {
      this._resPoints[0].sortAccordingToPenalty(this._resPoints);
    }

    if (this._numberResPoints == 0 && this._fallbackPoint != null) {
      this._resPoints[this._numberResPoints++] = this._fallbackPoint;
    }
  },

  searchTerminationPoints : function(terminationCoord, minLen, side, minGridIndex, numberOfGridLines, sidePreference, link, node, origin)
  {
    // summary:
    //		Search the termination points for a routing at a given side when the
    //		start/end points are not fixed. 
    // description:
    //		Search the termination points for a routing at a given side when the
    //		start/end points are not fixed. 
    //		The result is in the array &#95;resPoint where &#95;numberResPoints points
    //		to the first free index. Note that the array must be allocated
    //		before. The side is Direction (Left, Top, Right, Bottom). The object
    //		where the link terminates covers the grid from grid line
    //		"minGridIndex" to "minGridIndex + numberOfGridLines". The effective
    //		end point of the link is at terminationCoord. Side preference is a
    //		value between 0 and 4 to indicate how favourable this side is. This
    //		is needed to calculate a good order for the termination points. The
    //		flag "origin" is true if we search start points of the link, and
    //		false if we search end points of the link. termination Coord |      
    //		        side = Right V         | minGridIndex -> ,------.         |
    //		| node |&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;&#95;| link |      | mGI + nOGL   ->
    //		`------'^        ^ \      / minLen
    // tags:
    //		private
    
    if (numberOfGridLines <= 0) {
      return;
    }

    var larger;
    var dir;

    switch (side)
    {
      case Direction.RIGHT:
        dir = 0;
        larger = true;
        break;
      case Direction.LEFT:
        dir = 0;
        larger = false;
        break;
      case Direction.BOTTOM:
        dir = 1;
        larger = true;
        break;
      case Direction.TOP:
      default:
        dir = 1;
        larger = false;
        break;
    }

    var grid = this.getGrid();
    var terminationIndex = grid.getGridIndexAtMinDist(dir, terminationCoord, larger);
    var terminationIndexAtMinLen = grid.getGridIndex(dir, terminationCoord + minLen, larger);
    var nrGridLines0 = grid.getNumberOfGridLines(dir);
    var nrGridLines1 = grid.getNumberOfGridLines(1 - dir);

    if (larger)
    {
      if (terminationIndexAtMinLen < terminationIndex) {
        terminationIndexAtMinLen = terminationIndex;
      }
    }
    else
    {
      if (terminationIndexAtMinLen > terminationIndex) {
        terminationIndexAtMinLen = terminationIndex;
      }
    }

    if (terminationIndexAtMinLen < 0) {
      return;
    }
    if (terminationIndexAtMinLen >= nrGridLines0) {
      return;
    }

    var i;
    var index0;
    var index1;
    var incr = 0;
    var order = sidePreference * this._resPoints.length;
    var forbiddenMinIndex;
    var forbiddenMaxIndex;
    var penalty = 0;
    var overlapPenalty = terminationIndexAtMinLen - terminationIndex;
    if (overlapPenalty < 0) {
      overlapPenalty = -overlapPenalty;
    }
    var mid = minGridIndex + Math.floor((numberOfGridLines - 1) / 2);
    var segment;

    for (i = 0; i < numberOfGridLines; i++)
    {
      index1 = mid + incr;

      if (index1 < 0 || index1 >= nrGridLines1)
      {
        if (incr <= 0) {
          incr = -incr + 1;
        }
        else {
          incr = -incr;
        }
        continue;
      }

      penalty = 0;
      segment = grid.getGridLine(1 - dir, index1).searchSegment(terminationIndex);

      if (segment == null) {
        penalty = overlapPenalty;
      }
      else if (segment.getStartIndex() > terminationIndexAtMinLen || segment.getEndIndex() < terminationIndexAtMinLen)
      {
        penalty = overlapPenalty;
        if (larger) {
          penalty -= (segment.getEndIndex() - terminationIndex);
        }
        else {
          penalty -= (terminationIndex - segment.getStartIndex());
        }
        if (penalty <= 0) {
          penalty = 1;
        }
      }

      segment = grid.getGridLine(1 - dir, index1).searchClosestSegment(terminationIndexAtMinLen, larger);

      if (segment != null)
      {
        forbiddenMinIndex = -1;
        forbiddenMaxIndex = -1;

        if (segment.getStartIndex() > terminationIndexAtMinLen)
        {
          index0 = segment.getStartIndex();
          penalty += index0 - terminationIndexAtMinLen;
        }
        else if (segment.getEndIndex() < terminationIndexAtMinLen)
        {
          index0 = segment.getEndIndex();
          penalty += terminationIndexAtMinLen - index0;
        }
        else
        {
          index0 = terminationIndexAtMinLen;

          if (larger)
          {
            forbiddenMinIndex = terminationIndex;
            forbiddenMaxIndex = terminationIndexAtMinLen - 1;
            if (forbiddenMaxIndex < forbiddenMinIndex) {
              forbiddenMinIndex = forbiddenMaxIndex;
            }
          }
          else
          {
            forbiddenMinIndex = terminationIndexAtMinLen + 1;
            forbiddenMaxIndex = terminationIndex;
            if (forbiddenMaxIndex < forbiddenMinIndex) {
              forbiddenMaxIndex = forbiddenMinIndex;
            }
          }
        }

        penalty =
          this.getLayout().getTerminationPointPenalty(link.getLinkObject(), node, origin, grid.getGridCoord(0, ((dir == 0 ? index0 : index1))), grid.getGridCoord(1, ((dir == 1 ? index0 : index1))),
              side, penalty);

        if (penalty == LLTerminationCandAlgorithm.MAXVAL)
        {
          if (this._fallbackPoint == null) {
            this._fallbackPoint = new LLTerminationPoint(index0, index1, forbiddenMinIndex, forbiddenMaxIndex, dir, 0, order, side, segment);
          }
        }
        else
        {
          this._resPoints[this._numberResPoints++] = new LLTerminationPoint(index0, index1, forbiddenMinIndex, forbiddenMaxIndex, dir, penalty, order, side, segment);
        }
      }

      if (incr <= 0) {
        incr = -incr + 1;
      }
      else {
        incr = -incr;
      }
      order++;
    }
  },

  searchSingleTerminationPoint : function(x, y, terminationRect, minSegDist)
  {
    // summary:
    //		Search a single appropriate termination point if the fixed
    //		coordinates (x, y) are given as start/end point. 
    // description:
    //		Search a single appropriate termination point if the fixed
    //		coordinates (x, y) are given as start/end point. 
    //		Since (x, y) may not be a grid point, and the first/last segment
    //		needs length "minSegDist", we need a grid point that is minSegDist
    //		from (x, y) away. The result is in the array &#95;resPoint where
    //		&#95;numberResPoints points to the first free index. Different to
    //		searchTerminationPoints, the array must NOT be allocated before,
    //		because we know it is just one point.
    // tags:
    //		private
    
    var grid = this.getGrid();
    var xindex;
    var yindex;
    var left = terminationRect.x;
    var right = terminationRect.x + terminationRect.width;
    var top = terminationRect.y;
    var bottom = terminationRect.y + terminationRect.height;

    var bestXIndex = 0;
    var bestYIndex = 0;
    var bestForbiddenMinIndex = -1;
    var bestForbiddenMaxIndex = -1;

    var bestSide = Direction.LEFT;
    var testIndex;
    var testForbMinIndex;
    var testForbMaxIndex;

    var testSide;
    var bestDistToBorder = LLTerminationCandAlgorithm.FMAXVAL;
    var bestDistToSeg = LLTerminationCandAlgorithm.FMAXVAL;
    var h1;
    var h2;
    var bestSegment = null;
    var seg;
    var needOrthDummySeg = false;

    testSide = Direction.LEFT;
    xindex = grid.getGridIndex(0, x - minSegDist, false);
    yindex = grid.getClosestGridIndex(1, y);

    if (x <= right)
    {
      h1 = left - x;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (y < top || bottom < y) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(xindex, yindex, 0, false);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (!seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > xindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < xindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = xindex;
        testForbMinIndex = xindex + 1;
        testForbMaxIndex = grid.getGridIndex(0, x, false);
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMaxIndex = testForbMinIndex;
        }
      }

      h2 = grid.getGridCoord(0, testIndex) - (x - minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = testIndex;
        bestYIndex = yindex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = !seg.isHorizontal();
      }
    }

    testSide = Direction.RIGHT;
    xindex = grid.getGridIndex(0, x + minSegDist, true);

    if (x >= left)
    {
      h1 = right - x;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (y < top || bottom < y) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(xindex, yindex, 0, true);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (!seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > xindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < xindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = xindex;
        testForbMinIndex = grid.getGridIndex(0, x, true);
        testForbMaxIndex = xindex - 1;
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMinIndex = testForbMaxIndex;
        }
      }

      h2 = grid.getGridCoord(0, testIndex) - (x + minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = testIndex;
        bestYIndex = yindex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = !seg.isHorizontal();
      }
    }

    testSide = Direction.TOP;
    xindex = grid.getClosestGridIndex(0, x);
    yindex = grid.getGridIndex(1, y - minSegDist, false);

    if (y <= bottom)
    {
      h1 = top - y;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (x < left || right < x) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(yindex, xindex, 1, false);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > yindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < yindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = yindex;
        testForbMinIndex = yindex + 1;
        testForbMaxIndex = grid.getGridIndex(1, y, false);
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMaxIndex = testForbMinIndex;
        }
      }

      h2 = grid.getGridCoord(1, testIndex) - (y - minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = xindex;
        bestYIndex = testIndex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = seg.isHorizontal();
      }
    }

    testSide = Direction.BOTTOM;
    yindex = grid.getGridIndex(1, y + minSegDist, true);

    if (y >= top)
    {
      h1 = bottom - y;
      if (h1 < 0) {
        h1 = -h1;
      }
      if (x < left || right < x) {
        h1 = 100000.0 - h1;
      }
    }
    else {
      h1 = LLTerminationCandAlgorithm.FMAXVAL;
    }
    seg = grid.searchClosestSegmentAlive(yindex, xindex, 1, true);

    if (seg != null)
    {
      testForbMinIndex = -1;
      testForbMaxIndex = -1;

      if (seg.isHorizontal()) {
        testIndex = seg.getGridLine().getIndex();
      }
      else if (seg.getStartIndex() > yindex) {
        testIndex = seg.getStartIndex();
      }
      else if (seg.getEndIndex() < yindex) {
        testIndex = seg.getEndIndex();
      }
      else
      {
        testIndex = yindex;
        testForbMinIndex = grid.getGridIndex(1, y, true);
        testForbMaxIndex = yindex - 1;
        if (testForbMaxIndex < testForbMinIndex) {
          testForbMinIndex = testForbMaxIndex;
        }
      }

      h2 = grid.getGridCoord(1, testIndex) - (y + minSegDist);
      if (h2 < 0.0) {
        h2 = -h2;
      }
      if ((h1 < bestDistToBorder) || (h1 == bestDistToBorder && h2 < bestDistToSeg))
      {
        bestSide = testSide;
        bestDistToBorder = h1;
        bestDistToSeg = h2;
        bestXIndex = xindex;
        bestYIndex = testIndex;
        bestSegment = seg;
        bestForbiddenMinIndex = testForbMinIndex;
        bestForbiddenMaxIndex = testForbMaxIndex;
        needOrthDummySeg = seg.isHorizontal();
      }
    }

    if (bestSegment != null)
    {
      if (needOrthDummySeg)
      {
        var line;

        if (bestSegment.isHorizontal())
        {
          line = grid.getGridLine(0, bestXIndex);

          line.addSegment(bestYIndex, bestYIndex);
          bestSegment = line.searchSegment(bestYIndex);
        }
        else
        {
          line = grid.getGridLine(1, bestYIndex);

          line.addSegment(bestXIndex, bestXIndex);
          bestSegment = line.searchSegment(bestXIndex);
        }
      }

      this._resPoints = TranslateUtil.newArray(1, null);
      this._resPoints[0] = new LLTerminationPoint(bestXIndex, bestYIndex, bestForbiddenMinIndex, bestForbiddenMaxIndex, 0, 0, 0, bestSide, bestSegment);
      this._numberResPoints = 1;
    }
    else
    {
      this._numberResPoints = 0;
    }
  },

  calcSidePreferences : function(startRect, endRect, startPref, endPref)
  {
    // summary:
    //		Calculate the side preferences. 
    // description:
    //		Calculate the side preferences. 
    //		The result is in the arrays startPref[4] and endPref[4]. The sides
    //		are numbered: top = 0, left = 1, bottom = 2, right = 3.
    // tags:
    //		private
    
    var layout = this.getLayout();
    var minStartLen = layout.getMinStartSegmentLength();
    var minEndLen = layout.getMinEndSegmentLength();
    LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.x - minStartLen, startRect.x + startRect.width + minStartLen, endRect.x - minEndLen, endRect.x + endRect.width + minEndLen, 1, 3,
        startPref, endPref);
    LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.y - minStartLen, startRect.y + startRect.height + minStartLen, endRect.y - minEndLen, endRect.y + endRect.height + minEndLen, 0, 2,
        startPref, endPref);
  }

}
);

lang.mixin(LLTerminationCandAlgorithm,
{
  MAXVAL : null,
  FMAXVAL : null,

  calcSidePrefInternal : function(startMin, startMax, endMin, endMax, lowerIndex, upperIndex, startPref, endPref)
  {
    var startMid = 0.5 * (startMin + startMax);
    var endMid = 0.5 * (endMin + endMax);

    if (startMid <= endMid)
    {
      if (startMin >= endMin)
      {
        startPref[lowerIndex] = 3;
        startPref[upperIndex] = 2;
        endPref[lowerIndex] = 3;
        endPref[upperIndex] = 2;
      }
      else if (startMax >= endMax)
      {
        startPref[lowerIndex] = 2;
        startPref[upperIndex] = 3;
        endPref[lowerIndex] = 2;
        endPref[upperIndex] = 3;
      }
      else if (startMax >= endMin)
      {
        startPref[lowerIndex] = 3;
        startPref[upperIndex] = 2;
        endPref[lowerIndex] = 2;
        endPref[upperIndex] = 3;
      }
      else
      {
        startPref[lowerIndex] = 4;
        startPref[upperIndex] = 1;
        endPref[lowerIndex] = 1;
        endPref[upperIndex] = 4;
      }
    }
    else
    {
      if (endMin >= startMin)
      {
        startPref[lowerIndex] = 3;
        startPref[upperIndex] = 2;
        endPref[lowerIndex] = 3;
        endPref[upperIndex] = 2;
      }
      else if (endMax >= startMax)
      {
        startPref[lowerIndex] = 2;
        startPref[upperIndex] = 3;
        endPref[lowerIndex] = 2;
        endPref[upperIndex] = 3;
      }
      else if (endMax >= startMin)
      {
        startPref[lowerIndex] = 2;
        startPref[upperIndex] = 3;
        endPref[lowerIndex] = 3;
        endPref[upperIndex] = 2;
      }
      else
      {
        startPref[lowerIndex] = 1;
        startPref[upperIndex] = 4;
        endPref[lowerIndex] = 4;
        endPref[upperIndex] = 1;
      }
    }
  },

  _fields_init : function(c)
  {
    c = c || LLTerminationCandAlgorithm;
    
    c.MAXVAL = /*Number.MAX_VALUE*/2147483647;
    c.FMAXVAL = /*Number.MAX_VALUE*/(3.4028235E38);
  }
  
}
);

(function(){ LLTerminationCandAlgorithm._fields_init(); })();

return LLTerminationCandAlgorithm;
});

},
'ibm_ilog/graphlayout/longlink/LLLink':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLLink", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/FixedPoint",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/LinkStyles",	// #7
  "../internalutil/Point",	// #8
  "../internalutil/TranslateUtil",	// #9
  "../longlink/LLTerminationPoint",	// #10
  "../Direction"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    FixedPoint,	// #5
    GraphModelUtil,	// #6
    LinkStyles,	// #7
    Point,	// #8
    TranslateUtil,	// #9
    LLTerminationPoint,	// #10
    Direction	// #11
    ) {

var LLLink = declare("ibm_ilog.graphlayout.longlink.LLLink", null, 
{
  // summary:
  //		A link contains the information of the routing of the link in the
  //		grid. 
  // description:
  //		A link contains the information of the routing of the link in the
  //		grid. 
  //		This is a proxy for the real link to be routed, which is stored in
  //		&#95;linkObject. If the link is orthogonal, we store the routing in
  //		&#95;segments as follows: We store the start direction in &#95;startDir, and
  //		only store the length of the first segment (i.e., length in grid
  //		indexe), because the actual start point is anyway clear. Since the
  //		further directions are always orthogonal to the previous direction,
  //		we also need only to store the length of each further segment.
  //		Example, the route (0,0)-(10,0)-(10,7)-(14,7)-(14,15) is stored in
  //		this way: start point via calcRealStartPointOnGrid() is (0,0)
  //		&#95;startDir = 0  (means horizontal) &#95;segments = { 10, 7, 4, 8 }  
  // tags:
  //		private
  
  _graph: null,

  _linkObject: null,

  _fromNode: null,
  _toNode: null,

  _fromRect: null,
  _toRect: null,

  _fromPoint: null,
  _toPoint: null,

  _startPoints: null,
  _endPoints: null,

  _actStartPointIndex: null,
  _actEndPointIndex: null,

  _direct: null,

  _frozen: null,

  _gridBaseCoord: null,

  _startDir: null,

  _preferedSide: null,

  _sortValue: null,

  _segments: null,

  _prevObstacle: null,
  _nextObstacle: null,

  _bbox: null,

  constructor : function(graph, link)
  {
    this._graph = graph;
    this._linkObject = link;
    var model = graph.getModel();
    var layout = graph.getLayout();
    this._fromNode = model.getFrom(link);
    this._toNode = model.getTo(link);
    this._fromRect = layout.getNodeBoxForConnection(this._fromNode);
    this._toRect = layout.getNodeBoxForConnection(this._toNode);

    var isFromPointFixed = layout.isFromPointFixed(link);
    var isToPointFixed = layout.isToPointFixed(link);

    if (isFromPointFixed || isToPointFixed)
    {
      var linkPoints = model.getLinkPoints(link);
      if (isFromPointFixed) {
        this._fromPoint = linkPoints[0];
      }
      else {
        this._fromPoint = null;
      }
      if (isToPointFixed) {
        this._toPoint = linkPoints[linkPoints.length - 1];
      }
      else {
        this._toPoint = null;
      }
    }

    this._startPoints = null;
    this._endPoints = null;

    var globalStyle = layout.getGlobalLinkStyle();
    this._direct = (globalStyle == LinkStyles.DIRECT_STYLE || (globalStyle == LinkStyles.MIXED_STYLE && layout.getLinkStyle(link) == LinkStyles.DIRECT_STYLE));
    if (this.isSelfLoop()) {
      this._direct = false;
    }
    this._segments = null;
    this._prevObstacle = null;
    this._nextObstacle = null;
    this._gridBaseCoord = TranslateUtil.newArray(2, 0);
    this._gridBaseCoord[0] = 0.0;
    this._gridBaseCoord[1] = 0.0;
    this._bbox = null;
    this._frozen = false;
    this._preferedSide = Direction.NONE;
    this._sortValue = 0.0;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._fromPoint = null;
    this._toPoint = null;
    this._startPoints = null;
    this._endPoints = null;
    this._segments = null;
    this._prevObstacle = null;
    this._nextObstacle = null;
    this._bbox = null;

    this._graph = null;
    this._linkObject = null;
    this._fromNode = null;
    this._toNode = null;
    this._gridBaseCoord = null;
  },

  getGraph : function()
  {
    // summary:
    //		Returns the owner graph.
    // tags:
    //		private
    
    return this._graph; // ibm_ilog.graphlayout.longlink.LLGraph
  },

  getLinkObject : function()
  {
    // summary:
    //		Returns the corresponding real link.
    // tags:
    //		private
    
    return this._linkObject; // Object
  },

  getFromNode : function()
  {
    // summary:
    //		Returns the "from" node.
    // tags:
    //		private
    
    return this._fromNode; // Object
  },

  getToNode : function()
  {
    // summary:
    //		Returns the "to" node.
    // tags:
    //		private
    
    return this._toNode; // Object
  },

  getFromRect : function()
  {
    // summary:
    //		Returns the rectangle of the "from" node.
    // tags:
    //		private
    
    return this._fromRect; // Rectangle
  },

  getToRect : function()
  {
    // summary:
    //		Returns the rectangle of the "to" node.
    // tags:
    //		private
    
    return this._toRect; // Rectangle
  },

  isCandidateForStraightRoute : function()
  {
    // summary:
    //		Returns true if the horizontal or vertical projection of "from" and
    //		"to" node overlap.
    // tags:
    //		private
    
    return ((this._fromRect.x <= this._toRect.x + this._toRect.width && this._toRect.x <= this._fromRect.x + this._fromRect.width) ||
        (this._fromRect.y <= this._toRect.y + this._toRect.height && this._toRect.y <= this._fromRect.y + this._fromRect.height)); // Boolean
  },

  isSelfLoop : function()
  {
    // summary:
    //		Returns true if the link is a self loop.
    // tags:
    //		private
    
    return (this._fromNode == this._toNode); // Boolean
  },

  getFromPoint : function()
  {
    // summary:
    //		Returns the "from" point, or null if it is not fixed.
    // tags:
    //		private
    
    return this._fromPoint; // Point
  },

  setFromPoint : function(p)
  {
    // summary:
    //		Sets the "from" point.
    // tags:
    //		private
    
    this._fromPoint = p;
  },

  getToPoint : function()
  {
    // summary:
    //		Returns the "to" point, or null if it is not fixed.
    // tags:
    //		private
    
    return this._toPoint; // Point
  },

  setToPoint : function(p)
  {
    // summary:
    //		Sets the "to" point.
    // tags:
    //		private
    
    this._toPoint = p;
  },

  isFromPointFixed : function()
  {
    // summary:
    //		Returns true if the "from" point is fixed.
    // tags:
    //		private
    
    return (this._fromPoint != null); // Boolean
  },

  isToPointFixed : function()
  {
    // summary:
    //		Returns true if the "to" point is fixed.
    // tags:
    //		private
    
    return (this._toPoint != null); // Boolean
  },

  getStartPoints : function()
  {
    // summary:
    //		Returns the start point candidates.
    // tags:
    //		private
    
    return this._startPoints; // Array
  },

  getActStartPoint : function()
  {
    // summary:
    //		Returns the actual start point candidate.
    // tags:
    //		private
    
    return this._startPoints[this._actStartPointIndex]; // ibm_ilog.graphlayout.longlink.LLTerminationPoint
  },

  setStartPoints : function(points, numberOfPoints)
  {
    // summary:
    //		Sets the start point candidates.
    // tags:
    //		private
    
    this._startPoints = TranslateUtil.newArray(numberOfPoints, null);

    for (var i = 0; i < numberOfPoints; i++)
      this._startPoints[i] = points[i];
    var grid = this.getGraph().getGrid();
    this._gridBaseCoord[0] = grid.getMinCoord(0);
    this._gridBaseCoord[1] = grid.getMinCoord(1);
  },

  getEndPoints : function()
  {
    // summary:
    //		Returns the end point candidates.
    // tags:
    //		private
    
    return this._endPoints; // Array
  },

  getActEndPoint : function()
  {
    // summary:
    //		Returns the actual end point candidate.
    // tags:
    //		private
    
    return this._endPoints[this._actEndPointIndex]; // ibm_ilog.graphlayout.longlink.LLTerminationPoint
  },

  setEndPoints : function(points, numberOfPoints)
  {
    // summary:
    //		Sets the start point candidates.
    // tags:
    //		private
    
    this._endPoints = TranslateUtil.newArray(numberOfPoints, null);

    for (var i = 0; i < numberOfPoints; i++)
      this._endPoints[i] = points[i];
    var grid = this.getGraph().getGrid();
    this._gridBaseCoord[0] = grid.getMinCoord(0);
    this._gridBaseCoord[1] = grid.getMinCoord(1);
  },

  setActStartAndEndCandidates : function(startPointIndex, endPointIndex)
  {
    // summary:
    //		Sets the actual start and end point index.
    // tags:
    //		private
    
    this._actStartPointIndex = startPointIndex;
    this._actEndPointIndex = endPointIndex;
  },

  setActStartCandidate : function(startPointIndex)
  {
    // summary:
    //		Sets the actual start point index.
    // tags:
    //		private
    
    this._actStartPointIndex = startPointIndex;
  },

  setActEndCandidate : function(endPointIndex)
  {
    // summary:
    //		Sets the actual end point index.
    // tags:
    //		private
    
    this._actEndPointIndex = endPointIndex;
  },

  cleanUnusedStartAndEndPoints : function()
  {
    // summary:
    //		Cleans up the unused termination points except the actual ones. 
    // description:
    //		Cleans up the unused termination points except the actual ones. 
    //		This mainly saves memory.
    // tags:
    //		private
    
    var points = TranslateUtil.newArray(1, null);
    points[0] = this.getActStartPoint();
    this._actStartPointIndex = 0;
    this._startPoints = points;
    points = TranslateUtil.newArray(1, null);
    points[0] = this.getActEndPoint();
    this._actEndPointIndex = 0;
    this._endPoints = points;
  },

  isDirect : function()
  {
    // summary:
    //		Returns true if the link needs a direct route instead of an
    //		orthogonal route.
    // tags:
    //		private
    
    return this._direct; // Boolean
  },

  getSegmentDir : function(i)
  {
    // summary:
    //		Returns the direction of the ith segment (0 if horizontal, 1 if
    //		vertical). 
    // description:
    //		Returns the direction of the ith segment (0 if horizontal, 1 if
    //		vertical). 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    return ((this._startDir + i) % 2); // int
  },

  setSegmentStartDir : function(dir)
  {
    // summary:
    //		Sets the direction of the first segment.
    // tags:
    //		private
    
    this._startDir = dir;
  },

  getSegmentStartDir : function()
  {
    // summary:
    //		Returns the direction of the first segment. 
    // description:
    //		Returns the direction of the first segment. 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    return (this._startDir); // int
  },

  getSegmentEndDir : function()
  {
    // summary:
    //		Returns the direction of the last segment. 
    // description:
    //		Returns the direction of the last segment. 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    return ((this._startDir + this._segments.length - 1) % 2); // int
  },

  getGridCorrection : function(dir)
  {
    // summary:
    //		Returns the correction value needed if the grid has shifted. 
    // description:
    //		Returns the correction value needed if the grid has shifted. 
    //		<p>
    //		This is the problem: Assume we have a grid with mesh size (10, 10)
    //		which has its base (origin) at (50, 50). This means, the point with
    //		real coordinates (50, 50) has grid coordinate (0, 0), the point (60,
    //		100) has grid coordinates (1, 5), etc. During the route, we store
    //		only grid coordinates, but later we need real coordinates. If after
    //		route, we shift the grid to another base, we normally
    //		wouldn't be able to retrieve the correct real coordinates of the routing anymore. But this method allows to retrieve the coorection value. Assume we shift the grid to base (20, 30). If the grid was at base (50, 50) during the route, then the grid correction value is the difference in grid coordinates, i.e. gridCorrection(0) = 3 and gridCorrection(1) = 2. Adding this to the stored grid coordinates of the link allows to calculate the real coordinates of the link when the shifted grid is given.
    //		</p>
    // tags:
    //		private
    
    var grid = this.getGraph().getGrid();
    var offset = this._gridBaseCoord[dir] - grid.getMinCoord(dir);
    var gridDist = grid.getGridDist(dir);

    return (Math.floor(((offset + 0.4 * gridDist) / gridDist))); // int
  },

  getNextBend : function(point, i)
  {
    // summary:
    //		Retrieves the ith bend point from its previous point. 
    // description:
    //		Retrieves the ith bend point from its previous point. 
    //		The 0th bend point is the start point. (Note that this is not
    //		exactly the same as the one returned by getActStartPoint). The input
    //		point should contain the previous point (as grid indexe), and is
    //		filled with the next point. It returns true if there was a next
    //		point, otherwise false. This is only relevant in orthogonal link
    //		style.
    // tags:
    //		private
    
    if (i == 0)
    {
      this.calcRealStartPointOnGrid(point);

      point[0] += this.getGridCorrection(0);
      point[1] += this.getGridCorrection(1);
      return (true); // Boolean
    }
    else if (i <= this._segments.length)
    {
      point[this.getSegmentDir(i - 1)] += this._segments[i - 1];
      return (true); // Boolean
    }

    return (false); // Boolean
  },

  getPrevBend : function(point, i)
  {
    // summary:
    //		Retrieves the ith bend point from its next point. 
    // description:
    //		Retrieves the ith bend point from its next point. 
    //		The input point should contain the next point (as grid indexe), and
    //		is filled with the previous point. It returns true if there was a
    //		previous point, otherwise false. This is only relevant in orthogonal
    //		link style.
    // tags:
    //		private
    
    if (i == this._segments.length)
    {
      this.calcRealEndPointOnGrid(point);

      point[0] += this.getGridCorrection(0);
      point[1] += this.getGridCorrection(1);
      return (true); // Boolean
    }
    else if (i >= 0)
    {
      point[this.getSegmentDir(i)] -= this._segments[i];
      return (true); // Boolean
    }

    return (false); // Boolean
  },

  getSegments : function()
  {
    // summary:
    //		Returns the segment array of the link. 
    // description:
    //		Returns the segment array of the link. 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    return (this._segments); // Array
  },

  getPrevObstacleFlags : function()
  {
    // summary:
    //		Returns the array of previous obstacle flags of the link. 
    // description:
    //		Returns the array of previous obstacle flags of the link. 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    return (this._prevObstacle); // Array
  },

  getNextObstacleFlags : function()
  {
    // summary:
    //		Returns the array of next obstacle flags of the link. 
    // description:
    //		Returns the array of next obstacle flags of the link. 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    return (this._prevObstacle); // Array
  },

  setSegments : function(segments, prevObstacleFlags, nextObstacleFlags)
  {
    // summary:
    //		Sets the segment arrays of the link. 
    // description:
    //		Sets the segment arrays of the link. 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    this._segments = segments;
    this._prevObstacle = prevObstacleFlags;
    this._nextObstacle = nextObstacleFlags;
  },

  startsAtObstacle : function(i)
  {
    // summary:
    //		Returns true if the ith segment starts at an obstacle, i.e. 
    // description:
    //		Returns true if the ith segment starts at an obstacle, i.e. 
    //		we cannot enlarge it at its side that is closer to the start point
    //		on the path. This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    if (i < 0 || i >= this._prevObstacle.length) {
      return (true); // Boolean
    }
    return (this._prevObstacle[i]); // Boolean
  },

  endsAtObstacle : function(i)
  {
    // summary:
    //		Returns true if the ith segment end at an obstacle, i.e. 
    // description:
    //		Returns true if the ith segment end at an obstacle, i.e. 
    //		we cannot enlarge it at its side that is closer to the end point on
    //		the path. This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    if (i < 0 || i >= this._nextObstacle.length) {
      return (true); // Boolean
    }
    return (this._nextObstacle[i]); // Boolean
  },

  needsRoute : function()
  {
    // summary:
    //		Returns true if the link needs a route.
    // tags:
    //		private
    
    return (this._segments == null); // Boolean
  },

  isFrozen : function()
  {
    // summary:
    //		Returns true if the link is frozen.
    // tags:
    //		private
    
    return this._frozen; // Boolean
  },

  setFrozen : function(frozen)
  {
    // summary:
    //		Sets the link to frozen. 
    // description:
    //		Sets the link to frozen. 
    //		This means the link is not routed. This is used for incremental
    //		layout for all those links that need not be rerouted.
    // tags:
    //		private
    
    this._frozen = frozen;
  },

  getSortValue : function()
  {
    // summary:
    //		Returns the sort value of the link.
    // tags:
    //		private
    
    return this._sortValue; // Number
  },

  setSortValue : function(value)
  {
    // summary:
    //		Sets the sort value of the link.
    // tags:
    //		private
    
    this._sortValue = value;
  },

  getPreferedSide : function()
  {
    // summary:
    //		Returns the prefered side of the link (in link style DIRECT_STYLE).
    // tags:
    //		private
    
    return this._preferedSide; // int
  },

  setPreferedSide : function(side)
  {
    // summary:
    //		Sets the prefered side of the link.
    // tags:
    //		private
    
    this._preferedSide = side;
  },

  markLinkChanged : function()
  {
    // summary:
    //		Mark the link as changed, i.e. 
    // description:
    //		Mark the link as changed, i.e. 
    //		clear its bounding box.
    // tags:
    //		private
    
    this._bbox = null;
  },

  getBounds : function()
  {
    // summary:
    //		Returns the bounds of the link. 
    // description:
    //		Returns the bounds of the link. 
    //		The horizontal bounds are &#95;bbox[0] and &#95;bbox[1] The vertical bounds
    //		are &#95;bbox[2] and &#95;bbox[3] Note that result should not be modified
    //		directly.
    // tags:
    //		private
    
    if (this._bbox == null) {
      this.updateBoundingBox();
    }
    return this._bbox; // Array
  },

  mayIntersectLink : function(link)
  {
    // summary:
    //		Return true if this link may intersect the input link. 
    // description:
    //		Return true if this link may intersect the input link. 
    //		This is only relevant in orthogonal link style.  
    // tags:
    //		private
    
    if (this._bbox == null) {
      this.updateBoundingBox();
    }
    if (link._bbox == null) {
      link.updateBoundingBox();
    }
    return (this._bbox[0] <= link._bbox[1] && link._bbox[0] <= this._bbox[1] && this._bbox[2] <= link._bbox[3] && link._bbox[2] <= this._bbox[3]); // Boolean
  },

  mayIntersectBounds : function(inputBox)
  {
    // summary:
    //		Return true if this link may intersect the input bounding box. 
    // description:
    //		Return true if this link may intersect the input bounding box. 
    //		This is only relevant in orthogonal link style.  
    // tags:
    //		private
    
    if (this._bbox == null) {
      this.updateBoundingBox();
    }
    return (this._bbox[0] <= inputBox[1] && inputBox[0] <= this._bbox[1] && this._bbox[2] <= inputBox[3] && inputBox[2] <= this._bbox[3]); // Boolean
  },

  updateBoundingBox : function()
  {
    // summary:
    //		Update the cached bounding box of this link. 
    // description:
    //		Update the cached bounding box of this link. 
    //		This is only relevant in orthogonal link style.
    // tags:
    //		private
    
    this._bbox = TranslateUtil.newArray(4, 0);

    this._bbox[0] = /*Number.MAX_VALUE*/2147483647;
    this._bbox[1] = -1;
    this._bbox[2] = /*Number.MAX_VALUE*/2147483647;
    this._bbox[3] = -1;
    var point = TranslateUtil.newArray(2, 0);
    var i = 0;

    while (this.getNextBend(point, i++))
    {
      if (point[0] < this._bbox[0]) {
        this._bbox[0] = point[0];
      }
      if (point[0] > this._bbox[1]) {
        this._bbox[1] = point[0];
      }
      if (point[1] < this._bbox[2]) {
        this._bbox[2] = point[1];
      }
      if (point[1] > this._bbox[3]) {
        this._bbox[3] = point[1];
      }
    }
  },

  cleanRoute : function()
  {
    // summary:
    //		Clean the routing.
    // tags:
    //		private
    
    this._segments = null;
    this._prevObstacle = null;
    this._nextObstacle = null;
    this.markLinkChanged();
  },

  storeRoute : function(points)
  {
    // summary:
    //		Store the routing. 
    // description:
    //		Store the routing. 
    //		The input vector is the reversed vector of grid points, i.e. the
    //		intermediate bend points on the path from end to start. Note: The
    //		start and end point are NOT intermediate points.
    // tags:
    //		private
    
    var numSegments = 1;
    if (points != null) {
      numSegments = points.length + 1;
    }
    this._segments = TranslateUtil.newArray(numSegments, 0);
    this._prevObstacle = TranslateUtil.newArray(numSegments, false);
    this._nextObstacle = TranslateUtil.newArray(numSegments, false);
    var i = numSegments - 1;
    var dir = -1;
    var p;
    var p1 = TranslateUtil.newArray(2, 0);
    var p2 = TranslateUtil.newArray(2, 0);
    var grid = this.getGraph().getGrid();
    var segment;

    var endDirSame = true;
    this.calcRealEndPointOnGrid(p2);

    if (points != null)
    {
      var ea =  new TranslateUtil.ArrayIterator(points);

      while (ea.hasNext())
      {
        p = ea.next();
        p1[0] = Math.floor(p.x);
        p1[1] = Math.floor(p.y);

        if ((p1[0] != p2[0]) || (p1[1] != p2[1]))
        {
          if (p1[0] == p2[0]) {
            dir = 1;
          }
          else {
            dir = 0;
          }
          break;
        }

        p2[0] = p1[0];
        p2[1] = p1[1];
        endDirSame = !endDirSame;
      }
    }

    this.calcRealStartPointOnGrid(p1);

    if (dir == -1)
    {
      if (p1[0] == p2[0]) {
        dir = 1;
      }
      else {
        dir = 0;
      }
    }

    if (!endDirSame) {
      dir = 1 - dir;
    }
    this.calcRealEndPointOnGrid(p2);

    if (points != null)
    {
      var eb =  new TranslateUtil.ArrayIterator(points);

      while (eb.hasNext())
      {
        p = eb.next();
        p1[0] = Math.floor(p.x);
        p1[1] = Math.floor(p.y);

        this._segments[i] = p2[dir] - p1[dir];
        this._nextObstacle[i] = true;
        if (0 <= p1[1 - dir] && p1[1 - dir] < grid.getNumberOfGridLines(1 - dir)) {
          segment = grid.getGridLine(1 - dir, p1[1 - dir]).searchSegment(p2[dir]);
        }
        else {
          segment = null;
        }
        if (segment != null)
        {
          if (p1[dir] < p2[dir])
          {
            if (segment.getEndIndex() > p2[dir]) {
              this._nextObstacle[i] = false;
            }
          }
          else if (p1[dir] > p2[dir])
          {
            if (segment.getStartIndex() < p2[dir]) {
              this._nextObstacle[i] = false;
            }
          }
        }

        this._prevObstacle[i] = true;
        if (0 <= p1[1 - dir] && p1[1 - dir] < grid.getNumberOfGridLines(1 - dir)) {
          segment = grid.getGridLine(1 - dir, p1[1 - dir]).searchSegment(p1[dir]);
        }
        else {
          segment = null;
        }
        if (segment != null)
        {
          if (p1[dir] < p2[dir])
          {
            if (segment.getStartIndex() < p1[dir]) {
              this._prevObstacle[i] = false;
            }
          }
          else if (p1[dir] > p2[dir])
          {
            if (segment.getEndIndex() > p1[dir]) {
              this._prevObstacle[i] = false;
            }
          }
        }

        dir = 1 - dir;
        p2[0] = p1[0];
        p2[1] = p1[1];
        i--;
      }
    }

    this.calcRealStartPointOnGrid(p1);

    this._segments[0] = p2[dir] - p1[dir];
    this._startDir = dir;
    this._nextObstacle[0] = true;
    if (0 <= p1[1 - dir] && p1[1 - dir] < grid.getNumberOfGridLines(1 - dir)) {
      segment = grid.getGridLine(1 - dir, p1[1 - dir]).searchSegment(p2[dir]);
    }
    else {
      segment = null;
    }
    if (segment != null)
    {
      if (p1[dir] < p2[dir])
      {
        if (segment.getEndIndex() > p2[dir]) {
          this._nextObstacle[0] = false;
        }
      }
      else if (p1[dir] > p2[dir])
      {
        if (segment.getStartIndex() < p2[dir]) {
          this._nextObstacle[0] = false;
        }
      }
    }

    this._prevObstacle[0] = true;
    if (0 <= p1[1 - dir] && p1[1 - dir] < grid.getNumberOfGridLines(1 - dir)) {
      segment = grid.getGridLine(1 - dir, p1[1 - dir]).searchSegment(p1[dir]);
    }
    else {
      segment = null;
    }
    if (segment != null)
    {
      if (p1[dir] < p2[dir])
      {
        if (segment.getStartIndex() < p1[dir]) {
          this._prevObstacle[0] = false;
        }
      }
      else if (p1[dir] > p2[dir])
      {
        if (segment.getEndIndex() > p1[dir]) {
          this._prevObstacle[0] = false;
        }
      }
    }

    this.cleanUnusedStartAndEndPoints();
    this.markLinkChanged();
  },

  calcRealStartPointOnGrid : function(p)
  {
    // summary:
    //		Retrieve the real start point as grid index.
    // tags:
    //		private
    
    var grid = this.getGraph().getGrid();
    var terminationPoint = this.getActStartPoint();
    var segment = terminationPoint.getSegment();
    var fromPoint = this.getFromPoint();
    var startCoord;
    var dir;

    var isHorizontal = (this.getSegmentStartDir() == 0);
    if (segment != null && segment.getGridLine() != null) {
      isHorizontal = segment.isHorizontal();
    }
    if (isHorizontal)
    {
      if (fromPoint == null) {
        startCoord = this._fromRect.x + 0.5 * this._fromRect.width;
      }
      else {
        startCoord = fromPoint.x;
      }
      p[1] = terminationPoint.getIndex(1);
      dir = 0;
    }
    else
    {
      if (fromPoint == null) {
        startCoord = this._fromRect.y + 0.5 * this._fromRect.height;
      }
      else {
        startCoord = fromPoint.y;
      }
      p[0] = terminationPoint.getIndex(0);
      dir = 1;
    }

    var offset = this._gridBaseCoord[dir] - grid.getMinCoord(dir);
    var h = grid.getGridCoord(dir, terminationPoint.getIndex(dir)) + offset;
    p[dir] = grid.getGridIndex(dir, startCoord - offset, (h >= startCoord));
  },

  calcRealEndPointOnGrid : function(p)
  {
    // summary:
    //		Retrieve the real end point as grid index.
    // tags:
    //		private
    
    var grid = this.getGraph().getGrid();
    var terminationPoint = this.getActEndPoint();
    var segment = terminationPoint.getSegment();
    var toPoint = this.getToPoint();
    var endCoord;
    var dir;

    var isHorizontal = (this.getSegmentEndDir() == 0);
    if (segment != null && segment.getGridLine() != null) {
      isHorizontal = segment.isHorizontal();
    }
    if (isHorizontal)
    {
      if (toPoint == null) {
        endCoord = this._toRect.x + 0.5 * this._toRect.width;
      }
      else {
        endCoord = toPoint.x;
      }
      p[1] = terminationPoint.getIndex(1);
      dir = 0;
    }
    else
    {
      if (toPoint == null) {
        endCoord = this._toRect.y + 0.5 * this._toRect.height;
      }
      else {
        endCoord = toPoint.y;
      }
      p[0] = terminationPoint.getIndex(0);
      dir = 1;
    }

    var offset = this._gridBaseCoord[dir] - grid.getMinCoord(dir);
    var h = grid.getGridCoord(dir, terminationPoint.getIndex(dir)) + offset;
    p[dir] = grid.getGridIndex(dir, endCoord - offset, (h >= endCoord));
  },

  updateGrid : function()
  {
    // summary:
    //		Update the grid with the link.
    // tags:
    //		private
    
    if (this.needsRoute() || this.isFrozen() || this.isDirect()) {
      return;
    }
    this.updateGridImpl();
  },

  updateGridIfIntersects : function(bounds)
  {
    // summary:
    //		Updates the grid with the link if the link intersects the input
    //		bounds. 
    // description:
    //		Updates the grid with the link if the link intersects the input
    //		bounds. 
    //		The horizontal bounds are bounds[0] and bounds[1] The vertical
    //		bounds are bounds[2] and bounds[3]
    // tags:
    //		private
    
    if (this.needsRoute() || this.isFrozen() || this.isDirect()) {
      return;
    }
    if (!this.mayIntersectBounds(bounds)) {
      return;
    }
    this.updateGridImpl();
  },

  updateGridImpl : function()
  {
    // summary:
    //		Internal implementation of update grid.
    // tags:
    //		private
    
    var grid = this.getGraph().getGrid();
    var point = TranslateUtil.newArray(2, 0);
    var p1 = new Point();
    var p2 = new Point();
    var i = 0;
    this.getNextBend(point, i++);
    p1.x = grid.getGridCoord(0, point[0]);
    p1.y = grid.getGridCoord(1, point[1]);

    while (this.getNextBend(point, i++))
    {
      p2.x = grid.getGridCoord(0, point[0]);
      p2.y = grid.getGridCoord(1, point[1]);
      grid.setObstacle(p1, p2);
      p1.x = p2.x;
      p1.y = p2.y;
    }
  },

  reshapeLink : function(updateGrid, redraw)
  {
    // summary:
    //		Reshape the link according to the calculated points. 
    // description:
    //		Reshape the link according to the calculated points. 
    //		If "updateGrid" is true, we update the grid with the new link.
    // tags:
    //		private
    
    if (this.isFrozen()) {
      return;
    }
    if (this.needsRoute())
    {
      return;
    }

    var graph = this.getGraph();
    var layout = graph.getLayout();
    var model = layout.getGraphModel();
    var linkObject = this.getLinkObject();
    var fromPoint = this.getFromPoint();
    var toPoint = this.getToPoint();
    var grid = graph.getGrid();
    var i;
    var startX;
    var endX;
    var startY;
    var endY;
    var numPoints;
    var pointArray;
    var needArtificalSplit = false;
    var startSegmentIsHorizontal;
    var endSegmentIsHorizontal;
    var x;
    var y;

    if (this.isDirect())
    {
      var startPoint = this.getActStartPoint();
      var endPoint = this.getActEndPoint();

      var startSide = startPoint.getSide();

      var endSide = endPoint.getSide();
      startSegmentIsHorizontal = (startSide == Direction.LEFT || startSide == Direction.RIGHT);
      endSegmentIsHorizontal = (endSide == Direction.LEFT || endSide == Direction.RIGHT);
      startX = startPoint.getIndex(0);
      startY = startPoint.getIndex(1);
      endX = endPoint.getIndex(0);
      endY = endPoint.getIndex(1);
      startX += this.getGridCorrection(0);
      startY += this.getGridCorrection(1);
      endX += this.getGridCorrection(0);
      endY += this.getGridCorrection(1);

      numPoints = 0;

      if (startX == endX)
      {
        if (startSegmentIsHorizontal) {
          numPoints++;
        }
        if (endSegmentIsHorizontal) {
          numPoints++;
        }
      }
      else if (startY == endY)
      {
        if (!startSegmentIsHorizontal) {
          numPoints++;
        }
        if (!endSegmentIsHorizontal) {
          numPoints++;
        }
      }
      else {
        numPoints = 2;
      }
      if (numPoints == 0 && fromPoint != null && toPoint != null && fromPoint.x != toPoint.x && fromPoint.y != toPoint.y)
      {
        needArtificalSplit = true;
        numPoints = 2;
      }

      pointArray = TranslateUtil.newArray(numPoints, null);

      if (needArtificalSplit || numPoints == 2)
      {
        x = grid.getGridCoord(0, startX);
        y = grid.getGridCoord(1, startY);
        pointArray[0] = new Point(x, y);
        x = grid.getGridCoord(0, endX);
        y = grid.getGridCoord(1, endY);
        pointArray[1] = new Point(x, y);
      }
      else if (startX == endX)
      {
        if (startSegmentIsHorizontal)
        {
          x = grid.getGridCoord(0, startX);
          y = grid.getGridCoord(1, startY);
        }
        else
        {
          x = grid.getGridCoord(0, endX);
          y = grid.getGridCoord(1, endY);
        }

        if (numPoints == 1) {
          pointArray[0] = new Point(x, y);
        }
      }
      else if (startY == endY)
      {
        if (!startSegmentIsHorizontal)
        {
          x = grid.getGridCoord(0, startX);
          y = grid.getGridCoord(1, startY);
        }
        else
        {
          x = grid.getGridCoord(0, endX);
          y = grid.getGridCoord(1, endY);
        }

        if (numPoints == 1) {
          pointArray[0] = new Point(x, y);
        }
      }
    }
    else
    {
      numPoints = this._segments.length - 1;

      if (numPoints == 0 && fromPoint != null && toPoint != null && fromPoint.x != toPoint.x && fromPoint.y != toPoint.y)
      {
        needArtificalSplit = true;
        numPoints = 2;
      }

      pointArray = TranslateUtil.newArray(numPoints, null);
      startSegmentIsHorizontal = (this.getSegmentStartDir() == 0);
      endSegmentIsHorizontal = (this.getSegmentEndDir() == 0);

      if (needArtificalSplit)
      {
        var middle;

        if (startSegmentIsHorizontal)
        {
          middle = (fromPoint.x + toPoint.x) / 2.0;
          pointArray[0] = new Point(middle, fromPoint.y);
          pointArray[1] = new Point(middle, toPoint.y);
        }
        else
        {
          middle = (fromPoint.y + toPoint.y) / 2.0;
          pointArray[0] = new Point(fromPoint.x, middle);
          pointArray[1] = new Point(toPoint.x, middle);
        }
      }

      var point = TranslateUtil.newArray(2, 0);
      this.getNextBend(point, 0);
      startX = point[0];
      startY = point[1];

      if (!needArtificalSplit)
      {
        i = 0;

        while (i < numPoints && this.getNextBend(point, i + 1))
        {
          x = grid.getGridCoord(0, point[0]);
          y = grid.getGridCoord(1, point[1]);
          pointArray[i] = new Point(x, y);
          i++;
        }
      }

      this.getNextBend(point, numPoints + 1);
      endX = point[0];
      endY = point[1];
    }

    var otherX;
    var otherY;

    if (fromPoint == null)
    {
      fromPoint = new Point();

      if (numPoints > 0)
      {
        otherX = pointArray[0].x;
        otherY = pointArray[0].y;
      }
      else
      {
        otherX = grid.getGridCoord(0, endX);
        otherY = grid.getGridCoord(1, endY);
      }

      if (startSegmentIsHorizontal)
      {
        fromPoint.y = otherY;
        if (otherX > this._fromRect.x) {
          fromPoint.x = this._fromRect.x + this._fromRect.width;
        }
        else {
          fromPoint.x = this._fromRect.x;
        }
      }
      else
      {
        fromPoint.x = otherX;
        if (otherY > this._fromRect.y) {
          fromPoint.y = this._fromRect.y + this._fromRect.height;
        }
        else {
          fromPoint.y = this._fromRect.y;
        }
      }
    }

    if (toPoint == null)
    {
      toPoint = new Point();

      if (numPoints > 0)
      {
        otherX = pointArray[numPoints - 1].x;
        otherY = pointArray[numPoints - 1].y;
      }
      else
      {
        otherX = grid.getGridCoord(0, startX);
        otherY = grid.getGridCoord(1, startY);
      }

      if (endSegmentIsHorizontal)
      {
        toPoint.y = otherY;
        if (otherX > this._toRect.x) {
          toPoint.x = this._toRect.x + this._toRect.width;
        }
        else {
          toPoint.x = this._toRect.x;
        }
      }
      else
      {
        toPoint.x = otherX;
        if (otherY > this._toRect.y) {
          toPoint.y = this._toRect.y + this._toRect.height;
        }
        else {
          toPoint.y = this._toRect.y;
        }
      }
    }

    if (numPoints > 0)
    {
      if (startSegmentIsHorizontal) {
        pointArray[0].y = fromPoint.y;
      }
      else {
        pointArray[0].x = fromPoint.x;
      }
      if (endSegmentIsHorizontal) {
        pointArray[numPoints - 1].y = toPoint.y;
      }
      else {
        pointArray[numPoints - 1].x = toPoint.x;
      }
    }

    var isFromPointFixed = layout.isFromPointFixed(linkObject);
    var isToPointFixed = layout.isToPointFixed(linkObject);

    var linkStyle = ((this.isDirect() ? LinkStyles.DIRECT_STYLE : LinkStyles.ORTHOGONAL_STYLE));
    GraphModelUtil.reshapeLink(model, layout, linkObject, linkStyle, (isFromPointFixed ? ((fromPoint != null ? new FixedPoint(fromPoint .x, fromPoint .y) : null)) : fromPoint), pointArray, 0,
      numPoints, (isToPointFixed ? ((toPoint != null ? new FixedPoint(toPoint .x, toPoint .y) : null)) : toPoint));

    if (updateGrid)
    {
      if (numPoints > 0)
      {
        grid.setObstacle(fromPoint, pointArray[0]);
        grid.setObstacle(pointArray[numPoints - 1], toPoint);

        for (i = 1; i < numPoints; i++)
          grid.setObstacle(pointArray[i - 1], pointArray[i]);
      }
      else {
        grid.setObstacle(fromPoint, toPoint);
      }
    }
  },

  drawLinkForAnimation : function()
  {
    // summary:
    //		Reshape the link if animation is on.
    // tags:
    //		private
    
    if (this.isFrozen() || this.needsRoute()) {
      return;
    }
    var graph = this.getGraph();
    var layout = graph.getLayout();
    var model = layout.getGraphModel();
    TranslateUtil.noop();

    try
    {
      this.reshapeLink(false, true);
    }
    finally
    {
      TranslateUtil.noop();

      try
      {
        Thread.sleep(100);
      }
      catch (e)
      {
      }
    }
  },

  isWithinFromNodeRange : function(dir, gridIndex, minCornerOffset)
  {
    // summary:
    //		Return true if the input index in input dir is in the range of the
    //		from node of this link.
    // tags:
    //		private
    
    return (this.isWithinRectRange(this._fromRect, dir, gridIndex, minCornerOffset)); // Boolean
  },

  isWithinToNodeRange : function(dir, gridIndex, minCornerOffset)
  {
    // summary:
    //		Return true if the input index in input dir is in the range of the
    //		to node of this link.
    // tags:
    //		private
    
    return (this.isWithinRectRange(this._toRect, dir, gridIndex, minCornerOffset)); // Boolean
  },

  isWithinRectRange : function(rect, dir, gridIndex, minCornerOffset)
  {
    // summary:
    //		Return true if the input index in input dir is in the range of the
    //		input rect.
    // tags:
    //		private
    
    var grid = this.getGraph().getGrid();
    var minCoord;
    var maxCoord;

    if (dir == 0)
    {
      if (rect.width >= 0.0)
      {
        minCoord = rect.x + minCornerOffset;
        maxCoord = rect.x + rect.width - minCornerOffset;
      }
      else
      {
        minCoord = rect.x + rect.width + minCornerOffset;
        maxCoord = rect.x - minCornerOffset;
      }
    }
    else
    {
      if (rect.height >= 0.0)
      {
        minCoord = rect.y + minCornerOffset;
        maxCoord = rect.y + rect.height - minCornerOffset;
      }
      else
      {
        minCoord = rect.y + rect.height + minCornerOffset;
        maxCoord = rect.y - minCornerOffset;
      }
    }

    var minIndex = grid.getGridIndex(dir, minCoord, true);
    var maxIndex = grid.getGridIndex(dir, maxCoord, false);
    if (gridIndex < minIndex) {
      return (false); // Boolean
    }
    if (gridIndex > maxIndex) {
      return (false); // Boolean
    }
    return (true); // Boolean
  }

}
);

return LLLink;
});

},
'ibm_ilog/graphlayout/shortlink/_LinkShapeType':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_LinkShapeType", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/Point",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    Point,	// #6
    TranslateUtil	// #7
    ) {

var _LinkShapeType = declare("ibm_ilog.graphlayout.shortlink._LinkShapeType", null, 
{
  // summary:
  //		A class that is used to identify the types of link shapes produced by
  //		a link shape producer. 
  // description:
  //		A class that is used to identify the types of link shapes produced by
  //		a link shape producer. 
  //		There are a lot of predefined subclasses, but note that just one
  //		instance of each one is created (they follow the singleton pattern). 
  //		
  // tags:
  //		extension
  
  constructor : function()
  {
  },
  
  restoreIntermediatePoints : function(linkData, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    // summary:
    //		Restores the intermediate points.  
    // tags:
    //		private
    
    var fromPoint = linkData.getConnectionPoint(true);
    var toPoint = linkData.getConnectionPoint(false);

    this.doComputeIntermediatePoints(linkData, fromRect, toRect, fromSide, toSide, fromPoint, toPoint, minFinalSegment);

    linkData.getLinkShape().setNumberOfPoints(this.getNumberOfBends() + 2);
  },

  computeIntermediatePoints : function(layout, linkData, minFinalSegment, reversed)
  {
    // summary:
    //		Computes the intermediate points.
    
    minFinalSegment += linkData.getLinkWidth() * 0.5;

    var fromNodeData = (reversed ? linkData.getToNode() : linkData.getFromNode());
    var toNodeData = (reversed ? linkData.getFromNode() : linkData.getToNode());

    var fromSide = linkData.getFromNodeSide();
    var toSide = linkData.getToNodeSide();

    var fromRect = layout.getTempRect1();
    var toRect = layout.getTempRect2();
    var rect = fromNodeData.getLinkConnectionRect();
    TranslateUtil.reshapeRect(fromRect, rect.x, rect.y, rect.width, rect.height);
    rect = toNodeData.getLinkConnectionRect();
    TranslateUtil.reshapeRect(toRect, rect.x, rect.y, rect.width, rect.height);

    var fromPoint = linkData.getConnectionPoint(!reversed);
    var toPoint = linkData.getConnectionPoint(reversed);
    if (!linkData.isFromPointFixed()) {
      fromSide.moveToDefaultConnectionPoint(fromPoint, fromRect);
    }
    if (!linkData.isToPointFixed()) {
      toSide.moveToDefaultConnectionPoint(toPoint, toRect);
    }
    if (linkData.isFromPointFixed()) {
      TranslateUtil.addPoint(fromRect, fromPoint .x, fromPoint .y);
    }
    if (linkData.isToPointFixed()) {
      TranslateUtil.addPoint(toRect, toPoint .x, toPoint .y);
    }
    this.restoreIntermediatePoints(linkData, fromRect, toRect, fromSide, toSide, minFinalSegment);

    fromSide.setLayoutUpToDate(false);
    toSide.setLayoutUpToDate(false);
  },

  doComputeIntermediatePoints : function(linkData, fromRect, toRect, fromSide, toSide, fromPoint, toPoint, minFinalSegment)
  {
    // summary:
    //		Computes the intermediate points. 
    // description:
    //		Computes the intermediate points. 
    //		Only called from the base class itself.  
    // tags:
    //		protected
    },
  
  getOrCreatePoint : function(bends, index)
  {
    var point = bends[index];

    if (point == null)
    {
      point = new Point();
      bends[index] = point;
    }

    return point; // Point
  },

  getReversedShapeType : function()
  {
    // summary:
    //		Returns the shape type that corresponds to the reversed shape. 
    // description:
    //		Returns the shape type that corresponds to the reversed shape. 
    //		For instance, OneBendTB is the reversed shape of OneBendBT.
    },
  
  getFromNodeSide : function(linkData)
  {
    // summary:
    //		Returns the NodeSide to which the origin of the link having this
    //		shape type must be connected on nodeData (which is assumed to be its
    //		origin node).
    },
  
  getToNodeSide : function(linkData)
  {
    // summary:
    //		Returns the NodeSide to which the destination of the link having
    //		this shape type must be connected on nodeData (which is assumed to
    //		be its origin node).
    },
  
  getFromSide : function(linkData)
  {
    // summary:
    
    return this.getFromNodeSide(linkData).getSide(); // int
  },

  getToSide : function(linkData)
  {
    // summary:
    
    return this.getToNodeSide(linkData).getSide(); // int
  },

  isOrthogonal : function()
  {
    // summary:
    //		Returns true if this is an orthogonal shape and false otherwise.
    },
  
  getNumberOfBends : function()
  {
    // summary:
    //		Returns the number of bends (connection points are not counted).
    }
  
}
);

lang.mixin(_LinkShapeType,
{

  getXMiddle : function(fromPoint, toPoint, fromRect, toRect)
  {
    var xMiddle = 0;
    if (fromRect.x > (toRect.x + toRect.width)) {
      xMiddle = (fromRect.x + toRect.x + toRect.width) * 0.5;
    }
    else if (toRect.x > (fromRect.x + fromRect.width)) {
      xMiddle = (toRect.x + fromRect.x + fromRect.width) * 0.5;
    }
    else {
      xMiddle = (fromPoint.x + toPoint.x) * 0.5;
    }
    return xMiddle; // Number
  },

  getYMiddle : function(fromPoint, toPoint, fromRect, toRect)
  {
    var yMiddle = 0;
    if (fromRect.y > (toRect.y + toRect.height)) {
      yMiddle = (fromRect.y + toRect.y + toRect.height) * 0.5;
    }
    else if (toRect.y > (fromRect.y + fromRect.height)) {
      yMiddle = (toRect.y + fromRect.y + fromRect.height) * 0.5;
    }
    else {
      yMiddle = (fromPoint.y + toPoint.y) * 0.5;
    }
    return yMiddle; // Number
  }

}
);

_DirectLinkShapeType = declare(_LinkShapeType, 
{

  constructor : function()
  {
    
  },

  doComputeIntermediatePoints : function(linkData, fromRect, toRect, fromSide, toSide, fromPoint, toPoint, minFinalSegment)
  {
    // summary:
    //		
    // tags:
    //		protected
    
    var bends = linkData.getLinkShape().getIntermediatePoints();
    var point = this.getOrCreatePoint(bends, 0);
    TranslateUtil.setPointLocation(point, fromPoint);
    fromSide.translateOutside(point, minFinalSegment);
    point = this.getOrCreatePoint(bends, 1);
    TranslateUtil.setPointLocation(point, toPoint);
    toSide.translateOutside(point, minFinalSegment);
  },

  isOrthogonal : function()
  {
    return false; // Boolean
  },

  getNumberOfBends : function()
  {
    // summary:
    //		Returns the number of bends (connection points are not counted).
    
    return 2; // int
  }

}
);
_LinkShapeType.DirectTL = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectLT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectTL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectTL;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectTL();
  }
  
}
);

_LinkShapeType.DirectTB = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectBT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectTB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectTB;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectTB();
  }
  
}
);

_LinkShapeType.DirectTR = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectRT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectTR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectTR;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectTR();
  }
  
}
);

_LinkShapeType.DirectRL = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectLR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectRL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectRL;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectRL();
  }
  
}
);

_LinkShapeType.DirectRB = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectBR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectRB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectRB;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectRB();
  }
  
}
);

_LinkShapeType.DirectBT = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectTB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectBT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectBT;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectBT();
  }
  
}
);

_LinkShapeType.DirectBL = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectLB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectBL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectBL;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectBL();
  }
  
}
);

_LinkShapeType.DirectBR = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectRB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectBR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectBR;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectBR();
  }
  
}
);

_LinkShapeType.DirectRT = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectTR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectRT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectRT;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectRT();
  }
  
}
);

_LinkShapeType.DirectLR = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectRL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectLR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectLR;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectLR();
  }
  
}
);

_LinkShapeType.DirectLT = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectTL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectLT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectLT;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectLT();
  }
  
}
);

_LinkShapeType.DirectLB = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.DirectBL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectLB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectLB;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectLB();
  }
  
}
);

_LinkShapeType.DirectTT = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return this; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectTT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectTT;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectTT();
  }
  
}
);

_LinkShapeType.DirectBB = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return this; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectBB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectBB;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectBB();
  }
  
}
);

_LinkShapeType.DirectRR = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return this; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectRR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectRR;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectRR();
  }
  
}
);

_LinkShapeType.DirectLL = declare(_DirectLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return this; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.DirectLL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.DirectLL;
    if(_DirectLinkShapeType._fields_init)
      _DirectLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.DirectLL();
  }
  
}
);

_OrthogonalLinkShapeType = declare(_LinkShapeType, 
{

  constructor : function()
  {
    
  },
  isOrthogonal : function()
  {
    return true; // Boolean
  }

}
);
_OneBendLinkShapeType = declare(_OrthogonalLinkShapeType, 
{

  constructor : function()
  {
    
  },

  getNumberOfBends : function()
  {
    // summary:
    //		Returns the number of bends (connection points are not counted).
    
    return 1; // int
  },

  doComputeIntermediatePoints : function(linkData, fromRect, toRect, fromSide, toSide, fromPoint, toPoint, minFinalSegment)
  {
    // summary:
    //		
    // tags:
    //		protected
    
    var point = this.getOrCreatePoint(linkData.getLinkShape().getIntermediatePoints(), 0);
    fromSide.setTangentialCoord(point, fromSide.getTangentialCoord(fromPoint));
    toSide.setTangentialCoord(point, toSide.getTangentialCoord(toPoint));
  }

}
);
_LinkShapeType.OneBendTL = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendLT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendTL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendTL;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendTL();
  }
  
}
);

_LinkShapeType.OneBendTR = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendRT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendTR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendTR;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendTR();
  }
  
}
);

_LinkShapeType.OneBendBL = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendLB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendBL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendBL;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendBL();
  }
  
}
);

_LinkShapeType.OneBendBR = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendRB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendBR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendBR;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendBR();
  }
  
}
);

_LinkShapeType.OneBendLT = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendTL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendLT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendLT;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendLT();
  }
  
}
);

_LinkShapeType.OneBendLB = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendBL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendLB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendLB;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendLB();
  }
  
}
);

_LinkShapeType.OneBendRT = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendTR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendRT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendRT;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendRT();
  }
  
}
);

_LinkShapeType.OneBendRB = declare(_OneBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.OneBendBR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.OneBendRB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.OneBendRB;
    if(_OneBendLinkShapeType._fields_init)
      _OneBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.OneBendRB();
  }
  
}
);

_LinkShapeType.TwoBendLinkShapeType = declare(_OrthogonalLinkShapeType, 
{

  constructor : function()
  {
    
  },

  getNumberOfBends : function()
  {
    // summary:
    //		Returns the number of bends (connection points are not counted).
    
    return 2; // int
  },

  doComputeIntermediatePoints : function(linkData, fromRect, toRect, fromSide, toSide, fromPoint, toPoint, minFinalSegment)
  {
    // summary:
    //		
    // tags:
    //		protected
    
    var bends = linkData.getLinkShape().getIntermediatePoints();
    var point0 = this.getOrCreatePoint(bends, 0);
    TranslateUtil.setPointLocation(point0, fromPoint);
    var point1 = this.getOrCreatePoint(bends, 1);
    TranslateUtil.setPointLocation(point1, toPoint);

    if (fromSide.getDirection() == toSide.getDirection())
    {
      if (fromSide.getPointRadialDelta(point0, point1) <= 0)
      {
        fromSide.translateOutside(point0, minFinalSegment);
        fromSide.copyRadialCoord(point0, point1);
      }
      else
      {
        toSide.translateOutside(point1, minFinalSegment);
        fromSide.copyRadialCoord(point1, point0);
      }
    }
    else
    {
      var coord = (fromSide.getPointRadialCoord(fromPoint) + toSide.getPointRadialCoord(toPoint)) * 0.5;
      fromSide.setRadialCoord(point0, coord);
      toSide.setRadialCoord(point1, coord);
    }
  }

}
);
_LinkShapeType.TwoBendSameSide = declare(_LinkShapeType.TwoBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return this; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  }

}
);
_LinkShapeType.TwoBendTT = declare(_LinkShapeType.TwoBendSameSide, 
{

  constructor : function()
  {
    
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendTT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendTT;
    if(_LinkShapeType.TwoBendSameSide._fields_init)
      _LinkShapeType.TwoBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendTT();
  }
  
}
);

_LinkShapeType.TwoBendBB = declare(_LinkShapeType.TwoBendSameSide, 
{

  constructor : function()
  {
    
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendBB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendBB;
    if(_LinkShapeType.TwoBendSameSide._fields_init)
      _LinkShapeType.TwoBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendBB();
  }
  
}
);

_LinkShapeType.TwoBendLL = declare(_LinkShapeType.TwoBendSameSide, 
{

  constructor : function()
  {
    
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendLL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendLL;
    if(_LinkShapeType.TwoBendSameSide._fields_init)
      _LinkShapeType.TwoBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendLL();
  }
  
}
);

_LinkShapeType.TwoBendRR = declare(_LinkShapeType.TwoBendSameSide, 
{

  constructor : function()
  {
    
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendRR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendRR;
    if(_LinkShapeType.TwoBendSameSide._fields_init)
      _LinkShapeType.TwoBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendRR();
  }
  
}
);

_LinkShapeType.TwoBendDifferentSide = declare(_LinkShapeType.TwoBendLinkShapeType, 
{

  constructor : function()
  {
    
  }
}
);
_LinkShapeType.TwoBendBT = declare(_LinkShapeType.TwoBendDifferentSide, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.TwoBendTB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendBT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendBT;
    if(_LinkShapeType.TwoBendDifferentSide._fields_init)
      _LinkShapeType.TwoBendDifferentSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendBT();
  }
  
}
);

_LinkShapeType.TwoBendTB = declare(_LinkShapeType.TwoBendDifferentSide, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.TwoBendBT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendTB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendTB;
    if(_LinkShapeType.TwoBendDifferentSide._fields_init)
      _LinkShapeType.TwoBendDifferentSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendTB();
  }
  
}
);

_LinkShapeType.TwoBendRL = declare(_LinkShapeType.TwoBendDifferentSide, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.TwoBendLR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendRL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendRL;
    if(_LinkShapeType.TwoBendDifferentSide._fields_init)
      _LinkShapeType.TwoBendDifferentSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendRL();
  }
  
}
);

_LinkShapeType.TwoBendLR = declare(_LinkShapeType.TwoBendDifferentSide, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.TwoBendRL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.TwoBendLR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.TwoBendLR;
    if(_LinkShapeType.TwoBendDifferentSide._fields_init)
      _LinkShapeType.TwoBendDifferentSide._fields_init(c);
    
    c.instance = new _LinkShapeType.TwoBendLR();
  }
  
}
);

_ThreeBendLinkShapeType = declare(_OrthogonalLinkShapeType, 
{

  constructor : function()
  {
    
  },

  getNumberOfBends : function()
  {
    // summary:
    //		Returns the number of bends (connection points are not counted).
    
    return 3; // int
  },

  isOverlapping : function()
  {
    // summary:
    //		Returns true for shapes of links connecting overlapping end-nodes. 
    // description:
    //		Returns true for shapes of links connecting overlapping end-nodes. 
    //		(Notice this
    //		isn't exactly equivalent to the overlap of the bboxes of the end-nodes - the "bypass distance" parameter is also considered.) The default implementation returns always false.
    // tags:
    //		protected
    
    return false; // Boolean
  },

  doComputeIntermediatePoints : function(linkData, fromRect, toRect, fromSide, toSide, fromPoint, toPoint, minFinalSegment)
  {
    // summary:
    //		
    // tags:
    //		protected
    
    var bends = linkData.getLinkShape().getIntermediatePoints();
    var point0 = this.getOrCreatePoint(bends, 0);
    TranslateUtil.setPointLocation(point0, fromPoint);
    var point2 = this.getOrCreatePoint(bends, 2);
    TranslateUtil.setPointLocation(point2, toPoint);

    if (linkData.isOverlapping() || this.isOverlapping())
    {
      if (fromSide.getRectRadialDelta(point0, toRect) > 0) {
        fromSide.setRadialCoord(point0, fromSide.getRectRadialCoord(toRect));
      }
      if (toSide.getRectRadialDelta(point2, fromRect) > 0) {
        toSide.setRadialCoord(point2, toSide.getRectRadialCoord(fromRect));
      }
    }

    fromSide.translateOutside(point0, minFinalSegment);
    toSide.translateOutside(point2, minFinalSegment);
    var point1 = this.getOrCreatePoint(bends, 1);

    fromSide.setRadialCoord(point1, fromSide.getPointRadialCoord(point0));
    toSide.setRadialCoord(point1, toSide.getPointRadialCoord(point2));
  }

}
);
_LinkShapeType.ThreeBendRT = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendTR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendRT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendRT;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendRT();
  }
  
}
);

_LinkShapeType.ThreeBendRT_OVERLAP = declare(_LinkShapeType.ThreeBendRT, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendTR_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendRT_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendRT_OVERLAP;
    if(_LinkShapeType.ThreeBendRT._fields_init)
      _LinkShapeType.ThreeBendRT._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendRT_OVERLAP();
  }
  
}
);

_LinkShapeType.ThreeBendTR = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendRT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendTR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendTR;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendTR();
  }
  
}
);

_LinkShapeType.ThreeBendTR_OVERLAP = declare(_LinkShapeType.ThreeBendTR, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendRT_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendTR_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendTR_OVERLAP;
    if(_LinkShapeType.ThreeBendTR._fields_init)
      _LinkShapeType.ThreeBendTR._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendTR_OVERLAP();
  }
  
}
);

_LinkShapeType.ThreeBendBL = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendLB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendBL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendBL;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendBL();
  }
  
}
);

_LinkShapeType.ThreeBendBL_OVERLAP = declare(_LinkShapeType.ThreeBendBL, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendLB_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendBL_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendBL_OVERLAP;
    if(_LinkShapeType.ThreeBendBL._fields_init)
      _LinkShapeType.ThreeBendBL._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendBL_OVERLAP();
  }
  
}
);

_LinkShapeType.ThreeBendLB = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendBL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendLB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendLB;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendLB();
  }
  
}
);

_LinkShapeType.ThreeBendLB_OVERLAP = declare(_LinkShapeType.ThreeBendLB, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendBL_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendLB_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendLB_OVERLAP;
    if(_LinkShapeType.ThreeBendLB._fields_init)
      _LinkShapeType.ThreeBendLB._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendLB_OVERLAP();
  }
  
}
);

_LinkShapeType.ThreeBendBR = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendRB.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendBR,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendBR;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendBR();
  }
  
}
);

_LinkShapeType.ThreeBendBR_OVERLAP = declare(_LinkShapeType.ThreeBendBR, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendRB_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendBR_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendBR_OVERLAP;
    if(_LinkShapeType.ThreeBendBR._fields_init)
      _LinkShapeType.ThreeBendBR._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendBR_OVERLAP();
  }
  
}
);

_LinkShapeType.ThreeBendRB = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendBR.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendRB,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendRB;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendRB();
  }
  
}
);

_LinkShapeType.ThreeBendRB_OVERLAP = declare(_LinkShapeType.ThreeBendRB, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendBR_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendRB_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendRB_OVERLAP;
    if(_LinkShapeType.ThreeBendRB._fields_init)
      _LinkShapeType.ThreeBendRB._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendRB_OVERLAP();
  }
  
}
);

_LinkShapeType.ThreeBendTL = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendLT.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendTL,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendTL;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendTL();
  }
  
}
);

_LinkShapeType.ThreeBendTL_OVERLAP = declare(_LinkShapeType.ThreeBendTL, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendLT_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendTL_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendTL_OVERLAP;
    if(_LinkShapeType.ThreeBendTL._fields_init)
      _LinkShapeType.ThreeBendTL._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendTL_OVERLAP();
  }
  
}
);

_LinkShapeType.ThreeBendLT = declare(_ThreeBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendTL.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendLT,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendLT;
    if(_ThreeBendLinkShapeType._fields_init)
      _ThreeBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendLT();
  }
  
}
);

_LinkShapeType.ThreeBendLT_OVERLAP = declare(_LinkShapeType.ThreeBendLT, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.ThreeBendTL_OVERLAP.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  isOverlapping : function()
  {
    return true; // Boolean
  }

}
);

lang.mixin(_LinkShapeType.ThreeBendLT_OVERLAP,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.ThreeBendLT_OVERLAP;
    if(_LinkShapeType.ThreeBendLT._fields_init)
      _LinkShapeType.ThreeBendLT._fields_init(c);
    
    c.instance = new _LinkShapeType.ThreeBendLT_OVERLAP();
  }
  
}
);

_LinkShapeType.FourBendLinkShapeType = declare(_OrthogonalLinkShapeType, 
{

  constructor : function()
  {
    
  },

  doComputeIntermediatePoints : function(linkData, fromRect, toRect, fromSide, toSide, fromPoint, toPoint, minFinalSegment)
  {
    // summary:
    //		
    // tags:
    //		protected
    
    var bends = linkData.getLinkShape().getIntermediatePoints();
    var point0 = this.getOrCreatePoint(bends, 0);
    TranslateUtil.setPointLocation(point0, fromPoint);
    fromSide.translateOutside(point0, minFinalSegment);
    var point3 = this.getOrCreatePoint(bends, 3);
    TranslateUtil.setPointLocation(point3, toPoint);
    toSide.translateOutside(point3, minFinalSegment);
    var point1 = this.getOrCreatePoint(bends, 1);
    var point2 = this.getOrCreatePoint(bends, 2);
    this.computePoints(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment);
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {},
  
  getNumberOfBends : function()
  {
    // summary:
    //		Returns the number of bends (connection points are not counted).
    
    return 4; // int
  }

}
);
_LinkShapeType.FourBendTBleft = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendBTleft.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var xMin = Math.min(fromRect.x, toRect.x) - minFinalSegment;
    if (point0.x > point3.x) {
      TranslateUtil.movePoint(point0, point0.x, Math.min(fromRect.y, toRect.y) - minFinalSegment);
    }
    else {
      TranslateUtil.movePoint(point3, point3.x, Math.max(fromRect.y + fromRect.height, toRect.y + toRect.height) + minFinalSegment);
    }
    TranslateUtil.movePoint(point1, xMin, point0.y);
    TranslateUtil.movePoint(point2, xMin, point3.y);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendTBleft,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendTBleft;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendTBleft();
  }
  
}
);

_LinkShapeType.FourBendBTleft = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendTBleft.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var xMin = Math.min(fromRect.x, toRect.x) - minFinalSegment;
    if (point0.x > point3.x) {
      TranslateUtil.movePoint(point0, point0.x, Math.max(fromRect.y + fromRect.height, toRect.y + toRect.height) + minFinalSegment);
    }
    else {
      TranslateUtil.movePoint(point3, point3.x, Math.min(fromRect.y, toRect.y) - minFinalSegment);
    }
    TranslateUtil.movePoint(point1, xMin, point0.y);
    TranslateUtil.movePoint(point2, xMin, point3.y);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendBTleft,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendBTleft;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendBTleft();
  }
  
}
);

_LinkShapeType.FourBendTBright = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendBTright.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var xMax = Math.max(fromRect.x + fromRect.width, toRect.x + toRect.width) + minFinalSegment;
    if (point0.x <= point3.x) {
      TranslateUtil.movePoint(point0, point0.x, Math.min(fromRect.y, toRect.y) - minFinalSegment);
    }
    else {
      TranslateUtil.movePoint(point3, point3.x, Math.max(fromRect.y + fromRect.height, toRect.y + toRect.height) + minFinalSegment);
    }
    TranslateUtil.movePoint(point1, xMax, point0.y);
    TranslateUtil.movePoint(point2, xMax, point3.y);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendTBright,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendTBright;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendTBright();
  }
  
}
);

_LinkShapeType.FourBendBTright = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendTBright.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var xMax = Math.max(fromRect.x + fromRect.width, toRect.x + toRect.width) + minFinalSegment;
    if (point0.x <= point3.x) {
      TranslateUtil.movePoint(point0, point0.x, Math.max(fromRect.y + fromRect.height, toRect.y + toRect.height) + minFinalSegment);
    }
    else {
      TranslateUtil.movePoint(point3, point3.x, Math.min(fromRect.y, toRect.y) - minFinalSegment);
    }
    TranslateUtil.movePoint(point1, xMax, point0.y);
    TranslateUtil.movePoint(point2, xMax, point3.y);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendBTright,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendBTright;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendBTright();
  }
  
}
);

_LinkShapeType.FourBendTBmiddle = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendBTmiddle.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var xMiddle = _LinkShapeType.getXMiddle(point0, point3, fromRect, toRect);
    TranslateUtil.movePoint(point0, point0.x, fromRect.y - minFinalSegment);
    TranslateUtil.movePoint(point3, point3.x, toRect.y + toRect.height + minFinalSegment);
    TranslateUtil.movePoint(point1, xMiddle, point0.y);
    TranslateUtil.movePoint(point2, xMiddle, point3.y);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendTBmiddle,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendTBmiddle;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendTBmiddle();
  }
  
}
);

_LinkShapeType.FourBendBTmiddle = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendTBmiddle.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var xMiddle = _LinkShapeType.getXMiddle(point0, point3, fromRect, toRect);
    TranslateUtil.movePoint(point0, point0.x, fromRect.y + fromRect.height + minFinalSegment);
    TranslateUtil.movePoint(point3, point3.x, toRect.y - minFinalSegment);
    TranslateUtil.movePoint(point1, xMiddle, point0.y);
    TranslateUtil.movePoint(point2, xMiddle, point3.y);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendBTmiddle,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendBTmiddle;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendBTmiddle();
  }
  
}
);

_LinkShapeType.FourBendLRtop = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendRLtop.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var yMin = Math.min(fromRect.y, toRect.y) - minFinalSegment;
    if (point0.y <= point3.y) {
      TranslateUtil.movePoint(point3, Math.max(fromRect.x + fromRect.width, toRect.x + toRect.width) + minFinalSegment, point3.y);
    }
    else {
      TranslateUtil.movePoint(point0, Math.min(fromRect.x, toRect.x) - minFinalSegment, point0.y);
    }
    TranslateUtil.movePoint(point1, point0.x, yMin);
    TranslateUtil.movePoint(point2, point3.x, yMin);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendLRtop,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendLRtop;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendLRtop();
  }
  
}
);

_LinkShapeType.FourBendRLtop = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendLRtop.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var yMin = Math.min(fromRect.y, toRect.y) - minFinalSegment;
    if (point0.y <= point3.y) {
      TranslateUtil.movePoint(point3, Math.min(fromRect.x, toRect.x) - minFinalSegment, point3.y);
    }
    else {
      TranslateUtil.movePoint(point0, Math.max(fromRect.x + fromRect.width, toRect.x + toRect.width) + minFinalSegment, point0.y);
    }
    TranslateUtil.movePoint(point1, point0.x, yMin);
    TranslateUtil.movePoint(point2, point3.x, yMin);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendRLtop,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendRLtop;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendRLtop();
  }
  
}
);

_LinkShapeType.FourBendLRbottom = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendRLbottom.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var yMax = Math.max(fromRect.y + fromRect.height, toRect.y + toRect.height) + minFinalSegment;
    if (point0.y > point3.y) {
      TranslateUtil.movePoint(point3, Math.max(fromRect.x + fromRect.width, toRect.x + toRect.width) + minFinalSegment, point3.y);
    }
    else {
      TranslateUtil.movePoint(point0, Math.min(fromRect.x, toRect.x) - minFinalSegment, point0.y);
    }
    TranslateUtil.movePoint(point1, point0.x, yMax);
    TranslateUtil.movePoint(point2, point3.x, yMax);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendLRbottom,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendLRbottom;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendLRbottom();
  }
  
}
);

_LinkShapeType.FourBendLRmiddle = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendRLmiddle.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var yMiddle = _LinkShapeType.getYMiddle(point0, point3, fromRect, toRect);
    TranslateUtil.movePoint(point0, fromRect.x - minFinalSegment, point0.y);
    TranslateUtil.movePoint(point3, toRect.x + toRect.width + minFinalSegment, point3.y);
    TranslateUtil.movePoint(point1, point0.x, yMiddle);
    TranslateUtil.movePoint(point2, point3.x, yMiddle);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendLRmiddle,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendLRmiddle;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendLRmiddle();
  }
  
}
);

_LinkShapeType.FourBendRLmiddle = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendLRmiddle.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var yMiddle = _LinkShapeType.getYMiddle(point0, point3, fromRect, toRect);
    if (fromRect.y > (toRect.y + toRect.height)) {
      yMiddle = (fromRect.y + toRect.y + toRect.height) * 0.5;
    }
    else if (toRect.y > (fromRect.y + fromRect.height)) {
      yMiddle = (toRect.y + fromRect.y + fromRect.height) * 0.5;
    }
    else {
      yMiddle = (point0.y + point3.y) * 0.5;
    }
    TranslateUtil.movePoint(point0, fromRect.x + fromRect.width + minFinalSegment, point0.y);
    TranslateUtil.movePoint(point3, toRect.x - minFinalSegment, point3.y);
    TranslateUtil.movePoint(point1, point0.x, yMiddle);
    TranslateUtil.movePoint(point2, point3.x, yMiddle);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendRLmiddle,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendRLmiddle;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendRLmiddle();
  }
  
}
);

_LinkShapeType.FourBendRLbottom = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return _LinkShapeType.FourBendLRbottom.instance; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    var yMax = Math.max(fromRect.y + fromRect.height, toRect.y + toRect.height) + minFinalSegment;
    if (point0.y > point3.y) {
      TranslateUtil.movePoint(point3, Math.min(fromRect.x, toRect.x) - minFinalSegment, point3.y);
    }
    else {
      TranslateUtil.movePoint(point0, Math.max(fromRect.x + fromRect.width, toRect.x + toRect.width) + minFinalSegment, point0.y);
    }
    TranslateUtil.movePoint(point1, point0.x, yMax);
    TranslateUtil.movePoint(point2, point3.x, yMax);
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendRLbottom,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendRLbottom;
    if(_LinkShapeType.FourBendLinkShapeType._fields_init)
      _LinkShapeType.FourBendLinkShapeType._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendRLbottom();
  }
  
}
);

_LinkShapeType.FourBendSameSide = declare(_LinkShapeType.FourBendLinkShapeType, 
{

  constructor : function()
  {
    
  },
  getReversedShapeType : function()
  {
    return this; // ibm_ilog.graphlayout.shortlink._LinkShapeType
  },

  computePoints : function(point0, point1, point2, point3, fromRect, toRect, fromSide, toSide, minFinalSegment)
  {
    fromSide.setRadialCoord(point1, fromSide.getPointRadialCoord(point0));
    fromSide.setRadialCoord(point2, fromSide.getPointRadialCoord(point3));
    var upperRect;
    var upperPoint;
    var bottomPoint;

    if (fromSide.getPointRadialDelta(point0, point3) >= 0)
    {
      upperRect = toRect;
      upperPoint = point2;
      bottomPoint = point1;
    }
    else
    {
      upperRect = fromRect;
      upperPoint = point1;
      bottomPoint = point2;
    }

    fromSide.setTangentialCoord(upperPoint, ((this.isLeft() ? (fromSide.getMinTangentialCoord(upperRect) - minFinalSegment) : (fromSide.getMaxTangentialCoord(upperRect) + minFinalSegment))));
    fromSide.setTangentialCoord(bottomPoint, fromSide.getTangentialCoord(upperPoint));
  },

  isLeft : function()
  {}
  
}
);
_LinkShapeType.FourBendTTleft = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return true; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendTTleft,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendTTleft;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendTTleft();
  }
  
}
);

_LinkShapeType.FourBendTTright = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return false; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getTopSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendTTright,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendTTright;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendTTright();
  }
  
}
);

_LinkShapeType.FourBendBBleft = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return true; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendBBleft,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendBBleft;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendBBleft();
  }
  
}
);

_LinkShapeType.FourBendBBright = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return false; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getBottomSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendBBright,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendBBright;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendBBright();
  }
  
}
);

_LinkShapeType.FourBendLLleft = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return true; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendLLleft,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendLLleft;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendLLleft();
  }
  
}
);

_LinkShapeType.FourBendLLright = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return false; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getLeftSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendLLright,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendLLright;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendLLright();
  }
  
}
);

_LinkShapeType.FourBendRRleft = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return true; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendRRleft,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendRRleft;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendRRleft();
  }
  
}
);

_LinkShapeType.FourBendRRright = declare(_LinkShapeType.FourBendSameSide, 
{

  constructor : function()
  {
    
  },
  isLeft : function()
  {
    return false; // Boolean
  },

  getFromNodeSide : function(linkData)
  {
    return linkData.getFromNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkOrigin()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  },

  getToNodeSide : function(linkData)
  {
    return linkData.getToNode().getRightSide(true, linkData.isQuasiSelfInterGraphLinkDestination()); // ibm_ilog.graphlayout.shortlink._SLNodeSide
  }

}
);

lang.mixin(_LinkShapeType.FourBendRRright,
{
  instance : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeType.FourBendRRright;
    if(_LinkShapeType.FourBendSameSide._fields_init)
      _LinkShapeType.FourBendSameSide._fields_init(c);
    
    c.instance = new _LinkShapeType.FourBendRRright();
  }
  
}
);

(function(){ _LinkShapeType.FourBendRRright._fields_init(); })();
(function(){ _LinkShapeType.FourBendRRleft._fields_init(); })();
(function(){ _LinkShapeType.FourBendLLright._fields_init(); })();
(function(){ _LinkShapeType.FourBendLLleft._fields_init(); })();
(function(){ _LinkShapeType.FourBendBBright._fields_init(); })();
(function(){ _LinkShapeType.FourBendBBleft._fields_init(); })();
(function(){ _LinkShapeType.FourBendTTright._fields_init(); })();
(function(){ _LinkShapeType.FourBendTTleft._fields_init(); })();
(function(){ _LinkShapeType.FourBendRLbottom._fields_init(); })();
(function(){ _LinkShapeType.FourBendRLmiddle._fields_init(); })();
(function(){ _LinkShapeType.FourBendLRmiddle._fields_init(); })();
(function(){ _LinkShapeType.FourBendLRbottom._fields_init(); })();
(function(){ _LinkShapeType.FourBendRLtop._fields_init(); })();
(function(){ _LinkShapeType.FourBendLRtop._fields_init(); })();
(function(){ _LinkShapeType.FourBendBTmiddle._fields_init(); })();
(function(){ _LinkShapeType.FourBendTBmiddle._fields_init(); })();
(function(){ _LinkShapeType.FourBendBTright._fields_init(); })();
(function(){ _LinkShapeType.FourBendTBright._fields_init(); })();
(function(){ _LinkShapeType.FourBendBTleft._fields_init(); })();
(function(){ _LinkShapeType.FourBendTBleft._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendLT_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendLT._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendTL_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendTL._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendRB_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendRB._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendBR_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendBR._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendLB_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendLB._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendBL_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendBL._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendTR_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendTR._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendRT_OVERLAP._fields_init(); })();
(function(){ _LinkShapeType.ThreeBendRT._fields_init(); })();
(function(){ _LinkShapeType.TwoBendLR._fields_init(); })();
(function(){ _LinkShapeType.TwoBendRL._fields_init(); })();
(function(){ _LinkShapeType.TwoBendTB._fields_init(); })();
(function(){ _LinkShapeType.TwoBendBT._fields_init(); })();
(function(){ _LinkShapeType.TwoBendRR._fields_init(); })();
(function(){ _LinkShapeType.TwoBendLL._fields_init(); })();
(function(){ _LinkShapeType.TwoBendBB._fields_init(); })();
(function(){ _LinkShapeType.TwoBendTT._fields_init(); })();
(function(){ _LinkShapeType.OneBendRB._fields_init(); })();
(function(){ _LinkShapeType.OneBendRT._fields_init(); })();
(function(){ _LinkShapeType.OneBendLB._fields_init(); })();
(function(){ _LinkShapeType.OneBendLT._fields_init(); })();
(function(){ _LinkShapeType.OneBendBR._fields_init(); })();
(function(){ _LinkShapeType.OneBendBL._fields_init(); })();
(function(){ _LinkShapeType.OneBendTR._fields_init(); })();
(function(){ _LinkShapeType.OneBendTL._fields_init(); })();
(function(){ _LinkShapeType.DirectLL._fields_init(); })();
(function(){ _LinkShapeType.DirectRR._fields_init(); })();
(function(){ _LinkShapeType.DirectBB._fields_init(); })();
(function(){ _LinkShapeType.DirectTT._fields_init(); })();
(function(){ _LinkShapeType.DirectLB._fields_init(); })();
(function(){ _LinkShapeType.DirectLT._fields_init(); })();
(function(){ _LinkShapeType.DirectLR._fields_init(); })();
(function(){ _LinkShapeType.DirectRT._fields_init(); })();
(function(){ _LinkShapeType.DirectBR._fields_init(); })();
(function(){ _LinkShapeType.DirectBL._fields_init(); })();
(function(){ _LinkShapeType.DirectBT._fields_init(); })();
(function(){ _LinkShapeType.DirectRB._fields_init(); })();
(function(){ _LinkShapeType.DirectRL._fields_init(); })();
(function(){ _LinkShapeType.DirectTR._fields_init(); })();
(function(){ _LinkShapeType.DirectTB._fields_init(); })();
(function(){ _LinkShapeType.DirectTL._fields_init(); })();

return _LinkShapeType;
});

},
'ibm_ilog/graphlayout/shortlink/_OrthogonalIncidentLinksSorter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_OrthogonalIncidentLinksSorter", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_IncidentLinksSorter"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _IncidentLinksSorter	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _IncidentLinksSorter = ibm_ilog.graphlayout.shortlink._IncidentLinksSorter;
=====*/

var _OrthogonalIncidentLinksSorter = declare("ibm_ilog.graphlayout.shortlink._OrthogonalIncidentLinksSorter", _IncidentLinksSorter, 
{
  // summary:
  //		This subclass is used to sort the links incident to a node for the 
  //		orthogonal link style. 
  // description:
  //		This subclass is used to sort the links incident to a node for the 
  //		orthogonal link style. 
  //		A layout instance uses only one instance of this class.   
  // tags:
  //		private
  
  constructor : function(layout)
  {
  },
  
  compareLinks : function(link1, link2, origin1, origin2, nodeSide)
  {
    // summary:
    //		Decides about the order of the connection of two links on this side.
    //		
    // description:
    //		Decides about the order of the connection of two links on this side.
    //		
    //		Returns true if the links are in the correct order and false
    //		otherwise.
    // tags:
    //		protected
    
    return nodeSide.compareOrthogonalLinks(this._layout, link1, link2, origin1, origin2); // Boolean
  }

}
);

return _OrthogonalIncidentLinksSorter;
});

},
'ibm_ilog/graphlayout/longlink/LLDirectRouteAlgorithm':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLDirectRouteAlgorithm", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/QuickSort",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../longlink/LLTerminationCandAlgorithm",	// #9
  "../Direction"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    QuickSort,	// #7
    TranslateUtil,	// #8
    LLTerminationCandAlgorithm,	// #9
    Direction	// #10
    ) {

var LLDirectRouteAlgorithm = declare("ibm_ilog.graphlayout.longlink.LLDirectRouteAlgorithm", null, 
{
  // summary:
  //		The algorithm to calculate link routes if the style is DIRECT_STYLE. 
  // description:
  //		The algorithm to calculate link routes if the style is DIRECT&#95;STYLE. 
  //		Given all termination point candidates for all direct links, all we
  //		need to do is to choose the termination points wisely, i.e. such that
  //		the direct links
  //		don't overlap or cross each other. We do this per node, i.e. for all direct links at one node, we choose their order, and by that their termination points. We use a simple heuristic for this. Note that this alg. is by far not as intelligent as its counterpart in ShortLinkLayout. We did it only for compatibility, not because it is useful.  
  // tags:
  //		private
  
  _layout: null,

  _node: null,

  _occupiedMarker: null,
  _firstFreeOccupiedMarkerSlot: null,

  constructor : function(layout, grid)
  {
    this._layout = layout;
    this._node = null;
    this._occupiedMarker = null;
    this._firstFreeOccupiedMarkerSlot = -1;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._occupiedMarker = null;

    this._layout = null;
    this._node = null;
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current layout.
    // tags:
    //		private
    
    return (this._layout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getNode : function()
  {
    // summary:
    //		Returns the reference node.
    // tags:
    //		private
    
    return (this._node); // Object
  },

  getTerminationPoints : function(link)
  {
    // summary:
    //		Get the termination point array of the link.
    // tags:
    //		private
    
    if (link.getFromNode() == this.getNode()) {
      return link.getStartPoints(); // Array
    }
    else {
      return link.getEndPoints(); // Array
    }
  },

  setTerminationPoint : function(link, index)
  {
    // summary:
    //		Set the actual termination point of the link.
    // tags:
    //		private
    
    if (link.getFromNode() == this.getNode())
    {
      link.setActStartCandidate(index);
    }
    else
    {
      link.setActEndCandidate(index);
    }
  },

  route : function(node, links)
  {
    // summary:
    //		Route all direct links that are incident to this node. 
    // description:
    //		Route all direct links that are incident to this node. 
    //		This only calculates the actual termination point of the links in
    //		the input vector that corresponds to the end where the input node
    //		is. That is, for links that start at this node, it calculate
    //		actStartPoint, and for links that end at this node, it calculate
    //		actEndPoint.  Self loops are never in the link vector.
    // tags:
    //		private
    
    this._node = node;

    var e =  new TranslateUtil.ArrayIterator(links);
    var link;

    while (e.hasNext())
    {
      link = e.next();
      link.setPreferedSide(this.getLinkSide(link));
    }

    this.calcLinksAtSide(Direction.TOP, links);
    this.calcLinksAtSide(Direction.BOTTOM, links);
    this.calcLinksAtSide(Direction.LEFT, links);
    this.calcLinksAtSide(Direction.RIGHT, links);
    return (true); // Boolean
  },

  calcLinksAtSide : function(side, links)
  {
    // summary:
    //		Assign termination points for the links that want to go to the
    //		specified side.
    // tags:
    //		private
    
    var linksAtSide = new Array();
    var e =  new TranslateUtil.ArrayIterator(links);
    var link;

    while (e.hasNext())
    {
      link = e.next();
      if (link.getPreferedSide() == side) {
        linksAtSide.push(link);
      }
    }

    if (linksAtSide.length == 0) {
      return;
    }

    var correction = 0.0;
    var dir = 1;
    var sign = -1;
    var middleOffset = 0;
    var incrOnPositive = -1;
    var incrOnNegative = 0;

    switch (side)
    {
      case Direction.RIGHT:
        correction = -Math.PI;
        dir = 1;
        sign = 1;
        middleOffset = 1;
        incrOnPositive = 0;
        incrOnNegative = 1;
        break;
      case Direction.TOP:
        correction = -0.5 * Math.PI;
        dir = 0;
        sign = 1;
        middleOffset = 1;
        incrOnPositive = 0;
        incrOnNegative = 1;
        break;
      case Direction.BOTTOM:
        correction = -1.5 * Math.PI;
        dir = 0;
        sign = -1;
        middleOffset = 0;
        incrOnPositive = -1;
        incrOnNegative = 0;
        break;
    }
    
    e =  new TranslateUtil.ArrayIterator(linksAtSide);

    while (e.hasNext())
    {
      link = e.next();

      this.calcRadialSortValue(link, correction);
    }

    var sortAlg = new LLDirectRouteAlgorithm.LinkSort(this, linksAtSide);
    sortAlg.sort();
    sortAlg.dispose();

    this.clearOccupied(linksAtSide.length);

    var middle = Math.floor((linksAtSide.length - middleOffset) / 2);
    link = linksAtSide[middle];
    this.setTerminationPoint(link, 0);
    this.markOccupied(sign * this.getTerminationPoints(link)[0].getIndex(dir));
    var incr = incrOnPositive + incrOnNegative;
    var leftMostIndex = sign * this.getTerminationPoints(link)[0].getIndex(dir);
    var rightMostIndex = sign * this.getTerminationPoints(link)[0].getIndex(dir);
    var penalty;
    var bestPenalty;
    var bestPointIndex;
    var bestIsOccupied;
    var coordIndex;
    var points;

    var i;
    var j;

    for (i = 1; i < linksAtSide.length; i++)
    {
      link = linksAtSide[middle + incr];
      points = this.getTerminationPoints(link);
      bestPenalty = 0;
      bestPointIndex = 0;
      bestIsOccupied = true;

      if (incr > 0)
      {
        for (j = 0; j < points.length; j++)
        {
          if (points[j].getSide() == side)
          {
            coordIndex = sign * points[j].getIndex(dir);
            penalty = coordIndex - rightMostIndex;

            if (this.isOccupied(coordIndex))
            {
              if (!bestIsOccupied) {
                continue;
              }
            }
            else if (bestIsOccupied)
            {
              bestPenalty = penalty;
              bestPointIndex = j;
              bestIsOccupied = false;
            }

            if (penalty >= 0)
            {
              if (bestPenalty <= 0 || penalty < bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
            else
            {
              if (bestPenalty <= 0 && penalty > bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
          }
        }

        incr = incrOnPositive - incr;
      }
      else
      {
        for (j = 0; j < points.length; j++)
        {
          if (points[j].getSide() == side)
          {
            coordIndex = sign * points[j].getIndex(dir);
            penalty = leftMostIndex - coordIndex;

            if (this.isOccupied(coordIndex))
            {
              if (!bestIsOccupied) {
                continue;
              }
            }
            else if (bestIsOccupied)
            {
              bestPenalty = penalty;
              bestPointIndex = j;
              bestIsOccupied = false;
            }

            if (penalty >= 0)
            {
              if (bestPenalty <= 0 || penalty < bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
            else
            {
              if (bestPenalty <= 0 && penalty > bestPenalty)
              {
                bestPenalty = penalty;
                bestPointIndex = j;
              }
            }
          }
        }

        incr = incrOnNegative - incr;
      }

      coordIndex = sign * points[bestPointIndex].getIndex(dir);
      this.setTerminationPoint(link, bestPointIndex);
      this.markOccupied(coordIndex);
      if (coordIndex > rightMostIndex) {
        rightMostIndex = coordIndex;
      }
      if (coordIndex < leftMostIndex) {
        leftMostIndex = coordIndex;
      }
    }
  },

  getLinkSide : function(link)
  {
    // summary:
    //		Determines for a single link at which border it wants to go.
    // tags:
    //		private
    
    var node = this.getNode();
    var startRect = link.getFromRect();
    var endRect = link.getToRect();
    var layout = this.getLayout();
    var minStartLen = layout.getMinStartSegmentLength();
    var minEndLen = layout.getMinEndSegmentLength();
    var pref = TranslateUtil.newArray(4, 0);
    var aux = TranslateUtil.newArray(4, 0);
    var terminationPoints;
    pref[0] = pref[1] = pref[2] = pref[3] = 0;

    if (link.getFromNode() == node)
    {
      terminationPoints = link.getStartPoints();

      if (link.getFromPoint() != null)
      {
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.x - minStartLen, startRect.x + startRect.width + minStartLen, endRect.x - minEndLen, endRect.x + endRect.width + minEndLen, 1, 3, 
                pref, aux);
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.y - minStartLen, startRect.y + startRect.height + minStartLen, endRect.y - minEndLen, endRect.y + endRect.height + minEndLen, 0, 2,
            pref, aux);
      }
    }
    else
    {
      terminationPoints = link.getEndPoints();

      if (link.getToPoint() != null)
      {
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.x - minStartLen, startRect.x + startRect.width + minStartLen, endRect.x - minEndLen, endRect.x + endRect.width + minEndLen, 1, 3, aux,
            pref);
        LLTerminationCandAlgorithm.calcSidePrefInternal(startRect.y - minStartLen, startRect.y + startRect.height + minStartLen, endRect.y - minEndLen, endRect.y + endRect.height + minEndLen, 0, 2,
            aux, pref);
      }
    }

    if (terminationPoints.length > 1) {
      terminationPoints[0].sortAccordingToSide(terminationPoints, pref);
    }
    return (terminationPoints[0].getSide()); // int
  },

  calcRadialSortValue : function(link, correction)
  {
    // summary:
    //		Calculate the sort value as the angle of the ray starting at the
    //		reference node and going through the center of the other end node. 
    // description:
    //		Calculate the sort value as the angle of the ray starting at the
    //		reference node and going through the center of the other end node. 
    //		The input correction value allows use to rotate the coordinate
    //		system such that the 0-axis always is opposite to the current side.
    // tags:
    //		private
    
    var startX;
    var startY;
    var endX;
    var endY;
    var rect;

    if (link.getFromNode() == this.getNode())
    {
      rect = link.getFromRect();
      startX = rect.x + 0.5 * rect.width;
      startY = rect.y + 0.5 * rect.height;
      rect = link.getToRect();
      endX = rect.x + 0.5 * rect.width;
      endY = rect.y + 0.5 * rect.height;

      if (link.getFromPoint() != null)
      {
        startX = link.getFromPoint().x;
        startY = link.getFromPoint().y;
      }

      if (link.getToPoint() != null)
      {
        endX = link.getToPoint().x;
        endY = link.getToPoint().y;
      }
    }
    else
    {
      rect = link.getToRect();
      startX = rect.x + 0.5 * rect.width;
      startY = rect.y + 0.5 * rect.height;
      rect = link.getFromRect();
      endX = rect.x + 0.5 * rect.width;
      endY = rect.y + 0.5 * rect.height;

      if (link.getToPoint() != null)
      {
        startX = link.getToPoint().x;
        startY = link.getToPoint().y;
      }

      if (link.getFromPoint() != null)
      {
        endX = link.getFromPoint().x;
        endY = link.getFromPoint().y;
      }
    }

    var angle = LayoutUtil.angle(startX, startY, endX, endY);
    angle += correction;
    angle = LayoutUtil.correctAngle(angle);
    link.setSortValue(angle);
  },

  clearOccupied : function(numberOfMarkers)
  {
    // summary:
    //		Clear all occupied marker.
    // tags:
    //		private
    
    this._occupiedMarker = TranslateUtil.newArray(numberOfMarkers, 0);
    this._firstFreeOccupiedMarkerSlot = 0;
  },

  markOccupied : function(index)
  {
    // summary:
    //		Mark the index as occupied.
    // tags:
    //		private
    
    this._occupiedMarker[this._firstFreeOccupiedMarkerSlot++] = index;
  },

  isOccupied : function(index)
  {
    // summary:
    //		Returns true if the index is occupied.
    // tags:
    //		private
    
    for (var i = 0; i < this._firstFreeOccupiedMarkerSlot; i++)
      if (this._occupiedMarker[i] == index) {
        return (true); // Boolean
      }
    return (false); // Boolean
  }

}
);

LLDirectRouteAlgorithm.LinkSort = declare(QuickSort, 
{
  // summary:
  //		Inner auxiliary class to sort a vector of links according to the sort
  //		value.
  
  __outerThis: null,

  _links: null,

  constructor : function(input__outerThis, links)
  {
    this.__outerThis = input__outerThis;
    this._links = links;
  },
  
  dispose : function()
  {
    this._links = null;
  },

  sort : function()
  {
    // summary:
    // tags:
    //		private
    
    this.inherited("sort", arguments, [this._links.length]);
  },

  compare : function(loc1, loc2)
  {
    var link1 = this._links[loc1];
    var link2 = this._links[loc2];

    if (link1.getSortValue() == link2.getSortValue())
    {
      var link1Point;
      var link2Point;
      if (link1.getFromNode() == this.__outerThis.getNode()) {
        link1Point = link1.getActEndPoint();
      }
      else {
        link1Point = link1.getActStartPoint();
      }
      if (link2.getFromNode() == this.__outerThis.getNode()) {
        link2Point = link2.getActEndPoint();
      }
      else {
        link2Point = link2.getActStartPoint();
      }
      if (link1Point == null || link2Point == null) {
        return 0; // int
      }
      switch (link1Point.getSide())
      {
        case Direction.TOP:
          return (link2Point.getIndex(0) - link1Point.getIndex(0)); // int
        case Direction.BOTTOM:
          return (link1Point.getIndex(0) - link2Point.getIndex(0)); // int
        case Direction.LEFT:
          return (link1Point.getIndex(1) - link2Point.getIndex(1)); // int
        case Direction.RIGHT:
          return (link2Point.getIndex(1) - link1Point.getIndex(1)); // int
      }
      return 0; // int
    }

    if (link1.getSortValue() < link2.getSortValue()) {
      return -1; // int
    }
    else {
      return 1; // int
    }
  },

  swap : function(loc1, loc2)
  {
    var link1 = this._links[loc1];
    var link2 = this._links[loc2];
    this._links[loc2] =   link1;
    this._links[loc1] =   link2;
  }

}
);

return LLDirectRouteAlgorithm;
});

},
'ibm_ilog/graphlayout/shortlink/ShortLinkLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/ShortLinkLayout", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/ConnectionPointModes",	// #5
  "../internalutil/ConnectorStyles",	// #6
  "../internalutil/GraphModelUtil",	// #7
  "../internalutil/LayoutParametersUtil",	// #8
  "../internalutil/LinkStyles",	// #9
  "../internalutil/LogResUtil",	// #10
  "../internalutil/ReshapeModes",	// #11
  "../internalutil/TranslateUtil",	// #12
  "../shortlink/_SLBundlesModes",	// #13
  "../shortlink/_ShortLinkAlgorithm",	// #14
  "../GraphLayout"	// #15
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    ConnectionPointModes,	// #5
    ConnectorStyles,	// #6
    GraphModelUtil,	// #7
    LayoutParametersUtil,	// #8
    LinkStyles,	// #9
    LogResUtil,	// #10
    ReshapeModes,	// #11
    TranslateUtil,	// #12
    _SLBundlesModes,	// #13
    _ShortLinkAlgorithm,	// #14
    GraphLayout	// #15
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var GraphLayout = ibm_ilog.graphlayout.GraphLayout;
=====*/

var ShortLinkLayout = declare("ibm_ilog.graphlayout.shortlink.ShortLinkLayout", GraphLayout, 
{
  // summary:
  //		This class is the main class for the Short Link Layout algorithm. 
  // description:
  //		This class is the main class for the Short Link Layout algorithm. 
  //		<p>
  //		This algorithm reshapes the links of a graph without moving the
  //		nodes. By default, the algorithm computes orthogonal links (that is,
  //		links composed of alternating horizontal and vertical segments).
  //		Optionally, direct links can be computed. (See <code>
  //		ShortLinkLayout.setGlobalLinkStyle()</code> and <code>
  //		ShortLinkLayout.setLinkStyle()</code>.)  
  //		</p>
  //		<p>
  //		The links are placed freely in the plane by choosing the best shape
  //		from a fixed number of link shapes. Orthogonal links can a maximum of
  //		4 bends. Direct links have 3 bends.  
  //		</p>
  //		<p>
  //		This layout algorithm is recommended when any of the following apply:
  //		 
  //		<ul>
  //		<li> Most of the links are short, and it is not fatal if long links
  //		overlap some nodes.</li>  
  //		<li> The link routes must be placed freely and cannot be restricted
  //		to a grid.</li>  
  //		<li> It is important to limit the number of bends.</li>  
  //		</ul>
  //		Otherwise, the <code>
  //		ibm&#95;ilog.graphlayout.longlink.LongLinkLayout</code> may be more
  //		suitable.  
  //		</p>
  //		<p>
  //		Here are sample drawings produced by the Short Link Layout algorithm:
  //		 
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Ortho-Ortho-small.gif" alt="Sample drawing produced with the Short Link Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Ortho-Direct-small.gif" alt="Sample drawing produced with the Short Link Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Ortho-Java2D.gif" alt="Sample drawing produced with the Short Link Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		See the corresponding chapter of the
  //		User's Manual for details on the algorithm, the types of graphs for which this algorithm can be used, the features and limitations, code samples, and so on.   
  //		</p>
  //		<p>
  //		Note the following points:  
  //		<ul>
  //		<li> The algorithm never moves any nodes. It only reshapes links.</li>  
  //		<li> The algorithm normally routes normal links. It has a special mode to route the intergraph links of the graph model.</li>  
  //		<li> The algorithm has an incremental mode and a nonincremental mode. In nonincremental mode, the initial shape of the links (at the moment you start the layout) does not affect the resulting layout. However, links specified as fixed while <code> ibm&#95;ilog.graphlayout.GraphLayout.setPreserveFixedLinks()</code> is enabled or links that have the link style <code> ShortLinkLayout.NO&#95;RESHAPE&#95;STYLE</code> are not reshaped. The incremental mode cannot be used for intergraph links.</li>  
  //		<li> In incremental mode, the initial shape of the link does affect the resulting layout. The algorithm tries to preserve the shape of links that already fit the layout situation.</li>  
  //		<li> The algorithm supports self-links and multiple links between the same pair of nodes.</li>  
  //		</ul>  
  //		</p>
  /*=====
  // ORTHOGONAL_STYLE: int
  //		Orthogonal link style. 
  //		When used as the argument of the method  '
  //		ShortLinkLayout.setGlobalLinkStyle()', all links get an
  //		orthogonal shape (alternating horizontal and vertical segments).  
  //		
  //		To specify the style of an individual link, use this style as the
  //		argument of the method ' ShortLinkLayout.setLinkStyle()'. 
  //		
  //		
  ORTHOGONAL_STYLE : null,
  // DIRECT_STYLE: int
  //		Direct link style. 
  //		When used as the argument of the method  '
  //		ShortLinkLayout.setGlobalLinkStyle()', all links get a
  //		(semi)direct shape.  
  //		
  //		To specify the style of an individual link, use this style as the
  //		argument of the method ' ShortLinkLayout.setLinkStyle()'. 
  //		
  //		
  DIRECT_STYLE : null,
  // MIXED_STYLE: int
  //		Mixed link shape. 
  //		It can be used with the following meanings:  
  //		<ul>
  //		<li>Link style mode. When used as the argument of
  //		the method  ' ShortLinkLayout.setGlobalLinkStyle()', each
  //		link can have a different shape. The shape of an individual link can
  //		be set by the method '
  //		ShortLinkLayout.setLinkStyle()'.</li>  
  //		<li>Connector style mode. When used as the argument
  //		of the method  '
  //		ShortLinkLayout.setGlobalConnectorStyle()', each node can have
  //		a different connector style. The connector style of an individual
  //		node can be set by the method '
  //		ShortLinkLayout.setConnectorStyle()'.</li>  
  //		</ul>
  MIXED_STYLE : null,
  // NO_RESHAPE_STYLE: int
  //		No reshape link style. 
  //		This can be used for individual links if the global link style is
  //		' ShortLinkLayout.MIXED_STYLE'. To specify the style of an
  //		individual link, use this style as the argument of the method '
  //		ShortLinkLayout.setLinkStyle()'.
  NO_RESHAPE_STYLE : null,
  // NO_BUNDLE: int
  //		No link bundles. 
  //		When used as the argument of the method  '
  //		ShortLinkLayout.setLinkBundlesMode()', no link bundles are
  //		computed.
  NO_BUNDLE : null,
  // FIRST_LAST_SEGMENT: int
  //		Simple link bundles. 
  //		When used as the argument of the  method  '
  //		ShortLinkLayout.setLinkBundlesMode()', only first and last
  //		segments of the links are bundled.
  FIRST_LAST_SEGMENT : null,
  // IMPROVED_FIRST_LAST_SEGMENT: int
  //		Improved link bundles. 
  //		When used as the argument of the method  '
  //		ShortLinkLayout.setLinkBundlesMode()', the first and the last
  //		segments of the links are bundled.
  IMPROVED_FIRST_LAST_SEGMENT : null,
  // TWO_BENDS_ORTHOGONAL_STYLE: int
  //		Two-bends orthogonal self-link. 
  //		When used as the argument of the method '
  //		ShortLinkLayout.setGlobalSelfLinkStyle()', a two-bend shape is
  //		given to the self-links, that is, links with the same origin and
  //		destination node.
  TWO_BENDS_ORTHOGONAL_STYLE : null,
  // THREE_BENDS_ORTHOGONAL_STYLE: int
  //		Three-bends orthogonal self-link. 
  //		When used as the argument of the method  '
  //		ShortLinkLayout.setGlobalSelfLinkStyle()', a three-bend shape
  //		is given to the self-links, that is, links with the same origin and
  //		destination node.
  THREE_BENDS_ORTHOGONAL_STYLE : null,
  // FREE_MODE: int
  //		Free mode. 
  //		This value can be used for the following:  
  //		<ul>
  //		<li>Connection-point mode. When free mode is used as
  //		the argument of the method  '
  //		ShortLinkLayout.setGlobalOriginPointMode()' or '
  //		ShortLinkLayout.setGlobalDestinationPointMode()', the layout is
  //		free to choose the appropriate position for the connection points,
  //		except for "pinned" connection points (see '
  //		ibm_ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()').
  //		To specify the mode for the connection points of an individual link,
  //		use free mode as the argument of the method '
  //		ShortLinkLayout.setOriginPointMode()' or '
  //		ShortLinkLayout.setDestinationPointMode()'.</li>  
  //		<li>Incremental link-reshape mode. When free mode is
  //		used as the argument of the method  '
  //		ShortLinkLayout.setGlobalIncrementalUnmodifiedLinkReshapeMode()'
  //		or '
  //		ShortLinkLayout.setGlobalIncrementalModifiedLinkReshapeMode()'
  //		in incremental mode, the layout is allowed to reshape the links
  //		freely. Any settings done using '
  //		ibm_ilog.graphlayout.GraphLayout.isFixed()', '
  //		ShortLinkLayout.getGlobalOriginPointMode()' and '
  //		ShortLinkLayout.getGlobalDestinationPointMode()' are still
  //		obeyed. To specify the mode for the incremental reshape of an
  //		individual link, use free mode as the argument of the method '
  //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()' or
  //		'
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()'.</li> 
  //		
  //		</ul>
  FREE_MODE : null,
  // FIXED_MODE: int
  //		Fixed mode. 
  //		This value can be used for the following:  
  //		<ul>
  //		<li>Connection-point mode. When fixed mode is used
  //		as the argument of the method  '
  //		ShortLinkLayout.setGlobalOriginPointMode()' or '
  //		ShortLinkLayout.setGlobalDestinationPointMode()', the layout
  //		must keep the current position of the connection point. To specify
  //		the mode for the connection points of an individual link, use fixed
  //		mode as the argument of the method  '
  //		ShortLinkLayout.setOriginPointMode()' or '
  //		ShortLinkLayout.setDestinationPointMode()'. </li>  
  //		<li>Incremental link-reshape mode. When fixed mode
  //		is used as the argument of the method  '
  //		ShortLinkLayout.setGlobalIncrementalUnmodifiedLinkReshapeMode()'
  //		or '
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()' in
  //		incremental mode, the layout is not allowed to reshape the links in
  //		any way. To specify the mode for the incremental reshape of an
  //		individual link, use fixed mode as the argument of the method '
  //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()' or
  //		'
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()'.</li> 
  //		
  //		</ul>
  FIXED_MODE : null,
  // FIXED_SHAPE_TYPE_MODE: int
  //		Fixed shape-type mode. 
  //		When this value is used as the argument of the method  '
  //		ShortLinkLayout.setGlobalIncrementalUnmodifiedLinkReshapeMode()'
  //		or the  '
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()' in
  //		incremental mode, the layout preserves the shape type of the link.
  //		This means that both the number of bends and the node sides to which
  //		the links are connected are preserved. To specify the mode for the
  //		incremental reshape of an individual link, use fixed shape-type mode
  //		as the argument of the method '
  //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()' or
  //		'
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()'.
  FIXED_SHAPE_TYPE_MODE : null,
  // FIXED_NODE_SIDES_MODE: int
  //		Fixed node-side mode. 
  //		When this value is used as the argument of the method  '
  //		ShortLinkLayout.setGlobalIncrementalUnmodifiedLinkReshapeMode()'
  //		or the  '
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()' in
  //		incremental mode, the layout preserves the node sides to which the
  //		links are connected. To specify the mode for the incremental reshape
  //		of an individual link, use fixed node-side mode as the argument of
  //		the method '
  //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()' or
  //		'
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()'.
  FIXED_NODE_SIDES_MODE : null,
  // FIXED_CONNECTION_POINTS_MODE: int
  //		Fixed connection-points mode. 
  //		When this value is used as the argument of the method  '
  //		ShortLinkLayout.setGlobalIncrementalUnmodifiedLinkReshapeMode()'
  //		or the  '
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()' in
  //		incremental mode, the layout preserves the connection points of the
  //		links. To specify the mode for the incremental reshape of an
  //		individual link, use fixed connection-points mode as the argument of
  //		the method '
  //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()' or
  //		'
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()'.
  FIXED_CONNECTION_POINTS_MODE : null,
  // MIXED_MODE: int
  //		Mixed mode. 
  //		This value can be used for the following:  
  //		<ul>
  //		<li>Connection-point mode. When mixed mode is used
  //		as the argument of the methods  '
  //		ShortLinkLayout.setGlobalOriginPointMode()' or '
  //		ShortLinkLayout.setGlobalDestinationPointMode()', the
  //		connection point of each link can have a different mode. The mode for
  //		the connection points of an individual link can be set by the method
  //		' ShortLinkLayout.setOriginPointMode()' or '
  //		ShortLinkLayout.setDestinationPointMode()'.</li>  
  //		<li>Incremental link-reshape mode. When mixed mode
  //		is used as the argument of the method '
  //		ShortLinkLayout.setGlobalIncrementalUnmodifiedLinkReshapeMode()'
  //		or '
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()', each
  //		link can have a different incremental reshape mode. The mode for the
  //		incremental reshape of an individual link can be set by the method
  //		'
  //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()' or
  //		'
  //		ShortLinkLayout.setIncrementalModifiedLinkReshapeMode()'.</li> 
  //		
  //		</ul>
  MIXED_MODE : null,
  // AUTOMATIC_PINS: int
  //		Automatic pin style. 
  //		When this value is used as the argument of the method  '
  //		ShortLinkLayout.setGlobalConnectorStyle()', the connector style
  //		of all nodes is automatically selected as follows. The connection
  //		points are dispatched using the style '
  //		ShortLinkLayout.FIXED_OFFSET_PINS' if in so doing no connection
  //		point goes outside the border to which the link is attached.
  //		Otherwise, the connection points are dispatched using the style
  //		' ShortLinkLayout.EVENLY_SPACED_PINS'.  
  //		
  //		To specify automatic pins for an individual node, use automatic pin
  //		style as the argument of the method '
  //		ShortLinkLayout.setConnectorStyle()'.  
  //		
  AUTOMATIC_PINS : null,
  // FIXED_OFFSET_PINS: int
  //		Fixed-offset pin style. 
  //		When this value is used as the argument of the method  '
  //		ShortLinkLayout.setGlobalConnectorStyle()', the connection
  //		points of all the links are placed at a fixed distance, given by the
  //		method ' ShortLinkLayout.getLinkOffset()'.  
  //		
  //		To specify fixed-offset pina for an individual node, use fixed-offset
  //		pin style as the argument of the method '
  //		ShortLinkLayout.setConnectorStyle()'.  
  //		
  FIXED_OFFSET_PINS : null,
  // EVENLY_SPACED_PINS: int
  //		Evenly spaced pin style. 
  //		When this value is used as the argument of the method  '
  //		ShortLinkLayout.setGlobalConnectorStyle()', the connection
  //		points of all the links are evenly spaced along the node border to
  //		which the link is attached.  
  //		
  //		To specify evenly spaced pins for an individual node, use evenly
  //		spaced pin style as the argument of the method '
  //		ShortLinkLayout.setConnectorStyle()'.  
  //		
  EVENLY_SPACED_PINS : null,
  =====*/
  
  _globalLinkStyle: null,

  _globalSelfLinkStyle: null,

  _originPointMode: null,

  _destinationPointMode: null,

  _globalConnectorStyle: null,
  _evenlySpacedPinsMarginRatio: null,

  _incrementalUnmodifiedLinkReshapeMode: null,

  _incrementalModifiedLinkReshapeMode: null,

  _linkBundlesMode: null,
  _minFinalSegment: null,
  _bypassDistance: null,
  _linkOffset: null,
  _linkNodePenalty: null,
  _linkLinkPenalty: null,
  _numberOfIterations: null,
  _sameShapeForMultipleLinks: null,
  _incrementalMode: null,
  _interGraphLinksMode: null,
  _combinedInterGraphLinksMode: null,
  _linkOverlapNodesForbidden: null,
  _nodeSideFilter: null,
  _nodeBoxInterface: null,

  _shortLinkAlgorithm: null,

  constructor : function (a0)
  {
    // summary:
    //		Creates a new instance of the Short Link Layout algorithm. 
    // description:
    //		Creates a new instance of the Short Link Layout algorithm. 
    //		<br>To indicate the grapher you want to lay out, use the method
    //		<code> ibm&#95;ilog.graphlayout.GraphLayout.attach()</code>. <br>To
    //		indicate the graph model you want to lay out, use the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.attach()</code>. <br>To perform the
    //		layout, use the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>. <br>To
    //		modify the layout parameters, use the different methods provided in
    //		this class and its superclass.
    
    if(arguments.length == 0)
      this._constructor_overload_ShortLinkLayout_0();
    else if(arguments.length == 1)
      this._constructor_overload_ShortLinkLayout_1(a0);
  },
  
  _constructor_overload_ShortLinkLayout_0 : function()
  {

  },
  
  init : function()
  {
    // summary:
    //		Initializes instance variables. 
    // description:
    //		Initializes instance variables. 
    //		<p>
    //		You should not call this method directly. The method is called
    //		internally by the constructor without arguments and by the copy
    //		constructor. The method must be overridden by subclasses that need
    //		to initialize additional instance variables.
    //		</p>
    // tags:
    //		protected
    
    this.inherited("init", arguments, []);
    this._globalLinkStyle = ShortLinkLayout._defaultGlobalLinkStyle;
    this._globalSelfLinkStyle = ShortLinkLayout._defaultGlobalSelfLinkStyle;
    this._originPointMode = ShortLinkLayout._defaultOriginPointMode;
    this._destinationPointMode = ShortLinkLayout._defaultDestinationPointMode;
    this._globalConnectorStyle = ShortLinkLayout._defaultGlobalConnectorStyle;
    this._incrementalUnmodifiedLinkReshapeMode = ShortLinkLayout._defaultIncrementalUnmodifiedLinkReshapeMode;
    this._incrementalModifiedLinkReshapeMode = ShortLinkLayout._defaultIncrementalModifiedLinkReshapeMode;
    this._evenlySpacedPinsMarginRatio = ShortLinkLayout._defaultEvenlySpacedPinsMarginRatio;
    this._linkBundlesMode = ShortLinkLayout._defaultLinkBundlesMode;
    this._minFinalSegment = ShortLinkLayout._defaultMinFinalSegment;
    this._bypassDistance = ShortLinkLayout._defaultBypassDistance;
    this._linkOffset = ShortLinkLayout._defaultLinkOffset;
    this._linkNodePenalty = ShortLinkLayout._defaultLinkNodePenalty;
    this._linkLinkPenalty = ShortLinkLayout._defaultLinkLinkPenalty;
    this._numberOfIterations = ShortLinkLayout._defaultNumberOfIterations;
    this._sameShapeForMultipleLinks = ShortLinkLayout._defaultSameShapeForMultipleLinks;
    this._incrementalMode = ShortLinkLayout._defaultIncrementalMode;
    this._interGraphLinksMode = ShortLinkLayout._defaultInterGraphLinksMode;
    this._combinedInterGraphLinksMode = ShortLinkLayout._defaultCombinedInterGraphLinksMode;
    this._linkOverlapNodesForbidden = ShortLinkLayout._defaultLinkOverlapNodesForbidden;

    this._shortLinkAlgorithm = new _ShortLinkAlgorithm(this);
  },

  _constructor_overload_ShortLinkLayout_1 : function(source )
  {
  },
  
  copy : function()
  {
    // summary:
    //		Copies the layout instance. 
    // description:
    //		Copies the layout instance. 
    //		<p>
    //		This method copies the layout instance by calling the copy
    //		constructor.  
    //		</p>
    //		<p>
    //		When performing a recursive layout on a nested graph, this method is
    //		used by <code> ibm&#95;ilog.graphlayout.ILayoutProvider</code> to
    //		"clone" the layout instance of a parent graph.  
    //		</p>
    //		<p>
    //		Note that the parameters which are specific to a node or a link are
    //		not copied. The other parameters, including the customization
    //		interfaces, are also copied.  
    //		</p>
    
    return new ShortLinkLayout(this); // ibm_ilog.graphlayout.GraphLayout
  },

  copyParameters : function(source)
  {
    // summary:
    //		Copies the parameters from a given layout instance. 
    // description:
    //		Copies the parameters from a given layout instance. 
    //		<p>
    //		Note that the parameters which are specific to a node or a link are
    //		not copied. The other parameters, including the customization
    //		interfaces, are also copied.  
    //		</p>
    //		<p>
    //		If a method of the type <code>supportsXXX</code> is associated with
    //		a parameter, the parameter is copied only if the corresponding
    //		method returns <code>true</code>.  
    //		</p>
    
    this.inherited("copyParameters", arguments, [source]);

    if (source instanceof ShortLinkLayout)
    {
      var ssource = source;
      this.setGlobalLinkStyle(ssource.getGlobalLinkStyle());
      this.setGlobalSelfLinkStyle(ssource.getGlobalSelfLinkStyle());
      this.setGlobalOriginPointMode(ssource.getGlobalOriginPointMode());
      this.setGlobalDestinationPointMode(ssource.getGlobalDestinationPointMode());
      this.setGlobalConnectorStyle(ssource.getGlobalConnectorStyle());
      this.setGlobalIncrementalUnmodifiedLinkReshapeMode(ssource.getGlobalIncrementalUnmodifiedLinkReshapeMode());
      this.setGlobalIncrementalModifiedLinkReshapeMode(ssource.getGlobalIncrementalModifiedLinkReshapeMode());
      this.setEvenlySpacedPinsMarginRatio(ssource.getEvenlySpacedPinsMarginRatio());
      this.setLinkBundlesMode(ssource.getLinkBundlesMode());
      this.setMinFinalSegmentLength(ssource.getMinFinalSegmentLength());
      this.setBypassDistance(ssource.getBypassDistance());
      this.setLinkOffset(ssource.getLinkOffset());
      this.setLinkToNodeCrossingPenalty(ssource.getLinkToNodeCrossingPenalty());
      this.setLinkToLinkCrossingPenalty(ssource.getLinkToLinkCrossingPenalty());
      this.setAllowedNumberOfIterations(ssource.getAllowedNumberOfIterations());
      this.setSameShapeForMultipleLinks(ssource.isSameShapeForMultipleLinks());
      this.setIncrementalMode(ssource.isIncrementalMode());
      this.setInterGraphLinksMode(ssource.isInterGraphLinksMode());
      this.setCombinedInterGraphLinksMode(ssource.isCombinedInterGraphLinksMode());
      this.setNodeSideFilter(ssource.getNodeSideFilter());
      this.setNodeBoxInterface(ssource.getNodeBoxInterface());
      this.setLinkOverlapNodesForbidden(ssource.isLinkOverlapNodesForbidden());
    }
  },

  detach : function()
  {
    // summary:
    //		Detaches the graph model from the layout instance. 
    // description:
    //		Detaches the graph model from the layout instance. 
    //		When you attach a new graph model to the layout instance, you do not
    //		need to detach the old graph model because this is done
    //		automatically when you call <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.attach()</code>.  
    //		<p>
    //		The method performs cleaning operations on the graph model. In
    //		addition to the cleaning operations in the base class, the Short
    //		Link Layout removes the link style specifications of individual
    //		links.  
    //		</p>
    //		<p>
    //		Note that you must call this method when you no longer need the
    //		layout instance. Otherwise, some objects may not be garbage
    //		collected.  
    //		</p>
    
    if (this._shortLinkAlgorithm != null) {
      this._shortLinkAlgorithm.detach();
    }
    this.inherited("detach", arguments, []);
  },

  layout : function()
  {
    // summary:
    //		Computes the layout using the Short Link Layout algorithm. 
    // description:
    //		Computes the layout using the Short Link Layout algorithm. 
    //		To start the layout, call the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>.   
    // tags:
    //		protected
    
    if (this.getGlobalLinkStyle() == ShortLinkLayout.NO_RESHAPE_STYLE) {
      return;
    }
    var redraw = false;
    this._shortLinkAlgorithm.layout(redraw);
  },

  setLinkOverlapNodesForbidden : function(s)
  {
    // summary:
    //		Sets whether it is forbidden for the layout to reshape links such
    //		that they overlap nodes. 
    // description:
    //		Sets whether it is forbidden for the layout to reshape links such
    //		that they overlap nodes. 
    //		<p>
    //		The default value of this parameter is <code>false</code>.  
    //		</p>
    //		<p>
    //		If overlaps are not forbidden, the algorithm tries to avoid overlaps
    //		anyway. However, it may create overlaps if, for example, it needs to
    //		prevent a link from crossing other links.  
    //		</p>
    //		<p>
    //		If overlaps are forbidden, the algorithm uses the auxiliary long
    //		link layout (see <code>
    //		ShortLinkLayout.getAuxiliaryLongLinkLayout()</code> to lay out links
    //		that would otherwise overlap nodes in an orthogonal shape.  
    //		</p>
    //		<p>
    //		The auxiliary long link layout works on a grid and the default grid
    //		size may need to be reduced to allow this algorithm to avoid
    //		overlaps (see <code>
    //		ibm&#95;ilog.graphlayout.longlink.LongLinkLayout.setHorizontalGridOffset()</code>
    //		and <code>
    //		ibm&#95;ilog.graphlayout.longlink.LongLinkLayout.setVerticalGridOffset()</code>).
    //		 
    //		</p>
    //		<p>
    //		Forbidding overlaps can slow down the layout and can produce
    //		additional link bends and link crossings.  
    //		</p>
    
    if (s != this._linkOverlapNodesForbidden)
    {
      this._linkOverlapNodesForbidden = s;
      this.onParameterChanged("LinkOverlapNodesForbidden");
    }
  },

  isLinkOverlapNodesForbidden : function()
  {
    // summary:
    //		Returns 'true' if the overlap of links with nodes is
    //		forbidden; returns 'false' otherwise.
    
    return (false ? ShortLinkLayout._defaultLinkOverlapNodesForbidden : this._linkOverlapNodesForbidden); // Boolean
  },

  getAuxiliaryLongLinkLayout : function()
  {
    // summary:
    //		Returns the instance of '
    //		ibm_ilog.graphlayout.longlink.LongLinkLayout' that is used for
    //		avoiding overlaps between links and nodes when '
    //		ShortLinkLayout.isLinkOverlapNodesForbidden()' returns
    //		'true'. 
    // description:
    //		Returns the instance of <code>
    //		ibm&#95;ilog.graphlayout.longlink.LongLinkLayout</code> that is used for
    //		avoiding overlaps between links and nodes when <code>
    //		ShortLinkLayout.isLinkOverlapNodesForbidden()</code> returns
    //		<code>true</code>. 
    //		<p>
    //		This method allows you to get this auxiliary layout instance and to
    //		customize its parameters if needed. In particular, you can reduce or
    //		increase the grid size (see <code>
    //		ibm&#95;ilog.graphlayout.longlink.LongLinkLayout.setHorizontalGridOffset()</code>
    //		and <code>
    //		ibm&#95;ilog.graphlayout.longlink.LongLinkLayout.setVerticalGridOffset()</code>).
    //		 
    //		</p>
    //		<p>
    //		You should not modify the origin and destination point mode, nor
    //		disable the preservation of fixed links. An <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel</code> instance is
    //		attached to the <code>
    //		ibm&#95;ilog.graphlayout.longlink.LongLinkLayout</code> instance only if
    //		needed, therefore  
    //		</p>
    //		<pre>
    //		getAuxiliaryLongLinkLayout().getGraphModel()
    //		</pre>
    //		may return <code>null</code>.  
    // returns:
    //		The instance of '
    //		ibm_ilog.graphlayout.longlink.LongLinkLayout' that is used for
    //		avoiding overlaps between links and nodes when '
    //		ShortLinkLayout.isLinkOverlapNodesForbidden()' returns
    //		'true'.
    
    return this._shortLinkAlgorithm.getAuxiliaryLongLinkLayout(); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  setLinkConnectionBoxInterface : function(connectionBoxInterface)
  {
    // summary:
    //		Sets the link connection box interface for the connection points of
    //		links. 
    // description:
    //		Sets the link connection box interface for the connection points of
    //		links. 
    //		The link connection box interface is an object that provides the
    //		rectangle to which the links are connected for each node, and the
    //		tangential shift offset at each side for the connection points.  
    //		<p>
    //		By default, the layout algorithm places the connection points of the
    //		links relative to the bounding box of the nodes (see  ).  
    //		</p>
    //		<p>
    //		By setting a link connection box interface, the links can be
    //		connected relative to a different box (see <code>
    //		ibm&#95;ilog.graphlayout.ILinkConnectionBoxProvider.getBox()</code>).  
    //		</p>
    //		<p>
    //		By default, the links are connected symmetrically to the middle of
    //		the box, and it is possible to "shift" the links tangentially by
    //		using the method <code>
    //		ibm&#95;ilog.graphlayout.ILinkConnectionBoxProvider.getTangentialOffset()</code>
    //		from the interface <code>
    //		ibm&#95;ilog.graphlayout.ILinkConnectionBoxProvider</code>:  
    //		<ul>
    //		<li> On the "top" and "bottom" sides of a node, the connection
    //		points are moved to the right if the offset is positive and to the
    //		left if the offset is negative.</li>  
    //		<li> On the "left" and "right" sides of a node, the connection
    //		points are moved to the bottom if the offset is positive and to the
    //		top if the offset is negative.</li>  
    //		</ul>
    //		This method must return <code>0</code> if no offset is necessary on
    //		a certain node and/or side.  
    //		</p>
    //		<p>
    //		If you do not call this method , the layout algorithm places the
    //		connection points on the node box that has been customized using the
    //		method <code> ShortLinkLayout.setNodeBoxInterface()</code>. If this
    //		method has not been used, the bounding box of the node in the graph
    //		model (see  ) is used.  
    //		</p>
    //		<p>
    //		You can call this method with a <code>null</code> argument to go
    //		back to the default behavior.  
    //		</p>
    
    this.inherited("setLinkConnectionBoxInterface", arguments, [connectionBoxInterface]);
  },

  setNodeBoxInterface : function(nodeBoxInterface)
  {
    // summary:
    //		Sets an object that provides the rectangle defining the position and
    //		size of each node. 
    // description:
    //		Sets an object that provides the rectangle defining the position and
    //		size of each node. 
    //		<p>
    //		If you do not call this method , the layout algorithm uses the
    //		bounding box of the node (see  ).  
    //		</p>
    //		<p>
    //		You can call this method with a <code>null</code> argument  to go
    //		back to the default behavior.  
    //		</p>
    
    if (this._nodeBoxInterface != nodeBoxInterface)
    {
      this._nodeBoxInterface = nodeBoxInterface;
      this.onParameterChanged("NodeBoxInterface");
    }
  },

  getNodeBoxInterface : function()
  {
    // summary:
    //		Returns the node box interface used during the layout. 
    // description:
    //		Returns the node box interface used during the layout. 
    //		Returns <code>null</code> if none is specified.
    
    return this._nodeBoxInterface; // ibm_ilog.graphlayout.INodeBoxProvider
  },

  setNodeSideFilter : function(filter)
  {
    // summary:
    //		Sets the node-side filter, that is, an object that allows or
    //		prohibits the connection of the links on a given side of the nodes. 
    // description:
    //		Sets the node-side filter, that is, an object that allows or
    //		prohibits the connection of the links on a given side of the nodes. 
    //		<p>
    //		If a node-side filter is not installed, and the connection points of
    //		the links are not fixed (see <code>
    //		ShortLinkLayout.setOriginPointMode()</code>, <code>
    //		ShortLinkLayout.setDestinationPointMode()</code>), the layout
    //		algorithm is free to choose any side of the nodes, except for
    //		"pinned" connection points (see <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).
    //		 
    //		</p>
    //		<p>
    //		When a filter is installed, the layout algorithm asks the filter at
    //		which side of its end node each link is allowed to attach to.  
    //		</p>
    //		<p>
    //		Note the following points:  
    //		<ul>
    //		<li>A filter should always allow a link to connect to at least one
    //		side of a node; otherwise the link cannot be routed.</li>  
    //		<li>If the connection point on the origin or destination node is
    //		specified as fixed (see <code>
    //		ShortLinkLayout.setGlobalOriginPointMode()</code>, <code>
    //		ShortLinkLayout.setGlobalDestinationPointMode()</code>, <code>
    //		ShortLinkLayout.setOriginPointMode()</code>, <code>
    //		ShortLinkLayout.setDestinationPointMode()</code>), the filter is not
    //		used for the origin or the destination node, respectively.</li>  
    //		<li>When side constraints are added using the filter, the layout
    //		algorithm has less freedom for the choice of a link shape. Hence,
    //		laying out with a side filter usually produces more crossings and
    //		overlaps than laying out without side filter.</li>  
    //		</ul>
    //		You can call this method with a <code>null</code> argument  to
    //		return to the default behavior.  
    //		</p>
    // filter: INodeSideFilter
    //		The node-side filter.
    
    if (filter != this._nodeSideFilter)
    {
      this._nodeSideFilter = filter;
      this.onParameterChanged("NodeSideFilter");
    }
  },

  getNodeSideFilter : function()
  {
    // summary:
    //		Returns the node-side filter used during layout. 
    // description:
    //		Returns the node-side filter used during layout. 
    //		Returns <code>null</code> if none is specified.
    
    return this._nodeSideFilter; // ibm_ilog.graphlayout.INodeSideFilter
  },

  setGlobalLinkStyle : function(style)
  {
    // summary:
    //		Sets the global style of the shape of the links. 
    // description:
    //		Sets the global style of the shape of the links. 
    //		Valid values are the following:  
    //		<ul>
    //		<li><code> ShortLinkLayout.DIRECT&#95;STYLE</code>: the shape of the
    //		links is a polyline composed of a straight-line segment between the
    //		end nodes with two horizontal or vertical segments at its
    //		extremities (their length is controlled by <code>
    //		ShortLinkLayout.setMinFinalSegmentLength()</code> and may be
    //		0).</li>  
    //		<li><code> ShortLinkLayout.ORTHOGONAL&#95;STYLE</code>: the shape of the
    //		links is a polyline composed of alternating horizontal and vertical
    //		segments.</li>  
    //		<li><code> ShortLinkLayout.MIXED&#95;STYLE</code>: each link can have a
    //		different link style. The style of each individual link can be set
    //		by the method <code> ShortLinkLayout.setLinkStyle()</code> so that
    //		different link shapes can occur in the same graph.</li>  
    //		</ul>  
    //		<p>
    //		Unless the global link style is <code>
    //		ShortLinkLayout.MIXED&#95;STYLE</code>, all links have the same style of
    //		shape.  
    //		</p>
    //		<p>
    //		The default value is <code> ShortLinkLayout.ORTHOGONAL&#95;STYLE</code>.
    //		  
    //		</p>
    
    if (style != ShortLinkLayout.DIRECT_STYLE && style != ShortLinkLayout.ORTHOGONAL_STYLE && style != ShortLinkLayout.MIXED_STYLE && style != ShortLinkLayout.NO_RESHAPE_STYLE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6639E",  style);
    }

    if (style != this._globalLinkStyle)
    {
      this._globalLinkStyle = style;

      if (this._globalLinkStyle == ShortLinkLayout.MIXED_STYLE) {
        this._shortLinkAlgorithm.notifyMixedLinkStyle();
      }
      this.onParameterChanged("GlobalLinkStyle");
    }
  },

  getGlobalLinkStyle : function()
  {
    // summary:
    //		Returns the current choice for the style of the shape of the links.
    
    return (false ? ShortLinkLayout._defaultGlobalLinkStyle : this._globalLinkStyle); // int
  },

  setLinkStyle : function(link, style)
  {
    // summary:
    //		Sets the style of the shape of an individual link. 
    // description:
    //		Sets the style of the shape of an individual link. 
    //		This link style is used only if the global link style is set to
    //		<code> ShortLinkLayout.MIXED&#95;STYLE</code>. Otherwise, all links have
    //		the style that is specified as the global link style.  
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> ShortLinkLayout.ORTHOGONAL&#95;STYLE</code> - the link gets
    //		an orthogonal shape, that is, a shape consisting of a sequence of
    //		orthogonal line segments.</li>  
    //		<li> <code> ShortLinkLayout.DIRECT&#95;STYLE</code> - the link gets a
    //		straight-line shape.</li>  
    //		<li> <code> ShortLinkLayout.NO&#95;RESHAPE&#95;STYLE</code> - the link is
    //		not reshaped.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code> ShortLinkLayout.ORTHOGONAL&#95;STYLE</code>.
    //		 
    //		</p>
    //		<p>
    //		  
    //		</p>
    // link: Object
    //		The link instance.
    // style: int
    //		The style of the shape of an individual link.
    
    if (!(style == ShortLinkLayout.ORTHOGONAL_STYLE || style == ShortLinkLayout.NO_RESHAPE_STYLE || style == ShortLinkLayout.DIRECT_STYLE))
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6640E",  style);
    }

    LayoutParametersUtil.setLinkParameter(this, link, ShortLinkLayout._LINK_STYLE_PROPERTY, style, ShortLinkLayout._defaultGlobalLinkStyle);
  },

  getLinkStyle : function(link)
  {
    // summary:
    //		Returns the style of the shape of an individual link.  
    // link: Object
    //		The link instance.
    // returns:
    //		The style of the shape of an individual link. Possible values
    //		are: ' ShortLinkLayout.ORTHOGONAL_STYLE', '
    //		ShortLinkLayout.DIRECT_STYLE' or '
    //		ShortLinkLayout.NO_RESHAPE_STYLE'.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, ShortLinkLayout._LINK_STYLE_PROPERTY, ShortLinkLayout._defaultGlobalLinkStyle); // int
  },

  setGlobalSelfLinkStyle : function(style)
  {
    // summary:
    //		Sets the global style of the shape of the self-links. 
    // description:
    //		Sets the global style of the shape of the self-links. 
    //		Self-links are links that have the same origin and destination node.
    //		 
    //		<p>
    //		Valid values are the following:  
    //		<ul>
    //		<li> <code> ShortLinkLayout.TWO&#95;BENDS&#95;ORTHOGONAL&#95;STYLE</code>: the
    //		shape of the links is a polyline composed of three alternating
    //		horizontal and vertical segments, connected to the same side of the
    //		node.</li>  
    //		<li> <code> ShortLinkLayout.THREE&#95;BENDS&#95;ORTHOGONAL&#95;STYLE</code>: the
    //		shape of the links is a polyline composed of four alternating
    //		horizontal and vertical segments, connected to two adjacent sides of
    //		the node.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code>
    //		ShortLinkLayout.TWO&#95;BENDS&#95;ORTHOGONAL&#95;STYLE</code>.   
    //		</p>
    
    if (style != ShortLinkLayout.TWO_BENDS_ORTHOGONAL_STYLE && style != ShortLinkLayout.THREE_BENDS_ORTHOGONAL_STYLE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6641E",  style);
    }

    if (style != this._globalSelfLinkStyle)
    {
      this._globalSelfLinkStyle = style;
      this.onParameterChanged("GlobalSelfLinkStyle");
    }
  },

  getGlobalSelfLinkStyle : function()
  {
    // summary:
    //		Returns the current choice for the style of the shape of the
    //		self-links.
    
    return (false ? ShortLinkLayout._defaultGlobalSelfLinkStyle : this._globalSelfLinkStyle); // int
  },

  setGlobalOriginPointMode : function(mode)
  {
    // summary:
    //		Sets the global mode for the connection point of the links on the
    //		origin nodes. 
    // description:
    //		Sets the global mode for the connection point of the links on the
    //		origin nodes. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> ShortLinkLayout.FREE&#95;MODE</code> - For all links, the
    //		layout is free to choose the appropriate position of the connection
    //		point on the origin node, except for "pinned" connection points (see
    //		<code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> ShortLinkLayout.FIXED&#95;MODE</code> - For all links, the
    //		layout must keep the current position of the connection point on the
    //		origin node. This can be useful, in particular, if the node has
    //		fixed pins where the link is attached (as, for instance, with <code>
    //		ibm&#95;ilog.linkconnector.PinLinkConnector</code>) and the pins should
    //		not be moved by the layout algorithm.  
    //		<p>
    //		Note that, in this mode, the layout algorithm has less freedom for
    //		the choice of a link shape. Hence, this mode usually produces more
    //		crossings and overlaps than the mode <code>
    //		ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    //		</li>  
    //		<li> <code> ShortLinkLayout.MIXED&#95;MODE</code> - Each link can have a
    //		different mode for the connection point on the origin node. The mode
    //		of each individual link can be set by the method <code>
    //		ShortLinkLayout.setOriginPointMode()</code>.</li>  
    //		</ul>  
    //		<p>
    //		Unless the global mode is <code> ShortLinkLayout.MIXED&#95;MODE</code>,
    //		the connection points on the origin node have the same mode for all
    //		the links.  
    //		</p>
    //		<p>
    //		The default value is <code> ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // mode: int
    //		The global mode to set.
    
    if (mode != ShortLinkLayout.FREE_MODE && mode != ShortLinkLayout.FIXED_MODE && mode != ShortLinkLayout.MIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6637E",  mode);
    }

    if (mode != this._originPointMode)
    {
      this._originPointMode = mode;
      this.onParameterChanged("GlobalOriginPointMode");
    }
  },

  getGlobalOriginPointMode : function()
  {
    // summary:
    //		Returns the global mode for the connection points of the links on
    //		the origin nodes.
    
    return (false ? ShortLinkLayout._defaultOriginPointMode : this._originPointMode); // int
  },

  setGlobalDestinationPointMode : function(mode)
  {
    // summary:
    //		Sets the global mode for the connection point of the links on the
    //		destination nodes. 
    // description:
    //		Sets the global mode for the connection point of the links on the
    //		destination nodes. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> ShortLinkLayout.FREE&#95;MODE</code> - For all links, the
    //		layout is free to choose the appropriate position of the connection
    //		point on the destination node, except for "pinned" connection points
    //		(see <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> ShortLinkLayout.FIXED&#95;MODE</code> - For all links, the
    //		layout must keep the current position of the connection point on the
    //		destination node. This can be useful, in particular, if the node has
    //		fixed pins where the link is attached (as, for instance, with <code>
    //		ibm&#95;ilog.linkconnector.PinLinkConnector</code>) and the pins should
    //		not be moved by the layout algorithm.  
    //		<p>
    //		Note that, in this mode, the layout algorithm has less freedom for
    //		the choice of a link shape. Hence, this mode usually produces more
    //		crossings and overlaps than the mode <code>
    //		ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    //		</li>  
    //		<li> <code> ShortLinkLayout.MIXED&#95;MODE</code> - Each link can have a
    //		different mode for the connection point on the destination node. The
    //		mode of each individual link can be set by the method <code>
    //		ShortLinkLayout.setDestinationPointMode()</code>.</li>  
    //		</ul>  
    //		<p>
    //		Unless the global mode is <code> ShortLinkLayout.MIXED&#95;MODE</code>,
    //		the connection points on the destination node have the same mode for
    //		all the links.  
    //		</p>
    //		<p>
    //		The default value is <code> ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // mode: int
    //		The global mode to set.
    
    if (mode != ShortLinkLayout.FREE_MODE && mode != ShortLinkLayout.FIXED_MODE && mode != ShortLinkLayout.MIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6638E",  mode);
    }

    if (mode != this._destinationPointMode)
    {
      this._destinationPointMode = mode;
      this.onParameterChanged("GlobalDestinationPointMode");
    }
  },

  getGlobalDestinationPointMode : function()
  {
    // summary:
    //		Returns the global mode for the connection points of the links on
    //		the destination nodes.
    
    return (false ? ShortLinkLayout._defaultDestinationPointMode : this._destinationPointMode); // int
  },

  setOriginPointMode : function(link, mode)
  {
    // summary:
    //		Sets the mode for the connection point on an individual link on the
    //		origin node. 
    // description:
    //		Sets the mode for the connection point on an individual link on the
    //		origin node. 
    //		This mode is used only if the global from-point mode is set to
    //		<code> ShortLinkLayout.MIXED&#95;MODE</code>. Otherwise, the connection
    //		points on the origin nodes have, for all the links, the mode that is
    //		specified as the global origin point mode.  
    //		<p>
    //		Valid values are: 
    //		</p>
    //		<ul>
    //		<li> <code> ShortLinkLayout.FREE&#95;MODE</code> - The layout is free to
    //		choose, for this link, the appropriate position of the connection
    //		point on the origin node, except for "pinned" connection points (see
    //		<code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> ShortLinkLayout.FIXED&#95;MODE</code> - The layout must
    //		keep, for this link, the current position of the connection point on
    //		the origin node. This can be useful, in particular, if the node has
    //		fixed pins where the link is attached (as, for instance, with <code>
    //		ibm&#95;ilog.linkconnector.PinLinkConnector</code>) and the pins should
    //		not be moved by the layout algorithm.  
    //		<p>
    //		Note that, in this mode, the layout algorithm has less freedom for
    //		the choice of a link shape. Hence, this mode usually produces more
    //		crossings and overlaps than the mode <code>
    //		ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    //		</li>  
    //		</ul>  
    //		<p>
    //		The default value is <code> ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // link: Object
    //		The link.
    // mode: int
    //		The mode to set.
    
    if (mode != ShortLinkLayout.FREE_MODE && mode != ShortLinkLayout.FIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6637E",  mode);
    }

    LayoutParametersUtil.setLinkParameter(this, link, ShortLinkLayout._ORIGIN_POINT_MODE_PROPERTY, mode, ShortLinkLayout._defaultOriginPointMode);
  },

  getOriginPointMode : function(link)
  {
    // summary:
    //		Returns the origin-point mode of an individual link.  
    // link: Object
    //		The link.
    // returns:
    //		The mode for the connection point of an individual link on the
    //		origin node.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, ShortLinkLayout._ORIGIN_POINT_MODE_PROPERTY, ShortLinkLayout._defaultOriginPointMode); // int
  },

  setDestinationPointMode : function(link, mode)
  {
    // summary:
    //		Sets the mode for the connection point of an individual link on the
    //		destination node. 
    // description:
    //		Sets the mode for the connection point of an individual link on the
    //		destination node. 
    //		This mode is used only if the global from-point mode is set to
    //		<code> ShortLinkLayout.MIXED&#95;MODE</code>. Otherwise, the connection
    //		points on the destination nodes have, for all the links, the mode
    //		that is specified as the global destination point mode.  
    //		<p>
    //		Valid values are: 
    //		</p>
    //		<ul>
    //		<li> <code> ShortLinkLayout.FREE&#95;MODE</code> - The layout is free to
    //		choose, for this link, the appropriate position of the connection
    //		point on the destination node, except for "pinned" connection points
    //		(see <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> ShortLinkLayout.FIXED&#95;MODE</code> - The layout must
    //		keep, for this link, the current position of the connection point on
    //		the destination node. This can be useful, in particular, if the node
    //		has fixed pins where the link is attached (as, for instance, with
    //		<code> ibm&#95;ilog.linkconnector.PinLinkConnector</code>) and the pins
    //		should not be moved by the layout algorithm.  
    //		<p>
    //		Note that, in this mode, the layout algorithm has less freedom for
    //		the choice of a link shape. Hence, this mode usually produces more
    //		crossings and overlaps than the mode <code>
    //		ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    //		</li>  
    //		</ul>  
    //		<p>
    //		The default value is <code> ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // link: Object
    //		The link.
    // mode: int
    //		The mode to set.
    
    if (mode != ShortLinkLayout.FREE_MODE && mode != ShortLinkLayout.FIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6638E",  mode);
    }

    LayoutParametersUtil.setLinkParameter(this, link, ShortLinkLayout._DESTINATION_POINT_MODE_PROPERTY, mode, ShortLinkLayout._defaultDestinationPointMode);
  },

  getDestinationPointMode : function(link)
  {
    // summary:
    //		Returns the destination-point mode of an individual link.  
    // link: Object
    //		The link.
    // returns:
    //		The mode for the connection point of an individual link on the
    //		destination node.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, ShortLinkLayout._DESTINATION_POINT_MODE_PROPERTY, ShortLinkLayout._defaultOriginPointMode); // int
  },

  isFromPointFixed : function(link)
  {
    // summary:
    //		Returns 'true' if the connection point of
    //		'link' on the origin node must be fixed. 
    // description:
    //		Returns <code>true</code> if the connection point of
    //		<code>link</code> on the origin node must be fixed. 
    //		Returns <code>false</code> otherwise.
    // tags:
    //		private
    
    switch (this.getGlobalOriginPointMode())
    {
      case ShortLinkLayout.FREE_MODE:
        return this.getGraphModel().hasPinnedConnectionPoint(link, true); // Boolean
      case ShortLinkLayout.FIXED_MODE:
        return true; // Boolean
      case ShortLinkLayout.MIXED_MODE:
        return (this.getOriginPointMode(link) == ShortLinkLayout.FIXED_MODE ? true : this.getGraphModel().hasPinnedConnectionPoint(link, true)); // Boolean
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6637E",  this.getGlobalOriginPointMode());
        return false; // Boolean
    }
  },

  isToPointFixed : function(link)
  {
    // summary:
    //		Returns 'true' if the connection point of
    //		'link' on the destination node must be fixed. 
    // description:
    //		Returns <code>true</code> if the connection point of
    //		<code>link</code> on the destination node must be fixed. 
    //		Returns <code>false</code> otherwise.
    // tags:
    //		private
    
    switch (this.getGlobalDestinationPointMode())
    {
      case ShortLinkLayout.FREE_MODE:
        return this.getGraphModel().hasPinnedConnectionPoint(link, false); // Boolean
      case ShortLinkLayout.FIXED_MODE:
        return true; // Boolean
      case ShortLinkLayout.MIXED_MODE:
        return (this.getDestinationPointMode(link) == ShortLinkLayout.FIXED_MODE ? true : this.getGraphModel().hasPinnedConnectionPoint(link, false)); // Boolean
      default:
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6638E",  this.getGlobalDestinationPointMode());
        return false; // Boolean
    }
  },

  setGlobalConnectorStyle : function(style)
  {
    // summary:
    //		Sets the global style of connectors. 
    // description:
    //		Sets the global style of connectors. 
    //		Valid values are the following:  
    //		<ul>
    //		<li><code> ShortLinkLayout.FIXED&#95;OFFSET&#95;PINS</code> - The link
    //		connection points are spaced along the node border at a distance
    //		equal to the <i>link offset</i> (see <code>
    //		ShortLinkLayout.getLinkOffset()</code>).</li>  
    //		<li><code> ShortLinkLayout.EVENLY&#95;SPACED&#95;PINS</code> - The link
    //		connection points are evenly spaced along the node border,
    //		preserving a margin which is determined by <code>
    //		ShortLinkLayout.getEvenlySpacedPinsMarginRatio()</code>.</li>  
    //		<li><code> ShortLinkLayout.AUTOMATIC&#95;PINS</code> - Uses the
    //		connector style <code> ShortLinkLayout.FIXED&#95;OFFSET&#95;PINS</code>
    //		except if this pushes a connection point outside the border to which
    //		the link is attached, in which case it uses the connector style
    //		<code> ShortLinkLayout.EVENLY&#95;SPACED&#95;PINS</code>.</li>  
    //		<li><code> ShortLinkLayout.MIXED&#95;STYLE</code>: each node can have a
    //		different connector style. The style of each individual node can be
    //		set by the method <code> ShortLinkLayout.setConnectorStyle()</code>
    //		so that different connector styles can occur in the same graph.</li>
    //		 
    //		</ul>  
    //		<p>
    //		Unless the global connector style is <code>
    //		ShortLinkLayout.MIXED&#95;STYLE</code>, all links have the same
    //		connector style.  
    //		</p>
    //		<p>
    //		The default value is <code> ShortLinkLayout.AUTOMATIC&#95;PINS</code>.  
    //		</p>
    //		<p>
    //		The connector style is used to place the connection points of links
    //		that have the free origin-point or destination-point mode and no
    //		specified port index. Links with fixed-point mode do not change
    //		their connection point at the origin node. The placement of
    //		connection points also depends on the connection box interface in
    //		the following way: 
    //		<ol>
    //		 
    //		<li> First, the bounding box of an end node is obtained. If a link
    //		connection box interface was set by using <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setLinkConnectionBoxInterface()</code>,
    //		the bounding box and offsets are delivered by that interface.
    //		Otherwise, the node bounding box delivered by the graph model (see 
    //		) is used. </li>  
    //		<li> Next, the connection point of the link is placed at this
    //		bounding box in the manner specified by the connector style. </li> 
    //		</ol>   
    //		</p>
    // style: int
    //		The connector style.
    
    if (!(style == ShortLinkLayout.AUTOMATIC_PINS || style == ShortLinkLayout.FIXED_OFFSET_PINS || style == ShortLinkLayout.EVENLY_SPACED_PINS || style == ShortLinkLayout.MIXED_STYLE))
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6642E",  style);
    }

    if (style != this._globalConnectorStyle)
    {
      this._globalConnectorStyle = style;
      this.onParameterChanged("GlobalConnectorStyle");
    }
  },

  getGlobalConnectorStyle : function()
  {
    // summary:
    //		Returns the global style of the connectors.
    
    return (false ? ShortLinkLayout._defaultGlobalConnectorStyle : this._globalConnectorStyle); // int
  },

  setConnectorStyle : function(node, style)
  {
    // summary:
    //		Sets the connector style of an individual node. 
    // description:
    //		Sets the connector style of an individual node. 
    //		This connector style is used only if the global connector style is
    //		set to <code> ShortLinkLayout.MIXED&#95;STYLE</code>. Otherwise, all
    //		nodes have the connector style that is specified as the global
    //		connector style.  
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li><code> ShortLinkLayout.FIXED&#95;OFFSET&#95;PINS</code> - The link
    //		connection points are spaced along the node border at a distance
    //		equal to the <i>link offset</i> (see <code>
    //		ShortLinkLayout.getLinkOffset()</code>).</li>  
    //		<li><code> ShortLinkLayout.EVENLY&#95;SPACED&#95;PINS</code> - The link
    //		connection points are evenly spaced along the node border,
    //		preserving a margin which is determined by <code>
    //		ShortLinkLayout.getEvenlySpacedPinsMarginRatio()</code>.</li>  
    //		<li><code> ShortLinkLayout.AUTOMATIC&#95;PINS</code> - Uses the
    //		connector style <code> ShortLinkLayout.FIXED&#95;OFFSET&#95;PINS</code>
    //		except if this pushes a connection point outside the border the link
    //		is attached to, in which case it uses the connector style <code>
    //		ShortLinkLayout.EVENLY&#95;SPACED&#95;PINS</code>.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code> ShortLinkLayout.AUTOMATIC&#95;PINS</code>.  
    //		
    //		</p>
    // node: Object
    //		The node.
    // style: int
    //		The connector style.
    
    if (!(style == ShortLinkLayout.AUTOMATIC_PINS || style == ShortLinkLayout.FIXED_OFFSET_PINS || style == ShortLinkLayout.EVENLY_SPACED_PINS))
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6642E",  style);
    }

    LayoutParametersUtil.setNodeParameter(this, node, ShortLinkLayout._CONNECTOR_STYLE_PROPERTY, style, ShortLinkLayout._defaultGlobalConnectorStyle);
  },

  getConnectorStyle : function(node)
  {
    // summary:
    //		Returns the connector style of an individual node.  
    // node: Object
    //		The node.
    // returns:
    //		The connector style of an individual node.
    
    return LayoutParametersUtil.getNodeEnumParameter(this, node, ShortLinkLayout._CONNECTOR_STYLE_PROPERTY, ShortLinkLayout._defaultGlobalConnectorStyle); // int
  },

  setEvenlySpacedPinsMarginRatio : function(ratio)
  {
    // summary:
    //		Sets the margin ratio for the evenly spaced connector style. 
    // description:
    //		Sets the margin ratio for the evenly spaced connector style. 
    //		This option has no effect if the connector style <code>
    //		ShortLinkLayout.FIXED&#95;OFFSET&#95;PINS</code> is used.  
    //		<p>
    //		In evenly spaced connector style (see <code>
    //		ShortLinkLayout.setGlobalConnectorStyle()</code>), the connection
    //		points of the links are evenly spaced along the node border,
    //		preserving a margin at each extremity of the node border. The size
    //		of this margin is controlled by the margin ratio, and is computed by
    //		multiplying the offset between the links by the ratio.  
    //		</p>
    //		<p>
    //		The input value must be a positive or zero value. A ratio of
    //		<code>0</code> means no margin; a ratio of <code>1</code> means a
    //		margin equal to the offset between the links; a ratio of
    //		<code>0.5</code> means a margin equal to half the offset between the
    //		links; a ratio of <code>2</code> means a margin equal to twice the
    //		offset between the links and so on. There is no upper bound for the
    //		ratio; however, the margin which is obtained has an upper bound
    //		equal to <code>(node-side-dimension - total-width-of-links) /
    //		2</code>.  
    //		</p>
    //		<p>
    //		The default value for the ratio is <code>0.5</code>.  
    //		</p>
    // ratio: Number
    //		The margin ratio for the evenly spaced pins connector
    //		style.
    
    if (ratio < 0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6643E",  ratio);
    }

    if (ratio != this._evenlySpacedPinsMarginRatio)
    {
      this._evenlySpacedPinsMarginRatio = ratio;
      this.onParameterChanged("EvenlySpacedPinsMarginRatio");
    }
  },

  getEvenlySpacedPinsMarginRatio : function()
  {
    // summary:
    //		Returns the margin ratio for the evenly spaced connector style.
    
    return (false ? ShortLinkLayout._defaultEvenlySpacedPinsMarginRatio : this._evenlySpacedPinsMarginRatio); // Number
  },

  setLinkBundlesMode : function(mode)
  {
    // summary:
    //		Sets the link bundles mode. 
    // description:
    //		Sets the link bundles mode. 
    //		The valid modes are the following:  
    //		<ul>
    //		<li> <code> ShortLinkLayout.NO&#95;BUNDLE</code>: Several links
    //		connected to the same side of a node share the same connection
    //		point, that is, the middle of the node side. The final segments of
    //		the links overlap. No bundles are created and the link offset
    //		parameter is ignored.</li>   
    //		<li> <code> ShortLinkLayout.FIRST&#95;LAST&#95;SEGMENT</code>: The
    //		connection points of several links connected to the same side of a
    //		node are distributed according to the link offset value. With the
    //		orthogonal link style, the algorithm additionally ensures that the
    //		first and last segments of the links are always orthogonal to the
    //		side of the node to which they are connected.</li>  
    //		<li> <code> ShortLinkLayout.IMPROVED&#95;FIRST&#95;LAST&#95;SEGMENT</code>: The
    //		algorithm additionally tries to avoid overlapping the second segment
    //		and the segment next to the last. Note that with the direct link
    //		style, this mode is equivalent to <code>
    //		ShortLinkLayout.FIRST&#95;LAST&#95;SEGMENT</code>.</li>    
    //		</ul>  
    //		<p>
    //		The default value is <code>
    //		ShortLinkLayout.IMPROVED&#95;FIRST&#95;LAST&#95;SEGMENT</code>.  
    //		</p>
    
    if (mode != ShortLinkLayout.NO_BUNDLE && mode != ShortLinkLayout.FIRST_LAST_SEGMENT && mode != ShortLinkLayout.IMPROVED_FIRST_LAST_SEGMENT)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6644E",  mode);
    }

    if (mode != this._linkBundlesMode)
    {
      this._linkBundlesMode = mode;
      this.onParameterChanged("LinkBundlesMode");
    }
  },

  getLinkBundlesMode : function()
  {
    // summary:
    //		Returns the current choice for the link bundles mode.
    
    return (false ? ShortLinkLayout._defaultLinkBundlesMode : this._linkBundlesMode); // int
  },

  setIncrementalMode : function(enable)
  {
    // summary:
    //		Sets the incremental layout mode. 
    // description:
    //		Sets the incremental layout mode. 
    //		If the argument is <code>true</code>, the algorithm tries to create
    //		a layout with similar shapes for the links that were already laid
    //		out in the last layout  execution, in order to preserve the
    //		"mental map" of the user as much as  possible. 
    //		<p>
    //		The exact incremental behavior is controlled by the 
    //		"incremental link reshape" mode. For details, see <code>
    //		ShortLinkLayout.setGlobalIncrementalUnmodifiedLinkReshapeMode()</code>
    //		and <code>
    //		ShortLinkLayout.setGlobalIncrementalModifiedLinkReshapeMode()</code>.
    //		 
    //		</p>
    //		<p>
    //		Incremental layout works only if the intergraph links mode is
    //		disabled or if there are no intergraph links. Also, incremental
    //		layout has no effect if the layout parameters have been changed
    //		since the previous layout.  
    //		</p>
    //		<p>
    //		Incremental layout is disabled by default.  
    //		</p>
    // enable: Boolean
    //		Set to 'true' to enable incremental layout, or
    //		'false' to disable it.
    
    if (enable != this._incrementalMode)
    {
      this._incrementalMode = enable;
      this.onParameterChanged("IncrementalMode");
    }
  },

  isIncrementalMode : function()
  {
    // summary:
    //		Returns 'true' if the incremental mode is enabled. 
    // description:
    //		Returns <code>true</code> if the incremental mode is enabled. 
    //		It returns <code>false</code> otherwise.
    
    return (false ? ShortLinkLayout._defaultIncrementalMode : this._incrementalMode); // Boolean
  },

  setGlobalIncrementalUnmodifiedLinkReshapeMode : function(mode)
  {
    // summary:
    //		Sets the reshape mode used during the incremental layout of
    //		unmodified links, that is, links that have already been laid out in
    //		the previous layout execution and for which neither the bounding box
    //		(see ' ShortLinkLayout.getNodeBoxInterface()' nor the
    //		"link connection box" (see '
    //		ibm_ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()')
    //		of their origin and destination nodes have been modified. 
    // description:
    //		Sets the reshape mode used during the incremental layout of
    //		unmodified links, that is, links that have already been laid out in
    //		the previous layout execution and for which neither the bounding box
    //		(see <code> ShortLinkLayout.getNodeBoxInterface()</code> nor the
    //		"link connection box" (see <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()</code>)
    //		of their origin and destination nodes have been modified. 
    //		<p>
    //		This mode has no effect if the incremental mode is disabled.  
    //		</p>
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li><code> ShortLinkLayout.FIXED&#95;SHAPE&#95;TYPE&#95;MODE</code> - The
    //		incremental layout preserves preserves the shape type of the link.
    //		This means that both the number of bends and the node sides to which
    //		the link is connected are preserved.</li>  
    //		<li><code> ShortLinkLayout.FIXED&#95;NODE&#95;SIDES&#95;MODE</code> - The
    //		incremental layout preserves the node sides to which the links are
    //		connected.</li>  
    //		<li><code> ShortLinkLayout.FIXED&#95;CONNECTION&#95;POINTS&#95;MODE</code> - The
    //		incremental layout preserves the connection points of the
    //		links.</li>  
    //		<li><code> ShortLinkLayout.FIXED&#95;MODE</code> - The links are not
    //		reshaped at all during incremental layout. Only newly added links
    //		are rerouted.</li>  
    //		<li><code> ShortLinkLayout.FREE&#95;MODE</code> - The incremental layout
    //		is allowed to freely reshape the links. This is equivalent to a
    //		non-incremental behavior for all the links, hence it is recommended
    //		to disable the incremental mode (<code>
    //		ShortLinkLayout.setIncrementalMode()</code>) instead of setting
    //		<code> ShortLinkLayout.FREE&#95;MODE</code> as global incremental
    //		reshape mode. (Of course, settings that may have been done using
    //		<code> ibm&#95;ilog.graphlayout.GraphLayout.isFixed()</code>, <code>
    //		ShortLinkLayout.getGlobalOriginPointMode()</code> and <code>
    //		ShortLinkLayout.getGlobalDestinationPointMode()</code> are still
    //		obeyed.)</li>  
    //		<li><code> ShortLinkLayout.MIXED&#95;MODE</code> - Each link can have a
    //		different mode. The mode of each individual link can be set by the
    //		method <code>
    //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()</code>.</li>
    //		 
    //		</ul>  
    //		</p>
    //		<p>
    //		Unless the global mode is <code> ShortLinkLayout.MIXED&#95;MODE</code>,
    //		all links are reshaped according to the same mode.  
    //		</p>
    //		<p>
    //		The default mode is <code>
    //		ShortLinkLayout.FIXED&#95;SHAPE&#95;TYPE&#95;MODE</code>.  
    //		</p>
    // mode: int
    //		The link reshape mode to set.
    
    if (
      !
        (mode == ShortLinkLayout.FREE_MODE || mode == ShortLinkLayout.FIXED_SHAPE_TYPE_MODE || mode == ShortLinkLayout.FIXED_NODE_SIDES_MODE || mode == ShortLinkLayout.FIXED_CONNECTION_POINTS_MODE ||
            mode == ShortLinkLayout.FIXED_MODE || mode == ShortLinkLayout.MIXED_MODE))
      {
        LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6645E",  mode);
      }

    if (mode != this._incrementalUnmodifiedLinkReshapeMode)
    {
      this._incrementalUnmodifiedLinkReshapeMode = mode;
      this.onParameterChanged("GlobalIncrementalUnmodifiedLinkReshapeMode");
    }
  },

  getGlobalIncrementalUnmodifiedLinkReshapeMode : function()
  {
    // summary:
    //		Returns the reshape mode used during the incremental layout of
    //		unmodified links.
    
    return (false ? ShortLinkLayout._defaultIncrementalUnmodifiedLinkReshapeMode : this._incrementalUnmodifiedLinkReshapeMode); // int
  },

  setIncrementalUnmodifiedLinkReshapeMode : function(link, mode)
  {
    // summary:
    //		Sets the reshape mode of an individual link used during the
    //		incremental layout of unmodified links, that is, links that have
    //		already been laid out in the previous layout execution and for which
    //		neither the bounding box (see '
    //		ShortLinkLayout.getNodeBoxInterface()' nor the
    //		"link connection box" (see '
    //		ibm_ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()')
    //		of their origin and destination nodes have been modified. 
    // description:
    //		Sets the reshape mode of an individual link used during the
    //		incremental layout of unmodified links, that is, links that have
    //		already been laid out in the previous layout execution and for which
    //		neither the bounding box (see <code>
    //		ShortLinkLayout.getNodeBoxInterface()</code> nor the
    //		"link connection box" (see <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()</code>)
    //		of their origin and destination nodes have been modified. 
    //		<p>
    //		This reshape mode is used only if incremental layout is enabled and
    //		the global link-reshape mode is set to <code>
    //		ShortLinkLayout.MIXED&#95;MODE</code>. If the global mode is not <code>
    //		ShortLinkLayout.MIXED&#95;MODE</code>, all links have the mode that is
    //		specified as the global link reshape mode.  
    //		</p>
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> ShortLinkLayout.FIXED&#95;SHAPE&#95;TYPE&#95;MODE</code> -
    //		Incremental layout preserves the shape type of the link. This means
    //		that both the number of bends and the node sides to which the link
    //		is connected are preserved.</li>  
    //		<li> <code> ShortLinkLayout.FIXED&#95;NODE&#95;SIDES&#95;MODE</code> -
    //		Incremental layout preserves the node sides to which the links are
    //		connected.</li>  
    //		<li> <code> ShortLinkLayout.FIXED&#95;CONNECTION&#95;POINTS&#95;MODE</code> -
    //		Incremental layout preserves the connection points of the
    //		links.</li>  
    //		<li> <code> ShortLinkLayout.FIXED&#95;MODE</code> - The links are not
    //		reshaped at all during incremental layout. Only newly added links
    //		are rerouted.</li>  
    //		<li> <code> ShortLinkLayout.FREE&#95;MODE</code> - Incremental layout is
    //		allowed to reshape the links freely. This is equivalent to
    //		nonincremental behavior for the <code>link</code> argument. Any
    //		settings done using <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.isFixed()</code>, <code>
    //		ShortLinkLayout.getGlobalOriginPointMode()</code> and <code>
    //		ShortLinkLayout.getGlobalDestinationPointMode()</code> are still
    //		obeyed.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default mode is <code>
    //		ShortLinkLayout.FIXED&#95;SHAPE&#95;TYPE&#95;MODE</code>.  
    //		</p>
    // link: Object
    //		The link.
    // mode: int
    //		The link reshape mode to set.
    
    if (
      !
        (mode == ShortLinkLayout.FREE_MODE || mode == ShortLinkLayout.FIXED_SHAPE_TYPE_MODE || mode == ShortLinkLayout.FIXED_NODE_SIDES_MODE || mode == ShortLinkLayout.FIXED_CONNECTION_POINTS_MODE ||
            mode == ShortLinkLayout.FIXED_MODE))
      {
        LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6646E",  mode);
      }

    LayoutParametersUtil.setLinkParameter(this, link, ShortLinkLayout._INCREMENTAL_UNMODIFIED_LINK_RESHAPE_MODE_PROPERTY, mode, ShortLinkLayout._defaultIncrementalUnmodifiedLinkReshapeMode);
  },

  getIncrementalUnmodifiedLinkReshapeMode : function(link)
  {
    // summary:
    //		Returns the reshape mode of an individual link used during the
    //		incremental layout of unmodified links.  
    // link: Object
    //		The link.
    // returns:
    //		The reshape mode of an individual link used during the
    //		incremental layout of unmodified links, that is, links that have
    //		already been laid out in the previous layout execution and for
    //		which neither the bounding box (see '
    //		ShortLinkLayout.getNodeBoxInterface()'
    //		nor the "link connection box" (see '
    //		ibm_ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()')
    //		of their origin and destination
    //		nodes have been modified.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, ShortLinkLayout._INCREMENTAL_UNMODIFIED_LINK_RESHAPE_MODE_PROPERTY, ShortLinkLayout._defaultIncrementalUnmodifiedLinkReshapeMode); // int
  },

  setGlobalIncrementalModifiedLinkReshapeMode : function(mode)
  {
    // summary:
    //		Sets the reshape mode used during the incremental layout of modified
    //		links, that is, links that have already been laid out in the
    //		previous layout execution and for which either the bounding box (see
    //		' ShortLinkLayout.getNodeBoxInterface()' or the
    //		"link connection box" (see '
    //		ibm_ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()')
    //		of their origin and destination nodes has been modified. 
    // description:
    //		Sets the reshape mode used during the incremental layout of modified
    //		links, that is, links that have already been laid out in the
    //		previous layout execution and for which either the bounding box (see
    //		<code> ShortLinkLayout.getNodeBoxInterface()</code> or the
    //		"link connection box" (see <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()</code>)
    //		of their origin and destination nodes has been modified. 
    //		<p>
    //		This mode has no effect if incremental layout is disabled.  
    //		</p>
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li><code> ShortLinkLayout.FIXED&#95;SHAPE&#95;TYPE&#95;MODE</code> -
    //		Incremental layout preserves the shape type of the link. This means
    //		that both the number of bends and the node sides to which the link
    //		is connected are preserved.</li>  
    //		<li><code> ShortLinkLayout.FIXED&#95;NODE&#95;SIDES&#95;MODE</code> -
    //		Incremental layout preserves the node sides to which the links are
    //		connected.</li>  
    //		<li><code> ShortLinkLayout.FIXED&#95;CONNECTION&#95;POINTS&#95;MODE</code> -
    //		Incremental layout preserves the connection points of the
    //		links.</li>  
    //		<li><code> ShortLinkLayout.FIXED&#95;MODE</code> - The links are not
    //		reshaped at all during incremental layout. Only newly added links
    //		are rerouted.</li>  
    //		<li><code> ShortLinkLayout.FREE&#95;MODE</code> - Incremental layout is
    //		allowed to freely reshape the links. This is equivalent to
    //		nonincremental behavior for all the links, hence it is recommended
    //		to disable incremental layout ({link <code>
    //		ShortLinkLayout.setIncrementalMode()</code>) instead of setting
    //		<code> ShortLinkLayout.FREE&#95;MODE</code> as the global incremental
    //		reshape mode. Any settings done using <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.isFixed()</code>, <code>
    //		ShortLinkLayout.getGlobalOriginPointMode()</code>, and <code>
    //		ShortLinkLayout.getGlobalDestinationPointMode()</code> are still
    //		obeyed.</li>  
    //		<li><code> ShortLinkLayout.MIXED&#95;MODE</code> - Each link can have a
    //		different mode. The mode of each individual link can be set by the
    //		method <code>
    //		ShortLinkLayout.setIncrementalUnmodifiedLinkReshapeMode()</code>.</li>
    //		 
    //		</ul>  
    //		</p>
    //		<p>
    //		Unless the global mode is <code> ShortLinkLayout.MIXED&#95;MODE</code>,
    //		all links are reshaped according to the same mode.  
    //		</p>
    //		<p>
    //		The default mode is <code> ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // mode: int
    //		The link reshape mode to set.
    
    if (
      !
        (mode == ShortLinkLayout.FREE_MODE || mode == ShortLinkLayout.FIXED_SHAPE_TYPE_MODE || mode == ShortLinkLayout.FIXED_NODE_SIDES_MODE || mode == ShortLinkLayout.FIXED_CONNECTION_POINTS_MODE ||
            mode == ShortLinkLayout.FIXED_MODE || mode == ShortLinkLayout.MIXED_MODE))
      {
        LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6647E",  mode);
      }

    if (mode != this._incrementalModifiedLinkReshapeMode)
    {
      this._incrementalModifiedLinkReshapeMode = mode;
      this.onParameterChanged("GlobalIncrementalModifiedLinkReshapeMode");
    }
  },

  getGlobalIncrementalModifiedLinkReshapeMode : function()
  {
    // summary:
    //		Returns the reshape mode used during the incremental layout of
    //		modified links.
    
    return (false ? ShortLinkLayout._defaultIncrementalModifiedLinkReshapeMode : this._incrementalModifiedLinkReshapeMode); // int
  },

  setIncrementalModifiedLinkReshapeMode : function(link, mode)
  {
    // summary:
    //		Sets the reshape mode of an individual link used during the
    //		incremental layout of modified links, that is, links that have
    //		already been laid out in the previous layout execution and for which
    //		either the bounding box (see '
    //		ShortLinkLayout.getNodeBoxInterface()' or the
    //		"link connection box" (see '
    //		ibm_ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()')
    //		of their origin and destination nodes has been modified. 
    // description:
    //		Sets the reshape mode of an individual link used during the
    //		incremental layout of modified links, that is, links that have
    //		already been laid out in the previous layout execution and for which
    //		either the bounding box (see <code>
    //		ShortLinkLayout.getNodeBoxInterface()</code> or the
    //		"link connection box" (see <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()</code>)
    //		of their origin and destination nodes has been modified. 
    //		<p>
    //		This link reshape mode is used only if incremental layout is enabled
    //		and the global link reshape mode is set to <code>
    //		ShortLinkLayout.MIXED&#95;MODE</code>. If the global mode is not <code>
    //		ShortLinkLayout.MIXED&#95;MODE</code>, all links have the mode that is
    //		specified as the global link-reshape mode.  
    //		</p>
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> ShortLinkLayout.FIXED&#95;SHAPE&#95;TYPE&#95;MODE</code> -
    //		Incremental layout preserves the shape type of the link. This means
    //		that both the number of bends and the node sides to which the link
    //		is connected are preserved.</li>  
    //		<li> <code> ShortLinkLayout.FIXED&#95;NODE&#95;SIDES&#95;MODE</code> -
    //		Incremental layout preserves the node sides to which the links are
    //		connected.</li>  
    //		<li> <code> ShortLinkLayout.FIXED&#95;CONNECTION&#95;POINTS&#95;MODE</code> -
    //		Incremental layout preserves the connection points of the
    //		links.</li>  
    //		<li> <code> ShortLinkLayout.FIXED&#95;MODE</code> - The links are not
    //		reshaped at all during incremental layout. Only newly added links
    //		are rerouted.</li>  
    //		<li> <code> ShortLinkLayout.FREE&#95;MODE</code> - Incremental layout is
    //		allowed to freely reshape the links. This is equivalent to
    //		nonincremental behavior for the <code>link</code> argument. Any
    //		settings done using <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.isFixed()</code>, <code>
    //		ShortLinkLayout.getGlobalOriginPointMode()</code>, and <code>
    //		ShortLinkLayout.getGlobalDestinationPointMode()</code> are still
    //		obeyed.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default mode is <code> ShortLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // link: Object
    //		The link.
    // mode: int
    //		The link reshape mode to set.
    
    if (
      !
        (mode == ShortLinkLayout.FREE_MODE || mode == ShortLinkLayout.FIXED_SHAPE_TYPE_MODE || mode == ShortLinkLayout.FIXED_NODE_SIDES_MODE || mode == ShortLinkLayout.FIXED_CONNECTION_POINTS_MODE ||
            mode == ShortLinkLayout.FIXED_MODE))
      {
        LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6648E",  mode);
      }

    LayoutParametersUtil.setLinkParameter(this, link, ShortLinkLayout._INCREMENTAL_MODIFIED_LINK_RESHAPE_MODE_PROPERTY, mode, ShortLinkLayout._defaultIncrementalModifiedLinkReshapeMode);
  },

  getIncrementalModifiedLinkReshapeMode : function(link)
  {
    // summary:
    //		Returns the reshape mode of an individual link used during the
    //		incremental layout of modified links.  
    // link: Object
    //		The link.
    // returns:
    //		The reshape mode of an individual link used during the
    //		incremental layout of modified links, that is, links that have
    //		already been laid out in the previous layout execution and for
    //		which either the bounding box (see '
    //		ShortLinkLayout.getNodeBoxInterface()'
    //		or the "link connection box" (see '
    //		ibm_ilog.graphlayout.GraphLayout.getLinkConnectionBoxInterface()')
    //		of their origin and destination
    //		nodes has been modified.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, ShortLinkLayout._INCREMENTAL_MODIFIED_LINK_RESHAPE_MODE_PROPERTY, ShortLinkLayout._defaultIncrementalModifiedLinkReshapeMode); // int
  },

  setInterGraphLinksMode : function(enable)
  {
    // summary:
    //		Sets the routing of intergraph links. 
    // description:
    //		Sets the routing of intergraph links. 
    //		If the argument is <code>true</code>, the intergraph links are
    //		routed; otherwise, only the normal links are routed. An intergraph
    //		link is a link stored in one graph that has an origin or destination
    //		not stored in the same graph. If the combined intergraph link
    //		routing is enabled in addition, this means that normal links and
    //		intergraph links are routed together; otherwise only the intergraph
    //		links are routed.  
    //		<p>
    //		The default value is <code>true</code>.  
    //		</p>
    //		<p>
    //		If the intergraph link mode is enabled and the graph does contain
    //		some intergraph links, there are some restrictions:  
    //		<ul>
    //		 
    //		<li> Layout animation is not possible.</li>   
    //		<li> Incremental layout has no effect.</li>   
    //		<li> If layout is performed on a nested <code>
    //		ibm&#95;ilog.diagram.Graph</code> (or <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel</code>), the top-level
    //		grapher must have a manager view attached because of the coordinate
    //		transformations of the subgraphers. If you create your own subclass
    //		of <code> ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel</code>
    //		instead of <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel</code> to route
    //		intergraph links, you must make sure that all graph models of all
    //		subgraphers use the same coordinate system through appropriate
    //		coordinate transformations.</li>   
    //		</ul>  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (enable != this._interGraphLinksMode)
    {
      this._interGraphLinksMode = enable;
      this.onParameterChanged("InterGraphLinksMode");
    }
  },

  isInterGraphLinksMode : function()
  {
    // summary:
    //		Returns 'true' if the routing of intergraph links is
    //		enabled; returns 'false' otherwise.
    
    return (false ? ShortLinkLayout._defaultInterGraphLinksMode : this._interGraphLinksMode); // Boolean
  },

  setCombinedInterGraphLinksMode : function(enable)
  {
    // summary:
    //		Sets the routing of normal links in addition to intergraph links. 
    // description:
    //		Sets the routing of normal links in addition to intergraph links. 
    //		If intergraph link routing is disabled, the mode has no effect. If
    //		intergraph link routing is enabled and the argument is
    //		<code>true</code>, both normal links and intergraph links are routed
    //		in the same pass. Otherwise, only the intergraph links are routed.  
    //		<p>
    //		The default value is <code>true</code>.  
    //		</p>
    
    if (enable != this._combinedInterGraphLinksMode)
    {
      this._combinedInterGraphLinksMode = enable;
      this.onParameterChanged("CombinedInterGraphLinksMode");
    }
  },

  isCombinedInterGraphLinksMode : function()
  {
    // summary:
    //		Returns 'true' if the routing of normal links in addition
    //		to intergraph links is enabled; returns 'false'
    //		otherwise.
    
    return (false ? ShortLinkLayout._defaultCombinedInterGraphLinksMode : this._combinedInterGraphLinksMode); // Boolean
  },

  setSameShapeForMultipleLinks : function(option)
  {
    // summary:
    //		Sets whether the layout algorithm must make sure that all multiple
    //		links have the same type of shape, that is, a shape with the same
    //		number of bends and with parallel segments. 
    // description:
    //		Sets whether the layout algorithm must make sure that all multiple
    //		links have the same type of shape, that is, a shape with the same
    //		number of bends and with parallel segments. 
    //		<p>
    //		Multiple links are links that have the same origin and destination
    //		nodes.  
    //		</p>
    //		<p>
    //		If the argument is <code>true</code>, additional work is done to
    //		make sure that multiple links are reshaped in the same manner.  
    //		</p>
    //		<p>
    //		The default value is <code>false</code>.  
    //		</p>
    //		<p>
    //		For some categories of links, it cannot be guaranteed that they will
    //		be given the same type of shape as the other links with the same
    //		origin and destination nodes:  
    //		<ul>
    //		<li>Newly inserted links in incremental mode (see <code>
    //		ShortLinkLayout.setIncrementalMode()</code>.</li>  
    //		<li>Fixed links (see <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setFixed()</code>.</li>  
    //		<li>Links with at least one of their connection points fixed (see
    //		<code> ShortLinkLayout.setGlobalOriginPointMode()</code>, <code>
    //		ShortLinkLayout.setGlobalDestinationPointMode()</code>, <code>
    //		ShortLinkLayout.setOriginPointMode()</code>, <code>
    //		ShortLinkLayout.setDestinationPointMode()</code>).</li>  
    //		</ul>  
    //		</p>
    // option: Boolean
    //		Set to 'true' to enable the same shape option,
    //		or 'false' to disable it.
    
    if (option != this._sameShapeForMultipleLinks)
    {
      this._sameShapeForMultipleLinks = option;
      this.onParameterChanged("SameShapeForMultipleLinks");
    }
  },

  isSameShapeForMultipleLinks : function()
  {
    // summary:
    //		Returns 'true' if the algorithm that makes sure that
    //		multiple links are reshaped in the same manner is enabled, and
    //		'false' otherwise.
    
    return (false ? ShortLinkLayout._defaultSameShapeForMultipleLinks : this._sameShapeForMultipleLinks); // Boolean
  },

  setLinkToNodeCrossingPenalty : function(penalty)
  {
    // summary:
    //		Sets the link-to-node crossing penalty factor.
    
    if (this._linkNodePenalty != penalty)
    {
      this._linkNodePenalty = penalty;
      this.onParameterChanged("LinkToNodeCrossingPenalty");
    }
  },

  getLinkToNodeCrossingPenalty : function()
  {
    // summary:
    //		Returns the link-to-node crossing penalty factor. 
    // description:
    //		Returns the link-to-node crossing penalty factor. 
    //		When choosing the optimal shape for the links, the algorithm
    //		computes a penalty function expressed in the following manner: <br>
    //		<code>Link&#95;to&#95;Link&#95;Penalty * #Link&#95;to&#95;Link&#95;Crossings +
    //		Link&#95;to&#95;Node&#95;Penalty * #Link&#95;to&#95;Node&#95;Crossings</code>  
    //		<p>
    //		The default value is <code>1</code>.  
    //		</p>
    
    return (false ? ShortLinkLayout._defaultLinkNodePenalty : this._linkNodePenalty); // Number
  },

  setLinkToLinkCrossingPenalty : function(penalty)
  {
    // summary:
    //		Sets the link-to-link crossing penalty factor.
    
    if (this._linkLinkPenalty != penalty)
    {
      this._linkLinkPenalty = penalty;
      this.onParameterChanged("LinkToLinkCrossingPenalty");
    }
  },

  getLinkToLinkCrossingPenalty : function()
  {
    // summary:
    //		Returns the link-to-link crossing penalty factor. 
    // description:
    //		Returns the link-to-link crossing penalty factor. 
    //		When choosing the optimal shape for the links, the algorithm
    //		computes a penalty function expressed in the following manner: <br>
    //		<code>Link&#95;to&#95;Link&#95;Penalty * #Link&#95;to&#95;Link&#95;Crossings +
    //		Link&#95;to&#95;Node&#95;Penalty * #Link&#95;to&#95;Node&#95;Crossings</code>  
    //		<p>
    //		The default value is <code>1</code>.  
    //		</p>
    
    return (false ? ShortLinkLayout._defaultLinkLinkPenalty : this._linkLinkPenalty); // Number
  },

  setMinFinalSegmentLength : function(length)
  {
    // summary:
    //		Sets the minimum length of the final (first and last) segment of the
    //		links. 
    // description:
    //		Sets the minimum length of the final (first and last) segment of the
    //		links. 
    //		<p>
    //		The default value is <code>10</code>.  
    //		</p>
    
    if (this._minFinalSegment != length)
    {
      this._minFinalSegment = length;
      this.onParameterChanged("MinFinalSegmentLength");
    }
  },

  getMinFinalSegmentLength : function()
  {
    // summary:
    //		Returns the minimum length of the final segment of the links.
    
    return (false ? ShortLinkLayout._defaultMinFinalSegment : this._minFinalSegment); // Number
  },

  setBypassDistance : function(dist)
  {
    // summary:
    //		Sets the minimum distance between the origin and destination node
    //		for which a link shape going directly from one node to another is
    //		allowed. 
    // description:
    //		Sets the minimum distance between the origin and destination node
    //		for which a link shape going directly from one node to another is
    //		allowed. 
    //		The algorithm tries to avoid link shapes that directly connect the
    //		sides of end nodes that are closer than the bypass value.  
    //		<p>
    //		If the specified value is strictly negative, the value of the
    //		parameter <code> ShortLinkLayout.getMinFinalSegmentLength()</code>
    //		is used as the bypass distance. This allows the automatic adjustment
    //		of the bypass distance according to the current value of <code>
    //		ShortLinkLayout.getMinFinalSegmentLength()</code>. This behavior is
    //		suitable in most cases. However, you can specify a non-negative
    //		value in order to override the default behavior.  
    //		</p>
    //		<p>
    //		The default value is a negative value.  
    //		</p>
    // dist: Number
    //		The minimum distance for a direct connection.
    
    if (this._bypassDistance != dist)
    {
      this._bypassDistance = dist;
      this.onParameterChanged("BypassDistance");
    }
  },

  getBypassDistance : function()
  {
    // summary:
    //		Returns the minimum distance between the origin and destination node
    //		for which a link shape going directly from one node to another is
    //		allowed.
    
    return (false ? ShortLinkLayout._defaultBypassDistance : this._bypassDistance); // Number
  },

  setLinkOffset : function(offset)
  {
    // summary:
    //		Sets the offset between links connected on the same side of the
    //		node. 
    // description:
    //		Sets the offset between links connected on the same side of the
    //		node. 
    //		<p>
    //		The default value is <code>2</code>.  
    //		</p>
    
    if (this._linkOffset != offset)
    {
      this._linkOffset = offset;
      this.onParameterChanged("LinkOffset");
    }
  },

  getLinkOffset : function()
  {
    // summary:
    //		Returns the offset between links connected on the same side of the
    //		node.
    
    return (false ? ShortLinkLayout._defaultLinkOffset : this._linkOffset); // Number
  },

  getAllowedNumberOfIterations : function()
  {
    // summary:
    //		Returns the maximum number of allowed optimization iterations.
    
    return (false ? ShortLinkLayout._defaultNumberOfIterations : this._numberOfIterations); // int
  },

  setAllowedNumberOfIterations : function(iterations)
  {
    // summary:
    //		Sets the maximum number of iterations of the layout algorithm. 
    // description:
    //		Sets the maximum number of iterations of the layout algorithm. 
    //		<p>
    //		The default value is <code>3</code>.  
    //		</p>
    
    if (iterations < 0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6649E",  iterations);
    }

    if (this._numberOfIterations != iterations)
    {
      this._numberOfIterations = iterations;
      this.onParameterChanged("AllowedNumberOfIterations");
    }
  },

  stopImmediately : function()
  {
    // summary:
    //		Overridden version of the base class method. 
    // description:
    //		Overridden version of the base class method.
    
    this._shortLinkAlgorithm.stopImmediately();
    return this.inherited("stopImmediately", arguments, []); // Boolean
  },

  supportsPreserveFixedLinks : function()
  {
    // summary:
    //		Indicates that this layout class allows the user to specify fixed
    //		links. 
    // description:
    //		Indicates that this layout class allows the user to specify fixed
    //		links. 
    //		Fixed links are not reshaped during the layout if the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setPreserveFixedLinks()</code> is
    //		called with a <code>true</code> argument.
    
    return true; // Boolean
  },

  supportsAllowedTime : function()
  {
    // summary:
    //		Indicates that this layout class can stop the layout computation in
    //		the proper manner when the user-defined allowed time is exceeded. 
    // description:
    //		Indicates that this layout class can stop the layout computation in
    //		the proper manner when the user-defined allowed time is exceeded. 
    //		<p>
    //		The result code in the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code> in
    //		this case.  
    //		</p>
    
    return true; // Boolean
  },

  supportsStopImmediately : function()
  {
    // summary:
    //		Indicates that this layout class can interrupt the current run of
    //		the layout immediately in a controlled way. 
    // description:
    //		Indicates that this layout class can interrupt the current run of
    //		the layout immediately in a controlled way. 
    //		The result code in the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code> in
    //		this case.
    
    return true; // Boolean
  },

  supportsLinkConnectionBox : function()
  {
    // summary:
    //		Indicates that this layout class can use a link connection box
    //		interface to calculate the end points of a link. 
    // description:
    //		Indicates that this layout class can use a link connection box
    //		interface to calculate the end points of a link. 
    //		The link connection box interface is an object that provides the
    //		rectangle to which the links are connected for each node, and the
    //		tangential shift offset at each side for the connection points.  
    //		<p>
    //		The link connection box is used instead of the bounding box of each
    //		node (see  ) during the routing of links.  
    //		</p>
    
    return true; // Boolean
  },

  supportsSplineRouting : function()
  {
    // summary:
    //		Tests if this class supports the generic optimization of spline
    //		control points. 
    // description:
    //		Tests if this class supports the generic optimization of spline
    //		control points.
    
    return true; // Boolean
  },

  isLayoutTimeElapsedPackage : function()
  {
    // summary:
    //		This method is protected on GraphLayout, we need to provide a
    //		package access for ShortLinkAlgorithm.
    // tags:
    //		private
    
    return this.isLayoutTimeElapsed(); // Boolean
  }

}
);

lang.mixin(ShortLinkLayout,
{
  ORTHOGONAL_STYLE : null,

  DIRECT_STYLE : null,

  MIXED_STYLE : null,

  NO_RESHAPE_STYLE : null,

  NO_BUNDLE : null,

  FIRST_LAST_SEGMENT : null,

  IMPROVED_FIRST_LAST_SEGMENT : null,

  TWO_BENDS_ORTHOGONAL_STYLE : null,

  THREE_BENDS_ORTHOGONAL_STYLE : null,

  FREE_MODE : null,

  FIXED_MODE : null,

  FIXED_SHAPE_TYPE_MODE : null,

  FIXED_NODE_SIDES_MODE : null,

  FIXED_CONNECTION_POINTS_MODE : null,

  MIXED_MODE : null,

  AUTOMATIC_PINS : null,

  FIXED_OFFSET_PINS : null,

  EVENLY_SPACED_PINS : null,

  _defaultGlobalLinkStyle : null,

  _defaultGlobalSelfLinkStyle : null,

  _defaultOriginPointMode : null,

  _defaultDestinationPointMode : null,

  _defaultGlobalConnectorStyle : null,

  _defaultIncrementalUnmodifiedLinkReshapeMode : null,

  _defaultIncrementalModifiedLinkReshapeMode : null,

  _defaultEvenlySpacedPinsMarginRatio : null,

  _defaultLinkBundlesMode : null,

  _defaultMinFinalSegment : null,

  _defaultBypassDistance : null,

  _defaultLinkOffset : null,

  _defaultLinkNodePenalty : null,

  _defaultLinkLinkPenalty : null,

  _defaultNumberOfIterations : null,

  _defaultSameShapeForMultipleLinks : null,

  _defaultIncrementalMode : null,

  _defaultInterGraphLinksMode : null,

  _defaultCombinedInterGraphLinksMode : null,

  _defaultLinkOverlapNodesForbidden : null,

  _LINK_STYLE_PROPERTY : null,

  _ORIGIN_POINT_MODE_PROPERTY : null,

  _DESTINATION_POINT_MODE_PROPERTY : null,

  _CONNECTOR_STYLE_PROPERTY : null,

  _INCREMENTAL_UNMODIFIED_LINK_RESHAPE_MODE_PROPERTY : null,

  _INCREMENTAL_MODIFIED_LINK_RESHAPE_MODE_PROPERTY : null,
  _fields_init : function(c)
  {
    c = c || ShortLinkLayout;
    if(GraphLayout._fields_init)
      GraphLayout._fields_init(c);
    
    c.ORTHOGONAL_STYLE = LinkStyles.ORTHOGONAL_STYLE;
    c.DIRECT_STYLE = LinkStyles.DIRECT_STYLE;
    c.MIXED_STYLE = LinkStyles.MIXED_STYLE;
    c.NO_RESHAPE_STYLE = LinkStyles.NO_RESHAPE_STYLE;
    c.NO_BUNDLE = _SLBundlesModes.NO_BUNDLE;
    c.FIRST_LAST_SEGMENT = _SLBundlesModes.FIRST_LAST_SEGMENT;
    c.IMPROVED_FIRST_LAST_SEGMENT = _SLBundlesModes.IMPROVED_FIRST_LAST_SEGMENT;
    c.TWO_BENDS_ORTHOGONAL_STYLE = LinkStyles.TWO_BENDS_ORTHOGONAL_STYLE;
    c.THREE_BENDS_ORTHOGONAL_STYLE = LinkStyles.THREE_BENDS_ORTHOGONAL_STYLE;
    c.FREE_MODE = ConnectionPointModes.FREE_MODE;
    c.FIXED_MODE = ConnectionPointModes.FIXED_MODE;
    c.FIXED_SHAPE_TYPE_MODE = ReshapeModes.FIXED_SHAPE_TYPE_MODE;
    c.FIXED_NODE_SIDES_MODE = ReshapeModes.FIXED_NODE_SIDES_MODE;
    c.FIXED_CONNECTION_POINTS_MODE = ReshapeModes.FIXED_CONNECTION_POINTS_MODE;
    c.MIXED_MODE = ConnectionPointModes.MIXED_MODE;
    c.AUTOMATIC_PINS = ConnectorStyles.AUTOMATIC_PINS;
    c.FIXED_OFFSET_PINS = ConnectorStyles.FIXED_OFFSET_PINS;
    c.EVENLY_SPACED_PINS = ConnectorStyles.EVENLY_SPACED_PINS;
    c._defaultGlobalLinkStyle = ShortLinkLayout.ORTHOGONAL_STYLE;
    c._defaultGlobalSelfLinkStyle = ShortLinkLayout.TWO_BENDS_ORTHOGONAL_STYLE;
    c._defaultOriginPointMode = ShortLinkLayout.FREE_MODE;
    c._defaultDestinationPointMode = ShortLinkLayout._defaultOriginPointMode;
    c._defaultGlobalConnectorStyle = ShortLinkLayout.AUTOMATIC_PINS;
    c._defaultIncrementalUnmodifiedLinkReshapeMode = ShortLinkLayout.FIXED_SHAPE_TYPE_MODE;
    c._defaultIncrementalModifiedLinkReshapeMode = ShortLinkLayout.FREE_MODE;
    c._defaultEvenlySpacedPinsMarginRatio = 0.5;
    c._defaultLinkBundlesMode = ShortLinkLayout.IMPROVED_FIRST_LAST_SEGMENT;
    c._defaultMinFinalSegment = 10;
    c._defaultBypassDistance = -1;
    c._defaultLinkOffset = 2;
    c._defaultLinkNodePenalty = 1;
    c._defaultLinkLinkPenalty = 1;
    c._defaultNumberOfIterations = 3;
    c._defaultSameShapeForMultipleLinks = false;
    c._defaultIncrementalMode = false;
    c._defaultInterGraphLinksMode = true;
    c._defaultCombinedInterGraphLinksMode = true;
    c._defaultLinkOverlapNodesForbidden = false;
    c._LINK_STYLE_PROPERTY = "LinkStyle";
    c._ORIGIN_POINT_MODE_PROPERTY = "OriginPointMode";
    c._DESTINATION_POINT_MODE_PROPERTY = "DestinationPointMode";
    c._CONNECTOR_STYLE_PROPERTY = "ConnectorStyle";
    c._INCREMENTAL_UNMODIFIED_LINK_RESHAPE_MODE_PROPERTY = "IncrementalUnmodifiedLinkReshapeMode";
    c._INCREMENTAL_MODIFIED_LINK_RESHAPE_MODE_PROPERTY = "IncrementalModifiedLinkReshapeMode";
  }
  
}
);

ShortLinkLayout._getGlobalProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'allowedTime', defaultValue: 32000 },
    { name: 'preserveFixedLinks', defaultValue: false },
    { name: 'splineRoutingEnabled', defaultValue: false },
    { name: 'minSplineCurveSize', defaultValue: 5 },
    { name: 'maxSplineCurveSize', defaultValue: 30 },
    { name: 'balanceSplineCurveThreshold', defaultValue: 10 },
    { name: 'linkOverlapNodesForbidden', defaultValue: false },
    { name: 'globalLinkStyle', defaultValue: ShortLinkLayout.ORTHOGONAL_STYLE },
    { name: 'globalSelfLinkStyle', defaultValue: ShortLinkLayout.TWO_BENDS_ORTHOGONAL_STYLE },
    { name: 'globalOriginPointMode', defaultValue: ShortLinkLayout.FREE_MODE },
    { name: 'globalDestinationPointMode', defaultValue: ShortLinkLayout.FREE_MODE },
    { name: 'globalConnectorStyle', defaultValue: ShortLinkLayout.AUTOMATIC_PINS },
    { name: 'evenlySpacedPinsMarginRatio', defaultValue: 0.5 },
    { name: 'linkBundlesMode', defaultValue: ShortLinkLayout.IMPROVED_FIRST_LAST_SEGMENT },
    { name: 'incrementalMode', defaultValue: false },
    { name: 'globalIncrementalUnmodifiedLinkReshapeMode', defaultValue: ShortLinkLayout.FIXED_SHAPE_TYPE_MODE },
    { name: 'globalIncrementalModifiedLinkReshapeMode', defaultValue: ShortLinkLayout.FREE_MODE },
    { name: 'interGraphLinksMode', defaultValue: true },
    { name: 'combinedInterGraphLinksMode', defaultValue: true },
    { name: 'sameShapeForMultipleLinks', defaultValue: false },
    { name: 'linkToNodeCrossingPenalty', defaultValue: 1 },
    { name: 'linkToLinkCrossingPenalty', defaultValue: 1 },
    { name: 'minFinalSegmentLength', defaultValue: 10 },
    { name: 'bypassDistance', defaultValue: -1 },
    { name: 'linkOffset', defaultValue: 2 },
    { name: 'allowedNumberOfIterations', defaultValue: 3 }
  ];
  var baseProperties = GraphLayout._getGlobalProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

ShortLinkLayout._getNodeProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'connectorStyle', defaultValue: ShortLinkLayout.AUTOMATIC_PINS }
  ];
  var baseProperties = GraphLayout._getNodeProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

ShortLinkLayout._getLinkProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'fixed', defaultValue: false },
    { name: 'linkStyle', defaultValue: ShortLinkLayout.ORTHOGONAL_STYLE },
    { name: 'originPointMode', defaultValue: ShortLinkLayout.FREE_MODE },
    { name: 'destinationPointMode', defaultValue: ShortLinkLayout.FREE_MODE },
    { name: 'incrementalUnmodifiedLinkReshapeMode', defaultValue: ShortLinkLayout.FIXED_SHAPE_TYPE_MODE },
    { name: 'incrementalModifiedLinkReshapeMode', defaultValue: ShortLinkLayout.FREE_MODE }
  ];
  var baseProperties = GraphLayout._getLinkProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

(function(){ ShortLinkLayout._fields_init(); })();

return ShortLinkLayout;
});

},
'ibm_ilog/graphlayout/shortlink/_LinkShape':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_LinkShape", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/Point",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/TranslateUtil",	// #9
  "../shortlink/_OrthogonalLinkShapeProducer"	// #10
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    Point,	// #7
    Rectangle,	// #8
    TranslateUtil,	// #9
    _OrthogonalLinkShapeProducer	// #10
    ) {

var _LinkShape = declare("ibm_ilog.graphlayout.shortlink._LinkShape", null, 
{
  // summary:
  //		A class that is used to represent the shape of the link. 
  // description:
  //		A class that is used to represent the shape of the link. 
  //		This class is used by the link layout algorithms.   
  // tags:
  //		private
  
  _bendPoints: null,

  _from: null,

  _to: null,

  _nPoints : null,

  _bbox : null,
  _invalidBBox : null,

  constructor : function (a0, a1, a2, a3)
  {
    // summary:
    //		Creates a new LinkShape.  
    // maxNumberOfBends:
    //		the maximum number of bend points (excluding
    //		origin and destination points). This is 2 for direct style, 4 for
    //		orthogonal style, and anything for fixed links.
    
    if(arguments.length == 1)
      this._constructor_overload__LinkShape_1(a0);
    else if(arguments.length == 4)
      this._constructor_overload__LinkShape_4(a0, a1, a2, a3);
    else if(arguments.length == 3)
      this._constructor_overload__LinkShape_3(a0, a1, a2);
  },
  
  _constructor_overload__LinkShape_1 : function(maxNumberOfBends)
  {

    this._nPoints = 2;
    this._bbox = new Rectangle(0, 0, 0, 0);
    this._invalidBBox = true;
    this.initFromNumberOfBends(maxNumberOfBends);
  },
  
  initFromNumberOfBends : function(maxNumberOfBends)
  {
    if (maxNumberOfBends < 0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6617E",  maxNumberOfBends);
    }
    else if (maxNumberOfBends > 0)
    {
      this._bendPoints = this.allocatePoints(maxNumberOfBends);

      for (var i = 0; i < maxNumberOfBends; i++)
        this._bendPoints[i] = new Point();
    }
  },

  _constructor_overload__LinkShape_4 : function(layout, linkData, fromPoint, toPoint)
  {
    this._nPoints = 2;
    this._bbox = new Rectangle(0, 0, 0, 0);
    this._invalidBBox = true;
    this.initFromLinkData(layout, linkData, fromPoint, toPoint);
  },
  
  initFromLinkData : function(layout, linkData, fromPoint, toPoint)
  {
    var msg;

    if (!linkData.isFixed())
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6618E");
    }

    if (linkData.isMaster())
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6619E");
    }

    var link = linkData.nodeOrLink;
    var points = layout.getGraphModel().getLinkPoints(link);
    var nBends = points.length - 2;

    if (nBends < 0)
    {
      var params = [layout.getGraphModel(), link, (nBends + 2) ];
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6620E", params);
    }
    else if (nBends > 0)
    {
      this._bendPoints = this.allocatePoints(nBends);

      for (var i = 1; i <= nBends; i++)
        this._bendPoints[i - 1] = new Point(points[i] .x, points[i] .y);
      this._nPoints = nBends + 2;
    }

    this.setFrom(fromPoint);
    this.setTo(toPoint);
    this._invalidBBox = true;
    var fromNodeSide = linkData.getSideOfPoint(fromPoint, true);

    var toNodeSide = linkData.getSideOfPoint(toPoint, false);

    linkData.setNodeSides(fromNodeSide, toNodeSide, true);
  },

  _constructor_overload__LinkShape_3 : function(masterShape, reversed, copyPointLocations)
  {
    this._nPoints = 2;
    this._bbox = new Rectangle(0, 0, 0, 0);
    this._invalidBBox = true;
    this.copyFrom(masterShape, reversed, copyPointLocations);
  },
  
  copyFrom : function(masterShape, reversed, copyPointLocations)
  {
    // summary:
    //		Copies the points from another shape.  
    // copyPointLocations: Boolean
    //		If true, the locations of all the points are
    //		copied. Otherwise, we only ensure that the necessary number of
    //		points are instanciated.
    // tags:
    //		private
    
    this._nPoints = masterShape.getNumberOfPoints();
    var masterBendPoints = masterShape.getIntermediatePoints();

    var point;
    var p;
    var i;

    if (reversed)
    {
      if (masterBendPoints != null && this._nPoints > 2)
      {
        if (this._bendPoints == null || this._bendPoints.length < (this._nPoints - 2)) {
          this._bendPoints = this.allocatePoints(OrthogonalLinkShapeProducer.MAX_NUMBER_OF_BENDS);
        }
        if (copyPointLocations)
        {
          var k = 0;

          for (i = this._nPoints - 3; i >= 0; i--)
            this._bendPoints[k++] = new Point(masterBendPoints[i] .x, masterBendPoints[i] .y);
        }
      }

      point = masterShape.getTo();

      if (point != null)
      {
        p = this.getFrom();

        if (p != null)
        {
          if (copyPointLocations) {
            TranslateUtil.setPointLocation(p, point);
          }
        }
        else {
          this.setFrom(new Point(point .x, point .y));
        }
      }

      point = masterShape.getFrom();

      if (point != null)
      {
        p = this.getTo();

        if (p != null)
        {
          if (copyPointLocations) {
            TranslateUtil.setPointLocation(p, point);
          }
        }
        else {
          this.setTo(new Point(point .x, point .y));
        }
      }
    }
    else
    {
      if (masterBendPoints != null && this._nPoints > 2)
      {
        if (this._bendPoints == null || this._bendPoints.length < (this._nPoints - 2)) {
          this._bendPoints = this.allocatePoints(OrthogonalLinkShapeProducer.MAX_NUMBER_OF_BENDS);
        }
        if (copyPointLocations)
        {
          for (i = this._nPoints - 3; i >= 0; i--)
            this._bendPoints[i] = new Point(masterBendPoints[i] .x, masterBendPoints[i] .y);
        }
      }

      point = masterShape.getFrom();

      if (point != null)
      {
        p = this.getFrom();

        if (p != null)
        {
          if (copyPointLocations) {
            TranslateUtil.setPointLocation(p, point);
          }
        }
        else {
          this.setFrom(new Point(point .x, point .y));
        }
      }

      point = masterShape.getTo();

      if (point != null)
      {
        p = this.getTo();

        if (p != null)
        {
          if (copyPointLocations) {
            TranslateUtil.setPointLocation(p, point);
          }
        }
        else {
          this.setTo(new Point(point .x, point .y));
        }
      }
    }

    this.invalidateBBox();
  },

  allocatePoints : function(maxNumberOfPoints)
  {
    // summary:
    //		Returns a newly created arraw of points.
    // tags:
    //		private
    
    return TranslateUtil.newArray(Math.max(maxNumberOfPoints, _OrthogonalLinkShapeProducer.MAX_NUMBER_OF_BENDS), null); // Array
  },

  setNumberOfPoints : function(nPoints)
  {
    // summary:
    //		Sets the number of points including the connection points. 
    // description:
    //		Sets the number of points including the connection points. 
    //		LinkShape has an internal array of points. Typically, the points are
    //		modified using getIntermediatePoints, then changing the location of
    //		the existing instances of Point. This method allows to specify the
    //		number of points which are really used (the remaining points have no
    //		meaning). We do it this way to avoid the need of reallocating the
    //		array of points any time the number of bends changes.
    
    this._nPoints = nPoints;
  },

  getNumberOfPoints : function()
  {
    // summary:
    //		Returns the current number of points of the link (equal to the
    //		number of bends + 2). 
    // description:
    //		Returns the current number of points of the link (equal to the
    //		number of bends + 2). 
    //		Can be smaller than maxNumberOfPoints, but not smaller than 2. In
    //		practice, with our current implementation, is must be never smaller
    //		than 3 with orthogonal style, and 4 with direct style.
    // tags:
    //		private
    
    return this._nPoints; // int
  },

  getIntermediatePoints : function()
  {
    // summary:
    //		Returns the intermediate points. 
    // description:
    //		Returns the intermediate points. 
    //		Note that the length of the array can be bigger than the number of
    //		points. You should use getNumberOfPoints() in order to be aware of
    //		the actual number of points. (This is for efficiency reasons.)
    // tags:
    //		private
    
    return this._bendPoints; // Array
  },

  getPointAt : function(index)
  {
    // summary:
    //		Returns a point at a given index. 
    // description:
    //		Returns a point at a given index. 
    //		First point has index 0.
    //		It's the connection point on the origin node. Last point has index getNumberOfPoints() - 1. It's
    //		the connection point on the destination node.
    // tags:
    //		private
    
    if (index == 0) {
      return this._from; // Point
    }
    else if (index == (this._nPoints - 1)) {
      return this._to; // Point
    }
    else if (index > 0 && index < this._nPoints) {
      return this._bendPoints[index - 1]; // Point
    }
    else
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6621E");
      return null; // Point
    }
  },

  getPointAtFromEndPoint : function(origin, index)
  {
    // summary:
    //		Returns the point at a given index, starting either from the origin,
    //		or from the destination. 
    // description:
    //		Returns the point at a given index, starting either from the origin,
    //		or from the destination. 
    //		Index 0 = connection point on origin/dest node Index 1 = next point
    //		... Index n-1 = connection point on dest/orig node  
    // tags:
    //		private
    
    return (origin ? this.getPointAt(index) : this.getPointAt(this.getNumberOfPoints() - index - 1)); // Point
  },

  setFrom : function(point)
  {
    // summary:
    //		Sets the from point. 
    // description:
    //		Sets the from point. 
    //		WARNING: - this class just puts a reference on the point passed as
    //		argument. - you must call invalidateBBox
    // tags:
    //		private
    
    this._from = point;
    this._invalidBBox = true;
  },

  getFrom : function()
  {
    // summary:
    //		Returns the origin point.
    // tags:
    //		private
    
    return this._from; // Point
  },

  setTo : function(point)
  {
    // summary:
    //		Sets the to point. 
    // description:
    //		Sets the to point. 
    //		WARNING: - this class just puts a reference on the point passed as
    //		argument. - you must call invalidateBBox
    // tags:
    //		private
    
    this._to = point;
    this._invalidBBox = true;
  },

  getTo : function()
  {
    // summary:
    //		Returns the destination point.
    // tags:
    //		private
    
    return this._to; // Point
  },

  boundingBox : function()
  {
    // summary:
    //		Returns the bounding rectangle of the shape. 
    // description:
    //		Returns the bounding rectangle of the shape. 
    //		You should not modifiy this rectangle.
    // tags:
    //		private
    
    if (this._invalidBBox) {
      this.computeBBox(this._bbox);
    }
    return this._bbox; // Rectangle
  },

  invalidateBBox : function()
  {
    // summary:
    //		Invalidates the bbox cache.
    // tags:
    //		private
    
    this._invalidBBox = true;
  },

  computeBBox : function(bbox)
  {
    // summary:
    //		Re-computes the bounding rectangle of the link. 
    // description:
    //		Re-computes the bounding rectangle of the link. 
    //		This method is called when the shape of the link changes.  
    // bbox: Rectangle
    //		the rectangle to store the resulting bounding box.
    // tags:
    //		private
    
    bbox.x = this._from.x;
    bbox.y = this._from.y;
    bbox.width = 0;
    bbox.height = 0;
    TranslateUtil.addPoint(bbox, this._to.x, this._to.y);

    if (this._bendPoints != null)
    {
      var limit = this._nPoints - 2;
      var point;

      for (var i = 0; i < limit; i++)
      {
        point = this._bendPoints[i];
        if (point != null) {
          TranslateUtil.addPoint(bbox, point.x, point.y);
        }
      }
    }

    this._invalidBBox = false;
  }

}
);

return _LinkShape;
});

},
'ibm_ilog/graphlayout/longlink/LLGraph':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLGraph", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LinkStyles",	// #6
  "../internalutil/Point",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../longlink/LLCrossRedAlgorithm",	// #9
  "../longlink/LLDirectRouteAlgorithm",	// #10
  "../longlink/LLFallbackRouteAlgorithm",	// #11
  "../longlink/LLGrid",	// #12
  "../longlink/LLLink",	// #13
  "../longlink/LLLinkRouteAlgorithm",	// #14
  "../longlink/LLTerminationCandAlgorithm"	// #15
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LinkStyles,	// #6
    Point,	// #7
    TranslateUtil,	// #8
    LLCrossRedAlgorithm,	// #9
    LLDirectRouteAlgorithm,	// #10
    LLFallbackRouteAlgorithm,	// #11
    LLGrid,	// #12
    LLLink,	// #13
    LLLinkRouteAlgorithm,	// #14
    LLTerminationCandAlgorithm	// #15
    ) {

var LLGraph = declare("ibm_ilog.graphlayout.longlink.LLGraph", null, 
{
  // summary:
  //		A single link graph contains all information we need for fast access
  //		to the tree data structures.  
  // tags:
  //		private
  
  _graphLayout: null,

  _graphModel: null,

  _grid: null,

  _links: null,
  _firstFreeLinkSlot: null,

  _minX: null,
  _maxX: null,
  _minY: null,
  _maxY: null,

  _minimalBorder: null,

  _animation : null,

  _interGraphLinkRouting : null,

  _redraw : null,

  DIRECT_LINKS_AT_NODE_PROPERTY: null,

  constructor : function(graphLayout, redraw)
  {
    this._animation = false;
    this._interGraphLinkRouting = false;
    this._redraw = true;
    this._graphLayout = graphLayout;
    this._graphModel = null;
    this._grid = null;
    this.DIRECT_LINKS_AT_NODE_PROPERTY = "__ilvLongLinkLayoutLinkStyle" + graphLayout.getInstanceId();

    this._redraw = redraw;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._graphModel = null;
    if (this._grid != null) {
      this._grid.dispose();
    }
    this._grid = null;
  },

  attach : function(graphModel, layoutAlgorithm)
  {
    // summary:
    //		Attach a graph model and create a representation such that each node
    //		in the model corresponds to a tree node in this graph, each link in
    //		the model corresponds to a tree link in this graph, each link has
    //		exactly one segment, and there are no dummy nodes at all.
    // tags:
    //		private
    
    this._grid = null;

    this._graphModel = graphModel;
    this.layoutStepPerformed();
    var layout = this.getLayout();
    var h;
    var gridDistX = layout.getHorizontalGridOffset();
    var gridDistY = layout.getVerticalGridOffset();
    this._minimalBorder = 2;
    h = Math.floor(Math.ceil(layout.getMinStartSegmentLength() / gridDistX));
    if (h > this._minimalBorder) {
      this._minimalBorder = h;
    }
    h = Math.floor(Math.ceil(layout.getMinStartSegmentLength() / gridDistY));
    if (h > this._minimalBorder) {
      this._minimalBorder = h;
    }
    h = Math.floor(Math.ceil(layout.getMinEndSegmentLength() / gridDistX));
    if (h > this._minimalBorder) {
      this._minimalBorder = h;
    }
    h = Math.floor(Math.ceil(layout.getMinEndSegmentLength() / gridDistY));
    if (h > this._minimalBorder) {
      this._minimalBorder = h;
    }
  },

  detach : function()
  {
    // summary:
    //		Detach the graph model. 
    // description:
    //		Detach the graph model. 
    //		This also clears the attach properties of the nodes.
    // tags:
    //		private
    
    if (this._firstFreeLinkSlot > 0)
    {
      for (var i = 0; i < this._firstFreeLinkSlot; i++)
      {
        this._links[i].dispose();
        this._links[i] = null;
      }
    }

    this._links = null;
    this._firstFreeLinkSlot = -1;
  },

  createCompleteGrid : function()
  {
    // summary:
    //		Create a complete grid for the entire graph.
    // tags:
    //		private
    
    if (this._links == null || this._links.length == 0) {
      return;
    }
    this.setGrid(new LLGrid());
    this.initGrid(null);
  },

  initGrid : function(linkObject)
  {
    // summary:
    //		Initialize the grid for the entire graph, where the input link
    //		object, if any, is a potential intergraph link during intergraph
    //		link routing, and its end nodes are not considered rectangular
    //		obstacles.
    // tags:
    //		private
    
    var model = this.getModel();
    var layout = this.getLayout();
    var e = new TranslateUtil.ArrayIterator(model.getNodes());
    var obj;
    var rect;
    var i;

    this.calcDrawingArea();

    var gridDistX = layout.getHorizontalGridOffset();
    var gridDistY = layout.getVerticalGridOffset();
    var gridBaseX = layout.getHorizontalGridBase();
    var gridBaseY = layout.getVerticalGridBase();
    var minDistX = layout.getHorizontalMinOffset();
    var minDistY = layout.getVerticalMinOffset();
    var baseCoordX;
    var baseCoordY;
    var numberOfXGridLines;
    var numberOfYGridLines;
    var lowerIndex;
    var upperIndex;
    lowerIndex = Math.floor(Math.floor((this._minX - gridBaseX) / gridDistX));
    if (gridBaseX + lowerIndex * gridDistX >= this._minX) {
      lowerIndex--;
    }
    lowerIndex--;
    baseCoordX = gridBaseX + lowerIndex * gridDistX;
    upperIndex = Math.floor(Math.ceil((this._maxX - gridBaseX) / gridDistX));
    if (gridBaseX + upperIndex * gridDistX <= this._maxX) {
      upperIndex++;
    }
    upperIndex++;
    numberOfXGridLines = upperIndex - lowerIndex + 1;
    lowerIndex = Math.floor(Math.floor((this._minY - gridBaseY) / gridDistY));
    if (gridBaseY + lowerIndex * gridDistY >= this._minY) {
      lowerIndex--;
    }
    lowerIndex--;
    baseCoordY = gridBaseY + lowerIndex * gridDistY;
    upperIndex = Math.floor(Math.ceil((this._maxY - gridBaseY) / gridDistY));
    if (gridBaseY + upperIndex * gridDistY <= this._maxY) {
      upperIndex++;
    }
    upperIndex++;
    numberOfYGridLines = upperIndex - lowerIndex + 1;

    if (numberOfXGridLines < 1) {
      numberOfXGridLines = 1;
    }
    if (numberOfYGridLines < 1) {
      numberOfYGridLines = 1;
    }

    var grid = this.getGrid();
    grid.init(baseCoordX, baseCoordY, gridDistX, gridDistY, minDistX, minDistY, numberOfXGridLines, numberOfYGridLines);

    grid.setImmediateReorganizationNeeded(false);
    var interGraphModel = layout.getInterGraphLinksRoutingModel();
    var p1 = new Point();
    var p2 = new Point();

    if (layout.isNodeObstacleEnabled())
    {
      e = new TranslateUtil.ArrayIterator(model.getNodes());

      while (e.hasNext())
      {
        obj = e.next();
        rect = layout.getNodeBoxForObstacle(obj);

        if (linkObject == null) {
          grid.setRectObstacle(rect);
        }
        else
        {
          if (interGraphModel.fromEndsInside(linkObject, obj) || interGraphModel.toEndsInside(linkObject, obj))
          {
            p1.x = rect.x;
            p1.y = rect.y;
            p2.x = rect.x + rect.width;
            p2.y = rect.y;
            grid.setObstacle(p1, p2);
            p1.x = p2.x;
            p1.y = rect.y + rect.height;
            grid.setObstacle(p1, p2);
            p2.x = rect.x;
            p2.y = p1.y;
            grid.setObstacle(p1, p2);
            p1.x = rect.x;
            p1.y = rect.y;
            grid.setObstacle(p1, p2);
          }
          else {
            grid.setRectObstacle(rect);
          }
        }

        this.layoutStepPerformed();
      }
    }

    e =  new TranslateUtil.ArrayIterator(layout.getRectObstacles());

    while (e.hasNext())
    {
      rect = e.next();
      grid.setRectObstacle(rect);
      this.layoutStepPerformed();
    }

    if (layout.isLinkObstacleEnabled())
    {
      e = new TranslateUtil.ArrayIterator(model.getLinks());

      while (e.hasNext())
      {
        obj = e.next();
        if (!this.isRoutable(obj)) {
          this.updateGridWithLink(obj);
        }
        this.layoutStepPerformed();
      }
    }

    e =  new TranslateUtil.ArrayIterator(layout.getLineObstacles());

    while (e.hasNext())
    {
      rect = e.next();
      p1.x = rect.x;
      p1.y = rect.y;
      p2.x = rect.x + rect.width;
      p2.y = rect.y + rect.height;
      grid.setObstacle(p1, p2);
      this.layoutStepPerformed();
    }

    var e1 =  new TranslateUtil.ArrayIterator(layout.getFromLineObstacles());
    var e2 =  new TranslateUtil.ArrayIterator(layout.getToLineObstacles());

    while (e1.hasNext() && e2.hasNext())
    {
      p1 = e1.next();
      p2 = e2.next();
      grid.setObstacle(p1, p2);
      this.layoutStepPerformed();
    }

    var numLinks = this.getNumberOfLinks();
    var link;

    for (i = 0; i < numLinks; i++)
    {
      link = this.getLink(i);

      if (link.isFrozen())
      {
        this.updateGridWithLink(link.getLinkObject());
      }
      else if (!link.needsRoute())
      {
        if (link.isDirect()) {
          this.updateGridWithLink(link.getLinkObject());
        }
        else
        {
          link.updateGrid();
        }
      }

      this.layoutStepPerformed();
    }

    grid.reorganizeAllBeforeSearch();
    grid.setImmediateReorganizationNeeded(true);
  },

  updateGridWithLink : function(linkObj)
  {
    // summary:
    //		Updates the grid with one link that must not be routed.
    // tags:
    //		private
    
    var model = this.getModel();
    var points = model.getLinkPoints(linkObj);
    var grid = this.getGrid();

    if (points != null && points.length > 0)
    {
      if (points.length == 1) {
        grid.setPointObstacle(points[0]);
      }
      else {
        for (var i = 1; i < points.length; i++)
          grid.setObstacle(points[i - 1], points[i]);
      }
    }
  },

  calcDrawingArea : function()
  {
    // summary:
    //		Calculate the drawing area that is covered by obstacles, i.e. 
    // description:
    //		Calculate the drawing area that is covered by obstacles, i.e. 
    //		calculate &#95;minX, &#95;&#95;maxX, minY, &#95;maxY.
    // tags:
    //		private
    
    var model = this.getModel();
    var layout = this.getLayout();
    var e = new TranslateUtil.ArrayIterator(model.getNodes());
    var obj;
    var rect;
    this._minX = /*Number.MAX_VALUE*/(3.4028235E38);
    this._minY = /*Number.MAX_VALUE*/(3.4028235E38);
    this._maxX = -/*Number.MAX_VALUE*/(3.4028235E38);
    this._maxY = -/*Number.MAX_VALUE*/(3.4028235E38);

    while (e.hasNext())
    {
      obj = e.next();
      rect = layout.getNodeBoxForObstacle(obj);
      this.updateDrawingAreaWithRect(rect);
      rect = layout.getNodeBoxForConnection(obj);
      this.updateDrawingAreaWithRect(rect);
      this.layoutStepPerformed();
    }

    e =  new TranslateUtil.ArrayIterator(layout.getRectObstacles());

    while (e.hasNext())
    {
      rect = e.next();
      this.updateDrawingAreaWithRect(rect);
      this.layoutStepPerformed();
    }

    e =  new TranslateUtil.ArrayIterator(layout.getLineObstacles());

    while (e.hasNext())
    {
      rect = e.next();
      this.updateDrawingAreaWithRect(rect);
      this.layoutStepPerformed();
    }

    var e1 =  new TranslateUtil.ArrayIterator(layout.getFromLineObstacles());
    var e2 =  new TranslateUtil.ArrayIterator(layout.getToLineObstacles());

    while (e1.hasNext() && e2.hasNext())
    {
      this.updateDrawingAreaWithPoint(e1.next());
      this.updateDrawingAreaWithPoint(e2.next());
      this.layoutStepPerformed();
    }

    e = new TranslateUtil.ArrayIterator(model.getLinks());
    var isFromPointFixed;
    var isToPointFixed;
    var points;
    var len;

    while (e.hasNext())
    {
      obj = e.next();

      if (!this.isRoutable(obj)) {
        this.updateDrawingAreaWithLinkObject(obj);
      }
      else
      {
        isFromPointFixed = layout.isFromPointFixed(obj);
        isToPointFixed = layout.isToPointFixed(obj);

        if (isFromPointFixed || isToPointFixed)
        {
          points = model.getLinkPoints(obj);
          len = (points != null ? points.length : 0);

          if (len >= 2)
          {
            if (isFromPointFixed) {
              this.updateDrawingAreaWithPoint(points[0]);
            }
            if (isToPointFixed) {
              this.updateDrawingAreaWithPoint(points[len - 1]);
            }
          }
        }
      }

      this.layoutStepPerformed();
    }

    var numLinks = this.getNumberOfLinks();
    var link;
    var i;

    for (i = 0; i < numLinks; i++)
    {
      link = this.getLink(i);

      if (link.isFrozen()) {
        this.updateDrawingAreaWithLinkObject(link.getLinkObject());
      }
      else if (!link.needsRoute())
      {
        if (link.isDirect()) {
          this.updateDrawingAreaWithLinkObject(link.getLinkObject());
        }
        else {
          this.updateDrawingAreaWithLink(link);
        }
      }

      this.layoutStepPerformed();
    }

    var offset = Math.max(layout.getMinStartSegmentLength(), layout.getMinEndSegmentLength());
    this._minX -= offset;
    this._maxX += offset;
    this._minY -= offset;
    this._maxY += offset;
  },

  updateDrawingAreaWithLinkObject : function(linkObj)
  {
    // summary:
    //		Updates the drawing are with one link that must not be routed.
    // tags:
    //		private
    
    var model = this.getModel();
    var points = model.getLinkPoints(linkObj);
    var len = points.length;

    for (var i = 0; i < len; i++)
      this.updateDrawingAreaWithPoint(points[i]);
  },

  updateDrawingAreaWithLink : function(link)
  {
    // summary:
    //		Updates the drawing are with one link that can be routed. 
    // description:
    //		Updates the drawing are with one link that can be routed. 
    //		This happens only when we reupdate the grid, so we have an old grid
    //		already.
    // tags:
    //		private
    
    var grid = this.getGrid();

    if (grid != null)
    {
      var point = TranslateUtil.newArray(2, 0);
      var p = new Point();
      var i = 0;

      while (link.getNextBend(point, i++))
      {
        p.x = grid.getGridCoord(0, point[0]);
        p.y = grid.getGridCoord(1, point[1]);
        this.updateDrawingAreaWithPoint(p);
      }
    }
  },

  updateDrawingAreaWithRect : function(rect)
  {
    // summary:
    //		Enlarge the drawing area such that it includes rect.  
    // tags:
    //		private
    
    var coord = rect.x;
    if (coord < this._minX) {
      this._minX = coord;
    }
    if (coord > this._maxX) {
      this._maxX = coord;
    }
    coord = rect.y;
    if (coord < this._minY) {
      this._minY = coord;
    }
    if (coord > this._maxY) {
      this._maxY = coord;
    }
    coord = rect.x + rect.width;
    if (coord < this._minX) {
      this._minX = coord;
    }
    if (coord > this._maxX) {
      this._maxX = coord;
    }
    coord = rect.y + rect.height;
    if (coord < this._minY) {
      this._minY = coord;
    }
    if (coord > this._maxY) {
      this._maxY = coord;
    }
  },

  updateDrawingAreaWithPoint : function(point)
  {
    // summary:
    //		Enlarge the drawing area such that it includes point.  
    // tags:
    //		private
    
    var coord = point.x;
    if (coord < this._minX) {
      this._minX = coord;
    }
    if (coord > this._maxX) {
      this._maxX = coord;
    }
    coord = point.y;
    if (coord < this._minY) {
      this._minY = coord;
    }
    if (coord > this._maxY) {
      this._maxY = coord;
    }
  },

  attachAllLinks : function()
  {
    // summary:
    //		Attach and create internal data structure for all links to be
    //		routed.
    // tags:
    //		private
    
    var numLinks = 0;
    var model = this.getModel();
    var layout = this.getLayout();
    var linkObject;
    var e = new TranslateUtil.ArrayIterator(model.getLinks());

    while (e.hasNext())
    {
      linkObject = e.next();
      if (!this.isRoutable(linkObject)) {
        continue;
      }
      numLinks++;
    }

    this._links = TranslateUtil.newArray(numLinks, null);
    this._firstFreeLinkSlot = 0;
    e = new TranslateUtil.ArrayIterator(model.getLinks());

    while (e.hasNext())
    {
      linkObject = e.next();
      layout.addStepCount(1);
      if (!this.isRoutable(linkObject)) {
        continue;
      }
      this._links[this._firstFreeLinkSlot++] = new LLLink(this, linkObject);
      this.layoutStepPerformed();
    }
  },

  attachLinks : function(v)
  {
    // summary:
    //		Attach and create internal data structure for a group of links to be
    //		routed, that is given as enumeration.
    // tags:
    //		private
    
    var layout = this.getLayout();
    var numLinks = 0;
    var e =  new TranslateUtil.ArrayIterator(v);
    var linkObject;

    while (e.hasNext())
    {
      linkObject = e.next();
      if (!this.isRoutable(linkObject)) {
        continue;
      }
      numLinks++;
    }

    this._links = TranslateUtil.newArray(numLinks, null);
    this._firstFreeLinkSlot = 0;
    e =  new TranslateUtil.ArrayIterator(v);

    while (e.hasNext())
    {
      linkObject = e.next();
      layout.addStepCount(1);
      if (!this.isRoutable(linkObject)) {
        continue;
      }
      this._links[this._firstFreeLinkSlot++] = new LLLink(this, linkObject);
      this.layoutStepPerformed();
    }
  },

  prepareIncrementalLinks : function()
  {
    // summary:
    //		Prepare the links if they are incremental.
    // tags:
    //		private
    
    if (this._links == null || this._links.length == 0) {
      return;
    }
    var layout = this.getLayout();
    if (layout.getInterGraphLinksRoutingModel() != null) {
      return;
    }
    var idata = layout.getIncrementalData();
    var grid = this.getGrid();
    if (idata == null) {
      return;
    }
    idata.prepare(layout, grid);

    var model = this.getModel();
    var ilink;
    var link;
    var numLinks = this.getNumberOfLinks();

    for (var i = 0; i < numLinks; i++)
    {
      link = this.getLink(i);

      if (link.needsRoute())
      {
        ilink = idata.getLinkData(link.getLinkObject());

        if (ilink != null)
        {
          if (ilink.needsReroute(model, grid) || link.isDirect())
          {
            if (link.getFromPoint() == null) {
              link.setFromPoint(ilink.getNewFromPoint());
            }
            if (link.getToPoint() == null) {
              link.setToPoint(ilink.getNewToPoint());
            }
          }
          else
          {
            link.setFrozen(true);
          }
        }
      }
    }

    this.createCompleteGrid();
  },

  routeAttachedLinks : function()
  {
    // summary:
    //		Route attached links.
    // tags:
    //		private
    
    if (this._links == null || this._links.length == 0) {
      return;
    }
    var i;
    var grid = this.getGrid();
    var layout = this.getLayout();
    var routeAlg = new LLLinkRouteAlgorithm(layout, grid);
    var termCandAlg = new LLTerminationCandAlgorithm(layout, grid);
    var crossRedAlg = null;
    var link;
    var numLinks = this.getNumberOfLinks();
    var crossRedEnabled = layout.isCrossingReductionEnabled();

    this._animation = false;
    this._interGraphLinkRouting = false;

    if (layout.getInterGraphLinksRoutingModel() != null)
    {
      this._animation = false;
      crossRedEnabled = false;
      this._interGraphLinkRouting = true;
    }

    if (crossRedEnabled) {
      crossRedAlg = new LLCrossRedAlgorithm(layout, grid);
    }
    var crossingsFound = false;

    this.routeDirectLinks(termCandAlg);

    if (layout.isStraightRouteEnabled())
    {
      grid.setImmediateReorganizationNeeded(false);

      for (i = 0; i < numLinks; i++)
      {
        if (i % 20 == 0 && !layout.mayContinue()) {
          break;
        }
        link = this.getLink(i);
        layout.addStepCount(2);

        if (link.needsRoute() && link.isCandidateForStraightRoute() && !link.isFrozen())
        {
          this.beforeRoute(link);

          termCandAlg.run(link);

          routeAlg.routeStraight(link);

          this.afterRoute(link);
        }

        this.layoutStepPerformed();
      }
    }

    grid.setImmediateReorganizationNeeded(true);
    var reduceCrossingsResult;

    for (i = 0; i < numLinks; i++)
    {
      if (i % 20 == 0 && !layout.mayContinue()) {
        break;
      }
      link = this.getLink(i);
      layout.addStepCount(10);

      if (link.needsRoute() && !link.isFrozen())
      {
        this.beforeRoute(link);

        termCandAlg.run(link);

        routeAlg.route(link);

        this.afterRoute(link);

        if (crossRedEnabled)
        {
          reduceCrossingsResult = this.reduceCrossings(link, i, crossRedAlg);
          crossingsFound = reduceCrossingsResult || crossingsFound;
        }

        this.layoutStepPerformed();
      }
    }

    if (crossRedEnabled && crossingsFound)
    {
      this.reduceCrossings(layout.getNumberCrossingReductionIterations(), crossRedAlg);
      this.layoutStepPerformed();
    }

    if (crossRedAlg != null)
    {
      crossRedAlg.dispose();
      crossRedAlg = null;
    }

    grid.setImmediateReorganizationNeeded(false);
    var isFallbackRouteEnabled = layout.isFallbackRouteEnabled();
    var fallbackRouteAlg = new LLFallbackRouteAlgorithm(layout, grid);

    for (i = 0; i < numLinks; i++)
    {
      if (i % 20 == 0 && !layout.mayContinue()) {
        break;
      }
      link = this.getLink(i);
      layout.addStepCount(1);

      if (link.needsRoute() && !link.isFrozen())
      {
        layout.addCalcFallbackLink(link.getLinkObject());

        if (isFallbackRouteEnabled)
        {
          this.beforeRoute(link);

          termCandAlg.run(link);

          fallbackRouteAlg.route(link);

          this.afterRoute(link);
          this.layoutStepPerformed();
        }
      }
    }

    grid.setImmediateReorganizationNeeded(true);

    termCandAlg.dispose();
    routeAlg.dispose();
    fallbackRouteAlg.dispose();

    if (layout.mayContinue())
    {
      layout._allowStopImmediately = false;

      for (i = 0; i < numLinks; i++)
      {
        this.getLink(i).reshapeLink(true, this._redraw);
        this.layoutStepPerformed();
      }
    }
  },

  beforeRoute : function(link)
  {
    // summary:
    //		Common actions before routing.
    // tags:
    //		private
    
    if (this._interGraphLinkRouting)
    {
      this.initGrid(link.getLinkObject());
    }
    else
    {
      var grid = this.getGrid();
      grid.ensureBorder(this._minimalBorder + 1);
      grid.reorganizeDirtyBeforeSearch();
    }
  },

  afterRoute : function(link)
  {
    // summary:
    //		Common actions after routing.
    // tags:
    //		private
    
    if (!this._interGraphLinkRouting)
    {
      link.updateGrid();

      if (this._animation) {
        link.drawLinkForAnimation();
      }
    }
  },

  reduceCrossings : function (a0, a1, a2)
  {
    // summary:
    //		Performs an intensive crossing reduction on all routed links. 
    // description:
    //		Performs an intensive crossing reduction on all routed links. 
    //		It iterates several times, until either nothing changes anymore or
    //		the maximum number of iterations is done.
    // tags:
    //		private
    
    if(arguments.length == 2)
      return this.reduceCrossings_overload_2(a0, a1);
    else if(arguments.length == 3)
      return this.reduceCrossings_overload_3(a0, a1, a2);
  },
  
  reduceCrossings_overload_2 : function(numberOfIterations, crossRedAlg)
                                                                        
  {
    var layout = this.getLayout();
    var numLinks = this.getNumberOfLinks();
    var i;
    var count = 0;
    var somethingChanged = true;
    var reduceCrossingsResult;

    while (somethingChanged && count < numberOfIterations && layout.mayContinue())
    {
      count++;
      somethingChanged = false;

      for (i = 1; i < numLinks; i++)
      {
        layout.addStepCount(2);
        reduceCrossingsResult = this.reduceCrossings(this.getLink(i), i, crossRedAlg);
        somethingChanged = reduceCrossingsResult || somethingChanged;
        this.layoutStepPerformed();
      }
    }
  },

  reduceCrossings_overload_3 : function(link, linkIndex, crossRedAlg)
  {
    // summary:
    //		Performs a crossing reduction of this link with all links that are
    //		already routed. 
    // description:
    //		Performs a crossing reduction of this link with all links that are
    //		already routed. 
    //		It returns true if something has changed.
    // tags:
    //		private
    
    if (link.isDirect()) {
      return (false); // Boolean
    }
    if (link.isFrozen()) {
      return (false); // Boolean
    }
    if (link.needsRoute()) {
      return (false); // Boolean
    }
    var layout = this.getLayout();
    var somethingChanged = false;
    var crossingReduced = false;
    var link2;
    var i;
    var j;
    var numLinks = this.getNumberOfLinks();
    var queryBounds = TranslateUtil.newArray(4, 0);
    var bounds;

    bounds = link.getBounds();

    for (i = 0; i < 4; i++)
      queryBounds[i] = bounds[i];

    for (i = 0; i < numLinks; i++)
    {
      if (i == linkIndex) {
        continue;
      }
      if (i % 20 == 0 && !layout.mayContinue()) {
        break;
      }
      link2 = this.getLink(i);
      crossingReduced = false;

      if (!link2.isFrozen() && !link2.needsRoute() && !link2.isDirect())
      {
        bounds = link2.getBounds();
        crossingReduced = crossRedAlg.run(link, link2);
        somethingChanged = crossingReduced || somethingChanged;

        if (somethingChanged)
        {
          if (bounds[0] < queryBounds[0]) {
            queryBounds[0] = bounds[0];
          }
          if (bounds[1] > queryBounds[1]) {
            queryBounds[1] = bounds[1];
          }
          if (bounds[2] < queryBounds[2]) {
            queryBounds[2] = bounds[2];
          }
          if (bounds[3] > queryBounds[3]) {
            queryBounds[3] = bounds[3];
          }
          bounds = link.getBounds();
          if (bounds[0] < queryBounds[0]) {
            queryBounds[0] = bounds[0];
          }
          if (bounds[1] > queryBounds[1]) {
            queryBounds[1] = bounds[1];
          }
          if (bounds[2] < queryBounds[2]) {
            queryBounds[2] = bounds[2];
          }
          if (bounds[3] > queryBounds[3]) {
            queryBounds[3] = bounds[3];
          }
          bounds = link2.getBounds();
          if (bounds[0] < queryBounds[0]) {
            queryBounds[0] = bounds[0];
          }
          if (bounds[1] > queryBounds[1]) {
            queryBounds[1] = bounds[1];
          }
          if (bounds[2] < queryBounds[2]) {
            queryBounds[2] = bounds[2];
          }
          if (bounds[3] > queryBounds[3]) {
            queryBounds[3] = bounds[3];
          }
          this.layoutStepPerformed();
        }
      }
    }

    if (somethingChanged)
    {
      for (j = 0; j < numLinks; j++)
        this.getLink(j).updateGridIfIntersects(queryBounds);
    }

    return (somethingChanged); // Boolean
  },

  layoutStepPerformed : function()
  {
    // summary:
    //		Informs the layout instance that a step of the layout algorithm has
    //		been performed. 
    // description:
    //		Informs the layout instance that a step of the layout algorithm has
    //		been performed. 
    //		The method notified the listeners of the layout events.
    // tags:
    //		private
    
    var layout = this.getLayout();
    if (layout != null) {
      layout.callLayoutStepPerformedIfNeeded();
    }
  },

  routeDirectLinks : function(termCandAlg)
  {
    // summary:
    //		Route all direct links.
    // tags:
    //		private
    
    var layout = this.getLayout();
    var grid = this.getGrid();
    var numLinks = this.getNumberOfLinks();
    var i;
    var link;
    var foundDirectLink = false;

    for (i = 0; i < numLinks; i++)
    {
      link = this.getLink(i);
      layout.addStepCount(1);

      if (link.isDirect())
      {
        if (this._interGraphLinkRouting) {
          this.beforeRoute(link);
        }
        termCandAlg.run(link);

        if (link.getStartPoints() != null && link.getEndPoints() != null && link.getStartPoints().length > 0 && link.getEndPoints().length > 0)
        {
          link.setFrozen(false);

          this.addDirectLinkToNode(link.getFromNode(), link);
          this.addDirectLinkToNode(link.getToNode(), link);
          foundDirectLink = true;
        }
        else
        {
          link.setFrozen(true);
          layout.addCalcFallbackLink(link);
        }
      }
    }

    if (!foundDirectLink) {
      return;
    }

    var routeAlg = new LLDirectRouteAlgorithm(layout, grid);
    var model = this.getModel();
    var e = new TranslateUtil.ArrayIterator(model.getNodes());
    var node;
    var links;

    while (e.hasNext())
    {
      node = e.next();
      links = this.getDirectLinksAtNode(node);
      if (links != null) {
        routeAlg.route(node, links);
      }
    }

    routeAlg.dispose();
    this.cleanupDirectLinkProperties();

    for (i = 0; i < numLinks; i++)
    {
      link = this.getLink(i);

      if (link.isDirect() && !link.isFrozen())
      {
        if (link.getActStartPoint() != null && link.getActEndPoint() != null)
        {
          link.setSegments(TranslateUtil.newArray(0, 0), null, null);

          link.reshapeLink(true, this._redraw);
        }
        else
        {
          layout.addCalcFallbackLink(link);
        }

        link.setFrozen(true);
      }

      this.layoutStepPerformed();
    }
  },

  getDirectLinksAtNode : function(node)
  {
    // summary:
    //		Get the vector of direct links that are incident to this node.
    // tags:
    //		private
    
    var links = this.getModel().getParameter(node, this.DIRECT_LINKS_AT_NODE_PROPERTY);
    return (((links != null) ? links : null)); // Array
  },

  addDirectLinkToNode : function(node, link)
  {
    // summary:
    //		Add a direct link to the vector of links incident to the node.
    // tags:
    //		private
    
    var links = this.getDirectLinksAtNode(node);

    if (links == null)
    {
      links = new Array();
      this.getModel().setParameter(node, this.DIRECT_LINKS_AT_NODE_PROPERTY, links);
    }

    links.push(link);
  },

  cleanupDirectLinkProperties : function()
  {
    // summary:
    //		Cleanup all properties at all nodes.
    // tags:
    //		private
    
    var model = this.getModel();
    var e = new TranslateUtil.ArrayIterator(model.getNodes());

    while (e.hasNext())
      model.setParameter(e.next(), this.DIRECT_LINKS_AT_NODE_PROPERTY, null);
  },

  getModel : function()
  {
    // summary:
    //		Returns the current graph model.
    // tags:
    //		private
    
    return (this._graphModel); // ibm_ilog.graphlayout.AbstractGraphLayoutModel
  },

  getLayout : function()
  {
    // summary:
    //		Returns the current graph layout.
    // tags:
    //		private
    
    return (this._graphLayout); // ibm_ilog.graphlayout.longlink.LongLinkLayout
  },

  getGrid : function()
  {
    // summary:
    //		Returns the current routing grid.
    // tags:
    //		private
    
    return (this._grid); // ibm_ilog.graphlayout.longlink.LLGrid
  },

  setGrid : function(grid)
  {
    // summary:
    //		Sets the current routing grid.
    // tags:
    //		private
    
    this._grid = grid;
  },

  getLink : function(i)
  {
    // summary:
    //		Returns the ith link.
    // tags:
    //		private
    
    return this._links[i]; // ibm_ilog.graphlayout.longlink.LLLink
  },

  getNumberOfLinks : function()
  {
    // summary:
    //		Returns the number of links to be routed.
    // tags:
    //		private
    
    return ((this._firstFreeLinkSlot > 0 ? this._firstFreeLinkSlot : 0)); // int
  },

  isRoutable : function(link)
  {
    // summary:
    //		Returns true if the link is reshapable and not fixed (independent of
    //		whether it belongs to the links that need route or not).
    // tags:
    //		private
    
    var layout = this.getLayout();

    var interGraphModel = layout.getInterGraphLinksRoutingModel();
    if (interGraphModel != null && interGraphModel.isFixedLink(link)) {
      return (false); // Boolean
    }
    if (layout.isPreserveFixedLinks() && layout.isFixed(link)) {
      return (false); // Boolean
    }

    var globalLinkStyle = layout.getGlobalLinkStyle();
    if (globalLinkStyle == LinkStyles.NO_RESHAPE_STYLE) {
      return (false); // Boolean
    }
    if (globalLinkStyle == LinkStyles.MIXED_STYLE && layout.getLinkStyle(link) == LinkStyles.NO_RESHAPE_STYLE) {
      return (false); // Boolean
    }
    return (true); // Boolean
  }

}
);

return LLGraph;
});

},
'ibm_ilog/graphlayout/longlink/LLGridSegment':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLGridSegment", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var LLGridSegment = declare("ibm_ilog.graphlayout.longlink.LLGridSegment", null, 
{
  // summary:
  //		A grid segment of a grid line of the routing grid. 
  // description:
  //		A grid segment of a grid line of the routing grid. 
  //		<p>
  //		It represents an area where a link can be routed along the grid line.
  //		During a routing, it also stores the minimal number of segments
  //		needed to reach this segment, the backtrack distance of this segment
  //		(i.e. the amount we needed to go away from the target in order to
  //		bypass obstacles to reach the target), the area of the segment where
  //		the backtrack distance is valid (which corresponds to the visited
  //		part of the segment). For details, see class LLGrid.  
  //		</p>
  // tags:
  //		private
  
  _gridLine: null,

  _startIndex: null,
  _endIndex: null,

  _backtrackDist: null,

  _minSegmentNumber: null,

  _btStartIndex: null,
  _btEndIndex: null,

  _next0: null,
  _next1: null,

  _btnext: null,

  constructor : function(owner, startIndex, endIndex, next)
  {
    this._gridLine = owner;
    this._startIndex = startIndex;
    this._endIndex = endIndex;
    this._backtrackDist = /*Number.MAX_VALUE*/2147483647;
    this._btStartIndex = /*Number.MAX_VALUE*/2147483647;
    this._btEndIndex = -1;
    this._minSegmentNumber = /*Number.MAX_VALUE*/2147483647;
    this._next0 = null;
    this._next1 = next;
    this._btnext = null;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._next0 = null;
    this._next1 = null;
    this._btnext = null;
    this._gridLine = null;
  },

  getGridLine : function()
  {
    // summary:
    //		Returns the owner grid line of this segment.
    // tags:
    //		private
    
    return (this._gridLine); // ibm_ilog.graphlayout.longlink.LLGridLine
  },

  getStartIndex : function()
  {
    // summary:
    //		Returns the start index of the segment, i.e. 
    // description:
    //		Returns the start index of the segment, i.e. 
    //		the index of the grid point where the segment starts. If it is a
    //		horizontal segment, this is an x-index while the own index of the
    //		grid line is an y index, otherwise convers.
    // tags:
    //		private
    
    return (this._startIndex); // int
  },

  getEndIndex : function()
  {
    // summary:
    //		Returns the end index of the segment, i.e. 
    // description:
    //		Returns the end index of the segment, i.e. 
    //		the index of the grid point where the segment ends. If it is a
    //		horizontal segment, this is an x-index while the own index of the
    //		grid line is an y index, otherwise convers.
    // tags:
    //		private
    
    return (this._endIndex); // int
  },

  setStartIndex : function(index)
  {
    // summary:
    //		Sets the start index.
    // tags:
    //		private
    
    this._startIndex = index;
  },

  setEndIndex : function(index)
  {
    // summary:
    //		Sets the end index.
    // tags:
    //		private
    
    this._endIndex = index;
  },

  getNext1 : function()
  {
    // summary:
    //		Returns the next segment in the same gridline, or null.
    // tags:
    //		private
    
    return (this._next1); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  getNext0 : function()
  {
    // summary:
    //		Returns the next segment in the same gridline, if the grid line is
    //		organized as binary tree, or null.
    // tags:
    //		private
    
    return (this._next0); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  setNexts : function(seg0, seg1)
  {
    // summary:
    //		Sets the both next pointer during reorganizing the gridline into a
    //		binary tree, or back into a list.
    // tags:
    //		private
    
    this._next0 = seg0;
    this._next1 = seg1;
  },

  isHorizontal : function()
  {
    // summary:
    //		Returns true if the segment is horizontal.
    // tags:
    //		private
    
    return (this.getGridLine().isHorizontal()); // Boolean
  },

  isDeadlock : function(grid, minIndex, maxIndex)
  {
    // summary:
    //		Returns true if the segment is an immediate deadlock withing
    //		minIndex and maxIndex, i.e. 
    // description:
    //		Returns true if the segment is an immediate deadlock withing
    //		minIndex and maxIndex, i.e. 
    //		if there is no other segment to continue.
    // tags:
    //		private
    
    if (this.isInvalid()) {
      return (true); // Boolean
    }
    var line = this.getGridLine();
    if (minIndex < this._startIndex) {
      minIndex = this._startIndex;
    }
    if (maxIndex > this._endIndex) {
      maxIndex = this._endIndex;
    }
    for (var i = minIndex; i <= maxIndex; i++)
      if (line.searchOrthogonalSegment(grid, i) != null) {
        return (false); // Boolean
      }
    return (true); // Boolean
  },

  isInvalid : function()
  {
    // summary:
    //		Returns true if the segment is not valid anymore. 
    // description:
    //		Returns true if the segment is not valid anymore. 
    //		This happens if a segment gets completely overlapped by an obstacle.
    // tags:
    //		private
    
    return (this._startIndex == -1 && this._endIndex == -1); // Boolean
  },

  removeInvalidNext : function()
  {
    // summary:
    //		Removes the next segment if it is invalid. 
    // description:
    //		Removes the next segment if it is invalid. 
    //		This should only be called while the gridline is a list.
    // tags:
    //		private
    
    var seg = this._next1;
    var aux;

    while (seg != null && seg.isInvalid())
    {
      aux = seg.getNext1();
      seg.dispose();
      seg = aux;
    }

    this._next1 = seg;
  },

  setObstacle : function(obstacleStartIndex, obstacleEndIndex)
  {
    // summary:
    //		Mark that there is an obstacle in the segment. 
    // description:
    //		Mark that there is an obstacle in the segment. 
    //		If it is in the middle of the segment, this splits the segment into
    //		two. This even works correct if obstacleStartIndex =
    //		obstacleEndIndex + 1, which happens e.g., for objects that are in
    //		beetwen two grid lines. This theoretically works if the grid line is
    //		a list or a binary tree,  but it will make the binary tree
    //		unbalanced,
    // tags:
    //		private
    
    if (this.isInvalid()) {
      return;
    }
    if (obstacleStartIndex > this._endIndex) {
      return;
    }
    if (obstacleEndIndex < this._startIndex) {
      return;
    }
    if (obstacleStartIndex <= this._startIndex)
    {
      if (this._endIndex <= obstacleEndIndex)
      {
        this._startIndex = -1;
        this._endIndex = -1;
      }
      else {
        this._startIndex = obstacleEndIndex + 1;
      }
    }
    else if (this._endIndex <= obstacleEndIndex)
    {
      this._endIndex = obstacleStartIndex - 1;
    }
    else
    {
      var seg = new LLGridSegment(this._gridLine, obstacleEndIndex + 1, this._endIndex, this._next1);
      this._next1 = seg;
      this._endIndex = obstacleStartIndex - 1;
    }
  },

  canBacktrack : function()
  {
    // summary:
    //		Returns true if the segment is a backtrack segment. 
    // description:
    //		Returns true if the segment is a backtrack segment. 
    //		This is the case, if not the complete segment is already covered by
    //		a valid backtrack distance.
    // tags:
    //		private
    
    return (this._btStartIndex > this._startIndex || this._btEndIndex < this._endIndex); // Boolean
  },

  getBacktrackStartIndex : function()
  {
    // summary:
    //		Returns the start index of the area where the current backtrack
    //		distance is valid or smaller. 
    // description:
    //		Returns the start index of the area where the current backtrack
    //		distance is valid or smaller. 
    //		In the remaining area, the real backtrack distance is larger than
    //		the stored one.
    // tags:
    //		private
    
    return (this._btStartIndex); // int
  },

  getBacktrackEndIndex : function()
  {
    // summary:
    //		Returns the end index of the area where the current backtrack
    //		distance is valid or smaller. 
    // description:
    //		Returns the end index of the area where the current backtrack
    //		distance is valid or smaller. 
    //		In the remaining area, the real backtrack distance is larger than
    //		the stored one.
    // tags:
    //		private
    
    return (this._btEndIndex); // int
  },

  setBacktrackStartIndex : function(startIndex)
  {
    // summary:
    //		Sets the start index of the area where the current backtrack
    //		distance is valid or smaller.
    // tags:
    //		private
    
    this._btStartIndex = startIndex;
  },

  setBacktrackEndIndex : function(endIndex)
  {
    // summary:
    //		Sets the end index of the area where the current backtrack distance
    //		is valid or smaller.
    // tags:
    //		private
    
    this._btEndIndex = endIndex;
  },

  isOnBacktrackStack : function()
  {
    // summary:
    //		Returns true if the segment is on the backtrack stack. 
    // description:
    //		Returns true if the segment is on the backtrack stack. 
    //		Note that the backtrack stack is so organized such that all real
    //		segments have a next segment, i.e. the last real segment points to a
    //		dummy next segment. Therefore it is enough to check the next
    //		pointer.
    // tags:
    //		private
    
    return (this._btnext != null); // Boolean
  },

  getNextBacktrackSegement : function()
  {
    // summary:
    //		Returns the next segment in the stack of backtrack segments, or
    //		null, if it's not on the backtrack stack.
    // tags:
    //		private
    
    return (this._btnext); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  setNextBacktrackSegement : function(next)
  {
    // summary:
    //		Sets the next segment in the stack of backtrack segments.
    // tags:
    //		private
    
    this._btnext = next;
  },

  getBacktrackDist : function()
  {
    // summary:
    //		Returns the stored backtrack distance of the segment.
    // tags:
    //		private
    
    return (this._backtrackDist); // int
  },

  setBacktrackDist : function(dist)
  {
    // summary:
    //		Set the backtrack distance of the segment.
    // tags:
    //		private
    
    this._backtrackDist = dist;
  },

  calcBacktrackDist : function(i)
  {
    // summary:
    //		Returns the real backtrack distance at index position i.
    // tags:
    //		private
    
    var backtrackDist = this.getBacktrackDist();
    if (backtrackDist == /*Number.MAX_VALUE*/2147483647) {
      return /*Number.MAX_VALUE*/2147483647; // int
    }
    if (i < this._startIndex || i > this._endIndex) {
      return /*Number.MAX_VALUE*/2147483647; // int
    }
    if (i < this._btStartIndex) {
      return (backtrackDist + this._btStartIndex - i); // int
    }
    if (i > this._btEndIndex) {
      return (backtrackDist + i - this._btEndIndex); // int
    }
    return (backtrackDist); // int
  },

  getMinSegmentNumber : function()
  {
    // summary:
    //		Returns the minimal number of segments necessary to reach this
    //		segment.
    // tags:
    //		private
    
    return (this._minSegmentNumber); // int
  },

  setMinSegmentNumber : function(segNumber)
  {
    // summary:
    //		Sets the minimal number of segments necessary to reach this segment.
    // tags:
    //		private
    
    this._minSegmentNumber = segNumber;
  },

  updateMinSegmentNumber : function(segNumber)
  {
    // summary:
    //		Updates the minimal number of segments necessary to reach this
    //		segment.
    // tags:
    //		private
    
    if (segNumber < this._minSegmentNumber)
    {
      this._minSegmentNumber = segNumber;
    }
  },

  clear : function()
  {
    // summary:
    //		Clear this segment before routing.
    // tags:
    //		private
    
    this._btnext = null;
    this._backtrackDist = /*Number.MAX_VALUE*/2147483647;
    this._btStartIndex = /*Number.MAX_VALUE*/2147483647;
    this._btEndIndex = -1;
    this._minSegmentNumber = /*Number.MAX_VALUE*/2147483647;
  },

  clearRecursive : function()
  {
    // summary:
    //		Clear recursively all segments reaching from this segment.
    // tags:
    //		private
    
    this.clear();
    if (this._next0 != null) {
      this._next0.clearRecursive();
    }
    if (this._next1 != null) {
      this._next1.clearRecursive();
    }
  },

  shiftIndex : function(delta)
  {
    // summary:
    //		Shifts the segment indexe by delta.
    // tags:
    //		private
    
    if (!this.isInvalid())
    {
      this._startIndex += delta;
      this._endIndex += delta;
      if (this._btStartIndex != /*Number.MAX_VALUE*/2147483647) {
        this._btStartIndex += delta;
      }
      if (this._btEndIndex != -1) {
        this._btEndIndex += delta;
      }
    }

    if (this._next0 != null) {
      this._next0.shiftIndex(delta);
    }
    if (this._next1 != null) {
      this._next1.shiftIndex(delta);
    }
  },

  searchOrthogonalSegment : function(grid, i)
  {
    // summary:
    //		Returns the grid segment that intersects this grid line at index
    //		position i.
    // tags:
    //		private
    
    return (this.getGridLine().searchOrthogonalSegment(grid, i)); // ibm_ilog.graphlayout.longlink.LLGridSegment
  }

}
);

return LLGridSegment;
});

},
'ibm_ilog/graphlayout/shortlink/_DirectIncidentLinksSorter':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_DirectIncidentLinksSorter", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_IncidentLinksSorter"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _IncidentLinksSorter	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _IncidentLinksSorter = ibm_ilog.graphlayout.shortlink._IncidentLinksSorter;
=====*/

var _DirectIncidentLinksSorter = declare("ibm_ilog.graphlayout.shortlink._DirectIncidentLinksSorter", _IncidentLinksSorter, 
{
  // summary:
  //		This subclass is used to sort the links incident to a node for the 
  //		direct link style. 
  // description:
  //		This subclass is used to sort the links incident to a node for the 
  //		direct link style. 
  //		A layout instance uses only one instance of this class.   
  // tags:
  //		private
  
  constructor : function(layout)
  {
  },
  
  compareLinks : function(link1, link2, origin1, origin2, nodeSide)
  {
    // summary:
    //		Compare two elements. 
    // description:
    //		Compare two elements. 
    //		This is used when computing the order of the nodes connected to the
    //		same side of a node.
    // tags:
    //		protected
    
    var destination1 = !origin1;
    var destination2 = !origin2;

    if (link1.getConnectionNode(destination1) == link2.getConnectionNode(destination2) && link1.getNodeSide(destination1) == link2.getNodeSide(destination2))
    {
      var oppositeConnPoint1 = link1.getConnectionPoint(destination1);
      var oppositeConnPoint2 = link2.getConnectionPoint(destination2);
      if (TranslateUtil.pointEquals(oppositeConnPoint1, oppositeConnPoint2)) {
        return true; // Boolean
      }
      else {
        return nodeSide.compareLinksByOppositeDirection(link1.getNodeSide(destination1).getDirection(), oppositeConnPoint1, oppositeConnPoint2); // Boolean
      }
    }

    var point11 = link1.getPointAt(origin1, 1);
    var point12 = link1.getPointAt(origin1, 2);
    var point21 = link2.getPointAt(origin2, 1);
    var point22 = link2.getPointAt(origin2, 2);
    var tangentDelta1 = nodeSide.getTangentDelta(point11, point12);
    var tangentDelta2 = nodeSide.getTangentDelta(point21, point22);

    if (tangentDelta1 * tangentDelta2 < 0) {
      return tangentDelta1 >= tangentDelta2; // Boolean
    }

    var shapeType1 = link1.getCurrentShapeType();
    var shapeType2 = link2.getCurrentShapeType();
    var isOrtho1 = shapeType1.isOrthogonal();
    var isOrtho2 = shapeType2.isOrthogonal();

    if (isOrtho1)
    {
      if (isOrtho2) {
        return nodeSide.compareOrthogonalLinks(this._layout, link1, link2, origin1, origin2); // Boolean
      }
      else if (link1.isSelfLink() && shapeType1.getNumberOfBends() == 3) {
        return tangentDelta1 >= 0; // Boolean
      }
    }
    else if (isOrtho2)
    {
      if (link2.isSelfLink() && shapeType2.getNumberOfBends() == 3) {
        return tangentDelta2 < 0; // Boolean
      }
    }

    var radialDelta1 = nodeSide.getPointRadialDelta(point11, point12);
    var radialDelta2 = nodeSide.getPointRadialDelta(point21, point22);
    return (radialDelta2 * tangentDelta1 >= radialDelta1 * tangentDelta2); // Boolean
  }

}
);

return _DirectIncidentLinksSorter;
});

},
'ibm_ilog/graphlayout/longlink/LLGridLine':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLGridLine", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../longlink/LLGridSegment"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    LLGridSegment	// #7
    ) {

var LLGridLine = declare("ibm_ilog.graphlayout.longlink.LLGridLine", null, 
{
  // summary:
  //		A grid line is owned by the routing grid (LLGrid) and consists of a
  //		list of grid segments. 
  // description:
  //		A grid line is owned by the routing grid (LLGrid) and consists of a
  //		list of grid segments. 
  //		<p>
  //		The grid line can be horizontal or vertical. The crossing point
  //		between a horizontal and a vertical grid line is a grid point, but we
  //		don't have any special data structure for each single grid point in the LLGrid data structure.   
  //		</p>
  //		<p>
  //		Routing is done along grid lines. Therefore, the grid line is split into grid segments that are usable and grid segments that are not usable i.e., invalid, because covered by another object. In fact, we remove the invalid segments before routing (they only occur during the creation of the grid lines) in order to make the algorithm fast. So, during routing, the grid lines contain only segments that are usable.  
  //		</p>
  //		<p>
  //		If a grid line has many segments, we organize the segments as binary tree in order to find them faster. Otherwise we organze the segments as linear list.  
  //		</p>
  //		<p>
  //		For details, see class LLGrid.  
  //		</p>
  // tags:
  //		private
  
  _segments: null,

  _index: null,

  _isHorizontal: null,

  _isBinTree: null,

  _nextDirty: null,

  constructor : function(maxGridPointIndex, index, isHorizontal)
  {
    this._index = index;
    this._isHorizontal = isHorizontal;
    this._isBinTree = false;

    this._segments = new LLGridSegment(this, 0, maxGridPointIndex, null);
    this._nextDirty = null;
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.convertToList();
    var seg = this._segments;
    var aux;

    while (seg != null)
    {
      aux = seg.getNext1();
      seg.dispose();
      seg = aux;
    }

    this._segments = null;

    this._nextDirty = null;
  },

  isHorizontal : function()
  {
    // summary:
    //		Returns true if the grid line is horizontal.
    // tags:
    //		private
    
    return (this._isHorizontal); // Boolean
  },

  getIndex : function()
  {
    // summary:
    //		Returns the index of the grid line.
    // tags:
    //		private
    
    return (this._index); // int
  },

  isDirty : function()
  {
    // summary:
    //		Returns true if the grid line is dirty, i.e. 
    // description:
    //		Returns true if the grid line is dirty, i.e. 
    //		needs to be reorganized before we start the next routing. Note that
    //		the dirty list is so organized such that all real dirty grid lines
    //		have a next line, i.e. the last real grid line points to a dummy
    //		next grid line. Therefore it is enough to check the next pointer.
    // tags:
    //		private
    
    return (this._nextDirty != null); // Boolean
  },

  getNextDirty : function()
  {
    // summary:
    //		Returns the next dirty grid line.
    // tags:
    //		private
    
    return (this._nextDirty); // ibm_ilog.graphlayout.longlink.LLGridLine
  },

  setNextDirty : function(next)
  {
    // summary:
    //		Sets the next dirty grid line.
    // tags:
    //		private
    
    this._nextDirty = next;
  },

  clear : function()
  {
    // summary:
    //		Clear all segments before routing.
    // tags:
    //		private
    
    if (this._segments != null) {
      this._segments.clearRecursive();
    }
    this._nextDirty = null;
  },

  reorganizeBeforeSearch : function(keepDirty)
  {
    // summary:
    //		Reorganize the grid line such that the grid segments for a binary
    //		tree. 
    // description:
    //		Reorganize the grid line such that the grid segments for a binary
    //		tree. 
    //		This also clears all segments before the next routing. Afterwards,
    //		the grid line is not anymore dirty.
    // tags:
    //		private
    
    this.convertToList();

    var seg = this._segments;
    var aux;

    while (seg != null && seg.isInvalid())
    {
      aux = seg.getNext1();
      seg.dispose();
      seg = aux;
    }

    this._segments = seg;
    var numSegments = 0;

    while (seg != null)
    {
      seg.clear();
      seg.removeInvalidNext();
      numSegments++;
      seg = seg.getNext1();
    }

    if (numSegments >= 8) {
      this.convertToBinTree(numSegments);
    }
    if (!keepDirty) {
      this._nextDirty = null;
    }
  },

  convertToList : function()
  {
    // summary:
    //		Convert the binary tree of the grid line into a linear list.
    // tags:
    //		private
    
    if (this._isBinTree)
    {
      var nextseg = this._segments;
      this._segments = null;
      this.listConversionTraversal(nextseg);
      this._isBinTree = false;
    }
  },

  convertToBinTree : function(numSegments)
  {
    // summary:
    //		Convert the segment list of the grid line into a binary tree.
    // tags:
    //		private
    
    if (numSegments > 0 && !this._isBinTree)
    {
      var segTab = TranslateUtil.newArray(numSegments, null);
      var seg = this._segments;
      var i = 0;

      while (seg != null)
      {
        segTab[i++] = seg;
        seg = seg.getNext1();
      }

      this._segments = this.bintreeConversionTraversal(segTab, 0, numSegments - 1);
      this._isBinTree = true;
    }
  },

  listConversionTraversal : function(segment)
  {
    // summary:
    //		Traverse the binary tree to convert it to a linear list. 
    // description:
    //		Traverse the binary tree to convert it to a linear list. 
    //		It always prepends in in-order to the &#95;segments.
    // tags:
    //		private
    
    if (segment == null) {
      return;
    }
    this.listConversionTraversal(segment.getNext1());
    var nextseg = this._segments;
    this._segments = segment;
    this.listConversionTraversal(segment.getNext0());
    segment.setNexts(null, nextseg);
  },

  bintreeConversionTraversal : function(segTab, startIndex, endIndex)
  {
    // summary:
    //		Traverse the list to convert it to a binary tree. 
    // description:
    //		Traverse the list to convert it to a binary tree. 
    //		It returns the root of the tree.
    // tags:
    //		private
    
    if (endIndex < startIndex) {
      return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    var middle = Math.floor((startIndex + endIndex) / 2);
    var seg = segTab[middle];
    seg.setNexts(this.bintreeConversionTraversal(segTab, startIndex, middle - 1), this.bintreeConversionTraversal(segTab, middle + 1, endIndex));
    return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  shiftIndex : function(ownDelta, segmentDelta)
  {
    // summary:
    //		Shift indexe by delta.
    // tags:
    //		private
    
    this._index += ownDelta;
    if (segmentDelta != 0 && this._segments != null) {
      this._segments.shiftIndex(segmentDelta);
    }
  },

  isFree : function(maxGridPoint)
  {
    // summary:
    //		Returns true if the grid line is completely free, i.e. 
    // description:
    //		Returns true if the grid line is completely free, i.e. 
    //		without obstacles.
    // tags:
    //		private
    
    if (this._segments == null) {
      return (false); // Boolean
    }
    if (this._segments.getStartIndex() > 0) {
      return (false); // Boolean
    }
    if (this._segments.getEndIndex() < maxGridPoint) {
      return (false); // Boolean
    }
    return (true); // Boolean
  },

  addSegment : function(startIndex, endIndex)
  {
    // summary:
    //		Add a new segment to this grid line.
    // tags:
    //		private
    
    var seg = new LLGridSegment(this, startIndex, endIndex, null);

    if (this._segments == null)
    {
      this._segments = seg;
    }
    else if (this._isBinTree)
    {
      var otherseg = this.searchClosestSegment(startIndex, false);

      if (otherseg != null)
      {
        seg.setNexts(null, otherseg.getNext1());
        otherseg.setNexts(otherseg.getNext0(), seg);
      }
      else
      {
        otherseg = this.searchClosestSegment(endIndex, true);

        otherseg.setNexts(seg, otherseg.getNext1());
      }
    }
    else
    {
      var predseg = this.searchClosestSegment(startIndex, false);

      if (predseg != null)
      {
        seg.setNexts(null, predseg.getNext1());
        predseg.setNexts(null, seg);
      }
      else
      {
        seg.setNexts(null, this._segments);
        this._segments = seg;
      }
    }
  },

  searchSegment : function(i)
  {
    // summary:
    //		Search the segment of this line that has index i within its range.
    // tags:
    //		private
    
    var seg = this._segments;

    if (this._isBinTree)
    {
      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (i < seg.getStartIndex()) {
          seg = seg.getNext0();
        }
        else {
          seg = seg.getNext1();
        }
      }
    }
    else
    {
      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        seg = seg.getNext1();
      }
    }

    return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchClosestSegment : function(i, larger)
  {
    // summary:
    //		Search the closest segment to index i. 
    // description:
    //		Search the closest segment to index i. 
    //		It either has index i within its range, or is larger or smaller that
    //		i, depending on the flag "larger".
    // tags:
    //		private
    
    var seg = this._segments;

    if (this._isBinTree)
    {
      var bestSeg = null;

      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (larger == true && i < seg.getStartIndex()) {
          bestSeg = seg;
        }
        if (larger == false && seg.getEndIndex() < i) {
          bestSeg = seg;
        }
        if (i < seg.getStartIndex()) {
          seg = seg.getNext0();
        }
        else {
          seg = seg.getNext1();
        }
      }

      return (bestSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    else
    {
      var prevSeg = null;

      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (i < seg.getStartIndex())
        {
          if (larger) {
            return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
          else {
            return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
        }

        prevSeg = seg;
        seg = seg.getNext1();
      }

      if (!larger) {
        return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
    }

    return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchClosestSegmentAlive : function(grid, i, larger)
  {
    // summary:
    //		Search the closest segment to index i that is not a deadlock. 
    // description:
    //		Search the closest segment to index i that is not a deadlock. 
    //		It either has index i within its range, or is larger or smaller that
    //		i, depending on the flag "larger".
    // tags:
    //		private
    
    var seg = this._segments;
    var minIndex;
    var maxIndex;

    if (larger)
    {
      minIndex = i;
      maxIndex = /*Number.MAX_VALUE*/2147483647;
    }
    else
    {
      minIndex = 0;
      maxIndex = i;
    }

    if (this._isBinTree)
    {
      if (larger)
      {
        while (seg != null)
        {
          if (seg.getEndIndex() >= i) {
            break;
          }
          seg = seg.getNext1();
        }
      }
      else
      {
        while (seg != null)
        {
          if (seg.getStartIndex() <= i) {
            break;
          }
          seg = seg.getNext0();
        }
      }

      return this.searchClosestSegmentAliveBinTree(grid, i, larger, minIndex, maxIndex, seg, null); // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    else
    {
      var prevSeg = null;

      while (seg != null && seg.isDeadlock(grid, minIndex, maxIndex))
        seg = seg.getNext1();

      while (seg != null)
      {
        if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
          return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
        }
        if (i < seg.getStartIndex())
        {
          if (larger) {
            return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
          else {
            return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
          }
        }

        prevSeg = seg;
        seg = seg.getNext1();

        while (seg != null && seg.isDeadlock(grid, minIndex, maxIndex))
          seg = seg.getNext1();
      }

      if (!larger) {
        return (prevSeg); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
    }

    return (null); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchClosestSegmentAliveBinTree : function(grid, i, larger, minIndex, maxIndex, seg, bestSeg)
  {
    // summary:
    //		Search the closest segment to index i that is not a deadlock in the
    //		binary tree.
    // tags:
    //		private
    
    if (seg == null) {
      return bestSeg; // ibm_ilog.graphlayout.longlink.LLGridSegment
    }
    if (!seg.isDeadlock(grid, minIndex, maxIndex))
    {
      if (seg.getStartIndex() <= i && i <= seg.getEndIndex()) {
        return (seg); // ibm_ilog.graphlayout.longlink.LLGridSegment
      }
      if (larger)
      {
        if (i < seg.getStartIndex() && (bestSeg == null || seg.getStartIndex() < bestSeg.getStartIndex())) {
          bestSeg = seg;
        }
      }
      else
      {
        if (seg.getEndIndex() < i && (bestSeg == null || bestSeg.getEndIndex() < seg.getEndIndex())) {
          bestSeg = seg;
        }
      }
    }

    bestSeg = this.searchClosestSegmentAliveBinTree(grid, i, larger, minIndex, maxIndex, seg.getNext0(), bestSeg);
    bestSeg = this.searchClosestSegmentAliveBinTree(grid, i, larger, minIndex, maxIndex, seg.getNext1(), bestSeg);
    return bestSeg; // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  searchOrthogonalSegment : function(grid, i)
  {
    // summary:
    //		Returns the grid segment that intersects this grid line at index
    //		position i.
    // tags:
    //		private
    
    var dir;
    if (this._isHorizontal) {
      dir = 0;
    }
    else {
      dir = 1;
    }
    return (grid.getGridLine(dir, i).searchSegment(this._index)); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  calcApproxCrossings : function(grid, start, end)
  {
    // summary:
    //		Calculates the approximate number of link crossings (line obstacle
    //		crossings) of a segment ranging from start to end on this grid line.
    //		
    // description:
    //		Calculates the approximate number of link crossings (line obstacle
    //		crossings) of a segment ranging from start to end on this grid line.
    //		
    //		It is precise if all line obstacles are on the grid. But if a line
    //		obstacle is not orthogonal, or if two line obstacles overlap at the
    //		same grid line, the result may be slightly wrong. But still good
    //		enough to serve as cost value for the heuristic. Of course this
    //		makes sense only if there is a segment covering from start to end on
    //		this grid line.
    // tags:
    //		private
    
    var numCrossings = 0;
    var min = ((start < end ? start : end));
    var max = ((start > end ? start : end));

    for (var i = min; i < max; i++)
    {
      if (this.searchOrthogonalSegment(grid, i) == null) {
        numCrossings++;
      }
    }

    return (numCrossings); // int
  },

  setObstacle : function(grid, obstacleStartIndex, obstacleEndIndex)
  {
    // summary:
    //		Mark that there is an obstacle in the grid line.
    // tags:
    //		private
    
    var seg = this._segments;
    grid.markDirty(this);
    var hasInvalids = false;

    if (this._isBinTree)
    {
      hasInvalids = this.setObstacleRec(this._segments, obstacleStartIndex, obstacleEndIndex);
    }
    else
    {
      var segInv;

      while (seg != null && seg.getStartIndex() <= obstacleEndIndex)
      {
        seg.setObstacle(obstacleStartIndex, obstacleEndIndex);
        segInv = seg.isInvalid();
        hasInvalids = segInv || hasInvalids;
        seg = seg.getNext1();
      }
    }

    if (hasInvalids && grid.isImmediateReorganizationNeeded()) {
      this.reorganizeBeforeSearch(true);
    }
  },

  setObstacleRec : function(segment, obstacleStartIndex, obstacleEndIndex)
  {
    // summary:
    //		In case of a binary tree, this recursive method does the real work. 
    // description:
    //		In case of a binary tree, this recursive method does the real work. 
    //		It returns true if any segment becomes invalid during setting
    //		obstacles.
    // tags:
    //		private
    
    if (segment == null) {
      return (false); // Boolean
    }
    segment.setObstacle(obstacleStartIndex, obstacleEndIndex);
    var hasInvalids = segment.isInvalid();
    var result;

    if (segment.isInvalid() || segment.getStartIndex() > obstacleStartIndex)
    {
      result = this.setObstacleRec(segment.getNext0(), obstacleStartIndex, obstacleEndIndex);
      hasInvalids = result || hasInvalids;
    }

    if (segment.isInvalid() || segment.getEndIndex() < obstacleEndIndex)
    {
      result = this.setObstacleRec(segment.getNext1(), obstacleStartIndex, obstacleEndIndex);
      hasInvalids = result || hasInvalids;
    }

    return (hasInvalids); // Boolean
  }

}
);

return LLGridLine;
});

},
'ibm_ilog/graphlayout/longlink/LLTerminationPoint':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LLTerminationPoint", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/QuickSort",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../Direction"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    QuickSort,	// #6
    TranslateUtil,	// #7
    Direction	// #8
    ) {

var LLTerminationPoint = declare("ibm_ilog.graphlayout.longlink.LLTerminationPoint", null, 
{
  // summary:
  //		An auxiliary data structure to specify a point on the grid where a
  //		link can terminate, i.e. 
  // description:
  //		An auxiliary data structure to specify a point on the grid where a
  //		link can terminate, i.e. 
  //		the start or end point of a link. For both "from" and "to" node of a
  //		link, we calculate the set of possible termination points.  The
  //		termination points have a penalty and an internal order, indicating
  //		how "good" they are for this link (lower penalty == better for the
  //		link). The link starting at a termination point first runs along one
  //		segment. However, since the termination points must have a specified
  //		distance from the corresponding end node (specified by
  //		setMinStartSegmentLength or setMinEndSegmentLength), there is an area
  //		on this segment where the link cannot join the segment. This is to
  //		avoid 180 degree bends, such as WRONG:                           
  //		CORRECT: |                                                     |
  //		,-------.     |     term. point   ,-------.           term. point   |
  //		|       |-----+=========*         |       |---------------*---------+
  //		`-------'                         `-------' ^                        
  //		 ^              ^         ^ |                             \          
  //		/          | join point                        forbidden area     
  //		join point  
  // tags:
  //		private
  
  _index: null,

  _penalty: null,

  _internalOrder: null,

  _segment: null,

  _forbiddenMinIndex: null,
  _forbiddenMaxIndex: null,

  _side: null,

  constructor : function (a0, a1, a2, a3, a4, a5, a6, a7, a8)
  {
    // summary:
    //		The constructor.
    
    if(arguments.length == 0)
      this._constructor_overload_LLTerminationPoint_0();
    else if(arguments.length == 9)
      this._constructor_overload_LLTerminationPoint_9(a0, a1, a2, a3, a4, a5, a6, a7, a8);
  },
  
  _constructor_overload_LLTerminationPoint_0 : function()
  {

    this._index = TranslateUtil.newArray(2, 0);
  },
  
  _constructor_overload_LLTerminationPoint_9 : function(index0, index1, forbiddenMinIndex, forbiddenMaxIndex, dir0, penalty, order, side, segment)
  {
    this._index = TranslateUtil.newArray(2, 0);
    this._index[dir0] = index0;
    this._index[1 - dir0] = index1;
    this._penalty = penalty;
    this._internalOrder = order;
    this._side = side;
    this._segment = segment;

    if (forbiddenMinIndex <= forbiddenMaxIndex)
    {
      this._forbiddenMinIndex = forbiddenMinIndex;
      this._forbiddenMaxIndex = forbiddenMaxIndex;
    }
    else
    {
      this._forbiddenMinIndex = forbiddenMaxIndex;
      this._forbiddenMaxIndex = forbiddenMinIndex;
    }
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._index = null;
    this._segment = null;
  },

  copyData : function(p, offsetX, offsetY)
  {
    // summary:
    //		Copy all internal data from the input termination point. 
    // description:
    //		Copy all internal data from the input termination point. 
    //		The offsets allow to correct the grid coordinates.
    // tags:
    //		private
    
    this._index[0] = p._index[0] + offsetX;
    this._index[1] = p._index[1] + offsetY;
    this._penalty = p._penalty;
    this._internalOrder = p._internalOrder;
    this._side = p._side;
    this._segment = p._segment;
    this._forbiddenMinIndex = p._forbiddenMinIndex;
    this._forbiddenMaxIndex = p._forbiddenMaxIndex;
  },

  getIndex : function(dir)
  {
    // summary:
    //		Returns the grid index of the termination point in given direction.
    // tags:
    //		private
    
    return (this._index[dir]); // int
  },

  setIndex : function(dir, value)
  {
    // summary:
    //		Sets the grid index of the termination point in given direction.
    // tags:
    //		private
    
    this._index[dir] = value;
  },

  getPenalty : function()
  {
    // summary:
    //		Returns the penalty of the termination point. 
    // description:
    //		Returns the penalty of the termination point. 
    //		If we have several candidates where a link can start or end, we want
    //		to prefer the one with the lowest penalty.
    // tags:
    //		private
    
    return (this._penalty); // int
  },

  getOrder : function()
  {
    // summary:
    //		Returns the internal order value of the termination point. 
    // description:
    //		Returns the internal order value of the termination point. 
    //		Same purpose as the penalty, but lower priority. The penalty can be
    //		influenced by the user, but the internal order cannot. It is only
    //		used if many termination points have the same penalty.
    // tags:
    //		private
    
    return (this._internalOrder); // int
  },

  getSide : function()
  {
    // summary:
    //		Returns the side where this termination point is placed at the node.
    //		
    // description:
    //		Returns the side where this termination point is placed at the node.
    //		
    //		Valid values are Left, Top, Right, Bottom from Direction.
    // tags:
    //		private
    
    return (this._side); // int
  },

  getDeviation : function(grid, rect)
  {
    // summary:
    //		Returns the percentual deviation of the termination point from the
    //		center of the border of the input rectangle, relative to the size of
    //		the border. 
    // description:
    //		Returns the percentual deviation of the termination point from the
    //		center of the border of the input rectangle, relative to the size of
    //		the border. 
    //		For instance, if the termination point is placed at the middle of
    //		the top border, then the deviation is 0. If it is placed at the
    //		leftmost or rightmost point of the top border, the deviation is 100
    //		percent.
    // tags:
    //		private
    
    if (this._segment.isHorizontal()) {
      return (LLTerminationPoint.getDeviation(grid, rect, this.getIndex(1), 0)); // Number
    }
    return (LLTerminationPoint.getDeviation(grid, rect, this.getIndex(0), 1)); // Number
  },

  getSegment : function()
  {
    // summary:
    //		Returns the corresponding grid segment that has this termination
    //		point.
    // tags:
    //		private
    
    if (this._segment != null && this._segment.getGridLine() == null)
    {
      this._segment = null;
    }

    return (this._segment); // ibm_ilog.graphlayout.longlink.LLGridSegment
  },

  getForbiddenMinIndex : function()
  {
    // summary:
    //		Returns the lowest grid index on the segment that is forbidden to
    //		traverse in order to respect the minimal start/end segment length.
    // tags:
    //		private
    
    return (this._forbiddenMinIndex); // int
  },

  getForbiddenMaxIndex : function()
  {
    // summary:
    //		Returns the highest grid index on the segment that is forbidden to
    //		traverse in order to respect the minimal start/end segment length.
    // tags:
    //		private
    
    return (this._forbiddenMaxIndex); // int
  },

  sortAccordingToPenalty : function(array)
  {
    // summary:
    //		Sorts the array of termination points according to increasing
    //		penalty.  
    // tags:
    //		private
    
    if (array.length > 1)
    {
      var sortAlg = new LLTerminationPoint.TerminationPointSortAlg(array);
      sortAlg.sort(array.length);
      sortAlg.dispose();
    }
  },

  sortAccordingToSide : function(array, sidePref)
  {
    // summary:
    //		Sorts the array of termination points according to side preferences.
    //		
    // description:
    //		Sorts the array of termination points according to side preferences.
    //		
    //		sidePref is an array of four, and the sides have the following
    //		indexe in the array: top = 0, left = 1, bottom = 2, right = 3.  
    // tags:
    //		private
    
    if (array.length > 1)
    {
      var sortAlg = new LLTerminationPoint.TerminationPointSideSortAlg(array, sidePref);
      sortAlg.sort(array.length);
      sortAlg.dispose();
    }
  }

}
);

lang.mixin(LLTerminationPoint,
{

  getDeviation : function(grid, rect, index, dir)
  {
    var deviation;

    if (dir == 0)
    {
      if (rect.height == 0.0) {
        return (0.0); // Number
      }
      deviation = grid.getGridCoord(1, index) - rect.y - 0.5 * rect.height;
      if (deviation < 0.0) {
        deviation = -deviation;
      }
      return (200.0 * deviation / rect.height); // Number
    }
    else
    {
      if (rect.width == 0.0) {
        return (0.0); // Number
      }
      deviation = grid.getGridCoord(0, index) - rect.x - 0.5 * rect.width;
      if (deviation < 0.0) {
        deviation = -deviation;
      }
      return (200.0 * deviation / rect.width); // Number
    }
  }

}
);

LLTerminationPoint.TerminationPointSortAlg = declare(QuickSort, 
{
  _array: null,

  constructor : function(array)
  {
    this._array = array;
  },
  
  dispose : function()
  {
  },

  compare : function(loc1, loc2)
  {
    if (this._array[loc1] == this._array[loc2]) {
      return 0; // int
    }
    if (this._array[loc1] == null) {
      return (1); // int
    }
    if (this._array[loc2] == null) {
      return (-1); // int
    }
    if (this._array[loc1].getPenalty() == this._array[loc2].getPenalty()) {
      return (this._array[loc1].getOrder() - this._array[loc2].getOrder()); // int
    }
    return (this._array[loc1].getPenalty() - this._array[loc2].getPenalty()); // int
  },

  swap : function(loc1, loc2)
  {
    var h = this._array[loc1];
    this._array[loc1] = this._array[loc2];
    this._array[loc2] = h;
  }

}
);
LLTerminationPoint.TerminationPointSideSortAlg = declare(QuickSort, 
{
  _sidePref: null,
  _array: null,

  constructor : function(array, sidePref)
  {
    this._array = array;
    this._sidePref = sidePref;
  },
  
  dispose : function()
  {
    this._sidePref = null;
  },

  compare : function(loc1, loc2)
  {
    if (this._array[loc1] == this._array[loc2]) {
      return 0; // int
    }
    if (this._array[loc1] == null) {
      return (1); // int
    }
    if (this._array[loc2] == null) {
      return (-1); // int
    }
    var sidePref1 = 0;
    var sidePref2 = 0;

    switch (this._array[loc1].getSide())
    {
      case Direction.TOP:
        sidePref1 = this._sidePref[0];
        break;
      case Direction.LEFT:
        sidePref1 = this._sidePref[1];
        break;
      case Direction.BOTTOM:
        sidePref1 = this._sidePref[2];
        break;
      case Direction.RIGHT:
        sidePref1 = this._sidePref[3];
        break;
    }
    switch (this._array[loc2].getSide())
    {
      case Direction.TOP:
        sidePref2 = this._sidePref[0];
        break;
      case Direction.LEFT:
        sidePref2 = this._sidePref[1];
        break;
      case Direction.BOTTOM:
        sidePref2 = this._sidePref[2];
        break;
      case Direction.RIGHT:
        sidePref2 = this._sidePref[3];
        break;
    }
    if (sidePref1 != sidePref2) {
      return (sidePref2 - sidePref1); // int
    }
    if (this._array[loc1].getPenalty() != this._array[loc2].getPenalty()) {
      return (this._array[loc1].getPenalty() - this._array[loc2].getPenalty()); // int
    }
    return (this._array[loc1].getOrder() - this._array[loc2].getOrder()); // int
  },

  swap : function(loc1, loc2)
  {
    var h = this._array[loc1];
    this._array[loc1] = this._array[loc2];
    this._array[loc2] = h;
  }

}
);

return LLTerminationPoint;
});

},
'ibm_ilog/graphlayout/shortlink/_LinkShapeProducer':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_LinkShapeProducer", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LinkStyles",	// #6
  "../internalutil/LogResUtil",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/SubgraphData",	// #9
  "../internalutil/TranslateUtil",	// #10
  "../shortlink/_LinkShapeType",	// #11
  "../shortlink/_SLNodeData",	// #12
  "../Direction"	// #13
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LinkStyles,	// #6
    LogResUtil,	// #7
    Rectangle,	// #8
    SubgraphData,	// #9
    TranslateUtil,	// #10
    _LinkShapeType,	// #11
    _SLNodeData,	// #12
    Direction	// #13
    ) {

var _LinkShapeProducer = declare("ibm_ilog.graphlayout.shortlink._LinkShapeProducer", null, 
{
  // summary:
  //		A class that is used to produce candidate shapes for the links. 
  // description:
  //		A class that is used to produce candidate shapes for the links. 
  //		This class is used by the link layout algorithms.  
  // tags:
  //		extension
  
  _nodeSideFilter: null,

  _bypassDistance: null,
  _layout: null,
  _fromNode: null,
  _toNode: null,
  _link: null,
  _minFinalSegmentLength: null,

  _shapeTypes : null,

  _counter : null,

  _selfLinkTypes: null,

  constructor : function(layout)
  {
    this._shapeTypes = TranslateUtil.newArray(60, null);
    this._counter = 0;
    this.update(layout);
  },
  
  getMaxNumberOfBends : function()
  {
    // summary:
    //		Returns the maximum number of intermediate points of the link shapes
    //		produced by this Producer.
    // tags:
    //		extension
    },
  
  getTopToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getTopToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getTopToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getTopToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getBottomToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getBottomToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getBottomToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getBottomToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getLeftToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getLeftToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getLeftToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getLeftToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getRightToTopSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getRightToBottomSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getRightToLeftSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getRightToRightSideTypes : function(fromRect, toRect, isOverlapping, bypassDistance)
  {},
  
  getSelfLinkShapeTypes : function(linkData)
  {
    // summary:
    //		Returns the shape types for a self-link. 
    // description:
    //		Returns the shape types for a self-link. 
    //		Currently, the same kind of self-links for DIRECT and ORTHOGONAL
    //		styles.
    // tags:
    //		private
    
    var adjNode = linkData.getFromNode();

    var selfLinkIndex = adjNode.getSelfLinkIndex();

    var shapeTypes = this._selfLinkTypes[selfLinkIndex];

    adjNode.incrementSelfLinkIndex(this._selfLinkTypes.length - 1);
    return shapeTypes; // Array
  },

  getAllQuasiSelfInterGraphLinkShapeTypes : function()
  {
    // summary:
    //		Returns all the shape types for quasi-self-intergraph-links. 
    // description:
    //		Returns all the shape types for quasi-self-intergraph-links. 
    //		Used Only when the node side filtering or side-constraints have
    //		rejected all the shapes obtained by the fine-grain choice depending
    //		on the relative position of the end-nodes.
    // tags:
    //		protected
    },
  
  getFromBottomLeftToTopRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		Returns an array of shape types and computes the link region. 
    // description:
    //		Returns an array of shape types and computes the link region. 
    //		The selection of shapes depends on the relative position of the
    //		nodes and eventually on various settings. ----- |   | | T | |   |
    //		----- ----- | | | F | |   | -----
    // tags:
    //		protected
    },
  
  getFromTopLeftToBottomRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    },
  
  getFromLeftToRightPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		-----   ----- | F |   | T | -----   -----
    // tags:
    //		protected
    },
  
  getFromBottomRightToTopLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | T | ----- ----- | F | -----
    // tags:
    //		protected
    },
  
  getFromTopRightToBottomLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    },
  
  getFromRightToLeftPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		-----   ----- | T |   | F | -----   -----
    // tags:
    //		protected
    },
  
  getFromBottomToTopPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | T | ----- ----- | F | -----
    // tags:
    //		protected
    },
  
  getFromTopToBottomPositionTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		----- | F | ----- ----- | T | -----
    // tags:
    //		protected
    },
  
  getOverlapPositionTypes : function(fromRect, toRect, bypassDistance)
  {
    // summary:
    //		Currently, both DIRECT and ORTHOGONAL styles use the same shapes for
    //		overlapping nodes. 
    // description:
    //		Currently, both DIRECT and ORTHOGONAL styles use the same shapes for
    //		overlapping nodes. 
    //		F and T overlap on both x-axis and y-axis ------- |  T  | | --------
    //		| |   |  | -------  | |   F  | --------
    // tags:
    //		private
    
    if (fromRect.x == toRect.x && fromRect.width == toRect.width && fromRect.y == toRect.y && fromRect.height == toRect.height) {
      return _LinkShapeProducer.OVERLAPPING_NODES_TYPES; // Array
    }
    var condLeftHor = toRect.x < fromRect.x;
    var condTopVert = toRect.y < fromRect.y;
    var condRightHor = toRect.x + toRect.width < fromRect.x + fromRect.width;
    var condBottomVert = toRect.y + toRect.height < fromRect.y + fromRect.height;
    var count = 0;

    if (condRightHor)
    {
      if (condTopVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendRT_OVERLAP.instance;
      }
      if (!condBottomVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendRB_OVERLAP.instance;
      }
    }
    else
    {
      if (!condTopVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendTR_OVERLAP.instance;
      }
      if (condBottomVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendBR_OVERLAP.instance;
      }
    }

    if (condLeftHor)
    {
      if (condBottomVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendBL_OVERLAP.instance;
      }
      if (!condTopVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendTL_OVERLAP.instance;
      }
    }
    else
    {
      if (!condBottomVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendLB_OVERLAP.instance;
      }
      if (condTopVert) {
        this._shapeTypes[count++] = _LinkShapeType.ThreeBendLT_OVERLAP.instance;
      }
    }

    if (count < 1) {
      return _LinkShapeProducer.OVERLAPPING_NODES_TYPES; // Array
    }
    else
    {
      return this.copyShapeTypes(this._shapeTypes, count); // Array
    }
  },

  getQuasiSelfInterGraphLinkShapeTypes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		Returns the shape types for quasi self-intergraph links that are
    //		optimal for the relative position of the end-nodes.
    // tags:
    //		private
    
    var subgraphBox = null;
    var nodeBox = null;

    if (linkData.isQuasiSelfInterGraphLinkOrigin())
    {
      subgraphBox = fromRect;
      nodeBox = toRect;
    }
    else if (linkData.isQuasiSelfInterGraphLinkDestination())
    {
      subgraphBox = toRect;
      nodeBox = fromRect;
    }
    else
    {
      LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6622E");
    }

    if (
      (subgraphBox.x == nodeBox.x && subgraphBox.width == nodeBox.width && subgraphBox.y == nodeBox.y && subgraphBox.height == nodeBox.height) ||
        (subgraphBox.width <= nodeBox.width && subgraphBox.height <= nodeBox.height)) {
        return this.getAllQuasiSelfInterGraphLinkShapeTypes(); // Array
    }
    var enoughSpaceOnLeft = nodeBox.x >= subgraphBox.x + bypassDistance;
    var enoughSpaceOnRight = nodeBox.x + nodeBox.width <= subgraphBox.x + subgraphBox.width - bypassDistance;
    var enoughSpaceOnTop = nodeBox.y >= subgraphBox.y + bypassDistance;
    var enoughSpaceOnBottom = nodeBox.y + nodeBox.height <= subgraphBox.y + subgraphBox.height - bypassDistance;
    var count = 0;
    var allShapeTypes = this.getAllQuasiSelfInterGraphLinkShapeTypes();
    var nShapes = allShapeTypes.length;
    var shapeType;

    for (var i = 0; i < nShapes; i++)
    {
      shapeType = allShapeTypes[i];
      if (this.isEnoughSpace(linkData, shapeType, subgraphBox, nodeBox, bypassDistance, enoughSpaceOnLeft, enoughSpaceOnRight, enoughSpaceOnTop, enoughSpaceOnBottom)) {
        this._shapeTypes[count++] = shapeType;
      }
    }

    if (count < 1) {
      return this.getAllQuasiSelfInterGraphLinkShapeTypes(); // Array
    }
    else
    {
      return this.copyShapeTypes(this._shapeTypes, count); // Array
    }
  },

  isEnoughSpace : function(linkData, shapeType, subgraphBox, nodeBox, bypassDistance, enoughSpaceOnLeft, enoughSpaceOnRight, enoughSpaceOnTop, enoughSpaceOnBottom)
  {
    // summary:
    //		Returns true if
    //		there's enough space on the node side, and returns false otherwise. 
    // description:
    //		Returns true if there's enough space on the node side, and returns
    //		false otherwise. 
    //		Overridden in OrthogonalLinkShapeProducer to add more checks.
    // tags:
    //		protected
    
    return this.isEnoughNodeSideSpace(shapeType.getFromSide(linkData), enoughSpaceOnLeft, enoughSpaceOnRight, enoughSpaceOnTop, enoughSpaceOnBottom) &&
      this.isEnoughNodeSideSpace(shapeType.getToSide(linkData), enoughSpaceOnLeft, enoughSpaceOnRight, enoughSpaceOnTop, enoughSpaceOnBottom); // Boolean
  },

  isEnoughNodeSideSpace : function(nodeSide, enoughSpaceOnLeft, enoughSpaceOnRight, enoughSpaceOnTop, enoughSpaceOnBottom)
  {
    // summary:
    //		Returns true if
    //		there's enough space on the node side, and returns false otherwise.
    // tags:
    //		private
    
    switch (nodeSide)
    {
      case Direction.LEFT:
        if (!enoughSpaceOnLeft) {
          return false; // Boolean
        }
        break;
      case Direction.RIGHT:
        if (!enoughSpaceOnRight) {
          return false; // Boolean
        }
        break;
      case Direction.TOP:
        if (!enoughSpaceOnTop) {
          return false; // Boolean
        }
        break;
      case Direction.BOTTOM:
        if (!enoughSpaceOnBottom) {
          return false; // Boolean
        }
        break;
      default:

        var params = [nodeSide ];
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6623E", params);
    }
    return true; // Boolean
  },

  update : function(layout)
  {
    // summary:
    //		Updates parameters that are taken from the layout instance.
    // tags:
    //		private
    
    this._layout = layout;
    var shortLinkLayout = layout.getShortLinkLayout();
    this._nodeSideFilter = shortLinkLayout.getNodeSideFilter();
;
    this._minFinalSegmentLength = shortLinkLayout.getMinFinalSegmentLength();
    this._bypassDistance = shortLinkLayout.getBypassDistance();
    if (this._bypassDistance < 0) {
      this._bypassDistance = this._minFinalSegmentLength;
    }
    this.setGlobalSelfLinkStyle(shortLinkLayout.getGlobalSelfLinkStyle());
  },

  getBypassDistance : function(linkData, globalBypassDistance)
  {
    // summary:
    //		Computes the bypass distance to be used for a particular link. 
    // description:
    //		Computes the bypass distance to be used for a particular link. 
    //		The default implementation adds the link width to the global bypass
    //		distance. Overridden in DirectLinkShapeProducer because for direct
    //		links we get better results without adding the link width. Indeed,
    //		an increased (sometimes overestimated) bypass dist means an
    //		increased risk to get orthogonal shapes (which are used in direct
    //		link style for overlapping nodes), which are not pleasant in this
    //		style because of the mix of direct and orthogonal shapes.
    // tags:
    //		protected
    
    return globalBypassDistance + linkData.getLinkWidth(); // Number
  },

  setGlobalSelfLinkStyle : function(selfLinkStyle)
  {
    // summary:
    // tags:
    //		private
    
    if (selfLinkStyle == LinkStyles.TWO_BENDS_ORTHOGONAL_STYLE) {
      this._selfLinkTypes = _LinkShapeProducer.TWO_BENDS_SELF_LINK_TYPES;
    }
    else if (selfLinkStyle == LinkStyles.THREE_BENDS_ORTHOGONAL_STYLE) {
      this._selfLinkTypes = _LinkShapeProducer.THREE_BENDS_SELF_LINK_TYPES;
    }
    else
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6624E",  selfLinkStyle);
    }
  },

  clean : function()
  {
    // summary:
    //		Cleans the data stored by this class.
    // tags:
    //		private
    
    this._layout = null;
    this._link = null;
    this._fromNode = null;
    this._toNode = null;
    this._nodeSideFilter = null;
  },

  getAllowedLinkShapes : function(layout, linkData)
  {
    // summary:
    //		Returns an array of shape types and computes the link region. 
    // description:
    //		Returns an array of shape types and computes the link region. 
    //		The selection of shapes depends on the relative position of the
    //		nodes and eventually on various settings.  
    // layout: _ShortLinkAlgorithm
    //		the graph layout
    // linkData: _SLLinkData
    //		the link
    // returns:
    //		an array of LinkShapeType objects.
    // tags:
    //		private
    
    if (linkData == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6601E");
    }

    this._link = linkData.nodeOrLink;
    this._layout = layout;
    if (linkData.boundingBox == null) {
      linkData.boundingBox = new Rectangle();
    }
    var fromNodeData = linkData.getFromNode();
    var toNodeData = linkData.getToNode();

    this._fromNode = fromNodeData.nodeOrLink;
    this._toNode = toNodeData.nodeOrLink;

    var tempFromRect = layout.getTempRect1();
    var tempToRect = layout.getTempRect2();
    var rect = fromNodeData.getLinkConnectionRect();
    TranslateUtil.reshapeRect(tempFromRect, rect.x, rect.y, rect.width, rect.height);
    rect = toNodeData.getLinkConnectionRect();
    TranslateUtil.reshapeRect(tempToRect, rect.x, rect.y, rect.width, rect.height);
    var isFromPointFixed = linkData.isFromPointFixed();
    var isToPointFixed = linkData.isToPointFixed();
    var fromPoint = linkData.getConnectionPoint(true);
    var toPoint = linkData.getConnectionPoint(false);

    if (isFromPointFixed) {
      TranslateUtil.addPoint(tempFromRect, fromPoint .x, fromPoint .y);
    }
    if (isToPointFixed) {
      TranslateUtil.addPoint(tempToRect, toPoint .x, toPoint .y);
    }
    var isSelfLink = this._fromNode == this._toNode;

    var isOverlappingNodes = isSelfLink;

    var fixedFromSide = (linkData.isFromSideFixed() ? linkData.getSide(true) : ((isFromPointFixed ? _SLNodeData.getSideOfPoint(tempFromRect, fromPoint) : _LinkShapeProducer.NOT_FIXED_SIDE)));

    var fixedToSide = (linkData.isToSideFixed() ? linkData.getSide(false) : ((isToPointFixed ? _SLNodeData.getSideOfPoint(tempToRect, toPoint) : _LinkShapeProducer.NOT_FIXED_SIDE)));

    var shapeTypes = null;

    var bypassDistance = this.getBypassDistance(linkData, this._bypassDistance);

    if (isSelfLink) {
      shapeTypes = this.getSelfLinkShapeTypes(linkData);
    }
    else {
      shapeTypes = this.getShapeTypesFromRelativePositionOfNodes(linkData, tempFromRect, tempToRect, bypassDistance);
    }
    if (linkData.isOverlapping()) {
      isOverlappingNodes = true;
    }

    var filteredShapeTypes = this.computeFilteredShapeTypes(linkData, shapeTypes, tempFromRect, tempToRect, fixedFromSide, fixedToSide, isOverlappingNodes, bypassDistance);

    if (!isOverlappingNodes) {
      filteredShapeTypes = this.sortShapeTypes(filteredShapeTypes, linkData, this._minFinalSegmentLength, filteredShapeTypes == shapeTypes);
    }
    return filteredShapeTypes; // Array
  },

  getShapeTypesFromRelativePositionOfNodes : function(linkData, fromRect, toRect, bypassDistance)
  {
    // summary:
    //		Computes a set of shapes which is the most appropriate for the
    //		particular relative positions of the end nodes.  
    // linkData: _SLLinkData
    //		The link.
    // fromRect: Rectangle
    //		The link connection rectangle of the origin node,
    //		enlarged with the fixed connection point if appropriate.
    // toRect: Rectangle
    //		The link connection rectangle of the destination node,
    //		enlarged with the fixed connection point if appropriate.
    // bypassDistance: Number
    //		
    // returns:
    //		The set of shapes.
    // tags:
    //		private
    
    var shapeTypes = null;

    if (linkData.isQuasiSelfInterGraphLinkOrigin() || linkData.isQuasiSelfInterGraphLinkDestination())
    {
      shapeTypes = this.getQuasiSelfInterGraphLinkShapeTypes(linkData, fromRect, toRect, bypassDistance);
    }
    else
    {
      if (toRect.x > fromRect.x + fromRect.width + bypassDistance)
      {
        if (toRect.y + toRect.height + bypassDistance < fromRect.y) {
          shapeTypes = this.getFromBottomLeftToTopRightPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
        else if (toRect.y > fromRect.y + fromRect.height + bypassDistance) {
          shapeTypes = this.getFromTopLeftToBottomRightPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
        else {
          shapeTypes = this.getFromLeftToRightPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
      }
      else if (toRect.x + toRect.width + bypassDistance < fromRect.x)
      {
        if (toRect.y + toRect.height + bypassDistance < fromRect.y) {
          shapeTypes = this.getFromBottomRightToTopLeftPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
        else if (toRect.y > fromRect.y + fromRect.height + bypassDistance) {
          shapeTypes = this.getFromTopRightToBottomLeftPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
        else {
          shapeTypes = this.getFromRightToLeftPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
      }
      else
      {
        if (toRect.y + toRect.height + bypassDistance < fromRect.y) {
          shapeTypes = this.getFromBottomToTopPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
        else if (toRect.y > fromRect.y + fromRect.height + bypassDistance) {
          shapeTypes = this.getFromTopToBottomPositionTypes(linkData, fromRect, toRect, bypassDistance);
        }
        else
        {
          shapeTypes = this.getOverlapPositionTypes(fromRect, toRect, bypassDistance);
          linkData.setOverlapping(true);
        }
      }
    }

    return shapeTypes; // Array
  },

  sortShapeTypes : function(shapeTypes, linkData, minFinalSegmentLength, copyBeforeSort)
  {
    // summary:
    //		Sorts the shape types according to some quality criteria. 
    // description:
    //		Sorts the shape types according to some quality criteria. 
    //		The default implementation does nothing. Indeed, for orthogonal
    //		style, the criteria is the number of bends, and the orthogonal link
    //		shape producer already sorts the shapes on this criteria.
    //		There's no need for an additional sort. Well, except when side constraints apply... So the method is actually overridden only in the direct style shape producer.  
    // copyBeforeSort: Boolean
    //		If true, a clone of the array of shapes must be
    //		done before sorting, to avoid to modify the original. We pass true
    //		when the array of shape types have been obtained by assigning one
    //		of the "static final" arrays, and we pass false when we created a
    //		new array because of filtering needs.
    // tags:
    //		protected
    
    return shapeTypes; // Array
  },

  computeFilteredShapeTypes : function(linkData, shapeTypes, fromRect, toRect, fixedFromSide, fixedToSide, isOverlappingNodes, bypassDistance)
  {
    // summary:
    //		Filters an array of LinkShapeType objects. 
    // description:
    //		Filters an array of LinkShapeType objects. 
    //		If no filter is specified (either via an NodeSideFilter, or via
    //		fixed connection points), the original array is returned.
    // tags:
    //		private
    
    if (fixedFromSide == _LinkShapeProducer.NOT_FIXED_SIDE && fixedToSide == _LinkShapeProducer.NOT_FIXED_SIDE && this._nodeSideFilter == null) {
      return shapeTypes; // Array
    }
    this.clearFilteredShapes();
    this.addShapeIfAcceptedByFilter(shapeTypes, linkData, fixedFromSide, fixedToSide, true, true);

    if (this._counter == shapeTypes.length && this._counter > 0) {
      return shapeTypes; // Array
    }
    var filteredShapeTypes = this.getCompactedFilteredShapes();

    if (filteredShapeTypes == null)
    {
      filteredShapeTypes = this.computeAllowedLinkShapes(linkData, fromRect, toRect, fixedFromSide, fixedToSide, isOverlappingNodes, bypassDistance);

      if (filteredShapeTypes == null)
      {
        LogResUtil.logAndThrowRuntimeExc(" ShortLinkLayout ", "graphlayout.expert.message.6625E");
      }
    }

    return filteredShapeTypes; // Array
  },

  computeAllowedLinkShapes : function(linkData, fromRect, toRect, fixedFromSide, fixedToSide, isOverlappingNodes, bypassDistance)
  {
    // summary:
    //		Computes the link shape types that are allowed when either one, or
    //		both connection points are on fixed sides. 
    // description:
    //		Computes the link shape types that are allowed when either one, or
    //		both connection points are on fixed sides. 
    //		Notice that when at least one connection point is not fixed, we
    //		first try to get shape types depending on the relative position of
    //		the nodes. When a node side filter exists, we then apply the filter.
    //		If none of the shapes is accepted by the filter, this method is used
    //		to construct a set of shapes according to the node side
    //		specification.
    // tags:
    //		private
    
    this.clearFilteredShapes();

    if (isOverlappingNodes)
    {
      this.addShapeIfAcceptedByFilter(_LinkShapeProducer.ADDITIONAL_OVERLAPPING_NODES_TYPES, linkData, fixedFromSide, fixedToSide, true, true);
    }
    else if (linkData.isQuasiSelfInterGraphLinkOrigin() || linkData.isQuasiSelfInterGraphLinkDestination())
    {
      this.addShapeIfAcceptedByFilter(this.getAllQuasiSelfInterGraphLinkShapeTypes(), linkData, fixedFromSide, fixedToSide, true, true);
    }

    var msg;

    if (this._counter == 0)
    {
      if (fixedFromSide != _LinkShapeProducer.NOT_FIXED_SIDE && fixedToSide != _LinkShapeProducer.NOT_FIXED_SIDE)
      {
        switch (fixedFromSide)
        {
          case Direction.TOP:
            switch (fixedToSide)
            {
              case Direction.TOP:
                this.addShapeIfAcceptedByFilter(this.getTopToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.BOTTOM:
                this.addShapeIfAcceptedByFilter(this.getTopToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.LEFT:
                this.addShapeIfAcceptedByFilter(this.getTopToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.RIGHT:
                this.addShapeIfAcceptedByFilter(this.getTopToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              default:
                
                LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6626E",  fixedToSide);
            }
            break;
          case Direction.BOTTOM:
            switch (fixedToSide)
            {
              case Direction.TOP:
                this.addShapeIfAcceptedByFilter(this.getBottomToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.BOTTOM:
                this.addShapeIfAcceptedByFilter(this.getBottomToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.LEFT:
                this.addShapeIfAcceptedByFilter(this.getBottomToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.RIGHT:
                this.addShapeIfAcceptedByFilter(this.getBottomToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              default:
                
                LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6626E",  fixedToSide);
            }
            break;
          case Direction.LEFT:
            switch (fixedToSide)
            {
              case Direction.TOP:
                this.addShapeIfAcceptedByFilter(this.getLeftToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.BOTTOM:
                this.addShapeIfAcceptedByFilter(this.getLeftToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.LEFT:
                this.addShapeIfAcceptedByFilter(this.getLeftToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.RIGHT:
                this.addShapeIfAcceptedByFilter(this.getLeftToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              default:
                
                LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6626E",  fixedToSide);
            }
            break;
          case Direction.RIGHT:
            switch (fixedToSide)
            {
              case Direction.TOP:
                this.addShapeIfAcceptedByFilter(this.getRightToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.BOTTOM:
                this.addShapeIfAcceptedByFilter(this.getRightToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.LEFT:
                this.addShapeIfAcceptedByFilter(this.getRightToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              case Direction.RIGHT:
                this.addShapeIfAcceptedByFilter(this.getRightToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, false);
                break;
              default:
                
                LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6626E",  fixedToSide);
            }
            break;
          default:
            LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6627E",  fixedFromSide);
        }
      }
      else if (fixedFromSide != _LinkShapeProducer.NOT_FIXED_SIDE)
      {
        switch (fixedFromSide)
        {
          case Direction.TOP:
            this.addShapeIfAcceptedByFilter(this.getTopToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getTopToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getTopToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getTopToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            break;
          case Direction.BOTTOM:
            this.addShapeIfAcceptedByFilter(this.getBottomToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getBottomToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getBottomToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getBottomToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            break;
          case Direction.LEFT:
            this.addShapeIfAcceptedByFilter(this.getLeftToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getLeftToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getLeftToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getLeftToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            break;
          case Direction.RIGHT:
            this.addShapeIfAcceptedByFilter(this.getRightToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getRightToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getRightToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            this.addShapeIfAcceptedByFilter(this.getRightToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, false, true);
            break;
          default:
            LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6628E",  fixedFromSide);
        }
      }
      else if (fixedToSide != _LinkShapeProducer.NOT_FIXED_SIDE)
      {
        switch (fixedToSide)
        {
          case Direction.TOP:
            this.addShapeIfAcceptedByFilter(this.getTopToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getBottomToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getLeftToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getRightToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            break;
          case Direction.BOTTOM:
            this.addShapeIfAcceptedByFilter(this.getTopToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getBottomToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getLeftToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getRightToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            break;
          case Direction.LEFT:
            this.addShapeIfAcceptedByFilter(this.getTopToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getBottomToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getLeftToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getRightToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            break;
          case Direction.RIGHT:
            this.addShapeIfAcceptedByFilter(this.getTopToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getBottomToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getLeftToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            this.addShapeIfAcceptedByFilter(this.getRightToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, false);
            break;
          default:
            LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6629E",  fixedToSide);
        }
      }
      else
      {
        this.addShapeIfAcceptedByFilter(this.getTopToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getTopToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getTopToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getTopToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getBottomToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getBottomToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getBottomToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getBottomToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getLeftToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getLeftToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getLeftToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getLeftToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getRightToTopSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getRightToBottomSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getRightToLeftSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
        this.addShapeIfAcceptedByFilter(this.getRightToRightSideTypes(fromRect, toRect, isOverlappingNodes, bypassDistance), linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, true, true);
      }
    }

    return this.getCompactedFilteredShapes(); // Array
  },

  clearFilteredShapes : function()
  {
    // summary:
    //		Reinitializes the counter of filtered shapes.
    // tags:
    //		private
    
    this._counter = 0;
  },

  getCompactedFilteredShapes : function()
  {
    // summary:
    //		Collects the shapes stored in the temporary array of filtered shapes
    //		and returns a new array having the length equal to the number of
    //		shapes stored in the temporary array of filtered shapes. 
    // description:
    //		Collects the shapes stored in the temporary array of filtered shapes
    //		and returns a new array having the length equal to the number of
    //		shapes stored in the temporary array of filtered shapes. 
    //		Returns null if no shape was stored.
    // tags:
    //		private
    
    return (this._counter >= 1 ? this.copyShapeTypes(this._shapeTypes, this._counter) : null); // Array
  },

  copyShapeTypes : function(oldShapeTypes, counter)
  {
    // summary:
    //		"Clones" the array of shape types.
    // tags:
    //		protected
    
    var newShapeTypes = TranslateUtil.newArray(counter, null);
    TranslateUtil.arrayCopy(oldShapeTypes, 0, newShapeTypes, 0, counter);
    return newShapeTypes; // Array
  },

  addShapeIfAcceptedByFilter : function (a0, a1, a2, a3, a4, a5)
  {
    // summary:
    //		This version only uses the node side filter, not the fixed origin or
    //		destination connection points.  
    // shapes: Array
    //		The shapes
    // checkOrigin: Boolean
    //		If true, the filter is used for the origin point
    // checkDestination: Boolean
    //		If true, the filter is used for the destination
    //		point
    // tags:
    //		private
    
    if(arguments.length == 4)
      return this.addShapeIfAcceptedByFilter_overload_4(a0, a1, a2, a3);
    else if(arguments.length == 6)
      return this.addShapeIfAcceptedByFilter_overload_6(a0, a1, a2, a3, a4, a5);
  },
  
  addShapeIfAcceptedByFilter_overload_4 : function(shapes, linkData, checkOrigin, checkDestination)
                                                                                                  
  {
    this.addShapeIfAcceptedByFilter(shapes, linkData, _LinkShapeProducer.NOT_FIXED_SIDE, _LinkShapeProducer.NOT_FIXED_SIDE, checkOrigin, checkDestination);
  },

  addShapeIfAcceptedByFilter_overload_6 : function(shapes, linkData, fixedFromSide, fixedToSide, checkOrigin, checkDestination)
  {
    // summary:
    //		This method is used when adding an array of shapes for a link which
    //		is constrained to go from one given (fixed) side of a node to some
    //		other sides.  
    // shapes: Array
    //		The shapes
    // fixedFromSide: int
    //		The fixed side on the origin node (can be
    //		NOT_FIXED_SIDE)
    // fixedToSide: int
    //		The fixed side on the dest node (can be
    //		NOT_FIXED_SIDE)
    // checkOrigin: Boolean
    //		If true, the filter is used for the origin point
    // checkDestination: Boolean
    //		If true, the filter is used for the destination
    //		point
    // tags:
    //		private
    
    var shapeType;
    var isAllowed;

    var oppositeSide;

    for (var i = 0; i < shapes.length; i++)
    {
      shapeType = shapes[i];
      isAllowed = true;

      if (checkOrigin)
      {
        oppositeSide = shapeType.getFromSide(linkData);

        if (fixedFromSide == _LinkShapeProducer.NOT_FIXED_SIDE)
        {
          if (!this.isAllowedByFilter(oppositeSide, true)) {
            isAllowed = false;
          }
        }
        else if (oppositeSide != fixedFromSide) {
          isAllowed = false;
        }
      }

      if (isAllowed)
      {
        if (checkDestination)
        {
          oppositeSide = shapeType.getToSide(linkData);

          if (fixedToSide == _LinkShapeProducer.NOT_FIXED_SIDE)
          {
            if (!this.isAllowedByFilter(oppositeSide, false)) {
              isAllowed = false;
            }
          }
          else if (oppositeSide != fixedToSide) {
            isAllowed = false;
          }
        }

        if (isAllowed) {
          this._shapeTypes[this._counter++] = shapeType;
        }
      }
    }
  },

  isAllowedByFilter : function(side, origin)
  {
    // summary:
    //		Checks whether a link shape type going from a given from side to a
    //		given to side is allowed by the node side filter.
    
    if (this._nodeSideFilter == null) {
      return true; // Boolean
    }
    var graphModel;
    var link = this._link;
    var node;
    graphModel = this._layout.getGraphModel();
    node = (origin ? this._fromNode : this._toNode);

    if (graphModel instanceof SubgraphData)
    {
      var intergraphModel = graphModel;
      var origLink = intergraphModel.getOriginal(link);
      var origNode = intergraphModel.getOriginal(node);

      if (origLink != null && origNode != null)
      {
        graphModel = intergraphModel.getOriginalGraphModel(node);
        link = origLink;
        node = origNode;
      }
    }

    return this._nodeSideFilter.accept(graphModel, link, origin, node, side); // Boolean
  }

}
);

lang.mixin(_LinkShapeProducer,
{
  NOT_FIXED_SIDE : null,

  TWO_BENDS_SELF_LINK_TYPES : null,
  THREE_BENDS_SELF_LINK_TYPES : null,

  OVERLAPPING_NODES_TYPES : null,

  ADDITIONAL_OVERLAPPING_NODES_TYPES : null,
  _fields_init : function(c)
  {
    c = c || _LinkShapeProducer;
    
    c.NOT_FIXED_SIDE = Direction.NONE;
    c.TWO_BENDS_SELF_LINK_TYPES = 
    [
      [_LinkShapeType.TwoBendTT.instance, _LinkShapeType.TwoBendBB.instance, _LinkShapeType.TwoBendLL.instance, _LinkShapeType.TwoBendRR.instance, ],
      [_LinkShapeType.TwoBendBB.instance, _LinkShapeType.TwoBendLL.instance, _LinkShapeType.TwoBendRR.instance, _LinkShapeType.TwoBendTT.instance, ],
      [_LinkShapeType.TwoBendLL.instance, _LinkShapeType.TwoBendRR.instance, _LinkShapeType.TwoBendTT.instance, _LinkShapeType.TwoBendBB.instance, ],
      [_LinkShapeType.TwoBendRR.instance, _LinkShapeType.TwoBendTT.instance, _LinkShapeType.TwoBendBB.instance, _LinkShapeType.TwoBendLL.instance, ]
     ];
    c.THREE_BENDS_SELF_LINK_TYPES = 
    [
      [_LinkShapeType.ThreeBendTR.instance, _LinkShapeType.ThreeBendRB.instance, _LinkShapeType.ThreeBendBL.instance, _LinkShapeType.ThreeBendLT.instance, ],
      [_LinkShapeType.ThreeBendRB.instance, _LinkShapeType.ThreeBendBL.instance, _LinkShapeType.ThreeBendLT.instance, _LinkShapeType.ThreeBendTR.instance, ],
      [_LinkShapeType.ThreeBendBL.instance, _LinkShapeType.ThreeBendLT.instance, _LinkShapeType.ThreeBendTR.instance, _LinkShapeType.ThreeBendRB.instance, ],
      [_LinkShapeType.ThreeBendLT.instance, _LinkShapeType.ThreeBendTR.instance, _LinkShapeType.ThreeBendRB.instance, _LinkShapeType.ThreeBendBL.instance, ]
     ];
    c.OVERLAPPING_NODES_TYPES = 
    [
      _LinkShapeType.ThreeBendRT_OVERLAP.instance,
      _LinkShapeType.ThreeBendTR_OVERLAP.instance,
      _LinkShapeType.ThreeBendBL_OVERLAP.instance,
      _LinkShapeType.ThreeBendLB_OVERLAP.instance,
      _LinkShapeType.ThreeBendLT_OVERLAP.instance,
      _LinkShapeType.ThreeBendTL_OVERLAP.instance,
      _LinkShapeType.ThreeBendBR_OVERLAP.instance,
      _LinkShapeType.ThreeBendRB_OVERLAP.instance
     ];
    c.ADDITIONAL_OVERLAPPING_NODES_TYPES = 
    [
      _LinkShapeType.FourBendRLtop.instance,
      _LinkShapeType.FourBendRLbottom.instance,
      _LinkShapeType.FourBendBTleft.instance,
      _LinkShapeType.FourBendBTright.instance,
      _LinkShapeType.FourBendLRtop.instance,
      _LinkShapeType.FourBendLRbottom.instance,
      _LinkShapeType.FourBendTBleft.instance,
      _LinkShapeType.FourBendTBright.instance
     ];
  }
  
}
);

(function(){ _LinkShapeProducer._fields_init(); })();

return _LinkShapeProducer;
});

},
'ibm_ilog/graphlayout/shortlink/_CombinatorialOptimization':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_CombinatorialOptimization", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    TranslateUtil	// #7
    ) {

var _CombinatorialOptimization = declare("ibm_ilog.graphlayout.shortlink._CombinatorialOptimization", null, 
{
  // summary:
  //		A generic interface for combinatorial optimization algorithms. 
  // description:
  //		A generic interface for combinatorial optimization algorithms. 
  //		
  // tags:
  //		extension
  
  _variables: null,

  _variants: null,

  constructor : function(variables)
  {
    if (variables == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6600E");
    }

    this._variables = variables;
    this._variants = TranslateUtil.newArray(this._variables.length, 0);
  },
  
  optimize : function()
  {
    // summary:
    //		Performs the combinatorial optimization.
    },
  
  setVariant : function(indexVariable, variant)
  {
    // summary:
    //		Called from time to time when a step of the optimization has been
    //		performed. 
    // description:
    //		Called from time to time when a step of the optimization has been
    //		performed. 
    //		The default implementation does nothing. You can  override this
    //		method in order to notify your layout algorithm (see
    //		GraphLayout.layoutStepPerformed()).
    // tags:
    //		protected
    
    this._variants[indexVariable] = variant;
  },

  applyVariant : function(variable, variableIndex, variant)
  {
    // summary:
    //		Applies a variant.  
    // returns:
    //		true if the variant can be applied, false otherwise
    // tags:
    //		protected
    },
  
  getVariant : function(indexVariable)
  {
    return this._variants[indexVariable]; // int
  },

  getVariantsCount : function(variable)
  {},
  
  getCost : function(variable)
  {
    // summary:
    //		Returns the cost associated to a variable.
    // tags:
    //		protected
    },
  
  isBetterVariant : function(variable, variant1, cost1, variant2, cost2)
  {
    // summary:
    //		Returns true if the variant1 with cost1 is preferred to variant2
    //		with cost2 for variable. 
    // description:
    //		Returns true if the variant1 with cost1 is preferred to variant2
    //		with cost2 for variable. 
    //		Returns false otherwise. The default implementation returns true if
    //		cost1 < cost2 (strictly).
    // tags:
    //		protected
    
    return cost1 < cost2; // Boolean
  },

  isOptimalCost : function(cost)
  {
    // summary:
    //		Returns true if cost is the minimum cost.
    // tags:
    //		protected
    
    return cost <= 0; // Boolean
  }

}
);

return _CombinatorialOptimization;
});

},
'ibm_ilog/graphlayout/longlink/LongLinkLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/longlink/LongLinkLayout", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/ConnectionPointModes",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/LayoutParametersUtil",	// #7
  "../internalutil/LinkStyles",	// #8
  "../internalutil/LogResUtil",	// #9
  "../internalutil/Point",	// #10
  "../internalutil/SplineRouter",	// #11
  "../internalutil/SubgraphData",	// #12
  "../internalutil/TranslateUtil",	// #13
  "../longlink/LLGraph",	// #14
  "../longlink/LLIncrementalData",	// #15
  "../longlink/LongLinkLayoutReport",	// #16
  "../GraphLayout",	// #17
  "../GraphLayoutReport"	// #18
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    ConnectionPointModes,	// #5
    GraphModelUtil,	// #6
    LayoutParametersUtil,	// #7
    LinkStyles,	// #8
    LogResUtil,	// #9
    Point,	// #10
    SplineRouter,	// #11
    SubgraphData,	// #12
    TranslateUtil,	// #13
    LLGraph,	// #14
    LLIncrementalData,	// #15
    LongLinkLayoutReport,	// #16
    GraphLayout,	// #17
    GraphLayoutReport	// #18
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var GraphLayout = ibm_ilog.graphlayout.GraphLayout;
=====*/

var LongLinkLayout = declare("ibm_ilog.graphlayout.longlink.LongLinkLayout", GraphLayout, 
{
  // summary:
  //		The main class for the Long Link Layout algorithm. 
  // description:
  //		The main class for the Long Link Layout algorithm. 
  //		<p>
  //		The Long Link Layout algorithm reshapes the links of a graph without
  //		moving the nodes. It is specialized to route long orthogonal links on
  //		a grid without overlapping nodes. It uses a classical path search
  //		mechanism and an additional crossing reduction phase. It can be used
  //		to find an orthogonal route without overlaps in a labyrinth of node
  //		obstacles.  
  //		</p>
  //		<p>
  //		Optionally, direct links can be computed, which, however, may overlap
  //		nodes. See the method <code>
  //		LongLinkLayout.setGlobalLinkStyle()</code>.  
  //		</p>
  //		<p>
  //		This layout algorithm is recommended if any of the following
  //		conditions apply:  
  //		<ul>
  //		<li> Many links are long, and it is important that long orthogonal
  //		links do not overlap obstacles.</li>  
  //		<li> Routing on a grid is preferred because it guarantees a minimum
  //		distance between link segments.</li>  
  //		<li> An increasing number of bends is acceptable if this avoids any
  //		overlaps.</li>  
  //		</ul>
  //		Otherwise, the <code>
  //		ibm&#95;ilog.graphlayout.shortlink.ShortLinkLayout</code> may be more
  //		suitable.  
  //		</p>
  //		<p>
  //		Here are sample drawings produced by the Long Link Layout:
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Link-long2times.gif" alt="Sample drawing produced with the Long Link Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		The long link mode can be used to find a route through a labyrinth:  
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Link-labyrinth.gif" alt="Sample drawing produced with the Long Link Layout" border="0" align="bottom"/>
  //		 
  //		</p>
  //		<p>
  //		Note the following points:  
  //		<ul>
  //		<li> The algorithm never moves any nodes. It only reshapes
  //		links.</li>  
  //		<li> The algorithm normally routes normal links. It has a special
  //		mode to route the intergraph links of the graph model.</li>  
  //		<li> The algorithm has an incremental mode and a nonincremental mode.
  //		In nonincremental mode, all links are routed. In incremental mode,
  //		only those links are routed that overlap nodes or whose end nodes
  //		have changed. The incremental mode cannot be used for intergraph
  //		links.</li>  
  //		<li> The algorithm supports self-links and multiple links between the
  //		same pair of nodes.</li>  
  //		</ul>  
  //		</p>
  /*=====
  // ORTHOGONAL_STYLE: int
  //		Orthogonal link shape option. 
  //		When used as the argument of the method  '
  //		LongLinkLayout.setGlobalLinkStyle()', all links get an
  //		orthogonal shape (alternating horizontal and vertical segments).  
  //		
  //		To specify the shape of an individual link, use this option as the
  //		argument of the method ' LongLinkLayout.setLinkStyle()'.  
  //		
  ORTHOGONAL_STYLE : null,
  // DIRECT_STYLE: int
  //		Direct link style option. 
  //		When used as the argument of the method  '
  //		LongLinkLayout.setGlobalLinkStyle()', all links get a
  //		(semi)direct shape.  
  //		
  //		This link shape may cause overlaps between links and nodes. To
  //		specify the style of an individual link, use this option as the
  //		argument of the method ' LongLinkLayout.setLinkStyle()'.  
  //		
  DIRECT_STYLE : null,
  // MIXED_STYLE: int
  //		Mixed link shape option. 
  //		When used as the argument of the method  '
  //		LongLinkLayout.setGlobalLinkStyle()', each link can have a
  //		different shape. The shape of an individual link can be set by the
  //		method ' LongLinkLayout.setLinkStyle()'.
  MIXED_STYLE : null,
  // NO_RESHAPE_STYLE: int
  //		No reshape link style option. 
  //		This can be used for individual links if the global link style is
  //		' LongLinkLayout.MIXED_STYLE'. To specify the style of an
  //		individual link, use this option as the argument of the method '
  //		LongLinkLayout.setLinkStyle()'.
  NO_RESHAPE_STYLE : null,
  // FREE_MODE: int
  //		Free connection point mode. 
  //		When used as the argument of the method  '
  //		LongLinkLayout.setGlobalOriginPointMode()' or '
  //		LongLinkLayout.setGlobalDestinationPointMode()', the layout is
  //		free to choose the appropriate position of the connection points,
  //		except for "pinned" connection points (see '
  //		ibm_ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()').
  //		To specify the mode for the connection points of an individual link,
  //		use this mode as the argument of the method '
  //		LongLinkLayout.setOriginPointMode()' or '
  //		LongLinkLayout.setDestinationPointMode()'.
  FREE_MODE : null,
  // FIXED_MODE: int
  //		Fixed connection point mode. 
  //		When used as the argument of the method  '
  //		LongLinkLayout.setGlobalOriginPointMode()' or '
  //		LongLinkLayout.setGlobalDestinationPointMode()', the layout
  //		must keep the current position of the connection point. To specify
  //		the mode for the connection points of an individual link, use this
  //		mode as the argument of the method '
  //		LongLinkLayout.setOriginPointMode()' or '
  //		LongLinkLayout.setDestinationPointMode()'.
  FIXED_MODE : null,
  // MIXED_MODE: int
  //		Mixed connection point mode. 
  //		When used as the argument of the method  '
  //		LongLinkLayout.setGlobalOriginPointMode()' or '
  //		LongLinkLayout.setGlobalDestinationPointMode()', the connection
  //		point of each link can have a different mode. The mode for the
  //		connection points of an individual link can be set by the method
  //		' LongLinkLayout.setOriginPointMode()' or '
  //		LongLinkLayout.setDestinationPointMode()'.
  MIXED_MODE : null,
  =====*/
  
  _globalLinkStyle: null,
  _gridXBase: null,
  _gridYBase: null,
  _gridXOffset: null,
  _gridYOffset: null,
  _minXOffset: null,
  _minYOffset: null,
  _minNodeCornerOffset: null,
  _minStartSegmentLength: null,
  _minEndSegmentLength: null,
  _numberCrossingReductionIterations: null,

  _allowedTimePerLink: null,

  _exhaustiveSearching: null,

  _crossingReduction: null,

  _straightRoute: null,

  _maxBackTrack: null,

  _originPointMode: null,

  _destinationPointMode: null,

  _nodeObstacleEnabled: null,

  _linkObstacleEnabled: null,

  _interGraphLinksMode: null,

  _combinedInterGraphLinksMode: null,

  _interGraphLinksRoutingModel: null,

  _rectObstacles: null,
  _lineObstacles: null,
  _lineFromObstacles: null,
  _lineToObstacles: null,

  _linksToBeRouted: null,

  _fallbackLinks: null,
  _fallbackRouteEnabled: null,

  _incrementalMode: null,
  _incrementalData: null,
  _incrementalConnectionPreserving: null,

  _nodeBoxInterfaceForObstacle: null,

  _nodeSideFilter: null,

  _terminationPointFilter: null,

  _allowStopImmediately: null,

  _stoppedBeforeCompletion: null,

  _numberOfSteps: null,
  _stepCount: null,

  constructor : function(source )
  {
  },
  
  init : function()
  {
    // summary:
    //		Initializes instance variables. 
    // description:
    //		Initializes instance variables. 
    //		<p>
    //		You should not call this method directly. The method is called
    //		internally by the constructor without arguments and by the copy
    //		constructor. The method must be overridden by subclasses that need
    //		to initialize additional instance variables.
    //		</p>
    // tags:
    //		protected
    
    this.inherited("init", arguments, []);
    this._globalLinkStyle = LongLinkLayout._defaultLinkStyle;
    this._originPointMode = LongLinkLayout._defaultOriginPointMode;
    this._destinationPointMode = LongLinkLayout._defaultDestinationPointMode;
    this._gridXBase = LongLinkLayout._defaultGridXBase;
    this._gridYBase = LongLinkLayout._defaultGridYBase;
    this._gridXOffset = LongLinkLayout._defaultGridXOffset;
    this._gridYOffset = LongLinkLayout._defaultGridYOffset;
    this._minXOffset = LongLinkLayout._defaultMinXOffset;
    this._minYOffset = LongLinkLayout._defaultMinYOffset;
    this._minNodeCornerOffset = LongLinkLayout._defaultMinNodeCornerOffset;
    this._minStartSegmentLength = LongLinkLayout._defaultMinStartEndSegmentLength;
    this._minEndSegmentLength = LongLinkLayout._defaultMinStartEndSegmentLength;
    this._numberCrossingReductionIterations = LongLinkLayout._defaultNumberCrossingReductionIterations;
    this._allowedTimePerLink = LongLinkLayout._defaultAllowedTimePerLink;
    this._exhaustiveSearching = LongLinkLayout._defaultExhaustiveSearching;
    this._crossingReduction = LongLinkLayout._defaultCrossingReduction;
    this._straightRoute = LongLinkLayout._defaultStraightRoute;
    this._maxBackTrack = LongLinkLayout._defaultMaxBacktrack;
    this._nodeObstacleEnabled = true;
    this._linkObstacleEnabled = true;
    this._interGraphLinksMode = LongLinkLayout._defaultInterGraphLinksMode;
    this._combinedInterGraphLinksMode = LongLinkLayout._defaultCombinedInterGraphLinksMode;
    this._rectObstacles = new Array();
    this._lineObstacles = new Array();
    this._lineFromObstacles = new Array();
    this._lineToObstacles = new Array();
    this._fallbackLinks = new Array();
    this._fallbackRouteEnabled = true;
    this._nodeBoxInterfaceForObstacle = LongLinkLayout._defaultNodeBoxInterface;
    this._nodeSideFilter = LongLinkLayout._defaultNodeSideFilter;
    this._terminationPointFilter = LongLinkLayout._defaultTerminationPointFilter;
    this._incrementalMode = false;
    this._incrementalConnectionPreserving = false;
    this._allowStopImmediately = true;
  },

  copy : function()
  {
    // summary:
    //		Copies the layout instance. 
    // description:
    //		Copies the layout instance. 
    //		<p>
    //		This method copies the layout instance by calling the copy
    //		constructor.  
    //		</p>
    //		<p>
    //		When performing a recursive layout on a nested graph, <code>
    //		ibm&#95;ilog.graphlayout.ILayoutProvider</code> uses this method to
    //		"clone" the layout instance of a parent graph. Note that the
    //		parameters which are specific to a node or a link are not copied.
    //		Only the global parameters are copied. The customization interfaces
    //		(node side filter, node box interface, termination point filter) are
    //		also copied.  
    //		</p>
    // returns:
    //		A copy of the layout instance.
    
    return new LongLinkLayout(this); // ibm_ilog.graphlayout.GraphLayout
  },

  copyParameters : function(source)
  {
    // summary:
    //		Copies the parameters from a given layout instance. 
    // description:
    //		Copies the parameters from a given layout instance. 
    //		<p>
    //		Note that the parameters which are specific to a node or a link are
    //		not copied. Only the global parameters are copied. The customization
    //		interfaces (node side filter, node box interface, termination point
    //		filter) are also copied.  
    //		</p>
    // source: GraphLayout
    //		The layout instance from which the parameters are copied.
    
    this.inherited("copyParameters", arguments, [source]);

    if (source instanceof LongLinkLayout)
    {
      var lsource = source;
      this.setGlobalLinkStyle(lsource.getGlobalLinkStyle());
      this.setGlobalOriginPointMode(lsource.getGlobalOriginPointMode());
      this.setGlobalDestinationPointMode(lsource.getGlobalDestinationPointMode());
      this.setHorizontalGridBase(lsource.getHorizontalGridBase());
      this.setVerticalGridBase(lsource.getVerticalGridBase());
      this.setHorizontalGridOffset(lsource.getHorizontalGridOffset());
      this.setVerticalGridOffset(lsource.getVerticalGridOffset());
      this.setHorizontalMinOffset(lsource.getHorizontalMinOffset());
      this.setVerticalMinOffset(lsource.getVerticalMinOffset());
      this.setMinNodeCornerOffset(lsource.getMinNodeCornerOffset());
      this.setMinStartSegmentLength(lsource.getMinStartSegmentLength());
      this.setMinEndSegmentLength(lsource.getMinEndSegmentLength());
      this.setNumberCrossingReductionIterations(lsource.getNumberCrossingReductionIterations());
      this.setAllowedTimePerLink(lsource.getAllowedTimePerLink());
      this.setExhaustiveSearching(lsource.isExhaustiveSearching());
      this.setCrossingReductionEnabled(lsource.isCrossingReductionEnabled());
      this.setStraightRouteEnabled(lsource.isStraightRouteEnabled());
      this.setMaxBacktrack(lsource.getMaxBacktrack());
      this.setNodeObstacleEnabled(lsource.isNodeObstacleEnabled());
      this.setLinkObstacleEnabled(lsource.isLinkObstacleEnabled());
      this.setInterGraphLinksMode(lsource.isInterGraphLinksMode());
      this.setCombinedInterGraphLinksMode(lsource.isCombinedInterGraphLinksMode());
      this.setFallbackRouteEnabled(lsource.isFallbackRouteEnabled());
      this.setIncrementalMode(lsource.isIncrementalMode());
      this.setIncrementalConnectionPreserving(lsource.isIncrementalConnectionPreserving());
      this.setNodeBoxInterface(lsource.getNodeBoxInterface());
      this.setNodeSideFilter(lsource.getNodeSideFilter());
      this.setTerminationPointFilter(lsource.getTerminationPointFilter());
    }
  },

  detach : function()
  {
    // summary:
    //		Detaches the graph model from the layout instance. 
    // description:
    //		Detaches the graph model from the layout instance. 
    //		When you attach a new graph model to the layout instance, you do not
    //		need to detach the old graph model because this is done
    //		automatically when you call <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.attach()</code>.  
    //		<p>
    //		The method performs cleaning operations on the graph model. In
    //		addition to the cleaning operations in the base class, the Long Link
    //		Layout removes the link style specifications of individual links.  
    //		</p>
    //		<p>
    //		Note that you must call this method when you no longer need the
    //		layout instance. Otherwise, some objects may not be garbage
    //		collected.  
    //		</p>
    
    if (this.getGraphModel() != null)
    {
      this._interGraphLinksRoutingModel = null;
      this._linksToBeRouted = null;
      this._fallbackLinks.length = 0;

      this.clearIncrementalData();
    }

    this.inherited("detach", arguments, []);
  },

  cleanNode : function(graphModel, node)
  {
    // summary:
    //		Cleans a node. 
    // description:
    //		Cleans a node. 
    //		This method removes any data that has been stored by the layout
    //		algorithm on a node.  
    //		<p>
    //		The method is automatically called by <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.contentsChanged()</code> whenever a
    //		<code> ibm&#95;ilog.graphlayout.GraphLayoutModelEvent</code> of type 
    //		<code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutModelEvent.NODE&#95;REMOVED</code> is
    //		received.  
    //		</p>
    //		<p>
    //		Subclasses can override this method to remove additional information
    //		stored in the nodes.  
    //		</p>
    //		<p>
    //		Notice that the method may be called multiple times for the same
    //		node. At the time this method is called, the node may already be
    //		removed; therefore, overridden versions of this method should be
    //		implemented so that they work even if the object is no longer a node
    //		of <code>graphModel</code>.  
    //		</p>
    // graphModel: AbstractGraphLayoutModel
    //		The graph model to which the 'node'
    //		belongs.
    // node: Object
    //		The node to be cleaned.
    
    this.inherited("cleanNode", arguments, [graphModel, node]);
    if (this._incrementalData != null) {
      this._incrementalData.cleanNode(node);
    }
  },

  cleanLink : function(graphModel, link)
  {
    // summary:
    //		Cleans a link. 
    // description:
    //		Cleans a link. 
    //		This method removes any data that has been stored by the layout
    //		algorithm on a link.  
    //		<p>
    //		The method is automatically called by <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.contentsChanged()</code> whenever a
    //		<code> ibm&#95;ilog.graphlayout.GraphLayoutModelEvent</code> of type 
    //		<code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutModelEvent.LINK&#95;REMOVED</code> is
    //		received.  
    //		</p>
    //		<p>
    //		Subclasses can override this method to remove additional information
    //		stored in the links.  
    //		</p>
    //		<p>
    //		Notice that the method may be called multiple times for the same
    //		link. At the time this method is called, the link may already be
    //		removed; therefore, overridden versions of this method should be
    //		implemented so that they work even if the object is no longer a link
    //		of <code>graphModel</code>.  
    //		</p>
    // graphModel: AbstractGraphLayoutModel
    //		The graph model to which the 'link'
    //		belongs.
    // link: Object
    //		The link to be cleaned.
    
    this.inherited("cleanLink", arguments, [graphModel, link]);
    if (this._incrementalData != null) {
      this._incrementalData.cleanLink(link);
    }
  },

  layout : function()
  {
    // summary:
    //		Computes the layout using the Long Link Layout algorithm. 
    // description:
    //		Computes the layout using the Long Link Layout algorithm. 
    //		To start the layout, call the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code> .   
    // tags:
    //		protected
    
    var redraw = false;

    if (this.getGlobalLinkStyle() == LongLinkLayout.NO_RESHAPE_STYLE) {
      return;
    }

    var originalGraphModel = this.getGraphModel();

    if (this.isInterGraphLinksMode())
    {
      if ((new TranslateUtil.ArrayIterator(originalGraphModel.getIntergraphLinks())).hasNext())
      {
        this._interGraphLinksRoutingModel = new SubgraphData();
        this._interGraphLinksRoutingModel.collectData(this.getGraphModel());
        this._interGraphLinksRoutingModel.setNormalLinksFixed(!this.isCombinedInterGraphLinksMode());
        this.setGraphModel(this._interGraphLinksRoutingModel);
      }
      else
      {
        if (!this.isCombinedInterGraphLinksMode())
        {
          this.getLayoutReport().setCode(GraphLayoutReport.LAYOUT_DONE);
          return;
        }
      }
    }

    try
    {
      this.layoutInternal(redraw);
    }
    finally
    {
      if (this._interGraphLinksRoutingModel != null) {
        this._interGraphLinksRoutingModel.clean();
      }
      this.setGraphModel(originalGraphModel);
      this._interGraphLinksRoutingModel = null;
    }
  },

  layoutInternal : function(redraw)
  {
    // summary:
    //		This does the real work, both for normal links and for intergraph
    //		links. 
    // description:
    //		This does the real work, both for normal links and for intergraph
    //		links. 
    //		
    // tags:
    //		private
    
    this._stoppedBeforeCompletion = false;
    this._allowStopImmediately = true;

    var graphModel = this.getGraphModel();

    var layoutReport = this.getLayoutReport();
    layoutReport.setNumberOfFallbackLinks(0);
    this._numberOfSteps = 1;
    this._stepCount = 0;
    if (this.isSplineRoutingEnabled()) {
      this._numberOfSteps += 2;
    }
    if (this._linksToBeRouted == null)
    {
      this._numberOfSteps += 13 * graphModel.getLinksCount();
      if (this.isStraightRouteEnabled()) {
        this._numberOfSteps += 2 * graphModel.getLinksCount();
      }
      if (this.isCrossingReductionEnabled()) {
        this._numberOfSteps += 2 * (graphModel.getLinksCount() - 1) * this.getNumberCrossingReductionIterations();
      }
    }
    else
    {
      this._numberOfSteps += 13 * this._linksToBeRouted.length;
      if (this.isStraightRouteEnabled()) {
        this._numberOfSteps += 2 * this._linksToBeRouted.length;
      }
      if (this.isCrossingReductionEnabled()) {
        this._numberOfSteps += 2 * (this._linksToBeRouted.length - 1) * this.getNumberCrossingReductionIterations();
      }
    }

    if (this.isIncrementalMode() && this._interGraphLinksRoutingModel == null)
    {
      this._numberOfSteps += graphModel.getLinksCount();
      this._numberOfSteps += 2 * graphModel.getNodesCount();
    }

    this._stoppedBeforeCompletion = false;
    this._fallbackLinks.length = 0;
    var graph = new LLGraph(this, redraw);

    this.callLayoutStepPerformedIfNeeded();

    graph.attach(graphModel, this);

    if (this.mayContinue())
    {
      if (this._linksToBeRouted == null) {
        graph.attachAllLinks();
      }
      else {
        graph.attachLinks(this._linksToBeRouted);
      }
    }

    if (this.mayContinue())
    {
      graph.createCompleteGrid();
    }

    if (this.mayContinue())
    {
      graph.prepareIncrementalLinks();
    }

    if (this.mayContinue())
    {
      graph.routeAttachedLinks();
    }

    if (this.isIncrementalMode() && this._interGraphLinksRoutingModel == null)
    {
      this.getIncrementalData().update(this);
    }

    this._allowStopImmediately = false;
    graph.detach();
    graph.dispose();

    var gbls = this.getGlobalLinkStyle();

    if (this.mayContinue() && this.isSplineRoutingEnabled() && gbls != LongLinkLayout.NO_RESHAPE_STYLE)
    {
      var router = new LongLinkLayoutAnonClass_1(this, null, 0, this.getMinSplineCurveSize(), this.getMaxSplineCurveSize(), this.getBalanceSplineCurveThreshold(), true, false, this);

      router.run(redraw, true);
    }

    this.increasePercentageComplete(100);
    this.onLayoutStepPerformed(false, false);

    if (this._stoppedBeforeCompletion) {
      layoutReport.setCode(GraphLayoutReport.STOPPED_AND_INVALID);
    }
    else {
      layoutReport.setCode(GraphLayoutReport.LAYOUT_DONE);
    }
    this._allowStopImmediately = true;
  },

  createLayoutReport : function()
  {
    // summary:
    //		Creates the graph layout report instance.  
    // returns:
    //		A new instance of layout report. The current implementation
    //		creates an instance of ' LongLinkLayoutReport'.
    
    return new LongLinkLayoutReport(); // ibm_ilog.graphlayout.GraphLayoutReport
  },

  supportsPreserveFixedLinks : function()
  {
    // summary:
    //		Indicates that this layout class allows the user to specify fixed
    //		links. 
    // description:
    //		Indicates that this layout class allows the user to specify fixed
    //		links. 
    //		<p>
    //		Fixed links are not reshaped during the layout if the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setPreserveFixedLinks()</code> is
    //		called with a <code>true</code> argument.  The effect is very
    //		similar to the effect when the links are filtered during layout.   
    //		</p>
    
    return true; // Boolean
  },

  supportsAllowedTime : function()
  {
    // summary:
    //		Indicates that this layout class can stop the layout computation in
    //		a proper manner when the user-defined allowed time is exceeded. 
    // description:
    //		Indicates that this layout class can stop the layout computation in
    //		a proper manner when the user-defined allowed time is exceeded. 
    //		If the allowed time elapses before termination of the Long Link
    //		Layout algorithm, the links are not reshaped and remain at the same
    //		position as before the start of the algorithm.  
    //		<p>
    //		The result code in the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code> in
    //		this case.  
    //		</p>
    
    return true; // Boolean
  },

  supportsStopImmediately : function()
  {
    // summary:
    //		Indicates that this layout class can interrupt the current run of
    //		the layout immediately in a controlled way. 
    // description:
    //		Indicates that this layout class can interrupt the current run of
    //		the layout immediately in a controlled way. 
    //		If interrupted, the links are not reshaped and remain at the same
    //		position as before the start of the algorithm.  
    //		<p>
    //		The result code in the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code> in
    //		this case.  
    //		</p>
    
    return true; // Boolean
  },

  stopImmediately : function()
  {
    // summary:
    //		Stops the running of the layout algorithm as soon as possible. 
    // description:
    //		Stops the running of the layout algorithm as soon as possible. 
    //		<p>
    //		The Long Link Layout algorithm has several steps. In the first
    //		steps, the layout algorithm can be stopped and the method returns
    //		<code>true</code>. In the very last step,  the layout algorithm
    //		cannot be stopped and the method returns <code>false</code>.  
    //		</p>
    // returns:
    //		'true' if the layout algorithm can be stopped.
    
    if (this.isStoppedImmediately()) {
      return true; // Boolean
    }
    if (!this._allowStopImmediately) {
      return false; // Boolean
    }
    return this.inherited("stopImmediately", arguments, []); // Boolean
  },

  supportsPercentageComplete : function()
  {
    // summary:
    //		Indicates that this layout class can estimate the percentage of
    //		completion during the run of layout.
    
    return true; // Boolean
  },

  supportsLinkConnectionBox : function()
  {
    // summary:
    //		Indicates that this layout class can use a link connection box
    //		interface to calculate the end points of a link. 
    // description:
    //		Indicates that this layout class can use a link connection box
    //		interface to calculate the end points of a link. 
    //		The link connection box interface is an object that provides the
    //		rectangle to which the links are connected for each node.  
    //		<p>
    //		The link connection box is used instead of the bounding box of each
    //		node (see   ) during the routing of links. The tangential offsets
    //		provided by the link connection box interface are currently ignored.
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsSaveParametersToNamedProperties : function()
  {
    // summary:
    //		Indicates that this layout class can transfer the layout parameters
    //		to named properties. 
    // description:
    //		Indicates that this layout class can transfer the layout parameters
    //		to named properties. 
    //		This mechanism can be used if layout parameters must be stored
    //		persistently in an <code>.ivl</code> file.  
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsSplineRouting : function()
  {
    // summary:
    //		Tests if this class supports the generic optimization of spline
    //		control points. 
    // description:
    //		Tests if this class supports the generic optimization of spline
    //		control points. 
    //		
    // returns:
    //		This implementation always returns 'true'.
    
    return true; // Boolean
  },

  setHorizontalGridOffset : function(offset)
  {
    // summary:
    //		Sets the horizontal offset between the grid lines. 
    // description:
    //		Sets the horizontal offset between the grid lines. 
    //		Links are routed on an orthogonal grid.  
    //		<p>
    //		The default value is <code>5</code>.  
    //		</p>
    
    if (offset <= 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6550E",  offset);
    }

    if (offset != this._gridXOffset)
    {
      this._gridXOffset = offset;
      this.onParameterChanged("HorizontalGridOffset");
    }
  },

  getHorizontalGridOffset : function()
  {
    // summary:
    //		Returns the horizontal offset between the grid lines.
    
    return (false ? LongLinkLayout._defaultGridXOffset : this._gridXOffset); // Number
  },

  setVerticalGridOffset : function(offset)
  {
    // summary:
    //		Sets the vertical offset between the grid lines. 
    // description:
    //		Sets the vertical offset between the grid lines. 
    //		Links are routed on an orthogonal grid.  
    //		<p>
    //		The default value is <code>5</code>.  
    //		</p>
    
    if (offset <= 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6551E",  offset);
    }

    if (offset != this._gridYOffset)
    {
      this._gridYOffset = offset;
      this.onParameterChanged("VerticalGridOffset");
    }
  },

  getVerticalGridOffset : function()
  {
    // summary:
    //		Returns the vertical offset between the grid lines.
    
    return (false ? LongLinkLayout._defaultGridYOffset : this._gridYOffset); // Number
  },

  setHorizontalGridBase : function(coordinate)
  {
    // summary:
    //		Sets the horizontal base coordinate of the grid. 
    // description:
    //		Sets the horizontal base coordinate of the grid. 
    //		This is the <i>x</i>-coordinate of the first vertical grid line.
    //		Links are routed on an orthogonal grid.  
    //		<p>
    //		The default value is <code>0</code>.  
    //		</p>
    
    if (coordinate < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6552E",  coordinate);
    }

    if (coordinate != this._gridXBase)
    {
      this._gridXBase = coordinate;
      this.onParameterChanged("HorizontalGridBase");
    }
  },

  getHorizontalGridBase : function()
  {
    // summary:
    //		Returns the horizontal base coordinate of the grid.
    
    return (false ? LongLinkLayout._defaultGridXBase : this._gridXBase); // Number
  },

  setVerticalGridBase : function(coordinate)
  {
    // summary:
    //		Sets the vertical base coordinate of the grid. 
    // description:
    //		Sets the vertical base coordinate of the grid. 
    //		This is the <i>y</i>-coordinate of the first horizontal grid line.
    //		Links are routed on an orthogonal grid.  
    //		<p>
    //		The default value is <code>0</code>.  
    //		</p>
    
    if (coordinate < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6553E",  coordinate);
    }

    if (coordinate != this._gridYBase)
    {
      this._gridYBase = coordinate;
      this.onParameterChanged("VerticalGridBase");
    }
  },

  getVerticalGridBase : function()
  {
    // summary:
    //		Returns the vertical base coordinate of the grid.
    
    return (false ? LongLinkLayout._defaultGridYBase : this._gridYBase); // Number
  },

  setHorizontalMinOffset : function(minOffset)
  {
    // summary:
    //		Sets the minimum horizontal offset between the node border and a
    //		link segment that is (vertically) parallel to the node border. 
    // description:
    //		Sets the minimum horizontal offset between the node border and a
    //		link segment that is (vertically) parallel to the node border. 
    //		<p>
    //		The default value is <code>3</code>.  
    //		</p>
    
    if (minOffset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6554E",  minOffset);
    }

    if (minOffset != this._minXOffset)
    {
      this._minXOffset = minOffset;
      this.onParameterChanged("HorizontalMinOffset");
    }
  },

  getHorizontalMinOffset : function()
  {
    // summary:
    //		Returns the minimum horizontal offset between the node border and a
    //		link segment that is (vertically) parallel to the node border.
    
    return (false ? LongLinkLayout._defaultMinXOffset : this._minXOffset); // Number
  },

  setVerticalMinOffset : function(minOffset)
  {
    // summary:
    //		Sets the minimum vertical offset between the node border and a link
    //		segment that is (horizontally) parallel to the node border. 
    // description:
    //		Sets the minimum vertical offset between the node border and a link
    //		segment that is (horizontally) parallel to the node border. 
    //		<p>
    //		The default value is <code>3</code>.  
    //		</p>
    
    if (minOffset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6555E",  minOffset);
    }

    if (minOffset != this._minYOffset)
    {
      this._minYOffset = minOffset;
      this.onParameterChanged("VerticalMinOffset");
    }
  },

  getVerticalMinOffset : function()
  {
    // summary:
    //		Returns the minimum vertical offset between the node border and a
    //		link segment that is (horizontally) parallel to the node border.
    
    return (false ? LongLinkLayout._defaultMinYOffset : this._minYOffset); // Number
  },

  setMinNodeCornerOffset : function(offset)
  {
    // summary:
    //		Sets the minimum offset between a node corner and a link starting at
    //		this node. 
    // description:
    //		Sets the minimum offset between a node corner and a link starting at
    //		this node. 
    //		Setting this to <code>0</code> enables the layout to start a link
    //		exactly at a node corner.  
    //		<p>
    //		The default value is <code>3</code>.  
    //		</p>
    
    if (offset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6556E",  offset);
    }

    if (offset != this._minNodeCornerOffset)
    {
      this._minNodeCornerOffset = offset;
      this.onParameterChanged("MinNodeCornerOffset");
    }
  },

  getMinNodeCornerOffset : function()
  {
    // summary:
    //		Returns the minimum offset between a node corner and a link starting
    //		at this node.
    
    return (false ? LongLinkLayout._defaultMinNodeCornerOffset : this._minNodeCornerOffset); // Number
  },

  setMinStartSegmentLength : function(length)
  {
    // summary:
    //		Sets the minimum length of the first segment of each link, that is,
    //		the segment incident to the "from" node. 
    // description:
    //		Sets the minimum length of the first segment of each link, that is,
    //		the segment incident to the "from" node. 
    //		<p>
    //		The default value is <code>10</code>.  
    //		</p>
    
    if (length < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6557E",  length);
    }

    if (length != this._minStartSegmentLength)
    {
      this._minStartSegmentLength = length;

      this.clearIncrementalData();
      this.onParameterChanged("MinStartSegmentLength");
    }
  },

  getMinStartSegmentLength : function()
  {
    // summary:
    //		Returns the minimum length of the first segment of each link.
    
    return (false ? LongLinkLayout._defaultMinStartEndSegmentLength : this._minStartSegmentLength); // Number
  },

  setMinEndSegmentLength : function(length)
  {
    // summary:
    //		Sets the minimum length of the last segment of each link, that is,
    //		the segment incident to the "to" node. 
    // description:
    //		Sets the minimum length of the last segment of each link, that is,
    //		the segment incident to the "to" node. 
    //		<p>
    //		The default value is <code>10</code>.  
    //		</p>
    
    if (length < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6558E",  length);
    }

    if (length != this._minEndSegmentLength)
    {
      this._minEndSegmentLength = length;

      this.clearIncrementalData();
      this.onParameterChanged("MinEndSegmentLength");
    }
  },

  getMinEndSegmentLength : function()
  {
    // summary:
    //		Returns the minimum length of the last segment of each link.
    
    return (false ? LongLinkLayout._defaultMinStartEndSegmentLength : this._minEndSegmentLength); // Number
  },

  setAllowedTimePerLink : function(time)
  {
    // summary:
    //		Sets the upper limit for the time allowed to find a route for a long
    //		link. 
    // description:
    //		Sets the upper limit for the time allowed to find a route for a long
    //		link. 
    //		The algorithm tries to find a short route for the link. It uses a
    //		backtrack mechanism to find a route that bypasses obstacles. The
    //		runtime of the backtrack can be large if the size of the drawing
    //		area is large and the grid offset is small. The algorithm stops the
    //		backtrack when the specified time is elapsed and gives up without
    //		result.  
    //		<p>
    //		The default value is <code>2000</code> (2 seconds).  
    //		</p>
    //		<p>
    //		Note that the specified time is valid for each single link, that is,
    //		the <i>maximum time</i> to find routes for all links is
    //		approximately the specified time per link multiplied by the number
    //		of links. However, this happens only extremely seldom. The algorithm
    //		will most likely find a route for the majority of the links before
    //		the time elapses; therefore, the <i>average time</i> to find routes
    //		for all links is much smaller.  
    //		</p>
    //		<p>
    //		If the specified time elapsed for a link and the fallback routing is
    //		enabled, the link is quickly routed by the fallback mechanism
    //		instead.  
    //		</p>
    
    if (time <= 0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6559E");
    }

    if (time != this._allowedTimePerLink)
    {
      this._allowedTimePerLink = time;
      this.onParameterChanged("AllowedTimePerLink");
    }
  },

  getAllowedTimePerLink : function()
  {
    // summary:
    //		Returns the upper limit for the time allowed to find a route for a
    //		long link in  milliseconds.
    
    if (false) {
      return LongLinkLayout._defaultAllowedTimePerLink; // Number
    }
    return this._allowedTimePerLink; // Number
  },

  setMaxBacktrack : function(numberOfSteps)
  {
    // summary:
    //		Sets the maximum number of backtrack steps to find a route for a
    //		long link. 
    // description:
    //		Sets the maximum number of backtrack steps to find a route for a
    //		long link. 
    //		The algorithm tries to find a short route for the link. The
    //		backtrack number indicates that the algorithm tries to find a route
    //		that is so many grid points longer than the shortest possible route
    //		before it gives up the search.  
    //		<p>
    //		Decreasing the value causes the algorithm to give up earlier without
    //		result. If the "from" point and "to" point are fixed, this speeds up
    //		the search. Otherwise, the time behavior is more complex, so it may
    //		speed up or slow down the search.  
    //		</p>
    //		<p>
    //		The default value is <code>30000</code>.  
    //		</p>
    //		<p>
    //		If the maximum number of backtrack steps is exceeded and the
    //		fallback routing is enabled, the link is quickly routed by the
    //		fallback mechanism instead.  
    //		</p>
    
    if (numberOfSteps < 0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6560E",  numberOfSteps);
    }

    if (numberOfSteps != this._maxBackTrack)
    {
      this._maxBackTrack = numberOfSteps;
      this.onParameterChanged("MaxBacktrack");
    }
  },

  getMaxBacktrack : function()
  {
    // summary:
    //		Returns the maximum number of backtrack steps to find a route for a
    //		long link.
    
    if (false) {
      return LongLinkLayout._defaultMaxBacktrack; // int
    }
    return this._maxBackTrack; // int
  },

  setGlobalLinkStyle : function(style)
  {
    // summary:
    //		Sets the global style of the shapes of links. 
    // description:
    //		Sets the global style of the shapes of links. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> LongLinkLayout.ORTHOGONAL&#95;STYLE</code> - all links get
    //		an orthogonal shape, that is, a shape consisting of a sequence of
    //		alternating horizontal and vertical line segments.</li>  
    //		<li> <code> LongLinkLayout.DIRECT&#95;STYLE</code> - all links get a
    //		nearly straight-line shape. The shape of the links is a polyline
    //		composed of a straight-line segment between the end nodes with two
    //		short horizontal or vertical segments at its ends (their length is
    //		controlled by <code>
    //		LongLinkLayout.setMinStartSegmentLength()</code> and <code>
    //		LongLinkLayout.setMinEndSegmentLength()</code> and may be 0). In
    //		this link style, links may overlap nodes.</li>  
    //		<li> <code> LongLinkLayout.MIXED&#95;STYLE</code> - each link can have a
    //		different link style. The style of each individual link can be set
    //		by the method <code> LongLinkLayout.setLinkStyle()</code> so that
    //		different link shapes can occur in the same graph.</li>  
    //		</ul>  
    //		<p>
    //		Unless the global link style is <code>
    //		LongLinkLayout.MIXED&#95;STYLE</code>, all links have the same style of
    //		shape.  
    //		</p>
    //		<p>
    //		The default value is <code> LongLinkLayout.ORTHOGONAL&#95;STYLE</code>. 
    //		 
    //		</p>
    
    if (style != LongLinkLayout.DIRECT_STYLE && style != LongLinkLayout.ORTHOGONAL_STYLE && style != LongLinkLayout.MIXED_STYLE && style != LongLinkLayout.NO_RESHAPE_STYLE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6561E",  style);
    }

    if (style != this._globalLinkStyle)
    {
      this._globalLinkStyle = style;

      this.clearIncrementalData();
      this.onParameterChanged("GlobalLinkStyle");
    }
  },

  getGlobalLinkStyle : function()
  {
    // summary:
    //		Returns the global style of the shapes of links.
    
    return (false ? LongLinkLayout._defaultLinkStyle : this._globalLinkStyle); // int
  },

  setLinkStyle : function(link, style)
  {
    // summary:
    //		Sets the style for the shape of an individual link. 
    // description:
    //		Sets the style for the shape of an individual link. 
    //		This link style is used only if the global link style is set to
    //		<code> LongLinkLayout.MIXED&#95;STYLE</code>. Otherwise, all links have
    //		the style that is specified as the global link style.  
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> LongLinkLayout.ORTHOGONAL&#95;STYLE</code> - the link gets
    //		an orthogonal shape, that is, a shape consisting of a sequence of
    //		alternating horizontal and vertical line segments.</li>  
    //		<li> <code> LongLinkLayout.DIRECT&#95;STYLE</code> - the link gets a
    //		nearly straight-line shape. The shape of the link is a polyline
    //		composed of a straight-line segment between the end nodes with two
    //		short horizontal or vertical segments at its ends (their length is
    //		controlled by <code>
    //		LongLinkLayout.setMinStartSegmentLength()</code> and <code>
    //		LongLinkLayout.setMinEndSegmentLength()</code> and may be 0). In
    //		this link style, links may overlap nodes.</li>  
    //		<li> <code> LongLinkLayout.NO&#95;RESHAPE&#95;STYLE</code> - the link is not
    //		reshaped.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code> LongLinkLayout.ORTHOGONAL&#95;STYLE</code>. 
    //		 
    //		</p>
    // link: Object
    //		The link instance.
    // style: int
    //		The style for the shape of an individual link.
    
    if (style != LongLinkLayout.ORTHOGONAL_STYLE && style != LongLinkLayout.DIRECT_STYLE && style != LongLinkLayout.NO_RESHAPE_STYLE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6562E",  style);
    }

    var changed;
    changed = LayoutParametersUtil.setLinkParameter(this, link, LongLinkLayout._LINK_STYLE_PROPERTY, style, LongLinkLayout._defaultLinkStyle);
    if (changed) {
      this.markForIncremental(link);
    }
  },

  getLinkStyle : function(link)
  {
    // summary:
    //		Returns the style for the shape of an individual link.  
    // link: Object
    //		The link instance.
    // returns:
    //		The style for the shape of an individual link. Possible values
    //		are: ' LongLinkLayout.ORTHOGONAL_STYLE', '
    //		LongLinkLayout.DIRECT_STYLE', '
    //		LongLinkLayout.NO_RESHAPE_STYLE'.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, LongLinkLayout._LINK_STYLE_PROPERTY, LongLinkLayout._defaultLinkStyle); // int
  },

  setExhaustiveSearching : function(flag)
  {
    // summary:
    //		Sets the exhaustive searching of routes for links. 
    // description:
    //		Sets the exhaustive searching of routes for links. 
    //		If this option is enabled, the algorithm examines the complete grid
    //		exhaustively to find a route for an orthogonal link. This may result
    //		in fewer bends, but it takes a huge amount of runtime for large
    //		graphs.  
    //		<p>
    //		Exhaustive searching is disabled by default.  
    //		</p>
    
    if (flag != this._exhaustiveSearching)
    {
      this._exhaustiveSearching = flag;
      this.onParameterChanged("ExhaustiveSearching");
    }
  },

  isExhaustiveSearching : function()
  {
    // summary:
    //		Returns 'true' if routing uses exhaustive searching.
    
    return (false ? LongLinkLayout._defaultExhaustiveSearching : this._exhaustiveSearching); // Boolean
  },

  setStraightRouteEnabled : function(flag)
  {
    // summary:
    //		Sets the straight routing. 
    // description:
    //		Sets the straight routing. 
    //		If this option is enabled, the algorithm makes an additional attempt
    //		to find a route with zero or one bend before starting the normal
    //		backtrack mechanism. As result, the layout has fewer bends, but the
    //		distribution of links at the border may become uneven.  
    //		<p>
    //		Straight routing is enabled by default.  
    //		</p>
    
    if (flag != this._straightRoute)
    {
      this._straightRoute = flag;
      this.onParameterChanged("StraightRouteEnabled");
    }
  },

  isStraightRouteEnabled : function()
  {
    // summary:
    //		Returns 'true' if straight routing is enabled.
    
    return (false ? LongLinkLayout._defaultStraightRoute : this._straightRoute); // Boolean
  },

  setCrossingReductionEnabled : function(flag)
  {
    // summary:
    //		Sets the crossing reduction. 
    // description:
    //		Sets the crossing reduction. 
    //		If this option is enabled, the algorithm tries to reduce the
    //		crossings between orthogonal links that are routed.  
    //		<p>
    //		Crossing reduction is enabled by default.  
    //		</p>
    
    if (flag != this._crossingReduction)
    {
      this._crossingReduction = flag;
      this.onParameterChanged("CrossingReductionEnabled");
    }
  },

  isCrossingReductionEnabled : function()
  {
    // summary:
    //		Returns 'true' if crossing reduction is enabled.
    
    return (false ? LongLinkLayout._defaultCrossingReduction : this._crossingReduction); // Boolean
  },

  setNumberCrossingReductionIterations : function(iterations)
  {
    // summary:
    //		Sets the maximum number of iterations performed during crossing
    //		reduction. 
    // description:
    //		Sets the maximum number of iterations performed during crossing
    //		reduction. 
    //		This option has an effect only if crossing reduction is enabled.
    //		Reducing the value speeds up the algorithm, but the layout may
    //		contain more link crossings.  
    //		<p>
    //		The default value is <code>2</code>.  
    //		</p>
    
    if (iterations < 0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6563E",  iterations);
    }

    if (iterations != this._numberCrossingReductionIterations)
    {
      this._numberCrossingReductionIterations = iterations;
      this.onParameterChanged("NumberCrossingReductionIterations");
    }
  },

  getNumberCrossingReductionIterations : function()
  {
    // summary:
    //		Returns the maximum number of iterations performed during crossing
    //		reduction.
    
    if (false) {
      return LongLinkLayout._defaultNumberCrossingReductionIterations; // int
    }
    return this._numberCrossingReductionIterations; // int
  },

  setGlobalOriginPointMode : function(mode)
  {
    // summary:
    //		Sets the global mode for the connection point of the links on the
    //		origin nodes. 
    // description:
    //		Sets the global mode for the connection point of the links on the
    //		origin nodes. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> LongLinkLayout.FREE&#95;MODE</code> - For all links, the
    //		layout is free to choose the appropriate position of the connection
    //		point on the origin node, except for "pinned" connection points (see
    //		<code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> LongLinkLayout.FIXED&#95;MODE</code> - For all links, the
    //		layout must keep the current position of the connection point on the
    //		origin node. This can be useful, in particular, if the node has
    //		fixed pins where the link is attached (as, for instance, with <code>
    //		ibm&#95;ilog.linkconnector.PinLinkConnector</code>)  and the pins should
    //		not be moved by the layout algorithm. <br> Note that, in this mode,
    //		the first segment of a link is not placed on the grid. Therefore,
    //		the first segment may be closer than the specified minimum offset to
    //		neighbor links, or may even overlap them if they start at the same
    //		fixed point.   
    //		<p>
    //		The fixed mode works only if the link connector delivers
    //		consistently the same connection point when called multiple times.
    //		If a link is not connected to a link connector, or if the <code>
    //		ibm&#95;ilog.linkconnector.ClippingLinkConnector</code> is used, the
    //		real connection point is calculated on the fly and may change when
    //		called multiple times with changing link bends. In this case, the
    //		fixed mode cannot ensure a correct routing of the links.   
    //		</p>
    //		</li>  
    //		<li> <code> LongLinkLayout.MIXED&#95;MODE</code> - Each link can have a
    //		different mode for the connection point on the origin node. The mode
    //		of each individual link can be set by the method <code>
    //		LongLinkLayout.setOriginPointMode()</code>.</li>  
    //		</ul>  
    //		<p>
    //		Unless the global mode is <code> LongLinkLayout.MIXED&#95;MODE</code>,
    //		the connection points on the origin node have the same mode for all
    //		the links.  
    //		</p>
    //		<p>
    //		The default value is <code> LongLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // mode: int
    //		The global mode to set.
    
    if (mode != LongLinkLayout.FREE_MODE && mode != LongLinkLayout.FIXED_MODE && mode != LongLinkLayout.MIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6564E",  mode);
    }

    if (mode != this._originPointMode)
    {
      this._originPointMode = mode;
      this.onParameterChanged("GlobalOriginPointMode");
    }
  },

  getGlobalOriginPointMode : function()
  {
    // summary:
    //		Returns the global mode for the connection points of the links on
    //		the origin nodes.
    
    return (false ? LongLinkLayout._defaultOriginPointMode : this._originPointMode); // int
  },

  setGlobalDestinationPointMode : function(mode)
  {
    // summary:
    //		Sets the global mode for the connection point of the links on the
    //		destination nodes. 
    // description:
    //		Sets the global mode for the connection point of the links on the
    //		destination nodes. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> LongLinkLayout.FREE&#95;MODE</code> - For all links, the
    //		layout is free to choose the appropriate position of the connection
    //		point on the destination node, except for "pinned" connection points
    //		(see <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> LongLinkLayout.FIXED&#95;MODE</code> - For all links, the
    //		layout must keep the current position of the connection point on the
    //		destination node. This can be useful, in particular, if the node has
    //		fixed pins where the link is attached (as, for instance, with <code>
    //		ibm&#95;ilog.linkconnector.PinLinkConnector</code>)  and the pins should
    //		not be moved by the layout algorithm. <br> Note that, in this mode,
    //		the first segment of a link is not placed on the grid. Therefore,
    //		the first segment may be closer than the specified minimum offset to
    //		neighbor links, or may even overlap them if they start at the same
    //		fixed point.   
    //		<p>
    //		The fixed mode works only if the link connector delivers
    //		consistently the same connection point when called multiple times.
    //		If a link is not connected to a link connector, or if the <code>
    //		ibm&#95;ilog.linkconnector.ClippingLinkConnector</code> is used, the
    //		real connection point is calculated on the fly and may change when
    //		called multiple times with changing link bends. In this case, the
    //		fixed mode cannot ensure a correct routing of the links.   
    //		</p>
    //		</li>  
    //		<li> <code> LongLinkLayout.MIXED&#95;MODE</code> - Each link can have a
    //		different mode for the connection point on the destination node. The
    //		mode of each individual link can be set by the method <code>
    //		LongLinkLayout.setDestinationPointMode()</code>.</li>  
    //		</ul>  
    //		<p>
    //		Unless the global mode is <code> LongLinkLayout.MIXED&#95;MODE</code>,
    //		the connection points on the destination node have the same mode for
    //		all the links.  
    //		</p>
    //		<p>
    //		The default value is <code> LongLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // mode: int
    //		The global mode to set.
    
    if (mode != LongLinkLayout.FREE_MODE && mode != LongLinkLayout.FIXED_MODE && mode != LongLinkLayout.MIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6565E",  mode);
    }

    if (mode != this._destinationPointMode)
    {
      this._destinationPointMode = mode;
      this.onParameterChanged("GlobalDestinationPointMode");
    }
  },

  getGlobalDestinationPointMode : function()
  {
    // summary:
    //		Returns the global mode for the connection points of the links on
    //		the destination nodes.
    
    return (false ? LongLinkLayout._defaultDestinationPointMode : this._destinationPointMode); // int
  },

  setOriginPointMode : function(link, mode)
  {
    // summary:
    //		Sets the mode for the connection point on an individual link on the
    //		origin node. 
    // description:
    //		Sets the mode for the connection point on an individual link on the
    //		origin node. 
    //		This mode is used only if the global from point mode is set to
    //		<code> LongLinkLayout.MIXED&#95;MODE</code>. Otherwise, the connection
    //		points on the origin nodes have, for all the links, the mode that is
    //		specified as the global origin point mode.  
    //		<p>
    //		Valid values are: 
    //		</p>
    //		<ul>
    //		<li> <code> LongLinkLayout.FREE&#95;MODE</code> - The layout is free to
    //		choose, for this link, the appropriate position of the connection
    //		point on the origin node, except for "pinned" connection points (see
    //		<code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> LongLinkLayout.FIXED&#95;MODE</code> - The layout must keep,
    //		for this link, the current position of the connection point on the
    //		origin node. This can be useful, in particular, if the node has
    //		fixed pins where the link is attached (as, for instance, with <code>
    //		ibm&#95;ilog.linkconnector.PinLinkConnector</code>)  and the pins should
    //		not be moved by the layout algorithm. <br> Note that, in this mode,
    //		the first segment of a link is not placed on the grid. Therefore,
    //		the first segment may be closer than the specified minimum offset to
    //		neighbor links, or may even overlap them if they start at the same
    //		fixed point.   
    //		<p>
    //		The fixed mode works only if the link connector delivers
    //		consistently the same connection point when called multiple times.
    //		If a link is not connected to a link connector, or if the <code>
    //		ibm&#95;ilog.linkconnector.ClippingLinkConnector</code> is used, the
    //		real connection point is calculated on the fly and may change when
    //		called multiple times with changing link bends. In this case, the
    //		fixed mode cannot ensure a correct routing of the links.   
    //		</p>
    //		</li>  
    //		</ul>  
    //		<p>
    //		The default value is <code> LongLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // link: Object
    //		The link.
    // mode: int
    //		The mode to set.
    
    if (mode != LongLinkLayout.FREE_MODE && mode != LongLinkLayout.FIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6566E",  mode);
    }

    LayoutParametersUtil.setLinkParameter(this, link, LongLinkLayout._ORIGIN_POINT_MODE_PROPERTY, mode, LongLinkLayout._defaultOriginPointMode);
  },

  getOriginPointMode : function(link)
  {
    // summary:
    //		Returns the origin-point mode of an individual link.  
    // link: Object
    //		The link.
    // returns:
    //		The mode for the connection point of an individual link on the
    //		origin node.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, LongLinkLayout._ORIGIN_POINT_MODE_PROPERTY, LongLinkLayout._defaultOriginPointMode); // int
  },

  setDestinationPointMode : function(link, mode)
  {
    // summary:
    //		Sets the mode for the connection point of an individual link on the
    //		destination node. 
    // description:
    //		Sets the mode for the connection point of an individual link on the
    //		destination node. 
    //		This mode is used only if the global from-point mode is set to
    //		<code> LongLinkLayout.MIXED&#95;MODE</code>. Otherwise, the connection
    //		points on the destination nodes have, for all the links, the mode
    //		that is specified as the global destination-point mode.  
    //		<p>
    //		Valid values are: 
    //		</p>
    //		<ul>
    //		<li> <code> LongLinkLayout.FREE&#95;MODE</code> - The layout is free to
    //		choose, for this link, the appropriate position of the connection
    //		point on the destination node, except for "pinned" connection points
    //		(see <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.hasPinnedConnectionPoint()</code>).</li>
    //		 
    //		<li> <code> LongLinkLayout.FIXED&#95;MODE</code> - The layout must keep,
    //		for this link, the current position of the connection point on the
    //		destination node. This can be useful, in particular, if the node has
    //		fixed pins where the link is attached (as, for instance, with <code>
    //		ibm&#95;ilog.linkconnector.PinLinkConnector</code>)  and the pins should
    //		not be moved by the layout algorithm. <br> In this mode, the first
    //		segment of a link is not placed on the grid. Therefore, the first
    //		segment may be closer than the specified minimum offset to neighbor
    //		links, or may even overlap them if they start at the same fixed
    //		point.   
    //		<p>
    //		The fixed mode works only if the link connector delivers
    //		consistently the same connection point when called multiple times.
    //		If a link is not connected to a link connector, or if the <code>
    //		ibm&#95;ilog.linkconnector.ClippingLinkConnector</code> is used, the
    //		real connection point is calculated on the fly and may change when
    //		called multiple times with changing link bends. In this case, the
    //		fixed mode cannot ensure a correct routing of the links.   
    //		</p>
    //		</li>  
    //		</ul>  
    //		<p>
    //		The default value is <code> LongLinkLayout.FREE&#95;MODE</code>.  
    //		</p>
    // link: Object
    //		The link.
    // mode: int
    //		The mode to set.
    
    if (mode != LongLinkLayout.FREE_MODE && mode != LongLinkLayout.FIXED_MODE)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6567E",  mode);
    }

    LayoutParametersUtil.setLinkParameter(this, link, LongLinkLayout._DESTINATION_POINT_MODE_PROPERTY, mode, LongLinkLayout._defaultDestinationPointMode);
  },

  getDestinationPointMode : function(link)
  {
    // summary:
    //		Returns the destination-point mode of an individual link.  
    // link: Object
    //		The link.
    // returns:
    //		The mode for the connection point of an individual link on the
    //		destination node.
    
    return LayoutParametersUtil.getLinkEnumParameter(this, link, LongLinkLayout._DESTINATION_POINT_MODE_PROPERTY, LongLinkLayout._defaultOriginPointMode); // int
  },

  isFromPointFixed : function(link)
  {
    // summary:
    //		Returns 'true' if the connection point of
    //		'link' on the origin node must be fixed. 
    // description:
    //		Returns <code>true</code> if the connection point of
    //		<code>link</code> on the origin node must be fixed. 
    //		Returns <code>false</code> otherwise.
    // tags:
    //		private
    
    switch (this.getGlobalOriginPointMode())
    {
      case LongLinkLayout.FREE_MODE:
        return this.getGraphModel().hasPinnedConnectionPoint(link, true); // Boolean
      case LongLinkLayout.FIXED_MODE:
        return true; // Boolean
      case LongLinkLayout.MIXED_MODE:
        return (this.getOriginPointMode(link) == LongLinkLayout.FIXED_MODE ? true : this.getGraphModel().hasPinnedConnectionPoint(link, true)); // Boolean
      default:
        var params = [this.getGlobalOriginPointMode() ];
        LogResUtil.logAndThrowRuntimeExc(" LongLinkLayout ", "graphlayout.expert.message.6564E", params);
        return false; // Boolean
    }
  },

  isToPointFixed : function(link)
  {
    // summary:
    //		Returns 'true' if the connection point of
    //		'link' on the destination node must be fixed. 
    // description:
    //		Returns <code>true</code> if the connection point of
    //		<code>link</code> on the destination node must be fixed. 
    //		Returns <code>false</code> otherwise.
    // tags:
    //		private
    
    switch (this.getGlobalDestinationPointMode())
    {
      case LongLinkLayout.FREE_MODE:
        return this.getGraphModel().hasPinnedConnectionPoint(link, false); // Boolean
      case LongLinkLayout.FIXED_MODE:
        return true; // Boolean
      case LongLinkLayout.MIXED_MODE:
        return (this.getDestinationPointMode(link) == LongLinkLayout.FIXED_MODE ? true : this.getGraphModel().hasPinnedConnectionPoint(link, false)); // Boolean
      default:
        var params = [this.getGlobalDestinationPointMode() ];
        LogResUtil.logAndThrowRuntimeExc(" LongLinkLayout ", "graphlayout.expert.message.6565E", params);
        return false; // Boolean
    }
  },

  setIncrementalMode : function(enable)
  {
    // summary:
    //		Sets the incremental layout mode. 
    // description:
    //		Sets the incremental layout mode. 
    //		If the argument is <code>true</code>, only those links that were
    //		affected by changes since the last layout are rerouted. A link is
    //		rerouted under the following conditions:  
    //		<ul>
    //		<li> if the link is new,</li>  
    //		<li> if the link shape or link style has changed,</li>  
    //		<li> if the parameters controlling start and end segments have
    //		changed,</li>  
    //		<li> if the link now overlaps obstacles that were moved or
    //		resized,</li>  
    //		<li> if the link is no longer on the grid,</li>  
    //		<li> if the link is marked for incremental rerouting.</li>  
    //		</ul>
    //		The links preserve the side and relative order of their connection
    //		points from the previous layout, if this mode is selected by <code>
    //		LongLinkLayout.setIncrementalConnectionPreserving()</code>.  
    //		<p>
    //		The incremental mode works only if the intergraph links mode is
    //		disabled or if there are no intergraph links. The incremental mode
    //		is disabled by default.  
    //		</p>
    
    if (enable != this._incrementalMode)
    {
      this._incrementalMode = enable;
      this.onParameterChanged("IncrementalMode");

      this.ensureIncrementalData(enable);
    }
  },

  ensureIncrementalData : function(enable)
  {
    // summary:
    //		
    // tags:
    //		private
    
    if (enable)
    {
      if (this._incrementalData == null)
      {
        this._incrementalData = new LLIncrementalData();
        this._incrementalData.update(this);
      }
    }
    else
    {
      this._incrementalData = null;
    }
  },

  isIncrementalMode : function()
  {
    // summary:
    //		Returns 'true' if the incremental mode is enabled.
    
    return (false ? false : this._incrementalMode); // Boolean
  },

  getIncrementalData : function()
  {
    // summary:
    //		Returns the additional data container necessary for incremental
    //		layout.  
    // tags:
    //		private
    
    return this._incrementalData; // ibm_ilog.graphlayout.longlink.LLIncrementalData
  },

  clearIncrementalData : function()
  {
    // summary:
    //		Clear incremental data so that all links are rerouted next time.  
    // tags:
    //		private
    
    if (this._incrementalData != null)
    {
      this._incrementalData = new LLIncrementalData();
    }
  },

  setIncrementalConnectionPreserving : function(flag)
  {
    // summary:
    //		Sets whether the incremental layout mode preserves the side and
    //		relative order of the link connection points. 
    // description:
    //		Sets whether the incremental layout mode preserves the side and
    //		relative order of the link connection points. 
    //		The default is <code>false</code>.
    
    if (flag != this._incrementalConnectionPreserving)
    {
      this._incrementalConnectionPreserving = flag;
      this.onParameterChanged("IncrementalConnectionPreserving");
    }

    if (this._incrementalData != null) {
      this._incrementalData.update(this);
    }
  },

  isIncrementalConnectionPreserving : function()
  {
    // summary:
    //		Returns 'true' if the incremental mode preserves the side
    //		and relative order of the link connection points.
    
    if (false) {
      return false; // Boolean
    }
    return this._incrementalConnectionPreserving; // Boolean
  },

  markForIncremental : function(link)
  {
    // summary:
    //		Marks the input link to be rerouted with the next call of '
    //		ibm_ilog.graphlayout.GraphLayout.performLayout()', if
    //		incremental mode is enabled. 
    // description:
    //		Marks the input link to be rerouted with the next call of <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>, if
    //		incremental mode is enabled. 
    //		Normally the incremental layout reroutes only those links that have
    //		changed since the last layout. By setting a mark on the link, this
    //		link will be rerouted even if it has not changed.  
    // link: Object
    //		The link instance.
    
    if (this.getIncrementalData() != null)
    {
      this.getIncrementalData().clear(link);
      this.setParametersUpToDate(false);
    }
  },

  setInterGraphLinksMode : function(enable)
  {
    // summary:
    //		Sets the routing of intergraph links. 
    // description:
    //		Sets the routing of intergraph links. 
    //		If the argument is <code>true</code>, the intergraph links are
    //		routed; otherwise, only the normal links are routed. An intergraph
    //		link is a link stored in a given graph that has an origin or
    //		destination not stored in this graph. If the combined intergraph
    //		link routing is enabled in additional, this means that normal links
    //		and intergraph links are routed together;, otherwise, only the
    //		intergraph links are routed.  
    //		<p>
    //		The default value is <code>true</code>.  
    //		</p>
    //		<p>
    //		If the intergraph link mode is enabled and the graph actually
    //		contains some intergraph links, there are some restrictions:  
    //		<ul>
    //		<li> Layout animation is not possible.</li>  
    //		<li> The incremental mode has no effect.</li>   
    //		<li> If layout is performed on a nested <code>
    //		ibm&#95;ilog.diagram.Graph</code> (or <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel</code>), the top-level
    //		grapher must have a manager view attached because of the coordinate
    //		transformations of the subgraphers. If you create your own subclass
    //		of <code> ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel</code>
    //		instead of <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel</code> to route
    //		intergraph links, you must ensure that all graph models of all
    //		subgraphers use the same coordinate system through appropriate
    //		coordinate transformations.</li>   
    //		</ul>  
    //		</p>
    
    if (enable != this._interGraphLinksMode)
    {
      this._interGraphLinksMode = enable;
      this.onParameterChanged("InterGraphLinksMode");
    }
  },

  isInterGraphLinksMode : function()
  {
    // summary:
    //		Returns 'true' if the routing of intergraph links is
    //		enabled. 
    // description:
    //		Returns <code>true</code> if the routing of intergraph links is
    //		enabled. 
    //		It returns <code>false</code> otherwise.
    
    return (false ? LongLinkLayout._defaultInterGraphLinksMode : this._interGraphLinksMode); // Boolean
  },

  setCombinedInterGraphLinksMode : function(enable)
  {
    // summary:
    //		Sets the routing of normal links in addition to intergraph links. 
    // description:
    //		Sets the routing of normal links in addition to intergraph links. 
    //		If intergraph link routing is disabled, the option has no effect. If
    //		intergraph link routing is enabled and the argument is
    //		<code>true</code>, both normal links and intergraph links are routed
    //		in the same pass. Otherwise, only the intergraph links are routed.  
    //		<p>
    //		The default value is <code>true</code>.  
    //		</p>
    
    if (enable != this._combinedInterGraphLinksMode)
    {
      this._combinedInterGraphLinksMode = enable;
      this.onParameterChanged("CombinedInterGraphLinksMode");
    }
  },

  isCombinedInterGraphLinksMode : function()
  {
    // summary:
    //		Returns 'true' if the routing of normal links in addition
    //		to intergraph links is enabled. 
    // description:
    //		Returns <code>true</code> if the routing of normal links in addition
    //		to intergraph links is enabled. 
    //		Returns <code>false</code> otherwise.
    
    return (false ? LongLinkLayout._defaultCombinedInterGraphLinksMode : this._combinedInterGraphLinksMode); // Boolean
  },

  setNodeObstacleEnabled : function(flag)
  {
    // summary:
    //		Sets whether nodes should be obstacles for the link routes. 
    // description:
    //		Sets whether nodes should be obstacles for the link routes. 
    //		The default is <code>true</code>. There are, however, situations
    //		where it may be suitable to consider a set of obstacles different
    //		from the nodes. In this case you can switch nodes as obstacles off
    //		and use the method <code> LongLinkLayout.addRectObstacle()</code> to
    //		specify the obstacles that are needed.
    
    if (flag != this._nodeObstacleEnabled)
    {
      this._nodeObstacleEnabled = flag;
      this.onParameterChanged("NodeObstacleEnabled");
    }
  },

  isNodeObstacleEnabled : function()
  {
    // summary:
    //		Returns 'true' if nodes should be obstacles for the link
    //		routes.
    
    if (false) {
      return true; // Boolean
    }
    return this._nodeObstacleEnabled; // Boolean
  },

  setLinkObstacleEnabled : function(flag)
  {
    // summary:
    //		Sets whether links should be line obstacles for the link routes. 
    // description:
    //		Sets whether links should be line obstacles for the link routes. 
    //		<p>
    //		The default is <code>true</code>. There are, however, situations
    //		where it may be suitable to consider a set of obstacles different
    //		from the links. In this case you can switch links as obstacles off
    //		and use the methods <code>
    //		LongLinkLayout.addLineRectObstacle()</code> and <code>
    //		LongLinkLayout.addLineObstacle()</code> to specify the obstacles
    //		that are needed.  
    //		</p>
    
    if (flag != this._linkObstacleEnabled)
    {
      this._linkObstacleEnabled = flag;
      this.onParameterChanged("LinkObstacleEnabled");
    }
  },

  isLinkObstacleEnabled : function()
  {
    // summary:
    //		Returns 'true' if links should be line obstacles for the
    //		link routes.
    
    if (false) {
      return true; // Boolean
    }
    return this._linkObstacleEnabled; // Boolean
  },

  addRectObstacle : function(obstacleBoundingBox)
  {
    // summary:
    //		Adds a rectangular obstacle for the routing. 
    // description:
    //		Adds a rectangular obstacle for the routing. 
    //		<p>
    //		All nodes are considered rectangular obstacles by default for the
    //		routing of a link. The algorithm tries to find a link routing that
    //		avoids overlaps with rectangular obstacles. If graphic objects that
    //		are not nodes should also be considered as obstacles, you can add
    //		them by this method.  
    //		</p>
    // obstacleBoundingBox: Rectangle
    //		A rectangular obstacle for the routing.
    
    this._rectObstacles.push(obstacleBoundingBox);

    this.onParameterChanged("Obstacles");
  },

  removeAllRectObstacles : function()
  {
    // summary:
    //		Removes all additional rectangular obstacles.
    
    this._rectObstacles.length = 0;

    this.onParameterChanged("Obstacles");
  },

  getRectObstacles : function()
  {
    // summary:
    //		Returns the vector of rectangular obstacles.
    // tags:
    //		private
    
    return (this._rectObstacles); // Array
  },

  addLineRectObstacle : function(obstacleBoundingBox)
  {
    // summary:
    //		Adds a line segment obstacle for the routing. 
    // description:
    //		Adds a line segment obstacle for the routing. 
    //		<p>
    //		This must be either a horizontal line segment or a vertical line
    //		segment; hence, the input bounding box must either have zero width
    //		or zero height. Orthogonally routed link segments are considered
    //		line segment obstacles by default for the routing of another link.
    //		The algorithm tries to find a link routing that allows crossing line
    //		obstacles in the orthogonal direction but avoids overlaps with line
    //		obstacles in the parallel direction. If graphic objects that are not
    //		links should also be considered as obstacles, you can add them by
    //		this method.  
    //		</p>
    // obstacleBoundingBox: Rectangle
    //		A line segment obstacle for the routing.
    
    if (obstacleBoundingBox.width != 0.0 && obstacleBoundingBox.height != 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" LongLinkLayout ", "graphlayout.expert.message.6568E");
    }

    this._lineObstacles.push(obstacleBoundingBox);

    this.onParameterChanged("Obstacles");
  },

  addLineObstacle : function(p1, p2)
  {
    // summary:
    //		Adds a line segment obstacle for link routing. 
    // description:
    //		Adds a line segment obstacle for link routing. 
    //		<p>
    //		A line segment obstacle does not need to be strictly horizontal or
    //		vertical. Orthogonally routed link segments are considered line
    //		segment obstacles by default for the routing of another link. The
    //		algorithm tries to find link routing that allows crossing line
    //		obstacles in the orthogonal direction but avoids overlaps with line
    //		obstacles in the parallel direction. If graphical objects that are
    //		not links should also be considered as obstacles, you can add them
    //		by this method.  
    //		</p>
    // p1: Point
    //		The start point of the line segment obstacle.
    // p2: Point
    //		The end point of the line segment obstacle.
    
    this._lineFromObstacles.push(p1);
    this._lineToObstacles.push(p2);

    this.onParameterChanged("Obstacles");
  },

  removeAllLineObstacles : function()
  {
    // summary:
    //		Removes all additional line obstacles.
    
    this._lineObstacles.length = 0;
    this._lineFromObstacles.length = 0;
    this._lineToObstacles.length = 0;

    this.onParameterChanged("Obstacles");
  },

  getLineObstacles : function()
  {
    // summary:
    //		Returns the vector of line obstacles.
    // tags:
    //		private
    
    return (this._lineObstacles); // Array
  },

  getFromLineObstacles : function()
  {
    // summary:
    //		Returns the from point of line obstacles.
    // tags:
    //		private
    
    return (this._lineFromObstacles); // Array
  },

  getToLineObstacles : function()
  {
    // summary:
    //		Returns the to point of line obstacles.
    // tags:
    //		private
    
    return (this._lineToObstacles); // Array
  },

  addCalcFallbackLink : function(link)
  {
    // summary:
    //		Adds a link to the vector of links that are not normally routed. 
    // description:
    //		Adds a link to the vector of links that are not normally routed. 
    //		They need to be routed with a fallback mechanism. For internal usage
    //		only.  
    // tags:
    //		private
    
    (this.getLayoutReport()).incrNumberOfFallbackLinks();
    this._fallbackLinks.push(link);
  },

  getCalcFallbackLinks : function()
  {
    // summary:
    //		Returns the links that could not be routed on the grid without
    //		obstacle overlaps by the previous layout call. 
    // description:
    //		Returns the links that could not be routed on the grid without
    //		obstacle overlaps by the previous layout call. 
    //		A link may not be routed without overlaps if the allowed time to
    //		find a route for this single link elapsed (see <code>
    //		LongLinkLayout.getAllowedTimePerLink()</code>), or if the route
    //		would have to bypass too many obstacles (see <code>
    //		LongLinkLayout.getMaxBacktrack()</code>), or if there is no path on
    //		the grid from the start node to the end node that bypasses all
    //		obstacles.  
    //		<p>
    //		Such links must be routed by some fallback mechanism. If the
    //		fallback routing option was set to <code>true</code> by using <code>
    //		LongLinkLayout.setFallbackRouteEnabled()</code>, these links are
    //		routed by a simple fallback routing algorithm inside <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>. If the
    //		fallback routing option was set to <code>false</code>, such links
    //		are not routed. In this case, the user is supposed to implement a
    //		fallback mechanism to route such links after <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code> terminates
    //		successfully.  
    //		</p>
    //		<p>
    //		Note that there will be no calculated fallback links if the previous
    //		layout did not terminate successfully, for example, if the global
    //		layout time elapsed or if the algorithm was stopped immediately
    //		during the previous run of <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>.  
    //		</p>
    // returns:
    //		The links that could not be routed on the grid without obstacle
    //		overlaps by the previous layout call.
    
    return  new TranslateUtil.ArrayIterator(this._fallbackLinks); // ibm_ilog.graphlayout.IIterator
  },

  setFallbackRouteEnabled : function(flag)
  {
    // summary:
    //		Sets whether the fallback routing algorithm is enabled. 
    // description:
    //		Sets whether the fallback routing algorithm is enabled. 
    //		<p>
    //		This routing algorithm is used for those links that cannot be routed
    //		on the grid without overlapping other obstacles (nodes or links). If
    //		it is disabled, then these fallback links remain unrouted.  
    //		</p>
    //		<p>
    //		The default is <code>true</code>.  
    //		</p>
    
    if (flag != this._fallbackRouteEnabled)
    {
      this._fallbackRouteEnabled = flag;
      this.onParameterChanged("FallbackRouteEnabled");
    }
  },

  isFallbackRouteEnabled : function()
  {
    // summary:
    //		Returns 'true' if the fallback routing algorithm is
    //		enabled.
    
    if (false) {
      return true; // Boolean
    }
    return this._fallbackRouteEnabled; // Boolean
  },

  setNodeBoxInterface : function(nodeBoxInterface)
  {
    // summary:
    //		Sets the node box interface for the obstacle area. 
    // description:
    //		Sets the node box interface for the obstacle area. 
    //		<p>
    //		The node box interface is an object that provides the rectangle that
    //		determines the node box for each node. If none is given, the layout
    //		algorithm uses the bounding box of each node (see  ) by default.  
    //		</p>
    //		<p>
    //		This node box specifies the obstacle area of the node, that is, the
    //		area that links should avoid so they do not overlap the node. This
    //		can be different from the node box used to find the connection
    //		points of the links.  
    //		</p>
    //		<p>
    //		You can call this method with a <code>null</code> argument to go
    //		back to the default behavior.  
    //		</p>
    
    if (nodeBoxInterface != this._nodeBoxInterfaceForObstacle)
    {
      this._nodeBoxInterfaceForObstacle = nodeBoxInterface;
      this.onParameterChanged("NodeBoxInterface");

      this.clearIncrementalData();
    }
  },

  getNodeBoxInterface : function()
  {
    // summary:
    //		Returns the node box interface used for the obstacle area during the
    //		layout. 
    // description:
    //		Returns the node box interface used for the obstacle area during the
    //		layout. 
    //		Returns <code>null</code> if none is specified.
    
    return this._nodeBoxInterfaceForObstacle; // ibm_ilog.graphlayout.INodeBoxProvider
  },

  setLinkConnectionBoxInterface : function(connectionBoxInterface)
  {
    // summary:
    //		Sets the link connection box interface for the connection points of
    //		links. 
    // description:
    //		Sets the link connection box interface for the connection points of
    //		links. 
    //		The link connection box interface is an object that provides the
    //		rectangle to which the links are connected for each node.  
    //		<p>
    //		By default, the layout algorithm places the connection points of the
    //		links relative to the bounding box of the nodes (see   ).  
    //		</p>
    //		<p>
    //		By setting a link connection box interface, the links can be
    //		connected relative to a different box (see <code>
    //		ibm&#95;ilog.graphlayout.ILinkConnectionBoxProvider.getBox()</code>).
    //		The tangential offsets provided by the link connection box interface
    //		are currently ignored.  
    //		</p>
    //		<p>
    //		If you do not call this method , the layout algorithm places the
    //		connection points on the node box that has been customized using the
    //		method <code> LongLinkLayout.setNodeBoxInterface()</code>. If this
    //		method has not been used, the bounding box of the node in the graph
    //		model (see  ) is used.  
    //		</p>
    //		<p>
    //		You can call this method with a <code>null</code> argument to go
    //		back to the default behavior.  
    //		</p>
    
    if (connectionBoxInterface != this.getLinkConnectionBoxInterface())
    {
      this.clearIncrementalData();
      this.inherited("setLinkConnectionBoxInterface", arguments, [connectionBoxInterface]);
    }
  },

  getNodeBoxForConnection : function(node)
  {
    // summary:
    //		Returns the box of a node for determining the connection points of
    //		links.  
    // tags:
    //		private
    
    var model = this.getGraphModel();

    if (this.getLinkConnectionBoxInterface() != null)
    {
      if (model instanceof SubgraphData) {
        return (model).getNodeBox(this.getLinkConnectionBoxInterface(), node); // Rectangle
      }
      return this.getLinkConnectionBoxInterface().getBox(model, node); // Rectangle
    }

    if (this._nodeBoxInterfaceForObstacle != null)
    {
      if (model instanceof SubgraphData) {
        return (model).getNodeBox(this._nodeBoxInterfaceForObstacle, node); // Rectangle
      }
      return this._nodeBoxInterfaceForObstacle.getBox(model, node); // Rectangle
    }

    return this.getNodeBox(node); // Rectangle
  },

  getNodeBoxForObstacle : function(node)
  {
    // summary:
    //		Returns the box of a node for determining the obstacle area of a
    //		node.  
    // tags:
    //		private
    
    var model = this.getGraphModel();

    if (this._nodeBoxInterfaceForObstacle != null)
    {
      if (model instanceof SubgraphData) {
        return (model).getNodeBox(this._nodeBoxInterfaceForObstacle, node); // Rectangle
      }
      return this._nodeBoxInterfaceForObstacle.getBox(model, node); // Rectangle
    }

    return this.getNodeBox(node); // Rectangle
  },

  getNodeBox : function(node)
  {
    // summary:
    //		Returns the box of a node.  
    // tags:
    //		private
    
    var model = this.getGraphModel();

    return GraphModelUtil.getObjectBounds(model, node); // Rectangle
  },

  setNodeSideFilter : function(filter)
  {
    // summary:
    //		Sets the node-side filter, that is, an object that allows or
    //		prohibits the connection of the links on a given side of the nodes. 
    // description:
    //		Sets the node-side filter, that is, an object that allows or
    //		prohibits the connection of the links on a given side of the nodes. 
    //		<p>
    //		Note that the node-side filter serves the same purpose as the
    //		termination point filter; it is, however, easier to use. If a
    //		termination point filter is installed, the node-side filter is
    //		ignored. If neither a termination point filter nor a node-side
    //		filter is installed, the layout algorithm is free to choose any side
    //		of the nodes.  
    //		</p>
    //		<p>
    //		When a filter is specified, the layout algorithm asks the filter at
    //		which side of its end node each link is allowed to attach to. Note
    //		that a filter should always allow a link to connect to at least one
    //		side of a node; otherwise the link cannot be routed.  
    //		</p>
    //		<p>
    //		You can call this method  with a <code>null</code> argument to go
    //		back to the default behavior.  
    //		</p>
    
    if (filter != this._nodeSideFilter)
    {
      this._nodeSideFilter = filter;
      this.onParameterChanged("NodeSideFilter");

      this.clearIncrementalData();
    }
  },

  getNodeSideFilter : function()
  {
    // summary:
    //		Returns the node-side filter used during layout. 
    // description:
    //		Returns the node-side filter used during layout. 
    //		Returns <code>null</code> if none is specified.
    
    return this._nodeSideFilter; // ibm_ilog.graphlayout.INodeSideFilter
  },

  setTerminationPointFilter : function(filter)
  {
    // summary:
    //		Sets the termination point filter, that is, an object that allows
    //		filtering the termination points that are proposed by the layout
    //		algorithm. 
    // description:
    //		Sets the termination point filter, that is, an object that allows
    //		filtering the termination points that are proposed by the layout
    //		algorithm. 
    //		<p>
    //		For each side, the layout algorithm proposes some termination
    //		points. It also proposes a penalty that indicates how "good" this
    //		termination point is. The default penalty indicates how far the
    //		termination point is away from the border. By installing a
    //		termination point filter, you can change this penalty.  
    //		</p>
    //		<p>
    //		Note that the termination point filter serves the same purpose as
    //		the node-side filter but allows a greater flexibility. If a
    //		termination point filter and a node-side filter are installed, the
    //		node-side filter is ignored. If neither a termination point filter
    //		nor a node-side filter is installed, the layout algorithm is free to
    //		choose any termination point.  
    //		</p>
    //		<p>
    //		When a filter is specified, the layout algorithm proposes a sequence
    //		of termination points on the grid for each link. It proposes
    //		termination points at all four sides of the corresponding end node.
    //		It asks the filter for the penalty of each termination point and
    //		then chooses the termination points with the lowest penalty that are
    //		feasible to route the link.  
    //		</p>
    //		<p>
    //		You can call this method with a <code>null</code> argument to go
    //		back to the default behavior.  
    //		</p>
    
    if (filter != this._terminationPointFilter)
    {
      this._terminationPointFilter = filter;
      this.onParameterChanged("TerminationPointFilter");

      this.clearIncrementalData();
    }
  },

  getTerminationPointFilter : function()
  {
    // summary:
    //		Returns the termination point filter used during layout. 
    // description:
    //		Returns the termination point filter used during layout. 
    //		Returns <code>null</code> if none is specified.
    
    return this._terminationPointFilter; // ibm_ilog.graphlayout.longlink.ILongLinkTerminationPointFilter
  },

  getInterGraphLinksRoutingModel : function()
  {
    // summary:
    //		Return the intergraph link routing model.
    
    return this._interGraphLinksRoutingModel; // ibm_ilog.graphlayout.internalutil.SubgraphData
  },

  getTerminationPointPenalty : function(link, node, origin, startX, startY, side, proposedPenalty)
  {
    // summary:
    //		Returns the penalty of a starting point. 
    // description:
    //		Returns the penalty of a starting point. 
    //		The flag origin is true if this is the from side of the link,
    //		otherwise the to side. This is only interesting for self loops. Side
    //		is Top, Bottom, Left, Right from Direction. The coordonates startX,
    //		startY are where the link should start, which is typically a
    //		coordinate outside of the node. The proposed penalty is the penalty
    //		the system would choose for this starting point.  
    // tags:
    //		private
    
    var termPointFilter = this.getTerminationPointFilter();
    var model;
    var origNode;
    var origLink;

    if (termPointFilter != null)
    {
      var point = new Point(startX, startY);
      model = this.getGraphModel();

      if (model instanceof SubgraphData)
      {
        origNode = (model).getOriginal(node);
        origLink = (model).getOriginal(link);

        if (origNode != null && origLink != null)
        {
          model = (model).getOriginalGraphModel(node);
          node = origNode;
          link = origLink;
        }
      }

      return (termPointFilter.getPenalty(model, link, origin, node, point, side, proposedPenalty)); // int
    }

    var nodeSideFilter = this.getNodeSideFilter();

    if (nodeSideFilter != null)
    {
      model = this.getGraphModel();

      if (model instanceof SubgraphData)
      {
        origNode = (model).getOriginal(node);
        origLink = (model).getOriginal(link);

        if (origNode != null && origLink != null)
        {
          model = (model).getOriginalGraphModel(node);
          node = origNode;
          link = origLink;
        }
      }

      if (nodeSideFilter.accept(this.getGraphModel(), link, origin, node, side)) {
        return (proposedPenalty); // int
      }
      else {
        return (/*Number.MAX_VALUE*/2147483647); // int
      }
    }

    return (proposedPenalty); // int
  },

  addStepCount : function(x)
  {
    // summary:
    //		Adds x step points to the percentage completion calculator.
    // tags:
    //		private
    
    this._stepCount += x;
  },

  layoutStepPerformed : function()
  {
    // summary:
    //		Can be called by the layout classes when a step of the layout
    //		algorithm has been performed. 
    // description:
    //		Can be called by the layout classes when a step of the layout
    //		algorithm has been performed. 
    //		The method notifies the listeners of the layout events (if any). In
    //		addition to the base functionality, this method updates the
    //		percentage of completion value.
    
    if (this._numberOfSteps > 0) {
      this.increasePercentageComplete(Math.floor(this._stepCount / this._numberOfSteps));
    }
    this.inherited("onLayoutStepPerformed", arguments, [false, false]);
  },

  mayContinue : function()
  {
    // summary:
    //		Returns 'true' if the layout can continue.
    // tags:
    //		private
    
    if (!this._stoppedBeforeCompletion) {
      this._stoppedBeforeCompletion = this.isLayoutTimeElapsed() || this.isStoppedImmediately();
    }
    return (!this._stoppedBeforeCompletion); // Boolean
  }

}
);

lang.mixin(LongLinkLayout,
{
  _LINK_STYLE_PROPERTY : null,

  _ORIGIN_POINT_MODE_PROPERTY : null,

  _DESTINATION_POINT_MODE_PROPERTY : null,

  ORTHOGONAL_STYLE : null,

  DIRECT_STYLE : null,

  MIXED_STYLE : null,

  NO_RESHAPE_STYLE : null,

  FREE_MODE : null,

  FIXED_MODE : null,

  MIXED_MODE : null,

  _defaultLinkStyle : null,

  _defaultOriginPointMode : null,

  _defaultDestinationPointMode : null,

  _defaultGridXBase : null,

  _defaultGridYBase : null,

  _defaultGridXOffset : null,

  _defaultGridYOffset : null,

  _defaultMinXOffset : null,

  _defaultMinYOffset : null,

  _defaultMinNodeCornerOffset : null,

  _defaultMinStartEndSegmentLength : null,

  _defaultMaxBacktrack : null,

  _defaultNumberCrossingReductionIterations : null,

  _defaultAllowedTimePerLink : null,

  _defaultExhaustiveSearching : null,

  _defaultCrossingReduction : null,

  _defaultStraightRoute : null,

  _defaultInterGraphLinksMode : null,

  _defaultCombinedInterGraphLinksMode : null,

  _defaultNodeBoxInterface : null,

  _defaultNodeSideFilter : null,

  _defaultTerminationPointFilter : null,
  _fields_init : function(c)
  {
    c = c || LongLinkLayout;
    if(GraphLayout._fields_init)
      GraphLayout._fields_init(c);
    
    c._LINK_STYLE_PROPERTY = "LinkStyle";
    c._ORIGIN_POINT_MODE_PROPERTY = "OriginPointMode";
    c._DESTINATION_POINT_MODE_PROPERTY = "DestinationPointMode";
    c.ORTHOGONAL_STYLE = LinkStyles.ORTHOGONAL_STYLE;
    c.DIRECT_STYLE = LinkStyles.DIRECT_STYLE;
    c.MIXED_STYLE = LinkStyles.MIXED_STYLE;
    c.NO_RESHAPE_STYLE = LinkStyles.NO_RESHAPE_STYLE;
    c.FREE_MODE = ConnectionPointModes.FREE_MODE;
    c.FIXED_MODE = ConnectionPointModes.FIXED_MODE;
    c.MIXED_MODE = ConnectionPointModes.MIXED_MODE;
    c._defaultLinkStyle = LongLinkLayout.ORTHOGONAL_STYLE;
    c._defaultOriginPointMode = LongLinkLayout.FREE_MODE;
    c._defaultDestinationPointMode = LongLinkLayout._defaultOriginPointMode;
    c._defaultGridXBase = 0.0;
    c._defaultGridYBase = 0.0;
    c._defaultGridXOffset = 5.0;
    c._defaultGridYOffset = 5.0;
    c._defaultMinXOffset = 3.0;
    c._defaultMinYOffset = 3.0;
    c._defaultMinNodeCornerOffset = 3.0;
    c._defaultMinStartEndSegmentLength = 10.0;
    c._defaultMaxBacktrack = 30000;
    c._defaultNumberCrossingReductionIterations = 2;
    c._defaultAllowedTimePerLink = 2000;
    c._defaultExhaustiveSearching = false;
    c._defaultCrossingReduction = true;
    c._defaultStraightRoute = true;
    c._defaultInterGraphLinksMode = true;
    c._defaultCombinedInterGraphLinksMode = true;
    c._defaultNodeBoxInterface = null;
    c._defaultNodeSideFilter = null;
    c._defaultTerminationPointFilter = null;
  }
  
}
);

LongLinkLayout._getGlobalProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'allowedTime', defaultValue: 32000 },
    { name: 'preserveFixedLinks', defaultValue: false },
    { name: 'splineRoutingEnabled', defaultValue: false },
    { name: 'minSplineCurveSize', defaultValue: 5 },
    { name: 'maxSplineCurveSize', defaultValue: 30 },
    { name: 'balanceSplineCurveThreshold', defaultValue: 10 },
    { name: 'horizontalGridOffset', defaultValue: 5 },
    { name: 'verticalGridOffset', defaultValue: 5 },
    { name: 'horizontalGridBase', defaultValue: 0 },
    { name: 'verticalGridBase', defaultValue: 0 },
    { name: 'horizontalMinOffset', defaultValue: 3 },
    { name: 'verticalMinOffset', defaultValue: 3 },
    { name: 'minNodeCornerOffset', defaultValue: 3 },
    { name: 'minStartSegmentLength', defaultValue: 10 },
    { name: 'minEndSegmentLength', defaultValue: 10 },
    { name: 'allowedTimePerLink', defaultValue: 2000 },
    { name: 'maxBacktrack', defaultValue: 30000 },
    { name: 'globalLinkStyle', defaultValue: LongLinkLayout.ORTHOGONAL_STYLE },
    { name: 'exhaustiveSearching', defaultValue: false },
    { name: 'straightRouteEnabled', defaultValue: true },
    { name: 'crossingReductionEnabled', defaultValue: true },
    { name: 'numberCrossingReductionIterations', defaultValue: 2 },
    { name: 'globalOriginPointMode', defaultValue: LongLinkLayout.FREE_MODE },
    { name: 'globalDestinationPointMode', defaultValue: LongLinkLayout.FREE_MODE },
    { name: 'incrementalMode', defaultValue: false },
    { name: 'incrementalConnectionPreserving', defaultValue: false },
    { name: 'interGraphLinksMode', defaultValue: true },
    { name: 'combinedInterGraphLinksMode', defaultValue: true },
    { name: 'nodeObstacleEnabled', defaultValue: true },
    { name: 'linkObstacleEnabled', defaultValue: true },
    { name: 'fallbackRouteEnabled', defaultValue: true }
  ];
  var baseProperties = GraphLayout._getGlobalProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

LongLinkLayout._getNodeProperties = function()
{
  // tags:
  //		private
  return GraphLayout._getNodeProperties();
};

LongLinkLayout._getLinkProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'fixed', defaultValue: false },
    { name: 'linkStyle', defaultValue: LongLinkLayout.ORTHOGONAL_STYLE },
    { name: 'originPointMode', defaultValue: LongLinkLayout.FREE_MODE },
    { name: 'destinationPointMode', defaultValue: LongLinkLayout.FREE_MODE }
  ];
  var baseProperties = GraphLayout._getLinkProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

/**
 * @private
 */
LongLinkLayoutAnonClass_1 = declare(SplineRouter,
{
  // tags:
  //		private
  
  __outerThis : null,

  constructor : function(arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
  },

  isReshapeAllowed : function(link)
  {
    // tags:
    //		private

    var interGraphModel = this.__outerThis.getInterGraphLinksRoutingModel();
    if (interGraphModel != null && interGraphModel.isFixedLink(link)) {
      return (false); // Boolean
    }
    if (this.__outerThis.isPreserveFixedLinks() && this.__outerThis.isFixed(link)) {
      return false; // Boolean
    }
    var filter = this.__outerThis.getSplineLinkFilter();

    if (filter != null)
    {
      if (interGraphModel != null)
      {
        if (!interGraphModel.isSpline(link, filter)) {
          return false; // Boolean
        }
      }
      else
      {
        if (!filter.isSpline(link)) {
          return false; // Boolean
        }
      }
    }

    if (this.__outerThis.getGlobalLinkStyle() == LongLinkLayout.MIXED_STYLE)
    {
      var linkStyle = this.__outerThis.getLinkStyle(link);
      if (linkStyle == LongLinkLayout.NO_RESHAPE_STYLE) {
        return false; // Boolean
      }
    }

    return true; // Boolean
  }

});

(function(){ LongLinkLayout._fields_init(); })();

return LongLinkLayout;
});

},
'ibm_ilog/graphlayout/shortlink/_BestNeighborOptimization':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_BestNeighborOptimization", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../shortlink/_CombinatorialOptimization"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    _CombinatorialOptimization	// #7
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _CombinatorialOptimization = ibm_ilog.graphlayout.shortlink._CombinatorialOptimization;
=====*/

var _BestNeighborOptimization = declare("ibm_ilog.graphlayout.shortlink._BestNeighborOptimization", _CombinatorialOptimization, 
{
  // summary:
  //		A generic best neighbor algorithm. 
  // description:
  //		A generic best neighbor algorithm. 
  //		
  // tags:
  //		extension
  
  _allowedNumberOfIterations: null,
  _startTime: null,
  _allowedTime: null,
  _stoppedBeforeCompletion: null,

  constructor : function(variables, allowedNumberOfIterations, allowedTime)
  {
    this._allowedNumberOfIterations = allowedNumberOfIterations;
    this._allowedTime = allowedTime;
  },
  
  stopImmediately : function()
  {
    this._stoppedBeforeCompletion = true;
  },

  isStoppedBeforeCompletion : function()
  {
    return this._stoppedBeforeCompletion; // Boolean
  },

  optimize : function()
  {
    var nVariables = this._variables.length;
    if (nVariables < 1) {
      return;
    }
    var iterationCounter = 0;
    var variable;
    var variantsCount;
    var count;
    var currentVariant;
    var initialVariant;
    var bestCost;
    var cost;
    var bestVariant;
    var lastAppliedVariant;
    var atLeastOneShapeChanged = false;
    this._startTime = new Date().getTime();
    this._stoppedBeforeCompletion = false;

    while (this.mayContinue(iterationCounter))
    {
      for (var variableIndex = 0; variableIndex < nVariables; variableIndex++)
      {
        variable = this._variables[variableIndex];

        variantsCount = this.getVariantsCount(variable);
        if (variantsCount < 1) {
          continue;
        }
        currentVariant = 0;
        initialVariant = currentVariant;
        count = 0;
        bestCost = /*Number.MAX_VALUE*/(3.4028235E38);
        bestVariant = currentVariant;
        lastAppliedVariant = -1;

        while (count < variantsCount)
        {
          this.applyVariant(variable, variableIndex, currentVariant);
          cost = this.getCost(variable);
          lastAppliedVariant = currentVariant;
          count++;

          if (cost >= 0)
          {
            if (this.isBetterVariant(variable, currentVariant, cost, bestVariant, bestCost))
            {
              bestVariant = currentVariant;
              bestCost = cost;

              if (this.isOptimalCost(cost))
              {
                break;
              }
            }
          }

          currentVariant++;

          if (currentVariant >= variantsCount) {
            currentVariant = 0;
          }
        }

        if (bestVariant != lastAppliedVariant)
        {
          this.applyVariant(variable, variableIndex, bestVariant);
        }

        if (bestVariant != initialVariant) {
          atLeastOneShapeChanged = true;
        }
        this.variantStepPerformed();
      }

      if (!atLeastOneShapeChanged) {
        break;
      }
      atLeastOneShapeChanged = false;

      if (nVariables == 1) {
        break;
      }
      iterationCounter++;
    }
  },

  mayContinue : function(currentNumberOfIterations)
  {
    if (this._stoppedBeforeCompletion) {
      return false; // Boolean
    }
    var res = (new Date().getTime() - this._startTime) <= this._allowedTime;
    if (!res) {
      this._stoppedBeforeCompletion = true;
    }
    return res && currentNumberOfIterations < this._allowedNumberOfIterations; // Boolean
  },

  variantStepPerformed : function()
  {
  }

}
);

return _BestNeighborOptimization;
});

},
'ibm_ilog/graphlayout/shortlink/_LinkLayoutBestNeighborOptimization':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/shortlink/_LinkLayoutBestNeighborOptimization", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LogResUtil",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../shortlink/_BestNeighborOptimization",	// #8
  "../shortlink/_SLBundlesModes"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LogResUtil,	// #6
    TranslateUtil,	// #7
    _BestNeighborOptimization,	// #8
    _SLBundlesModes	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var _BestNeighborOptimization = ibm_ilog.graphlayout.shortlink._BestNeighborOptimization;
=====*/

var _LinkLayoutBestNeighborOptimization = declare("ibm_ilog.graphlayout.shortlink._LinkLayoutBestNeighborOptimization", _BestNeighborOptimization, 
{
  // summary:
  //		A specialized best neighbor algo for the link layout. 
  // description:
  //		A specialized best neighbor algo for the link layout. 
  //		
  // tags:
  //		private
  
  _graphModel: null,
  _layout: null,
  _isLinkBundlesRequired : null,

  constructor : function(layout, numberOfIterations, allowedTime, variables)
  {
    this._isLinkBundlesRequired = false;
    var msg;

    if (layout == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6616E");
    }

    this._graphModel = layout.getGraphModel();

    if (this._graphModel == null)
    {
      LogResUtil.logAndThrowIllegalArgExc(" ShortLinkLayout ", "graphlayout.expert.message.6012E");
    }

    this._layout = layout;
    this._isLinkBundlesRequired = (layout.getShortLinkLayout().getLinkBundlesMode() != _SLBundlesModes.NO_BUNDLE);
  },
  
  preamble:function(){
      return [arguments[3], arguments[1], arguments[2]];
  },
  variantStepPerformed : function()
  {
    this._layout.getShortLinkLayout().callLayoutStepPerformedIfNeeded();
  },

  applyVariant : function(variable, variableIndex, variant)
  {
    // summary:
    //		Applies a variant.  
    // returns:
    //		false if the variant was already applied, true otherwise
    // tags:
    //		protected
    
    if (variant == this.getVariant(variableIndex)) {
      return false; // Boolean
    }
    var linkData = variable;
    this.setVariant(variableIndex, variant);
    var shapeType = linkData.getShapeType(variant);
    if (shapeType == linkData.getCurrentShapeType()) {
      return false; // Boolean
    }
    this._layout.reshapeLink(this._graphModel, linkData, shapeType, this._isLinkBundlesRequired);

    return true; // Boolean
  },

  getCost : function(variable)
  {
    // summary:
    //		Returns the cost associated with variable.
    // tags:
    //		protected
    
    return this._layout.getLinkCost(this._graphModel, variable, false); // Number
  },

  getVariantsCount : function(variable)
  {
    return (variable).getShapeTypesCount(); // int
  },

  isBetterVariant : function(variable, variant1, cost1, variant2, cost2)
  {
    // summary:
    //		Returns true if the variant1 with cost1 is preferred to variant2
    //		with cost2 for variable. 
    // description:
    //		Returns true if the variant1 with cost1 is preferred to variant2
    //		with cost2 for variable. 
    //		Returns false otherwise. This overridden implementation adds the
    //		comparison of the number of bends in the case cost1 == cost2.
    // tags:
    //		protected
    
    if (cost1 < cost2) {
      return true; // Boolean
    }
    else if (cost1 > cost2) {
      return false; // Boolean
    }
    else
    {
      var linkData = variable;
      var shapeType1 = linkData.getShapeType(variant1);
      var shapeType2 = linkData.getShapeType(variant2);

      return shapeType1.getNumberOfBends() < shapeType2.getNumberOfBends(); // Boolean
    }
  }

}
);

return _LinkLayoutBestNeighborOptimization;
});

}}});

require(["dojo/i18n"], function(i18n){
i18n._preloadLocalizations("ibm_ilog/graphlayout/nls/ibm_graphlayout_link", []);
});
define("ibm_ilog/graphlayout/ibm_graphlayout_link", [], 1);
