/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

//>>built
require({cache:{
'ibm_ilog/graphlayout/tree/TAutoTipOver':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TAutoTipOver", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/TranslateUtil",	// #7
  "../tree/TGraphAlgorithm",	// #8
  "../tree/TLayoutModes"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    GraphModelUtil,	// #6
    TranslateUtil,	// #7
    TGraphAlgorithm,	// #8
    TLayoutModes	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var TGraphAlgorithm = ibm_ilog.graphlayout.tree.TGraphAlgorithm;
=====*/

var TAutoTipOver = declare("ibm_ilog.graphlayout.tree.TAutoTipOver", TGraphAlgorithm, 
{
  // summary:
  //		This class calls contains the support for determining the local tip
  //		over specification automatically to optimize the aspect ratio. 
  // description:
  //		This class calls contains the support for determining the local tip
  //		over specification automatically to optimize the aspect ratio. 
  //		To calculate the optimal result, it would need exponential time.
  //		Therefore this here is just a heuristic that tries a few hard coded
  //		approaches and chooses the best one. (See at the end in comments the
  //		code that would test all exponentially many possibilities).  
  // tags:
  //		private
  
  _aspectRatio: null,

  _bestArea: null,
  _maxDepth: null,

  constructor : function(graph)
  {
    this._aspectRatio = graph.getAspectRatio();

    if (graph.getFlowDirection() % 2 == 0) {
      this._aspectRatio = 1 / this._aspectRatio;
    }
    this._bestArea = Number.MAX_VALUE;
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.inherited("disposeIt", arguments, []);
  },

  getAspectRatio : function()
  {
    // summary:
    //		Returns the desired aspect ratio.
    // tags:
    //		private
    
    return this._aspectRatio; // Number
  },

  getMaxDepth : function()
  {
    // summary:
    //		Returns the maximal depth of the tree.
    // tags:
    //		private
    
    return this._maxDepth; // int
  },

  isBetterAspectRatio : function(width, hight)
  {
    // summary:
    //		Determines whether the input aspect ratio is better than the current
    //		best aspect ratio. 
    // description:
    //		Determines whether the input aspect ratio is better than the current
    //		best aspect ratio. 
    //		Returns <code>true</code> if the current situation is better than
    //		the best previous situation.
    // tags:
    //		private
    
    var a;

    if (width == 0.0 || hight == 0.0) {
      return (false); // Boolean
    }
    else if (width / hight < this.getAspectRatio()) {
      a = hight * hight * this.getAspectRatio();
    }
    else {
      a = width * width / this.getAspectRatio();
    }
    if (a < this._bestArea)
    {
      this._bestArea = a;
      return (true); // Boolean
    }
    else {
      return (false); // Boolean
    }
  },

  storeOriginalAlignmentStyle : function()
  {
    // summary:
    //		Stores the current alignment style as original alignment style for
    //		all nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().storeOriginalAlignmentStyle();
  },

  restoreOriginalAlignmentStyle : function()
  {
    // summary:
    //		Restores the original alignment style for all nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().restoreOriginalAlignmentStyle();
  },

  storeBestAlignmentStyle : function()
  {
    // summary:
    //		Stores the current alignment style as best alignment style for all
    //		nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().storeBestAlignmentStyle();
  },

  restoreBestAlignmentStyle : function()
  {
    // summary:
    //		Restores the best alignment style for all nodes.
    // tags:
    //		private
    
    var iter = this.getGraph().getNodes();

    while (iter.hasNext())
      iter.next().restoreBestAlignmentStyle();
  },

  doLayout : function()
  {
    // summary:
    //		Perform one layout and check aspect ratio afterwards. 
    // description:
    //		Perform one layout and check aspect ratio afterwards. 
    //		After that it restores the original situation again.
    // tags:
    //		private
    
    var boundingBox;
    var graph = this.getGraph();
    graph.calcCoords();
    var a = graph.calcBoundingBox();
    boundingBox = graph.calcBoundingBox();
    var w = boundingBox.width;
    var h = boundingBox.height;
    if (this.isBetterAspectRatio(w, h)) {
      this.storeBestAlignmentStyle();
    }
    this.restoreOriginalAlignmentStyle();
  },

  run : function(mode)
  {
    // summary:
    //		Performs several layouts with different local tip-over alignment
    //		strategies. 
    // description:
    //		Performs several layouts with different local tip-over alignment
    //		strategies. 
    //		It determines which one has the best space characteristics at given
    //		aspect ration. Finally, it sets to the best layout.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this._maxDepth = graph.getMaxDepth();

    this.storeOriginalAlignmentStyle();

    var numberOfTries = 3;

    switch (mode)
    {
      case AlignmentOptions.TIP_OVER:
        numberOfTries = 8;
        break;
      case TLayoutModes.TIP_LEAVES_OVER:
      case TLayoutModes.TIP_ROOTS_OVER:
        numberOfTries = this._maxDepth + 3;
        break;
      case TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER:
        numberOfTries = Math.floor(this._maxDepth * (this._maxDepth - 1) / 2) + 3;
        break;
    }
    
    this.startStep(0.0, 1);
    this.setStepEstimation(6 + numberOfTries * 2);
    graph.changePercParameterForAutoTipOverLayout(numberOfTries);

    this.doLayout();
    if (!this.mayContinue()) {
      return;
    }
    if (this.tipAllOver()) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    switch (mode)
    {
      case AlignmentOptions.TIP_OVER:
        this.fastTipOver();
        break;
      case TLayoutModes.TIP_LEAVES_OVER:
        this.fromLeavesToRoot();
        break;
      case TLayoutModes.TIP_ROOTS_OVER:
        this.fromRootToLeaves();
        break;
      case TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER:
        this.fromRootAndLeavesToInner();
        break;
    }
    
    this.restoreBestAlignmentStyle();
    graph.calcCoords();
  },

  tipAllOver : function()
  {
    // summary:
    //		Tip all nodes over. 
    // description:
    //		Tip all nodes over. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;
    var graph = this.getGraph();
    var node;
    var iter = graph.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();

      if (node.canAutoTipOver())
      {
        changed = true;
        node.setAlignmentStyle(AlignmentOptions.TIP_OVER);
      }
    }

    return (changed); // Boolean
  },

  fromLeavesToRoot : function (a0)
  {
    // summary:
    //		Tips over the leaves and their ancestors up to a certain level. 
    // description:
    //		Tips over the leaves and their ancestors up to a certain level. 
    //		It tries all levels.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.fromLeavesToRoot_overload_0();
    else if(arguments.length == 1)
      return this.fromLeavesToRoot_overload_1(a0);
  },
  
  fromLeavesToRoot_overload_0 : function()
                                          
  {
    var graph = this.getGraph();

    for (var i = 0; i < this.getMaxDepth(); i++)
    {
      if (this.fromLeavesToRoot(i)) {
        this.doLayout();
      }
      else {
        graph.correctPercentageForTipOver();
      }
      if (!this.mayContinue()) {
        return;
      }
    }
  },

  fromLeavesToRoot_overload_1 : function(tipOverLevel)
  {
    // summary:
    //		Tip over all nodes at depth starting at tip-over-level. 
    // description:
    //		Tip over all nodes at depth starting at tip-over-level. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    var iter = this.getGraph().getRoots();

    while (iter.hasNext())
    {
      changed = this.tipOverAtDepth(iter.next(), 0, -1, tipOverLevel) || changed;
    }

    return (changed); // Boolean
  },

  fromRootToLeaves : function (a0)
  {
    // summary:
    //		Tips over the root and their descendents up to a certain level. 
    // description:
    //		Tips over the root and their descendents up to a certain level. 
    //		It tries all levels.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.fromRootToLeaves_overload_0();
    else if(arguments.length == 1)
      return this.fromRootToLeaves_overload_1(a0);
  },
  
  fromRootToLeaves_overload_0 : function()
                                          
  {
    var graph = this.getGraph();

    for (var i = 0; i < this.getMaxDepth(); i++)
    {
      if (this.fromRootToLeaves(i)) {
        this.doLayout();
      }
      else {
        graph.correctPercentageForTipOver();
      }
      if (!this.mayContinue()) {
        return;
      }
    }
  },

  fromRootToLeaves_overload_1 : function(tipOverLevel)
  {
    // summary:
    //		Tip over all nodes at depth lower than tip-over-level. 
    // description:
    //		Tip over all nodes at depth lower than tip-over-level. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    var iter = this.getGraph().getRoots();

    while (iter.hasNext())
    {
      changed = this.tipOverAtDepth(iter.next(), 0, tipOverLevel, this.getMaxDepth() + 1) || changed;
    }

    return (changed); // Boolean
  },

  fromRootAndLeavesToInner : function (a0, a1)
  {
    // summary:
    //		Tips over the root and their descendents up to a certain level, and
    //		the leaves and their ancestors starting from a certain level. 
    // description:
    //		Tips over the root and their descendents up to a certain level, and
    //		the leaves and their ancestors starting from a certain level. 
    //		It tries all levels.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.fromRootAndLeavesToInner_overload_0();
    else if(arguments.length == 2)
      return this.fromRootAndLeavesToInner_overload_2(a0, a1);
  },
  
  fromRootAndLeavesToInner_overload_0 : function()
                                                  
  {
    var graph = this.getGraph();
    var i;
    var j;

    for (i = -1; i < this.getMaxDepth() - 1; i++)
    {
      for (j = i + 2; j <= this.getMaxDepth(); j++)
      {
        if (this.fromRootAndLeavesToInner(i, j)) {
          this.doLayout();
        }
        else {
          graph.correctPercentageForTipOver();
        }
        if (!this.mayContinue()) {
          return;
        }
      }
    }
  },

  fromRootAndLeavesToInner_overload_2 : function(tipOverRootLevel, tipOverLeavesLevel)
  {
    // summary:
    //		Tip over all nodes at depth 0 to tipOverRootLevel and at depth
    //		tipOverLeavesLevel to maxDepth. 
    // description:
    //		Tip over all nodes at depth 0 to tipOverRootLevel and at depth
    //		tipOverLeavesLevel to maxDepth. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    var iter = this.getGraph().getRoots();

    while (iter.hasNext())
    {
      changed = this.tipOverAtDepth(iter.next(), 0, tipOverRootLevel, tipOverLeavesLevel) || changed;
    }

    return (changed); // Boolean
  },

  fastTipOver : function()
  {
    // summary:
    //		Fast compromise. 
    // description:
    //		Fast compromise. 
    //		Doesn't try all levels, but only the first and second root and leaf level.
    // tags:
    //		private
    
    var graph = this.getGraph();
    if (this.fromLeavesToRoot(this.getMaxDepth() - 1)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromLeavesToRoot(this.getMaxDepth() - 2)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromRootToLeaves(0)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromRootToLeaves(1)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
    if (this.fromRootAndLeavesToInner(0, this.getMaxDepth() - 1)) {
      this.doLayout();
    }
    else {
      graph.correctPercentageForTipOver();
    }
    if (!this.mayContinue()) {
      return;
    }
  },

  tipOverAtDepth : function(node, depth, tipOverRootLevel, tipOverLeavesLevel)
  {
    // summary:
    //		Tip over all nodes at depth lower than 'tipOverRootLevel'
    //		or higher than 'tipOverLeavesLevel'. 
    // description:
    //		Tip over all nodes at depth lower than <code>tipOverRootLevel</code>
    //		or higher than <code>tipOverLeavesLevel</code>. 
    //		Returns <code>true</code> if this changes something.
    // tags:
    //		private
    
    var changed = false;

    if (node.canAutoTipOver())
    {
      if (depth >= tipOverLeavesLevel || depth <= tipOverRootLevel)
      {
        node.setAlignmentStyle(AlignmentOptions.TIP_OVER);
        changed = true;
      }
    }

    for (var i = 0; i < node.getNumberOfChildren(); i++)
    {
      changed = this.tipOverAtDepth(node.getChild(i), depth + 1, tipOverRootLevel, tipOverLeavesLevel) || changed;
    }

    return (changed); // Boolean
  }

}
);

return TAutoTipOver;
});

},
'ibm_ilog/graphlayout/tree/TLayoutModes':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TLayoutModes", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/TranslateUtil"	// #7
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    GraphModelUtil,	// #6
    TranslateUtil	// #7
    ) {

var TLayoutModes = declare("ibm_ilog.graphlayout.tree.TLayoutModes", null, 
{
  // summary:
  //		This class contains all enumeration constants used for the tree
  //		layout modes. 
  // description:
  //		This class contains all enumeration constants used for the tree
  //		layout modes. 
  //		It also contains some generic support for the tree layout modes.   
  // tags:
  //		private
  
  constructor : function()
  {
    
  }

}
);

lang.mixin(TLayoutModes,
{
  FREE : null,
  LEVEL : null,
  RADIAL : null,
  ALTERNATING_RADIAL : null,
  TIP_LEAVES_OVER : null,
  TIP_ROOTS_OVER : null,
  TIP_ROOTS_AND_LEAVES_OVER : null,
  BALLOON : null,

  TIP_OVER : null,

  toString : function(mode)
  {
    switch (mode)
    {
      case TLayoutModes.TIP_OVER:
        return ("TIP_OVER"); // String
      case TLayoutModes.TIP_LEAVES_OVER:
        return ("TIP_LEAVES_OVER"); // String
      case TLayoutModes.TIP_ROOTS_OVER:
        return ("TIP_ROOTS_OVER"); // String
      case TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER:
        return ("TIP_ROOTS_AND_LEAVES_OVER"); // String
      case TLayoutModes.FREE:
        return ("FREE"); // String
      case TLayoutModes.LEVEL:
        return ("LEVEL"); // String
      case TLayoutModes.RADIAL:
        return ("RADIAL"); // String
      case TLayoutModes.ALTERNATING_RADIAL:
        return ("ALTERNATING_RADIAL"); // String
      case TLayoutModes.BALLOON:
        return ("BALLOON"); // String
      default:
        throw new ArgumentError("Wrong layout mode " + mode); // String
    }
  },

  valueOf : function(s)
  {
    if (s == "TIP_OVER") {
      return (TLayoutModes.TIP_OVER); // int
    }
    if (s == "TIP_LEAVES_OVER") {
      return (TLayoutModes.TIP_LEAVES_OVER); // int
    }
    if (s == "TIP_ROOTS_OVER") {
      return (TLayoutModes.TIP_ROOTS_OVER); // int
    }
    if (s == "TIP_ROOTS_AND_LEAVES_OVER") {
      return (TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER); // int
    }
    if (s == "FREE") {
      return (TLayoutModes.FREE); // int
    }
    if (s == "LEVEL") {
      return (TLayoutModes.LEVEL); // int
    }
    if (s == "RADIAL") {
      return (TLayoutModes.RADIAL); // int
    }
    if (s == "ALTERNATING_RADIAL") {
      return (TLayoutModes.ALTERNATING_RADIAL); // int
    }
    if (s == "BALLOON") {
      return (TLayoutModes.BALLOON); // int
    }
    throw new ArgumentError("Wrong layout mode " + s); // int
  },

  _fields_init : function(c)
  {
    c = c || TLayoutModes;
    
    c.FREE = 0;
    c.LEVEL = 1;
    c.RADIAL = 2;
    c.ALTERNATING_RADIAL = 3;
    c.TIP_LEAVES_OVER = 4;
    c.TIP_ROOTS_OVER = 5;
    c.TIP_ROOTS_AND_LEAVES_OVER = 6;
    c.BALLOON = 7;
    c.TIP_OVER = AlignmentOptions.TIP_OVER;
  }
  
}
);

(function(){ TLayoutModes._fields_init(); })();

return TLayoutModes;
});

},
'ibm_ilog/graphlayout/tree/TRadial':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TRadial", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil",	// #6
  "../tree/TGraphAlgorithm",	// #7
  "../tree/TNode"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil,	// #6
    TGraphAlgorithm,	// #7
    TNode	// #8
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var TGraphAlgorithm = ibm_ilog.graphlayout.tree.TGraphAlgorithm;
=====*/

var TRadial = declare("ibm_ilog.graphlayout.tree.TRadial", TGraphAlgorithm, 
{
  // summary:
  //		This algorithm class contains the support for converting a normal
  //		tree layout into a radial or elliptic tree layout. 
  // description:
  //		This algorithm class contains the support for converting a normal
  //		tree layout into a radial or elliptic tree layout. 
  //		The main entry point for one subtree is <code>run</code>. However, if
  //		the layout is elliptic (aspectRatio != 1.0), the method
  //		<code>setEllipsoidTransTable</code> must be called once before that
  //		to set the translation table. This table can be reused for multiple
  //		calls of run, as long as the aspect ratio doesn't change.  
  // tags:
  //		private
  
  _root: null,

  _angleTransTable: null,

  _dir0: null,
  _dir1: null,

  _levelJustification: null,
  _aspectRatio: null,
  _minDistBetweenNodes: null,
  _minDistBetweenBranches: null,
  _allowAlternating : null,
  _maxChildrenAngle: null,
  _firstCircleEvenlySpaced: null,
  _allLevelsAlternating: null,
  _maxAlternatingCircles: null,

  _maxDepth: null,
  _width: null,
  _needsRetry: null,

  _firstNode: null,
  _lastNode: null,
  _layerSize: null,
  _minNodeDist: null,
  _desiredMinDist: null,
  _smallestWidth: null,
  _altInnerSmallestWidth: null,
  _altOuterSmallestWidth: null,
  _smallestAltWidth: null,
  _considerAlternating: null,
  _maxChildrenSpan: null,
  _bestNumCircles: null,

  constructor : function(graph, maxChildrenAngle)
  {
    this._allowAlternating = true;
    this._maxChildrenAngle = maxChildrenAngle;
    this._angleTransTable = null;
    this._root = null;
    this._dir0 = graph.getDirTowardsSiblings();
    this._dir1 = graph.getDirTowardsLeaves();
    this._levelJustification = graph.getLevelJustification();
    this._aspectRatio = graph.getAspectRatio();
;
    this._firstCircleEvenlySpaced = graph.getLayout().isFirstCircleEvenlySpacing();
    this._allLevelsAlternating = graph.getLayout().isAllLevelsAlternating();
    this._maxAlternatingCircles = graph.getLayout().getNumberOfAlternatingCircles();
    if (this._maxAlternatingCircles == 0) {
      this._allLevelsAlternating = false;
    }
    this._minDistBetweenNodes = TranslateUtil.newArray(2, 0);
    this._minDistBetweenNodes[0] = graph.getMinDistBetweenNodes(0);
    this._minDistBetweenNodes[1] = graph.getMinDistBetweenNodes(1);
    this._minDistBetweenBranches = TranslateUtil.newArray(2, 0);
    this._minDistBetweenBranches[0] = graph.getMinDistBetweenBranches(0);
    this._minDistBetweenBranches[1] = graph.getMinDistBetweenBranches(1);
    this._allowAlternating = graph.allowAlternating();
    this._maxDepth = graph.getMaxDepth();
    this._width = 0.0;
    this._needsRetry = false;
    this._firstNode = null;
    this._lastNode = null;
    this._layerSize = null;
    this._minNodeDist = null;
    this._desiredMinDist = null;
    this._smallestWidth = null;
    this._altInnerSmallestWidth = null;
    this._altOuterSmallestWidth = null;
    this._smallestAltWidth = null;
    this._considerAlternating = null;
    this._maxChildrenSpan = null;
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.inherited("disposeIt", arguments, []);

    this._root = null;

    this._angleTransTable = null;
    this._minDistBetweenNodes = null;
    this._minDistBetweenBranches = null;
    this._firstNode = null;
    this._lastNode = null;
    this._layerSize = null;
    this._minNodeDist = null;
    this._desiredMinDist = null;
    this._smallestWidth = null;
    this._altInnerSmallestWidth = null;
    this._altOuterSmallestWidth = null;
    this._considerAlternating = null;
    this._maxChildrenSpan = null;
  },

  getRoot : function()
  {
    // summary:
    //		Returns the current root.
    // tags:
    //		private
    
    return this._root; // ibm_ilog.graphlayout.tree.TNode
  },

  getDir0 : function()
  {
    // summary:
    //		Returns the index of the stretch direction (this is normally the
    //		x-direction in top-down layout). 
    // description:
    //		Returns the index of the stretch direction (this is normally the
    //		x-direction in top-down layout). 
    //		The x direction has index 0, and y-direction has index 1.
    // tags:
    //		private
    
    return this._dir0; // int
  },

  getDir1 : function()
  {
    // summary:
    //		Returns the index of the opposite stretch direction (this is
    //		normally the y-direction in top-down layout). 
    // description:
    //		Returns the index of the opposite stretch direction (this is
    //		normally the y-direction in top-down layout). 
    //		The x direction has index 0, and y-direction has index 1.
    // tags:
    //		private
    
    return this._dir1; // int
  },

  getLevelJustification : function()
  {
    // summary:
    //		Returns level justification of the nodes within the levels. 
    // description:
    //		Returns level justification of the nodes within the levels. 
    //		-1 for justify the nodes to the border with smaller coordinate, 1
    //		for justify the nodes to the border with larger coordinate, and 0
    //		for justify the nodes to their centers.
    // tags:
    //		private
    
    return this._levelJustification; // int
  },

  getMinDistBetweenNodes : function(direction)
  {
    // summary:
    //		Returns the minimal distance between nodes in the given direction. 
    // description:
    //		Returns the minimal distance between nodes in the given direction. 
    //		0 means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    return this._minDistBetweenNodes[direction % 2]; // Number
  },

  getMinDistBetweenBranches : function(direction)
  {
    // summary:
    //		Returns the minimal distance between different branches of the tree
    //		in the given direction. 
    // description:
    //		Returns the minimal distance between different branches of the tree
    //		in the given direction. 
    //		0 means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    return this._minDistBetweenBranches[direction % 2]; // Number
  },

  getAspectRatio : function()
  {
    // summary:
    //		Returns the aspect ratio.
    // tags:
    //		private
    
    return this._aspectRatio; // Number
  },

  allowAlternating : function()
  {
    // summary:
    //		Returns whether we allow alternating children positioning. 
    // description:
    //		Returns whether we allow alternating children positioning. 
    //		In this mode, the nodes of one layer are distributed onto two
    //		circles instead of one, which results in a better space
    //		characteristic.
    // tags:
    //		private
    
    return this._allowAlternating; // Boolean
  },

  forceAlternating : function()
  {
    // summary:
    //		Returns whether we force alternating children positioning. 
    // description:
    //		Returns whether we force alternating children positioning. 
    //		In this mode, the nodes of one layer are distributed onto two
    //		circles instead of one, which results in a better space
    //		characteristic.
    // tags:
    //		private
    
    return this._allLevelsAlternating; // Boolean
  },

  getMaxChildrenAngle : function()
  {
    // summary:
    //		Returns the maximal angle of the circle segment that children of the
    //		same nonroot parent can cover. 
    // description:
    //		Returns the maximal angle of the circle segment that children of the
    //		same nonroot parent can cover. 
    //		If unrestricted, it returns 0.
    // tags:
    //		private
    
    return this._maxChildrenAngle; // Number
  },

  isFirstCircleEvenlySpaced : function()
  {
    // summary:
    //		Returns true if the first circle must be evenly spaced.
    // tags:
    //		private
    
    return this._firstCircleEvenlySpaced; // Boolean
  },

  run : function(root, border0, border1)
  {
    // summary:
    //		Performs a polar transformation on the layered tree layout so that
    //		the root node is in the center and the nodes are in (potential
    //		elliptic) circles around the root.  
    // root: TNode
    //		The root node.
    // border0: Number
    //		The top left border of this tree (coordinate wrt. dir0).
    // border1: Number
    //		The top left border of this tree (coordinate wrt. dir1).
    // returns:
    //		The maximal coordinate of the layout in direction dir0.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this._root = root;
    this._maxDepth = graph.getMaxDepth() + 1;
    this._needsRetry = false;

    this._desiredMinDist = null;

    this.calcNonRadialLayout();

    this.buildLayerTable();

    if (this._needsRetry)
    {
      this.incrPercPointEstimation(this.getPercController().getPoints() + graph.getNumberOfNodes());

      this.getGraph().setDesiredMinDistTable(this._desiredMinDist);

      var dir1 = this.getDir1();
      var node;
      var lastNode;

      var delta = 10.0 + 2 * this.getMinDistBetweenNodes(dir1);
      var coord = 0.0;

      for (var i = 0; i < this._maxDepth; i++)
      {
        node = lastNode = this._lastNode[i];
        coord += this._layerSize[i] + delta;

        while (node != null)
        {
          node.setMaxCoord(dir1, coord);
          node = node.getPrevNode();
          if (node == lastNode) {
            break;
          }
        }
      }

      this.getRoot().initPrevSiblings();

      this.getGraph().calcSubtreeCoords(this.getRoot());

      this._maxDepth = this.getGraph().getMaxDepth() + 1;
      this.calcNonRadialLayout();
      this.buildLayerTable();
    }

    if (this.allowAlternating())
    {
      if (this._maxAlternatingCircles == 0 && !this.forceAlternating()) {
        this.optimizeAlternatingRadialMultiCircles();
      }
      if (this._maxAlternatingCircles == 2)
      {
        this.markAlternating(false);
        this.calcAlternatingSmallestWidths();
      }
      else
      {
        this.markAlternating(true);
        this.calcAlternatingSmallestWidthsMultiCircle();
      }
    }

    return (this.calcFinalCoordRadial(root, border0, border1)); // Number
  },

  calcNonRadialLayout : function()
  {
    // summary:
    //		Finalize the nonradial layout for the tree starting at root. 
    // description:
    //		Finalize the nonradial layout for the tree starting at root. 
    //		It converts the relative coordinates into absolute, nonpolar
    //		coordinates. This looks just like a normal tree layout. The tree is
    //		shifted such that, e.g., for a top-down layout (if dir0 == 0) the
    //		left border is set to zero. This also sets the width of the layout.
    // tags:
    //		private
    
    var dir0 = this.getDir0();
    var bounds = TranslateUtil.newArray(2, 0);
    bounds[0] = /*Number.MAX_VALUE*/(3.4028235E38);
    bounds[1] = -/*Number.MAX_VALUE*/(3.4028235E38);
    var root = this.getRoot();
    this.getGraph().calcFinalCoordNormal(root, 0.0, 0.0, bounds);
    if (this.isFirstCircleEvenlySpaced()) {
      this.getGraph().shiftCoordToEqualSpacingAtRoot(root, bounds);
    }
    var dx = ((dir0 == 0 ? -bounds[0] : 0.0));
    var dy = ((dir0 == 1 ? -bounds[0] : 0.0));
    this.getGraph().shiftSubtreeBy(this.getRoot(), dx, dy);
    this._width = (bounds[1] - bounds[0]);
    if (!this.isFirstCircleEvenlySpaced()) {
      this._width += this.getMinDistBetweenBranches(dir0);
    }
  },

  markAlternating : function (a0, a1)
  {
    // summary:
    //		For alternating layout, mark at each layer each second node. 
    // description:
    //		For alternating layout, mark at each layer each second node. 
    //		We use the sort value for this marker and set it alternating from
    //		0.0  to n (i.e. for 2 alternating circles from 0.0 to 1.0).
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.markAlternating_overload_1(a0);
    else if(arguments.length == 2)
      return this.markAlternating_overload_2(a0, a1);
  },
  
  markAlternating_overload_1 : function(useMultiCircles)
                                                        
  {
    for (var i = 0; i < this._maxDepth; i++)
    {
      this.markAlternating(i, (useMultiCircles ? this.getNumAltLevels(i) : 2));

      this.addPercPoints(1);
      this.layoutStepPerformed();
    }
  },

  markAlternating_overload_2 : function(level, numCircles)
  {
    // summary:
    //		This does the work for one level.
    // tags:
    //		private
    
    var node;
    var prevNode;
    var lastNode;
    var startNode;
    var mark;
    var largestgap;
    var gap;
    var dir0 = this.getDir0();
    node = lastNode = startNode = this._lastNode[level];
    prevNode = ((node != null ? node.getPrevNode() : null));
    largestgap = 0.0;

    while (node != null && prevNode != null)
    {
      gap = node.getMinCoord(dir0) - prevNode.getMaxCoord(dir0);

      if (gap > largestgap)
      {
        startNode = prevNode;
        largestgap = gap;
      }

      node = prevNode;
      prevNode = node.getPrevNode();
      if (node == lastNode) {
        break;
      }
    }

    var n = numCircles;
    node = lastNode = startNode;
    mark = 0.0;

    while (node != null)
    {
      node.setSortValue(mark);
      mark += 1.0;
      if (mark == n) {
        mark = 0.0;
      }
      node = node.getPrevNode();
      if (node == lastNode) {
        break;
      }
    }
  },

  buildLayerTable : function()
  {
    // summary:
    //		Calculates the layer table with information about each layer. 
    // description:
    //		Calculates the layer table with information about each layer. 
    //		This misuses the prev-sibling pointer to chain all nodes circular
    //		together. As a side effect it reinitializes the nodes for the case
    //		that we need to relayout. The flag &#95;needsRetry becomes true if it
    //		makes sense to retry a normal layout with modified parameters.
    // tags:
    //		private
    
    var i;
    var h;
    var dir0 = this.getDir0();
    var dir1 = this.getDir1();
    var minDist0 = this.getMinDistBetweenNodes(dir0);
    var minDist1 = this.getMinDistBetweenNodes(dir1);

    this._firstNode = TranslateUtil.newArray(this._maxDepth, null);
    this._lastNode = TranslateUtil.newArray(this._maxDepth, null);
    this._layerSize = TranslateUtil.newArray(this._maxDepth, 0);
    this._minNodeDist = TranslateUtil.newArray(this._maxDepth, 0);
    if (this.getMaxChildrenAngle() != 0.0) {
      this._maxChildrenSpan = TranslateUtil.newArray(this._maxDepth, 0);
    }
    for (i = 0; i < this._maxDepth; i++)
    {
      this._minNodeDist[i] = /*Number.MAX_VALUE*/(3.4028235E38);
    }

    this.buildLayerTableRecursive(this.getRoot(), 0);

    for (i = 0; i < this._maxDepth; i++)
    {
      if (this._firstNode[i] != null && this._firstNode[i] != this._lastNode[i])
      {
        this._firstNode[i].setPrevNode(this._lastNode[i]);
        h = this._width + this._firstNode[i].getMinCoord(dir0) - this._lastNode[i].getMaxCoord(dir0);

        if (h < this._minNodeDist[i])
        {
          if (this._desiredMinDist != null && h < this._desiredMinDist[i])
          {
            this._width += (this._desiredMinDist[i] - h);
            this._minNodeDist[i] = this._desiredMinDist[i];
          }
          else {
            this._minNodeDist[i] = h;
          }
        }
      }
    }

    this.calcSmallestWidths();

    this.calcMaxChildrenSpan();

    if (this._desiredMinDist == null) {
      this._desiredMinDist = TranslateUtil.newArray(this._maxDepth, 0);
    }
    for (i = 0; i < this._maxDepth; i++)
      this._desiredMinDist[i] = minDist0;

    var C;
    var radius = 0.0;
    var delta;
    var smallestWidth;

    for (i = 1; i < this._maxDepth; i++)
    {
      smallestWidth = this._smallestWidth[i];
      delta = 0.5 * (this._layerSize[i - 1] + this._layerSize[i]) + minDist1;

      if (this._maxChildrenSpan != null)
      {
        var d = this.maxChildrenAngleDelta(radius, this._maxChildrenSpan[i]);
        if (d > delta) {
          delta = d;
        }
      }

      if (this.getAspectRatio() < 1.0) {
        radius += delta / this.getAspectRatio();
      }
      else {
        radius += delta;
      }
      C = TRadial.circumference(radius, this.getAspectRatio());

      if (C < smallestWidth)
      {
        h = TRadial.radiusFromCircumference(smallestWidth, this.getAspectRatio());

        if (h - radius > 1.3 * delta)
        {
          this._desiredMinDist[i] *= this._width / C;
          this._needsRetry = true;
        }
        else
        {
          radius = h;
        }
      }
    }
  },

  buildLayerTableRecursive : function(node, depth)
  {
    // summary:
    //		This does recursivley the real work for the layer table calculation.
    //		
    // description:
    //		This does recursivley the real work for the layer table calculation.
    //		
    //		It calculates the first node and last node per layer, the layer size
    //		and the minimal distance between a pair of nodes.
    // tags:
    //		private
    
    this.addPercPoints(1);
    this.layoutStepPerformed();

    var size = node.getSize(this.getDir1());
    if (size > this._layerSize[depth]) {
      this._layerSize[depth] = size;
    }
    if (this._firstNode[depth] == null) {
      this._firstNode[depth] = node;
    }
    if (this._lastNode[depth] != null)
    {
      var dir = this.getDir0();
      var h = node.getMinCoord(dir) - this._lastNode[depth].getMaxCoord(dir);
      if (h < this._minNodeDist[depth]) {
        this._minNodeDist[depth] = h;
      }
    }

    node.reinitialize();
    node.setPrevNode(this._lastNode[depth]);
    this._lastNode[depth] = node;

    var iter = node.getVirtualChildren();

    while (iter.hasNext())
      this.buildLayerTableRecursive(iter.next(), depth + 1);
  },

  calcSmallestWidths : function()
  {
    // summary:
    //		Calculate the table of smallest possible width for each layer. 
    // description:
    //		Calculate the table of smallest possible width for each layer. 
    //		We want to scale the width by shrinking all spaces between nodes
    //		such that the smallest space between a pair of nodes is just the
    //		minimal distance allowed between nodes.
    // tags:
    //		private
    
    this._smallestWidth = TranslateUtil.newArray(this._maxDepth, 0);
    this._smallestWidth[0] = 0.0;
    var dir0 = this.getDir0();
    var minSDist0 = this.getMinDistBetweenNodes(dir0);
    var minBDist0 = this.getMinDistBetweenBranches(dir0);
    var factor;
    var f;
    var wn1;
    var wn2;
    var wsp;
    var node;
    var prevNode;
    var lastNode;

    for (var i = 1; i < this._maxDepth; i++)
    {
      factor = 0.0001;
      node = lastNode = this._lastNode[i];
      prevNode = ((node != null ? node.getPrevNode() : null));

      if (prevNode != null && prevNode != node)
      {
        do
        {
          wn1 = 0.5 * node.getSize(dir0);
          wn2 = 0.5 * prevNode.getSize(dir0);
          wsp = node.getMinCoord(dir0) - prevNode.getMaxCoord(dir0);

          if (wsp < 0.0) {
            wsp += this._width;
          }
          if (node.getImmediateParent() == prevNode.getImmediateParent()) {
            f = (minSDist0 + wn1 + wn2) / (wsp + wn1 + wn2);
          }
          else {
            f = (minBDist0 + wn1 + wn2) / (wsp + wn1 + wn2);
          }
          if (f > factor) {
            factor = f;
          }
          if (factor >= 1.0) {
            break;
          }
          node = prevNode;
          prevNode = node.getPrevNode();
        }
        while (node != lastNode);
      }

      if (factor > 1.0) {
        factor = 1.0;
      }
      this._smallestWidth[i] = this._width * factor;

      this.addPercPoints(1);
      this.layoutStepPerformed();
    }
  },

  calcAlternatingSmallestWidths : function()
  {
    // summary:
    //		Calculate the table of smallest possible width for each layer with
    //		an alternating layout onto two circles. 
    // description:
    //		Calculate the table of smallest possible width for each layer with
    //		an alternating layout onto two circles. 
    //		This is the same routine as above, except that we only have to
    //		consider each second node for the one, or for the other circle.
    // tags:
    //		private
    
    this._altInnerSmallestWidth = TranslateUtil.newArray(this._maxDepth, 0);
    this._altOuterSmallestWidth = TranslateUtil.newArray(this._maxDepth, 0);
    this._considerAlternating = TranslateUtil.newArray(this._maxDepth, false);
    this._altInnerSmallestWidth[0] = 0.0;
    this._altOuterSmallestWidth[0] = 0.0;
    var dir0 = this.getDir0();
    var count;
    var minDist0 = this.getMinDistBetweenNodes(dir0);
    var innerFactor;
    var outerFactor;
    var f;
    var wn1;
    var wn2;
    var wsp;
    var node;
    var prevNode;
    var prevPrevNode;
    var lastNode;
    var i;

    for (i = 1; i < this._maxDepth; i++)
    {
      innerFactor = outerFactor = 0.0001;
      node = lastNode = this._lastNode[i];
      prevNode = ((node != null ? node.getPrevNode() : null));
      prevPrevNode = ((prevNode != null ? prevNode.getPrevNode() : null));
      this._considerAlternating[i] = false;

      if (prevPrevNode != null && prevPrevNode != node)
      {
        count = 0;

        do
        {
          if (node.getSortValue() != prevNode.getSortValue())
          {
            wn1 = 0.5 * node.getSize(dir0);
            wn2 = 0.5 * prevPrevNode.getSize(dir0);
            wsp = node.getMinCoord(dir0) - prevPrevNode.getMaxCoord(dir0);

            if (wsp < 0.0) {
              wsp += this._width;
            }
          }
          else
          {
            wn1 = 0.5 * node.getSize(dir0);
            wn2 = 0.5 * prevNode.getSize(dir0);
            wsp = node.getMinCoord(dir0) - prevNode.getMaxCoord(dir0);

            if (wsp < 0.0) {
              wsp += this._width;
            }
          }

          count++;

          f = (minDist0 + wn1 + wn2) / (wsp + wn1 + wn2);

          if (node.getSortValue() == 0)
          {
            if (f <= 1.0 && f > outerFactor) {
              outerFactor = f;
            }
          }
          else
          {
            if (f <= 1.0 && f > innerFactor) {
              innerFactor = f;
            }
          }

          node = prevNode;
          prevNode = prevPrevNode;
          prevPrevNode = prevNode.getPrevNode();
        }
        while (node != lastNode);
        if (count > 4) {
          this._considerAlternating[i] = true;
        }
      }

      this._altInnerSmallestWidth[i] = this._width * innerFactor;
      this._altOuterSmallestWidth[i] = this._width * outerFactor;

      if (this._altInnerSmallestWidth[i] >= this._smallestWidth[i] || this._altOuterSmallestWidth[i] >= this._smallestWidth[i]) {
        this._considerAlternating[i] = false;
      }
      this.addPercPoints(1);
      this.layoutStepPerformed();
    }

    if (this.forceAlternating()) {
      for (i = 1; i < this._maxDepth; i++)
        this._considerAlternating[i] = true;
    }
    this._considerAlternating[0] = false;
  },

  calcAlternatingSmallestWidthsMultiCircle : function (a0, a1)
  {
    // summary:
    //		Calculate the table of smallest possible width for each layer with
    //		an alternating layout onto n circles. 
    // description:
    //		Calculate the table of smallest possible width for each layer with
    //		an alternating layout onto n circles. 
    //		This is the same routine as above, except that it works for multiple
    //		circle per level.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcAlternatingSmallestWidthsMultiCircle_overload_0();
    else if(arguments.length == 2)
      return this.calcAlternatingSmallestWidthsMultiCircle_overload_2(a0, a1);
  },
  
  calcAlternatingSmallestWidthsMultiCircle_overload_0 : function()
                                                                  
  {
    this._smallestAltWidth = TranslateUtil.newArray(this._maxDepth, 0);
    this._considerAlternating = TranslateUtil.newArray(this._maxDepth, false);

    for (var i = 1; i < this._maxDepth; i++)
    {
      this.calcAlternatingSmallestWidthsMultiCircle(i, this.getNumAltLevels(i));

      this.addPercPoints(1);
      this.layoutStepPerformed();
    }

    this._considerAlternating[0] = false;
  },

  calcAlternatingSmallestWidthsMultiCircle_overload_2 : function(level, numCircles)
  {
    // summary:
    //		Calculate the table of smallest possible width for each layer with
    //		an alternating layout onto n circles. 
    // description:
    //		Calculate the table of smallest possible width for each layer with
    //		an alternating layout onto n circles. 
    //		This is the same routine as above, except that it works for multiple
    //		circle per level. This does the work for one level.
    // tags:
    //		private
    
    if (level == 0)
    {
      this._smallestAltWidth[0] = TranslateUtil.newArray(1, 0);
      this._smallestAltWidth[0][0] = 0.0;
      this._considerAlternating[0] = false;
      return;
    }

    var n = numCircles;
    this._smallestAltWidth[level] = TranslateUtil.newArray(n, 0);
    var dir0 = this.getDir0();
    var minDist0 = this.getMinDistBetweenNodes(dir0);
    var f;
    var wn1;
    var wn2;
    var wsp;
    var node;
    var lastNode;
    var j;

    var nextNode = TranslateUtil.newArray(n, null);
    var factors = TranslateUtil.newArray(n, 0);

    for (j = 0; j < n; j++)
    {
      factors[j] = 0.0001;
      nextNode[j] = null;
    }

    node = lastNode = this._lastNode[level];

    if (node == null)
    {
      this._considerAlternating[level] = false;
      return;
    }
    else
    {
      this._considerAlternating[level] = true;
    }

    do
    {
      var circleIndex = Math.floor(node.getSortValue());

      if (nextNode[circleIndex] != null && nextNode[circleIndex] != node)
      {
        wn1 = 0.5 * node.getSize(dir0);
        wn2 = 0.5 * nextNode[circleIndex].getSize(dir0);
        wsp = nextNode[circleIndex].getMinCoord(dir0) - node.getMaxCoord(dir0);

        if (wsp < 0.0) {
          wsp += this._width;
        }
        f = (minDist0 + wn1 + wn2) / (wsp + wn1 + wn2);
        if (f <= 1.0 && f > factors[circleIndex]) {
          factors[circleIndex] = f;
        }
      }
      else if (nextNode[circleIndex] == null)
      {
        lastNode = node;
      }

      nextNode[circleIndex] = node;
      node = node.getPrevNode();
    }
    while (node != null && node != lastNode);
    for (j = 0; j < n; j++)
    {
      this._smallestAltWidth[level][j] = this._width * factors[j];
    }

    if (n < 2)
    {
      this._considerAlternating[level] = false;
    }
    else if (this.forceAlternating())
    {
      this._considerAlternating[level] = true;
    }
    else
    {
      for (j = 0; j < n; j++)
      {
        if (this._smallestAltWidth[level][j] >= this._smallestWidth[level]) {
          this._considerAlternating[level] = false;
        }
      }
    }
  },

  getNumAltLevels : function(level)
  {
    // summary:
    //		Returns the number of alternating circles for the given level.
    // tags:
    //		private
    
    if (level == 0) {
      return 1; // int
    }
    var node = this._lastNode[level];

    if (node == null)
    {
      return 0; // int
    }

    var lastnode = node;
    var count = 0;

    do
    {
      count++;
      node = node.getPrevNode();
    }
    while (node != null && node != lastnode);
    var desired = this._maxAlternatingCircles;

    if (this._bestNumCircles != null)
    {
      desired = this._bestNumCircles[level];
    }

    if (!this.forceAlternating())
    {
      while (count <= 2 * desired)
        desired--;
    }

    if (desired <= 0) {
      desired = 1;
    }
    return Math.min(desired, count); // int
  },

  calcMaxChildrenSpan : function()
  {
    // summary:
    //		If a maximal angle for children is given, calculate the largest span
    //		of siblings (children with the same parent). 
    // description:
    //		If a maximal angle for children is given, calculate the largest span
    //		of siblings (children with the same parent). 
    //		This is necessary to calculate later the smallest radius so that
    //		this angle can be satisfied. This is only necessary for the layers
    //		starting at 2, because layer 0 has the root, and layer 1 has the
    //		children of the root which are always unrestricted.
    // tags:
    //		private
    
    var maxAngle = this.getMaxChildrenAngle();

    if (maxAngle != 0.0)
    {
      var dir0 = this.getDir0();
      var node;
      var lastNode;
      var firstNodeOfParent;
      var lastNodeOfParent;
      var parent;
      var d;

      for (var i = 2; i < this._maxDepth; i++)
      {
        node = lastNode = this._lastNode[i];
        parent = ((node != null ? node.getImmediateParent() : null));
        firstNodeOfParent = lastNodeOfParent = node;
        node = ((node != null ? node.getPrevNode() : null));

        while (node != lastNode && node != null)
        {
          if (node.getImmediateParent() == parent)
          {
            lastNodeOfParent = node;
          }
          else
          {
            if (firstNodeOfParent != lastNodeOfParent)
            {
              d = firstNodeOfParent.getCenter(dir0) - lastNodeOfParent.getCenter(dir0);
              if (d > this._maxChildrenSpan[i]) {
                this._maxChildrenSpan[i] = d;
              }
            }

            parent = node.getImmediateParent();
            firstNodeOfParent = lastNodeOfParent = node;
          }

          node = node.getPrevNode();
        }

        if (firstNodeOfParent != lastNodeOfParent)
        {
          d = firstNodeOfParent.getCenter(dir0) - lastNodeOfParent.getCenter(dir0);
          if (d > this._maxChildrenSpan[i]) {
            this._maxChildrenSpan[i] = d;
          }
        }
      }
    }
  },

  optimizeAlternatingRadialMultiCircles : function()
  {
    // summary:
    //		We simply try 1 ... 
    // description:
    //		We simply try 1 ... 
    //		5 circles for each level, and keep that one that has the smallest
    //		radius.
    // tags:
    //		private
    
    this._bestNumCircles = TranslateUtil.newArray(this._maxDepth, 0);
    this._smallestAltWidth = TranslateUtil.newArray(this._maxDepth, 0);
    this._considerAlternating = TranslateUtil.newArray(this._maxDepth, false);
    var radius = 0;

    for (var i = 1; i < this._maxDepth; i++)
    {
      var best = Number.MAX_VALUE;
      var bestNumCircles = 1;

      for (var j = 1; j < 5; j++)
      {
        this.markAlternating(i, j);
        this.calcAlternatingSmallestWidthsMultiCircle(i, j);
        var r = this.getOuterRadiusMultiCircles(i - 1, radius);

        if (r < best)
        {
          best = r;
          bestNumCircles = j;
        }
      }

      this._bestNumCircles[i] = bestNumCircles;
      radius = best;
    }
  },

  getOuterRadiusMultiCircles : function(depth, radiusOfParent)
  {
    // summary:
    //		Returns the radius of one level. 
    // description:
    //		Returns the radius of one level. 
    //		Basically the same code as calcChildrenCoordRadial, but here we do
    //		it without placing the children, and it is not recursive. Calculates
    //		the radial coordinates of the children wrt. the parent. As side
    //		effect, it updates the bounds in dir0 that are passed as input.  
    // depth: int
    //		The layer index of the parent node.
    // radiusOfParent: Number
    //		The polar transformed coordinate of the parent,
    //		i.e. the radius between root and parent.
    // tags:
    //		private
    
    var dir1 = this.getDir1();
    var radius = 0.0;
    var delta = 0.0;
    var smallestWidth = this._width;
    var altSmallestWidth = this._width;
    var minDist1 = this.getMinDistBetweenNodes(dir1);
    var aRatio = this.getAspectRatio();
    var levelWidth = this._width;

    smallestWidth = this._smallestWidth[depth + 1];
    delta = 0.5 * (this._layerSize[depth] + this._layerSize[depth + 1]) + minDist1;

    if (this._maxChildrenSpan != null)
    {
      var d = this.maxChildrenAngleDelta(radiusOfParent, this._maxChildrenSpan[depth + 1]);
      if (d > delta) {
        delta = d;
      }
    }

    if (aRatio < 1.0) {
      radius = radiusOfParent + delta / aRatio;
    }
    else {
      radius = radiusOfParent + delta;
    }

    var C = TRadial.circumference(radius, aRatio);

    if (C > levelWidth)
    {
    }
    else if (C < smallestWidth)
    {
      var r1 = TRadial.radiusFromCircumference(smallestWidth, aRatio);

      if (this.allowAlternating() && this._considerAlternating[depth + 1])
      {
        delta = this._layerSize[depth + 1] + minDist1;
        if (aRatio < 1.0) {
          delta = delta / aRatio;
        }

        var r2 = radius;
        var n = this._smallestAltWidth[depth + 1].length;

        for (var j = 0; j < n; j++)
        {
          altSmallestWidth = this._smallestAltWidth[depth + 1][j];

          if (C < altSmallestWidth)
          {
            var r = TRadial.radiusFromCircumference(altSmallestWidth, aRatio);
            if (r - j * delta > r2) {
              r2 = r - j * delta;
            }
          }
        }

        if (r2 + (n - 1) * delta < r1)
        {
          radius = r2 + (n - 1) * delta;
        }
        else
        {
          radius = r1;
        }
      }
      else
      {
        radius = r1;
      }
    }

    return radius; // Number
  },

  calcFinalCoordRadial : function(root, border0, border1)
  {
    // summary:
    //		Performs finally the real polar transformation on the layered tree
    //		layout so that the root node is in the center and the nodes are in
    //		(elliptic) circles around the root.  
    // root: TNode
    //		The root node.
    // border0: Number
    //		The top left border of this tree (dir0).
    // border1: Number
    //		The top left border of this tree (dir1).
    // returns:
    //		The maximal coordinate of the layout in direction dir0.
    // tags:
    //		private
    
    var dir0 = this.getDir0();
    var dir1 = this.getDir1();

    var oldCoord0 = root.getCenter(dir0);

    var bounds = TranslateUtil.newArray(2, 0);

    root.setCenter(dir0, 0.0);
    root.setCenter(dir1, 0.0);
    bounds[0] = root.getMinCoord(dir0);
    bounds[1] = root.getMaxCoord(dir0);

    this.calcChildrenCoordRadial(0, root, oldCoord0, 0.0, 0.0, bounds);

    var dx;
    var dy;

    if (dir0 == 0)
    {
      dx = border0 - bounds[0];
      dy = border1;
      bounds[0] += dx;
      bounds[1] += dx;
    }
    else
    {
      dx = border1;
      dy = border0 - bounds[0];
      bounds[0] += dy;
      bounds[1] += dy;
    }

    this.getGraph().shiftSubtreeBy(root, dx, dy);

    this.layoutStepPerformed();
    return (bounds[1]); // Number
  },

  calcChildrenCoordRadial : function(depth, parent, parentOldCoord0, angleOfParent, radiusOfParent, bounds)
  {
    // summary:
    //		Calculates the radial coordinates of the children wrt. 
    // description:
    //		Calculates the radial coordinates of the children wrt. 
    //		the parent. As side effect, it updates the bounds in dir0 that are
    //		passed as input.  
    // depth: int
    //		The layer index of the parent node.
    // parent: TNode
    //		The parent node.
    // parentOldCoord0: Number
    //		The coordinate of the parent node before the
    //		polar transformation (in dir0).
    // angleOfParent: Number
    //		The polar transformed coordinate of the parent,
    //		i.e. the angle between root and parent.
    // radiusOfParent: Number
    //		The polar transformed coordinate of the parent,
    //		i.e. the radius between root and parent.
    // bounds: Array
    //		The minimal and maximal coordinate of a node
    //		after polar transformation. This gets updated.
    // tags:
    //		private
    
    var child;
    var dir0 = this.getDir0();
    var dir1 = this.getDir1();
    var oldCoord0;

    var rootStabilityCorrection = 0.0;
    var radius = 0.0;
    var dx;
    var dy;
    var delta = 0.0;
    var compressionFactor = 1.0;
    var angle;
    var rangle;
    var smallestWidth = this._width;
    var altSmallestWidth = this._width;
    var minDist1 = this.getMinDistBetweenNodes(dir1);
    var alternating = false;
    var aRatio = this.getAspectRatio();
    var levelWidth = this._width;

    this.addPercPoints(1);
    this.layoutStepPerformed();

    child = parent.getFirstRealChild();

    if (child != null)
    {
      smallestWidth = this._smallestWidth[depth + 1];
      delta = 0.5 * (this._layerSize[depth] + this._layerSize[depth + 1]) + minDist1;

      if (this._maxChildrenSpan != null)
      {
        var d = this.maxChildrenAngleDelta(radiusOfParent, this._maxChildrenSpan[depth + 1]);
        if (d > delta) {
          delta = d;
        }
      }

      if (aRatio < 1.0) {
        radius = radiusOfParent + delta / aRatio;
      }
      else {
        radius = radiusOfParent + delta;
      }

      var C = TRadial.circumference(radius, aRatio);

      var forceAltern = this.allowAlternating() && this._considerAlternating[depth + 1] && this.forceAlternating();

      if (C > levelWidth && !forceAltern)
      {
        compressionFactor = levelWidth / C;
        if (this.isFirstCircleEvenlySpaced() && depth == 0) {
          compressionFactor = 1.0;
        }
      }
      else if (C < smallestWidth || forceAltern)
      {
        var r1 = TRadial.radiusFromCircumference(smallestWidth, aRatio);

        if (this.allowAlternating() && this._considerAlternating[depth + 1])
        {
          delta = this._layerSize[depth + 1] + minDist1;
          if (aRatio < 1.0) {
            delta = delta / aRatio;
          }
          if (this._smallestAltWidth != null)
          {
            var r2 = radius;
            var n = this._smallestAltWidth[depth + 1].length;

            for (var j = 0; j < n; j++)
            {
              altSmallestWidth = this._smallestAltWidth[depth + 1][j];

              if (C < altSmallestWidth)
              {
                var r = TRadial.radiusFromCircumference(altSmallestWidth, aRatio);
                if (r - j * delta > r2) {
                  r2 = r - j * delta;
                }
              }
            }

            if (r2 + (n - 1) * delta < r1 || this.forceAlternating())
            {
              radius = r2 + (n - 1) * delta;
              alternating = true;
            }
            else
            {
              radius = r1;
            }
          }
          else if (this._altInnerSmallestWidth != null)
          {
            var rr2 = radius;
            var rr;
            altSmallestWidth = this._altInnerSmallestWidth[depth + 1];

            if (C < altSmallestWidth)
            {
              rr = TRadial.radiusFromCircumference(altSmallestWidth, aRatio);
              if (rr > rr2) {
                rr2 = rr;
              }
            }

            altSmallestWidth = this._altOuterSmallestWidth[depth + 1];

            if (C < altSmallestWidth)
            {
              rr = TRadial.radiusFromCircumference(altSmallestWidth, aRatio);
              if (rr - delta > rr2) {
                rr2 = rr - delta;
              }
            }

            if (rr2 + delta < r1 || this.forceAlternating())
            {
              radius = rr2 + delta;
              alternating = true;
            }
            else
            {
              radius = r1;
            }
          }
          else
          {
            radius = r1;
          }
        }
        else
        {
          radius = r1;
        }
      }

      if (parent.getImmediateParent() == null)
      {
        rootStabilityCorrection = parentOldCoord0 - child.getCenter(dir0);
      }
    }

    var iter = parent.getVirtualChildren();

    while (iter.hasNext())
    {
      child = iter.next();
      oldCoord0 = child.getCenter(dir0);

      angle = angleOfParent + TRadial._twopi * compressionFactor * (oldCoord0 - parentOldCoord0 + rootStabilityCorrection) / levelWidth;

      rangle = this.translateToEllipse(angle);

      if (alternating)
      {
        if (this._smallestAltWidth != null)
        {
          var circleIndex = Math.floor(child.getSortValue());
          var rad = radius - circleIndex * delta;
          dx = (aRatio * rad * Math.sin(rangle));
          dy = (rad * Math.cos(rangle));
        }
        else
        {
          if (child.getSortValue() == 1.0)
          {
            dx = (aRatio * (radius - delta) * Math.sin(rangle));
            dy = ((radius - delta) * Math.cos(rangle));
          }
          else
          {
            dx = (aRatio * radius * Math.sin(rangle));
            dy = (radius * Math.cos(rangle));
          }
        }
      }
      else
      {
        dx = (aRatio * radius * Math.sin(rangle));
        dy = (radius * Math.cos(rangle));
      }

      this.setNodeCoordinate(child, dx, dy);

      if (child.getMinCoord(dir0) < bounds[0]) {
        bounds[0] = child.getMinCoord(dir0);
      }
      if (child.getMaxCoord(dir0) > bounds[1]) {
        bounds[1] = child.getMaxCoord(dir0);
      }
      this.calcChildrenCoordRadial(depth + 1, child, oldCoord0, angle, radius, bounds);
    }
  },

  setNodeCoordinate : function(node, x, y)
  {
    // summary:
    //		Sets the real node coordinate if its calculated center coordinate is
    //		(x, y). 
    // description:
    //		Sets the real node coordinate if its calculated center coordinate is
    //		(x, y). 
    //		This handles the level justification. This treatment of the
    //		justification is a little bit cheating. Everything except
    //		justification zero (center) can produce overlaps.
    // tags:
    //		private
    
    var dir0 = this.getDir0();
    var dir1 = this.getDir1();

    switch (this.getLevelJustification())
    {
      case -1:
        if (x > 0.5 * node.getOriginalSize(dir0)) {
          node.setCenter(dir0, x + 0.5 * node.getOriginalSize(dir0));
        }
        else if (x < -0.5 * node.getOriginalSize(dir0)) {
          node.setCenter(dir0, x - 0.5 * node.getOriginalSize(dir0));
        }
        else {
          node.setCenter(dir0, x);
        }
        if (y > 0.5 * node.getOriginalSize(dir1)) {
          node.setCenter(dir1, y + 0.5 * node.getOriginalSize(dir1));
        }
        else if (y < -0.5 * node.getOriginalSize(dir1)) {
          node.setCenter(dir1, y - 0.5 * node.getOriginalSize(dir1));
        }
        else {
          node.setCenter(dir1, y);
        }
        break;
      case 1:
        if (x > 0.5 * node.getOriginalSize(dir0)) {
          node.setCenter(dir0, x - 0.5 * node.getOriginalSize(dir0));
        }
        else if (x < -0.5 * node.getOriginalSize(dir0)) {
          node.setCenter(dir0, x + 0.5 * node.getOriginalSize(dir0));
        }
        else {
          node.setCenter(dir0, x);
        }
        if (y > 0.5 * node.getOriginalSize(dir1)) {
          node.setCenter(dir1, y - 0.5 * node.getOriginalSize(dir1));
        }
        else if (y < -0.5 * node.getOriginalSize(dir1)) {
          node.setCenter(dir1, y + 0.5 * node.getOriginalSize(dir1));
        }
        else {
          node.setCenter(dir1, y);
        }
        break;
      default:
        node.setCenter(dir0, x);
        node.setCenter(dir1, y);
    }
  },

  maxChildrenAngleDelta : function(radius, childrenSpan)
  {
    // summary:
    //		Returns the delta the input radius must be enlarged so that the
    //		children are placed with angle 'maxChildrenAngle'.  
    // radius: Number
    //		The radius of the parent.
    // childrenSpan: Number
    //		The distance between the first and the last child.
    // tags:
    //		private
    
    var angle = this.getMaxChildrenAngle();
    if (angle < 0.01) {
      return (0.0); // Number
    }
    if (childrenSpan < 0.01) {
      return (0.0); // Number
    }
    var delta1 = 0.0;
    var delta2 = childrenSpan / angle;
    if (angle < 3.14) {
      delta1 = childrenSpan / (2 * Math.tan(angle / 2));
    }
    if (radius > 2 * childrenSpan) {
      return delta1; // Number
    }
    return radius / (2 * childrenSpan) * (delta1 - delta2) + delta2; // Number
  },

  setEllipsoidTransTable : function(table)
  {
    // summary:
    //		Sets the ellipoid translation table. 
    // description:
    //		Sets the ellipoid translation table. 
    //		See <code> TRadial.getEllipsoidTransTable()</code>.
    // tags:
    //		private
    
    this._angleTransTable = table;
  },

  translateToEllipse : function(angle)
  {
    // summary:
    //		Translates an angle suitable for mapping equidistant points on a
    //		circle to an angle suitable for mapping equidistant points on an
    //		ellipse. 
    // description:
    //		Translates an angle suitable for mapping equidistant points on a
    //		circle to an angle suitable for mapping equidistant points on an
    //		ellipse. 
    //		We use the translation table calculates above, because there is no
    //		closed formula for this.  
    // angle: Number
    //		The angle for the circle.
    // returns:
    //		The angle for the ellipse.
    // tags:
    //		private
    
    if (this._angleTransTable == null) {
      return (angle); // Number
    }
    if (angle < 0.0) {
      angle += TRadial._twopi;
    }
    if (angle > TRadial._twopi) {
      angle -= TRadial._twopi;
    }
    var x = 500.0 * angle / TRadial._twopi;
    var i = Math.floor(x);
    if (i >= 500) {
      return (TRadial._twopi); // Number
    }
    if (i < 0) {
      return (0.0); // Number
    }
    return (this._angleTransTable[i] + (this._angleTransTable[i + 1] - this._angleTransTable[i]) * (x - i)); // Number
  }

}
);

lang.mixin(TRadial,
{
  _pi : null,
  _twopi : null,

  circumference : function(r, sx)
  {
    if (sx == 1.0) {
      return (TRadial._twopi * r); // Number
    }

    var l = (sx - 1.0) / (sx + 1.0);
    var lsqr = l * l;
    var n = 64.0 - 16.0 * lsqr;

    return (TRadial._pi * r * (1.0 + sx) * (64.0 - 3.0 * lsqr * lsqr) / n); // Number
  },

  radiusFromCircumference : function(c, sx)
  {
    if (sx == 1.0) {
      return (c / TRadial._twopi); // Number
    }

    var l = (sx - 1.0) / (sx + 1.0);
    var lsqr = l * l;
    var n = 64.0 - 3.0 * lsqr * lsqr;

    return (c * (64.0 - 16.0 * lsqr) / n / (1.0 + sx) / TRadial._pi); // Number
  },

  getEllipsoidTransTable : function(sx)
  {
    if (sx == 1.0) {
      return null; // Array
    }
    else
    {
      var angleTransTable = TranslateUtil.newArray(501, 0);
      var C = TRadial.circumference(1.0, sx);
      var equiDist = C / 500;
      var angle;
      var prevAngle = 0.0;
      var arcOfAngle;
      var arcOfPrevAngle = 0.0;
      var x0;
      var x1;
      var y0;
      var y1;

      for (var i = 0; i < 500; i++)
      {
        angle = prevAngle;
        arcOfAngle = arcOfPrevAngle;
        x0 = sx * Math.sin(angle);
        y0 = Math.cos(angle);

        while (arcOfAngle < i * equiDist)
        {
          angle += 0.0001;
          x1 = sx * Math.sin(angle);
          y1 = Math.cos(angle);
          arcOfAngle += Math.sqrt((x1 - x0) * (x1 - x0) + (y1 - y0) * (y1 - y0));
          x0 = x1;
          y0 = y1;
        }

        angleTransTable[i] = angle;
        prevAngle = angle;
        arcOfPrevAngle = arcOfAngle;
      }

      angleTransTable[500] = TRadial._twopi;
      return angleTransTable; // Array
    }
  },

  _fields_init : function(c)
  {
    c = c || TRadial;
    if(TGraphAlgorithm._fields_init)
      TGraphAlgorithm._fields_init(c);
    
    c._pi = Math.PI;
    c._twopi = 2 * Math.PI;
  }
  
}
);

(function(){ TRadial._fields_init(); })();

return TRadial;
});

},
'ibm_ilog/graphlayout/tree/TreeLayout':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TreeLayout", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/ConnectorStyles",	// #6
  "../internalutil/GraphModelUtil",	// #7
  "../internalutil/LayoutParametersUtil",	// #8
  "../internalutil/LayoutUtil",	// #9
  "../internalutil/LinkStyles",	// #10
  "../internalutil/LogResUtil",	// #11
  "../internalutil/Point",	// #12
  "../internalutil/SplineRouter",	// #13
  "../internalutil/TranslateUtil",	// #14
  "../tree/TAngleSpacingModes",	// #15
  "../tree/TAutoTipOver",	// #16
  "../tree/TBalloon",	// #17
  "../tree/TBalloonRadiusModes",	// #18
  "../tree/TGraph",	// #19
  "../tree/TLayoutModes",	// #20
  "../tree/TLinkParameters",	// #21
  "../tree/TNodeParameters",	// #22
  "../tree/TRadial",	// #23
  "../tree/TSpanTree",	// #24
  "../Direction",	// #25
  "../GraphLayout",	// #26
  "../GraphLayoutReport"	// #27
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    ConnectorStyles,	// #6
    GraphModelUtil,	// #7
    LayoutParametersUtil,	// #8
    LayoutUtil,	// #9
    LinkStyles,	// #10
    LogResUtil,	// #11
    Point,	// #12
    SplineRouter,	// #13
    TranslateUtil,	// #14
    TAngleSpacingModes,	// #15
    TAutoTipOver,	// #16
    TBalloon,	// #17
    TBalloonRadiusModes,	// #18
    TGraph,	// #19
    TLayoutModes,	// #20
    TLinkParameters,	// #21
    TNodeParameters,	// #22
    TRadial,	// #23
    TSpanTree,	// #24
    Direction,	// #25
    GraphLayout,	// #26
    GraphLayoutReport	// #27
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var GraphLayout = ibm_ilog.graphlayout.GraphLayout;
=====*/

var TreeLayout = declare("ibm_ilog.graphlayout.tree.TreeLayout", GraphLayout, 
{
  // summary:
  //		The main class for the Tree Layout algorithm. 
  // description:
  //		The main class for the Tree Layout algorithm. 
  //		<p>
  //		The Tree Layout algorithm places the nodes of a tree starting from
  //		the <i>root</i> of the tree to the <i>leaves.</i> If the graph has
  //		parts that are disconnected, it arranges each connected component as
  //		an individual tree. The Tree Layout algorithm is primarily designed
  //		for a forest of pure trees. In a tree, each node except the root has
  //		a <i>parent</i> node. All the nodes that have the same parent are
  //		called <i>children</i> with respect to the parent and <i>siblings</i>
  //		with respect to each other. The Tree Layout algorithm can also be
  //		used for non-trees (for example, cyclic graphs). In this case, the
  //		layout algorithm ignores those links that violate the tree structure
  //		and takes only the <i>spanning tree</i> of the graph into account.  
  //		</p>
  //		<p>
  //		The algorithm has several layout modes:  
  //		<ul>
  //		<li> The nodes can be arranged freely in a horizontal or vertical
  //		way. This is suitable for a graph with a large variation of node
  //		sizes.</li>  
  //		<li> The nodes can be arranged in horizontal or vertical levels. This
  //		is suitable for a graph that contains nodes of approximately the same
  //		size.</li>  
  //		<li> The nodes can be arranged radially on concentric circles or on
  //		ellipses around the root of the tree.</li>  
  //		<li> The nodes can be arranged radially around parents like balloons.
  //		</li>  
  //		</ul>
  //		To simplify the explanations of the layout parameters, we use the
  //		compass directions <i>north, south, east</i>, and <i>west</i>. The
  //		center of the root node of a tree is the north pole. If the root node
  //		is placed at the top border, north is always upwards, south towards
  //		the bottom, west towards the left, and east towards the right side of
  //		the layout. If the root node is placed at the left border, north is
  //		left and south is right. In radial mode, the root is placed in the
  //		center, and so the notation of north and south depends on where a
  //		node is placed relative to the root. The north side of any node is
  //		the side that is closer to the root and the south side is the side
  //		that is farther away from the root. In balloon mode, the compass
  //		directions make no sense.   
  //		</p>
  //		<p>
  //		Several alignment options are provided. In particular, the free
  //		layout mode allows the <i>tip-over alignment</i>, which means that
  //		subtrees are arranged serially rather than in parallel. If the
  //		subtrees starting at one node are normally arranged horizontally, the
  //		tip-over alignment arranges them vertically. The tree layout provides
  //		layout modes that automatically adapt the tip-over alignment to best
  //		fit the layout to a given aspect ratio.  
  //		</p>
  //		<p>
  //		The following sample drawings are produced by the Tree Layout
  //		algorithm. The left drawing uses the level layout mode. The flow
  //		direction of the links is towards the bottom, and the nodes are
  //		south-justified (that is, they are aligned to the bottom) within each
  //		level. The right drawing is the same tree in free layout mode (which
  //		always uses north level alignment):  
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Tree-level-free.gif" alt="Sample drawing produced with the Tree Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		Here is another sample drawing in free layout mode with the tip-over
  //		alignment at some leaf nodes:  
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Tree-tipover.gif" alt="Sample drawing produced with the Tree Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		The following are two sample drawings in the two radial layout modes.
  //		The normal radial mode in the left drawing places the nodes of the
  //		same level on a circle around the root node. The alternating radial
  //		mode in the right drawing places the nodes of the same level
  //		alternating on two circles around the root node. This results in
  //		better usage of the space.  
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Tree-radial.gif" alt="Sample drawing produced with the Tree Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		In nonradial layout modes, the Tree Layout algorithm supports
  //		different styles of links in the same drawing. The following sample
  //		drawing shows orthogonal and straight-line links. It also shows nodes
  //		that are aligned to the center (see the red nodes), to the east
  //		(green), and to the west (blue) of their children. The drawing was
  //		created by the free layout mode with the link flow direction to the
  //		right, and so north is to the left.  
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Tree-mixedlinks.gif" alt="Sample drawing produced with the Tree Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		The Tree Layout algorithm supports node neighboring in all modes. The
  //		following sample drawing shows a tree layout with the flow direction
  //		towards the top. Some nodes are not placed as children but rather as
  //		neighbors at the same level as the corresponding parent node. The red
  //		links, which are perpendicular to the flow direction, are between
  //		nodes that were specified as neighbored.  
  //		</p>
  //		<p>
  //		<img src="../../../images/ibm&#95;graphlayout/layout-Tree-attachments.gif" alt="Sample drawing produced with the Tree Layout" border="0" align="bottom" />
  //		 
  //		</p>
  //		<p>
  //		See the corresponding chapter of the
  //		User's Manual for details on the algorithm, the types of graphs for which this algorithm can be used, the features and limitations, code samples, and so on.  
  //		</p>
  //		<p>
  //		Note the following points:  
  //		<ul>
  //		<li>  If no root node is specified, the layout algorithm takes the direction of links into account to calculate a root node. Other than that, the direction of links does not affect the layout.</li>  
  //		<li>  The layout algorithm has an incremental mode that is designed to be stable when incremental changes are made. If nodes are added or removed after a layout, the next layout uses a heuristic to try to adjust the layout only minimally and to avoid major structural reorganization.</li>  
  //		<li>  In incremental mode, the initial position of the nodes (at the moment you start the layout) affects the resulting layout because the algorithm tries to preserve the relative order of the children on incremental changes.</li>  
  //		</ul>  
  //		</p>
  /*=====
  // NO_RESHAPE_STYLE: int
  //		No reshape option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalLinkStyle()', none of the links are
  //		reshaped. To specify the shape of an individual link, use this option
  //		as the argument of the method '
  //		TreeLayout.setLinkStyle()'.
  NO_RESHAPE_STYLE : null,
  // STRAIGHT_LINE_STYLE: int
  //		Straight-line link shape option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalLinkStyle()', all links that are part of
  //		the spanning tree get a straight-line shape. To specify the shape of
  //		an individual link, use this option as the argument of the method
  //		' TreeLayout.setLinkStyle()'.
  STRAIGHT_LINE_STYLE : null,
  // ORTHOGONAL_STYLE: int
  //		Orthogonal link shape option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalLinkStyle()', all links that are part of
  //		the spanning tree get a shape consisting of a sequence of orthogonal
  //		line segments. To specify the shape of an individual link, use this
  //		option as the argument of the method '
  //		TreeLayout.setLinkStyle()'.
  ORTHOGONAL_STYLE : null,
  // MIXED_STYLE: int
  //		Mixed link shape option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalLinkStyle()', each link can have a
  //		different shape. The shape of an individual link can be set by '
  //		TreeLayout.setLinkStyle()'.  
  //		
  //		 
  //		
  MIXED_STYLE : null,
  // AUTOMATIC_PINS: int
  //		Automatic connection point option. 
  //		When used as the argument of the method '
  //		TreeLayout.setConnectorStyle()', the connector style is
  //		automatically selected depending on the global link style.  
  //		
  //		 
  //		
  AUTOMATIC_PINS : null,
  // CENTERED_PINS: int
  //		Centered connection point option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setConnectorStyle()', the connection points of links
  //		are placed at the center of the border to which the link is attached.
  //		 
  //		
  //		 
  //		
  CENTERED_PINS : null,
  // CLIPPED_PINS: int
  //		Clipped connection point option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setConnectorStyle()', the connection points of links
  //		are placed such that the link pointing toward the node center is
  //		clipped at the node border.  
  //		
  //		 
  //		
  CLIPPED_PINS : null,
  // EVENLY_SPACED_PINS: int
  //		Evenly spaced connection point option. 
  //		When used as the argument of the method  method '
  //		TreeLayout.setConnectorStyle()', the connection points of links
  //		are evenly spaced along the border to which the link is attached.  
  //		
  //		 
  //		
  EVENLY_SPACED_PINS : null,
  // EAST: int
  //		East alignment option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', all parent nodes are aligned
  //		to the east border of their children. The alignment of an individual
  //		node can be set by ' TreeLayout.setAlignment()'.  
  //		
  //		 
  //		
  EAST : null,
  // WEST: int
  //		West alignment option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', all parent nodes are aligned
  //		to the west border of their children. The alignment of an individual
  //		node can be set by ' TreeLayout.setAlignment()'.  
  //		
  //		 
  //		
  WEST : null,
  // NORTH: int
  //		North alignment option. 
  //		When used in level layout mode as the argument of the method  '
  //		TreeLayout.setLevelAlignment()', the nodes of the same level
  //		are justified to the north side (towards the root).  
  //		
  //		 
  //		
  NORTH : null,
  // SOUTH: int
  //		South alignment option. 
  //		When used in level layout mode as the argument of the method  '
  //		TreeLayout.setLevelAlignment()', the nodes of the same level
  //		are justified to the south side (away from the root).  
  //		
  //		 
  //		
  SOUTH : null,
  // CENTER: int
  //		Center alignment option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', all parent nodes are centered
  //		with respect to their children. The alignment of an individual node
  //		can be set by ' TreeLayout.setAlignment()'.  
  //		
  //		When used in level layout mode as the argument of the method  '
  //		TreeLayout.setLevelAlignment()', the nodes of the same level
  //		are justified to their centers.  
  //		
  CENTER : null,
  // BORDER_CENTER: int
  //		Border center alignment option. 
  //		When used as the argument of the method '
  //		TreeLayout.setGlobalAlignment()', all parent nodes are centered
  //		with respect to the border of their children. The alignment of an
  //		individual node can be set by '
  //		TreeLayout.setAlignment()'.  
  //		
  //		 
  //		
  BORDER_CENTER : null,
  // TIP_OVER: int
  //		Tip-over alignment option. 
  //		
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', it behaves like '
  //		TreeLayout.TIP_OVER_EAST'. The alignment of an individual
  //		parent node can be set by ' TreeLayout.setAlignment()'.  
  //		
  //		
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', all links flow roughly in the flow
  //		direction. In this layout mode, the algorithm tries to optimize the
  //		layout to automatically fit best to the specified aspect ratio by
  //		calculating which individual nodes need tip-over alignment and which
  //		nodes can use the specified alignment. The layout algorithm uses a
  //		fast heuristic.  
  //		
  //		
  //		 
  //		
  TIP_OVER : null,
  // TIP_OVER_EAST: int
  //		Tip-over east alignment option. 
  //		
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', the children are arranged
  //		serially rather than in parallel, on the east side of the parent
  //		node.. For instance, if the children are normally arranged
  //		horizontally below each parent node, the tip-over alignment arranges
  //		them vertically on the east side below the parent node. The alignment
  //		of an individual parent node can be set by '
  //		TreeLayout.setAlignment()'.  
  //		
  //		
  //		 
  //		
  TIP_OVER_EAST : null,
  // TIP_OVER_WEST: int
  //		Tip-over west alignment option. 
  //		
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', the children are arranged
  //		serially rather than in parallel, on the west side of the parent
  //		node.. For instance, if the children are normally arranged
  //		horizontally below each parent node, the tip-over alignment arranges
  //		them vertically on the west side below the parent node. The alignment
  //		of an individual parent node can be set by '
  //		TreeLayout.setAlignment()'.  
  //		
  //		
  //		 
  //		
  TIP_OVER_WEST : null,
  // TIP_OVER_EAST_WEST: int
  //		Tip-over east and west alignment option. 
  //		
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', the children are arranged
  //		serially rather than in parallel, alternating on the east and on the
  //		west side of the parent node. This is basically a combination of
  //		' TreeLayout.TIP_OVER_EAST' and '
  //		TreeLayout.TIP_OVER_WEST'. The first child is placed on the
  //		east side. Further children are placed on the east or west side, so
  //		that the space usage on both sides is roughly the same, i.e., the
  //		subtrees on the east side have roughly the same sum of height than
  //		the subtrees on the west side. The alignment of an individual parent
  //		node can be set by ' TreeLayout.setAlignment()'.  
  //		
  //		
  //		Since this alignment option decides between east and west side
  //		depending on subtree sizes, it is not well suited in incremental mode
  //		(see ' TreeLayout.setIncrementalMode()'), since little
  //		changes in the graph can cause big changes in the layout even if the
  //		incremental mode is switched on.  
  //		
  //		
  //		 
  //		
  TIP_OVER_EAST_WEST : null,
  // TIP_OVER_BOTH_SIDES: int
  //		Tip-over to both sides alignment option. 
  //		
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', the child nodes are arranged
  //		in pairs serially rather than in parallel. This style is very similar
  //		to the style ' TreeLayout.TIP_OVER_EAST_WEST', because
  //		both the east and west sides are used. However, the algorithm does
  //		not try to optimize the space used on both sides; instead it always
  //		puts the odd-numbered child nodes to the west and the even-numbered
  //		child nodes to the east. The pairs of child nodes, one on the east
  //		side and one on the west side, are aligned with one another. The
  //		alignment of an individual parent node can be set by '
  //		TreeLayout.setAlignment()'.  
  //		
  //		
  //		 
  //		
  TIP_OVER_BOTH_SIDES : null,
  // MIXED: int
  //		Mixed alignment option. 
  //		When used as the argument of the method  '
  //		TreeLayout.setGlobalAlignment()', each node can have a
  //		different alignment. The alignment of an individual node can be set
  //		by ' TreeLayout.setAlignment()'.  
  //		
  //		 
  //		
  MIXED : null,
  // UNSPECIFIED: int
  //		Unspecified child alignment option. 
  //		When used as the argument of the method '
  //		TreeLayout.setChildAlignment()', the child passed as argument
  //		uses the alignment of the parent node.
  UNSPECIFIED : null,
  // EAST_NEIGHBOR: int
  //		East neighbor child alignment option. 
  //		When used as the argument of the method '
  //		TreeLayout.setChildAlignment()', the child passed as argument
  //		is made a neighbor at the east side of its parent node.
  EAST_NEIGHBOR : null,
  // WEST_NEIGHBOR: int
  //		West neighbor child alignment option. 
  //		When used as the argument of the method '
  //		TreeLayout.setChildAlignment()', the child passed as argument
  //		is made a neighbor at the west side of its parent node.
  WEST_NEIGHBOR : null,
  // TIP_OVER_ASSOCIATE_BOTH_SIDES: int
  //		Associate child alignment option. 
  //		When used as the argument of the method '
  //		TreeLayout.setChildAlignment()', the child is laid out like an
  //		associate node of its parent node. Associate nodes occur alternating
  //		on the east or west side of its parent node.
  TIP_OVER_ASSOCIATE_BOTH_SIDES : null,
  // TIP_OVER_ASSOCIATE_EAST: int
  //		Associate east child alignment option. 
  //		When used as the argument of the method '
  //		TreeLayout.setChildAlignment()', the child is laid out like an
  //		associate node of its parent node. This associate nodes occurs on the
  //		east side of its parent node.
  TIP_OVER_ASSOCIATE_EAST : null,
  // TIP_OVER_ASSOCIATE_WEST: int
  //		Associate west child alignment option. 
  //		When used as the argument of the method '
  //		TreeLayout.setChildAlignment()', the child is laid out like an
  //		associate node of its parent node. This associate nodes occurs on the
  //		west side of its parent node.
  TIP_OVER_ASSOCIATE_WEST : null,
  // FREE: int
  //		Free layout mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', all links flow roughly in the flow
  //		direction. Parents and sibling nodes are aligned according to the
  //		specified alignment option, but nodes of different tree branches are
  //		not justified with each other,  
  //		
  //		 
  //		
  FREE : null,
  // LEVEL: int
  //		Level layout mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', all links flow roughly in the flow
  //		direction. All nodes are organized in levels and are justified to
  //		their borders or centered according to the level alignment option.
  //		Tip-over node alignment is not possible; all other alignment options
  //		are possible.  
  //		
  //		 
  //		
  LEVEL : null,
  // RADIAL: int
  //		Radial layout mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', the root node is in the center and
  //		the links flow radially away from the center. The nodes are placed in
  //		circular layers around the root node.  
  //		
  //		 
  //		
  RADIAL : null,
  // ALTERNATING_RADIAL: int
  //		Alternating radial layout mode. 
  //		When used as the argument of the method '
  //		TreeLayout.setLayoutMode()', the root node is in the center and
  //		the links flow radially away from the center. The nodes are placed in
  //		circular layers around the root node. To optimize space, siblings may
  //		be placed alternating in several  different circular layers. The
  //		alternating radial mode differs only from the radial mode if the
  //		alternating placement produces smaller circles.  
  //		
  //		 
  //		
  ALTERNATING_RADIAL : null,
  // BALLOON: int
  //		Balloon layout mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', the root node is in the center and
  //		the subtrees are arranged as balloon circles around the root. Each
  //		subtree froms its own balloon recursively.  
  //		
  //		 
  //		
  BALLOON : null,
  // TIP_LEAVES_OVER: int
  //		Tip-over layout mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', all links flow roughly in the flow
  //		direction. The algorithm tries to optimize the layout to
  //		automatically fit best to the specified aspect ratio by calculating
  //		which individual nodes need tip-over alignment and which nodes can
  //		use the specified alignment. The layout algorithm uses a heuristic
  //		that checks whether leaf nodes (and their immediate parent nodes)
  //		should be tipped over. It checks only the alignment mode '
  //		TreeLayout.TIP_OVER_EAST', but not '
  //		TreeLayout.TIP_OVER_BOTH_SIDES' or other variants.  
  //		
  //		 
  //		
  TIP_LEAVES_OVER : null,
  // TIP_ROOTS_OVER: int
  //		Tip-over layout mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', all links flow roughly in the flow
  //		direction. The algorithm tries to optimize the layout to
  //		automatically fit best to the specified aspect ratio by calculating
  //		which individual nodes need tip-over alignment and which nodes can
  //		use the specified alignment. The layout algorithm uses a heuristic
  //		that checks whether root nodes (and their immediate children nodes)
  //		should be tipped over. It checks only the alignment mode '
  //		TreeLayout.TIP_OVER_EAST', but not '
  //		TreeLayout.TIP_OVER_BOTH_SIDES' or other variants.  
  //		
  //		 
  //		
  TIP_ROOTS_OVER : null,
  // TIP_ROOTS_AND_LEAVES_OVER: int
  //		Tip-over layout mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setLayoutMode()', all links flow roughly in the flow
  //		direction. The algorithm tries to optimize the layout to
  //		automatically fit best to the specified aspect ratio by calculating
  //		which individual nodes need tip-over alignment and which nodes can
  //		use the specified alignment. The layout algorithm uses a heuristic
  //		that checks whether root nodes and leaf nodes should be tipped over.
  //		It checks only the alignment mode '
  //		TreeLayout.TIP_OVER_EAST', but not '
  //		TreeLayout.TIP_OVER_BOTH_SIDES' or other variants.  
  //		
  //		 
  //		
  TIP_ROOTS_AND_LEAVES_OVER : null,
  // REGULAR: int
  //		Regular angle spacing. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonAngleSpacing()' in balloon layout mode,
  //		the available angle range at a node is evenly split so that each
  //		subtrees at the node can occupy the same angle.  
  //		
  //		 
  //		
  REGULAR : null,
  // FAST_PROPORTIONAL: int
  //		Fast proportional angle spacing. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonAngleSpacing()' in balloon layout mode,
  //		the available angle range at a node is proprionally split so that
  //		larger subtrees at the node can occupy larger angles. The algorithm
  //		uses a fast heuristic to calculate the angle ranges.  
  //		
  //		 
  //		
  FAST_PROPORTIONAL : null,
  // PROPORTIONAL: int
  //		Slow proportional angle spacing. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonAngleSpacing()' in balloon layout mode,
  //		the available angle range at a node is proprionally split so that
  //		larger subtrees at the node can occupy larger angles. The algorithm
  //		uses a slow heuristic to calculate the angle ranges, which is often
  //		more precise than the fast heuristic.  
  //		
  //		 
  //		
  PROPORTIONAL : null,
  // VARIABLE_RADIUS: int
  //		Variable balloon radius mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonRadiusMode()' in balloon layout mode, the
  //		radius of different child balloons at the same parent node can vary. 
  //		
  //		
  //		 
  //		
  VARIABLE_RADIUS : null,
  // OPTIMIZED_VARIABLE_RADIUS: int
  //		Optimized variable balloon radius mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonRadiusMode()' in balloon layout mode, the
  //		radius of different child balloons at the same parent node can vary.
  //		An additional optimization heuristic reduces the radius. This has the
  //		advantage of smaller space requirements but looses symmetries in the
  //		diagram.  
  //		
  //		 
  //		
  OPTIMIZED_VARIABLE_RADIUS : null,
  // UNIFORM_LEAVES_RADIUS: int
  //		Uniform balloon radius mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonRadiusMode()' in balloon layout mode, the
  //		radiuses of all leaves that are children of the same node are
  //		uniform. The radius of balloons that contain more than one node can
  //		still vary.  
  //		
  //		 
  //		
  UNIFORM_LEAVES_RADIUS : null,
  // OPTIMIZED_UNIFORM_LEAVES_RADIUS: int
  //		Optimized uniform balloon radius mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonRadiusMode()' in balloon layout mode, the
  //		radiuses of all leaves that are children of the same node are
  //		uniform. The radius of balloons that contain more than one node can
  //		still vary. An additional optimization heuristic reduces the radius.
  //		This has the advantage of smaller space requirements but looses
  //		symmetries in the diagram.  
  //		
  //		 
  //		
  OPTIMIZED_UNIFORM_LEAVES_RADIUS : null,
  // UNIFORM_RADIUS: int
  //		Uniform radius mode. 
  //		When used as the argument of the method  '
  //		TreeLayout.setBalloonRadiusMode()' in balloon layout mode, the
  //		radiuses of all subtree balloons that are children of the same node
  //		are uniform.  
  //		
  //		 
  //		
  UNIFORM_RADIUS : null,
  =====*/
  
  _NODE_PROPERTY: null,

  _LINK_PROPERTY: null,

  _globalLinkStyle: null,

  _connectorStyle: null,

  _globalAlignment: null,

  _layoutMode: null,

  _balloonAngleSpacing: null,

  _balloonRadiusMode: null,

  _flowDirection: null,

  _levelAlignment: null,

  _siblingOffset: null,

  _parentChildOffset: null,

  _branchOffset: null,

  _tipOverBranchOffset: null,

  _assocOffset: null,

  _parentChildAssocOffset: null,

  _aspectRatio: null,

  _overlapPercentage: null,

  _orthForkPercentage: null,

  _maxChildrenAngle: null,

  _firstCircleEvenlySpacing: null,

  _respectNodeSizes: null,

  _incrementalMode: null,

  _allLevelsAlternating: null,

  _numberOfAlternatingCircles: null,

  _nonTreeLinksStraight: null,
  _position: null,
  _isRootPosition: null,
  _isInvisibleRootUsed: null,

  _intergraphLinkConnectivity: null,

  _balloonStartAngle: null,

  _balloonRootChildrenAngle: null,

  _balloonInnerChildrenAngle: null,

  _balloonLeafChildrenAngle: null,

  _categorizingLinksWithoutLayout: null,

  _nodeComparator: null,

  _allowStopImmediately: null,

  _stoppedBeforeCompletion: null,

  _percCompleteControler: null,

  _specRoots: null,

  _calcRoots: null,

  _calcForwardTreeLinks: null,

  _calcBackwardTreeLinks: null,

  _calcNonTreeLinks: null,

  _angleTransTable : null,
  _angleTransTableAspectRatio : null,

  constructor : function(source )
  {
    this._angleTransTable = null;
    this._angleTransTableAspectRatio = 1.0;
  },
  
  init : function()
  {
    // summary:
    //		Initializes instance variables. 
    // description:
    //		Initializes instance variables. 
    //		<p>
    //		You should not call this method directly. The method is called
    //		internally by the constructor without arguments and by the copy
    //		constructor. The method must be overridden by subclasses that need
    //		to initialize additional instance variables.  
    //		</p>
    //		<p>
    //		</p>
    // tags:
    //		protected
    
    this.inherited("init", arguments, []);
    this.initPropertiesNames();
    this._globalLinkStyle = TreeLayout._defaultLinkStyle;
    this._connectorStyle = TreeLayout._defaultConnectorStyle;
    this._globalAlignment = TreeLayout._defaultAlignment;
    this._layoutMode = TreeLayout._defaultLayoutMode;
    this._balloonAngleSpacing = TreeLayout._defaultBalloonAngleSpacing;
    this._balloonRadiusMode = TreeLayout._defaultBalloonRadiusMode;
    this._flowDirection = TreeLayout._defaultFlowDirection;
    this._levelAlignment = TreeLayout._defaultLevelAlignment;
    this._siblingOffset = TreeLayout._defaultSiblingOffset;
    this._parentChildOffset = TreeLayout._defaultParentChildOffset;
    this._branchOffset = TreeLayout._defaultBranchOffset;
    this._tipOverBranchOffset = TreeLayout._defaultTipOverBranchOffset;
    this._assocOffset = TreeLayout._defaultAssocOffset;
    this._parentChildAssocOffset = TreeLayout._defaultParentChildAssocOffset;
    this._aspectRatio = TreeLayout._defaultAspectRatio;
    this._overlapPercentage = TreeLayout._defaultOverlapPercentage;
    this._orthForkPercentage = TreeLayout._defaultOrthForkPercentage;
    this._maxChildrenAngle = TreeLayout._defaultMaxChildrenAngle;
    this._balloonStartAngle = TreeLayout._defaultBalloonStartAngle;
    this._balloonRootChildrenAngle = TreeLayout._defaultBalloonRootChildrenAngle;
    this._balloonInnerChildrenAngle = TreeLayout._defaultBalloonInnerChildrenAngle;
    this._balloonLeafChildrenAngle = TreeLayout._defaultBalloonLeafChildrenAngle;
    this._categorizingLinksWithoutLayout = TreeLayout._defaultCategorizingLinksWithoutLayout;
    this._firstCircleEvenlySpacing = TreeLayout._defaultFirstCircleEvenlySpacing;
    this._respectNodeSizes = TreeLayout._defaultRespectNodeSizes;
    this._incrementalMode = TreeLayout._defaultIncrementalMode;
    this._allLevelsAlternating = TreeLayout._defaultAllLevelsAlternating;
    this._numberOfAlternatingCircles = TreeLayout._defaultNumberOfAlternatingCircles;
    this._nonTreeLinksStraight = TreeLayout._defaultNonTreeLinksStraight;
    this._intergraphLinkConnectivity = TreeLayout._defaultIntergraphLinkConnectivity;
    this._isRootPosition = true;
    this._position = null;
    this._isInvisibleRootUsed = false;
    this._allowStopImmediately = true;
    this._specRoots = new Array();
    this._calcRoots = new Array();
    this._calcForwardTreeLinks = null;
    this._calcBackwardTreeLinks = null;
    this._calcNonTreeLinks = null;
  },

  initPropertiesNames : function()
  {
    // tags:
    //		private
    
    this._NODE_PROPERTY = "__ilvTreeLayoutNode" + this.getInstanceId();
    this._LINK_PROPERTY = "__ilvTreeLayoutLink" + this.getInstanceId();
  },

  copy : function()
  {
    // summary:
    //		Copies the layout instance. 
    // description:
    //		Copies the layout instance. 
    //		<p>
    //		This method copies the layout instance by calling the copy
    //		constructor.  
    //		</p>
    //		<p>
    //		When performing a recursive layout of a nested graph, this method is
    //		used by <code> ibm&#95;ilog.graphlayout.ILayoutProvider</code> to
    //		"clone" the layout instance of a parent graph. Note that the
    //		parameters which are specific to a node or a link are not copied.
    //		Only the global parameters are copied. The layout position is not
    //		copied.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // returns:
    //		A copy of the layout instance.
    
    return new TreeLayout(this); // ibm_ilog.graphlayout.GraphLayout
  },

  copyParameters : function(source)
  {
    // summary:
    //		Copies the parameters from a given layout instance. 
    // description:
    //		Copies the parameters from a given layout instance. 
    //		<p>
    //		Note that the parameters which are specific to a node or a link are
    //		not copied. Only the global parameters are copied. The layout
    //		position is not copied.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // source: GraphLayout
    //		The layout instance from which the parameters are copied.
    
    this.inherited("copyParameters", arguments, [source]);

    if (source instanceof TreeLayout)
    {
      var tsource = source;
      this.setGlobalLinkStyle(tsource.getGlobalLinkStyle());
      this.setConnectorStyle(tsource.getConnectorStyle());
      this.setGlobalAlignment(tsource.getGlobalAlignment());
      this.setLayoutMode(tsource.getLayoutMode());
      this.setBalloonAngleSpacing(tsource.getBalloonAngleSpacing());
      this.setBalloonRadiusMode(tsource.getBalloonRadiusMode());
      this.setFlowDirection(tsource.getFlowDirection());
      this.setLevelAlignment(tsource.getLevelAlignment());
      this.setSiblingOffset(tsource.getSiblingOffset());
      this.setParentChildOffset(tsource.getParentChildOffset());
      this.setBranchOffset(tsource.getBranchOffset());
      this.setTipOverBranchOffset(tsource.getTipOverBranchOffset());
      this.setAssociateOffset(tsource.getAssociateOffset());
      this.setParentChildAssociateOffset(tsource.getParentChildAssociateOffset());
      this.setAspectRatio(tsource.getAspectRatio());
      this.setOverlapPercentage(tsource.getOverlapPercentage());
      this.setOrthForkPercentage(tsource.getOrthForkPercentage());
      this.setMaxChildrenAngle(tsource.getMaxChildrenAngle());
      this.setBalloonStartAngle(tsource.getBalloonStartAngle());
      this.setBalloonRootChildrenAngle(tsource.getBalloonRootChildrenAngle());
      this.setBalloonInnerChildrenAngle(tsource.getBalloonInnerChildrenAngle());
      this.setBalloonLeafChildrenAngle(tsource.getBalloonLeafChildrenAngle());
      this.setNodeComparator(tsource.getNodeComparator());
      this.setFirstCircleEvenlySpacing(tsource.isFirstCircleEvenlySpacing());
      this.setRespectNodeSizes(tsource.isRespectNodeSizes());
      this.setIncrementalMode(tsource.isIncrementalMode());
      this.setAllLevelsAlternating(tsource.isAllLevelsAlternating());
      this.setNumberOfAlternatingCircles(tsource.getNumberOfAlternatingCircles());
      this.setNonTreeLinksStraight(tsource.isNonTreeLinksStraight());
      this.setInvisibleRootUsed(tsource.isInvisibleRootUsed());
      this.setCategorizingLinks(tsource.isCategorizingLinks());
      this.setCategorizingLinksWithoutLayout(tsource.isCategorizingLinksWithoutLayout());
      this.setIntergraphConnectivityMode(tsource.isIntergraphConnectivityMode());
    }
  },

  layout : function()
  {
    // summary:
    //		Computes the layout using the Tree Layout algorithm. 
    // description:
    //		Computes the layout using the Tree Layout algorithm. 
    //		To start the layout, call the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.performLayout()</code>.  
    //		<p>
    //		 
    //		</p>
    // tags:
    //		protected
    
    var redraw = false;

    this.prepareBeforeLayout();
    this.layoutFlat(redraw);
  },

  layoutFlat : function(redraw)
  {
    // summary:
    //		Lays out a flat graph. 
    // description:
    //		Lays out a flat graph. 
    //		
    // tags:
    //		private
    
    var graphModel = this.getGraphModel();

    var graph = new TGraph(this);
    this.callLayoutStepPerformedIfNeeded();

    this._percCompleteControler = graph.getPercController();
    this._percCompleteControler.setStepEstimation(graph.calcInitPercParameter(0));

    graph.attach(graphModel);

    if (this.isIntergraphConnectivityMode())
    {
      graph.createConvertedIntergraphLinks();
    }

    if (this.mayContinue())
    {
      var alg = new TSpanTree(graph, this.isIntergraphConnectivityMode());
      alg.run();
      alg.disposeIt();
    }

    if (this.isCategorizingLinksWithoutLayout())
    {
      graph.registerCalcRootsAndLinkCategories();
      graph.dispose();
    }
    else
    {
      this.layoutSpanningTreeFlat(graph);

      this._allowStopImmediately = false;
      graph.detach(redraw);
      graph.dispose();

      if (this.mayContinue())
      {
        this.routeSplines(redraw, graph.getPercSplineRoute(), true, false);
      }
    }

    this.updateReportAfterLayout();
  },

  routeSplines : function(redraw, percentage, normalLinks, intergraphLinks)
  {
    // summary:
    //		Fixes the link shaped if they are spline. 
    // description:
    //		Fixes the link shaped if they are spline. 
    //		This is the flat version.   
    // tags:
    //		private
    
    var gbls = this.getGlobalLinkStyle();

    if (this.isSplineRoutingEnabled() && gbls != TreeLayout.NO_RESHAPE_STYLE && gbls != TreeLayout.STRAIGHT_LINE_STYLE)
    {
      var router = new TreeLayoutAnonClass_1(this, this._percCompleteControler, percentage, this.getMinSplineCurveSize(), this.getMaxSplineCurveSize(), this.getBalanceSplineCurveThreshold(),
          normalLinks, intergraphLinks, this);

      router.run(redraw, true);
    }
  },

  prepareBeforeLayout : function()
  {
    // summary:
    //		Before doing the real layout some generic preparations are needed. 
    // description:
    //		Before doing the real layout some generic preparations are needed. 
    //		
    // tags:
    //		private
    
    this._stoppedBeforeCompletion = false;
    this._allowStopImmediately = true;

    this.cleanCalcRoots();
    this.cleanCalcForwardTreeLinks();
    this.cleanCalcBackwardTreeLinks();
    this.cleanCalcNonTreeLinks();
  },

  layoutSpanningTreeFlat : function(graph)
  {
    // summary:
    //		Perform the layout on the spanning tree. 
    // description:
    //		Perform the layout on the spanning tree. 
    //		This variant is flat.   
    // tags:
    //		private
    
    if (graph.getNumberOfNodes() == 0) {
      return;
    }

    var mode = this.getLayoutMode();

    if (this.mayContinue() && mode != TreeLayout.BALLOON)
    {
      graph.calcNorthSizeExtension();
    }

    if (mode == TreeLayout.BALLOON)
    {
      var algballoon = new TBalloon(graph);
      algballoon.setParameters(this.getBalloonAngleSpacing(), this.getBalloonRadiusMode(), this.getBranchOffset(), this.getParentChildOffset(), this.getAspectRatio(), this.getBalloonStartAngle(),
          this.getBalloonRootChildrenAngle(), this.getBalloonInnerChildrenAngle(), this.getBalloonLeafChildrenAngle(), this.isRespectNodeSizes());
      algballoon.run();
    }
    else if (mode == TreeLayout.TIP_OVER || mode == TreeLayout.TIP_LEAVES_OVER || mode == TreeLayout.TIP_ROOTS_OVER || mode == TreeLayout.TIP_ROOTS_AND_LEAVES_OVER)
    {
      var alg = new TAutoTipOver(graph);
      alg.run(mode);
      alg.disposeIt();
    }
    else
    {
      if (this.mayContinue())
      {
        graph.calcPrelimCoordInDirTowardsLeaves();
      }

      if (this.mayContinue())
      {
        graph.calcCoords();
      }

      if (this.mayContinue())
      {
        graph.calcFinalCoordInDirTowardsLeaves();
      }

      if (this.mayContinue())
      {
        graph.calcFinalCoordRadial();
      }
    }

    graph.finalPositionCorrection();
  },

  updateReportAfterLayout : function()
  {
    // summary:
    //		At the end of layout, set 100% complete, set the layout return code
    //		and cleanup. 
    // description:
    //		At the end of layout, set 100% complete, set the layout return code
    //		and cleanup. 
    //		
    // tags:
    //		private
    
    this._percCompleteControler.stopAll();
    this.increasePercentageComplete(100);
    this.onLayoutStepPerformed(false, false);

    var report = this.getLayoutReport();
    if (this._stoppedBeforeCompletion) {
      report.setCode(GraphLayoutReport.STOPPED_AND_INVALID);
    }
    else {
      report.setCode(GraphLayoutReport.LAYOUT_DONE);
    }
    this._allowStopImmediately = true;
  },

  updateAngleTransTable : function(graph)
  {
    // summary:
    //		Updates the ellipsoid angle translation table with a new aspect
    //		ratio. 
    // description:
    //		Updates the ellipsoid angle translation table with a new aspect
    //		ratio. 
    //		This is only needed in the radial modes.   
    // tags:
    //		private
    
    var aspectRatio = graph.getAspectRatio();

    if (graph.isRadialLayout() && aspectRatio != this._angleTransTableAspectRatio)
    {
      this._angleTransTable = TRadial.getEllipsoidTransTable(aspectRatio);
      this._angleTransTableAspectRatio = aspectRatio;
      graph.setAngleTranslationTable(this._angleTransTable);
    }
  },

  detach : function()
  {
    // summary:
    //		Detaches the graph model from the layout instance. 
    // description:
    //		Detaches the graph model from the layout instance. 
    //		When you attach a new graph model to the layout instance, you do not
    //		need to detach the old graph model because this is done
    //		automatically when you call <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.attach()</code>. The detach method
    //		performs cleaning operations on the graph model. In addition to the
    //		cleaning operations in the base class, the Tree Layout removes the
    //		alignment, root preference, and node neighboring specifications of
    //		individual nodes, and the link style specifications of individual
    //		links.  
    //		<p>
    //		Note that you must call this method when you no longer need the
    //		layout instance. Otherwise, some objects may not be garbage
    //		collected.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    this.cleanSpecRoots();
    this.cleanCalcRoots();
    this.cleanCalcForwardTreeLinks();
    this.cleanCalcBackwardTreeLinks();
    this.cleanCalcNonTreeLinks();

    this.inherited("detach", arguments, []);
  },

  cleanNode : function(graphModel, node)
  {
    // summary:
    //		Cleans a node. 
    // description:
    //		Cleans a node. 
    //		This method removes any data that has been stored by the layout
    //		algorithm on a node. In addition to the cleaning operations in the
    //		base class, Tree Layout removes the root node preference and the
    //		alignment and east/west attachment settings from the nodes.  
    //		<p>
    //		Subclasses can override this method to remove additional information
    //		stored in the nodes.  
    //		</p>
    // graphModel: AbstractGraphLayoutModel
    //		The graph model to which 'node' belongs.
    // node: Object
    //		The node to be cleaned.
    
    this.inherited("cleanNode", arguments, [graphModel, node]);
    this.removeSpecRoot(node);
    this.removeCalcRoot(node);
    TNodeParameters.dispose(this, node);
  },

  cleanLink : function(graphModel, link)
  {
    // summary:
    //		Cleans a link. 
    // description:
    //		Cleans a link. 
    //		This method removes any data that has been stored by the layout
    //		algorithm on a link. In addition to the cleaning operations in the
    //		base class, Tree Layout removes the link style settings from the
    //		nodes.  
    //		<p>
    //		Subclasses can override this method to remove additional information
    //		stored in the links.  
    //		</p>
    // graphModel: AbstractGraphLayoutModel
    //		The graph model to which 'link' belongs.
    // link: Object
    //		The link to be cleaned.
    
    this.inherited("cleanLink", arguments, [graphModel, link]);
    this.removeCalcLinkFromCategory(link);
    TLinkParameters.dispose(this, link);
  },

  supportsPreserveFixedNodes : function()
  {
    // summary:
    //		Indicates that this layout class allows the user to specify fixed
    //		nodes. 
    // description:
    //		Indicates that this layout class allows the user to specify fixed
    //		nodes. 
    //		<p>
    //		Fixed nodes are not moved during the layout if the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setPreserveFixedNodes()</code> is
    //		called with a <code>true</code> argument. Links that are incident to
    //		fixed nodes are not reshaped during the layout. Fixed nodes are
    //		ignored by the Tree Layout algorithm, which may lead to node
    //		overlaps. The effect is very similar to the effect when the node is
    //		filtered during layout.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsPreserveFixedLinks : function()
  {
    // summary:
    //		Indicates that this layout class allows the user to specify fixed
    //		links. 
    // description:
    //		Indicates that this layout class allows the user to specify fixed
    //		links. 
    //		Fixed links are not reshaped during the layout if the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setPreserveFixedLinks()</code> is
    //		called with a <code>true</code> argument. Fixed links are ignored by
    //		the Tree Layout algorithm, which may lead to overlaps. The effect is
    //		very similar to the effect when the link is filtered during layout. 
    //		
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsAllowedTime : function()
  {
    // summary:
    //		Indicates that this layout class can stop the layout computation in
    //		a proper manner when the user-defined allowed time is exceeded. 
    // description:
    //		Indicates that this layout class can stop the layout computation in
    //		a proper manner when the user-defined allowed time is exceeded. 
    //		If the allowed time elapses before termination of the Tree Layout
    //		algorithm, the nodes and links are not moved and remain at the same
    //		position as before the start of the algorithm. The result code in
    //		the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code> in
    //		this case.  
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsStopImmediately : function()
  {
    // summary:
    //		Indicates that this layout class can interrupt the current run of
    //		the layout immediately in a controlled way. 
    // description:
    //		Indicates that this layout class can interrupt the current run of
    //		the layout immediately in a controlled way. 
    //		If the algorithm is stopped before termination of the Tree Layout
    //		algorithm, the nodes and links are not moved and remain at the same
    //		position as before the start of the algorithm. The result code in
    //		the layout report is <code>
    //		ibm&#95;ilog.graphlayout.GraphLayoutReport.STOPPED&#95;AND&#95;INVALID</code> in
    //		this case.  
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  stopImmediately : function()
  {
    // summary:
    //		Stops the running layout algorithm as soon as possible. 
    // description:
    //		Stops the running layout algorithm as soon as possible. 
    //		The Tree Layout algorithm has several steps. During the early steps,
    //		the layout algorithm can be stopped and the method returns
    //		<code>true</code>. In the last step, the layout algorithm cannot be
    //		stopped and the method returns <code>false</code>.  
    //		<p>
    //		 
    //		</p>
    // returns:
    //		'true' during the early steps of the layout algorithm
    //		and 'false' in the last step when the layout algorithm
    //		cannot be stopped.
    
    if (this.isStoppedImmediately()) {
      return true; // Boolean
    }
    if (!this._allowStopImmediately) {
      return false; // Boolean
    }
    return this.inherited("stopImmediately", arguments, []); // Boolean
  },

  supportsPercentageComplete : function()
  {
    // summary:
    //		Indicates that this layout class can estimate the percentage of
    //		completion during the run of the layout. 
    // description:
    //		Indicates that this layout class can estimate the percentage of
    //		completion during the run of the layout. 
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsLayoutOfConnectedComponents : function()
  {
    // summary:
    //		Indicates that this layout class can use the generic connected
    //		component layout mechanism of the 'GraphLayout' base
    //		class. 
    // description:
    //		Indicates that this layout class can use the generic connected
    //		component layout mechanism of the <code>GraphLayout</code> base
    //		class. 
    //		This mechanism cuts the attached graph into connected components,
    //		applies itself to each connected component separately, and then uses
    //		the layout instance returned by the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.getLayoutOfConnectedComponents()</code>
    //		to position the connected components. By default, this layout is an
    //		instance of <code> ibm&#95;ilog.graphlayout.grid.GridLayout</code> which
    //		can be customized as needed.  
    //		<p>
    //		The generic connected component layout mechanism has the
    //		disadvantage that it moves connected components completely. Fixed
    //		nodes within a component do not preserve their old positions, and
    //		the resulting layout may be unstable on incremental changes in
    //		incremental mode, depending on which layout instance is used for the
    //		component layout.  
    //		</p>
    //		<p>
    //		If the generic connected component layout mechanism is disabled, the
    //		algorithm uses its own specialized internal mechanism instead of the
    //		generic mechanism to lay out each component as a separate tree. This
    //		is usually faster and more stable on incremental changes than the
    //		generic mechanism. In addition, it enables the user to set the
    //		position of the layout.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsLinkConnectionBox : function()
  {
    // summary:
    //		Indicates that this layout class can use a link connection box
    //		interface to calculate the end points of links. 
    // description:
    //		Indicates that this layout class can use a link connection box
    //		interface to calculate the end points of links. 
    //		The link connection box interface is an object that provides the
    //		rectangle to which the links are connected for each node and the
    //		tangential shift offset at each side for the connection points. This
    //		is useful if the connection points of the links at a node should be
    //		relative to a rectangle that is smaller or larger than the real
    //		bounding box of the node.  
    //		<p>
    //		Link connection points are not calculated for links that have the
    //		no-reshape link style, that are fixed, or that are not tree links.
    //		Only the connection points of reshapeable tree links are calculated.
    //		 
    //		</p>
    //		<p>
    //		If a connection box interface object is set using the method <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setLinkConnectionBoxInterface()</code>,
    //		the layout algorithm uses this object together with the connector
    //		style to calculate the connection points for links.  
    //		</p>
    //		<p>
    //		If a link clip interface is set additionally, the layout algorithm
    //		calculates the initial connection points at the node boxes with the
    //		link connection box interface and the connector style and obtains
    //		the final connection points at the border of the node shapes by
    //		clipping with the link clip interface. This works well particularly
    //		if the node box obtained by the link connection box interface is
    //		smaller than the real bounding box of the node.   
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    return true; // Boolean
  },

  supportsSplineRouting : function()
  {
    // summary:
    //		Tests if this class supports the generic optimization of spline
    //		control points. 
    // description:
    //		Tests if this class supports the generic optimization of spline
    //		control points. 
    //		<p>
    //		Due to the nature of a tree, the optimized spline routing is less
    //		seldom needed for tree layout, while it is highly useful for other
    //		layout styles such as hierarchical layout. Optimized spline routing
    //		is offered for tree layout for consistency reasons.  
    //		</p>
    
    return true; // Boolean
  },

  setGlobalLinkStyle : function(style)
  {
    // summary:
    //		Sets the global style of the shapes of links. 
    // description:
    //		Sets the global style of the shapes of links. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.ORTHOGONAL&#95;STYLE</code> - all links have an
    //		orthogonal shape, that is, a shape consisting of a sequence of
    //		orthogonal line segments. This option is not available for the
    //		radial layout modes.</li>  
    //		<li> <code> TreeLayout.STRAIGHT&#95;LINE&#95;STYLE</code> - all links have a
    //		straight-line shape.</li>  
    //		<li> <code> TreeLayout.NO&#95;RESHAPE&#95;STYLE</code> - no reshape is
    //		performed on any link.</li>  
    //		<li> <code> TreeLayout.MIXED&#95;STYLE</code> - each link can have a
    //		different link style. The style of each individual link can be set
    //		by the method <code> TreeLayout.setLinkStyle()</code> so that
    //		different link shapes can occur in the same graph.</li>  
    //		</ul>  
    //		<p>
    //		Unless the global link style is <code>
    //		TreeLayout.MIXED&#95;STYLE</code>, all links have the same shape style. 
    //		
    //		</p>
    //		<p>
    //		The default value is <code> TreeLayout.STRAIGHT&#95;LINE&#95;STYLE</code>.  
    //		
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (!(style == TreeLayout.ORTHOGONAL_STYLE || style == TreeLayout.STRAIGHT_LINE_STYLE || style == TreeLayout.MIXED_STYLE || style == TreeLayout.NO_RESHAPE_STYLE))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6700E",  style);
    }

    if (style != this._globalLinkStyle)
    {
      this._globalLinkStyle = style;
      this.onParameterChanged("GlobalLinkStyle");
    }
  },

  getGlobalLinkStyle : function()
  {
    // summary:
    //		Returns the global style of the shapes of links. 
    // description:
    //		Returns the global style of the shapes of links. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultLinkStyle; // int
    }
    return this._globalLinkStyle; // int
  },

  setConnectorStyle : function(style)
  {
    // summary:
    //		Sets the style of the connectors. 
    // description:
    //		Sets the style of the connectors. 
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.CENTERED&#95;PINS</code> - the link connection
    //		points are placed in the center of the border of the node bounding
    //		box to which the link is attached.</li>   
    //		<li> <code> TreeLayout.CLIPPED&#95;PINS</code> - each link pointing to
    //		the center of the node is clipped at the bounding box of the node.
    //		This style affects straight links; it behaves like centered
    //		connection points for orthogonal links.</li>   
    //		<li> <code> TreeLayout.EVENLY&#95;SPACED&#95;PINS</code> - the link
    //		connection points are evenly spaced along the border of the node
    //		bounding box.</li>  
    //		<li> <code> TreeLayout.AUTOMATIC&#95;PINS</code> - the connector style
    //		is selected depending on the link style.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code> TreeLayout.AUTOMATIC&#95;PINS</code>.  
    //		</p>
    //		<p>
    //		The connector style is ignored in the radial layout modes because
    //		clipped pins are always used in this case . The placement of
    //		connection points also depends on  the connection box interface, in
    //		the following way: 
    //		<ol>
    //		 
    //		<li> First, the bounding box of an end node is obtained. If a link
    //		connection box interface was set via <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setLinkConnectionBoxInterface()</code>,
    //		the bounding box and offsets are delivered by that interface.
    //		Otherwise, the node bounding box delivered by the graph model (see 
    //		<code>IGraphLayoutModel.getObjectBounds(Object)</code>) is used.
    //		</li>  
    //		<li> Next, the initial connection point of the link is placed at
    //		this bounding box in the manner specified by the connector style.
    //		</li> 
    //		</ol>   
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (!(style == TreeLayout.AUTOMATIC_PINS || style == TreeLayout.CENTERED_PINS || style == TreeLayout.CLIPPED_PINS || style == TreeLayout.EVENLY_SPACED_PINS))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6701E",  style);
    }

    if (style != this._connectorStyle)
    {
      this._connectorStyle = style;
      this.onParameterChanged("ConnectorStyle");
    }
  },

  getConnectorStyle : function()
  {
    // summary:
    //		Returns the style of the connectors. 
    // description:
    //		Returns the style of the connectors. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultConnectorStyle; // int
    }
    return this._connectorStyle; // int
  },

  setFlowDirection : function(direction)
  {
    // summary:
    //		Sets the direction of the flow. 
    // description:
    //		Sets the direction of the flow. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> ibm&#95;ilog.graphlayout.Direction.RIGHT</code> - the root
    //		is on the left and the links flow to the right.</li>  
    //		<li> <code> ibm&#95;ilog.graphlayout.Direction.LEFT</code> - the root is
    //		on the right and the links flow to the left.</li>  
    //		<li> <code> ibm&#95;ilog.graphlayout.Direction.BOTTOM</code> - the root
    //		is at the top and the links flow to the bottom.</li>  
    //		<li> <code> ibm&#95;ilog.graphlayout.Direction.TOP</code> - the root is
    //		at the bottom and the links flow to the top.</li>  
    //		</ul>  
    //		<p>
    //		Note that except for the radial layout modes, the compass direction
    //		<code> TreeLayout.SOUTH</code> is identical to the flow direction.  
    //		</p>
    //		<p>
    //		In the radial layout modes, all links flow radially from the center
    //		of the root outward. If connected component processing is disabled,
    //		the flow direction affects only the arrangement of disconnected
    //		trees in the radial modes.  
    //		</p>
    //		<p>
    //		The default value is <code>
    //		ibm&#95;ilog.graphlayout.Direction.RIGHT</code> .  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (!(direction == Direction.LEFT || direction == Direction.RIGHT || direction == Direction.TOP || direction == Direction.BOTTOM))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6702E",  direction);
    }

    if (direction != this._flowDirection)
    {
      this._flowDirection = direction;
      this.onParameterChanged("FlowDirection");
    }
  },

  getFlowDirection : function()
  {
    // summary:
    //		Returns the current direction of the flow. 
    // description:
    //		Returns the current direction of the flow. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultFlowDirection; // int
    }
    return this._flowDirection; // int
  },

  setLayoutMode : function(mode)
  {
    // summary:
    //		Sets the layout mode. 
    // description:
    //		Sets the layout mode. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.FREE</code> - all links flow roughly in the
    //		flow direction. Nodes of different tree branches (that is, nodes
    //		with a different parent node) are not justified to each other;
    //		therefore, the level alignment parameter is ignored. Tip-over
    //		alignment is possible if specified as alignment. All link styles are
    //		possible. The aspect ratio is ignored.</li>  
    //		<li> <code> TreeLayout.LEVEL</code> - all links flow roughly in the
    //		flow direction. Nodes are organized in levels and are justified to
    //		their borders or centered according to the level alignment
    //		parameter. Tip-over alignment is not possible, but all link styles
    //		are possible. The aspect ratio is ignored.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER</code> - similar to the mode <code>
    //		TreeLayout.FREE</code>. However, in mode <code>
    //		TreeLayout.TIP&#95;OVER</code>, the algorithm tries to optimize the
    //		layout to automatically fit best to the specified aspect ratio.
    //		Using a fast heuristic, it chooses the local alignment <code>
    //		TreeLayout.TIP&#95;OVER&#95;EAST</code> for some nodes but keeps the
    //		alignment of all other nodes as specified. This heuristic is a fast
    //		compromise between the modes <code>
    //		TreeLayout.TIP&#95;LEAVES&#95;OVER</code>, <code>
    //		TreeLayout.TIP&#95;ROOTS&#95;OVER</code>, and <code>
    //		TreeLayout.TIP&#95;ROOTS&#95;AND&#95;LEAVES&#95;OVER</code>.</li>  
    //		<li> <code> TreeLayout.TIP&#95;LEAVES&#95;OVER</code> - similar to the mode
    //		<code> TreeLayout.TIP&#95;OVER</code>. The algorithm tries to optimize
    //		the layout to automatically fit best to the specified aspect ratio.
    //		This slow heuristic tries to tip over beginning with the leaves and
    //		then chooses the best layout. It checks the alignment mode <code>
    //		TreeLayout.TIP&#95;OVER&#95;EAST</code>, but not <code>
    //		TreeLayout.TIP&#95;OVER&#95;BOTH&#95;SIDES</code> or other variants. It keeps
    //		the alignment of all nodes close to the root as specified.</li>  
    //		<li> <code> TreeLayout.TIP&#95;ROOTS&#95;OVER</code> - similar to the mode
    //		<code> TreeLayout.TIP&#95;OVER</code>. The algorithm tries to optimize
    //		the layout to automatically fit best to the specified aspect ratio.
    //		This slow heuristic tries to tip over beginning with the roots and
    //		then chooses the best layout. It checks the alignment mode <code>
    //		TreeLayout.TIP&#95;OVER&#95;EAST</code>, but not <code>
    //		TreeLayout.TIP&#95;OVER&#95;BOTH&#95;SIDES</code> or other variants. It keeps
    //		the alignment of all nodes close to the leaves as specified.</li>  
    //		<li> <code> TreeLayout.TIP&#95;ROOTS&#95;AND&#95;LEAVES&#95;OVER</code> - similar to
    //		the mode <code> TreeLayout.TIP&#95;OVER</code>. The algorithm tries to
    //		optimize the layout to automatically fit best to the specified
    //		aspect ratio. This slow heuristic tries to tip over beginning with
    //		the roots and with the leaves, and then chooses the best layout. It
    //		checks the alignment mode <code> TreeLayout.TIP&#95;OVER&#95;EAST</code>,
    //		but not <code> TreeLayout.TIP&#95;OVER&#95;BOTH&#95;SIDES</code> or other
    //		variants. It keeps the alignment of all nodes in the middle between
    //		roots and leaves as specified.</li>  
    //		<li> <code> TreeLayout.RADIAL</code> - the root node is in the
    //		center, the links flow radially away from the center, and the nodes
    //		are placed in circular layers around the root node according to the
    //		level alignment. Tip-over alignment and orthogonal link styles are
    //		not possible. Clipped connection points are used as the connector
    //		style. If an aspect ratio is specified, the layers are ellipses
    //		around the center.</li>  
    //		<li> <code> TreeLayout.ALTERNATING&#95;RADIAL</code> - this is the same
    //		as mode <code> TreeLayout.RADIAL</code>, with the exception that
    //		children of the same node are placed alternating in different
    //		circular layers when this results in a smaller radius and better
    //		usage of the space. Tip-over alignment and orthogonal link styles
    //		are not possible. Clipped connection points are used as the
    //		connector style. If an aspect ratio is specified, the layers are
    //		ellipses around the center.</li>  
    //		<li> <code> TreeLayout.BALLOON</code> - the root node is in the
    //		center and the subtrees are arranged as balloon circles around the
    //		root. It is very similar to the mode <code>
    //		TreeLayout.RADIAL</code>, however each subtree froms its own balloon
    //		recursively. If an aspect ratio is specified, the balloons are
    //		ellipses instead of circles. Tip-over alignment and orthogonal link
    //		styles are not possible. Clipped connection points are used as the
    //		connector style. The recursive leaf layout mode is not supported in 
    //		combination with the balloon mode. </li>  
    //		</ul>  
    //		<p>
    //		The default value is <code> TreeLayout.FREE</code>.  
    //		</p>
    //		<p>
    //		Note that the tip-over layout modes work best if the node sizes are
    //		respected during layout.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (!
      (mode == TreeLayout.TIP_OVER || mode == TreeLayout.TIP_LEAVES_OVER || mode == TreeLayout.TIP_ROOTS_OVER || mode == TreeLayout.TIP_ROOTS_AND_LEAVES_OVER || mode == TreeLayout.FREE ||
          mode == TreeLayout.LEVEL || mode == TreeLayout.RADIAL || mode == TreeLayout.ALTERNATING_RADIAL || mode == TreeLayout.BALLOON))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6703E",  mode);
    }

    if (mode != this._layoutMode)
    {
      this._layoutMode = mode;
      this.onParameterChanged("LayoutMode");
    }
  },

  getLayoutMode : function()
  {
    // summary:
    //		Returns the current layout mode. 
    // description:
    //		Returns the current layout mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultLayoutMode; // int
    }
    return this._layoutMode; // int
  },

  setBalloonAngleSpacing : function(angleSpacing)
  {
    // summary:
    //		Sets the current angle spacing for balloon layout mode. 
    // description:
    //		Sets the current angle spacing for balloon layout mode. 
    //		This setting has only an effect when the layout mode is <code>
    //		TreeLayout.BALLOON</code>. Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.REGULAR</code> - the available angle range at
    //		a node is evenly split so that each subtrees balloon at the node can
    //		occupy the same angle. </li>  
    //		<li> <code> TreeLayout.FAST&#95;PROPORTIONAL</code> - the available
    //		angle range at a node is proportonally split so that larger subtrees
    //		balloons at the node can occupy larger angles. The algorithm uses a
    //		fast heuristic to calculate the angle ranges. This heuristic is not
    //		very precise. </li>  
    //		<li> <code> TreeLayout.PROPORTIONAL</code> - the available angle
    //		range at a node is proportonally split so that larger subtrees
    //		balloons at the node can occupy larger angles. The algorithm uses a
    //		slower heuristic to calculate the angle ranges. This heuristic is
    //		more precise than the fast heuristic. </li>  
    //		</ul>  
    //		<p>
    //		The default value is <code> TreeLayout.PROPORTIONAL</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (!(angleSpacing == TreeLayout.REGULAR || angleSpacing == TreeLayout.FAST_PROPORTIONAL || angleSpacing == TreeLayout.PROPORTIONAL))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6704E",  angleSpacing);
    }

    if (angleSpacing != this._balloonAngleSpacing)
    {
      this._balloonAngleSpacing = angleSpacing;
      this.onParameterChanged("BalloonAngleSpacing");
    }
  },

  getBalloonAngleSpacing : function()
  {
    // summary:
    //		Returns the current angle spacing for balloon layout mode. 
    // description:
    //		Returns the current angle spacing for balloon layout mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultBalloonAngleSpacing; // int
    }
    return this._balloonAngleSpacing; // int
  },

  setBalloonRadiusMode : function(radiusMode)
  {
    // summary:
    //		Sets the current radius mode for balloon layout mode. 
    // description:
    //		Sets the current radius mode for balloon layout mode. 
    //		This setting has only an effect when the layout mode is <code>
    //		TreeLayout.BALLOON</code>. Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.VARIABLE&#95;RADIUS</code> - the radius of
    //		different child balloons at the same parent node can vary. </li>  
    //		<li> <code> TreeLayout.OPTIMIZED&#95;VARIABLE&#95;RADIUS</code> - the radius
    //		of different child balloons at the same parent node can vary. An
    //		additional optimization heuristic reduces the radius. This has the
    //		advantage of smaller space requirements but looses symmetries in the
    //		diagram. </li>  
    //		<li> <code> TreeLayout.UNIFORM&#95;LEAVES&#95;RADIUS</code> - the radiuses
    //		of all leaves that are children of the same node are uniform. The
    //		radius of balloons that contain more than one node can still vary.
    //		Use this mode if symmetries of the layout are important. </li>  
    //		<li> <code> TreeLayout.OPTIMIZED&#95;UNIFORM&#95;LEAVES&#95;RADIUS</code> - the
    //		radiuses of all leaves that are children of the same node are
    //		uniform. The radius of balloons that contain more than one node can
    //		still vary. An additional optimization heuristic reduces the radius.
    //		This has the advantage of smaller space requirements but looses
    //		symmetries in the diagram. </li>  
    //		<li> <code> TreeLayout.UNIFORM&#95;RADIUS</code> - the radiuses of all
    //		subtree balloons that are children of the same node are uniform.
    //		This mode requires the most area but also shows the most symmetries
    //		in the layout. </li>  
    //		</ul>  
    //		<p>
    //		The default value is <code>
    //		TreeLayout.OPTIMIZED&#95;UNIFORM&#95;LEAVES&#95;RADIUS</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (
      !
        (radiusMode == TreeLayout.VARIABLE_RADIUS || radiusMode == TreeLayout.OPTIMIZED_VARIABLE_RADIUS || radiusMode == TreeLayout.OPTIMIZED_UNIFORM_LEAVES_RADIUS ||
            radiusMode == TreeLayout.UNIFORM_LEAVES_RADIUS || radiusMode == TreeLayout.UNIFORM_RADIUS))
      {
        LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6705E",  radiusMode);
      }

    if (radiusMode != this._balloonRadiusMode)
    {
      this._balloonRadiusMode = radiusMode;
      this.onParameterChanged("BalloonRadiusMode");
    }
  },

  getBalloonRadiusMode : function()
  {
    // summary:
    //		Returns the radius mode for balloon layout mode. 
    // description:
    //		Returns the radius mode for balloon layout mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultBalloonRadiusMode; // int
    }
    return this._balloonRadiusMode; // int
  },

  setBalloonStartAngle : function(angle)
  {
    // summary:
    //		Sets the start angle of the layout in balloon mode. 
    // description:
    //		Sets the start angle of the layout in balloon mode. 
    //		The angle must be between 0 and 360 degrees.  
    //		<p>
    //		The default value is 0.  
    //		</p>
    //		<p>
    //		The angle orientation is clockwise for flow direction left and top
    //		and counter clockwise for flow direction right and bottom. The start
    //		angle 0 is always towards the right.  
    //		</p>
    //		<p>
    //		This is an expert option. It has no effect in layout modes other
    //		than balloon.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (angle < 0) {
      angle = 0;
    }
    if (angle > 360) {
      angle = 360;
    }
    if (angle != this._balloonStartAngle)
    {
      this._balloonStartAngle = angle;
      this.onParameterChanged("BalloonStartAngle");
    }
  },

  getBalloonStartAngle : function()
  {
    // summary:
    //		Returns the start angle of the layout in balloon mode. 
    // description:
    //		Returns the start angle of the layout in balloon mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultBalloonStartAngle; // int
    }
    return this._balloonStartAngle; // int
  },

  setBalloonRootChildrenAngle : function(angle)
  {
    // summary:
    //		Sets the maximum angle for children of the root node in balloon
    //		mode. 
    // description:
    //		Sets the maximum angle for children of the root node in balloon
    //		mode. 
    //		This is the maximum angle between the two rays from the parent to
    //		its two outermost children. If the angle is 360 degree, outermost
    //		children do not really esist as the children cover the entire circle
    //		around the root. The angle must be between 1 and 360 degrees.  
    //		<p>
    //		The default value is 360.  
    //		</p>
    //		<p>
    //		This is an expert option. It has no effect in layout modes other
    //		than balloon.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (angle < 1) {
      angle = 1;
    }
    if (angle > 360) {
      angle = 360;
    }
    if (angle != this._balloonRootChildrenAngle)
    {
      this._balloonRootChildrenAngle = angle;
      this.onParameterChanged("BalloonRootChildrenAngle");
    }
  },

  getBalloonRootChildrenAngle : function()
  {
    // summary:
    //		Returns the maximum angle for children of the root node in balloon
    //		mode. 
    // description:
    //		Returns the maximum angle for children of the root node in balloon
    //		mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultBalloonRootChildrenAngle; // int
    }
    return this._balloonRootChildrenAngle; // int
  },

  setBalloonInnerChildrenAngle : function(angle)
  {
    // summary:
    //		Sets the maximum angle for children of inner nodes in balloon mode. 
    // description:
    //		Sets the maximum angle for children of inner nodes in balloon mode. 
    //		This is the maximum angle between the two rays from the parent to
    //		its two outermost children. If the angle is 360 degree, outermost
    //		children do not really esist as the children cover the entire circle
    //		around the node. The angle must be between 1 and 360 degrees.  
    //		<p>
    //		The default value is 360.  
    //		</p>
    //		<p>
    //		This is an expert option. It has no effect in layout modes other
    //		than balloon.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (angle < 1) {
      angle = 1;
    }
    if (angle > 360) {
      angle = 360;
    }
    if (angle != this._balloonInnerChildrenAngle)
    {
      this._balloonInnerChildrenAngle = angle;
      this.onParameterChanged("BalloonInnerChildrenAngle");
    }
  },

  getBalloonInnerChildrenAngle : function()
  {
    // summary:
    //		Returns the maximum angle for children of inner nodes in balloon
    //		mode. 
    // description:
    //		Returns the maximum angle for children of inner nodes in balloon
    //		mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultBalloonInnerChildrenAngle; // int
    }
    return this._balloonInnerChildrenAngle; // int
  },

  setBalloonLeafChildrenAngle : function(angle)
  {
    // summary:
    //		Sets the maximum angle for children of nodes that have only leaves
    //		as children in balloon mode. 
    // description:
    //		Sets the maximum angle for children of nodes that have only leaves
    //		as children in balloon mode. 
    //		Basically, it is the maximum angle for the leaf balloons. This is
    //		the maximum angle between the two rays from the parent to its two
    //		outermost children. If the angle is 360 degree, outermost children
    //		do not really esist as the children cover the entire circle around
    //		the node. The angle must be between 1 and 360 degrees.  
    //		<p>
    //		The default value is 360.  
    //		</p>
    //		<p>
    //		This is an expert option. It has no effect in layout modes other
    //		than balloon.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (angle < 1) {
      angle = 1;
    }
    if (angle > 360) {
      angle = 360;
    }
    if (angle != this._balloonLeafChildrenAngle)
    {
      this._balloonLeafChildrenAngle = angle;
      this.onParameterChanged("BalloonLeafChildrenAngle");
    }
  },

  getBalloonLeafChildrenAngle : function()
  {
    // summary:
    //		Returns the maximum angle for children of nodes that have only
    //		leaves as children in balloon mode. 
    // description:
    //		Returns the maximum angle for children of nodes that have only
    //		leaves as children in balloon mode. 
    //		Basically, it is the maximum angle for the leaf balloons.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultBalloonLeafChildrenAngle; // int
    }
    return this._balloonLeafChildrenAngle; // int
  },

  setIncrementalMode : function(enable)
  {
    // summary:
    //		Sets the incremental mode. 
    // description:
    //		Sets the incremental mode. 
    //		<p>
    //		If the incremental mode is enabled, the algorithm tries to preserve
    //		the relative order of the children on incremental changes, unless a
    //		node comparator is specified. The logical order of children of nodes
    //		in the tree is calculated from their coordinates. In this case, the
    //		layout is stable when incremental changes are made. If nodes are
    //		added or removed after a layout, the next layout uses a heuristic to
    //		try to adjust the layout only minimally and to avoid major
    //		structural reorganization.  
    //		</p>
    //		<p>
    //		If the incremental mode is disabled, the algorithm calculates the
    //		logical order of children of nodes in the tree by using the order in
    //		which nodes are returned by <code>
    //		ibm&#95;ilog.graphlayout.AbstractGraphLayoutModel.getNodes()</code>.  
    //		</p>
    //		<p>
    //		If a node comparator is specified, the algorithm calculate the
    //		logical order of children of nodes in the tree from the node
    //		comparator, independent from whether the incremental mode is enabled
    //		or disabled.  
    //		</p>
    //		<p>
    //		The incremental mode is enabled by default.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (enable != this._incrementalMode)
    {
      this._incrementalMode = enable;
      this.onParameterChanged("IncrementalMode");
    }
  },

  isIncrementalMode : function()
  {
    // summary:
    //		Returns 'true' if the incremental mode is enabled. 
    // description:
    //		Returns <code>true</code> if the incremental mode is enabled. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultIncrementalMode; // Boolean
    }
    return this._incrementalMode; // Boolean
  },

  setNodeComparator : function(comparator)
  {
    // summary:
    //		Sets the comparator used for sorting the nodes. 
    // description:
    //		Sets the comparator used for sorting the nodes. 
    //		If the value <code>null</code> is passed, no comparator is used. If
    //		a node comparator is specified, the algorithm calculates the logical
    //		order of children of nodes in the tree by using this comparator.  
    //		<p>
    //		The default value is <code>null</code> (no comparator).  
    //		</p>
    
    if (comparator != this._nodeComparator)
    {
      this._nodeComparator = comparator;
      this.onParameterChanged("NodeComparator");
    }
  },

  getNodeComparator : function()
  {
    // summary:
    //		Returns the comparator used for sorting the nodes. 
    // description:
    //		Returns the comparator used for sorting the nodes. 
    //		<p>
    //		 
    //		</p>
    
    return this._nodeComparator; // Function
  },

  setAllLevelsAlternating : function(enable)
  {
    // summary:
    //		Sets whether at all levels, nodes are forced into alternating
    //		circular layers. 
    // description:
    //		Sets whether at all levels, nodes are forced into alternating
    //		circular layers. 
    //		<p>
    //		This option has only an effect in layout mode <code>
    //		TreeLayout.ALTERNATING&#95;RADIAL</code>. Alternating placement means
    //		that the nodes of one level are placed alternating in several
    //		(normally 2) circular layers. Nonalternating  placement means that
    //		the nodes of one level are placed in one circular layer. Normally,
    //		the layout analyses which levels must use the alternating placement
    //		in order to optimize space. If the alternating placement of a level
    //		does not improve the space usage, the nonalternating placement is
    //		used for the level even in alternating radial layout mode.  
    //		</p>
    //		<p>
    //		This option allows to change this behavior: if the option is
    //		enabled, all levels use the alternating placement even if the
    //		alternating placement uses more space than the nonalternating
    //		placement. The option is ignored if the number of alternating
    //		circles is set to 0, because in that more, the number of alternating
    //		circles is automatically determined from the space usage.  
    //		</p>
    //		<p>
    //		The option is disabled by default.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (enable != this._allLevelsAlternating)
    {
      this._allLevelsAlternating = enable;
      this.onParameterChanged("AllLevelsAlternating");
    }
  },

  isAllLevelsAlternating : function()
  {
    // summary:
    //		Returns 'true' if at all levels, nodes are forced into
    //		alternating circular layers. 
    // description:
    //		Returns <code>true</code> if at all levels, nodes are forced into
    //		alternating circular layers. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultAllLevelsAlternating; // Boolean
    }
    return this._allLevelsAlternating; // Boolean
  },

  setNumberOfAlternatingCircles : function(numCircles)
  {
    // summary:
    //		Sets the number of alternating circles. 
    // description:
    //		Sets the number of alternating circles. 
    //		<p>
    //		This option has only an effect in layout mode <code>
    //		TreeLayout.ALTERNATING&#95;RADIAL</code>. Alternating placement means
    //		that the nodes of one level are placed alternating in several
    //		circular layers. By default, it uses 2 circular layers for the
    //		alternating placement. Nonalternating placement means  that the
    //		nodes of one level are placed in one circular layer.  
    //		</p>
    //		<p>
    //		This option allows to define how many alternating layers are
    //		maximally used. You cannot have more layers than nodes in the level,
    //		hence it is only the maximum number of layers. Furthermore, the
    //		layout analyses which levels must use the alternating placement in
    //		order to optimize space, unless the option <code>
    //		TreeLayout.isAllLevelsAlternating()</code> returns true.  
    //		</p>
    //		<p>
    //		If you set the number to a positive value and <code>
    //		TreeLayout.isAllLevelsAlternating()</code> returns true, each level
    //		has exactly that many circular layers, unless the level has not
    //		sufficient nodes. If you set the number to a positive value and
    //		<code> TreeLayout.isAllLevelsAlternating()</code> returns false,
    //		each level has either exactly that many circular layers or one
    //		circular layer, depending on which version uses the space better. If
    //		you set the number to 0, a heuristic tries to determine for each
    //		level how many circular layers are needed in order to optimize the
    //		space. In this case, different levels can have different number of
    //		circular layers, and the value of <code>
    //		TreeLayout.isAllLevelsAlternating()</code> is ignored.  
    //		</p>
    //		<p>
    //		Note that the layout may have link crossings if a huge number of
    //		alternating circles per level is used.  
    //		</p>
    //		<p>
    //		The default value is 2.   
    //		</p>
    
    if (numCircles < 0) {
      numCircles = 0;
    }
    if (numCircles != this._numberOfAlternatingCircles)
    {
      this._numberOfAlternatingCircles = numCircles;
      this.onParameterChanged("NumberOfAlternatingCircles");
    }
  },

  getNumberOfAlternatingCircles : function()
  {
    // summary:
    //		Returns the number of alternating circles. 
    // description:
    //		Returns the number of alternating circles. 
    //		This option has only an effect in layout mode <code>
    //		TreeLayout.ALTERNATING&#95;RADIAL</code>.
    
    if (false) {
      return TreeLayout._defaultNumberOfAlternatingCircles; // int
    }
    return this._numberOfAlternatingCircles; // int
  },

  setRespectNodeSizes : function(respect)
  {
    // summary:
    //		Sets whether the algorithm must try to respect the size of the
    //		nodes. 
    // description:
    //		Sets whether the algorithm must try to respect the size of the
    //		nodes. 
    //		If the argument is <code>true</code>, the spacing parameters are
    //		interpreted as the minimum distance between the node borders. If the
    //		argument is <code>false</code>, the spacing parameters are
    //		interpreted as the minimum distance between the node centers.  
    //		<p>
    //		If respecting node sizes is disabled, the level alignment option has
    //		no effect.  
    //		</p>
    //		<p>
    //		This is an expert option. The default is <code>true</code>. It makes
    //		sense to switch this option off only if all nodes already have
    //		approximately the same size and the spacing parameters are larger
    //		than the largest node. If the option is switched off, large nodes
    //		may otherwise overlap. In particular, it is recommended keeping the
    //		option enabled if orthogonal links are used.  
    //		</p>
    
    if (respect != this._respectNodeSizes)
    {
      this._respectNodeSizes = respect;
      this.onParameterChanged("RespectNodeSizes");
    }
  },

  isRespectNodeSizes : function()
  {
    // summary:
    //		Returns 'true' if the algorithm tries to respect the size
    //		of the nodes, and 'false' otherwise.
    
    if (false) {
      return TreeLayout._defaultRespectNodeSizes; // Boolean
    }
    return this._respectNodeSizes; // Boolean
  },

  setLevelAlignment : function(alignment)
  {
    // summary:
    //		Sets the alignment within the levels. 
    // description:
    //		Sets the alignment within the levels. 
    //		Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.CENTER</code></li>  
    //		<li> <code> TreeLayout.NORTH</code></li>  
    //		<li> <code> TreeLayout.SOUTH</code></li>  
    //		</ul>  
    //		<p>
    //		This feature sets the alignment of the nodes within each level. It
    //		can be used in layout modes <code> TreeLayout.LEVEL</code>, <code>
    //		TreeLayout.RADIAL</code>, and <code>
    //		TreeLayout.ALTERNATING&#95;RADIAL</code>. It has an effect only if the
    //		respect node sizes option is enabled.  
    //		</p>
    //		<p>
    //		If the nodes are center-justified, all nodes of the same level are
    //		placed with their centers along a line. Otherwise, all nodes of the
    //		same level are placed with the specified border justified to a line.
    //		For instance, if the link flow direction is to the right, the
    //		alignment <code> TreeLayout.NORTH</code> means that the nodes are
    //		justified to the left and the alignment <code>
    //		TreeLayout.SOUTH</code> means that the nodes are justified to the
    //		right.  
    //		</p>
    //		<p>
    //		In the radial layout modes, the alignment <code>
    //		TreeLayout.NORTH</code> means that the nodes are aligned along a
    //		circle with the border that is closer to the root, and the alignment
    //		<code> TreeLayout.SOUTH</code> means that the nodes are aligned
    //		along a circle with the border that is farther away from the root.  
    //		</p>
    //		<p>
    //		The default value is <code> TreeLayout.CENTER</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (!(alignment == TreeLayout.CENTER || alignment == TreeLayout.NORTH || alignment == TreeLayout.SOUTH))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6706E",  alignment);
    }

    if (alignment != this._levelAlignment)
    {
      this._levelAlignment = alignment;
      this.onParameterChanged("LevelAlignment");
    }
  },

  getLevelAlignment : function()
  {
    // summary:
    //		Returns the current alignment within the levels. 
    // description:
    //		Returns the current alignment within the levels. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultLevelAlignment; // int
    }
    return this._levelAlignment; // int
  },

  setGlobalAlignment : function(alignment)
  {
    // summary:
    //		Sets the global alignment option. 
    // description:
    //		Sets the global alignment option. 
    //		This option has no effect in balloon layout mode. Valid values are: 
    //		
    //		<ul>
    //		<li> <code> TreeLayout.CENTER</code> - a parent is centered above
    //		its child nodes, taking the center of the child nodes into
    //		account.</li>  
    //		<li> <code> TreeLayout.BORDER&#95;CENTER</code> - a parent is centered
    //		above its child nodes, taking the border of the child nodes into
    //		account. This differs from the alignment <code>
    //		TreeLayout.CENTER</code> if the Respect node sizes option is
    //		enabled.</li>  
    //		<li> <code> TreeLayout.EAST</code> - a parent is aligned to the
    //		border of its easternmost child node.</li>  
    //		<li> <code> TreeLayout.WEST</code> - a parent is aligned to the
    //		border of its westernmost child node.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;EAST</code> - a parent is outside
    //		the border of the child nodes, and the child nodes are placed
    //		sequentially instead of in parallel. The child nodes are on the east
    //		side of the parent. This alignment option is not possible in the
    //		radial or level layout modes.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;WEST</code> - a parent is outside
    //		the border of the child nodes, and the child nodes are placed
    //		sequentially instead of in parallel. The child nodes are on the west
    //		side of the parent. This alignment option is not possible in the
    //		radial or level layout modes.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;EAST&#95;WEST</code> - similar to <code>
    //		TreeLayout.TIP&#95;OVER&#95;EAST</code> or <code>
    //		TreeLayout.TIP&#95;OVER&#95;WEST</code>, but the child nodes are arranged on
    //		both sides of the link segment that starts at the parent node. The
    //		first child node is placed on the east side. Further child nodes are
    //		placed on the east or west side, so that the space used on both
    //		sides is roughly the same, that is, the subtrees on the east side
    //		have roughly the same sum of heights as the subtrees on the west
    //		side. This alignment option is not possible in the radial or level
    //		layout modes.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;BOTH&#95;SIDES</code> - similar to
    //		<code> TreeLayout.TIP&#95;OVER&#95;EAST</code> or <code>
    //		TreeLayout.TIP&#95;OVER&#95;WEST</code>, but the child nodes are arranged in
    //		pairs on both sides of the link segment that starts at the parent
    //		node. Unlike <code> TreeLayout.TIP&#95;OVER&#95;EAST&#95;WEST</code>, this style
    //		does not try to optimize the space used on both sides but always
    //		puts the odd-numbered child nodes to the west and the even-numbered
    //		child nodes to the east. The pairs of child nodes, one on the east
    //		side and one on the west side, are aligned with one another. This
    //		alignment option is not possible in the radial or level layout
    //		modes.</li>  
    //		<li> <code> TreeLayout.MIXED</code> - each node can have a different
    //		alignment option. The alignment of each individual node can be set
    //		by the method <code> TreeLayout.setAlignment()</code> so that
    //		different alignments can occur in the same graph.</li>  
    //		</ul>  
    //		<p>
    //		The default value is <code> TreeLayout.CENTER</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (!(alignment == TreeLayout.CENTER || alignment == TreeLayout.BORDER_CENTER || alignment == TreeLayout.EAST || alignment == TreeLayout.WEST || alignment == TreeLayout.TIP_OVER ||
        alignment == TreeLayout.TIP_OVER_EAST || alignment == TreeLayout.TIP_OVER_WEST || alignment == TreeLayout.TIP_OVER_EAST_WEST || alignment == TreeLayout.TIP_OVER_BOTH_SIDES ||
        alignment == TreeLayout.MIXED))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6707E",  alignment);
    }

    if (alignment != this._globalAlignment)
    {
      this._globalAlignment = alignment;
      this.onParameterChanged("GlobalAlignment");
    }
  },

  getGlobalAlignment : function()
  {
    // summary:
    //		Returns the global alignment option. 
    // description:
    //		Returns the global alignment option. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultAlignment; // int
    }
    return this._globalAlignment; // int
  },

  setSiblingOffset : function(offset)
  {
    // summary:
    //		Sets the minimum offset between siblings, that is, between nodes
    //		that have the same parent in the tree structure. 
    // description:
    //		Sets the minimum offset between siblings, that is, between nodes
    //		that have the same parent in the tree structure. 
    //		<p>
    //		If the flow direction is towards the top or bottom and the layout
    //		mode is not radial, this is a horizontal offset. If the flow
    //		direction is towards the left or right and the layout mode is not
    //		radial, this is a vertical offset. If the layout mode is radial,
    //		this is an offset tangential to the circles or ellipses.  
    //		</p>
    //		<p>
    //		This offset has no effect in balloon layout mode. Use the branch
    //		offset to control the spacing between all nodes in balloon layout
    //		mode.  
    //		</p>
    //		<p>
    //		If the respect node sizes option is enabled, this is the minimum
    //		offset between node borders; otherwise, it is the minimum offset
    //		between node centers.  
    //		</p>
    //		<p>
    //		The default value is <code>40</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (offset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6708E",  offset);
    }

    if (offset != this._siblingOffset)
    {
      this._siblingOffset = offset;
      this.onParameterChanged("SiblingOffset");
    }
  },

  getSiblingOffset : function()
  {
    // summary:
    //		Returns the minimum offset between siblings. 
    // description:
    //		Returns the minimum offset between siblings. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultSiblingOffset; // Number
    }
    return this._siblingOffset; // Number
  },

  setParentChildOffset : function(offset)
  {
    // summary:
    //		Sets the minimum offset between a parent and its children. 
    // description:
    //		Sets the minimum offset between a parent and its children. 
    //		<p>
    //		If the flow direction is towards the top or bottom and the layout
    //		mode is not radial or balloon, this is a vertical offset. If the
    //		flow direction is towards the left or right and the layout mode is
    //		not radial or balloon, this is a horizontal offset. If the layout
    //		mode is radial or balloon, this is an offset radial to the circle or
    //		ellipses. In balloon mode, it corresponds to the minimal radius of
    //		the balloon.  
    //		</p>
    //		<p>
    //		If the respect node sizes option is enabled, this is the minimum
    //		offset between node borders; otherwise, it is the minimum offset
    //		between node centers.  
    //		</p>
    //		<p>
    //		The default value is <code>40</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (offset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6709E",  offset);
    }

    if (offset != this._parentChildOffset)
    {
      this._parentChildOffset = offset;
      this.onParameterChanged("ParentChildOffset");
    }
  },

  getParentChildOffset : function()
  {
    // summary:
    //		Returns the minimum offset between a parent and its children. 
    // description:
    //		Returns the minimum offset between a parent and its children. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultParentChildOffset; // Number
    }
    return this._parentChildOffset; // Number
  },

  setBranchOffset : function(offset)
  {
    // summary:
    //		Sets the minimum offset between neighbor nodes that belong to
    //		different branches in the tree. 
    // description:
    //		Sets the minimum offset between neighbor nodes that belong to
    //		different branches in the tree. 
    //		This is the offset between neighbor nodes that are not siblings. The
    //		offset is in the opposite direction of the link flow.  
    //		<p>
    //		If the flow direction is towards the top or bottom and the layout
    //		mode is not radial or balloon, this is a horizontal offset. If the
    //		flow direction is towards the left or right and the layout mode is
    //		not radial or balloon, this is a vertical offset. If the layout mode
    //		is radial, this is an offset tangential to the circles or ellipses. 
    //		
    //		</p>
    //		<p>
    //		If the layout mode is balloon, this is the minimal offset between
    //		nodes in all directions. The balloon mode does not distinguish
    //		between siblings and branches, hence the branch offset is used for
    //		all nodes.  
    //		</p>
    //		<p>
    //		If the respect node sizes option is enabled, this is the minimum
    //		offset between node borders; otherwise, it is the minimum offset
    //		between node centers.  
    //		</p>
    //		<p>
    //		The default value is <code>40</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (offset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6710E",  offset);
    }

    if (offset != this._branchOffset)
    {
      this._branchOffset = offset;
      this.onParameterChanged("BranchOffset");
    }
  },

  getBranchOffset : function()
  {
    // summary:
    //		Returns the minimum offset between neighbor nodes that belong to
    //		different branches in the tree. 
    // description:
    //		Returns the minimum offset between neighbor nodes that belong to
    //		different branches in the tree. 
    //		This is the offset between neighbor nodes that are not siblings. The
    //		offset is in the opposite direction of the link flow.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultBranchOffset; // Number
    }
    return this._branchOffset; // Number
  },

  setTipOverBranchOffset : function(offset)
  {
    // summary:
    //		Sets the minimum offset in the flow direction between neighbor nodes
    //		that belong to different branches in the tree. 
    // description:
    //		Sets the minimum offset in the flow direction between neighbor nodes
    //		that belong to different branches in the tree. 
    //		This offset is used to separate the branches if their common parent
    //		node has tip-over alignment. It has no effect if the layout mode is
    //		radial or if no tip-over alignment occurs.  
    //		<p>
    //		If the flow direction is towards the top or bottom, this is a
    //		vertical offset. If the flow direction is towards the left or right,
    //		this is a horizontal offset.  
    //		</p>
    //		<p>
    //		If the respect node sizes option is enabled, this is the minimum
    //		offset between node borders; otherwise, it is the minimum offset
    //		between node centers.  
    //		</p>
    //		<p>
    //		The default value is <code>40</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (offset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6711E",  offset);
    }

    if (offset != this._tipOverBranchOffset)
    {
      this._tipOverBranchOffset = offset;
      this.onParameterChanged("TipOverBranchOffset");
    }
  },

  getTipOverBranchOffset : function()
  {
    // summary:
    //		Returns the minimum offset in the flow direction between neighbor
    //		nodes that belong to different branches in the tree. 
    // description:
    //		Returns the minimum offset in the flow direction between neighbor
    //		nodes that belong to different branches in the tree. 
    //		This offset is used to separate the branches if their common parent
    //		node has tip-over alignment.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultTipOverBranchOffset; // Number
    }
    return this._tipOverBranchOffset; // Number
  },

  setAssociateOffset : function(offset)
  {
    // summary:
    //		Sets the minimum offset between associates and the orthogonal link
    //		segment starting at the parent. 
    // description:
    //		Sets the minimum offset between associates and the orthogonal link
    //		segment starting at the parent. 
    //		This offset is used if a parent has associate nodes. This method has
    //		no effect if the layout mode is radial or if no associates occur.  
    //		<p>
    //		If the flow direction is toward the top or bottom, the orthogonal
    //		link segment starting at the parent is vertical, and the associates
    //		are placed left or right of this link segment. Hence it is a
    //		horizontal offset. If the flow direction is toward the left or
    //		right, the orthogonal link segment starting at the parent is
    //		horizontal, and the associates are placed above or below this link
    //		segment. Hence it is a vertical offset.  
    //		</p>
    //		<p>
    //		If the Respect node sizes option is enabled, this is the minimum
    //		offset between node borders; otherwise, it is the minimum offset
    //		between node centers.  
    //		</p>
    //		<p>
    //		The default value is <code>30</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (offset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6712E",  offset);
    }

    if (offset != this._assocOffset)
    {
      this._assocOffset = offset;
      this.onParameterChanged("AssociateOffset");
    }
  },

  getAssociateOffset : function()
  {
    // summary:
    //		Returns the minimum offset between associates and the orthogonal
    //		link segment starting at the parent. 
    // description:
    //		Returns the minimum offset between associates and the orthogonal
    //		link segment starting at the parent. 
    //		This offset is used if a parent has associate nodes.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultAssocOffset; // Number
    }
    return this._assocOffset; // Number
  },

  setParentChildAssociateOffset : function(offset)
  {
    // summary:
    //		Sets the minimum offset between the parent and the first associate. 
    // description:
    //		Sets the minimum offset between the parent and the first associate. 
    //		This offset is used if a parent has associate nodes. This method has
    //		no effect if the layout mode is radial or if no associates occur.  
    //		<p>
    //		If the flow direction is toward the top or bottom, the orthogonal
    //		link segment starting at the parent is vertical, and the associates
    //		are placed left or right of this link segment. In this case, it is a
    //		vertical offset between parent and child. If the flow direction is
    //		toward the left or right, the orthogonal link segment starting at
    //		the parent is horizontal, and the associates are placed above or
    //		below this link segment. In this case, it is a horizontal offset
    //		between parent and child.  
    //		</p>
    //		<p>
    //		If the Respect node sizes option is enabled, this is the minimum
    //		offset between node borders; otherwise, it is the minimum offset
    //		between node centers.  
    //		</p>
    //		<p>
    //		The default value is <code>30</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (offset < 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6713E",  offset);
    }

    if (offset != this._parentChildAssocOffset)
    {
      this._parentChildAssocOffset = offset;
      this.onParameterChanged("ParentChildAssociateOffset");
    }
  },

  getParentChildAssociateOffset : function()
  {
    // summary:
    //		Returns the minimum offset between the parent and the first
    //		associate. 
    // description:
    //		Returns the minimum offset between the parent and the first
    //		associate. 
    //		This offset is used if a parent has associate nodes.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultParentChildAssocOffset; // Number
    }
    return this._parentChildAssocOffset; // Number
  },

  setAspectRatio : function(aspectRatio)
  {
    // summary:
    //		Sets the aspect ratio for the layout. 
    // description:
    //		Sets the aspect ratio for the layout. 
    //		This is the ratio <code>width / height</code> of the desired layout
    //		region.  
    //		<p>
    //		The effect of the aspect ratio depends on the layout mode:  
    //		<ul>
    //		<li> In layout modes <code> TreeLayout.RADIAL</code> and <code>
    //		TreeLayout.ALTERNATING&#95;RADIAL</code>, the nodes are placed in
    //		circular layers if the aspect ratio is <code>1</code>. Otherwise,
    //		they are placed in elliptical layers.</li>  
    //		<li> In layout modes <code> TreeLayout.BALLOON</code>, the nodes are
    //		placed in circular balloons if the aspect ratio is <code>1</code>.
    //		Otherwise, they are placed in elliptical ballons.</li>  
    //		<li> In layout modes <code> TreeLayout.TIP&#95;OVER</code>, <code>
    //		TreeLayout.TIP&#95;ROOTS&#95;OVER</code>, <code>
    //		TreeLayout.TIP&#95;LEAVES&#95;OVER</code>, and <code>
    //		TreeLayout.TIP&#95;ROOTS&#95;AND&#95;LEAVES&#95;OVER</code>, the nodes are placed as
    //		in in free mode, but local tip-over alignments are automatically
    //		added to better fit the layout to the aspect ratio.</li>  
    //		<li> The aspect ratio has no effect in other layout modes.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (aspectRatio <= 0.0)
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6714E",  aspectRatio);
    }

    if (aspectRatio != this._aspectRatio)
    {
      this._aspectRatio = aspectRatio;
      this.onParameterChanged("AspectRatio");
    }
  },

  getAspectRatio : function()
  {
    // summary:
    //		Returns the aspect ratio specified for the layout. 
    // description:
    //		Returns the aspect ratio specified for the layout. 
    //		This is the ratio <code>width / height</code> of the desired layout
    //		region.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultAspectRatio; // Number
    }
    return this._aspectRatio; // Number
  },

  setOverlapPercentage : function(percentage)
  {
    // summary:
    //		Sets the percentage for which a node is considered smaller in the
    //		flow direction. 
    // description:
    //		Sets the percentage for which a node is considered smaller in the
    //		flow direction. 
    //		This allows nodes to be placed closer together. However, in layout
    //		modes <code> TreeLayout.FREE</code>, <code>
    //		TreeLayout.TIP&#95;OVER</code>, <code> TreeLayout.TIP&#95;ROOTS&#95;OVER</code>,
    //		<code> TreeLayout.TIP&#95;LEAVES&#95;OVER</code>, and <code>
    //		TreeLayout.TIP&#95;ROOTS&#95;AND&#95;LEAVES&#95;OVER</code>, outgoing links of a
    //		node may, in rare cases, cross the neighbor leaf nodes if the
    //		overlap percentage is high. To avoid this, set the parameter to
    //		<code>0</code>%.  
    //		<p>
    //		The default value is <code>30</code>%, which is a good compromise
    //		for many graphs between wasting space and the risk of having links
    //		cross nodes.  
    //		</p>
    //		<p>
    //		This is an expert option. In all normal cases, it is not necessary
    //		to change the option. It is recommended choosing an orthogonal fork
    //		percentage that is always larger than the overlap percentage to
    //		obtain a good layout.  
    //		</p>
    //		<p>
    //		This option has no effect if the respect node sizes option is
    //		disabled.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (percentage < 0.0) {
      percentage = 0.0;
    }
    if (percentage > 100.0) {
      percentage = 100.0;
    }
    if (percentage != this._overlapPercentage)
    {
      this._overlapPercentage = percentage;
      this.onParameterChanged("OverlapPercentage");
    }
  },

  getOverlapPercentage : function()
  {
    // summary:
    //		Returns the percentage for which a node is considered smaller in the
    //		flow direction. 
    // description:
    //		Returns the percentage for which a node is considered smaller in the
    //		flow direction. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultOverlapPercentage; // Number
    }
    return this._overlapPercentage; // Number
  },

  setMaxChildrenAngle : function(angle)
  {
    // summary:
    //		Sets the maximum angle for children that are placed in radial mode. 
    // description:
    //		Sets the maximum angle for children that are placed in radial mode. 
    //		This is the maximum angle between the two rays from the parent to
    //		its two outermost children. The angle must be between 0 and 360
    //		degrees. The angle 0 means that the angle is unspecified,  
    //		<p>
    //		The default value is 0.  
    //		</p>
    //		<p>
    //		In radial mode, children of a single node may cover nearly
    //		completely one circular layer. This sometime appears to be
    //		unbalanced, in particular because the links from the node to its
    //		outermost children are likely to cross other nodes. To avoid this
    //		effect, a maximum angle can be specified so that the children are
    //		placed more in the same direction and are not spread too widely
    //		around the parent. This angular restriction is applied to all
    //		children except the children of the root node. As a side effect,
    //		specifying a small maximum children angle enlarges the radius of the
    //		circles.  
    //		</p>
    //		<p>
    //		It is recommended choosing a value between <code>30</code> and
    //		<code>180</code> degrees for good results. The algorithm uses a
    //		heuristic that has precision deviations below <code>30</code>
    //		degrees and above <code>180</code> degrees. If an aspect ratio is
    //		specified, the layout algorithm uses ellipses instead of circles. In
    //		this case, precision deviations may occur as well.  
    //		</p>
    //		<p>
    //		This is an expert option. It has no effect in layout modes other
    //		than radial.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (angle < 0) {
      angle = 0;
    }
    if (angle > 360) {
      angle = 360;
    }
    if (angle != this._maxChildrenAngle)
    {
      this._maxChildrenAngle = angle;
      this.onParameterChanged("MaxChildrenAngle");
    }
  },

  getMaxChildrenAngle : function()
  {
    // summary:
    //		Returns the maximum angle for children that are placed in radial
    //		mode. 
    // description:
    //		Returns the maximum angle for children that are placed in radial
    //		mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultMaxChildrenAngle; // int
    }
    return this._maxChildrenAngle; // int
  },

  setFirstCircleEvenlySpacing : function(enable)
  {
    // summary:
    //		Sets whether the nodes on the first circle must be evenly spaced in
    //		radial mode. 
    // description:
    //		Sets whether the nodes on the first circle must be evenly spaced in
    //		radial mode. 
    //		<p>
    //		By default, all nodes are placed as close as possible according to
    //		the offset parameters. In radial mode, this may cause the first
    //		circle to be only partially filled (that is, all nodes are placed
    //		close together at one part of the circle and there is a larger gap
    //		in another part of the circle).  
    //		</p>
    //		<p>
    //		To avoid this effect, you can enable the evenly spacing of the first
    //		circle. In this case, the children of the root node are placed such
    //		that their centers are approximately evenly spaced on the first
    //		circle, filling the entire circle. This may, however, cause the
    //		final result to waste much more space on the other circles.  
    //		</p>
    //		<p>
    //		It is disabled by default.  
    //		</p>
    //		<p>
    //		This is an expert option. It has no effect in layout modes other
    //		than radial. If there is east-west neighboring of the nodes in the
    //		first circle, these neighbors will not be equally spaced.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (enable != this._firstCircleEvenlySpacing)
    {
      this._firstCircleEvenlySpacing = enable;
      this.onParameterChanged("FirstCircleEvenlySpacing");
    }
  },

  isFirstCircleEvenlySpacing : function()
  {
    // summary:
    //		Returns 'true' if the nodes on the first circle must be
    //		evenly spaced in radial mode. 
    // description:
    //		Returns <code>true</code> if the nodes on the first circle must be
    //		evenly spaced in radial mode. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultFirstCircleEvenlySpacing; // Boolean
    }
    return this._firstCircleEvenlySpacing; // Boolean
  },

  setOrthForkPercentage : function(percentage)
  {
    // summary:
    //		Sets the percentage of the parent-child offset that determines the
    //		location where the orthogonal link routing places the bends in order
    //		to fork the routing towards the children. 
    // description:
    //		Sets the percentage of the parent-child offset that determines the
    //		location where the orthogonal link routing places the bends in order
    //		to fork the routing towards the children. 
    //		This has an effect only if the orthogonal link style is chosen.  
    //		<p>
    //		The default value is <code>45</code>%.  
    //		</p>
    //		<p>
    //		This is an expert option. In all normal cases, it is not necessary
    //		to change the option. It is recommended choosing an orthogonal fork
    //		percentage that is always larger than the overlap percentage to
    //		obtain a good layout.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (percentage < 0.0) {
      percentage = 0.0;
    }
    if (percentage > 100.0) {
      percentage = 100.0;
    }
    if (percentage != this._orthForkPercentage)
    {
      this._orthForkPercentage = percentage;
      this.onParameterChanged("OrthForkPercentage");
    }
  },

  getOrthForkPercentage : function()
  {
    // summary:
    //		Returns the percentage of the parent-child offset that determines
    //		the location where the orthogonal link routing places the bends in
    //		order to fork the routing towards the children. 
    // description:
    //		Returns the percentage of the parent-child offset that determines
    //		the location where the orthogonal link routing places the bends in
    //		order to fork the routing towards the children. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultOrthForkPercentage; // Number
    }
    return this._orthForkPercentage; // Number
  },

  setLinkStyle : function(link, style)
  {
    // summary:
    //		Sets the style of the shape of an individual link. 
    // description:
    //		Sets the style of the shape of an individual link. 
    //		This link style is used only if the global link style is set to
    //		<code> TreeLayout.MIXED&#95;STYLE</code> and the link is part of the
    //		spanning tree. If the global link style is not <code>
    //		TreeLayout.MIXED&#95;STYLE</code>, all links of the spanning tree have
    //		the style that is specified as the global link style. Links that are
    //		not part of the spanning tree have the link style <code>
    //		TreeLayout.NO&#95;RESHAPE&#95;STYLE</code>.  
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.ORTHOGONAL&#95;STYLE</code> - the link has an
    //		orthogonal shape, that is, a shape consisting of a sequence of
    //		orthogonal line segments. This option is not available for the
    //		radial layout modes.</li>  
    //		<li> <code> TreeLayout.STRAIGHT&#95;LINE&#95;STYLE</code> - the link has a
    //		straight-line shape.</li>  
    //		<li> <code> TreeLayout.NO&#95;RESHAPE&#95;STYLE</code> - no reshape is
    //		performed on the link.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code> TreeLayout.STRAIGHT&#95;LINE&#95;STYLE</code>.  
    //		
    //		</p>
    //		<p>
    //		 
    //		</p>
    // link: Object
    //		The link object.
    // style: int
    //		The style of the link shape.
    
    LayoutParametersUtil.checkInputLink(this, link, "input link");

    if (!(style == TreeLayout.ORTHOGONAL_STYLE || style == TreeLayout.STRAIGHT_LINE_STYLE || style == TreeLayout.NO_RESHAPE_STYLE))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6715E",  style);
    }

    TLinkParameters.setLinkStyle(this, link, style);
  },

  getLinkStyle : function(link)
  {
    // summary:
    //		Returns the style of the shape of an individual link. 
    // description:
    //		Returns the style of the shape of an individual link. 
    //		<p>
    //		 
    //		</p>
    // link: Object
    //		The individual link.
    // returns:
    //		The style of the link shape.
    
    return TLinkParameters.getLinkStyle(this, link); // int
  },

  setEastWestNeighboring : function(eastNode, westNode)
  {
    // summary:
    //		Creates an east-west neighboring of two nodes. 
    // description:
    //		Creates an east-west neighboring of two nodes. 
    //		The <code>westNode</code> is placed as the first neighbor on the
    //		same level to the west of the <code>eastNode</code>. This implies,
    //		conversely, that the <code>eastNode</code> is the first neighbor on
    //		the same level to the east of the <code>westNode</code>.  
    //		<p>
    //		For instance, in free layout mode and with the flow direction
    //		towards the bottom, the east node is placed as the immediate
    //		neighbor to the right of the west node at approximately the same
    //		<code>y</code> coordinate. A link between both nodes is routed as a
    //		straight horizontal line between both nodes. In radial layout mode,
    //		the east node is placed as the immediate neighbor in the same circle
    //		as the west node. A link between both nodes is routed tangentially
    //		to the node circle as a straight line.  
    //		</p>
    //		<p>
    //		Note that each node can have at most one east neighbor and one west
    //		neighbor.  
    //		</p>
    //		<p>
    //		East-west neighboring is possible even though there is no link
    //		between both nodes. The layout algorithm considers both nodes as if
    //		they were conceptually connected by a link.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // eastNode: Object
    //		The east neighbor of the 'westNode'.
    // westNode: Object
    //		The west neighbor of the 'eastNode'.
    
    if (eastNode != null) {
      LayoutParametersUtil.checkInputNode(this, eastNode, "input east node");
    }
    if (westNode != null) {
      LayoutParametersUtil.checkInputNode(this, westNode, "input west node");
    }
    if (eastNode == westNode) {
      return;
    }
    var somethingChanged = false;

    if (westNode != null)
    {
      var oldEastNeighbor = this.getEastNeighbor(westNode);
      this.setEastNeighbor(westNode, eastNode);
      if (eastNode != oldEastNeighbor && oldEastNeighbor != null) {
        this.setWestNeighbor(oldEastNeighbor, null);
      }
      if (oldEastNeighbor != this.getEastNeighbor(westNode)) {
        somethingChanged = true;
      }
    }

    if (eastNode != null)
    {
      var oldWestNeighbor = this.getWestNeighbor(eastNode);
      this.setWestNeighbor(eastNode, westNode);
      if (westNode != oldWestNeighbor && oldWestNeighbor != null) {
        this.setEastNeighbor(oldWestNeighbor, null);
      }
      if (oldWestNeighbor != this.getWestNeighbor(eastNode)) {
        somethingChanged = true;
      }
    }

    if (somethingChanged)
    {
      this.onParameterChanged("EastWestNeighoring");
    }
  },

  setEastNeighbor : function(node, eastNeighbor)
  {
    // summary:
    //		Auxiliary to set the east neighbor.
    // tags:
    //		private
    
    var prop = TNodeParameters.getOrAllocate(this, node);
    prop.setEastNeighbor(eastNeighbor);
    if (!prop.isNeeded()) {
      prop.dispose(this, node);
    }
  },

  setWestNeighbor : function(node, westNeighbor)
  {
    // summary:
    //		Auxiliary to set the west neighbor.
    // tags:
    //		private
    
    var prop = TNodeParameters.getOrAllocate(this, node);
    prop.setWestNeighbor(westNeighbor);
    if (!prop.isNeeded()) {
      prop.dispose(this, node);
    }
  },

  setWestEastNeighboring : function(westNode, eastNode)
  {
    // summary:
    //		Creates an east-west neighboring of two nodes. 
    // description:
    //		Creates an east-west neighboring of two nodes. 
    //		The <code>westNode</code> is placed as the first neighbor on the
    //		same level to the west of the <code>eastNode</code>. This implies,
    //		conversely, that the <code>eastNode</code> is the first neighbor on
    //		the same level to the east of the <code>westNode</code>.  
    //		<p>
    //		This method is identical to <code>
    //		TreeLayout.setEastWestNeighboring()</code> with a reversed argument
    //		order, and is provided for convenience. Note that, for instance,
    //		when the flow direction is towards the bottom, east is to the right
    //		of west, and therefore resembles more the argument order of
    //		<code>setWestEastNeighboring</code>. When the flow direction is
    //		towards the top, east is to the left of west, and therefore
    //		resembles more the argument order of
    //		<code>setEastWestNeighboring</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // westNode: Object
    //		The west neighbor of the 'eastNode'.
    // eastNode: Object
    //		The east neighbor of the 'westNode'.
    
    this.setEastWestNeighboring(eastNode, westNode);
  },

  getEastNeighbor : function(node)
  {
    // summary:
    //		Returns the specified east neighbor node of the input node, or
    //		'null' if none was specified. 
    // description:
    //		Returns the specified east neighbor node of the input node, or
    //		<code>null</code> if none was specified. 
    //		It returns only the neighbor node if it was specified by <code>
    //		TreeLayout.setEastWestNeighboring()</code> or <code>
    //		TreeLayout.setWestEastNeighboring()</code>, but not if the neighbor
    //		node was only specified by <code>
    //		TreeLayout.setChildAlignment()</code>.  
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node object.
    // returns:
    //		The specified east neighbor node of the given node.
    
    var prop = TNodeParameters.get(this, node);
    if (prop != null) {
      return prop.getEastNeighbor(); // Object
    }
    return null; // Object
  },

  getWestNeighbor : function(node)
  {
    // summary:
    //		Returns the specified west neighbor node of the input node, or
    //		'null' if none was specified. 
    // description:
    //		Returns the specified west neighbor node of the input node, or
    //		<code>null</code> if none was specified. 
    //		It returns only the neighbor node if it was specified by <code>
    //		TreeLayout.setEastWestNeighboring()</code> or <code>
    //		TreeLayout.setWestEastNeighboring()</code>, but not if the neighbor
    //		node was only specified by <code>
    //		TreeLayout.setChildAlignment()</code>.  
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The specified west neighbor node of the given node.
    
    var prop = TNodeParameters.get(this, node);
    if (prop != null) {
      return prop.getWestNeighbor(); // Object
    }
    return null; // Object
  },

  addAssociate : function(parentNode, associateNode)
  {
    // summary:
    //		Sets one node as associate of another node. 
    // description:
    //		Sets one node as associate of another node. 
    //		Associate nodes are placed as children of the parent node below the
    //		parent node. They are arranged in a tip over style. Different to all
    //		other children, the alignment of the associates does not obey the
    //		alignment option of the parent node. By default, associates are
    //		aligned in style <code>
    //		TreeLayout.TIP&#95;OVER&#95;ASSOCIATE&#95;BOTH&#95;SIDES</code>. It is further
    //		possible to specify the child alignment <code>
    //		TreeLayout.TIP&#95;OVER&#95;ASSOCIATE&#95;EAST</code> and <code>
    //		TreeLayout.TIP&#95;OVER&#95;ASSOCIATE&#95;WEST</code> for associate nodes.   
    //		<p>
    //		Associates can be laid out in free and level mode, but not in radial
    //		mode.  
    //		</p>
    //		<p>
    //		Associate nodes are possible even though there is no link between
    //		the associate node and its parent node. The layout algorithm
    //		considers both nodes as if they were conceptually connected by a
    //		link.  
    //		</p>
    //		<p>
    //		For instance, in free layout mode and with the flow direction
    //		towards the bottom, the first associate is placed to the left below
    //		the parent, the second associate at approximately the same
    //		<code>y</code> coordinate to the right below the parent, the third
    //		associate to the left below the first one, the forth to the right
    //		below the second one and so on. A link between parent and its
    //		associate node is always routed orthogonally.  
    //		</p>
    //		<p>
    //		A parent can have many associate nodes, but each associate node can
    //		only have one parent.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // parentNode: Object
    //		The parent node of the 'associateNode'.
    // associateNode: Object
    //		The associate node of the 'parentNode'.
    
    if (parentNode != null) {
      LayoutParametersUtil.checkInputNode(this, parentNode, "input parent node");
    }
    if (associateNode != null) {
      LayoutParametersUtil.checkInputNode(this, associateNode, "input associate node");
    }
    if (associateNode == null) {
      return;
    }
    if (parentNode == associateNode) {
      return;
    }
    var oldAssociateParent = this.getAssociateParent(associateNode);

    if (oldAssociateParent != parentNode)
    {
      this.setAssociateParent(associateNode, parentNode);
      if (oldAssociateParent != null) {
        this.removeAssociateChild(oldAssociateParent, associateNode);
      }
      if (parentNode != null) {
        this.addAssociateChild(parentNode, associateNode);
      }
      this.onParameterChanged("Associates");
    }
  },

  removeAssociate : function(parentNode, associateNode)
  {
    // summary:
    //		Removes the associate node from its parent node. 
    // description:
    //		Removes the associate node from its parent node. 
    //		<p>
    //		 
    //		</p>
    // parentNode: Object
    //		The parent node of the 'associateNode'.
    // associateNode: Object
    //		The associate node of the 'parentNode'.
    
    if (parentNode != this.getAssociateParent(associateNode)) {
      return;
    }
    this.addAssociate(null, associateNode);
  },

  removeAllAssociates : function(parentNode)
  {
    // summary:
    //		Removes all the associate nodes from the input parent node. 
    // description:
    //		Removes all the associate nodes from the input parent node. 
    //		<p>
    //		 
    //		</p>
    // parentNode: Object
    //		The parent node of the 'associateNode'.
    
    var a = new Array();
    var e = this.getAssociates(parentNode);

    while (e.hasNext())
      a.push(e.next());

    for (var i = 0; i < a.length; i++)
      this.addAssociate(null, a[i]);
  },

  setAssociateParent : function(node, parent)
  {
    // summary:
    //		Auxiliary to set the associate parent.
    // tags:
    //		private
    
    var prop = TNodeParameters.getOrAllocate(this, node);
    prop.setAssociateParent(parent);
    if (!prop.isNeeded()) {
      prop.dispose(this, node);
    }
  },

  addAssociateChild : function(node, child)
  {
    // summary:
    //		Auxiliary to add the associate child.
    // tags:
    //		private
    
    var prop = TNodeParameters.getOrAllocate(this, node);
    prop.addAssociateChild(child);
    if (!prop.isNeeded()) {
      prop.dispose(this, node);
    }
  },

  removeAssociateChild : function(node, child)
  {
    // summary:
    //		Auxiliary to remove the associate child.
    // tags:
    //		private
    
    var prop = TNodeParameters.getOrAllocate(this, node);
    prop.removeAssociateChild(child);
    if (!prop.isNeeded()) {
      prop.dispose(this, node);
    }
  },

  getAssociateParent : function(node)
  {
    // summary:
    //		Returns the parent node for which the input node is an associate. 
    // description:
    //		Returns the parent node for which the input node is an associate. 
    //		Returns <code>null</code> if the node was never added as an
    //		associate of a parent.  
    // node: Object
    //		The node instance.
    // returns:
    //		The parent node for which the given node is an associate or
    //		'null' if the node was never added as an associate of a
    //		parent.
    
    var prop = TNodeParameters.get(this, node);
    if (prop != null) {
      return prop.getAssociateParent(); // Object
    }
    return null; // Object
  },

  getAssociates : function(parentNode)
  {
    // summary:
    //		Returns all associate nodes of the input parent node. 
    // description:
    //		Returns all associate nodes of the input parent node. 
    //		The returned nodes have the input node as associate parent. It
    //		returns only the associate nodes specified by <code>
    //		TreeLayout.addAssociate()</code>, but not the associate nodes that
    //		were only specified by <code> TreeLayout.setChildAlignment()</code>.
    //		 
    // parentNode: Object
    //		The node instance.
    // returns:
    //		All associate nodes of the given parent node.
    
    var prop = TNodeParameters.get(this, parentNode);
    if (prop != null) {
      return prop.getAssociates(); // ibm_ilog.graphlayout.IIterator
    }
    return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
  },

  setRoot : function(node)
  {
    // summary:
    //		Allows you to specify the root(s) of the tree. 
    // description:
    //		Allows you to specify the root(s) of the tree. 
    //		If more than one connected component exists, it is possible to
    //		select a root for each component. If no root is specified, an
    //		appropriate node is automatically selected as root. For a directed
    //		tree, the canonical root node with degree 0 is selected unless
    //		otherwise specified.  
    //		<p>
    //		Setting the root node by this method is identical to setting the
    //		root preference of this node to 10000.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The root of the tree.
    
    if (node == null) {
      return;
    }
    this.setRootPreference(node, 10000);
    var prop = TNodeParameters.get(this, node);

    if (!prop.isSpecRoot())
    {
      prop.setSpecRoot(true);
      this._specRoots.push(node);

      this.onParameterChanged("Root");
    }
  },

  getSpecRoots : function()
  {
    // summary:
    //		Returns the nodes that were specified as root. 
    // description:
    //		Returns the nodes that were specified as root. 
    //		Note that this does not necessarily mean that these nodes are
    //		selected as real roots, because the specification may have conflicts
    //		(for example, each connected component can only have one root, thus
    //		specifying two roots for the same connected component causes a
    //		conflict). To retrieve the nodes that were really used as roots
    //		during the layout, call the method <code>
    //		TreeLayout.getCalcRoots()</code>.  
    //		<p>
    //		 
    //		</p>
    // returns:
    //		The nodes that were specified as root.
    
    return  new TranslateUtil.ArrayIterator(this._specRoots); // ibm_ilog.graphlayout.IIterator
  },

  isSpecRoot : function(node)
  {
    // summary:
    //		Returns 'true' if the input node was specified as root. 
    // description:
    //		Returns <code>true</code> if the input node was specified as root. 
    //		This does not necessarily mean that this node is selected as real
    //		root, because the specification may have conflicts (for example,
    //		each connected component can only have one root, so specifying two
    //		roots for the same connected component causes a conflict). To check
    //		whether the node was really used as root during the layout, call the
    //		method <code> TreeLayout.isCalcRoot()</code>.  
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		'true' if the input node was specified as root.
    
    if (node == null) {
      return false; // Boolean
    }
    var prop = TNodeParameters.get(this, node);
    if (prop == null) {
      return false; // Boolean
    }
    return prop.isSpecRoot(); // Boolean
  },

  removeSpecRoot : function(node)
  {
    // summary:
    //		Remove a specified root. 
    // description:
    //		Remove a specified root. 
    //		This does not try to dispose the node property holding the
    //		parameters, because it is used only in situation where it is not
    //		necessary.
    // tags:
    //		private
    
    var prop = TNodeParameters.get(this, node);

    if (prop != null && prop.isSpecRoot())
    {
      prop.setSpecRoot(false);
      TranslateUtil.removeArrayObject(this._specRoots, node);
    }
  },

  cleanSpecRoots : function()
  {
    // summary:
    //		Clean all specified roots. 
    // description:
    //		Clean all specified roots. 
    //		This does not try to dispose the node properties holding the
    //		parameters, because it is used only in situation where it is not
    //		necessary.
    // tags:
    //		private
    
    var e = this.getSpecRoots();
    var node;
    var prop;

    while (e.hasNext())
    {
      node = e.next();

      prop = TNodeParameters.get(this, node);
      if (prop != null && prop.isSpecRoot()) {
        prop.setSpecRoot(false);
      }
    }

    this._specRoots.length = 0;
  },

  addCalcRoot : function(node)
  {
    // summary:
    //		Adds a new calculated root to the vector of roots. 
    // description:
    //		Adds a new calculated root to the vector of roots. 
    //		For internal usage only.
    // tags:
    //		private
    
    var realNode = node;

    var prop = TNodeParameters.getOrAllocate(this, node);

    if (!prop.isCalcRoot())
    {
      prop.setCalcRoot(true);
      this._calcRoots.push(realNode);
    }
  },

  getCalcRoots : function()
  {
    // summary:
    //		Returns the nodes that were used as roots during layout. 
    // description:
    //		Returns the nodes that were used as roots during layout. 
    //		<p>
    //		This method can be called after layout. If more than one connected
    //		component exists, each component has a root. If no root was
    //		specified, the layout algorithm automatically selects an appropriate
    //		node. If too many roots were specified, some specified roots will
    //		not become calculated roots during layout.  
    //		</p>
    // returns:
    //		The nodes that were used as roots during layout.
    
    return  new TranslateUtil.ArrayIterator(this._calcRoots); // ibm_ilog.graphlayout.IIterator
  },

  isCalcRoot : function(node)
  {
    // summary:
    //		Returns 'true' if the input node was used as root during
    //		layout. 
    // description:
    //		Returns <code>true</code> if the input node was used as root during
    //		layout. 
    //		This method can be called after the layout. If more than one
    //		connected component exists, each component has a root. If no root
    //		was specified, the layout algorithm automatically selects an
    //		appropriate node. If too many roots were specified, some specified
    //		roots will not become calculated roots during layout.  
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		'true' if the input node was used as root during
    //		layout.
    
    if (node == null) {
      return false; // Boolean
    }
    var prop = TNodeParameters.get(this, node);
    if (prop == null) {
      return false; // Boolean
    }
    return prop.isCalcRoot(); // Boolean
  },

  removeCalcRoot : function(node)
  {
    // summary:
    //		Remove a calculated root. 
    // description:
    //		Remove a calculated root. 
    //		This does not try to dispose the node property holding the
    //		parameters, because it is used only in situation where it is not
    //		necessary.
    // tags:
    //		private
    
    var prop = TNodeParameters.get(this, node);

    if (prop != null && prop.isCalcRoot())
    {
      prop.setCalcRoot(false);
      TranslateUtil.removeArrayObject(this._calcRoots, node);
    }
  },

  cleanCalcRoots : function()
  {
    // summary:
    //		Clean all calculated roots.
    // tags:
    //		private
    
    var e = this.getCalcRoots();
    var node;
    var prop;

    while (e.hasNext())
    {
      node = e.next();

      prop = TNodeParameters.get(this, node);

      if (prop != null && prop.isCalcRoot())
      {
        prop.setCalcRoot(false);
        if (!prop.isNeeded()) {
          prop.dispose(this, node);
        }
      }
    }

    this._calcRoots.length = 0;
  },

  setRootPreference : function(node, preference)
  {
    // summary:
    //		Specifies the preference of the node to become root node. 
    // description:
    //		Specifies the preference of the node to become root node. 
    //		The higher the preference, the more likely the node will be selected
    //		as root node.  
    //		<p>
    //		If a negative value is passed as input, the preference of the node
    //		is unspecified. In this case, the algorithm calculates a preference
    //		so that for directed trees the canonical root nodes with degree 0
    //		have the highest preference.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // preference: int
    //		The preference of the node to become a root node.
    
    LayoutParametersUtil.checkInputNode(this, node, "root node");
    if (preference > 10000) {
      preference = 10000;
    }
    if (preference < 10000) {
      this.removeSpecRoot(node);
    }
    TNodeParameters.setRootPreference(this, node, preference);
  },

  getRootPreference : function(node)
  {
    // summary:
    //		Returns the specified preference of the node to become root node. 
    // description:
    //		Returns the specified preference of the node to become root node. 
    //		The higher the preference, the more likely the node will be selected
    //		as root node. If no preference is specified for the node, it returns
    //		<code>-1</code>.  
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The specified preference of the node to become a root.
    
    return TNodeParameters.getRootPreference(this, node); // int
  },

  setPosition : function(point, isRootPosition )
  {
    // summary:
    //		Sets the position of the layout. 
    // description:
    //		Sets the position of the layout. 
    //		This can be used if connected component processing is disabled, or
    //		if the graph is fully connected. It has no effect otherwise.  
    //		<p>
    //		If no position is specified, the layout algorithm keeps the position
    //		of the first root node unchanged.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // point: Point
    //		The position.
    // isRootPosition: Boolean
    //		If 'true', it specifies the position of
    //		the first root node. This is useful in incremental mode. If
    //		'false', it specifies the position of the top-left
    //		border of the layout.
    
    var oldPosition = this._position;
    var oldIsRootPosition = this._isRootPosition;
    this._position = (point == null ? null : new Point(point .x, point .y));
    this._isRootPosition = isRootPosition || false ;

    if (point != null && oldPosition != null && !TranslateUtil.pointEquals(point, oldPosition)) {
      this.onParameterChanged("Position");
    }
    else if (point == null && oldPosition != null) {
      this.onParameterChanged("Position");
    }
    else if (point != null && oldPosition == null) {
      this.onParameterChanged("Position");
    }
    else if (isRootPosition != oldIsRootPosition) {
      this.onParameterChanged("Position");
    }
  },

  getPosition : function()
  {
    // summary:
    //		Returns the specified position of the layout. 
    // description:
    //		Returns the specified position of the layout. 
    //		This can be used if connected component processing is disabled, or
    //		if the graph is fully connected. It has no effect otherwise.  
    //		<p>
    //		 
    //		</p>
    // returns:
    //		The specified position of the layout.
    
    if (false) {
      return null; // Point
    }
    return (this._position != null ? new Point(this._position .x, this._position .y) : null); // Point
  },

  setRootPosition : function(isRootPosition)
  {
    // summary:
    //		Sets the meaning of the specified position. 
    // description:
    //		Sets the meaning of the specified position. 
    //		The specified position is the one that can be obtained from <code>
    //		TreeLayout.getPosition()</code>. If passed <code>true</code> the
    //		specified position means the position of the root node. Otherwise,
    //		it means the position of the top-left border of the layout. This can
    //		be used if connected component processing is disabled, or if the
    //		graph is fully connected. It has no effect otherwise.  
    //		<p>
    //		 
    //		</p>
    
    if (isRootPosition != this._isRootPosition)
    {
      this._isRootPosition = isRootPosition;
      this.onParameterChanged("RootPosition");
    }
  },

  isRootPosition : function()
  {
    // summary:
    //		Returns 'true' if the specified position means the
    //		position of the root node. 
    // description:
    //		Returns <code>true</code> if the specified position means the
    //		position of the root node. 
    //		Otherwise, it means the position of the top-left border of the
    //		layout. This can be used if connected component processing is
    //		disabled, or if the graph is fully connected. It has no effect
    //		otherwise.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return true; // Boolean
    }
    return this._isRootPosition; // Boolean
  },

  setInvisibleRootUsed : function(flag)
  {
    // summary:
    //		Sets whether an invisible root is used to combine components that
    //		are disconnected from each other. 
    // description:
    //		Sets whether an invisible root is used to combine components that
    //		are disconnected from each other. 
    //		The roots that are specified by <code> TreeLayout.setRoot()</code>
    //		or <code> TreeLayout.setRootPreference()</code> are the roots of
    //		each individual component. If an invisible root is used, all roots
    //		of each individual component are added as children to an invisible
    //		dummy node and the layout is performed on the resulting connected
    //		graph. By doing this, the components are arranged in the same global
    //		structure. In particular, this is useful for the radial layout modes
    //		to fit all components into the same global circular layer structure
    //		instead of generating an individual circular layer structure for
    //		each component.  
    //		<p>
    //		Note that the layout is no longer stable on incremental changes if
    //		an invisible root is used. The layout heuristic cannot preserve the
    //		relative order of the components in this case. This means that the
    //		order of the components may change in subsequent layouts, and hence
    //		the next layout may look very different from the previous layout.  
    //		</p>
    //		<p>
    //		The default value is <code>false</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (flag != this._isInvisibleRootUsed)
    {
      this._isInvisibleRootUsed = flag;
      this.onParameterChanged("InvisibleRootUsed");
    }
  },

  isInvisibleRootUsed : function()
  {
    // summary:
    //		Returns 'true' if an invisible root is used to combine
    //		components that are disconnected from each other. 
    // description:
    //		Returns <code>true</code> if an invisible root is used to combine
    //		components that are disconnected from each other. 
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return false; // Boolean
    }
    return this._isInvisibleRootUsed; // Boolean
  },

  setIntergraphConnectivityMode : function(flag)
  {
    // summary:
    //		Sets whether the intergraph link connectivity is considered for the
    //		partitioning of the layout into subtrees. 
    // description:
    //		Sets whether the intergraph link connectivity is considered for the
    //		partitioning of the layout into subtrees. 
    //		This affects nodes that represent subgraphs and that are only
    //		connected to other nodes by intergraph links (i.e. there are no
    //		normal intragraph links between the node and other nodes). If the
    //		option is disabled, these nodes are considered disconnected and are
    //		placed like disconnected nodes. If the option is enabled, the nodes
    //		are considered connected and arranged inside the tree structure (for
    //		instance as parent and child).  
    //		<p>
    //		Note that this option has no effect if the generic disconnected node
    //		placement is enabled (see <code>
    //		ibm&#95;ilog.graphlayout.GraphLayout.setLayoutOfConnectedComponentsEnabled()</code>).
    //		 
    //		</p>
    //		<p>
    //		Even if this option is enabled, the Tree Layout algorithm does not
    //		route intergraph links. They should be routed by applying a Link
    //		Layout after the Tree Layout.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // flag: Boolean
    //		'true' to set intergraph connectivity mode, or
    //		'false' to disable the mode.
    
    if (flag != this._intergraphLinkConnectivity)
    {
      this._intergraphLinkConnectivity = flag;
      this.onParameterChanged("IntergraphConnectivityMode");
    }
  },

  isIntergraphConnectivityMode : function()
  {
    // summary:
    //		Returns 'true' if the intergraph link connectivity is
    //		considered for the partitioning of the layout into subtrees.
    
    return (false ? TreeLayout._defaultIntergraphLinkConnectivity : this._intergraphLinkConnectivity); // Boolean
  },

  setCategorizingLinks : function(flag)
  {
    // summary:
    //		Sets whether the layout algorithm stores the categorization of links
    //		into forward, backward, and non-tree links. 
    // description:
    //		Sets whether the layout algorithm stores the categorization of links
    //		into forward, backward, and non-tree links. 
    //		If <code>true</code>, the next run of the layout algorithm makes the
    //		link categorization available through the methods <code>
    //		TreeLayout.getCalcForwardTreeLinks()</code>, <code>
    //		TreeLayout.getCalcBackwardTreeLinks()</code>, and <code>
    //		TreeLayout.getCalcNonTreeLinks()</code>. If <code>false</code>, the
    //		link categorization is not made available. This saves memory and is
    //		faster. In this case, the methods <code>
    //		TreeLayout.getCalcForwardTreeLinks()</code>, <code>
    //		TreeLayout.getCalcBackwardTreeLinks()</code>, and <code>
    //		TreeLayout.getCalcNonTreeLinks()</code> will return empty
    //		enumerations.  
    //		<p>
    //		Note that only normal links can be categorized. Intergraph links are
    //		never categorized, even if <code>
    //		TreeLayout.setIntergraphConnectivityMode()</code> is set to
    //		<code>true</code>.  
    //		</p>
    //		<p>
    //		The default value is <code>false</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (flag)
    {
      if (this._calcForwardTreeLinks == null)
      {
        this._calcForwardTreeLinks = new Array();
        this._calcBackwardTreeLinks = new Array();
        this._calcNonTreeLinks = new Array();
        this.onParameterChanged("CategorizingLinks");
      }
    }
    else
    {
      if (this._calcForwardTreeLinks != null)
      {
        this.cleanCalcForwardTreeLinks();
        this.cleanCalcBackwardTreeLinks();
        this.cleanCalcNonTreeLinks();
        this._calcForwardTreeLinks = null;
        this._calcBackwardTreeLinks = null;
        this._calcNonTreeLinks = null;
        this.onParameterChanged("CategorizingLinks");
      }
    }
  },

  isCategorizingLinks : function()
  {
    // summary:
    //		Returns 'true' if the layout algorithm stores the
    //		categorization of links into forward, backward, and non-tree links. 
    // description:
    //		Returns <code>true</code> if the layout algorithm stores the
    //		categorization of links into forward, backward, and non-tree links. 
    //		<p>
    //		Note that a run of the layout is necessary to make the
    //		categorization available. Even if this method returns
    //		<code>true</code>, the methods <code>
    //		TreeLayout.getCalcForwardTreeLinks()</code>, <code>
    //		TreeLayout.getCalcBackwardTreeLinks()</code>, and <code>
    //		TreeLayout.getCalcNonTreeLinks()</code> return empty enumerations if
    //		there was no run of the layout.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    return (this._calcForwardTreeLinks != null); // Boolean
  },

  setCategorizingLinksWithoutLayout : function(flag)
  {
    // summary:
    //		Sets whether the layout algorithm does not really perform a full
    //		layout. 
    // description:
    //		Sets whether the layout algorithm does not really perform a full
    //		layout. 
    //		It only calculates a spanning tree but does not move nodes or links.
    //		If categorizing links is additionally enabled, it stores the
    //		calculated root nodes and the categorization of links into forward,
    //		backward, and non-tree links.  Since intergraph links are never
    //		categorized, this works only in nonrecursive mode.   
    //		<p>
    //		Note that a run of the layout is necessary to make the
    //		categorization available. Otherwise, the methods <code>
    //		TreeLayout.getCalcForwardTreeLinks()</code>, <code>
    //		TreeLayout.getCalcBackwardTreeLinks()</code>, and <code>
    //		TreeLayout.getCalcNonTreeLinks()</code> return empty enumerations if
    //		there was no run of the layout.  
    //		</p>
    //		<p>
    //		The default value is <code>false</code>.  
    //		</p>
    
    if (flag != this._categorizingLinksWithoutLayout)
    {
      this._categorizingLinksWithoutLayout = flag;
      this.onParameterChanged("CategorizingLinksWithoutLayout");
    }
  },

  isCategorizingLinksWithoutLayout : function()
  {
    // summary:
    //		Returns 'true' if the layout algorithm does not really
    //		perform a full layout. 
    // description:
    //		Returns <code>true</code> if the layout algorithm does not really
    //		perform a full layout. 
    //		It only calculates a spanning tree but does not move nodes or links.
    //		If categorizing links is additionally enabled, it stores the
    //		calculated root nodes and the categorization of links into forward,
    //		backward, and non-tree links.  Since intergraph links are never
    //		categorized, this works only in nonrecursive mode.   
    //		<p>
    //		Note that a run of the layout is necessary to make the
    //		categorization available. Otherwise, the methods <code>
    //		TreeLayout.getCalcForwardTreeLinks()</code>, <code>
    //		TreeLayout.getCalcBackwardTreeLinks()</code>, and <code>
    //		TreeLayout.getCalcNonTreeLinks()</code> return empty enumerations if
    //		there was no run of the layout.   
    //		</p>
    
    return (false ? TreeLayout._defaultCategorizingLinksWithoutLayout : this._categorizingLinksWithoutLayout); // Boolean
  },

  addCalcForwardTreeLink : function(link)
  {
    // summary:
    //		Adds a new link to the vector of calculated forward tree links. 
    // description:
    //		Adds a new link to the vector of calculated forward tree links. 
    //		For internal usage only.
    // tags:
    //		private
    
    if (link != null && this._calcForwardTreeLinks != null)
    {
      var realLink = link;

      var prop = TLinkParameters.getOrAllocate(this, link);
      prop.setCalcForwardTreeLink(true);
      this._calcForwardTreeLinks.push(realLink);
    }
  },

  getCalcForwardTreeLinks : function()
  {
    // summary:
    //		Returns the forward links that were used as part of the spanning
    //		tree during layout. 
    // description:
    //		Returns the forward links that were used as part of the spanning
    //		tree during layout. 
    //		This can be called <i>after</i> layout if the link categorization
    //		option was set to <code>true</code> <i>before</i> layout by using
    //		the method <code> TreeLayout.setCategorizingLinks()</code>.  
    //		<p>
    //		The forward tree links have the property that the "from" node is the
    //		parent of the "to" node in the tree layout. These links are reshaped
    //		by the layout algorithm.  
    //		</p>
    //		<p>
    //		If the graph was not a tree, some links will not be part of the
    //		spanning tree that forms the base structure for the layout. If the
    //		graph was not a directed tree, some links need to be reversed to
    //		form the spanning tree. These links are not part of this
    //		enumeration.  
    //		</p>
    //		<p>
    //		Depending on whether recursive leaf mode is set, the enumeration may
    //		or may not contain intergraph links:  
    //		<ul>
    //		<li>If recursive leaf mode is switched off, only normal links are
    //		categorized. Intergraph links are not categorized as forward tree
    //		links, even if <code>
    //		TreeLayout.setIntergraphConnectivityMode()</code> is set to
    //		<code>true</code>.</li>  
    //		<li>If recursive leaf mode is switched on, intergraph links that
    //		were reshaped by the tree layout are categorized as forward or
    //		backward links.</li>  
    //		</ul>    
    //		</p>
    //		<p>
    //		 
    //		</p>
    // returns:
    //		The forward links that were used as part of the spanning tree
    //		during layout.
    
    if (this._calcForwardTreeLinks == null) {
      return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
    }
    else {
      return  new TranslateUtil.ArrayIterator(this._calcForwardTreeLinks); // ibm_ilog.graphlayout.IIterator
    }
  },

  cleanCalcForwardTreeLinks : function()
  {
    // summary:
    //		Clean all calculated forward tree links.
    // tags:
    //		private
    
    if (this._calcForwardTreeLinks != null)
    {
      var e =  new TranslateUtil.ArrayIterator(this._calcForwardTreeLinks);
      var link;
      var prop;

      while (e.hasNext())
      {
        link = e.next();

        prop = TLinkParameters.get(this, link);

        if (prop != null && prop.isCalcForwardTreeLink())
        {
          prop.setCalcForwardTreeLink(false);
          if (!prop.isNeeded()) {
            prop.dispose(this, link);
          }
        }
      }

      this._calcForwardTreeLinks.length = 0;
    }
  },

  addCalcBackwardTreeLink : function(link)
  {
    // summary:
    //		Adds a new link to the vector of calculated backward tree links. 
    // description:
    //		Adds a new link to the vector of calculated backward tree links. 
    //		For internal usage only.
    // tags:
    //		private
    
    if (link != null && this._calcBackwardTreeLinks != null)
    {
      var realLink = link;

      var prop = TLinkParameters.getOrAllocate(this, link);
      prop.setCalcBackwardTreeLink(true);
      this._calcBackwardTreeLinks.push(realLink);
    }
  },

  getCalcBackwardTreeLinks : function()
  {
    // summary:
    //		Returns the backward links that were used in the reverse direction
    //		as part of the spanning tree during layout. 
    // description:
    //		Returns the backward links that were used in the reverse direction
    //		as part of the spanning tree during layout. 
    //		This method can be called <i>after</i> layout if the link
    //		categorization option was set to <code>true</code> <i>before</i> the
    //		layout by using the method <code>
    //		TreeLayout.setCategorizingLinks()</code>.  
    //		<p>
    //		The backward tree links have the property that the "to" node is the
    //		parent of the "from" node in the tree layout. These links are
    //		reshaped by the layout algorithm.  
    //		</p>
    //		<p>
    //		If the graph was not a tree, some links will not be part of the
    //		spanning tree that forms the base structure for the layout. These
    //		links are not part of this enumeration. Also, links that do not need
    //		to be reversed but form the spanning tree are not part of this
    //		enumeration.  
    //		</p>
    //		<p>
    //		Depending on whether recursive leaf mode is set, the enumeration may
    //		contain intergraph links:  
    //		<ul>
    //		<li>If recursive leaf mode is switched off, only normal links are
    //		categorized. Intergraph links are not categorized as backward tree
    //		links, even if <code>
    //		TreeLayout.setIntergraphConnectivityMode()</code> is set to
    //		<code>true</code>.</li>  
    //		<li>If recursive leaf mode is switched on, intergraph links that
    //		were reshaped by the tree layout are categorized as forward or
    //		backward links.</li>  
    //		</ul>    
    //		</p>
    //		<p>
    //		 
    //		</p>
    // returns:
    //		The backward links that were used in the reverse direction as
    //		part of the spanning tree during layout.
    
    if (this._calcBackwardTreeLinks == null) {
      return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
    }
    else {
      return  new TranslateUtil.ArrayIterator(this._calcBackwardTreeLinks); // ibm_ilog.graphlayout.IIterator
    }
  },

  cleanCalcBackwardTreeLinks : function()
  {
    // summary:
    //		Clean all calculated backward tree links.
    // tags:
    //		private
    
    if (this._calcBackwardTreeLinks != null)
    {
      var e =  new TranslateUtil.ArrayIterator(this._calcBackwardTreeLinks);
      var link;
      var prop;

      while (e.hasNext())
      {
        link = e.next();

        prop = TLinkParameters.get(this, link);

        if (prop != null && prop.isCalcBackwardTreeLink())
        {
          prop.setCalcBackwardTreeLink(false);
          if (!prop.isNeeded()) {
            prop.dispose(this, link);
          }
        }
      }

      this._calcBackwardTreeLinks.length = 0;
    }
  },

  addCalcNonTreeLink : function(link)
  {
    // summary:
    //		Adds a new link to the vector of calculated non-tree links. 
    // description:
    //		Adds a new link to the vector of calculated non-tree links. 
    //		For internal usage only.
    // tags:
    //		private
    
    if (link != null && this._calcNonTreeLinks != null)
    {
      var realLink = link;

      var prop = TLinkParameters.getOrAllocate(this, link);
      prop.setCalcNonTreeLink(true);
      this._calcNonTreeLinks.push(realLink);
    }
  },

  getCalcNonTreeLinks : function()
  {
    // summary:
    //		Returns the links that were neither forward nor backward links in
    //		the spanning tree during layout. 
    // description:
    //		Returns the links that were neither forward nor backward links in
    //		the spanning tree during layout. 
    //		This can be called <i>after</i> the layout if the link
    //		categorization option was set to <code>true</code> <i>before</i> the
    //		layout by using the method <code>
    //		TreeLayout.setCategorizingLinks()</code>.  
    //		<p>
    //		The non-tree links have the property that neither the "from" node is
    //		the parent of the "to" node nor the "to" node is the parent of the
    //		"from" node in the tree layout. These links are not reshaped by the
    //		layout algorithm.  
    //		</p>
    //		<p>
    //		If the graph was not a tree, some links will not be part of the
    //		spanning tree that forms the base structure for the layout. Only
    //		these links are part of this enumeration. If necessary, they can be
    //		treated by a link routing layout algorithm after the tree layout
    //		(for example, by <code> ibm&#95;ilog.graphlayout.link.LinkLayout</code>
    //		).  
    //		</p>
    //		<p>
    //		Note that links that are removed by a layout filter, and links that
    //		are specified as fixed links or that are adjacent to fixed nodes,
    //		are not part of this enumeration even though they are also not part
    //		of the spanning tree.  
    //		</p>
    //		<p>
    //		Depending on the recursive leaf mode, it may contain intergraph
    //		links. If the recursive leaf mode is switched off, only only normal
    //		links are categorized. Intergraph links are not categorized as
    //		non-tree links, even if <code>
    //		TreeLayout.setIntergraphConnectivityMode()</code> is set to
    //		<code>true</code>. However if the recursive leaf mode is switched
    //		on, intergraph links that could not get reshaped by the tree layout
    //		are categorized as non-tree links.    
    //		</p>
    //		<p>
    //		 
    //		</p>
    // returns:
    //		The links that were neither forward nor backward links in the
    //		spanning tree during layout.
    
    if (this._calcNonTreeLinks == null) {
      return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
    }
    else {
      return  new TranslateUtil.ArrayIterator(this._calcNonTreeLinks); // ibm_ilog.graphlayout.IIterator
    }
  },

  cleanCalcNonTreeLinks : function()
  {
    // summary:
    //		Clean all calculated non-tree links.
    // tags:
    //		private
    
    if (this._calcNonTreeLinks != null)
    {
      var e =  new TranslateUtil.ArrayIterator(this._calcNonTreeLinks);
      var link;
      var prop;

      while (e.hasNext())
      {
        link = e.next();

        prop = TLinkParameters.get(this, link);

        if (prop != null && prop.isCalcNonTreeLink())
        {
          prop.setCalcNonTreeLink(false);
          if (!prop.isNeeded()) {
            prop.dispose(this, link);
          }
        }
      }

      this._calcNonTreeLinks.length = 0;
    }
  },

  removeCalcLinkFromCategory : function(link)
  {
    // summary:
    //		Remove the link from the vector of forward, backward, or non-tree
    //		links. 
    // description:
    //		Remove the link from the vector of forward, backward, or non-tree
    //		links. 
    //		This does not try to dispose the link property holding the
    //		parameters, because it is used only in situation where it is not
    //		necessary.
    // tags:
    //		private
    
    if (this._calcNonTreeLinks != null)
    {
      var prop = TLinkParameters.get(this, link);

      if (prop != null)
      {
        if (prop.isCalcForwardTreeLink())
        {
          prop.setCalcForwardTreeLink(false);

          TranslateUtil.removeArrayObject(this._calcForwardTreeLinks, link);
        }

        if (prop.isCalcBackwardTreeLink())
        {
          prop.setCalcBackwardTreeLink(false);

          TranslateUtil.removeArrayObject(this._calcBackwardTreeLinks, link);
        }

        if (prop.isCalcNonTreeLink())
        {
          prop.setCalcNonTreeLink(false);

          TranslateUtil.removeArrayObject(this._calcNonTreeLinks, link);
        }
      }
    }
  },

  setNonTreeLinksStraight : function(enable)
  {
    // summary:
    //		Sets whether all non-tree links that
    //		don't have the no-reshape link style and that are not fixed and are not incident to fixed nodes are reshaped to be straight. 
    // description:
    //		Sets whether all non-tree links that don't have the no-reshape link
    //		style and that are not fixed and are not incident to fixed nodes are
    //		reshaped to be straight. 
    //		Otherwise, the non-tree links are not changed at all.  
    //		<p>
    //		This option is disabled by default. When disabled, the tree layout
    //		only routes the links of the spanning tree, but no other link. When
    //		enabled, the tree layout iterates over the links obtained by <code>
    //		TreeLayout.getCalcNonTreeLinks()</code> and makes them straight.  
    //		</p>
    //		<p>
    //		It obeys the link clip interface and  the link connection box
    //		interface when making the non-tree links straight. The tree layout
    //		can only obey the link style and the connector style of the tree
    //		links but not of the non-tree links. In particular, it is not able
    //		to route non-tree links orthogonally. Only tree-links can be routed
    //		orthogonally. Usually it is recommended to apply an additional link
    //		layout to the non-tree links if there are any. However, if the
    //		non-tree links should just be straight, this option comes in handy
    //		since an additional link layout can be omitted in this case.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    
    if (enable != this._nonTreeLinksStraight)
    {
      this._nonTreeLinksStraight = enable;
      this.onParameterChanged("NonTreeLinksStraight");
    }
  },

  isNonTreeLinksStraight : function()
  {
    // summary:
    //		Returns 'true' if all non-tree links that
    //		don't have the no-reshape link style and that are not fixed and are not incident to fixed nodes are reshaped to be straight. 
    // description:
    //		Returns <code>true</code> if all non-tree links that don't have the
    //		no-reshape link style and that are not fixed and are not incident to
    //		fixed nodes are reshaped to be straight. 
    //		Otherwise, the non-tree links are not changed at all.  
    //		<p>
    //		 
    //		</p>
    
    if (false) {
      return TreeLayout._defaultNonTreeLinksStraight; // Boolean
    }
    return this._nonTreeLinksStraight; // Boolean
  },

  setAlignment : function(node, alignment)
  {
    // summary:
    //		Sets the alignment option of an individual node. 
    // description:
    //		Sets the alignment option of an individual node. 
    //		This alignment option is used only if the global alignment is set to
    //		<code> TreeLayout.MIXED</code>. Otherwise, all nodes have the
    //		alignment that is specified as the global alignment. This option has
    //		no effect in balloon layout mode.  
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.CENTER</code> - the node is centered with
    //		respect to the centers of its child nodes.</li>  
    //		<li> <code> TreeLayout.BORDER&#95;CENTER</code> - the node is centered
    //		with respect to the border of its child nodes.</li>  
    //		<li> <code> TreeLayout.EAST</code> - the node is aligned to the
    //		border of its easternmost child node.</li>  
    //		<li> <code> TreeLayout.WEST</code> - the node is aligned to the
    //		border of its westernmost child node.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;EAST</code> - a parent is outside
    //		the border of the child nodes, and the child nodes are placed
    //		sequentially instead of in parallel on the east side of the parent.
    //		This alignment option is not possible in the radial or level layout
    //		modes. </li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;WEST</code> - a parent is outside
    //		the border of the child nodes, and the child nodes are placed
    //		sequentially instead of in parallel on the west side of the parent.
    //		This alignment option is not possible in the radial or level layout
    //		modes. </li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;EAST&#95;WEST</code> - similar to <code>
    //		TreeLayout.TIP&#95;OVER&#95;EAST</code> or <code>
    //		TreeLayout.TIP&#95;OVER&#95;WEST</code>, but the child nodes are arranged on
    //		both sides of the link segment that starts at the parent node. The
    //		first child is placed on the east side. Further child nodes are
    //		placed on the east or west side, so that the space used on both
    //		sides is roughly the same, that is, the subtrees on the east side
    //		have roughly the same sum of heights as the subtrees on the west
    //		side. This alignment option is not possible in the radial or level
    //		layout modes. </li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;BOTH&#95;SIDES</code> - similar to
    //		<code> TreeLayout.TIP&#95;OVER&#95;EAST</code> or <code>
    //		TreeLayout.TIP&#95;OVER&#95;WEST</code>, but the child nodes are arranged in
    //		pairs on both sides of the link segment that starts at the parent
    //		node. Unlike the behavior with <code>
    //		TreeLayout.TIP&#95;OVER&#95;EAST&#95;WEST</code>, the algorithm does not try to
    //		optimize the space used on both sides but always puts the
    //		odd-numbered child nodes to the west and the even-numbered child
    //		nodes to the east, and the pairs of child nodes, one on the east
    //		side and one on the west side, are aligned with one another. This
    //		alignment option is not possible in the radial or level layout
    //		modes. </li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code> TreeLayout.CENTER</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // alignment: int
    //		The alignment option of the individual node.
    
    LayoutParametersUtil.checkInputNode(this, node, "input node");

    if (!(alignment == TreeLayout.CENTER || alignment == TreeLayout.BORDER_CENTER || alignment == TreeLayout.EAST || alignment == TreeLayout.WEST || alignment == TreeLayout.TIP_OVER ||
        alignment == TreeLayout.TIP_OVER_EAST || alignment == TreeLayout.TIP_OVER_WEST || alignment == TreeLayout.TIP_OVER_EAST_WEST || alignment == TreeLayout.TIP_OVER_BOTH_SIDES))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6716E",  alignment);
    }

    TNodeParameters.setAlignment(this, node, alignment);
  },

  getAlignment : function(node)
  {
    // summary:
    //		Returns the local alignment option of the node. 
    // description:
    //		Returns the local alignment option of the node. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The local alignment of the given node.
    
    return TNodeParameters.getAlignment(this, node); // int
  },

  setChildAlignment : function(node, alignment)
  {
    // summary:
    //		Sets the child alignment option of the node. 
    // description:
    //		Sets the child alignment option of the node. 
    //		This child alignment option is used to specify exceptional
    //		alignments for the child that differ from the alignment specified at
    //		the parent for all child nodes. This method always overwrites the
    //		aligment specified using <code> TreeLayout.setAlignment()</code> and
    //		<code> TreeLayout.setGlobalAlignment()</code> at the parent. Child
    //		alignments only take effect in free and level mode but not in radial
    //		or balloon mode, and only  if the child node is not a root node of a
    //		tree.  
    //		<p>
    //		Valid values are:  
    //		<ul>
    //		<li> <code> TreeLayout.UNSPECIFIED</code> - no exceptional alignment
    //		is specified for this child node.</li>  
    //		<li> <code> TreeLayout.EAST&#95;NEIGHBOR</code> - the child node is made
    //		a neighbor on the east side of its parent node.</li>  
    //		<li> <code> TreeLayout.WEST&#95;NEIGHBOR</code> - the child node is made
    //		a neighbor on the west side of its parent node.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;ASSOCIATE&#95;BOTH&#95;SIDES</code> - the
    //		child is laid out like an associate node of its parent node.
    //		Associate nodes occur alternating on the east and west sides of
    //		their parent node.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;ASSOCIATE&#95;EAST</code> - the child is
    //		laid out like an associate node of its parent node. This associate
    //		nodes occurs on the east side of its parent node.</li>  
    //		<li> <code> TreeLayout.TIP&#95;OVER&#95;ASSOCIATE&#95;WEST</code> - the child is
    //		laid out like an associate node of its parent node. This associate
    //		nodes occurs on the west side of its parent node.</li>  
    //		</ul>  
    //		</p>
    //		<p>
    //		The default value is <code> TreeLayout.UNSPECIFIED</code>.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // alignment: int
    //		The child alignment option of the input node. This
    //		child alignment option is used to specify exceptional alignments
    //		for the child node that differ from the alignment specified at the
    //		parent for all child nodes.
    
    LayoutParametersUtil.checkInputNode(this, node, "input node");

    if (
      !(alignment == TreeLayout.EAST_NEIGHBOR || alignment == TreeLayout.WEST_NEIGHBOR || alignment == TreeLayout.UNSPECIFIED || alignment == TreeLayout.TIP_OVER_ASSOCIATE_BOTH_SIDES ||
          alignment == TreeLayout.TIP_OVER_ASSOCIATE_EAST || alignment == TreeLayout.TIP_OVER_ASSOCIATE_WEST))
      {
        LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6717E",  alignment);
      }

    TNodeParameters.setChildAlignment(this, node, alignment);
  },

  getChildAlignment : function(node)
  {
    // summary:
    //		Returns the child alignment option of the node. 
    // description:
    //		Returns the child alignment option of the node. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The child alignment of the input node.
    
    return TNodeParameters.getChildAlignment(this, node); // int
  },

  setEastSubtreeBlockMargin : function(node, margin)
  {
    // summary:
    //		Sets the margin on the east side of the subtree starting at the
    //		input node. 
    // description:
    //		Sets the margin on the east side of the subtree starting at the
    //		input node. 
    //		If the value is set to a positive number, the subtree starting at
    //		this node is considered a rectangular block, and other nodes not
    //		belonging to this subtree are moved at least the specified margin
    //		away from this block. Specifying subtree block margins is useful if
    //		you plan to draw a rectangle around the subtree, since it ensures
    //		that unrelated nodes do not extend into the rectangle.  
    //		<p>
    //		The default value is -1, which means no margin.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // margin: Number
    //		The margin on the east side of the subtree starting at the
    //		input node.
    
    LayoutParametersUtil.checkInputNode(this, node, "input node");

    TNodeParameters.setSubtreeBlockMargin(this, node, 0, margin);
  },

  getEastSubtreeBlockMargin : function(node)
  {
    // summary:
    //		Returns the margin on the east side of the subtree starting at the
    //		input node. 
    // description:
    //		Returns the margin on the east side of the subtree starting at the
    //		input node. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The margin on the east side of the subtree starting at the input
    //		node.
    
    return TNodeParameters.getSubtreeBlockMargin(this, node, 0); // Number
  },

  setWestSubtreeBlockMargin : function(node, margin)
  {
    // summary:
    //		Sets the margin on the west side of the subtree starting at the
    //		input node. 
    // description:
    //		Sets the margin on the west side of the subtree starting at the
    //		input node. 
    //		If the value is set to a positive number, the subtree starting at
    //		this node is considered a rectangular block, and other nodes not
    //		belonging to this subtree are moved at least the specified margin
    //		away from this block. Specifying subtree block margins is useful if
    //		you plan to draw a rectangle around this subtree, since it ensures
    //		that unrelated nodes do not extend into this rectangle.  
    //		<p>
    //		The default value is -1, means no margin.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // margin: Number
    //		The margin on the west side of the subtree starting at the
    //		input node.
    
    LayoutParametersUtil.checkInputNode(this, node, "input node");

    TNodeParameters.setSubtreeBlockMargin(this, node, 2, margin);
  },

  getWestSubtreeBlockMargin : function(node)
  {
    // summary:
    //		Returns the margin on the west side of the subtree starting at the
    //		input node. 
    // description:
    //		Returns the margin on the west side of the subtree starting at the
    //		input node. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The margin on the west side of the subtree starting at the input
    //		node.
    
    return TNodeParameters.getSubtreeBlockMargin(this, node, 2); // Number
  },

  setNorthSubtreeBlockMargin : function(node, margin)
  {
    // summary:
    //		Sets the margin on the north side of the subtree starting at the
    //		input node. 
    // description:
    //		Sets the margin on the north side of the subtree starting at the
    //		input node. 
    //		If the value is set to a positive number, the subtree starting at
    //		this node is considered a rectangular block, and other nodes not
    //		belonging to this subtree are moved at least the specified margin
    //		away from this block. Specifying subtree block margins is useful if
    //		you plan to draw a rectangle around this subtree, since it ensures
    //		that unrelated nodes do not extend into the rectangle.  
    //		<p>
    //		The default value is -1, means no margin.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // margin: Number
    //		The margin on the north side of the subtree starting at
    //		the input node.
    
    LayoutParametersUtil.checkInputNode(this, node, "input node");

    TNodeParameters.setSubtreeBlockMargin(this, node, 3, margin);
  },

  getNorthSubtreeBlockMargin : function(node)
  {
    // summary:
    //		Returns the margin on the north side of the subtree starting at the
    //		input node. 
    // description:
    //		Returns the margin on the north side of the subtree starting at the
    //		input node. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The margin on the north side of the subtree starting at the
    //		input node.
    
    return TNodeParameters.getSubtreeBlockMargin(this, node, 3); // Number
  },

  setSouthSubtreeBlockMargin : function(node, margin)
  {
    // summary:
    //		Sets the margin on the south side of the subtree starting at the
    //		input node. 
    // description:
    //		Sets the margin on the south side of the subtree starting at the
    //		input node. 
    //		If the value is set to a positive number, the subtree starting at
    //		this node is considered a rectangular block, and other nodes not
    //		belonging to this subtree are moved at least the specified margin
    //		away from this block. Specifying subtree block margins is useful if
    //		you plan to draw a rectangle around this subtree, since it ensures
    //		that unrelated nodes do not reach into the rectangle.  
    //		<p>
    //		The default value is -1, which means no margin.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // margin: Number
    //		The margin on the south side of the subtree starting at
    //		the input node.
    
    LayoutParametersUtil.checkInputNode(this, node, "input node");

    TNodeParameters.setSubtreeBlockMargin(this, node, 1, margin);
  },

  getSouthSubtreeBlockMargin : function(node)
  {
    // summary:
    //		Returns the margin on the south side of the subtree starting at the
    //		input node. 
    // description:
    //		Returns the margin on the south side of the subtree starting at the
    //		input node. 
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // returns:
    //		The margin on the south side of the subtree starting at the
    //		input node.
    
    return TNodeParameters.getSubtreeBlockMargin(this, node, 1); // Number
  },

  setSubtreeBlockMargin : function(node, side, margin)
  {
    // summary:
    //		Sets the margin on the specified side of the subtree starting at the
    //		input node. 
    // description:
    //		Sets the margin on the specified side of the subtree starting at the
    //		input node. 
    //		Possible values for the <code>side</code> parameter are <code>
    //		TreeLayout.EAST</code>, <code> TreeLayout.WEST</code>, <code>
    //		TreeLayout.NORTH</code>, and <code> TreeLayout.SOUTH</code>. If the
    //		value is set to a positive number, the subtree starting at this node
    //		is considered a rectangular block, and other nodes not belonging to
    //		this subtree are moved at least the specified margin away from this
    //		block. Specifying subtree block margins is useful if you plan to
    //		draw a rectangle around this subtree, since it ensures that
    //		unrelated nodes do not extend into the rectangle.  
    //		<p>
    //		The default value is -1, which means no margin on the specified
    //		side.  
    //		</p>
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // side: int
    //		The subtree side.
    // margin: Number
    //		The margin on the specified side of the subtree starting
    //		at the input node.
    
    if (!(side == TreeLayout.EAST || side == TreeLayout.WEST || side == TreeLayout.NORTH || side == TreeLayout.SOUTH))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6718E",  side);
    }

    switch (side)
    {
      case TreeLayout.EAST:
        this.setEastSubtreeBlockMargin(node, margin);
        break;
      case TreeLayout.WEST:
        this.setWestSubtreeBlockMargin(node, margin);
        break;
      case TreeLayout.NORTH:
        this.setNorthSubtreeBlockMargin(node, margin);
        break;
      case TreeLayout.SOUTH:
        this.setSouthSubtreeBlockMargin(node, margin);
        break;
    }
  },

  getSubtreeBlockMargin : function(node, side)
  {
    // summary:
    //		Returns the margin on the specified side of the subtree starting at
    //		the input node. 
    // description:
    //		Returns the margin on the specified side of the subtree starting at
    //		the input node. 
    //		Possible values for the <code>side</code> parameter are <code>
    //		TreeLayout.EAST</code>, <code> TreeLayout.WEST</code>, <code>
    //		TreeLayout.NORTH</code>, and <code> TreeLayout.SOUTH</code>.  
    //		<p>
    //		 
    //		</p>
    // node: Object
    //		The node instance.
    // side: int
    //		The subtree side.
    // returns:
    //		The margin on the specified side of the subtree starting at the
    //		input node.
    
    if (!(side == TreeLayout.EAST || side == TreeLayout.WEST || side == TreeLayout.NORTH || side == TreeLayout.SOUTH))
    {
      LogResUtil.logAndThrowIllegalArgExc(" TreeLayout ", "graphlayout.expert.message.6718E",  side);
    }

    switch (side)
    {
      case TreeLayout.EAST:
        return this.getEastSubtreeBlockMargin(node); // Number
      case TreeLayout.WEST:
        return this.getWestSubtreeBlockMargin(node); // Number
      case TreeLayout.NORTH:
        return this.getNorthSubtreeBlockMargin(node); // Number
      case TreeLayout.SOUTH:
        return this.getSouthSubtreeBlockMargin(node); // Number
    }
    return -1; // Number
  },

  _temporarilyDisabled: null,
  _temporarilyDisabledLayouts: null,
  _layoutProvider: null,
  _positionOfTreeLayout: null,
  _subgraphCorrection: null,

  isRootLayoutInsideRecursiveLayoutTraversal : function()
  {
    // summary:
    //		Returns true if we are inside a recursive leaf mode layout and this
    //		is the root layout (more precisely, the topmost layout that got
    //		started). 
    // description:
    //		Returns true if we are inside a recursive leaf mode layout and this
    //		is the root layout (more precisely, the topmost layout that got
    //		started). 
    //		Returns false if this is a layout of a subgraph, of if it is a flat
    //		layout. We can distinguish both cases from the layout provider since
    //		the layout provider is only needed to build the BaseSubgraphData,
    //		and this is only done for the top level layout.
    // tags:
    //		private
    
    return this._layoutProvider != null; // Boolean
  },

  getSubgraphCorrection : function()
  {
    // summary:
    //		Returns the subgraph correction if any. 
    // description:
    //		Returns the subgraph correction if any. 
    //		This is only set at the root layout in a recurse leaf layout.
    // tags:
    //		private
    
    return this._subgraphCorrection; // ibm_ilog.graphlayout.ISubgraphCorrection
  },

  getSubgraphLayout : function(subgraphModel)
  {
    // summary:
    //		Returns the layout of a subgraph that is at the same position inside
    //		multiple layouts as this master layout.
    // tags:
    //		private
    
    return LayoutUtil.getLayoutFromPositionInMultipleLayout(this._layoutProvider.getGraphLayout(subgraphModel), this._positionOfTreeLayout); // ibm_ilog.graphlayout.GraphLayout
  },

  getTreatedSubgraphLayout : function(subgraphModel, insideLayout)
  {
    // summary:
    //		Returns the layout of a subgraph that is at the same position inside
    //		multiple layouts as this master layout. 
    // description:
    //		Returns the layout of a subgraph that is at the same position inside
    //		multiple layouts as this master layout. 
    //		Returns null for all layouts that cannot be treated by this layout.
    //		The flag insideLayout indicates whether this is called from inside
    //		layout of from outside layout. The former case is faster.
    // tags:
    //		private
    
    var layout = this.getSubgraphLayout(subgraphModel);

    if (insideLayout)
    {
      if (!(layout instanceof TreeLayout)) {
        return null; // ibm_ilog.graphlayout.tree.TreeLayout
      }
      var tlayout = layout;
      if (!tlayout._temporarilyDisabled) {
        return null; // ibm_ilog.graphlayout.tree.TreeLayout
      }
      return tlayout; // ibm_ilog.graphlayout.tree.TreeLayout
    }
    else
    {
      if (!this.canBeHandledByThisLayout(layout)) {
        return null; // ibm_ilog.graphlayout.tree.TreeLayout
      }
      return layout; // ibm_ilog.graphlayout.tree.TreeLayout
    }
  },

  canBeHandledByThisLayout : function(layout)
  {
    // summary:
    //		Checks whether the input layout, which is a layout of a subgraph. 
    // description:
    //		Checks whether the input layout, which is a layout of a subgraph. 
    //		can be handled by this layout. This is used in the recursive mode
    //		when nested subgraphs are handled at once. Therefore the layout of
    //		the nested subgraph does not need to run separately.
    // tags:
    //		private
    
    if (!(layout instanceof TreeLayout)) {
      return false; // Boolean
    }
    var tlayout = layout;
    if (!tlayout.isRecursiveLayoutFromAncestorAllowed()) {
      return false; // Boolean
    }
    if (this.getFlowDirection() != tlayout.getFlowDirection()) {
      return false; // Boolean
    }
    if (this.getGlobalLinkStyle() != tlayout.getGlobalLinkStyle()) {
      return false; // Boolean
    }
    if (this.getConnectorStyle() != tlayout.getConnectorStyle()) {
      return false; // Boolean
    }
    if (this.getLayoutMode() != tlayout.getLayoutMode()) {
      return false; // Boolean
    }
    if (this.isIncrementalMode() != tlayout.isIncrementalMode()) {
      return false; // Boolean
    }
    if (this.isRespectNodeSizes() != tlayout.isRespectNodeSizes()) {
      return false; // Boolean
    }
    if (this.getLevelAlignment() != tlayout.getLevelAlignment()) {
      return false; // Boolean
    }
    if (this.getGlobalAlignment() != tlayout.getGlobalAlignment()) {
      return false; // Boolean
    }
    if (this.getSiblingOffset() != tlayout.getSiblingOffset()) {
      return false; // Boolean
    }
    if (this.getParentChildOffset() != tlayout.getParentChildOffset()) {
      return false; // Boolean
    }
    if (this.getBranchOffset() != tlayout.getBranchOffset()) {
      return false; // Boolean
    }
    if (this.getTipOverBranchOffset() != tlayout.getTipOverBranchOffset()) {
      return false; // Boolean
    }
    if (this.getAssociateOffset() != tlayout.getAssociateOffset()) {
      return false; // Boolean
    }
    if (this.getParentChildAssociateOffset() != tlayout.getParentChildAssociateOffset()) {
      return false; // Boolean
    }
    if (this.getOverlapPercentage() != tlayout.getOverlapPercentage()) {
      return false; // Boolean
    }
    if (this.getOrthForkPercentage() != tlayout.getOrthForkPercentage()) {
      return false; // Boolean
    }
    return true; // Boolean
  },

  markLayoutsOfSubgraphsReenabled : function()
  {
    // summary:
    //		Enables all previously temporarily disabled layouts.
    // tags:
    //		private
    
    if (this._temporarilyDisabledLayouts == null) {
      return;
    }
    var _for_each_array_1 = this._temporarilyDisabledLayouts;
    for (var _for_each_index_1 = 0; _for_each_index_1 < _for_each_array_1.length; _for_each_index_1++){
      var layoutObj = _for_each_array_1[_for_each_index_1];
      (layoutObj)._temporarilyDisabled = false;
    }
    
    this._temporarilyDisabledLayouts = null;
    this._layoutProvider = null;
  },

  /**
   * In addition to the base functionality, this method updates the
   * percentage of completion value.
   *
   * @inheritDoc
   */
  onLayoutStepPerformed : function(layoutStarted, layoutFinished)
  {
    if (this._percCompleteControler != null)
      this._percCompleteControler.notifyPercentageComplete(this);
    this.inherited("onLayoutStepPerformed", arguments, [layoutStarted,layoutFinished]);
  },
  
  mayContinue : function()
  {
    // summary:
    //		Returns 'true' if the layout can continue.
    // tags:
    //		private
    
    if (!this._stoppedBeforeCompletion) {
      this._stoppedBeforeCompletion = this.isLayoutTimeElapsed() || this.isStoppedImmediately();
    }
    return (!this._stoppedBeforeCompletion); // Boolean
  },

  transferLayoutOptions : function(graph)
  {
    // summary:
    //		Transfers the layout parameters into the graph.
    // tags:
    //		private
    
    var mode = this.getLayoutMode();
    graph.setRadialLayout(mode == TreeLayout.RADIAL || mode == TreeLayout.ALTERNATING_RADIAL);
    graph.setBalloonLayout(mode == TreeLayout.BALLOON);
    graph.setLevelLayout(mode == TreeLayout.LEVEL || graph.isRadialLayout());
    graph.setAllowAlternating(mode == TreeLayout.ALTERNATING_RADIAL);

    switch (this.getFlowDirection())
    {
      case Direction.LEFT:
        graph.setFlowDirection(2);
        break;
      case Direction.RIGHT:
        graph.setFlowDirection(0);
        break;
      case Direction.TOP:
        graph.setFlowDirection(3);
        break;
      case Direction.BOTTOM:
        graph.setFlowDirection(1);
        break;
    }
    switch (this.getLevelAlignment())
    {
      case TreeLayout.CENTER:
        graph.setLevelJustification(0);
        break;
      case TreeLayout.NORTH:
        graph.setLevelJustification(-1);
        break;
      case TreeLayout.SOUTH:
        graph.setLevelJustification(1);
        break;
    }
    switch (this.getFlowDirection())
    {
      case Direction.LEFT:
      case Direction.RIGHT:
        graph.setMinDistBetweenNodes(1, this.getSiblingOffset());
        graph.setMinDistBetweenBranches(1, this.getBranchOffset());
        graph.setMinDistBetweenAssocs(1, this.getAssociateOffset());
        graph.setMinDistBetweenNodes(0, this.getParentChildOffset());
        graph.setMinDistBetweenBranches(0, this.getTipOverBranchOffset());
        graph.setMinDistBetweenAssocs(0, this.getParentChildAssociateOffset());
        break;
      case Direction.TOP:
      case Direction.BOTTOM:
        graph.setMinDistBetweenNodes(0, this.getSiblingOffset());
        graph.setMinDistBetweenBranches(0, this.getBranchOffset());
        graph.setMinDistBetweenAssocs(0, this.getAssociateOffset());
        graph.setMinDistBetweenNodes(1, this.getParentChildOffset());
        graph.setMinDistBetweenBranches(1, this.getTipOverBranchOffset());
        graph.setMinDistBetweenAssocs(1, this.getParentChildAssociateOffset());
        break;
    }

    var connStyle = this.getConnectorStyle();

    if (connStyle == TreeLayout.AUTOMATIC_PINS)
    {
      if (this.isClippingEnabled()) {
        connStyle = TreeLayout.CLIPPED_PINS;
      }
      else {
        connStyle = TreeLayout.CENTERED_PINS;
      }
    }

    graph.setConnectorStyle(connStyle);

    graph.setOverlapFactor(this.getOverlapPercentage() / 100.0);
    graph.setOrthForkFactor(this.getOrthForkPercentage() / 100.0);
    if (this.getFlowDirection() == Direction.LEFT || this.getFlowDirection() == Direction.RIGHT) {
      graph.setAspectRatio(1 / this.getAspectRatio());
    }
    else {
      graph.setAspectRatio(this.getAspectRatio());
    }
    this.updateAngleTransTable(graph);
  },

  isClippingEnabled : function()
  {
    // summary:
    // tags:
    //		private
    
    return this.isLinkClipping();
    
  }

}
);

lang.mixin(TreeLayout,
{
  NO_RESHAPE_STYLE : null,

  STRAIGHT_LINE_STYLE : null,

  ORTHOGONAL_STYLE : null,

  MIXED_STYLE : null,

  AUTOMATIC_PINS : null,

  CENTERED_PINS : null,

  CLIPPED_PINS : null,

  EVENLY_SPACED_PINS : null,

  EAST : null,

  WEST : null,

  NORTH : null,

  SOUTH : null,

  CENTER : null,

  BORDER_CENTER : null,

  TIP_OVER : null,

  TIP_OVER_EAST : null,

  TIP_OVER_WEST : null,

  TIP_OVER_EAST_WEST : null,

  TIP_OVER_BOTH_SIDES : null,

  MIXED : null,

  UNSPECIFIED : null,

  EAST_NEIGHBOR : null,

  WEST_NEIGHBOR : null,

  TIP_OVER_ASSOCIATE_BOTH_SIDES : null,

  TIP_OVER_ASSOCIATE_EAST : null,

  TIP_OVER_ASSOCIATE_WEST : null,

  FREE : null,

  LEVEL : null,

  RADIAL : null,

  ALTERNATING_RADIAL : null,

  BALLOON : null,

  TIP_LEAVES_OVER : null,

  TIP_ROOTS_OVER : null,

  TIP_ROOTS_AND_LEAVES_OVER : null,

  REGULAR : null,

  FAST_PROPORTIONAL : null,

  PROPORTIONAL : null,

  VARIABLE_RADIUS : null,

  OPTIMIZED_VARIABLE_RADIUS : null,

  UNIFORM_LEAVES_RADIUS : null,

  OPTIMIZED_UNIFORM_LEAVES_RADIUS : null,

  UNIFORM_RADIUS : null,

  _defaultLinkStyle : null,

  _defaultConnectorStyle : null,

  _defaultAlignment : null,

  _defaultLayoutMode : null,

  _defaultBalloonAngleSpacing : null,

  _defaultBalloonRadiusMode : null,

  _defaultFlowDirection : null,

  _defaultLevelAlignment : null,

  _defaultSiblingOffset : null,

  _defaultParentChildOffset : null,

  _defaultBranchOffset : null,

  _defaultTipOverBranchOffset : null,

  _defaultAssocOffset : null,

  _defaultParentChildAssocOffset : null,

  _defaultAspectRatio : null,

  _defaultOverlapPercentage : null,

  _defaultOrthForkPercentage : null,

  _defaultMaxChildrenAngle : null,

  _defaultFirstCircleEvenlySpacing : null,

  _defaultRespectNodeSizes : null,

  _defaultIncrementalMode : null,

  _defaultAllLevelsAlternating : null,

  _defaultNumberOfAlternatingCircles : null,

  _defaultNonTreeLinksStraight : null,

  _defaultIntergraphLinkConnectivity : null,

  _defaultBalloonStartAngle : null,

  _defaultBalloonRootChildrenAngle : null,

  _defaultBalloonInnerChildrenAngle : null,

  _defaultBalloonLeafChildrenAngle : null,

  _defaultCategorizingLinksWithoutLayout : null,
  _fields_init : function(c)
  {
    c = c || TreeLayout;
    if(GraphLayout._fields_init)
      GraphLayout._fields_init(c);
    
    c.NO_RESHAPE_STYLE = LinkStyles.NO_RESHAPE_STYLE;
    c.STRAIGHT_LINE_STYLE = LinkStyles.STRAIGHT_LINE_STYLE;
    c.ORTHOGONAL_STYLE = LinkStyles.ORTHOGONAL_STYLE;
    c.MIXED_STYLE = LinkStyles.MIXED_STYLE;
    c.AUTOMATIC_PINS = ConnectorStyles.AUTOMATIC_PINS;
    c.CENTERED_PINS = ConnectorStyles.CENTERED_PINS;
    c.CLIPPED_PINS = ConnectorStyles.CLIPPED_PINS;
    c.EVENLY_SPACED_PINS = ConnectorStyles.EVENLY_SPACED_PINS;
    c.EAST = AlignmentOptions.EAST;
    c.WEST = AlignmentOptions.WEST;
    c.NORTH = AlignmentOptions.NORTH;
    c.SOUTH = AlignmentOptions.SOUTH;
    c.CENTER = AlignmentOptions.CENTER;
    c.BORDER_CENTER = AlignmentOptions.BORDER_CENTER;
    c.TIP_OVER = AlignmentOptions.TIP_OVER;
    c.TIP_OVER_EAST = AlignmentOptions.TIP_OVER;
    c.TIP_OVER_WEST = AlignmentOptions.TIP_OVER_WEST;
    c.TIP_OVER_EAST_WEST = AlignmentOptions.TIP_OVER_EAST_WEST;
    c.TIP_OVER_BOTH_SIDES = AlignmentOptions.TIP_OVER_BOTH_SIDES;
    c.MIXED = AlignmentOptions.MIXED;
    c.UNSPECIFIED = AlignmentOptions.UNSPECIFIED;
    c.EAST_NEIGHBOR = AlignmentOptions.EAST_NEIGHBOR;
    c.WEST_NEIGHBOR = AlignmentOptions.WEST_NEIGHBOR;
    c.TIP_OVER_ASSOCIATE_BOTH_SIDES = AlignmentOptions.TIP_OVER_ASSOCIATE_BOTH_SIDES;
    c.TIP_OVER_ASSOCIATE_EAST = AlignmentOptions.TIP_OVER_ASSOCIATE_EAST;
    c.TIP_OVER_ASSOCIATE_WEST = AlignmentOptions.TIP_OVER_ASSOCIATE_WEST;
    c.FREE = TLayoutModes.FREE;
    c.LEVEL = TLayoutModes.LEVEL;
    c.RADIAL = TLayoutModes.RADIAL;
    c.ALTERNATING_RADIAL = TLayoutModes.ALTERNATING_RADIAL;
    c.BALLOON = TLayoutModes.BALLOON;
    c.TIP_LEAVES_OVER = TLayoutModes.TIP_LEAVES_OVER;
    c.TIP_ROOTS_OVER = TLayoutModes.TIP_ROOTS_OVER;
    c.TIP_ROOTS_AND_LEAVES_OVER = TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER;
    c.REGULAR = TAngleSpacingModes.REGULAR;
    c.FAST_PROPORTIONAL = TAngleSpacingModes.FAST_PROPORTIONAL;
    c.PROPORTIONAL = TAngleSpacingModes.PROPORTIONAL;
    c.VARIABLE_RADIUS = TBalloonRadiusModes.VARIABLE_RADIUS;
    c.OPTIMIZED_VARIABLE_RADIUS = TBalloonRadiusModes.OPTIMIZED_VARIABLE_RADIUS;
    c.UNIFORM_LEAVES_RADIUS = TBalloonRadiusModes.UNIFORM_LEAVES_RADIUS;
    c.OPTIMIZED_UNIFORM_LEAVES_RADIUS = TBalloonRadiusModes.OPTIMIZED_UNIFORM_LEAVES_RADIUS;
    c.UNIFORM_RADIUS = TBalloonRadiusModes.UNIFORM_RADIUS;
    c._defaultLinkStyle = TreeLayout.STRAIGHT_LINE_STYLE;
    c._defaultConnectorStyle = TreeLayout.AUTOMATIC_PINS;
    c._defaultAlignment = AlignmentOptions.DEFAULT;
    c._defaultLayoutMode = TreeLayout.FREE;
    c._defaultBalloonAngleSpacing = TreeLayout.PROPORTIONAL;
    c._defaultBalloonRadiusMode = TreeLayout.OPTIMIZED_UNIFORM_LEAVES_RADIUS;
    c._defaultFlowDirection = Direction.RIGHT;
    c._defaultLevelAlignment = TreeLayout.CENTER;
    c._defaultSiblingOffset = 40.0;
    c._defaultParentChildOffset = 40.0;
    c._defaultBranchOffset = 40.0;
    c._defaultTipOverBranchOffset = 40.0;
    c._defaultAssocOffset = 30.0;
    c._defaultParentChildAssocOffset = 30.0;
    c._defaultAspectRatio = 1.0;
    c._defaultOverlapPercentage = 30.0;
    c._defaultOrthForkPercentage = 45.0;
    c._defaultMaxChildrenAngle = 0;
    c._defaultFirstCircleEvenlySpacing = false;
    c._defaultRespectNodeSizes = true;
    c._defaultIncrementalMode = true;
    c._defaultAllLevelsAlternating = false;
    c._defaultNumberOfAlternatingCircles = 2;
    c._defaultNonTreeLinksStraight = false;
    c._defaultIntergraphLinkConnectivity = true;
    c._defaultBalloonStartAngle = 0;
    c._defaultBalloonRootChildrenAngle = 360;
    c._defaultBalloonInnerChildrenAngle = 360;
    c._defaultBalloonLeafChildrenAngle = 360;
    c._defaultCategorizingLinksWithoutLayout = false;
  }
  
}
);

TreeLayout._getGlobalProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'allowedTime', defaultValue: 32000 },
    { name: 'layoutOfConnectedComponentsEnabled', defaultValue: false },
    { name: 'preserveFixedNodes', defaultValue: false },
    { name: 'preserveFixedLinks', defaultValue: false },
    { name: 'splineRoutingEnabled', defaultValue: false },
    { name: 'minSplineCurveSize', defaultValue: 5 },
    { name: 'maxSplineCurveSize', defaultValue: 30 },
    { name: 'balanceSplineCurveThreshold', defaultValue: 10 },
    { name: 'globalLinkStyle', defaultValue: TreeLayout.STRAIGHT_LINE_STYLE },
    { name: 'connectorStyle', defaultValue: TreeLayout.AUTOMATIC_PINS },
    { name: 'flowDirection', defaultValue: Direction.RIGHT },
    { name: 'layoutMode', defaultValue: TreeLayout.FREE },
    { name: 'balloonAngleSpacing', defaultValue: TreeLayout.PROPORTIONAL },
    { name: 'balloonRadiusMode', defaultValue: TreeLayout.OPTIMIZED_UNIFORM_LEAVES_RADIUS },
    { name: 'balloonStartAngle', defaultValue: 0 },
    { name: 'balloonRootChildrenAngle', defaultValue: 360 },
    { name: 'balloonInnerChildrenAngle', defaultValue: 360 },
    { name: 'balloonLeafChildrenAngle', defaultValue: 360 },
    { name: 'incrementalMode', defaultValue: true },
    { name: 'allLevelsAlternating', defaultValue: false },
    { name: 'numberOfAlternatingCircles', defaultValue: 2 },
    { name: 'respectNodeSizes', defaultValue: true },
    { name: 'levelAlignment', defaultValue: TreeLayout.CENTER },
    { name: 'globalAlignment', defaultValue: TreeLayout.CENTER },
    { name: 'siblingOffset', defaultValue: 40 },
    { name: 'parentChildOffset', defaultValue: 40 },
    { name: 'branchOffset', defaultValue: 40 },
    { name: 'tipOverBranchOffset', defaultValue: 40 },
    { name: 'associateOffset', defaultValue: 30 },
    { name: 'parentChildAssociateOffset', defaultValue: 30 },
    { name: 'aspectRatio', defaultValue: 1 },
    { name: 'overlapPercentage', defaultValue: 30 },
    { name: 'maxChildrenAngle', defaultValue: 0 },
    { name: 'firstCircleEvenlySpacing', defaultValue: false },
    { name: 'orthForkPercentage', defaultValue: 45 },
    { name: 'rootPosition', defaultValue: true },
    { name: 'invisibleRootUsed', defaultValue: false },
    { name: 'intergraphConnectivityMode', defaultValue: true },
    { name: 'categorizingLinks', defaultValue: false },
    { name: 'categorizingLinksWithoutLayout', defaultValue: false },
    { name: 'nonTreeLinksStraight', defaultValue: false }
  ];
  var baseProperties = GraphLayout._getGlobalProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

TreeLayout._getNodeProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'fixed', defaultValue: false },
    { name: 'rootPreference', defaultValue: -1 },
    { name: 'alignment', defaultValue: TreeLayout.CENTER },
    { name: 'childAlignment', defaultValue: TreeLayout.UNSPECIFIED },
    { name: 'eastSubtreeBlockMargin', defaultValue: -1 },
    { name: 'westSubtreeBlockMargin', defaultValue: -1 },
    { name: 'northSubtreeBlockMargin', defaultValue: -1 },
    { name: 'southSubtreeBlockMargin', defaultValue: -1 }
  ];
  var baseProperties = GraphLayout._getNodeProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

TreeLayout._getLinkProperties = function()
{
  // tags:
  //		private
  var properties = [
    { name: 'fixed', defaultValue: false },
    { name: 'linkStyle', defaultValue: TreeLayout.STRAIGHT_LINE_STYLE }
  ];
  var baseProperties = GraphLayout._getLinkProperties();
  return baseProperties ? baseProperties.concat(properties) : properties;
};

/**
 * @private
 */
TreeLayoutAnonClass_1 = declare(SplineRouter,
{
  // tags:
  //		private
  
  __outerThis : null,

  constructor : function(arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
  },

  isReshapeAllowed : function(link)
  {
    // tags:
    //		private

    if (this.__outerThis.isPreserveFixedLinks() && this.__outerThis.isFixed(link)) {
      return false; // Boolean
    }
    var filter = this.__outerThis.getSplineLinkFilter();
    if (filter != null && !filter.isSpline(link)) {
      return false; // Boolean
    }
    if (this.__outerThis.getGlobalLinkStyle() == TreeLayout.MIXED_STYLE)
    {
      var ls = this.__outerThis.getLinkStyle(link);
      if (ls == TreeLayout.NO_RESHAPE_STYLE || ls == TreeLayout.STRAIGHT_LINE_STYLE) {
        return false; // Boolean
      }
    }

    return true; // Boolean
  }

});

(function(){ TreeLayout._fields_init(); })();

return TreeLayout;
});

},
'ibm_ilog/graphlayout/tree/TGraph':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TGraph", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/TranslateUtil",	// #5
  "../internalutil/AlignmentOptions",	// #6
  "../internalutil/BaseSubgraphData",	// #7
  "../internalutil/ConnectorStyles",	// #8
  "../internalutil/GraphModelUtil",	// #9
  "../internalutil/HashTable",	// #10
  "../internalutil/LayoutUtil",	// #11
  "../internalutil/LinkStyles",	// #12
  "../internalutil/NestedData",	// #13
  "../internalutil/PercCompleteController",	// #14
  "../internalutil/Point",	// #15
  "../internalutil/Rectangle",	// #16
  "../internalutil/TranslateUtil",	// #17
  "../tree/TLayoutModes",	// #18
  "../tree/TNode",	// #19
  "../tree/TNodeIterator",	// #20
  "../tree/TNodeParameters",	// #21
  "../tree/TPath",	// #22
  "../tree/TRadial",	// #23
  "../Direction"	// #24
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    TranslateUtil,	// #5
    AlignmentOptions,	// #6
    BaseSubgraphData,	// #7
    ConnectorStyles,	// #8
    GraphModelUtil,	// #9
    HashTable,	// #10
    LayoutUtil,	// #11
    LinkStyles,	// #12
    NestedData,	// #13
    PercCompleteController,	// #14
    Point,	// #15
    Rectangle,	// #16
    TranslateUtil,	// #17
    TLayoutModes,	// #18
    TNode,	// #19
    TNodeIterator,	// #20
    TNodeParameters,	// #21
    TPath,	// #22
    TRadial,	// #23
    Direction	// #24
    ) {

var TGraph = declare("ibm_ilog.graphlayout.tree.TGraph", null, 
{
  // summary:
  //		A tree graph contains all tree nodes. 
  // description:
  //		A tree graph contains all tree nodes. 
  //		Mainly we need this as encapsulation of all internal data structures
  //		of tree layout, in order to facilitate passing them around between
  //		the different algorithms. We layout either top to bottom or left to
  //		right, and then mirror. Note that east/west/north/south are messed up
  //		during layout, that is: flow dir =  0          1        2         3
  //		to right  to bottom  to left  to top W          N        W         S
  //		N S        W E      S N       W E E          S        E         N
  //		This needs to be corrected by mirroring of parameters/coordinates at
  //		the beginning, and mirroring of the result at the end.  
  // tags:
  //		private
  
  _graphlayout: null,

  _rootlayout: null,

  _graphModel: null,

  _nodes: null,

  _firstFreeNodeSlot: null,
  _nodesMap: null,

  _nodesThatAreSubgraphs: null,

  _invisibleRoot: null,

  _levelSize: null,
  _levelCoord: null,
  _desiredMinDist: null,

  _levelLayout: null,
  _radialLayout: null,
  _balloonLayout: null,
  _allowAlternating: null,
  _flowDirection: null,
  _levelJustification: null,
  _minDistBetweenNodes: null,
  _minDistBetweenBranches: null,
  _minDistBetweenAssocs: null,
  _aspectRatio: null,
  _overlapFactor: null,
  _orthForkFactor: null,

  _connectorStyle: null,

  _connBoxInterface: null,

  _angleTranslationTable: null,

  _maxDepth: null,
  _maxNodeSize: null,
  _radialTransAlg: null,

  _shiftDelta: null,

  _flattened: null,

  _flattenedSubgraphs: null,

  _percCompleteController: null,
  _percentages: null,

  constructor : function(graphlayout)
  {
    this._graphlayout = graphlayout;
    this._rootlayout = graphlayout;
    this._graphModel = null;
    this._nodes = null;
    this._invisibleRoot = null;
    this._firstFreeNodeSlot = -1;
    this._levelSize = null;
    this._levelCoord = null;
    this._desiredMinDist = null;
    this._levelLayout = true;
    this._radialLayout = false;
    this._balloonLayout = false;
    this._allowAlternating = false;
    this._flowDirection = 0;
    this._levelJustification = 0;
    this._minDistBetweenNodes = TranslateUtil.newArray(2, 0);
    this._minDistBetweenNodes[0] = 30.0;
    this._minDistBetweenNodes[1] = 30.0;
    this._minDistBetweenBranches = TranslateUtil.newArray(2, 0);
    this._minDistBetweenBranches[0] = 50.0;
    this._minDistBetweenBranches[1] = 50.0;
    this._minDistBetweenAssocs = TranslateUtil.newArray(2, 0);
    this._minDistBetweenAssocs[0] = 20.0;
    this._minDistBetweenAssocs[1] = 20.0;
    this._aspectRatio = 1.0;
    this._overlapFactor = 0.3;
    this._orthForkFactor = 0.45;
    this._maxDepth = 0;
    this._maxNodeSize = 0.0;
    this._connectorStyle = ConnectorStyles.AUTOMATIC_PINS;
    this._radialTransAlg = null;
    this._connBoxInterface = null;
    this._percCompleteController = new PercCompleteController();
    this._angleTranslationTable = null;
    this._shiftDelta = TranslateUtil.newArray(2, 0);
    this._flattened = false;
    this._nodesMap = new HashTable();
    graphlayout.transferLayoutOptions(this);
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._graphModel = null;
    this._nodes = null;
    this._levelSize = null;
    this._levelCoord = null;
    if (this._radialTransAlg != null) {
      this._radialTransAlg.disposeIt();
    }
    this._radialTransAlg = null;
  },

  copyPercentages : function(graph)
  {
    // summary:
    //		Copies the percentage stuff from the input graph.
    // tags:
    //		private
    
    this._percCompleteController = graph._percCompleteController;
    this._percentages = graph._percentages;
  },

  attach : function(graphModel)
  {
    // summary:
    //		Attach a graph model and create a representation so that each node
    //		in the model corresponds to a tree node in this graph, each link in
    //		the model corresponds to a tree link in this graph, each link has
    //		exactly one segment, and there are no dummy nodes at all.
    // tags:
    //		private
    
    this.attachFlat(graphModel);
    
  },

  attachFlat : function(graphModel)
  {
    this._nodes = null;
    this._graphModel = graphModel;
    var layout = this.getLayout();
    this._connBoxInterface = layout.getLinkConnectionBoxInterface();
    var numNodes = graphModel.getNodesCount();
    var numLinks = graphModel.getLinksCount();
    if (layout.isInvisibleRootUsed()) {
      numNodes++;
    }
    this._percCompleteController.startStep(this._percentages.percForAttach, numNodes + numLinks, false);

    this._nodes = TranslateUtil.newArray(numNodes, null);

    this.createNodes();
    this.createLinks();

    if (this.isRadialLayout())
    {
      var angle = layout.getMaxChildrenAngle();
      var radAngle = 0.0;

      if (angle > 0)
      {
        radAngle = ((angle)/180.0*Math.PI);
        if (radAngle < 0.0) {
          radAngle += 6.283;
        }
      }

      this._radialTransAlg = new TRadial(this, radAngle);
    }

    if (layout.getNodeComparator() != null)
    {
      this.calcSpecOrderIndex(layout.getNodeComparator());
    }

    this.layoutStepPerformed();
  },

  increaseNodeCapacity : function(numNewNodes)
  {
    // summary:
    //		Increase the capacity of the graph by more nodes.
    // tags:
    //		private
    
    if (this._nodes == null) {
      this._nodes = TranslateUtil.newArray(numNewNodes, null);
    }
    else
    {
      var n = TranslateUtil.newArray(this._nodes.length + numNewNodes, null);

      for (var i = 0; i < this._nodes.length; i++)
        n[i] = this._nodes[i];
      this._nodes = n;
    }
  },

  createDummyNode : function()
  {
    // summary:
    //		Create a new dummy node.
    // tags:
    //		private
    
    var node = new TNode(null);
    this._nodes[this._firstFreeNodeSlot++] = node;
    return node; // ibm_ilog.graphlayout.tree.TNode
  },

  createNodes : function()
  {
    // summary:
    //		Create representatives for node objects in the tree. 
    // description:
    //		Create representatives for node objects in the tree. 
    //		It also stores the relationship between node object and tree node in
    //		the properties of the node object, so that we later can retrieve
    //		which node object corresponds to which tree node.
    // tags:
    //		private
    
    var model = this.getModel();
    var layout = this.getLayout();
    var e = new TranslateUtil.ArrayIterator(model.getNodes());
    var preserveFixed = layout.isPreserveFixedNodes();
    var obj;
    var eastNeighborObj;
    var westNeighborObj;
    var node;
    var neighbor;
    var indegree;
    var outdegree;
    var delta;
    var dirTowardsLeaves = this.getDirTowardsLeaves();

    var globalAlignment = layout.getGlobalAlignment();
    var isLevel = this.isLevelLayout();
    var incremental = layout.isIncrementalMode();
    var hasAssociates = false;
    this._firstFreeNodeSlot = 0;

    if (layout.isInvisibleRootUsed())
    {
      node = new TNode(null);
      this._nodes[this._firstFreeNodeSlot++] = node;
      this._invisibleRoot = node;

      if (this._maxNodeSize < node.getSize(dirTowardsLeaves)) {
        this._maxNodeSize = node.getSize(dirTowardsLeaves);
      }
    }

    var maxx = 0.0;
    var maxy = 0.0;

    while (e.hasNext())
    {
      obj = e.next();

      this._percCompleteController.addPoints(1);
      if (preserveFixed && layout.isFixed(obj)) {
        continue;
      }
      node = new TNode(obj);
      this._nodes[this._firstFreeNodeSlot++] = node;

      TNodeParameters.beforeLayout(layout, obj, node);
      this._nodesMap.add(obj, node);

      if (this.getFlowDirection() == 0 || this.getFlowDirection() == 3) {
        node.mirrorSubtreeBlockMarginsEastWest();
      }
      this.transferCoordinates(obj, node);

      if (!incremental)
      {
        node.setMinCoord(0, maxx + 1);
        node.setMinCoord(1, maxy + 1);
        maxx = node.getMaxCoord(0);
        maxy = node.getMaxCoord(1);
      }

      if (this._maxNodeSize < node.getSize(dirTowardsLeaves)) {
        this._maxNodeSize = node.getSize(dirTowardsLeaves);
      }
      eastNeighborObj = layout.getEastNeighbor(obj);
      westNeighborObj = layout.getWestNeighbor(obj);
      indegree = model.getLinksToCount(obj);
      outdegree = model.getLinksFromCount(obj);
      node.setNormalDegrees(indegree, outdegree);
      delta = 0;
      if (eastNeighborObj != null) {
        delta++;
      }
      if (westNeighborObj != null) {
        delta++;
      }
      var e2 = layout.getAssociates(obj);

      while (e2.hasNext())
      {
        hasAssociates = true;
        e2.next();
        delta++;
      }

      node.setChildrenCapacity(indegree + outdegree + delta);

      node.setSpecRootNodePreference(layout.getRootPreference(obj));

      var alignment = globalAlignment;
      if (globalAlignment == LinkStyles.MIXED_STYLE) {
        alignment = layout.getAlignment(obj);
      }
      if (this.getFlowDirection() == 0 || this.getFlowDirection() == 3)
      {
        if (alignment == AlignmentOptions.TIP_OVER) {
          alignment = AlignmentOptions.TIP_OVER_WEST;
        }
        else if (alignment == AlignmentOptions.TIP_OVER_WEST) {
          alignment = AlignmentOptions.TIP_OVER;
        }

        var aux = eastNeighborObj;
        eastNeighborObj = westNeighborObj;
        westNeighborObj = aux;
      }

      if (isLevel &&
        (alignment == AlignmentOptions.TIP_OVER || alignment == AlignmentOptions.TIP_OVER_WEST || alignment == AlignmentOptions.TIP_OVER_EAST_WEST || alignment == AlignmentOptions.TIP_OVER_BOTH_SIDES)) {
        alignment = AlignmentOptions.CENTER;
      }
      node.setAlignmentStyle(alignment);
      node.setChildAlignment(layout.getChildAlignment(obj));

      if (eastNeighborObj != null)
      {
        neighbor = this.getObjectNode(eastNeighborObj);

        if (neighbor != null)
        {
          node.addChild(neighbor);
          node.setEastWestNeighbor(1, neighbor);
          neighbor.addChild(node);
          neighbor.setEastWestNeighbor(0, node);
        }
      }

      if (westNeighborObj != null)
      {
        neighbor = this.getObjectNode(westNeighborObj);

        if (neighbor != null)
        {
          node.addChild(neighbor);
          node.setEastWestNeighbor(0, neighbor);
          neighbor.addChild(node);
          neighbor.setEastWestNeighbor(1, node);
        }
      }

      this.layoutStepPerformed();
    }

    if (hasAssociates)
    {
      e = new TranslateUtil.ArrayIterator(model.getNodes());

      while (e.hasNext())
      {
        obj = e.next();
        node = this.getObjectNode(obj);
        if (node == null) {
          continue;
        }
        var assocParent = layout.getAssociateParent(obj);
        if (assocParent == null) {
          continue;
        }
        var parent = this.getObjectNode(assocParent);
        if (parent == null) {
          continue;
        }
        parent.addAssociate(node);
        parent.addChild(node);
      }
    }
  },

  createLinks : function()
  {
    // summary:
    //		Stores a child-parent connection for all links. 
    // description:
    //		Stores a child-parent connection for all links. 
    //		We
    //		don't have data representatives for link objects in the tree. Initially, we don't
    //		know yet what will be child and parent, hence we store all as child.
    //		The calculation of the spanning tree will resolve this.
    // tags:
    //		private
    
    var model = this.getModel();
    var layout = this.getLayout();
    var e = new TranslateUtil.ArrayIterator(model.getLinks());
    var preserveFixed = layout.isPreserveFixedLinks();
    var obj;
    var fromNode;
    var toNode;

    while (e.hasNext())
    {
      obj = e.next();

      this._percCompleteController.addPoints(1);
      if (preserveFixed && layout.isFixed(obj)) {
        continue;
      }
      fromNode = this.getObjectNode(model.getFrom(obj));
      toNode = this.getObjectNode(model.getTo(obj));

      if (fromNode != null && toNode != null && fromNode != toNode)
      {
        fromNode.addChild(toNode);
        toNode.addChild(fromNode);
        this.layoutStepPerformed();
      }
    }
  },

  transferCoordinates : function(nodeObj, node)
  {
    // summary:
    //		Transfers all coordinate info of a real node to the TNode.
    // tags:
    //		private
    
    var model = this.getModel();
    var origBBox;
    var realBBox = GraphModelUtil.getObjectBounds(model, nodeObj);
    var offsetToVirtualCenter = null;

    if (this._connBoxInterface != null)
    {
      origBBox = this.getNodeBoxFromConnBoxIfc(nodeObj);
      var tOffs = this.getTangOffsetFromConnBoxIfc(nodeObj, Direction.TOP);
      var bOffs = this.getTangOffsetFromConnBoxIfc(nodeObj, Direction.BOTTOM);
      var lOffs = this.getTangOffsetFromConnBoxIfc(nodeObj, Direction.LEFT);
      var rOffs = this.getTangOffsetFromConnBoxIfc(nodeObj, Direction.RIGHT);

      if (tOffs != 0.0 || bOffs != 0.0 || lOffs != 0.0 || rOffs != 0.0)
      {
        offsetToVirtualCenter = TranslateUtil.newArray(4, 0);
        offsetToVirtualCenter[0] = tOffs;
        offsetToVirtualCenter[1] = bOffs;
        offsetToVirtualCenter[2] = lOffs;
        offsetToVirtualCenter[3] = rOffs;
      }
    }
    else {
      origBBox = realBBox;
    }
    node.setBoundingBoxes(origBBox, realBBox, offsetToVirtualCenter, this.getLayout().isRespectNodeSizes(), this.isRadialLayout());

    switch (this.getFlowDirection())
    {
      case 2:
        node.mirrorOffsetsToVirtualCenter(0);
        break;
      case 3:
        node.mirrorOffsetsToVirtualCenter(1);
        break;
    }
  },

  getNodeBoxFromConnBoxIfc : function(nodeObj)
  {
    // summary:
    //		Returns the node box from the node box interface.
    // tags:
    //		private
    
    var model = this.getModel();

    if (model instanceof BaseSubgraphData)
    {
      var m = model;
      return m.getNodeBox(this._connBoxInterface, nodeObj); // Rectangle
    }

    return this._connBoxInterface.getBox(model, nodeObj); // Rectangle
  },

  getTangOffsetFromConnBoxIfc : function(nodeObj, side)
  {
    // summary:
    //		Returns the tangential offset from the connection box interface.
    // tags:
    //		private
    
    var model = this.getModel();

    if (model instanceof BaseSubgraphData)
    {
      var m = model;
      return m.getTangentialOffset(this._connBoxInterface, nodeObj, side); // Number
    }

    return this._connBoxInterface.getTangentialOffset(model, nodeObj, side); // Number
  },

  detach : function(redraw)
  {
    // summary:
    //		Detach the graph model. 
    // description:
    //		Detach the graph model. 
    //		This also clears the attach properties of the nodes. The redraw flag
    //		is true if we want to redraw the layout at the movements.
    // tags:
    //		private
    
    if (this.getModel() instanceof BaseSubgraphData) {
      this.detachRecursive(redraw);
    }
    else {
      this.detachFlat(redraw);
    }
  },

  detachFlat : function(redraw)
  {
    var model = this.getModel();
    var numNodes = model.getNodesCount();
    var numLinks = model.getLinksCount();

    this._percCompleteController.startStep(this._percentages.percForDetachNodes, numNodes, false);
    this.positionRealNodes(redraw);

    var finalStep = !this._graphlayout.isSplineRoutingEnabled();
    this._percCompleteController.startStep(this._percentages.percForDetachLinks, numNodes + 3 * numLinks, finalStep);
    this.routeRealLinks(redraw);
  },

  positionRealNodes : function(redraw)
  {
    // summary:
    //		Position the real nodes finally. 
    // description:
    //		Position the real nodes finally. 
    //		The redraw flag is the one we want to use during moveNode.
    // tags:
    //		private
    
    var model = this.getModel();

    if (model != null)
    {
      var layout = this.getLayout();
      var obj;
      var node;

      var e = new TranslateUtil.ArrayIterator(model.getNodes());

      while (e.hasNext())
      {
        obj = e.next();

        this._percCompleteController.addPoints(1);
        node = this.getObjectNode(obj);
        if (node == null) {
          continue;
        }
        model.moveNode(obj, node.getFinalX(), node.getFinalY());

        if (node.getImmediateParent() == this._invisibleRoot) {
          layout.addCalcRoot(obj);
        }
        this.layoutStepPerformed();
      }
    }
  },

  routeRealLinks : function (a0, a1)
  {
    // summary:
    //		Route the real links finally. 
    // description:
    //		Route the real links finally. 
    //		The redraw flag is the one we want to use during reshapeLink.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.routeRealLinks_overload_1(a0);
    else if(arguments.length == 2)
      return this.routeRealLinks_overload_2(a0, a1);
  },
  
  routeRealLinks_overload_1 : function(redraw)
                                              
  {
    var model = this.getModel();
    if (model == null) {
      return;
    }
    this.routeRealLinks(new TranslateUtil.ArrayIterator(model.getLinks()), redraw);
  },

  routeRealLinks_overload_2 : function(e, redraw)
  {
    // summary:
    //		Route all links in the input enumeration.
    // tags:
    //		private
    
    var model = this.getModel();
    var layout = this.getLayout();
    var preserveFixed = layout.isPreserveFixedLinks();

    var ydir = this.getDirTowardsLeaves();
    var xdir = 1 - ydir;

    var xside0 = 2 * ydir;
    var xside1 = xside0 + 1;
    var yside0 = 2 * xdir;
    var yside1 = yside0 + 1;

    var globalLinkStyle = layout.getGlobalLinkStyle();

    var localLinkStyle;
    var isNonTreeLink;
    var obj;
    var fromNode;
    var toNode;
    var parentNode;
    var childNode;
    var fromPoint = new Point();
    var toPoint = new Point();
    var points = TranslateUtil.newArray(2, null);
    points[0] = new Point();
    points[1] = new Point();
    var fromP = TranslateUtil.newArray(2, 0);
    var toP = TranslateUtil.newArray(2, 0);
    var parentP;
    var childP;
    var orthSplitCoord;

    var auxPnt3 = null;
    var auxPnt4 = null;
    var auxRct2 = null;

    if (this._linkClipInterface != null)
    {
      auxPnt3 = new Point();
      auxPnt4 = new Point();
      auxRct2 = new Rectangle();
    }

    var auxPnt1 = TranslateUtil.newArray(2, 0);
    var auxPnt2 = TranslateUtil.newArray(2, 0);
    var auxRct1 = TranslateUtil.newArray(4, 0);

    var connectorStyle = this.getConnectorStyle();
    if (connectorStyle == ConnectorStyles.EVENLY_SPACED_PINS) {
      this.setChildIndexe();
    }
    var fromOffset;
    var toOffset;
    var dummyNode;

    while (e.hasNext())
    {
      obj = e.next();
      isNonTreeLink = false;

      this._percCompleteController.addPoints(3);
      if (preserveFixed && layout.isFixed(obj)) {
        continue;
      }
      fromNode = this.getObjectNode(model.getFrom(obj));
      toNode = this.getObjectNode(model.getTo(obj));
      if (fromNode == null || toNode == null) {
        continue;
      }
      if (fromNode.getOriginalParent() == toNode)
      {
        parentNode = toNode;
        childNode = fromNode;
        parentP = toP;
        childP = fromP;
        layout.addCalcBackwardTreeLink(obj);
      }
      else if (toNode.getOriginalParent() == fromNode)
      {
        parentNode = fromNode;
        childNode = toNode;
        parentP = fromP;
        childP = toP;
        layout.addCalcForwardTreeLink(obj);
      }
      else
      {
        parentNode = null;
        childNode = null;
        parentP = null;
        childP = null;
        layout.addCalcNonTreeLink(obj);
        isNonTreeLink = true;
      }

      localLinkStyle = layout.getLinkStyle(obj);
      if (globalLinkStyle != LinkStyles.MIXED_STYLE) {
        localLinkStyle = globalLinkStyle;
      }
      if (localLinkStyle == LinkStyles.NO_RESHAPE_STYLE) {
        continue;
      }
      if (localLinkStyle == LinkStyles.ORTHOGONAL_STYLE)
      {
        if (this.isBalloonLayout() || this.isRadialLayout()) {
          localLinkStyle = LinkStyles.STRAIGHT_LINE_STYLE;
        }
      }

      if (isNonTreeLink)
      {
        if (layout.isNonTreeLinksStraight())
        {
          localLinkStyle = LinkStyles.STRAIGHT_LINE_STYLE;
          this.calcLinkShapeForRadial(fromNode, toNode, fromP, toP, auxPnt1, auxPnt2, auxRct1);
          TranslateUtil.movePoint(fromPoint, fromP[0], fromP[1]);
          TranslateUtil.movePoint(toPoint, toP[0], toP[1]);
          GraphModelUtil.reshapeLink(model, this._graphlayout, obj, localLinkStyle, fromPoint, null, 0, 0, toPoint);
        }

        continue;
      }

      if (parentNode.getLastDummyTipOverBothSides() != null) {
        parentNode.setAlignmentStyle(parentNode.getLastDummyTipOverBothSides().getAlignmentStyle());
      }
      if (parentNode.isTipOver() || parentNode.isTipOverBothSides() || parentNode.hasAssociate(childNode)) {
        localLinkStyle = LinkStyles.ORTHOGONAL_STYLE;
      }
      if (this.isClippedConnectorStyle(localLinkStyle, parentNode, childNode)) {
        this.calcLinkShapeForRadial(fromNode, toNode, fromP, toP, auxPnt1, auxPnt2, auxRct1);
      }
      else if (parentNode.hasEastWestNeighbor(childNode))
      {
        if (fromNode.getCenter(xdir) < toNode.getCenter(xdir))
        {
          fromP[xdir] = fromNode.getMaxCoord(xdir);
          toP[xdir] = toNode.getMinCoord(xdir);
          fromOffset = fromNode.getOffsetToVirtualCenter(xside1);
          toOffset = toNode.getOffsetToVirtualCenter(xside0);
        }
        else
        {
          fromP[xdir] = fromNode.getMinCoord(xdir);
          toP[xdir] = toNode.getMaxCoord(xdir);
          fromOffset = fromNode.getOffsetToVirtualCenter(xside0);
          toOffset = toNode.getOffsetToVirtualCenter(xside1);
        }

        if (fromNode.getSize(ydir) < toNode.getSize(ydir)) {
          fromP[ydir] = fromNode.getCenter(ydir) + fromOffset;
        }
        else {
          fromP[ydir] = toNode.getCenter(ydir) + toOffset;
        }
        toP[ydir] = fromP[ydir];
      }
      else if (this.isOneBendOrthStyle(localLinkStyle, parentNode, childNode))
      {
        var parentOffset;
        var childOffset;

        if (parentNode.getCenter(ydir) < childNode.getCenter(ydir))
        {
          parentP[ydir] = parentNode.getMaxCoord(ydir);
          parentOffset = fromNode.getOffsetToVirtualCenter(yside1);
        }
        else
        {
          parentP[ydir] = parentNode.getMinCoord(ydir);
          parentOffset = fromNode.getOffsetToVirtualCenter(yside0);
        }

        if (parentNode.getCenter(xdir) + parentOffset < childNode.getCenter(xdir))
        {
          childP[xdir] = childNode.getMinCoord(xdir);
          childOffset = childNode.getOffsetToVirtualCenter(xside0);
        }
        else
        {
          childP[xdir] = childNode.getMaxCoord(xdir);
          childOffset = childNode.getOffsetToVirtualCenter(xside1);
        }

        parentP[xdir] = parentNode.getCenter(xdir) + parentOffset;
        childP[ydir] = childNode.getCenter(ydir) + childOffset;
      }
      else
      {
        if (fromNode.getCenter(ydir) < toNode.getCenter(ydir))
        {
          fromP[ydir] = fromNode.getMaxCoord(ydir);
          toP[ydir] = toNode.getMinCoord(ydir);
          fromOffset = fromNode.getOffsetToVirtualCenter(yside1);
          toOffset = toNode.getOffsetToVirtualCenter(yside0);
        }
        else
        {
          fromP[ydir] = fromNode.getMinCoord(ydir);
          toP[ydir] = toNode.getMaxCoord(ydir);
          fromOffset = fromNode.getOffsetToVirtualCenter(yside0);
          toOffset = toNode.getOffsetToVirtualCenter(yside1);
        }

        toP[xdir] = toNode.getCenter(xdir) + toOffset;

        var testNode = parentNode;
        if (parentNode.getLastDummyTipOverBothSides() != null) {
          testNode = parentNode.getLastDummyTipOverBothSides();
        }
        var numChildren = testNode.getNumberOfChildren();
        if (testNode.getEastWestNeighbor(0) != null) {
          numChildren--;
        }
        if (testNode.getEastWestNeighbor(1) != null) {
          numChildren--;
        }
        if (connectorStyle == ConnectorStyles.EVENLY_SPACED_PINS && numChildren > 0)
        {
          var virtualCenter = fromNode.getCenter(xdir) + fromOffset;
          var halfW = Math.min(virtualCenter - fromNode.getMinCoord(xdir), fromNode.getMaxCoord(xdir) - virtualCenter);
          if (halfW < 0) {
            halfW = 0;
          }
          fromP[xdir] = virtualCenter - halfW + 2 * halfW * ((childNode.getIndex() + 1) / (numChildren + 1));
        }
        else {
          fromP[xdir] = fromNode.getCenter(xdir) + fromOffset;
        }
      }

      if (parentNode.isTipOverBothSides() || parentNode.hasAssociate(childNode))
      {
        dummyNode = childNode.getImmediateParent();
        var node1 = dummyNode.getEastWestNeighbor(0);
        var node2 = dummyNode.getEastWestNeighbor(1);

        if (node1 != null && node2 != null)
        {
          var offset1;
          var offset2;

          if (node1.getCenter(xdir) < node2.getCenter(xdir))
          {
            offset1 = node1.getOffsetToVirtualCenter(xside1);
            offset2 = node2.getOffsetToVirtualCenter(xside0);
          }
          else
          {
            offset1 = node1.getOffsetToVirtualCenter(xside0);
            offset2 = node2.getOffsetToVirtualCenter(xside1);
          }

          if (node1.getSize(ydir) < node2.getSize(ydir)) {
            childP[ydir] = node1.getCenter(ydir) + offset1;
          }
          else {
            childP[ydir] = node2.getCenter(ydir) + offset2;
          }
        }
      }

      fromPoint.x = fromP[0];
      fromPoint.y = fromP[1];
      toPoint.x = toP[0];
      toPoint.y = toP[1];

      if (this.isOneBendOrthStyle(localLinkStyle, parentNode, childNode))
      {
        if (ydir == 0)
        {
          points[0].y = parentP[1];
          points[0].x = childP[0];
        }
        else
        {
          points[0].x = parentP[0];
          points[0].y = childP[1];
        }

        GraphModelUtil.reshapeLink(model, this._graphlayout, obj, localLinkStyle, fromPoint, points, 0, 1, toPoint);
      }
      else if (!this.isRadialLayout() && !this.isBalloonLayout() && !parentNode.hasEastWestNeighbor(childNode) && localLinkStyle == LinkStyles.ORTHOGONAL_STYLE)
      {
        points[0].x = fromPoint.x;
        points[0].y = fromPoint.y;
        points[1].x = toPoint.x;
        points[1].y = toPoint.y;
        orthSplitCoord = childNode.getOrthSplitCoord(ydir);

        if (this.getDirTowardsLeaves() == 0)
        {
          points[0].x = orthSplitCoord;
          points[1].x = orthSplitCoord;
        }
        else
        {
          points[0].y = orthSplitCoord;
          points[1].y = orthSplitCoord;
        }

        GraphModelUtil.reshapeLink(model, this._graphlayout, obj, localLinkStyle, fromPoint, points, 0, 2, toPoint);
      }
      else if (
        this.isRadialLayout() || this.isBalloonLayout() || this._graphlayout.isClippingEnabled() || parentNode.hasEastWestNeighbor(childNode) || localLinkStyle == LinkStyles.STRAIGHT_LINE_STYLE)
        {
          if (!parentNode.hasEastWestNeighbor(childNode) && parentNode.getLastDummyTipOverBothSides() != null)
          {
            dummyNode = parentNode.getLastDummyTipOverBothSides();
            var maxY = 0;

            if (this.getFlowDirection() == 2 || this.getFlowDirection() == 3)
            {
              maxY = dummyNode.getMinCoord(ydir);
              if (dummyNode.getEastWestNeighbor(0) != null) {
                maxY = Math.min(maxY, dummyNode.getEastWestNeighbor(0).getMinCoord(ydir));
              }
              if (dummyNode.getEastWestNeighbor(1) != null) {
                maxY = Math.min(maxY, dummyNode.getEastWestNeighbor(1).getMinCoord(ydir));
              }
            }
            else
            {
              maxY = dummyNode.getMaxCoord(ydir);
              if (dummyNode.getEastWestNeighbor(0) != null) {
                maxY = Math.max(maxY, dummyNode.getEastWestNeighbor(0).getMaxCoord(ydir));
              }
              if (dummyNode.getEastWestNeighbor(1) != null) {
                maxY = Math.max(maxY, dummyNode.getEastWestNeighbor(1).getMaxCoord(ydir));
              }
            }

            points[0].x = fromPoint.x;
            points[0].y = fromPoint.y;
            if (ydir == 0) {
              points[0].x = maxY;
            }
            else {
              points[0].y = maxY;
            }
            localLinkStyle = LinkStyles.POLYLINE_STYLE;
            GraphModelUtil.reshapeLink(model, this._graphlayout, obj, localLinkStyle, fromPoint, points, 0, 1, toPoint);
          }
          else
          {
            localLinkStyle = LinkStyles.STRAIGHT_LINE_STYLE;
            GraphModelUtil.reshapeLink(model, this._graphlayout, obj, localLinkStyle, fromPoint, null, 0, 0, toPoint);
          }
        }

      this.layoutStepPerformed();
    }
  },

  routeIntergraphLinks : function(nestedModel, filter, redraw)
  {
    // summary:
    //		Routes the intergraph links. 
    // description:
    //		Routes the intergraph links. 
    //		This routing is fairly blindly and can produce ugly shapes. But in
    //		many cases it works just fine. Occurs only in recursive mode.
    // tags:
    //		private
    
    this.routeRealLinks((new TranslateUtil.ArrayIterator(nestedModel.getChildrenLinks(filter, false, true))), redraw);
  },

  calcLinkShapeForRadial : function(fromNode, toNode, fromPoint, toPoint, fromP, toP, rect)
  {
    // summary:
    //		Fills the from and to point of the link for radial layout. 
    // description:
    //		Fills the from and to point of the link for radial layout. 
    //		If there is no link clip interface, it clips the link at the
    //		bounding box of the nodes. If there is a clip interface, it points
    //		the links to the virtual center and relies on the clip interface for
    //		clipping  
    // fromNode: TNode
    //		the source node of the link.
    // toNode: TNode
    //		the target node of the link.
    // fromPoint: Array
    //		the container for the result.
    // toPoint: Array
    //		the container for the result.
    // fromP: Array
    //		auxiliary field needed inside, allocated outside.
    // toP: Array
    //		auxiliary field needed inside, allocated outside.
    // rect: Array
    //		auxiliary field needed inside, allocated outside.
    // tags:
    //		private
    
    fromP[0] = fromNode.getVirtualCenter(0);
    fromP[1] = fromNode.getVirtualCenter(1);
    toP[0] = toNode.getVirtualCenter(0);
    toP[1] = toNode.getVirtualCenter(1);

    rect[0] = fromNode.getMinCoord(0);
    rect[1] = fromNode.getMaxCoord(0);
    rect[2] = fromNode.getMinCoord(1);
    rect[3] = fromNode.getMaxCoord(1);
    LayoutUtil.clip(fromP, toP, rect);
    fromPoint[0] = fromP[0];
    fromPoint[1] = fromP[1];
    fromP[0] = fromNode.getCenter(0);
    fromP[1] = fromNode.getCenter(1);
    rect[0] = toNode.getMinCoord(0);
    rect[1] = toNode.getMaxCoord(0);
    rect[2] = toNode.getMinCoord(1);
    rect[3] = toNode.getMaxCoord(1);
    LayoutUtil.clip(toP, fromP, rect);
    toPoint[0] = toP[0];
    toPoint[1] = toP[1];
  },

  registerCalcRootsAndLinkCategories : function()
  {
    // summary:
    //		This registers calculated roots and link categories without moving
    //		nodes or reshaping links.
    // tags:
    //		private
    
    var model = this.getModel();
    if (model == null) {
      return;
    }
    var layout = this.getLayout();
    var obj;
    var node;
    var e = new TranslateUtil.ArrayIterator(model.getNodes());

    while (e.hasNext())
    {
      obj = e.next();
      node = this.getObjectNode(obj);
      if (node == null) {
        continue;
      }
      if (node.getImmediateParent() == this._invisibleRoot) {
        layout.addCalcRoot(obj);
      }
    }

    var preserveFixed = layout.isPreserveFixedLinks();
    var fromNode;
    var toNode;
    e = new TranslateUtil.ArrayIterator(model.getLinks());

    while (e.hasNext())
    {
      obj = e.next();
      if (preserveFixed && layout.isFixed(obj)) {
        continue;
      }
      fromNode = this.getObjectNode(model.getFrom(obj));
      toNode = this.getObjectNode(model.getTo(obj));
      if (fromNode == null || toNode == null) {
        continue;
      }
      if (fromNode.getOriginalParent() == toNode)
      {
        layout.addCalcBackwardTreeLink(obj);
      }
      else if (toNode.getOriginalParent() == fromNode)
      {
        layout.addCalcForwardTreeLink(obj);
      }
      else
      {
        layout.addCalcNonTreeLink(obj);
      }
    }
  },

  layoutStepPerformed : function()
  {
    // summary:
    //		Informs the layout instance that a step of the layout algorithm has
    //		been performed. 
    // description:
    //		Informs the layout instance that a step of the layout algorithm has
    //		been performed. 
    //		The method notified the listeners of the layout events.
    // tags:
    //		private
    
    var layout = this.getRootLayout();
    if (layout != null) {
      layout.callLayoutStepPerformedIfNeeded();
    }
  },

  correctFlattenedSubgraphPositionAndSizes : function()
  {
    var n = this.getFlattenedSubgraphNumber();

    for (var i = n - 1; i >= 0; i--)
    {
      var node = this.getFlattenedSubgraph(i);
      var graph = node.getRepresentationGraph();
      graph.correctSubtreeFlattenedSubgraphPositionAndSizes(node);
    }
  },

  correctSubtreeFlattenedSubgraphPositionAndSizes : function(node)
  {
    // summary:
    //		Correct the node bounds during recursive layout. 
    // description:
    //		Correct the node bounds during recursive layout. 
    //		This happens when all subgraphs are already laid out, i.e. their
    //		nodes have final positions, which influences the bounds of the
    //		subgraph itself.  
    // node: TNode
    //		The node in the parent graph representing this subgraph.
    // tags:
    //		private
    
    var subgraphObj = node.getOriginalObject();
    var bounds = this.getBoundsFromNodesAndMargins(subgraphObj);

    node.setBoundingBoxes(bounds, bounds, null, this.getLayout().isRespectNodeSizes(), this.isRadialLayout());
    this.clearShiftDelta();
  },

  correctUnflattenedSubgraphSizes : function()
  {
    // summary:
    //		Correct the node sizes during recursive layout. 
    // description:
    //		Correct the node sizes during recursive layout. 
    //		This happens when all subgraphs are already laid out, i.e. their
    //		nodes have final positions, which influences the bounds of the
    //		subgraph itself.
    // tags:
    //		private
    
    for (var i = 0; i < this.getNumberOfNodes(); i++)
    {
      var node = this.getNode(i);
      var subgraph = node.getRepresentationGraph();
      if (subgraph == null) {
        continue;
      }
      var cx = node.getCenter(0);
      var cy = node.getCenter(1);
      subgraph.correctSubtreeFlattenedSubgraphPositionAndSizes(node);
      node.setCenter(0, cx);
      node.setCenter(1, cy);
    }

    this.clearShiftDelta();
  },

  getBoundsFromNodesAndMargins : function(subgraphObj)
  {
    // summary:
    //		Returns the bounds of this graph from its contained nodes, including
    //		the margin.  
    // subgraphObj: Object
    //		The object that represents this subgraph in the
    //		model.
    // tags:
    //		private
    
    var nestedModel = this.getModel();
    var rect = this.getBoundsFromNodes();
    rect.x -= nestedModel.getSubgraphMargin(subgraphObj, Direction.LEFT);
    rect.y -= nestedModel.getSubgraphMargin(subgraphObj, Direction.TOP);
    rect.width += nestedModel.getSubgraphMargin(subgraphObj, Direction.LEFT) + nestedModel.getSubgraphMargin(subgraphObj, Direction.RIGHT);
    rect.height += nestedModel.getSubgraphMargin(subgraphObj, Direction.TOP) + nestedModel.getSubgraphMargin(subgraphObj, Direction.BOTTOM);
    return rect; // Rectangle
  },

  getBoundsFromNodes : function()
  {
    // summary:
    //		Returns the bounds of this graph from its contained nodes.
    // tags:
    //		private
    
    var rect = null;

    for (var i = 0; i < this.getNumberOfNodes(); i++)
    {
      var node = this.getNode(i);
      var x = node.getFinalX();
      var y = node.getFinalY();
      var w = node.getRealSize(0);
      var h = node.getRealSize(1);

      if (rect == null) {
        rect = new Rectangle(x, y, w, h);
      }
      else
      {
        TranslateUtil.addPoint(rect, x, y);
        TranslateUtil.addPoint(rect, x + w, y + h);
      }
    }

    return rect; // Rectangle
  },

  eliminateSubgraphShiftDeltas : function(nestedModel, filter)
  {
    // summary:
    //		Eliminate the shift deltas of the subgraphs recursively. 
    // description:
    //		Eliminate the shift deltas of the subgraphs recursively. 
    //		At the end, this graph and all subgraphs have no shift deltas
    //		anymore, that is, the coordinates of all nodes are then absolute.  
    // tags:
    //		private
    
    this.eliminateShiftDelta();

    var e = (new TranslateUtil.ArrayIterator(nestedModel.getChildrenSubgraphs(filter)));

    while (e.hasNext())
    {
      var subgraphObj = e.next();
      var node = this.getObjectNode(subgraphObj);
      var subgraph = node.getRepresentationGraph();
      subgraph.eliminateSubgraphShiftDeltas(nestedModel, subgraphObj);
    }
  },

  eliminateShiftDelta : function()
  {
    // summary:
    //		Eliminates the shift delta. 
    // description:
    //		Eliminates the shift delta. 
    //		The shift delta is set when a subgraph moves, to indicate that its
    //		children are not up to date. The shifting of the children is delayed
    //		until we need the absolute coordinate of the children.
    // tags:
    //		private
    
    if (this._shiftDelta[0] == 0 && this._shiftDelta[1] == 0) {
      return;
    }
    var n = this.getNumberOfNodes();

    for (var i = 0; i < n; i++)
    {
      var node = this.getNode(i);
      node.shiftCoord(0, this._shiftDelta[0]);
      node.shiftCoord(1, this._shiftDelta[1]);
    }

    this.clearShiftDelta();
  },

  addShiftDelta : function(index, value)
  {
    // summary:
    //		Shift the subgraph by a delta.
    // tags:
    //		private
    
    this._shiftDelta[index] += value;
  },

  clearShiftDelta : function()
  {
    // summary:
    //		Clears the shift delta of the subgraph.
    // tags:
    //		private
    
    this._shiftDelta[0] = 0;
    this._shiftDelta[1] = 0;
  },

  isFlattened : function()
  {
    // summary:
    //		Returns whether the subgraph is flattened. 
    // description:
    //		Returns whether the subgraph is flattened. 
    //		A flattened subgraph is handled by its parent graph and does not
    //		need to be laid out itself.
    // tags:
    //		private
    
    return this._flattened; // Boolean
  },

  setFlattened : function(flag)
  {
    // summary:
    //		Sets whether the subgraph is flattened. 
    // description:
    //		Sets whether the subgraph is flattened. 
    //		A flattened subgraph is handled by its parent graph and does not
    //		need to be laid out itself.
    // tags:
    //		private
    
    this._flattened = flag;
  },

  flattenLeafSubtrees : function()
  {
    // summary:
    //		Flatten the subgraphs. 
    // description:
    //		Flatten the subgraphs. 
    //		Called only for the top level graph.
    // tags:
    //		private
    
    var n = this.getNumberOfNodes();

    for (var i = 0; i < n; i++)
    {
      var child = this.getNode(i);
      if (child.getRepresentationGraph() == null) {
        continue;
      }
      if (child.getRealChildren().hasNext()) {
        continue;
      }
      TGraph.flattenLeafSubtrees(this, child);
    }

    var numUnflattenedSubgraphs = this.getNumberSubgraphs() - 1;
    var numSubgraphs = (this.getModel()).getFlattenedSubgraphsCount();

    this._percCompleteController.setStepEstimation(this.percParameterForRecLeafLayout(numSubgraphs, numUnflattenedSubgraphs));
  },

  addFlattenedNodes : function(orig, replacement, graph)
  {
    // summary:
    //		Puts the nodes from the input graph to this graph. 
    // description:
    //		Puts the nodes from the input graph to this graph. 
    //		orig is the subgraph node representing the input graph. It gets
    //		replaced by the root node of the input graph.
    // tags:
    //		private
    
    var i;

    for (i = 0; i < this._firstFreeNodeSlot; i++)
      if (this._nodes[i] == orig) {
        this._nodes[i] = replacement;
      }
    var n = graph.getNumberOfNodes();
    this.increaseNodeCapacity(n - 1);

    for (i = 0; i < n; i++)
    {
      var node = graph.getNode(i);
      if (node != replacement) {
        this._nodes[this._firstFreeNodeSlot++] = node;
      }
    }

    var parent = orig.getImmediateParent();
    replacement.setOriginalParent(parent);
    parent.replaceChild(orig, replacement);
    this.addFlattenedSubgraph(orig);
  },

  addFlattenedSubgraph : function(subgraph)
  {
    // summary:
    //		Adds a flattened subgraph node. 
    // description:
    //		Adds a flattened subgraph node. 
    //		Since we remove the node by the root of the subgraph, it is
    //		afterwards not anymore in the node list. To keep it reachable, we
    //		add it to a separate list. Note that at the end, the graph inside
    //		which all subgraphs are flattened will contain all flattened
    //		subgraph nodes.
    // tags:
    //		private
    
    if (this._flattenedSubgraphs == null) {
      this._flattenedSubgraphs = new Array();
    }
    this._flattenedSubgraphs.push(subgraph);
  },

  getFlattenedSubgraphNumber : function()
  {
    // summary:
    //		Returns the number of flattened subgraphs.
    // tags:
    //		private
    
    return (this._flattenedSubgraphs == null ? 0 : this._flattenedSubgraphs.length); // int
  },

  getFlattenedSubgraph : function(i)
  {
    // summary:
    //		Returns the i-th flattened subgraph node.
    // tags:
    //		private
    
    return this._flattenedSubgraphs[i]; // ibm_ilog.graphlayout.tree.TNode
  },

  getNumberSubgraphs : function()
  {
    // summary:
    //		Count the number of unflattened subgraphs, including this one.
    // tags:
    //		private
    
    var result = 1;
    var n = this.getNumberOfNodes();

    for (var i = 0; i < n; i++)
    {
      var node = this.getNode(i);
      if (node.getRepresentationGraph() != null) {
        result += node.getRepresentationGraph().getNumberSubgraphs();
      }
    }

    return result; // int
  },

  getNumberOfNodes : function()
  {
    // summary:
    //		Returns the number of nodes.
    // tags:
    //		private
    
    return ((this._firstFreeNodeSlot > 0 ? this._firstFreeNodeSlot : 0)); // int
  },

  getObjectNode : function(nodeObj)
  {
    // summary:
    //		Returns the TNode that corresponds to the input node object. 
    // description:
    //		Returns the TNode that corresponds to the input node object. 
    //		Works also for nodes that are not in this subgraph but in the same
    //		nesting hierarchy that is handled alltogether.  
    // tags:
    //		private
    
    return this._nodesMap.item(nodeObj); // ibm_ilog.graphlayout.tree.TNode
  },

  getNode : function (a0, a1, a2, a3)
  {
    // summary:
    //		Returns the i-th node.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.getNode_overload_1(a0);
    else if(arguments.length == 4)
      return this.getNode_overload_4(a0, a1, a2, a3);
  },
  
  getNode_overload_1 : function(i)
                                  
  {
    return (this._nodes[i]); // ibm_ilog.graphlayout.tree.TNode
  },

  swapPosition : function(node, i)
  {
    // summary:
    //		Sets the node at index i. 
    // description:
    //		Sets the node at index i. 
    //		This means it swaps the position of the input node in the nodes
    //		array with the node that is at position i.
    // tags:
    //		private
    
    var index = node.getIndex();
    this._nodes[index] = this._nodes[i];
    this._nodes[index].setIndex(index);
    this._nodes[i] = node;
    node.setIndex(i);
  },

  getNodes : function()
  {
    // summary:
    //		Returns all the nodes.
    // tags:
    //		private
    
    return new TGraphAnonClass_1(this); // ibm_ilog.graphlayout.tree.TNodeIterator
  },

  getRoots : function()
  {
    // summary:
    //		Returns all the roots. 
    // description:
    //		Returns all the roots. 
    //		This can be used only after calculating the spanning tree. In this
    //		case, the roots are the first nodes in the node field.
    // tags:
    //		private
    
    return new TGraphAnonClass_2(this); // ibm_ilog.graphlayout.tree.TNodeIterator
  },

  getNodesThatAreSubgraphs : function()
  {
    // summary:
    //		Returns all the nodes that have a representation graph. 
    // description:
    //		Returns all the nodes that have a representation graph. 
    //		Only used in recursive mode.
    // tags:
    //		private
    
    return new TGraphAnonClass_3(this); // ibm_ilog.graphlayout.tree.TNodeIterator
  },

  calcSubtreeMaxDepth : function(node)
  {
    // summary:
    //		Returns the maximal depth of the tree starting at node.  
    // tags:
    //		private
    
    var maxDepth = 1;
    var h;

    for (var i = 0; i < node.getNumberOfChildren(); i++)
    {
      h = 1 + this.calcSubtreeMaxDepth(node.getChild(i));
      if (h > maxDepth) {
        maxDepth = h;
      }
    }

    return (maxDepth); // int
  },

  calcMaxDepth : function()
  {
    // summary:
    //		Calculates the maximal depth overall.
    // tags:
    //		private
    
    var h;
    this._maxDepth = 0;
    var iter = this.getRoots();

    while (iter.hasNext())
    {
      h = this.calcSubtreeMaxDepth(iter.next());
      if (h > this._maxDepth) {
        this._maxDepth = h;
      }
    }

    return (this._maxDepth); // int
  },

  initSiblingChains : function()
  {
    // summary:
    //		Initialize the sibling chains.
    // tags:
    //		private
    
    var iter = this.getRoots();

    while (iter.hasNext())
    {
      iter.next().initPrevSiblings();
    }
  },

  calcSpecOrderIndex : function(c)
  {
    // summary:
    //		Calculates the specified order index for each node. 
    // description:
    //		Calculates the specified order index for each node. 
    //		This is used when we have a node comparator and do not want to
    //		determine the children order from the coordinates.
    // tags:
    //		private
    
    var n = this.getNumberOfNodes();
    var nodeArray = TranslateUtil.newArray(n, null);

    for (var i = 0; i < n; i++)
    {
      nodeArray[i] = this.getNode(i);
    }

    var s = 1;

    if (this.isBalloonLayout())
    {
      switch (this.getFlowDirection())
      {
        case 0:
          s = -1;
          break;
        case 1:
          s = -1;
          break;
        case 2:
          s = 1;
          break;
        case 3:
          s = 1;
          break;
      }
    }

    if (this.isRadialLayout())
    {
      switch (this.getFlowDirection())
      {
        case 0:
          s = 1;
          break;
        case 1:
          s = -1;
          break;
        case 2:
          s = -1;
          break;
        case 3:
          s = 1;
          break;
      }
    }

    var sign = s;

    var aux = function(o1,o2) {
      return sign * c(o1.getOriginalObject(), o2.getOriginalObject());
    };
    nodeArray.sort(aux);
    for (var j = 0; j < n; j++)
    {
      nodeArray[j].setSpecChildrenOrder(j);
    }
  },

  calcNorthSizeExtension : function (a0)
  {
    // summary:
    //		Calculate the size extension at the north side of each node.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcNorthSizeExtension_overload_0();
    else if(arguments.length == 1)
      return this.calcNorthSizeExtension_overload_1(a0);
  },
  
  calcNorthSizeExtension_overload_0 : function()
                                                
  {
    var iter = this.getNodes();

    while (iter.hasNext())
    {
      this.calcNorthSizeExtension(iter.next());
    }
  },

  calcNorthSizeExtension_overload_1 : function(node)
  {
    // summary:
    //		Calculate the size extension at the north side. 
    // description:
    //		Calculate the size extension at the north side. 
    //		The size extension is used to shift the node away from its parent if
    //		we have a subtree block margin at the north side.
    // tags:
    //		private
    
    var val = node.getSubtreeBlockMargin(3);
    var child = node.getEastWestNeighbor(0);
    if (child != null) {
      val = Math.max(val, child.getSubtreeBlockMargin(3));
    }
    child = node.getEastWestNeighbor(1);
    if (child != null) {
      val = Math.max(val, child.getSubtreeBlockMargin(3));
    }
    if (val > 0)
    {
      node.setNorthSizeExtension(val);
      child = node.getEastWestNeighbor(0);
      if (child != null) {
        child.setNorthSizeExtension(val);
      }
      child = node.getEastWestNeighbor(1);
      if (child != null) {
        child.setNorthSizeExtension(val);
      }
    }
  },

  calcPrelimCoordInDirTowardsLeaves : function (a0, a1)
  {
    // summary:
    //		Calculates preliminary coordinates in direction towards leaves. 
    // description:
    //		Calculates preliminary coordinates in direction towards leaves. 
    //		This is actually only needed for level layout.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcPrelimCoordInDirTowardsLeaves_overload_0();
    else if(arguments.length == 2)
      return this.calcPrelimCoordInDirTowardsLeaves_overload_2(a0, a1);
  },
  
  calcPrelimCoordInDirTowardsLeaves_overload_0 : function()
                                                          
  {
    if (this.isLevelLayout())
    {
      this._percCompleteController.startStep(this._percentages.percPrelimCoordToLeaves, this.getNumberOfNodes(), false);
      this._levelSize = TranslateUtil.newArray(this._maxDepth + 1, 0);
      this._levelCoord = TranslateUtil.newArray(this._maxDepth + 1, 0);

      var iter = this.getRoots();

      while (iter.hasNext())
        this.calcPrelimCoordInDirTowardsLeaves(iter.next(), 0);
    }
  },

  calcFinalCoordInDirTowardsLeaves : function (a0, a1)
  {
    // summary:
    //		Calculates final coordinates in direction towards leaves. 
    // description:
    //		Calculates final coordinates in direction towards leaves. 
    //		This is only necessary for leveled layout.
    // tags:
    //		private
    
    if(arguments.length == 0)
      return this.calcFinalCoordInDirTowardsLeaves_overload_0();
    else if(arguments.length == 2)
      return this.calcFinalCoordInDirTowardsLeaves_overload_2(a0, a1);
  },
  
  calcFinalCoordInDirTowardsLeaves_overload_0 : function()
                                                          
  {
    if (this.isLevelLayout())
    {
      this._percCompleteController.startStep(this._percentages.percFinalCoordToLeaves, this.getNumberOfNodes(), false);

      var dir = this.getDirTowardsLeaves();
      var i;
      var coord = 0.0;
      var size = 0.0;

      for (i = 0; i < this._levelSize.length; i++)
      {
        size = this._levelSize[i];
        this._levelCoord[i] = coord;
        if (this._levelJustification == 0) {
          this._levelCoord[i] += 0.5 * size;
        }
        else if (this._levelJustification == 1) {
          this._levelCoord[i] += size;
        }
        coord += size + this._minDistBetweenNodes[dir];
      }

      for (i = 0; i < this._firstFreeNodeSlot && this._nodes[i].getImmediateParent() == null; i++)
        this.calcFinalCoordInDirTowardsLeaves(this._nodes[i], 0);

      this._levelSize = null;
      this._levelCoord = null;
    }
  },

  calcPrelimCoordInDirTowardsLeaves_overload_2 : function(node, depth)
  {
    // summary:
    //		Calculates the preliminary coordinate of this node and its
    //		descendent for leveled layout in edge flow direction.
    // tags:
    //		private
    
    var child;
    var dir = this.getDirTowardsLeaves();

    this._percCompleteController.addPoints(1);
    this.layoutStepPerformed();
    var size = node.getSize(dir) + node.getNorthSizeExtension();
    if (size > this._levelSize[depth]) {
      this._levelSize[depth] = size;
    }
    node.setVirtualCenter(1 - dir, 0.0);
    node.setMaxCoord(dir, depth * (this._maxNodeSize + 10.0));

    for (var i = 0; i < node.getNumberOfChildren(); i++)
    {
      child = node.getChild(i);
      if (node.hasEastWestNeighbor(child)) {
        this.calcPrelimCoordInDirTowardsLeaves(child, depth);
      }
      else {
        this.calcPrelimCoordInDirTowardsLeaves(child, depth + 1);
      }
    }
  },

  calcFinalCoordInDirTowardsLeaves_overload_2 : function(node, depth)
  {
    // summary:
    //		Calculates the final coordinate of this node and its descendent for
    //		leveled layout in edge flow direction.
    // tags:
    //		private
    
    var child;
    var dir = this.getDirTowardsLeaves();
    var orthFactor = this.getOrthForkFactor();

    this._percCompleteController.addPoints(1);
    this.layoutStepPerformed();
    var levelCoord = ((depth > 0 ? this._levelCoord[depth - 1] : 0));
    var levelSize = ((depth > 0 ? this._levelSize[depth - 1] : 0));

    switch (this._levelJustification)
    {
      case -1:
        node.setMinCoord(dir, this._levelCoord[depth] + node.getNorthSizeExtension());
        node.setOrthSplitCoord(dir, levelCoord + levelSize + orthFactor * this.getMinDistBetweenNodes(dir));
        break;
      case 0:
        var center = this._levelCoord[depth] + 0.5 * node.getNorthSizeExtension();
        node.setVirtualCenter(dir, center);
        node.setOrthSplitCoord(dir, levelCoord + 0.5 * levelSize + orthFactor * this.getMinDistBetweenNodes(dir));
        break;
      case 1:
        node.setMaxCoord(dir, this._levelCoord[depth]);
        node.setOrthSplitCoord(dir, levelCoord + orthFactor * this.getMinDistBetweenNodes(dir));
        break;
    }
    for (var i = 0; i < node.getNumberOfChildren(); i++)
    {
      child = node.getChild(i);
      if (node.hasEastWestNeighbor(child)) {
        this.calcFinalCoordInDirTowardsLeaves(child, depth);
      }
      else {
        this.calcFinalCoordInDirTowardsLeaves(child, depth + 1);
      }
    }
  },

  calcCoords : function()
  {
    // summary:
    //		Calculates the remaining coordinates. 
    // description:
    //		Calculates the remaining coordinates. 
    //		For the leveled layout, it only calculates the coordinate in
    //		direction towards siblings. For unleved and tip-over layout, it
    //		calculates all coordinates. For a leveled tree, this is O(n). For a
    //		nonleveled tree where node sizes differ, we are not sure about the
    //		complexity (Juutistenaho claims it is still O(n) without tip-over,
    //		but the proof is IMHO wrong).
    // tags:
    //		private
    
    var i;
    var numNodes = this.getNumberOfNodes();

    this._percCompleteController.startStep(this._percentages.percRelCoord, numNodes, false);

    for (i = 0; i < this._firstFreeNodeSlot && this._nodes[i].getImmediateParent() == null; i++)
      this.calcSubtreeCoords(this._nodes[i]);

    var coordBase = 0.0;
    var sdir = this.getDirTowardsSiblings();
    var bounds = TranslateUtil.newArray(2, 0);

    if (!this.isRadialLayout())
    {
      var numRoots = 0;

      for (i = 0; i < this._firstFreeNodeSlot && this._nodes[i].getImmediateParent() == null; i++)
        numRoots++;

      this._percCompleteController.startStep(this._percentages.percAbsCoord, (numRoots + 3) * numNodes, false);

      for (i = 0; i < this._firstFreeNodeSlot && this._nodes[i].getImmediateParent() == null; i++)
      {
        bounds[0] = /*Number.MAX_VALUE*/(3.4028235E38);
        bounds[1] = -/*Number.MAX_VALUE*/(3.4028235E38);
        this.calcFinalCoordNormal(this._nodes[i], 0.0, 0.0, bounds);
        if (sdir == 0) {
          this.shiftSubtreeBy(this._nodes[i], coordBase - bounds[0], 0.0);
        }
        else {
          this.shiftSubtreeBy(this._nodes[i], 0.0, coordBase - bounds[0]);
        }
        coordBase += bounds[1] - bounds[0] + 2.0 * this._minDistBetweenBranches[sdir];

        this._percCompleteController.addPoints(numNodes);
        this.layoutStepPerformed();
      }
    }
  },

  calcSubtreeCoords : function(root)
  {
    // summary:
    //		Calculates the relative coordinates for the subtree starting at
    //		root.  
    // tags:
    //		private
    
    var borders = this.calcCoordsAndGetBorders(root, 0, root.isTipOver());

    for (var i = 0; i < 4; i++)
    {
      if (borders[i] != null)
      {
        borders[i].disposeNextUntil(null);
        borders[i].dispose();
      }
    }
  },

  calcCoordsAndGetBorders : function(root, rootYCoord, needTopBottom)
  {
    // summary:
    //		Calculates the relative coordinates for the subtree starting at
    //		root. 
    // description:
    //		Calculates the relative coordinates for the subtree starting at
    //		root. 
    //		Returns the borders, left, right, top, bottom.
    // tags:
    //		private
    
    var sdir = this.getDirTowardsSiblings();
    var ldir = this.getDirTowardsLeaves();
    var leftBorder;
    var rightBorder;
    var topBorder;
    var bottomBorder;

    var firstNode = root;

    while (firstNode.getEastWestNeighbor(0) != null)
    {
      firstNode = firstNode.getEastWestNeighbor(0);
    }

    var oldSibling = firstNode.getPrevSibling();
    firstNode.setPrevSibling(null);

    leftBorder = new TPath(root, true, sdir);
    rightBorder = new TPath(root, false, sdir);

    if (needTopBottom)
    {
      topBorder = new TPath(root, true, ldir);
      bottomBorder = new TPath(root, false, ldir);
    }
    else
    {
      topBorder = null;
      bottomBorder = null;
    }

    this.initBorderCell(topBorder);
    this.initBorderCell(bottomBorder);
    this.initBorderCell(leftBorder);
    this.initBorderCell(rightBorder);
    this.calcPrelimCoord(ldir, null, root, leftBorder, rightBorder, topBorder, bottomBorder, 0, rootYCoord);

    firstNode.setPrevSibling(oldSibling);
    return [leftBorder, rightBorder, topBorder, bottomBorder ]; // Array
  },

  calcFinalCoordRadial : function()
  {
    // summary:
    //		Calls the polar transformation for radial layout for each tree in
    //		the graph.
    // tags:
    //		private
    
    var coordBase = 0.0;
    var sdir = this.getDirTowardsSiblings();

    if (this.isRadialLayout() && this.getNumberOfNodes() > 1)
    {
      var f = ((this.allowAlternating() ? 3 : 1));

      this._percCompleteController.startStep(this._percentages.percRadial, 5 * this.getNumberOfNodes() + f * this.getMaxDepth(), false);

      this._radialTransAlg.setEllipsoidTransTable(this._angleTranslationTable);
      var iter = this.getRoots();

      while (iter.hasNext())
        coordBase = this._radialTransAlg.run(iter.next(), coordBase, 0.0) + 2.0 * this._minDistBetweenBranches[sdir];
    }
  },

  setAngleTranslationTable : function(table)
  {
    // summary:
    //		Sets the angle translation table.
    // tags:
    //		private
    
    this._angleTranslationTable = table;
  },

  calcPrelimCoord : function(flowDir, virtualParent, node, parentLeftBorder, parentRightBorder, parentTopBorder, parentBottomBorder, depth, rootYCoord)
  {
    // summary:
    //		Calculate the preliminary coordinate of this node and all descendent
    //		nodes. 
    // description:
    //		Calculate the preliminary coordinate of this node and all descendent
    //		nodes. 
    //		Note that the real coordinate corresponds to the coordinate of the
    //		node plus the sum of the subtree coordinate corrections of all
    //		ancestors of the node.
    // tags:
    //		private
    
    this._percCompleteController.addPoints(1);

    var parent = node.getImmediateParent();
    var child;
    var oppositeFlowDir = 1 - flowDir;
    var minDist = this.getMinDistBetweenNodes(flowDir);

    var parentTipOverIsValid = (parent == virtualParent && virtualParent != null);

    var leftBorder;
    var rightBorder;
    var topBorder;
    var bottomBorder;

    if (parentTipOverIsValid && parent.isTipOver())
    {
      var yCoord = virtualParent.getMaxCoord(flowDir) + minDist + node.getNorthSizeExtension();
      var borders = this.calcCoordsAndGetBorders(node, yCoord, true);
      leftBorder = borders[0].cutPath(borders[0]);
      rightBorder = borders[1].cutPath(borders[1]);
      topBorder = borders[2].cutPath(borders[2]);
      bottomBorder = borders[3].cutPath(borders[3]);

      var shiftDist = 0;
      var isTipOverEast = virtualParent.isTipOverEast();
      var isTipOverWest = virtualParent.isTipOverWest();
      var topBorderTailOnTipOverEastWest = null;
      var bottomBorderTailOnTipOverEastWest = null;

      if (virtualParent.isTipOverEastWest())
      {
        var maxEast = parentBottomBorder.getMaxCoord(flowDir, false);
        var maxWest = parentBottomBorder.getMaxCoord(flowDir, true);

        if (this.getFlowDirection() == 0 || this.getFlowDirection() == 3)
        {
          isTipOverEast = maxEast < maxWest;
        }
        else
        {
          isTipOverEast = maxEast <= maxWest;
        }

        isTipOverWest = !isTipOverEast;
        if (isTipOverWest) {
          node.markTipOverEastWestChildAtWest();
        }
        topBorderTailOnTipOverEastWest = parentTopBorder.cutForEastWest(oppositeFlowDir, isTipOverWest);
        bottomBorderTailOnTipOverEastWest = parentBottomBorder.cutForEastWest(oppositeFlowDir, isTipOverWest);
      }

      if (isTipOverEast)
      {
        this.adjustTopBorderForTipOver(node, topBorder, flowDir, AlignmentOptions.TIP_OVER);

        shiftDist = -Math.min(topBorder.getStartCoord(oppositeFlowDir), bottomBorder.getStartCoord(oppositeFlowDir)) + this.getMinDistBetweenNodes(oppositeFlowDir);
      }
      else if (isTipOverWest)
      {
        this.adjustTopBorderForTipOver(node, topBorder, flowDir, AlignmentOptions.TIP_OVER_WEST);

        shiftDist = -Math.max(topBorder.getLastCellEndCoord(oppositeFlowDir), bottomBorder.getLastCellEndCoord(oppositeFlowDir)) - this.getMinDistBetweenNodes(oppositeFlowDir);
      }

      this.shiftSubtreeAndBorder(node, leftBorder, rightBorder, topBorder, bottomBorder, true, oppositeFlowDir, shiftDist);

      if (isTipOverWest)
      {
        parentTopBorder = this.reverseBorder(parentTopBorder, oppositeFlowDir, false);
        parentBottomBorder = this.reverseBorder(parentBottomBorder, oppositeFlowDir, false);
        topBorder = this.reverseBorder(topBorder, oppositeFlowDir, true);
        bottomBorder = this.reverseBorder(bottomBorder, oppositeFlowDir, true);
        this.shiftSubtreeAway(flowDir, virtualParent, node, parentTopBorder, parentBottomBorder, topBorder, bottomBorder, leftBorder, rightBorder, depth);
        parentTopBorder = this.reverseBorder(parentTopBorder, oppositeFlowDir, false);
        parentBottomBorder = this.reverseBorder(parentBottomBorder, oppositeFlowDir, false);
        this.updateOtherBorder(flowDir, parentLeftBorder, leftBorder, false);
        this.updateOtherBorder(flowDir, parentRightBorder, rightBorder, true);
      }
      else if (isTipOverEast)
      {
        this.shiftSubtreeAway(flowDir, virtualParent, node, parentTopBorder, parentBottomBorder, topBorder, bottomBorder, leftBorder, rightBorder, depth);
        this.updateOtherBorder(flowDir, parentLeftBorder, leftBorder, false);
        this.updateOtherBorder(flowDir, parentRightBorder, rightBorder, true);
      }

      var ewneighbor = node.getEastWestNeighbor(0);
      var dist;

      while (ewneighbor != null)
      {
        dist = node.getMinCoord(flowDir) - ewneighbor.getMinCoord(flowDir);
        ewneighbor.shiftCoord(flowDir, dist);
        ewneighbor.shiftSubtreeCoordCorrection(flowDir, dist);
        ewneighbor = ewneighbor.getEastWestNeighbor(0);
      }

      ewneighbor = node.getEastWestNeighbor(1);

      while (ewneighbor != null)
      {
        dist = node.getMinCoord(flowDir) - ewneighbor.getMinCoord(flowDir);
        ewneighbor.shiftCoord(flowDir, dist);
        ewneighbor.shiftSubtreeCoordCorrection(flowDir, dist);
        ewneighbor = ewneighbor.getEastWestNeighbor(1);
      }

      if (topBorderTailOnTipOverEastWest != null) {
        parentTopBorder.combineForEastWest(topBorderTailOnTipOverEastWest, isTipOverWest);
      }
      if (bottomBorderTailOnTipOverEastWest != null) {
        parentBottomBorder.combineForEastWest(bottomBorderTailOnTipOverEastWest, isTipOverWest);
      }
      return;
    }

    child = node.getEastWestNeighbor(0);

    if (child != null)
    {
      this.calcPrelimCoord(flowDir, virtualParent, child, parentLeftBorder, parentRightBorder, parentTopBorder, parentBottomBorder, depth, rootYCoord);
    }

    var nrOfChildren = node.getNumberOfChildren();

    leftBorder = new TPath(node, true, oppositeFlowDir);
    rightBorder = new TPath(node, false, oppositeFlowDir);
    topBorder = null;
    bottomBorder = null;

    if (node.isTipOver() || parentTopBorder != null)
    {
      topBorder = new TPath(node, true, flowDir);
      bottomBorder = new TPath(node, false, flowDir);
    }

    this.initBorderCell(topBorder);
    this.initBorderCell(bottomBorder);
    this.initBorderCell(leftBorder);
    this.initBorderCell(rightBorder);

    if (!this.isLevelLayout())
    {
      if (virtualParent != null)
      {
        var mdist = minDist;

        if (virtualParent.hasAssociateDummy(node) || virtualParent.hasAssociate(node))
        {
          mdist = this.getMinDistBetweenAssocs(flowDir);
        }

        node.setMinCoord(flowDir, virtualParent.getMaxCoord(flowDir) + mdist + node.getNorthSizeExtension());
      }
      else
      {
        node.setMinCoord(flowDir, rootYCoord);
      }

      node.setOrthSplitCoord(oppositeFlowDir, node.getLinkVirtualCenter(oppositeFlowDir, false));
      node.setOrthSplitCoord(flowDir, node.getMinCoord(flowDir) - node.getNorthSizeExtension() - (1 - this.getOrthForkFactor()) * minDist);
    }

    for (var i = 0; i < nrOfChildren; i++)
    {
      child = node.getChild(i);

      if (!node.hasEastWestNeighbor(child))
      {
        this.calcPrelimCoord(flowDir, node, child, leftBorder, rightBorder, topBorder, bottomBorder, depth + 1, rootYCoord);
      }
    }

    var j;
    var k;
    child = node.getLastVirtualChild();

    while (child != null)
    {
      for (j = 0; j < 2; j++)
        for (k = 0; k < 2; k++)
          if (child.getFirstVisibleAtBorder(j, k) != null)
          {
            child.getFirstVisibleAtBorder(j, k).setSubtreeRoot(null);
            child.setFirstVisibleAtBorder(j, k, null);
          }

      child = child.getPrevSibling();
    }

    var pos = this.getPositionFromChildren(node);
    node.setLinkVirtualCenter(oppositeFlowDir, false, pos);
    node.setSubtreeCoordCorrection(flowDir, 0.0);
    node.setSubtreeCoordCorrection(oppositeFlowDir, 0.0);

    this.updateBordersWithOwnCoord(node, flowDir, leftBorder, rightBorder, topBorder, bottomBorder);

    node.shiftCoord(oppositeFlowDir, -pos);
    node.shiftSubtreeCoordCorrection(oppositeFlowDir, -pos);
    this.shiftBorders(leftBorder, rightBorder, topBorder, bottomBorder, oppositeFlowDir, -pos);

    this.shiftSubtreeAway(oppositeFlowDir, virtualParent, node, parentLeftBorder, parentRightBorder, leftBorder, rightBorder, topBorder, bottomBorder, depth);

    if (parentTopBorder != null)
    {
      this.updateOtherBorder(oppositeFlowDir, parentTopBorder, topBorder, false);
      this.updateOtherBorder(oppositeFlowDir, parentBottomBorder, bottomBorder, true);
    }

    child = node.getEastWestNeighbor(1);

    if (child != null)
    {
      this.calcPrelimCoord(flowDir, virtualParent, child, parentLeftBorder, parentRightBorder, parentTopBorder, parentBottomBorder, depth, rootYCoord);
    }
  },

  initBorderCell : function(border)
  {
    // summary:
    //		Initialize the borders with negative infinite. 
    // description:
    //		Initialize the borders with negative infinite. 
    //		As long as the corresponding node of the border has no corrdinate,
    //		it ensures the border cell stays at the head of the border path,
    //		since whenever we append cells to the border, these cells will have
    //		real coordinates that are larger than negative infinite. This is
    //		important when shifting subtrees away (in opposite shift direction),
    //		since we assume for the parent border that the first cell is always
    //		the cell of the node itself.
    // tags:
    //		private
    
    if (border == null) {
      return;
    }
    var cmin = -/*Number.MAX_VALUE*/(3.4028235E38);
    border.setStartCoord(0, cmin);
    border.setEndCoord(0, cmin);
    border.setStartCoord(1, cmin);
    border.setEndCoord(1, cmin);
  },

  shiftSubtreeAway : function(shiftDir, virtualParent, node, parentLeftBorder, parentRightBorder, nodeLeftBorder, nodeRightBorder, nodeTopBorder, nodeBottomBorder, depth)
  {
    // summary:
    //		Shifts the subtree starting at this node in shift direction in order
    //		to avoid overlapping with its siblings. 
    // description:
    //		Shifts the subtree starting at this node in shift direction in order
    //		to avoid overlapping with its siblings. 
    //		It also updates the border of the parent. The conflict is between
    //		the border of the visited part of the parent at the side towards the
    //		larger coordinates (i.e. parentRightBorder) and the border of the
    //		node at the side towards the smaller coordinates (nodeLeftBorder).
    //		For simplicity of naming, we assume that the flow direction is top
    //		down, so that we have to compare the right border of the visited
    //		part of the parent with the left border of the child. Note that for
    //		this purpose, the subtree starting at the node is not yet condidered
    //		visited. Even though the naming is for flow direction to down
    //		without tip-over, this still works for tip-over, or flow direction
    //		left to right. Just leftBorder is not quite LEFT, but TOP in this
    //		case.
    // tags:
    //		private
    
    var prevSibling = node.getPrevSibling();
    var isTipOver = virtualParent != null && virtualParent.isTipOver();
    var xdir = shiftDir;
    var ydir = 1 - shiftDir;
    var minDist1 = this._minDistBetweenNodes[xdir];
    var minDist2 = this._minDistBetweenBranches[xdir];

    if (prevSibling != null)
    {
      if (prevSibling.isAssociate() || node.isAssociate()) {
        minDist1 = this.getMinDistBetweenAssocs(xdir);
      }

      var nodeCoordCorrSum = TranslateUtil.newArray(2, 0);
      var parentCoordCorrSum = TranslateUtil.newArray(2, 0);
      nodeCoordCorrSum[0] = 0.0;
      nodeCoordCorrSum[1] = 0.0;
      parentCoordCorrSum[0] = 0.0;
      parentCoordCorrSum[1] = 0.0;
      var nonLeafCorr = this.getOverlapFactor() * this.getMinDistBetweenNodes(ydir);

      if (isTipOver)
      {
        nonLeafCorr = 0.0;
        if (prevSibling.getNumberOfChildren() > 0) {
          minDist1 = this._minDistBetweenBranches[xdir];
        }
      }

      var minDist = minDist1;
      var leftCell = nodeLeftBorder;
      var rightCell = parentRightBorder.getNext();
      var subtreeRootOfLeft = null;
      var subtreeRootOfRight = null;
      var leftXPos = 0.0;
      var rightXPos = 0.0;
      var leftYPos = 0.0;
      var rightYPos = 0.0;

      if (leftCell != null)
      {
        subtreeRootOfLeft = leftCell.getNode();
        leftXPos = leftCell.getEndCoord(xdir);
        leftYPos = leftCell.getEndCoord(ydir);
        if (this.isLevelLayout() == false && leftCell.getNode().getNumberOfChildren() > 0) {
          leftYPos += nonLeafCorr;
        }
        nodeCoordCorrSum[0] += leftCell.getCoordCorrection(0);
        nodeCoordCorrSum[1] += leftCell.getCoordCorrection(1);
      }

      if (rightCell != null)
      {
        subtreeRootOfRight = rightCell.getNode();
        rightXPos = rightCell.getEndCoord(xdir);
        rightYPos = rightCell.getEndCoord(ydir);
        if (this.isLevelLayout() == false && rightCell.getNode().getNumberOfChildren() > 0) {
          rightYPos += nonLeafCorr;
        }
        parentCoordCorrSum[0] += rightCell.getCoordCorrection(0);
        parentCoordCorrSum[1] += rightCell.getCoordCorrection(1);
      }

      var needNextLeft;
      var needNextRight;
      var nrLeftChildren;
      var nrRightChildren;

      while (leftCell != null && rightCell != null)
      {
        if (this.isRadialLayout() && this._desiredMinDist != null)
        {
          if (depth < this._maxDepth + 1) {
            minDist = this._desiredMinDist[depth];
          }
          else {
            minDist = this._minDistBetweenNodes[xdir];
          }
        }

        if (this.isRadialLayout() && depth < 5)
        {
          minDist1 = minDist;
          nrLeftChildren = leftCell.getNode().getNumberOfChildren();
          nrRightChildren = rightCell.getNode().getNumberOfChildren();

          if (nrLeftChildren == 0 && nrRightChildren > 2)
          {
            minDist1 = rightCell.getNext().getEndCoord(xdir) + parentCoordCorrSum[xdir];
            minDist1 -= rightXPos;
          }

          if (nrLeftChildren > 2 && nrRightChildren == 0)
          {
            minDist1 = leftXPos;
            minDist1 -= leftCell.getNext().getEndCoord(xdir) + nodeCoordCorrSum[xdir];
          }

          if (minDist1 > minDist) {
            minDist = minDist1;
          }
        }

        if (rightXPos + minDist - leftXPos > 0)
        {
          this.layoutStepPerformed();

          var shiftDist = rightXPos + minDist - leftXPos;

          var count = 0;
          var tempnode1 = node;
          var tempnode2 = node.getPrevSibling();

          if (!node.isEastWestNeighbor(1) && !isTipOver)
          {
            while (tempnode1 != null && tempnode2 != null && tempnode1 != subtreeRootOfRight)
            {
              if (!tempnode1.hasEastWestNeighbor(tempnode2) && !tempnode2.hasEastWestNeighbor(tempnode1)) {
                count++;
              }
              tempnode1 = tempnode2;
              tempnode2 = tempnode2.getPrevSibling();
            }
          }

          if (count != 0)
          {
            var i = count;
            tempnode1 = node;
            tempnode2 = node.getPrevSibling();

            while (tempnode1 != null && tempnode2 != null && tempnode1 != subtreeRootOfRight)
            {
              var d = shiftDist * i / count;

              tempnode1.shiftCoord(xdir, d);

              tempnode1.shiftSubtreeCoordCorrection(xdir, d);
              if (!tempnode1.hasEastWestNeighbor(tempnode2) && !tempnode2.hasEastWestNeighbor(tempnode1)) {
                i--;
              }
              tempnode1 = tempnode2;
              tempnode2 = tempnode2.getPrevSibling();
            }
          }
          else
          {
            node.shiftCoord(xdir, shiftDist);
            node.shiftSubtreeCoordCorrection(xdir, shiftDist);
          }

          this.shiftBorders(nodeLeftBorder, nodeRightBorder, nodeTopBorder, nodeBottomBorder, xdir, shiftDist);

          nodeCoordCorrSum[xdir] += shiftDist;
          leftXPos += shiftDist;
        }

        minDist = minDist2;

        if (leftYPos < rightYPos)
        {
          needNextLeft = true;
          needNextRight = false;
        }
        else if (leftYPos > rightYPos)
        {
          needNextLeft = false;
          needNextRight = true;
          depth++;
        }
        else
        {
          needNextLeft = true;
          needNextRight = true;
          depth++;
        }

        if (needNextLeft)
        {
          leftCell = leftCell.getNext();

          if (leftCell != null)
          {
            if (leftCell.getSubtreeRoot() != null) {
              subtreeRootOfLeft = leftCell.getSubtreeRoot();
            }
            leftXPos = leftCell.getEndCoord(xdir) + nodeCoordCorrSum[xdir];
            leftYPos = leftCell.getEndCoord(ydir) + nodeCoordCorrSum[ydir];
            if (this.isLevelLayout() == false && leftCell.getNode().getNumberOfChildren() > 0) {
              leftYPos += nonLeafCorr;
            }
            nodeCoordCorrSum[0] += leftCell.getCoordCorrection(0);
            nodeCoordCorrSum[1] += leftCell.getCoordCorrection(1);
          }
        }

        if (needNextRight)
        {
          rightCell = rightCell.getNext();

          if (rightCell != null)
          {
            if (rightCell.getSubtreeRoot() != null) {
              subtreeRootOfRight = rightCell.getSubtreeRoot();
            }
            rightXPos = rightCell.getEndCoord(xdir) + parentCoordCorrSum[xdir];
            rightYPos = rightCell.getEndCoord(ydir) + parentCoordCorrSum[ydir];
            if (this.isLevelLayout() == false && rightCell.getNode().getNumberOfChildren() > 0) {
              rightYPos += nonLeafCorr;
            }
            parentCoordCorrSum[0] += rightCell.getCoordCorrection(0);
            parentCoordCorrSum[1] += rightCell.getCoordCorrection(1);
          }
        }
      }

      if (leftCell != null)
      {
        var h = parentLeftBorder.cutPath(parentLeftBorder);
        parentLeftBorder.append(nodeLeftBorder, nodeLeftBorder.getLast());
        nodeLeftBorder = h;

        this.addPrefixBorder(xdir, 0, parentLeftBorder, nodeLeftBorder, leftCell, subtreeRootOfLeft);

        parentRightBorder.disposeNextUntil(null);
        parentRightBorder.append(nodeRightBorder, nodeRightBorder.getLast());
      }
      else if (rightCell != null)
      {
        this.addPrefixBorder(xdir, 1, parentRightBorder, nodeRightBorder, rightCell, subtreeRootOfRight);
      }
      else
      {
        parentRightBorder.disposeNextUntil(null);
        parentRightBorder.append(nodeRightBorder, nodeRightBorder.getLast());
      }
    }
    else
    {
      parentLeftBorder.append(nodeLeftBorder, nodeLeftBorder.getLast());
      parentRightBorder.append(nodeRightBorder, nodeRightBorder.getLast());
    }
  },

  alignBorder : function(border, xdir, maximize)
  {
    // summary:
    //		Maximize or minize the border. 
    // description:
    //		Maximize or minize the border. 
    //		This creates waste of space.
    // tags:
    //		private
    
    this.eliminateCoordinateCoorection(border);
    var best;
    var cell = border;

    if (maximize)
    {
      best = -/*Number.MAX_VALUE*/(3.4028235E38);

      while (cell != null)
      {
        best = Math.max(best, cell.getStartCoord(xdir));
        best = Math.max(best, cell.getEndCoord(xdir));
        cell = cell.getNext();
      }
    }
    else
    {
      best = /*Number.MAX_VALUE*/(3.4028235E38);

      while (cell != null)
      {
        best = Math.min(best, cell.getStartCoord(xdir));
        best = Math.min(best, cell.getEndCoord(xdir));
        cell = cell.getNext();
      }
    }

    cell = border;

    while (cell != null)
    {
      cell.setStartCoord(xdir, best);
      cell.setEndCoord(xdir, best);
      cell = cell.getNext();
    }
  },

  enlargeBorder : function(border, ydir, startDelta, endDelta)
  {
    // summary:
    //		Endlarge the border in ydir at the beginning by startDelta and at
    //		the end by endDelta.
    // tags:
    //		private
    
    if (border == null) {
      return;
    }
    if (startDelta >= 0) {
      border.setStartCoord(ydir, border.getStartCoord(ydir) - startDelta);
    }
    if (endDelta <= 0) {
      return;
    }
    var cell = border;

    while (cell.getNext() != null)
    {
      cell = cell.getNext();
    }

    cell.setEndCoord(ydir, cell.getEndCoord(ydir) + endDelta);
  },

  updateOtherBorder : function (a0, a1, a2, a3, a4)
  {
    // summary:
    //		Updates the parent border with the node border. 
    // description:
    //		Updates the parent border with the node border. 
    //		The input border direction is the direction where the border
    //		coordinates are monotonuos increasing. Again, for simplicity of
    //		naming, we assume the flow goes top down, then this is the top and
    //		bottom border, which are monotonuos increasing in x-direction.
    //		However, it works of course if the flow goes from left to right.
    //		This does not shift any border anymore. It returns the resulting
    //		parent border path.
    // tags:
    //		private
    
    if(arguments.length == 4)
      return this.updateOtherBorder_overload_4(a0, a1, a2, a3);
    else if(arguments.length == 5)
      return this.updateOtherBorder_overload_5(a0, a1, a2, a3, a4);
  },
  
  updateOtherBorder_overload_4 : function(borderDir, parentBorder, nodeBorder, keepHigher)
                                                                                          
  {
    this.updateOtherBorder(borderDir, parentBorder, nodeBorder, keepHigher, true);
  },

  updateOtherBorder_overload_5 : function(borderDir, parentBorder, nodeBorder, keepHigher, needToEliminateCoordCorr

)
  {
    var xdir = borderDir;
    var ydir = 1 - borderDir;
    var path1 = nodeBorder;
    var path2 = parentBorder;
    var cell1;
    var cell2;

    if (needToEliminateCoordCorr)
    {
      this.eliminateCoordinateCoorection(path1);
      this.eliminateCoordinateCoorection(path2);
    }

    if (path1 == null) {
      return;
    }

    var x1 = path1.getStartCoord(xdir);
    var y1 = path1.getStartCoord(ydir);
    var x2 = path2.getStartCoord(xdir);
    var y2 = path2.getStartCoord(ydir);

    if (x1 < x2)
    {
      path1.swap(path2);
      x1 = path1.getStartCoord(xdir);
      y1 = path1.getStartCoord(ydir);
      x2 = path2.getStartCoord(xdir);
      y2 = path2.getStartCoord(ydir);
    }

    cell2 = path2;
    x2 = path2.getEndCoord(xdir);
    y2 = path2.getEndCoord(ydir);

    while (cell2 != null && x2 < x1)
    {
      cell2 = cell2.getNext();

      if (cell2 != null)
      {
        x2 = cell2.getEndCoord(xdir);
        y2 = cell2.getEndCoord(ydir);
      }
    }

    if (y2 == -/*Number.MAX_VALUE*/(3.4028235E38) && cell2 != null)
    {
      if (cell2.getNext() != null) {
        y2 = cell2.getNext().getStartCoord(ydir);
      }
    }

    if (cell2 != null)
    {
      var lastVisible = cell2;
      cell1 = path1;
      x1 = path1.getEndCoord(xdir);
      y1 = path1.getEndCoord(ydir);
      var path2Remains = (y2 > y1) && keepHigher || (y2 < y1) && !keepHigher;

      while (cell1 != null && cell2 != null)
      {
        if (x1 < x2)
        {
          cell1 = cell1.getNext();

          if (cell1 != null)
          {
            x1 = cell1.getEndCoord(xdir);
            y1 = cell1.getEndCoord(ydir);
          }
        }
        else if (x1 > x2)
        {
          cell2 = cell2.getNext();

          if (cell2 != null)
          {
            x2 = cell2.getEndCoord(xdir);
            y2 = cell2.getEndCoord(ydir);
          }
        }
        else
        {
          cell1 = cell1.getNext();

          if (cell1 != null)
          {
            x1 = cell1.getEndCoord(xdir);
            y1 = cell1.getEndCoord(ydir);
          }

          cell2 = cell2.getNext();

          if (cell2 != null)
          {
            x2 = cell2.getEndCoord(xdir);
            y2 = cell2.getEndCoord(ydir);
          }
        }
      }

      if (path2Remains == false)
      {
        var tail = path2.cutPath(lastVisible);

        if (cell2 == lastVisible)
        {
          cell2 = new TPath(lastVisible.getNode(), false, 0);
          cell2.setStartCoord(0, lastVisible.getStartCoord(0));
          cell2.setStartCoord(1, lastVisible.getStartCoord(1));
          cell2.setEndCoord(0, lastVisible.getEndCoord(0));
          cell2.setEndCoord(1, lastVisible.getEndCoord(1));
          if (tail != null) {
            cell2.append(tail, tail.getLast());
          }
          tail = cell2;
        }

        lastVisible.setEndCoord(xdir, path1.getStartCoord(xdir));
        path2.append(path1, path1.getLast());

        if (cell1 == null && cell2 != null)
        {
          cell2.setStartCoord(xdir, path2.getLast().getEndCoord(xdir));
          path2.append(cell2, tail.getLast());
        }

        if (tail != null && tail != cell2)
        {
          tail.disposeNextUntil(cell2);
          tail.dispose();
        }
      }
      else
      {
        if (cell1 != null)
        {
          cell1.setStartCoord(xdir, path2.getLast().getEndCoord(xdir));
          path2.append(cell1, path1.getLast());
          path1.disposeNextUntil(cell1);
        }

        if (path1 != cell1) {
          path1.dispose();
        }
      }
    }
    else
    {
      if (path2.getEndCoord(xdir) != -/*Number.MAX_VALUE*/(3.4028235E38)) {
        path1.setStartCoord(xdir, path2.getLast().getEndCoord(xdir));
      }
      else {
        path2.getLast().setEndCoord(xdir, path1.getStartCoord(xdir));
      }
      path2.append(path1, path1.getLast());
    }
  },

  eliminateCoordinateCoorection : function(border)
  {
    // summary:
    //		Eliminate the coordinate correction at a border.
    // tags:
    //		private
    
    if (border != null) {
      border.eliminateCoordinateCoorection();
    }
  },

  reverseBorder : function(border, dir, includeRoot)
  {
    // summary:
    //		Reverses a border. 
    // description:
    //		Reverses a border. 
    //		Also reverses the coordinates in given dir. Returns the reverse
    //		path. If includeRoot is true, the path is completely reversed. If
    //		includeRoot is false, the first cell is kept and only the remaining
    //		is reversed.
    // tags:
    //		private
    
    if (border == null) {
      return null; // ibm_ilog.graphlayout.tree.TPath
    }
    this.eliminateCoordinateCoorection(border);

    if (includeRoot)
    {
      return border.reverse(dir); // ibm_ilog.graphlayout.tree.TPath
    }
    else
    {
      var tail = border.cutPath(border);

      if (tail != null)
      {
        tail = tail.reverse(dir);
        border.append(tail, tail.getLast());
      }

      return border; // ibm_ilog.graphlayout.tree.TPath
    }
  },

  updateBordersWithOwnCoord : function(node, flowDir, leftBorder, rightBorder, topBorder, bottomBorder)
  {
    // summary:
    //		As soon as we know the coordinate of this node, we can update the
    //		borders of this node to include this node correctly. 
    // description:
    //		As soon as we know the coordinate of this node, we can update the
    //		borders of this node to include this node correctly. 
    //		Before, we temporarily assumed that the node gets placed top left of
    //		its first children (which
    //		doesn't need to be true anymore now). This assumption guaranteed that the border cell of the node is the first cell of each border.
    // tags:
    //		private
    
    var xdir = 1 - flowDir;
    var ydir = flowDir;

    var firstChild = node.getFirstRealChild();
    var maxChildCoordInYDir = node.getMaxCoord(ydir);

    if (firstChild != null)
    {
      if (node.isTipOverEastWest())
      {
        var iter = node.getRealChildren();

        while (iter.hasNext())
        {
          var child = iter.next();
          maxChildCoordInYDir = Math.max(maxChildCoordInYDir, child.getMaxCoord(ydir));
        }
      }
      else
      {
        maxChildCoordInYDir = node.getLastRealChild().getMaxCoord(ydir);
      }
    }

    var eastMargin = node.getSubtreeBlockMargin(0);
    var southMargin = node.getSubtreeBlockMargin(1);
    var westMargin = node.getSubtreeBlockMargin(2);
    var northMargin = node.getSubtreeBlockMargin(3);
    var tail;

    if (leftBorder != null)
    {
      if (leftBorder.getNext() == null || node.isTipOverEast() == false && node.isTipOverEastWest() == false && node.getMaxCoord(ydir) <= leftBorder.getEndCoord(ydir))
      {
        leftBorder.update(true, xdir);
      }
      else
      {
        this.eliminateCoordinateCoorection(leftBorder);
        tail = leftBorder.cutPath(leftBorder);
        leftBorder.update(true, xdir);

        if (node.isTipOverEast() || node.isTipOverEastWest())
        {
          leftBorder.setEndCoord(ydir, maxChildCoordInYDir);
        }

        this.updateOtherBorder(ydir, leftBorder, tail, false, false);
      }

      if (westMargin >= 0)
      {
        leftBorder.shiftCoord(xdir, -westMargin);
        leftBorder.shiftCoordCorrection(xdir, -westMargin);
        this.alignBorder(leftBorder, xdir, false);
      }

      this.enlargeBorder(leftBorder, ydir, northMargin, southMargin);
    }

    if (rightBorder != null)
    {
      if (rightBorder.getNext() == null || node.isTipOverWest() == false && node.isTipOverEastWest() == false && node.getMaxCoord(ydir) <= rightBorder.getEndCoord(ydir))
      {
        rightBorder.update(false, xdir);
      }
      else
      {
        this.eliminateCoordinateCoorection(rightBorder);
        tail = rightBorder.cutPath(rightBorder);
        rightBorder.update(false, xdir);

        if (node.isTipOverWest() || node.isTipOverEastWest())
        {
          rightBorder.setEndCoord(ydir, maxChildCoordInYDir);
        }

        this.updateOtherBorder(ydir, rightBorder, tail, true, false);
      }

      if (eastMargin >= 0)
      {
        rightBorder.shiftCoord(xdir, eastMargin);
        rightBorder.shiftCoordCorrection(xdir, eastMargin);
        this.alignBorder(rightBorder, xdir, true);
      }

      this.enlargeBorder(rightBorder, ydir, northMargin, southMargin);
    }

    if (topBorder != null)
    {
      if (topBorder.getNext() == null || node.isTipOver() == false && node.getMaxCoord(xdir) <= topBorder.getEndCoord(xdir))
      {
        topBorder.update(true, ydir);
      }
      else
      {
        this.eliminateCoordinateCoorection(topBorder);
        tail = topBorder.cutPath(topBorder);
        topBorder.update(true, ydir);
        this.updateOtherBorder(xdir, topBorder, tail, false, false);
      }

      if (northMargin >= 0)
      {
        topBorder.shiftCoord(ydir, -northMargin);
        topBorder.shiftCoordCorrection(ydir, -northMargin);
        this.alignBorder(topBorder, ydir, false);
      }

      this.enlargeBorder(topBorder, xdir, westMargin, eastMargin);
    }

    if (bottomBorder != null)
    {
      if (bottomBorder.getNext() == null || node.isTipOver() == false && node.getMaxCoord(xdir) <= bottomBorder.getEndCoord(xdir))
      {
        bottomBorder.update(false, ydir);
      }
      else
      {
        this.eliminateCoordinateCoorection(bottomBorder);
        tail = bottomBorder.cutPath(bottomBorder);
        bottomBorder.update(false, ydir);

        if (node.isTipOver())
        {
          bottomBorder.setEndCoord(ydir, maxChildCoordInYDir);
          bottomBorder.setStartCoord(ydir, maxChildCoordInYDir);
        }

        this.updateOtherBorder(xdir, bottomBorder, tail, true, false);
      }

      if (southMargin >= 0)
      {
        bottomBorder.shiftCoord(ydir, southMargin);
        bottomBorder.shiftCoordCorrection(ydir, southMargin);
        this.alignBorder(bottomBorder, ydir, true);
      }

      this.enlargeBorder(bottomBorder, xdir, westMargin, eastMargin);
    }
  },

  adjustTopBorderForTipOver : function(node, topBorder, flowDir, parentTipOver)
  {
    // summary:
    //		Adjusts the top border for tip over.
    // tags:
    //		private
    
    if (topBorder == null) {
      return;
    }
    this.eliminateCoordinateCoorection(topBorder);

    var xdir = 1 - flowDir;
    var ydir = flowDir;

    var miny = topBorder.getStartCoord(ydir);
    var cell = topBorder;
    var prevCell = null;
    var prevCutCell = null;

    while (cell != null)
    {
      if (cell.getStartCoord(ydir) < miny)
      {
        miny = cell.getStartCoord(ydir);
        prevCutCell = prevCell;
      }

      prevCell = cell;
      cell = cell.getNext();
    }

    var minx = topBorder.getStartCoord(xdir);
    var maxx = topBorder.getLastCellEndCoord(xdir);

    var cutCell;
    var head;

    if (prevCutCell != null)
    {
      head = topBorder;
      cutCell = topBorder.cutPath(prevCutCell);
    }
    else
    {
      head = null;
      cutCell = topBorder;
    }

    var tail = cutCell.cutPath(cutCell);

    var coord;

    switch (parentTipOver)
    {
      case AlignmentOptions.TIP_OVER_WEST:
        if (cutCell.getEndCoord(xdir) < maxx) {
          cutCell.setEndCoord(xdir, maxx);
        }
        while (node.getEastWestNeighbor(0) != null)
          node = node.getEastWestNeighbor(0);
        coord = node.getMinCoord(xdir);
        if (cutCell.getStartCoord(xdir) > coord) {
          cutCell.setStartCoord(xdir, coord);
        }

        var firstChild = node.getFirstRealChild();

        if (!node.isTipOver() && firstChild != null)
        {
          coord = firstChild.getMinCoord(xdir);
          if (cutCell.getStartCoord(xdir) > coord) {
            cutCell.setStartCoord(xdir, coord);
          }
        }

        break;
      case AlignmentOptions.TIP_OVER:
        if (cutCell.getStartCoord(xdir) > minx) {
          cutCell.setStartCoord(xdir, minx);
        }
        while (node.getEastWestNeighbor(1) != null)
          node = node.getEastWestNeighbor(1);
        coord = node.getMaxCoord(xdir);
        if (cutCell.getEndCoord(xdir) < coord) {
          cutCell.setEndCoord(xdir, coord);
        }

        var lastChild = node.getLastRealChild();

        if (!node.isTipOver() && lastChild != null)
        {
          coord = lastChild.getMaxCoord(xdir);
          if (cutCell.getEndCoord(xdir) < coord) {
            cutCell.setEndCoord(xdir, coord);
          }
        }

        break;
    }
    if (tail != null) {
      this.updateOtherBorder(xdir, cutCell, tail, false, false);
    }
    if (head != null) {
      this.updateOtherBorder(xdir, head, cutCell, false, false);
    }
  },

  getPositionFromChildren : function(node)
  {
    // summary:
    //		Finds the
    //		node's coordinate relative to the coordinate of its children. 
    // description:
    //		Finds the node's coordinate relative to the coordinate of its
    //		children. 
    //		Note that all children are in the same coordinate system, therefore
    //		we can ignore the coordinate correction value. The result is
    //		relative to that coordinate system.
    // tags:
    //		private
    
    var numChildren = node.getNumberOfChildren();
    var dir = this.getDirTowardsSiblings();

    if (numChildren == 0) {
      return (node.getSize(dir) * 0.5); // Number
    }
    else
    {
      var firstChild = node.getFirstRealChild();
      var lastChild = node.getLastRealChild();
      if (firstChild == null) {
        return (node.getSize(dir) * 0.5); // Number
      }
      switch (node.getAlignmentStyle())
      {
        case AlignmentOptions.TIP_OVER_BOTH_SIDES:
        case AlignmentOptions.CENTER:
          if (firstChild == lastChild) {
            return (firstChild.getLinkVirtualCenter(dir, true)); // Number
          }
          else {
            return ((firstChild.getLinkVirtualCenter(dir, true) + lastChild.getLinkVirtualCenter(dir, true)) / 2.0); // Number
          }
        case AlignmentOptions.BORDER_CENTER:
          if (firstChild == lastChild) {
            return (firstChild.getLinkVirtualCenter(dir, true)); // Number
          }
          else {
            return ((firstChild.getMinCoord(dir) + lastChild.getMaxCoord(dir)) / 2.0); // Number
          }
        case AlignmentOptions.EAST:
          if (this.getFlowDirection() == 0 || this.getFlowDirection() == 3) {
            return (firstChild.getMinCoord(dir) + 0.5 * node.getSize(dir)); // Number
          }
          else {
            return (lastChild.getMaxCoord(dir) - 0.5 * node.getSize(dir)); // Number
          }
        case AlignmentOptions.WEST:
          if (this.getFlowDirection() == 0 || this.getFlowDirection() == 3) {
            return (lastChild.getMaxCoord(dir) - 0.5 * node.getSize(dir)); // Number
          }
          else {
            return (firstChild.getMinCoord(dir) + 0.5 * node.getSize(dir)); // Number
          }
        case AlignmentOptions.TIP_OVER:
          return 0; // Number
        case AlignmentOptions.TIP_OVER_WEST:
          return 0; // Number
        case AlignmentOptions.TIP_OVER_EAST_WEST:
          return 0; // Number
      }
    }

    return (0.0); // Number
  },

  shiftSubtreeAndBorder : function(rootOfSubtree, leftBorder, rightBorder, topBorder, bottomBorder, includeEastWestNeighbors, dir, shiftDist)
  {
    // summary:
    //		Shifts a node, its subtree and its borders by the same distance. 
    // description:
    //		Shifts a node, its subtree and its borders by the same distance. 
    //		Depending on the flag includeEastWestNeighbors, it also shifts the
    //		entire chain of east and west neighbors starting at the root. It
    //		shifts by shiftDist in direction dir.
    // tags:
    //		private
    
    rootOfSubtree.shiftCoord(dir, shiftDist);
    rootOfSubtree.shiftSubtreeCoordCorrection(dir, shiftDist);
    if (includeEastWestNeighbors) {
      this.shiftEastWestNeighbors(rootOfSubtree, dir, shiftDist);
    }
    this.shiftBorders(leftBorder, rightBorder, topBorder, bottomBorder, dir, shiftDist);
  },

  shiftEastWestNeighbors : function(node, dir, shiftDist)
  {
    // summary:
    //		Shift the east west neighbors of the node. 
    // description:
    //		Shift the east west neighbors of the node. 
    //		It does not shift the node itself.
    // tags:
    //		private
    
    var ewneighbor = node.getEastWestNeighbor(0);

    while (ewneighbor != null)
    {
      ewneighbor.shiftCoord(dir, shiftDist);
      ewneighbor.shiftSubtreeCoordCorrection(dir, shiftDist);
      ewneighbor = ewneighbor.getEastWestNeighbor(0);
    }

    ewneighbor = node.getEastWestNeighbor(1);

    while (ewneighbor != null)
    {
      ewneighbor.shiftCoord(dir, shiftDist);
      ewneighbor.shiftSubtreeCoordCorrection(dir, shiftDist);
      ewneighbor = ewneighbor.getEastWestNeighbor(1);
    }
  },

  shiftBorders : function(leftBorder, rightBorder, topBorder, bottomBorder, dir, shiftDist)
  {
    // summary:
    //		Shifts the borders of a node by shiftDist in direction dir.
    // tags:
    //		private
    
    if (leftBorder != null)
    {
      leftBorder.shiftCoord(dir, shiftDist);
      leftBorder.shiftCoordCorrection(dir, shiftDist);
    }

    if (rightBorder != null)
    {
      rightBorder.shiftCoord(dir, shiftDist);
      rightBorder.shiftCoordCorrection(dir, shiftDist);
    }

    if (topBorder != null)
    {
      topBorder.shiftCoord(dir, shiftDist);
      topBorder.shiftCoordCorrection(dir, shiftDist);
    }

    if (bottomBorder != null)
    {
      bottomBorder.shiftCoord(dir, shiftDist);
      bottomBorder.shiftCoordCorrection(dir, shiftDist);
    }
  },

  addPrefixBorder : function(dir, lowerOrHigher, parentBorder, prefixBorder, parentBorderSuffix, rootOfParentBorderSuffix)
  {
    // summary:
    //		Adds the prefix border to the parent border, so that the part
    //		between next cell of parent border and the cell
    //		'parentBorderSuffix' (exclusively) is replaced by the
    //		prefix border. 
    // description:
    //		Adds the prefix border to the parent border, so that the part
    //		between next cell of parent border and the cell
    //		<code>parentBorderSuffix</code> (exclusively) is replaced by the
    //		prefix border. 
    //		It also updates the coordinate correction value and the first
    //		visible node. In this sketch, P is the parent, SS the already
    //		handled sibling of N and N the node whose prefix border is added. In
    //		the sketch, we update the right border, but the left border would
    //		work similar. P                      P SS     N              SS N / 
    //		\   /&#95;\    =>      /   &#95;\  
    //		<- prefixBorder /    \                /    \ /&#95;&#95;&#95;&#95;&#95;&#95;\              /&#95;&#95;&#95;&#95;&#95;&#95;\   <- parentBorderSuffix
    // tags:
    //		private
    
    if (prefixBorder == null) {
      return;
    }

    var oldCoordCorrection0 = 0.0;
    var oldCoordCorrection1 = 0.0;
    var newCoordCorrection0 = 0.0;
    var newCoordCorrection1 = 0.0;
    var cell = parentBorder.getNext();

    while (cell != null && cell != parentBorderSuffix)
    {
      oldCoordCorrection0 += cell.getCoordCorrection(0);
      oldCoordCorrection1 += cell.getCoordCorrection(1);
      cell = cell.getNext();
    }

    var lastCell = null;
    cell = prefixBorder;

    while (cell != null)
    {
      newCoordCorrection0 += cell.getCoordCorrection(0);
      newCoordCorrection1 += cell.getCoordCorrection(1);
      lastCell = cell;
      cell = cell.getNext();
    }

    lastCell.shiftCoordCorrection(0, oldCoordCorrection0 - newCoordCorrection0);
    lastCell.shiftCoordCorrection(1, oldCoordCorrection1 - newCoordCorrection1);

    rootOfParentBorderSuffix.setFirstVisibleAtBorder(dir, lowerOrHigher, parentBorderSuffix);
    parentBorderSuffix.setSubtreeRoot(rootOfParentBorderSuffix);

    prefixBorder.append(parentBorderSuffix, parentBorder.getLast());
    parentBorder.disposeNextUntil(parentBorderSuffix);
    parentBorder.append(prefixBorder, prefixBorder.getLast());
  },

  calcFinalCoordNormal : function(node, shiftCoord0, shiftCoord1, bounds)
  {
    // summary:
    //		Calculates the final coordinates of the subtree starting at root. 
    // description:
    //		Calculates the final coordinates of the subtree starting at root. 
    //		The input parameters are the shift values in direction towards
    //		siblings and leaves. It fills bounds with the minimal and maximal
    //		coordinate in this subtree in direction towards siblings.
    // tags:
    //		private
    
    this._percCompleteController.addPoints(3);
    this.layoutStepPerformed();

    if (node.getImmediateParent() == null)
    {
      var neighbor = node.getEastWestNeighbor(0);

      while (neighbor != null)
      {
        this.calcFinalCoordNormal(neighbor, shiftCoord0, shiftCoord1, bounds);
        neighbor = neighbor.getEastWestNeighbor(0);
      }

      neighbor = node.getEastWestNeighbor(1);

      while (neighbor != null)
      {
        this.calcFinalCoordNormal(neighbor, shiftCoord0, shiftCoord1, bounds);
        neighbor = neighbor.getEastWestNeighbor(1);
      }
    }

    var dir0 = this.getDirTowardsSiblings();
    var dir1 = this.getDirTowardsLeaves();
    node.shiftCoord(dir0, shiftCoord0);
    node.shiftCoord(dir1, shiftCoord1);
    shiftCoord0 += node.getSubtreeCoordCorrection(dir0);
    shiftCoord1 += node.getSubtreeCoordCorrection(dir1);
    if (node.getMinCoord(dir0) < bounds[0]) {
      bounds[0] = node.getMinCoord(dir0);
    }
    if (node.getMaxCoord(dir0) > bounds[1]) {
      bounds[1] = node.getMaxCoord(dir0);
    }
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
      this.calcFinalCoordNormal(iter.next(), shiftCoord0, shiftCoord1, bounds);
  },

  shiftCoordToEqualSpacingAtRoot : function(root, bounds)
  {
    // summary:
    //		Shift the nodes of the root such that they are equally spaced. 
    // description:
    //		Shift the nodes of the root such that they are equally spaced. 
    //		This is only used in radial mode. It also update the bounds.
    // tags:
    //		private
    
    var sdir = this.getDirTowardsSiblings();
    var maxDist = 0.0;
    var firstCoord = 0.0;
    var prevCoord = 0.0;
    var lastCoord = 0.0;
    var shiftValue = 0.0;
    var iter = root.getChildren();
    var node;

    if (iter.hasNext())
    {
      firstCoord = iter.next().getCenter(sdir);
      prevCoord = firstCoord;
    }
    else {
      return;
    }
    while (iter.hasNext())
    {
      lastCoord = iter.next().getCenter(sdir);

      if (lastCoord - prevCoord > maxDist) {
        maxDist = lastCoord - prevCoord;
      }
      prevCoord = lastCoord;
    }

    if (bounds[1] - bounds[0] + firstCoord - lastCoord > maxDist) {
      maxDist = bounds[1] - bounds[0] + firstCoord - lastCoord;
    }
    iter = root.getChildren();
    if (iter.hasNext()) {
      prevCoord = iter.next().getCenter(sdir);
    }
    while (iter.hasNext())
    {
      node = iter.next();
      lastCoord = node.getCenter(sdir);
      shiftValue = prevCoord + maxDist - lastCoord;
      if (sdir == 0) {
        this.shiftSubtreeBy(node, shiftValue, 0.0);
      }
      else {
        this.shiftSubtreeBy(node, 0.0, shiftValue);
      }
      prevCoord = node.getCenter(sdir);
    }

    bounds[1] += shiftValue;
    if (bounds[1] < prevCoord + maxDist - (firstCoord - bounds[0])) {
      bounds[1] = prevCoord + maxDist - (firstCoord - bounds[0]);
    }
  },

  calcBoundingBox : function()
  {
    // summary:
    //		Calculates the bounding box of the graph.
    // tags:
    //		private
    
    var node;
    var nodeRect = new Rectangle();
    var boundingRect = null;
    var boundingRectInitialized = false;

    for (var i = 0; i < this._firstFreeNodeSlot; i++)
    {
      node = this._nodes[i];

      nodeRect.x = node.getMinCoord(0);
      nodeRect.y = node.getMinCoord(1);
      nodeRect.width = node.getSize(0);
      nodeRect.height = node.getSize(1);

      if (boundingRectInitialized)
      {
        TranslateUtil.addRect(boundingRect, nodeRect);
      }
      else
      {
        boundingRect = nodeRect;
        nodeRect = new Rectangle();
        boundingRectInitialized = true;
      }
    }

    return boundingRect; // Rectangle
  },

  setBackToOriginalSizes : function()
  {
    // summary:
    //		Sets the node sizes that might have been manipulated by radial mode
    //		or by not respecting node sizes back to the original node sizes.
    // tags:
    //		private
    
    var respectNodeSizes = this.getLayout().isRespectNodeSizes();

    for (var i = 0; i < this._firstFreeNodeSlot; i++)
      this._nodes[i].setBackToOriginalSize(respectNodeSizes);
  },

  shiftBy : function(dx, dy)
  {
    // summary:
    //		Shifts the graph coordinates by a vector <tt>(dx, dy)</tt>.
    // tags:
    //		private
    
    for (var i = 0; i < this._firstFreeNodeSlot; i++)
    {
      this._nodes[i].shiftCoord(0, dx);
      this._nodes[i].shiftCoord(1, dy);
    }
  },

  shiftSubtreeBy : function(node, dx, dy)
  {
    // summary:
    //		Shifts the subtree starting at this node by a vector <tt>(dx,
    //		dy)</tt>.  
    // tags:
    //		private
    
    node.shiftCoord(0, dx);
    node.shiftCoord(1, dy);

    for (var i = 0; i < node.getNumberOfChildren(); i++)
      this.shiftSubtreeBy(node.getChild(i), dx, dy);
  },

  mirror : function(coordinateIndex)
  {
    // summary:
    //		Mirrors the whole layout at the 0-axis of the specified coodinate
    //		index. 
    // description:
    //		Mirrors the whole layout at the 0-axis of the specified coodinate
    //		index. 
    //		If the coordinate index is 0, it mirrors x coordinates, if 1, it
    //		mirrors y coordinates.
    // tags:
    //		private
    
    for (var i = 0; i < this._firstFreeNodeSlot; i++)
      this._nodes[i].mirror(coordinateIndex);
  },

  finalPositionCorrection : function()
  {
    if (this.getNumberOfNodes() < 1) {
      return;
    }
    this._percCompleteController.startStep(this._percentages.percFinalCorr, 3, false);

    this.setBackToOriginalSizes();
    if (this.getConnectorStyle() == ConnectorStyles.EVENLY_SPACED_PINS) {
      this.correctOrthSplitCoordinates();
    }
    switch (this.getFlowDirection())
    {
      case 2:
        this.mirror(0);
        break;
      case 3:
        this.mirror(1);
        break;
    }
    
    this._percCompleteController.addPoints(1);
    this.layoutStepPerformed();

    var layout = this.getLayout();
    var point = layout.getPosition();
    var x = 0;
    var y = 0;

    if (point != null)
    {
      x = point.x;
      y = point.y;
    }

    if (layout.isRootPosition())
    {
      var root = this.getNode(0);
      if (root == this._invisibleRoot && this.getNumberOfNodes() > 1) {
        root = this.getNode(1);
      }
      var model = this.getModel();

      if (point == null)
      {
        var e = new TranslateUtil.ArrayIterator(model.getNodes());
        var obj = null;

        while (e.hasNext())
        {
          obj = e.next();
          if (root == this.getObjectNode(obj)) {
            break;
          }
        }

        if (obj != null)
        {
          var rect = GraphModelUtil.getObjectBounds(model, obj);
          x = rect.x;
          y = rect.y;
        }
      }

      this._percCompleteController.addPoints(1);
      this.layoutStepPerformed();
      this.shiftBy(x - root.getRealMinCoord(0), y - root.getRealMinCoord(1));

      this._percCompleteController.addPoints(1);
      this.layoutStepPerformed();
    }
    else
    {
      var boundingBox = this.calcBoundingBox();

      this._percCompleteController.addPoints(1);
      this.layoutStepPerformed();
      this.shiftBy(x - boundingBox.x, y - boundingBox.y);

      this._percCompleteController.addPoints(1);
      this.layoutStepPerformed();
    }
  },

  setChildIndexe : function()
  {
    // summary:
    //		Store the number which child it is in the index of each node. 
    // description:
    //		Store the number which child it is in the index of each node. 
    //		East-west neighbors
    //		don't count here. This does it for the entire graph.
    // tags:
    //		private
    
    var iter = this.getNodes();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      node.setIndex(-1);
    }

    iter = this.getNodes();

    while (iter.hasNext())
    {
      node = iter.next();
      this.setSubtreeChildIndexe(node);
    }
  },

  setSubtreeChildIndexe : function(parent)
  {
    // summary:
    //		Store the number which child it is in the index of each node. 
    // description:
    //		Store the number which child it is in the index of each node. 
    //		East-west neighbors
    //		don't count here. This does it for the subtree starting at the input node.  
    // tags:
    //		private
    
    var iter = parent.getRealChildren();
    var node;
    var i = 0;

    while (iter.hasNext())
    {
      node = iter.next();
      node.setIndex(i++);
    }
  },

  isClippedConnectorStyle : function(linkStyle, parentNode, childNode)
  {
    // summary:
    //		Returns true if we want to clip the links at the bounding rectangle.
    // tags:
    //		private
    
    if (this.isRadialLayout()) {
      return true; // Boolean
    }
    if (this.isBalloonLayout()) {
      return true; // Boolean
    }
    if (parentNode.isTipOver() || parentNode.isTipOverBothSides()) {
      return false; // Boolean
    }
    if (parentNode.hasAssociate(childNode)) {
      return false; // Boolean
    }
    if (this.getConnectorStyle() == ConnectorStyles.CLIPPED_PINS && linkStyle == LinkStyles.STRAIGHT_LINE_STYLE) {
      return true; // Boolean
    }
    return false; // Boolean
  },

  isOneBendOrthStyle : function(linkStyle, parentNode, childNode)
  {
    // summary:
    //		Returns true if we need a 1 bend orthogonal shape, like for
    //		tip-over.
    // tags:
    //		private
    
    if (this.isRadialLayout()) {
      return false; // Boolean
    }
    if (this.isBalloonLayout()) {
      return false; // Boolean
    }
    if (parentNode.isTipOverEast() && childNode.getEastWestNeighbor(0) == null) {
      return true; // Boolean
    }
    if (parentNode.isTipOverWest() && childNode.getEastWestNeighbor(1) == null) {
      return true; // Boolean
    }
    if (parentNode.isTipOverEastWest() && childNode.isTipOverEastWestChildAtEast() && childNode.getEastWestNeighbor(0) == null) {
      return true; // Boolean
    }
    if (parentNode.isTipOverEastWest() && childNode.isTipOverEastWestChildAtWest() && childNode.getEastWestNeighbor(1) == null) {
      return true; // Boolean
    }
    if (parentNode.isTipOverBothSides()) {
      return true; // Boolean
    }
    if (parentNode.hasAssociate(childNode)) {
      return true; // Boolean
    }
    return false; // Boolean
  },

  correctOrthSplitCoordinates : function()
  {
    // summary:
    //		Correct all orth split coordinates when the connector style is
    //		evenly spaced.
    // tags:
    //		private
    
    var iter = this.getNodes();

    while (iter.hasNext())
      this.correctSubtreeOrthSplitCoordinate(iter.next());
  },

  correctSubtreeOrthSplitCoordinate : function(parent)
  {
    // summary:
    //		Correct the orth split coordinates of the children of the input node
    //		when the connector style is evenly spaced.  
    // tags:
    //		private
    
    if (parent.isTipOver() || parent.isTipOverBothSides()) {
      return;
    }
    var ydir = this.getDirTowardsLeaves();
    var xdir = 1 - ydir;

    var minCoord = /*Number.MAX_VALUE*/(3.4028235E38);
    var maxCoord = -/*Number.MAX_VALUE*/(3.4028235E38);
    var numEastNodes = 0;
    var numWestNodes = 0;
    var iter = parent.getRealChildren();
    var node;

    while (iter.hasNext())
    {
      node = iter.next();
      if (node.getMinCoord(ydir) < minCoord) {
        minCoord = node.getMinCoord(ydir);
      }
      if (node.getMaxCoord(ydir) < maxCoord) {
        maxCoord = node.getMaxCoord(ydir);
      }
      if (parent.getCenter(xdir) <= node.getCenter(xdir)) {
        numEastNodes++;
      }
      else {
        numWestNodes++;
      }
    }

    var numBendLevels = Math.max(numEastNodes, numWestNodes);
    if (numBendLevels < 2) {
      return;
    }

    var actEastBend = numEastNodes - 1;
    var actWestBend = 0;
    var nextLevelCoord;
    var oldOrthSplitCoord;

    if (parent.getCenter(ydir) <= minCoord)
    {
      nextLevelCoord = minCoord;
    }
    else
    {
      nextLevelCoord = maxCoord;
    }

    iter = parent.getRealChildren();

    while (iter.hasNext())
    {
      node = iter.next();
      oldOrthSplitCoord = node.getOrthSplitCoord(ydir);
      if (parent.getCenter(xdir) <= node.getCenter(xdir)) {
        node.setOrthSplitCoord(ydir, oldOrthSplitCoord + (actEastBend--) * (nextLevelCoord - oldOrthSplitCoord) / numBendLevels);
      }
      else {
        node.setOrthSplitCoord(ydir, oldOrthSplitCoord + (actWestBend++) * (nextLevelCoord - oldOrthSplitCoord) / numBendLevels);
      }
    }
  },

  getPercController : function()
  {
    // summary:
    //		Returns the instance of the percentage complete controller.
    // tags:
    //		private
    
    return this._percCompleteController; // ibm_ilog.graphlayout.internalutil.PercCompleteController
  },

  getPercForSpanTree : function()
  {
    // summary:
    //		Returns the percentage of completion needed fo calculation the
    //		spanning tree.
    // tags:
    //		private
    
    return this._percentages.percForSpanTree; // Number
  },

  getPercSplineRoute : function()
  {
    // summary:
    //		Returns the percentage for spline routing.
    // tags:
    //		private
    
    return this._percentages.percSplineRoute; // Number
  },

  getPercRadialOrBalloon : function()
  {
    // summary:
    //		Returns the percentage for radial/balloon.
    // tags:
    //		private
    
    return this._percentages.percRadial; // Number
  },

  calcInitPercParameter : function(numSubgraphs)
  {
    // summary:
    //		Sets the initial percentage completion. 
    // description:
    //		Sets the initial percentage completion. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    this._percentages = new TPercentage();
    if (numSubgraphs > 0) {
      return this.percParameterForRecLeafLayout(numSubgraphs, 0); // int
    }

    var mode = this.getLayout().getLayoutMode();
    if (mode == AlignmentOptions.TIP_OVER || mode == TLayoutModes.TIP_LEAVES_OVER || mode == TLayoutModes.TIP_ROOTS_OVER || mode == TLayoutModes.TIP_ROOTS_AND_LEAVES_OVER) {
      return this.percParameterForAutoTipOverLayout(8); // int
    }
    if (this.isRadialLayout()) {
      return this.percParameterForRadialLayout(); // int
    }
    if (this.isBalloonLayout()) {
      return this.percParameterForBalloonLayout(); // int
    }
    if (this.isLevelLayout()) {
      return this.percParameterForLevelLayout(); // int
    }
    return this.percParameterForFreeLayout(); // int
  },

  percParameterForRecLeafLayout : function(numSubgraphs, numUnflattenedSubgraphs)
  {
    // summary:
    //		Sets the percentage completion for layout with recursive leaf mode. 
    // description:
    //		Sets the percentage completion for layout with recursive leaf mode. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    var numGraphs = numSubgraphs + 1;
    var numUFGraphs = numUnflattenedSubgraphs + 1;

    this.percParameterForFreeLayout();
    this._percentages.percForAttach /= numGraphs;
    this._percentages.percForDetachNodes /= numGraphs;
    this._percentages.percForDetachLinks /= numGraphs;
    this._percentages.percForSpanTree /= numGraphs;

    this._percentages.percPrelimCoordToLeaves /= numUFGraphs;
    this._percentages.percRelCoord /= numUFGraphs;
    this._percentages.percAbsCoord /= numUFGraphs;
    this._percentages.percFinalCoordToLeaves /= numUFGraphs;
    this._percentages.percFinalCorr /= numUFGraphs;
    this._percentages.percSplineRoute /= numUFGraphs;

    this._percentages.percRadial = 0;
    return (4 * numGraphs + 6 * numUFGraphs); // int
  },

  percParameterForFreeLayout : function()
  {
    // summary:
    //		Sets the percentage completion for layout mode free. 
    // description:
    //		Sets the percentage completion for layout mode free. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    this._percentages.percForAttach = 11.0;
    this._percentages.percForDetachNodes = 41.1;
    this._percentages.percForDetachLinks = 28.5;
    this._percentages.percForSpanTree = 3.5;
    this._percentages.percPrelimCoordToLeaves = 0.3;
    this._percentages.percRelCoord = 9.0;
    this._percentages.percAbsCoord = 1.5;
    this._percentages.percFinalCoordToLeaves = 0.3;
    this._percentages.percRadial = 0.3;
    this._percentages.percFinalCorr = 2.5;
    this._percentages.percSplineRoute = 2;
    return (11); // int
  },

  percParameterForAutoTipOverLayout : function(numberTries)
  {
    // summary:
    //		Sets the percentage completion for layout mode auto tip over. 
    // description:
    //		Sets the percentage completion for layout mode auto tip over. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    var x = this.percParameterForFreeLayout() - 2;

    var percPos = this._percentages.percRelCoord + this._percentages.percAbsCoord;
    var factor = 100.0 / (100.0 + (numberTries - 1) * percPos);
    this._percentages.percForAttach *= factor;
    this._percentages.percForDetachNodes *= factor;
    this._percentages.percForDetachLinks *= factor;
    this._percentages.percForSpanTree *= factor;
    this._percentages.percPrelimCoordToLeaves *= factor;
    this._percentages.percRelCoord *= factor;
    this._percentages.percAbsCoord *= factor;
    this._percentages.percFinalCoordToLeaves *= factor;
    this._percentages.percRadial *= factor;
    this._percentages.percFinalCorr *= factor;
    this._percentages.percSplineRoute *= factor;
    return (x + numberTries * 2); // int
  },

  changePercParameterForAutoTipOverLayout : function(numberTries)
  {
    // summary:
    //		Changes the percentage completion for layout mode auto tip over. 
    // description:
    //		Changes the percentage completion for layout mode auto tip over. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    var percRemain = 100.0 - this._percentages.percForAttach - this._percentages.percForSpanTree;

    this.percParameterForFreeLayout();

    var percPos = this._percentages.percRelCoord + this._percentages.percAbsCoord;
    var factor = percRemain / (percRemain + (numberTries - 1) * percPos);
    this._percentages.percForDetachNodes *= factor;
    this._percentages.percForDetachLinks *= factor;
    this._percentages.percForSpanTree *= factor;
    this._percentages.percPrelimCoordToLeaves *= factor;
    this._percentages.percRelCoord *= factor;
    this._percentages.percAbsCoord *= factor;
    this._percentages.percFinalCoordToLeaves *= factor;
    this._percentages.percRadial *= factor;
    this._percentages.percFinalCorr *= factor;
    this._percentages.percSplineRoute *= factor;
  },

  percParameterForLevelLayout : function()
  {
    // summary:
    //		Sets the percentage completion for layout mode level. 
    // description:
    //		Sets the percentage completion for layout mode level. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    this._percentages.percForAttach = 11.0;
    this._percentages.percForDetachNodes = 40.8;
    this._percentages.percForDetachLinks = 28.0;
    this._percentages.percForSpanTree = 3.5;
    this._percentages.percPrelimCoordToLeaves = 0.8;
    this._percentages.percRelCoord = 9.0;
    this._percentages.percAbsCoord = 1.5;
    this._percentages.percFinalCoordToLeaves = 0.7;
    this._percentages.percRadial = 0.2;
    this._percentages.percFinalCorr = 2.5;
    this._percentages.percSplineRoute = 2;

    return (11); // int
  },

  percParameterForRadialLayout : function()
  {
    // summary:
    //		Sets the percentage completion for layout mode radial. 
    // description:
    //		Sets the percentage completion for layout mode radial. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    this._percentages.percForAttach = 8.0;
    this._percentages.percForDetachNodes = 35.8;
    this._percentages.percForDetachLinks = 28.0;
    this._percentages.percForSpanTree = 3.5;
    this._percentages.percPrelimCoordToLeaves = 0.8;
    this._percentages.percRelCoord = 4.5;
    this._percentages.percAbsCoord = 0.2;
    this._percentages.percFinalCoordToLeaves = 0.7;
    this._percentages.percRadial = 14.0;
    this._percentages.percFinalCorr = 1.5;
    this._percentages.percSplineRoute = 2;

    return (11); // int
  },

  percParameterForBalloonLayout : function()
  {
    // summary:
    //		Sets the percentage completion for layout mode balloon. 
    // description:
    //		Sets the percentage completion for layout mode balloon. 
    //		It returns the number of steps needed.
    // tags:
    //		private
    
    this._percentages.percForAttach = 10;
    this._percentages.percForDetachNodes = 40;
    this._percentages.percForDetachLinks = 20;
    this._percentages.percForSpanTree = 5;
    this._percentages.percPrelimCoordToLeaves = 0;
    this._percentages.percRelCoord = 0;
    this._percentages.percAbsCoord = 0;
    this._percentages.percFinalCoordToLeaves = 0;
    this._percentages.percRadial = 20;
    this._percentages.percFinalCorr = 2.5;
    this._percentages.percSplineRoute = 2.5;

    return (7); // int
  },

  correctPercentageForTipOver : function()
  {
    // summary:
    //		Perform dummy steps just to get percentage complete value right. 
    // description:
    //		Perform dummy steps just to get percentage complete value right. 
    //		Used only in auto-tip-over.
    // tags:
    //		private
    
    this._percCompleteController.startStep(this._percentages.percRelCoord, 1, false);
    this._percCompleteController.addPoints(1);
    this._percCompleteController.startStep(this._percentages.percAbsCoord, 1, false);
    this._percCompleteController.addPoints(1);
  },

  getModel : function()
  {
    // summary:
    //		Returns the instance of the graph model class.
    // tags:
    //		private
    
    return this._graphModel; // ibm_ilog.graphlayout.AbstractGraphLayoutModel
  },

  getLayout : function()
  {
    // summary:
    //		Returns the instance of the layout class that is attached to the
    //		model that manages the nodes of this TGraph.
    // tags:
    //		private
    
    return this._graphlayout; // ibm_ilog.graphlayout.tree.TreeLayout
  },

  getRootLayout : function()
  {
    // summary:
    //		Returns the instance of the layout class that controls the layout. 
    // description:
    //		Returns the instance of the layout class that controls the layout. 
    //		If flat layouts, this is the same as <code>
    //		TGraph.getLayout()</code>, but in recursive layout, this is the top
    //		level layout that runs the code.
    // tags:
    //		private
    
    return this._rootlayout; // ibm_ilog.graphlayout.tree.TreeLayout
  },

  isLevelLayout : function()
  {
    // summary:
    //		Returns true if the layout is a leveled layout, i.e. 
    // description:
    //		Returns true if the layout is a leveled layout, i.e. 
    //		nodes that have the same depth from the root are at the same level
    //		and seperated from nodes of a different level.
    // tags:
    //		private
    
    return this._levelLayout; // Boolean
  },

  setLevelLayout : function(levelLayout)
  {
    // summary:
    //		Sets whether the layout is a leveled layout.
    // tags:
    //		private
    
    this._levelLayout = levelLayout;
  },

  isBalloonLayout : function()
  {
    // summary:
    //		Returns true if the layout is a balloon layout, i.e. 
    // description:
    //		Returns true if the layout is a balloon layout, i.e. 
    //		children are on a circle around the parent. If it is a balloon
    //		layout, it does not use the normal coordinate calculation here, but
    //		the one in TBalloon. However it does use attach and detach.
    // tags:
    //		private
    
    return this._balloonLayout; // Boolean
  },

  setBalloonLayout : function(balloonLayout)
  {
    // summary:
    //		Sets whether the layout is a balloon layout.
    // tags:
    //		private
    
    this._balloonLayout = balloonLayout;
  },

  isRadialLayout : function()
  {
    // summary:
    //		Returns true if the layout is a radial layout, i.e. 
    // description:
    //		Returns true if the layout is a radial layout, i.e. 
    //		nodes are on a circle around the root. If
    //		it's a radial layout, it is currently also always a leveled layout.
    // tags:
    //		private
    
    return this._radialLayout; // Boolean
  },

  setRadialLayout : function(radialLayout)
  {
    // summary:
    //		Sets whether the layout is a radial layout.
    // tags:
    //		private
    
    this._radialLayout = radialLayout;
  },

  allowAlternating : function()
  {
    // summary:
    //		Returns true if the layout allows alternation during radial layout. 
    // description:
    //		Returns true if the layout allows alternation during radial layout. 
    //		Alternation means that we distribute the nodes of one layer onto two
    //		circles instead of one.
    // tags:
    //		private
    
    return this._allowAlternating; // Boolean
  },

  setAllowAlternating : function(allow)
  {
    // summary:
    //		Sets whether the layout allows alternation radial layout.
    // tags:
    //		private
    
    this._allowAlternating = allow;
  },

  setFlowDirection : function(flowDirection)
  {
    // summary:
    //		Sets the edge flow direction of the layout. 
    // description:
    //		Sets the edge flow direction of the layout. 
    //		This is the direction the majority of the edges flow into. The
    //		options are: 0 for left to right, 1 for top to bottom, 2 for right
    //		to left, 3 for bottom to top.
    // tags:
    //		private
    
    this._flowDirection = flowDirection;
  },

  getFlowDirection : function()
  {
    // summary:
    //		Returns the edge flow direction of the layout.
    // tags:
    //		private
    
    return this._flowDirection; // int
  },

  getDirTowardsSiblings : function()
  {
    // summary:
    //		Returns 0 if the siblings are normally placed in horizontal manner
    //		and 1 if the siblings are normally placed vertical. 
    // description:
    //		Returns 0 if the siblings are normally placed in horizontal manner
    //		and 1 if the siblings are normally placed vertical. 
    //		This does not hold for the tip-over placement.
    // tags:
    //		private
    
    return (this._flowDirection + 1) % 2; // int
  },

  getDirTowardsLeaves : function()
  {
    // summary:
    //		Returns 0 if the flow from root to leaves is horizontal and 1 if the
    //		flow is vertical.
    // tags:
    //		private
    
    return this._flowDirection % 2; // int
  },

  setLevelJustification : function(levelJustification)
  {
    // summary:
    //		Sets level justification of the nodes within the levels. 
    // description:
    //		Sets level justification of the nodes within the levels. 
    //		The options are: -1 for justify the nodes to the border with smaller
    //		coordinate, 1 for justify the nodes to the border with larger
    //		coordinate, and 0 for justify the nodes to their centers.
    // tags:
    //		private
    
    this._levelJustification = levelJustification;
  },

  getLevelJustification : function()
  {
    // summary:
    //		Returns level justification of the nodes within the levels.
    // tags:
    //		private
    
    return this._levelJustification; // int
  },

  setMinDistBetweenNodes : function(direction, dist)
  {
    // summary:
    //		Sets the minimal distance between nodes in the given direction. 
    // description:
    //		Sets the minimal distance between nodes in the given direction. 
    //		0 means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    if (dist <= 2.0) {
      dist = 2.0;
    }
    this._minDistBetweenNodes[direction % 2] = dist;
  },

  getMinDistBetweenNodes : function(direction)
  {
    // summary:
    //		Returns the minimal distance between nodes in the given direction. 
    // description:
    //		Returns the minimal distance between nodes in the given direction. 
    //		0 means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    return this._minDistBetweenNodes[direction % 2]; // Number
  },

  setMinDistBetweenBranches : function(direction, dist)
  {
    // summary:
    //		Sets the minimal distance between different branches of the tree in
    //		the given direction. 
    // description:
    //		Sets the minimal distance between different branches of the tree in
    //		the given direction. 
    //		0 means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    if (dist <= 2.0) {
      dist = 2.0;
    }
    this._minDistBetweenBranches[direction % 2] = dist;
  },

  getMinDistBetweenBranches : function(direction)
  {
    // summary:
    //		Returns the minimal distance between different branches of the tree
    //		in the given direction. 
    // description:
    //		Returns the minimal distance between different branches of the tree
    //		in the given direction. 
    //		0 means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    return this._minDistBetweenBranches[direction % 2]; // Number
  },

  setMinDistBetweenAssocs : function(direction, dist)
  {
    // summary:
    //		Sets the minimal distance between different the tip over link line
    //		and the association node. 
    // description:
    //		Sets the minimal distance between different the tip over link line
    //		and the association node. 
    //		Association nodes are always laid out in tip-over-both sides mode. 0
    //		means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    if (dist <= 2.0) {
      dist = 2.0;
    }
    this._minDistBetweenAssocs[direction % 2] = dist;
  },

  getMinDistBetweenAssocs : function(direction)
  {
    // summary:
    //		Returns the minimal distance between different the tip over link
    //		line and the association node. 
    // description:
    //		Returns the minimal distance between different the tip over link
    //		line and the association node. 
    //		0 means in x-direction, 1 means in y-direction.
    // tags:
    //		private
    
    return this._minDistBetweenAssocs[direction % 2]; // Number
  },

  setAspectRatio : function(aspectRatio)
  {
    // summary:
    //		Sets the aspect ratio.
    // tags:
    //		private
    
    if (aspectRatio > 0.0) {
      this._aspectRatio = aspectRatio;
    }
  },

  getAspectRatio : function()
  {
    // summary:
    //		Returns the aspect ratio.
    // tags:
    //		private
    
    return this._aspectRatio; // Number
  },

  setOverlapFactor : function(factor)
  {
    // summary:
    //		Sets the overlap factor. 
    // description:
    //		Sets the overlap factor. 
    //		It should be between 0 and 1.  
    // tags:
    //		private
    
    this._overlapFactor = factor;
  },

  getOverlapFactor : function()
  {
    // summary:
    //		Returns the overlap factor.
    // tags:
    //		private
    
    return this._overlapFactor; // Number
  },

  setOrthForkFactor : function(factor)
  {
    // summary:
    //		Sets the orthogonal fork factor. 
    // description:
    //		Sets the orthogonal fork factor. 
    //		It should be between 0 and 1, and the overlap factor should always
    //		be smaller (which is however in the responsibility of the user).  
    // tags:
    //		private
    
    this._orthForkFactor = factor;
  },

  getOrthForkFactor : function()
  {
    // summary:
    //		Returns the overlap factor.
    // tags:
    //		private
    
    return this._orthForkFactor; // Number
  },

  getMaxDepth : function()
  {
    // summary:
    //		Returns the maximal depth for layered layout.
    // tags:
    //		private
    
    return this._maxDepth; // int
  },

  setDesiredMinDistTable : function(table)
  {
    // summary:
    //		Sets the table of desired min distances for layered layout.
    // tags:
    //		private
    
    this._desiredMinDist = table;
  },

  getInvisibleRoot : function()
  {
    // summary:
    //		Returns the invisible root, if using an invisible root is enabled. 
    // description:
    //		Returns the invisible root, if using an invisible root is enabled. 
    //		Otherwise it returns null.
    // tags:
    //		private
    
    return this._invisibleRoot; // ibm_ilog.graphlayout.tree.TNode
  },

  setConnectorStyle : function(style)
  {
    // summary:
    //		Sets the connector style.  
    // tags:
    //		private
    
    this._connectorStyle = style;
  },

  getConnectorStyle : function()
  {
    // summary:
    //		Returns the overlap factor.
    // tags:
    //		private
    
    return this._connectorStyle; // int
  },

  createConvertedIntergraphLinks : function()
  {
    // summary:
    //		Create all converted intergraph links.
    // tags:
    //		private
    
    if (this._graphModel.getIntergraphLinksCount() == 0) {
      return;
    }

    var nodeTable = new HashTable();

    var nestedData = NestedData.getData(this._graphModel);

    var thisGraph = nestedData.getGraphModelNode(this._graphModel);

    var e = (new TranslateUtil.ArrayIterator(this._graphModel.getIntergraphLinks()));

    while (e.hasNext())
    {
      var link = e.next();
      var fromObj = this._graphModel.getFrom(link);
      var toObj = this._graphModel.getTo(link);
      var fromNode = this.getNode(fromObj, nestedData, thisGraph, nodeTable);
      var toNode = this.getNode(toObj, nestedData, thisGraph, nodeTable);
      TGraph.addConvertedIntergraphLink(fromNode, toNode);
    }
  },

  getNode_overload_4 : function(node, nestedData, thisGraph, nodeTable)
  {
    // summary:
    //		Returns the TNode in this graph model that corresponds to the input
    //		node, or corresponds to a subgraph that has the input node as
    //		descendent if the input node is nested inside subgraphs.
    // tags:
    //		private
    
    var result = nodeTable.item(node);
    if (result != null) {
      return result; // ibm_ilog.graphlayout.tree.TNode
    }

    var parent = nestedData.getParent(node);

    if (parent == thisGraph)
    {
      result = this.getObjectNode(node);
    }
    else
    {
      result = this.getNode(parent, nestedData, thisGraph, nodeTable);
    }

    nodeTable.add(node, result);
    return result; // ibm_ilog.graphlayout.tree.TNode
  },

  addIntergraphConnectivity : function(nestedModel, subgraphMap)
  {
    // summary:
    //		Adds intergraph connectivity during recursive layout. 
    // description:
    //		Adds intergraph connectivity during recursive layout. 
    //		Same as createConvertedIntergraphLinks(), see comments there,
    //		however now we have a nested model which makes things easier.
    // tags:
    //		private
    
    nestedModel.filter = null;

    var fromObj;
    var toObj;
    var linkObj;
    var subgraph;
    var fromNode;
    var toNode;
    var commonOwner;

    var e = new TranslateUtil.ArrayIterator(nestedModel.getLinks());

    while (e.hasNext())
    {
      linkObj = e.next();

      if (nestedModel.wasIntergraphLink(linkObj))
      {
        fromObj = nestedModel.getFrom(linkObj);
        toObj = nestedModel.getTo(linkObj);
        commonOwner = nestedModel.getLowestCommonAncestor(fromObj, toObj);
        subgraph = subgraphMap.item(commonOwner);
        fromNode = subgraph.getObjectNode(nestedModel.getAncestorContainedIn(fromObj, commonOwner));
        toNode = subgraph.getObjectNode(nestedModel.getAncestorContainedIn(toObj, commonOwner));
        TGraph.addConvertedIntergraphLink(fromNode, toNode);
      }
    }

    if (this._graphlayout.isIntergraphConnectivityMode() && nestedModel.getIntergraphLinksCount() > 0)
    {
      var nestedData = NestedData.getData(nestedModel.getOriginalGraphModel());
      e = (new TranslateUtil.ArrayIterator(this._graphModel.getIntergraphLinks()));

      while (e.hasNext())
      {
        linkObj = e.next();
        fromObj = this._graphModel.getFrom(linkObj);
        toObj = this._graphModel.getTo(linkObj);

        if (nestedModel.getOriginal(fromObj) == null)
        {
          while (nestedModel.getReplacement(fromObj) == null)
            fromObj = nestedData.getParent(fromObj);
          fromObj = nestedModel.getReplacement(fromObj);
        }

        if (nestedModel.getOriginal(toObj) == null)
        {
          while (nestedModel.getReplacement(toObj) == null)
            toObj = nestedData.getParent(toObj);
          toObj = nestedModel.getReplacement(toObj);
        }

        commonOwner = nestedModel.getLowestCommonAncestor(fromObj, toObj);
        subgraph = subgraphMap.item(commonOwner);
        fromNode = subgraph.getObjectNode(nestedModel.getAncestorContainedIn(fromObj, commonOwner));
        toNode = subgraph.getObjectNode(nestedModel.getAncestorContainedIn(toObj, commonOwner));
        TGraph.addConvertedIntergraphLink(fromNode, toNode);
      }
    }
  }

}
);

lang.mixin(TGraph,
{

  flattenLeafSubtrees : function(ancestor, node)
  {
    var canFlatten = node.getImmediateParent() != null;
    var graph = node.getRepresentationGraph();
    var root = null;

    var iter = graph.getRoots();

    while (iter.hasNext() && canFlatten)
    {
      if (root == null) {
        root = iter.next();
      }
      else {
        canFlatten = false;
      }
    }

    if (root == null) {
      canFlatten = false;
    }
    if (root.getImmediateParent() != null) {
      canFlatten = false;
    }

    var i;

    if (canFlatten)
    {
      ancestor.addFlattenedNodes(node, root, graph);

      var margins = TranslateUtil.newArray(4, 0);
      margins[0] = root.getSubtreeBlockMargin(0);
      margins[1] = root.getSubtreeBlockMargin(1);
      margins[2] = root.getSubtreeBlockMargin(2);
      margins[3] = root.getSubtreeBlockMargin(3);

      for (i = 0; i < 4; i++)
      {
        if (margins[i] < 0) {
          margins[i] = 0;
        }
      }

      var nestedModel = ancestor.getModel();
      var nodeObj = node.getOriginalObject();

      switch (ancestor.getFlowDirection())
      {
        case 0:
          
          margins[0] += nestedModel.getSubgraphMargin(nodeObj, Direction.BOTTOM);
          margins[1] += nestedModel.getSubgraphMargin(nodeObj, Direction.RIGHT);
          margins[2] += nestedModel.getSubgraphMargin(nodeObj, Direction.TOP);
          margins[3] += nestedModel.getSubgraphMargin(nodeObj, Direction.LEFT);
          break;
        case 1:
          margins[0] += nestedModel.getSubgraphMargin(nodeObj, Direction.RIGHT);
          margins[1] += nestedModel.getSubgraphMargin(nodeObj, Direction.BOTTOM);
          margins[2] += nestedModel.getSubgraphMargin(nodeObj, Direction.LEFT);
          margins[3] += nestedModel.getSubgraphMargin(nodeObj, Direction.TOP);
          break;
        case 2:
          margins[0] += nestedModel.getSubgraphMargin(nodeObj, Direction.BOTTOM);
          margins[1] += nestedModel.getSubgraphMargin(nodeObj, Direction.LEFT);
          margins[2] += nestedModel.getSubgraphMargin(nodeObj, Direction.TOP);
          margins[3] += nestedModel.getSubgraphMargin(nodeObj, Direction.RIGHT);
          break;
        case 3:
          margins[0] += nestedModel.getSubgraphMargin(nodeObj, Direction.RIGHT);
          margins[1] += nestedModel.getSubgraphMargin(nodeObj, Direction.TOP);
          margins[2] += nestedModel.getSubgraphMargin(nodeObj, Direction.LEFT);
          margins[3] += nestedModel.getSubgraphMargin(nodeObj, Direction.BOTTOM);
          break;
      }
      root.setSubtreeBlockMargins(margins);
      graph.setFlattened(true);
    }

    if (!canFlatten) {
      ancestor = graph;
    }
    var n = graph.getNumberOfNodes();

    for (i = 0; i < n; i++)
    {
      var child = graph.getNode(i);
      if (child.getRepresentationGraph() == null) {
        continue;
      }
      if (child.getRealChildren().hasNext()) {
        continue;
      }
      TGraph.flattenLeafSubtrees(ancestor, child);
    }
  },

  addConvertedIntergraphLink : function(fromNode, toNode)
  {
    if (fromNode == null || toNode == null || fromNode == toNode) {
      return;
    }
    fromNode.addIntergraphConnectivityChild(toNode);
    toNode.addIntergraphConnectivityChild(fromNode);
    fromNode.incrementIntergraphConnectivityOutdegree();
    toNode.incrementIntergraphConnectivityIndegree();
  }

}
);

/**
 * @private
 */
TGraphAnonClass_1 = declare(TNodeIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  count: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.count = 0;
  },

  hasNext : function()
  {
    return (this.count < this.__outerThis._firstFreeNodeSlot); // Boolean
  },

  next : function()
  {
    return (this.__outerThis.getNode(this.count++)); // ibm_ilog.graphlayout.tree.TNode
  }

});

/**
 * @private
 */
TGraphAnonClass_2 = declare(TNodeIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  count: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.count = 0;
  },

  hasNext : function()
  {
    return (this.count < this.__outerThis._firstFreeNodeSlot && this.__outerThis._nodes[this.count].getImmediateParent() == null); // Boolean
  },

  next : function()
  {
    return (this.__outerThis.getNode(this.count++)); // ibm_ilog.graphlayout.tree.TNode
  }

});

/**
 * @private
 */
TGraphAnonClass_3 = declare(TNodeIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  count: null,
  n: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.count = 0;
    this.n = this.__outerThis._nodesThatAreSubgraphs.length;
  },

  hasNext : function()
  {
    return (this.count < this.n); // Boolean
  },

  next : function()
  {
    return (this.__outerThis._nodesThatAreSubgraphs[this.count++]); // ibm_ilog.graphlayout.tree.TNode
  }

});

TPercentage = declare( null, 
{
  // summary:
  //		The percentages for the different steps.  
  // tags:
  //		private
  
  constructor : function()
  {
    
  },

  percForAttach: null,
  percForDetachNodes: null,
  percForDetachLinks: null,
  percForSpanTree: null,
  percPrelimCoordToLeaves: null,
  percRelCoord: null,
  percAbsCoord: null,
  percFinalCoordToLeaves: null,
  percRadial: null,
  percFinalCorr: null,
  percSplineRoute: null
}
);

return TGraph;
});

},
'ibm_ilog/graphlayout/layer_tree':function(){
// wrapped by build app
define(["dijit","dojo","dojox","dojo/require!ibm_ilog/graphlayout/tree/TreeLayout"], function(dijit,dojo,dojox){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */
dojo.provide('ibm_ilog.graphlayout.layer_tree');

dojo.require('ibm_ilog.graphlayout.tree.TreeLayout');

// The following modules will be recursively included by the module above
//dojo.require('ibm_ilog.graphlayout.tree.TAngleSpacingModes');
//dojo.require('ibm_ilog.graphlayout.tree.TAutoTipOver');
//dojo.require('ibm_ilog.graphlayout.tree.TBalloon');
//dojo.require('ibm_ilog.graphlayout.tree.TBalloonRadiusModes');
//dojo.require('ibm_ilog.graphlayout.tree.TGraph');
//dojo.require('ibm_ilog.graphlayout.tree.TGraphAlgorithm');
//dojo.require('ibm_ilog.graphlayout.tree.TLayoutModes');
//dojo.require('ibm_ilog.graphlayout.tree.TLinkParameters');
//dojo.require('ibm_ilog.graphlayout.tree.TNode');
//dojo.require('ibm_ilog.graphlayout.tree.TNodeIterator');
//dojo.require('ibm_ilog.graphlayout.tree.TNodeParameters');
//dojo.require('ibm_ilog.graphlayout.tree.TPath');
//dojo.require('ibm_ilog.graphlayout.tree.TRadial');
//dojo.require('ibm_ilog.graphlayout.tree.TSpanTree');

});

},
'ibm_ilog/graphlayout/tree/TNodeParameters':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TNodeParameters", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/BaseSubgraphData",	// #6
  "../internalutil/GraphModelUtil",	// #7
  "../internalutil/LayoutUtil",	// #8
  "../internalutil/TranslateUtil"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    BaseSubgraphData,	// #6
    GraphModelUtil,	// #7
    LayoutUtil,	// #8
    TranslateUtil	// #9
    ) {

var TNodeParameters = declare("ibm_ilog.graphlayout.tree.TNodeParameters", null, 
{
  // summary:
  //		A node parameter container. 
  // description:
  //		A node parameter container. 
  //		This node parameter container is used to store all the parameters and
  //		 data we need for the layout at the real node object (not the TNode).
  //		The main objective is to reduce the number of individual properties
  //		stored at the node, to avoid that we allocate a separate property for
  //		each bit or byte to be stored. Therefore we store only this
  //		TNodeParameters object, which contains all parameters.  
  // tags:
  //		private
  
  _rootPreference: null,

  _alignment: null,

  _childAlignment: null,
  _eastNeighbor: null,
  _westNeighbor: null,
  _associateParent: null,
  _associateChildren: null,
  _isSpecRoot: null,
  _isCalcRoot: null,
  _subtreeBlockMargin: null,

  constructor : function(layout, node)
  {
    this._rootPreference = -1;

    this._alignment = AlignmentOptions.DEFAULT;
    this._childAlignment = AlignmentOptions.UNSPECIFIED;
    this._eastNeighbor = null;
    this._westNeighbor = null;
    this._isSpecRoot = false;
    this._isCalcRoot = false;
    this._subtreeBlockMargin = null;

    var model = layout.getGraphModel();

    layout = TNodeParameters.adjust(layout, model, node);

    model.setParameter(node, layout._NODE_PROPERTY, this);
  },
  
  isNeeded : function()
  {
    // summary:
    //		Returns 'true' if the parameter container is still
    //		needed, and 'false' if the parameter container can be
    //		removed from the node.
    // tags:
    //		private
    
    return (this._rootPreference >= 0 || this._alignment != AlignmentOptions.DEFAULT || this._childAlignment != AlignmentOptions.UNSPECIFIED || this._eastNeighbor != null || this._westNeighbor != null ||
        this._associateParent != null || this._associateChildren != null || this._subtreeBlockMargin != null || this._isSpecRoot || this._isCalcRoot); // Boolean
  },

  dispose : function(layout, node)
  {
    // summary:
    //		Dispose the parameter container.
    // tags:
    //		private
    
    this._eastNeighbor = null;
    this._westNeighbor = null;
    var model = layout.getGraphModel();

    layout = TNodeParameters.adjust(layout, model, node);

    model.setParameter(node, layout._NODE_PROPERTY, null);
  },

  getEastNeighbor : function()
  {
    // summary:
    //		Returns the east neighbor node of this node.
    // tags:
    //		private
    
    return this._eastNeighbor; // Object
  },

  setEastNeighbor : function(node)
  {
    // summary:
    //		Sets the east neighbor node of this node.
    // tags:
    //		private
    
    this._eastNeighbor = node;
  },

  getWestNeighbor : function()
  {
    // summary:
    //		Returns the west neighbor node of this node.
    // tags:
    //		private
    
    return this._westNeighbor; // Object
  },

  setWestNeighbor : function(node)
  {
    // summary:
    //		Sets the west neighbor node of this node.
    // tags:
    //		private
    
    this._westNeighbor = node;
  },

  getAssociateParent : function()
  {
    // summary:
    //		Returns the associate parent of this node.
    // tags:
    //		private
    
    return this._associateParent; // Object
  },

  setAssociateParent : function(node)
  {
    // summary:
    //		Sets the associate parent of this node.
    // tags:
    //		private
    
    this._associateParent = node;
  },

  getAssociates : function()
  {
    // summary:
    //		Returns the associates of this node. 
    // description:
    //		Returns the associates of this node. 
    //		All associates have this node as associate parent.
    // tags:
    //		private
    
    if (this._associateChildren == null) {
      return LayoutUtil.getVoidEnumeration(); // ibm_ilog.graphlayout.IIterator
    }
    return ( new TranslateUtil.ArrayIterator(this._associateChildren)); // ibm_ilog.graphlayout.IIterator
  },

  addAssociateChild : function(node)
  {
    // summary:
    //		Adds an associate to this node.
    // tags:
    //		private
    
    if (this._associateChildren == null) {
      this._associateChildren = new Array();
    }
    this._associateChildren.push(node);
  },

  removeAssociateChild : function(node)
  {
    // summary:
    //		Remove an associate from this node.
    // tags:
    //		private
    
    if (this._associateChildren == null) {
      return;
    }
    TranslateUtil.removeArrayObject(this._associateChildren, node);
    if (this._associateChildren.length == 0) {
      this._associateChildren = null;
    }
  },

  isSpecRoot : function()
  {
    // summary:
    //		Returns 'true' if the node is a specified root. 
    // description:
    //		Returns <code>true</code> if the node is a specified root. 
    //		In this case, it must be in the specified root vector of the
    //		<code>TreeLayout</code> instance.
    // tags:
    //		private
    
    return this._isSpecRoot; // Boolean
  },

  setSpecRoot : function(flag)
  {
    // summary:
    //		Sets whether the node is a specified root.
    // tags:
    //		private
    
    this._isSpecRoot = flag;
  },

  isCalcRoot : function()
  {
    // summary:
    //		Returns 'true' if the node is a calculated root. 
    // description:
    //		Returns <code>true</code> if the node is a calculated root. 
    //		In this case, it must be in the calculated root vector of the
    //		<code>TreeLayout</code> instance.
    // tags:
    //		private
    
    return this._isCalcRoot; // Boolean
  },

  setCalcRoot : function(flag)
  {
    // summary:
    //		Sets whether the node is a calculated root.
    // tags:
    //		private
    
    this._isCalcRoot = flag;
  },

  getSubtreeBlockMargins : function()
  {
    // summary:
    //		Returns the subtree block margins.
    // tags:
    //		private
    
    return this._subtreeBlockMargin; // Array
  },

  getSubtreeBlockMargin : function(index)
  {
    // summary:
    //		Returns the subtree block margin at a given side.
    // tags:
    //		private
    
    if (this._subtreeBlockMargin == null) {
      return -1; // Number
    }
    return this._subtreeBlockMargin[index]; // Number
  },

  setSubtreeBlockMargin : function(index, margin)
  {
    // summary:
    //		Sets the subtree block margin at a given side. 
    // description:
    //		Sets the subtree block margin at a given side. 
    //		0 = east, 1 = south, 2 = west, 3 = north.
    // tags:
    //		private
    
    if (this._subtreeBlockMargin == null)
    {
      if (margin == -1) {
        return;
      }
      this._subtreeBlockMargin = TranslateUtil.newArray(4, 0);
      this._subtreeBlockMargin[0] = -1;
      this._subtreeBlockMargin[1] = -1;
      this._subtreeBlockMargin[2] = -1;
      this._subtreeBlockMargin[3] = -1;
    }

    this._subtreeBlockMargin[index] = margin;

    if (margin == -1 && this._subtreeBlockMargin[0] == -1 && this._subtreeBlockMargin[1] == -1 && this._subtreeBlockMargin[2] == -1 && this._subtreeBlockMargin[3] == -1)
    {
      this._subtreeBlockMargin = null;
    }
  }

}
);

lang.mixin(TNodeParameters,
{

  beforeLayout : function(layout, nodeObj, node)
  {
    var prop = TNodeParameters.get(layout, nodeObj);
    if (prop != null) {
      node.setSubtreeBlockMargins(prop.getSubtreeBlockMargins());
    }
  },

  getRootPreference : function(layout, node)
  {
    if (false) {
      return -1; // int
    }
    var prop = TNodeParameters.get(layout, node);
    if (prop != null) {
      return prop._rootPreference; // int
    }
    return -1; // int
  },

  setRootPreference : function(layout, node, value)
  {
    var prop = TNodeParameters.getOrAllocate(layout, node);
    var oldPref;

    oldPref = prop._rootPreference;

    prop._rootPreference = value;

    if (!prop.isNeeded()) {
      prop.dispose(layout, node);
    }
    if (oldPref != value) {
      layout.onObjectParameterChanged(node, "RootPreference");
    }
  },

  getAlignment : function(layout, node)
  {
    if (false) {
      return AlignmentOptions.DEFAULT; // int
    }
    var prop = TNodeParameters.get(layout, node);
    if (prop != null) {
      return prop._alignment; // int
    }
    return AlignmentOptions.DEFAULT; // int
  },

  setAlignment : function(layout, node, value)
  {
    var prop = TNodeParameters.getOrAllocate(layout, node);

    var oldAlignment;

    oldAlignment = prop._alignment;

    prop._alignment = value;

    if (!prop.isNeeded()) {
      prop.dispose(layout, node);
    }
    if (oldAlignment != value) {
      layout.onObjectParameterChanged(node, "Alignment");
    }
  },

  getChildAlignment : function(layout, node)
  {
    if (false) {
      return AlignmentOptions.UNSPECIFIED; // int
    }
    var prop = TNodeParameters.get(layout, node);
    if (prop != null) {
      return prop._childAlignment; // int
    }
    return AlignmentOptions.UNSPECIFIED; // int
  },

  setChildAlignment : function(layout, node, value)
  {
    var prop = TNodeParameters.getOrAllocate(layout, node);

    var oldAlignment;

    oldAlignment = prop._childAlignment;

    prop._childAlignment = value;

    if (!prop.isNeeded()) {
      prop.dispose(layout, node);
    }
    if (oldAlignment != value) {
      layout.onObjectParameterChanged(node, "ChildAlignment");
    }
  },

  getSubtreeBlockMargin : function(layout, node, index)
  {
    if (false) {
      return -1; // Number
    }
    var prop = TNodeParameters.get(layout, node);
    if (prop != null) {
      return prop.getSubtreeBlockMargin(index); // Number
    }
    return -1; // Number
  },

  setSubtreeBlockMargin : function(layout, node, index, value)
  {
    var prop = TNodeParameters.getOrAllocate(layout, node);
    var oldMargin;

    oldMargin = prop.getSubtreeBlockMargin(index);

    prop.setSubtreeBlockMargin(index, value);

    if (!prop.isNeeded()) {
      prop.dispose(layout, node);
    }
    if (oldMargin != value)
    {
      var side = "";

      switch (index)
      {
        case 0:
          side = "East";
          break;
        case 1:
          side = "South";
          break;
        case 2:
          side = "West";
          break;
        case 3:
          side = "North";
          break;
      }
      layout.onObjectParameterChanged(node, side + "SubtreeBlockMargin");
    }
  },

  get : function(layout, node)
  {
    var model = layout.getGraphModel();

    if (model == null) {
      return null; // ibm_ilog.graphlayout.tree.TNodeParameters
    }
    else
    {
      layout = TNodeParameters.adjust(layout, model, node);

      return model.getParameter(node, layout._NODE_PROPERTY); // ibm_ilog.graphlayout.tree.TNodeParameters
    }
  },

  getOrAllocate : function(layout, node)
  {
    var nodeParameters = TNodeParameters.get(layout, node);
    if (nodeParameters == null) {
      nodeParameters = new TNodeParameters(layout, node);
    }
    return nodeParameters; // ibm_ilog.graphlayout.tree.TNodeParameters
  },

  dispose : function(layout, node)
  {
    var prop = TNodeParameters.get(layout, node);
    if (prop != null) {
      prop.dispose(layout, node);
    }
  },

  adjust : function(layout, model, node)
  {
    if (!layout.isRootLayoutInsideRecursiveLayoutTraversal()) {
      return layout; // ibm_ilog.graphlayout.tree.TreeLayout
    }
    if (!(model instanceof BaseSubgraphData)) {
      return layout; // ibm_ilog.graphlayout.tree.TreeLayout
    }

    var nestedModel = model;
    var omodel = nestedModel.getOriginalGraphModel(node);

    if (omodel == nestedModel.getOriginalGraphModel()) {
      return layout; // ibm_ilog.graphlayout.tree.TreeLayout
    }
    return layout.getTreatedSubgraphLayout(omodel, true); // ibm_ilog.graphlayout.tree.TreeLayout
  }

}
);

return TNodeParameters;
});

},
'ibm_ilog/graphlayout/tree/TSpanTree':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TSpanTree", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/QuickSort",	// #7
  "../internalutil/TranslateUtil",	// #8
  "../tree/TGraphAlgorithm"	// #9
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    GraphModelUtil,	// #6
    QuickSort,	// #7
    TranslateUtil,	// #8
    TGraphAlgorithm	// #9
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var TGraphAlgorithm = ibm_ilog.graphlayout.tree.TGraphAlgorithm;
=====*/

var TSpanTree = declare("ibm_ilog.graphlayout.tree.TSpanTree", TGraphAlgorithm, 
{
  // summary:
  //		This algorithm class contains the mechanism to calculate the spanning
  //		tree, if the graph is not a tree. 
  // description:
  //		This algorithm class contains the mechanism to calculate the spanning
  //		tree, if the graph is not a tree. 
  //		Note that at the beginning, we
  //		don't know which are the roots and which the leaves, and which are parent and which are children. Therefore, each link from A to B is represented by a child entry B in node A and as a child entry A in node B, even though it may later turn out that either one (or even both) are not in the spanning tree. Further note that there are east-west neighbors, which are stored as normal children as well as in the east-west-neighbor field. During calculation of the spanning tree, these east-west neighbors are high priority. The task is: 1) figure out what are the roots of each component (Note: we can work with disconnected components. The roots are placed in the node array as the first nodes. 2) what is the spanning tree. Remove all child entries of links that are not part of the spanning tree. Reverse all child entries of links that are part of the spanning tree in reverse direction. 3) what is the order of the children (i.e. which is the first child and which the last child). We derive the order from the coordinates, to support incremental stablility.  
  // tags:
  //		private
  
  _firstUnvisitedIndex: null,

  _intergraphLinkConnectivity: null,

  constructor : function(graph, checkIntergraphLinks)
  {
    this._firstUnvisitedIndex = 0;
    this._intergraphLinkConnectivity = checkIntergraphLinks;
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.inherited("disposeIt", arguments, []);
  },

  run : function()
  {
    // summary:
    //		Calculates the children-parents relationship of the nodes. 
    // description:
    //		Calculates the children-parents relationship of the nodes. 
    //		By a DFS, it basically calculates a spanning tree of the graph.
    // tags:
    //		private
    
    var i;
    var graph = this.getGraph();
    var numOfNodes = graph.getNumberOfNodes();
    var node;

    this.startStep(graph.getPercForSpanTree(), 7 * numOfNodes);

    for (i = 0; i < numOfNodes; i++)
    {
      node = graph.getNode(i);
      node.calcRootNodePreference();
      node.setIndex(i);
      node.sortChildren(graph, true);
      this.addPercPoints(1);
      this.layoutStepPerformed();
    }

    this._firstUnvisitedIndex = 0;
    var invisibleRoot = graph.getInvisibleRoot();
    if (invisibleRoot != null) {
      this._firstUnvisitedIndex = 1;
    }
    node = this.searchRootNode();
    var numRoots = 0;

    while (node != null)
    {
      numRoots++;

      while (node != null)
      {
        this.calcSpanningTree(node);
        node = this.searchIntergraphConnectivityContinuation();
      }

      node = this.searchRootNode();
    }

    this.resortRootNodes();

    if (invisibleRoot != null)
    {
      if (numRoots > 1)
      {
        invisibleRoot.setChildrenCapacity(numRoots);

        for (i = 1; i < numOfNodes && graph.getNode(i).getImmediateParent() == null; i++)
        {
          node = graph.getNode(i);
          invisibleRoot.addChild(node);
          node.setOriginalParent(invisibleRoot);
        }
      }
      else if (numOfNodes > 1)
      {
        graph.swapPosition(invisibleRoot, 1);
      }

      invisibleRoot.setImmediateParent(null);
    }

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();

    var isRadial = graph.isRadialLayout();

    for (i = 0; i < numOfNodes && graph.getNode(i).getImmediateParent() == null; i++)
    {
      node = graph.getNode(i);

      if (isRadial)
      {
        node.setEastWestNeighbor(0, null);
        node.setEastWestNeighbor(1, null);
      }

      node.validateEastWestNeighbor(isRadial);
      node.validateAssociates(isRadial);
    }

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();

    this.rearrangeTipOverBothSides();
    this.layoutStepPerformed();
    numOfNodes = graph.getNumberOfNodes();

    for (i = 0; i < numOfNodes; i++)
    {
      graph.getNode(i).sortChildren(graph, false);

      this.addPercPoints(1);
      this.layoutStepPerformed();
    }

    graph.initSiblingChains();

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();

    graph.calcMaxDepth();

    this.addPercPoints(numOfNodes);
    this.layoutStepPerformed();
  },

  runRecursive : function()
  {
    // summary:
    //		Calculates the children-parents relationship of the nodes. 
    // description:
    //		Calculates the children-parents relationship of the nodes. 
    //		Same as run, but this does it for all nested TGraphs.
    // tags:
    //		private
    
    this.run();
    var iter = this.getGraph().getNodesThatAreSubgraphs();

    while (iter.hasNext())
    {
      var subgraphNode = iter.next();
      var subgraph = subgraphNode.getRepresentationGraph();
      if (subgraph == null) {
        continue;
      }
      var alg = new TSpanTree(subgraph, true);
      alg.runRecursive();
      alg.disposeIt();
    }
  },

  searchRootNode : function()
  {
    // summary:
    //		Searches the best unvisited root node. 
    // description:
    //		Searches the best unvisited root node. 
    //		It returns null if all nodes were visited.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var numberOfNodes = graph.getNumberOfNodes();
    var bestRootPreference = 0;
    var bestNode = null;
    var node;

    for (var i = this._firstUnvisitedIndex; i < numberOfNodes; i++)
    {
      node = graph.getNode(i);

      if (bestNode == null || node.getRootNodePreference() > bestRootPreference)
      {
        bestNode = node;
        bestRootPreference = node.getRootNodePreference();
      }
    }

    return bestNode; // ibm_ilog.graphlayout.tree.TNode
  },

  searchIntergraphConnectivityContinuation : function()
  {
    // summary:
    //		In intergraph connectivity mode, search a node that is connected to
    //		the already visited part with an intergraph connectivity. 
    // description:
    //		In intergraph connectivity mode, search a node that is connected to
    //		the already visited part with an intergraph connectivity. 
    //		Add this intergraph connectivity as child.
    // tags:
    //		private
    
    if (!this._intergraphLinkConnectivity) {
      return null; // ibm_ilog.graphlayout.tree.TNode
    }
    var graph = this.getGraph();
    var numberOfNodes = graph.getNumberOfNodes();
    var bestRootPreference = 0;
    var bestNode = null;
    var bestNodeVisitedParent = null;
    var node;
    var visitedParent;

    for (var i = 0; i < this._firstUnvisitedIndex; i++)
    {
      visitedParent = graph.getNode(i);
      var n = visitedParent.getNumberOfIntergraphConnectivityChildren();

      for (var j = 0; j < n; j++)
      {
        node = visitedParent.getIntergraphConnectivityChild(j);

        if (node.isVisited(this._firstUnvisitedIndex))
        {
          visitedParent.removeIntergraphConnectivityChild(node);
          node.removeIntergraphConnectivityChild(visitedParent);
          j--;
          n = visitedParent.getNumberOfIntergraphConnectivityChildren();
        }
        else
        {
          if (bestNode == null || node.getRootNodePreference() > bestRootPreference)
          {
            bestNode = node;
            bestNodeVisitedParent = visitedParent;
            bestRootPreference = node.getRootNodePreference();
          }
        }
      }
    }

    if (bestNode != null)
    {
      bestNodeVisitedParent.addChild(bestNode);
      bestNode.setOriginalParent(bestNodeVisitedParent);
      bestNodeVisitedParent.removeIntergraphConnectivityChild(bestNode);
      bestNode.removeIntergraphConnectivityChild(bestNodeVisitedParent);
    }

    return bestNode; // ibm_ilog.graphlayout.tree.TNode
  },

  calcSpanningTree : function(root)
  {
    // summary:
    //		Calculates the spanning tree by a BFS starting at the input root
    //		node.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var actIndex = this._firstUnvisitedIndex;
    var i;
    var node;
    var child;

    graph.swapPosition(root, this._firstUnvisitedIndex++);

    while (actIndex < this._firstUnvisitedIndex)
    {
      node = graph.getNode(actIndex++);

      this.addPercPoints(1);
      this.layoutStepPerformed();

      for (i = 0; i < node.getNumberOfChildren(); i++)
      {
        child = node.getChild(i);

        if (!child.isVisited(this._firstUnvisitedIndex))
        {
          child.setOriginalParent(node);
          graph.swapPosition(child, this._firstUnvisitedIndex++);
        }
        else
        {
          node.removeChild(child);
          i--;
        }
      }
    }
  },

  resortRootNodes : function()
  {
    // summary:
    //		Make sure that all the root nodes are first in the nodes array. 
    // description:
    //		Make sure that all the root nodes are first in the nodes array. 
    //		By that, we have later faster access to the root nodes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    var numberOfNodes = graph.getNumberOfNodes();
    var firstNonRootNodeIndex = 0;
    var node;
    var i = 0;

    for (i = 0; i < numberOfNodes; i++)
    {
      node = graph.getNode(i);
      node.cleanIntergraphConnectivityChildren();
      if (node.getImmediateParent() == null) {
        graph.swapPosition(node, firstNonRootNodeIndex++);
      }
    }

    var invisibleRoot = graph.getInvisibleRoot();
    var min = ((invisibleRoot == null ? -Number.MAX_VALUE : -Number.MAX_VALUE / 2));
    var hasNodeComparator = graph.getLayout().getNodeComparator() != null;

    for (i = 0; i < firstNonRootNodeIndex; i++)
    {
      node = graph.getNode(i);
      if (node == invisibleRoot) {
        node.setSortValue(-Number.MAX_VALUE);
      }
      else {
        node.setSortValue(Math.max(min, (hasNodeComparator ? node.getSpecChildrenOrder() : node.orthogonalOrder(graph, false))));
      }
    }

    var sortAlg = new TSpanTree.RootSort(graph, firstNonRootNodeIndex);
    sortAlg.sortRoots();
    sortAlg.dispose();
  },

  rearrangeTipOverBothSides : function()
  {
    // summary:
    //		Rearrange the graph such that we have dummy nodes and east west
    //		neighbors that simulate the tip-over-both-sides alignment.
    // tags:
    //		private
    
    var graph = this.getGraph();
    if (graph.isRadialLayout()) {
      return;
    }
    if (graph.isBalloonLayout()) {
      return;
    }

    var numberOfNodes = graph.getNumberOfNodes();
    var nrDummyNodes = 0;
    var i;

    for (i = 0; i < numberOfNodes; i++)
      nrDummyNodes += this.calcTipOverBothSidesDummyNodes(graph.getNode(i));
    if (nrDummyNodes > 0) {
      graph.increaseNodeCapacity(nrDummyNodes);
    }
    for (i = 0; i < numberOfNodes; i++)
      this.createTipOverBothSidesDummyNodes(graph.getNode(i));
  },

  calcTipOverBothSidesDummyNodes : function(node)
  {
    // summary:
    //		Calculate how many dummy nodes are necessary if the input node has
    //		tip-over-both-sides alignment.
    // tags:
    //		private
    
    var numChild = node.getNumberOfChildren();
    var numAssoc = node.getNumberAssociates();
    var numEastAssoc = node.getNumberEastAssociates();
    var numWestAssoc = node.getNumberWestAssociates();
    var numEWAssoc = numAssoc - numEastAssoc - numWestAssoc;
    if (node.getEastWestNeighbor(0) != null) {
      numChild--;
    }
    if (node.getEastWestNeighbor(1) != null) {
      numChild--;
    }
    if (numChild <= 1)
    {
      if (numAssoc == 0) {
        return 0; // int
      }
    }

    var ewmax = Math.max(numEastAssoc, numWestAssoc);
    var ewmin = Math.min(numEastAssoc, numWestAssoc);
    var remain = Math.max(0, numEWAssoc - ewmax + ewmin);
    if (!node.isTipOverBothSides()) {
      return ewmax + Math.floor((remain + 1) / 2); // int
    }
    return ewmax + Math.floor((remain + 1) / 2) + Math.floor((numChild - numAssoc + 1) / 2); // int
  },

  createTipOverBothSidesDummyNodes : function (a0, a1, a2, a3, a4)
  {
    // summary:
    //		Create the dummy nodes necessary if the input node has
    //		tip-over-both-sides alignment.
    // tags:
    //		private
    
    if(arguments.length == 1)
      return this.createTipOverBothSidesDummyNodes_overload_1(a0);
    else if(arguments.length == 5)
      return this.createTipOverBothSidesDummyNodes_overload_5(a0, a1, a2, a3, a4);
  },
  
  createTipOverBothSidesDummyNodes_overload_1 : function(node)
                                                              
  {
    var numberOfChildren = node.getNumberOfChildren();
    var numberChildrenNoEastWest = numberOfChildren;
    if (node.getEastWestNeighbor(0) != null) {
      numberChildrenNoEastWest--;
    }
    if (node.getEastWestNeighbor(1) != null) {
      numberChildrenNoEastWest--;
    }
    var numAssoc = node.getNumberAssociates();

    if (numberChildrenNoEastWest <= 1)
    {
      if (node.isTipOverBothSides())
      {
        node.setAlignmentStyle(AlignmentOptions.CENTER);
      }

      if (numAssoc == 0) {
        return;
      }
    }

    if (!node.isTipOverBothSides() && numAssoc == 0) {
      return;
    }

    var assocChildren = new Array();
    var tipChildren = new Array();
    var remainingChildren = new Array();
    var eastWestNeighbors = new Array();
    var child;
    var i = 0;

    for (i = 0; i < numberOfChildren; i++)
    {
      child = node.getChild(i);
      if (node.hasEastWestNeighbor(child)) {
        eastWestNeighbors.push(child);
      }
      else if (node.hasAssociate(child)) {
        assocChildren.push(child);
      }
      else if (node.isTipOverBothSides()) {
        tipChildren.push(child);
      }
      else {
        remainingChildren.push(child);
      }
    }

    if (assocChildren.length + tipChildren.length == 0) {
      return;
    }
    node.sortChildrenForTipOverBothSides(this.getGraph(), assocChildren);
    node.sortChildrenForTipOverBothSides(this.getGraph(), tipChildren);

    var n1 = 1 + eastWestNeighbors.length;
    var n2 = 2 + ((tipChildren.length == 0 ? remainingChildren.length : 1));
    var n3 = 2 + remainingChildren.length;
    var last;
    node.setChildrenCapacity(n1);

    var n = eastWestNeighbors.length;

    for (i = 0; i < n; i++)
    {
      child = eastWestNeighbors[i];
      node.addChild(child);
      child.setImmediateParent(node);
    }

    last = this.createTipOverBothSidesDummyNodes(node, assocChildren, n2, 0, true);
    last = this.createTipOverBothSidesDummyNodes(last, tipChildren, n3, 0, false);

    if (last != null)
    {
      last.setAlignmentStyle(node.getAlignmentStyle());
      node.setAlignmentStyle(AlignmentOptions.CENTER);
    }

    n = remainingChildren.length;

    for (i = 0; i < n; i++)
    {
      child = remainingChildren[i];
      last.addChild(child);
      child.setImmediateParent(last);
    }

    if (node != last) {
      node.setLastDummyTipOverBothSides(last);
    }
  },

  createTipOverBothSidesDummyNodes_overload_5 : function(parent, children, capacityLastDummy, startAtSide, useChildAlignment)
  {
    // summary:
    //		This creates the T-shape and returns the last dummy node, or the
    //		parent, if no dummy nodes are needed. 
    // description:
    //		This creates the T-shape and returns the last dummy node, or the
    //		parent, if no dummy nodes are needed. 
    //		Note that the parent has always already the correct children
    //		capacity. P | C --D-- C | C --D-- C
    // tags:
    //		private
    
    var numberOfChildren = children.length;
    if (numberOfChildren == 0) {
      return parent; // ibm_ilog.graphlayout.tree.TNode
    }

    var west = new Array();
    var east = new Array();
    var i;
    var child;

    if (useChildAlignment)
    {
      for (i = 0; i < numberOfChildren; i++)
      {
        child = children[i];

        if (child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_WEST)
        {
          west.push(child);
        }
        else if (child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_EAST)
        {
          east.push(child);
        }
      }
    }

    for (i = 0; i < numberOfChildren; i++)
    {
      child = children[i];

      if (useChildAlignment && child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_WEST)
      {
      }
      else if (useChildAlignment && child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_EAST)
      {
      }
      else if (west.length < east.length)
      {
        west.push(child);
      }
      else if (west.length > east.length)
      {
        east.push(child);
      }
      else if (startAtSide == 0)
      {
        west.push(child);
      }
      else
      {
        east.push(child);
      }
    }

    var graph = this.getGraph();
    var actDummy;
    var nextDummy = null;
    var lastDummy = parent;
    var size = 0;
    var last = false;
    var maxI = Math.max(east.length, west.length);
    var dirTowardsLeaves = graph.getDirTowardsLeaves();
    actDummy = graph.createDummyNode();
    parent.addChild(actDummy);
    actDummy.setImmediateParent(parent);
    actDummy.setAlignmentStyle(AlignmentOptions.CENTER);
    i = 0;

    while (i < maxI)
    {
      if (i == maxI - 1) {
        last = true;
      }
      lastDummy = actDummy;
      actDummy.setChildrenCapacity((last ? capacityLastDummy : 3));

      if (i < west.length)
      {
        child = west[i];
        size = child.getSize(dirTowardsLeaves);
        if (size > actDummy.getSize(dirTowardsLeaves)) {
          actDummy.setSize(dirTowardsLeaves, size);
        }
        actDummy.setSize(1 - dirTowardsLeaves, 0);
        child.setImmediateParent(actDummy);
        actDummy.addChild(child);
        actDummy.setEastWestNeighbor(0, child);
      }

      if (i < east.length)
      {
        child = east[i];
        size = child.getSize(dirTowardsLeaves);
        if (size > actDummy.getSize(dirTowardsLeaves)) {
          actDummy.setSize(dirTowardsLeaves, size);
        }
        actDummy.setSize(1 - dirTowardsLeaves, 0);
        child.setImmediateParent(actDummy);
        actDummy.addChild(child);
        actDummy.setEastWestNeighbor(1, child);
      }

      i++;

      if (!last)
      {
        nextDummy = graph.createDummyNode();
        actDummy.addChild(nextDummy);
        nextDummy.setImmediateParent(actDummy);
        actDummy.setAlignmentStyle(AlignmentOptions.CENTER);
      }

      actDummy = nextDummy;
    }

    return lastDummy; // ibm_ilog.graphlayout.tree.TNode
  }

}
);
TSpanTree.RootSort = declare(QuickSort, 
{
  // summary:
  //		Class to sort the root nodes.
  
  _numberOfElements: null,
  _graph: null,

  constructor : function(graph, numRoots)
  {
    this._graph = graph;
    this._numberOfElements = numRoots;
  },
  
  sortRoots : function()
  {
    // summary:
    //		
    // tags:
    //		private
    
    this.inherited("sort", arguments, [this._numberOfElements]);
  },

  dispose : function()
  {
  },

  compare : function(loc1, loc2)
  {
    var node1 = this._graph.getNode(loc1);
    var node2 = this._graph.getNode(loc2);
    if (node1.getSortValue() < node2.getSortValue()) {
      return -1; // int
    }
    if (node1.getSortValue() > node2.getSortValue()) {
      return 1; // int
    }
    return 0; // int
  },

  swap : function(loc1, loc2)
  {
    var node = this._graph.getNode(loc2);
    this._graph.swapPosition(node, loc1);
  }

}
);

return TSpanTree;
});

},
'ibm_ilog/graphlayout/tree/TBalloon':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TBalloon", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/LayoutUtil",	// #6
  "../internalutil/Point",	// #7
  "../internalutil/Rectangle",	// #8
  "../internalutil/TranslateUtil",	// #9
  "../tree/TAngleSpacingModes",	// #10
  "../tree/TBalloonRadiusModes",	// #11
  "../tree/TGraphAlgorithm",	// #12
  "../tree/TNode"	// #13
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    LayoutUtil,	// #6
    Point,	// #7
    Rectangle,	// #8
    TranslateUtil,	// #9
    TAngleSpacingModes,	// #10
    TBalloonRadiusModes,	// #11
    TGraphAlgorithm,	// #12
    TNode	// #13
    ) {

// Helper comment for the Dojo apidoc build tool:

/*=====
var TGraphAlgorithm = ibm_ilog.graphlayout.tree.TGraphAlgorithm;
=====*/

var TBalloon = declare("ibm_ilog.graphlayout.tree.TBalloon", TGraphAlgorithm, 
{
  // summary:
  //		This class is the Balloon Tree layout. 
  // description:
  //		This class is the Balloon Tree layout. 
  //		It runs after calculating the spanning tree, and is independent from
  //		the normal tree layout in TGraph. It does not support subtree blocks
  //		or recursive layout. In principle we could extend it one day to
  //		support recursive layout, but this is currently not done. The
  //		original implementation came from JLoox by Fabien HERTSCHUH e.a.  
  // tags:
  //		private
  
  _initialAngle : null,
  _generalAngleRange : null,
  _leafAngleRange : null,
  _rootAngleRange : null,
  _minimumRadius : null,

  _angleSpacing : null,
  _useNodeSizes : null,

  _radiusMode : null,
  _minNodeSpacing : null,
  _aspectRatio : null,

  compareNodeSurface : function(o1, o2)
  {
    // summary:
    //		Compares two tree nodes based on their surface.  
    // o1: Object
    //		First TNode
    // o2: Object
    //		Second TNode
    // tags:
    //		private
    
    var node1 = o1;
    var node2 = o2;
    var s1 = node1.getBTSubtreeSurface();
    var s2 = node2.getBTSubtreeSurface();

    return TranslateUtil.compareNumber(s1, s2); // int
  },

  constructor : function(graph)
  {
    this._initialAngle = 0;
    this._generalAngleRange = 360;
    this._leafAngleRange = 360;
    this._rootAngleRange = 360;
    this._minimumRadius = 20;
    this._angleSpacing = TBalloon.FAST_PROPORTIONAL;
    this._useNodeSizes = true;
    this._radiusMode = TBalloonRadiusModes.VARIABLE_RADIUS;
    this._minNodeSpacing = 10;
    this._aspectRatio = 1;
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this.inherited("disposeIt", arguments, []);
  },

  setParameters : function(angleSpacing, radiusMode, minNodeSpacing, minRadius, aspectRatio, initialAngle, rootAngleRange, innerAngleRange, leafAngleRange, useNodeSizes)
  {
    // summary:
    //		Set the layout parameters.
    // tags:
    //		private
    
    this._angleSpacing = angleSpacing;
    this._radiusMode = radiusMode;
    this._minNodeSpacing = minNodeSpacing;
    this._minimumRadius = minRadius;
    this._aspectRatio = aspectRatio;
    this._initialAngle = initialAngle;
    this._rootAngleRange = rootAngleRange;
    this._generalAngleRange = innerAngleRange;
    this._leafAngleRange = leafAngleRange;
    this._useNodeSizes = useNodeSizes;
    if (this.getGraph().getFlowDirection() == 2) {
      this._initialAngle += 180;
    }
  },

  run : function()
  {
    // summary:
    //		Places all nodes.
    // tags:
    //		private
    
    var graph = this.getGraph();
    this.startStep(graph.getPercRadialOrBalloon(), 2 * graph.getNumberOfNodes());
    var coordBase = 0;
    var sdir = graph.getDirTowardsSiblings();
    var offset = graph.getMinDistBetweenBranches(sdir);
    var iter = graph.getRoots();

    while (iter.hasNext())
      coordBase = this.calcBalloonLayout(iter.next(), coordBase, 0.0) + 2.0 * offset;
  },

  calcBalloonLayout : function(root, border0, border1)
  {
    // summary:
    //		Performs a balloon tree layout for the tree starting at input root. 
    //		
    // root: TNode
    //		The root node.
    // border0: Number
    //		The top left border of this tree (dir0).
    // border1: Number
    //		The top left border of this tree (dir1).
    // returns:
    //		The maximal coordinate of the layout in direction dir0.
    // tags:
    //		private
    
    this.incrPercPointEstimation(20);

    var graph = this.getGraph();

    var dir0 = graph.getDirTowardsSiblings();
    var dir1 = 1 - dir0;

    root.setBTCenter(0, 0.0);
    root.setBTCenter(1, 0.0);
    if (!this.mayContinue()) {
      return 0; // Number
    }
    this.calcSubtreeSurfaces(root);

    this.addPercPoints(10);
    this.layoutStepPerformed();

    root.setBTCenter(0, 0);
    root.setBTCenter(1, 0);
    if (!this.mayContinue()) {
      return 0; // Number
    }
    this.computeRadiuses(root);

    var sizeRatioX = Math.max(1.0, this._aspectRatio);
    var sizeRatioY = Math.max(1.0, 1 / this._aspectRatio);
    if (!this.mayContinue()) {
      return 0; // Number
    }
    this.calculateAbsolutePosition(root, this._initialAngle, sizeRatioX, sizeRatioY);

    if (!this.mayContinue()) {
      return 0; // Number
    }
    var bounds = TranslateUtil.newArray(2, 0);
    bounds[0] = root.getMinCoord(dir0);
    bounds[1] = root.getMaxCoord(dir0);
    this.calcSubtreeBounds(root, bounds, dir0);

    var dx;
    var dy;

    if (dir0 == 0)
    {
      dx = border0 - bounds[0];
      dy = border1;
      bounds[0] += dx;
      bounds[1] += dx;
    }
    else
    {
      dx = border1;
      dy = border0 - bounds[0];
      bounds[0] += dy;
      bounds[1] += dy;
    }

    if (!this.mayContinue()) {
      return 0; // Number
    }
    graph.shiftSubtreeBy(root, dx, dy);

    this.addPercPoints(10);
    this.layoutStepPerformed();
    return (bounds[1]); // Number
  },

  computeRadiuses : function(root)
  {
    // summary:
    //		Compute the radius needed for a node not to interfere with its
    //		siblings (and its subtrees).
    // tags:
    //		private
    
    var w = root.getBTSize(0);
    var h = root.getBTSize(1);
    var radius = 0.5 * Math.sqrt(w * w + h * h);
    if (this._angleSpacing == TBalloon.PROPORTIONAL) {
      this.computeRadiusesSlowPropAngleSpacing(root);
    }
    else {
      this.computeRadiusesNormAngleSpacing(root, 180, this._initialAngle, radius);
    }
  },

  computeRadiusesSlowPropAngleSpacing : function(node)
  {
    // summary:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    // description:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    //		This is used for the slow variant of proportional angle spacing.
    // tags:
    //		private
    
    this.addPercPoints(1);
    this.layoutStepPerformed();

    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var nodeHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
    var s = nodeHalfDiag + 0.5 * this._minNodeSpacing;
    var size = node.getNumberOfVirtualChildren();
    if (size == 0) {
      return;
    }
    var childSubtreeBounds = TranslateUtil.newArray(size, null);

    var iter = node.getVirtualChildren();
    var child;
    var i = 0;

    while (iter.hasNext())
    {
      child = iter.next();
      this.computeRadiusesSlowPropAngleSpacing(child);

      childSubtreeBounds[i++] = this.getSubtreeBounds(child, 0);
    }

    var sumChildAngleRange = this.getAngleRange(node);
    var totalSurface = 0;
    var surface;
    var children = TranslateUtil.newArray(size, null);
    i = 0;
    iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      child = iter.next();
      surface = childSubtreeBounds[i].height;
      if (surface == 0) {
        surface = 1;
      }
      child.setBTSubtreeSurface(surface);
      totalSurface += surface;
      children[i++] = child;
    }

    children.sort(lang.hitch(this,this.compareNodeSurface));
    var remainingAngle = sumChildAngleRange;
    var deltaStartAngle = (360 - sumChildAngleRange) / 2;

    for (var j = size - 1; j >= 0; j--)
    {
      child = children[j];
      var childrenSurface = child.getBTSubtreeSurface();
      var maxAngleRange;

      if (size == 1)
      {
        maxAngleRange = 360;
      }
      else
      {
        maxAngleRange = 180;
      }

      var range = (remainingAngle * childrenSurface) / totalSurface;
      child.setBTAngleRange(Math.min(maxAngleRange, range));
      remainingAngle -= child.getBTAngleRange();
      totalSurface -= childrenSurface;
    }

    var minimumRadius = this._minimumRadius;
    var radius;
    var beta;
    iter = node.getVirtualChildren();
    i = 0;
    var currentAngle = 180 + deltaStartAngle;

    while (iter.hasNext())
    {
      child = iter.next();

      if (this._useNodeSizes)
      {
        w = child.getBTSize(0);
        h = child.getBTSize(1);
        var childHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
        minimumRadius = this._minimumRadius + nodeHalfDiag + childHalfDiag;
      }

      var angleRange = child.getBTAngleRange();
      var rangleRange = (angleRange)/180.0*Math.PI;
      var H1 = Math.abs(childSubtreeBounds[i].y);
      var H2 = childSubtreeBounds[i].height - H1;
      var angle = 0;
      var Raux;

      if (H1 == 0)
      {
        angle = 0;
        if (angleRange >= 89) {
          Raux = 0;
        }
        else {
          Raux = H2 / Math.tan(rangleRange);
        }
      }
      else if (H2 == 0)
      {
        angle = angleRange;
        if (angleRange >= 89) {
          Raux = 0;
        }
        else {
          Raux = H1 / Math.tan(rangleRange);
        }
      }
      else
      {
        if (angleRange >= 179)
        {
          angle = angleRange / 2;
          Raux = 0;
        }
        else
        {
          angle = TBalloon.solveTanEquation(rangleRange, H1 / H2);
          Raux = H1 / Math.tan(angle);
          angle = (angle)*180.0/Math.PI;
        }
      }

      radius = Math.max(nodeHalfDiag, Raux) - childSubtreeBounds[i].x;
      radius = Math.max(radius, minimumRadius);
      child.setBTRadius(radius);
      child.setBTAngle(currentAngle + angleRange - angle);
      currentAngle += child.getBTAngleRange();
      i++;
    }

    this.optimizeRadius(node, childSubtreeBounds, true);
  },

  computeRadiusesNormAngleSpacing : function(node, angleRange, initialAngle, parentHalfDiag)
  {
    // summary:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    // description:
    //		Recusive method to compute the radius needed for a node not to
    //		interfere with its siblings (and its subtrees). 
    //		This is used for constant angle spacing and proportional angle
    //		spacing. Returns the bounds of the entire subtree starting at node
    //		relative to the node center, i.e. the bounds are exactly returned
    //		rectangle if the node center is placed at (0,0). In reality, the
    //		node center will be placed somewhere else, but at that time point we
    //		don't know yet where.
    // tags:
    //		private
    
    this.addPercPoints(1);
    this.layoutStepPerformed();
    node.setBTAngle(initialAngle);

    if (angleRange > 180.0)
    {
      angleRange = 180;
    }

    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var w2 = w + this._minNodeSpacing;
    var h2 = h + this._minNodeSpacing;
    var subtreeBounds = new Rectangle(-w2 / 2, -h2 / 2, w2, h2);
    var nodeHalfDiag = 0.5 * Math.sqrt(w * w + h * h);

    var sumChildAngleRange = this.getAngleRange(node);
    var currentAngle = initialAngle - (sumChildAngleRange / 2);
    var size = node.getNumberOfVirtualChildren();
    var iter;
    var child;

    var i = 0;

    if (this._angleSpacing == TBalloon.REGULAR)
    {
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        iter.next().setBTAngleRange((sumChildAngleRange / size));
      }
    }
    else
    {
      var totalSurface = 0;
      var children = TranslateUtil.newArray(size, null);
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        totalSurface += Math.sqrt(child.getBTSubtreeSurface());
        children[i++] = child;
      }

      children.sort(lang.hitch(this,this.compareNodeSurface));
      var remainingAngle = sumChildAngleRange;

      for (var j = size - 1; j >= 0; j--)
      {
        child = children[j];
        var childrenSurface = Math.sqrt(child.getBTSubtreeSurface());
        var maxAngleRange;

        if (size == 1)
        {
          maxAngleRange = 360;
        }
        else
        {
          maxAngleRange = 180;
        }

        var range = (remainingAngle * childrenSurface) / totalSurface;
        child.setBTAngleRange(Math.min(maxAngleRange, range));
        remainingAngle -= child.getBTAngleRange();
        totalSurface -= childrenSurface;
      }
    }

    var childSubtreeBounds = TranslateUtil.newArray(size, null);
    iter = node.getVirtualChildren();
    i = 0;

    while (iter.hasNext())
    {
      child = iter.next();
      currentAngle += (child.getBTAngleRange() / 2);
      childSubtreeBounds[i++] = this.computeRadiusesNormAngleSpacing(child, child.getBTAngleRange(), currentAngle, nodeHalfDiag);
      currentAngle += (child.getBTAngleRange() / 2);
    }

    this.optimizeRadius(node, childSubtreeBounds, false);

    iter = node.getVirtualChildren();
    i = 0;

    while (iter.hasNext())
    {
      child = iter.next();

      var ca = child.getBTAngle();
      var cr = child.getBTRadius();
      var childCenterX = cr * Math.cos((ca)/180.0*Math.PI);
      var childCenterY = -cr * Math.sin((ca)/180.0*Math.PI);
      var childSubtreeCenterX = childCenterX + TranslateUtil.getRectCenterX(childSubtreeBounds[i]);
      var childSubtreeCenterY = childCenterY + TranslateUtil.getRectCenterY(childSubtreeBounds[i]);
      var childSubtreeWidth = childSubtreeBounds[i].width;
      var childSubtreeHeight = childSubtreeBounds[i].height;
      TranslateUtil.addPoint(subtreeBounds, childSubtreeCenterX - (childSubtreeWidth / 2), childSubtreeCenterY - (childSubtreeHeight / 2));
      TranslateUtil.addPoint(subtreeBounds, childSubtreeCenterX + (childSubtreeWidth / 2), childSubtreeCenterY + (childSubtreeHeight / 2));
      i++;
    }

    var minimumRadius = this.getMinRadius(subtreeBounds, initialAngle, parentHalfDiag, nodeHalfDiag);
    node.setBTMinRadius(minimumRadius);
    var radius = 0;
    var alpha1 = initialAngle - (angleRange / 2);
    var alpha2 = initialAngle + (angleRange / 2);
    var sinInitialAngle = Math.sin((initialAngle)/180.0*Math.PI);
    var cosInitialAngle = Math.cos((initialAngle)/180.0*Math.PI);

    var offsets = 
      [
        new Point(subtreeBounds.x, subtreeBounds.y),
        new Point(subtreeBounds.x, TranslateUtil.getRectMaxY(subtreeBounds)),
        new Point(TranslateUtil.getRectMaxX(subtreeBounds), TranslateUtil.getRectMaxY(subtreeBounds)),
        new Point(TranslateUtil.getRectMaxX(subtreeBounds), subtreeBounds.y)
       ];
    var sinAlpha1 = Math.sin((alpha1)/180.0*Math.PI);
    var cosAlpha1 = Math.cos((alpha1)/180.0*Math.PI);
    var D = sinInitialAngle * cosAlpha1 - cosInitialAngle * sinAlpha1;
    var T = 0;

    if (D != 0)
    {
      for (i = 0; i < 4; i++)
      {
        T = offsets[i].y * cosAlpha1 + offsets[i].x * sinAlpha1;
        radius = Math.max(radius, T / D);
      }
    }

    var sinAlpha2 = Math.sin((alpha2)/180.0*Math.PI);
    var cosAlpha2 = Math.cos((alpha2)/180.0*Math.PI);
    D = sinInitialAngle * cosAlpha2 - cosInitialAngle * sinAlpha2;

    if (D != 0)
    {
      for (i = 0; i < 4; i++)
      {
        T = offsets[i].y * cosAlpha2 + offsets[i].x * sinAlpha2;
        radius = Math.max(radius, T / D);
      }
    }

    var radiusCorrection = 0;
    node.setBTRadius(Math.max(radius + radiusCorrection, minimumRadius));
    return subtreeBounds; // Rectangle
  },

  optimizeRadius : function(node, childSubtreeBounds, recalcSubtreeBounds)
  {
    // summary:
    //		Optimize the radius for the children of the input node. 
    // description:
    //		Optimize the radius for the children of the input node. 
    //		At that time point, all children have an angle and a radius. It
    //		assumes that the angle of node and of all its children is already
    //		stored in the node. The radius of the children must also be stored
    //		already, except for the node itself. The childSubtreeBounds must be
    //		relative to the child locations, i.e. they assume that the children
    //		are at (0,0).
    // tags:
    //		private
    
    var child;
    var iter;
    var maxChildRadius = 0;
    var maxMinChildRadius = 0;
    var i;

    if (this._radiusMode == TBalloonRadiusModes.UNIFORM_RADIUS)
    {
      var maxRadius = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        maxRadius = Math.max(maxRadius, child.getBTRadius());
      }

      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        child.setBTRadius(maxRadius);
      }

      return;
    }
    else if (this._radiusMode == TBalloonRadiusModes.UNIFORM_LEAVES_RADIUS || this._radiusMode == TBalloonRadiusModes.OPTIMIZED_UNIFORM_LEAVES_RADIUS)
    {
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (child.getNumberOfChildren() == 0)
        {
          maxChildRadius = Math.max(maxChildRadius, child.getBTRadius());
          maxMinChildRadius = Math.max(maxMinChildRadius, child.getBTMinRadius());
        }
      }

      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        if (child.getNumberOfChildren() == 0) {
          child.setBTRadius(maxChildRadius);
        }
      }
    }

    if (this._radiusMode == TBalloonRadiusModes.UNIFORM_RADIUS || this._radiusMode == TBalloonRadiusModes.UNIFORM_LEAVES_RADIUS || this._radiusMode == TBalloonRadiusModes.VARIABLE_RADIUS) {
      return;
    }
    if (recalcSubtreeBounds)
    {
      var w = node.getBTSize(0);
      var h = node.getBTSize(1);
      var nodeHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
      maxMinChildRadius = 0;
      i = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        w = child.getBTSize(0);
        h = child.getBTSize(1);
        var childHalfDiag = 0.5 * Math.sqrt(w * w + h * h);
        childSubtreeBounds[i] = this.getSubtreeBounds(child, child.getBTAngle());
        child.setBTMinRadius(this.getMinRadius(childSubtreeBounds[i], child.getBTAngle(), nodeHalfDiag, childHalfDiag));
        if (child.getNumberOfChildren() == 0) {
          maxMinChildRadius = Math.max(maxMinChildRadius, child.getBTMinRadius());
        }
        i++;
      }
    }

    var handled = TranslateUtil.newArray(node.getNumberOfVirtualChildren(), false);
    var minRadius;
    var actRadius;
    var testRadius;

    if (this._radiusMode == TBalloonRadiusModes.OPTIMIZED_UNIFORM_LEAVES_RADIUS)
    {
      minRadius = maxMinChildRadius;
      actRadius = maxChildRadius;

      while (actRadius > minRadius && actRadius - minRadius > 1)
      {
        testRadius = (minRadius + actRadius) / 2;

        iter = node.getVirtualChildren();

        while (iter.hasNext())
        {
          child = iter.next();

          if (child.getNumberOfChildren() == 0)
          {
            child.setBTRadius(testRadius);
          }
        }

        i = 0;
        var overlaps = false;
        iter = node.getVirtualChildren();

        while (iter.hasNext() && !overlaps)
        {
          child = iter.next();

          if (child.getNumberOfChildren() == 0)
          {
            overlaps = this.overlapsSubtree(node, i, child.getBTAngle(), testRadius, childSubtreeBounds);
          }

          i++;
        }

        if (overlaps)
        {
          iter = node.getVirtualChildren();

          while (iter.hasNext())
          {
            child = iter.next();

            if (child.getNumberOfChildren() == 0)
            {
              child.setBTRadius(actRadius);
            }
          }

          minRadius = testRadius;
        }
        else
        {
          actRadius = testRadius;
        }
      }

      i = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (child.getNumberOfChildren() == 0)
        {
          handled[i] = true;
        }

        i++;
      }
    }

    while (true)
    {
      var bestIndex = -1;
      var bestChild = null;
      var bestRadius = Number.MAX_VALUE;
      var bestChildAngle = 0;
      i = 0;
      iter = node.getVirtualChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (!handled[i] && child.getBTRadius() < bestRadius)
        {
          bestChild = child;
          bestRadius = child.getBTRadius();
          bestChildAngle = child.getBTAngle();
          bestIndex = i;
        }

        i++;
      }

      if (bestChild == null) {
        return;
      }
      minRadius = bestChild.getBTMinRadius();
      actRadius = bestRadius;

      while (actRadius > minRadius && actRadius - minRadius > 1)
      {
        testRadius = (minRadius + actRadius) / 2;

        if (this.overlapsSubtree(node, bestIndex, bestChildAngle, testRadius, childSubtreeBounds))
        {
          minRadius = testRadius;
        }
        else
        {
          bestChild.setBTRadius(testRadius);
          actRadius = testRadius;
        }
      }

      handled[bestIndex] = true;
    }
  },

  overlapsSubtree : function(node, index, childAngle, childRadius, childSubtreeBounds)
  {
    // summary:
    //		Tests whether the subtree of input index overlaps with the bounds of
    //		any other subtree in the current radius setting.  
    // tags:
    //		private
    
    var testRect1 = new Rectangle();
    var testRect2 = new Rectangle();
    testRect1.x = childSubtreeBounds[index].x;
    testRect1.y = childSubtreeBounds[index].y;
    testRect1.width = childSubtreeBounds[index].width;
    testRect1.height = childSubtreeBounds[index].height;
    testRect1.x += (childRadius * Math.cos((childAngle)/180.0*Math.PI));
    testRect1.y += (-childRadius * Math.sin((childAngle)/180.0*Math.PI));

    var cr = 100000;
    var ca = node.getBTAngle();
    var dx = (cr * Math.cos((ca + 180)/180.0*Math.PI));
    var dy = (-cr * Math.sin((ca + 180)/180.0*Math.PI));
    if (node.getImmediateParent() != null && this.overlapsLine(testRect1, 0, 0, dx, dy)) {
      return true; // Boolean
    }
    var i = 0;
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();

      if (i != index)
      {
        cr = child.getBTRadius();
        ca = child.getBTAngle();
        dx = (cr * Math.cos((ca)/180.0*Math.PI));
        dy = (-cr * Math.sin((ca)/180.0*Math.PI));
        testRect2.x = childSubtreeBounds[i].x + dx;
        testRect2.y = childSubtreeBounds[i].y + dy;
        testRect2.width = childSubtreeBounds[i].width;
        testRect2.height = childSubtreeBounds[i].height;
        if (TranslateUtil.rectIntersects(testRect1, testRect2)) {
          return true; // Boolean
        }
        if (this.overlapsLine(testRect1, 0, 0, dx, dy)) {
          return true; // Boolean
        }
      }

      i++;
    }

    return false; // Boolean
  },

  getMinRadius : function(subtreeBounds, angle, parentHalfDiag, nodeHalfDiag)
  {
    // summary:
    //		Compute the minimal radius of the node.
    // tags:
    //		private
    
    while (angle > 360)
      angle -= 360;

    while (angle < 0)
      angle += 360;
    var rangle = (angle)/180.0*Math.PI;
    var minimumRadius = this._minimumRadius;

    if (this._useNodeSizes)
    {
      minimumRadius += parentHalfDiag + nodeHalfDiag;
    }

    var dx;
    var dy;
    var sinAlpha;
    var cosAlpha;
    var M;

    if (angle < 1)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag - subtreeBounds.x);
    }
    else if (angle < 89)
    {
      dx = -subtreeBounds.x;
      dy = subtreeBounds.height + subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else if (angle < 91)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + subtreeBounds.height + subtreeBounds.y);
    }
    else if (angle < 179)
    {
      dx = subtreeBounds.width + subtreeBounds.x;
      dy = subtreeBounds.height + subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else if (angle < 181)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + subtreeBounds.width + subtreeBounds.x);
    }
    else if (angle < 269)
    {
      dx = subtreeBounds.width + subtreeBounds.x;
      dy = -subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else if (angle < 271)
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag - subtreeBounds.y);
    }
    else if (angle < 359)
    {
      dx = -subtreeBounds.x;
      dy = -subtreeBounds.y;
      sinAlpha = Math.sin(rangle);
      cosAlpha = Math.cos(rangle);
      M = Math.min(dx / Math.abs(cosAlpha), dy / Math.abs(sinAlpha));
      minimumRadius = Math.max(minimumRadius, parentHalfDiag + M);
    }
    else
    {
      minimumRadius = Math.max(minimumRadius, parentHalfDiag - subtreeBounds.x);
    }

    return minimumRadius; // Number
  },

  calculateAbsolutePosition : function(node, initialAngle, sizeRatioX, sizeRatioY)
  {
    // summary:
    //		Computes absolute positions of nodes from angles and radiuses
    //		computed beforehand.
    // tags:
    //		private
    
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();
      var currentAngle = child.getBTAngle();

      if (this._angleSpacing == TBalloon.PROPORTIONAL)
      {
        currentAngle += initialAngle;
      }

      var newX = (node.getBTCenter(0) + (child.getBTRadius() * sizeRatioX * Math.cos((currentAngle)/180.0*Math.PI)));
      var newY = (node.getBTCenter(1) - (child.getBTRadius() * sizeRatioY * Math.sin((currentAngle)/180.0*Math.PI)));
      child.setBTCenter(0, newX);
      child.setBTCenter(1, newY);
      this.calculateAbsolutePosition(child, currentAngle, sizeRatioX, sizeRatioY);
    }

    this.addPercPoints(1);
    this.layoutStepPerformed();
  },

  getSubtreeBounds : function(node, initialAngle)
  {
    // summary:
    //		Computes the bounds of the subtree for angle spacing slow
    //		proportional. 
    // description:
    //		Computes the bounds of the subtree for angle spacing slow
    //		proportional. 
    //		In this case, the angle of the link from node to child is stored in
    //		the child as offset to the parent angle.
    // tags:
    //		private
    
    var result = new Rectangle();
    this.addSubtreeBounds(node, 0, 0, initialAngle, result);
    result.x -= this._minNodeSpacing / 2;
    result.y -= this._minNodeSpacing / 2;
    result.width += this._minNodeSpacing;
    result.height += this._minNodeSpacing;

    if (result.width == 0) {
      result.width = 0.1;
    }
    if (result.height == 0) {
      result.height = 0.1;
    }
    return result; // Rectangle
  },

  addSubtreeBounds : function(node, nodeX, nodeY, initialAngle, bbox)
  {
    // summary:
    //		This does the real work.
    // tags:
    //		private
    
    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var r = Math.max(w, h) / 2;
    var newX;
    var newY;
    TranslateUtil.addPoint(bbox, nodeX - r, nodeY - r);
    TranslateUtil.addPoint(bbox, nodeX + r, nodeY + r);
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();
      var currentAngle = initialAngle + child.getBTAngle();
      newX = (nodeX + (child.getBTRadius() * Math.cos((currentAngle)/180.0*Math.PI)));
      newY = (nodeY - (child.getBTRadius() * Math.sin((currentAngle)/180.0*Math.PI)));
      this.addSubtreeBounds(child, newX, newY, currentAngle, bbox);
    }
  },

  calcSubtreeBounds : function(node, bounds, dir)
  {
    // summary:
    //		Calculate the bounds of the subtree.
    // tags:
    //		private
    
    bounds[0] = Math.min(bounds[0], node.getMinCoord(dir));
    bounds[1] = Math.max(bounds[1], node.getMaxCoord(dir));
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      this.calcSubtreeBounds(iter.next(), bounds, dir);
    }
  },

  calcSubtreeSurfaces : function(node)
  {
    // summary:
    //		This does the work for one subtree starting at the input node. 
    // description:
    //		This does the work for one subtree starting at the input node. 
    //		It returns the surface of the subtree, and additionally stores all
    //		surfaces of all subsubtrees in the nodes. These are used as
    //		heuristical value to estimate the area of the balloons for subtrees
    //		that have that node as root.  We use it to determine what angle
    //		range a subtree can occupy.
    // tags:
    //		private
    
    var w = node.getBTSize(0);
    var h = node.getBTSize(1);
    var surface = Math.sqrt(w * h);
    if (!this._useNodeSizes) {
      surface = 1;
    }
    var iter = node.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();
      surface += this.calcSubtreeSurfaces(child);
    }

    node.setBTSubtreeSurface(surface);
    return surface; // Number
  },

  getAngleRange : function(node)
  {
    // summary:
    //		Returns the angle raange granted to children of the input node to
    //		place themselves.
    // tags:
    //		private
    
    if (node.getImmediateParent() == null)
    {
      return this._rootAngleRange; // Number
    }
    else if (node.hasOnlyLeaves())
    {
      return this._leafAngleRange; // Number
    }
    else
    {
      return this._generalAngleRange; // Number
    }
  },

  overlapsLine : function(rect, x0, y0, x1, y1)
  {
    // summary:
    //		Check if the input rectangle overlaps the line segment.  
    // tags:
    //		private
    
    if (TranslateUtil.rectContains(rect, x0, y0)) {
      return true; // Boolean
    }
    if (TranslateUtil.rectContains(rect, x1, y1)) {
      return true; // Boolean
    }
    var h = rect.height;
    var w = rect.width;
    if (LayoutUtil.intersects(rect.x, rect.y, rect.x, rect.y + h, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    if (LayoutUtil.intersects(rect.x + w, rect.y, rect.x + w, rect.y + h, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    if (LayoutUtil.intersects(rect.x, rect.y, rect.x + w, rect.y, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    if (LayoutUtil.intersects(rect.x, rect.y + h, rect.x + w, rect.y + h, x0, y0, x1, y1)) {
      return true; // Boolean
    }
    return false; // Boolean
  }

}
);

lang.mixin(TBalloon,
{
  REGULAR : null,

  FAST_PROPORTIONAL : null,

  PROPORTIONAL : null,

  solveTanEquation : function(Z, F)
  {
    var limit = Math.PI / 2;
    var x = F * Z / (1 + F);
    if (x >= limit || Z - x >= limit) {
      x = Z / 2;
    }
    var bestX = x;
    var bestD = Math.abs(Math.tan(x) / Math.tan(Z - x) - F);
    var delta = Z / 3;

    while (delta > 0.0001)
    {
      x = x - delta;

      for (var i = 0; i < 2; i++)
      {
        if (x >= 0 && x < Z && x < limit && Z - x < limit)
        {
          var D = Math.abs(Math.tan(x) / Math.tan(Z - x) - F);

          if (D < bestD)
          {
            bestD = D;
            bestX = x;
          }
        }

        x += 2 * delta;
      }

      x = bestX;
      delta = delta / 2;
    }

    return x; // Number
  },

  _fields_init : function(c)
  {
    c = c || TBalloon;
    if(TGraphAlgorithm._fields_init)
      TGraphAlgorithm._fields_init(c);
    
    c.REGULAR = TAngleSpacingModes.REGULAR;
    c.FAST_PROPORTIONAL = TAngleSpacingModes.FAST_PROPORTIONAL;
    c.PROPORTIONAL = TAngleSpacingModes.PROPORTIONAL;
  }
  
}
);

(function(){ TBalloon._fields_init(); })();

return TBalloon;
});

},
'ibm_ilog/graphlayout/tree/TNode':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TNode", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/AlignmentOptions",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/LayoutUtil",	// #7
  "../internalutil/QuickSort",	// #8
  "../internalutil/TranslateUtil",	// #9
  "../tree/TNodeIterator",	// #10
  "../tree/TPath"	// #11
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    AlignmentOptions,	// #5
    GraphModelUtil,	// #6
    LayoutUtil,	// #7
    QuickSort,	// #8
    TranslateUtil,	// #9
    TNodeIterator,	// #10
    TPath	// #11
    ) {

var TNode = declare("ibm_ilog.graphlayout.tree.TNode", null, 
{
  // summary:
  //		A node for tree layout. 
  // description:
  //		A node for tree layout. 
  //		The node knows its children and its parent. The coordinate of the
  //		node is relative to the local coordinate system of its parent. This
  //		means in order to shift all subtrees of this node, it is enough to
  //		shift the subtree coordinate correction value of the this node, which
  //		translates the coordinate system of this node into the coordinate
  //		system of its parent. In order to be independent of the current
  //		layout flow direction, we use arrays of 2 floats for all coordinates.
  //		A node usually has children (except the leaf nodes). Two of its
  //		children can be marked as east-west-neighbors that are placed
  //		directly neighbored to the node (e.g. in flow top to bottom at the
  //		left and right side). These two children are not considered as real
  //		children, because during layout, we treat them as virtual children of
  //		its parent. Hence the real children are the children that are not
  //		east-west-neighbors. The virtual children are recursively defined as
  //		follows:<br> - the real children of a node are its virtual
  //		children.<br> - the east-west-neighbors of virtual children of a node
  //		are its virtual children.  
  // tags:
  //		private
  
  _original: null,

  _representationGraph: null,

  _immediateParent: null,

  _originalParent: null,

  _children: null,

  _firstFreeChildrenSlot: null,

  _indegree: null,
  _outdegree: null,

  _igchildren: null,
  _firstFreeIGChildrenSlot: null,

  _igIndegree: null,
  _igOutdegree: null,

  _ewNeighbors: null,

  _associates: null,

  _lastDummyTipOverBothSides: null,

  _specRootNodePreference: null,
  _rootNodePreference: null,

  _coord: null,
  _size: null,
  _origSize: null,

  _offsetToVirtualCenter: null,
  _realBounds: null,

  _orthSplitCoord: null,

  _alignmentStyle: null,

  _childAlignment: null,

  _origAlignmentStyle: null,

  _bestAlignmentStyle: null,

  _subtreeBlockMargins: null,

  _subtreeCoordCorrection: null,

  _prevSibling: null,

  _specChildrenOrder: null,

  _iflag: null,

  _sortValue: null,
  _sortValue2: null,

  _depth: null,

  _northSizeExtensions: null,

  _minRadius: null,

  _angle: null,

  _firstVisibleAtBorder: null,

  constructor : function(original)
  {
    this._original = original;
    this._immediateParent = null;
    this._originalParent = null;
    this._children = null;
    this._firstFreeChildrenSlot = -1;
    this._indegree = 0;
    this._outdegree = 0;
    this._igchildren = null;
    this._firstFreeIGChildrenSlot = -1;
    this._igIndegree = 0;
    this._igOutdegree = 0;
    this._ewNeighbors = null;
    this._associates = null;
    this._specRootNodePreference = -1;
    this._rootNodePreference = 0;
    this._coord = TranslateUtil.newArray(2, 0);
    this._coord[0] = 0.0;
    this._coord[1] = 0.0;
    this._orthSplitCoord = TranslateUtil.newArray(2, 0);
    this._orthSplitCoord[0] = 0.0;
    this._orthSplitCoord[1] = 0.0;
    this._size = TranslateUtil.newArray(2, 0);
    this._size[0] = 0.0;
    this._size[1] = 0.0;
    this._origSize = TranslateUtil.newArray(2, 0);
    this._origSize[0] = 0.0;
    this._origSize[1] = 0.0;
    this._offsetToVirtualCenter = null;
    this._realBounds = null;
    this._alignmentStyle = AlignmentOptions.CENTER;
    this._childAlignment = AlignmentOptions.UNSPECIFIED;
    this._subtreeBlockMargins = null;
    this._origAlignmentStyle = AlignmentOptions.CENTER;
    this._bestAlignmentStyle = AlignmentOptions.CENTER;
    this._subtreeCoordCorrection = TranslateUtil.newArray(2, 0);
    this._prevSibling = null;
    this._iflag = 0;
    this._sortValue = 0.0;
    this._sortValue2 = 0.0;
    this._depth = 1;
    this._northSizeExtensions = 0;

    this._firstVisibleAtBorder = TranslateUtil.newArray(2, 2, null);

    this.reinitialize();
  },
  
  reinitialize : function()
  {
    // summary:
    //		Reinitialize for a layout. 
    // description:
    //		Reinitialize for a layout. 
    //		This is called during radial layout when we backtrack to generate
    //		more space.
    // tags:
    //		private
    
    this._subtreeCoordCorrection[0] = 0.0;
    this._subtreeCoordCorrection[1] = 0.0;
    this._firstVisibleAtBorder[0][0] = null;
    this._firstVisibleAtBorder[1][0] = null;
    this._firstVisibleAtBorder[0][1] = null;
    this._firstVisibleAtBorder[1][1] = null;
  },

  getOriginalObject : function()
  {
    // summary:
    //		Returns the original object.
    
    return this._original; // Object
  },

  getRepresentationGraph : function()
  {
    // summary:
    //		Returns the representation graph. 
    // description:
    //		Returns the representation graph. 
    //		If this node represents a subgraph as node, then the representation
    //		graph is the contents of the subgraph.
    // tags:
    //		private
    
    return this._representationGraph; // ibm_ilog.graphlayout.tree.TGraph
  },

  setRepresentationGraph : function(graph)
  {
    // summary:
    //		Sets the representation graph.
    // tags:
    //		private
    
    this._representationGraph = graph;
  },

  getImmediateParent : function()
  {
    // summary:
    //		Returns the immediate parent node.
    // tags:
    //		private
    
    return this._immediateParent; // ibm_ilog.graphlayout.tree.TNode
  },

  setImmediateParent : function(parent)
  {
    // summary:
    //		Sets the immediate parent node. 
    // description:
    //		Sets the immediate parent node. 
    //		Initially, the parent is stored as child, so we also have to remove
    //		the node from the field of childrens. The immediate parent can also
    //		be a dummy node.
    // tags:
    //		private
    
    this._immediateParent = parent;
    if (this._children != null) {
      this.removeChild(parent);
    }
  },

  getOriginalParent : function()
  {
    // summary:
    //		Returns the original parent node. 
    // description:
    //		Returns the original parent node. 
    //		This is set only when we need to introduce dummy nodes that are used
    //		as intermediate points to implement certain alignment styles, e.g.,
    //		the TIP&#95;OVER&#95;BOTH&#95;SIDES style. A node that has a dummy node as
    //		actual parent will have the parent of the dummy node as original
    //		parent.
    // tags:
    //		private
    
    return this._originalParent; // ibm_ilog.graphlayout.tree.TNode
  },

  setOriginalParent : function(parent)
  {
    // summary:
    //		Sets the original parent node.
    // tags:
    //		private
    
    this._originalParent = parent;
    this.setImmediateParent(parent);
  },

  getNumberOfChildren : function()
  {
    // summary:
    //		Returns the number of children.
    // tags:
    //		private
    
    return ((this._firstFreeChildrenSlot > 0 ? this._firstFreeChildrenSlot : 0)); // int
  },

  getChild : function(i)
  {
    // summary:
    //		Returns the i-th child.
    // tags:
    //		private
    
    return (this._children[i]); // ibm_ilog.graphlayout.tree.TNode
  },

  getLastChild : function()
  {
    // summary:
    //		Returns the last child.
    // tags:
    //		private
    
    return (this._children[this._firstFreeChildrenSlot - 1]); // ibm_ilog.graphlayout.tree.TNode
  },

  getFirstRealChild : function()
  {
    // summary:
    //		Returns the first child that is not an east-west-neighbor.
    // tags:
    //		private
    
    if (this._firstFreeChildrenSlot <= 0) {
      return (null); // ibm_ilog.graphlayout.tree.TNode
    }
    var i = 0;
    var child = this.getChild(i);

    while (i < this.getNumberOfChildren() - 1 && this.hasEastWestNeighbor(child))
      child = this.getChild(++i);
    if (this.hasEastWestNeighbor(child)) {
      return (null); // ibm_ilog.graphlayout.tree.TNode
    }
    else {
      return (child); // ibm_ilog.graphlayout.tree.TNode
    }
  },

  getLastRealChild : function()
  {
    // summary:
    //		Returns the last child that is not an east-west-neighbor.
    // tags:
    //		private
    
    if (this._firstFreeChildrenSlot <= 0) {
      return (null); // ibm_ilog.graphlayout.tree.TNode
    }
    var i = this.getNumberOfChildren() - 1;
    var child = this.getChild(i);

    while (i > 0 && this.hasEastWestNeighbor(child))
      child = this.getChild(--i);
    if (this.hasEastWestNeighbor(child)) {
      return (null); // ibm_ilog.graphlayout.tree.TNode
    }
    else {
      return (child); // ibm_ilog.graphlayout.tree.TNode
    }
  },

  getLastVirtualChild : function()
  {
    // summary:
    //		Returns the last virtual child considering all east-west-neighbors.
    
    var child = this.getLastRealChild();

    if (child != null) {
      while (child.getEastWestNeighbor(1) != null)
        child = child.getEastWestNeighbor(1);
    }
    return (child); // ibm_ilog.graphlayout.tree.TNode
  },

  getNumberOfVirtualChildren : function()
  {
    // summary:
    //		Returns the number of virtual children.
    // tags:
    //		private
    
    if (this.getNumberOfChildren() == 0) {
      return 0; // int
    }
    var count = 0;
    var iter = this.getVirtualChildren();

    while (iter.hasNext())
    {
      iter.next();
      count++;
    }

    return count; // int
  },

  getChildren : function()
  {
    // summary:
    //		Returns the children of this node.
    // tags:
    //		private
    
    return new TNodeAnonClass_1(this); // ibm_ilog.graphlayout.tree.TNodeIterator
  },

  getRealChildren : function()
  {
    // summary:
    //		Returns the real children of this node. 
    // description:
    //		Returns the real children of this node. 
    //		If the node has east-west-neighbors, these are not considered real
    //		children and excluded from the iteration.
    // tags:
    //		private
    
    return new TNode.TRealChildrenIterator(this); // ibm_ilog.graphlayout.tree.TNodeIterator
  },

  getVirtualChildren : function()
  {
    // summary:
    //		Returns the virtual children of this node. 
    // description:
    //		Returns the virtual children of this node. 
    //		If a child has east-west-neighbors, these are considered virtual
    //		children of this node and are included into the iteration.
    // tags:
    //		private
    
    return new TNode.TVirtualChildrenIterator(this); // ibm_ilog.graphlayout.tree.TNodeIterator
  },

  setChildrenCapacity : function(maxNumberOfChildren)
  {
    // summary:
    //		Sets the children capacity, i.e., the maximal number of children at
    //		this node.
    // tags:
    //		private
    
    if (maxNumberOfChildren > 0)
    {
      this._children = TranslateUtil.newArray(maxNumberOfChildren, null);
      this._firstFreeChildrenSlot = 0;
    }
    else
    {
      this._children = null;
      this._firstFreeChildrenSlot = -1;
    }
  },

  addChild : function(child)
  {
    // summary:
    //		Adds a child to this node.
    // tags:
    //		private
    
    var iter = this.getChildren();

    while (iter.hasNext())
      if (iter.next() == child) {
        return;
      }
    if (this._children == null || this._firstFreeChildrenSlot < 0)
    {
      this.setChildrenCapacity(2);
    }

    if (this._firstFreeChildrenSlot >= this._children.length)
    {
      var oldChildren = this._children;
      var n = this._firstFreeChildrenSlot;
      this.setChildrenCapacity(n + 2);
      TranslateUtil.arrayCopy(oldChildren, 0, this._children, 0, n);
      this._firstFreeChildrenSlot = n;
    }

    this._children[this._firstFreeChildrenSlot++] = child;
  },

  removeChild : function(child)
  {
    // summary:
    //		Removes the child from this node.
    // tags:
    //		private
    
    for (var i = 0; i < this.getNumberOfChildren(); i++)
    {
      if (this.getChild(i) == child)
      {
        TranslateUtil.arrayCopy(this._children, i + 1, this._children, i, this._firstFreeChildrenSlot - i - 1);
        this._firstFreeChildrenSlot--;
        this._children[this._firstFreeChildrenSlot] = null;
        break;
      }
    }
  },

  replaceChild : function(orig, repl)
  {
    // summary:
    //		Replaces the child at this node.
    // tags:
    //		private
    
    for (var i = 0; i < this.getNumberOfChildren(); i++)
    {
      if (this.getChild(i) == orig)
      {
        this._children[i] = repl;
      }
    }
  },

  getNumberOfIntergraphConnectivityChildren : function()
  {
    // summary:
    //		Returns the number of intergraph connectivity children.
    // tags:
    //		private
    
    return ((this._firstFreeIGChildrenSlot > 0 ? this._firstFreeIGChildrenSlot : 0)); // int
  },

  getIntergraphConnectivityChild : function(i)
  {
    // summary:
    //		Returns the i-th intergraph connectivity child.
    // tags:
    //		private
    
    return (this._igchildren[i]); // ibm_ilog.graphlayout.tree.TNode
  },

  getIntergraphConnectivityChildIndex : function(child)
  {
    // summary:
    //		Returns the index of the intergraph connectivity child. 
    // description:
    //		Returns the index of the intergraph connectivity child. 
    //		Returns -1 if it is not a child.
    // tags:
    //		private
    
    var n = this.getNumberOfIntergraphConnectivityChildren();

    for (var i = 0; i < n; i++)
    {
      if (this.getIntergraphConnectivityChild(i) == child) {
        return i; // int
      }
    }

    return -1; // int
  },

  addIntergraphConnectivityChild : function(child)
  {
    // summary:
    //		Adds an intergraph connectivity child to this node.
    // tags:
    //		private
    
    if (this.getIntergraphConnectivityChildIndex(child) >= 0) {
      return;
    }
    if (this._igchildren == null || this._firstFreeIGChildrenSlot < 0)
    {
      this._igchildren = TranslateUtil.newArray(2, null);
      this._firstFreeIGChildrenSlot = 0;
    }

    if (this._firstFreeIGChildrenSlot >= this._igchildren.length)
    {
      var oldChildren = this._igchildren;
      this._igchildren = TranslateUtil.newArray(this._firstFreeIGChildrenSlot + 2, null);
      TranslateUtil.arrayCopy(oldChildren, 0, this._igchildren, 0, this._firstFreeIGChildrenSlot);
    }

    this._igchildren[this._firstFreeIGChildrenSlot++] = child;
  },

  removeIntergraphConnectivityChild : function(child)
  {
    // summary:
    //		Remove an intergraph connectivity child from this node.
    // tags:
    //		private
    
    var i = this.getIntergraphConnectivityChildIndex(child);
    if (i < 0) {
      return;
    }
    TranslateUtil.arrayCopy(this._igchildren, i + 1, this._igchildren, i, this._firstFreeIGChildrenSlot - i - 1);
    this._firstFreeIGChildrenSlot--;
    this._igchildren[this._firstFreeIGChildrenSlot] = null;
  },

  cleanIntergraphConnectivityChildren : function()
  {
    // summary:
    //		Removes all intergraph connectivity children.
    // tags:
    //		private
    
    this._firstFreeIGChildrenSlot = -1;
    this._igchildren = null;
  },

  setNormalDegrees : function(indeg, outdeg)
  {
    // summary:
    //		Sets the original degrees of the node. 
    // description:
    //		Sets the original degrees of the node. 
    //		These are the degrees before calculating the spanning tree, i.e. in
    //		the original graph that is potenially not a tree.
    // tags:
    //		private
    
    this._indegree = indeg;
    this._outdegree = outdeg;
  },

  incrementIntergraphConnectivityIndegree : function()
  {
    // summary:
    //		Increment the intergraph indegree.
    // tags:
    //		private
    
    this._igIndegree++;
  },

  incrementIntergraphConnectivityOutdegree : function()
  {
    // summary:
    //		Increment the intergraph outdegree.
    // tags:
    //		private
    
    this._igOutdegree++;
  },

  sortChildren : function(graph, initial)
  {
    // summary:
    //		Sorts the children.
    // tags:
    //		private
    
    if (graph.getLayout().getNodeComparator() != null) {
      this.sortChildrenAsSpecified(graph, initial);
    }
    else if (!graph.getLayout().isIncrementalMode()) {
      this.sortChildrenForNonRadial(graph, initial);
    }
    else if (graph.isRadialLayout()) {
      this.sortChildrenForRadial(graph, initial);
    }
    else if (graph.isBalloonLayout()) {
      this.sortChildrenForBalloon(graph, initial);
    }
    else {
      this.sortChildrenForNonRadial(graph, initial);
    }
  },

  sortChildrenAsSpecified : function(graph, initial)
  {
    // summary:
    //		Sorts the children according to the specified children order.
    // tags:
    //		private
    
    if (this.getNumberOfChildren() > 1)
    {
      var child;
      var iter = this.getChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        child.setSortValue(child.getSpecChildrenOrder());
      }

      var sortAlg = new TNodeChildrenSort(this, this, initial);
      sortAlg.sortChildren();
      sortAlg.dispose();
    }
  },

  sortChildrenForNonRadial : function(graph, initial)
  {
    // summary:
    //		Sorts the children for nonradial layout.
    // tags:
    //		private
    
    if (this.getNumberOfChildren() > 1)
    {
      var child;
      var iter = this.getChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        child.setSortValue(child.orthogonalOrder(graph, false));
        if (this.isTipOverEastWest()) {
          child.setSecondSortValue(-child.orthogonalOrder(graph, true));
        }
      }

      var sortAlg = new TNodeChildrenSort(this, this, initial);
      sortAlg.sortChildren();
      sortAlg.dispose();
    }
  },

  sortChildrenForTipOverBothSides : function(graph, children)
  {
    // summary:
    //		Sorts the children for tip over both sides alignment. 
    // description:
    //		Sorts the children for tip over both sides alignment. 
    //		The goal is to get this ordering (for top-down layout): N | 1-+-2 |
    //		3-+-4 | 5-'
    // tags:
    //		private
    
    if (children.length <= 1) {
      return;
    }
    var oldChildren = this._children;
    var oldFirstFreeChildrenSlot = this._firstFreeChildrenSlot;

    this._children = TranslateUtil.newArray(children.length, null);
    this._children = TranslateUtil.copyArrayInto(children,  this._children);

    this._firstFreeChildrenSlot = children.length;
    var child1;
    var child2;
    var flowDir = graph.getFlowDirection();
    var dir = graph.getDirTowardsLeaves();
    var otherDir = 1 - dir;
    var mirror = TranslateUtil.newArray(2, 0);
    mirror[0] = ((flowDir == 2 ? -1.0 : 1.0));
    mirror[1] = ((flowDir == 3 ? -1.0 : 1.0));
    var iter = this.getChildren();

    while (iter.hasNext())
    {
      child1 = iter.next();
      child1.setSortValue(child1.getCenter(dir) * mirror[dir]);
    }

    var sortAlg = new TNodeChildrenSort(this, this, false);
    sortAlg.sortChildren();
    sortAlg.dispose();

    var i;

    for (i = 0; i < this._firstFreeChildrenSlot - 1; i += 2)
    {
      child1 = this._children[i];
      child2 = this._children[i + 1];

      if (child1.getCenter(otherDir) * mirror[otherDir] > child2.getCenter(otherDir) * mirror[otherDir])
      {
        this._children[i] = child2;
        this._children[i + 1] = child1;
      }
    }

    children.length = 0;

    for (i = 0; i < this._firstFreeChildrenSlot; i++)
      children.push(this._children[i]);

    this._children = oldChildren;
    this._firstFreeChildrenSlot = oldFirstFreeChildrenSlot;
  },

  sortChildrenForRadial : function(graph, initial)
  {
    // summary:
    //		Sorts the children for radial layout.
    // tags:
    //		private
    
    if (this.getNumberOfChildren() > 1)
    {
      var x;
      var y;
      var root = this;

      while (root.getImmediateParent() != null)
        root = root.getImmediateParent();
      x = root.getCenter(0);
      y = root.getCenter(1);
      var child;
      var iter = this.getChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (graph.getFlowDirection() == 1 || graph.getFlowDirection() == 2)
        {
          child.setSortValue(2 * Math.PI - child.angle(x, y));
        }
        else
        {
          child.setSortValue(child.angle(x, y));
        }
      }

      var sortAlg = new TNodeChildrenSort(this, this, initial);
      sortAlg.sortChildren();

      var firstChild = this.getChild(0);
      var h;
      var i;

      if (this.getImmediateParent() == null)
      {
        var dir = graph.getDirTowardsLeaves();
        var increasing = (graph.getFlowDirection() < 2);
        x = firstChild.getCenter(dir);

        for (i = 1; i < this.getNumberOfChildren(); i++)
        {
          child = this.getChild(i);

          if ((increasing && child.getCenter(dir) > x) || (!increasing && child.getCenter(dir) < x))
          {
            x = child.getCenter(dir);
            firstChild = child;
          }
        }
      }
      else
      {
        var prevChild = this.getLastChild();
        var gap = firstChild.getSortValue() + 2 * Math.PI - prevChild.getSortValue();
        prevChild = firstChild;

        for (i = 1; i < this.getNumberOfChildren(); i++)
        {
          child = this.getChild(i);
          h = child.getSortValue() - prevChild.getSortValue();

          if (h > gap)
          {
            gap = h;
            firstChild = child;
          }

          prevChild = child;
        }
      }

      if (firstChild != this.getChild(0))
      {
        h = -firstChild.getSortValue();
        iter = this.getChildren();

        while (iter.hasNext())
        {
          child = iter.next();
          child.addSortValue(h);
          if (child.getSortValue() < 0.0) {
            child.addSortValue(2 * Math.PI);
          }
        }

        sortAlg.sortChildren();
      }

      sortAlg.dispose();
    }
  },

  sortChildrenForBalloon : function(graph, initial)
  {
    // summary:
    //		Sorts the children for ballon layout.
    // tags:
    //		private
    
    if (this.getNumberOfChildren() > 1)
    {
      var parentAngle;
      var angle;
      var x;
      var y;

      if (this.getImmediateParent() != null)
      {
        x = this.getImmediateParent().getCenter(0);
        y = this.getImmediateParent().getCenter(1);
        parentAngle = LayoutUtil.correctAngle(this.angle(x, y) + Math.PI);
      }
      else
      {
        if (graph.getFlowDirection() == 2) {
          parentAngle = 0;
        }
        else {
          parentAngle = Math.PI;
        }
      }

      x = this.getCenter(0);
      y = this.getCenter(1);
      var child;
      var iter = this.getChildren();

      while (iter.hasNext())
      {
        child = iter.next();
        angle = child.angle(x, y) - parentAngle;

        while (angle < 0)
          angle += 2 * Math.PI;

        if (graph.getFlowDirection() == 2 || graph.getFlowDirection() == 3)
        {
          child.setSortValue(angle);
        }
        else
        {
          child.setSortValue(-angle);
        }
      }

      var sortAlg = new TNodeChildrenSort(this, this, initial);
      sortAlg.sortChildren();

      sortAlg.dispose();
    }
  },

  getPrevSibling : function()
  {
    // summary:
    //		Returns the previous sibling of this node.
    // tags:
    //		private
    
    return this._prevSibling; // ibm_ilog.graphlayout.tree.TNode
  },

  setPrevSibling : function(sibling)
  {
    // summary:
    //		Sets the previous sibling of this node.
    // tags:
    //		private
    
    this._prevSibling = sibling;
  },

  initPrevSiblings : function()
  {
    // summary:
    //		Sets the previous siblings in the subtree starting at this node. 
    // description:
    //		Sets the previous siblings in the subtree starting at this node. 
    //		If the node has east-west-neighbors, these are considered virtual
    //		children of the parent of this node, hence they are virtual siblings
    //		of this node.
    // tags:
    //		private
    
    var prevChild = null;
    var child;

    child = this.getEastWestNeighbor(0);

    if (child != null)
    {
      this._prevSibling = child;
      child.initPrevSiblings();
    }

    child = this.getEastWestNeighbor(1);

    if (child != null)
    {
      child._prevSibling = this;
      child.initPrevSiblings();
    }

    var iter = this.getVirtualChildren();

    while (iter.hasNext())
    {
      child = iter.next();

      child._prevSibling = prevChild;
      prevChild = child;
      child.initPrevSiblings();
    }
  },

  setPrevNode : function(node)
  {
    // summary:
    //		Sets the previous node of this node. 
    // description:
    //		Sets the previous node of this node. 
    //		We (mis)use the same memory for this as for the previous sibling,
    //		because we never need both at the same time.
    // tags:
    //		private
    
    this._prevSibling = node;
  },

  getPrevNode : function()
  {
    // summary:
    //		Returns the previous node of this node. 
    // description:
    //		Returns the previous node of this node. 
    //		We (mis)use the same memory for this as for the previous sibling,
    //		because we never need both at the same time.
    // tags:
    //		private
    
    return this._prevSibling; // ibm_ilog.graphlayout.tree.TNode
  },

  getFirstVisibleAtBorder : function(dir, lowerOrHigher)
  {
    // summary:
    //		Returns the path cell of the first node of the subtree starting at
    //		this node that is visible at the corresponding border of the current
    //		root, iff. 
    // description:
    //		Returns the path cell of the first node of the subtree starting at
    //		this node that is visible at the corresponding border of the current
    //		root, iff. 
    //		this node is a child of the current root and is itself not visible
    //		at that border. If dir = 0, it is either the left or the right
    //		border, otherwise the top or bottom border. If lowerOrHigher = 0, it
    //		is the left or top border, otherwise the right or bottom border
    //		(which has the higher coordinate value). This is mainly needed to
    //		quickly clean the subtree roots.
    // tags:
    //		private
    
    return this._firstVisibleAtBorder[dir][lowerOrHigher]; // ibm_ilog.graphlayout.tree.TPath
  },

  setFirstVisibleAtBorder : function(dir, lowerOrHigher, pathCell)
  {
    // summary:
    //		Sets the first path cell of this subtree that is visible at the left
    //		border.
    // tags:
    //		private
    
    this._firstVisibleAtBorder[dir][lowerOrHigher] = pathCell;
  },

  getNumberAssociates : function()
  {
    // summary:
    //		Returns the number of associates. 
    // description:
    //		Returns the number of associates. 
    //		These are all associates. They can be assigned to the east side, the
    //		west side, of free to choose both sides.
    // tags:
    //		private
    
    if (this._associates == null) {
      return 0; // int
    }
    return this._associates.length; // int
  },

  getNumberEastAssociates : function()
  {
    // summary:
    //		Returns the number of associates strictly at the east side.
    // tags:
    //		private
    
    if (this._associates == null) {
      return 0; // int
    }
    var c = 0;

    for (var i = 0; i < this._associates.length; i++)
    {
      var child = this._associates[i];
      if (child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_EAST) {
        c++;
      }
    }

    return c; // int
  },

  getNumberWestAssociates : function()
  {
    // summary:
    //		Returns the number of associates strictly at the west side.
    // tags:
    //		private
    
    if (this._associates == null) {
      return 0; // int
    }
    var c = 0;

    for (var i = 0; i < this._associates.length; i++)
    {
      var child = this._associates[i];
      if (child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_WEST) {
        c++;
      }
    }

    return c; // int
  },

  getAssociate : function(i)
  {
    // summary:
    //		Returns the i-th associate.
    // tags:
    //		private
    
    if (this._associates == null) {
      return (null); // ibm_ilog.graphlayout.tree.TNode
    }
    return (this._associates[i]); // ibm_ilog.graphlayout.tree.TNode
  },

  hasAssociate : function(node)
  {
    // summary:
    //		Returns if the input node is an associate.
    // tags:
    //		private
    
    var n = this.getNumberAssociates();

    for (var i = 0; i < n; i++)
      if (this.getAssociate(i) == node) {
        return true; // Boolean
      }
    return false; // Boolean
  },

  isAssociate : function()
  {
    // summary:
    //		Returns whether this node is an associate.
    // tags:
    //		private
    
    var parent = this.getOriginalParent();
    if (parent == null) {
      return false; // Boolean
    }
    return parent.hasAssociate(this); // Boolean
  },

  hasAssociateDummy : function(node)
  {
    // summary:
    //		Returns true if the input node is a dummy node for associate nodes.
    // tags:
    //		private
    
    if (this.getNumberAssociates() == 0) {
      return false; // Boolean
    }
    if (node.getImmediateParent() != this) {
      return false; // Boolean
    }
    var aux = node.getEastWestNeighbor(0);
    if (aux != null && this.hasAssociate(aux)) {
      return true; // Boolean
    }
    aux = node.getEastWestNeighbor(1);
    if (aux != null && this.hasAssociate(aux)) {
      return true; // Boolean
    }
    return false; // Boolean
  },

  addAssociate : function(node)
  {
    // summary:
    //		Adds an associate.
    // tags:
    //		private
    
    if (this.hasAssociate(node)) {
      return;
    }
    var n = this.getNumberAssociates() + 1;
    var assoc = TranslateUtil.newArray(n, null);
    if (this._associates != null) {
      TranslateUtil.arrayCopy(this._associates, 0, assoc, 0, n - 1);
    }
    assoc[n - 1] = node;
    this._associates = assoc;
  },

  validateAssociates : function(isRadial)
  {
    // summary:
    //		Validates the associates for the subtree starting at this node. 
    // description:
    //		Validates the associates for the subtree starting at this node. 
    //		This is called after the nodes have their parents. It removes all
    //		associates that are impossible after calculating the spanning tree.
    // tags:
    //		private
    
    var numChildrenNoEastWest = this.getNumberOfChildren();
    if (this.getEastWestNeighbor(0) != null) {
      numChildrenNoEastWest--;
    }
    if (this.getEastWestNeighbor(1) != null) {
      numChildrenNoEastWest--;
    }
    if (isRadial || numChildrenNoEastWest <= 1) {
      this._associates = null;
    }

    var a = new Array();
    var changed = false;
    var n = this.getNumberAssociates();

    for (var i = 0; i < n; i++)
    {
      var assoc = this.getAssociate(i);

      if (assoc.getImmediateParent() != this || assoc.isEastWestNeighbor(0) || assoc.isEastWestNeighbor(1))
      {
        changed = true;
      }
      else
      {
        a.push(assoc);

        assoc.setEastWestNeighbor(0, null);
        assoc.setEastWestNeighbor(1, null);
      }
    }

    var child;
    var iter;

    if (!isRadial)
    {
      iter = this.getChildren();

      while (iter.hasNext())
      {
        child = iter.next();

        if (
          child.getImmediateParent() == this && !child.isEastWestNeighbor(0) && !child.isEastWestNeighbor(1) && child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_BOTH_SIDES &&
            !(arr.indexOf(a, child)>=0))
          {
            a.push(child);

            child.setEastWestNeighbor(0, null);
            child.setEastWestNeighbor(1, null);
            changed = true;
          }

        if (
          child.getImmediateParent() == this && !child.isEastWestNeighbor(0) && !child.isEastWestNeighbor(1) && child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_EAST &&
            !(arr.indexOf(a, child)>=0))
          {
            a.push(child);

            child.setEastWestNeighbor(0, null);
            child.setEastWestNeighbor(1, null);
            changed = true;
          }

        if (
          child.getImmediateParent() == this && !child.isEastWestNeighbor(0) && !child.isEastWestNeighbor(1) && child.getChildAlignment() == AlignmentOptions.TIP_OVER_ASSOCIATE_WEST &&
            !(arr.indexOf(a, child)>=0))
          {
            a.push(child);

            child.setEastWestNeighbor(0, null);
            child.setEastWestNeighbor(1, null);
            changed = true;
          }
      }
    }

    if (changed)
    {
      this._associates = TranslateUtil.newArray(a.length, null);
      this._associates = TranslateUtil.copyArrayInto(a,  this._associates);
    }

    iter = this.getChildren();

    while (iter.hasNext())
    {
      child = iter.next();
      child.validateAssociates(isRadial);
    }
  },

  setLastDummyTipOverBothSides : function(node)
  {
    // summary:
    //		Sets the dummy node that is the last of the chain of dummy nodes
    //		that are used when associates are there or in top-over-both-sides
    //		alignment.
    // tags:
    //		private
    
    this._lastDummyTipOverBothSides = node;
  },

  getLastDummyTipOverBothSides : function()
  {
    // summary:
    //		Returns the dummy node that is the last of the chain of dummy nodes
    //		that are used when associates are there or in top-over-both-sides
    //		alignment.
    // tags:
    //		private
    
    return this._lastDummyTipOverBothSides; // ibm_ilog.graphlayout.tree.TNode
  },

  getEastWestNeighbor : function(i)
  {
    // summary:
    //		Returns the i-th east-west-neighbor.
    // tags:
    //		private
    
    if (this._ewNeighbors == null) {
      return (null); // ibm_ilog.graphlayout.tree.TNode
    }
    return (this._ewNeighbors[i]); // ibm_ilog.graphlayout.tree.TNode
  },

  setEastWestNeighbor : function(i, node)
  {
    // summary:
    //		Sets the i-th east-west-neighbor. 
    // description:
    //		Sets the i-th east-west-neighbor. 
    //		The 0-th east-west-neighbor goes to the west, and the 1-th
    //		east-west-neighbor to the east.
    // tags:
    //		private
    
    if (node == null)
    {
      if (this._ewNeighbors != null) {
        this._ewNeighbors[i] = null;
      }
    }
    else
    {
      if (this._ewNeighbors == null)
      {
        this._ewNeighbors = TranslateUtil.newArray(2, null);
        this._ewNeighbors[0] = null;
        this._ewNeighbors[1] = null;
      }

      this._ewNeighbors[i] = node;
    }
  },

  validateEastWestNeighbor : function(isRadial)
  {
    // summary:
    //		Validates the east-west-neighbor for the subtree starting at this
    //		node. 
    // description:
    //		Validates the east-west-neighbor for the subtree starting at this
    //		node. 
    //		This is called after the nodes have their parents. It removes all
    //		east-west-neighbors that are impossible after calculating the
    //		spanning tree.
    // tags:
    //		private
    
    if (this.getEastWestNeighbor(0) == this.getEastWestNeighbor(1)) {
      this.setEastWestNeighbor(0, null);
    }
    var child;
    var iter = this.getChildren();

    while (iter.hasNext())
    {
      child = iter.next();

      if (!isRadial)
      {
        if (child.getChildAlignment() == AlignmentOptions.EAST_NEIGHBOR && this.getEastWestNeighbor(1) == null && child.getEastWestNeighbor(0) == null)
        {
          this.setEastWestNeighbor(1, child);
          child.setEastWestNeighbor(0, this);
        }

        if (child.getChildAlignment() == AlignmentOptions.WEST_NEIGHBOR && this.getEastWestNeighbor(0) == null && child.getEastWestNeighbor(1) == null)
        {
          this.setEastWestNeighbor(0, child);
          child.setEastWestNeighbor(1, this);
        }
      }

      if (this.isTipOverBothSides())
      {
        child.setEastWestNeighbor(0, null);
        child.setEastWestNeighbor(1, null);
      }

      child.validateEastWestNeighbor(isRadial);
    }

    if (this.getEastWestNeighbor(0) == this.getEastWestNeighbor(1)) {
      this.setEastWestNeighbor(0, null);
    }
    for (var i = 0; i <= 1; i++)
    {
      child = this.getEastWestNeighbor(i);

      if (child != null)
      {
        if (child.getImmediateParent() != this) {
          this.setEastWestNeighbor(i, null);
        }
        else if (child.getEastWestNeighbor(1 - i) != null) {
          this.setEastWestNeighbor(i, null);
        }
      }
    }
  },

  isEastWestNeighbor : function(i)
  {
    // summary:
    //		Returns 'true' if this node is east-west-neighbor of its
    //		parent.
    // tags:
    //		private
    
    var parent = this.getImmediateParent();
    if (parent == null) {
      return (false); // Boolean
    }
    if (parent._ewNeighbors == null) {
      return (false); // Boolean
    }
    if (parent._ewNeighbors[i] == this) {
      return (true); // Boolean
    }
    return (false); // Boolean
  },

  hasEastWestNeighbor : function(node)
  {
    // summary:
    //		Returns 'true' if the child is an east-west-neighbor to
    //		this node.
    // tags:
    //		private
    
    if (node.getImmediateParent() != this) {
      return (false); // Boolean
    }
    if (this._ewNeighbors == null) {
      return (false); // Boolean
    }
    if (this._ewNeighbors[0] == node) {
      return (true); // Boolean
    }
    if (this._ewNeighbors[1] == node) {
      return (true); // Boolean
    }
    return (false); // Boolean
  },

  hasEastWestNeighborAtIndex : function(i, node)
  {
    // summary:
    //		Returns 'true' if the child is the i-th
    //		east-west-neighbor to this node.  
    // tags:
    //		private
    
    if (node.getImmediateParent() != this) {
      return (false); // Boolean
    }
    if (this._ewNeighbors == null) {
      return (false); // Boolean
    }
    if (this._ewNeighbors[i] == node) {
      return (true); // Boolean
    }
    return (false); // Boolean
  },

  mirror : function(coordinateIndex)
  {
    // summary:
    //		Mirror the node at the 0-axis of the specified coodinate index. 
    // description:
    //		Mirror the node at the 0-axis of the specified coodinate index. 
    //		If the coordinate index is 0, it mirrors x coordinates, if 1, it
    //		mirrors y coordinates.
    // tags:
    //		private
    
    var old = this._coord[coordinateIndex];
    this._coord[coordinateIndex] = -this._coord[coordinateIndex] - this._size[coordinateIndex];
    this._orthSplitCoord[coordinateIndex] = -this._orthSplitCoord[coordinateIndex];
    this.mirrorOffsetsToVirtualCenter(coordinateIndex);
    if (this.getRepresentationGraph() != null) {
      this.getRepresentationGraph().addShiftDelta(coordinateIndex, this._coord[coordinateIndex] - old);
    }
  },

  mirrorOffsetsToVirtualCenter : function(coordinateIndex)
  {
    // summary:
    //		Mirror the offsets to the virtual center.
    // tags:
    //		private
    
    if (this._offsetToVirtualCenter != null)
    {
      var i = 2 * coordinateIndex;
      var j = 2 - 2 * coordinateIndex;

      this._offsetToVirtualCenter[i] = -this._offsetToVirtualCenter[i];
      this._offsetToVirtualCenter[i + 1] = -this._offsetToVirtualCenter[i + 1];

      var h = this._offsetToVirtualCenter[j];
      this._offsetToVirtualCenter[j] = this._offsetToVirtualCenter[j + 1];
      this._offsetToVirtualCenter[j + 1] = h;
    }
  },

  shiftCoord : function(index, coord)
  {
    // summary:
    //		Shifts a node coordinate.
    // tags:
    //		private
    
    this._coord[index] += coord;
    this._orthSplitCoord[index] += coord;
    if (this.getRepresentationGraph() != null) {
      this.getRepresentationGraph().addShiftDelta(index, coord);
    }
  },

  setCenter : function(index, coord)
  {
    // summary:
    //		Sets the node center coordinate.
    // tags:
    //		private
    
    var old = this._coord[index];
    this._coord[index] = coord - 0.5 * this._size[index];
    if (this.getRepresentationGraph() != null) {
      this.getRepresentationGraph().addShiftDelta(index, this._coord[index] - old);
    }
  },

  getCenter : function(index)
  {
    // summary:
    //		Returns a node center coordinate.
    // tags:
    //		private
    
    return this._coord[index] + 0.5 * this._size[index]; // Number
  },

  hasDifferentVirtualCenter : function()
  {
    // summary:
    //		Returns true if the virtual center should be the same as the stored
    //		center.
    // tags:
    //		private
    
    if (this._offsetToVirtualCenter == null) {
      return false; // Boolean
    }
    if (this._size[0] == 0.0 && this._size[1] == 0.0) {
      return false; // Boolean
    }
    return true; // Boolean
  },

  setVirtualCenter : function(index, coord)
  {
    // summary:
    //		Sets the averaged virtual center coordinate of a node.
    // tags:
    //		private
    
    if (this.hasDifferentVirtualCenter())
    {
      var old = this._coord[index];
      this._coord[index] = coord - 0.5 * (this._size[index] + this._offsetToVirtualCenter[2 * index] + this._offsetToVirtualCenter[2 * index + 1]);
      if (this.getRepresentationGraph() != null) {
        this.getRepresentationGraph().addShiftDelta(index, this._coord[index] - old);
      }
    }
    else
    {
      this.setCenter(index, coord);
    }
  },

  setLinkVirtualCenter : function(index, topleftside, coord)
  {
    // summary:
    //		Sets the virtual center coordinate for a link that connects at the
    //		specified side. 
    // description:
    //		Sets the virtual center coordinate for a link that connects at the
    //		specified side. 
    //		index = 0, topleftside = true  : left side index = 0, topleftside =
    //		false : right side index = 1, topleftside = true  : top side index =
    //		1, topleftside = false : bottom side   
    // tags:
    //		private
    
    if (this.hasDifferentVirtualCenter())
    {
      var old = this._coord[index];
      this._coord[index] = coord - 0.5 * this._size[index] - this._offsetToVirtualCenter[2 * index + ((topleftside ? 0 : 1))];
      if (this.getRepresentationGraph() != null) {
        this.getRepresentationGraph().addShiftDelta(index, this._coord[index] - old);
      }
    }
    else
    {
      this.setCenter(index, coord);
    }
  },

  getVirtualCenter : function(index)
  {
    // summary:
    //		Returns the averaged virtual center coordinate.
    // tags:
    //		private
    
    if (this.hasDifferentVirtualCenter()) {
      return this._coord[index] + 0.5 * (this._size[index] + this._offsetToVirtualCenter[2 * index] + this._offsetToVirtualCenter[2 * index + 1]); // Number
    }
    else {
      return this.getCenter(index); // Number
    }
  },

  getLinkVirtualCenter : function(index, topleftside)
  {
    // summary:
    //		Returns the virtual center coordinate for a link that connects at
    //		the specified side. 
    // description:
    //		Returns the virtual center coordinate for a link that connects at
    //		the specified side. 
    //		index = 0, topleftside = true  : left side index = 0, topleftside =
    //		false : right side index = 1, topleftside = true  : top side index =
    //		1, topleftside = false : bottom side  
    // tags:
    //		private
    
    if (this.hasDifferentVirtualCenter()) {
      return this._coord[index] + 0.5 * this._size[index] + this._offsetToVirtualCenter[2 * index + ((topleftside ? 0 : 1))]; // Number
    }
    else {
      return this.getCenter(index); // Number
    }
  },

  setMaxCoord : function(index, coord)
  {
    // summary:
    //		Sets the node right/bottom coordinate.
    // tags:
    //		private
    
    var old = this._coord[index];
    this._coord[index] = coord - this._size[index];
    if (this.getRepresentationGraph() != null) {
      this.getRepresentationGraph().addShiftDelta(index, this._coord[index] - old);
    }
  },

  getMaxCoord : function(index)
  {
    // summary:
    //		Returns a node right/bottom coordinate.
    // tags:
    //		private
    
    return this._coord[index] + this._size[index]; // Number
  },

  setMinCoord : function(index, coord)
  {
    // summary:
    //		Sets the node left/top coordinate.
    // tags:
    //		private
    
    var old = this._coord[index];
    this._coord[index] = coord;
    if (this.getRepresentationGraph() != null) {
      this.getRepresentationGraph().addShiftDelta(index, this._coord[index] - old);
    }
  },

  getMinCoord : function(index)
  {
    // summary:
    //		Returns a node left/top coordinate.
    // tags:
    //		private
    
    return this._coord[index]; // Number
  },

  setSize : function(index, size)
  {
    // summary:
    //		Sets a node size (index=0: the width, index=1: the height).
    // tags:
    //		private
    
    this._size[index] = size;
  },

  getSize : function(index)
  {
    // summary:
    //		Returns a node size.
    // tags:
    //		private
    
    return this._size[index]; // Number
  },

  getOriginalSize : function(index)
  {
    // summary:
    //		Returns an original node size.
    // tags:
    //		private
    
    return this._origSize[index]; // Number
  },

  setBoundingBoxes : function(origBBox, realBBox, offsetToVirtualCenter, respectNodeSizes, radialLayout)
  {
    // summary:
    //		Sets all the node bounding box data.
    // tags:
    //		private
    
    if (!TranslateUtil.rectEquals(realBBox, origBBox))
    {
      this._realBounds = TranslateUtil.newArray(4, 0);
      this._realBounds[0] = realBBox.x - origBBox.x;
      this._realBounds[1] = realBBox.y - origBBox.y;
      this._realBounds[2] = realBBox.width;
      this._realBounds[3] = realBBox.height;
    }

    this._offsetToVirtualCenter = offsetToVirtualCenter;

    if (origBBox.width >= 0.0)
    {
      this._coord[0] = origBBox.x;
      this._origSize[0] = origBBox.width;
    }
    else
    {
      this._coord[0] = origBBox.x + origBBox.width;
      this._origSize[0] = -origBBox.width;
    }

    if (origBBox.height >= 0.0)
    {
      this._coord[1] = origBBox.y;
      this._origSize[1] = origBBox.height;
    }
    else
    {
      this._coord[1] = origBBox.y + origBBox.height;
      this._origSize[1] = -origBBox.height;
    }

    if (!respectNodeSizes || (this._origSize[0] == 0.0 && this._origSize[1] == 0.0))
    {
      this._coord[0] += 0.5 * (this.getOriginalSize(0) + this.getOffsetToVirtualCenter(0) + this.getOffsetToVirtualCenter(1));
      this._coord[1] += 0.5 * (this.getOriginalSize(1) + this.getOffsetToVirtualCenter(2) + this.getOffsetToVirtualCenter(3));
      this._size[0] = this._size[1] = 0.0;
    }
    else if (radialLayout)
    {
      var a = this._origSize[0];
      var b = this._origSize[1];
      var d = Math.sqrt(a * a + b * b);
      this._size[0] = this._size[1] = d;
    }
    else
    {
      this._size[0] = this._origSize[0];
      this._size[1] = this._origSize[1];
    }
  },

  getOffsetToVirtualCenter : function(side)
  {
    // summary:
    //		Returns the shift value at the given side that defines the virtual
    //		center. 
    // description:
    //		Returns the shift value at the given side that defines the virtual
    //		center. 
    //		0 = top side, 1 = bottom side, 2 = left side, 3 = right side.
    // tags:
    //		private
    
    if (this._offsetToVirtualCenter == null) {
      return 0.0; // Number
    }
    return this._offsetToVirtualCenter[side]; // Number
  },

  getOffsetToRealTopLeft : function(index)
  {
    // summary:
    //		Returns the offset between real node bounding box and the stored
    //		original node box.
    // tags:
    //		private
    
    if (this._realBounds == null) {
      return 0.0; // Number
    }
    return this._realBounds[index]; // Number
  },

  getRealMinCoord : function(index)
  {
    // summary:
    //		Returns the minimal coordinate of the real node bounding box.
    // tags:
    //		private
    
    if (this._realBounds == null) {
      return this.getMinCoord(index); // Number
    }
    return this._coord[index] + this._realBounds[index]; // Number
  },

  getRealSize : function(index)
  {
    // summary:
    //		Returns the size of the real node bounding box.
    // tags:
    //		private
    
    if (this._realBounds == null) {
      return this.getSize(index); // Number
    }
    return this._realBounds[2 + index]; // Number
  },

  setBackToOriginalSize : function(respectNodeSizes)
  {
    // summary:
    //		Transfer the original size back to the actual size. 
    // description:
    //		Transfer the original size back to the actual size. 
    //		As side effect, it moves the node a little bit so that
    //		it's reference point remains the same. If respecting node sizes, the reference point is the center, otherwise it is the virtual center.
    // tags:
    //		private
    
    var x0 = this.getCenter(0);
    var x1 = this.getCenter(1);
    var size0 = this.getOriginalSize(0);
    var size1 = this.getOriginalSize(1);

    if (respectNodeSizes)
    {
      this.setSize(0, size0);
      this.setSize(1, size1);
      this.setCenter(0, x0);
      this.setCenter(1, x1);
    }
    else
    {
      x0 -= 0.5 * (size0 + this.getOffsetToVirtualCenter(0) + this.getOffsetToVirtualCenter(1));
      x1 -= 0.5 * (size1 + this.getOffsetToVirtualCenter(2) + this.getOffsetToVirtualCenter(3));
      this.setSize(0, size0);
      this.setSize(1, size1);
      this.setMinCoord(0, x0);
      this.setMinCoord(1, x1);
    }
  },

  getFinalX : function()
  {
    // summary:
    //		Returns the X Position where we finally move the node inside detach.
    // tags:
    //		private
    
    var x;

    if (this.getSize(0) >= 0.0) {
      x = this.getMinCoord(0);
    }
    else {
      x = this.getMaxCoord(0);
    }
    x += this.getOffsetToRealTopLeft(0);
    return x; // Number
  },

  getFinalY : function()
  {
    // summary:
    //		Returns the Y Position where we finally move the node inside detach.
    // tags:
    //		private
    
    var y;

    if (this.getSize(1) >= 0.0) {
      y = this.getMinCoord(1);
    }
    else {
      y = this.getMaxCoord(1);
    }
    y += this.getOffsetToRealTopLeft(1);
    return y; // Number
  },

  angle : function(x, y)
  {
    // summary:
    //		Returns the angle of the center of the node wrt. 
    // description:
    //		Returns the angle of the center of the node wrt. 
    //		a given point.
    // tags:
    //		private
    
    return (LayoutUtil.angle(x, y, this.getCenter(0), this.getCenter(1))); // Number
  },

  orthogonalOrder : function(graph, otherDirection)
  {
    // summary:
    //		Returns the orthogonal order value of the node. 
    // description:
    //		Returns the orthogonal order value of the node. 
    //		For instance, if orientation is top down, then the nodes with same y
    //		coordinate must be ordered according to increasing x coordinate. If
    //		orientation is left right, the nodes with same x coordinate must be
    //		ordered according to increasing y coordinate. And, if we have a
    //		tip-over, it's just converse to normal: x and y are exchanged.
    // tags:
    //		private
    
    var coord = TranslateUtil.newArray(2, 0);
    var mirror = TranslateUtil.newArray(2, 0);
    coord[0] = this.getCenter(0);
    coord[1] = this.getCenter(1);
    mirror[0] = 1;
    mirror[1] = 1;
    var flowDir = graph.getFlowDirection();

    switch (flowDir)
    {
      case 2:
        mirror[0] = -1;
        break;
      case 3:
        mirror[1] = -1;
        break;
    }
    if (this.getImmediateParent() != null)
    {
      if (this.getImmediateParent().isTipOverFromCoord(graph) || this.getImmediateParent().isTipOver())
      {
        if (otherDirection) {
          mirror[(flowDir % 2)] = 0;
        }
        else {
          mirror[1 - (flowDir % 2)] = 0;
        }
      }
      else
      {
        if (otherDirection) {
          mirror[1 - flowDir % 2] = 0;
        }
        else {
          mirror[flowDir % 2] = 0;
        }
      }
    }

    return (mirror[0] * coord[0] + mirror[1] * coord[1]); // Number
  },

  getOrthSplitCoord : function(index)
  {
    // summary:
    //		Returns the split coordinate for orthogonal routing. 
    // description:
    //		Returns the split coordinate for orthogonal routing. 
    //		This is the coordinate where the link pointing to this child will
    //		have its segement perpendicular to the flow.
    // tags:
    //		private
    
    return this._orthSplitCoord[index]; // Number
  },

  setOrthSplitCoord : function(index, coord)
  {
    // summary:
    //		Sets the split coordinate for orthogonal routing.
    // tags:
    //		private
    
    this._orthSplitCoord[index] = coord;
  },

  getSubtreeCoordCorrection : function(index)
  {
    // summary:
    //		Returns the correction value for the subtrees of this node, so that
    //		they are in the same coordinate system as this node. 
    // description:
    //		Returns the correction value for the subtrees of this node, so that
    //		they are in the same coordinate system as this node. 
    //		Manipulating this value allows to shift a complete subtree without
    //		visiting all nodes. The coordinate correction is finally evaluated
    //		at the end of the layout. If index is 0, it is in x-direction, if
    //		index is 1, it is in y-direction.
    // tags:
    //		private
    
    return this._subtreeCoordCorrection[index]; // Number
  },

  setSubtreeCoordCorrection : function(index, coord)
  {
    // summary:
    //		Sets the correction value of the subtrees of this node. 
    // description:
    //		Sets the correction value of the subtrees of this node. 
    //		If index is 0, it is in x-direction, if index is 1, it is in
    //		y-direction.
    // tags:
    //		private
    
    this._subtreeCoordCorrection[index] = coord;
  },

  shiftSubtreeCoordCorrection : function(index, dist)
  {
    // summary:
    //		Shifts the correction value of the subtrees of this node. 
    // description:
    //		Shifts the correction value of the subtrees of this node. 
    //		If index is 0, it is in x-direction, if index is 1, it is in
    //		y-direction.
    // tags:
    //		private
    
    this._subtreeCoordCorrection[index] += dist;
  },

  getSubtreeBlockMargin : function(side)
  {
    // summary:
    //		Returns the subtree block margin at the given side. 
    // description:
    //		Returns the subtree block margin at the given side. 
    //		Index: 0 = east, 1 = south, 2 = west, 3 = north.
    // tags:
    //		private
    
    if (this._subtreeBlockMargins == null) {
      return -1; // Number
    }
    return this._subtreeBlockMargins[side]; // Number
  },

  setSubtreeBlockMargins : function(margins)
  {
    // summary:
    //		Sets the subtree block margins.
    // tags:
    //		private
    
    this._subtreeBlockMargins = margins;
  },

  mirrorSubtreeBlockMarginsEastWest : function()
  {
    // summary:
    //		Mirrors the subtree block margins.
    // tags:
    //		private
    
    if (this._subtreeBlockMargins == null) {
      return;
    }
    var newMargins = TranslateUtil.newArray(4, 0);

    newMargins[0] = this._subtreeBlockMargins[2];
    newMargins[1] = this._subtreeBlockMargins[1];
    newMargins[2] = this._subtreeBlockMargins[0];
    newMargins[3] = this._subtreeBlockMargins[3];
    this._subtreeBlockMargins = newMargins;
  },

  setDepth : function(depth)
  {
    // summary:
    //		Sets the depth of the subtree.
    
    this._depth = depth;
  },

  getDepth : function()
  {
    // summary:
    //		Returns the depth of the subtree. 
    // description:
    //		Returns the depth of the subtree. 
    //		Leaves have depth 1, parents of leaves depth 2 etc. It is basically
    //		the length of the longest link path from this node to a leaf plus
    //		one. This is only available after calculating the spanning tree.
    // tags:
    //		private
    
    return this._depth; // int
  },

  setNorthSizeExtension : function(value)
  {
    // summary:
    //		Sets the size extension at the north side of the node.
    // tags:
    //		private
    
    this._northSizeExtensions = value;
  },

  getNorthSizeExtension : function()
  {
    // summary:
    //		Returns the size extension at the north side of the node.
    // tags:
    //		private
    
    return this._northSizeExtensions; // Number
  },

  getChildAlignment : function()
  {
    // summary:
    //		Returns the child alignment.
    // tags:
    //		private
    
    return this._childAlignment; // int
  },

  setChildAlignment : function(align)
  {
    // summary:
    //		Sets the child alignment.
    // tags:
    //		private
    
    this._childAlignment = align;
  },

  getAlignmentStyle : function()
  {
    // summary:
    //		Returns the style of the alignment.
    // tags:
    //		private
    
    return this._alignmentStyle & TNode.ALIGNMENT_MASK; // int
  },

  setAlignmentStyle : function(alignmentStyle)
  {
    // summary:
    //		Sets the style of the alignment.
    // tags:
    //		private
    
    this._alignmentStyle = alignmentStyle;
  },

  storeOriginalAlignmentStyle : function()
  {
    // summary:
    //		Stores the alignment style as original alignment style.
    // tags:
    //		private
    
    this._origAlignmentStyle = this.getAlignmentStyle();
  },

  restoreOriginalAlignmentStyle : function()
  {
    // summary:
    //		Restores the original alignment style.
    // tags:
    //		private
    
    this._alignmentStyle = this._origAlignmentStyle;
  },

  storeBestAlignmentStyle : function()
  {
    // summary:
    //		Stores the alignment style as best alignment style.
    // tags:
    //		private
    
    this._bestAlignmentStyle = this.getAlignmentStyle();
  },

  restoreBestAlignmentStyle : function()
  {
    // summary:
    //		Restores the best alignment style.
    // tags:
    //		private
    
    this._alignmentStyle = this._bestAlignmentStyle;
  },

  isTipOver : function()
  {
    // summary:
    //		Returns true if the children of this node are laid out as tip-over. 
    // description:
    //		Returns true if the children of this node are laid out as tip-over. 
    //		Note that tip-over-both sides is not a classical tip-over hence is
    //		not included here. Instead, it is handled via east-west-neighborings
    //		and dummy nodes.
    // tags:
    //		private
    
    var alignmentStyle = this.getAlignmentStyle();
    return alignmentStyle == AlignmentOptions.TIP_OVER || alignmentStyle == AlignmentOptions.TIP_OVER_WEST || alignmentStyle == AlignmentOptions.TIP_OVER_EAST_WEST; // Boolean
  },

  isTipOverEast : function()
  {
    // summary:
    //		Returns true if the children of this node are laid out as tip-over
    //		east.
    // tags:
    //		private
    
    return this.getAlignmentStyle() == AlignmentOptions.TIP_OVER; // Boolean
  },

  isTipOverWest : function()
  {
    // summary:
    //		Returns true if the children of this node are laid out as tip-over
    //		west.
    // tags:
    //		private
    
    return this.getAlignmentStyle() == AlignmentOptions.TIP_OVER_WEST; // Boolean
  },

  isTipOverEastWest : function()
  {
    // summary:
    //		Returns true if the children of this node are laid out as tip-over
    //		east-west.
    // tags:
    //		private
    
    return this.getAlignmentStyle() == AlignmentOptions.TIP_OVER_EAST_WEST; // Boolean
  },

  isTipOverBothSides : function()
  {
    // summary:
    //		Returns true if the children of this node are laid out as tip-over
    //		to both sides.
    // tags:
    //		private
    
    return this.getAlignmentStyle() == AlignmentOptions.TIP_OVER_BOTH_SIDES; // Boolean
  },

  markTipOverEastWestChildAtWest : function()
  {
    // summary:
    //		If the
    //		parent's alignment style is TIP_OVER_EAST_WEST, mark that this child actually goes to the west side, not to the east side.
    // tags:
    //		private
    
    this._alignmentStyle |= TNode.ALIGNMENT_USE_WEST;
  },

  isTipOverEastWestChildAtEast : function()
  {
    // summary:
    //		Returns  true if the parent of this node is laid out as tip-over
    //		east-west and this node has choosen the east alignment.
    // tags:
    //		private
    
    return (this._alignmentStyle & TNode.ALIGNMENT_USE_WEST) == 0; // Boolean
  },

  isTipOverEastWestChildAtWest : function()
  {
    // summary:
    //		Returns  true if the parent of this node is laid out as tip-over
    //		east-west and this node has choosen the west alignment.
    // tags:
    //		private
    
    return (this._alignmentStyle & TNode.ALIGNMENT_USE_WEST) != 0; // Boolean
  },

  isTipOverFromCoord : function(graph)
  {
    // summary:
    //		Returns true if the children of this node are laid out as tip-over. 
    // description:
    //		Returns true if the children of this node are laid out as tip-over. 
    //		Here we
    //		don't consider the specification but only the coordinates of the nodes. Works for tip-over east and west.
    // tags:
    //		private
    
    var dir = graph.getDirTowardsLeaves();

    var allSameCoordInDir = true;
    var minRange = 0.0;
    var maxRange = 0.0;

    var child = this.getFirstRealChild();
    if (child == null) {
      return false; // Boolean
    }
    var minx = child.getMinCoord(0);
    var maxx = child.getMaxCoord(0);
    var miny = child.getMinCoord(1);
    var maxy = child.getMaxCoord(1);
    var iter = this.getRealChildren();

    while (iter.hasNext())
    {
      child = iter.next();

      if (allSameCoordInDir)
      {
        if (minRange == 0.0 && maxRange == 0.0)
        {
          minRange = child.getMinCoord(dir);
          maxRange = child.getMaxCoord(dir);
        }
        else
        {
          if (child.getMinCoord(dir) > maxRange) {
            allSameCoordInDir = false;
          }
          else if (child.getMaxCoord(dir) < minRange) {
            allSameCoordInDir = false;
          }
          if (child.getMinCoord(dir) < minRange) {
            minRange = child.getMinCoord(dir);
          }
          if (child.getMaxCoord(dir) > maxRange) {
            maxRange = child.getMaxCoord(dir);
          }
        }
      }

      if (child.getMinCoord(0) < minx) {
        minx = child.getMinCoord(0);
      }
      if (child.getMaxCoord(0) > minx) {
        maxx = child.getMaxCoord(0);
      }
      if (child.getMinCoord(1) < miny) {
        miny = child.getMinCoord(1);
      }
      if (child.getMaxCoord(1) > miny) {
        maxy = child.getMaxCoord(1);
      }
    }

    var x = this.getCenter(0);
    var y = this.getCenter(1);
    return (!allSameCoordInDir && (x < minx || x > maxx) && (y < miny || y > maxy)); // Boolean
  },

  canAutoTipOver : function()
  {
    // summary:
    //		Returns true if we can automatically tip over.
    // tags:
    //		private
    
    if (this.isTipOver()) {
      return false; // Boolean
    }
    if (this.getNumberOfChildren() < 2) {
      return false; // Boolean
    }
    for (var i = 0; i < this.getNumberOfChildren(); i++)
    {
      if (this.getChild(i).getEastWestNeighbor(0) != null) {
        return false; // Boolean
      }
    }

    return true; // Boolean
  },

  isVisited : function(visitedLevel)
  {
    // summary:
    //		Returns whether the node was visited. 
    // description:
    //		Returns whether the node was visited. 
    //		Since we distribute indexe during visiting, it can be obtained from
    //		the index.
    // tags:
    //		private
    
    return (this._iflag < visitedLevel); // Boolean
  },

  getIndex : function()
  {
    // summary:
    //		Returns the index at this node.
    // tags:
    //		private
    
    return this._iflag; // int
  },

  setIndex : function(v)
  {
    // summary:
    //		Sets the index at this node.
    // tags:
    //		private
    
    this._iflag = v;
  },

  getRootNodePreference : function()
  {
    // summary:
    //		Return the preference that this node will be a root node.
    // tags:
    //		private
    
    return this._rootNodePreference; // int
  },

  setSpecRootNodePreference : function(x)
  {
    // summary:
    //		Sets the specified original preference that this node will be a root
    //		node.
    // tags:
    //		private
    
    this._specRootNodePreference = x;
  },

  calcRootNodePreference : function()
  {
    // summary:
    //		Calculates the root node preference from the specification and from
    //		indegree and outdegree.
    // tags:
    //		private
    
    if (this._specRootNodePreference > 0) {
      this._rootNodePreference = this._specRootNodePreference;
    }
    else
    {
      var f1 = 1;
      var f2 = 1;
      var f3 = 1;
      var f4 = 1;

      if (this._indegree == 0)
      {
        if (this._igIndegree == 0)
        {
          f1 = 500;
          f2 = 200;
        }
        else
        {
          f1 = 20;
        }
      }
      else if (this._outdegree == 0)
      {
        if (this._igOutdegree == 0)
        {
          f3 = 500;
          f4 = 200;
        }
        else
        {
          f3 = 20;
        }
      }

      this._rootNodePreference = f1 * this._outdegree + f2 * this._igOutdegree - f3 * this._indegree - f4 * this._igIndegree;
    }
  },

  getSpecChildrenOrder : function()
  {
    // summary:
    //		Returns the children order index.
    // tags:
    //		private
    
    return this._specChildrenOrder; // int
  },

  setSpecChildrenOrder : function(i)
  {
    // summary:
    //		Sets the children order index. 
    // description:
    //		Sets the children order index. 
    //		Only used when we have a node comparator and do not determine the
    //		children order incrementally or otherwise.
    // tags:
    //		private
    
    this._specChildrenOrder = i;
  },

  getSortValue : function()
  {
    // summary:
    //		Returns the sort value.
    // tags:
    //		private
    
    return this._sortValue; // Number
  },

  setSortValue : function(x)
  {
    // summary:
    //		Sets the sort value.
    // tags:
    //		private
    
    this._sortValue = x;
  },

  addSortValue : function(x)
  {
    // summary:
    //		Increments the sort value.
    // tags:
    //		private
    
    this._sortValue += x;
  },

  setSecondSortValue : function(x)
  {
    // summary:
    //		Sets the tie breaker sort value if the normal sort value is the
    //		same.
    // tags:
    //		private
    
    this._sortValue2 = x;
  },

  getSecondSortValue : function()
  {
    // summary:
    //		Returns the tie breaker sort value if the normal sort value is the
    //		same.
    // tags:
    //		private
    
    return this._sortValue2; // Number
  },

  setBTAngleRange : function(range)
  {
    // summary:
    //		Sets the angle range. 
    // description:
    //		Sets the angle range. 
    //		360 means the subtree uses the entire circle around its root. Other
    //		values mean the subtree used only a part of 360 degree. Since
    //		balloon tree layout does not support the subtree block margins we
    //		can reuse the &#95;northSideExtension field for this purpose.
    // tags:
    //		private
    
    this._northSizeExtensions = range;
  },

  getBTAngleRange : function()
  {
    // summary:
    //		Returns the angle range.
    // tags:
    //		private
    
    return this._northSizeExtensions; // Number
  },

  setBTAngle : function(angle)
  {
    // summary:
    //		Sets the angle of the subtree. 
    // description:
    //		Sets the angle of the subtree. 
    //		This is the angle of the link from the parent to this node. In
    //		normal mode, this is an absolute angle. In slow proportional angle
    //		spacing mode, it is relative to the angle of the parent.
    // tags:
    //		private
    
    this._angle = angle;
  },

  getBTAngle : function()
  {
    // summary:
    //		Returns the angle of the subtree. 
    // description:
    //		Returns the angle of the subtree. 
    //		This is the angle of the link from the parent to this node. In
    //		normal mode, this is an absolute angle. In slow proportional angle
    //		spacing mode, it is relative to the angle of the parent.
    // tags:
    //		private
    
    return this._angle; // Number
  },

  setBTSubtreeSurface : function(s)
  {
    // summary:
    //		Sets the subtree surface. 
    // description:
    //		Sets the subtree surface. 
    //		It is basically the sum of all squareroots of areas of nodes in  the
    //		subtree starting at this node. This is just a heuristical value how
    //		much area a subtree will occupy. We use it to determine what angle
    //		range a subtree can occupy.
    // tags:
    //		private
    
    this._sortValue = s;
  },

  getBTSubtreeSurface : function()
  {
    // summary:
    //		Returns the subtree surface.
    // tags:
    //		private
    
    return this._sortValue; // Number
  },

  setBTMinRadius : function(minradius)
  {
    // summary:
    //		Sets the minimal radius from the parent to this node.
    // tags:
    //		private
    
    this._minRadius = minradius;
  },

  getBTMinRadius : function()
  {
    // summary:
    //		Returns the minimal radius from the parent to this node.
    // tags:
    //		private
    
    return this._minRadius; // Number
  },

  setBTRadius : function(radius)
  {
    // summary:
    //		Sets the radius from the parent to this node.
    // tags:
    //		private
    
    this._sortValue2 = radius;
  },

  getBTRadius : function()
  {
    // summary:
    //		Returns the radius from the parent to this node.
    // tags:
    //		private
    
    return this._sortValue2; // Number
  },

  getBTSize : function(index)
  {
    // summary:
    //		Returns a node size used for balloon tree layout.
    // tags:
    //		private
    
    return this._size[index]; // Number
  },

  setBTCenter : function(index, coord)
  {
    // summary:
    //		Sets the node center coordinate used for balloon tree layout.
    // tags:
    //		private
    
    this._coord[index] = coord - 0.5 * this._size[index];
  },

  getBTCenter : function(index)
  {
    // summary:
    //		Returns a node center coordinate for balloon tree layout.
    // tags:
    //		private
    
    return this._coord[index] + 0.5 * this._size[index]; // Number
  },

  hasOnlyLeaves : function()
  {
    // summary:
    //		Returns whether this node has only leaves (children without
    //		children)
    // tags:
    //		private
    
    var iter = this.getVirtualChildren();

    while (iter.hasNext())
    {
      var child = iter.next();
      if (child.getNumberOfChildren() > 0) {
        return false; // Boolean
      }
    }

    return true; // Boolean
  }

}
);

lang.mixin(TNode,
{
  ALIGNMENT_MASK : null,
  ALIGNMENT_USE_WEST : null,

  _fields_init : function(c)
  {
    c = c || TNode;
    
    c.ALIGNMENT_MASK = 0xf;
    c.ALIGNMENT_USE_WEST = 0x100;
  }
  
}
);

TNodeChildrenSort = declare(QuickSort, 
{
  __outerThis: null,
  _initial: null,
  _numberOfElements: null,

  constructor : function(input__outerThis, node, initial)
  {
    this.__outerThis = input__outerThis;
    this._numberOfElements = node.getNumberOfChildren();
    this._initial = initial;
  },
  
  sortChildren : function()
  {
    // summary:
    //		
    // tags:
    //		private
    
    this.inherited("sort", arguments, [this._numberOfElements]);
  },

  dispose : function()
  {
  },

  compare : function(loc1, loc2)
  {
    var child1 = this.__outerThis.getChild(loc1);
    var child2 = this.__outerThis.getChild(loc2);

    if (this.__outerThis.hasEastWestNeighborAtIndex(0, child1)) {
      return -1; // int
    }
    if (this.__outerThis.hasEastWestNeighborAtIndex(0, child2)) {
      return 1; // int
    }
    if (this.__outerThis.hasEastWestNeighborAtIndex(1, child1)) {
      return ((this._initial ? -1 : 1)); // int
    }
    if (this.__outerThis.hasEastWestNeighborAtIndex(1, child2)) {
      return ((this._initial ? 1 : -1)); // int
    }
    if (this._initial)
    {
      if (this.__outerThis.hasAssociate(child1) && !this.__outerThis.hasAssociate(child2)) {
        return -1; // int
      }
      if (this.__outerThis.hasAssociate(child2) && !this.__outerThis.hasAssociate(child1)) {
        return 1; // int
      }
    }

    if (child1.getSortValue() == child2.getSortValue())
    {
      if (child1.getSecondSortValue() < child2.getSecondSortValue()) {
        return -1; // int
      }
      else if (child1.getSecondSortValue() > child2.getSecondSortValue()) {
        return 1; // int
      }
      else {
        return 0; // int
      }
    }
    else if (child1.getSortValue() < child2.getSortValue())
    {
      return -1; // int
    }
    else
    {
      return 1; // int
    }
  },

  swap : function(loc1, loc2)
  {
    var h = this.__outerThis.getChild(loc1);
    this.__outerThis._children[loc1] = this.__outerThis.getChild(loc2);
    this.__outerThis._children[loc2] = h;
  }

}
);
TNode.TRealChildrenIterator = declare( TNodeIterator, 
{
  __outerThis: null,
  count : null,

  constructor : function(input__outerThis)
  {
    this.__outerThis = input__outerThis;
    this.count = 0;
    this.count = 0;

    while (this.count < this.__outerThis._firstFreeChildrenSlot && this.__outerThis.hasEastWestNeighbor(this.__outerThis.getChild(this.count)))
      this.count++;
  },
  
  hasNext : function()
  {
    return this.count < this.__outerThis._firstFreeChildrenSlot; // Boolean
  },

  next : function()
  {
    var child = this.__outerThis.getChild(this.count++);

    while (this.count < this.__outerThis._firstFreeChildrenSlot && this.__outerThis.hasEastWestNeighbor(this.__outerThis.getChild(this.count)))
      this.count++;
    return child; // ibm_ilog.graphlayout.tree.TNode
  }

}
);
TNode.TVirtualChildrenIterator = declare( TNodeIterator, 
{
  __outerThis: null,
  count: null,
  isRight: null,
  actChild: null,

  constructor : function(input__outerThis)
  {
    this.__outerThis = input__outerThis;
    this.count = 0;
    this.isRight = false;
    this.actChild = null;
    if (this.moveCountToRealChild()) {
      this.actChild = this.getFirstChild(this.count);
    }
  },
  
  hasNext : function()
  {
    return (this.actChild != null); // Boolean
  },

  next : function()
  {
    var h = this.actChild;
    this.actChild = this.getChildAfterActChild(this.count);

    if (this.actChild == null)
    {
      this.count++;
      if (this.moveCountToRealChild()) {
        this.actChild = this.getFirstChild(this.count);
      }
    }

    return h; // ibm_ilog.graphlayout.tree.TNode
  },

  moveCountToRealChild : function()
  {
    var numChildren = this.__outerThis.getNumberOfChildren();

    while (this.count < numChildren && this.__outerThis.hasEastWestNeighbor(this.__outerThis.getChild(this.count)))
      this.count++;
    return (this.count < numChildren); // Boolean
  },

  getFirstChild : function(branch)
  {
    var h = this.__outerThis.getChild(branch);

    while (h.getEastWestNeighbor(0) != null)
      h = h.getEastWestNeighbor(0);
    this.isRight = false;
    return (h); // ibm_ilog.graphlayout.tree.TNode
  },

  getChildAfterActChild : function(branch)
  {
    if (this.actChild == this.__outerThis.getChild(branch) || this.isRight)
    {
      this.isRight = true;
      return (this.actChild.getEastWestNeighbor(1)); // ibm_ilog.graphlayout.tree.TNode
    }
    else
    {
      var h = this.__outerThis.getChild(branch);

      while (h.getEastWestNeighbor(0) != this.actChild)
        h = h.getEastWestNeighbor(0);
      return (h); // ibm_ilog.graphlayout.tree.TNode
    }
  }

}
);
/**
 * @private
 */
TNodeAnonClass_1 = declare(TNodeIterator,
{
  // tags:
  //		private
  
  __outerThis : null,
  count: null,

  constructor : function(input__outerThis )
  {
    
    this.__outerThis = input__outerThis;
    this.count = 0;
  },

  hasNext : function()
  {
    return this.count < this.__outerThis._firstFreeChildrenSlot; // Boolean
  },

  next : function()
  {
    return this.__outerThis.getChild(this.count++); // ibm_ilog.graphlayout.tree.TNode
  }

});

(function(){ TNode._fields_init(); })();

return TNode;
});

},
'ibm_ilog/graphlayout/tree/TAngleSpacingModes':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TAngleSpacingModes", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var TAngleSpacingModes = declare("ibm_ilog.graphlayout.tree.TAngleSpacingModes", null, 
{
  // summary:
  //		This class contains all enumeration constants used for the tree
  //		layout angle spacings. 
  // description:
  //		This class contains all enumeration constants used for the tree
  //		layout angle spacings. 
  //		It also contains some generic support for the tree layout angle
  //		spacings.   
  // tags:
  //		private
  
  constructor : function()
  {
    
  }

}
);

lang.mixin(TAngleSpacingModes,
{
  REGULAR : null,
  FAST_PROPORTIONAL : null,
  PROPORTIONAL : null,

  toString : function(mode)
  {
    switch (mode)
    {
      case TAngleSpacingModes.REGULAR:
        return ("REGULAR"); // String
      case TAngleSpacingModes.FAST_PROPORTIONAL:
        return ("FAST_PROPORTIONAL"); // String
      case TAngleSpacingModes.PROPORTIONAL:
        return ("PROPORTIONAL"); // String
      default:
        throw new ArgumentError("Wrong angle spacing " + mode); // String
    }
  },

  valueOf : function(s)
  {
    if (s == "REGULAR") {
      return (TAngleSpacingModes.REGULAR); // int
    }
    if (s == "FAST_PROPORTIONAL") {
      return (TAngleSpacingModes.FAST_PROPORTIONAL); // int
    }
    if (s == "PROPORTIONAL") {
      return (TAngleSpacingModes.PROPORTIONAL); // int
    }
    throw new ArgumentError("Wrong angle spacing " + s); // int
  },

  _fields_init : function(c)
  {
    c = c || TAngleSpacingModes;
    
    c.REGULAR = 0;
    c.FAST_PROPORTIONAL = 1;
    c.PROPORTIONAL = 2;
  }
  
}
);

(function(){ TAngleSpacingModes._fields_init(); })();

return TAngleSpacingModes;
});

},
'ibm_ilog/graphlayout/tree/TPath':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TPath", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var TPath = declare("ibm_ilog.graphlayout.tree.TPath", null, 
{
  // summary:
  //		A path for tree layout is needed to describe the left, right, top or
  //		bottom border of a subtree. 
  // description:
  //		A path for tree layout is needed to describe the left, right, top or
  //		bottom border of a subtree. 
  //		Fow instance when we compare two subtrees to determine how much the
  //		right subtree must be shifted in order not to overlap the left
  //		subtree, we basically compare the left border of the right subtree
  //		with the right border of the left subtree. The coordinate stored in a
  //		path cell is in the coordinate system local to the corresponding node
  //		(ignoring the subtree correction value stored in the node). Hence,
  //		the coordinate of two cells of a border path might be in completely
  //		different coordinate systems. We additional store the coordinate
  //		correction value which indicates how much we have to shift the
  //		coordinate system of the next cell compared to the coordinate system
  //		of the actual cell. Along the path, the sum of coordinate correction
  //		values of previous cells plus the local coordinate of the actual cell
  //		is the coordinate in the global coordinate system. This holds for the
  //		x-direction (index 0) as well as for the y direction (index 1).
  //		Therefore we store coordinates and coordinate correction values as
  //		array of 2 elements. We also store for each child of the current root
  //		node the first path cell of the subtree that is visible at the
  //		corresponding border, and we maintain a backpointer from this path
  //		cell to that child node, which is the root of the subtree the node
  //		corresponding to the path cell belongs to. This is needed for the
  //		case that two borders touch each other and must be shifted, because
  //		then we have fast (O(1)) access to the corresponding child whose
  //		subtree causes the shifting. The siblings of this child (together
  //		with their subtrees) must be shifted.  
  // tags:
  //		private
  
  _node: null,

  _subtreeRoot: null,

  _next: null,

  _last: null,

  _startCoord: null,
  _endCoord: null,

  _coordCorrection: null,

  constructor : function(node, isLeftOrTop, dir)
  {
    this._next = null;
    this._last = this;
    this._startCoord = TranslateUtil.newArray(2, 0);
    this._endCoord = TranslateUtil.newArray(2, 0);
    this._coordCorrection = TranslateUtil.newArray(2, 0);
    this._coordCorrection[0] = 0.0;
    this._coordCorrection[1] = 0.0;
    this._subtreeRoot = null;
    this._node = node;
    this.update(isLeftOrTop, dir);
  },
  
  dispose : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    if (this._subtreeRoot != null)
    {
      if (this._subtreeRoot.getFirstVisibleAtBorder(0, 0) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(0, 0, null);
      }
      if (this._subtreeRoot.getFirstVisibleAtBorder(0, 1) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(0, 1, null);
      }
      if (this._subtreeRoot.getFirstVisibleAtBorder(1, 0) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(1, 0, null);
      }
      if (this._subtreeRoot.getFirstVisibleAtBorder(1, 1) == this) {
        this._subtreeRoot.setFirstVisibleAtBorder(1, 1, null);
      }
    }

    this._subtreeRoot = null;

    this._startCoord = null;
    this._endCoord = null;
    this._coordCorrection = null;
    this._next = null;
    this._last = null;
    this._node = null;
  },

  update : function(isLeftOrTop, dir)
  {
    // summary:
    //		Updates the local coordinates of this cell.  
    // isLeftOrTop: Boolean
    //		is 'true' if this is a left or top border.
    // dir: int
    //		is 0 for left or right border and 1 for top or bottom border.
    // tags:
    //		private
    
    if (isLeftOrTop) {
      this._endCoord[dir] = this._node.getMinCoord(dir);
    }
    else {
      this._endCoord[dir] = this._node.getMaxCoord(dir);
    }
    this._startCoord[dir] = this._endCoord[dir];
    this._startCoord[1 - dir] = this._node.getMinCoord(1 - dir);
    this._endCoord[1 - dir] = this._node.getMaxCoord(1 - dir);
  },

  getNode : function()
  {
    // summary:
    //		Returns the current node.
    // tags:
    //		private
    
    return (this._node); // ibm_ilog.graphlayout.tree.TNode
  },

  getNext : function()
  {
    // summary:
    //		Returns the next path cell.
    // tags:
    //		private
    
    return (this._next); // ibm_ilog.graphlayout.tree.TPath
  },

  getLast : function()
  {
    // summary:
    //		Returns the last path cell.
    // tags:
    //		private
    
    return (this._last); // ibm_ilog.graphlayout.tree.TPath
  },

  getSubtreeRoot : function()
  {
    // summary:
    //		Returns the subtree root. 
    // description:
    //		Returns the subtree root. 
    //		Thats the root of the subtree this path belongs to. Because subtrees
    //		at the right side might be smaller than subtrees on the left side
    //		(in flow direction bottom), the combined border path might contain
    //		path segments that belong to different subtrees: P             The
    //		right border of P consists of parts originated C1    C2  C2       
    //		from C1, C2, and C3. The topmost border cell in / \   / \ /&#95;\       
    //		the combinde border of P originated from C1 or C2 /   \ /&#95;&#95;&#95;\       
    //		   have C2 or C2 as subtree root. /&#95;&#95;&#95;&#95;&#95;\      Needed because when a
    //		right subtree shifts away from the left subtree, some subtrees
    //		between left and right subtree gain space. With the subtree root, it
    //		is O(1) to find those subtrees.
    // tags:
    //		private
    
    return (this._subtreeRoot); // ibm_ilog.graphlayout.tree.TNode
  },

  setSubtreeRoot : function(root)
  {
    // summary:
    //		Sets the subtree root.
    // tags:
    //		private
    
    this._subtreeRoot = root;
  },

  getStartCoord : function(index)
  {
    // summary:
    //		Returns the start coordinate (local to the
    //		node's coordinate system). 
    // description:
    //		Returns the start coordinate (local to the node's coordinate
    //		system). 
    //		If index is 0, it is the x-coordinate, if index is 1, it is the
    //		y-coordinate.
    // tags:
    //		private
    
    return (this._startCoord[index]); // Number
  },

  setStartCoord : function(index, coord)
  {
    // summary:
    //		Sets the start coordinate. 
    // description:
    //		Sets the start coordinate. 
    //		If index is 0, it sets the x-coordinate, if index is 1, then the
    //		y-coordinate.
    // tags:
    //		private
    
    this._startCoord[index] = coord;
  },

  getEndCoord : function(index)
  {
    // summary:
    //		Returns the end coordinate (local to the node's coordinate system). 
    // description:
    //		Returns the end coordinate (local to the node's coordinate system). 
    //		If index is 0, it is the x-coordinate, if index is 1, it is the
    //		y-coordinate.
    // tags:
    //		private
    
    return (this._endCoord[index]); // Number
  },

  setEndCoord : function(index, coord)
  {
    // summary:
    //		Sets the end coordinate. 
    // description:
    //		Sets the end coordinate. 
    //		If index is 0, it sets the x-coordinate, if index is 1, then the
    //		y-coordinate.
    // tags:
    //		private
    
    this._endCoord[index] = coord;
  },

  getLastCellEndCoord : function(index)
  {
    // summary:
    //		Returns the end coordinate of the last cell.
    // tags:
    //		private
    
    var p = this;

    while (p.getNext() != null)
      p = p.getNext();
    return p.getEndCoord(index); // Number
  },

  shiftCoord : function(index, dist)
  {
    // summary:
    //		Shifts the coordinate. 
    // description:
    //		Shifts the coordinate. 
    //		If index is 0, it shifts the x-coordinate, if index is 1, then the
    //		y-coordinate.
    // tags:
    //		private
    
    this._startCoord[index] += dist;
    this._endCoord[index] += dist;
  },

  getCoordCorrection : function(index)
  {
    // summary:
    //		Returns the coordinate correction. 
    // description:
    //		Returns the coordinate correction. 
    //		If index is 0, it is the correction in x-direction, if index is 1,
    //		it is the correction in y-direction.
    // tags:
    //		private
    
    return (this._coordCorrection[index]); // Number
  },

  setCoordCorrection : function(index, coordCorrection)
  {
    // summary:
    //		Sets the coordinate correction. 
    // description:
    //		Sets the coordinate correction. 
    //		If index is 0, it sets the correction in x-direction, if index is 1,
    //		it sets the correction in y-direction.
    // tags:
    //		private
    
    this._coordCorrection[index] = coordCorrection;
  },

  shiftCoordCorrection : function(index, dist)
  {
    // summary:
    //		Shifts the coordinate correction. 
    // description:
    //		Shifts the coordinate correction. 
    //		If index is 0, it shifts the x-coordinate correction, if index is 1,
    //		then the y-coordinate correction.
    // tags:
    //		private
    
    this._coordCorrection[index] += dist;
  },

  cutPath : function(cutCell)
  {
    // summary:
    //		Cuts the path at the input cell (exclusively) and returns the
    //		remaining after the cutCell as indiviual path.
    // tags:
    //		private
    
    var tail = cutCell._next;
    if (tail != null) {
      tail._last = this._last;
    }
    cutCell._next = null;
    this._last = cutCell;
    return tail; // ibm_ilog.graphlayout.tree.TPath
  },

  append : function(path, last)
  {
    // summary:
    //		Appends a path. 
    // description:
    //		Appends a path. 
    //		Important that this cell is the current root of the path. The input
    //		cell need not to be a root, but the additional parameter
    //		<code>last</code> needs to point to the last cell of the input path.
    // tags:
    //		private
    
    this._last._next = path;
    this._last = last;
    path._last = null;
  },

  swap : function(path)
  {
    // summary:
    //		Swaps this path with the input path. 
    // description:
    //		Swaps this path with the input path. 
    //		This path looks afterwards exactly like the input path, and the
    //		input path exactly like this path.
    // tags:
    //		private
    
    var h = this._node;
    this._node = path.getNode();
    path._node = h;
    h = this._subtreeRoot;
    this._subtreeRoot = path.getSubtreeRoot();
    path._subtreeRoot = h;
    var x = this._startCoord[0];
    this._startCoord[0] = path.getStartCoord(0);
    path._startCoord[0] = x;
    x = this._startCoord[1];
    this._startCoord[1] = path.getStartCoord(1);
    path._startCoord[1] = x;
    x = this._endCoord[0];
    this._endCoord[0] = path.getEndCoord(0);
    path._endCoord[0] = x;
    x = this._endCoord[1];
    this._endCoord[1] = path.getEndCoord(1);
    path._endCoord[1] = x;
    x = this._coordCorrection[0];
    this._coordCorrection[0] = path.getCoordCorrection(0);
    path._coordCorrection[0] = x;
    x = this._coordCorrection[1];
    this._coordCorrection[1] = path.getCoordCorrection(1);
    path._coordCorrection[1] = x;
    var p = this._next;
    this._next = path.getNext();
    path._next = p;
    p = this._last;
    if (path.getLast() == path) {
      this._last = this;
    }
    else {
      this._last = path.getLast();
    }
    if (p == this) {
      path._last = path;
    }
    else {
      path._last = p;
    }
  },

  reverse : function(dir)
  {
    // summary:
    //		Reverses the path. 
    // description:
    //		Reverses the path. 
    //		Start and end coordinates in given dir are inverted and swapped. The
    //		next pointer is reversed. This makes only sense when the coordinate
    //		correction is 0. Returns the new first cell.
    // tags:
    //		private
    
    var newLast = this;
    var act = this;
    var prev = null;
    var next;
    var h;

    while (act != null)
    {
      h = act._startCoord[dir];
      act._startCoord[dir] = -act._endCoord[dir];
      act._endCoord[dir] = -h;
      next = act.getNext();
      act._next = prev;
      prev = act;
      act = next;
    }

    prev._last = newLast;
    return prev; // ibm_ilog.graphlayout.tree.TPath
  },

  eliminateCoordinateCoorection : function()
  {
    // summary:
    //		Eliminate the coordinate correction at a border.
    // tags:
    //		private
    
    var x = 0.0;
    var y = 0.0;
    var cell = this;

    while (cell != null)
    {
      cell.shiftCoord(0, x);
      cell.shiftCoord(1, y);
      x += cell.getCoordCorrection(0);
      y += cell.getCoordCorrection(1);
      cell.setCoordCorrection(0, 0.0);
      cell.setCoordCorrection(1, 0.0);
      cell = cell.getNext();
    }
  },

  getMaxCoord : function(dir, westSide)
  {
    // summary:
    //		Returns the maximal coordinate in given direction. 
    // description:
    //		Returns the maximal coordinate in given direction. 
    //		If westSide, it returns only the maximal coordinate in given
    //		direction of those cells that are in opposite direction before 0, if
    //		east side then after 0. This is used in tip-over-east-west to check
    //		the bottom border. The children laid out at the west side are then
    //		the children with x
    //		< 0, and the children at the east side are those with x > 0
    //		(assuming flow direction to the bottom).
    // tags:
    //		private
    
    var max = -/*Number.MAX_VALUE*/(3.4028235E38);
    var x = 0.0;
    var y = 0.0;
    var oppositeDir = 1 - dir;
    var cell = this;

    while (cell != null)
    {
      cell.shiftCoord(0, x);
      cell.shiftCoord(1, y);
      x += cell.getCoordCorrection(0);
      y += cell.getCoordCorrection(1);
      cell.setCoordCorrection(0, 0.0);
      cell.setCoordCorrection(1, 0.0);

      if (westSide)
      {
        if (cell.getStartCoord(oppositeDir) < 0) {
          max = Math.max(max, cell.getStartCoord(dir));
        }
        if (cell.getEndCoord(oppositeDir) < 0) {
          max = Math.max(max, cell.getEndCoord(dir));
        }
      }
      else
      {
        if (cell.getStartCoord(oppositeDir) > 0) {
          max = Math.max(max, cell.getStartCoord(dir));
        }
        if (cell.getEndCoord(oppositeDir) > 0) {
          max = Math.max(max, cell.getEndCoord(dir));
        }
      }

      cell = cell.getNext();
    }

    return max; // Number
  },

  cutForEastWest : function(oppositeDir, westSide)
  {
    // summary:
    //		Cuts this path into two parts. 
    // description:
    //		Cuts this path into two parts. 
    //		On the west side, this path contains this head cell and all its
    //		cells that are before 0 in opposite direction. On the east side,
    //		this path contains this head cell and all its cells that are after 0
    //		in opposite direction. Returns the other path that was cut out. Used
    //		in east-west tip over on the top and bottom border.
    // tags:
    //		private
    
    this.eliminateCoordinateCoorection();
    var head = this;
    var tail = head.cutPath(head);
    if (tail == null) {
      return null; // ibm_ilog.graphlayout.tree.TPath
    }
    var prevCell = null;
    var cell = tail;

    while (cell != null)
    {
      if (cell.getEndCoord(oppositeDir) > 0) {
        break;
      }
      prevCell = cell;
      cell = cell.getNext();
    }

    var eastPart = ((prevCell == null ? tail : tail.cutPath(prevCell)));
    var westPart = ((prevCell == null ? null : tail));

    if (westSide)
    {
      if (westPart != null) {
        this.append(westPart, westPart.getLast());
      }
      return eastPart; // ibm_ilog.graphlayout.tree.TPath
    }
    else
    {
      if (eastPart != null) {
        this.append(eastPart, eastPart.getLast());
      }
      return westPart; // ibm_ilog.graphlayout.tree.TPath
    }
  },

  combineForEastWest : function(otherPath, westSide)
  {
    // summary:
    //		This is the inverse of cutForEastWest. 
    // description:
    //		This is the inverse of cutForEastWest. 
    //		It combines this path with the input path again.
    // tags:
    //		private
    
    if (otherPath == null) {
      return;
    }
    if (westSide)
    {
      this.append(otherPath, otherPath.getLast());
    }
    else
    {
      var tail = this.cutPath(this);
      this.append(otherPath, otherPath.getLast());
      if (tail != null) {
        this.append(tail, tail.getLast());
      }
    }
  },

  disposeNextUntil : function(lastCell)
  {
    // summary:
    //		Delete a path from the next cell up to the input last cell
    //		(exclusively). 
    // description:
    //		Delete a path from the next cell up to the input last cell
    //		(exclusively). 
    //		Just to speed up garbage collection in java.
    // tags:
    //		private
    
    if (lastCell == this) {
      return;
    }
    var cell = this._next;
    var nextCell;

    while (cell != null && cell != lastCell)
    {
      nextCell = cell.getNext();
      cell.dispose();
      cell = nextCell;
    }

    this._next = null;
    this._last = this;
  }

}
);

return TPath;
});

},
'ibm_ilog/graphlayout/tree/TGraphAlgorithm':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TGraphAlgorithm", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var TGraphAlgorithm = declare("ibm_ilog.graphlayout.tree.TGraphAlgorithm", null, 
{
  // summary:
  //		The base class for algorithms that work on a TGraph.  
  // tags:
  //		private
  
  _graph: null,

  _percCompleteController: null,

  constructor : function(graph)
  {
    this._graph = graph;
    this._percCompleteController = graph.getPercController();
  },
  
  disposeIt : function()
  {
    // summary:
    //		The destructor.
    // tags:
    //		private
    
    this._graph = null;
  },

  getGraph : function()
  {
    // summary:
    //		Returns the current graph.
    // tags:
    //		private
    
    return this._graph; // ibm_ilog.graphlayout.tree.TGraph
  },

  getPercController : function()
  {
    // summary:
    //		Returns the current percentage controller.
    // tags:
    //		private
    
    return this._percCompleteController; // ibm_ilog.graphlayout.internalutil.PercCompleteController
  },

  layoutStepPerformed : function()
  {
    // summary:
    //		Informs the layout instance that a step of the layout algorithm has
    //		been performed. 
    // description:
    //		Informs the layout instance that a step of the layout algorithm has
    //		been performed. 
    //		The method notified the listeners of the layout events.
    // tags:
    //		private
    
    this.getGraph().layoutStepPerformed();
  },

  setStepEstimation : function(numberOfSteps)
  {
    // summary:
    //		Sets the estimation how many steps the layout will need. 
    // description:
    //		Sets the estimation how many steps the layout will need. 
    //		This tells the percentage controller to start a step.
    // tags:
    //		private
    
    this._percCompleteController.setStepEstimation(numberOfSteps);
  },

  startStep : function(percentageOfStep, pointsOfStep)
  {
    // summary:
    //		Starts a step of the layout. 
    // description:
    //		Starts a step of the layout. 
    //		This tells the percentage controller to start a step.
    // tags:
    //		private
    
    this._percCompleteController.startStep(percentageOfStep, pointsOfStep, false);
  },

  addPercPoints : function(points)
  {
    // summary:
    //		Adds a point to the percentage controller.
    // tags:
    //		private
    
    this._percCompleteController.addPoints(points);
  },

  incrPercPointEstimation : function(delta)
  {
    // summary:
    //		Increments the estimation how many points are needed to complete the
    //		step. 
    // description:
    //		Increments the estimation how many points are needed to complete the
    //		step. 
    //		This tells the percentage controller how many points still to come.
    // tags:
    //		private
    
    this._percCompleteController.incrementPointEstimation(delta);
  },

  mayContinue : function()
  {
    // summary:
    //		Returns true if we can continue the layout. 
    // description:
    //		Returns true if we can continue the layout. 
    //		Returns false if the layout time elapsed or the layout got
    //		interrupt.
    // tags:
    //		private
    
    return this._graph.getLayout().mayContinue(); // Boolean
  }

}
);

return TGraphAlgorithm;
});

},
'ibm_ilog/graphlayout/tree/TBalloonRadiusModes':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TBalloonRadiusModes", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var TBalloonRadiusModes = declare("ibm_ilog.graphlayout.tree.TBalloonRadiusModes", null, 
{
  // summary:
  //		This class contains all enumeration constants used for the tree
  //		layout balloon radius modes. 
  // description:
  //		This class contains all enumeration constants used for the tree
  //		layout balloon radius modes. 
  //		It also contains some generic support for the tree layout radius
  //		modes.   
  // tags:
  //		private
  
  constructor : function()
  {
    
  }

}
);

lang.mixin(TBalloonRadiusModes,
{
  VARIABLE_RADIUS : null,
  UNIFORM_LEAVES_RADIUS : null,
  UNIFORM_RADIUS : null,
  OPTIMIZED_VARIABLE_RADIUS : null,
  OPTIMIZED_UNIFORM_LEAVES_RADIUS : null,

  toString : function(mode)
  {
    switch (mode)
    {
      case TBalloonRadiusModes.VARIABLE_RADIUS:
        return ("VARIABLE_RADIUS"); // String
      case TBalloonRadiusModes.UNIFORM_LEAVES_RADIUS:
        return ("UNIFORM_LEAVES_RADIUS"); // String
      case TBalloonRadiusModes.UNIFORM_RADIUS:
        return ("UNIFORM_RADIUS"); // String
      case TBalloonRadiusModes.OPTIMIZED_VARIABLE_RADIUS:
        return ("OPTIMIZED_VARIABLE_RADIUS"); // String
      case TBalloonRadiusModes.OPTIMIZED_UNIFORM_LEAVES_RADIUS:
        return ("OPTIMIZED_UNIFORM_LEAVES_RADIUS"); // String
      default:
        throw new ArgumentError("Wrong balloon radius mode " + mode); // String
    }
  },

  valueOf : function(s)
  {
    if (s == "VARIABLE_RADIUS") {
      return (TBalloonRadiusModes.VARIABLE_RADIUS); // int
    }
    if (s == "UNIFORM_LEAVES_RADIUS") {
      return (TBalloonRadiusModes.UNIFORM_LEAVES_RADIUS); // int
    }
    if (s == "UNIFORM_RADIUS") {
      return (TBalloonRadiusModes.UNIFORM_RADIUS); // int
    }
    if (s == "OPTIMIZED_VARIABLE_RADIUS") {
      return (TBalloonRadiusModes.OPTIMIZED_VARIABLE_RADIUS); // int
    }
    if (s == "OPTIMIZED_UNIFORM_LEAVES_RADIUS") {
      return (TBalloonRadiusModes.OPTIMIZED_UNIFORM_LEAVES_RADIUS); // int
    }
    throw new ArgumentError("Wrong balloon radius mode " + s); // int
  },

  _fields_init : function(c)
  {
    c = c || TBalloonRadiusModes;
    
    c.VARIABLE_RADIUS = 0;
    c.UNIFORM_LEAVES_RADIUS = 1;
    c.UNIFORM_RADIUS = 2;
    c.OPTIMIZED_VARIABLE_RADIUS = 3;
    c.OPTIMIZED_UNIFORM_LEAVES_RADIUS = 4;
  }
  
}
);

(function(){ TBalloonRadiusModes._fields_init(); })();

return TBalloonRadiusModes;
});

},
'ibm_ilog/graphlayout/tree/TLinkParameters':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TLinkParameters", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/BaseSubgraphData",	// #5
  "../internalutil/GraphModelUtil",	// #6
  "../internalutil/LinkStyles",	// #7
  "../internalutil/TranslateUtil"	// #8
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    BaseSubgraphData,	// #5
    GraphModelUtil,	// #6
    LinkStyles,	// #7
    TranslateUtil	// #8
    ) {

var TLinkParameters = declare("ibm_ilog.graphlayout.tree.TLinkParameters", null, 
{
  // summary:
  //		A link parameter container. 
  // description:
  //		A link parameter container. 
  //		This link parameter container is used to store all the parameters and
  //		 data we need for the layout at the real link object. The main
  //		objective is to reduce the number of individual properties stored at
  //		the link, to avoid that we allocate a separate property for each bit
  //		or byte to be stored. Therefore we store only this TLinkParameters
  //		object, which contains all parameters.  
  // tags:
  //		private
  
  _linkStyle : null,
  _isCalcForwardTreeLink : null,
  _isCalcBackwardTreeLink : null,
  _isCalcNonTreeLink : null,

  constructor : function(layout, link)
  {
    this._linkStyle = LinkStyles.STRAIGHT_LINE_STYLE;
    this._isCalcForwardTreeLink = false;
    this._isCalcBackwardTreeLink = false;
    this._isCalcNonTreeLink = false;
    this._linkStyle = LinkStyles.STRAIGHT_LINE_STYLE;
    this._isCalcForwardTreeLink = false;
    this._isCalcBackwardTreeLink = false;
    this._isCalcNonTreeLink = false;

    var model = layout.getGraphModel();

    layout = TLinkParameters.adjust(layout, model, link);

    model.setParameter(link, layout._LINK_PROPERTY, this);
  },
  
  isNeeded : function()
  {
    // summary:
    //		Returns 'true' if the parameter container is still
    //		needed, and 'false' if the parameter container can be
    //		removed from the link.
    // tags:
    //		private
    
    return (this._linkStyle != LinkStyles.STRAIGHT_LINE_STYLE || this._isCalcForwardTreeLink || this._isCalcBackwardTreeLink || this._isCalcNonTreeLink); // Boolean
  },

  dispose : function(layout, link)
  {
    // summary:
    //		Dispose the parameter container.
    // tags:
    //		private
    
    var model = layout.getGraphModel();

    layout = TLinkParameters.adjust(layout, model, link);

    model.setParameter(link, layout._LINK_PROPERTY, null);
  },

  isCalcForwardTreeLink : function()
  {
    // summary:
    //		Returns 'true' if the link is a calculated forward tree
    //		link. 
    // description:
    //		Returns <code>true</code> if the link is a calculated forward tree
    //		link. 
    //		In this case, it must be in the calculated forward tree link vector
    //		of the <code>TreeLayout</code> instance.
    // tags:
    //		private
    
    return this._isCalcForwardTreeLink; // Boolean
  },

  setCalcForwardTreeLink : function(flag)
  {
    // summary:
    //		Sets whether the link is a calculated forward tree link.
    // tags:
    //		private
    
    this._isCalcForwardTreeLink = flag;
  },

  isCalcBackwardTreeLink : function()
  {
    // summary:
    //		Returns 'true' if the link is a calculated backward tree
    //		link. 
    // description:
    //		Returns <code>true</code> if the link is a calculated backward tree
    //		link. 
    //		In this case, it must be in the calculated backward tree link vector
    //		of the <code>TreeLayout</code> instance.
    // tags:
    //		private
    
    return this._isCalcBackwardTreeLink; // Boolean
  },

  setCalcBackwardTreeLink : function(flag)
  {
    // summary:
    //		Sets whether the link is a calculated backward tree link.
    // tags:
    //		private
    
    this._isCalcBackwardTreeLink = flag;
  },

  isCalcNonTreeLink : function()
  {
    // summary:
    //		Returns 'true' if the link is a calculated non tree link.
    //		
    // description:
    //		Returns <code>true</code> if the link is a calculated non tree link.
    //		
    //		In this case, it must be in the calculated non tree link vector of
    //		the <code>TreeLayout</code> instance.
    // tags:
    //		private
    
    return this._isCalcNonTreeLink; // Boolean
  },

  setCalcNonTreeLink : function(flag)
  {
    // summary:
    //		Sets whether the link is a calculated non tree link.
    // tags:
    //		private
    
    this._isCalcNonTreeLink = flag;
  }

}
);

lang.mixin(TLinkParameters,
{

  getLinkStyle : function(layout, link)
  {
    if (false) {
      return LinkStyles.STRAIGHT_LINE_STYLE; // int
    }
    var prop = TLinkParameters.get(layout, link);
    if (prop != null) {
      return prop._linkStyle; // int
    }
    return LinkStyles.STRAIGHT_LINE_STYLE; // int
  },

  setLinkStyle : function(layout, link, value)
  {
    var prop = TLinkParameters.getOrAllocate(layout, link);

    var oldStyle;

    oldStyle = prop._linkStyle;

    prop._linkStyle = value;

    if (!prop.isNeeded()) {
      prop.dispose(layout, link);
    }
    if (oldStyle != value) {
      layout.onObjectParameterChanged(link, "LinkStyle");
    }
  },

  get : function(layout, link)
  {
    var model = layout.getGraphModel();

    if (model == null) {
      return null; // ibm_ilog.graphlayout.tree.TLinkParameters
    }
    else
    {
      layout = TLinkParameters.adjust(layout, model, link);

      return model.getParameter(link, layout._LINK_PROPERTY); // ibm_ilog.graphlayout.tree.TLinkParameters
    }
  },

  getOrAllocate : function(layout, link)
  {
    var linkParameters = TLinkParameters.get(layout, link);
    if (linkParameters == null) {
      linkParameters = new TLinkParameters(layout, link);
    }
    return linkParameters; // ibm_ilog.graphlayout.tree.TLinkParameters
  },

  dispose : function(layout, link)
  {
    var prop = TLinkParameters.get(layout, link);
    if (prop != null) {
      prop.dispose(layout, link);
    }
  },

  adjust : function(layout, model, link)
  {
    if (!layout.isRootLayoutInsideRecursiveLayoutTraversal()) {
      return layout; // ibm_ilog.graphlayout.tree.TreeLayout
    }
    if (!(model instanceof BaseSubgraphData)) {
      return layout; // ibm_ilog.graphlayout.tree.TreeLayout
    }

    var nestedModel = model;
    var omodel = nestedModel.getOriginalGraphModel(link);

    if (omodel == nestedModel.getOriginalGraphModel()) {
      return layout; // ibm_ilog.graphlayout.tree.TreeLayout
    }
    return layout.getTreatedSubgraphLayout(omodel, true); // ibm_ilog.graphlayout.tree.TreeLayout
  }

}
);

return TLinkParameters;
});

},
'ibm_ilog/graphlayout/tree/TNodeIterator':function(){
/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

/*
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010,2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 */

// ------------------------------------------------------------------------
// Generated by J2JS
// DO NOT EDIT THIS FILE!
// ------------------------------------------------------------------------

define("ibm_ilog/graphlayout/tree/TNodeIterator", [
  "dojo/_base/declare",	// #0
  "dojo/_base/lang",	// #1
  "dojo/_base/connect",	// #2
  "dojo/_base/array",	// #3
  "dojox/gfx/matrix",	// #4
  "../internalutil/GraphModelUtil",	// #5
  "../internalutil/TranslateUtil"	// #6
  ],
  function(
    declare,	// #0
    lang,	// #1
    connect,	// #2
    arr,	// #3
    matrix,	// #4
    GraphModelUtil,	// #5
    TranslateUtil	// #6
    ) {

var TNodeIterator = declare("ibm_ilog.graphlayout.tree.TNodeIterator",null, 
{
  // summary:
  //		A tree node iterator. 
  // description:
  //		A tree node iterator. 
  //		For example, to access all children of a node in a tree: <blockquote>
  //		
  //		<pre>
  //		TNodeIterator iter = node.getChildren();
  //		while (iter.hasNext()) {
  //		  TNode child = iter.next();
  //		  ...
  //		}
  //		</pre>
  //		</blockquote>  
  // tags:
  //		private
  
  hasNext : function()
  {
    // summary:
    //		Returns <tt>true</tt> if the iteration has more elements.
    // tags:
    //		private
    
  },
  
  next : function()
  {
    // summary:
    //		Returns the next element of this iteration. 
    // description:
    //		Returns the next element of this iteration. 
    //		Different than standard iterations of class <tt>Iterator</tt>, we
    //		don't require to throw an exception if there are no more elements. The access to <tt>next</tt> should always be preceeded by a test <tt>hasNext</tt>.  
    // returns:
    //		The next node of this iteration.
    // tags:
    //		private
    
  }
  
});

return TNodeIterator;
});

}}});

require(["dojo/i18n"], function(i18n){
i18n._preloadLocalizations("ibm_ilog/graphlayout/nls/ibm_graphlayout_tree", []);
});
define("ibm_ilog/graphlayout/ibm_graphlayout_tree", [], 1);
